// Simple ffmpeg.js module wrapper based on (https://github.com/bgrins/videoconverter.js/blob/master/build/ffmpeg_pre.js)
function ffmpeg_run(opts) {
    var Module = {
        'outputDirectory': 'output'
    };
    // copy module arguments
    for (var i in opts) {
        Module[i] = opts[i];
    }
    // extract output file path from arguments
    var outputFilePath = Module['arguments'][Module['arguments'].length - 1];
    if (Module['arguments'].length > 2 && outputFilePath && outputFilePath.indexOf(".") > -1) {
        // append output prefix
        Module['arguments'][Module['arguments'].length - 1] = Module['outputDirectory'] + "/" + outputFilePath;
    }
    Module['preRun'] = function() {
        // create output directory folder
        FS.createFolder('/', Module['outputDirectory'], true, true);
        // create input files with data
        if (Module['files']) {
            Module['files'].forEach(function(file) {
                FS.createDataFile('/', file.name,  new Uint8Array(file.buffer), true, true);
            });
        }
        // convert \r into \n for stdout
        var curr_tty_put_char = TTY.default_tty_ops.put_char;
        TTY.default_tty_ops.put_char = function (tty, val) {
            curr_tty_put_char(tty, val === 13 ? 10 : val);
        }
        var curr_tty1_put_char = TTY.default_tty1_ops.put_char;
        TTY.default_tty1_ops.put_char = function (tty, val) {
            curr_tty1_put_char(tty, val === 13 ? 10 : val);
        }
    };
    Module['postRun'] = function() {
        var handle = FS.analyzePath(Module['outputDirectory']);
        Module['return'] = {'outputFiles': getOutputFiles(handle), 'code': EXITSTATUS};
    };
    function getOutputFiles(result) {
        // create buffer for each output file
        var outputFiles = {}
        if (result && result.object && result.object.contents) {
            for (var i in result.object.contents) {
                if (result.object.contents.hasOwnProperty(i)) {
                    outputFiles[i] = new Uint8Array(result.object.contents[i].contents).buffer;
                }
            }
        }
        return outputFiles;
    }
// Note: For maximum-speed code, see "Optimizing Code" on the Emscripten wiki, https://github.com/kripken/emscripten/wiki/Optimizing-Code
// Note: Some Emscripten settings may limit the speed of the generated code.
// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = eval('(function() { try { return Module || {} } catch(e) { return {} } })()');
// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}
// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function';
var ENVIRONMENT_IS_WEB = typeof window === 'object';
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;
if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  Module['print'] = function(x) {
    process['stdout'].write(x + '\n');
  };
  Module['printErr'] = function(x) {
    process['stderr'].write(x + '\n');
  };
  var nodeFS = require('fs');
  var nodePath = require('path');
  Module['read'] = function(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };
  Module['readBinary'] = function(filename) { return Module['read'](filename, true) };
  Module['load'] = function(f) {
    globalEval(read(f));
  };
  Module['arguments'] = process['argv'].slice(2);
  module['exports'] = Module;
}
else if (ENVIRONMENT_IS_SHELL) {
  Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm
  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function() { throw 'no read() available (jsc?)' };
  }
  Module['readBinary'] = function(f) {
    return read(f, 'binary');
  };
  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }
  this['Module'] = Module;
}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };
  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }
  if (typeof console !== 'undefined') {
    Module['print'] = function(x) {
      console.log(x);
    };
    Module['printErr'] = function(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }
  if (ENVIRONMENT_IS_WEB) {
    this['Module'] = Module;
  } else {
    Module['load'] = importScripts;
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}
function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] == 'undefined' && Module['read']) {
  Module['load'] = function(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
// *** Environment setup code ***
// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];
// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];
// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}
// === Auto-generated preamble library stuff ===
//========================================
// Runtime code shared with compiler
//========================================
var Runtime = {
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  forceAlign: function (target, quantum) {
    quantum = quantum || 4;
    if (quantum == 1) return target;
    if (isNumber(target) && isNumber(quantum)) {
      return Math.ceil(target/quantum)*quantum;
    } else if (isNumber(quantum) && isPowerOfTwo(quantum)) {
      return '(((' +target + ')+' + (quantum-1) + ')&' + -quantum + ')';
    }
    return 'Math.ceil((' + target + ')/' + quantum + ')*' + quantum;
  },
  isNumberType: function (type) {
    return type in Runtime.INT_TYPES || type in Runtime.FLOAT_TYPES;
  },
  isPointerType: function isPointerType(type) {
  return type[type.length-1] == '*';
},
  isStructType: function isStructType(type) {
  if (isPointerType(type)) return false;
  if (isArrayType(type)) return true;
  if (/<?{ ?[^}]* ?}>?/.test(type)) return true; // { i32, i8 } etc. - anonymous struct types
  // See comment in isStructPointerType()
  return type[0] == '%';
},
  INT_TYPES: {"i1":0,"i8":0,"i16":0,"i32":0,"i64":0},
  FLOAT_TYPES: {"float":0,"double":0},
  or64: function (x, y) {
    var l = (x | 0) | (y | 0);
    var h = (Math.round(x / 4294967296) | Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  and64: function (x, y) {
    var l = (x | 0) & (y | 0);
    var h = (Math.round(x / 4294967296) & Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  xor64: function (x, y) {
    var l = (x | 0) ^ (y | 0);
    var h = (Math.round(x / 4294967296) ^ Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  dedup: function dedup(items, ident) {
  var seen = {};
  if (ident) {
    return items.filter(function(item) {
      if (seen[item[ident]]) return false;
      seen[item[ident]] = true;
      return true;
    });
  } else {
    return items.filter(function(item) {
      if (seen[item]) return false;
      seen[item] = true;
      return true;
    });
  }
},
  set: function set() {
  var args = typeof arguments[0] === 'object' ? arguments[0] : arguments;
  var ret = {};
  for (var i = 0; i < args.length; i++) {
    ret[args[i]] = 0;
  }
  return ret;
},
  STACK_ALIGN: 8,
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (type == 'i64' || type == 'double' || vararg) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  calculateStructAlignment: function calculateStructAlignment(type) {
    type.flatSize = 0;
    type.alignSize = 0;
    var diffs = [];
    var prev = -1;
    var index = 0;
    type.flatIndexes = type.fields.map(function(field) {
      index++;
      var size, alignSize;
      if (Runtime.isNumberType(field) || Runtime.isPointerType(field)) {
        size = Runtime.getNativeTypeSize(field); // pack char; char; in structs, also char[X]s.
        alignSize = Runtime.getAlignSize(field, size);
      } else if (Runtime.isStructType(field)) {
        if (field[1] === '0') {
          // this is [0 x something]. When inside another structure like here, it must be at the end,
          // and it adds no size
          // XXX this happens in java-nbody for example... assert(index === type.fields.length, 'zero-length in the middle!');
          size = 0;
          if (Types.types[field]) {
            alignSize = Runtime.getAlignSize(null, Types.types[field].alignSize);
          } else {
            alignSize = type.alignSize || QUANTUM_SIZE;
          }
        } else {
          size = Types.types[field].flatSize;
          alignSize = Runtime.getAlignSize(null, Types.types[field].alignSize);
        }
      } else if (field[0] == 'b') {
        // bN, large number field, like a [N x i8]
        size = field.substr(1)|0;
        alignSize = 1;
      } else if (field[0] === '<') {
        // vector type
        size = alignSize = Types.types[field].flatSize; // fully aligned
      } else if (field[0] === 'i') {
        // illegal integer field, that could not be legalized because it is an internal structure field
        // it is ok to have such fields, if we just use them as markers of field size and nothing more complex
        size = alignSize = parseInt(field.substr(1))/8;
        assert(size % 1 === 0, 'cannot handle non-byte-size field ' + field);
      } else {
        assert(false, 'invalid type for calculateStructAlignment');
      }
      if (type.packed) alignSize = 1;
      type.alignSize = Math.max(type.alignSize, alignSize);
      var curr = Runtime.alignMemory(type.flatSize, alignSize); // if necessary, place this on aligned memory
      type.flatSize = curr + size;
      if (prev >= 0) {
        diffs.push(curr-prev);
      }
      prev = curr;
      return curr;
    });
    if (type.name_[0] === '[') {
      // arrays have 2 elements, so we get the proper difference. then we scale here. that way we avoid
      // allocating a potentially huge array for [999999 x i8] etc.
      type.flatSize = parseInt(type.name_.substr(1))*type.flatSize/2;
    }
    type.flatSize = Runtime.alignMemory(type.flatSize, type.alignSize);
    if (diffs.length == 0) {
      type.flatFactor = type.flatSize;
    } else if (Runtime.dedup(diffs).length == 1) {
      type.flatFactor = diffs[0];
    }
    type.needsFlattening = (type.flatFactor != 1);
    return type.flatIndexes;
  },
  generateStructInfo: function (struct, typeName, offset) {
    var type, alignment;
    if (typeName) {
      offset = offset || 0;
      type = (typeof Types === 'undefined' ? Runtime.typeInfo : Types.types)[typeName];
      if (!type) return null;
      if (type.fields.length != struct.length) {
        printErr('Number of named fields must match the type for ' + typeName + ': possibly duplicate struct names. Cannot return structInfo');
        return null;
      }
      alignment = type.flatIndexes;
    } else {
      var type = { fields: struct.map(function(item) { return item[0] }) };
      alignment = Runtime.calculateStructAlignment(type);
    }
    var ret = {
      __size__: type.flatSize
    };
    if (typeName) {
      struct.forEach(function(item, i) {
        if (typeof item === 'string') {
          ret[item] = alignment[i] + offset;
        } else {
          // embedded struct
          var key;
          for (var k in item) key = k;
          ret[key] = Runtime.generateStructInfo(item[key], type.fields[i], alignment[i]);
        }
      });
    } else {
      struct.forEach(function(item, i) {
        ret[item[1]] = alignment[i];
      });
    }
    return ret;
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[func]) {
      Runtime.funcWrappers[func] = function() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return Runtime.funcWrappers[func];
  },
  UTF8Processor: function () {
    var buffer = [];
    var needed = 0;
    this.processCChar = function (code) {
      code = code & 0xFF;
      if (buffer.length == 0) {
        if ((code & 0x80) == 0x00) {        // 0xxxxxxx
          return String.fromCharCode(code);
        }
        buffer.push(code);
        if ((code & 0xE0) == 0xC0) {        // 110xxxxx
          needed = 1;
        } else if ((code & 0xF0) == 0xE0) { // 1110xxxx
          needed = 2;
        } else {                            // 11110xxx
          needed = 3;
        }
        return '';
      }
      if (needed) {
        buffer.push(code);
        needed--;
        if (needed > 0) return '';
      }
      var c1 = buffer[0];
      var c2 = buffer[1];
      var c3 = buffer[2];
      var c4 = buffer[3];
      var ret;
      if (buffer.length == 2) {
        ret = String.fromCharCode(((c1 & 0x1F) << 6)  | (c2 & 0x3F));
      } else if (buffer.length == 3) {
        ret = String.fromCharCode(((c1 & 0x0F) << 12) | ((c2 & 0x3F) << 6)  | (c3 & 0x3F));
      } else {
        // http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
        var codePoint = ((c1 & 0x07) << 18) | ((c2 & 0x3F) << 12) |
                        ((c3 & 0x3F) << 6)  | (c4 & 0x3F);
        ret = String.fromCharCode(
          Math.floor((codePoint - 0x10000) / 0x400) + 0xD800,
          (codePoint - 0x10000) % 0x400 + 0xDC00);
      }
      buffer.length = 0;
      return ret;
    }
    this.processJSString = function(string) {
      string = unescape(encodeURIComponent(string));
      var ret = [];
      for (var i = 0; i < string.length; i++) {
        ret.push(string.charCodeAt(i));
      }
      return ret;
    }
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+7)&-8); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + size)|0;STATICTOP = (((STATICTOP)+7)&-8); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + size)|0;DYNAMICTOP = (((DYNAMICTOP)+7)&-8); if (DYNAMICTOP >= TOTAL_MEMORY) enlargeMemory();; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 8))*(quantum ? quantum : 8); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*(+4294967296))) : ((+((low>>>0)))+((+((high|0)))*(+4294967296)))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}
//========================================
// Runtime essentials
//========================================
var __THREW__ = 0; // Used in checking for thrown exceptions.
var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;
var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;
function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}
var globalScope = this;
// C calling interface. A convenient way to call C functions (in C files, or
// defined with extern "C").
//
// Note: LLVM optimizations can inline and remove functions, after which you will not be
//       able to call them. Closure can also do so. To avoid that, add your function to
//       the exports using something like
//
//         -s EXPORTED_FUNCTIONS='["_main", "_myfunc"]'
//
// @param ident      The name of the C function (note that C++ functions will be name-mangled - use extern "C")
// @param returnType The return type of the function, one of the JS types 'number', 'string' or 'array' (use 'number' for any C pointer, and
//                   'array' for JavaScript arrays and typed arrays; note that arrays are 8-bit).
// @param argTypes   An array of the types of arguments for the function (if there are no arguments, this can be ommitted). Types are as in returnType,
//                   except that 'array' is not possible (there is no way for us to know the length of the array)
// @param args       An array of the arguments to the function, as native JS values (as in returnType)
//                   Note that string arguments will be stored on the stack (the JS string will become a C string on the stack).
// @return           The return value, as a native JS value (as in returnType)
function ccall(ident, returnType, argTypes, args) {
  return ccallFunc(getCFunc(ident), returnType, argTypes, args);
}
Module["ccall"] = ccall;
// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  try {
    var func = Module['_' + ident]; // closure exported function
    if (!func) func = eval('_' + ident); // explicit lookup
  } catch(e) {
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}
// Internal function that does a C call using a function, not an identifier
function ccallFunc(func, returnType, argTypes, args) {
  var stack = 0;
  function toC(value, type) {
    if (type == 'string') {
      if (value === null || value === undefined || value === 0) return 0; // null string
      value = intArrayFromString(value);
      type = 'array';
    }
    if (type == 'array') {
      if (!stack) stack = Runtime.stackSave();
      var ret = Runtime.stackAlloc(value.length);
      writeArrayToMemory(value, ret);
      return ret;
    }
    return value;
  }
  function fromC(value, type) {
    if (type == 'string') {
      return Pointer_stringify(value);
    }
    assert(type != 'array');
    return value;
  }
  var i = 0;
  var cArgs = args ? args.map(function(arg) {
    return toC(arg, argTypes[i++]);
  }) : [];
  var ret = fromC(func.apply(null, cArgs), returnType);
  if (stack) Runtime.stackRestore(stack);
  return ret;
}
// Returns a native JS wrapper for a C function. This is similar to ccall, but
// returns a function you can call repeatedly in a normal way. For example:
//
//   var my_function = cwrap('my_c_function', 'number', ['number', 'number']);
//   alert(my_function(5, 22));
//   alert(my_function(99, 12));
//
function cwrap(ident, returnType, argTypes) {
  var func = getCFunc(ident);
  return function() {
    return ccallFunc(func, returnType, argTypes, Array.prototype.slice.call(arguments));
  }
}
Module["cwrap"] = cwrap;
// Sets a value in memory in a dynamic way at run-time. Uses the
// type data. This is the same as makeSetValue, except that
// makeSetValue is done at compile-time and generates the needed
// code then, whereas this function picks the right code at
// run-time.
// Note that setValue and getValue only do *aligned* writes and reads!
// Note that ccall uses JS types as for defining types, while setValue and
// getValue need LLVM types ('i8', 'i32') - this is a lower-level operation
function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[(ptr)]=value; break;
      case 'i8': HEAP8[(ptr)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= (+1) ? (tempDouble > (+0) ? ((Math_min((+(Math_floor((tempDouble)/(+4294967296)))), (+4294967295)))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/(+4294967296))))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module['setValue'] = setValue;
// Parallel to setValue.
function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[(ptr)];
      case 'i8': return HEAP8[(ptr)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module['getValue'] = getValue;
var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module['ALLOC_NORMAL'] = ALLOC_NORMAL;
Module['ALLOC_STACK'] = ALLOC_STACK;
Module['ALLOC_STATIC'] = ALLOC_STATIC;
Module['ALLOC_DYNAMIC'] = ALLOC_DYNAMIC;
Module['ALLOC_NONE'] = ALLOC_NONE;
// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }
  var singleType = typeof types === 'string' ? types : null;
  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }
  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)|0)]=0;
    }
    return ret;
  }
  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }
  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];
    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }
    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }
    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later
    setValue(ret+i, curr, type);
    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }
  return ret;
}
Module['allocate'] = allocate;
function Pointer_stringify(ptr, /* optional */ length) {
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = false;
  var t;
  var i = 0;
  while (1) {
    t = HEAPU8[(((ptr)+(i))|0)];
    if (t >= 128) hasUtf = true;
    else if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;
  var ret = '';
  if (!hasUtf) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }
  var utf8 = new Runtime.UTF8Processor();
  for (i = 0; i < length; i++) {
    t = HEAPU8[(((ptr)+(i))|0)];
    ret += utf8.processCChar(t);
  }
  return ret;
}
Module['Pointer_stringify'] = Pointer_stringify;
// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.
function UTF16ToString(ptr) {
  var i = 0;
  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module['UTF16ToString'] = UTF16ToString;
// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr', 
// null-terminated and encoded in UTF16LE form. The copy will require at most (str.length*2+1)*2 bytes of space in the HEAP.
function stringToUTF16(str, outPtr) {
  for(var i = 0; i < str.length; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[(((outPtr)+(i*2))>>1)]=codeUnit
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[(((outPtr)+(str.length*2))>>1)]=0
}
Module['stringToUTF16'] = stringToUTF16;
// Given a pointer 'ptr' to a null-terminated UTF32LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.
function UTF32ToString(ptr) {
  var i = 0;
  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module['UTF32ToString'] = UTF32ToString;
// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr', 
// null-terminated and encoded in UTF32LE form. The copy will require at most (str.length+1)*4 bytes of space in the HEAP,
// but can use less, since str.length does not return the number of characters in the string, but the number of UTF-16 code units in the string.
function stringToUTF32(str, outPtr) {
  var iChar = 0;
  for(var iCodeUnit = 0; iCodeUnit < str.length; ++iCodeUnit) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    var codeUnit = str.charCodeAt(iCodeUnit); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++iCodeUnit);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[(((outPtr)+(iChar*4))>>2)]=codeUnit
    ++iChar;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[(((outPtr)+(iChar*4))>>2)]=0
}
Module['stringToUTF32'] = stringToUTF32;
function demangle(func) {
  try {
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    var i = 3;
    // params, etc.
    var basicTypes = {
      'v': 'void',
      'b': 'bool',
      'c': 'char',
      's': 'short',
      'i': 'int',
      'l': 'long',
      'f': 'float',
      'd': 'double',
      'w': 'wchar_t',
      'a': 'signed char',
      'h': 'unsigned char',
      't': 'unsigned short',
      'j': 'unsigned int',
      'm': 'unsigned long',
      'x': 'long long',
      'y': 'unsigned long long',
      'z': '...'
    };
    function dump(x) {
      //return;
      if (x) Module.print(x);
      Module.print(func);
      var pre = '';
      for (var a = 0; a < i; a++) pre += ' ';
      Module.print (pre + '^');
    }
    var subs = [];
    function parseNested() {
      i++;
      if (func[i] === 'K') i++;
      var parts = [];
      while (func[i] !== 'E') {
        if (func[i] === 'S') { // substitution
          i++;
          var next = func.indexOf('_', i);
          var num = func.substring(i, next) || 0;
          parts.push(subs[num] || '?');
          i = next+1;
          continue;
        }
        var size = parseInt(func.substr(i));
        var pre = size.toString().length;
        if (!size || !pre) { i--; break; } // counter i++ below us
        var curr = func.substr(i + pre, size);
        parts.push(curr);
        subs.push(curr);
        i += pre + size;
      }
      i++; // skip E
      return parts;
    }
    function parse(rawList, limit, allowVoid) { // main parser
      limit = limit || Infinity;
      var ret = '', list = [];
      function flushList() {
        return '(' + list.join(', ') + ')';
      }
      var name;
      if (func[i] !== 'N') {
        // not namespaced
        if (func[i] === 'K') i++;
        var size = parseInt(func.substr(i));
        if (size) {
          var pre = size.toString().length;
          name = func.substr(i + pre, size);
          i += pre + size;
        }
      } else {
        // namespaced N-E
        name = parseNested().join('::');
        limit--;
        if (limit === 0) return rawList ? [name] : name;
      }
      if (func[i] === 'I') {
        i++;
        var iList = parse(true);
        var iRet = parse(true, 1, true);
        ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
      } else {
        ret = name;
      }
      paramLoop: while (i < func.length && limit-- > 0) {
        //dump('paramLoop');
        var c = func[i++];
        if (c in basicTypes) {
          list.push(basicTypes[c]);
        } else {
          switch (c) {
            case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
            case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
            case 'L': { // literal
              i++; // skip basic type
              var end = func.indexOf('E', i);
              var size = end - i;
              list.push(func.substr(i, size));
              i += size + 2; // size + 'EE'
              break;
            }
            case 'A': { // array
              var size = parseInt(func.substr(i));
              i += size.toString().length;
              if (func[i] !== '_') throw '?';
              i++; // skip _
              list.push(parse(true, 1, true)[0] + ' [' + size + ']');
              break;
            }
            case 'E': break paramLoop;
            default: ret += '?' + c; break paramLoop;
          }
        }
      }
      if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
      return rawList ? list : ret + flushList();
    }
    return parse();
  } catch(e) {
    return func;
  }
}
function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}
function stackTrace() {
  var stack = new Error().stack;
  return stack ? demangleAll(stack) : '(no stack trace available)'; // Stack trace is not available at least on IE10 and Safari 6.
}
// Memory management
var PAGE_SIZE = 4096;
function alignMemoryPage(x) {
  return (x+4095)&-4096;
}
var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;
var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk
function enlargeMemory() {
  abort('Cannot enlarge memory arrays in asm.js. Either (1) compile with -s TOTAL_MEMORY=X with X higher than the current value ' + TOTAL_MEMORY + ', or (2) set Module.TOTAL_MEMORY before the program runs.');
}
var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 67108864;
var FAST_MEMORY = Module['FAST_MEMORY'] || 2097152;
// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'Cannot fallback to non-typed array case: Code is too specialized');
var buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);
// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');
Module['HEAP'] = HEAP;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;
function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}
var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited
var runtimeInitialized = false;
function preRun() {
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}
function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}
function preMain() {
  callRuntimeCallbacks(__ATMAIN__);
}
function exitRuntime() {
  callRuntimeCallbacks(__ATEXIT__);
}
function postRun() {
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}
function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module['addOnPreRun'] = Module.addOnPreRun = addOnPreRun;
function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module['addOnInit'] = Module.addOnInit = addOnInit;
function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module['addOnPreMain'] = Module.addOnPreMain = addOnPreMain;
function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module['addOnExit'] = Module.addOnExit = addOnExit;
function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module['addOnPostRun'] = Module.addOnPostRun = addOnPostRun;
// Tools
// This processes a JS string into a C-line array of numbers, 0-terminated.
// For LLVM-originating strings, see parser.js:parseLLVMString function
function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var ret = (new Runtime.UTF8Processor()).processJSString(stringy);
  if (length) {
    ret.length = length;
  }
  if (!dontAddNull) {
    ret.push(0);
  }
  return ret;
}
Module['intArrayFromString'] = intArrayFromString;
function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module['intArrayToString'] = intArrayToString;
// Write a Javascript array to somewhere in the heap
function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))|0)]=chr
    i = i + 1;
  }
}
Module['writeStringToMemory'] = writeStringToMemory;
function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[(((buffer)+(i))|0)]=array[i];
  }
}
Module['writeArrayToMemory'] = writeArrayToMemory;
function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; i++) {
    HEAP8[(((buffer)+(i))|0)]=str.charCodeAt(i)
  }
  if (!dontAddNull) HEAP8[(((buffer)+(str.length))|0)]=0
}
Module['writeAsciiToMemory'] = writeAsciiToMemory;
function unSign(value, bits, ignore, sig) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore, sig) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}
if (!Math['imul']) Math['imul'] = function(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];
var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_toFloat32 = Math.toFloat32;
var Math_min = Math.min;
// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled
function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
}
Module['addRunDependency'] = addRunDependency;
function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module['removeRunDependency'] = removeRunDependency;
Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data
var memoryInitializer = null;
// === Body ===
STATIC_BASE = 8;
STATICTOP = STATIC_BASE + 3133832;
/* global initializers */ __ATINIT__.push({ func: function() { runPostSets() } });
var _stdout;
var _stdout=_stdout=allocate([0,0,0,0,0,0,0,0], "i8", ALLOC_STATIC);
var _stderr;
var _stderr=_stderr=allocate([0,0,0,0,0,0,0,0], "i8", ALLOC_STATIC);
var ___fsmu8;
/* memory initializer */ allocate([32,17,17,9,0,0,0,0,1,0,0,0,232,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,88,230,29,0,88,218,29,0,152,230,29,0,0,0,0,0,216,218,29,0,136,232,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,152,231,29,0,0,0,0,0,216,222,29,0,216,210,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,144,20,30,0,0,0,0,0,216,218,29,0,128,22,30,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,144,21,30,0,0,0,0,0,216,222,29,0,16,13,30,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,27,134,42,204,204,52,43,33,78,132,43,252,247,157,43,88,156,166,43,252,247,157,43,33,78,132,43,204,204,52,43,0,27,134,42,83,248,191,44,254,169,171,44,146,50,149,44,159,129,122,44,239,29,73,44,62,186,23,44,116,173,207,43,133,159,107,43,183,89,146,42,83,248,191,172,254,169,171,172,146,50,149,172,159,129,122,172,239,29,73,172,62,186,23,172,116,173,207,171,133,159,107,171,183,89,146,170,0,27,134,170,204,204,52,171,33,78,132,171,252,247,157,171,88,156,166,171,252,247,157,171,33,78,132,171,204,204,52,171,0,27,134,170,0,27,134,42,204,204,52,43,33,78,132,43,252,247,157,43,88,156,166,43,252,247,157,43,33,78,132,43,204,204,52,43,0,27,134,42,83,248,191,44,254,169,171,44,146,50,149,44,159,129,122,44,239,29,73,44,62,186,23,44,116,173,207,43,133,159,107,43,183,89,146,42,37,39,192,172,51,37,173,172,234,209,152,172,227,84,131,172,249,175,89,172,11,14,43,172,102,34,244,171,201,49,137,171,74,123,157,170,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,144,128,170,174,79,227,170,5,174,113,170,234,207,6,62,205,19,212,62,139,111,68,63,255,175,139,63,23,208,166,63,117,235,200,63,190,226,245,63,122,130,26,64,105,251,74,64,185,87,144,64,107,16,243,64,233,58,183,65,92,28,124,63,187,141,36,63,68,29,175,62,178,143,112,63,212,208,49,190,125,27,68,191,215,179,93,63,0,0,0,63,254,181,3,191,218,134,241,190,2,115,160,190,116,71,58,190,29,176,193,189,135,203,39,189,29,161,104,188,70,123,114,187,168,132,91,63,216,185,97,63,221,26,115,63,129,186,123,63,65,218,126,63,253,200,127,63,101,249,127,63,141,255,127,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,144,128,42,174,79,227,42,5,174,113,42,37,39,192,44,51,37,173,44,234,209,152,44,227,84,131,44,249,175,89,44,11,14,43,44,102,34,244,43,201,49,137,43,74,123,157,42,83,248,191,172,254,169,171,172,146,50,149,172,159,129,122,172,239,29,73,172,62,186,23,172,116,173,207,171,133,159,107,171,183,89,146,170,0,27,134,170,204,204,52,171,33,78,132,171,252,247,157,171,88,156,166,171,252,247,157,171,33,78,132,171,204,204,52,171,0,27,134,170,88,129,26,0,96,110,26,0,168,91,26,0,32,75,26,0,104,60,26,0,48,46,26,0,112,32,26,0,112,18,26,0,0,0,0,0,0,0,0,0,160,143,25,0,96,0,0,0,128,2,4,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,198,63,120,51,98,136,11,53,151,200,193,53,80,233,61,54,183,247,156,54,46,124,234,54,153,192,35,55,244,2,90,55,56,3,140,55,227,228,174,55,177,166,213,55,108,36,0,56,146,101,23,56,201,150,48,56,18,184,75,56,81,201,104,56,94,229,131,56,29,94,148,56,229,206,165,56,167,55,184,56,128,152,203,56,85,241,223,56,36,66,245,56,126,197,5,57,238,101,17,57,99,130,29,57,207,26,42,57,63,47,55,57,179,191,68,57,30,204,82,57,141,84,97,57,243,88,112,57,94,217,127,57,227,234,135,57,18,39,144,57,64,161,152,57,105,89,161,57,146,79,170,57,181,131,179,57,215,245,188,57,245,165,198,57,14,148,208,57,34,192,218,57,46,42,229,57,57,210,239,57,60,184,250,57,27,238,2,58,22,159,8,58,13,111,14,58,0,94,20,58,239,107,26,58,218,152,32,58,192,228,38,58,161,79,45,58,124,217,51,58,83,130,58,58,37,74,65,58,240,48,72,58,182,54,79,58,116,91,86,58,45,159,93,58,222,1,101,58,136,131,108,58,42,36,116,58,196,227,123,58,44,225,129,58,241,223,133,58,49,238,137,58,238,11,142,58,37,57,146,58,215,117,150,58,5,194,154,58,174,29,159,58,209,136,163,58,110,3,168,58,134,141,172,58,24,39,177,58,36,208,181,58,169,136,186,58,169,80,191,58,33,40,196,58,19,15,201,58,126,5,206,58,98,11,211,58,191,32,216,58,148,69,221,58,225,121,226,58,166,189,231,58,227,16,237,58,152,115,242,58,196,229,247,58,103,103,253,58,65,124,1,59,137,76,4,59,141,36,7,59,76,4,10,59,198,235,12,59,251,218,15,59,235,209,18,59,149,208,21,59,251,214,24,59,26,229,27,59,244,250,30,59,136,24,34,59,215,61,37,59,223,106,40,59,161,159,43,59,29,220,46,59,83,32,50,59,66,108,53,59,234,191,56,59,76,27,60,59,103,126,63,59,59,233,66,59,199,91,70,59,12,214,73,59,10,88,77,59,193,225,80,59,48,115,84,59,86,12,88,59,53,173,91,59,204,85,95,59,26,6,99,59,32,190,102,59,222,125,106,59,82,69,110,59,127,20,114,59,97,235,117,59,251,201,121,59,76,176,125,59,41,207,128,59,8,202,130,59,194,200,132,59,87,203,134,59,198,209,136,59,17,220,138,59,55,234,140,59,55,252,142,59,18,18,145,59,199,43,147,59,87,73,149,59,194,106,151,59,6,144,153,59,37,185,155,59,30,230,157,59,241,22,160,59,158,75,162,59,37,132,164,59,134,192,166,59,192,0,169,59,212,68,171,59,193,140,173,59,137,216,175,59,41,40,178,59,163,123,180,59,245,210,182,59,33,46,185,59,38,141,187,59,4,240,189,59,186,86,192,59,73,193,194,59,177,47,197,59,242,161,199,59,10,24,202,59,251,145,204,59,196,15,207,59,102,145,209,59,223,22,212,59,49,160,214,59,90,45,217,59,91,190,219,59,51,83,222,59,227,235,224,59,107,136,227,59,201,40,230,59,255,204,232,59,12,117,235,59,240,32,238,59,171,208,240,59,61,132,243,59,165,59,246,59,228,246,248,59,250,181,251,59,229,120,254,59,212,159,0,60,32,5,2,60,87,108,3,60,121,213,4,60,134,64,6,60,126,173,7,60,96,28,9,60,45,141,10,60,229,255,11,60,136,116,13,60,21,235,14,60,141,99,16,60,239,221,17,60,59,90,19,60,114,216,20,60,147,88,22,60,158,218,23,60,147,94,25,60,115,228,26,60,60,108,28,60,240,245,29,60,141,129,31,60,20,15,33,60,133,158,34,60,224,47,36,60,36,195,37,60,82,88,39,60,105,239,40,60,106,136,42,60,84,35,44,60,40,192,45,60,229,94,47,60,139,255,48,60,26,162,50,60,146,70,52,60,243,236,53,60,61,149,55,60,112,63,57,60,140,235,58,60,145,153,60,60,126,73,62,60,84,251,63,60,18,175,65,60,185,100,67,60,72,28,69,60,192,213,70,60,31,145,72,60,103,78,74,60,151,13,76,60,175,206,77,60,176,145,79,60,152,86,81,60,103,29,83,60,31,230,84,60,190,176,86,60,69,125,88,60,179,75,90,60,9,28,92,60,71,238,93,60,107,194,95,60,119,152,97,60,106,112,99,60,68,74,101,60,5,38,103,60,173,3,105,60,60,227,106,60,178,196,108,60,14,168,110,60,81,141,112,60,123,116,114,60,139,93,116,60,130,72,118,60,95,53,120,60,34,36,122,60,203,20,124,60,90,7,126,60,208,251,127,60,22,249,128,60,54,245,129,60,74,242,130,60,80,240,131,60,73,239,132,60,53,239,133,60,19,240,134,60,229,241,135,60,169,244,136,60,95,248,137,60,8,253,138,60,164,2,140,60,50,9,141,60,178,16,142,60,37,25,143,60,139,34,144,60,226,44,145,60,44,56,146,60,104,68,147,60,150,81,148,60,182,95,149,60,201,110,150,60,205,126,151,60,196,143,152,60,172,161,153,60,135,180,154,60,83,200,155,60,17,221,156,60,193,242,157,60,98,9,159,60,245,32,160,60,122,57,161,60,241,82,162,60,89,109,163,60,178,136,164,60,253,164,165,60,57,194,166,60,103,224,167,60,134,255,168,60,151,31,170,60,152,64,171,60,139,98,172,60,111,133,173,60,68,169,174,60,10,206,175,60,193,243,176,60,105,26,178,60,2,66,179,60,139,106,180,60,6,148,181,60,113,190,182,60,205,233,183,60,26,22,185,60,87,67,186,60,133,113,187,60,163,160,188,60,177,208,189,60,177,1,191,60,160,51,192,60,128,102,193,60,80,154,194,60,16,207,195,60,193,4,197,60,97,59,198,60,242,114,199,60,114,171,200,60,227,228,201,60,67,31,203,60,147,90,204,60,211,150,205,60,3,212,206,60,34,18,208,60,49,81,209,60,48,145,210,60,30,210,211,60,252,19,213,60,201,86,214,60,133,154,215,60,49,223,216,60,204,36,218,60,86,107,219,60,208,178,220,60,56,251,221,60,144,68,223,60,214,142,224,60,12,218,225,60,48,38,227,60,67,115,228,60,69,193,229,60,54,16,231,60,21,96,232,60,227,176,233,60,160,2,235,60,75,85,236,60,228,168,237,60,108,253,238,60,226,82,240,60,70,169,241,60,153,0,243,60,218,88,244,60,8,178,245,60,37,12,247,60,48,103,248,60,41,195,249,60,15,32,251,60,228,125,252,60,166,220,253,60,85,60,255,60,121,78,0,61,63,255,0,61,123,176,1,61,46,98,2,61,88,20,3,61,248,198,3,61,15,122,4,61,156,45,5,61,161,225,5,61,27,150,6,61,12,75,7,61,116,0,8,61,82,182,8,61,167,108,9,61,113,35,10,61,179,218,10,61,106,146,11,61,152,74,12,61,60,3,13,61,87,188,13,61,231,117,14,61,238,47,15,61,107,234,15,61,94,165,16,61,199,96,17,61,166,28,18,61,251,216,18,61,198,149,19,61,7,83,20,61,190,16,21,61,234,206,21,61,141,141,22,61,165,76,23,61,52,12,24,61,56,204,24,61,177,140,25,61,161,77,26,61,6,15,27,61,224,208,27,61,48,147,28,61,246,85,29,61,49,25,30,61,226,220,30,61,8,161,31,61,164,101,32,61,181,42,33,61,59,240,33,61,55,182,34,61,168,124,35,61,142,67,36,61,233,10,37,61,186,210,37,61,255,154,38,61,186,99,39,61,234,44,40,61,143,246,40,61,168,192,41,61,55,139,42,61,59,86,43,61,180,33,44,61,161,237,44,61,4,186,45,61,219,134,46,61,38,84,47,61,231,33,48,61,28,240,48,61,198,190,49,61,229,141,50,61,120,93,51,61,127,45,52,61,251,253,52,61,236,206,53,61,81,160,54,61,42,114,55,61,120,68,56,61,58,23,57,61,112,234,57,61,27,190,58,61,58,146,59,61,204,102,60,61,211,59,61,61,79,17,62,61,62,231,62,61,161,189,63,61,120,148,64,61,195,107,65,61,130,67,66,61,181,27,67,61,92,244,67,61,118,205,68,61,4,167,69,61,6,129,70,61,124,91,71,61,101,54,72,61,194,17,73,61,146,237,73,61,214,201,74,61,141,166,75,61,184,131,76,61,86,97,77,61,104,63,78,61,236,29,79,61,229,252,79,61,80,220,80,61,46,188,81,61,128,156,82,61,69,125,83,61,125,94,84,61,40,64,85,61,69,34,86,61,214,4,87,61,218,231,87,61,81,203,88,61,58,175,89,61,150,147,90,61,101,120,91,61,167,93,92,61,91,67,93,61,130,41,94,61,28,16,95,61,40,247,95,61,167,222,96,61,152,198,97,61,251,174,98,61,209,151,99,61,25,129,100,61,212,106,101,61,0,85,102,61,159,63,103,61,176,42,104,61,51,22,105,61,41,2,106,61,144,238,106,61,105,219,107,61,180,200,108,61,113,182,109,61,160,164,110,61,65,147,111,61,84,130,112,61,216,113,113,61,206,97,114,61,54,82,115,61,15,67,116,61,89,52,117,61,22,38,118,61,67,24,119,61,226,10,120,61,243,253,120,61,117,241,121,61,104,229,122,61,204,217,123,61,162,206,124,61,232,195,125,61,160,185,126,61,201,175,127,61,49,83,128,61,183,206,128,61,117,74,129,61,107,198,129,61,154,66,130,61,1,191,130,61,160,59,131,61,120,184,131,61,136,53,132,61,209,178,132,61,81,48,133,61,10,174,133,61,251,43,134,61,37,170,134,61,134,40,135,61,32,167,135,61,242,37,136,61,252,164,136,61,62,36,137,61,184,163,137,61,106,35,138,61,84,163,138,61,118,35,139,61,209,163,139,61,99,36,140,61,45,165,140,61,46,38,141,61,104,167,141,61,218,40,142,61,131,170,142,61,100,44,143,61,125,174,143,61,206,48,144,61,86,179,144,61,23,54,145,61,14,185,145,61,62,60,146,61,165,191,146,61,67,67,147,61,26,199,147,61,39,75,148,61,109,207,148,61,234,83,149,61,158,216,149,61,138,93,150,61,173,226,150,61,7,104,151,61,153,237,151,61,98,115,152,61,99,249,152,61,155,127,153,61,10,6,154,61,176,140,154,61,142,19,155,61,163,154,155,61,239,33,156,61,114,169,156,61,44,49,157,61,29,185,157,61,69,65,158,61,165,201,158,61,59,82,159,61,8,219,159,61,13,100,160,61,72,237,160,61,186,118,161,61,99,0,162,61,67,138,162,61,90,20,163,61,167,158,163,61,43,41,164,61,230,179,164,61,216,62,165,61,0,202,165,61,95,85,166,61,245,224,166,61,193,108,167,61,196,248,167,61,254,132,168,61,110,17,169,61,20,158,169,61,241,42,170,61,4,184,170,61,78,69,171,61,206,210,171,61,133,96,172,61,113,238,172,61,149,124,173,61,238,10,174,61,126,153,174,61,67,40,175,61,63,183,175,61,114,70,176,61,218,213,176,61,120,101,177,61,77,245,177,61,88,133,178,61,152,21,179,61,15,166,179,61,187,54,180,61,158,199,180,61,182,88,181,61,4,234,181,61,137,123,182,61,67,13,183,61,50,159,183,61,88,49,184,61,179,195,184,61,68,86,185,61,11,233,185,61,7,124,186,61,57,15,187,61,160,162,187,61,61,54,188,61,16,202,188,61,24,94,189,61,85,242,189,61,200,134,190,61,112,27,191,61,78,176,191,61,97,69,192,61,170,218,192,61,39,112,193,61,218,5,194,61,194,155,194,61,224,49,195,61,50,200,195,61,186,94,196,61,119,245,196,61,104,140,197,61,143,35,198,61,235,186,198,61,124,82,199,61,66,234,199,61,61,130,200,61,108,26,201,61,209,178,201,61,106,75,202,61,57,228,202,61,59,125,203,61,115,22,204,61,224,175,204,61,129,73,205,61,86,227,205,61,97,125,206,61,159,23,207,61,19,178,207,61,187,76,208,61,151,231,208,61,168,130,209,61,237,29,210,61,103,185,210,61,21,85,211,61,248,240,211,61,14,141,212,61,89,41,213,61,216,197,213,61,140,98,214,61,115,255,214,61,143,156,215,61,223,57,216,61,99,215,216,61,27,117,217,61,7,19,218,61,38,177,218,61,122,79,219,61,2,238,219,61,189,140,220,61,173,43,221,61,208,202,221,61,39,106,222,61,178,9,223,61,112,169,223,61,98,73,224,61,136,233,224,61,226,137,225,61,111,42,226,61,47,203,226,61,35,108,227,61,74,13,228,61,165,174,228,61,52,80,229,61,245,241,229,61,234,147,230,61,19,54,231,61,110,216,231,61,253,122,232,61,191,29,233,61,180,192,233,61,221,99,234,61,56,7,235,61,199,170,235,61,136,78,236,61,125,242,236,61,164,150,237,61,255,58,238,61,140,223,238,61,76,132,239,61,63,41,240,61,101,206,240,61,189,115,241,61,73,25,242,61,7,191,242,61,247,100,243,61,26,11,244,61,112,177,244,61,248,87,245,61,179,254,245,61,160,165,246,61,192,76,247,61,18,244,247,61,151,155,248,61,77,67,249,61,55,235,249,61,82,147,250,61,159,59,251,61,31,228,251,61,209,140,252,61,181,53,253,61,203,222,253,61,19,136,254,61,141,49,255,61,57,219,255,61,140,66,0,62,148,151,0,62,181,236,0,62,238,65,1,62,65,151,1,62,173,236,1,62,49,66,2,62,206,151,2,62,132,237,2,62,83,67,3,62,59,153,3,62,59,239,3,62,84,69,4,62,134,155,4,62,209,241,4,62,52,72,5,62,176,158,5,62,68,245,5,62,242,75,6,62,183,162,6,62,150,249,6,62,141,80,7,62,156,167,7,62,196,254,7,62,5,86,8,62,94,173,8,62,207,4,9,62,89,92,9,62,252,179,9,62,183,11,10,62,138,99,10,62,118,187,10,62,122,19,11,62,150,107,11,62,203,195,11,62,24,28,12,62,125,116,12,62,250,204,12,62,144,37,13,62,62,126,13,62,4,215,13,62,227,47,14,62,217,136,14,62,232,225,14,62,15,59,15,62,78,148,15,62,165,237,15,62,20,71,16,62,155,160,16,62,58,250,16,62,241,83,17,62,193,173,17,62,168,7,18,62,167,97,18,62,190,187,18,62,237,21,19,62,51,112,19,62,146,202,19,62,9,37,20,62,151,127,20,62,61,218,20,62,251,52,21,62,209,143,21,62,190,234,21,62,195,69,22,62,224,160,22,62,21,252,22,62,97,87,23,62,197,178,23,62,64,14,24,62,211,105,24,62,126,197,24,62,64,33,25,62,26,125,25,62,11,217,25,62,20,53,26,62,52,145,26,62,108,237,26,62,187,73,27,62,34,166,27,62,160,2,28,62,53,95,28,62,226,187,28,62,166,24,29,62,129,117,29,62,116,210,29,62,126,47,30,62,159,140,30,62,215,233,30,62,39,71,31,62,141,164,31,62,11,2,32,62,160,95,32,62,76,189,32,62,16,27,33,62,234,120,33,62,219,214,33,62,228,52,34,62,3,147,34,62,58,241,34,62,135,79,35,62,235,173,35,62,103,12,36,62,249,106,36,62,162,201,36,62,98,40,37,62,56,135,37,62,38,230,37,62,42,69,38,62,69,164,38,62,119,3,39,62,192,98,39,62,31,194,39,62,149,33,40,62,33,129,40,62,197,224,40,62,126,64,41,62,79,160,41,62,54,0,42,62,51,96,42,62,72,192,42,62,114,32,43,62,179,128,43,62,11,225,43,62,121,65,44,62,253,161,44,62,152,2,45,62,73,99,45,62,16,196,45,62,238,36,46,62,226,133,46,62,237,230,46,62,13,72,47,62,68,169,47,62,145,10,48,62,245,107,48,62,110,205,48,62,254,46,49,62,163,144,49,62,95,242,49,62,49,84,50,62,25,182,50,62,23,24,51,62,43,122,51,62,85,220,51,62,148,62,52,62,234,160,52,62,86,3,53,62,216,101,53,62,111,200,53,62,28,43,54,62,223,141,54,62,184,240,54,62,167,83,55,62,171,182,55,62,197,25,56,62,245,124,56,62,59,224,56,62,150,67,57,62,7,167,57,62,141,10,58,62,41,110,58,62,219,209,58,62,162,53,59,62,126,153,59,62,112,253,59,62,120,97,60,62,149,197,60,62,199,41,61,62,15,142,61,62,108,242,61,62,222,86,62,62,102,187,62,62,3,32,63,62,181,132,63,62,125,233,63,62,90,78,64,62,75,179,64,62,83,24,65,62,111,125,65,62,160,226,65,62,231,71,66,62,66,173,66,62,179,18,67,62,57,120,67,62,211,221,67,62,131,67,68,62,71,169,68,62,33,15,69,62,15,117,69,62,18,219,69,62,42,65,70,62,87,167,70,62,153,13,71,62,240,115,71,62,91,218,71,62,219,64,72,62,111,167,72,62,25,14,73,62,215,116,73,62,169,219,73,62,144,66,74,62,140,169,74,62,157,16,75,62,193,119,75,62,251,222,75,62,73,70,76,62,171,173,76,62,34,21,77,62,173,124,77,62,76,228,77,62,0,76,78,62,200,179,78,62,164,27,79,62,149,131,79,62,154,235,79,62,179,83,80,62,225,187,80,62,34,36,81,62,120,140,81,62,225,244,81,62,95,93,82,62,241,197,82,62,151,46,83,62,81,151,83,62,31,0,84,62,1,105,84,62,247,209,84,62,0,59,85,62,30,164,85,62,79,13,86,62,149,118,86,62,238,223,86,62,91,73,87,62,219,178,87,62,112,28,88,62,24,134,88,62,211,239,88,62,163,89,89,62,134,195,89,62,124,45,90,62,134,151,90,62,164,1,91,62,213,107,91,62,26,214,91,62,114,64,92,62,221,170,92,62,92,21,93,62,239,127,93,62,148,234,93,62,77,85,94,62,26,192,94,62,249,42,95,62,236,149,95,62,242,0,96,62,11,108,96,62,55,215,96,62,119,66,97,62,202,173,97,62,47,25,98,62,168,132,98,62,52,240,98,62,210,91,99,62,132,199,99,62,73,51,100,62,32,159,100,62,11,11,101,62,8,119,101,62,24,227,101,62,59,79,102,62,113,187,102,62,186,39,103,62,21,148,103,62,131,0,104,62,3,109,104,62,151,217,104,62,60,70,105,62,245,178,105,62,192,31,106,62,157,140,106,62,141,249,106,62,144,102,107,62,165,211,107,62,204,64,108,62,6,174,108,62,82,27,109,62,176,136,109,62,33,246,109,62,164,99,110,62,57,209,110,62,225,62,111,62,154,172,111,62,102,26,112,62,68,136,112,62,52,246,112,62,55,100,113,62,75,210,113,62,113,64,114,62,169,174,114,62,243,28,115,62,80,139,115,62,190,249,115,62,61,104,116,62,207,214,116,62,115,69,117,62,40,180,117,62,239,34,118,62,200,145,118,62,179,0,119,62,175,111,119,62,189,222,119,62,221,77,120,62,14,189,120,62,80,44,121,62,165,155,121,62,10,11,122,62,130,122,122,62,10,234,122,62,164,89,123,62,80,201,123,62,13,57,124,62,219,168,124,62,186,24,125,62,171,136,125,62,173,248,125,62,192,104,126,62,228,216,126,62,26,73,127,62,96,185,127,62,220,20,128,62,16,77,128,62,77,133,128,62,147,189,128,62,225,245,128,62,55,46,129,62,150,102,129,62,253,158,129,62,109,215,129,62,229,15,130,62,102,72,130,62,238,128,130,62,128,185,130,62,25,242,130,62,187,42,131,62,102,99,131,62,24,156,131,62,211,212,131,62,150,13,132,62,98,70,132,62,53,127,132,62,17,184,132,62,245,240,132,62,226,41,133,62,214,98,133,62,211,155,133,62,216,212,133,62,229,13,134,62,250,70,134,62,23,128,134,62,61,185,134,62,106,242,134,62,160,43,135,62,221,100,135,62,35,158,135,62,112,215,135,62,198,16,136,62,35,74,136,62,137,131,136,62,247,188,136,62,108,246,136,62,233,47,137,62,111,105,137,62,252,162,137,62,145,220,137,62,46,22,138,62,211,79,138,62,127,137,138,62,52,195,138,62,240,252,138,62,180,54,139,62,128,112,139,62,84,170,139,62,47,228,139,62,18,30,140,62,253,87,140,62,239,145,140,62,233,203,140,62,235,5,141,62,245,63,141,62,6,122,141,62,31,180,141,62,63,238,141,62,103,40,142,62,150,98,142,62,205,156,142,62,12,215,142,62,82,17,143,62,159,75,143,62,245,133,143,62,81,192,143,62,181,250,143,62,33,53,144,62,147,111,144,62,14,170,144,62,143,228,144,62,25,31,145,62,169,89,145,62,65,148,145,62,224,206,145,62,134,9,146,62,52,68,146,62,233,126,146,62,165,185,146,62,105,244,146,62,52,47,147,62,6,106,147,62,223,164,147,62,191,223,147,62,167,26,148,62,150,85,148,62,139,144,148,62,136,203,148,62,140,6,149,62,152,65,149,62,170,124,149,62,195,183,149,62,227,242,149,62,11,46,150,62,57,105,150,62,111,164,150,62,171,223,150,62,238,26,151,62,56,86,151,62,138,145,151,62,226,204,151,62,65,8,152,62,167,67,152,62,19,127,152,62,135,186,152,62,1,246,152,62,130,49,153,62,10,109,153,62,153,168,153,62,47,228,153,62,203,31,154,62,110,91,154,62,24,151,154,62,200,210,154,62,127,14,155,62,61,74,155,62,2,134,155,62,205,193,155,62,158,253,155,62,119,57,156,62,85,117,156,62,59,177,156,62,39,237,156,62,25,41,157,62,18,101,157,62,18,161,157,62,24,221,157,62,36,25,158,62,55,85,158,62,80,145,158,62,112,205,158,62,150,9,159,62,195,69,159,62,246,129,159,62,47,190,159,62,111,250,159,62,180,54,160,62,1,115,160,62,83,175,160,62,172,235,160,62,11,40,161,62,112,100,161,62,219,160,161,62,77,221,161,62,196,25,162,62,66,86,162,62,198,146,162,62,81,207,162,62,225,11,163,62,119,72,163,62,20,133,163,62,182,193,163,62,95,254,163,62,13,59,164,62,194,119,164,62,125,180,164,62,61,241,164,62,4,46,165,62,208,106,165,62,162,167,165,62,123,228,165,62,89,33,166,62,61,94,166,62,39,155,166,62,23,216,166,62,12,21,167,62,7,82,167,62,8,143,167,62,15,204,167,62,28,9,168,62,46,70,168,62,70,131,168,62,100,192,168,62,136,253,168,62,177,58,169,62,223,119,169,62,20,181,169,62,78,242,169,62,141,47,170,62,211,108,170,62,29,170,170,62,109,231,170,62,195,36,171,62,31,98,171,62,127,159,171,62,230,220,171,62,81,26,172,62,194,87,172,62,57,149,172,62,181,210,172,62,54,16,173,62,189,77,173,62,73,139,173,62,218,200,173,62,113,6,174,62,13,68,174,62,174,129,174,62,85,191,174,62,0,253,174,62,177,58,175,62,103,120,175,62,35,182,175,62,227,243,175,62,169,49,176,62,116,111,176,62,68,173,176,62,25,235,176,62,243,40,177,62,210,102,177,62,182,164,177,62,160,226,177,62,142,32,178,62,129,94,178,62,121,156,178,62,119,218,178,62,121,24,179,62,128,86,179,62,140,148,179,62,157,210,179,62,178,16,180,62,205,78,180,62,236,140,180,62,16,203,180,62,57,9,181,62,103,71,181,62,154,133,181,62,209,195,181,62,13,2,182,62,78,64,182,62,147,126,182,62,221,188,182,62,44,251,182,62,127,57,183,62,215,119,183,62,52,182,183,62,149,244,183,62,251,50,184,62,101,113,184,62,212,175,184,62,71,238,184,62,191,44,185,62,59,107,185,62,188,169,185,62,65,232,185,62,202,38,186,62,88,101,186,62,235,163,186,62,129,226,186,62,28,33,187,62,188,95,187,62,95,158,187,62,7,221,187,62,180,27,188,62,100,90,188,62,25,153,188,62,210,215,188,62,143,22,189,62,80,85,189,62,22,148,189,62,223,210,189,62,173,17,190,62,127,80,190,62,85,143,190,62,47,206,190,62,13,13,191,62,239,75,191,62,213,138,191,62,191,201,191,62,173,8,192,62,159,71,192,62,149,134,192,62,143,197,192,62,141,4,193,62,143,67,193,62,148,130,193,62,158,193,193,62,171,0,194,62,188,63,194,62,209,126,194,62,234,189,194,62,6,253,194,62,38,60,195,62,74,123,195,62,113,186,195,62,157,249,195,62,204,56,196,62,254,119,196,62,52,183,196,62,110,246,196,62,171,53,197,62,236,116,197,62,49,180,197,62,121,243,197,62,196,50,198,62,19,114,198,62,102,177,198,62,188,240,198,62,21,48,199,62,114,111,199,62,210,174,199,62,54,238,199,62,157,45,200,62,7,109,200,62,117,172,200,62,230,235,200,62,90,43,201,62,209,106,201,62,76,170,201,62,202,233,201,62,75,41,202,62,208,104,202,62,88,168,202,62,226,231,202,62,112,39,203,62,1,103,203,62,149,166,203,62,45,230,203,62,199,37,204,62,100,101,204,62,4,165,204,62,168,228,204,62,78,36,205,62,248,99,205,62,164,163,205,62,83,227,205,62,5,35,206,62,186,98,206,62,114,162,206,62,45,226,206,62,234,33,207,62,171,97,207,62,110,161,207,62,52,225,207,62,253,32,208,62,200,96,208,62,150,160,208,62,103,224,208,62,59,32,209,62,17,96,209,62,234,159,209,62,198,223,209,62,164,31,210,62,133,95,210,62,104,159,210,62,78,223,210,62,55,31,211,62,33,95,211,62,15,159,211,62,255,222,211,62,241,30,212,62,230,94,212,62,221,158,212,62,215,222,212,62,211,30,213,62,209,94,213,62,210,158,213,62,213,222,213,62,219,30,214,62,226,94,214,62,236,158,214,62,248,222,214,62,7,31,215,62,24,95,215,62,42,159,215,62,63,223,215,62,87,31,216,62,112,95,216,62,139,159,216,62,169,223,216,62,200,31,217,62,234,95,217,62,14,160,217,62,51,224,217,62,91,32,218,62,133,96,218,62,176,160,218,62,222,224,218,62,13,33,219,62,63,97,219,62,114,161,219,62,167,225,219,62,222,33,220,62,23,98,220,62,82,162,220,62,142,226,220,62,204,34,221,62,12,99,221,62,78,163,221,62,146,227,221,62,215,35,222,62,29,100,222,62,102,164,222,62,176,228,222,62,252,36,223,62,73,101,223,62,152,165,223,62,232,229,223,62,58,38,224,62,142,102,224,62,227,166,224,62,57,231,224,62,145,39,225,62,234,103,225,62,69,168,225,62,161,232,225,62,255,40,226,62,94,105,226,62,190,169,226,62,32,234,226,62,131,42,227,62,231,106,227,62,76,171,227,62,179,235,227,62,27,44,228,62,132,108,228,62,238,172,228,62,90,237,228,62,199,45,229,62,52,110,229,62,163,174,229,62,19,239,229,62,133,47,230,62,247,111,230,62,106,176,230,62,222,240,230,62,83,49,231,62,202,113,231,62,65,178,231,62,185,242,231,62,50,51,232,62,172,115,232,62,38,180,232,62,162,244,232,62,31,53,233,62,156,117,233,62,26,182,233,62,153,246,233,62,25,55,234,62,153,119,234,62,26,184,234,62,156,248,234,62,31,57,235,62,162,121,235,62,38,186,235,62,170,250,235,62,47,59,236,62,181,123,236,62,59,188,236,62,194,252,236,62,73,61,237,62,209,125,237,62,89,190,237,62,226,254,237,62,107,63,238,62,245,127,238,62,127,192,238,62,10,1,239,62,149,65,239,62,32,130,239,62,171,194,239,62,55,3,240,62,196,67,240,62,80,132,240,62,221,196,240,62,106,5,241,62,247,69,241,62,132,134,241,62,18,199,241,62,160,7,242,62,45,72,242,62,187,136,242,62,74,201,242,62,216,9,243,62,102,74,243,62,244,138,243,62,131,203,243,62,17,12,244,62,159,76,244,62,46,141,244,62,188,205,244,62,74,14,245,62,216,78,245,62,102,143,245,62,244,207,245,62,129,16,246,62,15,81,246,62,156,145,246,62,41,210,246,62,182,18,247,62,67,83,247,62,207,147,247,62,91,212,247,62,231,20,248,62,115,85,248,62,254,149,248,62,136,214,248,62,19,23,249,62,157,87,249,62,38,152,249,62,175,216,249,62,56,25,250,62,192,89,250,62,72,154,250,62,207,218,250,62,86,27,251,62,220,91,251,62,97,156,251,62,230,220,251,62,106,29,252,62,238,93,252,62,113,158,252,62,243,222,252,62,117,31,253,62,245,95,253,62,118,160,253,62,245,224,253,62,116,33,254,62,241,97,254,62,110,162,254,62,235,226,254,62,102,35,255,62,224,99,255,62,90,164,255,62,211,228,255,62,165,18,0,63,225,50,0,63,27,83,0,63,86,115,0,63,144,147,0,63,201,179,0,63,2,212,0,63,58,244,0,63,114,20,1,63,169,52,1,63,224,84,1,63,22,117,1,63,76,149,1,63,129,181,1,63,181,213,1,63,233,245,1,63,28,22,2,63,78,54,2,63,128,86,2,63,178,118,2,63,226,150,2,63,18,183,2,63,65,215,2,63,112,247,2,63,157,23,3,63,203,55,3,63,247,87,3,63,35,120,3,63,78,152,3,63,120,184,3,63,161,216,3,63,202,248,3,63,242,24,4,63,25,57,4,63,63,89,4,63,101,121,4,63,137,153,4,63,173,185,4,63,208,217,4,63,243,249,4,63,20,26,5,63,52,58,5,63,84,90,5,63,115,122,5,63,145,154,5,63,173,186,5,63,202,218,5,63,229,250,5,63,255,26,6,63,24,59,6,63,48,91,6,63,72,123,6,63,94,155,6,63,116,187,6,63,136,219,6,63,155,251,6,63,174,27,7,63,191,59,7,63,208,91,7,63,223,123,7,63,237,155,7,63,250,187,7,63,7,220,7,63,18,252,7,63,28,28,8,63,37,60,8,63,44,92,8,63,51,124,8,63,57,156,8,63,61,188,8,63,64,220,8,63,67,252,8,63,68,28,9,63,68,60,9,63,66,92,9,63,64,124,9,63,60,156,9,63,55,188,9,63,49,220,9,63,41,252,9,63,33,28,10,63,23,60,10,63,12,92,10,63,255,123,10,63,242,155,10,63,227,187,10,63,211,219,10,63,193,251,10,63,174,27,11,63,154,59,11,63,133,91,11,63,110,123,11,63,86,155,11,63,60,187,11,63,33,219,11,63,5,251,11,63,231,26,12,63,200,58,12,63,168,90,12,63,134,122,12,63,98,154,12,63,62,186,12,63,23,218,12,63,240,249,12,63,199,25,13,63,156,57,13,63,112,89,13,63,66,121,13,63,19,153,13,63,227,184,13,63,176,216,13,63,125,248,13,63,72,24,14,63,17,56,14,63,216,87,14,63,159,119,14,63,99,151,14,63,38,183,14,63,232,214,14,63,167,246,14,63,101,22,15,63,34,54,15,63,221,85,15,63,150,117,15,63,78,149,15,63,4,181,15,63,184,212,15,63,106,244,15,63,27,20,16,63,202,51,16,63,120,83,16,63,36,115,16,63,206,146,16,63,118,178,16,63,28,210,16,63,193,241,16,63,100,17,17,63,6,49,17,63,165,80,17,63,67,112,17,63,223,143,17,63,121,175,17,63,17,207,17,63,167,238,17,63,60,14,18,63,206,45,18,63,95,77,18,63,238,108,18,63,123,140,18,63,7,172,18,63,144,203,18,63,23,235,18,63,157,10,19,63,32,42,19,63,162,73,19,63,34,105,19,63,159,136,19,63,27,168,19,63,149,199,19,63,13,231,19,63,131,6,20,63,247,37,20,63,104,69,20,63,216,100,20,63,70,132,20,63,178,163,20,63,27,195,20,63,131,226,20,63,233,1,21,63,76,33,21,63,174,64,21,63,13,96,21,63,106,127,21,63,197,158,21,63,31,190,21,63,117,221,21,63,202,252,21,63,29,28,22,63,109,59,22,63,188,90,22,63,8,122,22,63,82,153,22,63,153,184,22,63,223,215,22,63,34,247,22,63,100,22,23,63,162,53,23,63,223,84,23,63,26,116,23,63,82,147,23,63,136,178,23,63,187,209,23,63,237,240,23,63,28,16,24,63,73,47,24,63,115,78,24,63,155,109,24,63,193,140,24,63,228,171,24,63,6,203,24,63,36,234,24,63,65,9,25,63,91,40,25,63,115,71,25,63,136,102,25,63,155,133,25,63,171,164,25,63,185,195,25,63,197,226,25,63,206,1,26,63,213,32,26,63,217,63,26,63,219,94,26,63,218,125,26,63,215,156,26,63,210,187,26,63,202,218,26,63,191,249,26,63,178,24,27,63,162,55,27,63,144,86,27,63,123,117,27,63,100,148,27,63,74,179,27,63,46,210,27,63,15,241,27,63,237,15,28,63,201,46,28,63,162,77,28,63,121,108,28,63,77,139,28,63,31,170,28,63,237,200,28,63,185,231,28,63,131,6,29,63,74,37,29,63,14,68,29,63,207,98,29,63,142,129,29,63,74,160,29,63,3,191,29,63,186,221,29,63,110,252,29,63,31,27,30,63,205,57,30,63,121,88,30,63,34,119,30,63,200,149,30,63,107,180,30,63,12,211,30,63,170,241,30,63,69,16,31,63,221,46,31,63,114,77,31,63,5,108,31,63,148,138,31,63,33,169,31,63,171,199,31,63,50,230,31,63,182,4,32,63,56,35,32,63,182,65,32,63,50,96,32,63,170,126,32,63,32,157,32,63,147,187,32,63,3,218,32,63,112,248,32,63,218,22,33,63,65,53,33,63,165,83,33,63,6,114,33,63,100,144,33,63,191,174,33,63,23,205,33,63,108,235,33,63,190,9,34,63,13,40,34,63,89,70,34,63,162,100,34,63,232,130,34,63,43,161,34,63,107,191,34,63,167,221,34,63,225,251,34,63,24,26,35,63,75,56,35,63,123,86,35,63,168,116,35,63,211,146,35,63,249,176,35,63,29,207,35,63,62,237,35,63,91,11,36,63,118,41,36,63,141,71,36,63,161,101,36,63,177,131,36,63,191,161,36,63,201,191,36,63,208,221,36,63,212,251,36,63,213,25,37,63,210,55,37,63,204,85,37,63,195,115,37,63,183,145,37,63,167,175,37,63,148,205,37,63,126,235,37,63,101,9,38,63,72,39,38,63,40,69,38,63,4,99,38,63,221,128,38,63,179,158,38,63,134,188,38,63,85,218,38,63,33,248,38,63,233,21,39,63,174,51,39,63,112,81,39,63,46,111,39,63,233,140,39,63,160,170,39,63,84,200,39,63,4,230,39,63,178,3,40,63,91,33,40,63,1,63,40,63,164,92,40,63,67,122,40,63,223,151,40,63,120,181,40,63,12,211,40,63,158,240,40,63,43,14,41,63,182,43,41,63,60,73,41,63,192,102,41,63,63,132,41,63,187,161,41,63,52,191,41,63,169,220,41,63,26,250,41,63,136,23,42,63,242,52,42,63,89,82,42,63,188,111,42,63,28,141,42,63,119,170,42,63,208,199,42,63,36,229,42,63,117,2,43,63,194,31,43,63,12,61,43,63,82,90,43,63,148,119,43,63,211,148,43,63,14,178,43,63,69,207,43,63,120,236,43,63,168,9,44,63,212,38,44,63,252,67,44,63,33,97,44,63,66,126,44,63,95,155,44,63,120,184,44,63,142,213,44,63,159,242,44,63,173,15,45,63,184,44,45,63,190,73,45,63,193,102,45,63,191,131,45,63,186,160,45,63,177,189,45,63,165,218,45,63,148,247,45,63,128,20,46,63,103,49,46,63,75,78,46,63,43,107,46,63,7,136,46,63,224,164,46,63,180,193,46,63,132,222,46,63,81,251,46,63,26,24,47,63,222,52,47,63,159,81,47,63,92,110,47,63,21,139,47,63,202,167,47,63,123,196,47,63,40,225,47,63,209,253,47,63,118,26,48,63,23,55,48,63,180,83,48,63,77,112,48,63,226,140,48,63,115,169,48,63,0,198,48,63,137,226,48,63,14,255,48,63,142,27,49,63,11,56,49,63,132,84,49,63,248,112,49,63,105,141,49,63,214,169,49,63,62,198,49,63,162,226,49,63,2,255,49,63,95,27,50,63,182,55,50,63,10,84,50,63,90,112,50,63,166,140,50,63,237,168,50,63,48,197,50,63,111,225,50,63,170,253,50,63,225,25,51,63,19,54,51,63,66,82,51,63,108,110,51,63,146,138,51,63,180,166,51,63,209,194,51,63,234,222,51,63,0,251,51,63,16,23,52,63,29,51,52,63,37,79,52,63,41,107,52,63,41,135,52,63,37,163,52,63,28,191,52,63,15,219,52,63,253,246,52,63,232,18,53,63,206,46,53,63,176,74,53,63,141,102,53,63,102,130,53,63,59,158,53,63,11,186,53,63,215,213,53,63,159,241,53,63,98,13,54,63,33,41,54,63,220,68,54,63,146,96,54,63,68,124,54,63,241,151,54,63,154,179,54,63,63,207,54,63,223,234,54,63,123,6,55,63,18,34,55,63,165,61,55,63,52,89,55,63,190,116,55,63,67,144,55,63,196,171,55,63,65,199,55,63,185,226,55,63,45,254,55,63,156,25,56,63,7,53,56,63,109,80,56,63,207,107,56,63,44,135,56,63,133,162,56,63,217,189,56,63,40,217,56,63,115,244,56,63,186,15,57,63,252,42,57,63,57,70,57,63,114,97,57,63,166,124,57,63,214,151,57,63,1,179,57,63,40,206,57,63,74,233,57,63,103,4,58,63,128,31,58,63,148,58,58,63,163,85,58,63,174,112,58,63,180,139,58,63,182,166,58,63,179,193,58,63,171,220,58,63,159,247,58,63,142,18,59,63,120,45,59,63,94,72,59,63,63,99,59,63,27,126,59,63,243,152,59,63,197,179,59,63,148,206,59,63,93,233,59,63,34,4,60,63,226,30,60,63,157,57,60,63,84,84,60,63,5,111,60,63,178,137,60,63,91,164,60,63,254,190,60,63,157,217,60,63,55,244,60,63,204,14,61,63,93,41,61,63,232,67,61,63,111,94,61,63,241,120,61,63,110,147,61,63,231,173,61,63,91,200,61,63,201,226,61,63,51,253,61,63,152,23,62,63,249,49,62,63,84,76,62,63,171,102,62,63,252,128,62,63,73,155,62,63,145,181,62,63,212,207,62,63,19,234,62,63,76,4,63,63,128,30,63,63,176,56,63,63,219,82,63,63,0,109,63,63,33,135,63,63,61,161,63,63,84,187,63,63,102,213,63,63,115,239,63,63,123,9,64,63,127,35,64,63,125,61,64,63,118,87,64,63,106,113,64,63,90,139,64,63,68,165,64,63,42,191,64,63,10,217,64,63,229,242,64,63,188,12,65,63,141,38,65,63,90,64,65,63,33,90,65,63,228,115,65,63,161,141,65,63,89,167,65,63,13,193,65,63,187,218,65,63,100,244,65,63,8,14,66,63,167,39,66,63,65,65,66,63,214,90,66,63,102,116,66,63,241,141,66,63,119,167,66,63,248,192,66,63,115,218,66,63,234,243,66,63,91,13,67,63,199,38,67,63,47,64,67,63,145,89,67,63,238,114,67,63,69,140,67,63,152,165,67,63,230,190,67,63,46,216,67,63,113,241,67,63,175,10,68,63,232,35,68,63,28,61,68,63,75,86,68,63,116,111,68,63,153,136,68,63,184,161,68,63,210,186,68,63,230,211,68,63,246,236,68,63,0,6,69,63,5,31,69,63,5,56,69,63,0,81,69,63,245,105,69,63,230,130,69,63,209,155,69,63,182,180,69,63,151,205,69,63,114,230,69,63,72,255,69,63,25,24,70,63,229,48,70,63,171,73,70,63,108,98,70,63,40,123,70,63,222,147,70,63,143,172,70,63,59,197,70,63,226,221,70,63,131,246,70,63,31,15,71,63,182,39,71,63,71,64,71,63,211,88,71,63,90,113,71,63,220,137,71,63,88,162,71,63,207,186,71,63,64,211,71,63,172,235,71,63,19,4,72,63,116,28,72,63,209,52,72,63,39,77,72,63,121,101,72,63,197,125,72,63,11,150,72,63,77,174,72,63,137,198,72,63,191,222,72,63,240,246,72,63,28,15,73,63,66,39,73,63,99,63,73,63,127,87,73,63,149,111,73,63,166,135,73,63,177,159,73,63,183,183,73,63,183,207,73,63,178,231,73,63,168,255,73,63,152,23,74,63,131,47,74,63,104,71,74,63,72,95,74,63,34,119,74,63,247,142,74,63,199,166,74,63,145,190,74,63,85,214,74,63,20,238,74,63,206,5,75,63,130,29,75,63,49,53,75,63,218,76,75,63,126,100,75,63,28,124,75,63,181,147,75,63,72,171,75,63,213,194,75,63,93,218,75,63,224,241,75,63,93,9,76,63,213,32,76,63,71,56,76,63,179,79,76,63,26,103,76,63,124,126,76,63,216,149,76,63,46,173,76,63,127,196,76,63,202,219,76,63,16,243,76,63,80,10,77,63,139,33,77,63,192,56,77,63,240,79,77,63,26,103,77,63,62,126,77,63,93,149,77,63,118,172,77,63,137,195,77,63,151,218,77,63,160,241,77,63,163,8,78,63,160,31,78,63,151,54,78,63,137,77,78,63,118,100,78,63,93,123,78,63,62,146,78,63,25,169,78,63,239,191,78,63,192,214,78,63,138,237,78,63,79,4,79,63,15,27,79,63,201,49,79,63,125,72,79,63,43,95,79,63,212,117,79,63,119,140,79,63,21,163,79,63,172,185,79,63,63,208,79,63,203,230,79,63,82,253,79,63,211,19,80,63,79,42,80,63,197,64,80,63,53,87,80,63,159,109,80,63,4,132,80,63,99,154,80,63,189,176,80,63,16,199,80,63,94,221,80,63,167,243,80,63,233,9,81,63,38,32,81,63].concat([93,54,81,63,143,76,81,63,187,98,81,63,225,120,81,63,1,143,81,63,28,165,81,63,48,187,81,63,64,209,81,63,73,231,81,63,77,253,81,63,75,19,82,63,67,41,82,63,53,63,82,63,34,85,82,63,9,107,82,63,234,128,82,63,198,150,82,63,155,172,82,63,107,194,82,63,53,216,82,63,250,237,82,63,185,3,83,63,113,25,83,63,37,47,83,63,210,68,83,63,121,90,83,63,27,112,83,63,183,133,83,63,77,155,83,63,222,176,83,63,104,198,83,63,237,219,83,63,108,241,83,63,230,6,84,63,89,28,84,63,199,49,84,63,46,71,84,63,145,92,84,63,237,113,84,63,67,135,84,63,148,156,84,63,223,177,84,63,35,199,84,63,99,220,84,63,156,241,84,63,207,6,85,63,253,27,85,63,37,49,85,63,71,70,85,63,99,91,85,63,121,112,85,63,138,133,85,63,149,154,85,63,153,175,85,63,152,196,85,63,146,217,85,63,133,238,85,63,114,3,86,63,90,24,86,63,60,45,86,63,24,66,86,63,238,86,86,63,190,107,86,63,136,128,86,63,76,149,86,63,11,170,86,63,196,190,86,63,118,211,86,63,35,232,86,63,203,252,86,63,108,17,87,63,7,38,87,63,156,58,87,63,44,79,87,63,182,99,87,63,58,120,87,63,183,140,87,63,47,161,87,63,162,181,87,63,14,202,87,63,116,222,87,63,213,242,87,63,47,7,88,63,132,27,88,63,211,47,88,63,28,68,88,63,95,88,88,63,156,108,88,63,211,128,88,63,4,149,88,63,47,169,88,63,85,189,88,63,116,209,88,63,142,229,88,63,162,249,88,63,175,13,89,63,183,33,89,63,185,53,89,63,181,73,89,63,171,93,89,63,155,113,89,63,134,133,89,63,106,153,89,63,72,173,89,63,33,193,89,63,243,212,89,63,192,232,89,63,135,252,89,63,71,16,90,63,2,36,90,63,183,55,90,63,102,75,90,63,15,95,90,63,178,114,90,63,79,134,90,63,230,153,90,63,119,173,90,63,3,193,90,63,136,212,90,63,7,232,90,63,129,251,90,63,244,14,91,63,98,34,91,63,201,53,91,63,43,73,91,63,135,92,91,63,220,111,91,63,44,131,91,63,118,150,91,63,186,169,91,63,248,188,91,63,47,208,91,63,97,227,91,63,141,246,91,63,179,9,92,63,212,28,92,63,238,47,92,63,2,67,92,63,16,86,92,63,24,105,92,63,26,124,92,63,23,143,92,63,13,162,92,63,253,180,92,63,232,199,92,63,204,218,92,63,171,237,92,63,131,0,93,63,86,19,93,63,34,38,93,63,233,56,93,63,169,75,93,63,100,94,93,63,24,113,93,63,199,131,93,63,112,150,93,63,18,169,93,63,175,187,93,63,70,206,93,63,215,224,93,63,97,243,93,63,230,5,94,63,101,24,94,63,222,42,94,63,81,61,94,63,190,79,94,63,36,98,94,63,133,116,94,63,224,134,94,63,53,153,94,63,132,171,94,63,205,189,94,63,16,208,94,63,77,226,94,63,132,244,94,63,181,6,95,63,224,24,95,63,5,43,95,63,36,61,95,63,61,79,95,63,80,97,95,63,93,115,95,63,101,133,95,63,102,151,95,63,97,169,95,63,86,187,95,63,69,205,95,63,46,223,95,63,18,241,95,63,239,2,96,63,198,20,96,63,151,38,96,63,98,56,96,63,40,74,96,63,231,91,96,63,160,109,96,63,84,127,96,63,1,145,96,63,168,162,96,63,73,180,96,63,229,197,96,63,122,215,96,63,10,233,96,63,147,250,96,63,22,12,97,63,148,29,97,63,11,47,97,63,125,64,97,63,232,81,97,63,77,99,97,63,173,116,97,63,6,134,97,63,90,151,97,63,167,168,97,63,239,185,97,63,48,203,97,63,108,220,97,63,162,237,97,63,209,254,97,63,251,15,98,63,30,33,98,63,60,50,98,63,84,67,98,63,101,84,98,63,113,101,98,63,119,118,98,63,119,135,98,63,112,152,98,63,100,169,98,63,82,186,98,63,58,203,98,63,28,220,98,63,247,236,98,63,205,253,98,63,157,14,99,63,103,31,99,63,43,48,99,63,233,64,99,63,161,81,99,63,83,98,99,63,255,114,99,63,165,131,99,63,69,148,99,63,224,164,99,63,116,181,99,63,2,198,99,63,138,214,99,63,13,231,99,63,137,247,99,63,255,7,100,63,112,24,100,63,218,40,100,63,62,57,100,63,157,73,100,63,246,89,100,63,72,106,100,63,149,122,100,63,219,138,100,63,28,155,100,63,87,171,100,63,140,187,100,63,186,203,100,63,227,219,100,63,6,236,100,63,35,252,100,63,58,12,101,63,75,28,101,63,86,44,101,63,91,60,101,63,91,76,101,63,84,92,101,63,71,108,101,63,53,124,101,63,28,140,101,63,254,155,101,63,217,171,101,63,175,187,101,63,126,203,101,63,72,219,101,63,12,235,101,63,202,250,101,63,130,10,102,63,52,26,102,63,224,41,102,63,134,57,102,63,38,73,102,63,193,88,102,63,85,104,102,63,227,119,102,63,108,135,102,63,238,150,102,63,107,166,102,63,226,181,102,63,83,197,102,63,190,212,102,63,35,228,102,63,130,243,102,63,219,2,103,63,46,18,103,63,124,33,103,63,195,48,103,63,5,64,103,63,64,79,103,63,118,94,103,63,166,109,103,63,208,124,103,63,244,139,103,63,18,155,103,63,42,170,103,63,61,185,103,63,73,200,103,63,80,215,103,63,80,230,103,63,75,245,103,63,64,4,104,63,47,19,104,63,24,34,104,63,251,48,104,63,217,63,104,63,176,78,104,63,130,93,104,63,78,108,104,63,20,123,104,63,212,137,104,63,142,152,104,63,66,167,104,63,240,181,104,63,153,196,104,63,60,211,104,63,217,225,104,63,112,240,104,63,1,255,104,63,140,13,105,63,17,28,105,63,145,42,105,63,11,57,105,63,127,71,105,63,237,85,105,63,85,100,105,63,183,114,105,63,20,129,105,63,106,143,105,63,187,157,105,63,6,172,105,63,75,186,105,63,139,200,105,63,196,214,105,63,248,228,105,63,38,243,105,63,78,1,106,63,112,15,106,63,141,29,106,63,163,43,106,63,180,57,106,63,191,71,106,63,196,85,106,63,196,99,106,63,189,113,106,63,177,127,106,63,159,141,106,63,135,155,106,63,106,169,106,63,70,183,106,63,29,197,106,63,238,210,106,63,186,224,106,63,127,238,106,63,63,252,106,63,249,9,107,63,173,23,107,63,91,37,107,63,4,51,107,63,167,64,107,63,68,78,107,63,219,91,107,63,109,105,107,63,249,118,107,63,127,132,107,63,255,145,107,63,122,159,107,63,238,172,107,63,94,186,107,63,199,199,107,63,42,213,107,63,136,226,107,63,224,239,107,63,51,253,107,63,128,10,108,63,198,23,108,63,8,37,108,63,67,50,108,63,121,63,108,63,169,76,108,63,211,89,108,63,248,102,108,63,23,116,108,63,48,129,108,63,68,142,108,63,82,155,108,63,90,168,108,63,92,181,108,63,89,194,108,63,80,207,108,63,65,220,108,63,45,233,108,63,19,246,108,63,243,2,109,63,206,15,109,63,163,28,109,63,114,41,109,63,60,54,109,63,0,67,109,63,190,79,109,63,119,92,109,63,42,105,109,63,215,117,109,63,127,130,109,63,33,143,109,63,189,155,109,63,84,168,109,63,229,180,109,63,113,193,109,63,247,205,109,63,119,218,109,63,242,230,109,63,103,243,109,63,214,255,109,63,64,12,110,63,164,24,110,63,3,37,110,63,91,49,110,63,175,61,110,63,253,73,110,63,69,86,110,63,135,98,110,63,196,110,110,63,252,122,110,63,45,135,110,63,90,147,110,63,128,159,110,63,161,171,110,63,189,183,110,63,211,195,110,63,227,207,110,63,238,219,110,63,243,231,110,63,243,243,110,63,237,255,110,63,226,11,111,63,209,23,111,63,186,35,111,63,158,47,111,63,125,59,111,63,85,71,111,63,41,83,111,63,247,94,111,63,191,106,111,63,130,118,111,63,63,130,111,63,247,141,111,63,169,153,111,63,86,165,111,63,253,176,111,63,159,188,111,63,59,200,111,63,210,211,111,63,99,223,111,63,239,234,111,63,117,246,111,63,246,1,112,63,114,13,112,63,231,24,112,63,88,36,112,63,195,47,112,63,40,59,112,63,137,70,112,63,227,81,112,63,56,93,112,63,136,104,112,63,210,115,112,63,23,127,112,63,87,138,112,63,145,149,112,63,197,160,112,63,244,171,112,63,30,183,112,63,66,194,112,63,97,205,112,63,123,216,112,63,143,227,112,63,157,238,112,63,167,249,112,63,171,4,113,63,169,15,113,63,162,26,113,63,150,37,113,63,132,48,113,63,109,59,113,63,81,70,113,63,47,81,113,63,8,92,113,63,219,102,113,63,170,113,113,63,114,124,113,63,54,135,113,63,244,145,113,63,173,156,113,63,96,167,113,63,14,178,113,63,183,188,113,63,91,199,113,63,249,209,113,63,146,220,113,63,37,231,113,63,179,241,113,63,60,252,113,63,192,6,114,63,62,17,114,63,183,27,114,63,43,38,114,63,154,48,114,63,3,59,114,63,103,69,114,63,197,79,114,63,31,90,114,63,115,100,114,63,194,110,114,63,11,121,114,63,79,131,114,63,143,141,114,63,200,151,114,63,253,161,114,63,44,172,114,63,87,182,114,63,123,192,114,63,155,202,114,63,182,212,114,63,203,222,114,63,219,232,114,63,230,242,114,63,235,252,114,63,236,6,115,63,231,16,115,63,221,26,115,63,206,36,115,63,186,46,115,63,160,56,115,63,130,66,115,63,94,76,115,63,53,86,115,63,7,96,115,63,212,105,115,63,155,115,115,63,94,125,115,63,27,135,115,63,211,144,115,63,134,154,115,63,52,164,115,63,221,173,115,63,128,183,115,63,31,193,115,63,184,202,115,63,77,212,115,63,220,221,115,63,102,231,115,63,235,240,115,63,107,250,115,63,230,3,116,63,92,13,116,63,204,22,116,63,56,32,116,63,159,41,116,63,0,51,116,63,93,60,116,63,180,69,116,63,6,79,116,63,84,88,116,63,156,97,116,63,223,106,116,63,29,116,116,63,87,125,116,63,139,134,116,63,186,143,116,63,228,152,116,63,9,162,116,63,41,171,116,63,68,180,116,63,91,189,116,63,108,198,116,63,120,207,116,63,127,216,116,63,129,225,116,63,127,234,116,63,119,243,116,63,106,252,116,63,89,5,117,63,66,14,117,63,38,23,117,63,6,32,117,63,225,40,117,63,182,49,117,63,135,58,117,63,83,67,117,63,26,76,117,63,220,84,117,63,153,93,117,63,81,102,117,63,4,111,117,63,179,119,117,63,92,128,117,63,1,137,117,63,160,145,117,63,59,154,117,63,209,162,117,63,98,171,117,63,239,179,117,63,118,188,117,63,249,196,117,63,118,205,117,63,239,213,117,63,99,222,117,63,210,230,117,63,61,239,117,63,162,247,117,63,3,0,118,63,95,8,118,63,182,16,118,63,8,25,118,63,86,33,118,63,159,41,118,63,227,49,118,63,34,58,118,63,92,66,118,63,146,74,118,63,195,82,118,63,239,90,118,63,22,99,118,63,57,107,118,63,86,115,118,63,112,123,118,63,132,131,118,63,148,139,118,63,158,147,118,63,165,155,118,63,166,163,118,63,163,171,118,63,155,179,118,63,142,187,118,63,125,195,118,63,103,203,118,63,76,211,118,63,45,219,118,63,9,227,118,63,224,234,118,63,178,242,118,63,128,250,118,63,74,2,119,63,14,10,119,63,206,17,119,63,137,25,119,63,64,33,119,63,242,40,119,63,160,48,119,63,72,56,119,63,237,63,119,63,140,71,119,63,39,79,119,63,190,86,119,63,79,94,119,63,220,101,119,63,101,109,119,63,233,116,119,63,105,124,119,63,228,131,119,63,90,139,119,63,204,146,119,63,57,154,119,63,162,161,119,63,6,169,119,63,101,176,119,63,192,183,119,63,23,191,119,63,105,198,119,63,182,205,119,63,255,212,119,63,68,220,119,63,132,227,119,63,191,234,119,63,246,241,119,63,41,249,119,63,87,0,120,63,129,7,120,63,166,14,120,63,198,21,120,63,227,28,120,63,250,35,120,63,14,43,120,63,28,50,120,63,39,57,120,63,45,64,120,63,46,71,120,63,44,78,120,63,36,85,120,63,25,92,120,63,9,99,120,63,244,105,120,63,219,112,120,63,190,119,120,63,156,126,120,63,118,133,120,63,76,140,120,63,29,147,120,63,234,153,120,63,179,160,120,63,119,167,120,63,55,174,120,63,242,180,120,63,169,187,120,63,92,194,120,63,11,201,120,63,181,207,120,63,91,214,120,63,252,220,120,63,154,227,120,63,51,234,120,63,199,240,120,63,88,247,120,63,228,253,120,63,108,4,121,63,240,10,121,63,111,17,121,63,234,23,121,63,97,30,121,63,211,36,121,63,66,43,121,63,172,49,121,63,18,56,121,63,116,62,121,63,209,68,121,63,42,75,121,63,127,81,121,63,208,87,121,63,29,94,121,63,101,100,121,63,170,106,121,63,234,112,121,63,38,119,121,63,93,125,121,63,145,131,121,63,193,137,121,63,236,143,121,63,19,150,121,63,54,156,121,63,85,162,121,63,112,168,121,63,134,174,121,63,153,180,121,63,167,186,121,63,178,192,121,63,184,198,121,63,186,204,121,63,184,210,121,63,178,216,121,63,168,222,121,63,154,228,121,63,135,234,121,63,113,240,121,63,87,246,121,63,56,252,121,63,22,2,122,63,239,7,122,63,197,13,122,63,150,19,122,63,100,25,122,63,45,31,122,63,243,36,122,63,180,42,122,63,113,48,122,63,43,54,122,63,224,59,122,63,146,65,122,63,63,71,122,63,233,76,122,63,142,82,122,63,48,88,122,63,206,93,122,63,103,99,122,63,253,104,122,63,143,110,122,63,29,116,122,63,167,121,122,63,45,127,122,63,175,132,122,63,45,138,122,63,168,143,122,63,30,149,122,63,145,154,122,63,255,159,122,63,106,165,122,63,209,170,122,63,52,176,122,63,147,181,122,63,239,186,122,63,70,192,122,63,154,197,122,63,234,202,122,63,54,208,122,63,126,213,122,63,194,218,122,63,3,224,122,63,64,229,122,63,121,234,122,63,174,239,122,63,223,244,122,63,13,250,122,63,55,255,122,63,93,4,123,63,127,9,123,63,157,14,123,63,184,19,123,63,207,24,123,63,227,29,123,63,242,34,123,63,254,39,123,63,6,45,123,63,10,50,123,63,11,55,123,63,8,60,123,63,1,65,123,63,247,69,123,63,233,74,123,63,215,79,123,63,193,84,123,63,168,89,123,63,139,94,123,63,107,99,123,63,71,104,123,63,31,109,123,63,243,113,123,63,196,118,123,63,146,123,123,63,91,128,123,63,33,133,123,63,228,137,123,63,163,142,123,63,94,147,123,63,22,152,123,63,202,156,123,63,122,161,123,63,39,166,123,63,208,170,123,63,118,175,123,63,24,180,123,63,183,184,123,63,82,189,123,63,233,193,123,63,125,198,123,63,14,203,123,63,155,207,123,63,36,212,123,63,170,216,123,63,45,221,123,63,172,225,123,63,39,230,123,63,159,234,123,63,19,239,123,63,132,243,123,63,242,247,123,63,92,252,123,63,195,0,124,63,38,5,124,63,133,9,124,63,226,13,124,63,58,18,124,63,144,22,124,63,226,26,124,63,48,31,124,63,123,35,124,63,195,39,124,63,7,44,124,63,72,48,124,63,134,52,124,63,192,56,124,63,247,60,124,63,42,65,124,63,90,69,124,63,135,73,124,63,176,77,124,63,214,81,124,63,249,85,124,63,24,90,124,63,52,94,124,63,77,98,124,63,98,102,124,63,116,106,124,63,131,110,124,63,142,114,124,63,150,118,124,63,155,122,124,63,157,126,124,63,155,130,124,63,150,134,124,63,142,138,124,63,130,142,124,63,116,146,124,63,98,150,124,63,77,154,124,63,52,158,124,63,24,162,124,63,249,165,124,63,215,169,124,63,178,173,124,63,137,177,124,63,94,181,124,63,47,185,124,63,253,188,124,63,199,192,124,63,143,196,124,63,83,200,124,63,20,204,124,63,211,207,124,63,141,211,124,63,69,215,124,63,250,218,124,63,171,222,124,63,90,226,124,63,5,230,124,63,173,233,124,63,82,237,124,63,244,240,124,63,147,244,124,63,46,248,124,63,199,251,124,63,93,255,124,63,239,2,125,63,127,6,125,63,11,10,125,63,148,13,125,63,27,17,125,63,158,20,125,63,30,24,125,63,155,27,125,63,21,31,125,63,140,34,125,63,0,38,125,63,114,41,125,63,224,44,125,63,75,48,125,63,179,51,125,63,24,55,125,63,122,58,125,63,217,61,125,63,54,65,125,63,143,68,125,63,229,71,125,63,56,75,125,63,137,78,125,63,214,81,125,63,33,85,125,63,104,88,125,63,173,91,125,63,239,94,125,63,46,98,125,63,106,101,125,63,163,104,125,63,217,107,125,63,12,111,125,63,61,114,125,63,106,117,125,63,149,120,125,63,189,123,125,63,226,126,125,63,4,130,125,63,36,133,125,63,64,136,125,63,90,139,125,63,112,142,125,63,133,145,125,63,150,148,125,63,164,151,125,63,176,154,125,63,185,157,125,63,191,160,125,63,194,163,125,63,194,166,125,63,192,169,125,63,187,172,125,63,179,175,125,63,168,178,125,63,155,181,125,63,139,184,125,63,120,187,125,63,99,190,125,63,74,193,125,63,48,196,125,63,18,199,125,63,241,201,125,63,206,204,125,63,169,207,125,63,128,210,125,63,85,213,125,63,39,216,125,63,247,218,125,63,196,221,125,63,142,224,125,63,85,227,125,63,26,230,125,63,220,232,125,63,156,235,125,63,89,238,125,63,19,241,125,63,203,243,125,63,128,246,125,63,51,249,125,63,227,251,125,63,144,254,125,63,59,1,126,63,227,3,126,63,137,6,126,63,44,9,126,63,204,11,126,63,106,14,126,63,6,17,126,63,158,19,126,63,53,22,126,63,200,24,126,63,90,27,126,63,232,29,126,63,116,32,126,63,254,34,126,63,133,37,126,63,10,40,126,63,140,42,126,63,12,45,126,63,137,47,126,63,4,50,126,63,124,52,126,63,242,54,126,63,101,57,126,63,214,59,126,63,68,62,126,63,176,64,126,63,26,67,126,63,129,69,126,63,230,71,126,63,72,74,126,63,168,76,126,63,5,79,126,63,96,81,126,63,185,83,126,63,15,86,126,63,99,88,126,63,181,90,126,63,4,93,126,63,81,95,126,63,155,97,126,63,227,99,126,63,41,102,126,63,108,104,126,63,173,106,126,63,236,108,126,63,40,111,126,63,98,113,126,63,154,115,126,63,208,117,126,63,3,120,126,63,51,122,126,63,98,124,126,63,142,126,126,63,184,128,126,63,224,130,126,63,5,133,126,63,40,135,126,63,73,137,126,63,104,139,126,63,132,141,126,63,159,143,126,63,183,145,126,63,204,147,126,63,224,149,126,63,241,151,126,63,0,154,126,63,13,156,126,63,24,158,126,63,32,160,126,63,38,162,126,63,42,164,126,63,44,166,126,63,44,168,126,63,41,170,126,63,37,172,126,63,30,174,126,63,21,176,126,63,10,178,126,63,253,179,126,63,238,181,126,63,220,183,126,63,201,185,126,63,179,187,126,63,155,189,126,63,129,191,126,63,101,193,126,63,71,195,126,63,39,197,126,63,5,199,126,63,224,200,126,63,186,202,126,63,145,204,126,63,103,206,126,63,58,208,126,63,12,210,126,63,219,211,126,63,168,213,126,63,115,215,126,63,61,217,126,63,4,219,126,63,201,220,126,63,140,222,126,63,77,224,126,63,12,226,126,63,202,227,126,63,133,229,126,63,62,231,126,63,245,232,126,63,170,234,126,63,94,236,126,63,15,238,126,63,190,239,126,63,108,241,126,63,23,243,126,63,193,244,126,63,104,246,126,63,14,248,126,63,178,249,126,63,84,251,126,63,243,252,126,63,145,254,126,63,46,0,127,63,200,1,127,63,96,3,127,63,247,4,127,63,139,6,127,63,30,8,127,63,175,9,127,63,62,11,127,63,203,12,127,63,86,14,127,63,223,15,127,63,103,17,127,63,237,18,127,63,112,20,127,63,242,21,127,63,115,23,127,63,241,24,127,63,110,26,127,63,233,27,127,63,98,29,127,63,217,30,127,63,78,32,127,63,194,33,127,63,52,35,127,63,164,36,127,63,18,38,127,63,127,39,127,63,234,40,127,63,83,42,127,63,186,43,127,63,32,45,127,63,131,46,127,63,230,47,127,63,70,49,127,63,165,50,127,63,2,52,127,63,93,53,127,63,182,54,127,63,14,56,127,63,100,57,127,63,185,58,127,63,12,60,127,63,93,61,127,63,172,62,127,63,250,63,127,63,70,65,127,63,145,66,127,63,217,67,127,63,33,69,127,63,102,70,127,63,170,71,127,63,236,72,127,63,45,74,127,63,108,75,127,63,169,76,127,63,229,77,127,63,31,79,127,63,88,80,127,63,143,81,127,63,196,82,127,63,248,83,127,63,42,85,127,63,91,86,127,63,138,87,127,63,184,88,127,63,228,89,127,63,14,91,127,63,55,92,127,63,94,93,127,63,132,94,127,63,169,95,127,63,203,96,127,63,237,97,127,63,12,99,127,63,42,100,127,63,71,101,127,63,98,102,127,63,124,103,127,63,148,104,127,63,171,105,127,63,192,106,127,63,212,107,127,63,230,108,127,63,247,109,127,63,6,111,127,63,20,112,127,63,33,113,127,63,44,114,127,63,53,115,127,63,61,116,127,63,68,117,127,63,73,118,127,63,77,119,127,63,79,120,127,63,80,121,127,63,80,122,127,63,78,123,127,63,75,124,127,63,70,125,127,63,64,126,127,63,57,127,127,63,48,128,127,63,38,129,127,63,27,130,127,63,14,131,127,63,0,132,127,63,240,132,127,63,223,133,127,63,205,134,127,63,185,135,127,63,164,136,127,63,142,137,127,63,118,138,127,63,93,139,127,63,67,140,127,63,40,141,127,63,11,142,127,63,237,142,127,63,205,143,127,63,173,144,127,63,139,145,127,63,103,146,127,63,67,147,127,63,29,148,127,63,246,148,127,63,205,149,127,63,164,150,127,63,121,151,127,63,77,152,127,63,31,153,127,63,241,153,127,63,193,154,127,63,144,155,127,63,93,156,127,63,42,157,127,63,245,157,127,63,191,158,127,63,136,159,127,63,79,160,127,63,22,161,127,63,219,161,127,63,159,162,127,63,98,163,127,63,36,164,127,63,228,164,127,63,163,165,127,63,98,166,127,63,31,167,127,63,219,167,127,63,149,168,127,63,79,169,127,63,7,170,127,63,190,170,127,63,117,171,127,63,42,172,127,63,221,172,127,63,144,173,127,63,66,174,127,63,242,174,127,63,162,175,127,63,80,176,127,63,253,176,127,63,169,177,127,63,85,178,127,63,254,178,127,63,167,179,127,63,79,180,127,63,246,180,127,63,156,181,127,63,64,182,127,63,228,182,127,63,134,183,127,63,40,184,127,63,200,184,127,63,103,185,127,63,6,186,127,63,163,186,127,63,63,187,127,63,219,187,127,63,117,188,127,63,14,189,127,63,166,189,127,63,61,190,127,63,212,190,127,63,105,191,127,63,253,191,127,63,144,192,127,63,34,193,127,63,180,193,127,63,68,194,127,63,211,194,127,63,98,195,127,63,239,195,127,63,123,196,127,63,7,197,127,63,145,197,127,63,27,198,127,63,163,198,127,63,43,199,127,63,178,199,127,63,56,200,127,63,189,200,127,63,65,201,127,63,196,201,127,63,70,202,127,63,199,202,127,63,71,203,127,63,199,203,127,63,69,204,127,63,195,204,127,63,64,205,127,63,187,205,127,63,54,206,127,63,177,206,127,63,42,207,127,63,162,207,127,63,26,208,127,63,144,208,127,63,6,209,127,63,123,209,127,63,239,209,127,63,98,210,127,63,213,210,127,63,70,211,127,63,183,211,127,63,39,212,127,63,150,212,127,63,4,213,127,63,114,213,127,63,222,213,127,63,74,214,127,63,181,214,127,63,32,215,127,63,137,215,127,63,242,215,127,63,89,216,127,63,192,216,127,63,39,217,127,63,140,217,127,63,241,217,127,63,85,218,127,63,184,218,127,63,27,219,127,63,124,219,127,63,221,219,127,63,61,220,127,63,157,220,127,63,251,220,127,63,89,221,127,63,183,221,127,63,19,222,127,63,111,222,127,63,202,222,127,63,36,223,127,63,126,223,127,63,215,223,127,63,47,224,127,63,134,224,127,63,221,224,127,63,51,225,127,63,137,225,127,63,221,225,127,63,49,226,127,63,133,226,127,63,215,226,127,63,41,227,127,63,122,227,127,63,203,227,127,63,27,228,127,63,106,228,127,63,185,228,127,63,7,229,127,63,84,229,127,63,161,229,127,63,237,229,127,63,56,230,127,63,131,230,127,63,205,230,127,63,23,231,127,63,96,231,127,63,168,231,127,63,239,231,127,63,54,232,127,63,125,232,127,63,195,232,127,63,8,233,127,63,76,233,127,63,144,233,127,63,212,233,127,63,23,234,127,63,89,234,127,63,154,234,127,63,219,234,127,63,28,235,127,63,92,235,127,63,155,235,127,63,218,235,127,63,24,236,127,63,86,236,127,63,147,236,127,63,207,236,127,63,11,237,127,63,71,237,127,63,130,237,127,63,188,237,127,63,246,237,127,63,47,238,127,63,104,238,127,63,160,238,127,63,216,238,127,63,15,239,127,63,69,239,127,63,123,239,127,63,177,239,127,63,230,239,127,63,27,240,127,63,79,240,127,63,130,240,127,63,182,240,127,63,232,240,127,63,26,241,127,63,76,241,127,63,125,241,127,63,174,241,127,63,222,241,127,63,14,242,127,63,61,242,127,63,108,242,127,63,154,242,127,63,200,242,127,63,245,242,127,63,34,243,127,63,79,243,127,63,123,243,127,63,166,243,127,63,209,243,127,63,252,243,127,63,38,244,127,63,80,244,127,63,121,244,127,63,162,244,127,63,203,244,127,63,243,244,127,63,27,245,127,63,66,245,127,63,105,245,127,63,143,245,127,63,181,245,127,63,219,245,127,63,0,246,127,63,37,246,127,63,73,246,127,63,109,246,127,63,145,246,127,63,180,246,127,63,215,246,127,63,250,246,127,63,28,247,127,63,62,247,127,63,95,247,127,63,128,247,127,63,160,247,127,63,193,247,127,63,225,247,127,63,0,248,127,63,31,248,127,63,62,248,127,63,93,248,127,63,123,248,127,63,152,248,127,63,182,248,127,63,211,248,127,63,240,248,127,63,12,249,127,63,40,249,127,63,68,249,127,63,95,249,127,63,122,249,127,63,149,249,127,63,175,249,127,63,202,249,127,63,227,249,127,63,253,249,127,63,22,250,127,63,47,250,127,63,71,250,127,63,96,250,127,63,120,250,127,63,143,250,127,63,166,250,127,63,190,250,127,63,212,250,127,63,235,250,127,63,1,251,127,63,23,251,127,63,44,251,127,63,66,251,127,63,87,251,127,63,108,251,127,63,128,251,127,63,148,251,127,63,168,251,127,63,188,251,127,63,208,251,127,63,227,251,127,63,246,251,127,63,8,252,127,63,27,252,127,63,45,252,127,63,63,252,127,63,81,252,127,63,98,252,127,63,115,252,127,63,132,252,127,63,149,252,127,63,165,252,127,63,182,252,127,63,198,252,127,63,213,252,127,63,229,252,127,63,244,252,127,63,3,253,127,63,18,253,127,63,33,253,127,63,47,253,127,63,62,253,127,63,76,253,127,63,89,253,127,63,103,253,127,63,116,253,127,63,130,253,127,63,143,253,127,63,155,253,127,63,168,253,127,63,181,253,127,63,193,253,127,63,205,253,127,63,217,253,127,63,228,253,127,63,240,253,127,63,251,253,127,63,6,254,127,63,17,254,127,63,28,254,127,63,38,254,127,63,49,254,127,63,59,254,127,63,69,254,127,63,79,254,127,63,89,254,127,63,98,254,127,63,108,254,127,63,117,254,127,63,126,254,127,63,135,254,127,63,144,254,127,63,152,254,127,63,161,254,127,63,169,254,127,63,177,254,127,63,185,254,127,63,193,254,127,63,201,254,127,63,208,254,127,63,216,254,127,63,223,254,127,63,230,254,127,63,237,254,127,63,244,254,127,63,251,254,127,63,2,255,127,63,8,255,127,63,14,255,127,63,21,255,127,63,27,255,127,63,33,255,127,63,39,255,127,63,45,255,127,63,50,255,127,63,56,255,127,63,61,255,127,63,67,255,127,63,72,255,127,63,77,255,127,63,82,255,127,63,87,255,127,63,92,255,127,63,96,255,127,63,101,255,127,63,105,255,127,63,110,255,127,63,114,255,127,63,118,255,127,63,122,255,127,63,126,255,127,63,130,255,127,63,134,255,127,63,138,255,127,63,142,255,127,63,145,255,127,63,149,255,127,63,152,255,127,63,155,255,127,63,159,255,127,63,162,255,127,63,165,255,127,63,168,255,127,63,171,255,127,63,174,255,127,63,176,255,127,63,179,255,127,63,182,255,127,63,184,255,127,63,187,255,127,63,189,255,127,63,192,255,127,63,194,255,127,63,196,255,127,63,198,255,127,63,201,255,127,63,203,255,127,63,205,255,127,63,207,255,127,63,209,255,127,63,210,255,127,63,212,255,127,63,214,255,127,63,216,255,127,63,217,255,127,63,219,255,127,63,220,255,127,63,222,255,127,63,223,255,127,63,225,255,127,63,226,255,127,63,227,255,127,63,229,255,127,63,230,255,127,63,231,255,127,63,232,255,127,63,233,255,127,63,234,255,127,63,235,255,127,63,236,255,127,63,237,255,127,63,238,255,127,63,239,255,127,63,240,255,127,63,241,255,127,63,241,255,127,63,242,255,127,63,243,255,127,63,244,255,127,63,244,255,127,63,245,255,127,63,246,255,127,63,246,255,127,63,247,255,127,63,247,255,127,63,248,255,127,63,248,255,127,63,249,255,127,63,249,255,127,63,250,255,127,63,250,255,127,63,250,255,127,63,251,255,127,63,251,255,127,63,251,255,127,63,252,255,127,63,252,255,127,63,252,255,127,63,253,255,127,63,253,255,127,63,253,255,127,63,253,255,127,63,254,255,127,63,254,255,127,63,254,255,127,63,254,255,127,63,254,255,127,63,254,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,24,0,120,58,76,70,11,60,242,204,192,60,116,252,59,61,86,73,154,61,241,93,228,61,248,163,29,62,180,231,78,62,54,157,130,62,78,220,159,62,193,174,190,62,65,132,222,62,173,194,254,62,186,101,15,63,248,0,31,63,29,233,45,63,249,219,59,63,45,162,72,63,160,17,84,63,38,15,94,63,46,143,102,63,112,149,109,63,174,51,115,63,159,135,119,63,66,184,122,63,196,242,124,63,75,103,126,63,196,69,127,63,241,186,127,63,217,237,127,63,162,253,127,63,248,255,127,63,169,12,120,55,54,134,11,57,38,198,193,57,94,226,61,58,234,237,156,58,85,101,234,58,56,170,35,59,207,219,89,59,169,226,139,59,42,178,174,59,13,91,213,59,204,219,255,59,91,25,23,60,250,46,48,60,194,45,75,60,156,20,104,60,46,113,131,60,225,202,147,60,185,22,165,60,1,84,183,60,245,129,202,60,198,159,222,60,155,172,243,60,199,211,4,61,213,71,16,61,250,49,28,61,174,145,40,61,101,102,53,61,141,175,66,61,140,108,80,61,193,156,94,61,133,63,109,61,41,84,124,61,252,236,133,61,26,232,141,61,13,27,150,61,110,133,158,61,212,38,167,61,210,254,175,61,245,12,185,61,200,80,194,61,209,201,203,61,146,119,213,61,139,89,223,61,51,111,233,61,2,184,243,61,105,51,254,61,106,112,4,62,214,223,9,62,171,103,15,62,153,7,21,62,77,191,26,62,116,142,32,62,181,116,38,62,184,113,44,62,34,133,50,62,149,174,56,62,178,237,62,62,21,66,69,62,92,171,75,62,30,41,82,62,243,186,88,62,112,96,95,62,40,25,102,62,170,228,108,62,132,194,115,62,68,178,122,62,185,217,128,62,203,98,132,62,26,244,135,62,105,141,139,62,120,46,143,62,6,215,146,62,211,134,150,62,156,61,154,62,29,251,157,62,19,191,161,62,57,137,165,62,71,89,169,62,249,46,173,62,5,10,177,62,36,234,180,62,13,207,184,62,117,184,188,62,18,166,192,62,153,151,196,62,190,140,200,62,52,133,204,62,175,128,208,62,225,126,212,62,125,127,216,62,52,130,220,62,184,134,224,62,185,140,228,62,233,147,232,62,248,155,236,62,150,164,240,62,117,173,244,62,67,182,248,62,178,190,252,62,57,99,0,63,153,102,2,63,82,105,4,63,60,107,6,63,48,108,8,63,6,108,10,63,151,106,12,63,188,103,14,63,78,99,16,63,39,93,18,63,33,85,20,63,21,75,22,63,222,62,24,63,87,48,26,63,92,31,28,63,199,11,30,63,117,245,31,63,66,220,33,63,12,192,35,63,176,160,37,63,12,126,39,63,254,87,41,63,104,46,43,63,39,1,45,63,29,208,46,63,43,155,48,63,51,98,50,63,23,37,52,63,188,227,53,63,4,158,55,63,214,83,57,63,23,5,59,63,173,177,60,63,128,89,62,63,120,252,63,63,126,154,65,63,124,51,67,63,93,199,68,63,12,86,70,63,119,223,71,63,138,99,73,63,54,226,74,63,104,91,76,63,17,207,77,63,35,61,79,63,145,165,80,63,76,8,82,63,75,101,83,63,130,188,84,63,231,13,86,63,114,89,87,63,26,159,88,63,218,222,89,63,172,24,91,63,138,76,92,63,113,122,93,63,93,162,94,63,78,196,95,63,67,224,96,63,58,246,97,63,54,6,99,63,56,16,100,63,67,20,101,63,92,18,102,63,133,10,103,63,198,252,103,63,37,233,104,63,168,207,105,63,89,176,106,63,64,139,107,63,102,96,108,63,216,47,109,63,159,249,109,63,201,189,110,63,97,124,111,63,118,53,112,63,23,233,112,63,81,151,113,63,53,64,114,63,212,227,114,63,61,130,115,63,131,27,116,63,184,175,116,63,238,62,117,63,56,201,117,63,171,78,118,63,90,207,118,63,90,75,119,63,192,194,119,63,162,53,120,63,21,164,120,63,48,14,121,63,8,116,121,63,182,213,121,63,79,51,122,63,235,140,122,63,162,226,122,63,139,52,123,63,191,130,123,63,85,205,123,63,102,20,124,63,9,88,124,63,88,152,124,63,106,213,124,63,88,15,125,63,58,70,125,63,41,122,125,63,62,171,125,63,143,217,125,63,54,5,126,63,75,46,126,63,228,84,126,63,27,121,126,63,7,155,126,63,190,186,126,63,88,216,126,63,236,243,126,63,144,13,127,63,91,37,127,63,99,59,127,63,188,79,127,63,125,98,127,63,185,115,127,63,135,131,127,63,249,145,127,63,36,159,127,63,26,171,127,63,238,181,127,63,179,191,127,63,122,200,127,63,85,208,127,63,84,215,127,63,136,221,127,63,0,227,127,63,204,231,127,63,249,235,127,63,150,239,127,63,177,242,127,63,85,245,127,63,144,247,127,63,109,249,127,63,246,250,127,63,54,252,127,63,55,253,127,63,1,254,127,63,156,254,127,63,18,255,127,63,103,255,127,63,163,255,127,63,204,255,127,63,229,255,127,63,244,255,127,63,252,255,127,63,255,255,127,63,0,0,128,63,0,0,128,63,204,8,120,52,171,134,11,54,79,202,193,54,190,233,61,55,238,247,156,55,192,123,234,55,43,192,35,56,161,2,90,56,189,2,140,56,76,228,174,56,227,165,213,56,199,35,0,57,168,100,23,57,134,149,48,57,104,182,75,57,64,199,104,57,7,228,131,57,105,92,148,57,191,204,165,57,6,53,184,57,65,149,203,57,105,237,223,57,120,61,245,57,184,194,5,58,166,98,17,58,134,126,29,58,81,22,42,58,9,42,55,58,172,185,68,58,54,197,82,58,165,76,97,58,250,79,112,58,47,207,127,58,34,229,135,58,154,32,144,58,255,153,152,58,80,81,161,58,139,70,170,58,174,121,179,58,186,234,188,58,171,153,198,58,129,134,208,58,58,177,218,58,212,25,229,58,79,192,239,58,167,164,250,58,109,227,2,59,117,147,8,59,105,98,14,59,73,80,20,59,19,93,26,59,199,136,32,59,100,211,38,59,232,60,45,59,83,197,51,59,164,108,58,59,218,50,65,59,243,23,72,59,239,27,79,59,204,62,86,59,138,128,93,59,38,225,100,59,161,96,108,59,249,254,115,59,45,188,123,59,29,204,129,59,145,201,133,59,113,214,137,59,188,242,141,59,113,30,146,59,145,89,150,59,26,164,154,59,12,254,158,59,102,103,163,59,40,224,167,59,80,104,172,59,222,255,176,59,209,166,181,59,40,93,186,59,228,34,191,59,2,248,195,59,131,220,200,59,101,208,205,59,168,211,210,59,74,230,215,59,76,8,221,59,172,57,226,59,105,122,231,59,131,202,236,59,249,41,242,59,202,152,247,59,245,22,253,59,60,82,1,60,170,32,4,60,196,246,6,60,137,212,9,60,249,185,12,60,19,167,15,60,216,155,18,60,69,152,21,60,92,156,24,60,26,168,27,60,129,187,30,60,143,214,33,60,69,249,36,60,160,35,40,60,162,85,43,60,73,143,46,60,149,208,49,60,133,25,53,60,26,106,56,60,81,194,59,60,44,34,63,60,168,137,66,60,199,248,69,60,134,111,73,60,230,237,76,60,231,115,80,60,134,1,84,60,197,150,87,60,162,51,91,60,28,216,94,60,52,132,98,60,232,55,102,60,56,243,105,60,35,182,109,60,170,128,113,60,202,82,117,60,131,44,121,60,214,13,125,60,96,123,128,60,161,115,130,60,174,111,132,60,134,111,134,60,40,115,136,60,149,122,138,60,205,133,140,60,206,148,142,60,152,167,144,60,44,190,146,60,136,216,148,60,173,246,150,60,154,24,153,60,78,62,155,60,202,103,157,60,13,149,159,60,23,198,161,60,231,250,163,60,125,51,166,60,217,111,168,60,249,175,170,60,223,243,172,60,137,59,175,60,247,134,177,60,40,214,179,60,29,41,182,60,213,127,184,60,80,218,186,60,140,56,189,60,138,154,191,60,74,0,194,60,202,105,196,60,11,215,198,60,12,72,201,60,205,188,203,60,77,53,206,60,140,177,208,60,137,49,211,60,69,181,213,60,189,60,216,60,243,199,218,60,230,86,221,60,149,233,223,60,0,128,226,60,39,26,229,60,8,184,231,60,164,89,234,60,250,254,236,60,9,168,239,60,210,84,242,60,83,5,245,60,141,185,247,60,126,113,250,60,39,45,253,60,134,236,255,60,206,87,1,61,52,187,2,61,117,32,4,61,144,135,5,61,133,240,6,61,84,91,8,61,253,199,9,61,128,54,11,61,219,166,12,61,16,25,14,61,29,141,15,61,3,3,17,61,193,122,18,61,87,244,19,61,197,111,21,61,10,237,22,61,39,108,24,61,26,237,25,61,228,111,27,61,132,244,28,61,251,122,30,61,71,3,32,61,105,141,33,61,96,25,35,61,45,167,36,61,206,54,38,61,67,200,39,61,141,91,41,61,171,240,42,61,156,135,44,61,96,32,46,61,248,186,47,61,99,87,49,61,160,245,50,61,175,149,52,61,144,55,54,61,67,219,55,61,199,128,57,61,28,40,59,61,65,209,60,61,56,124,62,61,254,40,64,61,148,215,65,61,250,135,67,61,47,58,69,61,51,238,70,61,5,164,72,61,166,91,74,61,20,21,76,61,80,208,77,61,90,141,79,61,49,76,81,61,212,12,83,61,68,207,84,61,128,147,86,61,135,89,88,61,90,33,90,61,248,234,91,61,97,182,93,61,148,131,95,61,145,82,97,61,88,35,99,61,232,245,100,61,65,202,102,61,100,160,104,61,78,120,106,61,1,82,108,61,123,45,110,61,188,10,112,61,197,233,113,61,148,202,115,61,41,173,117,61,133,145,119,61,166,119,121,61,140,95,123,61,55,73,125,61,166,52,127,61,237,144,128,61,105,136,129,61,198,128,130,61,5,122,131,61,37,116,132,61,39,111,133,61,9,107,134,61,204,103,135,61,112,101,136,61,244,99,137,61,88,99,138,61,157,99,139,61,193,100,140,61,196,102,141,61,167,105,142,61,106,109,143,61,11,114,144,61,139,119,145,61,234,125,146,61,40,133,147,61,67,141,148,61,61,150,149,61,20,160,150,61,201,170,151,61,92,182,152,61,203,194,153,61,24,208,154,61,66,222,155,61,72,237,156,61,42,253,157,61,233,13,159,61,132,31,160,61,250,49,161,61,76,69,162,61,122,89,163,61,130,110,164,61,101,132,165,61,35,155,166,61,188,178,167,61,47,203,168,61,124,228,169,61,162,254,170,61,163,25,172,61,124,53,173,61,47,82,174,61,187,111,175,61,31,142,176,61,92,173,177,61,113,205,178,61,94,238,179,61,35,16,181,61,192,50,182,61,52,86,183,61,127,122,184,61,160,159,185,61,153,197,186,61,104,236,187,61,13,20,189,61,136,60,190,61,217,101,191,61,255,143,192,61,250,186,193,61,202,230,194,61,111,19,196,61,233,64,197,61,55,111,198,61,89,158,199,61,78,206,200,61,23,255,201,61,179,48,203,61,35,99,204,61,101,150,205,61,121,202,206,61,96,255,207,61,25,53,209,61,164,107,210,61,0,163,211,61,45,219,212,61,44,20,214,61,251,77,215,61,154,136,216,61,10,196,217,61,74,0,219,61,89,61,220,61,56,123,221,61,230,185,222,61,99,249,223,61,174,57,225,61,200,122,226,61,176,188,227,61,102,255,228,61,233,66,230,61,58,135,231,61,88,204,232,61,66,18,234,61,249,88,235,61,124,160,236,61,203,232,237,61,230,49,239,61,204,123,240,61,125,198,241,61,249,17,243,61,63,94,244,61,79,171,245,61,42,249,246,61,206,71,248,61,60,151,249,61,114,231,250,61,114,56,252,61,58,138,253,61,202,220,254,61,17,24,0,62,33,194,0,62,149,108,1,62,108,23,2,62,166,194,2,62,68,110,3,62,69,26,4,62,168,198,4,62,111,115,5,62,152,32,6,62,35,206,6,62,17,124,7,62,98,42,8,62,20,217,8,62,40,136,9,62,157,55,10,62,117,231,10,62,173,151,11,62,71,72,12,62,66,249,12,62,158,170,13,62,91,92,14,62,120,14,15,62,246,192,15,62,213,115,16,62,19,39,17,62,177,218,17,62,175,142,18,62,13,67,19,62,202,247,19,62,231,172,20,62,99,98,21,62,62,24,22,62,120,206,22,62,16,133,23,62,7,60,24,62,92,243,24,62,16,171,25,62,33,99,26,62,145,27,27,62,94,212,27,62,137,141,28,62,17,71,29,62,246,0,30,62,56,187,30,62,215,117,31,62,211,48,32,62,43,236,32,62,224,167,33,62,241,99,34,62,93,32,35,62,38,221,35,62,74,154,36,62,202,87,37,62,165,21,38,62,219,211,38,62,108,146,39,62,88,81,40,62,159,16,41,62,64,208,41,62,59,144,42,62,144,80,43,62,63,17,44,62,72,210,44,62,170,147,45,62,102,85,46,62,122,23,47,62,232,217,47,62,175,156,48,62,206,95,49,62,69,35,50,62,21,231,50,62,61,171,51,62,189,111,52,62,148,52,53,62,195,249,53,62,73,191,54,62,38,133,55,62,91,75,56,62,230,17,57,62,199,216,57,62,255,159,58,62,141,103,59,62,113,47,60,62,171,247,60,62,59,192,61,62,31,137,62,62,89,82,63,62,232,27,64,62,204,229,64,62,5,176,65,62,146,122,66,62,115,69,67,62,168,16,68,62,49,220,68,62,14,168,69,62,62,116,70,62,194,64,71,62,152,13,72,62,193,218,72,62,61,168,73,62,12,118,74,62,44,68,75,62,159,18,76,62,100,225,76,62,122,176,77,62,225,127,78,62,154,79,79,62,164,31,80,62,255,239,80,62,170,192,81,62,166,145,82,62,242,98,83,62,141,52,84,62,121,6,85,62,180,216,85,62,63,171,86,62,25,126,87,62,65,81,88,62,185,36,89,62,126,248,89,62,147,204,90,62,245,160,91,62,165,117,92,62,163,74,93,62,238,31,94,62,135,245,94,62,109,203,95,62,159,161,96,62,30,120,97,62,233,78,98,62,1,38,99,62,100,253,99,62,19,213,100,62,14,173,101,62,84,133,102,62,229,93,103,62])
.concat([193,54,104,62,231,15,105,62,88,233,105,62,19,195,106,62,24,157,107,62,103,119,108,62,255,81,109,62,224,44,110,62,11,8,111,62,126,227,111,62,58,191,112,62,62,155,113,62,139,119,114,62,31,84,115,62,251,48,116,62,31,14,117,62,138,235,117,62,59,201,118,62,52,167,119,62,115,133,120,62,248,99,121,62,196,66,122,62,213,33,123,62,44,1,124,62,200,224,124,62,170,192,125,62,208,160,126,62,59,129,127,62,245,48,128,62,111,161,128,62,11,18,129,62,201,130,129,62,168,243,129,62,169,100,130,62,204,213,130,62,15,71,131,62,117,184,131,62,251,41,132,62,162,155,132,62,107,13,133,62,84,127,133,62,93,241,133,62,136,99,134,62,210,213,134,62,61,72,135,62,200,186,135,62,116,45,136,62,63,160,136,62,42,19,137,62,52,134,137,62,94,249,137,62,168,108,138,62,17,224,138,62,153,83,139,62,64,199,139,62,6,59,140,62,235,174,140,62,239,34,141,62,17,151,141,62,82,11,142,62,177,127,142,62,46,244,142,62,201,104,143,62,130,221,143,62,89,82,144,62,78,199,144,62,96,60,145,62,143,177,145,62,220,38,146,62,70,156,146,62,205,17,147,62,113,135,147,62,50,253,147,62,16,115,148,62,9,233,148,62,32,95,149,62,82,213,149,62,161,75,150,62,12,194,150,62,146,56,151,62,53,175,151,62,243,37,152,62,204,156,152,62,193,19,153,62,209,138,153,62,252,1,154,62,66,121,154,62,163,240,154,62,31,104,155,62,181,223,155,62,101,87,156,62,48,207,156,62,21,71,157,62,20,191,157,62,45,55,158,62,96,175,158,62,172,39,159,62,18,160,159,62,145,24,160,62,41,145,160,62,218,9,161,62,165,130,161,62,136,251,161,62,132,116,162,62,152,237,162,62,197,102,163,62,10,224,163,62,103,89,164,62,220,210,164,62,105,76,165,62,14,198,165,62,202,63,166,62,158,185,166,62,137,51,167,62,139,173,167,62,164,39,168,62,213,161,168,62,27,28,169,62,121,150,169,62,237,16,170,62,119,139,170,62,24,6,171,62,206,128,171,62,155,251,171,62,125,118,172,62,117,241,172,62,130,108,173,62,165,231,173,62,221,98,174,62,42,222,174,62,140,89,175,62,2,213,175,62,142,80,176,62,46,204,176,62,226,71,177,62,170,195,177,62,135,63,178,62,119,187,178,62,124,55,179,62,148,179,179,62,191,47,180,62,254,171,180,62,80,40,181,62,181,164,181,62,45,33,182,62,184,157,182,62,85,26,183,62,5,151,183,62,199,19,184,62,156,144,184,62,130,13,185,62,123,138,185,62,133,7,186,62,161,132,186,62,206,1,187,62,13,127,187,62,93,252,187,62,190,121,188,62,48,247,188,62,178,116,189,62,70,242,189,62,233,111,190,62,157,237,190,62,98,107,191,62,54,233,191,62,26,103,192,62,14,229,192,62,17,99,193,62,36,225,193,62,70,95,194,62,119,221,194,62,184,91,195,62,7,218,195,62,100,88,196,62,209,214,196,62,75,85,197,62,212,211,197,62,107,82,198,62,16,209,198,62,195,79,199,62,132,206,199,62,82,77,200,62,45,204,200,62,21,75,201,62,11,202,201,62,13,73,202,62,29,200,202,62,56,71,203,62,97,198,203,62,149,69,204,62,214,196,204,62,34,68,205,62,123,195,205,62,223,66,206,62,79,194,206,62,202,65,207,62,81,193,207,62,226,64,208,62,127,192,208,62,38,64,209,62,216,191,209,62,148,63,210,62,91,191,210,62,44,63,211,62,7,191,211,62,235,62,212,62,218,190,212,62,210,62,213,62,211,190,213,62,222,62,214,62,242,190,214,62,15,63,215,62,53,191,215,62,99,63,216,62,154,191,216,62,217,63,217,62,32,192,217,62,112,64,218,62,199,192,218,62,38,65,219,62,140,193,219,62,250,65,220,62,112,194,220,62,236,66,221,62,112,195,221,62,250,67,222,62,139,196,222,62,34,69,223,62,192,197,223,62,100,70,224,62,14,199,224,62,189,71,225,62,115,200,225,62,46,73,226,62,239,201,226,62,181,74,227,62,127,203,227,62,79,76,228,62,36,205,228,62,253,77,229,62,219,206,229,62,190,79,230,62,164,208,230,62,142,81,231,62,125,210,231,62,111,83,232,62,100,212,232,62,93,85,233,62,89,214,233,62,89,87,234,62,91,216,234,62,96,89,235,62,104,218,235,62,114,91,236,62,126,220,236,62,141,93,237,62,158,222,237,62,176,95,238,62,196,224,238,62,218,97,239,62,241,226,239,62,10,100,240,62,35,229,240,62,62,102,241,62,89,231,241,62,116,104,242,62,145,233,242,62,173,106,243,62,202,235,243,62,230,108,244,62,3,238,244,62,31,111,245,62,59,240,245,62,86,113,246,62,112,242,246,62,137,115,247,62,161,244,247,62,184,117,248,62,206,246,248,62,226,119,249,62,244,248,249,62,4,122,250,62,18,251,250,62,30,124,251,62,40,253,251,62,47,126,252,62,52,255,252,62,54,128,253,62,52,1,254,62,48,130,254,62,40,3,255,62,29,132,255,62,135,2,0,63,254,66,0,63,115,131,0,63,230,195,0,63,86,4,1,63,197,68,1,63,49,133,1,63,155,197,1,63,3,6,2,63,103,70,2,63,202,134,2,63,42,199,2,63,135,7,3,63,225,71,3,63,56,136,3,63,141,200,3,63,222,8,4,63,44,73,4,63,119,137,4,63,191,201,4,63,3,10,5,63,68,74,5,63,130,138,5,63,188,202,5,63,242,10,6,63,36,75,6,63,83,139,6,63,126,203,6,63,165,11,7,63,199,75,7,63,230,139,7,63,1,204,7,63,23,12,8,63,41,76,8,63,54,140,8,63,63,204,8,63,67,12,9,63,67,76,9,63,62,140,9,63,52,204,9,63,37,12,10,63,18,76,10,63,249,139,10,63,219,203,10,63,184,11,11,63,144,75,11,63,98,139,11,63,47,203,11,63,246,10,12,63,184,74,12,63,116,138,12,63,43,202,12,63,219,9,13,63,134,73,13,63,43,137,13,63,202,200,13,63,98,8,14,63,245,71,14,63,129,135,14,63,7,199,14,63,135,6,15,63,0,70,15,63,114,133,15,63,222,196,15,63,67,4,16,63,161,67,16,63,249,130,16,63,73,194,16,63,147,1,17,63,213,64,17,63,17,128,17,63,69,191,17,63,114,254,17,63,151,61,18,63,181,124,18,63,203,187,18,63,218,250,18,63,225,57,19,63,225,120,19,63,216,183,19,63,200,246,19,63,176,53,20,63,143,116,20,63,103,179,20,63,54,242,20,63,253,48,21,63,188,111,21,63,114,174,21,63,32,237,21,63,197,43,22,63,98,106,22,63,246,168,22,63,129,231,22,63,3,38,23,63,125,100,23,63,237,162,23,63,84,225,23,63,178,31,24,63,7,94,24,63,83,156,24,63,149,218,24,63,206,24,25,63,253,86,25,63,35,149,25,63,63,211,25,63,82,17,26,63,90,79,26,63,89,141,26,63,78,203,26,63,57,9,27,63,25,71,27,63,240,132,27,63,188,194,27,63,126,0,28,63,54,62,28,63,227,123,28,63,134,185,28,63,30,247,28,63,172,52,29,63,47,114,29,63,167,175,29,63,20,237,29,63,118,42,30,63,206,103,30,63,26,165,30,63,91,226,30,63,145,31,31,63,188,92,31,63,219,153,31,63,239,214,31,63,247,19,32,63,244,80,32,63,230,141,32,63,203,202,32,63,165,7,33,63,115,68,33,63,53,129,33,63,235,189,33,63,150,250,33,63,52,55,34,63,198,115,34,63,75,176,34,63,197,236,34,63,50,41,35,63,146,101,35,63,230,161,35,63,46,222,35,63,105,26,36,63,151,86,36,63,185,146,36,63,205,206,36,63,213,10,37,63,208,70,37,63,190,130,37,63,158,190,37,63,114,250,37,63,56,54,38,63,241,113,38,63,157,173,38,63,59,233,38,63,204,36,39,63,79,96,39,63,197,155,39,63,45,215,39,63,135,18,40,63,211,77,40,63,18,137,40,63,66,196,40,63,101,255,40,63,121,58,41,63,128,117,41,63,120,176,41,63,98,235,41,63,62,38,42,63,11,97,42,63,202,155,42,63,122,214,42,63,28,17,43,63,175,75,43,63,52,134,43,63,170,192,43,63,16,251,43,63,105,53,44,63,178,111,44,63,236,169,44,63,23,228,44,63,51,30,45,63,64,88,45,63,61,146,45,63,43,204,45,63,10,6,46,63,218,63,46,63,154,121,46,63,74,179,46,63,235,236,46,63,124,38,47,63,254,95,47,63,112,153,47,63,210,210,47,63,36,12,48,63,102,69,48,63,152,126,48,63,186,183,48,63,204,240,48,63,205,41,49,63,191,98,49,63,160,155,49,63,113,212,49,63,49,13,50,63,225,69,50,63,128,126,50,63,15,183,50,63,141,239,50,63,251,39,51,63,87,96,51,63,163,152,51,63,222,208,51,63,8,9,52,63,34,65,52,63,42,121,52,63,33,177,52,63,7,233,52,63,219,32,53,63,159,88,53,63,81,144,53,63,242,199,53,63,129,255,53,63,255,54,54,63,108,110,54,63,198,165,54,63,16,221,54,63,71,20,55,63,109,75,55,63,129,130,55,63,131,185,55,63,116,240,55,63,82,39,56,63,30,94,56,63,217,148,56,63,129,203,56,63,23,2,57,63,155,56,57,63,13,111,57,63,108,165,57,63,185,219,57,63,244,17,58,63,28,72,58,63,50,126,58,63,53,180,58,63,38,234,58,63,4,32,59,63,207,85,59,63,135,139,59,63,45,193,59,63,192,246,59,63,64,44,60,63,173,97,60,63,7,151,60,63,78,204,60,63,130,1,61,63,163,54,61,63,177,107,61,63,171,160,61,63,146,213,61,63,102,10,62,63,39,63,62,63,212,115,62,63,110,168,62,63,244,220,62,63,103,17,63,63,198,69,63,63,17,122,63,63,73,174,63,63,109,226,63,63,126,22,64,63,122,74,64,63,99,126,64,63,56,178,64,63,248,229,64,63,165,25,65,63,62,77,65,63,195,128,65,63,52,180,65,63,144,231,65,63,216,26,66,63,13,78,66,63,44,129,66,63,56,180,66,63,47,231,66,63,18,26,67,63,224,76,67,63,154,127,67,63,64,178,67,63,208,228,67,63,77,23,68,63,180,73,68,63,7,124,68,63,69,174,68,63,111,224,68,63,131,18,69,63,131,68,69,63,110,118,69,63,68,168,69,63,5,218,69,63,177,11,70,63,72,61,70,63,202,110,70,63,55,160,70,63,143,209,70,63,210,2,71,63,255,51,71,63,23,101,71,63,26,150,71,63,8,199,71,63,224,247,71,63,163,40,72,63,81,89,72,63,233,137,72,63,107,186,72,63,216,234,72,63,48,27,73,63,114,75,73,63,158,123,73,63,181,171,73,63,181,219,73,63,161,11,74,63,118,59,74,63,54,107,74,63,224,154,74,63,116,202,74,63,242,249,74,63,90,41,75,63,173,88,75,63,233,135,75,63,15,183,75,63,32,230,75,63,26,21,76,63,254,67,76,63,204,114,76,63,132,161,76,63,38,208,76,63,177,254,76,63,38,45,77,63,133,91,77,63,206,137,77,63,0,184,77,63,28,230,77,63,34,20,78,63,17,66,78,63,234,111,78,63,172,157,78,63,88,203,78,63,238,248,78,63,108,38,79,63,213,83,79,63,38,129,79,63,97,174,79,63,134,219,79,63,147,8,80,63,138,53,80,63,107,98,80,63,52,143,80,63,231,187,80,63,131,232,80,63,8,21,81,63,119,65,81,63,206,109,81,63,15,154,81,63,57,198,81,63,76,242,81,63,71,30,82,63,44,74,82,63,250,117,82,63,177,161,82,63,81,205,82,63,218,248,82,63,76,36,83,63,166,79,83,63,234,122,83,63,22,166,83,63,44,209,83,63,42,252,83,63,17,39,84,63,224,81,84,63,153,124,84,63,58,167,84,63,196,209,84,63,54,252,84,63,146,38,85,63,214,80,85,63,2,123,85,63,24,165,85,63,22,207,85,63,252,248,85,63,204,34,86,63,131,76,86,63,36,118,86,63,172,159,86,63,30,201,86,63,120,242,86,63,186,27,87,63,229,68,87,63,248,109,87,63,244,150,87,63,216,191,87,63,165,232,87,63,90,17,88,63,248,57,88,63,126,98,88,63,236,138,88,63,67,179,88,63,130,219,88,63,169,3,89,63,185,43,89,63,177,83,89,63,145,123,89,63,90,163,89,63,11,203,89,63,164,242,89,63,37,26,90,63,143,65,90,63,225,104,90,63,27,144,90,63,62,183,90,63,72,222,90,63,59,5,91,63,22,44,91,63,217,82,91,63,133,121,91,63,24,160,91,63,148,198,91,63,248,236,91,63,68,19,92,63,120,57,92,63,149,95,92,63,153,133,92,63,134,171,92,63,91,209,92,63,24,247,92,63,189,28,93,63,74,66,93,63,191,103,93,63,28,141,93,63,98,178,93,63,143,215,93,63,165,252,93,63,162,33,94,63,136,70,94,63,86,107,94,63,11,144,94,63,169,180,94,63,47,217,94,63,157,253,94,63,243,33,95,63,49,70,95,63,88,106,95,63,102,142,95,63,92,178,95,63,59,214,95,63,1,250,95,63,175,29,96,63,70,65,96,63,196,100,96,63,43,136,96,63,122,171,96,63,176,206,96,63,207,241,96,63,214,20,97,63,197,55,97,63,155,90,97,63,90,125,97,63,1,160,97,63,144,194,97,63,8,229,97,63,103,7,98,63,174,41,98,63,221,75,98,63,245,109,98,63,244,143,98,63,220,177,98,63,171,211,98,63,99,245,98,63,3,23,99,63,139,56,99,63,251,89,99,63,83,123,99,63,147,156,99,63,188,189,99,63,204,222,99,63,197,255,99,63,166,32,100,63,110,65,100,63,32,98,100,63,185,130,100,63,58,163,100,63,164,195,100,63,245,227,100,63,47,4,101,63,82,36,101,63,92,68,101,63,78,100,101,63,41,132,101,63,236,163,101,63,151,195,101,63,43,227,101,63,167,2,102,63,11,34,102,63,87,65,102,63,139,96,102,63,168,127,102,63,174,158,102,63,155,189,102,63,113,220,102,63,47,251,102,63,214,25,103,63,101,56,103,63,220,86,103,63,59,117,103,63,132,147,103,63,180,177,103,63,205,207,103,63,206,237,103,63,184,11,104,63,138,41,104,63,69,71,104,63,233,100,104,63,116,130,104,63,233,159,104,63,69,189,104,63,139,218,104,63,185,247,104,63,207,20,105,63,207,49,105,63,182,78,105,63,135,107,105,63,64,136,105,63,225,164,105,63,108,193,105,63,223,221,105,63,59,250,105,63,127,22,106,63,172,50,106,63,195,78,106,63,193,106,106,63,169,134,106,63,121,162,106,63,51,190,106,63,213,217,106,63,96,245,106,63,212,16,107,63,48,44,107,63,118,71,107,63,165,98,107,63,188,125,107,63,189,152,107,63,167,179,107,63,121,206,107,63,53,233,107,63,218,3,108,63,104,30,108,63,223,56,108,63,63,83,108,63,136,109,108,63,187,135,108,63,214,161,108,63,219,187,108,63,201,213,108,63,161,239,108,63,97,9,109,63,11,35,109,63,159,60,109,63,27,86,109,63,129,111,109,63,209,136,109,63,9,162,109,63,44,187,109,63,56,212,109,63,45,237,109,63,12,6,110,63,212,30,110,63,134,55,110,63,33,80,110,63,166,104,110,63,21,129,110,63,110,153,110,63,176,177,110,63,220,201,110,63,241,225,110,63,241,249,110,63,218,17,111,63,173,41,111,63,106,65,111,63,16,89,111,63,161,112,111,63,28,136,111,63,128,159,111,63,207,182,111,63,7,206,111,63,42,229,111,63,54,252,111,63,45,19,112,63,14,42,112,63,217,64,112,63,142,87,112,63,46,110,112,63,184,132,112,63,43,155,112,63,138,177,112,63,210,199,112,63,5,222,112,63,35,244,112,63,42,10,113,63,29,32,113,63,249,53,113,63,193,75,113,63,114,97,113,63,15,119,113,63,150,140,113,63,7,162,113,63,99,183,113,63,170,204,113,63,220,225,113,63,249,246,113,63,0,12,114,63,242,32,114,63,207,53,114,63,151,74,114,63,73,95,114,63,231,115,114,63,112,136,114,63,227,156,114,63,66,177,114,63,140,197,114,63,193,217,114,63,225,237,114,63,236,1,115,63,227,21,115,63,197,41,115,63,146,61,115,63,74,81,115,63,238,100,115,63,125,120,115,63,248,139,115,63,94,159,115,63,175,178,115,63,236,197,115,63,21,217,115,63,41,236,115,63,41,255,115,63,21,18,116,63,236,36,116,63,175,55,116,63,94,74,116,63,248,92,116,63,127,111,116,63,241,129,116,63,80,148,116,63,154,166,116,63,208,184,116,63,242,202,116,63,1,221,116,63,251,238,116,63,226,0,117,63,181,18,117,63,116,36,117,63,31,54,117,63,183,71,117,63,59,89,117,63,171,106,117,63,8,124,117,63,81,141,117,63,135,158,117,63,169,175,117,63,184,192,117,63,179,209,117,63,155,226,117,63,112,243,117,63,50,4,118,63,224,20,118,63,123,37,118,63,3,54,118,63,120,70,118,63,217,86,118,63,40,103,118,63,100,119,118,63,140,135,118,63,162,151,118,63,165,167,118,63,149,183,118,63,114,199,118,63,61,215,118,63,245,230,118,63,154,246,118,63,44,6,119,63,172,21,119,63,26,37,119,63,117,52,119,63,189,67,119,63,243,82,119,63,22,98,119,63,40,113,119,63,39,128,119,63,19,143,119,63,238,157,119,63,182,172,119,63,108,187,119,63,16,202,119,63,162,216,119,63,34,231,119,63,144,245,119,63,236,3,120,63,55,18,120,63,111,32,120,63,150,46,120,63,170,60,120,63,174,74,120,63,159,88,120,63,127,102,120,63,77,116,120,63,10,130,120,63,181,143,120,63,79,157,120,63,215,170,120,63,78,184,120,63,180,197,120,63,8,211,120,63,76,224,120,63,126,237,120,63,158,250,120,63,174,7,121,63,173,20,121,63,155,33,121,63,119,46,121,63,67,59,121,63,254,71,121,63,168,84,121,63,66,97,121,63,202,109,121,63,66,122,121,63,169,134,121,63,0,147,121,63,70,159,121,63,124,171,121,63,161,183,121,63,181,195,121,63,186,207,121,63,173,219,121,63,145,231,121,63,100,243,121,63,40,255,121,63,219,10,122,63,126,22,122,63,16,34,122,63,147,45,122,63,6,57,122,63,105,68,122,63,188,79,122,63,255,90,122,63,51,102,122,63,86,113,122,63,106,124,122,63,111,135,122,63,99,146,122,63,72,157,122,63,30,168,122,63,228,178,122,63,155,189,122,63,66,200,122,63,218,210,122,63,99,221,122,63,221,231,122,63,71,242,122,63,162,252,122,63,238,6,123,63,43,17,123,63,89,27,123,63,120,37,123,63,137,47,123,63,138,57,123,63,124,67,123,63,96,77,123,63,53,87,123,63,252,96,123,63,179,106,123,63,92,116,123,63,247,125,123,63,131,135,123,63,1,145,123,63,112,154,123,63,209,163,123,63,36,173,123,63,104,182,123,63,158,191,123,63,198,200,123,63,224,209,123,63,236,218,123,63,234,227,123,63,218,236,123,63,188,245,123,63,144,254,123,63,86,7,124,63,14,16,124,63,185,24,124,63,86,33,124,63,230,41,124,63,104,50,124,63,220,58,124,63,67,67,124,63,156,75,124,63,232,83,124,63,39,92,124,63,88,100,124,63,124,108,124,63,147,116,124,63,157,124,124,63,153,132,124,63,137,140,124,63,107,148,124,63,65,156,124,63,9,164,124,63,197,171,124,63,116,179,124,63,22,187,124,63,172,194,124,63,52,202,124,63,176,209,124,63,32,217,124,63,131,224,124,63,217,231,124,63,35,239,124,63,97,246,124,63,146,253,124,63,183,4,125,63,208,11,125,63,221,18,125,63,221,25,125,63,209,32,125,63,185,39,125,63,150,46,125,63,102,53,125,63,42,60,125,63,227,66,125,63,143,73,125,63,48,80,125,63,197,86,125,63,78,93,125,63,204,99,125,63,62,106,125,63,165,112,125,63,0,119,125,63,80,125,125,63,148,131,125,63,205,137,125,63,251,143,125,63,29,150,125,63,52,156,125,63,64,162,125,63,65,168,125,63,55,174,125,63,34,180,125,63,2,186,125,63,215,191,125,63,161,197,125,63,96,203,125,63,21,209,125,63,190,214,125,63,93,220,125,63,242,225,125,63,124,231,125,63,251,236,125,63,112,242,125,63,218,247,125,63,58,253,125,63,143,2,126,63,219,7,126,63,28,13,126,63,82,18,126,63,127,23,126,63,161,28,126,63,186,33,126,63,200,38,126,63,204,43,126,63,199,48,126,63,183,53,126,63,158,58,126,63,123,63,126,63,78,68,126,63,23,73,126,63,215,77,126,63,141,82,126,63,58,87,126,63,221,91,126,63,118,96,126,63,6,101,126,63,141,105,126,63,10,110,126,63,126,114,126,63,233,118,126,63,75,123,126,63,164,127,126,63,243,131,126,63,57,136,126,63,119,140,126,63,171,144,126,63,214,148,126,63,249,152,126,63,18,157,126,63,35,161,126,63,44,165,126,63,43,169,126,63,34,173,126,63,16,177,126,63,246,180,126,63,211,184,126,63,167,188,126,63,115,192,126,63,55,196,126,63,243,199,126,63,166,203,126,63,81,207,126,63,243,210,126,63,142,214,126,63,32,218,126,63,171,221,126,63,45,225,126,63,167,228,126,63,26,232,126,63,132,235,126,63,231,238,126,63,66,242,126,63,149,245,126,63,224,248,126,63,36,252,126,63,96,255,126,63,148,2,127,63,193,5,127,63,230,8,127,63,4,12,127,63,27,15,127,63,42,18,127,63,50,21,127,63,50,24,127,63,43,27,127,63,29,30,127,63,8,33,127,63,236,35,127,63,201,38,127,63,158,41,127,63,109,44,127,63,53,47,127,63,246,49,127,63,175,52,127,63,99,55,127,63,15,58,127,63,181,60,127,63,83,63,127,63,236,65,127,63,125,68,127,63,8,71,127,63,141,73,127,63,11,76,127,63,131,78,127,63,244,80,127,63,95,83,127,63,195,85,127,63,33,88,127,63,121,90,127,63,203,92,127,63,23,95,127,63,92,97,127,63,155,99,127,63,213,101,127,63,8,104,127,63,54,106,127,63,93,108,127,63,127,110,127,63,155,112,127,63,177,114,127,63,193,116,127,63,203,118,127,63,208,120,127,63,207,122,127,63,201,124,127,63,189,126,127,63,171,128,127,63,148,130,127,63,120,132,127,63,86,134,127,63,47,136,127,63,2,138,127,63,209,139,127,63,153,141,127,63,93,143,127,63,28,145,127,63,213,146,127,63,137,148,127,63,57,150,127,63,227,151,127,63,136,153,127,63,40,155,127,63,196,156,127,63,90,158,127,63,236,159,127,63,121,161,127,63,1,163,127,63,132,164,127,63,3,166,127,63,125,167,127,63,242,168,127,63,99,170,127,63,207,171,127,63,55,173,127,63,154,174,127,63,249,175,127,63,84,177,127,63,170,178,127,63,251,179,127,63,73,181,127,63,146,182,127,63,215,183,127,63,24,185,127,63,85,186,127,63,141,187,127,63,193,188,127,63,242,189,127,63,30,191,127,63,71,192,127,63,107,193,127,63,140,194,127,63,168,195,127,63,193,196,127,63,214,197,127,63,231,198,127,63,245,199,127,63,255,200,127,63,5,202,127,63,7,203,127,63,6,204,127,63,1,205,127,63,249,205,127,63,237,206,127,63,222,207,127,63,203,208,127,63,181,209,127,63,156,210,127,63,127,211,127,63,95,212,127,63,59,213,127,63,20,214,127,63,234,214,127,63,189,215,127,63,141,216,127,63,90,217,127,63,35,218,127,63,233,218,127,63,173,219,127,63,109,220,127,63,43,221,127,63,229,221,127,63,156,222,127,63,81,223,127,63,3,224,127,63,178,224,127,63,94,225,127,63,7,226,127,63,174,226,127,63,82,227,127,63,243,227,127,63,146,228,127,63,46,229,127,63,199,229,127,63,94,230,127,63,242,230,127,63,132,231,127,63,19,232,127,63,160,232,127,63,42,233,127,63,178,233,127,63,56,234,127,63,187,234,127,63,60,235,127,63,187,235,127,63,55,236,127,63,177,236,127,63,41,237,127,63,159,237,127,63,18,238,127,63,132,238,127,63,243,238,127,63,96,239,127,63,204,239,127,63,53,240,127,63,156,240,127,63,1,241,127,63,101,241,127,63,198,241,127,63,37,242,127,63,131,242,127,63,222,242,127,63,56,243,127,63,144,243,127,63,231,243,127,63,59,244,127,63,142,244,127,63,223,244,127,63,46,245,127,63,124,245,127,63,200,245,127,63,19,246,127,63,91,246,127,63,163,246,127,63,233,246,127,63,45,247,127,63,111,247,127,63,177,247,127,63,240,247,127,63,47,248,127,63,108,248,127,63,167,248,127,63,225,248,127,63,26,249,127,63,82,249,127,63,136,249,127,63,188,249,127,63,240,249,127,63,34,250,127,63,83,250,127,63,131,250,127,63,178,250,127,63,224,250,127,63,12,251,127,63,55,251,127,63,97,251,127,63,138,251,127,63,178,251,127,63,217,251,127,63,255,251,127,63,36,252,127,63,72,252,127,63,107,252,127,63,141,252,127,63,173,252,127,63,205,252,127,63,237,252,127,63,11,253,127,63,40,253,127,63,69,253,127,63,96,253,127,63,123,253,127,63,149,253,127,63,174,253,127,63,199,253,127,63,222,253,127,63,245,253,127,63,12,254,127,63,33,254,127,63,54,254,127,63,74,254,127,63,93,254,127,63,112,254,127,63,130,254,127,63,148,254,127,63,165,254,127,63,181,254,127,63,197,254,127,63,212,254,127,63,227,254,127,63,241,254,127,63,254,254,127,63,11,255,127,63,24,255,127,63,36,255,127,63,47,255,127,63,59,255,127,63,69,255,127,63,79,255,127,63,89,255,127,63,99,255,127,63,108,255,127,63,116,255,127,63,124,255,127,63,132,255,127,63,140,255,127,63,147,255,127,63,154,255,127,63,160,255,127,63,166,255,127,63,172,255,127,63,178,255,127,63,183,255,127,63,188,255,127,63,193,255,127,63,197,255,127,63,202,255,127,63,206,255,127,63,209,255,127,63,213,255,127,63,216,255,127,63,220,255,127,63,223,255,127,63,225,255,127,63,228,255,127,63,230,255,127,63,233,255,127,63,235,255,127,63,237,255,127,63,239,255,127,63,240,255,127,63,242,255,127,63,243,255,127,63,245,255,127,63,246,255,127,63,247,255,127,63,248,255,127,63,249,255,127,63,250,255,127,63,251,255,127,63,251,255,127,63,252,255,127,63,252,255,127,63,253,255,127,63,253,255,127,63,254,255,127,63,254,255,127,63,254,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,5,12,120,56,50,131,11,58,118,186,193,58,226,203,61,59,38,207,156,59,139,32,234,59,245,102,35,60,63,100,89,60,184,127,139,60,59,23,174,60,239,114,212,60,96,140,254,60,45,46,22,61,114,237,46,61,155,127,73,61,220,223,101,61,123,4,130,61,159,250,145,61,71,207,162,61,38,127,180,61,173,6,199,61,16,98,218,61,63,141,238,61,244,193,1,62,185,160,12,62,128,224,23,62,182,126,35,62,166,120,47,62,116,203,59,62,34,116,72,62,141,111,85,62,107,186,98,62,83,81,112,62,180,48,126,62,110,42,134,62,252,92,141,62,9,174,148,62,138,27,156,62,100,163,163,62,112,67,171,62,119,249,178,62,54,195,186,62,93,158,194,62,147,136,202,62,118,127,210,62,154,128,218,62,142,137,226,62,217,151,234,62,2,169,242,62,139,186,250,62,251,100,1,63,99,106,5,63,65,108,9,63,89,105,13,63,116,96,17,63,94,80,21,63,231,55,25,63,231,21,29,63,58,233,32,63,197,176,36,63,116,107,40,63,62,24,44,63,35,182,47,63,43,68,51,63,109,193,54,63,10,45,58,63,48,134,61,63,26,204,64,63,17,254,67,63,107,27,71,63,142,35,74,63,238,21,77,63,15,242,79,63,132,183,82,63,239,101,85,63,3,253,87,63,129,124,90,63,60,228,92,63,21,52,95,63,254,107,97,63,246,139,99,63,14,148,101,63,98,132,103,63,33,93,105,63,133,30,107,63,213,200,108,63,103,92,110,63,155,217,111,63,224,64,113,63,172,146,114,63,131,207,115,63,241,247,116,63,139,12,118,63,239,13,119,63,193,252,119,63,172,217,120,63,99,165,121,63,155,96,122,63,15,12,123,63,124,168,123,63,163,54,124,63,71,183,124,63,41,43,125,63,13,147,125,63,183,239,125,63,229,65,126,63,89,138,126,63,205,201,126,63,251,0,127,63,150,48,127,63,78,89,127,63,205,123,127,63,182,152,127,63,167,176,127,63,53,196,127,63,239,211,127,63,91,224,127,63,245,233,127,63,51,241,127,63,127,246,127,63,59,250,127,63,190,252,127,63,84,254,127,63,64,255,127,63,186,255,127,63,238,255,127,63,254,255,127,63,0,0,128,63,171,15,120,53,24,135,11,55,225,201,193,55,107,233,61,56,128,247,156,56,187,122,234,56,24,191,35,57,213,0,90,57,56,1,140,57,229,225,174,57,88,162,213,57,60,33,0,58,24,97,23,58,175,144,48,58,243,175,75,58,212,190,104,58,159,222,131,58,143,85,148,58,48,196,165,58,119,42,184,58,90,136,203,58,204,221,223,58,191,42,245,58,148,183,5,59,124,85,17,59,16,111,29,59,73,4,42,59,31,21,55,59,138,161,68,59,129,169,82,59,252,44,97,59,241,43,112,59,88,166,127,59,19,206,135,59,169,6,144,59,233,124,152,59,204,48,161,59,79,34,170,59,106,81,179,59,26,190,188,59,86,104,198,59,26,80,208,59,95,117,218,59,31,216,228,59,83,120,239,59,244,85,250,59,126,184,2,60,177,100,8,60,145,47,14,60,25,25,20,60,70,33,26,60,19,72,32,60,126,141,38,60,129,241,44,60,25,116,51,60,65,21,58,60,246,212,64,60,50,179,71,60,243,175,78,60,50,203,85,60,235,4,93,60,26,93,100,60,186,211,107,60,198,104,115,60,58,28,123,60,7,119,129,60,33,111,133,60,102,118,137,60,212,140,141,60,105,178,145,60,33,231,149,60,251,42,154,60,243,125,158,60,6,224,162,60,50,81,167,60,115,209,171,60,199,96,176,60,43,255,180,60,154,172,185,60,19,105,190,60,146,52,195,60,20,15,200,60,149,248,204,60,19,241,209,60,137,248,214,60,245,14,220,60,83,52,225,60,160,104,230,60,215,171,235,60,246,253,240,60,249,94,246,60,220,206,251,60,205,166,0,61,153,109,3,61,207,59,6,61,109,17,9,61,114,238,11,61,220,210,14,61,167,190,17,61,211,177,20,61,94,172,23,61,68,174,26,61,133,183,29,61,30,200,32,61,12,224,35,61,78,255,38,61,225,37,42,61,196,83,45,61,243,136,48,61,109,197,51,61,47,9,55,61,55,84,58,61,130,166,61,61,15,0,65,61,218,96,68,61,226,200,71,61,35,56,75,61,156,174,78,61,73,44,82,61,40,177,85,61,55,61,89,61,115,208,92,61,217,106,96,61,103,12,100,61,25,181,103,61,238,100,107,61,227,27,111,61,244,217,114,61,30,159,118,61,96,107,122,61,182,62,126,61,143,12,129,61,73,253,130,61,138,241,132,61,79,233,134,61,150,228,136,61,94,227,138,61,167,229,140,61,109,235,142,61,175,244,144,61,109,1,147,61,164,17,149,61,83,37,151,61,120,60,153,61,17,87,155,61,30,117,157,61,155,150,159,61,136,187,161,61,226,227,163,61,169,15,166,61,218,62,168,61,116,113,170,61,116,167,172,61,218,224,174,61,162,29,177,61,205,93,179,61,87,161,181,61,62,232,183,61,130,50,186,61,32,128,188,61,22,209,190,61,98,37,193,61,2,125,195,61,245,215,197,61,57,54,200,61,203,151,202,61,169,252,204,61,211,100,207,61,68,208,209,61,252,62,212,61,249,176,214,61,56,38,217,61,184,158,219,61,117,26,222,61,111,153,224,61,163,27,227,61,14,161,229,61,175,41,232,61,132,181,234,61,138,68,237,61,191,214,239,61,33,108,242,61,174,4,245,61,99,160,247,61,62,63,250,61,61,225,252,61,93,134,255,61,78,23,1,62,252,108,2,62,56,196,3,62,255,28,5,62,81,119,6,62,45,211,7,62,145,48,9,62,125,143,10,62,238,239,11,62,228,81,13,62,94,181,14,62,89,26,16,62,214,128,17,62,210,232,18,62,77,82,20,62,69,189,21,62,184,41,23,62,166,151,24,62,13,7,26,62,236,119,27,62,65,234,28,62,11,94,30,62,73,211,31,62,250,73,33,62,28,194,34,62,173,59,36,62,172,182,37,62,24,51,39,62,240,176,40,62,50,48,42,62,220,176,43,62,238,50,45,62,101,182,46,62,64,59,48,62,126,193,49,62,30,73,51,62,29,210,52,62,123,92,54,62,54,232,55,62,76,117,57,62,187,3,59,62,131,147,60,62,162,36,62,62,22,183,63,62,222,74,65,62,248,223,66,62,98,118,68,62,28,14,70,62,35,167,71,62,117,65,73,62,18,221,74,62,247,121,76,62,35,24,78,62,149,183,79,62,74,88,81,62,66,250,82,62,121,157,84,62,240,65,86,62,163,231,87,62,146,142,89,62,186,54,91,62,26,224,92,62,177,138,94,62,124,54,96,62,122,227,97,62,169,145,99,62,7,65,101,62,147,241,102,62,75,163,104,62,44,86,106,62,54,10,108,62,102,191,109,62,187,117,111,62,51,45,113,62,204,229,114,62,132,159,116,62,90,90,118,62,75,22,120,62,85,211,121,62,120,145,123,62,176,80,125,62,253,16,127,62,46,105,128,62,101,74,129,62,36,44,130,62,105,14,131,62,52,241,131,62,130,212,132,62,84,184,133,62,169,156,134,62,127,129,135,62,213,102,136,62,171,76,137,62,255,50,138,62,209,25,139,62,32,1,140,62,233,232,140,62,46,209,141,62,236,185,142,62,34,163,143,62,208,140,144,62,244,118,145,62,142,97,146,62,156,76,147,62,29,56,148,62,17,36,149,62,118,16,150,62,76,253,150,62,144,234,151,62,67,216,152,62,99,198,153,62,239,180,154,62,230,163,155,62,71,147,156,62,17,131,157,62,67,115,158,62,219,99,159,62,218,84,160,62,60,70,161,62,3,56,162,62,43,42,163,62,181,28,164,62,160,15,165,62,233,2,166,62,145,246,166,62,149,234,167,62,245,222,168,62,176,211,169,62,197,200,170,62,50,190,171,62,246,179,172,62,17,170,173,62,129,160,174,62,69,151,175,62,91,142,176,62,196,133,177,62,125,125,178,62,133,117,179,62,220,109,180,62,128,102,181,62,112,95,182,62,171,88,183,62,47,82,184,62,252,75,185,62,17,70,186,62,108,64,187,62,11,59,188,62,239,53,189,62,22,49,190,62,126,44,191,62,38,40,192,62,13,36,193,62,51,32,194,62,150,28,195,62,52,25,196,62,12,22,197,62,30,19,198,62,104,16,199,62,233,13,200,62,159,11,201,62,138,9,202,62,169,7,203,62,249,5,204,62,123,4,205,62,44,3,206,62,11,2,207,62,24,1,208,62,81,0,209,62,181,255,209,62,66,255,210,62,248,254,211,62,213,254,212,62,216,254,213,62,255,254,214,62,75,255,215,62,184,255,216,62,71,0,218,62,245,0,219,62,195,1,220,62,173,2,221,62,180,3,222,62,214,4,223,62,17,6,224,62,101,7,225,62,208,8,226,62,81,10,227,62,231,11,228,62,144,13,229,62,76,15,230,62,25,17,231,62,245,18,232,62,224,20,233,62,217,22,234,62,221,24,235,62,236,26,236,62,5,29,237,62,39,31,238,62,79,33,239,62,125,35,240,62,176,37,241,62,230,39,242,62,31,42,243,62,88,44,244,62,145,46,245,62,200,48,246,62,253,50,247,62,45,53,248,62,88,55,249,62,124,57,250,62,153,59,251,62,172,61,252,62,181,63,253,62,179,65,254,62,163,67,255,62,195,34,0,63,173,163,0,63,142,36,1,63,102,165,1,63,53,38,2,63,250,166,2,63,180,39,3,63,99,168,3,63,5,41,4,63,155,169,4,63,36,42,5,63,159,170,5,63,12,43,6,63,105,171,6,63,183,43,7,63,244,171,7,63,32,44,8,63,59,172,8,63,68,44,9,63,58,172,9,63,28,44,10,63,235,171,10,63,164,43,11,63,73,171,11,63,216,42,12,63,80,170,12,63,177,41,13,63,251,168,13,63,44,40,14,63,69,167,14,63,68,38,15,63,41,165,15,63,243,35,16,63,162,162,16,63,53,33,17,63,172,159,17,63,5,30,18,63,65,156,18,63,95,26,19,63,94,152,19,63,61,22,20,63,252,147,20,63,155,17,21,63,24,143,21,63,116,12,22,63,173,137,22,63,195,6,23,63,182,131,23,63,133,0,24,63,46,125,24,63,179,249,24,63,18,118,25,63,74,242,25,63,91,110,26,63,69,234,26,63,6,102,27,63,159,225,27,63,14,93,28,63,84,216,28,63,111,83,29,63,95,206,29,63,36,73,30,63,188,195,30,63,40,62,31,63,102,184,31,63,119,50,32,63,90,172,32,63,14,38,33,63,146,159,33,63,230,24,34,63,10,146,34,63,253,10,35,63,190,131,35,63,77,252,35,63,169,116,36,63,211,236,36,63,200,100,37,63,138,220,37,63,22,84,38,63,110,203,38,63,143,66,39,63,122,185,39,63,47,48,40,63,172,166,40,63,241,28,41,63,254,146,41,63,210,8,42,63,108,126,42,63,205,243,42,63,243,104,43,63,223,221,43,63,143,82,44,63,3,199,44,63,59,59,45,63,54,175,45,63,244,34,46,63,116,150,46,63,182,9,47,63,185,124,47,63,125,239,47,63,1,98,48,63,69,212,48,63,72,70,49,63,10,184,49,63,139,41,50,63,202,154,50,63,198,11,51,63,127,124,51,63,246,236,51,63,40,93,52,63,22,205,52,63,191,60,53,63,36,172,53,63,66,27,54,63,27,138,54,63,174,248,54,63,249,102,55,63,254,212,55,63,187,66,56,63,47,176,56,63,91,29,57,63,63,138,57,63,217,246,57,63,41,99,58,63,48,207,58,63,236,58,59,63,93,166,59,63,130,17,60,63,93,124,60,63,235,230,60,63,44,81,61,63,33,187,61,63,201,36,62,63,35,142,62,63,48,247,62,63,238,95,63,63,94,200,63,63,126,48,64,63,80,152,64,63,209,255,64,63,3,103,65,63,228,205,65,63,117,52,66,63,181,154,66,63,163,0,67,63,64,102,67,63,139,203,67,63,131,48,68,63,41,149,68,63,124,249,68,63,123,93,69,63,39,193,69,63,127,36,70,63,132,135,70,63,51,234,70,63,142,76,71,63,148,174,71,63,68,16,72,63,159,113,72,63,164,210,72,63,83,51,73,63,172,147,73,63,174,243,73,63,89,83,74,63,173,178,74,63,169,17,75,63,77,112,75,63,154,206,75,63,143,44,76,63,43,138,76,63,110,231,76,63,89,68,77,63,234,160,77,63,34,253,77,63,0,89,78,63,133,180,78,63,176,15,79,63,128,106,79,63,246,196,79,63,18,31,80,63,210,120,80,63,56,210,80,63,66,43,81,63,242,131,81,63,69,220,81,63,61,52,82,63,217,139,82,63,24,227,82,63,252,57,83,63,131,144,83,63,174,230,83,63,123,60,84,63,236,145,84,63,0,231,84,63,183,59,85,63,16,144,85,63,12,228,85,63,170,55,86,63,235,138,86,63,206,221,86,63,83,48,87,63,121,130,87,63,66,212,87,63,172,37,88,63,184,118,88,63,101,199,88,63,180,23,89,63,164,103,89,63,53,183,89,63,104,6,90,63,59,85,90,63,175,163,90,63,197,241,90,63,123,63,91,63,210,140,91,63,201,217,91,63,97,38,92,63,154,114,92,63,115,190,92,63,237,9,93,63,7,85,93,63,194,159,93,63,29,234,93,63,24,52,94,63,179,125,94,63,239,198,94,63,203,15,95,63,72,88,95,63,100,160,95,63,33,232,95,63,126,47,96,63,123,118,96,63,24,189,96,63,85,3,97,63,51,73,97,63,177,142,97,63,207,211,97,63,141,24,98,63,236,92,98,63,235,160,98,63,138,228,98,63,202,39,99,63,170,106,99,63,42,173,99,63,75,239,99,63,13,49,100,63,111,114,100,63,114,179,100,63,21,244,100,63,90,52,101,63,63,116,101,63,197,179,101,63,236,242,101,63,180,49,102,63,29,112,102,63,39,174,102,63,211,235,102,63,32,41,103,63,15,102,103,63,159,162,103,63,209,222,103,63,164,26,104,63,26,86,104,63,49,145,104,63,235,203,104,63,71,6,105,63,69,64,105,63,230,121,105,63,42,179,105,63,16,236,105,63,153,36,106,63,197,92,106,63,148,148,106,63,7,204,106,63,29,3,107,63,214,57,107,63,52,112,107,63,53,166,107,63,218,219,107,63,36,17,108,63,18,70,108,63,164,122,108,63,220,174,108,63,184,226,108,63,57,22,109,63,96,73,109,63,44,124,109,63,157,174,109,63,181,224,109,63,115,18,110,63,214,67,110,63,225,116,110,63,146,165,110,63,233,213,110,63,232,5,111,63,142,53,111,63,219,100,111,63,209,147,111,63,110,194,111,63,179,240,111,63,160,30,112,63,54,76,112,63,117,121,112,63,93,166,112,63,239,210,112,63,41,255,112,63,14,43,113,63,156,86,113,63,213,129,113,63,184,172,113,63,70,215,113,63,127,1,114,63,99,43,114,63,243,84,114,63,46,126,114,63,21,167,114,63,169,207,114,63,233,247,114,63,214,31,115,63,113,71,115,63,184,110,115,63,173,149,115,63,80,188,115,63,162,226,115,63,161,8,116,63,80,46,116,63,174,83,116,63,187,120,116,63,119,157,116,63,228,193,116,63,1,230,116,63,206,9,117,63,76,45,117,63,123,80,117,63,92,115,117,63,238,149,117,63,51,184,117,63,42,218,117,63,211,251,117,63,48,29,118,63,64,62,118,63,3,95,118,63,122,127,118,63,166,159,118,63,134,191,118,63,27,223,118,63,101,254,118,63,101,29,119,63,27,60,119,63,135,90,119,63,169,120,119,63,131,150,119,63,19,180,119,63,91,209,119,63,91,238,119,63,20,11,120,63,132,39,120,63,174,67,120,63,145,95,120,63,46,123,120,63,132,150,120,63,149,177,120,63,96,204,120,63,231,230,120,63,41,1,121,63,38,27,121,63,223,52,121,63,85,78,121,63,136,103,121,63,120,128,121,63,37,153,121,63,144,177,121,63,185,201,121,63,161,225,121,63,72,249,121,63,174,16,122,63,212,39,122,63,185,62,122,63,96,85,122,63,198,107,122,63,238,129,122,63,216,151,122,63,131,173,122,63,241,194,122,63,33,216,122,63,20,237,122,63,202,1,123,63,68,22,123,63,130,42,123,63,133,62,123,63,77,82,123,63,217,101,123,63,43,121,123,63,68,140,123,63,34,159,123,63,200,177,123,63,52,196,123,63,104,214,123,63,99,232,123,63,39,250,123,63,180,11,124,63,9,29,124,63,40,46,124,63,17,63,124,63,196,79,124,63,65,96,124,63,137,112,124,63,156,128,124,63,124,144,124,63,39,160,124,63,158,175,124,63,226,190,124,63,244,205,124,63,211,220,124,63,128,235,124,63,251,249,124,63,69,8,125,63,94,22,125,63,71,36,125,63,255,49,125,63,136,63,125,63,225,76,125,63,11,90,125,63,7,103,125,63,212,115,125,63,115,128,125,63,229,140,125,63,42,153,125,63,66,165,125,63,46,177,125,63,238,188,125,63,130,200,125,63,235,211,125,63,41,223,125,63,61,234,125,63,38,245,125,63,230,255,125,63,124,10,126,63,234,20,126,63,47,31,126,63,75,41,126,63,64,51,126,63,13,61,126,63,180,70,126,63,51,80,126,63,140,89,126,63,191,98,126,63,205,107,126,63,181,116,126,63,120,125,126,63,23,134,126,63,146,142,126,63,233,150,126,63,28,159,126,63,44,167,126,63,26,175,126,63,229,182,126,63,142,190,126,63,22,198,126,63,124,205,126,63,194,212,126,63,231,219,126,63,235,226,126,63,208,233,126,63,149,240,126,63,59,247,126,63,195,253,126,63,44,4,127,63,118,10,127,63,163,16,127,63,179,22,127,63,165,28,127,63,123,34,127,63,52,40,127,63,210,45,127,63,83,51,127,63,186,56,127,63,5,62,127,63,53,67,127,63,75,72,127,63,72,77,127,63,42,82,127,63,243,86,127,63,163,91,127,63,58,96,127,63,185,100,127,63,32,105,127,63,111,109,127,63,166,113,127,63,199,117,127,63,208,121,127,63,196,125,127,63,161,129,127,63])
.concat([104,133,127,63,25,137,127,63,182,140,127,63,61,144,127,63,176,147,127,63,14,151,127,63,89,154,127,63,143,157,127,63,179,160,127,63,195,163,127,63,192,166,127,63,171,169,127,63,132,172,127,63,74,175,127,63,255,177,127,63,163,180,127,63,53,183,127,63,183,185,127,63,40,188,127,63,137,190,127,63,217,192,127,63,26,195,127,63,76,197,127,63,111,199,127,63,130,201,127,63,135,203,127,63,126,205,127,63,102,207,127,63,65,209,127,63,14,211,127,63,205,212,127,63,128,214,127,63,38,216,127,63,191,217,127,63,76,219,127,63,204,220,127,63,65,222,127,63,170,223,127,63,8,225,127,63,91,226,127,63,163,227,127,63,224,228,127,63,19,230,127,63,59,231,127,63,90,232,127,63,110,233,127,63,122,234,127,63,124,235,127,63,116,236,127,63,100,237,127,63,75,238,127,63,42,239,127,63,1,240,127,63,207,240,127,63,149,241,127,63,84,242,127,63,12,243,127,63,188,243,127,63,101,244,127,63,7,245,127,63,162,245,127,63,55,246,127,63,198,246,127,63,78,247,127,63,209,247,127,63,77,248,127,63,196,248,127,63,54,249,127,63,162,249,127,63,9,250,127,63,108,250,127,63,201,250,127,63,34,251,127,63,118,251,127,63,198,251,127,63,18,252,127,63,89,252,127,63,157,252,127,63,221,252,127,63,26,253,127,63,83,253,127,63,136,253,127,63,187,253,127,63,234,253,127,63,22,254,127,63,64,254,127,63,103,254,127,63,139,254,127,63,173,254,127,63,204,254,127,63,234,254,127,63,5,255,127,63,30,255,127,63,53,255,127,63,74,255,127,63,94,255,127,63,112,255,127,63,128,255,127,63,143,255,127,63,157,255,127,63,169,255,127,63,180,255,127,63,191,255,127,63,200,255,127,63,208,255,127,63,215,255,127,63,221,255,127,63,227,255,127,63,232,255,127,63,236,255,127,63,239,255,127,63,243,255,127,63,245,255,127,63,248,255,127,63,249,255,127,63,251,255,127,63,252,255,127,63,253,255,127,63,254,255,127,63,255,255,127,63,255,255,127,63,255,255,127,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,168,9,120,57,17,119,11,59,135,139,193,59,74,113,61,60,148,82,156,60,94,8,233,60,42,83,34,61,74,118,87,61,138,227,137,61,7,140,171,61,34,154,208,61,108,239,248,61,164,52,18,62,100,112,41,62,65,21,66,62,67,11,92,62,47,56,119,62,197,191,137,62,92,97,152,62,135,112,167,62,4,220,182,62,188,145,198,62,231,126,214,62,48,144,230,62,227,177,246,62,13,104,3,63,121,107,11,63,98,89,19,63,42,40,27,63,137,206,34,63,166,67,42,63,49,127,49,63,126,121,56,63,153,43,63,63,92,143,69,63,127,159,75,63,165,87,81,63,104,180,86,63,89,179,91,63,8,83,96,63,252,146,100,63,177,115,104,63,138,246,107,63,198,29,111,63,109,236,113,63,62,102,116,63,154,143,118,63,104,109,120,63,3,5,122,63,26,92,123,63,153,120,124,63,143,96,125,63,17,26,126,63,39,171,126,63,176,25,127,63,74,107,127,63,68,165,127,63,132,204,127,63,123,229,127,63,17,244,127,63,158,251,127,63,219,254,127,63,218,255,127,63,0,0,128,63,60,12,120,54,253,134,11,56,19,201,193,56,248,231,61,57,148,245,156,57,115,118,234,57,238,186,35,58,113,249,89,58,32,251,139,58,96,216,174,58,34,148,213,58,3,23,0,59,209,82,23,59,65,125,48,59,21,150,75,59,8,157,104,59,233,200,131,59,20,58,148,59,218,161,165,59,16,0,184,59,136,84,203,59,16,159,223,59,118,223,244,59,194,138,5,60,128,32,17,60,217,48,29,60,172,187,41,60,219,192,54,60,67,64,68,60,194,57,82,60,52,173,96,60,115,154,111,60,88,1,127,60,222,112,135,60,186,157,143,60,42,7,152,60,25,173,160,60,112,143,169,60,23,174,178,60,246,8,188,60,243,159,197,60,245,114,207,60,225,129,217,60,156,204,227,60,10,83,238,60,14,21,249,60,70,9,2,61,177,165,7,61,187,95,13,61,81,55,19,61,102,44,25,61,230,62,31,61,195,110,37,61,233,187,43,61,71,38,50,61,202,173,56,61,97,82,63,61,247,19,70,61,121,242,76,61,210,237,83,61,240,5,91,61,187,58,98,61,32,140,105,61,8,250,112,61,93,132,120,61,132,21,128,61,249,246,131,61,130,230,135,61,19,228,139,61,159,239,143,61,26,9,148,61,119,48,152,61,169,101,156,61,163,168,160,61,88,249,164,61,186,87,169,61,186,195,173,61,76,61,178,61,95,196,182,61,230,88,187,61,209,250,191,61,18,170,196,61,152,102,201,61,85,48,206,61,56,7,211,61,48,235,215,61,47,220,220,61,34,218,225,61,248,228,230,61,161,252,235,61,11,33,241,61,35,82,246,61,217,143,251,61,13,109,0,62,105,24,3,62,247,201,5,62,174,129,8,62,133,63,11,62,113,3,14,62,104,205,16,62,96,157,19,62,79,115,22,62,42,79,25,62,232,48,28,62,124,24,31,62,221,5,34,62,255,248,36,62,215,241,39,62,90,240,42,62,125,244,45,62,51,254,48,62,114,13,52,62,45,34,55,62,88,60,58,62,232,91,61,62,208,128,64,62,3,171,67,62,118,218,70,62,26,15,74,62,229,72,77,62,199,135,80,62,181,203,83,62,162,20,87,62,127,98,90,62,63,181,93,62,213,12,97,62,50,105,100,62,73,202,103,62,12,48,107,62,108,154,110,62,92,9,114,62,203,124,117,62,173,244,120,62,241,112,124,62,138,241,127,62,52,187,129,62,190,127,131,62,91,70,133,62,4,15,135,62,176,217,136,62,89,166,138,62,245,116,140,62,126,69,142,62,234,23,144,62,50,236,145,62,78,194,147,62,54,154,149,62,224,115,151,62,70,79,153,62,93,44,155,62,31,11,157,62,130,235,158,62,127,205,160,62,11,177,162,62,31,150,164,62,177,124,166,62,186,100,168,62,47,78,170,62,9,57,172,62,62,37,174,62,198,18,176,62,150,1,178,62,167,241,179,62,238,226,181,62,100,213,183,62,254,200,185,62,179,189,187,62,122,179,189,62,74,170,191,62,25,162,193,62,221,154,195,62,142,148,197,62,34,143,199,62,142,138,201,62,203,134,203,62,205,131,205,62,140,129,207,62,253,127,209,62,24,127,211,62,210,126,213,62,33,127,215,62,252,127,217,62,88,129,219,62,45,131,221,62,112,133,223,62,23,136,225,62,25,139,227,62,108,142,229,62,5,146,231,62,219,149,233,62,228,153,235,62,21,158,237,62,102,162,239,62,203,166,241,62,59,171,243,62,173,175,245,62,21,180,247,62,107,184,249,62,164,188,251,62,181,192,253,62,150,196,255,62,30,228,0,63,207,229,1,63,88,231,2,63,182,232,3,63,226,233,4,63,215,234,5,63,146,235,6,63,12,236,7,63,66,236,8,63,45,236,9,63,202,235,10,63,19,235,11,63,4,234,12,63,151,232,13,63,200,230,14,63,145,228,15,63,239,225,16,63,220,222,17,63,84,219,18,63,81,215,19,63,208,210,20,63,202,205,21,63,61,200,22,63,34,194,23,63,117,187,24,63,50,180,25,63,85,172,26,63,215,163,27,63,182,154,28,63,236,144,29,63,117,134,30,63,77,123,31,63,110,111,32,63,214,98,33,63,126,85,34,63,100,71,35,63,130,56,36,63,212,40,37,63,87,24,38,63,5,7,39,63,219,244,39,63,213,225,40,63,239,205,41,63,36,185,42,63,113,163,43,63,209,140,44,63,64,117,45,63,188,92,46,63,63,67,47,63,199,40,48,63,78,13,49,63,211,240,49,63,80,211,50,63,195,180,51,63,39,149,52,63,122,116,53,63,184,82,54,63,220,47,55,63,229,11,56,63,206,230,56,63,149,192,57,63,54,153,58,63,174,112,59,63,249,70,60,63,21,28,61,63,255,239,61,63,179,194,62,63,48,148,63,63,113,100,64,63,116,51,65,63,55,1,66,63,182,205,66,63,239,152,67,63,224,98,68,63,134,43,69,63,222,242,69,63,230,184,70,63,156,125,71,63,253,64,72,63,7,3,73,63,184,195,73,63,14,131,74,63,6,65,75,63,159,253,75,63,215,184,76,63,172,114,77,63,28,43,78,63,38,226,78,63,199,151,79,63,253,75,80,63,201,254,80,63,39,176,81,63,22,96,82,63,150,14,83,63,164,187,83,63,63,103,84,63,103,17,85,63,26,186,85,63,86,97,86,63,28,7,87,63,105,171,87,63,62,78,88,63,152,239,88,63,120,143,89,63,221,45,90,63,198,202,90,63,50,102,91,63,33,0,92,63,147,152,92,63,134,47,93,63,251,196,93,63,242,88,94,63,105,235,94,63,98,124,95,63,219,11,96,63,213,153,96,63,80,38,97,63,76,177,97,63,201,58,98,63,199,194,98,63,70,73,99,63,71,206,99,63,202,81,100,63,208,211,100,63,88,84,101,63,100,211,101,63,244,80,102,63,9,205,102,63,163,71,103,63,195,192,103,63,107,56,104,63,154,174,104,63,82,35,105,63,147,150,105,63,96,8,106,63,184,120,106,63,157,231,106,63,16,85,107,63,19,193,107,63,166,43,108,63,203,148,108,63,132,252,108,63,209,98,109,63,180,199,109,63,48,43,110,63,68,141,110,63,244,237,110,63,64,77,111,63,42,171,111,63,181,7,112,63,225,98,112,63,177,188,112,63,38,21,113,63,67,108,113,63,10,194,113,63,123,22,114,63,155,105,114,63,106,187,114,63,234,11,115,63,31,91,115,63,9,169,115,63,172,245,115,63,9,65,116,63,35,139,116,63,252,211,116,63,151,27,117,63,245,97,117,63,26,167,117,63,8,235,117,63,193,45,118,63,72,111,118,63,159,175,118,63,202,238,118,63,201,44,119,63,161,105,119,63,84,165,119,63,228,223,119,63,85,25,120,63,168,81,120,63,226,136,120,63,3,191,120,63,16,244,120,63,11,40,121,63,247,90,121,63,215,140,121,63,173,189,121,63,125,237,121,63,73,28,122,63,20,74,122,63,226,118,122,63,181,162,122,63,144,205,122,63,118,247,122,63,107,32,123,63,112,72,123,63,138,111,123,63,186,149,123,63,5,187,123,63,109,223,123,63,245,2,124,63,160,37,124,63,113,71,124,63,108,104,124,63,147,136,124,63,233,167,124,63,114,198,124,63,48,228,124,63,38,1,125,63,89,29,125,63,201,56,125,63,124,83,125,63,115,109,125,63,178,134,125,63,60,159,125,63,19,183,125,63,60,206,125,63,184,228,125,63,139,250,125,63,184,15,126,63,66,36,126,63,44,56,126,63,120,75,126,63,43,94,126,63,70,112,126,63,204,129,126,63,194,146,126,63,41,163,126,63,4,179,126,63,86,194,126,63,35,209,126,63,109,223,126,63,55,237,126,63,131,250,126,63,85,7,127,63,175,19,127,63,148,31,127,63,7,43,127,63,10,54,127,63,160,64,127,63,205,74,127,63,146,84,127,63,242,93,127,63,239,102,127,63,141,111,127,63,206,119,127,63,181,127,127,63,67,135,127,63,124,142,127,63,98,149,127,63,247,155,127,63,61,162,127,63,56,168,127,63,233,173,127,63,83,179,127,63,120,184,127,63,90,189,127,63,252,193,127,63,95,198,127,63,134,202,127,63,116,206,127,63,41,210,127,63,168,213,127,63,244,216,127,63,13,220,127,63,247,222,127,63,179,225,127,63,67,228,127,63,168,230,127,63,229,232,127,63,252,234,127,63,237,236,127,63,188,238,127,63,105,240,127,63,246,241,127,63,101,243,127,63,183,244,127,63,238,245,127,63,11,247,127,63,16,248,127,63,254,248,127,63,214,249,127,63,155,250,127,63,76,251,127,63,236,251,127,63,124,252,127,63,252,252,127,63,110,253,127,63,211,253,127,63,44,254,127,63,121,254,127,63,189,254,127,63,247,254,127,63,42,255,127,63,84,255,127,63,120,255,127,63,150,255,127,63,175,255,127,63,195,255,127,63,211,255,127,63,224,255,127,63,234,255,127,63,241,255,127,63,246,255,127,63,250,255,127,63,253,255,127,63,254,255,127,63,255,255,127,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,144,67,0,0,16,122,0,0,16,104,0,0,16,68,0,0,16,123,0,0,16,106,0,0,16,72,0,0,144,3,0,0,112,108,25,0,96,0,0,0,144,186,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,24,54,26,0,208,2,0,0,224,1,0,0,224,52,26,0,208,2,0,0,64,2,0,0,248,51,26,0,96,1,0,0,240,0,0,0,64,51,26,0,96,1,0,0,32,1,0,0,72,50,26,0,128,2,0,0,224,1,0,0,152,49,26,0,0,3,0,0,64,2,0,0,0,48,26,0,96,1,0,0,240,0,0,0,96,46,26,0,96,1,0,0,240,0,0,0,120,41,26,0,128,0,0,0,96,0,0,0,56,40,26,0,176,0,0,0,144,0,0,0,200,38,26,0,96,1,0,0,32,1,0,0,248,37,26,0,192,2,0,0,64,2,0,0,152,37,26,0,128,5,0,0,128,4,0,0,248,36,26,0,160,0,0,0,120,0,0,0,72,36,26,0,64,1,0,0,240,0,0,0,200,35,26,0,128,2,0,0,224,1,0,0,8,34,26,0,32,3,0,0,88,2,0,0,152,32,26,0,0,4,0,0,0,3,0,0,160,31,26,0,64,6,0,0,176,4,0,0,72,30,26,0,0,8,0,0,0,6,0,0,200,28,26,0,0,5,0,0,0,4,0,0,40,28,26,0,0,10,0,0,0,8,0,0,8,27,26,0,0,20,0,0,0,16,0,0,168,21,26,0,84,3,0,0,224,1,0,0,80,21,26,0,86,5,0,0,0,3,0,0,240,20,26,0,64,6,0,0,0,4,0,0,168,19,26,0,128,7,0,0,176,4,0,0,128,18,26,0,0,10,0,0,64,6,0,0,0,18,26,0,128,12,0,0,0,8,0,0,224,16,26,0,0,15,0,0,96,9,0,0,128,15,26,0,0,25,0,0,0,16,0,0,16,15,26,0,0,30,0,0,192,18,0,0,168,14,26,0,64,1,0,0,200,0,0,0,200,13,26,0,128,2,0,0,94,1,0,0,136,13,26,0,84,3,0,0,224,1,0,0,240,12,26,0,0,5,0,0,208,2,0,0,120,11,26,0,128,7,0,0,56,4,0,0,120,10,26,0,0,8,0,0,56,4,0,0,224,9,26,0,206,7,0,0,56,4,0,0,232,8,26,0,0,8,0,0,90,3,0,0,0,7,26,0,0,16,0,0,112,8,0,0,104,6,26,0,156,15,0,0,112,8,0,0,24,6,26,0,0,16,0,0,180,6,0,0,0,0,0,0,24,54,26,0,48,117,0,0,233,3,0,0,224,52,26,0,25,0,0,0,1,0,0,0,248,51,26,0,48,117,0,0,233,3,0,0,64,51,26,0,25,0,0,0,1,0,0,0,72,50,26,0,48,117,0,0,233,3,0,0,152,49,26,0,25,0,0,0,1,0,0,0,0,48,26,0,24,0,0,0,1,0,0,0,96,46,26,0,192,93,0,0,233,3,0,0,11,0,0,0,136,79,1,0,192,82,1,0,255,255,255,255,80,195,0,0,64,13,3,0,32,174,4,0,80,174,4,0,136,68,27,0,8,70,27,0,24,68,27,0,144,141,24,0,192,144,24,0,144,141,24,0,32,88,27,0,48,117,27,0,16,98,27,0,128,88,27,0,160,107,27,0,32,141,27,0,64,156,27,0,40,154,27,0,136,154,27,0,96,161,27,0,144,161,27,0,216,161,27,0,8,162,27,0,208,160,27,0,64,160,27,0,144,160,27,0,104,141,27,0,136,144,27,0,144,254,28,0,0,0,0,0,248,255,28,0,200,1,29,0,2,0,0,0,176,255,28,0,144,162,27,0,0,0,0,0,11,0,0,0,136,79,1,0,192,82,1,0,2,0,0,0,80,195,0,0,64,13,3,0,32,174,4,0,80,174,4,0,136,68,27,0,8,70,27,0,24,68,27,0,144,141,24,0,192,144,24,0,144,141,24,0,32,88,27,0,48,117,27,0,16,98,27,0,128,88,27,0,160,107,27,0,32,141,27,0,64,156,27,0,40,154,27,0,136,154,27,0,96,161,27,0,144,161,27,0,216,161,27,0,8,162,27,0,208,160,27,0,64,160,27,0,144,160,27,0,104,141,27,0,136,144,27,0,144,254,28,0,8,73,27,0,248,255,28,0,200,1,29,0,2,0,0,0,176,255,28,0,232,162,27,0,0,0,0,0,2,0,0,0,136,79,1,0,168,82,1,0,255,255,255,255,0,0,0,0,64,31,0,0,128,174,4,0,128,174,4,0,72,68,27,0,8,70,27,0,24,68,27,0,240,147,24,0,0,0,0,0,240,147,24,0,8,88,27,0,192,126,27,0,192,126,27,0,0,0,0,0,0,0,0,0,32,141,27,0,0,155,27,0,232,154,27,0,0,0,0,0,192,161,27,0,192,161,27,0,56,162,27,0,56,162,27,0,192,160,27,0,32,160,27,0,112,160,27,0,80,141,27,0,136,144,27,0,120,254,28,0,56,70,27,0,248,255,28,0,200,1,29,0,1,0,0,0,24,255,28,0,112,162,27,0,0,0,0,0,2,0,0,0,136,79,1,0,168,82,1,0,2,0,0,0,0,0,0,0,64,31,0,0,128,174,4,0,128,174,4,0,72,68,27,0,8,70,27,0,24,68,27,0,240,147,24,0,0,0,0,0,240,147,24,0,8,88,27,0,192,126,27,0,192,126,27,0,0,0,0,0,0,0,0,0,32,141,27,0,0,155,27,0,232,154,27,0,0,0,0,0,192,161,27,0,192,161,27,0,56,162,27,0,56,162,27,0,192,160,27,0,32,160,27,0,112,160,27,0,80,141,27,0,136,144,27,0,120,254,28,0,56,70,27,0,248,255,28,0,200,1,29,0,1,0,0,0,24,255,28,0,128,162,27,0,0,0,0,0,2,0,0,0,232,79,1,0,168,82,1,0,255,255,255,255,64,31,0,0,40,35,0,0,128,174,4,0,128,174,4,0,72,68,27,0,8,70,27,0,24,68,27,0,240,147,24,0,0,0,0,0,240,147,24,0,8,88,27,0,192,126,27,0,192,126,27,0,0,0,0,0,0,0,0,0,32,141,27,0,0,155,27,0,232,154,27,0,0,0,0,0,192,161,27,0,192,161,27,0,56,162,27,0,56,162,27,0,192,160,27,0,32,160,27,0,112,160,27,0,80,141,27,0,136,144,27,0,120,254,28,0,56,70,27,0,248,255,28,0,200,1,29,0,1,0,0,0,24,255,28,0,112,162,27,0,0,0,0,0,2,0,0,0,0,80,1,0,168,82,1,0,2,0,0,0,64,31,0,0,40,35,0,0,128,174,4,0,128,174,4,0,72,68,27,0,8,70,27,0,24,68,27,0,240,147,24,0,0,0,0,0,240,147,24,0,8,88,27,0,192,126,27,0,192,126,27,0,0,0,0,0,0,0,0,0,32,141,27,0,0,155,27,0,232,154,27,0,0,0,0,0,192,161,27,0,192,161,27,0,56,162,27,0,56,162,27,0,192,160,27,0,32,160,27,0,112,160,27,0,80,141,27,0,136,144,27,0,120,254,28,0,56,70,27,0,248,255,28,0,200,1,29,0,1,0,0,0,24,255,28,0,128,162,27,0,0,0,0,0,11,0,0,0,120,80,1,0,192,82,1,0,255,255,255,255,64,156,0,0,80,195,0,0,32,174,4,0,80,174,4,0,136,68,27,0,8,70,27,0,24,68,27,0,144,141,24,0,192,144,24,0,144,141,24,0,32,88,27,0,48,117,27,0,16,98,27,0,128,88,27,0,160,107,27,0,32,141,27,0,64,156,27,0,40,154,27,0,136,154,27,0,96,161,27,0,144,161,27,0,216,161,27,0,8,162,27,0,208,160,27,0,64,160,27,0,144,160,27,0,104,141,27,0,136,144,27,0,144,254,28,0,8,73,27,0,248,255,28,0,200,1,29,0,2,0,0,0,176,255,28,0,144,162,27,0,0,0,0,0,11,0,0,0,216,80,1,0,192,82,1,0,2,0,0,0,64,156,0,0,80,195,0,0,32,174,4,0,80,174,4,0,136,68,27,0,8,70,27,0,24,68,27,0,144,141,24,0,192,144,24,0,144,141,24,0,32,88,27,0,48,117,27,0,16,98,27,0,128,88,27,0,160,107,27,0,32,141,27,0,64,156,27,0,40,154,27,0,136,154,27,0,96,161,27,0,144,161,27,0,216,161,27,0,8,162,27,0,208,160,27,0,64,160,27,0,144,160,27,0,104,141,27,0,136,144,27,0,144,254,28,0,8,73,27,0,248,255,28,0,200,1,29,0,2,0,0,0,176,255,28,0,232,162,27,0,0,0,0,0,11,0,0,0,24,80,1,0,192,82,1,0,6,0,0,0,64,156,0,0,112,17,1,0,32,174,4,0,80,174,4,0,136,68,27,0,8,70,27,0,24,68,27,0,144,141,24,0,192,144,24,0,144,141,24,0,32,88,27,0,48,117,27,0,16,98,27,0,128,88,27,0,160,107,27,0,32,141,27,0,64,156,27,0,40,154,27,0,136,154,27,0,96,161,27,0,144,161,27,0,216,161,27,0,8,162,27,0,208,160,27,0,64,160,27,0,144,160,27,0,104,141,27,0,136,144,27,0,144,254,28,0,8,73,27,0,248,255,28,0,200,1,29,0,3,0,0,0,176,255,28,0,64,163,27,0,0,0,0,0,11,0,0,0,56,81,1,0,192,82,1,0,255,255,255,255,144,101,0,0,64,156,0,0,32,174,4,0,80,174,4,0,136,68,27,0,8,70,27,0,24,68,27,0,144,141,24,0,192,144,24,0,144,141,24,0,32,88,27,0,48,117,27,0,16,98,27,0,128,88,27,0,160,107,27,0,32,141,27,0,64,156,27,0,40,154,27,0,136,154,27,0,96,161,27,0,144,161,27,0,216,161,27,0,8,162,27,0,208,160,27,0,64,160,27,0,144,160,27,0,200,141,27,0,136,144,27,0,144,254,28,0,0,0,0,0,248,255,28,0,200,1,29,0,2,0,0,0,176,255,28,0,144,162,27,0,0,0,0,0,11,0,0,0,152,81,1,0,192,82,1,0,2,0,0,0,144,101,0,0,64,156,0,0,32,174,4,0,80,174,4,0,136,68,27,0,8,70,27,0,24,68,27,0,144,141,24,0,192,144,24,0,144,141,24,0,32,88,27,0,48,117,27,0,16,98,27,0,128,88,27,0,160,107,27,0,32,141,27,0,64,156,27,0,40,154,27,0,136,154,27,0,96,161,27,0,144,161,27,0,216,161,27,0,8,162,27,0,208,160,27,0,64,160,27,0,144,160,27,0,200,141,27,0,136,144,27,0,144,254,28,0,8,73,27,0,248,255,28,0,200,1,29,0,2,0,0,0,176,255,28,0,232,162,27,0,0,0,0,0,3,0,0,0,248,81,1,0,32,83,1,0,255,255,255,255,56,74,0,0,144,101,0,0,136,174,4,0,152,174,4,0,120,69,27,0,8,70,27,0,24,68,27,0,192,148,24,0,192,148,24,0,192,148,24,0,80,88,27,0,88,132,27,0,40,129,27,0,40,129,27,0,136,135,27,0,32,141,27,0,0,155,27,0,0,160,27,0,0,160,27,0,200,161,27,0,200,161,27,0,64,162,27,0,64,162,27,0,40,161,27,0,48,160,27,0,128,160,27,0,40,142,27,0,136,144,27,0,240,254,28,0,72,84,27,0,248,255,28,0,200,1,29,0,2,0,0,0,224,255,28,0,152,163,27,0,0,0,0,0,3,0,0,0,24,82,1,0,32,83,1,0,2,0,0,0,56,74,0,0,144,101,0,0,136,174,4,0,152,174,4,0,120,69,27,0,8,70,27,0,24,68,27,0,192,148,24,0,192,148,24,0,192,148,24,0,80,88,27,0,88,132,27,0,40,129,27,0,40,129,27,0,136,135,27,0,32,141,27,0,0,155,27,0,0,160,27,0,0,160,27,0,200,161,27,0,200,161,27,0,64,162,27,0,64,162,27,0,40,161,27,0,48,160,27,0,128,160,27,0,40,142,27,0,136,144,27,0,240,254,28,0,72,84,27,0,248,255,28,0,200,1,29,0,2,0,0,0,224,255,28,0,176,163,27,0,0,0,0,0,3,0,0,0,56,82,1,0,32,83,1,0,255,255,255,255,152,58,0,0,56,74,0,0,136,174,4,0,152,174,4,0,120,69,27,0,8,70,27,0,24,68,27,0,192,148,24,0,192,148,24,0,192,148,24,0,80,88,27,0,88,132,27,0,40,129,27,0,40,129,27,0,136,135,27,0,32,141,27,0,0,155,27,0,0,160,27,0,0,160,27,0,200,161,27,0,200,161,27,0,64,162,27,0,64,162,27,0,40,161,27,0,48,160,27,0,128,160,27,0,72,142,27,0,136,144,27,0,240,254,28,0,72,84,27,0,248,255,28,0,200,1,29,0,2,0,0,0,224,255,28,0,152,163,27,0,0,0,0,0,3,0,0,0,88,82,1,0,32,83,1,0,2,0,0,0,152,58,0,0,56,74,0,0,136,174,4,0,152,174,4,0,120,69,27,0,8,70,27,0,24,68,27,0,192,148,24,0,192,148,24,0,192,148,24,0,80,88,27,0,88,132,27,0,40,129,27,0,40,129,27,0,136,135,27,0,32,141,27,0,0,155,27,0,0,160,27,0,0,160,27,0,200,161,27,0,200,161,27,0,64,162,27,0,64,162,27,0,40,161,27,0,48,160,27,0,128,160,27,0,72,142,27,0,136,144,27,0,240,254,28,0,72,84,27,0,248,255,28,0,200,1,29,0,2,0,0,0,224,255,28,0,176,163,27,0,0,0,0,0,2,0,0,0,120,82,1,0,64,83,1,0,255,255,255,255,40,35,0,0,152,58,0,0,168,174,4,0,168,174,4,0,200,69,27,0,8,70,27,0,24,68,27,0,208,149,24,0,0,0,0,0,208,149,24,0,8,88,27,0,184,138,27,0,184,138,27,0,0,0,0,0,0,0,0,0,32,141,27,0,0,155,27,0,232,154,27,0,0,0,0,0,192,161,27,0,192,161,27,0,56,162,27,0,56,162,27,0,72,161,27,0,32,160,27,0,112,160,27,0,104,142,27,0,136,144,27,0,120,254,28,0,56,70,27,0,248,255,28,0,200,1,29,0,1,0,0,0,240,255,28,0,112,162,27,0,0,0,0,0,2,0,0,0,144,82,1,0,64,83,1,0,2,0,0,0,40,35,0,0,152,58,0,0,168,174,4,0,168,174,4,0,200,69,27,0,8,70,27,0,24,68,27,0,208,149,24,0,0,0,0,0,208,149,24,0,8,88,27,0,184,138,27,0,184,138,27,0,0,0,0,0,0,0,0,0,32,141,27,0,0,155,27,0,232,154,27,0,0,0,0,0,192,161,27,0,192,161,27,0,56,162,27,0,56,162,27,0,72,161,27,0,32,160,27,0,112,160,27,0,104,142,27,0,136,144,27,0,120,254,28,0,56,70,27,0,248,255,28,0,200,1,29,0,1,0,0,0,240,255,28,0,128,162,27,0,0,0,0,0,137,158,227,63,229,83,236,63,167,94,245,63,155,20,249,63,14,217,252,63,123,143,234,63,218,151,217,63,226,132,191,63,124,145,168,63,0,0,128,63,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,102,102,166,64,0,0,250,66,102,102,134,192,154,153,201,192,154,153,153,64,0,0,128,63,0,0,0,0,0,0,0,0,2,0,0,0,21,0,0,0,236,81,120,63,0,0,160,64,0,0,200,66,1,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,154,153,169,64,0,0,250,66,102,102,102,192,51,51,179,192,0,0,144,64,0,0,192,63,0,0,0,0,0,0,0,0,2,0,0,0,21,0,0,0,205,204,172,63,0,0,160,64,0,0,200,66,2,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,51,51,179,64,0,0,250,66,205,204,12,192,0,0,96,192,51,51,51,64,0,0,0,64,0,0,0,0,0,0,0,0,2,0,0,0,21,0,0,0,82,184,190,63,0,0,160,64,0,0,200,66,3,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,154,153,185,64,0,0,2,67,102,102,230,191,51,51,51,192,102,102,38,64,0,0,64,64,0,0,128,192,0,0,0,0,2,0,0,0,20,0,0,0,133,235,209,63,0,0,160,64,0,0,200,66,4,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,0,0,192,64,0,0,7,67,51,51,51,191,205,204,140,191,205,204,140,63,0,0,96,64,0,0,0,193,0,0,0,0,2,0,0,0,0,0,0,0,184,30,229,63,0,0,160,64,0,0,200,66,5,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,205,204,204,64,0,0,12,67,0,0,0,63,205,204,204,62,0,0,240,192,0,0,128,64,0,0,64,193,23,183,81,57,0,0,0,0,0,0,0,0,154,153,249,63,0,0,160,64,0,0,200,66,6,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,51,51,211,64,0,0,17,67,31,133,43,63,102,102,38,63,51,51,107,193,0,0,208,64,0,0,152,193,23,183,209,57,0,0,0,0,0,0,0,0,51,51,19,64,0,0,160,64,0,0,200,66,7,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,51,51,211,64,0,0,17,67,205,204,76,63,0,0,64,63,154,153,157,193,0,0,0,65,0,0,176,193,82,73,29,58,0,0,0,0,0,0,0,0,205,204,44,64,0,0,160,64,0,0,200,66,8,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,51,51,211,64,0,0,17,67,154,153,153,63,51,51,147,63,0,0,220,193,0,0,32,65,0,0,184,193,52,128,55,58,0,0,0,0,0,0,0,0,0,0,0,0,0,0,160,64,0,0,200,66,9,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,51,51,211,64,0,0,17,67,205,204,204,63,205,204,204,63,0,0,16,194,0,0,48,65,0,0,200,193,23,183,81,58,0,0,0,0,0,0,0,0,0,0,0,0,0,0,160,64,0,0,200,66,10,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,51,51,211,64,0,0,17,67,0,0,0,64,0,0,0,64,0,0,16,194,0,0,64,65,0,0,200,193,23,183,81,58,0,0,0,0,0,0,0,0,0,0,0,0,0,0,160,64,0,0,200,66,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,102,102,134,64,0,0,200,65,154,153,217,192,154,153,217,192,51,51,227,64,0,0,128,63,0,0,0,0,0,0,0,0,2,0,0,0,31,0,0,0,0,0,128,63,0,0,160,64,0,0,200,66,1,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,102,102,134,64,0,0,200,65,154,153,153,192,154,153,153,192,205,204,172,64,51,51,179,63,0,0,128,191,0,0,0,0,2,0,0,0,27,0,0,0,178,157,143,63,0,0,160,64,0,0,196,66,2,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,102,102,134,64,0,0,200,65,102,102,38,192,102,102,38,192,205,204,108,64,0,0,0,64,0,0,64,192,0,0,0,0,2,0,0,0,23,0,0,0,47,221,164,63,0,0,160,64,0,0,194,66,3,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,102,102,134,64,0,0,200,65,205,204,204,191,205,204,204,191,0,0,0,64,0,0,0,64,0,0,160,192,0,0,0,0,2,0,0,0,18,0,0,0,223,79,189,63,0,0,160,64,0,0,192,66,4,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,102,102,134,64,0,0,200,65,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,0,0,0,193,0,0,0,0,2,0,0,0,12,0,0,0,16,88,217,63,0,0,160,64,0,0,190,66,5,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,102,102,134,64,0,0,200,65,102,102,166,63,102,102,166,63,0,0,192,192,0,0,96,64,0,0,48,193,0,0,0,0,2,0,0,0,8,0,0,0,154,153,249,63,0,0,160,64,102,102,188,66,6,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,0,0,144,64,0,0,200,66,205,204,12,64,51,51,19,64,0,0,64,193,0,0,192,64,0,0,96,193,0,0,0,0,2,0,0,0,4,0,0,0,199,75,15,64,0,0,64,64,205,204,187,66,7,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,154,153,153,64,0,0,72,67,205,204,44,64,205,204,44,64,0,0,144,193,0,0,16,65,0,0,136,193,0,0,0,0,2,0,0,0,0,0,0,0,225,122,36,64,0,0,128,63,51,51,187,66,8,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,154,153,169,64,0,0,150,67,51,51,51,64,51,51,51,64,0,0,168,193,0,0,32,65,0,0,184,193,23,183,81,57,0,0,0,0,0,0,0,0,47,221,60,64,0,0,0,0,154,153,186,66,9,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,51,51,211,64,0,0,150,67,51,51,51,64,51,51,51,64,0,0,184,193,0,0,48,65,0,0,200,193,82,73,29,58,0,0,0,0,0,0,0,0,254,212,88,64,0,0,0,0,154,153,186,66,10,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,0,0,200,65,0,0,150,67,51,51,51,64,51,51,51,64,0,0,200,193,0,0,64,65,0,0,216,193,10,215,35,59,0,0,0,0,0,0,0,0,0,0,96,64,0,0,0,0,154,153,186,66,0,0,0,0,240,157,25,0,192,93,25,0,240,27,25,0,112,233,24,0,152,207,24,0,64,182,24,0,24,166,24,0,96,158,26,0,168,132,26,0,0,115,26,0,248,94,26,0,120,78,26,0,80,64,26,0,192,48,26,0,96,34,26,0,8,20,26,0,216,11,26,0,232,3,26,0,200,251,25,0,168,241,25,0,0,0,0,0,0,0,0,0,248,29,25,0,240,23,25,0,224,16,25,0,0,0,0,0,18,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,17,16,16,16,14,14,14,13,12,11,8,6,4,2,1,3,5,7,9,11,13,14,14,15,16,17,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,0,20,20,20,20,20,20,20,18,19,19,19,19,18,18,20,19,17,18,17,16,16,15,14,12,11,10,9,8,6,4,2,1,3,5,8,9,10,11,12,13,14,15,15,16,16,17,17,18,18,18,20,19,19,19,20,19,19,20,20,20,20,20,20,0,18,18,18,18,18,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,17,18,16,17,18,17,16,16,16,16,15,14,14,13,13,12,11,10,9,8,7,6,5,4,3,2,2,3,4,5,6,7,8,9,10,12,13,14,14,15,16,17,16,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,0,0,0,0,0,0,0,19,19,20,20,20,20,20,20,20,19,20,20,20,20,19,20,19,19,20,18,20,20,20,19,20,20,20,19,20,19,18,19,18,18,17,18,17,17,17,16,16,16,15,15,14,13,13,12,12,11,10,9,9,8,7,6,5,4,3,2,2,3,4,5,6,8,8,9,10,11,11,11,12,12,13,13,14,14,16,16,17,17,18,18,18,18,18,18,18,20,19,20,20,20,20,20,20,19,20,20,20,20,19,20,18,20,20,19,19,20,20,20,20,20,20,20,20,20,20,20,20,0,0,0,0,0,0,0,237,255,3,0,238,255,3,0,222,255,7,0,223,255,7,0,224,255,7,0,225,255,7,0,226,255,7,0,227,255,7,0,228,255,7,0,229,255,7,0,230,255,7,0,231,255,7,0,232,255,7,0,233,255,7,0,234,255,7,0,235,255,7,0,236,255,7,0,244,255,1,0,247,255,0,0,249,255,0,0,248,255,0,0,251,63,0,0,250,63,0,0,248,63,0,0,250,31,0,0,252,15,0,0,252,7,0,0,254,0,0,0,62,0,0,0,14,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,30,0,0,0,126,0,0,0,254,1,0,0,253,7,0,0,251,31,0,0,249,63,0,0,252,63,0,0,250,127,0,0,246,255,0,0,245,255,1,0,236,255,3,0,237,255,7,0,238,255,7,0,239,255,7,0,240,255,7,0,241,255,7,0,242,255,7,0,243,255,7,0,244,255,7,0,245,255,7,0,246,255,7,0,247,255,7,0,248,255,7,0,249,255,7,0,250,255,7,0,251,255,7,0,252,255,7,0,253,255,7,0,254,255,7,0,255,255,7,0,0,0,0,0,240,255,15,0,241,255,15,0,242,255,15,0,243,255,15,0,244,255,15,0,245,255,15,0,246,255,15,0,243,255,3,0,245,255,7,0,238,255,7,0,239,255,7,0,246,255,7,0,244,255,3,0,242,255,3,0,247,255,15,0,240,255,7,0,245,255,1,0,240,255,3,0,244,255,1,0,247,255,0,0,246,255,0,0,248,127,0,0,251,63,0,0,253,15,0,0,253,7,0,0,253,3,0,0,253,1,0,0,253,0,0,0,62,0,0,0,14,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,30,0,0,0,252,0,0,0,252,1,0,0,252,3,0,0,252,7,0,0,252,15,0,0,252,31,0,0,250,63,0,0,249,127,0,0,250,127,0,0,248,255,0,0,249,255,0,0,246,255,1,0,247,255,1,0,245,255,3,0,246,255,3,0,241,255,3,0,248,255,15,0,241,255,7,0,242,255,7,0,243,255,7,0,249,255,15,0,247,255,7,0,244,255,7,0,250,255,15,0,251,255,15,0,252,255,15,0,253,255,15,0,254,255,15,0,255,255,15,0,0,0,0,0,214,255,3,0,215,255,3,0,216,255,3,0,217,255,3,0,218,255,3,0,219,255,3,0,184,255,7,0,185,255,7,0,186,255,7,0,187,255,7,0,188,255,7,0,189,255,7,0,190,255,7,0,191,255,7,0,192,255,7,0,193,255,7,0,194,255,7,0,195,255,7,0,196,255,7,0,197,255,7,0,198,255,7,0,199,255,7,0,200,255,7,0,201,255,7,0,202,255,7,0,203,255,7,0,204,255,7,0,205,255,7,0,206,255,7,0,207,255,7,0,208,255,7,0,209,255,7,0,210,255,7,0,211,255,7,0,230,255,1,0,212,255,3,0,240,255,0,0,233,255,1,0,213,255,3,0,231,255,1,0,241,255,0,0,236,255,0,0,237,255,0,0,238,255,0,0,244,127,0,0,249,63,0,0,247,63,0,0,250,31,0,0,249,31,0,0,251,15,0,0,252,7,0,0,252,3,0,0,253,1,0,0,253,0,0,0,125,0,0,0,61,0,0,0,29,0,0,0,13,0,0,0,5,0,0,0,1,0,0,0,0,0,0,0,4,0,0,0,12,0,0,0,28,0,0,0,60,0,0,0,124,0,0,0,252,0,0,0,252,1,0,0,253,3,0,0,250,15,0,0,248,31,0,0,246,63,0,0,248,63,0,0,245,127,0,0,239,255,0,0,232,255,1,0,242,255,0,0,212,255,7,0,213,255,7,0,214,255,7,0,215,255,7,0,216,255,7,0,217,255,7,0,218,255,7,0,219,255,7,0,220,255,7,0,221,255,7,0,222,255,7,0,223,255,7,0,224,255,7,0,225,255,7,0,226,255,7,0,227,255,7,0,228,255,7,0,229,255,7,0,230,255,7,0,231,255,7,0,232,255,7,0,233,255,7,0,234,255,7,0,235,255,7,0,236,255,7,0,237,255,7,0,238,255,7,0,239,255,7,0,240,255,7,0,241,255,7,0,242,255,7,0,243,255,7,0,244,255,7,0,245,255,7,0,246,255,7,0,247,255,7,0,248,255,7,0,249,255,7,0,250,255,7,0,251,255,7,0,252,255,7,0,253,255,7,0,254,255,7,0,255,255,7,0,0,0,0,0,231,255,7,0,232,255,7,0,210,255,15,0,211,255,15,0,212,255,15,0,213,255,15,0,214,255,15,0,215,255,15,0,216,255,15,0,218,255,7,0,217,255,15,0,218,255,15,0,219,255,15,0,220,255,15,0,219,255,7,0,221,255,15,0,220,255,7,0,221,255,7,0,222,255,15,0,228,255,3,0,223,255,15,0,224,255,15,0,225,255,15,0,222,255,7,0,226,255,15,0,227,255,15,0,228,255,15,0,223,255,7,0,229,255,15,0,224,255,7,0,232,255,3,0,225,255,7,0,224,255,3,0,233,255,3,0,239,255,1,0,229,255,3,0,236,255,1,0,237,255,1,0,238,255,1,0,244,255,0,0,243,255,0,0,240,255,0,0,247,127,0,0,246,127,0,0,250,63,0,0,250,31,0,0,249,31,0,0,250,15,0,0,248,15,0,0,249,7,0,0,251,3,0,0,252,1,0,0,250,1,0,0,251,0,0,0,124,0,0,0,60,0,0,0,28,0,0,0,12,0,0,0,5,0,0,0,1,0,0,0,0,0,0,0,4,0,0,0,13,0,0,0,29,0,0,0,61,0,0,0,250,0,0,0,252,0,0,0,251,1,0,0,250,3,0,0,248,7,0,0,250,7,0,0,251,7,0,0,249,15,0,0,251,15,0,0,248,31,0,0,251,31,0,0,248,63,0,0,249,63,0,0,241,255,0,0,242,255,0,0,234,255,1,0,235,255,1,0,225,255,3,0,226,255,3,0,234,255,3,0,227,255,3,0,230,255,3,0,231,255,3,0,235,255,3,0,230,255,15,0,226,255,7,0,231,255,15,0,232,255,15,0,233,255,15,0,234,255,15,0,235,255,15,0,236,255,15,0,227,255,7,0,237,255,15,0,238,255,15,0,239,255,15,0,240,255,15,0,228,255,7,0,241,255,15,0,236,255,3,0,242,255,15,0,243,255,15,0,229,255,7,0,230,255,7,0,244,255,15,0,245,255,15,0,246,255,15,0,247,255,15,0,248,255,15,0,249,255,15,0,250,255,15,0,251,255,15,0,252,255,15,0,253,255,15,0,254,255,15,0,255,255,15,0,0,0,0,0,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,11,8,6,4,3,1,2,5,8,10,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,14,14,0,206,31,0,0,207,31,0,0,208,31,0,0,209,31,0,0,210,31,0,0,211,31,0,0,212,31,0,0,213,31,0,0,214,31,0,0,215,31,0,0,216,31,0,0,217,31,0,0,218,31,0,0,219,31,0,0,220,31,0,0,221,31,0,0,222,31,0,0,223,31,0,0,224,31,0,0,225,31,0,0,226,31,0,0,227,31,0,0,228,31,0,0,229,31,0,0,230,31,0,0,231,31,0,0,242,7,0,0,253,0,0,0,62,0,0,0,14,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,30,0,0,0,252,0,0,0,248,3,0,0,204,31,0,0,232,31,0,0,233,31,0,0,234,31,0,0,235,31,0,0,236,31,0,0,205,31,0,0,237,31,0,0,238,31,0,0,239,31,0,0,240,31,0,0,241,31,0,0,242,31,0,0,243,31,0,0,244,31,0,0,245,31,0,0,246,31,0,0,247,31,0,0,248,31,0,0,249,31,0,0,250,31,0,0,251,31,0,0,252,31,0,0,253,31,0,0,254,31,0,0,254,63,0,0,255,63,0,0,0,0,0,0,255,255,255,255,0,0,0,0,64,34,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,104,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,144,157,25,0,104,93,25,0,16,0,0,0,3,0,0,0,255,255,255,255,255,255,239,127])
.concat([255,255,255,255,255,255,239,255,255,255,255,255,255,255,239,127,16,0,1,0,0,0,0,0,160,27,25,0,232,232,24,0,24,0,0,0,3,0,0,0,255,255,255,255,255,255,239,127,255,255,255,255,255,255,239,255,255,255,255,255,255,255,239,127,16,0,1,0,0,0,0,0,120,207,24,0,216,181,24,0,48,0,0,0,2,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,224,195,0,0,0,0,0,0,224,67,16,0,1,0,0,0,0,0,232,165,24,0,8,158,26,0,56,0,0,0,2,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,224,195,0,0,0,0,0,0,224,67,16,0,1,0,0,0,0,0,136,132,26,0,192,114,26,0,8,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,239,127,16,0,1,0,0,0,0,0,168,251,25,0,96,241,25,0,32,0,0,0,2,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,224,67,16,0,1,0,0,0,0,0,184,230,25,0,32,222,25,0,40,0,0,0,2,0,0,0,255,255,255,255,255,255,255,127,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,16,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,34,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,120,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,80,26,0,96,0,0,0,240,159,0,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,194,0,0,112,194,0,0,112,194,0,0,112,194,0,0,112,194,0,0,112,194,0,0,112,194,0,0,112,194,0,0,112,194,0,0,112,194,0,0,112,194,0,0,112,194,0,0,120,194,0,0,120,194,0,0,130,194,0,0,146,194,0,0,138,194,0,0,136,194,0,0,136,194,0,0,134,194,0,0,140,194,0,0,140,194,0,0,144,194,0,0,148,194,0,0,150,194,0,0,158,194,0,0,158,194,0,0,160,194,0,0,166,194,0,0,176,194,0,0,186,194,0,0,200,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,64,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,84,194,0,0,116,194,0,0,132,194,0,0,132,194,0,0,136,194,0,0,134,194,0,0,140,194,0,0,152,194,0,0,152,194,0,0,144,194,0,0,146,194,0,0,150,194,0,0,152,194,0,0,156,194,0,0,158,194,0,0,166,194,0,0,176,194,0,0,186,194,0,0,200,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,20,194,0,0,20,194,0,0,20,194,0,0,20,194,0,0,20,194,0,0,20,194,0,0,20,194,0,0,20,194,0,0,24,194,0,0,32,194,0,0,40,194,0,0,56,194,0,0,64,194,0,0,84,194,0,0,92,194,0,0,120,194,0,0,130,194,0,0,104,194,0,0,96,194,0,0,96,194,0,0,116,194,0,0,112,194,0,0,130,194,0,0,134,194,0,0,138,194,0,0,142,194,0,0,154,194,0,0,154,194,0,0,156,194,0,0,160,194,0,0,164,194,0,0,168,194,0,0,176,194,0,0,186,194,0,0,196,194,0,0,212,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,200,193,0,0,200,193,0,0,200,193,0,0,200,193,0,0,200,193,0,0,200,193,0,0,200,193,0,0,200,193,0,0,200,193,0,0,208,193,0,0,216,193,0,0,232,193,0,0,0,194,0,0,24,194,0,0,64,194,0,0,80,194,0,0,80,194,0,0,72,194,0,0,64,194,0,0,64,194,0,0,76,194,0,0,80,194,0,0,88,194,0,0,112,194,0,0,134,194,0,0,134,194,0,0,132,194,0,0,136,194,0,0,138,194,0,0,146,194,0,0,146,194,0,0,152,194,0,0,160,194,0,0,162,194,0,0,162,194,0,0,170,194,0,0,170,194,0,0,172,194,0,0,176,194,0,0,186,194,0,0,200,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,128,193,0,0,128,193,0,0,128,193,0,0,128,193,0,0,128,193,0,0,128,193,0,0,128,193,0,0,128,193,0,0,136,193,0,0,152,193,0,0,160,193,0,0,176,193,0,0,208,193,0,0,224,193,0,0,248,193,0,0,32,194,0,0,60,194,0,0,28,194,0,0,28,194,0,0,32,194,0,0,40,194,0,0,44,194,0,0,60,194,0,0,76,194,0,0,100,194,0,0,80,194,0,0,92,194,0,0,92,194,0,0,112,194,0,0,104,194,0,0,120,194,0,0,124,194,0,0,140,194,0,0,134,194,0,0,138,194,0,0,144,194,0,0,146,194,0,0,154,194,0,0,160,194,0,0,164,194,0,0,166,194,0,0,174,194,0,0,180,194,0,0,188,194,0,0,196,194,0,0,208,194,0,0,230,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,0,193,0,0,0,193,0,0,0,193,0,0,0,193,0,0,0,193,0,0,0,193,0,0,0,193,0,0,0,193,0,0,0,193,0,0,0,193,0,0,32,193,0,0,48,193,0,0,112,193,0,0,152,193,0,0,200,193,0,0,240,193,0,0,8,194,0,0,248,193,0,0,240,193,0,0,248,193,0,0,232,193,0,0,0,194,0,0,12,194,0,0,40,194,0,0,64,194,0,0,40,194,0,0,48,194,0,0,56,194,0,0,72,194,0,0,72,194,0,0,76,194,0,0,80,194,0,0,108,194,0,0,88,194,0,0,92,194,0,0,92,194,0,0,104,194,0,0,120,194,0,0,124,194,0,0,132,194,0,0,144,194,0,0,146,194,0,0,152,194,0,0,150,194,0,0,156,194,0,0,160,194,0,0,160,194,0,0,162,194,0,0,168,194,0,0,176,194,0,0,180,194,0,0,188,194,0,0,196,194,0,0,202,194,0,0,212,194,0,0,220,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,134,194,0,0,134,194,0,0,134,194,0,0,152,194,0,0,144,194,0,0,142,194,0,0,148,194,0,0,152,194,0,0,152,194,0,0,150,194,0,0,156,194,0,0,158,194,0,0,158,194,0,0,162,194,0,0,166,194,0,0,172,194,0,0,178,194,0,0,186,194,0,0,194,194,0,0,200,194,0,0,210,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,60,194,0,0,60,194,0,0,60,194,0,0,60,194,0,0,60,194,0,0,60,194,0,0,60,194,0,0,60,194,0,0,60,194,0,0,60,194,0,0,60,194,0,0,64,194,0,0,76,194,0,0,92,194,0,0,108,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,134,194,0,0,132,194,0,0,136,194,0,0,138,194,0,0,140,194,0,0,148,194,0,0,158,194,0,0,154,194,0,0,154,194,0,0,156,194,0,0,160,194,0,0,162,194,0,0,164,194,0,0,168,194,0,0,172,194,0,0,176,194,0,0,182,194,0,0,190,194,0,0,200,194,0,0,216,194,0,0,232,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,20,194,0,0,20,194,0,0,36,194,0,0,48,194,0,0,64,194,0,0,76,194,0,0,104,194,0,0,120,194,0,0,112,194,0,0,100,194,0,0,108,194,0,0,108,194,0,0,112,194,0,0,124,194,0,0,130,194,0,0,144,194,0,0,142,194,0,0,140,194,0,0,144,194,0,0,148,194,0,0,154,194,0,0,152,194,0,0,156,194,0,0,162,194,0,0,162,194,0,0,160,194,0,0,166,194,0,0,172,194,0,0,182,194,0,0,192,194,0,0,200,194,0,0,210,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,224,193,0,0,224,193,0,0,224,193,0,0,224,193,0,0,224,193,0,0,224,193,0,0,224,193,0,0,224,193,0,0,224,193,0,0,240,193,0,0,0,194,0,0,0,194,0,0,4,194,0,0,12,194,0,0,36,194,0,0,68,194,0,0,72,194,0,0,68,194,0,0,60,194,0,0,64,194,0,0,64,194,0,0,80,194,0,0,76,194,0,0,100,194,0,0,130,194,0,0,116,194,0,0,108,194,0,0,116,194,0,0,128,194,0,0,138,194,0,0,140,194,0,0,148,194,0,0,154,194,0,0,154,194,0,0,156,194,0,0,162,194,0,0,168,194,0,0,170,194,0,0,174,194,0,0,180,194,0,0,184,194,0,0,192,194,0,0,200,194,0,0,214,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,152,193,0,0,152,193,0,0,152,193,0,0,152,193,0,0,152,193,0,0,152,193,0,0,152,193,0,0,152,193,0,0,160,193,0,0,168,193,0,0,184,193,0,0,216,193,0,0,240,193,0,0,12,194,0,0,16,194,0,0,36,194,0,0,56,194,0,0,48,194,0,0,40,194,0,0,32,194,0,0,36,194,0,0,36,194,0,0,44,194,0,0,64,194,0,0,92,194,0,0,84,194,0,0,80,194,0,0,84,194,0,0,96,194,0,0,108,194,0,0,104,194,0,0,112,194,0,0,134,194,0,0,132,194,0,0,138,194,0,0,142,194,0,0,144,194,0,0,150,194,0,0,158,194,0,0,162,194,0,0,168,194,0,0,174,194,0,0,180,194,0,0,186,194,0,0,194,194,0,0,202,194,0,0,214,194,0,0,228,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,16,193,0,0,16,193,0,0,16,193,0,0,16,193,0,0,16,193,0,0,16,193,0,0,16,193,0,0,16,193,0,0,48,193,0,0,64,193,0,0,64,193,0,0,112,193,0,0,128,193,0,0,160,193,0,0,184,193,0,0,240,193,0,0,20,194,0,0,8,194,0,0,4,194,0,0,8,194,0,0,248,193,0,0,0,194,0,0,0,194,0,0,24,194,0,0,60,194,0,0,48,194,0,0,36,194,0,0,32,194,0,0,60,194,0,0,68,194,0,0,56,194,0,0,56,194,0,0,104,194,0,0,72,194,0,0,72,194,0,0,88,194,0,0,104,194,0,0,120,194,0,0,128,194,0,0,134,194,0,0,134,194,0,0,140,194,0,0,144,194,0,0,152,194,0,0,158,194,0,0,166,194,0,0,174,194,0,0,182,194,0,0,192,194,0,0,200,194,0,0,208,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,120,194,0,0,120,194,0,0,120,194,0,0,120,194,0,0,120,194,0,0,120,194,0,0,120,194,0,0,120,194,0,0,120,194,0,0,120,194,0,0,124,194,0,0,128,194,0,0,132,194,0,0,134,194,0,0,132,194,0,0,136,194,0,0,150,194,0,0,144,194,0,0,152,194,0,0,150,194,0,0,152,194,0,0,156,194,0,0,158,194,0,0,164,194,0,0,168,194,0,0,170,194,0,0,180,194,0,0,188,194,0,0,202,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,108,194,0,0,108,194,0,0,108,194,0,0,108,194,0,0,108,194,0,0,108,194,0,0,108,194,0,0,108,194,0,0,108,194,0,0,108,194,0,0,108,194,0,0,112,194,0,0,112,194,0,0,116,194,0,0,124,194,0,0,132,194,0,0,142,194,0,0,136,194,0,0,140,194,0,0,140,194,0,0,142,194,0,0,144,194,0,0,144,194,0,0,150,194,0,0,162,194,0,0,156,194,0,0,158,194,0,0,164,194,0,0,166,194,0,0,172,194,0,0,180,194,0,0,194,194,0,0,206,194,0,0,226,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,84,194,0,0,84,194,0,0,84,194,0,0,84,194,0,0,84,194,0,0,84,194,0,0,84,194,0,0,84,194,0,0,84,194,0,0,88,194,0,0,92,194,0,0,100,194,0,0,96,194,0,0,100,194,0,0,92,194,0,0,116,194,0,0,130,194,0,0,112,194,0,0,112,194,0,0,120,194,0,0,124,194,0,0,124,194,0,0,132,194,0,0,136,194,0,0,148,194,0,0,146,194,0,0,150,194,0,0,150,194,0,0,156,194,0,0,160,194,0,0,160,194,0,0,164,194,0,0,170,194,0,0,180,194,0,0,192,194,0,0,202,194,0,0,216,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,56,194,0,0,56,194,0,0,56,194,0,0,56,194,0,0,56,194,0,0,56,194,0,0,56,194,0,0,56,194,0,0,56,194,0,0,56,194,0,0,60,194,0,0,60,194,0,0,60,194,0,0,60,194,0,0,64,194,0,0,76,194,0,0,100,194,0,0,76,194,0,0,68,194,0,0,72,194,0,0,76,194,0,0,84,194,0,0,88,194,0,0,108,194,0,0,132,194,0,0,112,194,0,0,120,194,0,0,134,194,0,0,134,194,0,0,140,194,0,0,144,194,0,0,150,194,0,0,152,194,0,0,156,194,0,0,162,194,0,0,170,194,0,0,176,194,0,0,188,194,0,0,194,194,0,0,208,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,16,194,0,0,28,194,0,0,36,194,0,0,40,194,0,0,40,194,0,0,28,194,0,0,24,194,0,0,36,194,0,0,44,194,0,0,80,194,0,0,48,194,0,0,32,194,0,0,28,194,0,0,20,194,0,0,20,194,0,0,32,194,0,0,60,194,0,0,88,194,0,0,72,194,0,0,64,194,0,0,72,194,0,0,92,194,0,0,116,194,0,0,108,194,0,0,120,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,138,194,0,0,138,194,0,0,146,194,0,0,148,194,0,0,148,194,0,0,150,194,0,0,154,194,0,0,158,194,0,0,164,194,0,0,174,194,0,0,182,194,0,0,190,194,0,0,200,194,0,0,216,194,0,0,230,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,224,193,0,0,208,193,0,0,192,193,0,0,176,193,0,0,160,193,0,0,160,193,0,0,184,193,0,0,232,193,0,0,240,193,0,0,248,193,0,0,224,193,0,0,216,193,0,0,224,193,0,0,224,193,0,0,224,193,0,0,12,194,0,0,32,194,0,0,4,194,0,0,0,194,0,0,232,193,0,0,240,193,0,0,240,193,0,0,240,193,0,0,20,194,0,0,52,194,0,0,36,194,0,0,20,194,0,0,24,194,0,0,52,194,0,0,60,194,0,0,60,194,0,0,64,194,0,0,84,194,0,0,68,194,0,0,64,194,0,0,72,194,0,0,68,194,0,0,68,194,0,0,76,194,0,0,80,194,0,0,104,194,0,0,96,194,0,0,100,194,0,0,96,194,0,0,112,194,0,0,116,194,0,0,120,194,0,0,140,194,0,0,144,194,0,0,148,194,0,0,156,194,0,0,166,194,0,0,176,194,0,0,186,194,0,0,200,194,0,0,212,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,210,194,0,0,200,194,0,0,190,194,0,0,182,194,0,0,174,194,0,0,166,194,0,0,160,194,0,0,156,194,0,0,152,194,0,0,156,194,0,0,156,194,0,0,162,194,0,0,166,194,0,0,170,194,0,0,172,194,0,0,170,194,0,0,172,194,0,0,174,194,0,0,180,194,0,0,194,194,0,0,214,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,210,194,0,0,200,194,0,0,190,194,0,0,180,194,0,0,170,194,0,0,162,194,0,0,154,194,0,0,146,194,0,0,140,194,0,0,134,194,0,0,134,194,0,0,136,194,0,0,150,194,0,0,146,194,0,0,140,194,0,0,138,194,0,0,140,194,0,0,144,194,0,0,150,194,0,0,158,194,0,0,168,194,0,0,166,194,0,0,168,194,0,0,172,194,0,0,176,194,0,0,178,194,0,0,178,194,0,0,186,194,0,0,196,194,0,0,210,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,200,194,0,0,190,194,0,0,180,194,0,0,170,194,0,0,160,194,0,0,152,194,0,0,142,194,0,0,136,194,0,0,136,194,0,0,130,194,0,0,124,194,0,0,124,194,0,0,120,194,0,0,120,194,0,0,128,194,0,0,130,194,0,0,128,194,0,0,116,194,0,0,120,194,0,0,124,194,0,0,128,194,0,0,132,194,0,0,136,194,0,0,146,194,0,0,146,194,0,0,148,194,0,0,150,194,0,0,152,194,0,0,162,194,0,0,166,194,0,0,170,194,0,0,176,194,0,0,178,194,0,0,184,194,0,0,190,194,0,0,200,194,0,0,216,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,160,194,0,0,150,194,0,0,142,194,0,0,136,194,0,0,130,194,0,0,124,194,0,0,120,194,0,0,116,194,0,0,116,194,0,0,116,194,0,0,116,194,0,0,108,194,0,0,96,194,0,0,100,194,0,0,84,194,0,0,72,194,0,0,104,194,0,0,80,194,0,0,72,194,0,0,72,194,0,0,80,194,0,0,84,194,0,0,88,194,0,0,104,194,0,0,134,194,0,0,124,194,0,0,134,194,0,0,136,194,0,0,144,194,0,0,150,194,0,0,156,194,0,0,160,194,0,0,162,194,0,0,162,194,0,0,164,194,0,0,170,194,0,0,178,194,0,0,180,194,0,0,186,194,0,0,194,194,0,0,202,194,0,0,214,194,0,0,228,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,130,194,0,0,116,194,0,0,108,194,0,0,100,194,0,0,96,194,0,0,92,194,0,0,92,194,0,0,96,194,0,0,96,194,0,0,100,194,0,0,92,194,0,0,84,194,0,0,80,194,0,0,60,194,0,0,48,194,0,0,48,194,0,0,72,194,0,0,48,194,0,0,36,194,0,0,28,194,0,0,28,194,0,0,40,194,0,0,32,194,0,0,56,194,0,0,76,194,0,0,68,194,0,0,72,194,0,0,84,194,0,0,88,194,0,0,124,194,0,0,112,194,0,0,116,194,0,0,120,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,140,194,0,0,146,194,0,0,148,194,0,0,150,194,0,0,152,194,0,0,150,194,0,0,158,194,0,0,170,194,0,0,178,194,0,0,182,194,0,0,192,194,0,0,204,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,80,194,0,0,72,194,0,0,68,194,0,0,68,194,0,0,64,194,0,0,64,194,0,0,64,194,0,0,68,194,0,0,72,194,0,0,72,194,0,0,68,194,0,0,56,194,0,0,44,194,0,0,28,194,0,0,12,194,0,0,4,194,0,0,24,194,0,0,16,194,0,0,0,194,0,0,232,193,0,0,0,194,0,0,0,194,0,0,0,194,0,0,12,194,0,0,48,194,0,0,28,194,0,0,24,194,0,0,24,194,0,0,56,194,0,0,72,194,0,0,52,194,0,0,56,194,0,0,84,194,0,0,72,194,0,0,72,194,0,0,72,194,0,0,88,194,0,0,88,194,0,0,84,194,0,0,84,194,0,0,96,194,0,0,100,194,0,0,108,194,0,0,132,194,0,0,140,194,0,0,144,194,0,0,148,194,0,0,158,194,0,0,166,194,0,0,170,194,0,0,180,194,0,0,194,194,0,0,228,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,210,194,0,0,200,194,0,0,190,194,0,0,180,194,0,0,172,194,0,0,160,194,0,0,150,194,0,0,150,194,0,0,158,194,0,0,160,194,0,0,158,194,0,0,160,194,0,0,162,194,0,0,164,194,0,0,176,194,0,0,190,194,0,0,206,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,216,194,0,0,206,194,0,0,196,194,0,0,186,194,0,0,176,194,0,0,166,194,0,0,158,194,0,0,156,194,0,0,150,194,0,0,142,194,0,0,134,194,0,0,136,194,0,0,146,194,0,0,146,194,0,0,144,194,0,0,146,194,0,0,150,194,0,0,154,194,0,0,160,194,0,0,164,194,0,0,176,194,0,0,186,194,0,0,200,194,0,0,214,194,0,0,228,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,210,194,0,0,202,194,0,0,192,194,0,0,180,194,0,0,172,194,0,0,162,194,0,0,154,194,0,0,146,194,0,0,138,194,0,0,132,194,0,0,116,194,0,0,120,194,0,0,132,194,0,0,128,194,0,0,120,194,0,0,130,194,0,0,132,194,0,0,140,194,0,0,144,194,0,0,152,194,0,0,162,194,0,0,160,194,0,0,168,194,0,0,180,194,0,0,190,194,0,0,204,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,214,194,0,0,206,194,0,0,194,194,0,0,184,194,0,0,176,194,0,0,166,194,0,0,158,194,0,0,148,194,0,0,140,194,0,0,132,194,0,0,108,194,0,0,84,194,0,0,104,194,0,0,120,194,0,0,92,194,0,0,88,194,0,0,88,194,0,0,88,194,0,0,104,194,0,0,116,194,0,0,120,194,0,0,144,194,0,0,140,194,0,0,144,194,0,0,150,194,0,0,156,194,0,0,160,194,0,0,162,194,0,0,160,194,0,0,166,194,0,0,166,194,0,0,176,194,0,0,186,194,0,0,200,194,0,0,214,194,0,0,230,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,200,194,0,0,190,194,0,0,180,194,0,0,170,194,0,0,160,194,0,0,150,194,0,0,140,194,0,0,132,194,0,0,120,194,0,0,96,194,0,0,64,194,0,0,48,194,0,0,64,194,0,0,56,194,0,0,56,194,0,0,44,194,0,0,56,194,0,0,64,194,0,0,64,194,0,0,76,194,0,0,104,194,0,0,104,194,0,0,108,194,0,0,112,194,0,0,120,194,0,0,120,194,0,0,116,194,0,0,116,194,0,0,130,194,0,0,128,194,0,0,130,194,0,0,136,194,0,0,140,194,0,0,148,194,0,0,150,194,0,0,156,194,0,0,162,194,0,0,172,194,0,0,190,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,200,194,0,0,190,194,0,0,180,194,0,0,170,194,0,0,160,194,0,0,150,194,0,0,140,194,0,0,130,194,0,0,116,194,0,0,92,194,0,0,68,194,0,0,28,194,0,0,4,194,0,0,32,194,0,0,12,194,0,0,0,194,0,0,24,194,0,0,32,194,0,0,4,194,0,0,12,194,0,0,20,194,0,0,56,194,0,0,36,194,0,0,52,194,0,0,48,194,0,0,56,194,0,0,40,194,0,0,52,194,0,0,56,194,0,0,80,194,0,0,72,194,0,0,72,194,0,0,72,194,0,0,88,194,0,0,88,194,0,0,92,194,0,0,100,194,0,0,120,194,0,0,128,194,0,0,132,194,0,0,136,194,0,0,140,194,0,0,152,194,0,0,162,194,0,0,180,194,0,0,200,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,196,194,0,0,180,194,0,0,170,194,0,0,164,194,0,0,166,194,0,0,160,194,0,0,156,194,0,0,168,194,0,0,158,194,0,0,160,194,0,0,166,194,0,0,174,194,0,0,178,194,0,0,182,194,0,0,186,194,0,0,198,194,0,0,212,194,0,0,234,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,196,194,0,0,180,194,0,0,170,194,0,0,160,194,0,0,150,194,0,0,140,194,0,0,136,194,0,0,148,194,0,0,144,194,0,0,148,194,0,0,154,194,0,0,160,194,0,0,164,194,0,0,170,194,0,0,174,194,0,0,184,194,0,0,178,194,0,0,182,194,0,0,190,194,0,0,200,194,0,0,212,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,196,194,0,0,180,194,0,0,166,194,0,0,150,194,0,0,142,194,0,0,124,194,0,0,128,194,0,0,134,194,0,0,120,194,0,0,128,194,0,0,134,194,0,0,140,194,0,0,146,194,0,0,154,194,0,0,162,194,0,0,168,194,0,0,166,194,0,0,170,194,0,0,178,194,0,0,180,194,0,0,186,194,0,0,196,194,0,0,208,194,0,0,218,194,0,0,228,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,206,194,0,0,192,194,0,0,176,194,0,0,162,194,0,0,150,194,0,0,136,194,0,0,104,194,0,0,88,194,0,0,96,194,0,0,88,194,0,0,96,194,0,0,96,194,0,0,104,194,0,0,112,194,0,0,124,194,0,0,132,194,0,0,148,194,0,0,138,194,0,0,144,194,0,0,144,194,0,0,150,194,0,0,148,194,0,0,154,194,0,0,162,194,0,0,162,194,0,0,164,194,0,0,168,194,0,0,174,194,0,0,186,194,0,0,192,194,0,0,198,194,0,0,208,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,216,194,0,0,204,194,0,0,192,194,0,0,182,194,0,0,170,194,0,0,160,194,0,0,148,194,0,0,136,194,0,0,112,194,0,0,76,194,0,0,56,194,0,0,64,194,0,0,56,194,0,0,44,194,0,0,52,194,0,0,60,194,0,0,60,194,0,0,68,194,0,0,64,194,0,0,96,194,0,0,84,194,0,0,92,194,0,0,104,194,0,0,100,194,0,0,124,194,0,0,104,194,0,0,112,194,0,0,132,194,0,0,128,194,0,0,134,194,0,0,140,194,0,0,140,194,0,0,148,194,0,0,154,194,0,0,168,194,0,0,172,194,0,0,178,194,0,0,182,194,0,0,186,194,0,0,188,194,0,0,202,194,0,0,218,194,0,0,236,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,216,194,0,0,206,194,0,0,196,194,0,0,186,194,0,0,176,194,0,0,166,194,0,0,156,194,0,0,146,194,0,0,136,194,0,0,112,194,0,0,84,194,0,0,48,194,0,0,12,194,0,0,24,194,0,0,24,194,0,0,8,194,0,0,8,194,0,0,16,194,0,0,32,194,0,0,36,194,0,0,48,194,0,0,76,194,0,0,52,194,0,0,56,194,0,0,60,194,0,0,56,194,0,0,88,194,0,0,72,194,0,0,68,194,0,0,72,194,0,0,72,194,0,0,72,194,0,0,76,194,0,0,88,194,0,0,100,194,0,0,104,194,0,0,112,194,0,0,132,194,0,0,132,194,0,0,132,194,0,0,128,194,0,0,130,194,0,0,136,194,0,0,154,194,0,0,164,194,0,0,174,194,0,0,190,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,214,194,0,0,204,194,0,0,194,194,0,0,184,194,0,0,174,194,0,0,166,194,0,0,156,194,0,0,150,194,0,0,164,194,0,0,158,194,0,0,166,194,0,0,170,194,0,0,178,194,0,0,184,194,0,0,190,194,0,0,196,194,0,0,202,194,0,0,210,194,0,0,218,194,0,0,226,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,212,194,0,0,200,194,0,0,190,194,0,0,180,194,0,0,172,194,0,0,162,194,0,0,156,194,0,0,148,194,0,0,138,194,0,0,148,194,0,0,148,194,0,0,152,194,0,0,158,194,0,0,166,194,0,0,168,194,0,0,172,194,0,0,178,194,0,0,184,194,0,0,194,194,0,0,186,194,0,0,200,194,0,0,206,194,0,0,214,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,212,194,0,0,200,194,0,0,190,194,0,0,180,194,0,0,174,194,0,0,166,194,0,0,160,194,0,0,150,194,0,0,138,194,0,0,112,194,0,0,132,194,0,0,132,194,0,0,136,194,0,0,140,194,0,0,148,194,0,0,156,194,0,0,158,194,0,0,162,194,0,0,162,194,0,0,166,194,0,0,168,194,0,0,174,194,0,0,186,194,0,0,192,194,0,0,198,194,0,0,206,194,0,0,214,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,216,194,0,0,206,194,0,0,196,194,0,0,186,194,0,0,178,194,0,0,170,194,0,0,164,194,0,0,156,194,0,0,142,194,0,0,120,194,0,0,92,194,0,0,104,194,0,0,104,194,0,0,88,194,0,0,88,194,0,0,92,194,0,0,108,194,0,0,116,194,0,0,120,194,0,0,140,194,0,0,132,194,0,0,132,194,0,0,134,194,0,0,140,194,0,0,144,194,0,0,150,194,0,0,156,194,0,0,168,194,0,0,168,194,0,0,168,194,0,0,176,194,0,0,182,194,0,0,180,194,0,0,190,194,0,0,196,194,0,0,204,194,0,0,206,194,0,0,212,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,216,194,0,0,206,194,0,0,196,194,0,0,188,194,0,0,180,194,0,0,174,194,0,0,164,194,0,0,158,194,0,0,146,194,0,0,134,194,0,0,104,194,0,0,60,194,0,0,72,194,0,0,52,194,0,0,36,194,0,0,52,194,0,0,64,194,0,0,48,194,0,0,48,194,0,0,68,194,0,0,88,194,0,0,76,194,0,0,64,194,0,0,60,194,0,0,68,194,0,0,72,194,0,0,76,194,0,0,100,194,0,0,104,194,0,0,112,194,0,0,124,194,0,0,138,194,0,0,140,194,0,0,138,194,0,0,142,194,0,0,148,194,0,0,156,194,0,0,164,194,0,0,180,194,0,0,190,194,0,0,202,194,0,0,210,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,202,194,0,0,194,194,0,0,186,194,0,0,180,194,0,0,170,194,0,0,160,194,0,0,154,194,0,0,144,194,0,0,130,194,0,0,96,194,0,0,64,194,0,0,20,194,0,0,32,194,0,0,16,194,0,0,8,194,0,0,32,194,0,0,72,194,0,0,60,194,0,0,24,194,0,0,36,194,0,0,60,194,0,0,24,194,0,0,12,194,0,0,28,194,0,0,24,194,0,0,44,194,0,0,32,194,0,0,52,194,0,0,72,194,0,0,52,194,0,0,48,194,0,0,60,194,0,0,72,194,0,0,92,194,0,0,64,194,0,0,64,194,0,0,80,194,0,0,132,194,0,0,140,194,0,0,152,194,0,0,164,194,0,0,180,194,0,0,194,194,0,0,210,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,216,194,0,0,206,194,0,0,196,194,0,0,186,194,0,0,172,194,0,0,158,194,0,0,152,194,0,0,166,194,0,0,162,194,0,0,170,194,0,0,174,194,0,0,178,194,0,0,186,194,0,0,196,194,0,0,204,194,0,0,214,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,216,194,0,0,206,194,0,0,196,194,0,0,186,194,0,0,172,194,0,0,158,194,0,0,142,194,0,0,154,194,0,0,148,194,0,0,154,194,0,0,158,194])
.concat([0,0,162,194,0,0,168,194,0,0,170,194,0,0,180,194,0,0,184,194,0,0,186,194,0,0,184,194,0,0,196,194,0,0,202,194,0,0,216,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,216,194,0,0,206,194,0,0,196,194,0,0,186,194,0,0,174,194,0,0,156,194,0,0,136,194,0,0,130,194,0,0,132,194,0,0,120,194,0,0,130,194,0,0,134,194,0,0,140,194,0,0,146,194,0,0,150,194,0,0,156,194,0,0,164,194,0,0,164,194,0,0,166,194,0,0,168,194,0,0,182,194,0,0,186,194,0,0,196,194,0,0,204,194,0,0,212,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,200,194,0,0,190,194,0,0,180,194,0,0,164,194,0,0,148,194,0,0,120,194,0,0,100,194,0,0,104,194,0,0,96,194,0,0,76,194,0,0,80,194,0,0,80,194,0,0,88,194,0,0,88,194,0,0,104,194,0,0,132,194,0,0,108,194,0,0,112,194,0,0,124,194,0,0,132,194,0,0,138,194,0,0,146,194,0,0,158,194,0,0,166,194,0,0,168,194,0,0,160,194,0,0,162,194,0,0,162,194,0,0,164,194,0,0,176,194,0,0,184,194,0,0,196,194,0,0,210,194,0,0,226,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,214,194,0,0,204,194,0,0,194,194,0,0,184,194,0,0,168,194,0,0,158,194,0,0,138,194,0,0,100,194,0,0,60,194,0,0,80,194,0,0,60,194,0,0,48,194,0,0,52,194,0,0,72,194,0,0,80,194,0,0,40,194,0,0,40,194,0,0,84,194,0,0,44,194,0,0,44,194,0,0,64,194,0,0,76,194,0,0,96,194,0,0,92,194,0,0,80,194,0,0,100,194,0,0,108,194,0,0,116,194,0,0,120,194,0,0,134,194,0,0,142,194,0,0,156,194,0,0,166,194,0,0,172,194,0,0,188,194,0,0,196,194,0,0,206,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,200,194,0,0,190,194,0,0,180,194,0,0,168,194,0,0,156,194,0,0,140,194,0,0,116,194,0,0,76,194,0,0,36,194,0,0,32,194,0,0,24,194,0,0,32,194,0,0,56,194,0,0,80,194,0,0,76,194,0,0,36,194,0,0,32,194,0,0,56,194,0,0,32,194,0,0,24,194,0,0,24,194,0,0,36,194,0,0,56,194,0,0,36,194,0,0,56,194,0,0,60,194,0,0,44,194,0,0,44,194,0,0,52,194,0,0,36,194,0,0,52,194,0,0,96,194,0,0,134,194,0,0,136,194,0,0,166,194,0,0,174,194,0,0,180,194,0,0,190,194,0,0,204,194,0,0,214,194,0,0,226,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,218,194,0,0,210,194,0,0,202,194,0,0,192,194,0,0,182,194,0,0,168,194,0,0,154,194,0,0,164,194,0,0,164,194,0,0,170,194,0,0,178,194,0,0,188,194,0,0,200,194,0,0,212,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,212,194,0,0,206,194,0,0,196,194,0,0,184,194,0,0,170,194,0,0,160,194,0,0,142,194,0,0,150,194,0,0,144,194,0,0,152,194,0,0,160,194,0,0,168,194,0,0,172,194,0,0,178,194,0,0,186,194,0,0,200,194,0,0,214,194,0,0,226,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,214,194,0,0,208,194,0,0,202,194,0,0,194,194,0,0,184,194,0,0,176,194,0,0,168,194,0,0,160,194,0,0,128,194,0,0,132,194,0,0,124,194,0,0,128,194,0,0,132,194,0,0,138,194,0,0,146,194,0,0,154,194,0,0,166,194,0,0,166,194,0,0,172,194,0,0,182,194,0,0,196,194,0,0,208,194,0,0,222,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,214,194,0,0,208,194,0,0,202,194,0,0,194,194,0,0,184,194,0,0,180,194,0,0,168,194,0,0,148,194,0,0,100,194,0,0,104,194,0,0,80,194,0,0,92,194,0,0,88,194,0,0,72,194,0,0,80,194,0,0,72,194,0,0,80,194,0,0,124,194,0,0,120,194,0,0,138,194,0,0,152,194,0,0,154,194,0,0,156,194,0,0,156,194,0,0,158,194,0,0,164,194,0,0,176,194,0,0,188,194,0,0,200,194,0,0,212,194,0,0,222,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,212,194,0,0,204,194,0,0,196,194,0,0,190,194,0,0,180,194,0,0,170,194,0,0,166,194,0,0,156,194,0,0,140,194,0,0,72,194,0,0,72,194,0,0,36,194,0,0,48,194,0,0,68,194,0,0,60,194,0,0,72,194,0,0,72,194,0,0,48,194,0,0,92,194,0,0,56,194,0,0,60,194,0,0,64,194,0,0,64,194,0,0,88,194,0,0,68,194,0,0,68,194,0,0,104,194,0,0,120,194,0,0,142,194,0,0,162,194,0,0,174,194,0,0,184,194,0,0,194,194,0,0,204,194,0,0,216,194,0,0,228,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,212,194,0,0,204,194,0,0,196,194,0,0,190,194,0,0,180,194,0,0,170,194,0,0,166,194,0,0,156,194,0,0,140,194,0,0,52,194,0,0,44,194,0,0,36,194,0,0,60,194,0,0,72,194,0,0,76,194,0,0,72,194,0,0,68,194,0,0,52,194,0,0,60,194,0,0,36,194,0,0,48,194,0,0,36,194,0,0,28,194,0,0,44,194,0,0,24,194,0,0,20,194,0,0,32,194,0,0,36,194,0,0,48,194,0,0,72,194,0,0,104,194,0,0,130,194,0,0,146,194,0,0,158,194,0,0,170,194,0,0,184,194,0,0,194,194,0,0,202,194,0,0,210,194,0,0,218,194,0,0,226,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,214,194,0,0,200,194,0,0,190,194,0,0,174,194,0,0,162,194,0,0,170,194,0,0,166,194,0,0,176,194,0,0,186,194,0,0,200,194,0,0,214,194,0,0,228,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,214,194,0,0,202,194,0,0,190,194,0,0,176,194,0,0,166,194,0,0,152,194,0,0,146,194,0,0,144,194,0,0,158,194,0,0,168,194,0,0,180,194,0,0,190,194,0,0,200,194,0,0,210,194,0,0,220,194,0,0,230,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,208,194,0,0,196,194,0,0,184,194,0,0,174,194,0,0,162,194,0,0,140,194,0,0,130,194,0,0,120,194,0,0,134,194,0,0,142,194,0,0,148,194,0,0,160,194,0,0,170,194,0,0,182,194,0,0,190,194,0,0,198,194,0,0,206,194,0,0,216,194,0,0,222,194,0,0,228,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,206,194,0,0,194,194,0,0,180,194,0,0,170,194,0,0,152,194,0,0,112,194,0,0,96,194,0,0,88,194,0,0,112,194,0,0,120,194,0,0,116,194,0,0,96,194,0,0,124,194,0,0,130,194,0,0,146,194,0,0,148,194,0,0,154,194,0,0,150,194,0,0,156,194,0,0,162,194,0,0,172,194,0,0,174,194,0,0,176,194,0,0,182,194,0,0,188,194,0,0,196,194,0,0,206,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,200,194,0,0,194,194,0,0,184,194,0,0,172,194,0,0,162,194,0,0,158,194,0,0,140,194,0,0,100,194,0,0,76,194,0,0,60,194,0,0,76,194,0,0,104,194,0,0,112,194,0,0,96,194,0,0,84,194,0,0,72,194,0,0,104,194,0,0,80,194,0,0,72,194,0,0,72,194,0,0,84,194,0,0,92,194,0,0,128,194,0,0,138,194,0,0,142,194,0,0,170,194,0,0,164,194,0,0,156,194,0,0,162,194,0,0,170,194,0,0,190,194,0,0,204,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,200,194,0,0,194,194,0,0,184,194,0,0,170,194,0,0,166,194,0,0,158,194,0,0,144,194,0,0,68,194,0,0,32,194,0,0,44,194,0,0,44,194,0,0,88,194,0,0,96,194,0,0,76,194,0,0,72,194,0,0,32,194,0,0,44,194,0,0,24,194,0,0,16,194,0,0,12,194,0,0,20,194,0,0,24,194,0,0,20,194,0,0,48,194,0,0,88,194,0,0,112,194,0,0,100,194,0,0,112,194,0,0,140,194,0,0,150,194,0,0,168,194,0,0,184,194,0,0,206,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,204,194,0,0,190,194,0,0,178,194,0,0,164,194,0,0,166,194,0,0,168,194,0,0,180,194,0,0,184,194,0,0,198,194,0,0,214,194,0,0,226,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,214,194,0,0,202,194,0,0,190,194,0,0,178,194,0,0,166,194,0,0,144,194,0,0,148,194,0,0,156,194,0,0,170,194,0,0,176,194,0,0,176,194,0,0,180,194,0,0,184,194,0,0,196,194,0,0,210,194,0,0,222,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,218,194,0,0,206,194,0,0,194,194,0,0,186,194,0,0,174,194,0,0,162,194,0,0,140,194,0,0,140,194,0,0,134,194,0,0,150,194,0,0,146,194,0,0,152,194,0,0,158,194,0,0,162,194,0,0,166,194,0,0,176,194,0,0,178,194,0,0,194,194,0,0,206,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,214,194,0,0,200,194,0,0,188,194,0,0,176,194,0,0,166,194,0,0,150,194,0,0,124,194,0,0,108,194,0,0,108,194,0,0,124,194,0,0,132,194,0,0,112,194,0,0,120,194,0,0,134,194,0,0,134,194,0,0,154,194,0,0,152,194,0,0,162,194,0,0,176,194,0,0,172,194,0,0,184,194,0,0,192,194,0,0,204,194,0,0,218,194,0,0,232,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,196,194,0,0,184,194,0,0,172,194,0,0,162,194,0,0,146,194,0,0,96,194,0,0,80,194,0,0,60,194,0,0,92,194,0,0,112,194,0,0,104,194,0,0,80,194,0,0,76,194,0,0,52,194,0,0,68,194,0,0,72,194,0,0,84,194,0,0,88,194,0,0,116,194,0,0,142,194,0,0,140,194,0,0,138,194,0,0,156,194,0,0,158,194,0,0,174,194,0,0,180,194,0,0,192,194,0,0,208,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,206,194,0,0,192,194,0,0,180,194,0,0,172,194,0,0,156,194,0,0,140,194,0,0,76,194,0,0,40,194,0,0,60,194,0,0,64,194,0,0,92,194,0,0,88,194,0,0,88,194,0,0,84,194,0,0,40,194,0,0,12,194,0,0,224,193,0,0,4,194,0,0,24,194,0,0,20,194,0,0,48,194,0,0,60,194,0,0,68,194,0,0,88,194,0,0,124,194,0,0,136,194,0,0,156,194,0,0,164,194,0,0,178,194,0,0,188,194,0,0,198,194,0,0,208,194,0,0,218,194,0,0,228,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,200,194,0,0,180,194,0,0,158,194,0,0,170,194,0,0,162,194,0,0,164,194,0,0,164,194,0,0,178,194,0,0,188,194,0,0,198,194,0,0,206,194,0,0,218,194,0,0,230,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,194,194,0,0,170,194,0,0,144,194,0,0,148,194,0,0,140,194,0,0,140,194,0,0,140,194,0,0,152,194,0,0,170,194,0,0,182,194,0,0,186,194,0,0,194,194,0,0,206,194,0,0,218,194,0,0,230,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,224,194,0,0,186,194,0,0,162,194,0,0,136,194,0,0,120,194,0,0,112,194,0,0,112,194,0,0,100,194,0,0,124,194,0,0,140,194,0,0,154,194,0,0,164,194,0,0,180,194,0,0,186,194,0,0,196,194,0,0,208,194,0,0,218,194,0,0,226,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,226,194,0,0,200,194,0,0,186,194,0,0,168,194,0,0,124,194,0,0,104,194,0,0,64,194,0,0,84,194,0,0,88,194,0,0,80,194,0,0,80,194,0,0,100,194,0,0,128,194,0,0,132,194,0,0,152,194,0,0,166,194,0,0,162,194,0,0,170,194,0,0,170,194,0,0,180,194,0,0,190,194,0,0,196,194,0,0,202,194,0,0,206,194,0,0,212,194,0,0,216,194,0,0,222,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,210,194,0,0,190,194,0,0,172,194,0,0,148,194,0,0,84,194,0,0,72,194,0,0,24,194,0,0,44,194,0,0,68,194,0,0,44,194,0,0,40,194,0,0,28,194,0,0,28,194,0,0,56,194,0,0,80,194,0,0,100,194,0,0,96,194,0,0,144,194,0,0,138,194,0,0,148,194,0,0,162,194,0,0,174,194,0,0,184,194,0,0,188,194,0,0,194,194,0,0,198,194,0,0,204,194,0,0,210,194,0,0,216,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,216,194,0,0,198,194,0,0,180,194,0,0,152,194,0,0,132,194,0,0,52,194,0,0,44,194,0,0,36,194,0,0,48,194,0,0,60,194,0,0,44,194,0,0,60,194,0,0,32,194,0,0,240,193,0,0,248,193,0,0,248,193,0,0,28,194,0,0,4,194,0,0,32,194,0,0,36,194,0,0,44,194,0,0,84,194,0,0,108,194,0,0,140,194,0,0,146,194,0,0,154,194,0,0,158,194,0,0,164,194,0,0,168,194,0,0,174,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,182,194,0,0,152,194,0,0,150,194,0,0,170,194,0,0,186,194,0,0,196,194,0,0,208,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,182,194,0,0,140,194,0,0,140,194,0,0,150,194,0,0,172,194,0,0,178,194,0,0,188,194,0,0,196,194,0,0,202,194,0,0,212,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,190,194,0,0,160,194,0,0,112,194,0,0,130,194,0,0,128,194,0,0,148,194,0,0,166,194,0,0,176,194,0,0,182,194,0,0,190,194,0,0,198,194,0,0,206,194,0,0,214,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,190,194,0,0,160,194,0,0,104,194,0,0,92,194,0,0,68,194,0,0,132,194,0,0,136,194,0,0,142,194,0,0,156,194,0,0,156,194,0,0,160,194,0,0,176,194,0,0,170,194,0,0,178,194,0,0,194,194,0,0,200,194,0,0,210,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,190,194,0,0,160,194,0,0,84,194,0,0,80,194,0,0,36,194,0,0,108,194,0,0,108,194,0,0,68,194,0,0,104,194,0,0,96,194,0,0,124,194,0,0,172,194,0,0,158,194,0,0,180,194,0,0,186,194,0,0,196,194,0,0,206,194,0,0,214,194,0,0,224,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,194,194,0,0,182,194,0,0,146,194,0,0,52,194,0,0,32,194,0,0,4,194,0,0,84,194,0,0,116,194,0,0,68,194,0,0,88,194,0,0,72,194,0,0,72,194,0,0,112,194,0,0,80,194,0,0,134,194,0,0,148,194,0,0,162,194,0,0,184,194,0,0,192,194,0,0,200,194,0,0,210,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,226,194,0,0,212,194,0,0,198,194,0,0,184,194,0,0,154,194,0,0,160,194,0,0,176,194,0,0,194,194,0,0,212,194,0,0,230,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,232,194,0,0,218,194,0,0,204,194,0,0,190,194,0,0,178,194,0,0,148,194,0,0,144,194,0,0,176,194,0,0,174,194,0,0,190,194,0,0,204,194,0,0,218,194,0,0,232,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,232,194,0,0,218,194,0,0,204,194,0,0,190,194,0,0,178,194,0,0,150,194,0,0,132,194,0,0,148,194,0,0,154,194,0,0,156,194,0,0,172,194,0,0,174,194,0,0,180,194,0,0,192,194,0,0,210,194,0,0,230,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,230,194,0,0,216,194,0,0,202,194,0,0,188,194,0,0,176,194,0,0,132,194,0,0,96,194,0,0,116,194,0,0,140,194,0,0,130,194,0,0,156,194,0,0,144,194,0,0,166,194,0,0,168,194,0,0,186,194,0,0,196,194,0,0,210,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,210,194,0,0,190,194,0,0,178,194,0,0,164,194,0,0,100,194,0,0,80,194,0,0,80,194,0,0,108,194,0,0,96,194,0,0,108,194,0,0,104,194,0,0,138,194,0,0,134,194,0,0,176,194,0,0,164,194,0,0,164,194,0,0,178,194,0,0,188,194,0,0,200,194,0,0,216,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,202,194,0,0,192,194,0,0,180,194,0,0,166,194,0,0,154,194,0,0,88,194,0,0,44,194,0,0,24,194,0,0,72,194,0,0,64,194,0,0,80,194,0,0,64,194,0,0,40,194,0,0,40,194,0,0,76,194,0,0,80,194,0,0,84,194,0,0,108,194,0,0,130,194,0,0,142,194,0,0,156,194,0,0,170,194,0,0,190,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,240,194,0,0,210,194,0,0,172,194,0,0,136,194,0,0,156,194,0,0,158,194,0,0,180,194,0,0,200,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,240,194,0,0,210,194,0,0,172,194,0,0,132,194,0,0,146,194,0,0,154,194,0,0,176,194,0,0,192,194,0,0,210,194,0,0,230,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,240,194,0,0,210,194,0,0,184,194,0,0,160,194,0,0,116,194,0,0,128,194,0,0,136,194,0,0,160,194,0,0,174,194,0,0,184,194,0,0,200,194,0,0,220,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,240,194,0,0,208,194,0,0,182,194,0,0,158,194,0,0,80,194,0,0,112,194,0,0,88,194,0,0,128,194,0,0,138,194,0,0,154,194,0,0,160,194,0,0,164,194,0,0,168,194,0,0,170,194,0,0,174,194,0,0,176,194,0,0,180,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,236,194,0,0,200,194,0,0,174,194,0,0,154,194,0,0,68,194,0,0,72,194,0,0,48,194,0,0,104,194,0,0,116,194,0,0,116,194,0,0,134,194,0,0,130,194,0,0,120,194,0,0,120,194,0,0,120,194,0,0,130,194,0,0,136,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196])
.concat([0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,230,194,0,0,196,194,0,0,168,194,0,0,120,194,0,0,68,194,0,0,48,194,0,0,24,194,0,0,56,194,0,0,68,194,0,0,68,194,0,0,56,194,0,0,28,194,0,0,20,194,0,0,28,194,0,0,32,194,0,0,40,194,0,0,44,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,176,194,0,0,148,194,0,0,154,194,0,0,164,194,0,0,164,194,0,0,170,194,0,0,180,194,0,0,188,194,0,0,198,194,0,0,208,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,176,194,0,0,132,194,0,0,140,194,0,0,162,194,0,0,160,194,0,0,162,194,0,0,168,194,0,0,176,194,0,0,182,194,0,0,186,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,176,194,0,0,116,194,0,0,124,194,0,0,140,194,0,0,142,194,0,0,148,194,0,0,154,194,0,0,160,194,0,0,166,194,0,0,170,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,172,194,0,0,120,194,0,0,124,194,0,0,120,194,0,0,120,194,0,0,104,194,0,0,80,194,0,0,72,194,0,0,72,194,0,0,80,194,0,0,88,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,236,194,0,0,216,194,0,0,168,194,0,0,84,194,0,0,72,194,0,0,72,194,0,0,72,194,0,0,92,194,0,0,60,194,0,0,52,194,0,0,32,194,0,0,32,194,0,0,32,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,236,194,0,0,200,194,0,0,146,194,0,0,44,194,0,0,20,194,0,0,40,194,0,0,44,194,0,0,84,194,0,0,24,194,0,0,20,194,0,0,12,194,0,0,12,194,0,0,24,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,200,194,0,0,182,194,0,0,168,194,0,0,148,194,0,0,160,194,0,0,160,194,0,0,160,194,0,0,160,194,0,0,160,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,200,194,0,0,182,194,0,0,168,194,0,0,148,194,0,0,136,194,0,0,136,194,0,0,136,194,0,0,136,194,0,0,136,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,200,194,0,0,172,194,0,0,156,194,0,0,140,194,0,0,112,194,0,0,52,194,0,0,240,193,0,0,168,193,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,200,194,0,0,174,194,0,0,156,194,0,0,134,194,0,0,64,194,0,0,24,194,0,0,232,193,0,0,168,193,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,200,194,0,0,172,194,0,0,138,194,0,0,96,194,0,0,52,194,0,0,12,194,0,0,4,194,0,0,232,193,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,220,194,0,0,200,194,0,0,166,194,0,0,142,194,0,0,64,194,0,0,216,193,0,0,24,194,0,0,20,194,0,0,8,194,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,192,121,196,0,0,0,0,206,230,84,190,201,63,208,190,24,121,22,191,81,119,44,63,66,196,6,63,171,244,184,62,213,40,60,62,0,0,0,0,2,38,222,190,186,141,36,63,68,29,175,62,0,0,0,0,206,230,84,190,201,63,208,190,24,121,22,191,189,62,62,191,215,179,93,191,113,120,115,191,253,152,126,191,111,232,126,63,52,58,118,63,108,41,101,63,219,74,76,63,81,119,44,63,66,196,6,63,171,244,184,62,213,40,60,62,0,0,0,0,2,38,222,190,28,38,72,191,224,148,121,191,92,28,124,63,215,179,93,63,186,141,36,63,68,29,175,62,200,251,0,0,136,251,0,0,120,251,0,0,88,251,0,0,0,1,1,2,3,3,4,5,0,0,0,0,0,0,0,0,3,0,1,0,4,0,4,0,6,0,7,0,8,0,8,0,4,0,4,0,4,0,5,0,6,0,8,0,7,0,9,0,5,0,7,0,6,0,8,0,7,0,9,0,8,0,10,0,7,0,8,0,7,0,8,0,8,0,9,0,9,0,10,0,2,0,1,0,3,0,4,0,7,0,7,0,4,0,4,0,4,0,5,0,7,0,7,0,6,0,6,0,7,0,7,0,8,0,8,0,0,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,128,63,54,89,75,63,152,134,33,63,152,134,33,63,152,134,33,63,152,134,33,63,152,134,33,63,250,155,128,62,153,158,240,61,0,0,0,0,236,237,238,239,240,241,242,243,244,245,28,2,0,6,58,246,247,248,249,250,251,252,253,254,255,0,0,0,0,0,0,0,8,8,8,8,8,8,8,8,8,8,5,2,1,3,6,8,8,8,8,8,8,8,8,8,8,0,0,0,0,0,0,0,206,31,207,31,208,31,209,31,210,31,211,31,212,31,213,31,214,31,215,31,216,31,217,31,218,31,219,31,220,31,221,31,222,31,223,31,224,31,225,31,226,31,227,31,228,31,229,31,230,31,231,31,242,7,253,0,62,0,14,0,6,0,0,0,2,0,30,0,252,0,248,3,204,31,232,31,233,31,234,31,235,31,236,31,205,31,237,31,238,31,239,31,240,31,241,31,242,31,243,31,244,31,245,31,246,31,247,31,248,31,249,31,250,31,251,31,252,31,253,31,254,31,254,63,255,63,0,0,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,11,8,6,4,3,1,2,5,8,10,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,14,14,0,242,31,243,31,244,31,245,31,246,31,247,31,248,31,248,15,254,0,126,0,14,0,6,0,0,0,2,0,30,0,62,0,254,1,249,31,250,31,251,31,252,31,253,31,254,31,254,63,255,63,0,0,0,0,0,0,13,13,13,13,13,13,13,12,8,7,4,3,1,2,5,6,9,13,13,13,13,13,13,14,14,0,0,0,0,0,0,0,228,255,0,0,229,255,0,0,230,255,0,0,231,255,0,0,232,255,0,0,233,255,0,0,234,255,0,0,235,255,0,0,236,255,0,0,237,255,0,0,238,255,0,0,239,255,0,0,240,255,0,0,241,255,0,0,242,255,0,0,243,255,0,0,244,255,0,0,226,255,0,0,252,15,0,0,252,7,0,0,254,1,0,0,126,0,0,0,30,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,14,0,0,0,62,0,0,0,254,0,0,0,253,7,0,0,253,15,0,0,240,127,0,0,227,255,0,0,245,255,0,0,246,255,0,0,247,255,0,0,248,255,0,0,249,255,0,0,250,255,0,0,246,255,1,0,247,255,1,0,248,255,1,0,249,255,1,0,250,255,1,0,251,255,1,0,252,255,1,0,253,255,1,0,254,255,1,0,255,255,1,0,0,0,0,0,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,12,11,9,7,5,3,1,2,4,6,8,11,12,15,16,16,16,16,16,16,16,17,17,17,17,17,17,17,17,17,17,0,0,0,0,0,0,0,237,255,3,0,238,255,3,0,222,255,7,0,223,255,7,0,224,255,7,0,225,255,7,0,226,255,7,0,227,255,7,0,228,255,7,0,229,255,7,0,230,255,7,0,231,255,7,0,232,255,7,0,233,255,7,0,234,255,7,0,235,255,7,0,236,255,7,0,244,255,1,0,247,255,0,0,249,255,0,0,248,255,0,0,251,63,0,0,250,63,0,0,248,63,0,0,250,31,0,0,252,15,0,0,252,7,0,0,254,0,0,0,62,0,0,0,14,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,30,0,0,0,126,0,0,0,254,1,0,0,253,7,0,0,251,31,0,0,249,63,0,0,252,63,0,0,250,127,0,0,246,255,0,0,245,255,1,0,236,255,3,0,237,255,7,0,238,255,7,0,239,255,7,0,240,255,7,0,241,255,7,0,242,255,7,0,243,255,7,0,244,255,7,0,245,255,7,0,246,255,7,0,247,255,7,0,248,255,7,0,249,255,7,0,250,255,7,0,251,255,7,0,252,255,7,0,253,255,7,0,254,255,7,0,255,255,7,0,0,0,0,0,18,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,17,16,16,16,14,14,14,13,12,11,8,6,4,2,1,3,5,7,9,11,13,14,14,15,16,17,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,0,214,255,3,0,215,255,3,0,216,255,3,0,217,255,3,0,218,255,3,0,219,255,3,0,184,255,7,0,185,255,7,0,186,255,7,0,187,255,7,0,188,255,7,0,189,255,7,0,190,255,7,0,191,255,7,0,192,255,7,0,193,255,7,0,194,255,7,0,195,255,7,0,196,255,7,0,197,255,7,0,198,255,7,0,199,255,7,0,200,255,7,0,201,255,7,0,202,255,7,0,203,255,7,0,204,255,7,0,205,255,7,0,206,255,7,0,207,255,7,0,208,255,7,0,209,255,7,0,210,255,7,0,211,255,7,0,230,255,1,0,212,255,3,0,240,255,0,0,233,255,1,0,213,255,3,0,231,255,1,0,241,255,0,0,236,255,0,0,237,255,0,0,238,255,0,0,244,127,0,0,249,63,0,0,247,63,0,0,250,31,0,0,249,31,0,0,251,15,0,0,252,7,0,0,252,3,0,0,253,1,0,0,253,0,0,0,125,0,0,0,61,0,0,0,29,0,0,0,13,0,0,0,5,0,0,0,1,0,0,0,0,0,0,0,4,0,0,0,12,0,0,0,28,0,0,0,60,0,0,0,124,0,0,0,252,0,0,0,252,1,0,0,253,3,0,0,250,15,0,0,248,31,0,0,246,63,0,0,248,63,0,0,245,127,0,0,239,255,0,0,232,255,1,0,242,255,0,0,212,255,7,0,213,255,7,0,214,255,7,0,215,255,7,0,216,255,7,0,217,255,7,0,218,255,7,0,219,255,7,0,220,255,7,0,221,255,7,0,222,255,7,0,223,255,7,0,224,255,7,0,225,255,7,0,226,255,7,0,227,255,7,0,228,255,7,0,229,255,7,0,230,255,7,0,231,255,7,0,232,255,7,0,233,255,7,0,234,255,7,0,235,255,7,0,236,255,7,0,237,255,7,0,238,255,7,0,239,255,7,0,240,255,7,0,241,255,7,0,242,255,7,0,243,255,7,0,244,255,7,0,245,255,7,0,246,255,7,0,247,255,7,0,248,255,7,0,249,255,7,0,250,255,7,0,251,255,7,0,252,255,7,0,253,255,7,0,254,255,7,0,255,255,7,0,0,0,0,0,18,18,18,18,18,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,17,18,16,17,18,17,16,16,16,16,15,14,14,13,13,12,11,10,9,8,7,6,5,4,3,2,2,3,4,5,6,7,8,9,10,12,13,14,14,15,16,17,16,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,0,0,0,0,0,0,0,3,4,6,7,9,10,4,5,6,7,8,10,5,6,7,8,9,10,7,7,8,9,9,10,8,8,9,9,10,11,9,9,10,10,11,11,0,0,0,0,7,0,5,0,9,0,14,0,15,0,7,0,6,0,4,0,5,0,5,0,6,0,7,0,7,0,6,0,8,0,8,0,8,0,5,0,15,0,6,0,9,0,10,0,5,0,1,0,11,0,7,0,9,0,6,0,4,0,1,0,14,0,4,0,6,0,2,0,6,0,0,0,2,4,7,9,9,10,4,4,6,10,10,10,7,6,8,10,10,11,9,10,10,11,11,12,9,9,10,11,12,12,10,10,11,11,13,13,0,0,0,0,3,0,4,0,6,0,18,0,12,0,5,0,5,0,1,0,2,0,16,0,9,0,3,0,7,0,3,0,5,0,14,0,7,0,3,0,19,0,17,0,15,0,13,0,10,0,4,0,13,0,5,0,8,0,11,0,5,0,1,0,12,0,4,0,4,0,1,0,1,0,0,0,1,4,7,9,9,10,4,6,8,9,9,10,7,7,9,10,10,11,8,9,10,11,11,11,8,9,10,11,11,12,9,10,11,12,12,12,0,0,0,0,1,0,2,0,10,0,19,0,16,0,10,0,3,0,3,0,7,0,10,0,5,0,3,0,11,0,4,0,13,0,17,0,8,0,4,0,12,0,11,0,18,0,15,0,11,0,2,0,7,0,6,0,9,0,14,0,3,0,1,0,6,0,4,0,5,0,3,0,2,0,0,0,3,4,6,8,4,4,6,7,5,6,7,8,7,7,8,9,7,0,3,0,5,0,1,0,6,0,2,0,3,0,2,0,5,0,4,0,4,0,1,0,3,0,3,0,2,0,0,0,1,4,7,8,4,5,8,9,7,8,9,10,8,8,9,10,1,0,2,0,6,0,5,0,3,0,1,0,4,0,4,0,7,0,5,0,7,0,1,0,6,0,1,0,1,0,0,0,2,3,7,4,4,7,6,7,8,0,0,0,0,0,0,0,3,0,2,0,1,0,1,0,1,0,1,0,3,0,2,0,0,0,0,0,0,0,0,0,4,5,5,6,5,6,6,7,5,6,6,7,6,7,7,8,15,0,28,0,26,0,48,0,22,0,40,0,36,0,64,0,14,0,24,0,20,0,32,0,12,0,16,0,8,0,0,0,1,5,5,7,5,8,7,9,5,7,7,9,7,9,9,10,1,0,10,0,8,0,20,0,12,0,20,0,16,0,32,0,14,0,12,0,24,0,0,0,28,0,16,0,24,0,16,0,1,4,7,4,5,7,6,7,8,0,0,0,0,0,0,0,1,0,2,0,1,0,3,0,1,0,1,0,3,0,2,0,0,0,0,0,0,0,0,0,4,5,7,8,9,10,10,11,11,12,12,12,12,12,13,10,5,6,7,8,9,10,10,11,11,11,12,12,12,12,12,10,7,7,8,9,9,10,10,11,11,11,11,12,12,12,13,9,8,8,9,9,10,10,10,11,11,11,11,12,12,12,12,9,9,9,9,10,10,10,10,11,11,11,12,12,12,12,13,9,10,9,10,10,10,10,11,11,11,11,12,12,12,12,12,9,10,10,10,10,10,11,11,11,11,12,12,12,12,12,13,9,11,10,10,10,11,11,11,11,12,12,12,12,12,13,13,10,11,11,11,11,11,11,11,11,11,12,12,12,12,13,13,10,11,11,11,11,11,11,11,12,12,12,12,12,13,13,13,10,12,11,11,11,11,12,12,12,12,12,12,13,13,13,13,10,12,12,11,11,11,12,12,12,12,12,12,13,13,13,13,10,12,12,12,12,12,12,12,12,12,12,13,13,13,13,13,10,12,12,12,12,12,12,12,12,13,13,13,13,13,13,13,10,13,12,12,12,12,12,12,13,13,13,13,13,13,13,13,10,9,9,9,9,9,9,9,9,9,9,9,10,10,10,10,6,15,0,13,0,46,0,80,0,146,0,6,1,248,0,178,1,170,1,157,2,141,2,137,2,109,2,5,2,8,4,88,0,14,0,12,0,21,0,38,0,71,0,130,0,122,0,216,0,209,0,198,0,71,1,89,1,63,1,41,1,23,1,42,0,47,0,22,0,41,0,74,0,68,0,128,0,120,0,221,0,207,0,194,0,182,0,84,1,59,1,39,1,29,2,18,0,81,0,39,0,75,0,70,0,134,0,125,0,116,0,220,0,204,0,190,0,178,0,69,1,55,1,37,1,15,1,16,0,147,0,72,0,69,0,135,0,127,0,118,0,112,0,210,0,200,0,188,0,96,1,67,1,50,1,29,1,28,2,14,0,7,1,66,0,129,0,126,0,119,0,114,0,214,0,202,0,192,0,180,0,85,1,61,1,45,1,25,1,6,1,12,0,249,0,123,0,121,0,117,0,113,0,215,0,206,0,195,0,185,0,91,1,74,1,52,1,35,1,16,1,8,2,10,0,179,1,115,0,111,0,109,0,211,0,203,0,196,0,187,0,97,1,76,1,57,1,42,1,27,1,19,2,125,1,17,0,171,1,212,0,208,0,205,0,201,0,193,0,186,0,177,0,169,0,64,1,47,1,30,1,12,1,2,2,121,1,16,0,79,1,199,0,197,0,191,0,189,0,181,0,174,0,77,1,65,1,49,1,33,1,19,1,9,2,123,1,115,1,11,0,156,2,184,0,183,0,179,0,175,0,88,1,75,1,58,1,48,1,34,1,21,1,18,2,127,1,117,1,110,1,10,0,140,2,90,1,171,0,168,0,164,0,62,1,53,1,43,1,31,1,20,1,7,1,1,2,119,1,112,1,106,1,6,0,136,2,66,1,60,1,56,1,51,1,46,1,36,1,28,1,13,1,5,1,0,2,120,1,114,1,108,1,103,1,4,0,108,2,44,1,40,1,38,1,32,1,26,1,17,1,10,1,3,2,124,1,118,1,113,1,109,1,105,1,101,1,2,0,9,4,24,1,22,1,18,1,11,1,8,1,3,1,126,1,122,1,116,1,111,1,107,1,104,1,102,1,100,1,0,0,43,0,20,0,19,0,17,0,15,0,13,0,11,0,9,0,7,0,6,0,4,0,7,0,5,0,3,0,1,0,3,0,1,4,3,5,0,0,0,0,1,0,1,0,1,0,0,0,1,5,7,9,10,10,11,11,12,12,12,13,13,13,14,10,4,6,8,9,10,11,11,11,12,12,12,13,14,13,14,10,7,8,9,10,11,11,12,12,13,12,13,13,13,14,14,11,9,9,10,11,11,12,12,12,13,13,14,14,14,15,15,12,10,10,11,11,12,12,13,13,13,14,14,14,15,15,15,11,10,10,11,11,12,13,13,14,13,14,14,15,15,15,16,12,11,11,11,12,13,13,13,13,14,14,14,14,15,15,16,12,11,11,12,12,13,13,13,14,14,15,15,15,15,17,17,12,11,12,12,13,13,13,14,14,15,15,15,15,16,16,16,12,12,12,12,13,13,14,14,15,15,15,15,16,15,16,15,13,12,13,12,13,14,14,14,14,15,16,16,16,17,17,16,12,13,13,13,13,14,14,15,16,16,16,16,16,16,15,16,13,13,14,14,14,14,15,15,15,15,17,16,16,16,16,18,13,15,14,14,14,15,15,16,16,16,18,17,17,17,19,17,13,14,15,13,14,16,16,15,16,16,17,18,17,19,17,16,13,10,10,10,11,11,12,12,12,13,13,13,13,13,13,13,10,1,5,7,9,10,10,11,11,12,12,12,13,13,13,14,11,4,6,8,9,10,11,11,11,12,12,12,13,14,13,14,11,7,8,9,10,11,11,12,12,13,12,13,13,13,14,14,12,9,9,10,11,11,12,12,12,13,13,14,14,14,15,15,13,10,10,11,11,12,12,13,13,13,14,14,14,15,15,15,12,10,10,11,11,12,13,13,14,13,14,14,15,15,15,16,13,11,11,11,12,13,13,13,13,14,14,14,14,15,15,16,13,11,11,12,12,13,13,13,14,14,15,15,15,15,17,17,13,11,12,12,13,13,13,14,14,15,15,15,15,16,16,16,13,12,12,12,13,13,14,14,15,15,15,15,16,15,16,15,14,12,13,12,13,14,14,14,14,15,16,16,16,17,17,16,13,13,13,13,13,14,14,15,16,16,16,16,16,16,15,16,14,13,14,14,14,14,15,15,15,15,17,16,16,16,16,18,14,15,14,14,14,15,15,16,16,16,18,17,17,17,19,17,14,14,15,13,14,16,16,15,16,16,17,18,17,19,17,16,14,11,11,11,12,12,13,13,13,14,14,14,14,14,14,14,12,1,0,5,0,14,0,44,0,74,0,63,0,110,0,93,0,172,0,149,0,138,0,242,0,225,0,195,0,120,1,17,0,3,0,4,0,12,0,20,0,35,0,62,0,53,0,47,0,83,0,75,0,68,0,119,0,201,0,107,0,207,0,9,0,15,0,13,0,23,0,38,0,67,0,58,0,103,0,90,0,161,0,72,0,127,0,117,0,110,0,209,0,206,0,16,0,45,0,21,0,39,0,69,0,64,0,114,0,99,0,87,0,158,0,140,0,252,0,212,0,199,0,131,1,109,1,26,0,75,0,36,0,68,0,65,0,115,0,101,0,179,0,164,0,155,0,8,1,246,0,226,0,139,1,126,1,106,1,9,0,66,0,30,0,59,0,56,0,102,0,185,0,173,0,9,1,142,0,253,0,232,0,144,1,132,1,122,1,189,1,16,0,111,0,54,0,52,0,100,0,184,0,178,0,160,0,133,0,1,1,244,0,228,0,217,0,129,1,110,1,203,2,10,0,98,0,48,0,91,0,88,0,165,0,157,0,148,0,5,1,248,0,151,1,141,1,116,1,124,1,121,3,116,3,8,0,85,0,84,0,81,0,159,0,156,0,143,0,4,1,249,0,171,1,145,1,136,1,127,1,215,2,201,2,196,2,7,0,154,0,76,0,73,0,141,0,131,0,0,1,245,0,170,1,150,1,138,1,128,1,223,2,103,1,198,2,96,1,11,0,139,0,129,0,67,0,125,0,247,0,233,0,229,0,219,0,137,1,231,2,225,2,208,2,117,3,114,3,183,1,4,0,243,0,120,0,118,0,115,0,227,0,223,0,140,1,234,2,230,2,224,2,209,2,200,2,194,2,223,0,180,1,6,0,202,0,224,0,222,0,218,0,216,0,133,1,130,1,125,1,108,1,120,3,187,1,195,2,184,1,181,1,192,6,4,0,235,2,211,0,210,0,208,0,114,1,123,1,222,2,211,2,202,2,199,6,115,3,109,3,108,3,131,13,97,3,2,0,121,1,113,1,102,0,187,0,214,2,210,2,102,1,199,2,197,2,98,3,198,6,103,3,130,13,102,3,178,1,0,0,12,0,10,0,7,0,11,0,10,0,17,0,11,0,9,0,13,0,12,0,10,0,7,0,5,0,3,0,1,0,3,0,3,5,6,8,8,9,10,10,10,11,11,12,12,12,13,14,5,5,7,8,9,9,10,10,10,11,11,12,12,12,13,13,6,7,7,8,9,9,10,10,10,11,11,12,12,13,13,13,7,8,8,9,9,10,10,11,11,11,12,12,12,13,13,13,8,8,9,9,10,10,11,11,11,11,12,12,12,13,13,13,9,9,9,10,10,10,11,11,11,11,12,12,13,13,13,14,10,9,10,10,10,11,11,11,11,12,12,12,13,13,14,14,10,10,10,11,11,11,11,12,12,12,12,12,13,13,13,14,10,10,10,11,11,11,11,12,12,12,12,13,13,14,14,14,10,10,11,11,11,11,12,12,12,13,13,13,13,14,14,14,11,11,11,11,12,12,12,12,12,13,13,13,13,14,15,14,11,11,11,11,12,12,12,12,13,13,13,13,14,14,14,15,12,12,11,12,12,12,13,13,13,13,13,13,14,14,15,15,12,12,12,12,12,13,13,13,13,14,14,14,14,14,15,15,13,13,13,13,13,13,13,13,14,14,14,14,15,15,14,15,13,13,13,13,13,13,13,14,14,14,14,14,15,15,15,15,7,0,12,0,18,0,53,0,47,0,76,0,124,0,108,0,89,0,123,0,108,0,119,0,107,0,81,0,122,0,63,0,13,0,5,0,16,0,27,0,46,0,36,0,61,0,51,0,42,0,70,0,52,0,83,0,65,0,41,0,59,0,36,0,19,0,17,0,15,0,24,0,41,0,34,0,59,0,48,0,40,0,64,0,50,0,78,0,62,0,80,0,56,0,33,0,29,0,28,0,25,0,43,0,39,0,63,0,55,0,93,0,76,0,59,0,93,0,72,0,54,0,75,0,50,0,29,0,52,0,22,0,42,0,40,0,67,0,57,0,95,0,79,0,72,0,57,0,89,0,69,0,49,0,66,0,46,0,27,0,77,0,37,0,35,0,66,0,58,0,52,0,91,0,74,0,62,0,48,0,79,0,63,0,90,0,62,0,40,0,38,0,125,0,32,0,60,0,56,0,50,0,92,0,78,0,65,0,55,0,87,0,71,0,51,0,73,0,51,0,70,0,30,0,109,0,53,0,49,0,94,0,88,0,75,0,66,0,122,0,91,0,73,0,56,0,42,0,64,0,44,0,21,0,25,0,90,0,43,0,41,0,77,0,73,0,63,0,56,0,92,0,77,0,66,0,47,0,67,0,48,0,53,0,36,0,20,0,71,0,34,0,67,0,60,0,58,0,49,0,88,0,76,0,67,0,106,0,71,0,54,0,38,0,39,0,23,0,15,0,109,0,53,0,51,0,47,0,90,0,82,0,58,0,57,0,48,0,72,0,57,0,41,0,23,0,27,0,62,0,9,0,86,0,42,0,40,0,37,0,70,0,64,0,52,0,43,0,70,0,55,0,42,0,25,0,29,0,18,0,11,0,11,0,118,0,68,0,30,0,55,0,50,0,46,0,74,0,65,0,49,0,39,0,24,0,16,0,22,0,13,0,14,0,7,0,91,0,44,0,39,0,38,0,34,0,63,0,52,0,45,0,31,0,52,0,28,0,19,0,14,0,8,0,9,0,3,0,123,0,60,0,58,0,53,0,47,0,43,0,32,0,22,0,37,0,24,0,17,0,12,0,15,0,10,0,2,0,1,0,71,0,37,0,34,0,30,0,28,0,20,0,17,0,26,0,21,0,16,0,10,0,6,0,8,0,6,0,2,0,0,0,1,5,7,8,9,10,10,11,10,11,12,12,13,13,14,14,4,6,8,9,10,10,11,11,11,11,12,12,13,14,14,14,7,8,9,10,11,11,12,12,11,12,12,13,13,14,15,15,8,9,10,11,11,12,12,12,12,13,13,13,13,14,15,15,9,9,11,11,12,12,13,13,12,13,13,14,14,15,15,16,10,10,11,12,12,12,13,13,13,13,14,13,15,15,16,16,10,11,12,12,13,13,13,13,13,14,14,14,15,15,16,16,11,11,12,13,13,13,14,14,14,14,15,15,15,16,18,18,10,10,11,12,12,13,13,14,14,14,14,15,15,16,17,17,11,11,12,12,13,13,13,15,14,15,15,16,16,16,18,17,11,12,12,13,13,14,14,15,14,15,16,15,16,17,18,19,12,12,12,13,14,14,14,14,15,15,15,16,17,17,17,18,12,13,13,14,14,15,14,15,16,16,17,17,17,18,18,18,13,13,14,15,15,15,16,16,16,16,16,17,18,17,18,18,14,14,14,15,15,15,17,16,16,19,17,17,17,19,18,18,13,14,15,16,16,16,17,16,17,17,18,18,21,20,21,18,1,0,5,0,14,0,21,0,34,0,51,0,46,0,71,0,42,0,52,0,68,0,52,0,67,0,44,0,43,0,19,0,3,0,4,0,12,0,19,0,31,0,26,0,44,0,33,0,31,0,24,0,32,0,24,0,31,0,35,0,22,0,14,0,15,0,13,0,23,0,36,0,59,0,49,0,77,0,65,0,29,0,40,0,30,0,40,0,27,0,33,0,42,0,16,0,22,0,20,0,37,0,61,0,56,0,79,0,73,0,64,0,43,0,76,0,56,0,37,0,26,0,31,0,25,0,14,0,35,0,16,0,60,0,57,0,97,0,75,0,114,0,91,0,54,0,73,0,55,0,41,0,48,0,53,0,23,0,24,0,58,0,27,0,50,0,96,0,76,0,70,0,93,0,84,0,77,0,58,0,79,0,29,0,74,0,49,0,41,0,17,0,47,0,45,0,78,0,74,0,115,0,94,0,90,0,79,0,69,0,83,0,71,0,50,0,59,0,38,0,36,0,15,0,72,0,34,0,56,0,95,0,92,0,85,0,91,0,90,0,86,0,73,0,77,0,65,0,51,0,44,0,43,0,42,0,43,0,20,0,30,0,44,0,55,0,78,0,72,0,87,0,78,0,61,0,46,0,54,0,37,0,30,0,20,0,16,0,53,0,25,0,41,0,37,0,44,0,59,0,54,0,81,0,66,0,76,0,57,0,54,0,37,0,18,0,39,0,11,0,35,0,33,0,31,0,57,0,42,0,82,0,72,0,80,0,47,0,58,0,55,0,21,0,22,0,26,0,38,0,22,0,53,0,25,0,23,0,38,0,70,0,60,0,51,0,36,0,55,0,26,0,34,0,23,0,27,0,14,0,9,0,7,0,34,0,32,0,28,0,39,0,49,0,75,0,30,0,52,0,48,0,40,0,52,0,28,0,18,0,17,0,9,0,5,0,45,0,21,0,34,0,64,0,56,0,50,0,49,0,45,0,31,0,19,0,12,0,15,0,10,0,7,0,6,0,3,0,48,0,23,0,20,0,39,0,36,0,35,0,53,0,21,0,16,0,23,0,13,0,10,0,6,0,1,0,4,0,2,0,16,0,15,0,17,0,27,0,25,0,20,0,29,0,11,0,17,0,12,0,16,0,8,0,1,0,1,0,0,0,1,0,4,4,6,8,9,10,10,10,4,5,6,7,9,9,10,10,6,6,7,8,9,10,9,10,7,7,8,8,9,10,10,10,8,8,9,9,10,10,10,11,9,9,10,10,10,11,10,11,9,9,9,10,10,11,11,12,10,10,10,11,11,11,11,12,9,0,6,0,16,0,33,0,41,0,39,0,38,0,26,0,7,0,5,0,6,0,9,0,23,0,16,0,26,0,11,0,17,0,7,0,11,0,14,0,21,0,30,0,10,0,7,0,17,0,10,0,15,0,12,0,18,0,28,0,14,0,5,0,32,0,13,0,22,0,19,0,18,0,16,0,9,0,5,0,40,0,17,0,31,0,29,0,17,0,13,0,4,0,2,0,27,0,12,0,11,0,15,0,10,0,7,0,4,0,1,0,27,0,12,0,8,0,12,0,6,0,3,0,1,0,0,0,2,4,6,8,9,10,9,10,4,5,6,8,10,10,9,10,6,7,8,9,10,11,10,10,8,8,9,11,10,12,10,11,9,10,10,11,11,12,11,12,9,10,11,12,12,13,12,13,9,9,9,10,11,12,12,12,9,9,10,11,12,12,12,12,3,0,4,0,10,0,24,0,34,0,33,0,21,0,15,0,5,0,3,0,4,0,10,0,32,0,17,0,11,0,10,0,11,0,7,0,13,0,18,0,30,0,31,0,20,0,5,0,25,0,11,0,19,0,59,0,27,0,18,0,12,0,5,0,35,0,33,0,31,0,58,0,30,0,16,0,7,0,5,0,28,0,26,0,32,0,19,0,17,0,15,0,8,0,14,0,14,0,12,0,9,0,13,0,14,0,9,0,4,0,1,0,11,0,4,0,6,0,6,0,6,0,3,0,2,0,0,0,1,4,7,9,10,10,10,11,4,6,8,9,10,11,10,10,7,8,9,10,11,12,11,11,8,9,10,11,12,12,11,12,9,10,11,12,12,12,12,12,10,11,12,12,13,13,12,13,9,10,11,12,12,12,13,13,10,10,11,12,12,13,13,13,1,0,2,0,10,0,23,0,35,0,30,0,12,0,17,0,3,0,3,0,8,0,12,0,18,0,21,0,12,0,7,0,11,0,9,0,15,0,21,0,32,0,40,0,19,0,6,0,14,0,13,0,22,0,34,0,46,0,23,0,18,0,7,0,20,0,19,0,33,0,47,0,27,0,22,0,9,0,3,0,31,0,22,0,41,0,26,0,21,0,20,0,5,0,3,0,14,0,13,0,10,0,11,0,16,0,6,0,5,0,1,0,9,0,8,0,7,0,8,0,4,0,4,0,2,0,0,0,248,17,25,0,52,0,0,0,80,106,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,24,0,0,0,2,0,0,0,132,0,0,0,160,0,0,0,38,0,0,0,0,0,4,0,8,0,12,0,16,0,20,0,24,0,32,0,40,0,48,0,64,0,92,0,128,0,0,0,0,0,0,0,0,0,4,0,8,0,12,0,16,0,20,0,24,0,28,0,36,0,44,0,52,0,60,0,72,0,88,0,108,0,128,0,0,0,4,0,8,0,12,0,16,0,20,0,28,0,36,0,44,0,56,0,68,0,80,0,96,0,112,0,128,0,0,0,0,0,4,0,8,0,12,0,16,0,20,0,24,0,28,0,36,0,44,0,52,0,64,0,76,0,92,0,108,0,128,0,0,0,4,0,8,0,12,0,16,0,20,0,24,0,28,0,32,0,40,0,48,0,60,0,72,0,88,0,108,0,128,0,0,0,4,0,8,0,12,0,16,0,20,0,24,0,28,0,32,0,36,0,40,0,44,0,48,0,52,0,56,0,64,0,72,0,80,0,88,0,96,0,108,0,120,0,132,0,144,0,156,0,172,0,188,0,212,0,240,0,20,1,64,1,128,1,192,1,0,2,64,2,128,2,192,2,0,3,64,3,128,3,192,3,0,4,0,0,0,0,0,0,12,0,24,0,36,0,48,0,60,0,72,0,84,0,96,0,108,0,120,0,132,0,144,0,156,0,172,0,188,0,204,0,220,0,236,0,252,0,12,1,32,1,52,1,72,1,92,1,116,1,140,1,164,1,192,1,220,1,252,1,32,2,68,2,108,2,152,2,200,2,252,2,52,3,112,3,176,3,0,4,0,0,0,0,0,0,0,0,4,0,8,0,12,0,16,0,20,0,24,0,28,0,32,0,36,0,40,0,44,0,48,0,52,0,56,0,64,0,72,0,80,0,88,0,100,0,112,0,124,0,140,0,156,0,172,0,192,0,216,0,240,0,12,1,48,1,88,1,128,1,168,1,208,1,248,1,32,2,72,2,112,2,152,2,192,2,232,2,16,3,56,3,96,3,136,3,176,3,216,3,0,4,0,0,4,0,8,0,12,0,16,0,20,0,24,0,28,0,32,0,36,0,40,0,48,0,56,0,64,0,72,0,80,0,88,0,96,0,108,0,120,0,132,0,144,0,160,0,176,0,196,0,216,0,240,0,8,1,36,1,64,1,96,1,128,1,160,1,192,1,224,1,0,2,32,2,64,2,96,2,128,2,160,2,192,2,224,2,0,3,32,3,64,3,96,3,128,3,160,3,0,4,0,0,0,0,0,0,4,0,8,0,12,0,16,0,20,0,24,0,28,0,32,0,36,0,40,0,48,0,56,0,64,0,72,0,80,0,88,0,96,0,108,0,120,0,132,0,144,0,160,0,176,0,196,0,216,0,240,0,8,1,36,1,64,1,96,1,128,1,160,1,192,1,224,1,0,2,32,2,64,2,96,2,128,2,160,2,192,2,224,2,0,3,32,3,64,3,96,3,128,3,160,3,192,3,224,3,0,4,0,0,4,0,8,0,12,0,16,0,20,0,24,0,28,0,32,0,36,0,40,0,44,0,52,0,60,0,68,0,76,0,84,0,92,0,100,0,108,0,116,0,124,0,136,0,148,0,160,0,172,0,188,0,204,0,220,0,240,0,4,1,28,1,52,1,80,1,108,1,140,1,176,1,212,1,252,1,40,2,88,2,140,2,192,2,0,3,64,3,128,3,192,3,0,4,0,0,8,0,16,0,24,0,32,0,40,0,48,0,56,0,64,0,72,0,80,0,88,0,100,0,112,0,124,0,136,0,148,0,160,0,172,0,184,0,196,0,212,0,228,0,244,0,4,1,24,1,44,1,64,1,88,1,112,1,140,1,168,1,200,1,236,1,20,2,60,2,104,2,152,2,204,2,4,3,64,3,128,3,192,3,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,10,83,116,97,110,100,97,114,100,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,115,58,10,78,65,77,69,32,32,32,32,32,32,32,32,68,69,67,79,77,80,79,83,73,84,73,79,78,0,0,0,0,73,110,100,105,118,105,100,117,97,108,32,99,104,97,110,110,101,108,115,58,10,78,65,77,69,32,32,32,32,32,32,32,32,68,69,83,67,82,73,80,84,73,79,78,0,0,0,0,80,105,120,101,108,32,102,111,114,109,97,116,115,58,10,73,46,46,46,46,32,61,32,83,117,112,112,111,114,116,101,100,32,73,110,112,117,116,32,32,102,111,114,109,97,116,32,102,111,114,32,99,111,110,118,101,114,115,105,111,110,10,46,79,46,46,46,32,61,32,83,117,112,112,111,114,116,101,100,32,79,117,116,112,117,116,32,102,111,114,109,97,116,32,102,111,114,32,99,111,110,118,101,114,115,105,111,110,10,46,46,72,46,46,32,61,32,72,97,114,100,119,97,114,101,32,97,99,99,101,108,101,114,97,116,101,100,32,102,111,114,109,97,116,10,46,46,46,80,46,32,61,32,80,97,108,101,116,116,101,100,32,102,111,114,109,97,116,10,46,46,46,46,66,32,61,32,66,105,116,115,116,114,101,97,109,32,102,111,114,109,97,116,10,70,76,65,71,83,32,78,65,77,69,32,32,32,32,32,32,32,32,32,32,32,32,78,66,95,67,79,77,80,79,78,69,78,84,83,32,66,73,84,83,95,80,69,82,95,80,73,88,69,76,10,45,45,45,45,45,0,0,0,0,0,0,70,105,108,116,101,114,115,58,10,32,32,84,46,32,61,32,84,105,109,101,108,105,110,101,32,115,117,112,112,111,114,116,10,32,32,46,83,32,61,32,83,108,105,99,101,32,116,104,114,101,97,100,105,110,103,10,32,32,65,32,61,32,65,117,100,105,111,32,105,110,112,117,116,47,111,117,116,112,117,116,10,32,32,86,32,61,32,86,105,100,101,111,32,105,110,112,117,116,47,111,117,116,112,117,116,10,32,32,78,32,61,32,68,121,110,97,109,105,99,32,110,117,109,98,101,114,32,97,110,100,47,111,114,32,116,121,112,101,32,111,102,32,105,110,112,117,116,47,111,117,116,112,117,116,10,32,32,124,32,61,32,83,111,117,114,99,101,32,111,114,32,115,105,110,107,32,102,105,108,116,101,114,0,0,79,117,116,112,117,116,58,0,83,117,112,112,111,114,116,101,100,32,102,105,108,101,32,112,114,111,116,111,99,111,108,115,58,10,73,110,112,117,116,58,0,0,0,0,0,0,0,0,66,105,116,115,116,114,101,97,109,32,102,105,108,116,101,114,115,58,0,0,0,0,0,0,67,111,100,101,99,115,58,10,32,68,46,46,46,46,46,32,61,32,68,101,99,111,100,105,110,103,32,115,117,112,112,111,114,116,101,100,10,32,46,69,46,46,46,46,32,61,32,69,110,99,111,100,105,110,103,32,115,117,112,112,111,114,116,101,100,10,32,46,46,86,46,46,46,32,61,32,86,105,100,101,111,32,99,111,100,101,99,10,32,46,46,65,46,46,46,32,61,32,65,117,100,105,111,32,99,111,100,101,99,10,32,46,46,83,46,46,46,32,61,32,83,117,98,116,105,116,108,101,32,99,111,100,101,99,10,32,46,46,46,73,46,46,32,61,32,73,110,116,114,97,32,102,114,97,109,101,45,111,110,108,121,32,99,111,100,101,99,10,32,46,46,46,46,76,46,32,61,32,76,111,115,115,121,32,99,111,109,112,114,101,115,115,105,111,110,10,32,46,46,46,46,46,83,32,61,32,76,111,115,115,108,101,115,115,32,99,111,109,112,114,101,115,115,105,111,110,10,32,45,45,45,45,45,45,45,0,0,0,0,0,84,104,105,115,32,102,105,108,116,101,114,32,104,97,115,32,115,117,112,112,111,114,116,32,102,111,114,32,116,105,109,101,108,105,110,101,32,116,104,114,111,117,103,104,32,116,104,101,32,39,101,110,97,98,108,101,39,32,111,112,116,105,111,110,46,0,0,0,0,0,0,0,32,32,32,32,32,32,32,32,110,111,110,101,32,40,115,105,110,107,32,102,105,108,116,101,114,41,0,0,0,0,0,0,32,32,32,32,32,32,32,32,100,121,110,97,109,105,99,32,40,100,101,112,101,110,100,105,110,103,32,111,110,32,116,104,101,32,111,112,116,105,111,110,115,41,0,0,0,0,0,0,32,32,32,32,79,117,116,112,117,116,115,58,0,0,0,0,32,32,32,32,32,32,32,32,110,111,110,101,32,40,115,111,117,114,99,101,32,102,105,108,116,101,114,41,0,0,0,0,32,32,32,32,73,110,112,117,116,115,58,0,0,0,0,0,32,32,32,32,115,108,105,99,101,32,116,104,114,101,97,100,105,110,103,32,115,117,112,112,111,114,116,101,100,0,0,0,70,105,108,101,32,102,111,114,109,97,116,115,58,10,32,68,46,32,61,32,68,101,109,117,120,105,110,103,32,115,117,112,112,111,114,116,101,100,10,32,46,69,32,61,32,77,117,120,105,110,103,32,115,117,112,112,111,114,116,101,100,10,32,45,45,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,63,0,0,0,0,0,0,240,63,0,0,0,0,0,0,248,63,0,0,0,0,0,0,0,64,0,0,0,0,0,0,4,64,0,0,0,0,0,0,18,64,0,0,0,0,0,0,33,64,0,0,0,4,107,244,52,66,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,63,0,0,0,0,0,0,240,63,0,0,0,0,0,0,248,63,0,0,0,0,0,0,4,64,0,0,0,0,0,0,18,64,0,0,0,0,0,0,33,64,0,0,0,0,0,128,48,64,0,0,0,4,107,244,52,66,1,0,0,0,0,0,0,0,0,0,0,0,3,1,1,1,2,2,2,3,3,3,4,4,0,1,2,3,0,1,2,3,1,2,3,1,2,3,2,3,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,8,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,16,0,0,0,16,0,0,0,255,127,0,0,246,127,36,3,217,127,72,6,167,127,107,9,98,127,140,12,10,127,171,15,157,126,200,18,30,126,226,21,138,125,249,24,228,124,12,28,42,124,26,31,93,123,36,34,125,122,40,37,138,121,39,40,133,120,31,43,108,119,17,46,66,118,252,48,5,117,223,51,182,115,186,54,85,114,141,57,227,112,87,60,95,111,23,63,202,109,206,65,36,108,123,68,110,106,29,71,167,104,180,73,208,102,64,76,233,100,192,78,242,98,52,81,236,96,155,83,215,94,246,85,180,92,67,88,130,90,130,90,67,88,180,92,246,85,215,94,155,83,236,96,52,81,242,98,192,78,233,100,64,76,208,102,180,73,167,104,29,71,110,106,123,68,36,108,206,65,202,109,23,63,95,111,87,60,227,112,141,57,85,114,186,54,182,115,223,51,5,117,252,48,66,118,17,46,108,119,31,43,133,120,39,40,138,121,40,37,125,122,36,34,93,123,26,31,42,124,12,28,228,124,249,24,138,125,226,21,30,126,200,18,157,126,171,15,10,127,140,12,98,127,107,9,167,127,72,6,217,127,36,3,246,127,18,0,9,0,0,0,3,0,6,0,0,15,0,0,0,12,0,0,0,0,24,21,0,0,0,0,0,0,238,0,254,255,0,241,0,2,0,0,3,0,253,247,0,244,0,0,0,0,250,0,0,0,232,235,0,0,0,0,0,160,255,35,3,64,0,0,0,224,240,122,11,192,0,0,0,192,163,123,9,64,0,0,0,160,213,252,252,191,0,0,0,32,124,194,201,191,0,0,0,128,168,157,247,63,0,0,0,32,150,137,251,191,0,0,0,224,104,71,239,63,0,0,0,160,192,166,193,191,0,0,0,224,96,112,216,191,0,0,0,64,178,140,209,63,0,0,0,192,169,11,177,63,0,0,0,96,252,138,214,191,0,0,0,160,60,28,216,63,0,0,0,64,157,172,206,191,0,0,0,96,172,148,177,63,0,0,0,64,201,170,0,64,0,0,0,224,178,85,8,192,0,0,0,224,104,163,9,64,0,0,0,0,187,14,6,192,0,0,0,64,63,247,247,63,0,0,0,96,197,239,213,191,0,0,0,192,109,244,230,191,0,0,0,32,230,45,241,63,0,0,0,128,126,92,240,191,0,0,0,128,196,32,226,63,0,0,0,96,5,215,202,191,0,0,0,192,165,188,176,191,0,0,0,192,233,108,186,63,0,0,0,96,225,67,177,191,0,0,0,64,133,72,116,191,0,0,0,0,0,0,0,0,0,0,0,96,1,250,6,64,0,0,0,160,80,42,20,192,0,0,0,160,41,250,24,64,0,0,0,160,194,100,23,192,0,0,0,192,110,167,13,64,0,0,0,0,205,202,240,191,0,0,0,64,170,237,242,191,0,0,0,96,196,230,0,64,0,0,0,192,30,141,254,191,0,0,0,64,225,248,239,63,0,0,0,192,80,224,197,191,0,0,0,128,189,223,212,191,0,0,0,96,172,10,217,63,0,0,0,128,112,51,209,191,0,0,0,32,77,1,185,63,0,0,0,32,135,9,152,191,0,0,0,160,38,107,5,64,0,0,0,128,213,82,19,192,0,0,0,0,203,71,26,64,0,0,0,160,44,212,29,192,0,0,0,96,194,231,26,64,0,0,0,96,133,100,19,192,0,0,0,96,101,84,0,64,0,0,0,0,156,107,230,63,0,0,0,32,75,161,7,192,0,0,0,160,245,81,16,64,0,0,0,64,243,188,16,192,0,0,0,96,66,166,10,64,0,0,0,32,166,241,0,192,0,0,0,0,64,35,236,63,0,0,0,32,188,66,160,191,0,0,0,0,255,31,219,191,0,0,0,0,1,165,222,63,0,0,0,160,208,182,214,191,0,0,0,64,93,101,198,63,0,0,0,0,91,47,175,191,0,0,0,0,57,35,250,63,0,0,0,32,167,23,2,192,0,0,0,192,249,66,3,64,0,0,0,160,200,18,5,192,0,0,0,128,252,38,1,64,0,0,0,32,147,11,253,191,0,0,0,96,83,78,241,63,0,0,0,128,49,127,230,191,0,0,0,160,65,120,196,63,0,0,0,128,250,69,165,63,0,0,0,160,156,211,210,191,0,0,0,160,2,30,208,63,0,0,0,128,66,197,209,191,0,0,0,160,143,52,196,63,0,0,0,0,82,6,186,191,0,0,0,96,231,60,145,63,0,0,0,224,38,71,234,63,0,0,0,160,22,26,240,191,0,0,0,192,13,243,227,63,0,0,0,0,43,183,240,191,0,0,0,224,128,166,228,63,0,0,0,128,110,9,236,191,0,0,0,160,215,181,224,63,0,0,0,0,55,148,229,191,0,0,0,192,80,197,220,63,0,0,0,160,130,209,224,191,0,0,0,160,129,31,214,63,0,0,0,224,11,91,217,191,0,0,0,224,143,37,209,63,0,0,0,224,227,132,210,191,0,0,0,224,184,28,200,63,0,0,0,96,206,113,200,191,0,0,0,192,75,180,186,63,0,0,0,32,255,56,187,191,0,0,0,128,100,250,167,63,0,0,0,96,112,77,164,191,0,0,0,32,49,248,172,63,0,0,0,224,232,57,226,191,0,0,0,0,213,16,218,191,0,0,0,224,78,173,213,191,0,0,0,64,41,20,211,191,0,0,0,160,192,94,200,191,0,0,0,0,1,34,197,191,0,0,0,32,63,61,193,191,0,0,0,160,253,188,184,191,0,0,0,192,162,191,180,191,0,0,0,128,195,160,176,191,0,0,0,224,15,226,171,191,0,0,0,224,187,53,166,191,0,0,0,0,65,189,162,191,0,0,0,32,27,227,154,191,0,0,0,0,177,60,147,191,0,0,0,32,6,107,139,191,0,0,0,64,36,162,130,191,0,0,0,32,120,199,101,191,0,0,0,0,104,148,59,191,0,0,0,192,57,215,215,191,0,0,0,32,56,14,234,191,0,0,0,96,127,154,225,191,0,0,0,0,246,86,222,191,0,0,0,96,59,225,212,191,0,0,0,0,61,60,212,191,0,0,0,224,247,67,205,191,0,0,0,96,80,85,205,191,0,0,0,224,66,11,201,191,0,0,0,0,180,62,199,191,0,0,0,192,219,189,195,191,0,0,0,160,193,13,194,191,0,0,0,224,67,82,190,191,0,0,0,160,52,251,184,191,0,0,0,160,21,149,179,191,0,0,0,128,116,111,177,191,0,0,0,32,131,113,165,191,0,0,0,0,127,231,162,191,0,0,0,192,42,212,147,191,0,0,0,0,94,181,142,191,0,0,0,128,65,165,237,191,0,0,0,224,44,149,239,191,0,0,0,192,110,50,228,191,0,0,0,160,33,161,224,191,0,0,0,32,209,110,217,191,0,0,0,96,178,213,214,191,0,0,0,160,113,5,211,191,0,0,0,96,181,214,208,191,0,0,0,160,255,204,203,191,0,0,0,224,85,194,199,191,0,0,0,64,220,183,195,191,0,0,0,64,110,61,192,191,0,0,0,128,65,120,186,191,0,0,0,64,158,108,181,191,0,0,0,224,95,21,174,191,0,0,0,128,114,2,168,191,0,0,0,160,43,159,156,191,0,0,0,192,6,61,149,191,0,0,0,0,184,46,131,191,0,0,0,96,188,174,120,191,0,0,0,160,237,62,243,191,0,0,0,0,237,28,238,191,0,0,0,128,156,184,229,191,0,0,0,128,176,114,226,191,0,0,0,224,93,0,224,191,0,0,0,96,50,98,220,191,0,0,0,64,175,54,216,191,0,0,0,32,68,199,213,191,0,0,0,96,10,211,210,191,0,0,0,0,89,254,207,191,0,0,0,160,196,203,203,191,0,0,0,160,204,13,200,191,0,0,0,224,30,139,195,191,0,0,0,64,38,17,191,191,0,0,0,0,21,128,185,191,0,0,0,160,33,69,179,191,0,0,0,192,172,250,168,191,0,0,0,192,223,205,165,191,0,0,0,160,228,175,157,191,0,0,0,0,215,78,136,191,0,0,0,0,21,53,8,64,0,0,0,64,132,27,24,192,0,0,0,128,215,99,34,64,0,0,0,32,93,166,39,192,0,0,0,224,198,136,41,64,0,0,0,32,253,213,39,192,0,0,0,160,11,89,34,64,0,0,0,64,36,124,21,192,0,0,0,32,212,58,242,63,0,0,0,160,123,150,3,64,0,0,0,192,79,227,19,192,0,0,0,32,36,40,24,64,0,0,0,64,106,190,23,192,0,0,0,0,245,156,19,64,0,0,0,64,5,108,12,192,0,0,0,32,27,135,1,64,0,0,0,192,2,173,242,191,0,0,0,192,146,98,223,63,0,0,0,128,52,37,197,191,0,0,0,32,108,198,151,63,0,0,0,0,6,0,0,0,12,0,0,0,18,0,0,0,24,0,0,0,30,0,0,0,36,0,0,0,44,0,0,0,54,0,0,0,66,0,0,0,80,0,0,0,96,0,0,0,116,0,0,0,140,0,0,0,168,0,0,0,200,0,0,0,238,0,0,0,28,1,0,0,80,1,0,0,140,1,0,0,208,1,0,0,10,2,0,0,64,2,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,18,0,0,0,24,0,0,0,32,0,0,0,42,0,0,0,56,0,0,0,74,0,0,0,100,0,0,0,132,0,0,0,174,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,12,0,0,0,18,0,0,0,24,0,0,0,30,0,0,0,36,0,0,0,44,0,0,0,54,0,0,0,66,0,0,0,80,0,0,0,96,0,0,0,114,0,0,0,136,0,0,0,162,0,0,0,194,0,0,0,232,0,0,0,22,1,0,0,76,1,0,0,138,1,0,0,208,1,0,0,28,2,0,0,64,2,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,18,0,0,0,26,0,0,0,36,0,0,0,48,0,0,0,62,0,0,0,80,0,0,0,104,0,0,0,136,0,0,0,180,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,12,0,0,0,18,0,0,0,24,0,0,0,30,0,0,0,36,0,0,0,44,0,0,0,54,0,0,0,66,0,0,0,80,0,0,0,96,0,0,0,116,0,0,0,140,0,0,0,168,0,0,0,200,0,0,0,238,0,0,0,28,1,0,0,80,1,0,0,140,1,0,0,208,1,0,0,10,2,0,0,64,2,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,18,0,0,0,26,0,0,0,36,0,0,0,48,0,0,0,62,0,0,0,80,0,0,0,104,0,0,0,134,0,0,0,174,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,16,0,0,0,20,0,0,0,24,0,0,0,30,0,0,0,36,0,0,0,44,0,0,0,52,0,0,0,62,0,0,0,74,0,0,0,90,0,0,0,110,0,0,0,134,0,0,0,162,0,0,0,196,0,0,0,238,0,0,0,32,1,0,0,86,1,0,0,162,1,0,0,64,2,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,16,0,0,0,22,0,0,0,30,0,0,0,40,0,0,0,52,0,0,0,66,0,0,0,84,0,0,0,106,0,0,0,136,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,16,0,0,0,20,0,0,0,24,0,0,0,30,0,0,0,36,0,0,0,42,0,0,0,50,0,0,0,60,0,0,0,72,0,0,0,88,0,0,0,106,0,0,0,128,0,0,0,156,0,0,0,190,0,0,0,230,0,0,0,20,1,0,0,74,1,0,0,128,1,0,0,64,2,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,16,0,0,0,22,0,0,0,28,0,0,0,38,0,0,0,50,0,0,0,64,0,0,0,80,0,0,0,100,0,0,0,126,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,16,0,0,0,20,0,0,0,24,0,0,0,30,0,0,0,36,0,0,0,44,0,0,0,54,0,0,0,66,0,0,0,82,0,0,0,102,0,0,0,126,0,0,0,156,0,0,0,194,0,0,0,240,0,0,0,40,1,0,0,108,1,0,0,192,1,0,0,38,2,0,0,64,2,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,16,0,0,0,22,0,0,0,30,0,0,0,42,0,0,0,58,0,0,0,78,0,0,0,104,0,0,0,138,0,0,0,180,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,12,0,0,0,18,0,0,0,24,0,0,0,30,0,0,0,36,0,0,0,44,0,0,0,54,0,0,0,66,0,0,0,80,0,0,0,96,0,0,0,116,0,0,0,140,0,0,0,168,0,0,0,200,0,0,0,238,0,0,0,28,1,0,0,80,1,0,0,140,1,0,0,208,1,0,0,10,2,0,0,64,2,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,18,0,0,0,26,0,0,0,36,0,0,0,48,0,0,0,62,0,0,0,80,0,0,0,104,0,0,0,134,0,0,0,174,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,12,0,0,0,18,0,0,0,24,0,0,0,30,0,0,0,36,0,0,0,44,0,0,0,54,0,0,0,66,0,0,0,80,0,0,0,96,0,0,0,116,0,0,0,140,0,0,0,168,0,0,0,200,0,0,0,238,0,0,0,28,1,0,0,80,1,0,0,140,1,0,0,208,1,0,0,10,2,0,0,64,2,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,18,0,0,0,26,0,0,0,36,0,0,0,48,0,0,0,62,0,0,0,80,0,0,0,104,0,0,0,134,0,0,0,174,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,24,0,0,0,36,0,0,0,48,0,0,0,60,0,0,0,72,0,0,0,88,0,0,0,108,0,0,0,132,0,0,0,160,0,0,0,192,0,0,0,232,0,0,0,24,1,0,0,80,1,0,0,144,1,0,0,220,1,0,0,54,2,0,0,56,2,0,0,58,2,0,0,60,2,0,0,62,2,0,0,64,2,0,0,0,0,0,0,8,0,0,0,16,0,0,0,24,0,0,0,36,0,0,0,52,0,0,0,72,0,0,0,96,0,0,0,124,0,0,0,160,0,0,0,162,0,0,0,164,0,0,0,166,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,138,0,0,208,138,0,0,144,137,0,0,16,140,0,0,112,139,0,0,80,141,0,0,176,140,0,0,144,142,0,0,240,141,0,0,208,143,0,0,48,143,0,0,240,136,0,0,80,136,0,0,112,134,0,0,208,133,0,0,176,135,0,0,16,135,0,0,0,0,0,0,48,85,26,0,96,0,0,0,64,203,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,232,179,24,0,2,80,1,0,1,0,0,0,104,68,26,0,3,80,1,0,1,0,0,0,80,67,26,0,4,80,1,0,1,0,0,0,168,66,26,0,41,80,1,0,1,0,0,0,128,65,26,0,28,0,0,0,0,0,0,0,152,64,26,0,50,80,1,0,1,0,0,0,248,61,26,0,13,0,0,0,0,0,0,0,232,111,26,0,1,80,1,0,1,0,0,0,232,59,26,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,11,0,0,0,16,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,36,0,0,0,54,0,0,0,54,0,0,0,36,0,0,0,54,0,0,0,72,0,0,0,54,0,0,0,72,0,0,0,90,0,0,0,72,0,0,0,90,0,0,0,108,0,0,0,108,0,0,0,126,0,0,0,0,0,0,0,18,0,0,0,36,0,0,0,54,0,0,0,51,0,0,0,35,0,0,0,53,0,0,0,71,0,0,0,52,0,0,0,70,0,0,0,88,0,0,0,69,0,0,0,87,0,0,0,105,0,0,0,104,0,0,0,122,0,0,0,0,0,0,0,10,0,0,0,20,0,0,0,30,0,0,0,33,0,0,0,21,0,0,0,31,0,0,0,41,0,0,0,32,0,0,0,42,0,0,0,52,0,0,0,43,0,0,0,53,0,0,0,63,0,0,0,64,0,0,0,74,0,0,0,171,170,170,0,78,117,135,0,101,131,107,0,102,102,102,0,98,70,81,0,10,130,64,0,142,227,56,0,26,39,45,0,119,214,35,0,0,0,0,0,0,0,0,0,145,215,16,186,177,67,19,186,59,178,1,186,48,154,255,185,173,73,0,186,169,35,4,186,225,2,9,186,127,77,15,186,22,215,20,186,37,231,25,186,43,196,32,186,108,122,37,186,111,115,43,186,79,172,49,186,98,248,53,186,193,162,59,186,197,47,62,186,79,15,67,186,141,92,68,186,207,92,73,186,148,128,74,186,80,95,77,186,209,241,75,186,129,145,76,186,163,130,76,186,231,94,75,186,99,9,72,186,251,100,69,186,96,223,63,186,171,37,61,186,125,89,53,186,46,86,46,186,184,61,38,186,185,223,27,186,61,223,17,186,89,227,6,186,12,129,241,185,201,179,214,185,250,143,183,185,159,226,151,185,206,6,92,185,248,169,23,185,222,118,129,184,39,104,98,55,61,130,229,56,210,57,86,57,239,163,154,57,78,27,211,57,26,227,5,58,190,143,35,58,245,129,67,58,72,170,97,58,223,147,129,58,66,117,147,58,96,220,164,58,7,57,182,58,216,106,202,58,229,23,221,58,144,126,240,58,233,7,2,59,144,166,12,59,139,216,22,59,238,98,33,59,65,183,43,59,225,166,54,59,135,33,65,59,202,251,75,59,245,143,86,59,74,145,97,59,217,251,107,59,234,112,118,59,148,121,128,59,23,194,133,59,215,125,138,59,252,75,143,59,198,36,148,59,85,184,152,59,191,16,157,59,156,3,161,59,187,32,165,59,157,115,168,59,158,231,171,59,76,55,175,59,141,151,177,59,92,209,179,59,103,200,181,59,55,76,183,59,75,57,184,59,13,254,184,59,176,198,184,59,112,95,184,59,176,58,183,59,140,140,181,59,205,106,179,59,104,107,176,59,47,189,172,59,148,94,168,59,143,80,163,59,38,165,157,59,194,220,150,59,170,135,143,59,99,44,135,59,37,7,124,59,111,1,104,59,64,81,82,59,189,109,58,59,130,158,32,59,108,222,4,59,155,228,206,58,37,230,142,58,169,227,24,58,54,144,231,55,42,62,15,186,56,160,151,186,108,114,236,186,84,180,34,187,253,71,81,187,15,28,129,187,47,189,154,187,56,84,181,187,219,8,209,187,144,242,237,187,170,4,6,188,159,160,21,188,122,185,37,188,32,97,54,188,152,163,71,188,14,114,89,60,254,197,107,60,208,157,126,60,137,18,137,60,232,253,146,60,145,46,157,60,109,141,167,60,64,38,178,60,35,249,188,60,69,4,200,60,124,64,211,60,14,181,222,60,175,89,234,60,250,43,246,60,254,7,1,61,57,17,7,61,32,40,13,61,6,85,19,61,250,133,25,61,24,196,31,61,241,7,38,61,65,76,44,61,36,160,50,61,254,237,56,61,10,56,63,61,189,126,69,61,173,186,75,61,105,241,81,61,24,30,88,61,76,48,94,61,228,55,100,61,84,34,106,61,182,253,111,61,133,177,117,61,112,69,123,61,103,97,128,61,137,3,131,61,136,148,133,61,254,15,136,61,218,117,138,61,226,180,140,61,235,223,142,61,32,236,144,61,151,215,146,61,171,158,148,61,212,62,150,61,239,193,151,61,10,20,153,61,22,62,154,61,120,61,155,61,90,14,156,61,159,171,156,61,203,25,157,61,113,82,157,61,97,85,157,61,162,31,157,61,177,174,156,61,141,1,156,61,162,24,155,61,62,236,153,61,220,129,152,61,226,208,150,61,236,215,148,61,235,153,146,61,101,21,144,61,66,62,141,61,138,36,138,61,239,177,134,61,83,245,130,61,15,202,125,61,80,2,117,61,33,149,107,61,235,119,97,61,56,171,86,61,34,39,75,61,114,255,62,61,208,20,50,61,244,111,36,61,185,21,22,61,97,255,6,61,55,62,238,60,77,13,205,60,38,100,170,60,164,26,134,60,3,183,64,60,162,47,228,59,45,137,1,59,225,72,82,187,29,110,12,188,212,124,103,188,138,226,162,188,48,138,211,188,72,225,2,189,8,201,28,189,243,107,55,189,33,221,82,189,241,21,111,189,236,5,134,189,196,224,148,189,5,31,164,189,196,184,179,189,120,177,195,189,22,9,212,189,229,185,228,189,166,198,245,189,148,149,3,190,104,114,12,190,231,122,21,190,5,174,30,190,192,8,40,190,190,138,49,190,114,52,59,190,162,3,69,190,220,249,78,190,155,15,89,190,221,74,99,190,133,163,109,190,239,29,120,190,21,91,129,190,190,181,134,190,156,29,140,190,148,146,145,190,162,19,151,190,56,160,156,190,68,55,162,190,228,214,167,190,171,128,173,190,19,50,179,190,213,233,184,62,136,168,190,62,207,107,196,62,28,51,202,62,156,253,207,62,178,203,213,62,90,154,219,62,39,105,225,62,18,56,231,62,68,5,237,62,72,208,242,62,198,150,248,62,191,88,254,62,24,11,2,63,71,229,4,63,196,188,7,63,182,143,10,63,81,93,13,63,241,37,16,63,254,231,18,63,111,164,21,63,72,89,24,63,47,7,27,63,88,172,29,63,174,73,32,63,223,221,34,63,148,104,37,63,43,233,39,63,32,95,42,63,152,202,44,63,12,42,47,63,246,125,49,63,237,197,51,63,59,0,54,63,161,45,56,63,125,77,58,63,199,94,60,63,255,97,62,63,25,86,64,63,177,58,66,63,228,15,68,63,201,212,69,63,165,136,71,63,129,44,73,63,200,190,74,63,173,63,76,63,206,174,77,63,133,11,79,63,115,86,80,63,77,142,81,63,225,178,82,63,47,197,83,63,57,195,84,63,125,174,85,63,81,133,86,63,197,72,87,63,187,247,87,63,68,146,88,63,153,24,89,63,55,139,89,63,14,232,89,63,164,48,90,63,230,100,90,63,179,131,90,63,156,142,90,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,249,250,251,252,253,254,255,0,1,2,3,4,5,6,7,251,252,253,254,255,0,1,2,3,4,5,6,7,9,11,13,251,253,254,255,0,1,2,3,4,5,6,7,9,11,13,16,250,252,254,255,0,1,2,3,4,5,6,7,9,11,13,16,252,254,255,0,1,2,3,4,5,6,7,9,11,13,16,20,254,255,0,1,2,3,4,5,6,7,9,11,13,16,20,24,28,224,169,63,31,177,151,63,244,87,143,63,0,0,0,0,171,170,170,62,148,94,154,62,211,103,95,62,4,219,235,61,179,96,2,61,0,0,0,0,54,60,53,63,0,0,128,63,28,206,180,63,249,2,21,80,0,0,0,0,0,0,64,63,102,102,102,63,72,225,122,63,64,31,0,0,16,39,0,0,64,31,1,7,6,11,10,1,0,6,0,0,0,0,0,0,3,0,0,0,16,39,0,0,224,46,0,0,64,31,1,11,7,13,12,1,0,6,0,0,0,0,0,0,3,0,0,0,224,46,0,0,129,62,0,0,64,31,1,14,10,13,13,1,0,6,0,0,0,0,0,0,3,0,0,0,128,62,0,0,192,93,0,0,64,31,1,14,10,14,14,2,0,3,0,0,0,0,0,0,2,0,0,0,192,93,0,0,0,125,0,0,64,31,1,14,10,14,14,2,0,3,0,0,0,0,0,0,2,0,0,0,0,125,0,0,129,187,0,0,64,31,1,14,11,15,15,2,0,3,0,0,0,0,0,0,2,0,0,0,64,31,0,0,16,39,0,0,17,43,1,5,4,6,6,1,0,6,0,0,0,0,0,0,3,0,0,0,16,39,0,0,224,46,0,0,17,43,1,8,5,12,9,1,0,6,0,0,0,0,0,0,3,0,0,0,224,46,0,0,128,62,0,0,17,43,1,12,8,13,8,1,0,6,0,0,0,0,0,0,3,0,0,0,128,62,0,0,32,78,0,0,17,43,1,12,8,13,8,1,0,6,0,0,0,0,0,0,3,0,0,0,32,78,0,0,193,93,0,0,17,43,1,13,9,13,8,1,0,6,0,0,0,0,0,0,3,0,0,0,192,93,0,0,0,125,0,0,17,43,1,14,10,14,9,2,0,3,0,0,0,0,0,0,2,0,0,0,0,125,0,0,128,187,0,0,17,43,1,15,11,15,10,2,0,3,0,0,0,0,0,0,2,0,0,0,128,187,0,0,1,250,0,0,17,43,1,15,11,15,10,2,0,3,0,0,0,0,0,0,1,0,0,0,64,31,0,0,16,39,0,0,224,46,1,4,3,6,6,1,0,6,0,0,0,0,0,0,3,0,0,0,16,39,0,0,224,46,0,0,224,46,1,7,4,11,8,1,0,6,0,0,0,0,0,0,3,0,0,0,224,46,0,0,128,62,0,0,224,46,1,11,7,12,8,1,0,6,0,0,0,0,0,0,3,0,0,0,128,62,0,0,32,78,0,0,224,46,1,11,7,12,8,1,0,6,0,0,0,0,0,0,3,0,0,0,32,78,0,0,193,93,0,0,224,46,1,12,8,12,8,1,0,6,0,0,0,0,0,0,3,0,0,0,192,93,0,0,0,125,0,0,224,46,1,13,9,13,9,2,0,3,0,0,0,0,0,0,2,0,0,0,0,125,0,0,128,187,0,0,224,46,1,14,10,14,10,2,0,3,0,0,0,0,0,0,2,0,0,0,128,187,0,0,1,250,0,0,224,46,1,15,11,15,11,2,0,3,0,0,0,0,0,0,1,0,0,0,64,31,0,0,16,39,0,0,128,62,1,1,1,0,0,1,0,6,0,0,0,0,0,0,3,0,0,0,16,39,0,0,224,46,0,0,128,62,1,2,1,6,0,1,0,6,0,0,0,0,0,0,3,0,0,0,224,46,0,0,128,62,0,0,128,62,1,4,2,6,0,1,0,6,0,0,0,0,0,0,3,0,0,0,128,62,0,0,80,70,0,0,128,62,1,4,2,8,3,1,0,6,0,0,0,0,0,0,3,0,0,0,80,70,0,0,240,85,0,0,128,62,1,6,5,11,7,2,0,6,0,0,0,0,0,0,2,0,0,0,240,85,0,0,96,109,0,0,128,62,1,10,9,12,8,2,0,6,0,0,0,0,0,0,2,0,0,0,96,109,0,0,160,140,0,0,128,62,1,12,12,13,13,2,0,3,0,0,0,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,128,62,1,14,14,13,13,2,0,3,0,0,0,0,0,0,1,0,0,0,224,171,0,0,1,250,0,0,128,62,1,15,15,13,13,2,0,3,0,0,0,0,0,0,1,0,0,0,105,44,0,0,128,62,0,0,34,86,1,3,1,4,4,1,0,6,0,0,0,0,0,0,3,0,0,0,128,62,0,0,80,70,0,0,34,86,1,3,1,5,4,1,0,6,0,0,0,0,0,0,3,0,0,0,80,70,0,0,240,85,0,0,34,86,1,4,4,8,5,2,0,6,0,0,0,0,0,0,2,0,0,0,240,85,0,0,96,109,0,0,34,86,1,7,6,8,6,2,0,6,0,0,0,0,0,0,2,0,0,0,96,109,0,0,160,140,0,0,34,86,1,10,10,9,9,2,0,3,0,0,0,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,34,86,1,11,11,10,10,2,0,3,0,0,0,0,0,0,1,0,0,0,224,171,0,0,1,250,0,0,34,86,1,13,13,12,12,2,0,3,0,0,0,0,0,0,1,0,0,0,224,46,0,0,128,62,0,0,192,93,1,3,1,4,4,1,0,6,0,0,0,0,0,0,3,0,0,0,128,62,0,0,80,70,0,0,192,93,1,3,1,5,4,1,0,6,0,0,0,0,0,0,3,0,0,0,80,70,0,0,240,85,0,0,192,93,1,4,3,8,5,2,0,6,0,0,0,0,0,0,2,0,0,0,240,85,0,0,96,109,0,0,192,93,1,7,6,8,6,2,0,6,0,0,0,0,0,0,2,0,0,0,96,109,0,0,160,140,0,0,192,93,1,10,10,9,9,2,0,3,0,0,0,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,192,93,1,11,11,10,10,2,0,3,0,0,0,0,0,0,1,0,0,0,224,171,0,0,1,250,0,0,192,93,1,13,13,11,11,2,0,3,0,0,0,0,0,0,1,0,0,0,192,93,0,0,160,140,0,0,0,125,1,4,4,4,4,2,0,3,0,0,0,0,0,0,3,0,0,0,160,140,0,0,96,234,0,0,0,125,1,7,7,6,6,2,0,3,0,0,0,0,0,0,2,0,0,0,96,234,0,0,64,25,1,0,0,125,1,9,9,8,8,2,0,3,0,0,0,0,0,0,1,0,0,0,64,25,1,0,160,134,1,0,0,125,1,11,11,10,10,2,0,3,0,0,0,0,0,0,1,0,0,0,160,134,1,0,1,113,2,0,0,125,1,13,13,11,11,2,0,3,0,0,0,0,0,0,1,0,0,0,192,93,0,0,160,140,0,0,68,172,1,4,4,4,4,2,0,3,0,0,0,0,0,0,3,0,0,0,160,140,0,0,96,234,0,0,68,172,1,7,7,6,6,2,0,3,0,0])
.concat([0,0,0,0,2,0,0,0,96,234,0,0,64,25,1,0,68,172,1,9,9,8,8,2,0,3,0,0,0,0,0,0,1,0,0,0,64,25,1,0,160,134,1,0,68,172,1,11,11,10,10,2,0,3,0,0,0,0,0,0,1,0,0,0,160,134,1,0,1,113,2,0,68,172,1,13,13,11,11,2,0,3,0,0,0,0,0,0,1,0,0,0,0,125,0,0,160,140,0,0,128,187,1,4,4,9,9,2,0,3,0,0,0,0,0,0,3,0,0,0,160,140,0,0,96,234,0,0,128,187,1,7,7,10,10,2,0,3,0,0,0,0,0,0,2,0,0,0,96,234,0,0,64,25,1,0,128,187,1,9,9,10,10,2,0,3,0,0,0,0,0,0,1,0,0,0,64,25,1,0,160,134,1,0,128,187,1,11,11,11,11,2,0,3,0,0,0,0,0,0,1,0,0,0,160,134,1,0,1,113,2,0,128,187,1,13,13,11,11,2,0,3,0,0,0,0,0,0,1,0,0,0,128,62,0,0,192,93,0,0,64,31,2,6,6,9,7,1,0,253,0,0,3,0,0,0,3,0,0,0,192,93,0,0,96,109,0,0,64,31,2,9,9,11,9,1,0,253,0,0,3,0,0,0,3,0,0,0,96,109,0,0,160,140,0,0,64,31,2,11,9,11,9,2,0,253,0,0,3,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,64,31,2,13,11,13,11,2,0,253,0,0,3,0,0,0,2,0,0,0,224,171,0,0,32,203,0,0,64,31,2,14,12,13,12,2,0,253,0,0,3,0,0,0,2,0,0,0,32,203,0,0,96,234,0,0,64,31,2,15,15,13,13,3,0,253,0,0,3,0,0,0,1,0,0,0,96,234,0,0,224,40,1,0,64,31,2,15,15,13,13,3,0,253,0,0,1,0,0,0,1,0,0,0,224,40,1,0,1,244,1,0,64,31,2,15,15,13,13,3,0,253,0,0,1,0,0,0,1,0,0,0,128,62,0,0,192,93,0,0,17,43,2,7,5,9,7,1,0,253,0,0,3,0,0,0,3,0,0,0,192,93,0,0,96,109,0,0,17,43,2,10,8,10,8,1,0,253,0,0,3,0,0,0,3,0,0,0,96,109,0,0,160,140,0,0,17,43,2,12,8,12,8,2,0,253,0,0,3,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,17,43,2,13,9,13,9,2,0,253,0,0,3,0,0,0,2,0,0,0,224,171,0,0,32,203,0,0,17,43,2,14,11,13,11,2,0,253,0,0,3,0,0,0,2,0,0,0,32,203,0,0,96,234,0,0,17,43,2,15,15,13,13,3,0,253,0,0,3,0,0,0,1,0,0,0,96,234,0,0,224,40,1,0,17,43,2,15,15,13,13,3,0,253,0,0,1,0,0,0,1,0,0,0,224,40,1,0,1,244,1,0,17,43,2,15,15,13,13,3,0,253,0,0,1,0,0,0,1,0,0,0,128,62,0,0,192,93,0,0,224,46,2,6,4,9,7,1,0,253,0,0,3,0,0,0,3,0,0,0,192,93,0,0,96,109,0,0,224,46,2,9,7,10,8,1,0,253,0,0,3,0,0,0,3,0,0,0,96,109,0,0,160,140,0,0,224,46,2,11,7,12,8,2,0,253,0,0,3,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,224,46,2,12,9,12,9,2,0,253,0,0,3,0,0,0,2,0,0,0,224,171,0,0,32,203,0,0,224,46,2,13,12,13,12,2,0,253,0,0,3,0,0,0,2,0,0,0,32,203,0,0,96,234,0,0,224,46,2,14,14,13,13,3,0,253,0,0,3,0,0,0,1,0,0,0,96,234,0,0,224,40,1,0,224,46,2,15,15,13,13,3,0,253,0,0,1,0,0,0,1,0,0,0,224,40,1,0,1,244,1,0,224,46,2,15,15,13,13,3,0,253,0,0,1,0,0,0,1,0,0,0,128,62,0,0,192,93,0,0,128,62,2,4,2,1,0,1,0,253,0,0,3,0,0,0,3,0,0,0,192,93,0,0,96,109,0,0,128,62,2,8,7,10,8,1,0,253,0,0,3,0,0,0,3,0,0,0,96,109,0,0,160,140,0,0,128,62,2,10,9,12,11,2,0,253,0,0,3,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,128,62,2,13,13,13,13,2,0,253,0,0,3,0,0,0,2,0,0,0,224,171,0,0,32,203,0,0,128,62,2,15,15,13,13,2,0,253,0,0,3,0,0,0,2,0,0,0,32,203,0,0,96,234,0,0,128,62,2,15,15,13,13,3,0,253,0,0,3,0,0,0,1,0,0,0,96,234,0,0,224,40,1,0,128,62,2,15,15,13,13,3,0,253,0,0,1,0,0,0,1,0,0,0,224,40,1,0,1,244,1,0,128,62,2,15,15,13,13,3,0,253,0,0,1,0,0,0,1,0,0,0,128,62,0,0,192,93,0,0,34,86,2,2,1,1,0,1,0,253,0,0,3,0,0,0,3,0,0,0,192,93,0,0,96,109,0,0,34,86,2,5,4,6,5,1,0,253,0,0,3,0,0,0,3,0,0,0,96,109,0,0,0,125,0,0,34,86,2,5,4,8,7,2,0,253,0,0,3,0,0,0,2,0,0,0,0,125,0,0,160,140,0,0,34,86,2,7,6,8,7,2,0,253,0,0,3,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,34,86,2,10,10,9,9,2,0,253,0,0,3,0,0,0,2,0,0,0,224,171,0,0,32,203,0,0,34,86,2,12,12,9,9,3,0,253,0,0,3,0,0,0,2,0,0,0,32,203,0,0,96,234,0,0,34,86,2,13,13,10,10,3,0,253,0,0,3,0,0,0,1,0,0,0,96,234,0,0,224,40,1,0,34,86,2,14,14,12,12,3,0,253,0,0,1,0,0,0,1,0,0,0,224,40,1,0,1,244,1,0,34,86,2,14,14,12,12,3,0,253,0,0,1,0,0,0,1,0,0,0,128,62,0,0,192,93,0,0,192,93,2,2,1,1,0,1,0,253,0,0,3,0,0,0,3,0,0,0,192,93,0,0,96,109,0,0,192,93,2,5,5,6,6,1,0,253,0,0,3,0,0,0,3,0,0,0,96,109,0,0,160,140,0,0,192,93,2,7,6,8,7,2,0,253,0,0,3,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,192,93,2,10,10,9,9,2,0,253,0,0,3,0,0,0,2,0,0,0,224,171,0,0,32,203,0,0,192,93,2,12,12,9,9,3,0,253,0,0,3,0,0,0,2,0,0,0,32,203,0,0,96,234,0,0,192,93,2,13,13,10,10,3,0,253,0,0,3,0,0,0,1,0,0,0,96,234,0,0,224,40,1,0,192,93,2,14,14,12,12,3,0,253,0,0,1,0,0,0,1,0,0,0,224,40,1,0,1,244,1,0,192,93,2,15,15,12,12,3,0,253,0,0,1,0,0,0,1,0,0,0,0,125,0,0,96,234,0,0,0,125,2,4,4,4,4,2,0,253,0,0,3,0,0,0,3,0,0,0,96,234,0,0,128,56,1,0,0,125,2,7,7,6,6,3,0,253,0,0,3,0,0,0,2,0,0,0,128,56,1,0,128,181,1,0,0,125,2,9,9,8,8,3,0,253,0,0,1,0,0,0,1,0,0,0,128,181,1,0,128,50,2,0,0,125,2,11,11,10,10,3,0,253,0,0,1,0,0,0,1,0,0,0,128,50,2,0,1,232,3,0,0,125,2,13,13,11,11,3,0,253,0,0,1,0,0,0,1,0,0,0,0,125,0,0,96,234,0,0,68,172,2,4,4,4,4,2,0,253,0,0,3,0,0,0,3,0,0,0,96,234,0,0,128,56,1,0,68,172,2,7,7,6,6,3,0,253,0,0,3,0,0,0,2,0,0,0,128,56,1,0,128,181,1,0,68,172,2,9,9,8,8,3,0,253,0,0,1,0,0,0,1,0,0,0,128,181,1,0,128,50,2,0,68,172,2,11,11,10,10,3,0,253,0,0,1,0,0,0,1,0,0,0,128,50,2,0,1,232,3,0,68,172,2,13,13,11,11,3,0,253,0,0,1,0,0,0,1,0,0,0,160,140,0,0,96,234,0,0,128,187,2,4,4,9,9,2,0,253,0,0,3,0,0,0,3,0,0,0,96,234,0,0,128,56,1,0,128,187,2,7,7,9,9,3,0,253,0,0,3,0,0,0,2,0,0,0,128,56,1,0,128,181,1,0,128,187,2,9,9,10,10,3,0,253,0,0,1,0,0,0,1,0,0,0,128,181,1,0,128,50,2,0,128,187,2,11,11,11,11,3,0,253,0,0,1,0,0,0,1,0,0,0,128,50,2,0,1,232,3,0,128,187,2,13,13,11,11,3,0,253,0,0,1,0,0,0,1,0,0,0,80,70,0,0,240,85,0,0,34,86,1,4,4,5,5,2,0,6,0,0,0,0,0,0,2,0,0,0,240,85,0,0,96,109,0,0,34,86,1,4,4,6,5,2,0,6,0,0,0,0,0,0,2,0,0,0,96,109,0,0,160,140,0,0,34,86,1,7,8,8,8,2,0,3,0,0,0,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,34,86,1,9,9,9,9,2,0,3,0,0,0,0,0,0,1,0,0,0,224,171,0,0,32,203,0,0,34,86,1,11,11,11,11,2,0,3,0,0,0,0,0,0,1,0,0,0,32,203,0,0,1,250,0,0,34,86,1,12,11,11,11,2,0,3,0,0,0,0,0,0,1,0,0,0,32,78,0,0,240,85,0,0,192,93,1,4,4,5,5,2,0,6,0,0,0,0,0,0,2,0,0,0,240,85,0,0,96,109,0,0,192,93,1,4,4,6,5,2,0,6,0,0,0,0,0,0,2,0,0,0,96,109,0,0,160,140,0,0,192,93,1,6,8,8,8,2,0,3,0,0,0,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,192,93,1,8,9,9,9,2,0,3,0,0,0,0,0,0,1,0,0,0,224,171,0,0,32,203,0,0,192,93,1,12,11,11,10,2,0,3,0,0,0,0,0,0,1,0,0,0,32,203,0,0,1,250,0,0,192,93,1,13,11,11,10,2,0,3,0,0,0,0,0,0,1,0,0,0,0,125,0,0,160,140,0,0,34,86,2,5,4,7,6,2,0,253,0,0,3,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,34,86,2,5,8,8,8,2,0,253,0,0,3,0,0,0,2,0,0,0,224,171,0,0,32,203,0,0,34,86,2,7,10,8,8,3,0,253,0,0,3,0,0,0,2,0,0,0,32,203,0,0,96,234,0,0,34,86,2,9,11,9,9,3,0,253,0,0,3,0,0,0,1,0,0,0,96,234,0,0,224,40,1,0,34,86,2,10,12,10,11,3,0,253,0,0,1,0,0,0,1,0,0,0,224,40,1,0,80,64,1,0,34,86,2,12,12,11,11,3,0,253,0,0,1,0,0,0,1,0,0,0,80,64,1,0,1,244,1,0,34,86,2,13,12,11,11,3,0,253,0,0,1,0,0,0,1,0,0,0,0,125,0,0,160,140,0,0,192,93,2,5,4,7,6,2,0,253,0,0,3,0,0,0,2,0,0,0,160,140,0,0,224,171,0,0,192,93,2,4,8,8,8,2,0,253,0,0,3,0,0,0,2,0,0,0,224,171,0,0,32,203,0,0,192,93,2,6,10,8,8,3,0,253,0,0,3,0,0,0,2,0,0,0,32,203,0,0,96,234,0,0,192,93,2,9,11,9,9,3,0,253,0,0,3,0,0,0,1,0,0,0,96,234,0,0,224,40,1,0,192,93,2,11,12,10,11,3,0,253,0,0,1,0,0,0,1,0,0,0,224,40,1,0,192,87,1,0,192,93,2,12,13,11,11,3,0,253,0,0,1,0,0,0,1,0,0,0,192,87,1,0,1,244,1,0,192,93,2,13,13,11,11,3,0,253,0,0,1,0,0,0,1,0,0,0,117,56,0,0,0,0,0,0,8,0,0,0,0,0,0,0,5,0,0,0,115,49,54,0,0,0,0,0,16,0,0,0,0,0,0,0,6,0,0,0,115,51,50,0,0,0,0,0,32,0,0,0,0,0,0,0,7,0,0,0,102,108,116,0,0,0,0,0,32,0,0,0,0,0,0,0,8,0,0,0,100,98,108,0,0,0,0,0,64,0,0,0,0,0,0,0,9,0,0,0,117,56,112,0,0,0,0,0,8,0,0,0,1,0,0,0,0,0,0,0,115,49,54,112,0,0,0,0,16,0,0,0,1,0,0,0,1,0,0,0,115,51,50,112,0,0,0,0,32,0,0,0,1,0,0,0,2,0,0,0,102,108,116,112,0,0,0,0,32,0,0,0,1,0,0,0,3,0,0,0,100,98,108,112,0,0,0,0,64,0,0,0,1,0,0,0,4,0,0,0,0,128,64,192,32,160,96,224,16,144,80,208,48,176,112,240,8,136,72,200,40,168,104,232,24,152,88,216,56,184,120,248,4,132,68,196,36,164,100,228,20,148,84,212,52,180,116,244,12,140,76,204,44,172,108,236,28,156,92,220,60,188,124,252,2,130,66,194,34,162,98,226,18,146,82,210,50,178,114,242,10,138,74,202,42,170,106,234,26,154,90,218,58,186,122,250,6,134,70,198,38,166,102,230,22,150,86,214,54,182,118,246,14,142,78,206,46,174,110,238,30,158,94,222,62,190,126,254,0,0,0,0,0,0,0,0,0,0,0,0,80,67,77,85,0,0,0,0,1,0,0,0,6,0,1,0,64,31,0,0,1,0,0,0,3,0,0,0,71,83,77,0,0,0,0,0,1,0,0,0,0,0,0,0,64,31,0,0,1,0,0,0,4,0,0,0,71,55,50,51,0,0,0,0,1,0,0,0,53,80,1,0,64,31,0,0,1,0,0,0,5,0,0,0,68,86,73,52,0,0,0,0,1,0,0,0,0,0,0,0,64,31,0,0,1,0,0,0,6,0,0,0,68,86,73,52,0,0,0,0,1,0,0,0,0,0,0,0,128,62,0,0,1,0,0,0,7,0,0,0,76,80,67,0,0,0,0,0,1,0,0,0,0,0,0,0,64,31,0,0,1,0,0,0,8,0,0,0,80,67,77,65,0,0,0,0,1,0,0,0,7,0,1,0,64,31,0,0,1,0,0,0,9,0,0,0,71,55,50,50,0,0,0,0,1,0,0,0,28,16,1,0,64,31,0,0,1,0,0,0,10,0,0,0,76,49,54,0,0,0,0,0,1,0,0,0,1,0,1,0,68,172,0,0,2,0,0,0,11,0,0,0,76,49,54,0,0,0,0,0,1,0,0,0,1,0,1,0,68,172,0,0,1,0,0,0,12,0,0,0,81,67,69,76,80,0,0,0,1,0,0,0,24,80,1,0,64,31,0,0,1,0,0,0,13,0,0,0,67,78,0,0,0,0,0,0,1,0,0,0,0,0,0,0,64,31,0,0,1,0,0,0,14,0,0,0,77,80,65,0,0,0,0,0,1,0,0,0,0,80,1,0,255,255,255,255,255,255,255,255,14,0,0,0,77,80,65,0,0,0,0,0,1,0,0,0,1,80,1,0,255,255,255,255,255,255,255,255,15,0,0,0,71,55,50,56,0,0,0,0,1,0,0,0,0,0,0,0,64,31,0,0,1,0,0,0,16,0,0,0,68,86,73,52,0,0,0,0,1,0,0,0,0,0,0,0,17,43,0,0,1,0,0,0,17,0,0,0,68,86,73,52,0,0,0,0,1,0,0,0,0,0,0,0,34,86,0,0,1,0,0,0,18,0,0,0,71,55,50,57,0,0,0,0,1,0,0,0,0,0,0,0,64,31,0,0,1,0,0,0,25,0,0,0,67,101,108,66,0,0,0,0,0,0,0,0,0,0,0,0,144,95,1,0,255,255,255,255,26,0,0,0,74,80,69,71,0,0,0,0,0,0,0,0,8,0,0,0,144,95,1,0,255,255,255,255,28,0,0,0,110,118,0,0,0,0,0,0,0,0,0,0,0,0,0,0,144,95,1,0,255,255,255,255,31,0,0,0,72,50,54,49,0,0,0,0,0,0,0,0,4,0,0,0,144,95,1,0,255,255,255,255,32,0,0,0,77,80,86,0,0,0,0,0,0,0,0,0,1,0,0,0,144,95,1,0,255,255,255,255,32,0,0,0,77,80,86,0,0,0,0,0,0,0,0,0,2,0,0,0,144,95,1,0,255,255,255,255,33,0,0,0,77,80,50,84,0,0,0,0,2,0,0,0,0,0,2,0,144,95,1,0,255,255,255,255,34,0,0,0,72,50,54,51,0,0,0,0,0,0,0,0,5,0,0,0,144,95,1,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,46,0,0,0,90,0,0,0,86,0,0,0,4,0,0,0,36,0,0,0,10,0,0,0,2,0,0,0,30,0,0,0,46,0,0,0,90,0,0,0,86,0,0,0,4,0,0,0,36,0,0,0,28,0,0,0,6,0,0,0,12,0,0,0,0,0,0,0,90,0,0,0,86,0,0,0,4,0,0,0,36,0,0,0,0,0,0,0,0,0,0,0,34,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,232,3,0,0,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,191,0,0,0,0,0,64,191,64,0,0,0,0,0,88,203,64,0,0,0,0,0,130,228,64,0,0,0,0,0,112,183,64,0,0,0,0,0,148,193,64,0,0,0,0,0,64,223,64,0,0,0,0,0,88,203,64,0,0,0,0,0,136,211,64,0,0,0,0,0,88,219,64,0,0,0,0,0,142,226,64,0,0,0,0,0,118,230,64,0,0,0,0,0,94,234,64,0,0,0,0,128,79,242,64,0,0,0,0,0,112,247,64,0,0,0,0,0,76,253,64,0,0,0,0,0,23,1,65,0,0,0,0,0,249,5,65,0,0,0,0,8,76,13,65,0,0,0,0,0,64,223,64,0,0,0,0,0,112,231,64,0,0,0,0,0,76,237,64,0,0,0,0,0,23,241,64,0,0,0,0,0,136,243,64,0,0,0,0,0,255,244,64,0,0,0,0,0,112,247,64,0,0,0,0,0,219,250,64,0,0,0,0,0,76,253,64,0,0,0,0,0,23,1,65,0,0,0,0,0,136,3,65,0,0,0,0,8,76,13,65,0,0,0,0,0,249,213,64,0,0,0,0,0,64,223,64,0,0,0,0,0,136,227,64,0,0,0,0,0,112,231,64,0,0,0,0,0,88,235,64,0,0,0,0,0,64,239,64,0,0,0,0,0,136,243,64,0,0,0,0,0,112,247,64,0,0,0,0,0,88,251,64,0,0,0,0,0,64,255,64,0,0,0,0,0,136,3,65,0,0,0,0,136,132,14,65,0,0,0,0,0,76,221,64,0,0,0,0,0,130,228,64,0,0,0,0,0,100,233,64,0,0,0,0,0,64,239,64,0,0,0,0,0,148,241,64,0,0,0,0,0,11,243,64,0,0,0,0,0,255,244,64,0,0,0,0,0,118,246,64,0,0,0,0,0,219,250,64,0,0,0,0,0,76,253,64,0,0,0,0,0,23,1,65,0,0,0,0,128,49,7,65,0,0,0,0,0,148,209,64,0,0,0,0,0,88,219,64,0,0,0,0,0,23,225,64,0,0,0,0,0,249,229,64,0,0,0,0,0,88,235,64,0,0,0,0,0,76,237,64,0,0,0,0,128,79,242,64,0,0,0,0,0,249,245,64,0,0,0,0,0,106,248,64,0,0,0,0,128,19,252,64,0,0,0,0,128,79,2,65,0,0,0,0,128,49,7,65,0,0,0,0,0,64,207,64,0,0,0,0,0,88,219,64,0,0,0,0,0,106,232,64,0,0,0,0,0,249,245,64,0,0,0,0,0,76,205,64,0,0,0,0,0,136,211,64,0,0,0,0,0,124,229,64,0,0,0,0,0,255,244,64,0,0,0,0,0,64,207,64,0,0,0,0,0,88,219,64,0,0,0,0,0,64,239,64,0,0,0,0,0,106,248,64,0,0,0,0,0,112,199,64,0,0,0,0,0,136,211,64,0,0,0,0,0,124,229,64,0,0,0,0,0,255,244,64,0,0,0,0,0,112,199,64,0,0,0,0,0,136,211,64,0,0,0,0,0,106,232,64,0,0,0,0,0,64,191,64,0,0,0,0,0,100,201,64,0,0,0,0,0,124,229,64,154,153,153,153,153,153,185,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,154,153,153,153,153,153,185,191,0,0,0,0,0,0,0,0,154,153,153,153,153,153,185,63,154,153,153,153,153,153,201,63,51,51,51,51,51,51,211,63,154,153,153,153,153,153,217,63,0,0,0,0,0,0,224,63,51,51,51,51,51,51,227,63,102,102,102,102,102,102,230,63,154,153,153,153,153,153,233,63,205,204,204,204,204,204,236,63,0,0,0,0,0,0,240,63,154,153,153,153,153,153,185,191,154,153,153,153,153,153,169,63,0,0,0,0,0,0,224,63,0,0,0,0,0,0,240,63,154,153,153,153,153,153,185,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,130,90,130,90,126,165,126,165,130,90,130,90,126,165,126,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,36,3,107,9,171,15,226,21,12,28,36,34,39,40,17,46,223,51,141,57,23,63,123,68,180,73,192,78,155,83,67,88,180,92,236,96,233,100,167,104,36,108,95,111,85,114,5,117,108,119,138,121,93,123,228,124,30,126,10,127,167,127,246,127,246,127,167,127,10,127,30,126,228,124,93,123,138,121,108,119,5,117,85,114,95,111,36,108,167,104,233,100,236,96,180,92,67,88,155,83,192,78,180,73,123,68,23,63,141,57,223,51,17,46,39,40,36,34,12,28,226,21,171,15,107,9,36,3,130,90,130,90,126,165,126,165,130,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,182,4,28,14,110,23,159,32,164,41,110,50,243,58,38,67,251,74,105,82,100,89,228,95,222,101,75,107,35,112,96,116,251,119,239,122,58,125,214,126,194,127,254,127,135,127,96,126,137,124,6,122,217,118,8,115,151,110,140,105,239,99,200,93,130,90,126,165,126,165,130,90,130,90,126,165,126,165,130,90,130,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,131,90,126,165,125,165,130,90,246,127,167,127,10,127,30,126,228,124,93,123,138,121,108,119,5,117,85,114,95,111,36,108,167,104,233,100,236,96,180,92,67,88,155,83,192,78,180,73,123,68,23,63,141,57,223,51,17,46,39,40,36,34,12,28,226,21,171,15,107,9,36,3,220,252,149,246,85,240,30,234,244,227,220,221,217,215,239,209,33,204,115,198,233,192,133,187,76,182,64,177,101,172,189,167,76,163,20,159,23,155,89,151,220,147,161,144,171,141,251,138,148,136,118,134,163,132,28,131,226,129,246,128,89,128,10,128,126,165,130,90,130,90,126,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,126,165,131,90,130,90,125,165,234,127,56,127,214,125,198,123,10,121,166,117,158,113,249,108,189,103,241,97,157,91,202,84,129,77,205,69,184,61,78,53,153,44,167,35,131,26,58,17,217,7,110,254,5,245,171,235,109,226,88,217,121,208,219,199,140,191,150,183,5,176,226,168,0,0,0,0,236,255,147,0,211,32,0,0,0,0,236,255,158,0,235,33,0,0,0,0,237,255,169,0,3,35,0,0,0,0,238,255,181,0,27,36,0,0,0,0,239,255,194,0,50,37,0,0,0,0,240,255,208,0,73,38,0,0,0,0,241,255,223,0,94,39,0,0,0,0,240,255,239,0,114,40,0,0,0,0,239,255,255,0,131,41,0,0,0,0,237,255,17,1,147,42,0,0,0,0,235,255,35,1,160,43,0,0,0,0,236,255,55,1,171,44,0,0,0,0,240,255,76,1,178,45,0,0,0,0,243,255,97,1,182,46,0,0,0,0,245,255,120,1,182,47,0,0,0,0,247,255,144,1,179,48,0,0,0,0,248,255,170,1,171,49,0,0,0,0,251,255,196,1,158,50,0,0,0,0,1,0,224,1,141,51,0,0,1,0,8,0,254,1,119,52,0,0,1,0,13,0,28,2,91,53,0,0,1,0,18,0,60,2,58,54,0,0,255,255,23,0,94,2,19,55,0,0,1,0,28,0,128,2,230,55,0,0,1,0,32,0,164,2,179,56,0,0,1,0,33,0,202,2,121,57,0,0,1,0,37,0,242,2,55,58,0,0,1,0,40,0,27,3,239,58,0,0,1,0,43,0,69,3,158,59,0,0,1,0,44,0,112,3,70,60,0,0,1,0,45,0,156,3,230,60,0,0,4,0,45,0,201,3,127,61,0,0,10,0,41,0,247,3,15,62,0,0,1,0,36,0,38,4,151,62,0,0,1,0,31,0,85,4,22,63,0,0,1,0,26,0,134,4,142,63,0,0,255,255,20,0,182,4,253,63,0,0,255,255,11,0,231,4,100,64,0,0,0,0,2,0,23,5,194,64,0,0,255,255,5,0,72,5,24,65,0,0,1,0,9,0,120,5,101,65,0,0,3,0,11,0,169,5,169,65,0,0,0,0,12,0,216,5,229,65,0,0,0,0,12,0,8,6,24,66,0,0,0,0,12,0,54,6,67,66,0,0,0,0,11,0,100,6,101,66,0,0,0,0,11,0,144,6,128,66,0,0,255,255,11,0,187,6,148,66,0,0,255,255,11,0,229,6,160,66,0,0,255,255,10,0,12,7,167,66,0,0,255,255,8,0,50,7,167,66,0,0,255,255,5,0,86,7,161,66,0,0,255,255,2,0,119,7,149,66,0,0,254,255,254,255,150,7,131,66,0,0,252,255,249,255,178,7,105,66,0,0,252,255,243,255,204,7,73,66,0,0,253,255,237,255,227,7,32,66,0,0,253,255,231,255,247,7,239,65,0,0,253,255,225,255,8,8,180,65,0,0,253,255,218,255,21,8,112,65,0,0,253,255,212,255,32,8,34,65,0,0,253,255,207,255,39,8,205,64,0,0,253,255,202,255,42,8,113,64,0,0,253,255,197,255,40,8,24,64,0,0,255,255,193,255,39,8,137,63,0,0,0,0,188,255,37,8,197,62,0,0,0,0,185,255,30,8,4,62,0,0,0,0,182,255,18,8,62,61,0,0,0,0,180,255,2,8,113,60,0,0,0,0,179,255,238,7,156,59,0,0,0,0,179,255,212,7,192,58,0,0,1,0,180,255,182,7,221,57,0,0,1,0,181,255,147,7,245,56,0,0,1,0,184,255,106,7,8,56,0,0,0,0,189,255,60,7,23,55,0,0,0,0,194,255,8,7,34,54,0,0,1,0,199,255,206,6,44,53,0,0,0,0,206,255,143,6,52,52,0,0,0,0,213,255,74,6,58,51,0,0,0,0,221,255,255,5,63,50,0,0,0,0,230,255,174,5,67,49,0,0,0,0,239,255,88,5,69,48,0,0,1,0,246,255,251,4,70,47,0,0,1,0,253,255,153,4,69,46,0,0,2,0,3,0,49,4,66,45,0,0,2,0,9,0,196,3,61,44,0,0,2,0,15,0,80,3,53,43,0,0,0,0,22,0,215,2,44,42,0,0,1,0,30,0,89,2,32,41,0,0,1,0,36,0,212,1,19,40,0,0,1,0,33,0,74,1,5,39,0,0,1,0,28,0,186,0,245,37,0,0,1,0,24,0,36,0,229,36,0,0,2,0,20,0,136,255,212,35,0,0,3,0,15,0,232,254,194,34,0,0,0,0,10,0,65,254,176,33,1,0,250,255,10,0,149,253,157,32,0,0,253,255,7,0,228,252,137,31,0,0,254,255,2,0,45,252,117,30,0,0,255,255,255,255,113,251,96,29,0,0,1,0,251,255,176,250,74,28,0,0,1,0,247,255,234,249,51,27,0,0,0,0,243,255,31,249,29,26,0,0,2,0,240,255,79,248,7,25,0,0,1,0,237,255,122,247,241,23,0,0,254,255,235,255,161,246,220,22,0,0,0,0,232,255,194,245,200,21,0,0,0,0,228,255,224,244,181,20,0,0,0,0,224,255,248,243,163,19,0,0,1,0,221,255,13,243,146,18,0,0,1,0,217,255,29,242,129,17,0,0,2,0,213,255,41,241,112,16,0,0,3,0,208,255,49,240,93,15,0,0,3,0,203,255,53,239,73,14,0,0,4,0,199,255,54,238,51,13,0,0,4,0,194,255,51,237,27,12,0,0,4,0,188,255,44,236,2,11,0,0,5,0,183,255,35,235,231,9,0,0,10,0,178,255,22,234,207,8,0,0,13,0,173,255,7,233,185,7,0,0,15,0,168,255,245,231,171,6,0,0,17,0,162,255,225,230,167,5,0,0,18,0,156,255,202,229,176,4,0,0,18,0,149,255,179,228,202,3,0,0,19,0,143,255,154,227,247,2,255,255,20,0,135,255,127,226,53,2,255,255,21,0,127,255,100,225,130,1,255,255,21,0,118,255,72,224,185,0,0,0,0,0,236,255,153,0,95,33,0,0,0,0,238,255,175,0,143,35,0,0,0,0,239,255,201,0,190,37,0,0,0,0,240,255,231,0,232,39,0,0,0,0,238,255,8,1,11,42,0,0,0,0,236,255,45,1,38,44,0,0,0,0,241,255,86,1,52,46,0,0,0,0,246,255,132,1,53,48,0,0,0,0,250,255,183,1,36,50,0,0,1,0,5,0,239,1,2,52,0,0,1,0,16,0,44,2,203,53,0,0,0,0,26,0,111,2,125,55,0,0,1,0,33,0,183,2,22,57,0,0,1,0,39,0,6,3,147,58,0,0,1,0,44,0,90,3,242,59,0,0,3,0,45,0,178,3,51,61,0,0,5,0,39,0,14,4,83,62,0,0,1,0,28,0,109,4,82,63,0,0,255,255,15,0,206,4,49,64,0,0,255,255,3,0,48,5,237,64,0,0,2,0,10,0,145,5,135,65,0,0,0,0,12,0,240,5,254,65,0,0,0,0,12,0,77,6,84,66,0,0,0,0,11,0,166,6,138,66,0,0,255,255,10,0,248,6,164,66,0,0,255,255,7,0,68,7,164,66,0,0,255,255,0,0,134,7,140,66,0,0,252,255,246,255,191,7,89,66,0,0,253,255,234,255,237,7,7,66,0,0,253,255,222,255,14,8,146,65,0,0,253,255,210,255,35,8,248,64,0,0,253,255,199,255,41,8,68,64,0,0,0,0,190,255,38,8,39,63,0,0,0,0,183,255,24,8,161,61,0,0,0,0,179,255,248,7,6,60,0,0,1,0,179,255,197,7,79,58,0,0,1,0,183,255,126,7,126,56,0,0,0,0,191,255,34,7,156,54,0,0,0,0,203,255,175,6,176,52,0,0,0,0,217,255,36,6,189,50,0,0,0,0,234,255,131,5,196,48,0,0,1,0,249,255,202,4,198,46,0,0,2,0,6,0,250,3,191,44,0,0,1,0,19,0,20,3,176,42,0,0,1,0,33,0,22,2,154,40,0,0,1,0,30,0,2,1,125,38,0,0,1,0,22,0,214,255,92,36,0,0,1,0,12,0,148,254,57,34,0,0,252,255,9,0,61,253,19,32,0,0,255,255,0,0,207,251,234,29,0,0,1,0,249,255,77,250,191,27,0,0,1,0,242,255,183,248,146,25,0,0,0,0,236,255,14,247,102,23,0,0,0,0,230,255,81,245,62,21,0,0,0,0,223,255,130,243,26,19,0,0,1,0,215,255,163,241,248,16,0,0,3,0,206,255,179,239,211,14,0,0,4,0,196,255,180,237,167,12,0,0,5,0,186,255,168,235,116,10,0,0,12,0,176,255,142,233,68,8,0,0,16,0,165,255,107,231,41,6,0,0,18,0,153,255,63,229,61,4,255,255,20,0,139,255,12,227,150,2,255,255,21,0,123,255,214,224,29,1,0,0,179,1,58,46,198,209,77,254,238,255,216,1,170,47,51,211,113,254,238,255,253,1,27,49,160,212,147,254,240,255,36,2,141,50,10,214,181,254,240,255,76,2,255,51,114,215,213,254,240,255,117,2,115,53,216,216,244,254,239,255,158,2,231,54,59,218,18,255,239,255,201,2,90,56,155,219,47,255,238,255,244,2,206,57,249,220,75,255,237,255,32,3,65,59,83,222,101,255,237,255,77,3,180,60,169,223,127,255,236,255,123,3,38,62,252,224,151,255,235,255,169,3,150,63,76,226,175,255,235,255,217,3,6,65,151,227,197,255,234,255,8,4,115,66,222,228,218,255,233,255,57,4,222,67,33,230,238,255,233,255,105,4,72,69,96,231,1,0,232,255,155,4,175,70,153,232,19,0,232,255,204,4,19,72,207,233,36,0,231,255,254,4,116,73,255,234,52,0,231,255,48,5,210,74,42,236,66,0,231,255,98,5,45,76,81,237,80,0,230,255,149,5,132,77,114,238,93,0,231,255,199,5,214,78,141,239,105,0,230,255,250,5,37,80,164,240,116,0,230,255,44,6,111,81,180,241,126,0,231,255,94,6,180,82,191,242,135,0,231,255,144,6,245,83,197,243,144,0,231,255,193,6,48,85,196,244,151,0,232,255,242,6,101,86,190,245,158,0,232,255,34,7,149,87,178,246,163,0,233,255,81,7,191,88,160,247,168,0,234,255,128,7,227,89,135,248,173,0,235,255,174,7,0,91,105,249,176,0,237,255,218,7,23,92,69,250,179,0,238,255,6,8,39,93,26,251,182,0,239,255,48,8,47,94,233,251,183,0,241,255,89,8,49,95,178,252,184,0,243,255,129,8,43,96,116,253,185,0,245,255,167,8,29,97,49,254,185,0,247,255,203,8,8,98,231,254,184,0,249,255,238,8,234,98,151,255,183,0,251,255,15,9,196,99,65,0,182,0,254,255,45,9,150,100,228,0,180,0,0,0,74,9,95,101,129,1,178,0,4,0,100,9,32,102,24,2,175,0,7,0,124,9,215,102,170,2,172,0,10,0,145,9,134,103,52,3,168,0,13,0,164,9,43,104,185,3,165,0,17,0,180,9,199,104,56,4,161,0,20,0,193,9,89,105,177,4,157,0,24,0,203,9,227,105,35,5,153,0,28,0,210,9,98,106,145,5,148,0,32,0,213,9,215,106,248,5,143,0,37,0,213,9,67,107,89,6,138,0,41,0,210,9,164,107,181,6,134,0,46,0,203,9,252,107,12,7,128,0,51,0,192,9,73,108,93,7,123,0,55,0,178,9,140,108,168,7,118,0,60,0,159,9,198,108,238,7,113,0,65,0,136,9,244,108,47,8,107,0,70,0,109,9,24,109,107,8,102,0,75,0,77,9,50,109,162,8,97,0,81,0,42,9,66,109,212,8,91,0,86,0,1,9,71,109,1,9,86,0,77,254,198,209,58,46,179,1,0,0,0,0,0,0,205,204,236,192,205,204,236,192,205,204,236,192,0,0,24,193,205,204,236,192,51,51,195,192,0,0,176,192,102,102,150,192,102,102,150,192,102,102,150,192,102,102,150,192,0,0,0,0,0,0,0,0,243,4,53,63,0,0,128,63,243,4,53,63,0,0,0,0,243,4,53,191,0,0,128,191,243,4,53,191,0,0,128,63,243,4,53,63,0,0,0,0,243,4,53,191,0,0,128,191,243,4,53,191,0,0,0,0,243,4,53,63,195,85,102,61,249,233,0,62,155,80,76,62,155,232,161,62,182,179,228,62,108,134,33,63,24,89,75,63,0,0,128,63,120,36,161,63,200,221,202,63,53,71,15,64,194,98,74,64,66,97,160,64,94,47,254,64,42,67,142,65,55,62,79,59,156,68,184,59,10,215,35,60,57,173,145,60,226,134,1,61,195,85,102,61,173,173,162,61,21,202,229,61,6,75,34,62,187,62,101,62,155,232,161,62,180,212,203,62,206,77,0,63,108,134,33,63,24,89,75,63,0,0,128,63,120,36,161,63,200,221,202,63,193,100,255,63,191,194,32,64,194,98,74,64,82,240,142,64,7,232,201,64,163,153,14,65,150,109,73,65,42,67,142,65,114,251,252,65,192,239,96,66,0,0,200,66,244,211,49,67,39,29,158,67,0,0,128,63,59,223,111,63,147,87,87,63,229,213,25,63,79,59,188,62,0,0,0,0,180,200,22,191,0,0,128,191,2,6,10,14,18,22,26,30,34,246,250,254,51,57,15,21,27,33,39,45,54,66,78,42,102,66,78,90,102,114,126,90,253,255,1,3,5,7,10,14,18,22,0,0,0,0,0,0,0,0,0,0,188,181,182,62,253,66,18,63,210,23,109,63,175,223,152,63,219,15,201,63,233,214,12,64,219,15,73,64,64,31,0,0,240,85,0,0,10,0,0,0,1,0,0,0,0,0,0,96,240,85,0,0,96,109,0,0,20,0,0,0,1,0,0,0,0,0,0,64,96,109,0,0,160,140,0,0,20,0,0,0,2,0,0,0,0,0,0,48,160,140,0,0,1,113,2,0,20,0,0,0,4,0,0,0,64,51,51,35,102,102,109,112,101,103,0,0,208,7,0,0,0,0,0,0,1,0,0,0,56,115,25,0,4,0,0,0,104,110,25,0,28,0,0,0,32,105,25,0,22,0,0,0,136,99,25,0,38,0,0,0,184,93,25,0,157,255,255,255,0,0,0,0,255,255,255,255,0,0,0,0,89,85,86,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,36,64,0,0,0,0,0,0,89,64,0,0,0,0,0,136,195,64,0,0,0,0,132,215,151,65,0,128,224,55,121,195,65,67,23,110,5,181,181,184,147,70,245,249,63,233,3,79,56,77,50,29,48,249,72,119,130,90,60,191,115,127,221,79,21,117,83,119,58,57,229,83,87,57,20,168,120,57,114,146,143,57,95,203,165,57,215,116,191,57,38,23,221,57,173,79,255,57,22,106,19,58,67,59,42,58,140,148,68,58,226,1,99,58,111,18,131,58,14,92,151,58,142,201,174,58,92,215,201,58,40,21,233,58,111,148,6,59,13,105,27,59,15,119,51,59,55,62,79,59,13,82,111,59,119,46,138,59,204,145,159,59,156,68,184,59,20,202,212,59,176,185,245,59,45,225,13,60,10,215,35,60,17,51,61,60,241,123,90,60,51,77,124,60,57,173,145,60,139,57,168,60,80,67,194,60,210,84,224,60,226,134,1,61,72,147,21,61,21,186,44,61,63,118,71,61,195,85,102,61,77,254,132,61,14,148,153,61,120,89,177,61,205,204,204,61,213,127,236,61,119,141,8,62,64,176,29,62,135,24,54,62,237,71,82,62,36,212,114,62,4,53,140,62,155,232,161,62,26,248,186,62,155,232,215,62,207,83,249,62,154,245,15,63,224,61,38,63,17,249,63,63,215,175,93,63,0,0,128,63,229,207,147,63,213,176,170,63,80,28,197,63,169,158,227,63,244,108,3,64,150,196,23,64,68,66,47,64,194,98,74,64,33,182,105,64,97,241,134,64,97,212,155,64,0,243,179,64,88,205,207,64,85,247,239,64,230,141,10,65,0,0,32,65,223,195,56,65,10,93,85,65,100,99,118,65,42,67,142,65,49,72,164,65,188,181,189,65,214,18,219,65,114,251,252,65,213,17,18,66,185,173,40,66,121,201,66,66,192,239,96,66,87,224,129,66,149,250,149,66,96,49,173,66,0,0,200,66,214,244,230,66,38,90,5,67,30,254,25,67,244,211,49,67,62,90,77,67,43,35,109,67,197,235,136,67,39,29,158,67,74,150,182,67,39,217,210,67,216,123,243,67,216,149,12,68,109,88,34,68,59,121,59,68,183,125,88,68,0,0,122,68,6,89,144,68,176,176,166,68,166,125,192,68,241,72,222,68,103,88,0,69,251,53,20,69,183,38,43,69,113,164,69,69,220,59,100,69,184,199,131,69,103,45,152,69,78,187,175,69,136,238,202,69,137,87,234,69,147,78,7,70,0,64,28,70,72,111,52,70,219,92,80,70,16,157,112,70,151,237,138,70,128,110,160,70,121,67,185,70,101,240,213,70,142,13,247,70,106,165,14,71,166,185,36,71,193,56,62,71,34,170,91,71,42,170,125,71,182,118,146,71,55,34,169,71,0,80,195,71,25,139,225,71,9,58,2,72,42,98,22,72,252,168,45,72,32,138,72,72,88,148,103,72,63,182,133,72,121,104,154,72,196,78,178,72,16,232,205,72,241,198,237,72,85,74,9,73,90,138,30,73,99,20,55,73,197,106,83,73,8,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,10,0,0,0,1,0,0,0,12,0,0,0,1,0,0,0,205,204,60,65,154,153,89,65,154,153,137,65,0,0,0,66,0,0,58,66,51,51,77,66,0,0,102,66,51,51,134,66,0,0,143,66,51,51,169,66,51,51,195,66,0,0,2,67,154,153,217,64,154,153,185,64,154,153,185,64,205,204,204,64,0,0,208,64,102,102,30,65,154,153,65,65,102,102,102,65,0,0,112,65,51,51,151,65,205,204,172,65,51,51,215,65,205,204,8,66,205,204,32,66,51,51,59,66,0,0,98,66,205,204,114,66,205,204,147,66,102,102,171,66,205,204,186,66,51,51,252,66,0,0,0,0,12,4,4,4,4,4,4,8,8,8,16,28,36,0,0,0,41,4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,12,12,12,12,12,16,16,24,28,36,44,64,64,64,64,64,64,64,64,64,64,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,4,4,4,4,4,4,8,8,8,16,28,36,0,0,0,41,4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,12,12,12,12,12,16,16,24,28,36,44,64,64,64,64,64,64,64,64,64,64,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,4,4,4,4,4,4,4,8,8,8,8,12,16,20,20,40,12,12,12,12,12,12,12,12,12,12,12,12,12,16,16,16,16,16,16,16,20,20,20,20,24,24,24,28,28,32,36,36,40,44,48,52,56,60,64,80,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,4,4,4,4,4,4,8,8,8,16,28,36,0,0,0,47,4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,12,12,12,16,16,16,20,24,24,28,36,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,0,0,0,0,0,0,0,0,14,4,4,4,4,4,8,8,8,12,12,12,16,16,16,0,49,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,8,8,12,12,12,12,16,16,20,20,24,24,28,28,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,96,0,0,0,0,0,0,14,4,4,4,4,4,8,8,8,12,12,12,16,16,16,0,49,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,8,8,12,12,12,12,16,16,20,20,24,24,28,28,32,32,32,32,32,32,32,32,32,32])
.concat([32,32,32,32,32,32,32,32,32,96,0,0,0,0,0,0,14,4,4,4,4,4,8,8,8,12,12,12,16,16,16,0,51,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,8,8,12,12,12,12,16,16,20,20,24,24,28,28,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,0,0,0,0,15,4,4,4,4,4,4,4,8,8,8,12,12,16,16,20,47,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,8,8,8,8,8,12,12,12,12,16,16,16,20,20,24,24,28,28,32,36,36,40,44,48,52,52,64,64,64,64,64,0,0,0,0,0,0,0,0,15,4,4,4,4,4,4,4,8,8,8,12,12,16,16,20,47,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,8,8,8,8,8,12,12,12,12,16,16,16,20,20,24,24,28,28,32,36,36,40,44,48,52,52,64,64,64,64,64,0,0,0,0,0,0,0,0,15,4,4,4,4,4,4,4,4,8,8,12,12,16,20,20,43,8,8,8,8,8,8,8,8,8,8,8,12,12,12,12,12,12,12,12,12,16,16,16,16,20,20,20,24,24,28,28,32,36,40,40,44,48,52,56,60,64,64,64,0,0,0,0,0,0,0,0,0,0,0,0,15,4,4,4,4,4,4,4,4,8,8,12,12,16,20,20,43,8,8,8,8,8,8,8,8,8,8,8,12,12,12,12,12,12,12,12,12,16,16,16,16,20,20,20,24,24,28,28,32,36,40,40,44,48,52,56,60,64,64,64,0,0,0,0,0,0,0,0,0,0,0,0,15,4,4,4,4,4,4,4,4,8,8,12,12,16,20,20,43,8,8,8,8,8,8,8,8,8,8,8,12,12,12,12,12,12,12,12,12,16,16,16,16,20,20,20,24,24,28,28,32,36,40,40,44,48,52,56,60,64,64,64,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,66,15,0,0,0,0,0,1,0,0,0,16,0,0,0,17,0,0,0,8,0,0,0,9,0,0,0,24,0,0,0,25,0,0,0,4,0,0,0,5,0,0,0,20,0,0,0,21,0,0,0,12,0,0,0,13,0,0,0,28,0,0,0,29,0,0,0,2,0,0,0,3,0,0,0,18,0,0,0,19,0,0,0,10,0,0,0,11,0,0,0,26,0,0,0,27,0,0,0,6,0,0,0,7,0,0,0,22,0,0,0,23,0,0,0,14,0,0,0,15,0,0,0,30,0,0,0,31,0,0,0,72,60,26,0,144,206,25,0,236,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,255,239,65,17,0,0,0,72,60,26,0,208,128,25,0,72,72,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,64,254,24,0,64,254,24,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,240,221,24,0,240,221,24,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,216,197,24,0,216,197,24,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,184,175,24,0,240,156,24,0,0,0,0,0,128,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,48,141,26,0,128,123,26,0,0,0,0,0,128,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,104,104,26,0,64,87,26,0,0,0,0,0,128,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,128,70,26,0,192,54,26,0,0,0,0,0,128,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,72,39,26,0,72,39,26,0,0,0,0,0,128,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,64,29,26,0,64,29,26,0,0,0,0,0,128,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,16,16,26,0,72,7,26,0,0,0,0,0,128,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,128,0,26,0,24,247,25,0,0,0,0,0,128,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,248,234,25,0,248,226,25,0,0,0,0,0,128,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,208,217,25,0,192,203,25,0,0,0,0,0,128,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,224,191,25,0,144,185,25,0,0,0,0,0,128,0,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,224,177,25,0,248,151,47,0,0,0,0,0,128,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,184,163,25,0,48,156,25,0,0,0,0,0,128,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,72,60,26,0,88,148,25,0,144,141,25,0,8,0,0,0,1,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,176,134,25,0,176,126,25,0,12,0,0,0,1,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,200,119,25,0,32,114,25,0,136,76,0,0,1,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,56,109,25,0,128,103,25,0,16,0,0,0,1,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,64,98,25,0,64,92,25,0,56,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,116,64,17,0,0,0,0,0,0,0,216,86,25,0,248,81,25,0,52,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,116,64,17,0,0,0,0,0,0,0,80,77,25,0,32,69,25,0,252,43,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,17,0,0,0,0,0,0,0,32,62,25,0,152,56,25,0,0,44,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,17,0,0,0,0,0,0,0,88,49,25,0,48,43,25,0,104,0,0,0,3,0,0,0,0,0,0,0,0,36,254,64,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,0,37,25,0,0,27,25,0,112,0,0,0,3,0,0,0,0,0,0,0,0,36,254,64,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,157,24,0,16,82,25,0,208,22,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,96,77,25,0,16,82,25,0,208,22,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,144,141,26,0,48,69,25,0,48,25,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,56,62,25,0,48,69,25,0,48,25,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,232,123,26,0,176,56,25,0,76,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,96,49,25,0,176,56,25,0,76,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,128,198,24,0,64,43,25,0,40,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,0,0,0,0,0,0,8,37,25,0,64,43,25,0,40,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,0,0,0,0,0,0,40,74,25,0,32,27,25,0,44,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,0,0,0,0,0,0,48,20,25,0,32,27,25,0,44,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,0,0,0,0,0,0,216,222,24,0,40,10,25,0,12,0,0,0,84,77,70,83,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,34,64,8,0,0,0,0,0,0,0,64,4,25,0,40,10,25,0,12,0,0,0,84,77,70,83,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,34,64,8,0,0,0,0,0,0,0,88,131,25,0,192,252,24,0,20,0,0,0,84,77,70,83,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,34,64,8,0,0,0,0,0,0,0,200,245,24,0,192,252,24,0,20,0,0,0,84,77,70,83,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,34,64,8,0,0,0,0,0,0,0,240,175,24,0,160,243,24,0,16,0,0,0,84,77,70,83,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,34,64,8,0,0,0,0,0,0,0,16,241,24,0,160,243,24,0,16,0,0,0,84,77,70,83,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,34,64,8,0,0,0,0,0,0,0,240,255,24,0,0,238,24,0,24,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,8,0,0,0,56,236,24,0,104,232,24,0,0,238,24,0,24,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,8,0,0,0,56,236,24,0,240,212,25,0,0,229,24,0,32,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,8,0,0,0,56,236,24,0,200,226,24,0,0,229,24,0,32,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,8,0,0,0,56,236,24,0,168,224,24,0,32,221,24,0,56,0,0,0,4,0,0,0,205,59,127,102,158,160,230,63,0,0,0,0,0,0,64,192,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,80,218,24,0,32,221,24,0,56,0,0,0,4,0,0,0,205,59,127,102,158,160,230,63,0,0,0,0,0,0,64,192,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,144,216,24,0,200,214,24,0,52,0,0,0,4,0,0,0,205,59,127,102,158,160,230,63,0,0,0,0,0,0,64,192,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,128,212,24,0,120,210,24,0,52,0,0,0,4,0,0,0,205,59,127,102,158,160,230,63,0,0,0,0,0,0,64,192,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,48,207,24,0,232,203,24,0,60,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,192,0,0,0,0,0,0,64,64,8,0,0,0,0,0,0,0,160,201,24,0,224,199,24,0,64,0,0,0,4,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,64,143,192,0,0,0,0,0,64,143,64,8,0,0,0,0,0,0,0,224,196,24,0,224,199,24,0,64,0,0,0,4,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,64,143,192,0,0,0,0,0,64,143,64,8,0,0,0,0,0,0,0,136,194,24,0,232,192,24,0,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,255,239,65,8,0,0,0,136,194,24,0,184,191,24,0,232,192,24,0,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,255,239,65,8,0,0,0,136,194,24,0,48,190,24,0,184,184,24,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,136,194,24,0,152,181,24,0,240,179,24,0,92,0,0,0,4,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,0,0,0,0,0,0,104,178,24,0,248,176,24,0,84,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,81,64,8,0,0,0,104,178,24,0,24,175,24,0,24,173,24,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,104,178,24,0,208,171,24,0,128,170,24,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,104,178,24,0,24,169,24,0,72,168,24,0,0,0,0,0,128,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,104,178,24,0,192,165,24,0,72,163,24,0,0,0,0,0,128,0,0,0,65,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,104,178,24,0,216,160,24,0,64,159,24,0,0,0,0,0,128,0,0,0,69,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,104,178,24,0,208,155,24,0,136,154,24,0,0,0,0,0,128,0,0,0,70,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,104,178,24,0,56,153,24,0,224,151,24,0,0,0,0,0,128,0,0,0,71,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,104,178,24,0,16,151,24,0,104,160,26,0,0,0,0,0,128,0,0,0,66,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,104,178,24,0,160,157,26,0,96,155,26,0,0,0,0,0,128,0,0,0,67,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,104,178,24,0,104,149,26,0,160,143,26,0,0,0,0,0,128,0,0,0,68,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,104,178,24,0,112,140,26,0,128,138,26,0,248,21,0,0,1,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,0,0,0,0,0,0,96,137,26,0,80,136,26,0,252,21,0,0,1,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,56,64,8,0,0,0,0,0,0,0,96,135,26,0,240,133,26,0,0,22,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,8,0,0,0,0,0,0,0,72,132,26,0,80,130,26,0,8,22,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,8,0,0,0,0,0,0,0,104,128,26,0,80,130,26,0,8,22,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,8,0,0,0,0,0,0,0,64,126,26,0,104,122,26,0,80,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,8,0,0,0,64,126,26,0,248,120,26,0,216,119,26,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,64,126,26,0,192,118,26,0,120,117,26,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,64,126,26,0,16,116,26,0,80,114,26,0,24,22,0,0,3,0,0,0,0,0,0,0,0,0,52,64,0,0,0,0,0,0,46,64,0,0,0,0,0,128,64,64,8,0,0,0,0,0,0,0,240,111,26,0,16,109,26,0,32,22,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,8,0,0,0,0,0,0,0,168,106,26,0,48,103,26,0,36,22,0,0,4,0,0,0,0,0,0,224,255,255,239,71,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,71,8,0,0,0,0,0,0,0,8,102,26,0,224,100,26,0,40,22,0,0,4,0,0,0,154,153,153,153,153,153,185,63,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,0,0,0,0,0,0,128,99,26,0,216,97,26,0,44,22,0,0,4,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,0,0,0,0,0,0,40,97,26,0,48,94,26,0,48,22,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,0,0,0,0,224,91,26,0,72,90,26,0,52,22,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,0,0,0,0,32,89,26,0,128,86,26,0,56,22,0,0,2,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,224,195,0,0,0,0,0,0,224,67,8,0,0,0,0,0,0,0,136,84,26,0,128,83,26,0,68,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,8,0,0,0,136,84,26,0,128,82,26,0,176,80,26,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,136,84,26,0,224,79,26,0,200,77,26,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,136,84,26,0,176,75,26,0,96,74,26,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,136,84,26,0,216,72,26,0,216,69,26,0,16,22,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,8,0,0,0,216,72,26,0,232,68,26,0,32,68,26,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,216,72,26,0,32,67,26,0,48,66,26,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,216,72,26,0,56,65,26,0,200,63,26,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,8,0,0,0,216,72,26,0,80,61,26,0,160,59,26,0,20,22,0,0,1,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,0,0,0,0,0,0,48,64,8,0,0,0,0,0,0,0,80,57,26,0,40,54,26,0,244,21,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,80,64,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,176,212,24,0,160,210,24,0,248,21,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,0,64,10,0,0,0,176,212,24,0,88,207,24,0,16,204,24,0,0,0,0,0,128,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,10,0,0,0,176,212,24,0,168,201,24,0,248,199,24,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,10,0,0,0,176,212,24,0,240,196,24,0,208,194,24,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,10,0,0,0,176,212,24,0,248,192,24,0,200,191,24,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,10,0,0,0,176,212,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,176,219,25,0,192,205,25,0,72,0,0,0,1,0,0,0,64,13,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,25,0,0,0,0,0,0,0,128,197,25,0,192,205,25,0,72,0,0,0,1,0,0,0,0,244,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,9,0,0,0,0,0,0,0,192,186,25,0,104,179,25,0,76,0,0,0,1,0,0,0,0,9,61,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,104,127,26,0,0,0,0,0,88,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,255,239,65,59,0,0,0,104,127,26,0,224,172,25,0,8,165,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,104,127,26,0,200,157,25,0,72,149,25,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,224,142,25,0,32,136,25,0,0,0,0,0,128,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,24,128,25,0,104,121,25,0,0,0,0,0,128,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,48,115,25,0,80,110,25,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,104,127,26,0,24,105,25,0,128,99,25,0,0,0,0,0,128,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,176,93,25,0,144,87,25,0,0,0,0,0,128,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,152,82,25,0,0,0,0,0,0,0,0,0,128,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,104,127,26,0,240,77,25,0,168,70,25,0,0,0,0,0,128,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,104,127,26,0,24,63,25,0,192,57,25,0,0,0,0,0,128,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,104,127,26,0,40,50,25,0,0,44,25,0,0,0,0,0,128,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,104,127,26,0,104,37,25,0,168,27,25,0,0,0,0,0,128,0,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,104,127,26,0,8,21,25,0,184,10,25,0,0,0,0,0,128,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,16,5,25,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,104,127,26,0,80,253,24,0,96,246,24,0,0,0,0,0,128,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,232,243,24,0,144,241,24,0,0,0,0,0,128,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,152,238,24,0,112,236,24,0,0,0,0,0,128,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,104,127,26,0,48,233,24,0,96,229,24,0,0,0,0,0,128,0,0,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,104,127,26,0,248,226,24,0,224,224,24,0,0,0,0,0,128,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,59,0,0,0,104,127,26,0,128,221,24,0,184,218,24,0,0,0,0,0,128,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,208,216,24,0,24,215,24,0,0,0,0,0,128,0,0,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,240,212,24,0,232,210,24,0,0,0,0,0,128,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,104,127,26,0,136,207,24,0,64,204,24,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,184,201,24,0,40,200,24,0,96,197,24,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,184,201,24,0,248,194,24,0,16,193,24,0,0,0,0,0,128,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,184,201,24,0,240,191,24,0,72,190,24,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,184,201,24,0,16,189,24,0,16,182,24,0,0,0,0,0,128,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,184,201,24,0,16,180,24,0,120,178,24,0,0,0,0,0,128,0,0,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,184,201,24,0,80,177,24,0,64,175,24,0,144,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,128,173,24,0,248,171,24,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,208,170,24,0,96,169,24,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,112,168,24,0,240,165,24,0,0,0,0,0,128,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,168,163,24,0,0,161,24,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,144,159,24,0,72,156,24,0,0,0,0,0,128,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,216,154,24,0,72,153,24,0,0,0,0,0,128,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,16,152,24,0,32,151,24,0,0,0,0,0,128,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,144,160,26,0,72,158,26,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,152,155,26,0,128,149,26,0,0,0,0,0,128,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,208,143,26,0,152,140,26,0,0,0,0,0,128,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,160,138,26,0,112,137,26,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,112,136,26,0,112,135,26,0,0,0,0,0,128,0,0,0,50,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,80,177,24,0,16,134,26,0,0,0,0,0,100,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,152,132,26,0,0,0,0,0,104,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,128,130,26,0,120,128,26,0,136,0,0,0,1,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,128,126,26,0,176,122,26,0,156,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,11,0,0,0,0,0,0,0,0,121,26,0,240,119,26,0,160,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,11,0,0,0,0,0,0,0,232,118,26,0,144,117,26,0,180,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,0,0,0,0,32,116,26,0,0,0,0,0,168,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,0,0,0,0,240,114,26,0,0,0,0,0,172,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,0,112,26,0,0,0,0,0,116,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,128,109,26,0,184,106,26,0,236,1,0,0,4,0,0,0,0,0,0,0,0,0,224,63,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,216,103,26,0,24,102,26,0,240,1,0,0,4,0,0,0,0,0,0,0,0,0,224,63,0,0,0,0,0,0,240,191,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,96,101,26,0,144,99,26,0,244,1,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,240,191,0,0,0,0,0,64,81,64,17,0,0,0,0,0,0,0,64,98,26,0,56,97,26,0,248,1,0,0,1,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,0,0,240,191,0,0,0,0,0,0,144,64,17,0,0,0,0,0,0,0,240,94,26,0,240,91,26,0,252,1,0,0,1,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,216,90,26,0,48,89,26,0,156,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,191,0,0,0,0,0,0,48,64,17,0,0,0,0,0,0,0,232,86,26,0,152,84,26,0,160,0,0,0,4,0,0,0,0,0,0,0,0,0,244,63,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,160,83,26,0,176,82,26,0,164,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,208,80,26,0,232,79,26,0,168,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,112,78,26,0,192,75,26,0,116,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,128,74,26,0,0,0,0,0,120,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,232,72,26,0,0,0,0,0,124,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,24,70,26,0,0,0,0,0,128,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,240,68,26,0,0,0,0,0,132,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,64,68,26,0,0,0,0,0,136,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,56,67,26,0,0,0,0,0,140,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,128,66,26,0,0,0,0,0,144,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,64,65,26,0,0,0,0,0,148,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,64,64,26,0,0,0,0,0,152,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,144,61,26,0,0,0,0,0,52,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,192,59,26,0,112,57,26,0,164,2,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,112,54,26,0,0,0,0,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,0,53,26,0,0,52,26,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,112,51,26,0,80,50,26,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,168,49,26,0,152,48,26,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,144,46,26,0,128,41,26,0,0,0,0,0,128,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,136,40,26,0,0,0,0,0,0,0,0,0,128,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,24,39,26,0,0,38,26,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,160,37,26,0,0,0,0,0,0,0,0,0,128,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,0,37,26,0,80,36,26,0,0,0,0,0,128,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,208,35,26,0,0,0,0,0,0,0,0,0,128,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,72,34,26,0,192,32,26,0,0,0,0,0,128,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,168,31,26,0,152,30,26,0,0,0,0,0,128,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,0,29,26,0,0,0,0,0,0,0,0,0,128,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,48,28,26,0,0,0,0,0,0,0,0,0,128,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,72,27,26,0,176,21,26,0,0,0,0,0,128,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,88,21,26,0,248,20,26,0,0,0,0,0,128,0,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,192,59,26,0,0,20,26,0,184,18,26,0,168,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,27,0,0,0,0,20,26,0,8,18,26,0,40,17,26,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,0,20,26,0,0,20,26,0,160,15,26,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,0,20,26,0,24,15,26,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,0,20,26,0,176,14,26,0,208,13,26,0,0,0,0,0,128,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,0,20,26,0,144,13,26,0,248,12,26,0,0,0,0,0,128,0,0,0,254,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,0,20,26,0,200,11,26,0,168,10,26,0])
.concat([172,0,0,0,4,0,0,0,0,0,0,0,0,0,244,63,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,232,9,26,0,248,8,26,0,184,2,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,26,0,0,0,232,9,26,0,16,7,26,0,112,6,26,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,26,0,0,0,232,9,26,0,32,6,26,0,128,5,26,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,26,0,0,0,232,9,26,0,80,5,26,0,168,4,26,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,26,0,0,0,232,9,26,0,224,3,26,0,208,2,26,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,26,0,0,0,232,9,26,0,24,2,26,0,112,1,26,0,0,0,0,0,128,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,26,0,0,0,232,9,26,0,72,0,26,0,120,255,25,0,0,0,0,0,128,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,26,0,0,0,232,9,26,0,72,255,25,0,200,254,25,0,0,0,0,0,128,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,26,0,0,0,232,9,26,0,152,254,25,0,0,0,0,0,176,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,216,252,25,0,0,0,0,0,176,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,184,251,25,0,104,250,25,0,180,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,168,249,25,0,112,248,25,0,0,2,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,88,64,17,0,0,0,0,0,0,0,168,246,25,0,176,245,25,0,4,2,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,32,245,25,0,176,245,25,0,8,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,160,244,25,0,0,0,0,0,16,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,40,244,25,0,96,242,25,0,24,2,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,96,192,0,0,0,0,0,192,95,64,17,0,0,0,0,0,0,0,160,241,25,0,96,239,25,0,28,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,0,0,0,0,72,237,25,0,184,235,25,0,32,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,0,0,0,0,192,234,25,0,32,234,25,0,12,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,0,0,0,0,168,233,25,0,136,232,25,0,36,2,0,0,4,0,0,0,0,0,0,0,0,0,240,63,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,248,231,25,0,144,231,25,0,184,0,0,0,4,0,0,0,154,153,153,153,153,153,233,191,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,200,230,25,0,128,229,25,0,188,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,192,228,25,0,216,227,25,0,40,2,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,184,226,25,0,208,225,25,0,16,3,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,17,0,0,0,184,226,25,0,48,225,25,0,176,224,25,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,184,226,25,0,32,224,25,0,128,223,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,184,226,25,0,88,222,25,0,200,220,25,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,184,226,25,0,16,220,25,0,0,0,0,0,0,0,0,0,128,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,184,226,25,0,240,218,25,0,0,0,0,0,0,0,0,0,128,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,184,226,25,0,144,217,25,0,168,216,25,0,0,0,0,0,128,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,184,226,25,0,48,216,25,0,160,214,25,0,192,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,64,214,25,0,248,212,25,0,196,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,224,207,25,0,8,207,25,0,200,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,8,206,25,0,40,205,25,0,204,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,120,203,25,0,160,202,25,0,208,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,24,202,25,0,152,201,25,0,20,3,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,48,225,25,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,88,222,25,0,0,0,0,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,176,200,25,0,0,0,0,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,112,200,25,0,0,0,0,0,0,0,0,0,128,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,136,199,25,0,0,0,0,0,0,0,0,0,128,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,240,218,25,0,0,0,0,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,88,198,25,0,0,0,0,0,0,0,0,0,128,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,176,197,25,0,0,0,0,0,0,0,0,0,128,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,232,196,25,0,0,0,0,0,0,0,0,0,128,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,168,191,25,0,0,0,0,0,0,0,0,0,128,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,176,190,25,0,0,0,0,0,0,0,0,0,128,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,104,190,25,0,0,0,0,0,0,0,0,0,128,0,0,0,23,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,48,190,25,0,0,0,0,0,0,0,0,0,128,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,8,190,25,0,0,0,0,0,0,0,0,0,128,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,152,189,25,0,208,188,25,0,0,0,0,0,128,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,24,202,25,0,224,187,25,0,0,0,0,0,212,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,240,186,25,0,104,186,25,0,172,2,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,240,186,25,0,96,185,25,0,176,184,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,240,186,25,0,104,184,25,0,168,183,25,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,240,186,25,0,104,183,25,0,0,0,0,0,24,3,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,32,183,25,0,120,182,25,0,216,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,32,183,25,0,8,181,25,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,32,183,25,0,168,180,25,0,0,0,0,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,32,183,25,0,8,179,25,0,0,0,0,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,32,183,25,0,184,177,25,0,184,176,25,0,224,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,36,64,17,0,0,0,0,0,0,0,144,176,25,0,48,176,25,0,176,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,59,0,0,0,144,176,25,0,200,175,25,0,80,175,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,64,174,25,0,168,173,25,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,144,176,25,0,32,6,26,0,0,0,0,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,24,173,25,0,224,171,25,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,32,171,25,0,8,170,25,0,0,0,0,0,128,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,240,169,25,0,136,169,25,0,0,0,0,0,128,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,104,169,25,0,0,0,0,0,0,0,0,0,128,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,8,168,25,0,0,0,0,0,0,0,0,0,128,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,48,167,25,0,0,0,0,0,0,0,0,0,128,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,240,165,25,0,0,0,0,0,0,0,0,0,128,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,96,165,25,0,160,164,25,0,0,0,0,0,128,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,136,163,25,0,128,162,25,0,0,0,0,0,128,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,104,162,25,0,0,0,0,0,0,0,0,0,128,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,0,162,25,0,160,161,25,0,0,0,0,0,128,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,32,161,25,0,248,159,25,0,0,0,0,0,128,0,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,48,159,25,0,0,158,25,0,0,0,0,0,128,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,144,176,25,0,40,157,25,0,208,155,25,0,0,0,0,0,128,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,26,0,0,0,144,176,25,0,152,154,25,0,64,154,25,0,180,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,18,0,0,0,248,153,25,0,184,153,25,0,64,153,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,248,153,25,0,216,90,26,0,104,152,25,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,248,153,25,0,32,151,25,0,0,150,25,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,248,153,25,0,24,149,25,0,32,148,25,0,232,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,8,147,25,0,88,146,25,0,236,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,240,145,25,0,160,145,25,0,240,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,192,144,25,0,40,144,25,0,244,0,0,0,1,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,16,143,25,0,128,142,25,0,248,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,120,141,25,0,152,140,25,0,252,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,64,140,25,0,224,139,25,0,0,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,16,139,25,0,112,138,25,0,4,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,8,138,25,0,48,137,25,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,96,136,25,0,160,135,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,144,134,25,0,168,133,25,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,184,226,25,0,104,133,25,0,0,0,0,0,128,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,8,21,25,0,120,132,25,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,72,132,25,0,96,131,25,0,0,0,0,0,128,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,176,130,25,0,16,130,25,0,0,0,0,0,128,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,128,173,24,0,40,128,25,0,0,0,0,0,128,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,224,127,25,0,104,126,25,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,216,125,25,0,112,125,25,0,0,0,0,0,128,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,16,125,25,0,192,123,25,0,0,0,0,0,128,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,160,123,25,0,0,0,0,0,0,0,0,0,128,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,48,123,25,0,0,0,0,0,0,0,0,0,128,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,48,122,25,0,128,121,25,0,8,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,240,120,25,0,72,119,25,0,12,1,0,0,1,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,184,118,25,0,0,0,0,0,16,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,112,118,25,0,240,117,25,0,20,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,184,117,25,0,112,117,25,0,24,1,0,0,1,0,0,0,63,66,15,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,104,116,25,0,248,115,25,0,28,1,0,0,1,0,0,0,63,66,15,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,96,115,25,0,0,0,0,0,80,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,0,0,0,0,208,114,25,0,0,0,0,0,56,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,208,114,25,0,208,113,25,0,48,113,25,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,208,114,25,0,0,121,26,0,176,112,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,208,114,25,0,48,112,25,0,0,112,25,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,208,114,25,0,208,111,25,0,112,111,25,0,0,0,0,0,128,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,208,114,25,0,200,110,25,0,112,110,25,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,208,114,25,0,240,109,25,0,24,109,25,0,60,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,128,108,25,0,0,0,0,0,32,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,48,108,25,0,0,0,0,0,36,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,240,107,25,0,144,107,25,0,40,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,17,0,0,0,240,107,25,0,176,200,25,0,40,107,25,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,240,107,25,0,216,106,25,0,104,106,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,240,107,25,0,176,130,25,0,48,105,25,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,240,107,25,0,112,104,25,0,0,0,0,0,56,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,88,103,25,0,200,102,25,0,52,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,88,102,25,0,224,101,25,0,64,2,0,0,1,0,0,0,236,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,184,101,25,0,64,101,25,0,68,2,0,0,1,0,0,0,74,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,216,100,25,0,40,100,25,0,56,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,144,99,25,0,208,98,25,0,52,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,184,201,24,0,0,0,0,0,92,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,255,239,65,27,0,0,0,184,201,24,0,24,98,25,0,0,0,0,0,100,3,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,136,97,25,0,0,0,0,0,40,3,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,27,0,0,0,136,97,25,0,48,225,25,0,40,97,25,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,136,97,25,0,216,96,25,0,144,96,25,0,60,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,184,95,25,0,16,95,25,0,64,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,64,94,25,0,208,93,25,0,68,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,32,93,25,0,248,91,25,0,68,3,0,0,1,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,64,91,25,0,216,90,25,0,72,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,0,0,0,0,168,90,25,0,8,90,25,0,76,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,0,0,0,0,208,89,25,0,0,0,0,0,72,3,0,0,1,0,0,0,157,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,208,89,25,0,104,89,25,0,0,0,0,0,0,0,0,0,128,0,0,0,157,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,208,89,25,0,168,88,25,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,248,87,25,0,0,0,0,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,80,87,25,0,0,0,0,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,136,86,25,0,0,0,0,0,0,0,0,0,128,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,8,86,25,0,0,0,0,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,200,85,25,0,0,0,0,0,0,0,0,0,128,0,0,0,28,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,48,85,25,0,0,0,0,0,0,0,0,0,128,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,192,84,25,0,0,0,0,0,0,0,0,0,128,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,120,84,25,0,0,0,0,0,0,0,0,0,128,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,240,83,25,0,0,0,0,0,0,0,0,0,128,0,0,0,131,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,72,83,25,0,0,0,0,0,0,0,0,0,128,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,200,82,25,0,0,0,0,0,0,0,0,0,128,0,0,0,30,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,96,82,25,0,0,0,0,0,0,0,0,0,128,0,0,0,40,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,184,81,25,0,0,0,0,0,0,0,0,0,128,0,0,0,50,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,40,81,25,0,0,0,0,0,0,0,0,0,128,0,0,0,60,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,208,89,25,0,200,80,25,0,0,0,0,0,76,3,0,0,1,0,0,0,157,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,200,80,25,0,104,89,25,0,0,0,0,0,0,0,0,0,128,0,0,0,157,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,200,80,25,0,144,80,25,0,152,79,25,0,32,3,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,26,0,0,0,0,0,0,0,88,79,25,0,240,78,25,0,72,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,88,78,25,0,0,78,25,0,76,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,168,77,25,0,24,77,25,0,80,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,128,76,25,0,32,76,25,0,84,2,0,0,1,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,48,147,25,0,96,75,25,0,248,74,25,0,80,1,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,199,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,48,74,25,0,200,73,25,0,84,1,0,0,1,0,0,0,236,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,192,255,223,64,17,0,0,0,0,0,0,0,32,73,25,0,56,71,25,0,88,1,0,0,1,0,0,0,74,14,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,192,255,223,64,17,0,0,0,0,0,0,0,0,70,25,0,176,68,25,0,92,1,0,0,1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,208,67,25,0,248,66,25,0,80,3,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,136,66,25,0,72,65,25,0,240,64,25,0,84,3,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,136,66,25,0,128,64,25,0,160,63,25,0,88,3,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,136,66,25,0,48,63,25,0,192,62,25,0,0,0,0,0,128,0,0,0,240,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,136,66,25,0,240,61,25,0,64,61,25,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,136,66,25,0,24,61,25,0,208,60,25,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,136,66,25,0,152,60,25,0,64,60,25,0,0,0,0,0,128,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,136,66,25,0,24,60,25,0,248,58,25,0,0,0,0,0,128,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,136,66,25,0,16,58,25,0,16,57,25,0,0,0,0,0,128,0,0,0,48,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,18,0,0,0,136,66,25,0,8,56,25,0,176,54,25,0,96,1,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,64,17,0,0,0,0,0,0,0,96,54,25,0,144,53,25,0,100,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,36,64,17,0,0,0,0,0,0,0,80,52,25,0,248,51,25,0,104,1,0,0,1,0,0,0,25,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,176,51,25,0,184,50,25,0,108,1,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,56,50,25,0,232,49,25,0,112,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,248,48,25,0,240,47,25,0,88,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,0,0,0,0,120,47,25,0,248,46,25,0,116,1,0,0,1,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,144,46,25,0,0,0,0,0,120,1,0,0,1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,72,46,25,0,176,45,25,0,124,1,0,0,1,0,0,0,40,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,248,44,25,0,0,0,0,0,84,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,25,0,0,0,0,0,0,0,136,44,25,0,0,0,0,0,92,2,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,0,0,0,0,152,43,25,0,0,0,0,0,96,2,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,0,0,0,0,224,42,25,0,216,41,25,0,104,2,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,17,0,0,0,0,0,0,0,128,41,25,0,232,40,25,0,184,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,10,0,0,0,0,0,0,0,112,40,25,0,0,0,0,0,28,3,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,72,249,25,0,0,0,0,0,192,1,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,11,0,0,0,72,249,25,0,208,39,25,0,0,0,0,0,200,1,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,10,0,0,0,208,39,25,0,32,39,25,0,0,0,0,0,44,2,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,32,38,25,0,0,0,0,0,48,2,0,0,4,0,0,0,0,0,0,0,0,0,8,64,0,0,0,0,0,0,0,0,0,0,0,224,255,255,239,71,17,0,0,0,0,0,0,0,184,37,25,0,0,0,0,0,112,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,192,255,255,255,223,65,27,0,0,0,0,0,0,0,72,37,25,0,0,0,0,0,128,1,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,32,64,19,0,0,0,0,0,0,0,224,36,25,0,0,0,0,0,132,1,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,28,64,19,0,0,0,0,0,0,0,248,31,25,0,0,0,0,0,136,1,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,32,64,19,0,0,0,0,0,0,0,160,31,25,0,0,0,0,0,140,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,19,0,0,0,0,0,0,0,96,31,25,0,0,0,0,0,144,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,24,64,19,0,0,0,0,0,0,0,32,31,25,0,40,30,25,0,4,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,192,29,25,0,200,28,25,0,148,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,17,0,0,0,0,0,0,0,248,27,25,0,104,27,25,0,44,3,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,27,0,0,0,248,27,25,0,200,26,25,0,0,0,0,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,248,27,25,0,200,25,25,0,0,0,0,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,19,0,0,0,248,27,25,0,88,25,25,0,0,25,25,0,208,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,64,9,0,0,0,88,25,25,0,152,24,25,0,24,24,25,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,88,25,25,0,208,23,25,0,184,22,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,88,25,25,0,176,21,25,0,192,20,25,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,88,25,25,0,216,19,25,0,8,19,25,0,0,0,0,0,128,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,88,25,25,0,200,18,25,0,232,17,25,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,88,25,25,0,104,17,25,0,16,17,25,0,0,0,0,0,128,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,88,25,25,0,192,16,25,0,224,15,25,0,0,0,0,0,128,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,88,25,25,0,128,15,25,0,160,10,25,0,0,0,0,0,128,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,88,25,25,0,0,10,25,0,16,9,25,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,9,0,0,0,88,25,25,0,112,8,25,0,248,7,25,0,212,1,0,0,84,77,70,83,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,34,64,10,0,0,0,112,8,25,0,104,7,25,0,0,0,0,0,120,3,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,208,6,25,0,112,6,25,0,168,3,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,96,192,0,0,0,0,0,192,95,64,34,0,0,0,0,0,0,0,184,5,25,0,64,5,25,0,172,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,191,0,0,192,255,255,255,223,65,34,0,0,0,184,5,25,0,248,4,25,0,0,0,0,0,0,0,0,0,128,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,34,0,0,0,184,5,25,0,48,225,25,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,34,0,0,0,184,5,25,0,0,4,25,0,0,0,0,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,34,0,0,0,184,5,25,0,240,2,25,0,0,0,0,0,232,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,208,17,26,0,152,9,26,0,240,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,46,192,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,184,148,26,0,144,127,26,0,24,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,9,0,0,0,0,0,0,0,64,108,26,0,0,90,26,0,28,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,120,108,26,0,0,0,0,0,172,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,3,0,0,0,120,108,26,0,160,236,25,0,168,149,25,0,0,0,0,0,128,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,3,0,0,0,120,108,26,0,192,87,25,0,96,21,25,0,64,4,0,0,1,0,0,0,64,75,76,0,0,0,0,0,0,0,0,0,0,0,64,64,0,0,192,255,255,255,223,65,2,0,0,0,0,0,0,0,200,229,24,0,120,204,24,0,52,4,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,1,0,0,0,0,0,0,0,24,180,24,0,0,0,0,0,60,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,3,0,0,0,24,180,24,0,176,163,24,0,160,155,26,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,24,180,24,0,136,130,26,0,8,112,26,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,24,180,24,0,40,92,26,0,224,75,26,0,0,0,0,0,128,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,24,180,24,0,160,61,26,0,160,46,26,0,0,0,0,0,128,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,24,180,24,0,0,33,26,0,224,18,26,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,24,180,24,0,208,10,26,0,248,2,26,0,0,0,0,0,128,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,24,180,24,0,144,250,25,0,168,239,25,0,0,0,0,0,128,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,24,180,24,0,168,229,25,0,224,220,25,0,0,0,0,0,128,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,24,180,24,0,40,207,25,0,96,198,25,0,0,0,0,0,128,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,24,180,24,0,240,187,25,0,16,181,25,0,0,0,0,0,128,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,2,0,0,0,24,180,24,0,184,173,25,0,248,165,25,0,188,4,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,2,0,0,0,0,0,0,0,56,159,25,0,40,151,25,0,68,4,0,0,1,0,0,0,64,75,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,2,0,0,0,0,0,0,0,72,144,25,0,96,137,25,0,72,4,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,48,130,25,0,64,122,25,0,100,4,0,0,1,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,2,0,0,0,0,0,0,0,16,116,25,0,208,110,25,0,104,4,0,0,1,0,0,0,0,104,46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,2,0,0,0,0,0,0,0,120,106,25,0,56,100,25,0,144,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,3,0,0,0,120,106,25,0,72,94,25,0,0,0,0,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,3,0,0,0,120,106,25,0,184,88,25,0,80,83,25,0,56,4,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,192,255,255,255,223,65,3,0,0,0,0,0,0,0,104,78,25,0,40,73,25,0,128,4,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,128,255,255,255,223,65,2,0,0,0,0,0,0,0,200,63,25,0,32,59,25,0,152,4,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,255,255,255,223,65,1,0,0,0,0,0,0,0,240,50,25,0,16,45,25,0,156,4,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,255,255,255,223,65,1,0,0,0,0,0,0,0,48,38,25,0,248,28,25,0,160,4,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,255,255,255,223,65,1,0,0,0,0,0,0,0,192,22,25,0,240,15,25,0,132,4,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,208,5,25,0,208,5,25,0,160,254,24,0,132,4,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,208,5,25,0,240,246,24,0,96,244,24,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,208,5,25,0,48,242,24,0,72,239,24,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,208,5,25,0,184,236,24,0,232,233,24,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,208,5,25,0,88,230,24,0,56,227,24,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,208,5,25,0,96,225,24,0,56,222,24,0,0,0,0,0,128,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,208,5,25,0,72,219,24,0,32,217,24,0,0,0,0,0,128,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,208,5,25,0,168,215,24,0,96,213,24,0,0,0,0,0,128,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,2,0,0,0,208,5,25,0,128,211,24,0,16,208,24,0,164,4,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,255,255,255,223,65,2,0,0,0,0,0,0,0,56,205,24,0,240,201,24,0,168,4,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,63,1,0,0,0,0,0,0,0,104,200,24,0,0,198,24,0,180,4,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,255,255,255,223,65,2,0,0,0,0,0,0,0,48,195,24,0,144,193,24,0,184,4,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,2,0,0,0,0,0,0,0,40,192,24,0,184,190,24,0,192,4,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,160,250,25,0,240,239,25,0,8,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,192,239,64,1,0,0,0,0,0,0,0,192,229,25,0,8,221,25,0,8,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,192,239,64,1,0,0,0,0,0,0,0,88,207,25,0,160,198,25,0,16,0,0,0,2,0,0,0,64,66,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,24,188,25,0,96,181,25,0,32,0,0,0,1,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,8,64,0,0,0,0,0,0,16,64,1,0,0,0,0,0,0,0,216,173,25,0,88,166,25,0,36,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,168,180,24,0,104,164,24,0,36,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,63,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,131,26,0,80,113,26,0,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,72,131,26,0,96,93,26,0,240,76,26,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,72,131,26,0,192,62,26,0,120,47,26,0,60,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,1,0,0,0,0,0,0,0,144,33,26,0,72,19,26,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,72,131,26,0,48,11,26,0,64,3,26,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,72,131,26,0,72,251,25,0,152,240,25,0,0,0,0,0,128,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,72,131,26,0,48,230,25,0,104,221,25,0,0,0,0,0,128,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,72,131,26,0,152,207,25,0,24,199,25,0,0,0,0,0,128,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,72,131,26,0,96,188,25,0,224,181,25,0,0,0,0,0,128,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,72,131,26,0,0,174,25,0,232,166,25,0,56,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,0,174,25,0,136,159,25,0,240,151,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,0,174,25,0,120,144,25,0,152,137,25,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,0,174,25,0,128,130,25,0,184,122,25,0,0,0,0,0,128,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,0,174,25,0,56,116,25,0,24,111,25,0,0,0,0,0,128,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,193,0,0,192,255,255,255,223,65,1,0,0,0,0,174,25,0,152,106,25,0,112,100,25,0,72,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,1,0,0,0,0,0,0,0,176,94,25,0,32,89,25,0,80,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,224,111,64,1,0,0,0,0,0,0,0,160,83,25,0,128,78,25,0,76,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,224,111,64,1,0,0,0,0,0,0,0,112,73,25,0,232,63,25,0,88,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,1,0,0,0,0,0,0,0,120,59,25,0,16,51,25,0,92,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,1,0,0,0,0,0,0,0,64,45,25,0,120,38,25,0,96,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,1,0,0,0,0,0,0,0,72,29,25,0,64,23,25,0,100,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,1,0,0,0,0,0,0,0,64,16,25,0,8,6,25,0,108,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,63,1,0,0,0,0,0,0,0,248,254,24,0,8,247,24,0,112,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,230,25,0,144,221,25,0,104,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,18,0,0,0,0,0,0,0,160,207,25,0,8,152,47,0,108,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,136,234,24,0,224,230,24,0,208,99,2,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,136,232,24,0,0,0,0,0,4,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,56,157,25,0,40,93,25,0,4,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,216,230,25,0,96,222,25,0,4,0,0,0,5,0,0,0,248,94,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,24,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,74,25,0,160,64,25,0,52,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,0,0,1,0,8,74,25,0,48,60,25,0,0,0,0,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,74,25,0,80,49,26,0,216,51,25,0,64,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,176,223,25,0,0,8,0,0,126,0,0,0,200,213,25,0,0,0,0,0,168,200,25,0,0,8,0,0,30,0,0,0,248,189,25,0,80,183,25,0,192,175,25,0,0,8,0,0,30,0,0,0,248,189,25,0,80,183,25,0,112,168,25,0,0,8,0,0,30,0,0,0,248,189,25,0,80,183,25,0,152,161,25,0,0,8,0,0,30,0,0,0,248,189,25,0,80,183,25,0,152,153,25,0,0,8,0,0,20,0,0,0,224,145,25,0,0,0,0,0,200,138,25,0,0,8,0,0,24,0,0,0,224,131,25,0,0,0,0,0,184,123,25,0,0,8,0,0,80,0,0,0,160,117,25,0,0,0,0,0,240,111,25,0,0,8,0,0,110,0,0,0,120,107,25,0,0,0,0,0,120,101,25,0,0,8,0,0,164,0,0,0,120,96,25,0,0,0,0,0,248,89,25,0,0,8,0,0,220,0,0,0,152,84,25,0,0,0,0,0,136,79,25,0,0,8,0,0,128,0,0,0,168,74,25,0,0,0,0,0,64,65,25,0,0,8,0,0,226,0,0,0,128,60,25,0,0,0,0,0,64,52,25,0,0,8,0,0,8,0,0,0,112,46,25,0,0,0,0,0,40,40,25,0,0,8,0,0,114,0,0,0,0,31,25,0,0,0,0,0,120,24,25,0,0,8,0,0,206,0,0,0,64,17,25,0,0,0,0,0,88,7,25,0,1,0,0,0,40,0,0,0,160,0,25,0,88,7,25,0,216,247,24,0,1,0,0,0,40,0,0,0,160,0,25,0,88,7,25,0,240,244,24,0,0,0,0,0,8,0,0,0,232,242,24,0,0,0,0,0,56,240,24,0,1,0,0,0,130,0,0,0,24,237,24,0,136,235,24,0,64,231,24,0,5,0,0,0,132,0,0,0,216,227,24,0,176,225,24,0,16,223,24,0,9,64,12,0,16,0,0,0,248,219,24,0,184,217,24,0,0,216,24,0,2,0,0,0,88,52,34,0,8,214,24,0,0,0,0,0,232,211,24,0,2,0,0,0,240,159,33,0,192,209,24,0,0,0,0,0,232,205,24,0,9,128,12,0,20,0,0,0,40,203,24,0,184,200,24,0,184,200,24,0,9,128,12,0,20,0,0,0,40,203,24,0,184,200,24,0,136,198,24,0,9,128,8,0,60,1,0,0,240,195,24,0,32,194,24,0,136,192,24,0,5,32,8,0,170,0,0,0,48,191,24,0,120,189,24,0,104,184,24,0,5,32,8,0,136,0,0,0,192,180,24,0,56,179,24,0,176,177,24,0,9,128,8,0,52,1,0,0,248,175,24,0,240,173,24,0,112,172,24,0,133,64,8,0,136,0,0,0,88,171,24,0,8,170,24,0,184,168,24,0,1,64,13,0,144,0,0,0,192,167,24,0,152,164,24,0,208,161,24,0,1,64,9,0,152,0,0,0,240,159,24,0,80,157,24,0,48,155,24,0,1,68,8,0,160,0,0,0,232,153,24,0,152,152,24,0,144,151,24,0,1,64,13,0,8,0,0,0,240,160,26,0,248,159,26,0,200,156,26,0,5,64,5,0,72,0,0,0,104,154,26,0,248,159,26,0,32,144,26,0,5,128,6,0,84,0,0,0,152,141,26,0,24,139,26,0,16,138,26,0,1,32,0,0,162,0,0,0,192,136,26,0,0,136,26,0,224,134,26,0,9,128,8,0,204,0,0,0,152,133,26,0,56,131,26,0,16,129,26,0,5,32,8,0,154,0,0,0,64,127,26,0,112,124,26,0,144,121,26,0,6,0,0,0,168,106,45,0,104,120,26,0,0,0,0,0,88,119,26,0,6,0,0,0,160,106,45,0,24,118,26,0,0,0,0,0,128,116,26,0,5,0,0,0,178,0,0,0,128,115,26,0,40,113,26,0,24,110,26,0,6,0,0,0,216,29,1,0,176,107,26,0,0,0,0,0,232,104,26,0,5,0,0,0,184,0,0,0,136,102,26,0,200,101,26,0,16,100,26,0,6,0,0,0,136,106,45,0,40,99,26,0,0,0,0,0,136,97,26,0,6,0,0,0,144,106,45,0,128,96,26,0,0,0,0,0,88,93,26,0,6,64,4,0,80,0,0,0,104,91,26,0,72,152,47,0,224,87,26,0,1,32,8,0,198,0,0,0,56,85,26,0,24,84,26,0,24,83,26,0,5,0,0,0,46,0,0,0,88,81,26,0,72,152,47,0,72,80,26,0,133,0,0,0,224,211,46,0,16,79,26,0,72,152,47,0,216,76,26,0,5,1,0,0,192,74,5,0,208,74,26,0,120,73,26,0,16,71,26,0,6,0,0,0,208,125,45,0,128,69,26,0,0,0,0,0,112,68,26,0,133,0,0,0,88,7,4,0,120,67,26,0,176,66,26,0,192,65,26,0,6,64,8,0,176,0,0,0,248,64,26,0,0,0,0,0,184,62,26,0,9,128,8,0,148,1,0,0,32,60,26,0,72,152,47,0,48,58,26,0,5,1,0,0,56,250,3,0,104,55,26,0,120,73,26,0,136,53,26,0,5,1,0,0,48,250,3,0,96,52,26,0,120,73,26,0,168,51,26,0,6,0,0,0,112,106,45,0,224,50,26,0,192,49,26,0,88,49,26,0,6,128,8,0,68,1,0,0,88,47,26,0,0,0,0,0,200,41,26,0,133,128,8,0,76,1,0,0,176,40,26,0,0,0,0,0,208,39,26,0,1,132,8,0,212,0,0,0,112,38,26,0,112,124,26,0,216,37,26,0,13,128,8,0,228,0,0,0,56,37,26,0,160,36,26,0,240,35,26,0,5,128,10,0,244,0,0,0,88,35,26,0,240,35,26,0,136,33,26,0,5,32,8,0,102,0,0,0,88,35,26,0,240,35,26,0,32,32,26,0,5,32,8,0,218,0,0,0,0,31,26,0,32,32,26,0,200,29,26,0,9,128,8,0,84,1,0,0,88,28,26,0,144,27,26,0,248,21,26,0,9,128,8,0,92,1,0,0,112,21,26,0,24,21,26,0,168,20,26,0,129,128,4,0,100,1,0,0,24,19,26,0,72,152,47,0,32,18,26,0,5,0,0,0,208,0,0,0,152,17,26,0,56,16,26,0,80,15,26,0,5,0,0,0,208,0,0,0,152,17,26,0,56,16,26,0,216,14,26,0,5,0,0,0,166,0,0,0,248,13,26,0,24,21,26,0,184,13,26,0,2,0,0,0,152,98,1,0,56,13,26,0,0,0,0,0,224,12,26,0,5,32,8,0,100,0,0,0,8,11,26,0,24,21,26,0,40,10,26,0,13,128,4,0,92,0,0,0,56,9,26,0,24,21,26,0,0,8,26,0,6,0,0,0,64,125,45,0,176,6,26,0,0,0,0,0,96,6,26,0,17,32,8,0,14,0,0,0,184,5,26,0,112,124,26,0,120,5,26,0,25,128,12,0,44,0,0,0,216,4,26,0,160,4,26,0,56,3,26,0,25,130,12,0,52,0,0,0,88,2,26,0,224,1,26,0,208,0,26,0,25,128,8,0,12,1,0,0,216,255,25,0,208,0,26,0,112,255,25,0,29,128,12,0,60,0,0,0,32,255,25,0,192,254,25,0,120,254,25,0,145,0,0,0,0,244,33,0,168,252,25,0,112,124,26,0,64,251,25,0,22,0,0,0,232,173,33,0,232,249,25,0,0,0,0,0,64,249,25,0,18,64,12,0,180,0,0,0,136,247,25,0,0,0,0,0,112,246,25,0,149,0,0,0,240,124,30,0,152,245,25,0,232,211,24,0,216,244,25,0,29,128,8,0,20,1,0,0,80,244,25,0,216,243,25,0,88,242,25,0,17,32,12,0,212,0,0,0,88,240,25,0,184,200,24,0,112,238,25,0,20,0,0,0,86,0,0,0,96,236,25,0,0,0,0,0,72,235,25,0,20,0,0,0,86,0,0,0,96,236,25,0,0,0,0,0,104,234,25,0,17,32,8,0,10,0,0,0,0,234,25,0,0,233,25,0,112,232,25,0,145,128,8,0,124,1,0,0,208,231,25,0,232,211,24,0,112,231,25,0,29,128,8,0,132,1,0,0,8,230,25,0,248,228,25,0,104,228,25,0,22,0,0,0,152,106,45,0,72,227,25,0,0,0,0,0,32,226,25,0,22,0,0,0,128,106,45,0,168,225,25,0,0,0,0,0,224,224,25,0,20,0,0,0,98,0,0,0,72,224,25,0,0,0,0,0,160,223,25,0,21,0,0,0,28,0,0,0,72,224,25,0,120,223,25,0,96,221,25,0,17,32,8,0,122,0,0,0,104,220,25,0,144,27,26,0,160,219,25,0,29,128,8,0,28,1,0,0,40,218,25,0,72,217,25,0,152,216,25,0,29,128,8,0,36,1,0,0,128,215,25,0,72,217,25,0,152,214,25,0,149,128,12,0,44,1,0,0,160,213,25,0,72,152,47,0,224,212,25,0,149,0,0,0,120,163,2,0,128,207,25,0,96,206,25,0,184,205,25,0,21,32,8,0,222,0,0,0,64,204,25,0,160,36,26,0,64,203,25,0,22,0,0,0,64,5,33,0,104,202,25,0,0,0,0,0,0,202,25,0,22,128,8,0,4,1,0,0,8,201,25,0,0,0,0,0,152,200,25,0,21,32,8,0,204,0,0,0,64,200,25,0,0,199,25,0,232,197,25,0,29,128,8,0,252,0,0,0,80,197,25,0,96,196,25,0,56,191,25,0,17,32,8,0,148,0,0,0,144,190,25,0,88,190,25,0,40,190,25,0,33,32,8,0,182,0,0,0,200,189,25,0,112,124,26,0,144,189,25,0,33,32,8,0,120,0,0,0,56,188,25,0,32,187,25,0,184,186,25,0,161,128,12,0,36,0,0,0,200,185,25,0,160,4,26,0,40,185,25,0,161,128,12,0,28,0,0,0,144,184,25,0,48,184,25,0,160,183,25,0,34,64,12,0,184,0,0,0,64,183,25,0,0,0,0,0,24,183,25,0,33,32,12,0,228,0,0,0,176,181,25,0,184,200,24,0,216,180,25,0,37,32,8,0,222,0,0,0,80,179,25,0,160,36,26,0,40,178,25,0,161,0,0,0,184,74,5,0,104,177,25,0,160,176,25,0,128,176,25,0,45,128,12,0,236,0,0,0,24,176,25,0,192,254,25,0,176,175,25,0,37,32,12,0,16,0,0,0,56,175,25,0,248,173,25,0,104,173,25,0,33,32,8,0,48,0,0,0,200,172,25,0,144,27,26,0,96,171,25,0,165,128,4,0,140,1,0,0,152,170,25,0,0,0,0,0,0,170,25,0,2,66,12,0,188,0,0,0,216,169,25,0,0,0,0,0,128,169,25,0,1,34,12,0,192,0,0,0,64,168,25,0,184,200,24,0,0,168,25,0,5,34,8,0,222,0,0,0,200,166,25,0,160,36,26,0,136,165,25,0,6,130,4,0,108,1,0,0,240,164,25,0,0,0,0,0,240,163,25,0,9,130,4,0,116,1,0,0,48,163,25,0,224,1,26,0,120,162,25,0,21,0,0,0,32,0,0,0,72,162,25,0,248,161,25,0,144,161,25,0,21,0,0,0,6,0,0,0,248,160,25,0,120,159,25,0,128,158,25,0,6,0,0,0,248,173,33,0,152,157,25,0,72,152,47,0,152,156,25,0,5,65,8,0,172,0,0,0,88,155,25,0,144,154,25,0,48,154,25,0,5,65,8,0,168,0,0,0,208,153,25,0,144,154,25,0,128,153,25,0,6,0,8,0,104,159,33,0,24,153,25,0,72,152,47,0,232,151,25,0,13,128,8,0,220,0,0,0,128,150,25,0,48,149,25,0,152,148,25,0,37,32,8,0,222,0,0,0,208,147,25,0,24,147,25,0,216,146,25,0,21,32,8,0,222,0,0,0,208,147,25,0,56,146,25,0,216,145,25,0,37,32,8,0,112,0,0,0,104,145,25,0,32,194,24,0,112,144,25,0,21,32,8,0,112,0,0,0,96,143,25,0,32,194,24,0,216,142,25,0,5,34,8,0,112,0,0,0,224,141,25,0,32,194,24,0,72,141,25,0,5,32,8,0,112,0,0,0,112,140,25,0,24,21,26,0,48,140,25,0,5,48,12,0,50,0,0,0,56,139,25,0,184,200,24,0,192,138,25,0,18,64,12,0,192,0,0,0,96,138,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,208,7,0,0,16,0,0,0,116,14,0,0,24,0,0,0,60,15,0,0,32,0,0,0,124,21,0,0,40,0,0,0,88,27,0,0,48,0,0,0,76,29,0,0,56,0,0,0,16,39,0,0,64,0,0,0,248,42,0,0,80,0,0,0,188,52,0,0,96,0,0,0,252,58,0,0,112,0,0,0,240,60,0,0,128,0,0,0,104,66,0,0,160,0,0,0,92,68,0,0,192,0,0,0,168,72,0,0,224,0,0,0,200,75,0,0,0,1,0,0,244,76,0,0,64,1,0,0,20,80,0,0,72,110,25,0,104,109,25,0,232,108,25,0,0,0,0,0,200,90,25,0,248,255,255,255,80,85,25,0,0,0,0,0,168,80,25,0,8,0,0,0,128,75,25,0,16,0,0,0,208,66,25,0,24,0,0,0,8,61,25,0,32,0,0,0,216,53,25,0,40,0,0,0,16,202,25,0,48,0,0,0,32,3,26,0,96,0,0,0,96,195,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,162,3,0,120,212,3,0,248,161,3,0,16,161,3,0,88,161,3,0,8,212,3,0,56,231,3,0,80,189,3,0,160,189,3,0,120,189,3,0,200,189,3,0,64,190,3,0,240,189,3,0,24,190,3,0,0,0,0,0,0,0,0,0,0,1,2,4,1,2,3,4,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,3,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,6,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,15,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,3,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,0,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,0,0,0,0,6,0,0,0,18,0,0,0,9,0,0,0,0,0,0,0,5,11,17,5,11,17,0,0,10,20,34,10,20,34,0,0,8,0,0,0,16,0,0,0,24,0,0,0,32,0,0,0,40,0,0,0,48,0,0,0,56,0,0,0,64,0,0,0,80,0,0,0,96,0,0,0,112,0,0,0,128,0,0,0,160,0,0,0,192,0,0,0,224,0,0,0,0,1,0,0,64,1,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,1,5,4,5,6,5,4,4,7,3,6,0,7,2,3,1,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,1,4,4,5,4,6,5,6,4,5,5,6,5,6,6,6,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,2,2,3,3,3,2,0,0,0,0,0,7,0,5,0,9,0,14,0,15,0,7,0,6,0,4,0,5,0,5,0,6,0,7,0,7,0,6,0,8,0,8,0,8,0,5,0,15,0,6,0,9,0,10,0,5,0,1,0,11,0,7,0,9,0,6,0,4,0,1,0,14,0,4,0,6,0,2,0,6,0,0,0,3,0,4,0,6,0,18,0,12,0,5,0,5,0,1,0,2,0,16,0,9,0,3,0,7,0,3,0,5,0,14,0,7,0,3,0,19,0,17,0,15,0,13,0,10,0,4,0,13,0,5,0,8,0,11,0,5,0,1,0,12,0,4,0,4,0,1,0,1,0,0,0,1,0,2,0,10,0,19,0,16,0,10,0,3,0,3,0,7,0,10,0,5,0,3,0,11,0,4,0,13,0,17,0,8,0,4,0,12,0,11,0,18,0,15,0,11,0,2,0,7,0,6,0,9,0,14,0,3,0,1,0,6,0,4,0,5,0,3,0,2,0,0,0,7,0,3,0,5,0,1,0,6,0,2,0,3,0,2,0,5,0,4,0,4,0,1,0,3,0,3,0,2,0,0,0,1,0,2,0,6,0,5,0,3,0,1,0,4,0,4,0,7,0,5,0,7,0,1,0,6,0,1,0,1,0,0,0,3,0,2,0,1,0,1,0,1,0,1,0,3,0,2,0,0,0,0,0,0,0,0,0,15,0,13,0,46,0,80,0,146,0,6,1,248,0,178,1,170,1,157,2,141,2,137,2,109,2,5,2,8,4,88,0,14,0,12,0,21,0,38,0,71,0,130,0,122,0,216,0,209,0,198,0,71,1,89,1,63,1,41,1,23,1,42,0,47,0,22,0,41,0,74,0,68,0,128,0,120,0,221,0,207,0,194,0,182,0,84,1,59,1,39,1,29,2,18,0,81,0,39,0,75,0,70,0,134,0,125,0,116,0,220,0,204,0,190,0,178,0,69,1,55,1,37,1,15,1,16,0,147,0,72,0,69,0,135,0,127,0,118,0,112,0,210,0,200,0,188,0,96,1,67,1,50,1,29,1,28,2,14,0,7,1,66,0,129,0,126,0,119,0,114,0,214,0,202,0,192,0,180,0,85,1,61,1,45,1,25,1,6,1,12,0,249,0,123,0,121,0,117,0,113,0,215,0,206,0,195,0,185,0,91,1,74,1,52,1,35,1,16,1,8,2,10,0,179,1,115,0,111,0,109,0,211,0,203,0,196,0,187,0,97,1,76,1,57,1,42,1,27,1,19,2,125,1,17,0,171,1,212,0,208,0,205,0,201,0,193,0,186,0,177,0,169,0,64,1,47,1,30,1,12,1,2,2,121,1,16,0,79,1,199,0,197,0,191,0,189,0,181,0,174,0,77,1,65,1,49,1,33,1,19,1,9,2,123,1,115,1,11,0,156,2,184,0,183,0,179,0,175,0,88,1,75,1,58,1,48,1,34,1,21,1,18,2,127,1,117,1,110,1,10,0,140,2,90,1,171,0,168,0,164,0,62,1,53,1,43,1,31,1,20,1,7,1,1,2,119,1,112,1,106,1,6,0,136,2,66,1,60,1,56,1,51,1,46,1,36,1,28,1,13,1,5,1,0,2,120,1,114,1,108,1,103,1,4,0,108,2,44,1,40,1,38,1,32,1,26,1,17,1,10,1,3,2,124,1,118,1,113,1,109,1,105,1,101,1,2,0,9,4,24,1,22,1,18,1,11,1,8,1,3,1,126,1,122,1,116,1,111,1,107,1,104,1,102,1,100,1,0,0,43,0,20,0,19,0,17,0,15,0,13,0,11,0,9,0,7,0,6,0,4,0,7,0,5,0,3,0,1,0,3,0,1,0,2,0,1,0,3,0,1,0,1,0,3,0,2,0,0,0,0,0,0,0,0,0,1,0,5,0,14,0,44,0,74,0,63,0,110,0,93,0,172,0,149,0,138,0,242,0,225,0,195,0,120,1,17,0,3,0,4,0,12,0,20,0,35,0,62,0,53,0,47,0,83,0,75,0,68,0,119,0,201,0,107,0,207,0,9,0,15,0,13,0,23,0,38,0,67,0,58,0,103,0,90,0,161,0,72,0,127,0,117,0,110,0,209,0,206,0,16,0,45,0,21,0,39,0,69,0,64,0,114,0,99,0,87,0,158,0,140,0,252,0,212,0,199,0,131,1,109,1,26,0,75,0,36,0,68,0,65,0,115,0,101,0,179,0,164,0,155,0,8,1,246,0,226,0,139,1,126,1,106,1,9,0,66,0,30,0,59,0,56,0,102,0,185,0,173,0,9,1,142,0,253,0,232,0,144,1,132,1,122,1,189,1,16,0,111,0,54,0,52,0,100,0,184,0,178,0,160,0,133,0,1,1,244,0,228,0,217,0,129,1,110,1,203,2,10,0,98,0,48,0,91,0,88,0,165,0,157,0,148,0,5,1,248,0,151,1,141,1,116,1,124,1,121,3,116,3,8,0,85,0,84,0,81,0,159,0,156,0,143,0,4,1,249,0,171,1,145,1,136,1,127,1,215,2,201,2,196,2,7,0,154,0,76,0,73,0,141,0,131,0,0,1,245,0,170,1,150,1,138,1,128,1,223,2,103,1,198,2,96,1,11,0,139,0,129,0,67,0,125,0,247,0,233,0,229,0,219,0,137,1,231,2,225,2,208,2,117,3,114,3,183,1,4,0,243,0,120,0,118,0,115,0,227,0,223,0,140,1,234,2,230,2,224,2,209,2,200,2,194,2,223,0,180,1,6,0,202,0,224,0,222,0,218,0,216,0,133,1,130,1,125,1,108,1,120,3,187,1,195,2,184,1,181,1,192,6,4,0,235,2,211,0,210,0,208,0,114,1,123,1,222,2,211,2,202,2,199,6,115,3,109,3,108,3,131,13,97,3,2,0,121,1,113,1,102,0,187,0,214,2,210,2,102,1,199,2,197,2,98,3,198,6,103,3,130,13,102,3,178,1,0,0,12,0,10,0,7,0,11,0,10,0,17,0,11,0,9,0,13,0,12,0,10,0,7,0])
.concat([5,0,3,0,1,0,3,0,7,0,12,0,18,0,53,0,47,0,76,0,124,0,108,0,89,0,123,0,108,0,119,0,107,0,81,0,122,0,63,0,13,0,5,0,16,0,27,0,46,0,36,0,61,0,51,0,42,0,70,0,52,0,83,0,65,0,41,0,59,0,36,0,19,0,17,0,15,0,24,0,41,0,34,0,59,0,48,0,40,0,64,0,50,0,78,0,62,0,80,0,56,0,33,0,29,0,28,0,25,0,43,0,39,0,63,0,55,0,93,0,76,0,59,0,93,0,72,0,54,0,75,0,50,0,29,0,52,0,22,0,42,0,40,0,67,0,57,0,95,0,79,0,72,0,57,0,89,0,69,0,49,0,66,0,46,0,27,0,77,0,37,0,35,0,66,0,58,0,52,0,91,0,74,0,62,0,48,0,79,0,63,0,90,0,62,0,40,0,38,0,125,0,32,0,60,0,56,0,50,0,92,0,78,0,65,0,55,0,87,0,71,0,51,0,73,0,51,0,70,0,30,0,109,0,53,0,49,0,94,0,88,0,75,0,66,0,122,0,91,0,73,0,56,0,42,0,64,0,44,0,21,0,25,0,90,0,43,0,41,0,77,0,73,0,63,0,56,0,92,0,77,0,66,0,47,0,67,0,48,0,53,0,36,0,20,0,71,0,34,0,67,0,60,0,58,0,49,0,88,0,76,0,67,0,106,0,71,0,54,0,38,0,39,0,23,0,15,0,109,0,53,0,51,0,47,0,90,0,82,0,58,0,57,0,48,0,72,0,57,0,41,0,23,0,27,0,62,0,9,0,86,0,42,0,40,0,37,0,70,0,64,0,52,0,43,0,70,0,55,0,42,0,25,0,29,0,18,0,11,0,11,0,118,0,68,0,30,0,55,0,50,0,46,0,74,0,65,0,49,0,39,0,24,0,16,0,22,0,13,0,14,0,7,0,91,0,44,0,39,0,38,0,34,0,63,0,52,0,45,0,31,0,52,0,28,0,19,0,14,0,8,0,9,0,3,0,123,0,60,0,58,0,53,0,47,0,43,0,32,0,22,0,37,0,24,0,17,0,12,0,15,0,10,0,2,0,1,0,71,0,37,0,34,0,30,0,28,0,20,0,17,0,26,0,21,0,16,0,10,0,6,0,8,0,6,0,2,0,0,0,1,0,5,0,14,0,21,0,34,0,51,0,46,0,71,0,42,0,52,0,68,0,52,0,67,0,44,0,43,0,19,0,3,0,4,0,12,0,19,0,31,0,26,0,44,0,33,0,31,0,24,0,32,0,24,0,31,0,35,0,22,0,14,0,15,0,13,0,23,0,36,0,59,0,49,0,77,0,65,0,29,0,40,0,30,0,40,0,27,0,33,0,42,0,16,0,22,0,20,0,37,0,61,0,56,0,79,0,73,0,64,0,43,0,76,0,56,0,37,0,26,0,31,0,25,0,14,0,35,0,16,0,60,0,57,0,97,0,75,0,114,0,91,0,54,0,73,0,55,0,41,0,48,0,53,0,23,0,24,0,58,0,27,0,50,0,96,0,76,0,70,0,93,0,84,0,77,0,58,0,79,0,29,0,74,0,49,0,41,0,17,0,47,0,45,0,78,0,74,0,115,0,94,0,90,0,79,0,69,0,83,0,71,0,50,0,59,0,38,0,36,0,15,0,72,0,34,0,56,0,95,0,92,0,85,0,91,0,90,0,86,0,73,0,77,0,65,0,51,0,44,0,43,0,42,0,43,0,20,0,30,0,44,0,55,0,78,0,72,0,87,0,78,0,61,0,46,0,54,0,37,0,30,0,20,0,16,0,53,0,25,0,41,0,37,0,44,0,59,0,54,0,81,0,66,0,76,0,57,0,54,0,37,0,18,0,39,0,11,0,35,0,33,0,31,0,57,0,42,0,82,0,72,0,80,0,47,0,58,0,55,0,21,0,22,0,26,0,38,0,22,0,53,0,25,0,23,0,38,0,70,0,60,0,51,0,36,0,55,0,26,0,34,0,23,0,27,0,14,0,9,0,7,0,34,0,32,0,28,0,39,0,49,0,75,0,30,0,52,0,48,0,40,0,52,0,28,0,18,0,17,0,9,0,5,0,45,0,21,0,34,0,64,0,56,0,50,0,49,0,45,0,31,0,19,0,12,0,15,0,10,0,7,0,6,0,3,0,48,0,23,0,20,0,39,0,36,0,35,0,53,0,21,0,16,0,23,0,13,0,10,0,6,0,1,0,4,0,2,0,16,0,15,0,17,0,27,0,25,0,20,0,29,0,11,0,17,0,12,0,16,0,8,0,1,0,1,0,0,0,1,0,9,0,6,0,16,0,33,0,41,0,39,0,38,0,26,0,7,0,5,0,6,0,9,0,23,0,16,0,26,0,11,0,17,0,7,0,11,0,14,0,21,0,30,0,10,0,7,0,17,0,10,0,15,0,12,0,18,0,28,0,14,0,5,0,32,0,13,0,22,0,19,0,18,0,16,0,9,0,5,0,40,0,17,0,31,0,29,0,17,0,13,0,4,0,2,0,27,0,12,0,11,0,15,0,10,0,7,0,4,0,1,0,27,0,12,0,8,0,12,0,6,0,3,0,1,0,0,0,3,0,4,0,10,0,24,0,34,0,33,0,21,0,15,0,5,0,3,0,4,0,10,0,32,0,17,0,11,0,10,0,11,0,7,0,13,0,18,0,30,0,31,0,20,0,5,0,25,0,11,0,19,0,59,0,27,0,18,0,12,0,5,0,35,0,33,0,31,0,58,0,30,0,16,0,7,0,5,0,28,0,26,0,32,0,19,0,17,0,15,0,8,0,14,0,14,0,12,0,9,0,13,0,14,0,9,0,4,0,1,0,11,0,4,0,6,0,6,0,6,0,3,0,2,0,0,0,1,0,2,0,10,0,23,0,35,0,30,0,12,0,17,0,3,0,3,0,8,0,12,0,18,0,21,0,12,0,7,0,11,0,9,0,15,0,21,0,32,0,40,0,19,0,6,0,14,0,13,0,22,0,34,0,46,0,23,0,18,0,7,0,20,0,19,0,33,0,47,0,27,0,22,0,9,0,3,0,31,0,22,0,41,0,26,0,21,0,20,0,5,0,3,0,14,0,13,0,10,0,11,0,16,0,6,0,5,0,1,0,9,0,8,0,7,0,8,0,4,0,4,0,2,0,0,0,1,0,1,0,1,0,0,0,3,3,5,6,8,9,3,3,4,5,6,8,4,4,5,6,7,8,6,5,6,7,7,8,7,6,7,7,8,9,8,7,8,8,9,9,0,0,0,0,2,3,6,8,8,9,3,2,4,8,8,8,6,4,6,8,8,9,8,8,8,9,9,10,8,7,8,9,10,10,9,8,9,9,11,11,0,0,0,0,1,3,6,8,8,9,3,4,6,7,7,8,6,5,7,8,8,9,7,7,8,9,9,9,7,7,8,9,9,10,8,8,9,10,10,10,0,0,0,0,3,3,5,7,3,2,4,5,4,4,5,6,6,5,6,7,1,3,6,7,3,3,6,7,6,6,7,8,7,6,7,8,2,2,6,3,2,5,5,5,6,0,0,0,0,0,0,0,4,4,6,7,8,9,9,10,10,11,11,11,11,11,12,9,4,4,5,6,7,8,8,9,9,9,10,10,10,10,10,8,6,5,6,7,7,8,8,9,9,9,9,10,10,10,11,7,7,6,7,7,8,8,8,9,9,9,9,10,10,10,10,7,8,7,7,8,8,8,8,9,9,9,10,10,10,10,11,7,9,7,8,8,8,8,9,9,9,9,10,10,10,10,10,7,9,8,8,8,8,9,9,9,9,10,10,10,10,10,11,7,10,8,8,8,9,9,9,9,10,10,10,10,10,11,11,8,10,9,9,9,9,9,9,9,9,10,10,10,10,11,11,8,10,9,9,9,9,9,9,10,10,10,10,10,11,11,11,8,11,9,9,9,9,10,10,10,10,10,10,11,11,11,11,8,11,10,9,9,9,10,10,10,10,10,10,11,11,11,11,8,11,10,10,10,10,10,10,10,10,10,11,11,11,11,11,8,11,10,10,10,10,10,10,10,11,11,11,11,11,11,11,8,12,10,10,10,10,10,10,11,11,11,11,11,11,11,11,8,8,7,7,7,7,7,7,7,7,7,7,8,8,8,8,4,1,3,6,3,3,5,5,5,6,0,0,0,0,0,0,0,1,4,6,8,9,9,10,10,11,11,11,12,12,12,13,9,3,4,6,7,8,9,9,9,10,10,10,11,12,11,12,8,6,6,7,8,9,9,10,10,11,10,11,11,11,12,12,9,8,7,8,9,9,10,10,10,11,11,12,12,12,13,13,10,9,8,9,9,10,10,11,11,11,12,12,12,13,13,13,9,9,8,9,9,10,11,11,12,11,12,12,13,13,13,14,10,10,9,9,10,11,11,11,11,12,12,12,12,13,13,14,10,10,9,10,10,11,11,11,12,12,13,13,13,13,15,15,10,10,10,10,11,11,11,12,12,13,13,13,13,14,14,14,10,11,10,10,11,11,12,12,13,13,13,13,14,13,14,13,11,11,11,10,11,12,12,12,12,13,14,14,14,15,15,14,10,12,11,11,11,12,12,13,14,14,14,14,14,14,13,14,11,12,12,12,12,12,13,13,13,13,15,14,14,14,14,16,11,14,12,12,12,13,13,14,14,14,16,15,15,15,17,15,11,13,13,11,12,14,14,13,14,14,15,16,15,17,15,14,11,9,8,8,9,9,10,10,10,11,11,11,11,11,11,11,8,3,4,5,7,7,8,9,9,9,10,10,11,11,11,12,13,4,3,5,6,7,7,8,8,8,9,9,10,10,10,11,11,5,5,5,6,7,7,8,8,8,9,9,10,10,11,11,11,6,6,6,7,7,8,8,9,9,9,10,10,10,11,11,11,7,6,7,7,8,8,9,9,9,9,10,10,10,11,11,11,8,7,7,8,8,8,9,9,9,9,10,10,11,11,11,12,9,7,8,8,8,9,9,9,9,10,10,10,11,11,12,12,9,8,8,9,9,9,9,10,10,10,10,10,11,11,11,12,9,8,8,9,9,9,9,10,10,10,10,11,11,12,12,12,9,8,9,9,9,9,10,10,10,11,11,11,11,12,12,12,10,9,9,9,10,10,10,10,10,11,11,11,11,12,13,12,10,9,9,9,10,10,10,10,11,11,11,11,12,12,12,13,11,10,9,10,10,10,11,11,11,11,11,11,12,12,13,13,11,10,10,10,10,11,11,11,11,12,12,12,12,12,13,13,12,11,11,11,11,11,11,11,12,12,12,12,13,13,12,13,12,11,11,11,11,11,11,12,12,12,12,12,13,13,13,13,1,4,6,7,8,9,9,10,9,10,11,11,12,12,13,13,3,4,6,7,8,8,9,9,9,9,10,10,11,12,12,12,6,6,7,8,9,9,10,10,9,10,10,11,11,12,13,13,7,7,8,9,9,10,10,10,10,11,11,11,11,12,13,13,8,7,9,9,10,10,11,11,10,11,11,12,12,13,13,14,9,8,9,10,10,10,11,11,11,11,12,11,13,13,14,14,9,9,10,10,11,11,11,11,11,12,12,12,13,13,14,14,10,9,10,11,11,11,12,12,12,12,13,13,13,14,16,16,9,8,9,10,10,11,11,12,12,12,12,13,13,14,15,15,10,9,10,10,11,11,11,13,12,13,13,14,14,14,16,15,10,10,10,11,11,12,12,13,12,13,14,13,14,15,16,17,11,10,10,11,12,12,12,12,13,13,13,14,15,15,15,16,11,11,11,12,12,13,12,13,14,14,15,15,15,16,16,16,12,11,12,13,13,13,14,14,14,14,14,15,16,15,16,16,13,12,12,13,13,13,15,14,14,17,15,15,15,17,16,16,12,12,13,14,14,14,15,14,15,15,16,16,19,18,19,16,4,3,5,7,8,9,9,9,3,3,4,5,7,7,8,8,5,4,5,6,7,8,7,8,6,5,6,6,7,8,8,8,7,6,7,7,8,8,8,9,8,7,8,8,8,9,8,9,8,7,7,8,8,9,9,10,9,8,8,9,9,9,9,10,2,3,5,7,8,9,8,9,3,3,4,6,8,8,7,8,5,5,6,7,8,9,8,8,7,6,7,9,8,10,8,9,8,8,8,9,9,10,9,10,8,8,9,10,10,11,10,11,8,7,7,8,9,10,10,10,8,7,8,9,10,10,10,10,1,3,6,8,9,9,9,10,3,4,6,7,8,9,8,8,6,6,7,8,9,10,9,9,7,7,8,9,10,10,9,10,8,8,9,10,10,10,10,10,9,9,10,10,11,11,10,11,8,8,9,10,10,10,11,11,9,8,9,10,10,11,11,11,1,3,2,3,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,16,235,1,0,144,229,1,0,3,0,0,0,64,231,1,0,248,221,1,0,3,0,0,0,48,230,1,0,224,219,1,0,4,0,0,0,32,230,1,0,192,219,1,0,4,0,0,0,16,230,1,0,160,219,1,0,6,0,0,0,232,229,1,0,88,219,1,0,6,0,0,0,192,229,1,0,16,219,1,0,6,0,0,0,152,229,1,0,200,218,1,0,8,0,0,0,208,234,1,0,16,229,1,0,8,0,0,0,144,234,1,0,144,228,1,0,8,0,0,0,80,234,1,0,16,228,1,0,16,0,0,0,80,233,1,0,16,226,1,0,16,0,0,0,80,232,1,0,16,224,1,0,16,0,0,0,80,231,1,0,16,222,1,0,16,0,0,0,64,230,1,0,248,219,1,0,0,0,1,0,2,0,3,0,0,0,4,0,5,0,6,0,7,0,8,0,9,0,10,0,11,0,12,0,0,0,13,0,14,1,14,2,14,3,14,4,14,6,14,8,14,10,14,13,15,4,15,5,15,6,15,7,15,8,15,9,15,11,15,13,216,156,26,0,96,0,0,0,16,197,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,80,1,0,29,0,0,0,14,80,1,0,32,0,0,0,14,80,1,0,33,0,0,0,14,80,1,0,34,0,0,0,46,80,1,0,36,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,208,198,25,0,96,0,0,0,32,196,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,109,29,155,5,66,213,68,230,128,226,20,29,175,247,87,178,212,128,126,242,202,57,70,149,142,84,38,203,158,70,167,159,165,212,11,48,232,20,17,221,186,47,8,0,32,12,154,102,165,212,11,48,232,20,17,221,186,47,8,0,32,12,154,102,1,0,0,0,121,117,118,50,0,0,0,0,1,0,0,0,121,117,118,115,0,0,0,0,17,0,0,0,50,118,117,121,0,0,0,0,45,0,0,0,114,97,119,32,16,0,0,0,46,0,0,0,76,53,53,53,16,0,0,0,44,0,0,0,76,53,54,53,16,0,0,0,43,0,0,0,66,53,54,53,16,0,0,0,31,0,0,0,98,49,54,103,16,0,0,0,2,0,0,0,114,97,119,32,24,0,0,0,3,0,0,0,50,52,66,71,24,0,0,0,27,0,0,0,114,97,119,32,32,0,0,0,30,0,0,0,66,71,82,65,32,0,0,0,28,0,0,0,82,71,66,65,32,0,0,0,29,0,0,0,65,66,71,82,32,0,0,0,41,0,0,0,98,52,56,114,48,0,0,0,0,0,0,0,72,156,25,0,96,0,0,0,16,197,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,101,110,103,0,102,114,97,0,103,101,114,0,105,116,97,0,100,117,116,0,115,118,101,0,115,112,97,0,100,97,110,0,112,111,114,0,110,111,114,0,104,101,98,0,106,112,110,0,97,114,97,0,102,105,110,0,103,114,101,0,105,99,101,0,109,108,116,0,116,117,114,0,104,114,32,0,99,104,105,0,117,114,100,0,104,105,110,0,116,104,97,0,107,111,114,0,108,105,116,0,112,111,108,0,104,117,110,0,101,115,116,0,108,97,118,0,0,0,0,0,102,111,32,0,0,0,0,0,114,117,115,0,99,104,105,0,0,0,0,0,105,114,105,0,97,108,98,0,114,111,110,0,99,101,115,0,115,108,107,0,115,108,118,0,121,105,100,0,115,114,32,0,109,97,99,0,98,117,108,0,117,107,114,0,98,101,108,0,117,122,98,0,107,97,122,0,97,122,101,0,97,122,101,0,97,114,109,0,103,101,111,0,109,111,108,0,107,105,114,0,116,103,107,0,116,117,107,0,109,111,110,0,0,0,0,0,112,117,115,0,107,117,114,0,107,97,115,0,115,110,100,0,116,105,98,0,110,101,112,0,115,97,110,0,109,97,114,0,98,101,110,0,97,115,109,0,103,117,106,0,112,97,32,0,111,114,105,0,109,97,108,0,107,97,110,0,116,97,109,0,116,101,108,0,0,0,0,0,98,117,114,0,107,104,109,0,108,97,111,0,118,105,101,0,105,110,100,0,116,103,108,0,109,97,121,0,109,97,121,0,97,109,104,0,116,105,114,0,111,114,109,0,115,111,109,0,115,119,97,0,0,0,0,0,114,117,110,0,0,0,0,0,109,108,103,0,101,112,111,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,119,101,108,0,98,97,113,0,99,97,116,0,108,97,116,0,113,117,101,0,103,114,110,0,97,121,109,0,116,97,116,0,117,105,103,0,100,122,111,0,106,97,118,0,0,0,0,0,65,67,76,82,4,0,0,0,65,80,82,71,4,0,0,0,65,65,76,80,4,0,0,0,65,82,69,83,172,0,0,0,97,118,115,115,90,0,0,0,99,104,112,108,116,0,0,0,99,111,54,52,2,0,0,0,99,116,116,115,138,0,0,0,100,105,110,102,88,0,0,0,100,114,101,102,188,0,0,0,101,100,116,115,88,0,0,0,101,108,115,116,62,0,0,0,101,110,100,97,190,0,0,0,102,105,101,108,84,0,0,0,102,116,121,112,140,0,0,0,103,108,98,108,54,0,0,0,104,100,108,114,124,0,0,0,105,108,115,116,144,0,0,0,106,112,50,104,104,0,0,0,109,100,97,116,210,0,0,0,109,100,104,100,186,0,0,0,109,100,105,97,88,0,0,0,109,101,116,97,168,0,0,0,109,105,110,102,88,0,0,0,109,111,111,102,196,0,0,0,109,111,111,118,194,0,0,0,109,118,101,120,88,0,0,0,109,118,104,100,26,0,0,0,83,77,73,32,152,0,0,0,97,108,97,99,214,0,0,0,97,118,99,67,54,0,0,0,112,97,115,112,156,0,0,0,115,116,98,108,88,0,0,0,115,116,99,111,2,0,0,0,115,116,112,115,118,0,0,0,115,116,114,102,12,0,0,0,115,116,115,99,72,0,0,0,115,116,115,100,70,0,0,0,115,116,115,115,76,0,0,0,115,116,115,122,74,0,0,0,115,116,116,115,150,0,0,0,115,116,122,50,74,0,0,0,116,107,104,100,82,0,0,0,116,102,104,100,34,0,0,0,116,114,97,107,22,0,0,0,116,114,97,102,88,0,0,0,116,114,101,102,88,0,0,0,116,109,99,100,52,0,0,0,99,104,97,112,60,0,0,0,116,114,101,120,66,0,0,0,116,114,117,110,180,0,0,0,117,100,116,97,88,0,0,0,119,97,118,101,64,0,0,0,101,115,100,115,142,0,0,0,100,97,99,51,158,0,0,0,100,101,99,51,96,0,0,0,119,105,100,101,176,0,0,0,119,102,101,120,44,0,0,0,99,109,111,118,146,0,0,0,99,104,97,110,56,0,0,0,100,118,99,49,224,0,0,0,115,98,103,112,58,0,0,0,117,117,105,100,18,0,0,0,0,0,0,0,0,0,0,0,2,80,1,0,104,243,1,0,3,80,1,0,40,243,1,0,16,80,1,0,0,243,1,0,5,0,1,0,208,242,1,0,4,0,1,0,208,242,1,0,0,0,1,0,208,242,1,0,1,0,1,0,208,242,1,0,12,0,1,0,208,242,1,0,13,0,1,0,208,242,1,0,8,0,1,0,208,242,1,0,9,0,1,0,208,242,1,0,21,0,1,0,208,242,1,0,20,0,1,0,208,242,1,0,23,0,1,0,208,242,1,0,22,0,1,0,208,242,1,0,0,0,0,0,0,0,0,0,248,156,26,0,96,0,0,0,144,201,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,100,0,2,0,101,0,2,0,103,0,3,0,113,0,4,0,108,0,5,0,117,0,6,0,121,0,7,0,125,0,8,0,126,0,8,0,128,0,8,0,130,0,0,0,0,0,1,0,100,0,2,0,101,0,3,0,114,0,4,0,116,0,5,0,120,0,6,0,124,0,7,0,142,0,8,0,127,0,0,0,0,0,0,0,0,0,1,0,100,0,2,0,101,0,2,0,149,0,3,0,150,0,3,0,131,0,3,0,133,0,4,0,151,0,4,0,132,0,4,0,153,0,4,0,152,0,5,0,119,0,5,0,138,0,5,0,154,0,6,0,123,0,0,0,0,0,0,0,0,0,1,0,100,0,2,0,101,0,2,0,149,0,3,0,114,0,3,0,131,0,3,0,133,0,4,0,108,0,4,0,116,0,4,0,132,0,4,0,153,0,4,0,168,0,5,0,120,0,5,0,138,0,5,0,169,0,6,0,124,0,6,0,141,0,6,0,170,0,7,0,142,0,7,0,143,0,7,0,173,0,8,0,144,0,8,0,127,0,8,0,178,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,147,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,0,0,0,0,0,0,0,0,0,0,0,0,16,0,145,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,146,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,180,0,0,0,0,0,251,6,0,0,0,0,0,0,9,0,181,0,0,0,0,0,207,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,111,0,0,0,0,0,55,7,0,0,0,0,0,0,8,0,144,0,0,0,0,0,55,7,0,0,0,0,0,0,8,0,112,0,0,0,0,0,51,208,2,0,0,0,0,0,8,0,126,0,0,0,0,0,207,6,0,0,0,0,0,0,8,0,127,0,0,0,0,0,207,6,0,0,0,0,0,0,8,0,129,0,0,0,0,0,207,6,0,0,0,0,0,0,8,0,161,0,0,0,0,0,207,6,0,0,0,0,0,0,8,0,177,0,0,0,0,0,207,6,0,0,0,0,0,0,8,0,128,0,0,0,0,0,63,6,0,0,0,0,0,0,8,0,160,0,0,0,0,0,63,6,0,0,0,0,0,0,8,0,130,0,0,0,0,0,15,6,0,96,0,0,0,0,8,0,162,0,0,0,0,0,15,6,0,0,6,0,0,0,8,0,163,0,0,0,0,0,15,6,0,128,1,0,0,0,8,0,164,0,0,0,0,0,15,86,0,0,0,0,0,0,8,0,165,0,0,0,0,0,15,15,0,0,0,0,0,0,8,0,166,0,0,0,0,0,15,39,0,0,0,0,0,0,8,0,167,0,0,0,0,0,15,46,0,0,0,0,0,0,8,0,178,0,0,0,0,0,243,6,0,0,0,0,0,0,8,0,179,0,0,0,0,0,199,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,125,0,0,0,0,0,15,7,0,0,0,0,0,0,7,0,142,0,0,0,0,0,15,7,0,0,0,0,0,0,7,0,157,0,0,0,0,0,15,7,0,0,0,0,0,0,7,0,182,0,0,0,0,0,15,7,0,0,0,0,0,0,7,0,140,0,0,0,0,0,55,6,0,0,0,0,0,0,7,0,143,0,0,0,0,0,55,6,0,0,0,0,0,0,7,0,156,0,0,0,0,0,55,6,0,0,0,0,0,0,7,0,148,0,0,0,0,0,199,6,0,0,0,0,0,0,7,0,176,0,0,0,0,0,199,6,0,0,0,0,0,0,7,0,158,0,0,0,0,0,15,14,0,0,0,0,0,0,7,0,159,0,0,0,0,0,15,38,0,0,0,0,0,0,7,0,173,0,0,0,0,0,203,6,0,0,0,0,0,0,7,0,174,0,0,0,0,0,63,8,0,0,0,0,0,0,7,0,175,0,0,0,0,0,63,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,110,0,0,0,0,0,55,1,0,0,0,0,0,0,6,0,172,0,0,0,0,0,55,1,0,0,0,0,0,0,6,0,121,0,0,0,0,0,15,6,0,0,0,0,0,0,6,0,122,0,0,0,0,0,15,6,0,0,0,0,0,0,6,0,123,0,0,0,0,0,15,6,0,0,0,0,0,0,6,0,124,0,0,0,0,0,15,6,0,0,0,0,0,0,6,0,139,0,0,0,0,0,7,7,0,0,0,0,0,0,6,0,141,0,0,0,0,0,7,7,0,0,0,0,0,0,6,0,155,0,0,0,0,0,7,7,0,0,0,0,0,0,6,0,170,0,0,0,0,0,195,6,0,0,0,0,0,0,6,0,171,0,0,0,0,0,55,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,109,0,0,0,0,0,55,0,0,0,0,0,0,0,5,0,117,0,0,0,0,0,7,6,0,0,0,0,0,0,5,0,118,0,0,0,0,0,7,6,0,0,0,0,0,0,5,0,119,0,0,0,0,0,7,6,0,0,0,0,0,0,5,0,120,0,0,0,0,0,7,6,0,0,0,0,0,0,5,0,135,0,0,0,0,0,11,6,0,0,0,0,0,0,5,0,138,0,0,0,0,0,11,6,0,0,0,0,0,0,5,0,137,0,0,0,0,0,15,1,0,0,0,0,0,0,5,0,154,0,0,0,0,0,15,1,0,0,0,0,0,0,5,0,169,0,0,0,0,0,15,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,107,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,108,0,0,0,0,0,51,0,0,0,0,0,0,0,4,0,115,0,0,0,0,0,7,1,0,0,0,0,0,0,4,0,116,0,0,0,0,0,7,1,0,0,0,0,0,0,4,0,151,0,0,0,0,0,7,1,0,0,0,0,0,0,4,0,132,0,0,0,0,0,3,6,0,0,0,0,0,0,4,0,134,0,0,0,0,0,11,1,0,0,0,0,0,0,4,0,153,0,0,0,0,0,11,1,0,0,0,0,0,0,4,0,136,0,0,0,0,0,15,0,0,0,0,0,0,0,4,0,152,0,0,0,0,0,15,0,0,0,0,0,0,0,4,0,168,0,0,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,113,0,0,0,0,0,7,0,0,0,0,0,0,0,3,0,114,0,0,0,0,0,7,0,0,0,0,0,0,0,3,0,150,0,0,0,0,0,7,0,0,0,0,0,0,0,3,0,131,0,0,0,0,0,3,1,0,0,0,0,0,0,3,0,133,0,0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,101,0,0,0,0,0,3,0,0,0,0,0,0,0,2,0,102,0,0,0,0,0,3,0,0,0,0,0,0,0,2,0,106,0,0,0,0,0,3,0,0,0,0,0,0,0,2,0,104,0,0,0,0,0,3,0,0,0,0,0,0,0,2,0,105,0,0,0,0,0,3,0,0,0,0,0,0,0,2,0,103,0,0,0,0,0,0,0,0,96,0,0,0,0,2,0,149,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,100,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,200,243,1,0,168,249,1,0,40,249,1,0,200,248,1,0,8,248,1,0,88,247,1,0,152,246,1,0,168,245,1,0,104,244,1,0,56,244,1,0,100,59,223,79,141,151,250,63,104,145,237,124,63,53,244,191,106,222,113,138,142,228,222,63,134,201,84,193,168,164,210,191,151,255,144,126,251,58,192,63,188,5,18,20,63,198,188,191,17,141,238,32,118,166,160,63,99,238,90,66,62,232,137,191,9,167,5,47,250,10,162,191,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,0,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,7,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,31,0,0,0,31,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,7,7,7,7,7,7,3,3,3,3,3,0,0,0,0,0,0,0,0,0,0,0,0,0,15,15,15,15,15,15,15,15,15,15,15,7,7,7,7,7,7,7,7,7,7,0,0,0,255,255,255,127,0,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,7,0,0,0,15,0,0,0,31,0,0,0,63,0,0,0,127,0,0,0,255,0,0,0,255,1,0,0,255,3,0,0,255,7,0,0,255,15,0,0,255,31,0,0,255,63,0,0,255,127,0,0,255,255,0,0,255,255,1,0,255,255,3,0,255,255,7,0,255,255,15,0,255,255,31,0,255,255,63,0,255,255,127,0,255,255,255,0,255,255,255,1,255,255,255,3,255,255,255,7,255,255,255,15,255,255,255,31,255,255,255,63,255,255,255,127,255,255,255,255,0,0,0,0,14,0,0,0,64,0,0,0,44,0,0,0,68,0,0,0,62,0,0,0,0,0,0,0,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,62,255,255,255,63,52,53,54,55,56,57,58,59,60,61,255,255,255,254,255,255,255,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,255,255,255,255,255,255,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,196,0,0,0,197,0,0,0,199,0,0,0,201,0,0,0,209,0,0,0,214,0,0,0,220,0,0,0,225,0,0,0,224,0,0,0,226,0,0,0,228,0,0,0,227,0,0,0,229,0,0,0,231,0,0,0,233,0,0,0,232,0,0,0,234,0,0,0,235,0,0,0,237,0,0,0,236,0,0,0,238,0,0,0,239,0,0,0,241,0,0,0,243,0,0,0,242,0,0,0,244,0,0,0,246,0,0,0,245,0,0,0,250,0,0,0,249,0,0,0,251,0,0,0,252,0,0,0,32,32,0,0,176,0,0,0,162,0,0,0,163,0,0,0,167,0,0,0,34,32,0,0,182,0,0,0,223,0,0,0,174,0,0,0,169,0,0,0,34,33,0,0,180,0,0,0,168,0,0,0,96,34,0,0,198,0,0,0,216,0,0,0,30,34,0,0,177,0,0,0,100,34,0,0,101,34,0,0,165,0,0,0,181,0,0,0,2,34,0,0,17,34,0,0,15,34,0,0,192,3,0,0,43,34,0,0,170,0,0,0,186,0,0,0,169,3,0,0,230,0,0,0,248,0,0,0,191,0,0,0,161,0,0,0,172,0,0,0,26,34,0,0,146,1,0,0,72,34,0,0,6,34,0,0,171,0,0,0,187,0,0,0,38,32,0,0,160,0,0,0,192,0,0,0,195,0,0,0,213,0,0,0,82,1,0,0,83,1,0,0,19,32,0,0,20,32,0,0,28,32,0,0,29,32,0,0,24,32,0,0,25,32,0,0,247,0,0,0,202,37,0,0,255,0,0,0,120,1,0,0,68,32,0,0,172,32,0,0,57,32,0,0,58,32,0,0,1,251,0,0,2,251,0,0,33,32,0,0,183,0,0,0,26,32,0,0,30,32,0,0,48,32,0,0,194,0,0,0,202,0,0,0,193,0,0,0,203,0,0,0,200,0,0,0,205,0,0,0,206,0,0,0,207,0,0,0,204,0,0,0,211,0,0,0,212,0,0,0,255,248,0,0,210,0,0,0,218,0,0,0,219,0,0,0,217,0,0,0,49,1,0,0,198,2,0,0,220,2,0,0,175,0,0,0,216,2,0,0,217,2,0,0,218,2,0,0,184,0,0,0,221,2,0,0,219,2,0,0,199,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,217,33,18,63,109,85,50,63,8,33,80,63,56,75,105,63,104,34,124,63,192,176,136,63,176,232,152,63,219,76,175,63,6,5,5,5,9,9,9,9,6,9,9,9,6,5,7,3,9,9,12,6,6,9,12,6,11,10,0,0,18,18,0,0,15,18,0,0,7,7,7,0,12,12,12,0,6,15,12,0,6,6,6,3,12,9,9,6,6,12,9,6,8,8,5,0,15,12,9,0,6,18,9,0,1,0,0,0,0,0,0,0,221,36,6,129,149,67,0,64,82,184,30,133,235,81,1,192,242,210,77,98,16,88,255,63,113,61,10,215,163,112,249,191,204,238,201,195,66,173,227,63,68,172,0,0,128,187,0,0,0,125,0,0,34,86,0,0,192,93,0,0,128,62,0,0,17,43,0,0,224,46,0,0,64,31,0,0,0,0,0,0,0,201,24,0,248,177,24,0,0,0,0,0,0,0,0,0,80,160,24,0,96,0,0,0,240,186,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,214,0,72,0,42,0,40,0,18,0,4,0,4,0,2,0,2,0,2,0,2,0,2,0,1,0,1,0,2,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,113,0,68,0,49,0,42,0,40,0,32,0,27,0,15,0,10,0,5,0,3,0,3,0,3,0,3,0,2,0,2,0,2,0,2,0,2,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,0,0,0,0,95,1,122,0,76,0,61,0,41,0,42,0,24,0,30,0,22,0,19,0,11,0,9,0,10,0,8,0,5,0,5,0,4,0,5,0,5,0,3,0,3,0,3,0,3,0,3,0,3,0,3,0,2,0,2,0,3,0,2,0,2,0,2,0,3,0,3,0,2,0,2,0,2,0,3,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,1,0,1,0,2,0,2,0,1,0,2,0,1,0,2,0,2,0,2,0,2,0,2,0,2,0,1,0,2,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,2,0,2,0,1,0,2,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,181,0,110,0,78,0,63,0,61,0,62,0,60,0,61,0,33,0,41,0,41,0,19,0,17,0,19,0,12,0,11,0,9,0,11,0,10,0,6,0,8,0,7,0,6,0,4,0,5,0,5,0,4,0,4,0,3,0,4,0,3,0,5,0,3,0,4,0,3,0,3,0,3,0,3,0,3,0,3,0,2,0,2,0,4,0,2,0,3,0,2,0,3,0,3,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,3,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,1,0,2,0,1,0,2,0,2,0,2,0,2,0,1,0,2,0,1,0,1,0,1,0,2,0,2,0,1,0,2,0,1,0,2,0,2,0,2,0,2,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,55,1,91,0,61,0,28,0,10,0,6,0,5,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,61,1,92,0,62,0,60,0,19,0,17,0,10,0,7,0,6,0,5,0,5,0,3,0,3,0,3,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,1,0,2,0,2,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,0,0,1,0,160,0,167,0,168,0,169,0,175,0,176,0,178,0,183,0,184,0,198,0,215,0,216,0,230,0,247,0,248,0,0,1,1,1,2,1,3,1,4,1,5,1,6,1,7,1,8,1,9,1,10,1,11,1,12,1,13,1,14,1,15,1,16,1,17,1,18,1,19,1,22,1,23,1,24,1,25,1,26,1,27,1,28,1,29,1,30,1,31,1,32,1,33,1,34,1,35,1,36,1,37,1,38,1,39,1,40,1,41,1,42,1,43,1,46,1,47,1,48,1,49,1,52,1,53,1,54,1,55,1,56,1,57,1,58,1,59,1,60,1,61,1,62,1,65,1,66,1,67,1,68,1,69,1,70,1,71,1,72,1,74,1,75,1,76,1,77,1,80,1,81,1,82,1,83,1,84,1,85,1,86,1,87,1,88,1,89,1,90,1,91,1,92,1,93,1,94,1,95,1,96,1,97,1,98,1,99,1,100,1,101,1,102,1,103,1,104,1,105,1,106,1,107,1,108,1,109,1,110,1,111,1,112,1,113,1,114,1,115,1,116,1,117,1,118,1,119,1,120,1,121,1,122,1,123,1,124,1,125,1,126,1,146,1,160,1,161,1,175,1,176,1,24,2,25,2,26,2,27,2,198,2,199,2,216,2,217,2,219,2,220,2,221,2,0,3,1,3,3,3,9,3,35,3,122,3,132,3,133,3,134,3,136,3,137,3,138,3,140,3,142,3,143,3,144,3,145,3,146,3,147,3,148,3,149,3,150,3,151,3,152,3,153,3,154,3,155,3,156,3,157,3,158,3,159,3,160,3,161,3,163,3,164,3,165,3,166,3,167,3,168,3,169,3,170,3,171,3,172,3,173,3,174,3,175,3,176,3,177,3,178,3,179,3,180,3,181,3,182,3,183,3,184,3,185,3,186,3,187,3,188,3,189,3,190,3,191,3,192,3,193,3,194,3,195,3,196,3,197,3,198,3,199,3,200,3,201,3,202,3,203,3,204,3,205,3,206,3,1,4,2,4,3,4,4,4,5,4,6,4,7,4,8,4,9,4,10,4,11,4,12,4,14,4,15,4,16,4,17,4,18,4,19,4,20,4,21,4,22,4,23,4,24,4,25,4,26,4,27,4,28,4,29,4,30,4,31,4,32,4,33,4,34,4,35,4,36,4,37,4,38,4,39,4,40,4,41,4,42,4,43,4,44,4,45,4,46,4,47,4,48,4,49,4,50,4,51,4,52,4,53,4,54,4,55,4,56,4,57,4,58,4,59,4,60,4,61,4,62,4,63,4,64,4,65,4,66,4,67,4,68,4,69,4,70,4,71,4,72,4,73,4,74,4,75,4,76,4,77,4,78,4,79,4,81,4,82,4,83,4,84,4,85,4,86,4,87,4,88,4,89,4,90,4,91,4,92,4,94,4,95,4,144,4,145,4,176,5,177,5,178,5,179,5,180,5,181,5,182,5,183,5,184,5,185,5,187,5,188,5,189,5,190,5,191,5,192,5,193,5,194,5,195,5,208,5])
.concat([209,5,210,5,211,5,212,5,213,5,214,5,215,5,216,5,217,5,218,5,219,5,220,5,221,5,222,5,223,5,224,5,225,5,226,5,227,5,228,5,229,5,230,5,231,5,232,5,233,5,234,5,240,5,241,5,242,5,243,5,244,5,12,6,27,6,31,6,33,6,34,6,35,6,36,6,37,6,38,6,39,6,40,6,41,6,42,6,43,6,44,6,45,6,46,6,47,6,48,6,49,6,50,6,51,6,52,6,53,6,54,6,55,6,56,6,57,6,58,6,64,6,65,6,66,6,67,6,68,6,69,6,70,6,71,6,72,6,73,6,74,6,75,6,76,6,77,6,78,6,79,6,80,6,81,6,82,6,121,6,126,6,134,6,136,6,145,6,152,6,169,6,175,6,186,6,190,6,193,6,210,6,1,14,2,14,3,14,4,14,5,14,6,14,7,14,8,14,9,14,10,14,11,14,12,14,13,14,14,14,15,14,16,14,17,14,18,14,19,14,20,14,21,14,22,14,23,14,24,14,25,14,26,14,27,14,28,14,29,14,30,14,31,14,32,14,33,14,34,14,35,14,36,14,37,14,38,14,39,14,40,14,41,14,42,14,43,14,44,14,45,14,46,14,47,14,48,14,49,14,50,14,51,14,52,14,53,14,54,14,55,14,56,14,57,14,58,14,63,14,64,14,65,14,66,14,67,14,68,14,69,14,70,14,71,14,72,14,73,14,74,14,75,14,76,14,77,14,78,14,79,14,80,14,81,14,82,14,83,14,84,14,85,14,86,14,87,14,88,14,89,14,90,14,91,14,2,30,3,30,10,30,11,30,30,30,31,30,64,30,65,30,86,30,87,30,96,30,97,30,106,30,107,30,128,30,129,30,130,30,131,30,132,30,133,30,242,30,243,30,12,32,13,32,14,32,15,32,19,32,20,32,21,32,23,32,24,32,25,32,26,32,28,32,29,32,30,32,32,32,33,32,34,32,38,32,48,32,57,32,58,32,170,32,171,32,172,32,175,32,22,33,34,33,25,34,26,34,72,34,100,34,101,34,32,35,33,35,0,37,2,37,12,37,16,37,20,37,24,37,28,37,36,37,44,37,52,37,60,37,80,37,81,37,82,37,83,37,84,37,85,37,86,37,87,37,88,37,89,37,90,37,91,37,92,37,93,37,94,37,95,37,96,37,97,37,98,37,99,37,100,37,101,37,102,37,103,37,104,37,105,37,106,37,107,37,108,37,128,37,132,37,136,37,140,37,144,37,145,37,146,37,147,37,160,37,0,0,4,0,1,0,5,0,5,0,7,0,7,0,8,0,9,0,9,0,10,0,10,0,10,0,10,0,11,0,11,0,11,0,11,0,12,0,12,0,12,0,12,0,12,0,12,0,13,0,12,0,13,0,12,0,13,0,13,0,14,0,10,0,10,0,5,0,4,0,6,0,6,0,7,0,8,0,8,0,9,0,9,0,10,0,10,0,11,0,10,0,11,0,11,0,11,0,11,0,12,0,11,0,12,0,12,0,12,0,12,0,13,0,12,0,14,0,12,0,13,0,12,0,14,0,10,0,10,0,7,0,7,0,7,0,8,0,8,0,9,0,9,0,10,0,9,0,11,0,10,0,11,0,10,0,12,0,11,0,12,0,11,0,13,0,11,0,12,0,11,0,13,0,12,0,13,0,12,0,13,0,12,0,14,0,13,0,14,0,9,0,11,0,8,0,9,0,8,0,9,0,9,0,10,0,9,0,11,0,10,0,11,0,10,0,12,0,10,0,12,0,11,0,12,0,11,0,13,0,11,0,13,0,11,0,14,0,12,0,14,0,12,0,14,0,12,0,15,0,12,0,15,0,9,0,12,0,9,0,10,0,9,0,10,0,9,0,11,0,10,0,11,0,10,0,12,0,10,0,12,0,10,0,13,0,11,0,13,0,11,0,13,0,11,0,14,0,12,0,14,0,12,0,14,0,12,0,15,0,12,0,15,0,13,0,15,0,9,0,11,0,10,0,10,0,9,0,10,0,10,0,11,0,10,0,11,0,10,0,12,0,10,0,13,0,11,0,13,0,11,0,14,0,11,0,13,0,11,0,14,0,12,0,14,0,12,0,15,0,12,0,15,0,12,0,15,0,12,0,16,0,9,0,12,0,10,0,11,0,10,0,11,0,10,0,11,0,10,0,12,0,10,0,13,0,11,0,13,0,11,0,13,0,11,0,13,0,11,0,14,0,12,0,14,0,12,0,14,0,12,0,14,0,12,0,15,0,12,0,15,0,13,0,16,0,9,0,12,0,11,0,11,0,10,0,11,0,10,0,12,0,10,0,12,0,11,0,13,0,11,0,13,0,11,0,13,0,11,0,14,0,12,0,14,0,12,0,15,0,12,0,15,0,12,0,15,0,12,0,15,0,13,0,17,0,13,0,17,0,10,0,12,0,11,0,11,0,11,0,12,0,11,0,12,0,11,0,13,0,11,0,13,0,11,0,13,0,11,0,14,0,11,0,14,0,11,0,15,0,12,0,15,0,12,0,15,0,12,0,15,0,12,0,16,0,13,0,16,0,13,0,16,0,10,0,12,0,11,0,12,0,11,0,12,0,11,0,12,0,11,0,13,0,11,0,13,0,11,0,14,0,11,0,14,0,12,0,15,0,12,0,15,0,12,0,15,0,12,0,15,0,12,0,16,0,13,0,15,0,13,0,16,0,13,0,15,0,10,0,13,0,12,0,12,0,11,0,13,0,11,0,12,0,11,0,13,0,11,0,14,0,12,0,14,0,12,0,14,0,12,0,14,0,12,0,15,0,12,0,16,0,12,0,16,0,13,0,16,0,13,0,17,0,13,0,17,0,13,0,16,0,10,0,12,0,12,0,13,0,12,0,13,0,11,0,13,0,11,0,13,0,11,0,14,0,12,0,14,0,12,0,15,0,12,0,16,0,12,0,16,0,12,0,16,0,12,0,16,0,13,0,16,0,13,0,16,0,13,0,15,0,13,0,16,0,10,0,13,0,12,0,13,0,12,0,14,0,12,0,14,0,12,0,14,0,12,0,14,0,12,0,15,0,12,0,15,0,12,0,15,0,12,0,15,0,12,0,17,0,13,0,16,0,13,0,16,0,13,0,16,0,13,0,16,0,13,0,18,0,10,0,13,0,12,0,15,0,12,0,14,0,12,0,14,0,12,0,14,0,12,0,15,0,12,0,15,0,12,0,16,0,12,0,16,0,13,0,16,0,13,0,18,0,13,0,17,0,13,0,17,0,13,0,17,0,13,0,19,0,13,0,17,0,10,0,13,0,13,0,14,0,12,0,15,0,12,0,13,0,12,0,14,0,12,0,16,0,12,0,16,0,12,0,15,0,13,0,16,0,13,0,16,0,13,0,17,0,13,0,18,0,13,0,17,0,13,0,19,0,13,0,17,0,13,0,16,0,10,0,13,0,9,0,10,0,9,0,10,0,9,0,10,0,9,0,11,0,9,0,11,0,9,0,12,0,9,0,12,0,9,0,12,0,9,0,13,0,9,0,13,0,9,0,13,0,10,0,13,0,10,0,13,0,10,0,13,0,10,0,13,0,6,0,10,0,0,0,228,1,248,1,0,0,228,1,20,0,184,0,0,0,97,97,114,0,248,1,97,98,107,0,249,1,97,99,101,0,2,0,97,99,104,0,3,0,97,100,97,0,4,0,97,100,121,0,5,0,97,102,97,0,6,0,97,102,104,0,7,0,97,102,114,0,251,1,97,105,110,0,9,0,97,107,97,0,252,1,97,107,107,0,11,0,97,108,98,0,246,1,97,108,101,0,13,0,97,108,103,0,14,0,97,108,116,0,15,0,97,109,104,0,253,1,97,110,103,0,17,0,97,110,112,0,18,0,97,112,97,0,19,0,97,114,97,0,255,1,97,114,99,0,21,0,97,114,103,0,254,1,97,114,109,0,236,1,97,114,110,0,24,0,97,114,112,0,25,0,97,114,116,0,26,0,97,114,119,0,27,0,97,115,109,0,0,2,97,115,116,0,29,0,97,116,104,0,30,0,97,117,115,0,31,0,97,118,97,0,1,2,97,118,101,0,250,1,97,119,97,0,34,0,97,121,109,0,2,2,97,122,101,0,3,2,98,97,100,0,37,0,98,97,105,0,38,0,98,97,107,0,4,2,98,97,108,0,40,0,98,97,109,0,9,2,98,97,110,0,42,0,98,97,113,0,233,1,98,97,115,0,44,0,98,97,116,0,45,0,98,101,106,0,46,0,98,101,108,0,5,2,98,101,109,0,48,0,98,101,110,0,10,2,98,101,114,0,50,0,98,104,111,0,51,0,98,105,104,0,7,2,98,105,107,0,53,0,98,105,110,0,54,0,98,105,115,0,8,2,98,108,97,0,56,0,98,110,116,0,57,0,98,111,115,0,13,2,98,114,97,0,59,0,98,114,101,0,12,2,98,116,107,0,61,0,98,117,97,0,62,0,98,117,103,0,63,0,98,117,108,0,6,2,98,117,114,0,242,1,98,121,110,0,66,0,99,97,100,0,67,0,99,97,105,0,68,0,99,97,114,0,69,0,99,97,116,0,14,2,99,97,117,0,71,0,99,101,98,0,72,0,99,101,108,0,73,0,99,104,97,0,16,2,99,104,98,0,75,0,99,104,101,0,15,2,99,104,103,0,77,0,99,104,105,0,247,1,99,104,107,0,79,0,99,104,109,0,80,0,99,104,110,0,81,0,99,104,111,0,82,0,99,104,112,0,83,0,99,104,114,0,84,0,99,104,117,0,20,2,99,104,118,0,21,2,99,104,121,0,87,0,99,109,99,0,88,0,99,111,112,0,89,0,99,111,114,0,81,2,99,111,115,0,17,2,99,112,101,0,92,0,99,112,102,0,93,0,99,112,112,0,94,0,99,114,101,0,18,2,99,114,104,0,96,0,99,114,112,0,97,0,99,115,98,0,98,0,99,117,115,0,99,0,99,122,101,0,229,1,100,97,107,0,101,0,100,97,110,0,23,2,100,97,114,0,103,0,100,97,121,0,104,0,100,101,108,0,105,0,100,101,110,0,106,0,100,103,114,0,107,0,100,105,110,0,108,0,100,105,118,0,25,2,100,111,105,0,110,0,100,114,97,0,111,0,100,115,98,0,112,0,100,117,97,0,113,0,100,117,109,0,114,0,100,117,116,0,243,1,100,121,117,0,116,0,100,122,111,0,26,2,101,102,105,0,118,0,101,103,121,0,119,0,101,107,97,0,120,0,101,108,120,0,121,0,101,110,103,0,29,2,101,110,109,0,123,0,101,112,111,0,30,2,101,115,116,0,32,2,101,119,101,0,27,2,101,119,111,0,127,0,102,97,110,0,128,0,102,97,111,0,38,2,102,97,116,0,130,0,102,105,106,0,37,2,102,105,108,0,132,0,102,105,110,0,36,2,102,105,117,0,134,0,102,111,110,0,135,0,102,114,101,0,235,1,102,114,109,0,137,0,102,114,111,0,138,0,102,114,114,0,139,0,102,114,115,0,140,0,102,114,121,0,40,2,102,117,108,0,35,2,102,117,114,0,143,0,103,97,97,0,144,0,103,97,121,0,145,0,103,98,97,0,146,0,103,101,109,0,147,0,103,101,111,0,238,1,103,101,114,0,231,1,103,101,122,0,150,0,103,105,108,0,151,0,103,108,97,0,42,2,103,108,101,0,41,2,103,108,103,0,43,2,103,108,118,0,46,2,103,109,104,0,156,0,103,111,104,0,157,0,103,111,110,0,158,0,103,111,114,0,159,0,103,111,116,0,160,0,103,114,98,0,161,0,103,114,99,0,162,0,103,114,101,0,232,1,103,114,110,0,44,2,103,115,119,0,165,0,103,117,106,0,45,2,103,119,105,0,167,0,104,97,105,0,168,0,104,97,116,0,52,2,104,97,117,0,47,2,104,97,119,0,171,0,104,101,98,0,48,2,104,101,114,0,55,2,104,105,108,0,174,0,104,105,109,0,175,0,104,105,110,0,49,2,104,105,116,0,177,0,104,109,110,0,178,0,104,109,111,0,50,2,104,114,118,0,51,2,104,115,98,0,181,0,104,117,110,0,53,2,104,117,112,0,183,0,105,98,97,0,184,0,105,98,111,0,59,2,105,99,101,0,237,1,105,100,111,0,62,2,105,105,105,0,60,2,105,106,111,0,189,0,105,107,117,0,65,2,105,108,101,0,58,2,105,108,111,0,192,0,105,110,97,0,56,2,105,110,99,0,194,0,105,110,100,0,57,2,105,110,101,0,196,0,105,110,104,0,197,0,105,112,107,0,61,2,105,114,97,0,199,0,105,114,111,0,200,0,105,116,97,0,64,2,106,97,118,0,67,2,106,98,111,0,203,0,106,112,110,0,66,2,106,112,114,0,205,0,106,114,98,0,206,0,107,97,97,0,207,0,107,97,98,0,208,0,107,97,99,0,209,0,107,97,108,0,73,2,107,97,109,0,211,0,107,97,110,0,75,2,107,97,114,0,213,0,107,97,115,0,78,2,107,97,117,0,77,2,107,97,119,0,216,0,107,97,122,0,72,2,107,98,100,0,218,0,107,104,97,0,219,0,107,104,105,0,220,0,107,104,109,0,74,2,107,104,111,0,222,0,107,105,107,0,70,2,107,105,110,0,128,2,107,105,114,0,82,2,107,109,98,0,226,0,107,111,107,0,227,0,107,111,109,0,80,2,107,111,110,0,69,2,107,111,114,0,76,2,107,111,115,0,231,0,107,112,101,0,232,0,107,114,99,0,233,0,107,114,108,0,234,0,107,114,111,0,235,0,107,114,117,0,236,0,107,117,97,0,71,2,107,117,109,0,238,0,107,117,114,0,79,2,107,117,116,0,240,0,108,97,100,0,241,0,108,97,104,0,242,0,108,97,109,0,243,0,108,97,111,0,88,2,108,97,116,0,83,2,108,97,118,0,91,2,108,101,122,0,247,0,108,105,109,0,86,2,108,105,110,0,87,2,108,105,116,0,89,2,108,111,108,0,251,0,108,111,122,0,252,0,108,116,122,0,84,2,108,117,97,0,254,0,108,117,98,0,90,2,108,117,103,0,85,2,108,117,105,0,1,1,108,117,110,0,2,1,108,117,111,0,3,1,108,117,115,0,4,1,109,97,99,0,239,1,109,97,100,0,6,1,109,97,103,0,7,1,109,97,104,0,93,2,109,97,105,0,9,1,109,97,107,0,10,1,109,97,108,0,96,2,109,97,110,0,12,1,109,97,111,0,240,1,109,97,112,0,14,1,109,97,114,0,98,2,109,97,115,0,16,1,109,97,121,0,241,1,109,100,102,0,18,1,109,100,114,0,19,1,109,101,110,0,20,1,109,103,97,0,21,1,109,105,99,0,22,1,109,105,110,0,23,1,109,105,115,0,24,1,109,107,104,0,25,1,109,108,103,0,92,2,109,108,116,0,100,2,109,110,99,0,28,1,109,110,105,0,29,1,109,110,111,0,30,1,109,111,104,0,31,1,109,111,110,0,97,2,109,111,115,0,33,1,109,117,108,0,34,1,109,117,110,0,35,1,109,117,115,0,36,1,109,119,108,0,37,1,109,119,114,0,38,1,109,121,110,0,39,1,109,121,118,0,40,1,110,97,104,0,41,1,110,97,105,0,42,1,110,97,112,0,43,1,110,97,117,0,102,2,110,97,118,0,111,2,110,98,108,0,110,2,110,100,101,0,104,2,110,100,111,0,106,2,110,100,115,0,49,1,110,101,112,0,105,2,110,101,119,0,51,1,110,105,97,0,52,1,110,105,99,0,53,1,110,105,117,0,54,1,110,110,111,0,108,2,110,111,98,0,103,2,110,111,103,0,57,1,110,111,110,0,58,1,110,111,114,0,109,2,110,113,111,0,60,1,110,115,111,0,61,1,110,117,98,0,62,1,110,119,99,0,63,1,110,121,97,0,112,2,110,121,109,0,65,1,110,121,110,0,66,1,110,121,111,0,67,1,110,122,105,0,68,1,111,99,105,0,113,2,111,106,105,0,114,2,111,114,105,0,116,2,111,114,109,0,115,2,111,115,97,0,73,1,111,115,115,0,117,2,111,116,97,0,75,1,111,116,111,0,76,1,112,97,97,0,77,1,112,97,103,0,78,1,112,97,108,0,79,1,112,97,109,0,80,1,112,97,110,0,118,2,112,97,112,0,82,1,112,97,117,0,83,1,112,101,111,0,84,1,112,101,114,0,234,1,112,104,105,0,86,1,112,104,110,0,87,1,112,108,105,0,119,2,112,111,108,0,120,2,112,111,110,0,90,1,112,111,114,0,122,2,112,114,97,0,92,1,112,114,111,0,93,1,112,117,115,0,121,2,113,117,101,0,123,2,114,97,106,0,96,1,114,97,112,0,97,1,114,97,114,0,98,1,114,111,97,0,99,1,114,111,104,0,124,2,114,111,109,0,101,1,114,117,109,0,244,1,114,117,110,0,125,2,114,117,112,0,104,1,114,117,115,0,127,2,115,97,100,0,106,1,115,97,103,0,133,2,115,97,104,0,108,1,115,97,105,0,109,1,115,97,108,0,110,1,115,97,109,0,111,1,115,97,110,0,129,2,115,97,115,0,113,1,115,97,116,0,114,1,115,99,110,0,115,1,115,99,111,0,116,1,115,101,108,0,117,1,115,101,109,0,118,1,115,103,97,0,119,1,115,103,110,0,120,1,115,104,110,0,121,1,115,105,100,0,122,1,115,105,110,0,134,2,115,105,111,0,124,1,115,105,116,0,125,1,115,108,97,0,126,1,115,108,111,0,245,1,115,108,118,0,136,2,115,109,97,0,129,1,115,109,101,0,132,2,115,109,105,0,131,1,115,109,106,0,132,1,115,109,110,0,133,1,115,109,111,0,137,2,115,109,115,0,135,1,115,110,97,0,138,2,115,110,100,0,131,2,115,110,107,0,138,1,115,111,103,0,139,1,115,111,109,0,139,2,115,111,110,0,141,1,115,111,116,0,143,2,115,112,97,0,31,2,115,114,100,0,130,2,115,114,110,0,145,1,115,114,112,0,141,2,115,114,114,0,147,1,115,115,97,0,148,1,115,115,119,0,142,2,115,117,107,0,150,1,115,117,110,0,144,2,115,117,115,0,152,1,115,117,120,0,153,1,115,119,97,0,146,2,115,119,101,0,145,2,115,121,99,0,156,1,115,121,114,0,157,1,116,97,104,0,160,2,116,97,105,0,159,1,116,97,109,0,147,2,116,97,116,0,158,2,116,101,108,0,148,2,116,101,109,0,163,1,116,101,114,0,164,1,116,101,116,0,165,1,116,103,107,0,149,2,116,103,108,0,153,2,116,104,97,0,150,2,116,105,98,0,228,1,116,105,103,0,170,1,116,105,114,0,151,2,116,105,118,0,172,1,116,107,108,0,173,1,116,108,104,0,174,1,116,108,105,0,175,1,116,109,104,0,176,1,116,111,103,0,177,1,116,111,110,0,155,2,116,112,105,0,179,1,116,115,105,0,180,1,116,115,110,0,154,2,116,115,111,0,157,2,116,117,107,0,152,2,116,117,109,0,184,1,116,117,112,0,185,1,116,117,114,0,156,2,116,117,116,0,187,1,116,118,108,0,188,1,116,119,105,0,159,2,116,121,118,0,190,1,117,100,109,0,191,1,117,103,97,0,192,1,117,105,103,0,161,2,117,107,114,0,162,2,117,109,98,0,195,1,117,110,100,0,196,1,117,114,100,0,163,2,117,122,98,0,164,2,118,97,105,0,199,1,118,101,110,0,165,2,118,105,101,0,166,2,118,111,108,0,167,2,118,111,116,0,203,1,119,97,107,0,204,1,119,97,108,0,205,1,119,97,114,0,206,1,119,97,115,0,207,1,119,101,108,0,230,1,119,101,110,0,209,1,119,108,110,0,168,2,119,111,108,0,169,2,120,97,108,0,212,1,120,104,111,0,170,2,121,97,111,0,214,1,121,97,112,0,215,1,121,105,100,0,171,2,121,111,114,0,172,2,121,112,107,0,218,1,122,97,112,0,219,1,122,98,108,0,220,1,122,101,110,0,221,1,122,104,97,0,173,2,122,110,100,0,223,1,122,117,108,0,175,2,122,117,110,0,225,1,122,120,120,0,226,1,122,122,97,0,227,1,98,111,100,0,11,2,99,101,115,0,19,2,99,121,109,0,22,2,100,101,117,0,24,2,101,108,108,0,28,2,101,117,115,0,33,2,102,97,115,0,34,2,102,114,97,0,39,2,104,121,101,0,54,2,105,115,108,0,63,2,107,97,116,0,68,2,109,107,100,0,95,2,109,114,105,0,94,2,109,115,97,0,99,2,109,121,97,0,101,2,110,108,100,0,107,2,114,111,110,0,126,2,115,108,107,0,135,2,115,113,105,0,140,2,122,104,111,0,174,2,97,97,0,0,0,0,97,98,0,0,1,0,97,101,0,0,33,0,97,102,0,0,8,0,97,107,0,0,10,0,97,109,0,0,16,0,97,110,0,0,22,0,97,114,0,0,20,0,97,115,0,0,28,0,97,118,0,0,32,0,97,121,0,0,35,0,97,122,0,0,36,0,98,97,0,0,39,0,98,101,0,0,47,0,98,103,0,0,64,0,98,104,0,0,52,0,98,105,0,0,55,0,98,109,0,0,41,0,98,110,0,0,49,0,98,111,0,0,169,1,98,114,0,0,60,0,98,115,0,0,58,0,99,97,0,0,70,0,99,101,0,0,76,0,99,104,0,0,74,0,99,111,0,0,91,0,99,114,0,0,95,0,99,115,0,0,100,0,99,117,0,0,85,0,99,118,0,0,86,0,99,121,0,0,208,1,100,97,0,0,102,0,100,101,0,0,149,0,100,118,0,0,109,0,100,122,0,0,117,0,101,101,0,0,126,0,101,108,0,0,163,0,101,110,0,0,122,0,101,111,0,0,124,0,101,115,0,0,143,1,101,116,0,0,125,0,101,117,0,0,43,0,102,97,0,0,85,1,102,102,0,0,142,0,102,105,0,0,133,0,102,106,0,0,131,0,102,111,0,0,129,0,102,114,0,0,136,0,102,121,0,0,141,0,103,97,0,0,153,0,103,100,0,0,152,0,103,108,0,0,154,0,103,110,0,0,164,0,103,117,0,0,166,0,103,118,0,0,155,0,104,97,0,0,170,0,104,101,0,0,172,0,104,105,0,0,176,0,104,111,0,0,179,0,104,114,0,0,180,0,104,116,0,0,169,0,104,117,0,0,182,0,104,121,0,0,23,0,104,122,0,0,173,0,105,97,0,0,193,0,105,100,0,0,195,0,105,101,0,0,191,0,105,103,0,0,185,0,105,105,0,0,188,0,105,107,0,0,198,0,105,111,0,0,187,0,105,115,0,0,186,0,105,116,0,0,201,0,105,117,0,0,190,0,106,97,0,0,204,0,106,118,0,0,202,0,107,97,0,0,148,0,107,103,0,0,229,0,107,105,0,0,223,0,107,106,0,0,237,0,107,107,0,0,217,0,107,108,0,0,210,0,107,109,0,0,221,0,107,110,0,0,212,0,107,111,0,0,230,0,107,114,0,0,215,0,107,115,0,0,214,0,107,117,0,0,239,0,107,118,0,0,228,0,107,119,0,0,90,0,107,121,0,0,225,0,108,97,0,0,245,0,108,98,0,0,253,0,108,103,0,0,0,1,108,105,0,0,248,0,108,110,0,0,249,0,108,111,0,0,244,0,108,116,0,0,250,0,108,117,0,0,255,0,108,118,0,0,246,0,109,103,0,0,26,1,109,104,0,0,8,1,109,105,0,0,13,1,109,107,0,0,5,1,109,108,0,0,11,1,109,110,0,0,32,1,109,114,0,0,15,1,109,115,0,0,17,1,109,116,0,0,27,1,109,121,0,0,65,0,110,97,0,0,44,1,110,98,0,0,56,1,110,100,0,0,47,1,110,101,0,0,50,1,110,103,0,0,48,1,110,108,0,0,115,0,110,110,0,0,55,1,110,111,0,0,59,1,110,114,0,0,46,1,110,118,0,0,45,1,110,121,0,0,64,1,111,99,0,0,69,1,111,106,0,0,70,1,111,109,0,0,72,1,111,114,0,0,71,1,111,115,0,0,74,1,112,97,0,0,81,1,112,105,0,0,88,1,112,108,0,0,89,1,112,115,0,0,94,1,112,116,0,0,91,1,113,117,0,0,95,1,114,109,0,0,100,1,114,110,0,0,103,1,114,111,0,0,102,1,114,117,0,0,105,1,114,119,0,0,224,0,115,97,0,0,112,1,115,99,0,0,144,1,115,100,0,0,137,1,115,101,0,0,130,1,115,103,0,0,107,1,115,105,0,0,123,1,115,107,0,0,127,1,115,108,0,0,128,1,115,109,0,0,134,1,115,110,0,0,136,1,115,111,0,0,140,1,115,113,0,0,12,0,115,114,0,0,146,1,115,115,0,0,149,1,115,116,0,0,142,1,115,117,0,0,151,1,115,118,0,0,155,1,115,119,0,0,154,1,116,97,0,0,160,1,116,101,0,0,162,1,116,103,0,0,166,1,116,104,0,0,168,1,116,105,0,0,171,1,116,107,0,0,183,1,116,108,0,0,167,1,116,110,0,0,181,1,116,111,0,0,178,1,116,114,0,0,186,1,116,115,0,0,182,1,116,116,0,0,161,1,116,119,0,0,189,1,116,121,0,0,158,1,117,103,0,0,193,1,117,107,0,0,194,1,117,114,0,0,197,1,117,122,0,0,198,1,118,101,0,0,200,1,118,105,0,0,201,1,118,111,0,0,202,1,119,97,0,0,210,1,119,111,0,0,211,1,120,104,0,0,213,1,121,105,0,0,216,1,121,111,0,0,217,1,122,97,0,0,222,1,122,104,0,0,78,0,122,117,0,0,224,1,0,0,0,0,0,0,0,0,44,76,0,0,56,74,0,0,68,72,0,0,80,70,0,0,92,68,0,0,116,64,0,0,140,60,0,0,164,56,0,0,212,48,0,0,28,37,0,0,110,15,0,0,0,0,0,0,192,93,0,0,44,76,0,0,68,72,0,0,80,70,0,0,92,68,0,0,104,66,0,0,116,64,0,0,240,60,0,0,96,59,0,0,62,28,0,0,110,15,0,0,0,0,0,0,44,76,0,0,56,74,0,0,168,72,0,0,80,70,0,0,92,68,0,0,128,62,0,0,240,60,0,0,52,58,0,0,212,48,0,0,16,39,0,0,110,15,0,0,0,0,0,0,128,187,0,0,0,0,0,0,0,0,208,64,0,0,0,0,0,0,208,64,148,92,0,0,68,172,0,0,0,0,0,0,0,0,208,64,0,0,0,0,0,0,208,64,20,85,0,0,0,125,0,0,0,0,208,64,0,0,0,65,102,102,166,64,0,0,208,64,184,61,0,0,192,93,0,0,0,0,0,65,0,0,8,65,102,102,166,64,0,0,192,64,74,46,0,0,34,86,0,0,0,0,8,65,246,40,16,65,102,102,166,64,0,0,208,64,140,42,0,0,128,62,0,0,246,40,16,65,102,102,22,65,205,204,156,64,0,0,208,64,223,30,0,0,224,46,0,0,102,102,22,65,154,153,25,65,0,0,144,64,0,0,192,64,40,23,0,0,17,43,0,0,154,153,25,65,102,102,30,65,51,51,163,64,0,0,208,64,70,21,0,0,64,31,0,0,102,102,30,65,0,0,32,65,205,204,156,64,0,0,208,64,112,15,0,0,102,102,182,64,0,0,208,64,154,153,233,64,51,51,3,65,0,0,32,65,102,102,62,65,0,0,80,65,0,0,96,65,0,0,112,65,0,0,132,65,0,48,1,48,2,48,183,0,37,32,38,32,168,0,3,48,173,0,21,32,37,34,60,255,60,34,24,32,25,32,28,32,29,32,20,48,21,48,8,48,9,48,10,48,11,48,12,48,13,48,14,48,15,48,16,48,17,48,177,0,215,0,247,0,96,34,100,34,101,34,30,34,52,34,176,0,50,32,51,32,3,33,43,33,224,255,225,255,229,255,66,38,64,38,32,34,165,34,18,35,2,34,7,34,97,34,82,34,167,0,59,32,6,38,5,38,203,37,207,37,206,37,199,37,198,37,161,37,160,37,179,37,178,37,189,37,188,37,146,33,144,33,145,33,147,33,148,33,19,48,106,34,107,34,26,34,61,34,29,34,53,34,43,34,44,34,8,34,11,34,134,34,135,34,130,34,131,34,42,34,41,34,39,34,40,34,226,255,210,33,212,33,0,34,3,34,180,0,94,255,199,2,216,2,221,2,218,2,217,2,184,0,219,2,161,0,191,0,208,2,46,34,17,34,15,34,164,0,9,33,48,32,193,37,192,37,183,37,182,37,100,38,96,38,97,38,101,38,103,38,99,38,153,34,200,37,163,37,208,37,209,37,146,37,164,37,165,37,168,37,167,37,166,37,169,37,104,38,15,38,14,38,28,38,30,38,182,0,32,32,33,32,149,33,151,33,153,33,150,33,152,33,109,38,105,38,106,38,108,38,127,50,28,50,22,33,199,51,34,33,194,51,216,51,33,33,172,32,174,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,255,2,255,3,255,4,255,5,255,6,255,7,255,8,255,9,255,10,255,11,255,12,255,13,255,14,255,15,255,16,255,17,255,18,255,19,255,20,255,21,255,22,255,23,255,24,255,25,255,26,255,27,255,28,255,29,255,30,255,31,255,32,255,33,255,34,255,35,255,36,255,37,255,38,255,39,255,40,255,41,255,42,255,43,255,44,255,45,255,46,255,47,255,48,255,49,255,50,255,51,255,52,255,53,255,54,255,55,255,56,255,57,255,58,255,59,255,230,255,61,255,62,255,63,255,64,255,65,255,66,255,67,255,68,255,69,255,70,255,71,255,72,255,73,255,74,255,75,255,76,255,77,255,78,255,79,255,80,255,81,255,82,255,83,255,84,255,85,255,86,255,87,255,88,255,89,255,90,255,91,255,92,255,93,255,227,255,49,49,50,49,51,49,52,49,53,49,54,49,55,49,56,49,57,49,58,49,59,49,60,49,61,49,62,49,63,49,64,49,65,49,66,49,67,49,68,49,69,49,70,49,71,49,72,49,73,49,74,49,75,49,76,49,77,49,78,49,79,49,80,49,81,49,82,49,83,49,84,49,85,49,86,49,87,49,88,49,89,49,90,49,91,49,92,49,93,49,94,49,95,49,96,49,97,49,98,49,99,49,100,49,101,49,102,49,103,49,104,49,105,49,106,49,107,49,108,49,109,49,110,49,111,49,112,49,113,49,114,49,115,49,116,49,117,49,118,49,119,49,120,49,121,49,122,49,123,49,124,49,125,49,126,49,127,49,128,49,129,49,130,49,131,49,132,49,133,49,134,49,135,49,136,49,137,49,138,49,139,49,140,49,141,49,142,49,112,33,113,33,114,33,115,33,116,33,117,33,118,33,119,33,120,33,121,33,0,0,0,0,0,0,0,0,0,0,96,33,97,33,98,33,99,33,100,33,101,33,102,33,103,33,104,33,105,33,0,0,0,0,0,0,0,0,0,0,0,0,0,0,145,3,146,3,147,3,148,3,149,3,150,3,151,3,152,3,153,3,154,3,155,3,156,3,157,3,158,3,159,3,160,3,161,3,163,3,164,3,165,3,166,3,167,3,168,3,169,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,177,3,178,3,179,3,180,3,181,3,182,3,183,3,184,3,185,3,186,3,187,3,188,3,189,3,190,3,191,3,192,3,193,3,195,3,196,3,197,3,198,3,199,3,200,3,201,3,0,0,0,0,0,0,0,0,0,0,0,0,0,37,2,37,12,37,16,37,24,37,20,37,28,37,44,37,36,37,52,37,60,37,1,37,3,37,15,37,19,37,27,37,23,37,35,37,51,37,43,37,59,37,75,37,32,37,47,37,40,37,55,37,63,37,29,37,48,37,37,37,56,37,66,37,18,37,17,37,26,37,25,37,22,37,21,37,14,37,13,37,30,37,31,37,33,37,34,37,38,37,39,37,41,37,42,37,45,37,46,37,49,37,50,37,53,37,54,37,57,37,58,37,61,37,62,37,64,37,65,37,67,37,68,37,69,37,70,37,71,37,72,37,73,37,74,37,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,149,51,150,51,151,51,19,33,152,51,196,51,163,51,164,51,165,51,166,51,153,51,154,51,155,51,156,51,157,51,158,51,159,51,160,51,161,51,162,51,202,51,141,51,142,51,143,51,207,51,136,51,137,51,200,51,167,51,168,51,176,51,177,51,178,51,179,51,180,51,181,51,182,51,183,51,184,51,185,51,128,51,129,51,130,51,131,51,132,51,186,51,187,51,188,51,189,51,190,51,191,51,144,51,145,51,146,51,147,51,148,51,38,33,192,51,193,51,138,51,139,51,140,51,214,51,197,51,173,51,174,51,175,51,219,51,169,51,170,51,171,51,172,51,221,51,208,51,211,51,195,51,201,51,220,51,198,51,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,198,0,208,0,170,0,38,1,0,0,50,1,0,0,63,1,65,1,216,0,82,1,186,0,222,0,102,1,74,1,0,0,96,50,97,50,98,50,99,50,100,50,101,50,102,50,103,50,104,50,105,50,106,50,107,50,108,50,109,50,110,50,111,50,112,50,113,50,114,50,115,50,116,50,117,50,118,50,119,50,120,50,121,50,122,50,123,50,208,36,209,36,210,36,211,36,212,36,213,36,214,36,215,36,216,36,217,36,218,36,219,36,220,36,221,36,222,36,223,36,224,36,225,36,226,36,227,36,228,36,229,36,230,36,231,36,232,36,233,36,96,36,97,36,98,36,99,36,100,36,101,36,102,36,103,36,104,36,105,36,106,36,107,36,108,36,109,36,110,36,189,0,83,33,84,33,188,0,190,0,91,33,92,33,93,33,94,33,230,0,17,1,240,0,39,1,49,1,51,1,56,1,64,1,66,1,248,0,83,1,223,0,254,0,103,1,75,1,73,1,0,50,1,50,2,50,3,50,4,50,5,50,6,50,7,50,8,50,9,50,10,50,11,50,12,50,13,50,14,50,15,50,16,50,17,50,18,50,19,50,20,50,21,50,22,50,23,50,24,50,25,50,26,50,27,50,156,36,157,36,158,36,159,36,160,36,161,36,162,36,163,36,164,36,165,36,166,36,167,36,168,36,169,36,170,36,171,36,172,36,173,36,174,36,175,36,176,36,177,36,178,36,179,36,180,36,181,36,116,36,117,36,118,36,119,36,120,36,121,36,122,36,123,36,124,36,125,36,126,36,127,36,128,36,129,36,130,36,185,0,178,0,179,0,116,32,127,32,129,32,130,32,131,32,132,32,65,48,66,48,67,48,68,48,69,48,70,48,71,48,72,48,73,48,74,48,75,48,76,48,77,48,78,48,79,48,80,48,81,48,82,48,83,48,84,48,85,48,86,48,87,48,88,48,89,48,90,48,91,48,92,48,93,48,94,48,95,48,96,48,97,48,98,48,99,48,100,48,101,48,102,48,103,48,104,48,105,48,106,48,107,48,108,48,109,48,110,48,111,48,112,48,113,48,114,48,115,48,116,48,117,48,118,48,119,48,120,48,121,48,122,48,123,48,124,48,125,48,126,48,127,48,128,48,129,48,130,48,131,48,132,48,133,48,134,48,135,48,136,48,137,48,138,48,139,48,140,48,141,48,142,48,143,48,144,48,145,48,146,48,147,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,161,48,162,48,163,48,164,48,165,48,166,48,167,48,168,48,169,48,170,48,171,48,172,48,173,48,174,48,175,48,176,48,177,48,178,48,179,48,180,48,181,48,182,48,183,48,184,48,185,48,186,48,187,48,188,48,189,48,190,48,191,48,192,48,193,48,194,48,195,48,196,48,197,48,198,48,199,48,200,48,201,48,202,48,203,48,204,48,205,48,206,48,207,48,208,48,209,48,210,48,211,48,212,48,213,48,214,48,215,48,216,48,217,48,218,48,219,48,220,48,221,48,222,48,223,48,224,48,225,48,226,48,227,48,228,48,229,48,230,48,231,48,232,48,233,48,234,48,235,48,236,48,237,48,238,48,239,48,240,48,241,48,242,48,243,48,244,48,245,48,246,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,4,17,4,18,4,19,4,20,4,21,4,1,4,22,4,23,4,24,4,25,4,26,4,27,4,28,4,29,4,30,4,31,4,32,4,33,4,34,4,35,4,36,4,37,4,38,4,39,4,40,4,41,4,42,4,43,4,44,4,45,4,46,4,47,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,4,49,4,50,4,51,4,52,4,53,4,81,4,54,4,55,4,56,4,57,4,58,4,59,4,60,4,61,4,62,4,63,4,64,4,65,4,66,4,67,4,68,4,69,4,70,4,71,4,72,4,73,4,74,4,75,4,76,4,77,4,78,4,79,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,172,1,172,4,172,7,172,8,172,9,172,10,172,16,172,17,172,18,172,19,172,20,172,21,172,22,172,23,172,25,172,26,172,27,172,28,172,29,172,32,172,36,172,44,172,45,172,47,172,48,172,49,172,56,172,57,172,60,172,64,172,75,172,77,172,84,172,88,172,92,172,112,172,113,172,116,172,119,172,120,172,122,172,128,172,129,172,131,172,132,172,133,172,134,172,137,172,138,172,139,172,140,172,144,172,148,172,156,172,157,172,159,172,160,172,161,172,168,172,169,172,170,172,172,172,175,172,176,172,184,172,185,172,187,172,188,172,189,172,193,172,196,172,200,172,204,172,213,172,215,172,224,172,225,172,228,172,231,172,232,172,234,172,236,172,239,172,240,172,241,172,243,172,245,172,246,172,252,172,253,172,0,173,4,173,6,173,12,173,13,173,15,173,17,173,24,173,28,173,32,173,41,173,44,173,45,173,52,173,53,173,56,173,60,173,68,173,69,173,71,173,73,173,80,173,84,173,88,173,97,173,99,173,108,173,109,173,112,173,115,173,116,173,117,173,118,173,123,173,124,173,125,173,127,173,129,173,130,173,136,173,137,173,140,173,144,173,156,173,157,173,164,173,183,173,192,173,193,173,196,173,200,173,208,173,209,173,211,173,220,173,224,173,228,173,248,173,249,173,252,173,255,173,0,174,1,174,8,174,9,174,11,174,13,174,20,174,48,174,49,174,52,174,55,174,56,174,58,174,64,174,65,174,67,174,69,174,70,174,74,174,76,174,77,174,78,174,80,174,84,174,86,174,92,174,93,174,95,174,96,174,97,174,101,174,104,174,105,174,108,174,112,174,120,174,121,174,123,174,124,174,125,174,132,174,133,174,140,174,188,174,189,174,190,174,192,174,196,174,204,174,205,174,207,174,208,174,209,174,216,174,217,174,220,174,232,174,235,174,237,174,244,174,248,174,252,174,7,175,8,175,13,175,16,175,44,175,45,175,48,175,50,175,52,175,60,175,61,175,63,175,65,175,66,175,67,175,72,175,73,175,80,175,92,175,93,175,100,175,101,175,121,175,128,175,132,175,136,175,144,175,145,175,149,175,156,175,184,175,185,175,188,175,192,175,199,175,200,175,201,175,203,175,205,175,206,175,212,175,220,175,232,175,233,175,240,175,241,175,244,175,248,175,0,176,1,176,4,176,12,176,16,176,20,176,28,176,29,176,40,176,68,176,69,176,72,176,74,176,76,176,78,176,83,176,84,176,85,176,87,176,89,176,93,176,124,176,125,176,128,176,132,176,140,176,141,176,143,176,145,176,152,176,153,176,154,176,156,176,159,176,160,176,161,176,162,176,168,176,169,176,171,176,172,176,173,176,174,176,175,176,177,176,179,176,180,176,181,176,184,176,188,176,196,176,197,176,199,176,200,176,201,176,208,176,209,176,212,176,216,176,224,176,229,176,8,177,9,177,11,177,12,177,16,177,18,177,19,177,24,177,25,177,27,177,28,177,29,177,35,177,36,177,37,177,40,177,44,177,52,177,53,177,55,177,56,177,57,177,64,177,65,177,68,177,72,177,80,177,81,177,84,177,85,177,88,177,92,177,96,177,120,177,121,177,124,177,128,177,130,177,136,177,137,177,139,177,141,177,146,177,147,177,148,177,152,177,156,177,168,177,204,177,208,177,212,177,220,177,221,177,223,177,232,177,233,177,236,177,240,177,249,177,251,177,253,177,4,178,5,178,8,178,11,178,12,178,20,178,21,178,23,178,25,178,32,178,52,178,60,178,88,178,92,178,96,178,104,178,105,178,116,178,117,178,124,178,132,178,133,178,137,178,144,178,145,178,148,178,152,178,153,178,154,178,160,178,161,178,163,178,165,178,166,178,170,178,172,178,176,178,180,178,200,178,201,178,204,178,208,178,210,178,216,178,217,178,219,178,221,178,226,178,228,178,229,178,230,178,232,178,235,178,236,178,237,178,238,178,239,178,243,178,244,178,245,178,247,178,248,178,249,178,250,178,251,178,255,178,0,179,1,179,4,179,8,179,16,179,17,179,19,179,20,179,21,179,28,179,84,179,85,179,86,179,88,179,91,179,92,179,94,179,95,179,100,179,101,179,103,179,105,179,107,179,110,179,112,179,113,179,116,179,120,179,128,179,129,179,131,179,132,179,133,179,140,179,144,179,148,179,160,179,161,179,168,179,172,179,196,179,197,179,200,179,203,179,204,179,206,179,208,179,212,179,213,179,215,179,217,179,219,179,221,179,224,179,228,179,232,179,252,179,16,180,24,180,28,180,32,180,40,180,41,180,43,180,52,180,80,180,81,180,84,180,88,180,96,180,97,180,99,180,101,180,108,180,128,180,136,180,157,180,164,180,168,180,172,180,181,180,183,180,185,180,192,180,196,180,200,180,208,180,213,180,220,180,221,180,224,180,227,180,228,180,230,180,236,180,237,180,239,180,241,180,248,180,20,181,21,181,24,181,27,181,28,181,36,181,37,181,39,181,40,181,41,181,42,181,48,181,49,181,52,181,56,181,64,181,65,181,67,181,68,181,69,181,75,181,76,181,77,181,80,181,84,181,92,181,93,181,95,181,96,181,97,181,160,181,161,181,164,181,168,181,170,181,171,181,176,181,177,181,179,181,180,181,181,181,187,181,188,181,189,181,192,181,196,181,204,181,205,181,207,181,208,181,209,181,216,181,236,181,16,182,17,182,20,182,24,182,37,182,44,182,52,182,72,182,100,182,104,182,156,182,157,182,160,182,164,182,171,182,172,182,177,182,212,182,240,182,244,182,248,182,0,183,1,183,5,183,40,183,41,183,44,183,47,183,48,183,56,183,57,183,59,183,68,183,72,183,76,183,84,183])
.concat([85,183,96,183,100,183,104,183,112,183,113,183,115,183,117,183,124,183,125,183,128,183,132,183,140,183,141,183,143,183,144,183,145,183,146,183,150,183,151,183,152,183,153,183,156,183,160,183,168,183,169,183,171,183,172,183,173,183,180,183,181,183,184,183,199,183,201,183,236,183,237,183,240,183,244,183,252,183,253,183,255,183,0,184,1,184,7,184,8,184,9,184,12,184,16,184,24,184,25,184,27,184,29,184,36,184,37,184,40,184,44,184,52,184,53,184,55,184,56,184,57,184,64,184,68,184,81,184,83,184,92,184,93,184,96,184,100,184,108,184,109,184,111,184,113,184,120,184,124,184,141,184,168,184,176,184,180,184,184,184,192,184,193,184,195,184,197,184,204,184,208,184,212,184,221,184,223,184,225,184,232,184,233,184,236,184,240,184,248,184,249,184,251,184,253,184,4,185,24,185,32,185,60,185,61,185,64,185,68,185,76,185,79,185,81,185,88,185,89,185,92,185,96,185,104,185,105,185,107,185,109,185,116,185,117,185,120,185,124,185,132,185,133,185,135,185,137,185,138,185,141,185,142,185,172,185,173,185,176,185,180,185,188,185,189,185,191,185,193,185,200,185,201,185,204,185,206,185,207,185,208,185,209,185,210,185,216,185,217,185,219,185,221,185,222,185,225,185,227,185,228,185,229,185,232,185,236,185,244,185,245,185,247,185,248,185,249,185,250,185,0,186,1,186,8,186,21,186,56,186,57,186,60,186,64,186,66,186,72,186,73,186,75,186,77,186,78,186,83,186,84,186,85,186,88,186,92,186,100,186,101,186,103,186,104,186,105,186,112,186,113,186,116,186,120,186,131,186,132,186,133,186,135,186,140,186,168,186,169,186,171,186,172,186,176,186,178,186,184,186,185,186,187,186,189,186,196,186,200,186,216,186,217,186,252,186,0,187,4,187,13,187,15,187,17,187,24,187,28,187,32,187,41,187,43,187,52,187,53,187,54,187,56,187,59,187,60,187,61,187,62,187,68,187,69,187,71,187,73,187,77,187,79,187,80,187,84,187,88,187,97,187,99,187,108,187,136,187,140,187,144,187,164,187,168,187,172,187,180,187,183,187,192,187,196,187,200,187,208,187,211,187,248,187,249,187,252,187,255,187,0,188,2,188,8,188,9,188,11,188,12,188,13,188,15,188,17,188,20,188,21,188,22,188,23,188,24,188,27,188,28,188,29,188,30,188,31,188,36,188,37,188,39,188,41,188,45,188,48,188,49,188,52,188,56,188,64,188,65,188,67,188,68,188,69,188,73,188,76,188,77,188,80,188,93,188,132,188,133,188,136,188,139,188,140,188,142,188,148,188,149,188,151,188,153,188,154,188,160,188,161,188,164,188,167,188,168,188,176,188,177,188,179,188,180,188,181,188,188,188,189,188,192,188,196,188,205,188,207,188,208,188,209,188,213,188,216,188,220,188,244,188,245,188,246,188,248,188,252,188,4,189,5,189,7,189,9,189,16,189,20,189,36,189,44,189,64,189,72,189,73,189,76,189,80,189,88,189,89,189,100,189,104,189,128,189,129,189,132,189,135,189,136,189,137,189,138,189,144,189,145,189,147,189,149,189,153,189,154,189,156,189,164,189,176,189,184,189,212,189,213,189,216,189,220,189,233,189,240,189,244,189,248,189,0,190,3,190,5,190,12,190,13,190,16,190,20,190,28,190,29,190,31,190,68,190,69,190,72,190,76,190,78,190,84,190,85,190,87,190,89,190,90,190,91,190,96,190,97,190,100,190,104,190,106,190,112,190,113,190,115,190,116,190,117,190,123,190,124,190,125,190,128,190,132,190,140,190,141,190,143,190,144,190,145,190,152,190,153,190,168,190,208,190,209,190,212,190,215,190,216,190,224,190,227,190,228,190,229,190,236,190,1,191,8,191,9,191,24,191,25,191,27,191,28,191,29,191,64,191,65,191,68,191,72,191,80,191,81,191,85,191,148,191,176,191,197,191,204,191,205,191,208,191,212,191,220,191,223,191,225,191,60,192,81,192,88,192,92,192,96,192,104,192,105,192,144,192,145,192,148,192,152,192,160,192,161,192,163,192,165,192,172,192,173,192,175,192,176,192,179,192,180,192,181,192,182,192,188,192,189,192,191,192,192,192,193,192,197,192,200,192,201,192,204,192,208,192,216,192,217,192,219,192,220,192,221,192,228,192,229,192,232,192,236,192,244,192,245,192,247,192,249,192,0,193,4,193,8,193,16,193,21,193,28,193,29,193,30,193,31,193,32,193,35,193,36,193,38,193,39,193,44,193,45,193,47,193,48,193,49,193,54,193,56,193,57,193,60,193,64,193,72,193,73,193,75,193,76,193,77,193,84,193,85,193,88,193,92,193,100,193,101,193,103,193,104,193,105,193,112,193,116,193,120,193,133,193,140,193,141,193,142,193,144,193,148,193,150,193,156,193,157,193,159,193,161,193,165,193,168,193,169,193,172,193,176,193,189,193,196,193,200,193,204,193,212,193,215,193,216,193,224,193,228,193,232,193,240,193,241,193,243,193,252,193,253,193,0,194,4,194,12,194,13,194,15,194,17,194,24,194,25,194,28,194,31,194,32,194,40,194,41,194,43,194,45,194,47,194,49,194,50,194,52,194,72,194,80,194,81,194,84,194,88,194,96,194,101,194,108,194,109,194,112,194,116,194,124,194,125,194,127,194,129,194,136,194,137,194,144,194,152,194,155,194,157,194,164,194,165,194,168,194,172,194,173,194,180,194,181,194,183,194,185,194,220,194,221,194,224,194,227,194,228,194,235,194,236,194,237,194,239,194,241,194,246,194,248,194,249,194,251,194,252,194,0,195,8,195,9,195,12,195,13,195,19,195,20,195,21,195,24,195,28,195,36,195,37,195,40,195,41,195,69,195,104,195,105,195,108,195,112,195,114,195,120,195,121,195,124,195,125,195,132,195,136,195,140,195,192,195,216,195,217,195,220,195,223,195,224,195,226,195,232,195,233,195,237,195,244,195,245,195,248,195,8,196,16,196,36,196,44,196,48,196,52,196,60,196,61,196,72,196,100,196,101,196,104,196,108,196,116,196,117,196,121,196,128,196,148,196,156,196,184,196,188,196,233,196,240,196,241,196,244,196,248,196,250,196,255,196,0,197,1,197,12,197,16,197,20,197,28,197,40,197,41,197,44,197,48,197,56,197,57,197,59,197,61,197,68,197,69,197,72,197,73,197,74,197,76,197,77,197,78,197,83,197,84,197,85,197,87,197,88,197,89,197,93,197,94,197,96,197,97,197,100,197,104,197,112,197,113,197,115,197,116,197,117,197,124,197,125,197,128,197,132,197,135,197,140,197,141,197,143,197,145,197,149,197,151,197,152,197,156,197,160,197,169,197,180,197,181,197,184,197,185,197,187,197,188,197,189,197,190,197,196,197,197,197,198,197,199,197,200,197,201,197,202,197,204,197,206,197,208,197,209,197,212,197,216,197,224,197,225,197,227,197,229,197,236,197,237,197,238,197,240,197,244,197,246,197,247,197,252,197,253,197,254,197,255,197,0,198,1,198,5,198,6,198,7,198,8,198,12,198,16,198,24,198,25,198,27,198,28,198,36,198,37,198,40,198,44,198,45,198,46,198,48,198,51,198,52,198,53,198,55,198,57,198,59,198,64,198,65,198,68,198,72,198,80,198,81,198,83,198,84,198,85,198,92,198,93,198,96,198,108,198,111,198,113,198,120,198,121,198,124,198,128,198,136,198,137,198,139,198,141,198,148,198,149,198,152,198,156,198,164,198,165,198,167,198,169,198,176,198,177,198,180,198,184,198,185,198,186,198,192,198,193,198,195,198,197,198,204,198,205,198,208,198,212,198,220,198,221,198,224,198,225,198,232,198,233,198,236,198,240,198,248,198,249,198,253,198,4,199,5,199,8,199,12,199,20,199,21,199,23,199,25,199,32,199,33,199,36,199,40,199,48,199,49,199,51,199,53,199,55,199,60,199,61,199,64,199,68,199,74,199,76,199,77,199,79,199,81,199,82,199,83,199,84,199,85,199,86,199,87,199,88,199,92,199,96,199,104,199,107,199,116,199,117,199,120,199,124,199,125,199,126,199,131,199,132,199,133,199,135,199,136,199,137,199,138,199,142,199,144,199,145,199,148,199,150,199,151,199,152,199,154,199,160,199,161,199,163,199,164,199,165,199,166,199,172,199,173,199,176,199,180,199,188,199,189,199,191,199,192,199,193,199,200,199,201,199,204,199,206,199,208,199,216,199,221,199,228,199,232,199,236,199,0,200,1,200,4,200,8,200,10,200,16,200,17,200,19,200,21,200,22,200,28,200,29,200,32,200,36,200,44,200,45,200,47,200,49,200,56,200,60,200,64,200,72,200,73,200,76,200,77,200,84,200,112,200,113,200,116,200,120,200,122,200,128,200,129,200,131,200,133,200,134,200,135,200,139,200,140,200,141,200,148,200,157,200,159,200,161,200,168,200,188,200,189,200,196,200,200,200,204,200,212,200,213,200,215,200,217,200,224,200,225,200,228,200,245,200,252,200,253,200,0,201,4,201,5,201,6,201,12,201,13,201,15,201,17,201,24,201,44,201,52,201,80,201,81,201,84,201,88,201,96,201,97,201,99,201,108,201,112,201,116,201,124,201,136,201,137,201,140,201,144,201,152,201,153,201,155,201,157,201,192,201,193,201,196,201,199,201,200,201,202,201,208,201,209,201,211,201,213,201,214,201,217,201,218,201,220,201,221,201,224,201,226,201,228,201,231,201,236,201,237,201,239,201,240,201,241,201,248,201,249,201,252,201,0,202,8,202,9,202,11,202,12,202,13,202,20,202,24,202,41,202,76,202,77,202,80,202,84,202,92,202,93,202,95,202,96,202,97,202,104,202,125,202,132,202,152,202,188,202,189,202,192,202,196,202,204,202,205,202,207,202,209,202,211,202,216,202,217,202,224,202,236,202,244,202,8,203,16,203,20,203,24,203,32,203,33,203,65,203,72,203,73,203,76,203,80,203,88,203,89,203,93,203,100,203,120,203,121,203,156,203,184,203,212,203,228,203,231,203,233,203,12,204,13,204,16,204,20,204,28,204,29,204,33,204,34,204,39,204,40,204,41,204,44,204,46,204,48,204,56,204,57,204,59,204,60,204,61,204,62,204,68,204,69,204,72,204,76,204,84,204,85,204,87,204,88,204,89,204,96,204,100,204,102,204,104,204,112,204,117,204,152,204,153,204,156,204,160,204,168,204,169,204,171,204,172,204,173,204,180,204,181,204,184,204,188,204,196,204,197,204,199,204,201,204,208,204,212,204,228,204,236,204,240,204,1,205,8,205,9,205,12,205,16,205,24,205,25,205,27,205,29,205,36,205,40,205,44,205,57,205,92,205,96,205,100,205,108,205,109,205,111,205,113,205,120,205,136,205,148,205,149,205,152,205,156,205,164,205,165,205,167,205,169,205,176,205,196,205,204,205,208,205,232,205,236,205,240,205,248,205,249,205,251,205,253,205,4,206,8,206,12,206,20,206,25,206,32,206,33,206,36,206,40,206,48,206,49,206,51,206,53,206,88,206,89,206,92,206,95,206,96,206,97,206,104,206,105,206,107,206,109,206,116,206,117,206,120,206,124,206,132,206,133,206,135,206,137,206,144,206,145,206,148,206,152,206,160,206,161,206,163,206,164,206,165,206,172,206,173,206,193,206,228,206,229,206,232,206,235,206,236,206,244,206,245,206,247,206,248,206,249,206,0,207,1,207,4,207,8,207,16,207,17,207,19,207,21,207,28,207,32,207,36,207,44,207,45,207,47,207,48,207,49,207,56,207,84,207,85,207,88,207,92,207,100,207,101,207,103,207,105,207,112,207,113,207,116,207,120,207,128,207,133,207,140,207,161,207,168,207,176,207,196,207,224,207,225,207,228,207,232,207,240,207,241,207,243,207,245,207,252,207,0,208,4,208,17,208,24,208,45,208,52,208,53,208,56,208,60,208,68,208,69,208,71,208,73,208,80,208,84,208,88,208,96,208,108,208,109,208,112,208,116,208,124,208,125,208,129,208,164,208,165,208,168,208,172,208,180,208,181,208,183,208,185,208,192,208,193,208,196,208,200,208,201,208,208,208,209,208,211,208,212,208,213,208,220,208,221,208,224,208,228,208,236,208,237,208,239,208,240,208,241,208,248,208,13,209,48,209,49,209,52,209,56,209,58,209,64,209,65,209,67,209,68,209,69,209,76,209,77,209,80,209,84,209,92,209,93,209,95,209,97,209,104,209,108,209,124,209,132,209,136,209,160,209,161,209,164,209,168,209,176,209,177,209,179,209,181,209,186,209,188,209,192,209,216,209,244,209,248,209,7,210,9,210,16,210,44,210,45,210,48,210,52,210,60,210,61,210,63,210,65,210,72,210,92,210,100,210,128,210,129,210,132,210,136,210,144,210,145,210,149,210,156,210,160,210,164,210,172,210,177,210,184,210,185,210,188,210,191,210,192,210,194,210,200,210,201,210,203,210,212,210,216,210,220,210,228,210,229,210,240,210,241,210,244,210,248,210,0,211,1,211,3,211,5,211,12,211,13,211,14,211,16,211,20,211,22,211,28,211,29,211,31,211,32,211,33,211,37,211,40,211,41,211,44,211,48,211,56,211,57,211,59,211,60,211,61,211,68,211,69,211,124,211,125,211,128,211,132,211,140,211,141,211,143,211,144,211,145,211,152,211,153,211,156,211,160,211,168,211,169,211,171,211,173,211,180,211,184,211,188,211,196,211,197,211,200,211,201,211,208,211,216,211,225,211,227,211,236,211,237,211,240,211,244,211,252,211,253,211,255,211,1,212,8,212,29,212,64,212,68,212,92,212,96,212,100,212,109,212,111,212,120,212,121,212,124,212,127,212,128,212,130,212,136,212,137,212,139,212,141,212,148,212,169,212,204,212,208,212,212,212,220,212,223,212,232,212,236,212,240,212,248,212,251,212,253,212,4,213,8,213,12,213,20,213,21,213,23,213,60,213,61,213,64,213,68,213,76,213,77,213,79,213,81,213,88,213,89,213,92,213,96,213,101,213,104,213,105,213,107,213,109,213,116,213,117,213,120,213,124,213,132,213,133,213,135,213,136,213,137,213,144,213,165,213,200,213,201,213,204,213,208,213,210,213,216,213,217,213,219,213,221,213,228,213,229,213,232,213,236,213,244,213,245,213,247,213,249,213,0,214,1,214,4,214,8,214,16,214,17,214,19,214,20,214,21,214,28,214,32,214,36,214,45,214,56,214,57,214,60,214,64,214,69,214,72,214,73,214,75,214,77,214,81,214,84,214,85,214,88,214,92,214,103,214,105,214,112,214,113,214,116,214,131,214,133,214,140,214,141,214,144,214,148,214,157,214,159,214,161,214,168,214,172,214,176,214,185,214,187,214,196,214,197,214,200,214,204,214,209,214,212,214,215,214,217,214,224,214,228,214,232,214,240,214,245,214,252,214,253,214,0,215,4,215,17,215,24,215,25,215,28,215,32,215,40,215,41,215,43,215,45,215,52,215,53,215,56,215,60,215,68,215,71,215,73,215,80,215,81,215,84,215,86,215,87,215,88,215,89,215,96,215,97,215,99,215,101,215,105,215,108,215,112,215,116,215,124,215,125,215,129,215,136,215,137,215,140,215,144,215,152,215,153,215,155,215,157,215,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,61,79,115,79,71,80,249,80,160,82,239,83,117,84,229,84,9,86,193,90,182,91,135,102,182,103,183,103,239,103,76,107,194,115,194,117,60,122,219,130,4,131,87,136,136,136,54,138,200,140,207,141,251,142,230,143,213,153,59,82,116,83,4,84,106,96,100,97,188,107,207,115,26,129,186,137,210,137,163,149,131,79,10,82,190,88,120,89,230,89,114,94,121,94,199,97,192,99,70,103,236,103,127,104,151,111,78,118,11,119,245,120,8,122,255,122,33,124,157,128,110,130,113,130,235,138,147,149,107,78,157,85,247,102,52,110,163,120,237,122,91,132,16,137,78,135,168,151,216,82,78,87,42,88,76,93,31,97,190,97,33,98,98,101,209,103,68,106,27,110,24,117,179,117,227,118,176,119,58,125,175,144,81,148,82,148,149,159,35,83,172,92,50,117,219,128,64,146,152,149,91,82,8,88,220,89,161,92,23,93,183,94,58,95,74,95,119,97,95,108,122,117,134,117,224,124,115,125,177,125,140,127,84,129,33,130,145,133,65,137,27,139,252,146,77,150,71,156,203,78,247,78,11,80,241,81,79,88,55,97,62,97,104,97,57,101,234,105,17,111,165,117,134,118,214,118,135,123,165,130,203,132,0,249,167,147,139,149,128,85,162,91,81,87,1,249,179,124,185,127,181,145,40,80,187,83,69,92,232,93,210,98,110,99,218,100,231,100,32,110,172,112,91,121,221,141,30,142,2,249,125,144,69,146,248,146,126,78,246,78,101,80,254,93,250,94,6,97,87,105,113,129,84,134,71,142,117,147,43,154,94,78,145,80,112,103,64,104,9,81,141,82,146,82,162,106,188,119,16,146,212,158,171,82,47,96,242,143,72,80,169,97,237,99,202,100,60,104,132,106,192,111,136,129,161,137,148,150,5,88,125,114,172,114,4,117,121,125,109,126,169,128,139,137,116,139,99,144,81,157,137,98,122,108,84,111,80,125,58,127,35,138,124,81,74,97,157,123,25,139,87,146,140,147,172,78,211,79,30,80,190,80,6,81,193,82,205,82,127,83,112,87,131,88,154,94,145,95,118,97,172,97,206,100,108,101,111,102,187,102,244,102,151,104,135,109,133,112,241,112,159,116,165,116,202,116,217,117,108,120,236,120,223,122,246,122,69,125,147,125,21,128,63,128,27,129,150,131,102,139,21,143,21,144,225,147,3,152,56,152,90,154,232,155,194,79,83,85,58,88,81,89,99,91,70,92,184,96,18,98,66,104,176,104,232,104,170,110,76,117,120,118,206,120,61,122,251,124,107,126,124,126,8,138,161,138,63,140,142,150,196,157,228,83,233,83,74,84,113,84,250,86,209,89,100,91,59,92,171,94,247,98,55,101,69,101,114,101,160,102,175,103,193,105,189,108,252,117,144,118,126,119,63,122,148,127,3,128,161,128,143,129,230,130,253,130,240,131,193,133,49,136,180,136,165,138,3,249,156,143,46,147,199,150,103,152,216,154,19,159,237,84,155,101,242,102,143,104,64,122,55,140,96,157,240,86,100,87,17,93,6,102,177,104,205,104,254,110,40,116,158,136,228,155,104,108,4,249,168,154,155,79,108,81,113,81,159,82,84,91,229,93,80,96,109,96,241,98,167,99,59,101,217,115,122,122,163,134,162,140,143,151,50,78,225,91,8,98,156,103,220,116,209,121,211,131,135,138,178,138,232,141,78,144,75,147,70,152,211,94,232,105,255,133,237,144,5,249,160,81,152,91,236,91,99,97,250,104,62,107,76,112,47,116,216,116,161,123,80,127,197,131,192,137,171,140,220,149,40,153,46,82,93,96,236,98,2,144,138,79,73,81,33,83,217,88,227,94,224,102,56,109,154,112,194,114,214,115,80,123,241,128,91,148,102,83,155,99,107,127,86,78,128,80,74,88,222,88,42,96,39,97,208,98,208,105,65,155,143,91,24,125,177,128,95,143,164,78,209,80,172,84,172,85,12,91,160,93,231,93,42,101,78,101,33,104,75,106,225,114,142,118,239,119,94,125,249,127,160,129,78,133,223,134,3,143,78,143,202,144,3,153,85,154,171,155,24,78,69,78,93,78,199,78,241,79,119,81,254,82,64,83,227,83,229,83,142,84,20,86,117,87,162,87,199,91,135,93,208,94,252,97,216,98,81,101,184,103,233,103,203,105,80,107,198,107,236,107,66,108,157,110,120,112,215,114,150,115,3,116,191,119,233,119,118,122,127,125,9,128,252,129,5,130,10,130,223,130,98,136,51,139,252,140,192,142,17,144,177,144,100,146,182,146,210,153,69,154,233,156,215,157,156,159,11,87,64,92,202,131,160,151,171,151,180,158,27,84,152,122,164,127,217,136,205,142,225,144,0,88,72,92,152,99,159,122,174,91,19,95,121,122,174,122,142,130,172,142,38,80,56,82,248,82,119,83,8,87,243,98,114,99,10,107,195,109,55,119,165,83,87,115,104,133,118,142,213,149,58,103,195,106,112,111,109,138,204,142,75,153,6,249,119,102,120,107,180,140,60,155,7,249,235,83,45,87,78,89,198,99,251,105,234,115,69,120,186,122,197,122,254,124,117,132,143,137,115,141,53,144,168,149,251,82,71,87,71,117,96,123,204,131,30,146,8,249,88,106,75,81,75,82,135,82,31,98,216,104,117,105,153,150,197,80,164,82,228,82,195,97,164,101,57,104,255,105,126,116,75,123,185,130,235,131,178,137,57,139,209,143,73,153,9,249,202,78,151,89,210,100,17,102,142,106,52,116,129,121,189,121,169,130,126,136,127,136,95,137,10,249,38,147,11,79,202,83,37,96,113,98,114,108,26,125,102,125,152,78,98,81,220,119,175,128,1,79,14,79,118,81,128,81,220,85,104,86,59,87,250,87,252,87,20,89,71,89,147,89,196,91,144,92,14,93,241,93,126,94,204,95,128,98,215,101,227,101,30,103,31,103,94,103,203,104,196,104,95,106,58,107,35,108,125,108,130,108,199,109,152,115,38,116,42,116,130,116,163,116,120,117,127,117,129,120,239,120,65,121,71,121,72,121,122,121,149,123,0,125,186,125,136,127,6,128,45,128,140,128,24,138,79,139,72,140,119,141,33,147,36,147,226,152,81,153,14,154,15,154,101,154,146,158,202,125,118,79,9,84,238,98,84,104,209,145,171,85,58,81,11,249,12,249,28,90,230,97,13,249,207,98,255,98,14,249,15,249,16,249,17,249,18,249,19,249,163,144,20,249,21,249,22,249,23,249,24,249,254,138,25,249,26,249,27,249,28,249,150,102,29,249,86,113,30,249,31,249,227,150,32,249,79,99,122,99,87,83,33,249,143,103,96,105,115,110,34,249,55,117,35,249,36,249,37,249,13,125,38,249,39,249,114,136,202,86,24,90,40,249,41,249,42,249,43,249,44,249,67,78,45,249,103,81,72,89,240,103,16,128,46,249,115,89,116,94,154,100,202,121,245,95,108,96,200,98,123,99,231,91,215,91,170,82,47,249,116,89,41,95,18,96,48,249,49,249,50,249,89,116,51,249,52,249,53,249,54,249,55,249,56,249,209,153,57,249,58,249,59,249,60,249,61,249,62,249,63,249,64,249,65,249,66,249,67,249,195,111,68,249,69,249,191,129,178,143,241,96,70,249,71,249,102,129,72,249,73,249,63,92,74,249,75,249,76,249,77,249,78,249,79,249,80,249,81,249,233,90,37,138,123,103,16,125,82,249,83,249,84,249,85,249,86,249,87,249,253,128,88,249,89,249,60,92,229,108,63,83,186,110,26,89,54,131,57,78,182,78,70,79,174,85,24,87,199,88,86,95,183,101,230,101,128,106,181,107,77,110,237,119,239,122,30,124,222,125,203,134,146,136,50,145,91,147,187,100,190,111,122,115,184,117,84,144,86,85,77,87,186,97,212,100,199,102,225,109,91,110,109,111,185,111,240,117,67,128,189,129,65,133,131,137,199,138,90,139,31,147,147,108,83,117,84,123,15,142,93,144,16,85,2,88,88,88,98,94,7,98,158,100,224,104,118,117,214,124,179,135,232,158,227,78,136,87,110,87,39,89,13,92,177,92,54,94,133,95,52,98,225,100,179,115,250,129,139,136,184,140,138,150,219,158,133,91,183,95,179,96,18,80,0,82,48,82,22,87,53,88,87,88,14,92,96,92,246,92,139,93,166,94,146,95,188,96,17,99,137,99,23,100,67,104,249,104,194,106,216,109,33,110,212,110,228,111,254,113,220,118,121,119,177,121,59,122,4,132,169,137,237,140,243,141,72,142,3,144,20,144,83,144,253,144,77,147,118,150,220,151,210,107,6,112,88,114,162,114,104,115,99,119,191,121,228,123,155,126,128,139,169,88,199,96,102,101,253,101,190,102,140,108,30,113,201,113,90,140,19,152,109,78,129,122,221,78,172,81,205,81,213,82,12,84,167,97,113,103,80,104,223,104,30,109,124,111,188,117,179,119,229,122,244,128,99,132,133,146,92,81,151,101,92,103,147,103,216,117,199,122,115,131,90,249,70,140,23,144,45,152,111,92,192,129,154,130,65,144,111,144,13,146,151,95,157,93,89,106,200,113,123,118,73,123,228,133,4,139,39,145,48,154,135,85,246,97,91,249,105,118,133,127,63,134,186,135,248,136,143,144,92,249,27,109,217,112,222,115,97,125,61,132,93,249,106,145,241,153,94,249,130,78,117,83,4,107,18,107,62,112,27,114,45,134,30,158,76,82,163,143,80,93,229,100,44,101,22,107,235,111,67,124,156,126,205,133,100,137,189,137,201,98,216,129,31,136,202,94,23,103,106,109,252,114,5,116,111,116,130,135,222,144,134,79,13,93,160,95,10,132,183,81,160,99,101,117,174,78,6,80,105,81,201,81,129,104,17,106,174,124,177,124,231,124,111,130,210,138,27,143,207,145,182,79,55,81,245,82,66,84,236,94,110,97,62,98,197,101,218,106,254,111,42,121,220,133,35,136,173,149,98,154,106,154,151,158,206,158,155,82,198,102,119,107,29,112,43,121,98,143,66,151,144,97,0,98,35,101,35,111,73,113,137,116,244,125,111,128,238,132,38,143,35,144,74,147,189,81,23,82,163,82,12,109,200,112,194,136,201,94,130,101,174,107,194,111,62,124,117,115,228,78,54,79,249,86,95,249,186,92,186,93,28,96,178,115,45,123,154,127,206,127,70,128,30,144,52,146,246,150,72,151,24,152,97,159,139,79,167,111,174,121,180,145,183,150,222,82,96,249,136,100,196,100,211,106,94,111,24,112,16,114,231,118,1,128,6,134,92,134,239,141,5,143,50,151,111,155,250,157,117,158,140,120,127,121,160,125,201,131,4,147,127,158,147,158,214,138,223,88,4,95,39,103,39,112,207,116,96,124,126,128,33,81,40,112,98,114,202,120,194,140,218,140,244,140,247,150,134,78,218,80,238,91,214,94,153,101,206,113,66,118,173,119,74,128,252,132,124,144,39,155,141,159,216,88,65,90,98,92,19,106,218,109,15,111,59,118,47,125,55,126,30,133,56,137,228,147,75,150,137,82,210,101,243,103,180,105,65,109,156,110,15,112,9,116,96,116,89,117,36,118,107,120,44,139,94,152,109,81,46,98,120,150,150,79,43,80,25,93,234,109,184,125,42,143,139,95,68,97,23,104,97,249,134,150,210,82,139,128,220,81,204,81,94,105,28,122,190,125,241,131,117,150,218,79,41,82,152,83,15,84,14,85,101,92,167,96,78,103,168,104,108,109,129,114,248,114,6,116,131,116,98,249,226,117,108,124,121,127,184,127,137,131,207,136,225,136,204,145,208,145,226,150,201,155,29,84,126,111,208,113,152,116,250,133,170,142,163,150,87,156,159,158,151,103,203,109,51,116,232,129,22,151,44,120,203,122,32,123,146,124,105,100,106,116,242,117,188,120,232,120,172,153,84,155,187,158,222,91,85,94,32,111,156,129,171,131,136,144,7,78,77,83,41,90,210,93,78,95,98,97,61,99,105,102,252,102,255,110,43,111,99,112,158,119,44,132,19,133,59,136,19,143,69,153,59,156,28,85,185,98,43,103,171,108,9,131,106,137,122,151,161,78,132,89,216,95,217,95,27,103,178,125,84,127,146,130,43,131,189,131,30,143,153,144,203,87,185,89,146,90,208,91,39,102,154,103,133,104,207,107,100,113,117,127,183,140,227,140,129,144,69,155,8,129,138,140,76,150,64,154,165,158,95,91,19,108,27,115,242,118,223,118,12,132,170,81,147,137,77,81,149,81,201,82,201,104,148,108,4,119,32,119,191,125,236,125,98,151,181,158,197,110,17,133,165,81,13,84,125,84,14,102,157,102,39,105,159,110,191,118,145,119,23,131,194,132,159,135,105,145,152,146,244,156,130,136,174,79,146,81,223,82,198,89,61,94,85,97,120,100,121,100,174,102,208,103,33,106,205,107,219,107,95,114,97,114,65,116,56,119,219,119,23,128,188,130,5,131,0,139,40,139,140,140,40,103,144,108,103,114,238,118,102,119,70,122,169,157,127,107,146,108,34,89,38,103,153,132,111,83,147,88,153,89,223,94,207,99,52,102,115,103,58,110,43,115,215,122,215,130,40,147,217,82,235,93,174,97,203,97,10,98,199,98,171,100,224,101,89,105,102,107,203,107,33,113,247,115,93,117,70,126,30,130,2,131,106,133,163,138,191,140,39,151,97,157,168,88,216,158,17,80,14,82,59,84,79,85,135,101,118,108,10,125,11,125,94,128,138,134,128,149,239,150,255,82,149,108,105,114,115,84,154,90,62,92,75,93,76,95,174,95,42,103,182,104,99,105,60,110,68,110,9,119,115,124,142,127,135,133,14,139,247,143,97,151,244,158,183,92,182,96,13,97,171,97,79,101,251,101,252,101,17,108,239,108,159,115,201,115,225,125,148,149,198,91,28,135,16,139,93,82,90,83,205,98,15,100,178,100,52,103,56,106,202,108,192,115,158,116,148,123,149,124,27,126,138,129,54,130,132,133,235,143,249,150,193,153,52,79,74,83,205,83,219,83,204,98,44,100,0,101,145,101,195,105,238,108,88,111,237,115,84,117,34,118,228,118,252,118,208,120,251,120,44,121,70,125,44,130,224,135,212,143,18,152,239,152,195,82,212,98,165,100,36,110,81,111,124,118,203,141,177,145,98,146,238,154,67,155,35,80,141,80,74,87,168,89,40,92,71,94,119,95,63,98,62,101,185,101,193,101,9,102,139,103,156,105,194,110,197,120,33,125,170,128,128,129,43,130,179,130,161,132,140,134,42,138,23,139,166,144,50,150,144,159,13,80,243,79,99,249,249,87,152,95,220,98,146,99,111,103,67,110,25,113,195,118,204,128,218,128,244,136,245,136,25,137,224,140,41,143,77,145,106,150,47,79,112,79,27,94,207,103,34,104,125,118,126,118,68,155,97,94,10,106,105,113,212,113,106,117,100,249,65,126,67,133,233,133,220,152,16,79,79,123,112,127,165,149,225,81,6,94,181,104,62,108,78,108,219,108,175,114,196,123,3,131,213,108,58,116,251,80,136,82,193,88,216,100,151,106,167,116,86,118,167,120,23,134,226,149,57,151,101,249,94,83,1,95,138,139,168,143,175,143,138,144,37,82,165,119,73,156,8,159,25,78,2,80,117,81,91,92,119,94,30,102,58,102,196,103,197,104,179,112,1,117,197,117,201,121,221,122,39,143,32,153,8,154,221,79,33,88,49,88,246,91,110,102,101,107,17,109,122,110,125,111,228,115,43,117,233,131,220,136,19,137,92,139,20,143,15,79,213,80,16,83,92,83,147,91,169,95,13,103,143,121,121,129,47,131,20,133,7,137,134,137,57,143,59,143,165,153,18,156,44,103,118,78,248,79,73,89,1,92,239,92,240,92,103,99,210,104,253,112,162,113,43,116,43,126,236,132,2,135,34,144,210,146,243,156,13,78,216,78,239,79,133,80,86,82,111,82,38,84,144,84,224,87,43,89,102,90,90,91,117,91,204,91,156,94,102,249,118,98,119,101,167,101,110,109,165,110,54,114,38,123,63,124,54,127,80,129,81,129,154,129,64,130,153,130,169,131,3,138,160,140,230,140,251,140,116,141,186,141,232,144,220,145,28,150,68,150,217,153,231,156,23,83,6,82,41,84,116,86,179,88,84,89,110,89,255,95,164,97,110,98,16,102,126,108,26,113,198,118,137,124,222,124,27,125,172,130,193,140,240,150,103,249,91,79,23,95,127,95,194,98,41,93,11,103,218,104,124,120,67,126,108,157,21,78,153,80,21,83,42,83,81,83,131,89,98,90,135,94,178,96,138,97,73,98,121,98,144,101,135,103,167,105,212,107,214,107,215,107,216,107,184,108,104,249,53,116,250,117,18,120,145,120,213,121,216,121,131,124,203,125,225,127,165,128,62,129,194,129,242,131,26,135,232,136,185,138,108,139,187,140,25,145,94,151,219,152,59,159,172,86,42,91,108,95,140,101,179,106,175,107,92,109,241,111,21,112,93,114,173,115,167,140,211,140,59,152,145,97,55,108,88,128,1,154,77,78,139,78,155,78,213,78,58,79,60,79,127,79,223,79,255,80,242,83,248,83,6,85,227,85,219,86,235,88,98,89,17,90,235,91,250,91,4,92,243,93,43,94,153,95,29,96,104,99,156,101,175,101,246,103,251,103,173,104,123,107,153,108,215,108,35,110,9,112,69,115,2,120,62,121,64,121,96,121,193,121,233,123,23,125,114,125,134,128,13,130,142,131,209,132,199,134,223,136,80,138,94,138,29,139,220,140,102,141,173,143,170,144,252,152,223,153,157,158,74,82,105,249,20,103,106,249,152,80,42,82,113,92,99,101,85,108,202,115,35,117,157,117,151,123,156,132,120,145,48,151,119,78,146,100,186,107,94,113,169,133,9,78,107,249,73,103,238,104,23,110,159,130,24,133,107,136,247,99,129,111,18,146,175,152,10,78,183,80,207,80,31,81,70,85,170,85,23,86,64,91,25,92,224,92,56,94,138,94,160,94,194,94,243,96,81,104,97,106,88,110,61,114,64,114,192,114,248,118,101,121,177,123,212,127,243,136,244,137,115,138,97,140,222,140,28,151,94,88,189,116,253,140,199,85,108,249,97,122,34,125,114,130,114,114,31,117,37,117,109,249,25,123,133,88,251,88,188,93,143,94,182,94,144,95,85,96,146,98,127,99,77,101,145,102,217,102,248,102,22,104,242,104,128,114,94,116,110,123,110,125,214,125,114,127,229,128,18,130,175,133,127,137,147,138,29,144,228,146,205,158,32,159,21,89,109,89,45,94,220,96,20,102,115,102,144,103,80,108,197,109,95,111,243,119,169,120,198,132,203,145,43,147,217,78,202,80,72,81,132,85,11,91,163,91,71,98,126,101,203,101,50,110,125,113,1,116,68,116,135,116,191,116,108,118,170,121,218,125,85,126,168,127,122,129,179,129,57,130,26,134,236,135,117,138,227,141,120,144,145,146,37,148,77,153,174,155,104,83,81,92,84,105,196,108,41,109,43,110,12,130,155,133,59,137,45,138,170,138,234,150,103,159,97,82,185,102,178,107,150,126,254,135,13,141,131,149,93,150,29,101,137,109,238,113,110,249,206,87,211,89,172,91,39,96,250,96,16,98,31,102,95,102,41,115,249,115,219,118,1,119,108,123,86,128,114,128,101,129,160,138,146,145,22,78,226,82,114,107,23,109,5,122,57,123,48,125,111,249,176,140,236,83,47,86,81,88,181,91,15,92,17,92,226,93,64,98,131,99,20,100,45,102,179,104,188,108,136,109,175,110,31,112,164,112,210,113,38,117,143,117,142,117,25,118,17,123,224,123,43,124,32,125,57,125,44,133,109,133,7,134,52,138,13,144,97,144,181,144,183,146,246,151,55,154,215,79,108,92,95,103,145,109,159,124,140,126,22,139,22,141,31,144,107,91,253,93,13,100,192,132,92,144,225,152,135,115,139,91,154,96,126,103,222,109,31,138,166,138,1,144,12,152,55,82,112,249,81,112,142,120,150,147,112,136,215,145,238,79,215,83,253,85,218,86,130,87,253,88,194,90,136,91,171,92,192,92,37,94,1,97,13,98,75,98,136,99,28,100,54,101,120,101,57,106,138,107,52,108,25,109,49,111,231,113,233,114,120,115,7,116,178,116,38,118,97,119,192,121,87,122,234,122,185,124,143,125,172,125,97,126,158,127,41,129,49,131,144,132,218,132,234,133,150,136,176,138,144,139,56,143,66,144,131,144,108,145,150,146,185,146,139,150,167,150,168,150,214,150,0,151,8,152,150,153,211,154,26,155,212,83,126,88,25,89,112,91,191,91,209,109,90,111,159,113,33,116,185,116,133,128,253,131,225,93,135,95,170,95,66,96,236,101,18,104,111,105,83,106,137,107,53,109,243,109,227,115,254,118,172,119,77,123,20,125,35,129,28,130,64,131,244,132,99,133,98,138,196,138,135,145,30,147,6,152,180,153,12,98,83,136,240,143,101,146,7,93,39,93,105,93,95,116,157,129,104,135,213,111,254,98,210,127,54,137,114,137,30,78,88,78,231,80,221,82,71,83,127,98,7,102,105,126,5,136,94,150,141,79,25,83,54,86,203,89,164,90,56,92,78,92,77,92,2,94,17,95,67,96,189,101,47,102,66,102,190,103,244,103,28,115,226,119,58,121,197,127,148,132,205,132,150,137,102,138,105,138,225,138,85,140,122,140,244,87,212,91,15,95,111,96,237,98,13,105,150,107,92,110,132,113,210,123,85,135,88,139,254,142,223,152,254,152,56,79,129,79,225,79,123,84,32,90,184,91,60,97,176,101,104,102,252,113,51,117,94,121,51,125,78,129,227,129,152,131,170,133,206,133,3,135,10,138,171,142,155,143,113,249,197,143,49,89,164,91,230,91,137,96,233,91,11,92,195,95,129,108,114,249,241,109,11,112,26,117,175,130,246,138,192,78,65,83,115,249,217,150,15,108,158,78,196,79,82,81,94,85,37,90,232,92,17,98,89,114,189,130,170,131,254,134,89,136,29,138,63,150,197,150,19,153,9,157,93,157,10,88,179,92,189,93,68,94,225,96,21,97,225,99,2,106,37,110,2,145,84,147,78,152,16,156,119,159,137,91,184,92,9,99,79,102,72,104,60,119,193,150,141,151,84,152,159,155,161,101,1,139,203,142,188,149,53,85,169,92,214,93,181,94,151,102,76,118,244,131,199,149,211,88,188,98,206,114,40,157,240,78,46,89,15,96,59,102,131,107,231,121,38,157,147,83,192,84,195,87,22,93,27,97,214,102,175,109,141,120,126,130,152,150,68,151,132,83,124,98,150,99,178,109,10,126,75,129,77,152,251,106,76,127,175,157,26,158,95,78,59,80,182,81,28,89,249,96,246,99,48,105,58,114,54,128,116,249,206,145,49,95,117,249,118,249,4,125,229,130,111,132,187,132,229,133,141,142,119,249,111,79,120,249,121,249,228,88,67,91,89,96,218,99,24,101,109,101,152,102,122,249,74,105,35,106,11,109,1,112,108,113,210,117,13,118,179,121,112,122,123,249,138,127,124,249,68,137,125,249,147,139,192,145,125,150,126,249,10,153,4,87,161,95,188,101,1,111,0,118,166,121,158,138,173,153,90,155,108,159,4,81,182,97,145,98,141,106,198,129,67,80,48,88,102,95,9,113,0,138,250,138,124,91,22,134,250,79,60,81,180,86,68,89,169,99,249,109,170,93,109,105,134,81,136,78,89,79,127,249,128,249,129,249,130,89,130,249,131,249,95,107,93,108,132,249,181,116,22,121,133,249,7,130,69,130,57,131,63,143,93,143,134,249,24,153,135,249,136,249,137,249,166,78,138,249,223,87,121,95,19,102,139,249,140,249,171,117,121,126,111,139,141,249,6,144,91,154,165,86,39,88,248,89,31,90,180,91,142,249,246,94,143,249,144,249,80,99,59,99,145,249,61,105,135,108,191,108,142,109,147,109,245,109,20,111,146,249,223,112,54,113,89,113,147,249,195,113,213,113,148,249,79,120,111,120,149,249,117,123,227,125,150,249,47,126,151,249,77,136,223,142,152,249,153,249,154,249,91,146,155,249,246,156,156,249,157,249,158,249,133,96,133,109,159,249,177,113,160,249,161,249,177,149,173,83,162,249,163,249,164,249,211,103,165,249,142,112,48,113,48,116,118,130,210,130,166,249,187,149,229,154,125,158,196,102,167,249,193,113,73,132,168,249,169,249,75,88,170,249,171,249,184,93,113,95,172,249,32,102,142,102,121,105,174,105,56,108,243,108,54,110,65,111,218,111,27,112,47,112,80,113,223,113,112,115,173,249,91,116,174,249,212,116,200,118,78,122,147,126,175,249,176,249,241,130,96,138,206,143,177,249,72,147,178,249,25,151,179,249,180,249,66,78,42,80,181,249,8,82,225,83,243,102,109,108,202,111,10,115,127,119,98,122,174,130,221,133,2,134,182,249,212,136,99,138,125,139,107,140,183,249,179,146,184,249,19,151,16,152,148,78,13,79,201,79,178,80,72,83,62,84,51,84,218,85,98,88,186,88,103,89,27,90,228,91,159,96,185,249,202,97,86,101,255,101,100,102,167,104,90,108,179,111,207,112,172,113,82,115,125,123,8,135,164,138,50,156,7,159,75,92,131,108,68,115,137,115,58,146,171,110,101,116,31,118,105,122,21,126,10,134,64,81,197,88,193,100,238,116,21,117,112,118,193,127,149,144,205,150,84,153,38,110,230,116,169,122,170,122,229,129,217,134,120,135,27,138,73,90,140,91,155,91,161,104,0,105,99,109,169,115,19,116,44,116,151,120,233,125,235,127,24,129,85,129,158,131,76,140,46,150,17,152,240,102,128,95,250,101,137,103,106,108,139,115,45,80,3,90,106,107,238,119,22,89,108,93,205,93,37,115,79,117,186,249,187,249,229,80,249,81,47,88,45,89,150,89,218,89,229,91,188,249,189,249,162,93,215,98,22,100,147,100,254,100,190,249,220,102,191,249,72,106,192,249,255,113,100,116,193,249,136,122,175,122,71,126,94,126,0,128,112,129,194,249,239,135,129,137,32,139,89,144,195,249,128,144,82,153,126,97,50,107,116,109,31,126,37,137,177,143,209,79,173,80,151,81,199,82,199,87,137,88,185,91,184,94,66,97,149,105,140,109,103,110,182,110,148,113,98,116,40,117,44,117,115,128,56,131,201,132,10,142,148,147,222,147,196,249,142,78,81,79,118,80,42,81,200,83,203,83,243,83,135,91,211,91,36,92,26,97,130,97,244,101,91,114,151,115,64,116,194,118,80,121,145,121,185,121,6,125,189,127,139,130,213,133,94,134,194,143,71,144,245,144,234,145,133,150,232,150,233,150,214,82,103,95,237,101,49,102,47,104,92,113,54,122,193,144,10,152,145,78,197,249,82,106,158,107,144,111,137,113,24,128,184,130,83,133,75,144,149,150,242,150,251,151,26,133,49,155,144,78,138,113,196,150,67,81,159,83,225,84,19,87,18,87,163,87,155,90,196,90,195,91,40,96,63,97,244,99,133,108,57,109,114,110,144,110,48,114,63,115,87,116,209,130,129,136,69,143,96,144,198,249,98,150,88,152,27,157,8,103,138,141,94,146,77,79,73,80,222,80,113,83,13,87,212,89,1,90,9,92,112,97,144,102,45,110,50,114,75,116,239,125,195,128,14,132,102,132,63,133,95,135,91,136,24,137,2,139,85,144,203,151,79,155,115,78,145,79,18,81,106,81,199,249,47,85,169,85,122,91,165,91,124,94,125,94,190,94,160,96,223,96,8,97,9,97,196,99,56,101,9,103,200,249,212,103,218,103,201,249,97,105,98,105,185,108,39,109,202,249,56,110,203,249,225,111,54,115,55,115,204,249,92,116,49,117,205,249,82,118,206,249,207,249,173,125,254,129,56,132,213,136,152,138,219,138,237,138,48,142,66,142,74,144,62,144,122,144,73,145,201,145,110,147,208,249,209,249,9,88,210,249,211,107,137,128,178,128,211,249,212,249,65,81,107,89,57,92,213,249,214,249,100,111,167,115,228,128,7,141,215,249,23,146,143,149,216,249,217,249,218,249,219,249,127,128,14,98,28,112,104,125,141,135,220,249,160,87,105,96,71,97,183,107,190,138,128,146,177,150,89,78,31,84,235,109,45,133,112,150,243,151,238,152,214,99,227,108,145,144,221,81,201,97,186,129,249,157,157,79,26,80,0,81,156,91,15,97,255,97,236,100,5,105,197,107,145,117,227,119,169,127,100,130,143,133,251,135,99,136,188,138,112,139,171,145,140,78,229,78,10,79,221,249,222,249,55,89,232,89,223,249,242,93,27,95,91,95,33,96,224,249,225,249,226,249,227,249,62,114,229,115,228,249,112,117,205,117,229,249,251,121,230,249,12,128,51,128,132,128,225,130,81,131,231,249,232,249,189,140,179,140,135,144,233,249,234,249,244,152,12,153,235,249,236,249,55,112,202,118,202,127,204,127,252,127,26,139,186,78,193,78,3,82,112,83,237,249,189,84,224,86,251,89,197,91,21,95,205,95,110,110,238,249,239,249,106,125,53,131,240,249,147,134,141,138,241,249,109,151,119,151,242,249,243,249,0,78,90,79,126,79,249,88,229,101,162,110,56,144,176,147,185,153,251,78,236,88,138,89,217,89,65,96,244,249,245,249,20,122,246,249,79,131,195,140,101,81,68,83,247,249,248,249,249,249,205,78,105,82,85,91,191,130,212,78,58,82,168,84,201,89,255,89,80,91,87,91,92,91,99,96,72,97,203,110,153,112,110,113,134,115,247,116,181,117,193,120])
.concat([43,125,5,128,234,129,40,131,23,133,201,133,238,138,199,140,204,150,92,79,250,82,188,86,171,101,40,102,124,112,184,112,53,114,189,125,141,130,76,145,192,150,114,157,113,91,231,104,152,107,122,111,222,118,145,92,171,102,91,111,180,123,42,124,54,136,220,150,8,78,215,78,32,83,52,88,187,88,239,88,108,89,7,92,51,94,132,94,53,95,140,99,178,102,86,103,31,106,163,106,12,107,63,111,70,114,250,249,80,115,139,116,224,122,167,124,120,129,223,129,231,129,138,131,108,132,35,133,148,133,207,133,221,136,19,141,172,145,119,149,156,150,141,81,201,84,40,87,176,91,77,98,80,103,61,104,147,104,61,110,211,110,125,112,33,126,193,136,161,140,9,143,75,159,78,159,45,114,143,123,205,138,26,147,71,79,78,79,50,81,128,84,208,89,149,94,181,98,117,103,110,105,23,106,174,108,26,110,217,114,42,115,189,117,184,123,53,125,231,130,249,131,87,132,247,133,91,138,175,140,135,142,25,144,184,144,206,150,95,159,227,82,10,84,225,90,194,91,88,100,117,101,244,110,196,114,251,249,132,118,77,122,27,123,77,124,62,126,223,127,123,131,43,139,202,140,100,141,225,141,95,142,234,143,249,143,105,144,209,147,67,79,122,79,179,80,104,81,120,81,77,82,106,82,97,88,124,88,96,89,8,92,85,92,219,94,155,96,48,98,19,104,191,107,8,108,177,111,78,113,32,116,48,117,56,117,81,117,114,118,76,123,139,123,173,123,198,123,143,126,110,138,62,143,73,143,63,146,147,146,34,147,43,148,251,150,90,152,107,152,30,153,7,82,42,98,152,98,89,109,100,118,202,122,192,123,118,125,96,83,190,92,151,94,56,111,185,112,152,124,17,151,142,155,222,158,165,99,122,100,118,135,1,78,149,78,173,78,92,80,117,80,72,84,195,89,154,91,64,94,173,94,247,94,129,95,197,96,58,99,63,101,116,101,204,101,118,102,120,102,254,103,104,105,137,106,99,107,64,108,192,109,232,109,31,110,94,110,30,112,161,112,142,115,253,115,58,117,91,119,135,120,142,121,11,122,125,122,190,124,142,125,71,130,2,138,234,138,158,140,45,145,74,145,216,145,102,146,204,146,32,147,6,151,86,151,92,151,2,152,14,159,54,82,145,82,124,85,36,88,29,94,31,95,140,96,208,99,175,104,223,111,109,121,44,123,205,129,186,133,253,136,248,138,68,142,141,145,100,150,155,150,61,151,76,152,74,159,206,79,70,81,203,81,169,82,50,86,20,95,107,95,170,99,205,100,233,101,65,102,250,102,249,102,29,103,157,104,215,104,253,105,21,111,110,111,103,113,229,113,42,114,170,116,58,119,86,121,90,121,223,121,32,122,149,122,151,124,223,124,68,125,112,126,135,128,251,133,164,134,84,138,191,138,153,141,129,142,32,144,109,144,227,145,59,150,213,150,229,156,207,101,7,124,179,141,195,147,88,91,10,92,82,83,217,98,29,115,39,80,151,91,158,95,176,96,107,97,213,104,217,109,46,116,46,122,66,125,156,125,49,126,107,129,42,142,53,142,126,147,24,148,80,79,80,87,230,93,167,94,43,99,106,127,59,78,79,79,143,79,90,80,221,89,196,128,106,84,104,84,254,85,79,89,153,91,222,93,218,94,93,102,49,103,241,103,42,104,232,108,50,109,74,110,141,111,183,112,224,115,135,117,76,124,2,125,44,125,162,125,31,130,219,134,59,138,133,138,112,141,138,142,51,143,49,144,78,145,82,145,68,148,208,153,249,122,165,124,202,79,1,81,198,81,200,87,239,91,251,92,89,102,61,106,90,109,150,110,236,111,12,113,111,117,227,122,34,136,33,144,117,144,203,150,255,153,1,131,45,78,242,78,70,136,205,145,125,83,219,106,107,105,65,108,122,132,158,88,142,97,254,102,239,98,221,112,17,117,199,117,82,126,184,132,73,139,8,141,75,78,234,83,171,84,48,87,64,87,215,95,1,99,7,99,111,100,47,101,232,101,122,102,157,103,179,103,98,107,96,108,154,108,44,111,229,119,37,120,73,121,87,121,25,125,162,128,2,129,243,129,157,130,183,130,24,135,140,138,252,249,4,141,190,141,114,144,244,118,25,122,55,122,84,126,119,128,7,85,212,85,117,88,47,99,34,100,73,102,75,102,109,104,155,105,132,107,37,109,177,110,205,115,104,116,161,116,91,117,185,117,225,118,30,119,139,119,230,121,9,126,29,126,251,129,47,133,151,136,58,138,209,140,235,142,176,143,50,144,173,147,99,150,115,150,7,151,132,79,241,83,234,89,201,90,25,94,78,104,198,116,190,117,233,121,146,122,163,129,237,134,234,140,204,141,237,143,159,101,21,103,253,249,247,87,87,111,221,125,47,143,246,147,198,150,181,95,242,97,132,111,20,78,152,79,31,80,201,83,223,85,111,93,238,93,33,107,100,107,203,120,154,123,254,249,73,142,202,142,110,144,73,99,62,100,64,119,132,122,47,147,127,148,106,159,176,100,175,111,230,113,168,116,218,116,196,122,18,124,130,126,178,124,152,126,154,139,10,141,125,148,16,153,76,153,57,82,223,91,230,100,45,103,46,125,237,80,195,83,121,88,88,97,89,97,250,97,172,101,217,122,146,139,150,139,9,80,33,80,117,82,49,85,60,90,224,94,112,95,52,97,94,101,12,102,54,102,162,102,205,105,196,110,50,111,22,115,33,118,147,122,57,129,89,130,214,131,188,132,181,80,240,87,192,91,232,91,105,95,161,99,38,120,181,125,220,131,33,133,199,145,245,145,138,81,245,103,86,123,172,140,196,81,187,89,189,96,85,134,28,80,255,249,84,82,58,92,125,97,26,98,211,98,242,100,165,101,204,110,32,118,10,129,96,142,95,150,187,150,223,78,67,83,152,85,41,89,221,93,197,100,201,108,250,109,148,115,127,122,27,130,166,133,228,140,16,142,119,144,231,145,225,149,33,150,198,151,248,81,242,84,134,85,185,95,164,100,136,111,180,125,31,143,77,143,53,148,201,80,22,92,190,108,251,109,27,117,187,119,61,124,100,124,121,138,194,138,30,88,190,89,22,94,119,99,82,114,138,117,107,119,220,138,188,140,18,143,243,94,116,102,248,109,125,128,193,131,203,138,81,151,214,155,0,250,67,82,255,102,149,109,239,110,224,125,230,138,46,144,94,144,212,154,29,82,127,82,232,84,148,97,132,98,219,98,162,104,18,105,90,105,53,106,146,112,38,113,93,120,1,121,14,121,210,121,13,122,150,128,120,130,213,130,73,131,73,133,130,140,133,141,98,145,139,145,174,145,195,79,209,86,237,113,215,119,0,135,248,137,248,91,214,95,81,103,168,144,226,83,90,88,245,91,164,96,129,97,96,100,61,126,112,128,37,133,131,146,174,100,172,80,20,93,0,103,156,88,189,98,168,99,14,105,120,105,30,106,107,110,186,118,203,121,187,130,41,132,207,138,168,141,253,143,18,145,75,145,156,145,16,147,24,147,154,147,219,150,54,154,13,156,17,78,92,117,93,121,250,122,81,123,201,123,46,126,196,132,89,142,116,142,248,142,16,144,37,102,63,105,67,116,250,81,46,103,220,158,69,81,224,95,150,108,242,135,93,136,119,136,180,96,181,129,3,132,5,141,214,83,57,84,52,86,54,90,49,92,138,112,224,127,90,128,6,129,237,129,163,141,137,145,95,154,242,157,116,80,196,78,160,83,251,96,44,110,100,92,136,79,36,80,228,85,217,92,95,94,101,96,148,104,187,108,196,109,190,113,212,117,244,117,97,118,26,122,73,122,199,125,251,125,110,127,244,129,169,134,28,143,201,150,179,153,82,159,71,82,197,82,237,152,170,137,3,78,210,103,6,111,181,79,226,91,149,103,136,108,120,109,27,116,39,120,221,145,124,147,196,135,228,121,49,122,235,95,214,78,164,84,62,85,174,88,165,89,240,96,83,98,214,98,54,103,85,105,53,130,64,150,177,153,221,153,44,80,83,83,68,85,124,87,1,250,88,98,2,250,226,100,107,102,221,103,193,111,239,111,34,116,56,116,23,138,56,148,81,84,6,86,102,87,72,95,154,97,78,107,88,112,173,112,187,125,149,138,106,89,43,129,162,99,8,119,61,128,170,140,84,88,45,100,187,105,149,91,17,94,111,110,3,250,105,133,76,81,240,83,42,89,32,96,75,97,134,107,112,108,240,108,30,123,206,128,212,130,198,141,176,144,177,152,4,250,199,100,164,111,145,100,4,101,78,81,16,84,31,87,14,138,95,97,118,104,5,250,219,117,82,123,113,125,26,144,6,88,204,105,127,129,42,137,0,144,57,152,120,80,87,89,172,89,149,98,15,144,42,155,93,97,121,114,214,149,97,87,70,90,244,93,138,98,173,100,250,100,119,103,226,108,62,109,44,114,54,116,52,120,119,127,173,130,219,141,23,152,36,82,66,87,127,103,72,114,227,116,169,140,166,143,17,146,42,150,107,81,237,83,76,99,105,79,4,85,150,96,87,101,155,108,127,109,76,114,253,114,23,122,135,137,157,140,109,95,142,111,249,112,168,129,14,97,191,79,79,80,65,98,71,114,199,123,232,125,233,127,77,144,173,151,25,154,182,140,106,87,115,94,176,103,13,132,85,138,32,84,22,91,99,94,226,94,10,95,131,101,186,128,61,133,137,149,91,150,72,79,5,83,13,83,15,83,134,84,250,84,3,87,3,94,22,96,155,98,177,98,85,99,6,250,225,108,102,109,177,117,50,120,222,128,47,129,222,130,97,132,178,132,141,136,18,137,11,144,234,146,253,152,145,155,69,94,180,102,221,102,17,112,6,114,7,250,245,79,125,82,106,95,83,97,83,103,25,106,2,111,226,116,104,121,104,136,121,140,199,152,196,152,67,154,193,84,31,122,83,105,247,138,74,140,168,152,174,153,124,95,171,98,178,117,174,118,171,136,127,144,66,150,57,83,60,95,197,95,204,108,204,115,98,117,139,117,70,123,254,130,157,153,79,78,60,144,11,78,85,79,166,83,15,89,200,94,48,102,179,108,85,116,119,131,102,135,192,140,80,144,30,151,21,156,209,88,120,91,80,134,20,139,180,157,210,91,104,96,141,96,241,101,87,108,34,111,163,111,26,112,85,127,240,127,145,149,146,149,80,150,211,151,114,82,68,143,253,81,43,84,184,84,99,85,138,85,187,106,181,109,216,125,102,130,156,146,119,150,121,158,8,84,200,84,210,118,228,134,164,149,212,149,92,150,162,78,9,79,238,89,230,90,247,93,82,96,151,98,109,103,65,104,134,108,47,110,56,127,155,128,42,130,8,250,9,250,5,152,165,78,85,80,179,84,147,87,90,89,105,91,179,91,200,97,119,105,119,109,35,112,249,135,227,137,114,138,231,138,130,144,237,153,184,154,190,82,56,104,22,80,120,94,79,103,71,131,76,136,171,78,17,84,174,86,230,115,21,145,255,151,9,153,87,153,153,153,83,86,159,88,91,134,49,138,178,97,246,106,123,115,210,142,71,107,170,150,87,154,85,89,0,114,107,141,105,151,212,79,244,92,38,95,248,97,91,102,235,108,171,112,132,115,185,115,254,115,41,119,77,119,67,125,98,125,35,126,55,130,82,136,10,250,226,140,73,146,111,152,81,91,116,122,64,136,1,152,204,90,224,79,84,83,62,89,253,92,62,99,121,109,249,114,5,129,7,129,162,131,207,146,48,152,168,78,68,81,17,82,139,87,98,95,194,108,206,110,5,112,80,112,175,112,146,113,233,115,105,116,74,131,162,135,97,136,8,144,162,144,163,147,168,153,110,81,87,95,224,96,103,97,179,102,89,133,74,142,175,145,139,151,78,78,146,78,124,84,213,88,250,88,125,89,181,92,39,95,54,98,72,98,10,102,103,102,235,107,105,109,207,109,86,110,248,110,148,111,224,111,233,111,93,112,208,114,37,116,90,116,224,116,147,118,92,121,202,124,30,126,225,128,166,130,107,132,191,132,78,134,95,134,116,135,119,139,106,140,172,147,0,152,101,152,209,96,22,98,119,145,90,90,15,102,247,109,62,110,63,116,66,155,253,95,218,96,15,123,196,84,24,95,94,108,211,108,42,109,216,112,5,125,121,134,12,138,59,157,22,83,140,84,5,91,58,106,107,112,117,117,141,121,190,121,177,130,239,131,113,138,65,139,168,140,116,151,11,250,244,100,43,101,186,120,187,120,107,122,56,78,154,85,80,89,166,91,123,94,163,96,219,99,97,107,101,102,83,104,25,110,101,113,176,116,8,125,132,144,105,154,37,156,59,109,209,110,62,115,65,140,202,149,240,81,76,94,168,95,77,96,246,96,48,97,76,97,67,102,68,102,165,105,193,108,95,110,201,110,98,111,76,113,156,116,135,118,193,123,39,124,82,131,87,135,81,144,141,150,195,158,47,83,222,86,251,94,138,95,98,96,148,96,247,97,102,102,3,103,156,106,238,109,174,111,112,112,106,115,106,126,190,129,52,131,212,134,168,138,196,140,131,82,114,115,150,91,107,106,4,148,238,84,134,86,93,91,72,101,133,101,201,102,159,104,141,109,198,109,59,114,180,128,117,145,77,154,175,79,25,80,154,83,14,84,60,84,137,85,197,85,63,94,140,95,61,103,102,113,221,115,5,144,219,82,243,82,100,88,206,88,4,113,143,113,251,113,176,133,19,138,136,102,168,133,167,85,132,102,74,113,49,132,73,83,153,85,193,107,89,95,189,95,238,99,137,102,71,113,241,138,29,143,190,158,17,79,58,100,203,112,102,117,103,134,100,96,78,139,248,157,71,81,246,81,8,83,54,109,248,128,209,158,21,102,35,107,152,112,213,117,3,84,121,92,7,125,22,138,32,107,61,107,70,107,56,84,112,96,61,109,213,127,8,130,214,80,222,81,156,85,107,86,205,86,236,89,9,91,12,94,153,97,152,97,49,98,94,102,230,102,153,113,185,113,186,113,167,114,167,121,0,122,178,127,112,138,0,0,0,0,0,1,2,3,4,5,6,6,7,2,1,0,10,10,4,5,6,7,8,9,10,11,12,9,14,11,12,13,14,15,16,13,16,17,18,19,20,21,22,22,23,23,24,24,25,25,26,26,27,27,27,28,28,28,29,29,29,30,30,30,31,31,31,31,32,32,32,32,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,0,0,0,0,0,1,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,14,15,15,15,16,16,16,16,17,17,17,17,17,18,18,18,18,18,18,18,18,18,18,18,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,0,0,48,1,48,2,48,12,255,14,255,251,48,26,255,27,255,31,255,1,255,155,48,156,48,180,0,64,255,168,0,62,255,227,255,63,255,253,48,254,48,157,48,158,48,3,48,221,78,5,48,6,48,7,48,252,48,21,32,16,32,15,255,92,0,28,48,22,32,92,255,38,32,37,32,24,32,25,32,28,32,29,32,8,255,9,255,20,48,21,48,59,255,61,255,91,255,93,255,8,48,9,48,10,48,11,48,12,48,13,48,14,48,15,48,16,48,17,48,11,255,18,34,177,0,215,0,247,0,29,255,96,34,28,255,30,255,102,34,103,34,30,34,52,34,66,38,64,38,176,0,50,32,51,32,3,33,229,255,4,255,162,0,163,0,5,255,3,255,6,255,10,255,32,255,167,0,6,38,5,38,203,37,207,37,206,37,199,37,198,37,161,37,160,37,179,37,178,37,189,37,188,37,59,32,18,48,146,33,144,33,145,33,147,33,19,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,34,11,34,134,34,135,34,130,34,131,34,42,34,41,34,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,39,34,40,34,172,0,210,33,212,33,0,34,3,34,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,34,165,34,18,35,2,34,7,34,97,34,82,34,106,34,107,34,26,34,61,34,29,34,53,34,43,34,44,34,0,0,0,0,0,0,0,0,0,0,0,0,0,0,43,33,48,32,111,38,109,38,106,38,32,32,33,32,182,0,0,0,0,0,0,0,0,0,239,37,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,255,17,255,18,255,19,255,20,255,21,255,22,255,23,255,24,255,25,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,33,255,34,255,35,255,36,255,37,255,38,255,39,255,40,255,41,255,42,255,43,255,44,255,45,255,46,255,47,255,48,255,49,255,50,255,51,255,52,255,53,255,54,255,55,255,56,255,57,255,58,255,0,0,0,0,0,0,0,0,0,0,0,0,65,255,66,255,67,255,68,255,69,255,70,255,71,255,72,255,73,255,74,255,75,255,76,255,77,255,78,255,79,255,80,255,81,255,82,255,83,255,84,255,85,255,86,255,87,255,88,255,89,255,90,255,0,0,0,0,0,0,0,0,65,48,66,48,67,48,68,48,69,48,70,48,71,48,72,48,73,48,74,48,75,48,76,48,77,48,78,48,79,48,80,48,81,48,82,48,83,48,84,48,85,48,86,48,87,48,88,48,89,48,90,48,91,48,92,48,93,48,94,48,95,48,96,48,97,48,98,48,99,48,100,48,101,48,102,48,103,48,104,48,105,48,106,48,107,48,108,48,109,48,110,48,111,48,112,48,113,48,114,48,115,48,116,48,117,48,118,48,119,48,120,48,121,48,122,48,123,48,124,48,125,48,126,48,127,48,128,48,129,48,130,48,131,48,132,48,133,48,134,48,135,48,136,48,137,48,138,48,139,48,140,48,141,48,142,48,143,48,144,48,145,48,146,48,147,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,161,48,162,48,163,48,164,48,165,48,166,48,167,48,168,48,169,48,170,48,171,48,172,48,173,48,174,48,175,48,176,48,177,48,178,48,179,48,180,48,181,48,182,48,183,48,184,48,185,48,186,48,187,48,188,48,189,48,190,48,191,48,192,48,193,48,194,48,195,48,196,48,197,48,198,48,199,48,200,48,201,48,202,48,203,48,204,48,205,48,206,48,207,48,208,48,209,48,210,48,211,48,212,48,213,48,214,48,215,48,216,48,217,48,218,48,219,48,220,48,221,48,222,48,223,48,224,48,225,48,226,48,227,48,228,48,229,48,230,48,231,48,232,48,233,48,234,48,235,48,236,48,237,48,238,48,239,48,240,48,241,48,242,48,243,48,244,48,245,48,246,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,145,3,146,3,147,3,148,3,149,3,150,3,151,3,152,3,153,3,154,3,155,3,156,3,157,3,158,3,159,3,160,3,161,3,163,3,164,3,165,3,166,3,167,3,168,3,169,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,177,3,178,3,179,3,180,3,181,3,182,3,183,3,184,3,185,3,186,3,187,3,188,3,189,3,190,3,191,3,192,3,193,3,195,3,196,3,197,3,198,3,199,3,200,3,201,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,4,17,4,18,4,19,4,20,4,21,4,1,4,22,4,23,4,24,4,25,4,26,4,27,4,28,4,29,4,30,4,31,4,32,4,33,4,34,4,35,4,36,4,37,4,38,4,39,4,40,4,41,4,42,4,43,4,44,4,45,4,46,4,47,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,4,49,4,50,4,51,4,52,4,53,4,81,4,54,4,55,4,56,4,57,4,58,4,59,4,60,4,61,4,62,4,63,4,64,4,65,4,66,4,67,4,68,4,69,4,70,4,71,4,72,4,73,4,74,4,75,4,76,4,77,4,78,4,79,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,37,2,37,12,37,16,37,24,37,20,37,28,37,44,37,36,37,52,37,60,37,1,37,3,37,15,37,19,37,27,37,23,37,35,37,51,37,43,37,59,37,75,37,32,37,47,37,40,37,55,37,63,37,29,37,48,37,37,37,56,37,66,37,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,156,78,22,85,3,90,63,150,192,84,27,97,40,99,246,89,34,144,117,132,28,131,80,122,170,96,225,99,37,110,237,101,102,132,166,130,245,155,147,104,39,87,161,101,113,98,155,91,208,89,123,134,244,152,98,125,190,125,142,155,22,98,159,124,183,136,137,91,181,94,9,99,151,102,72,104,199,149,141,151,79,103,229,78,10,79,77,79,157,79,73,80,242,86,55,89,212,89,1,90,9,92,223,96,15,97,112,97,19,102,5,105,186,112,79,117,112,117,251,121,173,125,239,125,195,128,14,132,99,136,2,139,85,144,122,144,59,83,149,78,165,78,223,87,178,128,193,144,239,120,0,78,241,88,162,110,56,144,50,122,40,131,139,130,47,156,65,81,112,83,189,84,225,84,224,86,251,89,21,95,242,152,235,109,228,128,45,133,98,150,112,150,160,150,251,151,11,84,243,83,135,91,207,112,189,127,194,143,232,150,111,83,92,157,186,122,17,78,147,120,252,129,38,110,24,86,4,85,29,107,26,133,59,156,229,89,169,83,102,109,220,116,143,149,66,86,145,78,75,144,242,150,79,131,12,153,225,83,182,85,48,91,113,95,32,102,243,102,4,104,56,108,243,108,41,109,91,116,200,118,78,122,52,152,241,130,91,136,96,138,237,146,178,109,171,117,202,118,197,153,166,96,1,139,138,141,178,149,142,105,173,83,134,81,18,87,48,88,68,89,180,91,246,94,40,96,169,99,244,99,191,108,20,111,142,112,20,113,89,113,213,113,63,115,1,126,118,130,209,130,151,133,96,144,91,146,27,157,105,88,188,101,90,108,37,117,249,81,46,89,101,89,128,95,220,95,188,98,250,101,42,106,39,107,180,107,139,115,193,127,86,137,44,157,14,157,196,158,161,92,150,108,123,131,4,81,75,92,182,97,198,129,118,104,97,114,89,78,250,79,120,83,105,96,41,110,79,122,243,151,11,78,22,83,238,78,85,79,61,79,161,79,115,79,160,82,239,83,9,86,15,89,193,90,182,91,225,91,209,121,135,102,156,103,182,103,76,107,179,108,107,112,194,115,141,121,190,121,60,122,135,123,177,130,219,130,4,131,119,131,239,131,211,131,102,135,178,138,41,86,168,140,230,143,78,144,30,151,138,134,196,79,232,92,17,98,89,114,59,117,229,129,189,130,254,134,192,140,197,150,19,153,213,153,203,78,26,79,227,137,222,86,74,88,202,88,251,94,235,95,42,96,148,96,98,96,208,97,18,98,208,98,57,101,65,155,102,102,176,104,119,109,112,112,76,117,134,118,117,125,165,130,249,135,139,149,142,150,157,140,241,81,190,82,22,89,179,84,179,91,22,93,104,97,130,105,175,109,141,120,203,132,87,136,114,138,167,147,184,154,108,109,168,153,217,134,163,87,255,103,206,134,14,146,131,82,135,86,4,84,211,94,225,98,185,100,60,104,56,104,187,107,114,115,186,120,107,122,154,137,210,137,107,141,3,143,237,144,163,149,148,150,105,151,102,91,179,92,125,105,77,152,78,152,155,99,32,123,43,106,127,106,182,104,13,156,95,111,114,82,157,85,112,96,236,98,59,109,7,110,209,110,91,132,16,137,68,143,20,78,57,156,246,83,27,105,58,106,132,151,42,104,92,81,195,122,178,132,220,145,140,147,91,86,40,157,34,104,5,131,49,132,165,124,8,82,197,130,230,116,126,78,131,79,160,81,210,91,10,82,216,82,231,82,251,93,154,85,42,88,230,89,140,91,152,91,219,91,114,94,121,94,163,96,31,97,99,97,190,97,219,99,98,101,209,103,83,104,250,104,62,107,83,107,87,108,34,111,151,111,69,111,176,116,24,117,227,118,11,119,255,122,161,123,33,124,233,125,54,127,240,127,157,128,102,130,158,131,179,137,204,138,171,140,132,144,81,148,147,149,145,149,162,149,101,150,211,151,40,153,24,130,56,78,43,84,184,92,204,93,169,115,76,118,60,119,169,92,235,127,11,141,193,150,17,152,84,152,88,152,1,79,14,79,113,83,156,85,104,86,250,87,71,89,9,91,196,91,144,92,12,94,126,94,204,95,238,99,58,103,215,101,226,101,31,103,203,104,196,104,95,106,48,94,197,107,23,108,125,108,127,117,72,121,99,91,0,122,0,125,189,95,143,137,24,138,180,140,119,141,204,142,29,143,226,152,14,154,60,155,128,78,125,80,0,81,147,89,156,91,47,98,128,98,236,100,58,107,160,114,145,117,71,121,169,127,251,135,188,138,112,139,172,99,202,131,160,151,9,84,3,84,171,85,84,104,88,106,112,138,39,120,117,103,205,158,116,83,162,91,26,129,80,134,6,144,24,78,69,78,199,78,17,79,202,83,56,84,174,91,19,95,37,96,81,101,61,103,66,108,114,108,227,108,120,112,3,116,118,122,174,122,8,123,26,125,254,124,102,125,231,101,91,114,187,83,69,92,232,93,210,98,224,98,25,99,32,110,90,134,49,138,221,141,248,146,1,111,166,121,90,155,168,78,171,78,172,78,155,79,160,79,209,80,71,81,246,122,113,81,246,81,84,83,33,83,127,83,235,83,172,85,131,88,225,92,55,95,74,95,47,96,80,96,109,96,31,99,89,101,75,106,193,108,194,114,237,114,239,119,248,128,5,129,8,130,78,133,247,144,225,147,255,151,87,153,90,154,240,78,221,81,45,92,129,102,109,105,64,92,242,102,117,105,137,115,80,104,129,124,197,80,228,82,71,87,254,93,38,147,164,101,35,107,61,107,52,116,129,121,189,121,75,123,202,125,185,130,204,131,127,136,95,137,57,139,209,143,209,145,31,84,128,146,93,78,54,80,229,83,58,83,215,114,150,115,233,119,230,130,175,142,198,153,200,153,210,153,119,81,26,97,94,134,176,85,122,122,118,80,211,91,71,144,133,150,50,78,219,106,231,145,81,92,72,92,152,99,159,122,147,108,116,151,97,143,170,122,138,113,136,150,130,124,23,104,112,126,81,104,108,147,242,82,27,84,171,133,19,138,164,127,205,142,225,144,102,83,136,136,65,121,194,79,190,80,17,82,68,81,83,85,45,87,234,115,139,87,81,89,98,95,132,95,117,96,118,97,103,97,169,97,178,99,58,100,108,101,111,102,66,104,19,110,102,117,61,122,251,124,76,125,153,125,75,126,107,127,14,131,74,131,205,134,8,138,99,138,102,139,253,142,26,152,143,157,184,130,206,143,232,155,135,82,31,98,131,100,192,111,153,150,65,104,145,80,32,107,122,108,84,111,116,122,80,125,64,136,35,138,8,103,246,78,57,80,38,80,101,80,124,81,56,82,99,82,167,85,15,87,5,88,204,90,250,94,178,97,248,97,243,98,114,99,28,105,41,106,125,114,172,114,46,115,20,120,111,120,121,125,12,119,169,128,139,137,25,139,226,140,210,142,99,144,117,147,122,150,85,152,19,154,120,158,67,81,159,83,179,83,123,94,38,95,27,110,144,110,132,115,254,115,67,125,55,130,0,138,250,138,80,150,78,78,11,80,228,83,124,84,250,86,209,89,100,91,241,93,171,94,39,95,56,98,69,101,175,103,86,110,208,114,202,124,180,136,161,128,225,128,240,131,78,134,135,138,232,141,55,146,199,150,103,152,19,159,148,78,146,78,13,79,72,83,73,84,62,84,47,90,140,95,161,95,159,96,167,104,142,106,90,116,129,120,158,138,164,138,119,139,144,145,94,78,201,155,164,78,124,79,175,79,25,80,22,80,73,81,108,81,159,82,185,82,254,82,154,83,227,83,17,84,14,84,137,85,81,87,162,87,125,89,84,91,93,91,143,91,229,93,231,93,247,93,120,94,131,94,154,94,183,94,24,95,82,96,76,97,151,98,216,98,167,99,59,101,2,102,67,102,244,102,109,103,33,104,151,104,203,105,95,108,42,109,105,109,47,110,157,110,50,117,135,118,108,120,63,122,224,124,5,125,24,125,94,125,177,125,21,128,3,128,175,128,177,128,84,129,143,129,42,130,82,131,76,136,97,136,27,139,162,140,252,140,202,144,117,145,113,146,63,120,252,146,164,149,77,150,5,152,153,153,216,154,59,157,91,82,171,82,247,83,8,84,213,88,247,98,224,111,106,140,95,143,185,158,75,81,59,82,74,84,253,86,64,122,119,145,96,157,210,158,68,115,9,111,112,129,17,117,253,95,218,96,168,154,219,114,188,143,100,107,3,152,202,78,240,86,100,87,190,88,90,90,104,96,199,97,15,102,6,102,57,104,177,104,247,109,213,117,58,125,110,130,66,155,155,78,80,79,201,83,6,85,111,93,230,93,238,93,251,103,153,108,115,116,2,120,80,138,150,147,223,136,80,87,167,94,43,99,181,80,172,80,141,81,0,103,201,84,94,88,187,89,176,91,105,95,77,98,161,99,61,104,115,107,8,110,125,112,199,145,128,114,21,120,38,120,109,121,142,101,48,125,220,131,193,136,9,143,155,150,100,82,40,87,80,103,106,127,161,140,180,81,66,87,42,150,58,88,138,105,180,128,178,84,14,93,252,87,149,120,250,157,92,79,74,82,139,84,62,100,40,102,20,103,245,103,132,122,86,123,34,125,47,147,92,104,173,155,57,123,25,83,138,81,55,82,223,91,246,98,174,100,230,100,45,103,186,107,169,133,209,150,144,118,214,155,76,99,6,147,171,155,191,118,82,102,9,78,152,80,194,83,113,92,232,96,146,100,99,101,95,104,230,113,202,115,35,117,151,123,130,126,149,134,131,139,219,140,120,145,16,153,172,101,171,102,139,107,213,78,212,78,58,79,127,79,58,82,248,83,242,83,227,85,219,86,235,88,203,89,201,89,255,89,80,91,77,92,2,94,43,94,215,95,29,96,7,99,47,101,92,91,175,101,189,101,232,101,157,103,98,107,123,107,15,108,69,115,73,121,193,121,248,124,25,125,43,125,162,128,2,129,243,129,150,137,94,138,105,138,102,138,140,138,238,138,199,140,220,140,204,150,252,152,111,107,139,78,60,79,141,79,80,81,87,91,250,91,72,97,1,99,66,102,33,107,203,110,187,108,62,114,189,116,212,117,193,120,58,121,12,128,51,128,234,129,148,132,158,143,80,108,127,158,15,95,88,139,43,157,250,122,248,142,141,91,235,150,3,78,241,83,247,87,49,89,201,90,164,91,137,96,127,110,6,111,190,117,234,140,159,91,0,133,224,123,114,80,244,103,157,130,97,92,74,133,30,126,14,130,153,81,4,92,104,99,102,141,156,101,110,113,62,121,23,125,5,128,29,139,202,142,110,144,199,134,170,144,31,80,250,82,58,92,83,103,124,112,53,114,76,145,200,145,43,147,229,130,194,91,49,95,249,96,59,78,214,83,136,91,75,98,49,103,138,107,233,114,224,115,46,122,107,129,163,141,82,145,150,153,18,81,215,83,106,84,255,91,136,99,57,106,172,125,0,151,218,86,206,83,104,84,151,91,49,92,222,93,238,79,1,97,254,98,50,109,192,121,203,121,66,125,77,126,210,127,237,129,31,130,144,132,70,136,114,137,144,139,116,142,47,143,49,144,75,145,108,145,198,150,156,145,192,78,79,79,69,81,65,83,147,95,14,98,212,103,65,108,11,110,99,115,38,126,205,145,131,146,212,83,25,89,191,91,209,109,93,121,46,126,155,124,126,88,159,113,250,81,83,136,240,143,202,79,251,92,37,102,172,119,227,122,28,130,255,153,198,81,170,95,236,101,111,105,137,107,243,109,150,110,100,111,254,118,20,125,225,93,117,144,135,145,6,152,230,81,29,82,64,98,145,102,217,102,26,110,182,94,210,125,114,127,248,102,175,133,247,133,248,138,169,82,217,83,115,89,143,94,144,95,85,96,228,146,100,150,183,80,31,81,221,82,32,83,71,83,236,83,232,84,70,85,49,85,23,86,104,89,190,89,60,90,181,91,6,92,15,92,17,92,26,92,132,94,138,94,224,94,112,95,127,98,132,98,219,98,140,99,119,99,7,102,12,102,45,102,118,102,126,103,162,104,31,106,53,106,188,108,136,109,9,110,88,110,60,113,38,113,103,113,199,117,1,119,93,120,1,121,101,121,240,121,224,122,17,123,167,124,57,125,150,128,214,131,139,132,73,133,93,136,243,136,31,138,60,138,84,138,115,138,97,140,222,140,164,145,102,146,126,147,24,148,156,150,152,151,10,78,8,78,30,78,87,78,151,81,112,82,206,87,52,88,204,88,34,91,56,94,197,96,254,100,97,103,86,103,68,109,182,114,115,117,99,122,184,132,114,139,184,145,32,147,49,86,244,87,254,152,237,98,13,105,150,107,237,113,84,126,119,128,114,130,230,137,223,152,85,135,177,143,59,92,56,79,225,79,181,79,7,85,32,90,221,91,233,91,195,95,78,97,47,99,176,101,75,102,238,104,155,105,120,109,241,109,51,117,185,117,31,119,94,121,230,121,51,125,227,129,175,130,170,133,170,137,58,138,171,142,155,143,50,144,221,145,7,151,186,78,193,78,3,82,117,88,236,88,11,92,26,117,61,92,78,129,10,138,197,143,99,150,109,151,37,123,207,138,8,152,98,145,243,86,168,83,23,144,57,84,130,87,37,94,168,99,52,108,138,112,97,119,139,124,224,127,112,136,66,144,84,145,16,147,24,147,143,150,94,116,196,154,7,93,105,93,112,101,162,103,168,141,219,150,110,99,73,103,25,105,197,131,23,152,192,150,254,136,132,111,122,100,248,91,22,78,44,112,93,117,47,102,196,81,54,82,226,82,211,89,129,95,39,96,16,98,63,101,116,101,31,102,116,102,242,104,22,104,99,107,5,110,114,114,31,117,219,118,190,124,86,128,240,88,253,136,127,137,160,138,147,138,203,138,29,144,146,145,82,151,89,151,137,101,14,122,6,129,187,150,45,94,220,96,26,98,165,101,20,102,144,103,243,119,77,122,77,124,62,126,10,129,172,140,100,141,225,141,95,142,169,120,7,82,217,98,165,99,66,100,152,98,45,138,131,122,192,123,172,138,234,150,118,125,12,130,73,135,217,78,72,81,67,83,96,83,163,91,2,92,22,92,221,93,38,98,71,98,176,100,19,104,52,104,201,108,69,109,23,109,211,103,92,111,78,113,125,113,203,101,127,122,173,123,218,125,74,126,168,127,122,129,27,130,57,130,166,133,110,138,206,140,245,141,120,144,119,144,173,146,145,146,131,149,174,155,77,82,132,85,56,111,54,113,104,81,133,121,85,126,179,129,206,124,76,86,81,88,168,92,170,99,254,102,253,102,90,105,217,114,143,117,142,117,14,121,86,121,223,121,151,124,32,125,68,125,7,134,52,138,59,150,97,144,32,159,231,80,117,82,204,83,226,83,9,80,170,85,238,88,79,89,61,114,139,91,100,92,29,83,227,96,243,96,92,99,131,99,63,99,187,99,205,100,233,101,249,102,227,93,205,105,253,105,21,111,229,113,137,78,233,117,248,118,147,122,223,124,207,125,156,125,97,128,73,131,88,131,108,132,188,132,251,133,197,136,112,141,1,144,109,144,151,147,28,151,18,154,207,80,151,88,142,97,211,129,53,133,8,141,32,144,195,79,116,80,71,82,115,83,111,96,73,99,95,103,44,110,179,141,31,144,215,79,94,92,202,140,207,101,154,125,82,83,150,136,118,81,195,99,88,91,107,91,10,92,13,100,81,103,92,144,214,78,26,89,42,89,112,108,81,138,62,85,21,88,165,89,240,96,83,98,193,103,53,130,85,105,64,150,196,153,40,154,83,79,6,88,254,91,16,128,177,92,47,94,133,95,32,96,75,97,52,98,255,102,240,108,222,110,206,128,127,129,212,130,139,136,184,140,0,144,46,144,138,150,219,158,219,155,227,78,240,83,39,89,44,123,141,145,76,152,249,157,221,110,39,112,83,83,68,85,133,91,88,98,158,98,211,98,162,108,239,111,34,116,23,138,56,148,193,111,254,138,56,131,231,81,248,134,234,83,233,83,70,79,84,144,176,143,106,89,49,129,253,93,234,122,191,143,218,104,55,140,248,114,72,156,61,106,176,138,57,78,88,83,6,86,102,87,197,98,162,99,230,101,78,107,225,109,91,110,173,112,237,119,239,122,170,123,187,125,61,128,198,128,203,134,149,138,91,147,227,86,199,88,62,95,173,101,150,102,128,106,181,107,55,117,199,138,36,80,229,119,48,87,27,95,101,96,122,102,96,108,244,117,26,122,110,127,244,129,24,135,69,144,179,153,201,123,92,117,249,122,81,123,196,132,16,144,233,121,146,122,54,131,225,90,64,119,45,78,242,78,153,91,224,95,189,98,60,102,241,103,232,108,107,134,119,136,59,138,78,145,243,146,208,153,23,106,38,112,42,115,231,130,87,132,175,140,1,78,70,81,203,81,139,85,245,91,22,94,51,94,129,94,20,95,53,95,107,95,180,95,242,97,17,99,162,102,29,103,110,111,82,114,58,117,58,119,116,128,57,129,120,129,118,135,191,138,220,138,133,141,243,141,154,146,119,149,2,152,229,156,197,82,87,99,244,118,21,103,136,108,205,115,195,140,174,147,115,150,37,109,156,88,14,105,204,105,253,143,154,147,219,117,26,144,90,88,2,104,180,99,251,105,67,79,44,111,216,103,187,143,38,133,180,125,84,147,63,105,112,111,106,87,247,88,44,91,44,125,42,114,10,84,227,145,180,157,173,78,78,79,92,80,117,80,67,82,158,140,72,84,36,88,154,91,29,94,149,94,173,94,247,94,31,95,140,96,181,98,58,99,208,99,175,104,64,108,135,120,142,121,11,122,224,125,71,130,2,138,230,138,68,142,19,144,184,144,45,145,216,145,14,159,229,108,88,100,226,100,117,101,244,110,132,118,27,123,105,144,209,147,186,110,242,84,185,95,164,100,77,143,237,143,68,146,120,81,107,88,41,89,85,92,151,94,251,109,143,126,28,117,188,140,226,142,91,152,185,112,29,79,191,107,177,111,48,117,251,150,78,81,16,84,53,88,87,88,172,89,96,92,146,95,151,101,92,103,33,110,123,118,223,131,237,140,20,144,253,144,77,147,37,120,58,120,170,82,166,94,31,87,116,89,18,96,18,80,90,81,172,81,205,81,0,82,16,85,84,88,88,88,87,89,149,91,246,92,139,93,188,96,149,98,45,100,113,103])
.concat([67,104,188,104,223,104,215,118,216,109,111,110,155,109,111,112,200,113,83,95,216,117,119,121,73,123,84,123,82,123,214,124,113,125,48,82,99,132,105,133,228,133,14,138,4,139,70,140,15,142,3,144,15,144,25,148,118,150,45,152,48,154,216,149,205,80,213,82,12,84,2,88,14,92,167,97,158,100,30,109,179,119,229,122,244,128,4,132,83,144,133,146,224,92,7,157,63,83,151,95,179,95,156,109,121,114,99,119,191,121,228,123,210,107,236,114,173,138,3,104,97,106,248,81,129,122,52,105,74,92,246,156,235,130,197,91,73,145,30,112,120,86,111,92,199,96,102,101,140,108,90,140,65,144,19,152,81,84,199,102,13,146,72,89,163,144,133,81,77,78,234,81,153,133,14,139,88,112,122,99,75,147,98,105,180,153,4,126,119,117,87,83,96,105,223,142,227,150,93,108,140,78,60,92,16,95,233,143,2,83,209,140,137,128,121,134,255,94,229,101,115,78,101,81,130,89,63,92,238,151,251,78,138,89,205,95,141,138,225,111,176,121,98,121,231,91,113,132,43,115,177,113,116,94,245,95,123,99,154,100,195,113,152,124,67,78,252,94,75,78,220,87,162,86,169,96,195,111,13,125,253,128,51,129,191,129,178,143,151,137,164,134,244,93,138,98,173,100,135,137,119,103,226,108,62,109,54,116,52,120,70,90,117,127,173,130,172,153,243,79,195,94,221,98,146,99,87,101,111,103,195,118,76,114,204,128,186,128,41,143,77,145,13,80,249,87,146,90,133,104,115,105,100,113,253,114,183,140,242,88,224,140,106,150,25,144,127,135,228,121,231,119,41,132,47,79,101,82,90,83,205,98,207,103,202,108,125,118,148,123,149,124,54,130,132,133,235,143,221,102,32,111,6,114,27,126,171,131,193,153,166,158,253,81,177,123,114,120,184,123,135,128,72,123,232,106,97,94,140,128,81,117,96,117,107,81,98,146,140,110,122,118,151,145,234,154,16,79,112,127,156,98,79,123,165,149,233,156,122,86,89,88,228,134,188,150,52,79,36,82,74,83,205,83,219,83,6,94,44,100,145,101,127,103,62,108,78,108,72,114,175,114,237,115,84,117,65,126,44,130,233,133,169,140,196,123,198,145,105,113,18,152,239,152,61,99,105,102,106,117,228,118,208,120,67,133,238,134,42,83,81,83,38,84,131,89,135,94,124,95,178,96,73,98,121,98,171,98,144,101,212,107,204,108,178,117,174,118,145,120,216,121,203,125,119,127,165,128,171,136,185,138,187,140,127,144,94,151,219,152,11,106,56,124,153,80,62,92,174,95,135,103,216,107,53,116,9,119,142,127,59,159,202,103,23,122,57,83,139,117,237,154,102,95,157,129,241,131,152,128,60,95,197,95,98,117,70,123,60,144,103,104,235,89,155,90,16,125,126,118,44,139,245,79,106,95,25,106,55,108,2,111,226,116,104,121,104,136,85,138,121,140,223,94,207,99,197,117,210,121,215,130,40,147,242,146,156,132,237,134,45,156,193,84,108,95,140,101,92,109,21,112,167,140,211,140,59,152,79,101,246,116,13,78,216,78,224,87,43,89,102,90,204,91,168,81,3,94,156,94,22,96,118,98,119,101,167,101,110,102,110,109,54,114,38,123,80,129,154,129,153,130,92,139,160,140,230,140,116,141,28,150,68,150,174,79,171,100,102,107,30,130,97,132,106,133,232,144,1,92,83,105,168,152,122,132,87,133,15,79,111,82,169,95,69,94,13,103,143,121,121,129,7,137,134,137,245,109,23,95,85,98,184,108,207,78,105,114,146,155,6,82,59,84,116,86,179,88,164,97,110,98,26,113,110,89,137,124,222,124,27,125,240,150,135,101,94,128,25,78,117,79,117,81,64,88,99,94,115,94,10,95,196,103,38,78,61,133,137,149,91,150,115,124,1,152,251,80,193,88,86,118,167,120,37,82,165,119,17,133,134,123,79,80,9,89,71,114,199,123,232,125,186,143,212,143,77,144,191,79,201,82,41,90,1,95,173,151,221,79,23,130,234,146,3,87,85,99,105,107,43,117,220,136,20,143,66,122,223,82,147,88,85,97,10,98,174,102,205,107,63,124,233,131,35,80,248,79,5,83,70,84,49,88,73,89,157,91,240,92,239,92,41,93,150,94,177,98,103,99,62,101,185,101,11,103,213,108,225,108,249,112,50,120,43,126,222,128,179,130,12,132,236,132,2,135,18,137,42,138,74,140,166,144,210,146,253,152,243,156,108,157,79,78,161,78,141,80,86,82,74,87,168,89,61,94,216,95,217,95,63,98,180,102,27,103,208,103,210,104,146,81,33,125,170,128,168,129,0,139,140,140,191,140,126,146,50,150,32,84,44,152,23,83,213,80,92,83,168,88,178,100,52,103,103,114,102,119,70,122,230,145,195,82,161,108,134,107,0,88,76,94,84,89,44,103,251,127,225,81,198,118,105,100,232,120,84,155,187,158,203,87,185,89,39,102,154,103,206,107,233,84,217,105,85,94,156,129,149,103,170,155,254,103,82,156,93,104,166,78,227,79,200,83,185,98,43,103,171,108,196,143,173,79,109,126,191,158,7,78,98,97,128,110,43,111,19,133,115,84,42,103,69,155,243,93,149,123,172,92,198,91,28,135,74,110,209,132,20,122,8,129,153,89,141,124,17,108,32,119,217,82,34,89,33,113,95,114,219,119,39,151,97,157,11,105,127,90,24,90,165,81,13,84,125,84,14,102,223,118,247,143,152,146,244,156,234,89,93,114,197,110,77,81,201,104,191,125,236,125,98,151,186,158,120,100,33,106,2,131,132,89,95,91,219,107,27,115,242,118,178,125,23,128,153,132,50,81,40,103,217,158,238,118,98,103,255,82,5,153,36,92,59,98,126,124,176,140,79,85,182,96,11,125,128,149,1,83,95,78,182,81,28,89,58,114,54,128,206,145,37,95,226,119,132,83,121,95,4,125,172,133,51,138,141,142,86,151,243,103,174,133,83,148,9,97,8,97,185,108,82,118,237,138,56,143,47,85,81,79,42,81,199,82,203,83,165,91,125,94,160,96,130,97,214,99,9,103,218,103,103,110,140,109,54,115,55,115,49,117,80,121,213,136,152,138,74,144,145,144,245,144,196,150,141,135,21,89,136,78,89,79,14,78,137,138,63,143,16,152,173,80,124,94,150,89,185,91,184,94,218,99,250,99,193,100,220,102,74,105,216,105,11,109,182,110,148,113,40,117,175,122,138,127,0,128,73,132,201,132,129,137,33,139,10,142,101,144,125,150,10,153,126,97,145,98,50,107,131,108,116,109,204,127,252,127,192,109,133,127,186,135,248,136,101,103,177,131,60,152,247,150,27,109,97,125,61,132,106,145,113,78,117,83,80,93,4,107,235,111,205,133,45,134,167,137,41,82,15,84,101,92,78,103,168,104,6,116,131,116,226,117,207,136,225,136,204,145,226,150,120,150,139,95,135,115,203,122,78,132,160,99,101,117,137,82,65,109,156,110,9,116,89,117,107,120,146,124,134,150,220,122,141,159,182,79,110,97,197,101,92,134,134,78,174,78,218,80,33,78,204,81,238,91,153,101,129,104,188,109,31,115,66,118,173,119,28,122,231,124,111,130,210,138,124,144,207,145,117,150,24,152,155,82,209,125,43,80,152,83,151,103,203,109,208,113,51,116,232,129,42,143,163,150,87,156,159,158,96,116,65,88,153,109,47,125,94,152,228,78,54,79,139,79,183,81,177,82,186,93,28,96,178,115,60,121,211,130,52,146,183,150,246,150,10,151,151,158,98,159,166,102,116,107,23,82,163,82,200,112,194,136,201,94,75,96,144,97,35,111,73,113,62,124,244,125,111,128,238,132,35,144,44,147,66,84,111,155,211,106,137,112,194,140,239,141,50,151,180,82,65,90,202,94,4,95,23,103,124,105,148,105,106,109,15,111,98,114,252,114,237,123,1,128,126,128,75,135,206,144,109,81,147,158,132,121,139,128,50,147,214,138,45,80,140,84,113,138,106,107,196,140,7,129,209,96,160,103,242,157,153,78,152,78,16,156,107,138,193,133,104,133,0,105,126,110,151,120,85,129,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,95,16,78,21,78,42,78,49,78,54,78,60,78,63,78,66,78,86,78,88,78,130,78,133,78,107,140,138,78,18,130,13,95,142,78,158,78,159,78,160,78,162,78,176,78,179,78,182,78,206,78,205,78,196,78,198,78,194,78,215,78,222,78,237,78,223,78,247,78,9,79,90,79,48,79,91,79,93,79,87,79,71,79,118,79,136,79,143,79,152,79,123,79,105,79,112,79,145,79,111,79,134,79,150,79,24,81,212,79,223,79,206,79,216,79,219,79,209,79,218,79,208,79,228,79,229,79,26,80,40,80,20,80,42,80,37,80,5,80,28,79,246,79,33,80,41,80,44,80,254,79,239,79,17,80,6,80,67,80,71,80,3,103,85,80,80,80,72,80,90,80,86,80,108,80,120,80,128,80,154,80,133,80,180,80,178,80,201,80,202,80,179,80,194,80,214,80,222,80,229,80,237,80,227,80,238,80,249,80,245,80,9,81,1,81,2,81,22,81,21,81,20,81,26,81,33,81,58,81,55,81,60,81,59,81,63,81,64,81,82,81,76,81,84,81,98,81,248,122,105,81,106,81,110,81,128,81,130,81,216,86,140,81,137,81,143,81,145,81,147,81,149,81,150,81,164,81,166,81,162,81,169,81,170,81,171,81,179,81,177,81,178,81,176,81,181,81,189,81,197,81,201,81,219,81,224,81,85,134,233,81,237,81,240,81,245,81,254,81,4,82,11,82,20,82,14,82,39,82,42,82,46,82,51,82,57,82,79,82,68,82,75,82,76,82,94,82,84,82,106,82,116,82,105,82,115,82,127,82,125,82,141,82,148,82,146,82,113,82,136,82,145,82,168,143,167,143,172,82,173,82,188,82,181,82,193,82,205,82,215,82,222,82,227,82,230,82,237,152,224,82,243,82,245,82,248,82,249,82,6,83,8,83,56,117,13,83,16,83,15,83,21,83,26,83,35,83,47,83,49,83,51,83,56,83,64,83,70,83,69,83,23,78,73,83,77,83,214,81,94,83,105,83,110,83,24,89,123,83,119,83,130,83,150,83,160,83,166,83,165,83,174,83,176,83,182,83,195,83,18,124,217,150,223,83,252,102,238,113,238,83,232,83,237,83,250,83,1,84,61,84,64,84,44,84,45,84,60,84,46,84,54,84,41,84,29,84,78,84,143,84,117,84,142,84,95,84,113,84,119,84,112,84,146,84,123,84,128,84,118,84,132,84,144,84,134,84,199,84,162,84,184,84,165,84,172,84,196,84,200,84,168,84,171,84,194,84,164,84,190,84,188,84,216,84,229,84,230,84,15,85,20,85,253,84,238,84,237,84,250,84,226,84,57,85,64,85,99,85,76,85,46,85,92,85,69,85,86,85,87,85,56,85,51,85,93,85,153,85,128,85,175,84,138,85,159,85,123,85,126,85,152,85,158,85,174,85,124,85,131,85,169,85,135,85,168,85,218,85,197,85,223,85,196,85,220,85,228,85,212,85,20,86,247,85,22,86,254,85,253,85,27,86,249,85,78,86,80,86,223,113,52,86,54,86,50,86,56,86,107,86,100,86,47,86,108,86,106,86,134,86,128,86,138,86,160,86,148,86,143,86,165,86,174,86,182,86,180,86,194,86,188,86,193,86,195,86,192,86,200,86,206,86,209,86,211,86,215,86,238,86,249,86,0,87,255,86,4,87,9,87,8,87,11,87,13,87,19,87,24,87,22,87,199,85,28,87,38,87,55,87,56,87,78,87,59,87,64,87,79,87,105,87,192,87,136,87,97,87,127,87,137,87,147,87,160,87,179,87,164,87,170,87,176,87,195,87,198,87,212,87,210,87,211,87,10,88,214,87,227,87,11,88,25,88,29,88,114,88,33,88,98,88,75,88,112,88,192,107,82,88,61,88,121,88,133,88,185,88,159,88,171,88,186,88,222,88,187,88,184,88,174,88,197,88,211,88,209,88,215,88,217,88,216,88,229,88,220,88,228,88,223,88,239,88,250,88,249,88,251,88,252,88,253,88,2,89,10,89,16,89,27,89,166,104,37,89,44,89,45,89,50,89,56,89,62,89,210,122,85,89,80,89,78,89,90,89,88,89,98,89,96,89,103,89,108,89,105,89,120,89,129,89,157,89,94,79,171,79,163,89,178,89,198,89,232,89,220,89,141,89,217,89,218,89,37,90,31,90,17,90,28,90,9,90,26,90,64,90,108,90,73,90,53,90,54,90,98,90,106,90,154,90,188,90,190,90,203,90,194,90,189,90,227,90,215,90,230,90,233,90,214,90,250,90,251,90,12,91,11,91,22,91,50,91,208,90,42,91,54,91,62,91,67,91,69,91,64,91,81,91,85,91,90,91,91,91,101,91,105,91,112,91,115,91,117,91,120,91,136,101,122,91,128,91,131,91,166,91,184,91,195,91,199,91,201,91,212,91,208,91,228,91,230,91,226,91,222,91,229,91,235,91,240,91,246,91,243,91,5,92,7,92,8,92,13,92,19,92,32,92,34,92,40,92,56,92,57,92,65,92,70,92,78,92,83,92,80,92,79,92,113,91,108,92,110,92,98,78,118,92,121,92,140,92,145,92,148,92,155,89,171,92,187,92,182,92,188,92,183,92,197,92,190,92,199,92,217,92,233,92,253,92,250,92,237,92,140,93,234,92,11,93,21,93,23,93,92,93,31,93,27,93,17,93,20,93,34,93,26,93,25,93,24,93,76,93,82,93,78,93,75,93,108,93,115,93,118,93,135,93,132,93,130,93,162,93,157,93,172,93,174,93,189,93,144,93,183,93,188,93,201,93,205,93,211,93,210,93,214,93,219,93,235,93,242,93,245,93,11,94,26,94,25,94,17,94,27,94,54,94,55,94,68,94,67,94,64,94,78,94,87,94,84,94,95,94,98,94,100,94,71,94,117,94,118,94,122,94,188,158,127,94,160,94,193,94,194,94,200,94,208,94,207,94,214,94,227,94,221,94,218,94,219,94,226,94,225,94,232,94,233,94,236,94,241,94,243,94,240,94,244,94,248,94,254,94,3,95,9,95,93,95,92,95,11,95,17,95,22,95,41,95,45,95,56,95,65,95,72,95,76,95,78,95,47,95,81,95,86,95,87,95,89,95,97,95,109,95,115,95,119,95,131,95,130,95,127,95,138,95,136,95,145,95,135,95,158,95,153,95,152,95,160,95,168,95,173,95,188,95,214,95,251,95,228,95,248,95,241,95,221,95,179,96,255,95,33,96,96,96,25,96,16,96,41,96,14,96,49,96,27,96,21,96,43,96,38,96,15,96,58,96,90,96,65,96,106,96,119,96,95,96,74,96,70,96,77,96,99,96,67,96,100,96,66,96,108,96,107,96,89,96,129,96,141,96,231,96,131,96,154,96,132,96,155,96,150,96,151,96,146,96,167,96,139,96,225,96,184,96,224,96,211,96,180,96,240,95,189,96,198,96,181,96,216,96,77,97,21,97,6,97,246,96,247,96,0,97,244,96,250,96,3,97,33,97,251,96,241,96,13,97,14,97,71,97,62,97,40,97,39,97,74,97,63,97,60,97,44,97,52,97,61,97,66,97,68,97,115,97,119,97,88,97,89,97,90,97,107,97,116,97,111,97,101,97,113,97,95,97,93,97,83,97,117,97,153,97,150,97,135,97,172,97,148,97,154,97,138,97,145,97,171,97,174,97,204,97,202,97,201,97,247,97,200,97,195,97,198,97,186,97,203,97,121,127,205,97,230,97,227,97,246,97,250,97,244,97,255,97,253,97,252,97,254,97,0,98,8,98,9,98,13,98,12,98,20,98,27,98,30,98,33,98,42,98,46,98,48,98,50,98,51,98,65,98,78,98,94,98,99,98,91,98,96,98,104,98,124,98,130,98,137,98,126,98,146,98,147,98,150,98,212,98,131,98,148,98,215,98,209,98,187,98,207,98,255,98,198,98,212,100,200,98,220,98,204,98,202,98,194,98,199,98,155,98,201,98,12,99,238,98,241,98,39,99,2,99,8,99,239,98,245,98,80,99,62,99,77,99,28,100,79,99,150,99,142,99,128,99,171,99,118,99,163,99,143,99,137,99,159,99,181,99,107,99,105,99,190,99,233,99,192,99,198,99,227,99,201,99,210,99,246,99,196,99,22,100,52,100,6,100,19,100,38,100,54,100,29,101,23,100,40,100,15,100,103,100,111,100,118,100,78,100,42,101,149,100,147,100,165,100,169,100,136,100,188,100,218,100,210,100,197,100,199,100,187,100,216,100,194,100,241,100,231,100,9,130,224,100,225,100,172,98,227,100,239,100,44,101,246,100,244,100,242,100,250,100,0,101,253,100,24,101,28,101,5,101,36,101,35,101,43,101,52,101,53,101,55,101,54,101,56,101,75,117,72,101,86,101,85,101,77,101,88,101,94,101,93,101,114,101,120,101,130,101,131,101,138,139,155,101,159,101,171,101,183,101,195,101,198,101,193,101,196,101,204,101,210,101,219,101,217,101,224,101,225,101,241,101,114,103,10,102,3,102,251,101,115,103,53,102,54,102,52,102,28,102,79,102,68,102,73,102,65,102,94,102,93,102,100,102,103,102,104,102,95,102,98,102,112,102,131,102,136,102,142,102,137,102,132,102,152,102,157,102,193,102,185,102,201,102,190,102,188,102,196,102,184,102,214,102,218,102,224,102,63,102,230,102,233,102,240,102,245,102,247,102,15,103,22,103,30,103,38,103,39,103,56,151,46,103,63,103,54,103,65,103,56,103,55,103,70,103,94,103,96,103,89,103,99,103,100,103,137,103,112,103,169,103,124,103,106,103,140,103,139,103,166,103,161,103,133,103,183,103,239,103,180,103,236,103,179,103,233,103,184,103,228,103,222,103,221,103,226,103,238,103,185,103,206,103,198,103,231,103,156,106,30,104,70,104,41,104,64,104,77,104,50,104,78,104,179,104,43,104,89,104,99,104,119,104,127,104,159,104,143,104,173,104,148,104,157,104,155,104,131,104,174,106,185,104,116,104,181,104,160,104,186,104,15,105,141,104,126,104,1,105,202,104,8,105,216,104,34,105,38,105,225,104,12,105,205,104,212,104,231,104,213,104,54,105,18,105,4,105,215,104,227,104,37,105,249,104,224,104,239,104,40,105,42,105,26,105,35,105,33,105,198,104,121,105,119,105,92,105,120,105,107,105,84,105,126,105,110,105,57,105,116,105,61,105,89,105,48,105,97,105,94,105,93,105,129,105,106,105,178,105,174,105,208,105,191,105,193,105,211,105,190,105,206,105,232,91,202,105,221,105,187,105,195,105,167,105,46,106,145,105,160,105,156,105,149,105,180,105,222,105,232,105,2,106,27,106,255,105,10,107,249,105,242,105,231,105,5,106,177,105,30,106,237,105,20,106,235,105,10,106,18,106,193,106,35,106,19,106,68,106,12,106,114,106,54,106,120,106,71,106,98,106,89,106,102,106,72,106,56,106,34,106,144,106,141,106,160,106,132,106,162,106,163,106,151,106,23,134,187,106,195,106,194,106,184,106,179,106,172,106,222,106,209,106,223,106,170,106,218,106,234,106,251,106,5,107,22,134,250,106,18,107,22,107,49,155,31,107,56,107,55,107,220,118,57,107,238,152,71,107,67,107,73,107,80,107,89,107,84,107,91,107,95,107,97,107,120,107,121,107,127,107,128,107,132,107,131,107,141,107,152,107,149,107,158,107,164,107,170,107,171,107,175,107,178,107,177,107,179,107,183,107,188,107,198,107,203,107,211,107,223,107,236,107,235,107,243,107,239,107,190,158,8,108,19,108,20,108,27,108,36,108,35,108,94,108,85,108,98,108,106,108,130,108,141,108,154,108,129,108,155,108,126,108,104,108,115,108,146,108,144,108,196,108,241,108,211,108,189,108,215,108,197,108,221,108,174,108,177,108,190,108,186,108,219,108,239,108,217,108,234,108,31,109,77,136,54,109,43,109,61,109,56,109,25,109,53,109,51,109,18,109,12,109,99,109,147,109,100,109,90,109,121,109,89,109,142,109,149,109,228,111,133,109,249,109,21,110,10,110,181,109,199,109,230,109,184,109,198,109,236,109,222,109,204,109,232,109,210,109,197,109,250,109,217,109,228,109,213,109,234,109,238,109,45,110,110,110,46,110,25,110,114,110,95,110,62,110,35,110,107,110,43,110,118,110,77,110,31,110,67,110,58,110,78,110,36,110,255,110,29,110,56,110,130,110,170,110,152,110,201,110,183,110,211,110,189,110,175,110,196,110,178,110,212,110,213,110,143,110,165,110,194,110,159,110,65,111,17,111,76,112,236,110,248,110,254,110,63,111,242,110,49,111,239,110,50,111,204,110,62,111,19,111,247,110,134,111,122,111,120,111,129,111,128,111,111,111,91,111,243,111,109,111,130,111,124,111,88,111,142,111,145,111,194,111,102,111,179,111,163,111,161,111,164,111,185,111,198,111,170,111,223,111,213,111,236,111,212,111,216,111,241,111,238,111,219,111,9,112,11,112,250,111,17,112,1,112,15,112,254,111,27,112,26,112,116,111,29,112,24,112,31,112,48,112,62,112,50,112,81,112,99,112,153,112,146,112,175,112,241,112,172,112,184,112,179,112,174,112,223,112,203,112,221,112,217,112,9,113,253,112,28,113,25,113,101,113,85,113,136,113,102,113,98,113,76,113,86,113,108,113,143,113,251,113,132,113,149,113,168,113,172,113,215,113,185,113,190,113,210,113,201,113,212,113,206,113,224,113,236,113,231,113,245,113,252,113,249,113,255,113,13,114,16,114,27,114,40,114,45,114,44,114,48,114,50,114,59,114,60,114,63,114,64,114,70,114,75,114,88,114,116,114,126,114,130,114,129,114,135,114,146,114,150,114,162,114,167,114,185,114,178,114,195,114,198,114,196,114,206,114,210,114,226,114,224,114,225,114,249,114,247,114,15,80,23,115,10,115,28,115,22,115,29,115,52,115,47,115,41,115,37,115,62,115,78,115,79,115,216,158,87,115,106,115,104,115,112,115,120,115,117,115,123,115,122,115,200,115,179,115,206,115,187,115,192,115,229,115,238,115,222,115,162,116,5,116,111,116,37,116,248,115,50,116,58,116,85,116,63,116,95,116,89,116,65,116,92,116,105,116,112,116,99,116,106,116,118,116,126,116,139,116,158,116,167,116,202,116,207,116,212,116,241,115,224,116,227,116,231,116,233,116,238,116,242,116,240,116,241,116,248,116,247,116,4,117,3,117,5,117,12,117,14,117,13,117,21,117,19,117,30,117,38,117,44,117,60,117,68,117,77,117,74,117,73,117,91,117,70,117,90,117,105,117,100,117,103,117,107,117,109,117,120,117,118,117,134,117,135,117,116,117,138,117,137,117,130,117,148,117,154,117,157,117,165,117,163,117,194,117,179,117,195,117,181,117,189,117,184,117,188,117,177,117,205,117,202,117,210,117,217,117,227,117,222,117,254,117,255,117,252,117,1,118,240,117,250,117,242,117,243,117,11,118,13,118,9,118,31,118,39,118,32,118,33,118,34,118,36,118,52,118,48,118,59,118,71,118,72,118,70,118,92,118,88,118,97,118,98,118,104,118,105,118,106,118,103,118,108,118,112,118,114,118,118,118,120,118,124,118,128,118,131,118,136,118,139,118,142,118,150,118,147,118,153,118,154,118,176,118,180,118,184,118,185,118,186,118,194,118,205,118,214,118,210,118,222,118,225,118,229,118,231,118,234,118,47,134,251,118,8,119,7,119,4,119,41,119,36,119,30,119,37,119,38,119,27,119,55,119,56,119,71,119,90,119,104,119,107,119,91,119,101,119,127,119,126,119,121,119,142,119,139,119,145,119,160,119,158,119,176,119,182,119,185,119,191,119,188,119,189,119,187,119,199,119,205,119,215,119,218,119,220,119,227,119,238,119,252,119,12,120,18,120,38,121,32,120,42,121,69,120,142,120,116,120,134,120,124,120,154,120,140,120,163,120,181,120,170,120,175,120,209,120,198,120,203,120,212,120,190,120,188,120,197,120,202,120,236,120,231,120,218,120,253,120,244,120,7,121,18,121,17,121,25,121,44,121,43,121,64,121,96,121,87,121,95,121,90,121,85,121,83,121,122,121,127,121,138,121,157,121,167,121,75,159,170,121,174,121,179,121,185,121,186,121,201,121,213,121,231,121,236,121,225,121,227,121,8,122,13,122,24,122,25,122,32,122,31,122,128,121,49,122,59,122,62,122,55,122,67,122,87,122,73,122,97,122,98,122,105,122,157,159,112,122,121,122,125,122,136,122,151,122,149,122,152,122,150,122,169,122,200,122,176,122,182,122,197,122,196,122,191,122,131,144,199,122,202,122,205,122,207,122,213,122,211,122,217,122,218,122,221,122,225,122,226,122,230,122,237,122,240,122,2,123,15,123,10,123,6,123,51,123,24,123,25,123,30,123,53,123,40,123,54,123,80,123,122,123,4,123,77,123,11,123,76,123,69,123,117,123,101,123,116,123,103,123,112,123,113,123,108,123,110,123,157,123,152,123,159,123,141,123,156,123,154,123,139,123,146,123,143,123,93,123,153,123,203,123,193,123,204,123,207,123,180,123,198,123,221,123,233,123,17,124,20,124,230,123,229,123,96,124,0,124,7,124,19,124,243,123,247,123,23,124,13,124,246,123,35,124,39,124,42,124,31,124,55,124,43,124,61,124,76,124,67,124,84,124,79,124,64,124,80,124,88,124,95,124,100,124,86,124,101,124,108,124,117,124,131,124,144,124,164,124,173,124,162,124,171,124,161,124,168,124,179,124,178,124,177,124,174,124,185,124,189,124,192,124,197,124,194,124,216,124,210,124,220,124,226,124,59,155,239,124,242,124,244,124,246,124,250,124,6,125,2,125,28,125,21,125,10,125,69,125,75,125,46,125,50,125,63,125,53,125,70,125,115,125,86,125,78,125,114,125,104,125,110,125,79,125,99,125,147,125,137,125,91,125,143,125,125,125,155,125,186,125,174,125,163,125,181,125,199,125,189,125,171,125,61,126,162,125,175,125,220,125,184,125,159,125,176,125,216,125,221,125,228,125,222,125,251,125,242,125,225,125,5,126,10,126,35,126,33,126,18,126,49,126,31,126,9,126,11,126,34,126,70,126,102,126,59,126,53,126,57,126,67,126,55,126,50,126,58,126,103,126,93,126,86,126,94,126,89,126,90,126,121,126,106,126,105,126,124,126,123,126,131,126,213,125,125,126,174,143,127,126,136,126,137,126,140,126,146,126,144,126,147,126,148,126,150,126,142,126,155,126,156,126,56,127,58,127,69,127,76,127,77,127,78,127,80,127,81,127,85,127,84,127,88,127,95,127,96,127,104,127,105,127,103,127,120,127,130,127,134,127,131,127,136,127,135,127,140,127,148,127,158,127,157,127,154,127,163,127,175,127,178,127,185,127,174,127,182,127,184,127,113,139,197,127,198,127,202,127,213,127,212,127,225,127,230,127,233,127,243,127,249,127,220,152,6,128,4,128,11,128,18,128,24,128,25,128,28,128,33,128,40,128,63,128,59,128,74,128,70,128,82,128,88,128,90,128,95,128,98,128,104,128,115,128,114,128,112,128,118,128,121,128,125,128,127,128,132,128,134,128,133,128,155,128,147,128,154,128,173,128,144,81,172,128,219,128,229,128,217,128,221,128,196,128,218,128,214,128,9,129,239,128,241,128,27,129,41,129,35,129,47,129,75,129,139,150,70,129,62,129,83,129,81,129,252,128,113,129,110,129,101,129,102,129,116,129,131,129,136,129,138,129,128,129,130,129,160,129,149,129,164,129,163,129,95,129,147,129,169,129,176,129,181,129,190,129,184,129,189,129,192,129,194,129,186,129,201,129,205,129,209,129,217,129,216,129,200,129,218,129,223,129,224,129,231,129,250,129,251,129,254,129,1,130,2,130,5,130,7,130,10,130,13,130,16,130,22,130,41,130,43,130,56,130,51,130,64,130,89,130,88,130,93,130,90,130,95,130,100,130,98,130,104,130,106,130,107,130,46,130,113,130,119,130,120,130,126,130,141,130,146,130,171,130,159,130,187,130,172,130,225,130,227,130,223,130,210,130,244,130,243,130,250,130,147,131,3,131,251,130,249,130,222,130,6,131,220,130,9,131,217,130,53,131,52,131,22,131,50,131,49,131,64,131,57,131,80,131,69,131,47,131,43,131,23,131,24,131,133,131,154,131,170,131,159,131,162,131,150,131,35,131,142,131,135,131,138,131,124,131,181,131,115,131,117,131,160,131,137,131,168,131,244,131,19,132,235,131,206,131,253,131,3,132,216,131,11,132,193,131,247,131,7,132,224,131,242,131,13,132,34,132,32,132,189,131,56,132,6,133,251,131,109,132,42,132,60,132,90,133,132,132,119,132,107,132,173,132,110,132,130,132,105,132,70,132,44,132,111,132,121,132,53,132,202,132,98,132,185,132,191,132,159,132,217,132,205,132,187,132,218,132,208,132,193,132,198,132,214,132,161,132,33,133,255,132,244,132,23,133,24,133,44,133,31,133,21,133,20,133,252,132,64,133,99,133,88,133,72,133,65,133,2,134,75,133,85,133,128,133,164,133,136,133,145,133,138,133,168,133,109,133,148,133,155,133,234,133,135,133,156,133,119,133,126,133,144,133,201,133,186,133,207,133,185,133,208,133,213,133,221,133,229,133,220,133,249,133,10,134,19,134,11,134,254,133,250,133,6,134,34,134,26,134,48,134,63,134,77,134,85,78,84,134,95,134,103,134,113,134,147,134,163,134,169,134,170,134,139,134,140,134,182,134,175,134,196,134,198,134,176,134,201,134,35,136,171,134,212,134,222,134,233,134,236,134,223,134,219,134,239,134,18,135,6,135,8,135,0,135,3,135,251,134,17,135,9,135,13,135,249,134,10,135,52,135,63,135,55,135,59,135,37,135,41,135,26,135,96,135,95,135,120,135,76,135,78,135,116,135,87,135,104,135,110,135,89,135,83,135,99,135,106,135,5,136,162,135,159,135,130,135,175,135,203,135,189,135,192,135,208,135,214,150,171,135,196,135,179,135,199,135,198,135,187,135,239,135,242,135,224,135,15,136,13,136,254,135,246,135,247,135,14,136,210,135,17,136,22,136,21,136,34,136,33,136,49,136,54,136,57,136,39,136,59,136,68,136,66,136,82,136,89,136,94,136,98,136,107,136,129,136,126,136,158,136,117,136,125,136,181,136,114,136,130,136,151,136,146,136,174,136,153,136,162,136,141,136,164,136,176,136,191,136,177,136,195,136,196,136,212,136,216,136,217,136,221,136,249,136,2,137,252,136,244,136,232,136,242,136,4,137,12,137,10,137,19,137,67,137,30,137,37,137,42,137,43,137,65,137,68,137,59,137,54,137,56,137,76,137,29,137,96,137,94,137,102,137,100,137,109,137,106,137,111,137,116,137,119,137,126,137,131,137,136,137,138,137,147,137,152,137,161,137,169,137,166,137,172,137,175,137,178,137,186,137,189,137,191,137,192,137,218,137,220,137,221,137,231,137,244,137,248,137,3,138,22,138,16,138,12,138,27,138,29,138,37,138,54,138,65,138,91,138,82,138,70,138,72,138,124,138,109,138,108,138,98,138,133,138,130,138,132,138,168,138,161,138,145,138,165,138,166,138,154,138,163,138,196,138,205,138,194,138,218,138,235,138,243,138,231,138,228,138,241,138,20,139,224,138,226,138,247,138,222,138,219,138,12,139,7,139,26,139,225,138,22,139,16,139,23,139,32,139,51,139,171,151,38,139,43,139,62,139,40,139,65,139,76,139,79,139,78,139,73,139,86,139,91,139,90,139,107,139,95,139,108,139,111,139,116,139,125,139,128,139,140,139,142,139,146,139,147,139,150,139,153,139,154,139,58,140,65,140,63,140,72,140,76,140,78,140,80,140,85,140,98,140,108,140,120,140,122,140,130,140,137,140,133,140,138,140,141,140,142,140,148,140,124,140,152,140,29,98,173,140,170,140,189,140,178,140,179,140,174,140,182,140,200,140,193,140,228,140,227,140,218,140,253,140,250,140,251,140,4,141,5,141,10,141,7,141,15,141,13,141,16,141,78,159,19,141,205,140,20,141,22,141,103,141,109,141,113,141,115,141,129,141,153,141,194,141,190,141,186,141,207,141,218,141,214,141,204,141,219,141,203,141,234,141,235,141,223,141,227,141,252,141,8,142,9,142,255,141,29,142,30,142,16,142,31,142,66,142,53,142,48,142,52,142,74,142,71,142,73,142,76,142,80,142,72,142,89,142,100,142,96,142,42,142,99,142,85,142,118,142,114,142,124,142,129,142,135,142,133,142,132,142,139,142,138,142,147,142,145,142,148,142,153,142,170,142,161,142,172,142,176,142,198,142,177,142,190,142,197,142,200,142,203,142,219,142,227,142,252,142,251,142,235,142,254,142,10,143,5,143,21,143,18,143,25,143,19,143,28,143,31,143,27,143,12,143,38,143,51,143,59,143,57,143,69,143,66,143,62,143,76,143,73,143,70,143,78,143,87,143,92,143,98,143,99,143,100,143,156,143,159,143,163,143,173,143,175,143,183,143,218,143,229,143,226,143,234,143,239,143,135,144,244,143,5,144,249,143,250,143,17,144,21,144,33,144,13,144,30,144,22,144,11,144,39,144,54,144,53,144,57,144,248,143,79,144,80,144,81,144,82,144,14,144,73,144,62,144,86,144,88,144,94,144,104,144,111,144,118,144,168,150,114,144,130,144,125,144,129,144,128,144,138,144,137,144,143,144,168,144,175,144,177,144,181,144,226,144,228,144,72,98,219,144,2,145,18,145,25,145,50,145,48,145,74,145,86,145,88,145,99,145,101,145,105,145,115,145,114,145,139,145,137,145,130,145,162,145,171,145,175,145,170,145,181,145,180,145,186,145,192,145,193,145,201,145,203,145,208,145,214,145,223,145,225,145,219,145,252,145,245,145,246,145,30,146,255,145,20,146,44,146,21,146,17,146,94,146,87,146,69,146,73,146,100,146,72,146,149,146,63,146,75,146,80,146,156,146,150,146,147,146,155,146,90,146,207,146,185,146,183,146,233,146,15,147,250,146,68,147,46,147,25,147,34,147,26,147,35,147,58,147,53,147,59,147,92,147,96,147,124,147,110,147,86,147,176,147,172,147,173,147,148,147,185,147,214,147,215,147,232,147,229,147,216,147,195,147,221,147,208,147,200,147,228,147,26,148,20,148,19,148,3,148,7,148,16,148,54,148,43,148,53,148,33,148,58,148,65,148,82,148,68,148,91,148,96,148,98,148,94,148,106,148,41,146,112,148,117,148,119,148,125,148,90,148,124,148,126,148,129,148,127,148,130,149,135,149,138,149,148,149,150,149,152,149,153,149,160,149,168,149,167,149,173,149,188,149,187,149,185,149,190,149,202,149,246,111,195,149,205,149,204,149,213,149,212,149,214,149,220,149,225,149,229,149,226,149,33,150,40,150,46,150,47,150,66,150,76,150,79,150,75,150,119,150,92,150,94,150,93,150,95,150,102,150,114,150,108,150,141,150,152,150,149,150,151,150,170,150,167,150,177,150,178,150,176,150,180,150,182,150,184,150,185,150,206,150,203,150,201,150,205,150,77,137,220,150,13,151,213,150,249,150,4,151,6,151,8,151,19,151,14,151,17,151,15,151,22,151,25,151,36,151,42,151,48,151,57,151,61,151,62,151,68,151,70,151,72,151,66,151,73,151,92,151,96,151,100,151,102,151,104,151,210,82,107,151,113,151,121,151,133,151,124,151,129,151,122,151,134,151,139,151,143,151,144,151,156,151,168,151,166,151,163,151,179,151,180,151,195,151,198,151,200,151,203,151,220,151,237,151,79,159,242,151,223,122,246,151,245,151,15,152,12,152,56,152,36,152,33,152,55,152,61,152,70,152,79,152,75,152,107,152,111,152,112,152,113,152,116,152,115,152,170,152,175,152,177,152,182,152,196,152,195,152,198,152,233,152,235,152,3,153,9,153,18,153,20,153,24,153,33,153,29,153,30,153,36,153,32,153,44,153,46,153,61,153,62,153,66,153,73,153,69,153,80,153,75,153,81,153,82,153,76,153,85,153,151,153,152,153,165,153,173,153,174,153,188,153,223,153,219,153,221,153,216,153,209,153,237,153,238,153,241,153,242,153,251,153,248,153,1,154,15,154,5,154,226,153,25,154,43,154,55,154,69,154,66,154,64,154,67,154,62,154,85,154,77,154,91,154,87,154,95,154,98,154,101,154,100,154,105,154,107,154,106,154,173,154,176,154,188,154,192,154,207,154,209,154,211,154,212,154,222,154,223,154,226,154,227,154,230,154,239,154,235,154,238,154,244,154,241,154,247,154,251,154,6,155,24,155,26,155,31,155,34,155,35,155,37,155,39,155,40,155,41,155,42,155,46,155,47,155,50,155,68,155,67,155,79,155,77,155,78,155,81,155,88,155,116,155,147,155,131,155,145,155,150,155,151,155,159,155,160,155,168,155,180,155,192,155,202,155,185,155,198,155,207,155,209,155,210,155,227,155,226,155,228,155,212,155,225,155,58,156,242,155,241,155,240,155,21,156,20,156,9,156,19,156,12,156,6,156,8,156,18,156,10,156,4,156,46,156,27,156,37,156,36,156,33,156,48,156,71,156,50,156,70,156,62,156,90,156,96,156,103,156,118,156,120,156,231,156,236,156,240,156,9,157,8,157,235,156,3,157,6,157,42,157,38,157,175,157,35,157,31,157,68,157,21,157,18,157,65,157,63,157,62,157,70,157,72,157,93,157,94,157,100,157,81,157,80,157,89,157,114,157,137,157,135,157,171,157,111,157,122,157,154,157,164,157,169,157,178,157,196,157,193,157,187,157,184,157,186,157,198,157,207,157,194,157,217,157,211,157,248,157,230,157,237,157,239,157,253,157,26,158,27,158,30,158,117,158,121,158,125,158,129,158,136,158,139,158,140,158,146,158,149,158,145,158,157,158,165,158,169,158,184,158,170,158,173,158,97,151,204,158,206,158,207,158,208,158,212,158,220,158,222,158,221,158,224,158,229,158,232,158,239,158,244,158,246,158,247,158,249,158,251,158,252,158,253,158,7,159,8,159,183,118,21,159,33,159,44,159,62,159,74,159,82,159,84,159,99,159,95,159,96,159,97,159,102,159,103,159,108,159,106,159,119,159,114,159,118,159,149,159,156,159,160,159,47,88,199,105,89,144,100,116,220,81,153,113,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,127,0,0,0,64,171,170,170,42,0,0,0,32,154,153,153,25,85,85,85,21,146,36,73,18,0,0,0,16,142,227,56,14,205,204,204,12,186,232,162,11,171,170,170,10,138,157,216,9,73,146,36,9,137,136,136,8,0,0,0,8,136,135,135,7,199,113,28,7,175,161,188,6,102,102,102,6,134,97,24,6,93,116,209,5,22,178,144,5,85,85,85,5,82,184,30,5,197,78,236,4,47,161,189,4,37,73,146,4,88,238,105,4,68,68,68,4,66,8,33,4,0,0,0,4,62,248,224,3,196,195,195,3,132,58,168,3,228,56,142,3,35,159,117,3,216,80,94,3,131,52,72,3,51,51,51,3,50,56,31,3,195,48,12,3,232,11,250,2,47,186,232,2,131,45,216,2,11,89,200,2,5,49,185,2,171,170,170,2,21,188,156,2,8,0,0,0,0,0,0,0,164,0,0,0,0,0,0,0,232,12,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,88,220,24,0,96,0,0,0,0,0,0,0,100,39,52,0,4,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,45,178,157,239,167,198,6,64,61,10,215,163,112,189,18,192,66,96,229,208,34,219,24,64,35,219,249,126,106,188,28,192,117,147,24,4,86,142,26,64,84,227,165,155,196,32,20,192,180,200,118,190,159,26,10,64,233,38,49,8,172,28,250,191,251,58,112,206,136,210,218,63,216,230,24,0,152,227,24,0,90,0,0,0,32,0,0,0,136,225,24,0,224,236,24,0,28,0,0,0,56,0,0,0,200,222,24,0,200,222,24,0,110,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,176,168,24,0,56,120,26,0,104,217,24,0,152,25,25,0,72,167,24,0,120,131,26,0,144,164,24,0,248,164,24,0,136,161,24,0,128,63,26,0,232,159,24,0,248,33,26,0,64,157,24,0,104,11,26,0,32,155,24,0,176,207,25,0,0,0,0,0,0,0,0,0,43,125,128,62,238,131,132,62,125,59,141,62,87,66,156,62,243,4,181,62,68,41,223,62,217,111,151,62,234,70,247,62,0,0,0,0,0,0,0,0,143,149,62,64,6,247,65,66,107,190,157,70,190,43,33,78,154,121,130,90,222,161,148,111,98,236,183,75,29,117,163,123,0,0,0,0,0,0,0,0,43,125,0,63,238,131,4,63,125,59,13,63,87,66,28,63,243,4,53,63,68,41,95,63,217,111,151,63,234,70,247,63,84,148,183,64,0,0,0,0,150,62,64,0,247,65,66,0,190,157,70,0,44,33,78,0,122,130,90,0,162,148,111,0,217,111,151,0,234,70,247,0,81,81,222,2,0,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,0,0,0,128,0,0,0,128,0,0,0,130,0,0,0,128,0,0,0,154,0,0,0,166,0,0,0,142,0,0,0,204,0,0,0,190,0,0,0,170,0,0,0,30,2,0,0,204,1,0,0,150,2,0,0,158,1,0,0,1,2,1,7,6,0,2,3,1,3,4,5,5,4,4,3,1,1,6,4,15,14,5,0,1,3,4,4,5,5,4,3,30,30,14,14,7,7,0,0,0,0,0,0,0,0,0,0,1,2,2,3,2,0,3,3,1,3,4,5,5,4,4,3,1,0,6,4,2,3,5,7,1,3,4,4,4,4,4,4,212,78,213,78,206,78,207,78,204,78,214,78,216,78,70,79,96,79,24,39,25,39,100,39,101,39,109,39,177,39,183,19,214,19,199,9,233,9,237,9,238,4,247,4,120,2,57,1,154,0,159,0,32,0,17,0,10,0,3,0,1,0,0,0,11,0,18,0,33,0,76,0,155,0,58,1,121,2,112,2,239,4,226,4,234,9,216,9,215,19,208,19,178,39,162,39,26,39,27,39,102,79,103,79,97,79,71,79,217,78,215,78,205,78,210,78,211,78,208,78,209,78,0,0,0,0,0,0,16,16,16,16,16,16,16,16,16,15,15,15,15,15,15,14,14,13,13,13,12,12,11,10,9,9,7,6,5,3,1,2,5,6,7,8,9,10,11,11,12,12,13,13,14,14,15,15,15,15,16,16,16,16,16,16,16,16,16,16,16,0,0,0,249,255,7,0,250,255,7,0,251,255,7,0,248,255,15,0,249,255,15,0,250,255,15,0,253,255,1,0,254,127,0,0,254,15,0,0,254,3,0,0,254,0,0,0,62,0,0,0,14,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,30,0,0,0,126,0,0,0,254,1,0,0,254,7,0,0,254,31,0,0,254,63,0,0,252,255,1,0,248,255,7,0,251,255,15,0,252,255,15,0,253,255,15,0,254,255,15,0,255,255,15,0,0,0,0,0,19,19,19,20,20,20,17,15,12,10,8,6,4,2,1,3,5,7,9,11,13,14,17,19,20,20,20,20,20,0,0,0,180,254,1,0,181,254,1,0,118,253,1,0,119,253,1,0,116,253,1,0,117,253,1,0,138,254,1,0,139,254,1,0,136,254,1,0,128,254,0,0,182,254,1,0,130,254,0,0,184,254,0,0,66,127,0,0,174,127,0,0,175,63,0,0,209,31,0,0,233,31,0,0,233,15,0,0,234,7,0,0,251,7,0,0,251,3,0,0,251,1,0,0,255,1,0,0,124,0,0,0,60,0,0,0,28,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,1,0,0,0,13,0,0,0,29,0,0,0,61,0,0,0,125,0,0,0,252,0,0,0,252,1,0,0,252,3,0,0])
.concat([244,3,0,0,235,7,0,0,234,15,0,0,234,31,0,0,214,31,0,0,208,63,0,0,175,127,0,0,67,127,0,0,185,254,0,0,131,254,0,0,183,254,1,0,129,254,0,0,137,254,1,0,142,254,1,0,143,254,1,0,140,254,1,0,141,254,1,0,178,254,1,0,179,254,1,0,176,254,1,0,177,254,1,0,0,0,0,0,18,18,18,18,18,18,18,18,18,17,18,17,17,16,16,15,14,14,13,12,12,11,10,10,8,7,6,5,4,3,1,3,4,5,6,7,8,9,10,11,11,12,13,14,14,15,16,16,17,17,18,17,18,18,18,18,18,18,18,18,18,0,0,0,251,255,1,0,252,255,1,0,253,255,1,0,250,255,1,0,252,255,0,0,252,127,0,0,253,31,0,0,254,3,0,0,254,1,0,0,126,0,0,0,60,0,0,0,29,0,0,0,13,0,0,0,5,0,0,0,0,0,0,0,4,0,0,0,12,0,0,0,28,0,0,0,61,0,0,0,62,0,0,0,254,0,0,0,254,7,0,0,252,31,0,0,252,63,0,0,253,63,0,0,253,127,0,0,254,255,1,0,254,255,3,0,255,255,3,0,0,0,0,0,17,17,17,17,16,15,13,10,9,7,6,5,4,3,1,3,4,5,6,6,8,11,13,14,14,15,17,18,18,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,1,0,0,0,254,63,254,31,254,7,254,1,126,0,30,0,6,0,0,0,2,0,14,0,62,0,254,0,254,3,254,15,255,63,0,0,14,13,11,9,7,5,3,1,2,4,6,8,10,12,14,0,255,63,254,63,254,15,254,3,126,0,30,0,6,0,0,0,2,0,14,0,62,0,254,0,254,1,254,7,254,31,0,0,14,14,12,10,7,5,3,1,2,4,6,8,9,11,13,0,1,0,0,0,2,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,40,8,1,0,32,8,1,0,3,0,0,0,0,0,0,0,8,5,1,0,248,4,1,0,3,0,0,0,0,0,0,0,128,4,1,0,112,4,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,80,4,1,0,64,4,1,0,4,0,0,0,0,0,0,0,32,4,1,0,16,4,1,0,6,0,0,0,0,0,0,0,200,3,1,0,160,3,1,0,6,0,0,0,0,0,0,0,88,3,1,0,48,3,1,0,6,0,0,0,0,0,0,0,232,2,1,0,192,2,1,0,8,0,0,0,0,0,0,0,240,19,1,0,176,19,1,0,8,0,0,0,0,0,0,0,48,19,1,0,240,18,1,0,8,0,0,0,0,0,0,0,112,18,1,0,48,18,1,0,16,0,0,0,0,0,0,0,48,16,1,0,48,15,1,0,0,0,0,0,0,0,0,0,0,0,0,0,48,9,1,0,16,0,0,0,0,0,0,0,48,13,1,0,48,12,1,0,1,0,0,0,1,0,0,0,48,10,1,0,48,8,1,0,2,0,0,0,3,0,0,0,48,10,1,0,48,8,1,0,3,0,0,0,7,0,0,0,48,10,1,0,48,8,1,0,4,0,0,0,15,0,0,0,48,10,1,0,48,8,1,0,6,0,0,0,63,0,0,0,48,10,1,0,48,8,1,0,8,0,0,0,255,0,0,0,48,10,1,0,48,8,1,0,10,0,0,0,255,3,0,0,48,10,1,0,48,8,1,0,13,0,0,0,255,31,0,0,48,10,1,0,48,8,1,0,4,0,0,0,15,0,0,0,32,6,1,0,32,5,1,0,5,0,0,0,31,0,0,0,32,6,1,0,32,5,1,0,6,0,0,0,63,0,0,0,32,6,1,0,32,5,1,0,7,0,0,0,127,0,0,0,32,6,1,0,32,5,1,0,8,0,0,0,255,0,0,0,32,6,1,0,32,5,1,0,9,0,0,0,255,1,0,0,32,6,1,0,32,5,1,0,11,0,0,0,255,7,0,0,32,6,1,0,32,5,1,0,13,0,0,0,255,31,0,0,32,6,1,0,32,5,1,0,0,0,0,0,0,0,0,0,216,4,1,0,200,4,1,0,0,0,0,0,0,0,0,0,168,4,1,0,152,4,1,0,240,67,50,76,3,70,166,69,120,69,103,114,119,77,179,69,177,124,226,76,197,124,149,59,54,71,68,71,71,76,64,76,191,66,23,54,82,115,139,110,210,112,87,76,81,163,79,71,218,69,133,76,108,124,7,77,164,74,161,70,35,107,37,114,84,90,99,26,6,62,97,63,77,102,251,86,0,0,149,125,29,89,185,139,244,61,52,151,239,123,219,91,94,29,164,90,37,54,176,158,209,90,183,91,252,92,110,103,147,133,69,153,97,116,157,116,117,56,83,29,158,54,33,96,236,62,222,88,245,58,252,122,151,159,97,65,13,137,234,49,138,10,94,50,10,67,132,132,150,159,47,148,48,73,19,134,150,88,74,151,24,146,208,121,50,122,96,102,41,106,157,136,76,116,197,123,130,103,44,122,79,82,70,144,230,52,196,115,185,93,198,116,199,159,179,87,47,73,76,84,49,65,142,54,24,88,114,122,101,123,143,139,174,70,136,110,129,65,153,93,174,123,188,36,200,159,193,36,201,36,204,36,201,159,4,133,187,53,180,64,202,159,225,68,255,173,193,98,110,112,203,159,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,49,193,49,194,49,195,49,196,49,12,1,197,49,209,0,205,0,198,49,199,49,203,0,232,31,200,49,202,0,201,49,202,49,203,49,204,49,14,1,205,49,206,49,0,1,193,0,205,1,192,0,18,1,201,0,26,1,200,0,76,1,211,0,209,1,210,0,0,220,190,30,4,220,192,30,202,0,1,1,225,0,206,1,224,0,81,2,19,1,233,0,27,1,232,0,43,1,237,0,208,1,236,0,77,1,243,0,210,1,242,0,107,1,250,0,212,1,249,0,214,1,216,1,218,1,220,1,252,0,8,220,191,30,12,220,193,30,234,0,97,2,218,35,219,35,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,169,163,69,17,0,0,10,101,0,0,0,0,61,78,221,110,78,157,223,145,0,0,0,0,53,119,145,100,26,79,40,79,168,79,86,81,116,81,156,81,228,81,161,82,168,82,59,83,78,83,209,83,216,83,226,86,240,88,4,89,7,89,50,89,52,89,102,91,158,91,159,91,154,92,134,94,59,96,137,101,254,103,4,104,101,104,78,109,188,112,53,117,164,126,172,126,186,126,199,126,207,126,223,126,6,127,55,127,122,130,207,130,111,131,198,137,190,139,226,139,102,143,103,143,110,143,17,116,252,124,205,125,70,105,201,122,39,82,0,0,0,0,0,0,0,0,140,145,184,120,94,145,188,128,0,0,11,141,246,128,231,9,0,0,0,0,159,128,199,158,205,76,201,157,12,158,62,76,246,157,14,112,10,158,51,161,193,53,0,0,154,110,62,130,25,117,0,0,17,73,108,154,143,154,153,159,135,121,108,132,202,29,208,5,230,42,36,78,129,78,128,78,135,78,191,78,235,78,55,79,76,52,189,79,72,62,3,80,136,80,125,52,147,52,165,52,134,81,5,89,219,81,252,81,5,82,137,78,121,82,144,82,39,83,199,53,169,83,81,53,176,83,83,53,194,83,35,84,109,53,114,53,129,54,147,84,163,84,180,84,185,84,208,84,239,84,24,85,35,85,40,85,152,53,63,85,165,53,191,53,215,85,197,53,132,125,37,85,0,0,66,12,21,13,43,81,144,85,198,44,236,57,65,3,70,142,184,77,229,148,83,64,190,128,122,119,56,44,52,58,213,71,93,129,242,105,234,77,221,100,124,13,180,15,213,12,244,16,141,100,126,142,150,14,11,12,100,15,169,44,86,130,211,68,0,0,70,13,77,154,233,128,244,71,167,78,194,44,178,154,103,58,244,149,237,63,6,53,199,82,212,151,200,120,68,45,110,157,21,152,0,0,217,67,165,96,180,100,227,84,76,45,202,43,119,16,251,57,111,16,218,102,22,103,160,121,234,100,82,80,67,12,104,142,161,33,76,139,49,7,0,0,11,72,169,1,250,63,115,88,141,45,0,0,200,69,252,4,151,96,76,15,150,13,121,85,187,64,186,67,0,0,180,74,102,42,157,16,170,129,245,152,156,13,121,99,254,57,117,39,192,141,161,86,124,100,67,62,0,0,1,166,9,14,207,42,201,44,0,0,200,16,194,57,146,57,6,58,155,130,120,53,73,94,199,32,82,86,49,15,178,44,32,151,188,52,61,108,59,78,0,0,0,0,116,117,139,46,8,34,91,166,205,140,122,14,52,12,28,104,147,127,207,16,3,40,57,41,251,53,227,81,140,14,141,15,170,14,147,63,48,15,71,13,79,17,76,14,0,0,171,14,169,11,72,13,192,16,61,17,249,63,150,38,50,100,173,15,244,51,57,118,206,43,126,13,127,13,81,44,85,44,24,58,152,14,199,16,46,15,50,166,80,107,210,140,153,141,202,140,170,149,204,84,196,130,185,85,0,0,195,158,38,156,182,154,94,119,238,45,64,113,109,129,236,128,28,92,114,101,52,129,151,55,95,83,189,128,182,145,250,14,15,14,119,14,251,14,221,53,235,77,9,54,214,12,175,86,181,39,201,16,16,14,120,14,120,16,72,17,7,130,85,20,121,14,80,78,164,45,84,90,29,16,30,16,245,16,246,16,156,87,17,14,148,118,205,130,181,15,123,14,126,81,3,55,182,15,128,17,216,82,189,162,218,73,58,24,119,65,124,130,153,88,104,82,26,54,61,87,178,123,104,91,0,72,44,75,39,159,231,73,31,156,141,155,116,91,61,49,251,85,242,53,137,86,40,78,2,89,193,27,120,248,81,151,134,0,91,78,187,78,62,53,35,92,81,95,196,95,250,56,76,98,53,101,122,107,53,108,58,108,108,112,43,114,44,78,173,114,233,72,82,127,59,121,249,124,83,127,106,98,193,52,0,0,75,99,2,128,128,128,18,102,81,105,93,83,100,136,193,137,178,120,160,139,29,141,133,148,120,149,127,149,232,149,15,142,230,151,117,152,206,152,222,152,99,153,16,152,124,156,31,158,196,158,111,107,7,249,55,78,135,0,29,150,55,98,162,148,0,0,59,80,254,109,115,156,166,159,201,61,143,136,78,65,119,112,245,92,32,75,205,81,89,53,48,93,34,97,50,138,167,143,246,145,145,113,25,103,186,115,129,50,7,161,139,60,128,25,16,75,228,120,2,116,174,81,15,135,9,64,99,106,186,162,35,66,15,134,111,10,42,122,71,153,234,138,85,151,77,112,36,83,126,32,244,147,217,118,227,137,167,159,221,119,163,78,240,79,188,80,47,78,23,79,168,159,52,84,139,125,146,88,208,88,182,29,146,94,153,94,194,95,18,39,139,101,249,51,25,105,67,106,99,60,255,108,0,0,0,114,5,69,140,115,219,62,19,74,21,91,185,116,131,139,164,92,149,86,147,122,236,123,195,124,108,126,248,130,151,133,169,159,144,136,170,159,185,142,171,159,207,143,95,133,224,153,33,146,172,159,185,141,63,20,113,64,162,66,26,90,0,0,0,0,0,0,104,152,107,103,118,66,61,87,0,0,214,133,123,73,191,130,13,113,129,76,116,109,123,93,21,107,190,111,173,159,174,159,150,91,175,159,231,102,91,126,87,110,202,121,136,61,195,68,86,50,150,39,154,67,54,69,0,0,213,92,26,59,249,138,120,92,18,61,81,53,120,93,178,159,87,113,88,69,236,64,35,30,119,76,120,57,74,52,164,1,65,108,204,138,180,79,57,2,191,89,108,129,86,152,250,152,59,95,159,11,0,0,193,33,109,137,2,65,187,70,121,144,7,63,179,159,181,161,248,64,214,55,247,70,70,108,124,65,178,134,255,115,109,69,212,56,154,84,97,69,27,69,137,77,123,76,118,77,234,69,200,63,15,75,97,54,222,68,189,68,237,65,62,93,72,93,86,93,252,61,15,56,164,93,185,93,32,56,56,56,66,94,189,94,37,95,131,95,8,57,20,57,63,57,77,57,215,96,61,97,229,92,137,57,183,97,185,97,207,97,184,57,44,98,144,98,229,98,24,99,248,57,177,86,3,58,226,99,251,99,7,100,90,100,75,58,192,100,21,93,33,86,159,159,151,58,134,101,189,58,255,101,83,102,242,58,146,102,34,59,22,103,66,59,164,103,0,104,88,59,74,104,132,104,114,59,113,59,123,59,9,105,67,105,92,114,100,105,159,105,133,105,188,59,214,105,221,59,101,106,116,106,113,106,130,106,236,59,153,106,242,59,171,106,181,106,212,106,246,106,129,107,193,107,234,107,117,108,170,108,203,60,2,109,6,109,38,109,129,109,239,60,164,109,177,109,21,110,24,110,41,110,134,110,192,137,187,110,226,110,218,110,127,159,232,110,233,110,36,111,52,111,70,61,65,63,129,111,190,111,106,61,117,61,183,113,153,92,138,61,44,112,145,61,80,112,84,112,111,112,127,112,137,112,37,3,193,67,241,53,216,14,215,62,190,87,211,110,62,113,224,87,78,54,162,105,233,139,116,91,73,122,225,88,217,148,101,122,125,122,172,89,187,122,176,122,194,122,195,122,209,113,141,100,202,65,218,122,221,122,234,122,239,65,178,84,1,92,11,123,85,123,41,123,14,83,254,92,162,123,111,123,156,131,180,91,127,108,208,123,33,132,146,123,0,0,32,93,173,61,101,92,146,132,250,123,0,0,53,124,193,92,68,124,131,124,130,72,166,124,125,102,120,69,201,124,199,124,230,124,116,124,243,124,245,124,0,0,103,126,29,69,68,110,93,125,214,110,141,116,137,125,171,125,53,113,179,125,0,0,87,64,41,96,228,125,19,61,245,125,249,23,229,125,109,131,0,0,33,97,90,97,110,126,146,126,43,67,108,148,39,126,64,127,65,127,71,127,54,121,208,98,225,153,151,127,81,99,163,127,97,22,104,0,92,69,102,55,3,69,58,131,250,127,137,100,0,0,8,128,29,128,0,0,47,128,135,160,195,108,59,128,60,128,97,128,20,39,137,73,38,102,227,61,232,102,37,103,167,128,72,138,7,129,26,129,176,88,246,38,127,108,152,100,184,79,231,100,138,20,24,130,94,24,83,106,101,74,149,74,122,68,41,130,13,11,82,106,126,61,249,79,253,20,226,132,98,131,10,107,167,73,48,53,115,23,248,61,170,130,27,105,148,249,219,65,75,133,208,130,26,131,22,14,180,23,193,54,125,49,90,53,123,130,226,130,24,131,139,62,163,109,5,107,151,107,206,53,191,61,29,131,236,85,133,131,11,69,165,109,172,131,0,0,211,131,126,52,212,110,87,106,90,133,150,52,66,110,239,46,88,132,228,91,113,132,211,61,228,68,167,106,74,132,181,60,88,121,0,0,150,107,119,110,67,110,222,132,0,0,145,131,160,68,147,132,228,132,145,92,64,66,192,92,67,69,52,133,242,90,153,110,39,69,115,133,22,69,191,103,22,134,37,134,59,134,193,133,136,112,2,134,130,21,205,112,178,249,106,69,40,134,72,54,162,24,247,83,154,115,126,134,113,135,248,160,238,135,39,44,177,135,218,135,15,136,97,86,108,134,86,104,15,70,69,136,70,136,224,117,185,61,228,117,94,136,156,136,91,70,180,136,181,136,193,99,197,136,119,119,15,119,135,137,138,137,0,0,0,0,167,137,188,137,37,138,231,137,36,121,189,122,156,138,147,119,254,145,144,138,89,122,233,122,58,123,143,63,19,71,56,123,124,113,12,139,31,139,48,84,101,85,63,139,76,139,77,139,169,138,122,74,144,139,155,139,175,138,223,22,21,70,79,136,155,140,84,125,143,125,212,249,37,55,83,125,214,140,152,125,189,125,18,141,3,141,16,25,219,140,92,112,17,141,201,76,208,62,0,0,169,141,2,128,20,16,138,73,124,59,188,129,12,113,231,122,173,142,182,142,195,142,212,146,25,143,45,143,101,131,18,132,165,143,3,147,159,162,80,10,179,143,42,73,222,137,61,133,187,61,248,94,98,50,249,143,20,160,188,134,1,133,37,35,128,57,215,110,55,144,60,133,190,122,97,144,108,133,11,134,168,144,19,135,196,144,230,134,174,144,0,0,103,145,240,58,169,145,196,145,172,124,51,137,137,30,14,146,159,108,65,146,98,146,185,85,0,0,198,138,155,60,12,139,219,85,49,13,44,147,107,147,225,138,235,139,143,112,195,90,226,138,229,138,101,73,68,146,236,139,57,140,255,139,115,147,91,148,188,142,133,149,166,149,38,148,160,149,246,111,185,66,122,38,216,134,124,18,46,62,223,73,28,108,123,150,150,150,108,65,163,150,213,110,218,97,182,150,245,120,224,138,189,150,204,83,161,73,184,108,116,2,16,100,175,144,229,144,209,74,21,25,10,51,49,151,66,134,54,151,15,74,61,69,133,69,233,74,117,112,65,91,27,151,0,0,213,145,87,151,74,91,235,145,95,151,37,148,208,80,183,48,188,48,137,151,159,151,177,151,190,151,192,151,210,151,224,151,108,84,238,151,28,116,51,148,0,0,245,151,29,148,122,121,209,74,52,152,51,152,75,152,102,152,14,59,117,113,81,61,48,6,92,65,6,87,202,152,183,152,200,152,199,152,255,74,39,109,211,22,176,85,225,152,230,152,236,152,120,147,57,153,41,74,114,75,87,152,5,153,245,153,12,154,59,154,16,154,88,154,37,87,196,54,177,144,213,155,224,154,226,154,5,155,244,154,14,76,20,155,45,155,0,134,52,80,52,155,168,105,195,56,125,48,80,155,64,155,62,157,69,90,99,24,142,155,75,66,2,156,255,155,12,156,104,158,212,157,183,159,146,161,171,161,225,160,35,161,223,161,126,157,131,157,52,161,14,158,136,104,196,157,91,33,147,161,32,162,59,25,51,162,57,157,185,160,180,162,144,158,149,158,158,158,162,158,52,77,170,158,175,158,100,67,193,158,96,59,229,57,29,61,50,79,190,55,43,140,2,159,8,159,150,75,36,148,162,109,23,159,0,0,57,159,159,86,138,86,69,159,184,153,139,144,242,151,127,132,98,159,105,159,220,122,142,159,22,114,190,75,117,73,187,73,119,113,248,73,72,67,81,74,158,115,218,139,250,24,159,121,126,137,54,142,105,147,243,147,68,138,236,146,129,147,203,147,108,137,185,68,23,114,235,62,114,119,67,122,208,112,115,68,248,67,126,113,239,23,163,112,190,24,153,53,199,62,133,24,47,84,248,23,34,55,251,22,57,24,225,54,116,23,209,24,75,95,35,55,192,22,91,87,37,74,254,19,168,18,198,19,182,20,3,133,166,54,0,0,85,132,148,73,101,113,49,62,92,85,251,62,82,112,244,68,238,54,157,153,38,111,249,103,51,55,21,60,231,61,108,88,34,25,16,104,87,64,63,55,225,64,139,64,15,65,33,108,203,84,158,86,177,102,146,86,223,15,168,11,13,14,198,147,19,139,156,147,248,78,43,81,25,56,54,68,188,78,101,4,127,3,75,79,138,79,81,86,104,90,171,1,203,3,153,57,10,3,20,4,53,52,41,79,192,2,179,142,117,2,218,138,12,2,152,78,205,80,13,81,162,79,3,79,14,74,138,62,66,79,46,80,108,80,129,80,204,79,229,79,88,80,252,80,0,0,0,0,0,0,0,0,118,110,149,53,57,62,191,62,114,109,132,24,137,62,168,81,195,81,224,5,221,68,163,4,146,4,145,4,122,141,156,138,14,7,89,82,164,82,115,8,225,82,0,0,122,70,140,113,140,67,32,12,172,73,228,16,209,105,29,14,0,0,222,62,153,116,20,116,86,116,152,115,142,75,188,74,141,64,208,83,132,53,15,114,201,64,180,85,69,3,205,84,198,11,29,87,93,146,244,150,102,147,221,87,141,87,127,87,62,54,203,88,153,90,70,138,250,22,111,23,16,23,44,90,184,89,143,146,126,90,207,90,18,90,70,89,243,25,97,24,149,66,245,54,5,109,67,116,33,90,131,94,129,90,215,139,19,4,224,147,140,116,3,19,5,113,114,73,8,148,251,137,189,147,160,55,30,92,158,92,94,94,72,94,150,25,124,25,238,58,205,94,79,91,3,25,4,25,1,55,160,24,221,54,254,22,211,54,42,129,71,138,186,29,114,52,168,137,12,95,14,95,39,25,171,23,107,90,59,23,68,91,20,134,253,117,96,136,126,96,96,40,43,38,219,95,184,62,175,37,190,37,136,144,115,111,192,97,62,0,70,0,27,38,153,97,152,97,117,96,155,44,7,45,212,70,77,145,113,100,101,70,106,43,41,58,34,43,80,52,234,152,120,46,55,99,91,164,182,100,49,99,209,99,227,73,103,45,164,98,161,44,59,100,107,101,114,105,244,59,142,48,173,50,137,73,171,50,13,85,224,50,217,24,63,148,206,102,137,50,179,49,224,58,144,65,132,85,34,139,143,85,252,22,91,85,37,84,238,120,3,49,42,24,52,50,100,52,15,50,130,49,201,66,142,102,36,109,107,102,147,75,48,102,112,120,235,29,99,102,210,50,225,50,30,102,114,88,209,56,58,56,188,55,153,59,162,55,254,51,208,116,150,59,143,103,42,70,182,104,30,104,196,59,190,106,99,56,213,55,135,68,51,106,82,106,201,106,5,107,18,25,17,101,152,104,76,106,215,59,122,106,87,107,192,63,154,60,160,147,242,146,234,139,203,138,137,146,30,128,220,137,103,148,165,109,11,111,236,73,0,0,127,63,143,61,4,110,60,64,61,90,10,110,71,88,36,109,66,120,59,113,26,67,118,66,241,112,80,114,135,114,148,114,143,71,37,71,121,81,164,74,235,5,122,116,248,62,95,54,74,74,23,73,225,95,6,63,177,62,223,74,35,140,53,63,167,96,243,62,204,116,60,116,135,147,55,116,159,68,234,109,81,69,131,117,99,63,217,76,6,77,88,63,85,117,115,118,198,165,25,59,104,116,204,138,171,73,142,73,251,58,205,61,78,74,255,62,197,73,243,72,250,145,50,87,66,147,227,138,100,24,223,80,33,82,231,81,120,119,50,50,14,119,15,119,123,119,151,70,129,55,94,58,240,72,56,116,155,116,191,62,186,74,199,74,200,64,150,74,174,97,7,147,129,85,30,120,141,120,136,120,210,120,208,115,89,121,65,119,227,86,14,65,0,0,150,132,165,121,45,106,250,62,58,122,244,121,110,65,230,22,50,65,53,146,241,121,76,13,140,73,153,2,186,61,110,23,151,53,107,85,112,53,170,54,212,1,13,12,226,122,89,90,245,38,175,90,156,90,13,90,91,2,240,120,42,90,198,91,254,122,249,65,93,124,109,124,17,66,179,91,188,94,166,94,205,124,249,73,176,23,142,124,124,124,174,124,178,106,220,125,7,126,211,125,78,127,97,98,92,97,72,123,151,125,130,94,106,66,117,107,22,9,214,103,78,0,207,53,196,87,18,100,248,99,98,73,221,127,39,123,44,8,233,90,67,93,12,123,14,94,230,153,69,134,99,154,28,106,63,52,226,57,247,73,173,101,31,154,160,101,128,132,39,113,209,108,234,68,55,129,2,68,198,128,9,129,66,129,180,103,195,152,66,106,98,130,101,130,81,106,83,132,167,109,16,134,27,114,134,90,127,65,64,24,43,91,161,24,228,90,216,24,160,134,188,249,143,61,45,136,34,116,2,90,110,136,69,79,135,136,191,136,230,136,101,137,77,137,131,86,84,137,133,119,132,119,245,139,217,139,156,139,249,137,173,62,163,132,245,70,207,70,242,55,61,138,28,138,72,148,77,95,43,146,132,66,212,101,41,113,196,112,69,24,109,157,159,140,233,140,220,125,154,89,195,119,240,89,110,67,212,54,42,142,167,142,9,76,48,143,74,143,244,66,88,108,187,111,33,35,155,72,121,111,139,110,218,23,233,155,181,54,47,73,187,144,0,0,113,85,6,73,187,145,4,148,75,138,98,64,252,138,39,148,29,140,59,140,229,132,43,138,153,149,167,149,151,149,150,149,52,141,69,116,194,62,255,72,66,74,234,67,231,62,37,50,143,150,231,142,102,142,101,142,204,62,237,73,120,74,238,63,18,116,107,116,252,62,65,151,176,144,71,104,29,74,147,144,223,87,0,0,104,147,137,137,38,140,47,139,190,99,186,146,17,91,105,139,60,73,249,115,27,66,155,151,113,151,56,153,38,15,193,93,197,139,178,74,31,152,218,148,246,146,215,149,229,145,192,68,80,139,103,74,100,139,220,152,69,138,0,63,42,146,37,73,20,132,59,153,77,153,6,123,253,61,155,153,111,75,170,153,92,154,101,139,200,88,143,106,33,154,254,90,47,154,241,152,144,75,72,153,188,153,189,75,151,75,125,147,114,88,2,19,34,88,184,73,232,20,68,120,31,39,184,61,197,104,125,61,88,148,39,57,80,97,129,39,107,41,7,97,79,156,83,156,123,156,53,156,16,156,127,155,207,155,45,158,159,155,245,161,254,160,33,157,174,76,4,65,24,158,176,76,12,157,180,161,237,160,243,160,47,153,165,157,189,132,18,110,223,111,130,107,252,133,51,69,164,109,132,110,240,109,32,132,238,133,0,110,215,55,100,96,226,121,156,53,64,54,45,73,222,73,98,61,219,147,190,146,72,147,191,2,185,120,119,146,77,148,228,79,64,52,100,144,93,85,61,120,84,120,182,120,75,120,87,23,201,49,65,73,154,54,114,79,218,111,217,111,0,0,30,112,20,84,181,65,187,87,243,88,138,87,22,157,215,87,52,113,175,52,172,65,235,113,64,108,151,79,0,0,181,23,73,138,12,97,206,90,11,90,188,66,136,68,44,55,123,75,252,137,187,147,184,147,214,24,29,15,114,132,192,108,19,20,250,66,38,44,193,67,148,89,183,61,65,103,168,125,91,97,164,96,185,73,139,73,250,137,229,146,226,115,233,62,180,116,99,139,159,24,225,62,179,74,216,106,243,115,251,115,214,62,62,74,148,74,217,23,102,74,167,3,36,20,229,73,72,116,22,73,165,112,118,73,132,146,230,115,95,147,254,4,49,147,206,138,22,138,134,147,231,139,213,85,53,73,130,138,107,113,67,73,255,12,164,86,26,6,235,11,184,12,2,85,196,121,250,23,254,125,194,22,80,74,82,24,46,69,1,148,10,55,192,138,173,73,176,89,191,24,131,24,132,116,161,90,226,54,91,61,176,54,95,146,121,90,129,138,98,24,116,147,205,60,180,10,150,74,138,57,244,80,105,61,76,61,156,19,117,113,251,66,24,130,15,110,228,144,235,68,87,109,79,126,103,112,175,108,214,60,237,63,45,62,2,110,12,111,111,61,245,3,81,117,188,54,200,52,128,70,218,62,113,72,196,89,110,146,62,73,65,143,28,140,192,107,18,88,200,87,214,54,82,20,254,112,98,67,113,74,227,47,176,18,189,35,185,104,103,105,152,19,229,52,244,123,223,54,131,138,214,55,250,51,159,76,26,106,173,54,183,108,62,132,223,68,206,68,38,109,81,109,130,108,222,111,23,111,9,113,61,131,58,23,237,131,128,108,83,112,219,23,137,89,130,90,179,23,97,90,113,90,5,25,252,65,45,55,239,89,60,23,199,54,142,113,144,147,154,102,165,66,110,90,43,90,147,66,43,106,249,62,54,119,91,68,202,66,29,113,89,66,225,137,176,79,40,109,194,92,206,68,77,126,189,67,12,106,86,66,4,19,166,112,51,113,233,67,165,61,223,108,37,248,79,74,101,126,235,89,47,93,243,61,92,95,93,74,223,23,164,125,38,132,133,84,250,58,0,51,20,2,126,87,213,8,25,6,229,63,158,31,182,162,3,112,91,145,112,93,143,115,211,124,89,138,32,148,200,79,231,127,205,114,16,115,244,122,56,115,57,115,246,86,65,115,72,115,169,62,24,123,108,144,245,113,242,72,225,115,246,129,202,62,12,119,209,62,162,108,253,86,25,116,30,116,31,116,226,62,240,62,244,62,250,62,211,116,14,63,83,63,66,117,109,117,114,117,141,117,124,63,200,117,220,117,192,63,77,118,215,63,116,118,220,63,122,118,92,79,136,113,35,86,128,137,105,88,29,64,67,119,57,64,97,103,69,64,219,53,152,119,106,64,111,64,94,92,190,119,203,119,242,88,24,120,185,112,28,120,168,64,57,120,71,120,81,120,102,120,72,132,53,85,51,121,3,104,50,121,3,65,9,65,145,121,153,121,187,143,6,122,188,143,103,65,145,122,178,65,188,122,121,130,196,65,207,122,219,122,207,65,33,78,98,123,108,123,123,123,18,124,27,124,96,66,122,66,123,124,156,124,140,66,184,124,148,66,237,124,147,143,192,112,207,12,207,125,212,125,208,125,253,125,174,127,180,127,159,114,151,67,32,128,37,128,57,123,46,128,49,128,84,128,204,61,180,87,160,112,183,128,233,128,237,67,12,129,42,115,14,129,18,129,96,117,20,129,1,68,57,59,86,129,89,129,90,129,19,68,58,88,124,129,132,129,37,68,147,129,45,68,165,129,239,87,193,129,228,129,84,130,143,68,166,130,118,130,202,130,216,130,255,130,176,68,87,131,105,150,138,105,5,132,245,112,100,132,227,96,136,132,4,69,190,132,225,132,248,132,16,133,56,133,82,133,59,69,111,133,112,133,224,133,119,69,114,134,146,134,178,134,239,134,69,150,139,135,6,70,23,70,174,136,255,136,36,137,71,137,145,137,103,121,41,138,56,138,148,138,180,138,81,140,212,140,242,140,28,141,152,71,95,88,195,141,237,71,238,78,58,142,216,85,84,87,113,142,245,85,176,142,55,72,206,142,226,142,228,142,237,142,242,142,183,143,193,143,202,143,204,143,51,144,196,153,173,72,224,152,19,146,30,73,40,146,88,146,107,146,177,146,174,146,191,146,227,146,235,146,243,146,244,146,253,146,67,147,132,147,173,147,69,73,81,73,191,158,23,148,1,83,29,148,45,148,62,148,106,73,84,148,121,148,45,149,162,149,167,73,244,149,51,150,229,73,160,103,36,74,64,151,53,74,178,151,194,151,84,86,228,74,232,96,185,152,25,75,241,152,68,88,14,153,25,153,180,81,28,153,55,153,66,153,93,153,98,153,112,75,197,153,157,75,60,154,15,155,131,122,105,155,129,155,221,155,241,155,244,155,109,76,32,156,111,55,194,27,73,157,58,156,254,158,80,86,147,157,189,157,192,157,252,157,246,148,182,143,123,158,172,158,177,158,189,158,198,158,220,148,226,158,241,158,248,158,200,122,68,159,148,0,183,2,160,3,26,105,195,148,172,89,215,4,64,88,193,148,185,55,213,5,21,6,118,6,186,22,87,87,115,113,194,10,205,10,191,11,106,84,59,248,203,11,158,84,251,11,59,12,83,12,101,12,124,12,231,96,141,12,122,86,181,12,221,12,237,12,111,13,178,13,200,13,85,105,47,156,165,135,4,14,14,14,215,14,144,15,45,15,115,14,32,92,188,15,11,94,92,16,79,16,118,16,30,103,123,16,136,16,150,16,71,54,191,16,211,16,47,17,59,17,100,83,173,132,227,18,117,19,54,19,129,139,119,21,25,22,195,23,199,23,120,78,187,112,45,24,106,25,45,26,69,26,42,28,112,28,172,28,200,30,195,98,213,30,21,31,152,113,85,104,69,32,233,105,200,54,124,34,215,35,250,35,42,39,113,40,79,41,253,130,103,41,147,41,213,42,165,137,232,42,160,143,14,43,184,151,63,43,71,152,189,154,76,44,0,0,136,44,183,44,232,91,8,45,18,45,183,45,149,45,66,46,116,47,204,47,51,48,102,48,31,51,222,51,177,95,72,102,191,102,121,122,103,53,243,53,0,0,186,73,0,0,26,54,22,55,0,0,70,3,181,88,14,103,24,105,167,58,87,118,226,95,17,62,185,62,254,117,154,32,208,72,184,74,25,65,154,138,238,66,13,67,59,64,52,67,150,67,69,74,202,5,210,81,17,6,159,89,168,30,190,59,255,60,4,68,214,68,136,87,116,70,155,57,47,71,232,133,201,153,98,55,195,33,94,139,78,139,0,0,18,72,251,72,21,74,9,114,192,74,120,12,101,89,165,78,134,79,121,7,218,142,44,80,143,82,63,87,113,113,153,82,25,84,74,63,167,74,188,85,70,84,110,84,82,107,0,0,115,52,63,85,50,118,94,85,24,71,98,85,102,85,199,87,63,73,93,88,102,80,251,52,204,51,0,0,3,89,124,71,72,137,174,90,137,91,6,92,144,29,161,87,81,113,0,0,2,97,18,124,86,144,178,97,154,79,98,139,2,100,74,100,91,93,247,107,0,0,132,100,28,25,234,138,246,73,136,100,239,63,18,101,192,75,191,101,181,102,27,39,101,148,225,87,149,97,39,90,205,248,0,0,185,86,33,69,252,102,106,78,52,73,86,150,143,109,189,108,24,54,119,137,153,103,110,104,17,100,94,104,0,0,199,104,66,123,192,144,17,10,38,105,0,0,57,105,69,122,0,0,250,105,38,154,45,106,95,54,105,100,33,0,131,121,52,106,91,107,44,93,25,53,0,0,157,107,208,70,164,108,59,117,101,136,174,109,182,88,28,55,141,37,75,112,205,113,84,60,128,114,133,114,129,146,122,33,139,114,48,147,230,114,208,73,57,108,159,148,80,116,248,14,39,136,245,136,38,41,115,132,177,23,184,110,42,74,32,24,164,57,185,54,0,0,0,0,63,69,182,102,173,156,164,152,67,137,204,119,88,120,214,86,223,64,10,22,161,57,47,55,232,128,197,19,173,113,102,131,221,121,168,145,0,0,183,76,175,112,171,137,253,121,10,122,11,123,102,125,122,65,67,123,126,121,9,128,181,111,223,162,3,106,24,131,162,83,7,110,191,147,54,104,93,151,111,129,35,128,181,105,237,19,47,50,72,128,133,93,48,140,131,128,21,87,35,152,73,137,171,93,136,73,190,101,213,105,210,83,165,74,129,63,17,60,54,103,144,128,244,128,46,129,161,31,79,129,137,129,175,129,26,130,6,131,47,131,138,131,202,53,104,132,170,134,250,72,230,99,86,137,8,120,85,146,184,137,242,67,231,137,223,67,232,137,70,139,212,139,248,89,9,140,0,0,197,143,236,144,0,0,16,145,60,145,247,61,94,145,202,74,208,143,143,114,139,86,231,148,233,149,176,149,184,149,50,151,209,152,73,153,106,153,195,153,40,154,14,155,90,157,155,157,159,126,248,158,35,159,164,76,71,149,147,162,162,113,255,162,145,77,18,144,203,165,156,77,156,12,190,143,193,85,186,143,176,36,185,143,147,74,9,69,127,126,86,111,177,106,234,78,228,52,44,139,157,120,58,55,128,142,245,23,36,128,108,139,153,139,62,122,175,102,235,61,85,118,183,60,53,86,86,89,154,78,129,94,88,98,191,86,109,14,14,142,109,91,136,62,158,76,222,99,0,0,246,23,123,24,48,101,45,86,74,92,26,84,17,83,198,61,152,157,125,76,34,86,30,86,73,127,216,94,117,89,64,61,112,135,28,78,234,15,73,13,186,54,23,129,94,157,24,141,59,118,69,156,78,118,185,119,69,147,50,84,72,129,247,130,37,86,50,129,24,132,189,128,234,85,98,121,67,86,22,84,157,14,206,53,5,86,241,85,241,102,226,130,45,54,52,117,240,85,186,85,151,84,114,85,65,12,150,12,208,94,72,81,118,14,98,44,162,14,171,158,90,125,222,85,117,16,157,98,109,151,148,84,205,140,246,113,118,145,252,99,185,99,254,99,105,85,67,43,114,156,179,46,154,81,223,52,167,13,167,81,77,84,30,85,19,85,102,118,45,142,138,104,177,117,182,128,4,136,134,135,199,136,182,129,28,132,193,16,236,68,4,115,6,71,144,91,11,131,147,104,123,86,244,38,47,125,163,65,115,125,208,110,182,114,112,145,217,17,8,146,252,60,169,166,172,14,249,14,102,114,162,28,78,71,194,79,249,127,235,15,250,64,93,156,31,101,160,45,243,72,224,71,124,157,236,15,10,14,0,0,163,117,237,15,0,0,72,96,135,17,163,113,142,126,80,157,26,78,4,78,119,53,13,91,178,108,103,83,172,54,220,57,125,83,165,54,24,70,154,88,110,75,45,130,75,84,170,87,149,90,121,9,0,0,82,58,101,36,116,115,172,158,9,77,237,155,254,60,48,159,91,76,169,79,158,149,222,159,92,132,182,61,178,114,179,103,32,55,46,99,37,125,247,62,44,62,42,58,8,144,204,82,116,62,122,54,233,69,142,4,64,118,240,90,182,14,122,120,46,127,167,88,191,64,124,86,139,155,116,93,84,118,52,164,133,158,225,76,0,0,251,55,25,97,218,48,242,67,0,0,93,86,169,18,167,87,99,73,6,158,52,82,174,112,173,53,0,0,124,157,86,124,57,155,222,87,108,23,83,92,211,100,208,148,53,99,100,113,173,134,40,13,34,109,226,74,113,13,0,0,254,81,15,31,142,93,3,151,209,29,129,158,76,144,31,123,2,155,209,92,163,123,104,98,53,99,255,154,207,123,42,155,126,124,0,0,66,124,134,124,21,156,252,123,9,155,0,0,27,156,62,73,90,159,115,85,195,91,253,79,152,158,242,79,96,82,6,62,209,82,103,87,86,80,183,89,18,94,200,151,171,157,92,143,105,84,180,151,64,153,186,151,44,83,48,97,44,105,218,83,10,156,2,157,59,76,65,150,128,105,166,80,70,117,109,23,218,153,115,82,0,0,89,145,129,150,92,145,0,0,81,145,151,142,127,99,35,109,202,106,17,86,142,145,122,117,133,98,252,3,79,115,112,124,33,92,253,60,0,0,25,73,214,118,157,155,42,78,212,12,190,131,66,136,0,0,74,92,192,105,0,0,122,87,31,82,245,93,206,78,49,108,242,1,57,79,156,84,218,84,154,82,130,141,254,53,0,0,243,53,0,0,82,107,124,145,165,159,151,155,46,152,180,152,186,154,168,158,132,158,122,113,20,123,0,0,250,107,24,136,120,127,0,0,32,86,74,166,119,142,83,159,0,0,212,141,79,142,28,158,1,142,130,98,125,131,40,142,117,142,211,122,119,74,62,122,216,120,234,108,103,138,7,118,90,138,38,159,206,108,214,135,195,117,178,162,83,120,64,248,12,141,226,114,113,115,45,139,2,115,241,116,235,140,187,74,47,134,186,95,160,136,183,68,0,0,59,24,5,110,0,0,126,138,27,37,0,0,253,96,103,118,215,154,68,157,110,147,143,155,245,135,0,0,0,0,247,140,44,115,33,151,176,155,214,53,178,114,7,76,81,124,74,153,89,97,89,97,4,76,150,158,125,97,0,0,95,87,111,97,166,98,57,98,0,0,92,58,226,97,170,83,245,51,100,99,2,104,210,53,87,93,194,139,218,143,57,142,0,0,217,80,70,29,6,121,50,83,56,150,59,15,101,64,0,0,254,119,0,0,194,124,26,95,218,124,45,122,102,128,99,128,77,125,5,117,242,116,148,137,26,130,12,103,98,128,134,116,91,128,240,116,3,129,36,119,137,137,204,103,83,117,209,110,169,135,206,135,200,129,140,135,73,138,173,140,67,139,43,119,248,116,218,132,53,54,178,105,166,141,0,0,169,137,0,0,185,109,193,135,17,64,231,116,219,61,118,113,164,96,156,97,209,60,0,0,119,96,0,0,113,127,45,139,0,0,233,96,126,75,32,82,24,60,199,60,215,94,86,118,49,85,68,25,254,18,3,153,220,109,173,112,193,92,173,97,15,138,119,54,238,0,70,104,14,79,98,69,31,91,76,99,80,159,166,158,107,98,71,5,219,146,223,5,197,63,76,133,181,66,239,115,181,81,73,54,66,73,228,137,68,147,219,25,238,130,200,60,60,120,68,103,223,98,51,73,170,137,160,2,179,107,5,19,171,79,237,36,8,80,41,109,132,122,0,54,177,74,19,37,0,0,126,3,164,95,128,3,71,3,219,110,31,4,0,0,1,81,122,52,14,81,108,152,67,55,22,132,164,73,135,4,96,81,180,51,106,81,255,11,252,32,229,2,48,37,142,5,51,50,131,25,130,91,125,135,179,5,153,60,178,81,184,81,52,157,201,81,207,81,209,81,220,60,211,81,166,74,179,81,226,81,66,83,237,81,205,131,62,105,45,55,123,95,11,82,38,82,60,82,181,82,87,82,148,82,185,82,197,82,21,124,66,133,224,82,13,134,19,107,0,0,222,138,73,85,217,110,128,63,84,9,236,63,51,83,0,0,226,11,203,108,38,23,27,104,213,115,74,96,170,62,204,56,232,22,221,113,162,68,109,83,116,83,171,134,126,83,0,0,150,21,19,22,230,119,147,83,155,138,160,83,171,83,174,83,167,115,114,87,89,63,156,115,193,83,197,83,73,108,73,78,254,87,217,83,171,58,143,11,224,83,235,63,163,45,246,83,119,12,19,84,121,112,43,85,87,102,91,109,109,84,83,107,116,13,93,85,143,84,164,84,166,71,13,23,221,14,180,61,77,13,188,137,152,38,71,85,237,76,47,84,23,116,134,85,169,85,0,0,215,24,58,64,82,69,53,68,179,102,180,16,55,86,205,102,138,50,164,102,173,102,77,86,79,86,241,120,241,86,135,151,254,83,0,87,239,86,237,86,102,139,35,54,79,18,70,87,165,65,110,108,139,112,66,87,177,54,126,108,230,87,22,20,3,88,84,20,99,67,38,88,245,75,92,88,170,88,97,53,224,88,220,88,60,18,251,88,255,91,67,87,80,161,120,66,211,147,161,53,31,89,166,104,195,54,89,110,62,22,36,90,83,85,146,22,5,133,201,89,78,13,129,108,42,109,220,23,217,89,251,23,178,23,166,109,113,109,40,24,213,22,249,89,69,110,171,90,99,90,230,54,169,73,0,0,8,55,150,90,101,116,211,90,161,111,84,37,133,61,17,25,50,55,184,22,131,94,208,82,118,91,136,101,124,91,14,122,4,64,93,72,4,2,213,91,96,97,52,26,204,89,165,5,243,91,157,91,16,77,5,92,68,27,19,92,206,115,20,92,165,28,40,107,73,92,221,72,133,92,233,92,239,92,139,93,249,29,55,30,16,93,24,93,70,93,164,30,186,92,215,93,252,130,45,56,1,73,73,32,115,33,135,130,54,56,194,59,46,94,138,106,0,0,122,94,188,68,211,12,166,83,183,78,0,0,168,83,113,23,9,94,244,94,130,132,249,94,251,94,160,56,252,94,62,104,27,148,13,95,193,1,148,248,222,58,174,72,58,19,58,95,136,104,208,35,0,0,113,36,99,95,189,151,110,110,114,95,64,147,54,138,167,95,182,93,95,61,80,82,106,31,248,112,104,38,214,145,158,2,41,138,49,96,133,102,119,24,99,57,199,61,57,54,144,87,180,39,113,121,64,62,158,96,0,0,179,96,130,73,143,73,83,122,164,116,225,80,160,90,100,97,36,132,66,97,166,248,210,110,129,97,244,81,86,6,135,97,170,91,183,63,95,40,211,97,157,139,93,153,208,97,50,57,128,41,193,40,35,96,92,97,30,101,139,99,24,1,197,98,112,23,213,98,13,46,108,99,223,73,23,58,56,100,248,99,142,19,252,23,0,0,138,111,54,46,20,152,140,64,29,87,225,100,229,100,123,148,102,58,58,100,87,58,77,101,22,111,40,74,35,74,133,101,109,101,95,101,126,48,181,101,64,73,55,75,209,101,216,64,41,24,224,101,227,101,223,95,0,52,24,102,247,49,248,49,68,102,164,49,165,49,75,102,117,14,103,102,230,81,115,102,0,0,61,30,49,50,244,133,200,49,19,83,197,119,247,40,164,153,2,103,156,67,33,74,43,59,250,105,194,55,0,0,103,103,98,103,205,65,237,144,215,103,233,68,34,104,80,110,60,146,1,104,230,51,160,109,93,104,111,52,225,105,11,106,223,138,115,105,195,104,205,53,1,105,0,105,50,61,1,58,60,54,128,59,172,103,97,105,74,138,252,66,54,105,152,105,161,59,201,3,99,131,144,80,249,105,89,54,42,33,69,106,3,55,157,106,243,59,177,103,200,106,156,145,13,60,29,107,35,9,222,96,53,107,116,107,205,39,181,110,219,58,181,3,88,25,64,55,33,84,90,59,225,107,252,62,220,107,55,108,139,36,241,72,81,107,90,108])
.concat([38,130,121,108,188,61,197,68,189,61,164,65,12,73,0,73,201,60,229,54,235,60,50,13,131,155,249,49,145,36,143,127,55,104,37,109,161,109,235,109,150,109,92,109,124,110,4,111,127,73,133,64,114,110,51,133,116,111,199,81,0,0,0,0,46,132,33,139,0,0,47,62,83,116,130,63,204,121,79,110,145,90,75,48,248,111,13,55,157,111,48,62,250,110,151,20,61,64,85,69,240,147,68,111,92,111,78,61,116,111,112,145,59,61,159,111,68,65,211,111,145,64,85,65,57,64,240,63,180,63,63,65,223,81,86,65,87,65,64,65,221,97,75,112,126,112,167,112,129,112,204,112,213,112,214,112,223,112,4,65,232,61,180,113,150,113,119,66,43,113,69,113,136,90,74,113,0,0,156,92,101,67,79,113,98,147,193,66,44,113,90,68,39,74,34,74,186,113,232,139,189,112,14,114,66,148,21,114,17,89,67,148,36,114,65,147,5,86,46,114,64,114,116,73,189,104,85,114,87,114,85,62,68,48,13,104,61,111,130,114,0,0,43,115,35,72,43,136,237,72,4,136,40,115,46,115,207,115,170,115,58,12,46,106,201,115,73,116,226,65,231,22,36,74,35,102,197,54,183,73,141,73,251,73,247,115,21,116,3,105,38,74,57,116,195,5,215,62,0,0,173,40,96,116,178,142,71,116,228,115,118,116,185,131,108,116,48,55,116,116,241,147,44,106,130,116,83,73,140,74,95,65,121,74,143,139,70,91,3,140,158,24,200,116,136,25,14,117,0,0,30,117,217,142,75,26,215,91,172,142,133,147,77,117,74,117,103,117,110,117,130,79,4,63,19,77,142,117,93,116,158,117,180,117,2,118,44,118,81,118,79,118,111,118,118,118,245,99,144,118,239,129,248,55,17,105,14,105,161,118,165,118,183,118,204,118,159,111,98,132,157,80,125,81,28,30,30,119,38,119,64,119,175,100,32,82,88,119,172,50,175,119,100,137,104,137,193,22,244,119,0,0,118,19,18,74,202,104,175,120,199,120,211,120,165,150,46,121,224,85,215,120,52,121,177,120,12,118,184,143,132,136,43,139,131,96,28,38,134,121,0,137,2,105,128,121,87,88,157,121,57,123,60,121,169,121,42,110,38,113,168,62,198,121,13,145,212,121,32,5,95,68,15,82,130,184,248,0,0,64,32,169,78,4,0,0,0,0,52,11,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,64,0,0,0,0,0,0,0,0,4,88,0,60,0,0,0,0,0,128,92,173,220,243,187,201,67,238,237,164,15,38,193,155,118,242,239,222,251,250,247,15,50,68,175,254,239,253,254,17,96,176,0,0,33,25,129,32,16,136,168,96,33,105,36,0,68,137,196,0,0,3,64,53,0,67,132,49,81,147,104,2,2,0,0,0,0,0,0,0,0,0,0,0,64,0,1,128,172,100,130,64,140,161,144,4,80,96,212,0,210,42,24,157,104,53,151,79,216,248,32,196,130,28,40,130,117,148,2,56,64,160,161,197,16,113,77,196,13,152,185,94,91,1,67,159,153,3,120,32,226,31,8,137,1,201,64,7,3,13,154,144,82,209,79,124,3,2,33,32,192,110,19,131,181,219,245,229,119,144,79,23,154,54,23,12,0,110,186,9,47,96,84,0,30,120,104,34,112,86,158,37,243,142,183,23,11,239,230,119,23,91,27,193,16,152,141,18,24,182,207,1,49,114,26,91,44,182,192,32,226,195,92,130,27,224,218,206,133,214,64,169,84,11,208,15,72,68,64,16,9,212,64,174,59,100,30,68,43,23,8,134,130,54,24,160,137,231,156,91,8,132,3,2,218,144,236,219,135,133,191,162,173,157,142,157,152,32,37,70,0,70,159,127,227,213,201,132,244,214,77,6,227,182,24,18,9,0,0,0,4,0,0,32,0,0,0,0,2,0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,142,120,110,95,63,126,221,61,247,252,155,252,174,210,255,143,107,241,231,191,186,174,110,23,239,220,167,62,182,181,119,201,146,215,165,182,74,110,155,185,129,77,103,254,84,241,155,98,254,111,82,215,54,254,191,41,69,129,249,189,101,166,66,57,0,0,66,96,143,0,148,64,64,210,175,215,209,219,0,160,152,178,123,6,82,32,152,22,200,61,9,0,0,1,0,0,0,1,10,178,8,128,0,0,0,4,136,32,20,16,76,0,0,4,0,161,68,16,0,65,1,0,8,64,240,223,79,54,88,241,245,181,128,245,103,128,64,0,80,78,65,200,132,0,90,96,216,1,43,1,80,161,22,196,64,242,110,17,172,32,57,196,192,8,7,12,9,152,150,196,155,16,28,152,108,134,42,166,1,70,17,109,229,203,196,192,146,16,33,22,18,157,114,244,105,14,23,42,162,129,244,123,0,8,164,11,144,16,44,204,57,74,1,240,22,11,10,64,12,29,234,12,68,14,69,2,0,24,248,24,0,0,0,0,0,0,0,8,0,200,247,24,0,128,247,24,0,0,0,4,0,2,78,4,78,5,78,6,78,15,78,18,78,23,78,31,78,32,78,33,78,35,78,38,78,41,78,46,78,47,78,49,78,51,78,53,78,55,78,60,78,64,78,65,78,66,78,68,78,70,78,74,78,81,78,85,78,87,78,90,78,91,78,98,78,99,78,100,78,101,78,103,78,104,78,106,78,107,78,108,78,109,78,110,78,111,78,114,78,116,78,117,78,118,78,119,78,120,78,121,78,122,78,123,78,124,78,125,78,127,78,128,78,129,78,130,78,131,78,132,78,133,78,135,78,138,78,144,78,150,78,151,78,153,78,156,78,157,78,158,78,163,78,170,78,175,78,176,78,177,78,180,78,182,78,183,78,184,78,185,78,188,78,189,78,190,78,200,78,204,78,207,78,208,78,210,78,218,78,219,78,220,78,224,78,226,78,230,78,231,78,233,78,237,78,238,78,239,78,241,78,244,78,248,78,249,78,250,78,252,78,254,78,0,79,2,79,3,79,4,79,5,79,6,79,7,79,8,79,11,79,12,79,18,79,19,79,20,79,21,79,22,79,28,79,29,79,33,79,35,79,40,79,41,79,44,79,45,79,46,79,49,79,51,79,53,79,55,79,57,79,59,79,62,79,63,79,64,79,65,79,66,79,68,79,69,79,71,79,72,79,73,79,74,79,75,79,76,79,82,79,84,79,86,79,97,79,98,79,102,79,104,79,106,79,107,79,109,79,110,79,113,79,114,79,117,79,119,79,120,79,121,79,122,79,125,79,128,79,129,79,130,79,133,79,134,79,135,79,138,79,140,79,142,79,144,79,146,79,147,79,149,79,150,79,152,79,153,79,154,79,156,79,158,79,159,79,161,79,162,79,164,79,171,79,173,79,176,79,177,79,178,79,179,79,180,79,182,79,183,79,184,79,185,79,186,79,187,79,188,79,189,79,190,79,192,79,193,79,194,79,198,79,199,79,200,79,201,79,203,79,204,79,205,79,210,79,211,79,212,79,213,79,214,79,217,79,219,79,224,79,226,79,228,79,229,79,231,79,235,79,236,79,240,79,242,79,244,79,245,79,246,79,247,79,249,79,251,79,252,79,253,79,255,79,0,80,1,80,2,80,3,80,4,80,5,80,6,80,7,80,8,80,9,80,10,80,11,80,14,80,16,80,17,80,19,80,21,80,22,80,23,80,27,80,29,80,30,80,32,80,34,80,35,80,36,80,39,80,43,80,47,80,48,80,49,80,50,80,51,80,52,80,53,80,54,80,55,80,56,80,57,80,59,80,61,80,63,80,64,80,65,80,66,80,68,80,69,80,70,80,73,80,74,80,75,80,77,80,80,80,81,80,82,80,83,80,84,80,86,80,87,80,88,80,89,80,91,80,93,80,94,80,95,80,96,80,97,80,98,80,99,80,100,80,102,80,103,80,104,80,105,80,106,80,107,80,109,80,110,80,111,80,112,80,113,80,114,80,115,80,116,80,117,80,120,80,121,80,122,80,124,80,125,80,129,80,130,80,131,80,132,80,134,80,135,80,137,80,138,80,139,80,140,80,142,80,143,80,144,80,145,80,146,80,147,80,148,80,149,80,150,80,151,80,152,80,153,80,154,80,155,80,156,80,157,80,158,80,159,80,160,80,161,80,162,80,164,80,166,80,170,80,171,80,173,80,174,80,175,80,176,80,177,80,179,80,180,80,181,80,182,80,183,80,184,80,185,80,188,80,189,80,190,80,191,80,192,80,193,80,194,80,195,80,196,80,197,80,198,80,199,80,200,80,201,80,202,80,203,80,204,80,205,80,206,80,208,80,209,80,210,80,211,80,212,80,213,80,215,80,216,80,217,80,219,80,220,80,221,80,222,80,223,80,224,80,225,80,226,80,227,80,228,80,229,80,232,80,233,80,234,80,235,80,239,80,240,80,241,80,242,80,244,80,246,80,247,80,248,80,249,80,250,80,252,80,253,80,254,80,255,80,0,81,1,81,2,81,3,81,4,81,5,81,8,81,9,81,10,81,12,81,13,81,14,81,15,81,16,81,17,81,19,81,20,81,21,81,22,81,23,81,24,81,25,81,26,81,27,81,28,81,29,81,30,81,31,81,32,81,34,81,35,81,36,81,37,81,38,81,39,81,40,81,41,81,42,81,43,81,44,81,45,81,46,81,47,81,48,81,49,81,50,81,51,81,52,81,53,81,54,81,55,81,56,81,57,81,58,81,59,81,60,81,61,81,62,81,66,81,71,81,74,81,76,81,78,81,79,81,80,81,82,81,83,81,87,81,88,81,89,81,91,81,93,81,94,81,95,81,96,81,97,81,99,81,100,81,102,81,103,81,105,81,106,81,111,81,114,81,122,81,126,81,127,81,131,81,132,81,134,81,135,81,138,81,139,81,142,81,143,81,144,81,145,81,147,81,148,81,152,81,154,81,157,81,158,81,159,81,161,81,163,81,166,81,167,81,168,81,169,81,170,81,173,81,174,81,180,81,184,81,185,81,186,81,190,81,191,81,193,81,194,81,195,81,197,81,200,81,202,81,205,81,206,81,208,81,210,81,211,81,212,81,213,81,214,81,215,81,216,81,217,81,218,81,220,81,222,81,223,81,226,81,227,81,229,81,230,81,231,81,232,81,233,81,234,81,236,81,238,81,241,81,242,81,244,81,247,81,254,81,4,82,5,82,9,82,11,82,12,82,15,82,16,82,19,82,20,82,21,82,28,82,30,82,31,82,33,82,34,82,35,82,37,82,38,82,39,82,42,82,44,82,47,82,49,82,50,82,52,82,53,82,60,82,62,82,68,82,69,82,70,82,71,82,72,82,73,82,75,82,78,82,79,82,82,82,83,82,85,82,87,82,88,82,89,82,90,82,91,82,93,82,95,82,96,82,98,82,99,82,100,82,102,82,104,82,107,82,108,82,109,82,110,82,112,82,113,82,115,82,116,82,117,82,118,82,119,82,120,82,121,82,122,82,123,82,124,82,126,82,128,82,131,82,132,82,133,82,134,82,135,82,137,82,138,82,139,82,140,82,141,82,142,82,143,82,145,82,146,82,148,82,149,82,150,82,151,82,152,82,153,82,154,82,156,82,164,82,165,82,166,82,167,82,174,82,175,82,176,82,180,82,181,82,182,82,183,82,184,82,185,82,186,82,187,82,188,82,189,82,192,82,193,82,194,82,196,82,197,82,198,82,200,82,202,82,204,82,205,82,206,82,207,82,209,82,211,82,212,82,213,82,215,82,217,82,218,82,219,82,220,82,221,82,222,82,224,82,225,82,226,82,227,82,229,82,230,82,231,82,232,82,233,82,234,82,235,82,236,82,237,82,238,82,239,82,241,82,242,82,243,82,244,82,245,82,246,82,247,82,248,82,251,82,252,82,253,82,1,83,2,83,3,83,4,83,7,83,9,83,10,83,11,83,12,83,14,83,17,83,18,83,19,83,20,83,24,83,27,83,28,83,30,83,31,83,34,83,36,83,37,83,39,83,40,83,41,83,43,83,44,83,45,83,47,83,48,83,49,83,50,83,51,83,52,83,53,83,54,83,55,83,56,83,60,83,61,83,64,83,66,83,68,83,70,83,75,83,76,83,77,83,80,83,84,83,88,83,89,83,91,83,93,83,101,83,104,83,106,83,108,83,109,83,114,83,118,83,121,83,123,83,124,83,125,83,126,83,128,83,129,83,131,83,135,83,136,83,138,83,142,83,143,83,144,83,145,83,146,83,147,83,148,83,150,83,151,83,153,83,155,83,156,83,158,83,160,83,161,83,164,83,167,83,170,83,171,83,172,83,173,83,175,83,176,83,177,83,178,83,179,83,180,83,181,83,183,83,184,83,185,83,186,83,188,83,189,83,190,83,192,83,195,83,196,83,197,83,198,83,199,83,206,83,207,83,208,83,210,83,211,83,213,83,218,83,220,83,221,83,222,83,225,83,226,83,231,83,244,83,250,83,254,83,255,83,0,84,2,84,5,84,7,84,11,84,20,84,24,84,25,84,26,84,28,84,34,84,36,84,37,84,42,84,48,84,51,84,54,84,55,84,58,84,61,84,63,84,65,84,66,84,68,84,69,84,71,84,73,84,76,84,77,84,78,84,79,84,81,84,90,84,93,84,94,84,95,84,96,84,97,84,99,84,101,84,103,84,105,84,106,84,107,84,108,84,109,84,110,84,111,84,112,84,116,84,121,84,122,84,126,84,127,84,129,84,131,84,133,84,135,84,136,84,137,84,138,84,141,84,145,84,147,84,151,84,152,84,156,84,158,84,159,84,160,84,161,84,162,84,165,84,174,84,176,84,178,84,181,84,182,84,183,84,185,84,186,84,188,84,190,84,195,84,197,84,202,84,203,84,214,84,216,84,219,84,224,84,225,84,226,84,227,84,228,84,235,84,236,84,239,84,240,84,241,84,244,84,245,84,246,84,247,84,248,84,249,84,251,84,254,84,0,85,2,85,3,85,4,85,5,85,8,85,10,85,11,85,12,85,13,85,14,85,18,85,19,85,21,85,22,85,23,85,24,85,25,85,26,85,28,85,29,85,30,85,31,85,33,85,37,85,38,85,40,85,41,85,43,85,45,85,50,85,52,85,53,85,54,85,56,85,57,85,58,85,59,85,61,85,64,85,66,85,69,85,71,85,72,85,75,85,76,85,77,85,78,85,79,85,81,85,82,85,83,85,84,85,87,85,88,85,89,85,90,85,91,85,93,85,94,85,95,85,96,85,98,85,99,85,104,85,105,85,107,85,111,85,112,85,113,85,114,85,115,85,116,85,121,85,122,85,125,85,127,85,133,85,134,85,140,85,141,85,142,85,144,85,146,85,147,85,149,85,150,85,151,85,154,85,155,85,158,85,160,85,161,85,162,85,163,85,164,85,165,85,166,85,168,85,169,85,170,85,171,85,172,85,173,85,174,85,175,85,176,85,178,85,180,85,182,85,184,85,186,85,188,85,191,85,192,85,193,85,194,85,195,85,198,85,199,85,200,85,202,85,203,85,206,85,207,85,208,85,213,85,215,85,216,85,217,85,218,85,219,85,222,85,224,85,226,85,231,85,233,85,237,85,238,85,240,85,241,85,244,85,246,85,248,85,249,85,250,85,251,85,252,85,255,85,2,86,3,86,4,86,5,86,6,86,7,86,10,86,11,86,13,86,16,86,17,86,18,86,19,86,20,86,21,86,22,86,23,86,25,86,26,86,28,86,29,86,32,86,33,86,34,86,37,86,38,86,40,86,41,86,42,86,43,86,46,86,47,86,48,86,51,86,53,86,55,86,56,86,58,86,60,86,61,86,62,86,64,86,65,86,66,86,67,86,68,86,69,86,70,86,71,86,72,86,73,86,74,86,75,86,79,86,80,86,81,86,82,86,83,86,85,86,86,86,90,86,91,86,93,86,94,86,95,86,96,86,97,86,99,86,101,86,102,86,103,86,109,86,110,86,111,86,112,86,114,86,115,86,116,86,117,86,119,86,120,86,121,86,122,86,125,86,126,86,127,86,128,86,129,86,130,86,131,86,132,86,135,86,136,86,137,86,138,86,139,86,140,86,141,86,144,86,145,86,146,86,148,86,149,86,150,86,151,86,152,86,153,86,154,86,155,86,156,86,157,86,158,86,159,86,160,86,161,86,162,86,164,86,165,86,166,86,167,86,168,86,169,86,170,86,171,86,172,86,173,86,174,86,176,86,177,86,178,86,179,86,180,86,181,86,182,86,184,86,185,86,186,86,187,86,189,86,190,86,191,86,192,86,193,86,194,86,195,86,196,86,197,86,198,86,199,86,200,86,201,86,203,86,204,86,205,86,206,86,207,86,208,86,209,86,210,86,211,86,213,86,214,86,216,86,217,86,220,86,227,86,229,86,230,86,231,86,232,86,233,86,234,86,236,86,238,86,239,86,242,86,243,86,246,86,247,86,248,86,251,86,252,86,0,87,1,87,2,87,5,87,7,87,11,87,12,87,13,87,14,87,15,87,16,87,17,87,18,87,19,87,20,87,21,87,22,87,23,87,24,87,25,87,26,87,27,87,29,87,30,87,32,87,33,87,34,87,36,87,37,87,38,87,39,87,43,87,49,87,50,87,52,87,53,87,54,87,55,87,56,87,60,87,61,87,63,87,65,87,67,87,68,87,69,87,70,87,72,87,73,87,75,87,82,87,83,87,84,87,85,87,86,87,88,87,89,87,98,87,99,87,101,87,103,87,108,87,110,87,112,87,113,87,114,87,116,87,117,87,120,87,121,87,122,87,125,87,126,87,127,87,128,87,129,87,135,87,136,87,137,87,138,87,141,87,142,87,143,87,144,87,145,87,148,87,149,87,150,87,151,87,152,87,153,87,154,87,156,87,157,87,158,87,159,87,165,87,168,87,170,87,172,87,175,87,176,87,177,87,179,87,181,87,182,87,183,87,185,87,186,87,187,87,188,87,189,87,190,87,191,87,192,87,193,87,196,87,197,87,198,87,199,87,200,87,201,87,202,87,204,87,205,87,208,87,209,87,211,87,214,87,215,87,219,87,220,87,222,87,225,87,226,87,227,87,229,87,230,87,231,87,232,87,233,87,234,87,235,87,236,87,238,87,240,87,241,87,242,87,243,87,245,87,246,87,247,87,251,87,252,87,254,87,255,87,1,88,3,88,4,88,5,88,8,88,9,88,10,88,12,88,14,88,15,88,16,88,18,88,19,88,20,88,22,88,23,88,24,88,26,88,27,88,28,88,29,88,31,88,34,88,35,88,37,88,38,88,39,88,40,88,41,88,43,88,44,88,45,88,46,88,47,88,49,88,50,88,51,88,52,88,54,88,55,88,56,88,57,88,58,88,59,88,60,88,61,88,62,88,63,88,64,88,65,88,66,88,67,88,69,88,70,88,71,88,72,88,73,88,74,88,75,88,78,88,79,88,80,88,82,88,83,88,85,88,86,88,87,88,89,88,90,88,91,88,92,88,93,88,95,88,96,88,97,88,98,88,99,88,100,88,102,88,103,88,104,88,105,88,106,88,109,88,110,88,111,88,112,88,113,88,114,88,115,88,116,88,117,88,118,88,119,88,120,88,121,88,122,88,123,88,124,88,125,88,127,88,130,88,132,88,134,88,135,88,136,88,138,88,139,88,140,88,141,88,142,88,143,88,144,88,145,88,148,88,149,88,150,88,151,88,152,88,155,88,156,88,157,88,160,88,161,88,162,88,163,88,164,88,165,88,166,88,167,88,170,88,171,88,172,88,173,88,174,88,175,88,176,88,177,88,178,88,179,88,180,88,181,88,182,88,183,88,184,88,185,88,186,88,187,88,189,88,190,88,191,88,192,88,194,88,195,88,196,88,198,88,199,88,200,88,201,88,202,88,203,88,204,88,205,88,206,88,207,88,208,88,210,88,211,88,212,88,214,88,215,88,216,88,217,88,218,88,219,88,220,88,221,88,222,88,223,88,224,88,225,88,226,88,227,88,229,88,230,88,231,88,232,88,233,88,234,88,237,88,239,88,241,88,242,88,244,88,245,88,247,88,248,88,250,88,251,88,252,88,253,88,254,88,255,88,0,89,1,89,3,89,5,89,6,89,8,89,9,89,10,89,11,89,12,89,14,89,16,89,17,89,18,89,19,89,23,89,24,89,27,89,29,89,30,89,32,89,33,89,34,89,35,89,38,89,40,89,44,89,48,89,50,89,51,89,53,89,54,89,59,89,61,89,62,89,63,89,64,89,67,89,69,89,70,89,74,89,76,89,77,89,80,89,82,89,83,89,89,89,91,89,92,89,93,89,94,89,95,89,97,89,99,89,100,89,102,89,103,89,104,89,105,89,106,89,107,89,108,89,109,89,110,89,111,89,112,89,113,89,114,89,117,89,119,89,122,89,123,89,124,89,126,89,127,89,128,89,133,89,137,89,139,89,140,89,142,89,143,89,144,89,145,89,148,89,149,89,152,89,154,89,155,89,156,89,157,89,159,89,160,89,161,89,162,89,166,89,167,89,172,89,173,89,176,89,177,89,179,89,180,89,181,89,182,89,183,89,184,89,186,89,188,89,189,89,191,89,192,89,193,89,194,89,195,89,196,89,197,89,199,89,200,89,201,89,204,89,205,89,206,89,207,89,213,89,214,89,217,89,219,89,222,89,223,89,224,89,225,89,226,89,228,89,230,89,231,89,233,89,234,89,235,89,237,89,238,89,239,89,240,89,241,89,242,89,243,89,244,89,245,89,246,89,247,89,248,89,250,89,252,89,253,89,254,89,0,90,2,90,10,90,11,90,13,90,14,90,15,90,16,90,18,90,20,90,21,90,22,90,23,90,25,90,26,90,27,90,29,90,30,90,33,90,34,90,36,90,38,90,39,90,40,90,42,90,43,90,44,90,45,90,46,90,47,90,48,90,51,90,53,90,55,90,56,90,57,90,58,90,59,90,61,90,62,90,63,90,65,90,66,90,67,90,68,90,69,90,71,90,72,90,75,90,76,90,77,90,78,90,79,90,80,90,81,90,82,90,83,90,84,90,86,90,87,90,88,90,89,90,91,90,92,90,93,90,94,90,95,90,96,90,97,90,99,90,100,90,101,90,102,90,104,90,105,90,107,90,108,90,109,90,110,90,111,90,112,90,113,90,114,90,115,90,120,90,121,90,123,90,124,90,125,90,126,90,128,90,129,90,130,90,131,90,132,90,133,90,134,90,135,90,136,90,137,90,138,90,139,90,140,90,141,90,142,90,143,90,144,90,145,90,147,90,148,90,149,90,150,90,151,90,152,90,153,90,156,90,157,90,158,90,159,90,160,90,161,90,162,90,163,90,164,90,165,90,166,90,167,90,168,90,169,90,171,90,172,90,173,90,174,90,175,90,176,90,177,90,180,90,182,90,183,90,185,90,186,90,187,90,188,90,189,90,191,90,192,90,195,90,196,90,197,90,198,90,199,90,200,90,202,90,203,90,205,90,206,90,207,90,208,90,209,90,211,90,213,90,215,90,217,90,218,90,219,90,221,90,222,90,223,90,226,90,228,90,229,90,231,90,232,90,234,90,236,90,237,90,238,90,239,90,240,90,242,90,243,90,244,90,245,90,246,90,247,90,248,90,249,90,250,90,251,90,252,90,253,90,254,90,255,90,0,91,1,91,2,91,3,91,4,91,5,91,6,91,7,91,8,91,10,91,11,91,12,91,13,91,14,91,15,91,16,91,17,91,18,91,19,91,20,91,21,91,24,91,25,91,26,91,27,91,28,91,29,91,30,91,31,91,32,91,33,91,34,91,35,91,36,91,37,91,38,91,39,91,40,91,41,91,42,91,43,91,44,91,45,91,46,91,47,91,48,91,49,91,51,91,53,91,54,91,56,91,57,91,58,91,59,91,60,91,61,91,62,91,63,91,65,91,66,91,67,91,68,91,69,91,70,91,71,91,72,91,73,91,74,91,75,91,76,91,77,91,78,91,79,91,82,91,86,91,94,91,96,91,97,91,103,91,104,91,107,91,109,91,110,91,111,91,114,91,116,91,118,91,119,91,120,91,121,91,123,91,124,91,126,91,127,91,130,91,134,91,138,91,141,91,142,91,144,91,145,91,146,91,148,91,150,91,159,91,167,91,168,91,169,91,172,91,173,91,174,91,175,91,177,91,178,91,183,91,186,91,187,91,188,91,192,91,193,91,195,91,200,91,201,91,202,91,203,91,205,91,206,91,207,91,209,91,212,91,213,91,214,91,215,91,216,91,217,91,218,91,219,91,220,91,224,91,226,91,227,91,230,91,231,91,233,91,234,91,235,91,236,91,237,91,239,91,241,91,242,91,243,91,244,91,245,91,246,91,247,91,253,91,254,91,0,92,2,92,3,92,5,92,7,92,8,92,11,92,12,92,13,92,14,92,16,92,18,92,19,92,23,92,25,92,27,92,30,92,31,92,32,92,33,92,35,92,38,92,40,92,41,92,42,92,43,92,45,92,46,92,47,92,48,92,50,92,51,92,53,92,54,92,55,92,67,92,68,92,70,92,71,92,76,92,77,92,82,92,83,92,84,92,86,92,87,92,88,92,90,92,91,92,92,92,93,92,95,92,98,92,100,92,103,92,104,92,105,92,106,92,107,92,108,92,109,92,112,92,114,92,115,92,116,92,117,92,118,92,119,92,120,92,123,92,124,92,125,92,126,92,128,92,131,92,132,92,133,92,134,92,135,92,137,92,138,92,139,92,142,92,143,92,146,92,147,92,149,92,157,92,158,92,159,92,160,92,161,92,164,92,165,92,166,92,167,92,168,92,170,92,174,92,175,92,176,92,178,92,180,92,182,92,185,92,186,92,187,92,188,92,190,92,192,92,194,92,195,92,197,92,198,92,199,92,200,92,201,92,202,92,204,92,205,92,206,92,207,92,208,92,209,92,211,92,212,92,213,92,214,92,215,92,216,92,218,92,219,92,220,92,221,92,222,92,223,92,224,92,226,92,227,92,231,92,233,92,235,92,236,92,238,92,239,92,241,92,242,92,243,92,244,92,245,92,246,92,247,92,248,92,249,92,250,92,252,92,253,92,254,92,255,92,0,93,1,93,4,93,5,93,8,93,9,93,10,93,11,93,12,93,13,93,15,93,16,93,17,93,18,93,19,93,21,93,23,93,24,93,25,93,26,93,28,93,29,93,31,93,32,93,33,93,34,93,35,93,37,93,40,93,42,93,43,93,44,93,47,93,48,93,49,93,50,93,51,93,53,93,54,93,55,93,56,93,57,93,58,93,59,93,60,93,63,93,64,93,65,93,66,93,67,93,68,93,69,93,70,93,72,93,73,93,77,93,78,93,79,93,80,93,81,93,82,93,83,93,84,93,85,93,86,93,87,93,89,93,90,93,92,93,94,93,95,93,96,93,97,93,98,93,99,93,100,93,101,93,102,93,103,93,104,93,106,93,109,93,110,93,112,93,113,93,114,93,115,93,117,93,118,93,119,93,120,93,121,93,122,93,123,93,124,93,125,93,126,93,127,93,128,93,129,93,131,93,132,93,133,93,134,93,135,93,136,93,137,93,138,93,139,93,140,93,141,93,142,93,143,93,144,93,145,93,146,93,147,93,148,93,149,93,150,93,151,93,152,93,154,93,155,93,156,93,158,93,159,93,160,93,161,93,162,93,163,93,164,93,165,93,166,93,167,93,168,93,169,93,170,93,171,93,172,93,173,93,174,93,175,93,176,93,177,93,178,93,179,93,180,93,181,93,182,93,184,93,185,93,186,93,187,93,188,93,189,93,190,93,191,93,192,93,193,93,194,93,195,93,196,93,198,93,199,93,200,93,201,93,202,93,203,93,204,93,206,93,207,93,208,93,209,93,210,93,211,93,212,93,213,93,214,93,215,93,216,93,217,93,218,93,220,93,223,93,224,93,227,93,228,93,234,93,236,93,237,93,240,93,245,93,246,93,248,93,249,93,250,93,251,93,252,93,255,93,0,94,4,94,7,94,9,94,10,94,11,94,13,94,14,94,18,94,19,94,23,94,30,94,31,94,32,94,33,94,34,94,35,94,36,94,37,94,40,94,41,94,42,94,43,94,44,94,47,94,48,94,50,94,51,94,52,94,53,94,54,94,57,94,58,94,62,94,63,94,64,94,65,94,67,94,70,94,71,94,72,94,73,94,74,94,75,94,77,94,78,94,79,94,80,94,81,94,82,94,83,94,86,94,87,94,88,94,89,94,90,94,92,94,93,94,95,94,96,94,99,94,100,94,101,94,102,94,103,94,104,94,105,94,106,94,107,94,108,94,109,94,110,94,111,94,112,94,113,94,117,94,119,94,121,94,126,94,129,94,130,94,131,94,133,94,136,94,137,94,140,94,141,94,142,94,146,94,152,94,155,94,157,94,161,94,162,94,163,94,164,94,168,94,169,94,170,94,171,94,172,94,174,94,175,94,176,94,177,94,178,94,180,94,186,94,187,94,188,94,189,94,191,94,192,94,193,94,194,94,195,94,196,94,197,94,198,94,199,94,200,94,203,94,204,94,205,94,206,94,207,94,208,94,212,94,213,94,215,94,216,94,217,94,218,94,220,94,221,94,222,94,223,94,224,94,225,94,226,94,227,94,228,94,229,94,230,94,231,94,233,94,235,94,236,94,237,94,238,94,239,94,240,94,241,94,242,94,243,94,245,94,248,94,249,94,251,94,252,94,253,94,5,95,6,95,7,95,9,95,12,95,13,95,14,95,16,95,18,95,20,95,22,95,25,95,26,95,28,95,29,95,30,95,33,95,34,95,35,95,36,95,40,95,43,95,44,95,46,95,48,95,50,95,51,95,52,95,53,95,54,95,55,95,56,95,59,95,61,95,62,95,63,95,65,95,66,95,67,95,68,95,69,95,70,95,71,95,72,95,73,95,74,95,75,95,76,95,77,95,78,95,79,95,81,95,84,95,89,95,90,95,91,95,92,95,94,95,95,95,96,95,99,95,101,95,103,95,104,95,107,95,110,95,111,95,114,95,116,95,117,95,118,95,120,95,122,95,125,95,126,95,127,95,131,95,134,95,141,95,142,95,143,95,145,95,147,95,148,95,150,95,154,95,155,95,157,95,158,95,159,95,160,95,162,95,163,95,164,95,165,95,166,95,167,95,169,95,171,95,172,95,175,95,176,95,177,95,178,95,179,95,180,95,182,95,184,95,185,95,186,95,187,95,190,95,191,95,192,95,193,95,194,95,199,95,200,95,202,95,203,95,206,95,211,95,212,95,213,95,218,95,219,95,220,95,222,95,223,95,226,95,227,95,229,95,230,95,232,95,233,95,236,95,239,95,240,95,242,95,243,95,244,95,246,95,247,95,249,95,250,95,252,95,7,96,8,96,9,96,11,96,12,96,16,96,17,96,19,96,23,96,24,96,26,96,30,96,31,96,34,96,35,96,36,96,44,96,45,96,46,96,48,96,49,96,50,96,51,96,52,96,54,96,55,96,56,96,57,96,58,96,61,96,62,96,64,96,68,96,69,96,70,96,71,96,72,96,73,96,74,96,76,96,78,96,79,96,81,96,83,96,84,96,86,96,87,96,88,96,91,96,92,96,94,96,95,96,96,96,97,96,101,96,102,96,110,96,113,96,114,96,116,96,117,96,119,96,126,96,128,96,129,96,130,96,133,96,134,96,135,96,136,96,138,96,139,96,142,96,143,96,144,96,145,96,147,96,149,96,151,96,152,96,153,96,156,96,158,96,161,96,162,96,164,96,165,96,167,96,169,96,170,96,174,96,176,96,179,96,181,96,182,96,183,96,185,96,186,96,189,96,190,96,191,96,192,96,193,96,194,96,195,96,196,96,199,96,200,96,201,96,204,96,205,96,206,96,207,96,208,96,210,96,211,96,212,96,214,96,215,96,217,96,219,96,222,96,225,96,226,96,227,96,228,96,229,96,234,96,241,96,242,96,245,96,247,96,248,96,251,96,252,96,253,96,254,96,255,96,2,97,3,97,4,97,5,97,7,97,10,97,11,97,12,97,16,97,17,97,18,97,19,97,20,97,22,97,23,97,24,97,25,97,27,97,28,97,29,97,30,97,33,97,34,97,37,97,40,97,41,97,42,97,44,97,45,97,46,97,47,97,48,97,49,97,50,97,51,97,52,97,53,97,54,97,55,97,56,97,57,97,58,97,59,97,60,97,61,97,62,97,64,97,65,97,66,97,67,97,68,97,69,97,70,97,71,97,73,97,75,97,77,97,79,97,80,97,82,97,83,97,84,97,86,97,87,97,88,97,89,97,90,97,91,97,92,97,94,97,95,97,96,97,97,97,99,97,100,97,101,97,102,97,105,97,106,97,107,97,108,97,109,97,110,97,111,97,113,97,114,97,115,97,116,97,118,97,120,97,121,97,122,97,123,97,124,97,125,97,126,97,127,97,128,97,129,97,130,97,131,97,132,97,133,97,134,97,135,97,136,97,137,97,138,97,140,97,141,97,143,97,144,97,145,97,146,97,147,97,149,97,150,97,151,97,152,97,153,97,154,97,155,97,156,97,158,97,159,97,160,97,161,97,162,97,163,97,164,97,165,97,166,97,170,97,171,97,173,97,174,97,175,97,176,97,177,97,178,97,179,97,180,97,181,97,182,97,184,97,185,97,186,97,187,97,188,97,189,97,191,97,192,97,193,97,195,97,196,97,197,97,198,97,199,97,201,97,204,97,205,97,206,97,207,97,208,97,211,97,213,97,214,97,215,97,216,97,217,97,218,97,219,97,220,97,221,97,222,97,223,97,224,97,225,97,226,97,227,97,228,97,229,97,231,97,232,97,233,97,234,97,235,97,236,97,237,97,238,97,239,97,240,97,241,97,242,97,243,97,244,97,246,97,247,97,248,97,249,97,250,97,251,97,252,97,253,97,254,97,0,98,1,98,2,98,3,98,4,98,5,98,7,98,9,98,19,98,20,98,25,98,28,98,29,98,30,98,32,98,35,98,38,98,39,98,40,98,41,98,43,98,45,98,47,98,48,98,49,98,50,98,53,98,54,98,56,98,57,98,58,98,59,98,60,98,66,98,68,98,69,98,70,98,74,98,79,98,80,98,85,98,86,98,87,98,89,98,90,98,92,98,93,98,94,98,95,98,96,98,97,98,98,98,100,98,101,98,104,98,113,98,114,98,116,98,117,98,119,98,120,98,122,98,123,98,125,98,129,98,130,98,131,98,133,98,134,98,135,98,136,98,139,98,140,98,141,98,142,98,143,98,144,98,148,98,153,98,156,98,157,98,158,98,163,98,166,98,167,98,169,98,170,98,173,98,174,98,175,98,176,98,178,98,179,98,180,98,182,98,183,98,184,98,186,98,190,98,192,98,193,98,195,98,203,98,207,98,209,98,213,98,221,98,222,98,224,98,225,98,228,98,234,98,235,98,240,98,242,98,245,98,248,98,249,98,250,98,251,98,0,99,3,99,4,99,5,99,6,99,10,99,11,99,12,99,13,99,15,99,16,99,18,99,19,99,20,99,21,99,23,99,24,99,25,99,28,99,38,99,39,99,41,99,44,99,45,99,46,99,48,99,49,99,51,99,52,99,53,99,54,99,55,99,56,99,59,99,60,99,62,99,63,99,64,99,65,99,68,99,71,99,72,99,74,99,81,99,82,99,83,99,84,99,86,99,87,99,88,99,89,99,90,99,91,99,92,99,93,99,96,99,100,99,101,99,102,99,104,99,106,99,107,99,108,99,111,99,112,99,114,99,115,99,116,99,117,99,120,99,121,99,124,99,125,99,126,99,127,99,129,99,131,99,132,99,133,99,134,99,139,99,141,99,145,99,147,99,148,99,149,99,151,99,153,99,154,99,155,99,156,99,157,99,158,99,159,99,161,99,164,99,166,99,171,99,175,99,177,99,178,99,181,99,182,99,185,99,187,99,189,99,191,99,192,99,193,99,194,99,195,99,197,99,199,99,200,99,202,99,203,99,204,99,209,99,211,99,212,99,213,99,215,99,216,99,217,99,218,99,219,99,220,99,221,99,223,99,226,99,228,99,229,99,230,99,231,99,232,99,235,99,236,99,238,99,239,99,240,99,241,99,243,99,245,99,247,99,249,99,250,99,251,99,252,99,254,99,3,100,4,100,6,100,7,100,8,100,9,100,10,100,13,100,14,100,17,100,18,100,21,100,22,100,23,100,24,100,25,100,26,100,29,100,31,100,34,100,35,100,36,100,37,100,39,100,40,100,41,100,43,100,46,100,47,100,48,100,49,100,50,100,51,100,53,100,54,100,55,100,56,100,57,100,59,100,60,100,62,100,64,100,66,100,67,100,73,100,75,100,76,100,77,100,78,100,79,100,80,100,81,100,83,100,85,100,86,100,87,100,89,100,90,100,91,100,92,100,93,100,95,100,96,100,97,100,98,100,99,100,100,100,101,100,102,100,104,100,106,100,107,100,108,100,110,100,111,100,112,100,113,100,114,100,115,100,116,100,117,100,118,100,119,100,123,100,124,100,125,100,126,100,127,100,128,100,129,100,131,100,134,100,136,100,137,100,138,100,139,100,140,100,141,100,142,100,143,100,144,100,147,100,148,100,151,100,152,100,154,100,155,100,156,100,157,100,159,100,160,100,161,100,162,100,163,100,165,100,166,100,167,100,168,100,170,100,171,100,175,100,177,100,178,100,179,100,180,100,182,100,185,100,187,100,189,100,190,100,191,100,193,100,195,100,196,100,198,100,199,100,200,100,201,100,202,100,203,100,204,100,207,100,209,100,211,100,212,100,213,100,214,100,217,100,218,100,219,100,220,100,221,100,223,100,224,100,225,100,227,100,229,100,231,100,232,100,233,100,234,100,235,100,236,100,237,100,238,100,239,100,240,100,241,100,242,100,243,100,244,100,245,100,246,100,247,100,248,100,249,100,250,100,251,100,252,100,253,100,254,100,255,100,1,101,2,101,3,101,4,101,5,101,6,101,7,101,8,101,10,101,11,101,12,101,13,101,14,101,15,101,16,101,17,101,19,101,20,101,21,101,22,101,23,101,25,101,26,101,27,101,28,101,29,101,30,101,31,101,32,101,33,101,34,101,35,101,36,101,38,101,39,101,40,101,41,101,42,101,44,101,45,101,48,101,49,101,50,101,51,101,55,101,58,101,60,101,61,101,64,101,65,101,66,101,67,101,68,101,70,101,71,101,74,101,75,101,77,101,78,101,80,101,82,101,83,101,84,101,87,101,88,101,90,101,92,101,95,101,96,101,97,101,100,101,101,101,103,101,104,101,105,101,106,101,109,101,110,101,111,101,113,101,115,101,117,101,118,101,120,101,121,101,122,101,123,101,124,101,125,101,126,101,127,101,128,101,129,101,130,101,131,101,132,101,133,101,134,101,136,101,137,101,138,101,141,101,142,101,143,101,146,101,148,101,149,101,150,101,152,101,154,101,157,101,158,101,160,101,162,101,163,101,166,101,168,101,170,101,172,101,174,101,177,101,178,101,179,101,180,101,181,101,182,101,183,101,184,101,186,101,187,101,190,101,191,101,192,101,194,101,199,101,200,101,201,101,202,101,205,101,208,101,209,101,211,101,212,101,213,101,216,101,217,101,218,101,219,101,220,101,221,101,222,101,223,101,225,101,227,101,228,101,234,101,235,101,242,101,243,101,244,101,245,101,248,101,249,101,251,101,252,101,253,101,254,101,255,101,1,102,4,102,5,102,7,102,8,102,9,102,11,102,13,102,16,102,17,102,18,102,22,102,23,102,24,102,26,102,27,102,28,102,30,102,33,102,34,102,35,102,36,102,38,102,41,102,42,102,43,102,44,102,46,102,48,102,50,102,51,102,55,102,56,102,57,102,58,102,59,102,61,102,63,102,64,102,66,102,68,102,69,102,70,102,71,102,72,102,73,102,74,102,77,102,78,102,80,102,81,102,88,102,89,102,91,102,92,102,93,102,94,102,96,102,98,102,99,102,101,102,103,102,105,102,106,102,107,102,108,102,109,102,113,102,114,102,115,102,117,102,120,102,121,102,123,102,124,102,125,102,127,102,128,102,129,102,131,102,133,102,134,102,136,102,137,102,138,102,139,102,141,102,142,102,143,102,144,102,146,102,147,102,148,102,149,102,152,102,153,102,154,102,155,102,156,102,158,102,159,102,160,102,161,102,162,102,163,102,164,102,165,102,166,102,169,102,170,102,171,102,172,102,173,102,175,102,176,102,177,102,178,102,179,102,181,102,182,102,183,102,184,102,186,102,187,102,188,102,189,102,191,102,192,102,193,102,194,102,195,102,196,102,197,102,198,102,199,102,200,102,201,102,202,102,203,102,204,102,205,102,206,102,207,102,208,102,209,102,210,102,211,102,212,102,213,102,214,102,215,102,216,102,218,102,222,102,223,102,224,102,225,102,226,102,227,102,228,102,229,102,231,102,232,102,234,102,235,102,236,102,237,102,238,102,239,102,241,102,245,102,246,102,248,102,250,102,251,102,253,102,1,103,2,103,3,103,4,103,5,103,6,103,7,103,12,103,14,103,15,103,17,103,18,103,19,103,22,103,24,103,25,103,26,103,28,103,30,103,32,103,33,103,34,103,35,103,36,103,37,103,39,103,41,103,46,103,48,103,50,103,51,103,54,103,55,103,56,103,57,103,59,103,60,103,62,103,63,103,65,103,68,103,69,103,71,103,74,103,75,103,77,103,82,103,84,103,85,103,87,103,88,103,89,103,90,103,91,103,93,103,98,103,99,103,100,103,102,103,103,103,107,103,108,103,110,103,113,103,116,103,118,103,120,103,121,103,122,103,123,103,125,103,128,103,130,103,131,103,133,103,134,103,136,103,138,103,140,103,141,103,142,103,143,103,145,103,146,103,147,103,148,103,150,103,153,103,155,103,159,103,160,103,161,103,164,103,166,103,169,103,172,103,174,103,177,103,178,103,180,103,185,103,186,103,187,103,188,103,189,103,190,103,191,103,192,103,194,103,197,103,198,103,199,103,200,103,201,103,202,103,203,103,204,103,205,103,206,103,213,103,214,103,215,103,219,103,223,103,225,103,227,103,228,103,230,103,231,103,232,103,234,103,235,103,237,103,238,103,242,103,245,103,246,103,247,103,248,103,249,103,250,103,251,103,252,103,254,103,1,104,2,104,3,104,4,104,6,104,13,104,16,104,18,104,20,104,21,104,24,104,25,104,26,104,27,104,28,104,30,104,31,104,32,104,34,104,35,104,36,104,37,104,38,104,39,104,40,104,43,104,44,104,45,104,46,104,47,104,48,104,49,104,52,104,53,104,54,104,58,104,59,104,63,104,71,104,75,104,77,104,79,104,82,104,86,104,87,104,88,104,89,104,90,104,91,104,92,104,93,104,94,104,95,104,106,104,108,104,109,104,110,104,111,104,112,104,113,104,114,104,115,104,117,104,120,104,121,104,122,104,123,104,124,104,125,104,126,104,127,104,128,104,130,104,132,104,135,104,136,104,137,104,138,104,139,104,140,104,141,104,142,104,144,104,145,104,146,104,148,104,149,104,150,104,152,104,153,104,154,104,155,104,156,104,157,104,158,104,159,104,160,104,161,104,163,104,164,104,165,104,169,104,170,104,171,104,172,104,174,104,177,104,178,104,180,104,182,104,183,104,184,104,185,104,186,104,187,104,188,104,189,104,190,104,191,104,193,104,195,104,196,104,197,104,198,104,199,104,200,104,202,104,204,104,206,104,207,104,208,104,209,104,211,104,212,104,214,104,215,104,217,104,219,104,220,104,221,104,222,104,223,104,225,104,226,104,228,104,229,104,230,104,231,104,232,104,233,104,234,104,235,104,236,104,237,104,239,104,242,104,243,104,244,104,246,104,247,104,248,104,251,104,253,104,254,104,255,104,0,105,2,105,3,105,4,105,6,105,7,105,8,105,9,105,10,105,12,105,15,105,17,105,19,105,20,105,21,105,22,105,23,105,24,105,25,105,26,105,27,105,28,105,29,105,30,105,33,105,34,105,35,105,37,105,38,105,39,105,40,105,41,105,42,105,43,105,44,105,46,105,47,105,49,105,50,105,51,105,53,105,54,105,55,105,56,105,58,105,59,105,60,105,62,105,64,105,65,105,67,105,68,105,69,105,70,105,71,105,72,105,73,105,74,105,75,105,76,105,77,105,78,105,79,105,80,105,81,105,82,105,83,105,85,105,86,105,88,105,89,105,91,105,92,105,95,105,97,105,98,105,100,105,101,105,103,105,104,105,105,105,106,105,108,105,109,105,111,105,112,105,114,105,115,105,116,105,117,105,118,105,122,105,123,105,125,105,126,105,127,105,129,105,131,105,133,105,138,105,139,105,140,105,142,105,143,105,144,105,145,105,146,105,147,105,150,105,151,105,153,105,154,105,157,105,158,105,159,105,160,105,161,105,162,105,163,105,164,105,165,105,166,105,169,105,170,105,172,105,174,105,175,105,176,105,178,105,179,105,181,105,182,105,184,105,185,105,186,105,188,105,189,105,190,105,191,105,192,105,194,105,195,105,196,105,197,105,198,105,199,105,200,105,201,105,203,105,205,105,207,105,209,105,210,105,211,105,213,105,214,105,215,105,216,105,217,105,218,105,220,105,221,105,222,105,225,105,226,105,227,105,228,105,229,105,230,105,231,105,232,105,233,105,234,105,235,105,236,105,238,105,239,105,240,105,241,105,243,105,244,105,245,105,246,105,247,105,248,105,249,105,250,105,251,105,252,105,254,105,0,106,1,106,2,106,3,106,4,106,5,106,6,106,7,106,8,106,9,106,11,106,12,106,13,106,14,106,15,106,16,106,17,106,18,106,19,106,20,106,21,106,22,106,25,106,26,106,27,106,28,106,29,106,30,106,32,106,34,106,35,106,36,106,37,106,38,106,39,106,41,106,43,106,44,106,45,106,46,106,48,106,50,106,51,106,52,106,54,106,55,106,56,106,57,106,58,106,59,106,60,106,63,106,64,106,65,106,66,106,67,106,69,106,70,106])
.concat([72,106,73,106,74,106,75,106,76,106,77,106,78,106,79,106,81,106,82,106,83,106,84,106,85,106,86,106,87,106,90,106,92,106,93,106,94,106,95,106,96,106,98,106,99,106,100,106,102,106,103,106,104,106,105,106,106,106,107,106,108,106,109,106,110,106,111,106,112,106,114,106,115,106,116,106,117,106,118,106,119,106,120,106,122,106,123,106,125,106,126,106,127,106,129,106,130,106,131,106,133,106,134,106,135,106,136,106,137,106,138,106,139,106,140,106,141,106,143,106,146,106,147,106,148,106,149,106,150,106,152,106,153,106,154,106,155,106,156,106,157,106,158,106,159,106,161,106,162,106,163,106,164,106,165,106,166,106,167,106,168,106,170,106,173,106,174,106,175,106,176,106,177,106,178,106,179,106,180,106,181,106,182,106,183,106,184,106,185,106,186,106,187,106,188,106,189,106,190,106,191,106,192,106,193,106,194,106,195,106,196,106,197,106,198,106,199,106,200,106,201,106,202,106,203,106,204,106,205,106,206,106,207,106,208,106,209,106,210,106,211,106,212,106,213,106,214,106,215,106,216,106,217,106,218,106,219,106,220,106,221,106,222,106,223,106,224,106,225,106,226,106,227,106,228,106,229,106,230,106,231,106,232,106,233,106,234,106,235,106,236,106,237,106,238,106,239,106,240,106,241,106,242,106,243,106,244,106,245,106,246,106,247,106,248,106,249,106,250,106,251,106,252,106,253,106,254,106,255,106,0,107,1,107,2,107,3,107,4,107,5,107,6,107,7,107,8,107,9,107,10,107,11,107,12,107,13,107,14,107,15,107,16,107,17,107,18,107,19,107,20,107,21,107,22,107,23,107,24,107,25,107,26,107,27,107,28,107,29,107,30,107,31,107,37,107,38,107,40,107,41,107,42,107,43,107,44,107,45,107,46,107,47,107,48,107,49,107,51,107,52,107,53,107,54,107,56,107,59,107,60,107,61,107,63,107,64,107,65,107,66,107,68,107,69,107,72,107,74,107,75,107,77,107,78,107,79,107,80,107,81,107,82,107,83,107,84,107,85,107,86,107,87,107,88,107,90,107,91,107,92,107,93,107,94,107,95,107,96,107,97,107,104,107,105,107,107,107,108,107,109,107,110,107,111,107,112,107,113,107,114,107,115,107,116,107,117,107,118,107,119,107,120,107,122,107,125,107,126,107,127,107,128,107,133,107,136,107,140,107,142,107,143,107,144,107,145,107,148,107,149,107,151,107,152,107,153,107,156,107,157,107,158,107,159,107,160,107,162,107,163,107,164,107,165,107,166,107,167,107,168,107,169,107,171,107,172,107,173,107,174,107,175,107,176,107,177,107,178,107,182,107,184,107,185,107,186,107,187,107,188,107,189,107,190,107,192,107,195,107,196,107,198,107,199,107,200,107,201,107,202,107,204,107,206,107,208,107,209,107,216,107,218,107,220,107,221,107,222,107,223,107,224,107,226,107,227,107,228,107,229,107,230,107,231,107,232,107,233,107,236,107,237,107,238,107,240,107,241,107,242,107,244,107,246,107,247,107,248,107,250,107,251,107,252,107,254,107,255,107,0,108,1,108,2,108,3,108,4,108,8,108,9,108,10,108,11,108,12,108,14,108,18,108,23,108,28,108,29,108,30,108,32,108,35,108,37,108,43,108,44,108,45,108,49,108,51,108,54,108,55,108,57,108,58,108,59,108,60,108,62,108,63,108,67,108,68,108,69,108,72,108,75,108,76,108,77,108,78,108,79,108,81,108,82,108,83,108,86,108,88,108,89,108,90,108,98,108,99,108,101,108,102,108,103,108,107,108,108,108,109,108,110,108,111,108,113,108,115,108,117,108,119,108,120,108,122,108,123,108,124,108,127,108,128,108,132,108,135,108,138,108,139,108,141,108,142,108,145,108,146,108,149,108,150,108,151,108,152,108,154,108,156,108,157,108,158,108,160,108,162,108,168,108,172,108,175,108,176,108,180,108,181,108,182,108,183,108,186,108,192,108,193,108,194,108,195,108,198,108,199,108,200,108,203,108,205,108,206,108,207,108,209,108,210,108,216,108,217,108,218,108,220,108,221,108,223,108,228,108,230,108,231,108,233,108,236,108,237,108,242,108,244,108,249,108,255,108,0,109,2,109,3,109,5,109,6,109,8,109,9,109,10,109,13,109,15,109,16,109,17,109,19,109,20,109,21,109,22,109,24,109,28,109,29,109,31,109,32,109,33,109,34,109,35,109,36,109,38,109,40,109,41,109,44,109,45,109,47,109,48,109,52,109,54,109,55,109,56,109,58,109,63,109,64,109,66,109,68,109,73,109,76,109,80,109,85,109,86,109,87,109,88,109,91,109,93,109,95,109,97,109,98,109,100,109,101,109,103,109,104,109,107,109,108,109,109,109,112,109,113,109,114,109,115,109,117,109,118,109,121,109,122,109,123,109,125,109,126,109,127,109,128,109,129,109,131,109,132,109,134,109,135,109,138,109,139,109,141,109,143,109,144,109,146,109,150,109,151,109,152,109,153,109,154,109,156,109,162,109,165,109,172,109,173,109,176,109,177,109,179,109,180,109,182,109,183,109,185,109,186,109,187,109,188,109,189,109,190,109,193,109,194,109,195,109,200,109,201,109,202,109,205,109,206,109,207,109,208,109,210,109,211,109,212,109,213,109,215,109,218,109,219,109,220,109,223,109,226,109,227,109,229,109,231,109,232,109,233,109,234,109,237,109,239,109,240,109,242,109,244,109,245,109,246,109,248,109,250,109,253,109,254,109,255,109,0,110,1,110,2,110,3,110,4,110,6,110,7,110,8,110,9,110,11,110,15,110,18,110,19,110,21,110,24,110,25,110,27,110,28,110,30,110,31,110,34,110,38,110,39,110,40,110,42,110,44,110,46,110,48,110,49,110,51,110,53,110,54,110,55,110,57,110,59,110,60,110,61,110,62,110,63,110,64,110,65,110,66,110,69,110,70,110,71,110,72,110,73,110,74,110,75,110,76,110,79,110,80,110,81,110,82,110,85,110,87,110,89,110,90,110,92,110,93,110,94,110,96,110,97,110,98,110,99,110,100,110,101,110,102,110,103,110,104,110,105,110,106,110,108,110,109,110,111,110,112,110,113,110,114,110,115,110,116,110,117,110,118,110,119,110,120,110,121,110,122,110,123,110,124,110,125,110,128,110,129,110,130,110,132,110,135,110,136,110,138,110,139,110,140,110,141,110,142,110,145,110,146,110,147,110,148,110,149,110,150,110,151,110,153,110,154,110,155,110,157,110,158,110,160,110,161,110,163,110,164,110,166,110,168,110,169,110,171,110,172,110,173,110,174,110,176,110,179,110,181,110,184,110,185,110,188,110,190,110,191,110,192,110,195,110,196,110,197,110,198,110,200,110,201,110,202,110,204,110,205,110,206,110,208,110,210,110,214,110,216,110,217,110,219,110,220,110,221,110,227,110,231,110,234,110,235,110,236,110,237,110,238,110,239,110,240,110,241,110,242,110,243,110,245,110,246,110,247,110,248,110,250,110,251,110,252,110,253,110,254,110,255,110,0,111,1,111,3,111,4,111,5,111,7,111,8,111,10,111,11,111,12,111,13,111,14,111,16,111,17,111,18,111,22,111,23,111,24,111,25,111,26,111,27,111,28,111,29,111,30,111,31,111,33,111,34,111,35,111,37,111,38,111,39,111,40,111,44,111,46,111,48,111,50,111,52,111,53,111,55,111,56,111,57,111,58,111,59,111,60,111,61,111,63,111,64,111,65,111,66,111,67,111,68,111,69,111,72,111,73,111,74,111,76,111,78,111,79,111,80,111,81,111,82,111,83,111,84,111,85,111,86,111,87,111,89,111,90,111,91,111,93,111,95,111,96,111,97,111,99,111,100,111,101,111,103,111,104,111,105,111,106,111,107,111,108,111,111,111,112,111,113,111,115,111,117,111,118,111,119,111,121,111,123,111,125,111,126,111,127,111,128,111,129,111,130,111,131,111,133,111,134,111,135,111,138,111,139,111,143,111,144,111,145,111,146,111,147,111,148,111,149,111,150,111,151,111,152,111,153,111,154,111,155,111,157,111,158,111,159,111,160,111,162,111,163,111,164,111,165,111,166,111,168,111,169,111,170,111,171,111,172,111,173,111,174,111,175,111,176,111,177,111,178,111,180,111,181,111,183,111,184,111,186,111,187,111,188,111,189,111,190,111,191,111,193,111,195,111,196,111,197,111,198,111,199,111,200,111,202,111,203,111,204,111,205,111,206,111,207,111,208,111,211,111,212,111,213,111,214,111,215,111,216,111,217,111,218,111,219,111,220,111,221,111,223,111,226,111,227,111,228,111,229,111,230,111,231,111,232,111,233,111,234,111,235,111,236,111,237,111,240,111,241,111,242,111,243,111,244,111,245,111,246,111,247,111,248,111,249,111,250,111,251,111,252,111,253,111,254,111,255,111,0,112,1,112,2,112,3,112,4,112,5,112,6,112,7,112,8,112,9,112,10,112,11,112,12,112,13,112,14,112,15,112,16,112,18,112,19,112,20,112,21,112,22,112,23,112,24,112,25,112,28,112,29,112,30,112,31,112,32,112,33,112,34,112,36,112,37,112,38,112,39,112,40,112,41,112,42,112,43,112,44,112,45,112,46,112,47,112,48,112,49,112,50,112,51,112,52,112,54,112,55,112,56,112,58,112,59,112,60,112,61,112,62,112,63,112,64,112,65,112,66,112,67,112,68,112,69,112,70,112,71,112,72,112,73,112,74,112,75,112,77,112,78,112,80,112,81,112,82,112,83,112,84,112,85,112,86,112,87,112,88,112,89,112,90,112,91,112,92,112,93,112,95,112,96,112,97,112,98,112,99,112,100,112,101,112,102,112,103,112,104,112,105,112,106,112,110,112,113,112,114,112,115,112,116,112,119,112,121,112,122,112,123,112,125,112,129,112,130,112,131,112,132,112,134,112,135,112,136,112,139,112,140,112,141,112,143,112,144,112,145,112,147,112,151,112,152,112,154,112,155,112,158,112,159,112,160,112,161,112,162,112,163,112,164,112,165,112,166,112,167,112,168,112,169,112,170,112,176,112,178,112,180,112,181,112,182,112,186,112,190,112,191,112,196,112,197,112,198,112,199,112,201,112,203,112,204,112,205,112,206,112,207,112,208,112,209,112,210,112,211,112,212,112,213,112,214,112,215,112,218,112,220,112,221,112,222,112,224,112,225,112,226,112,227,112,229,112,234,112,238,112,240,112,241,112,242,112,243,112,244,112,245,112,246,112,248,112,250,112,251,112,252,112,254,112,255,112,0,113,1,113,2,113,3,113,4,113,5,113,6,113,7,113,8,113,11,113,12,113,13,113,14,113,15,113,17,113,18,113,20,113,23,113,27,113,28,113,29,113,30,113,31,113,32,113,33,113,34,113,35,113,36,113,37,113,39,113,40,113,41,113,42,113,43,113,44,113,45,113,46,113,50,113,51,113,52,113,53,113,55,113,56,113,57,113,58,113,59,113,60,113,61,113,62,113,63,113,64,113,65,113,66,113,67,113,68,113,70,113,71,113,72,113,73,113,75,113,77,113,79,113,80,113,81,113,82,113,83,113,84,113,85,113,86,113,87,113,88,113,89,113,90,113,91,113,93,113,95,113,96,113,97,113,98,113,99,113,101,113,105,113,106,113,107,113,108,113,109,113,111,113,112,113,113,113,116,113,117,113,118,113,119,113,121,113,123,113,124,113,126,113,127,113,128,113,129,113,130,113,131,113,133,113,134,113,135,113,136,113,137,113,139,113,140,113,141,113,142,113,144,113,145,113,146,113,147,113,149,113,150,113,151,113,154,113,155,113,156,113,157,113,158,113,161,113,162,113,163,113,164,113,165,113,166,113,167,113,169,113,170,113,171,113,173,113,174,113,175,113,176,113,177,113,178,113,180,113,182,113,183,113,184,113,186,113,187,113,188,113,189,113,190,113,191,113,192,113,193,113,194,113,196,113,197,113,198,113,199,113,200,113,201,113,202,113,203,113,204,113,205,113,207,113,208,113,209,113,210,113,211,113,214,113,215,113,216,113,217,113,218,113,219,113,220,113,221,113,222,113,223,113,225,113,226,113,227,113,228,113,230,113,232,113,233,113,234,113,235,113,236,113,237,113,239,113,240,113,241,113,242,113,243,113,244,113,245,113,246,113,247,113,248,113,250,113,251,113,252,113,253,113,254,113,255,113,0,114,1,114,2,114,3,114,4,114,5,114,7,114,8,114,9,114,10,114,11,114,12,114,13,114,14,114,15,114,16,114,17,114,18,114,19,114,20,114,21,114,22,114,23,114,24,114,25,114,26,114,27,114,28,114,30,114,31,114,32,114,33,114,34,114,35,114,36,114,37,114,38,114,39,114,41,114,43,114,45,114,46,114,47,114,50,114,51,114,52,114,58,114,60,114,62,114,64,114,65,114,66,114,67,114,68,114,69,114,70,114,73,114,74,114,75,114,78,114,79,114,80,114,81,114,83,114,84,114,85,114,87,114,88,114,90,114,92,114,94,114,96,114,99,114,100,114,101,114,104,114,106,114,107,114,108,114,109,114,112,114,113,114,115,114,116,114,118,114,119,114,120,114,123,114,124,114,125,114,130,114,131,114,133,114,134,114,135,114,136,114,137,114,140,114,142,114,144,114,145,114,147,114,148,114,149,114,150,114,151,114,152,114,153,114,154,114,155,114,156,114,157,114,158,114,160,114,161,114,162,114,163,114,164,114,165,114,166,114,167,114,168,114,169,114,170,114,171,114,174,114,177,114,178,114,179,114,181,114,186,114,187,114,188,114,189,114,190,114,191,114,192,114,197,114,198,114,199,114,201,114,202,114,203,114,204,114,207,114,209,114,211,114,212,114,213,114,214,114,216,114,218,114,219,114,198,228,199,228,200,228,201,228,202,228,203,228,204,228,205,228,206,228,207,228,208,228,209,228,210,228,211,228,212,228,213,228,214,228,215,228,216,228,217,228,218,228,219,228,220,228,221,228,222,228,223,228,224,228,225,228,226,228,227,228,228,228,229,228,230,228,231,228,232,228,233,228,234,228,235,228,236,228,237,228,238,228,239,228,240,228,241,228,242,228,243,228,244,228,245,228,246,228,247,228,248,228,249,228,250,228,251,228,252,228,253,228,254,228,255,228,0,229,1,229,2,229,3,229,4,229,5,229,6,229,7,229,8,229,9,229,10,229,11,229,12,229,13,229,14,229,15,229,16,229,17,229,18,229,19,229,20,229,21,229,22,229,23,229,24,229,25,229,26,229,27,229,28,229,29,229,30,229,31,229,32,229,33,229,34,229,35,229,36,229,37,229,0,48,1,48,2,48,183,0,201,2,199,2,168,0,3,48,5,48,20,32,94,255,22,32,38,32,24,32,25,32,28,32,29,32,20,48,21,48,8,48,9,48,10,48,11,48,12,48,13,48,14,48,15,48,22,48,23,48,16,48,17,48,177,0,215,0,247,0,54,34,39,34,40,34,17,34,15,34,42,34,41,34,8,34,55,34,26,34,165,34,37,34,32,34,18,35,153,34,43,34,46,34,97,34,76,34,72,34,61,34,29,34,96,34,110,34,111,34,100,34,101,34,30,34,53,34,52,34,66,38,64,38,176,0,50,32,51,32,3,33,4,255,164,0,224,255,225,255,48,32,167,0,22,33,6,38,5,38,203,37,207,37,206,37,199,37,198,37,161,37,160,37,179,37,178,37,59,32,146,33,144,33,145,33,147,33,19,48,38,229,39,229,40,229,41,229,42,229,43,229,44,229,45,229,46,229,47,229,48,229,49,229,50,229,51,229,52,229,53,229,54,229,55,229,56,229,57,229,58,229,59,229,60,229,61,229,62,229,63,229,64,229,65,229,66,229,67,229,68,229,69,229,70,229,71,229,72,229,73,229,74,229,75,229,76,229,77,229,78,229,79,229,80,229,81,229,82,229,83,229,84,229,85,229,86,229,87,229,88,229,89,229,90,229,91,229,92,229,93,229,94,229,95,229,96,229,97,229,98,229,99,229,100,229,101,229,102,229,103,229,104,229,105,229,106,229,107,229,108,229,109,229,110,229,111,229,112,229,113,229,114,229,115,229,116,229,117,229,118,229,119,229,120,229,121,229,122,229,123,229,124,229,125,229,126,229,127,229,128,229,129,229,130,229,131,229,132,229,133,229,112,33,113,33,114,33,115,33,116,33,117,33,118,33,119,33,120,33,121,33,102,231,103,231,104,231,105,231,106,231,107,231,136,36,137,36,138,36,139,36,140,36,141,36,142,36,143,36,144,36,145,36,146,36,147,36,148,36,149,36,150,36,151,36,152,36,153,36,154,36,155,36,116,36,117,36,118,36,119,36,120,36,121,36,122,36,123,36,124,36,125,36,126,36,127,36,128,36,129,36,130,36,131,36,132,36,133,36,134,36,135,36,96,36,97,36,98,36,99,36,100,36,101,36,102,36,103,36,104,36,105,36,172,32,109,231,32,50,33,50,34,50,35,50,36,50,37,50,38,50,39,50,40,50,41,50,110,231,111,231,96,33,97,33,98,33,99,33,100,33,101,33,102,33,103,33,104,33,105,33,106,33,107,33,112,231,113,231,134,229,135,229,136,229,137,229,138,229,139,229,140,229,141,229,142,229,143,229,144,229,145,229,146,229,147,229,148,229,149,229,150,229,151,229,152,229,153,229,154,229,155,229,156,229,157,229,158,229,159,229,160,229,161,229,162,229,163,229,164,229,165,229,166,229,167,229,168,229,169,229,170,229,171,229,172,229,173,229,174,229,175,229,176,229,177,229,178,229,179,229,180,229,181,229,182,229,183,229,184,229,185,229,186,229,187,229,188,229,189,229,190,229,191,229,192,229,193,229,194,229,195,229,196,229,197,229,198,229,199,229,200,229,201,229,202,229,203,229,204,229,205,229,206,229,207,229,208,229,209,229,210,229,211,229,212,229,213,229,214,229,215,229,216,229,217,229,218,229,219,229,220,229,221,229,222,229,223,229,224,229,225,229,226,229,227,229,228,229,229,229,1,255,2,255,3,255,229,255,5,255,6,255,7,255,8,255,9,255,10,255,11,255,12,255,13,255,14,255,15,255,16,255,17,255,18,255,19,255,20,255,21,255,22,255,23,255,24,255,25,255,26,255,27,255,28,255,29,255,30,255,31,255,32,255,33,255,34,255,35,255,36,255,37,255,38,255,39,255,40,255,41,255,42,255,43,255,44,255,45,255,46,255,47,255,48,255,49,255,50,255,51,255,52,255,53,255,54,255,55,255,56,255,57,255,58,255,59,255,60,255,61,255,62,255,63,255,64,255,65,255,66,255,67,255,68,255,69,255,70,255,71,255,72,255,73,255,74,255,75,255,76,255,77,255,78,255,79,255,80,255,81,255,82,255,83,255,84,255,85,255,86,255,87,255,88,255,89,255,90,255,91,255,92,255,93,255,227,255,230,229,231,229,232,229,233,229,234,229,235,229,236,229,237,229,238,229,239,229,240,229,241,229,242,229,243,229,244,229,245,229,246,229,247,229,248,229,249,229,250,229,251,229,252,229,253,229,254,229,255,229,0,230,1,230,2,230,3,230,4,230,5,230,6,230,7,230,8,230,9,230,10,230,11,230,12,230,13,230,14,230,15,230,16,230,17,230,18,230,19,230,20,230,21,230,22,230,23,230,24,230,25,230,26,230,27,230,28,230,29,230,30,230,31,230,32,230,33,230,34,230,35,230,36,230,37,230,38,230,39,230,40,230,41,230,42,230,43,230,44,230,45,230,46,230,47,230,48,230,49,230,50,230,51,230,52,230,53,230,54,230,55,230,56,230,57,230,58,230,59,230,60,230,61,230,62,230,63,230,64,230,65,230,66,230,67,230,68,230,69,230,65,48,66,48,67,48,68,48,69,48,70,48,71,48,72,48,73,48,74,48,75,48,76,48,77,48,78,48,79,48,80,48,81,48,82,48,83,48,84,48,85,48,86,48,87,48,88,48,89,48,90,48,91,48,92,48,93,48,94,48,95,48,96,48,97,48,98,48,99,48,100,48,101,48,102,48,103,48,104,48,105,48,106,48,107,48,108,48,109,48,110,48,111,48,112,48,113,48,114,48,115,48,116,48,117,48,118,48,119,48,120,48,121,48,122,48,123,48,124,48,125,48,126,48,127,48,128,48,129,48,130,48,131,48,132,48,133,48,134,48,135,48,136,48,137,48,138,48,139,48,140,48,141,48,142,48,143,48,144,48,145,48,146,48,147,48,114,231,115,231,116,231,117,231,118,231,119,231,120,231,121,231,122,231,123,231,124,231,70,230,71,230,72,230,73,230,74,230,75,230,76,230,77,230,78,230,79,230,80,230,81,230,82,230,83,230,84,230,85,230,86,230,87,230,88,230,89,230,90,230,91,230,92,230,93,230,94,230,95,230,96,230,97,230,98,230,99,230,100,230,101,230,102,230,103,230,104,230,105,230,106,230,107,230,108,230,109,230,110,230,111,230,112,230,113,230,114,230,115,230,116,230,117,230,118,230,119,230,120,230,121,230,122,230,123,230,124,230,125,230,126,230,127,230,128,230,129,230,130,230,131,230,132,230,133,230,134,230,135,230,136,230,137,230,138,230,139,230,140,230,141,230,142,230,143,230,144,230,145,230,146,230,147,230,148,230,149,230,150,230,151,230,152,230,153,230,154,230,155,230,156,230,157,230,158,230,159,230,160,230,161,230,162,230,163,230,164,230,165,230,161,48,162,48,163,48,164,48,165,48,166,48,167,48,168,48,169,48,170,48,171,48,172,48,173,48,174,48,175,48,176,48,177,48,178,48,179,48,180,48,181,48,182,48,183,48,184,48,185,48,186,48,187,48,188,48,189,48,190,48,191,48,192,48,193,48,194,48,195,48,196,48,197,48,198,48,199,48,200,48,201,48,202,48,203,48,204,48,205,48,206,48,207,48,208,48,209,48,210,48,211,48,212,48,213,48,214,48,215,48,216,48,217,48,218,48,219,48,220,48,221,48,222,48,223,48,224,48,225,48,226,48,227,48,228,48,229,48,230,48,231,48,232,48,233,48,234,48,235,48,236,48,237,48,238,48,239,48,240,48,241,48,242,48,243,48,244,48,245,48,246,48,125,231,126,231,127,231,128,231,129,231,130,231,131,231,132,231,166,230,167,230,168,230,169,230,170,230,171,230,172,230,173,230,174,230,175,230,176,230,177,230,178,230,179,230,180,230,181,230,182,230,183,230,184,230,185,230,186,230,187,230,188,230,189,230,190,230,191,230,192,230,193,230,194,230,195,230,196,230,197,230,198,230,199,230,200,230,201,230,202,230,203,230,204,230,205,230,206,230,207,230,208,230,209,230,210,230,211,230,212,230,213,230,214,230,215,230,216,230,217,230,218,230,219,230,220,230,221,230,222,230,223,230,224,230,225,230,226,230,227,230,228,230,229,230,230,230,231,230,232,230,233,230,234,230,235,230,236,230,237,230,238,230,239,230,240,230,241,230,242,230,243,230,244,230,245,230,246,230,247,230,248,230,249,230,250,230,251,230,252,230,253,230,254,230,255,230,0,231,1,231,2,231,3,231,4,231,5,231,145,3,146,3,147,3,148,3,149,3,150,3,151,3,152,3,153,3,154,3,155,3,156,3,157,3,158,3,159,3,160,3,161,3,163,3,164,3,165,3,166,3,167,3,168,3,169,3,133,231,134,231,135,231,136,231,137,231,138,231,139,231,140,231,177,3,178,3,179,3,180,3,181,3,182,3,183,3,184,3,185,3,186,3,187,3,188,3,189,3,190,3,191,3,192,3,193,3,195,3,196,3,197,3,198,3,199,3,200,3,201,3,141,231,142,231,143,231,144,231,145,231,146,231,147,231,53,254,54,254,57,254,58,254,63,254,64,254,61,254,62,254,65,254,66,254,67,254,68,254,148,231,149,231,59,254,60,254,55,254,56,254,49,254,150,231,51,254,52,254,151,231,152,231,153,231,154,231,155,231,156,231,157,231,158,231,159,231,6,231,7,231,8,231,9,231,10,231,11,231,12,231,13,231,14,231,15,231,16,231,17,231,18,231,19,231,20,231,21,231,22,231,23,231,24,231,25,231,26,231,27,231,28,231,29,231,30,231,31,231,32,231,33,231,34,231,35,231,36,231,37,231,38,231,39,231,40,231,41,231,42,231,43,231,44,231,45,231,46,231,47,231,48,231,49,231,50,231,51,231,52,231,53,231,54,231,55,231,56,231,57,231,58,231,59,231,60,231,61,231,62,231,63,231,64,231,65,231,66,231,67,231,68,231,69,231,70,231,71,231,72,231,73,231,74,231,75,231,76,231,77,231,78,231,79,231,80,231,81,231,82,231,83,231,84,231,85,231,86,231,87,231,88,231,89,231,90,231,91,231,92,231,93,231,94,231,95,231,96,231,97,231,98,231,99,231,100,231,101,231,16,4,17,4,18,4,19,4,20,4,21,4,1,4,22,4,23,4,24,4,25,4,26,4,27,4,28,4,29,4,30,4,31,4,32,4,33,4,34,4,35,4,36,4,37,4,38,4,39,4,40,4,41,4,42,4,43,4,44,4,45,4,46,4,47,4,160,231,161,231,162,231,163,231,164,231,165,231,166,231,167,231,168,231,169,231,170,231,171,231,172,231,173,231,174,231,48,4,49,4,50,4,51,4,52,4,53,4,81,4,54,4,55,4,56,4,57,4,58,4,59,4,60,4,61,4,62,4,63,4,64,4,65,4,66,4,67,4,68,4,69,4,70,4,71,4,72,4,73,4,74,4,75,4,76,4,77,4,78,4,79,4,175,231,176,231,177,231,178,231,179,231,180,231,181,231,182,231,183,231,184,231,185,231,186,231,187,231,202,2,203,2,217,2,19,32,21,32,37,32,53,32,5,33,9,33,150,33,151,33,152,33,153,33,21,34,31,34,35,34,82,34,102,34,103,34,191,34,80,37,81,37,82,37,83,37,84,37,85,37,86,37,87,37,88,37,89,37,90,37,91,37,92,37,93,37,94,37,95,37,96,37,97,37,98,37,99,37,100,37,101,37,102,37,103,37,104,37,105,37,106,37,107,37,108,37,109,37,110,37,111,37,112,37,113,37,114,37,115,37,129,37,130,37,131,37,132,37,133,37,134,37,135,37,136,37,137,37,138,37,139,37,140,37,141,37,142,37,143,37,147,37,148,37,149,37,188,37,189,37,226,37,227,37,228,37,229,37,9,38,149,34,18,48,29,48,30,48,188,231,189,231,190,231,191,231,192,231,193,231,194,231,195,231,196,231,197,231,198,231,1,1,225,0,206,1,224,0,19,1,233,0,27,1,232,0,43,1,237,0,208,1,236,0,77,1,243,0,210,1,242,0,107,1,250,0,212,1,249,0,214,1,216,1,218,1,220,1,252,0,234,0,81,2,199,231,68,1,72,1,249,1,97,2,201,231,202,231,203,231,204,231,5,49,6,49,7,49,8,49,9,49,10,49,11,49,12,49,13,49,14,49,15,49,16,49,17,49,18,49,19,49,20,49,21,49,22,49,23,49,24,49,25,49,26,49,27,49,28,49,29,49,30,49,31,49,32,49,33,49,34,49,35,49,36,49,37,49,38,49,39,49,40,49,41,49,205,231,206,231,207,231,208,231,209,231,210,231,211,231,212,231,213,231,214,231,215,231,216,231,217,231,218,231,219,231,220,231,221,231,222,231,223,231,224,231,225,231,33,48,34,48,35,48,36,48,37,48,38,48,39,48,40,48,41,48,163,50,142,51,143,51,156,51,157,51,158,51,161,51,196,51,206,51,209,51,210,51,213,51,48,254,226,255,228,255,226,231,33,33,49,50,227,231,16,32,228,231,229,231,230,231,252,48,155,48,156,48,253,48,254,48,6,48,157,48,158,48,73,254,74,254,75,254,76,254,77,254,78,254,79,254,80,254,81,254,82,254,84,254,85,254,86,254,87,254,89,254,90,254,91,254,92,254,93,254,94,254,95,254,96,254,97,254,98,254,99,254,100,254,101,254,102,254,104,254,105,254,106,254,107,254,62,48,240,47,241,47,242,47,243,47,244,47,245,47,246,47,247,47,248,47,249,47,250,47,251,47,7,48,244,231,245,231,246,231,247,231,248,231,249,231,250,231,251,231,252,231,253,231,254,231,255,231,0,232,0,37,1,37,2,37,3,37,4,37,5,37,6,37,7,37,8,37,9,37,10,37,11,37,12,37,13,37,14,37,15,37,16,37,17,37,18,37,19,37,20,37,21,37,22,37,23,37,24,37,25,37,26,37,27,37,28,37,29,37,30,37,31,37,32,37,33,37,34,37,35,37,36,37,37,37,38,37,39,37,40,37,41,37,42,37,43,37,44,37,45,37,46,37,47,37,48,37,49,37,50,37,51,37,52,37,53,37,54,37,55,37,56,37,57,37,58,37,59,37,60,37,61,37,62,37,63,37,64,37,65,37,66,37,67,37,68,37,69,37,70,37,71,37,72,37,73,37,74,37,75,37,1,232,2,232,3,232,4,232,5,232,6,232,7,232,8,232,9,232,10,232,11,232,12,232,13,232,14,232,15,232,220,114,221,114,223,114,226,114,227,114,228,114,229,114,230,114,231,114,234,114,235,114,245,114,246,114,249,114,253,114,254,114,255,114,0,115,2,115,4,115,5,115,6,115,7,115,8,115,9,115,11,115,12,115,13,115,15,115,16,115,17,115,18,115,20,115,24,115,25,115,26,115,31,115,32,115,35,115,36,115,38,115,39,115,40,115,45,115,47,115,48,115,50,115,51,115,53,115,54,115,58,115,59,115,60,115,61,115,64,115,65,115,66,115,67,115,68,115,69,115,70,115,71,115,72,115,73,115,74,115,75,115,76,115,78,115,79,115,81,115,83,115,84,115,85,115,86,115,88,115,89,115,90,115,91,115,92,115,93,115,94,115,95,115,97,115,98,115,99,115,100,115,101,115,102,115,103,115,104,115,105,115,106,115,107,115,110,115,112,115,113,115,0,224,1,224,2,224,3,224,4,224,5,224,6,224,7,224,8,224,9,224,10,224,11,224,12,224,13,224,14,224,15,224,16,224,17,224,18,224,19,224,20,224,21,224,22,224,23,224,24,224,25,224,26,224,27,224,28,224,29,224,30,224,31,224,32,224,33,224,34,224,35,224,36,224,37,224,38,224,39,224,40,224,41,224,42,224,43,224,44,224,45,224,46,224,47,224,48,224,49,224,50,224,51,224,52,224,53,224,54,224,55,224,56,224,57,224,58,224,59,224,60,224,61,224,62,224,63,224,64,224,65,224,66,224,67,224,68,224,69,224,70,224,71,224,72,224,73,224,74,224,75,224,76,224,77,224,78,224,79,224,80,224,81,224,82,224,83,224,84,224,85,224,86,224,87,224,88,224,89,224,90,224,91,224,92,224,93,224,114,115,115,115,116,115,117,115,118,115,119,115,120,115,121,115,122,115,123,115,124,115,125,115,127,115,128,115,129,115,130,115,131,115,133,115,134,115,136,115,138,115,140,115,141,115,143,115,144,115,146,115,147,115,148,115,149,115,151,115,152,115,153,115,154,115,156,115,157,115,158,115,160,115,161,115,163,115,164,115,165,115,166,115,167,115,168,115,170,115,172,115,173,115,177,115,180,115,181,115,182,115,184,115,185,115,188,115,189,115,190,115,191,115,193,115,195,115,196,115,197,115,198,115,199,115,203,115,204,115,206,115,210,115,211,115,212,115,213,115,214,115,215,115,216,115,218,115,219,115,220,115,221,115,223,115,225,115,226,115,227,115,228,115,230,115,232,115,234,115,235,115,236,115,238,115,239,115,240,115,241,115,243,115,244,115,245,115,246,115,247,115,94,224,95,224,96,224,97,224,98,224,99,224,100,224,101,224,102,224,103,224,104,224,105,224,106,224,107,224,108,224,109,224,110,224,111,224,112,224,113,224,114,224,115,224,116,224,117,224,118,224,119,224,120,224,121,224,122,224,123,224,124,224,125,224,126,224,127,224,128,224,129,224,130,224,131,224,132,224,133,224,134,224,135,224,136,224,137,224,138,224,139,224,140,224,141,224,142,224,143,224,144,224,145,224,146,224,147,224,148,224,149,224,150,224,151,224,152,224,153,224,154,224,155,224,156,224,157,224,158,224,159,224,160,224,161,224,162,224,163,224,164,224,165,224,166,224,167,224,168,224,169,224,170,224,171,224,172,224,173,224,174,224,175,224,176,224,177,224,178,224,179,224,180,224,181,224,182,224,183,224,184,224,185,224,186,224,187,224,248,115,249,115,250,115,251,115,252,115,253,115,254,115,255,115,0,116,1,116,2,116,4,116,7,116,8,116,11,116,12,116,13,116,14,116,17,116,18,116,19,116,20,116,21,116,22,116,23,116,24,116,25,116,28,116,29,116,30,116,31,116,32,116,33,116,35,116,36,116,39,116,41,116,43,116,45,116,47,116,49,116,50,116,55,116,56,116,57,116,58,116,59,116,61,116,62,116,63,116,64,116,66,116,67,116,68,116,69,116,70,116,71,116,72,116,73,116,74,116,75,116,76,116,77,116,78,116,79,116,80,116,81,116,82,116,83,116,84,116,86,116,88,116,93,116,96,116,97,116,98,116,99,116,100,116,101,116,102,116,103,116,104,116,105,116,106,116,107,116,108,116,110,116,111,116,113,116,114,116,115,116,116,116,117,116,120,116,121,116,122,116,188,224,189,224,190,224,191,224,192,224,193,224,194,224,195,224,196,224,197,224,198,224,199,224,200,224,201,224,202,224,203,224,204,224,205,224,206,224,207,224,208,224,209,224,210,224,211,224,212,224,213,224,214,224,215,224,216,224,217,224,218,224,219,224,220,224,221,224,222,224,223,224,224,224,225,224,226,224,227,224,228,224,229,224,230,224,231,224,232,224,233,224,234,224,235,224,236,224,237,224,238,224,239,224,240,224,241,224,242,224,243,224,244,224,245,224,246,224,247,224,248,224,249,224,250,224,251,224,252,224,253,224,254,224,255,224,0,225,1,225,2,225,3,225,4,225,5,225,6,225,7,225,8,225,9,225,10,225,11,225,12,225,13,225,14,225,15,225,16,225,17,225,18,225,19,225,20,225,21,225,22,225,23,225,24,225,25,225,123,116,124,116,125,116,127,116,130,116,132,116,133,116,134,116,136,116,137,116,138,116,140,116,141,116,143,116,145,116,146,116,147,116,148,116,149,116,150,116,151,116,152,116,153,116,154,116,155,116,157,116,159,116,160,116,161,116,162,116,163,116,164,116,165,116,166,116,170,116,171,116,172,116,173,116,174,116,175,116,176,116,177,116,178,116,179,116,180,116,181,116,182,116,183,116,184,116,185,116,187,116,188,116,189,116,190,116,191,116,192,116,193,116,194,116,195,116,196,116,197,116,198,116,199,116,200,116,201,116,202,116,203,116,204,116,205,116,206,116,207,116,208,116,209,116,211,116,212,116,213,116,214,116,215,116,216,116,217,116,218,116,219,116,221,116,223,116,225,116,229,116,231,116,232,116,233,116,234,116,235,116,236,116,237,116,240,116,241,116,242,116,26,225,27,225,28,225,29,225,30,225,31,225,32,225,33,225,34,225,35,225,36,225,37,225,38,225,39,225,40,225,41,225,42,225,43,225,44,225,45,225,46,225,47,225,48,225,49,225,50,225,51,225,52,225,53,225,54,225,55,225,56,225,57,225,58,225,59,225,60,225,61,225,62,225,63,225,64,225,65,225,66,225,67,225,68,225,69,225,70,225,71,225,72,225,73,225,74,225,75,225,76,225,77,225,78,225,79,225,80,225,81,225,82,225,83,225,84,225,85,225,86,225,87,225,88,225,89,225,90,225,91,225,92,225,93,225,94,225,95,225,96,225,97,225,98,225,99,225,100,225,101,225,102,225,103,225,104,225,105,225,106,225,107,225,108,225,109,225,110,225,111,225,112,225,113,225,114,225,115,225,116,225,117,225,118,225,119,225,243,116,245,116,248,116,249,116,250,116,251,116,252,116,253,116,254,116,0,117,1,117,2,117,3,117,5,117,6,117,7,117,8,117,9,117,10,117,11,117,12,117,14,117,16,117,18,117,20,117,21,117,22,117,23,117,27,117,29,117,30,117,32,117,33,117,34,117,35,117,36,117,38,117,39,117,42,117,46,117,52,117,54,117,57,117,60,117,61,117,63,117,65,117,66,117,67,117,68,117,70,117,71,117,73,117,74,117,77,117,80,117,81,117,82,117,83,117,85,117,86,117,87,117,88,117,93,117,94,117,95,117,96,117,97,117,98,117,99,117,100,117,103,117,104,117,105,117,107,117,108,117,109,117,110,117,111,117,112,117,113,117,115,117,117,117,118,117,119,117,122,117,123,117,124,117,125,117,126,117,128,117,129,117,130,117,132,117,133,117,135,117,120,225,121,225,122,225,123,225,124,225,125,225,126,225,127,225,128,225,129,225,130,225,131,225,132,225,133,225,134,225,135,225,136,225,137,225,138,225,139,225,140,225,141,225,142,225,143,225,144,225,145,225,146,225,147,225,148,225,149,225,150,225,151,225,152,225,153,225,154,225,155,225,156,225,157,225,158,225,159,225,160,225,161,225,162,225,163,225,164,225,165,225,166,225,167,225,168,225,169,225,170,225,171,225,172,225,173,225,174,225,175,225,176,225,177,225,178,225,179,225,180,225,181,225,182,225,183,225,184,225,185,225,186,225,187,225,188,225,189,225,190,225,191,225,192,225,193,225,194,225,195,225,196,225,197,225,198,225,199,225,200,225,201,225,202,225,203,225,204,225,205,225,206,225,207,225,208,225,209,225,210,225,211,225,212,225,213,225,136,117,137,117,138,117,140,117,141,117,142,117,144,117,147,117,149,117,152,117,155,117,156,117,158,117,162,117,166,117,167,117,168,117,169,117,170,117,173,117,182,117,183,117,186,117,187,117,191,117,192,117,193,117,198,117,203,117,204,117,206,117,207,117,208,117,209,117,211,117,215,117,217,117,218,117,220,117,221,117,223,117,224,117,225,117,229,117,233,117,236,117,237,117,238,117,239,117,242,117,243,117,245,117,246,117,247,117,248,117,250,117,251,117,253,117,254,117,2,118,4,118,6,118,7,118,8,118,9,118,11,118,13,118,14,118,15,118,17,118,18,118,19,118,20,118,22,118,26,118,28,118,29,118,30,118,33,118,35,118,39,118,40,118,44,118,46,118,47,118,49,118,50,118,54,118,55,118,57,118,58,118,59,118,61,118,65,118,66,118,68,118,214,225,215,225,216,225,217,225,218,225,219,225,220,225,221,225,222,225,223,225,224,225,225,225,226,225,227,225,228,225,229,225,230,225,231,225,232,225,233,225,234,225,235,225,236,225,237,225,238,225,239,225,240,225,241,225,242,225,243,225,244,225,245,225,246,225,247,225,248,225,249,225,250,225,251,225,252,225,253,225,254,225,255,225,0,226,1,226,2,226,3,226,4,226,5,226,6,226,7,226,8,226,9,226,10,226,11,226,12,226,13,226,14,226,15,226,16,226,17,226,18,226,19,226,20,226,21,226,22,226,23,226,24,226,25,226,26,226,27,226,28,226,29,226,30,226,31,226,32,226,33,226,34,226,35,226,36,226,37,226,38,226,39,226,40,226,41,226,42,226,43,226,44,226,45,226,46,226,47,226,48,226,49,226,50,226,51,226,69,118,70,118,71,118,72,118,73,118,74,118,75,118,78,118,79,118,80,118,81,118,82,118,83,118,85,118,87,118,88,118,89,118,90,118,91,118,93,118,95,118,96,118,97,118,98,118,100,118,101,118,102,118,103,118,104,118,105,118,106,118,108,118,109,118,110,118,112,118,113,118,114,118,115,118,116,118,117,118,118,118,119,118,121,118,122,118,124,118,127,118,128,118,129,118,131,118,133,118,137,118,138,118,140,118,141,118,143,118,144,118,146,118,148,118,149,118,151,118,152,118,154,118,155,118,156,118,157,118,158,118,159,118,160,118,161,118,162,118,163,118,165,118,166,118,167,118,168,118,169,118,170,118,171,118,172,118,173,118,175,118,176,118,179,118,181,118,182,118,183,118,184,118,185,118,186,118,187,118,188,118,189,118,190,118,192,118,193,118,195,118,74,85,63,150,195,87,40,99,206,84,9,85,192,84,145,118,76,118,60,133,238,119,126,130,141,120,49,114,152,150,141,151,40,108,137,91,250,79,9,99,151,102,184,92,250,128,72,104,174,128,2,102,206,118,249,81,86,101,172,113,241,127,132,136,178,80,101,89,202,97,179,111,173,130,76,99,82,98,237,83,39,84,6,123,107,81,164,117,244,93,212,98,203,141,118,151,138,98,25,128,93,87,56,151,98,127,56,114,125,118,207,103,126,118,70,100,112,79,37,141,220,98,23,122,145,101,237,115,44,100,115,98,44,130,129,152,127,103,72,114,110,98,204,98,52,79,227,116,74,83,158,82,202,126,166,144,46,94,134,104,156,105,128,129,209,126,210,104,197,120,140,134,81,149,141,80,36,140,222,130,222,128,5,83,18,137,101,82,196,118,199,118,201,118,203,118,204,118,211,118,213,118,217,118,218,118,220,118,221,118,222,118,224,118,225,118,226,118,227,118,228,118,230,118,231,118,232,118,233,118,234,118,235,118,236,118,237,118,240,118,243,118,245,118,246,118,247,118,250,118,251,118,253,118,255,118,0,119,2,119,3,119,5,119,6,119,10,119,12,119,14,119,15,119,16,119,17,119,18,119,19,119,20,119,21,119,22,119,23,119,24,119,27,119,28,119,29,119,30,119,33,119,35,119,36,119,37,119,39,119,42,119,43,119,44,119,46,119,48,119,49,119,50,119,51,119,52,119,57,119,59,119,61,119,62,119,63,119,66,119,68,119,69,119,70,119,72,119,73,119,74,119,75,119,76,119,77,119,78,119,79,119,82,119,83,119,84,119,85,119,86,119,87,119,88,119,89,119,92,119,132,133,249,150,221,79,33,88,113,153,157,91,177,98,165,98,180,102,121,140,141,156,6,114,111,103,145,120,178,96,81,83,23,83,136,143,204,128,29,141,161,148,13,80,200,114,7,89,235,96,25,113,171,136,84,89,239,130,44,103,40,123,41,93,247,126,45,117,245,108,102,142,248,143,60,144,59,159,212,107,25,145,20,123,124,95,167,120,214,132,61,133,213,107,217,107,214,107,1,94,135,94,249,117,237,149,93,101,10,95,197,95,159,143,193,88,194,129,127,144,91,150,173,151,185,143,22,127,44,141,65,98,191,79,216,83,94,83,168,143,169,143,171,143,77,144,7,104,106,95,152,129,104,136,214,156,139,97,43,82,42,118,108,95,140,101,210,111,232,110,190,91,72,100,117,81,176,81,196,103,25,78,201,121,124,153,179,112,93,119,94,119,95,119,96,119,100,119,103,119,105,119,106,119,109,119,110,119,111,119,112,119,113,119,114,119,115,119,116,119,117,119,118,119,119,119,120,119,122,119,123,119,124,119,129,119,130,119,131,119,134,119,135,119,136,119,137,119,138,119,139,119,143,119,144,119,147,119,148,119,149,119,150,119,151,119,152,119,153,119,154,119,155,119,156,119,157,119,158,119,161,119,163,119,164,119,166,119,168,119,171,119,173,119,174,119,175,119,177,119,178,119,180,119,182,119,183,119,184,119,185,119,186,119,188,119,190,119,192,119,193,119,194,119,195,119,196,119,197,119,198,119,199,119,200,119,201,119,202,119,203,119,204,119,206,119,207,119,208,119,209,119,210,119,211,119,212,119,213,119,214,119,216,119,217,119,218,119,221,119,222,119,223,119,224,119,225,119,228,119,197,117,118,94,187,115,224,131,173,100,232,98,181,148,226,108,90,83,195,82,15,100,194,148,148,123,47,79,27,94,54,130,22,129,138,129,36,110,202,108,115,154,85,99,92,83,250,84,101,136,224,87,13,78,3,94,101,107,63,124,232,144,22,96,230,100,28,115,193,136,80,103,77,98,34,141,108,119,41,142,199,145,105,95,220,131,33,133,16,153,194,83,149,134,139,107,237,96,232,96,127,112,205,130,49,130,211,78,167,108,207,133,205,100,217,124,253,105,249,102,73,131,149,83,86,123,167,79,140,81,75,109,66,92,109,142,210,99,201,83,44,131,54,131,229,103,180,120,61,100,223,91,148,92,238,93,231,139,198,98,244,103,122,140,0,100,186,99,73,135,139,153,23,140,32,127,242,148,167,78,16,150,164,152,12,102,22,115,230,119,232,119,234,119,239,119,240,119,241,119,242,119,244,119,245,119,247,119,249,119,250,119,251,119,252,119,3,120,4,120,5,120,6,120,7,120,8,120,10,120,11,120,14,120,15,120,16,120,19,120,21,120,25,120,27,120,30,120,32,120,33,120,34,120,36,120,40,120,42,120,43,120,46,120,47,120,49,120,50,120,51,120,53,120,54,120,61,120,63,120,65,120,66,120,67,120,68,120,70,120,72,120,73,120,74,120,75,120,77,120,79,120,81,120,83,120,84,120,88,120,89,120,90,120,91,120,92,120,94,120,95,120,96,120,97,120,98,120,99,120,100,120,101,120,102,120,103,120,104,120,105,120,111,120,112,120,113,120,114,120,115,120,116,120,117,120,118,120,120,120,121,120,122,120,123,120,125,120,126,120,127,120,128,120,129,120,130,120,131,120,58,87,29,92,56,94,127,149,127,80,160,128,130,83,94,101,69,117,49,85,33,80,133,141,132,98,158,148,29,103,50,86,110,111,226,93,53,84,146,112,102,143,111,98,164,100,163,99,123,95,136,111,244,144,227,129,176,143,24,92,104,102,241,95,137,108,72,150,129,141,108,136,145,100,240,121,206,87,89,106,16,98,72,84,88,78,11,122,233,96,132,111,218,139,127,98,30,144,139,154,228,121,3,84,244,117,1,99,25,83,96,108,223,143,27,95,112,154,59,128,127,159,136,79,58,92,100,141,197,127,165,101,189,112,69,81])
.concat([178,81,107,134,7,93,160,91,189,98,108,145,116,117,12,142,32,122,1,97,121,123,199,78,248,126,133,119,17,78,237,129,29,82,250,81,113,106,168,83,135,142,4,149,207,150,193,110,100,150,90,105,132,120,133,120,134,120,136,120,138,120,139,120,143,120,144,120,146,120,148,120,149,120,150,120,153,120,157,120,158,120,160,120,162,120,164,120,166,120,168,120,169,120,170,120,171,120,172,120,173,120,174,120,175,120,181,120,182,120,183,120,184,120,186,120,187,120,188,120,189,120,191,120,192,120,194,120,195,120,196,120,198,120,199,120,200,120,204,120,205,120,206,120,207,120,209,120,210,120,211,120,214,120,215,120,216,120,218,120,219,120,220,120,221,120,222,120,223,120,224,120,225,120,226,120,227,120,228,120,229,120,230,120,231,120,233,120,234,120,235,120,237,120,238,120,239,120,240,120,241,120,243,120,245,120,246,120,248,120,249,120,251,120,252,120,253,120,254,120,255,120,0,121,2,121,3,121,4,121,6,121,7,121,8,121,9,121,10,121,11,121,12,121,64,120,168,80,215,119,16,100,230,137,4,89,227,99,221,93,127,122,61,105,32,79,57,130,152,85,50,78,174,117,151,122,98,94,138,94,239,149,27,82,57,84,138,112,118,99,36,149,130,87,37,102,63,105,135,145,7,85,243,109,175,126,34,136,51,98,240,126,181,117,40,131,193,120,204,150,158,143,72,97,247,116,205,139,100,107,58,82,80,141,33,107,106,128,113,132,241,86,6,83,206,78,27,78,209,81,151,124,139,145,7,124,195,79,127,142,225,123,156,122,103,100,20,93,172,80,6,129,1,118,185,124,236,109,224,127,81,103,88,91,248,91,203,120,174,100,19,100,170,99,43,99,25,149,45,100,190,143,84,123,41,118,83,98,39,89,70,84,121,107,163,80,52,98,38,94,134,107,227,78,55,141,139,136,133,95,46,144,13,121,14,121,15,121,16,121,17,121,18,121,20,121,21,121,22,121,23,121,24,121,25,121,26,121,27,121,28,121,29,121,31,121,32,121,33,121,34,121,35,121,37,121,38,121,39,121,40,121,41,121,42,121,43,121,44,121,45,121,46,121,47,121,48,121,49,121,50,121,51,121,53,121,54,121,55,121,56,121,57,121,61,121,63,121,66,121,67,121,68,121,69,121,71,121,74,121,75,121,76,121,77,121,78,121,79,121,80,121,81,121,82,121,84,121,85,121,88,121,89,121,97,121,99,121,100,121,102,121,105,121,106,121,107,121,108,121,110,121,112,121,113,121,114,121,115,121,116,121,117,121,118,121,121,121,123,121,124,121,125,121,126,121,127,121,130,121,131,121,134,121,135,121,136,121,137,121,139,121,140,121,141,121,142,121,144,121,145,121,146,121,32,96,61,128,197,98,57,78,85,83,248,144,184,99,198,128,230,101,46,108,70,79,238,96,225,109,222,139,57,95,203,134,83,95,33,99,90,81,97,131,99,104,0,82,99,99,72,142,18,80,155,92,119,121,252,91,48,82,59,122,188,96,83,144,215,118,183,95,151,95,132,118,108,142,111,112,123,118,73,123,170,119,243,81,147,144,36,88,78,79,244,110,234,143,76,101,27,123,196,114,164,109,223,127,225,90,181,98,149,94,48,87,130,132,44,123,29,94,31,95,18,144,20,127,160,152,130,99,199,110,152,120,185,112,120,81,91,151,171,87,53,117,67,79,56,117,151,94,230,96,96,89,192,109,191,107,137,120,252,83,213,150,203,81,1,82,137,99,10,84,147,148,3,140,204,141,57,114,159,120,118,135,237,143,13,140,224,83,147,121,148,121,149,121,150,121,151,121,152,121,153,121,155,121,156,121,157,121,158,121,159,121,160,121,161,121,162,121,163,121,164,121,165,121,166,121,168,121,169,121,170,121,171,121,172,121,173,121,174,121,175,121,176,121,177,121,178,121,180,121,181,121,182,121,183,121,184,121,188,121,191,121,194,121,196,121,197,121,199,121,200,121,202,121,204,121,206,121,207,121,208,121,211,121,212,121,214,121,215,121,217,121,218,121,219,121,220,121,221,121,222,121,224,121,225,121,226,121,229,121,232,121,234,121,236,121,238,121,241,121,242,121,243,121,244,121,245,121,246,121,247,121,249,121,250,121,252,121,254,121,255,121,1,122,4,122,5,122,7,122,8,122,9,122,10,122,12,122,15,122,16,122,17,122,18,122,19,122,21,122,22,122,24,122,25,122,27,122,28,122,1,78,239,118,238,83,137,148,118,152,14,159,45,149,154,91,162,139,34,78,28,78,172,81,99,132,194,97,168,82,11,104,151,79,107,96,187,81,30,109,92,81,150,98,151,101,97,150,70,140,23,144,216,117,253,144,99,119,210,107,138,114,236,114,251,139,53,88,121,119,76,141,92,103,64,149,154,128,166,94,33,110,146,89,239,122,237,119,59,149,181,107,173,101,14,127,6,88,81,81,31,150,249,91,169,88,40,84,114,142,102,101,127,152,228,86,157,148,254,118,65,144,135,99,198,84,26,89,58,89,155,87,178,142,53,103,250,141,53,130,65,82,240,96,21,88,254,134,232,92,69,158,196,79,157,152,185,139,37,90,118,96,132,83,124,98,79,144,2,145,127,153,105,96,12,128,63,81,51,128,20,92,117,153,49,109,140,78,29,122,31,122,33,122,34,122,36,122,37,122,38,122,39,122,40,122,41,122,42,122,43,122,44,122,45,122,46,122,47,122,48,122,49,122,50,122,52,122,53,122,54,122,56,122,58,122,62,122,64,122,65,122,66,122,67,122,68,122,69,122,71,122,72,122,73,122,74,122,75,122,76,122,77,122,78,122,79,122,80,122,82,122,83,122,84,122,85,122,86,122,88,122,89,122,90,122,91,122,92,122,93,122,94,122,95,122,96,122,97,122,98,122,99,122,100,122,101,122,102,122,103,122,104,122,105,122,106,122,107,122,108,122,109,122,110,122,111,122,113,122,114,122,115,122,117,122,123,122,124,122,125,122,126,122,130,122,133,122,135,122,137,122,138,122,139,122,140,122,142,122,143,122,144,122,147,122,148,122,153,122,154,122,155,122,158,122,161,122,162,122,48,141,209,83,90,127,79,123,16,79,79,78,0,150,213,108,208,115,233,133,6,94,106,117,251,127,10,106,254,119,146,148,65,126,225,81,230,112,205,83,212,143,3,131,41,141,175,114,109,153,219,108,74,87,179,130,185,101,170,128,63,98,50,150,168,89,255,78,191,139,186,126,62,101,242,131,94,151,97,85,222,152,165,128,42,83,253,139,32,84,186,128,159,94,184,108,57,141,172,130,90,145,41,84,27,108,6,82,183,126,95,87,26,113,126,108,137,124,75,89,253,78,255,95,36,97,170,124,48,78,1,92,171,103,2,135,240,92,11,149,206,152,175,117,253,112,34,144,175,81,29,127,189,139,73,89,228,81,91,79,38,84,43,89,119,101,164,128,117,91,118,98,194,98,144,143,69,94,31,108,38,123,15,79,216,79,13,103,163,122,164,122,167,122,169,122,170,122,171,122,174,122,175,122,176,122,177,122,178,122,180,122,181,122,182,122,183,122,184,122,185,122,186,122,187,122,188,122,189,122,190,122,192,122,193,122,194,122,195,122,196,122,197,122,198,122,199,122,200,122,201,122,202,122,204,122,205,122,206,122,207,122,208,122,209,122,210,122,211,122,212,122,213,122,215,122,216,122,218,122,219,122,220,122,221,122,225,122,226,122,228,122,231,122,232,122,233,122,234,122,235,122,236,122,238,122,240,122,241,122,242,122,243,122,244,122,245,122,246,122,247,122,248,122,251,122,252,122,254,122,0,123,1,123,2,123,5,123,7,123,9,123,12,123,13,123,14,123,16,123,18,123,19,123,22,123,23,123,24,123,26,123,28,123,29,123,31,123,33,123,34,123,35,123,39,123,41,123,45,123,110,109,170,109,143,121,177,136,23,95,43,117,154,98,133,143,239,79,220,145,167,101,47,129,81,129,156,94,80,129,116,141,111,82,134,137,75,141,13,89,133,80,216,78,28,150,54,114,121,129,31,141,204,91,163,139,68,150,135,89,26,127,144,84,118,86,14,86,229,139,57,101,130,105,153,148,214,118,137,110,114,94,24,117,70,103,209,103,255,122,157,128,118,141,31,97,198,121,98,101,99,141,136,81,26,82,162,148,56,127,155,128,178,126,151,92,47,110,96,103,217,123,139,118,216,154,143,129,148,127,213,124,30,100,80,149,63,122,74,84,229,84,76,107,1,100,8,98,61,158,243,128,153,117,114,82,105,151,91,132,60,104,228,134,1,150,148,150,236,148,42,78,4,84,217,126,57,104,223,141,21,128,244,102,154,94,185,127,47,123,48,123,50,123,52,123,53,123,54,123,55,123,57,123,59,123,61,123,63,123,64,123,65,123,66,123,67,123,68,123,70,123,72,123,74,123,77,123,78,123,83,123,85,123,87,123,89,123,92,123,94,123,95,123,97,123,99,123,100,123,101,123,102,123,103,123,104,123,105,123,106,123,107,123,108,123,109,123,111,123,112,123,115,123,116,123,118,123,120,123,122,123,124,123,125,123,127,123,129,123,130,123,131,123,132,123,134,123,135,123,136,123,137,123,138,123,139,123,140,123,142,123,143,123,145,123,146,123,147,123,150,123,152,123,153,123,154,123,155,123,158,123,159,123,160,123,163,123,164,123,165,123,174,123,175,123,176,123,178,123,179,123,181,123,182,123,183,123,185,123,186,123,187,123,188,123,189,123,190,123,191,123,192,123,194,123,195,123,196,123,194,87,63,128,151,104,229,93,59,101,159,82,109,96,154,159,155,79,172,142,108,81,171,91,19,95,233,93,94,108,241,98,33,141,113,81,169,148,254,82,159,108,223,130,215,114,162,87,132,103,45,141,31,89,156,143,199,131,149,84,141,123,48,79,189,108,100,91,209,89,19,159,228,83,202,134,168,154,55,140,161,128,69,101,126,152,250,86,199,150,46,82,220,116,80,82,225,91,2,99,2,137,86,78,208,98,42,96,250,104,115,81,152,91,160,81,194,137,161,123,134,153,80,127,239,96,76,112,47,141,73,81,127,94,27,144,112,116,196,137,45,87,69,120,82,95,159,159,250,149,104,143,60,155,225,139,120,118,66,104,220,103,234,141,53,141,61,82,138,143,218,110,205,104,5,149,237,144,253,86,156,103,249,136,199,143,200,84,197,123,200,123,201,123,202,123,203,123,205,123,206,123,207,123,208,123,210,123,212,123,213,123,214,123,215,123,216,123,219,123,220,123,222,123,223,123,224,123,226,123,227,123,228,123,231,123,232,123,233,123,235,123,236,123,237,123,239,123,240,123,242,123,243,123,244,123,245,123,246,123,248,123,249,123,250,123,251,123,253,123,255,123,0,124,1,124,2,124,3,124,4,124,5,124,6,124,8,124,9,124,10,124,13,124,14,124,16,124,17,124,18,124,19,124,20,124,21,124,23,124,24,124,25,124,26,124,27,124,28,124,29,124,30,124,32,124,33,124,34,124,35,124,36,124,37,124,40,124,41,124,43,124,44,124,45,124,46,124,47,124,48,124,49,124,50,124,51,124,52,124,53,124,54,124,55,124,57,124,58,124,59,124,60,124,61,124,62,124,66,124,184,154,105,91,119,109,38,108,165,78,179,91,135,154,99,145,168,97,175,144,233,151,43,84,181,109,210,91,253,81,138,85,85,127,240,127,188,100,77,99,241,101,190,97,141,96,10,113,87,108,73,108,47,89,109,103,42,130,213,88,142,86,106,140,235,107,221,144,125,89,23,128,247,83,105,109,117,84,157,85,119,131,207,131,56,104,190,121,140,84,85,79,8,84,210,118,137,140,2,150,179,108,184,109,107,141,16,137,100,158,58,141,63,86,209,158,213,117,136,95,224,114,104,96,252,84,168,78,42,106,97,136,82,96,112,143,196,84,216,112,121,134,63,158,42,109,143,91,24,95,162,126,137,85,175,79,52,115,60,84,154,83,25,80,14,84,124,84,78,78,253,95,90,116,246,88,107,132,225,128,116,135,208,114,202,124,86,110,67,124,68,124,69,124,70,124,71,124,72,124,73,124,74,124,75,124,76,124,78,124,79,124,80,124,81,124,82,124,83,124,84,124,85,124,86,124,87,124,88,124,89,124,90,124,91,124,92,124,93,124,94,124,95,124,96,124,97,124,98,124,99,124,100,124,101,124,102,124,103,124,104,124,105,124,106,124,107,124,108,124,109,124,110,124,111,124,112,124,113,124,114,124,117,124,118,124,119,124,120,124,121,124,122,124,126,124,127,124,128,124,129,124,130,124,131,124,132,124,133,124,134,124,135,124,136,124,138,124,139,124,140,124,141,124,142,124,143,124,144,124,147,124,148,124,150,124,153,124,154,124,155,124,160,124,161,124,163,124,166,124,167,124,168,124,169,124,171,124,172,124,173,124,175,124,176,124,180,124,181,124,182,124,183,124,184,124,186,124,187,124,39,95,78,134,44,85,164,98,146,78,170,108,55,98,177,130,215,84,78,83,62,115,209,110,59,117,18,82,22,83,221,139,208,105,138,95,0,96,238,109,79,87,34,107,175,115,83,104,216,143,19,127,98,99,163,96,36,85,234,117,98,140,21,113,163,109,166,91,123,94,82,131,76,97,196,158,250,120,87,135,39,124,135,118,240,81,246,96,76,113,67,102,76,94,77,96,14,140,112,112,37,99,137,143,189,95,98,96,212,134,222,86,193,107,148,96,103,97,73,83,224,96,102,102,63,141,253,121,26,79,233,112,71,108,179,139,242,139,216,126,100,131,15,102,90,90,66,155,81,109,247,109,65,140,59,109,25,79,107,112,183,131,22,98,209,96,13,151,39,141,120,121,251,81,62,87,250,87,58,103,120,117,61,122,239,121,149,123,191,124,192,124,194,124,195,124,196,124,198,124,201,124,203,124,206,124,207,124,208,124,209,124,210,124,211,124,212,124,216,124,218,124,219,124,221,124,222,124,225,124,226,124,227,124,228,124,229,124,230,124,231,124,233,124,234,124,235,124,236,124,237,124,238,124,240,124,241,124,242,124,243,124,244,124,245,124,246,124,247,124,249,124,250,124,252,124,253,124,254,124,255,124,0,125,1,125,2,125,3,125,4,125,5,125,6,125,7,125,8,125,9,125,11,125,12,125,13,125,14,125,15,125,16,125,17,125,18,125,19,125,20,125,21,125,22,125,23,125,24,125,25,125,26,125,27,125,28,125,29,125,30,125,31,125,33,125,35,125,36,125,37,125,38,125,40,125,41,125,42,125,44,125,45,125,46,125,48,125,49,125,50,125,51,125,52,125,53,125,54,125,140,128,101,153,249,143,192,111,165,139,33,158,236,89,233,126,9,127,9,84,129,103,216,104,145,143,77,124,198,150,202,83,37,96,190,117,114,108,115,83,201,90,167,126,36,99,224,81,10,129,241,93,223,132,128,98,128,81,99,91,14,79,109,121,66,82,184,96,78,109,196,91,194,91,161,139,176,139,226,101,204,95,69,150,147,89,231,126,170,126,9,86,183,103,57,89,115,79,182,91,160,82,90,131,138,152,62,141,50,117,190,148,71,80,60,122,247,78,182,103,126,154,193,90,124,107,209,118,90,87,22,92,58,123,244,149,78,113,124,81,169,128,112,130,120,89,4,127,39,131,192,104,236,103,177,120,119,120,227,98,97,99,128,123,237,79,106,82,207,81,80,131,219,105,116,146,245,141,49,141,193,137,46,149,173,123,246,78,55,125,56,125,57,125,58,125,59,125,60,125,61,125,62,125,63,125,64,125,65,125,66,125,67,125,68,125,69,125,70,125,71,125,72,125,73,125,74,125,75,125,76,125,77,125,78,125,79,125,80,125,81,125,82,125,83,125,84,125,85,125,86,125,87,125,88,125,89,125,90,125,91,125,92,125,93,125,94,125,95,125,96,125,97,125,98,125,99,125,100,125,101,125,102,125,103,125,104,125,105,125,106,125,107,125,108,125,109,125,111,125,112,125,113,125,114,125,115,125,116,125,117,125,118,125,120,125,121,125,122,125,123,125,124,125,125,125,126,125,127,125,128,125,129,125,130,125,131,125,132,125,133,125,134,125,135,125,136,125,137,125,138,125,139,125,140,125,141,125,142,125,143,125,144,125,145,125,146,125,147,125,148,125,149,125,150,125,151,125,152,125,101,80,48,130,81,82,111,153,16,110,133,110,167,109,250,94,245,80,220,89,6,92,70,109,95,108,134,117,139,132,104,104,86,89,178,139,32,83,113,145,77,150,73,133,18,105,1,121,38,113,246,128,164,78,202,144,71,109,132,154,7,90,188,86,5,100,240,148,235,119,165,79,26,129,225,114,210,137,122,153,52,127,222,126,127,82,89,101,117,145,127,143,131,143,235,83,150,122,237,99,165,99,134,118,248,121,87,136,54,150,42,98,171,82,130,130,84,104,112,103,119,99,107,119,237,122,1,109,211,126,227,137,208,89,18,98,201,133,165,130,76,117,31,80,203,78,165,117,235,139,74,92,254,93,75,123,164,101,209,145,202,78,37,109,95,137,39,125,38,149,197,78,40,140,219,143,115,151,75,102,129,121,209,143,236,112,120,109,153,125,154,125,155,125,156,125,157,125,158,125,159,125,160,125,161,125,162,125,163,125,164,125,165,125,167,125,168,125,169,125,170,125,171,125,172,125,173,125,175,125,176,125,177,125,178,125,179,125,180,125,181,125,182,125,183,125,184,125,185,125,186,125,187,125,188,125,189,125,190,125,191,125,192,125,193,125,194,125,195,125,196,125,197,125,198,125,199,125,200,125,201,125,202,125,203,125,204,125,205,125,206,125,207,125,208,125,209,125,210,125,211,125,212,125,213,125,214,125,215,125,216,125,217,125,218,125,219,125,220,125,221,125,222,125,223,125,224,125,225,125,226,125,227,125,228,125,229,125,230,125,231,125,232,125,233,125,234,125,235,125,236,125,237,125,238,125,239,125,240,125,241,125,242,125,243,125,244,125,245,125,246,125,247,125,248,125,249,125,250,125,61,92,178,82,70,131,98,81,14,131,91,119,118,102,184,156,172,78,202,96,190,124,179,124,207,126,149,78,102,139,111,102,136,152,89,151,131,88,108,101,92,149,132,95,201,117,86,151,223,122,222,122,192,81,175,112,152,122,234,99,118,122,160,126,150,115,237,151,69,78,120,112,93,78,82,145,169,83,81,101,231,101,252,129,5,130,142,84,49,92,154,117,160,151,216,98,217,114,189,117,69,92,121,154,202,131,64,92,128,84,233,119,62,78,174,108,90,128,210,98,110,99,232,93,119,81,221,141,30,142,47,149,241,79,229,83,231,96,172,112,103,82,80,99,67,158,31,90,38,80,55,119,119,83,226,126,133,100,43,101,137,98,152,99,20,80,53,114,201,137,179,81,192,139,221,126,71,87,204,131,167,148,155,81,27,84,251,92,251,125,252,125,253,125,254,125,255,125,0,126,1,126,2,126,3,126,4,126,5,126,6,126,7,126,8,126,9,126,10,126,11,126,12,126,13,126,14,126,15,126,16,126,17,126,18,126,19,126,20,126,21,126,22,126,23,126,24,126,25,126,26,126,27,126,28,126,29,126,30,126,31,126,32,126,33,126,34,126,35,126,36,126,37,126,38,126,39,126,40,126,41,126,42,126,43,126,44,126,45,126,46,126,47,126,48,126,49,126,50,126,51,126,52,126,53,126,54,126,55,126,56,126,57,126,58,126,60,126,61,126,62,126,63,126,64,126,66,126,67,126,68,126,69,126,70,126,72,126,73,126,74,126,75,126,76,126,77,126,78,126,79,126,80,126,81,126,82,126,83,126,84,126,85,126,86,126,87,126,88,126,89,126,90,126,91,126,92,126,93,126,202,79,227,122,90,109,225,144,143,154,128,85,150,84,97,83,175,84,0,95,233,99,119,105,239,81,104,97,10,82,42,88,216,82,78,87,13,120,11,119,183,94,119,97,224,124,91,98,151,98,162,78,149,112,3,128,247,98,228,112,96,151,119,87,219,130,239,103,245,104,213,120,151,152,209,121,243,88,179,84,239,83,52,110,75,81,59,82,162,91,254,139,175,128,67,85,166,87,115,96,81,87,45,84,122,122,80,96,84,91,167,99,160,98,227,83,99,98,199,91,175,103,237,84,159,122,230,130,119,145,147,94,228,136,56,89,174,87,14,99,232,141,239,128,87,87,119,123,169,79,235,95,189,91,62,107,33,83,80,123,194,114,70,104,255,119,54,119,247,101,181,81,143,78,212,118,191,92,165,122,117,132,78,89,65,155,128,80,94,126,95,126,96,126,97,126,98,126,99,126,100,126,101,126,102,126,103,126,104,126,105,126,106,126,107,126,108,126,109,126,110,126,111,126,112,126,113,126,114,126,115,126,116,126,117,126,118,126,119,126,120,126,121,126,122,126,123,126,124,126,125,126,126,126,127,126,128,126,129,126,131,126,132,126,133,126,134,126,135,126,136,126,137,126,138,126,139,126,140,126,141,126,142,126,143,126,144,126,145,126,146,126,147,126,148,126,149,126,150,126,151,126,152,126,153,126,154,126,156,126,157,126,158,126,174,126,180,126,187,126,188,126,214,126,228,126,236,126,249,126,10,127,16,127,30,127,55,127,57,127,59,127,60,127,61,127,62,127,63,127,64,127,65,127,67,127,70,127,71,127,72,127,73,127,74,127,75,127,76,127,77,127,78,127,79,127,82,127,83,127,136,153,39,97,131,110,100,87,6,102,70,99,240,86,236,98,105,98,211,94,20,150,131,87,201,98,135,85,33,135,74,129,163,143,102,85,177,131,101,103,86,141,221,132,106,90,15,104,230,98,238,123,17,150,112,81,156,111,48,140,253,99,200,137,210,97,6,127,194,112,229,110,5,116,148,105,252,114,202,94,206,144,23,103,106,109,94,99,179,82,98,114,1,128,108,79,229,89,106,145,217,112,157,109,210,82,80,78,247,150,109,149,126,133,202,120,47,125,33,81,146,87,194,100,139,128,123,124,234,108,241,104,94,105,183,81,152,83,168,104,129,114,206,158,241,123,248,114,187,121,19,111,6,116,78,103,204,145,164,156,60,121,137,131,84,131,15,84,23,104,61,78,137,83,177,82,62,120,134,83,41,82,136,80,139,79,208,79,86,127,89,127,91,127,92,127,93,127,94,127,96,127,99,127,100,127,101,127,102,127,103,127,107,127,108,127,109,127,111,127,112,127,115,127,117,127,118,127,119,127,120,127,122,127,123,127,124,127,125,127,127,127,128,127,130,127,131,127,132,127,133,127,134,127,135,127,136,127,137,127,139,127,141,127,143,127,144,127,145,127,146,127,147,127,149,127,150,127,151,127,152,127,153,127,155,127,156,127,160,127,162,127,163,127,165,127,166,127,168,127,169,127,170,127,171,127,172,127,173,127,174,127,177,127,179,127,180,127,181,127,182,127,183,127,186,127,187,127,190,127,192,127,194,127,195,127,196,127,198,127,199,127,200,127,201,127,203,127,205,127,207,127,208,127,209,127,210,127,211,127,214,127,215,127,217,127,218,127,219,127,220,127,221,127,222,127,226,127,227,127,226,117,203,122,146,124,165,108,182,150,155,82,131,116,233,84,233,79,84,128,178,131,222,143,112,149,201,94,28,96,159,109,24,94,91,101,56,129,254,148,75,96,188,112,195,126,174,124,201,81,129,104,177,124,111,130,36,78,134,143,207,145,126,102,174,78,5,140,169,100,74,128,218,80,151,117,206,113,229,91,189,143,102,111,134,78,130,100,99,149,214,94,153,101,23,82,194,136,200,112,163,82,14,115,51,116,151,103,247,120,22,151,52,78,187,144,222,156,203,109,219,81,65,141,29,84,206,98,178,115,241,131,246,150,132,159,195,148,54,79,154,127,204,81,117,112,117,150,173,92,134,152,230,83,228,78,156,110,9,116,180,105,107,120,143,153,89,117,24,82,36,118,65,109,243,103,109,81,153,159,75,128,153,84,60,123,191,122,228,127,231,127,232,127,234,127,235,127,236,127,237,127,239,127,242,127,244,127,245,127,246,127,247,127,248,127,249,127,250,127,253,127,254,127,255,127,2,128,7,128,8,128,9,128,10,128,14,128,15,128,17,128,19,128,26,128,27,128,29,128,30,128,31,128,33,128,35,128,36,128,43,128,44,128,45,128,46,128,47,128,48,128,50,128,52,128,57,128,58,128,60,128,62,128,64,128,65,128,68,128,69,128,71,128,72,128,73,128,78,128,79,128,80,128,81,128,83,128,85,128,86,128,87,128,89,128,91,128,92,128,93,128,94,128,95,128,96,128,97,128,98,128,99,128,100,128,101,128,102,128,103,128,104,128,107,128,108,128,109,128,110,128,111,128,112,128,114,128,115,128,116,128,117,128,118,128,119,128,120,128,121,128,122,128,123,128,124,128,125,128,134,150,132,87,226,98,71,150,124,105,4,90,2,100,211,123,15,111,75,150,166,130,98,83,133,152,144,94,137,112,179,99,100,83,79,134,129,156,147,158,140,120,50,151,239,141,66,141,127,158,94,111,132,121,85,95,70,150,46,98,116,154,21,84,221,148,163,79,197,101,101,92,97,92,21,127,81,134,47,108,139,95,135,115,228,110,255,126,230,92,27,99,106,91,230,110,117,83,113,78,160,99,101,117,161,98,110,143,38,79,209,78,166,108,182,126,186,139,29,132,186,135,87,127,59,144,35,149,169,123,161,154,248,136,61,132,27,109,134,154,220,126,136,89,187,158,155,115,1,120,130,134,108,154,130,154,27,86,23,84,203,87,112,78,166,158,86,83,200,143,9,129,146,119,146,153,238,134,225,110,19,133,252,102,98,97,43,111,126,128,129,128,130,128,133,128,136,128,138,128,141,128,142,128,143,128,144,128,145,128,146,128,148,128,149,128,151,128,153,128,158,128,163,128,166,128,167,128,168,128,172,128,176,128,179,128,181,128,182,128,184,128,185,128,187,128,197,128,199,128,200,128,201,128,202,128,203,128,207,128,208,128,209,128,210,128,211,128,212,128,213,128,216,128,223,128,224,128,226,128,227,128,230,128,238,128,245,128,247,128,249,128,251,128,254,128,255,128,0,129,1,129,3,129,4,129,5,129,7,129,8,129,11,129,12,129,21,129,23,129,25,129,27,129,28,129,29,129,31,129,32,129,33,129,34,129,35,129,36,129,37,129,38,129,39,129,40,129,41,129,42,129,43,129,45,129,46,129,48,129,51,129,52,129,53,129,55,129,57,129,58,129,59,129,60,129,61,129,63,129,41,140,146,130,43,131,242,118,19,108,217,95,189,131,43,115,5,131,26,149,219,107,219,119,198,148,111,83,2,131,146,81,61,94,140,140,56,141,72,78,171,115,154,103,133,104,118,145,9,151,100,113,161,108,9,119,146,90,65,149,207,107,142,127,39,102,208,91,185,89,154,90,232,149,247,149,236,78,12,132,153,132,172,106,223,118,48,149,27,115,166,104,95,91,47,119,154,145,97,151,220,124,247,143,28,140,37,95,115,124,216,121,197,137,204,108,28,135,198,91,66,94,201,104,32,119,245,126,149,81,77,81,201,82,41,90,5,127,98,151,215,130,207,99,132,119,208,133,210,121,58,110,153,94,153,89,17,133,109,112,17,108,191,98,191,118,79,101,175,96,253,149,14,102,159,135,35,158,237,148,13,84,125,84,44,140,120,100,64,129,65,129,66,129,67,129,68,129,69,129,71,129,73,129,77,129,78,129,79,129,82,129,86,129,87,129,88,129,91,129,92,129,93,129,94,129,95,129,97,129,98,129,99,129,100,129,102,129,104,129,106,129,107,129,108,129,111,129,114,129,115,129,117,129,118,129,119,129,120,129,129,129,131,129,132,129,133,129,134,129,135,129,137,129,139,129,140,129,141,129,142,129,144,129,146,129,147,129,148,129,149,129,150,129,151,129,153,129,154,129,158,129,159,129,160,129,161,129,162,129,164,129,165,129,167,129,169,129,171,129,172,129,173,129,174,129,175,129,176,129,177,129,178,129,180,129,181,129,182,129,183,129,184,129,185,129,188,129,189,129,190,129,191,129,196,129,197,129,199,129,200,129,201,129,203,129,205,129,206,129,207,129,208,129,209,129,210,129,211,129,121,100,17,134,33,106,156,129,232,120,105,100,84,155,185,98,43,103,171,131,168,88,216,158,171,108,32,111,222,91,76,150,11,140,95,114,208,103,199,98,97,114,169,78,198,89,205,107,147,88,174,102,85,94,223,82,85,97,40,103,238,118,102,119,103,114,70,122,255,98,234,84,80,84,160,148,163,144,28,90,179,126,22,108,67,78,118,89,16,128,72,89,87,83,55,117,190,150,202,86,32,99,17,129,124,96,249,149,214,109,98,84,129,153,133,81,233,90,253,128,174,89,19,151,42,80,229,108,60,92,223,98,96,79,63,83,123,129,6,144,186,110,43,133,200,98,116,94,190,120,181,100,123,99,245,95,24,90,127,145,31,158,63,92,79,99,66,128,125,91,110,85,74,149,77,149,133,109,168,96,224,103,222,114,221,81,129,91,212,129,213,129,214,129,215,129,216,129,217,129,218,129,219,129,220,129,221,129,222,129,223,129,224,129,225,129,226,129,228,129,229,129,230,129,232,129,233,129,235,129,238,129,239,129,240,129,241,129,242,129,245,129,246,129,247,129,248,129,249,129,250,129,253,129,255,129,3,130,7,130,8,130,9,130,10,130,11,130,14,130,15,130,17,130,19,130,21,130,22,130,23,130,24,130,25,130,26,130,29,130,32,130,36,130,37,130,38,130,39,130,41,130,46,130,50,130,58,130,60,130,61,130,63,130,64,130,65,130,66,130,67,130,69,130,70,130,72,130,74,130,76,130,77,130,78,130,80,130,81,130,82,130,83,130,84,130,85,130,86,130,87,130,89,130,91,130,92,130,93,130,94,130,96,130,97,130,98,130,99,130,100,130,101,130,102,130,103,130,105,130,231,98,222,108,91,114,109,98,174,148,189,126,19,129,83,109,156,81,4,95,116,89,170,82,18,96,115,89,150,102,80,134,159,117,42,99,230,97,239,124,250,139,230,84,39,107,37,158,180,107,213,133,85,84,118,80,164,108,106,85,180,141,44,114,21,94,21,96,54,116,205,98,146,99,76,114,152,95,67,110,62,109,0,101,88,111,216,118,208,120,252,118,84,117,36,82,219,83,83,78,158,94,193,101,42,128,214,128,155,98,134,84,40,82,174,112,141,136,209,141,225,108,120,84,218,128,249,87,244,136,84,141,106,150,77,145,105,79,155,108,183,85,198,118,48,120,168,98,249,112,142,111,109,95,236,132,218,104,124,120,247,123,168,129,11,103,79,158,103,99,176,120,111,87,18,120,57,151,121,98,171,98,136,82,53,116,215,107,106,130,107,130,108,130,109,130,113,130,117,130,118,130,119,130,120,130,123,130,124,130,128,130,129,130,131,130,133,130,134,130,135,130,137,130,140,130,144,130,147,130,148,130,149,130,150,130,154,130,155,130,158,130,160,130,162,130,163,130,167,130,178,130,181,130,182,130,186,130,187,130,188,130,191,130,192,130,194,130,195,130,197,130,198,130,201,130,208,130,214,130,217,130,218,130,221,130,226,130,231,130,232,130,233,130,234,130,236,130,237,130,238,130,240,130,242,130,243,130,245,130,246,130,248,130,250,130,252,130,253,130,254,130,255,130,0,131,10,131,11,131,13,131,16,131,18,131,19,131,22,131,24,131,25,131,29,131,30,131,31,131,32,131,33,131,34,131,35,131,36,131,37,131,38,131,41,131,42,131,46,131,48,131,50,131,55,131,59,131,61,131,100,85,62,129,178,117,174,118,57,83,222,117,251,80,65,92,108,139,199,123,79,80,71,114,151,154,216,152,2,111,226,116,104,121,135,100,165,119,252,98,145,152,43,141,193,84,88,128,82,78,106,87,249,130,13,132,115,94,237,81,246,116,196,139,79,92,97,87,252,108,135,152,70,90,52,120,68,155,235,143,149,124,86,82,81,98,250,148,198,78,134,131,97,132,233,131,178,132,212,87,52,103,3,87,110,102,102,109,49,140,221,102,17,112,31,103,58,107,22,104,26,98,187,89,3,78,196,81,6,111,210,103,143,108,118,81,203,104,71,89,103,107,102,117,14,93,16,129,80,159,215,101,72,121,65,121,145,154,119,141,130,92,94,78,1,79,47,84,81,89,12,120,104,86,20,108,196,143,3,95,125,108,227,108,171,139,144,99,62,131,63,131,65,131,66,131,68,131,69,131,72,131,74,131,75,131,76,131,77,131,78,131,83,131,85,131,86,131,87,131,88,131,89,131,93,131,98,131,112,131,113,131,114,131,115,131,116,131,117,131,118,131,121,131,122,131,126,131,127,131,128,131,129,131,130,131,131,131,132,131,135,131,136,131,138,131,139,131,140,131,141,131,143,131,144,131,145,131,148,131,149,131,150,131,151,131,153,131,154,131,157,131,159,131,161,131,162,131,163,131,164,131,165,131,166,131,167,131,172,131,173,131,174,131,175,131,181,131,187,131,190,131,191,131,194,131,195,131,196,131,198,131,200,131,201,131,203,131,205,131,206,131,208,131,209,131,210,131,211,131,213,131,215,131,217,131,218,131,219,131,222,131,226,131,227,131,228,131,230,131,231,131,232,131,235,131,236,131,237,131,112,96,61,109,117,114,102,98,142,148,197,148,67,83,193,143,126,123,223,78,38,140,126,78,212,158,177,148,179,148,77,82,92,111,99,144,69,109,52,140,17,88,76,93,32,107,73,107,170,103,91,84,84,129,140,127,153,88,55,133,58,95,162,98,71,106,57,149,114,101,132,96,101,104,167,119,84,78,168,79,231,93,152,151,172,100,216,127,237,92,207,79,141,122,7,82,4,131,20,78,47,96,131,122,166,148,181,79,178,78,230,121,52,116,228,82,185,130,210,100,189,121,221,91,129,108,82,151,123,143,34,108,62,80,127,83,5,110,206,100,116,102,48,108,197,96,119,152,247,139,134,94,60,116,119,122,203,121,24,78,177,144,3,116,66,108,218,86,75,145,197,108,139,141,58,83,198,134,242,102,175,142,72,92,113,154,32,110,238,131,239,131,243,131,244,131,245,131,246,131,247,131,250,131,251,131,252,131,254,131,255,131,0,132,2,132,5,132,7,132,8,132,9,132,10,132,16,132,18,132,19,132,20,132,21,132,22,132,23,132,25,132,26,132,27,132,30,132,31,132,32,132,33,132,34,132,35,132,41,132,42,132,43,132,44,132,45,132,46,132,47,132,48,132,50,132,51,132,52,132,53,132,54,132,55,132,57,132,58,132,59,132,62,132,63,132,64,132,65,132,66,132,67,132,68,132,69,132,71,132,72,132,73,132,74,132,75,132,76,132,77,132,78,132,79,132,80,132,82,132,83,132,84,132,85,132,86,132,88,132,93,132,94,132,95,132,96,132,98,132,100,132,101,132,102,132,103,132,104,132,106,132,110,132,111,132,112,132,114,132,116,132,119,132,121,132,123,132,124,132,214,83,54,90,139,159,163,141,187,83,8,87,167,152,67,103,155,145,201,108,104,81,202,117,243,98,172,114,56,82,157,82,58,127,148,112,56,118,116,83,74,158,183,105,110,120,192,150,217,136,164,127,54,113,195,113,137,81,211,103,228,116,228,88,24,101,183,86,169,139,118,153,112,98,213,126,249,96,237,112,236,88,193,78,186,78,205,95,231,151,251,78,164,139,3,82,138,89,171,126,84,98,205,78,229,101,14,98,56,131,201,132,99,131,141,135,148,113,182,110,185,91,210,126,151,81,201,99,212,103,137,128,57,131,21,136,18,81,122,91,130,89,177,143,115,78,93,108,101,81,37,137,111,143,46,150,74,133,94,116,16,149,240,149,166,109,229,130,49,95,146,100,18,109,40,132,110,129,195,156,94,88,91,141,9,78,193,83,125,132,126,132,127,132,128,132,129,132,131,132,132,132,133,132,134,132,138,132,141,132,143,132,144,132,145,132,146,132,147,132,148,132,149,132,150,132,152,132,154,132,155,132,157,132,158,132,159,132,160,132,162,132,163,132,164,132,165,132,166,132,167,132,168,132,169,132,170,132,171,132,172,132,173,132,174,132,176,132,177,132,179,132,181,132,182,132,183,132,187,132,188,132,190,132,192,132,194,132,195,132,197,132,198,132,199,132,200,132,203,132,204,132,206,132,207,132,210,132,212,132,213,132,215,132,216,132,217,132,218,132,219,132,220,132,222,132,225,132,226,132,228,132,231,132,232,132,233,132,234,132,235,132,237,132,238,132,239,132,241,132,242,132,243,132,244,132,245,132,246,132,247,132,248,132,249,132,250,132,251,132,253,132,254,132,0,133,1,133,2,133,30,79,99,101,81,104,211,85,39,78,20,100,154,154,107,98,194,90,95,116,114,130,169,109,238,104,231,80,142,131,2,120,64,103,57,82,153,108,177,126,187,80,101,85,94,113,91,123,82,102,202,115,235,130,73,103,113,92,32,82,125,113,107,136,234,149,85,150,197,100,97,141,179,129,132,85,85,108,71,98,46,127,146,88,36,79,70,85,79,141,76,102,10,78,26,92,243,136,162,104,78,99,13,122,231,112,141,130,250,82,246,151,17,92,232,84,181,144,205,126,98,89,74,141,199,134,12,130,13,130,102,141,68,100,4,92,81,97,137,109,62,121,190,139,55,120,51,117,123,84,56,79,171,142,241,109,32,90,197,126,94,121,136,108,161,91,118,90,26,117,190,128,78,97,23,110,240,88,31,117,37,117,114,114,71,83,243,126,3,133,4,133,5,133,6,133,7,133,8,133,9,133,10,133,11,133,13,133,14,133,15,133,16,133,18,133,20,133,21,133,22,133,24,133,25,133,27,133,28,133,29,133,30,133,32,133,34,133,35,133,36,133,37,133,38,133,39,133,40,133,41,133,42,133,45,133,46,133,47,133,48,133,49,133,50,133,51,133,52,133,53,133,54,133,62,133,63,133,64,133,65,133,66,133,68,133,69,133,70,133,71,133,75,133,76,133,77,133,78,133,79,133,80,133,81,133,82,133,83,133,84,133,85,133,87,133,88,133,90,133,91,133,92,133,93,133,95,133,96,133,97,133,98,133,99,133,101,133,102,133,103,133,105,133,106,133,107,133,108,133,109,133,110,133,111,133,112,133,113,133,115,133,117,133,118,133,119,133,120,133,124,133,125,133,127,133,128,133,129,133,1,119,219,118,105,82,220,128,35,87,8,94,49,89,238,114,189,101,127,110,215,139,56,92,113,134,65,83,243,119,254,98,246,101,192,78,223,152,128,134,158,91,198,139,242,83,226,119,127,79,78,92,118,154,203,89,15,95,58,121,235,88,22,78,255,103,139,78,237,98,147,138,29,144,191,82,47,102,220,85,108,86,2,144,213,78,141,79,202,145,112,153,15,108,2,94,67,96,164,91,198,137,213,139,54,101,75,98,150,153,136,91,255,91,136,99,46,85,215,83,38,118,125,81,44,133,162,103,179,104,138,107,146,98,147,143,212,83,18,130,209,109,143,117,102,78,78,141,112,91,159,113,175,133,145,102,217,102,114,127,0,135,205,158,32,159,94,92,47,103,240,143,17,104,95,103,13,98,214,122,133,88,182,94,112,101,49,111,130,133,131,133,134,133,136,133,137,133,138,133,139,133,140,133,141,133,142,133,144,133,145,133,146,133,147,133,148,133,149,133,150,133,151,133,152,133,153,133,154,133,157,133,158,133,159,133,160,133,161,133,162,133,163,133,165,133,166,133,167,133,169,133,171,133,172,133,173,133,177,133,178,133,179,133,180,133,181,133,182,133,184,133,186,133,187,133,188,133,189,133,190,133,191,133,192,133,194,133,195,133,196,133,197,133,198,133,199,133,200,133,202,133,203,133,204,133,205,133,206,133,209,133,210,133,212,133,214,133,215,133,216,133,217,133,218,133,219,133,221,133,222,133,223,133,224,133,225,133,226,133,227,133,229,133,230,133,231,133,232,133,234,133,235,133,236,133,237,133,238,133,239,133,240,133,241,133,242,133,243,133,244,133,245,133,246,133,247,133,248,133,85,96,55,82,13,128,84,100,112,136,41,117,5,94,19,104,244,98,28,151,204,83,61,114,1,140,52,108,97,119,14,122,46,84,172,119,122,152,28,130,244,139,85,120,20,103,193,112,175,101,149,100,54,86,29,96,193,121,248,83,29,78,123,107,134,128,250,91,227,85,219,86,58,79,60,79,114,153,243,93,126,103,56,128,2,96,130,152,1,144,139,91,188,139,245,139,28,100,88,130,222,100,253,85,207,130,101,145,215,79,32,125,31,144,159,124,243,80,81,88,175,110,191,91,201,139,131,128,120,145,156,132,151,123,125,134,139,150,143,150,229,126,211,154,142,120,129,92,87,122,66,144,167,150,95,121,89,91,95,99,11,123,209,132,173,104,6,85,41,127,16,116,34,125,1,149,64,98,76,88,214,78,131,91,121,89,84,88,249,133,250,133,252,133,253,133,254,133,0,134,1,134,2,134,3,134,4,134,6,134,7,134,8,134,9,134,10,134,11,134,12,134,13,134,14,134,15,134,16,134,18,134,19,134,20,134,21,134,23,134,24,134,25,134,26,134,27,134,28,134,29,134,30,134,31,134,32,134,33,134,34,134,35,134,36,134,37,134,38,134,40,134,42,134,43,134,44,134,45,134,46,134,47,134,48,134,49,134,50,134,51,134,52,134,53,134,54,134,55,134,57,134,58,134,59,134,61,134,62,134,63,134,64,134,65,134,66,134,67,134,68,134,69,134,70,134,71,134,72,134,73,134,74,134,75,134,76,134,82,134,83,134,85,134,86,134,87,134,88,134,89,134,91,134,92,134,93,134,95,134,96,134,97,134,99,134,100,134,101,134,102,134,103,134,104,134,105,134,106,134,109,115,30,99,75,142,15,142,206,128,212,130,172,98,240,83,240,108,94,145,42,89,1,96,112,108,77,87,74,100,42,141,43,118,233,110,91,87,128,106,240,117,109,111,45,140,8,140,102,87,239,107,146,136,179,120,162,99,249,83,173,112,100,108,88,88,42,100,2,88,224,104,155,129,16,85,214,124,24,80,186,142,204,109,159,141,235,112,143,99,155,109,212,110,230,126,4,132,67,104,3,144,216,109,118,150,168,139,87,89,121,114,228,133,126,129,188,117,138,138,175,104,84,82,34,142,17,149,208,99,152,152,68,142,124,85,83,79,255,102,143,86,213,96,149,109,67,82,73,92,41,89,251,109,107,88,48,117,28,117,108,96,20,130,70,129,17,99,97,103,226,143,58,119,243,141,52,141,193,148,22,94,133,83,44,84,195,112,109,134,111,134,112,134,114,134,115,134,116,134,117,134,118,134,119,134,120,134,131,134,132,134,133,134,134,134,135,134,136,134,137,134,142,134,143,134,144,134,145,134,146,134,148,134,150,134,151,134,152,134,153,134,154,134,155,134,158,134,159,134,160,134,161,134,162,134,165,134,166,134,171,134,173,134,174,134,178,134,179,134,183,134,184,134,185,134,187,134,188,134,189,134,190,134,191,134,193,134,194,134,195,134,197,134,200,134,204,134,205,134,210,134,211,134,213,134,214,134,215,134,218,134,220,134,221,134,224,134,225,134,226,134,227,134,229,134,230,134,231,134,232,134,234,134,235,134,236,134,239,134,245,134,246,134,247,134,250,134,251,134,252,134,253,134,255,134,1,135,4,135,5,135,6,135,11,135,12,135,14,135,15,135,16,135,17,135,20,135,22,135,64,108,247,94,92,80,173,78,173,94,58,99,71,130,26,144,80,104,110,145,179,119,12,84,220,148,100,95,229,122,118,104,69,99,82,123,223,126,219,117,119,80,149,98,52,89,15,144,248,81,195,121,129,122,254,86,146,95,20,144,130,109,96,92,31,87,16,84,84,81,77,110,226,86,168,99,147,152,127,129,21,135,42,137,0,144,30,84,111,92,192,129,214,98,88,98,49,129,53,158,64,150,110,154,124,154,45,105,165,89,211,98,62,85,22,99,199,84,217,134,60,109,3,90,230,116,156,136,106,107,22,89,76,140,47,95,126,110,169,115,125,152,56,78,247,112,140,91,151,120,61,99,90,102,150,118,203,96,155,91,73,90,7,78,85,129,106,108,139,115,161,78,137,103,81,127,128,95,250,101,27,103,216,95,132,89,1,90,25,135,27,135,29,135,31,135,32,135,36,135,38,135,39,135,40,135,42,135,43,135,44,135,45,135,47,135,48,135,50,135,51,135,53,135,54,135,56,135,57,135,58,135,60,135,61,135,64,135,65,135,66,135,67,135,68,135,69,135,70,135,74,135,75,135,77,135,79,135,80,135,81,135,82,135,84,135,85,135,86,135,88,135,90,135,91,135,92,135,93,135,94,135,95,135,97,135,98,135,102,135,103,135,104,135,105,135,106,135,107,135,108,135,109,135,111,135,113,135,114,135,115,135,117,135,119,135,120,135,121,135,122,135,127,135,128,135,129,135,132,135,134,135,135,135,137,135,138,135,140,135,142,135,143,135,144,135,145,135,146,135,148,135,149,135,150,135,152,135,153,135,154,135,155,135,156,135,157,135,158,135,160,135,161,135,162,135,163,135,164,135,205,93,174,95,113,83,230,151,221,143,69,104,244,86,47,85,223,96,58,78,77,111,244,126,199,130,14,132,212,89,31,79,42,79,62,92,172,126,42,103,26,133,115,84,79,117,195,128,130,85,79,155,77,79,45,110,19,140,9,92,112,97,107,83,31,118,41,110,138,134,135,101,251,149,185,126,59,84,51,122,10,125,238,149,225,85,193,127,238,116,29,99,23,135,161,109,157,122,17,98,161,101,103,83,225,99,131,108,235,93,92,84,168,148,76,78])
.concat([97,108,236,139,75,92,224,101,156,130,167,104,62,84,52,84,203,107,102,107,148,78,66,99,72,83,30,130,13,79,174,79,94,87,10,98,254,150,100,102,105,114,255,82,161,82,159,96,239,139,20,102,153,113,144,103,127,137,82,120,253,119,112,102,59,86,56,84,33,149,122,114,165,135,166,135,167,135,169,135,170,135,174,135,176,135,177,135,178,135,180,135,182,135,183,135,184,135,185,135,187,135,188,135,190,135,191,135,193,135,194,135,195,135,196,135,197,135,199,135,200,135,201,135,204,135,205,135,206,135,207,135,208,135,212,135,213,135,214,135,215,135,216,135,217,135,218,135,220,135,221,135,222,135,223,135,225,135,226,135,227,135,228,135,230,135,231,135,232,135,233,135,235,135,236,135,237,135,239,135,240,135,241,135,242,135,243,135,244,135,245,135,246,135,247,135,248,135,250,135,251,135,252,135,253,135,255,135,0,136,1,136,2,136,4,136,5,136,6,136,7,136,8,136,9,136,11,136,12,136,13,136,14,136,15,136,16,136,17,136,18,136,20,136,23,136,24,136,25,136,26,136,28,136,29,136,30,136,31,136,32,136,35,136,0,122,111,96,12,94,137,96,157,129,21,89,220,96,132,113,239,112,170,110,80,108,128,114,132,106,173,136,45,94,96,78,179,90,156,85,227,148,23,109,251,124,153,150,15,98,198,126,142,119,126,134,35,83,30,151,150,143,135,102,225,92,160,79,237,114,11,78,166,83,15,89,19,84,128,99,40,149,72,81,217,78,156,156,164,126,184,84,36,141,84,136,55,130,242,149,142,109,38,95,204,90,62,102,105,150,176,115,46,115,191,83,122,129,133,153,161,127,170,91,119,150,80,150,191,126,248,118,162,83,118,149,153,153,177,123,68,137,88,110,97,78,212,127,101,121,230,139,243,96,205,84,171,78,121,152,247,93,97,106,207,80,17,84,97,140,39,132,93,120,4,151,74,82,238,84,163,86,0,149,136,109,181,91,198,109,83,102,36,136,37,136,38,136,39,136,40,136,41,136,42,136,43,136,44,136,45,136,46,136,47,136,48,136,49,136,51,136,52,136,53,136,54,136,55,136,56,136,58,136,59,136,61,136,62,136,63,136,65,136,66,136,67,136,70,136,71,136,72,136,73,136,74,136,75,136,78,136,79,136,80,136,81,136,82,136,83,136,85,136,86,136,88,136,90,136,91,136,92,136,93,136,94,136,95,136,96,136,102,136,103,136,106,136,109,136,111,136,113,136,115,136,116,136,117,136,118,136,120,136,121,136,122,136,123,136,124,136,128,136,131,136,134,136,135,136,137,136,138,136,140,136,142,136,143,136,144,136,145,136,147,136,148,136,149,136,151,136,152,136,153,136,154,136,155,136,157,136,158,136,159,136,160,136,161,136,163,136,165,136,166,136,167,136,168,136,169,136,170,136,15,92,93,91,33,104,150,128,120,85,17,123,72,101,84,105,155,78,71,107,78,135,139,151,79,83,31,99,58,100,170,144,156,101,193,128,16,140,153,81,176,104,120,83,249,135,200,97,196,108,251,108,34,140,81,92,170,133,175,130,12,149,35,107,155,143,176,101,251,95,195,95,225,79,69,136,31,102,101,129,41,115,250,96,116,81,17,82,139,87,98,95,162,144,76,136,146,145,120,94,79,103,39,96,211,89,68,81,246,81,248,128,8,83,121,108,196,150,138,113,17,79,238,79,158,127,61,103,197,85,8,149,192,121,150,136,227,126,159,88,12,98,0,151,90,134,24,86,123,152,144,95,184,139,196,132,87,145,217,83,237,101,143,94,92,117,100,96,110,125,127,90,234,126,237,126,105,143,167,85,163,91,172,96,203,101,132,115,172,136,174,136,175,136,176,136,178,136,179,136,180,136,181,136,182,136,184,136,185,136,186,136,187,136,189,136,190,136,191,136,192,136,195,136,196,136,199,136,200,136,202,136,203,136,204,136,205,136,207,136,208,136,209,136,211,136,214,136,215,136,218,136,219,136,220,136,221,136,222,136,224,136,225,136,230,136,231,136,233,136,234,136,235,136,236,136,237,136,238,136,239,136,242,136,245,136,246,136,247,136,250,136,251,136,253,136,255,136,0,137,1,137,3,137,4,137,5,137,6,137,7,137,8,137,9,137,11,137,12,137,13,137,14,137,15,137,17,137,20,137,21,137,22,137,23,137,24,137,28,137,29,137,30,137,31,137,32,137,34,137,35,137,36,137,38,137,39,137,40,137,41,137,44,137,45,137,46,137,47,137,49,137,50,137,51,137,53,137,55,137,9,144,99,118,41,119,218,126,116,151,155,133,102,91,116,122,234,150,64,136,203,82,143,113,170,95,236,101,226,139,251,91,111,154,225,93,137,107,91,108,173,139,175,139,10,144,197,143,139,83,188,98,38,158,45,158,64,84,43,78,189,130,89,114,156,134,22,93,89,136,175,109,197,150,209,84,154,78,182,139,9,113,189,84,9,150,223,112,249,109,208,118,37,78,20,120,18,135,169,92,246,94,0,138,156,152,14,150,142,112,191,108,68,89,169,99,60,119,77,136,20,111,115,130,48,88,213,113,140,83,26,120,193,150,1,85,102,95,48,113,180,91,26,140,140,154,131,107,46,89,47,158,231,121,104,103,108,98,111,79,161,117,138,127,11,109,51,150,39,108,240,78,210,117,123,81,55,104,62,111,128,144,112,129,150,89,118,116,56,137,57,137,58,137,59,137,60,137,61,137,62,137,63,137,64,137,66,137,67,137,69,137,70,137,71,137,72,137,73,137,74,137,75,137,76,137,77,137,78,137,79,137,80,137,81,137,82,137,83,137,84,137,85,137,86,137,87,137,88,137,89,137,90,137,91,137,92,137,93,137,96,137,97,137,98,137,99,137,100,137,101,137,103,137,104,137,105,137,106,137,107,137,108,137,109,137,110,137,111,137,112,137,113,137,114,137,115,137,116,137,117,137,118,137,119,137,120,137,121,137,122,137,124,137,125,137,126,137,128,137,130,137,132,137,133,137,135,137,136,137,137,137,138,137,139,137,140,137,141,137,142,137,143,137,144,137,145,137,146,137,147,137,148,137,149,137,150,137,151,137,152,137,153,137,154,137,155,137,156,137,157,137,158,137,159,137,160,137,161,137,71,100,39,92,101,144,145,122,35,140,218,89,172,84,0,130,111,131,129,137,0,128,48,105,78,86,54,128,55,114,206,145,182,81,95,78,117,152,150,99,26,78,246,83,243,102,75,129,28,89,178,109,0,78,249,88,59,83,214,99,241,148,157,79,10,79,99,136,144,152,55,89,87,144,251,121,234,78,240,128,145,117,130,108,156,91,232,89,93,95,5,105,129,134,26,80,242,93,89,78,227,119,229,78,122,130,145,98,19,102,145,144,121,92,191,78,121,95,198,129,56,144,132,128,171,117,166,78,212,136,15,97,197,107,198,95,73,78,202,118,162,110,227,139,174,139,10,140,209,139,2,95,252,127,204,127,206,126,53,131,107,131,224,86,183,107,243,151,52,150,251,89,31,84,246,148,235,109,197,91,110,153,57,92,21,95,144,150,162,137,163,137,164,137,165,137,166,137,167,137,168,137,169,137,170,137,171,137,172,137,173,137,174,137,175,137,176,137,177,137,178,137,179,137,180,137,181,137,182,137,183,137,184,137,185,137,186,137,187,137,188,137,189,137,190,137,191,137,192,137,195,137,205,137,211,137,212,137,213,137,215,137,216,137,217,137,219,137,221,137,223,137,224,137,225,137,226,137,228,137,231,137,232,137,233,137,234,137,236,137,237,137,238,137,240,137,241,137,242,137,244,137,245,137,246,137,247,137,248,137,249,137,250,137,251,137,252,137,253,137,254,137,255,137,1,138,2,138,3,138,4,138,5,138,6,138,8,138,9,138,10,138,11,138,12,138,13,138,14,138,15,138,16,138,17,138,18,138,19,138,20,138,21,138,22,138,23,138,24,138,25,138,26,138,27,138,28,138,29,138,112,83,241,130,49,106,116,90,112,158,148,94,40,127,185,131,36,132,37,132,103,131,71,135,206,143,98,141,200,118,113,95,150,152,108,120,32,102,223,84,229,98,99,79,195,129,200,117,184,94,205,150,10,142,249,134,143,84,243,108,140,109,56,108,127,96,199,82,40,117,125,94,24,79,160,96,231,95,36,92,49,117,174,144,192,148,185,114,185,108,56,110,73,145,9,103,203,83,243,83,81,79,201,145,241,139,200,83,124,94,194,143,228,109,142,78,194,118,134,105,94,134,26,97,6,130,89,79,222,79,62,144,124,156,9,97,29,110,20,110,133,150,136,78,49,90,232,150,14,78,127,92,185,121,135,91,237,139,189,127,137,115,223,87,139,130,193,144,1,84,71,144,187,85,234,92,161,95,8,97,50,107,241,114,178,128,137,138,30,138,31,138,32,138,33,138,34,138,35,138,36,138,37,138,38,138,39,138,40,138,41,138,42,138,43,138,44,138,45,138,46,138,47,138,48,138,49,138,50,138,51,138,52,138,53,138,54,138,55,138,56,138,57,138,58,138,59,138,60,138,61,138,63,138,64,138,65,138,66,138,67,138,68,138,69,138,70,138,71,138,73,138,74,138,75,138,76,138,77,138,78,138,79,138,80,138,81,138,82,138,83,138,84,138,85,138,86,138,87,138,88,138,89,138,90,138,91,138,92,138,93,138,94,138,95,138,96,138,97,138,98,138,99,138,100,138,101,138,102,138,103,138,104,138,105,138,106,138,107,138,108,138,109,138,110,138,111,138,112,138,113,138,114,138,115,138,116,138,117,138,118,138,119,138,120,138,122,138,123,138,124,138,125,138,126,138,127,138,128,138,116,109,211,91,213,136,132,152,107,140,109,154,51,158,10,110,164,81,67,81,163,87,129,136,159,83,244,99,149,143,237,86,88,84,6,87,63,115,144,110,24,127,220,143,209,130,63,97,40,96,98,150,240,102,166,126,138,141,195,141,165,148,179,92,164,124,8,103,166,96,5,150,24,128,145,78,231,144,0,83,104,150,65,81,208,143,116,133,93,145,85,102,245,151,85,91,29,83,56,120,66,103,61,104,201,84,126,112,176,91,125,143,141,81,40,87,177,84,18,101,130,102,94,141,67,141,15,129,108,132,109,144,223,124,255,81,251,133,163,103,233,101,161,111,164,134,129,142,106,86,32,144,130,118,118,112,229,113,35,141,233,98,25,82,253,108,60,141,14,96,158,88,142,97,254,102,96,141,78,98,179,85,35,110,45,103,103,143,129,138,130,138,131,138,132,138,133,138,134,138,135,138,136,138,139,138,140,138,141,138,142,138,143,138,144,138,145,138,146,138,148,138,149,138,150,138,151,138,152,138,153,138,154,138,155,138,156,138,157,138,158,138,159,138,160,138,161,138,162,138,163,138,164,138,165,138,166,138,167,138,168,138,169,138,170,138,171,138,172,138,173,138,174,138,175,138,176,138,177,138,178,138,179,138,180,138,181,138,182,138,183,138,184,138,185,138,186,138,187,138,188,138,189,138,190,138,191,138,192,138,193,138,194,138,195,138,196,138,197,138,198,138,199,138,200,138,201,138,202,138,203,138,204,138,205,138,206,138,207,138,208,138,209,138,210,138,211,138,212,138,213,138,214,138,215,138,216,138,217,138,218,138,219,138,220,138,221,138,222,138,223,138,224,138,225,138,226,138,227,138,225,148,248,149,40,119,5,104,168,105,139,84,77,78,184,112,200,139,88,100,139,101,133,91,132,122,58,80,232,91,187,119,225,107,121,138,152,124,190,108,207,118,169,101,151,143,45,93,85,92,56,134,8,104,96,83,24,98,217,122,91,110,253,126,31,106,224,122,112,95,51,111,32,95,140,99,168,109,86,103,8,78,16,94,38,141,215,78,192,128,52,118,156,150,219,98,45,102,126,98,188,108,117,141,103,113,105,127,70,81,135,128,236,83,110,144,152,98,242,84,240,134,153,143,5,128,23,149,23,133,217,143,89,109,205,115,159,101,31,119,4,117,39,120,251,129,30,141,136,148,166,79,149,103,185,117,202,139,7,151,47,99,71,149,53,150,184,132,35,99,65,119,129,95,240,114,137,78,20,96,116,101,239,98,99,107,63,101,228,138,229,138,230,138,231,138,232,138,233,138,234,138,235,138,236,138,237,138,238,138,239,138,240,138,241,138,242,138,243,138,244,138,245,138,246,138,247,138,248,138,249,138,250,138,251,138,252,138,253,138,254,138,255,138,0,139,1,139,2,139,3,139,4,139,5,139,6,139,8,139,9,139,10,139,11,139,12,139,13,139,14,139,15,139,16,139,17,139,18,139,19,139,20,139,21,139,22,139,23,139,24,139,25,139,26,139,27,139,28,139,29,139,30,139,31,139,32,139,33,139,34,139,35,139,36,139,37,139,39,139,40,139,41,139,42,139,43,139,44,139,45,139,46,139,47,139,48,139,49,139,50,139,51,139,52,139,53,139,54,139,55,139,56,139,57,139,58,139,59,139,60,139,61,139,62,139,63,139,64,139,65,139,66,139,67,139,68,139,69,139,39,94,199,117,209,144,193,139,157,130,157,103,47,101,49,84,24,135,229,119,162,128,2,129,65,108,75,78,199,126,76,128,244,118,13,105,150,107,103,98,60,80,132,79,64,87,7,99,98,107,190,141,234,83,232,101,184,126,215,95,26,99,183,99,243,129,244,129,110,127,28,94,217,92,54,82,122,102,233,121,26,122,40,141,153,112,212,117,222,110,187,108,146,122,45,78,197,118,224,95,159,148,119,136,200,126,205,121,191,128,205,145,242,78,23,79,31,130,104,84,222,93,50,109,204,139,165,124,116,143,152,128,26,94,146,84,177,118,153,91,60,102,164,154,224,115,42,104,219,134,49,103,42,115,248,139,219,139,16,144,249,122,219,112,110,113,196,98,169,119,49,86,59,78,87,132,241,103,169,82,192,134,46,141,248,148,81,123,70,139,71,139,72,139,73,139,74,139,75,139,76,139,77,139,78,139,79,139,80,139,81,139,82,139,83,139,84,139,85,139,86,139,87,139,88,139,89,139,90,139,91,139,92,139,93,139,94,139,95,139,96,139,97,139,98,139,99,139,100,139,101,139,103,139,104,139,105,139,106,139,107,139,109,139,110,139,111,139,112,139,113,139,114,139,115,139,116,139,117,139,118,139,119,139,120,139,121,139,122,139,123,139,124,139,125,139,126,139,127,139,128,139,129,139,130,139,131,139,132,139,133,139,134,139,135,139,136,139,137,139,138,139,139,139,140,139,141,139,142,139,143,139,144,139,145,139,146,139,147,139,148,139,149,139,150,139,151,139,152,139,153,139,154,139,155,139,156,139,157,139,158,139,159,139,172,139,177,139,187,139,199,139,208,139,234,139,9,140,30,140,79,79,232,108,93,121,123,154,147,98,42,114,253,98,19,78,22,120,108,143,176,100,90,141,198,123,105,104,132,94,197,136,134,89,158,100,238,88,182,114,14,105,37,149,253,143,88,141,96,87,0,127,6,140,198,81,73,99,217,98,83,83,76,104,34,116,1,131,76,145,68,85,64,119,124,112,74,109,121,81,168,84,68,141,255,89,203,110,196,109,92,91,43,125,212,78,125,124,211,110,80,91,234,129,13,110,87,91,3,155,213,104,42,142,151,91,252,126,59,96,181,126,185,144,112,141,79,89,205,99,223,121,179,141,82,83,207,101,86,121,197,139,59,150,196,126,187,148,130,126,52,86,137,145,0,103,106,127,10,92,117,144,40,102,230,93,80,79,222,103,90,80,92,79,80,87,167,94,16,232,17,232,18,232,19,232,20,232,56,140,57,140,58,140,59,140,60,140,61,140,62,140,63,140,64,140,66,140,67,140,68,140,69,140,72,140,74,140,75,140,77,140,78,140,79,140,80,140,81,140,82,140,83,140,84,140,86,140,87,140,88,140,89,140,91,140,92,140,93,140,94,140,95,140,96,140,99,140,100,140,101,140,102,140,103,140,104,140,105,140,108,140,109,140,110,140,111,140,112,140,113,140,114,140,116,140,117,140,118,140,119,140,123,140,124,140,125,140,126,140,127,140,128,140,129,140,131,140,132,140,134,140,135,140,136,140,139,140,141,140,142,140,143,140,144,140,145,140,146,140,147,140,149,140,150,140,151,140,153,140,154,140,155,140,156,140,157,140,158,140,159,140,160,140,161,140,162,140,163,140,164,140,165,140,166,140,167,140,168,140,169,140,170,140,171,140,172,140,173,140,141,78,12,78,64,81,16,78,255,94,69,83,21,78,152,78,30,78,50,155,108,91,105,86,40,78,186,121,63,78,21,83,71,78,45,89,59,114,110,83,16,108,223,86,228,128,151,153,211,107,126,119,23,159,54,78,159,78,16,159,92,78,105,78,147,78,136,130,91,91,108,85,15,86,196,78,141,83,157,83,163,83,165,83,174,83,101,151,93,141,26,83,245,83,38,83,46,83,62,83,92,141,102,83,99,83,2,82,8,82,14,82,45,82,51,82,63,82,64,82,76,82,94,82,97,82,92,82,175,132,125,82,130,82,129,82,144,82,147,82,130,81,84,127,187,78,195,78,201,78,194,78,232,78,225,78,235,78,222,78,27,79,243,78,34,79,100,79,245,78,37,79,39,79,9,79,43,79,94,79,103,79,56,101,90,79,93,79,174,140,175,140,176,140,177,140,178,140,179,140,180,140,181,140,182,140,183,140,184,140,185,140,186,140,187,140,188,140,189,140,190,140,191,140,192,140,193,140,194,140,195,140,196,140,197,140,198,140,199,140,200,140,201,140,202,140,203,140,204,140,205,140,206,140,207,140,208,140,209,140,210,140,211,140,212,140,213,140,214,140,215,140,216,140,217,140,218,140,219,140,220,140,221,140,222,140,223,140,224,140,225,140,226,140,227,140,228,140,229,140,230,140,231,140,232,140,233,140,234,140,235,140,236,140,237,140,238,140,239,140,240,140,241,140,242,140,243,140,244,140,245,140,246,140,247,140,248,140,249,140,250,140,251,140,252,140,253,140,254,140,255,140,0,141,1,141,2,141,3,141,4,141,5,141,6,141,7,141,8,141,9,141,10,141,11,141,12,141,13,141,95,79,87,79,50,79,61,79,118,79,116,79,145,79,137,79,131,79,143,79,126,79,123,79,170,79,124,79,172,79,148,79,230,79,232,79,234,79,197,79,218,79,227,79,220,79,209,79,223,79,248,79,41,80,76,80,243,79,44,80,15,80,46,80,45,80,254,79,28,80,12,80,37,80,40,80,126,80,67,80,85,80,72,80,78,80,108,80,123,80,165,80,167,80,169,80,186,80,214,80,6,81,237,80,236,80,230,80,238,80,7,81,11,81,221,78,61,108,88,79,101,79,206,79,160,159,70,108,116,124,110,81,253,93,201,158,152,153,129,81,20,89,249,82,13,83,7,138,16,83,235,81,25,89,85,81,160,78,86,81,179,78,110,136,164,136,181,78,20,129,210,136,128,121,52,91,3,136,184,127,171,81,177,81,189,81,188,81,14,141,15,141,16,141,17,141,18,141,19,141,20,141,21,141,22,141,23,141,24,141,25,141,26,141,27,141,28,141,32,141,81,141,82,141,87,141,95,141,101,141,104,141,105,141,106,141,108,141,110,141,111,141,113,141,114,141,120,141,121,141,122,141,123,141,124,141,125,141,126,141,127,141,128,141,130,141,131,141,134,141,135,141,136,141,137,141,140,141,141,141,142,141,143,141,144,141,146,141,147,141,149,141,150,141,151,141,152,141,153,141,154,141,155,141,156,141,157,141,158,141,160,141,161,141,162,141,164,141,165,141,166,141,167,141,168,141,169,141,170,141,171,141,172,141,173,141,174,141,175,141,176,141,178,141,182,141,183,141,185,141,187,141,189,141,192,141,193,141,194,141,197,141,199,141,200,141,201,141,202,141,205,141,208,141,210,141,211,141,212,141,199,81,150,81,162,81,165,81,160,139,166,139,167,139,170,139,180,139,181,139,183,139,194,139,195,139,203,139,207,139,206,139,210,139,211,139,212,139,214,139,216,139,217,139,220,139,223,139,224,139,228,139,232,139,233,139,238,139,240,139,243,139,246,139,249,139,252,139,255,139,0,140,2,140,4,140,7,140,12,140,15,140,17,140,18,140,20,140,21,140,22,140,25,140,27,140,24,140,29,140,31,140,32,140,33,140,37,140,39,140,42,140,43,140,46,140,47,140,50,140,51,140,53,140,54,140,105,83,122,83,29,150,34,150,33,150,49,150,42,150,61,150,60,150,66,150,73,150,84,150,95,150,103,150,108,150,114,150,116,150,136,150,141,150,151,150,176,150,151,144,155,144,157,144,153,144,172,144,161,144,180,144,179,144,182,144,186,144,213,141,216,141,217,141,220,141,224,141,225,141,226,141,229,141,230,141,231,141,233,141,237,141,238,141,240,141,241,141,242,141,244,141,246,141,252,141,254,141,255,141,0,142,1,142,2,142,3,142,4,142,6,142,7,142,8,142,11,142,13,142,14,142,16,142,17,142,18,142,19,142,21,142,22,142,23,142,24,142,25,142,26,142,27,142,28,142,32,142,33,142,36,142,37,142,38,142,39,142,40,142,43,142,45,142,48,142,50,142,51,142,52,142,54,142,55,142,56,142,59,142,60,142,62,142,63,142,67,142,69,142,70,142,76,142,77,142,78,142,79,142,80,142,83,142,84,142,85,142,86,142,87,142,88,142,90,142,91,142,92,142,93,142,94,142,95,142,96,142,97,142,98,142,99,142,100,142,101,142,103,142,104,142,106,142,107,142,110,142,113,142,184,144,176,144,207,144,197,144,190,144,208,144,196,144,199,144,211,144,230,144,226,144,220,144,215,144,219,144,235,144,239,144,254,144,4,145,34,145,30,145,35,145,49,145,47,145,57,145,67,145,70,145,13,82,66,89,162,82,172,82,173,82,190,82,255,84,208,82,214,82,240,82,223,83,238,113,205,119,244,94,245,81,252,81,47,155,182,83,1,95,90,117,239,93,76,87,169,87,161,87,126,88,188,88,197,88,209,88,41,87,44,87,42,87,51,87,57,87,46,87,47,87,92,87,59,87,66,87,105,87,133,87,107,87,134,87,124,87,123,87,104,87,109,87,118,87,115,87,173,87,164,87,140,87,178,87,207,87,167,87,180,87,147,87,160,87,213,87,216,87,218,87,217,87,210,87,184,87,244,87,239,87,248,87,228,87,221,87,115,142,117,142,119,142,120,142,121,142,122,142,123,142,125,142,126,142,128,142,130,142,131,142,132,142,134,142,136,142,137,142,138,142,139,142,140,142,141,142,142,142,145,142,146,142,147,142,149,142,150,142,151,142,152,142,153,142,154,142,155,142,157,142,159,142,160,142,161,142,162,142,163,142,164,142,165,142,166,142,167,142,168,142,169,142,170,142,173,142,174,142,176,142,177,142,179,142,180,142,181,142,182,142,183,142,184,142,185,142,187,142,188,142,189,142,190,142,191,142,192,142,193,142,194,142,195,142,196,142,197,142,198,142,199,142,200,142,201,142,202,142,203,142,204,142,205,142,207,142,208,142,209,142,210,142,211,142,212,142,213,142,214,142,215,142,216,142,217,142,218,142,219,142,220,142,221,142,222,142,223,142,224,142,225,142,226,142,227,142,228,142,11,88,13,88,253,87,237,87,0,88,30,88,25,88,68,88,32,88,101,88,108,88,129,88,137,88,154,88,128,88,168,153,25,159,255,97,121,130,125,130,127,130,143,130,138,130,168,130,132,130,142,130,145,130,151,130,153,130,171,130,184,130,190,130,176,130,200,130,202,130,227,130,152,130,183,130,174,130,203,130,204,130,193,130,169,130,180,130,161,130,170,130,159,130,196,130,206,130,164,130,225,130,9,131,247,130,228,130,15,131,7,131,220,130,244,130,210,130,216,130,12,131,251,130,211,130,17,131,26,131,6,131,20,131,21,131,224,130,213,130,28,131,81,131,91,131,92,131,8,131,146,131,60,131,52,131,49,131,155,131,94,131,47,131,79,131,71,131,67,131,95,131,64,131,23,131,96,131,45,131,58,131,51,131,102,131,101,131,229,142,230,142,231,142,232,142,233,142,234,142,235,142,236,142,237,142,238,142,239,142,240,142,241,142,242,142,243,142,244,142,245,142,246,142,247,142,248,142,249,142,250,142,251,142,252,142,253,142,254,142,255,142,0,143,1,143,2,143,3,143,4,143,5,143,6,143,7,143,8,143,9,143,10,143,11,143,12,143,13,143,14,143,15,143,16,143,17,143,18,143,19,143,20,143,21,143,22,143,23,143,24,143,25,143,26,143,27,143,28,143,29,143,30,143,31,143,32,143,33,143,34,143,35,143,36,143,37,143,38,143,39,143,40,143,41,143,42,143,43,143,44,143,45,143,46,143,47,143,48,143,49,143,50,143,51,143,52,143,53,143,54,143,55,143,56,143,57,143,58,143,59,143,60,143,61,143,62,143,63,143,64,143,65,143,66,143,67,143,68,143,104,131,27,131,105,131,108,131,106,131,109,131,110,131,176,131,120,131,179,131,180,131,160,131,170,131,147,131,156,131,133,131,124,131,182,131,169,131,125,131,184,131,123,131,152,131,158,131,168,131,186,131,188,131,193,131,1,132,229,131,216,131,7,88,24,132,11,132,221,131,253,131,214,131,28,132,56,132,17,132,6,132,212,131,223,131,15,132,3,132,248,131,249,131,234,131,197,131,192,131,38,132,240,131,225,131,92,132,81,132,90,132,89,132,115,132,135,132,136,132,122,132,137,132,120,132,60,132,70,132,105,132,118,132,140,132,142,132,49,132,109,132,193,132,205,132,208,132,230,132,189,132,211,132,202,132,191,132,186,132,224,132,161,132,185,132,180,132,151,132,229,132,227,132,12,133,13,117,56,133,240,132,57,133,31,133,58,133,69,143,70,143,71,143,72,143,73,143,74,143,75,143,76,143,77,143,78,143,79,143,80,143,81,143,82,143,83,143,84,143,85,143,86,143,87,143,88,143,89,143,90,143,91,143,92,143,93,143,94,143,95,143,96,143,97,143,98,143,99,143,100,143,101,143,106,143,128,143,140,143,146,143,157,143,160,143,161,143,162,143,164,143,165,143,166,143,167,143,170,143,172,143,173,143,174,143,175,143,178,143,179,143,180,143,181,143,183,143,184,143,186,143,187,143,188,143,191,143,192,143,195,143,198,143,201,143,202,143,203,143,204,143,205,143,207,143,210,143,214,143,215,143,218,143,224,143,225,143,227,143,231,143,236,143,239,143,241,143,242,143,244,143,245,143,246,143,250,143,251,143,252,143,254,143,255,143,7,144,8,144,12,144,14,144,19,144,21,144,24,144,86,133,59,133,255,132,252,132,89,133,72,133,104,133,100,133,94,133,122,133,162,119,67,133,114,133,123,133,164,133,168,133,135,133,143,133,121,133,174,133,156,133,133,133,185,133,183,133,176,133,211,133,193,133,220,133,255,133,39,134,5,134,41,134,22,134,60,134,254,94,8,95,60,89,65,89,55,128,85,89,90,89,88,89,15,83,34,92,37,92,44,92,52,92,76,98,106,98,159,98,187,98,202,98,218,98,215,98,238,98,34,99,246,98,57,99,75,99,67,99,173,99,246,99,113,99,122,99,142,99,180,99,109,99,172,99,138,99,105,99,174,99,188,99,242,99,248,99,224,99,255,99,196,99,222,99,206,99,82,100,198,99,190,99,69,100,65,100,11,100,27,100,32,100,12,100,38,100,33,100,94,100,132,100,109,100,150,100,25,144,28,144,35,144,36,144,37,144,39,144,40,144,41,144,42,144,43,144,44,144,48,144,49,144,50,144,51,144,52,144,55,144,57,144,58,144,61,144,63,144,64,144,67,144,69,144,70,144,72,144,73,144,74,144,75,144,76,144,78,144,84,144,85,144,86,144,89,144,90,144,92,144,93,144,94,144,95,144,96,144,97,144,100,144,102,144,103,144,105,144,106,144,107,144,108,144,111,144,112,144,113,144,114,144,115,144,118,144,119,144,120,144,121,144,122,144,123,144,124,144,126,144,129,144,132,144,133,144,134,144,135,144,137,144,138,144,140,144,141,144,142,144,143,144,144,144,146,144,148,144,150,144,152,144,154,144,156,144,158,144,159,144,160,144,164,144,165,144,167,144,168,144,169,144,171,144,173,144,178,144,183,144,188,144,189,144,191,144,192,144,122,100,183,100,184,100,153,100,186,100,192,100,208,100,215,100,228,100,226,100,9,101,37,101,46,101,11,95,210,95,25,117,17,95,95,83,241,83,253,83,233,83,232,83,251,83,18,84,22,84,6,84,75,84,82,84,83,84,84,84,86,84,67,84,33,84,87,84,89,84,35,84,50,84,130,84,148,84,119,84,113,84,100,84,154,84,155,84,132,84,118,84,102,84,157,84,208,84,173,84,194,84,180,84,210,84,167,84,166,84,211,84,212,84,114,84,163,84,213,84,187,84,191,84,204,84,217,84,218,84,220,84,169,84,170,84,164,84,221,84,207,84,222,84,27,85,231,84,32,85,253,84,20,85,243,84,34,85,35,85,15,85,17,85,39,85,42,85,103,85,143,85,181,85,73,85,109,85,65,85,85,85,63,85,80,85,60,85,194,144,195,144,198,144,200,144,201,144,203,144,204,144,205,144,210,144,212,144,213,144,214,144,216,144,217,144,218,144,222,144,223,144,224,144,227,144,228,144,229,144,233,144,234,144,236,144,238,144,240,144,241,144,242,144,243,144,245,144,246,144,247,144,249,144,250,144,251,144,252,144,255,144,0,145,1,145,3,145,5,145,6,145,7,145,8,145,9,145,10,145,11,145,12,145,13,145,14,145,15,145,16,145,17,145,18,145,19,145,20,145,21,145,22,145,23,145,24,145,26,145,27,145,28,145,29,145,31,145,32,145,33,145,36,145,37,145,38,145,39,145,40,145,41,145,42,145,43,145,44,145,45,145,46,145,48,145,50,145,51,145,52,145,53,145,54,145,55,145,56,145,58,145,59,145,60,145,61,145,62,145,63,145,64,145,65,145,66,145,68,145,55,85,86,85,117,85,118,85,119,85,51,85,48,85,92,85,139,85,210,85,131,85,177,85,185,85,136,85,129,85,159,85,126,85,214,85,145,85,123,85,223,85,189,85,190,85,148,85,153,85,234,85,247,85,201,85,31,86,209,85,235,85,236,85,212,85,230,85,221,85,196,85,239,85,229,85,242,85,243,85,204,85,205,85,232,85,245,85,228,85,148,143,30,86,8,86,12,86,1,86,36,86,35,86,254,85,0,86,39,86,45,86,88,86,57,86,87,86,44,86,77,86,98,86,89,86,92,86,76,86,84,86,134,86,100,86,113,86,107,86,123,86,124,86,133,86,147,86,175,86,212,86,215,86,221,86,225,86,245,86,235,86,249,86,255,86,4,87,10,87,9,87,28,87,15,94,25,94,20,94,17,94,49,94,59,94,60,94,69,145,71,145,72,145,81,145,83,145,84,145,85,145,86,145,88,145,89,145,91,145,92,145,95,145,96,145,102,145,103,145,104,145,107,145,109,145,115,145,122,145,123,145,124,145,128,145,129,145,130,145,131,145,132,145,134,145,136,145,138,145,142,145,143,145,147,145,148,145,149,145,150,145,151,145,152,145,153,145,156,145,157,145,158,145,159,145,160,145,161,145,164,145,165,145,166,145,167,145,168,145,169,145,171,145,172,145,176,145,177,145,178,145,179,145,182,145,183,145,184,145,185,145,187,145,188,145,189,145,190,145,191,145,192,145,193,145,194,145,195,145,196,145,197,145,198,145,200,145,203,145,208,145,210,145,211,145,212,145,213,145,214,145,215,145,216,145,217,145,218,145,219,145,221,145,222,145,223,145,224,145,225,145,226,145,227,145,228,145,229,145,55,94,68,94,84,94,91,94,94,94,97,94,140,92,122,92,141,92,144,92,150,92,136,92,152,92,153,92,145,92,154,92,156,92,181,92,162,92,189,92,172,92,171,92,177,92,163,92,193,92,183,92,196,92,210,92,228,92,203,92,229,92,2,93,3,93,39,93,38,93,46,93,36,93,30,93,6,93,27,93,88,93,62,93,52,93,61,93,108,93,91,93,111,93,93,93,107,93,75,93,74,93,105,93,116,93,130,93,153,93,157,93,115,140,183,93,197,93,115,95,119,95,130,95,135,95,137,95,140,95,149,95,153,95,156,95,168,95,173,95,181,95,188,95,98,136,97,95,173,114,176,114,180,114,183,114,184,114,195,114,193,114,206,114,205,114,210,114,232,114,239,114,233,114,242,114,244,114,247,114,1,115,243,114,3,115,250,114,230,145,231,145,232,145,233,145,234,145,235,145,236,145,237,145,238,145,239,145,240,145,241,145,242,145,243,145,244,145,245,145,246,145,247,145,248,145,249,145,250,145,251,145,252,145,253,145,254,145,255,145,0,146,1,146,2,146,3,146,4,146,5,146,6,146,7,146,8,146,9,146,10,146,11,146,12,146,13,146,14,146,15,146,16,146,17,146,18,146,19,146,20,146,21,146,22,146,23,146,24,146,25,146,26,146,27,146,28,146,29,146,30,146,31,146,32,146,33,146,34,146,35,146,36,146,37,146,38,146,39,146,40,146,41,146,42,146,43,146,44,146,45,146,46,146,47,146,48,146,49,146,50,146,51,146,52,146,53,146,54,146,55,146,56,146,57,146,58,146,59,146,60,146,61,146,62,146,63,146,64,146,65,146,66,146,67,146,68,146,69,146,251,114,23,115,19,115,33,115,10,115,30,115,29,115,21,115,34,115,57,115,37,115,44,115,56,115,49,115,80,115,77,115,87,115,96,115,108,115,111,115,126,115,27,130,37,89,231,152,36,89,2,89,99,153,103,153,104,153,105,153,106,153,107,153,108,153,116,153,119,153,125,153,128,153,132,153,135,153,138,153,141,153,144,153,145,153,147,153,148,153,149,153,128,94,145,94,139,94,150,94,165,94,160,94,185,94,181,94,190,94,179,94,83,141,210,94,209,94,219,94,232,94,234,94,186,129,196,95,201,95,214,95,207,95,3,96,238,95,4,96,225,95,228,95,254,95,5,96,6,96,234,95,237,95,248,95,25,96,53,96,38,96,27,96,15,96,13,96,41,96,43,96,10,96,63,96,33,96,120,96,121,96,123,96,122,96,66,96,70,146,71,146,72,146,73,146,74,146,75,146,76,146,77,146,78,146,79,146,80,146,81,146,82,146,83,146,84,146,85,146,86,146,87,146,88,146,89,146,90,146,91,146,92,146,93,146,94,146,95,146,96,146,97,146,98,146,99,146,100,146,101,146,102,146,103,146,104,146,105,146,106,146,107,146,108,146,109,146,110,146,111,146,112,146,113,146,114,146,115,146,117,146,118,146,119,146,120,146,121,146,122,146,123,146,124,146,125,146,126,146,127,146,128,146,129,146,130,146,131,146,132,146,133,146,134,146,135,146,136,146,137,146,138,146,139,146,140,146,141,146,143,146,144,146,145,146,146,146,147,146,148,146,149,146,150,146,151,146,152,146,153,146,154,146,155,146,156,146,157,146,158,146,159,146,160,146,161,146,162,146,163,146,164,146,165,146,166,146,167,146,106,96,125,96,150,96,154,96,173,96,157,96,131,96,146,96,140,96,155,96,236,96,187,96,177,96,221,96,216,96,198,96,218,96,180,96,32,97,38,97,21,97,35,97,244,96,0,97,14,97,43,97,74,97,117,97,172,97,148,97,167,97,183,97,212,97,245,97,221,95,179,150,233,149,235,149,241,149,243,149,245,149,246,149,252,149,254,149,3,150,4,150,6,150,8,150,10,150,11,150,12,150,13,150,15,150,18,150,21,150,22,150,23,150,25,150,26,150,44,78,63,114,21,98,53,108,84,108,92,108,74,108,163,108,133,108,144,108,148,108,140,108,104,108,105,108,116,108,118,108,134,108,169,108,208,108,212,108,173,108,247,108,248,108,241,108,215,108,178,108,224,108,214,108,250,108,235,108,238,108,177,108,211,108,239,108,254,108,168,146,169,146,170,146,171,146,172,146,173,146,175,146,176,146,177,146,178,146,179,146,180,146,181,146,182,146,183,146,184,146,185,146,186,146,187,146,188,146,189,146,190,146,191,146,192,146,193,146,194,146,195,146,196,146,197,146,198,146,199,146,201,146,202,146,203,146,204,146,205,146,206,146,207,146,208,146,209,146,210,146,211,146,212,146,213,146,214,146,215,146,216,146,217,146,218,146,219,146,220,146,221,146,222,146,223,146,224,146,225,146,226,146,227,146,228,146,229,146,230,146,231,146,232,146,233,146,234,146,235,146,236,146,237,146,238,146,239,146,240,146,241,146,242,146,243,146,244,146,245,146,246,146,247,146,248,146,249,146,250,146,251,146,252,146,253,146,254,146,255,146,0,147,1,147,2,147,3,147,4,147,5,147,6,147,7,147,8,147,9,147,57,109,39,109,12,109,67,109,72,109,7,109,4,109,25,109,14,109,43,109,77,109,46,109,53,109,26,109,79,109,82,109,84,109,51,109,145,109,111,109,158,109,160,109,94,109,147,109,148,109,92,109,96,109,124,109,99,109,26,110,199,109,197,109,222,109,14,110,191,109,224,109,17,110,230,109,221,109,217,109,22,110,171,109,12,110,174,109,43,110,110,110,78,110,107,110,178,110,95,110,134,110,83,110,84,110,50,110,37,110,68,110,223,110,177,110,152,110,224,110,45,111,226,110,165,110,167,110,189,110,187,110,183,110,215,110,180,110,207,110,143,110,194,110,159,110,98,111,70,111,71,111,36,111,21,111,249,110,47,111,54,111,75,111,116,111,42,111,9,111,41,111,137,111,141,111,140,111,120,111,114,111,124,111,122,111,209,111,10,147,11,147,12,147,13,147,14,147,15,147,16,147,17,147,18,147,19,147,20,147,21,147,22,147,23,147,24,147,25,147,26,147,27,147,28,147,29,147,30,147,31,147,32,147,33,147,34,147,35,147,36,147,37,147,38,147,39,147,40,147,41,147,42,147,43,147,44,147,45,147,46,147,47,147,48,147,49,147,50,147,51,147,52,147,53,147,54,147,55,147,56,147,57,147,58,147,59,147,60,147,61,147,63,147,64,147,65,147,66,147,67,147,68,147,69,147,70,147,71,147,72,147,73,147,74,147,75,147,76,147,77,147,78,147,79,147,80,147,81,147,82,147,83,147,84,147,85,147,86,147,87,147,88,147,89,147,90,147,91,147,92,147,93,147,94,147,95,147,96,147,97,147,98,147,99,147,100,147,101,147,102,147,103,147,104,147,105,147,107,147,201,111,167,111,185,111,182,111,194,111,225,111,238,111,222,111,224,111,239,111,26,112,35,112,27,112,57,112,53,112,79,112,94,112,128,91,132,91,149,91,147,91,165,91,184,91,47,117,158,154,52,100,228,91,238,91,48,137,240,91,71,142,7,139,182,143,211,143,213,143,229,143,238,143,228,143,233,143,230,143,243,143,232,143,5,144,4,144,11,144,38,144,17,144,13,144,22,144,33,144,53,144,54,144,45,144,47,144,68,144,81,144,82,144,80,144,104,144,88,144,98,144,91,144,185,102,116,144,125,144,130,144,136,144,131,144,139,144,80,95,87,95,86,95,88,95,59,92,171,84,80,92,89,92,113,91,99,92,102,92,188,127,42,95,41,95,45,95,116,130,60,95,59,155,110,92,129,89,131,89,141,89,169,89,170,89,163,89,108,147,109,147,110,147,111,147,112,147,113,147,114,147,115,147,116,147,117,147,118,147,119,147,120,147,121,147,122,147,123,147,124,147,125,147,126,147,127,147,128,147,129,147,130,147,131,147,132,147,133,147,134,147,135,147,136,147,137,147,138,147,139,147,140,147,141,147,142,147,144,147,145,147,146,147,147,147,148,147,149,147,150,147,151,147,152,147,153,147,154,147,155,147,156,147,157,147,158,147,159,147,160,147,161,147,162,147,163,147,164,147,165,147,166,147,167,147,168,147,169,147,170,147,171,147,172,147,173,147,174,147,175,147,176,147,177,147,178,147,179,147,180,147,181,147,182,147,183,147,184,147,185,147,186,147,187,147,188,147,189,147,190,147,191,147,192,147,193,147,194,147,195,147,196,147,197,147,198,147,199,147,200,147,201,147,203,147,204,147,205,147,151,89,202,89,171,89,158,89,164,89,210,89,178,89,175,89,215,89,190,89,5,90,6,90,221,89,8,90,227,89,216,89,249,89,12,90,9,90,50,90,52,90,17,90,35,90,19,90,64,90,103,90,74,90,85,90,60,90,98,90,117,90,236,128,170,90,155,90,119,90,122,90,190,90,235,90,178,90,210,90,212,90,184,90,224,90,227,90,241,90,214,90,230,90,216,90,220,90,9,91,23,91,22,91,50,91,55,91,64,91,21,92,28,92,90,91,101,91,115,91,81,91,83,91,98,91,117,154,119,154,120,154,122,154,127,154,125,154,128,154,129,154,133,154,136,154,138,154,144,154,146,154,147,154,150,154,152,154,155,154,156,154,157,154,159,154,160,154,162,154,163,154,165,154,167,154,159,126,161,126,163,126,165,126,168,126,169,126,206,147,207,147,208,147,209,147,210,147,211,147,212,147,213,147,215,147,216,147,217,147,218,147,219,147,220,147,221,147,222,147,223,147,224,147,225,147,226,147,227,147,228,147,229,147,230,147,231,147,232,147,233,147,234,147,235,147,236,147,237,147,238,147,239,147,240,147,241,147,242,147,243,147,244,147,245,147,246,147,247,147,248,147,249,147,250,147,251,147,252,147,253,147,254,147,255,147,0,148,1,148,2,148,3,148,4,148,5,148,6,148,7,148,8,148,9,148,10,148,11,148,12,148,13,148,14,148,15,148,16,148,17,148,18,148,19,148,20,148,21,148,22,148,23,148,24,148,25,148,26,148,27,148,28,148,29,148,30,148,31,148,32,148,33,148,34,148,35,148,36,148,37,148,38,148,39,148,40,148,41,148,42,148,43,148,44,148,45,148,46,148,173,126,176,126,190,126,192,126,193,126,194,126,201,126,203,126,204,126,208,126,212,126,215,126,219,126,224,126,225,126,232,126,235,126,238,126,239,126,241,126,242,126,13,127,246,126,250,126,251,126,254,126,1,127,2,127,3,127,7,127,8,127,11,127,12,127,15,127,17,127,18,127,23,127,25,127,28,127,27,127,31,127,33,127,34,127,35,127,36,127,37,127,38,127,39,127,42,127,43,127,44,127,45,127,47,127,48,127,49,127,50,127,51,127,53,127,122,94,127,117,219,93,62,117,149,144,142,115,145,115,174,115,162,115,159,115,207,115,194,115,209,115,183,115,179,115,192,115,201,115,200,115,229,115,217,115,124,152,10,116,233,115,231,115,222,115,186,115,242,115,15,116,42,116,91,116,38,116,37,116,40,116,48,116,46,116,44,116,47,148,48,148,49,148,50,148,51,148,52,148,53,148,54,148,55,148,56,148,57,148,58,148,59,148,60,148,61,148,63,148,64,148,65,148,66,148,67,148,68,148,69,148,70,148,71,148,72,148,73,148,74,148,75,148,76,148,77,148,78,148,79,148,80,148,81,148,82,148,83,148,84,148,85,148,86,148,87,148,88,148,89,148,90,148,91,148,92,148,93,148,94,148,95,148,96,148,97,148,98,148,99,148,100,148,101,148,102,148,103,148,104,148,105,148,106,148,108,148,109,148,110,148,111,148,112,148,113,148,114,148,115,148,116,148,117,148,118,148,119,148,120,148,121,148,122,148,123,148,124,148,125,148,126,148,127,148,128,148,129,148,130,148,131,148,132,148,145,148,150,148,152,148,199,148,207,148,211,148,212,148,218,148,230,148,251,148,28,149,32,149,27,116,26,116,65,116,92,116,87,116,85,116,89,116,119,116,109,116,126,116,156,116,142,116,128,116,129,116,135,116,139,116,158,116,168,116,169,116,144,116,167,116,210,116,186,116,234,151,235,151,236,151,76,103,83,103,94,103,72,103,105,103,165,103,135,103,106,103,115,103,152,103,167,103,117,103,168,103,158,103,173,103,139,103,119,103,124,103,240,103,9,104,216,103,10,104,233,103,176,103,12,104,217,103,181,103,218,103,179,103,221,103,0,104,195,103,184,103,226,103,14,104,193,103,253,103,50,104,51,104,96,104,97,104,78,104,98,104,68,104,100,104,131,104,29,104,85,104,102,104,65,104,103,104,64,104,62,104,74,104,73,104,41,104,181,104,143,104,116,104,119,104,147,104,107,104,194,104,110,105,252,104,31,105,32,105,249,104,39,149,51,149,61,149,67,149,72,149,75,149,85,149,90,149,96,149,110,149,116,149,117,149,119,149,120,149,121,149,122,149,123,149,124,149,125,149,126,149,128,149,129,149,130,149,131,149,132,149,133,149,134,149,135,149,136,149,137,149,138,149,139,149,140,149,141,149,142,149,143,149,144,149,145,149,146,149,147,149,148,149,149,149,150,149,151,149,152,149,153,149,154,149,155,149,156,149,157,149,158,149,159,149,160,149,161,149,162,149,163,149,164,149,165,149,166,149,167,149,168,149,169,149,170,149,171,149,172,149,173,149,174,149,175,149,176,149,177,149,178,149,179,149,180,149,181,149,182,149,183,149,184,149,185,149,186,149,187,149,188,149,189,149,190,149,191,149,192,149,193,149,194,149,195,149,196,149,197,149,198,149,199,149,200,149,201,149,202,149,203,149,36,105,240,104,11,105,1,105,87,105,227,104,16,105,113,105,57,105,96,105,66,105,93,105,132,105,107,105,128,105,152,105,120,105,52,105,204,105,135,105,136,105,206,105,137,105,102,105,99,105,121,105,155,105,167,105,187,105,171,105,173,105,212,105,177,105,193,105,202,105,223,105,149,105,224,105,141,105,255,105,47,106,237,105,23,106,24,106,101,106,242,105,68,106,62,106])
.concat([160,106,80,106,91,106,53,106,142,106,121,106,61,106,40,106,88,106,124,106,145,106,144,106,169,106,151,106,171,106,55,115,82,115,129,107,130,107,135,107,132,107,146,107,147,107,141,107,154,107,155,107,161,107,170,107,107,143,109,143,113,143,114,143,115,143,117,143,118,143,120,143,119,143,121,143,122,143,124,143,126,143,129,143,130,143,132,143,135,143,139,143,204,149,205,149,206,149,207,149,208,149,209,149,210,149,211,149,212,149,213,149,214,149,215,149,216,149,217,149,218,149,219,149,220,149,221,149,222,149,223,149,224,149,225,149,226,149,227,149,228,149,229,149,230,149,231,149,236,149,255,149,7,150,19,150,24,150,27,150,30,150,32,150,35,150,36,150,37,150,38,150,39,150,40,150,41,150,43,150,44,150,45,150,47,150,48,150,55,150,56,150,57,150,58,150,62,150,65,150,67,150,74,150,78,150,79,150,81,150,82,150,83,150,86,150,87,150,88,150,89,150,90,150,92,150,93,150,94,150,96,150,99,150,101,150,102,150,107,150,109,150,110,150,111,150,112,150,113,150,115,150,120,150,121,150,122,150,123,150,124,150,125,150,126,150,127,150,128,150,129,150,130,150,131,150,132,150,135,150,137,150,138,150,141,143,142,143,143,143,152,143,154,143,206,142,11,98,23,98,27,98,31,98,34,98,33,98,37,98,36,98,44,98,231,129,239,116,244,116,255,116,15,117,17,117,19,117,52,101,238,101,239,101,240,101,10,102,25,102,114,103,3,102,21,102,0,102,133,112,247,102,29,102,52,102,49,102,54,102,53,102,6,128,95,102,84,102,65,102,79,102,86,102,97,102,87,102,119,102,132,102,140,102,167,102,157,102,190,102,219,102,220,102,230,102,233,102,50,141,51,141,54,141,59,141,61,141,64,141,69,141,70,141,72,141,73,141,71,141,77,141,85,141,89,141,199,137,202,137,203,137,204,137,206,137,207,137,208,137,209,137,110,114,159,114,93,114,102,114,111,114,126,114,127,114,132,114,139,114,141,114,143,114,146,114,8,99,50,99,176,99,140,150,142,150,145,150,146,150,147,150,149,150,150,150,154,150,155,150,157,150,158,150,159,150,160,150,161,150,162,150,163,150,164,150,165,150,166,150,168,150,169,150,170,150,171,150,172,150,173,150,174,150,175,150,177,150,178,150,180,150,181,150,183,150,184,150,186,150,187,150,191,150,194,150,195,150,200,150,202,150,203,150,208,150,209,150,211,150,212,150,214,150,215,150,216,150,217,150,218,150,219,150,220,150,221,150,222,150,223,150,225,150,226,150,227,150,228,150,229,150,230,150,231,150,235,150,236,150,237,150,238,150,240,150,241,150,242,150,244,150,245,150,248,150,250,150,251,150,252,150,253,150,255,150,2,151,3,151,5,151,10,151,11,151,12,151,16,151,17,151,18,151,20,151,21,151,23,151,24,151,25,151,26,151,27,151,29,151,31,151,32,151,63,100,216,100,4,128,234,107,243,107,253,107,245,107,249,107,5,108,7,108,6,108,13,108,21,108,24,108,25,108,26,108,33,108,41,108,36,108,42,108,50,108,53,101,85,101,107,101,77,114,82,114,86,114,48,114,98,134,22,82,159,128,156,128,147,128,188,128,10,103,189,128,177,128,171,128,173,128,180,128,183,128,231,128,232,128,233,128,234,128,219,128,194,128,196,128,217,128,205,128,215,128,16,103,221,128,235,128,241,128,244,128,237,128,13,129,14,129,242,128,252,128,21,103,18,129,90,140,54,129,30,129,44,129,24,129,50,129,72,129,76,129,83,129,116,129,89,129,90,129,113,129,96,129,105,129,124,129,125,129,109,129,103,129,77,88,181,90,136,129,130,129,145,129,213,110,163,129,170,129,204,129,38,103,202,129,187,129,33,151,34,151,35,151,36,151,37,151,38,151,39,151,40,151,41,151,43,151,44,151,46,151,47,151,49,151,51,151,52,151,53,151,54,151,55,151,58,151,59,151,60,151,61,151,63,151,64,151,65,151,66,151,67,151,68,151,69,151,70,151,71,151,72,151,73,151,74,151,75,151,76,151,77,151,78,151,79,151,80,151,81,151,84,151,85,151,87,151,88,151,90,151,92,151,93,151,95,151,99,151,100,151,102,151,103,151,104,151,106,151,107,151,108,151,109,151,110,151,111,151,112,151,113,151,114,151,117,151,119,151,120,151,121,151,122,151,123,151,125,151,126,151,127,151,128,151,129,151,130,151,131,151,132,151,134,151,135,151,136,151,137,151,138,151,140,151,142,151,143,151,144,151,147,151,149,151,150,151,151,151,153,151,154,151,155,151,156,151,157,151,193,129,166,129,36,107,55,107,57,107,67,107,70,107,89,107,209,152,210,152,211,152,213,152,217,152,218,152,179,107,64,95,194,107,243,137,144,101,81,159,147,101,188,101,198,101,196,101,195,101,204,101,206,101,210,101,214,101,128,112,156,112,150,112,157,112,187,112,192,112,183,112,171,112,177,112,232,112,202,112,16,113,19,113,22,113,47,113,49,113,115,113,92,113,104,113,69,113,114,113,74,113,120,113,122,113,152,113,179,113,181,113,168,113,160,113,224,113,212,113,231,113,249,113,29,114,40,114,108,112,24,113,102,113,185,113,62,98,61,98,67,98,72,98,73,98,59,121,64,121,70,121,73,121,91,121,92,121,83,121,90,121,98,121,87,121,96,121,111,121,103,121,122,121,133,121,138,121,154,121,167,121,179,121,209,95,208,95,158,151,159,151,161,151,162,151,164,151,165,151,166,151,167,151,168,151,169,151,170,151,172,151,174,151,176,151,177,151,179,151,181,151,182,151,183,151,184,151,185,151,186,151,187,151,188,151,189,151,190,151,191,151,192,151,193,151,194,151,195,151,196,151,197,151,198,151,199,151,200,151,201,151,202,151,203,151,204,151,205,151,206,151,207,151,208,151,209,151,210,151,211,151,212,151,213,151,214,151,215,151,216,151,217,151,218,151,219,151,220,151,221,151,222,151,223,151,224,151,225,151,226,151,227,151,228,151,229,151,232,151,238,151,239,151,240,151,241,151,242,151,244,151,247,151,248,151,249,151,250,151,251,151,252,151,253,151,254,151,255,151,0,152,1,152,2,152,3,152,4,152,5,152,6,152,7,152,8,152,9,152,10,152,11,152,12,152,13,152,14,152,60,96,93,96,90,96,103,96,65,96,89,96,99,96,171,96,6,97,13,97,93,97,169,97,157,97,203,97,209,97,6,98,128,128,127,128,147,108,246,108,252,109,246,119,248,119,0,120,9,120,23,120,24,120,17,120,171,101,45,120,28,120,29,120,57,120,58,120,59,120,31,120,60,120,37,120,44,120,35,120,41,120,78,120,109,120,86,120,87,120,38,120,80,120,71,120,76,120,106,120,155,120,147,120,154,120,135,120,156,120,161,120,163,120,178,120,185,120,165,120,212,120,217,120,201,120,236,120,242,120,5,121,244,120,19,121,36,121,30,121,52,121,155,159,249,158,251,158,252,158,241,118,4,119,13,119,249,118,7,119,8,119,26,119,34,119,25,119,45,119,38,119,53,119,56,119,80,119,81,119,71,119,67,119,90,119,104,119,15,152,16,152,17,152,18,152,19,152,20,152,21,152,22,152,23,152,24,152,25,152,26,152,27,152,28,152,29,152,30,152,31,152,32,152,33,152,34,152,35,152,36,152,37,152,38,152,39,152,40,152,41,152,42,152,43,152,44,152,45,152,46,152,47,152,48,152,49,152,50,152,51,152,52,152,53,152,54,152,55,152,56,152,57,152,58,152,59,152,60,152,61,152,62,152,63,152,64,152,65,152,66,152,67,152,68,152,69,152,70,152,71,152,72,152,73,152,74,152,75,152,76,152,77,152,78,152,79,152,80,152,81,152,82,152,83,152,84,152,85,152,86,152,87,152,88,152,89,152,90,152,91,152,92,152,93,152,94,152,95,152,96,152,97,152,98,152,99,152,100,152,101,152,102,152,103,152,104,152,105,152,106,152,107,152,108,152,109,152,110,152,98,119,101,119,127,119,141,119,125,119,128,119,140,119,145,119,159,119,160,119,176,119,181,119,189,119,58,117,64,117,78,117,75,117,72,117,91,117,114,117,121,117,131,117,88,127,97,127,95,127,72,138,104,127,116,127,113,127,121,127,129,127,126,127,205,118,229,118,50,136,133,148,134,148,135,148,139,148,138,148,140,148,141,148,143,148,144,148,148,148,151,148,149,148,154,148,155,148,156,148,163,148,164,148,171,148,170,148,173,148,172,148,175,148,176,148,178,148,180,148,182,148,183,148,184,148,185,148,186,148,188,148,189,148,191,148,196,148,200,148,201,148,202,148,203,148,204,148,205,148,206,148,208,148,209,148,210,148,213,148,214,148,215,148,217,148,216,148,219,148,222,148,223,148,224,148,226,148,228,148,229,148,231,148,232,148,234,148,111,152,112,152,113,152,114,152,115,152,116,152,139,152,142,152,146,152,149,152,153,152,163,152,168,152,169,152,170,152,171,152,172,152,173,152,174,152,175,152,176,152,177,152,178,152,179,152,180,152,181,152,182,152,183,152,184,152,185,152,186,152,187,152,188,152,189,152,190,152,191,152,192,152,193,152,194,152,195,152,196,152,197,152,198,152,199,152,200,152,201,152,202,152,203,152,204,152,205,152,207,152,208,152,212,152,214,152,215,152,219,152,220,152,221,152,224,152,225,152,226,152,227,152,228,152,229,152,230,152,233,152,234,152,235,152,236,152,237,152,238,152,239,152,240,152,241,152,242,152,243,152,244,152,245,152,246,152,247,152,248,152,249,152,250,152,251,152,252,152,253,152,254,152,255,152,0,153,1,153,2,153,3,153,4,153,5,153,6,153,7,153,233,148,235,148,238,148,239,148,243,148,244,148,245,148,247,148,249,148,252,148,253,148,255,148,3,149,2,149,6,149,7,149,9,149,10,149,13,149,14,149,15,149,18,149,19,149,20,149,21,149,22,149,24,149,27,149,29,149,30,149,31,149,34,149,42,149,43,149,41,149,44,149,49,149,50,149,52,149,54,149,55,149,56,149,60,149,62,149,63,149,66,149,53,149,68,149,69,149,70,149,73,149,76,149,78,149,79,149,82,149,83,149,84,149,86,149,87,149,88,149,89,149,91,149,94,149,95,149,93,149,97,149,98,149,100,149,101,149,102,149,103,149,104,149,105,149,106,149,107,149,108,149,111,149,113,149,114,149,115,149,58,149,231,119,236,119,201,150,213,121,237,121,227,121,235,121,6,122,71,93,3,122,2,122,30,122,20,122,8,153,9,153,10,153,11,153,12,153,14,153,15,153,17,153,18,153,19,153,20,153,21,153,22,153,23,153,24,153,25,153,26,153,27,153,28,153,29,153,30,153,31,153,32,153,33,153,34,153,35,153,36,153,37,153,38,153,39,153,40,153,41,153,42,153,43,153,44,153,45,153,47,153,48,153,49,153,50,153,51,153,52,153,53,153,54,153,55,153,56,153,57,153,58,153,59,153,60,153,61,153,62,153,63,153,64,153,65,153,66,153,67,153,68,153,69,153,70,153,71,153,72,153,73,153,74,153,75,153,76,153,77,153,78,153,79,153,80,153,81,153,82,153,83,153,86,153,87,153,88,153,89,153,90,153,91,153,92,153,93,153,94,153,95,153,96,153,97,153,98,153,100,153,102,153,115,153,120,153,121,153,123,153,126,153,130,153,131,153,137,153,57,122,55,122,81,122,207,158,165,153,112,122,136,118,142,118,147,118,153,118,164,118,222,116,224,116,44,117,32,158,34,158,40,158,41,158,42,158,43,158,44,158,50,158,49,158,54,158,56,158,55,158,57,158,58,158,62,158,65,158,66,158,68,158,70,158,71,158,72,158,73,158,75,158,76,158,78,158,81,158,85,158,87,158,90,158,91,158,92,158,94,158,99,158,102,158,103,158,104,158,105,158,106,158,107,158,108,158,113,158,109,158,115,158,146,117,148,117,150,117,160,117,157,117,172,117,163,117,179,117,180,117,184,117,196,117,177,117,176,117,195,117,194,117,214,117,205,117,227,117,232,117,230,117,228,117,235,117,231,117,3,118,241,117,252,117,255,117,16,118,0,118,5,118,12,118,23,118,10,118,37,118,24,118,21,118,25,118,140,153,142,153,154,153,155,153,156,153,157,153,158,153,159,153,160,153,161,153,162,153,163,153,164,153,166,153,167,153,169,153,170,153,171,153,172,153,173,153,174,153,175,153,176,153,177,153,178,153,179,153,180,153,181,153,182,153,183,153,184,153,185,153,186,153,187,153,188,153,189,153,190,153,191,153,192,153,193,153,194,153,195,153,196,153,197,153,198,153,199,153,200,153,201,153,202,153,203,153,204,153,205,153,206,153,207,153,208,153,209,153,210,153,211,153,212,153,213,153,214,153,215,153,216,153,217,153,218,153,219,153,220,153,221,153,222,153,223,153,224,153,225,153,226,153,227,153,228,153,229,153,230,153,231,153,232,153,233,153,234,153,235,153,236,153,237,153,238,153,239,153,240,153,241,153,242,153,243,153,244,153,245,153,246,153,247,153,248,153,249,153,27,118,60,118,34,118,32,118,64,118,45,118,48,118,63,118,53,118,67,118,62,118,51,118,77,118,94,118,84,118,92,118,86,118,107,118,111,118,202,127,230,122,120,122,121,122,128,122,134,122,136,122,149,122,166,122,160,122,172,122,168,122,173,122,179,122,100,136,105,136,114,136,125,136,127,136,130,136,162,136,198,136,183,136,188,136,201,136,226,136,206,136,227,136,229,136,241,136,26,137,252,136,232,136,254,136,240,136,33,137,25,137,19,137,27,137,10,137,52,137,43,137,54,137,65,137,102,137,123,137,139,117,229,128,178,118,180,118,220,119,18,128,20,128,22,128,28,128,32,128,34,128,37,128,38,128,39,128,41,128,40,128,49,128,11,128,53,128,67,128,70,128,77,128,82,128,105,128,113,128,131,137,120,152,128,152,131,152,250,153,251,153,252,153,253,153,254,153,255,153,0,154,1,154,2,154,3,154,4,154,5,154,6,154,7,154,8,154,9,154,10,154,11,154,12,154,13,154,14,154,15,154,16,154,17,154,18,154,19,154,20,154,21,154,22,154,23,154,24,154,25,154,26,154,27,154,28,154,29,154,30,154,31,154,32,154,33,154,34,154,35,154,36,154,37,154,38,154,39,154,40,154,41,154,42,154,43,154,44,154,45,154,46,154,47,154,48,154,49,154,50,154,51,154,52,154,53,154,54,154,55,154,56,154,57,154,58,154,59,154,60,154,61,154,62,154,63,154,64,154,65,154,66,154,67,154,68,154,69,154,70,154,71,154,72,154,73,154,74,154,75,154,76,154,77,154,78,154,79,154,80,154,81,154,82,154,83,154,84,154,85,154,86,154,87,154,88,154,89,154,137,152,140,152,141,152,143,152,148,152,154,152,155,152,158,152,159,152,161,152,162,152,165,152,166,152,77,134,84,134,108,134,110,134,127,134,122,134,124,134,123,134,168,134,141,134,139,134,172,134,157,134,167,134,163,134,170,134,147,134,169,134,182,134,196,134,181,134,206,134,176,134,186,134,177,134,175,134,201,134,207,134,180,134,233,134,241,134,242,134,237,134,243,134,208,134,19,135,222,134,244,134,223,134,216,134,209,134,3,135,7,135,248,134,8,135,10,135,13,135,9,135,35,135,59,135,30,135,37,135,46,135,26,135,62,135,72,135,52,135,49,135,41,135,55,135,63,135,130,135,34,135,125,135,126,135,123,135,96,135,112,135,76,135,110,135,139,135,83,135,99,135,124,135,100,135,89,135,101,135,147,135,175,135,168,135,210,135,90,154,91,154,92,154,93,154,94,154,95,154,96,154,97,154,98,154,99,154,100,154,101,154,102,154,103,154,104,154,105,154,106,154,107,154,114,154,131,154,137,154,141,154,142,154,148,154,149,154,153,154,166,154,169,154,170,154,171,154,172,154,173,154,174,154,175,154,178,154,179,154,180,154,181,154,185,154,187,154,189,154,190,154,191,154,195,154,196,154,198,154,199,154,200,154,201,154,202,154,205,154,206,154,207,154,208,154,210,154,212,154,213,154,214,154,215,154,217,154,218,154,219,154,220,154,221,154,222,154,224,154,226,154,227,154,228,154,229,154,231,154,232,154,233,154,234,154,236,154,238,154,240,154,241,154,242,154,243,154,244,154,245,154,246,154,247,154,248,154,250,154,252,154,253,154,254,154,255,154,0,155,1,155,2,155,4,155,5,155,6,155,198,135,136,135,133,135,173,135,151,135,131,135,171,135,229,135,172,135,181,135,179,135,203,135,211,135,189,135,209,135,192,135,202,135,219,135,234,135,224,135,238,135,22,136,19,136,254,135,10,136,27,136,33,136,57,136,60,136,54,127,66,127,68,127,69,127,16,130,250,122,253,122,8,123,3,123,4,123,21,123,10,123,43,123,15,123,71,123,56,123,42,123,25,123,46,123,49,123,32,123,37,123,36,123,51,123,62,123,30,123,88,123,90,123,69,123,117,123,76,123,93,123,96,123,110,123,123,123,98,123,114,123,113,123,144,123,166,123,167,123,184,123,172,123,157,123,168,123,133,123,170,123,156,123,162,123,171,123,180,123,209,123,193,123,204,123,221,123,218,123,229,123,230,123,234,123,12,124,254,123,252,123,15,124,22,124,11,124,7,155,9,155,10,155,11,155,12,155,13,155,14,155,16,155,17,155,18,155,20,155,21,155,22,155,23,155,24,155,25,155,26,155,27,155,28,155,29,155,30,155,32,155,33,155,34,155,36,155,37,155,38,155,39,155,40,155,41,155,42,155,43,155,44,155,45,155,46,155,48,155,49,155,51,155,52,155,53,155,54,155,55,155,56,155,57,155,58,155,61,155,62,155,63,155,64,155,70,155,74,155,75,155,76,155,78,155,80,155,82,155,83,155,85,155,86,155,87,155,88,155,89,155,90,155,91,155,92,155,93,155,94,155,95,155,96,155,97,155,98,155,99,155,100,155,101,155,102,155,103,155,104,155,105,155,106,155,107,155,108,155,109,155,110,155,111,155,112,155,113,155,114,155,115,155,116,155,117,155,118,155,119,155,120,155,121,155,122,155,123,155,31,124,42,124,38,124,56,124,65,124,64,124,254,129,1,130,2,130,4,130,236,129,68,136,33,130,34,130,35,130,45,130,47,130,40,130,43,130,56,130,59,130,51,130,52,130,62,130,68,130,73,130,75,130,79,130,90,130,95,130,104,130,126,136,133,136,136,136,216,136,223,136,94,137,157,127,159,127,167,127,175,127,176,127,178,127,124,124,73,101,145,124,157,124,156,124,158,124,162,124,178,124,188,124,189,124,193,124,199,124,204,124,205,124,200,124,197,124,215,124,232,124,110,130,168,102,191,127,206,127,213,127,229,127,225,127,230,127,233,127,238,127,243,127,248,124,119,125,166,125,174,125,71,126,155,126,184,158,180,158,115,141,132,141,148,141,145,141,177,141,103,141,109,141,71,140,73,140,74,145,80,145,78,145,79,145,100,145,124,155,125,155,126,155,127,155,128,155,129,155,130,155,131,155,132,155,133,155,134,155,135,155,136,155,137,155,138,155,139,155,140,155,141,155,142,155,143,155,144,155,145,155,146,155,147,155,148,155,149,155,150,155,151,155,152,155,153,155,154,155,155,155,156,155,157,155,158,155,159,155,160,155,161,155,162,155,163,155,164,155,165,155,166,155,167,155,168,155,169,155,170,155,171,155,172,155,173,155,174,155,175,155,176,155,177,155,178,155,179,155,180,155,181,155,182,155,183,155,184,155,185,155,186,155,187,155,188,155,189,155,190,155,191,155,192,155,193,155,194,155,195,155,196,155,197,155,198,155,199,155,200,155,201,155,202,155,203,155,204,155,205,155,206,155,207,155,208,155,209,155,210,155,211,155,212,155,213,155,214,155,215,155,216,155,217,155,218,155,219,155,98,145,97,145,112,145,105,145,111,145,125,145,126,145,114,145,116,145,121,145,140,145,133,145,144,145,141,145,145,145,162,145,163,145,170,145,173,145,174,145,175,145,181,145,180,145,186,145,85,140,126,158,184,141,235,141,5,142,89,142,105,142,181,141,191,141,188,141,186,141,196,141,214,141,215,141,218,141,222,141,206,141,207,141,219,141,198,141,236,141,247,141,248,141,227,141,249,141,251,141,228,141,9,142,253,141,20,142,29,142,31,142,44,142,46,142,35,142,47,142,58,142,64,142,57,142,53,142,61,142,49,142,73,142,65,142,66,142,81,142,82,142,74,142,112,142,118,142,124,142,111,142,116,142,133,142,143,142,148,142,144,142,156,142,158,142,120,140,130,140,138,140,133,140,152,140,148,140,155,101,214,137,222,137,218,137,220,137,220,155,221,155,222,155,223,155,224,155,225,155,226,155,227,155,228,155,229,155,230,155,231,155,232,155,233,155,234,155,235,155,236,155,237,155,238,155,239,155,240,155,241,155,242,155,243,155,244,155,245,155,246,155,247,155,248,155,249,155,250,155,251,155,252,155,253,155,254,155,255,155,0,156,1,156,2,156,3,156,4,156,5,156,6,156,7,156,8,156,9,156,10,156,11,156,12,156,13,156,14,156,15,156,16,156,17,156,18,156,19,156,20,156,21,156,22,156,23,156,24,156,25,156,26,156,27,156,28,156,29,156,30,156,31,156,32,156,33,156,34,156,35,156,36,156,37,156,38,156,39,156,40,156,41,156,42,156,43,156,44,156,45,156,46,156,47,156,48,156,49,156,50,156,51,156,52,156,53,156,54,156,55,156,56,156,57,156,58,156,59,156,229,137,235,137,239,137,62,138,38,139,83,151,233,150,243,150,239,150,6,151,1,151,8,151,15,151,14,151,42,151,45,151,48,151,62,151,128,159,131,159,133,159,134,159,135,159,136,159,137,159,138,159,140,159,254,158,11,159,13,159,185,150,188,150,189,150,206,150,210,150,191,119,224,150,142,146,174,146,200,146,62,147,106,147,202,147,143,147,62,148,107,148,127,156,130,156,133,156,134,156,135,156,136,156,35,122,139,156,142,156,144,156,145,156,146,156,148,156,149,156,154,156,155,156,158,156,159,156,160,156,161,156,162,156,163,156,165,156,166,156,167,156,168,156,169,156,171,156,173,156,174,156,176,156,177,156,178,156,179,156,180,156,181,156,182,156,183,156,186,156,187,156,188,156,189,156,196,156,197,156,198,156,199,156,202,156,203,156,60,156,61,156,62,156,63,156,64,156,65,156,66,156,67,156,68,156,69,156,70,156,71,156,72,156,73,156,74,156,75,156,76,156,77,156,78,156,79,156,80,156,81,156,82,156,83,156,84,156,85,156,86,156,87,156,88,156,89,156,90,156,91,156,92,156,93,156,94,156,95,156,96,156,97,156,98,156,99,156,100,156,101,156,102,156,103,156,104,156,105,156,106,156,107,156,108,156,109,156,110,156,111,156,112,156,113,156,114,156,115,156,116,156,117,156,118,156,119,156,120,156,121,156,122,156,123,156,125,156,126,156,128,156,131,156,132,156,137,156,138,156,140,156,143,156,147,156,150,156,151,156,152,156,153,156,157,156,170,156,172,156,175,156,185,156,190,156,191,156,192,156,193,156,194,156,200,156,201,156,209,156,210,156,218,156,219,156,224,156,225,156,204,156,205,156,206,156,207,156,208,156,211,156,212,156,213,156,215,156,216,156,217,156,220,156,221,156,223,156,226,156,124,151,133,151,145,151,146,151,148,151,175,151,171,151,163,151,178,151,180,151,177,154,176,154,183,154,88,158,182,154,186,154,188,154,193,154,192,154,197,154,194,154,203,154,204,154,209,154,69,155,67,155,71,155,73,155,72,155,77,155,81,155,232,152,13,153,46,153,85,153,84,153,223,154,225,154,230,154,239,154,235,154,251,154,237,154,249,154,8,155,15,155,19,155,31,155,35,155,189,158,190,158,59,126,130,158,135,158,136,158,139,158,146,158,214,147,157,158,159,158,219,158,220,158,221,158,224,158,223,158,226,158,233,158,231,158,229,158,234,158,239,158,34,159,44,159,47,159,57,159,55,159,61,159,62,159,68,159,227,156,228,156,229,156,230,156,231,156,232,156,233,156,234,156,235,156,236,156,237,156,238,156,239,156,240,156,241,156,242,156,243,156,244,156,245,156,246,156,247,156,248,156,249,156,250,156,251,156,252,156,253,156,254,156,255,156,0,157,1,157,2,157,3,157,4,157,5,157,6,157,7,157,8,157,9,157,10,157,11,157,12,157,13,157,14,157,15,157,16,157,17,157,18,157,19,157,20,157,21,157,22,157,23,157,24,157,25,157,26,157,27,157,28,157,29,157,30,157,31,157,32,157,33,157,34,157,35,157,36,157,37,157,38,157,39,157,40,157,41,157,42,157,43,157,44,157,45,157,46,157,47,157,48,157,49,157,50,157,51,157,52,157,53,157,54,157,55,157,56,157,57,157,58,157,59,157,60,157,61,157,62,157,63,157,64,157,65,157,66,157,52,226,53,226,54,226,55,226,56,226,57,226,58,226,59,226,60,226,61,226,62,226,63,226,64,226,65,226,66,226,67,226,68,226,69,226,70,226,71,226,72,226,73,226,74,226,75,226,76,226,77,226,78,226,79,226,80,226,81,226,82,226,83,226,84,226,85,226,86,226,87,226,88,226,89,226,90,226,91,226,92,226,93,226,94,226,95,226,96,226,97,226,98,226,99,226,100,226,101,226,102,226,103,226,104,226,105,226,106,226,107,226,108,226,109,226,110,226,111,226,112,226,113,226,114,226,115,226,116,226,117,226,118,226,119,226,120,226,121,226,122,226,123,226,124,226,125,226,126,226,127,226,128,226,129,226,130,226,131,226,132,226,133,226,134,226,135,226,136,226,137,226,138,226,139,226,140,226,141,226,142,226,143,226,144,226,145,226,67,157,68,157,69,157,70,157,71,157,72,157,73,157,74,157,75,157,76,157,77,157,78,157,79,157,80,157,81,157,82,157,83,157,84,157,85,157,86,157,87,157,88,157,89,157,90,157,91,157,92,157,93,157,94,157,95,157,96,157,97,157,98,157,99,157,100,157,101,157,102,157,103,157,104,157,105,157,106,157,107,157,108,157,109,157,110,157,111,157,112,157,113,157,114,157,115,157,116,157,117,157,118,157,119,157,120,157,121,157,122,157,123,157,124,157,125,157,126,157,127,157,128,157,129,157,130,157,131,157,132,157,133,157,134,157,135,157,136,157,137,157,138,157,139,157,140,157,141,157,142,157,143,157,144,157,145,157,146,157,147,157,148,157,149,157,150,157,151,157,152,157,153,157,154,157,155,157,156,157,157,157,158,157,159,157,160,157,161,157,162,157,146,226,147,226,148,226,149,226,150,226,151,226,152,226,153,226,154,226,155,226,156,226,157,226,158,226,159,226,160,226,161,226,162,226,163,226,164,226,165,226,166,226,167,226,168,226,169,226,170,226,171,226,172,226,173,226,174,226,175,226,176,226,177,226,178,226,179,226,180,226,181,226,182,226,183,226,184,226,185,226,186,226,187,226,188,226,189,226,190,226,191,226,192,226,193,226,194,226,195,226,196,226,197,226,198,226,199,226,200,226,201,226,202,226,203,226,204,226,205,226,206,226,207,226,208,226,209,226,210,226,211,226,212,226,213,226,214,226,215,226,216,226,217,226,218,226,219,226,220,226,221,226,222,226,223,226,224,226,225,226,226,226,227,226,228,226,229,226,230,226,231,226,232,226,233,226,234,226,235,226,236,226,237,226,238,226,239,226,163,157,164,157,165,157,166,157,167,157,168,157,169,157,170,157,171,157,172,157,173,157,174,157,175,157,176,157,177,157,178,157,179,157,180,157,181,157,182,157,183,157,184,157,185,157,186,157,187,157,188,157,189,157,190,157,191,157,192,157,193,157,194,157,195,157,196,157,197,157,198,157,199,157,200,157,201,157,202,157,203,157,204,157,205,157,206,157,207,157,208,157,209,157,210,157,211,157,212,157,213,157,214,157,215,157,216,157,217,157,218,157,219,157,220,157,221,157,222,157,223,157,224,157,225,157,226,157,227,157,228,157,229,157,230,157,231,157,232,157,233,157,234,157,235,157,236,157,237,157,238,157,239,157,240,157,241,157,242,157,243,157,244,157,245,157,246,157,247,157,248,157,249,157,250,157,251,157,252,157,253,157,254,157,255,157,0,158,1,158,2,158,240,226,241,226,242,226,243,226,244,226,245,226,246,226,247,226,248,226,249,226,250,226,251,226,252,226,253,226,254,226,255,226,0,227,1,227,2,227,3,227,4,227,5,227,6,227,7,227,8,227,9,227,10,227,11,227,12,227,13,227,14,227,15,227,16,227,17,227,18,227,19,227,20,227,21,227,22,227,23,227,24,227,25,227,26,227,27,227,28,227,29,227,30,227,31,227,32,227,33,227,34,227,35,227,36,227,37,227,38,227,39,227,40,227,41,227,42,227,43,227,44,227,45,227,46,227,47,227,48,227,49,227,50,227,51,227,52,227,53,227,54,227,55,227,56,227,57,227,58,227,59,227,60,227,61,227,62,227,63,227,64,227,65,227,66,227,67,227,68,227,69,227,70,227,71,227,72,227,73,227,74,227,75,227,76,227,77,227,3,158,4,158,5,158,6,158,7,158,8,158,9,158,10,158,11,158,12,158,13,158,14,158,15,158,16,158,17,158,18,158,19,158,20,158,21,158,22,158,23,158,24,158,25,158,26,158,27,158,28,158,29,158,30,158,36,158,39,158,46,158,48,158,52,158,59,158,60,158,64,158,77,158,80,158,82,158,83,158,84,158,86,158,89,158,93,158,95,158,96,158,97,158,98,158,101,158,110,158,111,158,114,158,116,158,117,158,118,158,119,158,120,158,121,158,122,158,123,158,124,158,125,158,128,158,129,158,131,158,132,158,133,158,134,158,137,158,138,158,140,158,141,158,142,158,143,158,144,158,145,158,148,158,149,158,150,158,151,158,152,158,153,158,154,158,155,158,156,158,158,158,160,158,161,158,162,158,163,158,164,158,165,158,167,158,168,158,169,158,170,158,78,227,79,227,80,227,81,227,82,227,83,227,84,227,85,227,86,227,87,227,88,227,89,227,90,227,91,227,92,227,93,227,94,227,95,227,96,227,97,227,98,227,99,227,100,227,101,227,102,227,103,227,104,227,105,227,106,227,107,227,108,227,109,227,110,227,111,227,112,227,113,227,114,227,115,227,116,227,117,227,118,227,119,227,120,227,121,227,122,227,123,227,124,227,125,227,126,227,127,227,128,227,129,227,130,227,131,227,132,227,133,227,134,227,135,227,136,227,137,227,138,227,139,227,140,227,141,227,142,227,143,227,144,227,145,227,146,227,147,227,148,227,149,227,150,227,151,227,152,227,153,227,154,227,155,227,156,227,157,227,158,227,159,227,160,227,161,227,162,227,163,227,164,227,165,227,166,227,167,227,168,227,169,227,170,227,171,227,171,158,172,158,173,158,174,158,175,158,176,158,177,158,178,158,179,158,181,158,182,158,183,158,185,158,186,158,188,158,191,158,192,158,193,158,194,158,195,158,197,158,198,158,199,158,200,158,202,158,203,158,204,158,208,158,210,158,211,158,213,158,214,158,215,158,217,158,218,158,222,158,225,158,227,158,228,158,230,158,232,158,235,158,236,158,237,158,238,158,240,158,241,158,242,158,243,158,244,158,245,158,246,158,247,158,248,158,250,158,253,158,255,158,0,159,1,159,2,159,3,159,4,159,5,159,6,159,7,159,8,159,9,159,10,159,12,159,15,159,17,159,18,159,20,159,21,159,22,159,24,159,26,159,27,159,28,159,29,159,30,159,31,159,33,159,35,159,36,159,37,159,38,159,39,159,40,159,41,159,42,159,43,159,45,159,46,159,48,159,49,159,172,227,173,227,174,227,175,227,176,227,177,227,178,227,179,227,180,227,181,227,182,227,183,227,184,227,185,227,186,227,187,227,188,227,189,227,190,227,191,227,192,227,193,227,194,227,195,227,196,227,197,227,198,227,199,227,200,227,201,227,202,227,203,227,204,227,205,227,206,227,207,227,208,227,209,227,210,227,211,227,212,227,213,227,214,227,215,227,216,227,217,227,218,227,219,227,220,227,221,227,222,227,223,227,224,227,225,227,226,227,227,227,228,227,229,227,230,227,231,227,232,227,233,227,234,227,235,227,236,227,237,227,238,227,239,227,240,227,241,227,242,227,243,227,244,227,245,227,246,227,247,227,248,227,249,227,250,227,251,227,252,227,253,227,254,227,255,227,0,228,1,228,2,228,3,228,4,228,5,228,6,228,7,228,8,228,9,228,50,159,51,159,52,159,53,159,54,159,56,159,58,159,60,159,63,159,64,159,65,159,66,159,67,159,69,159,70,159,71,159,72,159,73,159,74,159,75,159,76,159,77,159,78,159,79,159,82,159,83,159,84,159,85,159,86,159,87,159,88,159,89,159,90,159,91,159,92,159,93,159,94,159,95,159,96,159,97,159,98,159,99,159,100,159,101,159,102,159,103,159,104,159,105,159,106,159,107,159,108,159,109,159,110,159,111,159,112,159,113,159,114,159,115,159,116,159,117,159,118,159,119,159,120,159,121,159,122,159,123,159,124,159,125,159,126,159,129,159,130,159,141,159,142,159,143,159,144,159,145,159,146,159,147,159,148,159,149,159,150,159,151,159,152,159,156,159,157,159,158,159,161,159,162,159,163,159,164,159,165,159,44,249,121,249,149,249,231,249,241,249,10,228,11,228,12,228,13,228,14,228,15,228,16,228,17,228,18,228,19,228,20,228,21,228,22,228,23,228,24,228,25,228,26,228,27,228,28,228,29,228,30,228,31,228,32,228,33,228,34,228,35,228,36,228,37,228,38,228,39,228,40,228,41,228,42,228,43,228,44,228,45,228,46,228,47,228,48,228,49,228,50,228,51,228,52,228,53,228,54,228,55,228,56,228,57,228,58,228,59,228,60,228,61,228,62,228,63,228,64,228,65,228,66,228,67,228,68,228,69,228,70,228,71,228,72,228,73,228,74,228,75,228,76,228,77,228,78,228,79,228,80,228,81,228,82,228,83,228,84,228,85,228,86,228,87,228,88,228,89,228,90,228,91,228,92,228,93,228,94,228,95,228,96,228,97,228,98,228,99,228,100,228,101,228,102,228,103,228,12,250,13,250,14,250,15,250,17,250,19,250,20,250,24,250,31,250,32,250,33,250,35,250,36,250,39,250,40,250,41,250,129,46,22,232,23,232,24,232,132,46,115,52,71,52,136,46,139,46,30,232,158,53,26,54,14,54,140,46,151,46,110,57,24,57,38,232,207,57,223,57,115,58,208,57,43,232,44,232,78,59,110,60,224,60,167,46,49,232,50,232,170,46,86,64,95,65,174,46,55,67,179,46,182,46,183,46,59,232,177,67,172,67,187,46,221,67,214,68,97,70,76,70,67,232,35,71,41,71,124,71,141,71,202,46,71,73,122,73,125,73,130,73,131,73,133,73,134,73,159,73,155,73,183,73,182,73,84,232,85,232,163,76,159,76,160,76,161,76,119,76,162,76,19,77,20,77,21,77,22,77,23,77,24,77,25,77,174,77,100,232,104,228,105,228,106,228,107,228,108,228,109,228,110,228,111,228,112,228,113,228,114,228,115,228,116,228,117,228,118,228,119,228,120,228,121,228,122,228,123,228,124,228,125,228,126,228,127,228,128,228,129,228,130,228,131,228,132,228,133,228,134,228,135,228,136,228,137,228,138,228,139,228,140,228,141,228,142,228,143,228,144,228,145,228,146,228,147,228,148,228,149,228,150,228,151,228,152,228,153,228,154,228,155,228,156,228,157,228,158,228,159,228,160,228,161,228,162,228,163,228,164,228,165,228,166,228,167,228,168,228,169,228,170,228,171,228,172,228,173,228,174,228,175,228,176,228,177,228,178,228,179,228,180,228,181,228,182,228,183,228,184,228,185,228,186,228,187,228,188,228,189,228,190,228,191,228,192,228,193,228,194,228,195,228,196,228,197,228,178,157,239,167,198,75,3,64,246,40,92,143,194,245,10,192,229,208,34,219,249,126,15,64,25,4,86,14,45,178,16,192,109,231,251,169,241,210,10,64,164,112,61,10,215,163,1,192,229,208,34,219,249,126,244,63,104,145,237,124,63,53,226,191,144,160,248,49,230,174,181,63,136,73,26,0,168,151,47,0,152,1,0,0,2,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,224,195,0,0,0,0,0,0,224,67,0,0,0,0,0,0,0,0,72,58,26,0,168,151,47,0,160,1,0,0,2,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,224,195,0,0,0,0,0,0,224,67,0,0,0,0,0,0,0,0,224,40,26,0,168,151,47,0,188,1,0,0,2,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,224,195,0,0,0,0,0,0,224,67,0,0,0,0,0,0,0,0,16,31,26,0,168,151,47,0,124,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,184,17,26,0,168,151,47,0,68,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,96,9,26,0,168,151,47,0,72,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,232,1,26,0,168,151,47,0,80,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,72,249,25,0,168,151,47,0,96,1,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,0,0,0,0,0,0,0,0,104,236,25,0,168,151,47,0,88,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,3,3,3,3,3,3,3,3,3,3,1,3,3,3,0,0,3,0,3,2,3,3,2,3,3,3,3,3,3,3,3,3,3,3,3,0,0,0,0,0,3,3,3,3,3,3,3,3,3,3,0,0,0,3,3,3,3,3,3,0,0,3,3,3,3,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,0,3,3,3,3,3,1,1,0,0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,0,5,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,3,0,0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,3,0,0,0,0,0,0,0,200,74,26,0,96,0,0,0,224,202,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,247,24,0,48,53,26,0,208,202,25,0,200,125,25,0,200,67,25,0,0,0,0,0,38,0,0,0,34,0,0,0,98,0,0,0,50,0,0,0,44,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,0,0,0,52,0,0,0,112,0,0,0,100,0,0,0,70,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,0,0,0,8,0,0,0,80,0,0,0,36,0,0,0,124,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,76,0,0,0,60,0,0,0,122,0,0,0,58,0,0,0,92,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,42,0,0,0,20,0,0,0,78,0,0,0,6,0,0,0,126,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,58,0,0,0,104,0,0,0,122,0,0,0,18,0,0,0,50,0,0,0,36,0,0,0,26,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,12,0,0,0,66,0,0,0,58,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,51,0,0,0,0,0,0,0,7,6,0,0,0,0,0,0,15,6,0,0,0,0,0,0,15,7,0,0,0,0,0,0,63,6,0,0,0,0,0,0,68,172,0,0,0,0,0,0,5,0,0,0,210,0,0,0,24,255,1,0,65,0,0,0,176,179,0,0,0,0,0,0,9,0,0,0,20,1,0,0,144,143,3,0,66,0,0,0,176,179,0,0,0,0,0,0,9,0,0,0,160,0,0,0,240,249,1,0,67,0,0,0,176,179,0,0,0,0,0,0,9,0,0,0,65,1,0,0,208,163,2,0,68,0,0,0,128,187,0,0,0,0,0,0,16,0,0,0,45,1,0,0,56,33,1,0,69,0,0,0,68,172,0,0,0,0,0,0,20,0,0,0,77,1,0,0,184,33,1,0,69,0,0,0,168,147,0,0,0,0,0,0,16,0,0,0,240,0,0,0,88,34,1,0,69,0,0,0,0,125,0,0,0,0,0,0,20,0,0,0,240,0,0,0,216,34,1,0,69,0,0,0,34,86,0,0,0,0,0,0,20,0,0,0,240,0,0,0,120,35,1,0,69,0,0,0,128,62,0,0,0,0,0,0,20,0,0,0,240,0,0,0,24,36,1,0,69,0,0,0,17,43,0,0,0,0,0,0,20,0,0,0,240,0,0,0,184,36,1,0,69,0,0,0,64,31,0,0,0,0,0,0,20,0,0,0,240,0,0,0,88,37,1,0,69,0,0,0,128,187,0,0,0,0,0,0,16,0,0,0,250,0,0,0,56,32,1,0,70,0,0,0,68,172,0,0,0,0,0,0,15,0,0,0,250,0,0,0,184,32,1,0,70,0,0,0,68,172,0,0,0,0,0,0,20,0,0,0,127,1,0,0,248,37,1,0,71,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,80,138,25,0,200,130,25,0,24,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,16,0,1,0,80,138,25,0,88,123,25,0,0,0,0,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,80,138,25,0,136,116,25,0,160,111,25,0,28,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,16,0,1,0,0,0,0,0,8,107,25,0,248,100,25,0,12,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,0,0,0,0,104,95,25,0,128,89,25,0,36,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,145,25,0,96,0,0,0,216,150,3,0,100,39,52,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,214,51,105,231,7,248,0,0,127,240,2,30,170,0,0,0,228,0,189,252,233,16,0,0,2,1,255,2,2,1,255,2,0,255,1,0,0,255,1,0,0,1,255,0,0,1,255,0,2,255,1,2,2,255,1,2,5,4,255,5,5,4,255,5,3,255,4,3,3,255,4,3,3,4,255,3,3,4,255,3,5,255,4,5,5,255,4,5,8,7,255,8,8,7,255,8,6,255,7,6,6,255,7,6,6,7,255,6,6,7,255,6,8,255,7,8,8,255,7,8,0,2,0,2,0,2,0,2,2,0,2,0,2,0,2,0,1,255,1,255,1,255,1,255,255,1,255,1,255,1,255,1,3,5,3,5,3,5,3,5,5,3,5,3,5,3,5,3,4,255,4,255,4,255,4,255,255,4,255,4,255,4,255,4,6,8,6,8,6,8,6,8,8,6,8,6,8,6,8,6,7,255,7,255,7,255,7,255,255,7,255,7,255,7,255,7,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,2,1,0,255,255,255,255,255,5,4,3,255,255,255,255,255,8,7,6,255,255,255,255,255,88,120,26,0,112,245,25,0,8,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,36,26,0,96,0,0,0,88,153,3,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,2,6,2,9,2,14,2,0,0,0,0,184,47,26,0,152,0,0,0,128,159,33,0,100,39,52,0,0,0,0,0,0,0,0,0,92,0,0,0,62,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,0,0,0,2,0,0,0,6,0,0,0,46,0,0,0,78,0,0,0,68,0,0,0,48,0,0,0,34,0,0,0,52,0,0,0,42,0,0,0,40,0,0,0,60,0,0,0,70,0,0,0,8,0,0,0,28,0,0,0,0,0,0,0,104,41,26,0,96,0,0,0,96,52,34,0,100,39,52,0,0,0,0,0,0,0,0,0,88,0,0,0,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,4,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,70,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,120,251,25,0,96,0,0,0,40,167,4,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,4,26,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,70,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,160,159,25,0,96,0,0,0,136,85,5,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,203,1,0,213,28,2,0,130,54,0,0,87,136,0,0,0,203,1,0,213,28,2,0,130,54,0,0,87,136,0,0,149,152,1,0,105,4,2,0,75,100,0,0,31,208,0,0,149,152,1,0,105,4,2,0,75,100,0,0,31,208,0,0,0,152,1,0,190,6,2,0,183,96,0,0,117,207,0,0,149,152,1,0,105,4,2,0,75,100,0,0,31,208,0,0,149,152,1,0,105,4,2,0,75,100,0,0,31,208,0,0,75,203,1,0,38,20,2,0,11,66,0,0,231,138,0,0,192,53,25,0,208,67,26,0,1,0,0,0,7,80,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,49,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,96,110,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,172,0,0,0,0,0,0,0,54,0,0,0,0,0,0,0,94,0,0,0,0,0,0,0,0,0,0,0,240,54,25,0,0,68,26,0,1,0,0,0,7,80,1,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,50,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,96,110,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,148,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,94,0,0,0,62,0,0,0,0,0,0,0,202,96,254,63,119,94,253,63,165,99,252,63,168,54,251,63,45,166,249,63,97,120,247,63,225,30,244,63,52,83,238,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,126,252,63,136,252,250,63,229,147,249,63,103,18,248,63,216,107,246,63,57,126,244,63,166,55,242,63,64,141,239,63,51,62,236,63,105,35,232,63,139,90,227,63,144,199,221,63,12,236,214,63,18,7,206,63,220,15,193,63,223,100,170,63,244,86,243,63,144,178,238,63,245,198,234,63,153,157,230,63,182,28,226,63,116,52,221,63,247,174,215,63,192,149,209,63,204,225,202,63,63,76,194,63,8,233,183,63,237,116,171,63,15,242,158,63,152,177,143,63,221,173,119,63,69,144,61,63,247,124,226,63,17,208,214,63,58,187,204,63,187,142,196,63,59,195,188,63,191,191,180,63,22,44,172,63,156,254,162,63,121,181,152,63,218,127,140,63,214,249,124,63,62,85,95,63,149,98,62,63,173,1,25,63,253,5,221,62,175,21,56,62,154,150,183,63,84,2,169,63,142,249,154,63,50,138,141,63,19,184,128,63,141,66,103,63,131,235,77,63,74,4,53,63,179,177,26,63,69,12,255,62,57,43,197,62,109,201,138,62,45,136,19,62,199,30,16,59,244,67,43,190,222,28,189,190,163,187,127,63,142,133,87,63,44,33,53,63,171,61,20,63,171,236,240,62,87,69,189,62,57,90,137,62,197,192,47,62,188,160,147,61,108,234,250,188,116,63,179,191,10,169,128,190,13,137,190,190,239,192,2,191,174,28,38,191,172,150,77,191,28,194,135,62,49,77,129,61,64,179,181,189,175,25,88,190,166,19,159,190,203,41,203,190,192,200,241,190,249,170,10,191,109,76,29,191,182,126,48,191,155,11,68,191,17,9,90,191,164,240,115,191,189,86,136,191,243,64,154,191,149,38,176,191,81,22,10,191,132,218,60,191,32,94,98,191,83,110,129,191,86,35,145,191,174,87,162,191,81,208,181,191,70,103,207,191,215,0,141,191,214,34,149,191,28,223,157,191,49,238,166,191,132,53,176,191,17,249,187,191,210,194,201,191,181,159,213,191,105,164,197,190,74,98,15,191,5,119,43,191,5,162,66,191,91,165,86,191,79,159,104,191,226,93,121,191,196,186,132,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,151,221,199,191,138,104,212,191,245,10,221,191,109,168,227,191,72,56,233,191,152,92,238,191,65,144,243,191,198,94,249,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,231,2,1,0,205,5,93,0,201,5,94,0,3,0,22,0,11,0,1,0,6,0,1,0,6,0,4,0,5,0,4,0,7,0,3,0,7,0,4,0,10,0,10,0,2,0,3,0,0,0,5,0,2,0,95,0,4,0,3,0,2,0,200,5,184,0,202,5,203,5,204,5,0,0,0,0,0,0,10,12,10,13,9,13,9,8,7,5,5,4,4,3,3,3,4,3,4,4,5,5,6,8,7,10,8,10,9,8,9,9,13,10,13,13,13,0,0,0,100,0,200,0,44,1,144,1,254,1,118,2,2,3,152,3,56,4,246,4,200,5,184,6,208,7,16,9,140,10,78,12,116,14,48,17,180,20,0,25,20,30,28,37,224,46,140,60,180,95,0,0,0,0,0,0,88,185,25,0,192,100,26,0,0,1,0,0,0,0,0,0,72,50,29,0,88,3,0,0,0,0,0,0,0,0,0,0,72,0,0,0,86,0,0,0,156,0,0,0,124,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,168,186,25,0,80,101,26,0,152,233,25,0,0,147,25,0,184,85,25,0,192,18,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,1,0,0,0,0,0,0,1,2,3,0,0,0,0,0,2,1,3,4,0,0,0,0,2,1,4,5,3,0,0,0,2,1,5,6,4,3,0,0,2,1,6,7,4,5,3,216,229,24,0,7,0,0,0,0,0,0,0,130,0,0,0,106,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,40,0,0,0,0,0,0,0,9,9,10,14,14,14,14,14,14,14,14,14,14,0,0,0,31,31,34,40,42,51,46,46,42,42,42,39,39,0,0,0,64,227,25,0,7,0,0,0,0,0,0,0,32,0,0,0,82,0,0,0,22,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,64,64,64,64,64,64,64,64,184,20,1,0,184,20,1,0,184,20,1,0,248,20,1,0,248,20,1,0,248,20,1,0,24,21,1,0,24,21,1,0,56,21,1,0,56,21,1,0,56,21,1,0,216,20,1,0,216,20,1,0,0,0,0,0,88,21,1,0,88,21,1,0,8,22,1,0,104,22,1,0,104,22,1,0,208,22,1,0,56,23,1,0,56,23,1,0,152,23,1,0,152,23,1,0,152,23,1,0,176,21,1,0,176,21,1,0,0,0,0,0,200,227,25,0,8,0,0,0,0,0,0,0,44,0,0,0,78,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,184,228,25,0,8,0,0,0,0,0,0,0,118,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,144,253,34,0,144,180,34,0,16,34,35,0,16,254,34,0,144,181,34,0,16,36,35,0,16,2,35,0,144,189,34,0,144,52,34,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,208,42,39,0,144,152,38,0,16,116,37,0,240,42,39,0,208,152,38,0,144,116,37,0,240,43,39,0,208,154,38,0,144,120,37,0,16,52,35,0,208,170,38,0,144,152,37,0,16,116,35,0,0,0,0,0,6,222,127,191,13,71,24,191,70,156,120,63,125,223,44,191,21,177,16,62,212,110,115,191,11,171,240,190,1,47,191,190,223,154,78,63,168,211,151,62,206,149,199,190,78,78,101,63,29,136,44,188,74,106,43,191,84,164,105,191,98,253,235,189,56,100,12,63,19,145,64,63,238,216,204,62,95,66,125,191,122,169,127,191,16,168,97,191,63,143,116,191,247,185,104,63,229,29,234,190,156,49,17,191,49,179,58,191,180,230,122,191,41,152,65,63,8,136,86,62,61,200,144,61,139,80,72,191,221,181,62,63,133,100,105,191,10,227,118,191,193,136,114,191,44,198,155,62,186,31,253,190,15,104,42,63,243,130,37,63,141,190,106,63,40,88,51,62,13,47,53,191,73,134,6,63,229,84,51,191,21,36,232,190,9,182,126,191,135,149,102,191,21,77,123,63,65,78,70,191,145,245,11,191,71,156,210,188,23,106,138,188,148,107,60,59,71,113,92,191,72,217,217,62,129,42,125,191,199,249,96,191,57,127,4,63,35,85,43,63,208,16,127,191,123,193,20,191,237,235,127,191,194,211,123,63,76,123,13,63,137,48,24,63,62,14,177,62,45,228,114,63,158,107,32,63,187,120,62,191,212,128,69,191,253,123,173,190,229,121,106,191,115,14,23,61,118,74,67,191,120,233,105,191,22,66,76,63,21,139,110,191,122,115,11,63,248,26,244,189,143,60,91,191,169,61,217,62,105,199,109,191,35,180,142,62,150,201,239,189,123,125,127,191,112,177,90,62,213,59,104,191,253,12,67,191,9,104,127,63,14,42,123,63,46,99,117,191,31,240,91,191,148,41,117,63,214,221,110,191,50,23,253,62,74,22,156,62,30,149,52,191,109,87,90,63,78,113,239,62,198,204,105,63,225,150,127,191,57,67,113,189,93,25,53,63,186,102,145,62,229,82,177,62,100,220,115,63,243,134,12,191,58,33,73,191,153,116,65,191,155,56,117,191,46,34,81,190,250,248,82,63,146,111,119,63,239,223,38,191,93,41,99,191,70,186,111,191,82,171,127,63,180,13,106,63,170,159,123,191,132,54,52,191,18,133,22,63,159,188,184,59,104,32,30,63,172,1,100,63,25,32,7,63,142,212,47,191,164,221,78,63,71,190,56,63,36,78,49,191,168,24,33,191,118,138,11,62,232,72,153,62,102,200,235,190,57,246,106,191,249,120,61,191,104,52,126,63,38,251,208,62,227,220,82,63,68,136,61,191,95,49,124,191,222,237,127,191,25,139,117,191,124,12,126,191,252,63,91,190,228,42,111,191,216,46,48,191,73,176,137,190,32,36,107,63,18,168,191,61,178,235,117,191,206,211,184,62,216,54,4,63,58,183,54,191,161,126,28,63,125,98,240,62,105,92,242,62,37,242,139,190,167,244,104,63,148,151,119,63,250,154,229,62,76,251,125,63,188,136,42,63,86,69,119,63,10,205,63,63,170,23,102,191,60,231,126,191,133,33,7,63,198,88,121,63,167,154,44,190,5,18,58,63,56,18,246,190,113,78,116,63,71,65,48,63,131,200,58,191,28,30,68,191,48,133,90,191,107,43,99,63,115,106,80,191,255,140,121,191,80,26,97,191,13,91,63,63,76,58,55,191,60,87,124,191,124,210,85,63,75,71,86,63,94,51,246,190,91,13,125,191,25,173,120,63,40,10,77,63,5,26,5,63,22,111,77,63,127,8,11,188,137,70,68,191,31,244,51,191,172,184,13,63,95,125,117,191,246,132,221,190,220,133,120,63,90,254,190,61,20,142,108,191,84,230,90,63,195,239,49,191,101,128,124,63,174,251,135,62,68,175,59,63,140,142,143,62,69,222,20,62,41,203,61,191,6,60,126,63,204,150,143,188,163,127,83,191,226,31,13,191,38,46,125,191,168,199,122,63,229,177,112,191,75,241,125,191,149,145,43,63,255,69,45,191,179,88,1,62,33,48,187,61,110,78,82,190,107,114,55,191,122,86,122,191,193,101,79,63,173,135,9,63,129,245,207,62,117,133,135,190,227,66,45,191,221,236,113,63,222,85,94,63,110,188,59,190,146,184,126,191,139,201,215,188,250,225,87,63,110,34,213,61,56,190,188,189,190,5,50,63,71,227,127,63,90,26,253,189,173,29,76,191,134,115,106,191,235,167,118,63,74,102,119,63,77,167,76,191,80,171,36,63,92,223,236,189,169,94,146,62,209,103,204,190,199,248,112,63,25,171,125,63,42,174,107,191,153,151,146,62,186,12,105,191,70,65,85,191,168,93,44,191,66,60,116,63,159,182,251,62,220,23,132,189,154,36,5,61,130,233,126,191,35,9,217,62,212,174,39,191,108,81,83,63,233,2,80,191,109,42,4,191,63,24,127,191,183,75,88,63,164,163,60,59,135,206,37,63,49,167,51,63,99,112,81,190,185,175,118,63,112,107,208,62,169,87,48,191,230,211,105,63,33,144,178,190,21,248,54,63,17,120,107,63,83,66,42,63,146,118,125,191,119,210,91,63,227,75,48,63,140,125,14,63,25,174,126,191,149,86,77,190,108,48,113,63,13,57,127,191,218,148,44,191,216,175,253,62,163,96,242,190,67,234,91,191,152,172,18,62,66,3,113,191,138,218,149,190,218,229,107,61,125,57,224,62,41,255,73,191,73,22,186,190,28,20,38,63,95,54,179,189,30,20,122,63,36,7,119,191,109,35,10,191,35,38,14,63,89,246,73,63,1,66,60,63,213,234,118,63,216,39,158,190,55,128,78,191,140,110,18,61,81,39,121,191,151,185,124,63,210,218,247,62,230,31,81,191,216,132,104,191,58,189,45,63,157,22,80,63,56,225,33,190,207,80,90,63,226,166,78,63,59,27,125,190,174,173,244,62,201,57,237,190,113,179,118,63,90,13,197,62,238,206,125,191,140,112,127,190,138,176,84,63,148,163,114,191,126,104,22,191,235,122,83,60,219,147,116,63,171,6,91,191,74,4,119,191,201,228,35,191,74,40,121,191,10,129,253,61,37,252,105,63,215,104,20,63,102,207,127,191,42,250,53,63,254,102,94,191,121,201,79,190,87,89,134,190,220,193,46,191,71,144,124,190,61,93,96,191,250,240,197,60,76,58,139,62,126,24,82,63,34,211,89,191,104,174,98,63,96,120,101,191,162,23,58,190,36,230,133,190,164,153,191,61,77,103,12,63,126,243,11,191,246,181,117,63,121,179,189,62,182,94,25,191,10,40,52,191,247,138,105,63,241,39,177,190,239,145,126,191,11,16,48,191,166,210,154,190,10,112,137,190,175,198,84,63,23,64,251,62,225,61,232,190,157,142,199,62,242,132,116,63,28,66,122,191,86,84,89,61,230,105,49,190,48,126,109,191,242,221,127,63,4,54,21,63,44,67,38,191,133,187,47,63,70,27,246,189,18,76,18,191,206,231,22,191,213,106,178,190,241,23,214,190,170,171,38,62,15,181,127,63,54,17,228,61,228,5,17,191,183,170,103,191,75,254,112,63,183,164,180,62,48,219,65,191,109,88,9,63,141,89,82,62,4,193,113,191,35,243,127,63,68,97,76,63,123,225,148,62,85,55,181,62,32,235,32,191,248,122,198,62,221,106,222,62,83,71,124,191,246,164,123,191,170,66,87,62,60,208,71,62,4,65,113,191,150,107,116,191,50,208,123,63,71,13,111,63,57,117,53,191,225,51,90,191,238,217,170,189,141,63,93,191,117,120,234,190,129,16,199,62,197,5,121,63,177,162,107,63,133,212,31,191,143,65,100,63,169,172,12,63,129,151,188,190,233,238,118,63,232,92,112,63,169,81,102,191,13,32,126,63,92,227,25,189,112,205,112,191,31,159,211,62,193,85,127,63,34,88,44,190,20,206,183,190,26,195,238,190,106,132,86,61,86,71,131,190,31,236,187,62,156,231,197,190,212,161,106,63,209,155,156,190,94,164,48,63,106,214,104,63,204,238,197,190,90,185,126,63,161,195,149,190,231,183,191,62,2,17,26,191,233,39,94,63,63,100,121,191,157,245,118,63,45,118,98,63,181,56,19,63,196,242,84,61,36,205,89,62,131,245,8,191,58,240,120,63,80,20,253,190,199,89,123,63,35,70,125,191,97,19,206,190,157,22,123,191,127,131,58,191,169,248,139,190,214,223,127,63,200,62,129,61,16,159,11,191,15,135,81,190,155,130,17,190,117,136,41,63,85,172,57,63,41,255,88,191,23,87,194,60,42,241,99,191,154,127,99,63,49,131,117,63,142,119,68,63,170,43,60,63,70,124,25,189,17,136,162,190,53,151,188,190,190,165,174,190,167,225,205,62,130,193,244,62,205,216,201,190,92,165,123,63,243,0,163,60,235,135,158,190,10,27,57,190,37,251,127,63,45,144,134,190,156,118,110,191,132,174,123,191,217,205,127,63,131,42,77,191,102,62,133,190,52,143,66,191,27,217,182,190,19,159,69,62,221,189,127,191,115,23,63,63,28,150,93,63,255,81,14,63,151,12,171,62,212,162,92,63,173,119,125,63,163,131,60,61,194,188,42,191,143,155,106,191,11,226,35,63,78,64,32,62,140,230,126,63,106,185,234,62,114,98,34,191,164,217,87,62,19,214,157,189,23,84,101,191,0,100,124,63,24,230,97,63,230,13,77,63,133,174,188,190,87,159,72,63,239,254,232,62,252,85,178,61,46,52,79,63,9,61,94,191,78,117,201,62,146,17,202,190,59,8,43,191,147,143,122,63,134,132,57,191,121,76,115,191,39,72,101,63,55,34,46,62,22,253,11,63,66,228,68,191,19,87,118,191,194,87,127,63,5,156,218,62,216,81,116,63,113,83,120,63,8,236,126,63,115,131,10,191,134,168,143,62,118,134,90,191,183,65,90,63,7,75,37,191,208,142,1,191,216,144,40,191,7,220,120,191,183,126,108,190,27,73,116,63,6,14,126,191,204,33,120,191,157,221,60,63,195,16,158,62,125,142,212,62,198,31,123,190,25,207,33,63,185,31,173,190,196,242,197,190,185,170,110,189,165,37,142,189,241,122,106,190,222,117,120,63,162,138,61,190,102,137,243,62,147,204,214,189,234,176,96,191,152,7,56,191,98,194,71,63,234,120,127,63,121,129,102,63,176,92,19,63,67,12,105,191,116,4,148,62,69,140,118,63,247,1,216,62,102,133,246,62,15,240,110,63,217,232,222,190,47,139,120,191,16,202,93,63,103,5,188,62,138,37,39,63,32,153,79,191,241,199,179,61,21,90,134,190,110,128,109,191,242,7,87,63,52,243,14,63,196,196,127,191,176,210,117,191,70,26,63,63,56,186,248,61,71,108,63,191,212,183,137,190,121,51,117,63,217,92,74,191,187,104,116,63,236,229,175,189,252,98,249,62,95,255,127,63,84,56,237,62,153,139,224,61,20,213,67,191,70,97,100,63,244,239,18,63,236,174,180,62,80,245,64,63,187,144,119,63,137,124,23,191,180,224,100,191,156,57,64,63,236,157,151,190,165,111,124,191,194,45,128,62,85,178,208,190,68,132,102,191,196,86,43,63,211,125,124,63,207,129,121,191,89,84,249,186,195,197,102,63,187,112,127,63,63,98,95,191,185,9,126,63,191,80,106,191,56,193,32,190,47,103,7,189,175,125,190,190,151,237,147,61,63,87,99,191,4,183,126,63,194,143,120,63,179,193,63,191,125,246,127,63,155,207,59,61,3,133,127,63,14,251,99,191,76,198,162,190,119,123,86,191,205,243,121,63,226,73,171,62,176,192,92,191,92,64,127,191,115,176,110,63,255,192,105,190,13,71,65,62,174,39,44,63,76,89,21,189,93,205,82,189,36,14,109,191,118,221,127,63,59,174,111,63,23,101,43,63,236,50,125,63,203,251,126,191,212,161,228,190,69,48,82,63,5,211,126,63,223,234,125,63,254,96,56,63,233,32,39,191,23,153,5,63,196,88,112,63,100,187,63,191,106,164,119,63,220,174,104,63,11,152,99,190,206,60,18,63,185,245,225,190,143,72,109,191,15,97,91,191,39,113,99,63,239,246,106,63,232,88,237,190,240,190,57,63,44,190,127,191,251,66,127,191,135,5,21,63,123,193,70,63,194,118,113,63,1,13,158,61,67,29,22,63,209,124,15,191,209,65,83,63,179,230,123,63,195,18,202,62,188,112,243,62,63,166,47,63,212,32,40,63,73,183,59,62,212,19,6,61,42,212,63,191,21,16,198,190,114,108,3,63,218,52,121,191,151,42,119,191,254,102,8,191,39,11,106,191,70,136,127,63,128,128,28,63,202,166,0,191,50,103,99,191,127,128,163,60,21,44,90,63,96,210,127,63,58,31,114,63,111,62,84,63,13,129,129,189,226,43,22,191,146,233,119,63,127,244,51,190,89,157,246,190,127,151,85,63,50,92,5,190,123,145,115,61,123,8,82,62,111,167,80,63,98,101,114,191,113,71,230,190,86,97,209,62,64,192,101,191,116,155,127,63,198,37,203,62,193,160,63,191,242,107,155,189,178,175,61,63,71,112,67,63,36,190,213,62,214,177,62,191,147,127,114,63,214,23,38,63,202,164,211,62,187,137,31,63,233,84,110,191,30,37,216,62,56,245,157,189,156,142,221,60,203,230,93,189,246,205,76,63,24,89,106,63,4,36,75,191,186,99,185,190,100,131,35,63,116,119,166,61,85,102,7,63,161,185,25,63,250,12,62,63,37,253,55,61,164,175,125,63,28,46,71,190,6,219,77,191,106,152,98,191,134,104,37,191,32,212,56,63,75,192,238,61,70,45,86,191,14,86,115,191,137,42,120,191,80,78,31,191,90,226,83,63,109,32,246,60,226,84,127,191,231,216,122,191,135,3,187,62,120,247,127,191,63,110,184,190,142,31,100,63,250,155,122,191,176,39,213,61,52,9,70,63,115,171,117,63,112,110,181,190,159,40,2,63,35,9,118,63,22,37,46,62,14,191,68,191,213,119,132,62,19,186,127,63,97,216,54,188,90,38,124,63,206,197,75,191,161,172,120,63,31,30,126,191,98,212,126,191,94,144,127,191,116,198,153,60,39,64,53,191,243,113,172,62,74,101,52,191,63,199,110,63,23,155,127,63,232,188,124,191,222,54,34,191,138,176,40,190,241,123,38,190,170,154,117,191,39,119,223,190,109,179,114,191,178,156,127,191,60,99,118,63,233,254,43,190,117,104,125,191,17,58,97,191,44,158,55,191,172,55,226,62,93,137,111,191,194,223,110,63,48,199,127,191,201,125,112,191,115,104,16,191,88,202,98,191,132,214,243,62,76,237,127,63,31,194,86,191,171,246,64,191,159,120,5,60,60,90,112,63,109,27,231,189,118,236,89,63,155,3,6,63,114,69,203,62,26,91,81,63,181,144,189,190,72,231,94,191,118,114,184,190,247,111,177,62,75,19,111,191,140,66,88,191,57,42,39,191,62,210,60,190,132,242,244,61,9,190,127,63,147,99,113,63,76,230,84,63,46,53,64,63,27,238,181,190,81,53,17,63,251,139,118,190,105,233,239,62,104,81,154,190,153,30,121,63,231,215,126,191,140,24,196,190,11,52,124,63,86,13,210,62,49,103,173,60,52,99,197,61,2,183,52,61,59,83,90,191,109,45,106,63,192,127,99,63,129,82,127,191,227,203,246,190,134,210,119,191,239,43,141,62,223,26,22,63,26,173,40,191,133,158,22,63,135,6,125,63,76,252,127,63,70,120,83,190,25,194,11,63,43,11,31,191,236,83,25,191,240,19,80,62,36,105,94,191,4,88,122,191,37,129,118,63,176,149,227,61,5,1,92,62,79,206,211,190,96,120,144,62,80,31,119,62,70,80,3,63,150,247,41,191,21,244,168,189,105,119,9,191,215,251,121,191,52,229,120,191,228,28,98,62,137,185,95,63,103,8,3,190,97,19,67,62,72,73,131,60,98,84,237,190,105,214,115,191,54,172,144,189,73,222,113,191,40,168,108,191,228,43,214,189,250,149,86,191,162,219,138,188,178,140,64,63,245,226,127,191,62,145,215,190,30,73,127,63,10,144,56,191,242,62,179,190,193,214,73,63,177,76,7,63,38,84,121,63,232,174,136,190,243,247,125,63,39,242,17,191,215,148,68,63,186,227,67,191,27,237,81,191,189,60,60,191,93,186,81,63,92,58,127,63,62,206,136,190,131,100,48,63,239,192,221,190,129,108,90,63,108,240,125,63,5,52,107,63,79,146,78,191,146,13,167,190,211,145,220,62,152,115,96,190,24,86,45,63,131,108,116,191,160,191,15,63,229,249,52,191,40,214,221,61,245,183,176,189,28,140,105,63,120,237,40,191,192,189,250,190,184,25,16,63,25,237,99,191,217,112,55,191,252,22,39,63,183,190,40,63,35,165,44,63,173,28,81,191,47,150,244,190,208,236,43,190,221,69,127,191,70,135,111,191,103,173,104,191,221,206,30,63,165,140,135,189,76,214,115,190,159,138,126,63,252,163,64,62,195,19,122,63,46,195,146,190,19,185,114,63,244,157,121,191,244,79,116,191,227,203,75,191,159,106,251,190,106,108,168,62,32,178,127,63,25,211,99,63,94,38,1,63,187,37,92,191,88,121,241,62,9,73,63,190,201,237,30,191,113,0,64,63,96,153,224,190,188,254,127,63,135,126,124,63,120,41,9,191,110,119,29,191,144,40,101,191,59,89,29,189,86,234,50,191,128,31,154,190,103,9,157,189,163,110,214,62,195,63,179,60,187,129,92,191,57,78,125,63,204,25,44,63,187,46,10,190,118,22,53,191,18,246,67,191,76,193,112,63,74,219,184,61,184,232,82,191,126,199,182,61,170,51,164,190,254,93,0,63,65,112,19,63,163,90,125,191,61,253,185,190,77,96,152,61,68,242,127,63,179,177,16,190,255,116,108,191,182,211,122,191,133,99,228,190,78,214,10,191,244,112,226,62,180,152,54,63,189,102,22,61,21,213,34,63,176,145,5,63,170,157,90,191,87,111,114,191,76,157,149,188,25,211,124,191,187,168,82,63,68,213,95,63,218,220,100,63,132,34,111,191,204,213,211,62,119,236,117,63,247,96,110,63,12,176,121,63,224,139,92,63,176,21,127,63,191,175,51,63,237,113,114,191,39,115,151,190,217,126,106,63,42,66,123,191,6,222,127,191,13,71,24,191,70,156,120,63,125,223,44,191,21,177,16,62,212,110,115,191,11,171,240,190,1,47,191,190,223,154,78,63,168,211,151,62,206,149,199,190,78,78,101,63,29,136,44,188,74,106,43,191,84,164,105,191,98,253,235,189,192,216,25,0,104,106,26,0,128,235,25,0,200,148,25,0,24,87,25,0,96,20,25,0,40,229,24,0,32,204,24,0,8,180,24,0,112,163,24,0,144,155,26,0,112,130,26,0,248,111,26,0,232,91,26,0,184,75,26,0,96,61,26,0,136,46,26,0,184,32,26,0,176,18,26,0,160,10,26,0,200,2,26,0,88,250,25,0,88,239,25,0,112,229,25,0,0,0,0,0,0,0,0,0,0,128,64,192,32,160,96,224,16,144,80,208,48,176,112,240,8,136,72,200,40,168,104,232,24,152,88,216,56,184,120,248,4,132,68,196,36,164,100,228,20,148,84,212,52,180,116,244,12,140,76,204,44,172,108,236,28,156,92,220,60,188,124,252,2,130,66,194,34,162,98,226,18,146,82,210,50,178,114,242,10,138,74,202,42,170,106,234,26,154,90,218,58,186,122,250,6,134,70,198,38,166,102,230,22,150,86,214,54,182,118,246,14,142,78,206,46,174,110,238,30,158,94,222,62,190,126,254,1,129,65,193,33,161,97,225,17,145,81,209,49,177,113,241,9,137,73,201,41,169,105,233,25,153,89,217,57,185,121,249,5,133,69,197,37,165,101,229,21,149,85,213,53,181,117,245,13,141,77,205,45,173,109,237,29,157,93,221,61,189,125,253,3,131,67,195,35,163,99,227,19,147,83,211,51,179,115,243,11,139,75,203,43,171,107,235,27,155,91,219,59,187,123,251,7,135,71,199,39,167,103,231,23,151,87,215,55,183,119,247,15,143,79,207,47,175,111,239,31,159,95,223,63,191,127,255,0,0,0,0,73,52,50,48,0,0,0,0,73,89,85,86,0,0,0,0,89,86,49,50,6,0,0,0,89,85,86,57,6,0,0,0,89,86,85,57,7,0,0,0,89,52,49,66,4,0,0,0,89,52,50,66,4,0,0,0,80,52,50,50,4,0,0,0,89,86,49,54,12,0,0,0,73,52,50,48,12,0,0,0,73,89,85,86,12,0,0,0,89,86,49,50,13,0,0,0,89,52,50,66,13,0,0,0,80,52,50,50,8,0,0,0,89,56,48,48,8,0,0,0,89,56,32,32,1,0,0,0,89,85,89,50,1,0,0,0,89,52,50,50,1,0,0,0,86,52,50,50,1,0,0,0,86,89,85,89,1,0,0,0,89,85,78,86,17,0,0,0,85,89,86,89,17,0,0,0,72,68,89,67,17,0,0,0,85,89,78,86,17,0,0,0,85,89,78,89,17,0,0,0,117,121,118,49,17,0,0,0,50,86,117,49,17,0,0,0,65,86,82,110,17,0,0,0,65,86,49,120,17,0,0,0,65,86,117,112,17,0,0,0,86,68,84,90,17,0,0,0,97,117,118,50,17,0,0,0,99,121,117,118,18,0,0,0,89,52,49,49,8,0,0,0,71,82,69,89,25,0,0,0,78,86,49,50,26,0,0,0,78,86,50,49,46,0,0,0,82,71,66,15,50,0,0,0,66,71,82,15,44,0,0,0,82,71,66,16,48,0,0,0,66,71,82,16,45,0,0,0,15,66,71,82,49,0,0,0,15,82,71,66,43,0,0,0,16,66,71,82,47,0,0,0,16,82,71,66,62,0,0,0,82,71,66,12,64,0,0,0,66,71,82,12,63,0,0,0,12,66,71,82,65,0,0,0,12,82,71,66,36,1,0,0,82,66,65,64,38,1,0,0,66,82,65,64,35,1,0,0,64,82,66,65,37,1,0,0,64,66,82,65,28,0,0,0,82,71,66,65,40,1,0,0,82,71,66,0,30,0,0,0,66,71,82,65,42,1,0,0,66,71,82,0,29,0,0,0,65,66,71,82,41,1,0,0,0,66,71,82,27,0,0,0,65,82,71,66,39,1,0,0,0,82,71,66,2,0,0,0,82,71,66,24,3,0,0,0,66,71,82,24,7,0,0,0,52,49,49,80,4,0,0,0,52,50,50,80,13,0,0,0,52,50,50,80,33,0,0,0,52,52,48,80,34,0,0,0,52,52,48,80,5,0,0,0,52,52,52,80,14,0,0,0,52,52,52,80,9,0,0,0,66,49,87,48,10,0,0,0,66,48,87,49,19,0,0,0,66,71,82,8,22,0,0,0,82,71,66,8,20,0,0,0,66,71,82,4,23,0,0,0,82,71,66,4,24,0,0,0,66,52,66,89,21,0,0,0,82,52,66,89,42,0,0,0,82,71,66,48,41,0,0,0,48,82,71,66,68,0,0,0,66,71,82,48,67,0,0,0,48,66,71,82,32,0,0,0,89,49,0,16,31,0,0,0,16,0,49,89,72,0,0,0,89,51,11,10,71,0,0,0,10,11,51,89,74,0,0,0,89,51,10,10,73,0,0,0,10,10,51,89,78,0,0,0,89,51,0,10,77,0,0,0,10,0,51,89,46,1,0,0,89,51,11,12,45,1,0,0,12,11,51,89,50,1,0,0,89,51,10,12,49,1,0,0,12,10,51,89,54,1,0,0,89,51,0,12,53,1,0,0,12,0,51,89,48,1,0,0,89,51,11,14,47,1,0,0,14,11,51,89,52,1,0,0,89,51,10,14,51,1,0,0,14,10,51,89,56,1,0,0,89,51,0,14,55,1,0,0,14,0,51,89,54,0,0,0,89,51,11,16,55,0,0,0,16,11,51,89,56,0,0,0,89,51,10,16,57,0,0,0,16,10,51,89,58,0,0,0,89,51,0,16,59,0,0,0,16,0,51,89,35,0,0,0,89,52,11,8,44,1,0,0,89,52,10,8,43,1,0,0,89,52,0,8,66,0,0,0,89,50,0,8,92,0,0,0,89,52,11,9,91,0,0,0,9,11,52,89,94,0,0,0,89,52,10,9,93,0,0,0,9,10,52,89,96,0,0,0,89,52,0,9,95,0,0,0,9,0,52,89,98,0,0,0,89,52,11,10,97,0,0,0,10,11,52,89,100,0,0,0,89,52,10,10,99,0,0,0,10,10,52,89,102,0,0,0,89,52,0,10,101,0,0,0,10,0,52,89,104,0,0,0,89,52,11,16,103,0,0,0,16,11,52,89,106,0,0,0,89,52,10,16,105,0,0,0,16,10,52,89,108,0,0,0,89,52,0,16,107,0,0,0,16,0,52,89,0,0,0,0,82,52,50,48,7,0,0,0,82,52,49,49,17,0,0,0,50,118,117,121,17,0,0,0,50,86,117,121,17,0,0,0,65,86,85,73,17,0,0,0,98,120,121,118,1,0,0,0,121,117,118,50,1,0,0,0,121,117,118,115,1,0,0,0,68,86,79,79,46,0,0,0,76,53,53,53,44,0,0,0,76,53,54,53,43,0,0,0,66,53,54,53,3,0,0,0,50,52,66,71,3,0,0,0,98,120,98,103,30,0,0,0,66,71,82,65,28,0,0,0,82,71,66,65,2,0,0,0,98,120,114,103,29,0,0,0,65,66,71,82,31,0,0,0,98,49,54,103,41,0,0,0,98,52,56,114,44,0,0,0,3,0,0,0,5,0,0,0,89,86,50,52,1,0,0,0,89,86,89,85,255,255,255,255,0,0,0,0,147,101,94,255,255,255,223,208,171,0,0,0,0,0,0,0,255,255,255,255,255,204,255,255,153,255,255,102,255,255,51,255,255,0,255,204,255,255,204,204,255,204,153,255,204,102,255,204,51,255,204,0,255,153,255,255,153,204,255,153,153,255,153,102,255,153,51,255,153,0,255,102,255,255,102,204,255,102,153,255,102,102,255,102,51,255,102,0,255,51,255,255,51,204,255,51,153,255,51,102,255,51,51,255,51,0,255,0,255,255,0,204,255,0,153,255,0,102,255,0,51,255,0,0,204,255,255,204,255,204,204,255,153,204,255,102,204,255,51,204,255,0,204,204,255,204,204,204,204,204,153,204,204,102,204,204,51,204,204,0,204,153,255,204,153,204,204,153,153,204,153,102,204,153,51,204,153,0,204,102,255,204,102,204,204,102,153,204,102,102,204,102,51,204,102,0,204,51,255,204,51,204,204,51,153,204,51,102,204,51,51,204,51,0,204,0,255,204,0,204,204,0,153,204,0,102,204,0,51,204,0,0,153,255,255,153,255,204,153,255,153,153,255,102,153,255,51,153,255,0,153,204,255,153,204,204,153,204,153,153,204,102,153,204,51,153,204,0,153,153,255,153,153,204,153,153,153,153,153,102,153,153,51,153,153,0,153,102,255,153,102,204,153,102,153,153,102,102,153,102,51,153,102,0,153,51,255,153,51,204,153,51,153,153,51,102,153,51,51,153,51,0,153,0,255,153,0,204,153,0,153,153,0,102,153,0,51,153,0,0,102,255,255,102,255,204,102,255,153,102,255,102,102,255,51,102,255,0,102,204,255,102,204,204,102,204,153,102,204,102,102,204,51,102,204,0,102,153,255,102,153,204,102,153,153,102,153,102,102,153,51,102,153,0,102,102,255,102,102,204,102,102,153,102,102,102,102,102,51,102,102,0,102,51,255,102,51,204,102,51,153,102,51,102,102,51,51,102,51,0,102,0,255,102,0,204,102,0,153,102,0,102,102,0,51,102,0,0,51,255,255,51,255,204,51,255,153,51,255,102,51,255,51,51,255,0,51,204,255,51,204,204,51,204,153,51,204,102,51,204,51,51,204,0,51,153,255,51,153,204,51,153,153,51,153,102,51,153,51,51,153,0,51,102,255,51,102,204,51,102,153,51,102,102,51,102,51,51,102,0,51,51,255,51,51,204,51,51,153,51,51,102,51,51,51,51,51,0,51,0,255,51,0,204,51,0,153,51,0,102,51,0,51,51,0,0,0,255,255,0,255,204,0,255,153,0,255,102,0,255,51,0,255,0,0,204,255,0,204,204,0,204,153,0,204,102,0,204,51,0,204,0,0,153,255,0,153,204,0,153,153,0,153,102,0,153,51,0,153,0,0,102,255,0,102,204,0,102,153,0,102,102,0,102,51,0,102,0,0,51,255,0,51,204,0,51,153,0,51,102,0,51,51,0,51,0,0,0,255,0,0,204,0,0,153,0,0,102,0,0,51,238,0,0,221,0,0,187,0,0,170,0,0,136,0,0,119,0,0,85,0,0,68,0,0,34,0,0,17,0,0,0,238,0,0,221,0,0,187,0,0,170,0,0,136,0,0,119,0,0,85,0,0,68,0,0,34,0,0,17,0,0,0,238,0,0,221,0,0,187,0,0,170,0,0,136,0,0,119,0,0,85,0,0,68,0,0,34,0,0,17,238,238,238,221,221,221,187,187,187,170,170,170,136,136,136,119,119,119,85,85,85,68,68,68,34,34,34,17,17,17,0,0,0,255,251,255,239,217,187,232,201,177,147,101,94,252,222,232,157,136,145,255,255,255,255,255,255,255,255,255,71,72,55,122,94,85,223,208,171,255,251,249,232,202,197,138,124,119,0,0,0,224,217,25,0,184,134,25,0,1,0,0,0,5,0,1,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,50,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,80,73,25,0,160,70,26,0,1,0,0,0,0,0,1,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,50,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,232,228,25,0,8,0,0,0,168,109,26,0,80,0,0,0,68,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,110,26,0,4,0,0,0,0,0,0,0,140,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,110,26,0,8,0,0,0,0,0,0,0,134,0,0,0,0,0,0,0,8,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,64,230,25,0,6,0,0,0,0,0,0,0,38,0,0,0,30,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,192,237,25,0,5,0,0,0,0,0,0,0,38,0,0,0,30,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,72,150,25,0,35,0,0,0,0,0,0,0,54,0,0,0,30,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,224,109,26,0,6,0,0,0,0,0,0,0,38,0,0,0,30,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,56,233,25,0,88,110,26,0,216,238,25,0,192,150,25,0,12,80,1,0,31,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,40,216,1,0,0,0,0,0,24,0,0,0,88,0,0,0,94,0,0,0,50,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,152,234,25,0,224,111,26,0,0,35,0,0,152,234,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,22,0,0,0,66,0,0,0,14,0,0,0,154,0,0,0,14,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,43,80,1,0,0,80,1,0,1,80,1,0,0,0,0,0,0,0,0,0,56,0,0,0,0,0,0,0,4,0,0,0,58,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,3,4,5,6,8,0,0,0,0,0,0,0,0,192,93,0,0,233,3,0,0,24,0,0,0,1,0,0,0,25,0,0,0,1,0,0,0,48,117,0,0,233,3,0,0,30,0,0,0,1,0,0,0,50,0,0,0,1,0,0,0,96,234,0,0,233,3,0,0,60,0,0,0,1,0,0,0,15,0,0,0,1,0,0,0,5,0,0,0,1,0,0,0,10,0,0,0,1,0,0,0,12,0,0,0,1,0,0,0,15,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,27,0,0,0,30,0,0,0,8,0,0,0,12,0,0,0,30,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,15,0,0,0,31,0,0,0,63,0,0,0,127,0,0,0,255,0,0,0,255,1,0,0,255,3,0,0,255,7,0,0,255,15,0,0,255,31,0,0,255,63,0,0,255,127,0,0])
.concat([255,255,0,0,0,0,0,0,251,255,255,255,249,255,255,255,3,0,0,0,246,255,255,255,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,255,255,255,254,255,255,255,254,255,255,255,254,255,255,255,253,255,255,255,253,255,255,255,252,255,255,255,252,255,255,255,251,255,255,255,251,255,255,255,250,255,255,255,249,255,255,255,249,255,255,255,248,255,255,255,247,255,255,255,246,255,255,255,245,255,255,255,243,255,255,255,242,255,255,255,240,255,255,255,239,255,255,255,237,255,255,255,235,255,255,255,232,255,255,255,230,255,255,255,227,255,255,255,225,255,255,255,221,255,255,255,218,255,255,255,215,255,255,255,211,255,255,255,207,255,255,255,203,255,255,255,198,255,255,255,193,255,255,255,188,255,255,255,183,255,255,255,177,255,255,255,171,255,255,255,165,255,255,255,159,255,255,255,152,255,255,255,145,255,255,255,139,255,255,255,131,255,255,255,124,255,255,255,117,255,255,255,109,255,255,255,102,255,255,255,95,255,255,255,87,255,255,255,80,255,255,255,73,255,255,255,66,255,255,255,60,255,255,255,54,255,255,255,48,255,255,255,213,0,0,0,218,0,0,0,222,0,0,0,225,0,0,0,227,0,0,0,228,0,0,0,228,0,0,0,227,0,0,0,224,0,0,0,221,0,0,0,215,0,0,0,208,0,0,0,200,0,0,0,189,0,0,0,177,0,0,0,163,0,0,0,146,0,0,0,127,0,0,0,106,0,0,0,83,0,0,0,57,0,0,0,29,0,0,0,254,255,255,255,220,255,255,255,184,255,255,255,145,255,255,255,103,255,255,255,59,255,255,255,12,255,255,255,218,254,255,255,165,254,255,255,111,254,255,255,53,254,255,255,249,253,255,255,187,253,255,255,123,253,255,255,57,253,255,255,245,252,255,255,176,252,255,255,105,252,255,255,33,252,255,255,216,251,255,255,143,251,255,255,70,251,255,255,253,250,255,255,180,250,255,255,108,250,255,255,38,250,255,255,225,249,255,255,158,249,255,255,94,249,255,255,33,249,255,255,231,248,255,255,178,248,255,255,129,248,255,255,86,248,255,255,47,248,255,255,16,248,255,255,247,247,255,255,229,247,255,255,219,247,255,255,217,247,255,255,224,247,255,255,241,247,255,255,245,7,0,0,208,7,0,0,160,7,0,0,101,7,0,0,30,7,0,0,203,6,0,0,108,6,0,0,255,5,0,0,134,5,0,0,0,5,0,0,107,4,0,0,202,3,0,0,26,3,0,0,93,2,0,0,146,1,0,0,185,0,0,0,211,255,255,255,224,254,255,255,223,253,255,255,210,252,255,255,185,251,255,255,148,250,255,255,100,249,255,255,42,248,255,255,230,246,255,255,153,245,255,255,68,244,255,255,233,242,255,255,135,241,255,255,33,240,255,255,183,238,255,255,76,237,255,255,223,235,255,255,115,234,255,255,9,233,255,255,163,231,255,255,67,230,255,255,233,228,255,255,153,227,255,255,83,226,255,255,26,225,255,255,239,223,255,255,213,222,255,255,205,221,255,255,218,220,255,255,253,219,255,255,56,219,255,255,143,218,255,255,1,218,255,255,146,217,255,255,68,217,255,255,25,217,255,255,18,217,255,255,49,217,255,255,121,217,255,255,234,217,255,255,136,218,255,255,83,219,255,255,77,220,255,255,120,221,255,255,212,222,255,255,100,224,255,255,40,226,255,255,34,228,255,255,174,25,0,0,71,23,0,0,168,20,0,0,209,17,0,0,192,14,0,0,119,11,0,0,245,7,0,0,58,4,0,0,70,0,0,0,26,252,255,255,182,247,255,255,28,243,255,255,75,238,255,255,70,233,255,255,14,228,255,255,164,222,255,255,9,217,255,255,65,211,255,255,76,205,255,255,44,199,255,255,229,192,255,255,121,186,255,255,234,179,255,255,59,173,255,255,111,166,255,255,138,159,255,255,142,152,255,255,127,145,255,255,96,138,255,255,53,131,255,255,1,124,255,255,200,116,255,255,143,109,255,255,88,102,255,255,40,95,255,255,2,88,255,255,235,80,255,255,231,73,255,255,250,66,255,255,39,60,255,255,115,53,255,255,226,46,255,255,118,40,255,255,54,34,255,255,35,28,255,255,66,22,255,255,151,16,255,255,36,11,255,255,237,5,255,255,246,0,255,255,66,252,254,255,211,247,254,255,172,243,254,255,209,239,254,255,66,236,254,255,4,233,254,255,23,230,254,255,125,227,254,255,57,225,254,255,76,223,254,255,183,221,254,255,122,220,254,255,152,219,254,255,16,219,254,255,30,37,1,0,0,0,0,0,104,83,5,0,104,83,5,0,248,82,5,0,248,82,5,0,48,82,5,0,0,0,0,0,5,112,1,0,8,0,0,0,13,0,0,0,32,0,0,0,28,0,0,0,33,0,0,0,2,80,1,0,64,0,0,0,46,80,1,0,64,0,0,0,2,0,0,0,97,0,0,0,2,0,0,0,96,0,0,0,2,0,0,0,98,0,0,0,2,0,0,0,99,0,0,0,2,0,0,0,100,0,0,0,2,0,0,0,101,0,0,0,2,80,1,0,102,0,0,0,2,80,1,0,103,0,0,0,2,80,1,0,104,0,0,0,1,80,1,0,105,0,0,0,0,80,1,0,105,0,0,0,1,0,0,0,106,0,0,0,1,80,1,0,107,0,0,0,8,0,0,0,108,0,0,0,62,0,0,0,109,0,0,0,89,0,0,0,110,0,0,0,71,0,0,0,163,0,0,0,117,0,0,0,164,0,0,0,3,80,1,0,165,0,0,0,4,80,1,0,169,0,0,0,5,80,1,0,221,0,0,0,0,112,1,0,224,0,0,0,24,80,1,0,225,0,0,0,1,0,2,0,1,0,0,0,1,0,2,0,2,0,0,0,0,0,0,0,0,0,0,0,80,231,24,0,240,205,24,0,240,180,24,0,80,231,24,0,2,80,1,0,13,0,0,0,0,0,0,0,64,0,5,0,232,49,29,0,24,236,1,0,0,0,0,0,120,0,0,0,106,0,0,0,8,0,0,0,136,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,244,25,0,48,113,26,0,136,240,25,0,8,244,25,0,1,80,1,0,62,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,152,236,1,0,0,0,0,0,224,12,0,0,100,0,0,0,42,0,0,0,44,0,0,0,0,0,0,0,138,0,0,0,0,0,0,0,184,244,25,0,136,113,26,0,0,1,0,0,16,241,25,0,0,0,0,0,80,2,4,0,0,0,0,0,0,0,0,0,48,0,0,0,140,0,0,0,116,0,0,0,4,0,0,0,0,0,0,0,36,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,80,107,25,0,56,81,26,0,1,0,0,0,1,80,1,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,24,50,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,129,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,166,0,0,0,0,0,0,0,0,0,0,0,36,0,0,0,0,0,0,0,80,0,0,0,0,0,0,0,13,0,14,0,16,0,18,0,20,0,21,0,27,0,32,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,90,25,0,136,24,25,0,0,0,0,0,0,90,25,0,2,80,1,0,13,0,0,0,0,0,0,0,64,0,5,0,104,50,29,0,192,237,1,0,0,0,0,0,120,0,0,0,106,0,0,0,8,0,0,0,136,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,156,26,0,88,131,26,0,0,128,0,0,0,0,0,0,0,0,0,0,160,242,1,0,0,0,0,0,0,0,0,0,128,0,0,0,30,0,0,0,142,0,0,0,114,0,0,0,122,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,112,108,25,0,112,108,25,0,1,0,0,0,5,80,1,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,50,29,0,0,0,0,0,0,0,0,0,48,131,0,0,0,0,0,0,56,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,2,4,0,0,0,0,0,10,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,26,0,0,0,0,0,0,0,0,0,0,0,200,108,25,0,200,108,25,0,1,0,0,0,5,80,1,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,102,0,0,0,0,0,0,0,0,0,0,0,38,0,0,0,158,0,0,0,0,0,0,0,0,0,0,0,248,109,25,0,136,82,26,0,1,0,0,0,1,80,1,0,96,0,0,0,0,0,0,0,0,0,0,0,64,255,1,0,48,50,29,0,80,50,29,0,0,0,0,0,120,255,1,0,0,0,0,0,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,104,255,1,0,0,0,0,0,46,0,0,0,0,0,0,0,42,0,0,0,0,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,112,112,25,0,48,83,26,0,1,0,0,0,2,80,1,0,96,0,0,0,0,0,0,0,0,0,0,0,40,88,5,0,64,50,29,0,144,91,5,0,0,0,0,0,128,90,5,0,104,98,1,0,56,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,96,88,5,0,0,0,0,0,110,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,70,0,0,0,0,0,0,0,0,0,0,0,15,16,7,7,17,18,8,8,3,3,3,3,3,3,3,3,19,20,9,9,21,22,10,10,4,4,4,4,4,4,4,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,23,24,11,11,25,26,12,12,5,5,5,5,5,5,5,5,27,28,13,13,29,30,14,14,6,6,6,6,6,6,6,6,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,9,7,7,9,9,7,7,5,5,5,5,5,5,5,5,9,9,7,7,9,9,7,7,5,5,5,5,5,5,5,5,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,9,9,7,7,9,9,7,7,5,5,5,5,5,5,5,5,9,9,7,7,9,9,7,7,5,5,5,5,5,5,5,5,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0,0,2,3,1,1,0,0,0,0,0,0,0,0,4,5,2,2,6,7,3,3,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,9,4,4,10,11,5,5,2,2,2,2,2,2,2,2,12,13,6,6,14,15,7,7,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,84,65,76,66,84,66,80,77,84,67,79,77,84,67,79,78,84,67,79,80,84,68,76,89,84,69,78,67,84,69,88,84,84,70,76,84,84,73,84,49,84,73,84,50,84,73,84,51,84,75,69,89,84,76,65,78,84,76,69,78,84,77,69,68,84,79,65,76,84,79,70,78,84,79,76,89,84,79,80,69,84,79,87,78,84,80,69,49,84,80,69,50,84,80,69,51,84,80,69,52,84,80,79,83,84,80,85,66,84,82,67,75,84,82,83,78,84,82,83,79,84,83,82,67,84,83,83,69,0,0,0,0,0,0,0,0,176,106,25,0,144,100,25,0,240,94,25,0,72,89,25,0,200,83,25,0,184,78,25,0,160,73,25,0,48,64,25,0,216,59,25,0,120,51,25,0,120,45,25,0,232,38,25,0,120,29,25,0,176,23,25,0,104,16,25,0,56,6,25,0,72,255,24,0,80,247,24,0,184,244,24,0,160,242,24,0,216,239,24,0,0,0,0,0,105,109,97,103,101,47,103,105,102,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,98,0,0,0,105,109,97,103,101,47,106,112,101,103,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,105,109,97,103,101,47,106,112,103,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,105,109,97,103,101,47,112,110,103,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,62,0,0,0,105,109,97,103,101,47,116,105,102,102,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,97,0,0,0,105,109,97,103,101,47,98,109,112,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,79,0,0,0,74,80,71,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,80,78,71,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,62,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,84,68,69,78,84,68,79,82,84,68,82,67,84,68,82,76,84,68,84,71,84,73,80,76,84,77,67,76,84,77,79,79,84,80,82,79,84,83,79,65,84,83,79,80,84,83,79,84,84,83,83,84,0,0,0,0,72,182,25,0,24,174,25,0,0,167,25,0,24,174,25,0,176,159,25,0,72,152,25,0,144,144,25,0,232,137,25,0,152,130,25,0,224,122,25,0,80,116,25,0,64,111,25,0,0,0,0,0,0,0,0,0,84,68,65,84,84,73,77,69,84,79,82,89,84,82,68,65,84,83,73,90,84,89,69,82,0,0,0,0,0,0,0,0,128,33,26,0,56,120,26,0,96,245,25,0,104,154,25,0,48,91,25,0,152,25,25,0,144,231,24,0,64,206,24,0,16,181,24,0,248,164,24,0,16,157,26,0,120,131,26,0,176,113,26,0,176,93,26,0,56,77,26,0,128,63,26,0,248,47,26,0,248,33,26,0,160,19,26,0,104,11,26,0,136,3,26,0,136,251,25,0,32,241,25,0,96,230,25,0,224,221,25,0,176,207,25,0,96,199,25,0,176,188,25,0,0,0,0,0,0,0,0,0,240,173,25,0,152,98,26,0,40,232,25,0,0,146,25,0,240,84,25,0,128,17,25,0,248,227,24,0,56,203,24,0,104,179,24,0,216,161,24,0,128,154,26,0,24,129,26,0,32,110,26,0,136,91,26,0,232,74,26,0,48,60,26,0,216,41,26,0,40,32,26,0,48,18,26,0,64,10,26,0,128,2,26,0,0,250,25,0,120,238,25,0,0,229,25,0,128,220,25,0,112,206,25,0,0,198,25,0,40,187,25,0,224,180,25,0,112,173,25,0,160,165,25,0,136,158,25,0,176,150,25,0,144,143,25,0,144,136,25,0,200,128,25,0,232,121,25,0,200,115,25,0,176,110,25,0,216,105,25,0,216,99,25,0,32,94,25,0,112,88,25,0,56,83,25,0,80,78,25,0,56,72,25,0,120,63,25,0,216,58,25,0,168,50,25,0,232,44,25,0,8,38,25,0,128,28,25,0,80,22,25,0,200,15,25,0,152,5,25,0,56,254,24,0,200,246,24,0,80,244,24,0,0,242,24,0,40,239,24,0,160,236,24,0,192,233,24,0,240,229,24,0,40,227,24,0,56,225,24,0,232,221,24,0,40,219,24,0,16,217,24,0,112,215,24,0,40,213,24,0,56,211,24,0,216,207,24,0,232,204,24,0,216,201,24,0,56,200,24,0,208,197,24,0,16,195,24,0,112,193,24,0,16,192,24,0,160,190,24,0,32,189,24,0,160,182,24,0,56,180,24,0,200,178,24,0,104,177,24,0,176,175,24,0,144,173,24,0,40,172,24,0,224,170,24,0,192,169,24,0,128,168,24,0,104,166,24,0,224,163,24,0,64,161,24,0,160,159,24,0,224,156,24,0,232,154,24,0,144,153,24,0,32,152,24,0,88,151,24,0,160,160,26,0,192,158,26,0,200,155,26,0,176,149,26,0,232,143,26,0,40,141,26,0,176,138,26,0,176,137,26,0,128,136,26,0,176,135,26,0,96,134,26,0,224,132,26,0,160,130,26,0,224,128,26,0,208,126,26,0,112,123,26,0,48,121,26,0,32,120,26,0,248,118,26,0,184,117,26,0,56,116,26,0,40,115,26,0,32,112,26,0,176,109,26,0,56,107,26,0,88,104,26,0,72,102,26,0,144,101,26,0,192,99,26,0,104,98,26,0,104,97,26,0,88,95,26,0,64,92,26,0,8,91,26,0,80,89,26,0,56,87,26,0,200,84,26,0,240,83,26,0,208,82,26,0,240,80,26,0,24,80,26,0,176,78,26,0,40,76,26,0,152,74,26,0,0,73,26,0,112,70,26,0,24,69,26,0,80,68,26,0,0,0,0,1,103,110,16,40,166,212,32,50,51,12,113,24,136,98,16,25,25,134,56,140,68,48,33,2,86,78,110,97,135,62,115,77,152,12,3,6,156,11,115,230,192,181,24,99,13,57,224,91,2,212,198,25,26,121,140,50,52,36,240,22,129,19,247,255,128,2,0,1,241,128,128,128,160,0,0,3,0,32,0,0,6,80,128,0,0,0,0,1,104,238,49,18,17,0,0,0,0,0,0,0,0,0,0,1,103,122,16,41,182,212,32,42,51,29,199,98,161,8,64,84,102,59,142,197,66,2,16,37,100,44,137,232,133,228,33,75,144,131,6,149,209,6,70,151,32,200,215,67,8,17,194,30,76,145,15,1,64,22,236,7,140,4,4,5,0,0,3,0,1,0,0,3,0,100,132,0,0,0,0,0,0,0,0,1,104,206,49,18,17,0,0,0,0,0,0,0,0,0,0,1,103,122,16,41,182,212,32,34,51,25,198,99,35,33,1,17,152,206,51,25,24,33,2,86,185,61,125,126,79,227,63,17,241,158,8,184,140,84,67,192,120,2,39,226,112,30,48,16,16,20,0,0,3,0,4,0,0,3,0,202,16,0,0,0,0,0,0,0,0,0,0,1,104,206,51,72,208,0,0,0,0,0,0,0,0,0,0,1,103,122,16,41,182,212,32,34,51,25,198,99,35,33,1,17,152,206,51,25,24,33,3,58,70,101,106,101,36,173,233,18,50,20,26,38,52,173,164,65,130,35,1,80,43,26,36,105,72,48,64,46,17,18,8,198,140,4,65,40,76,52,240,30,1,19,242,224,60,96,32,32,40,0,0,3,0,8,0,0,3,1,148,0,0,0,0,1,104,206,51,72,208,0,0,0,0,0,0,0,40,231,25,0,5,0,0,0,0,0,0,0,126,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,40,36,26,0,202,0,0,0,0,0,0,0,106,0,0,0,36,0,0,0,20,0,0,0,108,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,82,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,184,153,3,0,0,0,0,0,120,0,0,0,0,0,0,0,136,231,25,0,5,0,0,0,0,0,0,0,98,0,0,0,0,0,0,0,16,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,48,49,50,51,52,53,54,55,56,57,97,98,99,100,101,102,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,183,43,0,200,36,43,0,72,0,42,0,40,183,43,0,8,37,43,0,200,0,42,0,40,184,43,0,8,39,43,0,200,4,42,0,72,192,39,0,8,55,43,0,200,36,42,0,72,0,40,0,0,0,0,0,0,0,1,0,1,0,0,0,5,0,1,0,1,0,0,0,12,0,1,0,1,0,0,0,8,0,1,0,1,0,0,0,6,16,1,0,2,0,0,0,21,0,1,0,3,0,0,0,23,0,1,0,3,0,0,0,7,0,1,0,6,0,0,0,6,0,1,0,7,0,0,0,37,80,1,0,10,0,0,0,32,73,75,79,16,0,0,0,1,16,1,0,17,0,0,0,17,0,1,0,17,0,0,0,32,73,75,79,23,0,0,0,14,16,1,0,32,0,0,0,21,80,1,0,34,0,0,0,30,80,1,0,49,0,0,0,0,32,1,0,56,0,0,0,53,80,1,0,66,0,0,0,11,16,1,0,69,0,0,0,0,80,1,0,80,0,0,0,1,80,1,0,85,0,0,0,0,32,1,0,87,0,0,0,1,32,1,0,88,0,0,0,3,16,1,0,97,0,0,0,2,16,1,0,98,0,0,0,1,16,1,0,105,0,0,0,32,80,1,0,117,0,0,0,2,80,1,0,255,0,0,0,42,80,1,0,48,1,0,0,7,80,1,0,96,1,0,0,8,80,1,0,97,1,0,0,38,80,1,0,98,1,0,0,39,80,1,0,99,1,0,0,12,16,1,0,0,2,0,0,31,80,1,0,112,2,0,0,28,16,1,0,143,2,0,0,27,80,1,0,1,4,0,0,59,80,1,0,2,4,0,0,30,80,1,0,0,21,0,0,21,80,1,0,1,21,0,0,2,80,1,0,0,22,0,0,50,80,1,0,2,22,0,0,3,80,1,0,0,32,0,0,4,80,1,0,1,32,0,0,67,78,79,83,72,32,0,0,76,78,79,83,72,32,0,0,6,0,1,0,117,108,0,0,2,80,1,0,109,112,0,0,2,80,1,0,67,65,0,0,53,80,1,0,0,161,0,0,2,80,1,0,6,161,0,0,36,80,1,0,9,161,0,0,12,80,1,0,172,241,0,0,13,16,1,0,70,83,0,0,5,80,1,0,111,86,0,0,0,0,0,0,0,0,0,0,3,80,1,0,44,128,109,224,70,219,207,17,180,209,0,128,95,108,187,234,40,80,1,0,191,170,35,233,88,203,113,68,161,25,255,250,1,228,206,98,41,80,1,0,175,135,251,167,2,45,251,66,164,212,5,205,147,132,59,221,0,80,1,0,43,128,109,224,70,219,207,17,180,209,0,128,95,108,187,234,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,114,97,119,32,14,0,0,0,121,117,118,50,14,0,0,0,50,118,117,121,14,0,0,0,121,117,118,115,14,0,0,0,76,53,53,53,14,0,0,0,76,53,54,53,14,0,0,0,66,53,54,53,14,0,0,0,50,52,66,71,14,0,0,0,66,71,82,65,14,0,0,0,82,71,66,65,14,0,0,0,65,66,71,82,14,0,0,0,98,49,54,103,14,0,0,0,98,52,56,114,14,0,0,0,98,120,98,103,14,0,0,0,98,120,114,103,14,0,0,0,98,120,121,118,14,0,0,0,78,79,49,54,14,0,0,0,68,86,79,79,14,0,0,0,82,52,50,48,14,0,0,0,82,52,49,49,146,0,0,0,82,49,48,107,146,0,0,0,82,49,48,103,134,0,0,0,114,50,49,48,73,85,86,65,65,86,85,73,80,82,86,65,65,86,114,112,80,82,86,65,83,85,68,83,128,0,0,0,118,50,49,48,128,0,0,0,98,120,121,50,56,48,51,86,118,51,48,56,56,48,52,86,118,52,48,56,158,0,0,0,118,52,49,48,80,49,52,89,89,52,49,80,52,86,85,89,121,117,118,52,54,49,50,84,89,50,49,54,8,0,0,0,106,112,101,103,8,0,0,0,109,106,112,97,8,0,0,0,65,86,68,74,8,0,0,0,100,109,98,49,9,0,0,0,109,106,112,98,23,0,0,0,83,86,81,49,23,0,0,0,115,118,113,49,23,0,0,0,115,118,113,105,24,0,0,0,83,86,81,51,13,0,0,0,109,112,52,118,13,0,0,0,68,73,86,88,13,0,0,0,88,86,73,68,13,0,0,0,51,73,86,50,5,0,0,0,104,50,54,51,5,0,0,0,115,50,54,51,25,0,0,0,100,118,99,112,25,0,0,0,100,118,99,32,25,0,0,0,100,118,112,112,25,0,0,0,100,118,53,112,25,0,0,0,100,118,53,110,25,0,0,0,65,86,100,118,25,0,0,0,65,86,100,49,25,0,0,0,100,118,104,113,25,0,0,0,100,118,104,112,25,0,0,0,100,118,104,49,25,0,0,0,100,118,104,50,25,0,0,0,100,118,104,52,25,0,0,0,100,118,104,53,25,0,0,0,100,118,104,54,25,0,0,0,100,118,104,51,30,0,0,0,86,80,51,49,43,0,0,0,114,112,122,97,44,0,0,0,99,118,105,100,49,0,0,0,56,66,80,83,50,0,0,0,115,109,99,32,56,0,0,0,114,108,101,32,82,73,71,83,114,108,101,49,46,0,0,0,87,82,76,69,59,0,0,0,113,100,114,119,14,0,0,0,87,82,65,87,28,0,0,0,97,118,99,49,28,0,0,0,97,105,53,112,28,0,0,0,97,105,53,113,28,0,0,0,97,105,53,50,28,0,0,0,97,105,53,51,28,0,0,0,97,105,53,53,28,0,0,0,97,105,53,54,28,0,0,0,97,105,49,112,28,0,0,0,97,105,49,113,28,0,0,0,97,105,49,50,28,0,0,0,97,105,49,51,28,0,0,0,97,105,49,53,28,0,0,0,97,105,49,54,28,0,0,0,65,86,105,110,1,0,0,0,109,49,118,49,1,0,0,0,109,112,101,103,1,0,0,0,109,49,118,32,2,0,0,0,109,50,118,49,2,0,0,0,104,100,118,49,2,0,0,0,104,100,118,50,2,0,0,0,104,100,118,51,2,0,0,0,104,100,118,52,2,0,0,0,104,100,118,53,2,0,0,0,104,100,118,54,2,0,0,0,104,100,118,55,2,0,0,0,104,100,118,56,2,0,0,0,104,100,118,57,2,0,0,0,104,100,118,97,2,0,0,0,109,120,53,110,2,0,0,0,109,120,53,112,2,0,0,0,109,120,52,110,2,0,0,0,109,120,52,112,2,0,0,0,109,120,51,110,2,0,0,0,109,120,51,112,2,0,0,0,120,100,53,49,2,0,0,0,120,100,53,52,2,0,0,0,120,100,53,53,2,0,0,0,120,100,53,57,2,0,0,0,120,100,53,97,2,0,0,0,120,100,53,98,2,0,0,0,120,100,53,99,2,0,0,0,120,100,53,100,2,0,0,0,120,100,53,101,2,0,0,0,120,100,53,102,2,0,0,0,120,100,118,49,2,0,0,0,120,100,118,50,2,0,0,0,120,100,118,51,2,0,0,0,120,100,118,52,2,0,0,0,120,100,118,53,2,0,0,0,120,100,118,54,2,0,0,0,120,100,118,55,2,0,0,0,120,100,118,56,2,0,0,0,120,100,118,57,2,0,0,0,120,100,118,97,2,0,0,0,120,100,118,98,2,0,0,0,120,100,118,99,2,0,0,0,120,100,118,100,2,0,0,0,120,100,118,101,2,0,0,0,120,100,118,102,2,0,0,0,120,100,104,100,2,0,0,0,120,100,104,50,2,0,0,0,65,86,109,112,89,0,0,0,109,106,112,50,94,0,0,0,116,103,97,32,97,0,0,0,116,105,102,102,98,0,0,0,103,105,102,32,62,0,0,0,112,110,103,32,62,0,0,0,77,78,71,32,71,0,0,0,118,99,45,49,88,0,0,0,97,118,115,50,117,0,0,0,100,114,97,99,100,0,0,0,65,86,100,110,5,0,0,0,72,50,54,51,17,0,0,0,51,73,86,68,14,0,0,0,65,86,49,120,14,0,0,0,65,86,117,112,102,0,0,0,115,103,105,32,129,0,0,0,100,112,120,32,82,88,69,48,101,120,114,32,149,0,0,0,97,112,99,104,149,0,0,0,97,112,99,110,149,0,0,0,97,112,99,115,149,0,0,0,97,112,99,111,149,0,0,0,97,112,52,104,51,0,0,0,102,108,105,99,170,0,0,0,105,99,111,100,0,0,0,0,0,0,0,0,5,112,1,0,116,101,120,116,5,112,1,0,116,120,51,103,56,48,54,99,99,54,48,56,0,0,0,0,0,0,0,0,2,80,1,0,109,112,52,97,3,80,1,0,97,99,45,51,3,80,1,0,115,97,99,51,0,16,1,0,105,109,97,52,16,80,1,0,97,108,97,99,0,32,1,0,115,97,109,114,1,32,1,0,115,97,119,98,4,80,1,0,100,116,115,99,4,80,1,0,100,116,115,104,4,80,1,0,100,116,115,108,4,80,1,0,68,84,83,32,41,80,1,0,101,99,45,51,6,80,1,0,118,100,118,97,6,80,1,0,100,118,99,97,18,80,1,0,97,103,115,109,60,80,1,0,105,108,98,99,9,80,1,0,77,65,67,51,10,80,1,0,77,65,67,54,43,80,1,0,46,109,112,49,0,80,1,0,46,109,112,50,1,80,1,0,46,109,112,51,1,80,1,0,85,0,115,109,34,80,1,0,110,109,111,115,7,0,1,0,97,108,97,119,20,0,1,0,102,108,51,50,21,0,1,0,102,108,51,50,22,0,1,0,102,108,54,52,23,0,1,0,102,108,54,52,6,0,1,0,117,108,97,119,1,0,1,0,116,119,111,115,0,0,1,0,115,111,119,116,0,0,1,0,108,112,99,109,13,0,1,0,105,110,50,52,12,0,1,0,105,110,50,52,9,0,1,0,105,110,51,50,8,0,1,0,105,110,51,50,4,0,1,0,115,111,119,116,5,0,1,0,114,97,119,32,5,0,1,0,78,79,78,69,24,80,1,0,81,99,108,112,24,80,1,0,81,99,108,113,24,80,1,0,115,113,99,112,19,80,1,0,81,68,77,50,51,80,1,0,81,68,77,67,36,80,1,0,115,112,101,120,36,80,1,0,83,80,88,78,8,80,1,0,87,77,65,50,99,118,101,115,115,101,118,99,118,109,115,115,115,115,109,118,0,0,0,0,0,0,0,0,28,0,0,0,72,50,54,52,28,0,0,0,104,50,54,52,28,0,0,0,88,50,54,52,28,0,0,0,120,50,54,52,28,0,0,0,97,118,99,49,28,0,0,0,68,65,86,67,28,0,0,0,83,77,86,50,28,0,0,0,86,83,83,72,28,0,0,0,81,50,54,52,28,0,0,0,86,50,54,52,5,0,0,0,72,50,54,51,5,0,0,0,88,50,54,51,5,0,0,0,84,50,54,51,5,0,0,0,76,50,54,51,5,0,0,0,86,88,49,75,5,0,0,0,90,121,71,111,5,0,0,0,77,50,54,51,5,0,0,0,108,115,118,109,20,0,0,0,72,50,54,51,21,0,0,0,73,50,54,51,4,0,0,0,72,50,54,49,5,0,0,0,85,50,54,51,13,0,0,0,70,77,80,52,13,0,0,0,68,73,86,88,13,0,0,0,68,88,53,48,13,0,0,0,88,86,73,68,13,0,0,0,77,80,52,83,13,0,0,0,77,52,83,50,13,0,0,0,4,0,0,0,13,0,0,0,90,77,80,52,13,0,0,0,68,73,86,49,13,0,0,0,66,76,90,48,13,0,0,0,109,112,52,118,13,0,0,0,85,77,80,52,13,0,0,0,87,86,49,70,13,0,0,0,83,69,68,71,13,0,0,0,82,77,80,52,13,0,0,0,51,73,86,50,13,0,0,0,87,65,87,86,13,0,0,0,70,70,68,83,13,0,0,0,70,86,70,87,13,0,0,0,68,67,79,68,13,0,0,0,77,86,88,77,13,0,0,0,80,77,52,86,13,0,0,0,83,77,80,52,13,0,0,0,68,88,71,77,13,0,0,0,86,73,68,77,13,0,0,0,77,52,84,51,13,0,0,0,71,69,79,88,13,0,0,0,72,68,88,52,13,0,0,0,68,77,75,50,13,0,0,0,68,73,71,73,13,0,0,0,73,78,77,67,13,0,0,0,69,80,72,86,13,0,0,0,69,77,52,65,13,0,0,0,77,52,67,67,13,0,0,0,83,78,52,48,13,0,0,0,86,83,80,88,13,0,0,0,85,76,68,88,13,0,0,0,71,69,79,86,13,0,0,0,83,73,80,80,13,0,0,0,83,77,52,86,13,0,0,0,88,86,73,88,13,0,0,0,68,114,101,88,13,0,0,0,81,77,80,52,13,0,0,0,80,76,86,49,17,0,0,0,77,80,52,51,17,0,0,0,68,73,86,51,17,0,0,0,77,80,71,51,17,0,0,0,68,73,86,53,17,0,0,0,68,73,86,54,17,0,0,0,68,73,86,52,17,0,0,0,68,86,88,51,17,0,0,0,65,80,52,49,17,0,0,0,67,79,76,49,17,0,0,0,67,79,76,48,16,0,0,0,77,80,52,50,16,0,0,0,68,73,86,50,15,0,0,0,77,80,71,52,15,0,0,0,77,80,52,49,18,0,0,0,87,77,86,49,19,0,0,0,87,77,86,50,19,0,0,0,71,88,86,69,25,0,0,0,100,118,115,100,25,0,0,0,100,118,104,100,25,0,0,0,100,118,104,49,25,0,0,0,100,118,115,108,25,0,0,0,100,118,50,53,25,0,0,0,100,118,53,48,25,0,0,0,99,100,118,99,25,0,0,0,67,68,86,72,25,0,0,0,67,68,86,53,25,0,0,0,100,118,99,32,25,0,0,0,100,118,99,115,25,0,0,0,100,118,104,49,25,0,0,0,100,118,105,115,25,0,0,0,112,100,118,99,25,0,0,0,83,76,50,53,25,0,0,0,83,76,68,86,1,0,0,0,109,112,103,49,1,0,0,0,109,112,103,50,2,0,0,0,109,112,103,50,2,0,0,0,77,80,69,71,1,0,0,0,80,73,77,49,2,0,0,0,80,73,77,50,1,0,0,0,86,67,82,50,1,0,0,0,1,0,0,16,2,0,0,0,2,0,0,16,13,0,0,0,4,0,0,16,2,0,0,0,68,86,82,32,2,0,0,0,77,77,69,83,2,0,0,0,76,77,80,50,2,0,0,0,115,108,105,102,2,0,0,0,69,77,50,86,2,0,0,0,77,55,48,49,2,0,0,0,109,112,103,118,1,0,0,0,66,87,49,48,1,0,0,0,88,77,80,71,8,0,0,0,77,74,80,71,8,0,0,0,76,74,80,71,8,0,0,0,100,109,98,49,8,0,0,0,109,106,112,97,10,0,0,0,76,74,80,71,8,0,0,0,74,80,71,76,12,0,0,0,77,74,76,83,12,0,0,0,77,74,80,71,8,0,0,0,77,74,76,83,8,0,0,0,106,112,101,103,8,0,0,0,73,74,80,71,110,82,86,65,65,86,82,110,8,0,0,0,65,67,68,86,8,0,0,0,81,73,86,71,8,0,0,0,83,76,77,74,8,0,0,0,67,74,80,71,8,0,0,0,73,74,76,86,8,0,0,0,77,86,74,80,8,0,0,0,65,86,73,49,8,0,0,0,65,86,73,50,8,0,0,0,77,84,83,74,8,0,0,0,90,74,80,71,8,0,0,0,77,77,74,80,26,0,0,0,72,70,89,85,68,0,0,0,70,70,86,72,27,0,0,0,67,89,85,86,14,0,0,0,0,0,0,0,14,0,0,0,3,0,0,0,14,0,0,0,73,52,50,48,14,0,0,0,89,85,89,50,14,0,0,0,89,52,50,50,14,0,0,0,86,52,50,50,14,0,0,0,89,85,78,86,14,0,0,0,85,89,78,86,14,0,0,0,85,89,78,89,14,0,0,0,117,121,118,49,14,0,0,0,50,86,117,49,14,0,0,0,50,118,117,121,14,0,0,0,121,117,118,115,14,0,0,0,121,117,118,50,14,0,0,0,80,52,50,50,14,0,0,0,89,86,49,50,14,0,0,0,89,86,49,54,14,0,0,0,89,86,50,52,14,0,0,0,85,89,86,89,14,0,0,0,86,89,85,89,14,0,0,0,73,89,85,86,14,0,0,0,89,56,48,48,14,0,0,0,89,56,32,32,14,0,0,0,72,68,89,67,14,0,0,0,89,86,85,57,14,0,0,0,86,68,84,90,14,0,0,0,89,52,49,49,14,0,0,0,78,86,49,50,14,0,0,0,78,86,50,49,14,0,0,0,89,52,49,66,14,0,0,0,89,52,50,66,14,0,0,0,89,85,86,57,14,0,0,0,89,86,85,57,14,0,0,0,97,117,118,50,14,0,0,0,89,86,89,85,131,0,0,0,70,82,87,85,146,0,0,0,82,49,48,107,134,0,0,0,114,50,49,48,128,0,0,0,118,50,49,48,56,48,51,86,118,51,48,56,56,48,52,86,118,52,48,56,86,85,89,65,65,89,85,86,158,0,0,0,118,52,49,48,52,86,85,89,121,117,118,52,29,0,0,0,73,86,51,49,29,0,0,0,73,86,51,50,112,0,0,0,73,86,52,49,113,0,0,0,73,86,53,48,30,0,0,0,86,80,51,49,30,0,0,0,86,80,51,48,91,0,0,0,86,80,53,48,92,0,0,0,86,80,54,48,92,0,0,0,86,80,54,49,92,0,0,0,86,80,54,50,93,0,0,0,86,80,54,70,93,0,0,0,70,76,86,52,141,0,0,0,86,80,56,48,32,0,0,0,65,83,86,49,33,0,0,0,65,83,86,50,36,0,0,0,86,67,82,49,34,0,0,0,70,70,86,49,42,0,0,0,88,120,97,110,114,0,0,0,76,77,50,48,46,0,0,0,109,114,108,101,46,0,0,0,1,0,0,0,46,0,0,0,2,0,0,0,47,0,0,0,77,83,86,67,47,0,0,0,109,115,118,99,47,0,0,0,67,82,65,77,47,0,0,0,99,114,97,109,47,0,0,0,87,72,65,77,47,0,0,0,119,104,97,109,44,0,0,0,99,118,105,100,52,0,0,0,68,85,67,75,52,0,0,0,80,86,69,90,54,0,0,0,77,83,90,72,55,0,0,0,90,76,73,66,87,79,78,83,83,78,79,87,35,0,0,0,52,88,77,86,22,0,0,0,70,76,86,49,22,0,0,0,83,50,54,51,87,0,0,0,70,83,86,49,23,0,0,0,115,118,113,49,57,0,0,0,116,115,99,99,58,0,0,0,85,76,84,73,60,0,0,0,86,73,88,76,61,0,0,0,81,80,69,71,61,0,0,0,81,49,46,48,61,0,0,0,81,49,46,49,72,0,0,0,87,77,86,51,152,0,0,0,87,77,86,80,71,0,0,0,87,86,67,49,71,0,0,0,87,77,86,65,153,0,0,0,87,86,80,50,73,0,0,0,76,79,67,79,74,0,0,0,87,78,86,49,74,0,0,0,89,85,86,56,75,0,0,0,65,65,83,52,75,0,0,0,65,65,83,67,76,0,0,0,82,84,50,49,77,0,0,0,70,80,83,49,31,0,0,0,116,104,101,111,78,0,0,0,84,77,50,48,80,0,0,0,67,83,67,68,82,0,0,0,90,77,66,86,86,0,0,0,75,77,86,67,88,0,0,0,67,65,86,83,89,0,0,0,109,106,112,50,89,0,0,0,77,74,50,67,89,0,0,0,76,74,50,67,89,0,0,0,76,74,50,75,89,0,0,0,73,80,74,50,90,0,0,0,86,77,110,99,94,0,0,0,116,103,97,32,62,0,0,0,77,80,78,71,62,0,0,0,80,78,71,49,37,0,0,0,67,76,74,82,117,0,0,0,100,114,97,99,43,0,0,0,97,122,112,114,43,0,0,0,82,80,90,65,43,0,0,0,114,112,122,97,11,0,0,0,83,80,53,52,124,0,0,0,65,85,82,65,125,0,0,0,65,85,82,50,129,0,0,0,100,112,120,32,139,0,0,0,75,71,86,49,148,0,0,0,76,65,71,83,108,0,0,0,65,77,86,70,154,0,0,0,85,76,82,65,154,0,0,0,85,76,82,71,154,0,0,0,85,76,89,48,154,0,0,0,85,76,89,50,154,0,0,0,85,76,72,48,154,0,0,0,85,76,72,50,156,0,0,0,86,66,76,69,48,51,49,69,69,49,51,48,157,0,0,0,120,116,111,114,162,0,0,0,90,69,67,79,80,49,52,89,89,52,49,80,51,0,0,0,65,70,76,67,163,0,0,0,77,83,83,49,164,0,0,0,77,83,65,49,165,0,0,0,84,83,67,50,166,0,0,0,77,84,83,50,167,0,0,0,67,76,76,67,168,0,0,0,77,83,83,50,24,0,0,0,83,86,81,51,86,50,49,48,48,49,50,118,86,50,49,48,97,49,50,118,77,50,71,0,71,50,77,50,77,50,71,0,71,50,77,51,77,50,71,0,71,50,77,52,0,0,0,0,0,0,0,0,184,231,25,0,8,0,0,0,0,0,0,0,66,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,192,239,25,188,77,91,207,17,168,253,0,128,95,92,68,43,0,87,251,32,85,91,207,17,168,253,0,128,95,92,68,43,145,7,220,183,183,169,207,17,142,230,0,192,12,32,83,101,144,8,0,51,177,229,207,17,137,244,0,160,201,3,73,203,184,49,26,0,96,121,26,0,96,246,25,0,72,155,25,0,8,80,1,0,17,0,0,0,0,0,0,0,64,0,0,0,120,50,29,0,0,0,0,0,0,0,0,0,96,101,2,0,128,0,0,0,50,0,0,0,114,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,148,28,35,68,152,148,209,73,161,65,29,19,78,69,112,84,234,203,248,197,175,91,119,72,132,103,170,140,68,250,76,202])
.concat([168,176,25,0,88,99,26,0,120,232,25,0,80,146,25,0,40,85,25,0,224,17,25,0,24,228,24,0,72,203,24,0,112,179,24,0,224,161,24,0,136,154,26,0,64,129,26,0,40,110,26,0,160,91,26,0,16,75,26,0,64,60,26,0,32,46,26,0,96,32,26,0,88,18,26,0,80,10,26,0,144,2,26,0,8,250,25,0,200,238,25,0,8,229,25,0,144,220,25,0,160,91,26,0,128,206,25,0,8,198,25,0,56,187,25,0,232,180,25,0,128,173,25,0,168,165,25,0,0,0,0,0,0,0,0,0,1,205,135,244,81,169,207,17,142,230,0,192,12,32,83,101,169,70,67,124,224,239,252,75,178,41,57,62,222,65,92,133,0,225,27,182,78,91,207,17,168,253,0,128,95,92,68,43,48,38,178,117,142,102,207,17,166,217,0,170,0,98,206,108,17,210,211,171,186,169,207,17,142,230,0,192,12,32,83,101,181,3,191,95,46,169,207,17,142,227,0,192,12,32,83,101,161,220,171,140,71,169,207,17,142,228,0,192,12,32,83,101,64,164,208,210,7,227,210,17,151,240,0,160,201,94,168,80,203,165,230,20,114,198,50,67,131,153,169,105,82,6,91,90,226,101,251,58,239,71,242,64,172,44,112,169,13,113,211,67,157,140,23,49,225,3,40,69,181,130,61,249,219,34,245,3,20,230,138,41,34,38,23,76,185,53,218,224,126,233,40,156,252,179,17,34,35,189,210,17,180,183,0,160,201,85,252,110,200,59,26,0,32,123,26,0,0,96,0,0,0,0,0,0,0,0,0,0,104,77,5,0,0,0,0,0,0,0,0,0,216,99,2,0,42,0,0,0,160,0,0,0,48,0,0,0,56,0,0,0,26,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,54,38,178,117,142,102,207,17,166,217,0,170,0,98,206,108,251,179,17,34,35,189,210,17,180,183,0,160,201,85,252,110,51,38,178,117,142,102,207,17,166,217,0,170,0,98,206,108,192,207,218,89,230,89,208,17,163,172,0,160,201,3,72,246,64,82,209,134,29,49,208,17,163,164,0,160,201,3,72,246,65,82,209,134,29,49,208,17,163,164,0,160,201,3,72,246,64,158,105,248,77,91,207,17,168,253,0,128,95,92,68,43,80,205,195,191,143,97,207,17,139,178,0,170,0,180,226,32,81,0,81,0,81,0,81,0,81,0,81,0,64,0,64,0,169,0,169,0,33,1,0,0,24,111,4,0,208,106,4,0,40,106,4,0,128,105,4,0,216,104,4,0,48,104,4,0,176,103,4,0,48,103,4,0,216,101,4,0,192,109,4,0,120,107,4,0,0,0,0,0,112,179,4,0,64,177,4,0,232,176,4,0,144,176,4,0,56,176,4,0,224,175,4,0,160,175,4,0,96,175,4,0,176,174,4,0,192,178,4,0,152,177,4,0,0,0,0,0,232,255,3,0,230,255,3,0,231,255,3,0,229,255,3,0,245,255,7,0,241,255,7,0,237,255,7,0,246,255,7,0,238,255,7,0,239,255,7,0,240,255,7,0,252,255,7,0,253,255,7,0,255,255,7,0,254,255,7,0,247,255,7,0,248,255,7,0,251,255,7,0,249,255,7,0,228,255,3,0,250,255,7,0,227,255,3,0,239,255,1,0,240,255,1,0,245,255,0,0,238,255,1,0,242,255,0,0,243,255,0,0,244,255,0,0,241,255,0,0,246,127,0,0,247,127,0,0,249,63,0,0,245,63,0,0,247,63,0,0,243,63,0,0,246,63,0,0,242,63,0,0,247,31,0,0,245,31,0,0,249,15,0,0,247,15,0,0,246,15,0,0,249,7,0,0,244,15,0,0,248,7,0,0,249,3,0,0,247,3,0,0,245,3,0,0,248,1,0,0,247,1,0,0,250,0,0,0,248,0,0,0,246,0,0,0,121,0,0,0,58,0,0,0,56,0,0,0,26,0,0,0,11,0,0,0,4,0,0,0,0,0,0,0,10,0,0,0,12,0,0,0,27,0,0,0,57,0,0,0,59,0,0,0,120,0,0,0,122,0,0,0,247,0,0,0,249,0,0,0,246,1,0,0,249,1,0,0,244,3,0,0,246,3,0,0,248,3,0,0,245,7,0,0,244,7,0,0,246,7,0,0,247,7,0,0,245,15,0,0,248,15,0,0,244,31,0,0,246,31,0,0,248,31,0,0,248,63,0,0,244,63,0,0,240,255,0,0,244,127,0,0,246,255,0,0,245,127,0,0,226,255,3,0,217,255,7,0,218,255,7,0,219,255,7,0,220,255,7,0,221,255,7,0,222,255,7,0,216,255,7,0,210,255,7,0,211,255,7,0,212,255,7,0,213,255,7,0,214,255,7,0,242,255,7,0,223,255,7,0,231,255,7,0,232,255,7,0,233,255,7,0,234,255,7,0,235,255,7,0,230,255,7,0,224,255,7,0,225,255,7,0,226,255,7,0,227,255,7,0,228,255,7,0,229,255,7,0,215,255,7,0,236,255,7,0,244,255,7,0,243,255,7,0,0,0,0,0,18,18,18,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,18,19,18,17,17,16,17,16,16,16,16,15,15,14,14,14,14,14,14,13,13,12,12,12,11,12,11,10,10,10,9,9,8,8,8,7,6,6,5,4,3,1,4,4,5,6,6,7,7,8,8,9,9,10,10,10,11,11,11,11,12,12,13,13,13,14,14,16,15,16,15,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,0,0,0,0,0,0,0,33,33,38,40,40,40,41,41,37,37,37,34,34,0,0,0,12,12,12,14,14,14,15,15,15,15,15,15,15,0,0,0,41,41,47,49,49,51,47,47,43,43,43,40,40,0,0,0,152,183,25,0,64,100,26,0,0,1,0,0,152,183,25,0,0,0,0,0,0,0,0,0,0,0,0,0,2,80,1,0,0,0,0,0,28,0,0,0,74,0,0,0,112,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,52,25,0,88,67,26,0,1,0,0,0,2,80,1,0,2,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,49,29,0,80,91,5,0,0,0,0,0,176,90,5,0,0,0,0,0,64,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,138,0,0,0,26,0,0,0,0,0,0,0,64,146,4,0,64,146,4,0,184,143,4,0,184,143,4,0,232,142,4,0,232,142,4,0,184,143,4,0,184,143,4,0,184,143,4,0,184,143,4,0,184,143,4,0,0,0,0,0,80,146,4,0,80,146,4,0,80,146,4,0,80,146,4,0,16,143,4,0,16,143,4,0,104,142,4,0,104,142,4,0,16,141,4,0,16,141,4,0,248,143,4,0,0,0,0,0,247,31,248,31,249,31,250,31,251,31,248,63,249,63,252,7,254,0,126,0,14,0,2,0,0,0,6,0,30,0,62,0,254,1,250,15,246,31,250,63,251,63,252,63,253,63,254,63,255,63,0,0,0,0,0,0,13,13,13,13,13,14,14,11,8,7,4,2,1,3,5,6,9,12,13,14,14,14,14,14,14,0,0,0,0,0,0,0,226,255,3,0,227,255,3,0,228,255,3,0,229,255,3,0,230,255,3,0,231,255,3,0,232,255,3,0,233,255,3,0,234,255,3,0,235,255,3,0,236,255,3,0,237,255,3,0,238,255,3,0,239,255,3,0,240,255,3,0,247,255,0,0,240,255,1,0,252,63,0,0,254,7,0,0,252,7,0,0,254,0,0,0,126,0,0,0,14,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,30,0,0,0,62,0,0,0,254,1,0,0,253,7,0,0,254,15,0,0,250,127,0,0,246,255,0,0,241,255,3,0,242,255,3,0,243,255,3,0,244,255,3,0,245,255,3,0,246,255,3,0,247,255,3,0,248,255,3,0,249,255,3,0,250,255,3,0,251,255,3,0,252,255,3,0,253,255,3,0,254,255,3,0,254,255,7,0,255,255,7,0,0,0,0,0,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,16,17,14,11,11,8,7,4,2,1,3,5,6,9,11,12,15,16,18,18,18,18,18,18,18,18,18,18,18,18,18,18,19,19,0,0,0,0,0,0,0,240,255,15,0,241,255,15,0,242,255,15,0,243,255,15,0,244,255,15,0,245,255,15,0,246,255,15,0,243,255,3,0,245,255,7,0,238,255,7,0,239,255,7,0,246,255,7,0,244,255,3,0,242,255,3,0,247,255,15,0,240,255,7,0,245,255,1,0,240,255,3,0,244,255,1,0,247,255,0,0,246,255,0,0,248,127,0,0,251,63,0,0,253,15,0,0,253,7,0,0,253,3,0,0,253,1,0,0,253,0,0,0,62,0,0,0,14,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,30,0,0,0,252,0,0,0,252,1,0,0,252,3,0,0,252,7,0,0,252,15,0,0,252,31,0,0,250,63,0,0,249,127,0,0,250,127,0,0,248,255,0,0,249,255,0,0,246,255,1,0,247,255,1,0,245,255,3,0,246,255,3,0,241,255,3,0,248,255,15,0,241,255,7,0,242,255,7,0,243,255,7,0,249,255,15,0,247,255,7,0,244,255,7,0,250,255,15,0,251,255,15,0,252,255,15,0,253,255,15,0,254,255,15,0,255,255,15,0,0,0,0,0,20,20,20,20,20,20,20,18,19,19,19,19,18,18,20,19,17,18,17,16,16,15,14,12,11,10,9,8,6,4,2,1,3,5,8,9,10,11,12,13,14,15,15,16,16,17,17,18,18,18,20,19,19,19,20,19,19,20,20,20,20,20,20,0,231,255,7,0,232,255,7,0,210,255,15,0,211,255,15,0,212,255,15,0,213,255,15,0,214,255,15,0,215,255,15,0,216,255,15,0,218,255,7,0,217,255,15,0,218,255,15,0,219,255,15,0,220,255,15,0,219,255,7,0,221,255,15,0,220,255,7,0,221,255,7,0,222,255,15,0,228,255,3,0,223,255,15,0,224,255,15,0,225,255,15,0,222,255,7,0,226,255,15,0,227,255,15,0,228,255,15,0,223,255,7,0,229,255,15,0,224,255,7,0,232,255,3,0,225,255,7,0,224,255,3,0,233,255,3,0,239,255,1,0,229,255,3,0,236,255,1,0,237,255,1,0,238,255,1,0,244,255,0,0,243,255,0,0,240,255,0,0,247,127,0,0,246,127,0,0,250,63,0,0,250,31,0,0,249,31,0,0,250,15,0,0,248,15,0,0,249,7,0,0,251,3,0,0,252,1,0,0,250,1,0,0,251,0,0,0,124,0,0,0,60,0,0,0,28,0,0,0,12,0,0,0,5,0,0,0,1,0,0,0,0,0,0,0,4,0,0,0,13,0,0,0,29,0,0,0,61,0,0,0,250,0,0,0,252,0,0,0,251,1,0,0,250,3,0,0,248,7,0,0,250,7,0,0,251,7,0,0,249,15,0,0,251,15,0,0,248,31,0,0,251,31,0,0,248,63,0,0,249,63,0,0,241,255,0,0,242,255,0,0,234,255,1,0,235,255,1,0,225,255,3,0,226,255,3,0,234,255,3,0,227,255,3,0,230,255,3,0,231,255,3,0,235,255,3,0,230,255,15,0,226,255,7,0,231,255,15,0,232,255,15,0,233,255,15,0,234,255,15,0,235,255,15,0,236,255,15,0,227,255,7,0,237,255,15,0,238,255,15,0,239,255,15,0,240,255,15,0,228,255,7,0,241,255,15,0,236,255,3,0,242,255,15,0,243,255,15,0,229,255,7,0,230,255,7,0,244,255,15,0,245,255,15,0,246,255,15,0,247,255,15,0,248,255,15,0,249,255,15,0,250,255,15,0,251,255,15,0,252,255,15,0,253,255,15,0,254,255,15,0,255,255,15,0,0,0,0,0,19,19,20,20,20,20,20,20,20,19,20,20,20,20,19,20,19,19,20,18,20,20,20,19,20,20,20,19,20,19,18,19,18,18,17,18,17,17,17,16,16,16,15,15,14,13,13,12,12,11,10,9,9,8,7,6,5,4,3,2,2,3,4,5,6,8,8,9,10,11,11,11,12,12,13,13,14,14,16,16,17,17,18,18,18,18,18,18,18,20,19,20,20,20,20,20,20,19,20,20,20,20,19,20,18,20,20,19,19,20,20,20,20,20,20,20,20,20,20,20,20,0,0,0,0,0,0,0,12,4,4,4,4,4,8,8,8,12,16,20,36,0,0,0,0,0,0,0,0,0,0,0,0,15,4,8,4,8,8,4,8,8,12,12,12,24,28,40,76,0,0,0,0,0,0,0,0,0,17,4,8,8,4,12,12,8,8,24,16,20,24,32,40,60,80,152,0,0,0,0,0,0,0,0,0,0,0,0,11,4,4,8,4,4,12,16,24,20,28,4,0,0,0,0,0,0,0,0,0,0,0,0,0,15,4,8,4,4,8,8,16,20,12,20,20,28,40,56,8,0,0,0,0,0,0,0,0,0,16,8,4,8,8,12,16,20,24,40,32,32,44,56,80,112,16,0,0,0,0,0,0,0,0,0,0,0,0,0,10,4,8,4,8,8,12,20,24,24,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,4,8,8,4,12,12,16,24,16,20,24,32,40,36,0,0,0,0,0,0,0,0,0,0,23,4,4,4,8,4,4,8,8,8,8,8,12,12,16,16,24,24,32,44,48,60,84,72,0,0,0,0,0,0,208,3,25,0,96,0,0,0,0,0,0,0,100,39,52,0,40,0,0,0,44,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,189,172,185,232,52,26,0,128,202,25,0,190,170,184,222,192,125,25,0,160,67,25,0,190,170,184,223,208,251,24,0,160,67,25,0,190,170,185,172,136,220,24,0,112,196,24,0,8,187,186,188,208,174,24,0,152,155,24,0,8,187,186,178,40,140,26,0,48,122,26,0,8,186,177,188,240,102,26,0,64,86,26,0,187,176,185,223,184,69,26,0,216,53,26,0,187,167,182,171,192,38,26,0,168,28,26,0,187,167,171,223,112,15,26,0,216,6,26,0,8,185,182,179,24,0,26,0,136,246,25,0,183,177,187,190,160,234,25,0,72,226,25,0,8,178,170,167,88,217,25,0,96,203,25,0,8,176,175,171,128,191,25,0,56,185,25,0,176,190,168,186,152,177,25,0,232,170,25,0,8,175,173,176,96,163,25,0,176,155,25,0,8,172,171,173,240,147,25,0,88,141,25,0,171,177,180,177,80,134,25,0,72,126,25,0,88,80,77,212,40,119,25,0,176,113,25,0,0,0,0,0,121,207,23,190,138,59,1,66,164,51,148,67,155,200,92,68,202,167,45,70,175,40,132,68,192,222,152,67,129,155,246,65,199,156,118,64,77,183,109,66,194,101,49,68,74,15,165,69,82,45,182,197,71,104,76,196,73,213,153,194,66,4,147,192,94,6,104,63,54,189,72,62,3,97,30,190,44,76,9,66,68,231,150,67,96,102,76,68,47,215,52,70,17,168,147,68,117,204,160,67,46,219,249,65,68,124,109,64,146,154,86,66,183,10,43,68,136,68,163,69,35,243,198,197,129,62,99,196,80,169,179,194,43,42,173,192,1,24,82,63,194,197,199,62,223,144,36,190,144,150,16,66,32,15,152,67,140,47,55,68,113,86,59,70,101,128,162,68,120,164,167,67,193,231,251,65,149,237,87,64,209,237,60,66,46,47,35,68,80,99,160,69,178,232,215,197,240,127,122,196,100,62,207,194,121,91,195,192,207,220,61,63,49,160,20,63,61,91,42,190,177,1,23,66,106,129,151,67,98,254,28,68,14,27,65,70,229,136,176,68,246,95,173,67,75,201,252,65,52,59,74,64,173,80,34,66,178,10,26,68,170,126,156,69,83,240,232,197,121,249,136,196,253,124,236,194,231,48,218,192,193,13,43,63,21,239,67,63,139,188,47,190,75,118,28,66,177,43,149,67,81,195,251,67,92,30,70,70,161,146,189,68,23,254,177,67,116,41,251,65,165,166,58,64,77,48,7,66,62,185,15,68,225,169,151,69,144,236,249,197,102,184,148,196,253,164,5,195,130,12,247,192,196,112,25,63,234,90,113,63,120,177,52,190,11,224,32,66,197,255,144,67,75,169,179,67,9,89,74,70,63,131,201,68,227,108,181,67,12,94,248,65,73,159,52,64,49,233,215,65,148,121,4,68,250,250,145,69,153,95,5,198,224,82,160,196,230,149,21,195,193,75,10,193,185,213,8,63,218,57,142,63,244,54,185,190,93,45,36,66,238,197,138,67,123,163,67,67,193,197,77,70,150,52,212,68,118,180,183,67,208,116,244,65,169,3,34,64,173,143,160,65,68,192,240,67,195,135,139,69,122,165,13,198,28,180,171,196,130,42,38,195,136,83,25,193,112,40,242,62,153,103,162,63,55,74,189,190,167,146,37,66,148,165,130,67,182,247,78,65,135,96,80,70,71,144,221,68,247,225,184,67,182,2,238,65,153,191,25,64,113,224,84,65,226,71,215,67,116,104,132,69,186,183,21,198,32,182,182,196,153,32,55,195,248,124,43,193,205,19,212,62,243,4,181,63,187,232,192,190,91,122,38,66,227,13,113,67,88,242,59,195,65,40,82,70,237,132,229,68,213,190,184,67,201,3,232,65,16,147,4,64,105,242,216,64,110,227,188,67,47,102,121,69,214,134,29,198,81,62,193,196,85,96,72,195,235,212,61,193,80,50,183,62,3,228,197,63,71,16,196,190,73,155,36,66,18,122,88,67,23,20,203,195,140,28,83,70,216,249,235,68,185,166,183,67,247,22,225,65,11,250,244,63,71,16,196,62,69,237,161,67,91,2,105,69,239,4,37,198,124,38,203,196,16,160,89,195,54,63,80,193,66,80,155,62,49,219,212,63,46,15,21,191,242,108,33,66,98,51,60,67,83,17,32,196,220,60,83,70,70,243,240,68,238,104,181,67,38,192,215,65,112,137,223,63,88,12,180,192,157,166,134,67,47,214,87,69,149,32,44,198,6,85,212,196,16,196,106,195,193,157,98,193,212,63,128,62,152,197,225,63,57,182,22,191,234,239,28,66,206,194,27,67,244,79,94,196,226,141,82,70,182,97,244,68,249,56,178,67,221,40,207,65,124,229,200,63,57,233,50,193,16,207,86,67,160,18,70,69,73,205,50,198,21,165,220,196,104,176,123,195,1,246,119,193,175,175,75,62,94,131,236,63,230,143,74,191,36,147,21,66,35,102,239,66,16,227,143,196,201,17,81,70,166,76,246,68,130,2,174,67,22,218,197,65,28,72,177,63,12,95,131,193,224,12,33,67,81,229,51,69,247,251,56,198,140,255,227,196,139,36,134,195,184,137,134,193,100,229,23,62,11,250,244,63,223,202,75,191,201,237,12,66,223,9,160,66,174,0,178,196,45,207,78,70,187,185,246,68,213,254,168,67,51,80,186,65,197,91,178,63,32,204,168,193,139,247,216,66,54,123,33,69,232,158,62,198,230,72,234,196,148,31,142,195,218,232,144,193,220,181,201,61,190,20,251,63,15,177,127,191,152,64,2,66,94,213,19,66,106,66,213,196,38,205,75,70,66,172,245,68,70,55,163,67,112,102,177,65,251,108,153,63,81,248,202,193,231,35,102,66,180,6,15,69,179,170,67,198,226,90,239,196,151,161,149,195,66,6,155,193,60,57,73,61,109,196,254,63,54,211,37,70,68,177,165,69,175,113,104,68,69,51,54,68,128,12,144,67,180,213,129,66,2,0,241,65,34,63,131,64,49,19,72,70,167,49,243,68,86,182,156,67,170,105,166,65,251,100,249,68,112,3,16,65,17,158,233,193,0,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,240,210,71,0,0,0,0,0,0,32,65,0,0,0,0,4,0,0,0,2,0,0,0,3,0,0,0,5,0,0,0,176,124,45,0,176,122,45,0,0,0,0,0,176,106,45,0,0,1,0,1,0,1,0,1,1,0,1,0,1,0,1,0,0,1,0,1,0,1,0,1,1,0,1,0,1,0,1,0,0,1,0,1,0,1,0,1,1,0,1,0,1,0,1,0,0,1,0,1,0,1,0,1,1,0,1,0,1,0,1,0,1,2,1,2,1,2,1,2,3,0,3,0,3,0,3,0,1,2,1,2,1,2,1,2,3,0,3,0,3,0,3,0,1,2,1,2,1,2,1,2,3,0,3,0,3,0,3,0,1,2,1,2,1,2,1,2,3,0,3,0,3,0,3,0,2,4,3,5,2,4,3,5,6,0,7,1,6,0,7,1,3,5,2,4,3,5,2,4,7,1,6,0,7,1,6,0,2,4,3,5,2,4,3,5,6,0,7,1,6,0,7,1,3,5,2,4,3,5,2,4,7,1,6,0,7,1,6,0,4,8,7,11,4,8,7,11,12,0,15,3,12,0,15,3,6,10,5,9,6,10,5,9,14,2,13,1,14,2,13,1,4,8,7,11,4,8,7,11,12,0,15,3,12,0,15,3,6,10,5,9,6,10,5,9,14,2,13,1,14,2,13,1,9,17,15,23,8,16,14,22,25,1,31,7,24,0,30,6,13,21,11,19,12,20,10,18,29,5,27,3,28,4,26,2,8,16,14,22,9,17,15,23,24,0,30,6,25,1,31,7,12,20,10,18,13,21,11,19,28,4,26,2,29,5,27,3,18,34,30,46,17,33,29,45,50,2,62,14,49,1,61,13,26,42,22,38,25,41,21,37,58,10,54,6,57,9,53,5,16,32,28,44,19,35,31,47,48,0,60,12,51,3,63,15,24,40,20,36,27,43,23,39,56,8,52,4,59,11,55,7,18,34,30,46,17,33,29,45,50,2,62,14,49,1,61,13,26,42,22,38,25,41,21,37,58,10,54,6,57,9,53,5,16,32,28,44,19,35,31,47,48,0,60,12,51,3,63,15,24,40,20,36,27,43,23,39,56,8,52,4,59,11,55,7,36,68,60,92,34,66,58,90,100,4,124,28,98,2,122,26,52,84,44,76,50,82,42,74,116,20,108,12,114,18,106,10,32,64,56,88,38,70,62,94,96,0,120,24,102,6,126,30,48,80,40,72,54,86,46,78,112,16,104,8,118,22,110,14,2,0,3,0,3,0,5,0,5,0,5,0,5,0,5,0,5,0,5,0,5,0,5,0,5,0,5,0,5,0,5,0,2,0,3,0,7,0,7,0,13,0,13,0,25,0,25,0,25,0,25,0,25,0,25,0,25,0,25,0,25,0,25,0,3,0,3,0,4,0,15,0,15,0,29,0,57,0,57,0,57,0,113,0,113,0,113,0,113,0,113,0,113,0,113,0,3,0,4,0,4,0,5,0,31,0,31,0,61,0,121,0,241,0,241,0,241,0,241,0,225,1,225,1,225,1,225,1,3,0,4,0,5,0,5,0,6,0,63,0,63,0,125,0,249,0,241,1,225,3,225,3,225,3,225,3,225,3,193,7,3,0,5,0,6,0,6,0,6,0,7,0,127,0,127,0,253,0,249,1,241,3,225,7,193,15,193,15,193,15,193,15,3,0,5,0,6,0,7,0,7,0,7,0,8,0,255,0,255,0,253,1,249,3,241,7,225,15,193,31,129,63,129,63,3,0,5,0,6,0,8,0,8,0,8,0,8,0,9,0,255,1,255,1,253,3,249,7,241,15,225,31,193,63,129,127,3,0,5,0,7,0,8,0,9,0,9,0,9,0,9,0,10,0,255,3,255,3,253,7,249,15,241,31,225,63,193,127,3,0,5,0,7,0,8,0,10,0,10,0,10,0,10,0,10,0,11,0,255,7,255,7,253,15,249,31,241,63,225,127,3,0,5,0,7,0,8,0,10,0,11,0,11,0,11,0,11,0,11,0,12,0,255,15,255,15,253,31,249,63,241,127,3,0,5,0,7,0,9,0,10,0,12,0,12,0,12,0,12,0,12,0,12,0,13,0,255,31,255,31,253,63,249,127,3,0,5,0,7,0,9,0,10,0,12,0,13,0,13,0,13,0,13,0,13,0,13,0,14,0,255,63,255,63,253,127,3,0,5,0,7,0,9,0,10,0,12,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,15,0,255,127,255,127,3,0,5,0,7,0,9,0,11,0,12,0,14,0,15,0,15,0,15,0,15,0,15,0,15,0,15,0,16,0,255,255,0,55,14,68,3,58,17,72,37,18,50,32,40,22,54,35,9,64,5,59,13,67,8,63,46,27,41,23,49,31,44,26,2,57,16,71,1,56,15,70,39,21,52,34,38,19,51,33,11,66,7,62,10,65,6,60,48,30,43,25,47,29,42,24,0,55,14,68,3,58,17,72,17,9,23,15,16,8,22,14,5,29,3,27,4,28,2,26,21,13,19,11,20,12,18,10,0,24,6,30,1,25,7,31,16,8,22,14,17,9,23,15,4,28,2,26,5,29,3,27,20,12,18,10,21,13,19,11,1,25,7,31,0,24,6,30,17,9,23,15,16,8,22,14,117,62,158,103,113,58,155,100,34,199,21,186,31,196,17,182,144,89,131,76,141,86,127,72,0,165,41,206,10,175,52,217,110,55,151,96,120,65,162,107,28,193,14,179,38,203,24,189,138,83,124,69,148,93,134,79,7,172,48,213,3,168,45,210,117,62,158,103,113,58,155,100,36,68,60,92,34,66,58,90,100,4,124,28,98,2,122,26,52,84,44,76,50,82,42,74,116,20,108,12,114,18,106,10,32,64,56,88,38,70,62,94,96,0,120,24,102,6,126,30,48,80,40,72,54,86,46,78,112,16,104,8,118,22,110,14,8,4,11,7,8,4,11,7,2,14,1,13,2,14,1,13,10,6,9,5,10,6,9,5,0,12,3,15,0,12,3,15,8,4,11,7,8,4,11,7,6,2,6,2,6,2,6,2,0,4,0,4,0,4,0,4,6,2,6,2,6,2,6,2,1,3,1,3,1,3,1,3,2,0,2,0,2,0,2,0,1,3,1,3,1,3,1,3,128,2,0,0,224,1,0,0,2,0,0,1,1,0,128,2,224,1,0,0,0,0,1,0,176,0,0,0,120,0,0,0,2,0,0,9,2,0,176,0,120,0,0,0,0,0,1,1,176,0,0,0,144,0,0,0,2,0,1,10,3,0,176,0,144,0,0,0,0,0,1,2,96,1,0,0,240,0,0,0,2,0,0,9,2,0,96,1,240,0,0,0,0,0,1,1,96,1,0,0,32,1,0,0,2,0,1,10,3,0,96,1,32,1,0,0,0,0,1,2,192,2,0,0,224,1,0,0,2,0,0,9,2,0,192,2,224,1,0,0,0,0,1,1,192,2,0,0,64,2,0,0,2,0,1,10,3,0,192,2,64,2,0,0,0,0,1,2,208,2,0,0,224,1,0,0,1,1,0,4,2,0,192,2,224,1,8,0,0,0,3,1,208,2,0,0,64,2,0,0,1,1,1,3,3,0,192,2,64,2,8,0,0,0,3,2,0,5,0,0,208,2,0,0,1,0,1,7,1,0,0,5,208,2,0,0,0,0,3,3,0,5,0,0,208,2,0,0,1,0,1,6,1,0,0,5,208,2,0,0,0,0,3,3,128,7,0,0,56,4,0,0,1,1,1,4,1,0,128,7,56,4,0,0,0,0,3,3,128,7,0,0,56,4,0,0,1,1,1,3,1,0,128,7,56,4,0,0,0,0,3,3,128,7,0,0,56,4,0,0,1,0,1,7,1,0,128,7,56,4,0,0,0,0,3,3,128,7,0,0,56,4,0,0,1,0,1,6,1,0,128,7,56,4,0,0,0,0,3,3,0,8,0,0,56,4,0,0,0,0,1,2,1,0,0,8,56,4,0,0,0,0,4,4,0,16,0,0,112,8,0,0,0,0,1,2,1,0,0,16,112,8,0,0,0,0,4,4,0,15,0,0,112,8,0,0,1,0,1,7,1,0,0,15,112,8,0,0,0,0,3,3,0,15,0,0,112,8,0,0,1,0,1,6,1,0,0,15,112,8,0,0,0,0,3,3,0,30,0,0,224,16,0,0,1,0,1,7,1,0,0,15,112,8,0,0,0,0,3,3,0,30,0,0,224,16,0,0,1,0,1,6,1,0,0,15,112,8,0,0,0,0,3,3,1,0,0,0,6,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,40,0,0,0,33,0,0,0,16,0,0,0,11,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,152,58,0,0,233,3,0,0,25,0,0,0,2,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,6,0,0,0,5,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,184,244,25,0,96,0,0,0,176,196,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,152,136,25,0,0,129,25,0,56,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,41,26,0,96,31,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,192,0,0,129,193,0,0,64,1,0,0,1,195,0,0,192,3,0,0,128,2,0,0,65,194,0,0,1,198,0,0,192,6,0,0,128,7,0,0,65,199,0,0,0,5,0,0,193,197,0,0,129,196,0,0,64,4,0,0,1,204,0,0,192,12,0,0,128,13,0,0,65,205,0,0,0,15,0,0,193,207,0,0,129,206,0,0,64,14,0,0,0,10,0,0,193,202,0,0,129,203,0,0,64,11,0,0,1,201,0,0,192,9,0,0,128,8,0,0,65,200,0,0,1,216,0,0,192,24,0,0,128,25,0,0,65,217,0,0,0,27,0,0,193,219,0,0,129,218,0,0,64,26,0,0,0,30,0,0,193,222,0,0,129,223,0,0,64,31,0,0,1,221,0,0,192,29,0,0,128,28,0,0,65,220,0,0,0,20,0,0,193,212,0,0,129,213,0,0,64,21,0,0,1,215,0,0,192,23,0,0,128,22,0,0,65,214,0,0,1,210,0,0,192,18,0,0,128,19,0,0,65,211,0,0,0,17,0,0,193,209,0,0,129,208,0,0,64,16,0,0,1,240,0,0,192,48,0,0,128,49,0,0,65,241,0,0,0,51,0,0,193,243,0,0,129,242,0,0,64,50,0,0,0,54,0,0,193,246,0,0,129,247,0,0,64,55,0,0,1,245,0,0,192,53,0,0,128,52,0,0,65,244,0,0,0,60,0,0,193,252,0,0,129,253,0,0,64,61,0,0,1,255,0,0,192,63,0,0,128,62,0,0,65,254,0,0,1,250,0,0,192,58,0,0,128,59,0,0,65,251,0,0,0,57,0,0,193,249,0,0,129,248,0,0,64,56,0,0,0,40,0,0,193,232,0,0,129,233,0,0,64,41,0,0,1,235,0,0,192,43,0,0,128,42,0,0,65,234,0,0,1,238,0,0,192,46,0,0,128,47,0,0,65,239,0,0,0,45,0,0,193,237,0,0,129,236,0,0,64,44,0,0,1,228,0,0,192,36,0,0,128,37,0,0,65,229,0,0,0,39,0,0,193,231,0,0,129,230,0,0,64,38,0,0,0,34,0,0,193,226,0,0,129,227,0,0,64,35,0,0,1,225,0,0,192,33,0,0,128,32,0,0,65,224,0,0,1,160,0,0,192,96,0,0,128,97,0,0,65,161,0,0,0,99,0,0,193,163,0,0,129,162,0,0,64,98,0,0,0,102,0,0,193,166,0,0,129,167,0,0,64,103,0,0,1,165,0,0,192,101,0,0,128,100,0,0,65,164,0,0,0,108,0,0,193,172,0,0,129,173,0,0,64,109,0,0,1,175,0,0,192,111,0,0,128,110,0,0,65,174,0,0,1,170,0,0,192,106,0,0,128,107,0,0,65,171,0,0,0,105,0,0,193,169,0,0,129,168,0,0,64,104,0,0,0,120,0,0,193,184,0,0,129,185,0,0,64,121,0,0,1,187,0,0,192,123,0,0,128,122,0,0,65,186,0,0,1,190,0,0,192,126,0,0,128,127,0,0,65,191,0,0,0,125,0,0,193,189,0,0,129,188,0,0,64,124,0,0,1,180,0,0,192,116,0,0,128,117,0,0,65,181,0,0,0,119,0,0,193,183,0,0,129,182,0,0,64,118,0,0,0,114,0,0,193,178,0,0,129,179,0,0,64,115,0,0,1,177,0,0,192,113,0,0,128,112,0,0,65,176,0,0,0,80,0,0,193,144,0,0,129,145,0,0,64,81,0,0,1,147,0,0,192,83,0,0,128,82,0,0,65,146,0,0,1,150,0,0,192,86,0,0,128,87,0,0,65,151,0,0,0,85,0,0,193,149,0,0,129,148,0,0,64,84,0,0,1,156,0,0,192,92,0,0,128,93,0,0,65,157,0,0,0,95,0,0,193,159,0,0,129,158,0,0,64,94,0,0,0,90,0,0,193,154,0,0,129,155,0,0,64,91,0,0,1,153,0,0,192,89,0,0,128,88,0,0,65,152,0,0,1,136,0,0,192,72,0,0,128,73,0,0,65,137,0,0,0,75,0,0,193,139,0,0,129,138,0,0,64,74,0,0,0,78,0,0,193,142,0,0,129,143,0,0,64,79,0,0,1,141,0,0,192,77,0,0,128,76,0,0,65,140,0,0,0,68,0,0,193,132,0,0,129,133,0,0,64,69,0,0,1,135,0,0,192,71,0,0,128,70,0,0,65,134,0,0,1,130,0,0,192,66,0,0,128,67,0,0,65,131,0,0,0,65,0,0,193,129,0,0,129,128,0,0,64,64,0,0,6,0,0,0,46,0,0,0,44,0,0,0,44,0,0,0,30,0,0,0,30,0,0,0,30,0,0,0,30,0,0,0,30,0,0,0,30,0,0,0,30,0,0,0,30,0,0,0,30,0,0,0,30,0,0,0,30,0,0,0,30,0,0,0,94,131,108,63,21,239,195,62,109,196,126,63,54,189,200,61,67,236,127,63,176,10,201,60,196,254,127,63,136,15,201,59,255,255,255,255,0,0,0,0,184,26,25,0,0,0,0,0,105,87,20,139,10,191,5,64,184,19,25,0,0,0,0,0,24,45,68,84,251,33,9,64,240,9,25,0,0,0,0,0,168,244,151,155,119,227,249,63,24,45,68,84,251,33,9,64,105,87,20,139,10,191,5,64,0,0,0,0,0,128,93,64,0,0,0,0,0,0,0,0,24,190,24,0,168,184,24,0,104,181,24,0,0,0,0,0,0,0,0,0,0,0,0,0,72,53,26,0,240,248,255,0,72,38,26,0,250,235,215,0,72,28,26,0,0,255,255,0,56,15,26,0,127,255,212,0,152,6,26,0,240,255,255,0,184,255,25,0,245,245,220,0,80,246,25,0,255,228,196,0,88,234,25,0,0,0,0,0,16,226,25,0,255,235,205,0,64,217,25,0,0,0,255,0,48,203,25,0,138,43,226,0,48,191,25,0,165,42,42,0,24,185,25,0,222,184,135,0,88,177,25,0,95,158,160,0,136,170,25,0,127,255,0,0,32,163,25,0,210,105,30,0,64,155,25,0,255,127,80,0,192,147,25,0,100,149,237,0,56,141,25,0,255,248,220,0,56,134,25,0,220,20,60,0,24,126,25,0,0,255,255,0,240,118,25,0,0,0,139,0,120,113,25,0,0,139,139,0,216,108,25,0,184,134,11,0,40,103,25,0,169,169,169,0,224,97,25,0,0,100,0,0,192,91,25,0,189,183,107,0,80,86,25,0,139,0,139,0,128,81,25,0,85,107,47,0,200,76,25,0,255,140,0,0,48,68,25,0,153,50,204,0,144,61,25,0,139,0,0,0,168,55,25,0,233,150,122,0,136,48,25,0,143,188,143,0,128,42,25,0,72,61,139,0,64,32,25,0,47,79,79,0,136,26,25,0,0,206,209,0,88,19,25,0,148,0,211,0,72,9,25,0,255,20,147,0,112,3,25,0,0,191,255,0,120,252,24,0,105,105,105,0,104,245,24,0,30,144,255,0,104,243,24,0,178,34,34,0,208,240,24,0,255,250,240,0,184,237,24,0,34,139,34,0,248,235,24,0,255,0,255,0,8,232,24,0,220,220,220,0,152,228,24,0,248,248,255,0,128,226,24,0,255,215,0,0,88,224,24,0,218,165,32,0,208,220,24,0,128,128,128,0,24,218,24,0,0,128,0,0,104,216,24,0,173,255,47,0,136,214,24,0,240,255,240,0,96,212,24,0,255,105,180,0,72,210,24,0,205,92,92,0,184,206,24,0,75,0,130,0,136,203,24,0,255,255,240,0,80,201,24,0,240,230,140,0,160,199,24,0,230,230,250,0,176,196,24,0,255,240,245,0,88,194,24,0,124,252,0,0,192,192,24,0,255,250,205,0,112,191,24,0,173,216,230,0,8,190,24,0,240,128,128,0,152,184,24,0,224,255,255,0,72,181,24,0,250,250,210,0,216,179,24,0,144,238,144,0,88,178,24,0,211,211,211,0,200,176,24,0,255,182,193,0,248,174,24,0,255,160,122,0,248,172,24,0,32,178,170,0,192,171,24,0,135,206,250,0,112,170,24,0,119,136,153,0,8,169,24,0,176,196,222,0,56,168,24,0,255,255,224,0,112,165,24,0,0,255,0,0,240,162,24,0,50,205,50,0,208,160,24,0,250,240,230,0,192,158,24,0,255,0,255,0,192,155,24,0,128,0,0,0,80,154,24,0,102,205,170,0,40,153,24,0,0,0,205,0,208,151,24,0,186,85,211,0,0,151,24,0,147,112,216,0,88,160,26,0,60,179,113,0,88,157,26,0,123,104,238,0,24,155,26,0,0,250,154,0,48,149,26,0,72,209,204,0,32,143,26,0,199,21,133,0,80,140,26,0,25,25,112,0,88,138,26,0,245,255,250,0,80,137,26,0,255,228,225,0,64,136,26,0,255,228,181,0,80,135,26,0,255,222,173,0,232,133,26,0,0,0,128,0,8,132,26,0,253,245,230,0,224,129,26,0,128,128,0,0,48,128,26,0,107,142,35,0,208,125,26,0,255,165,0,0,80,122,26,0,255,69,0,0,216,120,26,0,218,112,214,0,200,119,26,0,238,232,170,0,176,118,26,0,152,251,152,0,224,116,26,0,175,238,238,0,0,116,26,0,216,112,147,0,216,113,26,0,255,239,213,0,208,111,26,0,255,218,185,0,200,108,26,0,205,133,63,0,128,106,26,0,255,192,203,0,24,103,26,0,221,160,221,0,248,101,26,0,176,224,230,0,184,100,26,0,128,0,128,0,120,99,26,0,255,0,0,0,200,97,26,0,188,143,143,0,24,97,26,0,65,105,225,0,232,93,26,0,139,69,19,0,208,91,26,0,250,128,114,0,56,90,26,0,244,164,96,0,8,89,26,0,46,139,87,0,104,86,26,0,255,245,238,0,128,84,26,0,160,82,45,0,120,83,26,0,192,192,192,0,120,82,26,0,135,206,235,0,160,80,26,0,106,90,205,0,208,79,26,0,112,128,144,0,80,77,26,0,255,250,250,0,104,75,26,0,0,255,127,0,80,74,26,0,70,130,180,0,200,72,26,0,210,180,140,0,192,69,26,0,0,128,128,0,224,68,26,0,216,191,216,0,24,68,26,0,255,99,71,0,16,67,26,0,64,224,208,0,40,66,26,0,238,130,238,0,48,65,26,0,245,222,179,0,160,63,26,0,255,255,255,0,24,61,26,0,245,245,245,0,152,59,26,0,255,255,0,0,200,56,26,0,154,205,50,0,65,196,52,0,65,208,0,0,17,196,0,0,3,226,0,0,9,253,0,0,2,40,0,0,2,34,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,250,0,0,21,219,0,0,5,201,0,0,21,213,0,0,5,207,0,0,22,51,0,0,6,39,0,0,18,155,0,0,20,192,0,0,20,153,0,0,20,147,0,0,0,0,0,0,154,2,0,0,60,0,0,0,208,88,4,0,56,99,4,0,232,4,2,0,43,2,0,0,40,0,0,0,240,77,4,0,160,86,4,0,152,4,2,0,56,5,0,0,84,1,0,0,216,51,4,0,184,72,4,0,240,1,2,0,48,4,0,0,180,0,0,0,232,30,4,0,168,47,4,0,136,0,2,0,220,1,0,0,70,0,0,0,152,21,4,0,8,29,4,0,248,255,1,0,179,1,0,0,40,0,0,0,16,13,4,0,224,19,4,0,168,255,1,0,71,3,0,0,11,0,0,0,1,0,0,0,1,0,0,0,12,0,0,0,4,0,0,0,16,0,0,0,21,0,0,0,31,0,0,0,11,0,0,0,35,0,0,0,38,0,0,0,41,0,0,0,53,0,0,0,55,0,0,0,1,0,0,0,21,0,0,0,26,0,0,0,29,0,0,0,28,0,0,0,30,0,0,0,78,0,0,0,73,0,0,0,81,0,0,0,120,0,0,0,4,0,0,0,0,0,0,0,8,0,0,0,13,0,0,0,123,0,0,0,5,0,0,0,50,0,0,0,149,0,0,0,145,0,0,0,150,0,0,0,161,0,0,0,217,0,0,0,3,0,0,0,25,0,0,0,97,0,0,0,102,0,0,0,96,0,0,0,23,0,0,0,14,0,0,0,99,0,0,0,160,1,0,0,183,1,0,0,230,1,0,0,231,1,0,0,182,1,0,0,24,0,0,0,232,1,0,0,56,0,0,0,49,0,0,0,5,0,0,0,61,0,0,0,39,0,0,0,234,1,0,0,26,0,0,0,197,0,0,0,249,0,0,0,255,0,0,0,219,0,0,0,80,2,0,0,252,0,0,0,92,2,0,0,8,0,0,0,117,0,0,0,215,3,0,0,211,3,0,0,176,1,0,0,124,0,0,0,202,3,0,0,54,0,0,0,137,1,0,0,166,4,0,0,162,4,0,0,251,4,0,0,192,0,0,0,127,0,0,0,154,0,0,0,17,3,0,0,110,0,0,0,155,0,0,0,140,6,0,0,192,6,0,0,132,4,0,0,18,0,0,0,195,0,0,0,79,9,0,0,121,9,0,0,249,9,0,0,9,13,0,0,166,13,0,0,168,13,0,0,1,9,0,0,193,0,0,0,115,3,0,0,8,13,0,0,250,9,0,0,139,13,0,0,133,13,0,0,134,13,0,0,223,0,0,0,226,6,0,0,206,0,0,0,36,15,0,0,254,9,0,0,247,1,0,0,193,7,0,0,207,0,0,0,252,9,0,0,255,9,0,0,137,13,0,0,169,13,0,0,253,9,0,0,248,1,0,0,54,26,0,0,140,18,0,0,157,18,0,0,55,26,0,0,150,1,0,0,234,3,0,0,139,15,0,0,147,13,0,0,69,30,0,0,88,30,0,0,75,30,0,0,89,30,0,0,241,19,0,0,9,3,0,0,101,2,0,0,8,3,0,0,58,36,0,0,225,39,0,0,137,15,0,0,36,3,0,0,188,60,0,0,134,60,0,0,149,54,0,0,60,36,0,0,59,36,0,0,62,36,0,0,74,30,0,0,165,3,0,0,104,52,0,0,40,52,0,0,132,60,0,0,224,39,0,0,226,37,0,0,128,24,0,0,151,1,0,0,37,3,0,0,183,60,0,0,30,121,0,0,236,7,0,0,117,108,0,0,200,4,0,0,199,75,0,0,198,4,0,0,131,9,0,0,30,72,0,0,83,27,0,0,27,37,0,0,88,27,0,0,132,9,0,0,168,79,0,0,187,60,0,0,138,15,0,0,34,3,0,0,106,52,0,0,61,36,0,0,38,3,0,0,105,52,0,0,31,72,0,0,29,72,0,0,70,7,0,0,50,144,0,0,80,27,0,0,19,29,0,0,228,216,0,0,27,72,0,0,116,108,0,0,107,121,0,0,105,121,0,0,133,9,0,0,227,216,0,0,134,9,0,0,162,15,0,0,1,19,0,0])
.concat([124,108,0,0,135,9,0,0,184,60,0,0,175,244,0,0,136,14,0,0,192,177,1,0,206,15,0,0,235,51,0,0,106,63,0,0,105,63,0,0,207,15,0,0,31,121,0,0,201,4,0,0,113,72,0,0,205,15,0,0,130,9,0,0,204,15,0,0,163,15,0,0,18,29,0,0,108,121,0,0,71,27,0,0,33,3,0,0,106,121,0,0,226,216,0,0,114,72,0,0,115,72,0,0,14,0,0,0,20,0,0,0,10,0,0,0,160,0,0,0,18,0,0,0,125,0,0,0,162,1,0,0,59,0,0,0,95,2,0,0,221,0,0,0,124,2,0,0,67,3,0,0,104,3,0,0,107,3,0,0,62,0,0,0,250,1,0,0,133,4,0,0,179,1,0,0,127,0,0,0,177,1,0,0,158,1,0,0,186,4,0,0,173,7,0,0,57,3,0,0,102,0,0,0,164,7,0,0,147,7,0,0,198,6,0,0,126,0,0,0,241,0,0,0,114,3,0,0,251,9,0,0,131,13,0,0,138,13,0,0,71,9,0,0,244,9,0,0,208,1,0,0,9,27,0,0,75,27,0,0,236,7,0,0,225,3,0,0,202,0,0,0,236,3,0,0,57,37,0,0,169,79,0,0,87,27,0,0,41,52,0,0,42,61,0,0,151,13,0,0,167,3,0,0,192,13,0,0,150,13,0,0,193,13,0,0,235,7,0,0,186,60,0,0,67,12,0,0,65,12,0,0,82,27,0,0,239,7,0,0,35,3,0,0,185,60,0,0,131,60,0,0,208,7,0,0,237,7,0,0,127,108,0,0,51,144,0,0,108,63,0,0,131,99,3,0,93,233,1,0,120,108,0,0,71,7,0,0,81,27,0,0,34,0,0,0,22,0,0,0,57,0,0,0,82,2,0,0,121,0,0,0,134,4,0,0,56,3,0,0,105,3,0,0,136,13,0,0,38,0,0,0,135,13,0,0,75,15,0,0,130,13,0,0,39,0,0,0,225,1,0,0,21,26,0,0,199,7,0,0,240,18,0,0,224,1,0,0,208,6,0,0,22,26,0,0,68,30,0,0,95,30,0,0,144,54,0,0,144,13,0,0,66,12,0,0,175,13,0,0,146,13,0,0,128,15,0,0,251,12,0,0,47,52,0,0,127,72,0,0,70,27,0,0,104,121,0,0,149,13,0,0,145,13,0,0,85,27,0,0,104,63,0,0,198,75,0,0,189,60,0,0,129,15,0,0,32,3,0,0,105,0,0,0,254,0,0,0,213,6,0,0,63,3,0,0,222,0,0,0,198,7,0,0,64,30,0,0,148,13,0,0,136,15,0,0,142,60,0,0,148,54,0,0,174,13,0,0,173,13,0,0,103,2,0,0,166,3,0,0,39,3,0,0,126,72,0,0,238,7,0,0,73,7,0,0,199,4,0,0,146,54,0,0,86,27,0,0,209,15,0,0,86,122,0,0,119,108,0,0,49,144,0,0,72,7,0,0,122,108,0,0,109,121,0,0,234,51,0,0,118,108,0,0,208,15,0,0,130,99,3,0,23,228,1,0,69,7,0,0,175,79,0,0,225,216,0,0,107,63,0,0,92,233,1,0,173,79,0,0,158,0,0,0,189,4,0,0,124,6,0,0,8,27,0,0,235,3,0,0,69,27,0,0,145,54,0,0,229,216,0,0,4,121,0,0,129,9,0,0,234,7,0,0,244,25,0,0,125,108,0,0,171,79,0,0,172,79,0,0,126,108,0,0,0,19,0,0,123,108,0,0,111,0,0,0,247,3,0,0,133,60,0,0,196,4,0,0,30,0,0,0,225,6,0,0,147,54,0,0,68,27,0,0,65,2,0,0,70,30,0,0,157,1,0,0,102,2,0,0,187,4,0,0,56,37,0,0,172,7,0,0,84,27,0,0,2,9,0,0,112,72,0,0,167,13,0,0,0,9,0,0,133,1,0,0,121,108,0,0,227,6,0,0,233,3,0,0,148,30,0,0,237,3,0,0,242,3,0,0,46,52,0,0,107,52,0,0,26,37,0,0,197,4,0,0,129,24,0,0,28,72,0,0,89,27,0,0,135,60,0,0,174,79,0,0,233,7,0,0,109,63,0,0,10,242,0,0,48,144,0,0,170,79,0,0,230,216,0,0,111,63,0,0,26,72,0,0,110,63,0,0,22,228,1,0,231,216,0,0,0,0,0,0,10,4,2,4,4,5,5,5,5,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,8,8,8,8,7,8,8,8,8,8,8,8,9,9,9,9,9,9,9,9,9,9,9,9,9,10,9,10,10,10,10,10,9,10,10,10,10,10,10,10,10,11,11,10,10,11,11,10,11,11,11,11,11,12,12,12,12,12,12,11,11,11,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,13,13,13,12,12,13,13,13,12,12,12,12,12,13,13,13,13,13,14,14,14,14,13,13,13,13,13,14,14,14,14,14,14,15,14,14,14,14,14,14,13,14,14,14,14,14,14,15,14,15,14,15,15,15,15,15,15,16,15,15,14,15,16,15,14,14,15,14,14,15,14,15,15,15,16,15,17,16,15,15,15,15,16,16,16,16,17,15,16,14,16,16,17,16,16,16,16,16,15,15,15,16,16,16,16,17,15,15,15,15,16,15,15,4,7,8,8,9,9,9,10,10,10,10,10,10,10,11,11,11,11,11,11,11,11,11,12,12,11,11,11,12,12,12,12,12,12,12,12,13,13,13,13,12,13,14,14,15,15,14,14,14,14,14,14,14,15,14,14,14,15,15,15,14,14,15,15,15,16,16,18,17,15,15,15,6,9,10,10,11,11,12,12,12,13,12,12,12,13,13,13,13,13,13,13,13,13,13,14,14,14,14,14,14,14,14,15,15,15,14,14,15,16,15,14,14,15,7,10,11,12,13,13,13,14,14,14,14,14,14,14,14,15,15,15,15,15,14,15,16,15,15,16,15,15,15,16,15,16,18,17,15,15,16,16,17,15,8,11,13,13,14,15,14,16,15,16,15,15,15,15,15,15,17,15,9,12,14,15,10,13,14,15,10,13,11,14,11,14,11,15,12,15,12,12,13,15,13,14,13,14,14,14,14,14,15,15,15,15,14,15,15,16,16,16,15,16,16,15,16,17,16,0,0,0,0,0,1,15,0,0,30,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,13,0,0,0,10,0,0,0,23,0,0,0,29,0,0,0,23,0,0,0,44,0,0,0,49,0,0,0,57,0,0,0,62,0,0,0,57,0,0,0,90,0,0,0,102,0,0,0,112,0,0,0,123,0,0,0,112,0,0,0,119,0,0,0,175,0,0,0,201,0,0,0,242,0,0,0,244,0,0,0,178,0,0,0,227,0,0,0,91,1,0,0,93,1,0,0,129,1,0,0,157,1,0,0,227,1,0,0,197,1,0,0,181,2,0,0,219,2,0,0,56,3,0,0,195,3,0,0,204,3,0,0,240,3,0,0,205,2,0,0,250,3,0,0,161,3,0,0,180,5,0,0,87,6,0,0,171,7,0,0,77,7,0,0,76,7,0,0,193,10,0,0,197,10,0,0,107,7,0,0,168,12,0,0,4,15,0,0,0,15,0,0,227,15,0,0,60,15,0,0,16,15,0,0,57,15,0,0,230,15,0,0,38,14,0,0,144,14,0,0,197,22,0,0,39,24,0,0,84,25,0,0,197,21,0,0,88,25,0,0,138,31,0,0,74,28,0,0,15,43,0,0,65,43,0,0,14,43,0,0,198,51,0,0,80,48,0,0,79,28,0,0,136,45,0,0,92,48,0,0,24,60,0,0,79,43,0,0,194,44,0,0,71,58,0,0,128,86,0,0,157,86,0,0,66,100,0,0,67,100,0,0,70,100,0,0,110,101,0,0,68,100,0,0,32,113,0,0,138,116,0,0,186,193,0,0,34,126,0,0,166,122,0,0,37,127,0,0,167,122,0,0,32,126,0,0,27,193,0,0,24,193,0,0,165,122,0,0,10,173,0,0,137,243,0,0,187,158,1,0,173,202,0,0,66,254,0,0,64,254,0,0,52,108,1,0,229,180,2,0,101,61,3,0,48,108,1,0,174,231,1,0,92,226,1,0,112,131,1,0,3,231,1,0,186,158,1,0,55,108,1,0,52,226,0,0,110,108,1,0,4,0,0,0,42,0,0,0,97,0,0,0,117,0,0,0,203,0,0,0,255,0,0,0,144,1,0,0,235,1,0,0,209,1,0,0,185,2,0,0,7,3,0,0,57,3,0,0,63,3,0,0,251,3,0,0,180,3,0,0,12,6,0,0,121,6,0,0,69,6,0,0,125,6,0,0,138,7,0,0,227,7,0,0,73,7,0,0,196,10,0,0,210,10,0,0,227,10,0,0,16,12,0,0,22,12,0,0,209,10,0,0,244,12,0,0,226,15,0,0,134,21,0,0,157,14,0,0,241,25,0,0,100,22,0,0,38,30,0,0,56,29,0,0,77,43,0,0,197,51,0,0,194,31,0,0,195,31,0,0,40,29,0,0,29,60,0,0,142,89,0,0,148,240,0,0,164,122,0,0,56,173,0,0,12,172,0,0,26,193,0,0,234,121,0,0,129,200,0,0,68,254,0,0,53,182,0,0,13,172,0,0,30,182,0,0,135,89,0,0,33,113,0,0,130,243,0,0,135,243,0,0,55,226,0,0,71,254,0,0,131,243,0,0,145,240,0,0,133,243,0,0,51,226,0,0,238,130,1,0,184,158,1,0,62,102,1,0,147,240,0,0,20,0,0,0,88,0,0,0,89,1,0,0,103,1,0,0,0,3,0,0,212,3,0,0,181,5,0,0,157,7,0,0,106,7,0,0,103,11,0,0,96,11,0,0,5,15,0,0,240,12,0,0,23,15,0,0,149,14,0,0,34,24,0,0,19,25,0,0,194,22,0,0,47,24,0,0,89,25,0,0,203,31,0,0,39,30,0,0,64,28,0,0,199,51,0,0,123,30,0,0,73,28,0,0,137,45,0,0,35,30,0,0,96,22,0,0,18,63,0,0,198,44,0,0,225,51,0,0,52,91,0,0,154,96,0,0,105,101,0,0,136,116,0,0,33,126,0,0,95,207,0,0,44,113,0,0,157,56,0,0,207,103,0,0,40,127,0,0,63,102,1,0,103,61,3,0,61,102,1,0,93,226,1,0,171,193,3,0,68,92,1,0,54,108,1,0,31,0,0,0,236,0,0,0,35,3,0,0,178,5,0,0,159,7,0,0,194,10,0,0,22,15,0,0,158,14,0,0,86,25,0,0,15,30,0,0,234,25,0,0,102,22,0,0,137,43,0,0,2,43,0,0,140,45,0,0,27,60,0,0,25,60,0,0,181,50,0,0,156,63,0,0,207,44,0,0,151,56,0,0,53,91,0,0,2,173,0,0,41,127,0,0,65,100,0,0,132,56,0,0,136,120,0,0,78,120,0,0,104,101,0,0,187,193,0,0,134,89,0,0,204,103,0,0,73,254,0,0,72,254,0,0,188,193,0,0,65,254,0,0,113,131,1,0,60,102,1,0,49,226,0,0,30,113,0,0,9,173,0,0,146,240,0,0,45,0,0,0,219,1,0,0,129,7,0,0,26,12,0,0,85,15,0,0,128,21,0,0,168,30,0,0,155,45,0,0,175,50,0,0,22,63,0,0,28,60,0,0,52,120,0,0,69,60,0,0,156,56,0,0,206,103,0,0,69,100,0,0,185,193,0,0,137,120,0,0,58,127,0,0,79,120,0,0,43,127,0,0,11,173,0,0,144,240,0,0,29,193,0,0,78,233,0,0,31,113,0,0,241,233,0,0,142,243,0,0,233,121,0,0,3,173,0,0,155,240,0,0,174,202,0,0,70,254,0,0,230,180,2,0,240,233,0,0,182,158,1,0,193,122,6,0,192,122,6,0,102,61,3,0,136,243,0,0,113,0,0,0,160,3,0,0,169,12,0,0,41,24,0,0,57,29,0,0,67,43,0,0,196,44,0,0,84,101,0,0,154,240,0,0,31,182,0,0,205,103,0,0,28,113,0,0,54,182,0,0,42,127,0,0,52,182,0,0,31,193,0,0,94,207,0,0,29,182,0,0,107,240,0,0,171,202,0,0,190,193,0,0,76,233,0,0,153,240,0,0,237,130,1,0,79,233,0,0,25,193,0,0,50,226,0,0,228,180,2,0,138,243,0,0,180,158,1,0,95,226,1,0,77,233,0,0,183,0,0,0,133,7,0,0,204,22,0,0,81,48,0,0,196,51,0,0,111,101,0,0,145,56,0,0,29,113,0,0,175,202,0,0,151,240,0,0,137,116,0,0,152,240,0,0,128,200,0,0,170,202,0,0,134,243,0,0,183,158,1,0,111,108,1,0,132,243,0,0,232,130,1,0,233,130,1,0,48,226,0,0,0,231,1,0,98,61,3,0,99,61,3,0,100,61,3,0,51,108,1,0,22,226,0,0,253,0,0,0,21,12,0,0,101,22,0,0,74,60,0,0,59,127,0,0,150,120,0,0,28,193,0,0,21,226,0,0,50,108,1,0,139,243,0,0,141,243,0,0,234,130,1,0,1,231,1,0,223,18,7,0,70,92,1,0,148,1,0,0,224,15,0,0,19,63,0,0,139,116,0,0,150,240,0,0,128,207,0,0,94,226,1,0,189,37,14,0,97,61,3,0,49,108,1,0,249,1,0,0,18,25,0,0,16,87,0,0,208,243,0,0,191,193,0,0,1,3,0,0,36,30,0,0,8,173,0,0,205,3,0,0,65,28,0,0,189,193,0,0,99,5,0,0,82,58,0,0,209,243,0,0,112,5,0,0,206,44,0,0,23,226,0,0,123,6,0,0,93,101,0,0,75,7,0,0,71,100,0,0,18,12,0,0,251,116,0,0,8,15,0,0,28,182,0,0,34,14,0,0,67,254,0,0,199,22,0,0,54,24,0,0,242,25,0,0,67,28,0,0,63,29,0,0,207,31,0,0,76,43,0,0,76,48,0,0,182,50,0,0,70,58,0,0,7,86,0,0,23,63,0,0,197,44,0,0,155,96,0,0,92,101,0,0,35,126,0,0,193,103,0,0,38,127,0,0,39,127,0,0,149,240,0,0,243,233,0,0,129,207,0,0,30,193,0,0,172,202,0,0,143,243,0,0,242,233,0,0,250,116,0,0,54,226,0,0,69,254,0,0,40,196,1,0,53,226,0,0,239,130,1,0,181,158,1,0,214,243,0,0,236,130,1,0,53,108,1,0,140,243,0,0,231,180,2,0,71,92,1,0,188,37,14,0,2,231,1,0,182,196,1,0,90,226,0,0,170,193,3,0,69,92,1,0,41,196,1,0,185,158,1,0,175,231,1,0,235,130,1,0,212,224,1,0,110,137,3,0,12,6,2,3,4,4,5,5,5,6,6,6,6,6,7,7,7,7,7,8,8,8,8,8,8,9,9,9,9,9,9,9,10,10,10,10,10,10,10,11,10,11,11,11,11,12,12,12,12,12,12,12,12,12,12,12,12,12,13,13,13,13,13,13,13,13,14,14,14,14,14,14,14,14,14,14,14,15,15,15,15,15,15,15,15,15,16,16,16,15,15,15,15,15,16,16,15,16,16,17,16,16,16,17,18,18,17,17,17,17,17,17,17,17,17,4,6,7,8,8,8,9,9,10,10,10,10,10,10,11,11,11,11,11,11,11,12,12,12,12,12,12,12,12,12,13,13,13,14,13,14,14,14,13,13,14,14,16,16,15,16,16,16,15,16,16,16,16,16,16,16,16,16,17,16,16,16,16,17,17,17,18,16,5,8,9,10,10,10,11,11,12,12,12,12,12,12,13,13,13,13,13,13,13,13,14,14,13,14,14,13,14,14,15,14,15,15,15,16,15,16,16,15,15,15,18,18,18,17,18,17,17,6,9,10,11,11,12,12,13,13,13,13,14,14,14,14,14,14,14,14,15,15,15,16,15,15,15,15,15,15,16,16,15,16,16,16,16,17,18,17,16,16,16,7,10,11,12,12,13,13,14,14,14,14,15,14,15,15,15,16,15,15,15,15,16,16,16,17,16,17,16,15,16,16,16,16,18,17,17,19,19,18,16,7,11,12,13,14,14,15,15,16,16,15,16,16,15,16,16,16,16,16,16,16,17,16,17,17,16,17,18,16,17,17,17,8,11,13,14,14,15,15,16,16,16,16,16,16,16,16,17,17,16,17,17,17,17,18,18,18,17,17,8,12,14,14,15,15,16,17,17,16,16,17,17,20,17,9,12,14,16,16,16,17,21,18,17,9,13,15,16,16,10,13,16,10,14,16,11,15,16,11,15,17,11,15,12,15,12,16,12,16,13,16,13,13,13,14,14,13,14,14,14,15,15,14,15,15,15,15,15,15,15,16,17,16,16,16,16,17,16,17,16,18,17,17,17,16,17,17,16,18,17,21,17,18,17,18,17,18,17,17,17,17,19,0,0,0,0,178,1,0,0,105,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,14,0,0,0,20,0,0,0,25,0,0,0,22,0,0,0,43,0,0,0,48,0,0,0,61,0,0,0,60,0,0,0,90,0,0,0,95,0,0,0,109,0,0,0,126,0,0,0,95,0,0,0,127,0,0,0,182,0,0,0,188,0,0,0,216,0,0,0,242,0,0,0,254,0,0,0,188,0,0,0,252,0,0,0,97,1,0,0,110,1,0,0,116,1,0,0,118,1,0,0,162,1,0,0,227,1,0,0,243,1,0,0,116,1,0,0,122,1,0,0,234,1,0,0,168,2,0,0,196,2,0,0,230,2,0,0,20,3,0,0,70,3,0,0,103,3,0,0,233,3,0,0,229,2,0,0,238,2,0,0,214,3,0,0,85,5,0,0,84,5,0,0,87,5,0,0,195,5,0,0,214,5,0,0,224,6,0,0,47,6,0,0,226,6,0,0,153,7,0,0,137,7,0,0,250,7,0,0,206,5,0,0,254,7,0,0,236,5,0,0,204,7,0,0,175,7,0,0,167,10,0,0,25,11,0,0,148,11,0,0,133,11,0,0,159,11,0,0,72,12,0,0,69,12,0,0,216,13,0,0,76,12,0,0,75,12,0,0,153,13,0,0,31,13,0,0,194,13,0,0,149,15,0,0,162,15,0,0,181,11,0,0,159,11,0,0,93,15,0,0,191,11,0,0,71,15,0,0,74,21,0,0,213,15,0,0,69,15,0,0,127,15,0,0,13,22,0,0,137,24,0,0,87,23,0,0,34,23,0,0,179,24,0,0,45,23,0,0,57,26,0,0,24,26,0,0,179,27,0,0,48,27,0,0,99,30,0,0,60,23,0,0,53,27,0,0,35,23,0,0,128,30,0,0,238,31,0,0,97,23,0,0,252,31,0,0,127,31,0,0,124,44,0,0,161,31,0,0,123,23,0,0,85,23,0,0,90,23,0,0,166,31,0,0,171,46,0,0,10,49,0,0,105,44,0,0,105,54,0,0,39,49,0,0,3,49,0,0,67,46,0,0,98,54,0,0,101,49,0,0,36,49,0,0,59,49,0,0,17,49,0,0,104,54,0,0,59,52,0,0,82,60,0,0,252,62,0,0,108,46,0,0,218,63,0,0,248,62,0,0,123,46,0,0,226,62,0,0,197,60,0,0,114,61,0,0,192,88,0,0,248,61,0,0,169,46,0,0,126,62,0,0,109,85,0,0,130,92,0,0,113,61,0,0,123,62,0,0,66,60,0,0,215,88,0,0,78,63,0,0,0,98,0,0,112,61,0,0,178,92,0,0,150,92,0,0,176,92,0,0,69,63,0,0,177,92,0,0,109,46,0,0,16,49,0,0,104,47,0,0,144,92,0,0,166,124,0,0,136,124,0,0,4,98,0,0,200,98,0,0,166,120,0,0,134,121,0,0,213,121,0,0,173,177,0,0,137,121,0,0,121,176,0,0,221,92,0,0,212,170,0,0,232,93,0,0,205,125,0,0,135,121,0,0,103,93,0,0,153,93,0,0,29,185,0,0,241,124,0,0,155,93,0,0,215,121,0,0,123,176,0,0,133,92,0,0,154,93,0,0,204,125,0,0,191,126,0,0,206,125,0,0,251,125,0,0,192,126,0,0,26,125,0,0,7,122,0,0,132,92,0,0,113,196,0,0,242,124,0,0,239,186,0,0,210,185,0,0,235,93,0,0,214,123,0,0,69,184,0,0,152,93,0,0,26,185,0,0,232,186,0,0,224,196,0,0,49,220,0,0,61,249,0,0,206,187,0,0,210,209,0,0,169,247,0,0,185,217,0,0,203,187,0,0,0,185,0,0,215,170,0,0,189,186,0,0,225,196,0,0,111,244,0,0,136,197,0,0,139,197,0,0,230,96,1,0,207,187,0,0,195,186,0,0,69,249,0,0,163,247,0,0,193,209,0,0,142,251,0,0,164,247,0,0,140,251,0,0,12,244,0,0,115,196,0,0,114,253,0,0,205,187,0,0,250,255,0,0,64,249,0,0,201,187,0,0,168,247,0,0,237,161,1,0,197,187,0,0,111,242,1,0,253,99,1,0,199,96,1,0,245,161,1,0,71,249,0,0,252,99,1,0,179,84,1,0,246,255,0,0,246,99,1,0,233,96,1,0,240,161,1,0,185,186,0,0,186,186,0,0,134,112,1,0,3,185,0,0,117,253,0,0,8,243,0,0,243,118,1,0,255,99,1,0,125,253,0,0,120,187,1,0,251,99,1,0,219,136,1,0,247,161,1,0,178,84,1,0,253,114,1,0,244,99,1,0,115,187,1,0,255,114,1,0,188,186,0,0,125,249,0,0,243,161,1,0,109,187,1,0,213,255,1,0,244,161,1,0,114,242,1,0,128,115,1,0,130,115,1,0,231,255,1,0,200,186,0,0,196,187,0,0,211,136,1,0,224,96,1,0,123,253,0,0,95,114,1,0,245,114,1,0,121,187,1,0,217,250,1,0,105,242,1,0,208,136,1,0,196,186,0,0,197,186,0,0,133,17,3,0,210,136,1,0,204,136,1,0,135,17,3,0,254,231,3,0,209,136,1,0,108,187,1,0,104,242,1,0,210,250,1,0,217,255,1,0,234,161,1,0,104,187,1,0,203,250,1,0,178,253,3,0,26,232,1,0,206,136,1,0,251,114,1,0,239,161,1,0,206,250,1,0,112,187,1,0,193,186,0,0,107,187,1,0,248,114,1,0,102,187,1,0,223,255,1,0,106,187,1,0,215,255,1,0,102,242,1,0,248,118,1,0,83,118,3,0,126,250,1,0,130,17,3,0,200,250,1,0,227,199,2,0,238,112,3,0,236,118,1,0,233,118,1,0,188,228,2,0,197,96,1,0,90,118,3,0,156,206,3,0,115,115,1,0,232,118,1,0,212,136,1,0,241,118,1,0,239,118,1,0,89,118,3,0,124,187,1,0,222,255,1,0,242,118,1,0,139,17,3,0,212,199,2,0,81,118,3,0,159,206,5,0,80,118,3,0,145,17,3,0,246,244,3,0,245,244,3,0,108,160,7,0,193,250,1,0,123,201,5,0,224,199,2,0,58,157,7,0,253,231,3,0,223,199,2,0,240,244,3,0,109,160,7,0,193,118,3,0,59,157,7,0,4,0,0,0,20,0,0,0,89,0,0,0,171,0,0,0,184,0,0,0,119,1,0,0,245,1,0,0,242,1,0,0,21,3,0,0,252,3,0,0,189,5,0,0,45,6,0,0,232,6,0,0,221,7,0,0,4,11,0,0,205,7,0,0,30,11,0,0,30,13,0,0,21,15,0,0,59,15,0,0,65,15,0,0,72,21,0,0,176,24,0,0,59,23,0,0,132,24,0,0,28,26,0,0,180,27,0,0,37,31,0,0,181,23,0,0,109,23,0,0,248,30,0,0,115,46,0,0,7,49,0,0,37,49,0,0,5,49,0,0,73,46,0,0,232,60,0,0,249,62,0,0,94,62,0,0,114,46,0,0,113,52,0,0,217,63,0,0,63,98,0,0,160,120,0,0,103,104,0,0,179,92,0,0,114,98,0,0,236,104,0,0,154,110,0,0,212,121,0,0,152,110,0,0,170,177,0,0,26,110,0,0,133,121,0,0,238,104,0,0,155,110,0,0,136,92,0,0,172,177,0,0,250,125,0,0,101,93,0,0,240,124,0,0,191,124,0,0,117,196,0,0,235,96,1,0,126,187,1,0,166,247,0,0,221,254,1,0,227,96,1,0,251,255,0,0,141,251,0,0,249,255,0,0,192,209,0,0,140,197,0,0,233,161,1,0,184,186,0,0,207,245,0,0,245,255,0,0,197,118,3,0,236,161,1,0,237,96,1,0,222,254,1,0,201,250,1,0,235,161,1,0,36,242,1,0,238,118,1,0,121,253,0,0,128,112,1,0,135,115,1,0,122,187,1,0,233,255,1,0,247,118,1,0,133,115,1,0,129,119,1,0,213,199,2,0,133,119,1,0,227,255,1,0,245,99,1,0,194,250,1,0,249,231,3,0,141,17,3,0,177,253,3,0,226,255,1,0,38,242,1,0,138,17,3,0,217,199,2,0,144,17,3,0,140,17,3,0,243,244,3,0,127,187,1,0,114,187,1,0,132,17,3,0,244,146,11,0,251,231,3,0,217,225,6,0,202,250,1,0,0,35,6,0,184,253,3,0,55,208,3,0,252,231,3,0,1,35,6,0,242,244,3,0,106,242,1,0,14,0,0,0,99,0,0,0,248,0,0,0,238,1,0,0,119,3,0,0,247,3,0,0,227,6,0,0,204,5,0,0,5,11,0,0,210,13,0,0,212,15,0,0,46,23,0,0,42,23,0,0,35,30,0,0,45,31,0,0,99,23,0,0,105,23,0,0,108,23,0,0,117,46,0,0,4,49,0,0,193,46,0,0,88,62,0,0,63,88,0,0,98,63,0,0,68,63,0,0,197,88,0,0,60,98,0,0,244,92,0,0,215,123,0,0,157,93,0,0,210,170,0,0,102,93,0,0,169,177,0,0,120,176,0,0,254,124,0,0,24,185,0,0,111,196,0,0,25,185,0,0,71,184,0,0,27,110,0,0,75,184,0,0,216,170,0,0,116,253,0,0,244,114,1,0,129,112,1,0,124,249,0,0,115,242,1,0,160,247,0,0,124,253,0,0,247,114,1,0,122,253,0,0,119,187,1,0,254,114,1,0,112,242,1,0,115,253,0,0,123,187,1,0,188,161,1,0,125,187,1,0,195,187,0,0,246,114,1,0,235,186,0,0,143,251,0,0,244,244,3,0,180,253,3,0,200,118,3,0,250,231,3,0,208,255,1,0,3,35,6,0,245,146,11,0,97,242,1,0,137,17,3,0,181,253,3,0,219,199,2,0,201,118,3,0,214,250,1,0,209,250,1,0,21,0,0,0,240,0,0,0,224,2,0,0,142,5,0,0,215,5,0,0,77,12,0,0,161,15,0,0,219,11,0,0,86,23,0,0,112,31,0,0,25,44,0,0,60,49,0,0,15,55,0,0,192,60,0,0,168,46,0,0,198,88,0,0,199,88,0,0,183,46,0,0,208,88,0,0,24,125,0,0,88,170,0,0,72,184,0,0,158,93,0,0,108,93,0,0,76,184,0,0,137,197,0,0,1,185,0,0,248,99,1,0,201,186,0,0,197,185,0,0,60,249,0,0,216,136,1,0,199,187,0,0,236,96,1,0,111,253,0,0,217,136,1,0,234,96,1,0,167,247,0,0,68,249,0,0,171,186,0,0,58,220,0,0,207,136,1,0,251,118,1,0,216,199,2,0,215,199,2,0,117,187,1,0,158,206,5,0,2,35,6,0,237,112,3,0,244,118,1,0,209,255,1,0,239,112,3,0,248,244,3,0,199,118,3,0,225,255,1,0,198,118,3,0,255,118,1,0,216,225,6,0,246,118,1,0,135,112,1,0,205,245,0,0,53,0,0,0,160,1,0,0,139,5,0,0,172,10,0,0,154,11,0,0,95,23,0,0,34,30,0,0,140,30,0,0,178,31,0,0,11,49,0,0,209,88,0,0,46,85,0,0,39,92,0,0,110,104,0,0,167,124,0,0,116,196,0,0,51,220,0,0,242,123,0,0,233,93,0,0,53,122,0,0,170,186,0,0,235,185,0,0,149,251,0,0,184,185,0,0,129,115,1,0,98,242,1,0,205,136,1,0,136,112,1,0,250,114,1,0,162,247,0,0,211,250,1,0,192,186,0,0,92,118,3,0,223,254,1,0,37,242,1,0,212,250,1,0,218,199,2,0,157,206,5,0,248,231,3,0,3,226,1,0,215,136,1,0,84,0,0,0,192,2,0,0,161,7,0,0,120,15,0,0,54,27,0,0,163,31,0,0,58,49,0,0,54,52,0,0,58,52,0,0,29,125,0,0,216,123,0,0,223,92,0,0,70,184,0,0,137,177,0,0,184,217,0,0,248,255,0,0,190,217,0,0,138,197,0,0,234,93,0,0,211,209,0,0,228,96,1,0,107,242,1,0,218,136,1,0,2,226,1,0,210,199,2,0,254,99,1,0,147,17,3,0,130,119,1,0,194,118,3,0,209,199,2,0,176,253,3,0,93,118,3,0,208,199,2,0,208,250,1,0,1,226,1,0,221,136,1,0,226,199,2,0,87,118,3,0,85,118,3,0,196,118,3,0,192,118,3,0,234,118,1,0,111,0,0,0,207,3,0,0,213,13,0,0,35,31,0,0,97,44,0,0,208,46,0,0,84,93,0,0,45,85,0,0,131,120,0,0,168,177,0,0,28,185,0,0,191,186,0,0,2,185,0,0,170,247,0,0,165,247,0,0,232,161,1,0,214,255,1,0,190,186,0,0,191,161,1,0,243,99,1,0,216,255,1,0,215,250,1,0,117,242,1,0,220,255,1,0,125,0,0,0,188,5,0,0,73,21,0,0,153,42,0,0,239,61,0,0,115,98,0,0,214,121,0,0,27,125,0,0,211,170,0,0,252,208,0,0,221,199,2,0,214,136,1,0,194,186,0,0,225,199,2,0,118,187,1,0,189,161,1,0,134,17,3,0,120,253,0,0,190,161,1,0,131,17,3,0,182,253,3,0,241,244,3,0,82,118,3,0,213,250,1,0,249,244,3,0,255,231,3,0,156,206,5,0,91,118,3,0,136,17,3,0,114,115,1,0,189,0,0,0,139,7,0,0,33,31,0,0,67,60,0,0,237,61,0,0,214,170,0,0,193,126,0,0,66,249,0,0,134,92,0,0,137,112,1,0,187,186,0,0,232,255,1,0,222,199,2,0,110,242,1,0,196,250,1,0,247,244,3,0,86,118,3,0,125,250,1,0,195,118,3,0,179,253,3,0,143,17,3,0,198,250,1,0,248,0,0,0,237,7,0,0,253,30,0,0,122,62,0,0,145,92,0,0,217,170,0,0,236,186,0,0,50,220,0,0,110,244,0,0,0,226,1,0,250,118,1,0,94,118,3,0,183,253,3,0,214,199,2,0,185,253,3,0,84,118,3,0,88,118,3,0,142,17,3,0,219,255,1,0,246,0,0,0,67,12,0,0,6,49,0,0,239,104,0,0,77,184,0,0,136,177,0,0,204,187,0,0,100,242,1,0,105,187,1,0,134,115,1,0,192,250,1,0,113,1,0,0,57,15,0,0,65,62,0,0,237,104,0,0,188,217,0,0,161,247,0,0,103,187,1,0,221,255,1,0,249,118,1,0,185,1,0,0,125,15,0,0,99,63,0,0,253,208,0,0,234,185,0,0,220,136,1,0,195,250,1,0,242,161,1,0,146,17,3,0,228,255,1,0,246,1,0,0,84,23,0,0,101,104,0,0,9,243,0,0,229,96,1,0,245,118,1,0,95,118,3,0,204,250,1,0,233,1,0,0,26,26,0,0,1,98,0,0,5,241,0,0,240,118,1,0,223,2,0,0,86,23,0,0,109,93,0,0,250,99,1,0,237,118,1,0,66,3,0,0,64,46,0,0,255,208,0,0,130,112,1,0,205,3,0,0,152,42,0,0,252,255,0,0,220,199,2,0,127,250,1,0,254,3,0,0,100,55,0,0,253,255,0,0,252,118,1,0,197,250,1,0,247,2,0,0,209,46,0,0,151,251,0,0,138,5,0,0,220,46,0,0,200,187,0,0,212,5,0,0,61,98,0,0,232,96,1,0,46,6,0,0,48,88,0,0,249,99,1,0,235,6,0,0,5,98,0,0,116,242,1,0,222,7,0,0,201,98,0,0,101,242,1,0,201,5,0,0,222,92,0,0,211,255,1,0,212,5,0,0,136,121,0,0,206,7,0,0,73,184,0,0,27,11,0,0,137,92,0,0,199,250,1,0,147,11,0,0,131,92,0,0,158,11,0,0,79,241,0,0,74,12,0,0,199,185,0,0,212,13,0,0,112,196,0,0,113,242,1,0,56,15,0,0,150,251,0,0,235,118,1,0,160,15,0,0,247,99,1,0,178,11,0,0,27,185,0,0,190,11,0,0,2,241,0,0,68,15,0,0,70,249,0,0,205,250,1,0,121,15,0,0,189,217,0,0,77,21,0,0,198,187,0,0,210,15,0,0,231,96,1,0,43,23,0,0,203,136,1,0,94,23,0,0,118,253,0,0,92,23,0,0,113,187,1,0,159,24,0,0,238,161,1,0,36,31,0,0,246,161,1,0,167,27,0,0,202,187,0,0,125,31,0,0,255,255,0,0,46,31,0,0,101,187,1,0,181,27,0,0,249,114,1,0,239,31,0,0,108,242,1,0,62,31,0,0,119,253,0,0,98,23,0,0,110,187,1,0,249,30,0,0,252,114,1,0,160,31,0,0,183,42,0,0,74,46,0,0,103,242,1,0,179,31,0,0,218,255,1,0,66,46,0,0,1,49,0,0,128,119,1,0,61,49,0,0,117,52,0,0,132,119,1,0,38,49,0,0,207,250,1,0,81,60,0,0,131,119,1,0,64,62,0,0,229,255,1,0,99,54,0,0,224,255,1,0,143,62,0,0,109,242,1,0,60,52,0,0,193,60,0,0,253,118,1,0,69,62,0,0,192,46,0,0,97,63,0,0,238,61,0,0,216,63,0,0,62,88,0,0,69,46,0,0,89,62,0,0,2,61,0,0,232,92,0,0,104,85,0,0,254,118,1,0,105,47,0,0,216,250,1,0,193,88,0,0,131,92,0,0,230,255,1,0,113,98,0,0,28,110,0,0,199,98,0,0,225,104,0,0,47,85,0,0,100,104,0,0,102,104,0,0,153,110,0,0,188,92,0,0,165,124,0,0,161,120,0,0,130,92,0,0,207,125,0,0,59,98,0,0,62,98,0,0,232,104,0,0,54,122,0,0,156,93,0,0,119,176,0,0,243,124,0,0,52,122,0,0,164,124,0,0,25,125,0,0,210,121,0,0,28,125,0,0,217,123,0,0,74,184,0,0,148,251,0,0,213,170,0,0,48,220,0,0,243,123,0,0,238,186,0,0,122,176,0,0,114,196,0,0,30,185,0,0,186,217,0,0,159,93,0,0,254,208,0,0,198,185,0,0,135,92,0,0,78,241,0,0,237,186,0,0,46,185,0,0,3,241,0,0,196,185,0,0,145,251,0,0,187,217,0,0,171,177,0,0,141,197,0,0,254,255,0,0,59,249,0,0,65,249,0,0,234,186,0,0,31,185,0,0,204,245,0,0,191,217,0,0,67,249,0,0,4,241,0,0,96,242,1,0,146,251,0,0,63,249,0,0,166,243,0,0,199,186,0,0,171,247,0,0,198,186,0,0,131,115,1,0,109,253,0,0,233,186,0,0,110,253,0,0,79,231,1,0,202,136,1,0,39,242,1,0,147,251,0,0,144,251,0,0,247,255,0,0,133,112,1,0,131,112,1,0,225,96,1,0,132,112,1,0,62,249,0,0,226,96,1,0,198,96,1,0,241,161,1,0,111,187,1,0,132,115,1,0,112,253,0,0,99,242,1,0,213,136,1,0,166,115,1,0,206,245,0,0,242,99,1,0,113,253,0,0,210,255,1,0,196,96,1,0,212,255,1,0,211,199,2,0,116,187,1,0,9,7,2,3,4,4,5,5,6,6,6,6,7,7,7,7,7,8,8,8,8,8,8,8,9,9,9,9,9,9,9,9,9,10,10,10,10,10,10,10,10,10,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,12,11,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,14,13,14,14,13,14,13,13,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,15,14,14,15,14,14,15,15,15,15,15,15,15,15,15,14,15,15,15,15,15,15,15,15,15,15,14,15,15,15,15,15,15,15,15,15,16,15,16,16,16,16,15,15,16,16,16,16,16,15,16,16,16,15,16,15,15,16,15,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,17,16,17,16,17,17,16,17,16,17,16,16,17,17,17,16,17,16,16,17,16,17,16,17,17,16,16,17,17,17,17,17,17,17,17,16,17,17,16,17,17,17,17,17,17,17,17,16,18,17,17,17,17,17,17,17,17,17,17,17,17,16,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,18,17,17,17,17,18,17,17,18,19,17,17,17,18,17,17,17,18,18,18,17,17,17,18,17,17,17,17,17,17,17,17,17,18,18,18,18,18,18,18,18,18,18,17,18,18,18,18,17,18,18,18,17,17,18,18,18,18,19,18,18,19,19,20,18,19,18,19,19,18,19,20,18,19,4,6,7,8,9,9,9,10,10,10,11,11,11,11,12,12,12,12,12,12,13,13,13,13,13,13,13,13,14,14,14,14,14,14,14,14,14,14,14,14,14,14,15,15,15,15,15,15,15,15,15,16,15,15,15,15,16,16,15,16,16,15,16,17,17,17,17,17,16,16,16,16,16,17,17,17,16,18,17,17,17,18,17,17,18,17,17,17,17,17,18,17,18,18,18,17,17,18,19,18,18,17,17,18,18,18,18,19,17,17,18,20,19,19,18,19,18,19,19,19,19,17,5,7,9,10,10,11,11,12,12,12,13,13,13,13,13,14,14,14,14,14,15,14,15,15,15,15,15,16,16,16,16,16,16,16,16,16,16,16,16,15,16,16,17,17,17,16,17,17,17,17,17,17,17,17,17,17,17,17,17,17,16,16,19,18,18,19,17,19,20,17,18,18,18,18,18,18,6,8,10,11,12,12,12,13,13,13,14,14,14,14,15,15,15,15,15,15,16,16,16,16,16,16,17,17,17,16,16,17,17,17,17,17,17,17,16,16,16,17,18,18,18,17,19,19,18,18,17,18,19,18,17,18,18,19,18,17,17,6,9,11,12,13,13,13,14,14,14,15,15,15,15,15,16,16,16,16,16,16,17,16,17,17,17,17,17,17,17,18,17,18,17,17,18,18,19,19,17,17,7,10,12,13,13,14,14,14,14,15,16,16,16,16,16,16,16,16,16,16,17,17,17,17,18,17,18,18,18,18,18,18,18,18,17,17,18,18,18,18,18,18,7,10,12,13,14,15,15,15,15,16,16,17,17,17,17,17,17,17,17,17,17,18,17,17,8,11,13,14,15,15,15,15,16,16,18,17,17,18,17,17,18,17,17,18,18,19,18,18,19,19,19,18,18,18,8,11,13,14,15,16,16,16,16,17,17,17,18,17,18,19,18,18,18,18,18,18,8,12,14,15,15,16,16,16,17,17,18,18,18,18,18,18,18,18,17,9,12,14,15,16,16,17,17,17,17,18,9,12,14,15,16,17,17,17,18,9,13,15,16,17,17,18,17,18,17,9,13,15,16,17,18,18,18,10,13,15,16,18,10,14,16,17,18,10,14,16,17,10,14,16,18,18,10,14,16,18,18,11,15,16,11,15,17,11,15,17,11,15,17,11,15,17,11,15,17,12,16,17,12,15,12,16,12,16,18,12,16,12,16,12,16,12,16,17,12,16,18,12,17,13,16,13,16,13,16,18,13,16,13,17,13,17,13,17,13,17,13,17,13,17,13,17,13,17,13,16,13,17,13,17,13,17,14,17,14,17,14,17,14,14,14,17,14,17,14,14,18,14,14,18,14,18,14,18,14,17,14,17,14,17,14,14,18,14,15,15,15,14,15,15,14,15,15,15,18,15,18,15,15,17,15,15,15,15,15,15,15,15,15,15,15,16,15,15,15,15,16,16,16,16,16,15,15,15,15,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,17,16,16,16,17,16,16,16,17,17,17,17,17,16,17,17,17,17,16,16,16,17,17,17,17,16,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,18,17,230,3,0,0,246,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,15,0,0,0,27,0,0,0,40,0,0,0,57,0,0,0,63,0,0,0,107,0,0,0,118,0,0,0,183,0,0,0,232,0,0,0,239,0,0,0,105,1,0,0,167,1,0,0,212,1,0,0,220,1,0,0,196,2,0,0,73,3,0,0,85,3,0,0,145,3,0,0,220,3,0,0,129,5,0,0,178,5,0,0,152,6,0,0,12,7,0,0,85,7,0,0,58,7,0,0,116,7,0,0,207,7,0,0,10,11,0,0,102,11,0,0,46,13,0,0,94,13,0,0,27,14,0,0,172,14,0,0,90,14,0,0,126,15,0,0,161,15,0,0,62,22,0,0,55,26,0,0,82,26,0,0,57,28,0,0,179,26,0,0,95,29,0,0,182,28,0,0,82,31,0,0,217,29,0,0,4,44,0,0,46,44,0,0,45,44,0,0,35,44,0,0,103,52,0,0,163,52,0,0,27,53,0,0,1,53,0,0,93,58,0,0,28,53,0,0,117,56,0,0,234,61,0,0,123,57,0,0,219,57,0,0,241,61,0,0,216,57,0,0,180,59,0,0,10,88,0,0,77,88,0,0,66,88,0,0,19,91,0,0,234,88,0,0,125,105,0,0,6,106,0,0,204,104,0,0,199,106,0,0,150,106,0,0,244,114,0,0,67,117,0,0,180,114,0,0,32,125,0,0,3,176,0,0,181,115,0,0,230,123,0,0,128,209,0,0,209,123,0,0,184,124,0,0,6,125,0,0,37,125,0,0,242,210,0,0,154,209,0,0,52,211,0,0,220,225,0,0,41,213,0,0,132,213,0,0,210,225,0,0,227,229,0,0,196,238,0,0,100,229,0,0,73,250,0,0,1,96,1,0,220,238,0,0,250,247,0,0,44,163,1,0,49,97,1,0,3,96,1,0,200,249,0,0,128,239,1,0,160,210,1,0,75,170,1,0,206,247,0,0,254,171,1,0,80,170,1,0,88,164,1,0,22,168,1,0,228,202,1,0,254,210,1,0,46,213,1,0,76,170,1,0,69,194,2,0,161,210,1,0,93,163,1,0,27,202,1,0,216,213,1,0,49,245,1,0,28,202,1,0,137,243,1,0,175,244,1,0,231,165,3,0,251,81,3,0,75,194,2,0,206,75,3,0,77,194,2,0,73,194,2,0,74,194,2,0,252,45,7,0,239,87,3,0,2,80,3,0,230,165,3,0,49,148,3,0,59,132,5,0,119,74,3,0,49,132,5,0,243,165,3,0,221,165,3,0,229,229,3,0,189,86,3,0,110,151,3,0,210,163,6,0,13,80,3,0,196,148,6,0,189,128,5,0,232,229,3,0,149,75,7,0,110,74,3,0,124,151,3,0,50,148,3,0,210,176,5,0,216,163,6,0,184,128,5,0,203,176,5,0,215,176,5,0,238,45,7,0,237,45,7,0,236,45,7,0,156,75,7,0,127,151,3,0,234,45,7,0,158,75,7,0,125,190,7,0,191,128,5,0,213,176,5,0,168,203,7,0,145,75,7,0,221,229,3,0,113,97,11,0,179,70,13,0,185,70,13,0,161,203,7,0,159,75,7,0,225,45,7,0,245,89,14,0,235,229,3,0,4,0,0,0,21,0,0,0,56,0,0,0,117,0,0,0,232,0,0,0,211,1,0,0,71,3,0,0,156,3,0,0,144,6,0,0,74,7,0,0,96,11,0,0,147,14,0,0,116,15,0,0,61,22,0,0,90,26,0,0,36,29,0,0,190,28,0,0,75,31,0,0,104,52,0,0,98,53,0,0,71,57,0,0,130,62,0,0,4,88,0,0,18,91,0,0,3,88,0,0,109,105,0,0,158,106,0,0,124,105,0,0,120,105,0,0,251,106,0,0,178,116,0,0,245,114,0,0,192,115,0,0,65,117,0,0,68,105,0,0,183,116,0,0,211,112,0,0,169,123,0,0,177,176,0,0,175,209,0,0,221,225,0,0,226,229,0,0,163,225,0,0,195,238,0,0,47,97,1,0,97,233,0,0,218,238,0,0,142,231,0,0,72,250,0,0,44,97,1,0,17,229,0,0,101,229,0,0,83,233,0,0,74,170,1,0,157,229,0,0,44,213,1,0,17,168,1,0,231,202,1,0,252,171,1,0,45,213,1,0,207,202,1,0,5,207,1,0,84,194,2,0,114,74,3,0,172,244,1,0,107,151,3,0,113,74,3,0,217,198,2,0,115,216,2,0,106,74,3,0,231,87,3,0,76,70,3,0,245,229,3,0,51,132,5,0,58,245,1,0,10,80,3,0,234,87,3,0,115,74,3,0,47,148,3,0,229,87,3,0,117,151,3,0,205,148,6,0,114,151,3,0,165,203,7,0,239,163,6,0])
.concat([131,84,3,0,152,75,7,0,193,176,5,0,112,151,3,0,215,165,3,0,51,148,3,0,52,148,3,0,206,148,6,0,190,128,5,0,255,229,3,0,236,163,6,0,111,97,11,0,177,70,13,0,209,163,6,0,229,45,7,0,110,75,7,0,233,45,7,0,0,231,3,0,182,70,13,0,233,163,6,0,105,75,7,0,117,86,14,0,184,70,13,0,170,203,7,0,209,165,3,0,12,0,0,0,60,0,0,0,235,0,0,0,241,1,0,0,164,3,0,0,168,6,0,0,213,7,0,0,67,13,0,0,119,14,0,0,197,22,0,0,177,28,0,0,93,44,0,0,85,58,0,0,86,58,0,0,81,62,0,0,181,59,0,0,10,91,0,0,159,106,0,0,184,116,0,0,40,125,0,0,135,209,0,0,14,212,0,0,46,213,0,0,37,212,0,0,227,234,0,0,211,225,0,0,46,97,1,0,158,229,0,0,194,238,0,0,120,229,0,0,26,229,0,0,121,229,0,0,21,229,0,0,96,233,0,0,131,209,0,0,32,210,0,0,203,210,0,0,18,229,0,0,62,108,1,0,2,96,1,0,66,108,1,0,233,202,1,0,26,70,3,0,250,210,1,0,8,163,1,0,73,168,1,0,7,207,1,0,143,243,1,0,101,75,3,0,83,194,2,0,158,239,1,0,195,203,1,0,193,203,1,0,85,194,2,0,132,243,1,0,53,132,5,0,205,197,2,0,247,165,3,0,82,194,2,0,156,149,3,0,216,198,2,0,211,165,3,0,120,173,6,0,242,163,6,0,169,203,7,0,118,97,11,0,235,45,7,0,100,151,3,0,246,229,3,0,216,165,3,0,140,74,7,0,230,163,6,0,209,148,6,0,121,173,6,0,146,69,26,0,251,89,14,0,179,203,7,0,205,176,5,0,23,0,0,0,181,0,0,0,195,2,0,0,183,5,0,0,28,11,0,0,92,14,0,0,63,22,0,0,178,26,0,0,250,30,0,0,138,52,0,0,110,57,0,0,218,88,0,0,99,105,0,0,48,106,0,0,205,114,0,0,207,115,0,0,231,124,0,0,202,210,0,0,216,210,0,0,100,231,0,0,148,231,0,0,8,96,1,0,103,97,1,0,126,97,1,0,73,170,1,0,11,163,1,0,19,168,1,0,218,198,2,0,128,165,1,0,194,203,1,0,202,249,0,0,127,97,1,0,254,210,1,0,252,247,0,0,64,108,1,0,19,229,0,0,197,238,0,0,195,247,0,0,8,213,1,0,30,168,1,0,253,210,1,0,48,148,3,0,134,84,3,0,253,229,3,0,76,194,2,0,90,199,2,0,116,74,3,0,244,165,3,0,77,70,3,0,202,148,6,0,241,165,3,0,9,213,1,0,192,213,1,0,72,70,3,0,78,70,3,0,213,163,6,0,232,163,6,0,231,163,6,0,195,176,5,0,72,194,2,0,138,243,1,0,242,165,3,0,229,163,6,0,41,0,0,0,104,1,0,0,140,5,0,0,103,11,0,0,157,15,0,0,61,28,0,0,191,28,0,0,32,44,0,0,29,53,0,0,246,61,0,0,249,106,0,0,181,114,0,0,215,177,0,0,178,176,0,0,10,212,0,0,43,213,0,0,82,233,0,0,151,231,0,0,195,99,1,0,160,195,1,0,134,243,1,0,33,202,1,0,85,70,3,0,71,194,2,0,59,245,1,0,80,194,2,0,79,194,2,0,133,243,1,0,93,239,1,0,21,207,1,0,234,202,1,0,10,171,1,0,25,207,1,0,61,245,1,0,194,213,1,0,251,210,1,0,88,239,1,0,120,74,3,0,236,87,3,0,51,245,1,0,225,165,3,0,210,148,6,0,130,132,5,0,238,165,3,0,220,198,2,0,235,87,3,0,196,176,5,0,120,151,3,0,225,163,6,0,180,203,7,0,225,165,3,0,104,75,7,0,239,165,3,0,210,165,3,0,36,148,3,0,226,45,7,0,246,89,14,0,247,89,14,0,2,231,3,0,236,229,3,0,139,243,1,0,59,0,0,0,240,1,0,0,119,7,0,0,168,15,0,0,178,28,0,0,132,45,0,0,87,58,0,0,214,61,0,0,23,105,0,0,17,106,0,0,7,125,0,0,226,234,0,0,150,231,0,0,201,249,0,0,251,247,0,0,102,97,1,0,96,97,1,0,27,171,1,0,250,171,1,0,123,216,2,0,247,210,1,0,104,151,3,0,140,243,1,0,83,70,3,0,81,70,3,0,217,163,6,0,1,80,3,0,189,171,3,0,66,135,3,0,38,148,3,0,118,74,3,0,236,165,3,0,117,74,3,0,0,80,3,0,136,84,3,0,16,207,1,0,219,198,2,0,237,87,3,0,232,87,3,0,233,87,3,0,240,165,3,0,194,148,6,0,120,97,11,0,245,45,7,0,37,148,3,0,43,148,3,0,109,75,7,0,111,75,7,0,119,97,11,0,121,97,11,0,106,75,7,0,114,97,11,0,135,132,5,0,238,229,3,0,237,229,3,0,242,45,7,0,244,45,7,0,174,203,7,0,202,163,6,0,134,14,7,0,207,75,3,0,200,163,6,0,89,0,0,0,132,3,0,0,91,13,0,0,56,28,0,0,96,53,0,0,91,57,0,0,78,88,0,0,100,105,0,0,205,115,0,0,231,177,0,0,152,231,0,0,141,231,0,0,67,250,0,0,72,168,1,0,47,163,1,0,78,170,1,0,74,70,3,0,171,244,1,0,141,243,1,0,235,165,3,0,212,165,3,0,138,84,3,0,199,163,6,0,208,176,5,0,197,163,6,0,176,203,7,0,203,148,6,0,229,165,3,0,226,229,3,0,44,148,3,0,114,216,2,0,174,244,1,0,213,165,3,0,211,148,6,0,129,132,5,0,9,80,3,0,116,151,3,0,50,132,5,0,108,97,11,0,219,176,5,0,139,84,3,0,116,97,11,0,149,93,29,0,76,0,11,0,178,203,7,0,229,165,3,0,143,74,7,0,249,89,14,0,246,45,7,0,253,89,14,0,173,203,7,0,125,66,13,0,255,44,7,0,122,151,3,0,217,176,5,0,109,97,11,0,107,97,11,0,147,69,26,0,175,203,7,0,218,176,5,0,113,0,0,0,235,3,0,0,3,22,0,0,108,44,0,0,97,57,0,0,200,104,0,0,49,106,0,0,189,114,0,0,194,210,0,0,27,229,0,0,230,229,0,0,251,171,1,0,255,210,1,0,229,202,1,0,92,239,1,0,94,239,1,0,19,207,1,0,109,74,3,0,109,151,3,0,106,97,11,0,242,229,3,0,196,163,6,0,105,97,11,0,220,229,3,0,185,128,5,0,153,75,7,0,100,87,7,0,52,132,5,0,217,165,3,0,90,148,6,0,89,148,6,0,140,84,3,0,233,165,3,0,87,148,6,0,241,45,7,0,94,148,6,0,94,163,6,0,1,231,3,0,104,97,11,0,221,176,5,0,222,165,3,0,194,163,6,0,120,66,13,0,204,163,6,0,253,45,7,0,101,97,11,0,154,0,22,0,177,203,7,0,124,66,13,0,98,97,11,0,94,118,14,0,190,236,28,0,182,203,7,0,84,148,6,0,96,97,11,0,122,66,13,0,150,93,29,0,109,29,11,0,244,89,14,0,232,45,7,0,219,165,3,0,122,0,0,0,174,6,0,0,60,28,0,0,186,58,0,0,233,88,0,0,204,114,0,0,221,210,0,0,45,210,0,0,193,238,0,0,219,238,0,0,162,210,1,0,91,239,1,0,226,87,3,0,191,171,3,0,249,210,1,0,4,80,3,0,220,165,3,0,252,81,3,0,108,151,3,0,198,163,6,0,203,163,6,0,234,229,3,0,243,89,14,0,206,163,6,0,82,148,6,0,240,89,14,0,144,75,7,0,121,66,13,0,123,66,13,0,181,203,7,0,197,176,5,0,227,165,3,0,226,165,3,0,208,0,0,0,117,7,0,0,254,30,0,0,213,61,0,0,140,114,0,0,185,124,0,0,162,225,0,0,133,234,0,0,216,238,0,0,10,163,1,0,79,170,1,0,223,165,3,0,8,80,3,0,224,165,3,0,244,229,3,0,247,229,3,0,108,29,11,0,62,132,5,0,112,74,3,0,248,45,7,0,107,75,7,0,127,66,13,0,240,45,7,0,191,176,5,0,192,176,5,0,176,70,13,0,239,45,7,0,248,89,14,0,100,46,22,0,111,29,11,0,224,165,3,0,39,148,3,0,102,145,6,0,226,163,6,0,227,163,6,0,141,74,7,0,126,66,13,0,151,93,29,0,180,70,13,0,216,176,5,0,211,163,6,0,224,0,0,0,99,11,0,0,204,52,0,0,51,106,0,0,201,115,0,0,160,225,0,0,253,247,0,0,204,249,0,0,125,97,1,0,235,202,1,0,169,244,1,0,179,171,3,0,80,148,6,0,32,148,3,0,119,151,3,0,224,229,3,0,212,163,6,0,237,163,6,0,102,97,11,0,241,89,14,0,110,29,11,0,118,86,14,0,234,163,6,0,116,86,14,0,99,97,11,0,183,70,13,0,166,203,7,0,186,70,13,0,148,93,29,0,100,97,11,0,241,163,6,0,162,203,7,0,81,148,6,0,250,45,7,0,187,70,13,0,247,45,7,0,148,75,7,0,191,236,28,0,250,89,14,0,155,0,22,0,228,163,6,0,230,0,0,0,148,14,0,0,118,56,0,0,239,112,0,0,42,213,0,0,21,96,1,0,20,96,1,0,249,171,1,0,23,207,1,0,121,74,3,0,80,70,3,0,5,231,3,0,208,163,6,0,48,132,5,0,157,75,7,0,126,190,7,0,190,176,5,0,115,151,3,0,222,163,6,0,251,0,0,0,123,15,0,0,215,61,0,0,208,123,0,0,156,229,0,0,205,249,0,0,24,207,1,0,255,210,1,0,122,74,3,0,41,148,3,0,12,80,3,0,224,45,7,0,86,148,6,0,124,190,7,0,181,70,13,0,178,70,13,0,221,163,6,0,162,1,0,0,59,22,0,0,19,105,0,0,22,176,0,0,66,250,0,0,45,163,1,0,6,207,1,0,124,74,3,0,125,74,3,0,97,97,11,0,129,84,3,0,250,229,3,0,160,203,7,0,127,190,7,0,163,203,7,0,167,203,7,0,211,176,5,0,230,45,7,0,220,163,6,0,169,1,0,0,180,26,0,0,52,106,0,0,106,212,0,0,48,97,1,0,95,239,1,0,50,245,1,0,54,245,1,0,46,148,3,0,54,132,5,0,219,163,6,0,91,148,6,0,201,1,0,0,160,28,0,0,139,114,0,0,217,238,0,0,57,245,1,0,29,202,1,0,101,151,3,0,102,151,3,0,57,132,5,0,93,148,6,0,103,151,3,0,211,1,0,0,44,31,0,0,252,123,0,0,97,97,1,0,82,70,3,0,237,165,3,0,141,84,3,0,56,132,5,0,218,163,6,0,193,2,0,0,94,44,0,0,53,211,0,0,26,171,1,0,116,216,2,0,6,80,3,0,132,84,3,0,204,176,5,0,154,75,7,0,243,45,7,0,214,163,6,0,218,2,0,0,179,52,0,0,174,213,0,0,238,202,1,0,113,216,2,0,227,87,3,0,151,75,7,0,249,45,7,0,186,128,5,0,212,176,5,0,77,3,0,0,78,53,0,0,80,247,0,0,192,203,1,0,231,165,3,0,228,165,3,0,133,3,0,0,88,58,0,0,65,108,1,0,207,197,2,0,225,229,3,0,108,75,7,0,119,86,14,0,223,163,6,0,144,3,0,0,80,62,0,0,194,99,1,0,118,216,2,0,130,84,3,0,214,176,5,0,58,132,5,0,159,3,0,0,94,88,0,0,131,165,1,0,15,80,3,0,147,75,7,0,113,151,3,0,228,3,0,0,18,105,0,0,67,108,1,0,225,87,3,0,138,5,0,0,111,105,0,0,56,245,1,0,201,176,5,0,207,163,6,0,182,5,0,0,248,106,0,0,52,245,1,0,131,132,5,0,224,163,6,0,149,6,0,0,2,125,0,0,232,202,1,0,133,132,5,0,162,6,0,0,74,117,0,0,238,87,3,0,123,151,3,0,72,7,0,0,178,116,0,0,123,74,3,0,41,7,0,0,224,177,0,0,73,70,3,0,227,229,3,0,61,7,0,0,196,210,0,0,230,229,3,0,187,7,0,0,153,176,0,0,98,151,3,0,206,176,5,0,95,148,6,0,209,7,0,0,171,213,0,0,121,151,3,0,211,7,0,0,47,213,0,0,99,151,3,0,92,148,6,0,26,11,0,0,197,210,0,0,137,84,3,0,35,13,0,0,237,234,0,0,248,229,3,0,50,13,0,0,22,96,1,0,251,229,3,0,65,13,0,0,104,231,0,0,237,165,3,0,31,14,0,0,23,96,1,0,39,128,5,0,173,14,0,0,7,250,0,0,85,148,6,0,84,14,0,0,43,97,1,0,85,14,0,0,129,165,1,0,120,15,0,0,43,163,1,0,188,128,5,0,238,163,6,0,121,15,0,0,253,171,1,0,149,15,0,0,24,171,1,0,240,163,6,0,55,22,0,0,77,170,1,0,45,22,0,0,60,245,1,0,243,163,6,0,49,26,0,0,16,168,1,0,105,151,3,0,80,26,0,0,239,202,1,0,54,26,0,0,46,163,1,0,103,26,0,0,142,243,1,0,133,26,0,0,89,239,1,0,166,26,0,0,131,239,1,0,81,29,0,0,18,192,2,0,83,29,0,0,121,216,2,0,94,29,0,0,5,80,3,0,186,28,0,0,4,207,1,0,83,148,6,0,45,29,0,0,255,81,3,0,45,31,0,0,111,216,2,0,41,31,0,0,7,80,3,0,34,44,0,0,250,81,3,0,3,44,0,0,236,165,3,0,95,44,0,0,235,165,3,0,88,44,0,0,107,74,3,0,105,52,0,0,190,86,3,0,89,44,0,0,108,74,3,0,106,52,0,0,234,165,3,0,189,52,0,0,191,52,0,0,191,86,3,0,106,56,0,0,185,58,0,0,63,132,5,0,107,56,0,0,245,165,3,0,75,58,0,0,33,148,3,0,164,58,0,0,233,165,3,0,90,58,0,0,96,57,0,0,126,151,3,0,233,61,0,0,88,57,0,0,247,61,0,0,225,57,0,0,228,229,3,0,95,57,0,0,88,148,6,0,145,62,0,0,242,61,0,0,40,148,3,0,242,88,0,0,128,62,0,0,195,163,6,0,147,62,0,0,192,148,6,0,184,88,0,0,202,176,5,0,79,88,0,0,193,148,6,0,241,88,0,0,214,104,0,0,16,106,0,0,195,106,0,0,50,106,0,0,210,112,0,0,17,105,0,0,177,116,0,0,148,116,0,0,212,106,0,0,214,106,0,0,184,114,0,0,250,106,0,0,179,116,0,0,64,117,0,0,206,115,0,0,5,176,0,0,179,116,0,0,149,116,0,0,185,116,0,0,54,211,0,0,255,123,0,0,99,119,0,0,200,115,0,0,41,125,0,0,34,182,0,0,33,210,0,0,129,209,0,0,209,177,0,0,184,116,0,0,208,177,0,0,155,209,0,0,195,210,0,0,114,177,0,0,220,210,0,0,35,182,0,0,170,213,0,0,38,212,0,0,130,209,0,0,149,231,0,0,209,225,0,0,55,211,0,0,108,233,0,0,228,229,0,0,20,229,0,0,238,234,0,0,0,96,1,0,103,231,0,0,161,225,0,0,143,231,0,0,4,96,1,0,194,247,0,0,153,231,0,0,231,229,0,0,102,229,0,0,105,231,0,0,81,247,0,0,222,238,0,0,6,250,0,0,5,96,1,0,159,250,0,0,230,165,1,0,102,231,0,0,111,99,1,0,221,238,0,0,192,238,0,0,9,163,1,0,202,206,1,0,205,99,1,0,203,249,0,0,223,238,0,0,130,165,1,0,45,97,1,0,229,229,0,0,248,171,1,0,12,163,1,0,31,202,1,0,204,99,1,0,92,163,1,0,30,202,1,0,81,170,1,0,172,99,1,0,78,168,1,0,63,165,1,0,22,207,1,0,252,210,1,0,179,165,1,0,25,171,1,0,31,168,1,0,195,213,1,0,63,108,1,0,193,213,1,0,252,210,1,0,170,244,1,0,18,168,1,0,53,245,1,0,18,207,1,0,23,168,1,0,124,97,1,0,11,171,1,0,248,210,1,0,130,239,1,0,122,216,2,0,47,213,1,0,48,245,1,0,72,170,1,0,135,84,3,0,253,210,1,0,173,244,1,0,17,207,1,0,27,70,3,0,133,84,3,0,32,202,1,0,237,202,1,0,230,202,1,0,255,171,1,0,79,70,3,0,111,74,3,0,129,239,1,0,75,70,3,0,150,157,3,0,131,243,1,0,55,245,1,0,20,207,1,0,206,197,2,0,14,80,3,0,81,194,2,0,236,202,1,0,135,243,1,0,84,70,3,0,228,87,3,0,120,216,2,0,11,80,3,0,128,84,3,0,232,165,3,0,142,84,3,0,100,75,3,0,168,244,1,0,3,80,3,0,223,229,3,0,112,216,2,0,230,87,3,0,240,229,3,0,90,239,1,0,234,165,3,0,136,243,1,0,3,231,3,0,78,194,2,0,226,165,3,0,253,81,3,0,221,198,2,0,4,231,3,0,254,81,3,0,117,216,2,0,199,176,5,0,106,151,3,0,230,165,3,0,35,148,3,0,128,132,5,0,70,194,2,0,227,165,3,0,119,216,2,0,241,229,3,0,190,171,3,0,137,132,5,0,249,229,3,0,224,87,3,0,188,171,3,0,198,176,5,0,103,145,6,0,101,145,6,0,233,229,3,0,34,148,3,0,111,151,3,0,125,151,3,0,222,229,3,0,201,163,6,0,152,139,5,0,246,165,3,0,208,165,3,0,134,132,5,0,193,163,6,0,252,229,3,0,220,176,5,0,143,84,3,0,45,148,3,0,201,148,6,0,132,132,5,0,232,165,3,0,155,75,7,0,150,75,7,0,208,148,6,0,136,132,5,0,228,165,3,0,42,148,3,0,194,46,7,0,118,151,3,0,209,176,5,0,207,176,5,0,214,165,3,0,252,89,14,0,200,176,5,0,231,229,3,0,183,203,7,0,135,14,7,0,171,203,7,0,194,176,5,0,195,148,6,0,142,74,7,0,243,229,3,0,205,163,6,0,254,45,7,0,46,59,7,0,192,46,7,0,197,148,6,0,55,132,5,0,200,148,6,0,255,45,7,0,53,148,3,0,61,132,5,0,215,163,6,0,193,46,7,0,200,34,13,0,207,148,6,0,115,97,11,0,254,229,3,0,187,128,5,0,242,89,14,0,110,97,11,0,117,97,11,0,218,165,3,0,189,176,5,0,204,148,6,0,60,132,5,0,199,148,6,0,146,75,7,0,195,46,7,0,198,148,6,0,112,97,11,0,172,203,7,0,51,23,11,0,164,203,7,0,103,97,11,0,231,45,7,0,228,45,7,0,192,163,6,0,239,229,3,0,101,46,22,0,227,45,7,0,251,45,7,0,95,163,6,0,235,163,6,0,11,9,2,3,4,4,5,6,6,7,7,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11,11,11,11,12,12,12,12,12,12,12,12,12,13,13,13,13,13,13,13,13,13,14,14,14,14,14,14,14,14,14,14,14,14,14,14,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,16,15,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,17,17,17,17,17,17,17,17,17,17,17,18,17,17,17,17,17,17,17,18,18,17,17,18,17,17,18,17,18,18,18,18,19,18,18,18,18,18,18,20,18,18,18,19,19,18,19,18,19,19,18,19,19,18,19,19,19,19,18,19,19,19,19,19,19,19,20,20,20,19,19,20,19,20,19,19,20,19,19,20,20,20,20,19,20,21,19,3,5,7,8,9,9,10,11,11,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15,15,15,15,15,15,15,15,16,16,15,15,15,15,16,16,16,16,17,16,17,17,16,17,17,17,17,17,17,16,17,17,17,17,18,17,17,18,18,18,18,18,19,18,18,18,18,18,18,19,19,18,18,18,18,19,18,19,19,19,20,19,18,19,19,19,19,19,19,19,19,19,19,20,20,19,20,19,20,19,20,19,19,21,20,20,19,4,7,8,10,11,11,12,12,13,13,14,14,14,14,15,15,15,15,15,16,16,16,16,16,16,16,17,17,17,17,17,17,17,16,16,16,16,17,17,17,17,18,18,18,17,17,18,18,18,18,18,18,18,18,18,19,18,18,18,19,18,19,19,19,20,20,20,19,19,19,19,19,19,19,21,21,20,19,5,8,10,11,12,13,13,13,14,14,15,15,15,15,16,16,16,16,16,17,17,17,17,17,17,17,17,18,17,18,17,17,17,17,17,17,17,17,17,17,17,19,18,19,18,18,18,18,18,19,18,17,17,18,18,19,19,19,19,18,18,18,19,6,9,11,12,13,13,14,14,14,15,15,16,16,16,16,16,16,17,17,17,18,18,18,18,18,18,18,18,18,18,18,17,18,18,17,18,18,18,18,18,18,19,19,18,18,18,19,19,19,20,19,19,18,19,19,20,21,21,19,19,18,6,10,12,13,14,14,14,15,15,15,16,16,17,17,17,17,17,17,17,18,18,19,18,18,18,19,18,18,18,19,18,18,18,18,18,18,18,18,18,18,18,19,20,20,19,19,19,19,20,20,19,20,19,19,19,20,20,20,19,19,18,19,7,10,12,13,14,15,15,15,16,16,17,17,17,17,17,17,18,18,18,18,19,18,19,19,19,20,19,18,19,19,18,18,19,19,19,18,19,19,20,19,18,20,21,20,20,19,19,21,20,21,20,20,20,19,19,20,20,21,20,19,7,11,13,14,15,15,15,16,16,17,17,17,17,18,18,18,18,18,19,20,19,19,20,19,19,19,19,19,19,19,19,18,18,19,20,19,19,19,20,19,19,19,20,19,20,20,21,20,20,20,21,22,20,19,20,20,21,20,21,20,19,8,11,13,14,15,16,16,16,17,17,17,18,18,18,18,18,19,18,19,19,19,19,21,19,19,21,19,20,20,20,19,18,18,8,12,14,15,16,16,16,16,17,17,17,19,18,18,19,19,20,19,18,20,19,20,20,19,19,20,20,21,21,20,19,19,19,19,19,19,20,21,20,19,19,8,12,14,15,16,16,17,17,17,18,18,18,19,19,19,19,19,19,20,21,20,21,19,21,20,20,20,20,21,20,19,20,19,20,20,20,19,22,21,21,19,9,12,14,15,16,17,17,17,18,18,18,19,19,19,19,20,19,19,19,9,13,15,16,17,17,18,18,18,19,18,20,19,20,20,20,19,9,13,15,16,17,17,18,18,18,20,18,19,20,20,20,20,19,20,19,9,13,15,16,17,18,18,18,19,19,19,19,10,14,16,17,18,18,19,19,19,19,19,10,14,16,17,18,18,18,19,19,10,14,16,17,18,18,18,19,19,20,19,10,14,16,18,18,18,19,20,19,19,10,14,17,18,18,18,10,15,17,18,19,19,21,19,11,15,17,18,18,19,19,11,15,17,18,19,19,11,15,17,18,11,15,18,19,19,11,15,18,19,19,11,16,18,19,11,15,18,19,11,16,18,12,16,18,19,12,16,19,12,16,19,19,19,12,16,19,12,16,19,19,12,16,18,12,16,19,12,17,19,12,17,19,12,17,19,12,17,19,13,17,13,17,13,17,19,19,13,17,13,17,19,13,17,13,18,19,13,17,19,13,18,13,17,13,18,13,18,13,18,13,18,13,18,13,18,14,18,19,14,18,14,18,14,18,14,18,14,19,14,19,14,18,14,18,14,18,14,19,14,14,18,14,14,19,14,18,14,19,14,19,14,15,19,15,15,15,15,19,15,19,15,15,19,15,15,19,15,19,15,19,15,19,15,15,15,15,15,15,15,15,15,15,15,16,15,15,15,16,16,16,15,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,17,16,16,16,17,17,16,17,17,16,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,17,18,17,17,17,17,17,17,17,17,18,17,17,18,17,17,17,17,18,18,17,17,17,17,17,17,17,18,17,18,18,17,17,17,18,18,18,17,18,17,18,18,18,18,18,18,18,18,18,17,18,18,18,18,19,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,19,18,18,19,18,18,18,19,18,19,18,18,19,18,18,19,19,19,19,19,18,19,18,19,18,19,19,18,18,19,19,19,19,19,19,19,19,19,19,18,19,19,19,19,19,18,19,19,19,19,19,19,19,19,19,19,20,19,19,19,19,21,19,19,20,19,20,19,19,19,19,19,20,20,20,19,19,19,20,19,19,19,20,20,19,20,19,19,21,20,20,19,19,19,19,19,19,20,19,20,20,20,20,20,20,20,19,19,21,20,20,19,19,21,1,0,0,2,0,0,0,1,0,0,0,0,0,0,0,13,0,0,0,7,0,0,0,19,0,0,0,29,0,0,0,8,0,0,0,12,0,0,0,35,0,0,0,43,0,0,0,63,0,0,0,23,0,0,0,27,0,0,0,67,0,0,0,73,0,0,0,80,0,0,0,85,0,0,0,84,0,0,0,103,0,0,0,100,0,0,0,123,0,0,0,45,0,0,0,40,0,0,0,42,0,0,0,133,0,0,0,137,0,0,0,43,0,0,0,53,0,0,0,144,0,0,0,145,0,0,0,148,0,0,0,136,0,0,0,193,0,0,0,198,0,0,0,242,0,0,0,227,0,0,0,197,0,0,0,226,0,0,0,54,0,0,0,240,0,0,0,167,0,0,0,205,0,0,0,251,0,0,0,89,0,0,0,22,1,0,0,3,1,0,0,8,1,0,0,43,1,0,0,45,1,0,0,136,1,0,0,46,1,0,0,76,1,0,0,195,1,0,0,135,1,0,0,231,1,0,0,111,0,0,0,148,0,0,0,105,0,0,0,230,1,0,0,202,1,0,0,71,1,0,0,149,1,0,0,167,0,0,0,19,2,0,0,9,2,0,0,3,3,0,0,149,2,0,0,137,2,0,0,140,2,0,0,141,2,0,0,18,3,0,0,48,3,0,0,155,2,0,0,8,3,0,0,40,3,0,0,154,2,0,0,94,2,0,0,197,3,0,0,132,3,0,0,159,3,0,0,151,3,0,0,150,2,0,0,46,3,0,0,50,3,0,0,198,3,0,0,230,3,0,0,45,1,0,0,209,0,0,0,2,4,0,0,221,0,0,0,97,1,0,0,43,1,0,0,39,1,0,0,93,4,0,0,1,6,0,0,171,4,0,0,95,4,0,0,16,4,0,0,191,4,0,0,40,5,0,0,92,4,0,0,36,4,0,0,0,4,0,0,17,5,0,0,24,6,0,0,61,7,0,0,58,6,0,0,20,6,0,0,60,7,0,0,192,7,0,0,207,7,0,0,2,8,0,0,102,9,0,0,100,9,0,0,81,9,0,0,160,8,0,0,70,3,0,0,3,8,0,0,82,10,0,0,74,2,0,0,193,7,0,0,63,6,0,0,38,1,0,0,6,4,0,0,137,7,0,0,162,8,0,0,96,9,0,0,103,9,0,0,5,12,0,0,112,12,0,0,121,12,0,0,93,10,0,0,38,12,0,0,77,12,0,0,114,3,0,0,165,8,0,0,8,12,0,0,197,2,0,0,17,15,0,0,196,12,0,0,142,15,0,0,22,14,0,0,150,4,0,0,119,14,0,0,156,15,0,0,37,12,0,0,30,15,0,0,39,12,0,0,31,15,0,0,23,14,0,0,205,12,0,0,85,3,0,0,9,12,0,0,120,12,0,0,144,15,0,0,33,5,0,0,87,3,0,0,86,3,0,0,142,6,0,0,157,15,0,0,4,12,0,0,88,14,0,0,32,10,0,0,44,10,0,0,76,12,0,0,47,5,0,0,141,15,0,0,120,17,0,0,83,16,0,0,151,16,0,0,15,24,0,0,13,24,0,0,251,18,0,0,170,18,0,0,42,32,0,0,64,10,0,0,237,24,0,0,235,28,0,0,85,20,0,0,227,24,0,0,161,18,0,0,84,3,0,0,83,3,0,0,28,15,0,0,123,12,0,0,55,12,0,0,29,16,0,0,203,18,0,0,66,17,0,0,125,25,0,0,149,16,0,0,59,30,0,0,107,24,0,0,136,5,0,0,42,28,0,0,184,20,0,0,58,30,0,0,236,24,0,0,70,31,0,0,250,18,0,0,83,10,0,0,232,28,0,0,85,10,0,0,41,28,0,0,123,17,0,0,82,16,0,0,160,18,0,0,137,5,0,0,80,9,0,0,43,28,0,0,80,10,0,0,139,32,0,0,14,24,0,0,39,32,0,0,86,37,0,0,32,30,0,0,231,6,0,0,40,28,0,0,122,25,0,0,132,6,0,0,162,32,0,0,34,31,0,0,24,48,0,0,207,57,0,0,37,62,0,0,87,37,0,0,76,41,0,0,166,40,0,0,17,13,0,0,169,40,0,0,121,41,0,0,70,13,0,0,86,10,0,0,206,57,0,0,204,48,0,0,154,50,0,0,157,20,0,0,15,81,0,0,28,69,0,0,40,32,0,0,153,50,0,0,237,28,0,0,185,20,0,0,133,15,0,0,122,12,0,0,0,24,0,0,65,3,0,0,202,18,0,0,200,57,0,0,157,50,0,0,13,13,0,0,32,62,0,0,68,81,0,0,69,13,0,0,208,48,0,0,109,24,0,0,213,48,0,0,15,13,0,0,64,13,0,0,20,65,0,0,161,32,0,0,127,41,0,0,36,62,0,0,241,50,0,0,71,64,0,0,212,48,0,0,168,40,0,0,14,13,0,0,29,69,0,0,68,64,0,0,126,41,0,0,66,64,0,0,210,48,0,0,207,48,0,0,33,62,0,0,38,62,0,0,165,40,0,0,26,69,0,0,72,13,0,0,22,26,0,0,68,13,0,0,24,69,0,0,155,20,0,0,202,57,0,0,152,20,0,0,61,64,0,0,27,69,0,0,156,20,0,0,243,50,0,0,203,48,0,0,115,128,0,0,34,62,0,0,154,82,0,0,170,32,0,0,204,57,0,0,138,115,0,0,48,101,0,0,137,115,0,0,147,97,0,0,113,128,0,0,67,64,0,0,206,48,0,0,71,81,0,0,136,115,0,0,69,81,0,0,114,128,0,0,33,69,0,0,71,13,0,0,124,41,0,0,205,48,0,0,202,48,0,0,11,0,0,0,12,0,0,0,131,0,0,0,228,0,0,0,72,0,0,0,2,1,0,0,204,1,0,0,245,1,0,0,151,0,0,0,11,2,0,0,36,1,0,0,83,4,0,0,39,6,0,0,57,6,0,0,5,6,0,0,23,5,0,0,184,1,0,0,99,6,0,0,103,6,0,0,195,7,0,0,35,8,0,0,97,9,0,0,99,9,0,0,90,14,0,0,89,14,0,0,43,10,0,0,191,12,0,0,146,2,0,0,45,10,0,0,208,7,0,0,83,9,0,0,197,12,0,0,132,15,0,0,171,4,0,0,167,20,0,0,138,6,0,0,122,17,0,0,46,5,0,0,66,20,0,0,44,5,0,0,119,12,0,0,143,15,0,0,170,4,0,0,148,16,0,0,1,24,0,0,196,18,0,0,123,41,0,0,82,9,0,0,25,31,0,0,165,6,0,0,73,17,0,0,197,18,0,0,3,24,0,0,242,34,0,0,155,50,0,0,32,69,0,0,158,20,0,0,19,13,0,0,22,31,0,0,233,28,0,0,28,16,0,0,230,6,0,0,201,57,0,0,145,97,0,0,142,124,0,0,146,97,0,0,99,202,0,0,205,57,0,0,144,97,0,0,132,104,0,0,133,104,0,0,130,115,0,0,73,13,0,0,65,13,0,0,12,69,0,0,154,20,0,0,209,48,0,0,119,128,0,0,35,62,0,0,21,26,0,0,1,231,0,0,2,231,0,0,121,128,0,0,42,130,0,0,24,162,0,0,135,120,0,0,63,64,0,0,11,82,0,0,155,82,0,0,0,231,0,0,25,69,0,0,7,0,0,0,224,0,0,0,208,0,0,0,155,3,0,0,229,3,0,0,99,1,0,0,62,6,0,0,201,7,0,0,6,8,0,0,84,9,0,0,68,16,0,0,68,31,0,0,124,25,0,0,69,31,0,0,81,10,0,0,71,31,0,0,81,9,0,0,45,5,0,0,145,34,0,0,47,9,0,0,84,10,0,0,18,13,0,0,125,41,0,0,12,13,0,0,153,20,0,0,158,50,0,0,240,50,0,0,37,32,0,0,198,57,0,0,87,10,0,0,70,62,0,0,66,13,0,0,139,115,0,0,70,81,0,0,70,64,0,0,120,128,0,0,14,81,0,0,134,120,0,0,4,41,0,0,86,65,0,0,87,65,0,0,50,96,0,0,211,48,0,0,206,139,0,0,64,64,0,0,62,64,0,0,20,164,0,0,87,4,1,0,117,128,0,0,135,104,0,0,143,124,0,0,199,57,0,0,135,115,0,0,112,128,0,0,207,139,0,0,42,72,1,0,86,4,1,0,43,72,1,0,23,26,0,0,134,104,0,0,13,69,0,0,19,0,0,0,107,0,0,0,21,6,0,0,11,8,0,0,43,8,0,0,82,9,0,0,91,14,0,0,226,24,0,0,108,24,0,0,24,31,0,0,159,50,0,0,67,13,0,0,41,62,0,0,64,81,0,0,65,81,0,0,98,202,0,0,51,96,0,0,66,60,0,0,40,62,0,0,15,69,0,0,26,162,0,0,132,115,0,0,25,162,0,0,3,231,0,0,27,162,0,0,20,26,0,0,131,115,0,0,230,69,0,0,122,0,0,0,44,1,0,0,204,12,0,0,143,6,0,0,2,24,0,0,82,10,0,0,83,9,0,0,69,64,0,0,32,26,0,0,31,69,0,0,164,0,0,0,53,7,0,0,236,28,0,0,41,32,0,0,163,32,0,0,30,69,0,0,105,0,0,0,36,12,0,0,36,32,0,0,242,50,0,0,66,81,0,0,150,1,0,0,35,5,0,0,166,0,0,0,123,25,0,0,11,3,0,0,46,9,0,0,233,3,0,0,39,62,0,0,96,1,0,0,67,81,0,0,82,6,0,0,65,64,0,0,52,7,0,0,167,40,0,0,15,8,0,0,131,20,0,0,124,9,0,0,64,3,0,0,139,6,0,0,34,5,0,0,84,16,0,0,150,16,0,0,23,31,0,0,43,32,0,0,234,28,0,0,160,32,0,0,120,41,0,0,38,32,0,0,122,41,0,0,203,57,0,0,43,62,0,0,159,20,0,0,156,50,0,0,133,115,0,0,116,128,0,0,14,69,0,0,42,62,0,0,73,81,0,0,118,128,0,0,134,115,0,0,72,81,0,0,0,0,0,0,9,5,2,4,4,5,5,5,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,9,8,8,8,8,9,9,9,9,9,9,9,9,9,9,9,9,10,10,10,9,9,9,9,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,12,12,12,12,12,12,12,12,12,11,11,11,11,11,12,12,12,12,12,12,12,12,12,13,12,12,12,12,12,12,12,13,12,12,12,12,12,12,12,12,13,12,12,12,13,13,13,13,12,12,12,12,12,12,13,12,13,13,13,13,13,13,13,14,14,13,13,13,13,13,13,13,12,12,12,13,13,13,13,13,13,13,13,13,13,13,13,13,13,14,13,14,13,13,13,13,13,14,13,14,14,13,14,14,13,14,13,13,14,14,13,14,14,14,14,14,14,14,14,14,15,14,14,14,14,15,15,15,14,14,13,13,12,12,13,13,13,14,14,15,14,15,15,14,13,14,15,15,15,14,14,14,14,15,14,14,15,15,15,14,15,14,14,14,14,14,15,15,16,15,15,15,14,15,15,15,15,14,14,16,14,15,14,14,15,15,15,15,16,15,14,15,15,15,16,15,15,14,14,14,4,7,8,8,9,9,9,9,10,10,11,11,11,11,11,11,12,11,11,11,12,12,12,12,12,12,12,12,12,11,12,12,12,13,13,13,13,13,13,13,12,12,13,13,13,13,14,14,13,14,13,13,13,14,14,15,15,14,13,13,13,14,14,15,15,15,16,14,15,17,17,15,15,15,15,15,14,16,14,16,16,16,16,16,16,15,15,17,15,16,15,6,8,10,10,10,11,11,11,12,12,13,13,13,13,14,13,14,13,14,14,14,14,14,15,15,14,14,14,14,14,14,15,15,15,15,16,15,15,16,15,15,15,14,16,15,15,18,17,16,17,15,14,15,16,16,19,17,19,16,17,15,7,10,11,12,12,12,12,13,13,13,14,15,14,15,15,16,15,14,14,15,16,15,16,16,16,16,15,15,7,11,12,13,13,14,14,15,15,15,8,11,13,14,14,15,9,12,14,14,15,9,13,10,13,10,14,10,14,11,15,11,15,11,14,12,15,12,13,13,13,13,13,13,14,13,14,14,14,14,14,14,15,14,15,16,15,14,15,16,15,15,0,0,0,0,0,88,2,0,0,61,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,27,0,0,0,31,0,0,0,21,0,0,0,36,0,0,0,50,0,0,0,58,0,0,0,38,0,0,0,44,0,0,0,47,0,0,0,74,0,0,0,77,0,0,0,97,0,0,0,112,0,0,0,115,0,0,0,72,0,0,0,82,0,0,0,90,0,0,0,93,0,0,0,110,0,0,0,153,0,0,0,158,0,0,0,193,0,0,0,206,0,0,0,228,0,0,0,240,0,0,0,147,0,0,0,158,0,0,0,162,0,0,0,161,0,0,0,184,0,0,0,210,0,0,0,211,0,0,0,46,1,0,0,48,1,0,0,222,0,0,0,45,1,0,0,155,1,0,0,228,1,0,0,57,1,0,0,58,1,0,0,63,1,0,0,79,1,0,0,109,1,0,0,162,1,0,0,124,2,0,0,126,2,0,0,50,3,0,0,60,3,0,0,63,3,0,0,139,3,0,0,150,3,0,0,197,3,0,0,112,2,0,0,124,2,0,0,90,2,0,0,149,3,0,0,72,2,0,0,189,4,0,0,251,4,0,0,98,6,0,0,97,6,0,0,27,7,0,0,230,4,0,0,255,4,0,0,102,6,0,0,28,7,0,0,26,7,0,0,31,7,0,0,148,7,0,0,54,5,0,0,226,4,0,0,142,7,0,0,238,4,0,0,24,5,0,0,53,5,0,0,251,4,0,0,141,7,0,0,48,5,0,0,128,6,0,0,143,6,0,0,203,5,0,0,101,9,0,0,166,6,0,0,103,9,0,0,127,9,0,0,130,6,0,0,174,6,0,0,208,12,0,0,40,14,0,0,19,15,0,0,31,15,0,0,245,9,0,0,211,12,0,0,17,15,0,0,38,9,0,0,100,9,0,0,50,15,0,0,18,15,0,0,48,15,0,0,102,9,0,0,11,13,0,0,104,10,0,0,145,11,0,0,199,9,0,0,115,11,0,0,250,18,0,0,29,19,0,0,249,19,0,0,160,28,0,0,156,25,0,0,122,28,0,0,140,25,0,0,72,18,0,0,116,28,0,0,100,28,0,0,158,19,0,0,253,18,0,0,119,10,0,0,252,18,0,0,123,28,0,0,202,18,0,0,204,20,0,0,210,20,0,0,227,20,0,0,220,20,0,0,220,18,0,0,68,51,0,0,152,37,0,0,60,38,0,0,59,51,0,0,230,37,0,0,28,26,0,0,60,30,0,0,226,20,0,0,212,51,0,0,17,26,0,0,73,51,0,0,206,60,0,0,225,20,0,0,52,26,0,0,62,39,0,0,39,38,0,0,63,39,0,0,238,56,0,0,113,57,0,0,103,60,0,0,97,60,0,0,61,51,0,0,194,56,0,0,63,38,0,0,205,56,0,0,56,38,0,0,65,46,0,0,31,53,0,0,72,51,0,0,102,60,0,0,98,53,0,0,137,41,0,0,213,39,0,0,60,51,0,0,79,46,0,0,59,52,0,0,223,45,0,0,200,75,0,0,192,41,0,0,87,46,0,0,114,76,0,0,183,37,0,0,71,53,0,0,64,53,0,0,211,41,0,0,69,76,0,0,187,37,0,0,0,102,0,0,115,76,0,0,206,75,0,0,123,53,0,0,166,41,0,0,210,41,0,0,62,38,0,0,138,41,0,0,131,113,0,0,2,102,0,0,88,121,0,0,102,75,0,0,125,83,0,0,117,83,0,0,233,79,0,0,103,75,0,0,159,121,0,0,201,75,0,0,254,81,0,0,59,106,0,0,182,91,0,0,168,79,0,0,143,114,0,0,118,83,0,0,44,73,0,0,126,83,0,0,90,121,0,0,60,106,0,0,21,229,0,0,135,120,0,0,58,104,0,0,249,81,0,0,253,81,0,0,106,204,0,0,138,106,0,0,109,204,0,0,179,91,0,0,59,104,0,0,252,81,0,0,120,83,0,0,142,114,0,0,134,120,0,0,183,91,0,0,164,242,0,0,91,121,0,0,60,104,0,0,193,159,0,0,61,104,0,0,82,183,0,0,120,150,0,0,232,163,0,0,199,106,0,0,240,81,0,0,89,183,0,0,243,106,0,0,107,75,0,0,160,242,0,0,173,242,0,0,195,150,0,0,24,229,0,0,92,183,0,0,88,212,0,0,107,204,0,0,124,83,0,0,170,103,0,0,234,79,0,0,58,52,0,0,113,204,0,0,127,150,0,0,196,159,0,0,194,150,0,0,22,229,0,0,161,242,0,0,92,212,0,0,93,212,0,0,94,212,0,0,185,47,1,0,126,150,0,0,47,152,1,0,131,152,0,0,196,150,0,0,83,183,0,0,184,47,1,0,168,242,0,0,33,202,1,0,197,150,0,0,26,229,0,0,39,202,1,0,60,47,1,0,113,212,0,0,170,242,0,0,91,183,0,0,187,47,1,0,169,242,0,0,172,242,0,0,90,212,0,0,79,183,0,0,200,150,0,0,145,110,1,0,202,150,0,0,191,47,1,0,167,208,0,0,3,49,1,0,22,213,0,0,153,110,1,0,189,44,1,0,234,163,0,0,41,152,1,0,85,183,0,0,167,155,2,0,40,202,1,0,165,155,2,0,147,110,1,0,44,152,1,0,40,152,1,0,148,89,2,0,235,163,0,0,41,202,1,0,144,110,1,0,37,202,1,0,45,152,1,0,38,202,1,0,155,110,1,0,86,183,0,0,124,150,0,0,151,89,2,0,95,183,0,0,211,152,1,0,87,183,0,0,42,154,1,0,91,212,0,0,23,229,0,0,36,202,1,0,35,202,1,0,34,202,1,0,88,183,0,0,151,110,1,0,20,205,0,0,0,49,1,0,7,0,0,0,59,0,0,0,107,0,0,0,151,0,0,0,56,1,0,0,37,1,0,0,115,1,0,0,88,2,0,0,53,3,0,0,142,2,0,0,198,4,0,0,21,7,0,0,41,7,0,0,239,4,0,0,25,5,0,0,237,4,0,0,50,5,0,0,140,6,0,0,134,6,0,0,120,9,0,0,93,14,0,0,49,14,0,0,244,9,0,0,146,11,0,0,248,18,0,0,6,13,0,0,103,10,0,0,68,13,0,0,118,10,0,0,89,13,0,0,205,18,0,0,120,28,0,0,117,28,0,0,159,25,0,0,143,25,0,0,103,28,0,0,198,20,0,0,121,28,0,0,118,28,0,0,148,11,0,0,27,13,0,0,50,30,0,0,49,30,0,0,176,26,0,0,5,26,0,0,161,26,0,0,58,51,0,0,229,37,0,0,38,38,0,0,65,53,0,0,68,53,0,0,33,52,0,0,70,53,0,0,85,46,0,0,86,46,0,0,45,73,0,0,222,45,0,0,155,41,0,0,220,45,0,0,122,53,0,0,156,36,0,0,139,102,0,0,127,199,1,0,32,202,1,0,95,212,0,0,134,152,0,0,154,110,1,0,167,242,0,0,81,183,0,0,238,163,0,0,89,207,0,0,87,207,0,0,84,183,0,0,166,208,0,0,152,110,1,0,96,183,0,0,198,106,0,0,240,163,0,0,190,47,1,0,4,49,1,0,165,242,0,0,239,163,0,0,114,212,0,0,186,44,1,0,46,152,1,0,156,110,1,0,126,199,1,0,208,152,1,0,5,49,1,0,146,110,1,0,93,183,0,0,89,212,0,0,26,0,0,0,192,0,0,0,108,1,0,0,205,3,0,0,80,3,0,0,123,6,0,0,30,5,0,0,169,6,0,0,244,9,0,0,114,11,0,0,9,13,0,0,73,18,0,0,61,30,0,0,161,28,0,0,31,26,0,0,33,23,0,0,138,26,0,0,232,22,0,0,71,51,0,0,53,26,0,0,157,36,0,0,154,41,0,0,150,37,0,0,78,46,0,0,139,41,0,0,130,113,0,0,70,76,0,0,186,37,0,0,64,46,0,0,214,39,0,0,232,79,0,0,7,102,0,0,16,83,0,0,132,152,0,0,225,114,0,0,61,106,0,0,106,75,0,0,122,76,0,0,3,102,0,0,123,76,0,0,40,52,0,0,5,102,0,0,100,150,0,0,192,159,0,0,222,113,0,0,1,102,0,0,178,91,0,0,133,152,0,0,226,163,0,0,31,198,1,0,187,44,1,0])
.concat([80,183,0,0,88,207,0,0,125,150,0,0,149,89,2,0,173,104,6,0,90,183,0,0,194,159,0,0,127,83,0,0,94,183,0,0,174,63,1,0,188,47,1,0,49,0,0,0,196,1,0,0,197,4,0,0,184,5,0,0,244,12,0,0,111,9,0,0,70,13,0,0,87,30,0,0,4,26,0,0,37,38,0,0,70,51,0,0,249,40,0,0,71,76,0,0,224,114,0,0,105,75,0,0,32,52,0,0,87,121,0,0,57,102,0,0,158,121,0,0,89,121,0,0,129,120,0,0,104,75,0,0,195,159,0,0,214,159,0,0,112,204,0,0,241,163,0,0,190,44,1,0,14,227,0,0,27,229,0,0,242,106,0,0,188,44,1,0,125,199,1,0,171,242,0,0,189,47,1,0,47,170,1,0,236,163,0,0,115,212,0,0,119,83,0,0,233,163,0,0,43,152,1,0,0,227,0,0,63,47,1,0,95,207,0,0,192,150,0,0,60,140,3,0,148,110,1,0,149,110,1,0,61,47,1,0,164,155,2,0,166,155,2,0,124,199,1,0,186,168,6,0,92,84,3,0,87,52,3,0,172,104,6,0,187,168,6,0,157,110,1,0,25,229,0,0,150,89,2,0,62,47,1,0,54,0,0,0,62,3,0,0,173,6,0,0,3,13,0,0,200,18,0,0,74,18,0,0,66,60,0,0,205,60,0,0,6,102,0,0,128,120,0,0,82,104,0,0,58,106,0,0,180,91,0,0,162,242,0,0,199,159,0,0,185,44,1,0,108,204,0,0,232,166,0,0,193,150,0,0,74,0,0,0,85,3,0,0,249,18,0,0,232,20,0,0,190,26,0,0,182,37,0,0,46,73,0,0,198,159,0,0,255,81,0,0,111,204,0,0,203,150,0,0,113,208,0,0,209,152,1,0,184,44,1,0,61,140,3,0,175,63,1,0,201,150,0,0,157,0,0,0,57,5,0,0,206,18,0,0,31,52,0,0,193,41,0,0,51,75,0,0,227,163,0,0,112,208,0,0,150,110,1,0,99,183,0,0,160,0,0,0,206,9,0,0,204,56,0,0,61,52,0,0,250,81,0,0,136,152,0,0,186,47,1,0,223,0,0,0,117,10,0,0,167,41,0,0,197,159,0,0,1,227,0,0,123,150,0,0,231,1,0,0,201,18,0,0,251,81,0,0,137,152,0,0,166,242,0,0,111,1,0,0,185,28,0,0,90,207,0,0,191,44,1,0,121,150,0,0,114,2,0,0,21,26,0,0,122,150,0,0,203,3,0,0,246,37,0,0,98,183,0,0,141,2,0,0,96,60,0,0,94,207,0,0,82,3,0,0,204,60,0,0,47,7,0,0,134,113,0,0,236,4,0,0,121,83,0,0,142,6,0,0,135,152,0,0,167,6,0,0,241,106,0,0,41,14,0,0,91,207,0,0,49,15,0,0,112,212,0,0,198,9,0,0,251,19,0,0,2,49,1,0,165,25,0,0,1,49,1,0,131,25,0,0,101,28,0,0,79,18,0,0,199,20,0,0,38,23,0,0,191,26,0,0,4,51,0,0,36,38,0,0,65,60,0,0,215,39,0,0,221,45,0,0,84,46,0,0,60,52,0,0,4,102,0,0,129,113,0,0,58,102,0,0,169,79,0,0,59,102,0,0,17,83,0,0,122,83,0,0,57,104,0,0,181,91,0,0,47,73,0,0,240,106,0,0,199,150,0,0,110,204,0,0,123,83,0,0,92,207,0,0,86,207,0,0,210,152,1,0,93,207,0,0,237,163,0,0,163,242,0,0,42,152,1,0,97,183,0,0,198,150,0,0,11,6,2,3,4,5,5,5,5,6,6,6,6,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,9,9,9,9,9,9,9,9,9,9,9,9,9,10,10,10,10,10,10,10,10,10,10,10,10,10,10,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,12,12,11,12,12,12,12,11,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,13,13,12,12,12,13,13,13,13,13,13,13,13,13,13,13,13,13,14,13,13,13,13,13,13,13,14,14,14,14,14,14,14,14,14,14,14,14,13,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,15,15,14,14,15,15,15,14,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,14,15,15,15,15,16,16,16,15,16,15,15,16,16,16,16,15,16,16,16,15,16,16,15,16,16,16,16,16,16,16,16,16,16,15,15,16,16,15,16,16,16,17,17,17,16,16,17,16,16,16,16,17,16,17,17,16,16,15,15,15,16,17,16,17,16,16,17,17,17,17,17,17,16,17,17,17,16,17,17,16,17,17,17,16,17,17,16,16,17,17,17,18,17,17,17,17,17,18,18,17,17,17,19,17,19,18,17,17,18,17,17,18,17,17,17,18,17,17,18,17,17,17,17,17,16,17,17,17,17,18,16,17,4,6,8,9,9,10,10,10,10,11,11,11,11,12,12,12,12,12,12,12,12,12,13,13,13,13,13,13,13,13,13,13,13,13,13,13,14,13,13,13,13,13,13,14,14,14,14,14,14,15,15,15,15,15,15,16,15,15,15,15,15,15,17,17,17,16,18,16,17,17,16,16,17,17,18,17,16,17,17,17,16,17,17,18,17,18,17,17,17,18,17,17,5,8,10,10,11,11,12,12,12,13,13,14,13,13,14,14,14,14,14,14,15,15,15,15,15,15,15,15,15,15,15,15,16,16,15,16,16,15,15,15,15,15,16,16,15,15,16,16,17,17,18,17,16,17,18,19,17,16,16,17,17,17,6,9,11,12,12,13,13,13,14,14,14,15,15,15,16,15,15,15,15,15,15,16,16,16,16,17,18,16,16,16,18,17,16,17,18,17,17,16,17,17,16,17,16,17,18,18,18,17,19,19,17,20,19,18,19,20,18,16,18,17,7,10,12,13,13,14,14,14,15,15,16,16,16,16,16,18,16,17,17,8,11,13,14,14,15,16,16,16,16,17,17,17,18,18,17,17,8,12,14,15,15,15,17,17,18,17,9,12,14,15,16,16,17,9,13,15,16,16,17,9,13,16,16,16,10,13,16,18,17,10,14,17,10,14,17,11,14,16,11,14,11,15,12,16,12,16,12,16,12,16,12,17,13,13,17,13,17,13,13,14,14,14,14,14,14,14,15,15,15,15,15,15,15,16,15,16,16,16,16,16,16,17,16,16,16,16,17,16,17,16,17,17,17,0,0,0,0,0,0,0,0,5,0,55,0,231,0,222,1,206,3,217,3,200,7,205,7,200,15,221,15,228,31,236,31,4,0,12,0,53,0,114,0,234,0,237,0,226,1,209,3,211,3,224,3,216,7,207,15,213,15,54,0,52,0,113,0,232,0,236,0,225,1,207,3,221,3,219,3,208,7,199,15,212,15,228,15,230,0,112,0,233,0,221,1,227,1,210,3,220,3,204,7,202,7,222,7,216,15,234,15,219,31,223,1,235,0,220,1,230,1,213,3,222,3,203,7,221,7,220,7,205,15,226,15,231,15,225,31,208,3,224,1,228,1,214,3,197,7,209,7,219,7,210,15,224,7,217,15,235,15,227,31,233,31,196,7,229,1,215,3,198,7,207,7,218,7,203,15,218,15,227,15,233,15,230,31,243,31,247,31,211,7,216,3,225,3,212,7,217,7,211,15,222,15,221,31,217,31,226,31,234,31,241,31,246,31,210,7,212,3,218,3,199,7,215,7,226,7,206,15,219,15,216,31,238,31,240,63,244,31,242,63,225,7,223,3,201,7,214,7,202,15,208,15,229,15,230,15,235,31,239,31,243,63,244,63,245,63,224,15,206,7,213,7,198,15,209,15,225,15,224,31,232,31,240,31,241,63,248,63,246,63,252,127,232,15,223,7,201,15,215,15,220,15,220,31,223,31,237,31,245,31,249,63,251,63,253,127,254,127,231,31,204,15,214,15,223,15,222,31,218,31,229,31,242,31,250,63,247,63,252,63,253,63,255,127,0,0,0,0,0,0,14,0,5,0,16,0,48,0,111,0,241,0,250,1,254,3,3,0,0,0,4,0,18,0,44,0,106,0,117,0,248,0,15,0,2,0,6,0,20,0,46,0,105,0,114,0,245,0,47,0,17,0,19,0,42,0,50,0,108,0,236,0,250,0,113,0,43,0,45,0,49,0,109,0,112,0,242,0,249,1,239,0,104,0,51,0,107,0,110,0,238,0,249,0,252,3,248,1,116,0,115,0,237,0,240,0,246,0,246,1,253,1,253,3,243,0,244,0,247,0,247,1,251,1,252,1,255,3,0,0,5,0,55,0,116,0,242,0,235,1,237,3,247,7,4,0,12,0,53,0,113,0,236,0,238,0,238,1,245,1,54,0,52,0,114,0,234,0,241,0,233,1,243,1,245,3,115,0,112,0,235,0,240,0,241,1,240,1,236,3,250,3,243,0,237,0,232,1,239,1,239,3,241,3,249,3,251,7,237,1,239,0,234,1,242,1,243,3,248,3,249,7,252,7,238,3,236,1,244,1,244,3,247,3,248,7,253,15,254,15,246,7,240,3,242,3,246,3,250,7,253,7,252,15,255,15,254,7,253,3,241,1,235,1,244,1,234,1,240,1,252,3,253,7,246,3,229,1,234,0,108,0,113,0,104,0,240,0,230,1,247,3,243,1,239,0,50,0,39,0,40,0,38,0,49,0,235,0,247,1,232,1,111,0,46,0,8,0,4,0,6,0,41,0,107,0,238,1,239,1,114,0,45,0,2,0,0,0,3,0,47,0,115,0,250,1,231,1,110,0,43,0,7,0,1,0,5,0,44,0,109,0,236,1,249,1,238,0,48,0,36,0,42,0,37,0,51,0,236,0,242,1,248,3,228,1,237,0,106,0,112,0,105,0,116,0,241,0,250,3,255,7,249,3,246,1,237,1,248,1,233,1,245,1,251,3,252,7,0,0,0,0,0,0,255,31,247,15,244,7,232,7,241,3,238,7,249,7,248,15,253,31,253,15,241,7,232,3,232,1,240,0,236,1,238,3,242,7,250,15,244,15,239,3,242,1,232,0,112,0,236,0,240,1,234,3,243,7,235,7,235,1,234,0,26,0,8,0,25,0,238,0,239,1,237,7,240,3,242,0,115,0,11,0,0,0,10,0,113,0,243,0,233,7,239,7,238,1,239,0,24,0,9,0,27,0,235,0,233,1,236,7,246,7,235,3,243,1,237,0,114,0,233,0,241,1,237,3,247,7,246,15,240,7,233,3,237,1,241,0,234,1,236,3,248,7,249,15,252,31,252,15,245,15,234,7,243,3,242,3,245,7,251,15,254,31,0,0,0,0,0,0,7,0,22,0,246,0,24,0,8,0,239,0,239,1,243,0,248,7,25,0,23,0,237,0,21,0,1,0,226,0,240,0,112,0,240,3,238,1,241,0,250,7,238,0,228,0,242,3,246,7,239,3,253,7,5,0,20,0,242,0,9,0,4,0,229,0,244,0,232,0,244,3,6,0,2,0,231,0,3,0,0,0,107,0,227,0,105,0,243,1,235,0,230,0,246,3,110,0,106,0,244,1,236,3,240,1,249,3,245,0,236,0,251,7,234,0,111,0,247,3,249,7,243,3,255,15,233,0,109,0,248,3,108,0,104,0,245,1,238,3,242,1,244,7,247,7,241,3,254,15,237,3,241,1,245,7,254,7,245,3,252,7,0,0,0,0,0,0,0,0,9,0,239,0,11,0,25,0,240,0,235,1,230,1,242,3,10,0,53,0,239,1,52,0,55,0,233,1,237,1,231,1,243,3,238,1,237,3,250,31,236,1,242,1,249,7,248,7,248,3,248,15,8,0,56,0,246,3,54,0,117,0,241,3,235,3,236,3,244,15,24,0,118,0,244,7,57,0,116,0,239,3,243,1,244,1,246,7,232,1,234,3,252,31,242,0,241,1,251,15,245,3,243,7,252,15,238,0,247,3,254,127,240,1,245,7,253,127,251,31,250,63,255,255,241,0,240,3,252,63,234,1,238,3,251,63,246,15,250,15,252,127,242,7,245,15,254,255,244,3,247,7,251,127,247,15,249,15,250,127,0,0,0,0,0,0,243,1,111,0,253,1,235,0,35,0,234,0,247,1,232,0,250,1,242,0,45,0,112,0,32,0,6,0,43,0,110,0,40,0,233,0,249,1,102,0,248,0,231,0,27,0,241,0,244,1,107,0,245,1,236,0,42,0,108,0,44,0,10,0,39,0,103,0,26,0,245,0,36,0,8,0,31,0,9,0,0,0,7,0,29,0,11,0,48,0,239,0,28,0,100,0,30,0,12,0,41,0,243,0,47,0,240,0,252,1,113,0,242,1,244,0,33,0,230,0,247,0,104,0,248,1,238,0,34,0,101,0,49,0,2,0,38,0,237,0,37,0,106,0,251,1,114,0,254,1,105,0,46,0,246,0,255,1,109,0,246,1,0,0,0,0,0,0,0,0,6,0,25,0,61,0,156,0,198,0,167,1,144,3,194,3,223,3,230,7,243,7,251,15,236,7,250,15,254,15,142,3,5,0,1,0,8,0,20,0,55,0,66,0,146,0,175,0,145,1,165,1,181,1,158,3,192,3,162,3,205,3,214,7,174,0,23,0,7,0,9,0,24,0,57,0,64,0,142,0,163,0,184,0,153,1,172,1,193,1,177,3,150,3,190,3,202,3,157,0,60,0,21,0,22,0,26,0,59,0,68,0,145,0,165,0,190,0,150,1,174,1,185,1,161,3,145,3,165,3,213,3,148,0,154,0,54,0,56,0,58,0,65,0,140,0,155,0,176,0,195,0,158,1,171,1,188,1,159,3,143,3,169,3,207,3,147,0,191,0,62,0,63,0,67,0,69,0,158,0,167,0,185,0,148,1,162,1,186,1,195,1,166,3,167,3,187,3,212,3,159,0,160,1,143,0,141,0,144,0,152,0,166,0,182,0,196,0,159,1,175,1,191,1,153,3,191,3,180,3,201,3,231,3,168,0,182,1,171,0,164,0,170,0,178,0,194,0,197,0,152,1,164,1,184,1,140,3,164,3,196,3,198,3,221,3,232,3,173,0,175,3,146,1,189,0,188,0,142,1,151,1,154,1,163,1,177,1,141,3,152,3,183,3,211,3,209,3,219,3,221,7,180,0,222,3,169,1,155,1,156,1,161,1,170,1,173,1,179,1,139,3,178,3,184,3,206,3,225,3,224,3,210,7,229,7,183,0,227,7,187,1,168,1,166,1,176,1,178,1,183,1,155,3,154,3,186,3,181,3,214,3,215,7,228,3,216,7,234,7,186,0,232,7,160,3,189,1,180,1,138,3,196,1,146,3,170,3,176,3,188,3,215,3,212,7,220,7,219,7,213,7,240,7,193,0,251,7,200,3,163,3,149,3,157,3,172,3,174,3,197,3,216,3,226,3,230,3,228,7,231,7,224,7,233,7,247,7,144,1,242,7,147,3,190,1,192,1,148,3,151,3,173,3,195,3,193,3,210,3,218,7,217,7,223,7,235,7,244,7,250,7,149,1,248,7,189,3,156,3,171,3,168,3,179,3,185,3,208,3,227,3,229,3,226,7,222,7,237,7,241,7,249,7,252,7,147,1,253,15,220,3,182,3,199,3,204,3,203,3,217,3,218,3,211,7,225,7,238,7,239,7,245,7,246,7,252,15,255,15,157,1,194,1,181,0,161,0,150,0,151,0,149,0,153,0,160,0,162,0,172,0,169,0,177,0,179,0,187,0,192,0,143,1,4,0,0,0,0,0,0,0,34,0,8,0,29,0,38,0,95,0,211,0,207,1,208,3,215,3,237,3,240,7,246,7,253,15,7,0,0,0,1,0,9,0,32,0,84,0,96,0,213,0,220,0,212,1,205,3,222,3,231,7,28,0,2,0,6,0,12,0,30,0,40,0,91,0,205,0,217,0,206,1,220,1,217,3,241,3,37,0,11,0,10,0,13,0,36,0,87,0,97,0,204,0,221,0,204,1,222,1,211,3,231,3,93,0,33,0,31,0,35,0,39,0,89,0,100,0,216,0,223,0,210,1,226,1,221,3,238,3,209,0,85,0,41,0,86,0,88,0,98,0,206,0,224,0,226,0,218,1,212,3,227,3,235,7,201,1,94,0,90,0,92,0,99,0,202,0,218,0,199,1,202,1,224,1,219,3,232,3,236,7,227,1,210,0,203,0,208,0,215,0,219,0,198,1,213,1,216,1,202,3,218,3,234,7,241,7,225,1,212,0,207,0,214,0,222,0,225,0,208,1,214,1,209,3,213,3,242,3,238,7,251,7,233,3,205,1,200,1,203,1,209,1,215,1,223,1,207,3,224,3,239,3,230,7,248,7,250,15,235,3,221,1,211,1,217,1,219,1,210,3,204,3,220,3,234,3,237,7,243,7,249,7,249,15,242,7,206,3,228,1,203,3,216,3,214,3,226,3,229,3,232,7,244,7,245,7,247,7,251,15,250,7,236,3,223,3,225,3,228,3,230,3,240,3,233,7,239,7,248,15,254,15,252,15,255,15,0,0,0,0,0,0,248,7,241,1,253,7,245,3,104,0,240,3,247,7,236,1,245,7,241,3,114,0,244,3,116,0,17,0,118,0,235,1,108,0,246,3,252,7,225,1,241,7,240,1,97,0,246,1,242,7,234,1,251,7,242,1,105,0,237,1,119,0,23,0,111,0,230,1,100,0,229,1,103,0,21,0,98,0,18,0,0,0,20,0,101,0,22,0,109,0,233,1,99,0,228,1,107,0,19,0,113,0,227,1,112,0,243,1,254,7,231,1,243,7,239,1,96,0,238,1,240,7,226,1,250,7,243,3,106,0,232,1,117,0,16,0,115,0,244,1,110,0,247,3,246,7,224,1,249,7,242,3,102,0,245,1,255,7,247,1,244,7,0,0,0,0,0,0,28,0,0,0,97,118,99,49,13,0,0,0,109,112,52,118,2,80,1,0,109,112,52,97,16,80,1,0,97,108,97,99,3,80,1,0,97,99,45,51,5,112,1,0,116,120,51,103,5,112,1,0,116,101,120,116,0,0,0,0,0,0,0,0,1,80,1,0,46,109,112,51,2,80,1,0,109,112,52,97,28,0,0,0,97,118,99,49,93,0,0,0,86,80,54,70,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,128,154,25,0,200,93,26,0,2,0,0,0,2,0,0,0,0,0,0,0,112,230,25,0,152,144,25,0,2,0,0,0,3,0,0,0,0,0,0,0,216,83,25,0,128,16,25,0,2,0,0,0,4,0,0,0,0,0,0,0,160,227,24,0,16,203,24,0,2,0,0,0,5,0,0,0,0,0,0,0,40,179,24,0,144,161,24,0,2,0,0,0,6,0,0,0,0,0,0,0,248,149,26,0,0,129,26,0,2,0,0,0,7,0,0,0,0,0,0,0,248,109,26,0,80,91,26,0,2,0,0,0,8,0,0,0,0,0,0,0,192,74,26,0,16,60,26,0,3,0,0,0,9,0,0,0,0,0,0,0,192,41,26,0,16,32,26,0,3,0,0,0,10,0,0,0,0,0,0,0,24,18,26,0,24,10,26,0,5,0,0,0,11,0,0,0,0,0,0,0,80,2,26,0,208,249,25,0,3,0,0,0,12,0,0,0,0,0,0,0,104,238,25,0,224,228,25,0,7,0,0,0,13,0,0,0,0,0,0,0,96,220,25,0,80,206,25,0,2,0,0,0,14,0,0,0,0,0,0,0,216,197,25,0,16,187,25,0,5,0,0,0,15,0,0,0,0,0,0,0,200,180,25,0,56,173,25,0,2,0,0,0,16,0,0,0,0,0,0,0,120,165,25,0,80,158,25,0,2,0,0,0,17,0,0,0,0,0,0,0,112,150,25,0,48,143,25,0,2,0,0,0,18,0,0,0,0,0,0,0,112,136,25,0,128,128,25,0,2,0,0,0,19,0,0,0,0,0,0,0,216,121,25,0,160,115,25,0,2,0,0,0,20,0,0,0,0,0,0,0,160,110,25,0,112,105,25,0,2,0,0,0,21,0,0,0,0,0,0,0,200,99,25,0,8,94,25,0,2,0,0,0,22,0,0,0,0,0,0,0,96,88,25,0,248,82,25,0,2,0,0,0,23,0,0,0,0,0,0,0,64,78,25,0,176,71,25,0,2,0,0,0,24,0,0,0,0,0,0,0,72,63,25,0,56,58,25,0,2,0,0,0,25,0,0,0,0,0,0,0,104,50,25,0,192,44,25,0,3,0,0,0,26,0,0,0,0,0,0,0,224,37,25,0,96,28,25,0,5,0,0,0,27,0,0,0,0,0,0,0,24,22,25,0,168,15,25,0,3,0,0,0,28,0,0,0,0,0,0,0,128,5,25,0,232,253,24,0,6,0,0,0,29,0,0,0,0,0,0,0,176,246,24,0,48,244,24,0,2,0,0,0,30,0,0,0,0,0,0,0,232,241,24,0,24,239,24,0,2,0,0,0,31,0,0,0,0,0,0,0,144,236,24,0,176,233,24,0,2,0,0,0,32,0,0,0,0,0,0,0,224,229,24,0,24,227,24,0,3,0,0,0,33,0,0,0,0,0,0,0,32,225,24,0,216,221,24,0,3,0,0,0,34,0,0,0,0,0,0,0,24,219,24,0,232,216,24,0,4,0,0,0,35,0,0,0,0,0,0,0,72,215,24,0,8,213,24,0,2,0,0,0,36,0,0,0,0,0,0,0,0,211,24,0,176,207,24,0,3,0,0,0,37,0,0,0,0,0,0,0,176,204,24,0,192,201,24,0,3,0,0,0,38,0,0,0,0,0,0,0,48,200,24,0,168,197,24,0,3,0,0,0,39,0,0,0,0,0,0,0,8,195,24,0,96,193,24,0,2,0,0,0,40,0,0,0,0,0,0,0,0,192,24,0,136,190,24,0,2,0,0,0,41,0,0,0,0,0,0,0,24,189,24,0,128,182,24,0,2,0,0,0,42,0,0,0,0,0,0,0,48,180,24,0,168,178,24,0,2,0,0,0,43,0,0,0,0,0,0,0,96,177,24,0,152,175,24,0,2,0,0,0,44,0,0,0,0,0,0,0,136,173,24,0,32,172,24,0,2,0,0,0,45,0,0,0,0,0,0,0,216,170,24,0,136,169,24,0,2,0,0,0,46,0,0,0,0,0,0,0,120,168,24,0,88,166,24,0,4,0,0,0,47,0,0,0,0,0,0,0,208,163,24,0,40,161,24,0,2,0,0,0,48,0,0,0,0,0,0,0,152,159,24,0,200,156,24,0,2,0,0,0,49,0,0,0,0,0,0,0,224,154,24,0,120,153,24,0,5,0,0,0,50,0,0,0,0,0,0,0,24,152,24,0,56,151,24,0,2,0,0,0,51,0,0,0,0,0,0,0,152,160,26,0,160,158,26,0,4,0,0,0,52,0,0,0,0,0,0,0,184,155,26,0,152,149,26,0,2,0,0,0,53,0,0,0,0,0,0,0,216,143,26,0,16,141,26,0,2,0,0,0,54,0,0,0,0,0,0,0,168,138,26,0,136,137,26,0,5,0,0,0,55,0,0,0,0,0,0,0,120,136,26,0,136,135,26,0,5,0,0,0,56,0,0,0,0,0,0,0,88,134,26,0,192,132,26,0,4,0,0,0,87,79,78,83,0,0,0,0,152,130,26,0,160,128,26,0,6,0,0,0,57,0,0,0,0,0,0,0,136,126,26,0,80,123,26,0,4,0,0,0,58,0,0,0,0,0,0,0,40,121,26,0,16,120,26,0,2,0,0,0,59,0,0,0,0,0,0,0,240,118,26,0,168,117,26,0,5,0,0,0,60,0,0,0,0,0,0,0,48,116,26,0,24,115,26,0,3,0,0,0,61,0,0,0,0,0,0,0,24,112,26,0,152,109,26,0,2,0,0,0,62,0,0,0,0,0,0,0,48,107,26,0,48,104,26,0,4,0,0,0,63,0,0,0,0,0,0,0,64,102,26,0,112,101,26,0,5,0,0,0,64,0,0,0,0,0,0,0,184,99,26,0,72,98,26,0,5,0,0,0,65,0,0,0,0,0,0,0,96,97,26,0,56,95,26,0,5,0,0,0,66,0,0,0,0,0,0,0,56,92,26,0,224,90,26,0,5,0,0,0,67,0,0,0,0,0,0,0,72,89,26,0,24,87,26,0,5,0,0,0,68,0,0,0,0,0,0,0,192,84,26,0,216,83,26,0,5,0,0,0,69,0,0,0,0,0,0,0,200,82,26,0,224,80,26,0,2,0,0,0,70,0,0,0,0,0,0,0,16,80,26,0,160,78,26,0,2,0,0,0,71,0,0,0,0,0,0,0,32,76,26,0,136,74,26,0,2,0,0,0,72,0,0,0,0,0,0,0,248,72,26,0,88,70,26,0,2,0,0,0,73,0,0,0,0,0,0,0,16,69,26,0,72,68,26,0,5,0,0,0,74,0,0,0,0,0,0,0,64,67,26,0,144,66,26,0,3,0,0,0,75,0,0,0,0,0,0,0,112,65,26,0,128,64,26,0,4,0,0,0,76,0,0,0,0,0,0,0,200,61,26,0,208,59,26,0,2,0,0,0,77,0,0,0,0,0,0,0,160,57,26,0,176,54,26,0,4,0,0,0,78,0,0,0,0,0,0,0,16,53,26,0,64,52,26,0,2,0,0,0,79,0,0,0,0,0,0,0,128,51,26,0,192,50,26,0,5,0,0,0,80,0,0,0,0,0,0,0,176,49,26,0,208,48,26,0,4,0,0,0,81,0,0,0,0,0,0,0,208,46,26,0,160,41,26,0,2,0,0,0,82,0,0,0,0,0,0,0,144,40,26,0,48,39,26,0,4,0,0,0,83,0,0,0,0,0,0,0,48,38,26,0,176,37,26,0,2,0,0,0,84,0,0,0,0,0,0,0,16,37,26,0,136,36,26,0,2,0,0,0,85,0,0,0,0,0,0,0,232,35,26,0,112,34,26,0,2,0,0,0,86,0,0,0,0,0,0,0,8,33,26,0,176,31,26,0,2,0,0,0,87,0,0,0,0,0,0,0,208,30,26,0,40,29,26,0,4,0,0,0,88,0,0,0,0,0,0,0,56,28,26,0,80,27,26,0,2,0,0,0,89,0,0,0,0,0,0,0,232,21,26,0,96,21,26,0,7,0,0,0,90,0,0,0,0,0,0,0,16,21,26,0,24,20,26,0,4,0,0,0,91,0,0,0,0,0,0,0,240,18,26,0,16,18,26,0,2,0,0,0,92,0,0,0,0,0,0,0,128,17,26,0,8,16,26,0,2,0,0,0,93,0,0,0,0,0,0,0,32,15,26,0,192,14,26,0,2,0,0,0,94,0,0,0,0,0,0,0,240,13,26,0,160,13,26,0,5,0,0,0,95,0,0,0,0,0,0,0,40,13,26,0,0,12,26,0,2,0,0,0,96,0,0,0,0,0,0,0,224,10,26,0,248,9,26,0,2,0,0,0,97,0,0,0,0,0,0,0,24,9,26,0,56,7,26,0,5,0,0,0,98,0,0,0,0,0,0,0,136,6,26,0,48,6,26,0,4,0,0,0,99,0,0,0,0,0,0,0,176,5,26,0,88,5,26,0,4,0,0,0,100,0,0,0,0,0,0,0,208,4,26,0,8,4,26,0,3,0,0,0,101,0,0,0,0,0,0,0,24,3,26,0,32,2,26,0,3,0,0,0,102,0,0,0,0,0,0,0,200,1,26,0,112,0,26,0,5,0,0,0,82,73,71,83,0,0,0,0,168,255,25,0,88,255,25,0,5,0,0,0,103,0,0,0,0,0,0,0,8,255,25,0,168,254,25,0,2,0,0,0,104,0,0,0,0,0,0,0,104,254,25,0,216,251,25,0,2,0,0,0,105,0,0,0,0,0,0,0,152,250,25,0,176,249,25,0,3,0,0,0,106,0,0,0,0,0,0,0,224,248,25,0,232,246,25,0,3,0,0,0,107,0,0,0,0,0,0,0,216,245,25,0,48,245,25,0,2,0,0,0,108,0,0,0,0,0,0,0,192,244,25,0,48,244,25,0,3,0,0,0,109,0,0,0,0,0,0,0,200,243,25,0,192,241,25,0,2,0,0,0,110,0,0,0,0,0,0,0,216,239,25,0,168,237,25,0,5,0,0,0,111,0,0,0,0,0,0,0,48,236,25,0,224,234,25,0,5,0,0,0,112,0,0,0,0,0,0,0,72,234,25,0,192,233,25,0,2,0,0,0,113,0,0,0,0,0,0,0,160,232,25,0,8,232,25,0,2,0,0,0,114,0,0,0,0,0,0,0,200,231,25,0,32,231,25,0,2,0,0,0,115,0,0,0,0,0,0,0,184,229,25,0,208,228,25,0,3,0,0,0,116,0,0,0,0,0,0,0,0,228,25,0,232,226,25,0,2,0,0,0,117,0,0,0,0,0,0,0,224,225,25,0,96,225,25,0,6,0,0,0,118,0,0,0,0,0,0,0,208,224,25,0,40,224,25,0,2,0,0,0,119,0,0,0,0,0,0,0,144,223,25,0,144,222,25,0,2,0,0,0,120,0,0,0,0,0,0,0,248,220,25,0,24,220,25,0,2,0,0,0,121,0,0,0,0,0,0,0,248,218,25,0,176,217,25,0,2,0,0,0,122,0,0,0,0,0,0,0,200,216,25,0,64,216,25,0,2,0,0,0,123,0,0,0,0,0,0,0,216,214,25,0,80,214,25,0,2,0,0,0,124,0,0,0,0,0,0,0,56,213,25,0,56,212,25,0,3,0,0,0,125,0,0,0,0,0,0,0,80,207,25,0,16,206,25,0,3,0,0,0,126,0,0,0,0,0,0,0,56,205,25,0,0,0,0,0,5,0,0,0,127,0,0,0,0,0,0,0,184,203,25,0,224,202,25,0,3,0,0,0,128,0,0,0,0,0,0,0,32,202,25,0,184,201,25,0,5,0,0,0,129,0,0,0,0,0,0,0,240,200,25,0,128,200,25,0,5,0,0,0,130,0,0,0,0,0,0,0,184,199,25,0,128,198,25,0,2,0,0,0,131,0,0,0,0,0,0,0,192,197,25,0,248,196,25,0,5,0,0,0,132,0,0,0,0,0,0,0,208,191,25,0,192,190,25,0,2,0,0,0,133,0,0,0,0,0,0,0,120,190,25,0,56,190,25,0,2,0,0,0,134,0,0,0,0,0,0,0,16,190,25,0,160,189,25,0,5,0,0,0,135,0,0,0,0,0,0,0,8,189,25,0,0,188,25,0,2,0,0,0,136,0,0,0,0,0,0,0,248,186,25,0,136,186,25,0,2,0,0,0,137,0,0,0,0,0,0,0,128,185,25,0,224,184,25,0,2,0,0,0,138,0,0,0,0,0,0,0,112,184,25,0,216,183,25,0,2,0,0,0,139,0,0,0,0,0,0,0,128,183,25,0,40,183,25,0,2,0,0,0,140,0,0,0,0,0,0,0,168,182,25,0,72,181,25,0,2,0,0,0,141,0,0,0,0,0,0,0,176,180,25,0,16,179,25,0,2,0,0,0,169,0,0,0,0,0,0,0,216,177,25,0,208,176,25,0,2,0,0,0,80,66,69,87,0,0,0,0,152,176,25,0,80,176,25,0,2,0,0,0,142,0,0,0,0,0,0,0,208,175,25,0,96,175,25,0,3,0,0,0,143,0,0,0,0,0,0,0,120,174,25,0,200,173,25,0,2,0,0,0,144,0,0,0,0,0,0,0,32,173,25,0,248,171,25,0,3,0,0,0,145,0,0,0,0,0,0,0,64,171,25,0,48,170,25,0,3,0,0,0,146,0,0,0,0,0,0,0,248,169,25,0,152,169,25,0,5,0,0,0,49,67,86,77,0,0,0,0,120,169,25,0,16,168,25,0,3,0,0,0,50,67,86,77,0,0,0,0,88,167,25,0,40,166,25,0,3,0,0,0,147,0,0,0,0,0,0,0,104,165,25,0,184,164,25,0,2,0,0,0,148,0,0,0,0,0,0,0,168,163,25,0,176,162,25,0,5,0,0,0,149,0,0,0,0,0,0,0,112,162,25,0,8,162,25,0,3,0,0,0,150,0,0,0,0,0,0,0,232,161,25,0,112,161,25,0,3,0,0,0,151,0,0,0,0,0,0,0,24,160,25,0,88,159,25,0,2,0,0,0,152,0,0,0,0,0,0,0,64,158,25,0,72,157,25,0,2,0,0,0,153,0,0,0,0,0,0,0,32,156,25,0,184,154,25,0,2,0,0,0,154,0,0,0,0,0,0,0,96,154,25,0,8,154,25,0,5,0,0,0,155,0,0,0,0,0,0,0,192,153,25,0,104,153,25,0,4,0,0,0,156,0,0,0,0,0,0,0,152,152,25,0,104,151,25,0,5,0,0,0,157,0,0,0,0,0,0,0,64,150,25,0,32,149,25,0,5,0,0,0,158,0,0,0,0,0,0,0,80,148,25,0,64,147,25,0,5,0,0,0,159,0,0,0,0,0,0,0,16,147,25,0,120,146,25,0,5,0,0,0,160,0,0,0,0,0,0,0,248,145,25,0,192,145,25,0,3,0,0,0,161,0,0,0,0,0,0,0,240,144,25,0,88,144,25,0,5,0,0,0,162,0,0,0,0,0,0,0,32,143,25,0,176,142,25,0,4,0,0,0,163,0,0,0,0,0,0,0,136,141,25,0,208,140,25,0,2,0,0,0,164,0,0,0,0,0,0,0,104,140,25,0,248,139,25,0,2,0,0,0,165,0,0,0,0,0,0,0,24,139,25,0,152,138,25,0,2,0,0,0,166,0,0,0,0,0,0,0,16,138,25,0,112,137,25,0,2,0,0,0,167,0,0,0,0,0,0,0,104,136,25,0,184,135,25,0,5,0,0,0,168,0,0,0,0,0,0,0,168,134,25,0,224,133,25,0,3,0,0,0,170,0,0,0,0,0,0,0,152,133,25,0,176,132,25,0,3,0,0,0,80,49,52,89,0,0,0,0,80,132,25,0,160,131,25,0,1,0,0,0,48,51,49,69,0,0,0,0,184,130,25,0,64,130,25,0,2,0,0,0,82,88,69,48,0,0,0,0,120,128,25,0,232,127,25,0,7,0,0,0,80,82,86,65,0,0,0,0,168,126,25,0,224,125,25,0,1,0,0,0,86,50,49,48,0,0,0,0,152,125,25,0,184,201,25,0,1,0,0,0,73,85,86,65,0,0,0,0,24,125,25,0,40,124,25,0,1,0,0,0,86,85,89,65,0,0,0,0,168,123,25,0,56,123,25,0,1,0,0,0,54,49,50,84,0,0,0,0,120,122,25,0,184,121,25,0,1,0,0,0,56,48,51,86,0,0,0,0,248,120,25,0,168,119,25,0,1,0,0,0,56,48,52,86,0,0,0,0,208,118,25,0,128,118,25,0,1,0,0,0,52,86,85,89,0,0,0,0,40,118,25,0,192,117,25,0,1,0,0,0,77,78,65,83,0,0,0,0,136,117,25,0,112,116,25,0,2,0,0,0,86,70,65,80,0,0,0,0,32,116,25,0,112,115,25,0,2,0,0,0,110,82,86,65,0,0,0,0,216,114,25,0,16,114,25,0,0,0,0,0,65,73,80,67,0,0,0,0,88,113,25,0,200,112,25,0,0,0,0,0,67,65,70,88,0,0,0,0,56,112,25,0,24,112,25,0,3,0,0,0,88,73,80,66,0,0,0,0,216,111,25,0,136,111,25,0,5,0,0,0,74,86,77,83,0,0,0,0,0,111,25,0,136,110,25,0,0,0,0,0,77,50,71,0,0,0,0,0,8,110,25,0,40,109,25,0,2,0,0,0,0,0,1,0,1,0,0,0,144,108,25,0,72,108,25,0,4,0,0,0,1,0,1,0,1,0,0,0,248,107,25,0,200,107,25,0,4,0,0,0,2,0,1,0,1,0,0,0,64,107,25,0,224,106,25,0,4,0,0,0,3,0,1,0,1,0,0,0,128,106,25,0,80,105,25,0,4,0,0,0,4,0,1,0,1,0,0,0,216,104,25,0,104,103,25,0,4,0,0,0,5,0,1,0,1,0,0,0,224,102,25,0,96,102,25,0,4,0,0,0,6,0,1,0,1,0,0,0,0,102,25,0,192,101,25,0,2,0,0,0,7,0,1,0,1,0,0,0,96,101,25,0,224,100,25,0,2,0,0,0,8,0,1,0,1,0,0,0,88,100,25,0,168,99,25,0,4,0,0,0,9,0,1,0,1,0,0,0,104,99,25,0,32,98,25,0,4,0,0,0,10,0,1,0,1,0,0,0,144,97,25,0,88,97,25,0,4,0,0,0,11,0,1,0,1,0,0,0,232,96,25,0,176,96,25,0,4,0,0,0,12,0,1,0,1,0,0,0,96,96,25,0,40,95,25,0,4,0,0,0,13,0,1,0,1,0,0,0,80,94,25,0,232,93,25,0,4,0,0,0,14,0,1,0,1,0,0,0,88,93,25,0,24,92,25,0,4,0,0,0,15,0,1,0,1,0,0,0,120,91,25,0,16,91,25,0,4,0,0,0,16,0,1,0,1,0,0,0,184,90,25,0,72,90,25,0,4,0,0,0,17,0,1,0,1,0,0,0,216,89,25,0,112,89,25,0,2,0,0,0,16,80,83,80,1,0,0,0,200,88,25,0,56,88,25,0,4,0,0,0,18,0,1,0,1,0,0,0,112,87,25,0,176,86,25,0,4,0,0,0,80,83,80,24,1,0,0,0,16,86,25,0,216,85,25,0,4,0,0,0,80,83,80,32,1,0,0,0,56,85,25,0,200,84,25,0,4,0,0,0,19,0,1,0,1,0,0,0,136,84,25,0,48,84,25,0,4,0,0,0,20,0,1,0,1,0,0,0,136,83,25,0,208,82,25,0,4,0,0,0,21,0,1,0,1,0,0,0,128,82,25,0,208,81,25,0,4,0,0,0,22,0,1,0,1,0,0,0,56,81,25,0,240,80,25,0,4,0,0,0,23,0,1,0,1,0,0,0,152,80,25,0,200,79,25,0,4,0,0,0,24,0,1,0,1,0,0,0,104,79,25,0,8,79,25,0,4,0,0,0,25,0,1,0,1,0,0,0,120,78,25,0,24,78,25,0,4,0,0,0,26,0,1,0,1,0,0,0,208,77,25,0,64,77,25,0,4,0,0,0,27,0,1,0,1,0,0,0,136,76,25,0,64,76,25,0,4,0,0,0,0,16,1,0,1,0,0,0,112,75,25,0,48,75,25,0,2,0,0,0,1,16,1,0,1,0,0,0,80,74,25,0,248,73,25,0,2,0,0,0,2,16,1,0,1,0,0,0,96,73,25,0,152,71,25,0,2,0,0,0,3,16,1,0,1,0,0,0,16,70,25,0,8,69,25,0,2,0,0,0,4,16,1,0,1,0,0,0,232,67,25,0,48,67,25,0,2,0,0,0,5,16,1,0,1,0,0,0,152,66,25,0,144,65,25,0,2,0,0,0,6,16,1,0,1,0,0,0,40,65,25,0,144,64,25,0,2,0,0,0,7,16,1,0,1,0,0,0,216,63,25,0,56,63,25,0,2,0,0,0,8,16,1,0,1,0,0,0,232,62,25,0,16,62,25,0,2,0,0,0,9,16,1,0,1,0,0,0,88,61,25,0,32,61,25,0,2,0,0,0,10,16,1,0,1,0,0,0,248,60,25,0,160,60,25,0,2,0,0,0,11,16,1,0,1,0,0,0,104,60,25,0,32,60,25,0,2,0,0,0,12,16,1,0,1,0,0,0,104,59,25,0,24,58,25,0,2,0,0,0,13,16,1,0,1,0,0,0,104,57,25,0,128,56,25,0,2,0,0,0,14,16,1,0,1,0,0,0,248,54,25,0,112,54,25,0,2,0,0,0,15,16,1,0,1,0,0,0,200,53,25,0,96,52,25,0,2,0,0,0,16,16,1,0,1,0,0,0,32,52,25,0,184,51,25,0,2,0,0,0,17,16,1,0,1,0,0,0,0,51,25,0,72,50,25,0,2,0,0,0,18,16,1,0,1,0,0,0,8,50,25,0,56,49,25,0,2,0,0,0,19,16,1,0,1,0,0,0,24,48,25,0,136,47,25,0,2,0,0,0,20,16,1,0,1,0,0,0,64,47,25,0,160,46,25,0,2,0,0,0,21,16,1,0,1,0,0,0,88,46,25,0,224,45,25,0,2,0,0,0,22,16,1,0,1,0,0,0,48,45,25,0,160,44,25,0,2,0,0,0,23,16,1,0,1,0,0,0,176,43,25,0,16,43,25,0,2,0,0,0,24,16,1,0,1,0,0,0,24,42,25,0,152,41,25,0,2,0,0,0,25,16,1,0,1,0,0,0,16,41,25,0,136,40,25,0,2,0,0,0,26,16,1,0,1,0,0,0,232,39,25,0,48,39,25,0,2,0,0,0,27,16,1,0,1,0,0,0,104,38,25,0,200,37,25,0,2,0,0,0,28,16,1,0,1,0,0,0,88,37,25,0,240,36,25,0,2,0,0,0,29,16,1,0,1,0,0,0,8,32,25,0,176,31,25,0,2,0,0,0,32,67,70,65,1,0,0,0,120,31,25,0,56,31,25,0,2,0,0,0,32,73,75,79,1,0,0,0,144,30,25,0,200,29,25,0,2,0,0,0,32,75,84,68,1,0,0,0,24,29,25,0,8,28,25,0,2,0,0,0,32,68,65,82,1,0,0,0,144,27,25,0,232,26,25,0,2,0,0,0,0,32,1,0,1,0,0,0,40,26,25,0,112,25,25,0,2,0,0,0,1,32,1,0,1,0,0,0,24,25,25,0,160,24,25,0,2,0,0,0,0,48,1,0,1,0,0,0,48,24,25,0,216,23,25,0,2,0,0,0,1,48,1,0,1,0,0,0,208,22,25,0,0,22,25,0,2,0,0,0,0,64,1,0,1,0,0,0,216,20,25,0,224,19,25,0,2,0,0,0,1,64,1,0,1,0,0,0,56,19,25,0,208,18,25,0,2,0,0,0,2,64,1,0,1,0,0,0,8,18,25,0,112,17,25,0,2,0,0,0,3,64,1,0,1,0,0,0,32,17,25,0,208,16,25,0,2,0,0,0,0,80,1,0,1,0,0,0,56,16,25,0,136,15,25,0,2,0,0,0,1,80,1,0,1,0,0,0,176,10,25,0,8,10,25,0,2,0,0,0,2,80,1,0,1,0,0,0,48,9,25,0,208,8,25,0,2,0,0,0,3,80,1,0,1,0,0,0,40,8,25,0,120,7,25,0,2,0,0,0,4,80,1,0,1,0,0,0,32,7,25,0,160,6,25,0,6,0,0,0,5,80,1,0,1,0,0,0,224,5,25,0,120,5,25,0,2,0,0,0,6,80,1,0,1,0,0,0,8,5,25,0,0,0,0,0,2,0,0,0,7,80,1,0,1,0,0,0,16,4,25,0,72,3,25,0,2,0,0,0,8,80,1,0,1,0,0,0,144,2,25,0,40,1,25,0,2,0,0,0,9,80,1,0,1,0,0,0,248,0,25,0,40,0,25,0,2,0,0,0,10,80,1,0,1,0,0,0,192,255,24,0,192,254,24,0,2,0,0,0,11,80,1,0,1,0,0,0,216,253,24,0,48,253,24,0,2,0,0,0,12,80,1,0,1,0,0,0,184,252,24,0,64,252,24,0,4,0,0,0,13,80,1,0,1,0,0,0,160,251,24,0,64,248,24,0,2,0,0,0,14,80,1,0,1,0,0,0,16,248,24,0,184,247,24,0,2,0,0,0,15,80,1,0,1,0,0,0,120,247,24,0,0,247,24,0,4,0,0,0,16,80,1,0,1,0,0,0,192,246,24,0,128,246,24,0,4,0,0,0,17,80,1,0,1,0,0,0,16,246,24,0,120,245,24,0,2,0,0,0,18,80,1,0,1,0,0,0,64,245,24,0,56,245,24,0,2,0,0,0,19,80,1,0,1,0,0,0,48,245,24,0,216,244,24,0,2,0,0,0,20,80,1,0,1,0,0,0,208,244,24,0,120,244,24,0,2,0,0,0,21,80,1,0,1,0,0,0,64,244,24,0,240,243,24,0,2,0,0,0,22,80,1,0,1,0,0,0,192,243,24,0,120,243,24,0,4,0,0,0,23,80,1,0,1,0,0,0,48,243,24,0,16,243,24,0,2,0,0,0,24,80,1,0,1,0,0,0,0,243,24,0,208,242,24,0,2,0,0,0,25,80,1,0,1,0,0,0,200,242,24,0,120,242,24,0,6,0,0,0,26,80,1,0,1,0,0,0,240,241,24,0,168,241,24,0,2,0,0,0,27,80,1,0,1,0,0,0,40,241,24,0,224,240,24,0,2,0,0,0,28,80,1,0,1,0,0,0,144,240,24,0,128,240,24,0,2,0,0,0,29,80,1,0,1,0,0,0,104,240,24,0,24,240,24,0,4,0,0,0,30,80,1,0,1,0,0,0,16,240,24,0,184,239,24,0,2,0,0,0,31,80,1,0,1,0,0,0,32,239,24,0,168,238,24,0,2,0,0,0,32,80,1,0,1,0,0,0,32,238,24,0,200,237,24,0,2,0,0,0,33,80,1,0,1,0,0,0,144,237,24,0,128,237,24,0,4,0,0,0,34,80,1,0,1,0,0,0,104,237,24,0,8,237,24,0,2,0,0,0,35,80,1,0,1,0,0,0,248,236,24,0,200,236,24,0,2,0,0,0,36,80,1,0,1,0,0,0,152,236,24,0,128,236,24,0,2,0,0,0,37,80,1,0,1,0,0,0,72,236,24,0,0,236,24,0,2,0,0,0,38,80,1,0,1,0,0,0,208,235,24,0])
.concat([168,235,24,0,2,0,0,0,39,80,1,0,1,0,0,0,144,235,24,0,104,235,24,0,4,0,0,0,40,80,1,0,1,0,0,0,192,234,24,0,48,234,24,0,2,0,0,0,41,80,1,0,1,0,0,0,184,233,24,0,64,233,24,0,2,0,0,0,42,80,1,0,1,0,0,0,128,232,24,0,40,232,24,0,2,0,0,0,43,80,1,0,1,0,0,0,152,231,24,0,112,231,24,0,2,0,0,0,44,80,1,0,1,0,0,0,88,231,24,0,48,231,24,0,2,0,0,0,45,80,1,0,1,0,0,0,40,231,24,0,144,230,24,0,4,0,0,0,46,80,1,0,1,0,0,0,232,229,24,0,160,229,24,0,4,0,0,0,47,80,1,0,1,0,0,0,32,229,24,0,176,228,24,0,2,0,0,0,48,80,1,0,1,0,0,0,112,228,24,0,48,228,24,0,2,0,0,0,49,80,1,0,1,0,0,0,8,228,24,0,192,227,24,0,2,0,0,0,50,80,1,0,1,0,0,0,168,227,24,0,96,227,24,0,2,0,0,0,51,80,1,0,1,0,0,0,32,227,24,0,8,227,24,0,2,0,0,0,52,80,1,0,1,0,0,0,224,226,24,0,144,226,24,0,2,0,0,0,53,80,1,0,1,0,0,0,96,226,24,0,64,226,24,0,2,0,0,0,54,80,1,0,1,0,0,0,240,225,24,0,168,225,24,0,2,0,0,0,55,80,1,0,1,0,0,0,144,225,24,0,104,225,24,0,2,0,0,0,56,80,1,0,1,0,0,0,40,225,24,0,16,225,24,0,2,0,0,0,57,80,1,0,1,0,0,0,176,224,24,0,112,224,24,0,2,0,0,0,58,80,1,0,1,0,0,0,32,224,24,0,168,223,24,0,4,0,0,0,59,80,1,0,1,0,0,0,112,223,24,0,248,222,24,0,2,0,0,0,60,80,1,0,1,0,0,0,208,222,24,0,144,222,24,0,2,0,0,0,65,77,73,86,1,0,0,0,224,221,24,0,136,221,24,0,2,0,0,0,83,87,70,70,1,0,0,0,56,221,24,0,224,220,24,0,0,0,0,0,67,78,79,83,1,0,0,0,128,220,24,0,112,220,24,0,0,0,0,0,76,78,79,83,1,0,0,0,56,220,24,0,232,219,24,0,0,0,0,0,65,70,65,80,1,0,0,0,192,219,24,0,88,219,24,0,2,0,0,0,83,85,80,79,1,0,0,0,32,219,24,0,240,218,24,0,2,0,0,0,62,80,1,0,1,0,0,0,144,218,24,0,40,218,24,0,2,0,0,0,75,97,66,116,1,0,0,0,248,217,24,0,208,217,24,0,4,0,0,0,99,118,101,115,1,0,0,0,192,217,24,0,144,217,24,0,2,0,0,0,118,109,115,115,1,0,0,0,0,111,25,0,112,217,24,0,2,0,0,0,0,112,1,0,3,0,0,0,80,217,24,0,0,217,24,0,0,0,1,0,1,112,1,0,3,0,0,0,216,216,24,0,152,216,24,0,0,0,1,0,2,112,1,0,3,0,0,0,128,216,24,0,48,216,24,0,0,0,2,0,3,112,1,0,3,0,0,0,40,216,24,0,8,216,24,0,0,0,1,0,32,83,83,65,3,0,0,0,248,215,24,0,216,215,24,0,0,0,2,0,4,112,1,0,3,0,0,0,184,215,24,0,80,215,24,0,0,0,2,0,5,112,1,0,3,0,0,0,56,215,24,0,224,214,24,0,0,0,2,0,6,112,1,0,3,0,0,0,160,214,24,0,64,214,24,0,0,0,1,0,7,112,1,0,3,0,0,0,48,214,24,0,32,214,24,0,0,0,0,0,8,112,1,0,3,0,0,0,0,214,24,0,216,213,24,0,0,0,2,0,112,105,82,83,3,0,0,0,160,213,24,0,24,213,24,0,0,0,2,0,68,86,68,109,3,0,0,0,248,212,24,0,152,212,24,0,0,0,2,0,50,76,80,77,3,0,0,0,112,212,24,0,40,212,24,0,0,0,2,0,56,48,54,99,3,0,0,0,32,212,24,0,8,212,24,0,0,0,0,0,66,85,83,74,3,0,0,0,224,211,24,0,200,211,24,0,0,0,2,0,83,74,104,80,3,0,0,0,160,211,24,0,8,211,24,0,0,0,2,0,73,77,65,83,3,0,0,0,248,210,24,0,144,210,24,0,0,0,2,0,84,88,84,82,3,0,0,0,88,210,24,0,8,210,24,0,0,0,2,0,49,86,98,83,3,0,0,0,248,209,24,0,224,209,24,0,0,0,2,0,86,98,117,83,3,0,0,0,176,209,24,0,48,209,24,0,0,0,2,0,114,108,80,86,3,0,0,0,120,208,24,0,192,207,24,0,0,0,2,0,84,84,86,87,3,0,0,0,144,207,24,0,64,207,24,0,0,0,2,0,84,88,84,66,0,0,0,0,248,206,24,0,80,206,24,0,1,0,0,0,78,73,66,88,0,0,0,0,56,206,24,0,8,206,24,0,1,0,0,0,70,68,73,0,0,0,0,0,224,205,24,0,208,205,24,0,1,0,0,0,65,86,76,75,2,0,0,0,104,205,24,0,184,204,24,0,0,0,0,0,86,65,78,68,2,0,0,0,104,204,24,0,0,204,24,0,0,0,0,0,0,0,0,0,13,0,0,0,77,52,83,50,13,0,0,0,77,80,52,83,17,0,0,0,77,80,52,51,0,0,0,0,0,0,0,0,5,0,0,0,115,50,54,51,28,0,0,0,97,118,99,49,13,0,0,0,109,112,52,118,2,80,1,0,109,112,52,97,0,32,1,0,115,97,109,114,1,32,1,0,115,97,119,98,5,112,1,0,116,120,51,103,0,0,0,0,0,0,0,0,0,0,16,1,32,1,48,1,64,1,80,1,96,1,112,1,128,1,144,1,160,1,176,1,192,1,1,17,17,2,33,2,49,2,65,2,81,2,97,2,113,2,129,2,145,2,161,2,177,2,193,2,2,17,18,2,34,2,50,2,66,2,82,2,98,2,114,2,130,2,146,2,162,2,178,2,194,2,3,17,19,2,35,2,51,2,67,2,83,2,99,2,115,2,131,2,147,2,163,2,179,2,195,2,4,17,20,2,36,2,52,2,68,2,84,2,100,2,116,2,132,2,148,2,164,2,180,2,196,2,5,17,21,2,37,2,53,2,69,2,85,2,101,2,117,2,133,2,149,2,165,2,181,2,197,2,6,17,22,2,38,2,54,2,70,2,86,2,102,2,118,2,134,2,150,2,166,2,182,2,198,2,7,17,23,2,39,2,55,2,71,2,87,2,103,2,119,2,135,2,151,2,167,2,183,2,199,2,8,17,24,2,40,2,56,2,72,2,88,2,104,2,120,2,136,2,152,2,168,2,184,2,200,2,9,17,25,2,41,2,57,2,73,2,89,2,105,2,121,2,137,2,153,2,169,2,185,2,201,2,10,17,26,2,42,2,58,2,74,2,90,2,106,2,122,2,138,2,154,2,170,2,186,2,202,2,11,17,27,2,43,2,59,2,75,2,91,2,107,2,123,2,139,2,155,2,171,2,187,2,203,2,12,17,28,2,44,2,60,2,76,2,92,2,108,2,124,2,140,2,156,2,172,2,188,2,204,2,0,0,0,0,0,0,0,0,16,1,32,1,48,1,64,1,80,1,96,1,112,1,1,17,17,2,33,2,49,2,65,2,81,2,97,2,113,2,2,17,18,2,34,2,50,2,66,2,82,2,98,2,114,2,3,17,19,2,35,2,51,2,67,2,83,2,99,2,115,2,4,17,20,2,36,2,52,2,68,2,84,2,100,2,116,2,5,17,21,2,37,2,53,2,69,2,85,2,101,2,117,2,6,17,22,2,38,2,54,2,70,2,86,2,102,2,118,2,7,17,23,2,39,2,55,2,71,2,87,2,103,2,119,2,245,47,203,192,186,116,138,192,24,69,33,192,0,0,128,191,0,0,0,0,0,0,128,63,24,69,33,64,186,116,138,64,245,47,203,64,0,0,0,0,0,0,16,0,32,0,48,0,64,0,80,0,96,0,112,0,128,0,1,0,17,0,33,0,49,0,65,0,81,0,97,0,113,0,129,0,2,0,18,0,34,0,50,0,66,0,82,0,98,0,114,0,130,0,3,0,19,0,35,0,51,0,67,0,83,0,99,0,115,0,131,0,4,0,20,0,36,0,52,0,68,0,84,0,100,0,116,0,132,0,5,0,21,0,37,0,53,0,69,0,85,0,101,0,117,0,133,0,6,0,22,0,38,0,54,0,70,0,86,0,102,0,118,0,134,0,7,0,23,0,39,0,55,0,71,0,87,0,103,0,119,0,135,0,8,0,24,0,40,0,56,0,72,0,88,0,104,0,120,0,136,0,0,0,0,0,0,0,0,0,0,0,0,0,128,63,24,69,33,64,186,116,138,64,245,47,203,64,79,204,8,65,142,113,46,65,144,63,86,65,0,0,128,65,27,196,149,65,211,90,172,65,211,181,195,65,255,200,219,65,239,137,244,65,205,247,6,66,4,249,19,66,0,0,16,16,32,16,48,16,64,16,80,16,96,16,112,16,128,16,144,16,160,16,176,16,192,16,208,16,224,16,240,16,0,18,1,16,17,32,33,32,49,32,65,32,81,32,97,32,113,32,129,32,145,32,161,32,177,32,193,32,209,32,225,32,241,32,1,34,2,16,18,32,34,32,50,32,66,32,82,32,98,32,114,32,130,32,146,32,162,32,178,32,194,32,210,32,226,32,242,32,2,34,3,16,19,32,35,32,51,32,67,32,83,32,99,32,115,32,131,32,147,32,163,32,179,32,195,32,211,32,227,32,243,32,3,34,4,16,20,32,36,32,52,32,68,32,84,32,100,32,116,32,132,32,148,32,164,32,180,32,196,32,212,32,228,32,244,32,4,34,5,16,21,32,37,32,53,32,69,32,85,32,101,32,117,32,133,32,149,32,165,32,181,32,197,32,213,32,229,32,245,32,5,34,6,16,22,32,38,32,54,32,70,32,86,32,102,32,118,32,134,32,150,32,166,32,182,32,198,32,214,32,230,32,246,32,6,34,7,16,23,32,39,32,55,32,71,32,87,32,103,32,119,32,135,32,151,32,167,32,183,32,199,32,215,32,231,32,247,32,7,34,8,16,24,32,40,32,56,32,72,32,88,32,104,32,120,32,136,32,152,32,168,32,184,32,200,32,216,32,232,32,248,32,8,34,9,16,25,32,41,32,57,32,73,32,89,32,105,32,121,32,137,32,153,32,169,32,185,32,201,32,217,32,233,32,249,32,9,34,10,16,26,32,42,32,58,32,74,32,90,32,106,32,122,32,138,32,154,32,170,32,186,32,202,32,218,32,234,32,250,32,10,34,11,16,27,32,43,32,59,32,75,32,91,32,107,32,123,32,139,32,155,32,171,32,187,32,203,32,219,32,235,32,251,32,11,34,12,16,28,32,44,32,60,32,76,32,92,32,108,32,124,32,140,32,156,32,172,32,188,32,204,32,220,32,236,32,252,32,12,34,13,16,29,32,45,32,61,32,77,32,93,32,109,32,125,32,141,32,157,32,173,32,189,32,205,32,221,32,237,32,253,32,13,34,14,16,30,32,46,32,62,32,78,32,94,32,110,32,126,32,142,32,158,32,174,32,190,32,206,32,222,32,238,32,254,32,14,34,15,16,31,32,47,32,63,32,79,32,95,32,111,32,127,32,143,32,159,32,175,32,191,32,207,32,223,32,239,32,255,32,15,34,0,17,16,33,32,33,48,33,64,33,80,33,96,33,112,33,128,33,144,33,160,33,176,33,192,33,208,33,224,33,240,33,0,35,0,0,0,0,0,0,0,0,128,191,0,0,0,0,0,0,128,63,0,0,0,0,0,0,64,129,128,129,16,65,80,194,144,194,32,65,96,194,160,194,4,33,68,162,132,162,20,98,84,227,148,227,36,98,100,227,164,227,8,33,72,162,136,162,24,98,88,227,152,227,40,98,104,227,168,227,1,17,65,146,129,146,17,82,81,211,145,211,33,82,97,211,161,211,5,50,69,179,133,179,21,115,85,244,149,244,37,115,101,244,165,244,9,50,73,179,137,179,25,115,89,244,153,244,41,115,105,244,169,244,2,17,66,146,130,146,18,82,82,211,146,211,34,82,98,211,162,211,6,50,70,179,134,179,22,115,86,244,150,244,38,115,102,244,166,244,10,50,74,179,138,179,26,115,90,244,154,244,42,115,106,244,170,244,0,0,0,0,0,0,154,153,25,191,195,245,8,191,195,245,168,190,164,112,61,190,92,143,194,189,158,239,39,189,30,167,104,188,179,123,114,187,24,0,0,0,25,0,0,0,30,0,0,0,50,0,0,0,60,0,0,0,0,0,0,0,117,116,102,56,0,0,200,119,99,104,97,114,116,0,0,198,117,99,115,50,0,117,99,115,50,98,101,0,0,196,117,99,115,50,108,101,0,0,197,117,116,102,49,54,0,117,116,102,49,54,98,101,0,0,194,117,116,102,49,54,108,101,0,0,193,117,99,115,52,0,117,99,115,52,98,101,0,117,116,102,51,50,0,117,116,102,51,50,98,101,0,0,192,117,99,115,52,108,101,0,117,116,102,51,50,108,101,0,0,195,97,115,99,105,105,0,117,115,97,115,99,105,105,0,105,115,111,54,52,54,0,105,115,111,54,52,54,117,115,0,0,199,101,117,99,106,112,0,0,208,115,104,105,102,116,106,105,115,0,115,106,105,115,0,0,209,103,98,49,56,48,51,48,0,0,216,103,98,107,0,0,217,103,98,50,51,49,50,0,0,218,98,105,103,53,0,98,105,103,102,105,118,101,0,99,112,57,53,48,0,98,105,103,53,104,107,115,99,115,0,0,224,101,117,99,107,114,0,107,115,99,53,54,48,49,0,107,115,120,49,48,48,49,0,99,112,57,52,57,0,0,232,105,115,111,56,56,53,57,49,0,108,97,116,105,110,49,0,0,128,105,115,111,56,56,53,57,50,0,0,32,0,84,0,137,18,0,32,1,6,0,0,152,65,134,26,127,0,48,72,32,0,88,32,201,18,0,36,17,198,35,0,156,81,198,26,128,80,66,136,32,90,0,0,192,4,0,16,113,1,0,29,0,112,2,0,41,0,0,192,7,33,48,1,5,0,0,88,1,0,0,94,208,1,128,29,0,0,128,6,0,91,0,0,0,5,0,20,129,1,0,30,0,128,2,0,42,0,0,0,8,34,52,17,5,0,0,92,1,0,0,95,212,1,192,29,0,0,144,70,36,105,115,111,56,56,53,57,51,0,0,32,0,212,0,9,0,0,4,48,3,0,0,244,64,70,11,63,0,16,64,32,0,216,0,0,0,0,0,64,3,0,0,248,80,134,11,64,0,16,128,32,0,0,0,64,0,0,108,144,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,188,0,0,0,43,0,0,0,0,0,200,33,6,0,0,0,0,64,0,0,112,160,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,192,0,0,0,44,0,0,0,0,0,204,49,70,36,105,115,111,56,56,53,57,52,0,0,32,0,84,48,4,23,0,220,96,4,0,0,152,49,66,12,108,0,48,8,0,0,88,32,73,23,0,224,112,196,35,0,156,65,130,12,109,72,65,200,20,17,0,0,0,0,0,0,0,192,14,29,0,112,2,0,37,0,0,64,14,33,56,65,69,16,0,0,0,0,0,0,224,1,0,0,0,184,1,7,0,18,0,0,0,0,0,0,0,0,15,30,0,128,2,0,38,0,0,128,14,34,60,81,133,16,0,0,0,0,0,0,228,1,0,0,0,188,17,71,36,105,115,111,56,56,53,57,53,0,0,32,0,136,51,14,57,229,152,115,14,58,233,168,179,14,59,237,0,224,206,59,240,196,35,207,60,244,212,99,207,61,248,228,163,207,62,252,244,227,207,63,0,5,36,208,64,4,21,100,208,65,8,37,164,208,66,12,53,228,208,67,16,69,36,209,68,20,85,100,209,69,24,101,164,209,70,28,117,228,209,71,32,133,36,210,72,36,149,100,210,73,40,165,164,210,74,44,181,228,210,75,53,194,20,147,76,51,209,84,147,77,55,225,148,147,78,59,13,192,83,79,105,115,111,56,56,53,57,54,0,0,32,0,4,16,64,0,0,4,16,64,0,1,4,16,64,0,115,1,16,64,0,1,4,16,64,0,1,4,16,64,0,1,4,16,0,93,1,4,16,64,93,1,216,117,23,94,121,233,181,23,95,125,249,245,23,96,129,9,54,24,97,133,25,118,24,98,137,41,182,24,99,141,57,246,88,0,1,4,16,64,0,144,69,38,217,100,148,85,102,217,101,152,101,166,217,102,156,117,230,217,103,160,133,38,90,0,1,4,16,64,0,1,4,16,64,0,1,4,16,64,0,105,115,111,56,56,53,57,55,0,0,32,0,144,88,34,0,51,210,8,0,0,0,0,160,9,0,0,0,16,128,136,0,0,0,0,0,155,112,210,9,0,158,124,2,10,0,161,0,32,202,40,164,148,98,202,41,168,164,162,202,42,172,180,226,202,43,176,196,34,203,44,180,212,18,128,45,183,224,146,139,46,187,240,210,139,47,191,0,19,140,48,195,16,83,140,49,199,32,147,140,50,203,48,211,140,51,207,64,19,141,52,211,80,83,141,53,215,96,147,141,54,219,112,211,141,55,223,128,19,78,0,105,115,111,56,56,53,57,56,0,0,32,0,4,0,0,0,0,0,0,0,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,0,0,0,0,0,64,0,1,4,16,64,0,1,4,16,64,0,1,4,16,64,0,1,4,16,64,0,1,4,16,64,0,1,4,16,64,0,1,4,16,64,0,1,4,16,192,136,83,81,85,149,85,87,97,149,149,86,91,113,213,149,87,95,129,21,150,88,99,145,85,150,89,103,161,149,150,90,107,177,213,86,0,1,120,248,97,0,105,115,111,56,56,53,57,57,0,0,80,45,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,244,64,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,80,6,0,105,115,111,56,56,53,57,49,48,0,0,32,0,84,48,66,12,57,220,16,4,0,70,132,96,6,27,131,0,0,135,20,0,88,64,130,12,58,224,32,4,0,71,136,112,70,27,132,136,24,199,20,17,0,0,0,0,0,0,0,192,14,29,0,112,2,0,37,0,0,0,0,0,56,65,5,0,0,0,0,128,27,0,224,1,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,0,15,30,0,128,2,0,38,0,0,0,0,0,60,81,5,0,0,0,0,192,27,0,228,1,0,0,0,0,0,192,16,105,115,111,56,56,53,57,49,49,0,116,105,115,54,50,48,0,0,32,0,188,6,91,108,178,205,70,91,109,182,221,134,91,110,186,237,198,91,111,190,253,6,92,112,194,13,71,92,113,198,29,135,92,114,202,45,199,92,115,206,61,7,93,116,210,77,71,93,117,214,93,135,93,118,218,109,199,93,119,222,125,7,94,120,226,141,71,94,121,230,157,135,94,0,1,4,16,64,122,234,173,199,94,123,238,189,7,95,124,242,205,71,95,125,246,221,135,95,126,250,237,199,95,127,254,253,7,96,128,2,14,72,96,129,1,4,16,64,0,105,115,111,56,56,53,57,49,51,0,0,32,0,160,8,0,0,0,164,8,0,0,13,0,192,5,0,0,0,0,192,2,0,0,0,0,0,39,2,0,0,0,16,0,208,5,0,0,0,0,128,3,21,236,16,193,5,0,0,112,194,8,29,0,240,71,9,49,4,145,131,17,102,48,225,4,0,84,0,0,0,0,120,40,1,6,28,0,4,50,8,0,22,240,32,1,6,0,0,128,2,9,30,0,0,136,9,50,8,161,195,17,103,52,241,4,0,85,0,0,0,0,121,44,17,70,28,0,8,66,72,137,105,115,111,56,56,53,57,49,52,0,0,32,0,24,120,32,0,27,112,128,32,0,20,2,96,97,130,26,2,0,128,31,10,46,248,2,12,12,54,8,128,131,21,62,120,33,132,27,98,152,97,132,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,122,0,0,0,0,0,0,0,128,132,0,0,0,0,0,0,0,192,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,123,0,0,0,0,0,0,0,192,132,0,0,0,0,0,0,0,208,7,0,105,115,111,56,56,53,57,49,53,0,108,97,116,105,110,57,0,0,36,51,2,96,6,0,103,0,0,0,0,0,0,0,0,0,0,0,0,0,0,131,0,0,0,0,132,0,0,0,0,88,100,225,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,105,115,111,56,56,53,57,49,54,0,0,32,0,84,96,129,18,51,166,104,6,0,103,0,160,8,0,127,0,0,72,32,0,0,208,193,18,131,160,8,0,0,132,120,176,8,0,88,100,225,135,32,0,0,0,192,4,0,92,0,0,0,0,0,0,0,0,0,0,0,0,0,33,48,1,0,0,0,88,1,0,24,118,0,0,0,0,0,156,192,8,0,0,0,0,0,5,0,96,0,0,0,0,0,0,0,0,0,0,0,0,0,34,52,1,0,0,0,92,1,64,24,119,0,0,0,0,0,160,208,8,0,99,112,49,50,53,48,0,119,105,110,100,111,119,115,49,50,53,48,0,0,0,51,6,96,98,0,41,182,168,226,138,1,184,104,198,139,96,168,49,200,31,1,144,88,226,137,40,178,8,98,136,1,216,120,6,140,97,172,65,8,32,0,60,2,137,18,0,84,0,0,0,0,0,64,6,0,0,0,0,64,32,0,0,32,201,18,0,0,0,0,0,0,88,80,6,0,72,80,146,132,32,90,0,0,192,4,0,16,113,1,0,29,0,112,2,0,41,0,0,192,7,33,48,1,5,0,0,88,1,0,0,94,208,1,128,29,0,0,128,6,0,91,0,0,0,5,0,20,129,1,0,30,0,128,2,0,42,0,0,0,8,34,52,17,5,0,0,92,1,0,0,95,212,1,192,29,0,0,144,70,36,99,112,49,50,53,49,0,119,105,110,100,111,119,115,49,50,53,49,0,0,0,227,144,99,162,76,41,182,168,226,138,51,186,168,206,139,235,180,195,206,59,49,145,88,226,137,40,178,8,98,136,1,216,136,19,140,57,237,164,83,79,0,184,195,83,58,0,248,4,0,0,226,0,80,14,0,0,0,0,0,58,0,0,112,78,77,63,1,0,0,0,48,213,56,19,0,55,153,67,147,77,240,196,35,207,60,244,212,99,207,61,248,228,163,207,62,252,244,227,207,63,0,5,36,208,64,4,21,100,208,65,8,37,164,208,66,12,53,228,208,67,16,69,36,209,68,20,85,100,209,69,24,101,164,209,70,28,117,228,209,71,32,133,36,210,72,36,149,100,210,73,40,165,164,210,74,44,181,228,210,75,99,112,49,50,53,50,0,119,105,110,100,111,119,115,49,50,53,50,0,0,0,51,6,96,98,33,41,182,168,226,138,142,184,104,198,139,88,4,48,72,0,1,144,88,226,137,40,178,8,98,136,147,216,120,6,140,89,4,64,136,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,112,49,50,53,51,0,119,105,110,100,111,119,115,49,50,53,51,0,0,0,51,6,96,98,33,41,182,168,226,138,1,184,24,192,139,1,4,16,64,0,1,144,88,226,137,40,178,8,98,136,1,216,24,0,140,1,4,16,64,0,0,112,210,9,0,0,0,0,0,0,0,0,16,0,0,0,0,0,128,136,0,0,0,0,0,155,0,0,0,0,158,124,2,10,0,161,0,32,202,40,164,148,98,202,41,168,164,162,202,42,172,180,226,202,43,176,196,34,203,44,180,212,18,128,45,183,224,146,139,46,187,240,210,139,47,191,0,19,140,48,195,16,83,140,49,199,32,147,140,50,203,48,211,140,51,207,64,19,141,52,211,80,83,141,53,215,96,147,141,54,219,112,211,141,55,223,128,19,78,0,99,112,49,50,53,52,0,119,105,110,100,111,119,115,49,50,53,52,0,0,0,51,6,96,98,33,41,182,168,226,138,142,184,104,198,139,88,4,16,64,0,1,144,88,226,137,40,178,8,98,136,147,216,120,6,140,89,4,16,128,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,45,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,244,64,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,80,6,0,99,112,49,50,53,53,0,119,105,110,100,111,119,115,49,50,53,53,0,0,0,51,6,96,98,33,41,182,168,226,138,142,184,24,192,139,1,4,16,64,0,1,144,88,226,137,40,178,8,98,136,147,216,24,0,140,1,4,16,64,0,0,0,0,0,0,49,2,0,0,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,0,0,0,0,0,0,0,64,5,37,212,80,68,21,101,212,81,72,37,21,128,82,75,49,213,148,83,79,65,21,149,84,110,189,5,87,92,114,5,16,64,0,1,4,16,64,0,83,81,85,149,85,87,97,149,149,86,91,113,213,149,87,95,129,21,150,88,99,145,85,150,89,103,161,149,150,90,107,177,213,86,0,1,120,248,97,0,99,112,49,50,53,54,0,119,105,110,100,111,119,115,49,50,53,54,0,0,0,51,146,102,98,33,41,182,168,226,138,142,184,56,218,139,88,148,134,154,105,170,145,88,226,137,40,178,8,98,136,169,217,120,26,140,89,112,216,225,106,0,204,5,0,0,0,0,0,0,0,0,0,192,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,23,0,0,0,0,64,93,173,217,117,23,94,121,233,181,23,95,125,249,245,23,96,129,9,54,24,97,133,25,118,24,98,137,41,182,24,0,140,53,230,216,99,144,69,38,217,100,0,80,6,64,101,150,93,134,25,0,0,0,0,0,0,153,105,6,0,0,155,113,214,153,103,0,124,6,26,0,161,1,32,26,0,0,120,248,161,107,99,112,49,50,53,55,0,119,105,110,100,111,119,115,49,50,53,55,0,0,0,51,6,96,98,0,41,182,168,226,138,1,184,24,192,139,1,16,240,136,2,1,144,88,226,137,40,178,8,98,136,1,216,24,0,140,1,24,32,73,0,0,4,0,0,0,0,4,0,0,0,13,0,192,5,0,0,0,0,192,2,0,0,0,0,0,0,0,0,0,0,16,0,208,5,0,0,0,0,128,3,21,236,16,193,5,0,0,112,194,8,29,0,240,71,9,49,4,145,131,17,102,48,225,4,0,84,0,0,0,0,120,40,1,6,28,0,4,50,8,0,22,240,32,1,6,0,0,128,2,9,30,0,0,136,9,50,8,161,195,17,103,52,241,4,0,85,0,0,0,0,121,44,17,70,28,0,8,66,72,36,99,112,49,50,53,56,0,119,105,110,100,111,119,115,49,50,53,56,0,0,0,51,6,96,98,33,41,182,168,226,138,142,184,24,192,139,88,4,16,64,0,1,144,88,226,137,40,178,8,98,136,147,216,24,0,140,89,4,16,128,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,4,0,0,0,0,0,0,0,0,0,0,149,0,0,0,0,33,0,128,9,0,0,24,2,0,0,0,0,0,0,0,0,32,114,9,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,150,0,0,0,0,34,0,144,9,0,0,28,2,0,0,0,0,0,0,0,0,36,34,35,0,107,111,105,56,114,0,0,0,62,254,8,100,144,66,14,73,100,145,70,30,137,164,153,103,162,153,166,154,107,178,217,38,143,110,222,136,99,142,58,238,40,64,143,7,32,144,192,3,73,42,185,36,76,76,54,233,228,147,80,70,41,229,148,84,86,105,229,149,88,102,169,165,56,91,114,217,165,151,95,130,25,166,152,99,146,89,102,1,46,65,20,145,73,20,85,68,210,68,37,97,148,145,70,27,113,212,145,71,31,189,4,82,72,34,141,100,145,68,44,173,116,17,74,45,165,116,146,74,14,193,19,143,65,244,212,67,208,60,5,225,147,143,62,251,240,211,143,63,255,60,4,80,64,2,13,100,143,60,12,45,116,15,66,13,37,116,144,66,107,111,105,56,117,0,0,0,62,254,8,100,144,66,14,73,100,145,70,30,137,164,153,103,162,153,166,154,107,178,217,38,143,110,222,136,99,142,58,238,40,64,143,7,32,144,192,3,73,42,185,36,76,51,53,89,147,77,80,70,41,229,148,84,254,100,229,149,88,102,169,165,56,229,112,121,14,58,95,130,25,166,152,99,250,84,102,1,46,65,20,145,73,20,85,68,210,68,37,97,148,145,70,27,113,212,145,71,31,189,4,82,72,34,141,100,145,68,44,173,116,17,74,45,165,116,146,74,14,193,19,143,65,244,212,67,208,60,5,225,147,143,62,251,240,211,143,63,255,60,4,80,64,2,13,100,143,60,12,45,116,15,66,13,37,116,144,66,0,0,0,0,0,0,0,0,176,252,24,0,16,221,24,0,216,196,24,0,8,175,24,0,200,155,24,0,96,140,26,0,96,122,26,0,32,103,26,0,120,86,26,0,200,69,26,0,32,54,26,0,208,38,26,0,208,28,26,0,136,15,26,0,8,7,26,0,48,0,26,0,160,246,25,0,176,234,25,0,176,226,25,0,128,217,25,0,112,203,25,0,152,191,25,0,80,185,25,0,168,177,25,0,24,171,25,0,120,163,25,0,200,155,25,0,8,148,25,0,112,141,25,0,128,134,25,0,96,126,25,0,56,119,25,0,200,113,25,0,8,109,25,0,80,103,25,0,8,98,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,91,25,0,120,86,25,0,176,81,25,0,8,77,25,0,168,68,25,0,224,61,25,0,0,56,25,0,232,48,25,0,216,42,25,0,200,36,25,0,192,26,25,0,192,19,25,0,248,9,25,0,240,3,25,0,168,252,24,0,1,0,0,0,4,0,0,0,0,0,0,0,184,245,24,0,2,0,0,0,3,0,0,0,0,0,0,0,152,243,24,0,3,0,0,0,11,0,0,0,0,0,0,0,0,241,24,0,3,0,0,0,7,0,0,0,0,0,0,0,240,237,24,0,3,0,0,0,3,1,0,0,0,0,0,0,48,236,24,0,4,0,0,0,7,1,0,0,0,0,0,0,80,232,24,0,4,0,0,0,51,0,0,0,0,0,0,0,224,228,24,0,4,0,0,0,3,6,0,0,0,0,0,0,192,226,24,0,4,0,0,0,15,0,0,0,0,0,0,0,136,224,24,0,5,0,0,0,55,0,0,0,0,0,0,0,0,221,24,0,5,0,0,0,7,6,0,0,0,0,0,0,64,218,24,0,5,0,0,0,15,1,0,0,0,0,0,0,136,216,24,0,6,0,0,0,63,0,0,0,0,0,0,0,184,214,24,0,6,0,0,0,15,6,0,0,0,0,0,0,120,212,24,0,6,0,0,0,7,7,0,0,0,0,0,0,104,210,24,0,6,0,0,0,195,6,0,0,0,0,0,0,0,207,24,0,6,0,0,0,55,1,0,0,0,0,0,0,208,203,24,0,7,0,0,0,15,7,0,0,0,0,0,0,208,203,24,0,7,0,0,0,63,1,0,0,0,0,0,0,144,201,24,0,7,0,0,0,203,6,0,0,0,0,0,0,184,199,24,0,7,0,0,0,55,6,0,0,0,0,0,0,200,196,24,0,7,0,0,0,199,6,0,0,0,0,0,0,112,194,24,0,8,0,0,0,63,6,0,0,0,0,0,0,216,192,24,0,8,0,0,0,255,0,0,0,0,0,0,0,168,191,24,0,8,0,0,0,207,6,0,0,0,0,0,0,32,190,24,0,8,0,0,0,55,7,0,0,0,0,0,0,176,184,24,0,2,0,0,0,0,0,0,96,0,0,0,0,3,0,0,0,0,0,0,0,192,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,128,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,192,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,192,0,0,0,0,0,0,0,0,0,0,128,1,0,0,0,192,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,128,1,0,0,0,3,0,0,0,0,0,0,0,0,0,0,128,1,0,0,0,192,0,0,0,0,0,0,0,0,0,0,128,1,0,0,0,4,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,192,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,128,1,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,6,0,0,0,0,0,0,48,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,0,0,0,48,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,48,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,48,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,48,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,1,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,1,2,2,3,3,0,0,194,149,139,63,240,55,152,63,243,4,181,63,0,0,0,64,72,231,25,0,200,222,25,0,12,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,119,26,0,96,0,0,0,40,167,4,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,12,26,0,0,0,0,0,32,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,16,0,1,0,0,0,0,0,24,4,26,0,0,0,0,0,32,0,0,0,69,90,73,83,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,0,0,0,0,240,251,25,0,0,0,0,0,36,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,16,0,1,0,0,0,0,0,216,241,25,0,0,0,0,0,40,0,0,0,84,77,70,80,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,0,0,0,0,56,231,25,0,176,222,25,0,8,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,16,0,1,0,0,0,0,0,72,212,25,0,176,222,25,0,12,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,16,0,1,0,0,0,0,0,192,199,25,0,176,222,25,0,48,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,16,0,1,0,0,0,0,0,16,189,25,0,176,222,25,0,52,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,16,0,1,0,0,0,0,0,176,182,25,0,128,174,25,0,48,0,0,0,6,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,0,0,255,255,255,255,255,255,239,127,16,0,1,0,0,0,0,0,96,167,25,0,128,174,25,0,48,0,0,0,6,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,0,0,255,255,255,255,255,255,239,127,16,0,1,0,0,0,0,0,48,159,26,0,0,0,0,0,8,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,239,127,16,0,1,0,0,0,0,0,32,160,25,0,0,0,0,0,16,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,239,127,16,0,1,0,0,0,0,0,160,152,25,0,0,0,0,0,56,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,176,130,26,0,96,0,0,0,184,167,4,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,8,8,8,8,8,8,8,8,8,5,2,1,3,6,8,8,8,8,8,8,8,8,8,8,0,0,0,0,0,0,0,236,0,0,0,237,0,0,0,238,0,0,0,239,0,0,0,240,0,0,0,241,0,0,0,242,0,0,0,243,0,0,0,244,0,0,0,245,0,0,0,28,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,58,0,0,0,246,0,0,0,247,0,0,0,248,0,0,0,249,0,0,0,250,0,0,0,251,0,0,0,252,0,0,0,253,0,0,0,254,0,0,0,255,0,0,0,0,0,0,0,13,13,13,13,13,13,13,12,8,7,4,3,1,2,5,6,9,13,13,13,13,13,13,14,14,0,0,0,0,0,0,0,13,13,13,13,13,14,14,11,8,7,4,2,1,3,5,6,9,12,13,14,14,14,14,14,14,0,0,0,0,0,0,0,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,12,11,9,7,5,3,1,2,4,6,8,11,12,15,16,16,16,16,16,16,16,17,17,17,17,17,17,17,17,17,17,0,0,0,0,0,0,0,18,18,18,18,18,18,18,18,18,18,18,18,18,18,18,16,17,14,11,11,8,7,4,2,1,3,5,6,9,11,12,15,16,18,18,18,18,18,18,18,18,18,18,18,18,18,18,19,19,0,0,0,0,0,0,0,242,31,0,0,243,31,0,0,244,31,0,0,245,31,0,0,246,31,0,0,247,31,0,0,248,31,0,0,248,15,0,0,254,0,0,0,126,0,0,0,14,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,30,0,0,0,62,0,0,0,254,1,0,0,249,31,0,0,250,31,0,0,251,31,0,0,252,31,0,0,253,31,0,0,254,31,0,0,254,63,0,0,255,63,0,0,0,0,0,0,247,31,0,0,248,31,0,0,249,31,0,0,250,31,0,0,251,31,0,0,248,63,0,0,249,63,0,0,252,7,0,0,254,0,0,0,126,0,0,0,14,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,30,0,0,0,62,0,0,0,254,1,0,0,250,15,0,0,246,31,0,0,250,63,0,0,251,63,0,0,252,63,0,0,253,63,0,0,254,63,0,0,255,63,0,0,0,0,0,0,228,255,0,0,229,255,0,0,230,255,0,0,231,255,0,0,232,255,0,0,233,255,0,0,234,255,0,0,235,255,0,0,236,255,0,0,237,255,0,0,238,255,0,0,239,255,0,0,240,255,0,0,241,255,0,0,242,255,0,0,243,255,0,0,244,255,0,0,226,255,0,0,252,15,0,0,252,7,0,0,254,1,0,0,126,0,0,0,30,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,14,0,0,0,62,0,0,0,254,0,0,0,253,7,0,0,253,15,0,0,240,127,0,0,227,255,0,0,245,255,0,0,246,255,0,0,247,255,0,0,248,255,0,0,249,255,0,0,250,255,0,0,246,255,1,0,247,255,1,0,248,255,1,0,249,255,1,0,250,255,1,0,251,255,1,0,252,255,1,0,253,255,1,0,254,255,1,0,255,255,1,0,0,0,0,0,226,255,3,0,227,255,3,0,228,255,3,0,229,255,3,0,230,255,3,0,231,255,3,0,232,255,3,0,233,255,3,0,234,255,3,0,235,255,3,0,236,255,3,0,237,255,3,0,238,255,3,0,239,255,3,0,240,255,3,0,247,255,0,0,240,255,1,0,252,63,0,0,254,7,0,0,252,7,0,0,254,0,0,0,126,0,0,0,14,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,30,0,0,0,62,0,0,0,254,1,0,0,253,7,0,0,254,15,0,0,250,127,0,0,246,255,0,0,241,255,3,0,242,255,3,0,243,255,3,0,244,255,3,0,245,255,3,0,246,255,3,0,247,255,3,0,248,255,3,0,249,255,3,0,250,255,3,0,251,255,3,0,252,255,3,0,253,255,3,0,254,255,3,0,254,255,7,0,255,255,7,0,0,0,0,0,0,2,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,16,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,2,0,0,1,3,6,8,9,10,10,11,11,12,12,13,13,3,4,6,7,8,8,9,10,10,10,11,12,12,6,6,7,8,8,9,10,10,10,11,12,12,12,8,7,8,9,9,10,10,11,11,11,12,12,13,9,8,9,9,10,10,11,11,11,12,12,12,13,10,9,9,10,11,11,11,12,11,12,12,13,13,11,9,10,11,11,11,12,12,12,12,13,13,13,11,10,10,11,11,12,12,13,13,13,13,13,13,11,10,10,11,11,11,12,12,13,13,14,13,14,11,10,11,11,12,12,12,12,13,13,14,14,14,12,11,11,12,12,12,13,13,13,14,14,14,15,12,11,12,12,12,13,13,13,13,14,14,15,15,13,12,12,12,13,13,13,13,14,14,14,14,15,0,0,0,0,0,0,0,5,4,5,6,7,8,9,10,4,3,4,5,6,7,7,8,5,4,4,5,6,7,7,8,6,5,5,6,6,7,8,8,7,6,6,6,7,7,8,9,8,7,6,7,7,8,8,10,9,7,7,8,8,8,9,9,10,8,8,8,9,9,9,10,1,3,6,7,8,9,10,11,3,4,6,7,8,8,9,9,6,6,7,8,8,9,9,10,7,7,8,8,9,9,10,10,8,8,9,9,10,10,10,11,9,8,9,9,10,10,11,11,10,9,9,10,10,11,12,12,11,10,10,10,11,11,12,12,11,10,9,9,9,9,9,10,11,10,9,8,7,7,7,8,9,10,9,8,6,6,6,6,6,8,9,9,7,6,4,4,4,6,7,9,9,7,6,4])
.concat([4,4,6,7,9,9,7,6,4,4,4,6,7,9,9,8,6,6,6,6,6,8,9,10,9,8,7,7,7,7,8,10,11,10,9,9,9,9,9,10,11,0,0,0,0,0,0,0,13,12,11,11,10,11,11,12,13,12,11,10,9,8,9,10,11,12,12,10,9,8,7,8,9,10,11,11,9,8,5,4,5,8,9,11,10,8,7,4,1,4,7,8,11,11,9,8,5,4,5,8,9,11,11,10,9,8,7,8,9,10,11,12,11,10,9,8,9,10,11,12,13,12,12,11,10,10,11,12,13,0,0,0,0,0,0,0,4,5,8,5,4,8,9,8,11,5,5,8,5,4,8,8,7,10,9,8,11,8,8,10,11,10,11,4,5,8,4,4,8,8,8,10,4,4,8,4,4,7,8,7,9,8,8,10,7,7,9,10,9,10,8,8,11,8,7,10,11,10,12,8,7,10,7,7,9,10,9,11,11,10,12,10,9,11,11,10,11,0,0,0,0,0,0,0,1,4,8,4,5,8,9,9,10,4,6,9,6,6,9,9,9,10,9,10,13,9,9,11,11,10,12,4,6,10,6,7,10,10,10,12,5,7,11,6,7,10,9,9,11,9,10,13,8,9,12,10,11,12,8,10,15,9,11,15,13,14,16,8,10,14,9,10,14,12,12,15,11,12,16,10,11,15,12,12,15,0,0,0,0,0,0,0,9,7,9,8,6,8,9,8,9,8,6,7,6,5,6,7,6,8,9,7,8,8,6,8,9,7,9,8,6,7,6,5,6,7,6,8,6,5,6,5,3,5,6,5,6,8,6,7,6,5,6,8,6,8,9,7,9,8,6,8,8,7,9,8,6,7,6,4,6,8,6,7,9,7,9,7,6,8,9,7,9,0,0,0,0,0,0,0,4,5,6,7,8,8,9,10,10,10,11,11,12,11,12,12,10,5,4,5,6,7,7,8,8,9,9,9,10,10,10,10,11,8,6,5,5,6,7,7,8,8,8,9,9,9,10,10,10,10,8,7,6,6,6,7,7,8,8,8,9,9,9,10,10,10,10,8,8,7,7,7,7,8,8,8,8,9,9,9,10,10,10,10,8,8,7,7,7,7,8,8,8,9,9,9,9,10,10,10,10,8,9,8,8,8,8,8,8,8,9,9,9,10,10,10,10,10,8,9,8,8,8,8,8,8,9,9,9,10,10,10,10,10,10,8,10,9,8,8,9,9,9,9,9,10,10,10,10,10,10,11,8,10,9,9,9,9,9,9,9,10,10,10,10,10,10,11,11,8,11,9,9,9,9,9,9,10,10,10,10,10,11,10,11,11,8,11,10,9,9,10,9,10,10,10,10,10,11,11,11,11,11,8,11,10,10,10,10,10,10,10,10,10,10,11,11,11,11,11,9,11,10,9,9,10,10,10,10,10,10,11,11,11,11,11,11,9,11,10,10,10,10,10,10,10,10,10,11,11,11,11,11,11,9,12,10,10,10,10,10,10,10,11,11,11,11,11,11,12,12,9,9,8,8,8,8,8,8,8,8,8,8,8,8,8,8,9,5,0,0,0,0,0,0,0,6,5,6,6,7,8,9,10,10,10,11,11,12,5,4,4,5,6,7,7,8,8,9,10,10,11,6,4,5,5,6,6,7,8,8,9,9,10,10,6,5,5,5,6,7,7,8,8,9,9,10,10,7,6,6,6,6,7,7,8,8,9,9,10,10,8,7,6,7,7,7,8,8,8,9,10,10,11,9,7,7,7,7,8,8,9,9,9,10,10,11,9,8,8,8,8,8,9,9,9,10,10,11,11,9,8,8,8,8,8,9,9,10,10,10,11,11,10,9,9,9,9,9,9,10,10,10,11,11,12,10,9,9,9,9,10,10,10,10,11,11,11,12,11,10,9,10,10,10,10,10,11,11,11,11,12,11,10,10,10,10,10,10,11,11,12,12,12,12,0,0,0,0,0,0,0,11,9,11,10,7,10,11,9,11,10,7,10,7,5,7,9,7,10,11,9,11,9,7,9,11,9,11,9,7,9,7,5,7,9,7,9,7,5,7,5,1,5,7,5,7,9,7,9,7,5,7,9,7,9,11,9,11,9,7,9,11,9,11,10,7,9,7,5,7,9,7,10,11,9,11,10,7,9,11,9,11,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,16,0,0,0,24,0,0,0,32,0,0,0,40,0,0,0,48,0,0,0,56,0,0,0,64,0,0,0,80,0,0,0,96,0,0,0,112,0,0,0,128,0,0,0,144,0,0,0,160,0,0,0,255,255,255,255,0,0,0,0,32,0,0,0,40,0,0,0,48,0,0,0,56,0,0,0,64,0,0,0,80,0,0,0,96,0,0,0,112,0,0,0,128,0,0,0,160,0,0,0,192,0,0,0,224,0,0,0,0,1,0,0,64,1,0,0,255,255,255,255,0,0,0,0,8,0,0,0,16,0,0,0,24,0,0,0,32,0,0,0,40,0,0,0,48,0,0,0,56,0,0,0,64,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,48,12,255,1,48,2,48,14,255,39,32,27,255,26,255,31,255,1,255,48,254,38,32,37,32,80,254,81,254,82,254,183,0,84,254,85,254,86,254,87,254,92,255,19,32,49,254,20,32,51,254,116,37,52,254,79,254,8,255,9,255,53,254,54,254,91,255,93,255,55,254,56,254,20,48,21,48,57,254,58,254,16,48,17,48,59,254,60,254,10,48,11,48,61,254,62,254,8,48,9,48,63,254,64,254,12,48,13,48,65,254,66,254,14,48,15,48,67,254,68,254,89,254,90,254,91,254,92,254,93,254,94,254,24,32,25,32,28,32,29,32,29,48,30,48,53,32,50,32,3,255,6,255,10,255,59,32,167,0,3,48,203,37,207,37,179,37,178,37,206,37,6,38,5,38,199,37,198,37,161,37,160,37,189,37,188,37,163,50,5,33,175,0,227,255,63,255,205,2,73,254,74,254,77,254,78,254,75,254,76,254,95,254,96,254,97,254,11,255,13,255,215,0,247,0,177,0,26,34,28,255,30,255,29,255,102,34,103,34,96,34,30,34,82,34,97,34,98,254,99,254,100,254,101,254,102,254,94,255,41,34,42,34,165,34,32,34,31,34,191,34,210,51,209,51,43,34,46,34,53,34,52,34,64,38,66,38,149,34,153,34,145,33,147,33,144,33,146,33,150,33,151,33,153,33,152,33,37,34,35,34,15,255,60,255,21,34,104,254,4,255,229,255,18,48,224,255,225,255,5,255,32,255,3,33,9,33,105,254,106,254,107,254,213,51,156,51,157,51,158,51,206,51,161,51,142,51,143,51,196,51,176,0,89,81,91,81,94,81,93,81,97,81,99,81,231,85,233,116,206,124,129,37,130,37,131,37,132,37,133,37,134,37,135,37,136,37,143,37,142,37,141,37,140,37,139,37,138,37,137,37,60,37,52,37,44,37,36,37,28,37,148,37,0,37,2,37,149,37,12,37,16,37,20,37,24,37,109,37,110,37,112,37,111,37,80,37,94,37,106,37,97,37,226,37,227,37,229,37,228,37,113,37,114,37,115,37,16,255,17,255,18,255,19,255,20,255,21,255,22,255,23,255,24,255,25,255,96,33,97,33,98,33,99,33,100,33,101,33,102,33,103,33,104,33,105,33,33,48,34,48,35,48,36,48,37,48,38,48,39,48,40,48,41,48,65,83,68,83,69,83,33,255,34,255,35,255,36,255,37,255,38,255,39,255,40,255,41,255,42,255,43,255,44,255,45,255,46,255,47,255,48,255,49,255,50,255,51,255,52,255,53,255,54,255,55,255,56,255,57,255,58,255,65,255,66,255,67,255,68,255,69,255,70,255,71,255,72,255,73,255,74,255,75,255,76,255,77,255,78,255,79,255,80,255,81,255,82,255,83,255,84,255,85,255,86,255,87,255,88,255,89,255,90,255,145,3,146,3,147,3,148,3,149,3,150,3,151,3,152,3,153,3,154,3,155,3,156,3,157,3,158,3,159,3,160,3,161,3,163,3,164,3,165,3,166,3,167,3,168,3,169,3,177,3,178,3,179,3,180,3,181,3,182,3,183,3,184,3,185,3,186,3,187,3,188,3,189,3,190,3,191,3,192,3,193,3,195,3,196,3,197,3,198,3,199,3,200,3,201,3,5,49,6,49,7,49,8,49,9,49,10,49,11,49,12,49,13,49,14,49,15,49,16,49,17,49,18,49,19,49,20,49,21,49,22,49,23,49,24,49,25,49,26,49,27,49,28,49,29,49,30,49,31,49,32,49,33,49,34,49,35,49,36,49,37,49,38,49,39,49,40,49,41,49,217,2,201,2,202,2,199,2,203,2,0,36,1,36,2,36,3,36,4,36,5,36,6,36,7,36,8,36,9,36,10,36,11,36,12,36,13,36,14,36,15,36,16,36,17,36,18,36,19,36,20,36,21,36,22,36,23,36,24,36,25,36,26,36,27,36,28,36,29,36,30,36,31,36,33,36,172,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,78,89,78,1,78,3,78,67,78,93,78,134,78,140,78,186,78,63,81,101,81,107,81,224,81,0,82,1,82,155,82,21,83,65,83,92,83,200,83,9,78,11,78,8,78,10,78,43,78,56,78,225,81,69,78,72,78,95,78,94,78,142,78,161,78,64,81,3,82,250,82,67,83,201,83,227,83,31,87,235,88,21,89,39,89,115,89,80,91,81,91,83,91,248,91,15,92,34,92,56,92,113,92,221,93,229,93,241,93,242,93,243,93,254,93,114,94,254,94,11,95,19,95,77,98,17,78,16,78,13,78,45,78,48,78,57,78,75,78,57,92,136,78,145,78,149,78,146,78,148,78,162,78,193,78,192,78,195,78,198,78,199,78,205,78,202,78,203,78,196,78,67,81,65,81,103,81,109,81,110,81,108,81,151,81,246,81,6,82,7,82,8,82,251,82,254,82,255,82,22,83,57,83,72,83,71,83,69,83,94,83,132,83,203,83,202,83,205,83,236,88,41,89,43,89,42,89,45,89,84,91,17,92,36,92,58,92,111,92,244,93,123,94,255,94,20,95,21,95,195,95,8,98,54,98,75,98,78,98,47,101,135,101,151,101,164,101,185,101,229,101,240,102,8,103,40,103,32,107,98,107,121,107,203,107,212,107,219,107,15,108,52,108,107,112,42,114,54,114,59,114,71,114,89,114,91,114,172,114,139,115,25,78,22,78,21,78,20,78,24,78,59,78,77,78,79,78,78,78,229,78,216,78,212,78,213,78,214,78,215,78,227,78,228,78,217,78,222,78,69,81,68,81,137,81,138,81,172,81,249,81,250,81,248,81,10,82,160,82,159,82,5,83,6,83,23,83,29,83,223,78,74,83,73,83,97,83,96,83,111,83,110,83,187,83,239,83,228,83,243,83,236,83,238,83,233,83,232,83,252,83,248,83,245,83,235,83,230,83,234,83,242,83,241,83,240,83,229,83,237,83,251,83,219,86,218,86,22,89,46,89,49,89,116,89,118,89,85,91,131,91,60,92,232,93,231,93,230,93,2,94,3,94,115,94,124,94,1,95,24,95,23,95,197,95,10,98,83,98,84,98,82,98,81,98,165,101,230,101,46,103,44,103,42,103,43,103,45,103,99,107,205,107,17,108,16,108,56,108,65,108,64,108,62,108,175,114,132,115,137,115,220,116,230,116,24,117,31,117,40,117,41,117,48,117,49,117,50,117,51,117,139,117,125,118,174,118,191,118,238,118,219,119,226,119,243,119,58,121,190,121,116,122,203,122,30,78,31,78,82,78,83,78,105,78,153,78,164,78,166,78,165,78,255,78,9,79,25,79,10,79,21,79,13,79,16,79,17,79,15,79,242,78,246,78,251,78,240,78,243,78,253,78,1,79,11,79,73,81,71,81,70,81,72,81,104,81,113,81,141,81,176,81,23,82,17,82,18,82,14,82,22,82,163,82,8,83,33,83,32,83,112,83,113,83,9,84,15,84,12,84,10,84,16,84,1,84,11,84,4,84,17,84,13,84,8,84,3,84,14,84,6,84,18,84,224,86,222,86,221,86,51,87,48,87,40,87,45,87,44,87,47,87,41,87,25,89,26,89,55,89,56,89,132,89,120,89,131,89,125,89,121,89,130,89,129,89,87,91,88,91,135,91,136,91,133,91,137,91,250,91,22,92,121,92,222,93,6,94,118,94,116,94,15,95,27,95,217,95,214,95,14,98,12,98,13,98,16,98,99,98,91,98,88,98,54,101,233,101,232,101,236,101,237,101,242,102,243,102,9,103,61,103,52,103,49,103,53,103,33,107,100,107,123,107,22,108,93,108,87,108,89,108,95,108,96,108,80,108,85,108,97,108,91,108,77,108,78,108,112,112,95,114,93,114,126,118,249,122,115,124,248,124,54,127,138,127,189,127,1,128,3,128,12,128,18,128,51,128,127,128,137,128,139,128,140,128,227,129,234,129,243,129,252,129,12,130,27,130,31,130,110,130,114,130,126,130,107,134,64,136,76,136,99,136,127,137,33,150,50,78,168,78,77,79,79,79,71,79,87,79,94,79,52,79,91,79,85,79,48,79,80,79,81,79,61,79,58,79,56,79,67,79,84,79,60,79,70,79,99,79,92,79,96,79,47,79,78,79,54,79,89,79,93,79,72,79,90,79,76,81,75,81,77,81,117,81,182,81,183,81,37,82,36,82,41,82,42,82,40,82,171,82,169,82,170,82,172,82,35,83,115,83,117,83,29,84,45,84,30,84,62,84,38,84,78,84,39,84,70,84,67,84,51,84,72,84,66,84,27,84,41,84,74,84,57,84,59,84,56,84,46,84,53,84,54,84,32,84,60,84,64,84,49,84,43,84,31,84,44,84,234,86,240,86,228,86,235,86,74,87,81,87,64,87,77,87,71,87,78,87,62,87,80,87,79,87,59,87,239,88,62,89,157,89,146,89,168,89,158,89,163,89,153,89,150,89,141,89,164,89,147,89,138,89,165,89,93,91,92,91,90,91,91,91,140,91,139,91,143,91,44,92,64,92,65,92,63,92,62,92,144,92,145,92,148,92,140,92,235,93,12,94,143,94,135,94,138,94,247,94,4,95,31,95,100,95,98,95,119,95,121,95,216,95,204,95,215,95,205,95,241,95,235,95,248,95,234,95,18,98,17,98,132,98,151,98,150,98,128,98,118,98,137,98,109,98,138,98,124,98,126,98,121,98,115,98,146,98,111,98,152,98,110,98,149,98,147,98,145,98,134,98,57,101,59,101,56,101,241,101,244,102,95,103,78,103,79,103,80,103,81,103,92,103,86,103,94,103,73,103,70,103,96,103,83,103,87,103,101,107,207,107,66,108,94,108,153,108,129,108,136,108,137,108,133,108,155,108,106,108,122,108,144,108,112,108,140,108,104,108,150,108,146,108,125,108,131,108,114,108,126,108,116,108,134,108,118,108,141,108,148,108,152,108,130,108,118,112,124,112,125,112,120,112,98,114,97,114,96,114,196,114,194,114,150,115,44,117,43,117,55,117,56,117,130,118,239,118,227,119,193,121,192,121,191,121,118,122,251,124,85,127,150,128,147,128,157,128,152,128,155,128,154,128,178,128,111,130,146,130,139,130,141,130,139,137,210,137,0,138,55,140,70,140,85,140,157,140,100,141,112,141,179,141,171,142,202,142,155,143,176,143,194,143,198,143,197,143,196,143,225,93,145,144,162,144,170,144,166,144,163,144,73,145,198,145,204,145,50,150,46,150,49,150,42,150,44,150,38,78,86,78,115,78,139,78,155,78,158,78,171,78,172,78,111,79,157,79,141,79,115,79,127,79,108,79,155,79,139,79,134,79,131,79,112,79,117,79,136,79,105,79,123,79,150,79,126,79,143,79,145,79,122,79,84,81,82,81,85,81,105,81,119,81,118,81,120,81,189,81,253,81,59,82,56,82,55,82,58,82,48,82,46,82,54,82,65,82,190,82,187,82,82,83,84,83,83,83,81,83,102,83,119,83,120,83,121,83,214,83,212,83,215,83,115,84,117,84,150,84,120,84,149,84,128,84,123,84,119,84,132,84,146,84,134,84,124,84,144,84,113,84,118,84,140,84,154,84,98,84,104,84,139,84,125,84,142,84,250,86,131,87,119,87,106,87,105,87,97,87,102,87,100,87,124,87,28,89,73,89,71,89,72,89,68,89,84,89,190,89,187,89,212,89,185,89,174,89,209,89,198,89,208,89,205,89,203,89,211,89,202,89,175,89,179,89,210,89,197,89,95,91,100,91,99,91,151,91,154,91,152,91,156,91,153,91,155,91,26,92,72,92,69,92,70,92,183,92,161,92,184,92,169,92,171,92,177,92,179,92,24,94,26,94,22,94,21,94,27,94,17,94,120,94,154,94,151,94,156,94,149,94,150,94,246,94,38,95,39,95,41,95,128,95,129,95,127,95,124,95,221,95,224,95,253,95,245,95,255,95,15,96,20,96,47,96,53,96,22,96,42,96,21,96,33,96,39,96,41,96,43,96,27,96,22,98,21,98,63,98,62,98,64,98,127,98,201,98,204,98,196,98,191,98,194,98,185,98,210,98,219,98,171,98,211,98,212,98,203,98,200,98,168,98,189,98,188,98,208,98,217,98,199,98,205,98,181,98,218,98,177,98,216,98,214,98,215,98,198,98,172,98,206,98,62,101,167,101,188,101,250,101,20,102,19,102,12,102,6,102,2,102,14,102,0,102,15,102,21,102,10,102,7,102,13,103,11,103,109,103,139,103,149,103,113,103,156,103,115,103,119,103,135,103,157,103,151,103,111,103,112,103,127,103,137,103,126,103,144,103,117,103,154,103,147,103,124,103,106,103,114,103,35,107,102,107,103,107,127,107,19,108,27,108,227,108,232,108,243,108,177,108,204,108,229,108,179,108,189,108,190,108,188,108,226,108,171,108,213,108,211,108,184,108,196,108,185,108,193,108,174,108,215,108,197,108,241,108,191,108,187,108,225,108,219,108,202,108,172,108,239,108,220,108,214,108,224,108,149,112,142,112,146,112,138,112,153,112,44,114,45,114,56,114,72,114,103,114,105,114,192,114,206,114,217,114,215,114,208,114,169,115,168,115,159,115,171,115,165,115,61,117,157,117,153,117,154,117,132,118,194,118,242,118,244,118,229,119,253,119,62,121,64,121,65,121,201,121,200,121,122,122,121,122,250,122,254,124,84,127,140,127,139,127,5,128,186,128,165,128,162,128,177,128,161,128,171,128,169,128,180,128,170,128,175,128,229,129,254,129,13,130,179,130,157,130,153,130,173,130,189,130,159,130,185,130,177,130,172,130,165,130,175,130,184,130,163,130,176,130,190,130,183,130,78,134,113,134,29,82,104,136,203,142,206,143,212,143,209,143,181,144,184,144,177,144,182,144,199,145,209,145,119,149,128,149,28,150,64,150,63,150,59,150,68,150,66,150,185,150,232,150,82,151,94,151,159,78,173,78,174,78,225,79,181,79,175,79,191,79,224,79,209,79,207,79,221,79,195,79,182,79,216,79,223,79,202,79,215,79,174,79,208,79,196,79,194,79,218,79,206,79,222,79,183,79,87,81,146,81,145,81,160,81,78,82,67,82,74,82,77,82,76,82,75,82,71,82,199,82,201,82,195,82,193,82,13,83,87,83,123,83,154,83,219,83,172,84,192,84,168,84,206,84,201,84,184,84,166,84,179,84,199,84,194,84,189,84,170,84,193,84,196,84,200,84,175,84,171,84,177,84,187,84,169,84,167,84,191,84,255,86,130,87,139,87,160,87,163,87,162,87,206,87,174,87,147,87,85,89,81,89,79,89,78,89,80,89,220,89,216,89,255,89,227,89,232,89,3,90,229,89,234,89,218,89,230,89,1,90,251,89,105,91,163,91,166,91,164,91,162,91,165,91,1,92,78,92,79,92,77,92,75,92,217,92,210,92,247,93,29,94,37,94,31,94,125,94,160,94,166,94,250,94,8,95,45,95,101,95,136,95,133,95,138,95,139,95,135,95,140,95,137,95,18,96,29,96,32,96,37,96,14,96,40,96,77,96,112,96,104,96,98,96,70,96,67,96,108,96,107,96,106,96,100,96,65,98,220,98,22,99,9,99,252,98,237,98,1,99,238,98,253,98,7,99,241,98,247,98,239,98,236,98,254,98,244,98,17,99,2,99,63,101,69,101,171,101,189,101,226,101,37,102,45,102,32,102,39,102,47,102,31,102,40,102,49,102,36,102,247,102,255,103,211,103,241,103,212,103,208,103,236,103,182,103,175,103,245,103,233,103,239,103,196,103,209,103,180,103,218,103,229,103,184,103,207,103,222,103,243,103,176,103,217,103,226,103,221,103,210,103,106,107,131,107,134,107,181,107,210,107,215,107,31,108,201,108,11,109,50,109,42,109,65,109,37,109,12,109,49,109,30,109,23,109,59,109,61,109,62,109,54,109,27,109,245,108,57,109,39,109,56,109,41,109,46,109,53,109,14,109,43,109,171,112,186,112,179,112,172,112,175,112,173,112,184,112,174,112,164,112,48,114,114,114,111,114,116,114,233,114,224,114,225,114,183,115,202,115,187,115,178,115,205,115,192,115,179,115,26,117,45,117,79,117,76,117,78,117,75,117,171,117,164,117,165,117,162,117,163,117,120,118,134,118,135,118,136,118,200,118,198,118,195,118,197,118,1,119,249,118,248,118,9,119,11,119,254,118,252,118,7,119,220,119,2,120,20,120,12,120,13,120,70,121,73,121,72,121,71,121,185,121,186,121,209,121,210,121,203,121,127,122,129,122,255,122,253,122,125,124,2,125,5,125,0,125,9,125,7,125,4,125,6,125,56,127,142,127,191,127,4,128,16,128,13,128,17,128,54,128,214,128,229,128,218,128,195,128,196,128,204,128,225,128,219,128,206,128,222,128,228,128,221,128,244,129,34,130,231,130,3,131,5,131,227,130,219,130,230,130,4,131,229,130,2,131,9,131,210,130,215,130,241,130,1,131,220,130,212,130,209,130,222,130,211,130,223,130,239,130,6,131,80,134,121,134,123,134,122,134,77,136,107,136,129,137,212,137,8,138,2,138,3,138,158,140,160,140,116,141,115,141,180,141,205,142,204,142,240,143,230,143,226,143,234,143,229,143,237,143,235,143,228,143,232,143,202,144,206,144,193,144,195,144,75,145,74,145,205,145,130,149,80,150,75,150,76,150,77,150,98,151,105,151,203,151,237,151,243,151,1,152,168,152,219,152,223,152,150,153,153,153,88,78,179,78,12,80,13,80,35,80,239,79,38,80,37,80,248,79,41,80,22,80,6,80,60,80,31,80,26,80,18,80,17,80,250,79,0,80,20,80,40,80,241,79,33,80,11,80,25,80,24,80,243,79,238,79,45,80,42,80,254,79,43,80,9,80,124,81,164,81,165,81,162,81,205,81,204,81,198,81,203,81,86,82,92,82,84,82,91,82,93,82,42,83,127,83,159,83,157,83,223,83,232,84,16,85,1,85,55,85,252,84,229,84,242,84,6,85,250,84,20,85,233,84,237,84,225,84,9,85,238,84,234,84,230,84,39,85,7,85,253,84,15,85,3,87,4,87,194,87,212,87,203,87,195,87,9,88,15,89,87,89,88,89,90,89,17,90,24,90,28,90,31,90,27,90,19,90,236,89,32,90,35,90,41,90,37,90,12,90,9,90,107,91,88,92,176,91,179,91,182,91,180,91,174,91,181,91,185,91,184,91,4,92,81,92,85,92,80,92,237,92,253,92,251,92,234,92,232,92,240,92,246,92,1,93,244,92,238,93,45,94,43,94,171,94,173,94,167,94,49,95,146,95,145,95,144,95,89,96,99,96,101,96,80,96,85,96,109,96,105,96,111,96,132,96,159,96,154,96,141,96,148,96,140,96,133,96,150,96,71,98,243,98,8,99,255,98,78,99,62,99,47,99,85,99,66,99,70,99,79,99,73,99,58,99,80,99,61,99,42,99,43,99,40,99,77,99,76,99,72,101,73,101,153,101,193,101,197,101,66,102,73,102,79,102,67,102,82,102,76,102,69,102,65,102,248,102,20,103,21,103,23,103,33,104,56,104,72,104,70,104,83,104,57,104,66,104,84,104,41,104,179,104,23,104,76,104,81,104,61,104,244,103,80,104,64,104,60,104,67,104,42,104,69,104,19,104,24,104,65,104,138,107,137,107,183,107,35,108,39,108,40,108,38,108,36,108,240,108,106,109,149,109,136,109,135,109,102,109,120,109,119,109,89,109,147,109,108,109,137,109,110,109,90,109,116,109,105,109,140,109,138,109,121,109,133,109,101,109,148,109,202,112,216,112,228,112,217,112,200,112,207,112,57,114,121,114,252,114,249,114,253,114,248,114,247,114,134,115,237,115,9,116,238,115,224,115,234,115,222,115,84,117,93,117,92,117,90,117,89,117,190,117,197,117,199,117,178,117,179,117,189,117,188,117,185,117,194,117,184,117,139,118,176,118,202,118,205,118,206,118,41,119,31,119,32,119,40,119,233,119,48,120,39,120,56,120,29,120,52,120,55,120,37,120,45,120,32,120,31,120,50,120,85,121,80,121,96,121,95,121,86,121,94,121,93,121,87,121,90,121,228,121,227,121,231,121,223,121,230,121,233,121,216,121,132,122,136,122,217,122,6,123,17,123,137,124,33,125,23,125,11,125,10,125,32,125,34,125,20,125,16,125,21,125,26,125,28,125,13,125,25,125,27,125,58,127,95,127,148,127,197,127,193,127,6,128,24,128,21,128,25,128,23,128,61,128,63,128,241,128,2,129,240,128,5,129,237,128,244,128,6,129,248,128,243,128,8,129,253,128,10,129,252,128,239,128,237,129,236,129,0,130,16,130,42,130,43,130,40,130,44,130,187,130,43,131,82,131,84,131,74,131,56,131,80,131,73,131,53,131,52,131,79,131,50,131,57,131,54,131,23,131,64,131,49,131,40,131,67,131,84,134,138,134,170,134,147,134,164,134,169,134,140,134,163,134,156,134,112,136,119,136,129,136,130,136,125,136,121,136,24,138,16,138,14,138,12,138,21,138,10,138,23,138,19,138,22,138,15,138,17,138,72,140,122,140,121,140,161,140,162,140,119,141,172,142,210,142,212,142,207,142,177,143,1,144,6,144,247,143,0,144,250,143,244,143,3,144,253,143,5,144,248,143,149,144,225,144,221,144,226,144,82,145,77,145,76,145,216,145,221,145,215,145,220,145,217,145,131,149,98,150,99,150,97,150,91,150,93,150,100,150,88,150,94,150,187,150,226,152,172,153,168,154,216,154,37,155,50,155,60,155,126,78,122,80,125,80,92,80,71,80,67,80,76,80,90,80,73,80,101,80,118,80,78,80,85,80,117,80,116,80,119,80,79,80,15,80,111,80,109,80,92,81,149,81,240,81,106,82,111,82,210,82,217,82,216,82,213,82,16,83,15,83,25,83,63,83,64,83,62,83,195,83,252,102,70,85,106,85,102,85,68,85,94,85,97,85,67,85,74,85,49,85,86,85,79,85,85,85,47,85,100,85,56,85,46,85,92,85,44,85,99,85,51,85,65,85,87,85,8,87,11,87,9,87,223,87,5,88,10,88,6,88,224,87,228,87,250,87,2,88,53,88,247,87,249,87,32,89,98,89,54,90,65,90,73,90,102,90,106,90,64,90,60,90,98,90,90,90,70,90,74,90,112,91,199,91,197,91,196,91,194,91,191,91,198,91,9,92,8,92,7,92,96,92,92,92,93,92,7,93,6,93,14,93,27,93,22,93,34,93,17,93,41,93,20,93,25,93,36,93,39,93,23,93,226,93,56,94,54,94,51,94,55,94,183,94,184,94,182,94,181,94,190,94,53,95,55,95,87,95,108,95,105,95,107,95,151,95,153,95,158,95,152,95,161,95,160,95,156,95,127,96,163,96,137,96,160,96,168,96,203,96,180,96,230,96,189,96,197,96,187,96,181,96,220,96,188,96,216,96,213,96,198,96,223,96,184,96,218,96,199,96,26,98,27,98,72,98,160,99,167,99,114,99,150,99,162,99,165,99,119,99,103,99,152,99,170,99,113,99,169,99,137,99,131,99,155,99,107,99,168,99,132,99,136,99,153,99,161,99,172,99,146,99,143,99,128,99,123,99,105,99,104,99,122,99,93,101,86,101,81,101,89,101,87,101,95,85,79,101,88,101,85,101,84,101,156,101,155,101,172,101,207,101,203,101,204,101,206,101,93,102,90,102,100,102,104,102,102,102,94,102,249,102,215,82,27,103,129,104,175,104,162,104,147,104,181,104,127,104,118,104,177,104,167,104,151,104,176,104,131,104,196,104,173,104,134,104,133,104,148,104,157,104,168,104,159,104,161,104,130,104,50,107,186,107,235,107,236,107,43,108,142,109,188,109,243,109,217,109,178,109,225,109,204,109,228,109,251,109,250,109,5,110,199,109,203,109,175,109,209,109,174,109,222,109,249,109,184,109,247,109,245,109,197,109,210,109,26,110,181,109,218,109,235,109,216,109,234,109,241,109,238,109,232,109,198,109,196,109,170,109,236,109,191,109,230,109,249,112,9,113,10,113,253,112,239,112,61,114,125,114,129,114,28,115,27,115,22,115,19,115,25,115,135,115,5,116,10,116,3,116,6,116,254,115,13,116,224,116,246,116,247,116,28,117,34,117,101,117,102,117,98,117,112,117,143,117,212,117,213,117,181,117,202,117,205,117,142,118,212,118,210,118,219,118,55,119,62,119,60,119,54,119,56,119,58,119,107,120,67,120,78,120,101,121,104,121,109,121,251,121,146,122,149,122,32,123,40,123,27,123,44,123,38,123,25,123,30,123,46,123,146,124,151,124,149,124,70,125,67,125,113,125,46,125,57,125,60,125,64,125,48,125,51,125,68,125,47,125,66,125,50,125,49,125,61,127,158,127,154,127,204,127,206,127,210,127,28,128,74,128,70,128,47,129,22,129,35,129,43,129,41,129,48,129,36,129,2,130,53,130,55,130,54,130,57,130,142,131,158,131,152,131,120,131,162,131,150,131,189,131,171,131,146,131,138,131,147,131,137,131,160,131,119,131,123,131,124,131,134,131,167,131,85,134,106,95,199,134,192,134,182,134,196,134,181,134,198,134,203,134,177,134,175,134,201,134,83,136,158,136,136,136,171,136,146,136,150,136,141,136,139,136,147,137,143,137,42,138,29,138,35,138,37,138,49,138,45,138,31,138,27,138,34,138,73,140,90,140,169,140,172,140,171,140,168,140,170,140,167,140,103,141,102,141,190,141,186,141,219,142,223,142,25,144,13,144,26,144,23,144,35,144,31,144,29,144,16,144,21,144,30,144,32,144,15,144,34,144,22,144,27,144,20,144,232,144,237,144,253,144,87,145,206,145,245,145,230,145,227,145,231,145,237,145,233,145,137,149,106,150,117,150,115,150,120,150,112,150,116,150,118,150,119,150,108,150,192,150,234,150,233,150,224,122,223,122,2,152,3,152,90,155,229,156,117,158,127,158,165,158,187,158,162,80,141,80,133,80,153,80,145,80,128,80,150,80,152,80,154,80,0,103,241,81,114,82,116,82,117,82,105,82,222,82,221,82,219,82,90,83,165,83,123,85,128,85,167,85,124,85,138,85,157,85,152,85,130,85,156,85,170,85,148,85,135,85,139,85,131,85,179,85,174,85,159,85,62,85,178,85,154,85,187,85,172,85,177,85,126,85,137,85,171,85,153,85,13,87,47,88,42,88,52,88,36,88,48,88,49,88,33,88,29,88,32,88,249,88,250,88,96,89,119,90,154,90,127,90,146,90,155,90,167,90,115,91,113,91,210,91,204,91,211,91,208,91,10,92,11,92,49,92,76,93,80,93,52,93,71,93,253,93,69,94,61,94,64,94,67,94,126,94,202,94,193,94,194,94,196,94,60,95,109,95,169,95,170,95,168,95,209,96,225,96,178,96,182,96,224,96,28,97,35,97,250,96,21,97,240,96,251,96,244,96,104,97,241,96,14,97,246,96,9,97,0,97,18,97,31,98,73,98,163,99,140,99,207,99,192,99,233,99,201,99,198,99,205,99,210,99,227,99,208,99,225,99,214,99,237,99,238,99,118,99,244,99,234,99,219,99,82,100,218,99,249,99,94,101,102,101,98,101,99,101,145,101,144,101,175,101,110,102,112,102,116,102,118,102,111,102,145,102,122,102,126,102,119,102,254,102,255,102,31,103,29,103,250,104,213,104,224,104,216,104,215,104,5,105,223,104,245,104,238,104,231,104,249,104,210,104,242,104,227,104,203,104,205,104,13,105,18,105,14,105,201,104,218,104,110,105,251,104,62,107,58,107,61,107,152,107,150,107,188,107,239,107,46,108,47,108,44,108,47,110,56,110,84,110,33,110,50,110,103,110,74,110,32,110,37,110,35,110,27,110,91,110,88,110,36,110,86,110,110,110,45,110,38,110,111,110,52,110,77,110,58,110,44,110,67,110,29,110,62,110,203,110,137,110,25,110,78,110,99,110,68,110,114,110,105,110,95,110,25,113,26,113,38,113,48,113,33,113,54,113,110,113,28,113,76,114,132,114,128,114,54,115,37,115,52,115,41,115,58,116,42,116,51,116,34,116,37,116,53,116,54,116,52,116,47,116,27,116,38,116,40,116,37,117,38,117,107,117,106,117,226,117,219,117,227,117,217,117,216,117,222,117,224,117,123,118,124,118,150,118,147,118,180,118,220,118,79,119,237,119,93,120,108,120,111,120,13,122,8,122,11,122,5,122,0,122,152,122,151,122,150,122,229,122,227,122,73,123,86,123,70,123,80,123,82,123,84,123,77,123,75,123,79,123,81,123,159,124,165,124,94,125,80,125,104,125,85,125,43,125,110,125,114,125,97,125,102,125,98,125,112,125,115,125,132,85,212,127,213,127,11,128,82,128,133,128,85,129,84,129,75,129,81,129,78,129,57,129,70,129,62,129,76,129,83,129,116,129,18,130,28,130,233,131,3,132,248,131,13,132,224,131,197,131,11,132,193,131,239,131,241,131,244,131,87,132,10,132,240,131,12,132,204,131,253,131,242,131,202,131,56,132,14,132,4,132,220,131,7,132,212,131,223,131,91,134,223,134,217,134,237,134,212,134,219,134,228,134,208,134,222,134,87,136,193,136,194,136,177,136,131,137,150,137,59,138,96,138,85,138,94,138,60,138,65,138,84,138,91,138,80,138,70,138,52,138,58,138,54,138,86,138,97,140,130,140,175,140,188,140,179,140,189,140,193,140,187,140,192,140,180,140,183,140,182,140,191,140,184,140,138,141,133,141,129,141,206,141,221,141,203,141,218,141,209,141,204,141,219,141,198,141,251,142,248,142,252,142,156,143,46,144,53,144,49,144,56,144,50,144,54,144,2,145,245,144,9,145,254,144,99,145,101,145,207,145,20,146,21,146,35,146,9,146,30,146,13,146,16,146,7,146,17,146,148,149,143,149,139,149,145,149,147,149,146,149,142,149,138,150,142,150,139,150,125,150,133,150,134,150,141,150,114,150,132,150,193,150,197,150,196,150,198,150,199,150,239,150,242,150,204,151,5,152,6,152,8,152,231,152,234,152,239,152,233,152,242,152,237,152,174,153,173,153,195,158,205,158,209,158,130,78,173,80,181,80,178,80,179,80,197,80,190,80,172,80,183,80,187,80,175,80,199,80,127,82,119,82,125,82,223,82,230,82,228,82,226,82,227,82,47,83,223,85,232,85,211,85,230,85,206,85,220,85,199,85,209,85,227,85,228,85,239,85,218,85,225,85,197,85,198,85,229,85,201,85,18,87,19,87,94,88,81,88,88,88,87,88,90,88,84,88,107,88,76,88,109,88,74,88,98,88,82,88,75,88,103,89,193,90,201,90,204,90,190,90,189,90,188,90,179,90,194,90,178,90,105,93,111,93,76,94,121,94,201,94,200,94,18,95,89,95,172,95,174,95,26,97,15,97,72,97,31,97,243,96,27,97,249,96,1,97,8,97,78,97,76,97,68,97,77,97,62,97,52,97,39,97,13,97,6,97,55,97,33,98,34,98,19,100,62,100,30,100,42,100,45,100,61,100,44,100,15,100,28,100,20,100,13,100,54,100,22,100,23,100,6,100,108,101,159,101,176,101,151,102,137,102,135,102,136,102,150,102,132,102,152,102,141,102,3,103,148,105,109,105,90,105,119,105,96,105,84,105,117,105,48,105,130,105,74,105,104,105,107,105,94,105,83,105,121,105,134,105,93,105,99,105,91,105,71,107,114,107,192,107,191,107,211,107,253,107,162,110,175,110,211,110,182,110,194,110,144,110,157,110,199,110,197,110,165,110,152,110,188,110,186,110,171,110,209,110,150,110,156,110,196,110,212,110,170,110,167,110,180,110,78,113,89,113,105,113,100,113,73,113,103,113,92,113,108,113,102,113,76,113,101,113,94,113,70,113,104,113,86,113,58,114,82,114,55,115,69,115,63,115,62,115,111,116,90,116,85,116,95,116,94,116,65,116,63,116,89,116,91,116,92,116,118,117,120,117,0,118,240,117,1,118,242,117,241,117,250,117,255,117,244,117,243,117,222,118,223,118,91,119,107,119,102,119,94,119,99,119,121,119,106,119,108,119,92,119,101,119,104,119,98,119,238,119,142,120,176,120,151,120,152,120,140,120,137,120,124,120,145,120,147,120,127,120,122,121,127,121,129,121,44,132,189,121,28,122,26,122,32,122,20,122,31,122,30,122,159,122,160,122,119,123,192,123,96,123,110,123,103,123,177,124,179,124,181,124,147,125,121,125,145,125,129,125,143,125,91,125,110,127,105,127,106,127,114,127,169,127,168,127,164,127,86,128,88,128,134,128,132,128,113,129,112,129,120,129,101,129,110,129,115,129,107,129,121,129,122,129,102,129,5,130,71,130,130,132,119,132,61,132,49,132,117,132,102,132,107,132,73,132,108,132,91,132,60,132,53,132,97,132,99,132,105,132,109,132,70,132,94,134,92,134,95,134,249,134,19,135,8,135,7,135,0,135,254,134,251,134,2,135,3,135,6,135,10,135,89,136,223,136,212,136,217,136,220,136,216,136,221,136,225,136,202,136,213,136,210,136,156,137,227,137,107,138,114,138,115,138,102,138,105,138,112,138,135,138,124,138,99,138,160,138,113,138,133,138,109,138,98,138,110,138,108,138,121,138,123,138,62,138,104,138,98,140,138,140,137,140,202,140,199,140,200,140,196,140,178,140,195,140,194,140,197,140,225,141,223,141,232,141,239,141,243,141,250,141,234,141,228,141,230,141,178,142,3,143,9,143,254,142,10,143,159,143,178,143,75,144,74,144,83,144,66,144,84,144,60,144,85,144,80,144,71,144,79,144,78,144,77,144,81,144,62,144,65,144,18,145,23,145,108,145,106,145,105,145,201,145,55,146,87,146,56,146,61,146,64,146,62,146,91,146,75,146,100,146,81,146,52,146,73,146,77,146,69,146,57,146,63,146,90,146,152,149,152,150,148,150,149,150,205,150,203,150,201,150,202,150,247,150,251,150,249,150,246,150,86,151,116,151,118,151,16,152,17,152,19,152,10,152,18,152,12,152,252,152,244,152,253,152,254,152,179,153,177,153,180,153,225,154,233,156,130,158,14,159,19,159,32,159,231,80,238,80,229,80,214,80,237,80,218,80,213,80,207,80,209,80,241,80,206,80,233,80,98,81,243,81,131,82,130,82,49,83,173,83,254,85,0,86,27,86,23,86,253,85,20,86,6,86,9,86,13,86,14,86,247,85,22,86,31,86,8,86,16,86,246,85,24,87,22,87,117,88,126,88,131,88,147,88,138,88,121,88,133,88,125,88,253,88,37,89,34,89,36,89,106,89,105,89,225,90,230,90,233,90,215,90,214,90,216,90,227,90,117,91,222,91,231,91,225,91,229,91,230,91,232,91,226,91,228,91,223,91,13,92,98,92,132,93,135,93,91,94,99,94,85,94,87,94,84,94,211,94,214,94,10,95,70,95,112,95,185,95,71,97,63,97,75,97,119,97,98,97,99,97,95,97,90,97,88,97,117,97,42,98,135,100,88,100,84,100,164,100,120,100,95,100,122,100,81,100,103,100,52,100,109,100,123,100,114,101,161,101,215,101,214,101,162,102,168,102,157,102,156,105,168,105,149,105,193,105,174,105,211,105,203,105,155,105,183,105,187,105,171,105,180,105,208,105,205,105,173,105,204,105,166,105,195,105,163,105,73,107,76,107,51,108,51,111,20,111,254,110,19,111,244,110,41,111,62,111,32,111,44,111,15,111,2,111,34,111,255,110,239,110,6,111,49,111,56,111,50,111,35,111,21,111,43,111,47,111,136,111,42,111,236,110,1,111,242,110,204,110,247,110,148,113,153,113,125,113,138,113,132,113,146,113,62,114,146,114,150,114,68,115,80,115,100,116,99,116,106,116,112,116,109,116,4,117,145,117,39,118,13,118,11,118,9,118,19,118,225,118,227,118,132,119,125,119,127,119,97,119,193,120,159,120,167,120,179,120,169,120,163,120,142,121,143,121,141,121,46,122,49,122,170,122,169,122,237,122,239,122,161,123,149,123,139,123,117,123,151,123,157,123,148,123,143,123,184,123,135,123,132,123,185,124,189,124,190,124,187,125,176,125,156,125,189,125,190,125,160,125,202,125,180,125,178,125,177,125,186,125,162,125,191,125,181,125,184,125,173,125,210,125,199,125,172,125,112,127,224,127,225,127,223,127,94,128,90,128,135,128,80,129,128,129,143,129,136,129,138,129,127,129,130,129,231,129,250,129,7,130,20,130,30,130,75,130,201,132,191,132,198,132,196,132,153,132,158,132,178,132,156,132,203,132,184,132,192,132,211,132,144,132,188,132,209,132,202,132,63,135,28,135,59,135,34,135,37,135,52,135,24,135,85,135,55,135,41,135,243,136,2,137,244,136,249,136,248,136,253,136,232,136,26,137,239,136,166,138,140,138,158,138,163,138,141,138,161,138,147,138,164,138,170,138,165,138,168,138,152,138,145,138,154,138,167,138,106,140,141,140,140,140,211,140,209,140,210,140,107,141,153,141,149,141,252,141,20,143,18,143,21,143,19,143,163,143,96,144,88,144,92,144,99,144,89,144,94,144,98,144,93,144,91,144,25,145,24,145,30,145,117,145,120,145,119,145,116,145,120,146,128,146,133,146,152,146,150,146,123,146,147,146,156,146,168,146,124,146,145,146,161,149,168,149,169,149,163,149,165,149,164,149,153,150,156,150,155,150,204,150,210,150,0,151,124,151,133,151,246,151,23,152,24,152,175,152,177,152,3,153,5,153,12,153,9,153,193,153,175,154,176,154,230,154,65,155,66,155,244,156,246,156,243,156,188,158,59,159,74,159,4,81,0,81,251,80,245,80,249,80,2,81,8,81,9,81,5,81,220,81,135,82,136,82,137,82,141,82,138,82,240,82,178,83,46,86,59,86,57,86,50,86,63,86,52,86,41,86,83,86,78,86,87,86,116,86,54,86,47,86,48,86,128,88,159,88,158,88,179,88,156,88,174,88,169,88,166,88,109,89,9,91,251,90,11,91,245,90,12,91,8,91,238,91,236,91,233,91,235,91,100,92,101,92,157,93,148,93,98,94,95,94,97,94,226,94,218,94,223,94,221,94,227,94,224,94,72,95,113,95,183,95,181,95,118,97,103,97,110,97,93,97,85,97,130,97,124,97,112,97,107,97,126,97,167,97,144,97,171,97,142,97,172,97,154,97,164,97,148,97,174,97,46,98,105,100,111,100,121,100,158,100,178,100,136,100,144,100,176,100,165,100,147,100,149,100,169,100,146,100,174,100,173,100,171,100,154,100,172,100,153,100,162,100,179,100,117,101,119,101,120,101,174,102,171,102,180,102,177,102,35,106,31,106,232,105,1,106,30,106,25,106,253,105,33,106,19,106,10,106,243,105,2,106,5,106,237,105,17,106,80,107,78,107,164,107,197,107,198,107,63,111,124,111,132,111,81,111,102,111,84,111,134,111,109,111,91,111,120,111,110,111,142,111,122,111,112,111,100,111,151,111,88,111,213,110,111,111,96,111,95,111,159,113,172,113,177,113,168,113,86,114,155,114,78,115,87,115,105,116,139,116,131,116,126,116,128,116,127,117,32,118,41,118,31,118,36,118,38,118,33,118,34,118,154,118,186,118,228,118,142,119,135,119,140,119,145,119,139,119,203,120,197,120,186,120,202,120,190,120,213,120,188,120,208,120,63,122,60,122,64,122,61,122,55,122,59,122,175,122,174,122,173,123,177,123,196,123,180,123,198,123,199,123,193,123,160,123,204,123,202,124,224,125,244,125,239,125,251,125,216,125,236,125,221,125,232,125,227,125,218,125,222,125,233,125,158,125,217,125,242,125,249,125,117,127,119,127,175,127,233,127,38,128,155,129,156,129,157,129,160,129,154,129,152,129,23,133,61,133,26,133,238,132,44,133,45,133,19,133,17,133,35,133,33,133,20,133,236,132,37,133,255,132,6,133,130,135,116,135,118,135,96,135,102,135,120,135,104,135,89,135,87,135,76,135,83,135,91,136,93,136,16,137,7,137,18,137,19,137,21,137,10,137,188,138,210,138,199,138,196,138,149,138,203,138,248,138,178,138,201,138,194,138,191,138,176,138,214,138,205,138,182,138,185,138,219,138,76,140,78,140,108,140,224,140,222,140,230,140,228,140,236,140,237,140,226,140,227,140,220,140,234,140,225,140,109,141,159,141,163,141,43,142,16,142,29,142,34,142,15,142,41,142,31,142,33,142,30,142])
.concat([186,142,29,143,27,143,31,143,41,143,38,143,42,143,28,143,30,143,37,143,105,144,110,144,104,144,109,144,119,144,48,145,45,145,39,145,49,145,135,145,137,145,139,145,131,145,197,146,187,146,183,146,234,146,172,146,228,146,193,146,179,146,188,146,210,146,199,146,240,146,178,146,173,149,177,149,4,151,6,151,7,151,9,151,96,151,141,151,139,151,143,151,33,152,43,152,28,152,179,152,10,153,19,153,18,153,24,153,221,153,208,153,223,153,219,153,209,153,213,153,210,153,217,153,183,154,238,154,239,154,39,155,69,155,68,155,119,155,111,155,6,157,9,157,3,157,169,158,190,158,206,158,168,88,82,159,18,81,24,81,20,81,16,81,21,81,128,81,170,81,221,81,145,82,147,82,243,82,89,86,107,86,121,86,105,86,100,86,120,86,106,86,104,86,101,86,113,86,111,86,108,86,98,86,118,86,193,88,190,88,199,88,197,88,110,89,29,91,52,91,120,91,240,91,14,92,74,95,178,97,145,97,169,97,138,97,205,97,182,97,190,97,202,97,200,97,48,98,197,100,193,100,203,100,187,100,188,100,218,100,196,100,199,100,194,100,205,100,191,100,210,100,212,100,190,100,116,101,198,102,201,102,185,102,196,102,199,102,184,102,61,106,56,106,58,106,89,106,107,106,88,106,57,106,68,106,98,106,97,106,75,106,71,106,53,106,95,106,72,106,89,107,119,107,5,108,194,111,177,111,161,111,195,111,164,111,193,111,167,111,179,111,192,111,185,111,182,111,166,111,160,111,180,111,190,113,201,113,208,113,210,113,200,113,213,113,185,113,206,113,217,113,220,113,195,113,196,113,104,115,156,116,163,116,152,116,159,116,158,116,226,116,12,117,13,117,52,118,56,118,58,118,231,118,229,118,160,119,158,119,159,119,165,119,232,120,218,120,236,120,231,120,166,121,77,122,78,122,70,122,76,122,75,122,186,122,217,123,17,124,201,123,228,123,219,123,225,123,233,123,230,123,213,124,214,124,10,126,17,126,8,126,27,126,35,126,30,126,29,126,9,126,16,126,121,127,178,127,240,127,241,127,238,127,40,128,179,129,169,129,168,129,251,129,8,130,88,130,89,130,74,133,89,133,72,133,104,133,105,133,67,133,73,133,109,133,106,133,94,133,131,135,159,135,158,135,162,135,141,135,97,136,42,137,50,137,37,137,43,137,33,137,170,137,166,137,230,138,250,138,235,138,241,138,0,139,220,138,231,138,238,138,254,138,1,139,2,139,247,138,237,138,243,138,246,138,252,138,107,140,109,140,147,140,244,140,68,142,49,142,52,142,66,142,57,142,53,142,59,143,47,143,56,143,51,143,168,143,166,143,117,144,116,144,120,144,114,144,124,144,122,144,52,145,146,145,32,147,54,147,248,146,51,147,47,147,34,147,252,146,43,147,4,147,26,147,16,147,38,147,33,147,21,147,46,147,25,147,187,149,167,150,168,150,170,150,213,150,14,151,17,151,22,151,13,151,19,151,15,151,91,151,92,151,102,151,152,151,48,152,56,152,59,152,55,152,45,152,57,152,36,152,16,153,40,153,30,153,27,153,33,153,26,153,237,153,226,153,241,153,184,154,188,154,251,154,237,154,40,155,145,155,21,157,35,157,38,157,40,157,18,157,27,157,216,158,212,158,141,159,156,159,42,81,31,81,33,81,50,81,245,82,142,86,128,86,144,86,133,86,135,86,143,86,213,88,211,88,209,88,206,88,48,91,42,91,36,91,122,91,55,92,104,92,188,93,186,93,189,93,184,93,107,94,76,95,189,95,201,97,194,97,199,97,230,97,203,97,50,98,52,98,206,100,202,100,216,100,224,100,240,100,230,100,236,100,241,100,226,100,237,100,130,101,131,101,217,102,214,102,128,106,148,106,132,106,162,106,156,106,219,106,163,106,126,106,151,106,144,106,160,106,92,107,174,107,218,107,8,108,216,111,241,111,223,111,224,111,219,111,228,111,235,111,239,111,128,111,236,111,225,111,233,111,213,111,238,111,240,111,231,113,223,113,238,113,230,113,229,113,237,113,236,113,244,113,224,113,53,114,70,114,112,115,114,115,169,116,176,116,166,116,168,116,70,118,66,118,76,118,234,118,179,119,170,119,176,119,172,119,167,119,173,119,239,119,247,120,250,120,244,120,239,120,1,121,167,121,170,121,87,122,191,122,7,124,13,124,254,123,247,123,12,124,224,123,224,124,220,124,222,124,226,124,223,124,217,124,221,124,46,126,62,126,70,126,55,126,50,126,67,126,43,126,61,126,49,126,69,126,65,126,52,126,57,126,72,126,53,126,63,126,47,126,68,127,243,127,252,127,113,128,114,128,112,128,111,128,115,128,198,129,195,129,186,129,194,129,192,129,191,129,189,129,201,129,190,129,232,129,9,130,113,130,170,133,132,133,126,133,156,133,145,133,148,133,175,133,155,133,135,133,168,133,138,133,103,134,192,135,209,135,179,135,210,135,198,135,171,135,187,135,186,135,200,135,203,135,59,137,54,137,68,137,56,137,61,137,172,137,14,139,23,139,25,139,27,139,10,139,32,139,29,139,4,139,16,139,65,140,63,140,115,140,250,140,253,140,252,140,248,140,251,140,168,141,73,142,75,142,72,142,74,142,68,143,62,143,66,143,69,143,63,143,127,144,125,144,132,144,129,144,130,144,128,144,57,145,163,145,158,145,156,145,77,147,130,147,40,147,117,147,74,147,101,147,75,147,24,147,126,147,108,147,91,147,112,147,90,147,84,147,202,149,203,149,204,149,200,149,198,149,177,150,184,150,214,150,28,151,30,151,160,151,211,151,70,152,182,152,53,153,1,154,255,153,174,155,171,155,170,155,173,155,59,157,63,157,139,158,207,158,222,158,220,158,221,158,219,158,62,159,75,159,226,83,149,86,174,86,217,88,216,88,56,91,93,95,227,97,51,98,244,100,242,100,254,100,6,101,250,100,251,100,247,100,183,101,220,102,38,103,179,106,172,106,195,106,187,106,184,106,194,106,174,106,175,106,95,107,120,107,175,107,9,112,11,112,254,111,6,112,250,111,17,112,15,112,251,113,252,113,254,113,248,113,119,115,117,115,167,116,191,116,21,117,86,118,88,118,82,118,189,119,191,119,187,119,188,119,14,121,174,121,97,122,98,122,96,122,196,122,197,122,43,124,39,124,42,124,30,124,35,124,33,124,231,124,84,126,85,126,94,126,90,126,97,126,82,126,89,126,72,127,249,127,251,127,119,128,118,128,205,129,207,129,10,130,207,133,169,133,205,133,208,133,201,133,176,133,186,133,185,133,166,133,239,135,236,135,242,135,224,135,134,137,178,137,244,137,40,139,57,139,44,139,43,139,80,140,5,141,89,142,99,142,102,142,100,142,95,142,85,142,192,142,73,143,77,143,135,144,131,144,136,144,171,145,172,145,208,145,148,147,138,147,150,147,162,147,179,147,174,147,172,147,176,147,152,147,154,147,151,147,212,149,214,149,208,149,213,149,226,150,220,150,217,150,219,150,222,150,36,151,163,151,166,151,173,151,249,151,77,152,79,152,76,152,78,152,83,152,186,152,62,153,63,153,61,153,46,153,165,153,14,154,193,154,3,155,6,155,79,155,78,155,77,155,202,155,201,155,253,155,200,155,192,155,81,157,93,157,96,157,224,158,21,159,44,159,51,81,165,86,222,88,223,88,226,88,245,91,144,159,236,94,242,97,247,97,246,97,245,97,0,101,15,101,224,102,221,102,229,106,221,106,218,106,211,106,27,112,31,112,40,112,26,112,29,112,21,112,24,112,6,114,13,114,88,114,162,114,120,115,122,115,189,116,202,116,227,116,135,117,134,117,95,118,97,118,199,119,25,121,177,121,107,122,105,122,62,124,63,124,56,124,61,124,55,124,64,124,107,126,109,126,121,126,105,126,106,126,133,127,115,126,182,127,185,127,184,127,216,129,233,133,221,133,234,133,213,133,228,133,229,133,247,133,251,135,5,136,13,136,249,135,254,135,96,137,95,137,86,137,94,137,65,139,92,139,88,139,73,139,90,139,78,139,79,139,70,139,89,139,8,141,10,141,124,142,114,142,135,142,118,142,108,142,122,142,116,142,84,143,78,143,173,143,138,144,139,144,177,145,174,145,225,147,209,147,223,147,195,147,200,147,220,147,221,147,214,147,226,147,205,147,216,147,228,147,215,147,232,147,220,149,180,150,227,150,42,151,39,151,97,151,220,151,251,151,94,152,88,152,91,152,188,152,69,153,73,153,22,154,25,154,13,155,232,155,231,155,214,155,219,155,137,157,97,157,114,157,106,157,108,157,146,158,151,158,147,158,180,158,248,82,168,86,183,86,182,86,180,86,188,86,228,88,64,91,67,91,125,91,246,91,201,93,248,97,250,97,24,101,20,101,25,101,230,102,39,103,236,106,62,112,48,112,50,112,16,114,123,115,207,116,98,118,101,118,38,121,42,121,44,121,43,121,199,122,246,122,76,124,67,124,77,124,239,124,240,124,174,143,125,126,124,126,130,126,76,127,0,128,218,129,102,130,251,133,249,133,17,134,250,133,6,134,11,134,7,134,10,134,20,136,21,136,100,137,186,137,248,137,112,139,108,139,102,139,111,139,95,139,107,139,15,141,13,141,137,142,129,142,133,142,130,142,180,145,203,145,24,148,3,148,253,147,225,149,48,151,196,152,82,153,81,153,168,153,43,154,48,154,55,154,53,154,19,156,13,156,121,158,181,158,232,158,47,159,95,159,99,159,97,159,55,81,56,81,193,86,192,86,194,86,20,89,108,92,205,93,252,97,254,97,29,101,28,101,149,101,233,102,251,106,4,107,250,106,178,107,76,112,27,114,167,114,214,116,212,116,105,118,211,119,80,124,143,126,140,126,188,127,23,134,45,134,26,134,35,136,34,136,33,136,31,136,106,137,108,137,189,137,116,139,119,139,125,139,19,141,138,142,141,142,139,142,95,143,175,143,186,145,46,148,51,148,53,148,58,148,56,148,50,148,43,148,226,149,56,151,57,151,50,151,255,151,103,152,101,152,87,153,69,154,67,154,64,154,62,154,207,154,84,155,81,155,45,156,37,156,175,157,180,157,194,157,184,157,157,158,239,158,25,159,92,159,102,159,103,159,60,81,59,81,200,86,202,86,201,86,127,91,212,93,210,93,78,95,255,97,36,101,10,107,97,107,81,112,88,112,128,115,228,116,138,117,110,118,108,118,179,121,96,124,95,124,126,128,125,128,223,129,114,137,111,137,252,137,128,139,22,141,23,141,145,142,147,142,97,143,72,145,68,148,81,148,82,148,61,151,62,151,195,151,193,151,107,152,85,153,85,154,77,154,210,154,26,155,73,156,49,156,62,156,59,156,211,157,215,157,52,159,108,159,106,159,148,159,204,86,214,93,0,98,35,101,43,101,42,101,236,102,16,107,218,116,202,122,100,124,99,124,101,124,147,126,150,126,148,126,226,129,56,134,63,134,49,136,138,139,144,144,143,144,99,148,96,148,100,148,104,151,111,152,92,153,90,154,91,154,87,154,211,154,212,154,209,154,84,156,87,156,86,156,229,157,159,158,244,158,209,86,233,88,44,101,94,112,113,118,114,118,215,119,80,127,136,127,54,136,57,136,98,136,147,139,146,139,150,139,119,130,27,141,192,145,106,148,66,151,72,151,68,151,198,151,112,152,95,154,34,155,88,155,95,156,249,157,250,157,124,158,125,158,7,159,119,159,114,159,243,94,22,107,99,112,108,124,110,124,59,136,192,137,161,142,193,145,114,148,112,148,113,152,94,153,214,154,35,155,204,158,100,112,218,119,154,139,119,148,201,151,98,154,101,154,156,126,156,139,170,142,197,145,125,148,126,148,124,148,119,156,120,156,247,158,84,140,127,148,26,158,40,114,106,154,49,155,27,158,30,158,114,124,96,36,97,36,98,36,99,36,100,36,101,36,102,36,103,36,104,36,105,36,116,36,117,36,118,36,119,36,120,36,121,36,122,36,123,36,124,36,125,36,112,33,113,33,114,33,115,33,116,33,117,33,118,33,119,33,120,33,121,33,54,78,63,78,133,78,160,78,130,81,150,81,171,81,249,82,56,83,105,83,182,83,10,89,128,91,219,93,122,94,127,94,244,94,80,95,97,95,52,101,224,101,146,117,118,118,181,143,182,150,168,0,198,2,253,48,254,48,157,48,158,48,0,0,0,0,5,48,6,48,7,48,252,48,59,255,61,255,61,39,65,48,66,48,67,48,68,48,69,48,70,48,71,48,72,48,73,48,74,48,75,48,76,48,77,48,78,48,79,48,80,48,81,48,82,48,83,48,84,48,85,48,86,48,87,48,88,48,89,48,90,48,91,48,92,48,93,48,94,48,95,48,96,48,97,48,98,48,99,48,100,48,101,48,102,48,103,48,104,48,105,48,106,48,107,48,108,48,109,48,110,48,111,48,112,48,113,48,114,48,115,48,116,48,117,48,118,48,119,48,120,48,121,48,122,48,123,48,124,48,125,48,126,48,127,48,128,48,129,48,130,48,131,48,132,48,133,48,134,48,135,48,136,48,137,48,138,48,139,48,140,48,141,48,142,48,143,48,144,48,145,48,146,48,147,48,161,48,162,48,163,48,164,48,165,48,166,48,167,48,168,48,169,48,170,48,171,48,172,48,173,48,174,48,175,48,176,48,177,48,178,48,179,48,180,48,181,48,182,48,183,48,184,48,185,48,186,48,187,48,188,48,189,48,190,48,191,48,192,48,193,48,194,48,195,48,196,48,197,48,198,48,199,48,200,48,201,48,202,48,203,48,204,48,205,48,206,48,207,48,208,48,209,48,210,48,211,48,212,48,213,48,214,48,215,48,216,48,217,48,218,48,219,48,220,48,221,48,222,48,223,48,224,48,225,48,226,48,227,48,228,48,229,48,230,48,231,48,232,48,233,48,234,48,235,48,236,48,237,48,238,48,239,48,240,48,241,48,242,48,243,48,244,48,245,48,246,48,16,4,17,4,18,4,19,4,20,4,21,4,1,4,22,4,23,4,24,4,25,4,26,4,27,4,28,4,29,4,30,4,31,4,32,4,33,4,34,4,35,4,36,4,37,4,38,4,39,4,40,4,41,4,42,4,43,4,44,4,45,4,46,4,47,4,48,4,49,4,50,4,51,4,52,4,53,4,81,4,54,4,55,4,56,4,57,4,58,4,59,4,60,4,61,4,62,4,63,4,64,4,65,4,66,4,67,4,68,4,69,4,70,4,71,4,72,4,73,4,74,4,75,4,76,4,77,4,78,4,79,4,231,33,184,33,185,33,207,49,204,0,90,78,138,0,2,82,145,68,176,159,136,81,177,159,7,118,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,226,255,228,255,7,255,2,255,49,50,22,33,33,33,155,48,156,48,128,46,132,46,134,46,135,46,136,46,138,46,140,46,141,46,149,46,156,46,157,46,165,46,167,46,170,46,172,46,174,46,182,46,188,46,190,46,198,46,202,46,204,46,205,46,207,46,214,46,215,46,222,46,227,46,0,0,0,0,0,0,131,2,80,2,91,2,84,2,117,2,83,1,248,0,75,1,138,2,106,2,66,78,92,78,245,81,26,83,130,83,7,78,12,78,71,78,141,78,215,86,12,250,110,92,115,95,15,78,135,81,14,78,46,78,147,78,194,78,201,78,200,78,152,81,252,82,108,83,185,83,32,87,3,89,44,89,16,92,255,93,225,101,179,107,204,107,20,108,63,114,49,78,60,78,232,78,220,78,233,78,225,78,221,78,218,78,12,82,28,83,76,83,34,87,35,87,23,89,47,89,129,91,132,91,18,92,59,92,116,92,115,92,4,94,128,94,130,94,201,95,9,98,80,98,21,108,54,108,67,108,63,108,59,108,174,114,176,114,138,115,184,121,138,128,30,150,14,79,24,79,44,79,245,78,20,79,241,78,0,79,247,78,8,79,29,79,2,79,5,79,34,79,19,79,4,79,244,78,18,79,177,81,19,82,9,82,16,82,166,82,34,83,31,83,77,83,138,83,7,84,225,86,223,86,46,87,42,87,52,87,60,89,128,89,124,89,133,89,123,89,126,89,119,89,127,89,86,91,21,92,37,92,124,92,122,92,123,92,126,92,223,93,117,94,132,94,2,95,26,95,116,95,213,95,212,95,207,95,92,98,94,98,100,98,97,98,102,98,98,98,89,98,96,98,90,98,101,98,239,101,238,101,62,103,57,103,56,103,59,103,58,103,63,103,60,103,51,103,24,108,70,108,82,108,92,108,79,108,74,108,84,108,75,108,76,108,113,112,94,114,180,114,181,114,142,115,42,117,127,118,117,122,81,127,120,130,124,130,128,130,125,130,127,130,77,134,126,137,153,144,151,144,152,144,155,144,148,144,34,150,36,150,32,150,35,150,86,79,59,79,98,79,73,79,83,79,100,79,62,79,103,79,82,79,95,79,65,79,88,79,45,79,51,79,63,79,97,79,143,81,185,81,28,82,30,82,33,82,173,82,174,82,9,83,99,83,114,83,142,83,143,83,48,84,55,84,42,84,84,84,69,84,25,84,28,84,37,84,24,84,61,84,79,84,65,84,40,84,36,84,71,84,238,86,231,86,229,86,65,87,69,87,76,87,73,87,75,87,82,87,6,89,64,89,166,89,152,89,160,89,151,89,142,89,162,89,144,89,143,89,167,89,161,89,142,91,146,91,40,92,42,92,141,92,143,92,136,92,139,92,137,92,146,92,138,92,134,92,147,92,149,92,224,93,10,94,14,94,139,94,137,94,140,94,136,94,141,94,5,95,29,95,120,95,118,95,210,95,209,95,208,95,237,95,232,95,238,95,243,95,225,95,228,95,227,95,250,95,239,95,247,95,251,95,0,96,244,95,58,98,131,98,140,98,142,98,143,98,148,98,135,98,113,98,123,98,122,98,112,98,129,98,136,98,119,98,125,98,114,98,116,98,55,101,240,101,244,101,243,101,242,101,245,101,69,103,71,103,89,103,85,103,76,103,72,103,93,103,77,103,90,103,75,103,208,107,25,108,26,108,120,108,103,108,107,108,132,108,139,108,143,108,113,108,111,108,105,108,154,108,109,108,135,108,149,108,156,108,102,108,115,108,101,108,123,108,142,108,116,112,122,112,99,114,191,114,189,114,195,114,198,114,193,114,186,114,197,114,149,115,151,115,147,115,148,115,146,115,58,117,57,117,148,117,149,117,129,118,61,121,52,128,149,128,153,128,144,128,146,128,156,128,144,130,143,130,133,130,142,130,145,130,147,130,138,130,131,130,132,130,120,140,201,143,191,143,159,144,161,144,165,144,158,144,167,144,160,144,48,150,40,150,47,150,45,150,51,78,152,79,124,79,133,79,125,79,128,79,135,79,118,79,116,79,137,79,132,79,119,79,76,79,151,79,106,79,154,79,121,79,129,79,120,79,144,79,156,79,148,79,158,79,146,79,130,79,149,79,107,79,110,79,158,81,188,81,190,81,53,82,50,82,51,82,70,82,49,82,188,82,10,83,11,83,60,83,146,83,148,83,135,84,127,84,129,84,145,84,130,84,136,84,107,84,122,84,126,84,101,84,108,84,116,84,102,84,141,84,111,84,97,84,96,84,152,84,99,84,103,84,100,84,247,86,249,86,111,87,114,87,109,87,107,87,113,87,112,87,118,87,128,87,117,87,123,87,115,87,116,87,98,87,104,87,125,87,12,89,69,89,181,89,186,89,207,89,206,89,178,89,204,89,193,89,182,89,188,89,195,89,214,89,177,89,189,89,192,89,200,89,180,89,199,89,98,91,101,91,147,91,149,91,68,92,71,92,174,92,164,92,160,92,181,92,175,92,168,92,172,92,159,92,163,92,173,92,162,92,170,92,167,92,157,92,165,92,182,92,176,92,166,92,23,94,20,94,25,94,40,95,34,95,35,95,36,95,84,95,130,95,126,95,125,95,222,95,229,95,45,96,38,96,25,96,50,96,11,96,52,96,10,96,23,96,51,96,26,96,30,96,44,96,34,96,13,96,16,96,46,96,19,96,17,96,12,96,9,96,28,96,20,98,61,98,173,98,180,98,209,98,190,98,170,98,182,98,202,98,174,98,179,98,175,98,187,98,169,98,176,98,184,98,61,101,168,101,187,101,9,102,252,101,4,102,18,102,8,102,251,101,3,102,11,102,13,102,5,102,253,101,17,102,16,102,246,102,10,103,133,103,108,103,142,103,146,103,118,103,123,103,152,103,134,103,132,103,116,103,141,103,140,103,122,103,159,103,145,103,153,103,131,103,125,103,129,103,120,103,121,103,148,103,37,107,128,107,126,107,222,107,29,108,147,108,236,108,235,108,238,108,217,108,182,108,212,108,173,108,231,108,183,108,208,108,194,108,186,108,195,108,198,108,237,108,242,108,210,108,221,108,180,108,138,108,157,108,128,108,222,108,192,108,48,109,205,108,199,108,176,108,249,108,207,108,233,108,209,108,148,112,152,112,133,112,147,112,134,112,132,112,145,112,150,112,130,112,154,112,131,112,106,114,214,114,203,114,216,114,201,114,220,114,210,114,212,114,218,114,204,114,209,114,164,115,161,115,173,115,166,115,162,115,160,115,172,115,157,115,221,116,232,116,63,117,64,117,62,117,140,117,152,117,175,118,243,118,241,118,240,118,245,118,248,119,252,119,249,119,251,119,250,119,247,119,66,121,63,121,197,121,120,122,123,122,251,122,117,124,253,124,53,128,143,128,174,128,163,128,184,128,181,128,173,128,32,130,160,130,192,130,171,130,154,130,152,130,155,130,181,130,167,130,174,130,188,130,158,130,186,130,180,130,168,130,161,130,169,130,194,130,164,130,195,130,182,130,162,130,112,134,111,134,109,134,110,134,86,140,210,143,203,143,211,143,205,143,214,143,213,143,215,143,178,144,180,144,175,144,179,144,176,144,57,150,61,150,60,150,58,150,67,150,205,79,197,79,211,79,178,79,201,79,203,79,193,79,212,79,220,79,217,79,187,79,179,79,219,79,199,79,214,79,186,79,192,79,185,79,236,79,68,82,73,82,192,82,194,82,61,83,124,83,151,83,150,83,153,83,152,83,186,84,161,84,173,84,165,84,207,84,195,84,13,131,183,84,174,84,214,84,182,84,197,84,198,84,160,84,112,84,188,84,162,84,190,84,114,84,222,84,176,84,181,87,158,87,159,87,164,87,140,87,151,87,157,87,155,87,148,87,152,87,143,87,153,87,165,87,154,87,149,87,244,88,13,89,83,89,225,89,222,89,238,89,0,90,241,89,221,89,250,89,253,89,252,89,246,89,228,89,242,89,247,89,219,89,233,89,243,89,245,89,224,89,254,89,244,89,237,89,168,91,76,92,208,92,216,92,204,92,215,92,203,92,219,92,222,92,218,92,201,92,199,92,202,92,214,92,211,92,212,92,207,92,200,92,198,92,206,92,223,92,248,92,249,93,33,94,34,94,35,94,32,94,36,94,176,94,164,94,162,94,155,94,163,94,165,94,7,95,46,95,86,95,134,95,55,96,57,96,84,96,114,96,94,96,69,96,83,96,71,96,73,96,91,96,76,96,64,96,66,96,95,96,36,96,68,96,88,96,102,96,110,96,66,98,67,98,207,98,13,99,11,99,245,98,14,99,3,99,235,98,249,98,15,99,12,99,248,98,246,98,0,99,19,99,20,99,250,98,21,99,251,98,240,98,65,101,67,101,170,101,191,101,54,102,33,102,50,102,53,102,28,102,38,102,34,102,51,102,43,102,58,102,29,102,52,102,57,102,46,102,15,103,16,103,193,103,242,103,200,103,186,103,220,103,187,103,248,103,216,103,192,103,183,103,197,103,235,103,228,103,223,103,181,103,205,103,179,103,247,103,246,103,238,103,227,103,194,103,185,103,206,103,231,103,240,103,178,103,252,103,198,103,237,103,204,103,174,103,230,103,219,103,250,103,201,103,202,103,195,103,234,103,203,103,40,107,130,107,132,107,182,107,214,107,216,107,224,107,32,108,33,108,40,109,52,109,45,109,31,109,60,109,63,109,18,109,10,109,218,108,51,109,4,109,25,109,58,109,26,109,17,109,0,109,29,109,66,109,1,109,24,109,55,109,3,109,15,109,64,109,7,109,32,109,44,109,8,109,34,109,9,109,16,109,183,112,159,112,190,112,177,112,176,112,161,112,180,112,181,112,169,112,65,114,73,114,74,114,108,114,112,114,115,114,110,114,202,114,228,114,232,114,235,114,223,114,234,114,230,114,227,114,133,115,204,115,194,115,200,115,197,115,185,115,182,115,181,115,180,115,235,115,191,115,199,115,190,115,195,115,198,115,184,115,203,115,236,116,238,116,46,117,71,117,72,117,167,117,170,117,121,118,196,118,8,119,3,119,4,119,5,119,10,119,247,118,251,118,250,118,231,119,232,119,6,120,17,120,18,120,5,120,16,120,15,120,14,120,9,120,3,120,19,120,74,121,76,121,75,121,69,121,68,121,213,121,205,121,207,121,214,121,206,121,128,122,126,122,209,122,0,123,1,123,122,124,120,124,121,124,127,124,128,124,129,124,3,125,8,125,1,125,88,127,145,127,141,127,190,127,7,128,14,128,15,128,20,128,55,128,216,128,199,128,224,128,209,128,200,128,194,128,208,128,197,128,227,128,217,128,220,128,202,128,213,128,201,128,207,128,215,128,230,128,205,128,255,129,33,130,148,130,217,130,254,130,249,130,7,131,232,130,0,131,213,130,58,131,235,130,214,130,244,130,236,130,225,130,242,130,245,130,12,131,251,130,246,130,240,130,234,130,228,130,224,130,250,130,243,130,237,130,119,134,116,134,124,134,115,134,65,136,78,136,103,136,106,136,105,136,211,137,4,138,7,138,114,141,227,143,225,143,238,143,224,143,241,144,189,144,191,144,213,144,197,144,190,144,199,144,203,144,200,144,212,145,211,145,84,150,79,150,81,150,83,150,74,150,78,150,30,80,5,80,7,80,19,80,34,80,48,80,27,80,245,79,244,79,51,80,55,80,44,80,246,79,247,79,23,80,28,80,32,80,39,80,53,80,47,80,49,80,14,80,90,81,148,81,147,81,202,81,196,81,197,81,200,81,206,81,97,82,90,82,82,82,94,82,95,82,85,82,98,82,205,82,14,83,158,83,38,85,226,84,23,85,18,85,231,84,243,84,228,84,26,85,255,84,4,85,8,85,235,84,17,85,5,85,241,84,10,85,251,84,247,84,248,84,224,84,14,85,3,85,11,85,1,87,2,87,204,87,50,88,213,87,210,87,186,87,198,87,189,87,188,87,184,87,182,87,191,87,199,87,208,87,185,87,193,87,14,89,74,89,25,90,22,90,45,90,46,90,21,90,15,90,23,90,10,90,30,90,51,90,108,91,167,91,173,91,172,91,3,92,86,92,84,92,236,92,255,92,238,92,241,92,247,92,0,93,249,92,41,94,40,94,168,94,174,94,170,94,172,94,51,95,48,95,103,95,93,96,90,96,103,96,65,96,162,96,136,96,128,96,146,96,129,96,157,96,131,96,149,96,155,96,151,96,135,96,156,96,142,96,25,98,70,98,242,98,16,99,86,99,44,99,68,99,69,99,54,99,67,99,228,99,57,99,75,99,74,99,60,99,41,99,65,99,52,99,88,99,84,99,89,99,45,99,71,99,51,99,90,99,81,99,56,99,87,99,64,99,72,99,74,101,70,101,198,101,195,101,196,101,194,101,74,102,95,102,71,102,81,102,18,103,19,103,31,104,26,104,73,104,50,104,51,104,59,104,75,104,79,104,22,104,49,104,28,104,53,104,43,104,45,104,47,104,78,104,68,104,52,104,29,104,18,104,20,104,38,104,40,104,46,104,77,104,58,104,37,104,32,104,44,107,47,107,45,107,49,107,52,107,109,107,130,128,136,107,230,107,228,107,232,107,227,107,226,107,231,107,37,108,122,109,99,109,100,109,118,109,13,109,97,109,146,109,88,109,98,109,109,109,111,109,145,109,141,109,239,109,127,109,134,109,94,109,103,109,96,109,151,109,112,109,124,109,95,109,130,109,152,109,47,109,104,109,139,109,126,109,128,109,132,109,22,109,131,109,123,109,125,109,117,109,144,109,220,112,211,112,209,112,221,112,203,112,57,127,226,112,215,112,210,112,222,112,224,112,212,112,205,112,197,112,198,112,199,112,218,112,206,112,225,112,66,114,120,114,119,114,118,114,0,115,250,114,244,114,254,114,246,114,243,114,251,114,1,115,211,115,217,115,229,115,214,115,188,115,231,115,227,115,233,115,220,115,210,115,219,115,212,115,221,115,218,115,215,115,216,115,232,115,222,116,223,116,244,116,245,116,33,117,91,117,95,117,176,117,193,117,187,117,196,117,192,117,191,117,182,117,186,117,138,118,201,118,29,119,27,119,16,119,19,119,18,119,35,119,17,119,21,119,25,119,26,119,34,119,39,119,35,120,44,120,34,120,53,120,47,120,40,120,46,120,43,120,33,120,41,120,51,120,42,120,49,120,84,121,91,121,79,121,92,121,83,121,82,121,81,121,235,121,236,121,224,121,238,121,237,121,234,121,220,121,222,121,221,121,134,122,137,122,133,122,139,122,140,122,138,122,135,122,216,122,16,123,4,123,19,123,5,123,15,123,8,123,10,123,14,123,9,123,18,123,132,124,145,124,138,124,140,124,136,124,141,124,133,124,30,125,29,125,17,125,14,125,24,125,22,125,19,125,31,125,18,125,15,125,12,125,92,127,97,127,94,127,96,127,93,127,91,127,150,127,146,127,195,127,194,127,192,127,22,128,62,128,57,128,250,128,242,128,249,128,245,128,1,129,251,128,0,129,1,130,47,130,37,130,51,131,45,131,68,131,25,131,81,131,37,131,86,131,63,131,65,131,38,131,28,131,34,131,66,131,78,131,27,131,42,131,8,131,60,131,77,131,22,131,36,131,32,131,55,131,47,131,41,131,71,131,69,131,76,131,83,131,30,131,44,131,75,131,39,131,72,131,83,134,82,134,162,134,168,134,150,134,141,134,145,134,158,134,135,134,151,134,134,134,139,134,154,134,133,134,165,134,153,134,161,134,167,134,149,134,152,134,142,134,157,134,144,134,148,134,67,136,68,136,109,136,117,136,118,136,114,136,128,136,113,136,127,136,111,136,131,136,126,136,116,136,124,136,18,138,71,140,87,140,123,140,164,140,163,140,118,141,120,141,181,141,183,141,182,141,209,142,211,142,254,143,245,143,2,144,255,143,251,143,4,144,252,143,246,143,214,144,224,144,217,144,218,144,227,144,223,144,229,144,216,144,219,144,215,144,220,144,228,144,80,145,78,145,79,145,213,145,226,145,218,145,92,150,95,150,188,150,227,152,223,154,47,155,127,78,112,80,106,80,97,80,94,80,96,80,83,80,75,80,93,80,114,80,72,80,77,80,65,80,91,80,74,80,98,80,21,80,69,80,95,80,105,80,107,80,99,80,100,80,70,80,64,80,110,80,115,80,87,80,81,80,208,81,107,82,109,82,108,82,110,82,214,82,211,82,45,83,156,83,117,85,118,85,60,85,77,85,80,85,52,85,42,85,81,85,98,85,54,85,53,85,48,85,82,85,69,85,12,85,50,85,101,85,78,85,57,85,72,85,45,85,59,85,64,85,75,85,10,87,7,87,251,87,20,88,226,87,246,87,220,87,244,87,0,88,237,87,253,87,8,88,248,87,11,88,243,87,207,87,7,88,238,87,227,87,242,87,229,87,236,87,225,87,14,88,252,87,16,88,231,87,1,88,12,88,241,87,233,87,240,87,13,88,4,88,92,89,96,90,88,90,85,90,103,90,94,90,56,90,53,90,109,90,80,90,95,90,101,90,108,90,83,90,100,90,87,90,67,90,93,90,82,90,68,90,91,90,72,90,142,90,62,90,77,90,57,90,76,90,112,90,105,90,71,90,81,90,86,90,66,90,92,90,114,91,110,91,193,91,192,91,89,92,30,93,11,93,29,93,26,93,32,93,12,93,40,93,13,93,38,93,37,93,15,93,48,93,18,93,35,93,31,93,46,93,62,94,52,94,177,94,180,94,185,94,178,94,179,94,54,95,56,95,155,95,150,95,159,95,138,96,144,96,134,96,190,96,176,96,186,96,211,96,212,96,207,96,228,96,217,96,221,96,200,96,177,96,219,96,183,96,202,96,191,96,195,96,205,96,192,96,50,99,101,99,138,99,130,99,125,99,189,99,158,99,173,99,157,99,151,99,171,99,142,99,111,99,135,99,144,99,110,99,175,99,117,99,156,99,109,99,174,99,124,99,164,99,59,99,159,99,120,99,133,99,129,99,145,99,141,99,112,99,83,101,205,101,101,102,97,102,91,102,89,102,92,102,98,102,24,103,121,104,135,104,144,104,156,104,109,104,110,104,174,104,171,104,86,105,111,104,163,104,172,104,169,104,117,104,116,104,178,104,143,104,119,104,146,104,124,104,107,104,114,104,170,104,128,104,113,104,126,104,155,104,150,104,139,104,160,104,137,104,164,104,120,104,123,104,145,104,140,104,138,104,125,104,54,107,51,107,55,107,56,107,145,107,143,107,141,107,142,107,140,107,42,108,192,109,171,109,180,109,179,109,116,110,172,109,233,109,226,109,183,109,246,109,212,109,0,110,200,109,224,109,223,109,214,109,190,109,229,109,220,109,221,109,219,109,244,109,202,109,189,109,237,109,240,109,186,109,213,109,194,109,207,109,201,109,208,109,242,109,211,109,253,109,215,109,205,109,227,109,187,109,250,112,13,113,247,112,23,113,244,112,12,113,240,112,4,113,243,112,16,113,252,112,255,112,6,113,19,113,0,113,248,112,246,112,11,113,2,113,14,113,126,114,123,114,124,114,127,114,29,115,23,115,7,115,17,115,24,115,10,115,8,115,255,114,15,115,30,115,136,115,246,115,248,115,245,115,4,116,1,116,253,115,7,116,0,116,250,115,252,115,255,115,12,116,11,116,244,115,8,116,100,117,99,117,206,117,210,117,207,117,203,117,204,117,209,117,208,117,143,118,137,118,211,118,57,119,47,119,45,119,49,119,50,119,52,119,51,119,61,119,37,119,59,119,53,119,72,120,82,120,73,120,77,120,74,120,76,120,38,120,69,120,80,120,100,121,103,121,105,121,106,121,99,121,107,121,97,121,187,121,250,121,248,121,246,121,247,121,143,122,148,122,144,122,53,123,71,123,52,123,37,123,48,123,34,123,36,123,51,123,24,123,42,123,29,123,49,123,43,123,45,123,47,123,50,123,56,123,26,123,35,123,148,124,152,124,150,124,163,124,53,125,61,125,56,125,54,125,58,125,69,125,44,125,41,125,65,125,71,125,62,125,63,125,74,125,59,125,40,125,99,127,149,127,156,127,157,127,155,127,202,127,203,127,205,127,208,127,209,127,199,127,207,127,201,127,31,128,30,128,27,128,71,128,67,128,72,128,24,129,37,129,25,129,27,129,45,129,31,129,44,129,30,129,33,129,21,129,39,129,29,129,34,129,17,130,56,130,51,130,58,130,52,130,50,130,116,130,144,131,163,131,168,131,141,131,122,131,115,131,164,131,116,131,143,131,129,131,149,131,153,131,117,131,148,131,169,131,125,131,131,131,140,131,157,131,155,131,170,131,139,131,126,131,165,131,175,131,136,131,151,131,176,131,127,131,166,131,135,131,174,131,118,131,154,131,89,134,86,134,191,134,183,134,194,134,193,134,197,134,186,134,176,134,200,134,185,134,179,134,184,134,204,134,180,134,187,134,188,134,195,134,189,134,190,134,82,136,137,136,149,136,168,136,162,136,170,136,154,136,145,136,161,136,159,136,152,136,167,136,153,136,155,136,151,136,164,136,172,136,140,136,147,136,142,136,130,137,214,137,217,137,213,137,48,138,39,138,44,138,30,138,57,140,59,140,92,140,93,140,125,140,165,140,125,141,123,141,121,141,188,141,194,141,185,141,191,141,193,141,216,142,222,142,221,142,220,142,215,142,224,142,225,142,36,144,11,144,17,144,28,144,12,144,33,144,239,144,234,144,240,144,244,144,242,144,243,144,212,144,235,144,236,144,233,144,86,145,88,145,90,145,83,145,85,145,236,145,244,145,241,145,243,145,248,145,228,145,249,145,234,145,235,145,247,145,232,145,238,145,122,149,134,149,136,149,124,150,109,150,107,150,113,150,111,150,191,150,106,151,4,152,229,152,151,153,155,80,149,80,148,80,158,80,139,80,163,80,131,80,140,80,142,80,157,80,104,80,156,80,146,80,130,80,135,80,95,81,212,81,18,83,17,83,164,83,167,83,145,85,168,85,165,85,173,85,119,85,69,86,162,85,147,85,136,85,143,85,181,85,129,85,163,85,146,85,164,85,125,85,140,85,166,85,127,85,149,85,161,85,142,85,12,87,41,88,55,88,25,88,30,88,39,88,35,88,40,88,245,87,72,88,37,88,28,88,27,88,51,88,63,88,54,88,46,88,57,88,56,88,45,88,44,88,59,88,97,89,175,90,148,90,159,90,122,90,162,90,158,90,120,90,166,90,124,90,165,90,172,90,149,90,174,90,55,90,132,90,138,90,151,90,131,90,139,90,169,90,123,90,125,90,140,90,156,90,143,90,147,90,157,90,234,91,205,91,203,91,212,91,209,91,202,91,206,91,12,92,48,92,55,93,67,93,107,93,65,93,75,93,63,93,53,93,81,93,78,93,85,93,51,93,58,93,82,93,61,93,49,93,89,93,66,93,57,93,73,93,56,93,60,93,50,93,54,93,64,93,69,93,68,94,65,94,88,95,166,95,165,95,171,95,201,96,185,96,204,96,226,96,206,96,196,96,20,97,242,96,10,97,22,97,5,97,245,96,19,97,248,96,252,96,254,96,193,96,3,97,24,97,29,97,16,97,255,96,4,97,11,97,74,98,148,99,177,99,176,99,206,99,229,99,232,99,239,99,195,99,157,100,243,99,202,99,224,99,246,99,213,99,242,99,245,99,97,100,223,99,190,99,221,99,220,99,196,99,216,99,211,99,194,99,199,99,204,99,203,99,200,99,240,99,215,99,217,99,50,101,103,101,106,101,100,101,92,101,104,101,101,101,140,101,157,101,158,101,174,101,208,101,210,101,124,102,108,102,123,102,128,102,113,102,121,102,106,102,114,102,1,103,12,105,211,104,4,105,220,104,42,105,236,104,234,104,241,104,15,105,214,104,247,104,235,104,228,104,246,104,19,105,16,105,243,104,225,104,7,105,204,104,8,105,112,105,180,104,17,105,239,104,198,104,20,105,248,104,208,104,253,104,252,104,232,104,11,105,10,105,23,105,206,104,200,104,221,104,222,104,230,104,244,104,209,104,6,105,212,104,233,104,21,105,37,105,199,104,57,107,59,107,63,107,60,107,148,107,151,107,153,107,149,107,189,107,240,107,242,107,243,107,48,108,252,109,70,110,71,110,31,110,73,110,136,110,60,110,61,110,69,110,98,110,43,110,63,110,65,110,93,110,115,110,28,110,51,110,75,110,64,110,81,110,59,110,3,110,46,110,94,110,104,110,92,110,97,110,49,110,40,110,96,110,113,110,107,110,57,110,34,110,48,110,83,110,101,110,39,110,120,110,100,110,119,110,85,110,121,110,82,110,102,110,53,110,54,110,90,110,32,113,30,113,47,113,251,112,46,113,49,113,35,113,37,113,34,113,50,113,31,113,40,113,58,113,27,113,75,114,90,114,136,114,137,114,134,114,133,114,139,114,18,115,11,115,48,115,34,115,49,115,51,115,39,115,50,115,45,115,38,115,35,115,53,115,12,115,46,116,44,116,48,116,43,116,22,116,26,116,33,116,45,116,49,116,36,116,35,116,29,116,41,116,32,116,50,116,251,116,47,117,111,117,108,117,231,117,218,117,225,117,230,117,221,117,223,117,228,117,215,117,149,118,146,118,218,118,70,119,71,119,68,119,77,119,69,119,74,119,78,119,75,119,76,119,222,119,236,119,96,120,100,120,101,120,92,120,109,120,113,120,106,120,110,120,112,120,105,120,104,120,94,120,98,120,116,121,115,121,114,121,112,121,2,122,10,122,3,122,12,122,4,122,153,122,230,122,228,122,74,123,59,123,68,123,72,123,76,123,78,123,64,123,88,123,69,123,162,124,158,124,168,124,161,124,88,125,111,125,99,125,83,125,86,125,103,125,106,125,79,125,109,125,92,125,107,125,82,125,84,125,105,125,81,125,95,125,78,125,62,127,63,127,101,127,102,127,162,127,160,127,161,127,215,127,81,128,79,128,80,128,254,128,212,128,67,129,74,129,82,129,79,129,71,129,61,129,77,129,58,129,230,129,238,129,247,129,248,129,249,129,4,130,60,130,61,130,63,130,117,130,59,131,207,131,249,131,35,132,192,131,232,131,18,132,231,131,228,131,252,131,246,131,16,132,198,131,200,131,235,131,227,131,191,131,1,132,221,131,229,131,216,131,255,131,225,131,203,131,206,131,214,131,245,131,201,131,9,132,15,132,222,131,17,132,6,132,194,131,243,131,213,131,250,131,199,131,209,131,234,131,19,132,195,131,236,131,238,131,196,131,251,131,215,131,226,131,27,132,219,131,254,131,216,134,226,134,230,134,211,134,227,134,218,134,234,134,221,134,235,134,220,134,236,134,233,134,215,134,232,134,209,134,72,136,86,136,85,136,186,136,215,136,185,136,184,136,192,136,190,136,182,136,188,136,183,136,189,136,178,136,1,137,201,136,149,137,152,137,151,137,221,137,218,137,219,137,78,138,77,138,57,138,89,138,64,138,87,138,88,138,68,138,69,138,82,138,72,138,81,138,74,138,76,138,79,138,95,140,129,140,128,140,186,140,190,140,176,140,185,140,181,140,132,141,128,141,137,141,216,141,211,141,205,141,199,141,214,141,220,141,207,141,213,141,217,141,200,141,215,141,197,141,239,142,247,142,250,142,249,142,230,142,238,142,229,142,245,142,231,142,232,142,246,142,235,142,241,142,236,142,244,142,233,142,45,144,52,144,47,144,6,145,44,145,4,145,255,144,252,144,8,145,249,144,251,144,1,145,0,145,7,145,5,145,3,145,97,145,100,145,95,145,98,145,96,145,1,146,10,146,37,146,3,146,26,146,38,146,15,146,12,146,0,146,18,146,255,145,253,145,6,146,4,146,39,146,2,146,28,146,36,146,25,146,23,146,5,146,22,146,123,149,141,149,140,149,144,149,135,150,126,150,136,150,137,150,131,150,128,150,194,150,200,150,195,150,241,150,240,150,108,151,112,151,110,151,7,152,169,152,235,152,230,156,249,158,131,78,132,78,182,78,189,80,191,80,198,80,174,80,196,80,202,80,180,80,200,80,194,80,176,80,193,80,186,80,177,80,203,80,201,80,182,80,184,80,215,81,122,82,120,82,123,82,124,82,195,85,219,85,204,85,208,85,203,85,202,85,221,85,192,85,212,85,196,85,233,85,191,85,210,85,141,85,207,85,213,85,226,85,214,85,200,85,242,85,205,85,217,85,194,85,20,87,83,88,104,88,100,88,79,88,77,88,73,88,111,88,85,88,78,88,93,88,89,88,101,88,91,88,61,88,99,88,113,88,252,88,199,90,196,90,203,90,186,90,184,90,177,90,181,90,176,90,191,90,200,90,187,90,198,90,183,90,192,90,202,90,180,90,182,90,205,90,185,90,144,90,214,91,216,91,217,91,31,92,51,92,113,93,99,93,74,93,101,93,114,93,108,93,94,93,104,93,103,93,98,93,240,93,79,94,78,94,74,94,77,94,75,94,197,94,204,94,198,94,203,94,199,94,64,95,175,95,173,95,247,96,73,97,74,97,43,97,69,97,54,97,50,97,46,97,70,97,47,97,79,97,41,97,64,97,32,98,104,145,35,98,37,98,36,98,197,99,241,99,235,99,16,100,18,100,9,100,32,100,36,100,51,100,67,100,31,100,21,100,24,100,57,100,55,100,34,100,35,100,12,100,38,100,48,100,40,100,65,100,53,100,47,100,10,100,26,100,64,100,37,100,39,100,11,100,231,99,27,100,46,100,33,100,14,100,111,101,146,101,211,101,134,102,140,102,149,102,144,102,139,102,138,102,153,102,148,102,120,102,32,103,102,105,95,105,56,105,78,105,98,105,113,105,63,105,69,105,106,105,57,105,66,105,87,105,89,105,122,105,72,105,73,105,53,105,108,105,51,105,61,105,101,105,240,104,120,105,52,105,105,105,64,105,111,105,68,105,118,105,88,105,65,105,116,105,76,105,59,105,75,105,55,105,92,105,79,105,81,105,50,105,82,105,47,105,123,105,60,105,70,107,69,107,67,107,66,107,72,107,65,107,155,107,13,250,251,107,252,107,249,107,247,107,248,107,155,110,214,110,200,110,143,110,192,110,159,110,147,110,148,110,160,110,177,110,185,110,198,110,210,110,189,110,193,110,158,110,201,110,183,110,176,110,205,110,166,110,207,110,178,110,190,110,195,110,220,110,216,110,153,110,146,110,142,110,141,110,164,110,161,110,191,110,179,110,208,110,202,110,151,110,174,110,163,110,71,113,84,113,82,113,99,113,96,113,65,113,93,113,98,113,114,113,120,113,106,113,97,113,66,113,88,113,67,113,75,113,112,113,95,113,80,113,83,113,68,113,77,113,90,113,79,114,141,114,140,114,145,114,144,114,142,114,60,115,66,115,59,115,58,115,64,115,74,115,73,115,68,116,74,116,75,116,82,116,81,116,87,116,64,116,79,116])
.concat([80,116,78,116,66,116,70,116,77,116,84,116,225,116,255,116,254,116,253,116,29,117,121,117,119,117,131,105,239,117,15,118,3,118,247,117,254,117,252,117,249,117,248,117,16,118,251,117,246,117,237,117,245,117,253,117,153,118,181,118,221,118,85,119,95,119,96,119,82,119,86,119,90,119,105,119,103,119,84,119,89,119,109,119,224,119,135,120,154,120,148,120,143,120,132,120,149,120,133,120,134,120,161,120,131,120,121,120,153,120,128,120,150,120,123,120,124,121,130,121,125,121,121,121,17,122,24,122,25,122,18,122,23,122,21,122,34,122,19,122,27,122,16,122,163,122,162,122,158,122,235,122,102,123,100,123,109,123,116,123,105,123,114,123,101,123,115,123,113,123,112,123,97,123,120,123,118,123,99,123,178,124,180,124,175,124,136,125,134,125,128,125,141,125,127,125,133,125,122,125,142,125,123,125,131,125,124,125,140,125,148,125,132,125,125,125,146,125,109,127,107,127,103,127,104,127,108,127,166,127,165,127,167,127,219,127,220,127,33,128,100,129,96,129,119,129,92,129,105,129,91,129,98,129,114,129,33,103,94,129,118,129,103,129,111,129,68,129,97,129,29,130,73,130,68,130,64,130,66,130,69,130,241,132,63,132,86,132,118,132,121,132,143,132,141,132,101,132,81,132,64,132,134,132,103,132,48,132,77,132,125,132,90,132,89,132,116,132,115,132,93,132,7,133,94,132,55,132,58,132,52,132,122,132,67,132,120,132,50,132,69,132,41,132,217,131,75,132,47,132,66,132,45,132,95,132,112,132,57,132,78,132,76,132,82,132,111,132,197,132,142,132,59,132,71,132,54,132,51,132,104,132,126,132,68,132,43,132,96,132,84,132,110,132,80,132,11,135,4,135,247,134,12,135,250,134,214,134,245,134,77,135,248,134,14,135,9,135,1,135,246,134,13,135,5,135,214,136,203,136,205,136,206,136,222,136,219,136,218,136,204,136,208,136,133,137,155,137,223,137,229,137,228,137,225,137,224,137,226,137,220,137,230,137,118,138,134,138,127,138,97,138,63,138,119,138,130,138,132,138,117,138,131,138,129,138,116,138,122,138,60,140,75,140,74,140,101,140,100,140,102,140,134,140,132,140,133,140,204,140,104,141,105,141,145,141,140,141,142,141,143,141,141,141,147,141,148,141,144,141,146,141,240,141,224,141,236,141,241,141,238,141,208,141,233,141,227,141,226,141,231,141,242,141,235,141,244,141,6,143,255,142,1,143,0,143,5,143,7,143,8,143,2,143,11,143,82,144,63,144,68,144,73,144,61,144,16,145,13,145,15,145,17,145,22,145,20,145,11,145,14,145,110,145,111,145,72,146,82,146,48,146,58,146,102,146,51,146,101,146,94,146,131,146,46,146,74,146,70,146,109,146,108,146,79,146,96,146,103,146,111,146,54,146,97,146,112,146,49,146,84,146,99,146,80,146,114,146,78,146,83,146,76,146,86,146,50,146,159,149,156,149,158,149,155,149,146,150,147,150,145,150,151,150,206,150,250,150,253,150,248,150,245,150,115,151,119,151,120,151,114,151,15,152,13,152,14,152,172,152,246,152,249,152,175,153,178,153,176,153,181,153,173,154,171,154,91,155,234,156,237,156,231,156,128,158,253,158,230,80,212,80,215,80,232,80,243,80,219,80,234,80,221,80,228,80,211,80,236,80,240,80,239,80,227,80,224,80,216,81,128,82,129,82,233,82,235,82,48,83,172,83,39,86,21,86,12,86,18,86,252,85,15,86,28,86,1,86,19,86,2,86,250,85,29,86,4,86,255,85,249,85,137,88,124,88,144,88,152,88,134,88,129,88,127,88,116,88,139,88,122,88,135,88,145,88,142,88,118,88,130,88,136,88,123,88,148,88,143,88,254,88,107,89,220,90,238,90,229,90,213,90,234,90,218,90,237,90,235,90,243,90,226,90,224,90,219,90,236,90,222,90,221,90,217,90,232,90,223,90,119,91,224,91,227,91,99,92,130,93,128,93,125,93,134,93,122,93,129,93,119,93,138,93,137,93,136,93,126,93,124,93,141,93,121,93,127,93,88,94,89,94,83,94,216,94,209,94,215,94,206,94,220,94,213,94,217,94,210,94,212,94,68,95,67,95,111,95,182,95,44,97,40,97,65,97,94,97,113,97,115,97,82,97,83,97,114,97,108,97,128,97,116,97,84,97,122,97,91,97,101,97,59,97,106,97,97,97,86,97,41,98,39,98,43,98,43,100,77,100,91,100,93,100,116,100,118,100,114,100,115,100,125,100,117,100,102,100,166,100,78,100,130,100,94,100,92,100,75,100,83,100,96,100,80,100,127,100,63,100,108,100,107,100,89,100,101,100,119,100,115,101,160,101,161,102,160,102,159,102,5,103,4,103,34,103,177,105,182,105,201,105,160,105,206,105,150,105,176,105,172,105,188,105,145,105,153,105,142,105,167,105,141,105,169,105,190,105,175,105,191,105,196,105,189,105,164,105,212,105,185,105,202,105,154,105,207,105,179,105,147,105,170,105,161,105,158,105,217,105,151,105,144,105,194,105,181,105,165,105,198,105,74,107,77,107,75,107,158,107,159,107,160,107,195,107,196,107,254,107,206,110,245,110,241,110,3,111,37,111,248,110,55,111,251,110,46,111,9,111,78,111,25,111,26,111,39,111,24,111,59,111,18,111,237,110,10,111,54,111,115,111,249,110,238,110,45,111,64,111,48,111,60,111,53,111,235,110,7,111,14,111,67,111,5,111,253,110,246,110,57,111,28,111,252,110,58,111,31,111,13,111,30,111,8,111,33,111,135,113,144,113,137,113,128,113,133,113,130,113,143,113,123,113,134,113,129,113,151,113,68,114,83,114,151,114,149,114,147,114,67,115,77,115,81,115,76,115,98,116,115,116,113,116,117,116,114,116,103,116,110,116,0,117,2,117,3,117,125,117,144,117,22,118,8,118,12,118,21,118,17,118,10,118,20,118,184,118,129,119,124,119,133,119,130,119,110,119,128,119,111,119,126,119,131,119,178,120,170,120,180,120,173,120,168,120,126,120,171,120,158,120,165,120,160,120,172,120,162,120,164,120,152,121,138,121,139,121,150,121,149,121,148,121,147,121,151,121,136,121,146,121,144,121,43,122,74,122,48,122,47,122,40,122,38,122,168,122,171,122,172,122,238,122,136,123,156,123,138,123,145,123,144,123,150,123,141,123,140,123,155,123,142,123,133,123,152,123,132,82,153,123,164,123,130,123,187,124,191,124,188,124,186,124,167,125,183,125,194,125,163,125,170,125,193,125,192,125,197,125,157,125,206,125,196,125,198,125,203,125,204,125,175,125,185,125,150,125,188,125,159,125,166,125,174,125,169,125,161,125,201,125,115,127,226,127,227,127,229,127,222,127,36,128,93,128,92,128,137,129,134,129,131,129,135,129,141,129,140,129,139,129,21,130,151,132,164,132,161,132,159,132,186,132,206,132,194,132,172,132,174,132,171,132,185,132,180,132,193,132,205,132,170,132,154,132,177,132,208,132,157,132,167,132,187,132,162,132,148,132,199,132,204,132,155,132,169,132,175,132,168,132,214,132,152,132,182,132,207,132,160,132,215,132,212,132,210,132,219,132,176,132,145,132,97,134,51,135,35,135,40,135,107,135,64,135,46,135,30,135,33,135,25,135,27,135,67,135,44,135,65,135,62,135,70,135,32,135,50,135,42,135,45,135,60,135,18,135,58,135,49,135,53,135,66,135,38,135,39,135,56,135,36,135,26,135,48,135,17,135,247,136,231,136,241,136,242,136,250,136,254,136,238,136,252,136,246,136,251,136,240,136,236,136,235,136,157,137,161,137,159,137,158,137,233,137,235,137,232,137,171,138,153,138,139,138,146,138,143,138,150,138,61,140,104,140,105,140,213,140,207,140,215,140,150,141,9,142,2,142,255,141,13,142,253,141,10,142,3,142,7,142,6,142,5,142,254,141,0,142,4,142,16,143,17,143,14,143,13,143,35,145,28,145,32,145,34,145,31,145,29,145,26,145,36,145,33,145,27,145,122,145,114,145,121,145,115,145,165,146,164,146,118,146,155,146,122,146,160,146,148,146,170,146,141,146,166,146,154,146,171,146,121,146,151,146,127,146,163,146,238,146,142,146,130,146,149,146,162,146,125,146,136,146,161,146,138,146,134,146,140,146,153,146,167,146,126,146,135,146,169,146,157,146,139,146,45,146,158,150,161,150,255,150,88,151,125,151,122,151,126,151,131,151,128,151,130,151,123,151,132,151,129,151,127,151,206,151,205,151,22,152,173,152,174,152,2,153,0,153,7,153,157,153,156,153,195,153,185,153,187,153,186,153,194,153,189,153,199,153,177,154,227,154,231,154,62,155,63,155,96,155,97,155,95,155,241,156,242,156,245,156,167,158,255,80,3,81,48,81,248,80,6,81,7,81,246,80,254,80,11,81,12,81,253,80,10,81,139,82,140,82,241,82,239,82,72,86,66,86,76,86,53,86,65,86,74,86,73,86,70,86,88,86,90,86,64,86,51,86,61,86,44,86,62,86,56,86,42,86,58,86,26,87,171,88,157,88,177,88,160,88,163,88,175,88,172,88,165,88,161,88,255,88,255,90,244,90,253,90,247,90,246,90,3,91,248,90,2,91,249,90,1,91,7,91,5,91,15,91,103,92,153,93,151,93,159,93,146,93,162,93,147,93,149,93,160,93,156,93,161,93,154,93,158,93,105,94,93,94,96,94,92,94,243,125,219,94,222,94,225,94,73,95,178,95,139,97,131,97,121,97,177,97,176,97,162,97,137,97,155,97,147,97,175,97,173,97,159,97,146,97,170,97,161,97,141,97,102,97,179,97,45,98,110,100,112,100,150,100,160,100,133,100,151,100,156,100,143,100,139,100,138,100,140,100,163,100,159,100,104,100,177,100,152,100,118,101,122,101,121,101,123,101,178,101,179,101,181,102,176,102,169,102,178,102,183,102,170,102,175,102,0,106,6,106,23,106,229,105,248,105,21,106,241,105,228,105,32,106,255,105,236,105,226,105,27,106,29,106,254,105,39,106,242,105,238,105,20,106,247,105,231,105,64,106,8,106,230,105,251,105,13,106,252,105,235,105,9,106,4,106,24,106,37,106,15,106,246,105,38,106,7,106,244,105,22,106,81,107,165,107,163,107,162,107,166,107,1,108,0,108,255,107,2,108,65,111,38,111,126,111,135,111,198,111,146,111,141,111,137,111,140,111,98,111,79,111,133,111,90,111,150,111,118,111,108,111,130,111,85,111,114,111,82,111,80,111,87,111,148,111,147,111,93,111,0,111,97,111,107,111,125,111,103,111,144,111,83,111,139,111,105,111,127,111,149,111,99,111,119,111,106,111,123,111,178,113,175,113,155,113,176,113,160,113,154,113,169,113,181,113,157,113,165,113,158,113,164,113,161,113,170,113,156,113,167,113,179,113,152,114,154,114,88,115,82,115,94,115,95,115,96,115,93,115,91,115,97,115,90,115,89,115,98,115,135,116,137,116,138,116,134,116,129,116,125,116,133,116,136,116,124,116,121,116,8,117,7,117,126,117,37,118,30,118,25,118,29,118,28,118,35,118,26,118,40,118,27,118,156,118,157,118,158,118,155,118,141,119,143,119,137,119,136,119,205,120,187,120,207,120,204,120,209,120,206,120,212,120,200,120,195,120,196,120,201,120,154,121,161,121,160,121,156,121,162,121,155,121,118,107,57,122,178,122,180,122,179,122,183,123,203,123,190,123,172,123,206,123,175,123,185,123,202,123,181,123,197,124,200,124,204,124,203,124,247,125,219,125,234,125,231,125,215,125,225,125,3,126,250,125,230,125,246,125,241,125,240,125,238,125,223,125,118,127,172,127,176,127,173,127,237,127,235,127,234,127,236,127,230,127,232,127,100,128,103,128,163,129,159,129,158,129,149,129,162,129,153,129,151,129,22,130,79,130,83,130,82,130,80,130,78,130,81,130,36,133,59,133,15,133,0,133,41,133,14,133,9,133,13,133,31,133,10,133,39,133,28,133,251,132,43,133,250,132,8,133,12,133,244,132,42,133,242,132,21,133,247,132,235,132,243,132,252,132,18,133,234,132,233,132,22,133,254,132,40,133,29,133,46,133,2,133,253,132,30,133,246,132,49,133,38,133,231,132,232,132,240,132,239,132,249,132,24,133,32,133,48,133,11,133,25,133,47,133,98,134,86,135,99,135,100,135,119,135,225,135,115,135,88,135,84,135,91,135,82,135,97,135,90,135,81,135,94,135,109,135,106,135,80,135,78,135,95,135,93,135,111,135,108,135,122,135,110,135,92,135,101,135,79,135,123,135,117,135,98,135,103,135,105,135,90,136,5,137,12,137,20,137,11,137,23,137,24,137,25,137,6,137,22,137,17,137,14,137,9,137,162,137,164,137,163,137,237,137,240,137,236,137,207,138,198,138,184,138,211,138,209,138,212,138,213,138,187,138,215,138,190,138,192,138,197,138,216,138,195,138,186,138,189,138,217,138,62,140,77,140,143,140,229,140,223,140,217,140,232,140,218,140,221,140,231,140,160,141,156,141,161,141,155,141,32,142,35,142,37,142,36,142,46,142,21,142,27,142,22,142,17,142,25,142,38,142,39,142,20,142,18,142,24,142,19,142,28,142,23,142,26,142,44,143,36,143,24,143,26,143,32,143,35,143,22,143,23,143,115,144,112,144,111,144,103,144,107,144,47,145,43,145,41,145,42,145,50,145,38,145,46,145,133,145,134,145,138,145,129,145,130,145,132,145,128,145,208,146,195,146,196,146,192,146,217,146,182,146,207,146,241,146,223,146,216,146,233,146,215,146,221,146,204,146,239,146,194,146,232,146,202,146,200,146,206,146,230,146,205,146,213,146,201,146,224,146,222,146,231,146,209,146,211,146,181,146,225,146,198,146,180,146,124,149,172,149,171,149,174,149,176,149,164,150,162,150,211,150,5,151,8,151,2,151,90,151,138,151,142,151,136,151,208,151,207,151,30,152,29,152,38,152,41,152,40,152,32,152,27,152,39,152,178,152,8,153,250,152,17,153,20,153,22,153,23,153,21,153,220,153,205,153,207,153,211,153,212,153,206,153,201,153,214,153,216,153,203,153,215,153,204,153,179,154,236,154,235,154,243,154,242,154,241,154,70,155,67,155,103,155,116,155,113,155,102,155,118,155,117,155,112,155,104,155,100,155,108,155,252,156,250,156,253,156,255,156,247,156,7,157,0,157,249,156,251,156,8,157,5,157,4,157,131,158,211,158,15,159,16,159,28,81,19,81,23,81,26,81,17,81,222,81,52,83,225,83,112,86,96,86,110,86,115,86,102,86,99,86,109,86,114,86,94,86,119,86,28,87,27,87,200,88,189,88,201,88,191,88,186,88,194,88,188,88,198,88,23,91,25,91,27,91,33,91,20,91,19,91,16,91,22,91,40,91,26,91,32,91,30,91,239,91,172,93,177,93,169,93,167,93,181,93,176,93,174,93,170,93,168,93,178,93,173,93,175,93,180,93,103,94,104,94,102,94,111,94,233,94,231,94,230,94,232,94,229,94,75,95,188,95,157,97,168,97,150,97,197,97,180,97,198,97,193,97,204,97,186,97,191,97,184,97,140,97,215,100,214,100,208,100,207,100,201,100,189,100,137,100,195,100,219,100,243,100,217,100,51,101,127,101,124,101,162,101,200,102,190,102,192,102,202,102,203,102,207,102,189,102,187,102,186,102,204,102,35,103,52,106,102,106,73,106,103,106,50,106,104,106,62,106,93,106,109,106,118,106,91,106,81,106,40,106,90,106,59,106,63,106,65,106,106,106,100,106,80,106,79,106,84,106,111,106,105,106,96,106,60,106,94,106,86,106,85,106,77,106,78,106,70,106,85,107,84,107,86,107,167,107,170,107,171,107,200,107,199,107,4,108,3,108,6,108,173,111,203,111,163,111,199,111,188,111,206,111,200,111,94,111,196,111,189,111,158,111,202,111,168,111,4,112,165,111,174,111,186,111,172,111,170,111,207,111,191,111,184,111,162,111,201,111,171,111,205,111,175,111,178,111,176,111,197,113,194,113,191,113,184,113,214,113,192,113,193,113,203,113,212,113,202,113,199,113,207,113,189,113,216,113,188,113,198,113,218,113,219,113,157,114,158,114,105,115,102,115,103,115,108,115,101,115,107,115,106,115,127,116,154,116,160,116,148,116,146,116,149,116,161,116,11,117,128,117,47,118,45,118,49,118,61,118,51,118,60,118,53,118,50,118,48,118,187,118,230,118,154,119,157,119,161,119,156,119,155,119,162,119,163,119,149,119,153,119,151,119,221,120,233,120,229,120,234,120,222,120,227,120,219,120,225,120,226,120,237,120,223,120,224,120,164,121,68,122,72,122,71,122,182,122,184,122,181,122,177,122,183,122,222,123,227,123,231,123,221,123,213,123,229,123,218,123,232,123,249,123,212,123,234,123,226,123,220,123,235,123,216,123,223,123,210,124,212,124,215,124,208,124,209,124,18,126,33,126,23,126,12,126,31,126,32,126,19,126,14,126,28,126,21,126,26,126,34,126,11,126,15,126,22,126,13,126,20,126,37,126,36,126,67,127,123,127,124,127,122,127,177,127,239,127,42,128,41,128,108,128,177,129,166,129,174,129,185,129,181,129,171,129,176,129,172,129,180,129,178,129,183,129,167,129,242,129,85,130,86,130,87,130,86,133,69,133,107,133,77,133,83,133,97,133,88,133,64,133,70,133,100,133,65,133,98,133,68,133,81,133,71,133,99,133,62,133,91,133,113,133,78,133,110,133,117,133,85,133,103,133,96,133,140,133,102,133,93,133,84,133,101,133,108,133,99,134,101,134,100,134,155,135,143,135,151,135,147,135,146,135,136,135,129,135,150,135,152,135,121,135,135,135,163,135,133,135,144,135,145,135,157,135,132,135,148,135,156,135,154,135,137,135,30,137,38,137,48,137,45,137,46,137,39,137,49,137,34,137,41,137,35,137,47,137,44,137,31,137,241,137,224,138,226,138,242,138,244,138,245,138,221,138,20,139,228,138,223,138,240,138,200,138,222,138,225,138,232,138,255,138,239,138,251,138,145,140,146,140,144,140,245,140,238,140,241,140,240,140,243,140,108,141,110,141,165,141,167,141,51,142,62,142,56,142,64,142,69,142,54,142,60,142,61,142,65,142,48,142,63,142,189,142,54,143,46,143,53,143,50,143,57,143,55,143,52,143,118,144,121,144,123,144,134,144,250,144,51,145,53,145,54,145,147,145,144,145,145,145,141,145,143,145,39,147,30,147,8,147,31,147,6,147,15,147,122,147,56,147,60,147,27,147,35,147,18,147,1,147,70,147,45,147,14,147,13,147,203,146,29,147,250,146,37,147,19,147,249,146,247,146,52,147,2,147,36,147,255,146,41,147,57,147,53,147,42,147,20,147,12,147,11,147,254,146,9,147,0,147,251,146,22,147,188,149,205,149,190,149,185,149,186,149,182,149,191,149,181,149,189,149,169,150,212,150,11,151,18,151,16,151,153,151,151,151,148,151,240,151,248,151,53,152,47,152,50,152,36,153,31,153,39,153,41,153,158,153,238,153,236,153,229,153,228,153,240,153,227,153,234,153,233,153,231,153,185,154,191,154,180,154,187,154,246,154,250,154,249,154,247,154,51,155,128,155,133,155,135,155,124,155,126,155,123,155,130,155,147,155,146,155,144,155,122,155,149,155,125,155,136,155,37,157,23,157,32,157,30,157,20,157,41,157,29,157,24,157,34,157,16,157,25,157,31,157,136,158,134,158,135,158,174,158,173,158,213,158,214,158,250,158,18,159,61,159,38,81,37,81,34,81,36,81,32,81,41,81,244,82,147,86,140,86,141,86,134,86,132,86,131,86,126,86,130,86,127,86,129,86,214,88,212,88,207,88,210,88,45,91,37,91,50,91,35,91,44,91,39,91,38,91,47,91,46,91,123,91,241,91,242,91,183,93,108,94,106,94,190,95,187,95,195,97,181,97,188,97,231,97,224,97,229,97,228,97,232,97,222,97,239,100,233,100,227,100,235,100,228,100,232,100,129,101,128,101,182,101,218,101,210,102,141,106,150,106,129,106,165,106,137,106,159,106,155,106,161,106,158,106,135,106,147,106,142,106,149,106,131,106,168,106,164,106,145,106,127,106,166,106,154,106,133,106,140,106,146,106,91,107,173,107,9,108,204,111,169,111,244,111,212,111,227,111,220,111,237,111,231,111,230,111,222,111,242,111,221,111,226,111,232,111,225,113,241,113,232,113,242,113,228,113,240,113,226,113,115,115,110,115,111,115,151,116,178,116,171,116,144,116,170,116,173,116,177,116,165,116,175,116,16,117,17,117,18,117,15,117,132,117,67,118,72,118,73,118,71,118,164,118,233,118,181,119,171,119,178,119,183,119,182,119,180,119,177,119,168,119,240,119,243,120,253,120,2,121,251,120,252,120,242,120,5,121,249,120,254,120,4,121,171,121,168,121,92,122,91,122,86,122,88,122,84,122,90,122,190,122,192,122,193,122,5,124,15,124,242,123,0,124,255,123,251,123,14,124,244,123,11,124,243,123,2,124,9,124,3,124,1,124,248,123,253,123,6,124,240,123,241,123,16,124,10,124,232,124,45,126,60,126,66,126,51,126,72,152,56,126,42,126,73,126,64,126,71,126,41,126,76,126,48,126,59,126,54,126,68,126,58,126,69,127,127,127,126,127,125,127,244,127,242,127,44,128,187,129,196,129,204,129,202,129,197,129,199,129,188,129,233,129,91,130,90,130,92,130,131,133,128,133,143,133,167,133,149,133,160,133,139,133,163,133,123,133,164,133,154,133,158,133,119,133,124,133,137,133,161,133,122,133,120,133,87,133,142,133,150,133,134,133,141,133,153,133,157,133,129,133,162,133,130,133,136,133,133,133,121,133,118,133,152,133,144,133,159,133,104,134,190,135,170,135,173,135,197,135,176,135,172,135,185,135,181,135,188,135,174,135,201,135,195,135,194,135,204,135,183,135,175,135,196,135,202,135,180,135,182,135,191,135,184,135,189,135,222,135,178,135,53,137,51,137,60,137,62,137,65,137,82,137,55,137,66,137,173,137,175,137,174,137,242,137,243,137,30,139,24,139,22,139,17,139,5,139,11,139,34,139,15,139,18,139,21,139,7,139,13,139,8,139,6,139,28,139,19,139,26,139,79,140,112,140,114,140,113,140,111,140,149,140,148,140,249,140,111,141,78,142,77,142,83,142,80,142,76,142,71,142,67,143,64,143,133,144,126,144,56,145,154,145,162,145,155,145,153,145,159,145,161,145,157,145,160,145,161,147,131,147,175,147,100,147,86,147,71,147,124,147,88,147,92,147,118,147,73,147,80,147,81,147,96,147,109,147,143,147,76,147,106,147,121,147,87,147,85,147,82,147,79,147,113,147,119,147,123,147,97,147,94,147,99,147,103,147,128,147,78,147,89,147,199,149,192,149,201,149,195,149,197,149,183,149,174,150,176,150,172,150,32,151,31,151,24,151,29,151,25,151,154,151,161,151,156,151,158,151,157,151,213,151,212,151,241,151,65,152,68,152,74,152,73,152,69,152,67,152,37,153,43,153,44,153,42,153,51,153,50,153,47,153,45,153,49,153,48,153,152,153,163,153,161,153,2,154,250,153,244,153,247,153,249,153,248,153,246,153,251,153,253,153,254,153,252,153,3,154,190,154,254,154,253,154,1,155,252,154,72,155,154,155,168,155,158,155,155,155,166,155,161,155,165,155,164,155,134,155,162,155,160,155,175,155,51,157,65,157,103,157,54,157,46,157,47,157,49,157,56,157,48,157,69,157,66,157,67,157,62,157,55,157,64,157,61,157,245,127,45,157,138,158,137,158,141,158,176,158,200,158,218,158,251,158,255,158,36,159,35,159,34,159,84,159,160,159,49,81,45,81,46,81,152,86,156,86,151,86,154,86,157,86,153,86,112,89,60,91,105,92,106,92,192,93,109,94,110,94,216,97,223,97,237,97,238,97,241,97,234,97,240,97,235,97,214,97,233,97,255,100,4,101,253,100,248,100,1,101,3,101,252,100,148,101,219,101,218,102,219,102,216,102,197,106,185,106,189,106,225,106,198,106,186,106,182,106,183,106,199,106,180,106,173,106,94,107,201,107,11,108,7,112,12,112,13,112,1,112,5,112,20,112,14,112,255,111,0,112,251,111,38,112,252,111,247,111,10,112,1,114,255,113,249,113,3,114,253,113,118,115,184,116,192,116,181,116,193,116,190,116,182,116,187,116,194,116,20,117,19,117,92,118,100,118,89,118,80,118,83,118,87,118,90,118,166,118,189,118,236,118,194,119,186,119,255,120,12,121,19,121,20,121,9,121,16,121,18,121,17,121,173,121,172,121,95,122,28,124,41,124,25,124,32,124,31,124,45,124,29,124,38,124,40,124,34,124,37,124,48,124,92,126,80,126,86,126,99,126,88,126,98,126,95,126,81,126,96,126,87,126,83,126,181,127,179,127,247,127,248,127,117,128,209,129,210,129,208,129,95,130,94,130,180,133,198,133,192,133,195,133,194,133,179,133,181,133,189,133,199,133,196,133,191,133,203,133,206,133,200,133,197,133,177,133,182,133,210,133,36,134,184,133,183,133,190,133,105,134,231,135,230,135,226,135,219,135,235,135,234,135,229,135,223,135,243,135,228,135,212,135,220,135,211,135,237,135,216,135,227,135,164,135,215,135,217,135,1,136,244,135,232,135,221,135,83,137,75,137,79,137,76,137,70,137,80,137,81,137,73,137,42,139,39,139,35,139,51,139,48,139,53,139,71,139,47,139,60,139,62,139,49,139,37,139,55,139,38,139,54,139,46,139,36,139,59,139,61,139,58,139,66,140,117,140,153,140,152,140,151,140,254,140,4,141,2,141,0,141,92,142,98,142,96,142,87,142,86,142,94,142,101,142,103,142,91,142,90,142,97,142,93,142,105,142,84,142,70,143,71,143,72,143,75,143,40,145,58,145,59,145,62,145,168,145,165,145,167,145,175,145,170,145,181,147,140,147,146,147,183,147,155,147,157,147,137,147,167,147,142,147,170,147,158,147,166,147,149,147,136,147,153,147,159,147,141,147,177,147,145,147,178,147,164,147,168,147,180,147,163,147,165,147,210,149,211,149,209,149,179,150,215,150,218,150,194,93,223,150,216,150,221,150,35,151,34,151,37,151,172,151,174,151,168,151,171,151,164,151,170,151,162,151,165,151,215,151,217,151,214,151,216,151,250,151,80,152,81,152,82,152,184,152,65,153,60,153,58,153,15,154,11,154,9,154,13,154,4,154,17,154,10,154,5,154,7,154,6,154,192,154,220,154,8,155,4,155,5,155,41,155,53,155,74,155,76,155,75,155,199,155,198,155,195,155,191,155,193,155,181,155,184,155,211,155,182,155,196,155,185,155,189,155,92,157,83,157,79,157,74,157,91,157,75,157,89,157,86,157,76,157,87,157,82,157,84,157,95,157,88,157,90,157,142,158,140,158,223,158,1,159,0,159,22,159,37,159,43,159,42,159,41,159,40,159,76,159,85,159,52,81,53,81,150,82,247,82,180,83,171,86,173,86,166,86,167,86,170,86,172,86,218,88,221,88,219,88,18,89,61,91,62,91,63,91,195,93,112,94,191,95,251,97,7,101,16,101,13,101,9,101,12,101,14,101,132,101,222,101,221,101,222,102,231,106,224,106,204,106,209,106,217,106,203,106,223,106,220,106,208,106,235,106,207,106,205,106,222,106,96,107,176,107,12,108,25,112,39,112,32,112,22,112,43,112,33,112,34,112,35,112,41,112,23,112,36,112,28,112,42,112,12,114,10,114,7,114,2,114,5,114,165,114,166,114,164,114,163,114,161,114,203,116,197,116,183,116,195,116,22,117,96,118,201,119,202,119,196,119,241,119,29,121,27,121,33,121,28,121,23,121,30,121,176,121,103,122,104,122,51,124,60,124,57,124,44,124,59,124,236,124,234,124,118,126,117,126,120,126,112,126,119,126,111,126,122,126,114,126,116,126,104,126,75,127,74,127,131,127,134,127,183,127,253,127,254,127,120,128,215,129,213,129,100,130,97,130,99,130,235,133,241,133,237,133,217,133,225,133,232,133,218,133,215,133,236,133,242,133,248,133,216,133,223,133,227,133,220,133,209,133,240,133,230,133,239,133,222,133,226,133,0,136,250,135,3,136,246,135,247,135,9,136,12,136,11,136,6,136,252,135,8,136,255,135,10,136,2,136,98,137,90,137,91,137,87,137,97,137,92,137,88,137,93,137,89,137,136,137,183,137,182,137,246,137,80,139,72,139,74,139,64,139,83,139,86,139,84,139,75,139,85,139,81,139,66,139,82,139,87,139,67,140,119,140,118,140,154,140,6,141,7,141,9,141,172,141,170,141,173,141,171,141,109,142,120,142,115,142,106,142,111,142,123,142,194,142,82,143,81,143,79,143,80,143,83,143,180,143,64,145,63,145,176,145,173,145,222,147,199,147,207,147,194,147,218,147,208,147,249,147,236,147,204,147,217,147,169,147,230,147,202,147,212,147,238,147,227,147,213,147,196,147,206,147,192,147,210,147,231,147,125,149,218,149,219,149,225,150,41,151,43,151,44,151,40,151,38,151,179,151,183,151,182,151,221,151,222,151,223,151,92,152,89,152,93,152,87,152,191,152,189,152,187,152,190,152,72,153,71,153,67,153,166,153,167,153,26,154,21,154,37,154,29,154,36,154,27,154,34,154,32,154,39,154,35,154,30,154,28,154,20,154,194,154,11,155,10,155,14,155,12,155,55,155,234,155,235,155,224,155,222,155,228,155,230,155,226,155,240,155,212,155,215,155,236,155,220,155,217,155,229,155,213,155,225,155,218,155,119,157,129,157,138,157,132,157,136,157,113,157,128,157,120,157,134,157,139,157,140,157,125,157,107,157,116,157,117,157,112,157,105,157,133,157,115,157,123,157,130,157,111,157,121,157,127,157,135,157,104,157,148,158,145,158,192,158,252,158,45,159,64,159,65,159,77,159,86,159,87,159,88,159,55,83,178,86,181,86,179,86,227,88,69,91,198,93,199,93,238,94,239,94,192,95,193,95,249,97,23,101,22,101,21,101,19,101,223,101,232,102,227,102,228,102,243,106,240,106,234,106,232,106,249,106,241,106,238,106,239,106,60,112,53,112,47,112,55,112,52,112,49,112,66,112,56,112,63,112,58,112,57,112,64,112,59,112,51,112,65,112,19,114,20,114,168,114,125,115,124,115,186,116,171,118,170,118,190,118,237,118,204,119,206,119,207,119,205,119,242,119,37,121,35,121,39,121,40,121,36,121,41,121,178,121,110,122,108,122,109,122,247,122,73,124,72,124,74,124,71,124,69,124,238,124,123,126,126,126,129,126,128,126,186,127,255,127,121,128,219,129,217,129,11,130,104,130,105,130,34,134,255,133,1,134,254,133,27,134,0,134,246,133,4,134,9,134,5,134,12,134,253,133,25,136,16,136,17,136,23,136,19,136,22,136,99,137,102,137,185,137,247,137,96,139,106,139,93,139,104,139,99,139,101,139,103,139,109,139,174,141,134,142,136,142,132,142,89,143,86,143,87,143,85,143,88,143,90,143,141,144,67,145,65,145,183,145,181,145,178,145,179,145,11,148,19,148,251,147,32,148,15,148,20,148,254,147,21,148,16,148,40,148,25,148,13,148,245,147,0,148,247,147,7,148,14,148,22,148,18,148,250,147,9,148,248,147,10,148,255,147,252,147,12,148,246,147,17,148,6,148,222,149,224,149,223,149,46,151,47,151,185,151,187,151,253,151,254,151,96,152,98,152,99,152,95,152,193,152,194,152,80,153,78,153,89,153,76,153,75,153,83,153,50,154,52,154,49,154,44,154,42,154,54,154,41,154,46,154,56,154,45,154,199,154,202,154,198,154,16,155,18,155,17,155,11,156,8,156,247,155,5,156,18,156,248,155,64,156,7,156,14,156,6,156,23,156,20,156,9,156,159,157,153,157,164,157,157,157,146,157,152,157,144,157,155,157,160,157,148,157,156,157,170,157,151,157,161,157,154,157,162,157,168,157,158,157,163,157,191,157,169,157,150,157,166,157,167,157,153,158,155,158,154,158,229,158,228,158,231,158,230,158,48,159,46,159,91,159,96,159,94,159,93,159,89,159,145,159,58,81,57,81,152,82,151,82,195,86,189,86,190,86,72,91,71,91,203,93,207,93,241,94,253,97,27,101,2,107,252,106,3,107,248,106,0,107,67,112,68,112,74,112,72,112,73,112,69,112,70,112,29,114,26,114,25,114,126,115,23,117,106,118,208,119,45,121,49,121,47,121,84,124,83,124,242,124,138,126,135,126,136,126,139,126,134,126,141,126,77,127,187,127,48,128,221,129,24,134,42,134,38,134,31,134,35,134,28,134,25,134,39,134,46,134,33,134,32,134,41,134,30,134,37,134,41,136,29,136,27,136,32,136,36,136,28,136,43,136,74,136,109,137,105,137,110,137,107,137,250,137,121,139,120,139,69,139,122,139,123,139,16,141,20,141,175,141,142,142,140,142,94,143,91,143,93,143,70,145,68,145,69,145,185,145,63,148,59,148,54,148,41,148,61,148,60,148,48,148,57,148,42,148,55,148,44,148,64,148,49,148,229,149,228,149,227,149,53,151,58,151,191,151,225,151,100,152,201,152,198,152,192,152,88,153,86,153,57,154,61,154,70,154,68,154,66,154,65,154,58,154,63,154,205,154,21,155,23,155,24,155,22,155,58,155,82,155,43,156,29,156,28,156,44,156,35,156,40,156,41,156,36,156,33,156,183,157,182,157,188,157,193,157,199,157,202,157,207,157,190,157,197,157,195,157,187,157,181,157,206,157,185,157,186,157,172,157,200,157,177,157,173,157,204,157,179,157,205,157,178,157,122,158,156,158,235,158,238,158,237,158,27,159,24,159,26,159,49,159,78,159,101,159,100,159,146,159,185,78,198,86,197,86,203,86,113,89,75,91,76,91,213,93,209,93,242,94,33,101,32,101,38,101,34,101,11,107,8,107,9,107,13,108,85,112,86,112,87,112,82,112,30,114,31,114,169,114,127,115,216,116,213,116,217,116,215,116,109,118,173,118,53,121,180,121,112,122,113,122,87,124,92,124,89,124,91,124,90,124,244,124,241,124,145,126,79,127,135,127,222,129,107,130,52,134,53,134,51,134,44,134,50,134,54,134,44,136,40,136,38,136,42,136,37,136,113,137,191,137,190,137,251,137,126,139,132,139,130,139,134,139,133,139,127,139,21,141,149,142,148,142,154,142,146,142,144,142,150,142,151,142,96,143,98,143,71,145,76,148,80,148,74,148,75,148,79,148,71,148,69,148,72,148,73,148,70,148,63,151,227,151,106,152,105,152,203,152,84,153,91,153,78,154,83,154,84,154,76,154,79,154,72,154,74,154,73,154,82,154,80,154,208,154,25,155,43,155,59,155,86,155,85,155,70,156,72,156,63,156,68,156,57,156,51,156,65,156,60,156,55,156,52,156,50,156,61,156,54,156,219,157,210,157,222,157,218,157,203,157,208,157,220,157,209,157,223,157,233,157,217,157,216,157,214,157,245,157,213,157,221,157,182,158,240,158,53,159,51,159,50,159,66,159,107,159,149,159,162,159,61,81,153,82,232,88,231,88,114,89,77,91,216,93,47,136,79,95,1,98,3,98,4,98,41,101,37,101,150,101,235,102,17,107,18,107,15,107,202,107,91,112,90,112,34,114,130,115,129,115,131,115,112,118,212,119,103,124,102,124,149,126,108,130,58,134,64,134,57,134,60,134,49,134,59,134,62,134,48,136,50,136,46,136,51,136,118,137,116,137,115,137,254,137,140,139,142,139,139,139,136,139,69,140,25,141,152,142,100,143,99,143,188,145,98,148,85,148,93,148,87,148,94,148,196,151,197,151,0,152,86,154,89,154,30,155,31,155,32,155,82,156,88,156,80,156,74,156,77,156,75,156,85,156,89,156,76,156,78,156,251,157,247,157,239,157,227,157,235,157,248,157,228,157,246,157,225,157,238,157,230,157,242,157,240,157,226,157,236,157,244,157,243,157,232,157,237,157,194,158,208,158,242,158,243,158,6,159,28,159,56,159,55,159,54,159,67,159,79,159,113,159,112,159,110,159,111,159,211,86,205,86,78,91,109,92,45,101,237,102,238,102,19,107,95,112,97,112,93,112,96,112,35,114,219,116,229,116,213,119,56,121,183,121,182,121,106,124,151,126,137,127,109,130,67,134,56,136,55,136,53,136,75,136,148,139,149,139,158,142,159,142,160,142,157,142,190,145,189,145,194,145,107,148,104,148,105,148,229,150,70,151,67,151,71,151,199,151,229,151,94,154,213,154,89,155,99,156,103,156,102,156,98,156,94,156,96,156,2,158,254,157,7,158,3,158,6,158,5,158,0,158,1,158,9,158,255,157,253,157,4,158,160,158,30,159,70,159,116,159,117,159,118,159,212,86,46,101,184,101,24,107,25,107,23,107,26,107,98,112,38,114,170,114,216,119,217,119,57,121,105,124,107,124,246,124,154,126,152,126,155,126,153,126,224,129,225,129,70,134,71,134,72,134,121,137,122,137,124,137,123,137,255,137,152,139,153,139,165,142,164,142,163,142,110,148,109,148,111,148,113,148,115,148,73,151,114,152,95,153,104,156,110,156,109,156,11,158,13,158,16,158,15,158,18,158,17,158,161,158,245,158,9,159,71,159,120,159,123,159,122,159,121,159,30,87,102,112,111,124,60,136,178,141,166,142,195,145,116,148,120,148,118,148,117,148,96,154,116,156,115,156,113,156,117,156,20,158,19,158,246,158,10,159,164,159,104,112,101,112,247,124,106,134,62,136,61,136,63,136,158,139,156,140,169,142,201,142,75,151,115,152,116,152,204,152,97,153,171,153,100,154,102,154,103,154,36,155,21,158,23,158,72,159,7,98,30,107,39,114,76,134,168,142,130,148,128,148,129,148,105,154,104,154,46,155,25,158,41,114,75,134,159,139,131,148,121,156,183,158,117,118,107,154,122,156,29,158,105,112,106,112,164,158,126,159,73,159,152,159,129,120,185,146,207,136,187,88,82,96,167,124,250,90,84,37,102,37,87,37,96,37,108,37,99,37,90,37,105,37,93,37,82,37,100,37,85,37,94,37,106,37,97,37,88,37,103,37,91,37,83,37,101,37,86,37,95,37,107,37,98,37,89,37,104,37,92,37,81,37,80,37,109,37,110,37,112,37,111,37,147,37,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,208,63,28,199,113,28,199,113,188,63,0,0,0,0,0,0,176,63,123,20,174,71,225,122,164,63,28,199,113,28,199,113,156,63,57,5,47,167,224,229,148,63,0,0,0,0,0,0,144,63,224,233,214,252,176,72,137,63,123,20,174,71,225,122,132,63,144,156,230,107,245,236,128,63,28,199,113,28,199,113,124,63,221,190,178,122,151,60,120,63,57,5,47,167,224,229,116,63,223,188,154,120,86,52,114,63,0,0,0,0,0,0,112,63,134,73,13,209,148,88,108,63,224,233,214,252,176,72,105,63,61,26,163,10,73,177,102,63,123,20,174,71,225,122,100,63,165,4,184,91,114,147,98,63,144,156,230,107,245,236,96,63,173,235,137,179,189,248,94,63,28,199,113,28,199,113,92,63,45,67,28,235,226,54,90,63,221,190,178,122,151,60,88,63,199,8,191,224,128,121,86,63,57,5,47,167,224,229,84,63,68,39,135,36,72,123,83,63,223,188,154,120,86,52,82,63,46,149,208,49,133,12,81,63,0,0,0,0,0,0,80,63,85,22,97,163,9,23,78,63,134,73,13,209,148,88,76,63,166,47,60,224,215,191,74,63,224,233,214,252,176,72,73,63,235,216,56,80,139,239,71,63,61,26,163,10,73,177,70,63,140,226,158,80,49,139,69,63,123,20,174,71,225,122,68,63,139,225,71,168,63,126,67,63,165,4,184,91,114,147,66,63,57,159,146,197,213,184,65,63,144,156,230,107,245,236,64,63,113,139,137,192,133,46,64,63,173,235,137,179,189,248,62,63,26,191,47,228,239,170,61,63,28,199,113,28,199,113,60,63,253,126,249,92,152,75,59,63,45,67,28,235,226,54,58,63,176,150,125,100,75,50,57,63,221,190,178,122,151,60,56,63,224,58,202,61,170,84,55,63,199,8,191,224,128,121,54,63,31,231,69,230,47,170,53,63,57,5,47,167,224,229,52,63,54,80,230,37,207,43,52,63,68,39,135,36,72,123,51,63,201,111,144,116,167,211,50,63,223,188,154,120,86,52,50,63,57,149,143,209,202,156,49,63,46,149,208,49,133,12,49,63,147,32,135,81,16,131,48,63,0,0,0,0,0,0,48,63,27,193,13,157,224,5,47,63,85,22,97,163,9,23,46,63,141,167,214,236,207,50,45,63,134,73,13,209,148,88,44,63,210,38,94,17,197,135,43,63,166,47,60,224,215,191,42,63,190,2,234,0,78,0,42,63,224,233,214,252,176,72,41,63,155,172,71,108,146,152,40,63,235,216,56,80,139,239,39,63,40,88,167,123,59,77,39,63,61,26,163,10,73,177,38,63,31,126,191,229,95,27,38,63,140,226,158,80,49,139,37,63,166,77,123,130,115,0,37,63,123,20,174,71,225,122,36,63,149,121,84,171,57,250,35,63,139,225,71,168,63,126,35,63,81,240,181,224,185,6,35,63,165,4,184,91,114,147,34,63,127,108,90,72,54,36,34,63,57,159,146,197,213,184,33,63,118,6,177,174,35,81,33,63,144,156,230,107,245,236,32,63,43,21,129,198,34,140,32,63,113,139,137,192,133,46,32,63,158,233,243,222,244,167,31,63,173,235,137,179,189,248,30,63,109,151,228,173,37,79,30,63,26,191,47,228,239,170,29,63,47,187,249,156,226,11,29,63,28,199,113,28,199,113,28,63,250,44,75,117,105,220,27,63,253,126,249,92,152,75,27,63,104,247,0,3,37,191,26,63,45,67,28,235,226,54,26,63,4,4,4,4,6,8,10,12,14,18,22,30,56,4,4,4,4,6,6,10,12,14,16,20,26,66,4,4,4,4,6,8,12,16,20,26,34,42,12,4,4,4,6,6,8,10,14,18,26,32,42,18,4,4,4,6,8,10,12,14,18,24,32,44,12,4,4,4,6,8,10,12,14,18,24,30,40,18,4,4,4,6,8,10,12,14,18,24,30,40,18,4,4,4,6,8,10,12,14,18,24,30,40,18,8,8,8,12,16,20,24,28,36,2,2,2,26,0,0,0,4,4,4,4,4,4,6,6,8,8,10,12,16,20,24,28,34,42,50,54,76,158,4,4,4,4,4,4,6,6,6,8,10,12,16,18,22,28,34,40,46,54,54,192,4,4,4,4,4,4,6,6,8,10,12,16,20,24,30,38,46,56,68,84,102,26,6,6,6,6,6,6,8,10,12,14,16,20,24,28,32,38,46,52,60,68,58,54,6,6,6,6,6,6,8,10,12,14,16,18,22,26,32,38,46,52,64,70,76,36,6,6,6,6,6,6,8,10,12,14,16,20,24,28,32,38,46,52,60,68,58,54,6,6,6,6,6,6,8,10,12,14,16,20,24,28,32,38,46,52,60,68,58,54,6,6,6,6,6,6,8,10,12,14,16,20,24,28,32,38,46,52,60,68,58,54,12,12,12,12,12,12,16,20,24,28,32,40,48,56,64,76,90,2,2,2,2,2,0,0,0,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,13,0,0,0,2,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,119,1,0,136,88,1,0,0,250,0,0,128,187,0,0,68,172,0,0,0,125,0,0,192,93,0,0,34,86,0,0,128,62,0,0,224,46,0,0,17,43,0,0,64,31,0,0,182,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,68,172,128,187,0,125,0,0,0,0,32,0,64,0,96,0,128,0,160,0,192,0,224,0,0,1,32,1,64,1,96,1,128,1,160,1,192,1,0,0,32,0,48,0,56,0,64,0,80,0,96,0,112,0,128,0,160,0,192,0,224,0,0,1,64,1,128,1,0,0,32,0,40,0,48,0,56,0,64,0,80,0,96,0,112,0,128,0,160,0,192,0,224,0,0,1,64,1,0,0,32,0,48,0,56,0,64,0,80,0,96,0,112,0,128,0,144,0,160,0,176,0,192,0,224,0,0,1,0,0,8,0,16,0,24,0,32,0,40,0,48,0,56,0,64,0,80,0,96,0,112,0,128,0,144,0,160,0,0,0,8,0,16,0,24,0,32,0,40,0,48,0,56,0,64,0,80,0,96,0,112,0,128,0,144,0,160,0,0,0,0,0,3,0,4,0,3,0,7,0,3,1,7,1,3,6,7,6,160,64,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,80,0,0,0,130,0,0,0,78,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,176,130,26,0,176,250,25,0,0,0,0,0,112,39,5,0,88,170,4,0,0,0,0,0,144,0,0,0,0,0,0,0])
.concat([16,0,0,0,58,0,0,0,96,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,120,251,25,0,112,183,24,0,152,154,3,0,0,0,0,0,16,155,3,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,24,0,0,0,72,0,0,0,0,0,0,0,0,0,0,0,76,0,0,0,40,4,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,70,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,119,26,0,112,183,24,0,88,40,5,0,0,0,0,0,136,167,4,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,24,0,0,0,72,0,0,0,0,0,0,0,0,0,0,0,76,0,0,0,64,80,26,0,0,127,26,0,112,161,0,0,120,159,0,0,232,161,0,0,0,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,212,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,212,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,200,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,132,0,0,0,0,0,0,0,0,0,0,0,84,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,85,26,0,64,128,26,0,184,41,5,0,64,41,5,0,16,46,1,0,0,0,0,0,118,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,184,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,104,248,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,104,248,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,73,26,0,80,126,26,0,224,42,5,0,104,42,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,136,27,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,136,27,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,200,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,200,74,26,0,144,126,26,0,8,44,5,0,144,43,5,0,0,147,3,0,0,0,0,0,76,0,0,0,0,0,0,0,0,0,0,0,90,0,0,0,12,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,94,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,94,25,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,200,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,104,101,26,0,104,129,26,0,48,45,5,0,184,44,5,0,0,0,0,0,0,0,0,0,92,0,0,0,0,0,0,0,64,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,79,25,0,16,0,0,0,160,203,1,0,100,39,52,0,0,0,0,0,0,0,0,0,22,0,0,0,14,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,160,64,26,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,80,0,0,0,130,0,0,0,78,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,159,25,0,96,94,25,0,0,0,0,0,24,46,5,0,248,87,5,0,0,0,0,0,134,0,0,0,0,0,0,0,16,0,0,0,58,0,0,0,96,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,160,159,25,0,248,166,24,0,64,155,3,0,0,0,0,0,184,155,3,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,150,0,0,0,72,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,40,4,26,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,70,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,104,131,26,0,248,166,24,0,0,47,5,0,0,0,0,0,168,86,5,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,150,0,0,0,72,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,56,249,25,0,0,127,26,0,192,76,5,0,200,74,5,0,56,77,5,0,0,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,232,67,26,0,8,125,26,0,16,78,5,0,152,77,5,0,136,78,5,0,0,0,0,0,0,0,0,0,72,0,0,0,38,0,0,0,112,0,0,0,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,247,25,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,247,25,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,96,68,26,0,80,125,26,0,152,48,5,0,32,48,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,162,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,64,26,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,64,26,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,120,70,26,0,0,126,26,0,192,49,5,0,72,49,5,0,88,85,5,0,0,0,0,0,36,0,0,0,0,0,0,0,0,0,0,0,38,0,0,0,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,94,25,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,94,25,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,110,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,250,25,0,192,158,25,0,232,50,5,0,112,50,5,0,0,0,0,0,0,0,0,0,92,0,0,0,0,0,0,0,64,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,232,3,0,0,120,0,0,0,0,0,0,0,0,128,64,192,32,160,96,224,16,144,80,208,48,176,112,240,8,136,72,200,40,168,104,232,24,152,88,216,56,184,120,248,4,132,68,196,36,164,100,228,20,148,84,212,52,180,116,244,12,140,76,204,44,172,108,236,28,156,92,220,60,188,124,252,2,130,66,194,34,162,98,226,18,146,82,210,50,178,114,242,10,138,74,202,42,170,106,234,26,154,90,218,58,186,122,250,6,134,70,198,38,166,102,230,22,150,86,214,54,182,118,246,14,142,78,206,46,174,110,238,30,158,94,222,62,190,126,254,1,129,65,193,33,161,97,225,17,145,81,209,49,177,113,241,9,137,73,201,41,169,105,233,25,153,89,217,57,185,121,249,5,133,69,197,37,165,101,229,21,149,85,213,53,181,117,245,13,141,77,205,45,173,109,237,29,157,93,221,61,189,125,253,3,131,67,195,35,163,99,227,19,147,83,211,51,179,115,243,11,139,75,203,43,171,107,235,27,155,91,219,59,187,123,251,7,135,71,199,39,167,103,231,23,151,87,215,55,183,119,247,15,143,79,207,47,175,111,239,31,159,95,223,63,191,127,255,0,0,0,0,0,0,0,0,104,228,24,0,3,1,1,16,32,56,33,56,34,56,0,0,88,46,26,0,3,1,0,0,36,56,76,56,140,56,0,0,16,198,25,0,3,0,0,32,40,56,72,56,104,56,0,0,240,121,25,0,3,0,0,32,104,56,72,56,40,56,0,0,144,63,25,0,3,1,0,16,32,56,33,56,34,56,0,0,216,246,24,0,3,0,0,16,32,56,33,56,34,56,0,0,56,219,24,0,3,2,2,16,32,56,33,56,34,56,0,0,24,195,24,0,3,2,0,16,32,56,33,56,34,56,0,0,152,173,24,0,1,0,0,64,32,56,0,0,0,0,0,0,248,154,24,0,1,0,0,4,32,0,0,0,0,0,0,0,192,138,26,0,1,0,0,4,32,7,0,0,0,0,0,0,56,121,26,0,1,0,0,2,32,56,0,0,0,0,0,0,80,102,26,0,3,1,1,16,32,56,33,56,34,56,0,0,224,84,26,0,3,1,0,16,32,56,33,56,34,56,0,0,32,69,26,0,3,0,0,16,32,56,33,56,34,56,0,0,64,53,26,0,0,0,0,8,0,0,0,0,0,0,0,0,56,38,26,0,0,0,0,8,0,0,0,0,0,0,0,0,64,28,26,0,3,1,0,0,68,56,44,56,108,56,0,0,40,15,26,0,3,2,0,0,76,56,52,56,148,56,0,0,144,6,26,0,3,0,0,96,32,16,32,19,32,14,0,0,176,255,25,0,3,0,0,36,140,0,76,8,44,0,0,0,224,245,25,0,3,0,0,96,32,0,32,9,32,3,0,0,80,234,25,0,3,0,0,96,32,14,32,19,32,16,0,0,232,225,25,0,3,0,0,36,44,0,76,8,140,0,0,0,208,216,25,0,3,0,0,96,32,3,32,9,32,0,0,0,240,202,25,0,3,1,1,16,32,56,37,56,69,56,0,0,216,190,25,0,3,1,1,16,32,56,69,56,37,56,0,0,240,184,25,0,4,0,0,160,76,56,108,56,140,56,44,56,224,176,25,0,4,0,0,160,44,56,76,56,108,56,140,56,120,170,25,0,4,0,0,160,140,56,108,56,76,56,44,56,200,162,25,0,4,0,0,160,108,56,76,56,44,56,140,56,216,154,25,0,1,0,0,1,36,120,0,0,0,0,0,0,96,147,25,0,1,0,0,0,36,120,0,0,0,0,0,0,224,140,25,0,3,0,1,16,32,56,33,56,34,56,0,0,8,134,25,0,3,0,1,16,32,56,33,56,34,56,0,0,0,126,25,0,4,1,1,144,32,56,33,56,34,56,35,56,216,118,25,0,0,1,1,8,0,0,0,0,0,0,0,0,96,113,25,0,0,1,1,8,0,0,0,0,0,0,0,0,160,108,25,0,0,1,1,8,0,0,0,0,0,0,0,0,232,102,25,0,0,1,1,8,0,0,0,0,0,0,0,0,160,97,25,0,0,1,1,8,0,0,0,0,0,0,0,0,136,91,25,0,3,0,0,33,52,120,116,120,180,120,0,0,40,86,25,0,3,0,0,32,52,120,116,120,180,120,0,0,72,81,25,0,3,0,0,33,4,35,36,45,36,32,0,0,152,76,25,0,3,0,0,32,68,35,36,45,36,32,0,0,248,67,25,0,3,0,0,33,4,34,36,37,36,32,0,0,128,61,25,0,3,0,0,32,68,34,36,37,36,32,0,0,152,55,25,0,3,0,0,33,36,32,36,45,4,35,0,0,120,48,25,0,3,0,0,32,36,32,36,45,68,35,0,0,80,42,25,0,3,0,0,33,36,32,36,37,4,34,0,0,48,32,25,0,3,0,0,32,36,32,36,37,68,34,0,0,48,26,25,0,0,1,1,8,0,0,0,0,0,0,0,0,72,19,25,0,0,1,1,8,0,0,0,0,0,0,0,0,56,9,25,0,0,1,1,8,0,0,0,0,0,0,0,0,96,3,25,0,3,1,1,16,36,120,37,120,38,120,0,0,104,252,24,0,3,1,1,17,36,120,37,120,38,120,0,0,88,245,24,0,3,1,0,16,36,120,37,120,38,120,0,0,88,243,24,0,3,1,0,17,36,120,37,120,38,120,0,0,192,240,24,0,3,0,0,16,36,120,37,120,38,120,0,0,168,237,24,0,3,0,0,17,36,120,37,120,38,120,0,0,232,235,24,0,0,1,1,8,0,0,0,0,0,0,0,0,192,231,24,0,0,1,1,8,0,0,0,0,0,0,0,0,136,228,24,0,3,0,0,32,68,24,36,28,36,24,0,0,112,226,24,0,3,0,0,33,4,24,36,28,36,24,0,0,72,224,24,0,3,0,0,32,36,24,36,28,68,24,0,0,192,220,24,0,3,0,0,33,36,24,36,28,4,24,0,0,16,218,24,0,2,0,0,128,36,56,68,56,0,0,0,0,96,216,24,0,3,0,0,33,180,120,116,120,52,120,0,0,128,214,24,0,3,0,0,32,180,120,116,120,52,120,0,0,80,212,24,0,3,1,1,17,36,64,37,64,38,64,0,0,56,210,24,0,3,1,1,16,36,64,37,64,38,64,0,0,112,206,24,0,3,1,1,17,36,72,37,72,38,72,0,0,120,203,24,0,3,1,1,16,36,72,37,72,38,72,0,0,64,201,24,0,3,1,0,17,36,72,37,72,38,72,0,0,144,199,24,0,3,1,0,16,36,72,37,72,38,72,0,0,160,196,24,0,3,0,0,17,36,64,37,64,38,64,0,0,72,194,24,0,3,0,0,16,36,64,37,64,38,64,0,0,176,192,24,0,3,0,0,17,36,72,37,72,38,72,0,0,96,191,24,0,3,0,0,16,36,72,37,72,38,72,0,0,248,189,24,0,3,1,0,17,36,64,37,64,38,64,0,0,136,184,24,0,3,1,0,16,36,64,37,64,38,64,0,0,40,181,24,0,0,1,1,8,0,0,0,0,0,0,0,0,208,179,24,0,3,0,0,48,34,56,32,56,33,56,0,0,80,178,24,0,3,0,0,49,38,64,36,64,37,64,0,0,192,176,24,0,3,0,0,48,38,64,36,64,37,64,0,0,232,174,24,0,3,0,0,49,38,72,36,72,37,72,0,0,232,172,24,0,3,0,0,48,38,72,36,72,37,72,0,0,176,171,24,0,3,0,0,49,38,120,36,120,37,120,0,0,96,170,24,0,3,0,0,48,38,120,36,120,37,120,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,168,24,0,4,1,1,145,36,64,37,64,38,64,39,64,40,168,24,0,4,1,1,144,36,64,37,64,38,64,39,64,56,165,24,0,4,1,0,145,36,64,37,64,38,64,39,64,224,162,24,0,4,1,0,144,36,64,37,64,38,64,39,64,192,160,24,0,4,0,0,145,36,64,37,64,38,64,39,64,176,158,24,0,4,0,0,144,36,64,37,64,38,64,39,64,176,155,24,0,4,1,1,145,36,72,37,72,38,72,39,72,64,154,24,0,4,1,1,144,36,72,37,72,38,72,39,72,24,153,24,0,4,1,0,145,36,72,37,72,38,72,39,72,192,151,24,0,4,1,0,144,36,72,37,72,38,72,39,72,240,150,24,0,4,0,0,145,36,72,37,72,38,72,39,72,72,160,26,0,4,0,0,144,36,72,37,72,38,72,39,72,64,157,26,0,4,1,1,145,36,120,37,120,38,120,39,120,8,155,26,0,4,1,1,144,36,120,37,120,38,120,39,120,32,149,26,0,4,1,0,145,36,120,37,120,38,120,39,120,16,143,26,0,4,1,0,144,36,120,37,120,38,120,39,120,64,140,26,0,4,0,0,145,36,120,37,120,38,120,39,120,72,138,26,0,4,0,0,144,36,120,37,120,38,120,39,120,72,137,26,0,0,1,1,8,0,0,0,0,0,0,0,0,56,136,26,0,3,0,0,0,52,92,116,92,180,92,0,0,72,135,26,0,3,0,0,1,52,92,116,92,180,92,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,216,133,26,0,4,0,0,161,60,120,124,120,188,120,252,120,248,131,26,0,4,0,0,160,60,120,124,120,188,120,252,120,208,129,26,0,4,0,0,161,188,120,124,120,60,120,252,120,32,128,26,0,4,0,0,160,188,120,124,120,60,120,252,120,200,125,26,0,3,0,0,32,76,56,108,56,140,56,0,0,72,122,26,0,3,0,0,32,44,56,76,56,108,56,0,0,208,120,26,0,3,0,0,32,140,56,108,56,76,56,0,0,192,119,26,0,3,0,0,32,108,56,76,56,44,56,0,0,160,118,26,0,4,0,0,144,32,56,33,56,34,56,35,56,208,116,26,0,4,1,0,144,32,56,33,56,34,56,35,56,240,115,26,0,3,1,1,17,36,88,37,88,38,88,0,0,200,113,26,0,3,1,1,16,36,88,37,88,38,88,0,0,192,111,26,0,3,1,1,17,36,104,37,104,38,104,0,0,184,108,26,0,3,1,1,16,36,104,37,104,38,104,0,0,112,106,26,0,3,1,0,17,36,88,37,88,38,88,0,0,8,103,26,0,3,1,0,16,36,88,37,88,38,88,0,0,232,101,26,0,3,1,0,17,36,104,37,104,38,104,0,0,168,100,26,0,3,1,0,16,36,104,37,104,38,104,0,0,104,99,26,0,3,0,0,17,36,88,37,88,38,88,0,0,184,97,26,0,3,0,0,16,36,88,37,88,38,88,0,0,8,97,26,0,3,0,0,17,36,104,37,104,38,104,0,0,216,93,26,0,3,0,0,16,36,104,37,104,38,104,0,0,192,91,26,0,3,0,0,49,38,88,36,88,37,88,0,0,40,90,26,0,3,0,0,48,38,88,36,88,37,88,0,0,248,88,26,0,3,0,0,49,38,104,36,104,37,104,0,0,88,86,26,0,3,0,0,48,38,104,36,104,37,104,0,0,120,84,26,0,4,0,0,176,34,56,32,56,33,56,35,56,104,83,26,0,4,0,0,177,38,120,36,120,37,120,39,120,72,82,26,0,4,0,0,176,38,120,36,120,37,120,39,120,144,80,26,0,3,2,0,16,32,56,33,56,34,56,0,0,8,139,26,0,88,121,26,0,224,173,24,0,40,155,24,0,184,246,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,195,0,0,0,0,0,0,224,67,0,0,0,0,184,246,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,53,26,0,96,0,0,0,232,72,5,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,192,89,26,0,0,0,0,0,216,143,3,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,152,225,25,0,146,0,0,0,128,187,1,0,100,39,52,0,0,0,0,0,0,0,0,0,40,0,0,0,48,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,7,0,0,0,0,16,0,0,5,128,0,0,0,16,0,0,33,16,0,0,0,32,0,0,183,29,193,4,1,32,0,0,32,131,184,237,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,24,0,0,251,76,134,0,120,228,25,0,54,0,0,0,208,125,1,0,100,39,52,0,4,0,0,0,0,0,0,0,28,0,0,0,12,0,0,0,5,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,232,86,25,0,124,0,0,0,192,111,1,0,100,39,52,0,4,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,205,204,204,61,0,0,0,0,64,34,26,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,104,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,144,157,25,0,104,93,25,0,16,0,0,0,3,0,0,0,255,255,255,255,255,255,239,127,255,255,255,255,255,255,239,255,255,255,255,255,255,255,239,127,8,0,1,0,0,0,0,0,160,27,25,0,232,232,24,0,24,0,0,0,3,0,0,0,255,255,255,255,255,255,239,127,255,255,255,255,255,255,239,255,255,255,255,255,255,255,239,127,8,0,1,0,0,0,0,0,120,207,24,0,216,181,24,0,48,0,0,0,2,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,224,195,0,0,0,0,0,0,224,67,8,0,1,0,0,0,0,0,232,165,24,0,8,158,26,0,56,0,0,0,2,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,224,195,0,0,0,0,0,0,224,67,8,0,1,0,0,0,0,0,136,132,26,0,192,114,26,0,8,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,239,127,8,0,1,0,0,0,0,0,224,94,26,0,40,78,26,0,64,0,0,0,2,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,224,67,8,0,1,0,0,0,0,0,48,64,26,0,88,48,26,0,72,0,0,0,2,0,0,0,255,255,255,255,255,255,255,127,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,67,8,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,34,26,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,108,0,0,0,0,0,0,0,0,0,0,0,120,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,56,249,25,0,96,0,0,0,64,75,5,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,232,236,24,0,96,0,0,0,32,202,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,80,207,24,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,60,0,0,0,170,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,80,207,24,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,116,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,232,67,26,0,96,0,0,0,128,202,1,0,100,39,52,0,0,0,0,0,0,0,0,0,60,0,0,0,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,51,51,211,64,0,0,17,67,51,51,115,63,0,0,0,0,0,0,240,193,0,0,48,65,82,73,157,58,1,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,51,51,211,64,0,0,17,67,51,51,115,63,0,0,0,0,0,0,200,193,0,0,48,65,111,18,131,58,1,0,0,0,24,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,51,51,211,64,0,0,17,67,51,51,115,63,0,0,0,0,0,0,160,193,0,0,48,65,111,18,131,58,1,0,0,0,32,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,51,51,211,64,0,0,17,67,51,51,115,63,0,0,0,0,0,0,112,193,0,0,48,65,111,18,131,58,1,0,0,0,40,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,51,51,211,64,0,0,17,67,51,51,115,63,0,0,0,0,0,0,32,193,0,0,48,65,250,237,107,58,1,0,0,0,48,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,51,51,211,64,0,0,17,67,51,51,115,63,0,0,0,0,0,0,32,193,0,0,48,65,250,237,107,58,1,0,0,0,56,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,51,51,211,64])
.concat([0,0,17,67,51,51,115,63,0,0,0,0,0,0,192,192,0,0,48,65,23,183,81,58,1,0,0,0,64,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,51,51,211,64,0,0,17,67,51,51,115,63,0,0,0,0,0,0,0,192,0,0,48,65,23,183,81,58,1,0,0,0,80,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,51,51,211,64,0,0,17,67,51,51,115,63,0,0,0,0,0,0,0,0,0,0,0,65,52,128,55,58,1,0,0,0,96,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,32,64,51,51,211,64,0,0,17,67,51,51,115,63,0,0,0,0,0,0,128,63,0,0,176,64,82,73,29,58,1,0,0,0,112,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,16,64,51,51,211,64,0,0,17,67,51,51,115,63,0,0,0,0,0,0,0,64,0,0,144,64,111,18,3,58,1,0,0,0,128,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,154,153,249,63,205,204,204,64,0,0,12,67,51,51,115,63,0,0,0,0,0,0,64,64,0,0,128,64,23,183,81,57,1,0,0,0,160,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,184,30,229,63,0,0,192,64,0,0,7,67,51,51,115,63,0,0,0,192,0,0,160,64,0,0,96,64,0,0,0,0,1,0,0,0,192,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,82,184,190,63,51,51,179,64,0,0,250,66,236,81,120,63,0,0,128,192,0,0,224,64,0,0,64,64,0,0,0,0,0,0,0,0,224,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,0,0,160,63,102,102,166,64,0,0,250,66,72,225,122,63,0,0,192,192,0,0,16,65,0,0,0,64,0,0,0,0,0,0,0,0,0,1,0,0,9,0,0,0,9,0,0,0,1,0,0,0,236,81,120,63,102,102,166,64,0,0,250,66,0,0,128,63,0,0,0,193,0,0,32,65,0,0,128,63,0,0,0,0,0,0,0,0,64,1,0,0,9,0,0,0,9,0,0,0,1,0,0,0,102,102,102,63,102,102,166,64,0,0,250,66,0,0,128,63,0,0,32,193,0,0,64,65,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,2,0,1,3,0,0,0,0,4,0,1,3,4,5,6,7,8,9,10,11,12,13,14,15,4,0,1,3,4,5,6,7,8,9,10,11,12,13,14,15,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,3,0,1,3,4,5,6,7,4,0,2,4,5,6,7,8,9,10,11,12,13,14,15,16,4,0,2,4,5,6,7,8,9,10,11,12,13,14,15,16,4,0,2,4,5,6,7,8,9,10,11,12,13,14,15,16,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,16,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,16,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,16,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,16,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,16,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,16,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,16,4,0,1,2,3,4,5,6,7,8,9,10,11,12,13,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,3,0,1,2,3,4,5,16,2,0,1,16,2,0,1,16,2,0,1,16,2,0,1,16,2,0,1,16,2,0,1,16,2,0,1,16,0,0,0,0,216,165,24,0,216,157,26,0,16,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,1,0,0,0,0,0,120,132,26,0,152,114,26,0,20,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,1,0,0,0,0,0,208,94,26,0,248,77,26,0,24,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,120,70,26,0,96,0,0,0,152,84,5,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,159,26,0,24,133,26,0,20,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,0,0,0,0,80,115,26,0,176,95,26,0,48,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,0,0,0,0,224,78,26,0,168,64,26,0,28,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,0,0,0,0,0,49,26,0,200,34,26,0,36,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,1,0,0,0,0,0,64,20,26,0,56,12,26,0,44,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,16,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,104,131,26,0,96,0,0,0,136,85,5,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,159,26,0,0,0,0,0,8,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,1,0,0,0,0,0,0,133,26,0,0,0,0,0,60,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,1,0,0,0,0,0,56,115,26,0,0,0,0,0,68,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,1,0,0,0,0,0,136,95,26,0,0,0,0,0,88,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,1,0,0,0,0,0,200,78,26,0,0,0,0,0,72,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,255,255,255,223,65,8,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,159,25,0,96,0,0,0,216,86,5,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,119,1,0,136,88,1,0,0,250,0,0,128,187,0,0,68,172,0,0,0,125,0,0,192,93,0,0,34,86,0,0,128,62,0,0,224,46,0,0,17,43,0,0,64,31,0,0,0,0,0,0,0,0,0,0,32,128,25,0,120,121,25,0,0,0,0,0,0,0,0,0,176,87,25,0,168,82,25,0,8,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,9,0,0,0,0,0,0,0,248,77,25,0,224,70,25,0,12,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,9,0,0,0,0,0,0,0,32,63,25,0,248,57,25,0,16,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,240,191,0,0,0,0,0,0,0,64,9,0,0,0,32,63,25,0,48,50,25,0,32,44,25,0,0,0,0,0,128,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,32,63,25,0,168,37,25,0,200,27,25,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,32,63,25,0,80,21,25,0,88,15,25,0,0,0,0,0,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,32,63,25,0,40,5,25,0,128,253,24,0,0,0,0,0,128,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,32,63,25,0,168,246,24,0,8,244,24,0,20,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,9,0,0,0,0,0,0,0,216,241,24,0,216,238,24,0,24,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,255,239,64,9,0,0,0,0,0,0,0,136,236,24,0,96,233,24,0,28,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,64,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,112,25,0,96,0,0,0,112,88,5,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,214,24,0,96,0,0,0,176,124,1,0,100,39,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,1,1,0,1,0,1,3,0,0,0,0,0,0,0,0,1,1,0,1,1,1,3,0,0,0,0,0,0,0,0,1,1,0,1,1,1,3,3,0,4,0,0,0,0,0,1,1,0,1,1,1,1,1,2,3,3,0,4,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,7,1,0,0,0,0,0,0,55,0,0,0,0,0,0,0,63,0,0,0,0,0,0,0,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,7,1,0,0,0,0,0,0,55,0,0,0,0,0,0,0,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,24,0,0,0,23,0,0,0,25,0,0,0,22,0,0,0,26,0,0,0,21,0,0,0,27,0,0,0,20,0,0,0,28,0,0,0,19,0,0,0,29,0,0,0,18,0,0,0,30,0,0,0,17,0,0,0,31,0,0,0,16,0,0,0,32,0,0,0,15,0,0,0,33,0,0,0,14,0,0,0,34,0,0,0,13,0,0,0,35,0,0,0,12,0,0,0,36,0,0,0,11,0,0,0,37,0,0,0,10,0,0,0,38,0,0,0,9,0,0,0,39,0,0,0,8,0,0,0,40,0,0,0,7,0,0,0,41,0,0,0,6,0,0,0,42,0,0,0,5,0,0,0,43,0,0,0,4,0,0,0,44,0,0,0,3,0,0,0,45,0,0,0,2,0,0,0,46,0,0,0,1,0,0,0,47,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,12,0,0,0,5,0,0,0,13,0,0,0,4,0,0,0,14,0,0,0,3,0,0,0,15,0,0,0,2,0,0,0,16,0,0,0,1,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,24,0,0,0,23,0,0,0,25,0,0,0,22,0,0,0,26,0,0,0,21,0,0,0,27,0,0,0,20,0,0,0,28,0,0,0,19,0,0,0,29,0,0,0,18,0,0,0,30,0,0,0,17,0,0,0,31,0,0,0,16,0,0,0,32,0,0,0,15,0,0,0,33,0,0,0,14,0,0,0,34,0,0,0,13,0,0,0,35,0,0,0,12,0,0,0,36,0,0,0,11,0,0,0,37,0,0,0,10,0,0,0,38,0,0,0,9,0,0,0,39,0,0,0,8,0,0,0,40,0,0,0,7,0,0,0,41,0,0,0,6,0,0,0,42,0,0,0,5,0,0,0,43,0,0,0,4,0,0,0,44,0,0,0,3,0,0,0,45,0,0,0,2,0,0,0,46,0,0,0,1,0,0,0,47,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,12,0,0,0,5,0,0,0,13,0,0,0,4,0,0,0,14,0,0,0,3,0,0,0,15,0,0,0,2,0,0,0,16,0,0,0,1,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,24,0,0,0,23,0,0,0,25,0,0,0,22,0,0,0,26,0,0,0,21,0,0,0,27,0,0,0,20,0,0,0,28,0,0,0,19,0,0,0,29,0,0,0,18,0,0,0,30,0,0,0,17,0,0,0,31,0,0,0,16,0,0,0,32,0,0,0,15,0,0,0,33,0,0,0,14,0,0,0,34,0,0,0,13,0,0,0,35,0,0,0,12,0,0,0,36,0,0,0,11,0,0,0,37,0,0,0,10,0,0,0,38,0,0,0,9,0,0,0,39,0,0,0,8,0,0,0,40,0,0,0,7,0,0,0,41,0,0,0,6,0,0,0,42,0,0,0,5,0,0,0,43,0,0,0,4,0,0,0,44,0,0,0,3,0,0,0,45,0,0,0,2,0,0,0,46,0,0,0,1,0,0,0,47,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0])
.concat([8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,8,0,0,0,16,0,0,0,7,0,0,0,17,0,0,0,6,0,0,0,18,0,0,0,5,0,0,0,19,0,0,0,4,0,0,0,20,0,0,0,3,0,0,0,21,0,0,0,2,0,0,0,22,0,0,0,1,0,0,0,23,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,24,0,0,0,23,0,0,0,25,0,0,0,22,0,0,0,26,0,0,0,21,0,0,0,27,0,0,0,20,0,0,0,28,0,0,0,19,0,0,0,29,0,0,0,18,0,0,0,30,0,0,0,17,0,0,0,31,0,0,0,16,0,0,0,32,0,0,0,15,0,0,0,33,0,0,0,14,0,0,0,34,0,0,0,13,0,0,0,35,0,0,0,12,0,0,0,36,0,0,0,11,0,0,0,37,0,0,0,10,0,0,0,38,0,0,0,9,0,0,0,39,0,0,0,8,0,0,0,40,0,0,0,7,0,0,0,41,0,0,0,6,0,0,0,42,0,0,0,5,0,0,0,43,0,0,0,4,0,0,0,44,0,0,0,3,0,0,0,45,0,0,0,2,0,0,0,46,0,0,0,1,0,0,0,47,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,12,0,0,0,5,0,0,0,13,0,0,0,4,0,0,0,14,0,0,0,3,0,0,0,15,0,0,0,2,0,0,0,16,0,0,0,1,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,12,0,0,0,5,0,0,0,13,0,0,0,4,0,0,0,14,0,0,0,3,0,0,0,15,0,0,0,2,0,0,0,16,0,0,0,1,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,24,0,0,0,23,0,0,0,25,0,0,0,22,0,0,0,26,0,0,0,21,0,0,0,27,0,0,0,20,0,0,0,28,0,0,0,19,0,0,0,29,0,0,0,18,0,0,0,30,0,0,0,17,0,0,0,31,0,0,0,16,0,0,0,32,0,0,0,15,0,0,0,33,0,0,0,14,0,0,0,34,0,0,0,13,0,0,0,35,0,0,0,12,0,0,0,36,0,0,0,11,0,0,0,37,0,0,0,10,0,0,0,38,0,0,0,9,0,0,0,39,0,0,0,8,0,0,0,40,0,0,0,7,0,0,0,41,0,0,0,6,0,0,0,42,0,0,0,5,0,0,0,43,0,0,0,4,0,0,0,44,0,0,0,3,0,0,0,45,0,0,0,2,0,0,0,46,0,0,0,1,0,0,0,47,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,12,0,0,0,5,0,0,0,13,0,0,0,4,0,0,0,14,0,0,0,3,0,0,0,15,0,0,0,2,0,0,0,16,0,0,0,1,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,24,0,0,0,23,0,0,0,25,0,0,0,22,0,0,0,26,0,0,0,21,0,0,0,27,0,0,0,20,0,0,0,28,0,0,0,19,0,0,0,29,0,0,0,18,0,0,0,30,0,0,0,17,0,0,0,31,0,0,0,16,0,0,0,32,0,0,0,15,0,0,0,33,0,0,0,14,0,0,0,34,0,0,0,13,0,0,0,35,0,0,0,12,0,0,0,36,0,0,0,11,0,0,0,37,0,0,0,10,0,0,0,38,0,0,0,9,0,0,0,39,0,0,0,8,0,0,0,40,0,0,0,7,0,0,0,41,0,0,0,6,0,0,0,42,0,0,0,5,0,0,0,43,0,0,0,4,0,0,0,44,0,0,0,3,0,0,0,45,0,0,0,2,0,0,0,46,0,0,0,1,0,0,0,47,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,24,0,0,0,23,0,0,0,25,0,0,0,22,0,0,0,26,0,0,0,21,0,0,0,27,0,0,0,20,0,0,0,28,0,0,0,19,0,0,0,29,0,0,0,18,0,0,0,30,0,0,0,17,0,0,0,31,0,0,0,16,0,0,0,32,0,0,0,15,0,0,0,33,0,0,0,14,0,0,0,34,0,0,0,13,0,0,0,35,0,0,0,12,0,0,0,36,0,0,0,11,0,0,0,37,0,0,0,10,0,0,0,38,0,0,0,9,0,0,0,39,0,0,0,8,0,0,0,40,0,0,0,7,0,0,0,41,0,0,0,6,0,0,0,42,0,0,0,5,0,0,0,43,0,0,0,4,0,0,0,44,0,0,0,3,0,0,0,45,0,0,0,2,0,0,0,46,0,0,0,1,0,0,0,47,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0])
.concat([5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,24,0,0,0,23,0,0,0,25,0,0,0,22,0,0,0,26,0,0,0,21,0,0,0,27,0,0,0,20,0,0,0,28,0,0,0,19,0,0,0,29,0,0,0,18,0,0,0,30,0,0,0,17,0,0,0,31,0,0,0,16,0,0,0,32,0,0,0,15,0,0,0,33,0,0,0,14,0,0,0,34,0,0,0,13,0,0,0,35,0,0,0,12,0,0,0,36,0,0,0,11,0,0,0,37,0,0,0,10,0,0,0,38,0,0,0,9,0,0,0,39,0,0,0,8,0,0,0,40,0,0,0,7,0,0,0,41,0,0,0,6,0,0,0,42,0,0,0,5,0,0,0,43,0,0,0,4,0,0,0,44,0,0,0,3,0,0,0,45,0,0,0,2,0,0,0,46,0,0,0,1,0,0,0,47,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,12,0,0,0,5,0,0,0,13,0,0,0,4,0,0,0,14,0,0,0,3,0,0,0,15,0,0,0,2,0,0,0,16,0,0,0,1,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,24,0,0,0,23,0,0,0,25,0,0,0,22,0,0,0,26,0,0,0,21,0,0,0,27,0,0,0,20,0,0,0,28,0,0,0,19,0,0,0,29,0,0,0,18,0,0,0,30,0,0,0,17,0,0,0,31,0,0,0,16,0,0,0,32,0,0,0,15,0,0,0,33,0,0,0,14,0,0,0,34,0,0,0,13,0,0,0,35,0,0,0,12,0,0,0,36,0,0,0,11,0,0,0,37,0,0,0,10,0,0,0,38,0,0,0,9,0,0,0,39,0,0,0,8,0,0,0,40,0,0,0,7,0,0,0,41,0,0,0,6,0,0,0,42,0,0,0,5,0,0,0,43,0,0,0,4,0,0,0,44,0,0,0,3,0,0,0,45,0,0,0,2,0,0,0,46,0,0,0,1,0,0,0,47,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,12,0,0,0,5,0,0,0,13,0,0,0,4,0,0,0,14,0,0,0,3,0,0,0,15,0,0,0,2,0,0,0,16,0,0,0,1,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,13,0,0,0,6,0,0,0,14,0,0,0,5,0,0,0,15,0,0,0,4,0,0,0,16,0,0,0,3,0,0,0,17,0,0,0,2,0,0,0,18,0,0,0,1,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,3,0,0,0,11,0,0,0,2,0,0,0,12,0,0,0,1,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,10,0,0,0,1,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,1,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,3,0,0,0,13,0,0,0,2,0,0,0,14,0,0,0,1,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,1,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,3,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0])
.concat([11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,3,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0])
.concat([10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,3,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,19,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,16,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,15,0,0,0,19,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0])
.concat([4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,3,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0])
.concat([11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,16,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,4,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,4,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0])
.concat([11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0])
.concat([10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0])
.concat([13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,18,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,18,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,0,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,19,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0])
.concat([5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,9,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,4,0,0,0,8,0,0,0,9,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,13,0,0,0,10,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,20,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,20,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,16,0,0,0,20,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,4,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,4,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,4,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0])
.concat([13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,0,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,20,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,20,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,3,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,20,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,18,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,20,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,20,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,16,0,0,0])
.concat([19,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,18,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,16,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,17,0,0,0,20,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,18,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,0,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,20,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,19,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,4,0,0,0,8,0,0,0])
.concat([8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,18,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,19,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0])
.concat([4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,18,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0])
.concat([10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,18,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,1,0,0,0,7,0,0,0])
.concat([7,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,16,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,18,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,20,0,0,0,18,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,20,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,18,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,18,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0])
.concat([13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,21,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,21,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0])
.concat([17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,21,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,21,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,21,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,18,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,22,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,22,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,16,0,0,0,16,0,0,0,4,0,0,0,9,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,4,0,0,0,12,0,0,0,8,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,11,0,0,0,8,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,15,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0])
.concat([16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,6,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0])
.concat([16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,1,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0])
.concat([13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0])
.concat([14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0])
.concat([12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0])
.concat([13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0])
.concat([16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0])
.concat([13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0])
.concat([8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0])
.concat([15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0])
.concat([11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0])
.concat([11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0])
.concat([14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,15,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0])
.concat([15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0])
.concat([10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0])
.concat([12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0])
.concat([12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,15,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0])
.concat([9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,19,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,22,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,22,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,17,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,21,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,19,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,22,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,20,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,16,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,19,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,22,0,0,0,20,0,0,0,17,0,0,0])
.concat([17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,23,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,17,0,0,0,21,0,0,0,22,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,19,0,0,0,23,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,20,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,16,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0])
.concat([12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0])
.concat([14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,19,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,15,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0])
.concat([9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,5,0,0,0,9,0,0,0,7,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,16,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,22,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,19,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,20,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,22,0,0,0,19,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,15,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,20,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,18,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,22,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,21,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,22,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0])
.concat([19,0,0,0,21,0,0,0,18,0,0,0,22,0,0,0,20,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,21,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,22,0,0,0,21,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,22,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,21,0,0,0,18,0,0,0,0,0,0,0,18,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,19,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,20,0,0,0,15,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,16,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,19,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,22,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,21,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,22,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,22,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,20,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,21,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,19,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,20,0,0,0,15,0,0,0,21,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,21,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,21,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,21,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,22,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,22,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,22,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,22,0,0,0,21,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,22,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,22,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,21,0,0,0,19,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,22,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,22,0,0,0,20,0,0,0,20,0,0,0,22,0,0,0,18,0,0,0,22,0,0,0,18,0,0,0,22,0,0,0,18,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,20,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,18,0,0,0,22,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,22,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,20,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,21,0,0,0,17,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,21,0,0,0,17,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,19,0,0,0,22,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,21,0,0,0,17,0,0,0,22,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,19,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,22,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,22,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,17,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,22,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,21,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,20,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,20,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,22,0,0,0,17,0,0,0,20,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,21,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,19,0,0,0,22,0,0,0,19,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,21,0,0,0,17,0,0,0,22,0,0,0,20,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,20,0,0,0,17,0,0,0,21,0,0,0,22,0,0,0,21,0,0,0,20,0,0,0,22,0,0,0,18,0,0,0,21,0,0,0,17,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,22,0,0,0,20,0,0,0,21,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,22,0,0,0,17,0,0,0,21,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,20,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,22,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,17,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,17,0,0,0,21,0,0,0,16,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0])
.concat([12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0])
.concat([12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0])
.concat([16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,19,0,0,0,15,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0])
.concat([8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,1,0,0,0,7,0,0,0,6,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0])
.concat([0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,23,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,23,0,0,0,22,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0])
.concat([11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0])
.concat([20,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,11,0,0,0])
.concat([11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0])
.concat([15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,22,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,22,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0])
.concat([8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,17,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,22,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0])
.concat([20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,19,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,22,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,22,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,22,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0])
.concat([14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0])
.concat([0,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,21,0,0,0,22,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0])
.concat([9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0])
.concat([19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0])
.concat([12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,20,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,19,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,21,0,0,0,20,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,19,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,21,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,18,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,17,0,0,0,18,0,0,0,21,0,0,0])
.concat([0,0,0,0,0,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,19,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,17,0,0,0,20,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,20,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,21,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,21,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,21,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0])
.concat([11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,20,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,16,0,0,0,15,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,17,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,3,0,0,0,8,0,0,0,3,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,5,0,0,0,14,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0])
.concat([10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,11,0,0,0,3,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0])
.concat([9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,21,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,21,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,21,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,21,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,21,0,0,0,20,0,0,0,20,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0])
.concat([17,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,21,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,21,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0])
.concat([0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,16,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,19,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,19,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,20,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,18,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,18,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,21,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,21,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,21,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,21,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,18,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,20,0,0,0,18,0,0,0,19,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,3,0,0,0,8,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0])
.concat([6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,19,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,3,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,18,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0])
.concat([12,0,0,0,12,0,0,0,2,0,0,0,9,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,18,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,18,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,4,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,2,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,20,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,4,0,0,0,8,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,8,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,19,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,19,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0])
.concat([10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,3,0,0,0,7,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,14,0,0,0,9,0,0,0,14,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,3,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0])
.concat([10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0])
.concat([10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,5,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,4,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,4,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,8,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,16,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,19,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,20,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,19,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0])
.concat([6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,19,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,0,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,0,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,19,0,0,0,17,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,18,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,18,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,0,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,14,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,0,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,19,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,18,0,0,0,0,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,0,0,0,0,17,0,0,0,14,0,0,0,18,0,0,0,16,0,0,0,18,0,0,0,0,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,19,0,0,0,18,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,19,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,0,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,0,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,17,0,0,0,0,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,4,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,4,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0])
.concat([12,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,20,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,20,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,20,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,21,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0])
.concat([6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,253,255,255,255,248,255,255,255,243,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,247,255,255,255,247,255,255,255,247,255,255,255,247,255,255,255,247,255,255,255,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,252,255,255,255,246,255,255,255,242,255,255,255,240,255,255,255,242,255,255,255,243,255,255,255,244,255,255,255,244,255,255,255,245,255,255,255,245,255,255,255,246,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,250,255,255,255,244,255,255,255,242,255,255,255,240,255,255,255,241,255,255,255,241,255,255,255,242,255,255,255,243,255,255,255,243,255,255,255,244,255,255,255,244,255,255,255,254,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,244,255,255,255,243,255,255,255,242,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,243,255,255,255,244,255,255,255,244,255,255,255,251,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,242,255,255,255,243,255,255,255,243,255,255,255,243,255,255,255,246,255,255,255,252,255,255,255,254,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,243,255,255,255,245,255,255,255,246,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,243,255,255,255,245,255,255,255,246,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,248,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,247,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,247,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,247,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,247,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,253,255,255,255,248,255,255,255,243,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,252,255,255,255,246,255,255,255,242,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,243,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,245,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,250,255,255,255,244,255,255,255,242,255,255,255,240,255,255,255,241,255,255,255,241,255,255,255,242,255,255,255,243,255,255,255,243,255,255,255,244,255,255,255,244,255,255,255,254,255,255,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,244,255,255,255,243,255,255,255,242,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,243,255,255,255,244,255,255,255,244,255,255,255,250,255,255,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,242,255,255,255,243,255,255,255,243,255,255,255,243,255,255,255,246,255,255,255,252,255,255,255,254,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,243,255,255,255,245,255,255,255,246,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,243,255,255,255,245,255,255,255,246,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,248,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,247,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,247,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,247,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,241,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,247,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,250,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,250,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,250,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,0,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,246,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,246,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,246,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,236,255,255,255,246,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,246,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,2,0,0,0,0,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,246,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,246,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,115,116,114,101,97,109,32,115,112,101,99,105,102,105,101,114,58,32,37,115,46,10,0,0,37,100,32,102,114,97,109,101,32,100,117,112,108,105,99,97,116,105,111,110,32,116,111,111,32,108,97,114,103,101,44,32,115,107,105,112,112,105,110,103,10,0,0,0,0,0,0,0,121,117,118,97,52,52,52,112,49,48,98,101,0,0,0,0,77,101,100,105,117,109,80,117,114,112,108,101,0,0,0,0,102,95,119,101,105,103,104,116,101,100,0,0,0,0,0,0,108,111,103,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,0,0,0,81,117,105,99,107,84,105,109,101,32,71,114,97,112,104,105,99,115,32,40,83,77,67,41,0,0,0,0,0,0,0,0,65,99,111,117,115,116,105,99,0,0,0,0,0,0,0,0,32,40,100,117,98,41,0,0,118,111,108,117,109,101,32,37,115,44,32,108,101,110,32,37,100,10,0,0,0,0,0,0,118,101,114,115,105,111,110,0,115,115,0,0,0,0,0,0,37,115,37,115,47,37,115,45,37,115,46,102,102,112,114,101,115,101,116,0,0,0,0,0,42,42,42,32,100,114,111,112,33,10,0,0,0,0,0,0,121,117,118,97,52,50,50,112,49,48,108,101,0,0,0,0,77,101,100,105,117,109,79,114,99,104,105,100,0,0,0,0,115,101,108,101,99,116,32,104,105,103,104,32,115,104,105,98,97,116,97,32,110,111,105,115,101,32,115,104,97,112,105,110,103,32,100,105,116,104,101,114,0,0,0,0,0,0,0,0,108,111,103,0,0,0,0,0,115,109,99,0,0,0,0,0,69,97,115,121,32,76,105,115,116,101,110,105,110,103,0,0,32,40,100,101,102,97,117,108,116,41,0,0,0,0,0,0,109,117,108,116,105,112,108,101,32,101,100,105,116,32,108,105,115,116,32,101,110,116,114,105,101,115,44,32,97,47,118,32,100,101,115,121,110,99,32,109,105,103,104,116,32,111,99,99,117,114,44,32,112,97,116,99,104,32,119,101,108,99,111,109,101,10,0,0,0,0,0,0,99,111,109,112,114,101,115,115,105,111,110,0,0,0,0,0,108,105,109,105,116,95,115,105,122,101,0,0,0,0,0,0,47,46,102,102,109,112,101,103,0,0,0,0,0,0,0,0,45,37,45,49,55,115,32,32,37,115,10,0,0,0,0,0,101,110,99,111,100,101,114,32,45,62,32,116,121,112,101,58,97,117,100,105,111,32,112,107,116,95,112,116,115,58,37,115,32,112,107,116,95,112,116,115,95,116,105,109,101,58,37,115,32,112,107,116,95,100,116,115,58,37,115,32,112,107,116,95,100,116,115,95,116,105,109,101,58,37,115,10,0,0,0,0,121,117,118,97,52,50,50,112,49,48,98,101,0,0,0,0,77,101,100,105,117,109,66,108,117,101,0,0,0,0,0,0,104,105,103,104,95,115,104,105,98,97,116,97,0,0,0,0,100,105,97,109,111,110,100,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,40,97,108,105,97,115,32,102,111,114,32,69,80,90,83,41,0,0,0,0,0,0,81,117,105,99,107,84,105,109,101,32,56,66,80,83,32,118,105,100,101,111,0,0,0,0,67,104,111,114,117,115,0,0,116,98,99,0,0,0,0,0,169,99,112,121,0,0,0,0,85,110,107,110,111,119,110,32,77,79,86,32,102,105,101,108,100,32,111,114,100,101,114,32,48,120,37,48,52,120,10,0,105,100,51,118,50,32,118,101,114,58,37,100,32,102,108,97,103,115,58,37,48,50,88,32,108,101,110,58,37,100,10,0,115,101,116,32,116,104,101,32,108,105,109,105,116,32,102,105,108,101,32,115,105,122,101,32,105,110,32,98,121,116,101,115,0,0,0,0,0,0,0,0,37,115,37,115,47,37,115,46,102,102,112,114,101,115,101,116,0,0,0,0,0,0,0,0,101,110,99,111,100,101,95,97,117,100,105,111,32,37,100,46,37,100,0,0,0,0,0,0,121,117,118,97,52,50,48,112,49,48,108,101,0,0,0,0,77,101,100,105,117,109,65,113,117,97,77,97,114,105,110,101,0,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,115,101,108,101,99,116,32,108,111,119,32,115,104,105,98,97,116,97,32,110,111,105,115,101,32,115,104,97,112,105,110,103,32,100,105,116,104,101,114,0,100,101,102,97,117,108,116,95,103,101,116,95,98,117,102,102,101,114,32,99,97,108,108,101,100,32,111,110,32,112,105,99,32,37,112,10,0,0,0,0,100,105,97,0,0,0,0,0,56,98,112,115,0,0,0,0,66,105,103,32,66,97,110,100,0,0,0,0,0,0,0,0,109,111,110,111,119,0,0,0,116,98,110,0,0,0,0,0,169,103,101,110,0,0,0,0,109,105,110,111,114,95,118,101,114,115,105,111,110,0,0,0,84,82,75,0,0,0,0,0,37,89,37,109,37,100,0,0,102,115,0,0,0,0,0,0,45,100,0,0,0,0,0,0,65,86,95,76,79,71,95,70,79,82,67,69,95,50,53,54,67,79,76,79,82,0,0,0,114,0,0,0,0,0,0,0,99,111,115,0,0,0,0,0,65,117,100,105,111,32,101,110,99,111,100,105,110,103,32,102,97,105,108,101,100,32,40,97,118,99,111,100,101,99,95,101,110,99,111,100,101,95,97,117,100,105,111,50,41,10,0,0,68,101,99,111,100,101,114,32,110,111,116,32,102,111,117,110,100,0,0,0,0,0,0,0,121,117,118,97,52,50,48,112,49,48,98,101,0,0,0,0,77,97,114,111,111,110,0,0,70,67,0,0,0,0,0,0,108,111,119,95,115,104,105,98,97,116,97,0,0,0,0,0,112,105,99,45,62,100,97,116,97,91,48,93,33,61,78,85,76,76,32,105,110,32,97,118,99,111,100,101,99,95,100,101,102,97,117,108,116,95,103,101,116,95,98,117,102,102,101,114,10,0,0,0,0,0,0,0,102,105,114,115,116,76,117,109,83,114,99,89,32,62,61,32,108,97,115,116,73,110,76,117,109,66,117,102,32,45,32,118,76,117,109,66,117,102,83,105,122,101,32,43,32,49,0,0,116,101,115,97,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,0,0,69,114,114,111,114,32,100,105,102,102,117,115,105,111,110,32,100,105,116,104,101,114,32,105,115,32,111,110,108,121,32,115,117,112,112,111,114,116,101,100,32,105,110,32,102,117,108,108,32,99,104,114,111,109,97,32,105,110,116,101,114,112,111,108,97,116,105,111,110,32,102,111,114,32,100,101,115,116,105,110,97,116,105,111,110,32,102,111,114,109,97,116,32,39,37,115,39,10,0,0,0,0,0,0,105,100,32,81,117,97,107,101,32,73,73,32,67,73,78,32,118,105,100,101,111,0,0,0,83,108,111,119,32,82,111,99,107,0,0,0,0,0,0,0,110,101,97,114,101,115,116,32,110,101,105,103,104,98,111,114,0,0,0,0,0,0,0,0,84,114,101,97,116,105,110,103,32,37,115,32,97,115,32,109,111,110,111,10,0,0,0,0,116,98,114,0,0,0,0,0,169,99,109,116,0,0,0,0,109,97,106,111,114,95,98,114,97,110,100,0,0,0,0,0,84,80,51,0,0,0,0,0,105,99,104,0,0,0,0,0,116,105,109,101,95,115,116,111,112,0,0,0,0,0,0,0,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,115,104,97,114,101,47,102,102,109,112,101,103,0,0,0,0,78,111,116,32,101,118,97,108,117,97,116,105,110,103,32,97,32,102,117,114,116,104,101,114,32,112,114,111,103,114,97,109,95,99,111,110,102,105,103,95,101,108,101,109,101,110,116,32,97,115,32,116,104,105,115,32,99,111,110,115,116,114,117,99,116,32,105,115,32,100,117,98,105,111,117,115,32,97,116,32,98,101,115,116,46,10,0,0,98,108,111,99,107,95,108,101,110,95,98,105,116,115,32,110,111,116,32,105,110,105,116,105,97,108,105,122,101,100,32,116,111,32,97,32,118,97,108,105,100,32,118,97,108,117,101,10,0,0,0,0,0,0,0,0,37,100,32,109,111,100,101,115,32,40,101,105,116,104,101,114,32,97,32,102,97,108,115,101,32,112,111,115,105,116,105,118,101,32,111,114,32,97,32,115,97,109,112,108,101,32,102,114,111,109,32,97,110,32,117,110,107,110,111,119,110,32,101,110,99,111,100,101,114,41,0,0,112,107,116,46,115,105,122,101,32,124,124,32,33,112,107,116,46,100,97,116,97,0,0,0,121,117,118,97,52,52,52,112,57,108,101,0,0,0,0,0,77,97,103,101,110,116,97,0,78,111,32,99,111,100,101,99,32,112,114,111,118,105,100,101,100,32,116,111,32,97,118,99,111,100,101,99,95,111,112,101,110,50,40,41,10,0,0,0,73,110,118,97,108,105,100,32,80,67,77,32,112,97,99,107,101,116,44,32,100,97,116,97,32,104,97,115,32,115,105,122,101,32,37,100,32,98,117,116,32,97,116,32,108,101,97,115,116,32,97,32,115,105,122,101,32,111,102,32,37,100,32,119,97,115,32,101,120,112,101,99,116,101,100,10,0,0,0,0,115,101,108,101,99,116,32,115,104,105,98,97,116,97,32,110,111,105,115,101,32,115,104,97,112,105,110,103,32,100,105,116,104,101,114,0,0,0,0,0,100,101,102,97,117,108,116,95,103,101,116,95,98,117,102,102,101,114,32,99,97,108,108,101,100,32,111,110,32,102,114,97,109,101,32,37,112,0,0,0,116,101,115,97,0,0,0,0,105,100,99,105,110,0,0,0,83,121,109,112,104,111,110,105,99,32,82,111,99,107,0,0,102,112,115,0,0,0,0,0,169,100,101,115,0,0,0,0,73,83,79,58,32,70,105,108,101,32,84,121,112,101,32,77,97,106,111,114,32,66,114,97,110,100,58,32,37,46,52,115,10,0,0,0,0,0,0,0,84,80,50,0,0,0,0,0,114,101,99,111,114,100,32,111,114,32,116,114,97,110,115,99,111,100,101,32,115,116,111,112,32,116,105,109,101,0,0,0,105,110,118,97,108,105,100,32,110,101,119,32,98,97,99,107,115,116,101,112,32,37,100,10,0,0,0,0,0,0,0,0,100,115,112,32,105,110,105,116,32,102,97,105,108,101,100,10,0,0,0,0,0,0,0,0,72,79,77,69,0,0,0,0,108,105,98,109,112,51,108,97,109,101,32,101,110,99,111,100,101,114,0,0,0,0,0,0,65,65,67,32,108,105,98,114,97,114,121,32,105,110,105,116,105,97,108,105,122,97,116,105,111,110,32,101,114,114,111,114,0,0,0,0,0,0,0,0,101,120,116,114,97,100,97,116,97,32,99,111,110,116,97,105,110,115,32,37,100,32,98,121,116,101,115,32,116,111,111,32,109,97,110,121,46,10,0,0,48,0,0,0,0,0,0,0,121,117,118,97,52,52,52,112,57,98,101,0,0,0,0,0,76,105,110,101,110,0,0,0,115,104,105,98,97,116,97,0,102,114,97,109,101,45,62,110,98,95,101,120,116,101,110,100,101,100,95,98,117,102,32,61,61,32,48,0,0,0,0,0,101,115,97,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,40,97,108,105,97,115,32,102,111,114,32,102,117,108,108,41,0,0,77,105,99,114,111,115,111,102,116,32,86,105,100,101,111,32,49,0,0,0,0,0,0,0,80,115,121,99,104,101,100,101,108,105,99,32,82,111,99,107,0,0,0,0,0,0,0,0,44,32,83,65,82,32,37,100,58,37,100,32,68,65,82,32,37,100,58,37,100,0,0,0,101,110,99,111,100,101,114,0,104,97,110,100,108,101,114,95,110,97,109,101,0,0,0,0,84,80,49,0,0,0,0,0,72,46,50,54,51,32,47,32,72,46,50,54,51,45,49,57,57,54,44,32,72,46,50,54,51,43,32,47,32,72,46,50,54,51,45,49,57,57,56,32,47,32,72,46,50,54,51,32,118,101,114,115,105,111,110,32,50,0,0,0,0,0,0,0,116,111,0,0,0,0,0,0,77,101,116,97,108,0,0,0,99,111,109,112,111,115,101,114,0,0,0,0,0,0,0,0,70,70,77,80,69,71,95,68,65,84,65,68,73,82,0,0,87,97,114,110,105,110,103,58,32,104,105,103,104,112,97,115,115,32,102,105,108,116,101,114,32,100,105,115,97,98,108,101,100,46,32,32,104,105,103,104,112,97,115,115,32,102,114,101,113,117,101,110,99,121,32,116,111,111,32,115,109,97,108,108,10,0,0,0,0,0,0,0,83,107,105,112,112,105,110,103,32,98,105,110,97,114,121,32,116,97,103,32,39,37,115,39,46,10,0,0,0,0,0,0,116,111,111,32,115,104,111,114,116,32,76,73,83,84,32,116,97,103,10,0,0,0,0,0,65,117,100,105,111,32,102,105,108,116,101,114,32,103,114,97,112,104,32,111,117,116,112,117,116,32,105,115,32,110,111,116,32,110,111,114,109,97,108,105,122,101,100,32,97,110,100,32,101,110,99,111,100,101,114,32,100,111,101,115,32,110,111,116,32,115,117,112,112,111,114,116,32,112,97,114,97,109,101,116,101,114,32,99,104,97,110,103,101,115,10,0,0,0,0,0,121,117,118,97,52,50,50,112,57,108,101,0,0,0,0,0,76,105,109,101,71,114,101,101,110,0,0,0,0,0,0,0,70,111,114,109,97,116,32,37,115,32,100,101,116,101,99,116,101,100,32,111,110,108,121,32,119,105,116,104,32,108,111,119,32,115,99,111,114,101,32,111,102,32,37,100,44,32,109,105,115,100,101,116,101,99,116,105,111,110,32,112,111,115,115,105,98,108,101,33,10,0,0,0,115,101,108,101,99,116,32,108,105,112,115,104,105,116,122,32,110,111,105,115,101,32,115,104,97,112,105,110,103,32,100,105,116,104,101,114,0,0,0,0,103,101,110,114,101,0,0,0,118,105,100,101,111,95,103,101,116,95,98,117,102,102,101,114,58,32,105,109,97,103,101,32,112,97,114,97,109,101,116,101,114,115,32,105,110,118,97,108,105,100,10,0,0,0,0,0,101,115,97,0,0,0,0,0,105,103,110,105,100,120,0,0,37,48,50,100,58,37,48,50,100,58,37,48,50,100,46,37,48,51,100,0,0,0,0,0,109,115,118,105,100,101,111,49,0,0,0,0,0,0,0,0,80,114,111,103,114,101,115,115,105,118,101,32,82,111,99,107,0,0,0,0,0,0,0,0,69,114,114,111,114,32,119,114,105,116,105,110,103,32,83,112,101,101,120,32,104,101,97,100,101,114,115,10,0,0,0,0,102,97,105,108,101,100,32,116,111,32,99,114,101,97,116,101,32,111,114,32,114,101,112,108,97,99,101,32,115,116,114,101,97,109,10,0,0,0,0,0,58,32,37,115,0,0,0,0,110,111,32,115,116,114,101,97,109,115,10,0,0,0,0,0,99,111,109,109,101,110,116,0,169,115,119,114,0,0,0,0,117,115,101,32,116,97,98,108,101,32,111,102,32,99,111,110,116,101,110,116,115,0,0,0,108,97,110,103,117,97,103,101,0,0,0,0,0,0,0,0,84,69,78,0,0,0,0,0,100,117,114,97,116,105,111,110,0,0,0,0,0,0,0,0,69,114,114,111,114,32,119,104,105,108,101,32,114,101,97,100,105,110,103,32,102,105,108,101,32,39,37,115,39,58,32,37,115,10,0,0,0,0,0,0,69,114,114,111,114,58,32,99,97,110,39,116,32,97,108,108,111,99,97,116,101,32,105,110,95,98,117,102,102,101,114,32,98,117,102,102,101,114,10,0,101,110,99,111,100,101,100,95,98,121,0,0,0,0,0,0,69,114,114,111,114,32,105,110,32,97,118,95,98,117,102,102,101,114,115,105,110,107,95,103,101,116,95,102,114,97,109,101,95,102,108,97,103,115,40,41,58,32,37,115,10,0,0,0,121,117,118,97,52,50,50,112,57,98,101,0,0,0,0,0,69,114,114,111,114,32,112,97,114,115,105,110,103,32,111,112,116,105,111,110,115,32,115,116,114,105,110,103,32,37,115,10,0,0,0,0,0,0,0,0,76,105,109,101,0,0,0,0,97,115,102,45,62,112,97,99,107,101,116,95,115,105,122,101,95,108,101,102,116,32,60,32,70,82,65,77,69,95,72,69,65,68,69,82,95,83,73,90,69,32,124,124,32,97,115,102,45,62,112,97,99,107,101,116,95,115,101,103,109,101,110,116,115,32,60,32,49,0,0,0,108,105,112,115,104,105,116,122,0,0,0,0,0,0,0,0,111,99,104,0,0,0,0,0,115,97,109,112,108,101,95,102,109,116,115,0,0,0,0,0,101,110,100,95,112,116,115,0,69,80,90,83,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,40,100,101,102,97,117,108,116,41,0,0,0,0,0,0,0,0,80,82,69,86,95,73,78,80,84,83,0,0,0,0,0,0,77,105,115,109,97,116,99,104,101,100,32,39,91,39,32,102,111,117,110,100,32,105,110,32,116,104,101,32,102,111,108,108,111,119,105,110,103,58,32,34,37,115,34,46,10,0,0,0,77,105,99,114,111,115,111,102,116,32,82,76,69,0,0,0,71,111,116,104,105,99,32,82,111,99,107,0,0,0,0,0,102,114,97,109,101,45,62,110,98,95,115,97,109,112,108,101,115,32,62,32,111,102,102,115,101,116,0,0,0,0,0,0,44,32,37,100,44,32,37,100,47,37,100,0,0,0,0,0,116,98,58,37,100,47,37,100,32,115,97,109,112,108,101,102,109,116,58,37,115,32,115,97,109,112,108,101,114,97,116,101,58,37,100,32,99,104,108,97,121,111,117,116,58,37,115,10,0,0,0,0,0,0,0,0,169,100,97,121,0,0,0,0,86,101,114,115,105,111,110,32,37,100,0,0,0,0,0,0,66,117,102,102,101,114,32,97,117,100,105,111,32,102,114,97,109,101,115,44,32,97,110,100,32,109,97,107,101,32,116,104,101,109,32,97,118,97,105,108,97,98,108,101,32,116,111,32,116,104,101,32,101,110,100,32,111,102,32,116,104,101,32,102,105,108,116,101,114,32,103,114,97,112,104,46,0,0,0,0,84,84,50,0,0,0,0,0,67,97,110,110,111,116,32,115,101,108,101,99,116,32,115,97,109,112,108,101,32,114,97,116,101,32,102,111,114,32,116,104,101,32,108,105,110,107,32,98,101,116,119,101,101,110,32,102,105,108,116,101,114,115,32,37,115,32,97,110,100,32,37,115,46,10,0,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,99,111,110,102,105,103,117,114,101,32,105,110,112,117,116,32,112,97,100,32,111,110,32,37,115,10,0,0,0,0,114,101,99,111,114,100,32,111,114,32,116,114,97,110,115,99,111,100,101,32,34,100,117,114,97,116,105,111,110,34,32,115,101,99,111,110,100,115,32,111,102,32,97,117,100,105,111,47,118,105,100,101,111,0,0,0,67,111,117,108,100,32,110,111,116,32,97,108,108,111,99,97,116,101,32,102,105,108,101,32,98,117,102,102,101,114,10,0,102,102,109,112,101,103,46,99,0,0,0,0,0,0,0,0,121,117,118,97,52,50,48,112,57,108,101,0,0,0,0,0,76,105,103,104,116,89,101,108,108,111,119,0,0,0,0,0,115,101,108,101,99,116,32,116,114,105,97,110,103,117,108,97,114,32,100,105,116,104,101,114,32,119,105,116,104,32,104,105,103,104,32,112,97,115,115,0,101,112,122,115,0,0,0,0,109,115,114,108,101,0,0,0,65,118,97,110,116,103,97,114,100,101,0,0,0,0,0,0,40,37,115,41,0,0,0,0,169,97,108,98,0,0,0,0,99,114,101,97,116,105,111,110,95,116,105,109,101,0,0,0,84,65,76,0,0,0,0,0,116,0,0,0,0,0,0,0,86,105,100,101,111,32,111,112,116,105,111,110,115,58,0,0,73,79,32,101,114,114,111,114,58,32,37,115,10,0,0,0,111,115,116,45,62,115,111,117,114,99,101,95,105,110,100,101,120,32,62,61,32,48,0,0,121,117,118,97,52,50,48,112,57,98,101,0,0,0,0,0,76,105,103,104,116,83,116,101,101,108,66,108,117,101,0,0,116,114,105,97,110,103,117,108,97,114,95,104,112,0,0,0,85,110,97,98,108,101,32,116,111,32,114,101,99,111,100,101,32,115,117,98,116,105,116,108,101,32,101,118,101,110,116,32,34,37,115,34,32,102,114,111,109,32,37,115,32,116,111,32,85,84,70,45,56,10,0,0,102,117,108,108,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,40,115,108,111,119,101,115,116,41,0,0,0,0,0,0,0,0,87,101,115,116,119,111,111,100,32,83,116,117,100,105,111,115,32,86,81,65,32,40,86,101,99,116,111,114,32,81,117,97,110,116,105,122,101,100,32,65,110,105,109,97,116,105,111,110,41,32,118,105,100,101,111,0,66,108,117,101,103,114,97,115,115,0,0,0,0,0,0,0,91,48,120,37,120,93,0,0,169,97,117,116,0,0,0,0,37,89,45,37,109,45,37,100,32,37,72,58,37,77,58,37,83,0,0,0,0,0,0,0,32,37,46,50,115,58,37,46,50,115,0,0,0,0,0,0,105,110,112,117,116,95,102,105,108,101,95,105,110,100,101,120,0,0,0,0,0,0,0,0,67,97,110,110,111,116,32,114,101,97,100,32,102,105,108,101,32,39,37,115,39,58,32,37,115,10,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,103,98,114,112,49,54,108,101,0,0,0,0,0,0,0,0,76,105,103,104,116,83,108,97,116,101,71,114,97,121,0,0,115,101,108,101,99,116,32,116,114,105,97,110,103,117,108,97,114,32,100,105,116,104,101,114,0,0,0,0,0,0,0,0,83,117,98,116,105,116,108,101,115,32,112,97,99,107,101,116,32,105,115,32,116,111,111,32,98,105,103,32,102,111,114,32,114,101,99,111,100,105,110,103,10,0,0,0,0,0,0,0,102,117,108,108,0,0,0,0,119,115,95,118,113,97,0,0,67,101,108,116,105,99,0,0,32,32,32,32,83,116,114,101,97,109,32,35,37,100,58,37,100,0,0,0,0,0,0,0,169,110,97,109,0,0,0,0,115,97,109,112,108,101,32,97,115,112,101,99,116,32,114,97,116,105,111,32,97,108,114,101,97,100,121,32,115,101,116,32,116,111,32,37,100,58,37,100,44,32,105,103,110,111,114,105,110,103,32,39,112,97,115,112,39,32,97,116,111,109,32,40,37,100,58,37,100,41,10,0,84,73,77,0,0,0,0,0,115,101,116,32,99,104,97,112,116,101,114,115,32,109,97,112,112,105,110,103,0,0,0,0,114,98,0,0,0,0,0,0,32,0,0,0,0,0,0,0,78,111,32,109,111,114,101,32,105,110,112,117,116,115,32,116,111,32,114,101,97,100,32,102,114,111,109,44,32,102,105,110,105,115,104,105,110,103,46,10,0,0,0,0,0,0,0,0,103,98,114,112,49,54,98,101,0,0,0,0,0,0,0,0,76,105,103,104,116,83,107,121,66,108,117,101,0,0,0,0,116,114,105,97,110,103,117,108,97,114,0,0,0,0,0,0,99,100,32,33,61,32,40,105,99,111,110,118,95,116,41,45,49,0,0,0,0,0,0,0,122,101,114,111,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,40,102,97,115,116,101,115,116,41,0,0,0,0,0,0,0,0,67,105,110,101,112,97,107,0,82,101,118,105,118,97,108,0,108,97,110,103,117,97,103,101,0,0,0,0,0,0,0,0,169,65,82,84,0,0,0,0,73,110,118,97,108,105,100,32,115,97,109,112,108,101,32,102,105,101,108,100,32,115,105,122,101,32,37,100,10,0,0,0,84,73,77,69,0,0,0,0,109,97,112,95,99,104,97,112,116,101,114,115,0,0,0,0,102,105,108,116,101,114,0,0,79,117,116,112,117,116,32,102,105,108,101,32,105,115,32,101,109,112,116,121,44,32,110,111,116,104,105,110,103,32,119,97,115,32,101,110,99,111,100,101,100,32,40,99,104,101,99,107,32,45,115,115,32,47,32,45,116,32,47,32,45,102,114,97,109,101,115,32,112,97,114,97,109,101,116,101,114,115,32,105,102,32,117,115,101,100,41,10,0,0,0,0,0,0,0,0,103,98,114,112,49,48,108,101,0,0,0,0,0,0,0,0,76,105,103,104,116,83,101,97,71,114,101,101,110,0,0,0,97,116,116,97,99,104,109,101,110,116,0,0,0,0,0,0,115,101,108,101,99,116,32,114,101,99,116,97,110,103,117,108,97,114,32,100,105,116,104,101,114,0,0,0,0,0,0,0,84,105,109,101,99,111,100,101,32,102,114,97,109,101,32,114,97,116,101,32,37,100,47,37,100,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,0,0,0,0,0,0,102,102,95,97,118,99,111,100,101,99,95,108,111,99,107,101,100,0,0,0,0,0,0,0,122,101,114,111,0,0,0,0,99,105,110,101,112,97,107,0,76,97,116,105,110,0,0,0,103,114,97,121,0,0,0,0,37,99,0,0,0,0,0,0,100,97,116,101,0,0,0,0,73,110,118,97,108,105,100,32,83,97,109,112,108,101,68,101,108,116,97,32,105,110,32,83,84,84,83,32,37,100,10,0,45,37,46,50,115,45,37,46,50,115,0,0,0,0,0,0,37,89,45,37,109,45,37,100,0,0,0,0,0,0,0,0,111,117,116,102,105,108,101,91,44,109,101,116,97,100,97,116,97,93,58,105,110,102,105,108,101,91,44,109,101,116,97,100,97,116,97,93,0,0,0,0,71,117,101,115,115,101,100,32,67,104,97,110,110,101,108,32,76,97,121,111,117,116,32,102,111,114,32,32,73,110,112,117,116,32,83,116,114,101,97,109,32,35,37,100,46,37,100,32,58,32,37,115,10,0,0,0,65,86,95,76,79,71,95,70,79,82,67,69,95,67,79,76,79,82,0,0,0,0,0,0,109,117,120,101,114,0,0,0,115,105,110,0,0,0,0,0,118,105,100,101,111,58,37,49,46,48,102,107,66,32,97,117,100,105,111,58,37,49,46,48,102,107,66,32,115,117,98,116,105,116,108,101,58,37,49,46,48,102,32,103,108,111,98,97,108,32,104,101,97,100,101,114,115,58,37,49,46,48,102,107,66,32,109,117,120,105,110,103,32,111,118,101,114,104,101,97,100,32,37,102,37,37,10,0,68,69,67,79,68,69,82,95,78,79,84,95,70,79,85,78,68,0,0,0,0,0,0,0,103,98,114,112,49,48,98,101,0,0,0,0,0,0,0,0,76,105,103,104,116,83,97,108,109,111,110,0,0,0,0,0,102,114,111,110,116,32,114,105,103,104,116,0,0,0,0,0,114,101,99,116,97,110,103,117,108,97,114,0,0,0,0,0,33,102,102,95,97,118,99,111,100,101,99,95,108,111,99,107,101,100,0,0,0,0,0,0,115,101,116,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,109,101,116,104,111,100,0,0,0,0,70,111,114,99,105,110,103,32,102,117,108,108,32,105,110,116,101,114,110,97,108,32,72,32,99,104,114,111,109,97,32,100,117,101,32,116,111,32,111,100,100,32,111,117,116,112,117,116,32,115,105,122,101,10,0,0,81,117,105,99,107,84,105,109,101,32,118,105,100,101,111,32,40,82,80,90,65,41,0,0,66,101,98,111,98,0,0,0,110,101,105,103,104,98,111,114,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,121,114,114,99,0,0,0,0,50,55,48,0,0,0,0,0,84,68,65,0,0,0,0,0,116,115,102,0,0,0,0,0,115,101,116,32,109,101,116,97,100,97,116,97,32,105,110,102])
.concat([111,114,109,97,116,105,111,110,32,111,102,32,111,117,116,102,105,108,101,32,102,114,111,109,32,105,110,102,105,108,101,0,105,110,99,111,114,114,101,99,116,32,99,111,100,101,115,10,0,0,0,0,0,0,0,0,100,101,109,117,120,101,114,0,99,104,97,110,110,101,108,32,101,108,101,109,101,110,116,32,37,100,46,37,100,32,105,115,32,110,111,116,32,97,108,108,111,99,97,116,101,100,10,0,48,0,0,0,0,0,0,0,110,101,120,116,95,98,108,111,99,107,95,108,101,110,95,98,105,116,115,32,37,100,32,111,117,116,32,111,102,32,114,97,110,103,101,10,0,0,0,0,73,110,118,97,108,105,100,32,83,101,116,117,112,32,104,101,97,100,101,114,10,0,0,0,10,0,0,0,0,0,0,0,103,98,114,112,57,108,101,0,76,105,103,104,116,80,105,110,107,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,99,111,100,101,99,32,105,100,115,32,109,105,115,109,97,116,99,104,10,0,0,0,0,0,115,101,116,32,100,105,116,104,101,114,32,109,101,116,104,111,100,0,0,0,0,0,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,116,104,114,101,97,100,32,108,111,99,107,105,110,103,32,97,114,111,117,110,100,32,97,118,99,111,100,101,99,95,111,112,101,110,47,99,108,111,115,101,40,41,10,0,0,0,0,0,0,0,0,109,101,95,109,101,116,104,111,100,0,0,0,0,0,0,0,114,112,122,97,0,0,0,0,70,97,115,116,32,70,117,115,105,111,110,0,0,0,0,0,32,32,32,0,0,0,0,0,99,111,112,121,114,105,103,104,116,0,0,0,0,0,0,0,49,56,48,0,0,0,0,0,84,68,65,84,0,0,0,0,102,108,97,103,115,61,48,120,37,88,0,0,0,0,0,0,109,97,112,95,109,101,116,97,100,97,116,97,0,0,0,0,105,110,118,97,108,105,100,32,111,108,100,32,98,97,99,107,115,116,101,112,32,37,100,10,0,0,0,0,0,0,0,0,97,110,97,108,121,115,105,115,32,105,110,105,116,32,102,97,105,108,101,100,10,0,0,0,48,0,0,0,0,0,0,0,101,110,99,111,100,101,114,0,73,110,105,116,105,97,108,105,122,97,116,105,111,110,32,101,114,114,111,114,0,0,0,0,101,120,116,114,97,100,97,116,97,32,78,85,76,76,32,111,114,32,116,111,111,32,115,109,97,108,108,46,10,0,0,0,99,111,110,116,105,110,117,101,0,0,0,0,0,0,0,0,103,98,114,112,57,98,101,0,76,105,103,104,116,71,114,101,121,0,0,0,0,0,0,0,100,105,116,104,101,114,95,109,101,116,104,111,100,0,0,0,83,104,111,119,32,97,108,108,32,102,114,97,109,101,115,32,98,101,102,111,114,101,32,116,104,101,32,102,105,114,115,116,32,107,101,121,102,114,97,109,101,0,0,0,0,0,0,0,87,105,110,103,32,67,111,109,109,97,110,100,101,114,32,73,86,32,47,32,88,120,97,110,0,0,0,0,0,0,0,0,83,119,105,110,103,0,0,0,32,37,48,50,120,0,0,0,99,112,114,116,0,0,0,0,73,78,84,69,82,78,65,76,32,69,82,82,79,82,32,73,78,32,86,66,82,32,78,69,87,32,67,79,68,69,44,32,112,108,101,97,115,101,32,115,101,110,100,32,98,117,103,32,114,101,112,111,114,116,10,0,57,48,0,0,0,0,0,0,84,89,69,0,0,0,0,0,104,50,54,51,0,0,0,0,84,68,65,84,0,0,0,0,102,105,108,101,46,115,116,114,101,97,109,46,99,104,97,110,110,101,108,91,58,115,121,110,99,102,105,108,101,46,115,121,110,99,115,116,114,101,97,109,93,0,0,0,0,0,0,0,74,97,122,122,0,0,0,0,87,77,47,67,111,109,112,111,115,101,114,0,0,0,0,0,100,101,99,111,100,101,114,0,73,110,118,97,108,105,100,32,65,80,69,32,116,97,103,32,107,101,121,32,39,37,115,39,46,10,0,0,0,0,0,0,67,111,117,108,100,32,110,111,116,32,97,108,108,111,99,97,116,101,32,101,120,116,114,97,100,97,116,97,46,10,0,0,101,110,100,0,0,0,0,0,103,98,114,112,0,0,0,0,76,105,103,104,116,71,114,101,101,110,0,0,0,0,0,0,97,97,99,0,0,0,0,0,115,101,116,32,100,105,116,104,101,114,32,115,99,97,108,101,0,0,0,0,0,0,0,0,73,71,78,82,0,0,0,0,115,104,111,119,97,108,108,0,102,102,108,97,103,115,0,0,67,72,65,80,84,69,82,37,48,50,100,0,0,0,0,0,120,97,110,95,119,99,52,0,78,97,116,105,111,110,97,108,32,70,111,108,107,0,0,0,69,114,114,111,114,32,119,114,105,116,105,110,103,32,70,76,65,67,32,104,101,97,100,101,114,115,10,0,0,0,0,0,111,103,103,32,112,97,103,101,44,32,117,110,115,117,112,112,111,114,116,101,100,32,118,101,114,115,105,111,110,10,0,0,37,48,56,120,32,0,0,0,37,108,108,100,0,0,0,0,84,68,82,76,0,0,0,0,97,108,98,117,109,0,0,0,117,115,101,116,111,99,0,0,114,111,116,97,116,101,0,0,84,89,69,82,0,0,0,0,109,97,112,32,97,110,32,97,117,100,105,111,32,99,104,97,110,110,101,108,32,102,114,111,109,32,111,110,101,32,115,116,114,101,97,109,32,116,111,32,97,110,111,116,104,101,114,0,97,112,112,108,105,99,97,116,105,111,110,47,109,112,52,0,113,116,32,32,0,0,0,0,43,0,0,0,0,0,0,0,84,69,78,67,0,0,0,0,112,114,111,103,114,101,115,115,61,37,115,10,0,0,0,0,118,100,97,95,118,108,100,0,75,101,121,32,39,37,115,39,32,110,111,116,32,102,111,117,110,100,46,10,0,0,0,0,76,105,103,104,116,71,111,108,100,101,110,82,111,100,89,101,108,108,111,119,0,0,0,0,114,97,116,105,110,103,0,0,81,80,50,76,65,77,66,68,65,0,0,0,0,0,0,0,112,107,116,110,117,109,58,37,100,44,32,112,107,116,99,116,58,37,100,32,32,112,116,115,58,32,37,108,108,100,10,0,100,105,116,104,101,114,95,115,99,97,108,101,0,0,0,0,105,99,104,0,0,0,0,0,69,114,114,111,114,32,112,97,114,115,105,110,103,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,58,32,37,115,46,10,0,0,0,0,0,0,84,105,109,101,115,116,97,109,112,32,111,102,32,116,104,101,32,102,105,114,115,116,32,102,114,97,109,101,32,116,104,97,116,32,115,104,111,117,108,100,32,98,101,32,32,112,97,115,115,101,100,0,0,0,0,0,70,114,97,109,101,32,100,97,116,97,32,109,105,103,104,116,32,98,101,32,115,112,108,105,116,32,105,110,116,111,32,109,117,108,116,105,112,108,101,32,99,104,117,110,107,115,0,0,80,79,83,0,0,0,0,0,66,97,100,32,40,101,109,112,116,121,63,41,32,108,97,98,101,108,32,102,111,117,110,100,32,105,110,32,116,104,101,32,102,111,108,108,111,119,105,110,103,58,32,34,37,115,34,46,10,0,0,0,0,0,0,0,87,105,110,103,32,67,111,109,109,97,110,100,101,114,32,73,73,73,32,47,32,88,97,110,0,0,0,0,0,0,0,0,70,111,108,107,45,82,111,99,107,0,0,0,0,0,0,0,114,101,113,117,101,115,116,95,115,97,109,112,108,101,115,32,99,104,97,110,103,101,100,32,98,101,102,111,114,101,32,116,104,101,32,98,117,102,102,101,114,32,119,97,115,32,114,101,116,117,114,110,101,100,46,10,0,0,0,0,0,0,0,0,32,32,115,105,122,101,61,37,100,10,0,0,0,0,0,0,78,101,105,116,104,101,114,32,110,117,109,98,101,114,32,111,102,32,99,104,97,110,110,101,108,115,32,110,111,114,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,115,112,101,99,105,102,105,101,100,10,0,0,0,0,0,0,0,0,97,108,98,109,0,0,0,0,99,111,117,108,100,32,110,111,116,32,102,105,110,100,32,99,111,114,114,101,115,112,111,110,100,105,110,103,32,116,114,101,120,10,0,0,0,0,0,0,66,117,102,102,101,114,32,118,105,100,101,111,32,102,114,97,109,101,115,44,32,97,110,100,32,109,97,107,101,32,116,104,101,109,32,97,118,97,105,108,97,98,108,101,32,116,111,32,116,104,101,32,101,110,100,32,111,102,32,116,104,101,32,102,105,108,116,101,114,32,103,114,97,112,104,46,0,0,0,0,69,114,114,111,114,32,114,101,97,100,105,110,103,32,71,69,79,66,32,102,114,97,109,101,44,32,100,97,116,97,32,116,114,117,110,99,97,116,101,100,46,10,0,0,0,0,0,0,112,105,99,107,105,110,103,32,37,115,32,111,117,116,32,111,102,32,37,100,32,114,101,102,58,37,115,32,97,108,112,104,97,58,37,100,10,0,0,0,37,108,117,0,0,0,0,0,86,105,100,101,111,32,115,111,117,114,99,101,32,102,105,108,116,101,114,115,32,109,117,115,116,32,115,101,116,32,116,104,101,105,114,32,111,117,116,112,117,116,32,108,105,110,107,39,115,32,119,105,100,116,104,32,97,110,100,32,104,101,105,103,104,116,10,0,0,0,0,0,109,97,112,95,99,104,97,110,110,101,108,0,0,0,0,0,37,115,37,115,0,0,0,0,37,115,32,32,32,32,13,0,121,117,118,52,50,50,112,57,108,101,0,0,0,0,0,0,76,105,103,104,116,67,121,97,110,0,0,0,0,0,0,0,69,0,0,0,0,0,0,0,100,111,119,110,109,105,120,0,102,111,114,99,101,32,114,101,115,97,109,112,108,105,110,103,0,0,0,0,0,0,0,0,45,45,99,114,111,115,115,45,112,114,101,102,105,120,61,101,109,32,45,45,99,99,61,101,109,99,99,32,45,45,101,110,97,98,108,101,45,99,114,111,115,115,45,99,111,109,112,105,108,101,32,45,45,116,97,114,103,101,116,45,111,115,61,110,111,110,101,32,45,45,97,114,99,104,61,120,56,54,95,51,50,32,45,45,112,114,101,102,105,120,61,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,32,45,45,101,120,116,114,97,45,99,102,108,97,103,115,61,45,73,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,105,110,99,108,117,100,101,32,45,45,99,112,117,61,103,101,110,101,114,105,99,32,45,45,100,105,115,97,98,108,101,45,104,119,97,99,99,101,108,115,32,45,45,100,105,115,97,98,108,101,45,115,116,114,105,112,112,105,110,103,32,45,45,100,105,115,97,98,108,101,45,112,116,104,114,101,97,100,115,32,45,45,100,105,115,97,98,108,101,45,100,111,99,32,45,45,100,105,115,97,98,108,101,45,100,101,98,117,103,32,45,45,100,105,115,97,98,108,101,45,97,115,109,32,45,45,100,105,115,97,98,108,101,45,110,101,116,119,111,114,107,32,45,45,100,105,115,97,98,108,101,45,101,118,101,114,121,116,104,105,110,103,32,45,45,100,105,115,97,98,108,101,45,102,102,112,108,97,121,32,45,45,100,105,115,97,98,108,101,45,102,102,112,114,111,98,101,32,45,45,100,105,115,97,98,108,101,45,102,102,115,101,114,118,101,114,32,45,45,100,105,115,97,98,108,101,45,111,117,116,100,101,118,61,115,100,108,32,45,45,101,110,97,98,108,101,45,111,112,116,105,109,105,122,97,116,105,111,110,115,32,45,45,101,110,97,98,108,101,45,112,114,111,116,111,99,111,108,61,102,105,108,101,32,45,45,101,110,97,98,108,101,45,102,105,108,116,101,114,61,97,114,101,115,97,109,112,108,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,119,97,118,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,115,49,54,108,101,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,117,56,32,45,45,101,110,97,98,108,101,45,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,108,105,98,102,100,107,45,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,111,118,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,102,100,107,95,97,97,99,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,52,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,115,102,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,97,115,102,0,99,104,117,110,107,115,0,0,120,97,110,95,119,99,51,0,70,111,108,107,0,0,0,0,32,32,112,116,115,61,0,0,78,85,76,76,0,0,0,0,99,111,109,109,101,110,116,0,119,114,111,110,103,32,99,104,117,110,107,32,99,111,117,110,116,32,37,100,10,0,0,0,70,97,105,108,101,100,32,116,111,32,97,108,108,111,99,32,37,100,32,98,121,116,101,115,10,0,0,0,0,0,0,0,91,45,93,105,110,112,117,116,95,102,105,108,101,95,105,100,91,58,115,116,114,101,97,109,95,115,112,101,99,105,102,105,101,114,93,91,44,115,121,110,99,95,102,105,108,101,95,105,100,91,58,115,116,114,101,97,109,95,115,112,101,99,105,102,105,101,114,93,93,0,0,0,65,100,118,97,110,99,101,100,32,112,101,114,45,102,105,108,101,32,111,112,116,105,111,110,115,58,0,0,0,0,0,0,37,45,49,50,115,0,0,0,100,114,111,112,95,102,114,97,109,101,115,61,37,100,10,0,121,117,118,52,50,50,112,57,98,101,0,0,0,0,0,0,76,105,103,104,116,67,111,114,97,108,0,0,0,0,0,0,80,73,0,0,0,0,0,0,111,99,116,97,103,111,110,97,108,0,0,0,0,0,0,0,114,101,115,0,0,0,0,0,44,32,37,100,32,107,98,47,115,0,0,0,0,0,0,0,112,108,97,99,101,32,103,108,111,98,97,108,32,104,101,97,100,101,114,115,32,97,116,32,101,118,101,114,121,32,107,101,121,102,114,97,109,101,32,105,110,115,116,101,97,100,32,111,102,32,105,110,32,101,120,116,114,97,100,97,116,97,0,0,73,110,116,101,114,112,108,97,121,32,77,86,69,32,118,105,100,101,111,0,0,0,0,0,72,97,114,100,32,82,111,99,107,0,0,0,0,0,0,0,37,48,46,51,102,0,0,0,101,110,97,98,108,101,32,102,108,117,115,104,105,110,103,32,111,102,32,116,104,101,32,73,47,79,32,99,111,110,116,101,120,116,32,97,102,116,101,114,32,101,97,99,104,32,112,97,99,107,101,116,0,0,0,0,100,115,99,112,0,0,0,0,101,114,114,111,114,32,117,110,97,108,105,103,110,101,100,32,99,104,117,110,107,10,0,0,70,97,105,108,101,100,32,116,111,32,97,108,108,111,99,32,37,122,117,32,98,121,116,101,115,10,0,0,0,0,0,0,115,101,116,32,105,110,112,117,116,32,115,116,114,101,97,109,32,109,97,112,112,105,110,103,0,0,0,0,0,0,0,0,100,117,112,95,102,114,97,109,101,115,61,37,100,10,0,0,121,117,118,52,52,52,112,49,48,108,101,0,0,0,0,0,76,105,103,104,116,66,108,117,101,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,112,97,114,115,101,32,111,112,116,105,111,110,32,118,97,108,117,101,32,34,37,115,34,10,0,0,0,0,0,0,55,46,49,40,119,105,100,101,45,115,105,100,101,41,0,0,115,119,114,95,102,108,97,103,115,0,0,0,0,0,0,0,83,101,108,101,99,116,32,98,111,116,104,32,99,104,97,110,110,101,108,115,0,0,0,0,44,32,112,97,115,115,32,50,0,0,0,0,0,0,0,0,108,111,99,97,108,95,104,101,97,100,101,114,0,0,0,0,105,110,116,101,114,112,108,97,121,118,105,100,101,111,0,0,82,111,99,107,32,38,32,82,111,108,108,0,0,0,0,0,32,32,100,116,115,61,0,0,102,108,117,115,104,95,112,97,99,107,101,116,115,0,0,0,103,101,110,114,101,0,0,0,119,114,111,110,103,32,115,97,109,112,108,101,32,99,111,117,110,116,10,0,0,0,0,0,69,114,114,111,114,32,100,101,99,111,100,105,110,103,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,100,101,115,99,114,105,112,116,105,111,110,46,10,0,0,0,109,97,112,0,0,0,0,0,37,45,49,50,115,37,115,10,0,0,0,0,0,0,0,0,32,100,117,112,61,37,100,32,100,114,111,112,61,37,100,0,121,117,118,52,52,52,112,49,48,98,101,0,0,0,0,0,76,101,109,111,110,67,104,105,102,102,111,110,0,0,0,0,97,108,108,0,0,0,0,0,55,46,49,40,119,105,100,101,41,0,0,0,0,0,0,0,115,101,116,32,102,108,97,103,115,0,0,0,0,0,0,0,98,111,116,104,0,0,0,0,44,32,112,97,115,115,32,49,0,0,0,0,0,0,0,0,105,103,110,111,114,101,32,99,114,111,112,112,105,110,103,32,105,110,102,111,114,109,97,116,105,111,110,32,102,114,111,109,32,115,112,115,0,0,0,0,69,120,112,101,99,116,101,100,32,110,117,109,98,101,114,32,102,111,114,32,37,115,32,98,117,116,32,102,111,117,110,100,58,32,37,115,10,0,0,0,105,100,32,82,111,81,32,118,105,100,101,111,0,0,0,0,77,117,115,105,99,97,108,0,32,32,100,117,114,97,116,105,111,110,61,37,48,46,51,102,10,0,0,0,0,0,0,0,99,111,114,114,101,99,116,32,115,105,110,103,108,101,32,116,105,109,101,115,116,97,109,112,32,111,118,101,114,102,108,111,119,115,0,0,0,0,0,0,103,110,114,101,0,0,0,0,83,84,83,90,32,115,97,109,112,108,101,32,115,105,122,101,32,37,100,32,105,110,118,97,108,105,100,32,40,116,111,111,32,115,109,97,108,108,41,44,32,105,103,110,111,114,105,110,103,10,0,0,0,0,0,0,85,110,107,110,111,119,110,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,116,121,112,101,32,37,100,46,10,0,0,0,0,0,0,112,114,101,115,101,116,0,0,111,117,116,95,116,105,109,101,61,37,48,50,100,58,37,48,50,100,58,37,48,50,100,46,37,48,54,100,10,0,0,0,121,117,118,52,52,52,112,57,108,101,0,0,0,0,0,0,76,97,119,110,71,114,101,101,110,0,0,0,0,0,0,0,109,105,110,0,0,0,0,0,55,46,49,0,0,0,0,0,115,117,98,116,105,116,108,101,0,0,0,0,0,0,0,0,102,108,97,103,115,0,0,0,68,114,111,112,32,102,114,97,109,101,32,105,115,32,111,110,108,121,32,97,108,108,111,119,101,100,32,119,105,116,104,32,51,48,48,48,48,47,49,48,48,49,32,111,114,32,54,48,48,48,48,47,49,48,48,49,32,70,80,83,10,0,0,0,83,101,108,101,99,116,32,83,117,98,47,82,105,103,104,116,32,99,104,97,110,110,101,108,0,0,0,0,0,0,0,0,44,32,0,0,0,0,0,0,105,103,110,111,114,101,99,114,111,112,0,0,0,0,0,0,114,111,113,0,0,0,0,0,82,101,116,114,111,0,0,0,121,117,118,52,49,49,112,0,32,32,107,101,121,102,114,97,109,101,61,37,100,10,0,0,99,111,114,114,101,99,116,95,116,115,95,111,118,101,114,102,108,111,119,0,0,0,0,0,97,117,116,104,111,114,0,0,83,84,83,90,32,115,97,109,112,108,101,32,115,105,122,101,32,37,100,32,105,110,118,97,108,105,100,32,40,116,111,111,32,108,97,114,103,101,41,44,32,105,103,110,111,114,105,110,103,10,0,0,0,0,0,0,85,110,107,110,111,119,110,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,109,105,109,101,116,121,112,101,58,32,37,115,44,32,115,107,105,112,112,105,110,103,46,10,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,97,108,112,104,97,32,118,97,108,117,101,32,115,112,101,99,105,102,105,101,114,32,39,37,115,39,32,105,110,32,39,37,115,39,10,0,0,0,0,0,112,114,101,115,101,116,32,110,97,109,101,0,0,0,0,0,73,110,118,97,108,105,100,32,111,112,116,105,111,110,32,116,121,112,101,46,10,0,0,0,79,112,116,105,111,110,32,37,115,32,110,111,116,32,102,111,117,110,100,46,10,0,0,0,84,69,82,77,0,0,0,0,37,99,37,99,37,99,37,99,37,99,32,37,45,49,54,115,32,32,32,32,32,32,32,37,100,32,32,32,32,32,32,32,32,32,32,32,32,37,50,100,10,0,0,0,0,0,0,0,116,97,110,104,0,0,0,0,66,117,102,102,101,114,32,116,111,111,32,115,109,97,108,108,0,0,0,0,0,0,0,0,111,117,116,95,116,105,109,101,95,109,115,61,37,108,108,100,10,0,0,0,0,0,0,0,121,117,118,52,52,52,112,57,98,101,0,0,0,0,0,0,76,97,118,101,110,100,101,114,66,108,117,115,104,0,0,0,109,97,120,0,0,0,0,0,55,46,48,40,102,114,111,110,116,41,0,0,0,0,0,0,70,82,0,0,0,0,0,0,114,101,109,97,116,114,105,120,95,118,111,108,117,109,101,0,115,117,98,0,0,0,0,0,44,32,37,100,32,72,122,0,87,97,114,110,105,110,103,58,32,100,115,116,83,116,114,105,100,101,32,105,115,32,110,111,116,32,97,108,105,103,110,101,100,33,10,32,32,32,32,32,32,32,32,32,45,62,99,97,110,110,111,116,32,100,111,32,97,108,105,103,110,101,100,32,109,101,109,111,114,121,32,97,99,99,101,115,115,101,115,32,97,110,121,109,111,114,101,10,0,0,0,0,0,0,0,0,115,107,105,112,32,98,105,116,115,116,114,101,97,109,32,101,110,99,111,100,105,110,103,0,37,100,120,37,100,32,45,62,32,37,100,120,37,100,32,105,115,32,105,110,118,97,108,105,100,32,115,99,97,108,105,110,103,32,100,105,109,101,110,115,105,111,110,10,0,0,0,0,83,111,110,121,32,80,108,97,121,83,116,97,116,105,111,110,32,77,68,69,67,32,40,77,111,116,105,111,110,32,68,69,67,111,100,101,114,41,0,0,80,111,108,107,97,0,0,0,101,120,112,101,114,105,109,101,110,116,97,108,0,0,0,0,37,102,32,0,0,0,0,0,115,116,114,101,97,109,32,35,37,100,58,10,0,0,0,0,115,107,105,112,32,105,110,105,116,105,97,108,32,98,121,116,101,115,0,0,0,0,0,0,97,117,116,104,0,0,0,0,85,115,105,110,103,32,97,98,115,111,108,117,116,101,32,112,97,116,104,32,111,110,32,117,115,101,114,32,114,101,113,117,101,115,116,44,32,116,104,105,115,32,105,115,32,97,32,112,111,115,115,105,98,108,101,32,115,101,99,117,114,105,116,121,32,105,115,115,117,101,10,0,85,110,107,110,111,119,110,32,101,110,99,111,100,105,110,103,10,0,0,0,0,0,0,0,105,115,114,0,0,0,0,0,112,114,101,0,0,0,0,0,73,110,118,97,108,105,100,32,99,111,100,101,32,105,110,32,105,110,105,116,95,118,108,99,10,0,0,0,0,0,0,0,84,114,121,105,110,103,32,116,111,32,114,101,109,111,118,101,32,37,100,32,109,111,114,101,32,115,97,109,112,108,101,115,32,116,104,97,110,32,116,104,101,114,101,32,97,114,101,32,105,110,32,116,104,101,32,113,117,101,117,101,10,0,0,0,105,110,118,97,108,105,100,32,115,97,109,112,108,105,110,103,32,114,97,116,101,32,105,110,100,101,120,32,37,100,10,0,112,117,116,95,98,105,116,115,95,112,116,114,40,38,115,45,62,112,98,41,32,45,32,115,45,62,112,98,46,98,117,102,32,61,61,32,97,118,99,116,120,45,62,98,108,111,99,107,95,97,108,105,103,110,0,0,98,108,111,99,107,95,108,101,110,95,98,105,116,115,32,37,100,32,111,117,116,32,111,102,32,114,97,110,103,101,10,0,79,117,116,32,111,102,32,109,101,109,111,114,121,10,0,0,116,111,116,97,108,95,115,105,122,101,61,37,108,108,100,10,0,0,0,0,0,0,0,0,121,117,118,52,50,50,112,49,48,108,101,0,0,0,0,0,76,97,118,101,110,100,101,114,0,0,0,0,0,0,0,0,100,101,102,97,117,108,116,0,55,46,48,0,0,0,0,0,73,110,118,97,108,105,100,32,110,117,109,98,101,114,32,111,102,32,99,104,97,110,110,101,108,115,10,0,0,0,0,0,115,101,116,32,114,101,109,97,116,114,105,120,32,118,111,108,117,109,101,0,0,0,0,0,83,101,108,101,99,116,32,77,97,105,110,47,76,101,102,116,32,99,104,97,110,110,101,108,0,0,0,0,0,0,0,0,44,32,113,61,37,100,45,37,100,0,0,0,0,0,0,0,110,111,111,117,116,0,0,0,109,100,101,99,0,0,0,0,65,99,105,100,32,74,97,122,122,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,115,116,114,101,97,109,32,115,112,101,99,105,102,105,101,114,58,32,37,115,46,10,0,0,115,107,105,112,95,105,110,105,116,105,97,108,95,98,121,116,101,115,0,0,0,0,0,0,116,105,116,108,101,0,0,0,46,46,47,0,0,0,0,0,73,110,99,111,114,114,101,99,116,32,66,79,77,32,118,97,108,117,101,10,0,0,0,0,102,102,109,112,101,103,95,102,105,108,116,101,114,46,99,0,99,111,100,101,99,0,0,0,69,114,114,111,114,32,119,104,105,108,101,32,100,101,99,111,100,105,110,103,32,77,80,69,71,32,97,117,100,105,111,32,102,114,97,109,101,46,10,0,101,110,99,111,100,101,114,32,115,101,116,117,112,32,102,97,105,108,101,100,10,0,0,0,98,0,0,0,0,0,0,0,32,37,99,37,99,32,37,45,49,54,115,32,37,45,49,48,115,32,37,115,10,0,0,0,73,110,118,97,108,105,100,32,99,111,110,102,105,103,0,0,116,111,116,97,108,95,115,105,122,101,61,78,47,65,10,0,121,117,118,52,50,50,112,49,48,98,101,0,0,0,0,0,75,104,97,107,105,0,0,0,86,97,108,117,101,32,37,102,32,102,111,114,32,112,97,114,97,109,101,116,101,114,32,39,37,115,39,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,37,103,32,45,32,37,103,93,10,0,0,0,0,0,54,46,49,40,102,114,111,110,116,41,0,0,0,0,0,0,114,109,118,111,108,0,0,0,109,97,105,110,0,0,0,0,44,32,37,100,47,37,100,0,102,108,97,103,115,50,0,0,67,105,114,114,117,115,32,76,111,103,105,99,32,65,99,99,117,80,97,107,0,0,0,0,65,99,105,100,32,80,117,110,107,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,115,104,105,102,116,32,116,105,109,101,115,116,97,109,112,115,32,116,111,32,109,97,107,101,32,116,104,101,109,32,110,111,110,45,110,101,103,97,116,105,118,101,46,32,49,32,101,110,97,98,108,101,115,44,32,48,32,100,105,115,97,98,108,101,115,44,32,100,101,102,97,117,108,116,32,111,102,32,45,49,32,101,110,97,98,108,101,115,32,119,104,101,110,32,114,101,113,117,105,114,101,100,32,98,121,32,116,97,114,103,101,116,32,102,111,114,109,97,116,46,0,0,0,0,0,0,0,0,116,105,116,108,0,0,0,0,115,116,114,101,97,109,32,37,100,44,32,101,114,114,111,114,32,111,112,101,110,105,110,103,32,97,108,105,97,115,58,32,112,97,116,104,61,39,37,115,39,44,32,100,105,114,61,39,37,115,39,44,32,102,105,108,101,110,97,109,101,61,39,37,115,39,44,32,118,111,108,117,109,101,61,39,37,115,39,44,32,110,108,118,108,95,102,114,111,109,61,37,100,44,32,110,108,118,108,95,116,111,61,37,100,10,0,0,0,0,0,0,67,97,110,110,111,116,32,114,101,97,100,32,66,79,77,32,118,97,108,117,101,44,32,105,110,112,117,116,32,116,111,111,32,115,104,111,114,116,10,0,72,46,50,54,49,0,0,0,37,46,50,115,37,46,50,115,0,0,0,0,0,0,0,0,99,111,100,101,99,32,110,97,109,101,0,0,0,0,0,0,72,105,112,45,72,111,112,0,116,105,116,108,101,0,0,0,99,111,109,109,101,110,116,0,70,111,117,110,100,32,83,77,86,32,100,97,116,97,10,0,98,105,116,114,97,116,101,61,37,54,46,49,102,107,98,105,116,115,47,115,0,0,0,0,121,117,118,52,50,48,112,49,48,108,101,0,0,0,0,0,73,118,111,114,121,0,0,0,86,97,108,117,101,32,37,100,32,102,111,114,32,112,97,114,97,109,101,116,101,114,32,39,37,115,39,32,111,117,116,32,111,102,32,37,115,32,102,111,114,109,97,116,32,114,97,110,103,101,32,91,37,100,32,45,32,37,100,93,10,0,0,0,54,46,49,0,0,0,0,0,97,117,100,105,111,47,97,97,99,112,0,0,0,0,0,0,115,101,116,32,76,70,69,32,109,105,120,32,108,101,118,101,108,0,0,0,0,0,0,0,68,86,68,32,78,97,118,32,112,97,99,107,101,116,0,0,97,117,116,111,115,101,108,101,99,116,105,111,110,0,0,0,100,97,116,101,0,0,0,0,32,91,83,65,82,32,37,100,58,37,100,32,68,65,82,32,37,100,58,37,100,93,0,0,97,108,108,111,119,32,110,111,110,45,115,112,101,99,45,99,111,109,112,108,105,97,110,116,32,115,112,101,101,100,117,112,32,116,114,105,99,107,115,0,100,118,100,95,110,97,118,95,112,97,99,107,101,116,0,0,115,101,116,32,112,97,99,107,101,116,32,115,105,122,101,0,67,104,97,110,110,101,108,32,99,104,97,110,103,101,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,0,0,0,0,0,0,99,108,106,114,0,0,0,0,83,77,80,84,69,32,51,51,54,77,32,75,101,121,45,76,101,110,103,116,104,45,86,97,108,117,101,32,40,75,76,86,41,32,109,101,116,97,100,97,116,97,0,0,0,0,0,0,84,114,105,98,97,108,0,0,78,111,32,101,120,116,114,97,100,97,116,97,32,112,114,101,115,101,110,116,10,0,0,0,99,97,110,110,111,116,32,102,105,110,100,32,115,121,110,99,32,119,111,114,100,10,0,0,37,89,32,45,32,37,109,32,45,32,37,100,84,37,72,58,37,77,58,37,83,0,0,0,97,118,111,105,100,95,110,101,103,97,116,105,118,101,95,116,115,0,0,0,0,0,0,0,78,79,80,84,83,0,0,0,84,65,76,66,0,0,0,0,97,114,116,105,115,116,0,0,107,108,118,0,0,0,0,0,73,68,51,0,0,0,0,0,115,116,114,101,97,109,32,37,100,44,32,109,105,115,115,105,110,103,32,109,97,110,100,97,116,111,114,121,32,97,116,111,109,115,44,32,98,114,111,107,101,110,32,104,101,97,100,101,114,10,0,0,0,0,0,0,69,114,114,111,114,32,111,112,101,110,105,110,103,32,109,101,109,111,114,121,32,115,116,114,101,97,109,10,0,0,0,0,105,67,69,68,114,97,119,32,116,101,120,116,0,0,0,0,105,100,102,0,0,0,0,0,99,0,0,0,0,0,0,0,77,80,52,32,40,77,80,69,71,45,52,32,80,97,114,116,32,49,52,41,0,0,0,0,101,88,116,101,110,100,101,100,32,66,73,78,97,114,121,32,116,101,120,116,0,0,0,0,99,111,109,112,97,116,105,98,108,101,95,98,114,97,110,100,115,0,0,0,0,0,0,0,120,98,105,110,0,0,0,0,99,111,112,121,114,105,103,104,116,0,0,0,0,0,0,0,66,105,110,97,114,121,32,116,101,120,116,0,0,0,0,0,98,105,116,114,97,116,101,61,78,47,65,0,0,0,0,0,121,117,118,52,50,48,112,49,48,98,101,0,0,0,0,0,73,109,112,111,115,115,105,98,108,101,32,116,111,32,111,112,101,110,32,116,104,101,32,39,37,115,39,32,112,114,111,116,111,99,111,108,32,102,111,114,32,119,114,105,116,105,110,103,10,0,0,0,0,0,0,0,73,110,100,105,103,111,0,0,99,111,109,109,101,110,116,0,84,104,101,32,118,97,108,117,101,32,115,101,116,32,98,121,32,111,112,116,105,111,110,32,39,37,115,39,32,105,115,32,110,111,116,32,97,32,37,115,32,102,111,114,109,97,116,0,98,105,110,116,101,120,116,0,104,101,120,97,103,111,110,97,108,0,0,0,0,0,0,0,105,116,105,109,101,58,48,120,37,108,108,120,44,32,112,99,116,58,37,100,44,32,105,99,116,58,37,100,10,0,0,0,108,102,101,95,109,105,120,95,108,101,118,101,108,0,0,0,87,101,98,86,84,84,32,115,117,98,116,105,116,108,101,0,100,101,102,97,117,108,116,0,97,117,116,111,0,0,0,0,44,32,37,100,120,37,100,0,99,104,97,110,110,101,108,32,108,97,121,111,117,116,0,0,115,116,97,114,116,95,112,116,115,0,0,0,0,0,0,0,102,97,115,116,0,0,0,0,119,101,98,118,116,116,0,0,78,66,95,83,65,77,80,76,69,83,0,0,0,0,0,0,93,0,0,0,0,0,0,0,65,84,73,32,86,67,82,49,0,0,0,0,0,0,0,0,86,80,108,97,121,101,114,32,115,117,98,116,105,116,108,101,0,0,0,0,0,0,0,0,76,111,45,70,105,0,0,0,108,105,98,97,118,102,105,108,116,101,114,47,102,105,102,111,46,99,0,0,0,0,0,0,37,89,32,45,32,37,109,32,45,32,37,100,32,37,72,58,37,77,58,37,83,0,0,0,117,115,101,32,119,97,108,108,99,108,111,99,107,32,97,115,32,116,105,109,101,115,116,97,109,112,115,0,0,0,0,0,77,105,115,109,97,116,99,104,105,110,103,32,99,104,97,110,110,101,108,32,99,111,117,110,116,32,37,100,32,97,110,100,32,108,97,121,111,117,116,32,39,37,115,39,32,40,37,100,32,99,104,97,110,110,101,108,115,41,10,0,0,0,0,0,112,101,114,102,0,0,0,0,118,112,108,97,121,101,114,0,99,111,117,108,100,32,110,111,116,32,102,105,110,100,32,99,111,114,114,101,115,112,111,110,100,105,110,103,32,116,114,97,99,107,32,105,100,32,37,100,10,0,0,0,0,0,0,0,33,115,116,114,99,109,112,40,99,116,120,45,62,102,105,108,116,101,114,45,62,110,97,109,101,44,32,34,98,117,102,102,101,114,115,105,110,107,34,41,32,124,124,32,33,115,116,114,99,109,112,40,99,116,120,45,62,102,105,108,116,101,114,45,62,110,97,109,101,44,32,34,102,102,98,117,102,102,101,114,115,105,110,107,34,41,0,0,84,88,88,0,0,0,0,0,113,117,101,114,121,95,102,111,114,109,97,116,115,32,110,111,116,32,102,105,110,105,115,104,101,100,10,0,0,0,0,0,83,117,98,86,105,101,119,101,114,32,115,117,98,116,105,116,108,101,0,0,0,0,0,0,37,100,0,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,99,111,110,102,105,103,117,114,101,32,111,117,116,112,117,116,32,112,97,100,32,111,110,32,37,115,10,0,0,0,69,114,114,111,114,58,32,77,65,88,95,72,69,65,68,69,82,95,66,85,70,32,116,111,111,32,115,109,97,108,108,32,105,110,32,98,105,116,115,116,114,101,97,109,46,99,32,10,0,0,0,0,0,0,0,0,115,117,98,118,105,101,119,101,114,0,0,0,0,0,0,0,100,111,32,110,111,116,32,111,118,101,114,119,114,105,116,101,32,111,117,116,112,117,116,32,102,105,108,101,115,0,0,0,83,117,98,86,105,101,119,101,114,32,118,49,32,115,117,98,116,105,116,108,101,0,0,0,115,117,98,118,105,101,119,101,114,49,0,0,0,0,0,0,82,101,97,108,84,101,120,116,32,115,117,98,116,105,116,108,101,0,0,0,0,0,0,0,37,48,50,100,58,37,48,50,100,58,37,48,50,100,46,37,48,50,100,32,0,0,0,0,121,117,118,52,50,48,112,57,108,101,0,0,0,0,0,0,73,110,100,105,97,110,82,101,100,0,0,0,0,0,0,0,114,101,97,108,116,101,120,116,0,0,0,0,0,0,0,0,54,46,48,40,102,114,111,110,116,41,0,0,0,0,0,0,115,101,116,32,115,117,114,114,111,117,110,100,32,109,105,120,32,76,101,118,101,108,0,0,83,65,77,73,32,115,117,98,116,105,116,108,101,0,0,0,83,101,108,101,99,116,32,116,104,101,32,99,104,97,110,110,101,108,32,116,111,32,100,101,99,111,100,101,32,102,111,114,32,100,117,97,108,32,109,111,110,111,0,0,0,0,0,0,102,111,114,109,97,116,0,0,32,40,37,100,32,98,112,99,41,0,0,0,0,0,0,0,99,108,111,115,101,100,32,71,79,80,0,0,0,0,0,0,115,97,109,105,0,0,0,0,118,99,114,49,0,0,0,0,80,74,83,32,40,80,104,111,101,110,105,120,32,74,97,112,97,110,105,109,97,116,105,111,110,32,83,111,99,105,101,116,121,41,32,115,117,98,116,105,116,108,101,0,0,0,0,0,84,114,97,105,108,101,114,0,73,103,110,111,114,105,110,103,32,97,116,116,101,109,112,116,32,116,111,32,115,101,116,32,105,110,118,97,108,105,100,32,116,105,109,101,98,97,115,101,32,37,100,47,37,100,32,102,111,114,32,115,116,58,37,100,10,0,0,0,0,0,0,0,117,115,101,95,119,97,108,108,99,108,111,99,107,95,97,115,95,116,105,109,101,115,116,97,109,112,115,0,0,0,0,0,112,106,115,0,0,0,0,0,103,101,116,95,119,97,118,95,104,101,97,100,101,114,32,102,97,105,108,101,100,10,0,0,84,88,88,88,0,0,0,0,74,65,67,79,115,117,98,32,115,117,98,116,105,116,108,101,0,0,0,0,0,0,0,0,106,97,99,111,115,117,98,0,110,0,0,0,0,0,0,0,80,101,114,45,102,105,108,101,32,109,97,105,110,32,111,112,116,105,111,110,115,58,0,0,69,73,65,45,54,48,56,32,99,108,111,115,101,100,32,99,97,112,116,105,111,110,115,0,101,105,97,95,54,48,56,0,77,80,76,50,32,115,117,98,116,105,116,108,101,0,0,0,115,105,122,101,61,37,56,46,48,102,107,66,32,116,105,109,101,61,0,0,0,0,0,0,121,117,118,52,50,48,112,57,98,101,0,0,0,0,0,0,72,111,116,80,105,110,107,0,37,103,0,0,0,0,0,0,109,112,108,50,0,0,0,0,54,46,48,0,0,0,0,0,115,117,114,114,111,117,110,100,95,109,105,120,95,108,101,118,101,108,0,0,0,0,0,0,77,105,99,114,111,68,86,68,32,115,117,98,116,105,116,108,101,0,0,0,0,0,0,0,100,117,97,108,95,109,111,110,111,95,109,111,100,101,0,0,112,105,120,101,108,32,102,111,114,109,97,116,32,102,111,114,32,111,117,116,112,117,116,32,115,116,114,101,97,109,32,37,100,58,37,100,0,0,0,0,44,32,37,115,0,0,0,0,99,103,111,112,0,0,0,0,109,105,99,114,111,100,118,100,0,0,0,0,0,0,0,0,52,88,32,77,111,118,105,101,0,0,0,0,0,0,0,0,83,117,98,82,105,112,32,115,117,98,116,105,116,108,101,0,83,104,111,119,116,117,110,101,115,0,0,0,0,0,0,0,115,116,58,37,100,32,104,97,115,32,116,111,111,32,108,97,114,103,101,32,116,105,109,101,98,97,115,101,44,32,114,101,100,117,99,105,110,103,10,0,99,111,110,115,105,100,101,114,32,116,104,105,110,103,115,32,116,104,97,116,32,97,32,115,97,110,101,32,101,110,99,111,100,101,114,32,115,104,111,117,108,100,110,39,116,32,100,111,32,97,115,32,97,110,32,101,114,114,111,114,0,0,0,0,115,117,98,114,105,112,0,0,117,110,107,110,111,119,110,32,99,111,109,112,114,101,115,115,105,111,110,32,102,111,114,32,99,109,111,118,32,97,116,111,109,32,33,10,0,0,0,0,37,100,0,0,0,0,0,0,83,117,98,82,105,112,32,115,117,98,116,105,116,108,101,32,119,105,116,104,32,101,109,98,101,100,100,101,100,32,116,105,109,105,110,103,0,0,0,0,115,114,116,0,0,0,0,0,111,118,101,114,119,114,105,116,101,32,111,117,116,112,117,116,32,102,105,108,101,115,0,0,68,86,66,32,116,101,108,101,116,101,120,116,0,0,0,0,100,118,98,95,116,101,108,101,116,101,120,116,0,0,0,0,72,68,77,86,32,80,114,101,115,101,110,116,97,116,105,111,110,32,71,114,97,112,104,105,99,32,83,116,114,101,97,109,32,115,117,98,116,105,116,108,101,115,0,0,0,0,0,0,115,105,122,101,61,78,47,65,32,116,105,109,101,61,0,0,98,103,114,52,56,108,101,0,72,111,110,101,121,68,101,119,0,0,0,0,0,0,0,0,70,76,84,95,77,73,78,0,104,100,109,118,95,112,103,115,95,115,117,98,116,105,116,108,101,0,0,0,0,0,0,0,53,46,49,40,115,105,100,101,41,0,0,0,0,0,0,0,115,101,116,32,115,117,114,114,111,117,110,100,32,109,105,120,32,108,101,118,101,108,0,0,77,79,86,32,116,101,120,116,0,0,0,0,0,0,0,0,65,65,67,32,100,101,99,111,100,101,114,0,0,0,0,0,115,99,97,108,101,0,0,0,32,40,37,115,32,47,32,48,120,37,48,52,88,41,0,0,105,110,116,101,114,108,97,99,101,100,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,0,0,0,0,109,111,118,95,116,101,120,116,0,0,0,0,0,0,0,0,52,120,109,0,0,0,0,0,83,83,65,32,40,83,117,98,83,116,97,116,105,111,110,32,65,108,112,104,97,41,32,115,117,98,116,105,116,108,101,0,82,97,118,101,0,0,0,0,115,116,58,37,100,32,114,101,109,111,118,105,110,103,32,99,111,109,109,111,110,32,102,97,99,116,111,114,32,37,100,32,102,114,111,109,32,116,105,109,101,98,97,115,101,10,0,0,97,103,103,114,101,115,115,105,118,101,0,0,0,0,0,0,115,115,97,0,0,0,0,0,115,121,115,116,101,109,66,105,116,114,97,116,101,61,34,0,40,37,100,41,0,0,0,0,65,83,83,32,40,65,100,118,97,110,99,101,100,32,83,83,65,41,32,115,117,98,116,105,116,108,101,0,0,0,0,0,97,115,115,0,0,0,0,0,121,0,0,0,0,0,0,0])
.concat([88,83,85,66,0,0,0,0,32,37,45,50,48,115,32,37,115,0,0,0,0,0,0,0,100,97,116,101,0,0,0,0,120,115,117,98,0,0,0,0,114,97,119,32,85,84,70,45,56,32,116,101,120,116,0,0,115,116,114,101,97,109,95,37,100,95,37,100,95,112,115,110,114,95,97,108,108,61,37,50,46,50,102,10,0,0,0,0,98,103,114,52,56,98,101,0,71,114,101,101,110,89,101,108,108,111,119,0,0,0,0,0,70,76,84,95,77,65,88,0,116,101,120,116,0,0,0,0,53,46,49,0,0,0,0,0,115,108,101,118,0,0,0,0,68,86,66,32,115,117,98,116,105,116,108,101,115,0,0,0,115,99,97,108,101,114,32,102,111,114,32,111,117,116,112,117,116,32,115,116,114,101,97,109,32,37,100,58,37,100,0,0,32,40,37,115,41,0,0,0,105,108,109,101,0,0,0,0,100,118,98,95,115,117,98,116,105,116,108,101,0,0,0,0,70,70,109,112,101,103,32,118,105,100,101,111,32,99,111,100,101,99,32,35,49,0,0,0,68,86,68,32,115,117,98,116,105,116,108,101,115,0,0,0,80,115,121,99,104,97,100,101,108,105,99,0,0,0,0,0,99,111,110,115,105,100,101,114,32,97,108,108,32,115,112,101,99,32,110,111,110,32,99,111,109,112,108,105,97,110,99,105,101,115,32,97,115,32,101,114,114,111,114,115,0,0,0,0,100,118,100,95,115,117,98,116,105,116,108,101,0,0,0,0,99,100,101,99,0,0,0,0,84,67,79,0,0,0,0,0,83,77,86,32,40,83,101,108,101,99,116,97,98,108,101,32,77,111,100,101,32,86,111,99,111,100,101,114,41,0,0,0,69,86,82,67,32,40,69,110,104,97,110,99,101,100,32,86,97,114,105,97,98,108,101,32,82,97,116,101,32,67,111,100,101,99,41,0,0,0,0,0,102,109,116,0,0,0,0,0,101,118,114,99,0,0,0,0,83,0,0,0,0,0,0,0,84,65,75,32,40,84,111,109,39,115,32,108,111,115,115,108,101,115,115,32,65,117,100,105,111,32,75,111,109,112,114,101,115,115,111,114,41,0,0,0,116,97,107,0,0,0,0,0,42,58,37,50,46,50,102,32,0,0,0,0,0,0,0,0,103,114,97,121,56,97,0,0,71,114,101,101,110,0,0,0,73,54,52,95,77,73,78,0,82,70,67,32,51,51,56,57,32,67,111,109,102,111,114,116,32,78,111,105,115,101,0,0,52,46,49,0,0,0,0,0,100,97,116,97,0,0,0,0,99,101,110,116,101,114,95,109,105,120,95,108,101,118,101,108,0,0,0,0,0,0,0,0,84,105,109,101,99,111,100,101,32,102,114,97,109,101,32,114,97,116,101,32,109,117,115,116,32,98,101,32,115,112,101,99,105,102,105,101,100,10,0,0,99,111,109,102,111,114,116,110,111,105,115,101,0,0,0,0,37,100,58,37,100,58,48,120,37,88,0,0,0,0,0,0,117,110,107,110,111,119,110,0,72,46,50,54,51,32,97,100,118,97,110,99,101,100,32,105,110,116,114,97,32,99,111,100,105,110,103,32,47,32,77,80,69,71,45,52,32,65,67,32,112,114,101,100,105,99,116,105,111,110,0,0,0,0,0,0,79,112,117,115,32,40,79,112,117,115,32,73,110,116,101,114,97,99,116,105,118,101,32,65,117,100,105,111,32,67,111,100,101,99,41,0,0,0,0,0,102,102,118,49,0,0,0,0,111,112,117,115,0,0,0,0,78,101,119,32,87,97,118,101,0,0,0,0,0,0,0,0,121,117,118,52,49,48,112,0,37,48,42,100,0,0,0,0,99,111,109,112,108,105,97,110,116,0,0,0,0,0,0,0,65,109,97,122,105,110,103,32,83,116,117,100,105,111,32,80,97,99,107,101,100,32,65,110,105,109,97,116,105,111,110,32,70,105,108,101,32,65,117,100,105,111,0,0,0,0,0,0,37,42,88,32,37,88,32,37,88,32,37,88,0,0,0,0,69,114,114,111,114,32,114,101,97,100,105,110,103,32,102,114,97,109,101,32,37,115,44,32,115,107,105,112,112,101,100,10,0,0,0,0,0,0,0,0,112,97,102,95,97,117,100,105,111,0,0,0,0,0,0,0,67,97,110,110,111,116,32,102,105,110,100,32,99,111,108,111,114,32,39,37,115,39,10,0,83,111,110,105,99,32,108,111,115,115,108,101,115,115,0,0,102,111,114,99,101,32,102,111,114,109,97,116,0,0,0,0,85,110,97,98,108,101,32,116,111,32,112,97,114,115,101,32,111,112,116,105,111,110,32,118,97,108,117,101,32,34,37,115,34,32,97,115,32,99,111,108,111,114,10,0,0,0,0,0,115,111,110,105,99,108,115,0,65,86,95,76,79,71,95,70,79,82,67,69,95,78,79,67,79,76,79,82,0,0,0,0,73,77,71,85,84,73,76,83,0,0,0,0,0,0,0,0,76,0,0,0,0,0,0,0,83,111,110,105,99,0,0,0,99,111,115,104,0,0,0,0,115,111,110,105,99,0,0,0,66,85,70,70,69,82,95,84,79,79,95,83,77,65,76,76,0,0,0,0,0,0,0,0,115,116,114,101,97,109,95,37,100,95,37,100,95,112,115,110,114,95,37,99,61,37,50,46,50,102,10,0,0,0,0,0,98,103,114,52,52,52,98,101,0,0,0,0,0,0,0,0,71,114,97,121,0,0,0,0,73,54,52,95,77,65,88,0,87,97,118,101,32,115,121,110,116,104,101,115,105,115,32,112,115,101,117,100,111,45,99,111,100,101,99,0,0,0,0,0,53,46,48,40,115,105,100,101,41,0,0,0,0,0,0,0,102,114,111,110,116,32,108,101,102,116,0,0,0,0,0,0,115,101,116,32,99,101,110,116,101,114,32,109,105,120,32,108,101,118,101,108,0,0,0,0,119,97,118,101,115,121,110,116,104,0,0,0,0,0,0,0,37,115,58,32,37,115,0,0,98,117,102,102,101,114,115,105,110,107,0,0,0,0,0,0,108,105,98,115,119,115,99,97,108,101,47,115,119,115,99,97,108,101,95,105,110,116,101,114,110,97,108,46,104,0,0,0,97,105,99,0,0,0,0,0,76,117,99,97,115,65,114,116,115,32,86,73,77,65,32,97,117,100,105,111,0,0,0,0,69,120,97,99,116,108,121,32,111,110,101,32,115,99,97,108,101,114,32,97,108,103,111,114,105,116,104,109,32,109,117,115,116,32,98,101,32,99,104,111,115,101,110,44,32,103,111,116,32,37,88,10,0,0,0,0,65,83,85,83,32,86,50,0,118,105,109,97,0,0,0,0,67,97,98,97,114,101,116,0,98,105,99,117,98,105,99,0,79,117,116,112,117,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,39,37,115,39,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,0,0,32,32,78,111,32,80,114,111,103,114,97,109,10,0,0,0,99,111,110,115,105,100,101,114,32,116,104,105,110,103,115,32,116,104,97,116,32,118,105,111,108,97,116,101,32,116,104,101,32,115,112,101,99,32,97,110,100,32,104,97,118,101,32,110,111,116,32,98,101,101,110,32,115,101,101,110,32,105,110,32,116,104,101,32,119,105,108,100,32,97,115,32,101,114,114,111,114,115,0,0,0,0,0,0,105,76,66,67,32,40,73,110,116,101,114,110,101,116,32,76,111,119,32,66,105,116,114,97,116,101,32,67,111,100,101,99,41,0,0,0,0,0,0,0,105,84,117,110,83,77,80,66,0,0,0,0,0,0,0,0,67,72,65,80,0,0,0,0,105,108,98,99,0,0,0,0,105,115,102,0,0,0,0,0,37,115,37,48,50,100,58,37,48,50,100,58,37,48,50,100,37,99,37,48,50,100,0,0,73,65,67,32,40,73,110,100,101,111,32,65,117,100,105,111,32,67,111,100,101,114,41,0,102,0,0,0,0,0,0,0,84,111,111,32,108,111,110,103,32,86,76,67,32,40,37,100,41,32,105,110,32,105,110,105,116,95,118,108,99,10,0,0,97,102,113,45,62,114,101,109,97,105,110,105,110,103,95,115,97,109,112,108,101,115,32,61,61,32,97,102,113,45,62,114,101,109,97,105,110,105,110,103,95,100,101,108,97,121,0,0,128,0,0,0,0,0,0,0,105,97,99,0,0,0,0,0,73,0,0,0,0,0,0,0,69,114,114,111,114,32,100,101,99,111,100,105,110,103,32,65,65,67,32,102,114,97,109,101,32,104,101,97,100,101,114,46,10,0,0,0,0,0,0,0,82,101,97,108,65,117,100,105,111,32,76,111,115,115,108,101,115,115,0,0,0,0,0,0,105,62,61,48,0,0,0,0,112,114,101,118,95,98,108,111,99,107,95,108,101,110,95,98,105,116,115,32,37,100,32,111,117,116,32,111,102,32,114,97,110,103,101,10,0,0,0,0,73,110,118,97,108,105,100,32,112,97,99,107,101,116,32,115,105,103,110,97,116,117,114,101,32,105,110,32,83,101,116,117,112,32,104,101,97,100,101,114,10,0,0,0,0,0,0,0,114,97,108,102,0,0,0,0,110,97,109,101,32,32,32,100,101,112,116,104,0,0,0,0,37,99,58,37,50,46,50,102,32,0,0,0,0,0,0,0,98,103,114,52,52,52,108,101,0,0,0,0,0,0,0,0,71,111,108,100,101,110,82,111,100,0,0,0,0,0,0,0,73,78,84,95,77,73,78,0,68,105,115,99,119,111,114,108,100,32,73,73,32,66,77,86,32,97,117,100,105,111,0,0,53,46,48,0,0,0,0,0,73,110,118,97,108,105,100,32,115,97,109,112,108,101,95,115,105,122,101,10,0,0,0,0,99,108,101,118,0,0,0,0,98,109,118,95,97,117,100,105,111,0,0,0,0,0,0,0,91,37,100,93,0,0,0,0,37,100,0,0,0,0,0,0,47,100,101,118,47,117,114,97,110,100,111,109,0,0,0,0,117,115,101,32,111,110,108,121,32,98,105,116,101,120,97,99,116,32,102,117,110,99,116,105,111,110,115,32,40,101,120,99,101,112,116,32,40,73,41,68,67,84,41,0,0,0,0,0,56,83,86,88,32,102,105,98,111,110,97,99,99,105,0,0,97,115,118,50,0,0,0,0,56,115,118,120,95,102,105,98,0,0,0,0,0,0,0,0,78,97,116,105,118,101,32,65,109,101,114,105,99,97,110,0,32,32,80,114,111,103,114,97,109,32,37,100,32,37,115,10,0,0,0,0,0,0,0,0,99,97,114,101,102,117,108,0,56,83,86,88,32,101,120,112,111,110,101,110,116,105,97,108,0,0,0,0,0,0,0,0,37,115,0,0,0,0,0,0,80,73,67,0,0,0,0,0,56,115,118,120,95,101,120,112,0,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,71,46,55,50,57,0,0,0,102,108,97,103,115,0,0,0,105,110,99,111,114,114,101,99,116,32,102,114,97,109,101,32,115,105,122,101,32,45,32,109,117,108,116,105,112,108,101,32,102,114,97,109,101,115,32,105,110,32,98,117,102,102,101,114,63,10,0,0,0,0,0,0,103,55,50,57,0,0,0,0,108,105,98,97,118,99,111,100,101,99,47,108,105,98,118,111,114,98,105,115,101,110,99,46,99,0,0,0,0,0,0,0,102,114,101,101,32,102,111,114,109,97,116,32,111,117,116,112,117,116,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,0,0,0,0,0,71,46,55,50,51,46,49,0,85,110,115,117,112,112,111,114,116,101,100,32,112,97,114,97,109,101,116,101,114,0,0,0,103,55,50,51,95,49,0,0,80,83,78,82,61,0,0,0,114,103,98,52,52,52,98,101,0,0,0,0,0,0,0,0,71,111,108,100,0,0,0,0,73,78,84,95,77,65,88,0,67,111,110,115,116,114,97,105,110,101,100,32,69,110,101,114,103,121,32,76,97,112,112,101,100,32,84,114,97,110,115,102,111,114,109,32,40,67,69,76,84,41,0,0,0,0,0,0,51,46,49,0,0,0,0,0,111,117,116,95,99,104,97,110,110,101,108,95,108,97,121,111,117,116,0,0,0,0,0,0,99,101,108,116,0,0,0,0,37,99,0,0,0,0,0,0,37,115,124,0,0,0,0,0,98,105,116,101,120,97,99,116,0,0,0,0,0,0,0,0,81,68,101,115,105,103,110,32,77,117,115,105,99,0,0,0,65,83,85,83,32,86,49,0,113,100,109,99,0,0,0,0,74,117,110,103,108,101,0,0,110,97,109,101,0,0,0,0,97,98,111,114,116,32,100,101,99,111,100,105,110,103,32,111,110,32,109,105,110,111,114,32,101,114,114,111,114,32,100,101,116,101,99,116,105,111,110,0,65,65,67,32,76,65,84,77,32,40,65,100,118,97,110,99,101,100,32,65,117,100,105,111,32,67,111,100,105,110,103,32,76,65,84,77,32,115,121,110,116,97,120,41,0,0,0,0,37,100,47,37,100,0,0,0,71,69,79,66,0,0,0,0,104,50,54,49,0,0,0,0,97,97,99,95,108,97,116,109,0,0,0,0,0,0,0,0,84,89,69,82,0,0,0,0,66,105,110,107,32,65,117,100,105,111,32,40,68,67,84,41,0,0,0,0,0,0,0,0,102,111,114,99,101,32,115,112,101,99,105,102,105,99,32,99,112,117,32,102,108,97,103,115,0,0,0,0,0,0,0,0,71,114,117,110,103,101,0,0,99,111,109,109,101,110,116,0,98,105,110,107,97,117,100,105,111,95,100,99,116,0,0,0,68,101,115,99,114,105,112,116,105,111,110,0,0,0,0,0,46,0,0,0,0,0,0,0,66,105,110,107,32,65,117,100,105,111,32,40,82,68,70,84,41,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,83,77,86,32,118,101,114,115,105,111,110,32,102,111,117,110,100,10,0,0,0,0,0,0,121,117,118,52,50,48,112,0,98,105,110,107,97,117,100,105,111,95,114,100,102,116,0,0,37,88,0,0,0,0,0,0,114,103,98,52,52,52,108,101,0,0,0,0,0,0,0,0,71,104,111,115,116,87,104,105,116,101,0,0,0,0,0,0,10,0,0,0,0,0,0,0,65,116,114,97,99,32,49,32,40,65,100,97,112,116,105,118,101,32,84,82,97,110,115,102,111,114,109,32,65,99,111,117,115,116,105,99,32,67,111,100,105,110,103,41,0,0,0,0,113,117,97,100,40,115,105,100,101,41,0,0,0,0,0,0,109,105,109,101,95,116,121,112,101,0,0,0,0,0,0,0,115,101,116,32,111,117,116,112,117,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,0,0,0,0,0,0,0,97,116,114,97,99,49,0,0,73,67,82,68,0,0,0,0,117,110,107,110,111,119,110,95,99,111,100,101,99,0,0,0,69,114,114,111,114,32,99,111,110,102,105,103,117,114,105,110,103,32,116,104,101,32,37,115,32,102,105,108,116,101,114,0,112,108,97,99,101,32,103,108,111,98,97,108,32,104,101,97,100,101,114,115,32,105,110,32,101,120,116,114,97,100,97,116,97,32,105,110,115,116,101,97,100,32,111,102,32,101,118,101,114,121,32,107,101,121,102,114,97,109,101,0,0,0,0,0,77,80,69,71,45,52,32,65,117,100,105,111,32,76,111,115,115,108,101,115,115,32,67,111,100,105,110,103,32,40,65,76,83,41,0,0,0,0,0,0,112,97,99,107,101,116,115,105,122,101,0,0,0,0,0,0,1,118,111,114,98,105,115,0,97,115,118,49,0,0,0,0,109,112,52,97,108,115,0,0,80,111,112,47,70,117,110,107,0,0,0,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,99,111,100,101,99,32,105,100,32,105,110,32,115,116,114,101,97,109,32,37,100,10,0,0,0,0,0,0,80,97,103,101,32,97,116,32,37,108,108,100,32,105,115,32,109,105,115,115,105,110,103,32,103,114,97,110,117,108,101,10,0,0,0,0,0,0,0,0,32,32,32,32,0,0,0,0,101,120,112,108,111,100,101,0,112,116,115,32,40,37,115,41,32,60,32,100,116,115,32,40,37,115,41,32,105,110,32,115,116,114,101,97,109,32,37,100,10,0,0,0,0,0,0,0,84,80,69,49,0,0,0,0,84,114,117,101,72,68,0,0,67,97,110,110,111,116,32,100,101,116,101,114,109,105,110,101,32,102,105,108,101,32,115,105,122,101,44,32,115,107,105,112,112,105,110,103,32,84,79,67,32,116,97,98,108,101,46,10,0,0,0,0,0,0,0,0,37,100,0,0,0,0,0,0,71,69,79,0,0,0,0,0,68,111,110,39,116,32,116,114,121,32,116,111,32,114,101,115,121,110,99,104,114,111,110,105,122,101,32,98,121,32,108,111,111,107,105,110,103,32,102,111,114,32,97,32,99,101,114,116,97,105,110,32,111,112,116,105,111,110,97,108,32,115,116,97,114,116,32,99,111,100,101,0,116,114,117,101,104,100,0,0,86,81,70,32,84,119,105,110,86,81,0,0,0,0,0,0,99,112,117,102,108,97,103,115,0,0,0,0,0,0,0,0,109,112,52,0,0,0,0,0,116,119,105,110,118,113,0,0,83,111,114,101,110,115,111,110,32,72,50,54,51,0,0,0,77,80,49,32,40,77,80,69,71,32,97,117,100,105,111,32,108,97,121,101,114,32,49,41,0,0,0,0,0,0,0,0,84,67,79,80,0,0,0,0,109,112,49,0,0,0,0,0,76,0,0,0,0,0,0,0,108,101,110,32,62,61,32,109,97,120,95,98,117,102,102,101,114,95,115,105,122,101,0,0,100,120,118,97,50,95,118,108,100,0,0,0,0,0,0,0,73,109,112,111,115,115,105,98,108,101,32,116,111,32,111,112,101,110,32,116,104,101,32,39,37,115,39,32,112,114,111,116,111,99,111,108,32,102,111,114,32,114,101,97,100,105,110,103,10,0,0,0,0,0,0,0,71,97,105,110,115,98,111,114,111,0,0,0,0,0,0,0,99,111,112,121,114,105,103,104,116,0,0,0,0,0,0,0,82,101,97,108,65,117,100,105,111,32,83,73,80,82,32,47,32,65,67,69,76,80,46,78,69,84,0,0,0,0,0,0,34,37,115,34,0,0,0,0,113,117,97,100,0,0,0,0,83,69,69,75,84,79,58,32,37,108,108,100,10,0,0,0,105,110,95,99,104,97,110,110,101,108,95,108,97,121,111,117,116,0,0,0,0,0,0,0,115,105,112,114,0,0,0,0,115,97,109,112,108,101,95,114,97,116,101,0,0,0,0,0,67,111,100,101,99,32,48,120,37,120,32,105,115,32,110,111,116,32,105,110,32,116,104,101,32,102,117,108,108,32,108,105,115,116,46,10,0,0,0,0,69,114,114,111,114,32,112,97,114,115,105,110,103,32,115,97,109,112,108,101,32,114,97,116,101,58,32,37,115,46,10,0,115,116,97,114,116,0,0,0,84,105,109,101,115,116,97,109,112,32,105,110,32,115,101,99,111,110,100,115,32,111,102,32,116,104,101,32,102,105,114,115,116,32,102,114,97,109,101,32,116,104,97,116,32,115,104,111,117,108,100,32,98,101,32,100,114,111,112,112,101,100,32,97,103,97,105,110,0,0,0,0,103,108,111,98,97,108,95,104,101,97,100,101,114,0,0,0,65,84,83,67,32,65,47,53,50,66,32,40,65,67,45,51,44,32,69,45,65,67,45,51,41,0,0,0,0,0,0,0,86,66,82,32,109,111,100,101,32,40,49,45,53,41,0,0,78,66,95,67,79,78,83,85,77,69,68,95,83,65,77,80,76,69,83,0,0,0,0,0,67,97,110,110,111,116,32,99,114,101,97,116,101,32,116,104,101,32,108,105,110,107,32,37,115,58,37,100,32,45,62,32,37,115,58,37,100,10,0,0,84,104,101,111,114,97,0,0,101,97,99,51,0,0,0,0,67,104,114,105,115,116,105,97,110,32,82,97,112,0,0,0,102,105,102,111,45,62,114,111,111,116,46,110,101,120,116,0,101,110,100,32,37,102,10,0,100,101,116,101,99,116,32,105,109,112,114,111,112,101,114,32,98,105,116,115,116,114,101,97,109,32,108,101,110,103,116,104,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,37,115,46,10,0,0,0,0,0,83,111,110,121,32,65,84,82,65,67,51,43,0,0,0,0,85,110,107,110,111,119,110,32,99,111,118,101,114,32,116,121,112,101,58,32,48,120,37,120,46,10,0,0,0,0,0,0,108,105,98,97,118,102,105,108,116,101,114,47,98,117,102,102,101,114,115,105,110,107,46,99,0,0,0,0,0,0,0,0,99,111,109,109,101,110,116,0,110,111,95,114,101,115,121,110,99,95,115,101,97,114,99,104,0,0,0,0,0,0,0,0,108,105,98,97,118,102,105,108,116,101,114,47,97,118,102,105,108,116,101,114,103,114,97,112,104,46,99,0,0,0,0,0,97,116,114,97,99,51,112,0,105,109,97,103,101,47,103,105,102,0,0,0,0,0,0,0,83,111,117,114,99,101,32,102,105,108,116,101,114,115,32,97,110,100,32,102,105,108,116,101,114,115,32,119,105,116,104,32,109,111,114,101,32,116,104,97,110,32,111,110,101,32,105,110,112,117,116,32,109,117,115,116,32,115,101,116,32,99,111,110,102,105,103,95,112,114,111,112,115,40,41,32,99,97,108,108,98,97,99,107,115,32,111,110,32,97,108,108,32,111,117,116,112,117,116,115,10,0,0,0,32,49,37,37,32,32,98,117,103,32,105,110,32,76,65,77,69,32,101,110,99,111,100,105,110,103,32,108,105,98,114,97,114,121,0,0,0,0,0,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,65,117,100,105,111,32,76,111,115,115,108,101,115,115,0,0,0,0,98,121,116,101,115,0,0,0,119,109,97,108,111,115,115,108,101,115,115,0,0,0,0,0,69,0,0,0,0,0,0,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,65,117,100,105,111,32,57,32,80,114,111,102,101,115,115,105,111,110,97,108,0,0,0,0,0,0,119,109,97,112,114,111,0,0,102,112,115,61,37,46,49,102,10,0,0,0,0,0,0,0,118,100,112,97,117,95,109,112,101,103,52,0,0,0,0,0,70,117,99,104,115,105,97,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,65,117,100,105,111,32,86,111,105,99,101,0,0,0,0,0,0,0,37,48,108,108,88,0,0,0,37,100,58,37,100,37,99,0,52,46,48,0,0,0,0,0,99,104,97,110,110,101,108,95,108,97,121,111,117,116,0,0,119,109,97,118,111,105,99,101,0,0,0,0,0,0,0,0,110,111,110,101,0,0,0,0,100,117,114,97,116,105,111,110,0,0,0,0,0,0,0,0,102,111,114,99,101,32,108,111,119,32,100,101,108,97,121,0,83,112,101,101,120,0,0,0,118,98,114,0,0,0,0,0,116,104,101,111,114,97,0,0,115,112,101,101,120,0,0,0,84,111,112,32,52,48,0,0,115,116,97,114,116,32,37,102,44,32,0,0,0,0,0,0,98,117,102,102,101,114,0,0,102,114,101,101,0,0,0,0,77,117,115,101,112,97,99,107,32,83,86,56,0,0,0,0,37,115,45,37,115,0,0,0,65,80,73,67,0,0,0,0,97,115,102,32,100,101,109,117,120,101,114,0,0,0,0,0,109,117,115,101,112,97,99,107,56,0,0,0,0,0,0,0,78,101,108,108,121,109,111,115,101,114,32,65,115,97,111,0,115,101,116,32,109,97,120,105,109,117,109,32,115,105,122,101,32,111,102,32,97,32,115,105,110,103,108,101,32,97,108,108,111,99,97,116,101,100,32,98,108,111,99,107,0,0,0,0,65,100,118,97,110,99,101,100,32,103,108,111,98,97,108,32,111,112,116,105,111,110,115,58,0,0,0,0,0,0,0,0,110,101,108,108,121,109,111,115,101,114,0,0,0,0,0,0,68,0,0,0,0,0,0,0,77,111,110,107,101,121,39,115,32,65,117,100,105,111,0,0,97,112,101,0,0,0,0,0,102,114,97,109,101,61,37,100,10,0,0,0,0,0,0,0,121,117,118,52,52,52,112,49,54,98,101,0,0,0,0,0,70,111,114,101,115,116,71,114,101,101,110,0,0,0,0,0,86,111,120,119,97,114,101,32,82,84,50,57,32,77,101,116,97,115,111,117,110,100,0,0,32,40,100,101,102,97,117,108,116,32,0,0,0,0,0,0,51,46,48,40,98,97,99,107,41,0,0,0,0,0,0,0,115,101,116,32,105,110,112,117,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,0,0,0,0,0,0,0,0,118,111,120,119,97,114,101,0,73,110,118,97,108,105,100,32,85,84,70,45,56,32,105,110,32,100,101,99,111,100,101,100,32,115,117,98,116,105,116,108,101,115,32,116,101,120,116,59,32,109,97,121,98,101,32,109,105,115,115,105,110,103,32,45,115,117,98,95,99,104,97,114,101,110,99,32,111,112,116,105,111,110,10,0,0,0,0,0,37,115,32,102,111,114,32,111,117,116,112,117,116,32,115,116,114,101,97,109,32,37,100,58,37,100,0,0,0,0,0,0,108,111,119,95,100,101,108,97,121,0,0,0,0,0,0,0,65,116,114,97,99,32,51,32,40,65,100,97,112,116,105,118,101,32,84,82,97,110,115,102,111,114,109,32,65,99,111,117,115,116,105,99,32,67,111,100,105,110,103,32,51,41,0,0,83,116,114,101,97,109,77,117,120,67,111,110,102,105,103,32,97,110,100,32,80,67,69,32,114,101,112,101,116,105,116,105,111,110,32,112,101,114,105,111,100,32,40,105,110,32,102,114,97,109,101,115,41,0,0,0,110,111,110,101,0,0,0,0,79,110,50,32,86,80,51,0,97,116,114,97,99,51,0,0,71,97,110,103,115,116,97,0,32,32,32,32,67,104,97,112,116,101,114,32,35,37,100,46,37,100,58,32,0,0,0,0,100,101,116,101,99,116,32,98,105,116,115,116,114,101,97,109,32,115,112,101,99,105,102,105,99,97,116,105,111,110,32,100,101,118,105,97,116,105,111,110,115,0,0,0,0,0,0,0,114,101,115,101,114,118,101,100,95,109,111,111,118,95,115,105,122,101,32,105,115,32,116,111,111,32,115,109,97,108,108,44,32,110,101,101,100,101,100,32,37,108,108,100,32,97,100,100,105,116,105,111,110,97,108,10,0,0,0,0,0,0,0,0,71,83,77,32,77,105,99,114,111,115,111,102,116,32,118,97,114,105,97,110,116,0,0,0,117,110,100,0,0,0,0,0,80,117,98,108,105,115,104,101,114,47,83,116,117,100,105,111,32,108,111,103,111,116,121,112,101,0,0,0,0,0,0,0,116,111,111,32,109,97,110,121,32,115,116,114,101,97,109,115,10,0,0,0,0,0,0,0,103,115,109,95,109,115,0,0,77,76,80,32,40,77,101,114,105,100,105,97,110,32,76,111,115,115,108,101,115,115,32,80,97,99,107,105,110,103,41,0,109,97,120,95,97,108,108,111,99,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,109,108,112,0,0,0,0,0,32,37,115,37,115,32,37,45,49,53,115,32,37,115,10,0,77,117,115,101,112,97,99,107,32,83,86,55,0,0,0,0,109,117,115,101,112,97,99,107,55,0,0,0,0,0,0,0,102,114,97,109,101,61,37,53,100,32,102,112,115,61,37,51,46,42,102,32,113,61,37,51,46,49,102,32,0,0,0,0,121,117,118,52,52,52,112,49,54,108,101,0,0,0,0,0,70,108,111,114,97,108,87,104,105,116,101,0,0,0,0,0,73,77,67,32,40,73,110,116,101,108,32,77,117,115,105,99,32,67,111,100,101,114,41,0,41,0,0,0,0,0,0,0,51,46,48,0,0,0,0,0,37,115,37,115,37,115,0,0,105,110,116,101,114,110,97,108,95,115,97,109,112,108,101,95,102,109,116,0,0,0,0,0,105,109,99,0,0,0,0,0,73,110,118,97,108,105,100,32,109,101,100,105,97,32,116,121,112,101,32,102,111,114,32,115,117,98,116,105,116,108,101,115,10,0,0,0,0,0,0,0,37,115,32,102,105,108,116,101,114,32,110,111,116,32,112,114,101,115,101,110,116,44,32,99,97,110,110,111,116,32,108,105,109,105,116,32,114,101,99,111,114,100,105,110,103,32,116,105,109,101,46,10,0,0,0,0,117,115,101,32,105,110,116,101,114,108,97,99,101,100,32,68,67,84,0,0,0,0,0,0,68,101,108,112,104,105,110,101,32,83,111,102,116,119,97,114,101,32,73,110,116,101,114,110,97,116,105,111,110,97,108,32,67,73,78,32,97,117,100,105,111,0,0,0,0,0,0,0,104,101,97,100,101,114,95,112,101,114,105,111,100,0,0,0,118,112,51,0,0,0,0,0,100,115,105,99,105,110,97,117,100,105,111,0,0,0,0,0,67,117,108,116,0,0,0,0,80,105,99,116,117,114,101,32,115,105,122,101,32,37,117,120,37,117,32,105,115,32,105,110,118,97,108,105,100,10,0,0,10,0,0,0,0,0,0,0,98,105,116,115,116,114,101,97,109,0,0,0,0,0,0,0,83,116,97,114,116,105,110,103,32,115,101,99,111,110,100,32,112,97,115,115,58,32,109,111,118,105,110,103,32,104,101,97,100,101,114,32,111,110,32,116,111,112,32,111,102,32,116,104,101,32,102,105,108,101,10,0,87,97,118,80,97,99,107,0,69,114,114,111,114,32,112,97,114,115,105,110,103,32,99,111,118,101,114,32,97,114,116,46,10,0,0,0,0,0,0,0,66,97,110,100,47,97,114,116,105,115,116,32,108,111,103,111,116,121,112,101,0,0,0,0,99,111,112,121,114,105,103,104,116,0,0,0,0,0,0,0,119,97,118,112,97,99,107,0,81,67,69,76,80,32,47,32,80,117,114,101,86,111,105,99,101,0,0,0,0,0,0,0,103,101,110,101,114,97,116,101,32,97,32,114,101,112,111,114,116,0,0,0,0,0,0,0,113,99,101,108,112,0,0,0,48,48,48,0,0,0,0,0,83,109,97,99,107,101,114,32,97,117,100,105,111,0,0,0,100,117,114,97,116,105,111,110,0,0,0,0,0,0,0,0,115,109,97,99,107,97,117,100,105,111,0,0,0,0,0,0,115,116,114,101,97,109,95,37,100,95,37,100,95,113,61,37,46,49,102,10,0,0,0,0,121,117,118,52,50,50,112,49,54,98,101,0,0,0,0,0,70,105,114,101,66,114,105,99,107,0,0,0,0,0,0,0,84,84,65,32,40,84,114,117,101,32,65,117,100,105,111,41,0,0,0,0,0,0,0,0,32,116,111,32,0,0,0,0,50,46,49,0,0,0,0,0,115,101,116,32,105,110,116,101,114,110,97,108,32,115,97,109,112,108,101,32,102,111,114,109,97,116,0,0,0,0,0,0,116,116,97,0,0,0,0,0,115,107,105,112,32,37,100,47,37,100,32,115,97,109,112,108,101,115,10,0,0,0,0,0,97,116,114,105,109,0,0,0,105,108,100,99,116,0,0,0,68,83,80,32,71,114,111,117,112,32,84,114,117,101,83,112,101,101,99,104,0,0,0,0,79,117,116,112,117,116,32,76,65,84,77,47,76,79,65,83,32,101,110,99,97,112,115,117,108,97,116,101,100,32,100,97,116,97,0,0,0,0,0,0,73,110,116,101,108,32,73,110,100,101,111,32,51,0,0,0,116,114,117,101,115,112,101,101,99,104,0,0,0,0,0,0,67,111,109,101,100,121,0,0,37,100,32,107,98,47,115,0,118,101,114,105,102,121,32,101,109,98,101,100,100,101,100,32,67,82,67,115,0,0,0,0,67,111,111,107,32,47,32,67,111,111,107,101,114,32,47,32,71,101,99,107,111,32,40,82,101,97,108,65,117,100,105,111,32,71,50,41,0,0,0,0,113,117,105,99,107,116,105,109,101,95,118,101,114,115,105,111,110,0,0,0,0,0,0,0,73,108,108,117,115,116,114,97,116,105,111,110,0,0,0,0,97,117,116,104,111,114,0,0,99,111,111,107,0,0,0,0,81,68,101,115,105,103,110,32,77,117,115,105,99,32,67,111,100,101,99,32,50,0,0,0,114,101,112,111,114,116,0,0,73,110,118,97,108,105,100,32,99,104,97,114,115,32,39,37,115,39,32,97,116,32,116,104,101,32,101,110,100,32,111,102,32,101,120,112,114,101,115,115,105,111,110,32,39,37,115,39,10,0,0,0,0,0,0,0,113,100,109,50,0,0,0,0,71,83,77,0,0,0,0,0,103,115,109,0,0,0,0,0,113,61,37,50,46,49,102,32,0,0,0,0,0,0,0,0,121,117,118,52,50,50,112,49,54,108,101,0,0,0,0,0,68,111,100,103,101,114,66,108,117,101,0,0,0,0,0,0,87,101,115,116,119,111,111,100,32,65,117,100,105,111,32,40,83,78,68,49,41,0,0,0,32,40,102,114,111,109,32,0,69,114,114,111,114,32,110,117,109,98,101,114,32,37,100,32,111,99,99,117,114,114,101,100,0,0,0,0,0,0,0,0,115,116,101,114,101,111,0,0,97,117,100,105,111,0,0,0,111,117,116,95,115,97,109,112,108,101,95,102,109,116,0,0,85,110,97,98,108,101,32,116,111,32,112,97,114,115,101,32,116,105,109,101,99,111,100,101,44,32,115,121,110,116,97,120,58,32,104,104,58,109,109,58,115,115,91,58,59,46,93,102,102,10,0,0,0,0,0,0,119,101,115,116,119,111,111,100,95,115,110,100,49,0,0,0,67,111,117,108,100,32,110,111,116,32,117,112,100,97,116,101,32,116,105,109,101,115,116,97,109,112,115,32,102,111,114,32,115,107,105,112,112,101,100,32,115,97,109,112,108,101,115,46,10,0,0,0,0,0,0,0,116,114,105,109,0,0,0,0,110,111,114,109,97,108,105,122,101,32,97,100,97,112,116,105,118,101,32,113,117,97,110,116,105,122,97,116,105,111,110,0,65,76,65,67,32,40,65,112,112,108,101,32,76,111,115,115,108,101,115,115,32,65,117,100,105,111,32,67,111,100,101,99,41,0,0,0,0,0,0,0,108,97,116,109,0,0,0,0,105,110,100,101,111,51,0,0,102,108,97,103,115,0,0,0,97,108,97,99,0,0,0,0,83,111,117,116,104,101,114,110,32,82,111,99,107,0,0,0,121,117,118,52,52,52,112,0,44,32,98,105,116,114,97,116,101,58,32,0,0,0,0,0,99,114,99,99,104,101,99,107,0,0,0,0,0,0,0,0,83,104,111,114,116,101,110,0,115,101,116,32,116,105,109,101,115,99,97,108,101,32,111,102,32,97,108,108,32,118,105,100,101,111,32,116,114,97,99,107,115,0,0,0,0,0,0,0,110,0,0,0,0,0,0,0,112,114,101,109,105,101,114,101,95,118,101,114,115,105,111,110,0,0,0,0,0,0,0,0,65,32,98,114,105,103,104,116,32,99,111,108,111,117,114,101,100,32,102,105,115,104,0,0,65,115,112,101,99,116,82,97,116,105,111,89,0,0,0,0,115,104,111,114,116,101,110,0,105,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,48,120,82,82,71,71,66,66,91,65,65,93,32,99,111,108,111,114,32,115,116,114,105,110,103,58,32,39,37,115,39,10,0,0,0,0,0,0,0,0,77,80,51,111,110,77,80,52,0,0,0,0,0,0,0,0,105,110,112,117,116,32,102,105,108,101,0,0,0,0,0,0,118,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,112,97,114,115,101,32,111,112,116,105,111,110,32,118,97,108,117,101,32,34,37,115,34,32,97,115,32,100,117,114,97,116,105,111,110,10,0,0,109,112,51,111,110,52,0,0,111,117,116,112,117,116,32,102,105,108,101,0,0,0,0,0,78,79,95,67,79,76,79,82,0,0,0,0,0,0,0,0,37,115,0,0,0,0,0,0,65,68,85,32,40,65,112,112,108,105,99,97,116,105,111,110,32,68,97,116,97,32,85,110,105,116,41,32,77,80,51,32,40,77,80,69,71,32,97,117,100,105,111,32,108,97,121,101,114,32,51,41,0,0,0,0,97,118,95,105,109,97,103,101,95,103,101,116,95,108,105,110,101,115,105,122,101,32,102,97,105,108,101,100,10,0,0,0,37,115,32,105,115,32,102,114,101,101,32,115,111,102,116,119,97,114,101,59,32,121,111,117,32,99,97,110,32,114,101,100,105,115,116,114,105,98,117,116,101,32,105,116,32,97,110,100,47,111,114,10,109,111,100,105,102,121,32,105,116,32,117,110,100,101,114,32,116,104,101,32,116,101,114,109,115,32,111,102,32,116,104,101,32,71,78,85,32,76,101,115,115,101,114,32,71,101,110,101,114,97,108,32,80,117,98,108,105,99,10,76,105,99,101,110,115,101,32,97,115,32,112,117,98,108,105,115,104,101,100,32,98,121,32,116,104,101,32,70,114,101,101,32,83,111,102,116,119,97,114,101,32,70,111,117,110,100,97,116,105,111,110,59,32,101,105,116,104,101,114,10,118,101,114,115,105,111,110,32,50,46,49,32,111,102,32,116,104,101,32,76,105,99,101,110,115,101,44,32,111,114,32,40,97,116,32,121,111,117,114,32,111,112,116,105,111,110,41,32,97,110,121,32,108,97,116,101,114,32,118,101,114,115,105,111,110,46,10,10,37,115,32,105,115,32,100,105,115,116,114,105,98,117,116,101,100,32,105,110,32,116,104,101,32,104,111,112,101,32,116,104,97,116,32,105,116,32,119,105,108,108,32,98,101,32,117,115,101,102,117,108,44,10,98,117,116,32,87,73,84,72,79,85,84,32,65,78,89,32,87,65,82,82,65,78,84,89,59,32,119,105,116,104,111,117,116,32,101,118,101,110,32,116,104,101,32,105,109,112,108,105,101,100,32,119,97,114,114,97,110,116,121,32,111,102,10,77,69,82,67,72,65,78,84,65,66,73,76,73,84,89,32,111,114,32,70,73,84,78,69,83,83,32,70,79,82,32,65,32,80,65,82,84,73,67,85,76,65,82,32,80,85,82,80,79,83,69,46,32,32,83,101,101,32,116,104,101,32,71,78,85,10,76,101,115,115,101,114,32,71,101,110,101,114,97,108,32,80,117,98,108,105,99,32,76,105,99,101,110,115,101,32,102,111,114,32,109,111,114,101,32,100,101,116,97,105,108,115,46,10,10,89,111,117,32,115,104,111,117,108,100,32,104,97,118,101,32,114,101,99,101,105,118,101,100,32,97,32,99,111,112,121,32,111,102,32,116,104,101,32,71,78,85,32,76,101,115,115,101,114,32,71,101,110,101,114,97,108,32,80,117,98,108,105,99,10,76,105,99,101,110,115,101,32,97,108,111,110,103,32,119,105,116,104,32,37,115,59,32,105,102,32,110,111,116,44,32,119,114,105,116,101,32,116,111,32,116,104,101,32,70,114,101,101,32,83,111,102,116,119,97,114,101,10,70,111,117,110,100,97,116,105,111,110,44,32,73,110,99,46,44,32,53,49,32,70,114,97,110,107,108,105,110,32,83,116,114,101,101,116,44,32,70,105,102,116,104,32,70,108,111,111,114,44,32,66,111,115,116,111,110,44,32,77,65,32,48,50,49,49,48,45,49,51,48,49,32,85,83,65,10,0,0,0,0,0,0,0,0,45,116,32,105,115,32,110,111,116,32,97,110,32,105,110,112,117,116,32,111,112,116,105,111,110,44,32,107,101,101,112,105,110,103,32,105,116,32,102,111,114,32,116,104,101,32,110,101,120,116,32,111,117,116,112,117,116,59,32,99,111,110,115,105,100,101,114,32,102,105,120,105,110,103,32,121,111,117,114,32,99,111,109,109,97,110,100,32,108,105,110,101,46,10,0,0,109,112,51,97,100,117,0,0,115,105,110,104,0,0,0,0,83,117,99,99,101,115,115,102,117,108,108,121,32,111,112,101,110,101,100,32,116,104,101,32,102,105,108,101,46,10,0,0,66,85,71,50,0,0,0,0,73,110,112,117,116,32,115,116,114,101,97,109,32,35,37,100,58,37,100,32,102,114,97,109,101,32,99,104,97,110,103,101,100,32,102,114,111,109,32,114,97,116,101,58,37,100,32,102,109,116,58,37,115,32,99,104,58,37,100,32,99,104,108,58,37,115,32,116,111,32,114,97,116,101,58,37,100,32,102,109,116,58,37,115,32,99,104,58,37,100,32,99,104,108,58,37,115,10,0,0,0,0,0,0,70,76,65,67,32,40,70,114,101,101,32,76,111,115,115,108,101,115,115,32,65,117,100,105,111,32,67,111,100,101,99,41,0,0,0,0,0,0,0,0,121,117,118,52,50,48,112,49,54,98,101,0,0,0,0,0,68,105,109,71,114,97,121,0,69,114,114,111,114,32,111,112,101,110,105,110,103,32,37,115,32,102,105,108,101,32,37,115,46,10,0,0,0,0,0,0,32,37,115,0,0,0,0,0,109,111,110,111,0,0,0,0,70,76,0,0,0,0,0,0,102,108,97,99,0,0,0,0,115,101,116,32,111,117,116,112,117,116,32,115,97,109,112,108,101,32,102,111,114,109,97,116,0,0,0,0,0,0,0,0,79,112,101,110,105,110,103,32,97,110,32,37,115,32,102,105,108,101,58,32,37,115,46,10,0,0,0,0,0,0,0,0,115,107,105,112,32,119,104,111,108,101,32,102,114,97,109,101,44,32,115,107,105,112,32,108,101,102,116,58,32,37,100,10,0,0,0,0,0,0,0,0,97,112,97,100,0,0,0,0,83,105,101,114,114,97,32,86,77,68,32,97,117,100,105,111,0,0,0,0,0,0,0,0,100,101,115,99,0,0,0,0,110,97,113,0,0,0,0,0,69,114,114,111,114,32,112,97,114,115,105,110,103,32,111,112,116,105,111,110,115,32,102,111,114,32,37,115,32,102,105,108,101,32,37,115,46,10,0,0,69,120,112,108,105,99,105,116,32,104,105,101,114,97,114,99,104,105,99,97,108,32,115,105,103,110,97,108,105,110,103,0,37,115,0,0,0,0,0,0,37,115,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,97,115,32,111,117,116,112,117,116,32,112,105,120,101,108,32,102,111,114,109,97,116,10,0,0,0,0,0,118,109,100,97,117,100,105,111,0,0,0,0,0,0,0,0,72,46,50,54,52,32,47,32,65,86,67,32,47,32,77,80,69,71,45,52,32,65,86,67,32,47,32,77,80,69,71,45,52,32,112,97,114,116,32,49,48,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,99,97,110,118,97,115,32,115,105,122,101,58,32,37,115,46,10,0,0,0,0,0,0,0,68,114,101,97,109,0,0,0,98,105,108,105,110,101,97,114,0,0,0,0,0,0,0,0,73,110,112,117,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,39,37,115,39,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,0,0,0,37,100,46,37,48,54,100,0,33,115,45,62,112,114,101,111,117,116,46,99,111,117,110,116,0,0,0,0,0,0,0,0,115,101,116,32,101,114,114,111,114,32,100,101,116,101,99,116,105,111,110,32,102,108,97,103,115,0,0,0,0,0,0,0,77,65,67,69,32,40,77,97,99,105,110,116,111,115,104,32,65,117,100,105,111,32,67,111,109,112,114,101,115,115,105,111,110,47,69,120,112,97,110,115,105,111,110,41,32,54,58,49,0,0,0,0,0,0,0,0,118,105,100,101,111,95,116,114,97,99,107,95,116,105,109,101,115,99,97,108,101,0,0,0,69,114,114,111,114,32,112,97,114,115,105,110,103,32,102,114,97,109,101,114,97,116,101,32,37,115,46,10,0,0,0,0,103,97,112,108,101,115,115,95,112,108,97,121,98,97,99,107,0,0,0,0,0,0,0,0,77,111,118,105,101,47,118,105,100,101,111,32,115,99,114,101,101,110,32,99,97,112,116,117,114,101,0,0,0,0,0,0,80,111,111,108,32,114,97,99,101,32,100,101,99,116,101,99,116,101,100,44,32,115,112,105,110,105,110,103,32,116,111,32,97,118,111,105,100,32,111,118,101,114,97,108,108,111,99,97,116,105,111,110,32,97,110,100,32,101,118,101,110,116,117,97,108,32,79,79,77,10,0,0,65,115,112,101,99,116,82,97,116,105,111,88,0,0,0,0,109,97,99,101,54,0,0,0,67,111,117,108,100,32,110,111,116,32,111,112,101,110,32,102,105,108,101,32,37,115,32,102,111,114,32,119,114,105,116,105,110,103,46,10,0,0,0,0,105,99,108,0,0,0,0,0,69,120,112,101,99,116,101,100,32,116,111,32,114,101,97,100])
.concat([32,37,100,32,80,83,32,98,105,116,115,32,97,99,116,117,97,108,108,121,32,114,101,97,100,32,37,100,46,10,0,0,77,65,67,69,32,40,77,97,99,105,110,116,111,115,104,32,65,117,100,105,111,32,67,111,109,112,114,101,115,115,105,111,110,47,69,120,112,97,110,115,105,111,110,41,32,51,58,49,0,0,0,0,0,0,0,0,78,111,32,102,105,108,101,110,97,109,101,32,115,112,101,99,105,102,105,101,100,32,97,110,100,32,110,111,32,39,102,105,108,101,110,97,109,101,39,32,116,97,103,105,110,32,115,116,114,101,97,109,32,35,37,100,58,37,100,46,10,0,0,0,115,101,116,32,108,111,103,103,105,110,103,32,108,101,118,101,108,0,0,0,0,0,0,0,115,121,109,98,111,108,115,95,115,105,122,101,32,60,61,32,50,32,124,124,32,33,115,121,109,98,111,108,115,0,0,0,108,105,98,97,118,99,111,100,101,99,47,97,117,100,105,111,95,102,114,97,109,101,95,113,117,101,117,101,46,99,0,0,109,97,99,101,51,0,0,0,78,111,32,101,120,116,114,97,100,97,116,97,32,116,111,32,100,117,109,112,32,105,110,32,115,116,114,101,97,109,32,35,37,100,58,37,100,46,10,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,65,117,100,105,111,32,50,0,0,0,118,101,114,115,105,111,110,0,67,111,100,101,99,32,65,86,79,112,116,105,111,110,32,37,115,32,40,37,115,41,32,115,112,101,99,105,102,105,101,100,32,102,111,114,32,105,110,112,117,116,32,102,105,108,101,32,35,37,100,32,40,37,115,41,32,104,97,115,32,110,111,116,32,98,101,101,110,32,117,115,101,100,32,102,111,114,32,97,110,121,32,115,116,114,101,97,109,46,32,84,104,101,32,109,111,115,116,32,108,105,107,101,108,121,32,114,101,97,115,111,110,32,105,115,32,101,105,116,104,101,114,32,119,114,111,110,103,32,116,121,112,101,32,40,101,46,103,46,32,97,32,118,105,100,101,111,32,111,112,116,105,111,110,32,119,105,116,104,32,110,111,32,118,105,100,101,111,32,115,116,114,101,97,109,115,41,32,111,114,32,116,104,97,116,32,105,116,32,105,115,32,97,32,112,114,105,118,97,116,101,32,111,112,116,105,111,110,32,111,102,32,115,111,109,101,32,100,101,99,111,100,101,114,32,119,104,105,99,104,32,119,97,115,32,110,111,116,32,97,99,116,117,97,108,108,121,32,117,115,101,100,32,102,111,114,32,97,110,121,32,115,116,114,101,97,109,46,10,0,0,108,105,98,97,118,99,111,100,101,99,47,119,109,97,101,110,99,46,99,0,0,0,0,0,108,101,110,32,37,100,32,105,110,118,97,108,105,100,10,0,118,111,114,98,105,115,0,0,78,85,76,76,0,0,0,0,119,109,97,118,50,0,0,0,67,111,100,101,99,32,65,86,79,112,116,105,111,110,32,37,115,32,40,37,115,41,32,115,112,101,99,105,102,105,101,100,32,102,111,114,32,105,110,112,117,116,32,102,105,108,101,32,35,37,100,32,40,37,115,41,32,105,115,32,110,111,116,32,97,32,100,101,99,111,100,105,110,103,32,111,112,116,105,111,110,46,10,0,0,0,0,0,114,101,102,99,111,117,110,116,101,100,95,102,114,97,109,101,115,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,102,105,110,100,32,100,101,102,97,117,108,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,102,111,114,32,73,110,112,117,116,32,83,116,114,101,97,109,32,35,37,100,46,37,100,10,0,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,65,117,100,105,111,32,49,0,0,0,121,117,118,52,50,48,112,49,54,108,101,0,0,0,0,0,68,101,101,112,83,107,121,66,108,117,101,0,0,0,0,0,37,115,58,32,99,111,117,108,100,32,110,111,116,32,115,101,101,107,32,116,111,32,112,111,115,105,116,105,111,110,32,37,48,46,51,102,10,0,0,0,37,99,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,69,118,97,108,0,0,0,0,103,101,116,95,98,117,102,102,101,114,40,41,32,102,97,105,108,101,100,10,0,0,0,0,108,111,119,32,102,114,101,113,117,101,110,99,121,32,50,0,112,114,101,95,100,101,99,111,100,101,114,0,0,0,0,0,119,109,97,118,49,0,0,0,80,67,77,32,68,86,68,32,117,110,115,117,112,112,111,114,116,101,100,32,115,97,109,112,108,101,32,100,101,112,116,104,32,37,105,10,0,0,0,0,105,110,95,115,97,109,112,108,101,95,102,109,116,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,37,115,58,32,99,111,117,108,100,32,110,111,116,32,102,105,110,100,32,99,111,100,101,99,32,112,97,114,97,109,101,116,101,114,115,10,0,0,0,0,115,107,105,112,32,37,100,32,115,97,109,112,108,101,115,32,100,117,101,32,116,111,32,115,105,100,101,32,100,97,116,97,10,0,0,0,0,0,0,0,45,97,112,97,100,32,105,115,32,102,111,114,119,97,114,100,101,100,32,116,111,32,108,97,118,102,105,32,115,105,109,105,108,97,114,108,121,32,116,111,32,45,97,102,32,97,112,97,100,61,37,115,46,10,0,0,100,111,95,110,111,116,104,105,110,103,0,0,0,0,0,0,100,118,97,117,100,105,111,0,116,114,117,110,99,97,116,101,100,0,0,0,0,0,0,0,97,0,0,0,0,0,0,0,101,120,112,108,105,99,105,116,95,104,105,101,114,97,114,99,104,105,99,97,108,0,0,0,115,101,116,32,105,110,112,117,116,32,116,101,120,116,32,115,117,98,116,105,116,108,101,115,32,99,104,97,114,97,99,116,101,114,32,101,110,99,111,100,105,110,103,32,109,111,100,101,0,0,0,0,0,0,0,0,86,111,114,98,105,115,0,0,104,50,54,52,0,0,0,0,112,105,120,101,108,95,102,111,114,109,97,116,0,0,0,0,69,117,114,111,100,97,110,99,101,0,0,0,0,0,0,0,44,32,115,116,97,114,116,58,32,0,0,0,0,0,0,0,115,117,98,95,99,104,97,114,101,110,99,95,109,111,100,101,0,0,0,0,0,0,0,0,101,114,114,95,100,101,116,101,99,116,0,0,0,0,0,0,118,111,114,98,105,115,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,117,115,101,32,101,100,105,116,32,108,105,115,116,0,0,0,118,105,100,101,111,95,115,105,122,101,0,0,0,0,0,0,104,100,95,118,105,100,101,111,0,0,0,0,0,0,0,0,68,117,114,105,110,103,32,112,101,114,102,111,114,109,97,110,99,101,0,0,0,0,0,0,80,97,121,108,111,97,100,32,101,120,116,101,110,115,105,111,110,32,37,120,32,37,100,10,0,0,0,0,0,0,0,0,115,101,116,32,105,110,112,117,116,32,116,101,120,116,32,115,117,98,116,105,116,108,101,115,32,99,104,97,114,97,99,116,101,114,32,101,110,99,111,100,105,110,103,0,0,0,0,0,68,67,65,32,40,68,84,83,32,67,111,104,101,114,101,110,116,32,65,99,111,117,115,116,105,99,115,41,0,0,0,0,102,114,97,109,101,114,97,116,101,0,0,0,0,0,0,0,115,117,98,95,99,104,97,114,101,110,99,0,0,0,0,0,78,111,32,97,99,99,101,108,101,114,97,116,101,100,32,99,111,108,111,114,115,112,97,99,101,32,99,111,110,118,101,114,115,105,111,110,32,102,111,117,110,100,32,102,114,111,109,32,37,115,32,116,111,32,37,115,46,10,0,0,0,0,0,0,100,116,115,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,115,97,109,112,108,101,95,114,97,116,101,0,0,0,0,0,108,111,103,108,101,118,101,108,0,0,0,0,0,0,0,0,112,107,116,95,116,105,109,101,98,97,115,101,0,0,0,0,65,84,83,67,32,65,47,53,50,65,32,40,65,67,45,51,41,0,0,0,0,0,0,0,105,110,99,111,109,112,108,101,116,101,32,102,114,97,109,101,10,0,0,0,0,0,0,0,47,100,101,118,47,115,116,100,105,110,0,0,0,0,0,0,33,115,45,62,97,102,113,46,114,101,109,97,105,110,105,110,103,95,100,101,108,97,121,0,101,114,114,111,114,32,114,101,97,108,108,111,99,97,116,105,110,103,32,111,117,116,112,117,116,32,98,117,102,102,101,114,10,0,0,0,0,0,0,0,115,97,109,112,108,101,32,102,111,114,109,97,116,32,97,117,100,105,111,32,100,101,99,111,100,101,114,115,32,115,104,111,117,108,100,32,112,114,101,102,101,114,0,0,0,0,0,0,97,99,51,0,0,0,0,0,37,115,58,32,37,115,10,0,85,110,107,110,111,119,110,32,105,110,112,117,116,32,102,111,114,109,97,116,58,32,39,37,115,39,10,0,0,0,0,0,77,101,109,111,114,121,32,97,108,108,111,99,97,116,105,111,110,32,101,114,114,111,114,0,114,101,113,117,101,115,116,95,115,97,109,112,108,101,95,102,109,116,0,0,0,0,0,0,68,105,109,101,110,115,105,111,110,115,32,109,117,115,116,32,98,101,32,97,32,105,110,116,101,103,101,114,32,109,117,108,116,105,112,108,121,32,111,102,32,116,104,101,32,99,104,114,111,109,97,32,115,117,98,115,97,109,112,108,105,110,103,10,0,0,0,0,0,0,0,0,65,65,67,32,40,65,100,118,97,110,99,101,100,32,65,117,100,105,111,32,67,111,100,105,110,103,41,0,0,0,0,0,69,114,114,111,114,32,99,111,110,102,105,103,117,114,105,110,103,32,102,105,108,116,101,114,46,10,0,0,0,0,0,0,75,97,114,97,111,107,101,0,83,97,109,112,108,101,32,114,97,116,101,32,37,100,32,105,110,118,97,108,105,100,10,0,97,97,99,0,0,0,0,0,118,97,97,112,105,95,118,108,100,0,0,0,0,0,0,0,68,101,101,112,80,105,110,107,0,0,0,0,0,0,0,0,83,116,114,101,97,109,99,111,112,121,32,114,101,113,117,101,115,116,101,100,32,102,111,114,32,111,117,116,112,117,116,32,115,116,114,101,97,109,32,37,100,58,37,100,44,32,119,104,105,99,104,32,105,115,32,102,101,100,32,102,114,111,109,32,97,32,99,111,109,112,108,101,120,32,102,105,108,116,101,114,103,114,97,112,104,46,32,70,105,108,116,101,114,105,110,103,32,97,110,100,32,115,116,114,101,97,109,99,111,112,121,32,99,97,110,110,111,116,32,98,101,32,117,115,101,100,32,116,111,103,101,116,104,101,114,46,10,0,0,0,0,0,0,0,60,99,111,108,111,114,62,0,80,72,73,0,0,0,0,0,76,70,69,50,0,0,0,0,107,97,0,0,0,0,0,0,77,80,51,32,40,77,80,69,71,32,97,117,100,105,111,32,108,97,121,101,114,32,51,41,0,0,0,0,0,0,0,0,115,101,116,32,105,110,112,117,116,32,115,97,109,112,108,101,32,102,111,114,109,97,116,0,79,110,108,121,32,118,105,100,101,111,32,97,110,100,32,97,117,100,105,111,32,102,105,108,116,101,114,115,32,97,114,101,32,115,117,112,112,111,114,116,101,100,32,99,117,114,114,101,110,116,108,121,46,10,0,0,73,110,118,97,108,105,100,32,109,101,100,105,97,32,116,121,112,101,32,102,111,114,32,97,117,100,105,111,10,0,0,0,97,102,111,114,109,97,116,0,86,111,105,99,101,32,79,118,101,114,0,0,0,0,0,0,109,112,51,0,0,0,0,0,101,114,114,111,114,91,63,93,32,118,97,114,105,97,98,108,101,115,32,119,105,108,108,32,98,101,32,115,101,116,32,100,117,114,105,110,103,32,101,110,99,111,100,105,110,103,0,0,45,45,99,114,111,115,115,45,112,114,101,102,105,120,61,101,109,32,45,45,99,99,61,101,109,99,99,32,45,45,101,110,97,98,108,101,45,99,114,111,115,115,45,99,111,109,112,105,108,101,32,45,45,116,97,114,103,101,116,45,111,115,61,110,111,110,101,32,45,45,97,114,99,104,61,120,56,54,95,51,50,32,45,45,112,114,101,102,105,120,61,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,32,45,45,101,120,116,114,97,45,99,102,108,97,103,115,61,45,73,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,105,110,99,108,117,100,101,32,45,45,99,112,117,61,103,101,110,101,114,105,99,32,45,45,100,105,115,97,98,108,101,45,104,119,97,99,99,101,108,115,32,45,45,100,105,115,97,98,108,101,45,115,116,114,105,112,112,105,110,103,32,45,45,100,105,115,97,98,108,101,45,112,116,104,114,101,97,100,115,32,45,45,100,105,115,97,98,108,101,45,100,111,99,32,45,45,100,105,115,97,98,108,101,45,100,101,98,117,103,32,45,45,100,105,115,97,98,108,101,45,97,115,109,32,45,45,100,105,115,97,98,108,101,45,110,101,116,119,111,114,107,32,45,45,100,105,115,97,98,108,101,45,101,118,101,114,121,116,104,105,110,103,32,45,45,100,105,115,97,98,108,101,45,102,102,112,108,97,121,32,45,45,100,105,115,97,98,108,101,45,102,102,112,114,111,98,101,32,45,45,100,105,115,97,98,108,101,45,102,102,115,101,114,118,101,114,32,45,45,100,105,115,97,98,108,101,45,111,117,116,100,101,118,61,115,100,108,32,45,45,101,110,97,98,108,101,45,111,112,116,105,109,105,122,97,116,105,111,110,115,32,45,45,101,110,97,98,108,101,45,112,114,111,116,111,99,111,108,61,102,105,108,101,32,45,45,101,110,97,98,108,101,45,102,105,108,116,101,114,61,97,114,101,115,97,109,112,108,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,119,97,118,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,115,49,54,108,101,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,117,56,32,45,45,101,110,97,98,108,101,45,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,108,105,98,102,100,107,45,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,111,118,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,102,100,107,95,97,97,99,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,52,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,115,102,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,97,115,102,0,83,121,110,116,97,120,32,101,114,114,111,114,32,105,110,32,109,97,116,114,105,120,32,34,37,115,34,32,97,116,32,99,111,101,102,102,32,37,100,10,0,0,0,0,0,0,0,0,69,120,112,108,105,99,105,116,32,83,66,82,44,32,105,109,112,108,105,99,105,116,32,80,83,32,115,105,103,110,97,108,105,110,103,0,0,0,0,0,118,111,0,0,0,0,0,0,77,80,50,32,40,77,80,69,71,32,97,117,100,105,111,32,108,97,121,101,114,32,50,41,0,0,0,0,0,0,0,0,67,114,101,97,116,105,118,101,32,89,85,86,32,40,67,89,85,86,41,0,0,0,0,0,43,112,97,115,115,50,0,0,80,111,112,45,70,111,108,107,0,0,0,0,0,0,0,0,78,47,65,0,0,0,0,0,69,109,101,114,103,101,110,99,121,0,0,0,0,0,0,0,115,101,116,32,101,114,114,111,114,32,100,101,116,101,99,116,105,111,110,32,102,108,97,103,115,32,40,100,101,112,114,101,99,97,116,101,100,59,32,117,115,101,32,101,114,114,95,100,101,116,101,99,116,44,32,115,97,118,101,32,118,105,97,32,97,118,99,111,110,118,41,0,109,112,50,0,0,0,0,0,117,115,101,95,101,100,105,116,108,105,115,116,0,0,0,0,43,112,97,115,115,49,0,0,109,101,100,105,97,95,116,121,112,101,0,0,0,0,0,0,68,117,114,105,110,103,32,114,101,99,111,114,100,105,110,103,0,0,0,0,0,0,0,0,77,80,69,71,45,49,47,50,32,118,105,100,101,111,32,88,118,77,67,32,40,88,45,86,105,100,101,111,32,77,111,116,105,111,110,32,67,111,109,112,101,110,115,97,116,105,111,110,41,0,0,0,0,0,0,0,99,111,109,109,101,110,116,0,101,109,0,0,0,0,0,0,48,0,0,0,0,0,0,0,68,80,67,77,32,83,111,108,0,0,0,0,0,0,0,0,112,111,115,0,0,0,0,0,101,114,114,111,114,32,112,97,114,115,105,110,103,32,114,99,95,111,118,101,114,114,105,100,101,10,0,0,0,0,0,0,100,97,116,101,0,0,0,0,67,111,109,109,101,110,116,97,114,121,0,0,0,0,0,0,115,111,108,95,100,112,99,109,0,0,0,0,0,0,0,0,37,100,44,37,100,44,37,100,0,0,0,0,0,0,0,0,115,104,111,119,32,97,118,97,105,108,97,98,108,101,32,97,117,100,105,111,32,115,97,109,112,108,101,32,102,111,114,109,97,116,115,0,0,0,0,0,99,111,0,0,0,0,0,0,68,80,67,77,32,88,97,110,0,0,0,0,0,0,0,0,70,117,110,107,0,0,0,0,69,114,114,111,114,32,114,101,97,100,105,110,103,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,100,97,116,97,46,10,0,0,0,67,111,117,108,100,32,110,111,116,32,97,108,108,111,99,97,116,101,32,109,101,109,111,114,121,32,102,111,114,32,105,110,116,101,114,32,109,97,116,114,105,120,46,10,0,0,0,0,97,114,116,105,115,116,0,0,68,105,97,108,111,103,117,101,0,0,0,0,0,0,0,0,83,87,83,99,97,108,101,114,0,0,0,0,0,0,0,0,120,97,110,95,100,112,99,109,0,0,0,0,0,0,0,0,115,101,116,114,108,105,109,105,116,0,0,0,0,0,0,0,65,80,69,32,84,97,103,32,105,115,32,97,32,104,101,97,100,101,114,10,0,0,0,0,67,111,117,108,100,32,110,111,116,32,97,108,108,111,99,97,116,101,32,109,101,109,111,114,121,32,102,111,114,32,105,110,116,114,97,32,109,97,116,114,105,120,46,10,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,102,111,117,110,100,32,110,111,32,39,102,109,116,32,39,32,116,97,103,32,98,101,102,111,114,101,32,116,104,101,32,39,83,77,86,48,39,32,116,97,103,10,0,0,0,0,0,0,100,105,115,99,0,0,0,0,100,105,0,0,0,0,0,0,68,80,67,77,32,73,110,116,101,114,112,108,97,121,0,0,85,110,107,110,111,119,110,32,112,105,120,101,108,32,102,111,114,109,97,116,32,114,101,113,117,101,115,116,101,100,58,32,37,115,46,10,0,0,0,0,72,101,97,114,105,110,103,32,73,109,112,97,105,114,101,100,0,0,0,0,0,0,0,0,100,101,99,111,100,101,95,97,117,100,105,111,32,37,100,46,37,100,0,0,0,0,0,0,105,110,116,101,114,112,108,97,121,95,100,112,99,109,0,0,118,97,97,112,105,95,105,100,99,116,0,0,0,0,0,0,68,97,114,107,86,105,111,108,101,116,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,73,110,118,97,108,105,100,32,97,115,112,101,99,116,32,114,97,116,105,111,58,32,37,115,10,0,0,0,0,0,0,0,60,100,117,114,97,116,105,111,110,62,0,0,0,0,0,0,80,73,0,0,0,0,0,0,115,117,114,114,111,117,110,100,32,100,105,114,101,99,116,32,114,105,103,104,116,0,0,0,104,105,0,0,0,0,0,0,68,80,67,77,32,105,100,32,82,111,81,0,0,0,0,0,115,119,115,99,97,108,101,114,0,0,0,0,0,0,0,0,83,112,101,99,105,102,105,101,100,32,112,114,111,98,101,32,115,105,122,101,32,118,97,108,117,101,32,37,117,32,99,97,110,110,111,116,32,98,101,32,60,32,37,117,10,0,0,0,111,117,116,95,115,97,109,112,108,101,95,114,97,116,101,0,73,110,118,97,108,105,100,32,102,114,97,109,101,114,97,116,101,32,118,97,108,117,101,58,32,37,115,10,0,0,0,0,99,111,112,121,114,105,103,104,116,0,0,0,0,0,0,0,105,110,118,97,108,105,100,32,112,97,99,107,101,116,58,32,78,85,76,76,32,100,97,116,97,44,32,115,105,122,101,32,33,61,32,48,10,0,0,0,97,117,100,105,111,32,102,111,114,109,97,116,32,102,111,114,32,111,117,116,112,117,116,32,115,116,114,101,97,109,32,37,100,58,37,100,0,0,0,0,86,105,115,117,97,108,108,121,32,73,109,112,97,105,114,101,100,0,0,0,0,0,0,0,114,111,113,95,100,112,99,109,0,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,112,115,110,114,0,0,0,0,66,111,116,104,32,45,102,105,108,116,101,114,32,97,110,100,32,45,102,105,108,116,101,114,95,115,99,114,105,112,116,32,115,101,116,32,102,111,114,32,111,117,116,112,117,116,32,115,116,114,101,97,109,32,35,37,100,58,37,100,46,10,0,0,101,120,112,108,105,99,105,116,95,115,98,114,0,0,0,0,115,101,116,32,112,114,111,98,105,110,103,32,115,105,122,101,0,0,0,0,0,0,0,0,116,114,117,110,99,97,116,101,100,32,99,111,109,109,101,110,116,32,104,101,97,100,101,114,44,32,37,105,32,99,111,109,109,101,110,116,115,32,110,111,116,32,102,111,117,110,100,10,0,0,0,0,0,0,0,0,118,105,0,0,0,0,0,0,77,117,108,116,105,112,108,101,32,102,105,115,98,111,110,101,32,102,111,114,32,116,104,101,32,115,97,109,101,32,115,116,114,101,97,109,10,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,82,101,97,108,65,117,100,105,111,32,50,46,48,32,40,50,56,46,56,75,41,0,0,0,99,121,117,118,0,0,0,0,77,97,120,32,99,104,97,110,110,101,108,32,109,97,112,112,105,110,103,32,102,111,114,32,111,117,116,112,117,116,32,37,100,46,37,100,32,114,101,97,99,104,101,100,10,0,0,0,69,108,101,99,116,114,111,110,105,99,0,0,0,0,0,0,84,104,101,32,112,97,103,101,115,105,122,101,32,111,112,116,105,111,110,32,105,115,32,100,101,112,114,101,99,97,116,101,100,10,0,0,0,0,0,0,67,111,100,101,99,32,110,111,116,32,102,111,117,110,100,10,0,0,0,0,0,0,0,0,37,48,50,100,58,37,48,50,100,58,37,48,50,100,46,37,48,50,100,0,0,0,0,0,69,102,102,101,99,116,115,0,102,95,101,114,114,95,100,101,116,101,99,116,0,0,0,0,114,97,95,50,56,56,0,0,65,112,112,108,105,99,97,116,105,111,110,32,112,114,111,118,105,100,101,100,32,105,110,118,97,108,105,100,44,32,110,111,110,32,109,111,110,111,116,111,110,105,99,97,108,108,121,32,105,110,99,114,101,97,115,105,110,103,32,100,116,115,32,116,111,32,109,117,120,101,114,32,105,110,32,115,116,114,101,97,109,32,37,100,58,32,37,115,32,62,61,32,37,115,10,0,84,73,84,50,0,0,0,0,78,117,109,98,101,114,32,111,102,32,108,111,111,107,97,104,101,97,100,32,101,110,116,114,105,101,115,32,102,111,114,32,73,83,77,32,102,105,108,101,115,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,115,97,109,112,108,101,32,102,111,114,109,97,116,32,39,37,115,39,10,0,0,0,0,0,112,97,100,32,37,100,32,37,100,10,0,0,0,0,0,0,115,101,97,115,111,110,95,110,117,109,98,101,114,0,0,0,82,101,99,111,114,100,105,110,103,32,76,111,99,97,116,105,111,110,0,0,0,0,0,0,116,105,116,108,101,0,0,0,101,102,0,0,0,0,0,0,82,101,97,108,65,117,100,105,111,32,49,46,48,32,40,49,52,46,52,75,41,0,0,0,110,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,102,114,97,109,101,32,115,105,122,101,58,32,37,115,46,10,0,0,0,0,0,0,0,0,77,97,105,110,32,65,117,100,105,111,32,83,101,114,118,105,99,101,0,0,0,0,0,0,114,97,95,49,52,52,0,0,68,97,116,97,32,115,116,114,101,97,109,32,101,110,99,111,100,105,110,103,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,121,101,116,32,40,111,110,108,121,32,115,116,114,101,97,109,99,111,112,121,41,10,0,0,0,0,0,0,0,115,97,109,112,108,101,95,102,109,116,115,0,0,0,0,0,81,117,105,99,107,84,105,109,101,32,47,32,77,79,86,0,109,97,0,0,0,0,0,0,65,77,82,45,87,66,32,40,65,100,97,112,116,105,118,101,32,77,117,108,116,105,45,82,97,116,101,32,87,105,100,101,66,97,110,100,41,0,0,0,73,110,118,97,108,105,100,32,37,115,32,116,121,112,101,32,39,37,115,39,10,0,0,0,80,108,97,110,97,114,32,89,39,67,98,67,114,32,56,45,98,105,116,32,52,58,50,58,48,0,0,0,0,0,0,0,97,117,100,105,111,32,115,101,114,118,105,99,101,32,116,121,112,101,0,0,0,0,0,0,97,109,114,95,119,98,0,0,73,110,118,97,108,105,100,32,109,97,120,95,97,108,108,111,99,32,34,37,115,34,46,10,0,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,37,115,32,39,37,115,39,10,0,0,0,0,0,0,0,0,97,117,100,105,111,95,115,101,114,118,105,99,101,95,116,121,112,101,0,0,0,0,0,0,65,77,82,45,78,66,32,40,65,100,97,112,116,105,118,101,32,77,117,108,116,105,45,82,97,116,101,32,78,97,114,114,111,119,66,97,110,100,41,0,103,101,110,114,101,0,0,0,77,97,116,99,104,101,100,32,37,115,32,39,37,115,39,32,102,111,114,32,99,111,100,101,99,32,39,37,115,39,46,10,0,0,0,0,0,0,0,0,102,114,97,109,101,0,0,0,83,116,97,116,105,115,116,105,99,115,58,32,37,108,108,100,32,98,121,116,101,115,32,114,101,97,100,44,32,37,100,32,115,101,101,107,115,10,0,0,70,97,105,108,101,100,32,116,111,32,105,110,106,101,99,116,32,102,114,97,109,101,32,105,110,116,111,32,102,105,108,116,101,114,32,110,101,116,119,111,114,107,58,32,37,115,10,0,97,109,114,95,110,98,0,0,118,97,97,112,105,95,109,111,99,111,0,0,0,0,0,0,104,116,116,112,115,32,112,114,111,116,111,99,111,108,32,110,111,116,32,102,111,117,110,100,44,32,114,101,99,111,109,112,105,108,101,32,119,105,116,104,32,111,112,101,110,115,115,108,32,111,114,32,103,110,117,116,108,115,32,101,110,97,98,108,101,100,46,10,0,0,0,0,68,97,114,107,84,117,114,113,117,111,105,115,101,0,0,0,97,117,116,104,111,114,0,0,100,101,99,111,100,101,114,0,60,115,97,109,112,108,101,95,102,109,116,62,0,0,0,0,69,0,0,0,0,0,0,0,83,68,82,0,0,0,0,0,115,108,105,99,101,0,0,0,108,105,98,97,118,102,111,114,109,97,116,47,97,115,102,100,101,99,46,99,0,0,0,0,65,68,80,67,77,32,73,77,65,32,82,97,100,105,99,97,108,0,0,0,0,0,0,0,115,99,97,108,101,114,32,112,97,114,97,109,32,49,0,0,115,114,99,45,62,97,117,100,105,111,0,0,0,0,0,0,115,101,116,32,111,117,116,112,117,116,32,115,97,109,112,108,101,32,114,97,116,101,0,0,101,110,99,111,100,101,114,0,115,97,109,112,108,101,32,114,97,116,101,0,0,0,0,0,99,104,97,110,110,101,108,95,108,97,121,111,117,116,115,61,37,115,58,0,0,0,0,0,115,101,108,101,99,116,32,109,117,108,116,105,116,104,114,101,97,100,105,110,103,32,116,121,112,101,0,0,0,0,0,0,100,101,102,97,117,108,116,0,97,100,112,99,109,95,105,109,97,95,114,97,100,0,0,0,101,110,100,0,0,0,0,0,100,111,32,110,111,116,32,100,114,97,119,32,101,100,103,101,115,0,0,0,0,0,0,0,99,111,112,121,0,0,0,0,73,109,112,108,105,99,105,116,32,98,97,99,107,119,97,114,100,115,32,99,111,109,112,97,116,105,98,108,101,32,115,105,103,110,97,108,105,110,103,0,78,0,0,0,0,0,0,0,116,104,114,101,97,100,95,116,121,112,101,0,0,0,0,0,65,68,80,67,77,32,78,105,110,116,101,110,100,111,32,71,97,109,101,99,117,98,101,32,68,84,75,0,0,0,0,0,78,111,32,111,117,116,112,117,116,32,112,97,100,32,99,97,110,32,98,101,32,97,115,115,111,99,105,97,116,101,100,32,116,111,32,108,105,110,107,32,108,97,98,101,108,32,39,37,115,39,46,10,0,0,0,0,72,117,102,102,89,85,86,0,37,115,37,115,47,37,115,46,97,118,112,114,101,115,101,116,0,0,0,0,0,0,0,0,84,101,99,104,110,111,45,73,110,100,117,115,116,114,105,97,108,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,32,32,68,117,114,97,116,105,111,110,58,32,0,0,0,0,110,117,109,98,101,114,32,111,102,32,115,108,105,99,101,115,44,32,117,115,101,100,32,105,110,32,112,97,114,97,108,108,101,108,105,122,101,100,32,101,110,99,111,100,105,110,103,0,115,105,122,101,32,105,110,32,98,121,116,101,115,32,102,111,114,32,101,97,99,104,32,99,104,117,110,107,0,0,0,0,97,100,112,99,109,95,100,116,107,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,115,97,109,112,108,101,32,102,111,114,109,97,116,32,37,115,10,0,0,0,0,0,0,0,105,115,109,95,108,111,111,107,97,104,101,97,100,0,0,0,47,46,97,118,99,111,110,118,0,0,0,0,0,0,0,0,101,112,105,115,111,100,101,95,115,111,114,116,0,0,0,0,76,121,114,105,99,105,115,116,47,116,101,120,116,32,119,114,105,116,101,114,0,0,0,0,73,110,118,97,108,105,100,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,100,97,116,97,32,115,105,122,101,58,32,37,100,32,62,61,32,37,100,46,10,0,0,115,108,105,99,101,115,0,0,65,68,80,67,77,32,73,77,65,32,68,105,97,108,111,103,105,99,32,79,75,73,0,0,110,32,61,61,32,115,105,110,107,95,108,105,110,107,115,95,99,111,117,110,116,0,0,0,116,0,0,0,0,0,0,0,37,115,37,115,47,37,115,45,37,115,46,97,118,112,114,101,115,101,116,0,0,0,0,0,105,109,97,103,101,47,112,110,103,0,0,0,0,0,0,0,115,101,116,32,116,104,101,32,108,111,103,32,108,101,118,101,108,32,111,102,102,115,101,116,0,0,0,0,0,0,0,0,99,105,114,99,117,108,97,114,32,102,105,108,116,101,114,32,99,104,97,105,110,32,100,101,116,101,99,116,101,100,10,0,32,57,37,37,32,32,89,111,117,114,32,115,121,115,116,101,109,32,105,115,32,111,118,101,114,99,108,111,99,107,101,100,0,0,0,0,0,0,0,0,97,100,112,99,109,95,105,109,97,95,111,107,105,0,0,0,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,115,104,97,114,101,47,102,102,109,112,101,103,0,0,0,0,115,104,111,119,32,115,116,97,110,100,97,114,100,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,115,0,0,0,108,111,103,95,108,101,118,101,108,95,111,102,102,115,101,116,0,0,0,0,0,0,0,0,65,68,80,67,77,32,78,105,110,116,101,110,100,111,32,71,97,109,101,99,117,98,101,32,65,70,67,0,0,0,0,0,72,79,77,69,0,0,0,0,99,104,114,111,109,97,95,115,97,109,112,108,101,95,108,111,99,97,116,105,111,110,0,0,97,100,112,99,109,95,97,102,99,0,0,0,0,0,0,0,34,37,115,34,10,0,0,0,65,86,67,79,78,86,95,68,65,84,65,68,73,82,0,0,99,111,108,111,114,95,114,97,110,103,101,0,0,0,0,0,65,68,80,67,77,32,73,77,65,32,67,82,89,79,32,65,80,67,0,0,0,0,0,0,67,111,117,108,100,32,110,111,116,32,97,108,108,111,99,32,98,117,102,102,101,114,32,102,111,114,32,114,101,97,100,105,110,103,32,112,114,101,115,101,116,46,10,0,0,0,0,0,99,111,108,111,114,115,112,97,99,101,0,0,0,0,0,0,97,100,112,99,109,95,105,109,97,95,97,112,99,0,0,0,115,97,109,112,108,101,95,97,115,112,101,99,116,95,114,97,116,105,111,0,0,0,0,0,98,103,114,53,53,53,108,101,0,0,0,0,0,0,0,0,68,97,114,107,83,108,97,116,101,71,114,97,121,0,0,0,50,52,0,0,0,0,0,0,60,112,105,120,95,102,109,116,62,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,102,117,110,99,116,105,111,110,32,105,110,32,39,37,115,39,10,0,0,0,0,0,0,0,45,45,99,114,111,115,115,45,112,114,101,102,105,120,61,101,109,32,45,45,99,99,61,101,109,99,99,32,45,45,101,110,97,98,108,101,45,99,114,111,115,115,45,99,111,109,112,105,108,101,32,45,45,116,97,114,103,101,116,45,111,115,61,110,111,110,101,32,45,45,97,114,99,104,61,120,56,54,95,51,50,32,45,45,112,114,101,102,105,120,61,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,32,45,45,101,120,116,114,97,45,99,102,108,97,103,115,61,45,73,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,105,110,99,108,117,100,101,32,45,45,99,112,117,61,103,101,110,101,114,105,99,32,45,45,100,105,115,97,98,108,101,45,104,119,97,99,99,101,108,115,32,45,45,100,105,115,97,98,108,101,45,115,116,114,105,112,112,105,110,103,32,45,45,100,105,115,97,98,108,101,45,112,116,104,114,101,97,100,115,32,45,45,100,105,115,97,98,108,101,45,100,111,99,32,45,45,100,105,115,97,98,108,101,45,100,101,98,117,103,32,45,45,100,105,115,97,98,108,101,45,97,115,109,32,45,45,100,105,115,97,98,108,101,45,110,101,116,119,111,114,107,32,45,45,100,105,115,97,98,108,101,45,101,118,101,114,121,116,104,105,110,103,32,45,45,100,105,115,97,98,108,101,45,102,102,112,108,97,121,32,45,45,100,105,115,97,98,108,101,45,102,102,112,114,111,98,101,32,45,45,100,105,115,97,98,108,101,45,102,102,115,101,114,118,101,114,32,45,45,100,105,115,97,98,108,101,45,111,117,116,100,101,118,61,115,100,108,32,45,45,101,110,97,98,108,101,45,111,112,116,105,109,105,122,97,116,105,111,110,115,32,45,45,101,110,97,98,108,101,45,112,114,111,116,111,99,111,108,61,102,105,108,101,32,45,45,101,110,97,98,108,101,45,102,105,108,116,101,114,61,97,114,101,115,97,109,112,108,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,119,97,118,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,115,49,54,108,101,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,117,56,32,45,45,101,110,97,98,108,101,45,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,108,105,98,102,100,107,45,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,111,118,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,102,100,107,95,97,97,99,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,52,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,115,102,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,97,115,102,0,115,117,114,114,111,117,110,100,32,100,105,114,101,99,116,32,108,101,102,116,0,0,0,0,99,111,108,111,114,95,116,114,99,0,0,0,0,0,0,0,71,46,55,50,50,32,65,68,80,67,77,0,0,0,0,0,112,97,114,97,109,49,0,0,105,110,95,115,97,109,112,108,101,95,114,97,116,101,0,0,111,117,116,112,117,116,95,115,97,109,112,108,101,95,98,105,116,115,0,0,0,0,0,0,115,97,109,112,108,101,95,114,97,116,101,115,61,37,115,58,0,0,0,0,0,0,0,0,99,111,108,111,114,95,112,114,105,109,97,114,105,101,115,0,97,100,112,99,109,95,103,55,50,50,0,0,0,0,0,0,101,109,117,95,101,100,103,101,0,0,0,0,0,0,0,0,115,119,115,95,102,108,97,103,115,0,0,0,0,0,0,0,105,105,100,95,109,111,100,101,32,37,100,32,105,115,32,114,101,115,101,114,118,101,100,46,10,0,0,0,0,0,0,0,105,109,112,108,105,99,105,116,0,0,0,0,0,0,0,0,116,105,99,107,115,95,112,101,114,95,102,114,97,109,101,0,65,68,80,67,77,32,73,77,65,32,70,117,110,99,111,109,32,73,83,83,0,0,0,0,104,117,102,102,121,117,118,0,85,110,107,110,111,119,110,32,98,105,116,115,116,114,101,97,109,32,102,105,108,116,101,114,32,37,115,10,0,0,0,0,68,97,114,107,119,97,118,101,0,0,0,0,0,0,0,0,32,32,0,0,0,0,0,0,114,99,95,109,105,110,95,118,98,118,95,117,115,101,0,0,99,104,117,110,107,95,115,105,122,101,0,0,0,0,0,0,98,121,116,101,95,111,102,102,115,101,116,95,98,105,116,115,32,37,100,32,105,115,32,116,111,111,32,108,97,114,103,101,10,0,0,0,0,0,0,0,97,100,112,99,109,95,105,109,97,95,105,115,115,0,0,0,77,97,120,105,109,117,109,32,102,114,97,103,109,101,110,116,32,115,105,122,101,0,0,0,65,112,112,108,121,105,110,103,32,117,110,115,112,101,99,105,102,105,99,32,45,102,114,97,109,101,115,32,116,111,32,110,111,110,32,118,105,100,101,111,32,115,116,114,101,97,109,115,44,32,109,97,121,98,101,32,121,111,117,32,109,101,97,110,116,32,45,118,102,114,97,109,101,115,32,63,10,0,0,0,100,105,115,99,0,0,0,0,67,111,109,112,111,115,101,114,0,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,109,105,109,101,116,121,112,101,58,32,37,115,46,10,0,114,99,95,109,97,120,95,118,98,118,95,117,115,101,0,0,65,68,80,67,77,32,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,77,97,120,105,115,32,67,68,82,79,77,32,88,65,0,0,0,0,69,114,114,111,114,32,119,104,101,110,32,101,118,97,108,117,97,116,105,110,103,32,116,104,101,32,101,120,112,114,101,115,115,105,111,110,32,39,37,115,39,32,102,111,114,32,101,110,97,98,108,101,10,0,0,0,80,114,101,115,101,116,32,37,115,32,115,112,101,99,105,102,105,101,100,32,102,111,114,32,115,116,114,101,97,109,32,37,100,58,37,100,44,32,98,117,116,32,99,111,117,108,100,32,110,111,116,32,98,101,32,111,112,101,110,101,100,46,10,0,114,101,113,117,101,115,116,95,99,104,97,110,110,101,108,95,108,97,121,111,117,116,0,0,97,100,112,99,109,95,101,97,95,109,97,120,105,115,95,120,97,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32])
.concat([108,105,110,101,32,102,111,117,110,100,32,105,110,32,116,104,101,32,112,114,101,115,101,116,32,102,105,108,101,46,10,0,108,97,121,111,117,116,115,0,71,108,111,98,97,108,32,111,112,116,105,111,110,115,32,40,97,102,102,101,99,116,32,119,104,111,108,101,32,112,114,111,103,114,97,109,32,105,110,115,116,101,97,100,32,111,102,32,106,117,115,116,32,111,110,101,32,102,105,108,101,58,0,0,98,105,116,115,95,112,101,114,95,114,97,119,95,115,97,109,112,108,101,0,0,0,0,0,65,68,80,67,77,32,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,88,65,83,0,0,0,0,0,0,0,67,111,117,108,100,32,110,111,116,32,97,108,108,111,99,32,115,116,114,101,97,109,46,10,0,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,115,101,116,32,100,101,115,105,114,101,100,32,110,117,109,98,101,114,32,111,102,32,97,117,100,105,111,32,99,104,97,110,110,101,108,115,0,0,0,0,97,100,112,99,109,95,101,97,95,120,97,115,0,0,0,0,73,110,118,97,108,105,100,32,108,111,103,108,101,118,101,108,32,34,37,115,34,46,32,80,111,115,115,105,98,108,101,32,108,101,118,101,108,115,32,97,114,101,32,110,117,109,98,101,114,115,32,111,114,58,10,0,70,105,108,101,32,39,37,115,39,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,46,32,69,120,105,116,105,110,103,46,10,0,0,0,0,0,114,101,113,117,101,115,116,95,99,104,97,110,110,101,108,115,0,0,0,0,0,0,0,0,65,68,80,67,77,32,73,77,65,32,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,69,65,67,83,0,0,78,111,116,32,111,118,101,114,119,114,105,116,105,110,103,32,45,32,101,120,105,116,105,110,103,10,0,0,0,0,0,0,71,79,80,32,116,105,109,101,99,111,100,101,32,102,114,97,109,101,32,115,116,97,114,116,32,110,117,109,98,101,114,44,32,105,110,32,110,111,110,45,100,114,111,112,45,102,114,97,109,101,32,102,111,114,109,97,116,0,0,0,0,0,0,0,97,100,112,99,109,95,105,109,97,95,101,97,95,101,97,99,115,0,0,0,0,0,0,0,69,114,114,111,114,32,114,101,105,110,105,116,105,97,108,105,122,105,110,103,32,102,105,108,116,101,114,115,33,10,0,0,98,103,114,53,53,53,98,101,0,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,68,97,114,107,83,108,97,116,101,66,108,117,101,0,0,0,70,105,108,101,32,39,37,115,39,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,46,32,79,118,101,114,119,114,105,116,101,32,63,32,91,121,47,78,93,32,0,0,0,0,60,118,105,100,101,111,95,114,97,116,101,62,0,0,0,0,98,101,116,119,101,101,110,0,83,68,76,0,0,0,0,0,116,105,109,101,99,111,100,101,95,102,114,97,109,101,95,115,116,97,114,116,0,0,0,0,115,119,115,58,32,105,110,105,116,70,105,108,116,101,114,32,102,97,105,108,101,100,10,0,65,68,80,67,77,32,73,77,65,32,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,83,69,65,68,0,0,115,99,97,108,101,114,32,112,97,114,97,109,32,48,0,0,115,101,116,32,105,110,112,117,116,32,115,97,109,112,108,101,32,114,97,116,101,0,0,0,102,105,108,101,58,0,0,0,37,100,32,102,114,97,109,101,115,32,108,101,102,116,32,105,110,32,116,104,101,32,113,117,101,117,101,32,111,110,32,99,108,111,115,105,110,103,10,0,115,97,109,112,108,101,95,102,109,116,115,61,37,115,58,0,109,97,120,95,112,114,101,100,105,99,116,105,111,110,95,111,114,100,101,114,0,0,0,0,97,100,112,99,109,95,105,109,97,95,101,97,95,115,101,97,100,0,0,0,0,0,0,0,83,116,114,101,97,109,32,115,112,101,99,105,102,105,101,114,32,37,115,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,32,97,110,121,32,115,116,114,101,97,109,115,46,10,0,0,0,0,0,0,0,111,110,108,121,32,100,101,99,111,100,101,47,101,110,99,111,100,101,32,103,114,97,121,115,99,97,108,101,0,0,0,0,67,104,111,111,115,101,32,115,105,103,110,97,108,105,110,103,32,105,109,112,108,105,99,105,116,108,121,32,40,101,120,112,108,105,99,105,116,32,104,105,101,114,97,114,99,104,105,99,97,108,32,98,121,32,100,101,102,97,117,108,116,44,32,105,109,112,108,105,99,105,116,32,105,102,32,103,108,111,98,97,108,32,104,101,97,100,101,114,32,105,115,32,100,105,115,97,98,108,101,100,41,0,0,0,109,105,110,95,112,114,101,100,105,99,116,105,111,110,95,111,114,100,101,114,0,0,0,0,65,68,80,67,77,32,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,82,50,0,0,0,0,0,0,0,0,68,86,32,40,68,105,103,105,116,97,108,32,86,105,100,101,111,41,0,0,0,0,0,0,102,102,109,112,101,103,95,111,112,116,46,99,0,0,0,0,71,111,116,104,105,99,0,0,102,114,111,109,0,0,0,0,99,111,109,112,114,101,115,115,105,111,110,95,108,101,118,101,108,0,0,0,0,0,0,0,109,105,99,114,111,115,101,99,111,110,100,115,32,102,111,114,32,101,97,99,104,32,99,104,117,110,107,0,0,0,0,0,97,100,112,99,109,95,101,97,95,114,50,0,0,0,0,0,102,114,97,103,95,115,105,122,101,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,116,114,97,99,107,0,0,0,66,97,110,100,47,79,114,99,104,101,115,116,114,97,0,0,85,110,107,110,111,119,110,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,116,121,112,101,58,32,37,100,46,10,0,0,0,0,0,97,100,106,117,115,116,32,115,101,110,115,105,116,105,118,105,116,121,32,111,102,32,98,95,102,114,97,109,101,95,115,116,114,97,116,101,103,121,32,49,0,0,0,0,0,0,0,0,65,68,80,67,77,32,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,82,51,0,0,0,0,0,0,0,0,84,105,109,101,108,105,110,101,32,40,39,101,110,97,98,108,101,39,32,111,112,116,105,111,110,41,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,119,105,116,104,32,102,105,108,116,101,114,32,39,37,115,39,10,0,0,0,0,0,0,112,114,111,103,114,97,109,0,98,95,115,101,110,115,105,116,105,118,105,116,121,0,0,0,97,100,112,99,109,95,101,97,95,114,51,0,0,0,0,0,99,104,97,112,116,101,114,0,115,104,111,119,32,97,118,97,105,108,97,98,108,101,32,112,105,120,101,108,32,102,111,114,109,97,116,115,0,0,0,0,109,118,48,95,116,104,114,101,115,104,111,108,100,0,0,0,65,68,80,67,77,32,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,82,49,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,37,115,32,105,110,100,101,120,32,37,100,32,119,104,105,108,101,32,112,114,111,99,101,115,115,105,110,103,32,109,101,116,97,100,97,116,97,32,109,97,112,115,46,10,0,0,0,0,109,117,108,116,105,112,108,105,101,100,32,98,121,32,113,115,99,97,108,101,32,102,111,114,32,101,97,99,104,32,102,114,97,109,101,32,97,110,100,32,97,100,100,101,100,32,116,111,32,115,99,101,110,101,95,99,104,97,110,103,101,95,115,99,111,114,101,0,0,0,0,0,97,100,112,99,109,95,101,97,95,114,49,0,0,0,0,0,114,101,112,101,97,116,0,0,73,110,118,97,108,105,100,32,109,101,116,97,100,97,116,97,32,116,121,112,101,32,37,99,46,10,0,0,0,0,0,0,115,99,95,102,97,99,116,111,114,0,0,0,0,0,0,0,65,68,80,67,77,32,73,77,65,32,65,77,86,0,0,0,69,120,112,101,99,116,101,100,32,116,111,32,114,101,97,100,32,37,100,32,83,66,82,32,98,121,116,101,115,32,97,99,116,117,97,108,108,121,32,114,101,97,100,32,37,100,46,10,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,109,101,116,97,100,97,116,97,32,115,112,101,99,105,102,105,101,114,32,37,115,46,10,0,114,97,116,101,45,100,105,115,116,111,114,116,105,111,110,32,111,112,116,105,109,97,108,32,113,117,97,110,116,105,122,97,116,105,111,110,0,0,0,0,97,100,112,99,109,95,105,109,97,95,97,109,118,0,0,0,73,110,112,117,116,32,115,116,114,101,97,109,32,35,37,100,58,37,100,32,102,114,97,109,101,32,99,104,97,110,103,101,100,32,102,114,111,109,32,115,105,122,101,58,37,100,120,37,100,32,102,109,116,58,37,115,32,116,111,32,115,105,122,101,58,37,100,120,37,100,32,102,109,116,58,37,115,10,0,0,98,103,114,53,54,53,108,101,0,0,0,0,0,0,0,0,68,97,114,107,83,101,97,71,114,101,101,110,0,0,0,0,73,110,118,97,108,105,100,32,99,104,97,112,116,101,114,32,105,110,100,101,120,32,37,100,32,105,110,32,109,101,116,97,100,97,116,97,32,115,112,101,99,105,102,105,101,114,46,10,0,0,0,0,0,0,0,0,60,105,109,97,103,101,95,115,105,122,101,62,0,0,0,0,98,105,116,111,114,0,0,0,119,105,100,101,32,114,105,103,104,116,0,0,0,0,0,0,116,114,101,108,108,105,115,0,83,119,83,99,97,108,101,114,58,32,114,101,100,117,99,105,110,103,32,47,32,97,108,105,103,110,105,110,103,32,102,105,108,116,101,114,115,105,122,101,32,37,100,32,45,62,32,37,100,10,0,0,0,0,0,0,65,68,80,67,77,32,78,105,110,116,101,110,100,111,32,71,97,109,101,99,117,98,101,32,84,72,80,0,0,0,0,0,112,97,114,97,109,48,0,0,117,115,101,100,95,99,104,97,110,110,101,108,95,99,111,117,110,116,0,0,0,0,0,0,78,111,32,39,61,39,32,99,104,97,114,97,99,116,101,114,32,105,110,32,109,101,116,97,100,97,116,97,32,115,116,114,105,110,103,32,37,115,46,10,0,0,0,0,0,0,0,0,57,54,48,47,49,50,48,32,77,68,67,84,32,119,105,110,100,111,119,0,0,0,0,0,73,110,118,97,108,105,100,32,109,101,100,105,97,32,116,121,112,101,32,102,111,114,32,118,105,100,101,111,10,0,0,0,112,97,110,0,0,0,0,0,99,104,114,111,109,97,32,81,80,32,111,102,102,115,101,116,32,102,114,111,109,32,108,117,109,97,0,0,0,0,0,0,97,100,112,99,109,95,116,104,112,0,0,0,0,0,0,0,99,114,101,97,116,105,111,110,95,116,105,109,101,0,0,0,103,114,97,121,0,0,0,0,100,101,102,97,117,108,116,0,99,104,114,111,109,97,111,102,102,115,101,116,0,0,0,0,65,68,80,67,77,32,83,111,117,110,100,32,66,108,97,115,116,101,114,32,80,114,111,32,50,45,98,105,116,0,0,0,100,118,118,105,100,101,111,0,73,110,118,97,108,105,100,32,105,110,112,117,116,32,102,105,108,101,32,105,110,100,101,120,32,37,100,32,105,110,32,99,104,97,112,116,101,114,32,109,97,112,112,105,110,103,46,10,0,0,0,0,0,0,0,0,69,116,104,110,105,99,0,0,116,111,0,0,0,0,0,0,114,101,102,101,114,101,110,99,101,32,102,114,97,109,101,115,32,116,111,32,99,111,110,115,105,100,101,114,32,102,111,114,32,109,111,116,105,111,110,32,99,111,109,112,101,110,115,97,116,105,111,110,0,0,0,0,99,104,117,110,107,95,100,117,114,97,116,105,111,110,0,0,97,100,112,99,109,95,115,98,112,114,111,95,50,0,0,0,77,105,110,105,109,117,109,32,102,114,97,103,109,101,110,116,32,100,117,114,97,116,105,111,110,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,105,110,112,117,116,32,102,105,108,101,32,105,110,100,101,120,32,37,100,32,119,104,105,108,101,32,112,114,111,99,101,115,115,105,110,103,32,109,101,116,97,100,97,116,97,32,109,97,112,115,10,0,0,0,0,0,110,101,116,119,111,114,107,0,67,111,110,100,117,99,116,111,114,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,115,105,122,101,58,32,37,100,46,10,0,0,0,0,0,114,101,102,115,0,0,0,0,65,68,80,67,77,32,83,111,117,110,100,32,66,108,97,115,116,101,114,32,80,114,111,32,50,46,54,45,98,105,116,0,115,101,116,32,101,110,97,98,108,101,32,101,120,112,114,101,115,115,105,111,110,0,0,0,112,114,101,108,111,97,100,0,109,105,110,105,109,117,109,32,105,110,116,101,114,118,97,108,32,98,101,116,119,101,101,110,32,73,68,82,45,102,114,97,109,101,115,0,0,0,0,0,97,100,112,99,109,95,115,98,112,114,111,95,51,0,0,0,97,97,99,0,0,0,0,0,105,109,97,103,101,50,0,0,112,105,120,95,102,109,116,115,0,0,0,0,0,0,0,0,107,101,121,105,110,116,95,109,105,110,0,0,0,0,0,0,65,68,80,67,77,32,83,111,117,110,100,32,66,108,97,115,116,101,114,32,80,114,111,32,52,45,98,105,116,0,0,0,67,111,100,101,99,32,65,86,79,112,116,105,111,110,32,37,115,32,40,37,115,41,32,115,112,101,99,105,102,105,101,100,32,102,111,114,32,111,117,116,112,117,116,32,102,105,108,101,32,35,37,100,32,40,37,115,41,32,104,97,115,32,110,111,116,32,98,101,101,110,32,117,115,101,100,32,102,111,114,32,97,110,121,32,115,116,114,101,97,109,46,32,84,104,101,32,109,111,115,116,32,108,105,107,101,108,121,32,114,101,97,115,111,110,32,105,115,32,101,105,116,104,101,114,32,119,114,111,110,103,32,116,121,112,101,32,40,101,46,103,46,32,97,32,118,105,100,101,111,32,111,112,116,105,111,110,32,119,105,116,104,32,110,111,32,118,105,100,101,111,32,115,116,114,101,97,109,115,41,32,111,114,32,116,104,97,116,32,105,116,32,105,115,32,97,32,112,114,105,118,97,116,101,32,111,112,116,105,111,110,32,111,102,32,115,111,109,101,32,101,110,99,111,100,101,114,32,119,104,105,99,104,32,119,97,115,32,110,111,116,32,97,99,116,117,97,108,108,121,32,117,115,101,100,32,102,111,114,32,97,110,121,32,115,116,114,101,97,109,46,10,0,100,111,119,110,115,99,97,108,101,32,102,114,97,109,101,115,32,102,111,114,32,100,121,110,97,109,105,99,32,66,45,102,114,97,109,101,32,100,101,99,105,115,105,111,110,0,0,0,119,109,97,118,49,0,0,0,97,100,112,99,109,95,115,98,112,114,111,95,52,0,0,0,118,101,114,98,111,115,101,0,67,111,100,101,99,32,65,86,79,112,116,105,111,110,32,37,115,32,40,37,115,41,32,115,112,101,99,105,102,105,101,100,32,102,111,114,32,111,117,116,112,117,116,32,102,105,108,101,32,35,37,100,32,40,37,115,41,32,105,115,32,110,111,116,32,97,110,32,101,110,99,111,100,105,110,103,32,111,112,116,105,111,110,46,10,0,0,0,73,110,118,97,108,105,100,32,37,115,32,115,112,101,99,105,102,105,99,97,116,105,111,110,32,102,111,114,32,37,115,58,32,37,115,10,0,0,0,0,98,114,100,95,115,99,97,108,101,0,0,0,0,0,0,0,65,68,80,67,77,32,89,97,109,97,104,97,0,0,0,0,65,116,116,97,99,104,109,101,110,116,32,37,115,32,116,111,111,32,108,97,114,103,101,32,116,111,32,102,105,116,32,105,110,116,111,32,109,101,109,111,114,121,46,10,0,0,0,0,114,101,102,105,110,101,32,116,104,101,32,116,119,111,32,109,111,116,105,111,110,32,118,101,99,116,111,114,115,32,117,115,101,100,32,105,110,32,98,105,100,105,114,101,99,116,105,111,110,97,108,32,109,97,99,114,111,98,108,111,99,107,115,0,119,109,97,118,49,0,0,0,97,100,112,99,109,95,121,97,109,97,104,97,0,0,0,0,100,101,99,111,100,101,114,32,45,62,32,105,115,116,95,105,110,100,101,120,58,37,100,32,116,121,112,101,58,118,105,100,101,111,32,102,114,97,109,101,95,112,116,115,58,37,115,32,102,114,97,109,101,95,112,116,115,95,116,105,109,101,58,37,115,32,98,101,115,116,95,101,102,102,111,114,116,95,116,115,58,37,108,108,100,32,98,101,115,116,95,101,102,102,111,114,116,95,116,115,95,116,105,109,101,58,37,115,32,107,101,121,102,114,97,109,101,58,37,100,32,102,114,97,109,101,95,116,121,112,101,58,37,100,32,10,0,0,0,0,0,0,0,0,98,103,114,53,54,53,98,101,0,0,0,0,0,0,0,0,68,97,114,107,83,97,108,109,111,110,0,0,0,0,0,0,67,111,117,108,100,32,110,111,116,32,103,101,116,32,115,105,122,101,32,111,102,32,116,104,101,32,97,116,116,97,99,104,109,101,110,116,32,37,115,46,10,0,0,0,0,0,0,0,60,98,105,110,97,114,121,62,0,0,0,0,0,0,0,0,98,105,116,97,110,100,0,0,87,82,0,0,0,0,0,0,98,105,100,105,114,95,114,101,102,105,110,101,0,0,0,0,115,119,115,58,32,102,105,108,116,101,114,83,105,122,101,32,37,100,32,105,115,32,116,111,111,32,108,97,114,103,101,44,32,116,114,121,32,108,101,115,115,32,101,120,116,114,101,101,109,32,115,99,97,108,105,110,103,32,111,114,32,105,110,99,114,101,97,115,101,32,77,65,88,95,70,73,76,84,69,82,95,83,73,90,69,32,97,110,100,32,114,101,99,111,109,112,105,108,101,10,0,0,0,0,65,68,80,67,77,32,83,104,111,99,107,119,97,118,101,32,70,108,97,115,104,0,0,0,100,101,115,116,105,110,97,116,105,111,110,32,114,97,110,103,101,0,0,0,0,0,0,0,115,101,116,32,117,115,101,100,32,99,104,97,110,110,101,108,32,99,111,117,110,116,0,0,67,111,117,108,100,32,110,111,116,32,111,112,101,110,32,97,116,116,97,99,104,109,101,110,116,32,102,105,108,101,32,37,115,46,10,0,0,0,0,0,115,116,97,114,116,95,100,105,115,112,108,97,121,95,116,105,109,101,32,109,117,115,116,32,98,101,32,48,46,10,0,0,100,105,115,99,97,114,100,32,97,108,108,32,102,114,97,109,101,115,0,0,0,0,0,0,45,109,97,112,95,99,104,97,110,110,101,108,32,105,115,32,102,111,114,119,97,114,100,101,100,32,116,111,32,108,97,118,102,105,32,115,105,109,105,108,97,114,108,121,32,116,111,32,45,97,102,32,112,97,110,61,37,115,46,10,0,0,0,0,97,100,112,99,109,95,115,119,102,0,0,0,0,0,0,0,69,120,116,114,97,100,97,116,97,32,99,111,114,114,117,112,116,46,10,0,0,0,0,0,67,97,110,110,111,116,32,109,97,112,32,115,116,114,101,97,109,32,35,37,100,58,37,100,32,45,32,117,110,115,117,112,112,111,114,116,101,100,32,116,121,112,101,46,10,0,0,0,117,115,101,32,105,110,116,101,114,110,97,108,32,50,45,112,97,115,115,32,114,97,116,101,99,111,110,116,114,111,108,32,105,110,32,115,101,99,111,110,100,32,112,97,115,115,32,109,111,100,101,0,0,0,0,0,83,66,82,47,80,83,32,115,105,103,110,97,108,105,110,103,32,115,116,121,108,101,0,0,97,108,108,0,0,0,0,0,65,68,80,67,77,32,67,114,101,97,116,105,118,101,32,84,101,99,104,110,111,108,111,103,121,0,0,0,0,0,0,0,83,111,114,101,110,115,111,110,32,86,101,99,116,111,114,32,81,117,97,110,116,105,122,101,114,32,51,32,47,32,83,111,114,101,110,115,111,110,32,86,105,100,101,111,32,51,32,47,32,83,86,81,51,0,0,0,79,117,116,112,117,116,32,119,105,116,104,32,108,97,98,101,108,32,39,37,115,39,32,100,111,101,115,32,110,111,116,32,101,120,105,115,116,32,105,110,32,97,110,121,32,100,101,102,105,110,101,100,32,102,105,108,116,101,114,32,103,114,97,112,104,44,32,111,114,32,119,97,115,32,97,108,114,101,97,100,121,32,117,115,101,100,32,101,108,115,101,119,104,101,114,101,46,10,0,0,0,0,0,0,73,110,115,116,114,117,109,101,110,116,97,108,32,82,111,99,107,0,0,0,0,0,0,0,73,110,112,117,116,0,0,0,100,105,115,99,97,114,100,32,97,108,108,32,102,114,97,109,101,115,32,101,120,99,101,112,116,32,107,101,121,102,114,97,109,101,115,0,0,0,0,0,109,105,99,114,111,115,101,99,111,110,100,115,32,98,121,32,119,104,105,99,104,32,97,117,100,105,111,32,112,97,99,107,101,116,115,32,115,104,111,117,108,100,32,98,101,32,105,110,116,101,114,108,101,97,118,101,100,32,101,97,114,108,105,101,114,0,0,0,0,0,0,0,97,100,112,99,109,95,99,116,0,0,0,0,0,0,0,0,109,105,110,95,102,114,97,103,95,100,117,114,97,116,105,111,110,0,0,0,0,0,0,0,77,105,115,115,105,110,103,32,37,115,32,115,116,114,101,97,109,32,119,104,105,99,104,32,105,115,32,114,101,113,117,105,114,101,100,32,98,121,32,116,104,105,115,32,102,102,109,10,0,0,0,0,0,0,0,0,101,112,105,115,111,100,101,95,105,100,0,0,0,0,0,0,65,114,116,105,115,116,47,112,101,114,102,111,114,109,101,114,0,0,0,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,118,97,108,117,101,32,116,121,112,101,32,37,100,32,105,110,32,116,97,103,32,37,115,46,10,0,0,0,110,111,107,101,121,0,0,0,71,46,55,50,54,32,65,68,80,67,77,0,0,0,0,0,115,108,105,99,101,0,0,0,110,117,108,108,0,0,0,0,100,105,115,99,97,114,100,32,97,108,108,32,98,105,100,105,114,101,99,116,105,111,110,97,108,32,102,114,97,109,101,115,0,0,0,0,0,0,0,0,97,100,112,99,109,95,103,55,50,54,0,0,0,0,0,0,97,110,117,108,108,0,0,0,115,104,111,119,32,97,118,97,105,108,97,98,108,101,32,102,105,108,116,101,114,115,0,0,98,105,100,105,114,0,0,0,65,68,80,67,77,32,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,0,0,0,99,114,101,97,116,105,111,110,95,116,105,109,101,61,110,111,119,0,0,0,0,0,0,0,100,105,115,99,97,114,100,32,97,108,108,32,110,111,110,45,114,101,102,101,114,101,110,99,101,32,102,114,97,109,101,115,0,0,0,0,0,0,0,0,97,100,112,99,109,95,101,97,0,0,0,0,0,0,0,0,105,110,102,111,0,0,0,0,104,116,116,112,58,0,0,0,110,111,114,101,102,0,0,0,83,69,71,65,32,67,82,73,32,65,68,88,32,65,68,80,67,77,0,0,0,0,0,0,102,102,109,0,0,0,0,0,100,105,115,99,97,114,100,32,117,115,101,108,101,115,115,32,102,114,97,109,101,115,0,0,97,100,112,99,109,95,97,100,120,0,0,0,0,0,0,0,100,101,99,111,100,101,95,118,105,100,101,111,32,37,100,46,37,100,0,0,0,0,0,0,114,103,98,53,53,53,108,101,0,0,0,0,0,0,0,0,68,97,114,107,82,101,100,0,45,116,111,32,118,97,108,117,101,32,115,109,97,108,108,101,114,32,116,104,97,110,32,45,115,115,59,32,105,103,110,111,114,105,110,103,32,45,116,111,46,10,0,0,0,0,0,0,60,114,97,116,105,111,110,97,108,62,0,0,0,0,0,0,105,102,110,111,116,0,0,0,119,105,100,101,32,108,101,102,116,0,0,0,0,0,0,0,100,101,102,97,117,108,116,0,109,105,110,70,105,108,116,101,114,83,105,122,101,32,62,32,48,0,0,0,0,0,0,0,65,68,80,67,77,32,67,68,82,79,77,32,88,65,0,0,100,115,116,95,114,97,110,103,101,0,0,0,0,0,0,0,118,105,100,101,111,0,0,0,111,117,116,95,99,104,97,110,110,101,108,95,99,111,117,110,116,0,0,0,0,0,0,0,37,100,58,37,100,58,37,100,37,99,37,100,0,0,0,0,45,116,32,97,110,100,32,45,116,111,32,99,97,110,110,111,116,32,98,101,32,117,115,101,100,32,116,111,103,101,116,104,101,114,59,32,117,115,105,110,103,32,45,116,46,10,0,0,83,66,82,43,0,0,0,0,114,101,116,32,60,61,32,48,0,0,0,0,0,0,0,0,108,105,98,97,118,117,116,105,108,47,114,97,110,100,111,109,95,115,101,101,100,46,99,0,100,105,115,99,97,114,100,32,110,111,32,102,114,97,109,101,0,0,0,0,0,0,0,0,58,99,37,100,61,99,37,100,0,0,0,0,0,0,0,0,97,100,112,99,109,95,120,97,0,0,0,0,0,0,0,0,69,114,114,111,114,32,99,111,110,102,105,103,117,114,105,110,103,32,102,105,108,116,101,114,115,46,10,0,0,0,0,0,112,97,115,115,50,0,0,0,115,105,103,110,97,108,105,110,103,0,0,0,0,0,0,0,110,111,110,101,0,0,0,0,65,68,80,67,77,32,52,88,32,77,111,118,105,101,0,0,115,118,113,51,0,0,0,0,83,116,114,101,97,109,32,109,97,112,32,39,37,115,39,32,109,97,116,99,104,101,115,32,110,111,32,115,116,114,101,97,109,115,46,10,0,0,0,0,73,110,115,116,114,117,109,101,110,116,97,108,32,80,111,112,0,0,0,0,0,0,0,0,121,117,118,52,50,50,112,0,79,117,116,112,117,116,0,0,115,107,105,112,32,100,101,99,111,100,105,110,103,32,102,111,114,32,116,104,101,32,115,101,108,101,99,116,101,100,32,102,114,97,109,101,115,0,0,0,97,117,100,105,111,95,112,114,101,108,111,97,100,0,0,0,97,100,112,99,109,95,52,120,109,0,0,0,0,0,0,0,77,97,120,105,109,117,109,32,102,114,97,103,109,101,110,116,32,100,117,114,97,116,105,111,110,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,105,110,112,117,116,32,102,105,108,101,32,105,110,100,101,120,58,32,37,100,46,10,0,0,115,104,111,119,0,0,0,0,76,101,97,100,32,97,114,116,105,115,116,47,108,101,97,100,32,112,101,114,102,111,114,109,101,114,47,115,111,108,111,105,115,116,0,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,71,85,73,68,32,118,97,108,117,101,32,105,110,32,116,97,103,32,37,115,46,10,0,0,0,0,0,0,115,107,105,112,95,102,114,97,109,101,0,0,0,0,0,0,65,68,80,67,77,32,77,105,99,114,111,115,111,102,116,0,65,108,108,111,119,101,100,32,116,104,114,101,97,100,32,116,121,112,101,115,0,0,0,0,73,110,118,97,108,105,100,32,111,117,116,112,117,116,32,108,105,110,107,32,108,97,98,101,108,58,32,37,115,46,10,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,97,98,99,100,101,102,0,0,115,107,105,112,32,73,68,67,84,47,100,101,113,117,97,110,116,105,122,97,116,105,111,110,32,102,111,114,32,116,104,101,32,115,101,108,101,99,116,101,100,32,102,114,97,109,101,115,0,0,0,0,0,0,0,0,97,100,112,99,109,95,109,115,0,0,0,0,0,0,0,0,93,0,0,0,0,0,0,0,102,105,108,116,101,114,115,0,115,107,105,112,95,105,100,99,116,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,112,97,114,115,101,32,111,112,116,105,111,110,32,118,97,108,117,101,32,34,37,115,34,32,97,115,32,115,97,109,112,108,101,32,102,111,114,109,97,116,10,0,0,0,0,0,65,68,80,67,77,32,73,77,65,32,76,111,107,105,32,83,68,76,32,77,74,80,69,71,0,0,0,0,0,0,0,0,83,121,110,99,32,115,116,114,101,97,109,32,115,112,101,99,105,102,105,99,97,116,105,111,110,32,105,110,32,109,97,112,32,37,115,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,97,110,121,32,115,116,114,101,97,109,115,46,10,0,0,0,0,0,0,0,0,73,102,32,121,111,117,32,119,97,110,116,32,116,111,32,104,101,108,112,44,32,117,112,108,111,97,100,32,97,32,115,97,109,112,108,101,32,111,102,32,116,104,105,115,32,102,105,108,101,32,116,111,32,102,116,112,58,47,47,117,112,108,111,97,100,46,102,102,109,112,101,103,46,111,114,103,47,77,80,108,97,121,101,114,47,105,110,99,111,109,105,110,103,47,32,97,110,100,32,99,111,110,116,97,99,116,32,116,104,101,32,102,102,109,112,101,103,45,100,101,118,101,108,32,109,97,105,108,105,110,103,32,108,105,115,116,46,10,0,0,0,0,0,0,97,118,100,105,115,99,97,114,100,0,0,0,0,0,0,0,97,100,112,99,109,95,105,109,97,95,115,109,106,112,101,103,0,0,0,0,0,0,0,0,97,98,115,40,100,115,116,95,108,105,110,101,115,105,122,101,41,32,62,61,32,98,121,116,101,119,105,100,116,104,0,0,119,97,114,110,105,110,103,0,73,110,118,97,108,105,100,32,115,121,110,99,32,102,105,108,101,32,105,110,100,101,120,58,32,37,100,46,10,0,0,0,115,107,105,112,32,108,111,111,112,32,102,105,108,116,101,114,105,110,103,32,112,114,111,99,101,115,115,32,102,111,114,32,116,104,101,32,115,101,108,101,99,116,101,100,32,102,114,97,109,101,115,0,0,0,0,0,65,68,80,67,77,32,73,77,65,32,87,101,115,116,119,111,111,100,0,0,0,0,0,0,77,105,115,115,105,110,103,32,39,41,39,32,111,114,32,116,111,111,32,109,97,110,121,32,97,114,103,115,32,105,110,32,39,37,115,39,10,0,0,0,109,97,112,99,104,97,110,58,32,105,110,118,97,108,105,100,32,97,117,100,105,111,32,99,104,97,110,110,101,108,32,35,37,100,46,37,100,46,37,100,10,0,0,0,0,0,0,0,73,110,116,101,114,110,97,108,32,98,117,103,44,32,115,104,111,117,108,100,32,110,111,116,32,104,97,118,101,32,104,97,112,112,101,110,101,100,0,0,116,0,0,0,0,0,0,0,115,107,105,112,95,108,111,111,112,95,102,105,108,116,101,114,0,0,0,0,0,0,0,0,97,100,112,99,109,95,105,109,97,95,119,115,0,0,0,0,114,103,98,53,53,53,98,101,0,0,0,0,0,0,0,0,115,117,98,50,118,105,100,101,111,58,32,114,101,99,116,97,110,103,108,101,32,111,118,101,114,102,108,111,119,105,110,103,10,0,0,0,0,0,0,0,68,97,114,107,79,114,99,104,105,100,0,0,0,0,0,0,109,97,112,99,104,97,110,58,32,115,116,114,101,97,109,32,35,37,100,46,37,100,32,105,115,32,110,111,116,32,97,110,32,97,117,100,105,111,32,115,116,114,101,97,109,46,10,0,60,115,116,114,105,110,103,62,0,0,0,0,0,0,0,0,105,102,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,87,76,0,0,0,0,0,0,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,98,105,116,114,97,116,101,32,112,101,110,97,108,116,121,32,99,111,109,112,101,110,115,97,116,105,111,110,32,40,49,46,48,32,61,32,50,53,54,41,0,0,0,0,0,0,41,0,0,0,0,0,0,0,102,105,108,116,101,114,50,83,105,122,101,32,62,32,48,0,65,68,80,67,77,32,73,77,65,32,68,117,99,107,32,68,75,52,0,0,0,0,0,0,115,111,117,114,99,101,32,114,97,110,103,101,0,0,0,0,115,101,116,32,111,117,116,112,117,116,32,99,104,97,110,110,101,108,32,99,111,117,110,116,0,0,0,0,0,0,0,0,109,97,112,99,104,97,110,58,32,105,110,118,97,108,105,100,32,105,110,112,117,116,32,102,105,108,101,32,115,116,114,101,97,109,32,105,110,100,101,120,32,35,37,100,46,37,100,10,0,0,0,0,0,0,0,0,65,117,100,105,111,32,111,98,106,101,99,116,32,116,121,112,101,32,37,115,37,100,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,46,10,0,0,0,0,0,0,0,97,118,99,116,120,45,62,99,111,100,101,99,45,62,101,110,99,111,100,101,50,0,0,0,117,110,115,117,112,112,111,114,116,101,100,32,112,108,97,110,97,114,32,82,71,66,32,99,111,110,118,101,114,115,105,111,110,32,37,115,32,45,62,32,37,115,10,0,0,0,0,0,109,101,112,99,0,0,0,0,48,120,37,108,108,120,0,0,97,100,112,99,109,95,105,109,97,95,100,107,52,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,108,105,98,115,119,115,99,97,108,101,47,115,119,115,99,97,108,101,95,105,110,116,101,114,110,97,108,46,104,0,0,0,108,105,98,115,119,115,99,97,108,101,47,115,119,115,99,97,108,101,95,105,110,116,101,114,110,97,108,46,104,0,0,0,109,97,112,99,104,97,110,58,32,105,110,118,97,108,105,100,32,105,110,112,117,116,32,102,105,108,101,32,105,110,100,101,120,58,32,37,100,10,0,0,117,115,101,32,105,110,116,101,114,110,97,108,32,50,45,112,97,115,115,32,114,97,116,101,99,111,110,116,114,111,108,32,105,110,32,102,105,114,115,116,32,32,112,97,115,115,32,109,111,100,101,0,0,0,0,0,69,110,97,98,108,101,32,83,66,82,32,102,111,114,32,69,76,68,32,40,102,111,114,32,83,66,82,32,105,110,32,111,116,104,101,114,32,99,111,110,102,105,103,117,114,97,116,105,111,110,115,44,32,117,115,101,32,116,104,101,32,45,112,114,111,102,105,108,101,32,112,97,114,97,109,101,116,101,114,41,0,0,0,0,0,0,0,0,109,97,120,105,109,117,109,32,109,97,99,114,111,98,108,111,99,107,32,76,97,103,114,97,110,103,101,32,102,97,99,116,111,114,32,40,86,66,82,41,0,0,0,0,0,0,0,0,37,115,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,97,115,32,105,110,112,117,116,32,112,105,120,101,108,32,102,111,114,109,97,116,10,0,0,0,0,0,0,65,68,80,67,77,32,73,77,65,32,68,117,99,107,32,68,75,51,0,0,0,0,0,0,83,111,114,101,110,115,111,110,32,86,101,99,116,111,114,32,81,117,97,110,116,105,122,101,114,32,49,32,47,32,83,111,114,101,110,115,111,110,32,86,105,100,101,111,32,49,32,47,32,83,86,81,49,0,0,0,83,121,110,116,97,120,32,101,114,114,111,114,44,32,109,97,112,99,104,97,110,32,117,115,97,103,101,58,32,91,102,105,108,101,46,115,116,114,101,97,109,46,99,104,97,110,110,101,108,124,45,49,93,91,58,115,121,110,99,102,105,108,101,58,115,121,110,99,115,116,114,101,97,109,93,10,0,0,0,0,77,101,100,105,116,97,116,105,118,101,0,0,0,0,0,0,102,97,115,116,32,98,105,108,105,110,101,97,114,0,0,0,48,0,0,0,0,0,0,0,105,110,32,45,62,99,104,95,99,111,117,110,116,32,61,61,32,97,118,95,103,101,116,95,99,104,97,110,110,101,108,95,108,97,121,111,117,116,95,110,98,95,99,104,97,110,110,101,108,115,40,115,45,62,32,105,110,95,99,104,95,108,97,121,111,117,116,41,0,0,0,0,37,115,32,35,37,100,44,32,37,115,44,32,37,115,32,39,37,115,39,58,10,0,0,0,82,101,113,117,101,115,116,101,100,32,110,111,105,115,101,32,115,104,97,112,105,110,103,32,100,105,116,104,101,114,32,110,111,116,32,97,118,97,105,108,97,98,108,101,32,97,116,32,116,104,105,115,32,115,97,109,112,108,105,110,103,32,114,97,116,101,44,32,117,115,105,110,103,32,116,114,105,97,110,103,117,108,97,114,32,104,112,32,100,105,116,104,101,114,10,0,109,98,108,109,97,120,0,0,110,117,109,98,101,114,32,111,102,32,102,114,97,109,101,115,32,117,115,101,100,32,116,111,32,112,114,111,98,101,32,102,112,115,0,0,0,0,0,0,112,99,109,95,115,49,54,108,101,0,0,0,0,0,0,0,97,100,112,99,109,95,105,109,97,95,100,107,51,0,0,0,102,114,97,103,95,100,117,114,97,116,105,111,110,0,0,0,37,100,46,37,100,46,37,100,58,37,100,46,37,100,0,0,115,121,110,111,112,115,105,115,0,0,0,0,0,0,0,0,77,101,100,105,97,32,40,101,46,103,46,32,108,97,98,101,108,32,115,105,100,101,32,111,102,32,67,68,41,0,0,0,37,108,108,117,0,0,0,0,109,105,110,105,109,117,109,32,109,97,99,114,111,98,108,111,99,107,32,76,97,103,114,97,110,103,101,32,102,97,99,116,111,114,32,40,86,66,82,41,0,0,0,0,0,0,0,0,65,68,80,67,77,32,73,77,65,32,87,65,86,0,0,0,116,104,114,101,97,100,95,116,121,112,101,0,0,0,0,0,37,100,58,37,100,46,37,100,0,0,0,0,0,0,0,0,111,115,114,0,0,0,0,0,109,98,108,109,105,110,0,0,105,99,99,95,112,97,114,32,105,110,118,97,108,105,100,10,0,0,0,0,0,0,0,0,97,100,112,99,109,95,105,109,97,95,119,97,118,0,0,0,37,115,32,105,115,32,100,101,112,114,101,99,97,116,101,100,44,32,115,101,116,32,116,104,101,32,39,99,114,101,97,116,105,111,110,95,116,105,109,101,39,32,109,101,116,97,100,97,116,97,32,116,97,103,32,105,110,115,116,101,97,100,46,10,0,0,0,0,0,0,0,0,115,104,111,119,32,97,118,97,105,108,97,98,108,101,32,112,114,111,116,111,99,111,108,115,0,0,0,0,0,0,0,0,110,101,101,100,101,100,32,37,100,32,104,97,100,32,37,100,10,0,0,0,0,0,0,0,33,97,102,113,45,62,102,114,97,109,101,95,99,111,117,110,116,0,0,0,0,0,0,0,105,110,99,114,101,97,115,101,32,116,104,101,32,113,117,97,110,116,105,122,101,114,32,102,111,114,32,109,97,99,114,111,98,108,111,99,107,115,32,99,108,111,115,101,32,116,111,32,98,111,114,100,101,114,115,0,65,68,80,67,77,32,73,77,65,32,81,117,105,99,107,84,105,109,101,0,0,0,0,0,99,114,101,97,116,105,111,110,95,116,105,109,101,61,37,70,84,37,84,37,122,0,0,0,98,111,114,100,101,114,95,109,97,115,107,0,0,0,0,0,97,100,112,99,109,95,105,109,97,95,113,116,0,0,0,0,101,114,114,111,114,0,0,0,102,114,97,109,101,115,58,100,0,0,0,0,0,0,0,0,40,112,117,116,95,98,105,116,115,95,99,111,117,110,116,40,38,115,45,62,112,98,41,32,38,32,55,41,32,61,61,32,48,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,108,97,115,116,32,102,114,97,109,101,32,98,105,116,32,111,102,102,115,101,116,32,37,100,32,62,32,98,117,102,32,115,105,122,101,32,37,100,32,40,37,100,41,10,0,0,0,0,87,114,111,110,103,32,112,97,99,107,101,116,32,116,121,112,101,32,105,110,32,83,101,116,117,112,32,104,101,97,100,101,114,10,0,0,0,0,0,0,102,114,97,109,101,32,115,107,105,112,32,99,111,109,112,97,114,101,32,102,117,110,99,116,105,111,110,0,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,56,45,98,105,116,32,112,108,97,110,97,114,0,70,97,105,108,101,100,32,116,111,32,111,112,101,110,32,112,114,111,103,114,101,115,115,32,85,82,76,32,34,37,115,34,58,32,37,115,10,0,0,0,115,107,105,112,99,109,112,0,112,99,109,95,115,56,95,112,108,97,110,97,114,0,0,0,114,103,98,53,54,53,108,101,0,0,0,0,0,0,0,0,115,117,98,50,118,105,100,101,111,58,32,110,111,110,45,98,105,116,109,97,112,32,115,117,98,116,105,116,108,101,10,0,68,97,114,107,111,114,97,110,103,101,0,0,0,0,0,0,112,105,112,101,58,0,0,0,60,102,108,111,97,116,62,0,103,99,100,0,0,0,0,0,84,111,111,32,109,97,110,121,32,99,104,97,110,110,101,108,115,58,32,37,100,46,10,0,100,111,119,110,109,105,120,32,114,105,103,104,116,0,0,0,102,114,97,109,101,32,115,107,105,112,32,101,120,112,111,110,101,110,116,0,0,0,0,0,102,105,108,116,101,114,83,105,122,101,32,62,32,48,0,0,83,77,80,84,69,32,51,48,50,77,0,0,0,0,0,0,115,114,99,95,114,97,110,103,101,0,0,0,0,0,0,0,105,110,95,99,104,97,110,110,101,108,95,99,111,117,110,116,0,0,0,0,0,0,0,0,45,0,0,0,0,0,0,0,77,111,114,101,32,116,104,97,110,32,111,110,101,32,65,65,67,32,82,68,66,32,112,101,114,32,65,68,84,83,32,102,114,97,109,101,0,0,0,0,115,107,105,112,95,101,120,112,0,0,0,0,0,0,0,0,97,108,108,95,99,104,97,110,110,101,108,95,99,111,117,110,116,115,0,0,0,0,0,0,115,51,48,50,109,0,0,0,85,110,107,110,111,119,110,32,116,97,114,103,101,116,58,32,37,115,10,0,0,0,0,0,112,97,115,115,49,0,0,0,101,108,100,95,115,98,114,0,102,114,97,109,101,32,115,107,105,112,32,102,97,99,116,111,114,0,0,0,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,50,48,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,32,112,108,97,110,97,114,0,0,115,118,113,49,0,0,0,0,121,117,118,52,49,49,112,0,83,112,97,99,101,0,0,0,115,107,105,112,95,102,97,99,116,111,114,0,0,0,0,0,102,112,115,112,114,111,98,101,115,105,122,101,0,0,0,0,112,99,109,95,108,120,102,0,105,111,100,115,32,118,105,100,101,111,32,112,114,111,102,105,108,101,32,97,116,111,109,46,0,0,0,0,0,0,0,0,121,117,118,52,50,50,112,0,100,101,115,99,114,105,112,116,105,111,110,0,0,0,0,0,76,101,97,102,108,101,116,32,112,97,103,101,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,98,121,116,101,32,97,114,114,97,121,32,105,110,32,116,97,103,32,37,115,46,10,0,0,0,0,0,0,102,114,97,109,101,32,115,107,105,112,32,116,104,114,101,115,104,111,108,100,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,49,54,124,50,48,124,50,52,45,98,105,116,32,98,105,103,45,101,110,100,105,97,110,32,102,111,114,32,66,108,117,45,114,97,121,32,109,101,100,105,97,0,0,0,0,65,86,70,105,108,116,101,114,0,0,0,0,0,0,0,0,100,118,53,48,0,0,0,0,115,107,105,112,95,116,104,114,101,115,104,111,108,100,0,0,112,99,109,95,98,108,117,114,97,121,0,0,0,0,0,0,58,37,115,0,0,0,0,0,100,118,0,0,0,0,0,0,112,114,111,116,111,99,111,108,115,0,0,0,0,0,0,0,100,101,99,111,100,101,32,97,116,32,49,61,32,49,47,50,44,32,50,61,49,47,52,44,32,51,61,49,47,56,32,114,101,115,111,108,117,116,105,111,110,115,0,0,0,0,0,0,80,67,77,32,54,52,45,98,105,116,32,102,108,111,97,116,105,110,103,32,112,111,105,110,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,0,72,101,97,100,101,114,32,109,105,115,115,105,110,103,10,0,52,56,48,48,48,0,0,0])
.concat([65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,37,100,46,32,118,111,114,98,105,115,32,104,101,97,100,101,114,32,100,97,109,97,103,101,100,10,0,0,0,0,0,0,108,97,109,101,58,32,111,117,116,112,117,116,32,98,117,102,102,101,114,32,116,111,111,32,115,109,97,108,108,32,40,98,117,102,102,101,114,32,105,110,100,101,120,58,32,37,100,44,32,102,114,101,101,32,98,121,116,101,115,58,32,37,100,41,10,0,0,0,0,0,0,0,108,111,119,114,101,115,0,0,112,99,109,95,102,54,52,108,101,0,0,0,0,0,0,0,102,97,116,97,108,0,0,0,52,52,56,48,48,48,0,0,73,110,118,97,108,105,100,32,104,97,110,100,108,101,0,0,108,101,118,101,108,0,0,0,66,105,116,100,101,112,116,104,32,103,114,101,97,116,101,114,32,116,104,97,110,32,56,10,0,0,0,0,0,0,0,0,80,67,77,32,54,52,45,98,105,116,32,102,108,111,97,116,105,110,103,32,112,111,105,110,116,32,98,105,103,45,101,110,100,105,97,110,0,0,0,0,49,48,48,56,48,48,48,48,0,0,0,0,0,0,0,0,100,116,115,95,104,100,95,109,97,0,0,0,0,0,0,0,112,99,109,95,102,54,52,98,101,0,0,0,0,0,0,0,114,103,98,53,54,53,98,101,0,0,0,0,0,0,0,0,73,109,112,111,115,115,105,98,108,101,32,116,111,32,103,101,116,32,97,32,98,108,97,110,107,32,99,97,110,118,97,115,46,10,0,0,0,0,0,0,68,97,114,107,79,108,105,118,101,71,114,101,101,110,0,0,50,48,52,56,0,0,0,0,60,100,111,117,98,108,101,62,0,0,0,0,0,0,0,0,104,121,112,111,116,0,0,0,68,82,0,0,0,0,0,0,100,116,115,95,104,100,95,104,114,97,0,0,0,0,0,0,48,0,0,0,0,0,0,0,80,67,77,32,51,50,45,98,105,116,32,102,108,111,97,116,105,110,103,32,112,111,105,110,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,0,100,101,115,116,105,110,97,116,105,111,110,32,102,111,114,109,97,116,0,0,0,0,0,0,115,101,116,32,105,110,112,117,116,32,99,104,97,110,110,101,108,32,99,111,117,110,116,0,57,48,48,48,48,48,48,0,105,110,118,97,108,105,100,32,100,101,102,97,117,108,116,32,99,104,97,110,110,101,108,32,99,111,110,102,105,103,117,114,97,116,105,111,110,32,40,37,100,41,10,0,0,0,0,0,100,116,115,95,57,54,95,50,52,0,0,0,0,0,0,0,97,98,117,102,102,101,114,115,105,110,107,0,0,0,0,0,112,99,109,95,102,51,50,108,101,0,0,0,0,0,0,0,54,48,48,48,48,48,48,0,105,110,112,117,116,95,112,114,101,115,101,114,118,101,100,0,65,102,116,101,114,98,117,114,110,101,114,32,40,105,109,112,114,111,118,101,100,32,113,117,97,108,105,116,121,41,0,0,100,116,115,95,101,115,0,0,80,67,77,32,51,50,45,98,105,116,32,102,108,111,97,116,105,110,103,32,112,111,105,110,116,32,98,105,103,45,101,110,100,105,97,110,0,0,0,0,70,76,86,32,47,32,83,111,114,101,110,115,111,110,32,83,112,97,114,107,32,47,32,83,111,114,101,110,115,111,110,32,72,46,50,54,51,32,40,70,108,97,115,104,32,86,105,100,101,111,41,0,0,0,0,0,55,50,48,120,52,56,48,0,80,117,110,107,0,0,0,0,116,105,116,108,101,0,0,0,100,116,115,0,0,0,0,0,109,97,120,105,109,117,109,32,109,117,120,105,110,103,32,111,114,32,100,101,109,117,120,105,110,103,32,100,101,108,97,121,32,105,110,32,109,105,99,114,111,115,101,99,111,110,100,115,0,0,0,0,0,0,0,0,112,99,109,95,102,51,50,98,101,0,0,0,0,0,0,0,55,50,48,120,53,55,54,0,105,111,100,115,95,118,105,100,101,111,95,112,114,111,102,105,108,101,0,0,0,0,0,0,108,111,99,97,116,105,111,110,0,0,0,0,0,0,0,0,67,111,118,101,114,32,40,98,97,99,107,41,0,0,0,0,109,112,101,103,118,105,100,101,111,95,120,118,109,99,0,0,73,68,51,0,0,0,0,0,109,112,101,103,50,95,97,97,99,95,104,101,0,0,0,0,115,105,122,101,60,61,48,120,55,102,102,102,102,102,102,102,32,38,38,32,112,32,45,32,112,107,116,45,62,100,97,116,97,32,62,61,32,115,105,122,101,0,0,0,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,50,48,124,50,52,45,98,105,116,32,98,105,103,45,101,110,100,105,97,110,0,79,112,116,105,111,110,32,39,37,115,39,32,110,111,116,32,102,111,117,110,100,10,0,0,97,99,51,0,0,0,0,0,99,104,37,100,0,0,0,0,109,112,101,103,50,95,97,97,99,95,108,111,119,0,0,0,112,99,109,95,100,118,100,0,100,118,100,0,0,0,0,0,115,104,111,119,32,97,118,97,105,108,97,98,108,101,32,98,105,116,32,115,116,114,101,97,109,32,102,105,108,116,101,114,115,0,0,0,0,0,0,0,97,97,99,95,101,108,100,0,80,67,77,32,115,105,103,110,101,100,32,51,50,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,32,112,108,97,110,97,114,0,0,68,105,115,99,111,0,0,0,73,110,118,97,108,105,100,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,115,105,122,101,58,32,37,100,46,10,0,0,0,0,0,49,0,0,0,0,0,0,0,65,117,116,104,111,114,0,0,97,97,99,95,108,100,0,0,112,99,109,95,115,51,50,108,101,95,112,108,97,110,97,114,0,0,0,0,0,0,0,0,112,97,110,105,99,0,0,0,84,111,111,32,109,97,110,121,32,116,97,103,32,102,105,101,108,100,115,32,40,37,100,41,10,0,0,0,0,0,0,0,115,99,97,110,95,111,102,102,115,101,116,0,0,0,0,0,102,111,117,110,100,32,110,111,32,39,102,109,116,32,39,32,116,97,103,32,98,101,102,111,114,101,32,116,104,101,32,39,100,97,116,97,39,32,116,97,103,10,0,0,0,0,0,0,68,73,83,67,78,85,77,66,69,82,0,0,0,0,0,0,97,97,99,95,104,101,95,118,50,0,0,0,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,50,52,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,32,112,108,97,110,97,114,0,0,49,56,51,53,48,48,56,0,97,97,99,95,104,101,0,0,112,99,109,95,115,50,52,108,101,95,112,108,97,110,97,114,0,0,0,0,0,0,0,0,114,103,98,52,56,108,101,0,83,117,98,116,105,116,108,101,32,101,110,99,111,100,105,110,103,32,102,97,105,108,101,100,10,0,0,0,0,0,0,0,68,97,114,107,77,97,103,101,110,116,97,0,0,0,0,0,48,0,0,0,0,0,0,0,60,105,110,116,54,52,62,0,114,97,110,100,111,109,0,0,100,111,119,110,109,105,120,32,108,101,102,116,0,0,0,0,97,97,99,95,108,116,112,0,108,105,98,115,119,115,99,97,108,101,47,115,119,115,99,97,108,101,95,105,110,116,101,114,110,97,108,46,104,0,0,0,80,67,77,32,115,105,103,110,101,100,32,49,54,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,32,112,108,97,110,97,114,0,0,100,115,116,95,102,111,114,109,97,116,0,0,0,0,0,0,83,87,82,101,115,97,109,112,108,101,114,0,0,0,0,0,50,53,49,54,48,48,48,0,115,116,101,114,101,111,32,119,105,116,104,32,83,67,69,10,0,0,0,0,0,0,0,0,73,67,79,80,0,0,0,0,80,114,111,118,105,100,101,100,32,112,97,99,107,101,116,32,105,115,32,116,111,111,32,115,109,97,108,108,44,32,110,101,101,100,115,32,116,111,32,98,101,32,37,100,10,0,0,0,97,97,99,95,115,115,114,0,111,117,116,112,117,116,32,115,116,114,101,97,109,32,37,100,58,37,100,0,0,0,0,0,112,99,109,95,115,49,54,108,101,95,112,108,97,110,97,114,0,0,0,0,0,0,0,0,50,48,52,48,48,48,48,0,97,108,119,97,121,115,32,116,114,121,32,97,32,109,98,32,119,105,116,104,32,109,118,61,60,48,44,48,62,0,0,0,97,102,116,101,114,98,117,114,110,101,114,0,0,0,0,0,112,114,111,98,101,115,105,122,101,0,0,0,0,0,0,0,37,116,105,32,98,121,116,101,115,32,111,102,32,99,111,109,109,101,110,116,32,104,101,97,100,101,114,32,114,101,109,97,105,110,10,0,0,0,0,0,97,97,99,95,108,111,119,0,83,101,114,105,97,108,32,110,117,109,98,101,114,32,105,110,32,102,105,115,98,111,110,101,32,100,111,101,115,110,39,116,32,109,97,116,99,104,32,97,110,121,32,115,116,114,101,97,109,10,0,0,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,49,54,45,98,105,116,32,98,105,103,45,101,110,100,105,97,110,32,112,108,97,110,97,114,0,0,0,0,0,102,108,118,49,0,0,0,0,121,117,118,52,50,48,112,0,83,111,117,108,0,0,0,0,110,111,110,45,0,0,0,0,115,45,62,115,116,114,101,97,109,115,91,32,115,45,62,110,98,95,115,116,114,101,97,109,115,45,49,32,93,32,61,61,32,115,116,0,0,0,0,0,97,97,99,95,109,97,105,110,0,0,0,0,0,0,0,0,109,97,120,95,100,101,108,97,121,0,0,0,0,0,0,0,112,99,109,95,115,49,54,98,101,95,112,108,97,110,97,114,0,0,0,0,0,0,0,0,69,110,99,111,100,101,114,32,100,105,100,32,110,111,116,32,112,114,111,100,117,99,101,32,112,114,111,112,101,114,32,112,116,115,44,32,109,97,107,105,110,103,32,115,111,109,101,32,117,112,46,10,0,0,0,0,52,56,48,120,52,56,48,0,105,111,100,115,32,97,117,100,105,111,32,112,114,111,102,105,108,101,32,97,116,111,109,46,0,0,0,0,0,0,0,0,109,111,100,101,108,0,0,0,67,111,118,101,114,32,40,102,114,111,110,116,41,0,0,0,87,77,47,80,105,99,116,117,114,101,0,0,0,0,0,0,117,110,107,110,111,119,110,0,80,67,77,32,90,111,114,107,0,0,0,0,0,0,0,0,100,101,102,97,117,108,116,32,97,114,101,115,97,109,112,108,101,32,102,105,108,116,101,114,32,111,112,116,105,111,110,115,0,0,0,0,0,0,0,0,83,101,116,116,105,110,103,32,39,37,115,39,32,116,111,32,118,97,108,117,101,32,39,37,115,39,10,0,0,0,0,0,52,56,48,120,53,55,54,0,112,114,111,102,105,108,101,0,112,99,109,95,122,111,114,107,0,0,0,0,0,0,0,0,109,112,101,103,50,118,105,100,101,111,0,0,0,0,0,0,98,115,102,115,0,0,0,0,109,111,118,0,0,0,0,0,110,117,109,98,101,114,32,111,102,32,109,97,99,114,111,98,108,111,99,107,32,114,111,119,115,32,97,116,32,116,104,101,32,98,111,116,116,111,109,32,119,104,105,99,104,32,97,114,101,32,115,107,105,112,112,101,100,0,0,0,0,0,0,0,80,67,77,32,68,45,67,105,110,101,109,97,32,97,117,100,105,111,32,115,105,103,110,101,100,32,50,52,45,98,105,116,0,0,0,0,0,0,0,0,115,118,99,100,0,0,0,0,67,111,110,99,97,116,101,110,97,116,101,100,32,72,46,50,54,52,32,109,105,103,104,116,32,110,111,116,32,112,108,97,121,32,99,111,114,114,101,110,116,108,121,46,10,0,0,0,115,107,105,112,95,98,111,116,116,111,109,0,0,0,0,0,112,99,109,95,115,50,52,100,97,117,100,0,0,0,0,0,113,117,105,101,116,0,0,0,49,52,49,49,50,48,48,0,110,117,109,98,101,114,32,111,102,32,109,97,99,114,111,98,108,111,99,107,32,114,111,119,115,32,97,116,32,116,104,101,32,116,111,112,32,119,104,105,99,104,32,97,114,101,32,115,107,105,112,112,101,100,0,0,80,67,77,32,117,110,115,105,103,110,101,100,32,50,52,45,98,105,116,32,98,105,103,45,101,110,100,105,97,110,0,0,84,67,79,78,0,0,0,0,109,117,120,114,97,116,101,0,115,107,105,112,95,116,111,112,0,0,0,0,0,0,0,0,83,116,97,116,105,115,116,105,99,115,58,32,37,100,32,115,101,101,107,115,44,32,37,100,32,119,114,105,116,101,111,117,116,115,10,0,0,0,0,0,112,99,109,95,117,50,52,98,101,0,0,0,0,0,0,0,114,103,98,52,56,98,101,0,83,117,98,116,105,116,108,101,32,112,97,99,107,101,116,115,32,109,117,115,116,32,104,97,118,101,32,97,32,112,116,115,10,0,0,0,0,0,0,0,104,116,116,112,115,0,0,0,68,97,114,107,75,104,97,107,105,0,0,0,0,0,0,0,116,105,116,108,101,0,0,0,50,51,50,52,0,0,0,0,60,105,110,116,62,0,0,0,112,114,105,110,116,0,0,0,68,76,0,0,0,0,0,0,110,115,115,101,32,119,101,105,103,104,116,0,0,0,0,0,97,115,102,95,115,116,0,0,100,101,115,99,0,0,0,0,80,67,77,32,117,110,115,105,103,110,101,100,32,50,52,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,0,0,0,0,0,0,0,115,111,117,114,99,101,32,102,111,114,109,97,116,0,0,0,115,114,99,45,62,118,105,100,101,111,0,0,0,0,0,0,115,45,62,100,105,116,104,101,114,46,110,111,105,115,101,46,99,104,95,99,111,117,110,116,32,61,61,32,112,114,101,111,117,116,45,62,99,104,95,99,111,117,110,116,0,0,0,0,112,97,99,107,101,116,115,105,122,101,0,0,0,0,0,0,109,111,110,111,32,119,105,116,104,32,67,80,69,10,0,0,111,115,102,0,0,0,0,0,110,98,95,115,97,109,112,108,101,115,32,40,37,100,41,32,33,61,32,102,114,97,109,101,95,115,105,122,101,32,40,37,100,41,32,40,97,118,99,111,100,101,99,95,101,110,99,111,100,101,95,97,117,100,105,111,50,41,10,0,0,0,0,0,69,114,114,111,114,32,112,97,114,115,105,110,103,32,115,97,109,112,108,101,32,102,111,114,109,97,116,58,32,37,115,46,10,0,0,0,0,0,0,0,110,115,115,101,119,0,0,0,65,32,39,124,39,45,115,101,112,97,114,97,116,101,100,32,108,105,115,116,32,111,102,32,112,105,120,101,108,32,102,111,114,109,97,116,115,0,0,0,105,115,116,0,0,0,0,0,112,99,109,95,117,50,52,108,101,0,0,0,0,0,0,0,84,105,109,101,115,116,97,109,112,32,105,110,32,115,101,99,111,110,100,115,32,111,102,32,116,104,101,32,102,105,114,115,116,32,102,114,97,109,101,32,116,104,97,116,32,115,104,111,117,108,100,32,98,101,32,112,97,115,115,101,100,0,0,0,50,0,0,0,0,0,0,0,109,118,48,0,0,0,0,0,69,76,68,0,0,0,0,0,73,78,84,69,82,76,65,67,69,68,0,0,0,0,0,0,105,110,116,114,97,95,100,99,95,112,114,101,99,105,115,105,111,110,0,0,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,50,52,45,98,105,116,32,98,105,103,45,101,110,100,105,97,110,0,0,0,0,73,110,116,101,108,32,72,46,50,54,51,0,0,0,0,0,52,52,49,48,48,0,0,0,66,97,115,115,0,0,0,0,100,101,102,97,117,108,116,0,115,45,62,110,98,95,115,116,114,101,97,109,115,62,48,0,100,99,0,0,0,0,0,0,116,115,0,0,0,0,0,0,112,99,109,95,115,50,52,98,101,0,0,0,0,0,0,0,66,117,102,102,101,114,32,97,117,100,105,111,32,102,114,97,109,101,115,44,32,97,110,100,32,109,97,107,101,32,116,104,101,109,32,97,99,99,101,115,115,105,98,108,101,32,116,111,32,116,104,101,32,102,105,108,116,101,114,99,104,97,105,110,46,0,0,0,0,0,0,0,50,50,52,48,48,48,0,0,105,111,100,115,95,97,117,100,105,111,95,112,114,111,102,105,108,101,0,0,0,0,0,0,109,97,107,101,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,79,116,104,101,114,32,102,105,108,101,32,105,99,111,110,0,108,97,110,103,117,97,103,101,0,0,0,0,0,0,0,0,109,97,99,114,111,98,108,111,99,107,32,116,104,114,101,115,104,111,108,100,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,50,52,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,97,114,101,115,97,109,112,108,101,95,115,119,114,95,111,112,116,115,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,112,97,114,115,101,32,39,37,115,39,58,32,37,115,10,0,0,0,0,0,0,0,98,58,97,0,0,0,0,0,105,109,97,103,101,47,106,112,101,103,0,0,0,0,0,0,109,98,95,116,104,114,101,115,104,111,108,100,0,0,0,0,97,117,116,111,45,105,110,115,101,114,116,105,110,103,32,102,105,108,116,101,114,32,39,37,115,39,32,98,101,116,119,101,101,110,32,116,104,101,32,102,105,108,116,101,114,32,39,37,115,39,32,97,110,100,32,116,104,101,32,102,105,108,116,101,114,32,39,37,115,39,10,0,57,48,37,37,32,32,76,65,77,69,32,99,111,109,112,105,108,101,100,32,119,105,116,104,32,98,117,103,103,121,32,118,101,114,115,105,111,110,32,111,102,32,103,99,99,32,117,115,105,110,103,32,97,100,118,97,110,99,101,100,32,111,112,116,105,109,105,122,97,116,105,111,110,115,0,0,0,0,0,0,112,99,109,95,115,50,52,108,101,0,0,0,0,0,0,0,51,50,55,54,56,48,0,0,115,104,111,119,32,97,118,97,105,108,97,98,108,101,32,101,110,99,111,100,101,114,115,0,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,116,104,114,101,115,104,111,108,100,0,0,0,0,0,80,67,77,32,117,110,115,105,103,110,101,100,32,51,50,45,98,105,116,32,98,105,103,45,101,110,100,105,97,110,0,0,98,117,102,115,105,122,101,0,109,101,95,116,104,114,101,115,104,111,108,100,0,0,0,0,112,99,109,95,117,51,50,98,101,0,0,0,0,0,0,0,70,105,110,105,115,104,101,100,32,115,112,108,105,116,116,105,110,103,32,116,104,101,32,99,111,109,109,97,110,100,108,105,110,101,46,10,0,0,0,0,109,105,110,114,97,116,101,0,97,117,116,111,100,101,116,101,99,116,32,97,32,115,117,105,116,97,98,108,101,32,110,117,109,98,101,114,32,111,102,32,116,104,114,101,97,100,115,32,116,111,32,117,115,101,0,0,80,67,77,32,117,110,115,105,103,110,101,100,32,51,50,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,0,0,0,0,0,0,0,109,97,120,114,97,116,101,0,116,104,114,101,97,100,115,0,112,99,109,95,117,51,50,108,101,0,0,0,0,0,0,0,118,100,112,97,117,95,118,99,49,0,0,0,0,0,0,0,83,117,98,116,105,116,108,101,32,100,117,114,97,116,105,111,110,32,114,101,100,117,99,101,100,32,102,114,111,109,32,37,100,32,116,111,32,37,100,10,0,0,0,0,0,0,0,0,68,97,114,107,71,114,101,101,110,0,0,0,0,0,0,0,49,49,53,48,48,48,48,0,60,102,108,97,103,115,62,0,112,111,119,0,0,0,0,0,116,111,112,32,98,97,99,107,32,114,105,103,104,116,0,0,101,114,114,111,114,0,0,0,80,67,77,32,115,105,103,110,101,100,32,51,50,45,98,105,116,32,98,105,103,45,101,110,100,105,97,110,0,0,0,0,115,114,99,95,102,111,114,109,97,116,0,0,0,0,0,0,115,45,62,105,110,46,112,108,97,110,97,114,0,0,0,0,49,56,0,0,0,0,0,0,78,117,109,98,101,114,32,111,102,32,98,97,110,100,115,32,40,37,100,41,32,101,120,99,101,101,100,115,32,108,105,109,105,116,32,40,37,100,41,46,10,0,0,0,0,0,0,0,109,111,114,101,32,115,97,109,112,108,101,115,32,116,104,97,110,32,102,114,97,109,101,32,115,105,122,101,32,40,97,118,99,111,100,101,99,95,101,110,99,111,100,101,95,97,117,100,105,111,50,41,10,0,0,0,110,117,109,98,101,114,32,111,102,32,98,105,116,115,32,119,104,105,99,104,32,115,104,111,117,108,100,32,98,101,32,108,111,97,100,101,100,32,105,110,116,111,32,116,104,101,32,114,99,32,98,117,102,102,101,114,32,98,101,102,111,114,101,32,100,101,99,111,100,105,110,103,32,115,116,97,114,116,115,0,67,97,110,110,111,116,32,102,105,110,100,32,97,32,109,97,116,99,104,105,110,103,32,115,116,114,101,97,109,32,102,111,114,32,117,110,108,97,98,101,108,101,100,32,105,110,112,117,116,32,112,97,100,32,37,100,32,111,110,32,102,105,108,116,101,114,32,37,115,10,0,0,112,99,109,95,115,51,50,98,101,0,0,0,0,0,0,0,49,53,0,0,0,0,0,0,117,115,101,32,103,109,99,0,76,68,0,0,0,0,0,0,114,99,95,105,110,105,116,95,111,99,99,117,112,97,110,99,121,0,0,0,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,51,50,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,0,104,50,54,51,105,0,0,0,103,0,0,0,0,0,0,0,65,108,116,101,114,110,82,111,99,107,0,0,0,0,0,0,70,105,108,101,32,112,111,115,105,116,105,111,110,32,97,102,116,101,114,32,97,118,102,111,114,109,97,116,95,102,105,110,100,95,115,116,114,101,97,109,95,105,110,102,111,40,41,32,105,115,32,37,108,108,100,10,0,0,0,0,0,0,0,0,110,111,105,115,101,32,114,101,100,117,99,116,105,111,110,0,112,114,105,110,116,32,115,112,101,99,105,102,105,99,32,100,101,98,117,103,32,105,110,102,111,0,0,0,0,0,0,0,112,99,109,95,115,51,50,108,101,0,0,0,0,0,0,0,51,53,50,120,50,52,48,0,83,107,105,112,32,119,114,105,116,105,110,103,32,105,111,100,115,32,97,116,111,109,46,0,101,110,99,111,100,101,114,0,51,50,120,51,50,32,112,105,120,101,108,115,32,39,102,105,108,101,32,105,99,111,110,39,0,0,0,0,0,0,0,0,103,112,111,115,32,109,105,115,109,97,116,99,104,32,111,117,114,32,112,111,115,61,37,108,108,117,44,32,101,110,100,61,37,108,108,100,10,0,0,0,110,114,0,0,0,0,0,0,80,67,77,32,65,45,108,97,119,32,47,32,71,46,55,49,49,32,65,45,108,97,119,0,100,101,102,97,117,108,116,32,115,99,97,108,101,32,102,105,108,116,101,114,32,111,112,116,105,111,110,115,0,0,0,0,78,111,32,111,112,116,105,111,110,32,110,97,109,101,32,110,101,97,114,32,39,37,115,39,10,0,0,0,0,0,0,0,51,53,50,120,50,56,56,0,109,97,120,105,109,117,109,32,76,97,103,114,97,110,103,101,32,102,97,99,116,111,114,32,40,86,66,82,41,0,0,0,112,99,109,95,97,108,97,119,0,0,0,0,0,0,0,0,109,112,50,0,0,0,0,0,101,110,99,111,100,101,114,115,0,0,0,0,0,0,0,0,80,114,105,110,116,32,104,101,108,112,32,47,32,105,110,102,111,114,109,97,116,105,111,110,32,47,32,99,97,112,97,98,105,108,105,116,105,101,115,58,0,0,0,0,0,0,0,0,108,109,97,120,0,0,0,0,80,67,77,32,109,117,45,108,97,119,32,47,32,71,46,55,49,49,32,109,117,45,108,97,119,0,0,0,0,0,0,0,109,105,110,105,109,117,109,32,76,97,103,114,97,110,103,101,32,102,97,99,116,111,114,32,40,86,66,82,41,0,0,0,112,99,109,95,109,117,108,97,119,0,0,0,0,0,0,0,84,114,97,105,108,105,110,103,32,111,112,116,105,111,110,115,32,119,101,114,101,32,102,111,117,110,100,32,111,110,32,116,104,101,32,99,111,109,109,97,110,100,108,105,110,101,46,10,0,0,0,0,0,0,0,0,109,112,101,103,49,118,105,100,101,111,0,0,0,0,0,0,108,109,105,110,0,0,0,0,80,67,77,32,117,110,115,105,103,110,101,100,32,56,45,98,105,116,0,0,0,0,0,0,73,110,99,111,109,112,97,116,105,98,108,101,32,112,105,120,101,108,32,102,111,114,109,97,116,32,39,37,115,39,32,102,111,114,32,99,111,100,101,99,32,39,37,115,39,44,32,97,117,116,111,45,115,101,108,101,99,116,105,110,103,32,102,111,114,109,97,116,32,39,37,115,39,10,0,0,0,0,0,0,115,99,101,110,101,32,99,104,97,110,103,101,32,116,104,114,101,115,104,111,108,100,0,0,112,99,109,95,117,56,0,0,118,100,112,97,117,95,119,109,118,51,0,0,0,0,0,0,77,117,108,116,105,112,108,101,32,102,114,97,109,101,115,32,105,110,32,97,32,112,97,99,107,101,116,32,102,114,111,109,32,115,116,114,101,97,109,32,37,100,10,0,0,0,0,0,68,97,114,107,71,114,97,121,0,0,0,0,0,0,0,0,118,99,100,0,0,0,0,0,37,45,49,50,115,32,0,0,110,111,116,0,0,0,0,0,84,66,82,0,0,0,0,0,115,99,95,116,104,114,101,115,104,111,108,100,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,56,45,98,105,116,0,0,0,0,0,0,0,0,100,101,115,116,105,110,97,116,105,111,110,32,104,101,105,103,104,116,0,0,0,0,0,0,115,45,62,109,105,100,98,117,102,46,99,104,95,99,111,117,110,116,32,61,61,32,115,45,62,111,117,116,46,99,104,95,99,111,117,110,116,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,111,114,32,115,101,116,32,97,32,102,114,97,109,101,114,97,116,101,32,119,105,116,104,32,34,45,114,32,120,120,120,34,46,10,0,0,0,0,0,0,100,101,99,111,100,101,95,98,97,110,100,95,116,121,112,101,115,58,32,73,110,112,117,116,32,98,117,102,102,101,114,32,101,120,104,97,117,115,116,101,100,32,98,101,102,111,114,101,32,69,78,68,32,101,108,101,109,101,110,116,32,102,111,117,110,100,10,0,0,0,0,0,101,120,116,101,110,100,101,100,95,100,97,116,97,32,105,115,32,110,111,116,32,115,101,116,46,10,0,0,0,0,0,0,115,116,114,101,97,109,95,99,111,100,101,99,95,116,97,103,0,0,0,0,0,0,0,0,83,116,114,101,97,109,32,115,112,101,99,105,102,105,101,114,32,39,37,115,39,32,105,110,32,102,105,108,116,101,114,103,114,97,112,104,32,100,101,115,99,114,105,112,116,105,111,110,32,37,115,32,109,97,116,99,104,101,115,32,110,111,32,115,116,114,101,97,109,115,46,10,0,0,0,0,0,0,0,0,112,99,109,95,115,56,0,0,80,108,101,97,115,101,32,112,114,101,102,105,120,32,116,97,114,103,101,116,32,119,105,116,104,32,34,112,97,108,45,34,44,32,34,110,116,115,99,45,34,32,111,114,32,34,102,105,108,109,45,34,44,10,0,0,103,109,99,0,0,0,0,0,72,69,45,65,65,67,118,50,0,0,0,0,0,0,0,0,117,115,101,32,98,101,115,116,32,114,97,116,101,32,100,105,115,116,111,114,116,105,111,110,0,0,0,0,0,0,0,0,80,67,77,32,117,110,115,105,103,110,101,100,32,49,54,45,98,105,116,32,98,105,103,45,101,110,100,105,97,110,0,0,72,46,50,54,51,43,32,47,32,72,46,50,54,51,45,49,57,57,56,32,47,32,72,46,50,54,51,32,118,101,114,115,105,111,110,32,50,0,0,0,67,111,117,108,100,32,110,111,116,32,100,101,116,101,114,109,105,110,101,32,110,111,114,109,32,40,80,65,76,47,78,84,83,67,47,78,84,83,67,45,70,105,108,109,41,32,102,111,114,32,116,97,114,103,101,116,46,10,0,0,0,0,0,0,78,111,105,115,101,0,0,0,67,111,117,108,100,32,110,111,116,32,102,105,110,100,32,99,111,100,101,99,32,112,97,114,97,109,101,116,101,114,115,32,102,111,114,32,115,116,114,101,97,109,32,37,100,32,40,37,115,41,58,32,37,115,10,67,111,110,115,105,100,101,114,32,105,110,99,114,101,97,115,105,110,103,32,116,104,101,32,118,97,108,117,101,32,102,111,114,32,116,104,101,32,39,97,110,97,108,121,122,101,100,117,114,97,116,105,111,110,39,32,97,110,100,32,39,112,114,111,98,101,115,105,122,101,39,32,111,112,116,105,111,110,115,10,0,117,115,101,32,102,101,119,101,115,116,32,98,105,116,115,0,102,100,101,98,117,103,0,0,112,99,109,95,117,49,54,98,101,0,0,0,0,0,0,0,78,84,83,67,0,0,0,0,115,107,105,112,95,105,111,100,115,0,0,0,0,0,0,0,103,101,110,114,101,0,0,0,79,116,104,101,114,0,0,0,68,105,103,105,116,97,108,32,115,105,103,110,97,116,117,114,101,32,100,101,116,101,99,116,101,100,33,10,0,0,0,0,98,105,116,115,0,0,0,0,80,67,77,32,117,110,115,105,103,110,101,100,32,49,54,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,0,0,0,0,0,0,0,115,99,97,108,101,95,115,119,115,95,111,112,116,115,0,0,58,0,0,0,0,0,0,0,80,65,76,0,0,0,0,0,117,115,101,32,109,98,99,109,112,32,40,100,101,102,97,117,108,116,41,0,0,0,0,0,112,99,109,95,117,49,54,108,101,0,0,0,0,0,0,0,109,112,51,0,0,0,0,0,65,115,115,117,109,105,110,103,32,37,115,32,102,111,114,32,116,97,114,103,101,116,46,10,0,0,0,0,0,0,0,0,115,104,111,119,32,97,118,97,105,108,97,98,108,101,32,100,101,99,111,100,101,114,115,0,109,97,99,114,111,98,108,111,99,107,32,100,101,99,105,115,105,111,110,32,97,108,103,111,114,105,116,104,109,32,40,104,105,103,104,32,113,117,97,108,105,116,121,32,109,111,100,101,41,0,0,0,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,49,54,45,98,105,116,32,98,105,103,45,101,110,100,105,97,110,0,0,0,0,102,105,108,109,45,0,0,0,109,98,100,0,0,0,0,0,112,99,109,95,115,49,54,98,101,0,0,0,0,0,0,0,85,110,114,101,99,111,103,110,105,122,101,100,32,111,112,116,105,111,110,32,39,37,115,39,46,10,0,0,0,0,0,0,110,116,115,99,45,0,0,0,120,118,109,99,95,97,99,99,101,108,101,114,97,116,105,111,110,0,0,0,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,49,54,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,0,112,97,108,45,0,0,0,0,108,105,98,118,111,114,98,105,115,0,0,0,0,0,0,0,115,108,105,99,101,95,102,108,97,103,115,0,0,0,0,0,112,99,109,95,115,49,54,108,101,0,0,0,0,0,0,0,118,100,112,97,117,95,109,112,101,103,50,0,0,0,0,0,98,101,110,99,104,58,32,37,56,108,108,117,32,37,115,32,10,0,0,0,0,0,0,0,108,105,98,118,111,114,98,105,115,0,0,0,0,0,0,0,68,97,114,107,71,111,108,100,101,110,82,111,100,0,0,0,50,52,48,48,48,47,49,48,48,49,0,0,0,0,0,0,45,0,0,0,0,0,0,0,115,113,114,116,0,0,0,0,116,111,112,32,98,97,99,107,32,99,101,110,116,101,114,0,99,111,110,116,101,120,116,32,109,111,100,101,108,0,0,0,71,111,50,77,101,101,116,105,110,103,0,0,0,0,0,0,100,115,116,104,0,0,0,0,115,45,62,109,105,100,98,117,102,46,99,104,95,99,111,117,110,116,32,61,61,32,115,45,62,117,115,101,100,95,99,104,95,99,111,117,110,116,0,0,51,48,48,48,48,47,49,48,48,49,0,0,0,0,0,0,105,110,118,97,108,105,100,32,98,97,110,100,32,116,121,112,101,10,0,0,0,0,0,0,69,110,99,111,100,105,110,103,32,116,111,32,97,32,112,108,97,110,97,114,32,115,97,109,112,108,101,32,102,111,114,109,97,116,44,32,119,105,116,104,32,109,111,114,101,32,116,104,97,110,32,37,100,32,99,104,97,110,110,101,108,115,44,32,98,117,116,32,101,120,116,101,110,100,101,100,95,100,97,116,97,32,105,115,32,110,111,116,32,115,101,116,46,10,0,0,99,111,110,116,101,120,116,0,108,105,98,109,112,51,108,97,109,101,0,0,0,0,0,0,103,50,109,0,0,0,0,0,73,110,118,97,108,105,100,32,102,105,108,101,32,105,110,100,101,120,32,37,100,32,105,110,32,102,105,108,116,101,114,103,114,97,112,104,32,100,101,115,99,114,105,112,116,105,111,110,32,37,115,46,10,0,0,0,50,53,0,0,0,0,0,0,117,115,101,32,102,105,120,101,100,32,113,115,99,97,108,101,0,0,0,0,0,0,0,0,72,69,45,65,65,67,0,0,100,101,102,108,97,116,101,45,98,97,115,101,100,32,99,111,100,101,114,0,0,0,0,0,83,105,103,109,97,116,101,108,32,77,111,116,105,111,110,32,86,105,100,101,111,0,0,0,104,50,54,51,112,0,0,0,100,114,111,112,0,0,0,0,71,111,115,112,101,108,0,0,114,102,112,115,58,32,37,102,32,37,102,10,0,0,0,0,100,101,102,108,97,116,101,0,109,97,120,32,109,101,109,111,114,121,32,117,115,101,100,32,102,111,114,32,98,117,102,102,101,114,105,110,103,32,114,101,97,108,45,116,105,109,101,32,102,114,97,109,101,115,0,0,115,109,118,0,0,0,0,0,112,97,115,115,116,104,114,111,117,103,104,0,0,0,0,0,85,115,101,32,109,111,100,101,32,48,32,102,111,114,32,72,50,54,52,32,105,110,32,82,84,80,0,0,0,0,0,0,100,97,116,101,0,0,0,0,116,105,116,108,101,45,115,111,114,116,0,0,0,0,0,0,65,83,70,32,69,120,116,101,110,100,101,100,32,67,111,110,116,101,110,116,32,69,110,99,114,121,112,116,105,111,110,0,114,117,110,45,108,101,110,103,116,104,32,99,111,100,101,114,0,0,0,0,0,0,0,0,66,82,101,110,100,101,114,32,80,73,88,32,105,109,97,103,101,0,0,0,0,0,0,0,77,97,120,105,109,117,109,32,110,117,109,98,101,114,32,111,102,32,116,104,114,101,97,100,115,0,0,0,0,0,0,0,61,0,0,0,0,0,0,0,118,102,114,0,0,0,0,0,114,108,101,0,0,0,0,0,98,114,101,110,100,101,114,95,112,105,120,0,0,0,0,0,99,102,114,0,0,0,0,0,100,101,99,111,100,101,114,115,0,0,0,0,0,0,0,0,114,97,119,32,40,110,111,32,101,110,99,111,100,105,110,103,41,0,0,0,0,0,0,0,88,45,102,97,99,101,32,105,109,97,103,101,0,0,0,0,113,37,115,0,0,0,0,0,114,97,119,0,0,0,0,0,120,102,97,99,101,0,0,0,32,109,97,116,99,104,101,100,32,97,115,32,111,112,116,105,111,110,32,39,37,115,39,32,40,37,115,41,32,119,105,116,104,32,97,114,103,117,109,101,110,116,32,48,46,10,0,0,108,105,98,102,100,107,95,97,97,99,0,0,0,0,0,0,113,58,118,0,0,0,0,0,69,120,112,101,99,116,101,100,32,105,110,116,32,102,111,114,32,37,115,32,98,117,116,32,102,111,117,110,100,32,37,115,10,0,0,0,0,0,0,0,97,114,105,116,104,109,101,116,105,99,32,99,111,100,101,114,0,0,0,0,0,0,0,0,67,80,105,65,32,118,105,100,101,111,32,102,111,114,109,97,116,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,80,108,101,97,115,101,32,117,115,101,32,45,113,58,97,32,111,114,32,45,113,58,118,44,32,45,113,115,99,97,108,101,32,105,115,32,97,109,98,105,103,117,111,117,115,10,0,0,118,97,114,105,97,98,108,101,32,108,101,110,103,116,104,32,99,111,100,101,114,32,47,32,72,117,102,102,109,97,110,32,99,111,100,101,114,0,0,0,99,112,105,97,0,0,0,0,118,100,112,97,117,95,109,112,101,103,49,0,0,0,0,0,37,108,108,100,0,0,0,0,68,97,114,107,67,121,97,110,0,0,0,0,0,0,0,0,112,114,111,102,105,108,101,58,118,0,0,0,0,0,0,0,32,32,37,115,37,45,49,55,115,32,0,0,0,0,0,0,116,114,117,110,99,0,0,0,69,120,112,101,114,105,109,101,110,116,97,108,32,102,101,97,116,117,114,101,0,0,0,0,84,66,67,0,0,0,0,0,118,108,99,0,0,0,0,0,99,104,114,32,115,114,99,87,61,37,100,32,115,114,99,72,61,37,100,32,100,115,116,87,61,37,100,32,100,115,116,72,61,37,100,32,120,73,110,99,61,37,100,32,121,73,110,99,61,37,100,10,0,0,0,0,65,118,105,100,32,65,86,73,32,67,111,100,101,99,0,0,100,101,115,116,105,110,97,116,105,111,110,32,119,105,100,116,104,0,0,0,0,0,0,0,33,115,45,62,114,101,115,97,109,112,108,101,0,0,0,0,80,108,101,97,115,101,32,117,115,101,32,45,112,114,111,102,105,108,101,58,97,32,111,114,32,45,112,114,111,102,105,108,101,58,118,44,32,45,112,114,111,102,105,108,101,32,105,115,32,97,109,98,105,103,117,111,117,115,10,0,0,0,0,0,83,99,97,108,101,102,97,99,116,111,114,32,40,37,100,41,32,111,117,116,32,111,102,32,114,97,110,103,101,46,10,0,70,97,105,108,101,100,32,116,111,32,97,108,108,111,99,97,116,101,32,112,97,99,107,101,116,32,111,102,32,115,105,122,101,32,37,100,10,0,0,0,99,111,100,101,114,0,0,0,97,118,114,110,0,0,0,0,79,110,108,121,32,118,105,100,101,111,32,97,110,100,32,97,117,100,105,111,32,102,105,108,116,101,114,115,32,115,117,112,112,111,114,116,101,100,32,99,117,114,114,101,110,116,108,121,46,10,0,0,0,0,0,0,69,114,114,111,114,32,111,112,101,110,105,110,103,32,102,105,108,101,32,37,115,46,10,0,113,115,99,97,108,101,0,0,76,67,0,0,0,0,0,0,83,116,114,101,97,109,32,105,115,32,111,108,100,32,97,110,100,32,109,97,121,32,110,111,116,32,119,111,114,107,10,0,103,108,111,98,97,108,95,113,117,97,108,105,116,121,0,0,65,109,97,122,105,110,103,32,83,116,117,100,105,111,32,80,97,99,107,101,100,32,65,110,105,109,97,116,105,111,110,32,70,105,108,101,32,86,105,100,101,111,0,0,0,0,0,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,86,105,100,101,111,32,56,0,0,0,102,114,97,109,101,115,58,118,0,0,0,0,0,0,0,0,83,111,117,110,100,32,67,108,105,112,0,0,0,0,0,0,100,101,99,111,100,105,110,103,32,102,111,114,32,115,116,114,101,97,109,32,37,100,32,102,97,105,108,101,100,10,0,0,105,110,116,101,114,32,113,117,97,110,116,32,98,105,97,115,0,0,0,0,0,0,0,0,114,116,98,117,102,115,105,122,101,0,0,0,0,0,0,0,112,97,102,95,118,105,100,101,111,0,0,0,0,0,0,0,99,111,100,101,99,58,118,0,104,50,54,52,95,109,111,100,101,48,0,0,0,0,0,0,97,108,98,117,109,0,0,0,84,83,79,84,0,0,0,0,101,110,99,114,121,112,116,105,111,110,0,0,0,0,0,0,112,98,105,97,115,0,0,0,76,117,99,97,115,65,114,116,115,32,83,77,85,83,72,32,118,105,100,101,111,0,0,0,116,104,114,101,97,100,115,0,80,114,111,99,101,115,115,105,110,103,32,99,111,109,109,97,110,100,32,116,105,109,101,58,37,102,32,99,111,109,109,97,110,100,58,37,115,32,97,114,103,58,37,115,10,0,0,0,79,112,116,105,111,110,32,39,37,115,39,32,119,97,115,32,114,101,109,111,118,101,100,46,32,73,102,32,121,111,117,32,97,114,101,32,108,111,111,107,105,110,103,32,102,111,114,32,97,110,32,111,112,116,105,111,110,32,116,111,32,112,114,101,115,101,114,118,101,32,116,104,101,32,113,117,97,108,105,116,121,32,40,119,104,105,99,104,32,105,115,32,110,111,116,32,119,104,97,116,32,45,37,115,32,119,97,115,32,102,111,114,41,44,32,117,115,101,32,45,113,115,99,97,108,101,32,48,32,111,114,32,97,110,32,101,113,117,105,118,97,108,101,110,116,32,113,117,97,108,105,116,121,32,102,97,99,116,111,114,32,111,112,116,105,111,110,46,10,0,0,0,0,0,0,0,105,110,116,114,97,32,113,117,97,110,116,32,98,105,97,115,0,0,0,0,0,0,0,0,115,97,110,109,0,0,0,0,103,111,112,95,116,105,109,101,99,111,100,101,0,0,0,0,115,104,111,119,32,97,118,97,105,108,97,98,108,101,32,99,111,100,101,99,115,0,0,0,105,98,105,97,115,0,0,0,85,110,99,111,109,112,114,101,115,115,101,100,32,112,97,99,107,101,100,32,52,58,50,58,48,0,0,0,0,0,0,0,109,101,116,97,100,97,116,97,58,103,0,0,0,0,0,0,108,105,109,105,116,32,109,111,116,105,111,110,32,118,101,99,116,111,114,115,32,114,97,110,103,101,32,40,49,48,50,51,32,102,111,114,32,68,105,118,88,32,112,108,97,121,101,114,41,0,0,0,0,0,0,0,121,117,118,52,0,0,0,0,69,114,114,111,114,32,112,97,114,115,105,110,103,32,111,112,116,105,111,110,32,39,37,115,39,32,119,105,116,104,32,97,114,103,117,109,101,110,116,32,39,37,115,39,46,10,0,0,116,105,109,101,99,111,100,101,61,37,115,0,0,0,0,0,109,101,95,114,97,110,103,101,0,0,0,0,0,0,0,0,85,110,99,111,109,112,114,101,115,115,101,100,32,112,97,99,107,101,100,32,81,84,32,52,58,52,58,52,58,52,0,0,118,115,116,97,116,115,95,37,48,50,100,37,48,50,100,37,48,50,100,46,108,111,103,0,100,116,103,95,97,99,116,105,118,101,95,102,111,114,109,97,116,0,0,0,0,0,0,0,118,52,48,56,0,0,0,0,118,100,112,97,117,95,104,50,54,52,0,0,0,0,0,0,37,46,54,103,0,0,0,0,68,97,114,107,66,108,117,101,0,0,0,0,0,0,0,0,102,105,108,116,101,114,58,118,0,0,0,0,0,0,0,0,32,32,32,32,32,37,45,49,53,115,32,0,0,0,0,0,99,101,105,108,0,0,0,0,69,88,80,69,82,73,77,69,78,84,65,76,0,0,0,0,116,111,112,32,98,97,99,107,32,108,101,102,116,0,0,0,115,117,98,45,112,101,108,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,113,117,97,108,105,116,121,0,0,0,0,0,0,0,108,117,109,32,115,114,99,87,61,37,100,32,115,114,99,72,61,37,100,32,100,115,116,87,61,37,100,32,100,115,116,72,61,37,100,32,120,73,110,99,61,37,100,32,121,73,110,99,61,37,100,10,0,0,0,0,85,110,99,111,109,112,114,101,115,115,101,100,32,112,97,99,107,101,100,32,52,58,52,58,52,0,0,0,0,0,0,0,100,115,116,119,0,0,0,0,111,117,116,45,62,99,104,95,99,111,117,110,116,32,61,61,32,105,110,45,62,99,104,95,99,111,117,110,116,0,0,0,37,115,58,37,99,0,0,0,73,110,118,97,108,105,100,32,98,105,116,115,116,114,101,97])
.concat([109,44,32,99,114,111,115,115,111,118,101,114,32,98,97,110,100,32,105,110,100,101,120,32,98,101,121,111,110,100,32,97,114,114,97,121,32,98,111,117,110,100,115,58,32,37,100,10,0,0,0,0,0,0,0,0,73,102,32,121,111,117,32,104,101,97,114,100,32,97,110,32,97,117,100,105,98,108,101,32,97,114,116,105,102,97,99,116,44,32,116,104,101,114,101,32,109,97,121,32,98,101,32,97,32,98,117,103,32,105,110,32,116,104,101,32,100,101,99,111,100,101,114,46,32,67,108,105,112,112,101,100,32,110,111,105,115,101,32,103,97,105,110,32,40,37,100,32,45,62,32,37,100,41,0,0,0,0,0,0,85,115,101,114,32,112,97,99,107,101,116,32,105,115,32,116,111,111,32,115,109,97,108,108,32,40,37,100,32,60,32,37,100,41,10,0,0,0,0,0,115,105,122,101,111,102,40,116,109,112,41,32,62,61,32,97,118,95,115,104,97,95,115,105,122,101,0,0,0,0,0,0,115,117,98,113,0,0,0,0,118,51,48,56,0,0,0,0,115,117,98,50,118,105,100,101,111,58,32,117,115,105,110,103,32,37,100,120,37,100,32,99,97,110,118,97,115,10,0,0,73,110,118,97,108,105,100,32,118,97,108,117,101,32,39,37,115,39,32,102,111,114,32,111,112,116,105,111,110,32,39,37,115,39,44,32,114,101,113,117,105,114,101,100,32,115,121,110,116,97,120,32,105,115,32,39,105,110,100,101,120,58,118,97,108,117,101,39,10,0,0,0,117,115,101,32,108,111,111,112,32,102,105,108,116,101,114,0,48,0,0,0,0,0,0,0,100,105,97,109,111,110,100,32,116,121,112,101,32,38,32,115,105,122,101,32,102,111,114,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,112,114,101,45,112,97,115,115,0,0,0,0,0,0,80,105,110,110,97,99,108,101,32,84,65,82,71,65,32,67,105,110,101,87,97,118,101,32,89,85,86,49,54,0,0,0,119,109,118,50,0,0,0,0,98,58,118,0,0,0,0,0,71,97,109,101,0,0,0,0,98,103,114,50,52,0,0,0,109,97,120,95,97,110,97,108,121,122,101,95,100,117,114,97,116,105,111,110,32,37,100,32,114,101,97,99,104,101,100,32,97,116,32,37,108,108,100,32,109,105,99,114,111,115,101,99,111,110,100,115,10,0,0,0,112,114,101,95,100,105,97,95,115,105,122,101,0,0,0,0,109,97,120,32,109,101,109,111,114,121,32,117,115,101,100,32,102,111,114,32,116,105,109,101,115,116,97,109,112,32,105,110,100,101,120,32,40,112,101,114,32,115,116,114,101,97,109,41,0,0,0,0,0,0,0,0,116,97,114,103,97,95,121,50,49,54,0,0,0,0,0,0,80,108,101,97,115,101,32,117,115,101,32,45,98,58,97,32,111,114,32,45,98,58,118,44,32,45,98,32,105,115,32,97,109,98,105,103,117,111,117,115,10,0,0,0,0,0,0,0,68,111,110,39,116,32,115,101,110,100,32,82,84,67,80,32,115,101,110,100,101,114,32,114,101,112,111,114,116,115,0,0,99,111,109,109,101,110,116,0,97,114,116,105,115,116,45,115,111,114,116,0,0,0,0,0,69,120,116,32,68,82,77,32,112,114,111,116,101,99,116,101,100,32,115,116,114,101,97,109,32,100,101,116,101,99,116,101,100,44,32,100,101,99,111,100,105,110,103,32,119,105,108,108,32,108,105,107,101,108,121,32,102,97,105,108,33,10,0,0,99,104,114,111,109,97,0,0,85,110,99,111,109,112,114,101,115,115,101,100,32,112,97,99,107,101,100,32,77,83,32,52,58,52,58,52,58,52,0,0,115,108,105,99,101,0,0,0,67,111,112,121,105,110,103,32,100,97,116,97,32,105,110,32,97,118,102,105,108,116,101,114,46,10,0,0,0,0,0,0,102,114,97,109,101,115,58,97,0,0,0,0,0,0,0,0,98,105,107,101,115,104,101,100,0,0,0,0,0,0,0,0,100,99,116,109,97,120,0,0,97,121,117,118,0,0,0,0,113,58,97,0,0,0,0,0,99,111,100,101,99,115,0,0,110,111,105,115,101,32,112,114,101,115,101,114,118,105,110,103,32,115,117,109,32,111,102,32,115,113,117,97,114,101,100,32,100,105,102,102,101,114,101,110,99,101,115,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,112,97,114,115,101,32,111,112,116,105,111,110,32,118,97,108,117,101,32,34,37,115,34,32,97,115,32,112,105,120,101,108,32,102,111,114,109,97,116,10,0,0,0,0,0,0,65,118,105,100,32,77,101,114,105,100,105,101,110,32,85,110,99,111,109,112,114,101,115,115,101,100,0,0,0,0,0,0,99,111,100,101,99,58,97,0,100,117,114,97,116,105,111,110,32,62,61,32,48,0,0,0,32,105,115,32,110,111,116,32,105,109,112,108,101,109,101,110,116,101,100,46,32,85,112,100,97,116,101,32,121,111,117,114,32,70,70,109,112,101,103,32,118,101,114,115,105,111,110,32,116,111,32,116,104,101,32,110,101,119,101,115,116,32,111,110,101,32,102,114,111,109,32,71,105,116,46,32,73,102,32,116,104,101,32,112,114,111,98,108,101,109,32,115,116,105,108,108,32,111,99,99,117,114,115,44,32,105,116,32,109,101,97,110,115,32,116,104,97,116,32,121,111,117,114,32,102,105,108,101,32,104,97,115,32,97,32,102,101,97,116,117,114,101,32,119,104,105,99,104,32,104,97,115,32,110,111,116,32,98,101,101,110,32,105,109,112,108,101,109,101,110,116,101,100,46,10,0,110,115,115,101,0,0,0,0,97,118,117,105,0,0,0,0,108,105,98,97,118,117,116,105,108,47,105,109,103,117,116,105,108,115,46,99,0,0,0,0,32,109,97,116,99,104,101,100,32,97,115,32,65,86,79,112,116,105,111,110,32,39,37,115,39,32,119,105,116,104,32,97,114,103,117,109,101,110,116,32,39,37,115,39,46,10,0,0,37,100,0,0,0,0,0,0,115,117,109,32,111,102,32,115,113,117,97,114,101,100,32,118,101,114,116,105,99,97,108,32,100,105,102,102,101,114,101,110,99,101,115,0,0,0,0,0,48,49,50,118,0,0,0,0,77,105,115,115,105,110,103,32,39,41,39,32,105,110,32,39,37,115,39,10,0,0,0,0,37,108,108,117,0,0,0,0,66,85,71,0,0,0,0,0,112,114,101,118,95,102,111,114,99,101,100,95,116,0,0,0,118,115,115,101,0,0,0,0,65,118,105,100,32,49,58,49,32,49,48,45,98,105,116,32,82,71,66,32,80,97,99,107,101,114,0,0,0,0,0,0,121,117,118,97,52,50,48,112,0,0,0,0,0,0,0,0,78,79,80,84,83,0,0,0,67,121,97,110,0,0,0,0,85,110,107,110,111,119,110,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,58,32,37,115,10,0,0,0,0,0,102,108,111,111,114,0,0,0,85,110,107,110,111,119,110,32,101,114,114,111,114,32,111,99,99,117,114,114,101,100,0,0,84,66,76,0,0,0,0,0,115,117,109,32,111,102,32,97,98,115,111,108,117,116,101,32,118,101,114,116,105,99,97,108,32,100,105,102,102,101,114,101,110,99,101,115,0,0,0,0,43,0,0,0,0,0,0,0,37,100,120,37,100,32,45,62,32,37,100,120,37,100,10,0,97,118,114,112,0,0,0,0,115,111,117,114,99,101,32,104,101,105,103,104,116,0,0,0,111,117,116,45,62,98,112,115,32,61,61,32,105,110,45,62,98,112,115,0,0,0,0,0,102,105,108,116,101,114,58,97,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,110,95,109,97,115,116,101,114,58,32,37,100,10,0,0,0,73,102,32,121,111,117,32,104,101,97,114,100,32,97,110,32,97,117,100,105,98,108,101,32,97,114,116,105,102,97,99,116,44,32,116,104,101,114,101,32,109,97,121,32,98,101,32,97,32,98,117,103,32,105,110,32,116,104,101,32,100,101,99,111,100,101,114,46,32,67,108,105,112,112,101,100,32,105,110,116,101,110,115,105,116,121,32,115,116,101,114,101,111,32,112,111,115,105,116,105,111,110,32,40,37,100,32,45,62,32,37,100,41,0,0,0,0,0,0,0,33,97,118,112,107,116,45,62,100,97,116,97,32,124,124,32,97,118,112,107,116,45,62,100,97,116,97,32,33,61,32,97,118,99,116,120,45,62,105,110,116,101,114,110,97,108,45,62,98,121,116,101,95,98,117,102,102,101,114,0,0,0,0,0,105,110,116,101,114,110,97,108,32,101,114,114,111,114,32,37,115,32,45,62,32,37,115,32,99,111,110,118,101,114,116,101,114,10,0,0,0,0,0,0,118,115,97,100,0,0,0,0,79,112,101,110,69,88,82,32,105,109,97,103,101,0,0,0,121,97,100,105,102,0,0,0,100,101,115,99,0,0,0,0,100,101,115,99,0,0,0,0,99,111,100,101,99,58,115,0,108,111,111,112,0,0,0,0,98,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,100,101,112,114,101,99,97,116,101,100,32,112,105,120,101,108,32,102,111,114,109,97,116,32,117,115,101,100,44,32,109,97,107,101,32,115,117,114,101,32,121,111,117,32,100,105,100,32,115,101,116,32,114,97,110,103,101,32,99,111,114,114,101,99,116,108,121,10,0,0,0,0,101,120,114,0,0,0,0,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,86,105,100,101,111,32,55,0,0,0,84,104,105,115,32,111,112,116,105,111,110,32,105,115,32,100,101,112,114,101,99,97,116,101,100,44,32,117,115,101,32,45,99,104,97,110,110,101,108,46,10,0,0,0,0,0,0,0,72,111,117,115,101,0,0,0,102,97,115,116,95,98,105,108,105,110,101,97,114,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,115,97,109,112,108,101,32,102,111,114,109,97,116,10,0,0,0,0,0,0,73,103,110,111,114,101,32,108,101,110,103,116,104,0,0,0,111,117,116,45,62,99,104,95,99,111,117,110,116,32,61,61,32,97,118,95,103,101,116,95,99,104,97,110,110,101,108,95,108,97,121,111,117,116,95,110,98,95,99,104,97,110,110,101,108,115,40,115,45,62,111,117,116,95,99,104,95,108,97,121,111,117,116,41,0,0,0,0,68,84,83,32,100,105,115,99,111,110,116,105,110,117,105,116,121,32,105,110,32,115,116,114,101,97,109,32,37,100,58,32,112,97,99,107,101,116,32,37,100,32,119,105,116,104,32,68,84,83,32,37,108,108,100,44,32,112,97,99,107,101,116,32,37,100,32,119,105,116,104,32,68,84,83,32,37,108,108,100,10,0,0,0,0,0,0,0,117,110,107,110,111,119,110,32,115,117,98,102,111,114,109,97,116,58,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,10,0,0,0,0,0,48,0,0,0,0,0,0,0,114,97,116,101,32,100,105,115,116,111,114,116,105,111,110,32,111,112,116,105,109,97,108,44,32,115,108,111,119,0,0,0,105,110,100,101,120,109,101,109,0,0,0,0,0,0,0,0,69,115,99,97,112,101,32,49,51,48,0,0,0,0,0,0,84,104,105,115,32,111,112,116,105,111,110,32,105,115,32,100,101,112,114,101,99,97,116,101,100,44,32,117,115,101,32,45,115,116,97,110,100,97,114,100,46,10,0,0,0,0,0,0,115,107,105,112,95,114,116,99,112,0,0,0,0,0,0,0,108,121,114,105,99,115,0,0,84,83,79,80,0,0,0,0,65,83,70,95,76,105,99,101,110,115,101,95,85,82,76,0,114,100,0,0,0,0,0,0,101,115,99,97,112,101,49,51,48,0,0,0,0,0,0,0,65,108,108,111,119,101,100,32,116,104,114,101,97,100,32,116,121,112,101,115,0,0,0,0,83,97,109,112,108,101,115,32,100,114,111,112,112,101,100,32,100,117,101,32,116,111,32,109,101,109,111,114,121,32,97,108,108,111,99,97,116,105,111,110,32,102,97,105,108,117,114,101,46,10,0,0,0,0,0,0,37,115,58,32,73,110,118,97,108,105,100,32,111,112,116,105,111,110,32,111,114,32,97,114,103,117,109,101,110,116,58,32,39,37,115,39,44,32,112,97,114,115,101,100,32,97,115,32,39,37,115,39,32,61,32,39,37,115,39,10,0,0,0,0,111,115,102,0,0,0,0,0,110,117,109,98,101,114,32,111,102,32,98,105,116,115,32,110,101,101,100,101,100,32,102,111,114,32,116,104,101,32,98,108,111,99,107,0,0,0,0,0,105,105,100,95,112,97,114,32,105,110,118,97,108,105,100,10,0,0,0,0,0,0,0,0,85,110,99,111,109,112,114,101,115,115,101,100,32,89,85,86,32,52,58,49,58,49,32,49,50,45,98,105,116,0,0,0,102,102,112,114,101,115,101,116,91,37,115,93,58,32,115,101,116,32,39,37,115,39,32,61,32,39,37,115,39,10,0,0,115,104,111,119,32,97,118,97,105,108,97,98,108,101,32,102,111,114,109,97,116,115,0,0,100,121,110,95,118,108,99,46,116,97,98,108,101,95,115,105,122,101,32,60,61,32,118,108,99,45,62,116,97,98,108,101,95,97,108,108,111,99,97,116,101,100,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,98,105,116,0,0,0,0,0,121,52,49,112,0,0,0,0,37,115,58,32,73,110,118,97,108,105,100,32,115,121,110,116,97,120,58,32,39,37,115,39,10,0,0,0,0,0,0,0,115,117,109,32,111,102,32,115,113,117,97,114,101,100,32,113,117,97,110,116,105,122,97,116,105,111,110,32,101,114,114,111,114,115,32,40,97,118,111,105,100,44,32,108,111,119,32,113,117,97,108,105,116,121,41,0,65,112,112,108,101,32,73,110,116,101,114,109,101,100,105,97,116,101,32,67,111,100,101,99,0,0,0,0,0,0,0,0,32,109,97,116,99,104,101,100,32,97,115,32,111,112,116,105,111,110,32,39,37,115,39,32,40,37,115,41,32,119,105,116,104,32,97,114,103,117,109,101,110,116,32,39,37,115,39,46,10,0,0,0,0,0,0,0,13,10,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,110,98,95,102,114,97,109,101,115,32,105,115,32,37,100,10,0,0,0,0,0,0,0,0,83,101,116,117,112,32,104,101,97,100,101,114,32,105,115,32,116,111,111,32,115,104,111,114,116,10,0,0,0,0,0,0,115,117,109,32,111,102,32,97,98,115,111,108,117,116,101,32,68,67,84,32,116,114,97,110,115,102,111,114,109,101,100,32,100,105,102,102,101,114,101,110,99,101,115,0,0,0,0,0,97,105,99,0,0,0,0,0,61,0,0,0,0,0,0,0,115,117,109,32,111,102,32,97,98,115,111,108,117,116,101,32,72,97,100,97,109,97,114,100,32,116,114,97,110,115,102,111,114,109,101,100,32,100,105,102,102,101,114,101,110,99,101,115,0,0,0,0,0,0,0,0,77,83,32,87,105,110,100,111,119,115,32,77,101,100,105,97,32,86,105,100,101,111,32,86,57,32,83,99,114,101,101,110,0,0,0,0,0,0,0,0,121,117,118,106,52,52,48,112,0,0,0,0,0,0,0,0,97,118,95,105,110,116,101,114,108,101,97,118,101,100,95,119,114,105,116,101,95,102,114,97,109,101,40,41,0,0,0,0,67,114,105,109,115,111,110,0,35,10,13,0,0,0,0,0,114,111,111,116,0,0,0,0,85,78,75,78,79,87,78,0,73,110,99,111,110,115,105,115,116,101,110,116,32,99,104,97,110,110,101,108,32,99,111,110,102,105,103,117,114,97,116,105,111,110,46,10,0,0,0,0,116,111,112,32,102,114,111,110,116,32,114,105,103,104,116,0,115,97,116,100,0,0,0,0,117,115,105,110,103,32,67,10,0,0,0,0,0,0,0,0,109,115,115,50,0,0,0,0,115,114,99,104,0,0,0,0,80,67,77,32,117,110,115,105,103,110,101,100,32,56,45,98,105,116,0,0,0,0,0,0,111,117,116,45,62,112,108,97,110,97,114,32,61,61,32,105,110,45,62,112,108,97,110,97,114,0,0,0,0,0,0,0,70,105,108,101,32,102,111,114,32,112,114,101,115,101,116,32,39,37,115,39,32,110,111,116,32,102,111,117,110,100,10,0,73,110,118,97,108,105,100,32,118,68,107,49,91,37,100,93,58,32,37,100,10,0,0,0,84,78,83,32,102,105,108,116,101,114,32,111,114,100,101,114,32,37,100,32,105,115,32,103,114,101,97,116,101,114,32,116,104,97,110,32,109,97,120,105,109,117,109,32,37,100,46,10,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,109,105,110,105,109,117,109,32,114,101,113,117,105,114,101,100,32,112,97,99,107,101,116,32,115,105,122,101,32,37,100,32,40,109,97,120,32,97,108,108,111,119,101,100,32,105,115,32,37,100,41,10,0,0,0,0,115,117,109,32,111,102,32,115,113,117,97,114,101,100,32,101,114,114,111,114,115,0,0,0,67,97,110,111,112,117,115,32,76,111,115,115,108,101,115,115,32,67,111,100,101,99,0,0,100,101,105,110,116,101,114,108,97,99,101,32,105,110,112,117,116,32,102,114,111,109,32,115,116,114,101,97,109,32,37,100,58,37,100,0,0,0,0,0,80,108,101,97,115,101,32,117,115,101,32,45,112,114,101,115,101,116,32,60,115,112,101,101,100,62,32,45,113,112,32,48,10,0,0,0,0,0,0,0,117,115,101,32,49,47,52,45,112,101,108,32,109,111,116,105,111,110,32,99,111,109,112,101,110,115,97,116,105,111,110,0,85,110,97,98,108,101,32,116,111,32,103,101,116,32,101,110,99,111,100,101,114,32,105,110,102,111,58,32,37,115,10,0,115,115,101,0,0,0,0,0,99,108,108,99,0,0,0,0,119,109,118,49,0,0,0,0,108,105,98,120,50,54,52,45,108,111,115,115,108,101,115,115,0,0,0,0,0,0,0,0,65,99,105,100,0,0,0,0,105,103,110,111,114,101,95,108,101,110,103,116,104,0,0,0,78,111,110,45,105,110,99,114,101,97,115,105,110,103,32,68,84,83,32,105,110,32,115,116,114,101,97,109,32,37,100,58,32,112,97,99,107,101,116,32,37,100,32,119,105,116,104,32,68,84,83,32,37,108,108,100,44,32,112,97,99,107,101,116,32,37,100,32,119,105,116,104,32,68,84,83,32,37,108,108,100,10,0,0,0,0,0,0,112,114,101,109,97,116,117,114,101,32,101,110,100,32,111,102,32,102,105,108,101,32,119,104,105,108,101,32,114,101,97,100,105,110,103,32,73,78,70,79,32,116,97,103,10,0,0,0,115,117,109,32,111,102,32,97,98,115,111,108,117,116,101,32,100,105,102,102,101,114,101,110,99,101,115,44,32,102,97,115,116,32,40,100,101,102,97,117,108,116,41,0,0,0,0,0,100,101,99,114,121,112,116,105,111,110,32,107,101,121,0,0,77,83,32,69,120,112,114,101,115,115,105,111,110,32,69,110,99,111,100,101,114,32,83,99,114,101,101,110,0,0,0,0,99,111,100,101,99,58,100,0,85,115,101,32,82,70,67,32,50,49,57,48,32,112,97,99,107,101,116,105,122,97,116,105,111,110,32,105,110,115,116,101,97,100,32,111,102,32,82,70,67,32,52,54,50,57,32,102,111,114,32,72,46,50,54,51,0,0,0,0,0,0,0,0,103,114,111,117,112,105,110,103,0,0,0,0,0,0,0,0,97,108,98,117,109,45,115,111,114,116,0,0,0,0,0,0,65,83,70,95,75,101,121,95,73,68,0,0,0,0,0,0,115,97,100,0,0,0,0,0,109,116,115,50,0,0,0,0,67,104,97,110,103,105,110,103,32,102,114,97,109,101,32,112,114,111,112,101,114,116,105,101,115,32,111,110,32,116,104,101,32,102,108,121,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,46,10,0,116,104,114,101,97,100,95,116,121,112,101,0,0,0,0,0,100,105,115,97,98,108,101,32,100,97,116,97,0,0,0,0,112,114,101,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,32,99,111,109,112,97,114,101,32,102,117,110,99,116,105,111,110,0,0,84,101,99,104,83,109,105,116,104,32,83,99,114,101,101,110,32,67,111,100,101,99,32,50,0,0,0,0,0,0,0,0,37,115,0,0,0,0,0,0,100,110,0,0,0,0,0,0,102,111,114,109,97,116,115,0,84,114,105,101,100,32,116,111,32,99,111,112,121,32,65,86,67,111,100,101,99,67,111,110,116,101,120,116,32,37,112,32,105,110,116,111,32,97,108,114,101,97,100,121,45,105,110,105,116,105,97,108,105,122,101,100,32,37,112,10,0,0,0,0,112,114,101,99,109,112,0,0,116,115,99,99,50,0,0,0,100,105,115,99,97,114,100,105,110,103,32,73,68,51,32,116,97,103,10,0,0,0,0,0,102,111,114,99,101,32,100,97,116,97,32,99,111,100,101,99,32,40,39,99,111,112,121,39,32,116,111,32,99,111,112,121,32,115,116,114,101,97,109,41,0,0,0,0,0,0,0,0,101,114,114,111,114,32,103,101,116,116,105,110,103,32,97,118,97,105,108,97,98,108,101,32,112,97,99,107,101,116,115,10,0,0,0,0,0,0,0,0,118,111,114,98,105,115,32,105,110,105,116,105,97,108,32,104,101,97,100,101,114,32,108,101,110,32,105,115,32,119,114,111,110,103,58,32,37,100,10,0,105,110,97,100,101,113,117,97,116,101,32,65,86,70,114,97,109,101,32,112,108,97,110,101,32,112,97,100,100,105,110,103,10,0,0,0,0,0,0,0,112,114,101,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,0,0,0,77,83,32,65,84,67,32,83,99,114,101,101,110,0,0,0,32,109,97,116,99,104,101,100,32,97,115,32,37,115,32,119,105,116,104,32,97,114,103,117,109,101,110,116,32,39,37,115,39,46,10,0,0,0,0,0,100,99,111,100,101,99,0,0,121,117,118,106,0,0,0,0,112,114,101,109,101,0,0,0,85,110,107,110,111,119,110,32,99,104,114,111,109,97,32,102,111,114,109,97,116,32,37,100,10,0,0,0,0,0,0,0,109,115,97,49,0,0,0,0,115,101,116,32,111,112,116,105,111,110,115,32,102,114,111,109,32,105,110,100,105,99,97,116,101,100,32,112,114,101,115,101,116,32,102,105,108,101,0,0,97,109,111,117,110,116,32,111,102,32,109,111,116,105,111,110,32,112,114,101,100,105,99,116,111,114,115,32,102,114,111,109,32,116,104,101,32,112,114,101,118,105,111,117,115,32,102,114,97,109,101,0,0,0,0,0,77,83,32,83,99,114,101,101,110,32,49,0,0,0,0,0,121,117,118,52,52,48,112,0,109,117,120,101,114,32,60,45,32,116,121,112,101,58,37,115,32,112,107,116,95,112,116,115,58,37,115,32,112,107,116,95,112,116,115,95,116,105,109,101,58,37,115,32,112,107,116,95,100,116,115,58,37,115,32,112,107,116,95,100,116,115,95,116,105,109,101,58,37,115,32,115,105,122,101,58,37,100,10,0,67,111,114,110,115,105,108,107,0,0,0,0,0,0,0,0,102,112,114,101,0,0,0,0,116,97,121,108,111,114,0,0,83,116,114,101,97,109,32,110,111,116,32,102,111,117,110,100,0,0,0,0,0,0,0,0,84,70,82,0,0,0,0,0,108,97,115,116,95,112,114,101,100,0,0,0,0,0,0,0,109,115,115,49,0,0,0,0,115,111,117,114,99,101,32,119,105,100,116,104,0,0,0,0,99,111,109,112,101,110,115,97,116,105,110,103,32,97,117,100,105,111,32,116,105,109,101,115,116,97,109,112,32,100,114,105,102,116,58,37,102,32,99,111,109,112,101,110,115,97,116,105,111,110,58,37,100,32,105,110,58,37,100,10,0,0,0,0,115,101,116,32,116,104,101,32,115,117,98,116,105,116,108,101,32,111,112,116,105,111,110,115,32,116,111,32,116,104,101,32,105,110,100,105,99,97,116,101,100,32,112,114,101,115,101,116,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,118,68,107,48,91,37,100,93,58,32,37,100,10,0,0,0,101,114,114,111,114,32,105,110,32,115,112,101,99,116,114,97,108,32,100,97,116,97,44,32,69,83,67,32,111,118,101,114,102,108,111,119,10,0,0,0,73,110,118,97,108,105,100,32,110,101,103,97,116,105,118,101,32,117,115,101,114,32,112,97,99,107,101,116,32,115,105,122,101,32,37,100,10,0,0,0,100,105,97,109,111,110,100,32,116,121,112,101,32,38,32,115,105,122,101,32,102,111,114,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,0,0,0,0,0,0,0,90,101,114,111,67,111,100,101,99,32,76,111,115,115,108,101,115,115,32,86,105,100,101,111,0,0,0,0,0,0,0,0,78,0,0,0,0,0,0,0,115,112,114,101,0,0,0,0,113,112,101,108,0,0,0,0,85,110,97,98,108,101,32,116,111,32,105,110,105,116,105,97,108,105,122,101,32,116,104,101,32,101,110,99,111,100,101,114,58,32,37,115,10,0,0,0,100,105,97,95,115,105,122,101,0,0,0,0,0,0,0,0,122,101,114,111,99,111,100,101,99,0,0,0,0,0,0,0,77,80,69,71,45,52,32,112,97,114,116,32,50,32,77,105,99,114,111,115,111,102,116,32,118,97,114,105,97,110,116,32,118,101,114,115,105,111,110,32,51,0,0,0,0,0,0,0,115,101,116,32,116,104,101,32,118,105,100,101,111,32,111,112,116,105,111,110,115,32,116,111,32,116,104,101,32,105,110,100,105,99,97,116,101,100,32,112,114,101,115,101,116,0,0,0,73,110,115,116,114,117,109,101,110,116,97,108,0,0,0,0,87,65,86,32,100,101,109,117,120,101,114,0,0,0,0,0,83,116,114,101,97,109,32,35,37,100,58,32,110,111,116,32,101,110,111,117,103,104,32,102,114,97,109,101,115,32,116,111,32,101,115,116,105,109,97,116,101,32,114,97,116,101,59,32,99,111,110,115,105,100,101,114,32,105,110,99,114,101,97,115,105,110,103,32,112,114,111,98,101,115,105,122,101,10,0,0,111,117,116,32,111,102,32,109,101,109,111,114,121,44,32,117,110,97,98,108,101,32,116,111,32,114,101,97,100,32,73,78,70,79,32,116,97,103,10,0,105,110,116,101,114,108,97,99,101,100,32,68,67,84,32,99,111,109,112,97,114,101,32,102,117,110,99,116,105,111,110,0,99,114,121,112,116,111,107,101,121,0,0,0,0,0,0,0,88,66,77,32,40,88,32,66,105,116,77,97,112,41,32,105,109,97,103,101,0,0,0,0,118,112,114,101,0,0,0,0,114,102,99,50,49,57,48,0,99,111,112,121,114,105,103,104,116,0,0,0,0,0,0,0,84,83,79,65,0,0,0,0,77,80,69,71,45,50,32,118,105,100,101,111,0,0,0,0,65,83,70,95,80,114,111,116,101,99,116,105,111,110,95,84,121,112,101,0,0,0,0,0,105,108,100,99,116,99,109,112,0,0,0,0,0,0,0,0,112,45,112,107,116,45,62,100,97,116,97,32,61,61,32,112,107,116,45,62,115,105,122,101,0,0,0,0,0,0,0,0,120,98,109,0,0,0,0,0,67,104,97,110,103,105,110,103,32,102,114,97,109,101,32,112,114,111,112,101,114,116,105,101,115,32,111,110,32,116,104,101,32,102,108,121,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,98,121,32,97,108,108,32,102,105,108,116,101,114,115,46,10,0,0,65,86,70,105,108,116,101,114,71,114,97,112,104,0,0,0,78,111,32,115,117,99,104,32,111,112,116,105,111,110,58,32,37,115,46,10,0,0,0,0,115,101,116,32,116,104,101,32,97,117,100,105,111,32,111,112,116,105,111,110,115,32,116,111,32,116,104,101,32,105,110,100,105,99,97,116,101,100,32,112,114,101,115,101,116,0,0,0,116,105,116,108,101,0,0,0,109,97,99,114,111,98,108,111,99,107,32,99,111,109,112,97,114,101,32,102,117,110,99,116,105,111,110,0,0,0,0,0,67,111,109,109,111,100,111,114,101,32,67,68,88,76,32,118,105,100,101,111,0,0,0,0,97,112,114,101,0,0,0,0,115,104,111,119,32,118,101,114,115,105,111,110,0,0,0,0,109,98,99,109,112,0,0,0,99,100,120,108,0,0,0,0,68,97,110,99,101,0,0,0,69,114,114,111,114,32,114,101,97,100,105,110,103,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,100,101,115,99,114,105,112,116,105,111,110,46,10,0,0,0,0,118,105,100,101,111,32,98,105,116,115,116,114,101,97,109,95,102,105,108,116,101,114,115,0,97,108,98,117,109,0,0,0,115,117,98,45,112,101,108,32,77,69,32,99,111,109,112,97,114,101,32,102,117,110,99,116,105,111,110,0,0,0,0,0,88,87,68,32,40,88,32,87,105,110,100,111,119,32,68,117,109,112,41,32,105,109,97,103,101,0,0,0,0,0,0,0,77,105,115,115,105,110,103,32,97,114,103,117,109,101,110,116,32,102,111,114,32,111,112,116,105,111,110,32,39,37,115,39,46,10,0,0,0,0,0,0,73,110,118,97,108,105,100,32,116,97,103,32,115,105,122,101,32,37,117,46,10,0,0,0,118,98,115,102,0,0,0,0,102,111,117,110,100,32,109,111,114,101,32,116,104,97,110,32,111,110,101,32,39,102,109,116,32,39,32,116,97,103,10,0,116,114,97,99,107,0,0,0,115,117,98,99,109,112,0,0,120,119,100,0,0,0,0,0,97,117,100,105,111,32,98,105,116,115,116,114,101,97,109,95,102,105,108,116,101,114,115,0,99,109,112,95,102,117,110,99,0,0,0,0,0,0,0,0,85,110,99,111,109,112,114,101,115,115,101,100,32,52,58,52,58,52,32,49,48,45,98,105,116,0,0,0,0,0,0,0,103,114,97,121,49,54,108,101,0,0,0,0,0,0,0,0,99,104,97,110,103,105,110,103,32,116,111,32,37,108,108,100,46,32,84,104,105,115,32,109,97,121,32,114,101,115,117,108,116,32,105,110,32,105,110,99,111,114,114,101,99,116,32,116,105,109,101,115,116,97,109,112,115,32,105,110,32,116,104,101,32,111,117,116,112,117,116,32,102,105,108,101,46,10,0,0,67,111,114,110,102,108,111,119,101,114,66,108,117,101,0,0,100,101,112,114,101,99,97,116,101,100,0,0,0,0,0,0,32,10,9,0,0,0,0,0,119,104,105,108,101,0,0,0,83,84,82,69,65,77,95,78,79,84,95,70,79,85,78,68,0,0,0,0,0,0,0,0,116,111,112,32,102,114,111,110,116,32,99,101,110,116,101,114,0,0,0,0,0,0,0,0,102,117,108,108,45,112,101,108,32,77,69,32,99,111,109,112,97,114,101,32,102,117,110,99,116,105,111,110,0,0,0,0,100,105,116,104,101,114,101,100,32,0,0,0,0,0,0,0,118,52,49,48,0,0,0,0,115,114,99,119,0,0,0,0,73,68,51,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,99,111,109,112,101,110,115,97,116,101,32,102,111,114,32,116,105,109,101,115,116,97,109,112,32,100,101,108,116,97,32,111,102,32,37,102,10,0,97,98,115,102,0,0,0,0,73,110,118,97,108,105,100,32,110,117,109,95,98,97,110,100,115,95,48,58,32,37,100,10,0,0,0,0,0,0,0,0,83,83,82,0,0,0,0,0,99,111,109,109,101,110,116,0,85,110,97,98,108,101,32,116,111,32,111,112,101,110,32,105,99,111,110,118,32,99,111,110,116,101,120,116,32,119,105,116,104,32,105,110,112,117,116,32,99,104,97,114,97,99,116,101,114,32,101,110,99,111,100,105,110,103,32,34,37,115,34,10,0,0,0,0,0,0,0,0,99,109,112,0,0,0,0,0,68,120,116,111,114,121,0,0,115,101,116,112,116,115,0,0,98,105,116,115,116,114,101,97,109,95,102,105,108,116,101,114,115,0,0,0,0,0,0,0,117,115,101,32,102,111,117,114,32,109,111,116,105,111,110,32,118,101,99,116,111,114,115,32,112,101,114,32,109,97,99,114,111,98,108,111,99,107,32,40,77,80,69,71,45,52,41,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,116,104,101,32,101,110,99,111,100,101,114,32,98,97,110,100,119,105,100,116,104,32,116,111,32,37,100,58,32,37,115,10,0,0,114,101,100,117,99,101,32,98,117,102,102,101,114,105,110,103,0,0,0,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,112,97,114,115,101,32,99,111,118,101,114,32,97,114,116,32,98,108,111,99,107,46,10,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,104,101,97,100,101,114,32,116,121,112,101,32,37,88,10,0,98,97,99,107,119,97,114,100,32,112,114,101,100,105,99,116,101,100,32,77,86,115,32,111,102,32,66,45,102,114,97,109,101,115,0,0,0,0,0,0,102,105,115,98,111,110,101,0,79,112,117,115,84,97,103,115,0,0,0,0,0,0,0,0,100,120,116,111,114,121,0,0,1,68,105,114,101,99,116,32,83,104,111,119,32,83,97,109,112,108,101,115,32,101,109,98,101,100,100,101,100,32,105,110,32,79,103,103,0,0,0,0,109,115,109,112,101,103,52,118,51,0,0,0,0,0,0,0,65,32,99,111,109,109,97,45,115,101,112,97,114,97,116,101,100,32,108,105,115,116,32,111,102,32,98,105,116,115,116,114,101,97,109,32,102,105,108,116,101,114,115,0,0,0,0,0,67,108,97,115,115,105,99,97,108,0,0,0,0,0,0,0,111,103,103,44,111,103,118,44,115,112,120,44,111,112,117,115,0,0,0,0,0,0,0,0,100,115,54,52,0,0,0,0,80,114,111,98,101,32,98,117,102,102,101,114,32,115,105,122,101,32,108,105,109,105,116,32,111,102,32,37,100,32,98,121,116,101,115,32,114,101,97,99,104,101,100,10,0,0,0,0,116,114,117,110,99,97,116,101,100,32,102,105,108,101,10,0,98,98,0,0,0,0,0,0,115,112,101,99,105,102,121,32,104,111,119,32,109,97,110,121,32,109,105,99,114,111,115,101,99,111,110,100,115,32,97,114,101,32,97,110,97,108,121,122,101,100,32,116,111,32,112,114,111,98,101,32,116,104,101,32,105,110,112,117,116,0,0,0,86,66,76,69,32,76,111,115,115,108,101,115,115,32,67,111,100,101,99,0,0,0,0,0,102,108,117,115,104,105,110,103,32,119,105,116,104,32,37,100,32,110,111,110,105,110,116,101,114,108,101,97,118,101,100,10,0,0,0,0,0,0,0,0,78,111,32,112,97,99,107,101,116,115,32,119,101,114,101,32,115,101,110,116,32,102,111,114,32,115,111,109,101,32,111,102,32,116,104,101,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,115,46,10,0,0,0,0,0,0,0,0,98,115,102,0,0,0,0,0,85,115,101,32,77,80,52,65,45,76,65,84,77,32,112,97,99,107,101,116,105,122,97,116,105,111,110,32,105,110,115,116,101,97,100,32,111,102,32,77,80,69,71,52,45,71,69,78,69,82,73,67,32,102,111,114,32,65,65,67,0,0,0,0,84,65,71,0,0,0,0,0,99,111,109,112,111,115,101,114,0,0,0,0,0,0,0,0,99,114,101,97,116,105,111,110,95,116,105,109,101,0,0,0,83,101,99,114,101,116,32,100,97,116,97,58,10,0,0,0,102,111,114,119,97,114,100,32,112,114,101,100,105,99,116,101,100,32,77,86,115,32,111,102,32,66,45,102,114,97,109,101,115,0,0,0,0,0,0,0,37,108,108,100,0,0,0,0,118,98,108,101,0,0,0,0,115,119,115,95,112,97,114,97,109,0,0,0,0,0,0,0,79,117,116,112,117,116,32,112,97,100,32,34,37,115,34,32,119,105,116,104,32,116,121,112,101,32,37,115,32,111,102,32,116,104,101,32,102,105,108,116,101,114,32,105,110,115,116,97,110,99,101,32,34,37,115,34,32,111,102,32,37,115,32,110,111,116,32,99,111,110,110,101,99,116,101,100,32,116,111,32,97,110,121,32,100,101,115,116,105,110,97,116,105,111,110,10,0,0,0,0,0,0,0,0,111,118,101,114,114,105,100,101,32,116,104,101,32,111,112,116,105,111,110,115,32,102,114,111,109,32,102,102,115,101,114,118,101,114,0,0,0,0,0,0,102,111,114,119,97,114,100,32,112,114,101,100,105,99,116,101,100,32,77,86,115,32,111,102,32,80,45,102,114,97,109,101,115,0,0,0,0,0,0,0,68,105,115,99,119,111,114,108,100,32,73,73,32,66,77,86,32,118,105,100,101,111,0,0,111,118,101,114,114,105,100,101,95,102,102,115,101,114,118,101,114,0,0,0,0,0,0,0,118,101,114,115,105,111,110,0,112,116,115,32,104,97,115,32,110,111,32,118,97,108,117,101,10,0,0,0,0,0,0,0,112,102,0,0,0,0,0,0,98,109,118,95,118,105,100,101,111,0,0,0,0,0,0,0,115,101,116,32,116,104,101,32,105,110,105,116,105,97,108,32,100,101,109,117,120,45,100,101,99,111,100,101,32,100,101,108,97,121,0,0,0,0,0,0,100,101,98,117,103,95,109,118,0,0,0,0,0,0,0,0,85,116,32,86,105,100,101,111,0,0,0,0,0,0,0,0,32,109,97,116,99,104,101,100,32,97,115,32,37,115,46,10,0,0,0,0,0,0,0,0,109,117,120,112,114,101,108,111,97,100,0,0,0,0,0,0,118,105,115,117,97,108,105,122,101,32,109,111,116,105,111,110,32,118,101,99,116,111,114,115,32,40,77,86,115,41,0,0,117,116,118,105,100,101,111,0,99,111,109,112,111,115,101,114,0,0,0,0,0,0,0,0,102,105,108,101,58,0,0,0,109,112,101,103,49,118,105,100,101,111,0,0,0,0,0,0,115,101,99,111,110,100,115,0,118,105,115,109,118,0,0,0,108,105,98,97,118,102,111,114,109,97,116,47,97,118,105,111,98,117,102,46,99,0,0,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,86,105,100,101,111,32,57,32,73,109,97,103,101,32,118,50,0,0,103,114,97,121,49,54,98,101,0,0,0,0,0,0,0,0,78,111,32,85,82,76,32,80,114,111,116,111,99,111,108,115,32,97,114,101,32,114,101,103,105,115,116,101,114,101,100,46,32,77,105,115,115,105,110,103,32,99,97,108,108,32,116,111,32,97,118,95,114,101,103,105,115,116,101,114,95,97,108,108,40,41,63,10,0,0,0,0,97,98,111,114,116,105,110,103,46,10,0,0,0,0,0,0,67,111,114,97,108,0,0,0,97,115,102,44,119,109,118,44,119,109,97,0,0,0,0,0,115,101,116,32,116,104,101,32,109,97,120,105,109,117,109,32,100,101,109,117,120,45,100,101,99,111,100,101,32,100,101,108,97,121,0,0,0,0,0,0,69,114,114,111,114,32,115,101,116,116,105,110,103,32,111,112,116,105,111,110,32,37,115,32,116,111,32,118,97,108,117,101,32,37,115,46,10,0,0,0,115,116,0,0,0,0,0,0,80,114,111,116,111,99,111,108,32,110,111,116,32,102,111,117,110,100,0,0,0,0,0,0,84,70,67,0,0,0,0,0,116,104,114,101,97,100,105,110,103,32,111,112,101,114,97,116,105,111,110,115,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,102,114,111,109,32,37,115,32,116,111,32,37,115,37,115,32,0,0,0,0,0,0,0,0,118,99,49,105,109,97,103,101,0,0,0,0,0,0,0,0,101,114,114,111,114,32,100,105,102,102,117,115,105,111,110,32,100,105,116,104,101,114,0,0,109,111,118,32,109,117,120,101,114,0,0,0,0,0,0,0,108,105,98,97,118,102,105,108,116,101,114,47,97,118,99,111,100,101,99,46,99,0,0,0,97,100,100,105,110,103,32,37,100,32,97,117,100,105,111,32,115,97,109,112,108,101,115,32,111,102,32,115,105,108,101,110,99,101,10,0,0,0,0,0,109,117,120,100,101,108,97,121,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,98,105,116,115,116,114,101,97,109,44,32,116,111,111,32,109,97,110,121,32,81,77,70,32,115,117,98,98,97,110,100,115,58,32,37,100,10,0,0,0,80,117,108,115,101,32,100,97,116,97,32,99,111,114,114,117,112,116,32,111,114,32,105,110,118,97,108,105,100,46,10,0,111,99,108,0,0,0,0,0,77,80,69,71,32,84,114,97,110,115,112,111,114,116,0,0,85,84,70,45,56,0,0,0,115,97,109,112,108,101,32,102,111,114,109,97,116,0,0,0,116,104,114,101,97,100,95,111,112,115,0,0,0,0,0,0,112,105,120,95,102,109,116,115,0,0,0,0,0,0,0,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,86,105,100,101,111,32,57,32,73,109,97,103,101,0,0,0,0,0,102,111,114,99,101,32,67,70,82,32,102,111,114,32,105,110,112,117,116,32,102,114,111,109,32,115,116,114,101,97,109,32,37,100,58,37,100,0,0,0,115,116,97,114,116,0,0,0,116,104,105,115,32,111,112,116,105,111,110,32,105,115,32,100,101,112,114,101,99,97,116,101,100,32,97,110,100,32,100,111,101,115,32,110,111,116,104,105,110,103,0,0,0,0,0,0,109,118,52,0,0,0,0,0,99,117,116,111,102,102,32,118,97,108,105,100,32,114,97,110,103,101,32,105,115,32,37,100,45,50,48,48,48,48,10,0,70,82,65,77,69,95,82,65,84,69,0,0,0,0,0,0,112,105,99,116,117,114,101,32,98,117,102,102,101,114,32,97,108,108,111,99,97,116,105,111,110,115,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,119,109,118,51,105,109,97,103,101,0,0,0,0,0,0,0,77,80,69,71,45,52,32,112,97,114,116,32,50,32,77,105,99,114,111,115,111,102,116,32,118,97,114,105,97,110,116,32,118,101,114,115,105,111,110,32,50,0,0,0,0,0,0,0,105,115,121,110,99,0,0,0,84,114,97,110,99,101,0,0,88,105,112,104,46,79,114,103,32,108,105,98,86,111,114,98,105,115,32,73,32,50,48,49,50,48,50,48,51,32,40,79,109,110,105,112,114,101,115,101,110,116,41,0,0,0,0,0,66,117,102,102,101,114,32,105,110,112,117,116,32,102,114,97,109,101,115,32,97,110,100,32,115,101,110,100,32,116,104,101,109,32,119,104,101,110,32,116,104,101,121,32,97,114,101,32,114,101,113,117,101,115,116,101,100,46,0,0,0,0,0,0,82,70,54,52,0,0,0,0,65,108,108,32,105,110,102,111,32,102,111,117,110,100,10,0,116,111,111,32,98,105,103,32,73,78,70,79,32,115,117,98,99,104,117,110,107,10,0,0,98,117,102,102,101,114,115,0,97,110,97,108,121,122,101,100,117,114,97,116,105,111,110,0,112,97,121,108,111,97,100,95,116,121,112,101,0,0,0,0,67,104,114,111,110,111,109,97,115,116,101,114,32,68,70,65,0,0,0,0,0,0,0,0,97,98,117,102,102,101,114,0,115,116,97,110,100,97,114,100,0,0,0,0,0,0,0,0,108,97,116,109,0,0,0,0,97,108,98,117,109,95,97,114,116,105,115,116,0,0,0,0,102,102,97,98,117,102,102,101,114,115,105,110,107,0,0,0,84,68,69,78,0,0,0,0,68,82,77,32,112,114,111,116,101,99,116,101,100,32,115,116,114,101,97,109,32,100,101,116,101,99,116,101,100,44,32,100,101,99,111,100,105,110,103,32,119,105,108,108,32,108,105,107,101,108,121,32,102,97,105,108,33,10,0,0,0,0,0,0,118,105,115,117,97,108,105,122,101,32,98,108,111,99,107,32])
.concat([116,121,112,101,115,0,0,0,110,97,110,0,0,0,0,0,100,102,97,0,0,0,0,0,102,114,97,109,101,95,114,97,116,101,0,0,0,0,0,0,73,110,118,97,108,105,100,32,115,105,122,101,32,102,111,114,32,112,105,120,101,108,95,102,109,116,115,58,32,37,100,44,32,115,104,111,117,108,100,32,98,101,32,109,117,108,116,105,112,108,101,32,111,102,32,37,100,10,0,0,0,0,0,0,117,110,107,110,111,119,110,0,73,110,112,117,116,32,112,97,100,32,34,37,115,34,32,119,105,116,104,32,116,121,112,101,32,37,115,32,111,102,32,116,104,101,32,102,105,108,116,101,114,32,105,110,115,116,97,110,99,101,32,34,37,115,34,32,111,102,32,37,115,32,110,111,116,32,99,111,110,110,101,99,116,101,100,32,116,111,32,97,110,121,32,115,111,117,114,99,101,10,0,0,0,0,0,0,99,111,109,112,97,116,58,32,99,97,108,108,101,100,32,119,105,116,104,32,97,114,103,115,61,91,37,115,93,10,0,0,100,101,112,114,101,99,97,116,101,100,44,32,117,115,101,32,45,115,116,97,110,100,97,114,100,0,0,0,0,0,0,0,117,110,100,0,0,0,0,0,118,105,115,95,109,98,95,116,121,112,101,0,0,0,0,0,63,0,0,0,0,0,0,0,84,104,105,115,32,105,115,32,97,32,102,97,116,97,108,32,101,114,114,111,114,46,32,32,73,116,32,104,97,115,32,115,101,118,101,114,97,108,32,112,111,115,115,105,98,108,101,32,99,97,117,115,101,115,58,0,66,105,116,109,97,112,32,66,114,111,116,104,101,114,115,32,74,86,32,118,105,100,101,111,0,0,0,0,0,0,0,0,116,118,115,116,100,0,0,0,45,104,101,108,112,0,0,0,118,105,115,117,97,108,105,122,101,32,113,117,97,110,116,105,122,97,116,105,111,110,32,112,97,114,97,109,101,116,101,114,32,40,81,80,41,44,32,108,111,119,101,114,32,81,80,32,97,114,101,32,116,105,110,116,101,100,32,103,114,101,101,110,101,114,0,0,0,0,0,0,106,118,0,0,0,0,0,0,77,52,65,32,0,0,0,0,99,104,97,110,110,101,108,0,118,105,115,95,113,112,0,0,65,112,112,108,101,32,80,114,111,82,101,115,32,40,105,67,111,100,101,99,32,80,114,111,41,0,0,0,0,0,0,0,77,52,86,32,0,0,0,0,82,101,97,100,105,110,103,32,111,112,116,105,111,110,32,39,37,115,39,32,46,46,46,0,100,101,112,114,101,99,97,116,101,100,44,32,117,115,101,32,45,99,104,97,110,110,101,108,0,0,0,0,0,0,0,0,98,117,103,115,0,0,0,0,112,114,111,114,101,115,0,0,118,99,0,0,0,0,0,0,109,101,109,111,114,121,32,109,97,110,97,103,101,109,101,110,116,32,99,111,110,116,114,111,108,32,111,112,101,114,97,116,105,111,110,115,32,40,72,46,50,54,52,41,0,0,0,0,76,97,103,97,114,105,116,104,32,108,111,115,115,108,101,115,115,0,0,0,0,0,0,0,98,103,114,97,0,0,0,0,78,111,110,45,109,111,110,111,116,111,110,111,117,115,32,68,84,83,32,105,110,32,111,117,116,112,117,116,32,115,116,114,101,97,109,32,37,100,58,37,100,59,32,112,114,101,118,105,111,117,115,58,32,37,108,108,100,44,32,99,117,114,114,101,110,116,58,32,37,108,108,100,59,32,0,0,0,0,0,0,67,104,111,99,111,108,97,116,101,0,0,0,0,0,0,0,115,101,116,32,99,97,110,118,97,115,32,115,105,122,101,32,40,87,120,72,32,111,114,32,97,98,98,114,101,118,105,97,116,105,111,110,41,0,0,0,105,115,105,110,102,0,0,0,80,82,79,84,79,67,79,76,95,78,79,84,95,70,79,85,78,68,0,0,0,0,0,0,116,111,112,32,102,114,111,110,116,32,108,101,102,116,0,0,109,109,99,111,0,0,0,0,101,104,104,32,102,108,97,103,115,32,105,110,118,97,108,105,100,63,33,32,0,0,0,0,108,97,103,97,114,105,116,104,0,0,0,0,0,0,0,0,101,114,114,111,114,95,100,105,102,102,117,115,105,111,110,0,51,103,50,97,0,0,0,0,100,105,115,99,97,114,100,105,110,103,32,37,100,32,97,117,100,105,111,32,115,97,109,112,108,101,115,10,0,0,0,0,99,97,110,118,97,115,95,115,105,122,101,0,0,0,0,0,108,105,98,97,118,99,111,100,101,99,47,97,97,99,115,98,114,46,99,0,0,0,0,0,80,117,108,115,101,32,116,111,111,108,32,110,111,116,32,97,108,108,111,119,101,100,32,105,110,32,101,105,103,104,116,32,115,104,111,114,116,32,115,101,113,117,101,110,99,101,46,10,0,0,0,0,0,0,0,0,67,111,100,101,99,32,39,37,115,39,32,105,115,32,98,105,116,109,97,112,45,98,97,115,101,100,44,32,115,117,98,116,105,116,108,101,115,32,99,104,97,114,97,99,116,101,114,32,101,110,99,111,100,105,110,103,32,119,105,108,108,32,98,101,32,105,103,110,111,114,101,100,10,0,0,0,0,0,0,0,101,114,114,111,114,32,114,101,99,111,103,110,105,116,105,111,110,0,0,0,0,0,0,0,77,111,98,111,116,105,120,32,77,120,80,69,71,32,118,105,100,101,111,0,0,0,0,0,58,102,114,97,109,101,95,114,97,116,101,61,37,100,47,37,100,0,0,0,0,0,0,0,51,103,50,98,0,0,0,0,102,105,120,32,115,117,98,116,105,116,108,101,115,32,100,117,114,97,116,105,111,110,0,0,97,108,108,111,119,32,100,101,99,111,100,101,114,115,32,116,111,32,112,114,111,100,117,99,101,32,117,110,97,108,105,103,110,101,100,32,111,117,116,112,117,116,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,97,102,116,101,114,98,117,114,110,101,114,32,116,111,32,37,100,58,32,37,115,10,0,0,0,0,101,114,0,0,0,0,0,0,109,120,112,101,103,0,0,0,51,103,112,52,0,0,0,0,109,115,109,112,101,103,52,118,50,0,0,0,0,0,0,0,102,105,120,95,115,117,98,95,100,117,114,97,116,105,111,110,0,0,0,0,0,0,0,0,70,117,115,105,111,110,0,0,115,101,114,118,105,99,101,95,110,97,109,101,0,0,0,0,82,73,70,70,0,0,0,0,105,110,116,101,114,114,117,112,116,101,100,10,0,0,0,0,73,78,70,79,32,115,117,98,99,104,117,110,107,32,116,114,117,110,99,97,116,101,100,10,0,0,0,0,0,0,0,0,112,116,115,0,0,0,0,0,102,111,114,99,101,115,32,115,101,101,107,105,110,103,32,116,111,32,101,110,97,98,108,101,32,115,101,101,107,32,116,111,32,97,110,121,32,109,111,100,101,0,0,0,0,0,0,0,83,105,108,105,99,111,110,32,71,114,97,112,104,105,99,115,32,77,111,116,105,111,110,32,86,105,100,101,111,32,67,111,109,112,114,101,115,115,111,114,32,50,0,0,0,0,0,0,69,110,97,98,108,101,32,73,68,51,118,49,32,119,114,105,116,105,110,103,46,32,73,68,51,118,49,32,116,97,103,115,32,97,114,101,32,119,114,105,116,116,101,110,32,105,110,32,85,84,70,45,56,32,119,104,105,99,104,32,109,97,121,32,110,111,116,32,98,101,32,115,117,112,112,111,114,116,101,100,32,98,121,32,109,111,115,116,32,115,111,102,116,119,97,114,101,46,0,0,0,0,0,0,51,103,112,54,0,0,0,0,102,111,114,99,101,32,115,117,98,116,105,116,108,101,32,116,97,103,47,102,111,117,114,99,99,0,0,0,0,0,0,0,82,84,80,32,109,117,120,101,114,32,102,108,97,103,115,0,97,114,116,105,115,116,0,0,84,68,82,67,0,0,0,0,112,97,99,107,101,116,95,102,114,97,103,95,115,105,122,101,32,105,115,32,105,110,118,97,108,105,100,32,40,37,100,45,37,100,41,10,0,0,0,0,115,116,97,114,116,99,111,100,101,0,0,0,0,0,0,0,32,45,62,32,80,84,83,58,37,115,32,84,58,37,102,10,0,0,0,0,0,0,0,0,109,118,99,50,0,0,0,0,112,105,120,101,108,95,97,115,112,101,99,116,0,0,0,0,67,111,110,102,108,105,99,116,105,110,103,32,97,108,108,95,99,104,97,110,110,101,108,95,99,111,117,110,116,115,32,97,110,100,32,108,105,115,116,32,105,110,32,111,112,116,105,111,110,115,10,0,0,0,0,0,97,117,116,111,45,105,110,115,101,114,116,101,100,32,102,105,102,111,32,37,100,0,0,0,84,104,105,115,32,115,121,110,116,97,120,32,105,115,32,100,101,112,114,101,99,97,116,101,100,46,32,85,115,101,32,39,124,39,32,116,111,32,115,101,112,97,114,97,116,101,32,116,104,101,32,108,105,115,116,32,105,116,101,109,115,46,10,0,115,116,97,103,0,0,0,0,115,107,105,112,0,0,0,0,83,105,108,105,99,111,110,32,71,114,97,112,104,105,99,115,32,77,111,116,105,111,110,32,86,105,100,101,111,32,67,111,109,112,114,101,115,115,111,114,32,49,0,0,0,0,0,0,102,111,114,99,101,32,115,117,98,116,105,116,108,101,32,99,111,100,101,99,32,40,39,99,111,112,121,39,32,116,111,32,99,111,112,121,32,115,116,114,101,97,109,41,0,0,0,0,104,101,108,112,0,0,0,0,71,101,116,116,105,110,103,32,104,101,108,112,58,10,32,32,32,32,45,104,32,32,32,32,32,32,45,45,32,112,114,105,110,116,32,98,97,115,105,99,32,111,112,116,105,111,110,115,10,32,32,32,32,45,104,32,108,111,110,103,32,45,45,32,112,114,105,110,116,32,109,111,114,101,32,111,112,116,105,111,110,115,10,32,32,32,32,45,104,32,102,117,108,108,32,45,45,32,112,114,105,110,116,32,97,108,108,32,111,112,116,105,111,110,115,32,40,105,110,99,108,117,100,105,110,103,32,97,108,108,32,102,111,114,109,97,116,32,97,110,100,32,99,111,100,101,99,32,115,112,101,99,105,102,105,99,32,111,112,116,105,111,110,115,44,32,118,101,114,121,32,108,111,110,103,41,10,32,32,32,32,83,101,101,32,109,97,110,32,37,115,32,102,111,114,32,100,101,116,97,105,108,101,100,32,100,101,115,99,114,105,112,116,105,111,110,32,111,102,32,116,104,101,32,111,112,116,105,111,110,115,46,10,10,0,0,0,0,0,0,100,99,116,95,99,111,101,102,102,0,0,0,0,0,0,0,109,118,99,49,0,0,0,0,115,99,111,100,101,99,0,0,109,111,116,105,111,110,32,118,101,99,116,111,114,0,0,0,65,74,65,32,75,111,110,97,32,49,48,45,98,105,116,32,82,71,66,32,67,111,100,101,99,0,0,0,0,0,0,0,83,112,108,105,116,116,105,110,103,32,116,104,101,32,99,111,109,109,97,110,100,108,105,110,101,46,10,0,0,0,0,0,100,105,115,97,98,108,101,32,115,117,98,116,105,116,108,101,0,0,0,0,0,0,0,0,109,118,0,0,0,0,0,0,114,49,48,107,0,0,0,0,115,110,0,0,0,0,0,0,112,101,114,45,98,108,111,99,107,32,113,117,97,110,116,105,122,97,116,105,111,110,32,112,97,114,97,109,101,116,101,114,32,40,81,80,41,0,0,0,77,117,108,116,105,99,111,108,111,114,32,99,104,97,114,115,101,116,32,102,111,114,32,67,111,109,109,111,100,111,114,101,32,54,52,44,32,101,120,116,101,110,100,101,100,32,119,105,116,104,32,53,116,104,32,99,111,108,111,114,32,40,99,111,108,114,97,109,41,0,0,0,97,98,103,114,0,0,0,0,99,111,112,121,0,0,0,0,67,104,97,114,116,114,101,117,115,101,0,0,0,0,0,0,115,101,116,32,116,104,101,32,109,97,120,105,109,117,109,32,110,117,109,98,101,114,32,111,102,32,99,104,97,110,110,101,108,115,32,116,111,32,116,114,121,32,116,111,32,103,117,101,115,115,32,116,104,101,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,0,0,0,105,115,110,97,110,0,0,0,78,111,116,32,121,101,116,32,105,109,112,108,101,109,101,110,116,101,100,32,105,110,32,70,70,109,112,101,103,44,32,112,97,116,99,104,101,115,32,119,101,108,99,111,109,101,0,0,84,70,76,0,0,0,0,0,113,112,0,0,0,0,0,0,66,105,99,117,98,105,99,32,115,112,108,105,110,101,32,115,99,97,108,101,114,44,32,0,97,54,52,95,109,117,108,116,105,53,0,0,0,0,0,0,97,45,62,99,104,95,99,111,117,110,116,0,0,0,0,0,103,117,101,115,115,95,108,97,121,111,117,116,95,109,97,120,0,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,80,114,101,100,105,99,116,111,114,32,82,101,115,101,116,32,71,114,111,117,112,46,10,0,67,104,97,114,97,99,116,101,114,32,101,110,99,111,100,105,110,103,32,105,115,32,111,110,108,121,32,115,117,112,112,111,114,116,101,100,32,119,105,116,104,32,115,117,98,116,105,116,108,101,115,32,99,111,100,101,99,115,10,0,0,0,0,0,109,97,99,114,111,98,108,111,99,107,32,40,77,66,41,32,116,121,112,101,0,0,0,0,77,117,108,116,105,99,111,108,111,114,32,99,104,97,114,115,101,116,32,102,111,114,32,67,111,109,109,111,100,111,114,101,32,54,52,0,0,0,0,0,118,105,100,101,111,95,115,105,122,101,61,37,100,120,37,100,58,112,105,120,95,102,109,116,61,37,100,58,116,105,109,101,95,98,97,115,101,61,37,100,47,37,100,58,112,105,120,101,108,95,97,115,112,101,99,116,61,37,100,47,37,100,58,115,119,115,95,112,97,114,97,109,61,102,108,97,103,115,61,37,100,0,0,0,0,0,0,0,87,97,114,110,105,110,103,44,32,101,120,116,101,110,115,105,111,110,32,105,115,32,110,111,116,32,46,109,52,97,32,110,111,114,32,46,109,52,118,32,81,117,105,99,107,116,105,109,101,47,73,112,111,100,32,109,105,103,104,116,32,110,111,116,32,112,108,97,121,32,116,104,101,32,102,105,108,101,10,0,115,101,116,32,97,117,100,105,111,32,102,105,108,116,101,114,115,0,0,0,0,0,0,0,117,110,97,108,105,103,110,101,100,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,115,105,103,110,97,108,105,110,103,32,109,111,100,101,32,37,100,58,32,37,115,10,0,0,0,0,109,98,95,116,121,112,101,0,97,54,52,95,109,117,108,116,105,0,0,0,0,0,0,0,109,52,118,0,0,0,0,0,77,80,69,71,45,52,32,112,97,114,116,32,50,32,77,105,99,114,111,115,111,102,116,32,118,97,114,105,97,110,116,32,118,101,114,115,105,111,110,32,49,0,0,0,0,0,0,0,97,102,0,0,0,0,0,0,74,97,122,122,43,70,117,110,107,0,0,0,0,0,0,0,87,77,47,77,101,100,105,97,83,116,97,116,105,111,110,78,97,109,101,0,0,0,0,0,87,65,86,69,0,0,0,0,49,0,0,0,0,0,0,0,114,97,116,101,32,99,111,110,116,114,111,108,0,0,0,0,115,101,101,107,50,97,110,121,0,0,0,0,0,0,0,0,65,83,67,73,73,47,65,78,83,73,32,97,114,116,0,0,119,114,105,116,101,95,105,100,51,118,49,0,0,0,0,0,109,52,97,0,0,0,0,0,66,108,117,101,115,0,0,0,108,97,121,111,117,116,0,0,114,116,112,102,108,97,103,115,0,0,0,0,0,0,0,0,116,105,116,108,101,0,0,0,100,97,116,101,0,0,0,0,112,97,99,107,101,116,95,114,101,112,108,105,99,95,115,105,122,101,32,105,115,32,105,110,118,97,108,105,100,10,0,0,114,99,0,0,0,0,0,0,32,78,66,95,83,65,77,80,76,69,83,58,37,108,108,100,32,78,66,95,67,79,78,83,85,77,69,68,95,83,65,77,80,76,69,83,58,37,108,108,100,0,0,0,0,0,0,0,97,110,115,105,0,0,0,0,115,97,109,112,108,101,32,97,115,112,101,99,116,32,114,97,116,105,111,0,0,0,0,0,73,110,118,97,108,105,100,32,115,105,122,101,32,102,111,114,32,99,104,97,110,110,101,108,95,99,111,117,110,116,115,58,32,37,100,44,32,115,104,111,117,108,100,32,98,101,32,109,117,108,116,105,112,108,101,32,111,102,32,37,100,10,0,0,117,110,115,117,112,112,111,114,116,101,100,32,104,101,105,103,104,116,32,102,111,114,32,100,118,32,99,111,100,101,99,10,0,0,0,0,0,0,0,0,97,102,105,102,111,0,0,0,97,101,118,97,108,115,114,99,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,112,105,99,116,117,114,101,32,116,121,112,101,58,32,37,100,46,10,0,0,0,0,0,0,115,101,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,0,0,0,0,0,0,112,105,99,116,117,114,101,32,105,110,102,111,0,0,0,0,80,105,99,116,111,114,47,80,67,32,80,97,105,110,116,0,85,115,105,110,103,32,77,83,32,115,116,121,108,101,32,97,117,100,105,111,32,99,111,100,101,99,32,116,97,103,44,32,116,104,101,32,102,105,108,101,32,109,97,121,32,98,101,32,117,110,112,108,97,121,97,98,108,101,33,10,0,0,0,0,99,104,97,110,110,101,108,95,108,97,121,111,117,116,0,0,63,0,0,0,0,0,0,0,112,105,99,116,0,0,0,0,112,105,99,116,111,114,0,0,85,115,105,110,103,32,77,83,32,115,116,121,108,101,32,118,105,100,101,111,32,99,111,100,101,99,32,116,97,103,44,32,116,104,101,32,102,105,108,101,32,109,97,121,32,98,101,32,117,110,112,108,97,121,97,98,108,101,33,10,0,0,0,0,115,101,116,32,115,97,109,112,108,101,32,102,111,114,109,97,116,0,0,0,0,0,0,0,112,114,105,110,116,32,115,112,101,99,105,102,105,99,32,100,101,98,117,103,32,105,110,102,111,0,0,0,0,0,0,0,87,101,98,80,0,0,0,0,119,105,100,101,0,0,0,0,69,114,114,111,114,32,115,101,116,116,105,110,103,32,111,112,116,105,111,110,32,37,115,46,10,0,0,0,0,0,0,0,115,97,109,112,108,101,95,102,109,116,0,0,0,0,0,0,100,101,98,117,103,0,0,0,119,101,98,112,0,0,0,0,118,111,108,117,109,101,0,0,87,77,47,65,108,98,117,109,65,114,116,105,115,116,0,0,115,97,109,112,108,101,32,97,115,112,101,99,116,32,114,97,116,105,111,0,0,0,0,0,71,111,111,103,108,101,32,86,80,57,0,0,0,0,0,0,114,103,98,97,0,0,0,0,116,105,109,101,99,111,100,101,58,32,116,98,99,61,37,100,47,37,100,32,105,110,118,97,108,105,100,44,32,102,97,108,108,98,97,99,107,32,111,110,32,37,100,47,37,100,10,0,70,97,105,108,101,100,32,116,111,32,111,112,101,110,32,98,105,116,115,116,114,101,97,109,32,102,105,108,116,101,114,32,37,115,32,102,111,114,32,115,116,114,101,97,109,32,37,100,32,119,105,116,104,32,99,111,100,101,99,32,37,115,0,0,67,97,100,101,116,66,108,117,101,0,0,0,0,0,0,0,99,104,97,110,103,101,32,97,117,100,105,111,32,118,111,108,117,109,101,32,40,50,53,54,61,110,111,114,109,97,108,41,0,0,0,0,0,0,0,0,108,100,0,0,0,0,0,0,80,65,84,67,72,87,69,76,67,79,77,69,0,0,0,0,116,111,112,32,99,101,110,116,101,114,0,0,0,0,0,0,97,115,112,101,99,116,0,0,76,97,110,99,122,111,115,32,115,99,97,108,101,114,44,32,0,0,0,0,0,0,0,0,118,112,57,0,0,0,0,0,98,105,116,101,120,97,99,116,0,0,0,0,0,0,0,0,60,112,97,114,97,109,32,110,97,109,101,61,34,37,115,34,32,118,97,108,117,101,61,34,37,100,34,32,118,97,108,117,101,116,121,112,101,61,34,100,97,116,97,34,47,62,10,0,97,45,62,98,112,115,0,0,118,111,108,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,78,117,109,98,101,114,32,111,102,32,115,99,97,108,101,102,97,99,116,111,114,32,98,97,110,100,115,32,105,110,32,103,114,111,117,112,32,40,37,100,41,32,101,120,99,101,101,100,115,32,108,105,109,105,116,32,40,37,100,41,46,10,0,0,67,104,97,110,110,101,108,32,108,97,121,111,117,116,32,39,37,115,39,32,119,105,116,104,32,37,100,32,99,104,97,110,110,101,108,115,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,115,112,101,99,105,102,105,101,100,32,110,117,109,98,101,114,32,111,102,32,99,104,97,110,110,101,108,115,32,37,100,58,32,105,103,110,111,114,105,110,103,32,115,112,101,99,105,102,105,101,100,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,10,0,109,101,100,105,97,110,0,0,79,110,50,32,86,80,56,0,98,117,102,102,101,114,0,0,60,112,97,114,97,109,32,110,97,109,101,61,34,37,115,34,32,118,97,108,117,101,61,34,37,115,34,32,118,97,108,117,101,116,121,112,101,61,34,100,97,116,97,34,47,62,10,0,102,111,114,99,101,32,97,117,100,105,111,32,116,97,103,47,102,111,117,114,99,99,0,0,83,101,116,32,118,105,100,101,111,32,98,105,116,114,97,116,101,32,116,111,108,101,114,97,110,99,101,32,40,105,110,32,98,105,116,115,47,115,41,46,32,73,110,32,49,45,112,97,115,115,32,109,111,100,101,44,32,98,105,116,114,97,116,101,32,116,111,108,101,114,97,110,99,101,32,115,112,101,99,105,102,105,101,115,32,104,111,119,32,102,97,114,32,114,97,116,101,99,111,110,116,114,111,108,32,105,115,32,119,105,108,108,105,110,103,32,116,111,32,100,101,118,105,97,116,101,32,102,114,111,109,32,116,104,101,32,116,97,114,103,101,116,32,97,118,101,114,97,103,101,32,98,105,116,114,97,116,101,32,118,97,108,117,101,46,32,84,104,105,115,32,105,115,32,110,111,116,32,114,101,108,97,116,101,100,32,116,111,32,109,105,110,105,109,117,109,47,109,97,120,105,109,117,109,32,98,105,116,114,97,116,101,46,32,76,111,119,101,114,105,110,103,32,116,111,108,101,114,97,110,99,101,32,116,111,111,32,109,117,99,104,32,104,97,115,32,97,110,32,97,100,118,101,114,115,101,32,101,102,102,101,99,116,32,111,110,32,113,117,97,108,105,116,121,46,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,104,101,97,100,101,114,32,112,101,114,105,111,100,58,32,37,115,10,0,0,0,0,0,0,0,0,112,108,97,110,101,0,0,0,118,112,56,0,0,0,0,0,60,47,115,109,105,108,62,10,0,0,0,0,0,0,0,0,109,115,109,112,101,103,52,118,49,0,0,0,0,0,0,0,97,116,97,103,0,0,0,0,86,111,99,97,108,0,0,0,115,101,114,118,105,99,101,95,112,114,111,118,105,100,101,114,0,0,0,0,0,0,0,0,116,104,114,101,97,100,115,0,108,101,102,116,0,0,0,0,114,101,100,117,99,101,32,116,104,101,32,108,97,116,101,110,99,121,32,105,110,116,114,111,100,117,99,101,100,32,98,121,32,111,112,116,105,111,110,97,108,32,98,117,102,102,101,114,105,110,103,0,0,0,0,0,80,115,121,103,110,111,115,105,115,32,89,79,80,32,86,105,100,101,111,0,0,0,0,0,83,101,108,101,99,116,32,73,68,51,118,50,32,118,101,114,115,105,111,110,32,116,111,32,119,114,105,116,101,46,32,67,117,114,114,101,110,116,108,121,32,51,32,97,110,100,32,52,32,97,114,101,32,115,117,112,112,111,114,116,101,100,46,0,60,47,98,111,100,121,62,10,0,0,0,0,0,0,0,0,102,111,114,99,101,32,97,117,100,105,111,32,99,111,100,101,99,32,40,39,99,111,112,121,39,32,116,111,32,99,111,112,121,32,115,116,114,101,97,109,41,0,0,0,0,0,0,0,82,117,110,32,97,32,115,101,99,111,110,100,32,112,97,115,115,32,116,111,32,112,117,116,32,116,104,101,32,109,111,111,118,32,97,116,32,116,104,101,32,98,101,103,105,110,110,105,110,103,32,111,102,32,116,104,101,32,102,105,108,101,0,0,117,110,100,111,105,110,103,32,111,118,101,114,114,101,97,100,32,111,102,32,37,108,108,100,32,105,110,32,39,37,46,52,115,39,10,0,0,0,0,0,84,68,82,76,0,0,0,0,117,110,101,120,112,101,99,116,101,100,32,112,97,99,107,101,116,95,114,101,112,108,105,99,95,115,105,122,101,32,111,102,32,37,100,10,0,0,0,0,112,114,101,100,105,99,116,105,111,110,32,109,101,116,104,111,100,0,0,0,0,0,0,0,32,73,78,84,69,82,76,65,67,69,68,58,37,108,108,100,0,0,0,0,0,0,0,0,121,111,112,0,0,0,0,0,115,97,114,0,0,0,0,0,73,110,118,97,108,105,100,32,115,105,122,101,32,102,111,114,32,99,104,97,110,110,101,108,95,108,97,121,111,117,116,115,58,32,37,100,44,32,115,104,111,117,108,100,32,98,101,32,109,117,108,116,105,112,108,101,32,111,102,32,37,100,10,0,60,47,115,119,105,116,99,104,62,10,0,0,0,0,0,0,102,105,102,111,0,0,0,0,112,112,0,0,0,0,0,0,97,99,111,100,101,99,0,0,112,114,101,100,0,0,0,0,75,101,103,97,32,71,97,109,101,32,86,105,100,101,111,0,60,47,37,115,62,10,0,0,100,105,115,97,98,108,101,32,97,117,100,105,111,0,0,0,116,111,112,105,99,0,0,0,65,80,69,84,65,71,69,88,0,0,0,0,0,0,0,0,98,105,116,115,95,112,101,114,95,99,111,100,101,100,95,115,97,109,112,108,101,0,0,0,107,103,118,49,0,0,0,0,80,97,99,107,101,116,83,105,122,101,0,0,0,0,0,0,97,97,99,0,0,0,0,0,97,110,0,0,0,0,0,0,117,115,101,32,115,116,114,111,110,103,32,100,101,98,108,111,99,107,32,102,105,108,116,101,114,32,102,111,114,32,100,97,109,97,103,101,100,32,77,66,115,0,0,0,0,0,0,0,73,70,70,32,66,121,116,101,82,117,110,49,0,0,0,0,66,105,116,115,80,101,114,83,97,109,112,108,101,0,0,0,82,111,117,116,105,110,103,32,111,112,116,105,111,110,32,37,115,32,116,111,32,98,111,116,104,32,99,111,100,101,99,32,97,110,100,32,109,117,120,101,114,32,108,97,121,101,114,10,0,0,0,0,0,0,0,0,99,104,97,110,110,101,108,115,0,0,0,0,0,0,0,0,69,120,112,101,99,116,101,100,32,105,110,116,54,52,32,102,111,114,32,37,115,32,98,117,116,32,102,111,117,110,100,32,37,115,10,0,0,0,0,0,100,101,98,108,111,99,107,0,105,102,102,95,98,121,116,101,114,117,110,49,0,0,0,0,83,97,109,112,108,105,110,103,82,97,116,101,0,0,0,0,115,101,116,32,110,117,109,98,101,114,32,111,102,32,97,117,100,105,111,32,99,104,97,110,110,101,108,115,0,0,0,0,105,116,101,114,97,116,105,118,101,32,109,111,116,105,111,110,32,118,101,99,116,111,114,32,40,77,86,41,32,115,101,97,114,99,104,32,40,115,108,111,119,41,0,0,0,0,0,0,73,70,70,32,73,76,66,77,0,0,0,0,0,0,0,0,97,114,103,98,0,0,0,0,67,104,97,110,110,101,108,115,0,0,0,0,0,0,0,0,116,121,112,101,61,32,37,99,10,0,0,0,0,0,0,0,66,117,114,108,121,87,111,111,100,0,0,0,0,0,0,0,97,99,0,0,0,0,0,0,108,116,0,0,0,0,0,0,79,112,116,105,111,110,32,110,111,116,32,102,111,117,110,100,0,0,0,0,0,0,0,0,84,67,0,0,0,0,0,0,119,97,118,0,0,0,0,0,103,117,101,115,115,95,109,118,115,0,0,0,0,0,0,0,83,105,110,99,32,115,99,97,108,101,114,44,32,0,0,0,105,102,102,95,105,108,98,109,0,0,0,0,0,0,0,0,102,117,108,108,32,99,104,114,111,109,97,32,105,110,112,117,116,0,0,0,0,0,0,0,65,117,100,105,111,84,97,103,0,0,0,0,0,0,0,0,115,45,62,111,117,116,46,99,104,95,99,111,117,110,116,0,115,101,116,32,97,117,100,105,111,32,115,97,109,112,108,105,110,103,32,114,97,116,101,32,40,105,110,32,72,122,41,0,73,110,118,97,108,105,100,32,98,115,95,115,116,111,112,95,102,114,101,113,58,32,37,100,10,0,0,0,0,0,0,0,80,114,101,100,105,99,116,105,111,110,32,105,115,32,110,111,116,32,97,108,108,111,119,101,100,32,105,110,32,65,65,67,45,76,67,46,10,0,0,0,66,105,116,114,97,116,101,32,37,100,32,105,115,32,101,120,116,114,101,109,101,108,121,32,108,111,119,44,32,109,97,121,98,101,32,121,111,117,32,109,101,97,110,32,37,100,107,10,0,0,0,0,0,0,0,0,115,101,116,32,101,114,114,111,114,32,99,111,110,99,101,97,108,109,101,110,116,32,115,116,114,97,116,101,103,121,0,0,66,105,110,107,32,118,105,100,101,111,0,0,0,0,0,0,118,111,108,117,109,101,0,0,87,77,65,80,0,0,0,0,65,76,66,85,77,65,82,84,73,83,84,0,0,0,0,0,97,114,0,0,0,0,0,0,98,116,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,116,104,101,32,116,114,97,110,115,109,117,120,32,102,111,114,109,97,116,58,32,37,115,10,0,0,101,99,0,0,0,0,0,0,98,105,110,107,118,105,100,101,111,0,0,0,0,0,0,0,65,65,67,76,0,0,0,0,114,97,119,32,118,105,100,101,111,0,0,0,0,0,0,0,113,117,97,108,105,116,121,0,84,114,105,112,45,72,111,112,0,0,0,0,0,0,0,0,87,77,47,77,101,100,105,97,83,116,97,116,105,111,110,67,97,108,108,83,105,103,110,0,112,97,114,115,101,114,32,110,111,116,32,102,111,117,110,100,32,102,111,114,32,99,111,100,101,99,32,37,115,44,32,112,97,99,107,101,116,115,32,111,114,32,116,105,109,101,115,32,109,97,121,32,98,101,32,105,110,118,97,108,105,100,46,10,0,0,0,0,0,0,0,0,114,101,113,117,101,115,116,101,100,32,98,105,116,115,95,112,101,114,95,99,111,100,101,100,95,115,97,109,112,108,101,32,40,37,100,41,32,97,110,100,32,97,99,116,117,97,108,108,121,32,115,116,111,114,101,100,32,40,37,100,41,32,100,105,102,102,101,114,10,0,0,0,115,108,105,99,101,95,99,111,117,110,116,0,0,0,0,0,110,111,98,117,102,102,101,114,0,0,0,0,0,0,0,0,68,101,108,117,120,101,32,80,97,105,110,116,32,65,110,105,109,97,116,105,111,110,0,0,105,100,51,118,50,95,118,101,114,115,105,111,110,0,0,0,68,105,115,112,108,97,121,72,101,105,103,104,116,0,0,0,115,101,116,32,97,117,100,105,111,32,113,117,97,108,105,116,121,32,40,99,111,100,101,99,45,115,112,101,99,105,102,105,99,41,0,0,0,0,0,0,102,97,115,116,115,116,97,114,116,0,0,0,0,0,0,0,8,4,4,8,4,4,8,2,4,0,0,0,0,0,0,0,66,114,111,107,101,110,32,102,105,108,101,44,32,116,114,97,107,47,109,100,97,116,32,110,111,116,32,97,116,32,116,111,112,45,108,101,118,101,108,10,0,0,0,0,0,0,0,0,101,110,99,111,100,101,114,0,116,111,111,32,108,111,110,103,32,112,97,121,108,111,97,100,10,0,0,0,0,0,0,0,102,108,111,97,116,105,110,103,32,112,111,105,110,116,32,65,65,78,32,73,68,67,84,0,78,58,37,108,108,100,32,80,84,83,58,37,115,32,84,58,37,102,32,80,79,83,58,37,115,0,0,0,0,0,0,0,97,110,109,0,0,0,0,0,115,97,114,95,100,101,110,0,73,110,118,97,108,105,100,32,115,105,122,101,32,102,111,114,32,115,97,109,112,108,101,95,114,97,116,101,115,58,32,37,100,44,32,115,104,111,117,108,100,32,98,101,32,109,117,108,116,105,112,108,101,32,111,102,32,37,100,10,0,0,0,0,68,105,115,112,108,97,121,87,105,100,116,104,0,0,0,0,65,108,108,32,99,111,117,110,116,115,32,119,105,116,104,111,117,116,32,97,108,108,32,108,97,121,111,117,116,115,10,0,112,97,110,0,0,0,0,0,97,113,0,0,0,0,0,0,102,97,97,110,105,0,0,0,85,110,99,111,109,112,114,101,115,115,101,100,32,82,71,66,32,49,48,45,98,105,116,0,77,97,120,72,101,105,103,104,116,0,0,0,0,0,0,0,115,101,116,32,116,104,101,32,110,117,109,98,101,114,32,111,102,32,97,117,100,105,111,32,102,114,97,109,101,115,32,116,111,32,114,101,99,111,114,100,0,0,0,0,0,0,0,0,115,104,111,119,32,104,101,108,112,0,0,0,0,0,0,0,120,118,105,100,109,109,120,0,114,50,49,48,0,0,0,0,77,97,120,87,105,100,116,104,0,0,0,0,0,0,0,0,97,102,114,97,109,101,115,0,105,112,112,0,0,0,0,0,67,68,32,71,114,97,112,104,105,99,115,32,118,105,100,101,111,0,0,0,0,0,0,0,87,86,67,49,0,0,0,0,98,105,116,114,97,116,101,0,102,100,101,98,117,103,0,0,115,105,109,112,108,101,97,108,112,104,97,0,0,0,0,0,99,100,103,114,97,112,104,105,99,115,0,0,0,0,0,0,72,50,54,52,0,0,0,0,118,105,100,101,111,32,98,105,116,114,97,116,101,32,40,112,108,101,97,115,101,32,117,115,101,32,45,98,58,118,41,0,115,105,109,112,108,101,110,101,111,110,0,0,0,0,0,0,70,108,97,115,104,32,83,99,114,101,101,110,32,86,105,100,101,111,32,118,50,0,0,0,110,118,50,49,0,0,0,0,70,111,117,114,67,67,0,0,115,95,115,105,122,101,61,32,37,56,46,48,102,107,66,32,116,105,109,101,61,32,37,48,46,51,102,32,98,114,61,32,37,55,46,49,102,107,98,105,116,115,47,115,32,97,118,103,95,98,114,61,32,37,55,46,49,102,107,98,105,116,115,47,115,32,0,0,0,0,0,0,66,114,111,119,110,0,0,0,98,0,0,0,0,0,0,0,65,86,79,112,116,105,111,110,32,116,121,112,101,32,37,100,32,111,102,32,111,112,116,105,111,110,32,37,115,32,110,111,116,32,105,109,112,108,101,109,101,110,116,101,100,32,121,101,116,10,0,0,0,0,0,0,108,116,101,0,0,0,0,0,79,80,84,73,79,78,95,78,79,84,95,70,79,85,78,68,0,0,0,0,0,0,0,0,115,105,100,101,32,114,105,103,104,116,0,0,0,0,0,0,115,105,109,112,108,101,97,114,109,118,54,0,0,0,0,0,71,97,117,115,115,105,97,110,32,115,99,97,108,101,114,44,32,0,0,0,0,0,0,0,102,108,97,115,104,115,118,50,0,0,0,0,0,0,0,0,102,117,108,108,95,99,104,114,111,109,97,95,105,110,112,0,45,45,99,114,111,115,115,45,112,114,101,102,105,120,61,101,109,32,45,45,99,99,61,101,109,99,99,32,45,45,101,110,97,98,108,101,45,99,114,111,115,115,45,99,111,109,112,105,108,101,32,45,45,116,97,114,103,101,116,45,111,115,61,110,111,110,101,32,45,45,97,114,99,104,61,120,56,54,95,51,50,32,45,45,112,114,101,102,105,120,61,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,32,45,45,101,120,116,114,97,45,99,102,108,97,103,115,61,45,73,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,105,110,99,108,117,100,101,32,45,45,99,112,117,61,103,101,110,101,114,105,99,32,45,45,100,105,115,97,98,108,101,45,104,119,97,99,99,101,108,115,32,45,45,100,105,115,97,98,108,101,45,115,116,114,105,112,112,105,110,103,32,45,45,100,105,115,97,98,108,101,45,112,116,104,114,101,97,100,115,32,45,45,100,105,115,97,98,108,101,45,100,111,99,32,45,45,100,105,115,97,98,108,101,45,100,101,98,117,103,32,45,45,100,105,115,97,98,108,101,45,97,115,109,32,45,45,100,105,115,97,98,108,101,45,110,101,116,119,111,114,107,32,45,45,100,105,115,97,98,108,101,45,101,118,101,114,121,116,104,105,110,103,32,45,45,100,105,115,97,98,108,101,45,102,102,112,108,97,121,32,45,45,100,105,115,97,98,108,101,45,102,102,112,114,111,98,101,32,45,45,100,105,115,97,98,108,101,45,102,102,115,101,114,118,101,114,32,45,45,100,105,115,97,98,108,101,45,111,117,116,100,101,118,61,115,100,108,32,45,45,101,110,97,98,108,101,45,111,112,116,105,109,105,122,97,116,105,111,110,115,32,45,45,101,110,97,98,108,101,45,112,114,111,116,111,99,111,108,61,102,105,108,101,32,45,45,101,110,97,98,108,101,45,102,105,108,116,101,114,61,97,114,101,115,97,109,112,108,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,119,97,118,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,115,49,54,108,101,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,117,56,32,45,45,101,110,97,98,108,101,45,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,108,105,98,102,100,107,45,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,111,118,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,102,100,107,95,97,97,99,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,52,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,115,102,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,97,115,102,0,67,111,100,101,99,80,114,105,118,97,116,101,68,97,116,97,0,0,0,0,0,0,0,0,115,45,62,117,115,101,100,95,99,104,95,99,111,117,110,116,0,0,0,0,0,0,0,0,116,105,109,101,115,116,97,109,112,115,0,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,115,97,109,112,108,101,32,114,97,116,101,32,102,111,114,32,83,66,82,58,32,37,100,10,0,0,0,0,82,101,115,101,114,118,101,100,32,98,105,116,32,115,101,116,46,10,0,0,0,0,0,0,100,105,109,101,110,115,105,111,110,115,32,110,111,116,32,115,101,116,10,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,115,105,109,112,108,101,97,114,109,118,53,116,101,0,0,0,70,111,114,119,97,114,100,32,85,110,99,111,109,112,114,101,115,115,101,100,0,0,0,0,45,118,111,108,32,105,115,32,102,111,114,119,97,114,100,101,100,32,116,111,32,108,97,118,102,105,32,115,105,109,105,108,97,114,108,121,32,116,111,32,45,97,102,32,118,111,108,117,109,101,61,37,115,46,10,0,116,114,97,99,107,73,68,0,102,111,114,99,101,32,107,101,121,32,102,114,97,109,101,115,32,97,116,32,115,112,101,99,105,102,105,101,100,32,116,105,109,101,115,116,97,109,112,115,0,0,0,0,0,0,0,0,97,98,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,116,104,101,32,98,105,116,114,97,116,101,32,37,100,58,32,37,115,10,0,0,0,0,0,0,0,115,105,109,112,108,101,97,114,109,0,0,0,0,0,0,0,102,114,119,117,0,0,0,0,115,121,115,116,101,109,66,105,116,114,97,116,101,0,0,0,114,97,119,118,105,100,101,111,0,0,0,0,0,0,0,0,102,111,114,99,101,95,107,101,121,95,102,114,97,109,101,115,0,0,0,0,0,0,0,0,65,109,98,105,101,110,116,0,116,114,97,99,107,0,0,0,114,103,98,50,52,0,0,0,70,105,108,101,32,112,111,115,105,116,105,111,110,32,98,101,102,111,114,101,32,97,118,102,111,114,109,97,116,95,102,105,110,100,95,115,116,114,101,97,109,95,105,110,102,111,40,41,32,105,115,32,37,108,108,100,10,0,0,0,0,0,0,0,115,104,52,0,0,0,0,0,101,110,97,98,108,101,32,82,84,80,32,77,80,52,65,45,76,65,84,77,32,112,97,121,108,111,97,100,0,0,0,0,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,77,97,100,99,111,119,32,86,105,100,101,111,0,0,0,0,112,114,101,102,101,114,114,101,100,32,112,97,103,101,32,100,117,114,97,116,105,111,110,44,32,105,110,32,109,105,99,114,111,115,101,99,111,110,100,115,0,0,0,0,0,0,0,0,77,80,51,32,109,117,120,101,114,0,0,0,0,0,0,0,60,37,115,32,115,121,115,116,101,109,66,105,116,114,97,116,101,61,34,37,100,34,62,10,0,0,0,0,0,0,0,0,115,116,114,101,97,109,73,110,100,101,120,58,118,97,108,117,101,0,0,0,0,0,0,0,67,114,101,97,116,101,32,97,32,108,105,118,101,32,115,109,111,111,116,104,32,115,116,114,101,97,109,105,110,103,32,102,101,101,100,32,40,102,111,114,32,112,117,115,104,105,110,103,32,116,111,32,97,32,112,117,98,108,105,115,104,105,110,103,32,112,111,105,110,116,41,0,84,83,83,69,0,0,0,0,112,97,99,107,101,116,95,111,98,106,95,115,105,122,101,32,105,110,118,97,108,105,100,10,0,0,0,0,0,0,0,0,97,114,109,0,0,0,0,0,84,66,58,37,102,32,70,82,65,77,69,95,82,65,84,69,58,37,102,32,83,65,77,80,76,69,95,82,65,84,69,58,37,102,10,0,0,0,0,0,109,97,100,0,0,0,0,0,115,97,114,95,110,117,109,0,73,110,118,97,108,105,100,32,115,105,122,101,32,102,111,114,32,115,97,109,112,108,101,95,102,109,116,115,58,32,37,100,44,32,115,104,111,117,108,100,32,98,101,32,109,117,108,116,105,112,108,101,32,111,102,32,37,100,10,0,0,0,0,0])
.concat([97,117,100,105,111,0,0,0,65,108,108,32,108,97,121,111,117,116,115,32,115,101,116,32,111,110,32,110,111,110,45,101,109,112,116,121,32,108,105,115,116,10,0,0,0,0,0,0,111,99,118,0,0,0,0,0,115,101,116,32,116,104,101,32,118,97,108,117,101,32,111,102,32,97,110,32,111,117,116,102,105,108,101,32,115,116,114,101,97,109,105,100,0,0,0,0,114,97,110,100,111,109,0,0,115,105,109,112,108,101,109,109,120,0,0,0,0,0,0,0,68,80,88,32,105,109,97,103,101,0,0,0,0,0,0,0,118,105,100,101,111,0,0,0,115,116,114,101,97,109,105,100,0,0,0,0,0,0,0,0,104,0,0,0,0,0,0,0,115,105,109,112,108,101,0,0,85,110,97,98,108,101,32,116,111,32,112,97,114,115,101,32,111,112,116,105,111,110,32,118,97,108,117,101,32,34,37,115,34,32,97,115,32,118,105,100,101,111,32,114,97,116,101,10,0,0,0,0,0,0,0,0,100,112,120,0,0,0,0,0,60,115,119,105,116,99,104,62,10,0,0,0,0,0,0,0,102,111,114,99,101,32,116,104,101,32,115,101,108,101,99,116,101,100,32,102,114,97,109,101,114,97,116,101,44,32,100,105,115,97,98,108,101,32,116,104,101,32,98,101,115,116,32,115,117,112,112,111,114,116,101,100,32,102,114,97,109,101,114,97,116,101,32,115,101,108,101,99,116,105,111,110,0,0,0,0,108,105,98,97,118,117,116,105,108,47,109,97,116,104,101,109,97,116,105,99,115,46,99,0,32,32,32,32,76,97,115,116,32,109,101,115,115,97,103,101,32,114,101,112,101,97,116,101,100,32,37,100,32,116,105,109,101,115,10,0,0,0,0,0,115,101,108,101,99,116,32,73,68,67,84,32,105,109,112,108,101,109,101,110,116,97,116,105,111,110,0,0,0,0,0,0,85,110,99,111,109,112,114,101,115,115,101,100,32,52,58,50,58,50,32,49,48,45,98,105,116,0,0,0,0,0,0,0,97,98,115,40,115,114,99,95,108,105,110,101,115,105,122,101,41,32,62,61,32,98,121,116,101,119,105,100,116,104,0,0,60,98,111,100,121,62,10,0,102,111,114,99,101,95,102,112,115,0,0,0,0,0,0,0,100,101,98,117,103,0,0,0,105,100,99,116,0,0,0,0,118,50,49,48,0,0,0,0,60,47,104,101,97,100,62,10,0,0,0,0,0,0,0,0,85,110,100,101,102,105,110,101,100,32,99,111,110,115,116,97,110,116,32,111,114,32,109,105,115,115,105,110,103,32,39,40,39,32,105,110,32,39,37,115,39,10,0,0,0,0,0,0,115,104,111,119,32,81,80,32,104,105,115,116,111,103,114,97,109,0,0,0,0,0,0,0,66,105,116,115,116,114,101,97,109,32,102,105,108,116,101,114,32,110,111,116,32,102,111,117,110,100,0,0,0,0,0,0,99,111,109,112,114,101,115,115,101,115,32,100,97,114,107,32,97,114,101,97,115,32,115,116,114,111,110,103,101,114,32,116,104,97,110,32,109,101,100,105,117,109,32,111,110,101,115,0,112,114,101,118,95,102,111,114,99,101,100,95,110,0,0,0,56,48,56,56,102,108,101,120,32,84,77,86,0,0,0,0,110,118,49,50,0,0,0,0,60,109,101,116,97,32,110,97,109,101,61,34,99,114,101,97,116,111,114,34,32,99,111,110,116,101,110,116,61,34,37,115,34,32,47,62,10,0,0,0,102,95,115,105,122,101,61,32,37,54,100,32,0,0,0,0,66,108,117,101,86,105,111,108,101,116,0,0,0,0,0,0,113,112,104,105,115,116,0,0,37,115,32,65,86,79,112,116,105,111,110,115,58,10,0,0,103,116,0,0,0,0,0,0,77,117,120,101,114,32,110,111,116,32,102,111,117,110,100,0,83,82,0,0,0,0,0,0,100,97,114,107,95,109,97,115,107,0,0,0,0,0,0,0,32,40,0,0,0,0,0,0,108,117,109,97,32,66,73,67,85,66,73,67,32,47,32,99,104,114,111,109,97,32,66,73,76,73,78,69,65,82,32,115,99,97,108,101,114,44,32,0,116,109,118,0,0,0,0,0,102,117,108,108,32,99,104,114,111,109,97,32,105,110,116,101,114,112,111,108,97,116,105,111,110,0,0,0,0,0,0,0,60,104,101,97,100,62,10,0,82,101,109,97,116,114,105,120,32,105,115,32,110,101,101,100,101,100,32,98,101,116,119,101,101,110,32,37,115,32,97,110,100,32,37,115,32,98,117,116,32,116,104,101,114,101,32,105,115,32,110,111,116,32,101,110,111,117,103,104,32,105,110,102,111,114,109,97,116,105,111,110,32,116,111,32,100,111,32,105,116,10,0,0,0,0,0,0,102,111,114,99,101,32,118,105,100,101,111,32,116,97,103,47,102,111,117,114,99,99,0,0,84,111,111,32,109,97,110,121,32,112,97,116,99,104,101,115,58,32,37,100,10,0,0,0,109,115,95,112,114,101,115,101,110,116,32,61,32,51,32,105,115,32,114,101,115,101,114,118,101,100,46,10,0,0,0,0,108,105,98,115,119,115,99,97,108,101,47,115,119,115,99,97,108,101,95,105,110,116,101,114,110,97,108,46,104,0,0,0,67,104,97,110,110,101,108,32,108,97,121,111,117,116,32,39,37,115,39,32,119,105,116,104,32,37,100,32,99,104,97,110,110,101,108,115,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,110,117,109,98,101,114,32,111,102,32,115,112,101,99,105,102,105,101,100,32,99,104,97,110,110,101,108,115,32,37,100,10,0,0,0,0,108,105,98,115,119,115,99,97,108,101,47,115,119,115,99,97,108,101,95,105,110,116,101,114,110,97,108,46,104,0,0,0,105,110,116,101,114,32,109,97,115,107,105,110,103,0,0,0,118,50,49,48,120,0,0,0,108,105,98,115,119,115,99,97,108,101,47,111,117,116,112,117,116,46,99,0,0,0,0,0,37,102,0,0,0,0,0,0,60,115,109,105,108,32,120,109,108,110,115,61,34,104,116,116,112,58,47,47,119,119,119,46,119,51,46,111,114,103,47,50,48,48,49,47,83,77,73,76,50,48,47,76,97,110,103,117,97,103,101,34,62,10,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,118,116,97,103,0,0,0,0,115,101,116,32,98,105,116,114,97,116,101,32,40,105,110,32,98,105,116,115,47,115,41,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,116,104,101,32,86,66,82,32,98,105,116,114,97,116,101,32,109,111,100,101,32,37,100,58,32,37,115,10,0,0,0,0,0,0,112,95,109,97,115,107,0,0,65,117,114,97,118,105,115,105,111,110,32,65,117,114,97,32,50,0,0,0,0,0,0,0,60,63,120,109,108,32,118,101,114,115,105,111,110,61,34,49,46,48,34,32,101,110,99,111,100,105,110,103,61,34,117,116,102,45,56,34,63,62,10,0,77,80,69,71,45,52,32,112,97,114,116,32,50,0,0,0,112,114,101,99,105,115,105,111,110,0,0,0,0,0,0,0,69,117,114,111,45,84,101,99,104,110,111,0,0,0,0,0,87,77,47,84,114,97,99,107,78,117,109,98,101,114,0,0,115,99,97,108,101,114,32,102,108,97,103,115,0,0,0,0,108,105,98,115,119,114,101,115,97,109,112,108,101,47,114,101,115,97,109,112,108,101,46,99,0,0,0,0,0,0,0,0,99,111,100,105,110,103,95,104,105,115,116,111,114,121,0,0,108,105,98,115,119,114,101,115,97,109,112,108,101,47,114,101,109,97,116,114,105,120,46,99,0,0,0,0,0,0,0,0,108,105,98,115,119,114,101,115,97,109,112,108,101,47,100,105,116,104,101,114,46,99,0,0,115,112,97,116,105,97,108,32,99,111,109,112,108,101,120,105,116,121,32,109,97,115,107,105,110,103,0,0,0,0,0,0,108,97,116,109,0,0,0,0,108,105,98,115,119,114,101,115,97,109,112,108,101,47,97,117,100,105,111,99,111,110,118,101,114,116,46,99,0,0,0,0,97,117,114,97,50,0,0,0,112,97,103,101,95,100,117,114,97,116,105,111,110,0,0,0,73,68,51,0,0,0,0,0,99,114,101,97,116,105,111,110,95,116,105,109,101,0,0,0,105,110,116,114,97,95,100,99,95,112,114,101,99,105,115,105,111,110,0,0,0,0,0,0,105,115,109,108,0,0,0,0,105,103,110,111,114,101,95,101,100,105,116,108,105,115,116,0,116,114,97,99,107,0,0,0,112,97,99,107,101,116,95,114,101,112,108,105,99,95,115,105,122,101,32,37,100,32,105,115,32,105,110,118,97,108,105,100,10,0,0,0,0,0,0,0,115,99,112,108,120,95,109,97,115,107,0,0,0,0,0,0,45,45,99,114,111,115,115,45,112,114,101,102,105,120,61,101,109,32,45,45,99,99,61,101,109,99,99,32,45,45,101,110,97,98,108,101,45,99,114,111,115,115,45,99,111,109,112,105,108,101,32,45,45,116,97,114,103,101,116,45,111,115,61,110,111,110,101,32,45,45,97,114,99,104,61,120,56,54,95,51,50,32,45,45,112,114,101,102,105,120,61,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,32,45,45,101,120,116,114,97,45,99,102,108,97,103,115,61,45,73,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,105,110,99,108,117,100,101,32,45,45,99,112,117,61,103,101,110,101,114,105,99,32,45,45,100,105,115,97,98,108,101,45,104,119,97,99,99,101,108,115,32,45,45,100,105,115,97,98,108,101,45,115,116,114,105,112,112,105,110,103,32,45,45,100,105,115,97,98,108,101,45,112,116,104,114,101,97,100,115,32,45,45,100,105,115,97,98,108,101,45,100,111,99,32,45,45,100,105,115,97,98,108,101,45,100,101,98,117,103,32,45,45,100,105,115,97,98,108,101,45,97,115,109,32,45,45,100,105,115,97,98,108,101,45,110,101,116,119,111,114,107,32,45,45,100,105,115,97,98,108,101,45,101,118,101,114,121,116,104,105,110,103,32,45,45,100,105,115,97,98,108,101,45,102,102,112,108,97,121,32,45,45,100,105,115,97,98,108,101,45,102,102,112,114,111,98,101,32,45,45,100,105,115,97,98,108,101,45,102,102,115,101,114,118,101,114,32,45,45,100,105,115,97,98,108,101,45,111,117,116,100,101,118,61,115,100,108,32,45,45,101,110,97,98,108,101,45,111,112,116,105,109,105,122,97,116,105,111,110,115,32,45,45,101,110,97,98,108,101,45,112,114,111,116,111,99,111,108,61,102,105,108,101,32,45,45,101,110,97,98,108,101,45,102,105,108,116,101,114,61,97,114,101,115,97,109,112,108,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,119,97,118,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,115,49,54,108,101,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,117,56,32,45,45,101,110,97,98,108,101,45,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,108,105,98,102,100,107,45,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,111,118,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,102,100,107,95,97,97,99,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,52,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,115,102,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,97,115,102,0,100,101,102,97,117,108,116,0,65,117,114,97,118,105,115,105,111,110,32,65,85,82,65,0,116,105,109,101,95,98,97,115,101,95,100,101,110,0,0,0,70,97,105,108,101,100,32,116,111,32,97,108,108,111,99,97,116,101,32,102,105,102,111,10,0,0,0,0,0,0,0,0,116,114,97,99,107,32,37,100,58,32,109,117,120,105,110,103,32,109,112,51,32,97,116,32,37,100,104,122,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,81,117,101,114,121,32,102,111,114,109,97,116,32,102,97,105,108,101,100,32,102,111,114,32,39,37,115,39,58,32,37,115,10,0,0,0,0,0,0,0,102,114,101,105,48,114,95,115,114,99,0,0,0,0,0,0,100,99,0,0,0,0,0,0,114,116,112,0,0,0,0,0,111,99,108,0,0,0,0,0,116,101,109,112,111,114,97,108,32,99,111,109,112,108,101,120,105,116,121,32,109,97,115,107,105,110,103,0,0,0,0,0,112,115,32,101,120,116,101,110,115,105,111,110,32,111,118,101,114,102,108,111,119,32,37,100,10,0,0,0,0,0,0,0,97,117,114,97,0,0,0,0,111,118,101,114,102,108,111,119,32,105,110,32,115,112,101,99,116,114,97,108,32,82,76,69,44,32,105,103,110,111,114,105,110,103,10,0,0,0,0,0,116,114,97,99,107,32,37,100,58,32,99,111,100,101,99,32,98,108,111,99,107,32,97,108,105,103,110,32,105,115,32,110,111,116,32,115,101,116,32,102,111,114,32,97,100,112,99,109,10,0,0,0,0,0,0,0,116,111,112,61,49,47,98,111,116,116,111,109,61,48,47,97,117,116,111,61,45,49,32,102,105,101,108,100,32,102,105,114,115,116,0,0,0,0,0,0,115,104,111,119,32,108,105,99,101,110,115,101,0,0,0,0,108,105,98,97,118,99,111,100,101,99,47,107,98,100,119,105,110,46,99,0,0,0,0,0,108,105,98,97,118,99,111,100,101,99,47,98,105,116,115,116,114,101,97,109,46,99,0,0,84,114,121,105,110,103,32,116,111,32,114,101,109,111,118,101,32,37,100,32,115,97,109,112,108,101,115,44,32,98,117,116,32,116,104,101,32,113,117,101,117,101,32,105,115,32,101,109,112,116,121,10,0,0,0,0,116,99,112,108,120,95,109,97,115,107,0,0,0,0,0,0,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,84,81,73,32,118,105,100,101,111,0,0,0,0,0,0,0,116,114,97,99,107,32,37,100,58,32,99,111,100,101,99,32,102,114,97,109,101,32,115,105,122,101,32,105,115,32,110,111,116,32,115,101,116,10,0,0,116,111,112,0,0,0,0,0,99,111,109,112,114,101,115,115,101,115,32,98,114,105,103,104,116,32,97,114,101,97,115,32,115,116,114,111,110,103,101,114,32,116,104,97,110,32,109,101,100,105,117,109,32,111,110,101,115,0,0,0,0,0,0,0,116,113,105,0,0,0,0,0,87,65,82,78,73,78,71,32,99,111,100,101,99,32,116,105,109,101,98,97,115,101,32,105,115,32,118,101,114,121,32,104,105,103,104,46,32,73,102,32,100,117,114,97,116,105,111,110,32,105,115,32,116,111,111,32,108,111,110,103,44,10,102,105,108,101,32,109,97,121,32,110,111,116,32,98,101,32,112,108,97,121,97,98,108,101,32,98,121,32,113,117,105,99,107,116,105,109,101,46,32,83,112,101,99,105,102,121,32,97,32,115,104,111,114,116,101,114,32,116,105,109,101,98,97,115,101,10,111,114,32,99,104,111,111,115,101,32,100,105,102,102,101,114,101,110,116,32,99,111,110,116,97,105,110,101,114,46,10,0,115,112,101,99,105,102,121,32,105,110,116,101,114,32,109,97,116,114,105,120,32,99,111,101,102,102,115,0,0,0,0,0,67,111,109,109,97,110,100,32,108,105,110,101,58,10,0,0,73,110,118,97,108,105,100,32,105,110,112,117,116,32,100,97,116,97,32,111,114,32,114,101,113,117,101,115,116,101,100,32,98,105,116,114,97,116,101,32,116,111,111,32,108,111,119,44,32,99,97,110,110,111,116,32,101,110,99,111,100,101,10,0,73,110,112,117,116,32,112,97,99,107,101,116,32,115,105,122,101,32,116,111,111,32,115,109,97,108,108,32,40,37,100,32,60,32,37,100,41,10,0,0,73,110,118,97,108,105,100,32,109,111,100,101,32,105,110,32,112,97,99,107,101,116,10,0,108,117,109,105,95,109,97,115,107,0,0,0,0,0,0,0,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,84,71,81,32,118,105,100,101,111,0,0,0,0,0,0,0,68,45,49,48,47,73,77,88,32,109,117,115,116,32,117,115,101,32,55,50,48,120,54,48,56,32,111,114,32,55,50,48,120,53,49,50,32,118,105,100,101,111,32,114,101,115,111,108,117,116,105,111,110,10,0,0,105,110,116,101,114,95,109,97,116,114,105,120,0,0,0,0,102,108,111,97,116,105,110,103,32,112,111,105,110,116,32,65,65,78,32,68,67,84,0,0,73,65,82,84,0,0,0,0,116,103,113,0,0,0,0,0,114,103,98,52,95,98,121,116,101,0,0,0,0,0,0,0,116,114,97,99,107,32,37,100,58,32,99,111,117,108,100,32,110,111,116,32,102,105,110,100,32,116,97,103,44,32,99,111,100,101,99,32,110,111,116,32,99,117,114,114,101,110,116,108,121,32,115,117,112,112,111,114,116,101,100,32,105,110,32,99,111,110,116,97,105,110,101,114,10,0,0,0,0,0,0,0,80,83,78,82,61,32,37,54,46,50,102,32,0,0,0,0,66,108,117,101,0,0,0,0,109,97,116,114,105,120,0,0,103,116,101,0,0,0,0,0,77,85,88,69,82,95,78,79,84,95,70,79,85,78,68,0,108,105,98,97,118,99,111,100,101,99,47,117,116,105,108,115,46,99,0,0,0,0,0,0,115,105,100,101,32,108,101,102,116,0,0,0,0,0,0,0,102,97,97,110,0,0,0,0,65,114,101,97,32,65,118,101,114,97,103,105,110,103,32,115,99,97,108,101,114,44,32,0,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,84,71,86,32,118,105,100,101,111,0,0,0,0,0,0,0,102,117,108,108,95,99,104,114,111,109,97,95,105,110,116,0,112,99,109,95,117,56,0,0,108,97,110,103,117,97,103,101,0,0,0,0,0,0,0,0,73,110,112,117,116,32,99,104,97,110,110,101,108,32,99,111,117,110,116,32,97,110,100,32,108,97,121,111,117,116,32,97,114,101,32,117,110,115,101,116,10,0,0,0,0,0,0,0,115,112,101,99,105,102,121,32,105,110,116,114,97,32,109,97,116,114,105,120,32,99,111,101,102,102,115,0,0,0,0,0,84,111,111,32,109,97,110,121,32,110,111,105,115,101,32,102,108,111,111,114,32,115,99,97,108,101,32,102,97,99,116,111,114,115,58,32,37,100,10,0,115,107,105,112,95,100,97,116,97,95,115,116,114,101,97,109,95,101,108,101,109,101,110,116,58,32,73,110,112,117,116,32,98,117,102,102,101,114,32,101,120,104,97,117,115,116,101,100,32,98,101,102,111,114,101,32,69,78,68,32,101,108,101,109,101,110,116,32,102,111,117,110,100,10,0,0,0,0,0,0,83,112,101,99,105,102,105,101,100,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,39,37,115,39,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,97,108,116,105,118,101,99,0,116,103,118,0,0,0,0,0,89,111,117,32,114,101,113,117,101,115,116,101,100,32,97,32,99,111,112,121,32,111,102,32,116,104,101,32,111,114,105,103,105,110,97,108,32,116,105,109,101,99,111,100,101,32,116,114,97,99,107,32,115,111,32,116,105,109,101,99,111,100,101,32,109,101,116,97,100,97,116,97,32,97,114,101,32,110,111,119,32,105,103,110,111,114,101,100,10,0,0,0,0,0,0,0,45,118,111,108,32,104,97,115,32,98,101,101,110,32,100,101,112,114,101,99,97,116,101,100,46,32,85,115,101,32,116,104,101,32,118,111,108,117,109,101,32,97,117,100,105,111,32,102,105,108,116,101,114,32,105,110,115,116,101,97,100,46,10,0,105,110,116,114,97,95,109,97,116,114,105,120,0,0,0,0,98,0,0,0,0,0,0,0,78,111,116,101,44,32,116,104,101,32,86,66,82,32,115,101,116,116,105,110,103,32,105,115,32,117,110,115,117,112,112,111,114,116,101,100,32,97,110,100,32,111,110,108,121,32,119,111,114,107,115,32,119,105,116,104,32,115,111,109,101,32,112,97,114,97,109,101,116,101,114,32,99,111,109,98,105,110,97,116,105,111,110,115,10,0,0,0,109,109,120,0,0,0,0,0,77,111,116,105,111,110,32,80,105,120,101,108,115,32,118,105,100,101,111,0,0,0,0,0,80,83,80,32,109,111,100,101,32,110,101,101,100,32,111,110,101,32,118,105,100,101,111,32,97,110,100,32,111,110,101,32,97,117,100,105,111,32,115,116,114,101,97,109,10,0,0,0,109,112,101,103,52,0,0,0,115,101,116,32,118,105,100,101,111,32,102,105,108,116,101,114,115,0,0,0,0,0,0,0,83,111,117,110,100,116,114,97,99,107,0,0,0,0,0,0,87,77,47,84,111,111,108,0,117,109,105,100,0,0,0,0,114,101,97,100,95,116,105,109,101,115,116,97,109,112,40,41,32,102,97,105,108,101,100,32,105,110,32,116,104,101,32,109,105,100,100,108,101,10,0,0,97,99,99,117,114,97,116,101,32,105,110,116,101,103,101,114,0,0,0,0,0,0,0,0,100,111,110,39,116,32,109,101,114,103,101,32,115,105,100,101,32,100,97,116,97,0,0,0,109,111,116,105,111,110,112,105,120,101,108,115,0,0,0,0,112,114,101,102,101,114,114,101,100,32,112,97,103,101,32,115,105,122,101,32,105,110,32,98,121,116,101,115,32,40,100,101,112,114,101,99,97,116,101,100,41,0,0,0,0,0,0,0,78,111,32,97,117,100,105,111,32,115,116,114,101,97,109,32,112,114,101,115,101,110,116,46,10,0,0,0,0,0,0,0,102,52,118,0,0,0,0,0,118,102,0,0,0,0,0,0,70,108,117,115,104,32,102,114,97,103,109,101,110,116,115,32,111,110,32,99,97,108,108,101,114,32,114,101,113,117,101,115,116,115,0,0,0,0,0,0,97,108,108,111,119,32,117,115,105,110,103,32,97,98,115,111,108,117,116,101,32,112,97,116,104,32,119,104,101,110,32,111,112,101,110,105,110,103,32,97,108,105,97,115,44,32,116,104,105,115,32,105,115,32,97,32,112,111,115,115,105,98,108,101,32,115,101,99,117,114,105,116,121,32,105,115,115,117,101,0,84,82,67,75,0,0,0,0,105,100,120,32,43,32,49,32,60,61,32,97,115,102,95,115,116,45,62,112,107,116,46,115,105,122,101,32,47,32,97,115,102,95,115,116,45,62,100,115,95,99,104,117,110,107,95,115,105,122,101,0,0,0,0,0,78,117,109,98,101,114,32,111,102,32,116,104,101,32,102,105,114,115,116,32,102,114,97,109,101,32,116,104,97,116,32,115,104,111,117,108,100,32,98,101,32,100,114,111,112,112,101,100,32,97,103,97,105,110,0,0,105,110,116,0,0,0,0,0,69,120,112,114,101,115,115,105,111,110,32,100,101,116,101,114,109,105,110,105,110,103,32,116,104,101,32,102,114,97,109,101,32,116,105,109,101,115,116,97,109,112,0,0,0,0,0,0,69,108,101,99,116,114,111,110,105,99,32,65,114,116,115,32,67,77,86,32,118,105,100,101,111,0,0,0,0,0,0,0,100,101,112,114,101,99,97,116,101,100,44,32,100,111,32,110,111,116,32,117,115,101,0,0,115,101,116,32,116,104,101,32,115,117,112,112,111,114,116,101,100,32,112,105,120,101,108,32,102,111,114,109,97,116,115,0,105,112,111,100,0,0,0,0,84,104,101,32,102,111,108,108,111,119,105,110,103,32,102,105,108,116,101,114,115,32,99,111,117,108,100,32,110,111,116,32,99,104,111,111,115,101,32,116,104,101,105,114,32,102,111,114,109,97,116,115,58,32,37,115,10,67,111,110,115,105,100,101,114,32,105,110,115,101,114,116,105,110,103,32,116,104,101,32,40,97,41,102,111,114,109,97,116,32,102,105,108,116,101,114,32,110,101,97,114,32,116,104,101,105,114,32,105,110,112,117,116,32,111,114,32,111,117,116,112,117,116,46,10,0,0,0,102,114,101,105,48,114,0,0,102,105,108,101,0,0,0,0,102,97,115,116,32,105,110,116,101,103,101,114,0,0,0,0,99,109,118,0,0,0,0,0,112,115,112,0,0,0,0,0,118,115,116,97,116,115,95,102,105,108,101,0,0,0,0,0,76,0,0,0,0,0,0,0,73,110,99,111,109,112,97,116,105,98,108,101,32,115,97,109,112,108,101,32,102,111,114,109,97,116,32,39,37,115,39,32,102,111,114,32,99,111,100,101,99,32,39,37,115,39,44,32,97,117,116,111,45,115,101,108,101,99,116,105,110,103,32,102,111,114,109,97,116,32,39,37,115,39,10,0,0,0,0,0,108,105,98,97,118,99,111,100,101,99,47,111,112,116,105,111,110,115,46,99,0,0,0,0,102,97,115,116,105,110,116,0,66,114,117,116,101,32,70,111,114,99,101,32,38,32,73,103,110,111,114,97,110,99,101,0,51,103,50,0,0,0,0,0,100,117,109,112,32,118,105,100,101,111,32,99,111,100,105,110,103,32,115,116,97,116,105,115,116,105,99,115,32,116,111,32,102,105,108,101,0,0,0,0,112,97,99,107,101,116,32,98,117,102,102,101,114,32,105,115,32,116,111,111,32,115,109,97,108,108,10,0,0,0,0,0,118,111,114,98,105,115,32,104,101,97,100,101,114,32,115,105,122,101,115,32,100,97,109,97,103,101,100,10,0,0,0,0,97,117,116,111,115,101,108,101,99,116,32,97,32,103,111,111,100,32,111,110,101,32,40,100,101,102,97,117,108,116,41,0,98,102,105,0,0,0,0,0,51,103,112,0,0,0,0,0,118,115,116,97,116,115,0,0,70,70,82,69,80,79,82,84,0,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,101,110,99,111,100,101,32,102,114,97,109,101,58,32,37,115,10,0,0,0,0,0,108,105,98,97,118,99,111,100,101,99,47,105,109,103,99,111,110,118,101,114,116,46,99,0,97,117,116,111,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,112,105,99,116,117,114,101,32,99,111,100,105,110,103,32,109,111,100,101,32,37,100,10,0,0,0,0,0,68,105,114,97,99,0,0,0,109,117,120,101,114,32,100,111,101,115,32,110,111,116,32,115,117,112,112,111,114,116,32,110,111,110,32,115,101,101,107,97,98,108,101,32,111,117,116,112,117,116,10,0,0,0,0,0,65,86,70,111,114,109,97,116,67,111,110,116,101,120,116,0,99,97,108,99,117,108,97,116,101,32,80,83,78,82,32,111,102,32,99,111,109,112,114,101,115,115,101,100,32,102,114,97,109,101,115,0,0,0,0,0,68,67,84,32,97,108,103,111,114,105,116,104,109,0,0,0,100,105,114,97,99,0,0,0,114,103,98,52,0,0,0,0,105,115,109,118,0,0,0,0,102,114,97,109,101,61,32,37,53,100,32,113,61,32,37,50,46,49,102,32,0,0,0,0,66,108,97,110,99,104,101,100,65,108,109,111,110,100,0,0,112,115,110,114,0,0,0,0,48,120,37,48,50,120,37,48,50,120,37,48,50,120,37,48,50,120,0,0,0,0,0,0,101,113,0,0,0,0,0,0,73,110,118,97,108,105,100,32,100,97,116,97,32,102,111,117,110,100,32,119,104,101,110,32,112,114,111,99,101,115,115,105,110,103,32,105,110,112,117,116,0,0,0,0,0,0,0,0,111,117,116,45,111,102,45,109,101,109,111,114,121,32,101,114,114,111,114,46,32,115,107,105,112,112,105,110,103,32,86,111,114,98,105,115,67,111,109,109,101,110,116,32,116,97,103,46,10,0,0,0,0,0,0,0,83,76,0,0,0,0,0,0,100,99,116,0,0,0,0,0,78,101,97,114,101,115,116,32,78,101,105,103,104,98,111,114,32,47,32,80,79,73,78,84,32,115,99,97,108,101,114,44,32,0,0,0,0,0,0,0,69,115,99,97,112,101,32,49,50,52,0,0,0,0,0,0,97,99,99,117,114,97,116,101,32,114,111,117,110,100,105,110,103,0,0,0,0,0,0,0,116,105,109,101,99,111,100,101,0,0,0,0,0,0,0,0,108,105,98,115,119,114,101,115,97,109,112,108,101,47,115,119,114,101,115,97,109,112,108,101,46,99,0,0,0,0,0,0,128,116,104,101,111,114,97,0,116,104,105,115,32,111,112,116,105,111,110,32,105,115,32,100,101,112,114,101,99,97,116,101,100,44,32,117,115,101,32,116,104,101,32,121,97,100,105,102,32,102,105,108,116,101,114,32,105,110,115,116,101,97,100,0,83,116,97,114,116,32,102,114,101,113,117,101,110,99,121,32,98,111,114,100,101,114,32,116,111,111,32,104,105,103,104,58,32,37,100,10,0,0,0,0,67,104,97,110,110,101,108,32,108,97,121,111,117,116,32,110,111,116,32,115,112,101,99,105,102,105,101,100,10,0,0,0,83,112,101,101,120,32,32,32,0,0,0,0,0,0,0,0,105,110,105,116,105,97,108,32,99,111,109,112,108,101,120,105,116,121,32,102,111,114,32,49,45,112,97,115,115,32,101,110,99,111,100,105,110,103,0,0,101,115,99,97,112,101,49,50,52,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,114,101,45,111,112,101,110,32,37,115,32,111,117,116,112,117,116,32,102,105,108,101,32,102,111,114,32,116,104,101,32,115,101,99,111,110,100,32,112,97,115,115,32,40,102,97,115,116,115,116,97,114,116,41,10,0,0,0,0,0,0,0,97,114,101,115,97,109,112,108,101,0,0,0,0,0,0,0,100,101,105,110,116,101,114,108,97,99,101,0,0,0,0,0,65,86,67,111,100,101,99,67,111,110,116,101,120,116,0,0,86,66,82,32,113,117,97,108,105,116,121,32,37,100,32,111,117,116,32,111,102,32,114,97,110,103,101,44,32,115,104,111,117,108,100,32,98,101,32,49,45,53,10,0,0,0,0,0,102,105,115,104,101,97,100,0,114,99,95,105,110,105,116,95,99,112,108,120,0,0,0,0,82,76,50,32,118,105,100,101,111,0,0,0,0,0,0,0,74,80,69,71,45,76,83,0,79,112,117,115,72,101,97,100,0,0,0,0,0,0,0,0,112,114,101,102,105,120,0,0,80,114,97,110,107,115,0,0,112,117,98,108,105,115,104,101,114,0,0,0,0,0,0,0,48,120,37,48,49,54,108,108,88,37,48,49,54,108,108,88,37,48,49,54,108,108,88,37,48,49,54,108,108,88,37,48,49,54,108,108,88,37,48,49,54,108,108,88,37,48,49,54,108,108,88,37,48,49,54,108,108,88,0,0,0,0,0,0,112,111,115,95,108,105,109,105,116,32,60,61,32,112,111,115,95,109,97,120,0,0,0,0,101,110,99,111,100,101,100,95,98,121,0,0,0,0,0,0,81,80,32,111,102,102,115,101,116,32,98,101,116,119,101,101,110,32,80,45,32,97,110,100,32,73,45,102,114,97,109,101,115,0,0,0,0,0,0,0,107,101,101,112,115,105,100,101,0,0,0,0,0,0,0,0,114,108,50,0,0,0,0,0,112,97,103,101,115,105,122,101,0,0,0,0,0,0,0,0,79,110,108,121,32,97,117,100,105,111,32,115,116,114,101,97,109,115,32,97,110,100,32,112,105,99,116,117,114,101,115,32,97,114,101,32,97,108,108,111,119,101,100,32,105,110,32,77,80,51,46,10,0,0,0,0,115,101,108,101,99,116,32,116,119,111,32,112,97,115,115,32,108,111,103,32,102,105,108,101,32,110,97,109,101,32,112,114,101,102,105,120,0,0,0,0,102,114,97,103,95,99,117,115,116,111,109,0,0,0,0,0,1,118,105,100,101,111,0,0,117,115,101,95,97,98,115,111,108,117,116,101,95,112,97,116,104,0,0,0,0,0,0,0,112,117,98,108,105,115,104,101,114,0,0,0,0,0,0,0,109,112,101,103,50,118,105,100,101,111,0,0,0,0,0,0,111,102,102,115,101,116,32,43,32,97,115,102,95,115,116,45,62,100,115,95,99,104,117,110,107,95,115,105,122,101,32,60,61,32,97,115,102,95,115,116,45,62,112,107,116,46,115,105,122,101,0,0,0,0,0,0,101,110,100,95,102,114,97,109,101,0,0,0,0,0,0,0,105,95,113,111,102,102,115,101,116,0,0,0,0,0,0,0,101,120,112,114,0,0,0,0,108,105,98,97,118,99,111,100,101,99,47,97,118,112,97,99,107,101,116,46,99,0,0,0,115,119,115,95,102,108,97,103,115,32,110,111,116,32,116,101,114,109,105,110,97,116,101,100,32,119,105,116,104,32,39,59,39,46,10,0,0,0,0,0,77,105,109,105,99,0,0,0,127,70,76,65,67,0,0,0,114,102,99,50,49,57,48,0,116,105,109,101,95,98,97,115,101,95,110,117,109,0,0,0,112,105,120,95,102,109,116,115,0,0,0,0,0,0,0,0,114,111,116,97,116,101,0,0,110,111,102,111,114,109,97,116,0,0,0,0,0,0,0,0,112,97,115,115,108,111,103,102,105,108,101,0,0,0,0,0,99,111,109,109,101,110,116,0,66,66,67,68,0,0,0,0,81,80,32,102,97,99,116,111,114,32,98,101,116,119,101,101,110,32,80,45,32,97,110,100,32,73,45,102,114,97,109,101,115,0,0,0,0,0,0,0,67,69,76,84,32,32,32,32,0,0,0,0,0,0,0,0,109,105,109,105,99,0,0,0,115,101,108,101,99,116,32,116,104,101,32,112,97,115,115,32,110,117,109,98,101,114,32,40,49,32,116,111,32,51,41,0,37,115,10,0,0,0,0,0,105,95,113,102,97,99,116,111,114,0,0,0,0,0,0,0,73,110,116,101,108,32,73,110,100,101,111,32,86,105,100,101,111,32,73,110,116,101,114,97,99,116,105,118,101,32,53,0,67,111,117,110,116,114,121,0,108,105,98,97,118,102,111,114,109,97,116,47,109,111,118,101,110,99,46,99,0,0,0,0,85,110,107,110,111,119,110,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,32,109,105,109,101,116,121,112,101,58,32,37,115,46,10,0,112,97,115,115,0,0,0,0,87,77,47,65,108,98,117,109,84,105,116,108,101,0,0,0,99,117,114,114,101,110,116,108,121,32,117,115,101,108,101,115,115,0,0,0,0,0,0,0,105,110,100,101,111,53,0,0,97,118,95,114,101,115,99,97,108,101,95,114,110,100,40,116,114,97,99,107,45,62,99,108,117,115,116,101,114,91,48,93,46,100,116,115,44,32,49,48,48,48,44,32,116,114,97,99,107,45,62,116,105,109,101,115,99,97,108,101,44,32,65,86,95,82,79,85,78,68,95,68,79,87,78,41,32,60,61,32,48,0,0,0,0,0,0,0,104,104,58,109,109,58,115,115,91,58,59,46,93,102,102,0,84,97,103,32,115,105,122,101,32,105,115,32,119,97,121,32,116,111,111,32,98,105,103,10,0,0,0,0,0,0,0,0,114,101,112,111,114,116,0,0,111,103,103,0,0,0,0,0,110,101,103,97,116,105,118,101,32,100,97,116,97,95,115,105,122,101,32,97,110,100,47,111,114,32,115,97,109,112,108,101,95,99,111,117,110,116,32,105,110,32,100,115,54,52,58,32,100,97,116,97,95,115,105,122,101,32,61,32,37,108,108,100,44,32,115,97,109,112,108,101,95,99,111,117,110,116,32,61,32,37,108,108,100,10,0,0,84,82,65,67,75,78,85,77,66,69,82,0,0,0,0,0,114,99,95,98,117,102,95,97,103,103,114,101,115,115,105,118,105,116,121,0,0,0,0,0,73,110,116,101,108,32,73,110,100,101,111,32,86,105,100,101,111,32,73,110,116,101,114,97,99,116,105,118,101,32,52,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,115,101,116,32,105,110,105,116,105,97,108,32,84,105,109,101,67,111,100,101,32,118,97,108,117,101,46,0,0,0,0,0,115,101,116,32,114,97,116,101,99,111,110,116,114,111,108,32,98,117,102,102,101,114,32,115,105,122,101,32,40,105,110,32,98,105,116,115,41,0,0,0,105,110,100,101,111,52,0,0,114,103,98,56,0,0,0,0,66,108,97,99,107,0,0,0,102,111,112,101,110,0,0,0,116,105,109,101,99,111,100,101,0,0,0,0,0,0,0,0,37,108,108,105,100,58,37,48,50,100,58,37,48,50,100,46,37,48,54,100,0,0,0,0,109,105,110,0,0,0,0,0,111,103,103,0,0,0,0,0,73,78,86,65,76,73,68,68,65,84,65,0,0,0,0,0,98,97,99,107,32,99,101,110,116,101,114,0,0,0,0,0,98,117,102,115,105,122,101,0,69,120,112,101,114,105,109,101,110,116,97,108,32,115,99,97,108,101,114,44,32,0,0,0,83,117,110,32,82,97,115,116,101,114,102,105,108,101,32,105,109,97,103,101,0,0,0,0,97,99,99,117,114,97,116,101,95,114,110,100,0,0,0,0,84,114,117,110,99,97,116,105,110,103,32,112,97,99,107,101,116,32,111,102,32,115,105,122,101,32,37,100,32,116,111,32,37,108,108,100,10,0,0,0,33,115,45,62,105,110,95,99,104,95,108,97,121,111,117,116,0,0,0,0,0,0,0,0,115,97,109,101,95,113,117,97,110,116,0,0,0,0,0,0,83,116,111,112,32,102,114,101,113,117,101,110,99,121,32,98,111,114,100,101,114,32,116,111,111,32,104,105,103,104,58,32,37,100,10,0,0,0,0,0,73,67,77,84,0,0,0,0,83,112,101,99,105,102,105,101,100,32,115,97,109,112,108,101,32,114,97,116,101,32,37,100,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,0,0,0,0,83,101,116,32,109,105,110,105,109,117,109,32,98,105,116,114,97,116,101,32,116,111,108,101,114,97,110,99,101,32,40,105,110,32,98,105,116,115,47,115,41,46,32,77,111,115,116,32,117,115,101,102,117,108,32,105,110,32,115,101,116,116,105,110,103,32,117,112,32,97,32,67,66,82,32,101,110,99,111,100,101,46,32,73,116,32,105,115,32,111,102,32,108,105,116,116,108,101,32,117,115,101,32,111,116,104,101,114,119,105,115,101,46,0,0,0,0,0,0,0,115,117,110,114,97,115,116,0,103,114,97,112,104,32,37,100,32,37,115,32,102,111,114,32,105,110,112,117,116,32,115,116,114,101,97,109,32,37,100,58,37,100,0,0,0,0,0,0,82,101,109,111,118,101,100,0,115,97,109,112,108,101,95,114,97,116,101,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,119,97,118,32,99,104,97,110,110,101,108,32,111,114,100,101,114,32,37,100,58,32,37,115,10,0,100,105,114,101,99,116,0,0,111,117,116,45,111,102,45,109,101,109,111,114,121,32,101,114,114,111,114,46,32,83,107,105,112,112,105,110,103,32,99,111,118,101,114,32,97,114,116,32,98,108,111,99,107,46,10,0,73,110,118,97,108,105,100,32,116,105,109,101,32,98,97,115,101,32,105,110,32,116,104,101,111,114,97,32,115,116,114,101,97,109,44,32,97,115,115,117,109,105,110,103,32,50,53,32,70,80,83,10,0,0,0,0,105,110,118,97,108,105,100,32,99,104,97,110,110,101,108,32,99,111,117,110,116,46,32,83,112,101,101,120,32,109,117,115,116,32,98,101,32,109,111,110,111,32,111,114,32,115,116,101,114,101,111,46,10,0,0,0,109,105,110,114,97,116,101,0,85,110,107,110,111,119,110,32,115,107,101,108,101,116,111,110,32,118,101,114,115,105,111,110,32,37,100,46,37,100,10,0,76,97,115,116,32,112,97,99,107,101,116,32,109,117,115,116,32,98,101,32,116,114,117,110,99,97,116,101,100,32,116,111,32,37,100,32,40,117,110,105,109,112,108,101,109,101,110,116,101,100,41,46,10,0,0,0,80,67,32,80,97,105,110,116,98,114,117,115,104,32,80,67,88,32,105,109,97,103,101,0,1,116,101,120,116,0,0,0,70,65,84,65,76,32,101,114,114,111,114,44,32,102,105,108,101,32,100,117,114,97,116,105,111,110,32,116,111,111,32,108,111,110,103,32,102,111,114,32,116,105,109,101,98,97,115,101,44,32,116,104,105,115,32,102,105,108,101,32,119,105,108,108,32,110,111,116,32,98,101,10,112,108,97,121,97,98,108,101,32,119,105,116,104,32,113,117,105,99,107,116,105,109,101,46,32,67,104,111,111,115,101,32,97,32,100,105,102,102,101,114,101,110,116,32,116,105,109,101,98,97,115,101,32,111,114,32,97,32,100,105,102,102,101,114,101,110,116,10,99,111,110,116,97,105,110,101,114,32,102,111,114,109,97,116,10,0,0,0,106,112,101,103,108,115,0,0,115,97,109,101,113,0,0,0,68,101,97,116,104,32,77,101,116,97,108,0,0,0,0,0,82,101,113,117,101,115,116,101,100,32,111,117,116,112,117,116,32,102,111,114,109,97,116,32,39,37,115,39,32,105,115,32,110,111,116,32,97,32,115,117,105,116,97,98,108,101,32,111,117,116,112,117,116,32,102,111,114,109,97,116,10,0,0,0,87,77,47,80,117,98,108,105,115,104,101,114,0,0,0,0,97,112,112,108,105,99,97,116,105,111,110,47,111,103,103,0,48,120,37,48,49,54,108,108,88,37,48,49,54,108,108,88,37,48,49,54,108,108,88,37,48,49,54,108,108,88,0,0,66,114,111,107,101,110,32,102,105,108,101,44,32,37,115,107,101,121,102,114,97,109,101,32,110,111,116,32,99,111,114,114,101,99,116,108,121,32,109,97,114,107,101,100,46,10,0,0,105,110,100,101,120,32,60,32,115,116,45,62,110,98,95,105,110,100,101,120,95,101,110,116,114,105,101,115,0,0,0,0,73,84,67,72,0,0,0,0,83,101,116,32,109,97,120,105,109,117,109,32,98,105,116,114,97,116,101,32,116,111,108,101,114,97,110,99,101,32,40,105,110,32,98,105,116,115,47,115,41,46,32,82,101,113,117,105,114,101,115,32,98,117,102,115,105,122,101,32,116,111,32,98,101,32,115,101,116,46,0,0,116,114,121,32,116,111,32,105,110,116,101,114,108,101,97,118,101,32,111,117,116,112,117,116,116,101,100,32,112,97,99,107,101,116,115,32,98,121,32,100,116,115,0,0,0,0,0,0,112,99,120,0,0,0,0,0,79,117,116,32,111,102,32,109,101,109,111,114,121,10,0,0,83,101,116,32,112,114,101,102,101,114,114,101,100,32,79,103,103,32,112,97,103,101,32,115])
.concat([105,122,101,46,0,0,0,0,73,110,118,97,108,105,100,32,97,117,100,105,111,32,115,116,114,101,97,109,46,32,69,120,97,99,116,108,121,32,111,110,101,32,77,80,51,32,97,117,100,105,111,32,115,116,114,101,97,109,32,105,115,32,114,101,113,117,105,114,101,100,46,10,0,0,0,0,0,0,0,0,102,111,114,99,101,32,118,105,100,101,111,32,99,111,100,101,99,32,40,39,99,111,112,121,39,32,116,111,32,99,111,112,121,32,115,116,114,101,97,109,41,0,0,0,0,0,0,0,97,117,100,105,111,47,120,45,109,112,101,103,0,0,0,0,87,114,105,116,101,32,115,101,112,97,114,97,116,101,32,109,111,111,102,47,109,100,97,116,32,97,116,111,109,115,32,102,111,114,32,101,97,99,104,32,116,114,97,99,107,0,0,0,70,111,117,110,100,32,109,101,100,105,97,32,100,97,116,97,32,116,97,103,32,77,80,69,71,32,105,110,100,105,99,97,116,105,110,103,32,116,104,105,115,32,105,115,32,97,32,77,79,86,45,112,97,99,107,101,100,32,77,80,69,71,45,80,83,46,10,0,0,0,0,0,109,112,50,44,109,112,51,44,109,50,97,0,0,0,0,0,84,80,85,66,0,0,0,0,112,107,116,46,115,105,122,101,32,33,61,32,100,115,95,112,97,99,107,101,116,95,115,105,122,101,32,42,32,100,115,95,115,112,97,110,32,40,37,100,32,37,100,32,37,100,41,10,0,0,0,0,0,0,0,0,78,117,109,98,101,114,32,111,102,32,116,104,101,32,102,105,114,115,116,32,102,114,97,109,101,32,116,104,97,116,32,115,104,111,117,108,100,32,98,101,32,112,97,115,115,101,100,32,116,111,32,116,104,101,32,111,117,116,112,117,116,0,0,0,109,97,120,114,97,116,101,0,83,82,0,0,0,0,0,0,115,119,115,95,102,108,97,103,115,61,0,0,0,0,0,0,66,101,97,109,32,83,111,102,116,119,97,114,101,32,86,66,0,0,0,0,0,0,0,0,112,105,120,95,102,109,116,0,67,97,110,110,111,116,32,98,117,102,102,101,114,32,109,111,114,101,32,102,114,97,109,101,115,46,32,67,111,110,115,117,109,101,32,115,111,109,101,32,97,118,97,105,108,97,98,108,101,32,102,114,97,109,101,115,32,98,101,102,111,114,101,32,97,100,100,105,110,103,32,110,101,119,32,111,110,101,115,46,10,0,0,0,0,0,0,0,72,105,110,116,72,97,110,100,108,101,114,0,0,0,0,0,44,32,0,0,0,0,0,0,102,111,114,109,97,116,0,0,118,99,111,100,101,99,0,0,83,101,116,32,114,97,116,101,32,99,111,110,116,114,111,108,32,101,113,117,97,116,105,111,110,46,32,87,104,101,110,32,99,111,109,112,117,116,105,110,103,32,116,104,101,32,101,120,112,114,101,115,115,105,111,110,44,32,98,101,115,105,100,101,115,32,116,104,101,32,115,116,97,110,100,97,114,100,32,102,117,110,99,116,105,111,110,115,32,100,101,102,105,110,101,100,32,105,110,32,116,104,101,32,115,101,99,116,105,111,110,32,39,69,120,112,114,101,115,115,105,111,110,32,69,118,97,108,117,97,116,105,111,110,39,44,32,116,104,101,32,102,111,108,108,111,119,105,110,103,32,102,117,110,99,116,105,111,110,115,32,97,114,101,32,97,118,97,105,108,97,98,108,101,58,32,98,105,116,115,50,113,112,40,98,105,116,115,41,44,32,113,112,50,98,105,116,115,40,113,112,41,46,32,65,108,115,111,32,116,104,101,32,102,111,108,108,111,119,105,110,103,32,99,111,110,115,116,97,110,116,115,32,97,114,101,32,97,118,97,105,108,97,98,108,101,58,32,105,84,101,120,32,112,84,101,120,32,116,101,120,32,109,118,32,102,67,111,100,101,32,105,67,111,117,110,116,32,109,99,86,97,114,32,118,97,114,32,105,115,73,32,105,115,80,32,105,115,66,32,97,118,103,81,80,32,113,67,111,109,112,32,97,118,103,73,73,84,101,120,32,97,118,103,80,73,84,101,120,32,97,118,103,80,80,84,101,120,32,97,118,103,66,80,84,101,120,32,97,118,103,84,101,120,46,0,0,0,0,0,118,98,0,0,0,0,0,0,104,105,110,116,0,0,0,0,111,118,101,114,114,105,100,101,0,0,0,0,0,0,0,0,69,114,114,111,114,32,111,112,101,110,105,110,103,32,111,117,116,112,117,116,32,102,105,108,101,115,58,32,0,0,0,0,109,112,51,0,0,0,0,0,97,97,99,32,98,105,116,115,116,114,101,97,109,32,101,114,114,111,114,10,0,0,0,0,114,99,95,101,113,0,0,0,65,77,86,32,86,105,100,101,111,0,0,0,0,0,0,0,84,105,109,101,67,111,100,101,72,97,110,100,108,101,114,0,114,97,116,101,32,99,111,110,116,114,111,108,32,111,118,101,114,114,105,100,101,32,102,111,114,32,115,112,101,99,105,102,105,99,32,105,110,116,101,114,118,97,108,115,0,0,0,0,109,117,108,116,105,112,108,101,32,102,111,117,114,99,99,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,114,99,95,111,118,101,114,114,105,100,101,95,99,111,117,110,116,0,0,0,0,0,0,0,109,112,51,0,0,0,0,0,97,109,118,0,0,0,0,0,83,117,98,116,105,116,108,101,72,97,110,100,108,101,114,0,114,99,95,111,118,101,114,114,105,100,101,0,0,0,0,0,118,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,105,110,105,116,105,97,108,105,122,101,32,104,105,110,116,105,110,103,32,111,102,32,115,116,114,101,97,109,32,37,100,10,0,0,0,0,0,0,114,99,95,113,109,111,100,95,102,114,101,113,0,0,0,0,79,110,50,32,86,80,54,32,40,70,108,97,115,104,32,118,101,114,115,105,111,110,44,32,119,105,116,104,32,97,108,112,104,97,32,99,104,97,110,110,101,108,41,0,0,0,0,0,84,67,79,77,0,0,0,0,115,98,116,108,0,0,0,0,84,114,117,110,99,97,116,101,32,101,120,105,115,116,105,110,103,32,102,105,108,101,115,32,111,110,32,119,114,105,116,101,0,0,0,0,0,0,0,0,100,105,115,99,97,114,100,32,116,104,114,101,115,104,111,108,100,0,0,0,0,0,0,0,101,120,112,101,114,105,109,101,110,116,97,108,32,113,117,97,110,116,105,122,101,114,32,109,111,100,117,108,97,116,105,111,110,0,0,0,0,0,0,0,118,112,54,97,0,0,0,0,98,103,114,52,95,98,121,116,101,0,0,0,0,0,0,0,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,48,49,50,51,52,53,54,55,56,57,43,45,46,0,0,0,0,0,0,0,67,108,111,115,101,100,67,97,112,116,105,111,110,72,97,110,100,108,101,114,0,0,0,0,66,105,115,113,117,101,0,0,119,0,0,0,0,0,0,0,118,105,100,101,111,47,120,45,109,115,45,97,115,102,0,0,118,100,116,0,0,0,0,0,37,115,0,0,0,0,0,0,109,97,120,0,0,0,0,0,70,105,108,116,101,114,32,110,111,116,32,102,111,117,110,100,0,0,0,0,0,0,0,0,66,67,0,0,0,0,0,0,114,99,95,113,109,111,100,95,97,109,112,0,0,0,0,0,97,115,102,95,114,101,97,100,95,112,116,115,32,102,97,105,108,101,100,10,0,0,0,0,66,73,67,85,66,73,67,32,115,99,97,108,101,114,44,32,0,0,0,0,0,0,0,0,82,101,110,100,101,114,119,97,114,101,32,84,88,68,32,40,84,101,88,116,117,114,101,32,68,105,99,116,105,111,110,97,114,121,41,32,105,109,97,103,101,0,0,0,0,0,0,0,112,114,105,110,116,32,105,110,102,111,0,0,0,0,0,0,108,105,98,97,118,102,105,108,116,101,114,47,98,117,102,102,101,114,46,99,0,0,0,0,99,108,99,112,0,0,0,0,115,114,99,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,108,105,98,97,118,102,105,108,116,101,114,47,97,117,100,105,111,46,99,0,0,0,0,0,100,105,115,97,98,108,101,32,118,105,100,101,111,0,0,0,83,66,82,32,114,101,115,101,116,32,102,97,105,108,101,100,46,32,83,119,105,116,99,104,105,110,103,32,83,66,82,32,116,111,32,112,117,114,101,32,117,112,115,97,109,112,108,105,110,103,32,109,111,100,101,46,10,0,0,0,0,0,0,0,111,115,114,0,0,0,0,0,100,101,102,97,117,108,116,0,50,49,58,50,51,58,52,54,0,0,0,0,0,0,0,0,83,112,101,99,105,102,105,101,100,32,112,105,120,101,108,32,102,111,114,109,97,116,32,37,115,32,105,115,32,105,110,118,97,108,105,100,32,111,114,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,84,104,105,115,32,115,121,110,116,97,120,32,105,115,32,100,101,112,114,101,99,97,116,101,100,44,32,117,115,101,32,39,124,39,32,116,111,32,115,101,112,97,114,97,116,101,32,37,115,46,10,0,0,0,0,0,100,101,102,97,117,108,116,0,104,111,119,32,116,111,32,107,101,101,112,32,113,117,97,110,116,105,122,101,114,32,98,101,116,119,101,101,110,32,113,109,105,110,32,97,110,100,32,113,109,97,120,32,40,48,32,61,32,99,108,105,112,44,32,49,32,61,32,117,115,101,32,100,105,102,102,101,114,101,110,116,105,97,98,108,101,32,102,117,110,99,116,105,111,110,41,0,70,111,114,109,97,116,32,110,97,109,101,32,116,111,111,32,108,111,110,103,10,0,0,0,116,120,100,0,0,0,0,0,83,111,117,110,100,72,97,110,100,108,101,114,0,0,0,0,45,97,115,121,110,99,32,105,115,32,102,111,114,119,97,114,100,101,100,32,116,111,32,108,97,118,102,105,32,115,105,109,105,108,97,114,108,121,32,116,111,32,45,97,102,32,97,114,101,115,97,109,112,108,101,61,37,115,46,10,0,0,0,0,97,116,114,105,109,0,0,0,118,110,0,0,0,0,0,0,99,104,97,110,110,101,108,95,108,97,121,111,117,116,0,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,99,104,97,110,110,101,108,32,109,111,100,101,32,37,100,58,32,37,115,10,0,0,0,0,0,0,69,114,114,111,114,32,119,104,105,108,101,32,112,97,114,115,105,110,103,32,101,120,112,114,101,115,115,105,111,110,32,39,37,115,39,10,0,0,0,0,113,115,113,117,105,115,104,0,86,46,70,108,97,115,104,32,80,84,88,32,105,109,97,103,101,0,0,0,0,0,0,0,115,111,117,110,0,0,0,0,83,117,110,112,108,117,115,32,74,80,69,71,32,40,83,80,53,88,41,0,0,0,0,0,100,101,112,114,101,99,97,116,101,100,32,117,115,101,32,45,103,32,49,0,0,0,0,0,83,107,97,0,0,0,0,0,100,105,115,99,0,0,0,0,97,102,105,102,111,0,0,0,116,105,109,101,95,114,101,102,101,114,101,110,99,101,0,0,105,110,100,101,120,61,61,48,32,124,124,32,105,101,91,45,49,93,46,116,105,109,101,115,116,97,109,112,32,60,32,116,105,109,101,115,116,97,109,112,0,0,0,0,0,0,0,0,116,105,109,101,99,111,100,101,0,0,0,0,0,0,0,0,117,115,101,32,77,80,69,71,32,113,117,97,110,116,105,122,101,114,115,32,105,110,115,116,101,97,100,32,111,102,32,72,46,50,54,51,0,0,0,0,115,111,114,116,100,116,115,0,112,116,120,0,0,0,0,0,111,103,103,112,97,103,101,115,105,122,101,0,0,0,0,0,66,117,102,102,101,114,32,118,105,100,101,111,32,102,114,97,109,101,115,44,32,97,110,100,32,109,97,107,101,32,116,104,101,109,32,97,99,99,101,115,115,105,98,108,101,32,116,111,32,116,104,101,32,102,105,108,116,101,114,99,104,97,105,110,46,0,0,0,0,0,0,0,71,111,116,32,109,111,114,101,32,116,104,97,110,32,111,110,101,32,112,105,99,116,117,114,101,32,105,110,32,115,116,114,101,97,109,32,37,100,44,32,105,103,110,111,114,105,110,103,46,10,0,0,0,0,0,0,86,105,100,101,111,72,97,110,100,108,101,114,0,0,0,0,105,110,116,114,97,0,0,0,115,101,112,97,114,97,116,101,95,109,111,111,102,0,0,0,67,104,97,112,116,101,114,32,37,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,102,105,108,101,10,0,0,0,102,102,98,117,102,102,101,114,115,105,110,107,0,0,0,0,100,105,115,99,0,0,0,0,100,105,115,99,97,114,100,105,110,103,32,109,115,32,102,97,114,116,10,0,0,0,0,0,115,116,97,114,116,95,102,114,97,109,101,0,0,0,0,0,109,112,101,103,95,113,117,97,110,116,0,0,0,0,0,0,83,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,66,101,116,104,101,115,100,97,32,86,73,68,32,118,105,100,101,111,0,0,0,0,0,0,104,101,105,103,104,116,0,0,37,100,32,98,117,102,102,101,114,115,32,113,117,101,117,101,100,32,105,110,32,37,115,44,32,115,111,109,101,116,104,105,110,103,32,109,97,121,32,98,101,32,119,114,111,110,103,46,10,0,0,0,0,0,0,0,118,105,100,101,0,0,0,0,69,79,70,32,111,110,32,115,105,110,107,32,108,105,110,107,32,37,115,58,37,115,46,10,0,0,0,0,0,0,0,0,37,115,37,115,0,0,0,0,84,104,105,115,32,102,105,108,116,101,114,32,100,111,101,115,32,110,111,116,32,116,97,107,101,32,97,110,121,32,111,112,116,105,111,110,115,44,32,98,117,116,32,111,112,116,105,111,110,115,32,119,101,114,101,32,112,114,111,118,105,100,101,100,58,32,37,115,46,10,0,0,115,101,116,32,116,104,101,32,110,117,109,98,101,114,32,111,102,32,98,105,116,115,32,112,101,114,32,114,97,119,32,115,97,109,112,108,101,0,0,0,88,88,88,0,0,0,0,0,98,108,111,99,107,95,97,108,105,103,110,0,0,0,0,0,77,101,100,105,97,32,116,121,112,101,32,109,105,115,109,97,116,99,104,32,98,101,116,119,101,101,110,32,116,104,101,32,39,37,115,39,32,102,105,108,116,101,114,32,111,117,116,112,117,116,32,112,97,100,32,37,100,32,40,37,115,41,32,97,110,100,32,116,104,101,32,39,37,115,39,32,102,105,108,116,101,114,32,105,110,112,117,116,32,112,97,100,32,37,100,32,40,37,115,41,10,0,0,0,98,105,116,32,114,101,115,101,114,118,111,105,114,32,101,114,114,111,114,58,32,10,108,51,95,115,105,100,101,45,62,109,97,105,110,95,100,97,116,97,95,98,101,103,105,110,58,32,37,105,32,10,82,101,115,118,111,105,114,32,115,105,122,101,58,32,32,32,32,32,32,32,32,32,32,32,32,32,37,105,32,10,114,101,115,118,32,100,114,97,105,110,32,40,112,111,115,116,41,32,32,32,32,32,32,32,32,32,37,105,32,10,114,101,115,118,32,100,114,97,105,110,32,40,112,114,101,41,32,32,32,32,32,32,32,32,32,32,37,105,32,10,104,101,97,100,101,114,32,97,110,100,32,115,105,100,101,105,110,102,111,58,32,32,32,32,32,32,37,105,32,10,100,97,116,97,32,98,105,116,115,58,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,37,105,32,10,116,111,116,97,108,32,98,105,116,115,58,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,37,105,32,40,114,101,109,97,105,110,100,101,114,58,32,37,105,41,32,10,98,105,116,115,112,101,114,102,114,97,109,101,58,32,32,32,32,32,32,32,32,32,32,32,32,32,37,105,32,10,0,0,98,101,116,104,115,111,102,116,118,105,100,0,0,0,0,0,98,105,116,115,95,112,101,114,95,114,97,119,95,115,97,109,112,108,101,0,0,0,0,0,111,117,116,112,117,116,0,0,104,97,115,95,98,95,102,114,97,109,101,115,0,0,0,0,73,110,116,101,114,112,108,97,121,32,67,57,51,0,0,0,109,104,108,114,0,0,0,0,102,111,114,109,97,116,0,0,99,111,110,115,105,100,101,114,32,116,104,105,110,103,115,32,116,104,97,116,32,97,32,115,97,110,101,32,101,110,99,111,100,101,114,32,115,104,111,117,108,100,32,110,111,116,32,100,111,32,97,115,32,97,110,32,101,114,114,111,114,0,0,0,99,57,51,0,0,0,0,0,68,97,116,97,72,97,110,100,108,101,114,0,0,0,0,0,115,101,116,32,112,105,120,101,108,32,102,111,114,109,97,116,0,0,0,0,0,0,0,0,108,111,103,108,101,118,101,108,0,0,0,0,0,0,0,0,97,103,103,114,101,115,115,105,118,101,0,0,0,0,0,0,83,71,73,32,82,76,69,32,56,45,98,105,116,0,0,0,100,104,108,114,0,0,0,0,112,105,120,95,102,109,116,0,99,111,110,115,105,100,101,114,32,97,108,108,32,115,112,101,99,32,110,111,110,32,99,111,109,112,108,105,97,110,99,105,101,115,32,97,115,32,101,114,114,111,114,115,0,0,0,0,115,103,105,114,108,101,0,0,98,103,114,52,0,0,0,0,66,101,105,103,101,0,0,0,37,115,32,101,110,99,111,100,105,110,103,32,102,97,105,108,101,100,10,0,0,0,0,0,115,101,116,32,97,115,112,101,99,116,32,114,97,116,105,111,32,40,52,58,51,44,32,49,54,58,57,32,111,114,32,49,46,51,51,51,51,44,32,49,46,55,55,55,55,41,0,0,37,100,120,37,100,0,0,0,109,111,100,0,0,0,0,0,70,73,76,84,69,82,95,78,79,84,95,70,79,85,78,68,0,0,0,0,0,0,0,0,102,114,111,110,116,32,114,105,103,104,116,45,111,102,45,99,101,110,116,101,114,0,0,0,99,111,109,112,108,105,97,110,116,0,0,0,0,0,0,0,66,73,76,73,78,69,65,82,32,115,99,97,108,101,114,44,32,0,0,0,0,0,0,0,83,71,73,32,105,109,97,103,101,0,0,0,0,0,0,0,112,114,105,110,116,95,105,110,102,111,0,0,0,0,0,0,82,101,115,97,109,112,108,105,110,103,32,111,110,108,121,32,115,117,112,112,111,114,116,101,100,32,119,105,116,104,32,105,110,116,101,114,110,97,108,32,115,49,54,47,115,51,50,47,102,108,116,47,100,98,108,10,0,0,0,0,0,0,0,0,97,115,112,101,99,116,0,0,78,111,110,32,109,111,110,111,116,111,110,101,32,116,105,109,101,32,98,111,114,100,101,114,115,10,0,0,0,0,0,0,100,101,99,111,100,101,95,112,99,101,58,32,73,110,112,117,116,32,98,117,102,102,101,114,32,101,120,104,97,117,115,116,101,100,32,98,101,102,111,114,101,32,69,78,68,32,101,108,101,109,101,110,116,32,102,111,117,110,100,10,0,0,0,0,83,112,101,99,105,102,105,101,100,32,115,97,109,112,108,101,32,102,111,114,109,97,116,32,37,115,32,105,115,32,105,110,118,97,108,105,100,32,111,114,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,99,111,110,115,105,100,101,114,32,116,104,105,110,103,115,32,116,104,97,116,32,118,105,111,108,97,116,101,32,116,104,101,32,115,112,101,99,32,97,110,100,32,104,97,118,101,32,110,111,116,32,98,101,101,110,32,115,101,101,110,32,105,110,32,116,104,101,32,119,105,108,100,32,97,115,32,101,114,114,111,114,115,0,0,0,0,0,0,115,103,105,0,0,0,0,0,58,102,105,114,115,116,95,112,116,115,61,48,0,0,0,0,115,105,122,101,0,0,0,0,102,111,114,109,97,116,0,0,85,110,115,117,112,112,111,114,116,101,100,32,110,117,109,98,101,114,32,111,102,32,99,104,97,110,110,101,108,115,32,37,100,10,0,0,0,0,0,0,99,97,114,101,102,117,108,0,78,105,110,116,101,110,100,111,32,71,97,109,101,99,117,98,101,32,84,72,80,32,118,105,100,101,111,0,0,0,0,0,76,117,99,105,100,97,32,71,114,97,110,100,101,0,0,0,115,112,53,120,0,0,0,0,115,101,116,32,102,114,97,109,101,32,115,105,122,101,32,40,87,120,72,32,111,114,32,97,98,98,114,101,118,105,97,116,105,111,110,41,0,0,0,0,65,108,116,101,114,110,97,116,105,118,101,0,0,0,0,0,87,77,47,80,97,114,116,79,102,83,101,116,0,0,0,0,37,108,108,117,0,0,0,0,100,101,109,117,120,101,114,32,105,110,106,101,99,116,105,110,103,32,115,107,105,112,32,37,100,10,0,0,0,0,0,0,73,83,77,80,0,0,0,0,97,98,111,114,116,32,100,101,99,111,100,105,110,103,32,111,110,32,109,105,110,111,114,32,101,114,114,111,114,32,100,101,116,101,99,116,105,111,110,0,100,105,115,99,97,114,100,32,99,111,114,114,117,112,116,101,100,32,102,114,97,109,101,115,0,0,0,0,0,0,0,0,116,104,112,0,0,0,0,0,79,103,103,32,109,117,120,101,114,0,0,0,0,0,0,0,116,101,120,116,0,0,0,0,115,0,0,0,0,0,0,0,70,114,97,103,109,101,110,116,32,97,116,32,118,105,100,101,111,32,107,101,121,102,114,97,109,101,115,0,0,0,0,0,82,101,102,101,114,101,110,99,101,100,32,81,84,32,99,104,97,112,116,101,114,32,116,114,97,99,107,32,110,111,116,32,102,111,117,110,100,10,0,0,84,80,79,83,0,0,0,0,112,97,99,107,101,116,32,102,114,97,103,109,101,110,116,32,112,111,115,105,116,105,111,110,32,105,110,118,97,108,105,100,32,37,117,44,37,117,32,110,111,116,32,105,110,32,37,117,10,0,0,0,0,0,0,0,108,105,98,97,118,102,105,108,116,101,114,47,116,114,105,109,46,99,0,0,0,0,0,0,101,120,112,108,111,100,101,0,82,84,67,83,84,65,82,84,0,0,0,0,0,0,0,0,32,119,105,116,104,32,97,114,103,115,32,39,37,115,39,0,86,67,51,47,68,78,120,72,68,0,0,0,0,0,0,0,118,105,100,101,111,95,115,105,122,101,0,0,0,0,0,0,100,101,102,97,117,108,116,0,113,117,101,114,121,95,102,111,114,109,97,116,115,58,32,37,100,32,113,117,101,114,105,101,100,44,32,37,100,32,109,101,114,103,101,100,44,32,37,100,32,97,108,114,101,97,100,121,32,100,111,110,101,44,32,37,100,32,100,101,108,97,121,101,100,10,0,0,0,0,0,0,69,114,114,111,114,32,97,112,112,108,121,105,110,103,32,111,112,116,105,111,110,115,32,116,111,32,116,104,101,32,102,105,108,116,101,114,46,10,0,0,114,97,116,101,0,0,0,0,100,101,116,101,99,116,32,105,109,112,114,111,112,101,114,32,98,105,116,115,116,114,101,97,109,32,108,101,110,103,116,104,0,0,0,0,0,0,0,0,100,110,120,104,100,0,0,0,115,101,116,32,102,114,97,109,101,32,114,97,116,101,32,40,72,122,32,118,97,108,117,101,44,32,102,114,97,99,116,105,111,110,32,111,114,32,97,98,98,114,101,118,105,97,116,105,111,110,41,0,0,0,0,0,69,114,114,111,114,32,111,112,101,110,105,110,103,32,105,110,112,117,116,32,102,105,108,101,115,58,32,0,0,0,0,0,85,110,107,110,111,119,110,32,104,101,108,112,32,111,112,116,105,111,110,32,39,37,115,39,46,10,0,0,0,0,0,0,98,117,102,102,101,114,0,0,70,101,101,98,108,101,32,70,105,108,101,115,47,83,99,117,109,109,86,77,32,68,88,65,0,0,0,0,0,0,0,0,114,0,0,0,0,0,0,0,100,101,116,101,99,116,32,98,105,116,115,116,114,101,97,109,32,115,112,101,99,105,102,105,99,97,116,105,111,110,32,100,101,118,105,97,116,105,111,110,115,0,0,0,0,0,0,0,100,120,97,0,0,0,0,0,115,101,116,32,116,104,101,32,110,117,109,98,101,114,32,111,102,32,118,105,100,101,111,32,102,114,97,109,101,115,32,116,111,32,114,101,99,111,114,100,0,0,0,0,0,0,0,0,83,117,99,99,101,115,115,102,117,108,108,121,32,112,97,114,115,101,100,32,97,32,103,114,111,117,112,32,111,102,32,111,112,116,105,111,110,115,46,10,0,0,0,0,0,0,0,0,52,107,115,99,111,112,101,0,98,105,116,115,116,114,101,97,109,0,0,0,0,0,0,0,71,73,70,32,40,71,114,97,112,104,105,99,115,32,73,110,116,101,114,99,104,97,110,103,101,32,70,111,114,109,97,116,41,0,0,0,0,0,0,0,117,114,108,32,0,0,0,0,118,102,114,97,109,101,115,0,52,107,102,108,97,116,0,0,118,101,114,105,102,121,32,101,109,98,101,100,100,101,100,32,67,82,67,115,0,0,0,0,103,105,102,0,0,0,0,0,98,103,114,56,0,0,0,0,65,122,117,114,101,0,0,0,102,108,117,115,104,32,37,115,32,37,100,46,37,100,0,0,112,114,105,110,116,32,116,105,109,101,115,116,97,109,112,32,100,101,98,117,103,103,105,110,103,32,105,110,102,111,0,0,103,97,117,115,115,0,0,0,71,101,110,101,114,105,99,32,101,114,114,111,114,32,105,110,32,97,110,32,101,120,116,101,114,110,97,108,32,108,105,98,114,97,114,121,0,0,0,0,52,107,0,0,0,0,0,0,70,82,67,0,0,0,0,0,99,114,99,99,104,101,99,107,0,0,0,0,0,0,0,0,70,65,83,84,95,66,73,76,73,78,69,65,82,32,115,99,97,108,101,114,44,32,0,0,84,73,70,70,32,105,109,97,103,101,0,0,0,0,0,0,110,97,116,117,114,97,108,32,98,105,99,117,98,105,99,32,115,112,108,105,110,101,0,0,102,97,116,97,108,32,101,114,114,111,114,44,32,105,110,112,117,116,32,105,115,32,110,111,116,32,97,32,115,105,110,103,108,101,32,112,97,99,107,101,116,44,32,105,109,112,108,101,109,101,110,116,32,97,32,65,86,80,97,114,115,101,114,32,102,111,114,32,105,116,10,0,82,101,113,117,101,115,116,101,100,32,115,97,109,112,108,101,32,102,111,114,109,97,116,32,37,115,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,105,110,116,101,114,110,97,108,108,121,44,32,83,49,54,47,83,51,50,47,70,76,84,47,68,66,76,32,105,115,32,115,117,112,112,111,114,116,101,100,10,0,0,0,100,101,98,117,103,95,116,115,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,98,105,116,115,116,114,101,97,109,44,32,98,115,95,112,111,105,110,116,101,114,32,112,111,105,110,116,115,32,116,111,32,97,32,109,105,100,100,108,101,32,110,111,105,115,101,32,98,111,114,100,101,114,32,111,117,116,115,105,100,101,32,116,104,101,32,116,105,109,101,32,98,111,114,100,101,114,115,32,116,97,98,108,101,58,32,37,100,10,0,0,0,0,0,0,0,83,97,109,112,108,101,32,114,97,116,101,32,105,110,100,101,120,32,105,110,32,112,114,111,103,114,97,109,32,99,111,110,102,105,103,32,101,108,101,109,101,110,116,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,116,104,101,32,115,97,109,112,108,101,32,114,97,116,101,32,105,110,100,101,120,32,99,111,110,102,105,103,117,114,101,100,32,98,121,32,116,104,101,32,99,111,110,116,97,105,110,101,114,46,10,0,0,50,107,115,99,111,112,101,0,37,100,0,0,0,0,0,0,115,101,116,32,101,114,114,111,114,32,100,101,116,101,99,116,105,111,110,32,102,108,97,103,115,0,0,0,0,0,0,0,116,105,102,102,0,0,0,0,58,109,105,110,95,104,97,114,100,95,99,111,109,112,61,37,102,0,0,0,0,0,0,0,101,120,116,114,97,99,116,32,97,110,32,97,116,116,97,99,104,109,101,110,116,32,105,110,116,111,32,97,32,102,105,108,101,0,0,0,0,0,0,0,104,101,105,103,104,116,0,0,70,97,105,108,101,100,32,116,111,32,115,101,116,32,118,97,108,117,101,32,39,37,115,39,32,102,111,114,32,111,112,116,105,111,110,32,39,37,115,39,58,32,37,115,10,0,0,0,83,101,116,115,32,116,104,101,32,105,109,112,117,108,115,101,32,98,108,111,99,107,32,98,105,97,115,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,116,104,101,32,115,97,109,112,108,101,32,114,97,116,101,32,37,100,58,32,37,115,10,0,0,0,50,107,102,108,97,116,0,0,101,114,114,95,100,101,116,101,99,116,0,0,0,0,0,0,84,105,101,114,116,101,120,32,76,105,109,105,116,101,100,32,83,69,81,32,118,105,100,101,111,0,0,0,0,0,0,0,76,111,115,115,108,101,115,115,32,74,80,69,71,0,0,0,100,117,109,112,95,97,116,116,97,99,104,109,101,110,116,0,92,120,37,48,50,120,0,0,73,110,100,117,115,116,114,105,97,108,0,0,0,0,0,0,102,105,108,101,110,97,109,101,0,0,0,0,0,0,0,0,111,114,105,103,105,110,97,116,111,114,95,114,101,102,101,114,101,110,99,101,0,0,0,0,50,107,0,0,0,0,0,0,73,110,118,97,108,105,100,32,115,116,114,101,97,109,32,105,110,100,101,120,32,37,100,10,0,0,0,0,0,0,0,0,101,110,99,111,100,101,114,0,81,80,32,111,102,102,115,101,116,32,98,101,116,119,101,101,110,32,80,45,32,97,110,100,32,66,45,102,114,97,109,101,115,0,0,0,0,0,0,0,100,105,115,99,97,114,100,99,111,114,114,117,112,116,0,0,116,105,101,114,116,101,120,115,101,113,118,105,100,101,111,0,108,105,98,97,118,102,111,114,109,97,116,47,109,112,51,101,110,99,46,99,0,0,0,0,97,100,100,32,97,110,32,97,116,116,97,99,104,109,101,110,116,32,116,111,32,116,104,101,32,111,117,116,112,117,116,32,102,105,108,101,0,0,0,0,102,114,97,103,95,107,101,121,102,114,97,109,101,0,0,0,92,37,99,0,0,0,0,0,115,116,114,101,97,109,32,37,100,44,32,116,105,109,101,115,99,97,108,101,32,110,111,116,32,115,101,116,10,0,0,0,112,101,114,102,111,114,109,101,114,0,0,0,0,0,0,0,104,100,49,48,56,48,0,0,67,97,110,110,111,116,32,97,112,112,101,110,100,32,112,97,108,101,116,116,101,32,116,111,32,112,97,99,107,101,116,10,0,0,0,0,0,0,0,0,115,116,97,114,116,95,115,97,109,112,108,101,32,60,32,101,110,100,95,115,97,109,112,108,101,0,0,0,0,0,0,0,98,95,113,111,102,102,115,101,116,0,0,0,0,0,0,0,82,84,67,84,73,77,69,0,69,114,114,111,114,32,105,110,105,116,105,97,108,105,122,105,110,103,32,102,105,108,116,101,114,32,39,37,115,39,0,0,68,101,108,112,104,105,110,101,32,83,111,102,116,119,97,114,101,32,73,110,116,101,114,110,97,116,105,111,110,97,108,32,67,73,78,32,118,105,100,101,111,0,0,0,0,0,0,0,119,105,100,116,104,0,0,0,97,99,99,101,112,116,32,97,108,108,32,99,104,97,110,110,101,108,32,99,111,117,110,116,115,0,0,0,0,0,0,0,73,109,112,111,115,115,105,98,108,101,32,116,111,32,99,111,110,118,101,114,116,32,98,101,116,119,101,101,110,32,116,104,101,32,102,111,114,109,97,116,115,32,115,117,112,112,111,114,116,101,100,32,98,121,32,116,104,101,32,102,105,108,116,101,114,32,39,37,115,39,32,97,110,100,32,116,104,101,32,102,105,108,116,101,114,32,39,37,115,39,10,0,0,0,0,0,69,114,114,111,114,32,97,112,112,108,121,105,110,103,32,103,101,110,101,114,105,99,32,102,105,108,116,101,114,32,111,112,116,105,111,110,115,46,10,0,97,116,116,97,99,104,0,0,103,108,111,98,97,108,0,0,104,100,55,50,48,0,0,0,97,108,108,111,119,32,110,111,110,45,115,116,97,110,100,97,114,100,105,122,101,100,32,101,120,112,101,114,105,109,101,110,116,97,108,32,116,104,105,110,103,115,0,0,0,0,0,0,100,115,105,99,105,110,118,105,100,101,111,0,0,0,0,0,112,114,105,110,116,32,112,114,111,103,114,101,115,115,32,114,101,112,111,114,116,32,100,117,114,105,110,103,32,101,110,99,111,100,105,110,103,0,0,0,105,110,112,117,116,0,0,0,37,48,52,100,37,48,50,100,37,48,50,100,45,37,48,50,100,37,48,50,100,37,48,50,100,0,0,0,0,0,0,0,104,100,52,56,48,0,0,0,101,120,112,101,114,105,109,101,110,116,97,108,0,0,0,0,84,114,117,101,118,105,115,105,111,110,32,84,97,114,103,97,32,105,109,97,103,101,0,0,115,116,97,116,115,0,0,0,37,115,0,0,0,0,0,0,101,103,97,0,0,0,0,0,97,108,108,111,119,32,117,110,111,102,102,105,99,105,97,108,32,101,120,116,101,110,115,105,111,110,115,0,0,0,0,0,116,97,114,103,97,0,0,0,114,101,97,100,32,99,111,109,112,108,101,120,32,102,105,108,116,101,114,103,114,97,112,104,32,100,101,115,99,114,105,112,116,105,111,110,32,102,114,111,109,32,97,32,102,105,108,101,0,0,0,0,0,0,0,0,37,115,32,115,116,97,114,116,101,100,32,111,110,32,37,48,52,100,45,37,48,50,100,45,37,48,50,100,32,97,116,32,37,48,50,100,58,37,48,50,100,58,37,48,50,100,10,82,101,112,111,114,116,32,119,114,105,116,116,101,110,32,116,111,32,34,37,115,34,10,0,0,65,112,112,108,121,105,110,103,32,111,112,116,105,111,110,32,37,115,32,40,37,115,41,32,119,105,116,104,32,97,114,103,117,109,101,110,116,32,37,115,46,10,0,0,0,0,0,0,99,103,97,0,0,0,0,0,117,110,111,102,102,105,99,105,97,108,0,0,0,0,0,0,79,110,50,32,86,80,54,32,40,70,108,97,115,104,32,118,101,114,115,105,111,110,41,0,102,105,108,116,101,114,95,99,111,109,112,108,101,120,95,115,99,114,105,112,116,0,0,0,70,97,105,108,101,100,32,116,111,32,111,112,101,110,32,114,101,112,111,114,116,32,34,37,115,34,58,32,37,115,10,0,119,104,117,120,103,97,0,0,110,111,114,109,97,108,0,0,118,112,54,102,0,0,0,0,117,121,121,118,121,121,52,49,49,0,0,0,0,0,0,0,65,113,117,97,109,97,114,105,110,101,0,0,0,0,0,0,86,105,100,101,111,0,0,0,108,97,118,102,105,0,0,0,37,48,50,88,0,0,0,0,119,0,0,0,0,0,0,0,115,113,117,105,115,104,0,0,69,88,84,69,82,78,65,76,0,0,0,0,0,0,0,0,119,104,115,120,103,97,0,0,102,114,111,110,116,32,108,101,102,116,45,111,102,45,99,101,110,116,101,114,0,0,0,0,115,116,114,105,99,116,108,121,32,99,111,110,102,111,114,109,32,116,111,32,97,108,108,32,116,104,101,32,116,104,105,110,103,115,32,105,110,32,116,104,101,32,115,112,101,99,32,110,111,32,109,97,116,116,101,114,32,119,104,97,116,32,116,104,101,32,99,111,110,115,101,113,117,101,110,99,101,115,0,0,99,45,62,99,104,114,68,115,116,72,32,60,61,32,100,115,116,72,0,0,0,0,0,0,79,110,50,32,86,80,54,0,115,112,108,105,110,101,0,0,85,115,105,110,103,32,100,111,117,98,108,101,32,112,114,101,99,105,115,105,111,110,32,109,111,100,101,10,0,0,0,0,103,114,97,112,104,95,100,101,115,99,114,105,112,116,105,111,110,0,0,0,0,0,0,0,79,117,116,32,111,102,32,109,101,109,111,114,121,32,98,117,105,108,100,105,110,103,32,114,101,112,111,114,116,32,102,105,108,101,32,110,97,109,101,10,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,98,105,116,115,116,114,101,97,109,44,32,116,111,111,32,109,97,110,121,32,83,66,82,32,101,110,118,101,108,111,112,101,115,32,105,110,32,86,65,82,86,65,82,32,116,121,112,101,32,83,66,82,32,102,114,97,109,101,58,32,37,100,10,0,84,111,111,32,109,97,110,121,32,99,104,97,110,110,101,108,115,10,0,0,0,0,0,0,119,113,117,120,103,97,0,0,84,104,101,32,109,97,120,105,109,117,109,32,118,97,108,117,101,32,102,111,114,32,108,111,119,114,101,115,32,115,117,112,112,111,114,116,101,100,32,98,121,32,116,104,101,32,100,101,99,111,100,101,114,32,105,115,32,37,100,10,0,0,0,0,115,116,114,105,99,116,108,121,32,99,111,110,102,111,114,109,32,116,111,32,97,32,111,108,100,101,114,32,109,111,114,101,32,115,116,114,105,99,116,32,118,101,114,115,105,111,110,32,111,102,32,116,104,101,32,115,112,101,99,32,111,114,32,114,101,102,101,114,101,110,99,101,32,115,111,102,116,119,97,114,101,0,0,0,0,0,0,0,118,112,54,0,0,0,0,0,97,115,121,110,99,61,37,100,0,0,0,0,0,0,0,0,99,114,101,97,116,101,32,97,32,99,111,109,112,108,101,120,32,102,105,108,116,101,114,103,114,97,112,104,0,0,0,0,119,105,100,116,104,0,0,0,37,112,45,37,116,46,108,111,103,0,0,0,0,0,0,0,105,98,108,111,99,107,0,0,85,110,97,98,108,101,32,116,111,32,101,110,97,98,108,101,32,83,66,82,32,102,111,114,32,69,76,68,58,32,37,115,10,0,0,0,0,0,0,0,119,113,115,120,103,97,0,0,118,101,114,121,0,0,0,0,79,110,50,32,86,80,53,0,108,106,112,101,103,0,0,0,102,105,108,116,101,114,95,99,111,109,112,108,101,120,0,0,84,101,99,104,110,111,0,0,85,110,107,110,111,119,110,32,107,101,121,32,39,37,115,39,32,105,110,32,70,70,82,69,80,79,82,84,10,0,0,0,87,77,47,79,114,105,103,105,110,97,108,70,105,108,101,110,97,109,101,0,0,0,0,0,99,114,101,97,116,105,111,110,95,116,105,109,101,0,0,0,119,111,120,103,97,0,0,0,68,114,111,112,112,101,100,32,99,111,114,114,117,112,116,101,100,32,112,97,99,107,101,116,32,40,115,116,114,101,97,109,32,61,32,37,100,41,10,0,73,83,70,84,0,0,0,0,104,111,119,32,115,116,114,105,99,116,108,121,32,116,111,32,102,111,108,108,111,119,32,116,104,101,32,115,116,97,110,100,97,114,100,115,0,0,0,0,105,103,110,111,114,101,32,100,116,115,0,0,0,0,0,0,118,112,53,0,0,0,0,0,79,103,103,83,0,0,0,0,98,101,115,116,95,98,105,116,114,97,116,101,95,105,100,120,32,62,61,32,48,0,0,0,114,101,105,110,105,116,32,102,105,108,116,101,114,103,114,97,112,104,32,111,110,32,105,110,112,117,116,32,112,97,114,97,109,101,116,101,114,32,99,104,97,110,103,101,115,0,0,0,77,97,107,101,32,116,104,101,32,105,110,105,116,105,97,108,32,109,111,111,118,32,97,116,111,109,32,101,109,112,116,121,32,40,110,111,116,32,115,117,112,112,111,114,116,101,100,32,98,121,32,81,117,105,99,107,84,105,109,101,41,0,0,0,102,105,108,101,0,0,0,0,116,105,109,101,99,111,100,101,0,0,0,0,0,0,0,0,84,80,69,51,0,0,0,0,119,117,120,103,97,0,0,0,102,114,101,101,105,110,103,32,105,110,99,111,109,112,108,101,116,101,32,112,97,99,107,101,116,32,115,105,122,101,32,37,100,44,32,110,101,119,32,37,100,10,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,115,116,114,105,99,116,0,0,84,66,0,0,0,0,0,0,37,115,58,37,115,0,0,0,86,77,119,97,114,101,32,83,99,114,101,101,110,32,67,111,100,101,99,32,47,32,86,77,119,97,114,101,32,86,105,100,101,111,0,0,0,0,0,0,97,108,108,95,99,104,97,110,110,101,108,95,99,111,117,110,116,115,0,0,0,0,0,0,37,115,0,0,0,0,0,0,33,105,110,112,117,116,45,62,102,105,108,116,101,114,95,102,114,97,109,101,32,124,124,32,40,33,105,110,112,117,116,45,62,115,116,97,114,116,95,102,114,97,109,101,32,38,38,32,33,105,110,112,117,116,45,62,101,110,100,95,102,114,97,109,101,41,0,0,0,0,0,0,114,101,105,110,105,116,95,102,105,108,116,101,114,0,0,0,70,97,105,108,101,100,32,116,111,32,112,97,114,115,101,32,70,70,82,69,80,79,82,84,32,101,110,118,105,114,111,110,109,101,110,116,32,118,97,114,105,97,98,108,101,58,32,37,115,10,0,0,0,0,0,0,119,115,120,103,97,0,0,0,116,114,117,110,99,97,116,101,100,32,102,114,97,109,101,115,0,0,0,0,0,0,0,0,118,109,110,99,0,0,0,0,102,105,108,101,110,97,109,101,0,0,0,0,0,0,0,0,69,114,114,111,114,32,112,97,114,115,105,110,103,32,103,108,111,98,97,108,32,111,112,116,105,111,110,115,58,32,0,0,58,0,0,0,0,0,0,0,119,120,103,97,0,0,0,0,116,114,117,110,99,0,0,0,74,80,69,71,32,50,48,48,48,0,0,0,0,0,0,0,114,101,97,100,32,115,116,114,101,97,109,32,102,105,108,116,101,114,103,114,97,112,104,32,100,101,115,99,114,105,112,116,105,111,110,32,102,114,111,109,32,97,32,102,105,108,101,0,61,0,0,0,0,0,0,0,119,118,103,97,0,0,0,0,119,111,114,107,32,97,114,111,117,110,100,32,118,97,114,105,111,117,115,32,98,117,103,115,32,105,110,32,77,105,99,114,111,115,111,102,116,39,115,32,98,114,111,107,101,110,32,100,101,99,111,100,101,114,115,0,106,112,101,103,50,48,48,48,0,0,0,0,0,0,0,0,102,105,108,116,101,114,95,115,99,114,105,112,116,0,0,0,37,115,99,111,110,102,105,103,117,114,97,116,105,111,110,58,32,45,45,99,114,111,115,115,45,112,114,101,102,105,120,61,101,109,32,45,45,99,99,61,101,109,99,99,32,45,45,101,110,97,98,108,101,45,99,114,111,115,115,45,99,111,109,112,105,108,101,32,45,45,116,97,114,103,101,116,45,111,115,61,110,111,110,101,32,45,45,97,114,99,104,61,120,56,54,95,51,50,32,45,45,112,114,101,102,105,120,61,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,32,45,45,101,120,116,114,97,45,99,102,108,97,103,115,61,45,73,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,105,110,99,108,117,100,101,32,45,45,99,112,117,61,103,101,110,101,114,105,99,32,45,45,100,105,115,97,98,108,101,45,104,119,97,99,99,101,108,115,32,45,45,100,105,115,97,98,108,101,45,115,116,114,105,112,112,105,110,103,32,45,45,100,105,115,97,98,108,101,45,112,116,104,114,101,97,100,115,32,45,45,100,105,115,97,98,108,101,45,100,111,99,32,45,45,100,105,115,97,98,108,101,45,100,101,98,117,103,32,45,45,100,105,115,97,98,108,101,45,97,115,109,32,45,45,100,105,115,97,98,108,101,45,110,101,116,119,111,114,107,32,45,45,100,105,115,97,98,108,101,45,101,118,101,114,121,116,104,105,110,103,32,45,45,100,105,115,97,98,108,101,45,102,102,112,108,97,121,32,45,45,100,105,115,97,98,108,101,45,102,102,112,114,111,98,101,32,45,45,100,105,115,97,98,108,101,45,102,102,115,101,114,118,101,114,32,45,45,100,105,115,97,98,108,101,45])
.concat([111,117,116,100,101,118,61,115,100,108,32,45,45,101,110,97,98,108,101,45,111,112,116,105,109,105,122,97,116,105,111,110,115,32,45,45,101,110,97,98,108,101,45,112,114,111,116,111,99,111,108,61,102,105,108,101,32,45,45,101,110,97,98,108,101,45,102,105,108,116,101,114,61,97,114,101,115,97,109,112,108,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,119,97,118,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,115,49,54,108,101,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,117,56,32,45,45,101,110,97,98,108,101,45,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,108,105,98,102,100,107,45,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,111,118,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,102,100,107,95,97,97,99,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,52,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,115,102,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,97,115,102,10,0,0,0,0,0,0,0,79,112,116,105,111,110,32,37,115,32,40,37,115,41,32,99,97,110,110,111,116,32,98,101,32,97,112,112,108,105,101,100,32,116,111,32,37,115,32,37,115,32,45,45,32,121,111,117,32,97,114,101,32,116,114,121,105,110,103,32,116,111,32,97,112,112,108,121,32,97,110,32,105,110,112,117,116,32,111,112,116,105,111,110,32,116,111,32,97,110,32,111,117,116,112,117,116,32,102,105,108,101,32,111,114,32,118,105,99,101,32,118,101,114,115,97,46,32,77,111,118,101,32,116,104,105,115,32,111,112,116,105,111,110,32,98,101,102,111,114,101,32,116,104,101,32,102,105,108,101,32,105,116,32,98,101,108,111,110,103,115,32,116,111,46,10,0,0,104,115,120,103,97,0,0,0,84,104,101,32,118,97,108,117,101,32,102,111,114,32,37,115,32,119,97,115,32,37,115,32,119,104,105,99,104,32,105,115,32,110,111,116,32,119,105,116,104,105,110,32,37,102,32,45,32,37,102,10,0,0,0,0,109,115,0,0,0,0,0,0,67,104,105,110,101,115,101,32,65,86,83,32,40,65,117,100,105,111,32,86,105,100,101,111,32,83,116,97,110,100,97,114,100,41,32,40,65,86,83,49,45,80,50,44,32,74,105,90,104,117,110,32,112,114,111,102,105,108,101,41,0,0,0,0,102,105,108,116,101,114,95,103,114,97,112,104,0,0,0,0,101,109,99,99,32,40,69,109,115,99,114,105,112,116,101,110,32,71,67,67,45,108,105,107,101,32,114,101,112,108,97,99,101,109,101,110,116,41,32,49,46,55,46,49,32,40,99,111,109,109,105,116,32,57,48,100,99,97,99,53,53,49,102,101,101,54,57,99,102,50,56,52,97,101,52,52,51,49,100,98,57,52,57,98,99,54,57,55,57,54,97,49,99,41,0,0,82,101,99,101,105,118,101,100,32,115,105,103,110,97,108,32,37,100,58,32,116,101,114,109,105,110,97,116,105,110,103,46,10,0,0,0,0,0,0,0,113,115,120,103,97,0,0,0,100,99,95,99,108,105,112,0,99,97,118,115,0,0,0,0,117,121,118,121,52,50,50,0,65,113,117,97,0,0,0,0,65,117,100,105,111,0,0,0,115,101,116,32,115,116,114,101,97,109,32,102,105,108,116,101,114,103,114,97,112,104,0,0,37,100,47,37,100,0,0,0,50,49,58,50,55,58,49,55,0,0,0,0,0,0,0,0,98,101,110,99,104,58,32,109,97,120,114,115,115,61,37,105,107,66,10,0,0,0,0,0,116,105,109,101,0,0,0,0,73,109,109,101,100,105,97,116,101,32,101,120,105,116,32,114,101,113,117,101,115,116,101,100,0,0,0,0,0,0,0,0,115,120,103,97,0,0,0,0,70,76,67,0,0,0,0,0,108,97,115,116,73,110,67,104,114,66,117,102,32,43,32,49,32,45,32,99,104,114,83,114,99,83,108,105,99,101,89,32,62,61,32,48,0,0,0,0,104,112,101,108,95,99,104,114,111,109,97,0,0,0,0,0,108,105,98,115,119,115,99,97,108,101,47,117,116,105,108,115,46,99,0,0,0,0,0,0,70,108,97,115,104,32,83,99,114,101,101,110,32,86,105,100,101,111,32,118,49,0,0,0,108,97,110,99,122,111,115,0,73,110,112,117,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,104,97,115,32,97,32,100,105,102,102,101,114,101,110,116,32,110,117,109,98,101,114,32,111,102,32,99,104,97,110,110,101,108,115,32,116,104,97,110,32,116,104,101,32,110,117,109,98,101,114,32,111,102,32,117,115,101,100,32,99,104,97,110,110,101,108,115,44,32,105,103,110,111,114,105,110,103,32,108,97,121,111,117,116,10,0,0,0,0,0,0,105,108,108,101,103,97,108,32,105,105,100,10,0,0,0,0,102,105,108,116,101,114,0,0,78,111,118,32,50,49,32,50,48,49,51,0,0,0,0,0,73,110,118,97,108,105,100,32,98,105,116,115,116,114,101,97,109,44,32,116,111,111,32,109,97,110,121,32,83,66,82,32,101,110,118,101,108,111,112,101,115,32,105,110,32,70,73,88,70,73,88,32,116,121,112,101,32,83,66,82,32,102,114,97,109,101,58,32,37,100,10,0,108,105,98,102,97,97,99,32,37,100,46,37,100,0,0,0,105,110,100,101,120,32,61,61,32,115,105,122,101,0,0,0,113,120,103,97,0,0,0,0,87,97,114,110,105,110,103,58,32,110,111,116,32,99,111,109,112,105,108,101,100,32,119,105,116,104,32,116,104,114,101,97,100,32,115,117,112,112,111,114,116,44,32,117,115,105,110,103,32,116,104,114,101,97,100,32,101,109,117,108,97,116,105,111,110,10,0,0,0,0,0,0,101,100,103,101,32,112,97,100,100,105,110,103,32,98,117,103,32,40,97,117,116,111,100,101,116,101,99,116,101,100,32,112,101,114,32,70,79,85,82,67,67,47,118,101,114,115,105,111,110,41,0,0,0,0,0,0,102,108,97,115,104,115,118,0,103,114,97,112,104,32,37,100,32,105,110,112,117,116,32,102,114,111,109,32,115,116,114,101,97,109,32,37,100,58,37,100,0,0,0,0,0,0,0,0,115,101,116,32,112,114,111,102,105,108,101,0,0,0,0,0,115,97,109,112,108,101,95,97,115,112,101,99,116,95,114,97,116,105,111,0,0,0,0,0,37,115,98,117,105,108,116,32,111,110,32,37,115,32,37,115,32,119,105,116,104,32,37,115,10,0,0,0,0,0,0,0,115,119,105,116,99,104,32,112,111,105,110,116,32,105,110,32,56,107,104,122,0,0,0,0,48,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,115,101,116,32,116,104,101,32,65,79,84,32,37,100,58,32,37,115,10,0,0,0,102,111,114,99,101,95,107,101,121,95,102,114,97,109,101,115,0,0,0,0,0,0,0,0,117,120,103,97,0,0,0,0,101,100,103,101,0,0,0,0,75,97,114,108,32,77,111,114,116,111,110,39,115,32,118,105,100,101,111,32,99,111,100,101,99,0,0,0,0,0,0,0,110,111,116,32,119,114,105,116,105,110,103,32,39,99,104,97,110,39,32,116,97,103,32,100,117,101,32,116,111,32,108,97,99,107,32,111,102,32,99,104,97,110,110,101,108,32,105,110,102,111,114,109,97,116,105,111,110,10,0,0,0,0,0,0,65,112,112,108,101,32,77,74,80,69,71,45,66,0,0,0,112,114,111,102,105,108,101,0,82,111,99,107,0,0,0,0,32,67,111,112,121,114,105,103,104,116,32,40,99,41,32,37,100,45,37,100,32,116,104,101,32,70,70,109,112,101,103,32,100,101,118,101,108,111,112,101,114,115,0,0,0,0,0,0,108,97,110,103,117,97,103,101,0,0,0,0,0,0,0,0,111,114,105,103,105,110,97,116,105,111,110,95,116,105,109,101,0,0,0,0,0,0,0,0,99,104,97,112,116,101,114,115,0,0,0,0,0,0,0,0,120,103,97,0,0,0,0,0,108,105,98,97,118,102,111,114,109,97,116,47,117,116,105,108,115,46,99,0,0,0,0,0,116,114,97,99,107,0,0,0,100,105,114,101,99,116,45,113,112,101,108,45,98,108,111,99,107,115,105,122,101,32,98,117,103,32,40,97,117,116,111,100,101,116,101,99,116,101,100,32,112,101,114,32,70,79,85,82,67,67,47,118,101,114,115,105,111,110,41,0,0,0,0,0,105,103,110,100,116,115,0,0,107,109,118,99,0,0,0,0,72,101,97,100,101,114,115,32,109,105,115,109,97,116,99,104,32,102,111,114,32,115,116,114,101,97,109,32,37,100,58,32,101,120,112,101,99,116,101,100,32,37,100,32,114,101,99,101,105,118,101,100,32,37,100,46,10,0,0,0,0,0,0,0,76,97,118,102,53,53,46,49,50,46,49,48,49,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,84,65,76,66,0,0,0,0,113,115,99,97,108,101,0,0,101,109,112,116,121,95,109,111,111,118,0,0,0,0,0,0,37,115,32,118,101,114,115,105,111,110,32,103,105,116,45,50,48,49,51,45,49,49,45,50,49,45,101,49,56,102,56,102,54,0,0,0,0,0,0,0,67,111,117,108,100,32,110,111,116,32,97,108,108,111,99,97,116,101,32,102,111,114,99,101,100,32,107,101,121,32,102,114,97,109,101,115,32,97,114,114,97,121,46,10,0,0,0,0,97,108,98,117,109,95,97,114,116,105,115,116,0,0,0,0,115,118,103,97,0,0,0,0,105,103,110,111,114,105,110,103,32,105,110,118,97,108,105,100,32,112,97,99,107,101,116,95,111,98,106,95,115,105,122,101,32,40,37,100,32,37,100,32,37,100,32,37,100,41,10,0,100,101,102,97,117,108,116,0,100,105,114,101,99,116,95,98,108,111,99,107,115,105,122,101,0,0,0,0,0,0,0,0,84,0,0,0,0,0,0,0,102,108,97,103,115,0,0,0,78,117,112,112,101,108,86,105,100,101,111,47,82,84,74,80,69,71,0,0,0,0,0,0,119,58,37,100,32,104,58,37,100,32,112,105,120,102,109,116,58,37,115,32,116,98,58,37,100,47,37,100,32,102,114,58,37,100,47,37,100,32,115,97,114,58,37,100,47,37,100,32,115,119,115,95,112,97,114,97,109,58,37,115,10,0,0,0,115,101,116,32,116,104,101,32,115,117,112,112,111,114,116,101,100,32,99,104,97,110,110,101,108,32,99,111,117,110,116,115,0,0,0,0,0,0,0,0,97,117,116,111,45,105,110,115,101,114,116,101,100,32,114,101,115,97,109,112,108,101,114,32,37,100,0,0,0,0,0,0,40,102,105,108,116,101,114,45,62,102,108,97,103,115,32,38,32,40,40,49,32,60,60,32,49,54,41,32,124,32,40,49,32,60,60,32,49,55,41,41,41,32,33,61,32,40,40,49,32,60,60,32,49,54,41,32,124,32,40,49,32,60,60,32,49,55,41,41,0,0,0,0,117,115,101,32,102,105,120,101,100,32,113,117,97,108,105,116,121,32,115,99,97,108,101,32,40,86,66,82,41,0,0,0,115,119,114,101,115,97,109,112,108,101,0,0,0,0,0,0,69,114,114,111,114,32,119,104,105,108,101,32,111,112,101,110,105,110,103,32,100,101,99,111,100,101,114,32,102,111,114,32,105,110,112,117,116,32,115,116,114,101,97,109,32,35,37,100,58,37,100,32,58,32,37,115,0,0,0,0,0,0,0,0,118,103,97,0,0,0,0,0,113,112,101,108,95,99,104,114,111,109,97,50,0,0,0,0,115,101,103,109,101,110,116,0,110,117,118,0,0,0,0,0,113,0,0,0,0,0,0,0,69,114,114,111,114,32,115,112,108,105,116,116,105,110,103,32,116,104,101,32,97,114,103,117,109,101,110,116,32,108,105,115,116,58,32,0,0,0,0,0,115,119,115,99,97,108,101,0,102,105,108,101,0,0,0,0,114,101,102,99,111,117,110,116,101,100,95,102,114,97,109,101,115,0,0,0,0,0,0,0,113,118,103,97,0,0,0,0,111,108,100,32,115,116,97,110,100,97,114,100,32,113,112,101,108,32,40,97,117,116,111,100,101,116,101,99,116,101,100,32,112,101,114,32,70,79,85,82,67,67,47,118,101,114,115,105,111,110,41,0,0,0,0,0,83,109,97,99,107,101,114,32,118,105,100,101,111,0,0,0,116,109,99,100,0,0,0,0,102,111,117,114,99,99,47,116,97,103,0,0,0,0,0,0,97,118,102,105,108,116,101,114,0,0,0,0,0,0,0,0,68,101,99,111,100,101,114,32,40,99,111,100,101,99,32,37,115,41,32,110,111,116,32,102,111,117,110,100,32,102,111,114,32,105,110,112,117,116,32,115,116,114,101,97,109,32,35,37,100,58,37,100,0,0,0,0,113,113,118,103,97,0,0,0,115,116,100,95,113,112,101,108,0,0,0,0,0,0,0,0,115,109,97,99,107,118,105,100,101,111,0,0,0,0,0,0,102,112,115,32,37,100,32,105,115,32,116,111,111,32,108,97,114,103,101,10,0,0,0,0,102,111,114,99,101,32,99,111,100,101,99,32,116,97,103,47,102,111,117,114,99,99,0,0,97,118,100,101,118,105,99,101,0,0,0,0,0,0,0,0,80,97,114,115,105,110,103,32,97,32,103,114,111,117,112,32,111,102,32,111,112,116,105,111,110,115,58,32,37,115,32,37,115,46,10,0,0,0,0,0,83,68,80,58,10,37,115,10,0,0,0,0,0,0,0,0,49,54,99,105,102,0,0,0,113,112,101,108,95,99,104,114,111,109,97,0,0,0,0,0,65,86,83,32,40,65,117,100,105,111,32,86,105,100,101,111,32,83,116,97,110,100,97,114,100,41,32,118,105,100,101,111,0,0,0,0,0,0,0,0,116,97,103,0,0,0,0,0,97,118,102,111,114,109,97,116,0,0,0,0,0,0,0,0,37,115,10,0,0,0,0,0,52,99,105,102,0,0,0,0,105,108,108,101,103,97,108,32,86,76,67,32,98,117,103,32,40,97,117,116,111,100,101,116,101,99,116,101,100,32,112,101,114,32,70,79,85,82,67,67,41,0,0,0,0,0,0,0,97,118,115,0,0,0,0,0,120,118,109,99,105,100,99,116,0,0,0,0,0,0,0,0,65,110,116,105,113,117,101,87,104,105,116,101,0,0,0,0,69,114,114,111,114,32,119,104,105,108,101,32,102,105,108,116,101,114,105,110,103,46,10,0,115,101,116,32,116,104,101,32,110,117,109,98,101,114,32,111,102,32,102,114,97,109,101,115,32,116,111,32,114,101,99,111,114,100,0,0,0,0,0,0,37,102,0,0,0,0,0,0,97,118,99,111,100,101,99,0,32,40,37,115,32,45,62,32,37,115,41,0,0,0,0,0,97,98,115,0,0,0,0,0,69,88,73,84,0,0,0,0,99,105,102,0,0,0,0,0,98,97,99,107,32,114,105,103,104,116,0,0,0,0,0,0,108,97,115,116,73,110,67,104,114,66,117,102,32,43,32,49,32,45,32,99,104,114,83,114,99,83,108,105,99,101,89,32,60,32,40,99,104,114,83,114,99,83,108,105,99,101,72,41,0,0,0,0,0,0,0,0,97,99,95,118,108,99,0,0,99,45,62,100,115,116,66,112,99,32,62,32,49,52,0,0,90,105,112,32,77,111,116,105,111,110,32,66,108,111,99,107,115,32,86,105,100,101,111,0,115,105,110,99,0,0,0,0,76,105,98,97,118,117,116,105,108,32,104,97,115,32,98,101,101,110,32,108,105,110,107,101,100,32,116,111,32,97,32,98,114,111,107,101,110,32,108,108,114,105,110,116,40,41,10,0,115,116,112,115,0,0,0,0,82,101,113,117,101,115,116,101,100,32,114,101,115,97,109,112,108,105,110,103,32,101,110,103,105,110,101,32,105,115,32,117,110,97,118,97,105,108,97,98,108,101,10,0,0,0,0,0,45,0,0,0,0,0,0,0,105,108,108,101,103,97,108,32,105,99,99,10,0,0,0,0,102,114,97,109,101,115,0,0,37,115,37,45,49,49,115,32,99,111,110,102,105,103,117,114,97,116,105,111,110,58,32,37,115,10,0,0,0,0,0,0,82,101,115,101,114,118,101,100,32,83,66,82,32,101,120,116,101,110,115,105,111,110,115,0,70,73,76,76,58,37,115,10,0,0,0,0,0,0,0,0,32,40,99,111,112,121,41,0,37,45,54,115,32,32,32,37,50,100,32,0,0,0,0,0,113,99,105,102,0,0,0,0,65,108,116,101,114,110,97,116,105,118,101,108,121,32,117,115,101,32,116,104,101,32,110,111,110,32,101,120,112,101,114,105,109,101,110,116,97,108,32,37,115,32,39,37,115,39,46,10,0,0,0,0,0,0,0,0,47,100,101,118,47,114,97,110,100,111,109,0,0,0,0,0,97,109,118,0,0,0,0,0,122,109,98,118,0,0,0,0,115,116,115,115,0,0,0,0,58,99,104,97,110,110,101,108,115,61,37,100,0,0,0,0,99,111,112,121,32,111,114,32,100,105,115,99,97,114,100,32,102,114,97,109,101,115,32,98,101,102,111,114,101,32,115,116,97,114,116,32,116,105,109,101,0,0,0,0,0,0,0,0,112,107,116,95,115,105,122,101,0,0,0,0,0,0,0,0,98,105,116,115,95,108,101,102,116,61,37,100,10,0,0,0,37,115,87,65,82,78,73,78,71,58,32,108,105,98,114,97,114,121,32,99,111,110,102,105,103,117,114,97,116,105,111,110,32,109,105,115,109,97,116,99,104,10,0,0,0,0,0,0,98,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,111,112,101,110,32,116,104,101,32,101,110,99,111,100,101,114,58,32,37,115,10,0,32,91,115,121,110,99,32,35,37,100,58,37,100,93,0,0,65,86,73,79,67,111,110,116,101,120,116,0,0,0,0,0,115,113,99,105,102,0,0,0,112,97,100,100,105,110,103,32,98,117,103,32,40,97,117,116,111,100,101,116,101,99,116,101,100,41,0,0,0,0,0,0,65,109,101,114,105,99,97,110,32,76,97,115,101,114,32,71,97,109,101,115,32,77,77,32,86,105,100,101,111,0,0,0,109,106,112,101,103,98,0,0,99,111,112,121,112,114,105,111,114,115,115,0,0,0,0,0,82,101,103,103,97,101,0,0,45,45,99,114,111,115,115,45,112,114,101,102,105,120,61,101,109,32,45,45,99,99,61,101,109,99,99,32,45,45,101,110,97,98,108,101,45,99,114,111,115,115,45,99,111,109,112,105,108,101,32,45,45,116,97,114,103,101,116,45,111,115,61,110,111,110,101,32,45,45,97,114,99,104,61,120,56,54,95,51,50,32,45,45,112,114,101,102,105,120,61,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,32,45,45,101,120,116,114,97,45,99,102,108,97,103,115,61,45,73,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,105,110,99,108,117,100,101,32,45,45,99,112,117,61,103,101,110,101,114,105,99,32,45,45,100,105,115,97,98,108,101,45,104,119,97,99,99,101,108,115,32,45,45,100,105,115,97,98,108,101,45,115,116,114,105,112,112,105,110,103,32,45,45,100,105,115,97,98,108,101,45,112,116,104,114,101,97,100,115,32,45,45,100,105,115,97,98,108,101,45,100,111,99,32,45,45,100,105,115,97,98,108,101,45,100,101,98,117,103,32,45,45,100,105,115,97,98,108,101,45,97,115,109,32,45,45,100,105,115,97,98,108,101,45,110,101,116,119,111,114,107,32,45,45,100,105,115,97,98,108,101,45,101,118,101,114,121,116,104,105,110,103,32,45,45,100,105,115,97,98,108,101,45,102,102,112,108,97,121,32,45,45,100,105,115,97,98,108,101,45,102,102,112,114,111,98,101,32,45,45,100,105,115,97,98,108,101,45,102,102,115,101,114,118,101,114,32,45,45,100,105,115,97,98,108,101,45,111,117,116,100,101,118,61,115,100,108,32,45,45,101,110,97,98,108,101,45,111,112,116,105,109,105,122,97,116,105,111,110,115,32,45,45,101,110,97,98,108,101,45,112,114,111,116,111,99,111,108,61,102,105,108,101,32,45,45,101,110,97,98,108,101,45,102,105,108,116,101,114,61,97,114,101,115,97,109,112,108,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,119,97,118,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,115,49,54,108,101,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,117,56,32,45,45,101,110,97,98,108,101,45,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,108,105,98,102,100,107,45,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,111,118,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,102,100,107,95,97,97,99,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,52,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,115,102,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,97,115,102,0,87,77,47,76,97,110,103,117,97,103,101,0,0,0,0,0,100,97,116,101,0,0,0,0,32,32,83,116,114,101,97,109,32,35,37,100,58,37,100,32,45,62,32,35,37,100,58,37,100,0,0,0,0,0,0,0,121,117,121,118,52,50,50,0,110,116,115,99,45,102,105,108,109,0,0,0,0,0,0,0,115,116,45,62,114,101,113,117,101,115,116,95,112,114,111,98,101,32,60,61,32,48,0,0,73,80,82,84,0,0,0,0,110,111,95,112,97,100,100,105,110,103,0,0,0,0,0,0,100,105,115,97,98,108,101,32,65,86,80,97,114,115,101,114,115,44,32,116,104,105,115,32,110,101,101,100,115,32,110,111,102,105,108,108,105,110,32,116,111,111,0,0,0,0,0,0,109,109,118,105,100,101,111,0,76,97,118,102,53,53,46,49,50,46,49,48,49,0,0,0,72,101,97,100,101,114,32,112,97,114,115,105,110,103,32,102,97,105,108,101,100,32,102,111,114,32,115,116,114,101,97,109,32,37,100,10,0,0,0,0,101,110,99,111,100,101,114,0,85,110,115,117,112,112,111,114,116,101,100,32,110,117,109,98,101,114,32,111,102,32,99,104,97,110,110,101,108,115,44,32,110,111,116,32,119,114,105,116,105,110,103,32,88,105,110,103,32,104,101,97,100,101,114,46,10,0,0,0,0,0,0,0,99,111,112,121,32,105,110,105,116,105,97,108,32,110,111,110,45,107,101,121,102,114,97,109,101,115,0,0,0,0,0,0,109,97,120,105,109,117,109,32,109,111,111,118,32,115,105,122,101,32,115,111,32,105,116,32,99,97,110,32,98,101,32,112,108,97,99,101,100,32,97,116,32,116,104,101,32,98,101,103,105,110,0,0,0,0,0,0,97,118,117,116,105,108,0,0,85,82,76,67,111,110,116,101,120,116,0,0,0,0,0,0,109,111,111,118,32,97,116,111,109,32,110,111,116,32,102,111,117,110,100,10,0,0,0,0,32,45,62,32,83,116,114,101,97,109,32,35,37,100,58,37,100,32,40,37,115,41,10,0,84,80,69,50,0,0,0,0,102,105,108,109,0,0,0,0,112,116,115,32,33,61,32,40,40,105,110,116,54,52,95,116,41,48,120,56,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,85,76,76,41,0,102,102,32,97,115,102,32,115,107,105,112,32,37,100,32,40,117,110,107,110,111,119,110,32,115,116,114,101,97,109,41,10,0,0,0,0,0,0,0,0,78,117,109,98,101,114,32,111,102,32,116,104,101,32,102,105,114,115,116,32,97,117,100,105,111,32,115,97,109,112,108,101,32,116,104,97,116,32,115,104,111,117,108,100,32,98,101,32,100,114,111,112,112,101,100,32,97,103,97,105,110,0,0,0,40,97,117,116,111,100,101,116,101,99,116,101,100,32,105,102,32,70,79,85,82,67,67,32,61,61,32,85,77,80,52,41,0,0,0,0,0,0,0,0,83,84,65,82,84,84,0,0,115,99,97,108,101,0,0,0,67,97,109,83,116,117,100,105,111,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,112,97,114,97,109,101,116,101,114,115,32,112,114,111,118,105,100,101,100,46,10,0,0,0,99,104,97,110,110,101,108,95,99,111,117,110,116,115,0,0,39,97,114,101,115,97,109,112,108,101,39,32,102,105,108,116,101,114,32,110,111,116,32,112,114,101,115,101,110,116,44,32,99,97,110,110,111,116,32,99,111,110,118,101,114,116,32,97,117,100,105,111,32,102,111,114,109,97,116,115,46,10,0,0,101,110,97,98,108,101,0,0,99,111,112,121,105,110,107,102,0,0,0,0,0,0,0,0,37,115,108,105,98,37,45,49,49,115,32,37,50,100,46,37,51,100,46,37,51,100,32,47,32,37,50,100,46,37,51,100,46,37,51,100,10,0,0,0,32,32,37,115,0,0,0,0,115,112,97,108,0,0,0,0,48,120,0,0,0,0,0,0,117,109,112,52,0,0,0,0,99,115,99,100,0,0,0,0,97,115,102,0,0,0,0,0,101,114,114,111,114,0,0,0,117,115,97,103,101,58,32,37,115,32,91,111,112,116,105,111,110,115,93,32,91,91,105,110,102,105,108,101,32,111,112,116,105,111,110,115,93,32,45,105,32,105,110,102,105,108,101,93,46,46,46,32,123,91,111,117,116,102,105,108,101,32,111,112,116,105,111,110,115,93,32,111,117,116,102,105,108,101,125,46,46,46,10,0,0,0,0,0,32,32,0,0,0,0,0,0,32,32,70,105,108,101,32,37,115,32,45,62,32,83,116,114,101,97,109,32,35,37,100,58,37,100,10,0,0,0,0,0,115,110,116,115,99,0,0,0,88,118,105,100,32,105,110,116,101,114,108,97,99,105,110,103,32,98,117,103,32,40,97,117,116,111,100,101,116,101,99,116,101,100,32,105,102,32,70,79,85,82,67,67,32,61,61,32,88,86,73,88,41,0,0,0,85,110,97,98,108,101,32,116,111,32,112,97,114,115,101,32,111,112,116,105,111,110,32,118,97,108,117,101,32,34,37,115,34,32,97,115,32,105,109,97,103,101,32,115,105,122,101,10,0,0,0,0,0,0,0,0,66,77,80,32,40,87,105,110,100,111,119,115,32,97,110,100,32,79,83,47,50,32,98,105,116,109,97,112,41,0,0,0,101,120,105,116,32,111,110,32,101,114,114,111,114,0,0,0,105,110,95,116,115,32,33,61,32,40,40,105,110,116,54,52,95,116,41,48,120,56,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,85,76,76,41,0,0,0,0,0,0,0,99,109,100,117,116,105,108,115,46,99,0,0,0,0,0,0,32,40,103,114,97,112,104,32,37,100,41,0,0,0,0,0,113,112,97,108,0,0,0,0,32,32,32,32,76,97,115,116,32,109,101,115,115,97,103,101,32,114,101,112,101,97,116,101,100,32,37,100,32,116,105,109,101,115,13,0,0,0,0,0,120,118,105,100,95,105,108,97,99,101,0,0,0,0,0,0,98,109,112,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,120,101,114,114,111,114,0,0,105,32,61,61,32,110,98,95,99,111,100,101,99,115,0,0,77,105,115,115,105,110,103,32,97,114,103,117,109,101,110,116,32,102,111,114,32,111,112,116,105,111,110,32,39,37,115,39,10,0,0,0,0,0,0,0,63,0,0,0,0,0,0,0,98,101,0,0,0,0,0,0,113,110,116,115,99,0,0,0,115,111,109,101,32,111,108,100,32,108,97,118,99,45,103,101,110,101,114,97,116,101,100,32,77,83,77,80,69,71,52,118,51,32,102,105,108,101,115,32,40,110,111,32,97,117,116,111,100,101,116,101,99,116,105,111,110,41,0,0,0,0,0,0,68,117,99,107,32,84,114,117,101,77,111,116,105,111,110,32,50,46,48,0,0,0,0,0,37,102,10,0,0,0,0,0,116,105,109,101,115,116,97,109,112,32,101,114,114,111,114,32,100,101,108,116,97,32,116,104,114,101,115,104,111,108,100,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,32,32,83,116,114,101,97,109,32,35,37,100,58,37,100,32,40,37,115,41,32,45,62,32,37,115,0,0,0,0,0,0,116,109,112,91,48,93,32,61,61,32,48,32,38,38,32,116,109,112,91,49,93,32,61,61,32,48,0,0,0,0,0,0,112,97,108,0,0,0,0,0,66,83,70,95,78,79,84,95,70,79,85,78,68,0,0,0,83,87,82,0,0,0,0,0,111,108,100,95,109,115,109,112,101,103,52,0,0,0,0,0,116,114,117,101,109,111,116,105,111,110,50,0,0,0,0,0,99,112,117,102,108,97,103,115,0,0,0,0,0,0,0,0,110,95,102,111,114,99,101,100,0,0,0,0,0,0,0,0,120,118,109,99,109,99,0,0,65,108,105,99,101,66,108,117,101,0,0,0,0,0,0,0,78,111,32,109,111,114,101,32,111,117,116,112,117,116,32,115,116,114,101,97,109,115,32,116,111,32,119,114,105,116,101,32,116,111,44,32,102,105,110,105,115,104,105,110,103,46,10,0,100,116,115,95,101,114,114,111,114,95,116,104,114,101,115,104,111,108,100,0,0,0,0,0,37,108,108,100,0,0,0,0,79,117,116,32,111,102,32,109,101,109,111,114,121,10,0,0,83,116,114,101,97,109,32,109,97,112,112,105,110,103,58,10,0,0,0,0,0,0,0,0,108,111,103,0,0,0,0,0,69,110,100,32,111,102,32,102,105,108,101,0,0,0,0,0,56,42,40,99,45,62,115,116,101,112,95,109,105,110,117,115,49,43,49,41,32,62,61,32,99,45,62,100,101,112,116,104,95,109,105,110,117,115,49,43,49,0,0,0,0,0,0,0,110,116,115,99,0,0,0,0,66,82,0,0,0,0,0,0,115,101,116,32,115,119,114,32,110,117,109,98,101,114,32,111,102,32,111,117,116,112,117,116,32,115,97,109,112,108,101,32,98,105,116,115,0,0,0,0,99,104,114,66,117,102,73,110,100,101,120,32,60,32,50,32,42,32,118,67,104,114,66,117,102,83,105,122,101,0,0,0,97,117,116,111,100,101,116,101,99,116,0,0,0,0,0,0,37,100,32,99,104,97,110,110,101,108,115,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,70,114,97,112,115,0,0,0,103,101,110,114,101,0,0,0,103,97,117,115,115,105,97,110,0,0,0,0,0,0,0,0,67,117,115,116,111,109,32,65,86,73,79,67,111,110,116,101,120,116,32,109,97,107,101,115,32,110,111,32,115,101,110,115,101,32,97,110,100,32,119,105,108,108,32,98,101,32,105,103,110,111,114,101,100,32,119,105,116,104,32,65,86,70,77,84,95,78,79,70,73,76,69,32,102,111,114,109,97,116,46,10,0,0,0,0,0,0,0,0,79,117,116,112,117,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,48,120,37,108,108,120,32,105,115,32,105,110,118,97,108,105,100,32,111,114,32,117,110,115,117,112,112,111,114,116,101,100,46,10,0,0,0,0,0,0,0,0,116,105,109,101,115,116,97,109,112,32,100,105,115,99,111,110,116,105,110,117,105,116,121,32,100,101,108,116,97,32,116,104,114,101,115,104,111,108,100,0,80,97,114,97,109,101,116,114,105,99,32,83,116,101,114,101,111,32,115,105,103,110,97,108,101,100,32,116,111,32,98,101,32,110,111,116,45,112,114,101,115,101,110,116,32,98,117,116,32,119,97,115,32,102,111,117,110,100,32,105,110,32,116,104,101,32,98,105,116,115,116,114,101,97,109,46,10,0,0,0,73,109,112,108,105,99,105,116,32,83,66,82,32,119,97,115,32,102,111,117,110,100,32,119,105,116,104,32,97,32,102,105,114,115,116,32,111,99,99,117,114,114,101,110,99,101,32,97,102,116,101,114,32,116,104,101,32,102,105,114,115,116,32,102,114,97,109,101,46,10,0,0,68,105,115,97,98,108,105,110,103,32,117,115,101,95,118,97,114,105,97,98,108,101,95,98,108,111,99,107,95,108,101,110,44,32,105,102,32,116,104,105,115,32,102,97,105,108,115,32,99,111,110,116,97,99,116,32,116,104,101,32,102,102,109,112,101,103,32,100,101,118,101,108,111,112,101,114,115,32,97,110,100,32,115,101,110,100,32,117,115,32,116,104,101,32,102,105,108,101,10,0,0,0,0,0,32,10,9,0,0,0,0,0,114,116,112,0,0,0,0,0,99,45,62,115,116,101,112,95,109,105,110,117,115,49,32,62,61,32,99,45,62,100,101,112,116,104,95,109,105,110,117,115,49,0,0,0,0,0,0,0,89,101,108,108,111,119,71,114,101,101,110,0,0,0,0,0,100,101,115,99,0,0,0,0,84,104,101,32,37,115,32,39,37,115,39,32,105,115,32,101,120,112,101,114,105,109,101,110,116,97,108,32,98,117,116,32,101,120,112,101,114,105,109,101,110,116,97,108,32,99,111,100,101,99,115,32,97,114,101,32,110,111,116,32,101,110,97,98,108,101,100,44,32,97,100,100,32,39,45,115,116,114,105,99,116,32,37,100,39,32,105,102,32,121,111,117,32,119,97,110,116,32,116,111,32,117,115,101,32,105,116,46,10,0,0,0,111,117,116,112,117,116,95,115,97,109,112,108,101,95,98,105,116,115,0,0,0,0,0,0,100,101,115,99,0,0,0,0,119,111,114,107,32,97,114,111,117,110,100,32,110,111,116,32,97,117,116,111,100,101,116,101,99,116,101,100,32,101,110,99,111,100,101,114,32,98,117,103,115,0,0,0,0,0,0,0,102,114,97,112,115,0,0,0,116,114,97,99,107,0,0,0,80,114,111,98,101,32,119,105,116,104,32,115,105,122,101,61,37,100,44,32,112,97,99,107,101,116,115,61,37,100,32,100,101,116,101,99,116,101,100,32,37,115,32,119,105,116,104,32,115,99,111,114,101,61,37,100,10,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,58,99,104,97,110,110,101,108,95,108,97,121,111,117,116,61,48,120,37,108,108,120,0,0,37,105,98,112,112,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,98,121,32,121,117,118,50,114,103,98,10,0,100,116,115,95,100,101,108,116,97,95,116,104,114,101,115,104,111,108,100,0,0,0,0,0,112,107,116,95,112,111,115,0,111,118,101,114,114,101,97,100,44,32,115,107,105,112,32,37,100,32,101,110,100,100,105,115,116,115,58,32,37,100,32,37,100,10,0,0,0,0,0,0,37,115,32,0,0,0,0,0,78,111,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,115,112,101,99,105,102,105,101,100,46,32,84,104,101,32,101,110,99,111,100,101,114,32,119,105,108,108,32,117,115,101,32,86,111,114,98,105,115,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,102,111,114,32,37,100,32,99,104,97,110,110,101,108,115,46,10,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,101,114,114,111,114,0,0,0,67,111,117,108,100,32,110,111,116,32,119,114,105,116,101,32,104,101,97,100,101,114,32,102,111,114,32,111,117,116,112,117,116,32,102,105,108,101,32,35,37,100,32,40,105,110,99,111,114,114,101,99,116,32,99,111,100,101,99,32,112,97,114,97,109,101,116,101,114,115,32,63,41,58,32,37,115,0,0,0,33,99,45,62,112,108,97,110,101,32,38,38,32,33,99,45,62,115,116,101,112,95,109,105,110,117,115,49,32,38,38,32,33,99,45,62,111,102,102,115,101,116,95,112,108,117,115,49,32,38,38,32,33,99,45,62,115,104,105,102,116,32,38,38,32,33,99,45,62,100,101,112,116,104,95,109,105,110,117,115,49,0,0,0,0,0,0,0,89,101,108,108,111,119,0,0,115,101,116,32,115,119,114,32,75,97,105,115,101,114,32,87,105,110,100,111,119,32,66,101,116,97,0,0,0,0,0,0,98,117,103,0,0,0,0,0,97,115,102,0,0,0,0,0,73,110,116,101,108,32,73,110,100,101,111,32,50,0,0,0,37,100,0,0,0,0,0,0,109,112,101,103,118,105,100,101,111,0,0,0,0,0,0,0,97,61,99,111,110,116,114,111,108,58,115,116,114,101,97,109,105,100,61,37,100,13,10,0,77,111,116,105,111,110,32,74,80,69,71,0,0,0,0,0,97,117,100,105,111,32,112,97,100,0,0,0,0,0,0,0,82,97,112,0,0,0,0,0,32,40,37,115,58,32,0,0,103,101,110,114,101,0,0,0,115,119,115,95,102,108,97,103,115,0,0,0,0,0,0,0,97,45,62,112,108,97,110,97,114,0,0,0,0,0,0,0,111,114,105,103,105,110,97,116,105,111,110,95,100,97,116,101,0,0,0,0,0,0,0,0,84,104,101,32,98,105,116,114,97,116,101,32,112,97,114,97,109,101,116,101,114,32,105,115,32,115,101,116,32,116,111,111,32,108,111,119,46,32,73,116,32,116,97,107,101,115,32,98,105,116,115,47,115,32,97,115,32,97,114,103,117,109,101,110,116,44,32,110,111,116,32,107,98,105,116,115,47,115,10,0,40,100,45,62,110,98,95,99,111,109,112,111,110,101,110,116,115,61,61,52,32,124,124,32,100,45,62,110,98,95,99,111,109,112,111,110,101,110,116,115,61,61,50,41,32,61,61,32,33,33,40,100,45,62,102,108,97,103,115,32,38,32,40,49,32,60,60,32,55,41,41,0,87,104,105,116,101,83,109,111,107,101,0,0,0,0,0,0,48,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,107,97,105,115,101,114,95,98,101,116,97,0,0,0,0,0,97,108,98,117,109,0,0,0,115,45,62,100,105,116,104,101,114,46,109,101,116,104,111,100,32,60,32,83,87,82,95,68,73,84,72,69,82,95,78,66,0,0,0,0,0,0,0,0,99,111,100,101,99,95,116,97,103,0,0,0,0,0,0,0,110,111,112,97,114,115,101,0,99,116,120,45,62,99,104,97,110,110,101,108,115,32,61,61,32,111,117,116,45,62,99,104,95,99,111,117,110,116,0,0,105,110,100,101,111,50,0,0,99,111,109,109,101,110,116,0,102,102,109,112,101,103,0,0,108,105,98,97,118,102,111,114,109,97,116,47,111,103,103,100,101,99,46,99,0,0,0,0,109,52,118,0,0,0,0,0,67,111,100,101,99,32,102,111,114,32,115,116,114,101,97,109,32,37,100,32,100,111,101,115,32,110,111,116,32,117,115,101,32,103,108,111,98,97,108,32,104,101,97,100,101,114,115,32,98,117,116,32,99,111,110,116,97,105,110,101,114,32,102,111,114,109,97,116,32,114,101,113,117,105,114,101,115,32,103,108,111,98,97,108,32,104,101,97,100,101,114,115,10,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,115,97,109,112,108,101,32,114,97,116,101,44,32,110,111,116,32,119,114,105,116,105,110,103,32,88,105,110,103,32,104,101,97,100,101,114,46,10,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,97,112,97,100,0,0,0,0,109,111,111,118,95,115,105,122,101,0,0,0,0,0,0,0,32,40,99,111,100,101,99,32,37,115,41,0,0,0,0,0,101,114,114,111,114,32,114,101,97,100,105,110,103,32,104,101,97,100,101,114,58,32,37,100,10,0,0,0,0,0,0,0,69,114,114,111,114,32,119,104,105,108,101,32,111,112,101,110,105,110,103,32,101,110,99,111,100,101,114,32,102,111,114,32,111,117,116,112,117,116,32,115,116,114,101,97,109,32,35,37,100,58,37,100,32,45,32,109,97,121,98,101,32,105,110,99,111,114,114,101,99,116,32,112,97,114,97,109,101,116,101,114,115,32,115,117,99,104,32,97,115,32,98,105,116,95,114,97,116,101,44,32,114,97,116,101,44,32,119,105,100,116,104,32,111,114,32,104,101,105,103,104,116,0,0,0,0,0,0,0,97,114,116,105,115,116,0,0,100,45,62,110,97,109,101,32,38,38,32,100,45,62,110,97,109,101,91,48,93,0,0,0,87,104,105,116,101,0,0,0,112,97,100,115,105,122,101,32,62,61,32,48,0,0,0,0,114,101,116,32,62,61,32,48,0,0,0,0,0,0,0,0,115,101,108,101,99,116,32,75,97,105,115,101,114,32,87,105,110,100,111,119,101,100,32,83,105,110,99,0,0,0,0,0,99,104,58,37,100,32,99,104,108,58,37,115,32,102,109,116,58,37,115,32,114,58,37,100,72,122,32,45,62,32,99,104])
.concat([58,37,100,32,99,104,108,58,37,115,32,102,109,116,58,37,115,32,114,58,37,100,72,122,10,0,0,0,0,0,0,0,100,101,102,97,117,108,116,0,101,110,100,95,115,97,109,112,108,101,0,0,0,0,0,0,102,114,97,109,101,95,98,105,116,115,0,0,0,0,0,0,83,84,65,82,84,80,84,83,0,0,0,0,0,0,0,0,69,114,114,111,114,32,99,114,101,97,116,105,110,103,32,102,105,108,116,101,114,32,39,37,115,39,10,0,0,0,0,0,65,117,116,111,100,101,115,107,32,82,76,69,0,0,0,0,100,97,116,101,0,0,0,0,108,111,97,115,0,0,0,0,100,101,102,97,117,108,116,0,115,101,116,32,116,104,101,32,115,117,112,112,111,114,116,101,100,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,115,0,0,0,0,0,0,0,97,114,101,115,97,109,112,108,101,0,0,0,0,0,0,0,112,111,110,103,32,102,114,111,109,58,37,115,32,37,115,10,0,0,0,0,0,0,0,0,102,105,110,105,115,104,32,101,110,99,111,100,105,110,103,32,119,105,116,104,105,110,32,115,104,111,114,116,101,115,116,32,105,110,112,117,116,0,0,0,66,0,0,0,0,0,0,0,97,117,116,111,0,0,0,0,87,104,101,97,116,0,0,0,107,97,105,115,101,114,0,0,109,105,115,99,95,98,105,116,115,0,0,0,0,0,0,0,105,99,99,95,109,111,100,101,32,37,100,32,105,115,32,114,101,115,101,114,118,101,100,46,10,0,0,0,0,0,0,0,97,97,115,99,0,0,0,0,97,108,98,117,109,0,0,0,104,50,54,52,0,0,0,0,98,114,111,107,101,110,32,101,115,99,97,112,101,32,115,101,113,117,101,110,99,101,10,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,115,104,111,114,116,101,115,116,0,0,0,0,0,0,0,0,72,121,112,101,114,32,102,97,115,116,32,65,117,100,105,111,32,97,110,100,32,86,105,100,101,111,32,101,110,99,111,100,101,114,10,0,0,0,0,0,88,0,0,0,0,0,0,0,110,32,60,61,32,49,48,50,52,0,0,0,0,0,0,0,116,104,114,101,97,100,115,0,114,101,116,32,62,61,32,48,0,0,0,0,0,0,0,0,86,105,111,108,101,116,0,0,115,101,108,101,99,116,32,66,108,97,99,107,109,97,110,32,78,117,116,116,97,108,108,32,87,105,110,100,111,119,101,100,32,83,105,110,99,0,0,0,81,117,101,117,101,32,105,110,112,117,116,32,105,115,32,98,97,99,107,119,97,114,100,32,105,110,32,116,105,109,101,10,0,0,0,0,0,0,0,0,115,107,105,112,95,99,111,117,110,116,0,0,0,0,0,0,87,105,110,110,111,118,32,87,78,86,49,0,0,0,0,0,97,114,116,105,115,116,0,0,101,97,99,51,0,0,0,0,109,111,100,101,0,0,0,0,70,0,0,0,0,0,0,0,67,97,110,110,111,116,32,119,114,105,116,101,32,108,111,103,32,102,105,108,101,32,39,37,115,39,32,102,111,114,32,112,97,115,115,45,49,32,101,110,99,111,100,105,110,103,58,32,37,115,10,0,0,0,0,0,108,105,98,97,118,117,116,105,108,47,112,105,120,100,101,115,99,46,99,0,0,0,0,0,84,117,114,113,117,111,105,115,101,0,0,0,0,0,0,0,98,108,97,99,107,109,97,110,95,110,117,116,116,97,108,108,0,0,0,0,0,0,0,0,112,95,99,111,117,110,116,0,119,110,118,49,0,0,0,0,116,105,116,108,101,0,0,0,100,116,115,0,0,0,0,0,65,65,67,32,40,65,100,118,97,110,99,101,100,32,65,117,100,105,111,32,67,111,100,105,110,103,41,0,0,0,0,0,99,111,112,121,32,105,110,112,117,116,32,115,116,114,101,97,109,32,116,105,109,101,32,98,97,115,101,32,119,104,101,110,32,115,116,114,101,97,109,32,99,111,112,121,105,110,103,0,32,37,99,0,0,0,0,0,85,110,114,101,99,111,103,110,105,122,101,100,32,111,112,116,105,111,110,32,39,37,115,39,10,0,0,0,0,0,0,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,65,117,100,105,111,32,49,0,0,0,97,114,101,115,97,109,112,108,101,0,0,0,0,0,0,0,119,98,0,0,0,0,0,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,65,117,100,105,111,32,49,0,0,0,84,111,109,97,116,111,0,0,115,101,108,101,99,116,32,99,117,98,105,99,0,0,0,0,73,110,118,97,108,105,100,32,112,97,99,107,101,116,10,0,105,95,99,111,117,110,116,0,76,79,67,79,0,0,0,0,83,121,110,116,104,80,111,112,0,0,0,0,0,0,0,0,97,110,117,108,108,0,0,0,97,99,51,0,0,0,0,0,99,111,112,121,116,98,0,0,68,101,99,111,100,101,114,115,0,0,0,0,0,0,0,0,69,114,114,111,114,32,114,101,97,100,105,110,103,32,108,111,103,32,102,105,108,101,32,39,37,115,39,32,102,111,114,32,112,97,115,115,45,50,32,101,110,99,111,100,105,110,103,10,0,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,84,104,105,115,116,108,101,0,99,117,98,105,99,0,0,0,112,95,116,101,120,95,98,105,116,115,0,0,0,0,0,0,37,100,46,37,100,46,37,100,0,0,0,0,0,0,0,0,108,111,99,111,0,0,0,0,74,80,111,112,0,0,0,0,121,117,118,106,52,52,52,112,0,0,0,0,0,0,0,0,112,114,111,98,101,100,32,115,116,114,101,97,109,32,37,100,32,102,97,105,108,101,100,10,0,0,0,0,0,0,0,0,37,77,58,37,83,0,0,0,80,114,101,115,115,32,91,113,93,32,116,111,32,115,116,111,112,44,32,91,63,93,32,102,111,114,32,104,101,108,112,10,0,0,0,0,0,0,0,0,99,111,112,121,32,116,105,109,101,115,116,97,109,112,115,0,37,100,0,0,0,0,0,0,69,110,99,111,100,101,114,115,0,0,0,0,0,0,0,0,115,116,97,116,115,0,0,0,101,120,112,0,0,0,0,0,69,79,70,0,0,0,0,0,84,101,97,108,0,0,0,0,98,97,99,107,32,108,101,102,116,0,0,0,0,0,0,0,115,101,108,101,99,116,32,115,119,114,32,102,105,108,116,101,114,32,116,121,112,101,0,0,108,97,115,116,73,110,76,117,109,66,117,102,32,43,32,49,32,45,32,115,114,99,83,108,105,99,101,89,32,62,61,32,48,0,0,0,0,0,0,0,105,95,116,101,120,95,98,105,116,115,0,0,0,0,0,0,117,115,105,110,103,32,117,110,115,99,97,108,101,100,32,37,115,32,45,62,32,37,115,32,115,112,101,99,105,97,108,32,99,111,110,118,101,114,116,101,114,10,0,0,0,0,0,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,86,105,100,101,111,32,57,0,0,0,65,110,105,109,101,0,0,0,97,102,111,114,109,97,116,0,103,97,117,115,115,0,0,0,112,114,111,98,101,100,32,115,116,114,101,97,109,32,37,100,10,0,0,0,0,0,0,0,80,67,77,32,115,105,103,110,101,100,32,49,54,45,98,105,116,32,108,105,116,116,108,101,45,101,110,100,105,97,110,0,50,48,48,54,47,48,52,47,48,49,32,49,49,58,49,49,58,49,49,0,0,0,0,0,73,110,112,117,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,48,120,37,108,108,120,32,105,115,32,105,110,118,97,108,105,100,32,111,114,32,117,110,115,117,112,112,111,114,116,101,100,46,10,0,99,111,112,121,116,115,0,0,37,115,58,10,32,86,46,46,46,46,46,32,61,32,86,105,100,101,111,10,32,65,46,46,46,46,46,32,61,32,65,117,100,105,111,10,32,83,46,46,46,46,46,32,61,32,83,117,98,116,105,116,108,101,10,32,46,70,46,46,46,46,32,61,32,70,114,97,109,101,45,108,101,118,101,108,32,109,117,108,116,105,116,104,114,101,97,100,105,110,103,10,32,46,46,83,46,46,46,32,61,32,83,108,105,99,101,45,108,101,118,101,108,32,109,117,108,116,105,116,104,114,101,97,100,105,110,103,10,32,46,46,46,88,46,46,32,61,32,67,111,100,101,99,32,105,115,32,101,120,112,101,114,105,109,101,110,116,97,108,10,32,46,46,46,46,66,46,32,61,32,83,117,112,112,111,114,116,115,32,100,114,97,119,95,104,111,114,105,122,95,98,97,110,100,10,32,46,46,46,46,46,68,32,61,32,83,117,112,112,111,114,116,115,32,100,105,114,101,99,116,32,114,101,110,100,101,114,105,110,103,32,109,101,116,104,111,100,32,49,10,32,45,45,45,45,45,45,10,0,0,0,0,0,0,0,83,66,82,32,69,120,116,101,110,115,105,111,110,32,111,118,101,114,32,114,101,97,100,46,10,0,0,0,0,0,0,0,83,66,82,32,115,105,103,110,97,108,101,100,32,116,111,32,98,101,32,110,111,116,45,112,114,101,115,101,110,116,32,98,117,116,32,119,97,115,32,102,111,117,110,100,32,105,110,32,116,104,101,32,98,105,116,115,116,114,101,97,109,46,10,0,98,108,111,99,107,95,97,108,105,103,110,32,105,115,32,110,111,116,32,115,101,116,10,0,73,110,118,97,108,105,100,32,102,114,97,109,105,110,103,32,98,105,116,32,105,110,32,73,100,32,104,101,97,100,101,114,10,0,0,0,0,0,0,0,102,102,109,112,101,103,50,112,97,115,115,0,0,0,0,0,84,97,110,0,0,0,0,0,100,101,99,111,100,101,114,0,102,105,108,116,101,114,95,116,121,112,101,0,0,0,0,0,104,101,97,100,101,114,95,98,105,116,115,0,0,0,0,0,119,109,118,51,0,0,0,0,84,104,114,97,115,104,32,77,101,116,97,108,0,0,0,0,110,111,116,104,105,110,103,32,116,111,32,112,114,111,98,101,32,102,111,114,32,115,116,114,101,97,109,32,37,100,10,0,76,97,118,99,53,53,46,49,56,46,49,48,50,0,0,0,110,117,108,108,0,0,0,0,116,105,109,101,95,98,97,115,101,61,37,100,47,37,100,58,115,97,109,112,108,101,95,114,97,116,101,61,37,100,58,115,97,109,112,108,101,95,102,109,116,61,37,115,0,0,0,0,116,104,114,101,115,104,111,108,100,0,0,0,0,0,0,0,98,101,115,116,95,101,102,102,111,114,116,95,116,105,109,101,115,116,97,109,112,0,0,0,105,110,118,97,108,105,100,32,98,108,111,99,107,32,116,121,112,101,10,0,0,0,0,0,32,32,32,32,83,117,112,112,111,114,116,101,100,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,115,58,0,0,37,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,98,121,32,86,111,114,98,105,115,58,32,111,117,116,112,117,116,32,115,116,114,101,97,109,32,119,105,108,108,32,104,97,118,101,32,105,110,99,111,114,114,101,99,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,46,10,0,0,69,110,100,32,111,102,32,102,105,108,101,0,0,0,0,0,37,115,45,37,100,46,108,111,103,0,0,0,0,0,0,0,108,101,0,0,0,0,0,0,83,116,101,101,108,66,108,117,101,0,0,0,0,0,0,0,115,101,108,101,99,116,32,68,111,108,98,121,32,80,114,111,32,76,111,103,105,99,32,73,73,0,0,0,0,0,0,0,109,118,95,98,105,116,115,0,83,77,80,84,69,32,86,67,45,49,0,0,0,0,0,0,83,97,108,115,97,0,0,0,112,114,111,98,105,110,103,32,115,116,114,101,97,109,32,37,100,32,112,112,58,37,100,10,0,0,0,0,0,0,0,0,109,106,112,101,103,0,0,0,102,111,114,109,97,116,0,0,97,117,100,105,111,32,100,114,105,102,116,32,116,104,114,101,115,104,111,108,100,0,0,0,82,38,66,0,0,0,0,0,32,32,32,32,83,117,112,112,111,114,116,101,100,32,115,97,109,112,108,101,32,102,111,114,109,97,116,115,58,0,0,0,87,77,47,71,101,110,114,101,0,0,0,0,0,0,0,0,101,110,99,111,100,101,100,95,98,121,0,0,0,0,0,0,73,110,118,97,108,105,100,32,102,111,114,99,101,95,107,101,121,95,102,114,97,109,101,115,32,101,120,112,114,101,115,115,105,111,110,32,39,37,115,39,10,0,0,0,0,0,0,0,37,115,37,115,0,0,0,0,83,112,114,105,110,103,71,114,101,101,110,0,0,0,0,0,100,101,109,117,120,101,114,32,100,111,101,115,32,110,111,116,32,115,117,112,112,111,114,116,32,97,100,100,105,116,105,111,110,97,108,32,105,100,51,32,100,97,116,97,44,32,115,107,105,112,112,105,110,103,10,0,100,112,108,105,105,0,0,0,73,80,82,68,0,0,0,0,82,84,80,32,112,97,121,108,111,97,100,32,115,105,122,101,32,105,110,32,98,121,116,101,115,0,0,0,0,0,0,0,100,111,32,110,111,116,32,102,105,108,108,32,105,110,32,109,105,115,115,105,110,103,32,118,97,108,117,101,115,32,116,104,97,116,32,99,97,110,32,98,101,32,101,120,97,99,116,108,121,32,99,97,108,99,117,108,97,116,101,100,0,0,0,0,118,99,49,0,0,0,0,0,77,101,114,101,110,103,117,101,0,0,0,0,0,0,0,0,116,104,101,111,114,97,32,107,102,103,115,104,105,102,116,32,37,100,44,32,118,114,101,118,32,37,100,10,0,0,0,0,115,116,114,101,97,109,95,105,110,100,101,120,32,60,32,111,103,103,45,62,110,115,116,114,101,97,109,115,0,0,0,0,42,112,107,116,95,98,117,102,102,101,114,0,0,0,0,0,84,97,103,32,37,115,47,48,120,37,48,56,120,32,105,110,99,111,109,112,97,116,105,98,108,101,32,119,105,116,104,32,111,117,116,112,117,116,32,99,111,100,101,99,32,105,100,32,39,37,100,39,32,40,37,115,41,10,0,0,0,0,0,0,76,97,118,102,53,53,46,49,50,46,49,48,49,0,0,0,97,100,114,105,102,116,95,116,104,114,101,115,104,111,108,100,0,0,0,0,0,0,0,0,65,100,100,32,82,84,80,32,104,105,110,116,32,116,114,97,99,107,115,0,0,0,0,0,37,100,0,0,0,0,0,0,67,97,110,110,111,116,32,97,112,112,101,110,100,32,112,97,108,101,116,116,101,32,116,111,32,112,97,99,107,101,116,10,0,0,0,0,0,0,0,0,84,80,69,49,0,0,0,0,101,120,112,114,58,0,0,0,98,103,114,51,50,0,0,0,83,110,111,119,0,0,0,0,112,97,99,107,101,116,95,104,100,114,95,115,105,122,101,32,60,61,32,97,115,102,45,62,112,97,99,107,101,116,95,115,105,122,101,95,108,101,102,116,0,0,0,0,0,0,0,0,105,110,118,97,108,105,100,32,112,97,99,107,101,116,32,104,101,97,100,101,114,32,108,101,110,103,116,104,32,37,100,32,102,111,114,32,112,107,116,108,101,110,32,37,100,45,37,100,32,97,116,32,37,108,108,100,10,0,0,0,0,0,0,0,115,101,108,101,99,116,32,68,111,108,98,121,0,0,0,0,111,117,116,108,105,110,107,45,62,102,111,114,109,97,116,32,61,61,32,111,117,116,95,102,111,114,109,97,116,0,0,0,65,32,99,111,109,109,97,45,115,101,112,97,114,97,116,101,100,32,108,105,115,116,32,111,102,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,115,46,0,0,0,0,0,0,78,117,109,98,101,114,32,111,102,32,116,104,101,32,102,105,114,115,116,32,97,117,100,105,111,32,115,97,109,112,108,101,32,116,104,97,116,32,115,104,111,117,108,100,32,98,101,32,112,97,115,115,101,100,32,116,111,32,116,104,101,32,111,117,116,112,117,116,0,0,0,0,112,115,0,0,0,0,0,0,83,65,77,80,76,69,95,82,65,84,69,0,0,0,0,0,78,111,32,115,117,99,104,32,102,105,108,116,101,114,58,32,39,37,115,39,10,0,0,0,82,101,97,108,86,105,100,101,111,32,52,46,48,0,0,0,67,104,114,105,115,116,105,97,110,32,82,111,99,107,0,0,37,108,108,100,0,0,0,0,99,104,97,110,110,101,108,115,0,0,0,0,0,0,0,0,101,110,103,0,0,0,0,0,99,104,97,110,110,101,108,95,108,97,121,111,117,116,115,0,97,117,116,111,45,105,110,115,101,114,116,101,100,32,115,99,97,108,101,114,32,37,100,0,112,105,110,103,0,0,0,0,97,117,100,105,111,32,115,121,110,99,32,109,101,116,104,111,100,0,0,0,0,0,0,0,32,32,32,32,83,117,112,112,111,114,116,101,100,32,115,97,109,112,108,101,32,114,97,116,101,115,58,0,0,0,0,0,78,111,32,112,105,120,101,108,32,102,111,114,109,97,116,32,115,112,101,99,105,102,105,101,100,44,32,37,115,32,102,111,114,32,72,46,50,54,52,32,101,110,99,111,100,105,110,103,32,99,104,111,115,101,110,46,10,85,115,101,32,45,112,105,120,95,102,109,116,32,121,117,118,52,50,48,112,32,102,111,114,32,99,111,109,112,97,116,105,98,105,108,105,116,121,32,119,105,116,104,32,111,117,116,100,97,116,101,100,32,109,101,100,105,97,32,112,108,97,121,101,114,115,46,10,0,0,0,114,103,98,51,50,0,0,0,83,108,97,116,101,71,114,97,121,0,0,0,0,0,0,0,100,111,108,98,121,0,0,0,115,116,114,97,116,101,103,121,32,116,111,32,99,104,111,111,115,101,32,98,101,116,119,101,101,110,32,73,47,80,47,66,45,102,114,97,109,101,115,0,114,118,52,48,0,0,0,0,67,111,110,116,101,109,112,111,114,97,114,121,32,67,104,114,105,115,116,105,97,110,0,0,78,79,80,84,83,0,0,0,37,115,45,0,0,0,0,0,116,114,105,109,0,0,0,0,97,115,121,110,99,0,0,0,10,0,0,0,0,0,0,0,32,37,115,0,0,0,0,0,67,111,110,118,101,114,115,105,111,110,32,119,105,108,108,32,110,111,116,32,98,101,32,108,111,115,115,108,101,115,115,46,10,0,0,0,0,0,0,0,108,105,98,120,50,54,52,0,121,117,118,106,52,49,49,112,0,0,0,0,0,0,0,0,83,108,97,116,101,66,108,117,101,0,0,0,0,0,0,0,115,101,108,101,99,116,32,110,111,110,101,0,0,0,0,0,114,101,116,32,62,61,32,48,0,0,0,0,0,0,0,0,98,95,115,116,114,97,116,101,103,121,0,0,0,0,0,0,82,101,97,108,86,105,100,101,111,32,51,46,48,0,0,0,67,114,111,115,115,111,118,101,114,0,0,0,0,0,0,0,102,105,114,115,116,95,100,116,115,32,37,115,32,98,117,116,32,110,111,32,112,97,99,107,101,116,32,119,105,116,104,32,100,116,115,32,105,110,32,116,104,101,32,113,117,101,117,101,10,0,0,0,0,0,0,0,77,80,51,32,40,77,80,69,71,32,97,117,100,105,111,32,108,97,121,101,114,32,51,41,0,0,0,0,0,0,0,0,118,105,100,101,111,32,115,121,110,99,32,109,101,116,104,111,100,0,0,0,0,0,0,0,32,32,32,32,83,117,112,112,111,114,116,101,100,32,112,105,120,101,108,32,102,111,114,109,97,116,115,58,0,0,0,0,70,114,97,109,101,32,114,97,116,101,32,118,101,114,121,32,104,105,103,104,32,102,111,114,32,97,32,109,117,120,101,114,32,110,111,116,32,101,102,102,105,99,105,101,110,116,108,121,32,115,117,112,112,111,114,116,105,110,103,32,105,116,46,10,80,108,101,97,115,101,32,99,111,110,115,105,100,101,114,32,115,112,101,99,105,102,121,105,110,103,32,97,32,108,111,119,101,114,32,102,114,97,109,101,114,97,116,101,44,32,97,32,100,105,102,102,101,114,101,110,116,32,109,117,120,101,114,32,111,114,32,45,118,115,121,110,99,32,50,10,0,0,0,0,101,114,114,111,114,32,105,110,32,118,111,114,98,105,115,95,97,110,97,108,121,115,105,115,95,119,114,111,116,101,40,41,10,0,0,0,0,0,0,0,103,98,114,97,112,49,54,108,101,0,0,0,0,0,0,0,118,111,114,98,105,115,32,101,120,116,114,97,100,97,116,97,32,97,98,115,101,110,116,10,0,0,0,0,0,0,0,0,83,107,121,66,108,117,101,0,110,111,110,101,0,0,0,0,108,105,98,109,112,51,108,97,109,101,32,77,80,51,32,40,77,80,69,71,32,97,117,100,105,111,32,108,97,121,101,114,32,51,41,0,0,0,0,0,114,97,116,101,99,111,110,116,114,111,108,32,109,101,116,104,111,100,0,0,0,0,0,0,114,118,51,48,0,0,0,0,66,108,97,99,107,32,77,101,116,97,108,0,0,0,0,0,102,105,114,115,116,95,100,116,115,32,37,115,32,110,111,116,32,109,97,116,99,104,105,110,103,32,102,105,114,115,116,32,100,116,115,32,37,115,32,105,110,32,116,104,101,32,113,117,101,117,101,10,0,0,0,0,118,115,121,110,99,0,0,0,32,37,100,47,37,100,0,0,100,101,102,97,117,108,116,0,70,114,97,117,110,104,111,102,101,114,32,70,68,75,32,65,65,67,0,0,0,0,0,0,100,101,115,99,0,0,0,0,69,114,114,111,114,32,111,112,101,110,105,110,103,32,102,105,108,116,101,114,115,33,10,0,103,98,114,97,112,49,54,98,101,0,0,0,0,0,0,0,83,105,108,118,101,114,0,0,115,101,116,32,109,97,116,114,105,120,101,100,32,115,116,101,114,101,111,32,101,110,99,111,100,105,110,103,0,0,0,0,114,99,95,115,116,114,97,116,101,103,121,0,0,0,0,0,83,116,114,101,97,109,32,109,97,121,32,104,97,118,101,32,117,110,104,97,110,100,108,101,100,32,102,101,97,116,117,114,101,115,10,0,0,0,0,0,72,117,102,102,121,117,118,32,70,70,109,112,101,103,32,118,97,114,105,97,110,116,0,0,72,101,97,118,121,32,77,101,116,97,108,0,0,0,0,0,109,111,118,44,109,112,52,44,109,52,97,44,51,103,112,44,51,103,50,44,109,106,50,0,116,121,112,101,0,0,0,0,32,32,32,32,83,117,112,112,111,114,116,101,100,32,102,114,97,109,101,114,97,116,101,115,58,0,0,0,0,0,0,0,69,110,99,111,100,101,114,32,40,99,111,100,101,99,32,37,115,41,32,110,111,116,32,102,111,117,110,100,32,102,111,114,32,111,117,116,112,117,116,32,115,116,114,101,97,109,32,35,37,100,58,37,100,0,0,0,103,98,114,97,112,0,0,0,83,105,101,110,110,97,0,0,109,97,116,114,105,120,95,101,110,99,111,100,105,110,103,0,81,80,32,102,97,99,116,111,114,32,98,101,116,119,101,101,110,32,80,45,32,97,110,100,32,66,45,102,114,97,109,101,115,0,0,0,0,0,0,0,102,102,118,104,117,102,102,0,67,104,114,105,115,116,105,97,110,32,71,97,110,103,115,116,97,0,0,0,0,0,0,0,121,117,118,106,52,50,50,112,0,0,0,0,0,0,0,0,105,110,118,97,108,105,100,32,100,116,115,47,112,116,115,32,99,111,109,98,105,110,97,116,105,111,110,32,37,108,108,105,10,0,0,0,0,0,0,0,117,110,100,0,0,0,0,0,37,74,58,37,77,58,37,83,0,0,0,0,0,0,0,0,115,101,116,112,116,115,0,0,115,112,101,99,105,102,121,32,116,97,114,103,101,116,32,102,105,108,101,32,116,121,112,101,32,40,34,118,99,100,34,44,32,34,115,118,99,100,34,44,32,34,100,118,100,34,44,32,34,100,118,34,44,32,34,100,118,53,48,34,44,32,34,112,97,108,45,118,99,100,34,44,32,34,110,116,115,99,45,115,118,99,100,34,44,32,46,46,46,41,0,0,0,0,0,0,48,120,37,48,56,88,0,0,37,108,108,117,32,102,114,97,109,101,115,32,115,117,99,99,101,115,115,102,117,108,108,121,32,100,101,99,111,100,101,100,44,32,37,108,108,117,32,100,101,99,111,100,105,110,103,32,101,114,114,111,114,115,10,0,110,111,0,0,0,0,0,0,91,37,115,32,64,32,37,112,93,32,0,0,0,0,0,0,79,118,101,114,114,105,100,105,110,103,32,97,115,112,101,99,116,32,114,97,116,105,111,32,119,105,116,104,32,115,116,114,101,97,109,32,99,111,112,121,32,109,97,121,32,112,114,111,100,117,99,101,32,105,110,118,97,108,105,100,32,102,105,108,101,115,10,0,0,0,0,0,97,99,111,115,0,0,0,0,69,110,99,111,100,101,114,32,110,111,116,32,102,111,117,110,100,0,0,0,0,0,0,0,103,98,114,112,49,52,108,101,0,0,0,0,0,0,0,0,83,101,97,83,104,101,108,108,0,0,0,0,0,0,0,0,66,76,0,0,0,0,0,0,65,115,115,117,109,101,32,116,104,101,32,102,105,114,115,116,32,112,116,115,32,115,104,111,117,108,100,32,98,101,32,116,104,105,115,32,118,97,108,117,101,32,40,105,110,32,115,97,109,112,108,101,115,41,46,0,108,97,115,116,73,110,76,117,109,66,117,102,32,43,32,49,32,45,32,115,114,99,83,108,105,99,101,89,32,60,32,115,114,99,83,108,105,99,101,72,0,0,0,0,0,0,0,0,98,95,113,102,97,99,116,111,114,0,0,0,0,0,0,0,67,97,110,110,111,116,32,97,108,108,111,99,97,116,101,32,109,101,109,111,114,121,46,10,0,0,0,0,0,0,0,0,80,65,77,32,40,80,111,114,116,97,98,108,101,32,65,110,121,77,97,112,41,32,105,109,97,103,101,0,0,0,0,0,66,101,97,116,0,0,0,0,108,117,109,97,32,98,105,99,117,98,105,99,44,32,99,104,114,111,109,97,32,98,105,108,105,110,101,97,114,0,0,0,114,101,97,100,95,102,114,97,109,101,95,105,110,116,101,114,110,97,108,32,115,116,114,101,97,109,61,37,100,44,32,112,116,115,61,37,115,44,32,100,116,115,61,37,115,44,32,115,105,122,101,61,37,100,44,32,100,117,114,97,116,105,111,110,61,37,100,44,32,102,108,97,103,115,61,37,100,10,0,0,82,101,113,117,101,115,116,101,100,32,111,117,116,112,117,116,32,115,97,109,112,108,101,32,102,111,114,109,97,116,32,37,100,32,105,115,32,105,110,118,97,108,105,100,10,0,0,0,116,97,114,103,101,116,0,0,115,108,105,99,101,0,0,0,73,110,118,97,108,105,100,32,98,105,116,115,116,114,101,97,109,32,45,32,99,97,110,110,111,116,32,97,112,112,108,121,32,83,66,82,32,116,111,32,101,108,101,109,101,110,116,32,116,121,112,101,32,37,100,10,0,0,0,0,0,0,0,0,83,66,82,32,119,97,115,32,102,111,117,110,100,32,98,101,102,111,114,101,32,116,104,101,32,102,105,114,115,116,32,99,104,97,110,110,101,108,32,101,108,101,109,101,110,116,46,10,0,0,0,0,0,0,0,0,69,120,112,111,110,101,110,116,32,111,117,116,32,111,102,32,114,97,110,103,101,58,32,37,100,10,0,0,0,0,0,0,73,110,118,97,108,105,100,32,112,97,99,107,101,116,32,115,105,103,110,97,116,117,114,101,32,105,110,32,73,100,32,104,101,97,100,101,114,10,0,0,45,97,99,111,100,101,99,32,99,111,112,121,32,97,110,100,32,45,118,111,108,32,97,114,101,32,105,110,99,111,109,112,97,116,105,98,108,101,32,40,102,114,97,109,101,115,32,97,114,101,32,110,111,116,32,100,101,99,111,100,101,100,41,10,0,0,0,0,0,0,0,0,103,98,114,112,49,52,98,101,0,0,0,0,0,0,0,0,83,101,97,71,114,101,101,110,0,0,0,0,0,0,0,0,101,110,99,111,100,101,114,0,102,105,114,115,116,95,112,116,115,0,0,0,0,0,0,0,117,115,101,32,39,102,114,97,109,101,115,39,32,66,32,102,114,97,109,101,115,0,0,0,112,97,109,0,0,0,0,0,80,111,108,115,107,32,80,117,110,107,0,0,0,0,0,0,102,102,95,114,101,97,100,95,112,97,99,107,101,116,32,115,116,114,101,97,109,61,37,100,44,32,112,116,115,61,37,115,44,32,100,116,115,61,37,115,44,32,115,105,122,101,61,37,100,44,32,100,117,114,97,116,105,111,110,61,37,100,44,32,102,108,97,103,115,61,37,100,10,0,0,0,0,0,0,0,116,114,97,99,107,0,0,0,97,98,117,102,102,101,114,0,65,86,70,114,97,109,101,0,98,105,103,95,118,97,108,117,101,115,32,116,111,111,32,98,105,103,10,0,0,0,0,0,102,114,97,109,101,0,0,0,105,110,118,97,108,105,100,32,101,120,116,114,97,100,97,116,97,10,0,0,0,0,0,0,85,115,101,32,106,111,105,110,116,32,115,116,101,114,101,111,46,0,0,0,0,0,0,0,69,110,99,111,100,105,110,103,32,101,114,114,111,114,0,0,103,98,114,112,49,50,108,101,0,0,0,0,0,0,0,0,83,97,110,100,121,66,114,111,119,110,0,0,0,0,0,0,115,105,109,112,108,105,102,105,101,100,32,49,32,112,97,114,97,109,101,116,101,114,32,97,117,100,105,111,32,116,105,109,101,115,116,97,109,112,32,109,97,116,99,104,105,110,103,44,32,48,40,100,105,115,97,98,108,101,100,41,44,32,49,40,102,105,108,108,105,110,103,32,97,110,100,32,116,114,105,109,109,105,110,103,41,44,32,62,49,40,109,97,120,105,109,117,109,32,115,116,114,101,116,99,104,47,115,113,117,101,101,122,101,32,105,110,32,115,97,109,112,108,101,115,32,112,101,114,32,115,101,99,111,110,100,41,0,0,0,0,0,0,0,0,98,102,0,0,0,0,0,0,80,71,77,89,85,86,32,40,80,111,114,116,97,98,108,101,32,71,114,97,121,77,97,112,32,89,85,86,41,32,105,109,97,103,101,0,0,0,0,0,78,101,103,101,114,112,117,110,107,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,116,105,109,101,115,116,97,109,112,115,32,115,116,114,101,97,109,61,37,100,44,32,112,116,115,61,37,115,44,32,100,116,115,61,37,115,44,32,115,105,122,101,61,37,100,10,0,0,82,101,97,108,86,105,100,101,111,32,50,46,48,0,0,0,32,10,9,0,0,0,0,0,114,101,97,100,32,105,110,112,117,116,32,97,116,32,110,97,116,105,118,101,32,102,114,97,109,101,32,114,97,116,101,0,80,111,112,0,0,0,0,0,102,114,97,109,101,32,97,110,100,32,115,108,105,99,101,0,101,110,99,111,100,101,114,0,111,114,105,103,105,110,97,116,111,114,0,0,0,0,0,0,102,52,118,0,0,0,0,0,103,98,114,112,49,50,98,101,0,0,0,0,0,0,0,0,83,97,108,109,111,110,0,0,116,116,97,0,0,0,0,0,97,115,121,110,99,0,0,0,116,105,116,108,101,0,0,0,109,97,120,105,109,117,109,32,100,105,102,102,101,114,101,110,99,101,32,98,101,116,119,101,101,110,32,116,104,101,32,113,117,97,110,116,105,122,101,114,32,115,99,97,108,101,115,32,40,86,66,82,41,0,0,0,110,111,102,105,108,108,105,110,0,0,0,0,0,0,0,0,112,103,109,121,117,118,0,0,66,114,105,116,80,111,112,0,69,120,116,114,97,100,97,116,97,32,99,111,114,114,117,112,116,101,100,10,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,115,101,101,107,95,102,114,97,109,101,95,103,101,110,101,114,105,99,32,102,97,105,108,101,100,32,97,115,32,116,104,105,115,32,115,116,114,101,97,109,32,115,101,101,109,115,32,116,111,32,99,111,110,116,97,105,110,32,110,111,32,107,101,121,102,114,97,109,101,115,32,97,102,116,101,114,32,116,104,101,32,116,97,114,103,101,116,32,116,105,109,101,115,116,97,109,112,44,32,37,100,32,110,111,110,32,107,101,121,102,114,97,109,101,115,32,102,111,117,110,100,10,0,0,0,0,0,0,65,115,112,101,99,116,32,114,97,116,105,111,32,109,105,115,109,97,116,99,104,32,98,101,116,119,101,101,110,32,109,117,120,101,114,32,40,37,100,47,37,100,41,32,97,110,100,32,101,110,99,111,100,101,114,32,108,97,121,101,114,32,40,37,100,47,37,100,41,10,0,0,76,97,118,102,0,0,0,0,116,109,112,111,0,0,0,0,114,101,0,0,0,0,0,0,114,116,112,104,105,110,116,0,32,32,32,32,84,104,114,101,97,100,105,110,103,32,99,97,112,97,98,105,108,105,116,105,101,115,58,32,0,0,0,0,115,116,114,101,97,109,32,37,100,44,32,111,102,102,115,101,116,32,48,120,37,108,108,120,58,32,112,97,114,116,105,97,108,32,102,105,108,101,10,0,108,97,110,103,117,97,103,101,0,0,0,0,0,0,0,0,105,112,111,100,0,0,0,0,77,80,69,71,45,49,32,118,105,100,101,111,0,0,0,0,121,117,118,52,52,52,112,49,52,108,101,0,0,0,0,0,83,97,100,100,108,101,66,114,111,119,110,0,0,0,0,0,108,105,98,97,118,102,111,114,109,97,116,47,97,115,102,101,110,99,46,99,0,0,0,0,105,110,118,97,108,105,100,32,112,97,100,115,105,122,101,32,37,100,32,97,116,58,37,108,108,100,10,0,0,0,0,0,115,101,116,32,109,97,120,105,109,117,109,32,102,97,99,116,111,114,32,98,121,32,119,104,105,99,104,32,100,97,116,97,32,105,115,32,115,116,114,101,116,99,104,101,100,47,115,113,117,101,101,122,101,100,32,116,111,32,109,97,107,101,32,105,116,32,109,97,116,99,104,32,116,104,101,32,116,105,109,101,115,116,97,109,112,115,46,0,111,117,116,108,105,110,107,45,62,99,104,97,110,110,101,108,95,108,97,121,111,117,116,32,61,61,32,111,117,116,95,108,97,121,111,117,116,32,124,124,32,33,111,117,116,108,105,110,107,45,62,99,104,97,110,110,101,108,95,108,97,121,111,117,116,0,0,0,0,0,0,0,99,104,97,110,110,101,108,95,108,97,121,111,117,116,115,0,115,116,97,114,116,95,115,97,109,112,108,101,0,0,0,0,113,100,105,102,102,0,0,0,80,84,83,0,0,0,0,0,40,117,110,115,105,103,110,101,100,41,112,107,116,45,62,115,105,122,101,32,60,61,32,48,120,55,102,102,102,102,102,102,102,32,45,32,49,54,0,0,80,97,114,115,101,100,95,37,115,95,37,100,0,0,0,0,80,71,77,32,40,80,111,114,116,97,98,108,101,32,71,114,97,121,77,97,112,41,32,105,109,97,103,101,0,0,0,0,73,110,100,105,101,0,0,0,115,116,45,62,105,110,100,101,120,95,101,110,116,114,105,101,115,0,0,0,0,0,0,0,114,116,112,102,108,97,103,115,0,0,0,0,0,0,0,0,99,104,97,110,110,101,108,95,108,97,121,111,117,116,0,0,103,97,112,108,101,115,115,95,112,108,97,121,98,97,99,107,0,0,0,0,0,0,0,0,115,101,116,32,116,104,101,32,115,117,112,112,111,114,116,101,100,32,115,97,109,112,108,101,32,114,97,116,101,115,0,0,39,115,99,97,108,101,39,32,102,105,108,116,101,114,32,110,111,116,32,112,114,101,115,101,110,116,44,32,99,97,110,110,111,116,32,99,111,110,118,101,114,116,32,112,105,120,101,108,32,102,111,114,109,97,116,115,46,10,0,0,0,0,0,0,99,111,100,101,99,32,111,112,101,110,32,105,110,32,114,101,97,100,95,100,101,99,95,99,111,110,102,105,103,95,100,101,115,99,114,10,0,0,0,0,33,108,105,110,107,45,62,102,114,97,109,101,95,114,101,113,117,101,115,116,101,100,32,124,124,32,108,105,110,107,45,62,102,108,97,103,115,32,38,32,70,70,95,76,73,78,75,95,70,76,65,71,95,82,69,81,85,69,83,84,95,76,79,79,80,0,0,0,0,0,0,0,119,104,101,110,32,100,117,109,112,105,110,103,32,112,97,99,107,101,116,115,44,32,97,108,115,111,32,100,117,109,112,32,116,104,101,32,112,97,121,108,111,97,100,0,0,0,0,0,68,101,99,111,100,101,114,0,78,111,32,109,105,109,101,116,121,112,101,32,105,115,32,107,110,111,119,110,32,102,111,114,32,115,116,114,101,97,109,32,37,100,44,32,99,97,110,110,111,116,32,119,114,105,116,101,32,97,110,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,46,10,0,0,112,115,112,0,0,0,0,0,121,117,118,52,52,52,112,49,52,98,101,0,0,0,0,0,82,111,121,97,108,66,108,117,101,0,0,0,0,0,0,0,109,97,120,95,115,111,102,116,95,99,111,109,112,0,0,0,109,97,120,105,109,117,109,32,118,105,100,101,111,32,113,117,97,110,116,105,122,101,114,32,115,99,97,108,101,32,40,86,66,82,41,0,0,0,0,0,112,103,109,0,0,0,0,0,84,101,114,114,111,114,0,0,117,110,107,110,111,119,110,32,99,111,100,101,99,0,0,0,112,103,97,112,0,0,0,0,104,101,120,0,0,0,0,0,83,117,98,116,105,116,108,101,32,111,112,116,105,111,110,115,58,0,0,0,0,0,0,0,69,110,99,111,100,101,114,0,51,103,50,0,0,0,0,0,121,117,118,52,52,52,112,49,50,108,101,0,0,0,0,0,82,111,115,121,66,114,111,119,110,0,0,0,0,0,0,0,115,101,116,32,100,117,114,97,116,105,111,110,32,40,105,110,32,115,101,99,111,110,100,115,41,32,111,118,101,114,32,119,104,105,99,104,32,100,97,116,97,32,105,115,32,115,116,114,101,116,99,104,101,100,47,115,113,117,101,101,122,101,100,32,116,111,32,109,97,107,101,32,105,116,32,109,97,116,99,104,32,116,104,101,32,116,105,109,101,115,116,97,109,112,115,46,0,0,0,0,0,0,0,0,113,109,97,120,0,0,0,0,80,66,77,32,40,80,111,114,116,97,98,108,101,32,66,105,116,77,97,112,41,32,105,109,97,103,101,0,0,0,0,0,72,97,114,100,99,111,114,101,0,0,0,0,0,0,0,0,110,111,32,102,114,97,109,101,32,105,110,32,114,118,51,48,47,52,48,32,97,110,100,32,110,111,32,115,97,114,0,0,67,108,97,115,115,105,99,32,82,111,99,107,0,0,0,0,104,100,95,118,105,100,101,111,0,0,0,0,0,0,0,0,67,111,117,108,100,32,110,111,116,32,114,101,97,100,32,109,105,109,101,116,121,112,101,32,102,114,111,109,32,97,110,32,97,116,116,97,99,104,101,100,32,112,105,99,116,117,114,101,46,10,0,0,0,0,0,0,68,117,112,108,105,99,97,116,101,32,102,111,114,109,97,116,115,32,105,110,32,97,118,102,105,108,116,101,114,95,109,101,114,103,101,95,102,111,114,109,97,116,115,40,41,32,100,101,116,101,99,116,101,100,10,0,100,117,109,112,32,101,97,99,104,32,105,110,112,117,116,32,112,97,99,107,101,116,0,0,37,115,32,37,115,32,91,37,115,93,58,10,0,0,0,0,51,103,112,0,0,0,0,0,97,108,98,117,109,95,97,114,116,105,115,116,0,0,0,0,121,117,118,52,52,52,112,49,50,98,101,0,0,0,0,0,82,101,100,0,0,0,0,0,99,111,109,112,95,100,117,114,97,116,105,111,110,0,0,0,109,105,110,105,109,117,109,32,118,105,100,101,111,32,113,117,97,110,116,105,122,101,114,32,115,99,97,108,101,32,40,86,66,82,41,0,0,0,0,0,112,98,109,0,0,0,0,0,67,108,117,98,45,72,111,117,115,101,0,0,0,0,0,0,117,110,115,112,101,99,105,102,105,101,100,32,112,105,120,101,108,32,102,111,114,109,97,116,0,0,0,0,0,0,0,0,104,100,118,100,0,0,0,0,195,138,204,132,195,138,204,140,195,170,204,132,195,170,204,140,0,0,0,0,0,0,0,0,100,117,109,112,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,116,97,103,32,118,101,114,115,105,111,110,46,32,40,62,61,37,100,41,10,0,0,0,0,0,0,0,0,114,97,119,32,65,68,84,83,32,65,65,67,32,40,65,100,118,97,110,99,101,100,32,65,117,100,105,111,32,67,111,100,105,110,103,41,0,0,0,0,67,111,100,101,99,32,39,37,115,39,32,105,115,32,110,111,116,32,114,101,99,111,103,110,105,122,101,100,32,98,121,32,70,70,109,112,101,103,46,10,0,0,0,0,0,0,0,0,49,0,0,0,0,0,0,0,109,112,52,0,0,0,0,0,121,117,118,52,50,50,112,49,52,108,101,0,0,0,0,0,80,117,114,112,108,101,0,0,87,65,86,32,47,32,87,65,86,69,32,40,87,97,118,101,102,111,114,109,32,65,117,100,105,111,41,0,0,0,0,0,115,101,116,32,109,105,110,105,109,117,109,32,100,105,102,102,101,114,101,110,99,101,32,98,101,116,119,101,101,110,32,116,105,109,101,115,116,97,109,112,115,32,97,110,100,32,97,117,100,105,111,32,100,97,116,97,32,40,105,110,32,115,101,99,111,110,100,115,41,32,116,111,32,116,114,105,103,103,101,114,32,112,97,100,100,105,110,103,47,116,114,105,109,109,105,110,103,32,116,104,101,32,100,97,116,97,46,0,0,0,0,0,97,108,98,117,109,95,97,114,116,105,115,116,0,0,0,0,113,109,105,110,0,0,0,0,102,105,102,111,0,0,0,0,80,80,77,32,40,80,111,114,116,97,98,108,101,32,80,105,120,101,108,77,97,112,41,32,105,109,97,103,101,0,0,0,68,114,117,109,32,38,32,66,97,115,115,0,0,0,0,0,117,110,115,112,101,99,105,102,105,101,100,32,115,105,122,101,0,0,0,0,0,0,0,0,109,101,100,105,97,95,116,121,112,101,0,0,0,0,0,0,108,105,109,105,116,0,0,0,100,101,99,111,100,101,114,115,0,0,0,0,0,0,0,0,109,111,118,0,0,0,0,0,121,117,118,52,50,50,112,49,52,98,101,0,0,0,0,0,80,111,119,100,101,114,66,108,117,101,0,0,0,0,0,0,109,105,110,95,104,97,114,100,95,99,111,109,112,0,0,0,118,105,100,101,111,32,113,117,97,110,116,105,122,101,114,32,115,99,97,108,101,32,98,108,117,114,32,40,86,66,82,41,0,0,0,0,0,0,0,0,112,112,109,0,0,0,0,0,71,111,97,0,0,0,0,0,121,117,118,106,52,50,48,112,0,0,0,0,0,0,0,0,110,111,32,100,101,99,111,100,97,98,108,101,32,68,84,83,32,102,114,97,109,101,115,0,115,116,105,107,0,0,0,0,110,111,119,0,0,0,0,0,115,101,116,32,109,97,120,32,114,117,110,116,105,109,101,32,105,110,32,115,101,99,111,110,100,115,0,0,0,0,0,0,115,97,109,112,108,101,0,0,98,101,110,99,104,58,32,117,116,105,109,101,61,37,48,46,51,102,115,10,0,0,0,0,101,110,99,111,100,101,114,115,0,0,0,0,0,0,0,0,27,91,48,109,0,0,0,0,97,115,105,110,0,0,0,0,97,118,105,0,0,0,0,0,69,78,67,79,68,69,82,95,78,79,84,95,70,79,85,78,68,0,0,0,0,0,0,0,121,117,118,52,50,50,112,49,50,108,101,0,0,0,0,0,80,108,117,109,0,0,0,0,108,111,119,32,102,114,101,113,117,101,110,99,121,0,0,0,115,101,116,32,109,105,110,105,109,117,109,32,100,105,102,102,101,114,101,110,99,101,32,98,101,116,119,101,101,110,32,116,105,109,101,115,116,97,109,112,115,32,97,110,100,32,97,117,100,105,111,32,100,97,116,97,32,40,105,110,32,115,101,99,111,110,100,115,41,32,98,101,108,111,119,32,119,104,105,99,104,32,110,111,32,116,105,109,101,115,116,97,109,112,32,99,111,109,112,101,110,115,97,116,105,111,110,32,111,102,32,101,105,116,104,101,114,32,107,105,110,100,32,105,115,32,97,112,112,108,105,101,100,0,0,0,108,117,109,66,117,102,73,110,100,101,120,32,60,32,50,32,42,32,118,76,117,109,66,117,102,83,105,122,101,0,0,0,113,98,108,117,114,0,0,0,102,117,108,108,32,99,104,114,111,109,97,32,105,110,116,101,114,112,111,108,97,116,105,111,110,32,102,111,114,32,100,101,115,116,105,110,97,116,105,111])
.concat([110,32,102,111,114,109,97,116,32,39,37,115,39,32,110,111,116,32,121,101,116,32,105,109,112,108,101,109,101,110,116,101,100,10,0,0,0,0,0,0,80,78,71,32,40,80,111,114,116,97,98,108,101,32,78,101,116,119,111,114,107,32,71,114,97,112,104,105,99,115,41,32,105,109,97,103,101,0,0,0,68,97,110,99,101,32,72,97,108,108,0,0,0,0,0,0,98,105,99,117,98,108,105,110,0,0,0,0,0,0,0,0,117,110,115,112,101,99,105,102,105,101,100,32,110,117,109,98,101,114,32,111,102,32,99,104,97,110,110,101,108,115,0,0,115,101,97,115,111,110,95,110,117,109,98,101,114,0,0,0,114,116,112,102,108,97,103,115,0,0,0,0,0,0,0,0,82,101,113,117,101,115,116,101,100,32,105,110,112,117,116,32,115,97,109,112,108,101,32,102,111,114,109,97,116,32,37,100,32,105,115,32,105,110,118,97,108,105,100,10,0,0,0,0,116,105,109,101,108,105,109,105,116,0,0,0,0,0,0,0,67,111,100,101,99,32,39,37,115,39,32,105,115,32,107,110,111,119,110,32,116,111,32,70,70,109,112,101,103,44,32,98,117,116,32,110,111,32,37,115,32,102,111,114,32,105,116,32,97,114,101,32,97,118,97,105,108,97,98,108,101,46,32,70,70,109,112,101,103,32,109,105,103,104,116,32,110,101,101,100,32,116,111,32,98,101,32,114,101,99,111,109,112,105,108,101,100,32,119,105,116,104,32,97,100,100,105,116,105,111,110,97,108,32,101,120,116,101,114,110,97,108,32,108,105,98,114,97,114,105,101,115,46,10,0,0,101,110,118,101,108,111,112,101,32,115,99,97,108,101,102,97,99,116,111,114,32,111,118,101,114,102,108,111,119,32,105,110,32,100,101,113,117,97,110,116,10,0,0,0,0,0,0,0,68,101,112,101,110,100,101,110,116,32,99,111,117,112,108,105,110,103,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,116,111,103,101,116,104,101,114,32,119,105,116,104,32,76,84,80,10,0,0,98,105,116,114,97,116,101,32,116,111,111,32,108,111,119,58,32,103,111,116,32,37,105,44,32,110,101,101,100,32,50,52,48,48,48,32,111,114,32,104,105,103,104,101,114,10,0,0,69,120,112,111,110,101,110,116,32,118,108,99,32,105,110,118,97,108,105,100,10,0,0,0,87,114,111,110,103,32,112,97,99,107,101,116,32,116,121,112,101,32,105,110,32,73,100,32,104,101,97,100,101,114,10,0,105,115,116,32,38,38,32,33,111,115,116,45,62,102,105,108,116,101,114,0,0,0,0,0,97,114,116,105,115,116,0,0,121,117,118,52,50,50,112,49,50,98,101,0,0,0,0,0,80,105,110,107,0,0,0,0,67,111,100,101,99,32,116,121,112,101,32,111,114,32,105,100,32,109,105,115,109,97,116,99,104,101,115,10,0,0,0,0,109,105,110,95,99,111,109,112,0,0,0,0,0,0,0,0,118,105,100,101,111,32,113,117,97,110,116,105,122,101,114,32,115,99,97,108,101,32,99,111,109,112,114,101,115,115,105,111,110,32,40,86,66,82,41,46,32,67,111,110,115,116,97,110,116,32,111,102,32,114,97,116,101,99,111,110,116,114,111,108,32,101,113,117,97,116,105,111,110,46,32,82,101,99,111,109,109,101,110,100,101,100,32,114,97,110,103,101,32,102,111,114,32,100,101,102,97,117,108,116,32,114,99,95,101,113,58,32,48,46,48,45,49,46,48,0,112,110,103,0,0,0,0,0,69,117,114,111,45,72,111,117,115,101,0,0,0,0,0,0,117,110,115,112,101,99,105,102,105,101,100,32,115,97,109,112,108,101,32,114,97,116,101,0,116,118,115,110,0,0,0,0,83,105,109,112,108,101,32,102,105,108,116,101,114,103,114,97,112,104,32,39,37,115,39,32,100,111,101,115,32,110,111,116,32,104,97,118,101,32,101,120,97,99,116,108,121,32,111,110,101,32,105,110,112,117,116,32,97,110,100,32,111,117,116,112,117,116,46,10,0,0,0,0,101,110,97,98,108,101,32,111,114,32,100,105,115,97,98,108,101,32,105,110,116,101,114,97,99,116,105,111,110,32,111,110,32,115,116,97,110,100,97,114,100,32,105,110,112,117,116,0,108,105,98,97,118,99,111,100,101,99,47,109,112,101,103,97,117,100,105,111,100,101,99,46,99,0,0,0,0,0,0,0,111,102,102,115,101,116,32,61,61,32,97,118,99,116,120,45,62,101,120,116,114,97,100,97,116,97,95,115,105,122,101,0,78,111,32,99,111,100,101,99,32,110,97,109,101,32,115,112,101,99,105,102,105,101,100,46,10,0,0,0,0,0,0,0,106,111,105,110,116,95,115,116,101,114,101,111,0,0,0,0,77,101,116,97,100,97,116,97,32,108,105,98,114,97,114,121,32,105,110,105,116,105,97,108,105,122,97,116,105,111,110,32,101,114,114,111,114,0,0,0,97,118,105,111,102,108,97,103,115,0,0,0,0,0,0,0,79,117,116,112,117,116,32,102,105,108,101,32,35,37,100,32,100,111,101,115,32,110,111,116,32,99,111,110,116,97,105,110,32,97,110,121,32,115,116,114,101,97,109,10,0,0,0,0,121,117,118,52,50,48,112,49,52,108,101,0,0,0,0,0,80,101,114,117,0,0,0,0,77,69,84,65,68,65,84,65,95,66,76,79,67,75,95,80,73,67,84,85,82,69,0,0,84,111,111,32,111,108,100,32,111,114,32,117,110,115,117,112,112,111,114,116,101,100,32,84,104,101,111,114,97,32,40,37,120,41,10,0,0,0,0,0,101,110,97,98,108,101,32,115,111,120,114,32,67,104,101,98,121,115,104,101,118,32,112,97,115,115,98,97,110,100,32,38,32,104,105,103,104,101,114,45,112,114,101,99,105,115,105,111,110,32,105,114,114,97,116,105,111,110,97,108,32,114,97,116,105,111,32,97,112,112,114,111,120,105,109,97,116,105,111,110,0,0,0,0,0,0,0,0,115,112,101,101,120,32,112,97,99,107,101,116,32,116,111,111,32,115,109,97,108,108,10,0,113,99,111,109,112,0,0,0,115,107,101,108,101,116,111,110,0,0,0,0,0,0,0,0,81,45,116,101,97,109,32,81,80,69,71,0,0,0,0,0,79,112,117,115,0,0,0,0,65,32,99,97,112,101,108,108,97,0,0,0,0,0,0,0,117,110,115,112,101,99,105,102,105,101,100,32,115,97,109,112,108,101,32,102,111,114,109,97,116,0,0,0,0,0,0,0,1,97,117,100,105,111,0,0,101,112,105,115,111,100,101,95,115,111,114,116,0,0,0,0,114,118,50,48,0,0,0,0,102,76,97,67,0,0,0,0,75,87,45,68,73,82,65,67,0,0,0,0,0,0,0,0,115,116,100,105,110,0,0,0,79,116,104,101,114,0,0,0,87,77,47,69,110,99,111,100,105,110,103,83,101,116,116,105,110,103,115,0,0,0,0,0,68,101,109,117,120,101,114,32,37,115,32,91,37,115,93,58,10,0,0,0,0,0,0,0,79,103,103,0,0,0,0,0,99,111,109,109,101,110,116,0,107,101,121,32,32,32,32,102,117,110,99,116,105,111,110,10,63,32,32,32,32,32,32,115,104,111,119,32,116,104,105,115,32,104,101,108,112,10,43,32,32,32,32,32,32,105,110,99,114,101,97,115,101,32,118,101,114,98,111,115,105,116,121,10,45,32,32,32,32,32,32,100,101,99,114,101,97,115,101,32,118,101,114,98,111,115,105,116,121,10,99,32,32,32,32,32,32,83,101,110,100,32,99,111,109,109,97,110,100,32,116,111,32,102,105,108,116,101,114,103,114,97,112,104,10,68,32,32,32,32,32,32,99,121,99,108,101,32,116,104,114,111,117,103,104,32,97,118,97,105,108,97,98,108,101,32,100,101,98,117,103,32,109,111,100,101,115,10,104,32,32,32,32,32,32,100,117,109,112,32,112,97,99,107,101,116,115,47,104,101,120,32,112,114,101,115,115,32,116,111,32,99,121,99,108,101,32,116,104,114,111,117,103,104,32,116,104,101,32,51,32,115,116,97,116,101,115,10,113,32,32,32,32,32,32,113,117,105,116,10,115,32,32,32,32,32,32,83,104,111,119,32,81,80,32,104,105,115,116,111,103,114,97,109,10,0,0,0,0,0,0,0,76,97,121,111,117,116,32,105,110,100,105,99,97,116,101,115,32,97,32,100,105,102,102,101,114,101,110,116,32,110,117,109,98,101,114,32,111,102,32,99,104,97,110,110,101,108,115,32,116,104,97,110,32,97,99,116,117,97,108,108,121,32,112,114,101,115,101,110,116,10,0,0,121,117,118,52,50,48,112,49,52,98,101,0,0,0,0,0,80,101,97,99,104,80,117,102,102,0,0,0,0,0,0,0,79,103,103,0,0,0,0,0,109,112,51,0,0,0,0,0,99,104,101,98,121,0,0,0,73,78,65,77,0,0,0,0,100,101,108,97,121,0,0,0,103,101,110,101,114,97,116,101,32,112,116,115,0,0,0,0,113,112,101,103,0,0,0,0,68,114,117,109,32,83,111,108,111,0,0,0,0,0,0,0,85,110,97,98,108,101,32,116,111,32,102,105,110,100,32,97,32,115,117,105,116,97,98,108,101,32,111,117,116,112,117,116,32,102,111,114,109,97,116,32,102,111,114,32,39,37,115,39,10,0,0,0,0,0,0,0,116,104,101,111,114,97,0,0,67,104,97,110,103,105,110,103,32,115,116,114,101,97,109,32,112,97,114,97,109,101,116,101,114,115,32,105,110,32,109,117,108,116,105,115,116,114,101,97,109,32,111,103,103,0,0,0,117,110,115,112,101,99,105,102,105,101,100,32,102,114,97,109,101,32,115,105,122,101,0,0,100,105,109,101,110,115,105,111,110,115,32,110,111,116,32,115,101,116,10,0,0,0,0,0,65,117,100,105,111,32,112,97,99,107,101,116,32,111,102,32,115,105,122,101,32,37,100,32,40,115,116,97,114,116,105,110,103,32,119,105,116,104,32,37,48,56,88,46,46,46,41,32,105,115,32,105,110,118,97,108,105,100,44,32,119,114,105,116,105,110,103,32,105,116,32,97,110,121,119,97,121,46,10,0,116,118,101,115,0,0,0,0,117,114,108,0,0,0,0,0,77,80,51,32,40,77,80,69,71,32,97,117,100,105,111,32,108,97,121,101,114,32,51,41,0,0,0,0,0,0,0,0,77,79,86,32,109,117,120,101,114,32,102,108,97,103,115,0,32,32,32,32,68,101,102,97,117,108,116,32,115,117,98,116,105,116,108,101,32,99,111,100,101,99,58,32,37,115,46,10,0,0,0,0,0,0,0,0,77,80,50,47,51,32,40,77,80,69,71,32,97,117,100,105,111,32,108,97,121,101,114,32,50,47,51,41,0,0,0,0,114,116,112,111,0,0,0,0,84,76,65,78,0,0,0,0,100,101,98,117,103,61,37,100,10,0,0,0,0,0,0,0,121,117,118,52,50,48,112,49,50,108,101,0,0,0,0,0,80,97,112,97,121,97,87,104,105,112,0,0,0,0,0,0,97,115,102,45,62,112,97,99,107,101,116,95,116,105,109,101,115,116,97,109,112,95,101,110,100,32,62,61,32,97,115,102,45,62,112,97,99,107,101,116,95,116,105,109,101,115,116,97,109,112,95,115,116,97,114,116,0,0,0,0,0,0,0,0,105,110,118,97,108,105,100,32,112,97,99,107,101,116,95,108,101,110,103,116,104,32,37,100,32,97,116,58,37,108,108,100,10,0,0,0,0,0,0,0,115,101,116,32,115,111,120,114,32,114,101,115,97,109,112,108,105,110,103,32,112,114,101,99,105,115,105,111,110,32,40,105,110,32,98,105,116,115,41,0,108,105,98,97,118,102,105,108,116,101,114,47,97,102,95,97,114,101,115,97,109,112,108,101,46,99,0,0,0,0,0,0,65,32,99,111,109,109,97,45,115,101,112,97,114,97,116,101,100,32,108,105,115,116,32,111,102,32,115,97,109,112,108,101,32,114,97,116,101,115,46,0,77,97,120,105,109,117,109,32,100,117,114,97,116,105,111,110,32,111,102,32,116,104,101,32,111,117,116,112,117,116,32,105,110,32,115,101,99,111,110,100,115,0,0,0,0,0,0,0,102,114,97,109,101,95,110,117,109,98,101,114,0,0,0,0,80,82,69,86,95,79,85,84,84,0,0,0,0,0,0,0,91,93,44,59,10,0,0,0,77,105,114,111,32,86,105,100,101,111,88,76,0,0,0,0,80,117,110,107,32,82,111,99,107,0,0,0,0,0,0,0,115,97,109,112,108,101,95,102,109,116,0,0,0,0,0,0,110,101,116,119,111,114,107,0,115,97,109,112,108,101,95,114,97,116,101,115,0,0,0,0,115,99,97,108,101,0,0,0,108,105,98,97,118,102,105,108,116,101,114,47,97,118,102,105,108,116,101,114,46,99,0,0,119,114,105,116,101,32,112,114,111,103,114,97,109,45,114,101,97,100,97,98,108,101,32,112,114,111,103,114,101,115,115,32,105,110,102,111,114,109,97,116,105,111,110,0,0,0,0,0,32,32,32,32,68,101,102,97,117,108,116,32,97,117,100,105,111,32,99,111,100,101,99,58,32,37,115,46,10,0,0,0,101,114,114,111,114,32,112,97,114,115,105,110,103,32,100,101,98,117,103,32,118,97,108,117,101,10,0,0,0,0,0,0,121,117,118,52,50,48,112,49,50,98,101,0,0,0,0,0,80,97,108,101,86,105,111,108,101,116,82,101,100,0,0,0,112,114,101,99,105,115,105,111,110,0,0,0,0,0,0,0,102,114,97,109,101,95,115,105,122,101,0,0,0,0,0,0,118,105,120,108,0,0,0,0,68,117,101,116,0,0,0,0,115,116,97,114,116,32,116,105,109,101,32,105,115,32,110,111,116,32,115,101,116,32,105,110,32,101,115,116,105,109,97,116,101,95,116,105,109,105,110,103,115,95,102,114,111,109,95,112,116,115,10,0,0,0,0,0,116,118,110,110,0,0,0,0,112,114,111,103,114,101,115,115,0,0,0,0,0,0,0,0,65,100,118,97,110,99,101,100,32,65,117,100,105,111,32,111,112,116,105,111,110,115,58,0,32,32,32,32,68,101,102,97,117,108,116,32,118,105,100,101,111,32,99,111,100,101,99,58,32,37,115,46,10,0,0,0,37,100,0,0,0,0,0,0,121,117,118,97,52,50,50,112,0,0,0,0,0,0,0,0,80,97,108,101,84,117,114,113,117,111,105,115,101,0,0,0,77,97,108,102,111,114,109,101,100,32,65,65,67,32,98,105,116,115,116,114,101,97,109,32,100,101,116,101,99,116,101,100,58,32,117,115,101,32,97,117,100,105,111,32,98,105,116,115,116,114,101,97,109,32,102,105,108,116,101,114,32,39,97,97,99,95,97,100,116,115,116,111,97,115,99,39,32,116,111,32,102,105,120,32,105,116,32,40,39,45,98,115,102,58,97,32,97,97,99,95,97,100,116,115,116,111,97,115,99,39,32,111,112,116,105,111,110,32,119,105,116,104,32,102,102,109,112,101,103,41,10,0,0,0,0,0,115,101,108,101,99,116,32,83,111,88,32,82,101,115,97,109,112,108,101,114,0,0,0,0,115,101,116,32,99,117,116,111,102,102,32,98,97,110,100,119,105,100,116,104,0,0,0,0,65,112,112,108,101,32,81,117,105,99,107,68,114,97,119,0,70,114,101,101,115,116,121,108,101,0,0,0,0,0,0,0,69,115,116,105,109,97,116,105,110,103,32,100,117,114,97,116,105,111,110,32,102,114,111,109,32,98,105,116,114,97,116,101,44,32,116,104,105,115,32,109,97,121,32,98,101,32,105,110,97,99,99,117,114,97,116,101,10,0,0,0,0,0,0,0,101,112,105,115,111,100,101,95,105,100,0,0,0,0,0,0,97,100,100,32,116,105,109,105,110,103,115,32,102,111,114,32,101,97,99,104,32,116,97,115,107,0,0,0,0,0,0,0,32,32,32,32,77,105,109,101,32,116,121,112,101,58,32,37,115,46,10,0,0,0,0,0,80,97,114,115,101,32,101,114,114,111,114,44,32,97,116,32,108,101,97,115,116,32,51,32,97,114,103,117,109,101,110,116,115,32,119,101,114,101,32,101,120,112,101,99,116,101,100,44,32,111,110,108,121,32,37,100,32,103,105,118,101,110,32,105,110,32,115,116,114,105,110,103,32,39,37,115,39,10,0,0,121,117,118,97,52,52,52,112,0,0,0,0,0,0,0,0,80,97,108,101,71,114,101,101,110,0,0,0,0,0,0,0,115,111,120,114,0,0,0,0,105,110,118,97,108,105,100,32,115,105,122,101,32,37,108,108,100,32,105,110,32,115,116,115,100,10,0,0,0,0,0,0,99,117,116,111,102,102,0,0,113,100,114,97,119,0,0,0,82,104,121,116,104,109,105,99,32,83,111,117,108,0,0,0,73,103,110,111,114,105,110,103,32,111,117,116,108,105,101,114,32,110,111,110,32,112,114,105,109,97,114,121,32,115,116,114,101,97,109,32,115,116,97,114,116,116,105,109,101,32,37,102,10,0,0,0,0,0,0,0,116,118,101,110,0,0,0,0,98,117,102,102,101,114,115,105,110,107,0,0,0,0,0,0,98,101,110,99,104,109,97,114,107,95,97,108,108,0,0,0,32,32,32,32,67,111,109,109,111,110,32,101,120,116,101,110,115,105,111,110,115,58,32,37,115,46,10,0,0,0,0,0,48,0,0,0,0,0,0,0,67,111,109,109,97,110,100,32,114,101,112,108,121,32,102,111,114,32,115,116,114,101,97,109,32,37,100,58,32,114,101,116,58,37,100,32,114,101,115,58,37,115,10,0,0,0,0,0,98,103,114,48,0,0,0,0,80,97,108,101,71,111,108,100,101,110,82,111,100,0,0,0,115,101,108,101,99,116,32,83,87,32,82,101,115,97,109,112,108,101,114,0,0,0,0,0,115,101,116,32,110,117,109,98,101,114,32,111,102,32,97,117,100,105,111,32,99,104,97,110,110,101,108,115,0,0,0,0,73,66,77,32,85,108,116,105,77,111,116,105,111,110,0,0,80,111,119,101,114,32,66,97,108,108,97,100,0,0,0,0,109,112,101,103,116,115,0,0,97,108,98,117,109,0,0,0,115,104,111,119,0,0,0,0,115,115,101,103,109,101,110,116,0,0,0,0,0,0,0,0,116,114,117,110,99,97,116,101,0,0,0,0,0,0,0,0,97,100,100,32,116,105,109,105,110,103,115,32,102,111,114,32,98,101,110,99,104,109,97,114,107,105,110,103,0,0,0,0,77,117,120,101,114,32,37,115,32,91,37,115,93,58,10,0,80,114,111,99,101,115,115,105,110,103,32,99,111,109,109,97,110,100,32,116,97,114,103,101,116,58,37,115,32,116,105,109,101,58,37,102,32,99,111,109,109,97,110,100,58,37,115,32,97,114,103,58,37,115,0,0,48,98,103,114,0,0,0,0,79,114,99,104,105,100,0,0,108,105,98,97,118,117,116,105,108,47,117,116,105,108,115,46,99,0,0,0,0,0,0,0,115,119,114,0,0,0,0,0,97,99,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,117,108,116,105,0,0,0,0,66,97,108,108,97,100,0,0,112,97,108,56,0,0,0,0,109,112,101,103,0,0,0,0,102,105,108,101,0,0,0,0,116,118,115,104,0,0,0,0,37,72,37,77,37,83,0,0,65,83,70,32,40,65,100,118,97,110,99,101,100,32,47,32,65,99,116,105,118,101,32,83,116,114,101,97,109,105,110,103,32,70,111,114,109,97,116,41,0,0,0,0,0,0,0,0,98,101,110,99,104,109,97,114,107,0,0,0,0,0,0,0,112,105,120,101,108,0,0,0,65,116,32,108,101,97,115,116,32,111,110,101,32,111,117,116,112,117,116,32,102,105,108,101,32,109,117,115,116,32,98,101,32,115,112,101,99,105,102,105,101,100,10,0,0,0,0,0,85,110,107,110,111,119,110,32,102,111,114,109,97,116,32,39,37,115,39,46,10,0,0,0,27,91,52,56,59,53,59,37,100,109,27,91,51,56,59,53,59,37,100,109,0,0,0,0,97,116,97,110,0,0,0,0,37,54,51,91,94,32,93,32,37,108,102,32,37,50,53,53,91,94,32,93,32,37,50,53,53,91,94,10,93,0,0,0,68,101,109,117,120,101,114,32,110,111,116,32,102,111,117,110,100,0,0,0,0,0,0,0,114,103,98,48,0,0,0,0,79,114,97,110,103,101,82,101,100,0,0,0,0,0,0,0,76,70,69,0,0,0,0,0,115,101,116,32,114,101,115,97,109,112,108,105,110,103,32,69,110,103,105,110,101,0,0,0,102,105,114,115,116,67,104,114,83,114,99,89,32,62,61,32,108,97,115,116,73,110,67,104,114,66,117,102,32,45,32,118,67,104,114,66,117,102,83,105,122,101,32,43,32,49,0,0,115,101,116,32,97,117,100,105,111,32,115,97,109,112,108,105,110,103,32,114,97,116,101,32,40,105,110,32,72,122,41,0,37,115,32,111,117,116,112,117,116,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,119,105,116,104,32,104,97,108,102,32,99,104,114,111,109,97,32,114,101,115,111,108,117,116,105,111,110,44,32,115,119,105,116,99,104,105,110,103,32,116,111,32,102,117,108,108,10,0,0,0,0,0,0,65,83,70,32,40,65,100,118,97,110,99,101,100,32,47,32,65,99,116,105,118,101,32,83,116,114,101,97,109,105,110,103,32,70,111,114,109,97,116,41,0,0,0,0,0,0,0,0,84,101,99,104,83,109,105,116,104,32,83,99,114,101,101,110,32,67,97,112,116,117,114,101,32,67,111,100,101,99,0,0,70,111,108,107,108,111,114,101,0,0,0,0,0,0,0,0,97,118,101,114,97,103,105,110,103,32,97,114,101,97,0,0,10,37,115,32,32,37,45,49,54,115,58,32,0,0,0,0,114,101,102,45,62,98,117,102,45,62,114,101,102,99,111,117,110,116,32,62,32,48,0,0,115,121,110,111,112,115,105,115,0,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,117,99,104,0,0,0,0,0,99,104,97,110,110,101,108,115,32,61,61,32,97,118,95,103,101,116,95,99,104,97,110,110,101,108,95,108,97,121,111,117,116,95,110,98,95,99,104,97,110,110,101,108,115,40,108,105,110,107,45,62,99,104,97,110,110,101,108,95,108,97,121,111,117,116,41,32,124,124,32,33,97,118,95,103,101,116,95,99,104,97,110,110,101,108,95,108,97,121,111,117,116,95,110,98,95,99,104,97,110,110,101,108,115,40,108,105,110,107,45,62,99,104,97,110,110,101,108,95,108,97,121,111,117,116,41,0,110,117,109,98,101,114,0,0,69,82,82,79,82,32,58,32,110,111,32,115,117,98,98,97,110,100,32,102,111,117,110,100,32,102,111,114,32,102,114,101,113,117,101,110,99,121,32,37,100,10,0,0,0,0,0,0,73,110,112,117,116,32,98,117,102,102,101,114,32,101,120,104,97,117,115,116,101,100,32,98,101,102,111,114,101,32,69,78,68,32,101,108,101,109,101,110,116,32,102,111,117,110,100,10,0,0,0,0,0,0,0,0,115,97,109,112,108,101,32,114,97,116,101,32,105,115,32,116,111,111,32,104,105,103,104,58,32,37,100,32,62,32,52,56,107,72,122,10,0,0,0,0,82,101,115,97,109,112,108,101,32,97,117,100,105,111,32,100,97,116,97,46,0,0,0,0,104,103,97,105,110,32,118,108,99,32,105,110,118,97,108,105,100,10,0,0,0,0,0,0,73,100,32,104,101,97,100,101,114,32,105,115,32,116,111,111,32,115,104,111,114,116,10,0,80,97,115,115,32,116,104,101,32,115,111,117,114,99,101,32,117,110,99,104,97,110,103,101,100,32,116,111,32,116,104,101,32,111,117,116,112,117,116,46,0,0,0,0,0,0,0,0,10,69,110,116,101,114,32,99,111,109,109,97,110,100,58,32,60,116,97,114,103,101,116,62,32,60,116,105,109,101,62,32,60,99,111,109,109,97,110,100,62,91,32,60,97,114,103,117,109,101,110,116,62,93,10,0,78,111,118,32,50,49,32,50,48,49,51,0,0,0,0,0,48,114,103,98,0,0,0,0,79,114,97,110,103,101,0,0,73,103,110,111,114,105,110,103,32,105,110,118,97,108,105,100,32,119,105,100,116,104,47,104,101,105,103,104,116,32,118,97,108,117,101,115,10,0,0,0,67,111,110,118,101,114,116,32,116,104,101,32,105,110,112,117,116,32,97,117,100,105,111,32,116,111,32,111,110,101,32,111,102,32,116,104,101,32,115,112,101,99,105,102,105,101,100,32,102,111,114,109,97,116,115,46,0,0,0,0,0,0,0,0,114,101,115,97,109,112,108,101,114,0,0,0,0,0,0,0,80,97,115,115,32,116,104,101,32,115,111,117,114,99,101,32,117,110,99,104,97,110,103,101,100,32,116,111,32,116,104,101,32,111,117,116,112,117,116,46,0,0,0,0,0,0,0,0,97,114,0,0,0,0,0,0,116,115,99,99,0,0,0,0,67,111,110,118,101,114,116,32,116,104,101,32,105,110,112,117,116,32,118,105,100,101,111,32,116,111,32,111,110,101,32,111,102,32,116,104,101,32,115,112,101,99,105,102,105,101,100,32,112,105,120,101,108,32,102,111,114,109,97,116,115,46,0,0,83,97,109,98,97,0,0,0,37,115,0,0,0,0,0,0,108,100,101,115,0,0,0,0,97,114,101,115,97,109,112,108,101,95,115,119,114,95,111,112,116,115,0,0,0,0,0,0,80,105,99,107,32,111,110,101,32,99,111,110,116,105,110,117,111,117,115,32,115,101,99,116,105,111,110,32,102,114,111,109,32,116,104,101,32,105,110,112,117,116,44,32,100,114,111,112,32,116,104,101,32,114,101,115,116,46,0,0,0,0,0,0,115,101,116,32,116,104,101,32,110,117,109,98,101,114,32,111,102,32,100,97,116,97,32,102,114,97,109,101,115,32,116,111,32,114,101,99,111,114,100,0,102,108,97,103,115,0,0,0,115,45,62,102,114,97,109,101,32,33,61,32,48,76,0,0,76,97,118,99,53,53,46,49,56,46,49,48,50,0,0,0,85,115,101,32,98,105,116,32,114,101,115,101,114,118,111,105,114,46,0,0,0,0,0,0,84,114,97,110,115,112,111,114,116,32,108,105,98,114,97,114,121,32,105,110,105,116,105,97,108,105,122,97,116,105,111,110,32,101,114,114,111,114,0,0,105,110,118,97,108,105,100,32,109,97,120,32,98,108,111,99,107,115,105,122,101,58,32,37,100,10,0,0,0,0,0,0,78,101,119,32,37,115,32,115,116,114,101,97,109,32,37,100,58,37,100,32,97,116,32,112,111,115,58,37,108,108,100,32,97,110,100,32,68,84,83,58,37,115,115,10,0,0,0,0,98,103,114,97,54,52,108,101,0,0,0,0,0,0,0,0,79,108,105,118,101,68,114,97,98,0,0,0,0,0,0,0,83,101,116,32,80,84,83,32,102,111,114,32,116,104,101,32,111,117,116,112,117,116,32,118,105,100,101,111,32,102,114,97,109,101,46,0,0,0,0,0,114,101,115,97,109,112,108,101,95,99,117,116,111,102,102,0,115,101,116,32,116,104,101,32,103,114,111,117,112,32,111,102,32,112,105,99,116,117,114,101,32,40,71,79,80,41,32,115,105,122,101,0,0,0,0,0,83,110,111,119,0,0,0,0,85,110,97,98,108,101,32,116,111,32,112,97,114,115,101,32,103,114,97,112,104,32,100,101,115,99,114,105,112,116,105,111,110,32,115,117,98,115,116,114,105,110,103,58,32,34,37,115,34,10,0,0,0,0,0,0,84,97,110,103,111,0,0,0,8,10,11,12,13,0,0,0,100,101,115,99,114,105,112,116,105,111,110,0,0,0,0,0,82,101,97,108,86,105,100,101,111,32,49,46,48,0,0,0,100,102,114,97,109,101,115,0,79,108,100,105,101,115,0,0,73,110,118,97,108,105,100,32,112,105,120,101,108,32,102,111,114,109,97,116,32,39,37,115,39,10,0,0,0,0,0,0,101,110,99,111,100,101,100,95,98,121,0,0,0,0,0,0,118,111,114,98,105,115,0,0,100,101,115,99,114,105,112,116,105,111,110,0,0,0,0,0,66,117,102,102,101,114,32,105,110,112,117,116,32,105,109,97,103,101,115,32,97,110,100,32,115,101,110,100,32,116,104,101,109,32,119,104,101,110,32,116,104,101,121,32,97,114,101,32,114,101,113,117,101,115,116,101,100,46,0,0,0,0,0,0,69,114,114,111,114,32,119,104,105,108,101,32,100,101,99,111,100,105,110,103,32,115,116,114,101,97,109,32,35,37,100,58,37,100,58,32,37,115,10,0,98,103,114,97,54,52,98,101,0,0,0,0,0,0,0,0,79,108,105,118,101,0,0,0,73,110,112,117,116,32,99,111,110,116,101,120,116,32,104,97,115,32,110,111,116,32,98,101,101,110,32,112,114,111,112,101,114,108,121,32,97,108,108,111,99,97,116,101,100,32,98,121,32,97,118,102,111,114,109,97,116,95,97,108,108,111,99,95,99,111,110,116,101,120,116,40,41,32,97,110,100,32,105,115,32,110,111,116,32,78,85,76,76,32,101,105,116,104,101,114,10,0,0,0,0,0,0,0,115,101,116,32,99,117,116,111,102,102,32,102,114,101,113,117,101,110,99,121,32,114,97,116,105,111,0,0,0,0,0,0,108,97,110,103,117,97,103,101,0,0,0,0,0,0,0,0,103,0,0,0,0,0,0,0,103,101,110,112,116,115,0,0,116,105,116,108,101,0,0,0,115,110,111,119,0,0,0,0,67,108,117,98,0,0,0,0,118,111,114,98,105,115,0,0,98,117,102,102,101,114,0,0,67,97,110,110,111,116,32,105,100,101,110,116,105,102,121,32,110,101,119,32,115,116,114,101,97,109,10,0,0,0,0,0,37,115,32,32,37,45,49,54,115,58,32,0,0,0,0,0,116,105,109,101,32,98,97,115,101,32,110,111,116,32,115,101,116,10,0,0,0,0,0,0,84,67,79,78,0,0,0,0,100,101,115,99,0,0,0,0,73,68,51,118,50,46,37,100,32,116,97,103,32,115,107,105,112,112,101,100,44,32,99,97,110,110,111,116,32,104,97,110,100,108,101,32,37,115,10,0,115,116,114,105,110,103,61,115,116,114,105,110,103,0,0,0,109,111,118,102,108,97,103,115,0,0,0,0,0,0,0,0,81,117,105,99,107,84,105,109,101,32,47,32,77,79,86,0,97,98,117,102,102,101,114,115,105,110,107,0,0,0,0,0,116,105,116,108,101,0,0,0,100,101,109,117,120,101,114,43,102,102,109,112,101,103,32,45,62,32,105,115,116,95,105,110,100,101,120,58,37,100,32,116,121,112,101,58,37,115,32,112,107,116,95,112,116,115,58,37,115,32,112,107,116,95,112,116,115,95,116,105,109,101,58,37,115,32,112,107,116,95,100,116,115,58,37,115,32,112,107,116,95,100,116,115,95,116,105,109,101,58,37,115,32,111,102,102,58,37,115,32,111,102,102,95,116,105,109,101,58,37,115,10,0,0,0,0,0,0,0,0,114,103,98,97,54,52,108,101,0,0,0,0,0,0,0,0,79,108,100,76,97,99,101,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,102,102,32,97,115,102,32,98,97,100,32,110,111,110,32,122,101,114,111,10,0,0,0,0,99,117,116,111,102,102,0,0,111,117,116,108,105,110,107,45,62,115,97,109,112,108,101,95,114,97,116,101,32,61,61,32,111,117,116,95,114,97,116,101,0,0,0,0,0,0,0,0,115,97,109,112,108,101,95,114,97,116,101,115,0,0,0,0,100,117,114,97,116,105,111,110,0,0,0,0,0,0,0,0,116,105,109,101,95,98,97,115,101,0,0,0,0,0,0,0,80,82,69,86,95,79,85,84,80,84,83,0,0,0,0,0,61,44,59,91,10,0,0,0,81,117,105,99,107,84,105,109,101,32,65,110,105,109,97,116,105,111,110,32,40,82,76,69,41,32,118,105,100,101,111,0,83,108,111,119,32,74,97,109,0,0,0,0,0,0,0,0,37,115,77,101,116,97,100,97,116,97,58,10,0,0,0,0,115,97,109,112,108,101,95,114,97,116,101,0,0,0,0,0,108,121,114,105,99,115,0,0,115,101,116,32,116,104,101,32,115,117,112,112,111,114,116,101,100,32,115,97,109,112,108,101,32,102,111,114,109,97,116,115,0,0,0,0,0,0,0,0,105,110,118,97,108,105,100,32,102,114,97,109,101,32,105,100,44,32,97,115,115,117,109,105,110,103,32,112,97,100,100,105,110,103,10,0,0,0,0,0,97,108,108,0,0,0,0,0,98,101,115,116,95,105,100,120,32,62,61,32,48,0,0,0,33,108,105,110,107,45,62,102,114,97,109,101,95,114,101,113,117,101,115,116,101,100,0,0,97,100,100,32,109,101,116,97,100,97,116,97,0,0,0,0,80,84,83,32,37,108,108,100,44,32,110,101,120,116,58,37,108,108,100,32,105,110,118,97,108,105,100,32,100,114,111,112,112,105,110,103,32,115,116,58,37,100,10,0,0,0,0,0,114,103,98,97,54,52,98,101,0,0,0,0,0,0,0,0,78,97,118,121,0,0,0,0,101,110,97,98,108,101,32,108,105,110,101,97,114,32,105,110,116,101,114,112,111,108,97,116,105,111,110,0,0,0,0,0,101,120,116,114,97,100,97,116,97,95,115,105,122,101,0,0,73,110,116,101,114,110,97,108,32,98,117,102,102,101,114,32,105,110,99,111,110,115,105,115,116,101,110,99,121,46,32,102,108,117,115,104,98,105,116,115,32,60,62,32,82,101,115,118,83,105,122,101,0,0,0,0,113,116,114,108,101,0,0,0,83,97,116,105,114,101,0,0,44,32,37,49,46,48,102,107,32,37,115,0,0,0,0,0,169,108,121,114,0,0,0,0,70,97,105,108,101,100,32,116,111,32,117,110,99,111,109,112,114,101,115,115,32,116,97,103,58,32,37,100,10,0,0,0,73,78,84,69,82,78,65,76,32,69,82,82,79,82,32,73,78,32,86,66,82,32,78,69,87,32,67,79,68,69,32,40,57,56,54,41,44,32,112,108,101,97,115,101,32,115,101,110,100,32,98,117,103,32,114,101,112,111,114,116,10,0,0,0,109,101,116,97,100,97,116,97,0,0,0,0,0,0,0,0,65,117,100,105,111,32,111,112,116,105,111,110,115,58,0,0,32,32,32,32,32,32,32,35,37,100,58,32,37,115,32,40,37,115,41,10,0,0,0,0,68,84,83,32,37,108,108,100,44,32,110,101,120,116,58,37,108,108,100,32,115,116,58,37,100,32,105,110,118,97,108,105,100,32,100,114,111,112,112,105,110,103,10,0,0,0,0,0,120,121,122,49,50,98,101,0,78,97,118,97,106,111,87,104,105,116,101,0,0,0,0,0,108,105,110,101,97,114,95,105,110,116,101,114,112,0,0,0,105,116,101,114,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,0,0,76,67,76,32,40,76,111,115,115,76,101,115,115,32,67,111,100,101,99,32,76,105,98,114,97,114,121,41,32,90,76,73,66,0,0,0,0,0,0,0,80,111,114,110,32,71,114,111,111,118,101,0,0,0,0,0,44,32,37,49,46,48,102,32,37,115,0,0,0,0,0,0,103,114,111,117,112,105,110,103,0,0,0,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,114,101,97,100,32,99,111,109,112,114,101,115,115,101,100,32,116,97,103,10,0,0,116,105,109,101,0,0,0,0,116,105,109,101,115,116,97,109,112,32,100,105,115,99,111,110,116,105,110,117,105,116,121,32,37,108,108,100,44,32,110,101,119,32,111,102,102,115,101,116,61,32,37,108,108,100,10,0,120,121,122,49,50,108,101,0,77,111,99,99,97,115,105,110,0,0,0,0,0,0,0,0,115,101,116,32,115,119,114,32,114,101,115,97,109,112,108,105,110,103,32,112,104,97,115,101,32,115,104,105,102,116,0,0,105,116,101,114,0,0,0,0,122,108,105,98,0,0,0,0,80,114,105,109,117,115,0,0,44,32,37,51,46,50,102,32,37,115,0,0,0,0,0,0,169,103,114,112,0,0,0,0,70,97,105,108,101,100,32,116,111,32,97,108,108,111,99,32,37,108,100,32,98,121,116,101,115,10,0,0,0,0,0,0,115,101,116,32,116,104,101,32,114,101,99,111,114,100,105,110,103,32,116,105,109,101,115,116,97,109,112,32,40,39,110,111,119,39,32,116,111,32,115,101,116,32,116,104,101,32,99,117,114,114,101,110,116,32,116,105,109,101,41,0,0,0,0,0,10,0,0,0,0,0,0,0,73,110,116,101,114,32,115,116,114,101,97,109,32,116,105,109,101,115,116,97,109,112,32,100,105,115,99,111,110,116,105,110,117,105,116,121,32,37,108,108,100,44,32,110,101,119,32,111,102,102,115,101,116,61,32,37,108,108,100,10,0,0,0,0,118,100,112,97,117,0,0,0,77,105,115,116,121,82,111,115,101,0,0,0,0,0,0,0,112,104,97,115,101,95,115,104,105,102,116,0,0,0,0,0,117,109,104,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,0,0,0,76,67,76,32,40,76,111,115,115,76,101,115,115,32,67,111,100,101,99,32,76,105,98,114,97,114,121,41,32,77,83,90,72,0,0,0,0,0,0,0,66,111,111,116,121,32,66,97,115,115,0,0,0,0,0,0,32,40,99,108,101,97,110,32,101,102,102,101,99,116,115,41,0,0,0,0,0,0,0,0,76,97,118,102,53,53,46,49,50,46,49,48,49,0,0,0,67,111,109,112,114,101,115,115,115,101,100,32,102,114,97,109,101,32,37,115,32,116,108,101,110,61,37,100,32,100,108,101,110,61,37,108,100,10,0,0,116,105,109,101,115,116,97,109,112,0,0,0,0,0,0,0,32,32,37,115,10,0,0,0,67,111,114,114,101,99,116,105,110,103,32,115,116,97,114,116,32,116,105,109,101,32,98,121,32,37,108,108,100,10,0,0,121,117,118,97,52,52,52,112,49,54,108,101,0,0,0,0,77,105,110,116,67,114,101,97,109,0,0,0,0,0,0,0,33,34,118,97,108,105,100,32,101,108,101,109,101,110,116,32,115,105,122,101,34,0,0,0,115,101,116,32,115,119,114,32,114,101,115,97,109,112,108,105,110,103,32,102,105,108,116,101,114,32,115,105,122,101,0,0,117,109,104,0,0,0,0,0,109,115,122,104,0,0,0,0,83,121,109,112,104,111,110,121,0,0,0,0,0,0,0,0,109,111,110,111,98,0,0,0,32,40,118,105,115,117,97,108,32,105,109,112,97,105,114,101,100,41,0,0,0,0,0,0,169,116,111,111,0,0,0,0,83,107,105,112,112,105,110,103,32,37,115,32,73,68,51,118,50,32,102,114,97,109,101,32,37,115,46,10,0,0,0,0,37,72,58,37,77,58,37,83,0,0,0,0,0,0,0,0,115,99,97,108,101,0,0,0,85,115,101,32,45,104,32,116,111,32,103,101,116,32,102,117,108,108,32,104,101,108,112,32,111,114,44,32,101,118,101,110,32,98,101,116,116,101,114,44,32,114,117,110,32,39,109,97,110,32,37,115,39,10,0,0,70,105,108,116,101,114,32,37,115,10,0,0,0,0,0,0,27,91,37,100,59,51,37,100,109,0,0,0,0,0,0,0,116,97,110,0,0,0,0,0,100,101,109,117,120,101,114,32,45,62,32,105,115,116,95,105,110,100,101,120,58,37,100,32,116,121,112,101,58,37,115,32,110,101,120,116,95,100,116,115,58,37,115,32,110,101,120,116,95,100,116,115,95,116,105,109,101,58,37,115,32,110,101,120,116,95,112,116,115,58,37,115,32,110,101,120,116,95,112,116,115,95,116,105,109,101,58,37,115,32,112,107,116,95,112,116,115,58,37,115,32,112,107,116,95,112,116,115,95,116,105,109,101,58,37,115,32,112,107,116,95,100,116,115,58,37,115,32,112,107,116,95,100,116,115,95,116,105,109,101,58,37,115,32,111,102,102,58,37,115,32,111,102,102,95,116,105,109,101,58,37,115,10,0,0,0,0,0,68,69,77,85,88,69,82,95,78,79,84,95,70,79,85,78,68,0,0,0,0,0,0,0,121,117,118,97,52,52,52,112,49,54,98,101,0,0,0,0,77,105,100,110,105,103,104,116,66,108,117,101,0,0,0,0,102,114,111,110,116,32,99,101,110,116,101,114,0,0,0,0,102,105,108,116,101,114,95,115,105,122,101,0,0,0,0,0,108,105,98,115,119,115,99,97,108,101,47,115,119,115,99,97,108,101,46,99,0,0,0,0,104,101,120,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,0,0,0,79,114,100,101,114,101,100,32,100,105,116,104,101,114,32,105,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,105,110,32,102,117,108,108,32,99,104,114,111,109,97,32,105,110,116,101,114,112,111,108,97,116,105,111,110,32,102,111,114,32,100,101,115,116,105,110,97,116,105,111,110,32,102,111,114,109,97,116,32,39,37,115,39,10,0,0,0,0,0,0,0,83,105,101,114,114,97,32,86,77,68,32,118,105,100,101,111,0,0,0,0,0,0,0,0,83,111,110,97,116,97,0,0,97,114,101,97,0,0,0,0,32,40,104,101,97,114,105,110,103,32,105,109,112,97,105,114,101,100,41,0,0,0,0,0,99,111,109,112,111,115,101,114,0,0,0,0,0,0,0,0,67,84,84,83,32,105,110,118,97,108,105,100,10,0,0,0,101,110,99,114,121,112,116,101,100,32,97,110,100,32,99,111,109,112,114,101,115,115,101,100,0,0,0,0,0,0,0,0,111,99,104,0,0,0,0,0,115,101,116,32,116,104,101,32,105,110,112,117,116,32,116,115,32,115,99,97,108,101,0,0,107,120,32,33,61,32,102,95,116,97,98,108,101,123,104,105,103,104,44,108,111,119,125,91,48,93,46,32,68,101,114,105,118,101,100,32,102,114,101,113,117,101,110,99,121,32,116,97,98,108,101,115,32,119,101,114,101,32,110,111,116,32,114,101,103,101,110,101,114,97,116,101,100,46,10,0,0,0,0,0,85,110,107,110,111,119,110,32,102,105,108,116,101,114,32,39,37,115,39,46,10,0,0,0,84,89,80,69,95,70,73,76,58,32,73,110,112,117,116,32,98,117,102,102,101,114,32,101,120,104,97,117,115,116,101,100,32,98,101,102,111,114,101,32,69,78,68,32,101,108,101,109,101,110,116,32,102,111,117,110,100,10,0,0,0,0,0,0,116,111,111,32,109,97,110,121,32,99,104,97,110,110,101,108,115,58,32,103,111,116,32,37,105,44,32,110,101,101,100,32,37,105,32,111,114,32,102,101,119,101,114,10,0,0,0,0,102,114,97,109,101,95,108,101,110,32,111,118,101,114,102,108,111,119,10,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,109,111,100,101,32,99,111,117,110,116,58,32,37,100,10,0,0,0,0,0,101,110,99,111,100,101,114,32,45,62,32,116,121,112,101,58,118,105,100,101,111,32,112,107,116,95,112,116,115,58,37,115,32,112,107,116,95,112,116,115,95,116,105,109,101,58,37,115,32,112,107,116,95,100,116,115,58,37,115,32,112,107,116,95,100,116,115,95,116,105,109,101,58,37,115,10,0,0,0,0,121,117,118,97,52,50,50,112,49,54,108,101,0,0,0,0,77,101,100,105,117,109,86,105,111,108,101,116,82,101,100,0,84,104,105,115,32,65,86,67,111,100,101,99,67,111,110,116,101,120,116,32,119,97,115,32,97,108,108,111,99,97,116,101,100,32,102,111,114,32,37,115,44,32,98,117,116,32,37,115,32,112,97,115,115,101,100,32,116,111,32,97,118,99,111,100,101,99,95,111,112,101,110,50,40,41,10,0,0,0,0,0,80,67,77,32,99,104,97,110,110,101,108,115,32,111,117,116,32,111,102,32,98,111,117,110,100,115,10,0,0,0,0,0,115,101,108,101,99,116,32,105,109,112,114,111,118,101,100,45,101,45,119,101,105,103,104,116,101,100,32,110,111,105,115,101,32,115,104,97,112,105,110,103,32,100,105,116,104,101,114,0,104,101,120,0,0,0,0,0,118,109,100,118,105,100,101,111,0,0,0,0,0,0,0,0,67,104,97,109,98,101,114,32,77,117,115,105,99,0,0,0,32,40,102,111,114,99,101,100,41,0,0,0,0,0,0,0])
.concat([169,119,114,116,0,0,0,0,100,105,114,32,37,115,10,0,37,115,61,37,115,58,0,0,105,116,115,115,99,97,108,101,0,0,0,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,45,45,99,114,111,115,115,45,112,114,101,102,105,120,61,101,109,32,45,45,99,99,61,101,109,99,99,32,45,45,101,110,97,98,108,101,45,99,114,111,115,115,45,99,111,109,112,105,108,101,32,45,45,116,97,114,103,101,116,45,111,115,61,110,111,110,101,32,45,45,97,114,99,104,61,120,56,54,95,51,50,32,45,45,112,114,101,102,105,120,61,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,32,45,45,101,120,116,114,97,45,99,102,108,97,103,115,61,45,73,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,105,110,99,108,117,100,101,32,45,45,99,112,117,61,103,101,110,101,114,105,99,32,45,45,100,105,115,97,98,108,101,45,104,119,97,99,99,101,108,115,32,45,45,100,105,115,97,98,108,101,45,115,116,114,105,112,112,105,110,103,32,45,45,100,105,115,97,98,108,101,45,112,116,104,114,101,97,100,115,32,45,45,100,105,115,97,98,108,101,45,100,111,99,32,45,45,100,105,115,97,98,108,101,45,100,101,98,117,103,32,45,45,100,105,115,97,98,108,101,45,97,115,109,32,45,45,100,105,115,97,98,108,101,45,110,101,116,119,111,114,107,32,45,45,100,105,115,97,98,108,101,45,101,118,101,114,121,116,104,105,110,103,32,45,45,100,105,115,97,98,108,101,45,102,102,112,108,97,121,32,45,45,100,105,115,97,98,108,101,45,102,102,112,114,111,98,101,32,45,45,100,105,115,97,98,108,101,45,102,102,115,101,114,118,101,114,32,45,45,100,105,115,97,98,108,101,45,111,117,116,100,101,118,61,115,100,108,32,45,45,101,110,97,98,108,101,45,111,112,116,105,109,105,122,97,116,105,111,110,115,32,45,45,101,110,97,98,108,101,45,112,114,111,116,111,99,111,108,61,102,105,108,101,32,45,45,101,110,97,98,108,101,45,102,105,108,116,101,114,61,97,114,101,115,97,109,112,108,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,119,97,118,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,115,49,54,108,101,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,117,56,32,45,45,101,110,97,98,108,101,45,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,108,105,98,102,100,107,45,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,111,118,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,102,100,107,95,97,97,99,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,52,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,115,102,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,97,115,102,0,101,110,99,111,100,101,114,0,78,111,32,102,105,108,116,101,114,32,110,97,109,101,32,115,112,101,99,105,102,105,101,100,46,10,0,0,0,0,0,0,114,101,115,101,114,118,111,105,114,0,0,0,0,0,0,0,83,66,82,32,108,105,98,114,97,114,121,32,105,110,105,116,105,97,108,105,122,97,116,105,111,110,32,101,114,114,111,114,0,0,0,0,0,0,0,0,101,120,116,114,97,100,97,116,97,32,116,111,111,32,115,109,97,108,108,46,10,0,0,0,86,105,100,101,111,32,101,110,99,111,100,105,110,103,32,102,97,105,108,101,100,10,0,0,121,117,118,97,52,50,50,112,49,54,98,101,0,0,0,0,77,101,100,105,117,109,84,117,114,113,117,111,105,115,101,0,115,116,114,97,110,103,101,32,101,114,114,111,114,32,102,108,117,115,104,105,110,103,32,98,117,102,102,101,114,32,46,46,46,32,10,0,0,0,0,0,105,109,112,114,111,118,101,100,95,101,95,119,101,105,103,104,116,101,100,0,0,0,0,0,88,49,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,0,0,0,0,68,117,99,107,32,84,114,117,101,77,111,116,105,111,110,32,49,46,48,0,0,0,0,0,79,112,101,114,97,0,0,0,32,40,107,97,114,97,111,107,101,41,0,0,0,0,0,0,97,108,98,117,109,95,97,114,116,105,115,116,0,0,0,0,112,97,116,104,32,37,115,10,0,0,0,0,0,0,0,0,101,110,99,114,121,112,116,101,100,0,0,0,0,0,0,0,114,118,49,48,0,0,0,0,45,45,99,114,111,115,115,45,112,114,101,102,105,120,61,101,109,32,45,45,99,99,61,101,109,99,99,32,45,45,101,110,97,98,108,101,45,99,114,111,115,115,45,99,111,109,112,105,108,101,32,45,45,116,97,114,103,101,116,45,111,115,61,110,111,110,101,32,45,45,97,114,99,104,61,120,56,54,95,51,50,32,45,45,112,114,101,102,105,120,61,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,32,45,45,101,120,116,114,97,45,99,102,108,97,103,115,61,45,73,47,85,115,101,114,115,47,107,97,114,111,108,47,68,101,115,107,116,111,112,47,80,114,111,106,101,107,116,121,47,106,115,95,109,101,100,105,97,95,99,111,110,118,101,114,116,47,97,117,100,105,111,99,111,110,118,101,114,116,101,114,46,106,115,47,102,102,109,112,101,103,46,106,115,47,100,105,115,116,47,105,110,99,108,117,100,101,32,45,45,99,112,117,61,103,101,110,101,114,105,99,32,45,45,100,105,115,97,98,108,101,45,104,119,97,99,99,101,108,115,32,45,45,100,105,115,97,98,108,101,45,115,116,114,105,112,112,105,110,103,32,45,45,100,105,115,97,98,108,101,45,112,116,104,114,101,97,100,115,32,45,45,100,105,115,97,98,108,101,45,100,111,99,32,45,45,100,105,115,97,98,108,101,45,100,101,98,117,103,32,45,45,100,105,115,97,98,108,101,45,97,115,109,32,45,45,100,105,115,97,98,108,101,45,110,101,116,119,111,114,107,32,45,45,100,105,115,97,98,108,101,45,101,118,101,114,121,116,104,105,110,103,32,45,45,100,105,115,97,98,108,101,45,102,102,112,108,97,121,32,45,45,100,105,115,97,98,108,101,45,102,102,112,114,111,98,101,32,45,45,100,105,115,97,98,108,101,45,102,102,115,101,114,118,101,114,32,45,45,100,105,115,97,98,108,101,45,111,117,116,100,101,118,61,115,100,108,32,45,45,101,110,97,98,108,101,45,111,112,116,105,109,105,122,97,116,105,111,110,115,32,45,45,101,110,97,98,108,101,45,112,114,111,116,111,99,111,108,61,102,105,108,101,32,45,45,101,110,97,98,108,101,45,102,105,108,116,101,114,61,97,114,101,115,97,109,112,108,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,119,97,118,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,115,49,54,108,101,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,112,99,109,95,117,56,32,45,45,101,110,97,98,108,101,45,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,109,112,51,108,97,109,101,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,51,32,45,45,101,110,97,98,108,101,45,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,118,111,114,98,105,115,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,111,103,103,32,45,45,101,110,97,98,108,101,45,108,105,98,102,100,107,45,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,109,111,118,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,97,97,99,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,108,105,98,102,100,107,95,97,97,99,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,109,112,52,32,45,45,101,110,97,98,108,101,45,100,101,109,117,120,101,114,61,97,115,102,32,45,45,101,110,97,98,108,101,45,100,101,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,101,110,99,111,100,101,114,61,119,109,97,118,49,32,45,45,101,110,97,98,108,101,45,109,117,120,101,114,61,97,115,102,0,69,114,114,111,114,58,32,99,97,110,39,116,32,97,108,108,111,99,97,116,101,32,86,98,114,70,114,97,109,101,115,32,98,117,102,102,101,114,10,0,115,101,116,32,116,104,101,32,105,110,112,117,116,32,116,115,32,111,102,102,115,101,116,0,78,101,119,32,65,103,101,0,87,77,47,69,110,99,111,100,101,100,66,121,0,0,0,0,67,111,117,108,100,32,110,111,116,32,97,108,108,111,99,32,98,117,102,102,101,114,46,10,0,0,0,0,0,0,0,0,69,114,114,111,114,32,114,101,97,100,105,110,103,32,99,111,118,101,114,32,97,114,116,46,10,0,0,0,0,0,0,0,110,111,32,39,100,97,116,97,39,32,116,97,103,32,102,111,117,110,100,10,0,0,0,0,101,110,99,111,100,101,95,118,105,100,101,111,32,37,100,46,37,100,0,0,0,0,0,0,121,117,118,97,52,50,48,112,49,54,108,101,0,0,0,0,77,101,100,105,117,109,83,112,114,105,110,103,71,114,101,101,110,0,0,0,0,0,0,0,70,111,114,109,97,116,32,37,115,32,112,114,111,98,101,100,32,119,105,116,104,32,115,105,122,101,61,37,100,32,97,110,100,32,115,99,111,114,101,61,37,100,10,0,0,0,0,0,115,101,108,101,99,116,32,109,111,100,105,102,105,101,100,45,101,45,119,101,105,103,104,116,101,100,32,110,111,105,115,101,32,115,104,97,112,105,110,103,32,100,105,116,104,101,114,0,73,76,78,71,0,0,0,0,120,49,0,0,0,0,0,0,105,103,110,111,114,101,32,105,110,100,101,120,0,0,0,0,78,65,77,69,0,0,0,0,116,114,117,101,109,111,116,105,111,110,49,0,0,0,0,0,67,104,97,110,115,111,110,0,69,114,114,111,114,32,119,114,105,116,105,110,103,32,79,112,117,115,32,104,101,97,100,101,114,115,10,0,0,0,0,0,78,101,119,32,115,116,114,101,97,109,115,32,97,114,101,32,110,111,116,32,115,117,112,112,111,115,101,100,32,116,111,32,98,101,32,97,100,100,101,100,32,105,110,32,98,101,116,119,101,101,110,32,79,103,103,32,99,111,110,116,101,120,116,32,115,97,118,101,47,114,101,115,116,111,114,101,32,111,112,101,114,97,116,105,111,110,115,46,10,0,0,0,0,0,0,0,32,40,108,121,114,105,99,115,41,0,0,0,0,0,0,0,115,97,109,112,108,101,32,114,97,116,101,32,110,111,116,32,115,101,116,10,0,0,0,0,84,82,67,75,0,0,0,0,97,65,82,84,0,0,0,0,116,121,112,101,32,37,100,44,32,108,101,110,32,37,100,10,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,101,109,112,116,121,32,102,114,97,109,101,32,37,115,44,32,115,107,105,112,112,105,110,103,46,10,0,0,0,0,0,0,105,116,115,111,102,102,115,101,116,0,0,0,0,0,0,0,109,112,52,32,109,117,120,101,114,0,0,0,0,0,0,0,65,114,114,97,121,32,116,111,111,32,98,105,103,46,10,0,109,111,118,44,109,112,52,44,109,52,97,44,51,103,112,44,51,103,50,44,109,106,50,0,84,73,84,50,0,0,0,0,51,46,57,57,46,53,0,0,70,111,114,99,101,100,32,107,101,121,102,114,97,109,101,32,97,116,32,116,105,109,101,32,37,102,10,0,0,0,0,0,121,117,118,97,52,50,48,112,49,54,98,101,0,0,0,0,78,85,76,76,0,0,0,0,77,101,100,105,117,109,83,108,97,116,101,66,108,117,101,0,87,105,110,100,111,119,115,32,77,101,100,105,97,32,65,117,100,105,111,32,86,56,0,0,102,102,32,97,115,102,32,98,97,100,32,104,101,97,100,101,114,32,37,120,32,32,97,116,58,37,108,108,100,10,0,0,109,111,100,105,102,105,101,100,95,101,95,119,101,105,103,104,116,101,100,0,0,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,65,32,99,111,109,109,97,45,115,101,112,97,114,97,116,101,100,32,108,105,115,116,32,111,102,32,115,97,109,112,108,101,32,102,111,114,109,97,116,115,46,0,0,0,0,0,0,0,84,105,109,101,115,116,97,109,112,32,111,102,32,116,104,101,32,102,105,114,115,116,32,102,114,97,109,101,32,116,104,97,116,32,115,104,111,117,108,100,32,98,101,32,100,114,111,112,112,101,100,32,97,103,97,105,110,0,0,0,0,0,0,0,112,104,111,100,115,32,109,111,116,105,111,110,32,101,115,116,105,109,97,116,105,111,110,0,80,82,69,86,95,73,78,84,0,0,0,0,0,0,0,0,84,111,111,32,109,97,110,121,32,105,110,112,117,116,115,32,115,112,101,99,105,102,105,101,100,32,102,111,114,32,116,104,101,32,34,37,115,34,32,102,105,108,116,101,114,46,10,0,65,117,116,111,100,101,115,107,32,65,110,105,109,97,116,111,114,32,70,108,105,99,32,118,105,100,101,111,0,0,0,0,83,112,101,101,99,104,0,0,73,78,84,69,82,78,65,76,32,69,82,82,79,82,32,73,78,32,86,66,82,32,78,69,87,32,67,79,68,69,32,40,49,51,49,51,41,44,32,112,108,101,97,115,101,32,115,101,110,100,32,98,117,103,32,114,101,112,111,114,116,10,109,97,120,98,105,116,115,61,37,100,32,117,115,101,100,98,105,116,115,61,37,100,10,0,0,0,32,40,99,111,109,109,101,110,116,41,0,0,0,0,0,0,116,105,109,101,95,98,97,115,101,0,0,0,0,0,0,0,115,97,109,112,108,101,95,102,109,116,115,0,0,0,0,0,110,108,118,108,32,102,114,111,109,32,37,100,44,32,110,108,118,108,32,116,111,32,37,100,10,0,0,0,0,0,0,0,101,120,116,101,110,100,101,100,32,104,101,97,100,101,114,32,116,111,111,32,108,111,110,103,46,0,0,0,0,0,0,0,67,97,110,110,111,116,32,115,101,108,101,99,116,32,99,104,97,110,110,101,108,32,108,97,121,111,117,116,32,102,111,114,32,116,104,101,32,108,105,110,107,32,98,101,116,119,101,101,110,32,102,105,108,116,101,114,115,32,37,115,32,97,110,100,32,37,115,46,10,0,0,0,65,115,115,101,114,116,105,111,110,32,37,115,32,102,97,105,108,101,100,32,97,116,32,37,115,58,37,100,10,0,0,0,116,105,109,101,95,111,102,102,0,0,0,0,0,0,0,0,67,111,117,108,100,32,110,111,116,32,97,108,108,111,99,32,109,101,109,111,114,121,32,102,111,114,32,115,116,114,101,97,109,32,111,112,116,105,111,110,115,46,10,0,0,0,0,0,42,42,42,32,37,100,32,100,117,112,33,10,0,0,0,0,121,117,118,97,52,52,52,112,49,48,108,101,0,0,0,0,77,101,100,105,117,109,83,101,97,71,114,101,101,110,0,0,115,101,108,101,99,116,32,102,45,119,101,105,103,104,116,101,100,32,110,111,105,115,101,32,115,104,97,112,105,110,103,32,100,105,116,104,101,114,0,0,112,104,111,100,115,0,0,0,102,108,105,99,0,0,0,0,72,117,109,111,117,114,0,0,32,40,111,114,105,103,105,110,97,108,41,0,0,0,0,0,102,105,108,101,110,97,109,101,32,37,115,44,32,108,101,110,32,37,100,10,0,0,0,0,105,110,118,97,108,105,100,32,101,120,116,101,110,100,101,100,32,104,101,97,100,101,114,32,108,101,110,103,116,104,0,0,115,101,116,32,116,104,101,32,115,116,97,114,116,32,116,105,109,101,32,111,102,102,115,101,116,0,0,0,0,0,0,0,65,100,118,97,110,99,101,100,32,86,105,100,101,111,32,111,112,116,105,111,110,115,58,0,102,117,108,108,0,0,0,0,108,111,110,103,0,0,0,0,88,79,1,0,56,79,1,0,24,79,1,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,7,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,7,0,0,0,17,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,7,0,0,0,17,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,8,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,7,0,0,0,17,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,7,0,0,0,17,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,8,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,7,0,0,0,17,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,99,0,0,0,7,0,0,0,17,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,10,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,16,0,0,0,60,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,30,0,0,0,255,255,255,255,50,0,0,0,255,255,255,255,80,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,10,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,231,3,0,0,0,0,0,0,231,3,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,8,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,25,0,0,0,255,255,255,255,45,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,9,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,231,3,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,10,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,71,0,0,0,157,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,71,0,0,0,157,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,10,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,71,0,0,0,157,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,117,29,0,0,0,0,0,0,0,0,0,0,0,0,0,88,117,29,0,0,0,0,0,0,0,0,0,0,0,0,0,48,117,29,0,0,0,0,0,0,0,0,0,0,0,0,0,8,117,29,0,0,0,0,0,0,0,0,0,0,0,0,0,224,116,29,0,0,0,0,0,0,0,0,0,0,0,0,0,184,116,29,0,0,0,0,0,144,116,29,0,104,116,29,0,0,0,0,0,0,0,0,0,64,116,29,0,24,116,29,0,0,0,0,0,0,0,0,0,240,115,29,0,200,115,29,0,160,115,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,119,29,0,0,0,0,0,0,0,0,0,0,0,0,0,232,118,29,0,0,0,0,0,0,0,0,0,0,0,0,0,192,118,29,0,0,0,0,0,0,0,0,0,0,0,0,0,152,118,29,0,0,0,0,0,0,0,0,0,0,0,0,0,112,118,29,0,0,0,0,0,72,118,29,0,32,118,29,0,0,0,0,0,0,0,0,0,248,117,29,0,208,117,29,0,168,117,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,120,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,200,120,29,0,0,0,0,0,0,0,0,0,0,0,0,0,160,120,29,0,0,0,0,0,0,0,0,0,0,0,0,0,120,120,29,0,0,0,0,0,0,0,0,0,0,0,0,0,80,120,29,0,0,0,0,0,40,120,29,0,0,120,29,0,0,0,0,0,0,0,0,0,216,119,29,0,176,119,29,0,0,0,0,0,0,0,0,0,136,119,29,0,96,119,29,0,56,119,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,208,122,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,168,122,29,0,0,0,0,0,0,0,0,0,0,0,0,0,128,122,29,0,0,0,0,0,0,0,0,0,0,0,0,0,88,122,29,0,0,0,0,0,0,0,0,0,0,0,0,0,48,122,29,0,0,0,0,0,8,122,29,0,224,121,29,0,0,0,0,0,0,0,0,0,184,121,29,0,144,121,29,0,0,0,0,0,0,0,0,0,104,121,29,0,64,121,29,0,24,121,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,96,124,29,0,0,0,0,0,0,0,0,0,0,0,0,0,56,124,29,0,0,0,0,0,0,0,0,0,0,0,0,0,16,124,29,0,0,0,0,0,0,0,0,0,0,0,0,0,232,123,29,0,0,0,0,0,0,0,0,0,0,0,0,0,192,123,29,0,0,0,0,0,152,123,29,0,112,123,29,0,0,0,0,0,0,0,0,0,72,123,29,0,32,123,29,0,248,122,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,184,126,29,0,0,0,0,0,0,0,0,0,0,0,0,0,144,126,29,0,0,0,0,0,0,0,0,0,0,0,0,0,104,126,29,0,0,0,0,0,0,0,0,0,0,0,0,0,64,126,29,0,0,0,0,0,24,126,29,0,240,125,29,0,0,0,0,0,0,0,0,0,200,125,29,0,160,125,29,0,0,0,0,0,0,0,0,0,120,125,29,0,80,125,29,0,0,0,0,0,0,0,0,0,40,125,29,0,0,125,29,0,0,0,0,0,0,0,0,0,216,124,29,0,176,124,29,0,136,124,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,129,29,0,0,0,0,0,0,0,0,0,0,0,0,0,232,128,29,0,0,0,0,0,0,0,0,0,0,0,0,0,192,128,29,0,0,0,0,0,0,0,0,0,0,0,0,0,152,128,29,0,0,0,0,0,112,128,29,0,72,128,29,0,0,0,0,0,0,0,0,0,32,128,29,0,248,127,29,0,0,0,0,0,0,0,0,0,208,127,29,0,168,127,29,0,0,0,0,0,0,0,0,0,128,127,29,0,88,127,29,0,0,0,0,0,0,0,0,0,48,127,29,0,8,127,29,0,224,126,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,24,131,29,0,0,0,0,0,0,0,0,0,0,0,0,0,240,130,29,0,0,0,0,0,0,0,0,0,0,0,0,0,200,130,29,0,0,0,0,0,0,0,0,0,0,0,0,0,160,130,29,0,0,0,0,0,0,0,0,0,0,0,0,0,120,130,29,0,0,0,0,0,0,0,0,0,0,0,0,0,80,130,29,0,0,0,0,0,40,130,29,0,0,130,29,0,0,0,0,0,0,0,0,0,216,129,29,0,176,129,29,0,0,0,0,0,0,0,0,0,136,129,29,0,96,129,29,0,56,129,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,133,29,0,0,0,0,0,0,0,0,0,0,0,0,0,248,132,29,0,0,0,0,0,0,0,0,0,0,0,0,0,208,132,29,0,0,0,0,0,0,0,0,0,0,0,0,0,168,132,29,0,0,0,0,0,0,0,0,0,0,0,0,0,128,132,29,0,0,0,0,0,0,0,0,0,0,0,0,0,88,132,29,0,0,0,0,0,48,132,29,0,8,132,29,0,0,0,0,0,0,0,0,0,224,131,29,0,184,131,29,0,0,0,0,0,0,0,0,0,144,131,29,0,104,131,29,0,64,131,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,135,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,135,29,0,0,0,0,0,0,0,0,0,0,0,0,0,216,134,29,0,0,0,0,0,0,0,0,0,0,0,0,0,176,134,29,0,0,0,0,0,0,0,0,0,0,0,0,0,136,134,29,0,0,0,0,0,0,0,0,0,0,0,0,0,96,134,29,0,0,0,0,0,56,134,29,0,16,134,29,0,0,0,0,0,0,0,0,0,232,133,29,0,192,133,29,0,0,0,0,0,0,0,0,0,152,133,29,0,112,133,29,0,72,133,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,184,136,29,0,0,0,0,0,0,0,0,0,0,0,0,0,144,136,29,0,0,0,0,0,0,0,0,0,0,0,0,0,104,136,29,0,0,0,0,0,0,0,0,0,0,0,0,0,64,136,29,0,0,0,0,0,0,0,0,0,0,0,0,0,24,136,29,0,0,0,0,0,240,135,29,0,200,135,29,0,0,0,0,0,0,0,0,0,160,135,29,0,120,135,29,0,80,135,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,138,29,0,0,0,0,0,0,0,0,0,0,0,0,0,32,138,29,0,0,0,0,0,0,0,0,0,0,0,0,0,248,137,29,0,0,0,0,0,0,0,0,0,0,0,0,0,208,137,29,0,0,0,0,0,0,0,0,0,0,0,0,0,168,137,29,0,0,0,0,0,128,137,29,0,88,137,29,0,0,0,0,0,0,0,0,0,48,137,29,0,8,137,29,0,224,136,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,216,139,29,0,0,0,0,0,0,0,0,0,0,0,0,0,176,139,29,0,0,0,0,0,0,0,0,0,0,0,0,0,136,139,29,0,0,0,0,0,0,0,0,0,0,0,0,0,96,139,29,0,0,0,0,0,0,0,0,0,0,0,0,0,56,139,29,0,0,0,0,0,16,139,29,0,232,138,29,0,0,0,0,0,0,0,0,0,192,138,29,0,152,138,29,0,112,138,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,104,141,29,0,0,0,0,0,0,0,0,0,0,0,0,0,64,141,29,0,0,0,0,0,0,0,0,0,0,0,0,0,24,141,29,0,0,0,0,0,0,0,0,0,0,0,0,0,240,140,29,0,0,0,0,0,0,0,0,0,0,0,0,0,200,140,29,0,0,0,0,0,160,140,29,0,120,140,29,0,0,0,0,0,0,0,0,0,80,140,29,0,40,140,29,0,0,140,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,142,29,0,0,0,0,0,0,0,0,0,0,0,0,0,208,142,29,0,0,0,0,0,0,0,0,0,0,0,0,0,168,142,29,0,0,0,0,0,0,0,0,0,0,0,0,0,128,142,29,0,0,0,0,0,0,0,0,0,0,0,0,0,88,142,29,0,0,0,0,0,48,142,29,0,8,142,29,0,0,0,0,0,0,0,0,0,224,141,29,0,184,141,29,0,144,141,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,184,171,29,0,0,0,0,0,0,0,0,0,0,0,0,0,144,171,29,0,0,0,0,0,0,0,0,0,0,0,0,0,104,171,29,0,0,0,0,0,0,0,0,0,0,0,0,0,64,171,29,0,0,0,0,0,0,0,0,0,0,0,0,0,24,171,29,0,0,0,0,0,240,170,29,0,200,170,29,0,0,0,0,0,0,0,0,0,160,170,29,0,120,170,29,0,0,0,0,0,0,0,0,0,80,170,29,0,40,170,29,0,0,170,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,192,29,0,0,0,0,0,0,0,0,0,0,0,0,0,208,192,29,0,0,0,0,0,0,0,0,0,0,0,0,0,168,192,29,0,0,0,0,0,0,0,0,0,0,0,0,0,128,192,29,0,0,0,0,0,0,0,0,0,0,0,0,0,88,192,29,0,0,0,0,0,48,192,29,0,8,192,29,0,0,0,0,0,0,0,0,0,224,191,29,0,184,191,29,0,0,0,0,0,0,0,0,0,144,191,29,0,104,191,29,0,64,191,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,184,196,29,0,0,0,0,0,0,0,0,0,0,0,0,0,144,196,29,0,0,0,0,0,0,0,0,0,0,0,0,0,104,196,29,0,0,0,0,0,0,0,0,0,0,0,0,0,64,196,29,0,0,0,0,0,0,0,0,0,0,0,0,0,24,196,29,0,0,0,0,0,240,195,29,0,200,195,29,0,0,0,0,0,0,0,0,0,160,195,29,0,120,195,29,0,0,0,0,0,0,0,0,0,80,195,29,0,40,195,29,0,0,195,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,152,173,29,0,0,0,0,0,0,0,0,0,0,0,0,0,112,173,29,0,0,0,0,0,0,0,0,0,0,0,0,0,72,173,29,0,0,0,0,0,0,0,0,0,0,0,0,0,32,173,29,0,0,0,0,0,0,0,0,0,0,0,0,0,248,172,29,0,0,0,0,0,208,172,29,0,168,172,29,0,0,0,0,0,0,0,0,0,128,172,29,0,88,172,29,0,0,0,0,0,0,0,0,0,48,172,29,0,8,172,29,0,224,171,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,175,29,0,0,0,0,0,0,0,0,0,0,0,0,0,200,175,29,0,0,0,0,0,0,0,0,0,0,0,0,0,160,175,29,0,0,0,0,0,0,0,0,0,0,0,0,0,120,175,29,0,0,0,0,0,80,175,29,0,40,175,29,0,0,0,0,0,0,0,0,0,0,175,29,0,216,174,29,0,0,0,0,0,0,0,0,0,176,174,29,0,136,174,29,0,0,0,0,0,0,0,0,0,96,174,29,0,56,174,29,0,0,0,0,0,0,0,0,0,16,174,29,0,232,173,29,0,192,173,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,72,178,29,0,0,0,0,0,0,0,0,0,0,0,0,0,32,178,29,0,0,0,0,0,0,0,0,0,0,0,0,0,248,177,29,0,0,0,0,0,0,0,0,0,0,0,0,0,208,177,29,0,0,0,0,0,168,177,29,0,128,177,29,0,0,0,0,0,0,0,0,0,88,177,29,0,48,177,29,0,0,0,0,0,0,0,0,0,8,177,29,0,224,176,29,0,0,0,0,0,0,0,0,0,184,176,29,0,144,176,29,0,0,0,0,0,0,0,0,0,104,176,29,0,64,176,29,0,24,176,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,160,180,29,0,0,0,0,0,0,0,0,0,0,0,0,0,120,180,29,0,0,0,0,0,0,0,0,0,0,0,0,0,80,180,29,0,0,0,0,0,0,0,0,0,0,0,0,0,40,180,29,0,0,0,0,0,0,180,29,0,216,179,29,0,0,0,0,0,0,0,0,0,176,179,29,0,136,179,29,0,0,0,0,0,0,0,0,0,96,179,29,0,56,179,29,0,0,0,0,0,0,0,0,0,16,179,29,0,232,178,29,0,0,0,0,0,0,0,0,0,192,178,29,0,152,178,29,0,112,178,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,182,29,0,0,0,0,0,0,0,0,0,0,0,0,0,208,182,29,0,0,0,0,0,0,0,0,0,0,0,0,0,168,182,29,0,0,0,0,0,0,0,0,0,0,0,0,0,128,182,29,0,0,0,0,0,88,182,29,0,48,182,29,0,0,0,0,0,0,0,0,0,8,182,29,0,224,181,29,0,0,0,0,0,0,0,0,0,184,181,29,0,144,181,29,0,0,0,0,0,0,0,0,0,104,181,29,0,64,181,29,0,0,0,0,0,0,0,0,0,24,181,29,0,240,180,29,0,200,180,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,185,29,0,0,0,0,0,0,0,0,0,0,0,0,0,216,184,29,0,0,0,0,0,0,0,0,0,0,0,0,0,176,184,29,0,0,0,0,0,0,0,0,0,0,0,0,0,136,184,29,0,0,0,0,0,0,0,0,0,0,0,0,0,96,184,29,0,0,0,0,0,0,0,0,0,0,0,0,0,56,184,29,0,0,0,0,0,16,184,29,0,232,183,29,0,0,0,0,0,0,0,0,0,192,183,29,0,152,183,29,0,0,0,0,0,0,0,0,0,112,183,29,0,72,183,29,0,32,183,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,187,29,0,0,0,0,0,0,0,0,0,0,0,0,0,224,186,29,0,0,0,0,0,0,0,0,0,0,0,0,0,184,186,29,0,0,0,0,0,0,0,0,0,0,0,0,0,144,186,29,0,0,0,0,0,0,0,0,0,0,0,0,0,104,186,29,0,0,0,0,0,0,0,0,0,0,0,0,0,64,186,29,0,0,0,0,0,24,186,29,0,240,185,29,0,0,0,0,0,0,0,0,0,200,185,29,0,160,185,29,0,0,0,0,0,0,0,0,0,120,185,29,0,80,185,29,0,40,185,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,189,29,0,0,0,0,0,0,0,0,0,0,0,0,0,232,188,29,0,0,0,0,0,0,0,0,0,0,0,0,0,192,188,29,0,0,0,0,0,0,0,0,0,0,0,0,0,152,188,29,0,0,0,0,0,0,0,0,0,0,0,0,0,112,188,29,0,0,0,0,0,0,0,0,0,0,0,0,0,72,188,29,0,0,0,0,0,32,188,29,0,248,187,29,0,0,0,0,0,0,0,0,0,208,187,29,0,168,187,29,0,0,0,0,0,0,0,0,0,128,187,29,0,88,187,29,0,48,187,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,24,191,29,0,0,0,0,0,0,0,0,0,0,0,0,0,240,190,29,0,0,0,0,0,0,0,0,0,0,0,0,0,200,190,29,0,0,0,0,0,0,0,0,0,0,0,0,0,160,190,29,0,0,0,0,0,0,0,0,0,0,0,0,0,120,190,29,0,0,0,0,0,0,0,0,0,0,0,0,0,80,190,29,0,0,0,0,0,40,190,29,0,0,190,29,0,0,0,0,0,0,0,0,0,216,189,29,0,176,189,29,0,0,0,0,0,0,0,0,0,136,189,29,0,96,189,29,0,56,189,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,216,194,29,0,0,0,0,0,0,0,0,0,0,0,0,0,176,194,29,0,0,0,0,0,0,0,0,0,0,0,0,0,136,194,29,0,0,0,0,0,0,0,0,0,0,0,0,0,96,194,29,0,0,0,0,0,0,0,0,0,0,0,0,0,56,194,29,0,0,0,0,0,16,194,29,0,232,193,29,0,0,0,0,0,0,0,0,0,192,193,29,0,152,193,29,0,0,0,0,0,0,0,0,0,112,193,29,0,72,193,29,0,32,193,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,152,198,29,0,0,0,0,0,0,0,0,0,0,0,0,0,112,198,29,0,0,0,0,0,0,0,0,0,0,0,0,0,72,198,29,0,0,0,0,0,0,0,0,0,0,0,0,0,32,198,29,0,0,0,0,0,0,0,0,0,0,0,0,0,248,197,29,0,0,0,0,0,208,197,29,0,168,197,29,0,0,0,0,0,0,0,0,0,128,197,29,0,88,197,29,0,0,0,0,0,0,0,0,0,48,197,29,0,8,197,29,0,224,196,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,216,144,29,0,0,0,0,0,0,0,0,0,176,144,29,0,136,144,29,0,0,0,0,0,0,0,0,0,96,144,29,0,56,144,29,0,0,0,0,0,0,0,0,0,16,144,29,0,232,143,29,0,0,0,0,0,0,0,0,0,192,143,29,0,152,143,29,0,232,143,29,0,0,0,0,0,112,143,29,0,72,143,29,0,32,143,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,80,145,29,0,40,145,29,0,0,0,0,0,0,0,0,0,0,145,29,0,72,143,29,0,32,143,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,136,154,29,0,96,154,29,0,0,0,0,0,0,0,0,0,56,154,29,0,48,152,29,0,8,152,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,157,29,0,8,157,29,0,0,0,0,0,0,0,0,0,224,156,29,0,216,154,29,0,176,154,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,216,159,29,0,176,159,29,0,0,0,0,0,0,0,0,0,136,159,29,0,128,157,29,0,88,157,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,162,29,0,88,162,29,0,0,0,0,0,0,0,0,0,48,162,29,0,40,160,29,0,0,160,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,40,165,29,0,0,165,29,0,0,0,0,0,0,0,0,0,216,164,29,0,208,162,29,0,168,162,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,167,29,0,88,167,29,0,0,0,0,0,0,0,0,0,48,167,29,0,120,165,29,0,80,165,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,216,169,29,0,176,169,29,0,0,0,0,0,0,0,0,0,136,169,29,0,208,167,29,0,168,167,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,147,29,0,0,0,0,0,0,0,0,0,88,147,29,0,0,0,0,0,0,0,0,0,48,147,29,0,8,147,29,0,0,0,0,0,0,0,0,0,224,146,29,0,184,146,29,0,0,0,0,0,0,0,0,0,144,146,29,0,104,146,29,0,0,0,0,0,0,0,0,0,64,146,29,0,24,146,29,0,104,146,29,0,0,0,0,0,240,145,29,0,200,145,29,0,160,145,29,0,120,145,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,176,149,29,0,0,0,0,0,0,0,0,0,136,149,29,0,0,0,0,0,0,0,0,0,96,149,29,0,56,149,29,0,0,0,0,0,0,0,0,0,16,149,29,0,232,148,29,0,0,0,0,0,0,0,0,0,192,148,29,0,152,148,29,0,0,0,0,0,0,0,0,0,112,148,29,0,72,148,29,0,152,148,29,0,0,0,0,0,32,148,29,0,248,147,29,0,208,147,29,0,168,147,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,151,29,0,0,0,0,0,0,0,0,0,184,151,29,0,0,0,0,0,0,0,0,0,144,151,29,0,104,151,29,0,0,0,0,0,0,0,0,0,64,151,29,0,24,151,29,0,0,0,0,0,0,0,0,0,240,150,29,0,200,150,29,0,0,0,0,0,0,0,0,0,160,150,29,0,120,150,29,0,200,150,29,0,0,0,0,0,80,150,29,0,40,150,29,0,0,150,29,0,216,149,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,154,29,0,0,0,0,0,0,0,0,0,232,153,29,0,0,0,0,0,0,0,0,0,192,153,29,0,152,153,29,0,0,0,0,0,0,0,0,0,112,153,29,0,72,153,29,0,0,0,0,0,0,0,0,0,32,153,29,0,248,152,29,0,0,0,0,0,0,0,0,0,208,152,29,0,168,152,29,0,248,152,29,0,0,0,0,0,128,152,29,0,88,152,29,0,48,152,29,0,8,152,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,184,156,29,0,0,0,0,0,0,0,0,0,144,156,29,0,0,0,0,0,0,0,0,0,104,156,29,0,64,156,29,0,0,0,0,0,0,0,0,0,24,156,29,0,240,155,29,0,0,0,0,0,0,0,0,0])
.concat([200,155,29,0,160,155,29,0,0,0,0,0,0,0,0,0,120,155,29,0,80,155,29,0,160,155,29,0,0,0,0,0,40,155,29,0,0,155,29,0,216,154,29,0,176,154,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,96,159,29,0,0,0,0,0,0,0,0,0,56,159,29,0,0,0,0,0,0,0,0,0,16,159,29,0,232,158,29,0,0,0,0,0,0,0,0,0,192,158,29,0,152,158,29,0,0,0,0,0,0,0,0,0,112,158,29,0,72,158,29,0,0,0,0,0,0,0,0,0,32,158,29,0,248,157,29,0,72,158,29,0,0,0,0,0,208,157,29,0,168,157,29,0,128,157,29,0,88,157,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,162,29,0,0,0,0,0,0,0,0,0,224,161,29,0,0,0,0,0,0,0,0,0,184,161,29,0,144,161,29,0,0,0,0,0,0,0,0,0,104,161,29,0,64,161,29,0,0,0,0,0,0,0,0,0,24,161,29,0,240,160,29,0,0,0,0,0,0,0,0,0,200,160,29,0,160,160,29,0,240,160,29,0,0,0,0,0,120,160,29,0,80,160,29,0,40,160,29,0,0,160,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,176,164,29,0,0,0,0,0,0,0,0,0,136,164,29,0,0,0,0,0,0,0,0,0,96,164,29,0,56,164,29,0,0,0,0,0,0,0,0,0,16,164,29,0,232,163,29,0,0,0,0,0,0,0,0,0,192,163,29,0,152,163,29,0,0,0,0,0,0,0,0,0,112,163,29,0,72,163,29,0,152,163,29,0,0,0,0,0,32,163,29,0,248,162,29,0,208,162,29,0,168,162,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,167,29,0,0,0,0,0,0,0,0,0,224,166,29,0,184,166,29,0,0,0,0,0,0,0,0,0,144,166,29,0,104,166,29,0,0,0,0,0,0,0,0,0,64,166,29,0,24,166,29,0,0,0,0,0,0,0,0,0,240,165,29,0,200,165,29,0,24,166,29,0,0,0,0,0,160,165,29,0,120,165,29,0,80,165,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,96,169,29,0,0,0,0,0,0,0,0,0,56,169,29,0,16,169,29,0,0,0,0,0,0,0,0,0,232,168,29,0,192,168,29,0,0,0,0,0,0,0,0,0,152,168,29,0,112,168,29,0,0,0,0,0,0,0,0,0,72,168,29,0,32,168,29,0,112,168,29,0,0,0,0,0,248,167,29,0,208,167,29,0,168,167,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,200,29,0,0,0,0,0,0,0,0,0,0,0,0,0,200,200,29,0,0,0,0,0,0,0,0,0,0,0,0,0,160,200,29,0,0,0,0,0,0,0,0,0,0,0,0,0,120,200,29,0,0,0,0,0,80,200,29,0,40,200,29,0,0,0,0,0,0,0,0,0,0,200,29,0,216,199,29,0,0,0,0,0,0,0,0,0,176,199,29,0,136,199,29,0,0,0,0,0,0,0,0,0,96,199,29,0,56,199,29,0,0,0,0,0,0,0,0,0,16,199,29,0,232,198,29,0,192,198,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,202,29,0,0,0,0,0,0,0,0,0,0,0,0,0,208,202,29,0,0,0,0,0,0,0,0,0,0,0,0,0,168,202,29,0,0,0,0,0,0,0,0,0,0,0,0,0,128,202,29,0,0,0,0,0,0,0,0,0,0,0,0,0,88,202,29,0,0,0,0,0,0,0,0,0,0,0,0,0,48,202,29,0,0,0,0,0,8,202,29,0,224,201,29,0,0,0,0,0,0,0,0,0,184,201,29,0,144,201,29,0,0,0,0,0,0,0,0,0,104,201,29,0,64,201,29,0,24,201,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,136,204,29,0,0,0,0,0,0,0,0,0,0,0,0,0,96,204,29,0,0,0,0,0,0,0,0,0,0,0,0,0,56,204,29,0,0,0,0,0,0,0,0,0,0,0,0,0,16,204,29,0,0,0,0,0,0,0,0,0,0,0,0,0,232,203,29,0,0,0,0,0,192,203,29,0,152,203,29,0,0,0,0,0,0,0,0,0,112,203,29,0,72,203,29,0,32,203,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,206,29,0,0,0,0,0,0,0,0,0,0,0,0,0,184,206,29,0,0,0,0,0,0,0,0,0,0,0,0,0,144,206,29,0,0,0,0,0,0,0,0,0,0,0,0,0,104,206,29,0,0,0,0,0,64,206,29,0,24,206,29,0,0,0,0,0,0,0,0,0,240,205,29,0,200,205,29,0,0,0,0,0,0,0,0,0,160,205,29,0,120,205,29,0,0,0,0,0,0,0,0,0,80,205,29,0,40,205,29,0,0,0,0,0,0,0,0,0,0,205,29,0,216,204,29,0,176,204,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,208,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,152,208,29,0,0,0,0,0,0,0,0,0,0,0,0,0,112,208,29,0,0,0,0,0,0,0,0,0,0,0,0,0,72,208,29,0,0,0,0,0,0,0,0,0,0,0,0,0,32,208,29,0,0,0,0,0,248,207,29,0,208,207,29,0,0,0,0,0,0,0,0,0,168,207,29,0,128,207,29,0,0,0,0,0,0,0,0,0,88,207,29,0,48,207,29,0,8,207,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,160,210,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,120,210,29,0,0,0,0,0,0,0,0,0,0,0,0,0,80,210,29,0,0,0,0,0,0,0,0,0,0,0,0,0,40,210,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,210,29,0,0,0,0,0,216,209,29,0,176,209,29,0,0,0,0,0,0,0,0,0,136,209,29,0,96,209,29,0,0,0,0,0,0,0,0,0,56,209,29,0,16,209,29,0,232,208,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,32,0,0,0,168,205,26,0,200,239,28,0,200,239,28,0,56,16,27,0,56,16,27,0,1,0,0,0,0,0,0,0,32,0,0,0,216,227,26,0,240,239,28,0,240,239,28,0,248,16,27,0,248,16,27,0,2,0,0,0,0,0,0,0,32,0,0,0,192,216,26,0,24,240,28,0,24,240,28,0,184,17,27,0,184,17,27,0,2,0,0,0,0,0,0,0,32,0,0,0,192,216,26,0,64,240,28,0,64,240,28,0,120,18,27,0,120,18,27,0,1,0,0,0,0,0,0,0,32,0,0,0,216,227,26,0,104,240,28,0,104,240,28,0,56,19,27,0,56,19,27,0,1,0,0,0,0,0,0,0,32,0,0,0,216,227,26,0,144,240,28,0,144,240,28,0,56,19,27,0,56,19,27,0,1,0,0,0,0,0,0,0,16,0,0,0,32,5,27,0,184,240,28,0,184,240,28,0,248,19,27,0,248,19,27,0,1,0,0,0,0,0,0,0,32,0,0,0,32,5,27,0,224,240,28,0,224,240,28,0,248,19,27,0,248,19,27,0,1,0,0,0,0,0,0,0,16,0,0,0,32,5,27,0,8,241,28,0,8,241,28,0,184,20,27,0,184,20,27,0,1,0,0,0,0,0,0,0,32,0,0,0,32,5,27,0,48,241,28,0,48,241,28,0,184,20,27,0,184,20,27,0,1,0,0,0,0,0,0,0,16,0,0,0,168,205,26,0,88,241,28,0,88,241,28,0,120,21,27,0,120,21,27,0,1,0,0,0,0,0,0,0,32,0,0,0,168,205,26,0,128,241,28,0,128,241,28,0,120,21,27,0,120,21,27,0,1,0,0,0,0,0,0,0,16,0,0,0,168,205,26,0,168,241,28,0,168,241,28,0,56,22,27,0,56,22,27,0,1,0,0,0,0,0,0,0,32,0,0,0,168,205,26,0,208,241,28,0,208,241,28,0,56,22,27,0,56,22,27,0,1,0,0,0,0,0,0,0,16,0,0,0,168,205,26,0,248,241,28,0,248,241,28,0,248,22,27,0,248,22,27,0,1,0,0,0,0,0,0,0,32,0,0,0,168,205,26,0,32,242,28,0,32,242,28,0,248,22,27,0,248,22,27,0,1,0,0,0,0,0,0,0,16,0,0,0,216,227,26,0,72,242,28,0,72,242,28,0,184,23,27,0,184,23,27,0,1,0,0,0,0,0,0,0,32,0,0,0,216,227,26,0,112,242,28,0,112,242,28,0,184,23,27,0,184,23,27,0,1,0,0,0,0,0,0,0,16,0,0,0,216,227,26,0,152,242,28,0,152,242,28,0,120,24,27,0,120,24,27,0,1,0,0,0,0,0,0,0,32,0,0,0,216,227,26,0,192,242,28,0,192,242,28,0,120,24,27,0,120,24,27,0,1,0,0,0,0,0,0,0,16,0,0,0,216,227,26,0,232,242,28,0,232,242,28,0,56,25,27,0,56,25,27,0,1,0,0,0,0,0,0,0,32,0,0,0,216,227,26,0,16,243,28,0,16,243,28,0,56,25,27,0,56,25,27,0,1,0,0,0,0,0,0,0,16,0,0,0,216,227,26,0,56,243,28,0,56,243,28,0,248,25,27,0,248,25,27,0,1,0,0,0,0,0,0,0,32,0,0,0,216,227,26,0,96,243,28,0,96,243,28,0,248,25,27,0,248,25,27,0,1,0,0,0,0,0,0,0,16,0,0,0,216,227,26,0,136,243,28,0,136,243,28,0,184,26,27,0,184,26,27,0,1,0,0,0,0,0,0,0,32,0,0,0,216,227,26,0,176,243,28,0,176,243,28,0,184,26,27,0,184,26,27,0,2,0,0,0,0,0,0,0,32,0,0,0,240,238,26,0,216,248,28,0,136,248,28,0,184,29,27,0,120,27,27,0,2,0,0,0,0,0,0,0,32,0,0,0,240,238,26,0,0,249,28,0,176,248,28,0,184,29,27,0,120,27,27,0,2,0,0,0,0,0,0,0,16,0,0,0,8,250,26,0,40,249,28,0,40,249,28,0,120,30,27,0,120,30,27,0,2,0,0,0,0,0,0,0,32,0,0,0,8,250,26,0,80,249,28,0,80,249,28,0,120,30,27,0,120,30,27,0,2,0,0,0,0,0,0,0,16,0,0,0,8,250,26,0,120,249,28,0,120,249,28,0,56,31,27,0,56,31,27,0,2,0,0,0,0,0,0,0,32,0,0,0,8,250,26,0,160,249,28,0,160,249,28,0,56,31,27,0,56,31,27,0,2,0,0,0,0,0,0,0,16,0,0,0,8,250,26,0,200,249,28,0,200,249,28,0,248,31,27,0,248,31,27,0,2,0,0,0,0,0,0,0,32,0,0,0,8,250,26,0,240,249,28,0,240,249,28,0,248,31,27,0,248,31,27,0,2,0,0,0,0,0,0,0,16,0,0,0,8,250,26,0,24,250,28,0,24,250,28,0,184,32,27,0,184,32,27,0,2,0,0,0,0,0,0,0,32,0,0,0,8,250,26,0,64,250,28,0,64,250,28,0,184,32,27,0,184,32,27,0,2,0,0,0,0,0,0,0,16,0,0,0,192,216,26,0,104,250,28,0,104,250,28,0,120,33,27,0,120,33,27,0,2,0,0,0,0,0,0,0,32,0,0,0,192,216,26,0,144,250,28,0,144,250,28,0,120,33,27,0,120,33,27,0,2,0,0,0,0,0,0,0,16,0,0,0,192,216,26,0,184,250,28,0,184,250,28,0,56,34,27,0,56,34,27,0,2,0,0,0,0,0,0,0,32,0,0,0,192,216,26,0,224,250,28,0,224,250,28,0,56,34,27,0,56,34,27,0,2,0,0,0,0,0,0,0,16,0,0,0,192,216,26,0,8,251,28,0,8,251,28,0,248,34,27,0,248,34,27,0,2,0,0,0,0,0,0,0,32,0,0,0,192,216,26,0,48,251,28,0,48,251,28,0,248,34,27,0,248,34,27,0,2,0,0,0,0,0,0,0,16,0,0,0,192,216,26,0,88,251,28,0,88,251,28,0,184,35,27,0,184,35,27,0,2,0,0,0,0,0,0,0,32,0,0,0,192,216,26,0,128,251,28,0,128,251,28,0,184,35,27,0,184,35,27,0,2,0,0,0,0,0,0,0,16,0,0,0,240,238,26,0,248,251,28,0,168,251,28,0,120,36,27,0,56,28,27,0,2,0,0,0,0,0,0,0,32,0,0,0,240,238,26,0,32,252,28,0,208,251,28,0,120,36,27,0,56,28,27,0,2,0,0,0,0,0,0,0,16,0,0,0,240,238,26,0,152,252,28,0,72,252,28,0,56,37,27,0,248,28,27,0,2,0,0,0,0,0,0,0,32,0,0,0,240,238,26,0,192,252,28,0,112,252,28,0,56,37,27,0,248,28,27,0,2,0,0,0,0,0,0,0,30,0,0,0,72,161,26,0,216,243,28,0,216,243,28,0,248,37,27,0,248,37,27,0,2,0,0,0,0,0,0,0,30,0,0,0,72,161,26,0,0,244,28,0,0,244,28,0,248,37,27,0,248,37,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,40,244,28,0,40,244,28,0,184,38,27,0,184,38,27,0,2,0,0,0,0,0,0,0,15,0,0,0,120,183,26,0,80,244,28,0,80,244,28,0,184,44,27,0,184,44,27,0,2,0,0,0,0,0,0,0,30,0,0,0,120,183,26,0,120,244,28,0,120,244,28,0,184,44,27,0,184,44,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,144,245,28,0,144,245,28,0,120,39,27,0,120,39,27,0,2,0,0,0,0,0,0,0,15,0,0,0,120,183,26,0,160,244,28,0,160,244,28,0,120,45,27,0,120,45,27,0,2,0,0,0,0,0,0,0,30,0,0,0,120,183,26,0,200,244,28,0,200,244,28,0,120,45,27,0,120,45,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,144,245,28,0,144,245,28,0,120,39,27,0,120,39,27,0,2,0,0,0,0,0,0,0,15,0,0,0,120,183,26,0,240,244,28,0,240,244,28,0,56,46,27,0,56,46,27,0,2,0,0,0,0,0,0,0,30,0,0,0,120,183,26,0,24,245,28,0,24,245,28,0,56,46,27,0,56,46,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,144,245,28,0,144,245,28,0,120,39,27,0,120,39,27,0,2,0,0,0,0,0,0,0,15,0,0,0,120,183,26,0,64,245,28,0,64,245,28,0,248,46,27,0,248,46,27,0,2,0,0,0,0,0,0,0,30,0,0,0,120,183,26,0,104,245,28,0,104,245,28,0,248,46,27,0,248,46,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,144,245,28,0,144,245,28,0,120,39,27,0,120,39,27,0,2,0,0,0,0,0,0,0,15,0,0,0,120,183,26,0,184,245,28,0,184,245,28,0,184,47,27,0,184,47,27,0,2,0,0,0,0,0,0,0,30,0,0,0,120,183,26,0,224,245,28,0,224,245,28,0,184,47,27,0,184,47,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,8,246,28,0,8,246,28,0,56,40,27,0,56,40,27,0,2,0,0,0,0,0,0,0,15,0,0,0,144,194,26,0,48,246,28,0,48,246,28,0,120,48,27,0,120,48,27,0,2,0,0,0,0,0,0,0,30,0,0,0,144,194,26,0,88,246,28,0,88,246,28,0,120,48,27,0,120,48,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,128,246,28,0,128,246,28,0,248,40,27,0,248,40,27,0,2,0,0,0,0,0,0,0,15,0,0,0,144,194,26,0,168,246,28,0,168,246,28,0,56,49,27,0,56,49,27,0,2,0,0,0,0,0,0,0,30,0,0,0,144,194,26,0,208,246,28,0,208,246,28,0,56,49,27,0,56,49,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,248,246,28,0,248,246,28,0,184,41,27,0,184,41,27,0,2,0,0,0,0,0,0,0,15,0,0,0,144,194,26,0,32,247,28,0,32,247,28,0,248,49,27,0,248,49,27,0,2,0,0,0,0,0,0,0,30,0,0,0,144,194,26,0,72,247,28,0,72,247,28,0,248,49,27,0,248,49,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,112,247,28,0,112,247,28,0,120,42,27,0,120,42,27,0,2,0,0,0,0,0,0,0,15,0,0,0,72,161,26,0,152,247,28,0,152,247,28,0,184,50,27,0,184,50,27,0,2,0,0,0,0,0,0,0,30,0,0,0,72,161,26,0,192,247,28,0,192,247,28,0,184,50,27,0,184,50,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,232,247,28,0,232,247,28,0,56,43,27,0,56,43,27,0,2,0,0,0,0,0,0,0,15,0,0,0,72,161,26,0,16,248,28,0,16,248,28,0,120,51,27,0,120,51,27,0,2,0,0,0,0,0,0,0,30,0,0,0,72,161,26,0,56,248,28,0,56,248,28,0,120,51,27,0,120,51,27,0,1,0,0,0,2,0,0,0,6,0,0,0,96,172,26,0,96,248,28,0,96,248,28,0,248,43,27,0,248,43,27,0,1,0,0,0,0,0,0,0,32,0,0,0,32,5,27,0,232,252,28,0,232,252,28,0,56,52,27,0,56,52,27,0,1,0,0,0,0,0,0,0,32,0,0,0,32,5,27,0,16,253,28,0,16,253,28,0,56,52,27,0,56,52,27,0,1,0,0,0,0,0,0,0,32,0,0,0,168,205,26,0,56,253,28,0,56,253,28,0,248,52,27,0,248,52,27,0,1,0,0,0,0,0,0,0,32,0,0,0,168,205,26,0,96,253,28,0,96,253,28,0,248,52,27,0,248,52,27,0,1,0,0,0,0,0,0,0,32,0,0,0,216,227,26,0,136,253,28,0,136,253,28,0,184,53,27,0,184,53,27,0,2,0,0,0,0,0,0,0,32,0,0,0,8,250,26,0,176,253,28,0,176,253,28,0,120,54,27,0,120,54,27,0,2,0,0,0,0,0,0,0,32,0,0,0,8,250,26,0,216,253,28,0,216,253,28,0,120,54,27,0,120,54,27,0,2,0,0,0,0,0,0,0,32,0,0,0,192,216,26,0,0,254,28,0,0,254,28,0,56,55,27,0,56,55,27,0,2,0,0,0,0,0,0,0,32,0,0,0,192,216,26,0,40,254,28,0,40,254,28,0,56,55,27,0,56,55,27,0,2,0,0,0,0,0,0,0,32,0,0,0,192,216,26,0,80,254,28,0,80,254,28,0,248,55,27,0,248,55,27,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,232,255,255,255,226,255,255,255,216,255,255,255,216,255,255,255,211,255,255,255,211,255,255,255,211,255,255,255,32,0,0,0,25,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,25,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,242,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,35,0,0,0,21,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,20,0,0,0,8,0,0,0,0,0,0,192,0,0,160,63,25,0,0,0,12,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,9,0,0,0,253,255,255,255,0,0,0,0,0,0,0,0,20,0,0,0,9,0,0,0,252,255,255,255,0,0,0,0,0,0,0,0,20,0,0,0,9,0,0,0,252,255,255,255,0,0,0,0,0,0,0,0,20,0,0,0,6,0,0,0,250,255,255,255,0,0,0,0,0,0,0,0,20,0,0,0,3,0,0,0,246,255,255,255,0,0,0,0,0,0,0,0,18,0,0,0,1,0,0,0,242,255,255,255,0,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,240,255,255,255,0,0,0,0,0,0,0,0,18,0,0,0,254,255,255,255,240,255,255,255,0,0,0,0,0,0,0,0,12,0,0,0,254,255,255,255,236,255,255,255,0,0,0,0,0,0,0,0,30,0,0,0,25,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,25,0,0,0,22,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,0,0,0,0,242,255,255,255,0,0,0,0,0,0,0,0,30,0,0,0,25,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,25,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,242,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,90,0,0,0,90,0,0,0,95,0,0,0,95,0,0,0,95,0,0,0,95,0,0,0,105,0,0,0,105,0,0,0,105,0,0,0,105,0,0,0,105,0,0,0,105,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,128,63,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,4,0,0,0,3,0,0,0,0,0,128,63,0,0,0,64,0,0,64,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,160,64,0,0,192,64,0,0,224,64,0,0,0,65,0,0,0,65,0,0,0,65,0,0,64,65,0,0,72,65,0,0,80,65,0,0,88,65,0,0,96,65,0,0,104,65,0,0,112,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,3,0,0,0,0,0,128,63,0,0,0,64,0,0,64,64,0,0,128,64,0,0,128,64,0,0,160,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,64,65,0,0,72,65,0,0,80,65,0,0,88,65,0,0,96,65,0,0,104,65,0,0,112,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,3,0,0,0,0,0,128,63,0,0,0,64,0,0,64,64,0,0,128,64,0,0,128,64,0,0,160,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,2,0,0,0,1,0,0,0,0,0,64,64,0,0,128,64,0,0,128,64,0,0,160,64,0,0,160,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,2,0,0,0,1,0,0,0,0,0,128,64,0,0,128,64,0,0,160,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,0,65,0,0,0,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,2,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,192,64,0,0,192,64,0,0,192,64,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,2,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,64,0,0,224,64,0,0,0,65,0,0,0,65,0,0,0,65,0,0,32,65,0,0,32,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,65,0,0,0,65,0,0,0,65,0,0,32,65,0,0,32,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,65,0,0,0,65,0,0,32,65,0,0,32,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,65,0,0,32,65,0,0,32,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,64,64,0,0,64,64,0,0,128,64,0,0,128,64,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,0,64,0,0,64,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,128,64,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66,0,0,198,66])
.concat([10,0,0,0,10,0,0,0,255,255,255,255,10,0,0,0,10,0,0,0,255,255,255,255,3,0,0,0,3,0,0,0,15,0,0,0,3,0,0,0,3,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,100,0,0,0,10,0,0,0,10,0,0,0,100,0,0,0,10,0,0,0,10,0,0,0,255,255,255,255,10,0,0,0,10,0,0,0,255,255,255,255,20,0,0,0,20,0,0,0,255,255,255,255,20,0,0,0,20,0,0,0,255,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,20,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,236,255,255,255,240,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,254,255,255,255,2,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,15,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,254,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,250,255,255,255,252,255,255,255,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,248,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,1,0,0,0,4,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,10,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,3,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,249,255,255,255,252,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,254,255,255,255,0,0,0,0,2,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,252,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,238,255,255,255,242,255,255,255,248,255,255,255,255,255,255,255,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,254,255,255,255,255,255,255,255,1,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,252,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,236,255,255,255,238,255,255,255,242,255,255,255,248,255,255,255,255,255,255,255,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,224,255,255,255,224,255,255,255,224,255,255,255,224,255,255,255,228,255,255,255,232,255,255,255,234,255,255,255,240,255,255,255,244,255,255,255,250,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,253,255,255,255,255,255,255,255,0,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,232,255,255,255,232,255,255,255,238,255,255,255,242,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,246,255,255,255,246,255,255,255,247,255,255,255,251,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,236,255,255,255,238,255,255,255,242,255,255,255,248,255,255,255,255,255,255,255,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,224,255,255,255,224,255,255,255,224,255,255,255,224,255,255,255,228,255,255,255,232,255,255,255,232,255,255,255,238,255,255,255,242,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,251,255,255,255,254,255,255,255,0,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,234,255,255,255,237,255,255,255,237,255,255,255,237,255,255,255,237,255,255,255,238,255,255,255,239,255,255,255,240,255,255,255,244,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,236,255,255,255,238,255,255,255,242,255,255,255,248,255,255,255,255,255,255,255,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,224,255,255,255,224,255,255,255,224,255,255,255,224,255,255,255,228,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,238,255,255,255,242,255,255,255,244,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,250,255,255,255,254,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,236,255,255,255,240,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,234,255,255,255,236,255,255,255,241,255,255,255,246,255,255,255,248,255,255,255,254,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,7,0,0,0,220,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,242,255,255,255,244,255,255,255,246,255,255,255,249,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,222,255,255,255,226,255,255,255,228,255,255,255,230,255,255,255,232,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,236,255,255,255,242,255,255,255,248,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,2,0,0,0,218,255,255,255,218,255,255,255,218,255,255,255,218,255,255,255,220,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,238,255,255,255,240,255,255,255,244,255,255,255,246,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,218,255,255,255,221,255,255,255,221,255,255,255,221,255,255,255,221,255,255,255,221,255,255,255,221,255,255,255,221,255,255,255,221,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,228,255,255,255,236,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,246,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,221,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,236,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,20,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,236,255,255,255,240,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,254,255,255,255,2,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,15,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,254,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,20,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,254,255,255,255,2,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,0,0,0,0,2,0,0,0,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,248,255,255,255,250,255,255,255,252,255,255,255,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,20,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,254,255,255,255,0,0,0,0,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,250,255,255,255,254,255,255,255,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,252,255,255,255,254,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,250,255,255,255,254,255,255,255,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,252,255,255,255,254,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,244,255,255,255,246,255,255,255,250,255,255,255,254,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,2,0,0,0,6,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,252,255,255,255,254,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,238,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,224,255,255,255,224,255,255,255,224,255,255,255,224,255,255,255,228,255,255,255,232,255,255,255,234,255,255,255,240,255,255,255,244,255,255,255,250,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,254,255,255,255,0,0,0,0,4,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,230,255,255,255,232,255,255,255,238,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,251,255,255,255,253,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,238,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,248,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,253,255,255,255,255,255,255,255,4,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,230,255,255,255,236,255,255,255,240,255,255,255,243,255,255,255,243,255,255,255,243,255,255,255,243,255,255,255,243,255,255,255,245,255,255,255,248,255,255,255,250,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,238,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,240,255,255,255,246,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,251,255,255,255,253,255,255,255,1,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,224,255,255,255,224,255,255,255,228,255,255,255,234,255,255,255,238,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,242,255,255,255,244,255,255,255,246,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,11,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,240,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,249,255,255,255,251,255,255,255,254,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,222,255,255,255,228,255,255,255,234,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,242,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,236,255,255,255,242,255,255,255,248,255,255,255,254,255,255,255,254,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,2,0,0,0,6,0,0,0,220,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,222,255,255,255,224,255,255,255,224,255,255,255,232,255,255,255,240,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,251,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,224,255,255,255,230,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,236,255,255,255,238,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,236,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,221,255,255,255,226,255,255,255,231,255,255,255,231,255,255,255,231,255,255,255,231,255,255,255,231,255,255,255,231,255,255,255,241,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,20,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,246,255,255,255,254,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,6,0,0,0,15,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,246,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,2,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,242,255,255,255,250,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,242,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,252,255,255,255,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,1,0,0,0,4,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,10,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,3,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,252,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,254,255,255,255,0,0,0,0,2,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,251,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,252,255,255,255,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,253,255,255,255,254,255,255,255,255,255,255,255,1,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,249,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,252,255,255,255,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,240,255,255,255,244,255,255,255,250,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,253,255,255,255,255,255,255,255,0,0,0,0,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,238,255,255,255,242,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,246,255,255,255,246,255,255,255,247,255,255,255,248,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,252,255,255,255,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,238,255,255,255,242,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,251,255,255,255,254,255,255,255,0,0,0,0,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,238,255,255,255,240,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,243,255,255,255,243,255,255,255,244,255,255,255,246,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,252,255,255,255,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,238,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,252,255,255,255,0,0,0,0,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,237,255,255,255,237,255,255,255,237,255,255,255,237,255,255,255,238,255,255,255,239,255,255,255,240,255,255,255,244,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,7,0,0,0,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,236,255,255,255,240,255,255,255,244,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,250,255,255,255,254,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,230,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,236,255,255,255,240,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,238,255,255,255,242,255,255,255,248,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,2,0,0,0,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,234,255,255,255,238,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,242,255,255,255,244,255,255,255,246,255,255,255,249,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,238,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,246,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,236,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,20,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,236,255,255,255,240,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,254,255,255,255,2,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,15,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,254,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,20,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,250,255,255,255,254,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,254,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,248,255,255,255,250,255,255,255,252,255,255,255,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,20,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,254,255,255,255,254,255,255,255,254,255,255,255,2,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,252,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,250,255,255,255,254,255,255,255,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,16,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,0,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,252,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,242,255,255,255,246,255,255,255,248,255,255,255,250,255,255,255,254,255,255,255,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,14,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,0,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,252,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,244,255,255,255,246,255,255,255,250,255,255,255,254,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,0,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,252,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,238,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,11,0,0,0,224,255,255,255,224,255,255,255,224,255,255,255,224,255,255,255,228,255,255,255,232,255,255,255,234,255,255,255,240,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,252,255,255,255,254,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,230,255,255,255,232,255,255,255,238,255,255,255,242,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,244,255,255,255,246,255,255,255,247,255,255,255,251,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,238,255,255,255,242,255,255,255,246,255,255,255,252,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,11,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,242,255,255,255,242,255,255,255,244,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,222,255,255,255,228,255,255,255,232,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,238,255,255,255,240,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,11,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,234,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,224,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,234,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,3,0,0,0,3,0,0,0,5,0,0,0,10,0,0,0,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,222,255,255,255,224,255,255,255,224,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,224,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,228,255,255,255,236,255,255,255,242,255,255,255,248,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,252,255,255,255,254,255,255,255,2,0,0,0,220,255,255,255,220,255,255,255,220,255,255,255,220,255,255,255,222,255,255,255,224,255,255,255,224,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,224,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,236,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,240,255,255,255,242,255,255,255,244,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,221,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,216,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,99,0,0,0,99,0,0,0,99,0,0,0,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([99,0,0,0,99,0,0,0,99,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,99,0,0,0,99,0,0,0,99,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,244,255,255,255,244,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,99,0,0,0,99,0,0,0,99,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,99,0,0,0,99,0,0,0,99,0,0,0,0,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,4,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,20,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,20,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,12,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,244,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,244,255,255,255,244,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,250,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,4,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,20,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,15,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,10,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,10,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,244,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,249,255,255,255,251,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,238,255,255,255,238,255,255,255,238,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,20,0,0,0,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,254,255,255,255,254,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,20,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,250,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,12,0,0,0,236,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,240,255,255,255,244,255,255,255,236,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,244,255,255,255,244,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,250,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,236,255,255,255,236,255,255,255,236,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,10,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,99,0,0,0,99,0,0,0,99,0,0,0,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,246,255,255,255,246,255,255,255,251,255,255,255,251,255,255,255,251,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,232,255,255,255,236,255,255,255,242,255,255,255,246,255,255,255,250,255,255,255,248,255,255,255,248,255,255,255,250,255,255,255,250,255,255,255,99,0,0,0,99,0,0,0,99,0,0,0,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,241,255,255,255,244,255,255,255,246,255,255,255,248,255,255,255,0,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,99,0,0,0,99,0,0,0,99,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,234,255,255,255,236,255,255,255,242,255,255,255,244,255,255,255,244,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,246,255,255,255,99,0,0,0,99,0,0,0,99,0,0,0,226,255,255,255,226,255,255,255,226,255,255,255,226,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,230,255,255,255,232,255,255,255,99,0,0,0,99,0,0,0,99,0,0,0,0,0,0,0,236,255,255,255,236,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,232,255,255,255,226,255,255,255,216,255,255,255,216,255,255,255,211,255,255,255,211,255,255,255,211,255,255,255,0,0,0,0,0,0,8,64,0,0,0,0,0,0,16,64,0,0,0,0,0,0,16,64,205,204,204,204,204,204,43,64,51,51,51,51,51,51,46,64,154,153,153,153,153,153,47,64,0,0,0,0,0,128,48,64,51,51,51,51,51,51,49,64,102,102,102,102,102,230,50,64,154,153,153,153,153,25,52,64,0,0,0,0,0,0,72,64,0,0,0,0,0,56,143,64,0,0,0,0,0,56,143,64,0,0,0,0,0,56,143,64,0,0,0,0,0,56,143,64,154,153,153,153,153,153,40,64,0,0,0,0,0,0,42,64,0,0,0,0,0,0,42,64,0,0,0,0,0,0,44,64,0,0,0,0,0,0,46,64,0,0,0,0,0,192,88,64,0,0,0,0,0,192,88,64,0,0,0,0,0,192,88,64,0,0,0,0,0,192,88,64,0,0,0,0,0,192,88,64,0,0,0,0,0,192,88,64,0,0,0,0,0,192,88,64,0,0,0,0,0,0,35,64,0,0,0,0,0,0,38,64,0,0,0,0,0,0,62,64,0,0,0,0,0,192,88,64,0,0,0,0,0,0,26,64,0,0,0,0,0,0,32,64,0,0,0,0,0,0,62,64,0,0,0,0,0,192,88,64,0,0,0,0,0,0,18,64,0,0,0,0,0,0,22,64,0,0,0,0,0,0,62,64,255,255,255,255,0,0,12,195,0,0,12,195,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,32,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,63,0,0,0,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,210,66,0,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,8,0,0,0,0,0,160,65,0,0,96,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,64,65,0,0,112,194,0,0,240,193,0,0,32,194,0,0,32,194,0,0,32,194,0,0,32,194,0,0,32,194,0,0,0,64,0,0,150,194,0,0,192,192,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,96,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,194,0,0,240,193,0,0,200,193,0,0,200,193,0,0,200,193,0,0,200,193,0,0,200,193,0,0,0,64,0,0,160,194,0,0,192,192,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,64,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,32,65,0,0,160,193,0,0,160,193,0,0,112,193,0,0,112,193,0,0,112,193,0,0,112,193,0,0,112,193,0,0,0,0,0,0,160,194,0,0,192,192,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,32,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,0,65,0,0,160,193,0,0,112,193,0,0,64,193,0,0,64,193,0,0,64,193,0,0,64,193,0,0,64,193,0,0,0,0,0,0,160,194,0,0,192,192,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,32,65,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,192,64,0,0,112,193,0,0,112,193,0,0,64,193,0,0,64,193,0,0,64,193,0,0,64,193,0,0,64,193,0,0,0,0,0,0,170,194,0,0,192,192,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,63,0,0,0,0,0,0,240,63,0,0,0,0,0,0,240,63,205,204,204,204,204,204,244,63,154,153,153,153,153,153,249,63,0,0,0,0,0,0,0,64,0,0,0,0,0,0,0,64,0,0,0,0,0,0,0,64,0,0,0,0,0,0,0,64,0,0,0,0,0,0,0,64,0,0,0,0,0,0,0,64,0,0,0,0,0,0,0,64,0,0,0,0,0,0,12,64,0,0,0,0,0,0,16,64,0,0,0,0,0,0,16,64,51,51,51,51,51,51,17,64,102,102,102,102,102,102,18,64,0,0,0,0,0,0,20,64,0,0,0,0,0,0,20,64,0,0,0,0,0,0,20,64,0,0,0,0,0,0,20,64,0,0,0,0,0,0,20,64,0,0,0,0,0,0,20,64,0,0,0,0,0,0,20,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,240,63,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,23,0,0,0,24,0,0,0,25,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,23,0,0,0,24,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,23,0,0,0,24,0,0,0,25,0,0,0,26,0,0,0,27,0,0,0,28,0,0,0,29,0,0,0,30,0,0,0,31,0,0,0,32,0,0,0,33,0,0,0,34,0,0,0,35,0,0,0,36,0,0,0,37,0,0,0,38,0,0,0,39,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,23,0,0,0,24,0,0,0,25,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,23,0,0,0,24,0,0,0,25,0,0,0,26,0,0,0,27,0,0,0,28,0,0,0,29,0,0,0,30,0,0,0,31,0,0,0,32,0,0,0,33,0,0,0,34,0,0,0,35,0,0,0,36,0,0,0,37,0,0,0,38,0,0,0,39,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,23,0,0,0,24,0,0,0,25,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,154,153,153,153,153,153,233,63,0,0,0,0,0,0,240,63,0,0,0,0,0,0,240,63,156,255,255,255,156,255,255,255,151,255,255,255,0,0,0,0,156,255,255,255,156,255,255,255,156,255,255,255,151,255,255,255,156,255,255,255,156,255,255,255,156,255,255,255,156,255,255,255,156,255,255,255,156,255,255,255,151,255,255,255,151,255,255,255,151,255,255,255,151,255,255,255,146,255,255,255,136,255,255,255,126,255,255,255,126,255,255,255,116,255,255,255,0,0,0,0,126,255,255,255,126,255,255,255,126,255,255,255,116,255,255,255,126,255,255,255,126,255,255,255,126,255,255,255,126,255,255,255,116,255,255,255,116,255,255,255,116,255,255,255,116,255,255,255,116,255,255,255,116,255,255,255,116,255,255,255,106,255,255,255,0,0,0,0,0,0,224,63,0,0,0,0,0,0,224,63,154,153,153,153,153,153,201,63,154,153,153,153,153,153,201,63,154,153,153,153,153,153,201,63,154,153,153,153,153,153,217,63,51,51,51,51,51,51,227,63,0,0,0,0,128,135,195,64,0,0,0,0,128,135,195,64,0,0,0,0,128,135,195,64,0,0,0,0,128,135,195,64,0,0,0,0,128,135,195,64,0,0,0,0,128,135,195,64,51,51,51,51,51,51,211,63,0,0,0,0,0,0,224,63,0,0,0,0,0,0,224,63,0,0,0,0,0,0,224,63,51,51,51,51,51,51,211,63,0,0,0,0,0,0,224,63,0,0,0,0,0,0,224,63,32,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,32,0,0,0,15,39,0,0,15,39,0,0,15,39,0,0,15,39,0,0,15,39,0,0,15,39,0,0,0,0,0,0,0,1,0,0,128,0,0,0,128,0,0,0,0,1,0,0,0,2,0,0,15,39,0,0,15,39,0,0,15,39,0,0,15,39,0,0,15,39,0,0,15,39,0,0,0,0,0,0,64,0,0,0,64,0,0,0,0,1,0,0,0,1,0,0,15,39,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,32,0,0,0,32,0,0,0,32,0,0,0,32,0,0,0,32,0,0,0,32,0,0,0,32,0,0,0,32,0,0,0,32,0,0,0,32,0,0,0,32,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,208,163,27,0,216,56,27,0,208,163,27,0,184,56,27,0,0,239,27,0,24,57,27,0,0,239,27,0,248,56,27,0,208,163,27,0,56,57,27,0,208,163,27,0,184,59,27,0,208,163,27,0,120,59,27,0,208,163,27,0,56,59,27,0,208,163,27,0,248,58,27,0,208,163,27,0,184,58,27,0,208,163,27,0,120,58,27,0,208,163,27,0,56,58,27,0,208,163,27,0,248,57,27,0,208,163,27,0,184,57,27,0,208,163,27,0,120,57,27,0,0,239,27,0,248,59,27,0,0,239,27,0,120,62,27,0,0,239,27,0,56,62,27,0,0,239,27,0,248,61,27,0,0,239,27,0,184,61,27,0,0,239,27,0,120,61,27,0,0,239,27,0,56,61,27,0,0,239,27,0,248,60,27,0,0,239,27,0,184,60,27,0,0,239,27,0,120,60,27,0,0,239,27,0,56,60,27,0,240,213,27,0,184,62,27,0,240,213,27,0,120,66,27,0,240,213,27,0,24,66,27,0,240,213,27,0,184,65,27,0,240,213,27,0,88,65,27,0,240,213,27,0,248,64,27,0,224,188,27,0,152,64,27,0,224,188,27,0,56,64,27,0,224,188,27,0,216,63,27,0,224,188,27,0,120,63,27,0,224,188,27,0,24,63,27,0,208,163,27,0,88,67,27,0,208,163,27,0,24,67,27,0,208,163,27,0,216,66,27,0,0,239,27,0,248,67,27,0,0,239,27,0,184,67,27,0,0,239,27,0,120,67,27,0,128,251,1,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,1,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,6,0,0,0,17,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,17,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,15,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,15,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,17,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,14,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,11,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,17,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,15,0,0,0,5,0,0,0,3,0,0,0,4,0,0,0,11,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,14,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,17,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,13,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,13,0,0,0,10,0,0,0,15,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,14,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,2,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,15,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,15,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,15,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,15,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,15,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,2,0,0,0,5,0,0,0,6,0,0,0,15,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,15,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,15,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,15,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,15,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,15,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,15,0,0,0,12,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,6,0,0,0,3,0,0,0,3,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,15,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,3,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,6,0,0,0,3,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,6,0,0,0,3,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,19,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,15,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,13,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,12,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,7,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,19,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,16,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,12,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,11,0,0,0,10,0,0,0,18,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,12,0,0,0,11,0,0,0])
.concat([10,0,0,0,16,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,11,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,7,0,0,0,13,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,15,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,2,0,0,0,4,0,0,0,7,0,0,0,16,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,16,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,3,0,0,0,3,0,0,0,6,0,0,0,16,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,16,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,16,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,16,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,16,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,15,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,15,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,15,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,16,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,16,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,16,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,2,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,1,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,3,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,2,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,8,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,14,0,0,0,9,0,0,0,15,0,0,0,10,0,0,0,15,0,0,0,10,0,0,0,15,0,0,0,10,0,0,0,15,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,15,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,12,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,13,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,8,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,8,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,6,0,0,0,10,0,0,0,16,0,0,0,17,0,0,0,6,0,0,0,10,0,0,0,15,0,0,0,17,0,0,0,8,0,0,0,10,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,8,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,6,0,0,0,11,0,0,0,15,0,0,0,17,0,0,0,7,0,0,0,10,0,0,0,15,0,0,0,17,0,0,0,8,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,4,0,0,0,8,0,0,0,13,0,0,0,17,0,0,0,4,0,0,0,7,0,0,0,13,0,0,0,17,0,0,0,6,0,0,0,8,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,6,0,0,0,11,0,0,0,15,0,0,0,17,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,17,0,0,0,8,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,5,0,0,0,10,0,0,0,14,0,0,0,17,0,0,0,5,0,0,0,9,0,0,0,14,0,0,0,17,0,0,0,7,0,0,0,9,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,3,0,0,0,7,0,0,0,12,0,0,0,17,0,0,0,3,0,0,0,6,0,0,0,11,0,0,0,17,0,0,0,5,0,0,0,7,0,0,0,13,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,5,0,0,0,9,0,0,0,14,0,0,0,17,0,0,0,3,0,0,0,7,0,0,0,11,0,0,0,17,0,0,0,5,0,0,0,8,0,0,0,13,0,0,0,17,0,0,0,13,0,0,0,11,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,8,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,8,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,5,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,5,0,0,0,9,0,0,0,15,0,0,0,17,0,0,0,7,0,0,0,9,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,7,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,6,0,0,0,10,0,0,0,15,0,0,0,17,0,0,0,7,0,0,0,9,0,0,0,15,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,10,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,13,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,7,0,0,0,2,0,0,0,7,0,0,0,3,0,0,0,8,0,0,0,4,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,3,0,0,0,6,0,0,0,3,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,3,0,0,0,8,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,17,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0])
.concat([4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,10,0,0,0,17,0,0,0,4,0,0,0,8,0,0,0,11,0,0,0,20,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,18,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,17,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,17,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,17,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,16,0,0,0,20,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,20,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,20,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,1,0,0,0,4,0,0,0,10,0,0,0,19,0,0,0,3,0,0,0,8,0,0,0,13,0,0,0,19,0,0,0,7,0,0,0,12,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,2,0,0,0,6,0,0,0,11,0,0,0,19,0,0,0,8,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,9,0,0,0,11,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,6,0,0,0,7,0,0,0,13,0,0,0,19,0,0,0,9,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,10,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,2,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,11,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,8,0,0,0,12,0,0,0,8,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,3,0,0,0,6,0,0,0,3,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,5,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,1,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,5,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,14,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,3,0,0,0,9,0,0,0,2,0,0,0,9,0,0,0,3,0,0,0,8,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,6,0,0,0,3,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,17,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,17,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,17,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,16,0,0,0,5,0,0,0,3,0,0,0,6,0,0,0,14,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,13,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,11,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,1,0,0,0,6,0,0,0,12,0,0,0,16,0,0,0,4,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,2,0,0,0,5,0,0,0,11,0,0,0,16,0,0,0,5,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,9,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,16,0,0,0,5,0,0,0,9,0,0,0,12,0,0,0,16,0,0,0,9,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,1,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,2,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,1,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,3,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,10,0,0,0,4,0,0,0,10,0,0,0,5,0,0,0,10,0,0,0,5,0,0,0,11,0,0,0,5,0,0,0,12,0,0,0,6,0,0,0,12,0,0,0,6,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,2,0,0,0,3,0,0,0,7,0,0,0,13,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,15,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,17,0,0,0,21,0,0,0,16,0,0,0,15,0,0,0,21,0,0,0,2,0,0,0,5,0,0,0,7,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,14,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,21,0,0,0,4,0,0,0,7,0,0,0,10,0,0,0,17,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,15,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,16,0,0,0,21,0,0,0,18,0,0,0,15,0,0,0,21,0,0,0,18,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,21,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,20,0,0,0,1,0,0,0,5,0,0,0,7,0,0,0,21,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,21,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,4,0,0,0,8,0,0,0,9,0,0,0,20,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,20,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,17,0,0,0,20,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,20,0,0,0,8,0,0,0,10,0,0,0,15,0,0,0,20,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,3,0,0,0,6,0,0,0,8,0,0,0,20,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,20,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,20,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,20,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,20,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,20,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,20,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,20,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,20,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,20,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,9,0,0,0,11,0,0,0,15,0,0,0,20,0,0,0,8,0,0,0,10,0,0,0,14,0,0,0,20,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,11,0,0,0,16,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,9,0,0,0,14,0,0,0,16,0,0,0,20,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,20,0,0,0,17,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,16,0,0,0,19,0,0,0,18,0,0,0,20,0,0,0,15,0,0,0,16,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,17,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,1,0,0,0,4,0,0,0,2,0,0,0,6,0,0,0,3,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,2,0,0,0,10,0,0,0,8,0,0,0,14,0,0,0,7,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,1,0,0,0,5,0,0,0,3,0,0,0,7,0,0,0,4,0,0,0,9,0,0,0,7,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,2,0,0,0,6,0,0,0,3,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,2,0,0,0,4,0,0,0,2,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,7,0,0,0,3,0,0,0,8,0,0,0,3,0,0,0,10,0,0,0,3,0,0,0,8,0,0,0,3,0,0,0,9,0,0,0,3,0,0,0,8,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,12,0,0,0,9,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,15,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,9,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,8,0,0,0,4,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,18,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,7,0,0,0,12,0,0,0,8,0,0,0,13,0,0,0,8,0,0,0,12,0,0,0,8,0,0,0,12,0,0,0,8,0,0,0,13,0,0,0,8,0,0,0,13,0,0,0,9,0,0,0,13,0,0,0,9,0,0,0,13,0,0,0,9,0,0,0,13,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,21,0,0,0,22,0,0,0,22,0,0,0,21,0,0,0,22,0,0,0,22,0,0,0,22,0,0,0,22,0,0,0,22,0,0,0,22,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,21,0,0,0,4,0,0,0,7,0,0,0,13,0,0,0,9,0,0,0,15,0,0,0,9,0,0,0,16,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,16,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,15,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,14,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,12,0,0,0,11,0,0,0,16,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,12,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,4,0,0,0,7,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,16,0,0,0,9,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,17,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,18,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,15,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,18,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,9,0,0,0,17,0,0,0,4,0,0,0,6,0,0,0,18,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,4,0,0,0,4,0,0,0,18,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,7,0,0,0,5,0,0,0,17,0,0,0,6,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,17,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,7,0,0,0,5,0,0,0,17,0,0,0,6,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,17,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,17,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,17,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,18,0,0,0,7,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,2,0,0,0,18,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,7,0,0,0,4,0,0,0,17,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,7,0,0,0,17,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,15,0,0,0,6,0,0,0,4,0,0,0,17,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,16,0,0,0,6,0,0,0,6,0,0,0,17,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,17,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,17,0,0,0,9,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,4,0,0,0,2,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,6,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,14,0,0,0,11,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,14,0,0,0,5,0,0,0,6,0,0,0,12,0,0,0,9,0,0,0,14,0,0,0,9,0,0,0,9,0,0,0,19,0,0,0,6,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,19,0,0,0,12,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,18,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,17,0,0,0,14,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,18,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,18,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,11,0,0,0,7,0,0,0,5,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,18,0,0,0,9,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,3,0,0,0,4,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,3,0,0,0,3,0,0,0,8,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,15,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,3,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,15,0,0,0,11,0,0,0,7,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,4,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,16,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,10,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,11,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,3,0,0,0,4,0,0,0,8,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,13,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,17,0,0,0,3,0,0,0,9,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,7,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,5,0,0,0,12,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,4,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,17,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,15,0,0,0,17,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,16,0,0,0,16,0,0,0,6,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,17,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,16,0,0,0,17,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,7,0,0,0,11,0,0,0,4,0,0,0,4,0,0,0,2,0,0,0,7,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,8,0,0,0,15,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,9,0,0,0,3,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,14,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,3,0,0,0,6,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,4,0,0,0,11,0,0,0,16,0,0,0,13,0,0,0,17,0,0,0,13,0,0,0,17,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,4,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,10,0,0,0,16,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,16,0,0,0,17,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,17,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,17,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,2,0,0,0,7,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,5,0,0,0,10,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,17,0,0,0,7,0,0,0,11,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,16,0,0,0,3,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,10,0,0,0,14,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,10,0,0,0,17,0,0,0,13,0,0,0,17,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,3,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,15,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,17,0,0,0,17,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,14,0,0,0,17,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,16,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,14,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,15,0,0,0,4,0,0,0,8,0,0,0,3,0,0,0,5,0,0,0,14,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,11,0,0,0,15,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,15,0,0,0,17,0,0,0,3,0,0,0,8,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,16,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,11,0,0,0,16,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,16,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,12,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,15,0,0,0,13,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,16,0,0,0,14,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,15,0,0,0,3,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,13,0,0,0,9,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,3,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,16,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,16,0,0,0,10,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,16,0,0,0,14,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,16,0,0,0,14,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,16,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,9,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,12,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,14,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,14,0,0,0,11,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,13,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,3,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,17,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,17,0,0,0,10,0,0,0,6,0,0,0,3,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,17,0,0,0,15,0,0,0,10,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,17,0,0,0,15,0,0,0,8,0,0,0,4,0,0,0,7,0,0,0,3,0,0,0,5,0,0,0,9,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,16,0,0,0,13,0,0,0,11,0,0,0,5,0,0,0,8,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,10,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,14,0,0,0,5,0,0,0,9,0,0,0,14,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,13,0,0,0,7,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,12,0,0,0,13,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,15,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,15,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,14,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,17,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,4,0,0,0,3,0,0,0,3,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,16,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,10,0,0,0,16,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,11,0,0,0,7,0,0,0,3,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,4,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,5,0,0,0,8,0,0,0,13,0,0,0,10,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,7,0,0,0,2,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,13,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,20,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,18,0,0,0,15,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,14,0,0,0,20,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,17,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,4,0,0,0,11,0,0,0,12,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,10,0,0,0,7,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,17,0,0,0,12,0,0,0,15,0,0,0,17,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,4,0,0,0,3,0,0,0,3,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,16,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,10,0,0,0,16,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,11,0,0,0,7,0,0,0,3,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,16,0,0,0,12,0,0,0,9,0,0,0,4,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,5,0,0,0,8,0,0,0,13,0,0,0,10,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,7,0,0,0,2,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,13,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,20,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,18,0,0,0,15,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,14,0,0,0,20,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,17,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,5,0,0,0,4,0,0,0,11,0,0,0,12,0,0,0,17,0,0,0,19,0,0,0,14,0,0,0,16,0,0,0,10,0,0,0,7,0,0,0,12,0,0,0,4,0,0,0,3,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,7,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,17,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,17,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,16,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,9,0,0,0,15,0,0,0,17,0,0,0,20,0,0,0,21,0,0,0,22,0,0,0,23,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,17,0,0,0,20,0,0,0,9,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,15,0,0,0,18,0,0,0,11,0,0,0,7,0,0,0])
.concat([5,0,0,0,4,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,17,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,20,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,8,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,23,0,0,0,17,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,6,0,0,0,4,0,0,0,4,0,0,0,2,0,0,0,7,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,21,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,8,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,17,0,0,0,11,0,0,0,8,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,17,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,19,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,8,0,0,0,21,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,3,0,0,0,9,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,22,0,0,0,22,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,17,0,0,0,20,0,0,0,9,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,15,0,0,0,19,0,0,0,11,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,17,0,0,0,14,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,11,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,19,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,21,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,6,0,0,0,4,0,0,0,4,0,0,0,2,0,0,0,7,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,20,0,0,0,6,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,7,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,16,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,17,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,19,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,20,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,2,0,0,0,8,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,21,0,0,0,22,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,17,0,0,0,21,0,0,0,9,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,15,0,0,0,19,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,9,0,0,0,11,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,4,0,0,0,3,0,0,0,7,0,0,0,10,0,0,0,22,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,21,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,2,0,0,0,7,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,6,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,7,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,16,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,18,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,20,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,2,0,0,0,3,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,18,0,0,0,20,0,0,0,20,0,0,0,5,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,15,0,0,0,19,0,0,0,9,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,18,0,0,0,11,0,0,0,6,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,13,0,0,0,19,0,0,0,14,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,16,0,0,0,20,0,0,0,16,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,14,0,0,0,16,0,0,0,21,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,11,0,0,0,14,0,0,0,21,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,10,0,0,0,12,0,0,0,3,0,0,0,7,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,6,0,0,0,2,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,15,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,17,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,19,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,13,0,0,0,18,0,0,0,21,0,0,0,4,0,0,0,2,0,0,0,21,0,0,0,6,0,0,0,6,0,0,0,10,0,0,0,15,0,0,0,21,0,0,0,16,0,0,0,19,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,13,0,0,0,16,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,21,0,0,0,18,0,0,0,13,0,0,0,13,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,12,0,0,0,3,0,0,0,5,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,16,0,0,0,18,0,0,0,4,0,0,0,2,0,0,0,20,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,15,0,0,0,20,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,17,0,0,0,19,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,17,0,0,0,18,0,0,0,4,0,0,0,2,0,0,0,18,0,0,0,6,0,0,0,5,0,0,0,9,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,16,0,0,0,12,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,3,0,0,0,4,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,17,0,0,0,4,0,0,0,2,0,0,0,18,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,18,0,0,0,11,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,17,0,0,0,17,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,15,0,0,0,17,0,0,0,4,0,0,0,2,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,9,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,4,0,0,0,7,0,0,0,4,0,0,0,3,0,0,0,6,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,9,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,12,0,0,0,17,0,0,0,12,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,3,0,0,0,4,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,4,0,0,0,2,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,18,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,16,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,17,0,0,0,18,0,0,0,16,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,4,0,0,0,2,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,7,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,12,0,0,0,14,0,0,0,8,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,16,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,6,0,0,0,10,0,0,0,13,0,0,0,17,0,0,0,7,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,18,0,0,0,18,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,16,0,0,0,6,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,10,0,0,0,14,0,0,0,15,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,16,0,0,0,3,0,0,0,2,0,0,0,5,0,0,0,7,0,0,0,11,0,0,0,14,0,0,0,17,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,5,0,0,0,6,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,4,0,0,0,2,0,0,0,5,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,11,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,3,0,0,0,2,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,16,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,0,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,16,0,0,0,7,0,0,0,1,0,0,0,5,0,0,0,14,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,16,0,0,0,8,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,14,0,0,0,4,0,0,0,7,0,0,0,16,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,16,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,14,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,8,0,0,0,9,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,8,0,0,0,8,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,17,0,0,0,3,0,0,0,1,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,17,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,17,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,19,0,0,0,20,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,20,0,0,0,0,0,0,0,5,0,0,0,13,0,0,0,18,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,19,0,0,0,19,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,14,0,0,0,17,0,0,0,19,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,19,0,0,0,17,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,15,0,0,0,19,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,10,0,0,0,14,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,4,0,0,0,2,0,0,0,6,0,0,0,12,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,15,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,3,0,0,0,8,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,9,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,4,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,18,0,0,0,17,0,0,0,19,0,0,0,17,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,18,0,0,0,19,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,17,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,19,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,10,0,0,0,15,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,2,0,0,0,7,0,0,0,12,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,19,0,0,0,18,0,0,0,17,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,3,0,0,0,8,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,4,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,15,0,0,0,19,0,0,0,19,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,11,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,15,0,0,0,19,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,10,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,4,0,0,0,2,0,0,0,7,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,19,0,0,0,19,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,3,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,14,0,0,0,6,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,7,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,13,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,3,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,19,0,0,0,17,0,0,0,17,0,0,0,19,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,18,0,0,0,19,0,0,0,7,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,17,0,0,0,19,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,18,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,17,0,0,0,19,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,17,0,0,0,19,0,0,0,13,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,16,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,17,0,0,0,18,0,0,0,3,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,6,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,7,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,15,0,0,0,17,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,15,0,0,0,18,0,0,0,18,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,8,0,0,0,3,0,0,0,4,0,0,0,6,0,0,0,10,0,0,0,14,0,0,0,16,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,9,0,0,0,14,0,0,0,18,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,14,0,0,0,18,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,18,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,14,0,0,0,18,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,16,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,13,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,8,0,0,0,5,0,0,0,4,0,0,0,10,0,0,0,4,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,8,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,10,0,0,0,6,0,0,0,4,0,0,0,9,0,0,0,3,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,15,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,14,0,0,0,10,0,0,0,15,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,4,0,0,0,2,0,0,0,11,0,0,0,5,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,9,0,0,0,4,0,0,0,11,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,14,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,17,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,17,0,0,0,4,0,0,0,7,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,5,0,0,0,2,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,5,0,0,0,11,0,0,0,7,0,0,0])
.concat([8,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,10,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,7,0,0,0,2,0,0,0,12,0,0,0,5,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,12,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,10,0,0,0,4,0,0,0,12,0,0,0,5,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,12,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,11,0,0,0,12,0,0,0,16,0,0,0,10,0,0,0,4,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,16,0,0,0,10,0,0,0,6,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,15,0,0,0,12,0,0,0,9,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,5,0,0,0,2,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,7,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,5,0,0,0,11,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,11,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,15,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,8,0,0,0,2,0,0,0,11,0,0,0,4,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,11,0,0,0,4,0,0,0,12,0,0,0,5,0,0,0,9,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,4,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,6,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,6,0,0,0,1,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,11,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,5,0,0,0,12,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,10,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,9,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,13,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,4,0,0,0,7,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,4,0,0,0,2,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,10,0,0,0,4,0,0,0,5,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,15,0,0,0,17,0,0,0,12,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,17,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,13,0,0,0,17,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,8,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,4,0,0,0,2,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,8,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,14,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,8,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,4,0,0,0,2,0,0,0,4,0,0,0,7,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,4,0,0,0,4,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,17,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,12,0,0,0,14,0,0,0,17,0,0,0,11,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,12,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,15,0,0,0,13,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,5,0,0,0,1,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,9,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,14,0,0,0,10,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,0,0,0,0,6,0,0,0,6,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,4,0,0,0,2,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,15,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,16,0,0,0,18,0,0,0,11,0,0,0,6,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,18,0,0,0,10,0,0,0,5,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,17,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,11,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,16,0,0,0,8,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,4,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,12,0,0,0,12,0,0,0,17,0,0,0,17,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,0,0,0,0,9,0,0,0,8,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,6,0,0,0,1,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,17,0,0,0,9,0,0,0,4,0,0,0,5,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,17,0,0,0,17,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,4,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,9,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,8,0,0,0,6,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,19,0,0,0,20,0,0,0,14,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,20,0,0,0,15,0,0,0,9,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,18,0,0,0,14,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,16,0,0,0,13,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,3,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,14,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,11,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,10,0,0,0,14,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,16,0,0,0,5,0,0,0,8,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,18,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,17,0,0,0,9,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,19,0,0,0,9,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,17,0,0,0,8,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,11,0,0,0,14,0,0,0,9,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,14,0,0,0,11,0,0,0,14,0,0,0,15,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,14,0,0,0,13,0,0,0,15,0,0,0,19,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,10,0,0,0,14,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,11,0,0,0,10,0,0,0,15,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,6,0,0,0,4,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,9,0,0,0,12,0,0,0,16,0,0,0,10,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,11,0,0,0,16,0,0,0,16,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,10,0,0,0,8,0,0,0,14,0,0,0,16,0,0,0,11,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,9,0,0,0,8,0,0,0,14,0,0,0,16,0,0,0,12,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,13,0,0,0,16,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,11,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,13,0,0,0,16,0,0,0,14,0,0,0,14,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,3,0,0,0,6,0,0,0,10,0,0,0,8,0,0,0,12,0,0,0,8,0,0,0,14,0,0,0,8,0,0,0,14,0,0,0,19,0,0,0,5,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,11,0,0,0,7,0,0,0,16,0,0,0,19,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,19,0,0,0,19,0,0,0,6,0,0,0,4,0,0,0,7,0,0,0,5,0,0,0,7,0,0,0,6,0,0,0,10,0,0,0,7,0,0,0,18,0,0,0,18,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,18,0,0,0,18,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,5,0,0,0,7,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,18,0,0,0,18,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,18,0,0,0,18,0,0,0,8,0,0,0,7,0,0,0,10,0,0,0,6,0,0,0,8,0,0,0,5,0,0,0,6,0,0,0,4,0,0,0,11,0,0,0,18,0,0,0,11,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,11,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,18,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,13,0,0,0,16,0,0,0,18,0,0,0,3,0,0,0,5,0,0,0,8,0,0,0,7,0,0,0,14,0,0,0,8,0,0,0,9,0,0,0,19,0,0,0,5,0,0,0,2,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,19,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,13,0,0,0,19,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,5,0,0,0,9,0,0,0,6,0,0,0,9,0,0,0,19,0,0,0,12,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,19,0,0,0,8,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,9,0,0,0,6,0,0,0,7,0,0,0,19,0,0,0,8,0,0,0,8,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,4,0,0,0,5,0,0,0,19,0,0,0,12,0,0,0,17,0,0,0,19,0,0,0,15,0,0,0,18,0,0,0,13,0,0,0,11,0,0,0,18,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,15,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,18,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,15,0,0,0,10,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,6,0,0,0,9,0,0,0,12,0,0,0,15,0,0,0,11,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,7,0,0,0,11,0,0,0,13,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,10,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,13,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,13,0,0,0,4,0,0,0,7,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,16,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,16,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,15,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,12,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,10,0,0,0,12,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,8,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,9,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,5,0,0,0,6,0,0,0,17,0,0,0,8,0,0,0,12,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,13,0,0,0,5,0,0,0,2,0,0,0,17,0,0,0,4,0,0,0,9,0,0,0,5,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,6,0,0,0,4,0,0,0,16,0,0,0,5,0,0,0,10,0,0,0,5,0,0,0,7,0,0,0,10,0,0,0,14,0,0,0,16,0,0,0,13,0,0,0,9,0,0,0,16,0,0,0,11,0,0,0,8,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,13,0,0,0,16,0,0,0,7,0,0,0,4,0,0,0,16,0,0,0,5,0,0,0,7,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,8,0,0,0,6,0,0,0,16,0,0,0,7,0,0,0,8,0,0,0,5,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,9,0,0,0,8,0,0,0,16,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,11,0,0,0,11,0,0,0,16,0,0,0,10,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,16,0,0,0,13,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,2,0,0,0,5,0,0,0,20,0,0,0,7,0,0,0,10,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,4,0,0,0,2,0,0,0,20,0,0,0,5,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,10,0,0,0,10,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,19,0,0,0,7,0,0,0,5,0,0,0,19,0,0,0,6,0,0,0,10,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,17,0,0,0,11,0,0,0,8,0,0,0,19,0,0,0,10,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,7,0,0,0,5,0,0,0,19,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,6,0,0,0,9,0,0,0,11,0,0,0,16,0,0,0,7,0,0,0,6,0,0,0,19,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,13,0,0,0,9,0,0,0,9,0,0,0,19,0,0,0,11,0,0,0,9,0,0,0,8,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,13,0,0,0,12,0,0,0,14,0,0,0,19,0,0,0,16,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,8,0,0,0,9,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,19,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,12,0,0,0,11,0,0,0,11,0,0,0,13,0,0,0,3,0,0,0,4,0,0,0,19,0,0,0,7,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,11,0,0,0,9,0,0,0,12,0,0,0,4,0,0,0,1,0,0,0,19,0,0,0,6,0,0,0,7,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,8,0,0,0,6,0,0,0,18,0,0,0,8,0,0,0,9,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,14,0,0,0,18,0,0,0,9,0,0,0,6,0,0,0,18,0,0,0,9,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,18,0,0,0,7,0,0,0,6,0,0,0,18,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,17,0,0,0,8,0,0,0,8,0,0,0,18,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,17,0,0,0,10,0,0,0,10,0,0,0,18,0,0,0,12,0,0,0,9,0,0,0,8,0,0,0,7,0,0,0,9,0,0,0,12,0,0,0,18,0,0,0,13,0,0,0,15,0,0,0,18,0,0,0,15,0,0,0,13,0,0,0,11,0,0,0,10,0,0,0,11,0,0,0,15,0,0,0,18,0,0,0,14,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,16,0,0,0,16,0,0,0,18,0,0,0,18,0,0,0,1,0,0,0,64,0,0,0,16,8,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,16,9,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,16,10,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,48,10,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,50,0,0,0,48,12,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,18,0,0,0,248,12,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,64,13,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,50,0,0,0,64,15,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,18,0,0,0,8,16,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,80,16,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,32,0,0,0,80,18,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,208,18,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,208,20,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,208,24,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,208,25,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,25,0,0,0,208,26,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,9,0,0,0,56,27,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,96,27,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,25,0,0,0,96,28,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,9,0,0,0,200,28,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,240,28,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,240,32,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,25,0,0,0,240,33,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,10,0,0,0,88,34,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,128,34,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,144,34,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,144,38,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,25,0,0,0,144,39,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,10,0,0,0,248,39,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,32,40,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,48,40,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,48,41,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,48,45,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,16,0,0,0,80,45,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,144,45,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,50,0,0,0,144,47,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,18,0,0,0,88,48,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,160,48,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,50,0,0,0,160,50,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,18,0,0,0,104,51,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,176,51,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,32,0,0,0,176,53,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,48,54,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,32,0,0,0,48,56,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,176,56,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,176,58,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,176,62,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,176,63,28,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,25,0,0,0,176,64,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,9,0,0,0,24,65,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,64,65,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,25,0,0,0,64,66,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,9,0,0,0,168,66,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,208,66,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,208,70,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,25,0,0,0,208,71,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,10,0,0,0,56,72,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,96,72,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,112,72,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,112,73,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,112,74,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,144,74,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,50,0,0,0,144,76,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,18,0,0,0,88,77,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,160,77,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,50,0,0,0,160,79,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,18,0,0,0,104,80,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,176,80,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,32,0,0,0,176,82,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,48,83,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,48,85,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,48,86,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,48,87,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,80,87,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,50,0,0,0,80,89,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,18,0,0,0,24,90,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,96,90,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,50,0,0,0,96,92,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,18,0,0,0,40,93,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,112,93,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,32,0,0,0,112,95,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,240,95,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,240,97,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,240,98,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,240,102,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,16,0,0,0,16,103,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,80,103,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,50,0,0,0,80,105,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,18,0,0,0,24,106,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,96,106,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,50,0,0,0,96,108,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,18,0,0,0,40,109,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,112,109,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,32,0,0,0,112,111,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,240,111,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,32,0,0,0,240,113,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,128,0,0,0,112,114,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,112,116,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,0,118,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,0,119,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,144,120,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,32,122,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,32,123,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,32,124,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,176,125,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,64,127,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,208,128,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,96,130,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,240,131,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,128,133,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,16,135,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,160,136,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,48,138,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,139,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,140,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,141,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,142,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,143,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,144,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,145,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,146,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,147,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,148,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,49,0,0,0,192,149,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,49,0,0,0,136,150,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,80,151,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,96,151,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,8,0,0,0,128,151,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,160,151,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,160,152,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,160,153,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,160,154,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,160,155,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,160,156,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,160,157,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,176,157,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,176,158,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,176,159,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,159,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,192,160,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,192,161,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,208,161,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,208,162,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,208,163,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,224,163,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,224,164,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,224,165,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,49,0,0,0,240,165,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,49,0,0,0,184,166,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,128,167,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,49,0,0,0,144,167,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,49,0,0,0,88,168,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,32,169,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,48,169,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,120,170,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,192,171,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,8,173,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,80,174,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,224,175,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,112,177,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,0,179,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,144,180,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,32,182,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,176,183,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,64,185,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,208,186,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,96,188,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,240,189,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,128,191,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,16,193,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,160,194,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,48,196,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,192,197,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,80,199,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,152,200,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,224,201,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,40,203,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,112,204,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,184,205,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,0,207,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,81,0,0,0,72,208,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,144,209,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,32,211,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,176,212,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,64,214,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,64,0,0,0,208,215,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,208,216,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,96,218,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,240,219,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,128,221,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,100,0,0,0,16,223,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,0,64,0,0,0,0,0,0,8,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,240,63,0,0,0,0,0,0,248,63,0,0,0,0,0,0,0,64,0,0,0,0,0,0,0,64,0,0,0,0,0,0,4,64,154,153,153,153,153,153,5,64,0,0,0,0,0,0,8,64,154,153,153,153,153,153,13,64,0,0,0,0,0,0,16,64,0,0,0,0,0,0,16,64,0,0,0,0,0,0,240,63,0,0,0,0,0,0,0,64,0,0,0,0,0,0,8,64,0,0,0,0,0,0,16,64,6,0,0,0,6,0,0,0,16,255,28,0,0,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,8,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,128,255,28,0,80,255,28,0,32,255,28,0,0,0,0,0,9,0,0,0,9,0,0,0,9,0,0,0,0,0,0,0,9,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,208,255,28,0,192,255,28,0,6,0,0,0,6,0,0,0,232,255,28,0,0,0,0,0,16,1,29,0,152,0,29,0,240,0,29,0,96,0,29,0,88,1,29,0,40,1,29,0,128,0,29,0,136,1,29,0,176,0,29,0,48,0,29,0,40,154,47,0,0,0,0,0,248,148,3,0,216,148,3,0,240,224,28,0,200,224,28,0,160,224,28,0,88,226,28,0,48,226,28,0,8,226,28,0,224,225,28,0,184,225,28,0,144,225,28,0,104,225,28,0,64,225,28,0,24,225,28,0,168,226,28,0,128,226,28,0,152,227,28,0,112,227,28,0,72,227,28,0,32,227,28,0,248,226,28,0,208,226,28,0,192,227,28,0,96,228,28,0,56,228,28,0,16,228,28,0,232,227,28,0,0,0,0,0,136,228,28,0,40,229,28,0,0,229,28,0,216,228,28,0,176,228,28,0,0,0,0,0,200,229,28,0,160,229,28,0,120,229,28,0,80,229,28,0,128,231,28,0,88,231,28,0,48,231,28,0,8,231,28,0,224,230,28,0,184,230,28,0,144,230,28,0,104,230,28,0,64,230,28,0,24,230,28,0,240,229,28,0,0,0,0,0,208,231,28,0,168,231,28,0,192,232,28,0,152,232,28,0,112,232,28,0,72,232,28,0,32,232,28,0,248,231,28,0,232,232,28,0,136,233,28,0,96,233,28,0,56,233,28,0,16,233,28,0,0,0,0,0,0,234,28,0,216,233,28,0,176,233,28,0,104,235,28,0,64,235,28,0,24,235,28,0,240,234,28,0,200,234,28,0,160,234,28,0,120,234,28,0,80,234,28,0,40,234,28,0,224,235,28,0,184,235,28,0,144,235,28,0,72,237,28,0,32,237,28,0,248,236,28,0,208,236,28,0,168,236,28,0,128,236,28,0,88,236,28,0,48,236,28,0,8,236,28,0,232,237,28,0,192,237,28,0,152,237,28,0,112,237,28,0,160,239,28,0,120,239,28,0,80,239,28,0,40,239,28,0,0,239,28,0,216,238,28,0,176,238,28,0,136,238,28,0,96,238,28,0,56,238,28,0,16,238,28,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,128,0,0,0,33,0,0,0,8,0,0,0,16,0,0,0,70,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,128,63,0,0,144,65,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,1,0,0,66,0,0,0,16,0,0,0,32,0,0,0,140,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,128,63,0,0,144,65,0,1,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,2,0,0,0,3,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,5,0,0,0,6,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,128,0,0,0,14,0,0,0,4,0,0,0,58,0,0,0,2,0,0,0,8,0,0,0,28,0,0,0,90,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,128,63,0,0,144,65,128,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
.concat([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,2,0,0,0,3,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,5,0,0,0,6,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,1,0,0,28,0,0,0,8,0,0,0,116,0,0,0,4,0,0,0,16,0,0,0,56,0,0,0,180,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,128,63,0,0,144,65,0,1,0,0,4,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,6,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,128,0,0,0,8,0,0,0,33,0,0,0,4,0,0,0,16,0,0,0,70,0,0,0,2,0,0,0,6,0,0,0,12,0,0,0,23,0,0,0,46,0,0,0,90,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,128,63,0,0,144,65,128,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,6,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,128,0,0,0,12,0,0,0,46,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,23,0,0,0,33,0,0,0,70,0,0,0,2,0,0,0,6,0,0,0,10,0,0,0,14,0,0,0,19,0,0,0,28,0,0,0,39,0,0,0,58,0,0,0,90,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,128,63,0,0,144,65,128,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,1,0,0,66,0,0,0,16,0,0,0,32,0,0,0,140,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,128,63,0,0,144,65,0,1,0,0,8,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,12,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,4,0,0,93,0,0,0,23,0,0,0,116,1,0,0,6,0,0,0,46,0,0,0,186,0,0,0,238,2,0,0,14,0,0,0,33,0,0,0,65,0,0,0,130,0,0,0,4,1,0,0,44,2,0,0,3,0,0,0,10,0,0,0,18,0,0,0,28,0,0,0,39,0,0,0,55,0,0,0,79,0,0,0,111,0,0,0,158,0,0,0,220,0,0,0,56,1,0,0,208,1,0,0,138,2,0,0,82,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,64,64,0,0,144,65,0,4,0,0,8,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,4,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,12,0,0,0,13,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,8,0,0,186,0,0,0,46,0,0,0,232,2,0,0,12,0,0,0,92,0,0,0,116,1,0,0,220,5,0,0,28,0,0,0,66,0,0,0,130,0,0,0,4,1,0,0,8,2,0,0,88,4,0,0,6,0,0,0,20,0,0,0,36,0,0,0,56,0,0,0,78,0,0,0,110,0,0,0,158,0,0,0,222,0,0,0,60,1,0,0,184,1,0,0,112,2,0,0,160,3,0,0,20,5,0,0,164,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,64,64,0,0,144,65,0,8,0,0,6,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,6,0,0,0,7,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,9,0,0,0,10,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,2,0,0,46,0,0,0,186,0,0,0,16,0,0,0,33,0,0,0,65,0,0,0,93,0,0,0,130,0,0,0,22,1,0,0,7,0,0,0,23,0,0,0,39,0,0,0,55,0,0,0,79,0,0,0,110,0,0,0,156,0,0,0,232,0,0,0,104,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,128,63,0,0,144,65,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,66,0,0,240,65,0,0,250,67,0,0,128,63,0,0,144,65,10,0,0,0,120,196,3,0,0,0,0,0,8,0,0,0,255,255,255,255,8,0,0,0,255,255,255,255,8,0,0,0,255,255,255,255,1,0,0,0,255,255,255,255,0,0,0,0,255,255,255,255,6,0,0,0,1,0,0,0,255,255,255,255,0,0,0,0,8,0,0,0,255,255,255,255,7,0,0,0,8,0,0,0,6,0,0,0,255,255,255,255,1,0,0,0,255,255,255,255,8,213,3,0,0,0,0,0,4,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,56,215,3,0,96,220,3,0,0,0,0,0,0,0,0,0,176,140,4,0,240,221,3,0,8,213,3,0,0,0,0,0,102,102,95,103,101,110,95,115,101,97,114,99,104,0,0,0,97,115,102,95,114,101,97,100,95,112,97,99,107,101,116,0,97,115,102,95,112,97,114,115,101,95,112,97,99,107,101,116,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,7,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,20,0,0,0,24,0,0,0,28,0,0,0,33,0,0,0,254,255,255,255,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,20,0,0,0,24,0,0,0,252,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,20,0,0,0,250,255,255,255,252,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,251,255,255,255,253,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,16,0,0,0,251,255,255,255,252,255,255,255,253,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,248,255,255,255,249,255,255,255,250,255,255,255,251,255,255,255,252,255,255,255,253,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,13,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,21,0,0,0,24,0,0,0,27,0,0,0,31,0,0,0,35,0,0,0,39,0,0,0,44,0,0,0,50,0,0,0,57,0,0,0,64,0,0,0,15,0,0,0,17,0,0,0,19,0,0,0,21,0,0,0,23,0,0,0,26,0,0,0,29,0,0,0,33,0,0,0,37,0,0,0,41,0,0,0,46,0,0,0,51,0,0,0,57,0,0,0,64,0,0,0,20,0,0,0,22,0,0,0,24,0,0,0,26,0,0,0,29,0,0,0,31,0,0,0,34,0,0,0,37,0,0,0,41,0,0,0,45,0,0,0,49,0,0,0,54,0,0,0,59,0,0,0,64,0,0,0,21,0,0,0,23,0,0,0,25,0,0,0,27,0,0,0,30,0,0,0,32,0,0,0,35,0,0,0,38,0,0,0,42,0,0,0,45,0,0,0,49,0,0,0,54,0,0,0,59,0,0,0,64,0,0,0,23,0,0,0,25,0,0,0,27,0,0,0,29,0,0,0,32,0,0,0,34,0,0,0,37,0,0,0,40,0,0,0,43,0,0,0,47,0,0,0,51,0,0,0,55,0,0,0,59,0,0,0,64,0,0,0,32,0,0,0,34,0,0,0,36,0,0,0,38,0,0,0,40,0,0,0,42,0,0,0,44,0,0,0,46,0,0,0,49,0,0,0,52,0,0,0,55,0,0,0,58,0,0,0,61,0,0,0,64,0,0,0,35,0,0,0,37,0,0,0,38,0,0,0,40,0,0,0,42,0,0,0,44,0,0,0,46,0,0,0,48,0,0,0,51,0,0,0,53,0,0,0,56,0,0,0,58,0,0,0,61,0,0,0,64,0,0,0,48,0,0,0,49,0,0,0,50,0,0,0,51,0,0,0,52,0,0,0,54,0,0,0,55,0,0,0,56,0,0,0,57,0,0,0,59,0,0,0,60,0,0,0,61,0,0,0,63,0,0,0,64,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,3,0,0,224,1,0,0,0,3,0,0,0,3,0,0,0,254,0,0,160,254,0,0,0,254,0,0,96,253,0,0,0,254,0,0,192,254,9,0,0,0,198,167,0,0,146,10,0,0,212,125,0,0,143,0,0,0,146,10,0,0,160,153,153,9,52,51,51,3,128,102,102,38,0,0,0,64,37,76,252,255,37,76,252,255,62,44,91,255,50,0,0,0,0,2,4,6,8,12,16,20,24,0,0,2,4,8,12,0,0,0,0,0,0,0,0,0,9,5,0,0,0,0,0,0,255,255,255,127,255,255,255,127,101,126,162,107,181,75,66,97,153,121,130,90,69,72,153,85,60,227,200,81,209,96,177,78,255,255,255,127,127,157,239,119,127,201,171,107,127,242,234,76,192,211,14,47,0,0,0,0,1,166,155,180,0,0,0,128,255,255,255,127,128,227,165,107,0,250,126,90,128,186,73,76,0,0,0,64,128,24,4,54,192,163,112,45,0,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,0,0,0,0,3,0,0,0,5,0,0,0,1,0,0,0,4,0,0,0,5,0,0,0,1,0,0,0,4,0,0,0,5,0,0,0,10,0,0,0,20,0,0,0,0,128,0,192,85,213,0,224,102,230,171,234,183,237,0,240,199,241,51,243,0,0,0,0,255,255,255,127,255,255,255,127,0,0,0,64,170,170,170,42,0,0,0,32,153,153,153,25,85,85,85,21,146,36,73,18,83,127,0,64,64,55,34,12,18,122,0,64,134,53,6,13,101,112,0,64,57,50,192,14,170,99,0,64,154,45,51,17,97,85,0,64,1,40,61,20])
.concat([215,70,0,64,203,33,183,23,0,57,0,64,83,27,127,27,114,44,0,64,230,20,120,31,125,33,0,64,198,14,142,35,59,24,0,64,37,9,180,39,177,16,1,64,43,4,227,43,199,10,255,63,243,255,31,48,114,6,2,64,149,252,112,52,152,3,255,63,39,250,227,56,48,2,1,64,190,248,141,61,233,126,0,64,90,41,242,6,157,118,0,64,44,39,82,8,7,104,0,64,16,35,244,10,29,86,0,64,119,29,163,14,159,67,0,64,232,22,30,19,110,50,0,64,237,15,40,24,141,35,0,64,0,9,145,29,92,23,0,64,131,2,57,35,224,13,0,64,195,252,17,41,249,6,0,64,246,247,33,47,124,2,0,64,77,244,126,53,70,0,0,64,241,241,84,60,144,125,0,64,213,10,211,0,27,108,0,64,252,7,113,3,117,81,0,64,198,2,99,8,137,54,0,64,255,251,56,15,63,32,0,64,136,244,133,23,0,16,0,64,40,237,16,33,161,5,0,64,119,230,238,43,158,0,0,64,233,224,163,56,182,123,0,64,254,246,203,0,76,95,0,64,80,243,199,4,200,58,0,64,183,236,78,12,29,29,0,64,85,228,215,22,29,10,0,64,54,219,64,36,26,1,0,64,3,210,91,53,157,118,0,64,16,211,122,8,159,67,0,64,121,205,32,15,92,23,0,64,222,194,90,28,124,2,0,64,81,179,80,49,0,1,2,3,4,1,1,2,2,4,2,2,2,2,5,3,2,2,3,4,4,4,5,4,4,0,0,0,0,0,0,0,0,0,0,1,1,2,2,3,19,0,0,0,19,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,17,0,0,0,15,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,6,0,0,0,4,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,17,0,0,0,19,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,20,0,0,0,0,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,15,0,0,0,13,0,0,0,10,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,3,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,6,0,0,0,8,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,4,16,215,3,123,20,76,4,120,5,0,64,16,0,31,0,4,16,102,6,205,12,130,5,120,5,0,64,16,0,31,0,4,16,102,6,205,12,76,4,120,5,0,64,16,0,31,0,4,16,154,25,205,12,130,5,120,5,0,64,16,0,31,0,0,0,0,0,95,109,0,0,0,0,0,0,0,0,0,0,96,109,0,0,255,124,0,0,2,2,2,2,2,0,0,0,0,125,0,0,127,187,0,0,3,3,3,3,3,0,0,0,128,187,0,0,128,187,0,0,4,4,4,4,4,0,0,0,129,187,0,0,63,66,15,0,0,0,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,13,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,9,0,0,0,9,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,3,0,0,0,1,0,0,0,2,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,12,0,0,0,13,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,0,0,0,0,249,255,7,0,250,255,7,0,251,255,7,0,248,255,15,0,249,255,15,0,250,255,15,0,253,255,1,0,254,127,0,0,254,15,0,0,254,3,0,0,254,0,0,0,62,0,0,0,14,0,0,0,2,0,0,0,0,0,0,0,6,0,0,0,30,0,0,0,126,0,0,0,254,1,0,0,254,7,0,0,254,31,0,0,254,63,0,0,252,255,1,0,248,255,7,0,251,255,15,0,252,255,15,0,253,255,15,0,254,255,15,0,255,255,15,0,0,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,17,0,0,0,16,0,0,0,16,0,0,0,15,0,0,0,14,0,0,0,14,0,0,0,13,0,0,0,12,0,0,0,12,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,8,0,0,0,7,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,3,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,16,0,0,0,17,0,0,0,17,0,0,0,18,0,0,0,17,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,18,0,0,0,0,0,0,0,251,255,1,0,252,255,1,0,253,255,1,0,250,255,1,0,252,255,0,0,252,127,0,0,253,31,0,0,254,3,0,0,254,1,0,0,126,0,0,0,60,0,0,0,29,0,0,0,13,0,0,0,5,0,0,0,0,0,0,0,4,0,0,0,12,0,0,0,28,0,0,0,61,0,0,0,62,0,0,0,254,0,0,0,254,7,0,0,252,31,0,0,252,63,0,0,253,63,0,0,253,127,0,0,254,255,1,0,254,255,3,0,255,255,3,0,0,0,0,0,48,101,29,0,0,100,29,0,64,110,29,0,4,0,0,0,4,0,0,0,4,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,252,255,255,255,253,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,64,31,0,0,208,7,0,0,208,7,0,0,192,93,0,0,208,7,0,0,208,7,0,0,0,125,0,0,48,17,0,0,68,22,0,0,64,156,0,0,232,28,0,0,96,34,0,0,128,187,0,0,40,35,0,0,204,41,0,0,192,218,0,0,248,42,0,0,0,50,0,0,0,250,0,0,220,55,0,0,40,60,0,0,128,56,1,0,128,62,0,0,72,63,0,0,0,119,1,0,116,64,0,0,128,62,0,0,0,244,1,0,128,62,0,0,128,62,0,0,65,126,5,0,128,62,0,0,128,62,0,0,0,0,0,0,64,31,0,0,208,7,0,0,208,7,0,0,192,93,0,0,208,7,0,0,208,7,0,0,0,125,0,0,48,17,0,0,68,22,0,0,64,156,0,0,232,28,0,0,96,34,0,0,128,187,0,0,40,35,0,0,204,41,0,0,192,218,0,0,248,42,0,0,100,50,0,0,0,250,0,0,64,56,0,0,140,60,0,0,128,56,1,0,128,62,0,0,72,63,0,0,0,119,1,0,116,64,0,0,128,62,0,0,0,244,1,0,128,62,0,0,128,62,0,0,65,126,5,0,128,62,0,0,128,62,0,0,0,0,0,0,64,31,0,0,208,7,0,0,208,7,0,0,224,46,0,0,208,7,0,0,208,7,0,0,192,93,0,0,154,16,0,0,80,20,0,0,0,125,0,0,208,32,0,0,40,35,0,0,64,156,0,0,184,36,0,0,36,44,0,0,128,187,0,0,124,46,0,0,132,53,0,0,0,250,0,0,208,57,0,0,128,62,0,0,224,40,1,0,128,62,0,0,128,62,0,0,65,126,5,0,128,62,0,0,128,62,0,0,0,0,0,0,64,31,0,0,208,7,0,0,208,7,0,0,224,46,0,0,208,7,0,0,252,8,0,0,128,62,0,0,152,8,0,0,196,9,0,0,192,93,0,0,18,22,0,0,0,25,0,0,0,125,0,0,80,45,0,0,224,46,0,0,64,156,0,0,224,46,0,0,128,62,0,0,128,187,0,0,128,62,0,0,128,62,0,0,0,250,0,0,128,62,0,0,128,62,0,0,65,126,5,0,128,62,0,0,128,62,0,0,0,0,0,0,64,31,0,0,208,7,0,0,96,9,0,0,224,46,0,0,196,9,0,0,140,10,0,0,128,62,0,0,228,12,0,0,28,12,0,0,192,93,0,0,106,24,0,0,32,28,0,0,0,125,0,0,240,35,0,0,4,41,0,0,64,156,0,0,128,62,0,0,128,62,0,0,128,187,0,0,128,62,0,0,128,62,0,0,65,126,5,0,128,62,0,0,128,62,0,0,1,0,0,0,3,0,0,0,3,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,2,0,0,0,3,0,0,0,1,0,0,0,3,0,0,0,1,0,0,0,3,0,0,0,1,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,1,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,1,0,0,0,1,0,0,0,212,78,0,0,213,78,0,0,206,78,0,0,207,78,0,0,204,78,0,0,214,78,0,0,216,78,0,0,70,79,0,0,96,79,0,0,24,39,0,0,25,39,0,0,100,39,0,0,101,39,0,0,109,39,0,0,177,39,0,0,183,19,0,0,214,19,0,0,199,9,0,0,233,9,0,0,237,9,0,0,238,4,0,0,247,4,0,0,120,2,0,0,57,1,0,0,154,0,0,0,159,0,0,0,32,0,0,0,17,0,0,0,10,0,0,0,3,0,0,0,1,0,0,0,0,0,0,0,11,0,0,0,18,0,0,0,33,0,0,0,76,0,0,0,155,0,0,0,58,1,0,0,121,2,0,0,112,2,0,0,239,4,0,0,226,4,0,0,234,9,0,0,216,9,0,0,215,19,0,0,208,19,0,0,178,39,0,0,162,39,0,0,26,39,0,0,27,39,0,0,102,79,0,0,103,79,0,0,97,79,0,0,71,79,0,0,217,78,0,0,215,78,0,0,205,78,0,0,210,78,0,0,211,78,0,0,208,78,0,0,209,78,0,0,0,0,0,0,180,254,1,0,181,254,1,0,118,253,1,0,119,253,1,0,116,253,1,0,117,253,1,0,138,254,1,0,139,254,1,0,136,254,1,0,128,254,0,0,182,254,1,0,130,254,0,0,184,254,0,0,66,127,0,0,174,127,0,0,175,63,0,0,209,31,0,0,233,31,0,0,233,15,0,0,234,7,0,0,251,7,0,0,251,3,0,0,251,1,0,0,255,1,0,0,124,0,0,0,60,0,0,0,28,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,1,0,0,0,13,0,0,0,29,0,0,0,61,0,0,0,125,0,0,0,252,0,0,0,252,1,0,0,252,3,0,0,244,3,0,0,235,7,0,0,234,15,0,0,234,31,0,0,214,31,0,0,208,63,0,0,175,127,0,0,67,127,0,0,185,254,0,0,131,254,0,0,183,254,1,0,129,254,0,0,137,254,1,0,142,254,1,0,143,254,1,0,140,254,1,0,141,254,1,0,178,254,1,0,179,254,1,0,176,254,1,0,177,254,1,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,5,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,1,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,2,0,0,0,0,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,3,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,4,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,5,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,6,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,7,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,2,3,4,5,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,18,0,0,0,21,0,0,0,25,0,0,0,30,0,0,0,42,0,0,0,71,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,225,255,0,0,225,255,0,0,0,0,0,0,225,255,0,0,0,0,0,0,0,0,154,0,170,0,186,0,147,0,156,0,165,0,114,0,120,0,186,0,126,0,132,0,138,0,142,0,146,0,154,0,166,0,174,0,182,0,190,0,198,0,206,0,210,0,214,0,110,0,114,0,118,0,120,0,122,0,218,0,230,0,242,0,254,0,10,1,22,1,30,1,38,1,62,1,86,1,102,1,118,1,134,1,150,1,166,1,136,0,142,0,148,0,154,0,160,0,166,0,170,0,174,0,186,0,198,0,206,0,214,0,222,0,230,0,238,0,216,0,160,0,24,1,82,1,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,1,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,2,0,0,0,0,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,3,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,4,0,0,0,3,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,5,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,6,0,0,0,4,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,5,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,7,0,0,0,6,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,7,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,1,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,2,0,0,0,0,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,3,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,4,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,5,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,6,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,7,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,6,0,0,0,7,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,18,0,0,0,19,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,3,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,5,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,1,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,14,0,0,0,13,0,0,0,11,0,0,0,9,0,0,0,7,0,0,0,5,0,0,0,3,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,0,0,0,0,14,0,0,0,14,0,0,0,12,0,0,0,10,0,0,0,7,0,0,0,5,0,0,0,3,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,13,0,0,0,0,0,0,0,0,0,5,128,15,128,10,0,27,128,30,0,20,0,17,128,51,128,54,0,60,0,57,128,40,0,45,128,39,128,34,0,99,128,102,0,108,0,105,128,120,0,125,128,119,128,114,0,80,0,85,128,95,128,90,0,75,128,78,0,68,0,65,128,195,128,198,0,204,0,201,128,216,0,221,128,215,128,210,0,240,0,245,128,255,128,250,0,235,128,238,0,228,0,225,128,160,0,165,128,175,128,170,0,187,128,190,0,180,0,177,128,147,128,150,0,156,0,153,128,136,0,141,128,135,128,130,0,131,129,134,1,140,1,137,129,152,1,157,129,151,129,146,1,176,1,181,129,191,129,186,1,171,129,174,1,164,1,161,129,224,1,229,129,239,129,234,1,251,129,254,1,244,1,241,129,211,129,214,1,220,1,217,129,200,1,205,129,199,129,194,1,64,1,69,129,79,129,74,1,91,129,94,1,84,1,81,129,115,129,118,1,124,1,121,129,104,1,109,129,103,129,98,1,35,129,38,1,44,1,41,129,56,1,61,129,55,129,50,1,16,1,21,129,31,129,26,1,11,129,14,1,4,1,1,129,3,131,6,3,12,3,9,131,24,3,29,131,23,131,18,3,48,3,53,131,63,131,58,3,43,131,46,3,36,3,33,131,96,3,101,131,111,131,106,3,123,131,126,3,116,3,113,131,83,131,86,3,92,3,89,131,72,3,77,131,71,131,66,3,192,3,197,131,207,131,202,3,219,131,222,3,212,3,209,131,243,131,246,3,252,3,249,131,232,3,237,131,231,131,226,3,163,131,166,3,172,3,169,131,184,3,189,131,183,131,178,3,144,3,149,131,159,131,154,3,139,131,142,3,132,3,129,131,128,2,133,130,143,130,138,2,155,130,158,2,148,2,145,130,179,130,182,2,188,2,185,130,168,2,173,130,167,130,162,2,227,130,230,2,236,2,233,130,248,2,253,130,247,130,242,2,208,2,213,130,223,130,218,2,203,130,206,2,196,2,193,130,67,130,70,2,76,2,73,130,88,2,93,130,87,130,82,2,112,2,117,130,127,130,122,2,107,130,110,2,100,2,97,130,32,2,37,130,47,130,42,2,59,130,62,2,52,2,49,130,19,130,22,2,28,2,25,130,8,2,13,130,7,130,2,2,0,0,33,16,66,32,99,48,132,64,165,80,198,96,231,112,8,129,41,145,74,161,107,177,140,193,173,209,206,225,239,241,49,18,16,2,115,50,82,34,181,82,148,66,247,114,214,98,57,147,24,131,123,179,90,163,189,211,156,195,255,243,222,227,98,36,67,52,32,4,1,20,230,100,199,116,164,68,133,84,106,165,75,181,40,133,9,149,238,229,207,245,172,197,141,213,83,54,114,38,17,22,48,6,215,118,246,102,149,86,180,70,91,183,122,167,25,151,56,135,223,247,254,231,157,215,188,199,196,72,229,88,134,104,167,120,64,8,97,24,2,40,35,56,204,201,237,217,142,233,175,249,72,137,105,153,10,169,43,185,245,90,212,74,183,122,150,106,113,26,80,10,51,58,18,42,253,219,220,203,191,251,158,235,121,155,88,139,59,187,26,171,166,108,135,124,228,76,197,92,34,44,3,60,96,12,65,28,174,237,143,253,236,205,205,221,42,173,11,189,104,141,73,157,151,126,182,110,213,94,244,78,19,62,50,46,81,30,112,14,159,255,190,239,221,223,252,207,27,191,58,175,89,159,120,143,136,145,169,129,202,177,235,161,12,209,45,193,78,241,111,225,128,16,161,0,194,48,227,32,4,80,37,64,70,112,103,96,185,131,152,147,251,163,218,179,61,195,28,211,127,227,94,243,177,2,144,18,243,34,210,50,53,66,20,82,119,98,86,114,234,181,203,165,168,149,137,133,110,245,79,229,44,213,13,197,226,52,195,36,160,20,129,4,102,116,71,100,36,84,5,68,219,167,250,183,153,135,184,151,95,231,126,247,29,199,60,215,211,38,242,54,145,6,176,22,87,102,118,118,21,70,52,86,76,217,109,201,14,249,47,233,200,153,233,137,138,185,171,169,68,88,101,72,6,120,39,104,192,24,225,8,130,56,163,40,125,203,92,219,63,235,30,251,249,139,216,155,187,171,154,187,117,74,84,90,55,106,22,122,241,10,208,26,179,42,146,58,46,253,15,237,108,221,77,205,170,189,139,173,232,157,201,141,38,124,7,108,100,92,69,76,162,60,131,44,224,28,193,12,31,239,62,255,93,207,124,223,155,175,186,191,217,143,248,159,23,110,54,126,85,78,116,94,147,46,178,62,209,14,240,30,0,119,1,0,31,9,0,0,136,88,1,0,31,9,0,0,0,250,0,0,34,10,0,0,128,187,0,0,40,14,0,0,68,172,0,0,42,14,0,0,0,125,0,0,51,14,0,0,192,93,0,0,46,14,0,0,34,86,0,0,46,14,0,0,128,62,0,0,42,14,0,0,224,46,0,0,42,14,0,0,17,43,0,0,42,14,0,0,64,31,0,0,39,14,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,225,122,20,2,0,0,0,0,170,241,18,3,0,0,0,32,174,71,17,4,0,0,0,128,63,53,14,5,0,0,0,144,194,245,8,7,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,240,89,29,0,0,0,0,0,0,0,0,0,0,0,0,0,128,90,29,0,0,0,0,0,0,0,0,0,0,0,0,0,224,90,29,0,0,0,0,0,0,0,0,0,0,0,0,0,96,91,29,0,0,0,0,0,0,0,0,0,0,0,0,0,128,187,0,0,31,255,0,0,68,172,0,0,32,255,0,0,0,125,0,0,37,255,0,0,192,93,0,0,31,255,0,0,34,86,0,0,31,255,0,0,128,187,0,0,31,255,0,0,68,172,0,0,32,255,0,0,0,125,0,0,37,255,0,0,192,93,0,0,30,255,0,0,34,86,0,0,30,255,0,0,1,0,0,0,2,0,0,0,1,0,0,0,7,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,1,0,0,0,1,0,0,0,6,0,0,0,4,0,0,0,15,0,0,0,14,0,0,0,5,0,0,0,0,0,0,0,96,115,29,0,0,0,0,0,0,0,0,0,0,0,0,0,128,106,29,0,0,0,0,0,0,0,0,0,0,0,0,0,32,98,29,0,0,0,0,0,0,0,0,0,0,0,0,0,72,98,29,0,0,0,0,0,0,0,0,0,0,0,0,0,168,98,29,0,0,0,0,0,0,0,0,0,0,0,0,0,248,98,29,0,0,0,0,0,0,0,0,0,0,0,0,0,48,99,29,0,0,0,0,0,0,0,0,0,0,0,0,0,144,115,29,0,192,79,29,0,160,79,29,0,0,0,0,0,144,115,29,0,208,79,29,0,176,79,29,0,0,0,0,0,0,0,0,0,223,46,0,0,1,1,1,1,1,0,0,0,224,46,0,0,31,78,0,0,3,1,1,1,1,0,0,0,32,78,0,0,71,113,0,0,3,3,3,2,2,0,0,0,72,113,0,0,39,160,0,0,7,6,6,5,5,0,0,0,40,160,0,0,191,218,0,0,9,9,7,7,7,0,0,0,192,218,0,0,127,56,1,0,0,0,0,0,0,0,0,0,128,56,1,0,159,134,1,0,0,0,0,0,0,0,0,0,160,134,1,0,63,66,15,0,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,1,0,0,0,0,0,0,0,6,0,0,0,4,0,0,0,2,0,0,0,3,0,0,0,5,0,0,0,7,0,0,0,254,63,0,0,254,31,0,0,254,7,0,0,254,1,0,0,126,0,0,0,30,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,14,0,0,0,62,0,0,0,254,0,0,0,254,3,0,0,254,15,0,0,255,63,0,0,0,0,0,0,255,63,0,0,254,63,0,0,254,15,0,0,254,3,0,0,126,0,0,0,30,0,0,0,6,0,0,0,0,0,0,0,2,0,0,0,14,0,0,0,62,0,0,0,254,0,0,0,254,1,0,0,254,7,0,0,254,31,0,0,0,0,0,0,48,101,29,0,0,100,29,0,64,110,29,0,4,0,0,0,4,0,0,0,4,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,252,255,255,255,253,255,255,255,254,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,3,0,0,0,3,0,0,0,3,0,0,0,2,0,0,0,4,0,0,0,4,0,0,0,4,0,0,0,3,0,0,0,5,0,0,0,5,0,0,0,5,0,0,0,3,0,0,0,6,0,0,0,6,0,0,0,5,0,0,0,4,0,0,0,7,0,0,0,8,0,0,0,7,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,250,50,0,0,250,50,0,0,2,0,0,0,250,50,0,0,250,50,0,0,3,0,0,0,180,55,0,0,180,55,0,0,4,0,0,0,140,60,0,0,140,60,0,0,5,0,0,0,128,187,0,0,128,187,0,0,7,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,119,1,0,136,88,1,0,0,250,0,0,128,187,0,0,68,172,0,0,0,125,0,0,192,93,0,0,34,86,0,0,128,62,0,0,224,46,0,0,17,43,0,0,64,31,0,0,182,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,119,1,0,136,88,1,0,0,250,0,0,128,187,0,0,68,172,0,0,0,125,0,0,192,93,0,0,34,86,0,0,128,62,0,0,224,46,0,0,17,43,0,0,64,31,0,0,182,28,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,192,0,0,0,192,0,0,0,192,0,0,0,0,0,0,0,192,0,0,0,192,36,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,12,12,12,12,16,20,24,28,32,32,32,32,32,32,32,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,35,4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,12,12,12,12,12,16,16,24,28,32,32,32,32,32,32,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,37,4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,12,12,12,12,16,16,16,20,24,24,28,32,32,32,32,32,32,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,37,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,8,12,12,12,16,16,20,24,32,32,32,32,32,32,32,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,31,4,4,4,4,4,4,4,4,4,4,4,8,8,8,12,12,12,16,20,24,28,32,32,32,32,32,32,32,32,32,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,30,4,4,4,4,4,4,4,4,4,4,4,8,8,8,12,12,12,16,20,24,28,32,32,32,32,32,32,32,32,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,4,0,0,0,6,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,255,255,255,255,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,6,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,3,0,0,0,7,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,4,0,0,0,8,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,5,0,0,0,9,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,6,0,0,0,10,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,7,0,0,0,11,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,8,0,0,0,12,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,9,0,0,0,13,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,10,0,0,0,14,0,0,0,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,11,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,12,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,13,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,14,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,15,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,255,255,255,255,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,6,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,3,0,0,0,7,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,4,0,0,0,8,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,5,0,0,0,9,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,6,0,0,0,10,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,7,0,0,0,11,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,8,0,0,0,12,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,9,0,0,0,13,0,0,0,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,10,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,11,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,12,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,13,0,0,0,255,255,255,255,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,14,0,0,0,255,255,255,255,255,255,255,255,3,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,35,0,0,0,6,0,0,0,7,0,0,0,2,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,35,0,0,0,15,0,0,0,35,0,0,0,15,0,0,0,35,0,0,0,16,0,0,0,35,0,0,0,16,0,0,0,35,0,0,0,12,0,0,0,35,0,0,0,12,0,0,0,35,0,0,0,17,0,0,0,35,0,0,0,17,0,0,0,38,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,17,0,0,0,35,0,0,0,2,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,17,0,0,0,38,0,0,0,2,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,35,0,0,0,2,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,35,0,0,0,15,0,0,0,35,0,0,0,15,0,0,0,35,0,0,0,16,0,0,0,35,0,0,0,16,0,0,0,35,0,0,0,12,0,0,0,35,0,0,0,12,0,0,0,35,0,0,0,17,0,0,0,35,0,0,0,17,0,0,0,38,0,0,0,2,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,17,0,0,0,35,0,0,0,2,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,17,0,0,0,38,0,0,0,64,31,0,0,128,85,29,0,0,0,0,0,17,43,0,0,128,85,29,0,0,0,0,0,224,46,0,0,128,85,29,0,0,0,0,0,128,62,0,0,128,85,29,0,0,0,0,0,34,86,0,0,128,85,29,0,0,0,0,0,192,93,0,0,128,85,29,0,0,0,0,0,0,125,0,0,16,85,29,0,0,0,0,0,68,172,0,0,160,84,29,0,0,0,0,0,128,187,0,0,160,84,29,0,0,0,0,0,0,250,0,0,160,84,29,0,0,0,0,0,136,88,1,0,160,84,29,0,0,0,0,0,0,119,1,0,160,84,29,0,0,0,0,0,64,31,0,0,184,85,29,0,0,0,0,0,17,43,0,0,184,85,29,0,0,0,0,0,224,46,0,0,184,85,29,0,0,0,0,0,128,62,0,0,184,85,29,0,0,0,0,0,34,86,0,0,184,85,29,0,0,0,0,0,192,93,0,0,184,85,29,0,0,0,0,0,0,125,0,0,72,85,29,0,0,0,0,0,68,172,0,0,216,84,29,0,0,0,0,0,128,187,0,0,216,84,29,0,0,0,0,0,0,250,0,0,216,84,29,0,0,0,0,0,136,88,1,0,216,84,29,0,0,0,0,0,0,119,1,0,216,84,29,0,0,0,0,0,0,0,0,0,223,46,0,0,1,1,1,1,1,0,0,0,224,46,0,0,31,78,0,0,1,1,1,1,1,0,0,0,32,78,0,0,71,113,0,0,2,1,1,1,1,0,0,0,72,113,0,0,39,160,0,0,4,4,4,2,2,0,0,0,40,160,0,0,191,218,0,0,9,9,7,7,7,0,0,0,192,218,0,0,127,56,1,0,0,0,0,9,9,0,0,0,128,56,1,0,159,134,1,0,0,0,0,0,0,0,0,0,160,134,1,0,63,66,15,0,0,0,0,0,0,0,0,0,1,0,0,156,1,0,0,166,1,0,0,176,1,0,0,186,0,0,0,196,0,0,0,206,0,0,0,212,0,0,0,218,0,0,0,224,0,0,0,230,0,0,0,236,0,0,0,240,0,0,0,244,0,0,0,248,0,0,0,252,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,16,0,0,0,20,0,0,0,26,0,0,0,32,0,0,0,38,0,0,0,44,0,0,0,50,0,0,0,60,255,255,255,69,255,255,255,79,255,255,255,89,255,255,255,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,5,0,0,0,9,0,0,0,14,0,0,0,18,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,9,0,0,0,18,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,16,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,16,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,12,0,0,0,15,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,15,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,5,0,0,0,7,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,5,0,0,0,9,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,9,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,9,0,0,0,18,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,16,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,15,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0])
.concat([5,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,5,0,0,0,9,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,200,105,29,0,224,105,29,0,248,105,29,0,16,106,29,0,40,106,29,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,255,255,255,255,2,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,255,255,255,255,3,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,3,0,0,0,1,0,0,0,1,0,0,0,3,0,0,0,5,0,0,0,3,0,0,0,1,0,0,0,1,0,0,0,4,0,0,0,6,0,0,0,2,0,0,0,1,0,0,0,1,0,0,0,5,0,0,0,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,6,0,0,0,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,7,0,0,0,255,255,255,255,2,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,38,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,35,0,0,0,2,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,35,0,0,0,12,0,0,0,35,0,0,0,12,0,0,0,35,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,35,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,38,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,35,0,0,0,2,0,0,0,8,0,0,0,9,0,0,0,11,0,0,0,12,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,38,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,17,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,12,0,0,0,17,0,0,0,38,0,0,0,0,0,160,250,0,0,96,249,0,0,32,249,0,0,224,247,0,0,192,249,0,0,0,251,0,0,192,252,0,0,96,253,0,0,192,252,0,0,96,253,0,0,192,252,0,0,128,253,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,0,0,0,0,0,0,192,0,0,0,192,1,0,0,128,2,28,0,0,0,28,0,0,0,12,0,0,0,96,0,0,0,96,0,0,0,46,0,0,0,46,0,0,0,46,0,0,0,80,0,0,0,80,0,0,0,80,0,0,0,80,0,0,0,80,0,0,0,118,0,0,0,118,0,0,0,118,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,116,14,0,0,136,19,0,0,224,46,0,0,136,19,0,0,0,25,0,0,32,78,0,0,244,26,0,0,168,37,0,0,96,109,0,0,128,37,0,0,250,50,0,0,64,156,0,0,28,47,0,0,180,55,0,0,192,218,0,0,126,54,0,0,140,60,0,0,64,25,1,0,120,55,0,0,248,62,0,0,0,119,1,0,104,66,0,0,104,66,0,0,1,202,8,0,104,66,0,0,104,66,0,0,0,0,0,0,255,255,255,127,0,0,0,126,0,0,0,120,0,0,0,110,0,0,0,96,0,0,0,78,0,0,0,56,0,0,0,30,51,51,3,0,51,51,3,0,51,51,3,0,51,51,3,0,51,51,3,0,0,0,0,0,236,81,0,0,236,81,0,0,236,81,0,0,236,81,0,0,236,81,0,0,0,0,0,0,0,0,0,192,0,0,0,192,0,0,0,192,0,0,0,192,128,153,153,153,128,153,153,153,0,0,64,0,0,0,128,2,0,0,128,3,0,0,192,4,128,106,29,0,0,0,0,0,0,0,0,0,0,0,0,0,224,106,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,0,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,5,0,0,0,5,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,5,0,0,0,5,0,0,0,255,255,255,127,0,0,184,127,0,0,224,126,0,0,120,125,0,0,128,123,0,0,248,120,0,0,224,117,0,0,56,114,0,0,0,110,0,0,56,105,0,0,224,99,0,0,248,93,0,0,128,87,0,0,120,80,0,0,224,72,0,0,184,64,0,0,96,0,0,0,96,0,0,0,64,1,0,0,96,0,0,0,32,0,0,0,0,0,0,0,128,0,0,0,32,1,0,0,128,0,0,0,32,1,0,0,128,0,0,0,128,0,0,0,32,0,0,0,32,0,0,0,32,0,0,0,32,0,102,102,6,0,0,0,0,0,0,0,224,1,0,0,224,1,0,0,224,1,0,0,224,1,0,0,64,1,0,0,0,0,61,129,127,7,149,153,153,25,245,177,179,43,53,51,51,51,1,0,0,0,1,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,2,0,0,0,1,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,3,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,4,0,0,0,2,0,1,0,0,0,0,0,1,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,5,0,0,0,2,0,1,0,0,0,0,0,1,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,6,0,0,0,2,0,1,1,0,0,0,0,1,0,0,0,1,0,0,0,3,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,7,0,0,0,2,1,1,1,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,3,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,16,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,17,0,0,0,2,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,18,0,0,0,2,2,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,19,0,0,0,3,2,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,1,0,1,0,1,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,22,0,0,0,1,1,1,0,1,0,0,0,1,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,23,0,0,0,4,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,30,0,0,0,1,0,1,0,1,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,88,115,29,0,0,0,0,0,0,0,0,0,0,0,0,0,144,115,29,0,80,101,29,0,64,101,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,125,0,0,32,78,0,0,2,0,0,0,64,156,0,0,0,125,0,0,3,0,0,0,192,218,0,0,128,187,0,0,4,0,0,0,64,25,1,0,0,250,0,0,5,0,0,0,128,181,1,0,0,119,1,0,0,0,192,0,0,0,192,0,0,0,128,1,0,0,128,1,0,0,224,1,0,0,224,1,0,0,128,2,0,0,128,2,0,0,128,2,0,0,128,2,0,0,64,1,0,0,0,0,128,102,102,134,128,102,102,134,128,102,102,134,0,0,0,192,128,102,102,134,128,102,102,134,0,0,32,252,0,0,224,250,0,0,32,252,0,0,224,250,0,0,32,252,0,0,32,252,16,0,0,0,183,209,0,0,146,10,0,0,212,125,0,0,143,0,0,0,146,10,0,0,160,153,153,9,52,51,51,3,128,102,102,38,160,153,153,25,37,76,252,255,37,76,252,255,62,44,91,255,50,0,0,0,224,55,29,0,45,101,88,0,224,1,0,0,15,0,0,0,4,0,0,0,0,0,0,0,88,56,29,0,215,10,79,0,194,1,0,0,12,0,0,0,4,0,0,0,0,0,0,0,184,56,29,0,158,26,51,0,154,1,0,0,8,0,0,0,5,0,0,0,0,0,0,0,248,56,29,0,72,70,53,0,94,1,0,0,6,0,0,0,4,0,0,0,0,0,0,0,40,57,29,0,197,248,30,0,250,0,0,0,4,0,0,0,5,0,0,0,0,0,0,0,0,0,0,206,0,0,0,220,0,0,0,228,0,0,0,236,0,0,0,242,0,0,0,248,0,0,0,252,0,0,0,0,0,0,0,4,0,0,0,8,0,0,0,14,0,0,0,20,0,0,0,28,0,0,0,36,0,0,0,50,0,0,0,0,3,0,0,0,5,0,0,0,2,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,35,0,0,0,30,0,0,0,2,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,31,0,0,0,32,0,0,0,38,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,35,0,0,0,30,0,0,0,2,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,31,0,0,0,32,0,0,0,38,0,0,0,0,0,0,0,128,62,0,0,188,52,0,0,0,125,0,0,96,109,0,0,1,0,0,0,1,0,0,0,157,5,0,0,220,5,0,0,120,5,0,0,88,2,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,1,0,0,0,160,153,153,25,0,205,204,76,1,0,0,0,1,0,0,0,1,0,0,0,157,5,0,0,220,5,0,0,120,5,0,0,88,2,0,0,12,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,1,0,0,0,160,153,153,25,0,205,204,76,1,0,0,0,1,125,0,0,97,109,0,0,96,234,0,0,32,203,0,0,1,0,0,0,1,0,0,0,157,5,0,0,220,5,0,0,120,5,0,0,88,2,0,0,12,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,1,0,0,0,160,153,153,25,0,0,0,64,1,0,0,0,1,0,0,0,1,0,0,0,157,5,0,0,220,5,0,0,120,5,0,0,88,2,0,0,12,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,1,0,0,0,160,153,153,25,0,0,0,64,1,0,0,0,97,234,0,0,33,203,0,0,0,220,5,0,0,220,5,0,1,0,0,0,1,0,0,0,157,5,0,0,220,5,0,0,120,5,0,0,88,2,0,0,12,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,1,0,0,0,160,153,153,25,0,0,0,64,1,0,0,0,1,0,0,0,1,0,0,0,157,5,0,0,220,5,0,0,120,5,0,0,88,2,0,0,12,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,1,0,0,0,160,153,153,25,0,0,0,64,1,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,0,0,0,0,64,31,0,0,8,103,1,0,248,102,1,0,17,43,0,0,144,105,1,0,128,105,1,0,224,46,0,0,72,105,1,0,56,105,1,0,128,62,0,0,0,105,1,0,240,104,1,0,34,86,0,0,184,104,1,0,168,104,1,0,192,93,0,0,112,104,1,0,96,104,1,0,0,125,0,0,40,104,1,0,24,104,1,0,68,172,0,0,224,103,1,0,208,103,1,0,128,187,0,0,152,103,1,0,136,103,1,0,0,250,0,0,80,103,1,0,64,103,1,0,136,88,1,0,192,102,1,0,176,102,1,0,0,119,1,0,120,102,1,0,104,102,1,0,3,8,4,4,8,4,4,13,6,0,0,0,152,99,29,0,3,8,2,2,8,2,2,13,6,0,0,0,152,99,29,0,3,6,2,2,248,254,2,13,6,0,0,0,152,99,29,0,160,15,31,5,174,7,126,4,176,4,143,2,8,0,15,0,160,15,31,5,246,8,106,4,20,5,102,6,8,0,15,0,4,16,31,5,246,8,76,4,120,5,205,12,8,0,15,0,4,16,215,3,205,12,76,4,120,5,51,19,8,0,15,0,204,16,215,3,205,12,76,4,120,5,51,19,8,0,31,0,136,19,215,3,205,12,76,4,120,5,0,32,8,0,31,0,124,21,215,3,92,15,76,4,120,5,205,44,8,0,31,0,112,23,215,3,92,15,56,4,120,5,51,51,8,0,31,0,112,23,215,3,236,17,46,4,120,5,154,57,8,0,31,0,0,0,128,12,0,0,0,15,0,0,128,17,0,0,0,20,1,0,0,128,0,0,0,0,0,0,0,2,52,128,43,3,0,0,0,4,194,211,164,4,52,128,43,5,160,93,157,5,0,0,0,6,105,0,87,6,194,211,164,6,159,58,235,6,52,128,43,7,9,160,102,7,160,93,157,7,247,83,208,7,0,0,0,8,238,199,44,8,105,0,87,8,91,240,126,8,194,211,164,8,212,221,200,8,159,58,235,8,80,16,12,9,52,128,43,9,133,167,73,9,9,160,102,9,157,128,130,9,160,93,157,9,73,73,183,9,247,83,208,9,107,140,232,9,0,0,0,10,211,186,22,10,238,199,44,10,98,49,66,10,105,0,87,10,121,61,107,10,91,240,126,10,61,32,146,10,194,211,164,10,14,17,183,10,212,221,200,10,96,63,218,10,159,58,235,10,43,212,251,10,80,16,12,11,18,243,27,11,52,128,43,11,64,187,58,11,133,167,73,11,34,72,88,11,9,160,102,11,253,177,116,11,157,128,130,11,97,14,144,11,160,93,157,11,143,112,170,11,73,73,183,11,202,233,195,11,247,83,208,11,155,137,220,11,107,140,232,11,9,94,244,11,0,0,0,12,203,115,11,12,211,186,22,12,113,214,33,12,238,199,44,12,133,144,55,12,98,49,66,12,168,171,76,12,105,0,87,12,175,48,97,12,121,61,107,12,185,39,117,12,91,240,126,12,63,152,136,12,61,32,146,12,38,137,155,12,194,211,164,12,210,0,174,12,14,17,183,12,43,5,192,12,212,221,200,12,176,155,209,12,96,63,218,12,125,201,226,12,159,58,235,12,85,147,243,12,43,212,251,12,169,253,3,13,80,16,12,13,160,12,20,13,18,243,27,13,29,196,35,13,52,128,43,13,199,39,51,13,64,187,58,13,8,59,66,13,133,167,73,13,24,1,81,13,34,72,88,13,255,124,95,13,9,160,102,13,151,177,109,13,253,177,116,13,144,161,123,13,157,128,130,13,117,79,137,13,97,14,144,13,173,189,150,13,160,93,157,13,127,238,163,13,143,112,170,13,18,228,176,13,73,73,183,13,114,160,189,13,202,233,195,13,142,37,202,13,247,83,208,13,62,117,214,13,155,137,220,13,67,145,226,13,107,140,232,13,71,123,238,13,9,94,244,13,225,52,250,13,0,0,0,14,148,191,5,14,203,115,11,14,210,28,17,14,211,186,22,14,251,77,28,14,113,214,33,14,96,84,39,14,238,199,44,14,67,49,50,14,133,144,55,14,216,229,60,14,98,49,66,14,70,115,71,14,168,171,76,14,168,218,81,14,105,0,87,14,11,29,92,14,175,48,97,14,116,59,102,14,121,61,107,14,219,54,112,14,185,39,117,14,48,16,122,14,91,240,126,14,87,200,131,14,63,152,136,14,46,96,141,14,61,32,146,14,136,216,150,14,38,137,155,14,50,50,160,14,194,211,164,14,240,109,169,14,210,0,174,14,127,140,178,14,14,17,183,14,150,142,187,14,43,5,192,14,228,116,196,14,212,221,200,14,18,64,205,14,176,155,209,14,196,240,213,14,96,63,218,14,151,135,222,14,125,201,226,14,37,5,231,14,159,58,235,14,255,105,239,14,85,147,243,14,180,182,247,14,43,212,251,14,205,235,255,14,169,253,3,15,207,9,8,15,80,16,12,15,59,17,16,15,160,12,20,15,141,2,24,15,18,243,27,15,61,222,31,15,29,196,35,15,192,164,39,15,52,128,43,15,0,0,0,0,153,121,130,90,3,126,40,90,187,140,207,89,171,160,119,89,222,180,32,89,128,196,202,88,222,202,117,88,100,195,33,88,156,169,206,87,47,121,124,87,223,45,43,87,141,195,218,86,49,54,139,86,223,129,60,86,195,162,238,85,33,149,161,85,85,85,85,85,208,223,9,85,26,49,191,84,208,69,117,84,163,26,44,84,90,172,227,83,204,247,155,83,230,249,84,83,164,175,14,83,23,22,201,82,94,42,132,82,171,233,63,82,63,81,252,81,107,94,185,81,142,14,119,81,25,95,53,81,136,77,244,80,104,215,179,80,79,250,115,80,230,179,52,80,223,1,246,79,249,225,183,79,1,82,122,79,206,79,61,79,67,217,0,79,78,236,196,78,233,134,137,78,24,167,78,78,232,74,20,78,114,112,218,77,217,21,161,77,72,57,104,77,244,216,47,77,27,243,247,76,4,134,192,76,255,143,137,76,100,15,83,76,147,2,29,76,245,103,231,75,249,61,178,75,23,131,125,75,206,53,73,75,166,84,21,75,42,222,225,74,240,208,174,74,146,43,124,74,179,236,73,74,250,18,24,74,22,157,230,73,187,137,181,73,164,215,132,73,145,133,84,73,73,146,36,73,150,252,244,72,74,195,197,72,60,229,150,72,71,97,104,72,76,54,58,72,49,99,12,72,224,230,222,71,73,192,177,71,95,238,132,71,28,112,88,71,124,68,44,71,128,106,0,71,48,225,212,70,147,167,169,70,185,188,126,70,179,31,84,70,152,207,41,70,128,203,255,69,137,18,214,69,213,163,172,69,136,126,131,69,202,161,90,69,200,12,50,69,176,190,9,69,180,182,225,68,11,244,185,68,236,117,146,68,149,59,107,68,68,68,68,68,59,143,29,68,190,27,247,67,23,233,208,67,142,246,170,67,115,67,133,67,20,207,95,67,197,152,58,67,219,159,21,67,174,227,240,66,151,99,204,66,245,30,168,66,39,21,132,66,141,69,96,66,140,175,60,66,139,82,25,66,241,45,246,65,42,65,211,65,161,139,176,65,199,12,142,65,13,196,107,65,228,176,73,65,195,210,39,65,32,41,6,65,116,179,228,64,58,113,195,64,239,97,162,64,17,133,129,64,33,218,96,64,161,96,64,64,20,24,32,64,29,0,0,0,0,0,0,0,2,0,0,0,3,0,0,0,8,0,0,0,9,0,0,0,10,0,0,0,11,0,0,0,12,0,0,0,13,0,0,0,17,0,0,0,31,0,0,0,38,0,0,0,0,0,0,0,29,0,0,0,0,0,0,0,1,0,0,0,37,0,0,0,2,0,0,0,33,1,0,0,160,191,5,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,200,91,5,0,0,0,0,0,2,0,0,0,225,0,0,0,40,196,5,0,1,0,0,0,0,192,221,224,0,0,145,96,4,0,0,0,0,0,0,0,16,92,5,0,0,0,0,0,2,0,0,0,225,0,0,0,176,199,5,0,1,0,0,0,0,228,91,225,0,224,255,96,4,0,0,0,0,0,0,0,80,92,5,0,0,0,0,0,2,0,0,0,121,0,0,0,56,203,5,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,144,92,5,0,0,0,0,0,2,0,0,0,121,0,0,0,32,205,5,0,1,0,0,0,0,128,187,224,0,0,118,96,4,0,0,0,0,0,0,0,192,92,5,0,0,0,0,0,2,0,0,0,121,0,0,0,8,207,5,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,240,92,5,0,0,0,0,0,4,0,0,0,81,0,0,0,240,208,5,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,32,93,5,0,0,0,0,0,2,0,0,0,81,0,0,0,56,210,5,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,48,93,5,0,0,0,0,0,2,0,0,0,81,0,0,0,128,211,5,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,88,93,5,0,0,0,0,0,4,0,0,0,113,2,0,0,200,212,5,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,128,93,5,0,0,0,0,0,4,0,0,0,113,2,0,0,144,222,5,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,152,93,5,0,0,0,0,0,4,0,0,0,81,0,0,0,88,232,5,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,176,93,5,0,0,0,0,0,4,0,0,0,81,0,0,0,160,233,5,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,192,93,5,0,0,0,0,0,2,0,0,0,185,1,0,0,232,234,5,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,208,93,5,0,0,0,0,0,2,0,0,0,225,0,0,0,208,241,5,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,40,94,5,0,0,0,0,0,4,0,0,0,81,0,0,0,88,245,5,0,1,0,0,0,0,176,19,225,0,176,19,97,2,0,0,0,0,0,0,0,104,94,5,0,0,0,0,0,2,0,0,0,25,0,0,0,160,246,5,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,120,94,5,0,0,0,0,0,2,0,0,0,169,0,0,0,8,247,5,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,144,94,5,0,0,0,0,0,2,0,0,0,81,0,0,0,176,249,5,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,200,94,5,0,0,0,0,0,4,0,0,0,113,2,0,0,248,250,5,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,240,94,5,0,0,0,0,0,4,0,0,0,113,2,0,0,192,4,6,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,8,95,5,0,0,0,0,0,4,0,0,0,81,0,0,0,136,14,6,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,32,95,5,0,0,0,0,0,4,0,0,0,81,0,0,0,208,15,6,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,48,95,5,0,0,0,0,0,2,0,0,0,185,1,0,0,24,17,6,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,64,95,5,0,0,0,0,0,2,0,0,0,225,0,0,0,0,24,6,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,152,95,5,0,0,0,0,0,2,0,0,0,169,0,0,0,136,27,6,0,1,0,0,0,0,136,93,225,0,176,19,97,4,0,0,0,0,0,0,0,216,95,5,0,0,0,0,0,2,0,0,0,25,0,0,0,48,30,6,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,16,96,5,0,0,0,0,0,2,0,0,0,169,0,0,0,152,30,6,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,40,96,5,0,0,0,0,0,2,0,0,0,121,0,0,0,64,33,6,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,96,96,5,0,0,0,0,0,4,0,0,0,81,0,0,0,40,35,6,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,144,96,5,0,0,0,0,0,2,0,0,0,33,1,0,0,112,36,6,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,160,96,5,0,0,0,0,0,2,0,0,0,81,0,0,0,248,40,6,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,232,96,5,0,0,0,0,0,2,0,0,0,81,0,0,0,64,42,6,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,16,97,5,0,0,0,0,0,4,0,0,0,113,2,0,0,136,43,6,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,56,97,5,0,0,0,0,0,8,0,0,0,161,25,0,0,80,53,6,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,80,97,5,0,0,0,0,0,2,0,0,0,185,1,0,0,216,155,6,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,96,97,5,0,0,0,0,0,2,0,0,0,225,0,0,0,192,162,6,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,184,97,5,0,0,0,0,0,4,0,0,0,81,0,0,0,72,166,6,0,1,0,0,0,0,176,19,225,0,176,19,97,2,0,0,0,0,0,0,0,248,97,5,0,0,0,0,0,2,0,0,0,25,0,0,0,144,167,6,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,8,98,5,0,0,0,0,0,2,0,0,0,169,0,0,0,248,167,6,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,32,98,5,0,0,0,0,0,2,0,0,0,121,0,0,0,160,170,6,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,88,98,5,0,0,0,0,0,4,0,0,0,81,0,0,0,136,172,6,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,136,98,5,0,0,0,0,0,2,0,0,0,33,1,0,0,208,173,6,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,152,98,5,0,0,0,0,0,2,0,0,0,81,0,0,0,88,178,6,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,224,98,5,0,0,0,0,0,2,0,0,0,81,0,0,0,160,179,6,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,8,99,5,0,0,0,0,0,4,0,0,0,113,2,0,0,232,180,6,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,48,99,5,0,0,0,0,0,8,0,0,0,161,25,0,0,176,190,6,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,72,99,5,0,0,0,0,0,2,0,0,0,169,0,0,0,56,37,7,0,1,0,0,0,0,0,88,224,0,0,16,96,4,0,0,0,0,0,0,0,88,99,5,0,0,0,0,0,2,0,0,0,169,0,0,0,224,39,7,0,1,0,0,0,0,128,211,224,0,0,122,96,4,0,0,0,0,0,0,0,144,99,5,0,0,0,0,0,4,0,0,0,113,2,0,0,136,42,7,0,1,0,0,0,0,32,21,225,0,32,245,96,3,0,0,0,0,0,0,0,200,99,5,0,0,0,0,0,2,0,0,0,25,0,0,0,80,52,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,224,99,5,0,0,0,0,0,2,0,0,0,169,0,0,0,184,52,7,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,248,99,5,0,0,0,0,0,2,0,0,0,81,0,0,0,96,55,7,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,48,100,5,0,0,0,0,0,4,0,0,0,113,2,0,0,168,56,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,88,100,5,0,0,0,0,0,4,0,0,0,113,2,0,0,112,66,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,112,100,5,0,0,0,0,0,4,0,0,0,81,0,0,0,56,76,7,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,136,100,5,0,0,0,0,0,4,0,0,0,81,0,0,0,128,77,7,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,152,100,5,0,0,0,0,0,1,0,0,0,49,0,0,0,200,78,7,0,1,0,0,0,0,0,152,224,0,0,16,96,6,0,0,0,0,0,0,0,168,100,5,0,0,0,0,0,2,0,0,0,105,1,0,0,144,79,7,0,1,0,0,0,0,144,27,225,0,128,184,96,5,0,0,0,0,0,0,0,112,101,5,0,0,0,0,0,2,0,0,0,225,0,0,0,56,85,7,0,1,0,0,0,0,117,153,225,0,24,61,97,4,0,0,0,0,0,0,0,192,101,5,0,0,0,0,0,2,0,0,0,185,1,0,0,192,88,7,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,0,102,5,0,0,0,0,0,2,0,0,0,225,0,0,0,168,95,7,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,88,102,5,0,0,0,0,0,2,0,0,0,121,0,0,0,48,99,7,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,152,102,5,0,0,0,0,0,2,0,0,0,169,0,0,0,24,101,7,0,1,0,0,0,0,128,208,224,0,0,118,96,4,0,0,0,0,0,0,0,200,102,5,0,0,0,0,0,2,0,0,0,25,0,0,0,192,103,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,0,103,5,0,0,0,0,0,2,0,0,0,169,0,0,0,40,104,7,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,24,103,5,0,0,0,0,0,2,0,0,0,121,0,0,0,208,106,7,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,80,103,5,0,0,0,0,0,4,0,0,0,81,0,0,0,184,108,7,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,128,103,5,0,0,0,0,0,2,0,0,0,33,1,0,0,0,110,7,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,144,103,5,0,0,0,0,0,2,0,0,0,81,0,0,0,136,114,7,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,216,103,5,0,0,0,0,0,4,0,0,0,113,2,0,0,208,115,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,0,104,5,0,0,0,0,0,4,0,0,0,81,0,0,0,152,125,7,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,24,104,5,0,0,0,0,0,1,0,0,0,49,0,0,0,224,126,7,0,1,0,0,0,0,0,152,224,0,0,16,96,6,0,0,0,0,0,0,0,40,104,5,0,0,0,0,0,2,0,0,0,105,1,0,0,168,127,7,0,1,0,0,0,0,144,27,225,0,128,184,96,5,0,0,0,0,0,0,0,240,104,5,0,0,0,0,0,2,0,0,0,81,0,0,0,80,133,7,0,1,0,0,0,0,24,125,225,0,24,61,97,4,0,0,0,0,0,0,0,64,105,5,0,0,0,0,0,2,0,0,0,185,1,0,0,152,134,7,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,104,105,5,0,0,0,0,0,2,0,0,0,225,0,0,0,128,141,7,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,192,105,5,0,0,0,0,0,2,0,0,0,121,0,0,0,8,145,7,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,0,106,5,0,0,0,0,0,2,0,0,0,169,0,0,0,240,146,7,0,1,0,0,0,0,128,208,224,0,0,118,96,4,0,0,0,0,0,0,0,48,106,5,0,0,0,0,0,2,0,0,0,25,0,0,0,152,149,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,104,106,5,0,0,0,0,0,2,0,0,0,169,0,0,0,0,150,7,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,128,106,5,0,0,0,0,0,2,0,0,0,121,0,0,0,168,152,7,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,184,106,5,0,0,0,0,0,4,0,0,0,81,0,0,0,144,154,7,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,232,106,5,0,0,0,0,0,2,0,0,0,33,1,0,0,216,155,7,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,248,106,5,0,0,0,0,0,2,0,0,0,81,0,0,0,96,160,7,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,64,107,5,0,0,0,0,0,4,0,0,0,113,2,0,0,168,161,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,104,107,5,0,0,0,0,0,4,0,0,0,81,0,0,0,112,171,7,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,128,107,5,0,0,0,0,0,1,0,0,0,49,0,0,0,184,172,7,0,1,0,0,0,0,0,152,224,0,0,16,96,6,0,0,0,0,0,0,0,144,107,5,0,0,0,0,0,2,0,0,0,169,0,0,0,128,173,7,0,1,0,0,0,0,96,18,225,0,128,184,96,4,0,0,0,0,0,0,0,88,108,5,0,0,0,0,0,2,0,0,0,121,0,0,0,40,176,7,0,1,0,0,0,0,226,120,225,0,232,51,97,4,0,0,0,0,0,0,0,144,108,5,0,0,0,0,0,2,0,0,0,121,0,0,0,16,178,7,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,192,108,5,0,0,0,0,0,2,0,0,0,121,0,0,0,248,179,7,0,1,0,0,0,0,128,187,224,0,0,118,96,4,0,0,0,0,0,0,0,240,108,5,0,0,0,0,0,2,0,0,0,121,0,0,0,224,181,7,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,32,109,5,0,0,0,0,0,4,0,0,0,81,0,0,0,200,183,7,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,80,109,5,0,0,0,0,0,2,0,0,0,81,0,0,0,16,185,7,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,96,109,5,0,0,0,0,0,2,0,0,0,81,0,0,0,88,186,7,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,136,109,5,0,0,0,0,0,4,0,0,0,113,2,0,0,160,187,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,176,109,5,0,0,0,0,0,4,0,0,0,113,2,0,0,104,197,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,200,109,5,0,0,0,0,0,4,0,0,0,81,0,0,0,48,207,7,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,224,109,5,0,0,0,0,0,4,0,0,0,81,0,0,0,120,208,7,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,240,109,5,0,0,0,0,0,2,0,0,0,33,1,0,0,192,209,7,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,0,110,5,0,0,0,0,0,2,0,0,0,225,0,0,0,72,214,7,0,1,0,0,0,0,192,221,224,0,0,145,96,4,0,0,0,0,0,0,0,72,110,5,0,0,0,0,0,2,0,0,0,225,0,0,0,208,217,7,0,1,0,0,0,0,228,91,225,0,224,255,96,4,0,0,0,0,0,0,0,136,110,5,0,0,0,0,0,2,0,0,0,121,0,0,0,88,221,7,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,200,110,5,0,0,0,0,0,2,0,0,0,121,0,0,0,64,223,7,0,1,0,0,0,0,128,187,224,0,0,118,96,4,0,0,0,0,0,0,0,248,110,5,0,0,0,0,0,2,0,0,0,121,0,0,0,40,225,7,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,40,111,5,0,0,0,0,0,4,0,0,0,81,0,0,0,16,227,7,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,88,111,5,0,0,0,0,0,2,0,0,0,81,0,0,0,88,228,7,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,104,111,5,0,0,0,0,0,2,0,0,0,81,0,0,0,160,229,7,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,144,111,5,0,0,0,0,0,4,0,0,0,113,2,0,0,232,230,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,184,111,5,0,0,0,0,0,4,0,0,0,113,2,0,0,176,240,7,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,208,111,5,0,0,0,0,0,4,0,0,0,81,0,0,0,120,250,7,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,232,111,5,0,0,0,0,0,4,0,0,0,81,0,0,0,192,251,7,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,248,111,5,0,0,0,0,0,2,0,0,0,33,1,0,0,8,253,7,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,8,112,5,0,0,0,0,0,2,0,0,0,225,0,0,0,144,1,8,0,1,0,0,0,0,192,221,224,0,0,145,96,4,0,0,0,0,0,0,0,80,112,5,0,0,0,0,0,2,0,0,0,169,0,0,0,24,5,8,0,1,0,0,0,0,232,87,225,0,224,255,96,4,0,0,0,0,0,0,0,144,112,5,0,0,0,0,0,2,0,0,0,121,0,0,0,192,7,8,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,200,112,5,0,0,0,0,0,2,0,0,0,121,0,0,0,168,9,8,0,1,0,0,0,0,128,187,224,0,0,118,96,4,0,0,0,0,0,0,0,248,112,5,0,0,0,0,0,2,0,0,0,121,0,0,0,144,11,8,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,40,113,5,0,0,0,0,0,4,0,0,0,81,0,0,0,120,13,8,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,88,113,5,0,0,0,0,0,2,0,0,0,81,0,0,0,192,14,8,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,104,113,5,0,0,0,0,0,2,0,0,0,81,0,0,0,8,16,8,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,144,113,5,0,0,0,0,0,4,0,0,0,113,2,0,0,80,17,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,184,113,5,0,0,0,0,0,4,0,0,0,113,2,0,0,24,27,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,208,113,5,0,0,0,0,0,4,0,0,0,81,0,0,0,224,36,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,232,113,5,0,0,0,0,0,4,0,0,0,81,0,0,0,40,38,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,248,113,5,0,0,0,0,0,2,0,0,0,33,1,0,0,112,39,8,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,8,114,5,0,0,0,0,0,2,0,0,0,225,0,0,0,248,43,8,0,1,0,0,0,0,192,221,224,0,0,145,96,4,0,0,0,0,0,0,0,80,114,5,0,0,0,0,0,2,0,0,0,169,0,0,0,128,47,8,0,1,0,0,0,0,232,87,225,0,224,255,96,4,0,0,0,0,0,0,0,144,114,5,0,0,0,0,0,2,0,0,0,25,0,0,0,40,50,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,200,114,5,0,0,0,0,0,2,0,0,0,169,0,0,0,144,50,8,0,1,0,0,0,0,0,158,224,0,0,84,96])
.concat([4,0,0,0,0,0,0,0,224,114,5,0,0,0,0,0,2,0,0,0,81,0,0,0,56,53,8,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,24,115,5,0,0,0,0,0,4,0,0,0,113,2,0,0,128,54,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,64,115,5,0,0,0,0,0,4,0,0,0,113,2,0,0,72,64,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,88,115,5,0,0,0,0,0,4,0,0,0,81,0,0,0,16,74,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,112,115,5,0,0,0,0,0,4,0,0,0,81,0,0,0,88,75,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,128,115,5,0,0,0,0,0,2,0,0,0,33,1,0,0,160,76,8,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,144,115,5,0,0,0,0,0,2,0,0,0,225,0,0,0,40,81,8,0,1,0,0,0,0,192,221,224,0,0,145,96,4,0,0,0,0,0,0,0,216,115,5,0,0,0,0,0,2,0,0,0,81,0,0,0,176,84,8,0,1,0,0,0,0,224,63,225,0,224,255,96,4,0,0,0,0,0,0,0,24,116,5,0,0,0,0,0,2,0,0,0,25,0,0,0,248,85,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,64,116,5,0,0,0,0,0,2,0,0,0,169,0,0,0,96,86,8,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,88,116,5,0,0,0,0,0,2,0,0,0,81,0,0,0,8,89,8,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,144,116,5,0,0,0,0,0,4,0,0,0,113,2,0,0,80,90,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,184,116,5,0,0,0,0,0,4,0,0,0,113,2,0,0,24,100,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,208,116,5,0,0,0,0,0,4,0,0,0,81,0,0,0,224,109,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,232,116,5,0,0,0,0,0,4,0,0,0,81,0,0,0,40,111,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,248,116,5,0,0,0,0,0,2,0,0,0,169,0,0,0,112,112,8,0,1,0,0,0,0,0,88,224,0,0,16,96,4,0,0,0,0,0,0,0,8,117,5,0,0,0,0,0,2,0,0,0,169,0,0,0,24,115,8,0,1,0,0,0,0,128,211,224,0,0,122,96,4,0,0,0,0,0,0,0,64,117,5,0,0,0,0,0,2,0,0,0,81,0,0,0,192,117,8,0,1,0,0,0,0,32,53,225,0,32,245,96,4,0,0,0,0,0,0,0,120,117,5,0,0,0,0,0,2,0,0,0,25,0,0,0,8,119,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,160,117,5,0,0,0,0,0,2,0,0,0,169,0,0,0,112,119,8,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,184,117,5,0,0,0,0,0,2,0,0,0,81,0,0,0,24,122,8,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,240,117,5,0,0,0,0,0,4,0,0,0,113,2,0,0,96,123,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,24,118,5,0,0,0,0,0,4,0,0,0,113,2,0,0,40,133,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,48,118,5,0,0,0,0,0,4,0,0,0,81,0,0,0,240,142,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,72,118,5,0,0,0,0,0,4,0,0,0,81,0,0,0,56,144,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,88,118,5,0,0,0,0,0,2,0,0,0,169,0,0,0,128,145,8,0,1,0,0,0,0,0,88,224,0,0,16,96,4,0,0,0,0,0,0,0,104,118,5,0,0,0,0,0,2,0,0,0,169,0,0,0,40,148,8,0,1,0,0,0,0,128,211,224,0,0,122,96,4,0,0,0,0,0,0,0,160,118,5,0,0,0,0,0,2,0,0,0,49,0,0,0,208,150,8,0,1,0,0,0,0,176,31,225,0,32,245,96,3,0,0,0,0,0,0,0,216,118,5,0,0,0,0,0,2,0,0,0,25,0,0,0,152,151,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,248,118,5,0,0,0,0,0,2,0,0,0,169,0,0,0,0,152,8,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,16,119,5,0,0,0,0,0,2,0,0,0,81,0,0,0,168,154,8,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,72,119,5,0,0,0,0,0,4,0,0,0,113,2,0,0,240,155,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,112,119,5,0,0,0,0,0,4,0,0,0,113,2,0,0,184,165,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,136,119,5,0,0,0,0,0,4,0,0,0,81,0,0,0,128,175,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,160,119,5,0,0,0,0,0,4,0,0,0,81,0,0,0,200,176,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,176,119,5,0,0,0,0,0,2,0,0,0,169,0,0,0,16,178,8,0,1,0,0,0,0,0,88,224,0,0,16,96,4,0,0,0,0,0,0,0,192,119,5,0,0,0,0,0,2,0,0,0,169,0,0,0,184,180,8,0,1,0,0,0,0,128,211,224,0,0,122,96,4,0,0,0,0,0,0,0,248,119,5,0,0,0,0,0,4,0,0,0,113,2,0,0,96,183,8,0,1,0,0,0,0,32,21,225,0,32,245,96,3,0,0,0,0,0,0,0,48,120,5,0,0,0,0,0,2,0,0,0,25,0,0,0,40,193,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,72,120,5,0,0,0,0,0,2,0,0,0,169,0,0,0,144,193,8,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,96,120,5,0,0,0,0,0,2,0,0,0,81,0,0,0,56,196,8,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,152,120,5,0,0,0,0,0,4,0,0,0,113,2,0,0,128,197,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,192,120,5,0,0,0,0,0,4,0,0,0,113,2,0,0,72,207,8,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,216,120,5,0,0,0,0,0,4,0,0,0,81,0,0,0,16,217,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,240,120,5,0,0,0,0,0,4,0,0,0,81,0,0,0,88,218,8,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,0,121,5,0,0,0,0,0,1,0,0,0,25,0,0,0,160,219,8,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,16,121,5,0,0,0,0,0,1,0,0,0,25,0,0,0,8,220,8,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,120,121,5,0,0,0,0,0,5,0,0,0,243,0,0,0,112,220,8,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,224,121,5,0,0,0,0,0,5,0,0,0,243,0,0,0,64,224,8,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,240,121,5,0,0,0,0,0,5,0,0,0,243,0,0,0,16,228,8,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,0,122,5,0,0,0,0,0,1,0,0,0,7,0,0,0,224,231,8,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,16,122,5,0,0,0,0,0,5,0,0,0,53,12,0,0,0,232,8,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,48,122,5,0,0,0,0,0,5,0,0,0,53,12,0,0,216,24,9,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,72,122,5,0,0,0,0,0,5,0,0,0,243,0,0,0,176,73,9,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,96,122,5,0,0,0,0,0,5,0,0,0,243,0,0,0,128,77,9,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,112,122,5,0,0,0,0,0,5,0,0,0,243,0,0,0,80,81,9,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,128,122,5,0,0,0,0,0,5,0,0,0,243,0,0,0,32,85,9,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,144,122,5,0,0,0,0,0,2,0,0,0,9,0,0,0,240,88,9,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,160,122,5,0,0,0,0,0,2,0,0,0,25,0,0,0,24,89,9,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,176,122,5,0,0,0,0,0,2,0,0,0,169,0,0,0,128,89,9,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,200,122,5,0,0,0,0,0,1,0,0,0,25,0,0,0,40,92,9,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,0,123,5,0,0,0,0,0,1,0,0,0,25,0,0,0,144,92,9,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,104,123,5,0,0,0,0,0,5,0,0,0,53,12,0,0,248,92,9,0,1,0,0,0,0,136,83,225,0,136,51,97,3,0,0,0,0,0,0,0,208,123,5,0,0,0,0,0,5,0,0,0,53,12,0,0,208,141,9,0,1,0,0,0,0,106,152,225,0,106,120,97,3,0,0,0,0,0,0,0,232,123,5,0,0,0,0,0,5,0,0,0,243,0,0,0,168,190,9,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,0,124,5,0,0,0,0,0,5,0,0,0,243,0,0,0,120,194,9,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,16,124,5,0,0,0,0,0,1,0,0,0,7,0,0,0,72,198,9,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,32,124,5,0,0,0,0,0,5,0,0,0,53,12,0,0,104,198,9,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,64,124,5,0,0,0,0,0,5,0,0,0,53,12,0,0,64,247,9,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,88,124,5,0,0,0,0,0,5,0,0,0,243,0,0,0,24,40,10,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,112,124,5,0,0,0,0,0,5,0,0,0,243,0,0,0,232,43,10,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,128,124,5,0,0,0,0,0,5,0,0,0,243,0,0,0,184,47,10,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,144,124,5,0,0,0,0,0,5,0,0,0,53,12,0,0,136,51,10,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,160,124,5,0,0,0,0,0,5,0,0,0,243,0,0,0,96,100,10,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,184,124,5,0,0,0,0,0,1,0,0,0,25,0,0,0,48,104,10,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,200,124,5,0,0,0,0,0,1,0,0,0,25,0,0,0,152,104,10,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,48,125,5,0,0,0,0,0,5,0,0,0,53,12,0,0,0,105,10,0,1,0,0,0,0,136,83,225,0,136,51,97,3,0,0,0,0,0,0,0,152,125,5,0,0,0,0,0,5,0,0,0,243,0,0,0,216,153,10,0,1,0,0,0,0,106,120,225,0,106,120,97,2,0,0,0,0,0,0,0,176,125,5,0,0,0,0,0,5,0,0,0,243,0,0,0,168,157,10,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,192,125,5,0,0,0,0,0,5,0,0,0,243,0,0,0,120,161,10,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,208,125,5,0,0,0,0,0,1,0,0,0,7,0,0,0,72,165,10,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,224,125,5,0,0,0,0,0,5,0,0,0,53,12,0,0,104,165,10,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,0,126,5,0,0,0,0,0,5,0,0,0,53,12,0,0,64,214,10,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,24,126,5,0,0,0,0,0,5,0,0,0,243,0,0,0,24,7,11,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,48,126,5,0,0,0,0,0,5,0,0,0,243,0,0,0,232,10,11,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,64,126,5,0,0,0,0,0,5,0,0,0,243,0,0,0,184,14,11,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,80,126,5,0,0,0,0,0,5,0,0,0,53,12,0,0,136,18,11,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,96,126,5,0,0,0,0,0,5,0,0,0,243,0,0,0,96,67,11,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,120,126,5,0,0,0,0,0,1,0,0,0,25,0,0,0,48,71,11,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,136,126,5,0,0,0,0,0,1,0,0,0,25,0,0,0,152,71,11,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,240,126,5,0,0,0,0,0,5,0,0,0,243,0,0,0,0,72,11,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,88,127,5,0,0,0,0,0,5,0,0,0,243,0,0,0,208,75,11,0,1,0,0,0,0,76,93,225,0,76,93,97,2,0,0,0,0,0,0,0,104,127,5,0,0,0,0,0,5,0,0,0,243,0,0,0,160,79,11,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,120,127,5,0,0,0,0,0,5,0,0,0,243,0,0,0,112,83,11,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,136,127,5,0,0,0,0,0,1,0,0,0,7,0,0,0,64,87,11,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,152,127,5,0,0,0,0,0,5,0,0,0,53,12,0,0,96,87,11,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,184,127,5,0,0,0,0,0,5,0,0,0,53,12,0,0,56,136,11,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,208,127,5,0,0,0,0,0,5,0,0,0,243,0,0,0,16,185,11,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,232,127,5,0,0,0,0,0,5,0,0,0,243,0,0,0,224,188,11,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,248,127,5,0,0,0,0,0,5,0,0,0,243,0,0,0,176,192,11,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,8,128,5,0,0,0,0,0,5,0,0,0,53,12,0,0,128,196,11,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,24,128,5,0,0,0,0,0,5,0,0,0,243,0,0,0,88,245,11,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,48,128,5,0,0,0,0,0,1,0,0,0,25,0,0,0,40,249,11,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,64,128,5,0,0,0,0,0,1,0,0,0,25,0,0,0,144,249,11,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,168,128,5,0,0,0,0,0,5,0,0,0,243,0,0,0,248,249,11,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,16,129,5,0,0,0,0,0,5,0,0,0,243,0,0,0,200,253,11,0,1,0,0,0,0,76,93,225,0,76,93,97,2,0,0,0,0,0,0,0,32,129,5,0,0,0,0,0,5,0,0,0,243,0,0,0,152,1,12,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,48,129,5,0,0,0,0,0,5,0,0,0,243,0,0,0,104,5,12,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,64,129,5,0,0,0,0,0,1,0,0,0,7,0,0,0,56,9,12,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,80,129,5,0,0,0,0,0,5,0,0,0,53,12,0,0,88,9,12,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,112,129,5,0,0,0,0,0,5,0,0,0,53,12,0,0,48,58,12,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,136,129,5,0,0,0,0,0,5,0,0,0,243,0,0,0,8,107,12,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,160,129,5,0,0,0,0,0,5,0,0,0,243,0,0,0,216,110,12,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,176,129,5,0,0,0,0,0,5,0,0,0,243,0,0,0,168,114,12,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,192,129,5,0,0,0,0,0,5,0,0,0,53,12,0,0,120,118,12,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,208,129,5,0,0,0,0,0,5,0,0,0,243,0,0,0,80,167,12,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,232,129,5,0,0,0,0,0,2,0,0,0,9,0,0,0,32,171,12,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,248,129,5,0,0,0,0,0,2,0,0,0,25,0,0,0,72,171,12,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,8,130,5,0,0,0,0,0,2,0,0,0,169,0,0,0,176,171,12,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,32,130,5,0,0,0,0,0,1,0,0,0,25,0,0,0,88,174,12,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,88,130,5,0,0,0,0,0,1,0,0,0,25,0,0,0,192,174,12,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,192,130,5,0,0,0,0,0,5,0,0,0,243,0,0,0,40,175,12,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,40,131,5,0,0,0,0,0,5,0,0,0,243,0,0,0,248,178,12,0,1,0,0,0,0,76,93,225,0,76,93,97,2,0,0,0,0,0,0,0,56,131,5,0,0,0,0,0,5,0,0,0,243,0,0,0,200,182,12,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,72,131,5,0,0,0,0,0,5,0,0,0,243,0,0,0,152,186,12,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,88,131,5,0,0,0,0,0,1,0,0,0,7,0,0,0,104,190,12,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,104,131,5,0,0,0,0,0,5,0,0,0,53,12,0,0,136,190,12,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,136,131,5,0,0,0,0,0,5,0,0,0,53,12,0,0,96,239,12,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,160,131,5,0,0,0,0,0,5,0,0,0,243,0,0,0,56,32,13,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,184,131,5,0,0,0,0,0,5,0,0,0,243,0,0,0,8,36,13,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,200,131,5,0,0,0,0,0,5,0,0,0,243,0,0,0,216,39,13,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,216,131,5,0,0,0,0,0,5,0,0,0,53,12,0,0,168,43,13,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,232,131,5,0,0,0,0,0,5,0,0,0,243,0,0,0,128,92,13,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,0,132,5,0,0,0,0,0,2,0,0,0,9,0,0,0,80,96,13,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,16,132,5,0,0,0,0,0,2,0,0,0,25,0,0,0,120,96,13,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,32,132,5,0,0,0,0,0,2,0,0,0,169,0,0,0,224,96,13,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,56,132,5,0,0,0,0,0,1,0,0,0,25,0,0,0,136,99,13,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,112,132,5,0,0,0,0,0,1,0,0,0,25,0,0,0,240,99,13,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,216,132,5,0,0,0,0,0,5,0,0,0,243,0,0,0,88,100,13,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,64,133,5,0,0,0,0,0,5,0,0,0,243,0,0,0,40,104,13,0,1,0,0,0,0,76,93,225,0,76,93,97,2,0,0,0,0,0,0,0,80,133,5,0,0,0,0,0,5,0,0,0,243,0,0,0,248,107,13,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,96,133,5,0,0,0,0,0,5,0,0,0,243,0,0,0,200,111,13,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,112,133,5,0,0,0,0,0,1,0,0,0,7,0,0,0,152,115,13,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,128,133,5,0,0,0,0,0,5,0,0,0,53,12,0,0,184,115,13,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,160,133,5,0,0,0,0,0,5,0,0,0,53,12,0,0,144,164,13,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,184,133,5,0,0,0,0,0,5,0,0,0,243,0,0,0,104,213,13,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,208,133,5,0,0,0,0,0,5,0,0,0,243,0,0,0,56,217,13,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,224,133,5,0,0,0,0,0,5,0,0,0,243,0,0,0,8,221,13,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,240,133,5,0,0,0,0,0,5,0,0,0,53,12,0,0,216,224,13,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,0,134,5,0,0,0,0,0,5,0,0,0,243,0,0,0,176,17,14,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,24,134,5,0,0,0,0,0,2,0,0,0,9,0,0,0,128,21,14,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,40,134,5,0,0,0,0,0,2,0,0,0,25,0,0,0,168,21,14,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,56,134,5,0,0,0,0,0,2,0,0,0,169,0,0,0,16,22,14,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,80,134,5,0,0,0,0,0,1,0,0,0,25,0,0,0,184,24,14,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,136,134,5,0,0,0,0,0,1,0,0,0,25,0,0,0,32,25,14,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,240,134,5,0,0,0,0,0,5,0,0,0,243,0,0,0,136,25,14,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,88,135,5,0,0,0,0,0,5,0,0,0,243,0,0,0,88,29,14,0,1,0,0,0,0,76,93,225,0,76,93,97,2,0,0,0,0,0,0,0,104,135,5,0,0,0,0,0,5,0,0,0,243,0,0,0,40,33,14,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,120,135,5,0,0,0,0,0,5,0,0,0,243,0,0,0,248,36,14,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,136,135,5,0,0,0,0,0,1,0,0,0,7,0,0,0,200,40,14,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,152,135,5,0,0,0,0,0,5,0,0,0,53,12,0,0,232,40,14,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,184,135,5,0,0,0,0,0,5,0,0,0,53,12,0,0,192,89,14,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,208,135,5,0,0,0,0,0,5,0,0,0,243,0,0,0,152,138,14,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,232,135,5,0,0,0,0,0,5,0,0,0,243,0,0,0,104,142,14,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,248,135,5,0,0,0,0,0,5,0,0,0,243,0,0,0,56,146,14,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,8,136,5,0,0,0,0,0,5,0,0,0,53,12,0,0,8,150,14,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,24,136,5,0,0,0,0,0,5,0,0,0,243,0,0,0,224,198,14,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,48,136,5,0,0,0,0,0,2,0,0,0,9,0,0,0,176,202,14,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,64,136,5,0,0,0,0,0,2,0,0,0,25,0,0,0,216,202,14,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,80,136,5,0,0,0,0,0,2,0,0,0,169,0,0,0,64,203,14,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,104,136,5,0,0,0,0,0,1,0,0,0,25,0,0,0,232,205,14,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,160,136,5,0,0,0,0,0,1,0,0,0,25,0,0,0,80,206,14,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,8,137,5,0,0,0,0,0,5,0,0,0,243,0,0,0,184,206,14,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,112,137,5,0,0,0,0,0,5,0,0,0,243,0,0,0,136,210,14,0,1,0,0,0,0,76,93,225,0,76,93,97,2,0,0,0,0,0,0,0,128,137,5,0,0,0,0,0,5,0,0,0,243,0,0,0,88,214,14,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,144,137,5,0,0,0,0,0,5,0,0,0,243,0,0,0,40,218,14,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,160,137,5,0,0,0,0,0,1,0,0,0,7,0,0,0,248,221,14,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,176,137,5,0,0,0,0,0,5,0,0,0,53,12,0,0,24,222,14,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,208,137,5,0,0,0,0,0,5,0,0,0,53,12,0,0,240,14,15,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,232,137,5,0,0,0,0,0,5,0,0,0,243,0,0,0,200,63,15,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,0,138,5,0,0,0,0,0,5,0,0,0,243,0,0,0,152,67,15,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,16,138,5,0,0,0,0,0,5,0,0,0,243,0,0,0,104,71,15,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,32,138,5,0,0,0,0,0,5,0,0,0,53,12,0,0,56,75,15,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,48,138,5,0,0,0,0,0,5,0,0,0,243,0,0,0,16,124,15,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,72,138,5,0,0,0,0,0,2,0,0,0,9,0,0,0,224,127,15,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,88,138,5,0,0,0,0,0,2,0,0,0,25,0,0,0,8,128,15,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,104,138,5,0,0,0,0,0,2,0,0,0,169,0,0,0,112,128,15,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,128,138,5,0,0,0,0,0,1,0,0,0,25,0,0,0,24,131,15,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,184,138,5,0,0,0,0,0,1,0,0,0,25,0,0,0,128,131,15,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,32,139,5,0,0,0,0,0,5,0,0,0,243,0,0,0,232,131,15,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,136,139,5,0,0,0,0,0,5,0,0,0,243,0,0,0,184,135,15,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,152,139,5,0,0,0,0,0,5,0,0,0,243,0,0,0,136,139,15,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,168,139,5,0,0,0,0,0,1,0,0,0,7,0,0,0,88,143,15,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,184,139,5,0,0,0,0,0,5,0,0,0,53,12,0,0,120,143,15,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,216,139,5,0,0,0,0,0,5,0,0,0,53,12,0,0,80,192,15,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,240,139,5,0,0,0,0,0,5,0,0,0,243,0,0,0,40,241,15,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,8,140,5,0,0,0,0,0,5,0,0,0,243,0,0,0,248,244,15,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,24,140,5,0,0,0,0,0,5,0,0,0,243,0,0,0,200,248,15,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,40,140,5,0,0,0,0,0,5,0,0,0,243,0,0,0,152,252,15,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,56,140,5,0,0,0,0,0,2,0,0,0,9,0,0,0,104,0,16,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,72,140,5,0,0,0,0,0,2,0,0,0,25,0,0,0,144,0,16,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,88,140,5,0,0,0,0,0,2,0,0,0,169,0,0,0,248,0,16,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,112,140,5,0,0,0,0,0,1,0,0,0,25,0,0,0,160,3,16,0,1,0,0,0,0,0,120,224,0,0,16,96,5,0,0,0,0,0,0,0,168,140,5,0,0,0,0,0,1,0,0,0,25,0,0,0,8,4,16,0,1,0,0,0,0,192,18,225,0,0,153,96,5,0,0,0,0,0,0,0,16,141,5,0,0,0,0,0,5,0,0,0,243,0,0,0,112,4,16,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,120,141,5,0,0,0,0,0,5,0,0,0,243,0,0,0,64,8,16,0,1,0,0,0,0,0,92,224,0,0,92,96,2,0,0,0,0,0,0,0,136,141,5,0,0,0,0,0,5,0,0,0,243,0,0,0,16,12,16,0,1,0,0,0,0,0,149,224,0,0,149,96,2,0,0,0,0,0,0,0,152,141,5,0,0,0,0,0,1,0,0,0,7,0,0,0,224,15,16,0,1,0,0,0,0,0,56,224,0,0,16,96,3,0,0,0,0,0,0,0,168,141,5,0,0,0,0,0,5,0,0,0,53,12,0,0,0,16,16,0,1,0,0,0,0,0,124,224,0,0,92,96,3,0,0,0,0,0,0,0,200,141,5,0,0,0,0,0,5,0,0,0,53,12,0,0,216,64,16,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,224,141,5,0,0,0,0,0,5,0,0,0,243,0,0,0,176,113,16,0,1,0,0,0,0,0,84,224,0,0,84,96,2,0,0,0,0,0,0,0,248,141,5,0,0,0,0,0,5,0,0,0,243,0,0,0,128,117,16,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,8,142,5,0,0,0,0,0,5,0,0,0,243,0,0,0,80,121,16,0,1,0,0,0,0,0,56,224,0,0,56,96,2,0,0,0,0,0,0,0,24,142,5,0,0,0,0,0,5,0,0,0,243,0,0,0,32,125,16,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,40,142,5,0,0,0,0,0,2,0,0,0,9,0,0,0,240,128,16,0,1,0,0,0,0,136,51,225,0,136,51,97,2,0,0,0,0,0,0,0,56,142,5,0,0,0,0,0,2,0,0,0,25,0,0,0,24,129,16,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,72,142,5,0,0,0,0,0,2,0,0,0,169,0,0,0,128,129,16,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,96,142,5,0,0,0,0,0,2,0,0,0,33,1,0,0,40,132,16,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,152,142,5,0,0,0,0,0,2,0,0,0,169,0,0,0,176,136,16,0,1,0,0,0,0,128,217,224,0,0,145,96,4,0,0,0,0,0,0,0,224,142,5,0,0,0,0,0,2,0,0,0,81,0,0,0,88,139,16,0,1,0,0,0,0,160,59,225,0,160,251,96,4,0,0,0,0,0,0,0,24,143,5,0,0,0,0,0,2,0,0,0,25,0,0,0,160,140,16,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,64,143,5,0,0,0,0,0,2,0,0,0,169,0,0,0,8,141,16,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,88,143,5,0,0,0,0,0,2,0,0,0,121,0,0,0,176,143,16,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,144,143,5,0,0,0,0,0,4,0,0,0,81,0,0,0,152,145,16,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,192,143,5,0,0,0,0,0,2,0,0,0,33,1,0,0,224,146,16,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,208,143,5,0,0,0,0,0,2,0,0,0,81,0,0,0,104,151,16,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,24,144,5,0,0,0,0,0,2,0,0,0,81,0,0,0,176,152,16,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,64,144,5,0,0,0,0,0,4,0,0,0,113,2,0,0,248,153,16,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,104,144,5,0,0,0,0,0,8,0,0,0,161,25,0,0,192,163,16,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,128,144,5,0,0,0,0,0,2,0,0,0,33,1,0,0,72,10,17,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,144,144,5,0,0,0,0,0,2,0,0,0,169,0,0,0,208,14,17,0,1,0,0,0,0,128,217,224,0,0,145,96,4,0,0,0,0,0,0,0,216,144,5,0,0,0,0,0,4,0,0,0,113,2,0,0,120,17,17,0,1,0,0,0,0,160,27,225,0,160,251,96,3,0,0,0,0,0,0,0,16,145,5,0,0,0,0,0,2,0,0,0,25,0,0,0,64,27,17,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,40,145,5,0,0,0,0,0,2,0,0,0,169,0,0,0,168,27,17,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,64,145,5,0,0,0,0,0,2,0,0,0,121,0,0,0,80,30,17,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,120,145,5,0,0,0,0,0,4,0,0,0,81,0,0,0,56,32,17,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,168,145,5,0,0,0,0,0,2,0,0,0,33,1,0,0,128,33,17,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,184,145,5,0,0,0,0,0,2,0,0,0,81,0,0,0,8,38,17,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,0,146,5,0,0,0,0,0,2,0,0,0,81,0,0,0,80,39,17,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,40,146,5,0,0,0,0,0,4,0,0,0,113,2,0,0,152,40,17,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,80,146,5,0,0,0,0,0,8,0,0,0,161,25,0,0,96,50,17,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,104,146,5,0,0,0,0,0,1,0,0,0,49,0,0,0,232,152,17,0,1,0,0,0,0,0,152,224,0,0,16,96,6,0,0,0,0,0,0,0,120,146,5,0,0,0,0,0,2,0,0,0,105,1,0,0,176,153,17,0,1,0,0,0,0,144,27,225,0,128,184,96,5,0,0,0,0,0,0,0,64,147,5,0,0,0,0,0,2,0,0,0,105,1,0,0,88,159,17,0,1,0,0,0,128,93,176,225,0,24,61,97,5,0,0,0,0,0,0,0,144,147,5,0,0,0,0,0,2,0,0,0,185,1,0,0,0,165,17,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,224,147,5,0,0,0,0,0,2,0,0,0,225,0,0,0,232,171,17,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,56,148,5,0,0,0,0,0,2,0,0,0,121,0,0,0,112,175,17,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,120,148,5,0,0,0,0,0,2,0,0,0,169,0,0,0,88,177,17,0,1,0,0,0,0,128,208,224,0,0,118,96,4,0,0,0,0,0,0,0,168,148,5,0,0,0,0,0,2,0,0,0,25,0,0,0,0,180,17,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,224,148,5,0,0,0,0,0,2,0,0,0,169,0,0,0,104,180,17,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,248,148,5,0,0,0,0,0,2,0,0,0,121,0,0,0,16,183,17,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,48,149,5,0,0,0,0,0,4,0,0,0,81,0,0,0,248,184,17,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,96,149,5,0,0,0,0,0,2,0,0,0,33,1,0,0,64,186,17,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,112,149,5,0,0,0,0,0,2,0,0,0,81,0,0,0,200,190,17,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,184,149,5,0,0,0,0,0,4,0,0,0,113,2,0,0,16,192,17,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,224,149,5,0,0,0,0,0,4,0,0,0,81,0,0,0,216,201,17,0,1,0,0,0,0,0,16,224,0,0,16,96])
.concat([2,0,0,0,0,0,0,0,248,149,5,0,0,0,0,0,1,0,0,0,49,0,0,0,32,203,17,0,1,0,0,0,0,0,152,224,0,0,16,96,6,0,0,0,0,0,0,0,8,150,5,0,0,0,0,0,2,0,0,0,105,1,0,0,232,203,17,0,1,0,0,0,0,144,27,225,0,128,184,96,5,0,0,0,0,0,0,0,208,150,5,0,0,0,0,0,2,0,0,0,33,1,0,0,144,209,17,0,1,0,0,0,0,24,157,225,0,24,61,97,5,0,0,0,0,0,0,0,32,151,5,0,0,0,0,0,2,0,0,0,185,1,0,0,24,214,17,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,104,151,5,0,0,0,0,0,2,0,0,0,225,0,0,0,0,221,17,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,192,151,5,0,0,0,0,0,2,0,0,0,121,0,0,0,136,224,17,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,0,152,5,0,0,0,0,0,2,0,0,0,169,0,0,0,112,226,17,0,1,0,0,0,0,128,208,224,0,0,118,96,4,0,0,0,0,0,0,0,48,152,5,0,0,0,0,0,2,0,0,0,25,0,0,0,24,229,17,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,104,152,5,0,0,0,0,0,2,0,0,0,169,0,0,0,128,229,17,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,128,152,5,0,0,0,0,0,2,0,0,0,121,0,0,0,40,232,17,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,184,152,5,0,0,0,0,0,4,0,0,0,81,0,0,0,16,234,17,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,232,152,5,0,0,0,0,0,2,0,0,0,33,1,0,0,88,235,17,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,248,152,5,0,0,0,0,0,2,0,0,0,81,0,0,0,224,239,17,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,64,153,5,0,0,0,0,0,4,0,0,0,113,2,0,0,40,241,17,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,104,153,5,0,0,0,0,0,4,0,0,0,81,0,0,0,240,250,17,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,128,153,5,0,0,0,0,0,1,0,0,0,49,0,0,0,56,252,17,0,1,0,0,0,0,0,152,224,0,0,16,96,6,0,0,0,0,0,0,0,144,153,5,0,0,0,0,0,2,0,0,0,169,0,0,0,0,253,17,0,1,0,0,0,0,96,18,225,0,128,184,96,4,0,0,0,0,0,0,0,88,154,5,0,0,0,0,0,2,0,0,0,169,0,0,0,168,255,17,0,1,0,0,0,0,220,125,225,0,232,51,97,4,0,0,0,0,0,0,0,144,154,5,0,0,0,0,0,2,0,0,0,185,1,0,0,80,2,18,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,200,154,5,0,0,0,0,0,2,0,0,0,225,0,0,0,56,9,18,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,32,155,5,0,0,0,0,0,2,0,0,0,121,0,0,0,192,12,18,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,96,155,5,0,0,0,0,0,2,0,0,0,169,0,0,0,168,14,18,0,1,0,0,0,0,128,208,224,0,0,118,96,4,0,0,0,0,0,0,0,144,155,5,0,0,0,0,0,2,0,0,0,25,0,0,0,80,17,18,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,200,155,5,0,0,0,0,0,2,0,0,0,169,0,0,0,184,17,18,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,224,155,5,0,0,0,0,0,2,0,0,0,121,0,0,0,96,20,18,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,24,156,5,0,0,0,0,0,4,0,0,0,81,0,0,0,72,22,18,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,72,156,5,0,0,0,0,0,2,0,0,0,33,1,0,0,144,23,18,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,88,156,5,0,0,0,0,0,2,0,0,0,81,0,0,0,24,28,18,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,160,156,5,0,0,0,0,0,4,0,0,0,113,2,0,0,96,29,18,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,200,156,5,0,0,0,0,0,4,0,0,0,81,0,0,0,40,39,18,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,224,156,5,0,0,0,0,0,1,0,0,0,49,0,0,0,112,40,18,0,1,0,0,0,0,0,152,224,0,0,16,96,6,0,0,0,0,0,0,0,240,156,5,0,0,0,0,0,2,0,0,0,169,0,0,0,56,41,18,0,1,0,0,0,0,96,18,225,0,128,184,96,4,0,0,0,0,0,0,0,184,157,5,0,0,0,0,0,2,0,0,0,169,0,0,0,224,43,18,0,1,0,0,0,0,220,125,225,0,232,51,97,4,0,0,0,0,0,0,0,240,157,5,0,0,0,0,0,2,0,0,0,185,1,0,0,136,46,18,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,40,158,5,0,0,0,0,0,2,0,0,0,225,0,0,0,112,53,18,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,128,158,5,0,0,0,0,0,2,0,0,0,121,0,0,0,248,56,18,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,192,158,5,0,0,0,0,0,2,0,0,0,169,0,0,0,224,58,18,0,1,0,0,0,0,128,208,224,0,0,118,96,4,0,0,0,0,0,0,0,240,158,5,0,0,0,0,0,2,0,0,0,25,0,0,0,136,61,18,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,40,159,5,0,0,0,0,0,2,0,0,0,169,0,0,0,240,61,18,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,64,159,5,0,0,0,0,0,2,0,0,0,121,0,0,0,152,64,18,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,120,159,5,0,0,0,0,0,4,0,0,0,81,0,0,0,128,66,18,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,168,159,5,0,0,0,0,0,2,0,0,0,33,1,0,0,200,67,18,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,184,159,5,0,0,0,0,0,2,0,0,0,81,0,0,0,80,72,18,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,0,160,5,0,0,0,0,0,4,0,0,0,113,2,0,0,152,73,18,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,40,160,5,0,0,0,0,0,4,0,0,0,81,0,0,0,96,83,18,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,64,160,5,0,0,0,0,0,2,0,0,0,185,1,0,0,168,84,18,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,80,160,5,0,0,0,0,0,2,0,0,0,33,1,0,0,144,91,18,0,1,0,0,0,0,0,245,224,0,0,149,96,5,0,0,0,0,0,0,0,168,160,5,0,0,0,0,0,2,0,0,0,225,0,0,0,24,96,18,0,1,0,0,0,0,134,115,225,0,80,22,97,4,0,0,0,0,0,0,0,240,160,5,0,0,0,0,0,2,0,0,0,25,0,0,0,160,99,18,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,48,161,5,0,0,0,0,0,2,0,0,0,169,0,0,0,8,100,18,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,72,161,5,0,0,0,0,0,2,0,0,0,121,0,0,0,176,102,18,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,128,161,5,0,0,0,0,0,4,0,0,0,81,0,0,0,152,104,18,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,176,161,5,0,0,0,0,0,2,0,0,0,33,1,0,0,224,105,18,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,192,161,5,0,0,0,0,0,2,0,0,0,81,0,0,0,104,110,18,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,8,162,5,0,0,0,0,0,2,0,0,0,81,0,0,0,176,111,18,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,48,162,5,0,0,0,0,0,4,0,0,0,113,2,0,0,248,112,18,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,88,162,5,0,0,0,0,0,4,0,0,0,113,2,0,0,192,122,18,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,112,162,5,0,0,0,0,0,8,0,0,0,161,25,0,0,136,132,18,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,136,162,5,0,0,0,0,0,2,0,0,0,185,1,0,0,16,235,18,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,152,162,5,0,0,0,0,0,2,0,0,0,225,0,0,0,248,241,18,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,240,162,5,0,0,0,0,0,2,0,0,0,169,0,0,0,128,245,18,0,1,0,0,0,0,136,93,225,0,176,19,97,4,0,0,0,0,0,0,0,48,163,5,0,0,0,0,0,2,0,0,0,25,0,0,0,40,248,18,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,104,163,5,0,0,0,0,0,2,0,0,0,169,0,0,0,144,248,18,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,128,163,5,0,0,0,0,0,2,0,0,0,121,0,0,0,56,251,18,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,184,163,5,0,0,0,0,0,4,0,0,0,81,0,0,0,32,253,18,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,232,163,5,0,0,0,0,0,2,0,0,0,33,1,0,0,104,254,18,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,248,163,5,0,0,0,0,0,2,0,0,0,81,0,0,0,240,2,19,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,64,164,5,0,0,0,0,0,2,0,0,0,81,0,0,0,56,4,19,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,104,164,5,0,0,0,0,0,4,0,0,0,113,2,0,0,128,5,19,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,144,164,5,0,0,0,0,0,4,0,0,0,113,2,0,0,72,15,19,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,168,164,5,0,0,0,0,0,8,0,0,0,161,25,0,0,16,25,19,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,192,164,5,0,0,0,0,0,2,0,0,0,33,1,0,0,152,127,19,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,208,164,5,0,0,0,0,0,2,0,0,0,225,0,0,0,32,132,19,0,1,0,0,0,0,192,221,224,0,0,145,96,4,0,0,0,0,0,0,0,24,165,5,0,0,0,0,0,2,0,0,0,169,0,0,0,168,135,19,0,1,0,0,0,0,232,87,225,0,224,255,96,4,0,0,0,0,0,0,0,88,165,5,0,0,0,0,0,2,0,0,0,25,0,0,0,80,138,19,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,144,165,5,0,0,0,0,0,2,0,0,0,169,0,0,0,184,138,19,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,168,165,5,0,0,0,0,0,2,0,0,0,121,0,0,0,96,141,19,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,224,165,5,0,0,0,0,0,4,0,0,0,81,0,0,0,72,143,19,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,16,166,5,0,0,0,0,0,2,0,0,0,33,1,0,0,144,144,19,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,32,166,5,0,0,0,0,0,2,0,0,0,81,0,0,0,24,149,19,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,104,166,5,0,0,0,0,0,2,0,0,0,81,0,0,0,96,150,19,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,144,166,5,0,0,0,0,0,4,0,0,0,113,2,0,0,168,151,19,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,184,166,5,0,0,0,0,0,4,0,0,0,113,2,0,0,112,161,19,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,208,166,5,0,0,0,0,0,8,0,0,0,161,25,0,0,56,171,19,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,232,166,5,0,0,0,0,0,2,0,0,0,33,1,0,0,192,17,20,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,248,166,5,0,0,0,0,0,2,0,0,0,169,0,0,0,72,22,20,0,1,0,0,0,0,128,217,224,0,0,145,96,4,0,0,0,0,0,0,0,64,167,5,0,0,0,0,0,2,0,0,0,169,0,0,0,240,24,20,0,1,0,0,0,0,184,84,225,0,160,251,96,4,0,0,0,0,0,0,0,120,167,5,0,0,0,0,0,2,0,0,0,25,0,0,0,152,27,20,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,176,167,5,0,0,0,0,0,2,0,0,0,169,0,0,0,0,28,20,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,200,167,5,0,0,0,0,0,2,0,0,0,121,0,0,0,168,30,20,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,0,168,5,0,0,0,0,0,4,0,0,0,81,0,0,0,144,32,20,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,48,168,5,0,0,0,0,0,2,0,0,0,33,1,0,0,216,33,20,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,64,168,5,0,0,0,0,0,2,0,0,0,81,0,0,0,96,38,20,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,136,168,5,0,0,0,0,0,2,0,0,0,81,0,0,0,168,39,20,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,176,168,5,0,0,0,0,0,4,0,0,0,113,2,0,0,240,40,20,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,216,168,5,0,0,0,0,0,4,0,0,0,113,2,0,0,184,50,20,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,240,168,5,0,0,0,0,0,8,0,0,0,161,25,0,0,128,60,20,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,8,169,5,0,0,0,0,0,2,0,0,0,33,1,0,0,8,163,20,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,24,169,5,0,0,0,0,0,2,0,0,0,169,0,0,0,144,167,20,0,1,0,0,0,0,128,217,224,0,0,145,96,4,0,0,0,0,0,0,0,96,169,5,0,0,0,0,0,2,0,0,0,169,0,0,0,56,170,20,0,1,0,0,0,0,184,84,225,0,160,251,96,4,0,0,0,0,0,0,0,152,169,5,0,0,0,0,0,2,0,0,0,25,0,0,0,224,172,20,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,208,169,5,0,0,0,0,0,2,0,0,0,169,0,0,0,72,173,20,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,232,169,5,0,0,0,0,0,2,0,0,0,121,0,0,0,240,175,20,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,32,170,5,0,0,0,0,0,4,0,0,0,81,0,0,0,216,177,20,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,80,170,5,0,0,0,0,0,2,0,0,0,33,1,0,0,32,179,20,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,96,170,5,0,0,0,0,0,2,0,0,0,81,0,0,0,168,183,20,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,168,170,5,0,0,0,0,0,2,0,0,0,81,0,0,0,240,184,20,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,208,170,5,0,0,0,0,0,4,0,0,0,113,2,0,0,56,186,20,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,248,170,5,0,0,0,0,0,8,0,0,0,161,25,0,0,0,196,20,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,16,171,5,0,0,0,0,0,2,0,0,0,33,1,0,0,136,42,21,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,32,171,5,0,0,0,0,0,2,0,0,0,169,0,0,0,16,47,21,0,1,0,0,0,0,128,217,224,0,0,145,96,4,0,0,0,0,0,0,0,104,171,5,0,0,0,0,0,2,0,0,0,169,0,0,0,184,49,21,0,1,0,0,0,0,184,84,225,0,160,251,96,4,0,0,0,0,0,0,0,160,171,5,0,0,0,0,0,2,0,0,0,25,0,0,0,96,52,21,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,216,171,5,0,0,0,0,0,2,0,0,0,169,0,0,0,200,52,21,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,240,171,5,0,0,0,0,0,2,0,0,0,121,0,0,0,112,55,21,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,40,172,5,0,0,0,0,0,4,0,0,0,81,0,0,0,88,57,21,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,88,172,5,0,0,0,0,0,2,0,0,0,33,1,0,0,160,58,21,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,104,172,5,0,0,0,0,0,2,0,0,0,81,0,0,0,40,63,21,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,176,172,5,0,0,0,0,0,2,0,0,0,81,0,0,0,112,64,21,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,216,172,5,0,0,0,0,0,4,0,0,0,113,2,0,0,184,65,21,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,0,173,5,0,0,0,0,0,8,0,0,0,161,25,0,0,128,75,21,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,24,173,5,0,0,0,0,0,2,0,0,0,33,1,0,0,8,178,21,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,40,173,5,0,0,0,0,0,2,0,0,0,169,0,0,0,144,182,21,0,1,0,0,0,0,128,217,224,0,0,145,96,4,0,0,0,0,0,0,0,112,173,5,0,0,0,0,0,2,0,0,0,81,0,0,0,56,185,21,0,1,0,0,0,0,160,59,225,0,160,251,96,4,0,0,0,0,0,0,0,168,173,5,0,0,0,0,0,2,0,0,0,25,0,0,0,128,186,21,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,208,173,5,0,0,0,0,0,2,0,0,0,169,0,0,0,232,186,21,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,232,173,5,0,0,0,0,0,2,0,0,0,121,0,0,0,144,189,21,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,32,174,5,0,0,0,0,0,4,0,0,0,81,0,0,0,120,191,21,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,80,174,5,0,0,0,0,0,2,0,0,0,33,1,0,0,192,192,21,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,96,174,5,0,0,0,0,0,2,0,0,0,81,0,0,0,72,197,21,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,168,174,5,0,0,0,0,0,2,0,0,0,81,0,0,0,144,198,21,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,208,174,5,0,0,0,0,0,4,0,0,0,113,2,0,0,216,199,21,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,248,174,5,0,0,0,0,0,8,0,0,0,161,25,0,0,160,209,21,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,16,175,5,0,0,0,0,0,2,0,0,0,33,1,0,0,40,56,22,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,32,175,5,0,0,0,0,0,2,0,0,0,169,0,0,0,176,60,22,0,1,0,0,0,0,128,217,224,0,0,145,96,4,0,0,0,0,0,0,0,104,175,5,0,0,0,0,0,4,0,0,0,113,2,0,0,88,63,22,0,1,0,0,0,0,160,27,225,0,160,251,96,3,0,0,0,0,0,0,0,160,175,5,0,0,0,0,0,2,0,0,0,25,0,0,0,32,73,22,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,184,175,5,0,0,0,0,0,2,0,0,0,169,0,0,0,136,73,22,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,208,175,5,0,0,0,0,0,2,0,0,0,121,0,0,0,48,76,22,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,8,176,5,0,0,0,0,0,4,0,0,0,81,0,0,0,24,78,22,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,56,176,5,0,0,0,0,0,2,0,0,0,33,1,0,0,96,79,22,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,72,176,5,0,0,0,0,0,2,0,0,0,81,0,0,0,232,83,22,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,144,176,5,0,0,0,0,0,2,0,0,0,81,0,0,0,48,85,22,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,184,176,5,0,0,0,0,0,4,0,0,0,113,2,0,0,120,86,22,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,224,176,5,0,0,0,0,0,8,0,0,0,161,25,0,0,64,96,22,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,248,176,5,0,0,0,0,0,1,0,0,0,49,0,0,0,200,198,22,0,1,0,0,0,0,0,152,224,0,0,16,96,6,0,0,0,0,0,0,0,8,177,5,0,0,0,0,0,2,0,0,0,105,1,0,0,144,199,22,0,1,0,0,0,0,144,27,225,0,128,184,96,5,0,0,0,0,0,0,0,208,177,5,0,0,0,0,0,2,0,0,0,225,0,0,0,56,205,22,0,1,0,0,0,0,117,153,225,0,24,61,97,4,0,0,0,0,0,0,0,32,178,5,0,0,0,0,0,2,0,0,0,185,1,0,0,192,208,22,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,96,178,5,0,0,0,0,0,2,0,0,0,225,0,0,0,168,215,22,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,184,178,5,0,0,0,0,0,2,0,0,0,121,0,0,0,48,219,22,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,248,178,5,0,0,0,0,0,2,0,0,0,169,0,0,0,24,221,22,0,1,0,0,0,0,128,208,224,0,0,118,96,4,0,0,0,0,0,0,0,40,179,5,0,0,0,0,0,2,0,0,0,25,0,0,0,192,223,22,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,96,179,5,0,0,0,0,0,2,0,0,0,169,0,0,0,40,224,22,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,120,179,5,0,0,0,0,0,2,0,0,0,121,0,0,0,208,226,22,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,176,179,5,0,0,0,0,0,4,0,0,0,81,0,0,0,184,228,22,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,224,179,5,0,0,0,0,0,2,0,0,0,33,1,0,0,0,230,22,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,240,179,5,0,0,0,0,0,2,0,0,0,81,0,0,0,136,234,22,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,56,180,5,0,0,0,0,0,4,0,0,0,113,2,0,0,208,235,22,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,96,180,5,0,0,0,0,0,4,0,0,0,81,0,0,0,152,245,22,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,120,180,5,0,0,0,0,0,2,0,0,0,33,1,0,0,224,246,22,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,136,180,5,0,0,0,0,0,2,0,0,0,225,0,0,0,104,251,22,0,1,0,0,0,0,192,221,224,0,0,145,96,4,0,0,0,0,0,0,0,208,180,5,0,0,0,0,0,2,0,0,0,225,0,0,0,240,254,22,0,1,0,0,0,0,228,91,225,0,224,255,96,4,0,0,0,0,0,0,0,16,181,5,0,0,0,0,0,2,0,0,0,121,0,0,0,120,2,23,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,80,181,5,0,0,0,0,0,2,0,0,0,121,0,0,0,96,4,23,0,1,0,0,0,0,128,187,224,0,0,118,96,4,0,0,0,0,0,0,0,128,181,5,0,0,0,0,0,2,0,0,0,121,0,0,0,72,6,23,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,176,181,5,0,0,0,0,0,4,0,0,0,81,0,0,0,48,8,23,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,224,181,5,0,0,0,0,0,2,0,0,0,81,0,0,0,120,9,23,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,240,181,5,0,0,0,0,0,2,0,0,0,81,0,0,0,192,10,23,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,24,182,5,0,0,0,0,0,4,0,0,0,113,2,0,0,8,12,23,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,64,182,5,0,0,0,0,0,4,0,0,0,113,2,0,0,208,21,23,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,88,182,5,0,0,0,0,0,4,0,0,0,81,0,0,0,152,31,23,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,112,182,5,0,0,0,0,0,4,0,0,0,81,0,0,0,224,32,23,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,128,182,5,0,0,0,0,0,2,0,0,0,185,1,0,0,40,34,23,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,144,182,5,0,0,0,0,0,2,0,0,0,225,0,0,0,16,41,23,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,232,182,5,0,0,0,0,0,4,0,0,0,81,0,0,0,152,44,23,0,1,0,0,0,0,176,19,225,0,176,19,97,2,0,0,0,0,0,0,0,40,183,5,0,0,0,0,0,2,0,0,0,25,0,0,0,224,45,23,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,56,183,5,0,0,0,0,0,2,0,0,0,169,0,0,0,72,46,23,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,80,183,5,0,0,0,0,0,2,0,0,0,81,0,0,0,240,48,23,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,136,183,5,0,0,0,0,0,4,0,0,0,113,2,0,0,56,50,23,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,176,183,5,0,0,0,0,0,4,0,0,0,113,2,0,0,0,60,23,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,200,183,5,0,0,0,0,0,4,0,0,0,81,0,0,0,200,69,23,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,224,183,5,0,0,0,0,0,4,0,0,0,81,0,0,0,16,71,23,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,240,183,5,0,0,0,0,0,1,0,0,0,49,0,0,0,88,72,23,0,1,0,0,0,0,0,152,224,0,0,16,96,6,0,0,0,0,0,0,0,0,184,5,0,0,0,0,0,2,0,0,0,105,1,0,0,32,73,23,0,1,0,0,0,0,144,27,225,0,128,184,96,5,0,0,0,0,0,0,0,200,184,5,0,0,0,0,0,2,0,0,0,33,1,0,0,200,78,23,0,1,0,0,0,0,24,157,225,0,24,61,97,5,0,0,0,0,0,0,0,24,185,5,0,0,0,0,0,2,0,0,0,185,1,0,0,80,83,23,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,96,185,5,0,0,0,0,0,2,0,0,0,225,0,0,0,56,90,23,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,184,185,5,0,0,0,0,0,2,0,0,0,121,0,0,0,192,93,23,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,248,185,5,0,0,0,0,0,2,0,0,0,169,0,0,0,168,95,23,0,1,0,0,0,0,128,208,224,0,0,118,96,4,0,0,0,0,0,0,0,40,186,5,0,0,0,0,0,2,0,0,0,25,0,0,0,80,98,23,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,96,186,5,0,0,0,0,0,2,0,0,0,169,0,0,0,184,98,23,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,120,186,5,0,0,0,0,0,2,0,0,0,121,0,0,0,96,101,23,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,176,186,5,0,0,0,0,0,4,0,0,0,81,0,0,0,72,103,23,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,224,186,5,0,0,0,0,0,2,0,0,0,33,1,0,0,144,104,23,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,240,186,5,0,0,0,0,0,2,0,0,0,81,0,0,0,24,109,23,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,56,187,5,0,0,0,0,0,4,0,0,0,113,2,0,0,96,110,23,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,96,187,5,0,0,0,0,0,4,0,0,0,81,0,0,0,40,120,23,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,120,187,5,0,0,0,0,0,2,0,0,0,185,1,0,0,112,121,23,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,136,187,5,0,0,0,0,0,2,0,0,0,225,0,0,0,88,128,23,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,224,187,5,0,0,0,0,0,2,0,0,0,169,0,0,0,224,131,23,0,1,0,0,0,0,136,93,225,0,176,19,97,4,0,0,0,0,0,0,0,32,188,5,0,0,0,0,0,2,0,0,0,25,0,0,0,136,134,23,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,88,188,5,0,0,0,0,0,2,0,0,0,169,0,0,0,240,134,23,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,112,188,5,0,0,0,0,0,2,0,0,0,121,0,0,0,152,137,23,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,168,188,5,0,0,0,0,0,4,0,0,0,81,0,0,0,128,139,23,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,216,188,5,0,0,0,0,0,2,0,0,0,33,1,0,0,200,140,23,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,232,188,5,0,0,0,0,0,2,0,0,0,81,0,0,0,80,145,23,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,48,189,5,0,0,0,0,0,2,0,0,0,81,0,0,0,152,146,23,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,88,189,5,0,0,0,0,0,4,0,0,0,113,2,0,0,224,147,23,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,128,189,5,0,0,0,0,0,8,0,0,0,161,25,0,0,168,157,23,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,152,189,5,0,0,0,0,0,2,0,0,0,185,1,0,0,48,4,24,0,1,0,0,0,0,0,116,224,0,0,16,96,5,0,0,0,0,0,0,0,168,189,5,0,0,0,0,0,2,0,0,0,225,0,0,0,24,11,24,0,1,0,0,0,0,96,242,224,0,0,149,96,4,0,0,0,0,0,0,0,0,190,5,0,0,0,0,0,4,0,0,0,81,0,0,0,160,14,24,0,1,0,0,0,0,176,19,225,0,176,19,97,2,0,0,0,0,0,0,0,64,190,5,0,0,0,0,0,2,0,0,0,25,0,0,0,232,15,24,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,80,190,5,0,0,0,0,0,2,0,0,0,169,0,0,0,80,16,24,0,1,0,0,0,0,0,158,224,0,0,84,96,4,0,0,0,0,0,0,0,104,190,5,0,0,0,0,0,2,0,0,0,121,0,0,0,248,18,24,0,1,0,0,0,0,0,84,224,0,0,16,96,4,0,0,0,0,0,0,0,160,190,5,0,0,0,0,0,4,0,0,0,81,0,0,0,224,20,24,0,1,0,0,0,0,0,118,224,0,0,118,96,2,0,0,0,0,0,0,0,208,190,5,0,0,0,0,0,2,0,0,0,33,1,0,0,40,22,24,0,1,0,0,0,0,0,112,224,0,0,16,96,5,0,0,0,0,0,0,0,224,190,5,0,0,0,0,0,2,0,0,0,81,0,0,0,176,26,24,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,40,191,5,0,0,0,0,0,2,0,0,0,81,0,0,0,248,27,24,0,1,0,0,0,0,0,80,224,0,0,16,96,4,0,0,0,0,0,0,0,80,191,5,0,0,0,0,0,4,0,0,0,113,2,0,0,64,29,24,0,1,0,0,0,0,0,48,224,0,0,16,96,3,0,0,0,0,0,0,0,120,191,5,0,0,0,0,0,8,0,0,0,161,25,0,0,8,39,24,0,1,0,0,0,0,0,16,224,0,0,16,96,2,0,0,0,0,0,0,0,144,191,5,0,0,0,0,0,0,16,0,0,0,16,0,0,0,16,0,0,0,16,0,0,255,127,27,0,255,127,80,0,255,127,134,0,255,127,188,0,255,127,241,0,255,127,39,1,254,127,93,1,254,127,146,1,253,127,200,1,252,127,253,1,251,127,51,2,250,127,105,2,249,127,158,2,248,127,212,2,247,127,9,3,245,127,63,3,244,127,117,3,243,127,170,3,241,127,224,3,239,127,21,4,238,127,75,4,236,127,129,4,234,127,182,4,232,127,236,4,230,127,33,5,227,127,87,5,225,127,140,5,223,127,194,5,220,127,248,5,218,127,45,6,215,127,99,6,212,127,152,6,210,127,206,6,207,127,3,7,204,127,57,7,201,127,110,7,198,127,164,7,194,127,217,7,191,127,15,8,188,127,68,8,184,127,122,8,180,127,175,8,177,127,229,8,173,127,26,9,169,127,80,9,165,127,133,9,161,127,187,9,157,127,240,9,153,127,38,10,149,127,91,10,144,127,145,10,140,127,198,10,135,127,251,10,131,127,49,11,126,127,102,11,121,127,156,11,116,127,209,11,111,127,6,12,106,127,60,12,101,127,113,12,96,127,167,12,90,127,220,12,85,127,17,13,79,127,71,13,74,127,124,13,68,127,177,13,62,127,230,13,56,127,28,14,50,127,81,14,44,127,134,14,38,127,188,14,32,127,241,14,26,127,38,15,19,127,91,15,13,127,145,15,6,127,198,15,0,127,251,15,249,126,48,16,242,126,101,16,235,126,155,16,228,126,208,16,221,126,5,17,214,126,58,17,207,126,111,17,199,126,164,17,192,126,217,17,184,126,14,18,177,126,67,18,169,126,120,18,161,126,174,18,153,126,227,18,145,126,24,19,137,126,77,19,129,126,130,19,121,126,183,19,113,126,236,19,104,126,33,20,96,126,85,20,87,126,138,20,79,126,191,20,70,126,244,20,61,126,41,21,52,126,94,21,43,126,147,21,34,126,200,21,25,126,252,21,16,126,49,22,6,126,102,22,253,125,155,22,243,125,208,22,234,125,4,23,224,125,57,23,214,125,110,23,205,125,163,23,195,125,215,23,185,125,12,24,174,125,65,24,164,125,117,24,154,125,170,24,144,125,222,24,133,125,19,25,123,125,72,25,112,125,124,25,101,125,177,25,90,125,229,25,80,125,26,26,69,125,78,26,58,125,131,26,46,125,183,26,35,125,235,26,24,125,32,27,12,125,84,27,1,125,137,27,245,124,189,27,234,124,241,27,222,124,38,28,210,124,90,28,198,124,142,28,186,124,194,28,174,124,247,28,162,124,43,29,150,124,95,29,137,124,147,29,125,124,199,29,112,124,252,29,100,124,48,30,87,124,100,30,74,124,152,30,61,124,204,30,48,124,0,31,35,124,52,31,22,124,104,31,9,124,156,31,252,123,208,31,238,123,4,32,225,123,56,32,212,123,108,32,198,123,159,32,184,123,211,32,170,123,7,33,157,123,59,33,143,123,111,33,129,123,162,33,114,123,214,33,100,123,10,34,86,123,61,34,71,123,113,34,57,123,165,34,42,123,216,34,28,123,12,35,13,123,63,35,254,122,115,35,239,122,167,35,224,122,218,35,209,122,13,36,194,122,65,36,179,122,116,36,164,122,168,36,148,122,219,36,133,122,14,37,117,122,66,37,102,122,117,37,86,122,168,37,70,122,219,37,54,122,15,38,38,122,66,38,22,122,117,38,6,122,168,38,246,121,219,38,229,121,14,39,213,121,65,39,196,121,116,39,180,121,167,39,163,121,218,39,147,121,13,40,130,121,64,40,113,121,115,40,96,121,166,40,79,121,217,40,62,121,12,41,44,121,62,41,27,121,113,41,10,121,164,41,248,120,214,41,231,120,9,42,213,120,60,42,195,120,110,42,177,120,161,42,160,120,211,42,142,120,6,43,123,120,56,43,105,120,107,43,87,120,157,43,69,120,208,43,50,120,2,44,32,120,52,44,13,120,103,44,251,119,153,44,232,119,203,44,213,119,253,44,194,119,48,45,175,119,98,45,156,119,148,45,137,119,198,45,118,119,248,45,99,119,42,46,79,119,92,46,60,119,142,46,40,119,192,46,21,119,242,46,1,119,36,47,237,118,86,47,217,118,135,47,197,118,185,47,177,118,235,47,157,118,29,48,137,118,78,48,117,118,128,48,96,118,177,48,76,118,227,48,55,118,21,49,35,118,70,49,14,118,120,49,249,117,169,49,229,117,218,49,208,117,12,50,187,117,61,50,166,117,110,50,144,117,160,50,123,117,209,50,102,117,2,51,80,117,51,51,59,117,100,51,37,117,149,51,16,117,198,51,250,116,247,51,228,116,40,52,206,116,89,52,184,116,138,52,162,116,187,52,140,116,236,52,118,116,29,53,96,116,78,53,73,116,126,53,51,116,175,53,28,116,224,53,6,116,16,54,239,115,65,54,216,115,113,54,193,115,162,54,170,115,210,54,147,115,3,55,124,115,51,55,101,115,100,55,78,115,148,55,55,115,196,55,31,115,244,55,8,115,37,56,240,114,85,56,217,114,133,56,193,114,181,56,169,114,229,56,145,114,21,57,121,114,69,57,97,114,117,57,73,114,165,57,49,114,213,57,25,114,4,58,0,114,52,58,232,113,100,58,207,113,148,58,183,113,195,58,158,113,243,58,133,113,35,59,109,113,82,59,84,113,130,59,59,113,177,59,34,113,224,59,9,113,16,60,239,112,63,60,214,112,110,60,189,112,158,60,163,112,205,60,138,112,252,60,112,112,43,61,87,112,90,61,61,112,137,61,35,112,184,61,9,112,231,61,239,111,22,62,213,111,69,62,187,111,116,62,161,111,163,62,135,111,209,62,108,111,0,63,82,111,47,63,55,111,93,63,29,111,140,63,2,111,186,63,231,110,233,63,205,110,23,64,178,110,70,64,151,110,116,64,124,110,162,64,97,110,209,64,69,110,255,64,42,110,45,65,15,110,91,65,243,109,137,65,216,109,183,65,188,109,229,65,161,109,19,66])
.concat([133,109,65,66,105,109,111,66,77,109,157,66,49,109,202,66,21,109,248,66,249,108,38,67,221,108,83,67,193,108,129,67,164,108,174,67,136,108,220,67,108,108,9,68,79,108,55,68,50,108,100,68,22,108,145,68,249,107,191,68,220,107,236,68,191,107,25,69,162,107,70,69,133,107,115,69,104,107,160,69,75,107,205,69,46,107,250,69,16,107,39,70,243,106,84,70,213,106,129,70,184,106,173,70,154,106,218,70,124,106,7,71,95,106,51,71,65,106,96,71,35,106,140,71,5,106,185,71,231,105,229,71,201,105,17,72,170,105,62,72,140,105,106,72,110,105,150,72,79,105,194,72,49,105,238,72,18,105,26,73,244,104,70,73,213,104,114,73,182,104,158,73,151,104,202,73,120,104,246,73,89,104,34,74,58,104,77,74,27,104,121,74,252,103,164,74,220,103,208,74,189,103,251,74,158,103,39,75,126,103,82,75,95,103,126,75,63,103,169,75,31,103,212,75,255,102,255,75,223,102,42,76,192,102,85,76,160,102,128,76,127,102,171,76,95,102,214,76,63,102,1,77,31,102,44,77,254,101,87,77,222,101,129,77,189,101,172,77,157,101,215,77,124,101,1,78,92,101,44,78,59,101,86,78,26,101,128,78,249,100,171,78,216,100,213,78,183,100,255,78,150,100,41,79,117,100,84,79,83,100,126,79,50,100,168,79,17,100,210,79,239,99,251,79,206,99,37,80,172,99,79,80,138,99,121,80,105,99,162,80,71,99,204,80,37,99,246,80,3,99,31,81,225,98,73,81,191,98,114,81,157,98,155,81,123,98,197,81,88,98,238,81,54,98,23,82,19,98,64,82,241,97,105,82,206,97,146,82,172,97,187,82,137,97,228,82,102,97,13,83,68,97,53,83,33,97,94,83,254,96,135,83,219,96,175,83,184,96,216,83,148,96,0,84,113,96,41,84,78,96,81,84,43,96,122,84,7,96,162,84,228,95,202,84,192,95,242,84,157,95,26,85,121,95,66,85,85,95,106,85,49,95,146,85,13,95,186,85,233,94,226,85,197,94,10,86,161,94,49,86,125,94,89,86,89,94,128,86,53,94,168,86,16,94,207,86,236,93,247,86,200,93,30,87,163,93,69,87,127,93,108,87,90,93,148,87,53,93,187,87,16,93,226,87,236,92,9,88,199,92,47,88,162,92,86,88,125,92,125,88,88,92,164,88,50,92,202,88,13,92,241,88,232,91,24,89,194,91,62,89,157,91,100,89,120,91,139,89,82,91,177,89,44,91,215,89,7,91,253,89,225,90,35,90,187,90,74,90,149,90,112,90,254,127,146,1,234,127,182,4,194,127,217,7,135,127,251,10,56,127,28,14,214,126,58,17,96,126,85,20,214,125,110,23,58,125,131,26,137,124,147,29,198,123,159,32,239,122,167,35,6,122,168,38,10,121,164,41,251,119,153,44,217,118,135,47,166,117,110,50,96,116,78,53,8,115,37,56,158,113,243,58,35,112,184,61,151,110,116,64,249,108,38,67,75,107,205,69,140,105,106,72,189,103,251,74,222,101,129,77,239,99,251,79,241,97,105,82,228,95,202,84,200,93,30,87,157,91,100,89,255,127,50,0,255,127,151,0,255,127,251,0,254,127,96,1,253,127,196,1,251,127,41,2,249,127,141,2,247,127,242,2,245,127,86,3,242,127,187,3,239,127,31,4,236,127,132,4,232,127,232,4,228,127,77,5,224,127,177,5,219,127,22,6,214,127,122,6,209,127,222,6,203,127,67,7,197,127,167,7,191,127,12,8,185,127,112,8,178,127,212,8,171,127,56,9,163,127,157,9,156,127,1,10,148,127,101,10,139,127,201,10,131,127,45,11,122,127,146,11,113,127,246,11,103,127,90,12,93,127,190,12,83,127,34,13,73,127,134,13,62,127,234,13,51,127,78,14,39,127,178,14,28,127,21,15,16,127,121,15,3,127,221,15,247,126,65,16,234,126,164,16,221,126,8,17,207,126,108,17,193,126,207,17,179,126,51,18,165,126,150,18,150,126,250,18,135,126,93,19,120,126,193,19,104,126,36,20,88,126,135,20,72,126,234,20,55,126,77,21,38,126,177,21,21,126,20,22,3,126,119,22,242,125,218,22,224,125,60,23,205,125,159,23,186,125,2,24,167,125,101,24,148,125,199,24,129,125,42,25,109,125,141,25,88,125,239,25,68,125,81,26,47,125,180,26,26,125,22,27,5,125,120,27,239,124,218,27,217,124,61,28,194,124,159,28,172,124,1,29,149,124,98,29,126,124,196,29,102,124,38,30,78,124,136,30,54,124,233,30,30,124,75,31,5,124,172,31,236,123,14,32,211,123,111,32,185,123,208,32,159,123,49,33,133,123,146,33,106,123,243,33,80,123,84,34,52,123,181,34,25,123,22,35,253,122,118,35,225,122,215,35,197,122,55,36,168,122,152,36,140,122,248,36,110,122,88,37,81,122,184,37,51,122,24,38,21,122,120,38,247,121,216,38,216,121,56,39,185,121,151,39,154,121,247,39,122,121,86,40,91,121,182,40,58,121,21,41,26,121,116,41,249,120,211,41,216,120,50,42,183,120,145,42,149,120,240,42,116,120,79,43,81,120,173,43,47,120,12,44,12,120,106,44,233,119,200,44,198,119,38,45,162,119,132,45,126,119,226,45,90,119,64,46,54,119,158,46,17,119,251,46,236,118,89,47,199,118,182,47,161,118,19,48,123,118,112,48,85,118,205,48,46,118,42,49,8,118,135,49,225,117,228,49,185,117,64,50,146,117,157,50,106,117,249,50,66,117,85,51,25,117,177,51,240,116,13,52,199,116,105,52,158,116,196,52,117,116,32,53,75,116,123,53,33,116,215,53,246,115,50,54,203,115,141,54,160,115,232,54,117,115,66,55,74,115,157,55,30,115,247,55,242,114,82,56,197,114,172,56,153,114,6,57,108,114,96,57,63,114,186,57,17,114,19,58,227,113,109,58,181,113,198,58,135,113,32,59,88,113,121,59,42,113,210,59,250,112,42,60,203,112,131,60,155,112,220,60,107,112,52,61,59,112,140,61,11,112,228,61,218,111,60,62,169,111,148,62,120,111,236,62,70,111,67,63,20,111,154,63,226,110,241,63,176,110,72,64,125,110,159,64,74,110,246,64,23,110,77,65,228,109,163,65,176,109,249,65,124,109,79,66,72,109,165,66,20,109,251,66,223,108,81,67,170,108,166,67,117,108,251,67,63,108,80,68,9,108,165,68,211,107,250,68,157,107,79,69,102,107,163,69,48,107,247,69,248,106,75,70,193,106,159,70,137,106,243,70,82,106,71,71,26,106,154,71,225,105,237,71,169,105,64,72,112,105,147,72,55,105,230,72,253,104,57,73,196,104,139,73,138,104,221,73,80,104,47,74,21,104,129,74,218,103,211,74,160,103,36,75,100,103,117,75,41,103,199,75,237,102,23,76,178,102,104,76,117,102,185,76,57,102,9,77,252,101,89,77,192,101,169,77,130,101,249,77,69,101,73,78,7,101,152,78,202,100,232,78,139,100,55,79,77,100,133,79,15,100,212,79,208,99,35,80,145,99,113,80,81,99,191,80,18,99,13,81,210,98,91,81,146,98,168,81,82,98,245,81,17,98,67,82,209,97,144,82,144,97,220,82,78,97,41,83,13,97,117,83,203,96,193,83,137,96,13,84,71,96,89,84,5,96,164,84,194,95,240,84,128,95,59,85,60,95,134,85,249,94,208,85,182,94,27,86,114,94,101,86,46,94,175,86,234,93,249,86,165,93,67,87,97,93,140,87,28,93,213,87,215,92,30,88,145,92,103,88,76,92,176,88,6,92,248,88,192,91,64,89,122,91,136,89,52,91,208,89,237,90,24,90,166,90,95,90,255,127,54,0,255,127,161,0,255,127,12,1,254,127,119,1,252,127,227,1,251,127,78,2,249,127,185,2,246,127,36,3,243,127,143,3,240,127,251,3,237,127,102,4,233,127,209,4,229,127,60,5,224,127,167,5,219,127,18,6,214,127,125,6,208,127,232,6,202,127,84,7,196,127,191,7,189,127,42,8,182,127,149,8,175,127,0,9,167,127,107,9,159,127,213,9,151,127,64,10,142,127,171,10,133,127,22,11,123,127,129,11,114,127,236,11,103,127,86,12,93,127,193,12,82,127,44,13,71,127,151,13,59,127,1,14,47,127,108,14,35,127,214,14,23,127,65,15,10,127,171,15,252,126,22,16,239,126,128,16,225,126,234,16,210,126,85,17,196,126,191,17,181,126,41,18,165,126,147,18,149,126,253,18,133,126,103,19,117,126,209,19,100,126,59,20,83,126,165,20,66,126,15,21,48,126,120,21,30,126,226,21,11,126,76,22,248,125,181,22,229,125,31,23,209,125,136,23,190,125,242,23,169,125,91,24,149,125,196,24,128,125,45,25,107,125,150,25,85,125,255,25,63,125,104,26,41,125,209,26,18,125,58,27,251,124,163,27,228,124,12,28,204,124,116,28,180,124,221,28,156,124,69,29,131,124,173,29,106,124,22,30,81,124,126,30,55,124,230,30,29,124,78,31,3,124,182,31,232,123,30,32,205,123,134,32,177,123,237,32,150,123,85,33,121,123,188,33,93,123,36,34,64,123,139,34,35,123,242,34,6,123,89,35,232,122,192,35,202,122,39,36,171,122,142,36,141,122,245,36,109,122,91,37,78,122,194,37,46,122,40,38,14,122,143,38,238,121,245,38,205,121,91,39,172,121,193,39,138,121,39,40,104,121,140,40,70,121,242,40,36,121,88,41,1,121,189,41,222,120,34,42,186,120,136,42,151,120,237,42,114,120,82,43,78,120,183,43,41,120,27,44,4,120,128,44,223,119,228,44,185,119,73,45,147,119,173,45,108,119,17,46,70,119,117,46,30,119,217,46,247,118,61,47,207,118,160,47,167,118,4,48,127,118,103,48,86,118,202,48,45,118,45,49,4,118,144,49,218,117,243,49,176,117,86,50,134,117,184,50,91,117,27,51,48,117,125,51,5,117,223,51,217,116,65,52,173,116,163,52,129,116,4,53,84,116,102,53,40,116,199,53,250,115,41,54,205,115,138,54,159,115,235,54,113,115,75,55,66,115,172,55,20,115,12,56,228,114,109,56,181,114,205,56,133,114,45,57,85,114,141,57,37,114,237,57,244,113,76,58,195,113,171,58,146,113,11,59,96,113,106,59,46,113,201,59,252,112,39,60,201,112,134,60,151,112,228,60,99,112,67,61,48,112,161,61,252,111,255,61,200,111,92,62,148,111,186,62,95,111,23,63,42,111,117,63,245,110,210,63,191,110,46,64,137,110,139,64,83,110,232,64,28,110,68,65,230,109,160,65,174,109,252,65,119,109,88,66,63,109,179,66,7,109,15,67,207,108,106,67,150,108,197,67,93,108,32,68,36,108,123,68,235,107,213,68,177,107,48,69,119,107,138,69,60,107,228,69,2,107,61,70,199,106,151,70,139,106,240,70,80,106,73,71,20,106,162,71,216,105,251,71,155,105,84,72,95,105,172,72,34,105,4,73,228,104,92,73,167,104,180,73,105,104,12,74,43,104,99,74,236,103,186,74,173,103,17,75,110,103,104,75,47,103,190,75,239,102,21,76,176,102,107,76,111,102,193,76,47,102,23,77,238,101,108,77,173,101,193,77,108,101,22,78,42,101,107,78,233,100,192,78,166,100,20,79,100,100,105,79,33,100,189,79,222,99,16,80,155,99,100,80,88,99,183,80,20,99,10,81,208,98,93,81,140,98,176,81,71,98,2,82,2,98,85,82,189,97,167,82,120,97,248,82,50,97,74,83,236,96,155,83,166,96,236,83,96,96,61,84,25,96,142,84,210,95,222,84,139,95,46,85,67,95,126,85,251,94,206,85,179,94,29,86,107,94,109,86,35,94,188,86,218,93,10,87,145,93,89,87,72,93,167,87,254,92,245,87,180,92,67,88,106,92,144,88,32,92,222,88,213,91,43,89,138,91,120,89,63,91,196,89,244,90,16,90,168,90,93,90,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,246,127,36,3,167,127,107,9,10,127,171,15,30,126,226,21,228,124,12,28,93,123,36,34,138,121,39,40,108,119,17,46,5,117,223,51,85,114,141,57,95,111,23,63,36,108,123,68,167,104,180,73,233,100,192,78,236,96,155,83,180,92,67,88,255,127,201,0,250,127,91,2,241,127,237,3,226,127,127,5,206,127,17,7,181,127,162,8,152,127,51,10,117,127,196,11,78,127,84,13,34,127,228,14,240,126,115,16,186,126,1,18,127,126,143,19,63,126,28,21,251,125,168,22,177,125,51,24,99,125,190,25,15,125,71,27,183,124,208,28,90,124,87,30,249,123,221,31,146,123,98,33,39,123,229,34,183,122,103,36,66,122,232,37,201,121,104,39,74,121,229,40,200,120,98,42,64,120,220,43,180,119,85,45,35,119,204,46,142,118,66,48,244,117,181,49,86,117,39,51,179,116,151,52,11,116,4,54,95,115,112,55,175,114,217,56,250,113,64,58,65,113,165,59,131,112,8,61,194,111,104,62,251,110,198,63,49,110,33,65,98,109,122,66,143,108,209,67,184,107,36,69,221,106,117,70,253,105,196,71,26,105,15,73,50,104,88,74,71,103,158,75,87,102,225,76,100,101,33,78,108,100,94,79,113,99,152,80,114,98,207,81,111,97,3,83,104,96,51,84,94,95,96,85,80,94,138,86,62,93,177,87,41,92,212,88,16,91,244,89,255,127,214,0,250,127,131,2,238,127,48,4,222,127,221,5,199,127,137,7,171,127,53,9,137,127,225,10,98,127,140,12,53,127,54,14,3,127,224,15,203,126,138,17,141,126,50,19,74,126,218,20,2,126,128,22,180,125,38,24,96,125,203,25,7,125,110,27,168,124,17,29,68,124,178,30,218,123,82,32,107,123,240,33,247,122,141,35,125,122,40,37,254,121,194,38,121,121,90,40,239,120,240,41,96,120,132,43,204,119,23,45,50,119,167,46,147,118,53,48,239,117,194,49,70,117,76,51,151,116,212,52,228,115,89,54,43,115,220,55,109,114,93,57,171,113,219,58,227,112,87,60,22,112,208,61,69,111,70,63,110,110,185,64,147,109,42,66,179,108,152,67,206,107,2,69,228,106,106,70,246,105,207,71,3,105,48,73,11,104,143,74,15,103,234,75,15,102,65,77,9,101,150,78,0,100,230,79,242,98,52,81,224,97,126,82,201,96,196,83,174,95,6,85,143,94,69,86,108,93,128,87,69,92,183,88,26,91,234,89,255,127,25,0,255,127,75,0,255,127,126,0,255,127,176,0,255,127,226,0,255,127,20,1,254,127,71,1,254,127,121,1,253,127,171,1,253,127,222,1,252,127,16,2,251,127,66,2,250,127,116,2,249,127,167,2,248,127,217,2,247,127,11,3,246,127,61,3,244,127,112,3,243,127,162,3,241,127,212,3,240,127,6,4,238,127,57,4,236,127,107,4,235,127,157,4,233,127,207,4,231,127,1,5,229,127,52,5,227,127,102,5,225,127,152,5,222,127,202,5,220,127,253,5,218,127,47,6,215,127,97,6,213,127,147,6,210,127,197,6,207,127,248,6,205,127,42,7,202,127,92,7,199,127,142,7,196,127,192,7,193,127,242,7,190,127,37,8,186,127,87,8,183,127,137,8,180,127,187,8,176,127,237,8,173,127,31,9,169,127,81,9,165,127,132,9,162,127,182,9,158,127,232,9,154,127,26,10,150,127,76,10,146,127,126,10,142,127,176,10,137,127,226,10,133,127,20,11,129,127,71,11,124,127,121,11,120,127,171,11,115,127,221,11,110,127,15,12,106,127,65,12,101,127,115,12,96,127,165,12,91,127,215,12,86,127,9,13,80,127,59,13,75,127,109,13,70,127,159,13,65,127,209,13,59,127,3,14,54,127,53,14,48,127,103,14,42,127,153,14,36,127,203,14,31,127,252,14,25,127,46,15,19,127,96,15,13,127,146,15,6,127,196,15,0,127,246,15,250,126,40,16,244,126,90,16,237,126,140,16,231,126,189,16,224,126,239,16,217,126,33,17,211,126,83,17,204,126,133,17,197,126,182,17,190,126,232,17,183,126,26,18,176,126,76,18,168,126,125,18,161,126,175,18,154,126,225,18,146,126,19,19,139,126,68,19,131,126,118,19,123,126,168,19,116,126,217,19,108,126,11,20,100,126,61,20,92,126,110,20,84,126,160,20,76,126,209,20,67,126,3,21,59,126,53,21,51,126,102,21,42,126,152,21,34,126,201,21,25,126,251,21,17,126,44,22,8,126,94,22,255,125,143,22,246,125,193,22,237,125,242,22,228,125,36,23,219,125,85,23,210,125,135,23,201,125,184,23,191,125,233,23,182,125,27,24,172,125,76,24,163,125,125,24,153,125,175,24,143,125,224,24,133,125,17,25,124,125,67,25,114,125,116,25,104,125,165,25,93,125,214,25,83,125,8,26,73,125,57,26,63,125,106,26,52,125,155,26,42,125,204,26,31,125,254,26,21,125,47,27,10,125,96,27,255,124,145,27,244,124,194,27,233,124,243,27,222,124,36,28,211,124,85,28,200,124,134,28,189,124,183,28,177,124,232,28,166,124,25,29,155,124,74,29,143,124,123,29,131,124,172,29,120,124,221,29,108,124,14,30,96,124,62,30,84,124,111,30,72,124,160,30,60,124,209,30,48,124,2,31,36,124,50,31,24,124,99,31,11,124,148,31,255,123,197,31,242,123,245,31,230,123,38,32,217,123,87,32,204,123,135,32,191,123,184,32,179,123,232,32,166,123,25,33,153,123,73,33,139,123,122,33,126,123,170,33,113,123,219,33,100,123,11,34,86,123,60,34,73,123,108,34,59,123,157,34,46,123,205,34,32,123,253,34,18,123,46,35,4,123,94,35,246,122,142,35,232,122,191,35,218,122,239,35,204,122,31,36,190,122,79,36,176,122,128,36,161,122,176,36,147,122,224,36,132,122,16,37,118,122,64,37,103,122,112,37,88,122,160,37,73,122,208,37,59,122,0,38,44,122,48,38,29,122,96,38,14,122,144,38,254,121,192,38,239,121,240,38,224,121,32,39,208,121,80,39,193,121,128,39,177,121,175,39,162,121,223,39,146,121,15,40,130,121,63,40,114,121,110,40,98,121,158,40,83,121,206,40,66,121,253,40,50,121,45,41,34,121,92,41,18,121,140,41,1,121,188,41,241,120,235,41,225,120,27,42,208,120,74,42,191,120,121,42,175,120,169,42,158,120,216,42,141,120,8,43,124,120,55,43,107,120,102,43,90,120,149,43,73,120,197,43,56,120,244,43,38,120,35,44,21,120,82,44,3,120,129,44,242,119,177,44,224,119,224,44,207,119,15,45,189,119,62,45,171,119,109,45,153,119,156,45,135,119,203,45,117,119,250,45,99,119,40,46,81,119,87,46,63,119,134,46,45,119,181,46,26,119,228,46,8,119,19,47,245,118,65,47,227,118,112,47,208,118,159,47,189,118,205,47,170,118,252,47,152,118,42,48,133,118,89,48,114,118,136,48,94,118,182,48,75,118,229,48,56,118,19,49,37,118,65,49,17,118,112,49,254,117,158,49,234,117,204,49,215,117,251,49,195,117,41,50,175,117,87,50,156,117,133,50,136,117,180,50,116,117,226,50,96,117,16,51,76,117,62,51,56,117,108,51,35,117,154,51,15,117,200,51,251,116,246,51,230,116,36,52,210,116,82,52,189,116,128,52,168,116,173,52,148,116,219,52,127,116,9,53,106,116,55,53,85,116,100,53,64,116,146,53,43,116,192,53,22,116,237,53,1,116,27,54,235,115,72,54,214,115,118,54,193,115,163,54,171,115,209,54,150,115,254,54,128,115,44,55,106,115,89,55,85,115,134,55,63,115,180,55,41,115,225,55,19,115,14,56,253,114,59,56,231,114,104,56,208,114,149,56,186,114,194,56,164,114,240,56,141,114,29,57,119,114,73,57,96,114,118,57,74,114,163,57,51,114,208,57,28,114,253,57,6,114,42,58,239,113,87,58,216,113,131,58,193,113,176,58,170,113,221,58,147,113,9,59,123,113,54,59,100,113,98,59,77,113,143,59,53,113,187,59,30,113,232,59,6,113,20,60,239,112,65,60,215,112,109,60,191,112,153,60,167,112,197,60,143,112,242,60,119,112,30,61,95,112,74,61,71,112,118,61,47,112,162,61,23,112,206,61,255,111,250,61,230,111,38,62,206,111,82,62,181,111,126,62,157,111,170,62,132,111,214,62,107,111,1,63,83,111,45,63,58,111,89,63,33,111,133,63,8,111,176,63,239,110,220,63,214,110,7,64,189,110,51,64,163,110,94,64,138,110,138,64,113,110,181,64,87,110,224,64,62,110,12,65,36,110,55,65,10,110,98,65,241,109,141,65,215,109,185,65,189,109,228,65,163,109,15,66,137,109,58,66,111,109,101,66,85,109,144,66,59,109,187,66,33,109,230,66,6,109,16,67,236,108,59,67,210,108,102,67,183,108,145,67,157,108,187,67,130,108,230,67,103,108,17,68,76,108,59,68,50,108,102,68,23,108,144,68,252,107,186,68,225,107,229,68,198,107,15,69,170,107,57,69,143,107,100,69,116,107,142,69,89,107,184,69,61,107,226,69,34,107,12,70,6,107,54,70,235,106,96,70,207,106,138,70,179,106,180,70,151,106,222,70,124,106,8,71,96,106,50,71,68,106,92,71,40,106,133,71,11,106,175,71,239,105,217,71,211,105,2,72,183,105,44,72,154,105,85,72,126,105,127,72,97,105,168,72,69,105,209,72,40,105,251,72,12,105,36,73,239,104,77,73,210,104,118,73,181,104,160,73,152,104,201,73,123,104,242,73,94,104,27,74,65,104,68,74,36,104,109,74,6,104,149,74,233,103,190,74,204,103,231,74,174,103,16,75,145,103,56,75,115,103,97,75,86,103,138,75,56,103,178,75,26,103,219,75,252,102,3,76,222,102,44,76,193,102,84,76,163,102,124,76,132,102,165,76,102,102,205,76,72,102,245,76,42,102,29,77,12,102,69,77,237,101,109,77,207,101,149,77,176,101,189,77,146,101,229,77,115,101,13,78,84,101,53,78,54,101,93,78,23,101,132,78,248,100,172,78,217,100,212,78,186,100,251,78,155,100,35,79,124,100,74,79,93,100,114,79,62,100,153,79,30,100,192,79,255,99,232,79,223,99,15,80,192,99,54,80,160,99,93,80,129,99,132,80,97,99,172,80,66,99,211,80,34,99,249,80,2,99,32,81,226,98,71,81,194,98,110,81,162,98,149,81,130,98,187,81,98,98,226,81,66,98,9,82,33,98,47,82,1,98,86,82,225,97,124,82,192,97,163,82,160,97,201,82,127,97,239,82,95,97,22,83,62,97,60,83,29,97,98,83,253,96,136,83,220,96,174,83,187,96,212,83,154,96,250,83,121,96,32,84,88,96,70,84,55,96,108,84,22,96,145,84,244,95,183,84,211,95,221,84,178,95,2,85,144,95,40,85,111,95,78,85,77,95,115,85,44,95,152,85,10,95,190,85,232,94,227,85,199,94,8,86,165,94,45,86,131,94,83,86,97,94,120,86,63,94,157,86,29,94,194,86,251,93,231,86,217,93,12,87,183,93,48,87,148,93,85,87,114,93,122,87,80,93,159,87,45,93,195,87,11,93,232,87,232,92,12,88,197,92,49,88,163,92,85,88,128,92,121,88,93,92,158,88,58,92,194,88,24,92,230,88,245,91,10,89,210,91,46,89,175,91,82,89,140,91,118,89,104,91,154,89,69,91,190,89,34,91,226,89,255,90,6,90,219,90,41,90,184,90,77,90,148,90,113,90,255,127,0,0,255,127,101,0,255,127,201,0,255,127,46,1,254,127,146,1,252,127,247,1,250,127,91,2,248,127,192,2,246,127,36,3,244,127,137,3,241,127,237,3,237,127,82,4,234,127,182,4,230,127,27,5,226,127,127,5,221,127,227,5,217,127,72,6,211,127,172,6,206,127,17,7,200,127,117,7,194,127,217,7,188,127,62,8,181,127,162,8,174,127,6,9,167,127,107,9,160,127,207,9,152,127,51,10,144,127,151,10,135,127,251,10,126,127,96,11,117,127,196,11,108,127,40,12,98,127,140,12,88,127,240,12,78,127,84,13,67,127,184,13,56,127,28,14,45,127,128,14,34,127,228,14,22,127,71,15,10,127,171,15,253,126,15,16,240,126,115,16,227,126,214,16,214,126,58,17,200,126,158,17,186,126,1,18,172,126,101,18,157,126,200,18,142,126,43,19,127,126,143,19,112,126,242,19,96,126,85,20,80,126,185,20,63,126,28,21,47,126,127,21,30,126,226,21,12,126,69,22,251,125,168,22,233,125,11,23,214,125,110,23,196,125,209,23,177,125,51,24,158,125,150,24,138,125,249,24,119,125,91,25,99,125,190,25,78,125,32,26,58,125,131,26,37,125,229,26,15,125,71,27,250,124,169,27,228,124,12,28,206,124,110,28,183,124,208,28,160,124,49,29,137,124,147,29,114,124,245,29,90,124,87,30,66,124,184,30,42,124,26,31,17,124,123,31,249,123,221,31,223,123,62,32,198,123,159,32,172,123,1,33,146,123,98,33,120,123,195,33,93,123,36,34,66,123,132,34,39,123,229,34,11,123,70,35,239,122,167,35,211,122,7,36,183,122,103,36,154,122,200,36,125,122,40,37,96,122,136,37,66,122,232,37,36,122,72,38,6,122,168,38,231,121,8,39,201,121,104,39,170,121,199,39,138,121,39,40,106,121,134,40,74,121,229,40,42,121,69,41,10,121,164,41,233,120,3,42,200,120,98,42,166,120,193,42,133,120,31,43,99,120,126,43,64,120,220,43,30,120,59,44,251,119,153,44,216,119,247,44,180,119,85,45,144,119,179,45,108,119,17,46,72,119,111,46,35,119,204,46,254,118,42,47,217,118,135,47,180,118,229,47,142,118,66,48,104,118,159,48,66,118,252,48,27,118,89,49,244,117,181,49,205,117,18,50,166,117,110,50,126,117,203,50,86,117,39,51,45,117,131,51,5,117,223,51,220,116,59,52,179,116,151,52,137,116,242,52,96,116,78,53,54,116,169,53,11,116,4,54,225,115,95,54,182,115,186,54,139,115,21,55,95,115,112,55,52,115,202,55,8,115,37,56,220,114,127,56,175,114,217,56,130,114,51,57,85,114,141,57,40,114,231,57,250,113,64,58,204,113,154,58,158,113,243,58,112,113,76,59,65,113,165,59,18,113,254,59,227,112,87,60,179,112,175,60,131,112,8,61,83,112,96,61,35,112,184,61,242,111,16,62,194,111,104,62,144,111,192,62,95,111,23,63,45,111,111,63,251,110,198,63,201,110,29,64,151,110,116,64,100,110,203,64,49,110,33,65,254,109,120,65,202,109,206,65,150,109,36,66,98,109,122,66,46,109,208,66,249,108,38,67,196,108,123,67,143,108,209,67,90,108,38,68,36,108,123,68,238,107,208,68,184,107,36,69,130,107,121,69,75,107,205,69,20,107,33,70,221,106,117,70,165,106,201,70,110,106,29,71,54,106,112,71,253,105,196,71,197,105,23,72,140,105,106,72,83,105,189,72,26,105,15,73,224,104,98,73,167,104,180,73,109,104,6,74,50,104,88,74,248,103,170,74,189,103,251,74,130,103,77,75,71,103,158,75,11,103,239,75,208,102,64,76,147,102,145,76,87,102,225,76,27,102,49,77,222,101,129,77,161,101,209,77,100,101,33,78,38,101,113,78,233,100,192,78,171,100,15,79,108,100,94,79,46,100,173,79,239,99,251,79,176,99,74,80,113,99,152,80,50,99,230,80,242,98,52,81,178,98,129,81,114,98,207,81,50,98,28,82,241,97,105,82,176,97,182,82,111,97,3,83,46,97,79,83,236,96,155,83,170,96,231,83,104,96,51,84,38,96,127,84,228,95,202,84,161,95,21,85,94,95,96,85,27,95,171,85,215,94,246,85,148,94,64,86,80,94,138,86,12,94,212,86,200,93,30,87,131,93,103,87,62,93,177,87,249,92,250,87,180,92,67,88,111,92,140,88,41,92,212,88,227,91,28,89,157,91,100,89,87,91,172,89,16,91,244,89,201,90,59,90,130,90,130,90,0,0,0,0,255,127,0,0,255,127,107,0,255,127,214,0,254,127,66,1,253,127,173,1,252,127,24,2,250,127,131,2,247,127,239,2,245,127,90,3,242,127,197,3,238,127,48,4,235,127,155,4,231,127,6,5,226,127,114,5,222,127,221,5,217,127,72,6,211,127,179,6,205,127,30,7,199,127,137,7,193,127,244,7,186,127,95,8,179,127,202,8,171,127,53,9,163,127,160,9,155,127,11,10,146,127,118,10,137,127,225,10,128,127,76,11,119,127,182,11,109,127,33,12,98,127,140,12,88,127,247,12,76,127,97,13,65,127,204,13,53,127,54,14,41,127,161,14,29,127,11,15,16,127,118,15,3,127,224,15,245,126,75,16,232,126,181,16,217,126,31,17,203,126,138,17,188,126,244,17,173,126,94,18,157,126,200,18,141,126,50,19,125,126,156,19,109,126,6,20,92,126,112,20,74,126,218,20,57,126,68,21,39,126,173,21,20,126,23,22,2,126,128,22,239,125,234,22,219,125,83,23,200,125,189,23,180,125,38,24,159,125,144,24,138,125,249,24,117,125,98,25,96,125,203,25,74,125,52,26,52,125,157,26,29,125,6,27,7,125,110,27,240,124,215,27,216,124,64,28,192,124,168,28,168,124,17,29,143,124,121,29,119,124,226,29,93,124,74,30,68,124,178,30,42,124,26,31,16,124,130,31,245,123,234,31,218,123,82,32,191,123,185,32,163,123,33,33,136,123,137,33,107,123,240,33,79,123,87,34,50,123,191,34,20,123,38,35,247,122,141,35,217,122,244,35,187,122,91,36,156,122,193,36,125,122,40,37,94,122,143,37,62,122,245,37,30,122,91,38,254,121,194,38,221,121,40,39,188,121,142,39,155,121,244,39,121,121,90,40,87,121,191,40,53,121,37,41,18,121,138,41,239,120,240,41,204,120,85,42,168,120,186,42,133,120,31,43,96,120,132,43,60,120,233,43,23,120,78,44,241,119,178,44,204,119,23,45,166,119,123,45,128,119,223,45,89,119,67,46,50,119,167,46,11,119,11,47,227,118,110,47,187,118,210,47,147,118,53,48,107,118,153,48,66,118,252,48,24,118,95,49,239,117,194,49,197,117,36,50,155,117,135,50,112,117,233,50,70,117,76,51,27,117,174,51,239,116,16,52,195,116,114,52,151,116,212,52,107,116,53,53,62,116,151,53,17,116,248,53,228,115,89,54,182,115,186,54,136,115,27,55,90,115,124,55,43,115,220,55,252,114,61,56,205,114,157,56,157,114,253,56,109,114,93,57,61,114,189,57,13,114,28,58,220,113,124,58,171,113,219,58,121,113,58,59,71,113,153,59,21,113,248,59,227,112,87,60,176,112,181,60,125,112,20,61,74,112,114,61,22,112,208,61,226,111,45,62,174,111,139,62,121,111,233,62,69,111,70,63,15,111,163,63,218,110,0,64,164,110,93,64,110,110,185,64,56,110,22,65,1,110,114,65,202,109,206,65,147,109,42,66,91,109,134,66,35,109,225,66,235,108,61,67,179,108,152,67,122,108,243,67,65,108,77,68,7,108,168,68,206,107,2,69,148,107,93,69,90,107,183,69,31,107,17,70,228,106,106,70,169,106,196,70,110,106,29,71,50,106,118,71,246,105,207,71,186,105,40,72,125,105,128,72,64,105,216,72,3,105,48,73,197,104,136,73,136,104,224,73,74,104,55,74,11,104,143,74,205,103,230,74,142,103,61,75,79,103,147,75,15,103,234,75,208,102,64,76,143,102,150,76,79,102,236,76,15,102,65,77,206,101,151,77,141,101,236,77,75,101,65,78,9,101,150,78,200,100,234,78,133,100,62,79,67,100,147,79,0,100,230,79,189,99,58,80,122,99,142,80,54,99,225,80,242,98,52,81,174,98,135,81,105,98,217,81,37,98,43,82,224,97,126,82,154,97,207,82,85,97,33,83,15,97,115,83,201,96,196,83,131,96,21,84,60,96,101,84,245,95,182,84,174,95,6,85,103,95,86,85,31,95,166,85,215,94,246,85,143,94,69,86,71,94,148,86,254,93,227,86,181,93,50,87,108,93,128,87,35,93,206,87,217,92,28,88,143,92,106,88,69,92,183,88,250,91,4,89,176,91,81,89,101,91,158,89,26,91,234,89,206,90,55,90,130,90,130,90,0,0,0,0,0,130,128,0,0,0,0,0,0,128,0,0,2,130,128,0,2,128,128,0,2,130,0,0,2,0,0,0,0,128,0,0,0,2,0,0,0,130,128,0,2,130,128,0,0,2,0,0,2,2,128,0,2,128,128,0,0,0,128,0,2,0,0,0,2,2,0,0,0,2,128,0,0,2,128,0,0,130,0,0,0,130,0,0,0,128,128,0,0,128,128,0,2,2,128,0,2,128,0,0,2,0,128,0,2,0,128,0,2,128,0,0,0,0,0,0,2,2,0,0,2,130,0,0,0,0,128,0,0,128,0,0,2,130,128,0,2,0,0,0,0,128,128,0,0,130,128,0,0,0,128,0,0,0,128,0,0,2,0,0,2,128,128,0,0,128,0,0,0,130,0,0,2,0,128,0,0,2,0,0,2,0,0,0,2,2,128,0,2,130,0,0,2,130,128,0,2,128,0,0,0,128,128,0,2,2,128,0,2,0,128,0,2,2,0,0,2,130,0,0,0,130,128,0,2,2,0,0,0,2,128,0,0,2,128,0,0,0,0,0,2,128,0,0,0,130,0,0,0,0,0,0,2,128,128,0,16,64,8,64,0,64,0,64,0,64,0,0,16,64,8,0,0,0,8,0,16,0,0,0,16,0,8,64,16,64,0,64,16,0,0,64,16,64,8,64,0,64,8,64,0,0,0,64,0,64,0,64,0,0,8,0,16,0,0,0,16,0,8,64,0,64,8,0,16,0,8,0,16,64,0,64,0,0,0,0,0,0,0,64,0,64,0,0,16,64,8,0,0,0,8,64,16,0,8,0,16,0,0,64,0,0,0,0,0,64,8,0,16,64,0,0,0,64,8,64,0,0,8,64,16,64,0,0,0,0,0,0,16,64,8,0,16,0,8,64,0,0,8,0,16,64,0,64,0,0,8,64,0,64,8,64,0,64,0,0,0,0,8,64,0,64,0,64,16,0,0,0,16,64,8,64,16,64,8,0,16,0,0,0,0,64,0,0,0,0,0,64,16,64,0,0,0,64,8,64,0,0,8,0,16,0,0,64,16,0,8,0,16,64,0,64,16,0,0,64,16,0,8,0,0,64,8,0,0,0,0,0,0,64,0,64,16,64,0,0,0,0,0,64,16,0,8,64,16,64,8,64,0,64,8,0,4,1,0,0,0,1,1,4,0,0,0,0,4,0,1,4,0,1,0,4,0,0,0,0,4,1,1,0,0,1,0,4,4,0,1,0,4,0,0,4,4,0,0,4,0,0,1,0,4,1,1,4,4,0,1,0,0,0,1,4,4,1,0,0,0,0,0,4,4,0,0,0,0,1,1,4,0,1,0,0,0,1,1,0,0,0,1,4,4,0,1,4,4,1,1,0,4,1,0,4,0,1,1,0,0,0,1,0,4,1,0,4,4,0,0,0,4,1,1,4,0,1,0,0,0,0,0,4,0,1,1,4,0,0,0,4,4,0,1,0,4,1,0,0,0,0,1,0,0,1,1,4,0,1,0,4,0,0,0,0,0,1,0,0,4,0,1,0,4,1,1,4,0,1,0,4,4,0,0,4,0,1,0,0,0,0,0,0,4,0,1,4,4,1,0,4,0,0,1,0,0,0,0,4,4,1,1,4,4,0,0,0,4,1,1,0,0,1,1,0,4,0,0,4,0,0,1,4,4,1,0,4,4,1,0,0,0,0,1,4,4,1,1,0,4,0,0,0,4,0,1,4,0,1,1,0,0,16,64,128,64,16,0,128,64,16,0,128,64,0,0,0,64,16,64,0,64,0,64,128,0,0,64,128,0,16,0,128,0,0,0,0,0,16,64,0,0,16,64,0,64,16,64,128,64,0,0,128,0,0,0,0,64,0,64,0,0,0,64,128,0,0,0,128,0,16,0,0,0,0,64,0,0,16,64,128,64,0,0,0,0,0,64,0,0,16,0,128,64,16,0,0,64,0,64,128,0,0,0,128,64,16,0,0,64,0,64,0,0,16,0,0,64,16,64,0,64,16,64,128,64,0,0,128,64,0,64,0,0,0,64,128,0,16,64,0,64,16,64,128,64,0,0,128,0,0,0,0,0,0,0,0,0,16,64,0,64,16,0,0,64,0,64,0,64,0,64,128,0,0,0,128,0,16,64,128,64,16,0,128,64,16,0,128,64,0,0,0,64,16,64,128,64,0,0,128,0,0,0,128,0,16,0,0,0,0,64,128,0,16,0,128,64,16,64,0,64,0,64,128,0,16,0,128,64,16,0,0,0,0,64,0,0,16,64,128,64,0,0,0,0,0,64,0,0,16,0,0,64,16,64,0,128,0,0,0,128,0,4,1,0,0,4,1,128,0,0,33,0,0,4,0,128,0,0,0,0,0,0,32,0,0,4,1,128,0,4,32,0,0,4,0,128,0,0,1,128,0,4,32,128,0,0,33,0,0,4,33,128,0,4,0,0,0,0,32,0,0,0,1,0,0,4,32,0,0,4,32,0,0,0,0,128,0,0,32,128,0,4,33,128,0,4,33,128,0,0,1,0,0,4,33,128,0,0,32,0,0,0,0,0,0,0,33,128,0,4,1,0,0,0,1,0,0,0,33,128,0,4,0,0,0,4,0,128,0,0,33,128,0,0,0,0,0,0,1,0,0,0,32,0,0,4,1,128,0,0,33,128,0,4,32,128,0,0,1,0,0,0,32,0,0,4,33,128,0,4,1,128,0,4,32,128,0,0,0,0,0,0,1,0,0,4,33,128,0,4,33,128,0,4,0,0,0,0,33,128,0,4,33,0,0,4,1,0,0,0,0,0,0,4,32,0,0,0,33,128,0,4,0,128,0,0,1,128,0,0,32,0,0,4,0,0,0,0,0,0,0,4,32,128,0,4,1,128,0,0,32,8,0,0,16,0,0,32,16,0,32,0,0,8,32,32,16,0,0,32,16,8,0,0,0,8,32,32,16,0,0,32,0,0,32,0,16,8,32,32,0,0,0,32,0,8,0,0,16,8,0,32,0,0,32,0,16,0,0,0,16,8,32,0,0,0,0,0,0,8,0,32,0,8,32,0,16,0,32,0,0,0,32,32,0,8,32,0,16,8,0,0,0,8,0,32,16,8,0,32,16,0,0,0,0,8,32,32,0,0,32,32,16,8,32,0,0,0,32,32,0,0,32,32,16,0,0,0,16,0,32,0,16,8,0,0,0,8,0,32,16,0,32,32,0,8,32,32,16,0,0,32,0,8,32,0,0,8,0,0,16,0,0,32,0,0,32,0,16,0,0,0,16,8,32,0,0,8,0,0,16,8,32,32,16,0,32,32,0,0,0,32,16,8,32,32,0,0,32,32,16,0,0,0,0,8,0,32,16,8,0,0,0,0,32,0,0,0,0,32,16,8,32,32,0,0,32,0,0,8,0,32,0,8,32,0,16,0,0,0,0,0,32,32,16,0,0,0,16,8,0,32,0,8,32,0,16,0,0,16,0,1,0,16,2,1,4,0,2,0,0,0,0,0,4,0,0,1,4,0,2,1,4,16,0,0,4,16,2,1,4,16,2,0,0,16,0,0,0,0,0,1,0,0,2,1,0,0,0,0,0,0,2,1,0,16,2,1,4,0,0,0,4,0,2,1,4,16,0,1,0,16,0,0,4,0,2,1,0,0,2,0,0,16,2,0,4,16,2,1,0,16,0,0,0,16,2,0,4,0,0,1,4,0,0,1,4,16,2,0,4,16,0,1,0,0,0,0,0,0,2,0,4,16,0,0,0,0,2,0,4,16,0,0,0,16,0,1,4,0,2,1,4,0,2,1,0,16,2,1,0,16,2,1,0,0,0,1,0,16,0,0,0,0,2,0,4,0,2,0,0,16,0,0,4,16,2,1,4,0,0,1,4,16,0,0,4,16,2,1,4,0,0,1,0,0,2,1,4,16,2,0,0,16,2,0,4,16,0,0,0,0,0,1,0,0,0,1,4,16,2,0,0,0,0,1,4,16,0,0,0,16,2,0,4,0,0,1,0,0,2,0,4,0,2,0,4,0,0,1,0,16,0,32,8,0,8,0,8,0,0,0,0,2,0,32,8,2,8,0,0,0,8,32,8,0,8,32,0,0,0,0,0,0,8,32,0,2,0,0,0,2,8,32,8,2,8,0,8,2,0,0,8,2,8,32,8,2,0,0,8,0,0,32,0,0,0,0,0,2,8,32,0,0,8,0,8,0,8,32,8,0,0,0,8,2,0,32,0,2,0,32,0,2,8,0,8,2,8,32,8,0,0,0,0,0,0,0,0,0,0,32,0,2,8,32,0,0,8,0,8,0,8,32,8,2,0,0,0,2,0,32,8,2,0,0,0,2,0,0,8,2,8,0,8,0,0,32,0,0,0,32,0,2,8,0,8,0,0,32,8,2,0,0,8,0,8,32,0,0,0,32,0,0,8,0,0,2,8])
.concat([32,0,2,8,0,0,0,8,0,0,2,0,32,8,0,8,0,0,0,0,32,8,2,8,32,0,2,0,32,0,0,8,0,0,2,8,0,8,0,8,32,8,0,8,0,0,0,0,32,8,2,8,0,8,2,0,0,8,2,0,32,8,0,0,32,8,0,0,32,0,2,0,0,0,0,8,0,8,2,8,42,0,0,0,62,0,0,0,76,127,52,125,188,121,52,125,239,116,142,103,188,121,142,103,61,75,239,116,166,85,142,39,218,110,0,64,0,0,142,103,142,39,114,216,31,95,97,13,195,180,166,85,159,242,114,152,61,75,114,216,68,134,0,64,0,192,0,128,16,52,90,170,68,134,142,39,114,152,114,152,157,26,17,139,195,180,97,13,204,130,114,216,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,253,127,245,127,231,127,211,127,186,127,155,127,119,127,76,127,29,127,232,126,173,126,109,126,39,126,219,125,138,125,52,125,216,124,119,124,16,124,163,123,50,123,187,122,62,122,188,121,53,121,168,120,23,120,128,119,227,118,66,118,155,117,245,127,211,127,155,127,76,127,232,126,109,126,219,125,52,125,119,124,163,123,187,122,188,121,168,120,128,119,66,118,239,116,136,115,13,114,125,112,218,110,35,109,90,107,125,105,142,103,141,101,122,99,85,97,31,95,217,92,130,90,28,88,231,127,155,127,29,127,109,126,138,125,119,124,50,123,188,121,23,120,66,118,62,116,13,114,174,111,35,109,110,106,142,103,133,100,85,97,254,93,130,90,227,86,33,83,62,79,61,75,29,71,225,66,139,62,28,58,151,53,252,48,78,44,211,127,76,127,109,126,52,125,163,123,188,121,128,119,239,116,13,114,218,110,90,107,142,103,122,99,31,95,130,90,166,85,142,80,61,75,183,69,0,64,28,58,16,52,223,45,142,39,33,33,157,26,6,20,97,13,179,6,0,0,77,249,186,127,232,126,138,125,163,123,53,121,66,118,205,114,218,110,110,106,141,101,60,96,130,90,101,84,236,77,29,71,0,64,157,56,252,48,37,41,33,33,249,24,181,16,95,8,0,0,161,247,75,239,7,231,223,222,219,214,4,207,99,199,155,127,109,126,119,124,188,121,66,118,13,114,35,109,142,103,85,97,130,90,33,83,61,75,225,66,28,58,252,48,142,39,226,29,6,20,11,10,0,0,245,245,250,235,30,226,114,216,4,207,228,197,31,189,195,180,223,172,126,165,171,158,119,127,219,125,50,123,128,119,205,114,35,109,143,102,31,95,227,86,236,77,77,68,28,58,110,47,91,36,249,24,97,13,173,1,245,245,83,234,223,222,178,211,229,200,142,190,195,180,155,171,39,163,123,155,166,148,185,142,190,137,194,133,76,127,52,125,188,121,239,116,218,110,142,103,31,95,166,85,61,75,0,64,16,52,142,39,157,26,97,13,0,0,159,242,99,229,114,216,240,203,0,192,195,180,90,170,225,160,114,152,38,145,17,139,68,134,204,130,180,128,0,128,180,128,29,127,119,124,23,120,13,114,110,106,85,97,227,86,61,75,139,62,252,48,191,34,6,20,6,5,245,245,7,231,114,216,105,202,31,189,194,176,126,165,123,155,221,146,194,139,68,134,118,130,101,128,25,128,147,129,206,132,190,137,82,144,232,126,163,123,66,118,218,110,141,101,130,90,236,77,0,64,252,48,33,33,181,16,0,0,75,239,223,222,4,207,0,192,20,178,126,165,115,154,38,145,190,137,93,132,24,129,0,128,24,129,93,132,190,137,38,145,115,154,126,165,20,178,173,126,187,122,62,116,90,107,60,96,33,83,77,68,16,52,191,34,181,16,83,254,250,235,11,218,229,200,227,184,90,170,151,157,221,146,101,138,93,132,227,128,11,128,217,129,68,134,51,141,131,150,2,162,114,175,142,190,4,207,126,224,109,126,188,121,13,114,142,103,130,90,61,75,28,58,142,39,6,20,0,0,250,235,114,216,228,197,195,180,126,165,114,152,243,141,68,134,147,129,0,128,147,129,68,134,243,141,114,152,126,165,195,180,228,197,114,216,250,235,0,0,6,20,39,126,168,120,174,111,122,99,101,84,225,66,110,47,157,26,6,5,75,239,11,218,228,197,106,179,39,163,146,149,17,139,240,131,101,128,137,128,93,132,194,139,131,150,80,164,195,180,99,199,165,219,245,240,179,6,64,28,252,48,77,68,219,125,128,119,35,109,31,95,236,77,28,58,91,36,97,13,245,245,223,222,229,200,195,180,39,163,166,148,190,137,204,130,11,128,147,129,88,135,38,145,171,158,114,175,236,194,114,216,75,239,179,6,226,29,16,52,128,72,130,90,125,105,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,245,127,211,127,155,127,76,127,232,126,109,126,219,125,52,125,119,124,163,123,187,122,188,121,168,120,128,119,66,118,211,127,76,127,109,126,52,125,163,123,188,121,128,119,239,116,13,114,218,110,90,107,142,103,122,99,31,95,130,90,155,127,109,126,119,124,188,121,66,118,13,114,35,109,142,103,85,97,130,90,33,83,61,75,225,66,28,58,252,48,76,127,52,125,188,121,239,116,218,110,142,103,31,95,166,85,61,75,0,64,16,52,142,39,157,26,97,13,0,0,232,126,163,123,66,118,218,110,141,101,130,90,236,77,0,64,252,48,33,33,181,16,0,0,75,239,223,222,4,207,109,126,188,121,13,114,142,103,130,90,61,75,28,58,142,39,6,20,0,0,250,235,114,216,228,197,195,180,126,165,219,125,128,119,35,109,31,95,236,77,28,58,91,36,97,13,245,245,223,222,229,200,195,180,39,163,166,148,190,137,52,125,239,116,142,103,166,85,0,64,142,39,97,13,159,242,114,216,0,192,90,170,114,152,17,139,204,130,0,128,119,124,13,114,85,97,61,75,252,48,6,20,245,245,114,216,31,189,126,165,221,146,68,134,101,128,147,129,190,137,163,123,218,110,130,90,0,64,33,33,0,0,223,222,0,192,126,165,38,145,93,132,0,128,93,132,38,145,126,165,187,122,90,107,33,83,16,52,181,16,250,235,229,200,90,170,221,146,93,132,11,128,68,134,131,150,114,175,4,207,188,121,142,103,61,75,142,39,0,0,114,216,195,180,114,152,68,134,0,128,68,134,114,152,195,180,114,216,0,0,168,120,122,99,225,66,157,26,75,239,228,197,39,163,17,139,101,128,93,132,131,150,195,180,165,219,179,6,252,48,128,119,31,95,28,58,97,13,223,222,195,180,166,148,204,130,147,129,38,145,114,175,114,216,179,6,16,52,130,90,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,97,13,157,26,142,39,157,26,16,52,61,75,142,39,61,75,142,103,16,52,31,95,188,121,0,64,218,110,255,127,61,75,188,121,188,121,166,85,76,127,142,103,31,95,76,127,61,75,142,103,188,121,142,39,218,110,218,110,0,0,239,116,31,95,114,216,188,121,61,75,195,180,52,125,16,52,114,152,76,127,157,26,68,134,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,173,1,90,3,6,5,179,6,95,8,11,10,182,11,97,13,11,15,181,16,94,18,6,20,173,21,83,23,249,24,157,26,64,28,226,29,130,31,33,33,191,34,91,36,245,37,142,39,37,41,186,42,78,44,223,45,110,47,252,48,135,50,90,3,179,6,11,10,97,13,181,16,6,20,83,23,157,26,226,29,33,33,91,36,142,39,186,42,223,45,252,48,16,52,27,55,28,58,20,61,0,64,225,66,183,69,128,72,61,75,236,77,142,80,33,83,166,85,28,88,130,90,217,92,6,5,11,10,11,15,6,20,249,24,226,29,191,34,142,39,78,44,252,48,151,53,28,58,139,62,225,66,29,71,61,75,62,79,33,83,227,86,130,90,254,93,85,97,133,100,142,103,110,106,35,109,174,111,13,114,62,116,66,118,23,120,179,6,97,13,6,20,157,26,33,33,142,39,223,45,16,52,28,58,0,64,183,69,61,75,142,80,166,85,130,90,31,95,122,99,142,103,90,107,218,110,13,114,239,116,128,119,188,121,163,123,52,125,109,126,76,127,211,127,255,127,211,127,95,8,181,16,249,24,33,33,37,41,252,48,157,56,0,64,29,71,236,77,101,84,130,90,60,96,141,101,110,106,218,110,205,114,66,118,53,121,163,123,138,125,232,126,186,127,255,127,186,127,232,126,138,125,163,123,53,121,66,118,205,114,11,10,6,20,226,29,142,39,252,48,28,58,225,66,61,75,33,83,130,90,85,97,142,103,35,109,13,114,66,118,188,121,119,124,109,126,155,127,255,127,155,127,109,126,119,124,188,121,66,118,13,114,35,109,142,103,85,97,130,90,33,83,182,11,83,23,191,34,223,45,157,56,225,66,150,76,166,85,254,93,141,101,65,108,13,114,227,118,187,122,138,125,76,127,253,127,155,127,39,126,163,123,23,120,136,115,1,110,142,103,60,96,28,88,62,79,183,69,153,59,252,48,245,37,97,13,157,26,142,39,16,52,0,64,61,75,166,85,31,95,142,103,218,110,239,116,188,121,52,125,76,127,255,127,76,127,52,125,188,121,239,116,218,110,142,103,31,95,166,85,61,75,0,64,16,52,142,39,157,26,97,13,0,0,159,242,11,15,226,29,78,44,28,58,29,71,33,83,254,93,142,103,174,111,66,118,50,123,109,126,231,127,155,127,138,125,188,121,62,116,35,109,133,100,130,90,62,79,225,66,151,53,142,39,249,24,11,10,250,250,250,235,65,221,4,207,117,193,181,16,33,33,252,48,0,64,236,77,130,90,141,101,218,110,66,118,163,123,232,126,255,127,232,126,163,123,66,118,218,110,141,101,130,90,236,77,0,64,252,48,33,33,181,16,0,0,75,239,223,222,4,207,0,192,20,178,126,165,115,154,94,18,91,36,151,53,183,69,101,84,85,97,65,108,239,116,50,123,232,126,253,127,109,126,62,122,136,115,110,106,31,95,217,81,225,66,135,50,33,33,11,15,166,252,83,234,114,216,99,199,128,183,29,169,134,156,255,145,190,137,240,131,6,20,142,39,28,58,61,75,130,90,142,103,13,114,188,121,109,126,255,127,109,126,188,121,13,114,142,103,130,90,61,75,28,58,142,39,6,20,0,0,250,235,114,216,228,197,195,180,126,165,114,152,243,141,68,134,147,129,0,128,147,129,173,21,186,42,139,62,142,80,60,96,35,109,227,118,52,125,231,127,232,126,62,122,13,114,143,102,28,88,29,71,16,52,130,31,11,10,74,244,223,222,105,202,128,183,175,166,114,152,51,141,69,133,227,128,45,128,40,131,190,137,191,147,83,23,223,45,225,66,166,85,141,101,13,114,187,122,76,127,155,127,163,123,136,115,142,103,28,88,183,69,252,48,157,26,90,3,250,235,70,213,0,192,223,172,134,156,131,143,68,134,24,129,45,128,137,131,17,139,131,150,126,165,128,183,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,90,3,179,6,11,10,97,13,181,16,6,20,83,23,157,26,226,29,33,33,91,36,142,39,186,42,223,45,252,48,179,6,97,13,6,20,157,26,33,33,142,39,223,45,16,52,28,58,0,64,183,69,61,75,142,80,166,85,130,90,11,10,6,20,226,29,142,39,252,48,28,58,225,66,61,75,33,83,130,90,85,97,142,103,35,109,13,114,66,118,97,13,157,26,142,39,16,52,0,64,61,75,166,85,31,95,142,103,218,110,239,116,188,121,52,125,76,127,255,127,181,16,33,33,252,48,0,64,236,77,130,90,141,101,218,110,66,118,163,123,232,126,255,127,232,126,163,123,66,118,6,20,142,39,28,58,61,75,130,90,142,103,13,114,188,121,109,126,255,127,109,126,188,121,13,114,142,103,130,90,83,23,223,45,225,66,166,85,141,101,13,114,187,122,76,127,155,127,163,123,136,115,142,103,28,88,183,69,252,48,157,26,16,52,61,75,31,95,218,110,188,121,76,127,76,127,188,121,218,110,31,95,61,75,16,52,157,26,0,0,226,29,28,58,33,83,142,103,66,118,109,126,155,127,188,121,35,109,130,90,225,66,142,39,11,10,250,235,4,207,33,33,0,64,130,90,218,110,163,123,255,127,163,123,218,110,130,90,0,64,33,33,0,0,223,222,0,192,126,165,91,36,183,69,85,97,239,116,232,126,109,126,136,115,31,95,225,66,33,33,166,252,114,216,128,183,134,156,190,137,142,39,61,75,142,103,188,121,255,127,188,121,142,103,61,75,142,39,0,0,114,216,195,180,114,152,68,134,0,128,186,42,142,80,35,109,52,125,232,126,13,114,28,88,16,52,11,10,223,222,128,183,114,152,69,133,45,128,190,137,223,45,166,85,13,114,76,127,163,123,142,103,183,69,157,26,250,235,0,192,134,156,68,134,45,128,17,139,126,165,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,42,39,45,32,55,51,53,28,41,50,35,46,33,37,44,52,30,48,40,49,29,36,43,54,15,4,25,19,9,1,26,16,5,11,23,8,12,7,17,0,22,3,10,14,6,20,27,24,7,15,23,31,39,47,55,63,6,14,22,30,38,46,54,62,5,13,21,29,37,45,53,61,4,12,20,28,1,9,17,25,33,41,49,57,2,10,18,26,34,42,50,58,3,11,19,27,35,43,51,59,36,44,52,60,20,0,0,0,34,0,0,0,71,0,0,0,91,0,0,0,30,0,0,0,50,0,0,0,255,127,10,0,255,127,15,0,255,127,19,0,255,127,22,0,255,127,26,0,255,127,29,0,255,127,33,0,255,127,37,0,255,127,40,0,255,127,44,0,255,127,47,0,255,127,51,0,255,127,55,0,255,127,59,0,255,127,63,0,255,127,67,0,255,127,71,0,255,127,75,0,255,127,80,0,255,127,84,0,255,127,89,0,255,127,93,0,255,127,98,0,255,127,103,0,255,127,108,0,255,127,113,0,255,127,118,0,255,127,123,0,255,127,128,0,255,127,134,0,255,127,140,0,255,127,145,0,255,127,151,0,255,127,157,0,255,127,163,0,255,127,169,0,255,127,176,0,255,127,182,0,255,127,189,0,255,127,195,0,255,127,202,0,255,127,209,0,255,127,216,0,255,127,224,0,255,127,231,0,255,127,239,0,255,127,246,0,255,127,254,0,255,127,6,1,255,127,14,1,255,127,22,1,255,127,31,1,255,127,39,1,255,127,48,1,255,127,57,1,254,127,66,1,254,127,75,1,254,127,84,1,254,127,94,1,254,127,103,1,254,127,113,1,254,127,123,1,254,127,133,1,254,127,143,1,253,127,154,1,253,127,164,1,253,127,175,1,253,127,186,1,253,127,197,1,253,127,208,1,253,127,220,1,252,127,231,1,252,127,243,1,252,127,255,1,252,127,11,2,252,127,24,2,251,127,36,2,251,127,49,2,251,127,62,2,251,127,75,2,251,127,88,2,250,127,101,2,250,127,115,2,250,127,129,2,249,127,143,2,249,127,157,2,249,127,172,2,249,127,186,2,248,127,201,2,248,127,216,2,248,127,231,2,247,127,247,2,247,127,6,3,246,127,22,3,246,127,38,3,246,127,54,3,245,127,71,3,245,127,87,3,244,127,104,3,244,127,121,3,243,127,138,3,243,127,156,3,242,127,174,3,242,127,192,3,241,127,210,3,241,127,228,3,240,127,247,3,240,127,9,4,239,127,28,4,238,127,48,4,238,127,67,4,237,127,87,4,236,127,107,4,236,127,127,4,235,127,147,4,234,127,168,4,234,127,189,4,233,127,210,4,232,127,231,4,231,127,253,4,230,127,19,5,229,127,41,5,228,127,63,5,228,127,85,5,227,127,108,5,226,127,131,5,225,127,154,5,224,127,178,5,222,127,202,5,221,127,226,5,220,127,250,5,219,127,18,6,218,127,43,6,217,127,68,6,215,127,93,6,214,127,119,6,213,127,144,6,212,127,170,6,210,127,197,6,209,127,223,6,207,127,250,6,206,127,21,7,204,127,48,7,203,127,76,7,201,127,104,7,199,127,132,7,198,127,160,7,196,127,189,7,194,127,217,7,193,127,247,7,191,127,20,8,189,127,50,8,187,127,80,8,185,127,110,8,183,127,140,8,181,127,171,8,179,127,202,8,177,127,233,8,174,127,9,9,172,127,41,9,170,127,73,9,167,127,105,9,165,127,138,9,162,127,171,9,160,127,204,9,157,127,237,9,155,127,15,10,152,127,49,10,149,127,83,10,146,127,118,10,144,127,153,10,141,127,188,10,138,127,223,10,135,127,3,11,131,127,39,11,128,127,75,11,125,127,112,11,122,127,149,11,118,127,186,11,115,127,223,11,111,127,5,12,108,127,43,12,104,127,81,12,100,127,120,12,96,127,158,12,92,127,197,12,89,127,237,12,84,127,20,13,80,127,60,13,76,127,101,13,72,127,141,13,67,127,182,13,63,127,223,13,58,127,9,14,54,127,50,14,49,127,92,14,44,127,134,14,39,127,177,14,34,127,220,14,29,127,7,15,24,127,50,15,19,127,94,15,14,127,138,15,8,127,182,15,3,127,227,15,253,126,16,16,247,126,61,16,241,126,106,16,235,126,152,16,229,126,198,16,223,126,244,16,217,126,35,17,211,126,82,17,204,126,129,17,198,126,176,17,191,126,224,17,184,126,16,18,177,126,64,18,170,126,113,18,163,126,162,18,156,126,211,18,148,126,4,19,141,126,54,19,133,126,104,19,125,126,155,19,118,126,205,19,110,126,0,20,101,126,51,20,93,126,102,20,85,126,154,20,76,126,206,20,68,126,2,21,59,126,55,21,50,126,108,21,41,126,161,21,32,126,214,21,22,126,12,22,13,126,66,22,3,126,120,22,249,125,175,22,240,125,229,22,229,125,28,23,219,125,84,23,209,125,139,23,198,125,195,23,188,125,251,23,177,125,52,24,166,125,108,24,155,125,165,24,144,125,222,24,132,125,24,25,121,125,82,25,109,125,140,25,97,125,198,25,85,125,1,26,73,125,59,26,60,125,118,26,48,125,178,26,35,125,237,26,22,125,41,27,9,125,101,27,251,124,162,27,238,124,222,27,224,124,27,28,211,124,88,28,197,124,149,28,182,124,211,28,168,124,17,29,153,124,79,29,139,124,141,29,124,124,204,29,109,124,11,30,93,124,74,30,78,124,137,30,62,124,201,30,46,124,9,31,30,124,73,31,14,124,137,31,253,123,202,31,237,123,10,32,220,123,75,32,203,123,141,32,186,123,206,32,168,123,16,33,150,123,82,33,132,123,148,33,114,123,214,33,96,123,25,34,78,123,91,34,59,123,158,34,40,123,226,34,21,123,37,35,1,123,105,35,238,122,173,35,218,122,241,35,198,122,53,36,177,122,122,36,157,122,190,36,136,122,3,37,115,122,72,37,94,122,142,37,73,122,211,37,51,122,25,38,29,122,95,38,7,122,165,38,241,121,235,38,218,121,50,39,195,121,120,39,172,121,191,39,149,121,6,40,125,121,78,40,101,121,149,40,77,121,221,40,53,121,36,41,29,121,108,41,4,121,181,41,235,120,253,41,210,120,69,42,184,120,142,42,158,120,215,42,132,120,32,43,106,120,105,43,80,120,178,43,53,120,251,43,26,120,69,44,254,119,143,44,227,119,217,44,199,119,35,45,171,119,109,45,143,119,183,45,114,119,2,46,85,119,76,46,56,119,151,46,27,119,226,46,253,118,45,47,223,118,120,47,193,118,196,47,163,118,15,48,132,118,91,48,101,118,166,48,70,118,242,48,38,118,62,49,6,118,138,49,230,117,214,49,198,117,34,50,165,117,111,50,133,117,187,50,99,117,8,51,66,117,84,51,32,117,161,51,254,116,238,51,220,116,59,52,185,116,136,52,150,116,213,52,115,116,34,53,80,116,112,53,44,116,189,53,8,116,11,54,228,115,88,54,192,115,166,54,155,115,243,54,118,115,65,55,80,115,143,55,43,115,221,55,5,115,43,56,222,114,121,56,184,114,199,56,145,114,21,57,106,114,100,57,67,114,178,57,27,114,0,58,243,113,79,58,203,113,157,58,162,113,235,58,121,113,58,59,80,113,136,59,39,113,215,59,253,112,38,60,211,112,116,60,169,112,195,60,126,112,18,61,83,112,96,61,40,112,175,61,253,111,254,61,209,111,76,62,165,111,155,62,121,111,234,62,76,111,57,63,31,111,136,63,242,110,214,63,196,110,37,64,151,110,116,64,105,110,195,64,58,110,18,65,12,110,96,65,221,109,175,65,173,109,254,65,126,109,76,66,78,109,155,66,30,109,234,66,238,108,56,67,189,108,135,67,140,108,214,67,91,108,36,68,41,108,115,68,247,107,193,68,197,107,16,69,147,107,94,69,96,107,172,69,45,107,251,69,250,106,73,70,198,106,151,70,147,106,229,70,95,106,51,71,42,106,130,71,245,105,207,71,192,105,29,72,139,105,107,72,86,105,185,72,32,105,7,73,234,104,84,73,179,104,162,73,125,104,239,73,70,104,61,74,15,104,138,74,215,103,215,74,159,103,36,75,103,103,113,75,47,103,190,75,247,102,11,76,190,102,88,76,133,102,165,76,75,102,241,76,18,102,61,77,216,101,138,77,157,101,214,77,99,101,34,78,40,101,110,78,237,100,186,78,178,100,6,79,118,100,81,79,59,100,157,79,255,99,232,79,194,99,51,80,134,99,127,80,73,99,202,80,12,99,20,81,206,98,95,81,145,98,170,81,83,98,244,81,21,98,62,82,215,97,136,82,152,97,210,82,89,97,28,83,26,97,102,83,219,96,175,83,155,96,249,83,91,96,66,84,27,96,139,84,219,95,212,84,154,95,29,85,90,95,101,85,25,95,174,85,215,94,246,85,150,94,62,86,84,94,134,86,18,94,205,86,208,93,21,87,142,93,92,87,75,93,163,87,8,93,234,87,197,92,49,88,130,92,119,88,62,92,190,88,251,91,4,89,183,91,74,89,115,91,144,89,46,91,213,89,234,90,27,90,165,90,96,90,255,127,1,0,255,127,4,0,255,127,8,0,255,127,13,0,255,127,20,0,255,127,29,0,255,127,41,0,255,127,57,0,255,127,76,0,255,127,99,0,255,127,128,0,255,127,162,0,255,127,203,0,255,127,251,0,255,127,51,1,254,127,116,1,253,127,190,1,252,127,20,2,250,127,117,2,248,127,227,2,245,127,94,3,241,127,232,3,236,127,129,4,229,127,43,5,221,127,231,5,211,127,180,6,198,127,149,7,183,127,138,8,164,127,147,9,141,127,178,10,114,127,231,11,81,127,50,13,43,127,149,14,253,126,14,16,200,126,159,17,138,126,71,19,67,126,7,21,241,125,222,22,147,125,204,24,41,125,208,26,177,124,235,28,42,124,27,31,147,123,95,33,235,122,183,35,49,122,33,38,99,121,156,40,130,120,39,43,139,119,192,45,127,118,102,48,92,117,24,51,34,116,210,53,209,114,149,56,103,113,93,59,229,111,41,62,74,110,246,64,151,108,196,67,204,106,144,70,232,104,87,73,237,102,24,76,218,100,210,78,178,98,130,81,116,96,38,84,34,94,189,86,188,91,69,89,255,127,1,0,255,127,4,0,255,127,8,0,255,127,14,0,255,127,22,0,255,127,33,0,255,127,48,0,255,127,66,0,255,127,89,0,255,127,117,0,255,127,151,0,255,127,193,0,255,127,242,0,255,127,45,1,254,127,113,1,253,127,193,1,252,127,29,2,250,127,134,2,247,127,254,2,244,127,134,3,239,127,30,4,233,127,201,4,225,127,134,5,216,127,89,6,203,127,64,7,188,127,62,8,169,127,83,9,146,127,129,10,117,127,199,11,83,127,39,13,41,127,160,14,248,126,53,16,190,126,228,17,123,126,173,19,43,126,145,21,208,125,144,23,103,125,169,25,239,124,219,27,102,124,37,30,204,123,136,32,31,123,0,35,94,122,143,37,135,121,49,40,153,120,229,42,148,119,170,45,118,118,125,48,62,117,93,51,236,115,71,54,127,114,58,57,246,112,51,60,81,111,48,63,145,109,45,66,180,107,42,69,188,105,35,72,169,103,23,75,124,101,2,78,53,99,226,80,213,96,182,83,94,94,123,86,209,91,47,89,255,127,10,0,255,127,14,0,255,127,18,0,255,127,21,0,255,127,25,0,255,127,28,0,255,127,32,0,255,127,35,0,255,127,38,0,255,127,42,0,255,127,45,0,255,127,48,0,255,127,52,0,255,127,56,0,255,127,59,0,255,127,63,0,255,127,67,0,255,127,71,0,255,127,75,0,255,127,79,0,255,127,83,0,255,127,87,0,255,127,91,0,255,127,96,0,255,127,100,0,255,127,105,0,255,127,109,0,255,127,114,0,255,127,119,0,255,127,124,0,255,127,129,0,255,127,134,0,255,127,139,0,255,127,145,0,255,127,150,0,255,127,156,0,255,127,161,0,255,127,167,0,255,127,173,0,255,127,179,0,255,127,185,0,255,127,191,0,255,127,198,0,255,127,204,0,255,127,211,0,255,127,218,0,255,127,224,0,255,127,231,0,255,127,238,0,255,127,246,0,255,127,253,0,255,127,4,1,255,127,12,1,255,127,19,1,255,127,27,1,255,127,35,1,255,127,43,1,255,127,51,1,254,127,60,1,254,127,68,1,254,127,77,1,254,127,86,1,254,127,95,1,254,127,104,1,254,127,113,1,254,127,122,1,254,127,131,1,254,127,141,1,253,127,151,1,253,127,161,1,253,127,171,1,253,127,181,1,253,127,191,1,253,127,202,1,253,127,212,1,252,127,223,1,252,127,234,1,252,127,245,1,252,127,0,2,252,127,12,2,252,127,23,2,251,127,35,2,251,127,47,2,251,127,59,2,251,127,71,2,251,127,83,2,250,127,96,2,250,127,109,2,250,127,122,2,250,127,135,2,249,127,148,2,249,127,161,2,249,127,175,2,249,127,188,2,248,127,202,2,248,127,216,2,248,127,231,2,247,127,245,2,247,127,4,3,247,127,18,3,246,127,33,3,246,127,49,3,245,127,64,3,245,127,79,3,245,127,95,3,244,127,111,3,244,127,127,3,243,127,143,3,243,127,160,3,242,127,176,3,242,127,193,3,241,127,210,3,241,127,227,3,240,127,245,3,240,127,6,4,239,127,24,4,239,127,42,4,238,127,60,4,237,127,79,4,237,127,97,4,236,127,116,4,235,127,135,4,235,127,154,4,234,127,174,4,233,127,193,4,233,127,213,4,232,127,233,4,231,127,253,4,230,127,18,5,229,127,39,5,229,127,59,5,228,127,80,5,227,127,102,5,226,127,123,5,225,127,145,5,224,127,167,5,223,127,189,5,222,127,211,5,221,127,234,5,220,127,1,6,219,127,24,6,218,127,47,6,217,127,70,6,215,127,94,6,214,127,118,6,213,127,142,6,212,127,166,6,210,127,191,6,209,127,216,6,208,127,241,6,206,127,10,7,205,127,35,7,204,127,61,7,202,127,87,7,201,127,113,7,199,127,140,7,197,127,166,7,196,127,193,7,194,127,220,7,192,127,247,7,191,127,19,8,189,127,47,8,187,127,75,8,185,127,103,8,183,127,132,8,182,127,160,8,180,127,189,8,178,127,218,8,175,127,248,8,173,127,22,9,171,127,52,9,169,127,82,9,167,127,112,9,165,127,143,9,162,127,174,9,160,127,205,9,157,127,236,9,155,127,12,10,152,127,44,10,150,127,76,10,147,127,108,10,145,127,141,10,142,127,174,10,139,127,207,10,136,127,240,10,133,127,18,11,130,127,51,11,127,127,85,11,124,127,120,11,121,127,154,11,118,127,189,11,115,127,224,11,111,127,4,12,108,127,39,12,105,127,75,12,101,127,111,12,97,127,147,12,94,127,184,12,90,127,221,12,86,127,2,13,83,127,39,13,79,127,77,13,75,127,115,13,71,127,153,13,66,127,191,13,62,127,230,13,58,127,12,14,54,127,52,14,49,127,91,14,45,127,131,14,40,127,170,14,36,127,210,14,31,127,251,14,26,127,35,15,21,127,76,15,16,127,117,15,11,127,159,15,6,127,200,15,1,127,242,15,251,126,28,16,246,126,71,16,240,126,113,16,235,126,156,16,229,126,200,16,223,126,243,16,218,126,31,17,212,126,74,17,206,126,119,17,199,126,163,17,193,126,208,17,187,126,253,17,180,126,42,18,174,126,87,18,167,126,133,18,160,126,179,18,154,126,225,18,147,126,15,19,140,126,62,19,132,126,109,19,125,126,156,19,118,126,203,19,110,126,251,19,103,126,43,20,95,126,91,20,87,126,140,20,79,126,188,20,71,126,237,20,63,126,30,21,55,126,80,21,46,126,129,21,38,126,179,21,29,126,229,21,20,126,24,22,11,126,74,22,2,126,125,22,249,125,176,22,240,125,228,22,230,125,23,23,221,125,75,23,211,125,127,23,201,125,179,23,191,125,232,23,181,125,29,24,171,125,82,24,161,125,135,24,150,125,188,24,140,125,242,24,129,125,40,25,118,125,94,25,107,125,149,25,96,125,203,25,84,125,2,26,73,125,57,26,61,125,113,26,49,125,168,26,38,125,224,26,25,125,24,27,13,125,80,27,1,125,137,27,244,124,194,27,232,124,251,27,219,124,52,28,206,124,109,28,193,124,167,28,179,124,225,28,166,124,27,29,152,124,85,29,138,124,143,29,124,124,202,29,110,124,5,30,96,124,64,30,81,124,123,30,67,124,183,30,52,124,243,30,37,124,47,31,22,124,107,31,6,124,167,31,247,123,228,31,231,123,33,32,215,123,94,32,199,123,155,32,183,123,216,32,166,123,22,33,150,123,84,33,133,123,146,33,116,123,208,33,99,123,14,34,82,123,77,34,64,123,140,34,46,123,203,34,28,123,10,35,10,123,73,35,248,122,137,35,230,122,200,35,211,122,8,36,192,122,72,36,173,122,137,36,154,122,201,36,134,122,10,37,115,122,75,37,95,122,140,37,75,122,205,37,54,122,14,38,34,122,80,38,13,122,145,38,248,121,211,38,227,121,21,39,206,121,87,39,184,121,154,39,163,121,220,39,141,121,31,40,119,121,98,40,96,121,165,40,74,121,232,40,51,121,43,41,28,121,111,41,5,121,178,41,237,120,246,41,214,120,58,42,190,120,126,42,166,120,194,42,141,120,7,43,117,120,75,43,92,120,144,43,67,120,212,43,42,120,25,44,16,120,94,44,247,119,164,44,221,119,233,44,195,119,46,45,168,119,116,45,142,119,186,45,115,119,255,45,88,119,69,46,61,119,139,46,33,119,210,46,6,119,24,47,234,118,94,47,205,118,165,47,177,118,236,47,148,118,50,48,119,118,121,48,90,118,192,48,61,118,7,49,31,118,78,49,1,118,150,49,227,117,221,49,197,117,37,50,166,117,108,50,136,117,180,50,105,117,252,50,73,117,68,51,42,117,140,51,10,117,212,51,234,116,28,52,202,116,100,52,169,116,172,52,136,116,245,52,103,116,61,53,70,116,133,53,36,116,206,53,3,116,23,54,225,115,95,54,190,115,168,54,156,115,241,54,121,115,58,55,86,115,131,55,51,115,204,55,15,115,21,56,236,114,94,56,200,114,167,56,163,114,241,56,127,114,58,57,90,114,131,57,53,114,205,57,16,114,22,58,234,113,96,58,196,113,169,58,158,113,243,58,120,113,60,59,81,113,134,59,43,113,208,59,4,113,25,60,220,112,99,60,181,112,173,60,141,112,247,60,101,112,64,61,60,112,138,61,20,112,212,61,235,111,30,62,194,111,104,62,152,111,177,62,111,111,251,62,69,111,69,63,27,111,143,63,240,110,217,63,198,110,35,64,155,110,109,64,112,110,182,64,68,110,0,65,25,110,74,65,237,109,148,65,193,109,222,65,148,109,40,66,104,109,113,66,59,109,187,66,13,109,5,67,224,108,79,67,178,108,152,67,132,108,226,67,86,108,44,68,40,108,117,68,249,107,191,68,202,107,8,69,155,107,82,69,107,107,155,69,60,107,229,69,12,107,46,70,219,106,119,70,171,106,193,70,122,106,10,71,73,106,83,71,24,106,156,71,231,105,229,71,181,105,46,72,131,105,119,72,81,105,192,72,30,105,9,73,235,104,82,73,185,104,155,73,133,104,227,73,82,104,44,74,30,104,116,74,234,103,189,74,182,103,5,75,130,103,77,75,77,103,149,75,24,103,222,75,227,102,38,76,174,102,109,76,120,102,181,76,66,102,253,76,12,102,69,77,214,101,140,77,159,101,212,77,104,101,27,78,49,101,98,78,250,100,169,78,195,100,240,78,139,100,55,79,83,100,126,79,27,100,197,79,226,99,11,80,170,99,82,80,113,99,152,80,56,99,223,80,254,98,37,81,197,98,107,81,139,98,177,81,81,98,246,81,23,98,60,82,220,97,129,82,162,97,199,82,103,97,12,83,44,97,81,83,241,96,150,83,181,96,219,83,121,96,32,84,61,96,100,84,1,96,169,84,197,95,237,84,136,95,49,85,75,95,117,85,14,95,185,85,209,94,253,85,148,94,64,86,86,94,131,86,24,94,199,86,218,93,10,87,156,93,77,87,94,93,143,87,31,93,210,87,224,92,20,88,161,92,86,88,98,92,152,88,35,92,218,88,227,91,28,89,164,91,94,89,100,91,159,89,36,91,224,89,227,90,33,90,163,90,98,90,152,47,138,66,145,68,55,113,207,251,192,181,165,219,181,233,91,194,86,57,241,17,241,89,164,130,63,146,213,94,28,171,152,170,7,216,1,91,131,18,190,133,49,36,195,125,12,85,116,93,190,114,254,177,222,128,167,6,220,155,116,241,155,193,193,105,155,228,134,71,190,239,198,157,193,15,204,161,12,36,111,44,233,45,170,132,116,74,220,169,176,92,218,136,249,118,82,81,62,152,109,198,49,168,200,39,3,176,199,127,89,191,243,11,224,198,71,145,167,213,81,99,202,6,103,41,41,20,133,10,183,39,56,33,27,46,252,109,44,77,19,13,56,83,84,115,10,101,187,10,106,118,46,201,194,129,133,44,114,146,161,232,191,162,75,102,26,168,112,139,75,194,163,81,108,199,25,232,146,209,36,6,153,214,133,53,14,244,112,160,106,16,22,193,164,25,8,108,55,30,76,119,72,39,181,188,176,52,179,12,28,57,74,170,216,78,79,202,156,91,243,111,46,104,238,130,143,116,111,99,165,120,20,120,200,132,8,2,199,140,250,255,190,144,235,108,80,164,247,163,249,190,242,120,113,198,6,14,22,30,38,46,54,62,4,12,20,28,36,44,52,60,2,10,18,26,34,42,50,58,0,8,16,24,32,40,48,56,7,15,23,31,39,47,55,63,5,13,21,29,37,45,53,61,3,11,19,27,35,43,51,59,1,9,17,25,33,41,49,57,62,180,228,51,9,145,243,51,139,178,1,52,60,32,10,52,35,26,19,52,96,169,28,52,167,215,38,52,75,175,49,52,80,59,61,52,112,135,73,52,35,160,86,52,184,146,100,52,85,109,115,52,136,159,129,52,252,11,138,52,147,4,147,52,105,146,156,52,50,191,166,52,63,149,177,52,147,31,189,52,228,105,201,52,173,128,214,52,54,113,228,52,166,73,243,52,136,140,1,53,192,247,9,53,6,239,18,53,118,123,28,53,192,166,38,53,55,123,49,53,218,3,61,53,94,76,73,53,59,97,86,53,185,79,100,53,252,37,115,53,138,121,129,53,134,227,137,53,124,217,146,53,133,100,156,53,82,142,166,53,51,97,177,53,37,232,188,53,220,46,201,53,206,65,214,53,65,46,228,53,87,2,243,53,143,102,1,54,79,207,9,54,245,195,18,54,152,77,28,54,232,117,38,54,50,71,49,54,116,204,60,54,94,17,73,54,101,34,86,54,206,12,100,54,184,222,114,54,151,83,129,54,28,187,137,54,114,174,146,54,175,54,156,54,129,93,166,54,53,45,177,54,199,176,188,54,228,243,200,54,1,3,214,54,96,235,227,54,30,187,242,54,162,64,1,55,235,166,9,55,241,152,18,55,201,31,28,55,30,69,38,55,61,19,49,55,30,149,60,55,111,214,72,55,162,227,85,55,247,201,99,55,137,151,114,55,175,45,129,55,190,146,137,55,116,131,146,55,230,8,156,55,190,44,166,55,71,249,176,55,121,121,188,55,254,184,200,55,71,196,213,55,146,168,227,55,248,115,242,55,192,26,1,56,147,126,9,56,249,109,18,56,6,242,27,56,98,20,38,56,86,223,48,56,216,93,60,56,146,155,72,56,242,164,85,56,51,135,99,56,110,80,114,56,211,7,129,56,107,106,137,56,130,88,146,56,42,219,155,56,9,252,165,56,104,197,176,56,59,66,188,56,41,126,200,56,160,133,213,56,217,101,227,56,232,44,242,56,233,244,0,57,70,86,9,57,14,67,18,57,81,196,27,57,181,227,37,57,127,171,48,57,162,38,60,57,197,96,72,57,83,102,85,57,131,68,99,57,104,9,114,57,1,226,128,57,36,66,137,57,157,45,146,57,123,173,155,57,99,203,165,57,153,145,176,57,13,11,188,57,102,67,200,57,11,71,213,57,50,35,227,57,237,229,241,57,29,207,0,58,5,46,9,58,48,24,18,58,169,150,27,58,21,179,37,58,183,119,48,58,124,239,59,58,10,38,72,58,199,39,85,58,230,1,99,58,120,194,113,58,59,188,128,58,233,25,137,58,198,2,146,58,219,127,155,58,203,154,165,58,216,93,176,58,239,211,187,58,179,8,200,58,136,8,213,58,159,224,226,58,7,159,241,58,92,169,0,59,208,5,9,59,94,237,17,59,15,105,27,59,132,130,37,59,253,67,48,59,103,184,59,59,97,235,71,59,77,233,84,59,93,191,98,59,156,123,113,59,127,150,128,59,186,241,136,59,249,215,145,59,71,82,155,59,65,106,165,59,39,42,176,59,226,156,187,59,18,206,199,59,23,202,212,59,32,158,226,59,53,88,241,59,166,131,0,60,167,221,8,60,152,194,17,60,130,59,27,60,1,82,37,60,84,16,48,60,97,129,59,60,200,176,71,60,229,170,84,60,232,124,98,60,212,52,113,60,207,112,128,60,150,201,136,60,58,173,145,60,192,36,155,60,197,57,165,60,133,246,175,60,229,101,187,60,130,147,199,60,185,139,212,60,180,91,226,60,121,17,241,60,251,93,0,61,137,181,8,61,223,151,17,61,2,14,27,61,141,33,37,61,185,220,47,61,109,74,59,61,64,118,71,61,145,108,84,61,133,58,98,61,34,238,112,61,42,75,128,61,127,161,136,61,136,130,145,61,72,247,154,61,88,9,165,61,242,194,175,61,248,46,187,61,3,89,199,61,109,77,212,61,92,25,226,61,209,202,240,61,91,56,0,62,119,141,8,62,51,109,17,62,144,224,26,62,39,241,36,62,46,169,47,62,135,19,59,62,202,59,71,62,77,46,84,62,55,248,97,62,132,167,112,62,143,37,128,62,115,121,136,62,226,87,145,62,220,201,154,62,249,216,164,62,109,143,175,62,27,248,186,62,149,30,199,62,51,15,212,62,23,215,225,62,61,132,240,62,198,18,0,63,114,101,8,63,147,66,17,63,43,179,26,63,206,192,36,63,177,117,47,63,178,220,58,63,101,1,71,63,29,240,83,63,251,181,97,63,251,96,112,63,0,0,128,63,62,180,228,51,9,145,243,51,139,178,1,52,60,32,10,52,35,26,19,52,96,169,28,52,167,215,38,52,75,175,49,52,80,59,61,52,112,135,73,52,35,160,86,52,184,146,100,52,85,109,115,52,136,159,129,52,252,11,138,52,147,4,147,52,105,146,156,52,50,191,166,52,63,149,177,52,147,31,189,52,228,105,201,52,173,128,214,52,54,113,228,52,166,73,243,52,136,140,1,53,192,247,9,53,6,239,18,53,118,123,28,53,192,166,38,53,55,123,49,53,218,3,61,53,94,76,73,53,59,97,86,53,185,79,100,53,252,37,115,53,138,121,129,53,134,227,137,53,124,217,146,53,133,100,156,53,82,142,166,53,51,97,177,53,37,232,188,53,220,46,201,53,206,65,214,53,65,46,228,53,87,2,243,53,143,102,1,54,79,207,9,54,245,195,18,54,152,77,28,54,232,117,38,54,50,71,49,54,116,204,60,54,94,17,73,54,101,34,86,54,206,12,100,54,184,222,114,54,151,83,129,54,28,187,137,54,114,174,146,54,175,54,156,54,129,93,166,54,53,45,177,54,199,176,188,54,228,243,200,54,1,3,214,54,96,235,227,54,30,187,242,54,162,64,1,55,235,166,9,55,241,152,18,55,201,31,28,55,30,69,38,55,61,19,49,55,30,149,60,55,111,214,72,55,162,227,85,55,247,201,99,55,137,151,114,55,175,45,129,55,190,146,137,55,116,131,146,55,230,8,156,55,190,44,166,55,71,249,176,55,121,121,188,55,254,184,200,55,71,196,213,55,146,168,227,55,248,115,242,55,192,26,1,56,147,126,9,56,249,109,18,56,6,242,27,56,98,20,38,56,86,223,48,56,216,93,60,56,146,155,72,56,242,164,85,56,51,135,99,56,110,80,114,56,211,7,129,56,107,106,137,56,130,88,146,56,42,219,155,56,9,252,165,56,104,197,176,56,59,66,188,56,41,126,200,56,160,133,213,56,217,101,227,56,232,44,242,56,233,244,0,57,70,86,9,57,14,67,18,57,81,196,27,57,181,227,37,57,127,171,48,57,162,38,60,57,197,96,72,57,83,102,85,57,131,68,99,57,104,9,114,57,1,226,128,57,36,66,137,57,157,45,146,57,123,173,155,57,99,203,165,57,153,145,176,57,13,11,188,57,102,67,200,57,11,71,213,57,50,35,227,57,237,229,241,57,29,207,0,58,5,46,9,58,48,24,18,58,169,150,27,58,21,179,37,58,183,119,48,58,124,239,59,58,10,38,72,58,199,39,85,58,230,1,99,58,120,194,113,58,59,188,128,58,233,25,137,58,198,2,146,58,219,127,155,58,203,154,165,58,216,93,176,58,239,211,187,58,179,8,200,58,136,8,213,58,159,224,226,58,7,159,241,58,92,169,0,59,208,5,9,59,94,237,17,59,15,105,27,59,132,130,37,59,253,67,48,59,103,184,59,59,97,235,71,59,77,233,84,59,93,191,98,59,156,123,113,59,127,150,128,59,186,241,136,59,249,215,145,59,71,82,155,59,65,106,165,59,39,42,176,59,226,156,187,59,18,206,199,59,23,202,212,59,32,158,226,59,53,88,241,59,166,131,0,60,167,221,8,60,152,194,17,60,130,59,27,60,1,82,37,60,84,16,48,60,97,129,59,60,200,176,71,60,229,170,84,60,232,124,98,60,212,52,113,60,207,112,128,60,150,201,136,60,58,173,145,60,192,36,155,60,197,57,165,60,133,246,175,60,229,101,187,60,130,147,199,60,185,139,212,60,180,91,226,60,121,17,241,60,251,93,0,61,137,181,8,61,223,151,17,61,2,14,27,61,141,33,37,61,185,220,47,61,109,74,59,61,64,118,71,61,145,108,84,61,133,58,98,61,34,238,112,61,42,75,128,61,127,161,136,61,136,130,145,61,72,247,154,61,88,9,165,61,242,194,175,61,248,46,187,61,3,89,199,61,109,77,212,61,92,25,226,61,209,202,240,61,91,56,0,62,119,141,8,62,51,109,17,62,144,224,26,62,39,241,36,62,46,169,47,62,135,19,59,62,202,59,71,62,77,46,84,62,55,248,97,62,132,167,112,62,143,37,128,62,115,121,136,62,226,87,145,62,220,201,154,62,249,216,164,62,109,143,175,62,27,248,186,62,149,30,199,62,51,15,212,62,23,215,225,62,61,132,240,62,198,18,0,63,114,101,8,63,147,66,17,63,43,179,26,63,206,192,36,63,177,117,47,63,178,220,58,63,101,1,71,63,29,240,83,63,251,181,97,63,251,96,112,63,0,0,128,63,129,200,139,128,129,229,226,132,1,74,107,141,1,146,218,153,1,87,196,169,129,222,157,188,0,213,194,209,64,229,122,232,0,0,0,0,192,217,156,26,64,242,15,52,255,139,60,75,127,94,31,95,127,235,217,110,127,56,188,121,127,126,76,127,1,210,241,129,129,20,38,145])
.concat([1,35,185,173,0,175,56,212,0,0,0,0,128,128,137,55,255,13,19,100,255,111,202,124,1,0,0,128,240,239,45,130,230,191,164,136,157,21,44,147,194,39,104,161,39,222,220,178,145,11,242,198,100,156,248,220,225,140,48,244,84,48,97,13,128,222,141,39,27,0,0,64,123,18,166,85,143,222,141,103,215,14,239,116,218,240,51,125,1,0,0,128,223,38,184,135,84,65,242,157,229,255,255,191,120,229,197,233,240,144,123,28,169,131,206,79,195,242,82,115,1,2,3,5,6,7,9,10,11,13,14,15,17,18,1,2,3,5,6,7,9,10,11,13,14,15,17,18,1,2,4,5,6,8,9,10,12,13,14,16,17,18,1,3,4,5,7,8,9,11,12,13,15,16,17,19,0,0,0,64,230,139,162,80,149,250,151,101,0,0,0,64,230,139,162,80,149,250,151,101,0,0,0,64,230,139,162,80,149,250,151,101,0,0,0,64,230,139,162,80,149,250,151,101,0,0,0,64,230,139,162,80,41,248,27,76,94,67,228,95,156,223,208,120,41,248,27,76,94,67,228,95,156,223,208,120,41,248,27,76,94,67,228,95,156,223,208,120,41,248,27,76,94,67,228,95,156,223,208,120,41,248,27,76,94,67,228,95,154,121,130,90,29,248,8,114,15,107,214,71,154,121,130,90,29,248,8,114,15,107,214,71,154,121,130,90,29,248,8,114,15,107,214,71,154,121,130,90,29,248,8,114,15,107,214,71,154,121,130,90,29,248,8,114,101,126,162,107,75,62,206,67,36,4,110,85,101,126,162,107,75,62,206,67,36,4,110,85,101,126,162,107,75,62,206,67,36,4,110,85,101,126,162,107,75,62,206,67,36,4,110,85,101,126,162,107,75,62,206,67,7,0,7,0,7,0,7,0,7,0,7,0,7,0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,13,0,13,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,9,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,14,0,0,64,28,76,130,90,162,107,0,16,233,26,65,45,28,76,74,253,203,50,112,221,237,50,57,195,15,51,163,174,49,51,172,159,83,51,82,150,117,51,148,146,151,51,112,148,185,51,228,155,219,51,237,168,253,51,139,187,31,52,187,211,65,52,124,241,99,52,204,20,134,52,168,61,168,52,16,108,202,52,1,160,236,52,121,217,14,53,119,24,49,53,250,92,83,53,254,166,117,53,131,246,151,53,135,75,186,53,7,166,220,53,3,6,255,53,120,107,33,54,101,214,67,54,199,70,102,54,158,188,136,54,232,55,171,54,162,184,205,54,203,62,240,54,98,202,18,55,100,91,53,55,209,241,87,55,165,141,122,55,224,46,157,55,128,213,191,55,132,129,226,55,232,50,5,56,173,233,39,56,208,165,74,56,79,103,109,56,42,46,144,56,93,250,178,56,233,203,213,56,202,162,248,56,255,126,27,57,136,96,62,57,97,71,97,57,138,51,132,57,1,37,167,57,196,27,202,57,209,23,237,57,40,25,16,58,198,31,51,58,170,43,86,58,210,60,121,58,61,83,156,58,233,110,191,58,213,143,226,58,255,181,5,59,101,225,40,59,6,18,76,59,224,71,111,59,242,130,146,59,58,195,181,59,183,8,217,59,104,83,252,59,73,163,31,60,91,248,66,60,156,82,102,60,9,178,137,60,162,22,173,60,101,128,208,60,81,239,243,60,100,99,23,61,156,220,58,61,248,90,94,61,119,222,129,61,23,103,165,61,214,244,200,61,180,135,236,61,174,31,16,62,195,188,51,62,242,94,87,62,58,6,123,62,152,178,158,62,12,100,194,62,147,26,230,62,45,214,9,63,216,150,45,63,147,92,81,63,91,39,117,63,49,247,152,63,17,204,188,63,252,165,224,63,239,132,4,64,234,104,40,64,233,81,76,64,238,63,112,64,245,50,148,64,253,42,184,64,6,40,220,64,13,42,0,65,17,49,36,65,18,61,72,65,13,78,108,65,1,100,144,65,237,126,180,65,207,158,216,65,167,195,252,65,114,237,32,66,48,28,69,66,222,79,105,66,125,136,141,66,9,198,177,66,131,8,214,66,232,79,250,66,55,156,30,67,112,237,66,67,144,67,103,67,150,158,139,67,130,254,175,67,81,99,212,67,3,205,248,67,149,59,29,68,8,175,65,68,88,39,102,68,135,164,138,68,144,38,175,68,117,173,211,68,51,57,248,68,200,201,28,69,53,95,65,69,119,249,101,69,141,152,138,69,118,60,175,69,49,229,211,69,188,146,248,69,22,69,29,70,62,252,65,70,50,184,102,70,242,120,139,70,124,62,176,70,207,8,213,70,233,215,249,70,202,171,30,71,112,132,67,71,217,97,104,71,6,68,141,71,243,42,178,71,161,22,215,71,14,7,252,71,57,252,32,72,32,246,69,72,195,244,106,72,32,248,143,72,53,0,181,72,3,13,218,72,135,30,255,72,192,52,36,73,173,79,73,73,77,111,110,73,159,147,147,73,162,188,184,73,84,234,221,73,180,28,3,74,193,83,40,74,122,143,77,74,222,207,114,74,235,20,152,74,161,94,189,74,253,172,226,74,0,0,8,75,168,87,45,75,243,179,82,75,225,20,120,75,112,122,157,75,159,228,194,75,110,83,232,75,219,198,13,76,228,62,51,76,137,187,88,76,201,60,126,76,162,194,163,76,20,77,201,76,28,220,238,76,187,111,20,77,239,7,58,77,182,164,95,77,17,70,133,77,253,235,170,77,121,150,208,77,133,69,246,77,31,249,27,78,70,177,65,78,250,109,103,78,56,47,141,78,1,245,178,78,82,191,216,78,43,142,254,78,138,97,36,79,112,57,74,79,217,21,112,79,198,246,149,79,54,220,187,79,38,198,225,79,151,180,7,80,135,167,45,80,245,158,83,80,225,154,121,80,72,155,159,80,42,160,197,80,133,169,235,80,90,183,17,81,166,201,55,81,105,224,93,81,162,251,131,81,79,27,170,81,112,63,208,81,3,104,246,81,8,149,28,82,125,198,66,82,98,252,104,82,181,54,143,82,117,117,181,82,162,184,219,82,58,0,2,83,60,76,40,83,168,156,78,83,124,241,116,83,183,74,155,83,88,168,193,83,95,10,232,83,202,112,14,84,152,219,52,84,200,74,91,84,90,190,129,84,75,54,168,84,156,178,206,84,76,51,245,84,88,184,27,85,193,65,66,85,133,207,104,85,163,97,143,85,27,248,181,85,235,146,220,85,18,50,3,86,144,213,41,86,99,125,80,86,138,41,119,86,5,218,157,86,211,142,196,86,242,71,235,86,98,5,18,87,33,199,56,87,47,141,95,87,138,87,134,87,51,38,173,87,39,249,211,87,102,208,250,87,239,171,33,88,192,139,72,88,218,111,111,88,59,88,150,88,226,68,189,88,206,53,228,88,255,42,11,89,115,36,50,89,41,34,89,89,32,36,128,89,89,42,167,89,208,52,206,89,135,67,245,89,123,86,28,90,172,109,67,90,25,137,106,90,193,168,145,90,163,204,184,90,190,244,223,90,17,33,7,91,156,81,46,91,94,134,85,91,84,191,124,91,128,252,163,91,223,61,203,91,113,131,242,91,53,205,25,92,42,27,65,92,79,109,104,92,164,195,143,92,39,30,183,92,215,124,222,92,180,223,5,93,189,70,45,93,240,177,84,93,78,33,124,93,212,148,163,93,131,12,203,93,89,136,242,93,86,8,26,94,120,140,65,94,190,20,105,94,41,161,144,94,183,49,184,94,103,198,223,94,56,95,7,95,41,252,46,95,58,157,86,95,106,66,126,95,183,235,165,95,33,153,205,95,168,74,245,95,74,0,29,96,6,186,68,96,220,119,108,96,202,57,148,96,208,255,187,96,238,201,227,96,33,152,11,97,106,106,51,97,200,64,91,97,57,27,131,97,189,249,170,97,83,220,210,97,250,194,250,97,178,173,34,98,121,156,74,98,79,143,114,98,51,134,154,98,35,129,194,98,32,128,234,98,41,131,18,99,60,138,58,99,89,149,98,99,127,164,138,99,173,183,178,99,226,206,218,99,30,234,2,100,96,9,43,100,166,44,83,100,241,83,123,100,63,127,163,100,143,174,203,100,226,225,243,100,53,25,28,101,136,84,68,101,219,147,108,101,44,215,148,101,123,30,189,101,199,105,229,101,15,185,13,102,83,12,54,102,145,99,94,102,201,190,134,102,250,29,175,102,35,129,215,102,68,232,255,102,91,83,40,103,104,194,80,103,107,53,121,103,98,172,161,103,76,39,202,103,41,166,242,103,249,40,27,104,185,175,67,104,106,58,108,104,11,201,148,104,155,91,189,104,25,242,229,104,132,140,14,105,221,42,55,105,33,205,95,105,80,115,136,105,106,29,177,105,109,203,217,105,90,125,2,106,47,51,43,106,235,236,83,106,141,170,124,106,22,108,165,106,132,49,206,106,214,250,246,106,12,200,31,107,37,153,72,107,32,110,113,107,253,70,154,107,187,35,195,107,88,4,236,107,213,232,20,108,48,209,61,108,105,189,102,108,128,173,143,108,114,161,184,108,64,153,225,108,233,148,10,109,109,148,51,109,202,151,92,109,255,158,133,109,13,170,174,109,241,184,215,109,173,203,0,110,62,226,41,110,164,252,82,110,223,26,124,110,238,60,165,110,207,98,206,110,131,140,247,110,9,186,32,111,95,235,73,111,133,32,115,111,123,89,156,111,64,150,197,111,211,214,238,111,51,27,24,112,96,99,65,112,89,175,106,112,29,255,147,112,171,82,189,112,4,170,230,112,37,5,16,113,15,100,57,113,193,198,98,113,58,45,140,113,122,151,181,113,127,5,223,113,73,119,8,114,216,236,49,114,42,102,91,114,63,227,132,114,23,100,174,114,176,232,215,114,10,113,1,115,36,253,42,115,254,140,84,115,151,32,126,115,238,183,167,115,3,83,209,115,213,241,250,115,98,148,36,116,172,58,78,116,176,228,119,116,110,146,161,116,230,67,203,116,23,249,244,116,1,178,30,117,161,110,72,117,249,46,114,117,7,243,155,117,202,186,197,117,66,134,239,117,111,85,25,118,79,40,67,118,226,254,108,118,40,217,150,118,31,183,192,118,199,152,234,118,32,126,20,119,40,103,62,119,223,83,104,119,69,68,146,119,88,56,188,119,25,48,230,119,133,43,16,120,158,42,58,120,98,45,100,120,209,51,142,120,233,61,184,120,171,75,226,120,21,93,12,121,40,114,54,121,225,138,96,121,66,167,138,121,72,199,180,121,244,234,222,121,69,18,9,122,58,61,51,122,210,107,93,122,14,158,135,122,236,211,177,122,107,13,220,122,140,74,6,123,77,139,48,123,174,207,90,123,174,23,133,123,76,99,175,123,137,178,217,123,99,5,4,124,218,91,46,124,236,181,88,124,155,19,131,124,228,116,173,124,199,217,215,124,68,66,2,125,90,174,44,125,9,30,87,125,79,145,129,125,45,8,172,125,161,130,214,125,172,0,1,126,75,130,43,126,128,7,86,126,72,144,128,126,165,28,171,126,148,172,213,126,21,64,0,127,41,215,42,127,205,113,85,127,3,16,128,127,200,177,170,127,29,87,213,127,28,76,57,76,85,76,114,76,142,76,171,76,199,76,227,76,0,77,28,77,57,77,85,77,113,77,142,77,170,77,199,77,227,77,255,77,27,78,56,78,84,78,112,78,141,78,169,78,197,78,225,78,253,78,26,79,54,79,82,79,110,79,138,79,166,79,194,79,223,79,251,79,23,80,51,80,79,80,107,80,135,80,163,80,191,80,219,80,247,80,19,81,47,81,75,81,102,81,130,81,158,81,186,81,214,81,242,81,14,82,42,82,69,82,97,82,125,82,153,82,180,82,208,82,236,82,8,83,35,83,63,83,91,83,119,83,146,83,174,83,202,83,229,83,1,84,28,84,56,84,84,84,111,84,139,84,166,84,194,84,221,84,249,84,20,85,48,85,75,85,103,85,130,85,158,85,185,85,213,85,240,85,11,86,39,86,66,86,94,86,121,86,148,86,176,86,203,86,230,86,2,87,29,87,56,87,83,87,111,87,138,87,165,87,192,87,220,87,247,87,18,88,45,88,72,88,99,88,127,88,154,88,181,88,208,88,235,88,6,89,33,89,60,89,87,89,114,89,141,89,168,89,195,89,223,89,250,89,20,90,47,90,74,90,101,90,128,90,155,90,182,90,209,90,236,90,7,91,34,91,61,91,88,91,114,91,141,91,168,91,195,91,222,91,248,91,19,92,46,92,73,92,100,92,126,92,153,92,180,92,207,92,233,92,4,93,31,93,57,93,84,93,111,93,137,93,164,93,191,93,217,93,244,93,14,94,41,94,68,94,94,94,121,94,147,94,174,94,200,94,227,94,253,94,24,95,50,95,77,95,103,95,130,95,156,95,183,95,209,95,236,95,6,96,32,96,59,96,85,96,112,96,138,96,164,96,191,96,217,96,243,96,14,97,40,97,66,97,93,97,119,97,145,97,171,97,198,97,224,97,250,97,20,98,47,98,73,98,99,98,125,98,151,98,178,98,204,98,230,98,0,99,26,99,52,99,78,99,105,99,131,99,157,99,183,99,209,99,235,99,5,100,31,100,57,100,83,100,109,100,135,100,161,100,187,100,213,100,239,100,9,101,35,101,61,101,87,101,113,101,139,101,165,101,191,101,217,101,243,101,12,102,38,102,64,102,90,102,116,102,142,102,168,102,193,102,219,102,245,102,15,103,41,103,66,103,92,103,118,103,144,103,169,103,195,103,221,103,247,103,16,104,42,104,68,104,94,104,119,104,145,104,171,104,196,104,222,104,247,104,17,105,43,105,68,105,94,105,120,105,145,105,171,105,196,105,222,105,247,105,17,106,43,106,68,106,94,106,119,106,145,106,170,106,196,106,221,106,247,106,16,107,42,107,67,107,93,107,118,107,143,107,169,107,194,107,220,107,245,107,15,108,40,108,65,108,91,108,116,108,141,108,167,108,192,108,217,108,243,108,12,109,37,109,63,109,88,109,113,109,139,109,164,109,189,109,214,109,240,109,9,110,34,110,59,110,85,110,110,110,135,110,160,110,185,110,211,110,236,110,5,111,30,111,55,111,80,111,105,111,131,111,156,111,181,111,206,111,231,111,0,112,25,112,50,112,75,112,100,112,126,112,151,112,176,112,201,112,226,112,251,112,20,113,45,113,70,113,95,113,120,113,145,113,170,113,195,113,220,113,245,113,14,114,38,114,63,114,88,114,113,114,138,114,163,114,188,114,213,114,238,114,7,115,31,115,56,115,81,115,106,115,131,115,156,115,180,115,205,115,230,115,255,115,24,116,49,116,73,116,98,116,123,116,148,116,172,116,197,116,222,116,247,116,15,117,40,117,65,117,89,117,114,117,139,117,164,117,188,117,213,117,238,117,6,118,31,118,56,118,80,118,105,118,129,118,154,118,179,118,203,118,228,118,252,118,21,119,46,119,70,119,95,119,119,119,144,119,168,119,193,119,218,119,242,119,11,120,35,120,60,120,84,120,109,120,133,120,158,120,182,120,207,120,231,120,255,120,24,121,48,121,73,121,97,121,122,121,146,121,170,121,195,121,219,121,244,121,12,122,36,122,61,122,85,122,110,122,134,122,158,122,183,122,207,122,231,122,0,123,24,123,48,123,73,123,97,123,121,123,145,123,170,123,194,123,218,123,242,123,11,124,35,124,59,124,83,124,108,124,132,124,156,124,180,124,205,124,229,124,253,124,21,125,45,125,70,125,94,125,118,125,142,125,166,125,190,125,214,125,239,125,7,126,31,126,55,126,79,126,103,126,127,126,151,126,175,126,200,126,224,126,248,126,16,127,40,127,64,127,88,127,112,127,136,127,160,127,184,127,208,127,232,127,18,18,18,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,18,19,18,17,17,16,17,16,16,16,16,15,15,14,14,14,14,14,14,13,13,12,12,12,11,12,11,10,10,10,9,9,8,8,8,7,6,6,5,4,3,1,4,4,5,6,6,7,7,8,8,9,9,10,10,10,11,11,11,11,12,12,13,13,13,14,14,16,15,16,15,18,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,0,0,0,0,0,0,0,6,0,1,0,5,0,3,0,6,0,6,0,6,0,8,0,7,0,9,0,8,0,10,0,9,0,10,0,10,0,11,0,10,0,11,0,10,0,12,0,11,0,12,0,11,0,13,0,12,0,13,0,5,0,3,0,4,0,4,0,4,0,6,0,5,0,7,0,6,0,8,0,7,0,8,0,7,0,9,0,8,0,10,0,8,0,10,0,9,0,10,0,10,0,11,0,10,0,12,0,11,0,12,0,6,0,6,0,4,0,6,0,5,0,7,0,5,0,8,0,6,0,8,0,6,0,9,0,7,0,10,0,8,0,10,0,8,0,10,0,9,0,11,0,9,0,12,0,10,0,12,0,10,0,12,0,6,0,8,0,5,0,7,0,5,0,8,0,5,0,9,0,6,0,9,0,7,0,10,0,7,0,10,0,8,0,11,0,8,0,11,0,9,0,11,0,9,0,12,0,10,0,12,0,10,0,13,0,7,0,9,0,6,0,8,0,6,0,9,0,6,0,9,0,6,0,10,0,7,0,10,0,7,0,11,0,8,0,11,0,8,0,11,0,9,0,12,0,9,0,12,0,10,0,12,0,10,0,13,0,8,0,10,0,7,0,9,0,6,0,9,0,7,0,10,0,7,0,11,0,7,0,11,0,8,0,11,0,8,0,12,0,8,0,11,0,9,0,12,0,10,0,12,0,10,0,13,0,11,0,13,0,9,0,11,0,7,0,9,0,7,0,10,0,7,0,11,0,7,0,11,0,8,0,11,0,8,0,12,0,9,0,12,0,9,0,12,0,9,0,12,0,10,0,13,0,10,0,13,0,11,0,13,0,9,0,11,0,8,0,10,0,8,0,10,0,8,0,11,0,8,0,11,0,8,0,12,0,9,0,12,0,9,0,13,0,9,0,13,0,10,0,13,0,10,0,13,0,11,0,13,0,11,0,13,0,9,0,11,0,8,0,10,0,8,0,10,0,8,0,11,0,8,0,11,0,8,0,11,0,9,0,12,0,9,0,12,0,10,0,13,0,10,0,13,0,10,0,14,0,11,0,13,0,11,0,14,0,10,0,11,0,9,0,10,0,9,0,11,0,9,0,11,0,9,0,12,0,9,0,12,0,9,0,12,0,10,0,12,0,10,0,13,0,10,0,13,0,11,0,14,0,11,0,14,0,12,0,14,0,10,0,12,0,9,0,11,0,9,0,11,0,9,0,12,0,9,0,12,0,10,0,12,0,10,0,13,0,10,0,13,0,10,0,13,0,11,0,14,0,11,0,14,0,11,0,14,0,12,0,15,0,11,0,12,0,10,0,11,0,9,0,12,0,10,0,12,0,10,0,12,0,10,0,13,0,10,0,13,0,10,0,13,0,11,0,13,0,11,0,14,0,11,0,14,0,11,0,15,0,12,0,15,0,11,0,13,0,10,0,12,0,10,0,12,0,10,0,12,0,10,0,13,0,10,0,13,0,10,0,13,0,11,0,13,0,11,0,14,0,12,0,14,0,12,0,14,0,12,0,14,0,12,0,15,0,0,0,0,0,5,0,1,0,4,0,3,0,5,0,6,0,6,0,7,0,7,0,8,0,8,0,9,0,9,0,10,0,10,0,11,0,4,0,3,0,3,0,4,0,4,0,6,0,5,0,7,0,6,0,8,0,7,0,8,0,7,0,9,0,8,0,9,0,5,0,6,0,4,0,6,0,4,0,7,0,5,0,8,0,6,0,8,0,7,0,9,0,7,0,9,0,8,0,10,0,6,0,7,0,5,0,7,0,5,0,8,0,6,0,8,0,6,0,9,0,7,0,9,0,8,0,10,0,8,0,10,0,7,0,8,0,6,0,8,0,6,0,9,0,6,0,9,0,7,0,10,0,7,0,10,0,8,0,10,0,9,0,11,0,8,0,9,0,7,0,8,0,6,0,9,0,7,0,9,0,7,0,10,0,8,0,10,0,8,0,11,0,10,0,11,0,9,0,10,0,7,0,9,0,7,0,9,0,8,0,10,0,8,0,10,0,8,0,11,0,9,0,12,0,9,0,12,0,10,0,11,0,8,0,10,0,8,0,10,0,8,0,10,0,9,0,11,0,9,0,11,0,9,0,12,0,10,0,12,0,11,0,13,0,10,0,12,0,9,0,11,0,9,0,11,0,9,0,10,0,9,0,11,0,9,0,11,0,10,0,12,0,11,0,13,0,10,0,12,0,9,0,11,0,8,0,10,0,7,0,9,0,7,0,8,0,7,0,9,0,8,0,10,0,9,0,11,0,10,0,12,0,9,0,12,0,8,0,10,0,6,0,9,0,6,0,8,0,6,0,7,0,6,0,8,0,6,0,9,0,8,0,10,0,9,0,11,0,9,0,11,0,7,0,9,0,6,0,8,0,4,0,5,0,4,0,4,0,4,0,5,0,6,0,8,0,7,0,9,0,9,0,11,0,9,0,10,0,7,0,8,0,6,0,7,0,4,0,4,0,4,0,1,0,4,0,4,0,6,0,7,0,7,0,8,0,9,0,11,0,9,0,11,0,7,0,9,0,6,0,8,0,4,0,5,0,4,0,4,0,4,0,5,0,6,0,8,0,7,0,9,0,9,0,11,0,9,0,11,0,8,0,10,0,6,0,9,0,6,0,8,0,6,0,7,0,6,0,8,0,6,0,9,0,8,0,10,0,9,0,11,0,10,0,12,0,9,0,11,0,8,0,10,0,7,0,9,0,7,0,8,0,7,0,9,0,7,0,10,0,8,0,11,0,10,0,12,0,11,0,13,0,10,0,12,0,9,0,12,0,9,0,11,0,9,0,10,0,9,0,10,0,9,0,11,0,10,0,12,0,11,0,13,0,0,0,0,0,4,0,1,0,5,0,4,0,8,0,8,0,5,0,4,0,4,0,5,0,8,0,8,0,9,0,9,0,8,0,9,0,11,0,10,0,5,0,4,0,5,0,6,0,8,0,9,0,5,0,6,0,4,0,6,0,8,0,9,0,8,0,9,0,7,0,9,0,10,0,10,0,9,0,9,0,8,0,10,0,11,0,13,0,8,0,9,0,8,0,9,0,10,0,11,0,11,0,11,0,10,0,10,0,11,0,12,0,4,0,4,0,5,0,6,0,8,0,10,0,4,0,6,0,4,0,7,0,8,0,10,0,8,0,10,0,8,0,10,0,10,0,12,0,4,0,5,0,4,0,7,0,8,0,11,0,4,0,6,0,4,0,7,0,7,0,10,0,8,0,9,0,7,0,9,0,9,0,11,0,8,0,9,0,8,0,10,0,10,0,13,0,7,0,8,0,7,0,9,0,9,0,12,0,10,0,10,0,9,0,11,0,10,0,12,0,8,0,8,0,8,0,10,0,11,0,15,0,8,0,9,0,7,0,11,0,10,0,15,0,11,0,13,0,10,0,14,0,12,0,16,0,8,0,8,0,7,0,10,0,10,0,14,0,7,0,9,0,7,0,10,0,9,0,14,0,10,0,12,0,9,0,12,0,11,0,15,0,11,0,11,0,10,0,12,0,12,0,16,0,10,0,10,0,9,0,11,0,11,0,15,0,11,0,12,0,10,0,12,0,11,0,15,0,0,0,0,0,9,0,11,0,7,0,9,0,9,0,11,0,8,0,10,0,6,0,7,0,8,0,10,0,9,0,11,0,8,0,9,0,9,0,11,0,8,0,10,0,6,0,7,0,7,0,10,0,6,0,7,0,5,0,5,0,6,0,7,0,7,0,9,0,6,0,7,0,8,0,10,0,9,0,11,0,7,0,9,0,8,0,11,0,8,0,9,0,6,0,7,0,8,0,9,0,9,0,11,0,7,0,9,0,9,0,11,0,8,0,9,0,6,0,7,0,7,0,9,0,6,0,7,0,5,0,5,0,6,0,7,0,7,0,9,0,6,0,7,0,8,0,9,0,6,0,7,0,5,0,5,0,6,0,7,0,5,0,5,0,3,0,1,0,5,0,5,0,6,0,7,0,5,0,5,0,6,0,7,0,8,0,9,0,6,0,7,0,7,0,9,0,6,0,7,0,5,0,5,0,6,0,7,0,8,0,9,0,6,0,7,0,8,0,9,0,9,0,11,0,7,0,9,0,9,0,11,0,8,0,9,0,6,0,7,0,8,0,9,0,8,0,11,0,7,0,9,0,9,0,11,0,8,0,10,0,6,0,7,0,7,0,9,0,6,0,7,0,4,0,5,0,6,0,7,0,8,0,9,0,6,0,7,0,7,0,10,0,9,0,11,0,7,0,9,0,9,0,11,0,7,0,10,0,6,0,7,0,8,0,9,0,9,0,11,0,7,0,9,0,9,0,11,0,0,0,0,0,4,5,6,7,8,8,9,10,10,10,11,11,12,11,12,12,10,5,4,5,6,7,7,8,8,9,9,9,10,10,10,10,11,8,6,5,5,6,7,7,8,8,8,9,9,9,10,10,10,10,8,7,6,6,6,7,7,8,8,8,9,9,9,10,10,10,10,8,8,7,7,7,7,8,8,8,8,9,9,9,10,10,10,10,8,8,7,7,7,7,8,8,8,9,9,9,9,10,10,10,10,8,9,8,8,8,8,8,8,8,9,9,9,10,10,10,10,10,8,9,8,8,8,8,8,8,9,9,9,10,10,10,10,10,10,8,10,9,8,8,9,9,9,9,9,10,10,10,10,10,10,11,8,10,9,9,9,9,9,9,9,10,10,10,10,10,10,11,11,8,11,9,9,9,9,9,9,10,10,10,10,10,11,10,11,11,8,11,10,9,9,10,9,10,10,10,10,10,11,11,11,11,11,8,11,10,10,10,10,10,10,10,10,10,10,11,11,11,11,11,9,11,10,9,9,10,10,10,10,10,10,11,11,11,11,11,11,9,11,10,10,10,10,10,10,10,10,10,11,11,11,11,11,11,9,12,10,10,10,10,10,10,10,11,11,11,11,11,11,12,12,9,9,8,8,8,8,8,8,8,8,8,8,8,8,8,8,9,5,0,0,0,0,0,0,0,232,255,3,0,230,255,3,0,231,255,3,0,229,255,3,0,245,255,7,0,241,255,7,0,237,255,7,0,246,255,7,0,238,255,7,0,239,255,7,0,240,255,7,0,252,255,7,0,253,255,7,0,255,255,7,0,254,255,7,0,247,255,7,0,248,255,7,0,251,255,7,0,249,255,7,0,228,255,3,0,250,255,7,0,227,255,3,0,239,255,1,0,240,255,1,0,245,255,0,0,238,255,1,0,242,255,0,0,243,255,0,0,244,255,0,0,241,255,0,0,246,127,0,0,247,127,0,0,249,63,0,0,245,63,0,0,247,63,0,0,243,63,0,0,246,63,0,0,242,63,0,0,247,31,0,0,245,31,0,0,249,15,0,0,247,15,0,0,246,15,0,0,249,7,0,0,244,15,0,0,248,7,0,0,249,3,0,0,247,3,0,0,245,3,0,0,248,1,0,0,247,1,0,0,250,0,0,0,248,0,0,0,246,0,0,0,121,0,0,0,58,0,0,0,56,0,0,0,26,0,0,0,11,0,0,0,4,0,0,0,0,0,0,0,10,0,0,0,12,0,0,0,27,0,0,0,57,0,0,0,59,0,0,0,120,0,0,0,122,0,0,0,247,0,0,0,249,0,0,0,246,1,0,0,249,1,0,0,244,3,0,0,246,3,0,0,248,3,0,0,245,7,0,0,244,7,0,0,246,7,0,0,247,7,0,0,245,15,0,0,248,15,0,0,244,31,0,0,246,31,0,0,248,31,0,0,248,63,0,0,244,63,0,0,240,255,0,0,244,127,0,0,246,255,0,0,245,127,0,0,226,255,3,0,217,255,7,0,218,255,7,0,219,255,7,0,220,255,7,0,221,255,7,0,222,255,7,0,216,255,7,0,210,255,7,0,211,255,7,0,212,255,7,0,213,255,7,0,214,255,7,0,242,255,7,0,223,255,7,0,231,255,7,0,232,255,7,0,233,255,7,0,234,255,7,0,235,255,7,0,230,255,7,0,224,255,7,0,225,255,7,0,226,255,7,0,227,255,7,0,228,255,7,0,229,255,7,0,215,255,7,0,236,255,7,0,244,255,7,0,243,255,7,0,0,0,0,0,0,0,5,0,55,0,231,0,222,1,206,3,217,3,200,7,205,7,200,15,221,15,228,31,236,31,4,0,12,0,53,0,114,0,234,0,237,0,226,1,209,3,211,3,224,3,216,7,207,15,213,15,54,0,52,0,113,0,232,0,236,0,225,1,207,3,221,3,219,3,208,7,199,15,212,15,228,15,230,0,112,0,233,0,221,1,227,1,210,3,220,3,204,7,202,7,222,7,216,15,234,15,219,31,223,1,235,0,220,1,230,1,213,3,222,3,203,7,221,7,220,7,205,15,226,15,231,15,225,31,208,3,224,1,228,1,214,3,197,7,209,7,219,7,210,15,224,7,217,15,235,15,227,31,233,31,196,7,229,1,215,3,198,7,207,7,218,7,203,15,218,15,227,15,233,15,230,31,243,31,247,31,211,7,216,3,225,3,212,7,217,7,211,15,222,15,221,31,217,31,226,31,234,31,241,31,246,31,210,7,212,3,218,3,199,7,215,7,226,7,206,15,219,15,216,31,238,31,240,63,244,31,242,63,225,7,223,3,201,7,214,7,202,15,208,15,229,15,230,15,235,31,239,31,243,63,244,63,245,63,224,15,206,7,213,7,198,15,209,15,225,15,224,31,232,31,240,31,241,63,248,63,246,63,252,127,232,15,223,7,201,15,215,15,220,15,220,31,223,31,237,31,245,31,249,63,251,63,253,127,254,127,231,31,204,15,214,15,223,15,222,31,218,31,229,31,242,31,250,63,247,63,252,63,253,63,255,127,0,0,0,0,0,0,14,0,5,0,16,0,48,0,111,0,241,0,250,1,254,3,3,0,0,0,4,0,18,0,44,0,106,0,117,0,248,0,15,0,2,0,6,0,20,0,46,0,105,0,114,0,245,0,47,0,17,0,19,0,42,0,50,0,108,0,236,0,250,0,113,0,43,0,45,0,49,0,109,0,112,0,242,0,249,1,239,0,104,0,51,0,107,0,110,0,238,0,249,0,252,3,248,1,116,0,115,0,237,0,240,0,246,0,246,1,253,1,253,3,243,0,244,0,247,0,247,1,251,1,252,1,255,3,0,0,5,0,55,0,116,0,242,0,235,1,237,3,247,7,4,0,12,0,53,0,113,0,236,0,238,0,238,1,245,1,54,0,52,0,114,0,234,0,241,0,233,1,243,1,245,3,115,0,112,0,235,0,240,0,241,1,240,1,236,3,250,3,243,0,237,0,232,1,239,1,239,3,241,3,249,3,251,7,237,1,239,0,234,1,242,1,243,3,248,3,249,7,252,7,238,3,236,1,244,1,244,3,247,3,248,7,253,15,254,15,246,7,240,3,242,3,246,3,250,7,253,7,252,15,255,15,254,7,253,3,241,1,235,1,244,1,234,1,240,1,252,3,253,7,246,3,229,1,234,0,108,0,113,0,104,0,240,0,230,1,247,3,243,1,239,0,50,0,39,0,40,0,38,0,49,0,235,0,247,1,232,1,111,0,46,0,8,0,4,0,6,0,41,0,107,0,238,1,239,1,114,0,45,0,2,0,0,0,3,0,47,0,115,0,250,1,231,1,110,0,43,0,7,0,1,0,5,0,44,0,109,0,236,1,249,1,238,0,48,0,36,0,42,0,37,0,51,0,236,0,242,1,248,3,228,1,237,0,106,0,112,0,105,0,116,0,241,0,250,3,255,7,249,3,246,1,237,1,248,1,233,1,245,1,251,3,252,7,0,0,0,0,0,0,255,31,247,15,244,7,232,7,241,3,238,7,249,7,248,15,253,31,253,15,241,7,232,3,232,1,240,0,236,1,238,3,242,7,250,15,244,15,239,3,242,1,232,0,112,0,236,0,240,1,234,3,243,7,235,7,235,1,234,0,26,0,8,0,25,0,238,0,239,1,237,7,240,3,242,0,115,0,11,0,0,0,10,0,113,0,243,0,233,7,239,7,238,1,239,0,24,0,9,0,27,0,235,0,233,1,236,7,246,7,235,3,243,1,237,0,114,0,233,0,241,1,237,3,247,7,246,15,240,7,233,3,237,1,241,0,234,1,236,3,248,7,249,15,252,31,252,15,245,15,234,7,243,3,242,3,245,7,251,15,254,31,0,0,0,0,0,0,7,0,22,0,246,0,24,0,8,0,239,0,239,1,243,0,248,7,25,0,23,0,237,0,21,0,1,0,226,0,240,0,112,0,240,3,238,1,241,0,250,7,238,0,228,0,242,3,246,7,239,3,253,7,5,0,20,0,242,0,9,0,4,0,229,0,244,0,232,0,244,3,6,0,2,0,231,0,3,0,0,0,107,0,227,0,105,0,243,1,235,0,230,0,246,3,110,0,106,0,244,1,236,3,240,1,249,3,245,0,236,0,251,7,234,0,111,0,247,3,249,7,243,3,255,15,233,0,109,0,248,3,108,0,104,0,245,1,238,3,242,1,244,7,247,7,241,3,254,15,237,3,241,1,245,7,254,7,245,3,252,7,0,0,0,0,0,0,0,0,9,0,239,0,11,0,25,0,240,0,235,1,230,1,242,3,10,0,53,0,239,1,52,0,55,0,233,1,237,1,231,1,243,3,238,1,237,3,250,31,236,1,242,1,249,7,248,7,248,3,248,15,8,0,56,0,246,3,54,0,117,0,241,3,235,3,236,3,244,15,24,0,118,0,244,7,57,0,116,0,239,3,243,1,244,1,246,7,232,1,234,3,252,31,242,0,241,1,251,15,245,3,243,7,252,15,238,0,247,3,254,127,240,1,245,7,253,127,251,31,250,63,255,255,241,0,240,3,252,63,234,1,238,3,251,63,246,15,250,15,252,127,242,7,245,15,254,255,244,3,247,7,251,127,247,15,249,15,250,127,0,0,0,0,0,0,243,1,111,0,253,1,235,0,35,0,234,0,247,1,232,0,250,1,242,0,45,0,112,0,32,0,6,0,43,0,110,0,40,0,233,0,249,1,102,0,248,0,231,0,27,0,241,0,244,1,107,0,245,1,236,0,42,0,108,0,44,0,10,0,39,0,103,0,26,0,245,0,36,0,8,0,31,0,9,0,0,0,7,0,29,0,11,0,48,0,239,0,28,0,100,0,30,0,12,0,41,0,243,0,47,0,240,0,252,1,113,0,242,1,244,0,33,0,230,0,247,0,104,0,248,1,238,0,34,0,101,0,49,0,2,0,38,0,237,0,37,0,106,0,251,1,114,0,254,1,105,0,46,0,246,0,255,1,109,0,246,1,0,0,0,0,0,0,0,0,6,0,25,0,61,0,156,0,198,0,167,1,144,3,194,3,223,3,230,7,243,7,251,15,236,7,250,15,254,15,142,3,5,0,1,0,8,0,20,0,55,0,66,0,146,0,175,0,145,1,165,1,181,1,158,3,192,3,162,3,205,3,214,7,174,0,23,0,7,0,9,0,24,0,57,0,64,0,142,0,163,0,184,0,153,1,172,1,193,1,177,3,150,3,190,3,202,3,157,0,60,0,21,0,22,0,26,0,59,0,68,0,145,0,165,0,190,0,150,1,174,1,185,1,161,3,145,3,165,3,213,3,148,0,154,0,54,0,56,0,58,0,65,0,140,0,155,0,176,0,195,0,158,1,171,1,188,1,159,3,143,3,169,3,207,3,147,0,191,0,62,0,63,0,67,0,69,0,158,0,167,0,185,0,148,1,162,1,186,1,195,1,166,3,167,3,187,3,212,3,159,0,160,1,143,0,141,0,144,0,152,0,166,0,182,0,196,0,159,1,175,1,191,1,153,3,191,3,180,3,201,3,231,3,168,0,182,1,171,0,164,0,170,0,178,0,194,0,197,0,152,1,164,1,184,1,140,3,164,3,196,3,198,3,221,3,232,3,173,0,175,3,146,1,189,0,188,0,142,1,151,1,154,1,163,1,177,1,141,3,152,3,183,3,211,3,209,3,219,3,221,7,180,0,222,3,169,1,155,1,156,1,161,1,170,1,173,1,179,1,139,3,178,3,184,3,206,3,225,3,224,3,210,7,229,7,183,0,227,7,187,1,168,1,166,1,176,1,178,1,183,1,155,3,154,3,186,3,181,3,214,3,215,7,228,3,216,7,234,7,186,0,232,7,160,3,189,1,180,1,138,3,196,1,146,3,170,3,176,3,188,3,215,3,212,7,220,7,219,7,213,7,240,7,193,0,251,7,200,3,163,3,149,3,157,3,172,3,174,3,197,3,216,3,226,3,230,3,228,7,231,7,224,7,233,7,247,7,144,1,242,7,147,3,190,1,192,1,148,3,151,3,173,3,195,3,193,3,210,3,218,7,217,7,223,7,235,7,244,7,250,7,149,1,248,7,189,3,156,3,171,3,168,3,179,3,185,3,208,3,227,3,229,3,226,7,222,7,237,7,241,7,249,7,252,7,147,1,253,15,220,3,182,3,199,3,204,3,203,3,217,3,218,3,211,7,225,7,238,7,239,7,245,7,246,7,252,15,255,15,157,1,194,1,181,0,161,0,150,0,151,0,149,0,153,0,160,0,162,0,172,0,169,0,177,0,179,0,187,0,192,0,143,1,4,0,24,0,46,0,0,0,90,0,165,0,248,0,183,0,148,0,249,0,77,0,33,0,43,0,79,0,123,0,188,0,70,0,21,0,66,0,55,0,120,0,13,0,104,0,95,0,13,0,94,0,90,0,190,0,99,0,126,0,31,0,146,0,26,0,171,0,50,0,230,0,55,0,0,0,88,0,11,0,90,0,225,0,93,0,41,0,23,0,126,0,105,0,170,0,84,0,41,0,50,0,65,0,70,0,234,0,52,0,234,0,17,0,27,0,169,0,148,0,226,0,49,0,208,0,229,0,7,0,112,0,105,0,62,0,33,0,0,0,0,0,0,0,34,0,8,0,29,0,38,0,95,0,211,0,207,1,208,3,215,3,237,3,240,7,246,7,253,15,7,0,0,0,1,0,9,0,32,0,84,0,96,0,213,0,220,0,212,1,205,3,222,3,231,7,28,0,2,0,6,0,12,0,30,0,40,0,91,0,205,0,217,0,206,1,220,1,217,3,241,3,37,0,11,0,10,0,13,0,36,0,87,0,97,0,204,0,221,0,204,1,222,1,211,3,231,3,93,0,33,0,31,0,35,0,39,0,89,0,100,0,216,0,223,0,210,1,226,1,221,3,238,3,209,0,85,0,41,0,86,0,88,0,98,0,206,0,224,0,226,0,218,1,212,3,227,3,235,7,201,1,94,0,90,0,92,0,99,0,202,0,218,0,199,1,202,1,224,1,219,3,232,3,236,7,227,1,210,0,203,0,208,0,215,0,219,0,198,1,213,1,216,1,202,3,218,3,234,7,241,7,225,1,212,0,207,0,214,0,222,0,225,0,208,1,214,1,209,3,213,3,242,3,238,7,251,7,233,3,205,1,200,1,203,1,209,1,215,1,223,1,207,3,224,3,239,3,230,7,248,7,250,15,235,3,221,1,211,1,217,1,219,1,210,3,204,3,220,3,234,3,237,7,243,7,249,7,249,15,242,7,206,3,228,1,203,3,216,3,214,3,226,3,229,3,232,7,244,7,245,7,247,7,251,15,250,7,236,3,223,3,225,3,228,3,230,3,240,3,233,7,239,7,248,15,254,15,252,15,255,15,0,0,0,0,0,0,248,7,241,1,253,7,245,3,104,0,240,3,247,7,236,1,245,7,241,3,114,0,244,3,116,0,17,0,118,0,235,1,108,0,246,3,252,7,225,1,241,7,240,1,97,0,246,1,242,7,234,1,251,7,242,1,105,0,237,1,119,0,23,0,111,0,230,1,100,0,229,1,103,0,21,0,98,0,18,0,0,0,20,0,101,0,22,0,109,0,233,1,99,0,228,1,107,0,19,0,113,0,227,1,112,0,243,1,254,7,231,1,243,7,239,1,96,0,238,1,240,7,226,1,250,7,243,3,106,0,232,1,117,0,16,0,115,0,244,1,110,0,247,3,246,7,224,1,249,7,242,3,102,0,245,1,255,7,247,1,244,7,0,0,0,0,0,0,198,250,176,250,153,250,131,250,108,250,84,250,61,250,37,250,13,250,244,249,220,249,195,249,170,249,144,249,119,249,93,249,67,249,41,249,15,249,244,248,217,248,190,248,162,248,135,248,107,248,79,248,50,248,22,248,249,247,220,247,191,247,162,247,132,247,103,247,73,247,44,247,14,247,240,246,210,246,179,246,149,246,119,246,88,246,58,246,28,246,254,245,225,245,196,245,167,245,139,245,111,245,83,245,57,245,31,245,5,245,236,244,212,244,189,244,167,244,145,244,124,244,104,244,86,244,67,244,49,244,31,244,12,244,248,243,226,243,204,243,181,243,157,243,132,243,106,243,79,243,51,243,22,243,248,242,217,242,186,242,153,242,120,242,86,242,51,242,15,242,235,241,197,241,159,241,121,241,81,241,41,241,255,240,213,240,170,240,126,240,82,240,36,240,245,239,197,239,148,239,99,239,49,239,255,238,205,238,154,238,103,238,52,238,0,238,204,237,150,237,96,237,41,237,241,236,185,236,128,236,70,236,12,236,210,235,153,235,94,235,36,235,233,234,173,234,112,234,50,234,243,233,179,233,113,233,47,233,235,232,167,232,99,232,31,232,219,231,152,231,85,231,19,231,210,230,144,230,79,230,12,230,201,229,132,229,61,229,245,228,170,228,95,228,18,228,197,227,119,227,42,227,222,226,147,226,73,226,0,226,185,225,114,225,45,225,232,224,163,224,93,224,24,224,209,223,138,223,66,223,249,222,175,222,100,222,24,222,204,221,128,221,51,221,230,220,153,220,76,220,0,220,180,219,105,219,30,219,213,218,140,218,68,218,252,217,181,217,111,217,41,217,227,216,158,216,88,216,19,216,206,215,136,215,67,215,253,214,183,214,113,214,43,214,228,213,157,213,86,213,16,213,202,212,132,212,64,212,253,211,188,211,123,211,60,211,253,210,191,210,129,210,67,210,5,210,200,209,138,209,76,209,14,209,208,208,146,208,85,208,23,208,219,207,160,207,101,207,43,207,243,206,187,206,133,206,79,206,26,206,228,205,175,205,121,205,68,205,15,205,218,204,166,204,114,204,63,204,13,204,220,203,172,203,124,203,77,203,30,203,239,202,193,202,147,202,102,202,57,202,13,202,225,201,182,201,140,201,98,201,56,201,10,201,225,200,183,200,141,200,99,200,56,200,14,200,227,199,184,199,140,199,97,199,53,199,10,199,222,198,179,198,135,198,91,198,46,198,2,198,213,197,168,197,122,197,78,197,33,197,245,196,201,196,158,196,115,196,72,196,30,196,243,195,200,195,156,195,113,195,69,195,25,195,238,194,195,194,152,194,110,194,68,194,27,194,243,193,203,193,164,193,125,193,87,193,50,193,13,193,232,192,195,192,158,192,121,192,84,192,48,192,11,192,231,191,195,191,161,191,127,191,94,191,61,191,30,191,255,190,225,190,196,190,168,190,140,190,113,190,87,190,61,190,35,190,11,190,242,189,218,189,195,189,171,189,149,189,126,189,104,189,83,189,62,189,41,189,22,189,2,189,240,188,222,188,205,188,188,188,173,188,159,188,145,188,132,188,120,188,108,188,98,188,88,188,78,188,69,188,60,188,51,188,43,188,34,188,26,188,18,188,11,188,3,188,252,187,245,187,239,187,234,187,229,187,224,187,220,187,217,187,214,187,212,187,210,187,208,187,206,187,205,187,203,187,203,187,202,187,203,187,204,187,207,187,210,187,215,187,220,187,226,187,233,187,240,187,247,187,254,187,5,188,12,188,19,188,26,188,33,188,40,188,47,188,54,188,61,188,68,188,75,188,83,188,90,188,97,188,104,188,112,188,119,188,126,188,134,188,141,188,149,188,156,188,164,188,171,188,179,188,187,188,194,188,202,188,210,188,217,188,225,188,233,188,241,188,249,188,1,189,9,189,17,189,25,189,33,189,41,189,49,189,57,189,65,189,74,189,82,189,90,189,98,189,107,189,115,189,123,189,132,189,140,189,149,189,157,189,166,189,174,189,183,189,192,189,200,189,209,189,218,189,226,189,235,189,244,189,253,189,6,190,14,190,23,190,32,190,41,190,50,190,59,190,68,190,77,190,86,190,95,190,104,190,113,190,122,190,131,190,140,190,149,190,158,190,167,190,176,190,186,190,195,190,204,190,213,190,222,190,231,190,240,190,250,190,3,191,12,191,21,191,30,191,40,191,49,191,58,191,67,191,76,191,86,191,95,191,104,191,113,191,122,191,132,191,141,191,150,191,159,191,168,191,178,191,187,191,196,191,205,191,215,191,224,191,233,191,242,191,251,191,9,128,28,128,46,128,64,128,83,128,101,128,119,128,138,128,156,128,174,128,192,128,211,128,229,128,247,128,9,129,27,129,45,129,63,129,81,129,99,129,117,129,135,129,153,129,171,129,189,129,207,129,225,129,243,129,5,130,22,130,40,130,58,130,75,130,93,130,111,130,128,130,146,130,163,130,181,130,198,130,215,130,233,130,250,130,11,131])
.concat([29,131,46,131,63,131,80,131,97,131,114,131,131,131,148,131,164,131,181,131,198,131,214,131,231,131,248,131,8,132,24,132,41,132,57,132,73,132,89,132,106,132,122,132,138,132,153,132,169,132,185,132,201,132,216,132,232,132,247,132,7,133,22,133,37,133,52,133,68,133,83,133,98,133,112,133,127,133,142,133,157,133,171,133,186,133,200,133,215,133,229,133,243,133,1,134,15,134,29,134,43,134,57,134,71,134,85,134,98,134,112,134,126,134,139,134,152,134,166,134,179,134,192,134,205,134,218,134,231,134,244,134,1,135,14,135,27,135,40,135,52,135,65,135,78,135,90,135,102,135,115,135,127,135,139,135,152,135,165,135,177,135,189,135,199,135,209,135,254,135,30,136,66,136,107,136,150,136,196,136,244,136,38,137,88,137,140,137,191,137,242,137,38,138,90,138,141,138,193,138,245,138,42,139,94,139,147,139,201,139,255,139,53,140,108,140,164,140,220,140,21,141,79,141,138,141,197,141,1,142,63,142,125,142,187,142,251,142,60,143,126,143,192,143,4,144,72,144,142,144,212,144,28,145,100,145,174,145,248,145,67,146,144,146,221,146,43,147,123,147,203,147,28,148,111,148,194,148,22,149,107,149,193,149,25,150,113,150,202,150,35,151,126,151,218,151,55,152,148,152,243,152,82,153,178,153,20,154,118,154,216,154,60,155,161,155,6,156,108,156,211,156,59,157,164,157,13,158,120,158,227,158,78,159,187,159,40,160,150,160,5,161,117,161,229,161,86,162,200,162,58,163,174,163,34,164,150,164,11,165,129,165,248,165,112,166,232,166,96,167,218,167,84,168,207,168,74,169,198,169,67,170,193,170,63,171,190,171,61,172,190,172,63,173,192,173,66,174,197,174,73,175,206,175,83,176,217,176,95,177,230,177,110,178,247,178,129,179,11,180,150,180,33,181,109,74,224,73,82,73,196,72,53,72,166,71,21,71,133,70,243,69,97,69,206,68,58,68,166,67,17,67,123,66,229,65,78,65,182,64,30,64,133,63,235,62,81,62,182,61,26,61,126,60,225,59,68,59,166,58,8,58,105,57,202,56,41,56,137,55,231,54,69,54,163,53,0,53,92,52,184,51,20,51,111,50,202,49,37,49,128,48,218,47,53,47,143,46,233,45,67,45,157,44,246,43,80,43,168,42,1,42,89,41,178,40,10,40,98,39,187,38,20,38,109,37,199,36,33,36,124,35,215,34,51,34,144,33,236,32,74,32,168,31,7,31,102,30,198,29,39,29,136,28,234,27,76,27,175,26,19,26,119,25,221,24,67,24,169,23,17,23,122,22,227,21,78,21,187,20,40,20,151,19,8,19,122,18,238,17,100,17,219,16,83,16,205,15,72,15,195,14,63,14,188,13,58,13,183,12,54,12,180,11,50,11,177,10,47,10,173,9,43,9,170,8,40,8,167,7,37,7,162,6,31,6,155,5,22,5,143,4,6,4,126,3,248,2,116,2,246,1,128,1,22,1,186,0,111,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,243,255,219,255,195,255,170,255,145,255,121,255,97,255,72,255,48,255,24,255,0,255,232,254,208,254,184,254,160,254,137,254,114,254,91,254,68,254,45,254,23,254,0,254,234,253,212,253,190,253,169,253,148,253,128,253,108,253,88,253,69,253,49,253,30,253,12,253,249,252,230,252,212,252,194,252,176,252,159,252,141,252,125,252,109,252,94,252,79,252,66,252,53,252,41,252,29,252,18,252,7,252,252,251,241,251,231,251,220,251,210,251,200,251,190,251,180,251,171,251,163,251,155,251,149,251,143,251,139,251,135,251,133,251,132,251,132,251,133,251,134,251,137,251,140,251,144,251,148,251,152,251,157,251,162,251,167,251,172,251,177,251,183,251,188,251,195,251,201,251,209,251,217,251,227,251,237,251,249,251,7,252,23,252,40,252,59,252,80,252,103,252,128,252,153,252,180,252,208,252,237,252,9,253,38,253,66,253,94,253,121,253,149,253,177,253,205,253,235,253,10,254,44,254,79,254,117,254,157,254,200,254,246,254,37,255,85,255,134,255,185,255,236,255,31,0,84,0,136,0,190,0,245,0,45,1,102,1,161,1,221,1,27,2,91,2,157,2,224,2,36,3,106,3,177,3,249,3,67,4,142,4,219,4,41,5,121,5,202,5,29,6,113,6,199,6,30,7,118,7,208,7,44,8,137,8,232,8,73,9,172,9,17,10,119,10,223,10,72,11,180,11,32,12,142,12,254,12,111,13,226,13,86,14,204,14,68,15,189,15,55,16,180,16,50,17,178,17,51,18,182,18,59,19,194,19,74,20,212,20,96,21,238,21,125,22,14,23,161,23,54,24,204,24,101,25,255,25,154,26,56,27,215,27,120,28,27,29,191,29,102,30,14,31,184,31,100,32,18,33,194,33,115,34,38,35,219,35,146,36,75,37,6,38,194,38,128,39,64,40,2,41,198,41,140,42,83,43,29,44,232,44,181,45,132,46,85,47,39,48,252,48,210,49,170,50,132,51,96,52,62,53,29,54,255,54,226,55,199,56,174,57,150,58,129,59,109,60,90,61,74,62,59,63,46,64,35,65,25,66,17,67,11,68,6,69,3,70,1,71,1,72,2,73,5,74,10,75,16,76,25,77,36,78,49,79,64,80,82,81,103,82,22,0,6,0,251,255,246,255,246,255,251,255,6,0,22,0,43,0,69,0,100,0,136,0,176,0,222,0,16,1,71,1,130,1,194,1,7,2,79,2,156,2,238,2,67,3,156,3,250,3,91,4,192,4,41,5,150,5,7,6,123,6,242,6,109,7,236,7,109,8,242,8,122,9,6,10,148,10,37,11,185,11,80,12,234,12,134,13,37,14,198,14,106,15,17,16,186,16,101,17,18,18,193,18,114,19,38,20,219,20,146,21,75,22,6,23,194,23,128,24,63,25,0,26,194,26,133,27,74,28,16,29,215,29,159,30,103,31,49,32,252,32,199,33,147,34,95,35,44,36,250,36,200,37,150,38,100,39,51,40,2,41,208,41,159,42,110,43,60,44,10,45,216,45,166,46,115,47,63,48,11,49,214,49,161,50,107,51,52,52,251,52,194,53,136,54,77,55,16,56,211,56,148,57,83,58,17,59,205,59,136,60,65,61,248,61,174,62,97,63,19,64,194,64,112,65,27,66,196,66,106,67,14,68,176,68,79,69,236,69,133,70,29,71,177,71,66,72,209,72,92,73,228,73,106,74,235,74,106,75,229,75,93,76,209,76,66,77,175,77,24,78,126,78,223,78,61,79,151,79,237,79,62,80,139,80,212,80,25,81,89,81,149,81,204,81,255,81,44,82,85,82,122,82,153,82,179,82,201,82,217,82,228,82,234,82,234,82,229,82,219,82,203,82,181,82,154,82,121,82,82,82,37,82,242,81,186,81,123,81,54,81,235,80,153,80,66,80,227,79,126,79,19,79,161,78,40,78,169,77,34,77,149,76,1,76,101,75,195,74,25,74,104,73,175,72,240,71,40,71,90,70,133,69,176,68,220,67,15,67,75,66,143,65,218,64,46,64,138,63,237,62,88,62,201,61,65,61,191,60,66,60,203,59,89,59,235,58,129,58,27,58,184,57,89,57,252,56,161,56,71,56,239,55,151,55,64,55,232,54,144,54,56,54,225,53,138,53,51,53,220,52,134,52,47,52,216,51,129,51,41,51,209,50,120,50,31,50,196,49,105,49,12,49,175,48,81,48,243,47,148,47,52,47,211,46,114,46,15,46,172,45,72,45,228,44,127,44,26,44,180,43,77,43,229,42,125,42,20,42,171,41,208,250,185,250,161,250,137,250,112,250,87,250,62,250,37,250,11,250,241,249,215,249,188,249,161,249,134,249,107,249,79,249,51,249,23,249,251,248,222,248,194,248,164,248,135,248,105,248,75,248,45,248,14,248,239,247,208,247,177,247,146,247,115,247,83,247,51,247,20,247,244,246,212,246,179,246,147,246,115,246,82,246,50,246,18,246,243,245,211,245,180,245,150,245,120,245,91,245,62,245,34,245,7,245,236,244,211,244,186,244,162,244,139,244,118,244,97,244,77,244,58,244,38,244,18,244,253,243,231,243,207,243,183,243,157,243,130,243,102,243,73,243,43,243,12,243,236,242,203,242,168,242,133,242,97,242,60,242,22,242,240,241,200,241,159,241,118,241,76,241,33,241,244,240,199,240,153,240,106,240,57,240,8,240,213,239,161,239,109,239,56,239,2,239,205,238,151,238,96,238,41,238,242,237,186,237,129,237,71,237,11,237,207,236,147,236,85,236,24,236,218,235,156,235,94,235,32,235,225,234,161,234,96,234,30,234,218,233,148,233,78,233,6,233,190,232,117,232,44,232,228,231,156,231,85,231,15,231,201,230,131,230,61,230,246,229,174,229,99,229,23,229,200,228,120,228,39,228,212,227,130,227,47,227,222,226,142,226,63,226,242,225,166,225,91,225,17,225,200,224,126,224,52,224,233,223,157,223,80,223,3,223,180,222,100,222,19,222,194,221,112,221,30,221,204,220,122,220,40,220,215,219,135,219,55,219,232,218,154,218,77,218,1,218,181,217,106,217,32,217,213,216,139,216,65,216,247,215,173,215,99,215,25,215,207,214,132,214,57,214,237,213,162,213,86,213,11,213,192,212,119,212,46,212,231,211,162,211,93,211,26,211,216,210,150,210,84,210,18,210,208,209,142,209,76,209,9,209,199,208,134,208,68,208,3,208,195,207,132,207,70,207,9,207,206,206,147,206,90,206,33,206,232,205,175,205,118,205,61,205,4,205,204,204,148,204,94,204,40,204,243,203,191,203,140,203,89,203,39,203,245,202,196,202,147,202,99,202,51,202,4,202,214,201,168,201,123,201,78,201,29,201,241,200,197,200,153,200,107,200,62,200,17,200,227,199,181,199,134,199,88,199,42,199,251,198,205,198,158,198,111,198,64,198,17,198,225,197,177,197,128,197,81,197,33,197,242,196,195,196,149,196,104,196,58,196,13,196,223,195,177,195,130,195,84,195,37,195,247,194,201,194,155,194,110,194,66,194,22,194,235,193,193,193,151,193,110,193,70,193,30,193,247,192,207,192,168,192,129,192,89,192,50,192,11,192,229,191,191,191,154,191,118,191,83,191,49,191,16,191,239,190,208,190,177,190,148,190,118,190,90,190,62,190,35,190,9,190,239,189,213,189,188,189,164,189,140,189,116,189,93,189,70,189,48,189,27,189,6,189,242,188,223,188,205,188,187,188,171,188,156,188,141,188,128,188,115,188,103,188,92,188,82,188,72,188,62,188,53,188,44,188,35,188,26,188,18,188,10,188,2,188,250,187,243,187,237,187,231,187,226,187,222,187,218,187,215,187,212,187,210,187,208,187,206,187,204,187,203,187,202,187,202,187,203,187,205,187,208,187,212,187,218,187,224,187,231,187,238,187,246,187,253,187,5,188,12,188,20,188,27,188,35,188,42,188,50,188,57,188,65,188,73,188,80,188,88,188,96,188,103,188,111,188,119,188,127,188,135,188,143,188,151,188,159,188,167,188,175,188,183,188,191,188,199,188,208,188,216,188,224,188,233,188,241,188,249,188,2,189,10,189,19,189,27,189,36,189,45,189,53,189,62,189,71,189,80,189,88,189,97,189,106,189,115,189,124,189,133,189,142,189,151,189,160,189,169,189,178,189,188,189,197,189,206,189,215,189,225,189,234,189,243,189,253,189,6,190,16,190,25,190,35,190,44,190,54,190,63,190,73,190,82,190,92,190,101,190,111,190,121,190,130,190,140,190,150,190,159,190,169,190,179,190,189,190,198,190,208,190,218,190,228,190,237,190,247,190,1,191,11,191,21,191,30,191,40,191,50,191,60,191,70,191,79,191,89,191,99,191,109,191,119,191,129,191,138,191,148,191,158,191,168,191,178,191,188,191,197,191,207,191,217,191,227,191,237,191,246,191,19,128,39,128,58,128,78,128,97,128,117,128,136,128,156,128,175,128,195,128,214,128,234,128,253,128,16,129,36,129,55,129,74,129,93,129,113,129,132,129,151,129,170,129,189,129,208,129,227,129,246,129,9,130,28,130,47,130,66,130,85,130,104,130,122,130,141,130,160,130,178,130,197,130,215,130,234,130,252,130,15,131,33,131,51,131,70,131,88,131,106,131,124,131,142,131,160,131,178,131,196,131,213,131,231,131,249,131,10,132,28,132,45,132,62,132,80,132,97,132,114,132,131,132,148,132,165,132,182,132,199,132,215,132,232,132,248,132,9,133,25,133,41,133,58,133,74,133,90,133,106,133,121,133,137,133,153,133,168,133,184,133,199,133,215,133,230,133,245,133,4,134,19,134,34,134,49,134,64,134,78,134,93,134,108,134,122,134,136,134,151,134,165,134,179,134,193,134,207,134,221,134,235,134,249,134,6,135,20,135,34,135,47,135,61,135,74,135,88,135,101,135,114,135,127,135,140,135,154,135,167,135,180,135,192,135,203,135,213,135,231,135,18,136,53,136,96,136,141,136,190,136,241,136,38,137,92,137,146,137,201,137,0,138,55,138,110,138,166,138,221,138,21,139,77,139,133,139,190,139,247,139,49,140,108,140,168,140,228,140,33,141,95,141,157,141,221,141,30,142,96,142,162,142,230,142,43,143,112,143,183,143,255,143,72,144,146,144,222,144,42,145,120,145,198,145,22,146,103,146,185,146,12,147,96,147,182,147,12,148,100,148,188,148,22,149,113,149,205,149,42,150,136,150,231,150,72,151,169,151,11,152,111,152,211,152,57,153,159,153,7,154,111,154,216,154,67,155,174,155,26,156,136,156,246,156,101,157,213,157,70,158,184,158,42,159,158,159,18,160,136,160,254,160,117,161,237,161,101,162,223,162,89,163,212,163,80,164,205,164,74,165,201,165,72,166,199,166,72,167,202,167,76,168,207,168,82,169,215,169,92,170,226,170,105,171,241,171,121,172,2,173,140,173,23,174,162,174,47,175,188,175,74,176,217,176,104,177,249,177,138,178,28,179,175,179,66,180,215,180,90,74,197,73,46,73,151,72,255,71,103,71,205,70,51,70,152,69,252,68,95,68,193,67,35,67,132,66,229,65,68,65,164,64,2,64,95,63,188,62,23,62,113,61,203,60,35,60,123,59,211,58,42,58,130,57,217,56,47,56,132,55,217,54,45,54,127,53,210,52,35,52,115,51,196,50,19,50,99,49,178,48,1,48,79,47,158,46,237,45,60,45,139,44,218,43,41,43,119,42,197,41,19,41,96,40,172,39,249,38,69,38,146,37,224,36,46,36,125,35,204,34,28,34,109,33,190,32,16,32,99,31,183,30,12,30,97,29,183,28,14,28,102,27,190,26,23,26,113,25,203,24,39,24,131,23,224,22,61,22,156,21,253,20,94,20,193,19,38,19,141,18,245,17,96,17,204,16,58,16,170,15,27,15,141,14,0,14,116,13,233,12,94,12,211,11,73,11,190,10,50,10,166,9,26,9,141,8,0,8,116,7,230,6,89,6,202,5,59,5,170,4,24,4,134,3,245,2,103,2,222,1,95,1,235,0,144,0,33,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,229,255,203,255,177,255,150,255,124,255,98,255,72,255,46,255,20,255,251,254,225,254,200,254,175,254,150,254,125,254,100,254,75,254,51,254,27,254,3,254,235,253,212,253,189,253,166,253,144,253,122,253,101,253,80,253,60,253,39,253,19,253,255,252,235,252,216,252,196,252,177,252,159,252,140,252,123,252,106,252,90,252,75,252,61,252,47,252,34,252,22,252,10,252,255,251,244,251,232,251,221,251,210,251,199,251,188,251,178,251,169,251,160,251,153,251,146,251,141,251,137,251,134,251,132,251,132,251,133,251,134,251,137,251,140,251,144,251,148,251,153,251,158,251,164,251,169,251,175,251,180,251,186,251,193,251,200,251,208,251,217,251,227,251,238,251,251,251,10,252,27,252,46,252,67,252,91,252,116,252,143,252,171,252,201,252,231,252,6,253,36,253,66,253,96,253,125,253,155,253,184,253,215,253,247,253,26,254,62,254,101,254,144,254,189,254,236,254,30,255,82,255,134,255,188,255,243,255,42,0,98,0,154,0,212,0,15,1,75,1,137,1,201,1,11,2,78,2,148,2,219,2,36,3,110,3,186,3,8,4,87,4,167,4,250,4,78,5,164,5,252,5,85,6,176,6,12,7,107,7,202,7,44,8,144,8,245,8,93,9,199,9,50,10,160,10,16,11,129,11,245,11,106,12,224,12,89,13,211,13,79,14,204,14,76,15,205,15,80,16,213,16,92,17,229,17,112,18,253,18,140,19,29,20,175,20,68,21,219,21,116,22,14,23,171,23,74,24,235,24,141,25,50,26,217,26,130,27,45,28,217,28,136,29,57,30,236,30,162,31,89,32,18,33,205,33,139,34,74,35,12,36,208,36,149,37,93,38,39,39,243,39,193,40,146,41,100,42,57,43,15,44,232,44,195,45,160,46,127,47,96,48,67,49,41,50,16,51,249,51,229,52,211,53,195,54,180,55,168,56,158,57,150,58,144,59,140,60,138,61,138,62,140,63,144,64,150,65,157,66,167,67,178,68,191,69,206,70,223,71,241,72,5,74,27,75,51,76,78,77,107,78,139,79,174,80,211,81,32,0,12,0,254,255,247,255,246,255,250,255,5,0,22,0,44,0,73,0,107,0,146,0,191,0,241,0,41,1,102,1,168,1,239,1,60,2,141,2,227,2,61,3,156,3,0,4,104,4,213,4,70,5,187,5,53,6,178,6,51,7,185,7,66,8,206,8,95,9,243,9,138,10,37,11,195,11,100,12,9,13,176,13,91,14,8,15,184,15,107,16,32,17,216,17,146,18,79,19,14,20,207,20,146,21,88,22,31,23,232,23,179,24,127,25,77,26,29,27,238,27,193,28,148,29,105,30,63,31,22,32,238,32,199,33,160,34,123,35,85,36,49,37,12,38,232,38,197,39,161,40,126,41,90,42,54,43,19,44,239,44,202,45,166,46,128,47,90,48,52,49,12,50,228,50,187,51,145,52,102,53,57,54,12,55,220,55,172,56,122,57,70,58,17,59,218,59,161,60,102,61,41,62,234,62,168,63,101,64,31,65,214,65,140,66,62,67,238,67,155,68,69,69,236,69,144,70,49,71,206,71,105,72,0,73,147,73,35,74,175,74,56,75,189,75,61,76,186,76,51,77,168,77,24,78,132,78,236,78,79,79,174,79,8,80,93,80,174,80,249,80,64,81,129,81,190,81,245,81,39,82,83,82,122,82,155,82,183,82,204,82,220,82,230,82,234,82,232,82,224,82,210,82,189,82,162,82,128,82,87,82,40,82,242,81,182,81,114,81,39,81,214,80,125,80,29,80,181,79,70,79,207,78,81,78,203,77,62,77,168,76,11,76,101,75,184,74,2,74,68,73,125,72,174,71,215,70,247,69,19,69,48,68,83,67,126,66,180,65,242,64,58,64,138,63,227,62,68,62,173,61,29,61,148,60,18,60,149,59,29,59,171,58,61,58,210,57,108,57,8,57,167,56,71,56,233,55,140,55,46,55,208,54,114,54,21,54,184,53,92,53,255,52,163,52,70,52,234,51,140,51,47,51,209,50,114,50,19,50,178,49,80,49,237,48,137,48,37,48,192,47,90,47,244,46,140,46,35,46,185,45,79,45,228,44,121,44,12,44,159,43,49,43,195,42,83,42,227,41,10,0,0,0,32,0,0,0,0,0,0,0,1,0,0,0,3,0,0,0,7,0,0,0,15,0,0,0,31,0,0,0,63,0,0,0,127,0,0,0,255,0,0,0,255,1,0,0,255,3,0,0,255,7,0,0,255,15,0,0,255,31,0,0,255,63,0,0,255,127,0,0,255,255,0,0,255,255,1,0,255,255,3,0,255,255,7,0,255,255,15,0,255,255,31,0,255,255,63,0,255,255,127,0,255,255,255,0,255,255,255,1,255,255,255,3,255,255,255,7,255,255,255,15,255,255,255,31,255,255,255,63,255,255,255,127,255,255,255,255,0,0,0,0,0,0,76,194,0,0,80,194,0,0,84,194,0,0,88,194,0,0,92,194,0,0,96,194,0,0,100,194,0,0,104,194,0,0,108,194,0,0,112,194,0,0,116,194,0,0,120,194,0,0,124,194,0,0,128,194,0,0,130,194,0,0,132,194,0,0,134,194,0,0,136,194,0,0,138,194,0,0,140,194,0,0,142,194,0,0,144,194,0,0,146,194,0,0,148,194,0,0,150,194,0,0,152,194,0,0,154,194,0,0,156,194,0,0,160,194,0,0,162,194,0,0,164,194,0,0,166,194,0,0,168,194,0,0,170,194,0,0,172,194,0,0,174,194,0,0,176,194,0,0,176,194,0,0,178,194,0,0,178,194,0,0,180,194,0,0,182,194,0,0,182,194,0,0,184,194,0,0,186,194,0,0,188,194,0,0,190,194,0,0,192,194,0,0,192,194,0,0,194,194,0,0,196,194,0,0,196,194,0,0,198,194,0,0,198,194,0,0,200,194,0,0,200,194,0,0,202,194,0,0,204,194,0,0,206,194,0,0,208,194,0,0,212,194,0,0,214,194,0,0,214,194,0,0,214,194,0,0,214,194,0,0,210,194,0,0,206,194,0,0,204,194,0,0,202,194,0,0,198,194,0,0,196,194,0,0,192,194,0,0,190,194,0,0,190,194,0,0,192,194,0,0,194,194,0,0,192,194,0,0,190,194,0,0,186,194,0,0,180,194,0,0,160,194,0,0,140,194,0,0,72,194,0,0,32,194,0,0,240,193,0,0,240,193,0,0,240,193,0,0,240,193,221,1,30,61,115,47,118,192,47,250,176,188,158,20,250,64,153,188,161,186,158,119,53,193,81,220,194,184,116,225,80,65,83,153,135,188,1,154,68,193,129,18,177,60,29,186,23,65,225,231,169,188,42,236,187,192,86,189,194,59,84,76,48,64,23,210,72,59,21,174,94,191,117,48,252,56,166,136,14,62,45,12,61,59,187,242,93,61,21,159,94,192,66,120,238,188,39,159,203,64,116,13,11,188,159,194,8,193,122,116,11,188,136,161,23,65,15,206,8,188,48,10,13,193,54,239,183,60,24,84,219,64,42,177,212,188,119,161,140,192,227,27,133,60,46,141,12,64,204,220,29,187,91,68,64,191,179,14,221,59,38,166,6,62,18,27,246,186,98,72,30,62,88,65,24,192,146,25,191,189,204,80,54,64,198,233,127,189,83,84,41,192,195,60,177,60,160,42,15,64,141,230,100,189,27,243,213,191,107,217,67,61,72,195,128,63,221,177,17,59,30,72,235,190,198,2,2,61,96,182,39,62,140,213,99,188,41,29,78,189,32,117,213,59,250,86,192,60,8,103,16,188,195,30,155,62,254,109,206,191,55,145,103,190,17,54,138,63,79,222,175,189,44,92,131,190,5,120,6,61,113,172,38,190,93,7,22,188,128,210,103,190,162,171,193,188,106,76,200,62,186,131,191,187,206,177,98,190,217,136,128,61,99,84,56,61,14,238,10,183,195,81,164,60,229,233,6,59,220,52,70,59,209,172,241,188,164,63,172,62,202,209,191,191,12,238,130,190,224,157,95,63,198,63,242,189,120,245,249,61,39,37,244,61,171,200,78,191,74,115,160,189,61,4,245,62,155,0,154,187,253,11,255,189,221,42,193,187,240,154,38,189,226,118,106,61,225,172,170,61,116,82,8,60,208,143,45,189,111,248,133,188,144,228,243,60,148,49,144,188,83,247,229,62,31,210,32,191,69,246,18,190,75,222,151,62,236,79,105,190,172,192,190,190,13,131,104,188,76,24,12,59,175,11,39,61,83,49,215,190,21,234,253,189,13,83,99,62,22,214,39,61,196,1,201,59,137,153,214,61,247,48,138,61,143,176,152,188,61,242,108,61,134,205,2,189,7,1,4,61,132,146,177,59,35,242,16,63,249,36,134,191,99,48,65,191,195,71,149,62,202,81,38,62,41,63,137,190,8,118,43,62,71,89,6,60,108,141,65,190,36,174,230,62,232,94,158,62,59,32,169,190,83,31,141,190,179,5,138,61,91,28,212,59,139,246,67,189,211,25,177,61,92,87,134,60,98,50,27,189,45,15,148,60,22,191,192,187,190,188,20,63,131,166,2,191,181,32,8,191,54,36,163,190,218,83,18,190,249,108,79,190,122,105,51,62,249,208,22,62,32,205,194,60,1,112,199,62,138,81,31,62,88,186,110,190,236,195,129,190,127,224,86,189,85,103,133,60,212,73,205,188,47,187,141,61,242,19,200,60,237,111,24,189,6,255,148,60,149,162,245,187,69,87,9,63,94,65,128,190,239,223,215,190,42,39,221,190,85,217,52,187,98,70,12,189,146,207,46,61,213,159,63,189,79,51,209,189,227,53,135,62,214,104,21,62,42,194,26,62,27,131,201,188,75,199,51,190,101,108,229,189,100,191,64,190,139,76,38,189,16,94,96,61,204,36,68,61,80,177,64,61,130,177,181,188,0,0,0,0,98,120,124,63,40,114,252,191,98,120,252,191,59,253,120,63,98,120,124,63,19,41,124,63,180,33,252,191,19,41,252,191,229,96,120,63,19,41,124,63,66,185,122,63,86,171,250,191,66,185,250,191,92,142,117,63,66,185,122,63,120,174,121,63,129,154,249,191,120,174,249,191,222,132,115,63,120,174,121,63,91,33,121,63,194,9,249,191,91,33,249,191,234,113,114,63,91,33,121,63,110,236,118,63,58,195,246,191,110,236,246,191,69,43,110,63,110,236,118,63,141,200,117,63,87,148,245,191,141,200,245,191,134,249,107,63,141,200,117,63,202,100,117,63,133,44,245,191,202,100,245,191,31,58,107,63,202,100,117,63,138,43,114,63,214,203,241,191,138,43,242,191,124,22,101,63,138,43,114,63,0,0,0,0])
, "i8", ALLOC_NONE, Runtime.GLOBAL_BASE)
var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);
assert(tempDoublePtr % 8 == 0);
function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much
  HEAP8[tempDoublePtr] = HEAP8[ptr];
  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];
  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];
  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];
}
function copyTempDouble(ptr) {
  HEAP8[tempDoublePtr] = HEAP8[ptr];
  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];
  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];
  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];
  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];
  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];
  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];
  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];
}
  function _strncmp(px, py, n) {
      var i = 0;
      while (i < n) {
        var x = HEAPU8[(((px)+(i))|0)];
        var y = HEAPU8[(((py)+(i))|0)];
        if (x == y && x == 0) return 0;
        if (x == 0) return -1;
        if (y == 0) return 1;
        if (x == y) {
          i ++;
          continue;
        } else {
          return x > y ? 1 : -1;
        }
      }
      return 0;
    }function _strcmp(px, py) {
      return _strncmp(px, py, TOTAL_MEMORY);
    }
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  var ___errno_state=0;function ___setErrNo(value) {
      // For convenient setting and returning of errno.
      HEAP32[((___errno_state)>>2)]=value
      return value;
    }
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            continue;
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          if (stream.tty.output.length) {
            stream.tty.ops.put_char(stream.tty, 10);
          }
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              result = process['stdin']['read']();
              if (!result) {
                if (process['stdin']['_readableState'] && process['stdin']['_readableState']['ended']) {
                  return null;  // EOF
                }
                return undefined;  // no data available
              }
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }}};
  var MEMFS={ops_table:null,CONTENT_OWNING:1,CONTENT_FLEXIBLE:2,CONTENT_FIXED:3,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 0777, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            },
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.contents = [];
          node.contentMode = MEMFS.CONTENT_FLEXIBLE;
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },ensureFlexible:function (node) {
        if (node.contentMode !== MEMFS.CONTENT_FLEXIBLE) {
          var contents = node.contents;
          node.contents = Array.prototype.slice.call(contents);
          node.contentMode = MEMFS.CONTENT_FLEXIBLE;
        }
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.contents.length;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.ensureFlexible(node);
            var contents = node.contents;
            if (attr.size < contents.length) contents.length = attr.size;
            else while (attr.size > contents.length) contents.push(0);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 0777 | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else
          {
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          var node = stream.node;
          node.timestamp = Date.now();
          var contents = node.contents;
          if (length && contents.length === 0 && position === 0 && buffer.subarray) {
            // just replace it with the new data
            if (canOwn && buffer.buffer === HEAP8.buffer && offset === 0) {
              node.contents = buffer; // this is a subarray of the heap, and we can own it
              node.contentMode = MEMFS.CONTENT_OWNING;
            } else {
              node.contents = new Uint8Array(buffer.subarray(offset, offset+length));
              node.contentMode = MEMFS.CONTENT_FIXED;
            }
            return length;
          }
          MEMFS.ensureFlexible(node);
          var contents = node.contents;
          while (contents.length < position) contents.push(0);
          for (var i = 0; i < length; i++) {
            contents[position + i] = buffer[offset + i];
          }
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.contents.length;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          stream.ungotten = [];
          stream.position = position;
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.ensureFlexible(stream.node);
          var contents = stream.node.contents;
          var limit = offset + length;
          while (limit > contents.length) contents.push(0);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < contents.length) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        }}};
  var IDBFS={dbs:{},indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },reconcile:function (src, dst, callback) {
        var total = 0;
        var create = {};
        for (var key in src.files) {
          if (!src.files.hasOwnProperty(key)) continue;
          var e = src.files[key];
          var e2 = dst.files[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create[key] = e;
            total++;
          }
        }
        var remove = {};
        for (var key in dst.files) {
          if (!dst.files.hasOwnProperty(key)) continue;
          var e = dst.files[key];
          var e2 = src.files[key];
          if (!e2) {
            remove[key] = e;
            total++;
          }
        }
        if (!total) {
          // early out
          return callback(null);
        }
        var completed = 0;
        var done = function(err) {
          if (err) return callback(err);
          if (++completed >= total) {
            return callback(null);
          }
        };
        // create a single transaction to handle and IDB reads / writes we'll need to do
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        transaction.onerror = function() { callback(this.error); };
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
        for (var path in create) {
          if (!create.hasOwnProperty(path)) continue;
          var entry = create[path];
          if (dst.type === 'local') {
            // save file to local
            try {
              if (FS.isDir(entry.mode)) {
                FS.mkdir(path, entry.mode);
              } else if (FS.isFile(entry.mode)) {
                var stream = FS.open(path, 'w+', 0666);
                FS.write(stream, entry.contents, 0, entry.contents.length, 0, true /* canOwn */);
                FS.close(stream);
              }
              done(null);
            } catch (e) {
              return done(e);
            }
          } else {
            // save file to IDB
            var req = store.put(entry, path);
            req.onsuccess = function() { done(null); };
            req.onerror = function() { done(this.error); };
          }
        }
        for (var path in remove) {
          if (!remove.hasOwnProperty(path)) continue;
          var entry = remove[path];
          if (dst.type === 'local') {
            // delete file from local
            try {
              if (FS.isDir(entry.mode)) {
                // TODO recursive delete?
                FS.rmdir(path);
              } else if (FS.isFile(entry.mode)) {
                FS.unlink(path);
              }
              done(null);
            } catch (e) {
              return done(e);
            }
          } else {
            // delete file from IDB
            var req = store.delete(path);
            req.onsuccess = function() { done(null); };
            req.onerror = function() { done(this.error); };
          }
        }
      },getLocalSet:function (mount, callback) {
        var files = {};
        var isRealDir = function(p) {
          return p !== '.' && p !== '..';
        };
        var toAbsolute = function(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
        var check = FS.readdir(mount.mountpoint)
          .filter(isRealDir)
          .map(toAbsolute(mount.mountpoint));
        while (check.length) {
          var path = check.pop();
          var stat, node;
          try {
            var lookup = FS.lookupPath(path);
            node = lookup.node;
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path)
              .filter(isRealDir)
              .map(toAbsolute(path)));
            files[path] = { mode: stat.mode, timestamp: stat.mtime };
          } else if (FS.isFile(stat.mode)) {
            files[path] = { contents: node.contents, mode: stat.mode, timestamp: stat.mtime };
          } else {
            return callback(new Error('node type not supported'));
          }
        }
        return callback(null, { type: 'local', files: files });
      },getDB:function (name, callback) {
        // look it up in the cache
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        req.onupgradeneeded = function() {
          db = req.result;
          db.createObjectStore(IDBFS.DB_STORE_NAME);
        };
        req.onsuccess = function() {
          db = req.result;
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function() {
          callback(this.error);
        };
      },getRemoteSet:function (mount, callback) {
        var files = {};
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function() { callback(this.error); };
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          store.openCursor().onsuccess = function(event) {
            var cursor = event.target.result;
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, files: files });
            }
            files[cursor.key] = cursor.value;
            cursor.continue();
          };
        });
      }};
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so 
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          return flags;
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          stream.position = position;
          return position;
        }}};
  var _stdin=allocate(1, "i32*", ALLOC_STATIC);
  var _stdout=allocate(1, "i32*", ALLOC_STATIC);
  var _stderr=allocate(1, "i32*", ALLOC_STATIC);
  function _fflush(stream) {
      // int fflush(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fflush.html
      // we don't currently perform any user-space buffering of data
    }var FS={root:null,mounts:[],devices:[null],streams:[null],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,ErrnoError:null,genericErrors:{},handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || { recurse_count: 0 };
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
        // start at the root
        var current = FS.root;
        var current_path = '/';
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            current = current.mount.root;
          }
          // follow symlinks
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
            this.parent = null;
            this.mount = null;
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            FS.hashAddNode(this);
          };
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
          FS.FSNode.prototype = {};
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); },
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); },
            },
          });
        }
        return new FS.FSNode(parent, name, mode, rdev);
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var accmode = flag & 2097155;
        var perms = ['r', 'w', 'rw'][accmode];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        return FS.nodePermissions(dir, 'x');
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 1;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = {};
          // compatibility
          Object.defineProperties(FS.FSStream, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        stream.prototype = FS.FSStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
        var completed = 0;
        var total = FS.mounts.length;
        var done = function(err) {
          if (err) {
            return callback(err);
          }
          if (++completed >= total) {
            callback(null);
          }
        };
        // sync all mounts
        for (var i = 0; i < FS.mounts.length; i++) {
          var mount = FS.mounts[i];
          if (!mount.type.syncfs) {
            done(null);
            continue;
          }
          mount.type.syncfs(mount, populate, done);
        }
      },mount:function (type, opts, mountpoint) {
        var lookup;
        if (mountpoint) {
          lookup = FS.lookupPath(mountpoint, { follow: false });
          mountpoint = lookup.path;  // use the absolute path
        }
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          root: null
        };
        // create a root node for the fs
        var root = type.mount(mount);
        root.mount = mount;
        mount.root = root;
        // assign the mount info to the mountpoint's node
        if (lookup) {
          lookup.node.mount = mount;
          lookup.node.mounted = true;
          // compatibility update FS.root if we mount to /
          if (mountpoint === '/') {
            FS.root = mount.root;
          }
        }
        // add to our cached list of mounts
        FS.mounts.push(mount);
        return root;
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 0666;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 0777;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 0666;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
      },readlink:function (path) {
        var lookup = FS.lookupPath(path, { follow: false });
        var link = lookup.node;
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return link.node_ops.readlink(link);
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 0666 : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // check permissions
        var err = FS.mayOpen(node, flags);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        return stream;
      },close:function (stream) {
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        return stream.stream_ops.llseek(stream, offset, whence);
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.errnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = '';
          var utf8 = new Runtime.UTF8Processor();
          for (var i = 0; i < length; i++) {
            ret += utf8.processCChar(buf[i]);
          }
        } else if (opts.encoding === 'binary') {
          ret = buf;
        } else {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var utf8 = new Runtime.UTF8Processor();
          var buf = new Uint8Array(utf8.processJSString(data));
          FS.write(stream, buf, 0, buf.length, 0);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0);
        } else {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function() { return 0; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        HEAP32[((_stdin)>>2)]=stdin.fd;
        assert(stdin.fd === 1, 'invalid handle for stdin (' + stdin.fd + ')');
        var stdout = FS.open('/dev/stdout', 'w');
        HEAP32[((_stdout)>>2)]=stdout.fd;
        assert(stdout.fd === 2, 'invalid handle for stdout (' + stdout.fd + ')');
        var stderr = FS.open('/dev/stderr', 'w');
        HEAP32[((_stderr)>>2)]=stderr.fd;
        assert(stderr.fd === 3, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno) {
          this.errno = errno;
          for (var key in ERRNO_CODES) {
            if (ERRNO_CODES[key] === errno) {
              this.code = key;
              break;
            }
          }
          this.message = ERRNO_MESSAGES[errno];
          this.stack = stackTrace();
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
        FS.nameTable = new Array(4096);
        FS.root = FS.createNode(null, '/', 16384 | 0777, 0);
        FS.mount(MEMFS, {}, '/');
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
        FS.ensureErrnoError();
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
          var LazyUint8Array = function() {
            this.lengthKnown = false;
            this.chunks = []; // Loaded chunks. Index is the chunk number
          }
          LazyUint8Array.prototype.get = function(idx) {
            if (idx > this.length-1 || idx < 0) {
              return undefined;
            }
            var chunkOffset = idx % this.chunkSize;
            var chunkNum = Math.floor(idx / this.chunkSize);
            return this.getter(chunkNum)[chunkOffset];
          }
          LazyUint8Array.prototype.setDataGetter = function(getter) {
            this.getter = getter;
          }
          LazyUint8Array.prototype.cacheLength = function() {
              // Find length
              var xhr = new XMLHttpRequest();
              xhr.open('HEAD', url, false);
              xhr.send(null);
              if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
              var datalength = Number(xhr.getResponseHeader("Content-length"));
              var header;
              var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
              var chunkSize = 1024*1024; // Chunk size in bytes
              if (!hasByteServing) chunkSize = datalength;
              // Function to get a range from the remote URL.
              var doXHR = (function(from, to) {
                if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
                if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
                // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
                var xhr = new XMLHttpRequest();
                xhr.open('GET', url, false);
                if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
                // Some hints to the browser that we want binary data.
                if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
                if (xhr.overrideMimeType) {
                  xhr.overrideMimeType('text/plain; charset=x-user-defined');
                }
                xhr.send(null);
                if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
                if (xhr.response !== undefined) {
                  return new Uint8Array(xhr.response || []);
                } else {
                  return intArrayFromString(xhr.responseText || '', true);
                }
              });
              var lazyArray = this;
              lazyArray.setDataGetter(function(chunkNum) {
                var start = chunkNum * chunkSize;
                var end = (chunkNum+1) * chunkSize - 1; // including this byte
                end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
                if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
                  lazyArray.chunks[chunkNum] = doXHR(start, end);
                }
                if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
                return lazyArray.chunks[chunkNum];
              });
              this._length = datalength;
              this._chunkSize = chunkSize;
              this.lengthKnown = true;
          }
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        function processData(byteArray) {
          function finish(byteArray) {
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency('cp ' + fullname);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency('cp ' + fullname);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency('cp ' + fullname);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};
  var SOCKFS={mount:function (mount) {
        return FS.createNode(null, '/', 16384 | 0777, 0);
      },createSocket:function (family, type, protocol) {
        var streaming = type == 1;
        if (protocol) {
          assert(streaming == (protocol == 6)); // if SOCK_STREAM, must be tcp
        }
        // create our internal socket structure
        var sock = {
          family: family,
          type: type,
          protocol: protocol,
          server: null,
          peers: {},
          pending: [],
          recv_queue: [],
          sock_ops: SOCKFS.websocket_sock_ops
        };
        // create the filesystem node to store the socket structure
        var name = SOCKFS.nextname();
        var node = FS.createNode(SOCKFS.root, name, 49152, 0);
        node.sock = sock;
        // and the wrapping stream that enables library functions such
        // as read and write to indirectly interact with the socket
        var stream = FS.createStream({
          path: name,
          node: node,
          flags: FS.modeStringToFlags('r+'),
          seekable: false,
          stream_ops: SOCKFS.stream_ops
        });
        // map the new stream to the socket structure (sockets have a 1:1
        // relationship with a stream)
        sock.stream = stream;
        return sock;
      },getSocket:function (fd) {
        var stream = FS.getStream(fd);
        if (!stream || !FS.isSocket(stream.node.mode)) {
          return null;
        }
        return stream.node.sock;
      },stream_ops:{poll:function (stream) {
          var sock = stream.node.sock;
          return sock.sock_ops.poll(sock);
        },ioctl:function (stream, request, varargs) {
          var sock = stream.node.sock;
          return sock.sock_ops.ioctl(sock, request, varargs);
        },read:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          var msg = sock.sock_ops.recvmsg(sock, length);
          if (!msg) {
            // socket is closed
            return 0;
          }
          buffer.set(msg.buffer, offset);
          return msg.buffer.length;
        },write:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          return sock.sock_ops.sendmsg(sock, buffer, offset, length);
        },close:function (stream) {
          var sock = stream.node.sock;
          sock.sock_ops.close(sock);
        }},nextname:function () {
        if (!SOCKFS.nextname.current) {
          SOCKFS.nextname.current = 0;
        }
        return 'socket[' + (SOCKFS.nextname.current++) + ']';
      },websocket_sock_ops:{createPeer:function (sock, addr, port) {
          var ws;
          if (typeof addr === 'object') {
            ws = addr;
            addr = null;
            port = null;
          }
          if (ws) {
            // for sockets that've already connected (e.g. we're the server)
            // we can inspect the _socket property for the address
            if (ws._socket) {
              addr = ws._socket.remoteAddress;
              port = ws._socket.remotePort;
            }
            // if we're just now initializing a connection to the remote,
            // inspect the url property
            else {
              var result = /ws[s]?:\/\/([^:]+):(\d+)/.exec(ws.url);
              if (!result) {
                throw new Error('WebSocket URL must be in the format ws(s)://address:port');
              }
              addr = result[1];
              port = parseInt(result[2], 10);
            }
          } else {
            // create the actual websocket object and connect
            try {
              var url = 'ws://' + addr + ':' + port;
              // the node ws library API is slightly different than the browser's
              var opts = ENVIRONMENT_IS_NODE ? {} : ['binary'];
              ws = new WebSocket(url, opts);
              ws.binaryType = 'arraybuffer';
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EHOSTUNREACH);
            }
          }
          var peer = {
            addr: addr,
            port: port,
            socket: ws,
            dgram_send_queue: []
          };
          SOCKFS.websocket_sock_ops.addPeer(sock, peer);
          SOCKFS.websocket_sock_ops.handlePeerEvents(sock, peer);
          // if this is a bound dgram socket, send the port number first to allow
          // us to override the ephemeral port reported to us by remotePort on the
          // remote end.
          if (sock.type === 2 && typeof sock.sport !== 'undefined') {
            peer.dgram_send_queue.push(new Uint8Array([
                255, 255, 255, 255,
                'p'.charCodeAt(0), 'o'.charCodeAt(0), 'r'.charCodeAt(0), 't'.charCodeAt(0),
                ((sock.sport & 0xff00) >> 8) , (sock.sport & 0xff)
            ]));
          }
          return peer;
        },getPeer:function (sock, addr, port) {
          return sock.peers[addr + ':' + port];
        },addPeer:function (sock, peer) {
          sock.peers[peer.addr + ':' + peer.port] = peer;
        },removePeer:function (sock, peer) {
          delete sock.peers[peer.addr + ':' + peer.port];
        },handlePeerEvents:function (sock, peer) {
          var first = true;
          var handleOpen = function () {
            try {
              var queued = peer.dgram_send_queue.shift();
              while (queued) {
                peer.socket.send(queued);
                queued = peer.dgram_send_queue.shift();
              }
            } catch (e) {
              // not much we can do here in the way of proper error handling as we've already
              // lied and said this data was sent. shut it down.
              peer.socket.close();
            }
          };
          var handleMessage = function(data) {
            assert(typeof data !== 'string' && data.byteLength !== undefined);  // must receive an ArrayBuffer
            data = new Uint8Array(data);  // make a typed array view on the array buffer
            // if this is the port message, override the peer's port with it
            var wasfirst = first;
            first = false;
            if (wasfirst &&
                data.length === 10 &&
                data[0] === 255 && data[1] === 255 && data[2] === 255 && data[3] === 255 &&
                data[4] === 'p'.charCodeAt(0) && data[5] === 'o'.charCodeAt(0) && data[6] === 'r'.charCodeAt(0) && data[7] === 't'.charCodeAt(0)) {
              // update the peer's port and it's key in the peer map
              var newport = ((data[8] << 8) | data[9]);
              SOCKFS.websocket_sock_ops.removePeer(sock, peer);
              peer.port = newport;
              SOCKFS.websocket_sock_ops.addPeer(sock, peer);
              return;
            }
            sock.recv_queue.push({ addr: peer.addr, port: peer.port, data: data });
          };
          if (ENVIRONMENT_IS_NODE) {
            peer.socket.on('open', handleOpen);
            peer.socket.on('message', function(data, flags) {
              if (!flags.binary) {
                return;
              }
              handleMessage((new Uint8Array(data)).buffer);  // copy from node Buffer -> ArrayBuffer
            });
            peer.socket.on('error', function() {
              // don't throw
            });
          } else {
            peer.socket.onopen = handleOpen;
            peer.socket.onmessage = function(event) {
              handleMessage(event.data);
            };
          }
        },poll:function (sock) {
          if (sock.type === 1 && sock.server) {
            // listen sockets should only say they're available for reading
            // if there are pending clients.
            return sock.pending.length ? (64 | 1) : 0;
          }
          var mask = 0;
          var dest = sock.type === 1 ?  // we only care about the socket state for connection-based sockets
            SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport) :
            null;
          if (sock.recv_queue.length ||
              !dest ||  // connection-less sockets are always ready to read
              (dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {  // let recv return 0 once closed
            mask |= (64 | 1);
          }
          if (!dest ||  // connection-less sockets are always ready to write
              (dest && dest.socket.readyState === dest.socket.OPEN)) {
            mask |= 4;
          }
          if ((dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {
            mask |= 16;
          }
          return mask;
        },ioctl:function (sock, request, arg) {
          switch (request) {
            case 21531:
              var bytes = 0;
              if (sock.recv_queue.length) {
                bytes = sock.recv_queue[0].data.length;
              }
              HEAP32[((arg)>>2)]=bytes;
              return 0;
            default:
              return ERRNO_CODES.EINVAL;
          }
        },close:function (sock) {
          // if we've spawned a listen server, close it
          if (sock.server) {
            try {
              sock.server.close();
            } catch (e) {
            }
            sock.server = null;
          }
          // close any peer connections
          var peers = Object.keys(sock.peers);
          for (var i = 0; i < peers.length; i++) {
            var peer = sock.peers[peers[i]];
            try {
              peer.socket.close();
            } catch (e) {
            }
            SOCKFS.websocket_sock_ops.removePeer(sock, peer);
          }
          return 0;
        },bind:function (sock, addr, port) {
          if (typeof sock.saddr !== 'undefined' || typeof sock.sport !== 'undefined') {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already bound
          }
          sock.saddr = addr;
          sock.sport = port || _mkport();
          // in order to emulate dgram sockets, we need to launch a listen server when
          // binding on a connection-less socket
          // note: this is only required on the server side
          if (sock.type === 2) {
            // close the existing server if it exists
            if (sock.server) {
              sock.server.close();
              sock.server = null;
            }
            // swallow error operation not supported error that occurs when binding in the
            // browser where this isn't supported
            try {
              sock.sock_ops.listen(sock, 0);
            } catch (e) {
              if (!(e instanceof FS.ErrnoError)) throw e;
              if (e.errno !== ERRNO_CODES.EOPNOTSUPP) throw e;
            }
          }
        },connect:function (sock, addr, port) {
          if (sock.server) {
            throw new FS.ErrnoError(ERRNO_CODS.EOPNOTSUPP);
          }
          // TODO autobind
          // if (!sock.addr && sock.type == 2) {
          // }
          // early out if we're already connected / in the middle of connecting
          if (typeof sock.daddr !== 'undefined' && typeof sock.dport !== 'undefined') {
            var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
            if (dest) {
              if (dest.socket.readyState === dest.socket.CONNECTING) {
                throw new FS.ErrnoError(ERRNO_CODES.EALREADY);
              } else {
                throw new FS.ErrnoError(ERRNO_CODES.EISCONN);
              }
            }
          }
          // add the socket to our peer list and set our
          // destination address / port to match
          var peer = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
          sock.daddr = peer.addr;
          sock.dport = peer.port;
          // always "fail" in non-blocking mode
          throw new FS.ErrnoError(ERRNO_CODES.EINPROGRESS);
        },listen:function (sock, backlog) {
          if (!ENVIRONMENT_IS_NODE) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
          if (sock.server) {
             throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already listening
          }
          var WebSocketServer = require('ws').Server;
          var host = sock.saddr;
          sock.server = new WebSocketServer({
            host: host,
            port: sock.sport
            // TODO support backlog
          });
          sock.server.on('connection', function(ws) {
            if (sock.type === 1) {
              var newsock = SOCKFS.createSocket(sock.family, sock.type, sock.protocol);
              // create a peer on the new socket
              var peer = SOCKFS.websocket_sock_ops.createPeer(newsock, ws);
              newsock.daddr = peer.addr;
              newsock.dport = peer.port;
              // push to queue for accept to pick up
              sock.pending.push(newsock);
            } else {
              // create a peer on the listen socket so calling sendto
              // with the listen socket and an address will resolve
              // to the correct client
              SOCKFS.websocket_sock_ops.createPeer(sock, ws);
            }
          });
          sock.server.on('closed', function() {
            sock.server = null;
          });
          sock.server.on('error', function() {
            // don't throw
          });
        },accept:function (listensock) {
          if (!listensock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          var newsock = listensock.pending.shift();
          newsock.stream.flags = listensock.stream.flags;
          return newsock;
        },getname:function (sock, peer) {
          var addr, port;
          if (peer) {
            if (sock.daddr === undefined || sock.dport === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            }
            addr = sock.daddr;
            port = sock.dport;
          } else {
            // TODO saddr and sport will be set for bind()'d UDP sockets, but what
            // should we be returning for TCP sockets that've been connect()'d?
            addr = sock.saddr || 0;
            port = sock.sport || 0;
          }
          return { addr: addr, port: port };
        },sendmsg:function (sock, buffer, offset, length, addr, port) {
          if (sock.type === 2) {
            // connection-less sockets will honor the message address,
            // and otherwise fall back to the bound destination address
            if (addr === undefined || port === undefined) {
              addr = sock.daddr;
              port = sock.dport;
            }
            // if there was no address to fall back to, error out
            if (addr === undefined || port === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.EDESTADDRREQ);
            }
          } else {
            // connection-based sockets will only use the bound
            addr = sock.daddr;
            port = sock.dport;
          }
          // find the peer for the destination address
          var dest = SOCKFS.websocket_sock_ops.getPeer(sock, addr, port);
          // early out if not connected with a connection-based socket
          if (sock.type === 1) {
            if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            } else if (dest.socket.readyState === dest.socket.CONNECTING) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
          // create a copy of the incoming data to send, as the WebSocket API
          // doesn't work entirely with an ArrayBufferView, it'll just send
          // the entire underlying buffer
          var data;
          if (buffer instanceof Array || buffer instanceof ArrayBuffer) {
            data = buffer.slice(offset, offset + length);
          } else {  // ArrayBufferView
            data = buffer.buffer.slice(buffer.byteOffset + offset, buffer.byteOffset + offset + length);
          }
          // if we're emulating a connection-less dgram socket and don't have
          // a cached connection, queue the buffer to send upon connect and
          // lie, saying the data was sent now.
          if (sock.type === 2) {
            if (!dest || dest.socket.readyState !== dest.socket.OPEN) {
              // if we're not connected, open a new connection
              if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                dest = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
              }
              dest.dgram_send_queue.push(data);
              return length;
            }
          }
          try {
            // send the actual data
            dest.socket.send(data);
            return length;
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
        },recvmsg:function (sock, length) {
          // http://pubs.opengroup.org/onlinepubs/7908799/xns/recvmsg.html
          if (sock.type === 1 && sock.server) {
            // tcp servers should not be recv()'ing on the listen socket
            throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
          }
          var queued = sock.recv_queue.shift();
          if (!queued) {
            if (sock.type === 1) {
              var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
              if (!dest) {
                // if we have a destination address but are not connected, error out
                throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
              }
              else if (dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                // return null if the socket has closed
                return null;
              }
              else {
                // else, our socket is in a valid state but truly has nothing available
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
            } else {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
          // queued.data will be an ArrayBuffer if it's unadulterated, but if it's
          // requeued TCP data it'll be an ArrayBufferView
          var queuedLength = queued.data.byteLength || queued.data.length;
          var queuedOffset = queued.data.byteOffset || 0;
          var queuedBuffer = queued.data.buffer || queued.data;
          var bytesRead = Math.min(length, queuedLength);
          var res = {
            buffer: new Uint8Array(queuedBuffer, queuedOffset, bytesRead),
            addr: queued.addr,
            port: queued.port
          };
          // push back any unread data for TCP connections
          if (sock.type === 1 && bytesRead < queuedLength) {
            var bytesRemaining = queuedLength - bytesRead;
            queued.data = new Uint8Array(queuedBuffer, queuedOffset + bytesRead, bytesRemaining);
            sock.recv_queue.unshift(queued);
          }
          return res;
        }}};function _send(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _write(fd, buf, len);
    }
  function _pwrite(fildes, buf, nbyte, offset) {
      // ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _write(fildes, buf, nbyte) {
      // ssize_t write(int fildes, const void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fwrite(ptr, size, nitems, stream) {
      // size_t fwrite(const void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fwrite.html
      var bytesToWrite = nitems * size;
      if (bytesToWrite == 0) return 0;
      var bytesWritten = _write(stream, ptr, bytesToWrite);
      if (bytesWritten == -1) {
        var streamObj = FS.getStream(stream);
        if (streamObj) streamObj.error = true;
        return 0;
      } else {
        return Math.floor(bytesWritten / size);
      }
    }
  Module["_strlen"] = _strlen;
  function __reallyNegative(x) {
      return x < 0 || (x === 0 && (1/x) === -Infinity);
    }function __formatString(format, varargs) {
      var textIndex = format;
      var argIndex = 0;
      function getNextArg(type) {
        // NOTE: Explicitly ignoring type safety. Otherwise this fails:
        //       int x = 4; printf("%c\n", (char)x);
        var ret;
        if (type === 'double') {
          ret = HEAPF64[(((varargs)+(argIndex))>>3)];
        } else if (type == 'i64') {
          ret = [HEAP32[(((varargs)+(argIndex))>>2)],
                 HEAP32[(((varargs)+(argIndex+8))>>2)]];
          argIndex += 8; // each 32-bit chunk is in a 64-bit block
        } else {
          type = 'i32'; // varargs are always i32, i64, or double
          ret = HEAP32[(((varargs)+(argIndex))>>2)];
        }
        argIndex += Math.max(Runtime.getNativeFieldSize(type), Runtime.getAlignSize(type, null, true));
        return ret;
      }
      var ret = [];
      var curr, next, currArg;
      while(1) {
        var startTextIndex = textIndex;
        curr = HEAP8[(textIndex)];
        if (curr === 0) break;
        next = HEAP8[((textIndex+1)|0)];
        if (curr == 37) {
          // Handle flags.
          var flagAlwaysSigned = false;
          var flagLeftAlign = false;
          var flagAlternative = false;
          var flagZeroPad = false;
          var flagPadSign = false;
          flagsLoop: while (1) {
            switch (next) {
              case 43:
                flagAlwaysSigned = true;
                break;
              case 45:
                flagLeftAlign = true;
                break;
              case 35:
                flagAlternative = true;
                break;
              case 48:
                if (flagZeroPad) {
                  break flagsLoop;
                } else {
                  flagZeroPad = true;
                  break;
                }
              case 32:
                flagPadSign = true;
                break;
              default:
                break flagsLoop;
            }
            textIndex++;
            next = HEAP8[((textIndex+1)|0)];
          }
          // Handle width.
          var width = 0;
          if (next == 42) {
            width = getNextArg('i32');
            textIndex++;
            next = HEAP8[((textIndex+1)|0)];
          } else {
            while (next >= 48 && next <= 57) {
              width = width * 10 + (next - 48);
              textIndex++;
              next = HEAP8[((textIndex+1)|0)];
            }
          }
          // Handle precision.
          var precisionSet = false;
          if (next == 46) {
            var precision = 0;
            precisionSet = true;
            textIndex++;
            next = HEAP8[((textIndex+1)|0)];
            if (next == 42) {
              precision = getNextArg('i32');
              textIndex++;
            } else {
              while(1) {
                var precisionChr = HEAP8[((textIndex+1)|0)];
                if (precisionChr < 48 ||
                    precisionChr > 57) break;
                precision = precision * 10 + (precisionChr - 48);
                textIndex++;
              }
            }
            next = HEAP8[((textIndex+1)|0)];
          } else {
            var precision = 6; // Standard default.
          }
          // Handle integer sizes. WARNING: These assume a 32-bit architecture!
          var argSize;
          switch (String.fromCharCode(next)) {
            case 'h':
              var nextNext = HEAP8[((textIndex+2)|0)];
              if (nextNext == 104) {
                textIndex++;
                argSize = 1; // char (actually i32 in varargs)
              } else {
                argSize = 2; // short (actually i32 in varargs)
              }
              break;
            case 'l':
              var nextNext = HEAP8[((textIndex+2)|0)];
              if (nextNext == 108) {
                textIndex++;
                argSize = 8; // long long
              } else {
                argSize = 4; // long
              }
              break;
            case 'L': // long long
            case 'q': // int64_t
            case 'j': // intmax_t
              argSize = 8;
              break;
            case 'z': // size_t
            case 't': // ptrdiff_t
            case 'I': // signed ptrdiff_t or unsigned size_t
              argSize = 4;
              break;
            default:
              argSize = null;
          }
          if (argSize) textIndex++;
          next = HEAP8[((textIndex+1)|0)];
          // Handle type specifier.
          switch (String.fromCharCode(next)) {
            case 'd': case 'i': case 'u': case 'o': case 'x': case 'X': case 'p': {
              // Integer.
              var signed = next == 100 || next == 105;
              argSize = argSize || 4;
              var currArg = getNextArg('i' + (argSize * 8));
              var origArg = currArg;
              var argText;
              // Flatten i64-1 [low, high] into a (slightly rounded) double
              if (argSize == 8) {
                currArg = Runtime.makeBigInt(currArg[0], currArg[1], next == 117);
              }
              // Truncate to requested size.
              if (argSize <= 4) {
                var limit = Math.pow(256, argSize) - 1;
                currArg = (signed ? reSign : unSign)(currArg & limit, argSize * 8);
              }
              // Format the number.
              var currAbsArg = Math.abs(currArg);
              var prefix = '';
              if (next == 100 || next == 105) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], null); else
                argText = reSign(currArg, 8 * argSize, 1).toString(10);
              } else if (next == 117) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], true); else
                argText = unSign(currArg, 8 * argSize, 1).toString(10);
                currArg = Math.abs(currArg);
              } else if (next == 111) {
                argText = (flagAlternative ? '0' : '') + currAbsArg.toString(8);
              } else if (next == 120 || next == 88) {
                prefix = (flagAlternative && currArg != 0) ? '0x' : '';
                if (argSize == 8 && i64Math) {
                  if (origArg[1]) {
                    argText = (origArg[1]>>>0).toString(16);
                    var lower = (origArg[0]>>>0).toString(16);
                    while (lower.length < 8) lower = '0' + lower;
                    argText += lower;
                  } else {
                    argText = (origArg[0]>>>0).toString(16);
                  }
                } else
                if (currArg < 0) {
                  // Represent negative numbers in hex as 2's complement.
                  currArg = -currArg;
                  argText = (currAbsArg - 1).toString(16);
                  var buffer = [];
                  for (var i = 0; i < argText.length; i++) {
                    buffer.push((0xF - parseInt(argText[i], 16)).toString(16));
                  }
                  argText = buffer.join('');
                  while (argText.length < argSize * 2) argText = 'f' + argText;
                } else {
                  argText = currAbsArg.toString(16);
                }
                if (next == 88) {
                  prefix = prefix.toUpperCase();
                  argText = argText.toUpperCase();
                }
              } else if (next == 112) {
                if (currAbsArg === 0) {
                  argText = '(nil)';
                } else {
                  prefix = '0x';
                  argText = currAbsArg.toString(16);
                }
              }
              if (precisionSet) {
                while (argText.length < precision) {
                  argText = '0' + argText;
                }
              }
              // Add sign if needed
              if (currArg >= 0) {
                if (flagAlwaysSigned) {
                  prefix = '+' + prefix;
                } else if (flagPadSign) {
                  prefix = ' ' + prefix;
                }
              }
              // Move sign to prefix so we zero-pad after the sign
              if (argText.charAt(0) == '-') {
                prefix = '-' + prefix;
                argText = argText.substr(1);
              }
              // Add padding.
              while (prefix.length + argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad) {
                    argText = '0' + argText;
                  } else {
                    prefix = ' ' + prefix;
                  }
                }
              }
              // Insert the result into the buffer.
              argText = prefix + argText;
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 'f': case 'F': case 'e': case 'E': case 'g': case 'G': {
              // Float.
              var currArg = getNextArg('double');
              var argText;
              if (isNaN(currArg)) {
                argText = 'nan';
                flagZeroPad = false;
              } else if (!isFinite(currArg)) {
                argText = (currArg < 0 ? '-' : '') + 'inf';
                flagZeroPad = false;
              } else {
                var isGeneral = false;
                var effectivePrecision = Math.min(precision, 20);
                // Convert g/G to f/F or e/E, as per:
                // http://pubs.opengroup.org/onlinepubs/9699919799/functions/printf.html
                if (next == 103 || next == 71) {
                  isGeneral = true;
                  precision = precision || 1;
                  var exponent = parseInt(currArg.toExponential(effectivePrecision).split('e')[1], 10);
                  if (precision > exponent && exponent >= -4) {
                    next = ((next == 103) ? 'f' : 'F').charCodeAt(0);
                    precision -= exponent + 1;
                  } else {
                    next = ((next == 103) ? 'e' : 'E').charCodeAt(0);
                    precision--;
                  }
                  effectivePrecision = Math.min(precision, 20);
                }
                if (next == 101 || next == 69) {
                  argText = currArg.toExponential(effectivePrecision);
                  // Make sure the exponent has at least 2 digits.
                  if (/[eE][-+]\d$/.test(argText)) {
                    argText = argText.slice(0, -1) + '0' + argText.slice(-1);
                  }
                } else if (next == 102 || next == 70) {
                  argText = currArg.toFixed(effectivePrecision);
                  if (currArg === 0 && __reallyNegative(currArg)) {
                    argText = '-' + argText;
                  }
                }
                var parts = argText.split('e');
                if (isGeneral && !flagAlternative) {
                  // Discard trailing zeros and periods.
                  while (parts[0].length > 1 && parts[0].indexOf('.') != -1 &&
                         (parts[0].slice(-1) == '0' || parts[0].slice(-1) == '.')) {
                    parts[0] = parts[0].slice(0, -1);
                  }
                } else {
                  // Make sure we have a period in alternative mode.
                  if (flagAlternative && argText.indexOf('.') == -1) parts[0] += '.';
                  // Zero pad until required precision.
                  while (precision > effectivePrecision++) parts[0] += '0';
                }
                argText = parts[0] + (parts.length > 1 ? 'e' + parts[1] : '');
                // Capitalize 'E' if needed.
                if (next == 69) argText = argText.toUpperCase();
                // Add sign.
                if (currArg >= 0) {
                  if (flagAlwaysSigned) {
                    argText = '+' + argText;
                  } else if (flagPadSign) {
                    argText = ' ' + argText;
                  }
                }
              }
              // Add padding.
              while (argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad && (argText[0] == '-' || argText[0] == '+')) {
                    argText = argText[0] + '0' + argText.slice(1);
                  } else {
                    argText = (flagZeroPad ? '0' : ' ') + argText;
                  }
                }
              }
              // Adjust case.
              if (next < 97) argText = argText.toUpperCase();
              // Insert the result into the buffer.
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 's': {
              // String.
              var arg = getNextArg('i8*');
              var argLength = arg ? _strlen(arg) : '(null)'.length;
              if (precisionSet) argLength = Math.min(argLength, precision);
              if (!flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              if (arg) {
                for (var i = 0; i < argLength; i++) {
                  ret.push(HEAPU8[((arg++)|0)]);
                }
              } else {
                ret = ret.concat(intArrayFromString('(null)'.substr(0, argLength), true));
              }
              if (flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              break;
            }
            case 'c': {
              // Character.
              if (flagLeftAlign) ret.push(getNextArg('i8'));
              while (--width > 0) {
                ret.push(32);
              }
              if (!flagLeftAlign) ret.push(getNextArg('i8'));
              break;
            }
            case 'n': {
              // Write the length written so far to the next parameter.
              var ptr = getNextArg('i32*');
              HEAP32[((ptr)>>2)]=ret.length
              break;
            }
            case '%': {
              // Literal percent sign.
              ret.push(curr);
              break;
            }
            default: {
              // Unknown specifiers remain untouched.
              for (var i = startTextIndex; i < textIndex + 2; i++) {
                ret.push(HEAP8[(i)]);
              }
            }
          }
          textIndex += 2;
          // TODO: Support a/A (hex float) and m (last error) specifiers.
          // TODO: Support %1${specifier} for arg selection.
        } else {
          ret.push(curr);
          textIndex += 1;
        }
      }
      return ret;
    }function _fprintf(stream, format, varargs) {
      // int fprintf(FILE *restrict stream, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var result = __formatString(format, varargs);
      var stack = Runtime.stackSave();
      var ret = _fwrite(allocate(result, 'i8', ALLOC_STACK), 1, result.length, stream);
      Runtime.stackRestore(stack);
      return ret;
    }function _printf(format, varargs) {
      // int printf(const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var stdout = HEAP32[((_stdout)>>2)];
      return _fprintf(stdout, format, varargs);
    }
  Module["_memset"] = _memset;var _llvm_memset_p0i8_i32=_memset;
  function _recv(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _read(fd, buf, len);
    }
  function _pread(fildes, buf, nbyte, offset) {
      // ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _read(fildes, buf, nbyte) {
      // ssize_t read(int fildes, void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fread(ptr, size, nitems, stream) {
      // size_t fread(void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fread.html
      var bytesToRead = nitems * size;
      if (bytesToRead == 0) {
        return 0;
      }
      var bytesRead = 0;
      var streamObj = FS.getStream(stream);
      while (streamObj.ungotten.length && bytesToRead > 0) {
        HEAP8[((ptr++)|0)]=streamObj.ungotten.pop()
        bytesToRead--;
        bytesRead++;
      }
      var err = _read(stream, ptr, bytesToRead);
      if (err == -1) {
        if (streamObj) streamObj.error = true;
        return 0;
      }
      bytesRead += err;
      if (bytesRead < bytesToRead) streamObj.eof = true;
      return Math.floor(bytesRead / size);
    }function _fgetc(stream) {
      // int fgetc(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fgetc.html
      var streamObj = FS.getStream(stream);
      if (!streamObj) return -1;
      if (streamObj.eof || streamObj.error) return -1;
      var ret = _fread(_fgetc.ret, 1, 1, stream);
      if (ret == 0) {
        return -1;
      } else if (ret == -1) {
        streamObj.error = true;
        return -1;
      } else {
        return HEAPU8[((_fgetc.ret)|0)];
      }
    }function _fgets(s, n, stream) {
      // char *fgets(char *restrict s, int n, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fgets.html
      var streamObj = FS.getStream(stream);
      if (!streamObj) return 0;
      if (streamObj.error || streamObj.eof) return 0;
      var byte_;
      for (var i = 0; i < n - 1 && byte_ != 10; i++) {
        byte_ = _fgetc(stream);
        if (byte_ == -1) {
          if (streamObj.error || (streamObj.eof && i == 0)) return 0;
          else if (streamObj.eof) break;
        }
        HEAP8[(((s)+(i))|0)]=byte_
      }
      HEAP8[(((s)+(i))|0)]=0
      return s;
    }
  function _strcspn(pstr, pset) {
      var str = pstr, set, strcurr, setcurr;
      while (1) {
        strcurr = HEAP8[(str)];
        if (!strcurr) return str - pstr;
        set = pset;
        while (1) {
          setcurr = HEAP8[(set)];
          if (!setcurr || setcurr == strcurr) break;
          set++;
        }
        if (setcurr) return str - pstr;
        str++;
      }
    }
  function _close(fildes) {
      // int close(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/close.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        FS.close(stream);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }
  function _fsync(fildes) {
      // int fsync(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fsync.html
      var stream = FS.getStream(fildes);
      if (stream) {
        // We write directly to the file system, so there's nothing to do here.
        return 0;
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }function _fclose(stream) {
      // int fclose(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fclose.html
      _fsync(stream);
      return _close(stream);
    }
  function _snprintf(s, n, format, varargs) {
      // int snprintf(char *restrict s, size_t n, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var result = __formatString(format, varargs);
      var limit = (n === undefined) ? result.length
                                    : Math.min(result.length, Math.max(n - 1, 0));
      if (s < 0) {
        s = -s;
        var buf = _malloc(limit+1);
        HEAP32[((s)>>2)]=buf;
        s = buf;
      }
      for (var i = 0; i < limit; i++) {
        HEAP8[(((s)+(i))|0)]=result[i];
      }
      if (limit < n || (n === undefined)) HEAP8[(((s)+(i))|0)]=0;
      return result.length;
    }
  function _strchr(ptr, chr) {
      ptr--;
      do {
        ptr++;
        var val = HEAP8[(ptr)];
        if (val == chr) return ptr;
      } while (val);
      return 0;
    }
  function _time(ptr) {
      var ret = Math.floor(Date.now()/1000);
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret
      }
      return ret;
    }
  var ___tm_current=allocate(44, "i8", ALLOC_STATIC);
  var ___tm_timezone=allocate(intArrayFromString("GMT"), "i8", ALLOC_STATIC);
  var _tzname=allocate(8, "i32*", ALLOC_STATIC);
  var _daylight=allocate(1, "i32*", ALLOC_STATIC);
  var _timezone=allocate(1, "i32*", ALLOC_STATIC);function _tzset() {
      // TODO: Use (malleable) environment variables instead of system settings.
      if (_tzset.called) return;
      _tzset.called = true;
      HEAP32[((_timezone)>>2)]=-(new Date()).getTimezoneOffset() * 60
      var winter = new Date(2000, 0, 1);
      var summer = new Date(2000, 6, 1);
      HEAP32[((_daylight)>>2)]=Number(winter.getTimezoneOffset() != summer.getTimezoneOffset())
      var winterName = 'GMT'; // XXX do not rely on browser timezone info, it is very unpredictable | winter.toString().match(/\(([A-Z]+)\)/)[1];
      var summerName = 'GMT'; // XXX do not rely on browser timezone info, it is very unpredictable | summer.toString().match(/\(([A-Z]+)\)/)[1];
      var winterNamePtr = allocate(intArrayFromString(winterName), 'i8', ALLOC_NORMAL);
      var summerNamePtr = allocate(intArrayFromString(summerName), 'i8', ALLOC_NORMAL);
      HEAP32[((_tzname)>>2)]=winterNamePtr
      HEAP32[(((_tzname)+(4))>>2)]=summerNamePtr
    }function _localtime_r(time, tmPtr) {
      _tzset();
      var date = new Date(HEAP32[((time)>>2)]*1000);
      HEAP32[((tmPtr)>>2)]=date.getSeconds()
      HEAP32[(((tmPtr)+(4))>>2)]=date.getMinutes()
      HEAP32[(((tmPtr)+(8))>>2)]=date.getHours()
      HEAP32[(((tmPtr)+(12))>>2)]=date.getDate()
      HEAP32[(((tmPtr)+(16))>>2)]=date.getMonth()
      HEAP32[(((tmPtr)+(20))>>2)]=date.getFullYear()-1900
      HEAP32[(((tmPtr)+(24))>>2)]=date.getDay()
      var start = new Date(date.getFullYear(), 0, 1);
      var yday = Math.floor((date.getTime() - start.getTime()) / (1000 * 60 * 60 * 24));
      HEAP32[(((tmPtr)+(28))>>2)]=yday
      HEAP32[(((tmPtr)+(36))>>2)]=start.getTimezoneOffset() * 60
      var dst = Number(start.getTimezoneOffset() != date.getTimezoneOffset());
      HEAP32[(((tmPtr)+(32))>>2)]=dst
      HEAP32[(((tmPtr)+(40))>>2)]=___tm_timezone
      return tmPtr;
    }function _localtime(time) {
      return _localtime_r(time, ___tm_current);
    }
  var _llvm_memset_p0i8_i64=_memset;
  function _gmtime_r(time, tmPtr) {
      var date = new Date(HEAP32[((time)>>2)]*1000);
      HEAP32[((tmPtr)>>2)]=date.getUTCSeconds()
      HEAP32[(((tmPtr)+(4))>>2)]=date.getUTCMinutes()
      HEAP32[(((tmPtr)+(8))>>2)]=date.getUTCHours()
      HEAP32[(((tmPtr)+(12))>>2)]=date.getUTCDate()
      HEAP32[(((tmPtr)+(16))>>2)]=date.getUTCMonth()
      HEAP32[(((tmPtr)+(20))>>2)]=date.getUTCFullYear()-1900
      HEAP32[(((tmPtr)+(24))>>2)]=date.getUTCDay()
      HEAP32[(((tmPtr)+(36))>>2)]=0
      HEAP32[(((tmPtr)+(32))>>2)]=0
      var start = new Date(date); // define date using UTC, start from Jan 01 00:00:00 UTC
      start.setUTCDate(1);
      start.setUTCMonth(0);
      start.setUTCHours(0);
      start.setUTCMinutes(0);
      start.setUTCSeconds(0);
      start.setUTCMilliseconds(0);
      var yday = Math.floor((date.getTime() - start.getTime()) / (1000 * 60 * 60 * 24));
      HEAP32[(((tmPtr)+(28))>>2)]=yday
      HEAP32[(((tmPtr)+(40))>>2)]=___tm_timezone
      return tmPtr;
    }function _gmtime(time) {
      return _gmtime_r(time, ___tm_current);
    }
  Module["_memcpy"] = _memcpy;var _llvm_memcpy_p0i8_p0i8_i32=_memcpy;
  function __isLeapYear(year) {
        return year%4 === 0 && (year%100 !== 0 || year%400 === 0);
    }
  function __arraySum(array, index) {
      var sum = 0;
      for (var i = 0; i <= index; sum += array[i++]);
      return sum;
    }
  var __MONTH_DAYS_LEAP=[31,29,31,30,31,30,31,31,30,31,30,31];
  var __MONTH_DAYS_REGULAR=[31,28,31,30,31,30,31,31,30,31,30,31];function __addDays(date, days) {
      var newDate = new Date(date.getTime());
      while(days > 0) {
        var leap = __isLeapYear(newDate.getFullYear());
        var currentMonth = newDate.getMonth();
        var daysInCurrentMonth = (leap ? __MONTH_DAYS_LEAP : __MONTH_DAYS_REGULAR)[currentMonth];
        if (days > daysInCurrentMonth-newDate.getDate()) {
          // we spill over to next month
          days -= (daysInCurrentMonth-newDate.getDate()+1);
          newDate.setDate(1);
          if (currentMonth < 11) {
            newDate.setMonth(currentMonth+1)
          } else {
            newDate.setMonth(0);
            newDate.setFullYear(newDate.getFullYear()+1);
          }
        } else {
          // we stay in current month 
          newDate.setDate(newDate.getDate()+days);
          return newDate;
        }
      }
      return newDate;
    }function _strftime(s, maxsize, format, tm) {
      // size_t strftime(char *restrict s, size_t maxsize, const char *restrict format, const struct tm *restrict timeptr);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/strftime.html
      var date = {
        tm_sec: HEAP32[((tm)>>2)],
        tm_min: HEAP32[(((tm)+(4))>>2)],
        tm_hour: HEAP32[(((tm)+(8))>>2)],
        tm_mday: HEAP32[(((tm)+(12))>>2)],
        tm_mon: HEAP32[(((tm)+(16))>>2)],
        tm_year: HEAP32[(((tm)+(20))>>2)],
        tm_wday: HEAP32[(((tm)+(24))>>2)],
        tm_yday: HEAP32[(((tm)+(28))>>2)],
        tm_isdst: HEAP32[(((tm)+(32))>>2)]
      };
      var pattern = Pointer_stringify(format);
      // expand format
      var EXPANSION_RULES_1 = {
        '%c': '%a %b %d %H:%M:%S %Y',     // Replaced by the locale's appropriate date and time representation - e.g., Mon Aug  3 14:02:01 2013
        '%D': '%m/%d/%y',                 // Equivalent to %m / %d / %y
        '%F': '%Y-%m-%d',                 // Equivalent to %Y - %m - %d
        '%h': '%b',                       // Equivalent to %b
        '%r': '%I:%M:%S %p',              // Replaced by the time in a.m. and p.m. notation
        '%R': '%H:%M',                    // Replaced by the time in 24-hour notation
        '%T': '%H:%M:%S',                 // Replaced by the time
        '%x': '%m/%d/%y',                 // Replaced by the locale's appropriate date representation
        '%X': '%H:%M:%S',                 // Replaced by the locale's appropriate date representation
      };
      for (var rule in EXPANSION_RULES_1) {
        pattern = pattern.replace(new RegExp(rule, 'g'), EXPANSION_RULES_1[rule]);
      }
      var WEEKDAYS = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
      var MONTHS = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
      var leadingSomething = function(value, digits, character) {
        var str = typeof value === 'number' ? value.toString() : (value || '');
        while (str.length < digits) {
          str = character[0]+str;
        }
        return str;
      };
      var leadingNulls = function(value, digits) {
        return leadingSomething(value, digits, '0');
      };
      var compareByDay = function(date1, date2) {
        var sgn = function(value) {
          return value < 0 ? -1 : (value > 0 ? 1 : 0);
        };
        var compare;
        if ((compare = sgn(date1.getFullYear()-date2.getFullYear())) === 0) {
          if ((compare = sgn(date1.getMonth()-date2.getMonth())) === 0) {
            compare = sgn(date1.getDate()-date2.getDate());
          }
        }
        return compare;
      };
      var getFirstWeekStartDate = function(janFourth) {
          switch (janFourth.getDay()) {
            case 0: // Sunday
              return new Date(janFourth.getFullYear()-1, 11, 29);
            case 1: // Monday
              return janFourth;
            case 2: // Tuesday
              return new Date(janFourth.getFullYear(), 0, 3);
            case 3: // Wednesday
              return new Date(janFourth.getFullYear(), 0, 2);
            case 4: // Thursday
              return new Date(janFourth.getFullYear(), 0, 1);
            case 5: // Friday
              return new Date(janFourth.getFullYear()-1, 11, 31);
            case 6: // Saturday
              return new Date(janFourth.getFullYear()-1, 11, 30);
          }
      };
      var getWeekBasedYear = function(date) {
          var thisDate = __addDays(new Date(date.tm_year+1900, 0, 1), date.tm_yday);
          var janFourthThisYear = new Date(thisDate.getFullYear(), 0, 4);
          var janFourthNextYear = new Date(thisDate.getFullYear()+1, 0, 4);
          var firstWeekStartThisYear = getFirstWeekStartDate(janFourthThisYear);
          var firstWeekStartNextYear = getFirstWeekStartDate(janFourthNextYear);
          if (compareByDay(firstWeekStartThisYear, thisDate) <= 0) {
            // this date is after the start of the first week of this year
            if (compareByDay(firstWeekStartNextYear, thisDate) <= 0) {
              return thisDate.getFullYear()+1;
            } else {
              return thisDate.getFullYear();
            }
          } else { 
            return thisDate.getFullYear()-1;
          }
      };
      var EXPANSION_RULES_2 = {
        '%a': function(date) {
          return WEEKDAYS[date.tm_wday].substring(0,3);
        },
        '%A': function(date) {
          return WEEKDAYS[date.tm_wday];
        },
        '%b': function(date) {
          return MONTHS[date.tm_mon].substring(0,3);
        },
        '%B': function(date) {
          return MONTHS[date.tm_mon];
        },
        '%C': function(date) {
          var year = date.tm_year+1900;
          return leadingNulls(Math.floor(year/100),2);
        },
        '%d': function(date) {
          return leadingNulls(date.tm_mday, 2);
        },
        '%e': function(date) {
          return leadingSomething(date.tm_mday, 2, ' ');
        },
        '%g': function(date) {
          // %g, %G, and %V give values according to the ISO 8601:2000 standard week-based year. 
          // In this system, weeks begin on a Monday and week 1 of the year is the week that includes 
          // January 4th, which is also the week that includes the first Thursday of the year, and 
          // is also the first week that contains at least four days in the year. 
          // If the first Monday of January is the 2nd, 3rd, or 4th, the preceding days are part of 
          // the last week of the preceding year; thus, for Saturday 2nd January 1999, 
          // %G is replaced by 1998 and %V is replaced by 53. If December 29th, 30th, 
          // or 31st is a Monday, it and any following days are part of week 1 of the following year. 
          // Thus, for Tuesday 30th December 1997, %G is replaced by 1998 and %V is replaced by 01.
          return getWeekBasedYear(date).toString().substring(2);
        },
        '%G': function(date) {
          return getWeekBasedYear(date);
        },
        '%H': function(date) {
          return leadingNulls(date.tm_hour, 2);
        },
        '%I': function(date) {
          return leadingNulls(date.tm_hour < 13 ? date.tm_hour : date.tm_hour-12, 2);
        },
        '%j': function(date) {
          // Day of the year (001-366)
          return leadingNulls(date.tm_mday+__arraySum(__isLeapYear(date.tm_year+1900) ? __MONTH_DAYS_LEAP : __MONTH_DAYS_REGULAR, date.tm_mon-1), 3);
        },
        '%m': function(date) {
          return leadingNulls(date.tm_mon+1, 2);
        },
        '%M': function(date) {
          return leadingNulls(date.tm_min, 2);
        },
        '%n': function() {
          return '\n';
        },
        '%p': function(date) {
          if (date.tm_hour > 0 && date.tm_hour < 13) {
            return 'AM';
          } else {
            return 'PM';
          }
        },
        '%S': function(date) {
          return leadingNulls(date.tm_sec, 2);
        },
        '%t': function() {
          return '\t';
        },
        '%u': function(date) {
          var day = new Date(date.tm_year+1900, date.tm_mon+1, date.tm_mday, 0, 0, 0, 0);
          return day.getDay() || 7;
        },
        '%U': function(date) {
          // Replaced by the week number of the year as a decimal number [00,53]. 
          // The first Sunday of January is the first day of week 1; 
          // days in the new year before this are in week 0. [ tm_year, tm_wday, tm_yday]
          var janFirst = new Date(date.tm_year+1900, 0, 1);
          var firstSunday = janFirst.getDay() === 0 ? janFirst : __addDays(janFirst, 7-janFirst.getDay());
          var endDate = new Date(date.tm_year+1900, date.tm_mon, date.tm_mday);
          // is target date after the first Sunday?
          if (compareByDay(firstSunday, endDate) < 0) {
            // calculate difference in days between first Sunday and endDate
            var februaryFirstUntilEndMonth = __arraySum(__isLeapYear(endDate.getFullYear()) ? __MONTH_DAYS_LEAP : __MONTH_DAYS_REGULAR, endDate.getMonth()-1)-31;
            var firstSundayUntilEndJanuary = 31-firstSunday.getDate();
            var days = firstSundayUntilEndJanuary+februaryFirstUntilEndMonth+endDate.getDate();
            return leadingNulls(Math.ceil(days/7), 2);
          }
          return compareByDay(firstSunday, janFirst) === 0 ? '01': '00';
        },
        '%V': function(date) {
          // Replaced by the week number of the year (Monday as the first day of the week) 
          // as a decimal number [01,53]. If the week containing 1 January has four 
          // or more days in the new year, then it is considered week 1. 
          // Otherwise, it is the last week of the previous year, and the next week is week 1. 
          // Both January 4th and the first Thursday of January are always in week 1. [ tm_year, tm_wday, tm_yday]
          var janFourthThisYear = new Date(date.tm_year+1900, 0, 4);
          var janFourthNextYear = new Date(date.tm_year+1901, 0, 4);
          var firstWeekStartThisYear = getFirstWeekStartDate(janFourthThisYear);
          var firstWeekStartNextYear = getFirstWeekStartDate(janFourthNextYear);
          var endDate = __addDays(new Date(date.tm_year+1900, 0, 1), date.tm_yday);
          if (compareByDay(endDate, firstWeekStartThisYear) < 0) {
            // if given date is before this years first week, then it belongs to the 53rd week of last year
            return '53';
          } 
          if (compareByDay(firstWeekStartNextYear, endDate) <= 0) {
            // if given date is after next years first week, then it belongs to the 01th week of next year
            return '01';
          }
          // given date is in between CW 01..53 of this calendar year
          var daysDifference;
          if (firstWeekStartThisYear.getFullYear() < date.tm_year+1900) {
            // first CW of this year starts last year
            daysDifference = date.tm_yday+32-firstWeekStartThisYear.getDate()
          } else {
            // first CW of this year starts this year
            daysDifference = date.tm_yday+1-firstWeekStartThisYear.getDate();
          }
          return leadingNulls(Math.ceil(daysDifference/7), 2);
        },
        '%w': function(date) {
          var day = new Date(date.tm_year+1900, date.tm_mon+1, date.tm_mday, 0, 0, 0, 0);
          return day.getDay();
        },
        '%W': function(date) {
          // Replaced by the week number of the year as a decimal number [00,53]. 
          // The first Monday of January is the first day of week 1; 
          // days in the new year before this are in week 0. [ tm_year, tm_wday, tm_yday]
          var janFirst = new Date(date.tm_year, 0, 1);
          var firstMonday = janFirst.getDay() === 1 ? janFirst : __addDays(janFirst, janFirst.getDay() === 0 ? 1 : 7-janFirst.getDay()+1);
          var endDate = new Date(date.tm_year+1900, date.tm_mon, date.tm_mday);
          // is target date after the first Monday?
          if (compareByDay(firstMonday, endDate) < 0) {
            var februaryFirstUntilEndMonth = __arraySum(__isLeapYear(endDate.getFullYear()) ? __MONTH_DAYS_LEAP : __MONTH_DAYS_REGULAR, endDate.getMonth()-1)-31;
            var firstMondayUntilEndJanuary = 31-firstMonday.getDate();
            var days = firstMondayUntilEndJanuary+februaryFirstUntilEndMonth+endDate.getDate();
            return leadingNulls(Math.ceil(days/7), 2);
          }
          return compareByDay(firstMonday, janFirst) === 0 ? '01': '00';
        },
        '%y': function(date) {
          // Replaced by the last two digits of the year as a decimal number [00,99]. [ tm_year]
          return (date.tm_year+1900).toString().substring(2);
        },
        '%Y': function(date) {
          // Replaced by the year as a decimal number (for example, 1997). [ tm_year]
          return date.tm_year+1900;
        },
        '%z': function(date) {
          // Replaced by the offset from UTC in the ISO 8601:2000 standard format ( +hhmm or -hhmm ),
          // or by no characters if no timezone is determinable. 
          // For example, "-0430" means 4 hours 30 minutes behind UTC (west of Greenwich). 
          // If tm_isdst is zero, the standard time offset is used. 
          // If tm_isdst is greater than zero, the daylight savings time offset is used. 
          // If tm_isdst is negative, no characters are returned. 
          // FIXME: we cannot determine time zone (or can we?)
          return '';
        },
        '%Z': function(date) {
          // Replaced by the timezone name or abbreviation, or by no bytes if no timezone information exists. [ tm_isdst]
          // FIXME: we cannot determine time zone (or can we?)
          return '';
        },
        '%%': function() {
          return '%';
        }
      };
      for (var rule in EXPANSION_RULES_2) {
        if (pattern.indexOf(rule) >= 0) {
          pattern = pattern.replace(new RegExp(rule, 'g'), EXPANSION_RULES_2[rule](date));
        }
      }
      var bytes = intArrayFromString(pattern, false);
      if (bytes.length > maxsize) {
        return 0;
      } 
      writeArrayToMemory(bytes, s);
      return bytes.length-1;
    }
  function __getFloat(text) {
      return /^[+-]?[0-9]*\.?[0-9]+([eE][+-]?[0-9]+)?/.exec(text);
    }function __scanString(format, get, unget, varargs) {
      if (!__scanString.whiteSpace) {
        __scanString.whiteSpace = {};
        __scanString.whiteSpace[32] = 1;
        __scanString.whiteSpace[9] = 1;
        __scanString.whiteSpace[10] = 1;
        __scanString.whiteSpace[11] = 1;
        __scanString.whiteSpace[12] = 1;
        __scanString.whiteSpace[13] = 1;
      }
      // Supports %x, %4x, %d.%d, %lld, %s, %f, %lf.
      // TODO: Support all format specifiers.
      format = Pointer_stringify(format);
      var soFar = 0;
      if (format.indexOf('%n') >= 0) {
        // need to track soFar
        var _get = get;
        get = function() {
          soFar++;
          return _get();
        }
        var _unget = unget;
        unget = function() {
          soFar--;
          return _unget();
        }
      }
      var formatIndex = 0;
      var argsi = 0;
      var fields = 0;
      var argIndex = 0;
      var next;
      mainLoop:
      for (var formatIndex = 0; formatIndex < format.length;) {
        if (format[formatIndex] === '%' && format[formatIndex+1] == 'n') {
          var argPtr = HEAP32[(((varargs)+(argIndex))>>2)];
          argIndex += Runtime.getAlignSize('void*', null, true);
          HEAP32[((argPtr)>>2)]=soFar;
          formatIndex += 2;
          continue;
        }
        if (format[formatIndex] === '%') {
          var nextC = format.indexOf('c', formatIndex+1);
          if (nextC > 0) {
            var maxx = 1;
            if (nextC > formatIndex+1) {
              var sub = format.substring(formatIndex+1, nextC);
              maxx = parseInt(sub);
              if (maxx != sub) maxx = 0;
            }
            if (maxx) {
              var argPtr = HEAP32[(((varargs)+(argIndex))>>2)];
              argIndex += Runtime.getAlignSize('void*', null, true);
              fields++;
              for (var i = 0; i < maxx; i++) {
                next = get();
                HEAP8[((argPtr++)|0)]=next;
              }
              formatIndex += nextC - formatIndex + 1;
              continue;
            }
          }
        }
        // handle %[...]
        if (format[formatIndex] === '%' && format.indexOf('[', formatIndex+1) > 0) {
          var match = /\%([0-9]*)\[(\^)?(\]?[^\]]*)\]/.exec(format.substring(formatIndex));
          if (match) {
            var maxNumCharacters = parseInt(match[1]) || Infinity;
            var negateScanList = (match[2] === '^');
            var scanList = match[3];
            // expand "middle" dashs into character sets
            var middleDashMatch;
            while ((middleDashMatch = /([^\-])\-([^\-])/.exec(scanList))) {
              var rangeStartCharCode = middleDashMatch[1].charCodeAt(0);
              var rangeEndCharCode = middleDashMatch[2].charCodeAt(0);
              for (var expanded = ''; rangeStartCharCode <= rangeEndCharCode; expanded += String.fromCharCode(rangeStartCharCode++));
              scanList = scanList.replace(middleDashMatch[1] + '-' + middleDashMatch[2], expanded);
            }
            var argPtr = HEAP32[(((varargs)+(argIndex))>>2)];
            argIndex += Runtime.getAlignSize('void*', null, true);
            fields++;
            for (var i = 0; i < maxNumCharacters; i++) {
              next = get();
              if (negateScanList) {
                if (scanList.indexOf(String.fromCharCode(next)) < 0) {
                  HEAP8[((argPtr++)|0)]=next;
                } else {
                  unget();
                  break;
                }
              } else {
                if (scanList.indexOf(String.fromCharCode(next)) >= 0) {
                  HEAP8[((argPtr++)|0)]=next;
                } else {
                  unget();
                  break;
                }
              }
            }
            // write out null-terminating character
            HEAP8[((argPtr++)|0)]=0;
            formatIndex += match[0].length;
            continue;
          }
        }      
        // remove whitespace
        while (1) {
          next = get();
          if (next == 0) return fields;
          if (!(next in __scanString.whiteSpace)) break;
        }
        unget();
        if (format[formatIndex] === '%') {
          formatIndex++;
          var suppressAssignment = false;
          if (format[formatIndex] == '*') {
            suppressAssignment = true;
            formatIndex++;
          }
          var maxSpecifierStart = formatIndex;
          while (format[formatIndex].charCodeAt(0) >= 48 &&
                 format[formatIndex].charCodeAt(0) <= 57) {
            formatIndex++;
          }
          var max_;
          if (formatIndex != maxSpecifierStart) {
            max_ = parseInt(format.slice(maxSpecifierStart, formatIndex), 10);
          }
          var long_ = false;
          var half = false;
          var longLong = false;
          if (format[formatIndex] == 'l') {
            long_ = true;
            formatIndex++;
            if (format[formatIndex] == 'l') {
              longLong = true;
              formatIndex++;
            }
          } else if (format[formatIndex] == 'h') {
            half = true;
            formatIndex++;
          }
          var type = format[formatIndex];
          formatIndex++;
          var curr = 0;
          var buffer = [];
          // Read characters according to the format. floats are trickier, they may be in an unfloat state in the middle, then be a valid float later
          if (type == 'f' || type == 'e' || type == 'g' ||
              type == 'F' || type == 'E' || type == 'G') {
            next = get();
            while (next > 0 && (!(next in __scanString.whiteSpace)))  {
              buffer.push(String.fromCharCode(next));
              next = get();
            }
            var m = __getFloat(buffer.join(''));
            var last = m ? m[0].length : 0;
            for (var i = 0; i < buffer.length - last + 1; i++) {
              unget();
            }
            buffer.length = last;
          } else {
            next = get();
            var first = true;
            // Strip the optional 0x prefix for %x.
            if ((type == 'x' || type == 'X') && (next == 48)) {
              var peek = get();
              if (peek == 120 || peek == 88) {
                next = get();
              } else {
                unget();
              }
            }
            while ((curr < max_ || isNaN(max_)) && next > 0) {
              if (!(next in __scanString.whiteSpace) && // stop on whitespace
                  (type == 's' ||
                   ((type === 'd' || type == 'u' || type == 'i') && ((next >= 48 && next <= 57) ||
                                                                     (first && next == 45))) ||
                   ((type === 'x' || type === 'X') && (next >= 48 && next <= 57 ||
                                     next >= 97 && next <= 102 ||
                                     next >= 65 && next <= 70))) &&
                  (formatIndex >= format.length || next !== format[formatIndex].charCodeAt(0))) { // Stop when we read something that is coming up
                buffer.push(String.fromCharCode(next));
                next = get();
                curr++;
                first = false;
              } else {
                break;
              }
            }
            unget();
          }
          if (buffer.length === 0) return 0;  // Failure.
          if (suppressAssignment) continue;
          var text = buffer.join('');
          var argPtr = HEAP32[(((varargs)+(argIndex))>>2)];
          argIndex += Runtime.getAlignSize('void*', null, true);
          switch (type) {
            case 'd': case 'u': case 'i':
              if (half) {
                HEAP16[((argPtr)>>1)]=parseInt(text, 10);
              } else if (longLong) {
                (tempI64 = [parseInt(text, 10)>>>0,(tempDouble=parseInt(text, 10),(+(Math_abs(tempDouble))) >= (+1) ? (tempDouble > (+0) ? ((Math_min((+(Math_floor((tempDouble)/(+4294967296)))), (+4294967295)))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/(+4294967296))))))>>>0) : 0)],HEAP32[((argPtr)>>2)]=tempI64[0],HEAP32[(((argPtr)+(4))>>2)]=tempI64[1]);
              } else {
                HEAP32[((argPtr)>>2)]=parseInt(text, 10);
              }
              break;
            case 'X':
            case 'x':
              HEAP32[((argPtr)>>2)]=parseInt(text, 16)
              break;
            case 'F':
            case 'f':
            case 'E':
            case 'e':
            case 'G':
            case 'g':
            case 'E':
              // fallthrough intended
              if (long_) {
                HEAPF64[((argPtr)>>3)]=parseFloat(text)
              } else {
                HEAPF32[((argPtr)>>2)]=parseFloat(text)
              }
              break;
            case 's':
              var array = intArrayFromString(text);
              for (var j = 0; j < array.length; j++) {
                HEAP8[(((argPtr)+(j))|0)]=array[j]
              }
              break;
          }
          fields++;
        } else if (format[formatIndex].charCodeAt(0) in __scanString.whiteSpace) {
          next = get();
          while (next in __scanString.whiteSpace) {
            if (next <= 0) break mainLoop;  // End of input.
            next = get();
          }
          unget(next);
          formatIndex++;
        } else {
          // Not a specifier.
          next = get();
          if (format[formatIndex].charCodeAt(0) !== next) {
            unget(next);
            break mainLoop;
          }
          formatIndex++;
        }
      }
      return fields;
    }function _sscanf(s, format, varargs) {
      // int sscanf(const char *restrict s, const char *restrict format, ... );
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/scanf.html
      var index = 0;
      var get = function() { return HEAP8[(((s)+(index++))|0)]; };
      var unget = function() { index--; };
      return __scanString(format, get, unget, varargs);
    }
  function _isspace(chr) {
      return (chr == 32) || (chr >= 9 && chr <= 13);
    }function __parseInt(str, endptr, base, min, max, bits, unsign) {
      // Skip space.
      while (_isspace(HEAP8[(str)])) str++;
      // Check for a plus/minus sign.
      var multiplier = 1;
      if (HEAP8[(str)] == 45) {
        multiplier = -1;
        str++;
      } else if (HEAP8[(str)] == 43) {
        str++;
      }
      // Find base.
      var finalBase = base;
      if (!finalBase) {
        if (HEAP8[(str)] == 48) {
          if (HEAP8[((str+1)|0)] == 120 ||
              HEAP8[((str+1)|0)] == 88) {
            finalBase = 16;
            str += 2;
          } else {
            finalBase = 8;
            str++;
          }
        }
      } else if (finalBase==16) {
        if (HEAP8[(str)] == 48) {
          if (HEAP8[((str+1)|0)] == 120 ||
              HEAP8[((str+1)|0)] == 88) {
            str += 2;
          }
        }
      }
      if (!finalBase) finalBase = 10;
      // Get digits.
      var chr;
      var ret = 0;
      while ((chr = HEAP8[(str)]) != 0) {
        var digit = parseInt(String.fromCharCode(chr), finalBase);
        if (isNaN(digit)) {
          break;
        } else {
          ret = ret * finalBase + digit;
          str++;
        }
      }
      // Apply sign.
      ret *= multiplier;
      // Set end pointer.
      if (endptr) {
        HEAP32[((endptr)>>2)]=str
      }
      // Unsign if needed.
      if (unsign) {
        if (Math.abs(ret) > max) {
          ret = max;
          ___setErrNo(ERRNO_CODES.ERANGE);
        } else {
          ret = unSign(ret, bits);
        }
      }
      // Validate range.
      if (ret > max || ret < min) {
        ret = ret > max ? max : min;
        ___setErrNo(ERRNO_CODES.ERANGE);
      }
      if (bits == 64) {
        return ((asm["setTempRet0"]((tempDouble=ret,(+(Math_abs(tempDouble))) >= (+1) ? (tempDouble > (+0) ? ((Math_min((+(Math_floor((tempDouble)/(+4294967296)))), (+4294967295)))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/(+4294967296))))))>>>0) : 0)),ret>>>0)|0);
      }
      return ret;
    }function _strtol(str, endptr, base) {
      return __parseInt(str, endptr, base, -2147483648, 2147483647, 32);  // LONG_MIN, LONG_MAX.
    }
  function _strrchr(ptr, chr) {
      var ptr2 = ptr + _strlen(ptr);
      do {
        if (HEAP8[(ptr2)] == chr) return ptr2;
        ptr2--;
      } while (ptr2 >= ptr);
      return 0;
    }
  function _abort() {
      Module['abort']();
    }
  function _signal(sig, func) {
      // TODO
      return 0;
    }
  var _environ=allocate(1, "i32*", ALLOC_STATIC);var ___environ=_environ;function ___buildEnvironment(env) {
      // WARNING: Arbitrary limit!
      var MAX_ENV_VALUES = 64;
      var TOTAL_ENV_SIZE = 1024;
      // Statically allocate memory for the environment.
      var poolPtr;
      var envPtr;
      if (!___buildEnvironment.called) {
        ___buildEnvironment.called = true;
        // Set default values. Use string keys for Closure Compiler compatibility.
        ENV['USER'] = 'root';
        ENV['PATH'] = '/';
        ENV['PWD'] = '/';
        ENV['HOME'] = '/home/emscripten';
        ENV['LANG'] = 'en_US.UTF-8';
        ENV['_'] = './this.program';
        // Allocate memory.
        poolPtr = allocate(TOTAL_ENV_SIZE, 'i8', ALLOC_STATIC);
        envPtr = allocate(MAX_ENV_VALUES * 4,
                          'i8*', ALLOC_STATIC);
        HEAP32[((envPtr)>>2)]=poolPtr
        HEAP32[((_environ)>>2)]=envPtr;
      } else {
        envPtr = HEAP32[((_environ)>>2)];
        poolPtr = HEAP32[((envPtr)>>2)];
      }
      // Collect key=value lines.
      var strings = [];
      var totalSize = 0;
      for (var key in env) {
        if (typeof env[key] === 'string') {
          var line = key + '=' + env[key];
          strings.push(line);
          totalSize += line.length;
        }
      }
      if (totalSize > TOTAL_ENV_SIZE) {
        throw new Error('Environment size exceeded TOTAL_ENV_SIZE!');
      }
      // Make new.
      var ptrSize = 4;
      for (var i = 0; i < strings.length; i++) {
        var line = strings[i];
        for (var j = 0; j < line.length; j++) {
          HEAP8[(((poolPtr)+(j))|0)]=line.charCodeAt(j);
        }
        HEAP8[(((poolPtr)+(j))|0)]=0;
        HEAP32[(((envPtr)+(i * ptrSize))>>2)]=poolPtr;
        poolPtr += line.length + 1;
      }
      HEAP32[(((envPtr)+(strings.length * ptrSize))>>2)]=0;
    }var ENV={};function _getenv(name) {
      // char *getenv(const char *name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/getenv.html
      if (name === 0) return 0;
      name = Pointer_stringify(name);
      if (!ENV.hasOwnProperty(name)) return 0;
      if (_getenv.ret) _free(_getenv.ret);
      _getenv.ret = allocate(intArrayFromString(ENV[name]), 'i8', ALLOC_NORMAL);
      return _getenv.ret;
    }
  function _atoi(ptr) {
      return _strtol(ptr, null, 10);
    }
  function _fputc(c, stream) {
      // int fputc(int c, FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fputc.html
      var chr = unSign(c & 0xFF);
      HEAP8[((_fputc.ret)|0)]=chr
      var ret = _write(stream, _fputc.ret, 1);
      if (ret == -1) {
        var streamObj = FS.getStream(stream);
        if (streamObj) streamObj.error = true;
        return -1;
      } else {
        return chr;
      }
    }function _putchar(c) {
      // int putchar(int c);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/putchar.html
      return _fputc(c, HEAP32[((_stdout)>>2)]);
    }
  function _llvm_lifetime_start() {}
  function _llvm_lifetime_end() {}
  function _tcsetattr(fildes, optional_actions, termios_p) {
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/tcsetattr.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      if (!stream.tty) {
        ___setErrNo(ERRNO_CODES.ENOTTY);
        return -1;
      }
      return 0;
    }
  function _isatty(fildes) {
      // int isatty(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/isatty.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return 0;
      }
      // HACK - implement tcgetattr
      if (!stream.tty) {
        ___setErrNo(ERRNO_CODES.ENOTTY);
        return 0;
      }
      return 1;
    }
  function _tcgetattr(fildes, termios_p) {
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/tcgetattr.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      if (!stream.tty) {
        ___setErrNo(ERRNO_CODES.ENOTTY);
        return -1;
      }
      return 0;
    }
  function _setvbuf(stream, buf, type, size) {
      // int setvbuf(FILE *restrict stream, char *restrict buf, int type, size_t size);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/setvbuf.html
      // TODO: Implement custom buffering.
      return 0;
    }
  function _open(path, oflag, varargs) {
      // int open(const char *path, int oflag, ...);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/open.html
      var mode = HEAP32[((varargs)>>2)];
      path = Pointer_stringify(path);
      try {
        var stream = FS.open(path, oflag, mode);
        return stream.fd;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fopen(filename, mode) {
      // FILE *fopen(const char *restrict filename, const char *restrict mode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fopen.html
      var flags;
      mode = Pointer_stringify(mode);
      if (mode[0] == 'r') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 0;
        }
      } else if (mode[0] == 'w') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 64;
        flags |= 512;
      } else if (mode[0] == 'a') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 64;
        flags |= 1024;
      } else {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return 0;
      }
      var ret = _open(filename, flags, allocate([0x1FF, 0, 0, 0], 'i32', ALLOC_STACK));  // All creation permissions.
      return (ret == -1) ? 0 : ret;
    }
  function _fputs(s, stream) {
      // int fputs(const char *restrict s, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fputs.html
      return _write(stream, s, _strlen(s));
    }function _puts(s) {
      // int puts(const char *s);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/puts.html
      // NOTE: puts() always writes an extra newline.
      var stdout = HEAP32[((_stdout)>>2)];
      var ret = _fputs(s, stdout);
      if (ret < 0) {
        return ret;
      } else {
        var newlineRet = _fputc(10, stdout);
        return (newlineRet < 0) ? -1 : ret + 1;
      }
    }
  function _strerror_r(errnum, strerrbuf, buflen) {
      if (errnum in ERRNO_MESSAGES) {
        if (ERRNO_MESSAGES[errnum].length > buflen - 1) {
          return ___setErrNo(ERRNO_CODES.ERANGE);
        } else {
          var msg = ERRNO_MESSAGES[errnum];
          for (var i = 0; i < msg.length; i++) {
            HEAP8[(((strerrbuf)+(i))|0)]=msg.charCodeAt(i)
          }
          HEAP8[(((strerrbuf)+(i))|0)]=0
          return 0;
        }
      } else {
        return ___setErrNo(ERRNO_CODES.EINVAL);
      }
    }function _strerror(errnum) {
      if (!_strerror.buffer) _strerror.buffer = _malloc(256);
      _strerror_r(errnum, _strerror.buffer, 256);
      return _strerror.buffer;
    }
  function ___errno_location() {
      return ___errno_state;
    }function _perror(s) {
      // void perror(const char *s);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/perror.html
      var stdout = HEAP32[((_stdout)>>2)];
      if (s) {
        _fputs(s, stdout);
        _fputc(58, stdout);
        _fputc(32, stdout);
      }
      var errnum = HEAP32[((___errno_location())>>2)];
      _puts(_strerror(errnum));
    }
  var _log=Math_log;
  var _llvm_va_start=undefined;
  function _vsnprintf(s, n, format, va_arg) {
      return _snprintf(s, n, format, HEAP32[((va_arg)>>2)]);
    }
  function _llvm_va_end() {}
  function _rint(x) {
      if (Math.abs(x % 1) !== 0.5) return Math.round(x);
      return x + x % 2 + ((x < 0) ? 1 : -1);
    }var _lrintf=_rint;
  function _log2(x) {
      return Math.log(x) / Math.LN2;
    }
  var _lrint=_rint;
  function _ungetc(c, stream) {
      // int ungetc(int c, FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ungetc.html
      stream = FS.getStream(stream);
      if (!stream) {
        return -1;
      }
      if (c === -1) {
        // do nothing for EOF character
        return c;
      }
      c = unSign(c & 0xFF);
      stream.ungotten.push(c);
      stream.eof = false;
      return c;
    }function _fscanf(stream, format, varargs) {
      // int fscanf(FILE *restrict stream, const char *restrict format, ... );
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/scanf.html
      var streamObj = FS.getStream(stream);
      if (!streamObj) {
        return -1;
      }
      var buffer = [];
      var get = function() {
        var c = _fgetc(stream);
        buffer.push(c);
        return c;
      };
      var unget = function() {
        _ungetc(buffer.pop(), stream);
      };
      return __scanString(format, get, unget, varargs);
    }function _scanf(format, varargs) {
      // int scanf(const char *restrict format, ... );
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/scanf.html
      var stdin = HEAP32[((_stdin)>>2)];
      return _fscanf(stdin, format, varargs);
    }
  var ___DEFAULT_POLLMASK=5;function _select(nfds, readfds, writefds, exceptfds, timeout) {
      // readfds are supported,
      // writefds checks socket open status
      // exceptfds not supported
      // timeout is always 0 - fully async
      assert(nfds <= 64, 'nfds must be less than or equal to 64');  // fd sets have 64 bits
      assert(!exceptfds, 'exceptfds not supported');
      var total = 0;
      var srcReadLow = (readfds ? HEAP32[((readfds)>>2)] : 0),
          srcReadHigh = (readfds ? HEAP32[(((readfds)+(4))>>2)] : 0);
      var srcWriteLow = (writefds ? HEAP32[((writefds)>>2)] : 0),
          srcWriteHigh = (writefds ? HEAP32[(((writefds)+(4))>>2)] : 0);
      var srcExceptLow = (exceptfds ? HEAP32[((exceptfds)>>2)] : 0),
          srcExceptHigh = (exceptfds ? HEAP32[(((exceptfds)+(4))>>2)] : 0);
      var dstReadLow = 0,
          dstReadHigh = 0;
      var dstWriteLow = 0,
          dstWriteHigh = 0;
      var dstExceptLow = 0,
          dstExceptHigh = 0;
      var allLow = (readfds ? HEAP32[((readfds)>>2)] : 0) |
                   (writefds ? HEAP32[((writefds)>>2)] : 0) |
                   (exceptfds ? HEAP32[((exceptfds)>>2)] : 0);
      var allHigh = (readfds ? HEAP32[(((readfds)+(4))>>2)] : 0) |
                    (writefds ? HEAP32[(((writefds)+(4))>>2)] : 0) |
                    (exceptfds ? HEAP32[(((exceptfds)+(4))>>2)] : 0);
      function get(fd, low, high, val) {
        return (fd < 32 ? (low & val) : (high & val));
      }
      for (var fd = 0; fd < nfds; fd++) {
        var mask = 1 << (fd % 32);
        if (!(get(fd, allLow, allHigh, mask))) {
          continue;  // index isn't in the set
        }
        var stream = FS.getStream(fd);
        if (!stream) {
          ___setErrNo(ERRNO_CODES.EBADF);
          return -1;
        }
        var flags = ___DEFAULT_POLLMASK;
        if (stream.stream_ops.poll) {
          flags = stream.stream_ops.poll(stream);
        }
        if ((flags & 1) && get(fd, srcReadLow, srcReadHigh, mask)) {
          fd < 32 ? (dstReadLow = dstReadLow | mask) : (dstReadHigh = dstReadHigh | mask);
          total++;
        }
        if ((flags & 4) && get(fd, srcWriteLow, srcWriteHigh, mask)) {
          fd < 32 ? (dstWriteLow = dstWriteLow | mask) : (dstWriteHigh = dstWriteHigh | mask);
          total++;
        }
        if ((flags & 2) && get(fd, srcExceptLow, srcExceptHigh, mask)) {
          fd < 32 ? (dstExceptLow = dstExceptLow | mask) : (dstExceptHigh = dstExceptHigh | mask);
          total++;
        }
      }
      if (readfds) {
        HEAP32[((readfds)>>2)]=dstReadLow;
        HEAP32[(((readfds)+(4))>>2)]=dstReadHigh;
      }
      if (writefds) {
        HEAP32[((writefds)>>2)]=dstWriteLow;
        HEAP32[(((writefds)+(4))>>2)]=dstWriteHigh;
      }
      if (exceptfds) {
        HEAP32[((exceptfds)>>2)]=dstExceptLow;
        HEAP32[(((exceptfds)+(4))>>2)]=dstExceptHigh;
      }
      return total;
    }
  Module["_memcmp"] = _memcmp;
  function _qsort(base, num, size, cmp) {
      if (num == 0 || size == 0) return;
      // forward calls to the JavaScript sort method
      // first, sort the items logically
      var keys = [];
      for (var i = 0; i < num; i++) keys.push(i);
      keys.sort(function(a, b) {
        return Module['dynCall_iii'](cmp, base+a*size, base+b*size);
      });
      // apply the sort
      var temp = _malloc(num*size);
      _memcpy(temp, base, num*size);
      for (var i = 0; i < num; i++) {
        if (keys[i] == i) continue; // already in place
        _memcpy(base+i*size, temp+keys[i]*size, size);
      }
      _free(temp);
    }
  function _getrusage(resource, rlp) {
      // int getrusage(int resource, struct rusage *rlp);
      HEAP32[((rlp)>>2)]=1
      HEAP32[(((rlp)+(4))>>2)]=2
      HEAP32[(((rlp)+(8))>>2)]=3
      HEAP32[(((rlp)+(12))>>2)]=4
      return 0;
    }
  function _vfprintf(s, f, va_arg) {
      return _fprintf(s, f, HEAP32[((va_arg)>>2)]);
    }
  function __exit(status) {
      // void _exit(int status);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/exit.html
      Module.print('exit(' + status + ') called');
      Module['exit'](status);
    }function _exit(status) {
      __exit(status);
    }
  function _strstr(ptr1, ptr2) {
      var check = 0, start;
      do {
        if (!check) {
          start = ptr1;
          check = ptr2;
        }
        var curr1 = HEAP8[((ptr1++)|0)];
        var curr2 = HEAP8[((check++)|0)];
        if (curr2 == 0) return start;
        if (curr2 != curr1) {
          // rewind to one character after start, to find ez in eeez
          ptr1 = start + 1;
          check = 0;
        }
      } while (curr1);
      return 0;
    }
  function _setrlimit(resource, rlp) {
      // int setrlimit(int resource, const struct rlimit *rlp)
      return 0;
    }
  function _getchar() {
      // int getchar(void);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/getchar.html
      return _fgetc(HEAP32[((_stdin)>>2)]);
    }
  function _lseek(fildes, offset, whence) {
      // off_t lseek(int fildes, off_t offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/lseek.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        return FS.llseek(stream, offset, whence);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fseek(stream, offset, whence) {
      // int fseek(FILE *stream, long offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fseek.html
      var ret = _lseek(stream, offset, whence);
      if (ret == -1) {
        return -1;
      }
      stream = FS.getStream(stream);
      stream.eof = false;
      return 0;
    }
  function _ftell(stream) {
      // long ftell(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ftell.html
      stream = FS.getStream(stream);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      if (FS.isChrdev(stream.node.mode)) {
        ___setErrNo(ERRNO_CODES.ESPIPE);
        return -1;
      } else {
        return stream.position;
      }
    }
  function _ferror(stream) {
      // int ferror(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ferror.html
      stream = FS.getStream(stream);
      return Number(stream && stream.error);
    }
  function _llvm_va_copy(ppdest, ppsrc) {
      // copy the list start
      HEAP8[(ppdest)]=HEAP8[(ppsrc)];HEAP8[(((ppdest)+(1))|0)]=HEAP8[(((ppsrc)+(1))|0)];HEAP8[(((ppdest)+(2))|0)]=HEAP8[(((ppsrc)+(2))|0)];HEAP8[(((ppdest)+(3))|0)]=HEAP8[(((ppsrc)+(3))|0)];
      // copy the list's current offset (will be advanced with each call to va_arg)
      HEAP8[((ppdest+4)|0)]=HEAP8[((ppsrc+4)|0)];HEAP8[(((ppdest+4)+(1))|0)]=HEAP8[(((ppsrc+4)+(1))|0)];HEAP8[(((ppdest+4)+(2))|0)]=HEAP8[(((ppsrc+4)+(2))|0)];HEAP8[(((ppdest+4)+(3))|0)]=HEAP8[(((ppsrc+4)+(3))|0)];
    }
  Module["_memmove"] = _memmove;var _llvm_memmove_p0i8_p0i8_i32=_memmove;
  function _strspn(pstr, pset) {
      var str = pstr, set, strcurr, setcurr;
      while (1) {
        strcurr = HEAP8[(str)];
        if (!strcurr) return str - pstr;
        set = pset;
        while (1) {
          setcurr = HEAP8[(set)];
          if (!setcurr || setcurr == strcurr) break;
          set++;
        }
        if (!setcurr) return str - pstr;
        str++;
      }
    }
  function ___assert_fail(condition, filename, line, func) {
      ABORT = true;
      throw 'Assertion failed: ' + Pointer_stringify(condition) + ', at: ' + [filename ? Pointer_stringify(filename) : 'unknown filename', line, func ? Pointer_stringify(func) : 'unknown function'] + ' at ' + stackTrace();
    }
  Module["_strcpy"] = _strcpy;
  function _llvm_uadd_with_overflow_i32(x, y) {
      x = x>>>0;
      y = y>>>0;
      return ((asm["setTempRet0"](x+y > 4294967295),(x+y)>>>0)|0);
    }
  function _llvm_bswap_i32(x) {
      return ((x&0xff)<<24) | (((x>>8)&0xff)<<16) | (((x>>16)&0xff)<<8) | (x>>>24);
    }
  function _bsearch(key, base, num, size, compar) {
      var cmp = function(x, y) {
        return Module['dynCall_iii'](compar, x, y);
      };
      var left = 0;
      var right = num;
      var mid, test, addr;
      while (left < right) {
        mid = (left + right) >>> 1;
        addr = base + (mid * size);
        test = cmp(key, addr);
        if (test < 0) {
          right = mid;
        } else if (test > 0) {
          left = mid + 1;
        } else {
          return addr;
        }
      }
      return 0;
    }
  function _access(path, amode) {
      // int access(const char *path, int amode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/access.html
      path = Pointer_stringify(path);
      if (amode & ~7) {
        // need a valid mode
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      }
      var node;
      try {
        var lookup = FS.lookupPath(path, { follow: true });
        node = lookup.node;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
      var perms = '';
      if (amode & 4) perms += 'r';
      if (amode & 2) perms += 'w';
      if (amode & 1) perms += 'x';
      if (perms /* otherwise, they've just passed F_OK */ && FS.nodePermissions(node, perms)) {
        ___setErrNo(ERRNO_CODES.EACCES);
        return -1;
      }
      return 0;
    }
  function _stat(path, buf, dontResolveLastLink) {
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/stat.html
      // int stat(const char *path, struct stat *buf);
      // NOTE: dontResolveLastLink is a shortcut for lstat(). It should never be
      //       used in client code.
      path = typeof path !== 'string' ? Pointer_stringify(path) : path;
      try {
        var stat = dontResolveLastLink ? FS.lstat(path) : FS.stat(path);
        HEAP32[((buf)>>2)]=stat.dev;
        HEAP32[(((buf)+(4))>>2)]=0;
        HEAP32[(((buf)+(8))>>2)]=stat.ino;
        HEAP32[(((buf)+(12))>>2)]=stat.mode
        HEAP32[(((buf)+(16))>>2)]=stat.nlink
        HEAP32[(((buf)+(20))>>2)]=stat.uid
        HEAP32[(((buf)+(24))>>2)]=stat.gid
        HEAP32[(((buf)+(28))>>2)]=stat.rdev
        HEAP32[(((buf)+(32))>>2)]=0;
        HEAP32[(((buf)+(36))>>2)]=stat.size
        HEAP32[(((buf)+(40))>>2)]=4096
        HEAP32[(((buf)+(44))>>2)]=stat.blocks
        HEAP32[(((buf)+(48))>>2)]=Math.floor(stat.atime.getTime() / 1000)
        HEAP32[(((buf)+(52))>>2)]=0
        HEAP32[(((buf)+(56))>>2)]=Math.floor(stat.mtime.getTime() / 1000)
        HEAP32[(((buf)+(60))>>2)]=0
        HEAP32[(((buf)+(64))>>2)]=Math.floor(stat.ctime.getTime() / 1000)
        HEAP32[(((buf)+(68))>>2)]=0
        HEAP32[(((buf)+(72))>>2)]=stat.ino
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fstat(fildes, buf) {
      // int fstat(int fildes, struct stat *buf);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/fstat.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      return _stat(stream.path, buf);
    }
  function _uncompress() {
  Module['printErr']('missing function: uncompress'); abort(-1);
  }
  function _llvm_bswap_i16(x) {
      return ((x&0xff)<<8) | ((x>>8)&0xff);
    }
  function _memchr(ptr, chr, num) {
      chr = unSign(chr);
      for (var i = 0; i < num; i++) {
        if (HEAP8[(ptr)] == chr) return ptr;
        ptr++;
      }
      return 0;
    }
  function _llrint(x) {
      x = (x < 0) ? -Math.round(-x) : Math.round(x);
      return ((asm["setTempRet0"]((tempDouble=x,(+(Math_abs(tempDouble))) >= (+1) ? (tempDouble > (+0) ? ((Math_min((+(Math_floor((tempDouble)/(+4294967296)))), (+4294967295)))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/(+4294967296))))))>>>0) : 0)),x>>>0)|0);
    }
  var _fabs=Math_abs;
  function _ff_guess_image2_codec() {
  Module['printErr']('missing function: ff_guess_image2_codec'); abort(-1);
  }
  var _tan=Math_tan;
  function _exp2(x) {
      return Math.pow(2, x);
    }
  var _sqrt=Math_sqrt;
  function _cbrt(x) {
      return Math.pow(x, 1/3);
    }var _cbrtf=_cbrt;
  function _frexp(x, exp_addr) {
      var sig = 0, exp_ = 0;
      if (x !== 0) {
        var sign = 1;
        if (x < 0) {
          x = -x;
          sign = -1;
        }
        var raw_exp = Math.log(x)/Math.log(2);
        exp_ = Math.ceil(raw_exp);
        if (exp_ === raw_exp) exp_ += 1;
        sig = sign*x/Math.pow(2, exp_);
      }
      HEAP32[((exp_addr)>>2)]=exp_
      return sig;
    }
  var _sin=Math_sin;
  var _cos=Math_cos;
  var _llvm_pow_f64=Math_pow;
  var _sqrtf=Math_sqrt;
  var _llvm_pow_f32=Math_pow;
  var _fabsf=Math_abs;
  var _exp2f=_exp2;
  var _log2f=_log2;
  var _sinf=Math_sin;
  var _cosf=Math_cos;
  var _atan2f=Math_atan2;
  var _atanf=Math_atan;
  var _llrintf=_llrint;
  var _exp=Math_exp;
  var _ceil=Math_ceil;
  function __parseInt64(str, endptr, base, min, max, unsign) {
      var isNegative = false;
      // Skip space.
      while (_isspace(HEAP8[(str)])) str++;
      // Check for a plus/minus sign.
      if (HEAP8[(str)] == 45) {
        str++;
        isNegative = true;
      } else if (HEAP8[(str)] == 43) {
        str++;
      }
      // Find base.
      var ok = false;
      var finalBase = base;
      if (!finalBase) {
        if (HEAP8[(str)] == 48) {
          if (HEAP8[((str+1)|0)] == 120 ||
              HEAP8[((str+1)|0)] == 88) {
            finalBase = 16;
            str += 2;
          } else {
            finalBase = 8;
            ok = true; // we saw an initial zero, perhaps the entire thing is just "0"
          }
        }
      } else if (finalBase==16) {
        if (HEAP8[(str)] == 48) {
          if (HEAP8[((str+1)|0)] == 120 ||
              HEAP8[((str+1)|0)] == 88) {
            str += 2;
          }
        }
      }
      if (!finalBase) finalBase = 10;
      start = str;
      // Get digits.
      var chr;
      while ((chr = HEAP8[(str)]) != 0) {
        var digit = parseInt(String.fromCharCode(chr), finalBase);
        if (isNaN(digit)) {
          break;
        } else {
          str++;
          ok = true;
        }
      }
      if (!ok) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return ((asm["setTempRet0"](0),0)|0);
      }
      // Set end pointer.
      if (endptr) {
        HEAP32[((endptr)>>2)]=str
      }
      try {
        var numberString = isNegative ? '-'+Pointer_stringify(start, str - start) : Pointer_stringify(start, str - start);
        i64Math.fromString(numberString, finalBase, min, max, unsign);
      } catch(e) {
        ___setErrNo(ERRNO_CODES.ERANGE); // not quite correct
      }
      return ((asm["setTempRet0"](((HEAP32[(((tempDoublePtr)+(4))>>2)])|0)),((HEAP32[((tempDoublePtr)>>2)])|0))|0);
    }function _strtoll(str, endptr, base) {
      return __parseInt64(str, endptr, base, '-9223372036854775808', '9223372036854775807');  // LLONG_MIN, LLONG_MAX.
    }
  function _strtoul(str, endptr, base) {
      return __parseInt(str, endptr, base, 0, 4294967295, 32, true);  // ULONG_MAX.
    }
  var _floor=Math_floor;
  function _trunc(x) {
      return (x < 0) ? Math.ceil(x) : Math.floor(x);
    }
  function _sinh(x) {
      var p = Math.pow(Math.E, x);
      return (p - (1 / p)) / 2;
    }
  function _cosh(x) {
      var p = Math.pow(Math.E, x);
      return (p + (1 / p)) / 2;
    }
  function _tanh(x) {
      return _sinh(x) / _cosh(x);
    }
  var _atan=Math_atan;
  var _asin=Math_asin;
  var _acos=Math_acos;
  Module["_llvm_uadd_with_overflow_i64"] = _llvm_uadd_with_overflow_i64;
  function _mktime(tmPtr) {
      _tzset();
      var year = HEAP32[(((tmPtr)+(20))>>2)];
      var timestamp = new Date(year >= 1900 ? year : year + 1900,
                               HEAP32[(((tmPtr)+(16))>>2)],
                               HEAP32[(((tmPtr)+(12))>>2)],
                               HEAP32[(((tmPtr)+(8))>>2)],
                               HEAP32[(((tmPtr)+(4))>>2)],
                               HEAP32[((tmPtr)>>2)],
                               0).getTime() / 1000;
      HEAP32[(((tmPtr)+(24))>>2)]=new Date(timestamp).getDay()
      var yday = Math.round((timestamp - (new Date(year, 0, 1)).getTime()) / (1000 * 60 * 60 * 24));
      HEAP32[(((tmPtr)+(28))>>2)]=yday
      return timestamp;
    }
  function _clock() {
      if (_clock.start === undefined) _clock.start = Date.now();
      return Math.floor((Date.now() - _clock.start) * (1000000/1000));
    }
  function _gettimeofday(ptr) {
      var now = Date.now();
      HEAP32[((ptr)>>2)]=Math.floor(now/1000); // seconds
      HEAP32[(((ptr)+(4))>>2)]=Math.floor((now-1000*Math.floor(now/1000))*1000); // microseconds
      return 0;
    }
  function _usleep(useconds) {
      // int usleep(useconds_t useconds);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/usleep.html
      // We're single-threaded, so use a busy loop. Super-ugly.
      var msec = useconds / 1000;
      if (ENVIRONMENT_IS_WEB && window['performance'] && window['performance']['now']) {
        var start = window['performance']['now']();
        while (window['performance']['now']() - start < msec) {
          // Do nothing.
        }
      } else {
        var start = Date.now();
        while (Date.now() - start < msec) {
          // Do nothing.
        }
      }
      return 0;
    }function _nanosleep(rqtp, rmtp) {
      // int nanosleep(const struct timespec  *rqtp, struct timespec *rmtp);
      var seconds = HEAP32[((rqtp)>>2)];
      var nanoseconds = HEAP32[(((rqtp)+(4))>>2)];
      HEAP32[((rmtp)>>2)]=0
      HEAP32[(((rmtp)+(4))>>2)]=0
      return _usleep((seconds * 1e6) + (nanoseconds / 1000));
    }
  function _log10(x) {
      return Math.log(x) / Math.LN10;
    }
  var _floorf=Math_floor;
  var _log10f=_log10;
  function _sprintf(s, format, varargs) {
      // int sprintf(char *restrict s, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      return _snprintf(s, undefined, format, varargs);
    }
  Module["_tolower"] = _tolower;
  Module["_strcat"] = _strcat;
  function _ldexp(x, exp_) {
      return x * Math.pow(2, exp_);
    }
  var _rintf=_rint;
  function _vsprintf(s, format, va_arg) {
      return _sprintf(s, format, HEAP32[((va_arg)>>2)]);
    }
  function _sbrk(bytes) {
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) self.alloc(bytes);
      return ret;  // Previous break location.
    }
  function _sysconf(name) {
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 79:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: return 1;
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }
  var Browser={mainLoop:{scheduler:null,shouldPause:false,paused:false,queue:[],pause:function () {
          Browser.mainLoop.shouldPause = true;
        },resume:function () {
          if (Browser.mainLoop.paused) {
            Browser.mainLoop.paused = false;
            Browser.mainLoop.scheduler();
          }
          Browser.mainLoop.shouldPause = false;
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
        if (Browser.initted || ENVIRONMENT_IS_WORKER) return;
        Browser.initted = true;
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
        var imagePlugin = {};
        imagePlugin['canHandle'] = function(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          var img = new Image();
          img.onload = function() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
        var audioPlugin = {};
        audioPlugin['canHandle'] = function(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
        // Canvas event setup
        var canvas = Module['canvas'];
        canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                    canvas['mozRequestPointerLock'] ||
                                    canvas['webkitRequestPointerLock'];
        canvas.exitPointerLock = document['exitPointerLock'] ||
                                 document['mozExitPointerLock'] ||
                                 document['webkitExitPointerLock'] ||
                                 function(){}; // no-op if function does not exist
        canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas;
        }
        document.addEventListener('pointerlockchange', pointerLockChange, false);
        document.addEventListener('mozpointerlockchange', pointerLockChange, false);
        document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
        if (Module['elementPointerLock']) {
          canvas.addEventListener("click", function(ev) {
            if (!Browser.pointerLock && canvas.requestPointerLock) {
              canvas.requestPointerLock();
              ev.preventDefault();
            }
          }, false);
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        var ctx;
        try {
          if (useWebGL) {
            var contextAttributes = {
              antialias: false,
              alpha: false
            };
            if (webGLContextAttributes) {
              for (var attribute in webGLContextAttributes) {
                contextAttributes[attribute] = webGLContextAttributes[attribute];
              }
            }
            ctx = canvas.getContext('experimental-webgl', contextAttributes);
          } else {
            ctx = canvas.getContext('2d');
          }
          if (!ctx) throw ':(';
        } catch (e) {
          Module.print('Could not create canvas - ' + e);
          return null;
        }
        if (useWebGL) {
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
          // Warn on context loss
          canvas.addEventListener('webglcontextlost', function(event) {
            alert('WebGL context lost. You will need to reload the page.');
          }, false);
        }
        if (setInModule) {
          Module.ctx = ctx;
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement']) === canvas) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'];
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else if (Browser.resizeCanvas){
            Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
        }
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
        }
        canvas.requestFullScreen = canvas['requestFullScreen'] ||
                                   canvas['mozRequestFullScreen'] ||
                                   (canvas['webkitRequestFullScreen'] ? function() { canvas['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
        canvas.requestFullScreen();
      },requestAnimationFrame:function (func) {
        if (!window.requestAnimationFrame) {
          window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                         window['mozRequestAnimationFrame'] ||
                                         window['webkitRequestAnimationFrame'] ||
                                         window['msRequestAnimationFrame'] ||
                                         window['oRequestAnimationFrame'] ||
                                         window['setTimeout'];
        }
        window.requestAnimationFrame(func);
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (!ABORT) func();
        });
      },safeSetTimeout:function (func, timeout) {
        return setTimeout(function() {
          if (!ABORT) func();
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        return setInterval(function() {
          if (!ABORT) func();
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var x, y;
          if (event.type == 'touchstart' ||
              event.type == 'touchend' ||
              event.type == 'touchmove') {
            var t = event.touches.item(0);
            if (t) {
              x = t.pageX - (window.scrollX + rect.left);
              y = t.pageY - (window.scrollY + rect.top);
            } else {
              return;
            }
          } else {
            x = event.pageX - (window.scrollX + rect.left);
            y = event.pageY - (window.scrollY + rect.top);
          }
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        canvas.width = width;
        canvas.height = height;
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        var canvas = Module['canvas'];
        this.windowedWidth = canvas.width;
        this.windowedHeight = canvas.height;
        canvas.width = screen.width;
        canvas.height = screen.height;
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        var canvas = Module['canvas'];
        canvas.width = this.windowedWidth;
        canvas.height = this.windowedHeight;
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      }};
FS.staticInit();__ATINIT__.unshift({ func: function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() } });__ATMAIN__.push({ func: function() { FS.ignorePermissions = false } });__ATEXIT__.push({ func: function() { FS.quit() } });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;
___errno_state = Runtime.staticAlloc(4); HEAP32[((___errno_state)>>2)]=0;
__ATINIT__.unshift({ func: function() { TTY.init() } });__ATEXIT__.push({ func: function() { TTY.shutdown() } });TTY.utf8 = new Runtime.UTF8Processor();
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); NODEFS.staticInit(); }
__ATINIT__.push({ func: function() { SOCKFS.root = FS.mount(SOCKFS, {}, null); } });
_fgetc.ret = allocate([0], "i8", ALLOC_STATIC);
___buildEnvironment(ENV);
_fputc.ret = allocate([0], "i8", ALLOC_STATIC);
Module["requestFullScreen"] = function(lockPointer, resizeCanvas) { Browser.requestFullScreen(lockPointer, resizeCanvas) };
  Module["requestAnimationFrame"] = function(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function() { Browser.getUserMedia() }
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);
staticSealed = true; // seal the static portion of memory
STACK_MAX = STACK_BASE + 5242880;
DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);
assert(DYNAMIC_BASE < TOTAL_MEMORY); // Stack must fit in TOTAL_MEMORY; allocations from here on may enlarge TOTAL_MEMORY
 var ctlz_i8 = allocate([8,7,6,6,5,5,5,5,4,4,4,4,4,4,4,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_DYNAMIC);
 var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);
var Math_min = Math.min;
function invoke_viiiii(index,a1,a2,a3,a4,a5) {
  try {
    Module["dynCall_viiiii"](index,a1,a2,a3,a4,a5);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viifi(index,a1,a2,a3,a4) {
  try {
    Module["dynCall_viifi"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_fiff(index,a1,a2,a3) {
  try {
    return Module["dynCall_fiff"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_vi(index,a1) {
  try {
    Module["dynCall_vi"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_vii(index,a1,a2) {
  try {
    Module["dynCall_vii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
  try {
    return Module["dynCall_iiiiiii"](index,a1,a2,a3,a4,a5,a6);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_ii(index,a1) {
  try {
    return Module["dynCall_ii"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viiiiiifi(index,a1,a2,a3,a4,a5,a6,a7,a8) {
  try {
    Module["dynCall_viiiiiifi"](index,a1,a2,a3,a4,a5,a6,a7,a8);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_iiiiiiii(index,a1,a2,a3,a4,a5,a6,a7) {
  try {
    return Module["dynCall_iiiiiiii"](index,a1,a2,a3,a4,a5,a6,a7);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_iiiiii(index,a1,a2,a3,a4,a5) {
  try {
    return Module["dynCall_iiiiii"](index,a1,a2,a3,a4,a5);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_iiifii(index,a1,a2,a3,a4,a5) {
  try {
    return Module["dynCall_iiifii"](index,a1,a2,a3,a4,a5);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_iiii(index,a1,a2,a3) {
  try {
    return Module["dynCall_iiii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_fif(index,a1,a2) {
  try {
    return Module["dynCall_fif"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viiiiiiii(index,a1,a2,a3,a4,a5,a6,a7,a8) {
  try {
    Module["dynCall_viiiiiiii"](index,a1,a2,a3,a4,a5,a6,a7,a8);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viiiiii(index,a1,a2,a3,a4,a5,a6) {
  try {
    Module["dynCall_viiiiii"](index,a1,a2,a3,a4,a5,a6);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_fiii(index,a1,a2,a3) {
  try {
    return Module["dynCall_fiii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_iiiiiiiiii(index,a1,a2,a3,a4,a5,a6,a7,a8,a9) {
  try {
    return Module["dynCall_iiiiiiiiii"](index,a1,a2,a3,a4,a5,a6,a7,a8,a9);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viiiiiii(index,a1,a2,a3,a4,a5,a6,a7) {
  try {
    Module["dynCall_viiiiiii"](index,a1,a2,a3,a4,a5,a6,a7);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_ff(index,a1) {
  try {
    return Module["dynCall_ff"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viiiiiiiii(index,a1,a2,a3,a4,a5,a6,a7,a8,a9) {
  try {
    Module["dynCall_viiiiiiiii"](index,a1,a2,a3,a4,a5,a6,a7,a8,a9);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viiiiiiiiii(index,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10) {
  try {
    Module["dynCall_viiiiiiiiii"](index,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_iii(index,a1,a2) {
  try {
    return Module["dynCall_iii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_fii(index,a1,a2) {
  try {
    return Module["dynCall_fii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_iiiiiiifiiifi(index,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12) {
  try {
    return Module["dynCall_iiiiiiifiiifi"](index,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viiiiiiiiiiii(index,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12) {
  try {
    Module["dynCall_viiiiiiiiiiii"](index,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_iiiii(index,a1,a2,a3,a4) {
  try {
    return Module["dynCall_iiiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viiiifii(index,a1,a2,a3,a4,a5,a6,a7) {
  try {
    Module["dynCall_viiiifii"](index,a1,a2,a3,a4,a5,a6,a7);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viii(index,a1,a2,a3) {
  try {
    Module["dynCall_viii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_v(index) {
  try {
    Module["dynCall_v"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_iiiiiiiii(index,a1,a2,a3,a4,a5,a6,a7,a8) {
  try {
    return Module["dynCall_iiiiiiiii"](index,a1,a2,a3,a4,a5,a6,a7,a8);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function invoke_viiii(index,a1,a2,a3,a4) {
  try {
    Module["dynCall_viiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}
function asmPrintInt(x, y) {
  Module.print('int ' + x + ',' + y);// + ' ' + new Error().stack);
}
function asmPrintFloat(x, y) {
  Module.print('float ' + x + ',' + y);// + ' ' + new Error().stack);
}
// EMSCRIPTEN_START_ASM
var asm=(function(global,env,buffer){"use asm";var a=new global.Int8Array(buffer);var b=new global.Int16Array(buffer);var c=new global.Int32Array(buffer);var d=new global.Uint8Array(buffer);var e=new global.Uint16Array(buffer);var f=new global.Uint32Array(buffer);var g=new global.Float32Array(buffer);var h=new global.Float64Array(buffer);var i=env.STACKTOP|0;var j=env.STACK_MAX|0;var k=env.tempDoublePtr|0;var l=env.ABORT|0;var m=env.cttz_i8|0;var n=env.ctlz_i8|0;var o=env._stderr|0;var p=env._stdout|0;var q=env.___fsmu8|0;var r=+env.NaN;var s=+env.Infinity;var t=0;var u=0;var v=0;var w=0;var x=0,y=0,z=0,A=0,B=0.0,C=0,D=0,E=0,F=0.0;var G=0;var H=0;var I=0;var J=0;var K=0;var L=0;var M=0;var N=0;var O=0;var P=0;var Q=global.Math.floor;var R=global.Math.abs;var S=global.Math.sqrt;var T=global.Math.pow;var U=global.Math.cos;var V=global.Math.sin;var W=global.Math.tan;var X=global.Math.acos;var Y=global.Math.asin;var Z=global.Math.atan;var _=global.Math.atan2;var $=global.Math.exp;var aa=global.Math.log;var ab=global.Math.ceil;var ac=global.Math.imul;var ad=env.abort;var ae=env.assert;var af=env.asmPrintInt;var ag=env.asmPrintFloat;var ah=env.min;var ai=env.invoke_viiiii;var aj=env.invoke_viifi;var ak=env.invoke_fiff;var al=env.invoke_vi;var am=env.invoke_vii;var an=env.invoke_iiiiiii;var ao=env.invoke_ii;var ap=env.invoke_viiiiiifi;var aq=env.invoke_iiiiiiii;var ar=env.invoke_iiiiii;var as=env.invoke_iiifii;var at=env.invoke_iiii;var au=env.invoke_fif;var av=env.invoke_viiiiiiii;var aw=env.invoke_viiiiii;var ax=env.invoke_fiii;var ay=env.invoke_iiiiiiiiii;var az=env.invoke_viiiiiii;var aA=env.invoke_ff;var aB=env.invoke_viiiiiiiii;var aC=env.invoke_viiiiiiiiii;var aD=env.invoke_iii;var aE=env.invoke_fii;var aF=env.invoke_iiiiiiifiiifi;var aG=env.invoke_viiiiiiiiiiii;var aH=env.invoke_iiiii;var aI=env.invoke_viiiifii;var aJ=env.invoke_viii;var aK=env.invoke_v;var aL=env.invoke_iiiiiiiii;var aM=env.invoke_viiii;var aN=env._llvm_lifetime_end;var aO=env._lseek;var aP=env._sysconf;var aQ=env.__scanString;var aR=env._fclose;var aS=env._strtoul;var aT=env._fflush;var aU=env._strtol;var aV=env.__isLeapYear;var aW=env._fwrite;var aX=env._send;var aY=env._frexp;var aZ=env._fputs;var a_=env._cbrt;var a$=env._isspace;var a0=env._localtime;var a1=env._read;var a2=env._ceil;var a3=env._perror;var a4=env._strstr;var a5=env._fsync;var a6=env._signal;var a7=env._tanh;var a8=env._strcmp;var a9=env._memchr;var ba=env._strncmp;var bb=env._cosf;var bc=env._snprintf;var bd=env._fgetc;var be=env._cosh;var bf=env.__getFloat;var bg=env._fgets;var bh=env._close;var bi=env._strchr;var bj=env._tcgetattr;var bk=env._clock;var bl=env.___setErrNo;var bm=env._uncompress;var bn=env._access;var bo=env._ftell;var bp=env._exit;var bq=env._sprintf;var br=env._strrchr;var bs=env._atan2f;var bt=env._setrlimit;var bu=env._log2;var bv=env._strcspn;var bw=env._fputc;var bx=env._ferror;var by=env._llvm_uadd_with_overflow_i32;var bz=env._gmtime;var bA=env._localtime_r;var bB=env._sinh;var bC=env._sinf;var bD=env._recv;var bE=env.__parseInt64;var bF=env._trunc;var bG=env._cos;var bH=env._putchar;var bI=env._bsearch;var bJ=env.__exit;var bK=env._nanosleep;var bL=env._strftime;var bM=env._llvm_va_end;var bN=env._tzset;var bO=env._fabsf;var bP=env._floorf;var bQ=env._llvm_va_copy;var bR=env._rint;var bS=env._ldexp;var bT=env._printf;var bU=env._pread;var bV=env._fopen;var bW=env._open;var bX=env._sqrtf;var bY=env._select;var bZ=env._getchar;var b_=env._usleep;var b$=env._llvm_pow_f32;var b0=env.__arraySum;var b1=env._log;var b2=env._puts;var b3=env._mktime;var b4=env._exp2;var b5=env._scanf;var b6=env._qsort;var b7=env._log10;var b8=env._isatty;var b9=env.__formatString;var ca=env._getenv;var cb=env._gettimeofday;var cc=env._atoi;var cd=env._vfprintf;var ce=env._llvm_pow_f64;var cf=env._sbrk;var cg=env._fscanf;var ch=env.___errno_location;var ci=env._strerror;var cj=env._getrusage;var ck=env._fstat;var cl=env._llvm_lifetime_start;var cm=env._strspn;var cn=env.__parseInt;var co=env._ungetc;var cp=env._exp;var cq=env._vsprintf;var cr=env._gmtime_r;var cs=env._vsnprintf;var ct=env._sscanf;var cu=env.___assert_fail;var cv=env._fread;var cw=env._abort;var cx=env._fprintf;var cy=env._strtoll;var cz=env._tan;var cA=env.___buildEnvironment;var cB=env.__reallyNegative;var cC=env.__addDays;var cD=env._atanf;var cE=env._fabs;var cF=env._floor;var cG=env._fseek;var cH=env._sqrt;var cI=env._write;var cJ=env._sin;var cK=env._stat;var cL=env._ff_guess_image2_codec;var cM=env._tcsetattr;var cN=env._atan;var cO=env._asin;var cP=env._acos;var cQ=env._pwrite;var cR=env._strerror_r;var cS=env._llrint;var cT=env._llvm_bswap_i16;var cU=env._time;var cV=env._llvm_bswap_i32;var cW=env._setvbuf;
// EMSCRIPTEN_START_FUNCS
function ug(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;if((j|0)<=0){q=0;r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}A=d+11284|0;B=d+11288|0;C=d+11292|0;D=d+11296|0;E=d+11300|0;F=d+11304|0;G=i;i=0;while(1){H=ac(b[m+(i<<1)>>1]|0,g)|0;I=(ac(b[h+(i<<1)>>1]|0,k)|0)+H>>10;H=ac(b[e+(i<<1)>>1]|0,p)|0;J=H-67108864+(ac(b[n+(i<<1)>>1]|0,l)|0)>>10;H=ac(b[f+(i<<1)>>1]|0,p)|0;K=H-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;H=(ac(I-(c[A>>2]|0)|0,c[B>>2]|0)|0)+2097152|0;I=(ac(K,c[C>>2]|0)|0)+H|0;L=(ac(K,c[D>>2]|0)|0)+H|0;K=L+(ac(c[E>>2]|0,J)|0)|0;L=(ac(c[F>>2]|0,J)|0)+H|0;H=L>>>22&255;J=K>>>22&255;M=I>>>22&255;do{if((K|I|L)>>>0>1073741823){if(I>>>0>1073741823){N=-I>>31>>>22&255}else{N=M}if(K>>>0>1073741823){O=-K>>31>>>22&255}else{O=J}if(L>>>0<=1073741823){P=N;Q=O;R=H;break}P=N;Q=O;R=-L>>31>>>22&255}else{P=M;Q=J;R=H}}while(0);a[G]=R;a[G+1|0]=Q;a[G+2|0]=P;H=i+1|0;if((H|0)<(j|0)){G=G+3|0;i=H}else{q=j;break}}r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}function uh(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;l=c[f>>2]|0;h=c[g>>2]|0;do{if((k|0)<2048){if((j|0)<=0){m=0;break}n=d+11284|0;o=d+11288|0;p=d+11292|0;q=d+11296|0;r=d+11300|0;s=d+11304|0;t=i;u=0;while(1){v=(b[l+(u<<1)>>1]<<2)-65536|0;w=(b[h+(u<<1)>>1]<<2)-65536|0;x=(ac((b[e+(u<<1)>>1]<<2)-(c[n>>2]|0)|0,c[o>>2]|0)|0)+2097152|0;y=x+(ac(w,c[p>>2]|0)|0)|0;z=(ac(c[q>>2]|0,w)|0)+x|0;w=z+(ac(c[r>>2]|0,v)|0)|0;z=(ac(c[s>>2]|0,v)|0)+x|0;x=z>>>22&255;v=w>>>22&255;A=y>>>22&255;do{if((w|y|z)>>>0>1073741823){if(y>>>0>1073741823){B=-y>>31>>>22&255}else{B=A}if(w>>>0>1073741823){C=-w>>31>>>22&255}else{C=v}if(z>>>0<=1073741823){D=B;E=C;F=x;break}D=B;E=C;F=-z>>31>>>22&255}else{D=A;E=v;F=x}}while(0);a[t]=F;a[t+1|0]=E;a[t+2|0]=D;x=u+1|0;if((x|0)<(j|0)){t=t+3|0;u=x}else{m=j;break}}}else{u=c[f+4>>2]|0;t=c[g+4>>2]|0;if((j|0)<=0){m=0;break}s=d+11284|0;r=d+11288|0;q=d+11292|0;p=d+11296|0;o=d+11300|0;n=d+11304|0;x=i;v=0;while(1){A=((b[u+(v<<1)>>1]|0)+(b[l+(v<<1)>>1]|0)<<1)-65536|0;z=((b[t+(v<<1)>>1]|0)+(b[h+(v<<1)>>1]|0)<<1)-65536|0;w=(ac((b[e+(v<<1)>>1]<<2)-(c[s>>2]|0)|0,c[r>>2]|0)|0)+2097152|0;y=(ac(z,c[q>>2]|0)|0)+w|0;G=(ac(z,c[p>>2]|0)|0)+w|0;z=G+(ac(c[o>>2]|0,A)|0)|0;G=(ac(c[n>>2]|0,A)|0)+w|0;w=G>>>22&255;A=z>>>22&255;H=y>>>22&255;do{if((z|y|G)>>>0>1073741823){if(y>>>0>1073741823){I=-y>>31>>>22&255}else{I=H}if(z>>>0>1073741823){J=-z>>31>>>22&255}else{J=A}if(G>>>0<=1073741823){K=I;L=J;M=w;break}K=I;L=J;M=-G>>31>>>22&255}else{K=H;L=A;M=w}}while(0);a[x]=M;a[x+1|0]=L;a[x+2|0]=K;w=v+1|0;if((w|0)<(j|0)){x=x+3|0;v=w}else{m=j;break}}}}while(0);c[(c[d+11200>>2]|0)+(m<<2)>>2]=0;c[(c[d+11204>>2]|0)+(m<<2)>>2]=0;c[(c[d+11208>>2]|0)+(m<<2)>>2]=0;return}function ui(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0;if((n|0)<=0){p=0;q=0;r=0;s=0;t=d+11200|0;u=d+11204|0;v=d+11208|0;w=c[t>>2]|0;x=w+(s<<2)|0;c[x>>2]=r;y=c[u>>2]|0;z=y+(s<<2)|0;c[z>>2]=q;A=c[v>>2]|0;B=A+(s<<2)|0;c[B>>2]=p;return}o=(g|0)>0;l=(k|0)>0;C=d+11284|0;D=d+11288|0;E=d+11292|0;F=d+11296|0;G=d+11300|0;H=d+11304|0;I=d+11200|0;J=d+11204|0;K=d+11208|0;d=0;L=0;M=0;N=m;m=0;while(1){if(o){O=0;P=512;do{P=(ac(b[e+(O<<1)>>1]|0,b[(c[f+(O<<2)>>2]|0)+(m<<1)>>1]|0)|0)+P|0;O=O+1|0;}while((O|0)<(g|0));Q=P>>10}else{Q=0}if(l){O=0;R=-67108352;S=-67108352;do{T=b[h+(O<<1)>>1]|0;R=(ac(T,b[(c[i+(O<<2)>>2]|0)+(m<<1)>>1]|0)|0)+R|0;S=(ac(b[(c[j+(O<<2)>>2]|0)+(m<<1)>>1]|0,T)|0)+S|0;O=O+1|0;}while((O|0)<(k|0));U=R>>10;V=S>>10}else{U=-65536;V=-65536}O=(ac(Q-(c[C>>2]|0)|0,c[D>>2]|0)|0)+2097152|0;P=O+(ac(c[E>>2]|0,V)|0)|0;T=(ac(c[F>>2]|0,V)|0)+O|0;W=T+(ac(c[G>>2]|0,U)|0)|0;T=(ac(c[H>>2]|0,U)|0)+O|0;do{if((W|P|T)>>>0>1073741823){if(P>>>0>1073741823){X=-P>>31&1073741823}else{X=P}if(W>>>0>1073741823){Y=-W>>31&1073741823}else{Y=W}if(T>>>0<=1073741823){Z=X;_=Y;$=T;break}Z=X;_=Y;$=-T>>31&1073741823}else{Z=P;_=W;$=T}}while(0);T=c[I>>2]|0;W=T+(m<<2)|0;P=m+1|0;S=m+2|0;R=((c[W>>2]|0)+(M*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+(Z>>22)|0;T=c[J>>2]|0;O=((c[T+(m<<2)>>2]|0)+(L*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+(_>>22)|0;T=c[K>>2]|0;aa=((c[T+(m<<2)>>2]|0)+(d*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+($>>22)|0;c[W>>2]=M;c[(c[J>>2]|0)+(m<<2)>>2]=L;c[(c[K>>2]|0)+(m<<2)>>2]=d;W=R>>7;S=O>>6;T=aa>>7;if((W|0)<0){ab=0}else{ab=(W|0)>1?1:W}if((S|0)<0){ad=0}else{ad=(S|0)>3?3:S}if((T|0)<0){ae=0}else{ae=(T|0)>1?1:T}T=(ab*-255|0)+R|0;R=(ad*-85|0)+O|0;O=(ae*-255|0)+aa|0;a[N]=(ad<<1)+ab+(ae<<3)&255;if((P|0)<(n|0)){d=O;L=R;M=T;N=N+1|0;m=P}else{p=O;q=R;r=T;s=n;t=I;u=J;v=K;break}}w=c[t>>2]|0;x=w+(s<<2)|0;c[x>>2]=r;y=c[u>>2]|0;z=y+(s<<2)|0;c[z>>2]=q;A=c[v>>2]|0;B=A+(s<<2)|0;c[B>>2]=p;return}function uj(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;if((j|0)<=0){q=0;r=0;s=0;t=0;u=d+11200|0;v=d+11204|0;w=d+11208|0;x=c[u>>2]|0;y=x+(t<<2)|0;c[y>>2]=s;z=c[v>>2]|0;A=z+(t<<2)|0;c[A>>2]=r;B=c[w>>2]|0;C=B+(t<<2)|0;c[C>>2]=q;return}D=d+11284|0;E=d+11288|0;F=d+11292|0;G=d+11296|0;H=d+11300|0;I=d+11304|0;J=d+11200|0;K=d+11204|0;L=d+11208|0;d=0;M=0;N=0;O=i;i=0;while(1){P=ac(b[m+(i<<1)>>1]|0,g)|0;Q=(ac(b[h+(i<<1)>>1]|0,k)|0)+P>>10;P=ac(b[e+(i<<1)>>1]|0,p)|0;R=P-67108864+(ac(b[n+(i<<1)>>1]|0,l)|0)>>10;P=ac(b[f+(i<<1)>>1]|0,p)|0;S=P-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;P=(ac(Q-(c[D>>2]|0)|0,c[E>>2]|0)|0)+2097152|0;Q=(ac(S,c[F>>2]|0)|0)+P|0;T=(ac(S,c[G>>2]|0)|0)+P|0;S=T+(ac(c[H>>2]|0,R)|0)|0;T=(ac(c[I>>2]|0,R)|0)+P|0;do{if((S|Q|T)>>>0>1073741823){if(Q>>>0>1073741823){U=-Q>>31&1073741823}else{U=Q}if(S>>>0>1073741823){V=-S>>31&1073741823}else{V=S}if(T>>>0<=1073741823){W=U;X=V;Y=T;break}W=U;X=V;Y=-T>>31&1073741823}else{W=Q;X=S;Y=T}}while(0);T=c[J>>2]|0;S=T+(i<<2)|0;Q=i+1|0;P=i+2|0;R=((c[S>>2]|0)+(N*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(W>>22)|0;T=c[K>>2]|0;Z=((c[T+(i<<2)>>2]|0)+(M*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(X>>22)|0;T=c[L>>2]|0;_=((c[T+(i<<2)>>2]|0)+(d*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(Y>>22)|0;c[S>>2]=N;c[(c[K>>2]|0)+(i<<2)>>2]=M;c[(c[L>>2]|0)+(i<<2)>>2]=d;S=R>>7;P=Z>>6;T=_>>7;if((S|0)<0){$=0}else{$=(S|0)>1?1:S}if((P|0)<0){aa=0}else{aa=(P|0)>3?3:P}if((T|0)<0){ab=0}else{ab=(T|0)>1?1:T}T=($*-255|0)+R|0;R=(aa*-85|0)+Z|0;Z=(ab*-255|0)+_|0;a[O]=(aa<<1)+$+(ab<<3)&255;if((Q|0)<(j|0)){d=Z;M=R;N=T;O=O+1|0;i=Q}else{q=Z;r=R;s=T;t=j;u=J;v=K;w=L;break}}x=c[u>>2]|0;y=x+(t<<2)|0;c[y>>2]=s;z=c[v>>2]|0;A=z+(t<<2)|0;c[A>>2]=r;B=c[w>>2]|0;C=B+(t<<2)|0;c[C>>2]=q;return}function uk(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0;l=c[f>>2]|0;h=c[g>>2]|0;if((k|0)<2048){if((j|0)<=0){m=0;n=0;o=0;p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}k=d+11284|0;z=d+11288|0;A=d+11292|0;B=d+11296|0;C=d+11300|0;D=d+11304|0;E=d+11200|0;F=d+11204|0;G=d+11208|0;H=0;I=0;J=0;K=i;L=0;while(1){M=(b[l+(L<<1)>>1]<<2)-65536|0;N=(b[h+(L<<1)>>1]<<2)-65536|0;O=(ac((b[e+(L<<1)>>1]<<2)-(c[k>>2]|0)|0,c[z>>2]|0)|0)+2097152|0;P=O+(ac(N,c[A>>2]|0)|0)|0;Q=(ac(c[B>>2]|0,N)|0)+O|0;N=Q+(ac(c[C>>2]|0,M)|0)|0;Q=(ac(c[D>>2]|0,M)|0)+O|0;do{if((N|P|Q)>>>0>1073741823){if(P>>>0>1073741823){R=-P>>31&1073741823}else{R=P}if(N>>>0>1073741823){S=-N>>31&1073741823}else{S=N}if(Q>>>0<=1073741823){T=R;U=S;V=Q;break}T=R;U=S;V=-Q>>31&1073741823}else{T=P;U=N;V=Q}}while(0);Q=c[E>>2]|0;N=Q+(L<<2)|0;P=L+1|0;O=L+2|0;M=((c[N>>2]|0)+(J*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(T>>22)|0;Q=c[F>>2]|0;W=((c[Q+(L<<2)>>2]|0)+(I*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(U>>22)|0;Q=c[G>>2]|0;X=((c[Q+(L<<2)>>2]|0)+(H*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(V>>22)|0;c[N>>2]=J;c[(c[F>>2]|0)+(L<<2)>>2]=I;c[(c[G>>2]|0)+(L<<2)>>2]=H;N=M>>7;O=W>>6;Q=X>>7;if((N|0)<0){Y=0}else{Y=(N|0)>1?1:N}if((O|0)<0){Z=0}else{Z=(O|0)>3?3:O}if((Q|0)<0){_=0}else{_=(Q|0)>1?1:Q}Q=(Y*-255|0)+M|0;M=(Z*-85|0)+W|0;W=(_*-255|0)+X|0;a[K]=(Z<<1)+Y+(_<<3)&255;if((P|0)<(j|0)){H=W;I=M;J=Q;K=K+1|0;L=P}else{m=j;n=Q;o=M;p=W;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}else{L=c[f+4>>2]|0;f=c[g+4>>2]|0;if((j|0)<=0){m=0;n=0;o=0;p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}g=d+11284|0;K=d+11288|0;J=d+11292|0;I=d+11296|0;H=d+11300|0;_=d+11304|0;Y=d+11200|0;Z=d+11204|0;G=d+11208|0;F=0;V=0;U=0;T=i;i=0;while(1){E=((b[L+(i<<1)>>1]|0)+(b[l+(i<<1)>>1]|0)<<1)-65536|0;S=((b[f+(i<<1)>>1]|0)+(b[h+(i<<1)>>1]|0)<<1)-65536|0;R=(ac((b[e+(i<<1)>>1]<<2)-(c[g>>2]|0)|0,c[K>>2]|0)|0)+2097152|0;D=(ac(S,c[J>>2]|0)|0)+R|0;C=(ac(S,c[I>>2]|0)|0)+R|0;S=C+(ac(c[H>>2]|0,E)|0)|0;C=(ac(c[_>>2]|0,E)|0)+R|0;do{if((S|D|C)>>>0>1073741823){if(D>>>0>1073741823){$=-D>>31&1073741823}else{$=D}if(S>>>0>1073741823){aa=-S>>31&1073741823}else{aa=S}if(C>>>0<=1073741823){ab=$;ad=aa;ae=C;break}ab=$;ad=aa;ae=-C>>31&1073741823}else{ab=D;ad=S;ae=C}}while(0);C=c[Y>>2]|0;S=C+(i<<2)|0;D=i+1|0;R=i+2|0;E=((c[S>>2]|0)+(U*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ab>>22)|0;C=c[Z>>2]|0;B=((c[C+(i<<2)>>2]|0)+(V*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ad>>22)|0;C=c[G>>2]|0;A=((c[C+(i<<2)>>2]|0)+(F*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ae>>22)|0;c[S>>2]=U;c[(c[Z>>2]|0)+(i<<2)>>2]=V;c[(c[G>>2]|0)+(i<<2)>>2]=F;S=E>>7;R=B>>6;C=A>>7;if((S|0)<0){af=0}else{af=(S|0)>1?1:S}if((R|0)<0){ag=0}else{ag=(R|0)>3?3:R}if((C|0)<0){ah=0}else{ah=(C|0)>1?1:C}C=(af*-255|0)+E|0;E=(ag*-85|0)+B|0;B=(ah*-255|0)+A|0;a[T]=(ag<<1)+af+(ah<<3)&255;if((D|0)<(j|0)){F=B;V=E;U=C;T=T+1|0;i=D}else{m=j;n=C;o=E;p=B;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}}function ul(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0;if((n|0)<=0){p=0;q=0;r=0;s=0;t=d+11200|0;u=d+11204|0;v=d+11208|0;w=c[t>>2]|0;x=w+(s<<2)|0;c[x>>2]=r;y=c[u>>2]|0;z=y+(s<<2)|0;c[z>>2]=q;A=c[v>>2]|0;B=A+(s<<2)|0;c[B>>2]=p;return}o=(g|0)>0;l=(k|0)>0;C=d+11284|0;D=d+11288|0;E=d+11292|0;F=d+11296|0;G=d+11300|0;H=d+11304|0;I=d+11200|0;J=d+11204|0;K=d+11208|0;d=0;L=0;M=0;N=m;m=0;while(1){if(o){O=0;P=512;do{P=(ac(b[e+(O<<1)>>1]|0,b[(c[f+(O<<2)>>2]|0)+(m<<1)>>1]|0)|0)+P|0;O=O+1|0;}while((O|0)<(g|0));Q=P>>10}else{Q=0}if(l){O=0;R=-67108352;S=-67108352;do{T=b[h+(O<<1)>>1]|0;R=(ac(T,b[(c[i+(O<<2)>>2]|0)+(m<<1)>>1]|0)|0)+R|0;S=(ac(b[(c[j+(O<<2)>>2]|0)+(m<<1)>>1]|0,T)|0)+S|0;O=O+1|0;}while((O|0)<(k|0));U=R>>10;V=S>>10}else{U=-65536;V=-65536}O=(ac(Q-(c[C>>2]|0)|0,c[D>>2]|0)|0)+2097152|0;P=O+(ac(c[E>>2]|0,V)|0)|0;T=(ac(c[F>>2]|0,V)|0)+O|0;W=T+(ac(c[G>>2]|0,U)|0)|0;T=(ac(c[H>>2]|0,U)|0)+O|0;do{if((W|P|T)>>>0>1073741823){if(P>>>0>1073741823){X=-P>>31&1073741823}else{X=P}if(W>>>0>1073741823){Y=-W>>31&1073741823}else{Y=W}if(T>>>0<=1073741823){Z=X;_=Y;$=T;break}Z=X;_=Y;$=-T>>31&1073741823}else{Z=P;_=W;$=T}}while(0);T=c[I>>2]|0;W=T+(m<<2)|0;P=m+1|0;S=m+2|0;R=((c[W>>2]|0)+(M*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+(Z>>22)|0;T=c[J>>2]|0;O=((c[T+(m<<2)>>2]|0)+(L*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+(_>>22)|0;T=c[K>>2]|0;aa=((c[T+(m<<2)>>2]|0)+(d*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+($>>22)|0;c[W>>2]=M;c[(c[J>>2]|0)+(m<<2)>>2]=L;c[(c[K>>2]|0)+(m<<2)>>2]=d;W=R>>7;S=O>>6;T=aa>>7;if((W|0)<0){ab=0}else{ab=(W|0)>1?1:W}if((S|0)<0){ad=0}else{ad=(S|0)>3?3:S}if((T|0)<0){ae=0}else{ae=(T|0)>1?1:T}T=(ab*-255|0)+R|0;R=(ad*-85|0)+O|0;O=(ae*-255|0)+aa|0;a[N]=(ad<<1)+(ab<<3)+ae&255;if((P|0)<(n|0)){d=O;L=R;M=T;N=N+1|0;m=P}else{p=O;q=R;r=T;s=n;t=I;u=J;v=K;break}}w=c[t>>2]|0;x=w+(s<<2)|0;c[x>>2]=r;y=c[u>>2]|0;z=y+(s<<2)|0;c[z>>2]=q;A=c[v>>2]|0;B=A+(s<<2)|0;c[B>>2]=p;return}function um(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;if((j|0)<=0){q=0;r=0;s=0;t=0;u=d+11200|0;v=d+11204|0;w=d+11208|0;x=c[u>>2]|0;y=x+(t<<2)|0;c[y>>2]=s;z=c[v>>2]|0;A=z+(t<<2)|0;c[A>>2]=r;B=c[w>>2]|0;C=B+(t<<2)|0;c[C>>2]=q;return}D=d+11284|0;E=d+11288|0;F=d+11292|0;G=d+11296|0;H=d+11300|0;I=d+11304|0;J=d+11200|0;K=d+11204|0;L=d+11208|0;d=0;M=0;N=0;O=i;i=0;while(1){P=ac(b[m+(i<<1)>>1]|0,g)|0;Q=(ac(b[h+(i<<1)>>1]|0,k)|0)+P>>10;P=ac(b[e+(i<<1)>>1]|0,p)|0;R=P-67108864+(ac(b[n+(i<<1)>>1]|0,l)|0)>>10;P=ac(b[f+(i<<1)>>1]|0,p)|0;S=P-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;P=(ac(Q-(c[D>>2]|0)|0,c[E>>2]|0)|0)+2097152|0;Q=(ac(S,c[F>>2]|0)|0)+P|0;T=(ac(S,c[G>>2]|0)|0)+P|0;S=T+(ac(c[H>>2]|0,R)|0)|0;T=(ac(c[I>>2]|0,R)|0)+P|0;do{if((S|Q|T)>>>0>1073741823){if(Q>>>0>1073741823){U=-Q>>31&1073741823}else{U=Q}if(S>>>0>1073741823){V=-S>>31&1073741823}else{V=S}if(T>>>0<=1073741823){W=U;X=V;Y=T;break}W=U;X=V;Y=-T>>31&1073741823}else{W=Q;X=S;Y=T}}while(0);T=c[J>>2]|0;S=T+(i<<2)|0;Q=i+1|0;P=i+2|0;R=((c[S>>2]|0)+(N*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(W>>22)|0;T=c[K>>2]|0;Z=((c[T+(i<<2)>>2]|0)+(M*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(X>>22)|0;T=c[L>>2]|0;_=((c[T+(i<<2)>>2]|0)+(d*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(Y>>22)|0;c[S>>2]=N;c[(c[K>>2]|0)+(i<<2)>>2]=M;c[(c[L>>2]|0)+(i<<2)>>2]=d;S=R>>7;P=Z>>6;T=_>>7;if((S|0)<0){$=0}else{$=(S|0)>1?1:S}if((P|0)<0){aa=0}else{aa=(P|0)>3?3:P}if((T|0)<0){ab=0}else{ab=(T|0)>1?1:T}T=($*-255|0)+R|0;R=(aa*-85|0)+Z|0;Z=(ab*-255|0)+_|0;a[O]=(aa<<1)+($<<3)+ab&255;if((Q|0)<(j|0)){d=Z;M=R;N=T;O=O+1|0;i=Q}else{q=Z;r=R;s=T;t=j;u=J;v=K;w=L;break}}x=c[u>>2]|0;y=x+(t<<2)|0;c[y>>2]=s;z=c[v>>2]|0;A=z+(t<<2)|0;c[A>>2]=r;B=c[w>>2]|0;C=B+(t<<2)|0;c[C>>2]=q;return}function un(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0;l=c[f>>2]|0;h=c[g>>2]|0;if((k|0)<2048){if((j|0)<=0){m=0;n=0;o=0;p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}k=d+11284|0;z=d+11288|0;A=d+11292|0;B=d+11296|0;C=d+11300|0;D=d+11304|0;E=d+11200|0;F=d+11204|0;G=d+11208|0;H=0;I=0;J=0;K=i;L=0;while(1){M=(b[l+(L<<1)>>1]<<2)-65536|0;N=(b[h+(L<<1)>>1]<<2)-65536|0;O=(ac((b[e+(L<<1)>>1]<<2)-(c[k>>2]|0)|0,c[z>>2]|0)|0)+2097152|0;P=O+(ac(N,c[A>>2]|0)|0)|0;Q=(ac(c[B>>2]|0,N)|0)+O|0;N=Q+(ac(c[C>>2]|0,M)|0)|0;Q=(ac(c[D>>2]|0,M)|0)+O|0;do{if((N|P|Q)>>>0>1073741823){if(P>>>0>1073741823){R=-P>>31&1073741823}else{R=P}if(N>>>0>1073741823){S=-N>>31&1073741823}else{S=N}if(Q>>>0<=1073741823){T=R;U=S;V=Q;break}T=R;U=S;V=-Q>>31&1073741823}else{T=P;U=N;V=Q}}while(0);Q=c[E>>2]|0;N=Q+(L<<2)|0;P=L+1|0;O=L+2|0;M=((c[N>>2]|0)+(J*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(T>>22)|0;Q=c[F>>2]|0;W=((c[Q+(L<<2)>>2]|0)+(I*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(U>>22)|0;Q=c[G>>2]|0;X=((c[Q+(L<<2)>>2]|0)+(H*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(V>>22)|0;c[N>>2]=J;c[(c[F>>2]|0)+(L<<2)>>2]=I;c[(c[G>>2]|0)+(L<<2)>>2]=H;N=M>>7;O=W>>6;Q=X>>7;if((N|0)<0){Y=0}else{Y=(N|0)>1?1:N}if((O|0)<0){Z=0}else{Z=(O|0)>3?3:O}if((Q|0)<0){_=0}else{_=(Q|0)>1?1:Q}Q=(Y*-255|0)+M|0;M=(Z*-85|0)+W|0;W=(_*-255|0)+X|0;a[K]=(Z<<1)+(Y<<3)+_&255;if((P|0)<(j|0)){H=W;I=M;J=Q;K=K+1|0;L=P}else{m=j;n=Q;o=M;p=W;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}else{L=c[f+4>>2]|0;f=c[g+4>>2]|0;if((j|0)<=0){m=0;n=0;o=0;p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}g=d+11284|0;K=d+11288|0;J=d+11292|0;I=d+11296|0;H=d+11300|0;_=d+11304|0;Y=d+11200|0;Z=d+11204|0;G=d+11208|0;F=0;V=0;U=0;T=i;i=0;while(1){E=((b[L+(i<<1)>>1]|0)+(b[l+(i<<1)>>1]|0)<<1)-65536|0;S=((b[f+(i<<1)>>1]|0)+(b[h+(i<<1)>>1]|0)<<1)-65536|0;R=(ac((b[e+(i<<1)>>1]<<2)-(c[g>>2]|0)|0,c[K>>2]|0)|0)+2097152|0;D=(ac(S,c[J>>2]|0)|0)+R|0;C=(ac(S,c[I>>2]|0)|0)+R|0;S=C+(ac(c[H>>2]|0,E)|0)|0;C=(ac(c[_>>2]|0,E)|0)+R|0;do{if((S|D|C)>>>0>1073741823){if(D>>>0>1073741823){$=-D>>31&1073741823}else{$=D}if(S>>>0>1073741823){aa=-S>>31&1073741823}else{aa=S}if(C>>>0<=1073741823){ab=$;ad=aa;ae=C;break}ab=$;ad=aa;ae=-C>>31&1073741823}else{ab=D;ad=S;ae=C}}while(0);C=c[Y>>2]|0;S=C+(i<<2)|0;D=i+1|0;R=i+2|0;E=((c[S>>2]|0)+(U*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ab>>22)|0;C=c[Z>>2]|0;B=((c[C+(i<<2)>>2]|0)+(V*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ad>>22)|0;C=c[G>>2]|0;A=((c[C+(i<<2)>>2]|0)+(F*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ae>>22)|0;c[S>>2]=U;c[(c[Z>>2]|0)+(i<<2)>>2]=V;c[(c[G>>2]|0)+(i<<2)>>2]=F;S=E>>7;R=B>>6;C=A>>7;if((S|0)<0){af=0}else{af=(S|0)>1?1:S}if((R|0)<0){ag=0}else{ag=(R|0)>3?3:R}if((C|0)<0){ah=0}else{ah=(C|0)>1?1:C}C=(af*-255|0)+E|0;E=(ag*-85|0)+B|0;B=(ah*-255|0)+A|0;a[T]=(ag<<1)+(af<<3)+ah&255;if((D|0)<(j|0)){F=B;V=E;U=C;T=T+1|0;i=D}else{m=j;n=C;o=E;p=B;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}}function uo(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0;if((n|0)<=0){p=0;q=0;r=0;s=0;t=d+11200|0;u=d+11204|0;v=d+11208|0;w=c[t>>2]|0;x=w+(s<<2)|0;c[x>>2]=r;y=c[u>>2]|0;z=y+(s<<2)|0;c[z>>2]=q;A=c[v>>2]|0;B=A+(s<<2)|0;c[B>>2]=p;return}o=(g|0)>0;l=(k|0)>0;C=d+11284|0;D=d+11288|0;E=d+11292|0;F=d+11296|0;G=d+11300|0;H=d+11304|0;I=d+11200|0;J=d+11204|0;K=d+11208|0;d=0;L=0;M=0;N=m;m=0;while(1){if(o){O=0;P=512;do{P=(ac(b[e+(O<<1)>>1]|0,b[(c[f+(O<<2)>>2]|0)+(m<<1)>>1]|0)|0)+P|0;O=O+1|0;}while((O|0)<(g|0));Q=P>>10}else{Q=0}if(l){O=0;R=-67108352;S=-67108352;do{T=b[h+(O<<1)>>1]|0;R=(ac(T,b[(c[i+(O<<2)>>2]|0)+(m<<1)>>1]|0)|0)+R|0;S=(ac(b[(c[j+(O<<2)>>2]|0)+(m<<1)>>1]|0,T)|0)+S|0;O=O+1|0;}while((O|0)<(k|0));U=R>>10;V=S>>10}else{U=-65536;V=-65536}O=(ac(Q-(c[C>>2]|0)|0,c[D>>2]|0)|0)+2097152|0;P=O+(ac(c[E>>2]|0,V)|0)|0;T=(ac(c[F>>2]|0,V)|0)+O|0;W=T+(ac(c[G>>2]|0,U)|0)|0;T=(ac(c[H>>2]|0,U)|0)+O|0;do{if((W|P|T)>>>0>1073741823){if(P>>>0>1073741823){X=-P>>31&1073741823}else{X=P}if(W>>>0>1073741823){Y=-W>>31&1073741823}else{Y=W}if(T>>>0<=1073741823){Z=X;_=Y;$=T;break}Z=X;_=Y;$=-T>>31&1073741823}else{Z=P;_=W;$=T}}while(0);T=c[I>>2]|0;W=T+(m<<2)|0;P=m+1|0;S=m+2|0;R=((c[W>>2]|0)+(M*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+(Z>>22)|0;T=c[J>>2]|0;O=((c[T+(m<<2)>>2]|0)+(L*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+(_>>22)|0;T=c[K>>2]|0;aa=((c[T+(m<<2)>>2]|0)+(d*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+($>>22)|0;c[W>>2]=M;c[(c[J>>2]|0)+(m<<2)>>2]=L;c[(c[K>>2]|0)+(m<<2)>>2]=d;W=R>>5;S=O>>5;T=aa>>6;if((W|0)<0){ab=0}else{ab=(W|0)>7?7:W}if((S|0)<0){ad=0}else{ad=(S|0)>7?7:S}if((T|0)<0){ae=0}else{ae=(T|0)>3?3:T}T=(ab*-36|0)+R|0;R=(ad*-36|0)+O|0;O=(ae*-85|0)+aa|0;a[N]=(ad<<3)+ab+(ae<<6)&255;if((P|0)<(n|0)){d=O;L=R;M=T;N=N+1|0;m=P}else{p=O;q=R;r=T;s=n;t=I;u=J;v=K;break}}w=c[t>>2]|0;x=w+(s<<2)|0;c[x>>2]=r;y=c[u>>2]|0;z=y+(s<<2)|0;c[z>>2]=q;A=c[v>>2]|0;B=A+(s<<2)|0;c[B>>2]=p;return}function up(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;if((j|0)<=0){q=0;r=0;s=0;t=0;u=d+11200|0;v=d+11204|0;w=d+11208|0;x=c[u>>2]|0;y=x+(t<<2)|0;c[y>>2]=s;z=c[v>>2]|0;A=z+(t<<2)|0;c[A>>2]=r;B=c[w>>2]|0;C=B+(t<<2)|0;c[C>>2]=q;return}D=d+11284|0;E=d+11288|0;F=d+11292|0;G=d+11296|0;H=d+11300|0;I=d+11304|0;J=d+11200|0;K=d+11204|0;L=d+11208|0;d=0;M=0;N=0;O=i;i=0;while(1){P=ac(b[m+(i<<1)>>1]|0,g)|0;Q=(ac(b[h+(i<<1)>>1]|0,k)|0)+P>>10;P=ac(b[e+(i<<1)>>1]|0,p)|0;R=P-67108864+(ac(b[n+(i<<1)>>1]|0,l)|0)>>10;P=ac(b[f+(i<<1)>>1]|0,p)|0;S=P-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;P=(ac(Q-(c[D>>2]|0)|0,c[E>>2]|0)|0)+2097152|0;Q=(ac(S,c[F>>2]|0)|0)+P|0;T=(ac(S,c[G>>2]|0)|0)+P|0;S=T+(ac(c[H>>2]|0,R)|0)|0;T=(ac(c[I>>2]|0,R)|0)+P|0;do{if((S|Q|T)>>>0>1073741823){if(Q>>>0>1073741823){U=-Q>>31&1073741823}else{U=Q}if(S>>>0>1073741823){V=-S>>31&1073741823}else{V=S}if(T>>>0<=1073741823){W=U;X=V;Y=T;break}W=U;X=V;Y=-T>>31&1073741823}else{W=Q;X=S;Y=T}}while(0);T=c[J>>2]|0;S=T+(i<<2)|0;Q=i+1|0;P=i+2|0;R=((c[S>>2]|0)+(N*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(W>>22)|0;T=c[K>>2]|0;Z=((c[T+(i<<2)>>2]|0)+(M*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(X>>22)|0;T=c[L>>2]|0;_=((c[T+(i<<2)>>2]|0)+(d*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(Y>>22)|0;c[S>>2]=N;c[(c[K>>2]|0)+(i<<2)>>2]=M;c[(c[L>>2]|0)+(i<<2)>>2]=d;S=R>>5;P=Z>>5;T=_>>6;if((S|0)<0){$=0}else{$=(S|0)>7?7:S}if((P|0)<0){aa=0}else{aa=(P|0)>7?7:P}if((T|0)<0){ab=0}else{ab=(T|0)>3?3:T}T=($*-36|0)+R|0;R=(aa*-36|0)+Z|0;Z=(ab*-85|0)+_|0;a[O]=(aa<<3)+$+(ab<<6)&255;if((Q|0)<(j|0)){d=Z;M=R;N=T;O=O+1|0;i=Q}else{q=Z;r=R;s=T;t=j;u=J;v=K;w=L;break}}x=c[u>>2]|0;y=x+(t<<2)|0;c[y>>2]=s;z=c[v>>2]|0;A=z+(t<<2)|0;c[A>>2]=r;B=c[w>>2]|0;C=B+(t<<2)|0;c[C>>2]=q;return}function uq(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0;l=c[f>>2]|0;h=c[g>>2]|0;if((k|0)<2048){if((j|0)<=0){m=0;n=0;o=0;p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}k=d+11284|0;z=d+11288|0;A=d+11292|0;B=d+11296|0;C=d+11300|0;D=d+11304|0;E=d+11200|0;F=d+11204|0;G=d+11208|0;H=0;I=0;J=0;K=i;L=0;while(1){M=(b[l+(L<<1)>>1]<<2)-65536|0;N=(b[h+(L<<1)>>1]<<2)-65536|0;O=(ac((b[e+(L<<1)>>1]<<2)-(c[k>>2]|0)|0,c[z>>2]|0)|0)+2097152|0;P=O+(ac(N,c[A>>2]|0)|0)|0;Q=(ac(c[B>>2]|0,N)|0)+O|0;N=Q+(ac(c[C>>2]|0,M)|0)|0;Q=(ac(c[D>>2]|0,M)|0)+O|0;do{if((N|P|Q)>>>0>1073741823){if(P>>>0>1073741823){R=-P>>31&1073741823}else{R=P}if(N>>>0>1073741823){S=-N>>31&1073741823}else{S=N}if(Q>>>0<=1073741823){T=R;U=S;V=Q;break}T=R;U=S;V=-Q>>31&1073741823}else{T=P;U=N;V=Q}}while(0);Q=c[E>>2]|0;N=Q+(L<<2)|0;P=L+1|0;O=L+2|0;M=((c[N>>2]|0)+(J*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(T>>22)|0;Q=c[F>>2]|0;W=((c[Q+(L<<2)>>2]|0)+(I*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(U>>22)|0;Q=c[G>>2]|0;X=((c[Q+(L<<2)>>2]|0)+(H*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(V>>22)|0;c[N>>2]=J;c[(c[F>>2]|0)+(L<<2)>>2]=I;c[(c[G>>2]|0)+(L<<2)>>2]=H;N=M>>5;O=W>>5;Q=X>>6;if((N|0)<0){Y=0}else{Y=(N|0)>7?7:N}if((O|0)<0){Z=0}else{Z=(O|0)>7?7:O}if((Q|0)<0){_=0}else{_=(Q|0)>3?3:Q}Q=(Y*-36|0)+M|0;M=(Z*-36|0)+W|0;W=(_*-85|0)+X|0;a[K]=(Z<<3)+Y+(_<<6)&255;if((P|0)<(j|0)){H=W;I=M;J=Q;K=K+1|0;L=P}else{m=j;n=Q;o=M;p=W;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}else{L=c[f+4>>2]|0;f=c[g+4>>2]|0;if((j|0)<=0){m=0;n=0;o=0;p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}g=d+11284|0;K=d+11288|0;J=d+11292|0;I=d+11296|0;H=d+11300|0;_=d+11304|0;Y=d+11200|0;Z=d+11204|0;G=d+11208|0;F=0;V=0;U=0;T=i;i=0;while(1){E=((b[L+(i<<1)>>1]|0)+(b[l+(i<<1)>>1]|0)<<1)-65536|0;S=((b[f+(i<<1)>>1]|0)+(b[h+(i<<1)>>1]|0)<<1)-65536|0;R=(ac((b[e+(i<<1)>>1]<<2)-(c[g>>2]|0)|0,c[K>>2]|0)|0)+2097152|0;D=(ac(S,c[J>>2]|0)|0)+R|0;C=(ac(S,c[I>>2]|0)|0)+R|0;S=C+(ac(c[H>>2]|0,E)|0)|0;C=(ac(c[_>>2]|0,E)|0)+R|0;do{if((S|D|C)>>>0>1073741823){if(D>>>0>1073741823){$=-D>>31&1073741823}else{$=D}if(S>>>0>1073741823){aa=-S>>31&1073741823}else{aa=S}if(C>>>0<=1073741823){ab=$;ad=aa;ae=C;break}ab=$;ad=aa;ae=-C>>31&1073741823}else{ab=D;ad=S;ae=C}}while(0);C=c[Y>>2]|0;S=C+(i<<2)|0;D=i+1|0;R=i+2|0;E=((c[S>>2]|0)+(U*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ab>>22)|0;C=c[Z>>2]|0;B=((c[C+(i<<2)>>2]|0)+(V*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ad>>22)|0;C=c[G>>2]|0;A=((c[C+(i<<2)>>2]|0)+(F*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ae>>22)|0;c[S>>2]=U;c[(c[Z>>2]|0)+(i<<2)>>2]=V;c[(c[G>>2]|0)+(i<<2)>>2]=F;S=E>>5;R=B>>5;C=A>>6;if((S|0)<0){af=0}else{af=(S|0)>7?7:S}if((R|0)<0){ag=0}else{ag=(R|0)>7?7:R}if((C|0)<0){ah=0}else{ah=(C|0)>3?3:C}C=(af*-36|0)+E|0;E=(ag*-36|0)+B|0;B=(ah*-85|0)+A|0;a[T]=(ag<<3)+af+(ah<<6)&255;if((D|0)<(j|0)){F=B;V=E;U=C;T=T+1|0;i=D}else{m=j;n=C;o=E;p=B;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}}function ur(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0;if((n|0)<=0){p=0;q=0;r=0;s=0;t=d+11200|0;u=d+11204|0;v=d+11208|0;w=c[t>>2]|0;x=w+(s<<2)|0;c[x>>2]=r;y=c[u>>2]|0;z=y+(s<<2)|0;c[z>>2]=q;A=c[v>>2]|0;B=A+(s<<2)|0;c[B>>2]=p;return}o=(g|0)>0;l=(k|0)>0;C=d+11284|0;D=d+11288|0;E=d+11292|0;F=d+11296|0;G=d+11300|0;H=d+11304|0;I=d+11200|0;J=d+11204|0;K=d+11208|0;d=0;L=0;M=0;N=m;m=0;while(1){if(o){O=0;P=512;do{P=(ac(b[e+(O<<1)>>1]|0,b[(c[f+(O<<2)>>2]|0)+(m<<1)>>1]|0)|0)+P|0;O=O+1|0;}while((O|0)<(g|0));Q=P>>10}else{Q=0}if(l){O=0;R=-67108352;S=-67108352;do{T=b[h+(O<<1)>>1]|0;R=(ac(T,b[(c[i+(O<<2)>>2]|0)+(m<<1)>>1]|0)|0)+R|0;S=(ac(b[(c[j+(O<<2)>>2]|0)+(m<<1)>>1]|0,T)|0)+S|0;O=O+1|0;}while((O|0)<(k|0));U=R>>10;V=S>>10}else{U=-65536;V=-65536}O=(ac(Q-(c[C>>2]|0)|0,c[D>>2]|0)|0)+2097152|0;P=O+(ac(c[E>>2]|0,V)|0)|0;T=(ac(c[F>>2]|0,V)|0)+O|0;W=T+(ac(c[G>>2]|0,U)|0)|0;T=(ac(c[H>>2]|0,U)|0)+O|0;do{if((W|P|T)>>>0>1073741823){if(P>>>0>1073741823){X=-P>>31&1073741823}else{X=P}if(W>>>0>1073741823){Y=-W>>31&1073741823}else{Y=W}if(T>>>0<=1073741823){Z=X;_=Y;$=T;break}Z=X;_=Y;$=-T>>31&1073741823}else{Z=P;_=W;$=T}}while(0);T=c[I>>2]|0;W=T+(m<<2)|0;P=m+1|0;S=m+2|0;R=((c[W>>2]|0)+(M*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+(Z>>22)|0;T=c[J>>2]|0;O=((c[T+(m<<2)>>2]|0)+(L*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+(_>>22)|0;T=c[K>>2]|0;aa=((c[T+(m<<2)>>2]|0)+(d*7|0)+((c[T+(P<<2)>>2]|0)*5|0)+((c[T+(S<<2)>>2]|0)*3|0)>>4)+($>>22)|0;c[W>>2]=M;c[(c[J>>2]|0)+(m<<2)>>2]=L;c[(c[K>>2]|0)+(m<<2)>>2]=d;W=R>>5;S=O>>5;T=aa>>6;if((W|0)<0){ab=0}else{ab=(W|0)>7?7:W}if((S|0)<0){ad=0}else{ad=(S|0)>7?7:S}if((T|0)<0){ae=0}else{ae=(T|0)>3?3:T}T=(ab*-36|0)+R|0;R=(ad*-36|0)+O|0;O=(ae*-85|0)+aa|0;a[N]=(ad<<2)+(ab<<5)+ae&255;if((P|0)<(n|0)){d=O;L=R;M=T;N=N+1|0;m=P}else{p=O;q=R;r=T;s=n;t=I;u=J;v=K;break}}w=c[t>>2]|0;x=w+(s<<2)|0;c[x>>2]=r;y=c[u>>2]|0;z=y+(s<<2)|0;c[z>>2]=q;A=c[v>>2]|0;B=A+(s<<2)|0;c[B>>2]=p;return}function us(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;if((j|0)<=0){q=0;r=0;s=0;t=0;u=d+11200|0;v=d+11204|0;w=d+11208|0;x=c[u>>2]|0;y=x+(t<<2)|0;c[y>>2]=s;z=c[v>>2]|0;A=z+(t<<2)|0;c[A>>2]=r;B=c[w>>2]|0;C=B+(t<<2)|0;c[C>>2]=q;return}D=d+11284|0;E=d+11288|0;F=d+11292|0;G=d+11296|0;H=d+11300|0;I=d+11304|0;J=d+11200|0;K=d+11204|0;L=d+11208|0;d=0;M=0;N=0;O=i;i=0;while(1){P=ac(b[m+(i<<1)>>1]|0,g)|0;Q=(ac(b[h+(i<<1)>>1]|0,k)|0)+P>>10;P=ac(b[e+(i<<1)>>1]|0,p)|0;R=P-67108864+(ac(b[n+(i<<1)>>1]|0,l)|0)>>10;P=ac(b[f+(i<<1)>>1]|0,p)|0;S=P-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;P=(ac(Q-(c[D>>2]|0)|0,c[E>>2]|0)|0)+2097152|0;Q=(ac(S,c[F>>2]|0)|0)+P|0;T=(ac(S,c[G>>2]|0)|0)+P|0;S=T+(ac(c[H>>2]|0,R)|0)|0;T=(ac(c[I>>2]|0,R)|0)+P|0;do{if((S|Q|T)>>>0>1073741823){if(Q>>>0>1073741823){U=-Q>>31&1073741823}else{U=Q}if(S>>>0>1073741823){V=-S>>31&1073741823}else{V=S}if(T>>>0<=1073741823){W=U;X=V;Y=T;break}W=U;X=V;Y=-T>>31&1073741823}else{W=Q;X=S;Y=T}}while(0);T=c[J>>2]|0;S=T+(i<<2)|0;Q=i+1|0;P=i+2|0;R=((c[S>>2]|0)+(N*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(W>>22)|0;T=c[K>>2]|0;Z=((c[T+(i<<2)>>2]|0)+(M*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(X>>22)|0;T=c[L>>2]|0;_=((c[T+(i<<2)>>2]|0)+(d*7|0)+((c[T+(Q<<2)>>2]|0)*5|0)+((c[T+(P<<2)>>2]|0)*3|0)>>4)+(Y>>22)|0;c[S>>2]=N;c[(c[K>>2]|0)+(i<<2)>>2]=M;c[(c[L>>2]|0)+(i<<2)>>2]=d;S=R>>5;P=Z>>5;T=_>>6;if((S|0)<0){$=0}else{$=(S|0)>7?7:S}if((P|0)<0){aa=0}else{aa=(P|0)>7?7:P}if((T|0)<0){ab=0}else{ab=(T|0)>3?3:T}T=($*-36|0)+R|0;R=(aa*-36|0)+Z|0;Z=(ab*-85|0)+_|0;a[O]=(aa<<2)+($<<5)+ab&255;if((Q|0)<(j|0)){d=Z;M=R;N=T;O=O+1|0;i=Q}else{q=Z;r=R;s=T;t=j;u=J;v=K;w=L;break}}x=c[u>>2]|0;y=x+(t<<2)|0;c[y>>2]=s;z=c[v>>2]|0;A=z+(t<<2)|0;c[A>>2]=r;B=c[w>>2]|0;C=B+(t<<2)|0;c[C>>2]=q;return}function ut(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0;l=c[f>>2]|0;h=c[g>>2]|0;if((k|0)<2048){if((j|0)<=0){m=0;n=0;o=0;p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}k=d+11284|0;z=d+11288|0;A=d+11292|0;B=d+11296|0;C=d+11300|0;D=d+11304|0;E=d+11200|0;F=d+11204|0;G=d+11208|0;H=0;I=0;J=0;K=i;L=0;while(1){M=(b[l+(L<<1)>>1]<<2)-65536|0;N=(b[h+(L<<1)>>1]<<2)-65536|0;O=(ac((b[e+(L<<1)>>1]<<2)-(c[k>>2]|0)|0,c[z>>2]|0)|0)+2097152|0;P=O+(ac(N,c[A>>2]|0)|0)|0;Q=(ac(c[B>>2]|0,N)|0)+O|0;N=Q+(ac(c[C>>2]|0,M)|0)|0;Q=(ac(c[D>>2]|0,M)|0)+O|0;do{if((N|P|Q)>>>0>1073741823){if(P>>>0>1073741823){R=-P>>31&1073741823}else{R=P}if(N>>>0>1073741823){S=-N>>31&1073741823}else{S=N}if(Q>>>0<=1073741823){T=R;U=S;V=Q;break}T=R;U=S;V=-Q>>31&1073741823}else{T=P;U=N;V=Q}}while(0);Q=c[E>>2]|0;N=Q+(L<<2)|0;P=L+1|0;O=L+2|0;M=((c[N>>2]|0)+(J*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(T>>22)|0;Q=c[F>>2]|0;W=((c[Q+(L<<2)>>2]|0)+(I*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(U>>22)|0;Q=c[G>>2]|0;X=((c[Q+(L<<2)>>2]|0)+(H*7|0)+((c[Q+(P<<2)>>2]|0)*5|0)+((c[Q+(O<<2)>>2]|0)*3|0)>>4)+(V>>22)|0;c[N>>2]=J;c[(c[F>>2]|0)+(L<<2)>>2]=I;c[(c[G>>2]|0)+(L<<2)>>2]=H;N=M>>5;O=W>>5;Q=X>>6;if((N|0)<0){Y=0}else{Y=(N|0)>7?7:N}if((O|0)<0){Z=0}else{Z=(O|0)>7?7:O}if((Q|0)<0){_=0}else{_=(Q|0)>3?3:Q}Q=(Y*-36|0)+M|0;M=(Z*-36|0)+W|0;W=(_*-85|0)+X|0;a[K]=(Z<<2)+(Y<<5)+_&255;if((P|0)<(j|0)){H=W;I=M;J=Q;K=K+1|0;L=P}else{m=j;n=Q;o=M;p=W;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}else{L=c[f+4>>2]|0;f=c[g+4>>2]|0;if((j|0)<=0){m=0;n=0;o=0;p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}g=d+11284|0;K=d+11288|0;J=d+11292|0;I=d+11296|0;H=d+11300|0;_=d+11304|0;Y=d+11200|0;Z=d+11204|0;G=d+11208|0;F=0;V=0;U=0;T=i;i=0;while(1){E=((b[L+(i<<1)>>1]|0)+(b[l+(i<<1)>>1]|0)<<1)-65536|0;S=((b[f+(i<<1)>>1]|0)+(b[h+(i<<1)>>1]|0)<<1)-65536|0;R=(ac((b[e+(i<<1)>>1]<<2)-(c[g>>2]|0)|0,c[K>>2]|0)|0)+2097152|0;D=(ac(S,c[J>>2]|0)|0)+R|0;C=(ac(S,c[I>>2]|0)|0)+R|0;S=C+(ac(c[H>>2]|0,E)|0)|0;C=(ac(c[_>>2]|0,E)|0)+R|0;do{if((S|D|C)>>>0>1073741823){if(D>>>0>1073741823){$=-D>>31&1073741823}else{$=D}if(S>>>0>1073741823){aa=-S>>31&1073741823}else{aa=S}if(C>>>0<=1073741823){ab=$;ad=aa;ae=C;break}ab=$;ad=aa;ae=-C>>31&1073741823}else{ab=D;ad=S;ae=C}}while(0);C=c[Y>>2]|0;S=C+(i<<2)|0;D=i+1|0;R=i+2|0;E=((c[S>>2]|0)+(U*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ab>>22)|0;C=c[Z>>2]|0;B=((c[C+(i<<2)>>2]|0)+(V*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ad>>22)|0;C=c[G>>2]|0;A=((c[C+(i<<2)>>2]|0)+(F*7|0)+((c[C+(D<<2)>>2]|0)*5|0)+((c[C+(R<<2)>>2]|0)*3|0)>>4)+(ae>>22)|0;c[S>>2]=U;c[(c[Z>>2]|0)+(i<<2)>>2]=V;c[(c[G>>2]|0)+(i<<2)>>2]=F;S=E>>5;R=B>>5;C=A>>6;if((S|0)<0){af=0}else{af=(S|0)>7?7:S}if((R|0)<0){ag=0}else{ag=(R|0)>7?7:R}if((C|0)<0){ah=0}else{ah=(C|0)>3?3:C}C=(af*-36|0)+E|0;E=(ag*-36|0)+B|0;B=(ah*-85|0)+A|0;a[T]=(ag<<2)+(af<<5)+ah&255;if((D|0)<(j|0)){F=B;V=E;U=C;T=T+1|0;i=D}else{m=j;n=C;o=E;p=B;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(m<<2)|0;c[s>>2]=n;t=d+11204|0;u=c[t>>2]|0;v=u+(m<<2)|0;c[v>>2]=o;w=d+11208|0;x=c[w>>2]|0;y=x+(m<<2)|0;c[y>>2]=p;return}}function uu(d,f,g,h,j,k,l,m,n,o,p,q){d=d|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;var r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0;q=i;r=d+52|0;s=BP(c[r>>2]|0)|0;t=(a[s+7|0]|0)<0;u=(n|0)!=0;v=t&u;w=29-((e[s+8>>1]|0)>>>11&15)|0;s=(p|0)>0;if(s){x=(h|0)>0;y=(m|0)>0;z=d+11284|0;A=d+11288|0;B=d+11292|0;C=d+11296|0;D=d+11300|0;E=d+11304|0;d=(w|0)==22;F=o+4|0;G=o+8|0;H=o+12|0;I=0;J=0;K=0;while(1){if(x){L=0;M=512;do{M=(ac(b[f+(L<<1)>>1]|0,b[(c[g+(L<<2)>>2]|0)+(I<<1)>>1]|0)|0)+M|0;L=L+1|0;}while((L|0)<(h|0));N=M>>10}else{N=0}if(y){L=0;O=-67108352;P=-67108352;do{Q=b[j+(L<<1)>>1]|0;O=(ac(Q,b[(c[k+(L<<2)>>2]|0)+(I<<1)>>1]|0)|0)+O|0;P=(ac(b[(c[l+(L<<2)>>2]|0)+(I<<1)>>1]|0,Q)|0)+P|0;L=L+1|0;}while((L|0)<(m|0));R=O>>10;S=P>>10}else{R=-65536;S=-65536}L=t&u;M=L?0:J;Q=L?0:K;do{if(L&x){T=262144;U=0;do{T=(ac(b[f+(U<<1)>>1]|0,b[(c[n+(U<<2)>>2]|0)+(I<<1)>>1]|0)|0)+T|0;U=U+1|0;}while((U|0)<(h|0));U=T>>19;V=U&255;if((U&256|0)==0){W=U&65535;X=V;break}if(U>>>0>255){Y=-U>>31&255}else{Y=V}W=Y&255;X=Y}else{W=Q;X=M}}while(0);M=(ac(N-(c[z>>2]|0)|0,c[A>>2]|0)|0)+2097152|0;Q=M+(ac(c[B>>2]|0,S)|0)|0;L=(ac(c[C>>2]|0,S)|0)+M|0;P=L+(ac(c[D>>2]|0,R)|0)|0;L=(ac(c[E>>2]|0,R)|0)+M|0;do{if((P|Q|L)>>>0>1073741823){if(Q>>>0>1073741823){Z=-Q>>31&1073741823}else{Z=Q}if(P>>>0>1073741823){_=-P>>31&1073741823}else{_=P}if(L>>>0<=1073741823){$=L;aa=_;ab=Z;break}$=-L>>31&1073741823;aa=_;ab=Z}else{$=L;aa=P;ab=Q}}while(0);do{if(d){a[(c[o>>2]|0)+I|0]=aa>>>22&255;a[(c[F>>2]|0)+I|0]=$>>>22&255;a[(c[G>>2]|0)+I|0]=ab>>>22&255;if(!v){break}a[(c[H>>2]|0)+I|0]=X}else{b[(c[o>>2]|0)+(I<<1)>>1]=aa>>w&65535;b[(c[F>>2]|0)+(I<<1)>>1]=$>>w&65535;b[(c[G>>2]|0)+(I<<1)>>1]=ab>>w&65535;if(!v){break}b[(c[H>>2]|0)+(I<<1)>>1]=W}}while(0);Q=I+1|0;if((Q|0)<(p|0)){I=Q;J=X;K=W}else{break}}}if((w|0)==22){i=q;return}w=BP(c[r>>2]|0)|0;if((w|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}if((a[w+7|0]&1)==0|s^1){i=q;return}s=o+4|0;w=o+8|0;r=o+12|0;W=0;do{K=(c[o>>2]|0)+(W<<1)|0;b[K>>1]=cT(b[K>>1]|0)|0;K=(c[s>>2]|0)+(W<<1)|0;b[K>>1]=cT(b[K>>1]|0)|0;K=(c[w>>2]|0)+(W<<1)|0;b[K>>1]=cT(b[K>>1]|0)|0;if(v){K=(c[r>>2]|0)+(W<<1)|0;b[K>>1]=cT(b[K>>1]|0)|0}W=W+1|0;}while((W|0)<(p|0));i=q;return}function uv(b,d,e,f,g,h,j,k,l){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0;l=i;m=d;d=g;g=h;h=c[e>>2]|0;n=c[f>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){i=l;return}o=b+11284|0;p=b+11288|0;q=b+11292|0;r=b+11296|0;s=b+11300|0;t=b+11304|0;u=g;v=0;while(1){w=v<<1;x=w|1;z=(c[h+(v<<2)>>2]|0)-262144>>2;A=(c[n+(v<<2)>>2]|0)-262144>>2;B=c[o>>2]|0;C=(c[m+(x<<2)>>2]>>2)-B|0;D=c[p>>2]|0;E=ac((c[m+(w<<2)>>2]>>2)-B|0,D)|0;B=E+8192|0;E=(ac(C,D)|0)+8192|0;D=c[d+(w<<2)>>2]<<11;w=c[d+(x<<2)>>2]<<11;x=D+8192|0;C=w+8192|0;F=ac(c[q>>2]|0,A)|0;G=ac(c[r>>2]|0,A)|0;A=(ac(c[s>>2]|0,z)|0)+G|0;G=ac(c[t>>2]|0,z)|0;z=BP(292)|0;if((z|0)==0){H=7202;break}I=F+B|0;J=I>>>0>1073741823;K=I>>>14&65535;do{if((a[z+7|0]&1)==0){if(!J){L=K;break}L=-I>>31>>>14&65535}else{if(J){M=-I>>31>>>14&65535}else{M=K}L=cT(M|0)|0}}while(0);y=L;a[u]=y&255;y=y>>8;a[u+1|0]=y&255;K=BP(292)|0;if((K|0)==0){H=7210;break}I=A+B|0;J=I>>>0>1073741823;z=I>>>14&65535;if((a[K+7|0]&1)==0){if(J){N=-I>>31>>>14&65535}else{N=z}K=u+2|0;y=N;a[K]=y&255;y=y>>8;a[K+1|0]=y&255}else{if(J){O=-I>>31>>>14&65535}else{O=z}z=u+2|0;y=cT(O|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(292)|0;if((z|0)==0){H=7219;break}I=G+B|0;J=I>>>0>1073741823;K=I>>>14&65535;if((a[z+7|0]&1)==0){if(J){P=-I>>31>>>14&65535}else{P=K}z=u+4|0;y=P;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(J){Q=-I>>31>>>14&65535}else{Q=K}K=u+4|0;y=cT(Q|0)|0;a[K]=y&255;y=y>>8;a[K+1|0]=y&255}K=BP(292)|0;if((K|0)==0){H=7228;break}I=x>>>0>1073741823;J=x>>>14&65535;if((a[K+7|0]&1)==0){if(I){R=-8192-D>>31>>>14&65535}else{R=J}K=u+6|0;y=R;a[K]=y&255;y=y>>8;a[K+1|0]=y&255}else{if(I){S=-8192-D>>31>>>14&65535}else{S=J}J=u+6|0;y=cT(S|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(292)|0;if((J|0)==0){H=7237;break}I=F+E|0;K=I>>>0>1073741823;z=I>>>14&65535;if((a[J+7|0]&1)==0){if(K){T=-I>>31>>>14&65535}else{T=z}J=u+8|0;y=T;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(K){U=-I>>31>>>14&65535}else{U=z}z=u+8|0;y=cT(U|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(292)|0;if((z|0)==0){H=7246;break}I=A+E|0;K=I>>>0>1073741823;J=I>>>14&65535;if((a[z+7|0]&1)==0){if(K){V=-I>>31>>>14&65535}else{V=J}z=u+10|0;y=V;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(K){W=-I>>31>>>14&65535}else{W=J}J=u+10|0;y=cT(W|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(292)|0;if((J|0)==0){H=7255;break}I=G+E|0;K=I>>>0>1073741823;z=I>>>14&65535;if((a[J+7|0]&1)==0){if(K){X=-I>>31>>>14&65535}else{X=z}J=u+12|0;y=X;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(K){Y=-I>>31>>>14&65535}else{Y=z}z=u+12|0;y=cT(Y|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(292)|0;if((z|0)==0){H=7264;break}I=C>>>0>1073741823;K=C>>>14&65535;if((a[z+7|0]&1)==0){if(I){Z=-8192-w>>31>>>14&65535}else{Z=K}z=u+14|0;y=Z;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(I){_=-8192-w>>31>>>14&65535}else{_=K}K=u+14|0;y=cT(_|0)|0;a[K]=y&255;y=y>>8;a[K+1|0]=y&255}K=v+1|0;if((K|0)<(k|0)){u=u+16|0;v=K}else{H=7348;break}}if((H|0)==7246){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7237){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7348){i=l;return}else if((H|0)==7255){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7202){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7219){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7264){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7228){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7210){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}}else{v=c[e+4>>2]|0;e=c[f+4>>2]|0;f=j+1>>1;if((f|0)<=0){i=l;return}j=b+11284|0;u=b+11288|0;k=b+11292|0;_=b+11296|0;Z=b+11300|0;Y=b+11304|0;b=g;g=0;while(1){X=g<<1;W=X|1;V=(c[h+(g<<2)>>2]|0)-524288+(c[v+(g<<2)>>2]|0)>>3;U=(c[n+(g<<2)>>2]|0)-524288+(c[e+(g<<2)>>2]|0)>>3;T=c[j>>2]|0;S=(c[m+(W<<2)>>2]>>2)-T|0;R=c[u>>2]|0;Q=ac((c[m+(X<<2)>>2]>>2)-T|0,R)|0;T=Q+8192|0;Q=(ac(S,R)|0)+8192|0;R=c[d+(X<<2)>>2]<<11;X=c[d+(W<<2)>>2]<<11;W=R+8192|0;S=X+8192|0;P=ac(c[k>>2]|0,U)|0;O=ac(c[_>>2]|0,U)|0;U=(ac(c[Z>>2]|0,V)|0)+O|0;O=ac(c[Y>>2]|0,V)|0;V=BP(292)|0;if((V|0)==0){H=7276;break}N=P+T|0;L=N>>>0>1073741823;M=N>>>14&65535;do{if((a[V+7|0]&1)==0){if(!L){aa=M;break}aa=-N>>31>>>14&65535}else{if(L){ab=-N>>31>>>14&65535}else{ab=M}aa=cT(ab|0)|0}}while(0);y=aa;a[b]=y&255;y=y>>8;a[b+1|0]=y&255;M=BP(292)|0;if((M|0)==0){H=7284;break}N=U+T|0;L=N>>>0>1073741823;V=N>>>14&65535;if((a[M+7|0]&1)==0){if(L){ad=-N>>31>>>14&65535}else{ad=V}M=b+2|0;y=ad;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}else{if(L){ae=-N>>31>>>14&65535}else{ae=V}V=b+2|0;y=cT(ae|0)|0;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}V=BP(292)|0;if((V|0)==0){H=7293;break}N=O+T|0;L=N>>>0>1073741823;M=N>>>14&65535;if((a[V+7|0]&1)==0){if(L){af=-N>>31>>>14&65535}else{af=M}V=b+4|0;y=af;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}else{if(L){ag=-N>>31>>>14&65535}else{ag=M}M=b+4|0;y=cT(ag|0)|0;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}M=BP(292)|0;if((M|0)==0){H=7302;break}N=W>>>0>1073741823;L=W>>>14&65535;if((a[M+7|0]&1)==0){if(N){ah=-8192-R>>31>>>14&65535}else{ah=L}M=b+6|0;y=ah;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}else{if(N){ai=-8192-R>>31>>>14&65535}else{ai=L}L=b+6|0;y=cT(ai|0)|0;a[L]=y&255;y=y>>8;a[L+1|0]=y&255}L=BP(292)|0;if((L|0)==0){H=7311;break}N=P+Q|0;M=N>>>0>1073741823;V=N>>>14&65535;if((a[L+7|0]&1)==0){if(M){aj=-N>>31>>>14&65535}else{aj=V}L=b+8|0;y=aj;a[L]=y&255;y=y>>8;a[L+1|0]=y&255}else{if(M){ak=-N>>31>>>14&65535}else{ak=V}V=b+8|0;y=cT(ak|0)|0;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}V=BP(292)|0;if((V|0)==0){H=7320;break}N=U+Q|0;M=N>>>0>1073741823;L=N>>>14&65535;if((a[V+7|0]&1)==0){if(M){al=-N>>31>>>14&65535}else{al=L}V=b+10|0;y=al;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}else{if(M){am=-N>>31>>>14&65535}else{am=L}L=b+10|0;y=cT(am|0)|0;a[L]=y&255;y=y>>8;a[L+1|0]=y&255}L=BP(292)|0;if((L|0)==0){H=7329;break}N=O+Q|0;M=N>>>0>1073741823;V=N>>>14&65535;if((a[L+7|0]&1)==0){if(M){an=-N>>31>>>14&65535}else{an=V}L=b+12|0;y=an;a[L]=y&255;y=y>>8;a[L+1|0]=y&255}else{if(M){ao=-N>>31>>>14&65535}else{ao=V}V=b+12|0;y=cT(ao|0)|0;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}V=BP(292)|0;if((V|0)==0){H=7338;break}N=S>>>0>1073741823;M=S>>>14&65535;if((a[V+7|0]&1)==0){if(N){ap=-8192-X>>31>>>14&65535}else{ap=M}V=b+14|0;y=ap;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}else{if(N){aq=-8192-X>>31>>>14&65535}else{aq=M}M=b+14|0;y=cT(aq|0)|0;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}M=g+1|0;if((M|0)<(f|0)){b=b+16|0;g=M}else{H=7350;break}}if((H|0)==7276){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7320){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7350){i=l;return}else if((H|0)==7284){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7338){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7293){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7329){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7302){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7311){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}}}function uw(b,d,e,f,g,h,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0;m=i;n=c[d>>2]|0;o=c[d+4>>2]|0;d=c[e>>2]|0;p=c[e+4>>2]|0;e=c[f>>2]|0;q=c[f+4>>2]|0;f=c[g>>2]|0;r=c[g+4>>2]|0;g=4096-k|0;s=4096-l|0;t=j+1>>1;if((t|0)<=0){i=m;return}j=b+11284|0;u=b+11288|0;v=b+11292|0;w=b+11296|0;x=b+11300|0;z=b+11304|0;b=0;A=h;while(1){h=b<<1;B=ac(c[n+(h<<2)>>2]|0,g)|0;C=(ac(c[o+(h<<2)>>2]|0,k)|0)+B>>14;B=h|1;D=ac(c[n+(B<<2)>>2]|0,g)|0;E=(ac(c[o+(B<<2)>>2]|0,k)|0)+D>>14;D=ac(c[d+(b<<2)>>2]|0,s)|0;F=D-1073741824+(ac(c[p+(b<<2)>>2]|0,l)|0)>>14;D=ac(c[e+(b<<2)>>2]|0,s)|0;G=D-1073741824+(ac(c[q+(b<<2)>>2]|0,l)|0)>>14;D=c[j>>2]|0;H=c[u>>2]|0;I=ac(C-D|0,H)|0;C=I+8192|0;I=(ac(E-D|0,H)|0)+8192|0;H=ac(G,c[v>>2]|0)|0;D=ac(c[w>>2]|0,G)|0;G=(ac(c[x>>2]|0,F)|0)+D|0;D=ac(c[z>>2]|0,F)|0;F=ac(c[f+(h<<2)>>2]|0,g)|0;E=(ac(c[r+(h<<2)>>2]|0,k)|0)+F>>1;F=ac(c[f+(B<<2)>>2]|0,g)|0;h=(ac(c[r+(B<<2)>>2]|0,k)|0)+F>>1;F=E+8192|0;B=h+8192|0;J=BP(292)|0;if((J|0)==0){K=7355;break}L=H+C|0;M=L>>>0>1073741823;N=L>>>14&65535;do{if((a[J+7|0]&1)==0){if(!M){O=N;break}O=-L>>31>>>14&65535}else{if(M){P=-L>>31>>>14&65535}else{P=N}O=cT(P|0)|0}}while(0);y=O;a[A]=y&255;y=y>>8;a[A+1|0]=y&255;N=BP(292)|0;if((N|0)==0){K=7363;break}L=G+C|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Q=-L>>31>>>14&65535}else{Q=J}N=A+2|0;y=Q;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){R=-L>>31>>>14&65535}else{R=J}J=A+2|0;y=cT(R|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(292)|0;if((J|0)==0){K=7372;break}L=D+C|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){S=-L>>31>>>14&65535}else{S=N}J=A+4|0;y=S;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){T=-L>>31>>>14&65535}else{T=N}N=A+4|0;y=cT(T|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(292)|0;if((N|0)==0){K=7381;break}L=F>>>0>1073741823;M=F>>>14&65535;if((a[N+7|0]&1)==0){if(L){U=-8192-E>>31>>>14&65535}else{U=M}N=A+6|0;y=U;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(L){V=-8192-E>>31>>>14&65535}else{V=M}M=A+6|0;y=cT(V|0)|0;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}M=BP(292)|0;if((M|0)==0){K=7390;break}L=H+I|0;N=L>>>0>1073741823;J=L>>>14&65535;if((a[M+7|0]&1)==0){if(N){W=-L>>31>>>14&65535}else{W=J}M=A+8|0;y=W;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}else{if(N){X=-L>>31>>>14&65535}else{X=J}J=A+8|0;y=cT(X|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(292)|0;if((J|0)==0){K=7399;break}L=G+I|0;N=L>>>0>1073741823;M=L>>>14&65535;if((a[J+7|0]&1)==0){if(N){Y=-L>>31>>>14&65535}else{Y=M}J=A+10|0;y=Y;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(N){Z=-L>>31>>>14&65535}else{Z=M}M=A+10|0;y=cT(Z|0)|0;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}M=BP(292)|0;if((M|0)==0){K=7408;break}L=D+I|0;N=L>>>0>1073741823;J=L>>>14&65535;if((a[M+7|0]&1)==0){if(N){_=-L>>31>>>14&65535}else{_=J}M=A+12|0;y=_;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}else{if(N){$=-L>>31>>>14&65535}else{$=J}J=A+12|0;y=cT($|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(292)|0;if((J|0)==0){K=7417;break}L=B>>>0>1073741823;N=B>>>14&65535;if((a[J+7|0]&1)==0){if(L){aa=-8192-h>>31>>>14&65535}else{aa=N}J=A+14|0;y=aa;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(L){ab=-8192-h>>31>>>14&65535}else{ab=N}N=A+14|0;y=cT(ab|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=b+1|0;if((N|0)<(t|0)){b=N;A=A+16|0}else{K=7428;break}}if((K|0)==7428){i=m;return}else if((K|0)==7408){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7399){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7363){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7355){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7390){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7381){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7417){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7372){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}}function ux(d,e,f,g,h,j,k,l,m,n,o,p){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0;p=i;q=o+1>>1;if((q|0)<=0){i=p;return}o=(g|0)>0;r=(l|0)>0;s=d+11284|0;t=d+11288|0;u=d+11292|0;v=d+11296|0;w=d+11300|0;x=d+11304|0;d=0;z=n;while(1){if(o){n=d<<1;A=n|1;B=0;C=-1073741824;D=-1073741824;do{E=c[f+(B<<2)>>2]|0;F=b[e+(B<<1)>>1]|0;C=(ac(F,c[E+(n<<2)>>2]|0)|0)+C|0;D=(ac(c[E+(A<<2)>>2]|0,F)|0)+D|0;B=B+1|0;}while((B|0)<(g|0));G=(C>>14)+65536|0;H=(D>>14)+65536|0}else{G=0;H=0}if(r){B=0;A=-1073741824;n=-1073741824;do{F=b[h+(B<<1)>>1]|0;A=(ac(F,c[(c[j+(B<<2)>>2]|0)+(d<<2)>>2]|0)|0)+A|0;n=(ac(c[(c[k+(B<<2)>>2]|0)+(d<<2)>>2]|0,F)|0)+n|0;B=B+1|0;}while((B|0)<(l|0));I=A>>14;J=n>>14}else{I=-65536;J=-65536}if(o){B=d<<1;D=B|1;C=0;F=-1073741824;E=-1073741824;do{K=c[m+(C<<2)>>2]|0;L=b[e+(C<<1)>>1]|0;E=(ac(L,c[K+(B<<2)>>2]|0)|0)+E|0;F=(ac(c[K+(D<<2)>>2]|0,L)|0)+F|0;C=C+1|0;}while((C|0)<(g|0));M=F>>1;N=E>>1}else{M=-536870912;N=-536870912}C=N+536879104|0;D=M+536879104|0;B=c[s>>2]|0;n=c[t>>2]|0;A=ac(G-B|0,n)|0;L=A+8192|0;A=(ac(H-B|0,n)|0)+8192|0;n=ac(c[u>>2]|0,J)|0;B=ac(c[v>>2]|0,J)|0;K=(ac(c[w>>2]|0,I)|0)+B|0;B=ac(c[x>>2]|0,I)|0;O=BP(292)|0;if((O|0)==0){P=7443;break}Q=n+L|0;R=Q>>>0>1073741823;S=Q>>>14&65535;do{if((a[O+7|0]&1)==0){if(!R){T=S;break}T=-Q>>31>>>14&65535}else{if(R){U=-Q>>31>>>14&65535}else{U=S}T=cT(U|0)|0}}while(0);y=T;a[z]=y&255;y=y>>8;a[z+1|0]=y&255;S=BP(292)|0;if((S|0)==0){P=7451;break}Q=K+L|0;R=Q>>>0>1073741823;O=Q>>>14&65535;if((a[S+7|0]&1)==0){if(R){V=-Q>>31>>>14&65535}else{V=O}S=z+2|0;y=V;a[S]=y&255;y=y>>8;a[S+1|0]=y&255}else{if(R){W=-Q>>31>>>14&65535}else{W=O}O=z+2|0;y=cT(W|0)|0;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}O=BP(292)|0;if((O|0)==0){P=7460;break}Q=B+L|0;R=Q>>>0>1073741823;S=Q>>>14&65535;if((a[O+7|0]&1)==0){if(R){X=-Q>>31>>>14&65535}else{X=S}O=z+4|0;y=X;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}else{if(R){Y=-Q>>31>>>14&65535}else{Y=S}S=z+4|0;y=cT(Y|0)|0;a[S]=y&255;y=y>>8;a[S+1|0]=y&255}S=BP(292)|0;if((S|0)==0){P=7469;break}Q=C>>>0>1073741823;R=C>>>14&65535;if((a[S+7|0]&1)==0){if(Q){Z=-536879104-N>>31>>>14&65535}else{Z=R}S=z+6|0;y=Z;a[S]=y&255;y=y>>8;a[S+1|0]=y&255}else{if(Q){_=-536879104-N>>31>>>14&65535}else{_=R}R=z+6|0;y=cT(_|0)|0;a[R]=y&255;y=y>>8;a[R+1|0]=y&255}R=BP(292)|0;if((R|0)==0){P=7478;break}Q=n+A|0;S=Q>>>0>1073741823;O=Q>>>14&65535;if((a[R+7|0]&1)==0){if(S){$=-Q>>31>>>14&65535}else{$=O}R=z+8|0;y=$;a[R]=y&255;y=y>>8;a[R+1|0]=y&255}else{if(S){aa=-Q>>31>>>14&65535}else{aa=O}O=z+8|0;y=cT(aa|0)|0;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}O=BP(292)|0;if((O|0)==0){P=7487;break}Q=K+A|0;S=Q>>>0>1073741823;R=Q>>>14&65535;if((a[O+7|0]&1)==0){if(S){ab=-Q>>31>>>14&65535}else{ab=R}O=z+10|0;y=ab;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}else{if(S){ad=-Q>>31>>>14&65535}else{ad=R}R=z+10|0;y=cT(ad|0)|0;a[R]=y&255;y=y>>8;a[R+1|0]=y&255}R=BP(292)|0;if((R|0)==0){P=7496;break}Q=B+A|0;S=Q>>>0>1073741823;O=Q>>>14&65535;if((a[R+7|0]&1)==0){if(S){ae=-Q>>31>>>14&65535}else{ae=O}R=z+12|0;y=ae;a[R]=y&255;y=y>>8;a[R+1|0]=y&255}else{if(S){af=-Q>>31>>>14&65535}else{af=O}O=z+12|0;y=cT(af|0)|0;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}O=BP(292)|0;if((O|0)==0){P=7505;break}Q=D>>>0>1073741823;S=D>>>14&65535;if((a[O+7|0]&1)==0){if(Q){ag=-536879104-M>>31>>>14&65535}else{ag=S}O=z+14|0;y=ag;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}else{if(Q){ah=-536879104-M>>31>>>14&65535}else{ah=S}S=z+14|0;y=cT(ah|0)|0;a[S]=y&255;y=y>>8;a[S+1|0]=y&255}S=d+1|0;if((S|0)<(q|0)){d=S;z=z+16|0}else{P=7515;break}}if((P|0)==7515){i=p;return}else if((P|0)==7469){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==7443){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==7460){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==7451){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==7487){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==7505){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==7496){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==7478){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}}function uy(b,d,e,f,g,h,j,k,l){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0;l=i;g=d;d=h;h=c[e>>2]|0;m=c[f>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){i=l;return}n=b+11284|0;o=b+11288|0;p=b+11292|0;q=b+11296|0;r=b+11300|0;s=b+11304|0;t=d;u=0;while(1){v=u<<1;w=(c[h+(u<<2)>>2]|0)-262144>>2;x=(c[m+(u<<2)>>2]|0)-262144>>2;z=c[n>>2]|0;A=(c[g+((v|1)<<2)>>2]>>2)-z|0;B=c[o>>2]|0;C=ac((c[g+(v<<2)>>2]>>2)-z|0,B)|0;z=C+8192|0;C=(ac(A,B)|0)+8192|0;B=ac(c[p>>2]|0,x)|0;A=ac(c[q>>2]|0,x)|0;x=(ac(c[r>>2]|0,w)|0)+A|0;A=ac(c[s>>2]|0,w)|0;w=BP(292)|0;if((w|0)==0){D=7521;break}v=B+z|0;E=v>>>0>1073741823;F=v>>>14&65535;do{if((a[w+7|0]&1)==0){if(!E){G=F;break}G=-v>>31>>>14&65535}else{if(E){H=-v>>31>>>14&65535}else{H=F}G=cT(H|0)|0}}while(0);y=G;a[t]=y&255;y=y>>8;a[t+1|0]=y&255;F=BP(292)|0;if((F|0)==0){D=7529;break}v=x+z|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){I=-v>>31>>>14&65535}else{I=w}F=t+2|0;y=I;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){J=-v>>31>>>14&65535}else{J=w}w=t+2|0;y=cT(J|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(292)|0;if((w|0)==0){D=7538;break}v=A+z|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){K=-v>>31>>>14&65535}else{K=F}w=t+4|0;y=K;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){L=-v>>31>>>14&65535}else{L=F}F=t+4|0;y=cT(L|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}if((BP(292)|0)==0){D=7547;break}F=t+6|0;y=-1;a[F]=y&255;y=y>>8;a[F+1|0]=y&255;F=BP(292)|0;if((F|0)==0){D=7549;break}v=B+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){M=-v>>31>>>14&65535}else{M=w}F=t+8|0;y=M;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){N=-v>>31>>>14&65535}else{N=w}w=t+8|0;y=cT(N|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(292)|0;if((w|0)==0){D=7558;break}v=x+C|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){O=-v>>31>>>14&65535}else{O=F}w=t+10|0;y=O;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){P=-v>>31>>>14&65535}else{P=F}F=t+10|0;y=cT(P|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}F=BP(292)|0;if((F|0)==0){D=7567;break}v=A+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){Q=-v>>31>>>14&65535}else{Q=w}F=t+12|0;y=Q;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){R=-v>>31>>>14&65535}else{R=w}w=t+12|0;y=cT(R|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}if((BP(292)|0)==0){D=7576;break}w=t+14|0;y=-1;a[w]=y&255;y=y>>8;a[w+1|0]=y&255;w=u+1|0;if((w|0)<(k|0)){t=t+16|0;u=w}else{D=7639;break}}if((D|0)==7576){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7639){i=l;return}else if((D|0)==7567){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7529){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7521){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7549){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7538){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7558){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7547){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}else{u=c[e+4>>2]|0;e=c[f+4>>2]|0;f=j+1>>1;if((f|0)<=0){i=l;return}j=b+11284|0;t=b+11288|0;k=b+11292|0;R=b+11296|0;Q=b+11300|0;P=b+11304|0;b=d;d=0;while(1){O=d<<1;N=(c[h+(d<<2)>>2]|0)-524288+(c[u+(d<<2)>>2]|0)>>3;M=(c[m+(d<<2)>>2]|0)-524288+(c[e+(d<<2)>>2]|0)>>3;L=c[j>>2]|0;K=(c[g+((O|1)<<2)>>2]>>2)-L|0;J=c[t>>2]|0;I=ac((c[g+(O<<2)>>2]>>2)-L|0,J)|0;L=I+8192|0;I=(ac(K,J)|0)+8192|0;J=ac(c[k>>2]|0,M)|0;K=ac(c[R>>2]|0,M)|0;M=(ac(c[Q>>2]|0,N)|0)+K|0;K=ac(c[P>>2]|0,N)|0;N=BP(292)|0;if((N|0)==0){D=7581;break}O=J+L|0;G=O>>>0>1073741823;H=O>>>14&65535;do{if((a[N+7|0]&1)==0){if(!G){T=H;break}T=-O>>31>>>14&65535}else{if(G){U=-O>>31>>>14&65535}else{U=H}T=cT(U|0)|0}}while(0);y=T;a[b]=y&255;y=y>>8;a[b+1|0]=y&255;H=BP(292)|0;if((H|0)==0){D=7589;break}O=M+L|0;G=O>>>0>1073741823;N=O>>>14&65535;if((a[H+7|0]&1)==0){if(G){V=-O>>31>>>14&65535}else{V=N}H=b+2|0;y=V;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){W=-O>>31>>>14&65535}else{W=N}N=b+2|0;y=cT(W|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(292)|0;if((N|0)==0){D=7598;break}O=K+L|0;G=O>>>0>1073741823;H=O>>>14&65535;if((a[N+7|0]&1)==0){if(G){X=-O>>31>>>14&65535}else{X=H}N=b+4|0;y=X;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){Y=-O>>31>>>14&65535}else{Y=H}H=b+4|0;y=cT(Y|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}if((BP(292)|0)==0){D=7607;break}H=b+6|0;y=-1;a[H]=y&255;y=y>>8;a[H+1|0]=y&255;H=BP(292)|0;if((H|0)==0){D=7609;break}O=J+I|0;G=O>>>0>1073741823;N=O>>>14&65535;if((a[H+7|0]&1)==0){if(G){Z=-O>>31>>>14&65535}else{Z=N}H=b+8|0;y=Z;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){_=-O>>31>>>14&65535}else{_=N}N=b+8|0;y=cT(_|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(292)|0;if((N|0)==0){D=7618;break}O=M+I|0;G=O>>>0>1073741823;H=O>>>14&65535;if((a[N+7|0]&1)==0){if(G){$=-O>>31>>>14&65535}else{$=H}N=b+10|0;y=$;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){aa=-O>>31>>>14&65535}else{aa=H}H=b+10|0;y=cT(aa|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(292)|0;if((H|0)==0){D=7627;break}O=K+I|0;G=O>>>0>1073741823;N=O>>>14&65535;if((a[H+7|0]&1)==0){if(G){ab=-O>>31>>>14&65535}else{ab=N}H=b+12|0;y=ab;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){ad=-O>>31>>>14&65535}else{ad=N}N=b+12|0;y=cT(ad|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}if((BP(292)|0)==0){D=7636;break}N=b+14|0;y=-1;a[N]=y&255;y=y>>8;a[N+1|0]=y&255;N=d+1|0;if((N|0)<(f|0)){b=b+16|0;d=N}else{D=7641;break}}if((D|0)==7607){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7598){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7636){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7581){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7609){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7627){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7641){i=l;return}else if((D|0)==7618){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==7589){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}}function uz(b,d,e,f,g,h,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;m=i;g=c[d>>2]|0;n=c[d+4>>2]|0;d=c[e>>2]|0;o=c[e+4>>2]|0;e=c[f>>2]|0;p=c[f+4>>2]|0;f=4096-k|0;q=4096-l|0;r=j+1>>1;if((r|0)<=0){i=m;return}j=b+11284|0;s=b+11288|0;t=b+11292|0;u=b+11296|0;v=b+11300|0;w=b+11304|0;b=0;x=h;while(1){h=b<<1;z=ac(c[g+(h<<2)>>2]|0,f)|0;A=(ac(c[n+(h<<2)>>2]|0,k)|0)+z>>14;z=h|1;h=ac(c[g+(z<<2)>>2]|0,f)|0;B=(ac(c[n+(z<<2)>>2]|0,k)|0)+h>>14;h=ac(c[d+(b<<2)>>2]|0,q)|0;z=h-1073741824+(ac(c[o+(b<<2)>>2]|0,l)|0)>>14;h=ac(c[e+(b<<2)>>2]|0,q)|0;C=h-1073741824+(ac(c[p+(b<<2)>>2]|0,l)|0)>>14;h=c[j>>2]|0;D=c[s>>2]|0;E=ac(A-h|0,D)|0;A=E+8192|0;E=(ac(B-h|0,D)|0)+8192|0;D=ac(C,c[t>>2]|0)|0;h=ac(c[u>>2]|0,C)|0;C=(ac(c[v>>2]|0,z)|0)+h|0;h=ac(c[w>>2]|0,z)|0;z=BP(292)|0;if((z|0)==0){F=7646;break}B=D+A|0;G=B>>>0>1073741823;H=B>>>14&65535;do{if((a[z+7|0]&1)==0){if(!G){I=H;break}I=-B>>31>>>14&65535}else{if(G){J=-B>>31>>>14&65535}else{J=H}I=cT(J|0)|0}}while(0);y=I;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;H=BP(292)|0;if((H|0)==0){F=7654;break}B=C+A|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){K=-B>>31>>>14&65535}else{K=z}H=x+2|0;y=K;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){L=-B>>31>>>14&65535}else{L=z}z=x+2|0;y=cT(L|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(292)|0;if((z|0)==0){F=7663;break}B=h+A|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){M=-B>>31>>>14&65535}else{M=H}z=x+4|0;y=M;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){N=-B>>31>>>14&65535}else{N=H}H=x+4|0;y=cT(N|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}if((BP(292)|0)==0){F=7672;break}H=x+6|0;y=-1;a[H]=y&255;y=y>>8;a[H+1|0]=y&255;H=BP(292)|0;if((H|0)==0){F=7674;break}B=D+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){O=-B>>31>>>14&65535}else{O=z}H=x+8|0;y=O;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){P=-B>>31>>>14&65535}else{P=z}z=x+8|0;y=cT(P|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(292)|0;if((z|0)==0){F=7683;break}B=C+E|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){Q=-B>>31>>>14&65535}else{Q=H}z=x+10|0;y=Q;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){R=-B>>31>>>14&65535}else{R=H}H=x+10|0;y=cT(R|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(292)|0;if((H|0)==0){F=7692;break}B=h+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){S=-B>>31>>>14&65535}else{S=z}H=x+12|0;y=S;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){T=-B>>31>>>14&65535}else{T=z}z=x+12|0;y=cT(T|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}if((BP(292)|0)==0){F=7701;break}z=x+14|0;y=-1;a[z]=y&255;y=y>>8;a[z+1|0]=y&255;z=b+1|0;if((z|0)<(r|0)){b=z;x=x+16|0}else{F=7704;break}}if((F|0)==7663){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==7654){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==7674){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==7704){i=m;return}else if((F|0)==7646){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==7672){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==7692){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==7683){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==7701){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}}function uA(d,e,f,g,h,j,k,l,m,n,o,p){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0;p=i;m=o+1>>1;if((m|0)<=0){i=p;return}o=(g|0)>0;q=(l|0)>0;r=d+11284|0;s=d+11288|0;t=d+11292|0;u=d+11296|0;v=d+11300|0;w=d+11304|0;d=0;x=n;while(1){if(o){n=d<<1;z=n|1;A=0;B=-1073741824;C=-1073741824;do{D=c[f+(A<<2)>>2]|0;E=b[e+(A<<1)>>1]|0;B=(ac(E,c[D+(n<<2)>>2]|0)|0)+B|0;C=(ac(c[D+(z<<2)>>2]|0,E)|0)+C|0;A=A+1|0;}while((A|0)<(g|0));F=(B>>14)+65536|0;G=(C>>14)+65536|0}else{F=0;G=0}if(q){A=0;z=-1073741824;n=-1073741824;do{E=b[h+(A<<1)>>1]|0;z=(ac(E,c[(c[j+(A<<2)>>2]|0)+(d<<2)>>2]|0)|0)+z|0;n=(ac(c[(c[k+(A<<2)>>2]|0)+(d<<2)>>2]|0,E)|0)+n|0;A=A+1|0;}while((A|0)<(l|0));H=z>>14;I=n>>14}else{H=-65536;I=-65536}A=c[r>>2]|0;C=c[s>>2]|0;B=ac(F-A|0,C)|0;E=B+8192|0;B=(ac(G-A|0,C)|0)+8192|0;C=ac(c[t>>2]|0,I)|0;A=ac(c[u>>2]|0,I)|0;D=(ac(c[v>>2]|0,H)|0)+A|0;A=ac(c[w>>2]|0,H)|0;J=BP(292)|0;if((J|0)==0){K=7716;break}L=C+E|0;M=L>>>0>1073741823;N=L>>>14&65535;do{if((a[J+7|0]&1)==0){if(!M){O=N;break}O=-L>>31>>>14&65535}else{if(M){P=-L>>31>>>14&65535}else{P=N}O=cT(P|0)|0}}while(0);y=O;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;N=BP(292)|0;if((N|0)==0){K=7724;break}L=D+E|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Q=-L>>31>>>14&65535}else{Q=J}N=x+2|0;y=Q;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){R=-L>>31>>>14&65535}else{R=J}J=x+2|0;y=cT(R|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(292)|0;if((J|0)==0){K=7733;break}L=A+E|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){S=-L>>31>>>14&65535}else{S=N}J=x+4|0;y=S;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){T=-L>>31>>>14&65535}else{T=N}N=x+4|0;y=cT(T|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}if((BP(292)|0)==0){K=7742;break}N=x+6|0;y=-1;a[N]=y&255;y=y>>8;a[N+1|0]=y&255;N=BP(292)|0;if((N|0)==0){K=7744;break}L=C+B|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){U=-L>>31>>>14&65535}else{U=J}N=x+8|0;y=U;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){V=-L>>31>>>14&65535}else{V=J}J=x+8|0;y=cT(V|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(292)|0;if((J|0)==0){K=7753;break}L=D+B|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){W=-L>>31>>>14&65535}else{W=N}J=x+10|0;y=W;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){X=-L>>31>>>14&65535}else{X=N}N=x+10|0;y=cT(X|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(292)|0;if((N|0)==0){K=7762;break}L=A+B|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Y=-L>>31>>>14&65535}else{Y=J}N=x+12|0;y=Y;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){Z=-L>>31>>>14&65535}else{Z=J}J=x+12|0;y=cT(Z|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}if((BP(292)|0)==0){K=7771;break}J=x+14|0;y=-1;a[J]=y&255;y=y>>8;a[J+1|0]=y&255;J=d+1|0;if((J|0)<(m|0)){d=J;x=x+16|0}else{K=7775;break}}if((K|0)==7716){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==7753){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==7762){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==7742){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==7744){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==7724){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==7775){i=p;return}else if((K|0)==7733){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==7771){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}}function uB(b,d,e,f,g,h,j,k,l){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0;l=i;m=d;d=g;g=h;h=c[e>>2]|0;n=c[f>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){i=l;return}o=b+11284|0;p=b+11288|0;q=b+11292|0;r=b+11296|0;s=b+11300|0;t=b+11304|0;u=g;v=0;while(1){w=v<<1;x=w|1;z=(c[h+(v<<2)>>2]|0)-262144>>2;A=(c[n+(v<<2)>>2]|0)-262144>>2;B=c[o>>2]|0;C=(c[m+(x<<2)>>2]>>2)-B|0;D=c[p>>2]|0;E=ac((c[m+(w<<2)>>2]>>2)-B|0,D)|0;B=E+8192|0;E=(ac(C,D)|0)+8192|0;D=c[d+(w<<2)>>2]<<11;w=c[d+(x<<2)>>2]<<11;x=D+8192|0;C=w+8192|0;F=ac(c[q>>2]|0,A)|0;G=ac(c[r>>2]|0,A)|0;A=(ac(c[s>>2]|0,z)|0)+G|0;G=ac(c[t>>2]|0,z)|0;z=BP(291)|0;if((z|0)==0){H=7780;break}I=F+B|0;J=I>>>0>1073741823;K=I>>>14&65535;do{if((a[z+7|0]&1)==0){if(!J){L=K;break}L=-I>>31>>>14&65535}else{if(J){M=-I>>31>>>14&65535}else{M=K}L=cT(M|0)|0}}while(0);y=L;a[u]=y&255;y=y>>8;a[u+1|0]=y&255;K=BP(291)|0;if((K|0)==0){H=7788;break}I=A+B|0;J=I>>>0>1073741823;z=I>>>14&65535;if((a[K+7|0]&1)==0){if(J){N=-I>>31>>>14&65535}else{N=z}K=u+2|0;y=N;a[K]=y&255;y=y>>8;a[K+1|0]=y&255}else{if(J){O=-I>>31>>>14&65535}else{O=z}z=u+2|0;y=cT(O|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(291)|0;if((z|0)==0){H=7797;break}I=G+B|0;J=I>>>0>1073741823;K=I>>>14&65535;if((a[z+7|0]&1)==0){if(J){P=-I>>31>>>14&65535}else{P=K}z=u+4|0;y=P;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(J){Q=-I>>31>>>14&65535}else{Q=K}K=u+4|0;y=cT(Q|0)|0;a[K]=y&255;y=y>>8;a[K+1|0]=y&255}K=BP(291)|0;if((K|0)==0){H=7806;break}I=x>>>0>1073741823;J=x>>>14&65535;if((a[K+7|0]&1)==0){if(I){R=-8192-D>>31>>>14&65535}else{R=J}K=u+6|0;y=R;a[K]=y&255;y=y>>8;a[K+1|0]=y&255}else{if(I){S=-8192-D>>31>>>14&65535}else{S=J}J=u+6|0;y=cT(S|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(291)|0;if((J|0)==0){H=7815;break}I=F+E|0;K=I>>>0>1073741823;z=I>>>14&65535;if((a[J+7|0]&1)==0){if(K){T=-I>>31>>>14&65535}else{T=z}J=u+8|0;y=T;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(K){U=-I>>31>>>14&65535}else{U=z}z=u+8|0;y=cT(U|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(291)|0;if((z|0)==0){H=7824;break}I=A+E|0;K=I>>>0>1073741823;J=I>>>14&65535;if((a[z+7|0]&1)==0){if(K){V=-I>>31>>>14&65535}else{V=J}z=u+10|0;y=V;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(K){W=-I>>31>>>14&65535}else{W=J}J=u+10|0;y=cT(W|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(291)|0;if((J|0)==0){H=7833;break}I=G+E|0;K=I>>>0>1073741823;z=I>>>14&65535;if((a[J+7|0]&1)==0){if(K){X=-I>>31>>>14&65535}else{X=z}J=u+12|0;y=X;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(K){Y=-I>>31>>>14&65535}else{Y=z}z=u+12|0;y=cT(Y|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(291)|0;if((z|0)==0){H=7842;break}I=C>>>0>1073741823;K=C>>>14&65535;if((a[z+7|0]&1)==0){if(I){Z=-8192-w>>31>>>14&65535}else{Z=K}z=u+14|0;y=Z;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(I){_=-8192-w>>31>>>14&65535}else{_=K}K=u+14|0;y=cT(_|0)|0;a[K]=y&255;y=y>>8;a[K+1|0]=y&255}K=v+1|0;if((K|0)<(k|0)){u=u+16|0;v=K}else{H=7926;break}}if((H|0)==7842){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7797){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7788){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7815){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7824){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7926){i=l;return}else if((H|0)==7833){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7806){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7780){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}}else{v=c[e+4>>2]|0;e=c[f+4>>2]|0;f=j+1>>1;if((f|0)<=0){i=l;return}j=b+11284|0;u=b+11288|0;k=b+11292|0;_=b+11296|0;Z=b+11300|0;Y=b+11304|0;b=g;g=0;while(1){X=g<<1;W=X|1;V=(c[h+(g<<2)>>2]|0)-524288+(c[v+(g<<2)>>2]|0)>>3;U=(c[n+(g<<2)>>2]|0)-524288+(c[e+(g<<2)>>2]|0)>>3;T=c[j>>2]|0;S=(c[m+(W<<2)>>2]>>2)-T|0;R=c[u>>2]|0;Q=ac((c[m+(X<<2)>>2]>>2)-T|0,R)|0;T=Q+8192|0;Q=(ac(S,R)|0)+8192|0;R=c[d+(X<<2)>>2]<<11;X=c[d+(W<<2)>>2]<<11;W=R+8192|0;S=X+8192|0;P=ac(c[k>>2]|0,U)|0;O=ac(c[_>>2]|0,U)|0;U=(ac(c[Z>>2]|0,V)|0)+O|0;O=ac(c[Y>>2]|0,V)|0;V=BP(291)|0;if((V|0)==0){H=7854;break}N=P+T|0;L=N>>>0>1073741823;M=N>>>14&65535;do{if((a[V+7|0]&1)==0){if(!L){aa=M;break}aa=-N>>31>>>14&65535}else{if(L){ab=-N>>31>>>14&65535}else{ab=M}aa=cT(ab|0)|0}}while(0);y=aa;a[b]=y&255;y=y>>8;a[b+1|0]=y&255;M=BP(291)|0;if((M|0)==0){H=7862;break}N=U+T|0;L=N>>>0>1073741823;V=N>>>14&65535;if((a[M+7|0]&1)==0){if(L){ad=-N>>31>>>14&65535}else{ad=V}M=b+2|0;y=ad;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}else{if(L){ae=-N>>31>>>14&65535}else{ae=V}V=b+2|0;y=cT(ae|0)|0;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}V=BP(291)|0;if((V|0)==0){H=7871;break}N=O+T|0;L=N>>>0>1073741823;M=N>>>14&65535;if((a[V+7|0]&1)==0){if(L){af=-N>>31>>>14&65535}else{af=M}V=b+4|0;y=af;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}else{if(L){ag=-N>>31>>>14&65535}else{ag=M}M=b+4|0;y=cT(ag|0)|0;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}M=BP(291)|0;if((M|0)==0){H=7880;break}N=W>>>0>1073741823;L=W>>>14&65535;if((a[M+7|0]&1)==0){if(N){ah=-8192-R>>31>>>14&65535}else{ah=L}M=b+6|0;y=ah;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}else{if(N){ai=-8192-R>>31>>>14&65535}else{ai=L}L=b+6|0;y=cT(ai|0)|0;a[L]=y&255;y=y>>8;a[L+1|0]=y&255}L=BP(291)|0;if((L|0)==0){H=7889;break}N=P+Q|0;M=N>>>0>1073741823;V=N>>>14&65535;if((a[L+7|0]&1)==0){if(M){aj=-N>>31>>>14&65535}else{aj=V}L=b+8|0;y=aj;a[L]=y&255;y=y>>8;a[L+1|0]=y&255}else{if(M){ak=-N>>31>>>14&65535}else{ak=V}V=b+8|0;y=cT(ak|0)|0;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}V=BP(291)|0;if((V|0)==0){H=7898;break}N=U+Q|0;M=N>>>0>1073741823;L=N>>>14&65535;if((a[V+7|0]&1)==0){if(M){al=-N>>31>>>14&65535}else{al=L}V=b+10|0;y=al;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}else{if(M){am=-N>>31>>>14&65535}else{am=L}L=b+10|0;y=cT(am|0)|0;a[L]=y&255;y=y>>8;a[L+1|0]=y&255}L=BP(291)|0;if((L|0)==0){H=7907;break}N=O+Q|0;M=N>>>0>1073741823;V=N>>>14&65535;if((a[L+7|0]&1)==0){if(M){an=-N>>31>>>14&65535}else{an=V}L=b+12|0;y=an;a[L]=y&255;y=y>>8;a[L+1|0]=y&255}else{if(M){ao=-N>>31>>>14&65535}else{ao=V}V=b+12|0;y=cT(ao|0)|0;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}V=BP(291)|0;if((V|0)==0){H=7916;break}N=S>>>0>1073741823;M=S>>>14&65535;if((a[V+7|0]&1)==0){if(N){ap=-8192-X>>31>>>14&65535}else{ap=M}V=b+14|0;y=ap;a[V]=y&255;y=y>>8;a[V+1|0]=y&255}else{if(N){aq=-8192-X>>31>>>14&65535}else{aq=M}M=b+14|0;y=cT(aq|0)|0;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}M=g+1|0;if((M|0)<(f|0)){b=b+16|0;g=M}else{H=7928;break}}if((H|0)==7880){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7854){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7907){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7928){i=l;return}else if((H|0)==7871){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7916){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7862){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7898){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}else if((H|0)==7889){AN(0,0,1639912,($=i,i=i+24|0,c[$>>2]=1671168,c[$+8>>2]=1656384,c[$+16>>2]=635,$)|0);i=$;cw()}}}function uC(b,d,e,f,g,h,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0;m=i;n=c[d>>2]|0;o=c[d+4>>2]|0;d=c[e>>2]|0;p=c[e+4>>2]|0;e=c[f>>2]|0;q=c[f+4>>2]|0;f=c[g>>2]|0;r=c[g+4>>2]|0;g=4096-k|0;s=4096-l|0;t=j+1>>1;if((t|0)<=0){i=m;return}j=b+11284|0;u=b+11288|0;v=b+11292|0;w=b+11296|0;x=b+11300|0;z=b+11304|0;b=0;A=h;while(1){h=b<<1;B=ac(c[n+(h<<2)>>2]|0,g)|0;C=(ac(c[o+(h<<2)>>2]|0,k)|0)+B>>14;B=h|1;D=ac(c[n+(B<<2)>>2]|0,g)|0;E=(ac(c[o+(B<<2)>>2]|0,k)|0)+D>>14;D=ac(c[d+(b<<2)>>2]|0,s)|0;F=D-1073741824+(ac(c[p+(b<<2)>>2]|0,l)|0)>>14;D=ac(c[e+(b<<2)>>2]|0,s)|0;G=D-1073741824+(ac(c[q+(b<<2)>>2]|0,l)|0)>>14;D=c[j>>2]|0;H=c[u>>2]|0;I=ac(C-D|0,H)|0;C=I+8192|0;I=(ac(E-D|0,H)|0)+8192|0;H=ac(G,c[v>>2]|0)|0;D=ac(c[w>>2]|0,G)|0;G=(ac(c[x>>2]|0,F)|0)+D|0;D=ac(c[z>>2]|0,F)|0;F=ac(c[f+(h<<2)>>2]|0,g)|0;E=(ac(c[r+(h<<2)>>2]|0,k)|0)+F>>1;F=ac(c[f+(B<<2)>>2]|0,g)|0;h=(ac(c[r+(B<<2)>>2]|0,k)|0)+F>>1;F=E+8192|0;B=h+8192|0;J=BP(291)|0;if((J|0)==0){K=7933;break}L=H+C|0;M=L>>>0>1073741823;N=L>>>14&65535;do{if((a[J+7|0]&1)==0){if(!M){O=N;break}O=-L>>31>>>14&65535}else{if(M){P=-L>>31>>>14&65535}else{P=N}O=cT(P|0)|0}}while(0);y=O;a[A]=y&255;y=y>>8;a[A+1|0]=y&255;N=BP(291)|0;if((N|0)==0){K=7941;break}L=G+C|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Q=-L>>31>>>14&65535}else{Q=J}N=A+2|0;y=Q;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){R=-L>>31>>>14&65535}else{R=J}J=A+2|0;y=cT(R|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(291)|0;if((J|0)==0){K=7950;break}L=D+C|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){S=-L>>31>>>14&65535}else{S=N}J=A+4|0;y=S;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){T=-L>>31>>>14&65535}else{T=N}N=A+4|0;y=cT(T|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(291)|0;if((N|0)==0){K=7959;break}L=F>>>0>1073741823;M=F>>>14&65535;if((a[N+7|0]&1)==0){if(L){U=-8192-E>>31>>>14&65535}else{U=M}N=A+6|0;y=U;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(L){V=-8192-E>>31>>>14&65535}else{V=M}M=A+6|0;y=cT(V|0)|0;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}M=BP(291)|0;if((M|0)==0){K=7968;break}L=H+I|0;N=L>>>0>1073741823;J=L>>>14&65535;if((a[M+7|0]&1)==0){if(N){W=-L>>31>>>14&65535}else{W=J}M=A+8|0;y=W;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}else{if(N){X=-L>>31>>>14&65535}else{X=J}J=A+8|0;y=cT(X|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(291)|0;if((J|0)==0){K=7977;break}L=G+I|0;N=L>>>0>1073741823;M=L>>>14&65535;if((a[J+7|0]&1)==0){if(N){Y=-L>>31>>>14&65535}else{Y=M}J=A+10|0;y=Y;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(N){Z=-L>>31>>>14&65535}else{Z=M}M=A+10|0;y=cT(Z|0)|0;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}M=BP(291)|0;if((M|0)==0){K=7986;break}L=D+I|0;N=L>>>0>1073741823;J=L>>>14&65535;if((a[M+7|0]&1)==0){if(N){_=-L>>31>>>14&65535}else{_=J}M=A+12|0;y=_;a[M]=y&255;y=y>>8;a[M+1|0]=y&255}else{if(N){$=-L>>31>>>14&65535}else{$=J}J=A+12|0;y=cT($|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(291)|0;if((J|0)==0){K=7995;break}L=B>>>0>1073741823;N=B>>>14&65535;if((a[J+7|0]&1)==0){if(L){aa=-8192-h>>31>>>14&65535}else{aa=N}J=A+14|0;y=aa;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(L){ab=-8192-h>>31>>>14&65535}else{ab=N}N=A+14|0;y=cT(ab|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=b+1|0;if((N|0)<(t|0)){b=N;A=A+16|0}else{K=8006;break}}if((K|0)==7959){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7933){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==8006){i=m;return}else if((K|0)==7941){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7977){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7986){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7968){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7995){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}else if((K|0)==7950){AN(0,0,1639912,(ad=i,i=i+24|0,c[ad>>2]=1671168,c[ad+8>>2]=1656384,c[ad+16>>2]=635,ad)|0);i=ad;cw()}}function uD(d,e,f,g,h,j,k,l,m,n,o,p){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0;p=i;q=o+1>>1;if((q|0)<=0){i=p;return}o=(g|0)>0;r=(l|0)>0;s=d+11284|0;t=d+11288|0;u=d+11292|0;v=d+11296|0;w=d+11300|0;x=d+11304|0;d=0;z=n;while(1){if(o){n=d<<1;A=n|1;B=0;C=-1073741824;D=-1073741824;do{E=c[f+(B<<2)>>2]|0;F=b[e+(B<<1)>>1]|0;C=(ac(F,c[E+(n<<2)>>2]|0)|0)+C|0;D=(ac(c[E+(A<<2)>>2]|0,F)|0)+D|0;B=B+1|0;}while((B|0)<(g|0));G=(C>>14)+65536|0;H=(D>>14)+65536|0}else{G=0;H=0}if(r){B=0;A=-1073741824;n=-1073741824;do{F=b[h+(B<<1)>>1]|0;A=(ac(F,c[(c[j+(B<<2)>>2]|0)+(d<<2)>>2]|0)|0)+A|0;n=(ac(c[(c[k+(B<<2)>>2]|0)+(d<<2)>>2]|0,F)|0)+n|0;B=B+1|0;}while((B|0)<(l|0));I=A>>14;J=n>>14}else{I=-65536;J=-65536}if(o){B=d<<1;D=B|1;C=0;F=-1073741824;E=-1073741824;do{K=c[m+(C<<2)>>2]|0;L=b[e+(C<<1)>>1]|0;E=(ac(L,c[K+(B<<2)>>2]|0)|0)+E|0;F=(ac(c[K+(D<<2)>>2]|0,L)|0)+F|0;C=C+1|0;}while((C|0)<(g|0));M=F>>1;N=E>>1}else{M=-536870912;N=-536870912}C=N+536879104|0;D=M+536879104|0;B=c[s>>2]|0;n=c[t>>2]|0;A=ac(G-B|0,n)|0;L=A+8192|0;A=(ac(H-B|0,n)|0)+8192|0;n=ac(c[u>>2]|0,J)|0;B=ac(c[v>>2]|0,J)|0;K=(ac(c[w>>2]|0,I)|0)+B|0;B=ac(c[x>>2]|0,I)|0;O=BP(291)|0;if((O|0)==0){P=8021;break}Q=n+L|0;R=Q>>>0>1073741823;S=Q>>>14&65535;do{if((a[O+7|0]&1)==0){if(!R){T=S;break}T=-Q>>31>>>14&65535}else{if(R){U=-Q>>31>>>14&65535}else{U=S}T=cT(U|0)|0}}while(0);y=T;a[z]=y&255;y=y>>8;a[z+1|0]=y&255;S=BP(291)|0;if((S|0)==0){P=8029;break}Q=K+L|0;R=Q>>>0>1073741823;O=Q>>>14&65535;if((a[S+7|0]&1)==0){if(R){V=-Q>>31>>>14&65535}else{V=O}S=z+2|0;y=V;a[S]=y&255;y=y>>8;a[S+1|0]=y&255}else{if(R){W=-Q>>31>>>14&65535}else{W=O}O=z+2|0;y=cT(W|0)|0;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}O=BP(291)|0;if((O|0)==0){P=8038;break}Q=B+L|0;R=Q>>>0>1073741823;S=Q>>>14&65535;if((a[O+7|0]&1)==0){if(R){X=-Q>>31>>>14&65535}else{X=S}O=z+4|0;y=X;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}else{if(R){Y=-Q>>31>>>14&65535}else{Y=S}S=z+4|0;y=cT(Y|0)|0;a[S]=y&255;y=y>>8;a[S+1|0]=y&255}S=BP(291)|0;if((S|0)==0){P=8047;break}Q=C>>>0>1073741823;R=C>>>14&65535;if((a[S+7|0]&1)==0){if(Q){Z=-536879104-N>>31>>>14&65535}else{Z=R}S=z+6|0;y=Z;a[S]=y&255;y=y>>8;a[S+1|0]=y&255}else{if(Q){_=-536879104-N>>31>>>14&65535}else{_=R}R=z+6|0;y=cT(_|0)|0;a[R]=y&255;y=y>>8;a[R+1|0]=y&255}R=BP(291)|0;if((R|0)==0){P=8056;break}Q=n+A|0;S=Q>>>0>1073741823;O=Q>>>14&65535;if((a[R+7|0]&1)==0){if(S){$=-Q>>31>>>14&65535}else{$=O}R=z+8|0;y=$;a[R]=y&255;y=y>>8;a[R+1|0]=y&255}else{if(S){aa=-Q>>31>>>14&65535}else{aa=O}O=z+8|0;y=cT(aa|0)|0;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}O=BP(291)|0;if((O|0)==0){P=8065;break}Q=K+A|0;S=Q>>>0>1073741823;R=Q>>>14&65535;if((a[O+7|0]&1)==0){if(S){ab=-Q>>31>>>14&65535}else{ab=R}O=z+10|0;y=ab;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}else{if(S){ad=-Q>>31>>>14&65535}else{ad=R}R=z+10|0;y=cT(ad|0)|0;a[R]=y&255;y=y>>8;a[R+1|0]=y&255}R=BP(291)|0;if((R|0)==0){P=8074;break}Q=B+A|0;S=Q>>>0>1073741823;O=Q>>>14&65535;if((a[R+7|0]&1)==0){if(S){ae=-Q>>31>>>14&65535}else{ae=O}R=z+12|0;y=ae;a[R]=y&255;y=y>>8;a[R+1|0]=y&255}else{if(S){af=-Q>>31>>>14&65535}else{af=O}O=z+12|0;y=cT(af|0)|0;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}O=BP(291)|0;if((O|0)==0){P=8083;break}Q=D>>>0>1073741823;S=D>>>14&65535;if((a[O+7|0]&1)==0){if(Q){ag=-536879104-M>>31>>>14&65535}else{ag=S}O=z+14|0;y=ag;a[O]=y&255;y=y>>8;a[O+1|0]=y&255}else{if(Q){ah=-536879104-M>>31>>>14&65535}else{ah=S}S=z+14|0;y=cT(ah|0)|0;a[S]=y&255;y=y>>8;a[S+1|0]=y&255}S=d+1|0;if((S|0)<(q|0)){d=S;z=z+16|0}else{P=8094;break}}if((P|0)==8047){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==8065){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==8021){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==8094){i=p;return}else if((P|0)==8038){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==8074){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==8029){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==8056){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}else if((P|0)==8083){AN(0,0,1639912,(ai=i,i=i+24|0,c[ai>>2]=1671168,c[ai+8>>2]=1656384,c[ai+16>>2]=635,ai)|0);i=ai;cw()}}function uE(b,d,e,f,g,h,j,k,l){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0;l=i;g=d;d=h;h=c[e>>2]|0;m=c[f>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){i=l;return}n=b+11284|0;o=b+11288|0;p=b+11292|0;q=b+11296|0;r=b+11300|0;s=b+11304|0;t=d;u=0;while(1){v=u<<1;w=(c[h+(u<<2)>>2]|0)-262144>>2;x=(c[m+(u<<2)>>2]|0)-262144>>2;z=c[n>>2]|0;A=(c[g+((v|1)<<2)>>2]>>2)-z|0;B=c[o>>2]|0;C=ac((c[g+(v<<2)>>2]>>2)-z|0,B)|0;z=C+8192|0;C=(ac(A,B)|0)+8192|0;B=ac(c[p>>2]|0,x)|0;A=ac(c[q>>2]|0,x)|0;x=(ac(c[r>>2]|0,w)|0)+A|0;A=ac(c[s>>2]|0,w)|0;w=BP(291)|0;if((w|0)==0){D=8099;break}v=B+z|0;E=v>>>0>1073741823;F=v>>>14&65535;do{if((a[w+7|0]&1)==0){if(!E){G=F;break}G=-v>>31>>>14&65535}else{if(E){H=-v>>31>>>14&65535}else{H=F}G=cT(H|0)|0}}while(0);y=G;a[t]=y&255;y=y>>8;a[t+1|0]=y&255;F=BP(291)|0;if((F|0)==0){D=8107;break}v=x+z|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){I=-v>>31>>>14&65535}else{I=w}F=t+2|0;y=I;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){J=-v>>31>>>14&65535}else{J=w}w=t+2|0;y=cT(J|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(291)|0;if((w|0)==0){D=8116;break}v=A+z|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){K=-v>>31>>>14&65535}else{K=F}w=t+4|0;y=K;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){L=-v>>31>>>14&65535}else{L=F}F=t+4|0;y=cT(L|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}if((BP(291)|0)==0){D=8125;break}F=t+6|0;y=-1;a[F]=y&255;y=y>>8;a[F+1|0]=y&255;F=BP(291)|0;if((F|0)==0){D=8127;break}v=B+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){M=-v>>31>>>14&65535}else{M=w}F=t+8|0;y=M;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){N=-v>>31>>>14&65535}else{N=w}w=t+8|0;y=cT(N|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(291)|0;if((w|0)==0){D=8136;break}v=x+C|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){O=-v>>31>>>14&65535}else{O=F}w=t+10|0;y=O;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){P=-v>>31>>>14&65535}else{P=F}F=t+10|0;y=cT(P|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}F=BP(291)|0;if((F|0)==0){D=8145;break}v=A+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){Q=-v>>31>>>14&65535}else{Q=w}F=t+12|0;y=Q;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){R=-v>>31>>>14&65535}else{R=w}w=t+12|0;y=cT(R|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}if((BP(291)|0)==0){D=8154;break}w=t+14|0;y=-1;a[w]=y&255;y=y>>8;a[w+1|0]=y&255;w=u+1|0;if((w|0)<(k|0)){t=t+16|0;u=w}else{D=8218;break}}if((D|0)==8099){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8125){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8107){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8218){i=l;return}else if((D|0)==8154){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8127){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8145){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8116){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8136){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}else{u=c[e+4>>2]|0;e=c[f+4>>2]|0;f=j+1>>1;if((f|0)<=0){i=l;return}j=b+11284|0;t=b+11288|0;k=b+11292|0;R=b+11296|0;Q=b+11300|0;P=b+11304|0;b=d;d=0;while(1){O=d<<1;N=(c[h+(d<<2)>>2]|0)-524288+(c[u+(d<<2)>>2]|0)>>3;M=(c[m+(d<<2)>>2]|0)-524288+(c[e+(d<<2)>>2]|0)>>3;L=c[j>>2]|0;K=(c[g+((O|1)<<2)>>2]>>2)-L|0;J=c[t>>2]|0;I=ac((c[g+(O<<2)>>2]>>2)-L|0,J)|0;L=I+8192|0;I=(ac(K,J)|0)+8192|0;J=ac(c[k>>2]|0,M)|0;K=ac(c[R>>2]|0,M)|0;M=(ac(c[Q>>2]|0,N)|0)+K|0;K=ac(c[P>>2]|0,N)|0;N=BP(291)|0;if((N|0)==0){D=8159;break}O=J+L|0;G=O>>>0>1073741823;H=O>>>14&65535;do{if((a[N+7|0]&1)==0){if(!G){T=H;break}T=-O>>31>>>14&65535}else{if(G){U=-O>>31>>>14&65535}else{U=H}T=cT(U|0)|0}}while(0);y=T;a[b]=y&255;y=y>>8;a[b+1|0]=y&255;H=BP(291)|0;if((H|0)==0){D=8167;break}O=M+L|0;G=O>>>0>1073741823;N=O>>>14&65535;if((a[H+7|0]&1)==0){if(G){V=-O>>31>>>14&65535}else{V=N}H=b+2|0;y=V;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){W=-O>>31>>>14&65535}else{W=N}N=b+2|0;y=cT(W|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(291)|0;if((N|0)==0){D=8176;break}O=K+L|0;G=O>>>0>1073741823;H=O>>>14&65535;if((a[N+7|0]&1)==0){if(G){X=-O>>31>>>14&65535}else{X=H}N=b+4|0;y=X;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){Y=-O>>31>>>14&65535}else{Y=H}H=b+4|0;y=cT(Y|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}if((BP(291)|0)==0){D=8185;break}H=b+6|0;y=-1;a[H]=y&255;y=y>>8;a[H+1|0]=y&255;H=BP(291)|0;if((H|0)==0){D=8187;break}O=J+I|0;G=O>>>0>1073741823;N=O>>>14&65535;if((a[H+7|0]&1)==0){if(G){Z=-O>>31>>>14&65535}else{Z=N}H=b+8|0;y=Z;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){_=-O>>31>>>14&65535}else{_=N}N=b+8|0;y=cT(_|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(291)|0;if((N|0)==0){D=8196;break}O=M+I|0;G=O>>>0>1073741823;H=O>>>14&65535;if((a[N+7|0]&1)==0){if(G){$=-O>>31>>>14&65535}else{$=H}N=b+10|0;y=$;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){aa=-O>>31>>>14&65535}else{aa=H}H=b+10|0;y=cT(aa|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(291)|0;if((H|0)==0){D=8205;break}O=K+I|0;G=O>>>0>1073741823;N=O>>>14&65535;if((a[H+7|0]&1)==0){if(G){ab=-O>>31>>>14&65535}else{ab=N}H=b+12|0;y=ab;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){ad=-O>>31>>>14&65535}else{ad=N}N=b+12|0;y=cT(ad|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}if((BP(291)|0)==0){D=8214;break}N=b+14|0;y=-1;a[N]=y&255;y=y>>8;a[N+1|0]=y&255;N=d+1|0;if((N|0)<(f|0)){b=b+16|0;d=N}else{D=8220;break}}if((D|0)==8196){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8220){i=l;return}else if((D|0)==8214){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8159){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8205){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8185){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8187){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8176){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8167){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}}function uF(b,d,e,f,g,h,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;m=i;g=c[d>>2]|0;n=c[d+4>>2]|0;d=c[e>>2]|0;o=c[e+4>>2]|0;e=c[f>>2]|0;p=c[f+4>>2]|0;f=4096-k|0;q=4096-l|0;r=j+1>>1;if((r|0)<=0){i=m;return}j=b+11284|0;s=b+11288|0;t=b+11292|0;u=b+11296|0;v=b+11300|0;w=b+11304|0;b=0;x=h;while(1){h=b<<1;z=ac(c[g+(h<<2)>>2]|0,f)|0;A=(ac(c[n+(h<<2)>>2]|0,k)|0)+z>>14;z=h|1;h=ac(c[g+(z<<2)>>2]|0,f)|0;B=(ac(c[n+(z<<2)>>2]|0,k)|0)+h>>14;h=ac(c[d+(b<<2)>>2]|0,q)|0;z=h-1073741824+(ac(c[o+(b<<2)>>2]|0,l)|0)>>14;h=ac(c[e+(b<<2)>>2]|0,q)|0;C=h-1073741824+(ac(c[p+(b<<2)>>2]|0,l)|0)>>14;h=c[j>>2]|0;D=c[s>>2]|0;E=ac(A-h|0,D)|0;A=E+8192|0;E=(ac(B-h|0,D)|0)+8192|0;D=ac(C,c[t>>2]|0)|0;h=ac(c[u>>2]|0,C)|0;C=(ac(c[v>>2]|0,z)|0)+h|0;h=ac(c[w>>2]|0,z)|0;z=BP(291)|0;if((z|0)==0){F=8224;break}B=D+A|0;G=B>>>0>1073741823;H=B>>>14&65535;do{if((a[z+7|0]&1)==0){if(!G){I=H;break}I=-B>>31>>>14&65535}else{if(G){J=-B>>31>>>14&65535}else{J=H}I=cT(J|0)|0}}while(0);y=I;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;H=BP(291)|0;if((H|0)==0){F=8232;break}B=C+A|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){K=-B>>31>>>14&65535}else{K=z}H=x+2|0;y=K;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){L=-B>>31>>>14&65535}else{L=z}z=x+2|0;y=cT(L|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(291)|0;if((z|0)==0){F=8241;break}B=h+A|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){M=-B>>31>>>14&65535}else{M=H}z=x+4|0;y=M;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){N=-B>>31>>>14&65535}else{N=H}H=x+4|0;y=cT(N|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}if((BP(291)|0)==0){F=8250;break}H=x+6|0;y=-1;a[H]=y&255;y=y>>8;a[H+1|0]=y&255;H=BP(291)|0;if((H|0)==0){F=8252;break}B=D+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){O=-B>>31>>>14&65535}else{O=z}H=x+8|0;y=O;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){P=-B>>31>>>14&65535}else{P=z}z=x+8|0;y=cT(P|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(291)|0;if((z|0)==0){F=8261;break}B=C+E|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){Q=-B>>31>>>14&65535}else{Q=H}z=x+10|0;y=Q;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){R=-B>>31>>>14&65535}else{R=H}H=x+10|0;y=cT(R|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(291)|0;if((H|0)==0){F=8270;break}B=h+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){S=-B>>31>>>14&65535}else{S=z}H=x+12|0;y=S;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){T=-B>>31>>>14&65535}else{T=z}z=x+12|0;y=cT(T|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}if((BP(291)|0)==0){F=8279;break}z=x+14|0;y=-1;a[z]=y&255;y=y>>8;a[z+1|0]=y&255;z=b+1|0;if((z|0)<(r|0)){b=z;x=x+16|0}else{F=8283;break}}if((F|0)==8241){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8283){i=m;return}else if((F|0)==8232){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8224){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8250){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8279){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8270){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8252){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8261){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}}function uG(d,e,f,g,h,j,k,l,m,n,o,p){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0;p=i;m=o+1>>1;if((m|0)<=0){i=p;return}o=(g|0)>0;q=(l|0)>0;r=d+11284|0;s=d+11288|0;t=d+11292|0;u=d+11296|0;v=d+11300|0;w=d+11304|0;d=0;x=n;while(1){if(o){n=d<<1;z=n|1;A=0;B=-1073741824;C=-1073741824;do{D=c[f+(A<<2)>>2]|0;E=b[e+(A<<1)>>1]|0;B=(ac(E,c[D+(n<<2)>>2]|0)|0)+B|0;C=(ac(c[D+(z<<2)>>2]|0,E)|0)+C|0;A=A+1|0;}while((A|0)<(g|0));F=(B>>14)+65536|0;G=(C>>14)+65536|0}else{F=0;G=0}if(q){A=0;z=-1073741824;n=-1073741824;do{E=b[h+(A<<1)>>1]|0;z=(ac(E,c[(c[j+(A<<2)>>2]|0)+(d<<2)>>2]|0)|0)+z|0;n=(ac(c[(c[k+(A<<2)>>2]|0)+(d<<2)>>2]|0,E)|0)+n|0;A=A+1|0;}while((A|0)<(l|0));H=z>>14;I=n>>14}else{H=-65536;I=-65536}A=c[r>>2]|0;C=c[s>>2]|0;B=ac(F-A|0,C)|0;E=B+8192|0;B=(ac(G-A|0,C)|0)+8192|0;C=ac(c[t>>2]|0,I)|0;A=ac(c[u>>2]|0,I)|0;D=(ac(c[v>>2]|0,H)|0)+A|0;A=ac(c[w>>2]|0,H)|0;J=BP(291)|0;if((J|0)==0){K=8294;break}L=C+E|0;M=L>>>0>1073741823;N=L>>>14&65535;do{if((a[J+7|0]&1)==0){if(!M){O=N;break}O=-L>>31>>>14&65535}else{if(M){P=-L>>31>>>14&65535}else{P=N}O=cT(P|0)|0}}while(0);y=O;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;N=BP(291)|0;if((N|0)==0){K=8302;break}L=D+E|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Q=-L>>31>>>14&65535}else{Q=J}N=x+2|0;y=Q;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){R=-L>>31>>>14&65535}else{R=J}J=x+2|0;y=cT(R|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(291)|0;if((J|0)==0){K=8311;break}L=A+E|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){S=-L>>31>>>14&65535}else{S=N}J=x+4|0;y=S;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){T=-L>>31>>>14&65535}else{T=N}N=x+4|0;y=cT(T|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}if((BP(291)|0)==0){K=8320;break}N=x+6|0;y=-1;a[N]=y&255;y=y>>8;a[N+1|0]=y&255;N=BP(291)|0;if((N|0)==0){K=8322;break}L=C+B|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){U=-L>>31>>>14&65535}else{U=J}N=x+8|0;y=U;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){V=-L>>31>>>14&65535}else{V=J}J=x+8|0;y=cT(V|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(291)|0;if((J|0)==0){K=8331;break}L=D+B|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){W=-L>>31>>>14&65535}else{W=N}J=x+10|0;y=W;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){X=-L>>31>>>14&65535}else{X=N}N=x+10|0;y=cT(X|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(291)|0;if((N|0)==0){K=8340;break}L=A+B|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Y=-L>>31>>>14&65535}else{Y=J}N=x+12|0;y=Y;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){Z=-L>>31>>>14&65535}else{Z=J}J=x+12|0;y=cT(Z|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}if((BP(291)|0)==0){K=8349;break}J=x+14|0;y=-1;a[J]=y&255;y=y>>8;a[J+1|0]=y&255;J=d+1|0;if((J|0)<(m|0)){d=J;x=x+16|0}else{K=8353;break}}if((K|0)==8311){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8322){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8331){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8294){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8349){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8302){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8320){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8340){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8353){i=p;return}}function uH(b,d,e,f,g,h,j,k,l){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0;l=i;g=d;d=h;h=c[e>>2]|0;m=c[f>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){i=l;return}n=b+11284|0;o=b+11288|0;p=b+11292|0;q=b+11296|0;r=b+11300|0;s=b+11304|0;t=0;u=d;while(1){v=t<<1;w=(c[h+(t<<2)>>2]|0)-262144>>2;x=(c[m+(t<<2)>>2]|0)-262144>>2;z=c[n>>2]|0;A=(c[g+((v|1)<<2)>>2]>>2)-z|0;B=c[o>>2]|0;C=ac((c[g+(v<<2)>>2]>>2)-z|0,B)|0;z=C+8192|0;C=(ac(A,B)|0)+8192|0;B=ac(c[p>>2]|0,x)|0;A=ac(c[q>>2]|0,x)|0;x=(ac(c[r>>2]|0,w)|0)+A|0;A=ac(c[s>>2]|0,w)|0;w=BP(42)|0;if((w|0)==0){D=8358;break}v=B+z|0;E=v>>>0>1073741823;F=v>>>14&65535;do{if((a[w+7|0]&1)==0){if(!E){G=F;break}G=-v>>31>>>14&65535}else{if(E){H=-v>>31>>>14&65535}else{H=F}G=cT(H|0)|0}}while(0);y=G;a[u]=y&255;y=y>>8;a[u+1|0]=y&255;F=BP(42)|0;if((F|0)==0){D=8366;break}v=x+z|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){I=-v>>31>>>14&65535}else{I=w}F=u+2|0;y=I;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){J=-v>>31>>>14&65535}else{J=w}w=u+2|0;y=cT(J|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(42)|0;if((w|0)==0){D=8375;break}v=A+z|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){K=-v>>31>>>14&65535}else{K=F}w=u+4|0;y=K;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){L=-v>>31>>>14&65535}else{L=F}F=u+4|0;y=cT(L|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}F=BP(42)|0;if((F|0)==0){D=8384;break}v=B+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){M=-v>>31>>>14&65535}else{M=w}F=u+6|0;y=M;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){N=-v>>31>>>14&65535}else{N=w}w=u+6|0;y=cT(N|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(42)|0;if((w|0)==0){D=8393;break}v=x+C|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){O=-v>>31>>>14&65535}else{O=F}w=u+8|0;y=O;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){P=-v>>31>>>14&65535}else{P=F}F=u+8|0;y=cT(P|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}F=BP(42)|0;if((F|0)==0){D=8402;break}v=A+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){Q=-v>>31>>>14&65535}else{Q=w}F=u+10|0;y=Q;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){R=-v>>31>>>14&65535}else{R=w}w=u+10|0;y=cT(R|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=t+1|0;if((w|0)<(k|0)){t=w;u=u+12|0}else{D=8471;break}}if((D|0)==8384){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8471){i=l;return}else if((D|0)==8375){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8393){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8366){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8358){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8402){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}else{u=c[e+4>>2]|0;e=c[f+4>>2]|0;f=j+1>>1;if((f|0)<=0){i=l;return}j=b+11284|0;t=b+11288|0;k=b+11292|0;R=b+11296|0;Q=b+11300|0;P=b+11304|0;b=0;O=d;while(1){d=b<<1;N=(c[h+(b<<2)>>2]|0)-524288+(c[u+(b<<2)>>2]|0)>>3;M=(c[m+(b<<2)>>2]|0)-524288+(c[e+(b<<2)>>2]|0)>>3;L=c[j>>2]|0;K=(c[g+((d|1)<<2)>>2]>>2)-L|0;J=c[t>>2]|0;I=ac((c[g+(d<<2)>>2]>>2)-L|0,J)|0;L=I+8192|0;I=(ac(K,J)|0)+8192|0;J=ac(c[k>>2]|0,M)|0;K=ac(c[R>>2]|0,M)|0;M=(ac(c[Q>>2]|0,N)|0)+K|0;K=ac(c[P>>2]|0,N)|0;N=BP(42)|0;if((N|0)==0){D=8414;break}d=J+L|0;G=d>>>0>1073741823;H=d>>>14&65535;do{if((a[N+7|0]&1)==0){if(!G){T=H;break}T=-d>>31>>>14&65535}else{if(G){U=-d>>31>>>14&65535}else{U=H}T=cT(U|0)|0}}while(0);y=T;a[O]=y&255;y=y>>8;a[O+1|0]=y&255;H=BP(42)|0;if((H|0)==0){D=8422;break}d=M+L|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){V=-d>>31>>>14&65535}else{V=N}H=O+2|0;y=V;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){W=-d>>31>>>14&65535}else{W=N}N=O+2|0;y=cT(W|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(42)|0;if((N|0)==0){D=8431;break}d=K+L|0;G=d>>>0>1073741823;H=d>>>14&65535;if((a[N+7|0]&1)==0){if(G){X=-d>>31>>>14&65535}else{X=H}N=O+4|0;y=X;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){Y=-d>>31>>>14&65535}else{Y=H}H=O+4|0;y=cT(Y|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(42)|0;if((H|0)==0){D=8440;break}d=J+I|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){Z=-d>>31>>>14&65535}else{Z=N}H=O+6|0;y=Z;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){_=-d>>31>>>14&65535}else{_=N}N=O+6|0;y=cT(_|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(42)|0;if((N|0)==0){D=8449;break}d=M+I|0;G=d>>>0>1073741823;H=d>>>14&65535;if((a[N+7|0]&1)==0){if(G){$=-d>>31>>>14&65535}else{$=H}N=O+8|0;y=$;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){aa=-d>>31>>>14&65535}else{aa=H}H=O+8|0;y=cT(aa|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(42)|0;if((H|0)==0){D=8458;break}d=K+I|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){ab=-d>>31>>>14&65535}else{ab=N}H=O+10|0;y=ab;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){ad=-d>>31>>>14&65535}else{ad=N}N=O+10|0;y=cT(ad|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=b+1|0;if((N|0)<(f|0)){b=N;O=O+12|0}else{D=8469;break}}if((D|0)==8431){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8458){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8422){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8449){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8469){i=l;return}else if((D|0)==8414){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8440){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}}function uI(b,d,e,f,g,h,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;m=i;g=c[d>>2]|0;n=c[d+4>>2]|0;d=c[e>>2]|0;o=c[e+4>>2]|0;e=c[f>>2]|0;p=c[f+4>>2]|0;f=4096-k|0;q=4096-l|0;r=j+1>>1;if((r|0)<=0){i=m;return}j=b+11284|0;s=b+11288|0;t=b+11292|0;u=b+11296|0;v=b+11300|0;w=b+11304|0;b=0;x=h;while(1){h=b<<1;z=ac(c[g+(h<<2)>>2]|0,f)|0;A=(ac(c[n+(h<<2)>>2]|0,k)|0)+z>>14;z=h|1;h=ac(c[g+(z<<2)>>2]|0,f)|0;B=(ac(c[n+(z<<2)>>2]|0,k)|0)+h>>14;h=ac(c[d+(b<<2)>>2]|0,q)|0;z=h-1073741824+(ac(c[o+(b<<2)>>2]|0,l)|0)>>14;h=ac(c[e+(b<<2)>>2]|0,q)|0;C=h-1073741824+(ac(c[p+(b<<2)>>2]|0,l)|0)>>14;h=c[j>>2]|0;D=c[s>>2]|0;E=ac(A-h|0,D)|0;A=E+8192|0;E=(ac(B-h|0,D)|0)+8192|0;D=ac(C,c[t>>2]|0)|0;h=ac(c[u>>2]|0,C)|0;C=(ac(c[v>>2]|0,z)|0)+h|0;h=ac(c[w>>2]|0,z)|0;z=BP(42)|0;if((z|0)==0){F=8475;break}B=D+A|0;G=B>>>0>1073741823;H=B>>>14&65535;do{if((a[z+7|0]&1)==0){if(!G){I=H;break}I=-B>>31>>>14&65535}else{if(G){J=-B>>31>>>14&65535}else{J=H}I=cT(J|0)|0}}while(0);y=I;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;H=BP(42)|0;if((H|0)==0){F=8483;break}B=C+A|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){K=-B>>31>>>14&65535}else{K=z}H=x+2|0;y=K;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){L=-B>>31>>>14&65535}else{L=z}z=x+2|0;y=cT(L|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(42)|0;if((z|0)==0){F=8492;break}B=h+A|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){M=-B>>31>>>14&65535}else{M=H}z=x+4|0;y=M;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){N=-B>>31>>>14&65535}else{N=H}H=x+4|0;y=cT(N|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(42)|0;if((H|0)==0){F=8501;break}B=D+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){O=-B>>31>>>14&65535}else{O=z}H=x+6|0;y=O;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){P=-B>>31>>>14&65535}else{P=z}z=x+6|0;y=cT(P|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(42)|0;if((z|0)==0){F=8510;break}B=C+E|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){Q=-B>>31>>>14&65535}else{Q=H}z=x+8|0;y=Q;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){R=-B>>31>>>14&65535}else{R=H}H=x+8|0;y=cT(R|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(42)|0;if((H|0)==0){F=8519;break}B=h+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){S=-B>>31>>>14&65535}else{S=z}H=x+10|0;y=S;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){T=-B>>31>>>14&65535}else{T=z}z=x+10|0;y=cT(T|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=b+1|0;if((z|0)<(r|0)){b=z;x=x+12|0}else{F=8530;break}}if((F|0)==8475){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8483){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8530){i=m;return}else if((F|0)==8501){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8510){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8519){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8492){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}}function uJ(d,e,f,g,h,j,k,l,m,n,o,p){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0;p=i;m=o+1>>1;if((m|0)<=0){i=p;return}o=(g|0)>0;q=(l|0)>0;r=d+11284|0;s=d+11288|0;t=d+11292|0;u=d+11296|0;v=d+11300|0;w=d+11304|0;d=0;x=n;while(1){if(o){n=d<<1;z=n|1;A=-1073741824;B=-1073741824;C=0;do{D=c[f+(C<<2)>>2]|0;E=b[e+(C<<1)>>1]|0;B=(ac(E,c[D+(n<<2)>>2]|0)|0)+B|0;A=(ac(c[D+(z<<2)>>2]|0,E)|0)+A|0;C=C+1|0;}while((C|0)<(g|0));F=(A>>14)+65536|0;G=(B>>14)+65536|0}else{F=0;G=0}if(q){C=-1073741824;z=-1073741824;n=0;do{E=b[h+(n<<1)>>1]|0;z=(ac(E,c[(c[j+(n<<2)>>2]|0)+(d<<2)>>2]|0)|0)+z|0;C=(ac(c[(c[k+(n<<2)>>2]|0)+(d<<2)>>2]|0,E)|0)+C|0;n=n+1|0;}while((n|0)<(l|0));H=C>>14;I=z>>14}else{H=-65536;I=-65536}n=c[r>>2]|0;B=c[s>>2]|0;A=ac(G-n|0,B)|0;E=A+8192|0;A=(ac(F-n|0,B)|0)+8192|0;B=ac(c[t>>2]|0,H)|0;n=ac(c[u>>2]|0,H)|0;D=(ac(c[v>>2]|0,I)|0)+n|0;n=ac(c[w>>2]|0,I)|0;J=BP(42)|0;if((J|0)==0){K=8541;break}L=B+E|0;M=L>>>0>1073741823;N=L>>>14&65535;do{if((a[J+7|0]&1)==0){if(!M){O=N;break}O=-L>>31>>>14&65535}else{if(M){P=-L>>31>>>14&65535}else{P=N}O=cT(P|0)|0}}while(0);y=O;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;N=BP(42)|0;if((N|0)==0){K=8549;break}L=D+E|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Q=-L>>31>>>14&65535}else{Q=J}N=x+2|0;y=Q;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){R=-L>>31>>>14&65535}else{R=J}J=x+2|0;y=cT(R|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(42)|0;if((J|0)==0){K=8558;break}L=n+E|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){S=-L>>31>>>14&65535}else{S=N}J=x+4|0;y=S;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){T=-L>>31>>>14&65535}else{T=N}N=x+4|0;y=cT(T|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(42)|0;if((N|0)==0){K=8567;break}L=B+A|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){U=-L>>31>>>14&65535}else{U=J}N=x+6|0;y=U;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){V=-L>>31>>>14&65535}else{V=J}J=x+6|0;y=cT(V|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(42)|0;if((J|0)==0){K=8576;break}L=D+A|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){W=-L>>31>>>14&65535}else{W=N}J=x+8|0;y=W;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){X=-L>>31>>>14&65535}else{X=N}N=x+8|0;y=cT(X|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(42)|0;if((N|0)==0){K=8585;break}L=n+A|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Y=-L>>31>>>14&65535}else{Y=J}N=x+10|0;y=Y;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){Z=-L>>31>>>14&65535}else{Z=J}J=x+10|0;y=cT(Z|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=d+1|0;if((J|0)<(m|0)){d=J;x=x+12|0}else{K=8596;break}}if((K|0)==8576){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8585){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8549){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8596){i=p;return}else if((K|0)==8541){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8567){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8558){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}}function uK(b,d,e,f,g,h,j,k,l){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0;l=i;g=d;d=h;h=c[e>>2]|0;m=c[f>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){i=l;return}n=b+11284|0;o=b+11288|0;p=b+11292|0;q=b+11296|0;r=b+11300|0;s=b+11304|0;t=0;u=d;while(1){v=t<<1;w=(c[h+(t<<2)>>2]|0)-262144>>2;x=(c[m+(t<<2)>>2]|0)-262144>>2;z=c[n>>2]|0;A=(c[g+((v|1)<<2)>>2]>>2)-z|0;B=c[o>>2]|0;C=ac((c[g+(v<<2)>>2]>>2)-z|0,B)|0;z=C+8192|0;C=(ac(A,B)|0)+8192|0;B=ac(c[p>>2]|0,x)|0;A=ac(c[q>>2]|0,x)|0;x=(ac(c[r>>2]|0,w)|0)+A|0;A=ac(c[s>>2]|0,w)|0;w=BP(41)|0;if((w|0)==0){D=8601;break}v=B+z|0;E=v>>>0>1073741823;F=v>>>14&65535;do{if((a[w+7|0]&1)==0){if(!E){G=F;break}G=-v>>31>>>14&65535}else{if(E){H=-v>>31>>>14&65535}else{H=F}G=cT(H|0)|0}}while(0);y=G;a[u]=y&255;y=y>>8;a[u+1|0]=y&255;F=BP(41)|0;if((F|0)==0){D=8609;break}v=x+z|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){I=-v>>31>>>14&65535}else{I=w}F=u+2|0;y=I;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){J=-v>>31>>>14&65535}else{J=w}w=u+2|0;y=cT(J|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(41)|0;if((w|0)==0){D=8618;break}v=A+z|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){K=-v>>31>>>14&65535}else{K=F}w=u+4|0;y=K;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){L=-v>>31>>>14&65535}else{L=F}F=u+4|0;y=cT(L|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}F=BP(41)|0;if((F|0)==0){D=8627;break}v=B+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){M=-v>>31>>>14&65535}else{M=w}F=u+6|0;y=M;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){N=-v>>31>>>14&65535}else{N=w}w=u+6|0;y=cT(N|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(41)|0;if((w|0)==0){D=8636;break}v=x+C|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){O=-v>>31>>>14&65535}else{O=F}w=u+8|0;y=O;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){P=-v>>31>>>14&65535}else{P=F}F=u+8|0;y=cT(P|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}F=BP(41)|0;if((F|0)==0){D=8645;break}v=A+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){Q=-v>>31>>>14&65535}else{Q=w}F=u+10|0;y=Q;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){R=-v>>31>>>14&65535}else{R=w}w=u+10|0;y=cT(R|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=t+1|0;if((w|0)<(k|0)){t=w;u=u+12|0}else{D=8714;break}}if((D|0)==8627){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8601){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8714){i=l;return}else if((D|0)==8609){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8645){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8636){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8618){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}else{u=c[e+4>>2]|0;e=c[f+4>>2]|0;f=j+1>>1;if((f|0)<=0){i=l;return}j=b+11284|0;t=b+11288|0;k=b+11292|0;R=b+11296|0;Q=b+11300|0;P=b+11304|0;b=0;O=d;while(1){d=b<<1;N=(c[h+(b<<2)>>2]|0)-524288+(c[u+(b<<2)>>2]|0)>>3;M=(c[m+(b<<2)>>2]|0)-524288+(c[e+(b<<2)>>2]|0)>>3;L=c[j>>2]|0;K=(c[g+((d|1)<<2)>>2]>>2)-L|0;J=c[t>>2]|0;I=ac((c[g+(d<<2)>>2]>>2)-L|0,J)|0;L=I+8192|0;I=(ac(K,J)|0)+8192|0;J=ac(c[k>>2]|0,M)|0;K=ac(c[R>>2]|0,M)|0;M=(ac(c[Q>>2]|0,N)|0)+K|0;K=ac(c[P>>2]|0,N)|0;N=BP(41)|0;if((N|0)==0){D=8657;break}d=J+L|0;G=d>>>0>1073741823;H=d>>>14&65535;do{if((a[N+7|0]&1)==0){if(!G){T=H;break}T=-d>>31>>>14&65535}else{if(G){U=-d>>31>>>14&65535}else{U=H}T=cT(U|0)|0}}while(0);y=T;a[O]=y&255;y=y>>8;a[O+1|0]=y&255;H=BP(41)|0;if((H|0)==0){D=8665;break}d=M+L|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){V=-d>>31>>>14&65535}else{V=N}H=O+2|0;y=V;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){W=-d>>31>>>14&65535}else{W=N}N=O+2|0;y=cT(W|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(41)|0;if((N|0)==0){D=8674;break}d=K+L|0;G=d>>>0>1073741823;H=d>>>14&65535;if((a[N+7|0]&1)==0){if(G){X=-d>>31>>>14&65535}else{X=H}N=O+4|0;y=X;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){Y=-d>>31>>>14&65535}else{Y=H}H=O+4|0;y=cT(Y|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(41)|0;if((H|0)==0){D=8683;break}d=J+I|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){Z=-d>>31>>>14&65535}else{Z=N}H=O+6|0;y=Z;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){_=-d>>31>>>14&65535}else{_=N}N=O+6|0;y=cT(_|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(41)|0;if((N|0)==0){D=8692;break}d=M+I|0;G=d>>>0>1073741823;H=d>>>14&65535;if((a[N+7|0]&1)==0){if(G){$=-d>>31>>>14&65535}else{$=H}N=O+8|0;y=$;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){aa=-d>>31>>>14&65535}else{aa=H}H=O+8|0;y=cT(aa|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(41)|0;if((H|0)==0){D=8701;break}d=K+I|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){ab=-d>>31>>>14&65535}else{ab=N}H=O+10|0;y=ab;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){ad=-d>>31>>>14&65535}else{ad=N}N=O+10|0;y=cT(ad|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=b+1|0;if((N|0)<(f|0)){b=N;O=O+12|0}else{D=8711;break}}if((D|0)==8692){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8701){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8674){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8711){i=l;return}else if((D|0)==8665){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8683){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8657){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}}function uL(b,d,e,f,g,h,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;m=i;g=c[d>>2]|0;n=c[d+4>>2]|0;d=c[e>>2]|0;o=c[e+4>>2]|0;e=c[f>>2]|0;p=c[f+4>>2]|0;f=4096-k|0;q=4096-l|0;r=j+1>>1;if((r|0)<=0){i=m;return}j=b+11284|0;s=b+11288|0;t=b+11292|0;u=b+11296|0;v=b+11300|0;w=b+11304|0;b=0;x=h;while(1){h=b<<1;z=ac(c[g+(h<<2)>>2]|0,f)|0;A=(ac(c[n+(h<<2)>>2]|0,k)|0)+z>>14;z=h|1;h=ac(c[g+(z<<2)>>2]|0,f)|0;B=(ac(c[n+(z<<2)>>2]|0,k)|0)+h>>14;h=ac(c[d+(b<<2)>>2]|0,q)|0;z=h-1073741824+(ac(c[o+(b<<2)>>2]|0,l)|0)>>14;h=ac(c[e+(b<<2)>>2]|0,q)|0;C=h-1073741824+(ac(c[p+(b<<2)>>2]|0,l)|0)>>14;h=c[j>>2]|0;D=c[s>>2]|0;E=ac(A-h|0,D)|0;A=E+8192|0;E=(ac(B-h|0,D)|0)+8192|0;D=ac(C,c[t>>2]|0)|0;h=ac(c[u>>2]|0,C)|0;C=(ac(c[v>>2]|0,z)|0)+h|0;h=ac(c[w>>2]|0,z)|0;z=BP(41)|0;if((z|0)==0){F=8718;break}B=D+A|0;G=B>>>0>1073741823;H=B>>>14&65535;do{if((a[z+7|0]&1)==0){if(!G){I=H;break}I=-B>>31>>>14&65535}else{if(G){J=-B>>31>>>14&65535}else{J=H}I=cT(J|0)|0}}while(0);y=I;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;H=BP(41)|0;if((H|0)==0){F=8726;break}B=C+A|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){K=-B>>31>>>14&65535}else{K=z}H=x+2|0;y=K;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){L=-B>>31>>>14&65535}else{L=z}z=x+2|0;y=cT(L|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(41)|0;if((z|0)==0){F=8735;break}B=h+A|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){M=-B>>31>>>14&65535}else{M=H}z=x+4|0;y=M;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){N=-B>>31>>>14&65535}else{N=H}H=x+4|0;y=cT(N|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(41)|0;if((H|0)==0){F=8744;break}B=D+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){O=-B>>31>>>14&65535}else{O=z}H=x+6|0;y=O;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){P=-B>>31>>>14&65535}else{P=z}z=x+6|0;y=cT(P|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(41)|0;if((z|0)==0){F=8753;break}B=C+E|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){Q=-B>>31>>>14&65535}else{Q=H}z=x+8|0;y=Q;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){R=-B>>31>>>14&65535}else{R=H}H=x+8|0;y=cT(R|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(41)|0;if((H|0)==0){F=8762;break}B=h+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){S=-B>>31>>>14&65535}else{S=z}H=x+10|0;y=S;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){T=-B>>31>>>14&65535}else{T=z}z=x+10|0;y=cT(T|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=b+1|0;if((z|0)<(r|0)){b=z;x=x+12|0}else{F=8773;break}}if((F|0)==8735){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8753){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8726){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8744){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8718){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8773){i=m;return}else if((F|0)==8762){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}}function uM(d,e,f,g,h,j,k,l,m,n,o,p){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0;p=i;m=o+1>>1;if((m|0)<=0){i=p;return}o=(g|0)>0;q=(l|0)>0;r=d+11284|0;s=d+11288|0;t=d+11292|0;u=d+11296|0;v=d+11300|0;w=d+11304|0;d=0;x=n;while(1){if(o){n=d<<1;z=n|1;A=-1073741824;B=-1073741824;C=0;do{D=c[f+(C<<2)>>2]|0;E=b[e+(C<<1)>>1]|0;B=(ac(E,c[D+(n<<2)>>2]|0)|0)+B|0;A=(ac(c[D+(z<<2)>>2]|0,E)|0)+A|0;C=C+1|0;}while((C|0)<(g|0));F=(A>>14)+65536|0;G=(B>>14)+65536|0}else{F=0;G=0}if(q){C=-1073741824;z=-1073741824;n=0;do{E=b[h+(n<<1)>>1]|0;z=(ac(E,c[(c[j+(n<<2)>>2]|0)+(d<<2)>>2]|0)|0)+z|0;C=(ac(c[(c[k+(n<<2)>>2]|0)+(d<<2)>>2]|0,E)|0)+C|0;n=n+1|0;}while((n|0)<(l|0));H=C>>14;I=z>>14}else{H=-65536;I=-65536}n=c[r>>2]|0;B=c[s>>2]|0;A=ac(G-n|0,B)|0;E=A+8192|0;A=(ac(F-n|0,B)|0)+8192|0;B=ac(c[t>>2]|0,H)|0;n=ac(c[u>>2]|0,H)|0;D=(ac(c[v>>2]|0,I)|0)+n|0;n=ac(c[w>>2]|0,I)|0;J=BP(41)|0;if((J|0)==0){K=8784;break}L=B+E|0;M=L>>>0>1073741823;N=L>>>14&65535;do{if((a[J+7|0]&1)==0){if(!M){O=N;break}O=-L>>31>>>14&65535}else{if(M){P=-L>>31>>>14&65535}else{P=N}O=cT(P|0)|0}}while(0);y=O;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;N=BP(41)|0;if((N|0)==0){K=8792;break}L=D+E|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Q=-L>>31>>>14&65535}else{Q=J}N=x+2|0;y=Q;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){R=-L>>31>>>14&65535}else{R=J}J=x+2|0;y=cT(R|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(41)|0;if((J|0)==0){K=8801;break}L=n+E|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){S=-L>>31>>>14&65535}else{S=N}J=x+4|0;y=S;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){T=-L>>31>>>14&65535}else{T=N}N=x+4|0;y=cT(T|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(41)|0;if((N|0)==0){K=8810;break}L=B+A|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){U=-L>>31>>>14&65535}else{U=J}N=x+6|0;y=U;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){V=-L>>31>>>14&65535}else{V=J}J=x+6|0;y=cT(V|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(41)|0;if((J|0)==0){K=8819;break}L=D+A|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){W=-L>>31>>>14&65535}else{W=N}J=x+8|0;y=W;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){X=-L>>31>>>14&65535}else{X=N}N=x+8|0;y=cT(X|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(41)|0;if((N|0)==0){K=8828;break}L=n+A|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Y=-L>>31>>>14&65535}else{Y=J}N=x+10|0;y=Y;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){Z=-L>>31>>>14&65535}else{Z=J}J=x+10|0;y=cT(Z|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=d+1|0;if((J|0)<(m|0)){d=J;x=x+12|0}else{K=8839;break}}if((K|0)==8839){i=p;return}else if((K|0)==8792){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8784){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8801){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8810){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8819){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==8828){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}}function uN(b,d,e,f,g,h,j,k,l){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0;l=i;g=d;d=h;h=c[e>>2]|0;m=c[f>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){i=l;return}n=b+11284|0;o=b+11288|0;p=b+11292|0;q=b+11296|0;r=b+11300|0;s=b+11304|0;t=0;u=d;while(1){v=t<<1;w=(c[h+(t<<2)>>2]|0)-262144>>2;x=(c[m+(t<<2)>>2]|0)-262144>>2;z=c[n>>2]|0;A=(c[g+((v|1)<<2)>>2]>>2)-z|0;B=c[o>>2]|0;C=ac((c[g+(v<<2)>>2]>>2)-z|0,B)|0;z=C+8192|0;C=(ac(A,B)|0)+8192|0;B=ac(c[p>>2]|0,x)|0;A=ac(c[q>>2]|0,x)|0;x=(ac(c[r>>2]|0,w)|0)+A|0;A=ac(c[s>>2]|0,w)|0;w=BP(68)|0;if((w|0)==0){D=8844;break}v=A+z|0;E=v>>>0>1073741823;F=v>>>14&65535;do{if((a[w+7|0]&1)==0){if(!E){G=F;break}G=-v>>31>>>14&65535}else{if(E){H=-v>>31>>>14&65535}else{H=F}G=cT(H|0)|0}}while(0);y=G;a[u]=y&255;y=y>>8;a[u+1|0]=y&255;F=BP(68)|0;if((F|0)==0){D=8852;break}v=x+z|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){I=-v>>31>>>14&65535}else{I=w}F=u+2|0;y=I;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){J=-v>>31>>>14&65535}else{J=w}w=u+2|0;y=cT(J|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(68)|0;if((w|0)==0){D=8861;break}v=B+z|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){K=-v>>31>>>14&65535}else{K=F}w=u+4|0;y=K;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){L=-v>>31>>>14&65535}else{L=F}F=u+4|0;y=cT(L|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}F=BP(68)|0;if((F|0)==0){D=8870;break}v=A+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){M=-v>>31>>>14&65535}else{M=w}F=u+6|0;y=M;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){N=-v>>31>>>14&65535}else{N=w}w=u+6|0;y=cT(N|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(68)|0;if((w|0)==0){D=8879;break}v=x+C|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){O=-v>>31>>>14&65535}else{O=F}w=u+8|0;y=O;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){P=-v>>31>>>14&65535}else{P=F}F=u+8|0;y=cT(P|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}F=BP(68)|0;if((F|0)==0){D=8888;break}v=B+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){Q=-v>>31>>>14&65535}else{Q=w}F=u+10|0;y=Q;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){R=-v>>31>>>14&65535}else{R=w}w=u+10|0;y=cT(R|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=t+1|0;if((w|0)<(k|0)){t=w;u=u+12|0}else{D=8955;break}}if((D|0)==8955){i=l;return}else if((D|0)==8861){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8879){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8852){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8844){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8888){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8870){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}else{u=c[e+4>>2]|0;e=c[f+4>>2]|0;f=j+1>>1;if((f|0)<=0){i=l;return}j=b+11284|0;t=b+11288|0;k=b+11292|0;R=b+11296|0;Q=b+11300|0;P=b+11304|0;b=0;O=d;while(1){d=b<<1;N=(c[h+(b<<2)>>2]|0)-524288+(c[u+(b<<2)>>2]|0)>>3;M=(c[m+(b<<2)>>2]|0)-524288+(c[e+(b<<2)>>2]|0)>>3;L=c[j>>2]|0;K=(c[g+((d|1)<<2)>>2]>>2)-L|0;J=c[t>>2]|0;I=ac((c[g+(d<<2)>>2]>>2)-L|0,J)|0;L=I+8192|0;I=(ac(K,J)|0)+8192|0;J=ac(c[k>>2]|0,M)|0;K=ac(c[R>>2]|0,M)|0;M=(ac(c[Q>>2]|0,N)|0)+K|0;K=ac(c[P>>2]|0,N)|0;N=BP(68)|0;if((N|0)==0){D=8900;break}d=K+L|0;G=d>>>0>1073741823;H=d>>>14&65535;do{if((a[N+7|0]&1)==0){if(!G){T=H;break}T=-d>>31>>>14&65535}else{if(G){U=-d>>31>>>14&65535}else{U=H}T=cT(U|0)|0}}while(0);y=T;a[O]=y&255;y=y>>8;a[O+1|0]=y&255;H=BP(68)|0;if((H|0)==0){D=8908;break}d=M+L|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){V=-d>>31>>>14&65535}else{V=N}H=O+2|0;y=V;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){W=-d>>31>>>14&65535}else{W=N}N=O+2|0;y=cT(W|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(68)|0;if((N|0)==0){D=8917;break}d=J+L|0;G=d>>>0>1073741823;H=d>>>14&65535;if((a[N+7|0]&1)==0){if(G){X=-d>>31>>>14&65535}else{X=H}N=O+4|0;y=X;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){Y=-d>>31>>>14&65535}else{Y=H}H=O+4|0;y=cT(Y|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(68)|0;if((H|0)==0){D=8926;break}d=K+I|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){Z=-d>>31>>>14&65535}else{Z=N}H=O+6|0;y=Z;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){_=-d>>31>>>14&65535}else{_=N}N=O+6|0;y=cT(_|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(68)|0;if((N|0)==0){D=8935;break}d=M+I|0;G=d>>>0>1073741823;H=d>>>14&65535;if((a[N+7|0]&1)==0){if(G){$=-d>>31>>>14&65535}else{$=H}N=O+8|0;y=$;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){aa=-d>>31>>>14&65535}else{aa=H}H=O+8|0;y=cT(aa|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(68)|0;if((H|0)==0){D=8944;break}d=J+I|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){ab=-d>>31>>>14&65535}else{ab=N}H=O+10|0;y=ab;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){ad=-d>>31>>>14&65535}else{ad=N}N=O+10|0;y=cT(ad|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=b+1|0;if((N|0)<(f|0)){b=N;O=O+12|0}else{D=8954;break}}if((D|0)==8954){i=l;return}else if((D|0)==8944){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8900){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8908){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8917){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8935){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==8926){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}}function uO(b,d,e,f,g,h,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;m=i;g=c[d>>2]|0;n=c[d+4>>2]|0;d=c[e>>2]|0;o=c[e+4>>2]|0;e=c[f>>2]|0;p=c[f+4>>2]|0;f=4096-k|0;q=4096-l|0;r=j+1>>1;if((r|0)<=0){i=m;return}j=b+11284|0;s=b+11288|0;t=b+11292|0;u=b+11296|0;v=b+11300|0;w=b+11304|0;b=0;x=h;while(1){h=b<<1;z=ac(c[g+(h<<2)>>2]|0,f)|0;A=(ac(c[n+(h<<2)>>2]|0,k)|0)+z>>14;z=h|1;h=ac(c[g+(z<<2)>>2]|0,f)|0;B=(ac(c[n+(z<<2)>>2]|0,k)|0)+h>>14;h=ac(c[d+(b<<2)>>2]|0,q)|0;z=h-1073741824+(ac(c[o+(b<<2)>>2]|0,l)|0)>>14;h=ac(c[e+(b<<2)>>2]|0,q)|0;C=h-1073741824+(ac(c[p+(b<<2)>>2]|0,l)|0)>>14;h=c[j>>2]|0;D=c[s>>2]|0;E=ac(A-h|0,D)|0;A=E+8192|0;E=(ac(B-h|0,D)|0)+8192|0;D=ac(C,c[t>>2]|0)|0;h=ac(c[u>>2]|0,C)|0;C=(ac(c[v>>2]|0,z)|0)+h|0;h=ac(c[w>>2]|0,z)|0;z=BP(68)|0;if((z|0)==0){F=8961;break}B=h+A|0;G=B>>>0>1073741823;H=B>>>14&65535;do{if((a[z+7|0]&1)==0){if(!G){I=H;break}I=-B>>31>>>14&65535}else{if(G){J=-B>>31>>>14&65535}else{J=H}I=cT(J|0)|0}}while(0);y=I;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;H=BP(68)|0;if((H|0)==0){F=8969;break}B=C+A|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){K=-B>>31>>>14&65535}else{K=z}H=x+2|0;y=K;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){L=-B>>31>>>14&65535}else{L=z}z=x+2|0;y=cT(L|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(68)|0;if((z|0)==0){F=8978;break}B=D+A|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){M=-B>>31>>>14&65535}else{M=H}z=x+4|0;y=M;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){N=-B>>31>>>14&65535}else{N=H}H=x+4|0;y=cT(N|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(68)|0;if((H|0)==0){F=8987;break}B=h+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){O=-B>>31>>>14&65535}else{O=z}H=x+6|0;y=O;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){P=-B>>31>>>14&65535}else{P=z}z=x+6|0;y=cT(P|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(68)|0;if((z|0)==0){F=8996;break}B=C+E|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){Q=-B>>31>>>14&65535}else{Q=H}z=x+8|0;y=Q;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){R=-B>>31>>>14&65535}else{R=H}H=x+8|0;y=cT(R|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(68)|0;if((H|0)==0){F=9005;break}B=D+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){S=-B>>31>>>14&65535}else{S=z}H=x+10|0;y=S;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){T=-B>>31>>>14&65535}else{T=z}z=x+10|0;y=cT(T|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=b+1|0;if((z|0)<(r|0)){b=z;x=x+12|0}else{F=9016;break}}if((F|0)==8961){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==9016){i=m;return}else if((F|0)==8987){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8996){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==9005){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8978){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==8969){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}}function uP(d,e,f,g,h,j,k,l,m,n,o,p){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0;p=i;m=o+1>>1;if((m|0)<=0){i=p;return}o=(g|0)>0;q=(l|0)>0;r=d+11284|0;s=d+11288|0;t=d+11292|0;u=d+11296|0;v=d+11300|0;w=d+11304|0;d=0;x=n;while(1){if(o){n=d<<1;z=n|1;A=-1073741824;B=-1073741824;C=0;do{D=c[f+(C<<2)>>2]|0;E=b[e+(C<<1)>>1]|0;B=(ac(E,c[D+(n<<2)>>2]|0)|0)+B|0;A=(ac(c[D+(z<<2)>>2]|0,E)|0)+A|0;C=C+1|0;}while((C|0)<(g|0));F=(A>>14)+65536|0;G=(B>>14)+65536|0}else{F=0;G=0}if(q){C=-1073741824;z=-1073741824;n=0;do{E=b[h+(n<<1)>>1]|0;z=(ac(E,c[(c[j+(n<<2)>>2]|0)+(d<<2)>>2]|0)|0)+z|0;C=(ac(c[(c[k+(n<<2)>>2]|0)+(d<<2)>>2]|0,E)|0)+C|0;n=n+1|0;}while((n|0)<(l|0));H=C>>14;I=z>>14}else{H=-65536;I=-65536}n=c[r>>2]|0;B=c[s>>2]|0;A=ac(G-n|0,B)|0;E=A+8192|0;A=(ac(F-n|0,B)|0)+8192|0;B=ac(c[t>>2]|0,H)|0;n=ac(c[u>>2]|0,H)|0;D=(ac(c[v>>2]|0,I)|0)+n|0;n=ac(c[w>>2]|0,I)|0;J=BP(68)|0;if((J|0)==0){K=9027;break}L=n+E|0;M=L>>>0>1073741823;N=L>>>14&65535;do{if((a[J+7|0]&1)==0){if(!M){O=N;break}O=-L>>31>>>14&65535}else{if(M){P=-L>>31>>>14&65535}else{P=N}O=cT(P|0)|0}}while(0);y=O;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;N=BP(68)|0;if((N|0)==0){K=9035;break}L=D+E|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Q=-L>>31>>>14&65535}else{Q=J}N=x+2|0;y=Q;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){R=-L>>31>>>14&65535}else{R=J}J=x+2|0;y=cT(R|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(68)|0;if((J|0)==0){K=9044;break}L=B+E|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){S=-L>>31>>>14&65535}else{S=N}J=x+4|0;y=S;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){T=-L>>31>>>14&65535}else{T=N}N=x+4|0;y=cT(T|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(68)|0;if((N|0)==0){K=9053;break}L=n+A|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){U=-L>>31>>>14&65535}else{U=J}N=x+6|0;y=U;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){V=-L>>31>>>14&65535}else{V=J}J=x+6|0;y=cT(V|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(68)|0;if((J|0)==0){K=9062;break}L=D+A|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){W=-L>>31>>>14&65535}else{W=N}J=x+8|0;y=W;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){X=-L>>31>>>14&65535}else{X=N}N=x+8|0;y=cT(X|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(68)|0;if((N|0)==0){K=9071;break}L=B+A|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Y=-L>>31>>>14&65535}else{Y=J}N=x+10|0;y=Y;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){Z=-L>>31>>>14&65535}else{Z=J}J=x+10|0;y=cT(Z|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=d+1|0;if((J|0)<(m|0)){d=J;x=x+12|0}else{K=9082;break}}if((K|0)==9035){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==9027){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==9082){i=p;return}else if((K|0)==9044){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==9071){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==9053){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==9062){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}}function uQ(b,d,e,f,g,h,j,k,l){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0;l=i;g=d;d=h;h=c[e>>2]|0;m=c[f>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){i=l;return}n=b+11284|0;o=b+11288|0;p=b+11292|0;q=b+11296|0;r=b+11300|0;s=b+11304|0;t=0;u=d;while(1){v=t<<1;w=(c[h+(t<<2)>>2]|0)-262144>>2;x=(c[m+(t<<2)>>2]|0)-262144>>2;z=c[n>>2]|0;A=(c[g+((v|1)<<2)>>2]>>2)-z|0;B=c[o>>2]|0;C=ac((c[g+(v<<2)>>2]>>2)-z|0,B)|0;z=C+8192|0;C=(ac(A,B)|0)+8192|0;B=ac(c[p>>2]|0,x)|0;A=ac(c[q>>2]|0,x)|0;x=(ac(c[r>>2]|0,w)|0)+A|0;A=ac(c[s>>2]|0,w)|0;w=BP(67)|0;if((w|0)==0){D=9087;break}v=A+z|0;E=v>>>0>1073741823;F=v>>>14&65535;do{if((a[w+7|0]&1)==0){if(!E){G=F;break}G=-v>>31>>>14&65535}else{if(E){H=-v>>31>>>14&65535}else{H=F}G=cT(H|0)|0}}while(0);y=G;a[u]=y&255;y=y>>8;a[u+1|0]=y&255;F=BP(67)|0;if((F|0)==0){D=9095;break}v=x+z|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){I=-v>>31>>>14&65535}else{I=w}F=u+2|0;y=I;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){J=-v>>31>>>14&65535}else{J=w}w=u+2|0;y=cT(J|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(67)|0;if((w|0)==0){D=9104;break}v=B+z|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){K=-v>>31>>>14&65535}else{K=F}w=u+4|0;y=K;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){L=-v>>31>>>14&65535}else{L=F}F=u+4|0;y=cT(L|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}F=BP(67)|0;if((F|0)==0){D=9113;break}v=A+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){M=-v>>31>>>14&65535}else{M=w}F=u+6|0;y=M;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){N=-v>>31>>>14&65535}else{N=w}w=u+6|0;y=cT(N|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=BP(67)|0;if((w|0)==0){D=9122;break}v=x+C|0;E=v>>>0>1073741823;F=v>>>14&65535;if((a[w+7|0]&1)==0){if(E){O=-v>>31>>>14&65535}else{O=F}w=u+8|0;y=O;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}else{if(E){P=-v>>31>>>14&65535}else{P=F}F=u+8|0;y=cT(P|0)|0;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}F=BP(67)|0;if((F|0)==0){D=9131;break}v=B+C|0;E=v>>>0>1073741823;w=v>>>14&65535;if((a[F+7|0]&1)==0){if(E){Q=-v>>31>>>14&65535}else{Q=w}F=u+10|0;y=Q;a[F]=y&255;y=y>>8;a[F+1|0]=y&255}else{if(E){R=-v>>31>>>14&65535}else{R=w}w=u+10|0;y=cT(R|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255}w=t+1|0;if((w|0)<(k|0)){t=w;u=u+12|0}else{D=9197;break}}if((D|0)==9122){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==9087){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==9197){i=l;return}else if((D|0)==9104){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==9095){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==9131){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==9113){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}else{u=c[e+4>>2]|0;e=c[f+4>>2]|0;f=j+1>>1;if((f|0)<=0){i=l;return}j=b+11284|0;t=b+11288|0;k=b+11292|0;R=b+11296|0;Q=b+11300|0;P=b+11304|0;b=0;O=d;while(1){d=b<<1;N=(c[h+(b<<2)>>2]|0)-524288+(c[u+(b<<2)>>2]|0)>>3;M=(c[m+(b<<2)>>2]|0)-524288+(c[e+(b<<2)>>2]|0)>>3;L=c[j>>2]|0;K=(c[g+((d|1)<<2)>>2]>>2)-L|0;J=c[t>>2]|0;I=ac((c[g+(d<<2)>>2]>>2)-L|0,J)|0;L=I+8192|0;I=(ac(K,J)|0)+8192|0;J=ac(c[k>>2]|0,M)|0;K=ac(c[R>>2]|0,M)|0;M=(ac(c[Q>>2]|0,N)|0)+K|0;K=ac(c[P>>2]|0,N)|0;N=BP(67)|0;if((N|0)==0){D=9143;break}d=K+L|0;G=d>>>0>1073741823;H=d>>>14&65535;do{if((a[N+7|0]&1)==0){if(!G){T=H;break}T=-d>>31>>>14&65535}else{if(G){U=-d>>31>>>14&65535}else{U=H}T=cT(U|0)|0}}while(0);y=T;a[O]=y&255;y=y>>8;a[O+1|0]=y&255;H=BP(67)|0;if((H|0)==0){D=9151;break}d=M+L|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){V=-d>>31>>>14&65535}else{V=N}H=O+2|0;y=V;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){W=-d>>31>>>14&65535}else{W=N}N=O+2|0;y=cT(W|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(67)|0;if((N|0)==0){D=9160;break}d=J+L|0;G=d>>>0>1073741823;H=d>>>14&65535;if((a[N+7|0]&1)==0){if(G){X=-d>>31>>>14&65535}else{X=H}N=O+4|0;y=X;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){Y=-d>>31>>>14&65535}else{Y=H}H=O+4|0;y=cT(Y|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(67)|0;if((H|0)==0){D=9169;break}d=K+I|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){Z=-d>>31>>>14&65535}else{Z=N}H=O+6|0;y=Z;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){_=-d>>31>>>14&65535}else{_=N}N=O+6|0;y=cT(_|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(67)|0;if((N|0)==0){D=9178;break}d=M+I|0;G=d>>>0>1073741823;H=d>>>14&65535;if((a[N+7|0]&1)==0){if(G){$=-d>>31>>>14&65535}else{$=H}N=O+8|0;y=$;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(G){aa=-d>>31>>>14&65535}else{aa=H}H=O+8|0;y=cT(aa|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(67)|0;if((H|0)==0){D=9187;break}d=J+I|0;G=d>>>0>1073741823;N=d>>>14&65535;if((a[H+7|0]&1)==0){if(G){ab=-d>>31>>>14&65535}else{ab=N}H=O+10|0;y=ab;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){ad=-d>>31>>>14&65535}else{ad=N}N=O+10|0;y=cT(ad|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=b+1|0;if((N|0)<(f|0)){b=N;O=O+12|0}else{D=9199;break}}if((D|0)==9143){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==9199){i=l;return}else if((D|0)==9169){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==9178){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==9160){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==9187){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}else if((D|0)==9151){AN(0,0,1639912,(S=i,i=i+24|0,c[S>>2]=1671168,c[S+8>>2]=1656384,c[S+16>>2]=635,S)|0);i=S;cw()}}}function uR(b,d,e,f,g,h,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;m=i;g=c[d>>2]|0;n=c[d+4>>2]|0;d=c[e>>2]|0;o=c[e+4>>2]|0;e=c[f>>2]|0;p=c[f+4>>2]|0;f=4096-k|0;q=4096-l|0;r=j+1>>1;if((r|0)<=0){i=m;return}j=b+11284|0;s=b+11288|0;t=b+11292|0;u=b+11296|0;v=b+11300|0;w=b+11304|0;b=0;x=h;while(1){h=b<<1;z=ac(c[g+(h<<2)>>2]|0,f)|0;A=(ac(c[n+(h<<2)>>2]|0,k)|0)+z>>14;z=h|1;h=ac(c[g+(z<<2)>>2]|0,f)|0;B=(ac(c[n+(z<<2)>>2]|0,k)|0)+h>>14;h=ac(c[d+(b<<2)>>2]|0,q)|0;z=h-1073741824+(ac(c[o+(b<<2)>>2]|0,l)|0)>>14;h=ac(c[e+(b<<2)>>2]|0,q)|0;C=h-1073741824+(ac(c[p+(b<<2)>>2]|0,l)|0)>>14;h=c[j>>2]|0;D=c[s>>2]|0;E=ac(A-h|0,D)|0;A=E+8192|0;E=(ac(B-h|0,D)|0)+8192|0;D=ac(C,c[t>>2]|0)|0;h=ac(c[u>>2]|0,C)|0;C=(ac(c[v>>2]|0,z)|0)+h|0;h=ac(c[w>>2]|0,z)|0;z=BP(67)|0;if((z|0)==0){F=9204;break}B=h+A|0;G=B>>>0>1073741823;H=B>>>14&65535;do{if((a[z+7|0]&1)==0){if(!G){I=H;break}I=-B>>31>>>14&65535}else{if(G){J=-B>>31>>>14&65535}else{J=H}I=cT(J|0)|0}}while(0);y=I;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;H=BP(67)|0;if((H|0)==0){F=9212;break}B=C+A|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){K=-B>>31>>>14&65535}else{K=z}H=x+2|0;y=K;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){L=-B>>31>>>14&65535}else{L=z}z=x+2|0;y=cT(L|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(67)|0;if((z|0)==0){F=9221;break}B=D+A|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){M=-B>>31>>>14&65535}else{M=H}z=x+4|0;y=M;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){N=-B>>31>>>14&65535}else{N=H}H=x+4|0;y=cT(N|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(67)|0;if((H|0)==0){F=9230;break}B=h+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){O=-B>>31>>>14&65535}else{O=z}H=x+6|0;y=O;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){P=-B>>31>>>14&65535}else{P=z}z=x+6|0;y=cT(P|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=BP(67)|0;if((z|0)==0){F=9239;break}B=C+E|0;G=B>>>0>1073741823;H=B>>>14&65535;if((a[z+7|0]&1)==0){if(G){Q=-B>>31>>>14&65535}else{Q=H}z=x+8|0;y=Q;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}else{if(G){R=-B>>31>>>14&65535}else{R=H}H=x+8|0;y=cT(R|0)|0;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}H=BP(67)|0;if((H|0)==0){F=9248;break}B=D+E|0;G=B>>>0>1073741823;z=B>>>14&65535;if((a[H+7|0]&1)==0){if(G){S=-B>>31>>>14&65535}else{S=z}H=x+10|0;y=S;a[H]=y&255;y=y>>8;a[H+1|0]=y&255}else{if(G){T=-B>>31>>>14&65535}else{T=z}z=x+10|0;y=cT(T|0)|0;a[z]=y&255;y=y>>8;a[z+1|0]=y&255}z=b+1|0;if((z|0)<(r|0)){b=z;x=x+12|0}else{F=9259;break}}if((F|0)==9221){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==9259){i=m;return}else if((F|0)==9212){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==9230){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==9239){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==9248){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}else if((F|0)==9204){AN(0,0,1639912,(U=i,i=i+24|0,c[U>>2]=1671168,c[U+8>>2]=1656384,c[U+16>>2]=635,U)|0);i=U;cw()}}function uS(d,e,f,g,h,j,k,l,m,n,o,p){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0;p=i;m=o+1>>1;if((m|0)<=0){i=p;return}o=(g|0)>0;q=(l|0)>0;r=d+11284|0;s=d+11288|0;t=d+11292|0;u=d+11296|0;v=d+11300|0;w=d+11304|0;d=0;x=n;while(1){if(o){n=d<<1;z=n|1;A=-1073741824;B=-1073741824;C=0;do{D=c[f+(C<<2)>>2]|0;E=b[e+(C<<1)>>1]|0;B=(ac(E,c[D+(n<<2)>>2]|0)|0)+B|0;A=(ac(c[D+(z<<2)>>2]|0,E)|0)+A|0;C=C+1|0;}while((C|0)<(g|0));F=(A>>14)+65536|0;G=(B>>14)+65536|0}else{F=0;G=0}if(q){C=-1073741824;z=-1073741824;n=0;do{E=b[h+(n<<1)>>1]|0;z=(ac(E,c[(c[j+(n<<2)>>2]|0)+(d<<2)>>2]|0)|0)+z|0;C=(ac(c[(c[k+(n<<2)>>2]|0)+(d<<2)>>2]|0,E)|0)+C|0;n=n+1|0;}while((n|0)<(l|0));H=C>>14;I=z>>14}else{H=-65536;I=-65536}n=c[r>>2]|0;B=c[s>>2]|0;A=ac(G-n|0,B)|0;E=A+8192|0;A=(ac(F-n|0,B)|0)+8192|0;B=ac(c[t>>2]|0,H)|0;n=ac(c[u>>2]|0,H)|0;D=(ac(c[v>>2]|0,I)|0)+n|0;n=ac(c[w>>2]|0,I)|0;J=BP(67)|0;if((J|0)==0){K=9270;break}L=n+E|0;M=L>>>0>1073741823;N=L>>>14&65535;do{if((a[J+7|0]&1)==0){if(!M){O=N;break}O=-L>>31>>>14&65535}else{if(M){P=-L>>31>>>14&65535}else{P=N}O=cT(P|0)|0}}while(0);y=O;a[x]=y&255;y=y>>8;a[x+1|0]=y&255;N=BP(67)|0;if((N|0)==0){K=9278;break}L=D+E|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Q=-L>>31>>>14&65535}else{Q=J}N=x+2|0;y=Q;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){R=-L>>31>>>14&65535}else{R=J}J=x+2|0;y=cT(R|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(67)|0;if((J|0)==0){K=9287;break}L=B+E|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){S=-L>>31>>>14&65535}else{S=N}J=x+4|0;y=S;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){T=-L>>31>>>14&65535}else{T=N}N=x+4|0;y=cT(T|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(67)|0;if((N|0)==0){K=9296;break}L=n+A|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){U=-L>>31>>>14&65535}else{U=J}N=x+6|0;y=U;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){V=-L>>31>>>14&65535}else{V=J}J=x+6|0;y=cT(V|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=BP(67)|0;if((J|0)==0){K=9305;break}L=D+A|0;M=L>>>0>1073741823;N=L>>>14&65535;if((a[J+7|0]&1)==0){if(M){W=-L>>31>>>14&65535}else{W=N}J=x+8|0;y=W;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}else{if(M){X=-L>>31>>>14&65535}else{X=N}N=x+8|0;y=cT(X|0)|0;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}N=BP(67)|0;if((N|0)==0){K=9314;break}L=B+A|0;M=L>>>0>1073741823;J=L>>>14&65535;if((a[N+7|0]&1)==0){if(M){Y=-L>>31>>>14&65535}else{Y=J}N=x+10|0;y=Y;a[N]=y&255;y=y>>8;a[N+1|0]=y&255}else{if(M){Z=-L>>31>>>14&65535}else{Z=J}J=x+10|0;y=cT(Z|0)|0;a[J]=y&255;y=y>>8;a[J+1|0]=y&255}J=d+1|0;if((J|0)<(m|0)){d=J;x=x+12|0}else{K=9324;break}}if((K|0)==9270){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==9324){i=p;return}else if((K|0)==9278){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==9296){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==9305){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==9287){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}else if((K|0)==9314){AN(0,0,1639912,(_=i,i=i+24|0,c[_>>2]=1671168,c[_+8>>2]=1656384,c[_+16>>2]=635,_)|0);i=_;cw()}}function uT(a,d,e,f,g,h,i,j,k){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;k=c[e>>2]|0;l=c[f>>2]|0;if((j|0)<2048){j=i+1>>1;if((j|0)<=0){return}m=h;n=0;do{o=n<<1;p=(b[d+(o<<1)>>1]|0)+64>>7;q=o|1;r=(b[d+(q<<1)>>1]|0)+64>>7;s=((b[l+(n<<1)>>1]|0)+64>>7)+128|0;t=((b[k+(n<<1)>>1]|0)+64>>7)+128|0;u=((b[g+(o<<1)>>1]|0)*255|0)+16384>>15;v=((b[g+(q<<1)>>1]|0)*255|0)+16384>>15;if(u>>>0>255){w=-u>>31&255}else{w=u&255}if(v>>>0>255){x=-v>>31&255}else{x=v&255}v=c[a+2292+(s<<2)>>2]|0;u=(c[a+4340+(t<<2)>>2]|0)+(c[a+6388+(s<<2)>>2]|0)|0;s=c[a+8436+(t<<2)>>2]|0;c[m+(o<<2)>>2]=(c[v+(p<<2)>>2]|0)+((w&255)<<24)+(c[u+(p<<2)>>2]|0)+(c[s+(p<<2)>>2]|0);c[m+(q<<2)>>2]=(c[v+(r<<2)>>2]|0)+((x&255)<<24)+(c[u+(r<<2)>>2]|0)+(c[s+(r<<2)>>2]|0);n=n+1|0;}while((n|0)<(j|0));return}else{j=c[e+4>>2]|0;e=c[f+4>>2]|0;f=i+1>>1;if((f|0)<=0){return}i=h;h=0;do{n=h<<1;x=(b[d+(n<<1)>>1]|0)+64>>7;m=n|1;w=(b[d+(m<<1)>>1]|0)+64>>7;r=((b[l+(h<<1)>>1]|0)+128+(b[e+(h<<1)>>1]|0)>>8)+128|0;s=((b[k+(h<<1)>>1]|0)+128+(b[j+(h<<1)>>1]|0)>>8)+128|0;u=(b[g+(n<<1)>>1]|0)+64>>7;v=(b[g+(m<<1)>>1]|0)+64>>7;if(u>>>0>255){y=-u>>31&255}else{y=u&255}if(v>>>0>255){z=-v>>31&255}else{z=v&255}v=c[a+2292+(r<<2)>>2]|0;u=(c[a+4340+(s<<2)>>2]|0)+(c[a+6388+(r<<2)>>2]|0)|0;r=c[a+8436+(s<<2)>>2]|0;c[i+(n<<2)>>2]=(c[v+(x<<2)>>2]|0)+((y&255)<<24)+(c[u+(x<<2)>>2]|0)+(c[r+(x<<2)>>2]|0);c[i+(m<<2)>>2]=(c[v+(w<<2)>>2]|0)+((z&255)<<24)+(c[u+(w<<2)>>2]|0)+(c[r+(w<<2)>>2]|0);h=h+1|0;}while((h|0)<(f|0));return}}function uU(a,d,e,f,g,h,i,j,k,l){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;l=c[d>>2]|0;m=c[d+4>>2]|0;d=c[e>>2]|0;n=c[e+4>>2]|0;e=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=4096-j|0;q=4096-k|0;r=i+1>>1;if((r|0)<=0){return}i=h;h=0;do{s=h<<1;t=ac(b[l+(s<<1)>>1]|0,g)|0;u=(ac(b[m+(s<<1)>>1]|0,j)|0)+t>>19;t=s|1;v=ac(b[l+(t<<1)>>1]|0,g)|0;w=(ac(b[m+(t<<1)>>1]|0,j)|0)+v>>19;v=ac(b[d+(h<<1)>>1]|0,q)|0;x=(ac(b[n+(h<<1)>>1]|0,k)|0)+v>>19;v=ac(b[e+(h<<1)>>1]|0,q)|0;y=((ac(b[o+(h<<1)>>1]|0,k)|0)+v>>19)+128|0;v=c[a+2292+(y<<2)>>2]|0;z=x+128|0;x=(c[a+4340+(z<<2)>>2]|0)+(c[a+6388+(y<<2)>>2]|0)|0;y=c[a+8436+(z<<2)>>2]|0;z=ac(b[f+(s<<1)>>1]|0,g)|0;A=(ac(b[p+(s<<1)>>1]|0,j)|0)+z>>19;z=ac(b[f+(t<<1)>>1]|0,g)|0;B=(ac(b[p+(t<<1)>>1]|0,j)|0)+z>>19;if(A>>>0>255){C=-A>>31&255}else{C=A&255}if(B>>>0>255){D=-B>>31&255}else{D=B&255}B=v;v=x;x=y;c[i+(s<<2)>>2]=(c[B+(u<<2)>>2]|0)+((C&255)<<24)+(c[v+(u<<2)>>2]|0)+(c[x+(u<<2)>>2]|0);c[i+(t<<2)>>2]=(c[B+(w<<2)>>2]|0)+((D&255)<<24)+(c[v+(w<<2)>>2]|0)+(c[x+(w<<2)>>2]|0);h=h+1|0;}while((h|0)<(r|0));return}function uV(a,d,e,f,g,h,i,j,k,l,m,n){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0;n=m+1>>1;if((n|0)<=0){return}m=(f|0)>0;o=(j|0)>0;p=l;l=0;do{if(m){q=l<<1;r=q|1;s=0;t=262144;u=262144;do{v=c[e+(s<<2)>>2]|0;w=b[d+(s<<1)>>1]|0;t=(ac(w,b[v+(q<<1)>>1]|0)|0)+t|0;u=(ac(b[v+(r<<1)>>1]|0,w)|0)+u|0;s=s+1|0;}while((s|0)<(f|0));x=t>>19;y=u>>19}else{x=0;y=0}if(o){s=0;r=262144;q=262144;do{w=b[g+(s<<1)>>1]|0;r=(ac(w,b[(c[h+(s<<2)>>2]|0)+(l<<1)>>1]|0)|0)+r|0;q=(ac(b[(c[i+(s<<2)>>2]|0)+(l<<1)>>1]|0,w)|0)+q|0;s=s+1|0;}while((s|0)<(j|0));z=(r>>19)+128|0;A=(q>>19)+128|0}else{z=128;A=128}if(m){s=l<<1;u=s|1;t=0;w=262144;v=262144;do{B=c[k+(t<<2)>>2]|0;C=b[d+(t<<1)>>1]|0;w=(ac(C,b[B+(s<<1)>>1]|0)|0)+w|0;v=(ac(b[B+(u<<1)>>1]|0,C)|0)+v|0;t=t+1|0;}while((t|0)<(f|0));D=w>>19;E=v>>19}else{D=0;E=0}if(((E|D)&256|0)==0){F=E;G=D}else{if(D>>>0>255){H=-D>>31&255}else{H=D&255}if(E>>>0>255){I=-E>>31&255}else{I=E&255}F=I&255;G=H&255}t=c[a+2292+(A<<2)>>2]|0;u=(c[a+4340+(z<<2)>>2]|0)+(c[a+6388+(A<<2)>>2]|0)|0;s=c[a+8436+(z<<2)>>2]|0;q=l<<1;c[p+(q<<2)>>2]=(c[t+(x<<2)>>2]|0)+(G<<24)+(c[u+(x<<2)>>2]|0)+(c[s+(x<<2)>>2]|0);c[p+((q|1)<<2)>>2]=(c[t+(y<<2)>>2]|0)+(F<<24)+(c[u+(y<<2)>>2]|0)+(c[s+(y<<2)>>2]|0);l=l+1|0;}while((l|0)<(n|0));return}function uW(a,d,e,f,g,h,i,j,k){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;k=c[e>>2]|0;g=c[f>>2]|0;if((j|0)<2048){j=i+1>>1;if((j|0)<=0){return}l=h;m=0;do{n=m<<1;o=(b[d+(n<<1)>>1]|0)+64>>7;p=n|1;q=(b[d+(p<<1)>>1]|0)+64>>7;r=((b[g+(m<<1)>>1]|0)+64>>7)+128|0;s=((b[k+(m<<1)>>1]|0)+64>>7)+128|0;t=c[a+2292+(r<<2)>>2]|0;u=(c[a+4340+(s<<2)>>2]|0)+(c[a+6388+(r<<2)>>2]|0)|0;r=c[a+8436+(s<<2)>>2]|0;c[l+(n<<2)>>2]=(c[u+(o<<2)>>2]|0)+(c[t+(o<<2)>>2]|0)+(c[r+(o<<2)>>2]|0);c[l+(p<<2)>>2]=(c[u+(q<<2)>>2]|0)+(c[t+(q<<2)>>2]|0)+(c[r+(q<<2)>>2]|0);m=m+1|0;}while((m|0)<(j|0));return}else{j=c[e+4>>2]|0;e=c[f+4>>2]|0;f=i+1>>1;if((f|0)<=0){return}i=h;h=0;do{m=h<<1;l=(b[d+(m<<1)>>1]|0)+64>>7;q=m|1;r=(b[d+(q<<1)>>1]|0)+64>>7;t=((b[g+(h<<1)>>1]|0)+128+(b[e+(h<<1)>>1]|0)>>8)+128|0;u=((b[k+(h<<1)>>1]|0)+128+(b[j+(h<<1)>>1]|0)>>8)+128|0;p=c[a+2292+(t<<2)>>2]|0;o=(c[a+4340+(u<<2)>>2]|0)+(c[a+6388+(t<<2)>>2]|0)|0;t=c[a+8436+(u<<2)>>2]|0;c[i+(m<<2)>>2]=(c[o+(l<<2)>>2]|0)+(c[p+(l<<2)>>2]|0)+(c[t+(l<<2)>>2]|0);c[i+(q<<2)>>2]=(c[o+(r<<2)>>2]|0)+(c[p+(r<<2)>>2]|0)+(c[t+(r<<2)>>2]|0);h=h+1|0;}while((h|0)<(f|0));return}}function uX(a,d,e,f,g,h,i,j,k,l){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;l=c[d>>2]|0;g=c[d+4>>2]|0;d=c[e>>2]|0;m=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=4096-j|0;o=4096-k|0;p=i+1>>1;if((p|0)<=0){return}i=h;h=0;do{q=h<<1;r=ac(b[l+(q<<1)>>1]|0,f)|0;s=(ac(b[g+(q<<1)>>1]|0,j)|0)+r>>19;r=q|1;t=ac(b[l+(r<<1)>>1]|0,f)|0;u=(ac(b[g+(r<<1)>>1]|0,j)|0)+t>>19;t=ac(b[d+(h<<1)>>1]|0,o)|0;v=(ac(b[m+(h<<1)>>1]|0,k)|0)+t>>19;t=ac(b[e+(h<<1)>>1]|0,o)|0;w=((ac(b[n+(h<<1)>>1]|0,k)|0)+t>>19)+128|0;t=v+128|0;v=c[a+2292+(w<<2)>>2]|0;x=(c[a+4340+(t<<2)>>2]|0)+(c[a+6388+(w<<2)>>2]|0)|0;w=c[a+8436+(t<<2)>>2]|0;c[i+(q<<2)>>2]=(c[x+(s<<2)>>2]|0)+(c[v+(s<<2)>>2]|0)+(c[w+(s<<2)>>2]|0);c[i+(r<<2)>>2]=(c[x+(u<<2)>>2]|0)+(c[v+(u<<2)>>2]|0)+(c[w+(u<<2)>>2]|0);h=h+1|0;}while((h|0)<(p|0));return}function uY(a,d,e,f,g,h,i,j,k,l,m,n){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;n=m+1>>1;if((n|0)<=0){return}m=(f|0)>0;k=(j|0)>0;o=l;l=0;do{if(m){p=l<<1;q=p|1;r=0;s=262144;t=262144;do{u=c[e+(r<<2)>>2]|0;v=b[d+(r<<1)>>1]|0;s=(ac(v,b[u+(p<<1)>>1]|0)|0)+s|0;t=(ac(b[u+(q<<1)>>1]|0,v)|0)+t|0;r=r+1|0;}while((r|0)<(f|0));w=s>>19;x=t>>19}else{w=0;x=0}if(k){r=0;q=262144;p=262144;do{v=b[g+(r<<1)>>1]|0;q=(ac(v,b[(c[h+(r<<2)>>2]|0)+(l<<1)>>1]|0)|0)+q|0;p=(ac(b[(c[i+(r<<2)>>2]|0)+(l<<1)>>1]|0,v)|0)+p|0;r=r+1|0;}while((r|0)<(j|0));y=(q>>19)+128|0;z=(p>>19)+128|0}else{y=128;z=128}r=c[a+2292+(z<<2)>>2]|0;t=(c[a+4340+(y<<2)>>2]|0)+(c[a+6388+(z<<2)>>2]|0)|0;s=c[a+8436+(y<<2)>>2]|0;v=l<<1;c[o+(v<<2)>>2]=(c[t+(w<<2)>>2]|0)+(c[r+(w<<2)>>2]|0)+(c[s+(w<<2)>>2]|0);c[o+((v|1)<<2)>>2]=(c[t+(x<<2)>>2]|0)+(c[r+(x<<2)>>2]|0)+(c[s+(x<<2)>>2]|0);l=l+1|0;}while((l|0)<(n|0));return}function uZ(a,d,e,f,g,h,i,j,k){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;k=c[e>>2]|0;l=c[f>>2]|0;if((j|0)<2048){j=i+1>>1;if((j|0)<=0){return}m=h;n=0;do{o=n<<1;p=(b[d+(o<<1)>>1]|0)+64>>7;q=o|1;r=(b[d+(q<<1)>>1]|0)+64>>7;s=((b[l+(n<<1)>>1]|0)+64>>7)+128|0;t=((b[k+(n<<1)>>1]|0)+64>>7)+128|0;u=((b[g+(o<<1)>>1]|0)*255|0)+16384>>15;v=((b[g+(q<<1)>>1]|0)*255|0)+16384>>15;if(u>>>0>255){w=-u>>31&255}else{w=u&255}if(v>>>0>255){x=-v>>31&255}else{x=v&255}v=c[a+2292+(s<<2)>>2]|0;u=(c[a+4340+(t<<2)>>2]|0)+(c[a+6388+(s<<2)>>2]|0)|0;s=c[a+8436+(t<<2)>>2]|0;c[m+(o<<2)>>2]=(c[v+(p<<2)>>2]|0)+(w&255)+(c[u+(p<<2)>>2]|0)+(c[s+(p<<2)>>2]|0);c[m+(q<<2)>>2]=(c[v+(r<<2)>>2]|0)+(x&255)+(c[u+(r<<2)>>2]|0)+(c[s+(r<<2)>>2]|0);n=n+1|0;}while((n|0)<(j|0));return}else{j=c[e+4>>2]|0;e=c[f+4>>2]|0;f=i+1>>1;if((f|0)<=0){return}i=h;h=0;do{n=h<<1;x=(b[d+(n<<1)>>1]|0)+64>>7;m=n|1;w=(b[d+(m<<1)>>1]|0)+64>>7;r=((b[l+(h<<1)>>1]|0)+128+(b[e+(h<<1)>>1]|0)>>8)+128|0;s=((b[k+(h<<1)>>1]|0)+128+(b[j+(h<<1)>>1]|0)>>8)+128|0;u=(b[g+(n<<1)>>1]|0)+64>>7;v=(b[g+(m<<1)>>1]|0)+64>>7;if(u>>>0>255){y=-u>>31&255}else{y=u&255}if(v>>>0>255){z=-v>>31&255}else{z=v&255}v=c[a+2292+(r<<2)>>2]|0;u=(c[a+4340+(s<<2)>>2]|0)+(c[a+6388+(r<<2)>>2]|0)|0;r=c[a+8436+(s<<2)>>2]|0;c[i+(n<<2)>>2]=(c[v+(x<<2)>>2]|0)+(y&255)+(c[u+(x<<2)>>2]|0)+(c[r+(x<<2)>>2]|0);c[i+(m<<2)>>2]=(c[v+(w<<2)>>2]|0)+(z&255)+(c[u+(w<<2)>>2]|0)+(c[r+(w<<2)>>2]|0);h=h+1|0;}while((h|0)<(f|0));return}}function u_(a,d,e,f,g,h,i,j,k,l){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;l=c[d>>2]|0;m=c[d+4>>2]|0;d=c[e>>2]|0;n=c[e+4>>2]|0;e=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=4096-j|0;q=4096-k|0;r=i+1>>1;if((r|0)<=0){return}i=h;h=0;do{s=h<<1;t=ac(b[l+(s<<1)>>1]|0,g)|0;u=(ac(b[m+(s<<1)>>1]|0,j)|0)+t>>19;t=s|1;v=ac(b[l+(t<<1)>>1]|0,g)|0;w=(ac(b[m+(t<<1)>>1]|0,j)|0)+v>>19;v=ac(b[d+(h<<1)>>1]|0,q)|0;x=(ac(b[n+(h<<1)>>1]|0,k)|0)+v>>19;v=ac(b[e+(h<<1)>>1]|0,q)|0;y=((ac(b[o+(h<<1)>>1]|0,k)|0)+v>>19)+128|0;v=c[a+2292+(y<<2)>>2]|0;z=x+128|0;x=(c[a+4340+(z<<2)>>2]|0)+(c[a+6388+(y<<2)>>2]|0)|0;y=c[a+8436+(z<<2)>>2]|0;z=ac(b[f+(s<<1)>>1]|0,g)|0;A=(ac(b[p+(s<<1)>>1]|0,j)|0)+z>>19;z=ac(b[f+(t<<1)>>1]|0,g)|0;B=(ac(b[p+(t<<1)>>1]|0,j)|0)+z>>19;if(A>>>0>255){C=-A>>31&255}else{C=A&255}if(B>>>0>255){D=-B>>31&255}else{D=B&255}B=v;v=x;x=y;c[i+(s<<2)>>2]=(c[B+(u<<2)>>2]|0)+(C&255)+(c[v+(u<<2)>>2]|0)+(c[x+(u<<2)>>2]|0);c[i+(t<<2)>>2]=(c[B+(w<<2)>>2]|0)+(D&255)+(c[v+(w<<2)>>2]|0)+(c[x+(w<<2)>>2]|0);h=h+1|0;}while((h|0)<(r|0));return}function u$(a,d,e,f,g,h,i,j,k,l,m,n){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0;n=m+1>>1;if((n|0)<=0){return}m=(f|0)>0;o=(j|0)>0;p=l;l=0;do{if(m){q=l<<1;r=q|1;s=0;t=262144;u=262144;do{v=c[e+(s<<2)>>2]|0;w=b[d+(s<<1)>>1]|0;t=(ac(w,b[v+(q<<1)>>1]|0)|0)+t|0;u=(ac(b[v+(r<<1)>>1]|0,w)|0)+u|0;s=s+1|0;}while((s|0)<(f|0));x=t>>19;y=u>>19}else{x=0;y=0}if(o){s=0;r=262144;q=262144;do{w=b[g+(s<<1)>>1]|0;r=(ac(w,b[(c[h+(s<<2)>>2]|0)+(l<<1)>>1]|0)|0)+r|0;q=(ac(b[(c[i+(s<<2)>>2]|0)+(l<<1)>>1]|0,w)|0)+q|0;s=s+1|0;}while((s|0)<(j|0));z=(r>>19)+128|0;A=(q>>19)+128|0}else{z=128;A=128}if(m){s=l<<1;u=s|1;t=0;w=262144;v=262144;do{B=c[k+(t<<2)>>2]|0;C=b[d+(t<<1)>>1]|0;w=(ac(C,b[B+(s<<1)>>1]|0)|0)+w|0;v=(ac(b[B+(u<<1)>>1]|0,C)|0)+v|0;t=t+1|0;}while((t|0)<(f|0));D=w>>19;E=v>>19}else{D=0;E=0}if(((E|D)&256|0)==0){F=E;G=D}else{if(D>>>0>255){H=-D>>31&255}else{H=D&255}if(E>>>0>255){I=-E>>31&255}else{I=E&255}F=I&255;G=H&255}t=c[a+2292+(A<<2)>>2]|0;u=(c[a+4340+(z<<2)>>2]|0)+(c[a+6388+(A<<2)>>2]|0)|0;s=c[a+8436+(z<<2)>>2]|0;q=l<<1;c[p+(q<<2)>>2]=(c[t+(x<<2)>>2]|0)+G+(c[u+(x<<2)>>2]|0)+(c[s+(x<<2)>>2]|0);c[p+((q|1)<<2)>>2]=(c[t+(y<<2)>>2]|0)+F+(c[u+(y<<2)>>2]|0)+(c[s+(y<<2)>>2]|0);l=l+1|0;}while((l|0)<(n|0));return}function u0(a,d,e,f,g,h,i,j,k){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;k=c[e>>2]|0;g=c[f>>2]|0;if((j|0)<2048){j=i+1>>1;if((j|0)<=0){return}l=h;m=0;do{n=m<<1;o=(b[d+(n<<1)>>1]|0)+64>>7;p=n|1;q=(b[d+(p<<1)>>1]|0)+64>>7;r=((b[g+(m<<1)>>1]|0)+64>>7)+128|0;s=((b[k+(m<<1)>>1]|0)+64>>7)+128|0;t=c[a+2292+(r<<2)>>2]|0;u=(c[a+4340+(s<<2)>>2]|0)+(c[a+6388+(r<<2)>>2]|0)|0;r=c[a+8436+(s<<2)>>2]|0;c[l+(n<<2)>>2]=(c[u+(o<<2)>>2]|0)+(c[t+(o<<2)>>2]|0)+(c[r+(o<<2)>>2]|0);c[l+(p<<2)>>2]=(c[u+(q<<2)>>2]|0)+(c[t+(q<<2)>>2]|0)+(c[r+(q<<2)>>2]|0);m=m+1|0;}while((m|0)<(j|0));return}else{j=c[e+4>>2]|0;e=c[f+4>>2]|0;f=i+1>>1;if((f|0)<=0){return}i=h;h=0;do{m=h<<1;l=(b[d+(m<<1)>>1]|0)+64>>7;q=m|1;r=(b[d+(q<<1)>>1]|0)+64>>7;t=((b[g+(h<<1)>>1]|0)+128+(b[e+(h<<1)>>1]|0)>>8)+128|0;u=((b[k+(h<<1)>>1]|0)+128+(b[j+(h<<1)>>1]|0)>>8)+128|0;p=c[a+2292+(t<<2)>>2]|0;o=(c[a+4340+(u<<2)>>2]|0)+(c[a+6388+(t<<2)>>2]|0)|0;t=c[a+8436+(u<<2)>>2]|0;c[i+(m<<2)>>2]=(c[o+(l<<2)>>2]|0)+(c[p+(l<<2)>>2]|0)+(c[t+(l<<2)>>2]|0);c[i+(q<<2)>>2]=(c[o+(r<<2)>>2]|0)+(c[p+(r<<2)>>2]|0)+(c[t+(r<<2)>>2]|0);h=h+1|0;}while((h|0)<(f|0));return}}function u1(a,d,e,f,g,h,i,j,k,l){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;l=c[d>>2]|0;g=c[d+4>>2]|0;d=c[e>>2]|0;m=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=4096-j|0;o=4096-k|0;p=i+1>>1;if((p|0)<=0){return}i=h;h=0;do{q=h<<1;r=ac(b[l+(q<<1)>>1]|0,f)|0;s=(ac(b[g+(q<<1)>>1]|0,j)|0)+r>>19;r=q|1;t=ac(b[l+(r<<1)>>1]|0,f)|0;u=(ac(b[g+(r<<1)>>1]|0,j)|0)+t>>19;t=ac(b[d+(h<<1)>>1]|0,o)|0;v=(ac(b[m+(h<<1)>>1]|0,k)|0)+t>>19;t=ac(b[e+(h<<1)>>1]|0,o)|0;w=((ac(b[n+(h<<1)>>1]|0,k)|0)+t>>19)+128|0;t=v+128|0;v=c[a+2292+(w<<2)>>2]|0;x=(c[a+4340+(t<<2)>>2]|0)+(c[a+6388+(w<<2)>>2]|0)|0;w=c[a+8436+(t<<2)>>2]|0;c[i+(q<<2)>>2]=(c[x+(s<<2)>>2]|0)+(c[v+(s<<2)>>2]|0)+(c[w+(s<<2)>>2]|0);c[i+(r<<2)>>2]=(c[x+(u<<2)>>2]|0)+(c[v+(u<<2)>>2]|0)+(c[w+(u<<2)>>2]|0);h=h+1|0;}while((h|0)<(p|0));return}function u2(a,d,e,f,g,h,i,j,k,l,m,n){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;n=m+1>>1;if((n|0)<=0){return}m=(f|0)>0;k=(j|0)>0;o=l;l=0;do{if(m){p=l<<1;q=p|1;r=0;s=262144;t=262144;do{u=c[e+(r<<2)>>2]|0;v=b[d+(r<<1)>>1]|0;s=(ac(v,b[u+(p<<1)>>1]|0)|0)+s|0;t=(ac(b[u+(q<<1)>>1]|0,v)|0)+t|0;r=r+1|0;}while((r|0)<(f|0));w=s>>19;x=t>>19}else{w=0;x=0}if(k){r=0;q=262144;p=262144;do{v=b[g+(r<<1)>>1]|0;q=(ac(v,b[(c[h+(r<<2)>>2]|0)+(l<<1)>>1]|0)|0)+q|0;p=(ac(b[(c[i+(r<<2)>>2]|0)+(l<<1)>>1]|0,v)|0)+p|0;r=r+1|0;}while((r|0)<(j|0));y=(q>>19)+128|0;z=(p>>19)+128|0}else{y=128;z=128}r=c[a+2292+(z<<2)>>2]|0;t=(c[a+4340+(y<<2)>>2]|0)+(c[a+6388+(z<<2)>>2]|0)|0;s=c[a+8436+(y<<2)>>2]|0;v=l<<1;c[o+(v<<2)>>2]=(c[t+(w<<2)>>2]|0)+(c[r+(w<<2)>>2]|0)+(c[s+(w<<2)>>2]|0);c[o+((v|1)<<2)>>2]=(c[t+(x<<2)>>2]|0)+(c[r+(x<<2)>>2]|0)+(c[s+(x<<2)>>2]|0);l=l+1|0;}while((l|0)<(n|0));return}function u3(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;l=c[f>>2]|0;h=c[g>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)>0){m=0}else{return}do{n=m<<1;o=(b[e+(n<<1)>>1]|0)+64>>7;p=(b[e+((n|1)<<1)>>1]|0)+64>>7;n=((b[h+(m<<1)>>1]|0)+64>>7)+128|0;q=c[d+2292+(n<<2)>>2]|0;r=((b[l+(m<<1)>>1]|0)+64>>7)+128|0;s=c[d+4340+(r<<2)>>2]|0;t=c[d+6388+(n<<2)>>2]|0;n=c[d+8436+(r<<2)>>2]|0;r=m*6|0;a[i+r|0]=a[q+o|0]|0;a[i+(r|1)|0]=a[s+(t+o)|0]|0;a[i+(r+2)|0]=a[n+o|0]|0;a[i+(r+3)|0]=a[q+p|0]|0;a[i+(r+4)|0]=a[s+(t+p)|0]|0;a[i+(r+5)|0]=a[n+p|0]|0;m=m+1|0;}while((m|0)<(k|0));return}else{k=c[f+4>>2]|0;f=c[g+4>>2]|0;g=j+1>>1;if((g|0)>0){u=0}else{return}do{j=u<<1;m=(b[e+(j<<1)>>1]|0)+64>>7;p=(b[e+((j|1)<<1)>>1]|0)+64>>7;j=((b[h+(u<<1)>>1]|0)+128+(b[f+(u<<1)>>1]|0)>>8)+128|0;n=c[d+2292+(j<<2)>>2]|0;r=((b[l+(u<<1)>>1]|0)+128+(b[k+(u<<1)>>1]|0)>>8)+128|0;t=c[d+4340+(r<<2)>>2]|0;s=c[d+6388+(j<<2)>>2]|0;j=c[d+8436+(r<<2)>>2]|0;r=u*6|0;a[i+r|0]=a[n+m|0]|0;a[i+(r|1)|0]=a[t+(s+m)|0]|0;a[i+(r+2)|0]=a[j+m|0]|0;a[i+(r+3)|0]=a[n+p|0]|0;a[i+(r+4)|0]=a[t+(s+p)|0]|0;a[i+(r+5)|0]=a[j+p|0]|0;u=u+1|0;}while((u|0)<(g|0));return}}function u4(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;q=j+1>>1;if((q|0)>0){r=0}else{return}do{j=r<<1;s=ac(b[m+(j<<1)>>1]|0,g)|0;t=(ac(b[h+(j<<1)>>1]|0,k)|0)+s>>19;s=j|1;j=ac(b[m+(s<<1)>>1]|0,g)|0;u=(ac(b[h+(s<<1)>>1]|0,k)|0)+j>>19;j=ac(b[e+(r<<1)>>1]|0,p)|0;s=(ac(b[n+(r<<1)>>1]|0,l)|0)+j>>19;j=ac(b[f+(r<<1)>>1]|0,p)|0;v=((ac(b[o+(r<<1)>>1]|0,l)|0)+j>>19)+128|0;j=c[d+2292+(v<<2)>>2]|0;w=s+128|0;s=c[d+4340+(w<<2)>>2]|0;x=c[d+6388+(v<<2)>>2]|0;v=c[d+8436+(w<<2)>>2]|0;w=r*6|0;a[i+w|0]=a[j+t|0]|0;a[i+(w|1)|0]=a[s+(x+t)|0]|0;a[i+(w+2)|0]=a[v+t|0]|0;a[i+(w+3)|0]=a[j+u|0]|0;a[i+(w+4)|0]=a[s+(x+u)|0]|0;a[i+(w+5)|0]=a[v+u|0]|0;r=r+1|0;}while((r|0)<(q|0));return}function u5(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;o=n+1>>1;if((o|0)<=0){return}n=(g|0)>0;l=(k|0)>0;p=0;do{if(n){q=p<<1;r=q|1;s=0;t=262144;u=262144;do{v=c[f+(s<<2)>>2]|0;w=b[e+(s<<1)>>1]|0;t=(ac(w,b[v+(q<<1)>>1]|0)|0)+t|0;u=(ac(b[v+(r<<1)>>1]|0,w)|0)+u|0;s=s+1|0;}while((s|0)<(g|0));x=t>>19;y=u>>19}else{x=0;y=0}if(l){s=0;r=262144;q=262144;do{w=b[h+(s<<1)>>1]|0;r=(ac(w,b[(c[i+(s<<2)>>2]|0)+(p<<1)>>1]|0)|0)+r|0;q=(ac(b[(c[j+(s<<2)>>2]|0)+(p<<1)>>1]|0,w)|0)+q|0;s=s+1|0;}while((s|0)<(k|0));z=(r>>19)+128|0;A=(q>>19)+128|0}else{z=128;A=128}s=c[d+2292+(A<<2)>>2]|0;u=c[d+4340+(z<<2)>>2]|0;t=c[d+6388+(A<<2)>>2]|0;w=c[d+8436+(z<<2)>>2]|0;v=p*6|0;a[m+v|0]=a[s+x|0]|0;a[m+(v|1)|0]=a[u+(t+x)|0]|0;a[m+(v+2)|0]=a[w+x|0]|0;a[m+(v+3)|0]=a[s+y|0]|0;a[m+(v+4)|0]=a[u+(t+y)|0]|0;a[m+(v+5)|0]=a[w+y|0]|0;p=p+1|0;}while((p|0)<(o|0));return}function u6(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;l=c[f>>2]|0;h=c[g>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)>0){m=0}else{return}do{n=m<<1;o=(b[e+(n<<1)>>1]|0)+64>>7;p=(b[e+((n|1)<<1)>>1]|0)+64>>7;n=((b[h+(m<<1)>>1]|0)+64>>7)+128|0;q=c[d+2292+(n<<2)>>2]|0;r=((b[l+(m<<1)>>1]|0)+64>>7)+128|0;s=c[d+4340+(r<<2)>>2]|0;t=c[d+6388+(n<<2)>>2]|0;n=c[d+8436+(r<<2)>>2]|0;r=m*6|0;a[i+r|0]=a[n+o|0]|0;a[i+(r|1)|0]=a[s+(t+o)|0]|0;a[i+(r+2)|0]=a[q+o|0]|0;a[i+(r+3)|0]=a[n+p|0]|0;a[i+(r+4)|0]=a[s+(t+p)|0]|0;a[i+(r+5)|0]=a[q+p|0]|0;m=m+1|0;}while((m|0)<(k|0));return}else{k=c[f+4>>2]|0;f=c[g+4>>2]|0;g=j+1>>1;if((g|0)>0){u=0}else{return}do{j=u<<1;m=(b[e+(j<<1)>>1]|0)+64>>7;p=(b[e+((j|1)<<1)>>1]|0)+64>>7;j=((b[h+(u<<1)>>1]|0)+128+(b[f+(u<<1)>>1]|0)>>8)+128|0;q=c[d+2292+(j<<2)>>2]|0;r=((b[l+(u<<1)>>1]|0)+128+(b[k+(u<<1)>>1]|0)>>8)+128|0;t=c[d+4340+(r<<2)>>2]|0;s=c[d+6388+(j<<2)>>2]|0;j=c[d+8436+(r<<2)>>2]|0;r=u*6|0;a[i+r|0]=a[j+m|0]|0;a[i+(r|1)|0]=a[t+(s+m)|0]|0;a[i+(r+2)|0]=a[q+m|0]|0;a[i+(r+3)|0]=a[j+p|0]|0;a[i+(r+4)|0]=a[t+(s+p)|0]|0;a[i+(r+5)|0]=a[q+p|0]|0;u=u+1|0;}while((u|0)<(g|0));return}}function u7(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;q=j+1>>1;if((q|0)>0){r=0}else{return}do{j=r<<1;s=ac(b[m+(j<<1)>>1]|0,g)|0;t=(ac(b[h+(j<<1)>>1]|0,k)|0)+s>>19;s=j|1;j=ac(b[m+(s<<1)>>1]|0,g)|0;u=(ac(b[h+(s<<1)>>1]|0,k)|0)+j>>19;j=ac(b[e+(r<<1)>>1]|0,p)|0;s=(ac(b[n+(r<<1)>>1]|0,l)|0)+j>>19;j=ac(b[f+(r<<1)>>1]|0,p)|0;v=((ac(b[o+(r<<1)>>1]|0,l)|0)+j>>19)+128|0;j=c[d+2292+(v<<2)>>2]|0;w=s+128|0;s=c[d+4340+(w<<2)>>2]|0;x=c[d+6388+(v<<2)>>2]|0;v=c[d+8436+(w<<2)>>2]|0;w=r*6|0;a[i+w|0]=a[v+t|0]|0;a[i+(w|1)|0]=a[s+(x+t)|0]|0;a[i+(w+2)|0]=a[j+t|0]|0;a[i+(w+3)|0]=a[v+u|0]|0;a[i+(w+4)|0]=a[s+(x+u)|0]|0;a[i+(w+5)|0]=a[j+u|0]|0;r=r+1|0;}while((r|0)<(q|0));return}function u8(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;o=n+1>>1;if((o|0)<=0){return}n=(g|0)>0;l=(k|0)>0;p=0;do{if(n){q=p<<1;r=q|1;s=0;t=262144;u=262144;do{v=c[f+(s<<2)>>2]|0;w=b[e+(s<<1)>>1]|0;t=(ac(w,b[v+(q<<1)>>1]|0)|0)+t|0;u=(ac(b[v+(r<<1)>>1]|0,w)|0)+u|0;s=s+1|0;}while((s|0)<(g|0));x=t>>19;y=u>>19}else{x=0;y=0}if(l){s=0;r=262144;q=262144;do{w=b[h+(s<<1)>>1]|0;r=(ac(w,b[(c[i+(s<<2)>>2]|0)+(p<<1)>>1]|0)|0)+r|0;q=(ac(b[(c[j+(s<<2)>>2]|0)+(p<<1)>>1]|0,w)|0)+q|0;s=s+1|0;}while((s|0)<(k|0));z=(r>>19)+128|0;A=(q>>19)+128|0}else{z=128;A=128}s=c[d+2292+(A<<2)>>2]|0;u=c[d+4340+(z<<2)>>2]|0;t=c[d+6388+(A<<2)>>2]|0;w=c[d+8436+(z<<2)>>2]|0;v=p*6|0;a[m+v|0]=a[w+x|0]|0;a[m+(v|1)|0]=a[u+(t+x)|0]|0;a[m+(v+2)|0]=a[s+x|0]|0;a[m+(v+3)|0]=a[w+y|0]|0;a[m+(v+4)|0]=a[u+(t+y)|0]|0;a[m+(v+5)|0]=a[s+y|0]|0;p=p+1|0;}while((p|0)<(o|0));return}function u9(a,e,f,g,h,i,j,k,l){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;h=c[f>>2]|0;m=c[g>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){return}n=i;o=l&1;p=o^1;q=d[262017+(p<<3)|0]|0;r=d[262041+(o<<3)|0]|0;s=d[262017+(o<<3)|0]|0;t=d[262016+(p<<3)|0]|0;p=d[262040+(o<<3)|0]|0;u=d[262016+(o<<3)|0]|0;o=0;do{v=o<<1;w=(b[e+(v<<1)>>1]|0)+64>>7;x=v|1;y=(b[e+(x<<1)>>1]|0)+64>>7;z=((b[m+(o<<1)>>1]|0)+64>>7)+128|0;A=((b[h+(o<<1)>>1]|0)+64>>7)+128|0;B=c[a+2292+(z<<2)>>2]|0;C=(c[a+4340+(A<<2)>>2]|0)+(c[a+6388+(z<<2)>>2]|0)|0;z=c[a+8436+(A<<2)>>2]|0;b[n+(v<<1)>>1]=((b[C+(p+w<<1)>>1]|0)+(b[B+(u+w<<1)>>1]|0)&65535)+(b[z+(t+w<<1)>>1]|0)&65535;b[n+(x<<1)>>1]=((b[C+(r+y<<1)>>1]|0)+(b[B+(s+y<<1)>>1]|0)&65535)+(b[z+(q+y<<1)>>1]|0)&65535;o=o+1|0;}while((o|0)<(k|0));return}else{k=c[f+4>>2]|0;f=c[g+4>>2]|0;g=j+1>>1;if((g|0)<=0){return}j=i;i=l&1;l=i^1;o=d[262017+(l<<3)|0]|0;q=d[262041+(i<<3)|0]|0;s=d[262017+(i<<3)|0]|0;r=d[262016+(l<<3)|0]|0;l=d[262040+(i<<3)|0]|0;n=d[262016+(i<<3)|0]|0;i=0;do{t=i<<1;u=(b[e+(t<<1)>>1]|0)+64>>7;p=t|1;y=(b[e+(p<<1)>>1]|0)+64>>7;z=((b[m+(i<<1)>>1]|0)+128+(b[f+(i<<1)>>1]|0)>>8)+128|0;B=((b[h+(i<<1)>>1]|0)+128+(b[k+(i<<1)>>1]|0)>>8)+128|0;C=c[a+2292+(z<<2)>>2]|0;x=(c[a+4340+(B<<2)>>2]|0)+(c[a+6388+(z<<2)>>2]|0)|0;z=c[a+8436+(B<<2)>>2]|0;b[j+(t<<1)>>1]=((b[x+(l+u<<1)>>1]|0)+(b[C+(n+u<<1)>>1]|0)&65535)+(b[z+(r+u<<1)>>1]|0)&65535;b[j+(p<<1)>>1]=((b[x+(q+y<<1)>>1]|0)+(b[C+(s+y<<1)>>1]|0)&65535)+(b[z+(o+y<<1)>>1]|0)&65535;i=i+1|0;}while((i|0)<(g|0));return}}function va(a,e,f,g,h,i,j,k,l,m){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;h=c[e>>2]|0;n=c[e+4>>2]|0;e=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=4096-k|0;q=4096-l|0;r=j+1>>1;if((r|0)<=0){return}j=i;i=m&1;m=i^1;s=d[262017+(m<<3)|0]|0;t=d[262041+(i<<3)|0]|0;u=d[262017+(i<<3)|0]|0;v=d[262016+(m<<3)|0]|0;m=d[262040+(i<<3)|0]|0;w=d[262016+(i<<3)|0]|0;i=0;do{x=i<<1;y=ac(b[h+(x<<1)>>1]|0,g)|0;z=(ac(b[n+(x<<1)>>1]|0,k)|0)+y>>19;y=x|1;A=ac(b[h+(y<<1)>>1]|0,g)|0;B=(ac(b[n+(y<<1)>>1]|0,k)|0)+A>>19;A=ac(b[e+(i<<1)>>1]|0,q)|0;C=(ac(b[o+(i<<1)>>1]|0,l)|0)+A>>19;A=ac(b[f+(i<<1)>>1]|0,q)|0;D=((ac(b[p+(i<<1)>>1]|0,l)|0)+A>>19)+128|0;A=C+128|0;C=c[a+2292+(D<<2)>>2]|0;E=(c[a+4340+(A<<2)>>2]|0)+(c[a+6388+(D<<2)>>2]|0)|0;D=c[a+8436+(A<<2)>>2]|0;b[j+(x<<1)>>1]=((b[E+(m+z<<1)>>1]|0)+(b[C+(w+z<<1)>>1]|0)&65535)+(b[D+(v+z<<1)>>1]|0)&65535;b[j+(y<<1)>>1]=((b[E+(t+B<<1)>>1]|0)+(b[C+(u+B<<1)>>1]|0)&65535)+(b[D+(s+B<<1)>>1]|0)&65535;i=i+1|0;}while((i|0)<(r|0));return}function vb(a,e,f,g,h,i,j,k,l,m,n,o){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;l=n+1>>1;if((l|0)<=0){return}n=(g|0)>0;p=(k|0)>0;q=m;m=o&1;o=m^1;r=d[262017+(o<<3)|0]|0;s=d[262041+(m<<3)|0]|0;t=d[262017+(m<<3)|0]|0;u=d[262016+(o<<3)|0]|0;o=d[262040+(m<<3)|0]|0;v=d[262016+(m<<3)|0]|0;m=0;do{if(n){w=m<<1;x=w|1;y=0;z=262144;A=262144;do{B=c[f+(y<<2)>>2]|0;C=b[e+(y<<1)>>1]|0;z=(ac(C,b[B+(w<<1)>>1]|0)|0)+z|0;A=(ac(b[B+(x<<1)>>1]|0,C)|0)+A|0;y=y+1|0;}while((y|0)<(g|0));D=z>>19;E=A>>19}else{D=0;E=0}if(p){y=0;x=262144;w=262144;do{C=b[h+(y<<1)>>1]|0;x=(ac(C,b[(c[i+(y<<2)>>2]|0)+(m<<1)>>1]|0)|0)+x|0;w=(ac(b[(c[j+(y<<2)>>2]|0)+(m<<1)>>1]|0,C)|0)+w|0;y=y+1|0;}while((y|0)<(k|0));F=(x>>19)+128|0;G=(w>>19)+128|0}else{F=128;G=128}y=c[a+2292+(G<<2)>>2]|0;A=(c[a+4340+(F<<2)>>2]|0)+(c[a+6388+(G<<2)>>2]|0)|0;z=c[a+8436+(F<<2)>>2]|0;C=m<<1;b[q+(C<<1)>>1]=((b[A+(o+D<<1)>>1]|0)+(b[y+(v+D<<1)>>1]|0)&65535)+(b[z+(u+D<<1)>>1]|0)&65535;b[q+((C|1)<<1)>>1]=((b[A+(s+E<<1)>>1]|0)+(b[y+(t+E<<1)>>1]|0)&65535)+(b[z+(r+E<<1)>>1]|0)&65535;m=m+1|0;}while((m|0)<(l|0));return}function vc(a,e,f,g,h,i,j,k,l){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;h=c[f>>2]|0;m=c[g>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){return}n=i;o=l&1;p=d[262016+(o<<3)|0]|0;q=d[262017+(o<<3)|0]|0;r=o^1;o=d[262017+(r<<3)|0]|0;s=d[262016+(r<<3)|0]|0;r=0;do{t=r<<1;u=(b[e+(t<<1)>>1]|0)+64>>7;v=t|1;w=(b[e+(v<<1)>>1]|0)+64>>7;x=((b[m+(r<<1)>>1]|0)+64>>7)+128|0;y=((b[h+(r<<1)>>1]|0)+64>>7)+128|0;z=c[a+2292+(x<<2)>>2]|0;A=(c[a+4340+(y<<2)>>2]|0)+(c[a+6388+(x<<2)>>2]|0)|0;x=c[a+8436+(y<<2)>>2]|0;b[n+(t<<1)>>1]=((b[A+(q+u<<1)>>1]|0)+(b[z+(p+u<<1)>>1]|0)&65535)+(b[x+(s+u<<1)>>1]|0)&65535;b[n+(v<<1)>>1]=((b[A+(p+w<<1)>>1]|0)+(b[z+(q+w<<1)>>1]|0)&65535)+(b[x+(o+w<<1)>>1]|0)&65535;r=r+1|0;}while((r|0)<(k|0));return}else{k=c[f+4>>2]|0;f=c[g+4>>2]|0;g=j+1>>1;if((g|0)<=0){return}j=i;i=l&1;l=d[262016+(i<<3)|0]|0;r=d[262017+(i<<3)|0]|0;o=i^1;i=d[262017+(o<<3)|0]|0;q=d[262016+(o<<3)|0]|0;o=0;do{p=o<<1;n=(b[e+(p<<1)>>1]|0)+64>>7;s=p|1;w=(b[e+(s<<1)>>1]|0)+64>>7;x=((b[m+(o<<1)>>1]|0)+128+(b[f+(o<<1)>>1]|0)>>8)+128|0;z=((b[h+(o<<1)>>1]|0)+128+(b[k+(o<<1)>>1]|0)>>8)+128|0;A=c[a+2292+(x<<2)>>2]|0;v=(c[a+4340+(z<<2)>>2]|0)+(c[a+6388+(x<<2)>>2]|0)|0;x=c[a+8436+(z<<2)>>2]|0;b[j+(p<<1)>>1]=((b[v+(r+n<<1)>>1]|0)+(b[A+(l+n<<1)>>1]|0)&65535)+(b[x+(q+n<<1)>>1]|0)&65535;b[j+(s<<1)>>1]=((b[v+(l+w<<1)>>1]|0)+(b[A+(r+w<<1)>>1]|0)&65535)+(b[x+(i+w<<1)>>1]|0)&65535;o=o+1|0;}while((o|0)<(g|0));return}}function vd(a,e,f,g,h,i,j,k,l,m){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;h=c[e>>2]|0;n=c[e+4>>2]|0;e=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=4096-k|0;q=4096-l|0;r=j+1>>1;if((r|0)<=0){return}j=i;i=m&1;m=d[262016+(i<<3)|0]|0;s=d[262017+(i<<3)|0]|0;t=i^1;i=d[262017+(t<<3)|0]|0;u=d[262016+(t<<3)|0]|0;t=0;do{v=t<<1;w=ac(b[h+(v<<1)>>1]|0,g)|0;x=(ac(b[n+(v<<1)>>1]|0,k)|0)+w>>19;w=v|1;y=ac(b[h+(w<<1)>>1]|0,g)|0;z=(ac(b[n+(w<<1)>>1]|0,k)|0)+y>>19;y=ac(b[e+(t<<1)>>1]|0,q)|0;A=(ac(b[o+(t<<1)>>1]|0,l)|0)+y>>19;y=ac(b[f+(t<<1)>>1]|0,q)|0;B=((ac(b[p+(t<<1)>>1]|0,l)|0)+y>>19)+128|0;y=A+128|0;A=c[a+2292+(B<<2)>>2]|0;C=(c[a+4340+(y<<2)>>2]|0)+(c[a+6388+(B<<2)>>2]|0)|0;B=c[a+8436+(y<<2)>>2]|0;b[j+(v<<1)>>1]=((b[C+(s+x<<1)>>1]|0)+(b[A+(m+x<<1)>>1]|0)&65535)+(b[B+(u+x<<1)>>1]|0)&65535;b[j+(w<<1)>>1]=((b[C+(m+z<<1)>>1]|0)+(b[A+(s+z<<1)>>1]|0)&65535)+(b[B+(i+z<<1)>>1]|0)&65535;t=t+1|0;}while((t|0)<(r|0));return}function ve(a,e,f,g,h,i,j,k,l,m,n,o){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;l=n+1>>1;if((l|0)<=0){return}n=(g|0)>0;p=(k|0)>0;q=m;m=o&1;o=d[262016+(m<<3)|0]|0;r=d[262017+(m<<3)|0]|0;s=m^1;m=d[262017+(s<<3)|0]|0;t=d[262016+(s<<3)|0]|0;s=0;do{if(n){u=s<<1;v=u|1;w=0;x=262144;y=262144;do{z=c[f+(w<<2)>>2]|0;A=b[e+(w<<1)>>1]|0;x=(ac(A,b[z+(u<<1)>>1]|0)|0)+x|0;y=(ac(b[z+(v<<1)>>1]|0,A)|0)+y|0;w=w+1|0;}while((w|0)<(g|0));B=x>>19;C=y>>19}else{B=0;C=0}if(p){w=0;v=262144;u=262144;do{A=b[h+(w<<1)>>1]|0;v=(ac(A,b[(c[i+(w<<2)>>2]|0)+(s<<1)>>1]|0)|0)+v|0;u=(ac(b[(c[j+(w<<2)>>2]|0)+(s<<1)>>1]|0,A)|0)+u|0;w=w+1|0;}while((w|0)<(k|0));D=(v>>19)+128|0;E=(u>>19)+128|0}else{D=128;E=128}w=c[a+2292+(E<<2)>>2]|0;y=(c[a+4340+(D<<2)>>2]|0)+(c[a+6388+(E<<2)>>2]|0)|0;x=c[a+8436+(D<<2)>>2]|0;A=s<<1;b[q+(A<<1)>>1]=((b[y+(r+B<<1)>>1]|0)+(b[w+(o+B<<1)>>1]|0)&65535)+(b[x+(t+B<<1)>>1]|0)&65535;b[q+((A|1)<<1)>>1]=((b[y+(o+C<<1)>>1]|0)+(b[w+(r+C<<1)>>1]|0)&65535)+(b[x+(m+C<<1)>>1]|0)&65535;s=s+1|0;}while((s|0)<(l|0));return}function vf(a,e,f,g,h,i,j,k,l){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;h=c[f>>2]|0;m=c[g>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)<=0){return}n=i;o=l&3;p=d[261976+(o<<3)|0]|0;q=d[261977+(o<<3)|0]|0;r=o^3;o=d[261977+(r<<3)|0]|0;s=d[261976+(r<<3)|0]|0;r=0;do{t=r<<1;u=(b[e+(t<<1)>>1]|0)+64>>7;v=t|1;w=(b[e+(v<<1)>>1]|0)+64>>7;x=((b[m+(r<<1)>>1]|0)+64>>7)+128|0;y=((b[h+(r<<1)>>1]|0)+64>>7)+128|0;z=c[a+2292+(x<<2)>>2]|0;A=(c[a+4340+(y<<2)>>2]|0)+(c[a+6388+(x<<2)>>2]|0)|0;x=c[a+8436+(y<<2)>>2]|0;b[n+(t<<1)>>1]=((b[A+(q+u<<1)>>1]|0)+(b[z+(p+u<<1)>>1]|0)&65535)+(b[x+(s+u<<1)>>1]|0)&65535;b[n+(v<<1)>>1]=((b[A+(p+w<<1)>>1]|0)+(b[z+(q+w<<1)>>1]|0)&65535)+(b[x+(o+w<<1)>>1]|0)&65535;r=r+1|0;}while((r|0)<(k|0));return}else{k=c[f+4>>2]|0;f=c[g+4>>2]|0;g=j+1>>1;if((g|0)<=0){return}j=i;i=l&3;l=d[261976+(i<<3)|0]|0;r=d[261977+(i<<3)|0]|0;o=i^3;i=d[261977+(o<<3)|0]|0;q=d[261976+(o<<3)|0]|0;o=0;do{p=o<<1;n=(b[e+(p<<1)>>1]|0)+64>>7;s=p|1;w=(b[e+(s<<1)>>1]|0)+64>>7;x=((b[m+(o<<1)>>1]|0)+128+(b[f+(o<<1)>>1]|0)>>8)+128|0;z=((b[h+(o<<1)>>1]|0)+128+(b[k+(o<<1)>>1]|0)>>8)+128|0;A=c[a+2292+(x<<2)>>2]|0;v=(c[a+4340+(z<<2)>>2]|0)+(c[a+6388+(x<<2)>>2]|0)|0;x=c[a+8436+(z<<2)>>2]|0;b[j+(p<<1)>>1]=((b[v+(r+n<<1)>>1]|0)+(b[A+(l+n<<1)>>1]|0)&65535)+(b[x+(q+n<<1)>>1]|0)&65535;b[j+(s<<1)>>1]=((b[v+(l+w<<1)>>1]|0)+(b[A+(r+w<<1)>>1]|0)&65535)+(b[x+(i+w<<1)>>1]|0)&65535;o=o+1|0;}while((o|0)<(g|0));return}}function vg(a,e,f,g,h,i,j,k,l,m){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;h=c[e>>2]|0;n=c[e+4>>2]|0;e=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=4096-k|0;q=4096-l|0;r=j+1>>1;if((r|0)<=0){return}j=i;i=m&3;m=d[261976+(i<<3)|0]|0;s=d[261977+(i<<3)|0]|0;t=i^3;i=d[261977+(t<<3)|0]|0;u=d[261976+(t<<3)|0]|0;t=0;do{v=t<<1;w=ac(b[h+(v<<1)>>1]|0,g)|0;x=(ac(b[n+(v<<1)>>1]|0,k)|0)+w>>19;w=v|1;y=ac(b[h+(w<<1)>>1]|0,g)|0;z=(ac(b[n+(w<<1)>>1]|0,k)|0)+y>>19;y=ac(b[e+(t<<1)>>1]|0,q)|0;A=(ac(b[o+(t<<1)>>1]|0,l)|0)+y>>19;y=ac(b[f+(t<<1)>>1]|0,q)|0;B=((ac(b[p+(t<<1)>>1]|0,l)|0)+y>>19)+128|0;y=A+128|0;A=c[a+2292+(B<<2)>>2]|0;C=(c[a+4340+(y<<2)>>2]|0)+(c[a+6388+(B<<2)>>2]|0)|0;B=c[a+8436+(y<<2)>>2]|0;b[j+(v<<1)>>1]=((b[C+(s+x<<1)>>1]|0)+(b[A+(m+x<<1)>>1]|0)&65535)+(b[B+(u+x<<1)>>1]|0)&65535;b[j+(w<<1)>>1]=((b[C+(m+z<<1)>>1]|0)+(b[A+(s+z<<1)>>1]|0)&65535)+(b[B+(i+z<<1)>>1]|0)&65535;t=t+1|0;}while((t|0)<(r|0));return}function vh(a,e,f,g,h,i,j,k,l,m,n,o){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;l=n+1>>1;if((l|0)<=0){return}n=(g|0)>0;p=(k|0)>0;q=m;m=o&3;o=d[261976+(m<<3)|0]|0;r=d[261977+(m<<3)|0]|0;s=m^3;m=d[261977+(s<<3)|0]|0;t=d[261976+(s<<3)|0]|0;s=0;do{if(n){u=s<<1;v=u|1;w=0;x=262144;y=262144;do{z=c[f+(w<<2)>>2]|0;A=b[e+(w<<1)>>1]|0;x=(ac(A,b[z+(u<<1)>>1]|0)|0)+x|0;y=(ac(b[z+(v<<1)>>1]|0,A)|0)+y|0;w=w+1|0;}while((w|0)<(g|0));B=x>>19;C=y>>19}else{B=0;C=0}if(p){w=0;v=262144;u=262144;do{A=b[h+(w<<1)>>1]|0;v=(ac(A,b[(c[i+(w<<2)>>2]|0)+(s<<1)>>1]|0)|0)+v|0;u=(ac(b[(c[j+(w<<2)>>2]|0)+(s<<1)>>1]|0,A)|0)+u|0;w=w+1|0;}while((w|0)<(k|0));D=(v>>19)+128|0;E=(u>>19)+128|0}else{D=128;E=128}w=c[a+2292+(E<<2)>>2]|0;y=(c[a+4340+(D<<2)>>2]|0)+(c[a+6388+(E<<2)>>2]|0)|0;x=c[a+8436+(D<<2)>>2]|0;A=s<<1;b[q+(A<<1)>>1]=((b[y+(r+B<<1)>>1]|0)+(b[w+(o+B<<1)>>1]|0)&65535)+(b[x+(t+B<<1)>>1]|0)&65535;b[q+((A|1)<<1)>>1]=((b[y+(o+C<<1)>>1]|0)+(b[w+(r+C<<1)>>1]|0)&65535)+(b[x+(m+C<<1)>>1]|0)&65535;s=s+1|0;}while((s|0)<(l|0));return}function vi(e,f,g,h,i,j,k,l,m){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;i=c[g>>2]|0;n=c[h>>2]|0;if((l|0)<2048){l=k+1>>1;if((l|0)<=0){return}o=m&7;p=0;do{q=p<<1;r=(b[f+(q<<1)>>1]|0)+64>>7;s=q|1;t=(b[f+(s<<1)>>1]|0)+64>>7;u=((b[n+(p<<1)>>1]|0)+64>>7)+128|0;v=c[e+2292+(u<<2)>>2]|0;w=((b[i+(p<<1)>>1]|0)+64>>7)+128|0;x=c[e+4340+(w<<2)>>2]|0;y=c[e+6388+(u<<2)>>2]|0;u=c[e+8436+(w<<2)>>2]|0;w=q&6;z=s&7;A=d[261768+(o<<3)+z|0]|0;B=d[261696+(o<<3)+z|0]|0;z=(d[261768+(o<<3)+w|0]|0)+r|0;a[j+q|0]=((a[v+z|0]|0)+(a[x+(z+y)|0]|0)&255)+(a[u+((d[261696+(o<<3)+w|0]|0)+r)|0]|0)&255;r=A+t|0;a[j+s|0]=((a[x+(r+y)|0]|0)+(a[v+r|0]|0)&255)+(a[u+(B+t)|0]|0)&255;p=p+1|0;}while((p|0)<(l|0));return}else{l=c[g+4>>2]|0;g=c[h+4>>2]|0;h=k+1>>1;if((h|0)<=0){return}k=m&7;m=0;do{p=m<<1;o=(b[f+(p<<1)>>1]|0)+64>>7;t=p|1;B=(b[f+(t<<1)>>1]|0)+64>>7;u=((b[n+(m<<1)>>1]|0)+128+(b[g+(m<<1)>>1]|0)>>8)+128|0;r=c[e+2292+(u<<2)>>2]|0;v=((b[i+(m<<1)>>1]|0)+128+(b[l+(m<<1)>>1]|0)>>8)+128|0;y=c[e+4340+(v<<2)>>2]|0;x=c[e+6388+(u<<2)>>2]|0;u=c[e+8436+(v<<2)>>2]|0;v=p&6;s=t&7;A=d[261768+(k<<3)+s|0]|0;w=d[261696+(k<<3)+s|0]|0;s=(d[261768+(k<<3)+v|0]|0)+o|0;a[j+p|0]=((a[r+s|0]|0)+(a[y+(s+x)|0]|0)&255)+(a[u+((d[261696+(k<<3)+v|0]|0)+o)|0]|0)&255;o=A+B|0;a[j+t|0]=((a[y+(o+x)|0]|0)+(a[r+o|0]|0)&255)+(a[u+(w+B)|0]|0)&255;m=m+1|0;}while((m|0)<(h|0));return}}function vj(e,f,g,h,i,j,k,l,m,n){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;i=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=c[h>>2]|0;q=c[h+4>>2]|0;h=4096-l|0;r=4096-m|0;s=k+1>>1;if((s|0)<=0){return}k=n&7;n=0;do{t=n<<1;u=ac(b[i+(t<<1)>>1]|0,h)|0;v=(ac(b[o+(t<<1)>>1]|0,l)|0)+u>>19;u=t|1;w=ac(b[i+(u<<1)>>1]|0,h)|0;x=(ac(b[o+(u<<1)>>1]|0,l)|0)+w>>19;w=ac(b[f+(n<<1)>>1]|0,r)|0;y=(ac(b[p+(n<<1)>>1]|0,m)|0)+w>>19;w=ac(b[g+(n<<1)>>1]|0,r)|0;z=((ac(b[q+(n<<1)>>1]|0,m)|0)+w>>19)+128|0;w=c[e+2292+(z<<2)>>2]|0;A=y+128|0;y=c[e+4340+(A<<2)>>2]|0;B=c[e+6388+(z<<2)>>2]|0;z=c[e+8436+(A<<2)>>2]|0;A=t&6;C=u&7;D=d[261768+(k<<3)+C|0]|0;E=d[261696+(k<<3)+C|0]|0;C=(d[261768+(k<<3)+A|0]|0)+v|0;a[j+t|0]=((a[w+C|0]|0)+(a[y+(C+B)|0]|0)&255)+(a[z+((d[261696+(k<<3)+A|0]|0)+v)|0]|0)&255;v=D+x|0;a[j+u|0]=((a[y+(v+B)|0]|0)+(a[w+v|0]|0)&255)+(a[z+(E+x)|0]|0)&255;n=n+1|0;}while((n|0)<(s|0));return}function vk(e,f,g,h,i,j,k,l,m,n,o,p){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;m=o+1>>1;if((m|0)<=0){return}o=(h|0)>0;q=(l|0)>0;r=p&7;p=0;do{if(o){s=p<<1;t=s|1;u=0;v=262144;w=262144;do{x=c[g+(u<<2)>>2]|0;y=b[f+(u<<1)>>1]|0;v=(ac(y,b[x+(s<<1)>>1]|0)|0)+v|0;w=(ac(b[x+(t<<1)>>1]|0,y)|0)+w|0;u=u+1|0;}while((u|0)<(h|0));z=v>>19;A=w>>19}else{z=0;A=0}if(q){u=0;t=262144;s=262144;do{y=b[i+(u<<1)>>1]|0;t=(ac(y,b[(c[j+(u<<2)>>2]|0)+(p<<1)>>1]|0)|0)+t|0;s=(ac(b[(c[k+(u<<2)>>2]|0)+(p<<1)>>1]|0,y)|0)+s|0;u=u+1|0;}while((u|0)<(l|0));B=(t>>19)+128|0;C=(s>>19)+128|0}else{B=128;C=128}u=c[e+2292+(C<<2)>>2]|0;w=c[e+4340+(B<<2)>>2]|0;v=c[e+6388+(C<<2)>>2]|0;y=c[e+8436+(B<<2)>>2]|0;x=p<<1;D=x&6;E=D|1;F=d[261768+(r<<3)+E|0]|0;G=d[261696+(r<<3)+E|0]|0;E=(d[261768+(r<<3)+D|0]|0)+z|0;a[n+x|0]=((a[u+E|0]|0)+(a[w+(E+v)|0]|0)&255)+(a[y+((d[261696+(r<<3)+D|0]|0)+z)|0]|0)&255;D=F+A|0;a[n+(x|1)|0]=((a[w+(D+v)|0]|0)+(a[u+D|0]|0)&255)+(a[y+(G+A)|0]|0)&255;p=p+1|0;}while((p|0)<(m|0));return}function vl(e,f,g,h,i,j,k,l,m){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;i=c[g>>2]|0;n=c[h>>2]|0;if((l|0)<2048){l=k+1>>1;if((l|0)<=0){return}o=m&7;p=0;do{q=p<<1;r=(b[f+(q<<1)>>1]|0)+64>>7;s=q|1;t=(b[f+(s<<1)>>1]|0)+64>>7;u=((b[n+(p<<1)>>1]|0)+64>>7)+128|0;v=c[e+2292+(u<<2)>>2]|0;w=((b[i+(p<<1)>>1]|0)+64>>7)+128|0;x=c[e+4340+(w<<2)>>2]|0;y=c[e+6388+(u<<2)>>2]|0;u=c[e+8436+(w<<2)>>2]|0;w=q&6;q=s&7;s=(d[261840+(o<<3)+w|0]|0)+r|0;z=(d[261840+(o<<3)+q|0]|0)+t|0;a[j+p|0]=(((a[v+s|0]|0)+(a[x+(y+r+(d[261696+(o<<3)+w|0]|0))|0]|0)&255)+(a[u+s|0]|0)&255)+((((a[x+(y+t+(d[261696+(o<<3)+q|0]|0))|0]|0)+(a[v+z|0]|0)&255)+(a[u+z|0]|0)&255)<<4)&255;p=p+1|0;}while((p|0)<(l|0));return}else{l=c[g+4>>2]|0;g=c[h+4>>2]|0;h=k+1>>1;if((h|0)<=0){return}k=m&7;m=0;do{p=m<<1;o=(b[f+(p<<1)>>1]|0)+64>>7;z=p|1;u=(b[f+(z<<1)>>1]|0)+64>>7;v=((b[n+(m<<1)>>1]|0)+128+(b[g+(m<<1)>>1]|0)>>8)+128|0;q=c[e+2292+(v<<2)>>2]|0;t=((b[i+(m<<1)>>1]|0)+128+(b[l+(m<<1)>>1]|0)>>8)+128|0;y=c[e+4340+(t<<2)>>2]|0;x=c[e+6388+(v<<2)>>2]|0;v=c[e+8436+(t<<2)>>2]|0;t=p&6;p=z&7;z=(d[261840+(k<<3)+t|0]|0)+o|0;s=(d[261840+(k<<3)+p|0]|0)+u|0;a[j+m|0]=(((a[q+z|0]|0)+(a[y+(x+o+(d[261696+(k<<3)+t|0]|0))|0]|0)&255)+(a[v+z|0]|0)&255)+((((a[y+(x+u+(d[261696+(k<<3)+p|0]|0))|0]|0)+(a[q+s|0]|0)&255)+(a[v+s|0]|0)&255)<<4)&255;m=m+1|0;}while((m|0)<(h|0));return}}function vm(e,f,g,h,i,j,k,l,m,n){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;i=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=c[h>>2]|0;q=c[h+4>>2]|0;h=4096-l|0;r=4096-m|0;s=k+1>>1;if((s|0)<=0){return}k=n&7;n=0;do{t=n<<1;u=ac(b[i+(t<<1)>>1]|0,h)|0;v=(ac(b[o+(t<<1)>>1]|0,l)|0)+u>>19;u=t|1;w=ac(b[i+(u<<1)>>1]|0,h)|0;x=(ac(b[o+(u<<1)>>1]|0,l)|0)+w>>19;w=ac(b[f+(n<<1)>>1]|0,r)|0;y=(ac(b[p+(n<<1)>>1]|0,m)|0)+w>>19;w=ac(b[g+(n<<1)>>1]|0,r)|0;z=((ac(b[q+(n<<1)>>1]|0,m)|0)+w>>19)+128|0;w=c[e+2292+(z<<2)>>2]|0;A=y+128|0;y=c[e+4340+(A<<2)>>2]|0;B=c[e+6388+(z<<2)>>2]|0;z=c[e+8436+(A<<2)>>2]|0;A=t&6;t=u&7;u=(d[261840+(k<<3)+A|0]|0)+v|0;C=(d[261840+(k<<3)+t|0]|0)+x|0;a[j+n|0]=(((a[w+u|0]|0)+(a[y+(B+v+(d[261696+(k<<3)+A|0]|0))|0]|0)&255)+(a[z+u|0]|0)&255)+((((a[y+(B+x+(d[261696+(k<<3)+t|0]|0))|0]|0)+(a[w+C|0]|0)&255)+(a[z+C|0]|0)&255)<<4)&255;n=n+1|0;}while((n|0)<(s|0));return}function vn(e,f,g,h,i,j,k,l,m,n,o,p){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;m=o+1>>1;if((m|0)<=0){return}o=(h|0)>0;q=(l|0)>0;r=p&7;p=0;do{if(o){s=p<<1;t=s|1;u=0;v=262144;w=262144;do{x=c[g+(u<<2)>>2]|0;y=b[f+(u<<1)>>1]|0;v=(ac(y,b[x+(s<<1)>>1]|0)|0)+v|0;w=(ac(b[x+(t<<1)>>1]|0,y)|0)+w|0;u=u+1|0;}while((u|0)<(h|0));z=v>>19;A=w>>19}else{z=0;A=0}if(q){u=0;t=262144;s=262144;do{y=b[i+(u<<1)>>1]|0;t=(ac(y,b[(c[j+(u<<2)>>2]|0)+(p<<1)>>1]|0)|0)+t|0;s=(ac(b[(c[k+(u<<2)>>2]|0)+(p<<1)>>1]|0,y)|0)+s|0;u=u+1|0;}while((u|0)<(l|0));B=(t>>19)+128|0;C=(s>>19)+128|0}else{B=128;C=128}u=c[e+2292+(C<<2)>>2]|0;w=c[e+4340+(B<<2)>>2]|0;v=c[e+6388+(C<<2)>>2]|0;y=c[e+8436+(B<<2)>>2]|0;x=p<<1&6;D=x|1;E=(d[261840+(r<<3)+x|0]|0)+z|0;F=(d[261840+(r<<3)+D|0]|0)+A|0;a[n+p|0]=(((a[u+E|0]|0)+(a[w+(v+z+(d[261696+(r<<3)+x|0]|0))|0]|0)&255)+(a[y+E|0]|0)&255)+((((a[w+(v+A+(d[261696+(r<<3)+D|0]|0))|0]|0)+(a[u+F|0]|0)&255)+(a[y+F|0]|0)&255)<<4)&255;p=p+1|0;}while((p|0)<(m|0));return}function vo(e,f,g,h,i,j,k,l,m){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;i=c[g>>2]|0;n=c[h>>2]|0;if((l|0)<2048){l=k+1>>1;if((l|0)<=0){return}o=m&7;p=0;do{q=p<<1;r=(b[f+(q<<1)>>1]|0)+64>>7;s=q|1;t=(b[f+(s<<1)>>1]|0)+64>>7;u=((b[n+(p<<1)>>1]|0)+64>>7)+128|0;v=c[e+2292+(u<<2)>>2]|0;w=((b[i+(p<<1)>>1]|0)+64>>7)+128|0;x=c[e+4340+(w<<2)>>2]|0;y=c[e+6388+(u<<2)>>2]|0;u=c[e+8436+(w<<2)>>2]|0;w=q&6;z=s&7;A=d[261840+(o<<3)+z|0]|0;B=d[261696+(o<<3)+z|0]|0;z=(d[261840+(o<<3)+w|0]|0)+r|0;a[j+q|0]=((a[v+z|0]|0)+(a[x+(y+r+(d[261696+(o<<3)+w|0]|0))|0]|0)&255)+(a[u+z|0]|0)&255;z=A+t|0;a[j+s|0]=((a[x+(y+t+B)|0]|0)+(a[v+z|0]|0)&255)+(a[u+z|0]|0)&255;p=p+1|0;}while((p|0)<(l|0));return}else{l=c[g+4>>2]|0;g=c[h+4>>2]|0;h=k+1>>1;if((h|0)<=0){return}k=m&7;m=0;do{p=m<<1;o=(b[f+(p<<1)>>1]|0)+64>>7;z=p|1;u=(b[f+(z<<1)>>1]|0)+64>>7;v=((b[n+(m<<1)>>1]|0)+128+(b[g+(m<<1)>>1]|0)>>8)+128|0;B=c[e+2292+(v<<2)>>2]|0;t=((b[i+(m<<1)>>1]|0)+128+(b[l+(m<<1)>>1]|0)>>8)+128|0;y=c[e+4340+(t<<2)>>2]|0;x=c[e+6388+(v<<2)>>2]|0;v=c[e+8436+(t<<2)>>2]|0;t=p&6;s=z&7;A=d[261840+(k<<3)+s|0]|0;w=d[261696+(k<<3)+s|0]|0;s=(d[261840+(k<<3)+t|0]|0)+o|0;a[j+p|0]=((a[B+s|0]|0)+(a[y+(x+o+(d[261696+(k<<3)+t|0]|0))|0]|0)&255)+(a[v+s|0]|0)&255;s=A+u|0;a[j+z|0]=((a[y+(x+u+w)|0]|0)+(a[B+s|0]|0)&255)+(a[v+s|0]|0)&255;m=m+1|0;}while((m|0)<(h|0));return}}function vp(e,f,g,h,i,j,k,l,m,n){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;i=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=c[h>>2]|0;q=c[h+4>>2]|0;h=4096-l|0;r=4096-m|0;s=k+1>>1;if((s|0)<=0){return}k=n&7;n=0;do{t=n<<1;u=ac(b[i+(t<<1)>>1]|0,h)|0;v=(ac(b[o+(t<<1)>>1]|0,l)|0)+u>>19;u=t|1;w=ac(b[i+(u<<1)>>1]|0,h)|0;x=(ac(b[o+(u<<1)>>1]|0,l)|0)+w>>19;w=ac(b[f+(n<<1)>>1]|0,r)|0;y=(ac(b[p+(n<<1)>>1]|0,m)|0)+w>>19;w=ac(b[g+(n<<1)>>1]|0,r)|0;z=((ac(b[q+(n<<1)>>1]|0,m)|0)+w>>19)+128|0;w=c[e+2292+(z<<2)>>2]|0;A=y+128|0;y=c[e+4340+(A<<2)>>2]|0;B=c[e+6388+(z<<2)>>2]|0;z=c[e+8436+(A<<2)>>2]|0;A=t&6;C=u&7;D=d[261840+(k<<3)+C|0]|0;E=d[261696+(k<<3)+C|0]|0;C=(d[261840+(k<<3)+A|0]|0)+v|0;a[j+t|0]=((a[w+C|0]|0)+(a[y+(B+v+(d[261696+(k<<3)+A|0]|0))|0]|0)&255)+(a[z+C|0]|0)&255;C=D+x|0;a[j+u|0]=((a[y+(B+x+E)|0]|0)+(a[w+C|0]|0)&255)+(a[z+C|0]|0)&255;n=n+1|0;}while((n|0)<(s|0));return}function vq(e,f,g,h,i,j,k,l,m,n,o,p){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;m=o+1>>1;if((m|0)<=0){return}o=(h|0)>0;q=(l|0)>0;r=p&7;p=0;do{if(o){s=p<<1;t=s|1;u=0;v=262144;w=262144;do{x=c[g+(u<<2)>>2]|0;y=b[f+(u<<1)>>1]|0;v=(ac(y,b[x+(s<<1)>>1]|0)|0)+v|0;w=(ac(b[x+(t<<1)>>1]|0,y)|0)+w|0;u=u+1|0;}while((u|0)<(h|0));z=v>>19;A=w>>19}else{z=0;A=0}if(q){u=0;t=262144;s=262144;do{y=b[i+(u<<1)>>1]|0;t=(ac(y,b[(c[j+(u<<2)>>2]|0)+(p<<1)>>1]|0)|0)+t|0;s=(ac(b[(c[k+(u<<2)>>2]|0)+(p<<1)>>1]|0,y)|0)+s|0;u=u+1|0;}while((u|0)<(l|0));B=(t>>19)+128|0;C=(s>>19)+128|0}else{B=128;C=128}u=c[e+2292+(C<<2)>>2]|0;w=c[e+4340+(B<<2)>>2]|0;v=c[e+6388+(C<<2)>>2]|0;y=c[e+8436+(B<<2)>>2]|0;x=p<<1;D=x&6;E=D|1;F=d[261840+(r<<3)+E|0]|0;G=d[261696+(r<<3)+E|0]|0;E=(d[261840+(r<<3)+D|0]|0)+z|0;a[n+x|0]=((a[u+E|0]|0)+(a[w+(v+z+(d[261696+(r<<3)+D|0]|0))|0]|0)&255)+(a[y+E|0]|0)&255;E=F+A|0;a[n+(x|1)|0]=((a[w+(v+A+G)|0]|0)+(a[u+E|0]|0)&255)+(a[y+E|0]|0)&255;p=p+1|0;}while((p|0)<(m|0));return}function vr(e,f,g,h,i,j,k,l,m){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;l=m&7;m=261840+(l<<3)|0;i=(k|0)>0;if((c[e+2284>>2]&8388608|0)==0){if(!i){return}h=d[261847+(l<<3)|0]|0;g=d[261846+(l<<3)|0]|0;n=d[261845+(l<<3)|0]|0;o=d[261844+(l<<3)|0]|0;p=d[261843+(l<<3)|0]|0;q=d[261842+(l<<3)|0]|0;r=d[261841+(l<<3)|0]|0;l=d[m]|0;m=0;s=j;while(1){a[s]=~(((((((((l+((b[f+(m<<1)>>1]|0)+64>>7)|0)>237)<<1|(r+((b[f+((m|1)<<1)>>1]|0)+64>>7)|0)>237)<<1|(q+((b[f+((m|2)<<1)>>1]|0)+64>>7)|0)>237)<<1|(p+((b[f+((m|3)<<1)>>1]|0)+64>>7)|0)>237)<<1|(o+((b[f+((m|4)<<1)>>1]|0)+64>>7)|0)>237)<<1|(n+((b[f+((m|5)<<1)>>1]|0)+64>>7)|0)>237)<<1|(g+((b[f+((m|6)<<1)>>1]|0)+64>>7)|0)>237)<<1|(h+((b[f+((m|7)<<1)>>1]|0)+64>>7)|0)>237);t=m+8|0;if((t|0)<(k|0)){m=t;s=s+1|0}else{break}}return}s=e+11200|0;if(i){i=0;e=0;m=0;h=j;while(1){j=c[s>>2]|0;g=j+(m<<2)|0;n=m|1;o=m+2|0;p=((e*7|0)-248+(c[g>>2]|0)+((c[j+(n<<2)>>2]|0)*5|0)+((c[j+(o<<2)>>2]|0)*3|0)>>4)+((b[f+(m<<1)>>1]|0)+64>>7)|0;c[g>>2]=e;g=(p|0)>127|0;j=p-(-g&220)|0;p=c[s>>2]|0;q=p+(n<<2)|0;r=((c[q>>2]|0)-248+((c[p+(o<<2)>>2]|0)*5|0)+((c[p+(m+3<<2)>>2]|0)*3|0)+(j*7|0)>>4)+((b[f+(n<<1)>>1]|0)+64>>7)|0;c[q>>2]=j;j=(r|0)>127|0;q=j|(g|i)<<1;g=r-(-j&220)|0;if((m&6|0)==6){a[h]=(q^255)&255;u=h+1|0}else{u=h}if((o|0)<(k|0)){i=q<<1;e=g;m=o;h=u}else{v=g;w=o;break}}}else{v=0;w=0}c[(c[s>>2]|0)+(w<<2)>>2]=v;return}function vs(e,f,g,h,i,j,k,l,m,n){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0;m=c[f>>2]|0;i=c[f+4>>2]|0;f=n&7;n=261840+(f<<3)|0;h=4096-l|0;g=(k|0)>0;if((c[e+2284>>2]&8388608|0)==0){if(!g){return}o=d[261847+(f<<3)|0]|0;p=d[261846+(f<<3)|0]|0;q=d[261845+(f<<3)|0]|0;r=d[261844+(f<<3)|0]|0;s=d[261843+(f<<3)|0]|0;t=d[261842+(f<<3)|0]|0;u=d[261841+(f<<3)|0]|0;f=d[n]|0;n=0;v=j;while(1){w=n|1;x=n|2;y=n|3;z=n|4;A=n|5;B=n|6;C=n|7;D=b[m+(C<<1)>>1]|0;E=b[i+(B<<1)>>1]|0;F=b[m+(B<<1)>>1]|0;B=b[i+(A<<1)>>1]|0;G=b[m+(A<<1)>>1]|0;A=b[i+(z<<1)>>1]|0;H=b[m+(z<<1)>>1]|0;z=b[i+(y<<1)>>1]|0;I=b[m+(y<<1)>>1]|0;y=b[i+(x<<1)>>1]|0;J=b[m+(x<<1)>>1]|0;x=b[i+(w<<1)>>1]|0;K=b[m+(w<<1)>>1]|0;w=b[i+(n<<1)>>1]|0;L=b[m+(n<<1)>>1]|0;M=ac(b[i+(C<<1)>>1]|0,l)|0;C=ac(D,h)|0;D=ac(E,l)|0;E=ac(F,h)|0;F=ac(B,l)|0;B=ac(G,h)|0;G=ac(A,l)|0;A=ac(H,h)|0;H=ac(z,l)|0;z=ac(I,h)|0;I=ac(y,l)|0;y=ac(J,h)|0;J=ac(x,l)|0;x=ac(K,h)|0;K=ac(w,l)|0;a[v]=~(((((((((f+((ac(L,h)|0)+K>>19)|0)>237)<<1|(u+(x+J>>19)|0)>237)<<1|(t+(y+I>>19)|0)>237)<<1|(s+(z+H>>19)|0)>237)<<1|(r+(A+G>>19)|0)>237)<<1|(q+(B+F>>19)|0)>237)<<1|(p+(E+D>>19)|0)>237)<<1|(o+(C+M>>19)|0)>237);M=n+8|0;if((M|0)<(k|0)){n=M;v=v+1|0}else{break}}return}v=e+11200|0;if(g){g=0;e=0;n=0;o=j;while(1){j=ac(b[m+(n<<1)>>1]|0,h)|0;p=(ac(b[i+(n<<1)>>1]|0,l)|0)+j>>19;j=c[v>>2]|0;q=j+(n<<2)|0;r=n|1;s=n+2|0;t=((e*7|0)-248+(c[q>>2]|0)+((c[j+(r<<2)>>2]|0)*5|0)+((c[j+(s<<2)>>2]|0)*3|0)>>4)+p|0;c[q>>2]=e;q=(t|0)>127|0;p=t-(-q&220)|0;t=ac(b[m+(r<<1)>>1]|0,h)|0;j=(ac(b[i+(r<<1)>>1]|0,l)|0)+t>>19;t=c[v>>2]|0;u=t+(r<<2)|0;r=((c[u>>2]|0)-248+((c[t+(s<<2)>>2]|0)*5|0)+((c[t+(n+3<<2)>>2]|0)*3|0)+(p*7|0)>>4)+j|0;c[u>>2]=p;p=(r|0)>127|0;u=p|(q|g)<<1;q=r-(-p&220)|0;if((n&6|0)==6){a[o]=(u^255)&255;N=o+1|0}else{N=o}if((s|0)<(k|0)){g=u<<1;e=q;n=s;o=N}else{O=q;P=s;break}}}else{O=0;P=0}c[(c[v>>2]|0)+(P<<2)>>2]=O;return}function vt(e,f,g,h,i,j,k,l,m,n,o,p){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0;m=p&7;if((o|0)<=0){c[c[e+11200>>2]>>2]=0;return}p=(h|0)>0;l=e+2284|0;k=e+11200|0;e=0;j=0;i=0;q=n;while(1){if(p){n=i|1;r=262144;s=262144;t=0;do{u=c[g+(t<<2)>>2]|0;v=b[f+(t<<1)>>1]|0;s=(ac(v,b[u+(i<<1)>>1]|0)|0)+s|0;r=(ac(b[u+(n<<1)>>1]|0,v)|0)+r|0;t=t+1|0;}while((t|0)<(h|0));w=r>>19;x=s>>19}else{w=0;x=0}if(((x|w)&256|0)==0){y=x;z=w}else{if(x>>>0>255){A=-x>>31&255}else{A=x&255}if(w>>>0>255){B=-w>>31&255}else{B=w&255}y=A&255;z=B&255}if((c[l>>2]&8388608|0)==0){t=i&6;C=(((d[261840+(m<<3)+t|0]|0)+y|0)>237|j<<1)<<1|((d[(t|1)+(261840+(m<<3))|0]|0)+z|0)>237;D=e;E=t}else{t=c[k>>2]|0;n=t+(i<<2)|0;v=i|1;u=i+2|0;F=((e*7|0)-248+(c[n>>2]|0)+((c[t+(v<<2)>>2]|0)*5|0)+((c[t+(u<<2)>>2]|0)*3|0)>>4)+y|0;c[n>>2]=e;n=(F|0)>127|0;t=F-(-n&220)|0;F=c[k>>2]|0;G=F+(v<<2)|0;v=((c[G>>2]|0)-248+((c[F+(u<<2)>>2]|0)*5|0)+((c[F+(i+3<<2)>>2]|0)*3|0)+(t*7|0)>>4)+z|0;c[G>>2]=t;t=(v|0)>127|0;C=t|(n|j<<1)<<1;D=v-(-t&220)|0;E=i&6}if((E|0)==6){a[q]=(C^255)&255;H=q+1|0}else{H=q}I=i+2|0;if((I|0)<(o|0)){e=D;j=C;i=I;q=H}else{break}}c[(c[k>>2]|0)+(I<<2)>>2]=D;if((I&6|0)==0){return}a[H]=(C^255)&255;return}function vu(e,f,g,h,i,j,k,l,m){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;l=m&7;m=261840+(l<<3)|0;i=(k|0)>0;if((c[e+2284>>2]&8388608|0)==0){if(!i){return}h=d[261847+(l<<3)|0]|0;g=d[261846+(l<<3)|0]|0;n=d[261845+(l<<3)|0]|0;o=d[261844+(l<<3)|0]|0;p=d[261843+(l<<3)|0]|0;q=d[261842+(l<<3)|0]|0;r=d[261841+(l<<3)|0]|0;l=d[m]|0;m=0;s=j;while(1){a[s]=((((((((l+((b[f+(m<<1)>>1]|0)+64>>7)|0)>237)<<1|(r+((b[f+((m|1)<<1)>>1]|0)+64>>7)|0)>237)<<1|(q+((b[f+((m|2)<<1)>>1]|0)+64>>7)|0)>237)<<1|(p+((b[f+((m|3)<<1)>>1]|0)+64>>7)|0)>237)<<1|(o+((b[f+((m|4)<<1)>>1]|0)+64>>7)|0)>237)<<1|(n+((b[f+((m|5)<<1)>>1]|0)+64>>7)|0)>237)<<1|(g+((b[f+((m|6)<<1)>>1]|0)+64>>7)|0)>237)<<1|(h+((b[f+((m|7)<<1)>>1]|0)+64>>7)|0)>237;t=m+8|0;if((t|0)<(k|0)){m=t;s=s+1|0}else{break}}return}s=e+11200|0;if(i){i=0;e=0;m=0;h=j;while(1){j=c[s>>2]|0;g=j+(m<<2)|0;n=m|1;o=m+2|0;p=((e*7|0)-248+(c[g>>2]|0)+((c[j+(n<<2)>>2]|0)*5|0)+((c[j+(o<<2)>>2]|0)*3|0)>>4)+((b[f+(m<<1)>>1]|0)+64>>7)|0;c[g>>2]=e;g=(p|0)>127|0;j=p-(-g&220)|0;p=c[s>>2]|0;q=p+(n<<2)|0;r=((c[q>>2]|0)-248+((c[p+(o<<2)>>2]|0)*5|0)+((c[p+(m+3<<2)>>2]|0)*3|0)+(j*7|0)>>4)+((b[f+(n<<1)>>1]|0)+64>>7)|0;c[q>>2]=j;j=(r|0)>127|0;q=j|(g|i)<<1;g=r-(-j&220)|0;if((m&6|0)==6){a[h]=q&255;u=h+1|0}else{u=h}if((o|0)<(k|0)){i=q<<1;e=g;m=o;h=u}else{v=g;w=o;break}}}else{v=0;w=0}c[(c[s>>2]|0)+(w<<2)>>2]=v;return}function vv(e,f,g,h,i,j,k,l,m,n){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0;m=c[f>>2]|0;i=c[f+4>>2]|0;f=n&7;n=261840+(f<<3)|0;h=4096-l|0;g=(k|0)>0;if((c[e+2284>>2]&8388608|0)==0){if(!g){return}o=d[261847+(f<<3)|0]|0;p=d[261846+(f<<3)|0]|0;q=d[261845+(f<<3)|0]|0;r=d[261844+(f<<3)|0]|0;s=d[261843+(f<<3)|0]|0;t=d[261842+(f<<3)|0]|0;u=d[261841+(f<<3)|0]|0;f=d[n]|0;n=0;v=j;while(1){w=n|1;x=n|2;y=n|3;z=n|4;A=n|5;B=n|6;C=n|7;D=b[m+(C<<1)>>1]|0;E=b[i+(B<<1)>>1]|0;F=b[m+(B<<1)>>1]|0;B=b[i+(A<<1)>>1]|0;G=b[m+(A<<1)>>1]|0;A=b[i+(z<<1)>>1]|0;H=b[m+(z<<1)>>1]|0;z=b[i+(y<<1)>>1]|0;I=b[m+(y<<1)>>1]|0;y=b[i+(x<<1)>>1]|0;J=b[m+(x<<1)>>1]|0;x=b[i+(w<<1)>>1]|0;K=b[m+(w<<1)>>1]|0;w=b[i+(n<<1)>>1]|0;L=b[m+(n<<1)>>1]|0;M=ac(b[i+(C<<1)>>1]|0,l)|0;C=ac(D,h)|0;D=ac(E,l)|0;E=ac(F,h)|0;F=ac(B,l)|0;B=ac(G,h)|0;G=ac(A,l)|0;A=ac(H,h)|0;H=ac(z,l)|0;z=ac(I,h)|0;I=ac(y,l)|0;y=ac(J,h)|0;J=ac(x,l)|0;x=ac(K,h)|0;K=ac(w,l)|0;a[v]=((((((((f+((ac(L,h)|0)+K>>19)|0)>237)<<1|(u+(x+J>>19)|0)>237)<<1|(t+(y+I>>19)|0)>237)<<1|(s+(z+H>>19)|0)>237)<<1|(r+(A+G>>19)|0)>237)<<1|(q+(B+F>>19)|0)>237)<<1|(p+(E+D>>19)|0)>237)<<1|(o+(C+M>>19)|0)>237;M=n+8|0;if((M|0)<(k|0)){n=M;v=v+1|0}else{break}}return}v=e+11200|0;if(g){g=0;e=0;n=0;o=j;while(1){j=ac(b[m+(n<<1)>>1]|0,h)|0;p=(ac(b[i+(n<<1)>>1]|0,l)|0)+j>>19;j=c[v>>2]|0;q=j+(n<<2)|0;r=n|1;s=n+2|0;t=((e*7|0)-248+(c[q>>2]|0)+((c[j+(r<<2)>>2]|0)*5|0)+((c[j+(s<<2)>>2]|0)*3|0)>>4)+p|0;c[q>>2]=e;q=(t|0)>127|0;p=t-(-q&220)|0;t=ac(b[m+(r<<1)>>1]|0,h)|0;j=(ac(b[i+(r<<1)>>1]|0,l)|0)+t>>19;t=c[v>>2]|0;u=t+(r<<2)|0;r=((c[u>>2]|0)-248+((c[t+(s<<2)>>2]|0)*5|0)+((c[t+(n+3<<2)>>2]|0)*3|0)+(p*7|0)>>4)+j|0;c[u>>2]=p;p=(r|0)>127|0;u=p|(q|g)<<1;q=r-(-p&220)|0;if((n&6|0)==6){a[o]=u&255;N=o+1|0}else{N=o}if((s|0)<(k|0)){g=u<<1;e=q;n=s;o=N}else{O=q;P=s;break}}}else{O=0;P=0}c[(c[v>>2]|0)+(P<<2)>>2]=O;return}function vw(e,f,g,h,i,j,k,l,m,n,o,p){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0;m=p&7;if((o|0)<=0){c[c[e+11200>>2]>>2]=0;return}p=(h|0)>0;l=e+2284|0;k=e+11200|0;e=0;j=0;i=0;q=n;while(1){if(p){n=i|1;r=262144;s=262144;t=0;do{u=c[g+(t<<2)>>2]|0;v=b[f+(t<<1)>>1]|0;s=(ac(v,b[u+(i<<1)>>1]|0)|0)+s|0;r=(ac(b[u+(n<<1)>>1]|0,v)|0)+r|0;t=t+1|0;}while((t|0)<(h|0));w=r>>19;x=s>>19}else{w=0;x=0}if(((x|w)&256|0)==0){y=x;z=w}else{if(x>>>0>255){A=-x>>31&255}else{A=x&255}if(w>>>0>255){B=-w>>31&255}else{B=w&255}y=A&255;z=B&255}if((c[l>>2]&8388608|0)==0){t=i&6;C=(((d[261840+(m<<3)+t|0]|0)+y|0)>237|j<<1)<<1|((d[(t|1)+(261840+(m<<3))|0]|0)+z|0)>237;D=e;E=t}else{t=c[k>>2]|0;n=t+(i<<2)|0;v=i|1;u=i+2|0;F=((e*7|0)-248+(c[n>>2]|0)+((c[t+(v<<2)>>2]|0)*5|0)+((c[t+(u<<2)>>2]|0)*3|0)>>4)+y|0;c[n>>2]=e;n=(F|0)>127|0;t=F-(-n&220)|0;F=c[k>>2]|0;G=F+(v<<2)|0;v=((c[G>>2]|0)-248+((c[F+(u<<2)>>2]|0)*5|0)+((c[F+(i+3<<2)>>2]|0)*3|0)+(t*7|0)>>4)+z|0;c[G>>2]=t;t=(v|0)>127|0;C=t|(n|j<<1)<<1;D=v-(-t&220)|0;E=i&6}if((E|0)==6){a[q]=C&255;H=q+1|0}else{H=q}I=i+2|0;if((I|0)<(o|0)){e=D;j=C;i=I;q=H}else{break}}c[(c[k>>2]|0)+(I<<2)>>2]=D;if((I&6|0)==0){return}a[H]=C&255;return}function vx(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;l=c[f>>2]|0;h=c[g>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)>0){m=0}else{return}do{d=m<<1;n=(b[e+(d<<1)>>1]|0)+64>>7;o=(b[e+((d|1)<<1)>>1]|0)+64>>7;d=(b[l+(m<<1)>>1]|0)+64>>7;p=(b[h+(m<<1)>>1]|0)+64>>7;q=n>>>0>255;r=n&255;do{if(((o|n|d|p)&256|0)==0){if(q){s=-n>>31&255}else{s=r}if(o>>>0>255){t=-o>>31&255}else{t=o&255}if(d>>>0>255){u=-d>>31&255}else{u=d&255}if(p>>>0<=255){v=p&255;w=t;x=s;y=u;break}v=-p>>31&255;w=t;x=s;y=u}else{if(q){z=-n>>31&255}else{z=r}if(o>>>0>255){A=-o>>31&255}else{A=o&255}if(d>>>0>255){B=-d>>31&255}else{B=d&255}if(p>>>0<=255){v=p&255;w=A;x=z;y=B;break}v=-p>>31&255;w=A;x=z;y=B}}while(0);p=m<<2;a[i+p|0]=x;a[i+(p|1)|0]=y;a[i+(p|2)|0]=w;a[i+(p|3)|0]=v;m=m+1|0;}while((m|0)<(k|0));return}else{k=c[f+4>>2]|0;f=c[g+4>>2]|0;g=j+1>>1;if((g|0)>0){C=0}else{return}do{j=C<<1;m=(b[e+(j<<1)>>1]|0)+64>>7;v=(b[e+((j|1)<<1)>>1]|0)+64>>7;j=(b[l+(C<<1)>>1]|0)+128+(b[k+(C<<1)>>1]|0)>>8;w=(b[h+(C<<1)>>1]|0)+128+(b[f+(C<<1)>>1]|0)>>8;y=m>>>0>255;x=m&255;do{if(((v|m|j|w)&256|0)==0){if(y){D=-m>>31&255}else{D=x}if(v>>>0>255){E=-v>>31&255}else{E=v&255}if(j>>>0>255){F=-j>>31&255}else{F=j&255}if(w>>>0<=255){G=w&255;H=E;I=D;J=F;break}G=-w>>31&255;H=E;I=D;J=F}else{if(y){K=-m>>31&255}else{K=x}if(v>>>0>255){L=-v>>31&255}else{L=v&255}if(j>>>0>255){M=-j>>31&255}else{M=j&255}if(w>>>0<=255){G=w&255;H=L;I=K;J=M;break}G=-w>>31&255;H=L;I=K;J=M}}while(0);w=C<<2;a[i+w|0]=I;a[i+(w|1)|0]=J;a[i+(w|2)|0]=H;a[i+(w|3)|0]=G;C=C+1|0;}while((C|0)<(g|0));return}}function vy(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;d=c[f+4>>2]|0;f=c[g>>2]|0;n=c[g+4>>2]|0;g=4096-k|0;o=4096-l|0;p=j+1>>1;if((p|0)>0){q=0}else{return}do{j=q<<1;r=ac(b[m+(j<<1)>>1]|0,g)|0;s=(ac(b[h+(j<<1)>>1]|0,k)|0)+r>>19;r=j|1;j=ac(b[m+(r<<1)>>1]|0,g)|0;t=(ac(b[h+(r<<1)>>1]|0,k)|0)+j>>19;j=ac(b[e+(q<<1)>>1]|0,o)|0;r=(ac(b[d+(q<<1)>>1]|0,l)|0)+j>>19;j=ac(b[f+(q<<1)>>1]|0,o)|0;u=(ac(b[n+(q<<1)>>1]|0,l)|0)+j>>19;j=u&255;v=r&255;w=t&255;x=s&255;do{if(((t|s|r|u)&256|0)==0){y=x;z=w;A=v;B=j}else{if(s>>>0>255){C=-s>>31&255}else{C=x}if(t>>>0>255){D=-t>>31&255}else{D=w}if(r>>>0>255){E=-r>>31&255}else{E=v}if(u>>>0<=255){y=C;z=D;A=E;B=j;break}y=C;z=D;A=E;B=-u>>31&255}}while(0);u=q<<2;a[i+u|0]=y;a[i+(u|1)|0]=A;a[i+(u|2)|0]=z;a[i+(u|3)|0]=B;q=q+1|0;}while((q|0)<(p|0));return}function vz(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;o=n+1>>1;if((o|0)<=0){return}n=(g|0)>0;l=(k|0)>0;d=0;do{if(n){p=d<<1;q=p|1;r=262144;s=262144;t=0;do{u=c[f+(t<<2)>>2]|0;v=b[e+(t<<1)>>1]|0;s=(ac(v,b[u+(p<<1)>>1]|0)|0)+s|0;r=(ac(b[u+(q<<1)>>1]|0,v)|0)+r|0;t=t+1|0;}while((t|0)<(g|0));w=r>>19;x=s>>19}else{w=0;x=0}if(l){t=262144;q=262144;p=0;do{v=b[h+(p<<1)>>1]|0;q=(ac(v,b[(c[i+(p<<2)>>2]|0)+(d<<1)>>1]|0)|0)+q|0;t=(ac(b[(c[j+(p<<2)>>2]|0)+(d<<1)>>1]|0,v)|0)+t|0;p=p+1|0;}while((p|0)<(k|0));y=t>>19;z=q>>19}else{y=0;z=0}p=y&255;s=z&255;r=w&255;v=x&255;do{if(((x|w|y|z)&256|0)==0){A=v;B=r;C=s;D=p}else{if(x>>>0>255){E=-x>>31&255}else{E=v}if(w>>>0>255){F=-w>>31&255}else{F=r}if(z>>>0>255){G=-z>>31&255}else{G=s}if(y>>>0<=255){A=E;B=F;C=G;D=p;break}A=E;B=F;C=G;D=-y>>31&255}}while(0);p=d<<2;a[m+p|0]=A;a[m+(p|1)|0]=C;a[m+(p|2)|0]=B;a[m+(p|3)|0]=D;d=d+1|0;}while((d|0)<(o|0));return}function vA(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;l=c[f>>2]|0;h=c[g>>2]|0;if((k|0)<2048){k=j+1>>1;if((k|0)>0){m=0}else{return}do{d=m<<1;n=(b[e+(d<<1)>>1]|0)+64>>7;o=(b[e+((d|1)<<1)>>1]|0)+64>>7;d=(b[l+(m<<1)>>1]|0)+64>>7;p=(b[h+(m<<1)>>1]|0)+64>>7;q=n>>>0>255;r=n&255;do{if(((o|n|d|p)&256|0)==0){if(q){s=-n>>31&255}else{s=r}if(o>>>0>255){t=-o>>31&255}else{t=o&255}if(d>>>0>255){u=-d>>31&255}else{u=d&255}if(p>>>0<=255){v=p&255;w=t;x=s;y=u;break}v=-p>>31&255;w=t;x=s;y=u}else{if(q){z=-n>>31&255}else{z=r}if(o>>>0>255){A=-o>>31&255}else{A=o&255}if(d>>>0>255){B=-d>>31&255}else{B=d&255}if(p>>>0<=255){v=p&255;w=A;x=z;y=B;break}v=-p>>31&255;w=A;x=z;y=B}}while(0);p=m<<2;a[i+p|0]=y;a[i+(p|1)|0]=x;a[i+(p|2)|0]=v;a[i+(p|3)|0]=w;m=m+1|0;}while((m|0)<(k|0));return}else{k=c[f+4>>2]|0;f=c[g+4>>2]|0;g=j+1>>1;if((g|0)>0){C=0}else{return}do{j=C<<1;m=(b[e+(j<<1)>>1]|0)+64>>7;w=(b[e+((j|1)<<1)>>1]|0)+64>>7;j=(b[l+(C<<1)>>1]|0)+128+(b[k+(C<<1)>>1]|0)>>8;v=(b[h+(C<<1)>>1]|0)+128+(b[f+(C<<1)>>1]|0)>>8;x=m>>>0>255;y=m&255;do{if(((w|m|j|v)&256|0)==0){if(x){D=-m>>31&255}else{D=y}if(w>>>0>255){E=-w>>31&255}else{E=w&255}if(j>>>0>255){F=-j>>31&255}else{F=j&255}if(v>>>0<=255){G=v&255;H=E;I=D;J=F;break}G=-v>>31&255;H=E;I=D;J=F}else{if(x){K=-m>>31&255}else{K=y}if(w>>>0>255){L=-w>>31&255}else{L=w&255}if(j>>>0>255){M=-j>>31&255}else{M=j&255}if(v>>>0<=255){G=v&255;H=L;I=K;J=M;break}G=-v>>31&255;H=L;I=K;J=M}}while(0);v=C<<2;a[i+v|0]=J;a[i+(v|1)|0]=I;a[i+(v|2)|0]=G;a[i+(v|3)|0]=H;C=C+1|0;}while((C|0)<(g|0));return}}function vB(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;d=c[f+4>>2]|0;f=c[g>>2]|0;n=c[g+4>>2]|0;g=4096-k|0;o=4096-l|0;p=j+1>>1;if((p|0)>0){q=0}else{return}do{j=q<<1;r=ac(b[m+(j<<1)>>1]|0,g)|0;s=(ac(b[h+(j<<1)>>1]|0,k)|0)+r>>19;r=j|1;j=ac(b[m+(r<<1)>>1]|0,g)|0;t=(ac(b[h+(r<<1)>>1]|0,k)|0)+j>>19;j=ac(b[e+(q<<1)>>1]|0,o)|0;r=(ac(b[d+(q<<1)>>1]|0,l)|0)+j>>19;j=ac(b[f+(q<<1)>>1]|0,o)|0;u=(ac(b[n+(q<<1)>>1]|0,l)|0)+j>>19;j=u&255;v=r&255;w=t&255;x=s&255;do{if(((t|s|r|u)&256|0)==0){y=x;z=w;A=v;B=j}else{if(s>>>0>255){C=-s>>31&255}else{C=x}if(t>>>0>255){D=-t>>31&255}else{D=w}if(r>>>0>255){E=-r>>31&255}else{E=v}if(u>>>0<=255){y=C;z=D;A=E;B=j;break}y=C;z=D;A=E;B=-u>>31&255}}while(0);u=q<<2;a[i+u|0]=A;a[i+(u|1)|0]=y;a[i+(u|2)|0]=B;a[i+(u|3)|0]=z;q=q+1|0;}while((q|0)<(p|0));return}function vC(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;o=n+1>>1;if((o|0)<=0){return}n=(g|0)>0;l=(k|0)>0;d=0;do{if(n){p=d<<1;q=p|1;r=262144;s=262144;t=0;do{u=c[f+(t<<2)>>2]|0;v=b[e+(t<<1)>>1]|0;s=(ac(v,b[u+(p<<1)>>1]|0)|0)+s|0;r=(ac(b[u+(q<<1)>>1]|0,v)|0)+r|0;t=t+1|0;}while((t|0)<(g|0));w=r>>19;x=s>>19}else{w=0;x=0}if(l){t=262144;q=262144;p=0;do{v=b[h+(p<<1)>>1]|0;q=(ac(v,b[(c[i+(p<<2)>>2]|0)+(d<<1)>>1]|0)|0)+q|0;t=(ac(b[(c[j+(p<<2)>>2]|0)+(d<<1)>>1]|0,v)|0)+t|0;p=p+1|0;}while((p|0)<(k|0));y=t>>19;z=q>>19}else{y=0;z=0}p=y&255;s=z&255;r=w&255;v=x&255;do{if(((x|w|y|z)&256|0)==0){A=v;B=r;C=s;D=p}else{if(x>>>0>255){E=-x>>31&255}else{E=v}if(w>>>0>255){F=-w>>31&255}else{F=r}if(z>>>0>255){G=-z>>31&255}else{G=s}if(y>>>0<=255){A=E;B=F;C=G;D=p;break}A=E;B=F;C=G;D=-y>>31&255}}while(0);p=d<<2;a[m+p|0]=C;a[m+(p|1)|0]=A;a[m+(p|2)|0]=D;a[m+(p|3)|0]=B;d=d+1|0;}while((d|0)<(o|0));return}function vD(b,e,f,g,h,i,j,k,l,m){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;n=c[m>>2]|0;o=c[m+4>>2]|0;p=c[m+8>>2]|0;q=c[m+12>>2]|0;r=c[m+16>>2]|0;s=c[m+20>>2]|0;t=c[m+24>>2]|0;u=c[m+28>>2]|0;v=c[m+32>>2]|0;m=h>>1;if((i|0)<=0){return}h=j<<1;w=l<<1;x=l+2|0;y=l+3|0;z=l+4|0;A=l+5|0;if((m|0)>0){B=0;C=g;D=f;E=e;F=b}else{b=0;while(1){if((b|1|0)==(i|0)){G=10008;break}e=b+2|0;if((e|0)<(i|0)){b=e}else{G=10009;break}}if((G|0)==10008){return}else if((G|0)==10009){return}}while(1){b=0;do{e=b*6|0;f=d[F+e|0]|0;g=d[F+(e|1)|0]|0;H=d[F+(e+2)|0]|0;I=ac(H,n)|0;J=ac(g,o)|0;K=((J+(ac(f,p)|0)+I|0)>>>15)+16|0;I=ac(H,t)|0;J=ac(g,u)|0;L=((J+(ac(f,v)|0)+I|0)>>>15)+128|0;I=ac(H,q)|0;H=ac(g,r)|0;a[D+b|0]=((H+(ac(f,s)|0)+I|0)>>>15)+128&255;a[C+b|0]=L&255;L=b<<1;a[E+L|0]=K&255;K=d[F+(e+3)|0]|0;I=d[F+(e+4)|0]|0;f=ac(d[F+(e+5)|0]|0,n)|0;e=ac(I,o)|0;a[E+(L|1)|0]=((e+(ac(K,p)|0)+f|0)>>>15)+16&255;b=b+1|0;}while((b|0)<(m|0));if((B|1|0)==(i|0)){G=10010;break}else{M=0}do{b=M*6|0;f=d[F+(b+l)|0]|0;K=d[F+((b|1)+l)|0]|0;e=ac(d[F+(x+b)|0]|0,n)|0;L=ac(K,o)|0;K=M<<1;a[E+(K+j)|0]=((L+(ac(f,p)|0)+e|0)>>>15)+16&255;e=d[F+(y+b)|0]|0;f=d[F+(z+b)|0]|0;L=ac(d[F+(A+b)|0]|0,n)|0;b=ac(f,o)|0;a[E+((K|1)+j)|0]=((b+(ac(e,p)|0)+L|0)>>>15)+16&255;M=M+1|0;}while((M|0)<(m|0));L=B+2|0;if((L|0)<(i|0)){B=L;C=C+k|0;D=D+k|0;E=E+h|0;F=F+w|0}else{G=10012;break}}if((G|0)==10012){return}else if((G|0)==10010){return}}function vE(){c[540998]=60;c[540994]=100;c[540996]=52;c[540988]=72;c[540990]=76;c[540992]=74;c[540980]=68;c[540982]=44;c[540976]=50;c[540972]=92;c[540974]=70;c[540966]=98;c[540986]=56;c[540984]=82;c[540978]=108;c[540566]=104;c[540968]=84;c[540970]=16;c[498214]=42;c[498216]=80;c[498224]=26;c[498226]=12;c[498222]=38;c[549648]=8;c[642300]=50;c[551804]=2;c[499380]=44;c[498212]=54;c[499520]=48;c[499518]=20;c[498220]=40;c[498218]=60;return}function vF(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0;e=d>>2;if((e|0)>0){f=0}else{return}do{d=f<<2;g=f*3|0;a[c+g|0]=a[b+(d|2)|0]|0;a[c+(g+1)|0]=a[b+(d|1)|0]|0;a[c+(g+2)|0]=a[b+d|0]|0;f=f+1|0;}while((f|0)<(e|0));return}function vG(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0;if((d|0)>0){e=0;f=0}else{return}do{g=e<<2;a[c+g|0]=a[b+(f+2)|0]|0;a[c+(g|1)|0]=a[b+(f+1)|0]|0;a[c+(g|2)|0]=a[b+f|0]|0;a[c+(g|3)|0]=-1;e=e+1|0;f=e*3|0;}while((f|0)<(d|0));return}function vH(b,c,d){b=b|0;c=c|0;d=d|0;var f=0,g=0,h=0;f=b;b=f+(((d|0)/2|0)<<1)|0;if(f>>>0<b>>>0){g=f;h=c}else{return}while(1){c=g+2|0;f=e[g>>1]|0;a[h]=(f>>>8&248|f>>>13)&255;d=f&2016;a[h+1|0]=(d>>>3|d>>>9)&255;a[h+2|0]=(f>>>2&7|f<<3)&255;a[h+3|0]=-1;if(c>>>0<b>>>0){g=c;h=h+4|0}else{break}}return}function vI(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0,g=0;e=a;a=e+(((d|0)/2|0)<<1)|0;if(e>>>0>=a>>>0){return}d=e;e=c;while(1){c=d+2|0;f=b[d>>1]|0;g=f&15;b[e>>1]=f<<3&30720|(f&65535)>>>1&1024|(f&65535)>>>2&32|f<<2&960|(g&65535)>>>3|g<<1;if(c>>>0<a>>>0){d=c;e=e+2|0}else{break}}return}function vJ(b,c,d){b=b|0;c=c|0;d=d|0;var f=0,g=0,h=0;f=b;b=f+(((d|0)/2|0)<<1)|0;if(f>>>0<b>>>0){g=f;h=c}else{return}while(1){c=g+2|0;f=e[g>>1]|0;a[h]=(f>>>8&248|f>>>13)&255;d=f&2016;a[h+1|0]=(d>>>3|d>>>9)&255;a[h+2|0]=(f>>>2&7|f<<3)&255;if(c>>>0<b>>>0){g=c;h=h+3|0}else{break}}return}function vK(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0;e=d>>1;if((e|0)<=0){return}d=a;a=c;c=0;do{f=b[d+(c<<1)>>1]|0;b[a+(c<<1)>>1]=(f&65535)>>>11|f&2016|f<<11;c=c+1|0;}while((c|0)<(e|0));return}function vL(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0;e=d>>1;if((e|0)<=0){return}d=a;a=c;c=0;do{f=b[d+(c<<1)>>1]|0;b[a+(c<<1)>>1]=(f&65535)>>>1&992|(f&65535)>>>11|f<<10&31744;c=c+1|0;}while((c|0)<(e|0));return}function vM(b,c,d){b=b|0;c=c|0;d=d|0;var f=0,g=0,h=0;f=b;b=f+(((d|0)/2|0)<<1)|0;if(f>>>0<b>>>0){g=f;h=c}else{return}while(1){c=g+2|0;f=e[g>>1]|0;d=f&31744;a[h]=(d>>>7|d>>>12)&255;d=f&992;a[h+1|0]=(d>>>2|d>>>7)&255;a[h+2|0]=(f>>>2&7|f<<3)&255;a[h+3|0]=-1;if(c>>>0<b>>>0){g=c;h=h+4|0}else{break}}return}function vN(b,c,d){b=b|0;c=c|0;d=d|0;var f=0,g=0,h=0;f=b;b=f+(((d|0)/2|0)<<1)|0;if(f>>>0<b>>>0){g=f;h=c}else{return}while(1){c=g+2|0;f=e[g>>1]|0;d=f&31744;a[h]=(d>>>7|d>>>12)&255;d=f&992;a[h+1|0]=(d>>>2|d>>>7)&255;a[h+2|0]=(f>>>2&7|f<<3)&255;if(c>>>0<b>>>0){g=c;h=h+3|0}else{break}}return}function vO(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0;e=d>>1;if((e|0)<=0){return}d=a;a=c;c=0;do{f=b[d+(c<<1)>>1]|0;b[a+(c<<1)>>1]=f<<1&1984|f<<11|(f&65535)>>>10&31;c=c+1|0;}while((c|0)<(e|0));return}function vP(a,c,d){a=a|0;c=c|0;d=d|0;var f=0,g=0,h=0;f=d>>1;if((f|0)<=0){return}d=a;a=c;c=0;do{g=e[d+(c<<1)>>1]|0;h=g&31775;b[a+(c<<1)>>1]=(h>>>10|g&992|h<<10)&65535;c=c+1|0;}while((c|0)<(f|0));return}function vQ(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0;e=c;c=a;a=d>>1;if((a|0)>0){f=0}else{return}do{d=b[c+(f<<1)>>1]|0;b[e+(f<<1)>>1]=(d&65535)>>>8|d&240|d<<8&3840;f=f+1|0;}while((f|0)<(a|0));return}function vR(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0;if((d|0)>0){e=0}else{return}do{a[c+e|0]=a[b+e|0]|0;f=e|3;g=e|1;a[c+g|0]=a[b+f|0]|0;h=e|2;a[c+h|0]=a[b+h|0]|0;a[c+f|0]=a[b+g|0]|0;e=e+4|0;}while((e|0)<(d|0));return}function vS(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0;if((d|0)>0){e=0}else{return}do{f=e|1;a[c+e|0]=a[b+f|0]|0;g=e|2;a[c+f|0]=a[b+g|0]|0;f=e|3;a[c+g|0]=a[b+f|0]|0;a[c+f|0]=a[b+e|0]|0;e=e+4|0;}while((e|0)<(d|0));return}function vT(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0;if((d|0)>0){e=0}else{return}do{f=e|3;a[c+e|0]=a[b+f|0]|0;g=e|1;a[c+g|0]=a[b+e|0]|0;h=e|2;a[c+h|0]=a[b+g|0]|0;a[c+f|0]=a[b+h|0]|0;e=e+4|0;}while((e|0)<(d|0));return}function vU(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0;if((d|0)>0){e=0}else{return}do{f=e|3;a[c+e|0]=a[b+f|0]|0;g=e|2;h=e|1;a[c+h|0]=a[b+g|0]|0;a[c+g|0]=a[b+h|0]|0;a[c+f|0]=a[b+e|0]|0;e=e+4|0;}while((e|0)<(d|0));return}function vV(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0,g=0;e=c;c=a;a=d>>1;if((a|0)>0){f=0}else{return}do{d=f+2|0;b[e+(f<<1)>>1]=b[c+(d<<1)>>1]|0;g=f+1|0;b[e+(g<<1)>>1]=b[c+(g<<1)>>1]|0;b[e+(d<<1)>>1]=b[c+(f<<1)>>1]|0;f=f+3|0;}while((f|0)<(a|0));return}function vW(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0,g=0;e=c;c=a;a=d>>1;if((a|0)>0){f=0}else{return}do{d=f+2|0;b[e+(f<<1)>>1]=cT(b[c+(d<<1)>>1]|0)|0;g=f+1|0;b[e+(g<<1)>>1]=cT(b[c+(g<<1)>>1]|0)|0;b[e+(d<<1)>>1]=cT(b[c+(f<<1)>>1]|0)|0;f=f+3|0;}while((f|0)<(a|0));return}function vX(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0,g=0;e=c;c=a;a=d>>3;if((a|0)>0){f=0}else{return}do{d=f<<2;g=f*3|0;b[e+(g<<1)>>1]=b[c+((d|2)<<1)>>1]|0;b[e+(g+1<<1)>>1]=b[c+((d|1)<<1)>>1]|0;b[e+(g+2<<1)>>1]=b[c+(d<<1)>>1]|0;f=f+1|0;}while((f|0)<(a|0));return}function vY(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0,g=0;e=c;c=a;a=d>>3;if((a|0)>0){f=0}else{return}do{d=f<<2;g=f*3|0;b[e+(g<<1)>>1]=cT(b[c+((d|2)<<1)>>1]|0)|0;b[e+(g+1<<1)>>1]=cT(b[c+((d|1)<<1)>>1]|0)|0;b[e+(g+2<<1)>>1]=cT(b[c+(d<<1)>>1]|0)|0;f=f+1|0;}while((f|0)<(a|0));return}function vZ(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0,g=0;e=c;c=a;a=d>>3;if((a|0)>0){f=0}else{return}do{d=f<<2;g=f*3|0;b[e+(g<<1)>>1]=b[c+(d<<1)>>1]|0;b[e+(g+1<<1)>>1]=b[c+((d|1)<<1)>>1]|0;b[e+(g+2<<1)>>1]=b[c+((d|2)<<1)>>1]|0;f=f+1|0;}while((f|0)<(a|0));return}function v_(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0,g=0;e=c;c=a;a=d>>3;if((a|0)>0){f=0}else{return}do{d=f<<2;g=f*3|0;b[e+(g<<1)>>1]=cT(b[c+(d<<1)>>1]|0)|0;b[e+(g+1<<1)>>1]=cT(b[c+((d|1)<<1)>>1]|0)|0;b[e+(g+2<<1)>>1]=cT(b[c+((d|2)<<1)>>1]|0)|0;f=f+1|0;}while((f|0)<(a|0));return}function v$(a,d,e){a=a|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0;f=a+e|0;g=e-3|0;e=a+g|0;if((g|0)>0){g=a;h=d;while(1){i=c[g>>2]|0;c[h>>2]=(i&2147450879)+(i&2145419232);i=h+4|0;j=g+4|0;if(j>>>0<e>>>0){g=j;h=i}else{k=j;l=i;break}}}else{k=a;l=d}if(k>>>0>=f>>>0){return}f=b[k>>1]|0;b[l>>1]=(f&32767)+(f&32736)&65535;return}function v0(b,c,d){b=b|0;c=c|0;d=d|0;var f=0,g=0,h=0;f=b;b=f+(((d|0)/2|0)<<1)|0;if(f>>>0<b>>>0){g=f;h=c}else{return}while(1){c=g+2|0;f=e[g>>1]|0;a[h]=(f>>>2&7|f<<3)&255;d=f&992;a[h+1|0]=(d>>>2|d>>>7)&255;d=f&31744;a[h+2|0]=(d>>>7|d>>>12)&255;if(c>>>0<b>>>0){g=c;h=h+3|0}else{break}}return}function v1(b,c,d){b=b|0;c=c|0;d=d|0;var f=0,g=0,h=0;f=b;b=f+(((d|0)/2|0)<<1)|0;if(f>>>0<b>>>0){g=f;h=c}else{return}while(1){c=g+2|0;f=e[g>>1]|0;a[h]=(f>>>2&7|f<<3)&255;d=f&992;a[h+1|0]=(d>>>2|d>>>7)&255;d=f&31744;a[h+2|0]=(d>>>7|d>>>12)&255;a[h+3|0]=-1;if(c>>>0<b>>>0){g=c;h=h+4|0}else{break}}return}function v2(b,c,d){b=b|0;c=c|0;d=d|0;var f=0,g=0,h=0;f=b;b=f+(((d|0)/2|0)<<1)|0;if(f>>>0<b>>>0){g=f;h=c}else{return}while(1){c=g+2|0;f=e[g>>1]|0;a[h]=(f>>>2&7|f<<3)&255;d=f&2016;a[h+1|0]=(d>>>3|d>>>9)&255;a[h+2|0]=(f>>>8&248|f>>>13)&255;if(c>>>0<b>>>0){g=c;h=h+3|0}else{break}}return}function v3(b,c,d){b=b|0;c=c|0;d=d|0;var f=0,g=0,h=0;f=b;b=f+(((d|0)/2|0)<<1)|0;if(f>>>0<b>>>0){g=f;h=c}else{return}while(1){c=g+2|0;f=e[g>>1]|0;a[h]=(f>>>2&7|f<<3)&255;d=f&2016;a[h+1|0]=(d>>>3|d>>>9)&255;a[h+2|0]=(f>>>8&248|f>>>13)&255;a[h+3|0]=-1;if(c>>>0<b>>>0){g=c;h=h+4|0}else{break}}return}function v4(a,d,e){a=a|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0;f=a+e|0;g=e-3|0;e=a+g|0;if((g|0)>0){g=a;h=d;while(1){i=c[g>>2]|0;c[h>>2]=i>>>1&2145419232|i&2031647;i=g+4|0;j=h+4|0;if(i>>>0<e>>>0){g=i;h=j}else{k=i;l=j;break}}}else{k=a;l=d}if(k>>>0>=f>>>0){return}f=b[k>>1]|0;b[l>>1]=(f&65535)>>>1&32736|f&31;return}function v5(c,e,f){c=c|0;e=e|0;f=f|0;var g=0;g=c+f|0;if((f|0)<=0){return}f=c;c=e;while(1){e=f+3|0;b[c>>1]=(d[f+1|0]|0)<<3&2016|(a[f]&255)>>>3|(d[f+2|0]|0)<<8&-2048;if(e>>>0<g>>>0){f=e;c=c+2|0}else{break}}return}function v6(c,e,f){c=c|0;e=e|0;f=f|0;var g=0;g=c+f|0;if((f|0)<=0){return}f=c;c=e;while(1){e=f+3|0;b[c>>1]=(d[f+1|0]|0)<<2&992|(a[f]&255)>>>3|(d[f+2|0]|0)<<7&31744;if(e>>>0<g>>>0){f=e;c=c+2|0}else{break}}return}function v7(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0;e=b+d|0;if((d|0)>0){f=b;g=c}else{return}while(1){a[g]=a[f]|0;a[g+1|0]=a[f+1|0]|0;c=f+3|0;a[g+2|0]=a[f+2|0]|0;a[g+3|0]=-1;if(c>>>0<e>>>0){f=c;g=g+4|0}else{break}}return}function v8(a,d,e){a=a|0;d=d|0;e=e|0;var f=0,g=0;f=a+e|0;if((e|0)<=0){return}e=a;a=d;while(1){d=c[e>>2]|0;g=e+4|0;b[a>>1]=(d>>>3&31|d>>>5&2016|d>>>8&63488)&65535;if(g>>>0<f>>>0){e=g;a=a+2|0}else{break}}return}function v9(a,d,e){a=a|0;d=d|0;e=e|0;var f=0,g=0;f=a+e|0;if((e|0)<=0){return}e=a;a=d;while(1){d=c[e>>2]|0;g=e+4|0;b[a>>1]=(d>>>3&31|d>>>6&992|d>>>9&31744)&65535;if(g>>>0<f>>>0){e=g;a=a+2|0}else{break}}return}function wa(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0;e=b+d|0;if((d|0)>0){f=b;g=c}else{return}while(1){a[g]=a[f]|0;a[g+1|0]=a[f+1|0]|0;a[g+2|0]=a[f+2|0]|0;c=f+4|0;if(c>>>0<e>>>0){f=c;g=g+3|0}else{break}}return}function wb(c,e,f){c=c|0;e=e|0;f=f|0;var g=0;g=c+f|0;if((f|0)<=0){return}f=c;c=e;while(1){e=f+3|0;b[c>>1]=(d[f+1|0]|0)<<2&992|(d[f]|0)<<7&31744|(a[f+2|0]&255)>>>3;if(e>>>0<g>>>0){f=e;c=c+2|0}else{break}}return}function wc(c,e,f){c=c|0;e=e|0;f=f|0;var g=0;g=c+f|0;if((f|0)<=0){return}f=c;c=e;while(1){e=f+3|0;b[c>>1]=(d[f+1|0]|0)<<3&2016|(d[f]|0)<<8&-2048|(a[f+2|0]&255)>>>3;if(e>>>0<g>>>0){f=e;c=c+2|0}else{break}}return}function wd(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0;if((d|0)==0){return}else{e=0}do{f=e+2|0;g=a[b+f|0]|0;h=e+1|0;a[c+h|0]=a[b+h|0]|0;a[c+f|0]=a[b+e|0]|0;a[c+e|0]=g;e=e+3|0;}while(e>>>0<d>>>0);return}function we(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=15-d|0;if((d|0)>0){f=e}else{return}do{d=f-e|0;g=c[a+d>>2]|0;h=g&16711935;c[b+d>>2]=h>>>16|g&-16711936|h<<16;f=f+4|0;}while((f|0)<15);return}function wf(a,d,e){a=a|0;d=d|0;e=e|0;var f=0,g=0;f=a+e|0;if((e|0)<=0){return}e=a;a=d;while(1){d=c[e>>2]|0;g=e+4|0;b[a>>1]=(d<<8&63488|d>>>5&2016|d>>>19&31)&65535;if(g>>>0<f>>>0){e=g;a=a+2|0}else{break}}return}function wg(a,d,e){a=a|0;d=d|0;e=e|0;var f=0,g=0;f=a+e|0;if((e|0)<=0){return}e=a;a=d;while(1){d=c[e>>2]|0;g=e+4|0;b[a>>1]=(d<<7&31744|d>>>6&992|d>>>19&31)&65535;if(g>>>0<f>>>0){e=g;a=a+2|0}else{break}}return}function wh(a,b,e,f,g,h,i,j,k){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;l=g>>1;if((h|0)<=0){return}g=(l|0)>0;m=f;f=e;e=b;b=0;n=a;while(1){if(g){a=0;o=m;p=n;q=e;r=f;while(1){c[o>>2]=(d[q]|0)<<8|(d[p]|0)|(d[p+1|0]|0)<<16|(d[r]|0)<<24;s=a+1|0;if((s|0)<(l|0)){a=s;o=o+4|0;p=p+2|0;q=q+1|0;r=r+1|0}else{break}}}if((b&1|0)==0){t=e;u=f}else{t=e+j|0;u=f+j|0}r=b+1|0;if((r|0)<(h|0)){m=m+k|0;f=u;e=t;b=r;n=n+i|0}else{break}}return}function wi(a,b,e,f,g,h,i,j,k){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;l=g>>1;if((h|0)<=0){return}g=(l|0)>0;m=f;f=e;e=b;b=0;n=a;while(1){if(g){a=0;o=m;p=n;q=e;r=f;while(1){c[o>>2]=(d[p]|0)<<8|(d[q]|0)|(d[r]|0)<<16|(d[p+1|0]|0)<<24;s=a+1|0;if((s|0)<(l|0)){a=s;o=o+4|0;p=p+2|0;q=q+1|0;r=r+1|0}else{break}}}if((b&1|0)==0){t=e;u=f}else{t=e+j|0;u=f+j|0}r=b+1|0;if((r|0)<(h|0)){m=m+k|0;f=u;e=t;b=r;n=n+i|0}else{break}}return}function wj(a,b,e,f,g,h,i,j,k){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;l=g>>1;if((h|0)<=0){return}g=(l|0)>0;m=f;f=e;e=b;b=0;n=a;while(1){if(g){a=0;o=m;p=n;q=e;r=f;while(1){c[o>>2]=(d[q]|0)<<8|(d[p]|0)|(d[p+1|0]|0)<<16|(d[r]|0)<<24;s=a+1|0;if((s|0)<(l|0)){a=s;o=o+4|0;p=p+2|0;q=q+1|0;r=r+1|0}else{break}}}r=b+1|0;if((r|0)<(h|0)){m=m+k|0;f=f+j|0;e=e+j|0;b=r;n=n+i|0}else{break}}return}function wk(a,b,e,f,g,h,i,j,k){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;l=g>>1;if((h|0)<=0){return}g=(l|0)>0;m=f;f=e;e=b;b=0;n=a;while(1){if(g){a=0;o=m;p=n;q=e;r=f;while(1){c[o>>2]=(d[p]|0)<<8|(d[q]|0)|(d[r]|0)<<16|(d[p+1|0]|0)<<24;s=a+1|0;if((s|0)<(l|0)){a=s;o=o+4|0;p=p+2|0;q=q+1|0;r=r+1|0}else{break}}}r=b+1|0;if((r|0)<(h|0)){m=m+k|0;f=f+j|0;e=e+j|0;b=r;n=n+i|0}else{break}}return}function wl(b,c,d,e,f,g,h,i,j){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;k=f>>1;if((g|0)<=0){return}f=(k|0)>0;l=h<<1;m=j<<1;n=e;e=d;d=c;c=0;o=b;while(1){do{if(f){b=0;do{p=b<<2;q=b<<1;a[d+q|0]=a[o+p|0]|0;a[e+b|0]=a[o+(p|1)|0]|0;a[d+(q|1)|0]=a[o+(p|2)|0]|0;a[n+b|0]=a[o+(p|3)|0]|0;b=b+1|0;}while((b|0)<(k|0));if(f){r=0}else{break}do{b=r<<2;p=r<<1;a[d+(p+h)|0]=a[o+(b+j)|0]|0;a[d+((p|1)+h)|0]=a[o+((b|2)+j)|0]|0;r=r+1|0;}while((r|0)<(k|0))}}while(0);b=c+2|0;if((b|0)<(g|0)){n=n+i|0;e=e+i|0;d=d+l|0;c=b;o=o+m|0}else{break}}return}function wm(b,c,e,f,g,h){b=b|0;c=c|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0;a[c]=a[b]|0;i=e-1|0;j=(i|0)>0;if(j){k=0;while(1){l=b+k|0;m=k+1|0;n=b+m|0;o=k<<1;a[c+(o|1)|0]=(((d[l]|0)*3|0)+(d[n]|0)|0)>>>2&255;a[c+(o+2)|0]=(((d[n]|0)*3|0)+(d[l]|0)|0)>>>2&255;if((m|0)<(i|0)){k=m}else{break}}}k=(e<<1)-1|0;a[c+k|0]=a[b+i|0]|0;e=c+h|0;m=a[b]|0;if((f|0)>1){l=i+g|0;n=k+h|0;o=h<<1;p=ac((f<<1)-1|0,h)|0;q=ac(f-1|0,g)|0;r=e;s=1;t=b;u=m;while(1){v=t+g|0;a[r]=((d[v]|0)+((u&255)*3|0)|0)>>>2&255;a[r+h|0]=(((d[v]|0)*3|0)+(d[t]|0)|0)>>>2&255;if(j){w=0;do{x=t+w|0;y=w+g|0;z=t+(y+1)|0;A=w<<1;a[r+(A|1)|0]=(((d[x]|0)*3|0)+(d[z]|0)|0)>>>2&255;B=A+h|0;a[r+(B+2)|0]=(((d[z]|0)*3|0)+(d[x]|0)|0)>>>2&255;w=w+1|0;x=t+w|0;z=t+y|0;a[r+(B+1)|0]=(((d[z]|0)*3|0)+(d[x]|0)|0)>>>2&255;a[r+(A+2)|0]=(((d[x]|0)*3|0)+(d[z]|0)|0)>>>2&255;}while((w|0)<(i|0))}w=t+i|0;z=t+l|0;a[r+k|0]=(((d[w]|0)*3|0)+(d[z]|0)|0)>>>2&255;a[r+n|0]=(((d[z]|0)*3|0)+(d[w]|0)|0)>>>2&255;w=s+1|0;C=a[v]|0;if((w|0)<(f|0)){r=r+o|0;s=w;t=v;u=C}else{break}}D=c+p|0;E=b+q|0;F=C}else{D=e;E=b;F=m}a[D]=F;if(j){G=0}else{H=E+i|0;I=a[H]|0;J=D+k|0;a[J]=I;return}while(1){j=E+G|0;F=G+1|0;m=E+F|0;b=G<<1;a[D+(b|1)|0]=(((d[j]|0)*3|0)+(d[m]|0)|0)>>>2&255;a[D+(b+2)|0]=(((d[m]|0)*3|0)+(d[j]|0)|0)>>>2&255;if((F|0)<(i|0)){G=F}else{break}}H=E+i|0;I=a[H]|0;J=D+k|0;a[J]=I;return}function wn(b,c,d,e,f,g,h,i){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0;if((f|0)<=0){return}j=(e|0)>0;k=d;d=c;c=b;b=0;while(1){if(j){l=0;do{m=l<<1;a[k+m|0]=a[c+l|0]|0;a[k+(m|1)|0]=a[d+l|0]|0;l=l+1|0;}while((l|0)<(e|0))}l=b+1|0;if((l|0)<(f|0)){k=k+i|0;d=d+h|0;c=c+g|0;b=l}else{break}}return}function wo(b,c,d,e,f,g,h,i,j,k){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;l=(f|0)/2|0;m=(g|0)/2|0;n=(g|0)>1;if(!n){return}g=(f|0)>1;o=0;do{p=ac(o>>1,h)|0;q=ac(o,j)|0;if(g){r=0;do{s=a[b+(r+p)|0]|0;t=r<<1;a[d+((t|1)+q)|0]=s;a[d+(t+q)|0]=s;r=r+1|0;}while((r|0)<(l|0))}o=o+1|0;}while((o|0)<(m|0));if(!n){return}n=(f|0)>1;f=0;do{o=ac(f>>1,i)|0;d=ac(f,k)|0;if(n){b=0;do{g=a[c+(b+o)|0]|0;j=b<<1;a[e+((j|1)+d)|0]=g;a[e+(j+d)|0]=g;b=b+1|0;}while((b|0)<(l|0))}f=f+1|0;}while((f|0)<(m|0));return}function wp(b,c,d,e,f,g,h,i,j,k){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;l=(f|0)/2|0;if((g|0)<=0){return}m=(f|0)>1;f=0;do{n=ac(f,h)|0;o=f>>2;p=ac(o,i)|0;q=ac(o,j)|0;o=ac(f,k)|0;if(m){r=0;do{s=r<<2;t=r<<3;a[e+(t+o)|0]=a[b+(s+n)|0]|0;u=c+(r+p)|0;a[e+((t|1)+o)|0]=a[u]|0;a[e+((t|2)+o)|0]=a[b+((s|1)+n)|0]|0;v=d+(r+q)|0;a[e+((t|3)+o)|0]=a[v]|0;a[e+((t|4)+o)|0]=a[b+((s|2)+n)|0]|0;a[e+((t|5)+o)|0]=a[u]|0;a[e+((t|6)+o)|0]=a[b+((s|3)+n)|0]|0;a[e+((t|7)+o)|0]=a[v]|0;r=r+1|0;}while((r|0)<(l|0))}f=f+1|0;}while((f|0)<(g|0));return}function wq(b,c,e,f,g,h,i,j,k){b=b|0;c=c|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;l=g+1>>1;if((h|0)<=0){return}m=(g|0)>0;n=-g|0;o=l<<2;p=(l|0)>0;q=-l|0;r=f;f=e;e=c;c=0;s=b;while(1){if(m){b=n;do{t=b+g|0;a[s+t|0]=a[r+(t<<1|1)|0]|0;b=b+1|0;}while((b|0)<0)}if((c&1|0)==0){u=e;v=f}else{if(p){b=q;do{t=b<<2;w=t+o|0;x=b+l|0;a[e+x|0]=((d[r+w|0]|0)+(d[r+(w-k)|0]|0)|0)>>>1&255;w=(t|2)+o|0;a[f+x|0]=((d[r+w|0]|0)+(d[r+(w-k)|0]|0)|0)>>>1&255;b=b+1|0;}while((b|0)<0)}u=e+j|0;v=f+j|0}b=c+1|0;if((b|0)<(h|0)){r=r+k|0;f=v;e=u;c=b;s=s+i|0}else{break}}return}function wr(b,c,d,e,f,g,h,i,j){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;k=f+1>>1;if((g|0)<=0){return}l=(f|0)>0;m=-f|0;n=k<<2;o=(k|0)>0;p=-k|0;q=e;e=d;d=c;c=0;r=b;while(1){if(l){b=m;do{s=b+f|0;a[r+s|0]=a[q+(s<<1|1)|0]|0;b=b+1|0;}while((b|0)<0)}if(o){b=p;do{s=b<<2;t=b+k|0;a[d+t|0]=a[q+(s+n)|0]|0;a[e+t|0]=a[q+((s|2)+n)|0]|0;b=b+1|0;}while((b|0)<0)}b=c+1|0;if((b|0)<(g|0)){q=q+j|0;e=e+i|0;d=d+i|0;c=b;r=r+h|0}else{break}}return}function ws(b,c,e,f,g,h,i,j,k){b=b|0;c=c|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;l=g+1>>1;if((h|0)<=0){return}m=(g|0)>0;n=-g|0;o=l<<2|1;p=(l|0)>0;q=-l|0;r=f;f=e;e=c;c=0;s=b;while(1){if(m){b=n;do{t=b+g|0;a[s+t|0]=a[r+(t<<1)|0]|0;b=b+1|0;}while((b|0)<0)}if((c&1|0)==0){u=e;v=f}else{if(p){b=q;do{t=b<<2;w=t+o|0;x=b+l|0;a[e+x|0]=((d[r+w|0]|0)+(d[r+(w-k)|0]|0)|0)>>>1&255;w=(t|2)+o|0;a[f+x|0]=((d[r+w|0]|0)+(d[r+(w-k)|0]|0)|0)>>>1&255;b=b+1|0;}while((b|0)<0)}u=e+j|0;v=f+j|0}b=c+1|0;if((b|0)<(h|0)){r=r+k|0;f=v;e=u;c=b;s=s+i|0}else{break}}return}function wt(b,c,d,e,f,g,h,i,j){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;k=f+1>>1;if((g|0)<=0){return}l=(f|0)>0;m=-f|0;n=k<<2|1;o=(k|0)>0;p=-k|0;q=e;e=d;d=c;c=0;r=b;while(1){if(l){b=m;do{s=b+f|0;a[r+s|0]=a[q+(s<<1)|0]|0;b=b+1|0;}while((b|0)<0)}if(o){b=p;do{s=b<<2;t=b+k|0;a[d+t|0]=a[q+(s+n)|0]|0;a[e+t|0]=a[q+((s|2)+n)|0]|0;b=b+1|0;}while((b|0)<0)}b=c+1|0;if((b|0)<(g|0)){q=q+j|0;e=e+i|0;d=d+i|0;c=b;r=r+h|0}else{break}}return}function wu(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0;d=i;e=c[b+56>>2]|0;ts(b,b+23844|0,b+23848|0,b+23852|0,b+23856|0,b+23860|0,b+23864|0,b+23868|0);xa(b);f=b+68|0;g=c[f>>2]|0;do{if((c[b+72>>2]|0)==8){h=b+23908|0;if((g|0)>=15){c[h>>2]=128;c[b+23904>>2]=128;break}c[h>>2]=142;c[b+23904>>2]=142;if((c[b+2284>>2]&1|0)==0){break}c[b+23896>>2]=54;c[b+23900>>2]=4}else{h=(g|0)>14?140:20;c[b+23908>>2]=h;c[b+23904>>2]=h}}while(0);g=b+11260|0;L14201:do{if((c[g>>2]|0)!=(c[b+11264>>2]|0)){h=c[b+52>>2]|0;switch(h|0){case 41:case 42:case 291:case 292:case 30:case 29:case 2:case 43:case 44:case 45:case 46:case 63:case 62:case 22:case 23:case 24:case 10:case 294:case 293:case 68:case 67:case 65:case 64:case 50:case 49:case 48:case 47:case 28:case 27:case 21:case 20:case 19:case 3:case 9:{break L14201;break};default:{}}j=BP(h)|0;if((j|0)==0){AN(0,0,1656352,(h=i,i=i+24|0,c[h>>2]=1637704,c[h+8>>2]=1629536,c[h+16>>2]=656,h)|0);i=h;cw();return 0}if((a[j+7|0]&32)!=0){break}j=(c[g>>2]|0)!=0;h=b+23912|0;if((c[f>>2]|0)<15){if(j){c[h>>2]=64;c[b+23916>>2]=54;break}else{c[h>>2]=66;c[b+23916>>2]=32;break}}else{if(j){c[h>>2]=20;c[b+23916>>2]=58;break}else{c[h>>2]=62;c[b+23916>>2]=106;break}}}}while(0);if((e|0)==66|(e|0)==32|(e|0)==31|(e|0)==8){i=d;return 46}f=c[b+52>>2]|0;if((f|0)==8|(f|0)==66|(f|0)==31|(f|0)==32){i=d;return 46}if((e-9|0)>>>0<2){i=d;return 46}c[b+23920>>2]=1;i=d;return 46}function wv(f,g,h,j,k,l,m){f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0,cb=0,cc=0,cd=0,ce=0,cf=0,cg=0,ch=0,ci=0,cj=0,ck=0,cl=0,cm=0,cn=0,co=0,cp=0;n=i;i=i+104|0;o=n|0;p=n+8|0;q=n+16|0;r=n+24|0;s=n+32|0;t=n+40|0;u=n+48|0;v=n+56|0;w=n+72|0;x=n+88|0;z=c[f+8>>2]|0;A=c[f+19592>>2]|0;B=c[f+16>>2]|0;C=c[f+28>>2]|0;D=c[f+20>>2]|0;E=c[f+36>>2]|0;F=c[f+40>>2]|0;G=c[f+52>>2]|0;H=c[f+2284>>2]|0;I=c[f+2236>>2]|0;J=c[f+2240>>2]|0;K=c[f+2228>>2]|0;L=c[f+2232>>2]|0;M=c[f+2212>>2]|0;N=c[f+2216>>2]|0;O=f+11400|0;P=f+15496|0;Q=c[f+2252>>2]|0;R=c[f+2256>>2]|0;S=c[f+2244>>2]|0;T=c[f+2248>>2]|0;U=c[f+2168>>2]|0;V=c[f+2172>>2]|0;W=c[f+2176>>2]|0;X=c[f+2180>>2]|0;Y=c[f+2184>>2]|0;Z=c[f+2188>>2]|0;_=c[f+2208>>2]|0;$=f+120|0;c[o>>2]=c[f+23844>>2];c[p>>2]=c[f+23848>>2];c[q>>2]=c[f+23852>>2];aa=f+23856|0;c[r>>2]=c[aa>>2];ab=f+23860|0;c[s>>2]=c[ab>>2];c[t>>2]=c[f+23864>>2];c[u>>2]=c[f+23868>>2];ad=f+80|0;ae=c[ad>>2]|0;af=j>>ae;ag=-k>>ae;ae=-ag|0;ah=f+56|0;ai=BP(c[ah>>2]|0)|0;if((ai|0)==0){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=626,aj)|0);i=aj;cw();return 0}do{if((((e[ai+8>>1]|0)>>>11&15)-8&65535)<6){ak=1}else{al=BP(c[ah>>2]|0)|0;if((al|0)==0){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=619,aj)|0);i=aj;cw();return 0}else{ak=(b[al+8>>1]&30720)==30720;break}}}while(0);ai=f+2280|0;al=c[ai>>2]|0;am=f+2200|0;an=c[am>>2]|0;ao=f+2204|0;ap=c[ao>>2]|0;aq=f+2192|0;ar=c[aq>>2]|0;as=f+2196|0;at=c[as>>2]|0;au=BP(c[ah>>2]|0)|0;if((au|0)==0){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=794,aj)|0);i=aj;cw();return 0}av=d[au+7|0]|0;if((av&2|0)==0){aw=(av&64|0)!=0}else{aw=1}av=f+10496|0;au=aw?$:av;switch(c[ah>>2]|0){case 11:case 1:case 17:case 66:case 41:case 42:case 291:case 292:case 30:case 29:case 2:case 43:case 44:case 45:case 46:case 63:case 62:case 22:case 23:case 24:case 10:case 294:case 293:case 68:case 67:case 65:case 64:case 50:case 49:case 48:case 47:case 28:case 27:case 21:case 20:case 19:case 3:case 9:{ah=c[g>>2]|0;c[g+12>>2]=ah;c[g+8>>2]=ah;c[g+4>>2]=ah;ah=c[h>>2]|0;c[h+12>>2]=ah;c[h+8>>2]=ah;c[h+4>>2]=ah;ax=ah;ay=ah;break};default:{ax=c[h+4>>2]|0;ay=c[h+8>>2]|0}}ah=f+92|0;$=h+4|0;c[$>>2]=ax<<c[ah>>2];ax=h+8|0;c[ax>>2]=ay<<c[ah>>2];do{if((c[m>>2]&15|0)==0){if((c[m+4>>2]&15|0)!=0){az=10412;break}if((c[m+8>>2]&15|0)!=0){az=10412;break}if((c[m+12>>2]&15|0)!=0){az=10412}}else{az=10412}}while(0);do{if((az|0)==10412){if((H&4096|0)==0|a[71664]){break}AN(f,24,1623296,(aj=i,i=i+1|0,i=i+7&-8,c[aj>>2]=0,aj)|0);i=aj;a[71664]=1}}while(0);do{if((c[l>>2]&15|0)==0){if((c[l+4>>2]&15|0)!=0){az=10428;break}if((c[l+8>>2]&15|0)!=0){az=10428;break}if((c[g>>2]&15|0)!=0){az=10428;break}if((c[g+4>>2]&15|0)!=0){az=10428;break}if((c[g+8>>2]&15|0)!=0){az=10428;break}if((c[m>>2]&15|0)!=0){az=10428;break}if((c[m+4>>2]&15|0)!=0){az=10428;break}if((c[m+8>>2]&15|0)!=0){az=10428;break}if((c[m+12>>2]&15|0)!=0){az=10428;break}if((c[h>>2]&15|0)!=0){az=10428;break}if((c[$>>2]&15|0)!=0){az=10428;break}if((c[ax>>2]&15|0)!=0){az=10428;break}if((c[h+12>>2]&15|0)!=0){az=10428}}else{az=10428}}while(0);if((az|0)==10428){zz()|0}H=(j|0)==0;ah=H?-1:at;at=H?-1:ar;ar=H?-1:ap;ap=H?-1:an;an=H?0:al;if(!ak){c[f+23804>>2]=237952;c[f+23800>>2]=237952}L14276:do{if((an|0)<(B|0)){al=f+88|0;H=v|0;ay=v+4|0;aw=l+4|0;aA=m+4|0;aB=v+8|0;aC=l+8|0;aD=m+8|0;aE=v+12|0;aF=(X|0)!=0;aG=l+12|0;aH=m+12|0;aI=f+23808|0;aJ=1-Q|0;aK=B-1|0;aL=1-R|0;aM=f+12|0;aN=f+24|0;aO=1-Y|0;aP=1-Z|0;aQ=k+j|0;aR=-aQ|0;aS=af+~ag|0;aT=f+23800|0;aU=f+23804|0;aV=B-2|0;aW=f+2220|0;aX=f+2224|0;aY=(Q|0)==1;aZ=(R|0)<3;a_=(R|0)==1;a$=(Q|0)==2;a0=(R|0)==2;a1=f+11412|0;a2=f+11408|0;a3=f+15508|0;a4=f+15504|0;a5=(G|0)==11;a6=O;a7=P;a8=f+23744|0;a9=f+19640|0;ba=x|0;bb=x+4|0;bc=g+4|0;bd=x+8|0;be=g+8|0;bf=x+12|0;bg=g+12|0;bh=h+12|0;bi=Z<<1;bj=f+23920|0;bk=f+23880|0;bl=f+23888|0;bm=_+((D<<1)+93&-16)|0;bn=f+23900|0;bo=f+23908|0;bp=f+23916|0;bq=w|0;br=w+4|0;bs=w+8|0;bt=w+12|0;bu=Y<<1;bv=f+23872|0;bw=f+23912|0;bx=f+23884|0;by=f+23896|0;bz=f+23904|0;bA=f+23876|0;bB=f+23892|0;bC=ah;bD=at;bE=ar;bF=ap;bG=an;L14278:while(1){bH=c[al>>2]|0;bI=bG>>bH;bJ=c[l>>2]|0;c[H>>2]=bJ+(ac(c[m>>2]|0,bG)|0);bJ=c[aw>>2]|0;c[ay>>2]=bJ+(ac(c[aA>>2]|0,bI)|0);bJ=c[aC>>2]|0;c[aB>>2]=bJ+(ac(c[aD>>2]|0,bI)|0);if(aF){bJ=c[aG>>2]|0;bK=bJ+(ac(c[aH>>2]|0,bG)|0)|0}else{bK=0}c[aE>>2]=bK;bJ=c[aI>>2]|0;bL=c[I+(bG<<2)>>2]|0;bM=(aJ|0)>(bL|0)?aJ:bL;bL=(1<<bH)-1|bG;bH=c[I+(((bL|0)>(aK|0)?aK:bL)<<2)>>2]|0;bL=c[J+(bI<<2)>>2]|0;bN=(aL|0)>(bL|0)?aL:bL;bL=c[aM>>2]|0;bO=bM+Q|0;bP=(bL|0)>(bO|0)?bO:bL;bO=((aJ|0)>(bH|0)?aJ:bH)+Q|0;bH=c[aN>>2]|0;bQ=bN+R|0;bR=((bH|0)>(bQ|0)?bQ:bH)-1|0;bH=(bM|0)>(bD|0)?bM-1|0:bD;bQ=(bN|0)>(bC|0)?bN-1|0:bC;if((bM|0)<(aO+bH|0)){az=10436;break}if((bN|0)<(aP+bQ|0)){az=10438;break}if((((bL|0)>(bO|0)?bO:bL)-1|0)<(aQ|0)){if((bR|0)<(-(aR>>c[ad>>2])|0)){bS=bR;bT=bP;bU=1}else{az=10441}}else{az=10441}if((az|0)==10441){az=0;bS=aS;bT=aQ;bU=0}bP=bT-1|0;if((bH|0)<(bP|0)){bR=bH;bL=bF;while(1){bO=c[g>>2]|0;bV=bR+1|0;bW=bV-j|0;bX=bO+(ac(c[h>>2]|0,bW)|0)|0;c[bq>>2]=bX;bO=c[bc>>2]|0;bY=bO+(ac(c[$>>2]|0,bW)|0)|0;c[br>>2]=bY;bO=c[be>>2]|0;bZ=bO+(ac(c[ax>>2]|0,bW)|0)|0;c[bs>>2]=bZ;bO=c[bg>>2]|0;c[bt>>2]=bO+(ac(c[bh>>2]|0,bW)|0);bO=bL+1|0;if((bO|0)>=(bu|0)){az=10444;break L14278}if((bW|0)>=(k|0)){az=10446;break L14278}if((bW|0)<=-1){az=10448;break L14278}bW=c[U+(bO<<2)>>2]|0;b_=c[bv>>2]|0;b$=c[bw>>2]|0;do{if((b_|0)==0){b0=c[bx>>2]|0;if((b0|0)==0){b1=bX;break}dq[b0&127](_,bq,z,av);b1=_}else{c9[b_&127](_,bX,bY,bZ,z,au);b1=_}}while(0);bZ=c[by>>2]|0;if((bZ|0)==0){dc[c[bz>>2]&255](f,bW,A,b1,M,K,S)}else{c9[bZ&127](f,bW,A,b1,z,E)}if((b$|0)!=0){c$[b$&127](bW,A)}do{if(aF){bZ=c[X+(bO<<2)>>2]|0;bY=c[bA>>2]|0;bX=c[bt>>2]|0;do{if((bY|0)==0){b_=c[bB>>2]|0;if((b_|0)==0){b2=bX;break}dq[b_&127](_,bq,z,0);b2=_}else{c9[bY&127](_,bX,c[br>>2]|0,c[bs>>2]|0,z,au);b2=_}}while(0);bX=c[by>>2]|0;if((bX|0)==0){dc[c[bz>>2]&255](f,bZ,A,b2,M,K,S);break}else{c9[bX&127](f,bZ,A,b2,z,E);break}}}while(0);if((bV|0)<(bP|0)){bR=bV;bL=bO}else{b3=bV;b4=bO;break}}}else{b3=bH;b4=bF}if((bQ|0)<(bS|0)){bL=bQ;bR=bE;while(1){bP=c[g>>2]|0;bW=bL+1|0;b$=bW-af|0;bX=bP+(ac(c[h>>2]|0,b$)|0)|0;c[ba>>2]=bX;bP=c[bc>>2]|0;bY=bP+(ac(c[$>>2]|0,b$)|0)|0;c[bb>>2]=bY;bP=c[be>>2]|0;b_=bP+(ac(c[ax>>2]|0,b$)|0)|0;c[bd>>2]=b_;bP=c[bg>>2]|0;c[bf>>2]=bP+(ac(c[bh>>2]|0,b$)|0);bP=bR+1|0;if((bP|0)>=(bi|0)){az=10469;break L14278}if((b$|0)>=(ae|0)){az=10471;break L14278}if((b$|0)<=-1){az=10473;break L14278}do{if((c[bj>>2]|0)!=0){b$=c[V+(bP<<2)>>2]|0;b0=c[W+(bP<<2)>>2]|0;b5=c[bk>>2]|0;do{if((b5|0)==0){b6=c[bl>>2]|0;if((b6|0)==0){b7=bY;b8=b_;break}cX[b6&255](_,bm,ba,D,av);b7=_;b8=bm}else{dc[b5&255](_,bm,bX,bY,b_,D,au);b7=_;b8=bm}}while(0);b5=c[bn>>2]|0;if((b5|0)==0){dc[c[bo>>2]&255](f,b$,C,b7,N,L,T);dc[c[bo>>2]&255](f,b0,C,b8,N,L,T)}else{c8[b5&7](f,b$,b0,C,b7,b8,D,F)}b5=c[bp>>2]|0;if((b5|0)==0){break}dm[b5&127](b$,b0,C)}}while(0);if((bW|0)<(bS|0)){bL=bW;bR=bP}else{b9=bW;ca=bP;break}}}else{b9=bQ;ca=bE}bR=b4-((b4|0)<(Y|0)?0:Y)|0;bL=ca-((ca|0)<(Z|0)?0:Z)|0;if(!bU){cb=bR;cc=bL;cd=b3;ce=b9;cf=bG;break L14276}if(ak){c[aT>>2]=261912+((bI&7)<<3);c[aU>>2]=261912+((bG&7)<<3)}if((bG|0)<(aV|0)){cg=bJ}else{ts(f,o,p,q,r,s,t,u);cg=0}bH=bM+Y-b3+bR|0;b_=U+(bH<<2)|0;bY=bN+Z-b9+bL|0;bX=V+(bY<<2)|0;bO=W+(bY<<2)|0;if(aF){ch=X+(bH<<2)|0}else{ch=0}bH=c[aW>>2]|0;bY=c[aX>>2]|0;bV=BP(G)|0;if((bV|0)==0){az=10493;break}do{if((a[bV+7|0]&16)==0){az=10499}else{b5=BP(G)|0;if((b5|0)==0){az=10496;break L14278}if((a[b5+7|0]&32)!=0){az=10499;break}if((d[b5+4|0]|0)>1){az=10504}else{az=10499}}}while(0);L14355:do{if((az|0)==10499){az=0;do{if((G|0)==66|(G|0)==32|(G|0)==31|(G|0)==8){bV=BP(G)|0;if((bV|0)==0){az=10501;break L14278}if(a5){break}if((a[bV+7|0]|0)>=0){az=10504;break L14355}}}while(0);bP=c[t>>2]|0;if((bP|0)==0){bW=c[u>>2]|0;bV=bH+((ac(bG,Q)|0)<<1)|0;bN=bY+((ac(bG,R)|0)<<1)|0;dj[bW&127](f,bV,b_,Q,bN,bX,bO,R,ch,H,A,bG);break}if(!((c[aa>>2]|0)!=0&aY&aZ)){if((c[ab>>2]|0)!=0&a$&a0){bN=bG<<1;bV=bN|1;bW=b[bH+(bV<<1)>>1]|0;bM=b[bY+(bV<<1)>>1]|0;bV=(b[bH+(bN<<1)>>1]|0)*65537|0;c[a1>>2]=bV;c[a2>>2]=bV;bV=(b[bY+(bI<<1<<1)>>1]|0)*65537|0;c[a3>>2]=bV;c[a4>>2]=bV;df[c[s>>2]&127](f,b_,bX,bO,aF?ch:0,c[H>>2]|0,A,bW,bM,bG);break}else{bM=bH+((ac(bG,Q)|0)<<1)|0;bW=bY+((ac(bG,R)|0)<<1)|0;dj[bP&127](f,bM,b_,Q,bW,bX,bO,R,ch,c[H>>2]|0,A,bG);break}}if(a_){ci=0}else{ci=b[bY+((bG<<1|1)<<1)>>1]|0}if(aF){cj=c[ch>>2]|0}else{cj=0}de[c[r>>2]&127](f,c[b_>>2]|0,bX,bO,cj,c[H>>2]|0,A,ci,bG)}}while(0);do{if((az|0)==10504){az=0;bW=(1<<c[al>>2])-1|0;bM=bH+((ac(bG,Q)|0)<<1)|0;bP=(cg|0)!=0;bV=bP?a7:bY+((ac(bI,R)|0)<<1)|0;if(aY){cX[c[o>>2]&255](c[b_>>2]|0,c[H>>2]|0,A,c[aU>>2]|0,0)}else{dc[c[p>>2]&255](bP?a6:bM,Q,b_,c[H>>2]|0,A,c[aU>>2]|0,0)}do{if((bW&bG|0)==0){if((G|0)==66|(G|0)==32|(G|0)==31|(G|0)==8){break}bN=c[q>>2]|0;if((bN|0)!=0){dc[bN&255](f,bV,R,bX,bO,c[ay>>2]|0,C);break}if(a_){cX[c[o>>2]&255](c[bX>>2]|0,c[ay>>2]|0,C,c[aT>>2]|0,0);cX[c[o>>2]&255](c[bO>>2]|0,c[aB>>2]|0,C,c[aT>>2]|0,3);break}dc[c[p>>2]&255](bV,R,bX,c[ay>>2]|0,C,c[aT>>2]|0,0);if(bP){ck=c[a8>>2]>>1}else{ck=3}dc[c[p>>2]&255](bV,R,bO,c[aB>>2]|0,C,c[aT>>2]|0,ck)}}while(0);if(!aF){break}if(aY){cX[c[o>>2]&255](c[ch>>2]|0,c[aE>>2]|0,A,c[aU>>2]|0,0);break}else{dc[c[p>>2]&255](bP?a9:bM,Q,ch,c[aE>>2]|0,A,c[aU>>2]|0,0);break}}}while(0);bO=bG+1|0;if((bO|0)<(B|0)){bC=b9;bD=b3;bE=bL;bF=bR;bG=bO}else{cb=bR;cc=bL;cd=b3;ce=b9;cf=bO;break L14276}}if((az|0)==10493){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=649,aj)|0);i=aj;cw();return 0}else if((az|0)==10446){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1726136,c[aj+8>>2]=1739904,c[aj+16>>2]=513,aj)|0);i=aj;cw();return 0}else if((az|0)==10496){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=642,aj)|0);i=aj;cw();return 0}else if((az|0)==10448){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1721840,c[aj+8>>2]=1739904,c[aj+16>>2]=514,aj)|0);i=aj;cw();return 0}else if((az|0)==10473){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1711320,c[aj+8>>2]=1739904,c[aj+16>>2]=536,aj)|0);i=aj;cw();return 0}else if((az|0)==10444){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1730488,c[aj+8>>2]=1739904,c[aj+16>>2]=512,aj)|0);i=aj;cw();return 0}else if((az|0)==10469){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1717840,c[aj+8>>2]=1739904,c[aj+16>>2]=534,aj)|0);i=aj;cw();return 0}else if((az|0)==10471){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1713888,c[aj+8>>2]=1739904,c[aj+16>>2]=535,aj)|0);i=aj;cw();return 0}else if((az|0)==10438){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1735296,c[aj+8>>2]=1739904,c[aj+16>>2]=484,aj)|0);i=aj;cw();return 0}else if((az|0)==10436){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1612824,c[aj+8>>2]=1739904,c[aj+16>>2]=483,aj)|0);i=aj;cw();return 0}else if((az|0)==10501){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=744,aj)|0);i=aj;cw();return 0}}else{cb=ap;cc=ar;cd=at;ce=ah;cf=an}}while(0);ah=BP(G)|0;if((ah|0)==0){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=772,aj)|0);i=aj;cw();return 0}if((d[ah+4|0]|0)<=1){c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}if((a[ah+7|0]&16)==0){c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}ah=BP(G)|0;if((ah|0)==0){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=744,aj)|0);i=aj;cw();return 0}do{if((G|0)==11){if((X|0)==0){break}c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}else{if((a[ah+7|0]|0)<0&(X|0)==0){break}c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}}while(0);X=cf-an|0;ah=BP(G)|0;if((ah|0)==0){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=619,aj)|0);i=aj;cw();return 0}do{if((b[ah+8>>1]&30720)!=30720){at=BP(G)|0;if((at|0)==0){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=626,aj)|0);i=aj;cw();return 0}if((((e[at+8>>1]|0)>>>11&15)-8&65535)<6){break}at=c[m+12>>2]|0;if((X|0)<=0){c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}ar=ac(at,an)|0;ap=(c[l+12>>2]|0)+ar|0;ar=0;while(1){NM(ap|0,-1|0,A|0);az=ar+1|0;if((az|0)<(X|0)){ap=ap+at|0;ar=az}else{break}}c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}}while(0);ah=BP(G)|0;ar=c[l+12>>2]|0;l=c[m+12>>2]|0;m=b[ah+14>>1]|0;ah=BP(G)|0;if((ah|0)==0){AN(0,0,1656352,(aj=i,i=i+24|0,c[aj>>2]=1637704,c[aj+8>>2]=1629536,c[aj+16>>2]=635,aj)|0);i=aj;cw();return 0}aj=ar+(ac(l,an)|0)|0;ar=65535>>>(((m&65535)>>>11&15^15)>>>0);if((X|0)<=0){c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}m=(A|0)>0;G=ar&65535;at=(G>>>8|G<<8)&65535;G=ar&65535;if((a[ah+7|0]&1)==0){if(m){cm=aj;cn=0}else{c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}while(1){ah=0;do{ar=cm+(ah<<1)|0;y=G;a[ar]=y&255;y=y>>8;a[ar+1|0]=y&255;ah=ah+1|0;}while((ah|0)<(A|0));ah=cn+1|0;if((ah|0)<(X|0)){cm=cm+l|0;cn=ah}else{break}}c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}else{if(m){co=aj;cp=0}else{c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}while(1){aj=0;do{m=co+(aj<<1)|0;y=at;a[m]=y&255;y=y>>8;a[m+1|0]=y&255;aj=aj+1|0;}while((aj|0)<(A|0));aj=cp+1|0;if((aj|0)<(X|0)){co=co+l|0;cp=aj}else{break}}c[ai>>2]=cf;c[am>>2]=cb;c[ao>>2]=cc;c[aq>>2]=cd;c[as>>2]=ce;cl=cf-an|0;i=n;return cl|0}return 0}function ww(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0;if((f|0)<=0){return}a=(j|0)>0;k=0;do{l=c[i+(k<<2)>>2]|0;if(a){m=ac(k,j)|0;n=0;o=0;do{o=(ac(b[h+(n+m<<1)>>1]|0,d[g+(n+l)|0]|0)|0)+o|0;n=n+1|0;}while((n|0)<(j|0));p=o>>7}else{p=0}b[e+(k<<1)>>1]=(p|0)>32767?32767:p&65535;k=k+1|0;}while((k|0)<(f|0));return}function wx(a,c,e,f,g,h){a=a|0;c=c|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0;if((e|0)>0){a=0;i=0;while(1){j=a>>>16;k=d[f+j|0]|0;b[c+(i<<1)>>1]=(ac((d[f+(j+1)|0]|0)-k|0,a>>>9&127)|0)+(k<<7)&65535;k=i+1|0;if((k|0)<(e|0)){a=a+h|0;i=k}else{break}}}i=e-1|0;e=g-1|0;if(((ac(i,h)|0)>>16|0)<(e|0)){return}g=f+e|0;f=i;do{b[c+(f<<1)>>1]=(d[g]|0)<<7;f=f-1|0;}while(((ac(f,h)|0)>>16|0)>=(e|0));return}function wy(a,c,e,f,g,h,i,j){a=a|0;c=c|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0;if((f|0)>0){a=0;k=0;while(1){l=a>>>16;m=a>>>9&127;n=m^127;o=ac(d[g+l|0]|0,n)|0;p=l+1|0;b[c+(k<<1)>>1]=(ac(d[g+p|0]|0,m)|0)+o&65535;o=ac(d[h+l|0]|0,n)|0;b[e+(k<<1)>>1]=(ac(d[h+p|0]|0,m)|0)+o&65535;o=k+1|0;if((o|0)<(f|0)){a=a+j|0;k=o}else{break}}}k=f-1|0;f=i-1|0;if(((ac(k,j)|0)>>16|0)<(f|0)){return}i=g+f|0;g=h+f|0;h=k;do{b[c+(h<<1)>>1]=(d[i]|0)<<7;b[e+(h<<1)>>1]=(d[g]|0)<<7;h=h-1|0;}while(((ac(h,j)|0)>>16|0)>=(f|0));return}function wz(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0;a=e;if((f|0)<=0){return}e=(j|0)>0;k=0;do{l=c[i+(k<<2)>>2]|0;if(e){m=ac(k,j)|0;n=0;o=0;do{o=(ac(b[h+(n+m<<1)>>1]|0,d[g+(n+l)|0]|0)|0)+o|0;n=n+1|0;}while((n|0)<(j|0));p=o>>3}else{p=0}c[a+(k<<2)>>2]=(p|0)>524287?524287:p;k=k+1|0;}while((k|0)<(f|0));return}function wA(d,f,g,h,j,k,l){d=d|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;m=i;n=d+56|0;d=f;f=h;h=(BP(c[n>>2]|0)|0)+8|0;o=((e[h>>1]|0)>>>11&15)-4|0;p=c[n>>2]|0;L14522:do{switch(p|0){case 41:case 42:case 291:case 292:case 30:case 29:case 2:case 43:case 44:case 45:case 46:case 63:case 62:case 22:case 23:case 24:case 10:case 294:case 293:case 68:case 67:case 65:case 64:case 50:case 49:case 48:case 47:case 28:case 27:case 21:case 20:case 19:case 3:case 9:{q=10615;break};default:{r=BP(p)|0;if((r|0)==0){AN(0,0,1656352,(s=i,i=i+24|0,c[s>>2]=1637704,c[s+8>>2]=1629536,c[s+16>>2]=656,s)|0);i=s;cw()}if((a[r+7|0]&32)!=0){q=10615;break L14522}if((c[n>>2]|0)==11){q=10615}else{t=o}}}}while(0);if((q|0)==10615){t=(b[h>>1]&30720)==30720?o:9}if((g|0)<=0){i=m;return}o=(l|0)>0;h=0;do{q=c[k+(h<<2)>>2]|0;if(o){n=ac(h,l)|0;p=0;r=0;while(1){s=(ac(b[j+(p+n<<1)>>1]|0,e[f+(p+q<<1)>>1]|0)|0)+r|0;u=p+1|0;if((u|0)<(l|0)){p=u;r=s}else{v=s;break}}}else{v=0}r=v>>t;c[d+(h<<2)>>2]=(r|0)>524287?524287:r;h=h+1|0;}while((h|0)<(g|0));i=m;return}function wB(d,f,g,h,j,k,l){d=d|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;m=i;n=d+56|0;d=h;h=(BP(c[n>>2]|0)|0)+8|0;o=(e[h>>1]|0)>>>11&15;L14543:do{if(o<<16>>16==15){p=o&65535}else{q=c[n>>2]|0;switch(q|0){case 41:case 42:case 291:case 292:case 30:case 29:case 2:case 43:case 44:case 45:case 46:case 63:case 62:case 22:case 23:case 24:case 10:case 294:case 293:case 68:case 67:case 65:case 64:case 50:case 49:case 48:case 47:case 28:case 27:case 21:case 20:case 19:case 3:case 9:{p=13;break L14543;break};default:{}}r=BP(q)|0;if((r|0)==0){AN(0,0,1656352,(q=i,i=i+24|0,c[q>>2]=1637704,c[q+8>>2]=1629536,c[q+16>>2]=656,q)|0);i=q;cw()}if((a[r+7|0]&32)!=0){p=13;break}if((c[n>>2]|0)==11){p=13;break}p=(e[h>>1]|0)>>>11&15}}while(0);if((g|0)<=0){i=m;return}h=(l|0)>0;n=0;do{o=c[k+(n<<2)>>2]|0;if(h){r=ac(n,l)|0;q=0;s=0;while(1){t=(ac(b[j+(q+r<<1)>>1]|0,e[d+(q+o<<1)>>1]|0)|0)+s|0;u=q+1|0;if((u|0)<(l|0)){q=u;s=t}else{v=t;break}}}else{v=0}s=v>>p;b[f+(n<<1)>>1]=(s|0)>32767?32767:s&65535;n=n+1|0;}while((n|0)<(g|0));i=m;return}function wC(a,c){a=a|0;c=c|0;var d=0,e=0;if((c|0)>0){d=0}else{return}do{e=a+(d<<1)|0;b[e>>1]=(((b[e>>1]|0)*14071|0)+33561947|0)>>>14&65535;d=d+1|0;}while((d|0)<(c|0));return}function wD(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0;if((d|0)>0){e=0}else{return}do{f=a+(e<<1)|0;b[f>>1]=(((b[f>>1]|0)*1799|0)+4081085|0)>>>11&65535;f=c+(e<<1)|0;b[f>>1]=(((b[f>>1]|0)*1799|0)+4081085|0)>>>11&65535;e=e+1|0;}while((e|0)<(d|0));return}function wE(a,c){a=a|0;c=c|0;var d=0,e=0,f=0,g=0;if((c|0)>0){d=0}else{return}do{e=a+(d<<1)|0;f=b[e>>1]|0;if(f<<16>>16>30189){g=32767}else{g=(((f<<16>>16)*19077|0)+1034684463|0)>>>14&65535}b[e>>1]=g;d=d+1|0;}while((d|0)<(c|0));return}function wF(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;if((d|0)>0){e=0}else{return}do{f=a+(e<<1)|0;g=b[f>>1]|0;if(g<<16>>16>30775){h=32767}else{h=(((g<<16>>16)*4663|0)+259145464|0)>>>12&65535}b[f>>1]=h;f=c+(e<<1)|0;g=b[f>>1]|0;if(g<<16>>16>30775){i=32767}else{i=(((g<<16>>16)*4663|0)+259145464|0)>>>12&65535}b[f>>1]=i;e=e+1|0;}while((e|0)<(d|0));return}function wG(a,b){a=a|0;b=b|0;var d=0,e=0;d=a;if((b|0)>0){e=0}else{return}do{a=d+(e<<2)|0;c[a>>2]=((c[a>>2]|0)*3517|0)+134247788>>12;e=e+1|0;}while((e|0)<(b|0));return}function wH(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=a;a=b;if((d|0)>0){f=0}else{return}do{b=e+(f<<2)|0;c[b>>2]=((c[b>>2]|0)*1799|0)+65297360>>11;b=a+(f<<2)|0;c[b>>2]=((c[b>>2]|0)*1799|0)+65297360>>11;f=f+1|0;}while((f|0)<(d|0));return}function wI(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=a;if((b|0)>0){e=0}else{return}do{a=d+(e<<2)|0;f=c[a>>2]|0;if((f|0)>483024){g=524246}else{g=(f*4769|0)-156229444>>12}c[a>>2]=g;e=e+1|0;}while((e|0)<(b|0));return}function wJ(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;e=a;a=b;if((d|0)>0){f=0}else{return}do{b=e+(f<<2)|0;g=c[b>>2]|0;if((g|0)>492400){h=524272}else{h=(g*4663|0)-148639872>>12}c[b>>2]=h;b=a+(f<<2)|0;g=c[b>>2]|0;if((g|0)>492400){i=524272}else{i=(g*4663|0)-148639872>>12}c[b>>2]=i;f=f+1|0;}while((f|0)<(d|0));return}function wK(e){e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0;f=i;g=e+56|0;h=c[g>>2]|0;j=c[e+52>>2]|0;k=e+2284|0;l=c[k>>2]|0;m=c[e+16>>2]|0;n=(j-41|0)>>>0<2;o=(j|0)==291;p=(j|0)==292;q=(j|0)==2;r=(j|0)==43;s=(j|0)==44;t=(j|0)==45;u=(j|0)==46;v=(j|0)==63;w=(j|0)==62;x=(j|0)==10;y=(j|0)==9;z=(j|0)==67;A=(j|0)==68;B=(j|0)==293;C=(j|0)==294;D=(j|0)==28;E=(j|0)==27;F=(j|0)==3;G=(j|0)==47;H=(j|0)==48;I=(j|0)==49;J=(j|0)==50;K=(j|0)==65;L=(j|0)==64;M=n|o|p|(j|0)==30|(j|0)==29|q|r|s|t|u|v|w|(j|0)==22|(j|0)==23|(j|0)==24|x|y|z|A|B|C|D|E|F|G|H|I|J|K|L|(j|0)==19|(j|0)==20|(j|0)==21|x|y;do{if(M){N=10697}else{y=BP(j)|0;if((y|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=656,O)|0);i=O;cw()}else{if((a[y+7|0]&32)==0){P=0;break}else{N=10697;break}}}}while(0);do{if((N|0)==10697){y=c[e+60>>2]|0;if((y|0)>=24){P=0;break}if((y|0)<(c[e+64>>2]|0)){P=1;break}y=(h|0)==10;x=(h|0)==9;if((h-41|0)>>>0<2|(h|0)==291|(h|0)==292|(h|0)==30|(h|0)==29|(h|0)==2|(h|0)==43|(h|0)==44|(h|0)==45|(h|0)==46|(h|0)==63|(h|0)==62|(h|0)==22|(h|0)==23|(h|0)==24|y|x|(h|0)==67|(h|0)==68|(h|0)==293|(h|0)==294|(h|0)==28|(h|0)==27|(h|0)==3|(h|0)==47|(h|0)==48|(h|0)==49|(h|0)==50|(h|0)==65|(h|0)==64|(h|0)==19|(h|0)==20|(h|0)==21|y|x){P=0;break}x=BP(h)|0;if((x|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=656,O)|0);i=O;cw()}else{P=(a[x+7|0]&32)==0;break}}}while(0);x=(h|0)==0;y=(h|0)==35;do{if((h|0)==35|(h|0)==0){if((j-25|0)>>>0>=2){break}c[e+4>>2]=24}}while(0);Q=(h|0)==4;switch(h|0){case 35:case 4:case 0:{do{if(M){N=10711}else{R=BP(j)|0;if((R|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=656,O)|0);i=O;cw()}else{if((a[R+7|0]&32)==0){break}else{N=10711;break}}}}while(0);do{if((N|0)==10711){if((l&8650752|0)!=0){break}if((m&1|0)!=0){break}c[e+4>>2]=tc(e)|0}}while(0);if((h|0)==6){N=10715}else if((h|0)==3){N=10718}else{S=0}break};case 6:{N=10715;break};case 3:{N=10718;break};default:{S=0}}do{if((N|0)==10715){if(!((j|0)==35|(j|0)==0)){S=0;break}if((l&524288|0)!=0){S=0;break}c[e+4>>2]=18;S=0}else if((N|0)==10718){if(!((j|0)==35|(j|0)==0)){S=1;break}if((l&262144|0)!=0){S=1;break}c[e+4>>2]=66;S=1}}while(0);l=(h|0)==291;m=(h|0)==292;R=(h|0)==43;T=(h|0)==44;U=(h|0)==45;V=(h|0)==46;W=(h|0)==63;X=(h|0)==62;Y=(h|0)==10;Z=(h|0)==9;_=(h|0)==67;$=(h|0)==68;aa=(h|0)==293;ab=(h|0)==294;ac=(h|0)==47;ad=(h|0)==48;ae=(h|0)==49;af=(h|0)==50;ag=(h|0)==65;ah=(h|0)==64;do{if((h-41|0)>>>0<2|l|m|(h|0)==30|(h|0)==29|(h|0)==2|R|T|U|V|W|X|(h|0)==22|(h|0)==23|(h|0)==24|Y|Z|_|$|aa|ab|(h|0)==28|(h|0)==27|S|ac|ad|ae|af|ag|ah|(h|0)==19|(h|0)==20|(h|0)==21|Y|Z){N=10725}else{ai=BP(h)|0;if((ai|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=656,O)|0);i=O;cw()}else{if((a[ai+7|0]&32)==0){break}else{N=10725;break}}}}while(0);L14673:do{if((N|0)==10725){do{if(!M){Z=BP(j)|0;if((Z|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=656,O)|0);i=O;cw()}else{if((a[Z+7|0]&32)==0){break L14673}else{break}}}}while(0);if((wO(e)|0)==0){break}if(P){if((c[k>>2]&17|0)==0){break}}c[e+4>>2]=52}}while(0);P=(h|0)==82;if(P&(j|0)==317){N=10735}else{if((h|0)==317&(j|0)==82){N=10735}}if((N|0)==10735){c[e+4>>2]=32}do{if(P){M=BP(82)|0;if((M|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=772,O)|0);i=O;cw()}if((d[M+4|0]|0)<=1){break}if((a[M+7|0]&16)==0){break}if(!((j-29|0)>>>0<2|q|D|E|F)){break}c[e+4>>2]=50}}while(0);P=(h|0)==84;M=(h|0)==83;Z=(h|0)==88;Y=(h|0)==87;S=(h|0)==86;ai=(h|0)==85;aj=(h|0)==314;ak=(h|0)==313;al=(h|0)==316;am=(h|0)==315;do{if((h-83|0)>>>0<2|Z|Y|S|ai|aj|ak|al|am){if(!(n|A|z|p|o|C|B)){break}c[e+4>>2]=44}}while(0);do{if((b[(BP(h)|0)+8>>1]&30720)==14336){n=BP(h)|0;if((n|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=779,O)|0);i=O;cw()}if(!((a[n+7|0]&48)==32&(j|0)==82)){break}c[e+4>>2]=20}}while(0);do{if(ag&L|ah&K|_&A|$&z|aa&C|ab&B|ae&J|af&I|ac&H|ad&G){N=10790}else{if((h|0)==31&(j|0)==32){N=10790;break}if((h|0)==32&(j|0)==31|M&(j|0)==84|P&(j|0)==83|ai&(j|0)==86|S&(j|0)==85|ak&(j|0)==314|aj&(j|0)==313|am&(j|0)==316|al&(j|0)==315|Y&(j|0)==88|Z&(j|0)==87){N=10790;break}if((h|0)==318&(j|0)==319){N=10790;break}if((h|0)==319&(j|0)==318|W&w|X&v){N=10790;break}if(!((h|0)!=41|(j|0)!=42)){N=10790;break}if((j|0)==41&(h|0)==42|l&p|m&o|U&u|V&t|R&s|T&r){N=10790;break}if((h|0)==111&(j|0)==110){N=10790;break}if((h|0)==110&(j|0)==111){N=10790;break}if((h|0)==69&(j|0)==70){N=10790;break}if((h|0)==70&(j|0)==69){N=10790;break}if((h|0)==71&(j|0)==72){N=10790;break}if((h|0)==72&(j|0)==71){N=10790;break}if((h|0)==301&(j|0)==302){N=10790;break}if((h|0)==302&(j|0)==301){N=10790;break}if((h|0)==303&(j|0)==304){N=10790;break}if((h|0)==304&(j|0)==303){N=10790;break}if((h|0)==55&(j|0)==54){N=10790;break}if((h|0)==54&(j|0)==55){N=10790;break}if((h|0)==79&(j|0)==80){N=10790;break}if((h|0)==80&(j|0)==79){N=10790;break}if((h|0)==73&(j|0)==74){N=10790;break}if((h|0)==74&(j|0)==73){N=10790;break}if((h|0)==305&(j|0)==306){N=10790;break}if((h|0)==306&(j|0)==305){N=10790;break}if((h|0)==307&(j|0)==308){N=10790;break}if((h|0)==308&(j|0)==307){N=10790;break}if((h|0)==57&(j|0)==56){N=10790;break}if((h|0)==56&(j|0)==57){N=10790;break}if((h|0)==75&(j|0)==76){N=10790;break}if((h|0)==76&(j|0)==75){N=10790;break}if((h|0)==77&(j|0)==78){N=10790;break}if((h|0)==78&(j|0)==77){N=10790;break}if((h|0)==309&(j|0)==310){N=10790;break}if((h|0)==310&(j|0)==309){N=10790;break}if((h|0)==311&(j|0)==312){N=10790;break}if((h|0)==312&(j|0)==311){N=10790;break}if((h|0)==59&(j|0)==58){N=10790;break}if((h|0)==58&(j|0)==59){N=10790}}}while(0);if((N|0)==10790){c[e+4>>2]=56}r=BP(h)|0;if((r|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=794,O)|0);i=O;cw()}do{if((a[r+7|0]&66)!=0){if(!((j-29|0)>>>0<2|q|D|E|F)){break}c[e+4>>2]=62}}while(0);do{if(Q){if((j|0)==17){c[e+4>>2]=16;break}else if((j|0)==1){c[e+4>>2]=22;break}else{break}}}while(0);do{if((c[k>>2]&17|0)!=0){if(!((h|0)==35|(h|0)==0)){break}if((j|0)==17){c[e+4>>2]=70;break}else if((j|0)==1){c[e+4>>2]=28;break}else{break}}}while(0);k=(h|0)==1;do{if(k){if(!((j|0)==35|(j|0)==0)){an=0;N=10811;break}c[e+4>>2]=60;an=0;N=10811}else{if((h|0)!=17){break}if(!((j|0)==35|(j|0)==0)){an=1;N=10811;break}c[e+4>>2]=4;an=1;N=10811}}while(0);do{if((N|0)==10811){Q=(j|0)==4;if(k&Q){c[e+4>>2]=2}if(!(an&Q)){break}c[e+4>>2]=6}}while(0);L14786:do{if(!((h|0)==(j|0)|y&(j|0)==0|x&(j|0)==35)){an=BP(h)|0;if((an|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=649,O)|0);i=O;cw()}do{if((a[an+7|0]&16)!=0){k=BP(h)|0;if((k|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=642,O)|0);i=O;cw()}if((a[k+7|0]&32)!=0){break}if((d[k+4|0]|0)<=1){break}if(((j|0)==8|(j|0)==66|(j|0)==31|(j|0)==32)&(j|0)!=66){break L14786}}}while(0);an=BP(j)|0;if((an|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=649,O)|0);i=O;cw()}do{if((a[an+7|0]&16)!=0){k=BP(j)|0;if((k|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=642,O)|0);i=O;cw()}if((a[k+7|0]&32)!=0){break}if((d[k+4|0]|0)<=1){break}if(((h|0)==8|(h|0)==66|(h|0)==31|(h|0)==32)&(h|0)!=66){break L14786}}}while(0);if(((j|0)==8|(j|0)==66|(j|0)==31|(j|0)==32)&(j|0)!=66){if(((h|0)==8|(h|0)==66|(h|0)==31|(h|0)==32)&(h|0)!=66){break}}an=BP(h)|0;if((an|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=649,O)|0);i=O;cw()}if((a[an+7|0]&16)==0){i=f;return}an=BP(h)|0;if((an|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=642,O)|0);i=O;cw()}if((a[an+7|0]&32)!=0){i=f;return}if((d[an+4|0]|0)<=1){i=f;return}an=BP(j)|0;if((an|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=649,O)|0);i=O;cw()}if((a[an+7|0]&16)==0){i=f;return}an=BP(j)|0;if((an|0)==0){AN(0,0,1639504,(O=i,i=i+24|0,c[O>>2]=1718632,c[O+8>>2]=1690888,c[O+16>>2]=642,O)|0);i=O;cw()}if((a[an+7|0]&32)!=0){i=f;return}if((d[an+4|0]|0)<=1){i=f;return}if((c[e+84>>2]|0)!=(c[e+76>>2]|0)){i=f;return}if((c[e+88>>2]|0)!=(c[e+80>>2]|0)){i=f;return}if((j-25|0)>>>0<=1){i=f;return}if((h-25|0)>>>0>1){break}i=f;return}}while(0);switch(c[g>>2]|0){case 11:case 1:case 17:case 66:case 41:case 42:case 291:case 292:case 30:case 29:case 2:case 43:case 44:case 45:case 46:case 63:case 62:case 22:case 23:case 24:case 10:case 294:case 293:case 68:case 67:case 65:case 64:case 50:case 49:case 48:case 47:case 28:case 27:case 21:case 20:case 19:case 3:case 9:{c[e+4>>2]=68;i=f;return};default:{c[e+4>>2]=36;i=f;return}}}function wL(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;i=c[g+4>>2]|0;j=i+((ac(c[h+4>>2]|0,e)|0)/2|0)|0;i=c[b>>2]|0;k=c[d>>2]|0;l=a+8|0;m=c[l>>2]|0;n=c[g>>2]|0;g=c[h>>2]|0;o=n+(ac(g,e)|0)|0;do{if((g|0)==(k|0)&(k|0)>0){e=ac(k,f)|0;NN(o|0,i|0,e)|0}else{if((f|0)>0){p=o;q=i;r=0}else{break}while(1){NN(p|0,q|0,m)|0;e=r+1|0;if((e|0)<(f|0)){p=p+g|0;q=q+k|0;r=e}else{break}}}}while(0);r=c[551804]|0;if((c[a+52>>2]|0)==25){c8[r&7](c[b+4>>2]|0,c[b+8>>2]|0,j,(c[l>>2]|0)/2|0,(f|0)/2|0,c[d+4>>2]|0,c[d+8>>2]|0,c[h>>2]|0);return f|0}else{c8[r&7](c[b+8>>2]|0,c[b+4>>2]|0,j,(c[l>>2]|0)/2|0,(f|0)/2|0,c[d+8>>2]|0,c[d+4>>2]|0,c[h>>2]|0);return f|0}return 0}function wM(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;i=c[b>>2]|0;j=c[d>>2]|0;k=a+8|0;l=c[k>>2]|0;m=c[g>>2]|0;n=c[h>>2]|0;o=m+(ac(n,e)|0)|0;do{if((n|0)==(j|0)&(j|0)>0){m=ac(j,f)|0;NN(o|0,i|0,m)|0}else{if((f|0)>0){p=o;q=i;r=0}else{break}while(1){NN(p|0,q|0,l)|0;m=r+1|0;if((m|0)<(f|0)){p=p+n|0;q=q+j|0;r=m}else{break}}}}while(0);r=c[549648]|0;j=c[b+4>>2]|0;q=c[g+4>>2]|0;n=c[h+4>>2]|0;p=e>>1;l=q+(ac(n,p)|0)|0;q=a+20|0;a=f>>2;c9[r&127](j,l,c[q>>2]|0,a,c[d+4>>2]|0,n);n=c[549648]|0;l=c[b+8>>2]|0;b=c[g+8>>2]|0;j=c[h+8>>2]|0;r=b+(ac(j,p)|0)|0;c9[n&127](l,r,c[q>>2]|0,a,c[d+8>>2]|0,j);j=c[g+12>>2]|0;if((j|0)==0){return f|0}g=c[h+12>>2]|0;h=c[k>>2]|0;if((f|0)<=0){return f|0}k=0;d=j+(ac(g,e)|0)|0;while(1){NM(d|0,-1|0,h|0);e=k+1|0;if((e|0)<(f|0)){k=e;d=d+g|0}else{break}}return f|0}function wN(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;i=c[642300]|0;j=c[b>>2]|0;b=c[g>>2]|0;k=c[h>>2]|0;l=b+(ac(k,e)|0)|0;b=c[g+4>>2]|0;m=e>>1;n=c[h+4>>2]|0;o=b+(ac(n,m)|0)|0;b=c[g+8>>2]|0;p=b+(ac(c[h+8>>2]|0,m)|0)|0;m=a+8|0;df[i&127](j,l,o,p,c[m>>2]|0,f,k,n,c[d>>2]|0,a+10496|0);a=c[g+12>>2]|0;if((a|0)==0){return f|0}g=c[h+12>>2]|0;h=c[m>>2]|0;if((f|0)<=0){return f|0}m=0;d=a+(ac(g,e)|0)|0;while(1){NM(d|0,-1|0,h|0);e=m+1|0;if((e|0)<(f|0)){m=e;d=d+g|0}else{break}}return f|0}function wO(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;b=c[a+56>>2]|0;d=c[a+52>>2]|0;e=c[a+64>>2]|0;f=c[a+60>>2]|0;a=(b|0)==27;g=(b|0)==28;h=(b|0)==30;i=(b|0)==29;do{if((b-27|0)>>>0<2|h|i){j=(d|0)==30;k=(d|0)==29;if(!((d-27|0)>>>0<2|j|k)){break}l=i^1;m=(d|0)!=28;if(!(m|l)){n=2;return n|0}o=a^1;p=j^1;if(!(o|p)){n=2;return n|0}j=h^1;q=(d|0)!=27;if(!(q|j)){n=2;return n|0}r=g^1;s=k^1;if(!(r|s)){n=2;return n|0}if(!((q|l)&(o|s))){n=38;return n|0}if(!((l|p)&(m|o))){n=110;return n|0}if((m|j)&(r|p)){return((j|s)&(q|r)?0:88)|0}n=c[540566]|0;return n|0}}while(0);r=(b-41|0)>>>0<2;q=(b|0)==68;s=(b|0)==67;do{if(r|q|s){j=(d|0)==68;p=(d|0)==67;if(!((d-41|0)>>>0<2|j|p)){break}m=(b|0)!=42;o=j^1;if(!(m|o)){n=66;return n|0}j=q^1;l=(d|0)!=42;if(!(l|j)){n=66;return n|0}k=(b|0)!=41;t=p^1;if(!(k|t)){n=66;return n|0}p=s^1;u=(d|0)!=41;if(!(u|p)){n=66;return n|0}n=(m|t)&(u|j)&(k|o)&(l|p)?0:22;return n|0}}while(0);s=(b|0)==292;q=(b|0)==291;p=(b|0)==294;l=(b|0)==293;do{if((b-291|0)>>>0<2|p|l){o=(d|0)==68;k=(d|0)==67;if(!((d-41|0)>>>0<2|o|k)){break}j=s^1;u=o^1;if(!(j|u)){n=48;return n|0}o=p^1;t=(d|0)!=42;if(!(t|o)){n=48;return n|0}m=q^1;v=k^1;if(!(m|v)){n=48;return n|0}k=l^1;w=(d|0)!=41;if(!(w|k)){n=48;return n|0}if(!((j|v)&(w|o)&(m|u)&(t|k))){n=34;return n|0}if(!((t|j)&(o|u)&(w|m)&(k|v))){n=124;return n|0}n=(w|j)&(o|v)&(t|m)&(k|u)?0:86;return n|0}}while(0);u=(b|0)==10;k=(b|0)==9;m=(b-67|0)>>>0<2|l|p|g|a|(b|0)==3|(b|0)==47|(b|0)==48|(b|0)==49|(b|0)==50|(b|0)==65|(b|0)==64|(b|0)==19|(b|0)==20|(b|0)==21|u|k;if(m){if(!((d-67|0)>>>0<2|(d|0)==293|(d|0)==294|(d|0)==28|(d|0)==27|(d|0)==3|(d|0)==47|(d|0)==48|(d|0)==49|(d|0)==50|(d|0)==65|(d|0)==64|(d|0)==19|(d|0)==20|(d|0)==21|(d|0)==10|(d|0)==9)){x=10927}}else{x=10927}do{if((x|0)==10927){a=r|q|s|h|i|(b|0)==2|(b|0)==43|(b|0)==44|(b|0)==45|(b|0)==46|(b|0)==63|(b|0)==62|(b|0)==22|(b|0)==23|(b|0)==24|u|k;if(a){if((d-41|0)>>>0<2|(d|0)==291|(d|0)==292|(d|0)==30|(d|0)==29|(d|0)==2|(d|0)==43|(d|0)==44|(d|0)==45|(d|0)==46|(d|0)==63|(d|0)==62|(d|0)==22|(d|0)==23|(d|0)==24|(d|0)==10|(d|0)==9){break}}if(m){if(!((d-41|0)>>>0<2|(d|0)==291|(d|0)==292|(d|0)==30|(d|0)==29|(d|0)==2|(d|0)==43|(d|0)==44|(d|0)==45|(d|0)==46|(d|0)==63|(d|0)==62|(d|0)==22|(d|0)==23|(d|0)==24|(d|0)==10|(d|0)==9)){x=10945}}else{x=10945}do{if((x|0)==10945){if(!a){n=0;return n|0}if((d-67|0)>>>0<2|(d|0)==293|(d|0)==294|(d|0)==28|(d|0)==27|(d|0)==3|(d|0)==47|(d|0)==48|(d|0)==49|(d|0)==50|(d|0)==65|(d|0)==64|(d|0)==19|(d|0)==20|(d|0)==21|(d|0)==10|(d|0)==9){break}else{n=0}return n|0}}while(0);a=f<<16|e;if((a|0)==786444){n=102;return n|0}else if((a|0)==2097176){n=c[540976]|0;return n|0}else if((a|0)==983055){n=4;return n|0}else if((a|0)==983056){n=18;return n|0}else if((a|0)==1048600){n=c[540980]|0;return n|0}else if((a|0)==1048608){n=c[540968]|0;return n|0}else if((a|0)==983064){n=c[540982]|0;return n|0}else if((a|0)==983072){n=c[540970]|0;return n|0}else if((a|0)==1572888){n=c[540978]|0;return n|0}else if((a|0)==1572896){n=c[540966]|0;return n|0}else if((a|0)==1048591){n=6;return n|0}else if((a|0)==1048592){n=20;return n|0}else if((a|0)==1572879){n=c[540994]|0;return n|0}else if((a|0)==1572880){n=c[540988]|0;return n|0}else if((a|0)==2097167){n=126;return n|0}else if((a|0)==2097168){n=36;return n|0}else{n=0;return n|0}}}while(0);d=f<<16|e;if((d|0)==983064){n=c[540986]|0;return n|0}else if((d|0)==983072){n=c[540974]|0;return n|0}else if((d|0)==983056){n=c[540992]|0;return n|0}else if((d|0)==2097176){n=90;return n|0}else if((d|0)==1048591){n=c[540998]|0;return n|0}else if((d|0)==1048600){n=c[540984]|0;return n|0}else if((d|0)==983052){n=40;return n|0}else if((d|0)==2097167){n=c[540996]|0;return n|0}else if((d|0)==2097168){n=c[540990]|0;return n|0}else if((d|0)==1048608){n=c[540972]|0;return n|0}else if((d|0)==1572879){n=120;return n|0}else if((d|0)==1572880){n=122;return n|0}else if((d|0)==1572896){n=64;return n|0}else{n=0;return n|0}return 0}function wP(d,e,f,g,h,j,k){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;l=i;m=c[d+56>>2]|0;n=d+52|0;o=c[n>>2]|0;p=BP(m)|0;q=BP(c[n>>2]|0)|0;n=(c[d+64>>2]|0)+7|0;r=n>>3;s=(c[d+60>>2]|0)+7|0;t=wO(d)|0;if((t|0)==0){u=BL(m)|0;v=BL(o)|0;AN(d,16,1671096,(w=i,i=i+16|0,c[w>>2]=u,c[w+8>>2]=v,w)|0);i=w;i=l;return h|0}w=c[e>>2]|0;e=c[j>>2]|0;if((n&-8|0)==16){x=(a[p+7|0]&1)!=0}else{x=0}if((s&-8|0)==16){y=(a[q+7|0]&1)!=0}else{y=0}if((m|0)==29|(m|0)==27){z=(o-27|0)>>>0<2|(o|0)==30|(o|0)==29?w:w+1|0}else{z=w}if((o|0)==29|(o|0)==27){A=(m-27|0)>>>0<2|(m|0)==30|(m|0)==29?e:e+1|0}else{A=e}e=c[k>>2]|0;m=ac(e,r)|0;o=c[f>>2]|0;do{if((m|0)==(ac(o,s>>3)|0)&(o|0)>0){if(((o|0)%(r|0)|0|0)!=0|y|x){break}w=A+(ac(e,g)|0)|0;q=ac(o,h)|0;dm[t&127](z,w,q);i=l;return h|0}}while(0);o=A+(ac(e,g)|0)|0;if((h|0)<=0){i=l;return h|0}g=d+8|0;e=d+2208|0;if(x){B=o;C=z;D=0}else{x=o;o=z;z=0;while(1){d=ac(c[g>>2]|0,r)|0;dm[t&127](o,x,d);do{if(y){if((c[g>>2]|0)<=0){break}d=x;A=0;do{s=d+(A<<1)|0;b[s>>1]=cT(b[s>>1]|0)|0;A=A+1|0;}while((A|0)<(c[g>>2]|0))}}while(0);A=z+1|0;if((A|0)<(h|0)){x=x+(c[k>>2]|0)|0;o=o+(c[f>>2]|0)|0;z=A}else{break}}i=l;return h|0}while(1){z=c[g>>2]|0;if((z|0)>0){o=C;x=0;while(1){A=cT(b[o+(x<<1)>>1]|0)|0;b[(c[e>>2]|0)+(x<<1)>>1]=A;A=x+1|0;d=c[g>>2]|0;if((A|0)<(d|0)){x=A}else{E=d;break}}}else{E=z}x=c[e>>2]|0;o=ac(E,r)|0;dm[t&127](x,B,o);do{if(y){if((c[g>>2]|0)<=0){break}o=B;x=0;do{d=o+(x<<1)|0;b[d>>1]=cT(b[d>>1]|0)|0;x=x+1|0;}while((x|0)<(c[g>>2]|0))}}while(0);z=D+1|0;if((z|0)<(h|0)){B=B+(c[k>>2]|0)|0;C=C+(c[f>>2]|0)|0;D=z}else{break}}i=l;return h|0}function wQ(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;i=c[b>>2]|0;j=c[d>>2]|0;k=a+8|0;a=c[k>>2]|0;l=c[g>>2]|0;m=c[h>>2]|0;n=l+(ac(m,e)|0)|0;do{if((m|0)==(j|0)&(j|0)>0){l=ac(j,f)|0;NN(n|0,i|0,l)|0}else{if((f|0)>0){o=n;p=i;q=0}else{break}while(1){NN(o|0,p|0,a)|0;l=q+1|0;if((l|0)<(f|0)){o=o+m|0;p=p+j|0;q=l}else{break}}}}while(0);q=c[b+4>>2]|0;j=c[d+4>>2]|0;p=c[k>>2]|0;m=c[g+4>>2]|0;o=c[h+4>>2]|0;a=m+(ac(o,e)|0)|0;do{if((o|0)==(j|0)&(j|0)>0){m=ac(j,f)|0;NN(a|0,q|0,m)|0}else{if((f|0)>0){r=a;s=q;t=0}else{break}while(1){NN(r|0,s|0,p)|0;m=t+1|0;if((m|0)<(f|0)){r=r+o|0;s=s+j|0;t=m}else{break}}}}while(0);t=c[b+8>>2]|0;b=c[d+8>>2]|0;d=c[k>>2]|0;j=c[g+8>>2]|0;s=c[h+8>>2]|0;o=j+(ac(s,e)|0)|0;do{if((s|0)==(b|0)&(b|0)>0){j=ac(b,f)|0;NN(o|0,t|0,j)|0}else{if((f|0)>0){u=o;v=t;w=0}else{break}while(1){NN(u|0,v|0,d)|0;j=w+1|0;if((j|0)<(f|0)){u=u+s|0;v=v+b|0;w=j}else{break}}}}while(0);w=c[g+12>>2]|0;if((w|0)==0){return f|0}g=c[h+12>>2]|0;h=c[k>>2]|0;if((f|0)<=0){return f|0}k=0;b=w+(ac(g,e)|0)|0;while(1){NM(b|0,-1|0,h|0);e=k+1|0;if((e|0)<(f|0)){k=e;b=b+g|0}else{break}}return f|0}function wR(b,d,e,f,g,h,j){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0;k=i;l=c[d+4>>2]|0;m=c[d>>2]|0;n=c[d+8>>2]|0;d=c[e+4>>2]|0;o=c[e>>2]|0;p=c[e+8>>2]|0;e=c[b+56>>2]|0;if((e|0)!=82){q=BL(e)|0;e=BL(c[b+52>>2]|0)|0;AN(b,16,1656272,(r=i,i=i+16|0,c[r>>2]=q,c[r+8>>2]=e,r)|0);i=r;i=k;return g|0}e=b+52|0;switch(c[e>>2]|0){case 2:{q=c[h>>2]|0;s=c[j>>2]|0;t=c[b+8>>2]|0;if((g|0)>0&(t|0)>0){u=0;v=n;w=m;x=l}else{i=k;return g|0}while(1){y=0;z=q+(ac(u+f|0,s)|0)|0;while(1){a[z]=a[v+y|0]|0;a[z+1|0]=a[w+y|0]|0;a[z+2|0]=a[x+y|0]|0;A=y+1|0;if((A|0)<(t|0)){y=A;z=z+3|0}else{break}}z=u+1|0;if((z|0)<(g|0)){u=z;v=v+p|0;w=w+o|0;x=x+d|0}else{break}}i=k;return g|0};case 29:{B=1;C=11092;break};case 28:{D=0;C=11083;break};case 30:{B=0;C=11092;break};case 27:{D=1;C=11083;break};case 3:{x=c[h>>2]|0;w=c[j>>2]|0;v=c[b+8>>2]|0;if((g|0)>0&(v|0)>0){E=0;F=l;G=m;H=n}else{i=k;return g|0}while(1){u=0;t=x+(ac(E+f|0,w)|0)|0;while(1){a[t]=a[F+u|0]|0;a[t+1|0]=a[G+u|0]|0;a[t+2|0]=a[H+u|0]|0;s=u+1|0;if((s|0)<(v|0)){u=s;t=t+3|0}else{break}}t=E+1|0;if((t|0)<(g|0)){E=t;F=F+d|0;G=G+o|0;H=H+p|0}else{break}}i=k;return g|0};default:{H=BL(82)|0;G=BL(c[e>>2]|0)|0;AN(b,16,1656272,(r=i,i=i+16|0,c[r>>2]=H,c[r+8>>2]=G,r)|0);i=r;i=k;return g|0}}if((C|0)==11083){r=c[h>>2]|0;G=c[j>>2]|0;H=c[b+8>>2]|0;if((g|0)<=0){i=k;return g|0}e=(D|0)==0;D=(H|0)>0;F=0;E=n;v=m;w=l;while(1){x=r+(ac(F+f|0,G)|0)|0;do{if(e){if(D){I=0;J=x}else{break}while(1){a[J]=a[E+I|0]|0;a[J+1|0]=a[v+I|0]|0;a[J+2|0]=a[w+I|0]|0;a[J+3|0]=-1;t=I+1|0;if((t|0)<(H|0)){I=t;J=J+4|0}else{break}}}else{if(D){K=0;L=x}else{break}while(1){a[L]=-1;a[L+1|0]=a[E+K|0]|0;a[L+2|0]=a[v+K|0]|0;a[L+3|0]=a[w+K|0]|0;t=K+1|0;if((t|0)<(H|0)){K=t;L=L+4|0}else{break}}}}while(0);x=F+1|0;if((x|0)<(g|0)){F=x;E=E+p|0;v=v+o|0;w=w+d|0}else{break}}i=k;return g|0}else if((C|0)==11092){C=c[h>>2]|0;h=c[j>>2]|0;j=c[b+8>>2]|0;if((g|0)<=0){i=k;return g|0}b=(B|0)==0;B=(j|0)>0;w=0;v=l;l=m;m=n;while(1){n=C+(ac(w+f|0,h)|0)|0;do{if(b){if(B){M=0;N=n}else{break}while(1){a[N]=a[v+M|0]|0;a[N+1|0]=a[l+M|0]|0;a[N+2|0]=a[m+M|0]|0;a[N+3|0]=-1;E=M+1|0;if((E|0)<(j|0)){M=E;N=N+4|0}else{break}}}else{if(B){O=0;P=n}else{break}while(1){a[P]=-1;a[P+1|0]=a[v+O|0]|0;a[P+2|0]=a[l+O|0]|0;a[P+3|0]=a[m+O|0]|0;E=O+1|0;if((E|0)<(j|0)){O=E;P=P+4|0}else{break}}}}while(0);n=w+1|0;if((n|0)<(g|0)){w=n;v=v+d|0;l=l+o|0;m=m+p|0}else{break}}i=k;return g|0}return 0}function wS(b,d,f,g,h,j,k){b=b|0;d=d|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;l=i;i=i+64|0;m=l|0;n=l+16|0;o=l+32|0;p=l+48|0;q=m|0;r=c[d+4>>2]|0;c[q>>2]=r;s=c[d>>2]|0;c[m+4>>2]=s;t=c[d+8>>2]|0;c[m+8>>2]=t;m=n|0;c[m>>2]=t;c[n+4>>2]=s;c[n+8>>2]=r;r=o|0;n=c[f+4>>2]|0;c[r>>2]=n;s=c[f>>2]|0;c[o+4>>2]=s;t=c[f+8>>2]|0;c[o+8>>2]=t;o=p|0;c[o>>2]=t;c[p+4>>2]=s;c[p+8>>2]=n;n=BP(c[b+56>>2]|0)|0;p=b+52|0;s=BP(c[p>>2]|0)|0;t=((e[n+8>>1]|0)>>>11&15)+1|0;f=a[n+7|0]|0;d=f&1;u=(a[s+7|0]&1)==0?d:d|2;if((f&48)!=48|t>>>0<9){f=c[s>>2]|0;AN(b,16,1656272,(v=i,i=i+16|0,c[v>>2]=c[n>>2],c[v+8>>2]=f,v)|0);i=v;i=l;return h|0}switch(c[p>>2]|0){case 292:case 291:{p=c[k>>2]|0;w9(m,o,(c[j>>2]|0)+(ac(p,g)|0)|0,p,h,1,u,t,c[b+8>>2]|0);i=l;return h|0};case 294:case 293:{p=c[k>>2]|0;w9(q,r,(c[j>>2]|0)+(ac(p,g)|0)|0,p,h,1,u,t,c[b+8>>2]|0);i=l;return h|0};case 42:case 41:{p=c[k>>2]|0;w9(m,o,(c[j>>2]|0)+(ac(p,g)|0)|0,p,h,0,u,t,c[b+8>>2]|0);i=l;return h|0};case 68:case 67:{p=c[k>>2]|0;w9(q,r,(c[j>>2]|0)+(ac(p,g)|0)|0,p,h,0,u,t,c[b+8>>2]|0);i=l;return h|0};default:{t=c[s>>2]|0;AN(b,16,1656272,(v=i,i=i+16|0,c[v>>2]=c[n>>2],c[v+8>>2]=t,v)|0);i=v;i=l;return h|0}}return 0}function wT(b,d,e,f,g,h,j){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0;k=i;l=c[j+4>>2]|0;m=c[j>>2]|0;n=c[j+8>>2]|0;j=c[h+4>>2]|0;o=j+(ac(l,f)|0)|0;j=c[h>>2]|0;p=j+(ac(m,f)|0)|0;j=c[h+8>>2]|0;h=j+(ac(n,f)|0)|0;f=c[b+56>>2]|0;switch(f|0){case 2:{j=c[b+8>>2]|0;if((g|0)<=0){i=k;return g|0}q=(j|0)>0;r=j*3|0;s=(c[e>>2]|0)-r|0;t=c[d>>2]|0;u=o;v=p;w=h;x=0;while(1){if(q){y=t;z=0;while(1){a[w+z|0]=a[y]|0;a[v+z|0]=a[y+1|0]|0;a[u+z|0]=a[y+2|0]|0;A=z+1|0;if((A|0)<(j|0)){y=y+3|0;z=A}else{break}}B=t+r|0}else{B=t}z=x+1|0;if((z|0)<(g|0)){t=B+s|0;u=u+l|0;v=v+m|0;w=w+n|0;x=z}else{break}}i=k;return g|0};case 27:{C=1;D=11141;break};case 3:{x=c[b+8>>2]|0;if((g|0)<=0){i=k;return g|0}w=(x|0)>0;v=x*3|0;u=(c[e>>2]|0)-v|0;s=c[d>>2]|0;B=h;t=p;r=o;j=0;while(1){if(w){q=s;z=0;while(1){a[r+z|0]=a[q]|0;a[t+z|0]=a[q+1|0]|0;a[B+z|0]=a[q+2|0]|0;y=z+1|0;if((y|0)<(x|0)){q=q+3|0;z=y}else{break}}E=s+v|0}else{E=s}z=j+1|0;if((z|0)<(g|0)){s=E+u|0;B=B+n|0;t=t+m|0;r=r+l|0;j=z}else{break}}i=k;return g|0};case 28:{C=0;D=11141;break};case 29:{F=1;D=11148;break};case 30:{F=0;D=11148;break};default:{j=BL(f)|0;f=BL(c[b+52>>2]|0)|0;AN(b,16,1656272,(r=i,i=i+16|0,c[r>>2]=j,c[r+8>>2]=f,r)|0);i=r;i=k;return g|0}}if((D|0)==11141){r=c[d>>2]|0;f=c[b+8>>2]|0;if((g|0)<=0){i=k;return g|0}j=(f|0)>0;t=f<<2;B=(c[e>>2]|0)-t|0;u=(C|0)==0?r:r+1|0;r=o;C=p;E=h;s=0;while(1){if(j){v=u;x=0;while(1){a[E+x|0]=a[v]|0;a[C+x|0]=a[v+1|0]|0;a[r+x|0]=a[v+2|0]|0;w=x+1|0;if((w|0)<(f|0)){v=v+4|0;x=w}else{break}}G=u+t|0}else{G=u}x=s+1|0;if((x|0)<(g|0)){u=G+B|0;r=r+l|0;C=C+m|0;E=E+n|0;s=x}else{break}}i=k;return g|0}else if((D|0)==11148){D=c[d>>2]|0;d=c[b+8>>2]|0;if((g|0)<=0){i=k;return g|0}b=(d|0)>0;s=d<<2;E=(c[e>>2]|0)-s|0;e=(F|0)==0?D:D+1|0;D=h;h=p;p=o;o=0;while(1){if(b){F=e;C=0;while(1){a[p+C|0]=a[F]|0;a[h+C|0]=a[F+1|0]|0;a[D+C|0]=a[F+2|0]|0;r=C+1|0;if((r|0)<(d|0)){F=F+4|0;C=r}else{break}}H=e+s|0}else{H=e}C=o+1|0;if((C|0)<(g|0)){e=H+E|0;D=D+n|0;h=h+m|0;p=p+l|0;o=C}else{break}}i=k;return g|0}return 0}function wU(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;f=a+88|0;a=0;do{j=c[e+(a<<2)>>2]|0;k=(j|0)/2|0;l=c[i+(a<<2)>>2]|0;m=(l|0)/2|0;n=c[h+(a<<2)>>2]|0;o=n;p=c[d+(a<<2)>>2]|0;q=p;r=(j|0)>-2?k:-k|0;j=(l|0)>-2?m:-m|0;l=(r|0)>(j|0)?j:r;do{if(!((n|0)==0|(p|0)==0)){r=c[f>>2]|0;if((g>>r|0)<=0){break}j=(l|0)>0;s=o;t=q;u=0;v=r;while(1){if(j){r=0;do{b[s+(r<<1)>>1]=cT(b[t+(r<<1)>>1]|0)|0;r=r+1|0;}while((r|0)<(l|0));w=c[f>>2]|0}else{w=v}r=u+1|0;if((r|0)<(g>>w|0)){s=s+(m<<1)|0;t=t+(k<<1)|0;u=r;v=w}else{break}}}}while(0);a=a+1|0;}while((a|0)<4);return g|0}function wV(b,d,e,f,g,h,j){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0;k=i;l=c[b+56>>2]|0;m=c[b+52>>2]|0;n=c[h>>2]|0;h=n+(ac(c[j>>2]|0,f)|0)|0;f=c[d>>2]|0;L15266:do{if((l|0)==66){switch(m|0){case 2:{o=78;break L15266;break};case 3:{o=78;break L15266;break};case 29:{o=70;break L15266;break};case 27:{o=70;break L15266;break};case 30:case 28:{o=18;break L15266;break};default:{p=11189;break L15266}}}else{d=BP(l)|0;if((d|0)==0){AN(0,0,1639504,(q=i,i=i+24|0,c[q>>2]=1718632,c[q+8>>2]=1690888,c[q+16>>2]=794,q)|0);i=q;cw();return 0}if((a[d+7|0]&66)==0){p=11189;break}switch(m|0){case 2:{o=16;break L15266;break};case 3:{o=16;break L15266;break};case 30:case 28:case 27:case 29:{o=90;break L15266;break};default:{p=11189;break L15266}}}}while(0);if((p|0)==11189){p=BL(l)|0;l=BL(m)|0;AN(b,16,1671096,(q=i,i=i+16|0,c[q>>2]=p,c[q+8>>2]=l,q)|0);i=q;i=k;return g|0}if((g|0)<=0){i=k;return g|0}q=b+8|0;l=b+1144|0;b=0;p=h;h=f;while(1){dq[o&127](h,p,c[q>>2]|0,l);f=b+1|0;if((f|0)<(g|0)){b=f;p=p+(c[j>>2]|0)|0;h=h+(c[e>>2]|0)|0}else{break}}i=k;return g|0}function wW(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;i=c[g>>2]|0;g=c[h>>2]|0;h=i+(ac(g,e)|0)|0;de[c[498224]&127](c[b>>2]|0,c[b+4>>2]|0,c[b+8>>2]|0,h,c[a+8>>2]|0,f,c[d>>2]|0,c[d+4>>2]|0,g);return f|0}function wX(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;i=c[g>>2]|0;g=c[h>>2]|0;h=i+(ac(g,e)|0)|0;de[c[498226]&127](c[b>>2]|0,c[b+4>>2]|0,c[b+8>>2]|0,h,c[a+8>>2]|0,f,c[d>>2]|0,c[d+4>>2]|0,g);return f|0}function wY(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;i=c[g>>2]|0;g=c[h>>2]|0;h=i+(ac(g,e)|0)|0;de[c[498214]&127](c[b>>2]|0,c[b+4>>2]|0,c[b+8>>2]|0,h,c[a+8>>2]|0,f,c[d>>2]|0,c[d+4>>2]|0,g);return f|0}function wZ(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;i=c[g>>2]|0;g=c[h>>2]|0;h=i+(ac(g,e)|0)|0;de[c[498216]&127](c[b>>2]|0,c[b+4>>2]|0,c[b+8>>2]|0,h,c[a+8>>2]|0,f,c[d>>2]|0,c[d+4>>2]|0,g);return f|0}function w_(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0;i=c[g>>2]|0;j=c[h>>2]|0;k=i+(ac(j,e)|0)|0;i=c[g+4>>2]|0;l=c[h+4>>2]|0;m=i+((ac(l,e)|0)/2|0)|0;i=c[g+8>>2]|0;n=i+((ac(c[h+8>>2]|0,e)|0)/2|0)|0;i=a+8|0;de[c[498220]&127](k,m,n,c[b>>2]|0,c[i>>2]|0,f,j,l,c[d>>2]|0);d=c[g+12>>2]|0;if((d|0)==0){return f|0}g=c[h+12>>2]|0;h=c[i>>2]|0;if((f|0)<=0){return f|0}i=0;l=d+(ac(g,e)|0)|0;while(1){NM(l|0,-1|0,h|0);e=i+1|0;if((e|0)<(f|0)){i=e;l=l+g|0}else{break}}return f|0}function w$(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0;i=c[g>>2]|0;j=c[h>>2]|0;k=i+(ac(j,e)|0)|0;i=c[g+4>>2]|0;l=c[h+4>>2]|0;m=i+((ac(l,e)|0)/2|0)|0;i=c[g+8>>2]|0;n=i+((ac(c[h+8>>2]|0,e)|0)/2|0)|0;i=a+8|0;de[c[499520]&127](k,m,n,c[b>>2]|0,c[i>>2]|0,f,j,l,c[d>>2]|0);d=c[g+12>>2]|0;if((d|0)==0){return f|0}g=c[h+12>>2]|0;h=c[i>>2]|0;if((f|0)<=0){return f|0}i=0;l=d+(ac(g,e)|0)|0;while(1){NM(l|0,-1|0,h|0);e=i+1|0;if((e|0)<(f|0)){i=e;l=l+g|0}else{break}}return f|0}function w0(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0;i=c[g>>2]|0;j=c[h>>2]|0;k=i+(ac(j,e)|0)|0;i=c[g+4>>2]|0;l=c[h+4>>2]|0;m=i+(ac(l,e)|0)|0;i=c[g+8>>2]|0;g=i+(ac(c[h+8>>2]|0,e)|0)|0;de[c[498218]&127](k,m,g,c[b>>2]|0,c[a+8>>2]|0,f,j,l,c[d>>2]|0);return f|0}function w1(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0;i=c[g>>2]|0;j=c[h>>2]|0;k=i+(ac(j,e)|0)|0;i=c[g+4>>2]|0;l=c[h+4>>2]|0;m=i+(ac(l,e)|0)|0;i=c[g+8>>2]|0;g=i+(ac(c[h+8>>2]|0,e)|0)|0;de[c[499518]&127](k,m,g,c[b>>2]|0,c[a+8>>2]|0,f,j,l,c[d>>2]|0);return f|0}function w2(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;i=c[h>>2]|0;j=c[d>>2]|0;if((i|0)==(j|0)&(j|0)>0){k=c[g>>2]|0;l=k+(ac(i,e)|0)|0;k=c[b>>2]|0;m=ac(i,f)|0;NN(l|0,k|0,m)|0;return f|0}m=c[b>>2]|0;b=c[g>>2]|0;g=b+(ac(i,e)|0)|0;e=c[a+8>>2]|0;a=(i|0)>-1?i:-i|0;i=0;while(1){b=e+i|0;if((b|0)>(a|0)){break}if((b|0)>(((j|0)>-1?j:-j|0)|0)){break}else{i=b}}if((f|0)>0){n=0;o=m;p=g}else{return f|0}while(1){NN(p|0,o|0,i)|0;g=n+1|0;if((g|0)<(f|0)){n=g;o=o+(c[d>>2]|0)|0;p=p+(c[h>>2]|0)|0}else{break}}return f|0}function w3(f,g,h,j,k,l,m){f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0;n=i;o=f+56|0;p=BP(c[o>>2]|0)|0;q=f+52|0;r=BP(c[q>>2]|0)|0;s=f+8|0;t=f+84|0;u=-j|0;v=f+88|0;w=-k|0;x=l+8|0;z=g+8|0;A=p+8|0;B=f+11260|0;f=0;L15332:while(1){C=(f|0)==3;D=c[s>>2]|0;if((f|0)==3|(f|0)==0){E=D}else{E=-(-D>>c[t>>2])|0}if((f|0)==3|(f|0)==0){F=j}else{F=-(u>>c[v>>2])|0}if((f|0)==3|(f|0)==0){G=k}else{G=-(w>>c[v>>2])|0}D=g+(f<<2)|0;H=c[D>>2]|0;I=l+(f<<2)|0;J=c[I>>2]|0;K=m+(f<<2)|0;L=J+(ac(c[K>>2]|0,F)|0)|0;M=(f|0)==1;if((f-1|0)>>>0<2){N=1}else{N=(c[B>>2]|f|0)==0}L15346:do{if((J|0)!=0){do{if(M){if((c[x>>2]|0)!=0){break}f=f+1|0;continue L15332}}while(0);do{if((H|0)!=0){if(M){if((c[z>>2]|0)==0){break}}O=BP(c[o>>2]|0)|0;if((O|0)==0){P=11267;break L15332}do{if((((e[O+8>>1]|0)>>>11&15)-8&65535)>=6){Q=BP(c[q>>2]|0)|0;if((Q|0)==0){P=11270;break L15332}if((((e[Q+8>>1]|0)>>>11&15)-8&65535)<6){break}Q=BP(c[o>>2]|0)|0;if((Q|0)==0){P=11273;break L15332}R=b[Q+8>>1]|0;Q=BP(c[q>>2]|0)|0;if((Q|0)==0){P=11275;break L15332}if((R&30720)==30720^(b[Q+8>>1]&30720)==30720){break}Q=BP(c[o>>2]|0)|0;if((Q|0)==0){P=11400;break L15332}do{if((b[Q+8>>1]&30720)==30720){R=BP(c[q>>2]|0)|0;if((R|0)==0){P=11403;break L15332}if((b[R+8>>1]&30720)!=30720){break}R=BP(c[o>>2]|0)|0;if((R|0)==0){P=11406;break L15332}S=a[R+7|0]|0;R=BP(c[q>>2]|0)|0;if((R|0)==0){P=11408;break L15332}if(((a[R+7|0]^S)&1)==0){break}if((G|0)<=0){break L15346}S=(E|0)>0;R=h+(f<<2)|0;T=L;U=H;V=0;while(1){if(S){W=U;X=T;Y=0;do{b[X+(Y<<1)>>1]=cT(b[W+(Y<<1)>>1]|0)|0;Y=Y+1|0;}while((Y|0)<(E|0))}Y=V+1|0;if((Y|0)<(G|0)){T=T+(c[K>>2]|0)|0;U=U+(c[R>>2]|0)|0;V=Y}else{break L15346}}}}while(0);Q=c[K>>2]|0;V=h+(f<<2)|0;R=c[V>>2]|0;if((Q|0)==(R|0)&(R|0)>0&(R|0)==(E|0)){R=c[I>>2]|0;U=R+(ac(Q,F)|0)|0;R=c[D>>2]|0;T=ac(Q,G)|0;NN(U|0,R|0,T)|0;break L15346}T=BP(c[o>>2]|0)|0;if((T|0)==0){P=11419;break L15332}do{if((b[T+8>>1]&30720)==30720){R=BP(c[q>>2]|0)|0;if((R|0)==0){P=11422;break L15332}if((b[R+8>>1]&30720)!=30720){P=11425;break}Z=E<<1}else{P=11425}}while(0);if((P|0)==11425){P=0;Z=(b[A>>1]&30720)==0?E>>3:E}if((G|0)>0){_=L;$=H;aa=0}else{break L15346}while(1){NN(_|0,$|0,Z)|0;T=aa+1|0;if((T|0)<(G|0)){_=_+(c[K>>2]|0)|0;$=$+(c[V>>2]|0)|0;aa=T}else{break L15346}}}}while(0);O=(e[p+8+(f<<1)>>1]|0)>>>11&15;V=O+1|0;T=(e[r+8+(f<<1)>>1]|0)>>>11&15;R=T+1|0;U=H;Q=L;if((R|0)==8){S=BP(c[o>>2]|0)|0;if((S|0)==0){P=11279;break L15332}Y=b[261216+(T<<5)+(O<<1)>>1]|0;W=(e[261216+(O-1<<5)+(T<<1)>>1]|0)+(O-7)|0;X=(G|0)>0;if((a[S+7|0]&1)==0){if(!X){break L15346}S=O-8|0;ab=E-7|0;ad=(ab|0)>0;ae=h+(f<<2)|0;af=Y&65535;ag=U;ah=L;ai=0;while(1){aj=ai&7;if(ad){ak=d[260704+(S<<6)+(aj<<3)|0]|0;al=d[260704+(S<<6)+(aj<<3)+1|0]|0;am=d[260704+(S<<6)+(aj<<3)+2|0]|0;an=d[260704+(S<<6)+(aj<<3)+3|0]|0;ao=d[260704+(S<<6)+(aj<<3)+4|0]|0;ap=d[260704+(S<<6)+(aj<<3)+5|0]|0;aq=d[260704+(S<<6)+(aj<<3)+6|0]|0;ar=d[260704+(S<<6)+(aj<<3)+7|0]|0;as=0;while(1){a[ah+as|0]=(ac(ak+(e[ag+(as<<1)>>1]|0)|0,af)|0)>>>(W>>>0)&255;at=as|1;a[ah+at|0]=(ac(al+(e[ag+(at<<1)>>1]|0)|0,af)|0)>>>(W>>>0)&255;at=as|2;a[ah+at|0]=(ac(am+(e[ag+(at<<1)>>1]|0)|0,af)|0)>>>(W>>>0)&255;at=as|3;a[ah+at|0]=(ac(an+(e[ag+(at<<1)>>1]|0)|0,af)|0)>>>(W>>>0)&255;at=as|4;a[ah+at|0]=(ac(ao+(e[ag+(at<<1)>>1]|0)|0,af)|0)>>>(W>>>0)&255;at=as|5;a[ah+at|0]=(ac(ap+(e[ag+(at<<1)>>1]|0)|0,af)|0)>>>(W>>>0)&255;at=as|6;a[ah+at|0]=(ac(aq+(e[ag+(at<<1)>>1]|0)|0,af)|0)>>>(W>>>0)&255;at=as|7;a[ah+at|0]=(ac(ar+(e[ag+(at<<1)>>1]|0)|0,af)|0)>>>(W>>>0)&255;at=as+8|0;if((at|0)<(ab|0)){as=at}else{au=at;break}}}else{au=0}if((au|0)<(E|0)){as=au;do{a[ah+as|0]=(ac((d[(as&7)+(260704+(S<<6)+(aj<<3))|0]|0)+(e[ag+(as<<1)>>1]|0)|0,af)|0)>>>(W>>>0)&255;as=as+1|0;}while((as|0)<(E|0))}as=ai+1|0;if((as|0)<(G|0)){ag=ag+(((c[ae>>2]|0)/2|0)<<1)|0;ah=ah+(c[K>>2]|0)|0;ai=as}else{break L15346}}}else{if(!X){break L15346}ai=O-8|0;ah=E-7|0;ae=(ah|0)>0;ag=h+(f<<2)|0;af=Y&65535;S=U;ab=L;ad=0;while(1){as=ad&7;if(ae){aj=d[260704+(ai<<6)+(as<<3)|0]|0;ar=d[260704+(ai<<6)+(as<<3)+1|0]|0;aq=d[260704+(ai<<6)+(as<<3)+2|0]|0;ap=d[260704+(ai<<6)+(as<<3)+3|0]|0;ao=d[260704+(ai<<6)+(as<<3)+4|0]|0;an=d[260704+(ai<<6)+(as<<3)+5|0]|0;am=d[260704+(ai<<6)+(as<<3)+6|0]|0;al=d[260704+(ai<<6)+(as<<3)+7|0]|0;ak=0;while(1){a[ab+ak|0]=(ac(aj+((cT(b[S+(ak<<1)>>1]|0)|0)&65535)|0,af)|0)>>>(W>>>0)&255;at=ak|1;a[ab+at|0]=(ac(ar+((cT(b[S+(at<<1)>>1]|0)|0)&65535)|0,af)|0)>>>(W>>>0)&255;at=ak|2;a[ab+at|0]=(ac(aq+((cT(b[S+(at<<1)>>1]|0)|0)&65535)|0,af)|0)>>>(W>>>0)&255;at=ak|3;a[ab+at|0]=(ac(ap+((cT(b[S+(at<<1)>>1]|0)|0)&65535)|0,af)|0)>>>(W>>>0)&255;at=ak|4;a[ab+at|0]=(ac(ao+((cT(b[S+(at<<1)>>1]|0)|0)&65535)|0,af)|0)>>>(W>>>0)&255;at=ak|5;a[ab+at|0]=(ac(an+((cT(b[S+(at<<1)>>1]|0)|0)&65535)|0,af)|0)>>>(W>>>0)&255;at=ak|6;a[ab+at|0]=(ac(am+((cT(b[S+(at<<1)>>1]|0)|0)&65535)|0,af)|0)>>>(W>>>0)&255;at=ak|7;a[ab+at|0]=(ac(al+((cT(b[S+(at<<1)>>1]|0)|0)&65535)|0,af)|0)>>>(W>>>0)&255;at=ak+8|0;if((at|0)<(ah|0)){ak=at}else{av=at;break}}}else{av=0}if((av|0)<(E|0)){ak=av;do{al=(cT(b[S+(ak<<1)>>1]|0)|0)&65535;a[ab+ak|0]=(ac((d[(ak&7)+(260704+(ai<<6)+(as<<3))|0]|0)+al|0,af)|0)>>>(W>>>0)&255;ak=ak+1|0;}while((ak|0)<(E|0))}ak=ad+1|0;if((ak|0)<(G|0)){S=S+(((c[ag>>2]|0)/2|0)<<1)|0;ab=ab+(c[K>>2]|0)|0;ad=ak}else{break L15346}}}}if((V|0)==8){if((G|0)<=0){break L15346}ad=(E|0)>0;ab=T-7|0;ag=T^15;S=h+(f<<2)|0;W=Q;af=H;ai=0;while(1){ah=BP(c[q>>2]|0)|0;if((ah|0)==0){P=11301;break L15332}do{if((a[ah+7|0]&1)==0){if(N){if(ad){aw=0}else{break}do{ae=W+(aw<<1)|0;y=(d[af+aw|0]|0)<<ab&65535;a[ae]=y&255;y=y>>8;a[ae+1|0]=y&255;aw=aw+1|0;}while((aw|0)<(E|0))}else{if(ad){ax=0}else{break}do{ae=d[af+ax|0]|0;Y=W+(ax<<1)|0;y=(ae<<ab|ae>>>(ag>>>0))&65535;a[Y]=y&255;y=y>>8;a[Y+1|0]=y&255;ax=ax+1|0;}while((ax|0)<(E|0))}}else{if(N){if(ad){ay=0}else{break}do{Y=(d[af+ay|0]|0)<<ab&65535;ae=W+(ay<<1)|0;y=(Y>>>8|Y<<8)&65535;a[ae]=y&255;y=y>>8;a[ae+1|0]=y&255;ay=ay+1|0;}while((ay|0)<(E|0))}else{if(ad){az=0}else{break}do{ae=d[af+az|0]|0;Y=(ae<<ab|ae>>>(ag>>>0))&65535;ae=W+(az<<1)|0;y=(Y>>>8|Y<<8)&65535;a[ae]=y&255;y=y>>8;a[ae+1|0]=y&255;az=az+1|0;}while((az|0)<(E|0))}}}while(0);ah=ai+1|0;if((ah|0)<(G|0)){W=W+(((c[K>>2]|0)/2|0)<<1)|0;af=af+(c[S>>2]|0)|0;ai=ah}else{break L15346}}}if(V>>>0>R>>>0){ai=BP(c[o>>2]|0)|0;if((ai|0)==0){P=11359;break L15332}S=(a[ai+7|0]&1)==0;ai=BP(c[q>>2]|0)|0;af=(ai|0)==0;if(S){if(af){P=11362;break L15332}S=b[261216+(T<<5)+(O<<1)>>1]|0;W=(e[261216+(O-1<<5)+(T<<1)>>1]|0)+(V-R)|0;ag=(G|0)>0;if((a[ai+7|0]&1)==0){if(!ag){break L15346}ab=O-8|0;ad=E-7|0;ah=(ad|0)>0;ae=h+(f<<2)|0;Y=S&65535;X=Q;ak=U;as=0;while(1){al=as&7;if(ah){am=d[260704+(ab<<6)+(al<<3)|0]|0;an=d[260704+(ab<<6)+(al<<3)+1|0]|0;ao=d[260704+(ab<<6)+(al<<3)+2|0]|0;ap=d[260704+(ab<<6)+(al<<3)+3|0]|0;aq=d[260704+(ab<<6)+(al<<3)+4|0]|0;ar=d[260704+(ab<<6)+(al<<3)+5|0]|0;aj=d[260704+(ab<<6)+(al<<3)+6|0]|0;at=d[260704+(ab<<6)+(al<<3)+7|0]|0;aA=0;while(1){b[X+(aA<<1)>>1]=(ac(am+(e[ak+(aA<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;aB=aA|1;b[X+(aB<<1)>>1]=(ac(an+(e[ak+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;aB=aA|2;b[X+(aB<<1)>>1]=(ac(ao+(e[ak+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;aB=aA|3;b[X+(aB<<1)>>1]=(ac(ap+(e[ak+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;aB=aA|4;b[X+(aB<<1)>>1]=(ac(aq+(e[ak+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;aB=aA|5;b[X+(aB<<1)>>1]=(ac(ar+(e[ak+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;aB=aA|6;b[X+(aB<<1)>>1]=(ac(aj+(e[ak+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;aB=aA|7;b[X+(aB<<1)>>1]=(ac(at+(e[ak+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;aB=aA+8|0;if((aB|0)<(ad|0)){aA=aB}else{aC=aB;break}}}else{aC=0}if((aC|0)<(E|0)){aA=aC;do{b[X+(aA<<1)>>1]=(ac((d[(aA&7)+(260704+(ab<<6)+(al<<3))|0]|0)+(e[ak+(aA<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;aA=aA+1|0;}while((aA|0)<(E|0))}aA=as+1|0;if((aA|0)<(G|0)){X=X+(((c[K>>2]|0)/2|0)<<1)|0;ak=ak+(((c[ae>>2]|0)/2|0)<<1)|0;as=aA}else{break L15346}}}else{if(!ag){break L15346}as=O-8|0;ae=E-7|0;ak=(ae|0)>0;X=h+(f<<2)|0;Y=S&65535;ab=Q;ad=U;ah=0;while(1){aA=ah&7;if(ak){al=d[260704+(as<<6)+(aA<<3)|0]|0;at=d[260704+(as<<6)+(aA<<3)+1|0]|0;aj=d[260704+(as<<6)+(aA<<3)+2|0]|0;ar=d[260704+(as<<6)+(aA<<3)+3|0]|0;aq=d[260704+(as<<6)+(aA<<3)+4|0]|0;ap=d[260704+(as<<6)+(aA<<3)+5|0]|0;ao=d[260704+(as<<6)+(aA<<3)+6|0]|0;an=d[260704+(as<<6)+(aA<<3)+7|0]|0;am=0;while(1){aB=(ac(al+(e[ad+(am<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;b[ab+(am<<1)>>1]=(aB>>>8|aB<<8)&65535;aB=am|1;aD=(ac(at+(e[ad+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;b[ab+(aB<<1)>>1]=(aD>>>8|aD<<8)&65535;aD=am|2;aB=(ac(aj+(e[ad+(aD<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;b[ab+(aD<<1)>>1]=(aB>>>8|aB<<8)&65535;aB=am|3;aD=(ac(ar+(e[ad+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;b[ab+(aB<<1)>>1]=(aD>>>8|aD<<8)&65535;aD=am|4;aB=(ac(aq+(e[ad+(aD<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;b[ab+(aD<<1)>>1]=(aB>>>8|aB<<8)&65535;aB=am|5;aD=(ac(ap+(e[ad+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;b[ab+(aB<<1)>>1]=(aD>>>8|aD<<8)&65535;aD=am|6;aB=(ac(ao+(e[ad+(aD<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;b[ab+(aD<<1)>>1]=(aB>>>8|aB<<8)&65535;aB=am|7;aD=(ac(an+(e[ad+(aB<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;b[ab+(aB<<1)>>1]=(aD>>>8|aD<<8)&65535;aD=am+8|0;if((aD|0)<(ae|0)){am=aD}else{aE=aD;break}}}else{aE=0}if((aE|0)<(E|0)){am=aE;do{an=(ac((d[(am&7)+(260704+(as<<6)+(aA<<3))|0]|0)+(e[ad+(am<<1)>>1]|0)|0,Y)|0)>>>(W>>>0)&65535;b[ab+(am<<1)>>1]=(an>>>8|an<<8)&65535;am=am+1|0;}while((am|0)<(E|0))}am=ah+1|0;if((am|0)<(G|0)){ab=ab+(((c[K>>2]|0)/2|0)<<1)|0;ad=ad+(((c[X>>2]|0)/2|0)<<1)|0;ah=am}else{break L15346}}}}else{if(af){P=11381;break L15332}ah=b[261216+(T<<5)+(O<<1)>>1]|0;X=(e[261216+(O-1<<5)+(T<<1)>>1]|0)+(V-R)|0;ad=(G|0)>0;if((a[ai+7|0]&1)==0){if(!ad){break L15346}ab=O-8|0;W=E-7|0;Y=(W|0)>0;as=h+(f<<2)|0;ae=ah&65535;ak=Q;S=U;ag=0;while(1){am=ag&7;if(Y){aA=d[260704+(ab<<6)+(am<<3)|0]|0;an=d[260704+(ab<<6)+(am<<3)+1|0]|0;ao=d[260704+(ab<<6)+(am<<3)+2|0]|0;ap=d[260704+(ab<<6)+(am<<3)+3|0]|0;aq=d[260704+(ab<<6)+(am<<3)+4|0]|0;ar=d[260704+(ab<<6)+(am<<3)+5|0]|0;aj=d[260704+(ab<<6)+(am<<3)+6|0]|0;at=d[260704+(ab<<6)+(am<<3)+7|0]|0;al=0;while(1){b[ak+(al<<1)>>1]=(ac(aA+((cT(b[S+(al<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;aD=al|1;b[ak+(aD<<1)>>1]=(ac(an+((cT(b[S+(aD<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;aD=al|2;b[ak+(aD<<1)>>1]=(ac(ao+((cT(b[S+(aD<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;aD=al|3;b[ak+(aD<<1)>>1]=(ac(ap+((cT(b[S+(aD<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;aD=al|4;b[ak+(aD<<1)>>1]=(ac(aq+((cT(b[S+(aD<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;aD=al|5;b[ak+(aD<<1)>>1]=(ac(ar+((cT(b[S+(aD<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;aD=al|6;b[ak+(aD<<1)>>1]=(ac(aj+((cT(b[S+(aD<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;aD=al|7;b[ak+(aD<<1)>>1]=(ac(at+((cT(b[S+(aD<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;aD=al+8|0;if((aD|0)<(W|0)){al=aD}else{aF=aD;break}}}else{aF=0}if((aF|0)<(E|0)){al=aF;do{at=(cT(b[S+(al<<1)>>1]|0)|0)&65535;b[ak+(al<<1)>>1]=(ac((d[(al&7)+(260704+(ab<<6)+(am<<3))|0]|0)+at|0,ae)|0)>>>(X>>>0)&65535;al=al+1|0;}while((al|0)<(E|0))}al=ag+1|0;if((al|0)<(G|0)){ak=ak+(((c[K>>2]|0)/2|0)<<1)|0;S=S+(((c[as>>2]|0)/2|0)<<1)|0;ag=al}else{break L15346}}}else{if(!ad){break L15346}ag=O-8|0;as=E-7|0;S=(as|0)>0;ak=h+(f<<2)|0;ae=ah&65535;ab=Q;W=U;Y=0;while(1){ai=Y&7;if(S){T=d[260704+(ag<<6)+(ai<<3)|0]|0;af=d[260704+(ag<<6)+(ai<<3)+1|0]|0;al=d[260704+(ag<<6)+(ai<<3)+2|0]|0;am=d[260704+(ag<<6)+(ai<<3)+3|0]|0;at=d[260704+(ag<<6)+(ai<<3)+4|0]|0;aj=d[260704+(ag<<6)+(ai<<3)+5|0]|0;ar=d[260704+(ag<<6)+(ai<<3)+6|0]|0;aq=d[260704+(ag<<6)+(ai<<3)+7|0]|0;ap=0;while(1){ao=(ac(T+((cT(b[W+(ap<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;b[ab+(ap<<1)>>1]=(ao>>>8|ao<<8)&65535;ao=ap|1;an=(ac(af+((cT(b[W+(ao<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;b[ab+(ao<<1)>>1]=(an>>>8|an<<8)&65535;an=ap|2;ao=(ac(al+((cT(b[W+(an<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;b[ab+(an<<1)>>1]=(ao>>>8|ao<<8)&65535;ao=ap|3;an=(ac(am+((cT(b[W+(ao<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;b[ab+(ao<<1)>>1]=(an>>>8|an<<8)&65535;an=ap|4;ao=(ac(at+((cT(b[W+(an<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;b[ab+(an<<1)>>1]=(ao>>>8|ao<<8)&65535;ao=ap|5;an=(ac(aj+((cT(b[W+(ao<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;b[ab+(ao<<1)>>1]=(an>>>8|an<<8)&65535;an=ap|6;ao=(ac(ar+((cT(b[W+(an<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;b[ab+(an<<1)>>1]=(ao>>>8|ao<<8)&65535;ao=ap|7;an=(ac(aq+((cT(b[W+(ao<<1)>>1]|0)|0)&65535)|0,ae)|0)>>>(X>>>0)&65535;b[ab+(ao<<1)>>1]=(an>>>8|an<<8)&65535;an=ap+8|0;if((an|0)<(as|0)){ap=an}else{aG=an;break}}}else{aG=0}if((aG|0)<(E|0)){ap=aG;do{aq=(cT(b[W+(ap<<1)>>1]|0)|0)&65535;ar=(ac((d[(ap&7)+(260704+(ag<<6)+(ai<<3))|0]|0)+aq|0,ae)|0)>>>(X>>>0)&65535;b[ab+(ap<<1)>>1]=(ar>>>8|ar<<8)&65535;ap=ap+1|0;}while((ap|0)<(E|0))}ap=Y+1|0;if((ap|0)<(G|0)){ab=ab+(((c[K>>2]|0)/2|0)<<1)|0;W=W+(((c[ak>>2]|0)/2|0)<<1)|0;Y=ap}else{break L15346}}}}}else{if((G|0)<=0){break L15346}Y=N^1;ak=R-V|0;W=E-1|0;ab=(W|0)>0;X=E&1;ae=(V<<1)-R|0;ag=h+(f<<2)|0;as=Q;S=U;ah=E;O=0;while(1){ad=BP(c[o>>2]|0)|0;if((ad|0)==0){P=11318;break L15332}do{if((a[ad+7|0]&1)==0){ap=BP(c[q>>2]|0)|0;if((ap|0)==0){P=11321;break L15332}if((a[ap+7|0]&1)!=0|Y){aH=ah;break}if((ak|0)==6){if(ab){aI=0}else{aH=X;break}while(1){c[as+(aI<<1)>>2]=c[S+(aI<<1)>>2]<<6;ap=aI+2|0;if((ap|0)<(W|0)){aI=ap}else{aH=X;break}}}else if((ak|0)==7){if(ab){aJ=0}else{aH=X;break}while(1){c[as+(aJ<<1)>>2]=c[S+(aJ<<1)>>2]<<7;ap=aJ+2|0;if((ap|0)<(W|0)){aJ=ap}else{aH=X;break}}}else{aH=E;break}}else{aH=ah}}while(0);ad=BP(c[o>>2]|0)|0;if((ad|0)==0){P=11329;break L15332}ap=(a[ad+7|0]&1)==0;ad=BP(c[q>>2]|0)|0;ai=(ad|0)==0;do{if(ap){if(ai){P=11345;break L15332}ar=(aH|0)>0;if((a[ad+7|0]&1)==0){if(N){if(ar){aK=0}else{break}do{aq=S+(aK<<1)|0;aj=as+(aK<<1)|0;y=((d[aq]|d[aq+1|0]<<8)<<16>>16&65535)<<ak&65535;a[aj]=y&255;y=y>>8;a[aj+1|0]=y&255;aK=aK+1|0;}while((aK|0)<(aH|0))}else{if(ar){aL=0}else{break}do{aj=S+(aL<<1)|0;aq=(d[aj]|d[aj+1|0]<<8)<<16>>16&65535;aj=as+(aL<<1)|0;y=(aq<<ak|aq>>>(ae>>>0))&65535;a[aj]=y&255;y=y>>8;a[aj+1|0]=y&255;aL=aL+1|0;}while((aL|0)<(aH|0))}}else{if(N){if(ar){aM=0}else{break}do{aj=S+(aM<<1)|0;aq=((d[aj]|d[aj+1|0]<<8)<<16>>16&65535)<<ak&65535;aj=as+(aM<<1)|0;y=(aq>>>8|aq<<8)&65535;a[aj]=y&255;y=y>>8;a[aj+1|0]=y&255;aM=aM+1|0;}while((aM|0)<(aH|0))}else{if(ar){aN=0}else{break}do{aj=S+(aN<<1)|0;aq=(d[aj]|d[aj+1|0]<<8)<<16>>16&65535;aj=(aq<<ak|aq>>>(ae>>>0))&65535;aq=as+(aN<<1)|0;y=(aj>>>8|aj<<8)&65535;a[aq]=y&255;y=y>>8;a[aq+1|0]=y&255;aN=aN+1|0;}while((aN|0)<(aH|0))}}}else{if(ai){P=11332;break L15332}ar=(aH|0)>0;if((a[ad+7|0]&1)==0){if(N){if(ar){aO=0}else{break}do{aq=S+(aO<<1)|0;aj=as+(aO<<1)|0;y=((cT((d[aq]|d[aq+1|0]<<8)<<16>>16|0)|0)&65535)<<ak&65535;a[aj]=y&255;y=y>>8;a[aj+1|0]=y&255;aO=aO+1|0;}while((aO|0)<(aH|0))}else{if(ar){aP=0}else{break}do{aj=S+(aP<<1)|0;aq=(cT((d[aj]|d[aj+1|0]<<8)<<16>>16|0)|0)&65535;aj=as+(aP<<1)|0;y=(aq<<ak|aq>>>(ae>>>0))&65535;a[aj]=y&255;y=y>>8;a[aj+1|0]=y&255;aP=aP+1|0;}while((aP|0)<(aH|0))}}else{if(N){if(ar){aQ=0}else{break}do{aj=S+(aQ<<1)|0;aq=((cT((d[aj]|d[aj+1|0]<<8)<<16>>16|0)|0)&65535)<<ak&65535;aj=as+(aQ<<1)|0;y=(aq>>>8|aq<<8)&65535;a[aj]=y&255;y=y>>8;a[aj+1|0]=y&255;aQ=aQ+1|0;}while((aQ|0)<(aH|0))}else{if(ar){aR=0}else{break}do{aj=S+(aR<<1)|0;aq=(cT((d[aj]|d[aj+1|0]<<8)<<16>>16|0)|0)&65535;aj=(aq<<ak|aq>>>(ae>>>0))&65535;aq=as+(aR<<1)|0;y=(aj>>>8|aj<<8)&65535;a[aq]=y&255;y=y>>8;a[aq+1|0]=y&255;aR=aR+1|0;}while((aR|0)<(aH|0))}}}}while(0);ad=O+1|0;if((ad|0)<(G|0)){as=as+(((c[K>>2]|0)/2|0)<<1)|0;S=S+(((c[ag>>2]|0)/2|0)<<1)|0;ah=aH;O=ad}else{break L15346}}}}}while(0);O=BP(c[q>>2]|0)|0;if((O|0)==0){P=11245;break L15332}do{if((b[O+8>>1]&30720)!=30720){ah=BP(c[q>>2]|0)|0;if((ah|0)==0){P=11248;break L15332}if((((e[ah+8>>1]|0)>>>11&15)-8&65535)<6){break}ah=c[K>>2]|0;ag=C?-1:-128;if((G|0)<=0){break L15346}S=c[I>>2]|0;as=0;ae=S+(ac(ah,F)|0)|0;while(1){NM(ae|0,ag|0,E|0);S=as+1|0;if((S|0)<(G|0)){as=S;ae=ae+ah|0}else{break L15346}}}}while(0);O=c[I>>2]|0;ah=c[K>>2]|0;ae=b[r+8+(f<<1)>>1]|0;as=BP(c[q>>2]|0)|0;if((as|0)==0){P=11251;break L15332}ag=(ae&65535)>>>11&15;ae=a[as+7|0]|0;as=O+(ac(ah,F)|0)|0;if(C){aS=65535>>>((ag^15)>>>0)&65535}else{aS=1<<ag&65535}if((G|0)<=0){break}ag=(ae&1)==0;ae=(E|0)>0;O=cT(aS|0)|0;S=as;as=0;while(1){do{if(ag){if(ae){aT=0}else{break}do{ak=S+(aT<<1)|0;y=aS;a[ak]=y&255;y=y>>8;a[ak+1|0]=y&255;aT=aT+1|0;}while((aT|0)<(E|0))}else{if(ae){aU=0}else{break}do{ak=S+(aU<<1)|0;y=O;a[ak]=y&255;y=y>>8;a[ak+1|0]=y&255;aU=aU+1|0;}while((aU|0)<(E|0))}}while(0);ak=as+1|0;if((ak|0)<(G|0)){S=S+ah|0;as=ak}else{break}}}}while(0);C=f+1|0;if((C|0)<4){f=C}else{P=11429;break}}if((P|0)==11275){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=619,aV)|0);i=aV;cw();return 0}else if((P|0)==11267){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=626,aV)|0);i=aV;cw();return 0}else if((P|0)==11321){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11248){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=626,aV)|0);i=aV;cw();return 0}else if((P|0)==11362){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11400){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=619,aV)|0);i=aV;cw();return 0}else if((P|0)==11403){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=619,aV)|0);i=aV;cw();return 0}else if((P|0)==11419){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=619,aV)|0);i=aV;cw();return 0}else if((P|0)==11332){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11429){i=n;return k|0}else if((P|0)==11381){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11359){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11406){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11408){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11273){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=619,aV)|0);i=aV;cw();return 0}else if((P|0)==11251){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11422){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=619,aV)|0);i=aV;cw();return 0}else if((P|0)==11301){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11318){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11345){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11279){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11270){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=626,aV)|0);i=aV;cw();return 0}else if((P|0)==11329){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=635,aV)|0);i=aV;cw();return 0}else if((P|0)==11245){AN(0,0,1639504,(aV=i,i=i+24|0,c[aV>>2]=1718632,c[aV+8>>2]=1690888,c[aV+16>>2]=619,aV)|0);i=aV;cw();return 0}return 0}function w4(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0;if((e|0)<=0){return}g=f;f=b;b=0;do{c[f+(b<<2)>>2]=c[g+((d[a+b|0]|0)<<2)>>2];b=b+1|0;}while((b|0)<(e|0));return}function w5(b,c,e,f){b=b|0;c=c|0;e=e|0;f=f|0;var g=0,h=0;if((e|0)>0){g=c;h=0}else{return}while(1){c=b+h|0;a[g]=a[f+((d[c]|0)<<2)|0]|0;a[g+1|0]=a[f+((d[c]|0)<<2|1)|0]|0;a[g+2|0]=a[f+((d[c]|0)<<2|2)|0]|0;c=h+1|0;if((c|0)<(e|0)){g=g+3|0;h=c}else{break}}return}function w6(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0;if((e|0)<=0){return}g=f;f=b;b=0;do{h=b<<1;c[f+(b<<2)>>2]=(d[a+(h|1)|0]|0)<<24|c[g+((d[a+h|0]|0)<<2)>>2];b=b+1|0;}while((b|0)<(e|0));return}function w7(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0;if((e|0)<=0){return}g=f;f=b;b=0;do{h=b<<1;c[f+(b<<2)>>2]=d[a+(h|1)|0]|0|c[g+((d[a+h|0]|0)<<2)>>2];b=b+1|0;}while((b|0)<(e|0));return}function w8(b,c,e,f){b=b|0;c=c|0;e=e|0;f=f|0;var g=0,h=0;if((e|0)>0){g=c;h=0}else{return}while(1){c=b+(h<<1)|0;a[g]=a[f+((d[c]|0)<<2)|0]|0;a[g+1|0]=a[f+((d[c]|0)<<2|1)|0]|0;a[g+2|0]=a[f+((d[c]|0)<<2|2)|0]|0;c=h+1|0;if((c|0)<(e|0)){g=g+3|0;h=c}else{break}}return}function w9(a,d,f,g,h,i,j,k,l){a=a|0;d=d|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0;m=16-k|0;n=(k<<1)-16|0;if((h|0)<=0){return}k=(i|0)==0;i=(l|0)>0;o=a+4|0;p=a+8|0;q=0;do{r=f+(ac(q,g)|0)|0;L15663:do{if((j|0)==2){if(k){if(i){s=0;t=r}else{break}while(1){u=e[(c[a>>2]|0)+(s<<1)>>1]|0;v=(u<<m|u>>>(n>>>0))&65535;b[t>>1]=(v>>>8|v<<8)&65535;v=e[(c[o>>2]|0)+(s<<1)>>1]|0;u=(v<<m|v>>>(n>>>0))&65535;b[t+2>>1]=(u>>>8|u<<8)&65535;u=e[(c[p>>2]|0)+(s<<1)>>1]|0;v=(u<<m|u>>>(n>>>0))&65535;b[t+4>>1]=(v>>>8|v<<8)&65535;v=s+1|0;if((v|0)>=(l|0)){break L15663}s=v;t=t+6|0}}else{if(i){w=0;x=r}else{break}while(1){v=e[(c[a>>2]|0)+(w<<1)>>1]|0;u=(v<<m|v>>>(n>>>0))&65535;b[x>>1]=(u>>>8|u<<8)&65535;u=e[(c[o>>2]|0)+(w<<1)>>1]|0;v=(u<<m|u>>>(n>>>0))&65535;b[x+2>>1]=(v>>>8|v<<8)&65535;v=e[(c[p>>2]|0)+(w<<1)>>1]|0;u=(v<<m|v>>>(n>>>0))&65535;b[x+4>>1]=(u>>>8|u<<8)&65535;b[x+6>>1]=-1;u=w+1|0;if((u|0)>=(l|0)){break L15663}w=u;x=x+8|0}}}else if((j|0)==3){if(k){if(i){y=0;z=r}else{break}while(1){u=(cT(b[(c[a>>2]|0)+(y<<1)>>1]|0)|0)&65535;v=(u<<m|u>>>(n>>>0))&65535;b[z>>1]=(v>>>8|v<<8)&65535;v=(cT(b[(c[o>>2]|0)+(y<<1)>>1]|0)|0)&65535;u=(v<<m|v>>>(n>>>0))&65535;b[z+2>>1]=(u>>>8|u<<8)&65535;u=(cT(b[(c[p>>2]|0)+(y<<1)>>1]|0)|0)&65535;v=(u<<m|u>>>(n>>>0))&65535;b[z+4>>1]=(v>>>8|v<<8)&65535;v=y+1|0;if((v|0)>=(l|0)){break L15663}y=v;z=z+6|0}}else{if(i){A=0;B=r}else{break}while(1){v=(cT(b[(c[a>>2]|0)+(A<<1)>>1]|0)|0)&65535;u=(v<<m|v>>>(n>>>0))&65535;b[B>>1]=(u>>>8|u<<8)&65535;u=(cT(b[(c[o>>2]|0)+(A<<1)>>1]|0)|0)&65535;v=(u<<m|u>>>(n>>>0))&65535;b[B+2>>1]=(v>>>8|v<<8)&65535;v=(cT(b[(c[p>>2]|0)+(A<<1)>>1]|0)|0)&65535;u=(v<<m|v>>>(n>>>0))&65535;b[B+4>>1]=(u>>>8|u<<8)&65535;b[B+6>>1]=-1;u=A+1|0;if((u|0)>=(l|0)){break L15663}A=u;B=B+8|0}}}else if((j|0)==1){if(k){if(i){C=0;D=r}else{break}while(1){u=(cT(b[(c[a>>2]|0)+(C<<1)>>1]|0)|0)&65535;b[D>>1]=(u<<m|u>>>(n>>>0))&65535;u=(cT(b[(c[o>>2]|0)+(C<<1)>>1]|0)|0)&65535;b[D+2>>1]=(u<<m|u>>>(n>>>0))&65535;u=(cT(b[(c[p>>2]|0)+(C<<1)>>1]|0)|0)&65535;b[D+4>>1]=(u<<m|u>>>(n>>>0))&65535;u=C+1|0;if((u|0)>=(l|0)){break L15663}C=u;D=D+6|0}}else{if(i){E=0;F=r}else{break}while(1){u=(cT(b[(c[a>>2]|0)+(E<<1)>>1]|0)|0)&65535;b[F>>1]=(u<<m|u>>>(n>>>0))&65535;u=(cT(b[(c[o>>2]|0)+(E<<1)>>1]|0)|0)&65535;b[F+2>>1]=(u<<m|u>>>(n>>>0))&65535;u=(cT(b[(c[p>>2]|0)+(E<<1)>>1]|0)|0)&65535;b[F+4>>1]=(u<<m|u>>>(n>>>0))&65535;b[F+6>>1]=-1;u=E+1|0;if((u|0)>=(l|0)){break L15663}E=u;F=F+8|0}}}else{if(k){if(i){G=0;H=r}else{break}while(1){u=e[(c[a>>2]|0)+(G<<1)>>1]|0;b[H>>1]=(u<<m|u>>>(n>>>0))&65535;u=e[(c[o>>2]|0)+(G<<1)>>1]|0;b[H+2>>1]=(u<<m|u>>>(n>>>0))&65535;u=e[(c[p>>2]|0)+(G<<1)>>1]|0;b[H+4>>1]=(u<<m|u>>>(n>>>0))&65535;u=G+1|0;if((u|0)>=(l|0)){break L15663}G=u;H=H+6|0}}else{if(i){I=0;J=r}else{break}while(1){u=e[(c[a>>2]|0)+(I<<1)>>1]|0;b[J>>1]=(u<<m|u>>>(n>>>0))&65535;u=e[(c[o>>2]|0)+(I<<1)>>1]|0;b[J+2>>1]=(u<<m|u>>>(n>>>0))&65535;u=e[(c[p>>2]|0)+(I<<1)>>1]|0;b[J+4>>1]=(u<<m|u>>>(n>>>0))&65535;b[J+6>>1]=-1;u=I+1|0;if((u|0)>=(l|0)){break L15663}I=u;J=J+8|0}}}}while(0);c[a>>2]=(c[a>>2]|0)+(c[d>>2]>>1<<1);c[o>>2]=(c[o>>2]|0)+(c[d+4>>2]>>1<<1);c[p>>2]=(c[p>>2]|0)+(c[d+8>>2]>>1<<1);q=q+1|0;}while((q|0)<(h|0));return}function xa(d){d=d|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0;f=i;g=c[d+56>>2]|0;h=d+23880|0;c[h>>2]=0;switch(g|0){case 83:{c[d+23888>>2]=68;break};case 1:{c[h>>2]=48;break};case 26:{c[h>>2]=2;break};case 86:{c[d+23888>>2]=46;break};case 22:case 19:case 11:case 21:case 24:{c[h>>2]=134;break};case 85:{c[d+23888>>2]=30;break};case 313:{c[d+23888>>2]=74;break};case 75:case 79:case 69:case 77:case 73:case 71:case 309:case 305:case 301:case 311:case 307:case 303:case 55:case 57:case 59:case 95:case 93:case 91:case 101:case 99:case 97:case 103:case 105:case 107:{c[h>>2]=24;break};case 84:{c[d+23888>>2]=104;break};case 25:{c[h>>2]=58;break};case 319:case 88:{c[d+23888>>2]=116;break};case 17:{c[h>>2]=64;break};case 315:{c[d+23888>>2]=118;break};case 318:case 87:{c[d+23888>>2]=26;break};case 317:case 82:{c[d+23888>>2]=2;break};case 314:{c[d+23888>>2]=54;break};case 316:{c[d+23888>>2]=96;break};default:{}}L15726:do{if((c[d+76>>2]|0)==0){switch(g|0){case 47:{j=126;k=11517;break};case 68:{l=98;k=11536;break};case 2:{m=88;k=11522;break};case 49:{n=62;k=11519;break};case 67:{o=22;k=11535;break};case 29:{p=112;k=11530;break};case 50:{q=150;k=11518;break};case 44:{r=110;k=11523;break};case 3:{s=80;k=11515;break};case 41:{t=12;k=11533;break};case 28:{u=102;k=11531;break};case 64:{v=146;k=11520;break};case 63:{w=50;k=11528;break};case 30:{x=118;k=11529;break};case 65:{y=8;k=11521;break};case 42:{z=10;k=11534;break};case 27:{A=116;k=11532;break};case 292:{B=70;k=11538;break};case 291:{C=44;k=11537;break};case 45:{D=52;k=11526;break};case 62:{E=30;k=11527;break};case 48:{F=148;k=11516;break};case 43:{G=108;k=11524;break};case 46:{H=34;k=11525;break};default:{k=11539}}}else{switch(g|0){case 47:{j=46;k=11517;break L15726;break};case 68:{l=100;k=11536;break L15726;break};case 2:{m=68;k=11522;break L15726;break};case 49:{n=104;k=11519;break L15726;break};case 67:{o=4;k=11535;break L15726;break};case 29:{p=92;k=11530;break L15726;break};case 50:{q=138;k=11518;break L15726;break};case 44:{r=38;k=11523;break L15726;break};case 3:{s=54;k=11515;break L15726;break};case 41:{t=76;k=11533;break L15726;break};case 28:{u=96;k=11531;break L15726;break};case 64:{v=122;k=11520;break L15726;break};case 317:case 82:{break};case 63:{w=18;k=11528;break L15726;break};case 30:{x=106;k=11529;break L15726;break};case 65:{y=82;k=11521;break L15726;break};case 42:{z=28;k=11534;break L15726;break};case 27:{A=144;k=11532;break L15726;break};case 292:{B=132;k=11538;break L15726;break};case 291:{C=16;k=11537;break L15726;break};case 45:{D=36;k=11526;break L15726;break};case 62:{E=114;k=11527;break L15726;break};case 48:{F=130;k=11516;break L15726;break};case 43:{G=86;k=11524;break L15726;break};case 46:{H=124;k=11525;break L15726;break};default:{k=11539;break L15726}}c[h>>2]=84;k=11539}}while(0);L15730:do{if((k|0)==11517){c[h>>2]=j;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;K=I;L=J;k=11558}else if((k|0)==11536){c[h>>2]=l;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;M=J;N=I;k=11580}else if((k|0)==11522){c[h>>2]=m;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;O=I;P=J;k=11563}else if((k|0)==11519){c[h>>2]=n;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;Q=J;R=I;k=11560}else if((k|0)==11539){I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;switch(g|0){case 319:case 88:{c[d+23884>>2]=72;S=J;break L15730;break};case 83:{c[d+23884>>2]=76;S=J;break L15730;break};case 291:{T=I;U=J;k=11581;break L15730;break};case 292:{V=I;W=J;k=11582;break L15730;break};case 22:case 19:case 11:case 21:case 24:{c[I>>2]=64;S=J;break L15730;break};case 10:{c[I>>2]=70;S=J;break L15730;break};case 9:{c[I>>2]=58;S=J;break L15730;break};case 3:{X=I;Y=J;k=11556;break L15730;break};case 48:{Z=I;_=J;k=11557;break L15730;break};case 47:{K=I;L=J;k=11558;break L15730;break};case 50:{$=I;aa=J;k=11559;break L15730;break};case 49:{Q=I;R=J;k=11560;break L15730;break};case 64:{ab=I;ac=J;k=11561;break L15730;break};case 65:{ad=I;ae=J;k=11562;break L15730;break};case 2:{O=I;P=J;k=11563;break L15730;break};case 44:{af=I;ag=J;k=11564;break L15730;break};case 43:{ah=I;ai=J;k=11565;break L15730;break};case 46:{aj=I;ak=J;k=11566;break L15730;break};case 45:{al=I;am=J;k=11567;break L15730;break};case 62:{an=I;ao=J;k=11568;break L15730;break};case 63:{ap=I;aq=J;k=11569;break L15730;break};case 30:{ar=I;as=J;k=11573;break L15730;break};case 29:{at=I;au=J;k=11574;break L15730;break};case 28:{av=I;aw=J;k=11575;break L15730;break};case 27:{ax=I;ay=J;k=11576;break L15730;break};case 317:{c[d+23892>>2]=34;break};case 82:{break};case 314:{c[d+23884>>2]=56;S=J;break L15730;break};case 316:{c[d+23884>>2]=68;S=J;break L15730;break};case 85:{c[d+23884>>2]=50;S=J;break L15730;break};case 313:{c[d+23884>>2]=36;S=J;break L15730;break};case 75:case 79:case 69:case 77:case 73:case 71:case 309:case 305:case 301:case 311:case 307:case 303:case 55:case 57:case 59:case 31:{c[I>>2]=48;S=J;break L15730;break};case 95:case 93:case 91:case 101:case 99:case 97:case 103:case 105:case 107:{c[I>>2]=48;c[J>>2]=48;S=J;break L15730;break};case 1:case 66:{c[I>>2]=82;S=J;break L15730;break};case 17:{c[I>>2]=76;S=J;break L15730;break};case 315:{c[d+23884>>2]=62;S=J;break L15730;break};case 318:case 87:{c[d+23884>>2]=48;S=J;break L15730;break};case 86:{c[d+23884>>2]=20;S=J;break L15730;break};case 84:{c[d+23884>>2]=88;S=J;break L15730;break};case 41:{az=I;aA=J;k=11577;break L15730;break};case 42:{aB=I;aC=J;k=11578;break L15730;break};case 67:{aD=I;aE=J;k=11579;break L15730;break};case 68:{M=I;N=J;k=11580;break L15730;break};default:{S=J;break L15730}}c[d+23884>>2]=80;S=J}else if((k|0)==11535){c[h>>2]=o;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;aD=J;aE=I;k=11579}else if((k|0)==11530){c[h>>2]=p;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;at=I;au=J;k=11574}else if((k|0)==11518){c[h>>2]=q;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;$=J;aa=I;k=11559}else if((k|0)==11523){c[h>>2]=r;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;af=I;ag=J;k=11564}else if((k|0)==11515){c[h>>2]=s;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;X=J;Y=I;k=11556}else if((k|0)==11533){c[h>>2]=t;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;az=I;aA=J;k=11577}else if((k|0)==11531){c[h>>2]=u;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;av=J;aw=I;k=11575}else if((k|0)==11520){c[h>>2]=v;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;ab=I;ac=J;k=11561}else if((k|0)==11528){c[h>>2]=w;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;ap=J;aq=I;k=11569}else if((k|0)==11529){c[h>>2]=x;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;ar=I;as=J;k=11573}else if((k|0)==11521){c[h>>2]=y;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;ad=J;ae=I;k=11562}else if((k|0)==11534){c[h>>2]=z;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;aB=I;aC=J;k=11578}else if((k|0)==11532){c[h>>2]=A;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;ax=J;ay=I;k=11576}else if((k|0)==11538){c[h>>2]=B;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;V=I;W=J;k=11582}else if((k|0)==11537){c[h>>2]=C;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;T=J;U=I;k=11581}else if((k|0)==11526){c[h>>2]=D;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;al=I;am=J;k=11567}else if((k|0)==11527){c[h>>2]=E;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;an=J;ao=I;k=11568}else if((k|0)==11516){c[h>>2]=F;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;Z=I;_=J;k=11557}else if((k|0)==11524){c[h>>2]=G;J=d+23872|0;c[J>>2]=0;I=d+23876|0;c[I>>2]=0;ah=J;ai=I;k=11565}else if((k|0)==11525){c[h>>2]=H;I=d+23872|0;c[I>>2]=0;J=d+23876|0;c[J>>2]=0;aj=I;ak=J;k=11566}}while(0);if((k|0)==11581){c[T>>2]=78;S=U}else if((k|0)==11582){c[V>>2]=2;S=W}else if((k|0)==11556){c[X>>2]=30;S=Y}else if((k|0)==11557){c[Z>>2]=68;S=_}else if((k|0)==11558){c[K>>2]=36;S=L}else if((k|0)==11559){c[$>>2]=56;S=aa}else if((k|0)==11560){c[Q>>2]=16;S=R}else if((k|0)==11561){c[ab>>2]=14;S=ac}else if((k|0)==11562){c[ad>>2]=4;S=ae}else if((k|0)==11563){c[O>>2]=74;S=P}else if((k|0)==11564){c[af>>2]=12;S=ag}else if((k|0)==11565){c[ah>>2]=52;S=ai}else if((k|0)==11566){c[aj>>2]=6;S=ak}else if((k|0)==11567){c[al>>2]=84;S=am}else if((k|0)==11568){c[an>>2]=40;S=ao}else if((k|0)==11569){c[ap>>2]=10;S=aq}else if((k|0)==11573){c[ar>>2]=42;S=as}else if((k|0)==11574){c[at>>2]=32;S=au}else if((k|0)==11575){c[av>>2]=50;S=aw}else if((k|0)==11576){c[ax>>2]=66;S=ay}else if((k|0)==11577){c[az>>2]=72;S=aA}else if((k|0)==11578){c[aB>>2]=38;S=aC}else if((k|0)==11579){c[aD>>2]=62;S=aE}else if((k|0)==11580){c[M>>2]=60;S=N}if((c[d+2180>>2]|0)==0){i=f;return}d=BP(g)|0;if((d|0)==0){AN(0,0,1639344,(aF=i,i=i+24|0,c[aF>>2]=1718488,c[aF+8>>2]=1690768,c[aF+16>>2]=619,aF)|0);i=aF;cw()}do{if((b[d+8>>1]&30720)==30720){k=11590}else{N=BP(g)|0;if((N|0)==0){AN(0,0,1639344,(aF=i,i=i+24|0,c[aF>>2]=1718488,c[aF+8>>2]=1690768,c[aF+16>>2]=626,aF)|0);i=aF;cw()}else{if((((e[N+8>>1]|0)>>>11&15)-8&65535)<6){k=11590;break}else{break}}}}while(0);do{if((k|0)==11590){d=BP(g)|0;if((d|0)==0){AN(0,0,1639344,(aF=i,i=i+24|0,c[aF>>2]=1718488,c[aF+8>>2]=1690768,c[aF+16>>2]=635,aF)|0);i=aF;cw()}if((a[d+7|0]&1)==0){break}c[S>>2]=48}}while(0);switch(g|0){case 29:case 27:{c[S>>2]=80;i=f;return};case 66:{c[S>>2]=76;i=f;return};case 11:{c[S>>2]=18;i=f;return};case 292:case 291:{c[S>>2]=46;i=f;return};case 30:case 28:{c[S>>2]=34;i=f;return};default:{i=f;return}}}function xb(b,c,d,e,f,g,h){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;if((g|0)>0){i=0}else{return}do{h=i<<2;a[b+i|0]=a[e+(h|1)|0]|0;a[c+i|0]=a[e+(h|3)|0]|0;i=i+1|0;}while((i|0)<(g|0));return}function xc(b,c,d,e,f,g,h){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;if((g|0)>0){i=0}else{return}do{h=i<<2;a[b+i|0]=a[e+h|0]|0;a[c+i|0]=a[e+(h|2)|0]|0;i=i+1|0;}while((i|0)<(g|0));return}function xd(b,c,d,e,f,g,h){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;if((g|0)>0){i=0}else{return}do{h=i<<1;a[b+i|0]=a[e+h|0]|0;a[c+i|0]=a[e+(h|1)|0]|0;i=i+1|0;}while((i|0)<(g|0));return}function xe(b,c,d,e,f,g,h){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;if((g|0)>0){i=0}else{return}do{h=i<<1;a[c+i|0]=a[e+h|0]|0;a[b+i|0]=a[e+(h|1)|0]|0;i=i+1|0;}while((i|0)<(g|0));return}function xf(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0;h=a;a=e;if((i|0)>0){k=0}else{return}do{e=c[j+((d[g+k|0]|0)<<2)>>2]|0;b[h+(k<<1)>>1]=e>>>8<<6&16320;b[a+(k<<1)>>1]=e>>>16<<6&16320;k=k+1|0;}while((k|0)<(i|0));return}function xg(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=(c[f>>2]|0)+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=(c[h>>2]|0)+(p<<1)|0;s=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=(c[o>>2]|0)+(p<<1)|0;t=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=ac(t,e)|0;u=ac(r,j)|0;b[i+(p<<1)>>1]=(u+8421376+(ac(s,k)|0)+q|0)>>>10&65535;q=ac(t,l)|0;t=ac(r,m)|0;b[a+(p<<1)>>1]=(t+8421376+(ac(s,n)|0)+q|0)>>>10&65535;p=p+1|0;}while((p|0)<(g|0));return}function xh(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=(c[f>>2]|0)+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=(c[h>>2]|0)+(p<<1)|0;s=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=(c[o>>2]|0)+(p<<1)|0;t=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=ac(t,e)|0;u=ac(r,j)|0;b[i+(p<<1)>>1]=(u+16842752+(ac(s,k)|0)+q|0)>>>11&65535;q=ac(t,l)|0;t=ac(r,m)|0;b[a+(p<<1)>>1]=(t+16842752+(ac(s,n)|0)+q|0)>>>11&65535;p=p+1|0;}while((p|0)<(g|0));return}function xi(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=(c[f>>2]|0)+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=(c[h>>2]|0)+(p<<1)|0;s=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=(c[o>>2]|0)+(p<<1)|0;t=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=ac(t,e)|0;u=ac(r,j)|0;b[i+(p<<1)>>1]=(u+67371008+(ac(s,k)|0)+q|0)>>>13&65535;q=ac(t,l)|0;t=ac(r,m)|0;b[a+(p<<1)>>1]=(t+67371008+(ac(s,n)|0)+q|0)>>>13&65535;p=p+1|0;}while((p|0)<(g|0));return}function xj(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=(c[f>>2]|0)+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=(c[h>>2]|0)+(p<<1)|0;s=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=(c[o>>2]|0)+(p<<1)|0;t=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=ac(t,e)|0;u=ac(r,j)|0;b[i+(p<<1)>>1]=(u+269484032+(ac(s,k)|0)+q|0)>>>15&65535;q=ac(t,l)|0;t=ac(r,m)|0;b[a+(p<<1)>>1]=(t+269484032+(ac(s,n)|0)+q|0)>>>15&65535;p=p+1|0;}while((p|0)<(g|0));return}function xk(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=(c[f>>2]|0)+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=(c[h>>2]|0)+(p<<1)|0;s=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=(c[o>>2]|0)+(p<<1)|0;t=(d[q]|d[q+1|0]<<8)<<16>>16&65535;q=ac(t,e)|0;u=ac(r,j)|0;b[i+(p<<1)>>1]=u+1077936128+(ac(s,k)|0)+q>>17&65535;q=ac(t,l)|0;t=ac(r,m)|0;b[a+(p<<1)>>1]=t+1077936128+(ac(s,n)|0)+q>>17&65535;p=p+1|0;}while((p|0)<(g|0));return}function xl(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=(c[f>>2]|0)+(p<<1)|0;r=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=(c[h>>2]|0)+(p<<1)|0;s=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=(c[o>>2]|0)+(p<<1)|0;t=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=ac(t,e)|0;u=ac(r,j)|0;b[i+(p<<1)>>1]=(u+8421376+(ac(s,k)|0)+q|0)>>>10&65535;q=ac(t,l)|0;t=ac(r,m)|0;b[a+(p<<1)>>1]=(t+8421376+(ac(s,n)|0)+q|0)>>>10&65535;p=p+1|0;}while((p|0)<(g|0));return}function xm(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=(c[f>>2]|0)+(p<<1)|0;r=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=(c[h>>2]|0)+(p<<1)|0;s=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=(c[o>>2]|0)+(p<<1)|0;t=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=ac(t,e)|0;u=ac(r,j)|0;b[i+(p<<1)>>1]=(u+16842752+(ac(s,k)|0)+q|0)>>>11&65535;q=ac(t,l)|0;t=ac(r,m)|0;b[a+(p<<1)>>1]=(t+16842752+(ac(s,n)|0)+q|0)>>>11&65535;p=p+1|0;}while((p|0)<(g|0));return}function xn(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=(c[f>>2]|0)+(p<<1)|0;r=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=(c[h>>2]|0)+(p<<1)|0;s=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=(c[o>>2]|0)+(p<<1)|0;t=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=ac(t,e)|0;u=ac(r,j)|0;b[i+(p<<1)>>1]=(u+67371008+(ac(s,k)|0)+q|0)>>>13&65535;q=ac(t,l)|0;t=ac(r,m)|0;b[a+(p<<1)>>1]=(t+67371008+(ac(s,n)|0)+q|0)>>>13&65535;p=p+1|0;}while((p|0)<(g|0));return}function xo(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=(c[f>>2]|0)+(p<<1)|0;r=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=(c[h>>2]|0)+(p<<1)|0;s=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=(c[o>>2]|0)+(p<<1)|0;t=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=ac(t,e)|0;u=ac(r,j)|0;b[i+(p<<1)>>1]=(u+269484032+(ac(s,k)|0)+q|0)>>>15&65535;q=ac(t,l)|0;t=ac(r,m)|0;b[a+(p<<1)>>1]=(t+269484032+(ac(s,n)|0)+q|0)>>>15&65535;p=p+1|0;}while((p|0)<(g|0));return}function xp(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=(c[f>>2]|0)+(p<<1)|0;r=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=(c[h>>2]|0)+(p<<1)|0;s=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=(c[o>>2]|0)+(p<<1)|0;t=(cT((d[q]|d[q+1|0]<<8)<<16>>16|0)|0)&65535;q=ac(t,e)|0;u=ac(r,j)|0;b[i+(p<<1)>>1]=u+1077936128+(ac(s,k)|0)+q>>17&65535;q=ac(t,l)|0;t=ac(r,m)|0;b[a+(p<<1)>>1]=t+1077936128+(ac(s,n)|0)+q>>17&65535;p=p+1|0;}while((p|0)<(g|0));return}function xq(a,e,f,g,h){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;i=a;a=e;e=c[h+12>>2]|0;j=c[h+16>>2]|0;k=c[h+20>>2]|0;l=c[h+24>>2]|0;m=c[h+28>>2]|0;n=c[h+32>>2]|0;if((g|0)<=0){return}h=f+4|0;o=f+8|0;p=0;do{q=d[(c[f>>2]|0)+p|0]|0;r=d[(c[h>>2]|0)+p|0]|0;s=d[(c[o>>2]|0)+p|0]|0;t=ac(s,e)|0;u=ac(q,j)|0;b[i+(p<<1)>>1]=(u+4194560+(ac(r,k)|0)+t|0)>>>9&65535;t=ac(s,l)|0;s=ac(q,m)|0;b[a+(p<<1)>>1]=(s+4194560+(ac(r,n)|0)+t|0)>>>9&65535;p=p+1|0;}while((p|0)<(g|0));return}function xr(a,c,d,e,f,g,h){a=a|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;h=e;e=f;f=a;a=c;if((g|0)>0){i=0}else{return}do{b[f+(i<<1)>>1]=cT(b[h+(i<<1)>>1]|0)|0;b[a+(i<<1)>>1]=cT(b[e+(i<<1)>>1]|0)|0;i=i+1|0;}while((i|0)<(g|0));return}function xs(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(291)|0;if((l|0)==0){s=11706;break}t=r<<3;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=BP(291)|0;if((v|0)==0){s=11710;break}l=g+((t|4)<<1)|0;u=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){x=u}else{x=cT(u|0)|0}u=((w&65535)+1+(x&65535)|0)>>>1;v=BP(291)|0;if((v|0)==0){s=11714;break}l=g+((t|1)<<1)|0;y=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){z=y}else{z=cT(y|0)|0}y=BP(291)|0;if((y|0)==0){s=11718;break}v=g+((t|5)<<1)|0;l=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){A=l}else{A=cT(l|0)|0}l=((z&65535)+1+(A&65535)|0)>>>1;y=BP(291)|0;if((y|0)==0){s=11722;break}v=g+((t|2)<<1)|0;B=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){C=B}else{C=cT(B|0)|0}B=BP(291)|0;if((B|0)==0){s=11726;break}y=g+((t|6)<<1)|0;t=(d[y]|d[y+1|0]<<8)<<16>>16;if((a[B+7|0]&1)==0){D=t}else{D=cT(t|0)|0}t=((C&65535)+1+(D&65535)|0)>>>1;B=ac(u,f)|0;y=ac(l,m)|0;b[h+(r<<1)>>1]=(B+1073758208+y+(ac(t,n)|0)|0)>>>15&65535;y=ac(u,o)|0;u=ac(l,p)|0;b[e+(r<<1)>>1]=(y+1073758208+u+(ac(t,q)|0)|0)>>>15&65535;t=r+1|0;if((t|0)<(k|0)){r=t}else{s=11731;break}}if((s|0)==11706){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11710){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11718){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11714){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11722){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11731){i=j;return}else if((s|0)==11726){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}}function xt(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(292)|0;if((l|0)==0){s=11735;break}t=r<<3;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=BP(292)|0;if((v|0)==0){s=11739;break}l=g+((t|4)<<1)|0;u=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){x=u}else{x=cT(u|0)|0}u=((w&65535)+1+(x&65535)|0)>>>1;v=BP(292)|0;if((v|0)==0){s=11743;break}l=g+((t|1)<<1)|0;y=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){z=y}else{z=cT(y|0)|0}y=BP(292)|0;if((y|0)==0){s=11747;break}v=g+((t|5)<<1)|0;l=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){A=l}else{A=cT(l|0)|0}l=((z&65535)+1+(A&65535)|0)>>>1;y=BP(292)|0;if((y|0)==0){s=11751;break}v=g+((t|2)<<1)|0;B=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){C=B}else{C=cT(B|0)|0}B=BP(292)|0;if((B|0)==0){s=11755;break}y=g+((t|6)<<1)|0;t=(d[y]|d[y+1|0]<<8)<<16>>16;if((a[B+7|0]&1)==0){D=t}else{D=cT(t|0)|0}t=((C&65535)+1+(D&65535)|0)>>>1;B=ac(u,f)|0;y=ac(l,m)|0;b[h+(r<<1)>>1]=(B+1073758208+y+(ac(t,n)|0)|0)>>>15&65535;y=ac(u,o)|0;u=ac(l,p)|0;b[e+(r<<1)>>1]=(y+1073758208+u+(ac(t,q)|0)|0)>>>15&65535;t=r+1|0;if((t|0)<(k|0)){r=t}else{s=11760;break}}if((s|0)==11743){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11739){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11735){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11747){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11751){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11760){i=j;return}else if((s|0)==11755){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}}function xu(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(41)|0;if((l|0)==0){s=11764;break}t=r*6|0;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=BP(41)|0;if((v|0)==0){s=11768;break}l=g+(t+3<<1)|0;u=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){x=u}else{x=cT(u|0)|0}u=((w&65535)+1+(x&65535)|0)>>>1;v=BP(41)|0;if((v|0)==0){s=11772;break}l=g+((t|1)<<1)|0;y=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){z=y}else{z=cT(y|0)|0}y=BP(41)|0;if((y|0)==0){s=11776;break}v=g+(t+4<<1)|0;l=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){A=l}else{A=cT(l|0)|0}l=((z&65535)+1+(A&65535)|0)>>>1;y=BP(41)|0;if((y|0)==0){s=11780;break}v=g+(t+2<<1)|0;B=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){C=B}else{C=cT(B|0)|0}B=BP(41)|0;if((B|0)==0){s=11784;break}y=g+(t+5<<1)|0;t=(d[y]|d[y+1|0]<<8)<<16>>16;if((a[B+7|0]&1)==0){D=t}else{D=cT(t|0)|0}t=((C&65535)+1+(D&65535)|0)>>>1;B=ac(u,f)|0;y=ac(l,m)|0;b[h+(r<<1)>>1]=(B+1073758208+y+(ac(t,n)|0)|0)>>>15&65535;y=ac(u,o)|0;u=ac(l,p)|0;b[e+(r<<1)>>1]=(y+1073758208+u+(ac(t,q)|0)|0)>>>15&65535;t=r+1|0;if((t|0)<(k|0)){r=t}else{s=11790;break}}if((s|0)==11764){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11768){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11776){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11790){i=j;return}else if((s|0)==11780){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11772){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11784){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}}function xv(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(42)|0;if((l|0)==0){s=11793;break}t=r*6|0;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=BP(42)|0;if((v|0)==0){s=11797;break}l=g+(t+3<<1)|0;u=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){x=u}else{x=cT(u|0)|0}u=((w&65535)+1+(x&65535)|0)>>>1;v=BP(42)|0;if((v|0)==0){s=11801;break}l=g+((t|1)<<1)|0;y=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){z=y}else{z=cT(y|0)|0}y=BP(42)|0;if((y|0)==0){s=11805;break}v=g+(t+4<<1)|0;l=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){A=l}else{A=cT(l|0)|0}l=((z&65535)+1+(A&65535)|0)>>>1;y=BP(42)|0;if((y|0)==0){s=11809;break}v=g+(t+2<<1)|0;B=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){C=B}else{C=cT(B|0)|0}B=BP(42)|0;if((B|0)==0){s=11813;break}y=g+(t+5<<1)|0;t=(d[y]|d[y+1|0]<<8)<<16>>16;if((a[B+7|0]&1)==0){D=t}else{D=cT(t|0)|0}t=((C&65535)+1+(D&65535)|0)>>>1;B=ac(u,f)|0;y=ac(l,m)|0;b[h+(r<<1)>>1]=(B+1073758208+y+(ac(t,n)|0)|0)>>>15&65535;y=ac(u,o)|0;u=ac(l,p)|0;b[e+(r<<1)>>1]=(y+1073758208+u+(ac(t,q)|0)|0)>>>15&65535;t=r+1|0;if((t|0)<(k|0)){r=t}else{s=11819;break}}if((s|0)==11813){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11801){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11805){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11819){i=j;return}else if((s|0)==11797){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11809){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11793){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}}function xw(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(67)|0;if((l|0)==0){s=11822;break}t=r*6|0;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=BP(67)|0;if((v|0)==0){s=11826;break}l=g+(t+3<<1)|0;u=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){x=u}else{x=cT(u|0)|0}u=((w&65535)+1+(x&65535)|0)>>>1;v=BP(67)|0;if((v|0)==0){s=11830;break}l=g+((t|1)<<1)|0;y=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){z=y}else{z=cT(y|0)|0}y=BP(67)|0;if((y|0)==0){s=11834;break}v=g+(t+4<<1)|0;l=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){A=l}else{A=cT(l|0)|0}l=((z&65535)+1+(A&65535)|0)>>>1;y=BP(67)|0;if((y|0)==0){s=11838;break}v=g+(t+2<<1)|0;B=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){C=B}else{C=cT(B|0)|0}B=BP(67)|0;if((B|0)==0){s=11842;break}y=g+(t+5<<1)|0;t=(d[y]|d[y+1|0]<<8)<<16>>16;if((a[B+7|0]&1)==0){D=t}else{D=cT(t|0)|0}t=((C&65535)+1+(D&65535)|0)>>>1;B=ac(t,f)|0;y=ac(l,m)|0;b[h+(r<<1)>>1]=((ac(u,n)|0)+1073758208+y+B|0)>>>15&65535;B=ac(t,o)|0;t=ac(l,p)|0;b[e+(r<<1)>>1]=((ac(u,q)|0)+1073758208+t+B|0)>>>15&65535;B=r+1|0;if((B|0)<(k|0)){r=B}else{s=11847;break}}if((s|0)==11822){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11838){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11826){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11834){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11842){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11830){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11847){i=j;return}}function xx(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(68)|0;if((l|0)==0){s=11851;break}t=r*6|0;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=BP(68)|0;if((v|0)==0){s=11855;break}l=g+(t+3<<1)|0;u=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){x=u}else{x=cT(u|0)|0}u=((w&65535)+1+(x&65535)|0)>>>1;v=BP(68)|0;if((v|0)==0){s=11859;break}l=g+((t|1)<<1)|0;y=(d[l]|d[l+1|0]<<8)<<16>>16;if((a[v+7|0]&1)==0){z=y}else{z=cT(y|0)|0}y=BP(68)|0;if((y|0)==0){s=11863;break}v=g+(t+4<<1)|0;l=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){A=l}else{A=cT(l|0)|0}l=((z&65535)+1+(A&65535)|0)>>>1;y=BP(68)|0;if((y|0)==0){s=11867;break}v=g+(t+2<<1)|0;B=(d[v]|d[v+1|0]<<8)<<16>>16;if((a[y+7|0]&1)==0){C=B}else{C=cT(B|0)|0}B=BP(68)|0;if((B|0)==0){s=11871;break}y=g+(t+5<<1)|0;t=(d[y]|d[y+1|0]<<8)<<16>>16;if((a[B+7|0]&1)==0){D=t}else{D=cT(t|0)|0}t=((C&65535)+1+(D&65535)|0)>>>1;B=ac(t,f)|0;y=ac(l,m)|0;b[h+(r<<1)>>1]=((ac(u,n)|0)+1073758208+y+B|0)>>>15&65535;B=ac(t,o)|0;t=ac(l,p)|0;b[e+(r<<1)>>1]=((ac(u,q)|0)+1073758208+t+B|0)>>>15&65535;B=r+1|0;if((B|0)<(k|0)){r=B}else{s=11876;break}}if((s|0)==11871){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11859){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11867){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11876){i=j;return}else if((s|0)==11863){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11855){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}else if((s|0)==11851){AN(0,0,1639344,(E=i,i=i+24|0,c[E>>2]=1718488,c[E+8>>2]=1690768,c[E+16>>2]=635,E)|0);i=E;cw()}}function xy(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;g=a;a=d;d=c[i+12>>2]|0;e=c[i+16>>2]|0;j=c[i+20>>2]|0;k=c[i+24>>2]|0;l=c[i+28>>2]|0;m=c[i+32>>2]|0;if((h|0)>0){n=0}else{return}do{i=n<<3;o=c[f+i>>2]|0;p=c[f+(i|4)>>2]|0;i=(p&-16711936)+(o&-16711936)|0;q=p+o-i|0;o=q&511;p=i&130816;i=q>>>16&511;q=ac(i,d)|0;r=ac(p,e)|0;b[g+(n<<1)>>1]=(r-2147352576+(q+(ac(o,j)|0)<<8)|0)>>>18&65535;q=ac(i,k)|0;i=ac(p,l)|0;b[a+(n<<1)>>1]=(i-2147352576+(q+(ac(o,m)|0)<<8)|0)>>>18&65535;n=n+1|0;}while((n|0)<(h|0));return}function xz(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;g=a;a=d;d=c[i+12>>2]|0;e=c[i+16>>2]|0;j=c[i+20>>2]|0;k=c[i+24>>2]|0;l=c[i+28>>2]|0;m=c[i+32>>2]|0;if((h|0)>0){n=0}else{return}do{i=n<<3;o=(c[f+i>>2]|0)>>>8;p=(c[f+(i|4)>>2]|0)>>>8;i=(p&65280)+(o&65280)|0;q=p+o-i|0;o=q&511;p=q>>>16&511;q=ac(p,d)|0;r=ac(i,e)|0;b[g+(n<<1)>>1]=(r-2147352576+(q+(ac(o,j)|0)<<8)|0)>>>18&65535;q=ac(p,k)|0;p=ac(i,l)|0;b[a+(n<<1)>>1]=(p-2147352576+(q+(ac(o,m)|0)<<8)|0)>>>18&65535;n=n+1|0;}while((n|0)<(h|0));return}function xA(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;h=a;a=e;e=c[j+12>>2]|0;f=c[j+16>>2]|0;k=c[j+20>>2]|0;l=c[j+24>>2]|0;m=c[j+28>>2]|0;n=c[j+32>>2]|0;if((i|0)>0){o=0}else{return}do{j=o*6|0;p=(d[g+(j+3)|0]|0)+(d[g+j|0]|0)|0;q=(d[g+(j+4)|0]|0)+(d[g+(j|1)|0]|0)|0;r=(d[g+(j+5)|0]|0)+(d[g+(j+2)|0]|0)|0;j=ac(r,e)|0;s=ac(q,f)|0;b[h+(o<<1)>>1]=((ac(p,k)|0)+8389120+s+j|0)>>>10&65535;j=ac(r,l)|0;r=ac(q,m)|0;b[a+(o<<1)>>1]=((ac(p,n)|0)+8389120+r+j|0)>>>10&65535;o=o+1|0;}while((o|0)<(i|0));return}function xB(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]<<11;m=c[l+16>>2]<<5;n=c[l+20>>2]|0;o=c[l+24>>2]<<11;p=c[l+28>>2]<<5;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(48)|0;if((l|0)==0){s=11895;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(48)|0;if((l|0)==0){s=11899;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&2016)+(v&2016)|0;u=t+v-l|0;v=u&129024;t=u&63;u=ac(f,t)|0;y=ac(m,l)|0;b[g+(r<<1)>>1]=(y-2147352576+u+(ac(v,n)|0)|0)>>>18&65535;u=ac(o,t)|0;t=ac(p,l)|0;b[e+(r<<1)>>1]=(t-2147352576+u+(ac(v,q)|0)|0)>>>18&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=11904;break}}if((s|0)==11895){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==11899){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==11904){i=j;return}}function xC(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]<<11;m=c[l+16>>2]<<5;n=c[l+20>>2]|0;o=c[l+24>>2]<<11;p=c[l+28>>2]<<5;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(47)|0;if((l|0)==0){s=11908;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(47)|0;if((l|0)==0){s=11912;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&2016)+(v&2016)|0;u=t+v-l|0;v=u&129024;t=u&63;u=ac(f,t)|0;y=ac(m,l)|0;b[g+(r<<1)>>1]=(y-2147352576+u+(ac(v,n)|0)|0)>>>18&65535;u=ac(o,t)|0;t=ac(p,l)|0;b[e+(r<<1)>>1]=(t-2147352576+u+(ac(v,q)|0)|0)>>>18&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=11918;break}}if((s|0)==11908){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==11912){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==11918){i=j;return}}function xD(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]<<10;m=c[l+16>>2]<<5;n=c[l+20>>2]|0;o=c[l+24>>2]<<10;p=c[l+28>>2]<<5;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(50)|0;if((l|0)==0){s=11921;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(50)|0;if((l|0)==0){s=11925;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&33760)+(v&33760)|0;u=t+v-l|0;v=u&64512;t=l&2016;l=u&63;u=ac(f,l)|0;y=ac(m,t)|0;b[g+(r<<1)>>1]=(y+1073807360+u+(ac(v,n)|0)|0)>>>17&65535;u=ac(o,l)|0;l=ac(p,t)|0;b[e+(r<<1)>>1]=(l+1073807360+u+(ac(v,q)|0)|0)>>>17&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=11931;break}}if((s|0)==11925){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==11931){i=j;return}else if((s|0)==11921){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}}function xE(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]<<10;m=c[l+16>>2]<<5;n=c[l+20>>2]|0;o=c[l+24>>2]<<10;p=c[l+28>>2]<<5;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(49)|0;if((l|0)==0){s=11934;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(49)|0;if((l|0)==0){s=11938;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&33760)+(v&33760)|0;u=t+v-l|0;v=u&64512;t=l&2016;l=u&63;u=ac(f,l)|0;y=ac(m,t)|0;b[g+(r<<1)>>1]=(y+1073807360+u+(ac(v,n)|0)|0)>>>17&65535;u=ac(o,l)|0;l=ac(p,t)|0;b[e+(r<<1)>>1]=(l+1073807360+u+(ac(v,q)|0)|0)>>>17&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=11944;break}}if((s|0)==11938){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==11934){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==11944){i=j;return}}function xF(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;k=a;a=e;e=c[j+12>>2]|0;l=c[j+16>>2]|0;m=c[j+20>>2]|0;n=c[j+24>>2]|0;o=c[j+28>>2]|0;p=c[j+32>>2]|0;if((i|0)>0){q=0}else{return}do{j=q<<1;r=j|1;s=(d[f+r|0]|0)+(d[f+j|0]|0)|0;t=(d[g+r|0]|0)+(d[g+j|0]|0)|0;u=(d[h+r|0]|0)+(d[h+j|0]|0)|0;j=ac(u,e)|0;r=ac(s,l)|0;b[k+(q<<1)>>1]=(r+8389120+(ac(t,m)|0)+j|0)>>>10&65535;j=ac(u,n)|0;u=ac(s,o)|0;b[a+(q<<1)>>1]=(u+8389120+(ac(t,p)|0)+j|0)>>>10&65535;q=q+1|0;}while((q|0)<(i|0));return}function xG(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]<<8;m=c[l+16>>2]<<4;n=c[l+20>>2]|0;o=c[l+24>>2]<<8;p=c[l+28>>2]<<4;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(64)|0;if((l|0)==0){s=11952;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(64)|0;if((l|0)==0){s=11956;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&61680)+(v&61680)|0;u=t+v-l|0;v=u&7936;t=l&496;l=u&31;u=ac(f,l)|0;y=ac(m,t)|0;b[g+(r<<1)>>1]=(y+134225920+u+(ac(v,n)|0)|0)>>>14&65535;u=ac(o,l)|0;l=ac(p,t)|0;b[e+(r<<1)>>1]=(l+134225920+u+(ac(v,q)|0)|0)>>>14&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=11961;break}}if((s|0)==11956){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==11952){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==11961){i=j;return}}function xH(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]<<8;m=c[l+16>>2]<<4;n=c[l+20>>2]|0;o=c[l+24>>2]<<8;p=c[l+28>>2]<<4;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(65)|0;if((l|0)==0){s=11965;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(65)|0;if((l|0)==0){s=11969;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&61680)+(v&61680)|0;u=t+v-l|0;v=u&7936;t=l&496;l=u&31;u=ac(f,l)|0;y=ac(m,t)|0;b[g+(r<<1)>>1]=(y+134225920+u+(ac(v,n)|0)|0)>>>14&65535;u=ac(o,l)|0;l=ac(p,t)|0;b[e+(r<<1)>>1]=(l+134225920+u+(ac(v,q)|0)|0)>>>14&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=11975;break}}if((s|0)==11975){i=j;return}else if((s|0)==11965){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==11969){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}}function xI(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;g=a;a=d;d=c[i+12>>2]|0;e=c[i+16>>2]|0;j=c[i+20>>2]|0;k=c[i+24>>2]|0;l=c[i+28>>2]|0;m=c[i+32>>2]|0;if((h|0)>0){n=0}else{return}do{i=n<<3;o=c[f+i>>2]|0;p=c[f+(i|4)>>2]|0;i=(p&-16711936)+(o&-16711936)|0;q=p+o-i|0;o=q>>>16&511;p=i&130816;i=q&511;q=ac(i,d)|0;r=ac(p,e)|0;b[g+(n<<1)>>1]=(r-2147352576+((ac(o,j)|0)+q<<8)|0)>>>18&65535;q=ac(i,k)|0;i=ac(p,l)|0;b[a+(n<<1)>>1]=(i-2147352576+((ac(o,m)|0)+q<<8)|0)>>>18&65535;n=n+1|0;}while((n|0)<(h|0));return}function xJ(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;g=a;a=d;d=c[i+12>>2]|0;e=c[i+16>>2]|0;j=c[i+20>>2]|0;k=c[i+24>>2]|0;l=c[i+28>>2]|0;m=c[i+32>>2]|0;if((h|0)>0){n=0}else{return}do{i=n<<3;o=(c[f+i>>2]|0)>>>8;p=(c[f+(i|4)>>2]|0)>>>8;i=(p&65280)+(o&65280)|0;q=p+o-i|0;o=q>>>16&511;p=q&511;q=ac(p,d)|0;r=ac(i,e)|0;b[g+(n<<1)>>1]=(r-2147352576+((ac(o,j)|0)+q<<8)|0)>>>18&65535;q=ac(p,k)|0;p=ac(i,l)|0;b[a+(n<<1)>>1]=(p-2147352576+((ac(o,m)|0)+q<<8)|0)>>>18&65535;n=n+1|0;}while((n|0)<(h|0));return}function xK(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;h=a;a=e;e=c[j+12>>2]|0;f=c[j+16>>2]|0;k=c[j+20>>2]|0;l=c[j+24>>2]|0;m=c[j+28>>2]|0;n=c[j+32>>2]|0;if((i|0)>0){o=0}else{return}do{j=o*6|0;p=(d[g+(j+3)|0]|0)+(d[g+j|0]|0)|0;q=(d[g+(j+4)|0]|0)+(d[g+(j|1)|0]|0)|0;r=(d[g+(j+5)|0]|0)+(d[g+(j+2)|0]|0)|0;j=ac(p,e)|0;s=ac(q,f)|0;b[h+(o<<1)>>1]=(j+8389120+s+(ac(r,k)|0)|0)>>>10&65535;s=ac(p,l)|0;p=ac(q,m)|0;b[a+(o<<1)>>1]=(s+8389120+p+(ac(r,n)|0)|0)>>>10&65535;o=o+1|0;}while((o|0)<(i|0));return}function xL(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<5;n=c[l+20>>2]<<11;o=c[l+24>>2]|0;p=c[l+28>>2]<<5;q=c[l+32>>2]<<11;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(44)|0;if((l|0)==0){s=11993;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(44)|0;if((l|0)==0){s=11997;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&2016)+(v&2016)|0;u=t+v-l|0;v=u&63;t=u&129024;u=ac(t,f)|0;y=ac(m,l)|0;b[g+(r<<1)>>1]=(y-2147352576+u+(ac(n,v)|0)|0)>>>18&65535;u=ac(t,o)|0;t=ac(p,l)|0;b[e+(r<<1)>>1]=(t-2147352576+u+(ac(q,v)|0)|0)>>>18&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=12003;break}}if((s|0)==11997){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==12003){i=j;return}else if((s|0)==11993){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}}function xM(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<5;n=c[l+20>>2]<<11;o=c[l+24>>2]|0;p=c[l+28>>2]<<5;q=c[l+32>>2]<<11;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(43)|0;if((l|0)==0){s=12006;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(43)|0;if((l|0)==0){s=12010;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&2016)+(v&2016)|0;u=t+v-l|0;v=u&63;t=u&129024;u=ac(t,f)|0;y=ac(m,l)|0;b[g+(r<<1)>>1]=(y-2147352576+u+(ac(n,v)|0)|0)>>>18&65535;u=ac(t,o)|0;t=ac(p,l)|0;b[e+(r<<1)>>1]=(t-2147352576+u+(ac(q,v)|0)|0)>>>18&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=12016;break}}if((s|0)==12006){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==12016){i=j;return}else if((s|0)==12010){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}}function xN(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<5;n=c[l+20>>2]<<10;o=c[l+24>>2]|0;p=c[l+28>>2]<<5;q=c[l+32>>2]<<10;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(46)|0;if((l|0)==0){s=12019;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(46)|0;if((l|0)==0){s=12023;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&33760)+(v&33760)|0;u=t+v-l|0;v=u&63;t=l&2016;l=u&64512;u=ac(l,f)|0;y=ac(m,t)|0;b[g+(r<<1)>>1]=(y+1073807360+u+(ac(n,v)|0)|0)>>>17&65535;u=ac(l,o)|0;l=ac(p,t)|0;b[e+(r<<1)>>1]=(l+1073807360+u+(ac(q,v)|0)|0)>>>17&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=12029;break}}if((s|0)==12023){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==12029){i=j;return}else if((s|0)==12019){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}}function xO(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<5;n=c[l+20>>2]<<10;o=c[l+24>>2]|0;p=c[l+28>>2]<<5;q=c[l+32>>2]<<10;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(45)|0;if((l|0)==0){s=12032;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(45)|0;if((l|0)==0){s=12036;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&33760)+(v&33760)|0;u=t+v-l|0;v=u&63;t=l&2016;l=u&64512;u=ac(l,f)|0;y=ac(m,t)|0;b[g+(r<<1)>>1]=(y+1073807360+u+(ac(n,v)|0)|0)>>>17&65535;u=ac(l,o)|0;l=ac(p,t)|0;b[e+(r<<1)>>1]=(l+1073807360+u+(ac(q,v)|0)|0)>>>17&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=12042;break}}if((s|0)==12032){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==12036){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==12042){i=j;return}}function xP(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<4;n=c[l+20>>2]<<8;o=c[l+24>>2]|0;p=c[l+28>>2]<<4;q=c[l+32>>2]<<8;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(62)|0;if((l|0)==0){s=12045;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(62)|0;if((l|0)==0){s=12049;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&61680)+(v&61680)|0;u=t+v-l|0;v=u&31;t=l&496;l=u&7936;u=ac(l,f)|0;y=ac(m,t)|0;b[g+(r<<1)>>1]=(y+134225920+u+(ac(n,v)|0)|0)>>>14&65535;u=ac(l,o)|0;l=ac(p,t)|0;b[e+(r<<1)>>1]=(l+134225920+u+(ac(q,v)|0)|0)>>>14&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=12055;break}}if((s|0)==12049){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==12055){i=j;return}else if((s|0)==12045){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}}function xQ(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<4;n=c[l+20>>2]<<8;o=c[l+24>>2]|0;p=c[l+28>>2]<<4;q=c[l+32>>2]<<8;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(63)|0;if((l|0)==0){s=12058;break}t=r<<2;u=h+t|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(63)|0;if((l|0)==0){s=12062;break}u=h+(t|2)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){x=t}else{x=cT(t|0)|0}t=x&65535;l=(t&61680)+(v&61680)|0;u=t+v-l|0;v=u&31;t=l&496;l=u&7936;u=ac(l,f)|0;y=ac(m,t)|0;b[g+(r<<1)>>1]=(y+134225920+u+(ac(n,v)|0)|0)>>>14&65535;u=ac(l,o)|0;l=ac(p,t)|0;b[e+(r<<1)>>1]=(l+134225920+u+(ac(q,v)|0)|0)>>>14&65535;v=r+1|0;if((v|0)<(k|0)){r=v}else{s=12068;break}}if((s|0)==12062){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==12058){AN(0,0,1639344,(z=i,i=i+24|0,c[z>>2]=1718488,c[z+8>>2]=1690768,c[z+16>>2]=635,z)|0);i=z;cw()}else if((s|0)==12068){i=j;return}}function xR(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(291)|0;if((l|0)==0){s=12071;break}t=r<<2;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(291)|0;if((l|0)==0){s=12075;break}u=g+((t|1)<<1)|0;x=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){y=x}else{y=cT(x|0)|0}x=y&65535;l=BP(291)|0;if((l|0)==0){s=12079;break}u=g+((t|2)<<1)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){z=t}else{z=cT(t|0)|0}t=z&65535;l=ac(v,f)|0;u=ac(x,m)|0;b[h+(r<<1)>>1]=(l+1073758208+u+(ac(t,n)|0)|0)>>>15&65535;u=ac(v,o)|0;v=ac(x,p)|0;b[e+(r<<1)>>1]=(u+1073758208+v+(ac(t,q)|0)|0)>>>15&65535;t=r+1|0;if((t|0)<(k|0)){r=t}else{s=12084;break}}if((s|0)==12075){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12084){i=j;return}else if((s|0)==12079){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12071){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}}function xS(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(292)|0;if((l|0)==0){s=12088;break}t=r<<2;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(292)|0;if((l|0)==0){s=12092;break}u=g+((t|1)<<1)|0;x=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){y=x}else{y=cT(x|0)|0}x=y&65535;l=BP(292)|0;if((l|0)==0){s=12096;break}u=g+((t|2)<<1)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){z=t}else{z=cT(t|0)|0}t=z&65535;l=ac(v,f)|0;u=ac(x,m)|0;b[h+(r<<1)>>1]=(l+1073758208+u+(ac(t,n)|0)|0)>>>15&65535;u=ac(v,o)|0;v=ac(x,p)|0;b[e+(r<<1)>>1]=(u+1073758208+v+(ac(t,q)|0)|0)>>>15&65535;t=r+1|0;if((t|0)<(k|0)){r=t}else{s=12102;break}}if((s|0)==12092){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12088){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12102){i=j;return}else if((s|0)==12096){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}}function xT(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(41)|0;if((l|0)==0){s=12105;break}t=r*3|0;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(41)|0;if((l|0)==0){s=12109;break}u=g+(t+1<<1)|0;x=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){y=x}else{y=cT(x|0)|0}x=y&65535;l=BP(41)|0;if((l|0)==0){s=12113;break}u=g+(t+2<<1)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){z=t}else{z=cT(t|0)|0}t=z&65535;l=ac(v,f)|0;u=ac(x,m)|0;b[h+(r<<1)>>1]=(l+1073758208+u+(ac(t,n)|0)|0)>>>15&65535;u=ac(v,o)|0;v=ac(x,p)|0;b[e+(r<<1)>>1]=(u+1073758208+v+(ac(t,q)|0)|0)>>>15&65535;t=r+1|0;if((t|0)<(k|0)){r=t}else{s=12118;break}}if((s|0)==12118){i=j;return}else if((s|0)==12109){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12113){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12105){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}}function xU(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(42)|0;if((l|0)==0){s=12122;break}t=r*3|0;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(42)|0;if((l|0)==0){s=12126;break}u=g+(t+1<<1)|0;x=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){y=x}else{y=cT(x|0)|0}x=y&65535;l=BP(42)|0;if((l|0)==0){s=12130;break}u=g+(t+2<<1)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){z=t}else{z=cT(t|0)|0}t=z&65535;l=ac(v,f)|0;u=ac(x,m)|0;b[h+(r<<1)>>1]=(l+1073758208+u+(ac(t,n)|0)|0)>>>15&65535;u=ac(v,o)|0;v=ac(x,p)|0;b[e+(r<<1)>>1]=(u+1073758208+v+(ac(t,q)|0)|0)>>>15&65535;t=r+1|0;if((t|0)<(k|0)){r=t}else{s=12136;break}}if((s|0)==12126){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12136){i=j;return}else if((s|0)==12122){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12130){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}}function xV(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(67)|0;if((l|0)==0){s=12139;break}t=r*3|0;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(67)|0;if((l|0)==0){s=12143;break}u=g+(t+1<<1)|0;x=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){y=x}else{y=cT(x|0)|0}x=y&65535;l=BP(67)|0;if((l|0)==0){s=12147;break}u=g+(t+2<<1)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){z=t}else{z=cT(t|0)|0}t=z&65535;l=ac(t,f)|0;u=ac(x,m)|0;b[h+(r<<1)>>1]=((ac(v,n)|0)+1073758208+u+l|0)>>>15&65535;l=ac(t,o)|0;t=ac(x,p)|0;b[e+(r<<1)>>1]=((ac(v,q)|0)+1073758208+t+l|0)>>>15&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12152;break}}if((s|0)==12152){i=j;return}else if((s|0)==12143){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12147){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12139){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}}function xW(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;j=i;g=h;h=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]|0;n=c[l+20>>2]|0;o=c[l+24>>2]|0;p=c[l+28>>2]|0;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(68)|0;if((l|0)==0){s=12156;break}t=r*3|0;u=g+(t<<1)|0;v=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){w=v}else{w=cT(v|0)|0}v=w&65535;l=BP(68)|0;if((l|0)==0){s=12160;break}u=g+(t+1<<1)|0;x=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){y=x}else{y=cT(x|0)|0}x=y&65535;l=BP(68)|0;if((l|0)==0){s=12164;break}u=g+(t+2<<1)|0;t=(d[u]|d[u+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){z=t}else{z=cT(t|0)|0}t=z&65535;l=ac(t,f)|0;u=ac(x,m)|0;b[h+(r<<1)>>1]=((ac(v,n)|0)+1073758208+u+l|0)>>>15&65535;l=ac(t,o)|0;t=ac(x,p)|0;b[e+(r<<1)>>1]=((ac(v,q)|0)+1073758208+t+l|0)>>>15&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12170;break}}if((s|0)==12160){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12156){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}else if((s|0)==12170){i=j;return}else if((s|0)==12164){AN(0,0,1639344,(A=i,i=i+24|0,c[A>>2]=1718488,c[A+8>>2]=1690768,c[A+16>>2]=635,A)|0);i=A;cw()}}function xX(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;g=a;a=d;d=c[i+12>>2]|0;e=c[i+16>>2]|0;j=c[i+20>>2]|0;k=c[i+24>>2]|0;l=c[i+28>>2]|0;m=c[i+32>>2]|0;if((h|0)>0){n=0}else{return}do{i=c[f+(n<<2)>>2]|0;o=i&255;p=i&65280;q=i>>>16&255;i=ac(q,d)|0;r=ac(p,e)|0;b[g+(n<<1)>>1]=(r+1073807360+(i+(ac(o,j)|0)<<8)|0)>>>17&65535;i=ac(q,k)|0;q=ac(p,l)|0;b[a+(n<<1)>>1]=(q+1073807360+(i+(ac(o,m)|0)<<8)|0)>>>17&65535;n=n+1|0;}while((n|0)<(h|0));return}function xY(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;g=a;a=d;d=c[i+12>>2]|0;e=c[i+16>>2]|0;j=c[i+20>>2]|0;k=c[i+24>>2]|0;l=c[i+28>>2]|0;m=c[i+32>>2]|0;if((h|0)>0){n=0}else{return}do{i=c[f+(n<<2)>>2]|0;o=i>>>8;p=o&255;q=o&65280;o=i>>>24;i=ac(o,d)|0;r=ac(q,e)|0;b[g+(n<<1)>>1]=(r+1073807360+((ac(p,j)|0)+i<<8)|0)>>>17&65535;i=ac(o,k)|0;o=ac(q,l)|0;b[a+(n<<1)>>1]=(o+1073807360+((ac(p,m)|0)+i<<8)|0)>>>17&65535;n=n+1|0;}while((n|0)<(h|0));return}function xZ(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;h=a;a=e;e=c[j+12>>2]|0;f=c[j+16>>2]|0;k=c[j+20>>2]|0;l=c[j+24>>2]|0;m=c[j+28>>2]|0;n=c[j+32>>2]|0;if((i|0)>0){o=0}else{return}do{j=o*3|0;p=d[g+j|0]|0;q=d[g+(j+1)|0]|0;r=d[g+(j+2)|0]|0;j=ac(r,e)|0;s=ac(q,f)|0;b[h+(o<<1)>>1]=((ac(p,k)|0)+4194560+s+j|0)>>>9&65535;j=ac(r,l)|0;r=ac(q,m)|0;b[a+(o<<1)>>1]=((ac(p,n)|0)+4194560+r+j|0)>>>9&65535;o=o+1|0;}while((o|0)<(i|0));return}function x_(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]<<11;m=c[l+16>>2]<<5;n=c[l+20>>2]|0;o=c[l+24>>2]<<11;p=c[l+28>>2]<<5;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(48)|0;if((l|0)==0){s=12188;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&63488;t=u&2016;w=u&31;u=ac(f,w)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+1073807360+u+(ac(l,n)|0)|0)>>>17&65535;u=ac(o,w)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+1073807360+u+(ac(l,q)|0)|0)>>>17&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12193;break}}if((s|0)==12193){i=j;return}else if((s|0)==12188){AN(0,0,1639344,(s=i,i=i+24|0,c[s>>2]=1718488,c[s+8>>2]=1690768,c[s+16>>2]=635,s)|0);i=s;cw()}}function x$(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]<<11;m=c[l+16>>2]<<5;n=c[l+20>>2]|0;o=c[l+24>>2]<<11;p=c[l+28>>2]<<5;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(47)|0;if((l|0)==0){s=12197;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&63488;t=u&2016;w=u&31;u=ac(f,w)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+1073807360+u+(ac(l,n)|0)|0)>>>17&65535;u=ac(o,w)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+1073807360+u+(ac(l,q)|0)|0)>>>17&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12202;break}}if((s|0)==12197){AN(0,0,1639344,(r=i,i=i+24|0,c[r>>2]=1718488,c[r+8>>2]=1690768,c[r+16>>2]=635,r)|0);i=r;cw()}else if((s|0)==12202){i=j;return}}function x0(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]<<10;m=c[l+16>>2]<<5;n=c[l+20>>2]|0;o=c[l+24>>2]<<10;p=c[l+28>>2]<<5;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(50)|0;if((l|0)==0){s=12206;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&31744;t=u&992;w=u&31;u=ac(f,w)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+536903680+u+(ac(l,n)|0)|0)>>>16&65535;u=ac(o,w)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+536903680+u+(ac(l,q)|0)|0)>>>16&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12211;break}}if((s|0)==12206){AN(0,0,1639344,(r=i,i=i+24|0,c[r>>2]=1718488,c[r+8>>2]=1690768,c[r+16>>2]=635,r)|0);i=r;cw()}else if((s|0)==12211){i=j;return}}function x1(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]<<10;m=c[l+16>>2]<<5;n=c[l+20>>2]|0;o=c[l+24>>2]<<10;p=c[l+28>>2]<<5;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(49)|0;if((l|0)==0){s=12215;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&31744;t=u&992;w=u&31;u=ac(f,w)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+536903680+u+(ac(l,n)|0)|0)>>>16&65535;u=ac(o,w)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+536903680+u+(ac(l,q)|0)|0)>>>16&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12220;break}}if((s|0)==12215){AN(0,0,1639344,(r=i,i=i+24|0,c[r>>2]=1718488,c[r+8>>2]=1690768,c[r+16>>2]=635,r)|0);i=r;cw()}else if((s|0)==12220){i=j;return}}function x2(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]<<8;m=c[l+16>>2]<<4;n=c[l+20>>2]|0;o=c[l+24>>2]<<8;p=c[l+28>>2]<<4;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(64)|0;if((l|0)==0){s=12224;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&3840;t=u&240;w=u&15;u=ac(f,w)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+67112960+u+(ac(l,n)|0)|0)>>>13&65535;u=ac(o,w)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+67112960+u+(ac(l,q)|0)|0)>>>13&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12229;break}}if((s|0)==12229){i=j;return}else if((s|0)==12224){AN(0,0,1639344,(s=i,i=i+24|0,c[s>>2]=1718488,c[s+8>>2]=1690768,c[s+16>>2]=635,s)|0);i=s;cw()}}function x3(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]<<8;m=c[l+16>>2]<<4;n=c[l+20>>2]|0;o=c[l+24>>2]<<8;p=c[l+28>>2]<<4;q=c[l+32>>2]|0;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(65)|0;if((l|0)==0){s=12233;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&3840;t=u&240;w=u&15;u=ac(f,w)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+67112960+u+(ac(l,n)|0)|0)>>>13&65535;u=ac(o,w)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+67112960+u+(ac(l,q)|0)|0)>>>13&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12239;break}}if((s|0)==12233){AN(0,0,1639344,(r=i,i=i+24|0,c[r>>2]=1718488,c[r+8>>2]=1690768,c[r+16>>2]=635,r)|0);i=r;cw()}else if((s|0)==12239){i=j;return}}function x4(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;g=a;a=d;d=c[i+12>>2]|0;e=c[i+16>>2]|0;j=c[i+20>>2]|0;k=c[i+24>>2]|0;l=c[i+28>>2]|0;m=c[i+32>>2]|0;if((h|0)>0){n=0}else{return}do{i=c[f+(n<<2)>>2]|0;o=i>>>16&255;p=i&65280;q=i&255;i=ac(q,d)|0;r=ac(p,e)|0;b[g+(n<<1)>>1]=(r+1073807360+((ac(o,j)|0)+i<<8)|0)>>>17&65535;i=ac(q,k)|0;q=ac(p,l)|0;b[a+(n<<1)>>1]=(q+1073807360+((ac(o,m)|0)+i<<8)|0)>>>17&65535;n=n+1|0;}while((n|0)<(h|0));return}function x5(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;g=a;a=d;d=c[i+12>>2]|0;e=c[i+16>>2]|0;j=c[i+20>>2]|0;k=c[i+24>>2]|0;l=c[i+28>>2]|0;m=c[i+32>>2]|0;if((h|0)>0){n=0}else{return}do{i=c[f+(n<<2)>>2]|0;o=i>>>8;p=i>>>24;i=o&65280;q=o&255;o=ac(q,d)|0;r=ac(i,e)|0;b[g+(n<<1)>>1]=(r+1073807360+(o+(ac(p,j)|0)<<8)|0)>>>17&65535;o=ac(q,k)|0;q=ac(i,l)|0;b[a+(n<<1)>>1]=(q+1073807360+(o+(ac(p,m)|0)<<8)|0)>>>17&65535;n=n+1|0;}while((n|0)<(h|0));return}function x6(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;h=a;a=e;e=c[j+12>>2]|0;f=c[j+16>>2]|0;k=c[j+20>>2]|0;l=c[j+24>>2]|0;m=c[j+28>>2]|0;n=c[j+32>>2]|0;if((i|0)>0){o=0}else{return}do{j=o*3|0;p=d[g+j|0]|0;q=d[g+(j+1)|0]|0;r=d[g+(j+2)|0]|0;j=ac(p,e)|0;s=ac(q,f)|0;b[h+(o<<1)>>1]=(j+4194560+s+(ac(r,k)|0)|0)>>>9&65535;s=ac(p,l)|0;p=ac(q,m)|0;b[a+(o<<1)>>1]=(s+4194560+p+(ac(r,n)|0)|0)>>>9&65535;o=o+1|0;}while((o|0)<(i|0));return}function x7(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<5;n=c[l+20>>2]<<11;o=c[l+24>>2]|0;p=c[l+28>>2]<<5;q=c[l+32>>2]<<11;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(44)|0;if((l|0)==0){s=12257;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&31;t=u&2016;w=u&63488;u=ac(w,f)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+1073807360+u+(ac(n,l)|0)|0)>>>17&65535;u=ac(w,o)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+1073807360+u+(ac(q,l)|0)|0)>>>17&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12262;break}}if((s|0)==12257){AN(0,0,1639344,(r=i,i=i+24|0,c[r>>2]=1718488,c[r+8>>2]=1690768,c[r+16>>2]=635,r)|0);i=r;cw()}else if((s|0)==12262){i=j;return}}function x8(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<5;n=c[l+20>>2]<<11;o=c[l+24>>2]|0;p=c[l+28>>2]<<5;q=c[l+32>>2]<<11;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(43)|0;if((l|0)==0){s=12266;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&31;t=u&2016;w=u&63488;u=ac(w,f)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+1073807360+u+(ac(n,l)|0)|0)>>>17&65535;u=ac(w,o)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+1073807360+u+(ac(q,l)|0)|0)>>>17&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12272;break}}if((s|0)==12272){i=j;return}else if((s|0)==12266){AN(0,0,1639344,(s=i,i=i+24|0,c[s>>2]=1718488,c[s+8>>2]=1690768,c[s+16>>2]=635,s)|0);i=s;cw()}}function x9(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<5;n=c[l+20>>2]<<10;o=c[l+24>>2]|0;p=c[l+28>>2]<<5;q=c[l+32>>2]<<10;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(46)|0;if((l|0)==0){s=12275;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&31;t=u&992;w=u&31744;u=ac(w,f)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+536903680+u+(ac(n,l)|0)|0)>>>16&65535;u=ac(w,o)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+536903680+u+(ac(q,l)|0)|0)>>>16&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12280;break}}if((s|0)==12275){AN(0,0,1639344,(r=i,i=i+24|0,c[r>>2]=1718488,c[r+8>>2]=1690768,c[r+16>>2]=635,r)|0);i=r;cw()}else if((s|0)==12280){i=j;return}}function ya(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<5;n=c[l+20>>2]<<10;o=c[l+24>>2]|0;p=c[l+28>>2]<<5;q=c[l+32>>2]<<10;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(45)|0;if((l|0)==0){s=12284;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&31;t=u&992;w=u&31744;u=ac(w,f)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+536903680+u+(ac(n,l)|0)|0)>>>16&65535;u=ac(w,o)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+536903680+u+(ac(q,l)|0)|0)>>>16&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12290;break}}if((s|0)==12284){AN(0,0,1639344,(r=i,i=i+24|0,c[r>>2]=1718488,c[r+8>>2]=1690768,c[r+16>>2]=635,r)|0);i=r;cw()}else if((s|0)==12290){i=j;return}}function yb(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<4;n=c[l+20>>2]<<8;o=c[l+24>>2]|0;p=c[l+28>>2]<<4;q=c[l+32>>2]<<8;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(62)|0;if((l|0)==0){s=12293;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&15;t=u&240;w=u&3840;u=ac(w,f)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+67112960+u+(ac(n,l)|0)|0)>>>13&65535;u=ac(w,o)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+67112960+u+(ac(q,l)|0)|0)>>>13&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12299;break}}if((s|0)==12293){AN(0,0,1639344,(r=i,i=i+24|0,c[r>>2]=1718488,c[r+8>>2]=1690768,c[r+16>>2]=635,r)|0);i=r;cw()}else if((s|0)==12299){i=j;return}}function yc(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=i;g=e;e=f;f=c[l+12>>2]|0;m=c[l+16>>2]<<4;n=c[l+20>>2]<<8;o=c[l+24>>2]|0;p=c[l+28>>2]<<4;q=c[l+32>>2]<<8;if((k|0)>0){r=0}else{i=j;return}while(1){l=BP(63)|0;if((l|0)==0){s=12302;break}t=h+(r<<1)|0;u=(d[t]|d[t+1|0]<<8)<<16>>16;if((a[l+7|0]&1)==0){v=u}else{v=cT(u|0)|0}u=v&65535;l=u&15;t=u&240;w=u&3840;u=ac(w,f)|0;x=ac(m,t)|0;b[g+(r<<1)>>1]=(x+67112960+u+(ac(n,l)|0)|0)>>>13&65535;u=ac(w,o)|0;w=ac(p,t)|0;b[e+(r<<1)>>1]=(w+67112960+u+(ac(q,l)|0)|0)>>>13&65535;l=r+1|0;if((l|0)<(k|0)){r=l}else{s=12308;break}}if((s|0)==12308){i=j;return}else if((s|0)==12302){AN(0,0,1639344,(s=i,i=i+24|0,c[s>>2]=1718488,c[s+8>>2]=1690768,c[s+16>>2]=635,s)|0);i=s;cw()}}function yd(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=(c[e>>2]|0)+(l<<1)|0;n=(d[m]|d[m+1|0]<<8)<<16>>16&65535;m=(c[g>>2]|0)+(l<<1)|0;o=(c[k>>2]|0)+(l<<1)|0;p=(d[m]|d[m+1|0]<<8)<<16>>16&65535;m=ac((d[o]|d[o+1|0]<<8)<<16>>16&65535,a)|0;o=ac(n,i)|0;b[h+(l<<1)>>1]=(o+1081344+(ac(p,j)|0)+m|0)>>>10&65535;l=l+1|0;}while((l|0)<(f|0));return}function ye(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=(c[e>>2]|0)+(l<<1)|0;n=(d[m]|d[m+1|0]<<8)<<16>>16&65535;m=(c[g>>2]|0)+(l<<1)|0;o=(c[k>>2]|0)+(l<<1)|0;p=(d[m]|d[m+1|0]<<8)<<16>>16&65535;m=ac((d[o]|d[o+1|0]<<8)<<16>>16&65535,a)|0;o=ac(n,i)|0;b[h+(l<<1)>>1]=(o+2162688+(ac(p,j)|0)+m|0)>>>11&65535;l=l+1|0;}while((l|0)<(f|0));return}function yf(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=(c[e>>2]|0)+(l<<1)|0;n=(d[m]|d[m+1|0]<<8)<<16>>16&65535;m=(c[g>>2]|0)+(l<<1)|0;o=(c[k>>2]|0)+(l<<1)|0;p=(d[m]|d[m+1|0]<<8)<<16>>16&65535;m=ac((d[o]|d[o+1|0]<<8)<<16>>16&65535,a)|0;o=ac(n,i)|0;b[h+(l<<1)>>1]=(o+8650752+(ac(p,j)|0)+m|0)>>>13&65535;l=l+1|0;}while((l|0)<(f|0));return}function yg(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=(c[e>>2]|0)+(l<<1)|0;n=(d[m]|d[m+1|0]<<8)<<16>>16&65535;m=(c[g>>2]|0)+(l<<1)|0;o=(c[k>>2]|0)+(l<<1)|0;p=(d[m]|d[m+1|0]<<8)<<16>>16&65535;m=ac((d[o]|d[o+1|0]<<8)<<16>>16&65535,a)|0;o=ac(n,i)|0;b[h+(l<<1)>>1]=(o+34603008+(ac(p,j)|0)+m|0)>>>15&65535;l=l+1|0;}while((l|0)<(f|0));return}function yh(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=(c[e>>2]|0)+(l<<1)|0;n=(d[m]|d[m+1|0]<<8)<<16>>16&65535;m=(c[g>>2]|0)+(l<<1)|0;o=(c[k>>2]|0)+(l<<1)|0;p=(d[m]|d[m+1|0]<<8)<<16>>16&65535;m=ac((d[o]|d[o+1|0]<<8)<<16>>16&65535,a)|0;o=ac(n,i)|0;b[h+(l<<1)>>1]=o+138412032+(ac(p,j)|0)+m>>17&65535;l=l+1|0;}while((l|0)<(f|0));return}function yi(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=(c[e>>2]|0)+(l<<1)|0;n=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;m=(c[g>>2]|0)+(l<<1)|0;o=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;m=(c[k>>2]|0)+(l<<1)|0;p=ac((cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535,a)|0;m=ac(n,i)|0;b[h+(l<<1)>>1]=(m+1081344+(ac(o,j)|0)+p|0)>>>10&65535;l=l+1|0;}while((l|0)<(f|0));return}function yj(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=(c[e>>2]|0)+(l<<1)|0;n=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;m=(c[g>>2]|0)+(l<<1)|0;o=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;m=(c[k>>2]|0)+(l<<1)|0;p=ac((cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535,a)|0;m=ac(n,i)|0;b[h+(l<<1)>>1]=(m+2162688+(ac(o,j)|0)+p|0)>>>11&65535;l=l+1|0;}while((l|0)<(f|0));return}function yk(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=(c[e>>2]|0)+(l<<1)|0;n=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;m=(c[g>>2]|0)+(l<<1)|0;o=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;m=(c[k>>2]|0)+(l<<1)|0;p=ac((cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535,a)|0;m=ac(n,i)|0;b[h+(l<<1)>>1]=(m+8650752+(ac(o,j)|0)+p|0)>>>13&65535;l=l+1|0;}while((l|0)<(f|0));return}function yl(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=(c[e>>2]|0)+(l<<1)|0;n=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;m=(c[g>>2]|0)+(l<<1)|0;o=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;m=(c[k>>2]|0)+(l<<1)|0;p=ac((cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535,a)|0;m=ac(n,i)|0;b[h+(l<<1)>>1]=(m+34603008+(ac(o,j)|0)+p|0)>>>15&65535;l=l+1|0;}while((l|0)<(f|0));return}function ym(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=(c[e>>2]|0)+(l<<1)|0;n=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;m=(c[g>>2]|0)+(l<<1)|0;o=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;m=(c[k>>2]|0)+(l<<1)|0;p=ac((cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535,a)|0;m=ac(n,i)|0;b[h+(l<<1)>>1]=m+138412032+(ac(o,j)|0)+p>>17&65535;l=l+1|0;}while((l|0)<(f|0));return}function yn(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;g=a;if((f|0)<=0){return}a=e+12|0;e=0;do{b[g+(e<<1)>>1]=(d[(c[a>>2]|0)+e|0]|0)<<6;e=e+1|0;}while((e|0)<(f|0));return}function yo(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=a;a=c[g>>2]|0;i=c[g+4>>2]|0;j=c[g+8>>2]|0;if((f|0)<=0){return}g=e+4|0;k=e+8|0;l=0;do{m=d[(c[e>>2]|0)+l|0]|0;n=d[(c[g>>2]|0)+l|0]|0;o=ac(d[(c[k>>2]|0)+l|0]|0,a)|0;p=ac(m,i)|0;b[h+(l<<1)>>1]=(p+524544+(ac(n,j)|0)+o|0)>>>9&65535;l=l+1|0;}while((l|0)<(f|0));return}function yp(a,c,d,e,f,g){a=a|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0;g=c;c=a;if((f|0)>0){h=0}else{return}do{b[c+(h<<1)>>1]=cT(b[g+(h<<1)>>1]|0)|0;h=h+1|0;}while((h|0)<(f|0));return}function yq(b,c,d,e,f,g){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0;if((f|0)>0){h=0}else{return}do{a[b+h|0]=a[c+(h<<1)|0]|0;h=h+1|0;}while((h|0)<(f|0));return}function yr(b,c,d,e,f,g){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0;if((f|0)>0){h=0}else{return}do{a[b+h|0]=a[c+(h<<1|1)|0]|0;h=h+1|0;}while((h|0)<(f|0));return}function ys(a,e,f,g,h,i){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0;g=a;a=c[i>>2]|0;f=c[i+4>>2]|0;j=c[i+8>>2]|0;if((h|0)>0){k=0}else{return}do{i=k*3|0;l=d[e+i|0]|0;m=d[e+(i+1)|0]|0;n=ac(d[e+(i+2)|0]|0,a)|0;i=ac(m,f)|0;b[g+(k<<1)>>1]=((ac(l,j)|0)+524544+i+n|0)>>>9&65535;k=k+1|0;}while((k|0)<(h|0));return}function yt(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]<<11;l=c[k+4>>2]<<5;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(48)|0;if((k|0)==0){o=12403;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(e,q&31)|0;p=ac(l,q&2016)|0;b[g+(n<<1)>>1]=(p+134283264+k+(ac(q&63488,m)|0)|0)>>>17&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12408;break}}if((o|0)==12403){AN(0,0,1639344,(n=i,i=i+24|0,c[n>>2]=1718488,c[n+8>>2]=1690768,c[n+16>>2]=635,n)|0);i=n;cw()}else if((o|0)==12408){i=h;return}}function yu(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]<<11;l=c[k+4>>2]<<5;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(47)|0;if((k|0)==0){o=12412;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(e,q&31)|0;p=ac(l,q&2016)|0;b[g+(n<<1)>>1]=(p+134283264+k+(ac(q&63488,m)|0)|0)>>>17&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12418;break}}if((o|0)==12412){AN(0,0,1639344,(n=i,i=i+24|0,c[n>>2]=1718488,c[n+8>>2]=1690768,c[n+16>>2]=635,n)|0);i=n;cw()}else if((o|0)==12418){i=h;return}}function yv(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]<<10;l=c[k+4>>2]<<5;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(50)|0;if((k|0)==0){o=12421;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(e,q&31)|0;p=ac(l,q&992)|0;b[g+(n<<1)>>1]=(p+67141632+k+(ac(q&31744,m)|0)|0)>>>16&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12426;break}}if((o|0)==12426){i=h;return}else if((o|0)==12421){AN(0,0,1639344,(o=i,i=i+24|0,c[o>>2]=1718488,c[o+8>>2]=1690768,c[o+16>>2]=635,o)|0);i=o;cw()}}function yw(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]<<10;l=c[k+4>>2]<<5;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(49)|0;if((k|0)==0){o=12430;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(e,q&31)|0;p=ac(l,q&992)|0;b[g+(n<<1)>>1]=(p+67141632+k+(ac(q&31744,m)|0)|0)>>>16&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12436;break}}if((o|0)==12430){AN(0,0,1639344,(n=i,i=i+24|0,c[n>>2]=1718488,c[n+8>>2]=1690768,c[n+16>>2]=635,n)|0);i=n;cw()}else if((o|0)==12436){i=h;return}}function yx(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]<<8;l=c[k+4>>2]<<4;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(64)|0;if((k|0)==0){o=12439;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(e,q&15)|0;p=ac(l,q&240)|0;b[g+(n<<1)>>1]=(p+8392704+k+(ac(q&3840,m)|0)|0)>>>13&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12444;break}}if((o|0)==12439){AN(0,0,1639344,(n=i,i=i+24|0,c[n>>2]=1718488,c[n+8>>2]=1690768,c[n+16>>2]=635,n)|0);i=n;cw()}else if((o|0)==12444){i=h;return}}function yy(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]<<8;l=c[k+4>>2]<<4;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(65)|0;if((k|0)==0){o=12448;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(e,q&15)|0;p=ac(l,q&240)|0;b[g+(n<<1)>>1]=(p+8392704+k+(ac(q&3840,m)|0)|0)>>>13&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12454;break}}if((o|0)==12448){AN(0,0,1639344,(n=i,i=i+24|0,c[n>>2]=1718488,c[n+8>>2]=1690768,c[n+16>>2]=635,n)|0);i=n;cw()}else if((o|0)==12454){i=h;return}}function yz(a,e,f,g,h,i){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0;g=a;a=c[i>>2]|0;f=c[i+4>>2]|0;j=c[i+8>>2]|0;if((h|0)>0){k=0}else{return}do{i=k*3|0;l=d[e+(i+1)|0]|0;m=d[e+(i+2)|0]|0;n=ac(d[e+i|0]|0,a)|0;i=ac(l,f)|0;b[g+(k<<1)>>1]=(n+524544+i+(ac(m,j)|0)|0)>>>9&65535;k=k+1|0;}while((k|0)<(h|0));return}function yA(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]|0;l=c[k+4>>2]<<5;m=c[k+8>>2]<<11;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(44)|0;if((k|0)==0){o=12462;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(q&63488,e)|0;p=ac(l,q&2016)|0;b[g+(n<<1)>>1]=(p+134283264+k+(ac(m,q&31)|0)|0)>>>17&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12467;break}}if((o|0)==12467){i=h;return}else if((o|0)==12462){AN(0,0,1639344,(o=i,i=i+24|0,c[o>>2]=1718488,c[o+8>>2]=1690768,c[o+16>>2]=635,o)|0);i=o;cw()}}function yB(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]|0;l=c[k+4>>2]<<5;m=c[k+8>>2]<<11;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(43)|0;if((k|0)==0){o=12471;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(q&63488,e)|0;p=ac(l,q&2016)|0;b[g+(n<<1)>>1]=(p+134283264+k+(ac(m,q&31)|0)|0)>>>17&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12476;break}}if((o|0)==12476){i=h;return}else if((o|0)==12471){AN(0,0,1639344,(o=i,i=i+24|0,c[o>>2]=1718488,c[o+8>>2]=1690768,c[o+16>>2]=635,o)|0);i=o;cw()}}function yC(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]|0;l=c[k+4>>2]<<5;m=c[k+8>>2]<<10;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(46)|0;if((k|0)==0){o=12480;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(q&31744,e)|0;p=ac(l,q&992)|0;b[g+(n<<1)>>1]=(p+67141632+k+(ac(m,q&31)|0)|0)>>>16&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12486;break}}if((o|0)==12480){AN(0,0,1639344,(n=i,i=i+24|0,c[n>>2]=1718488,c[n+8>>2]=1690768,c[n+16>>2]=635,n)|0);i=n;cw()}else if((o|0)==12486){i=h;return}}function yD(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]|0;l=c[k+4>>2]<<5;m=c[k+8>>2]<<10;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(45)|0;if((k|0)==0){o=12489;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(q&31744,e)|0;p=ac(l,q&992)|0;b[g+(n<<1)>>1]=(p+67141632+k+(ac(m,q&31)|0)|0)>>>16&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12495;break}}if((o|0)==12489){AN(0,0,1639344,(n=i,i=i+24|0,c[n>>2]=1718488,c[n+8>>2]=1690768,c[n+16>>2]=635,n)|0);i=n;cw()}else if((o|0)==12495){i=h;return}}function yE(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]|0;l=c[k+4>>2]<<4;m=c[k+8>>2]<<8;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(62)|0;if((k|0)==0){o=12498;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(q&3840,e)|0;p=ac(l,q&240)|0;b[g+(n<<1)>>1]=(p+8392704+k+(ac(m,q&15)|0)|0)>>>13&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12504;break}}if((o|0)==12498){AN(0,0,1639344,(n=i,i=i+24|0,c[n>>2]=1718488,c[n+8>>2]=1690768,c[n+16>>2]=635,n)|0);i=n;cw()}else if((o|0)==12504){i=h;return}}function yF(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;g=e;e=c[k>>2]|0;l=c[k+4>>2]<<4;m=c[k+8>>2]<<8;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(63)|0;if((k|0)==0){o=12507;break}p=f+(n<<1)|0;q=(d[p]|d[p+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){r=q}else{r=cT(q|0)|0}q=r&65535;k=ac(q&3840,e)|0;p=ac(l,q&240)|0;b[g+(n<<1)>>1]=(p+8392704+k+(ac(m,q&15)|0)|0)>>>13&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12513;break}}if((o|0)==12507){AN(0,0,1639344,(n=i,i=i+24|0,c[n>>2]=1718488,c[n+8>>2]=1690768,c[n+16>>2]=635,n)|0);i=n;cw()}else if((o|0)==12513){i=h;return}}function yG(a,e,f,g,h,i){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0;g=a;if((h|0)>0){j=0}else{return}do{b[g+(j<<1)>>1]=c[i+((d[e+j|0]|0)<<2)>>2]<<6&16320;j=j+1|0;}while((j|0)<(h|0));return}function yH(a,c,e,f,g,h){a=a|0;c=c|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;h=a;a=g+7>>3;if((a|0)>0){g=0;while(1){f=d[c+g|0]|0;e=g<<3;b[h+(e<<1)>>1]=-(f>>>7)&16383;b[h+((e|1)<<1)>>1]=-(f>>>6&1)&16383;b[h+((e|2)<<1)>>1]=-(f>>>5&1)&16383;b[h+((e|3)<<1)>>1]=-(f>>>4&1)&16383;b[h+((e|4)<<1)>>1]=-(f>>>3&1)&16383;b[h+((e|5)<<1)>>1]=-(f>>>2&1)&16383;b[h+((e|6)<<1)>>1]=-(f>>>1&1)&16383;b[h+((e|7)<<1)>>1]=-(f&1)&16383;f=g+1|0;if((f|0)<(a|0)){g=f}else{i=a;break}}}else{i=0}g=a&7;if((g|0)==0){return}a=d[c+i|0]|0;c=i<<3;i=0;do{b[h+(i+c<<1)>>1]=-(a>>>((7-i|0)>>>0)&1)&16383;i=i+1|0;}while((i|0)<(g|0));return}function yI(a,c,e,f,g,h){a=a|0;c=c|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;h=a;a=g+7>>3;if((a|0)>0){g=0;while(1){f=~(d[c+g|0]|0);e=g<<3;b[h+(e<<1)>>1]=-(f>>>7&1)&16383;b[h+((e|1)<<1)>>1]=-(f>>>6&1)&16383;b[h+((e|2)<<1)>>1]=-(f>>>5&1)&16383;b[h+((e|3)<<1)>>1]=-(f>>>4&1)&16383;b[h+((e|4)<<1)>>1]=-(f>>>3&1)&16383;b[h+((e|5)<<1)>>1]=-(f>>>2&1)&16383;b[h+((e|6)<<1)>>1]=-(f>>>1&1)&16383;b[h+((e|7)<<1)>>1]=-(f&1)&16383;f=g+1|0;if((f|0)<(a|0)){g=f}else{i=a;break}}}else{i=0}g=a&7;if((g|0)==0){return}a=~(d[c+i|0]|0);c=i<<3;i=0;do{b[h+(i+c<<1)>>1]=-(a>>>((7-i|0)>>>0)&1)&16383;i=i+1|0;}while((i|0)<(g|0));return}function yJ(a,d,e,f,g,h){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0;f=a;a=c[h>>2]|0;e=c[h+4>>2]|0;i=c[h+8>>2]|0;if((g|0)>0){j=0}else{return}do{h=c[d+(j<<2)>>2]|0;k=ac(h>>>16&255,a)|0;l=ac(h&65280,e)|0;b[f+(j<<1)>>1]=(l+134283264+(k+(ac(h&255,i)|0)<<8)|0)>>>17&65535;j=j+1|0;}while((j|0)<(g|0));return}function yK(a,d,e,f,g,h){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0;f=a;a=c[h>>2]|0;e=c[h+4>>2]|0;i=c[h+8>>2]|0;if((g|0)>0){j=0}else{return}do{h=c[d+(j<<2)>>2]|0;k=h>>>8;l=ac(h>>>24,a)|0;h=ac(k&65280,e)|0;b[f+(j<<1)>>1]=(h+134283264+((ac(k&255,i)|0)+l<<8)|0)>>>17&65535;j=j+1|0;}while((j|0)<(g|0));return}function yL(a,d,e,f,g,h){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0;f=a;a=c[h>>2]|0;e=c[h+4>>2]|0;i=c[h+8>>2]|0;if((g|0)>0){j=0}else{return}do{h=c[d+(j<<2)>>2]|0;k=ac(h&255,a)|0;l=ac(h&65280,e)|0;b[f+(j<<1)>>1]=(l+134283264+((ac(h>>>16&255,i)|0)+k<<8)|0)>>>17&65535;j=j+1|0;}while((j|0)<(g|0));return}function yM(a,d,e,f,g,h){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0;f=a;a=c[h>>2]|0;e=c[h+4>>2]|0;i=c[h+8>>2]|0;if((g|0)>0){j=0}else{return}do{h=c[d+(j<<2)>>2]|0;k=h>>>8;l=ac(k&255,a)|0;m=ac(k&65280,e)|0;b[f+(j<<1)>>1]=(m+134283264+(l+(ac(h>>>24,i)|0)<<8)|0)>>>17&65535;j=j+1|0;}while((j|0)<(g|0));return}function yN(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=i;g=f;f=e;e=c[k>>2]|0;l=c[k+4>>2]|0;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(41)|0;if((k|0)==0){o=12557;break}p=n*3|0;q=g+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){s=r}else{s=cT(r|0)|0}r=BP(41)|0;if((r|0)==0){o=12561;break}k=g+(p+1<<1)|0;q=(d[k]|d[k+1|0]<<8)<<16>>16;if((a[r+7|0]&1)==0){t=q}else{t=cT(q|0)|0}q=BP(41)|0;if((q|0)==0){o=12565;break}r=g+(p+2<<1)|0;p=(d[r]|d[r+1|0]<<8)<<16>>16;if((a[q+7|0]&1)==0){u=p}else{u=cT(p|0)|0}p=ac(s&65535,e)|0;q=ac(t&65535,l)|0;b[f+(n<<1)>>1]=(p+134234112+q+(ac(u&65535,m)|0)|0)>>>15&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12571;break}}if((o|0)==12565){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12571){i=h;return}else if((o|0)==12557){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12561){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}}function yO(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=i;g=f;f=e;e=c[k>>2]|0;l=c[k+4>>2]|0;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(42)|0;if((k|0)==0){o=12574;break}p=n*3|0;q=g+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){s=r}else{s=cT(r|0)|0}r=BP(42)|0;if((r|0)==0){o=12578;break}k=g+(p+1<<1)|0;q=(d[k]|d[k+1|0]<<8)<<16>>16;if((a[r+7|0]&1)==0){t=q}else{t=cT(q|0)|0}q=BP(42)|0;if((q|0)==0){o=12582;break}r=g+(p+2<<1)|0;p=(d[r]|d[r+1|0]<<8)<<16>>16;if((a[q+7|0]&1)==0){u=p}else{u=cT(p|0)|0}p=ac(s&65535,e)|0;q=ac(t&65535,l)|0;b[f+(n<<1)>>1]=(p+134234112+q+(ac(u&65535,m)|0)|0)>>>15&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12587;break}}if((o|0)==12578){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12582){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12574){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12587){i=h;return}}function yP(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=i;g=f;f=e;e=c[k>>2]|0;l=c[k+4>>2]|0;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(67)|0;if((k|0)==0){o=12591;break}p=n*3|0;q=g+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){s=r}else{s=cT(r|0)|0}r=BP(67)|0;if((r|0)==0){o=12595;break}k=g+(p+1<<1)|0;q=(d[k]|d[k+1|0]<<8)<<16>>16;if((a[r+7|0]&1)==0){t=q}else{t=cT(q|0)|0}q=BP(67)|0;if((q|0)==0){o=12599;break}r=g+(p+2<<1)|0;p=(d[r]|d[r+1|0]<<8)<<16>>16;if((a[q+7|0]&1)==0){u=p}else{u=cT(p|0)|0}p=ac(u&65535,e)|0;q=ac(t&65535,l)|0;b[f+(n<<1)>>1]=((ac(s&65535,m)|0)+134234112+q+p|0)>>>15&65535;p=n+1|0;if((p|0)<(j|0)){n=p}else{o=12604;break}}if((o|0)==12591){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12599){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12604){i=h;return}else if((o|0)==12595){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}}function yQ(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=i;g=f;f=e;e=c[k>>2]|0;l=c[k+4>>2]|0;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(68)|0;if((k|0)==0){o=12608;break}p=n*3|0;q=g+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){s=r}else{s=cT(r|0)|0}r=BP(68)|0;if((r|0)==0){o=12612;break}k=g+(p+1<<1)|0;q=(d[k]|d[k+1|0]<<8)<<16>>16;if((a[r+7|0]&1)==0){t=q}else{t=cT(q|0)|0}q=BP(68)|0;if((q|0)==0){o=12616;break}r=g+(p+2<<1)|0;p=(d[r]|d[r+1|0]<<8)<<16>>16;if((a[q+7|0]&1)==0){u=p}else{u=cT(p|0)|0}p=ac(u&65535,e)|0;q=ac(t&65535,l)|0;b[f+(n<<1)>>1]=((ac(s&65535,m)|0)+134234112+q+p|0)>>>15&65535;p=n+1|0;if((p|0)<(j|0)){n=p}else{o=12621;break}}if((o|0)==12612){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12608){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12621){i=h;return}else if((o|0)==12616){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}}function yR(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=i;g=f;f=e;e=c[k>>2]|0;l=c[k+4>>2]|0;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(291)|0;if((k|0)==0){o=12625;break}p=n<<2;q=g+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){s=r}else{s=cT(r|0)|0}r=BP(291)|0;if((r|0)==0){o=12629;break}k=g+((p|1)<<1)|0;q=(d[k]|d[k+1|0]<<8)<<16>>16;if((a[r+7|0]&1)==0){t=q}else{t=cT(q|0)|0}q=BP(291)|0;if((q|0)==0){o=12633;break}r=g+((p|2)<<1)|0;p=(d[r]|d[r+1|0]<<8)<<16>>16;if((a[q+7|0]&1)==0){u=p}else{u=cT(p|0)|0}p=ac(s&65535,e)|0;q=ac(t&65535,l)|0;b[f+(n<<1)>>1]=(p+134234112+q+(ac(u&65535,m)|0)|0)>>>15&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12638;break}}if((o|0)==12625){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12633){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12638){i=h;return}else if((o|0)==12629){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}}function yS(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=i;g=f;f=e;e=c[k>>2]|0;l=c[k+4>>2]|0;m=c[k+8>>2]|0;if((j|0)>0){n=0}else{i=h;return}while(1){k=BP(292)|0;if((k|0)==0){o=12642;break}p=n<<2;q=g+(p<<1)|0;r=(d[q]|d[q+1|0]<<8)<<16>>16;if((a[k+7|0]&1)==0){s=r}else{s=cT(r|0)|0}r=BP(292)|0;if((r|0)==0){o=12646;break}k=g+((p|1)<<1)|0;q=(d[k]|d[k+1|0]<<8)<<16>>16;if((a[r+7|0]&1)==0){t=q}else{t=cT(q|0)|0}q=BP(292)|0;if((q|0)==0){o=12650;break}r=g+((p|2)<<1)|0;p=(d[r]|d[r+1|0]<<8)<<16>>16;if((a[q+7|0]&1)==0){u=p}else{u=cT(p|0)|0}p=ac(s&65535,e)|0;q=ac(t&65535,l)|0;b[f+(n<<1)>>1]=(p+134234112+q+(ac(u&65535,m)|0)|0)>>>15&65535;q=n+1|0;if((q|0)<(j|0)){n=q}else{o=12655;break}}if((o|0)==12646){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12642){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12650){AN(0,0,1639344,(v=i,i=i+24|0,c[v>>2]=1718488,c[v+8>>2]=1690768,c[v+16>>2]=635,v)|0);i=v;cw()}else if((o|0)==12655){i=h;return}}function yT(a,c,d,e,f,g){a=a|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0;g=a;a=c;if((f|0)>0){h=0}else{return}do{b[g+(h<<1)>>1]=b[a+((h<<2|3)<<1)>>1]|0;h=h+1|0;}while((h|0)<(f|0));return}function yU(a,c,e,f,g,h){a=a|0;c=c|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;h=a;if((g|0)>0){i=0}else{return}do{b[h+(i<<1)>>1]=(d[c+(i<<2|3)|0]|0)<<6;i=i+1|0;}while((i|0)<(g|0));return}function yV(a,c,e,f,g,h){a=a|0;c=c|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;h=a;if((g|0)>0){i=0}else{return}do{b[h+(i<<1)>>1]=(d[c+(i<<2)|0]|0)<<6;i=i+1|0;}while((i|0)<(g|0));return}function yW(a,e,f,g,h,i){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0;g=a;if((h|0)>0){j=0}else{return}do{b[g+(j<<1)>>1]=(c[i+((d[e+j|0]|0)<<2)>>2]|0)>>>24<<6&65535;j=j+1|0;}while((j|0)<(h|0));return}function yX(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;if((a|0)==0){return}b=a|0;d=c[b>>2]|0;if((d|0)!=0){e=a+4|0;f=c[e>>2]|0;if((f|0)>0){g=0;h=d;i=f;while(1){f=c[h+(g<<2)>>2]|0;if((f|0)==0){j=i;k=h}else{z0(f);j=c[e>>2]|0;k=c[b>>2]|0}f=g+1|0;if((f|0)<(j|0)){g=f;h=k;i=j}else{l=k;break}}}else{l=d}A4(l)}A4(a);return}function yY(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;f=a[d]|0;L17207:do{if(f<<24>>24==0){g=d;h=b;i=12691}else{j=d;k=b;l=f;while(1){if(l<<24>>24!=(a[k]|0)){m=l;break L17207}n=j+1|0;o=k+1|0;p=a[n]|0;if(p<<24>>24==0){g=n;h=o;i=12691;break}else{j=n;k=o;l=p}}}}while(0);do{if((i|0)==12691){if((e|0)==0){m=0;break}c[e>>2]=h;m=a[g]|0}}while(0);return m<<24>>24==0|0}function yZ(b,c){b=b|0;c=c|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;d=a[c]|0;if(d<<24>>24==0){e=b;return e|0}else{f=b}L17218:while(1){b=c;g=f;h=d;while(1){i=h<<24>>24;j=a[g]|0;if((((i-97|0)>>>0<26?i^32:i)|0)!=(((j-97|0)>>>0<26?j^32:j)|0)){break}j=b+1|0;i=a[j]|0;if(i<<24>>24==0){e=f;k=12702;break L17218}else{b=j;g=g+1|0;h=i}}if(h<<24>>24==0){e=f;k=12705;break}if((a[f]|0)==0){e=0;k=12704;break}f=f+1|0}if((k|0)==12704){return e|0}else if((k|0)==12702){return e|0}else if((k|0)==12705){return e|0}return 0}function y_(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;L17231:do{if(d>>>0>1){e=c;f=b;g=1;while(1){h=a[e]|0;if(h<<24>>24==0){i=e;j=f;k=g;break L17231}l=e+1|0;m=f+1|0;a[f]=h;h=g+1|0;if(h>>>0<d>>>0){e=l;f=m;g=h}else{i=l;j=m;k=h;break}}}else{i=c;j=b;k=1}}while(0);if(k>>>0<=d>>>0){a[j]=0}return k-1+(NL(i|0)|0)|0}function y$(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=NL(b|0)|0;if((e+1|0)>>>0>=d>>>0){f=NL(c|0)|0;g=f+e|0;return g|0}h=b+e|0;b=d-e|0;L17244:do{if(b>>>0>1){d=c;i=h;j=1;while(1){k=a[d]|0;if(k<<24>>24==0){l=d;m=i;n=j;break L17244}o=d+1|0;p=i+1|0;a[i]=k;k=j+1|0;if(k>>>0<b>>>0){d=o;i=p;j=k}else{l=o;m=p;n=k;break}}}else{l=c;m=h;n=1}}while(0);if(n>>>0<=b>>>0){a[m]=0}f=n-1+(NL(l|0)|0)|0;g=f+e|0;return g|0}function y0(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0;f=i;i=i+16|0;g=f|0;h=NL(a|0)|0;j=g;c[j>>2]=e;c[j+4>>2]=0;j=(cs(a+h|0,(h>>>0<b>>>0?b-h|0:0)|0,d|0,g|0)|0)+h|0;i=f;return j|0}function y1(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0;d=i;i=i+24|0;e=d|0;f=d+8|0;c[e>>2]=0;g=f|0;h=f;c[h>>2]=b;c[h+4>>2]=0;f=cs(0,0,a|0,g|0)|0;do{if((f|0)>=0){j=f+1|0;k=A1(j)|0;c[e>>2]=k;if((k|0)==0){break}c[h>>2]=b;c[h+4>>2]=0;if((cs(k|0,j|0,a|0,g|0)|0)>=0){break}A6(e)}}while(0);i=d;return c[e>>2]|0}function y2(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0;e=A1((NL(c[b>>2]|0)|0)+1|0)|0;f=c[b>>2]|0;if((e|0)==0){g=0;return g|0}h=f+(cm(f|0,1718416)|0)|0;f=a[h]|0;L17264:do{if(f<<24>>24==0){i=e;j=h;k=e}else{l=e;m=e;n=h;o=f;while(1){p=l;q=n;r=o;while(1){if((cm(q|0,d|0)|0)!=0){i=p;j=q;k=m;break L17264}s=q+1|0;if((r<<24>>24|0)==39){t=12735;break}else if((r<<24>>24|0)==92){u=a[s]|0;if(u<<24>>24!=0){t=12737;break}}v=p+1|0;a[p]=r;w=a[s]|0;if(w<<24>>24==0){i=v;j=s;k=m;break L17264}else{p=v;q=s;r=w}}if((t|0)==12735){t=0;r=a[s]|0;w=r<<24>>24==0;v=q+2|0;if(r<<24>>24!=39&(w^1)){x=p;y=r;r=v;while(1){z=x+1|0;a[x]=y;A=a[r]|0;B=A<<24>>24==0;C=r+1|0;if(A<<24>>24!=39&(B^1)){x=z;y=A;r=C}else{D=z;E=r;F=B;G=C;break}}}else{D=p;E=s;F=w;G=v}H=F?E:G;I=F?m:D;J=D}else if((t|0)==12737){t=0;r=p+1|0;a[p]=u;H=q+2|0;I=r;J=r}r=a[H]|0;if(r<<24>>24==0){i=J;j=H;k=I;break}else{l=J;m=I;n=H;o=r}}}}while(0);H=i;while(1){i=H-1|0;a[H]=0;if(i>>>0<k>>>0){break}if((cm(i|0,1718416)|0)==0){break}else{H=i}}c[b>>2]=j;g=e;return g|0}function y3(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0;do{if((b|0)==0){f=c[e>>2]|0;if((f|0)==0){g=0}else{h=f;break}return g|0}else{h=b}}while(0);b=cm(h|0,d|0)|0;f=h+b|0;if((a[f]|0)==0){c[e>>2]=0;g=0;return g|0}i=b+1|0;b=(bv(h+i|0,d|0)|0)+i|0;i=h+b|0;if((a[i]|0)==0){j=0}else{a[i]=0;j=h+(b+1)|0}c[e>>2]=j;g=f;return g|0}function y4(b,c){b=b|0;c=c|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0;d=b;b=c;while(1){c=a[d]|0;e=a[b]|0;f=(e-65|0)>>>0<26?e^32:e;e=((c-65|0)>>>0<26?c^32:c)&255;if((e|0)==0){break}c=f&255;if((e|0)==(c|0)){d=d+1|0;b=b+1|0}else{g=e;h=c;i=12764;break}}if((i|0)==12764){j=g-h|0;return j|0}g=0;h=f&255;j=g-h|0;return j|0}function y5(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;e=b+d|0;d=b;b=c;while(1){c=d+1|0;f=a[d]|0;g=a[b]|0;h=(g-65|0)>>>0<26?g^32:g;g=((f-65|0)>>>0<26?f^32:f)&255;if(c>>>0>=e>>>0){i=g;break}if((g|0)==0){i=0;break}if((g|0)==(h&255|0)){d=c;b=b+1|0}else{i=g;break}}return i-(h&255)|0}function y6(a){a=a|0;var b=0;if((a|0)>47){b=(a|0)<58|0}else{b=0}return b|0}function y7(a){a=a|0;var b=0;switch(a|0){case 32:case 13:case 12:case 10:case 9:{b=1;break};default:{b=(a|0)==11|0}}return b|0}function y8(b,c,e){b=b|0;c=c|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;f=b+e|0;g=f;L17323:do{if((e|0)>3){h=b;i=c;while(1){j=d[129944+(d[i]|0)|0]|0;if((j&128|0)!=0){k=j;l=h;break L17323}m=d[129944+(d[i+1|0]|0)|0]|0;if((m&128|0)!=0){k=m;l=h;break L17323}n=m+(j<<6)|0;o=d[129944+(d[i+2|0]|0)|0]|0;if((o&128|0)!=0){p=12800;break}q=o+(n<<6)|0;r=d[129944+(d[i+3|0]|0)|0]|0;if((r&128|0)!=0){p=12798;break}j=r+(q<<6)|0;m=h;y=j>>>16&255|j&65280|j<<16&16711680;a[m]=y&255;y=y>>8;a[m+1|0]=y&255;y=y>>8;a[m+2|0]=y&255;y=y>>8;a[m+3|0]=y&255;m=h+3|0;j=i+4|0;if((g-m|0)>3){h=m;i=j}else{s=m;t=j;p=12783;break L17323}}if((p|0)==12798){u=q>>>10&255;v=q>>>2&255;w=r;x=h;p=12799;break}else if((p|0)==12800){z=n>>>4&255;A=o;B=h;p=12801;break}}else{s=b;t=c;p=12783}}while(0);L17333:do{if((p|0)==12783){if((f|0)==(s|0)){C=t;D=f}else{c=d[129944+(d[t]|0)|0]|0;if((c&128|0)!=0){k=c;l=s;break}o=d[129944+(d[t+1|0]|0)|0]|0;if((o&128|0)!=0){k=o;l=s;break}n=o+(c<<6)|0;c=d[129944+(d[t+2|0]|0)|0]|0;if((c&128|0)!=0){z=n>>>4&255;A=c;B=s;p=12801;break}o=c+(n<<6)|0;n=d[129944+(d[t+3|0]|0)|0]|0;if((n&128|0)!=0){u=o>>>10&255;v=o>>>2&255;w=n;x=s;p=12799;break}c=n+(o<<6)|0;o=s+1|0;a[s]=c>>>16&255;if((f|0)==(o|0)){E=f}else{a[o]=c>>>8&255;E=s+2|0}if((f|0)==(E|0)){F=f}else{a[E]=c&255;F=E+1|0}C=t+4|0;D=F}c=C;while(1){o=d[129944+(d[c]|0)|0]|0;if((o&128|0)!=0){k=o;l=D;break L17333}o=d[129944+(d[c+1|0]|0)|0]|0;if((o&128|0)!=0){k=o;l=D;break L17333}o=d[129944+(d[c+2|0]|0)|0]|0;if((o&128|0)!=0){k=o;l=D;break L17333}o=d[129944+(d[c+3|0]|0)|0]|0;if((o&128|0)==0){c=c+4|0}else{k=o;l=D;break}}}}while(0);if((p|0)==12799){a[x]=u;z=v;A=w;B=x+1|0;p=12801}if((p|0)==12801){a[B]=z;k=A;l=B+1|0}if((k&1|0)!=0){G=-1094995529;return G|0}G=l-b|0;return G|0}function y9(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0;f=b+16|0;g=(e|0)==1?1008:e;e=b|0;c[e>>2]=f;h=b+4|0;c[h>>2]=0;i=g>>>0<1008?g:1008;j=b+8|0;c[j>>2]=i;c[b+12>>2]=g;a[f]=0;if(i>>>0>=d>>>0){return}if((i|0)==(g|0)|(i|0)==0){return}f=(d-1|0)>>>0<4294967294?d:-1;d=i>>>0>g>>>1>>>0?g:i<<1;if(d>>>0<f>>>0){k=g>>>0>f>>>0?f:g}else{k=d}d=A2(0,k)|0;if((d|0)==0){return}g=c[e>>2]|0;f=(c[h>>2]|0)+1|0;NN(d|0,g|0,f)|0;c[e>>2]=d;c[j>>2]=k;return}function za(b,d,e){b=b|0;d=d|0;e=e|0;c[b>>2]=d;c[b+4>>2]=0;c[b+8>>2]=e;c[b+12>>2]=e;a[d]=0;return}function zb(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;f=i;i=i+16|0;g=f|0;h=b+8|0;j=b+4|0;k=g|0;l=g;g=b+12|0;m=b|0;n=b+16|0;b=c[h>>2]|0;while(1){o=c[j>>2]|0;p=o>>>0>b>>>0?b:o;q=b-p|0;if((b|0)==(p|0)){r=0}else{r=(c[m>>2]|0)+o|0}c[l>>2]=e;c[l+4>>2]=0;s=cs(r|0,q|0,d|0,k|0)|0;if((s|0)<1){t=12835;break}if(s>>>0<q>>>0){break}q=c[h>>2]|0;o=c[g>>2]|0;if((q|0)==(o|0)){break}p=c[j>>2]|0;if(p>>>0>=q>>>0){break}u=-2-p|0;v=p+1+(u>>>0>s>>>0?s:u)|0;u=q>>>0>o>>>1>>>0?o:q<<1;if(u>>>0<v>>>0){w=o>>>0>v>>>0?v:o}else{w=u}u=c[m>>2]|0;o=(u|0)==(n|0)?0:u;u=A2(o,w)|0;if((u|0)==0){break}if((o|0)==0){o=c[m>>2]|0;v=(c[j>>2]|0)+1|0;NN(u|0,o|0,v)|0}c[m>>2]=u;c[h>>2]=w;b=w}if((t|0)==12835){i=f;return}t=c[j>>2]|0;w=-6-t|0;b=(w>>>0<s>>>0?w:s)+t|0;c[j>>2]=b;j=c[h>>2]|0;if((j|0)==0){i=f;return}h=j-1|0;a[(c[m>>2]|0)+(b>>>0>h>>>0?h:b)|0]=0;i=f;return}function zc(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;f=b+12|0;g=b+4|0;h=b+8|0;i=c[h>>2]|0;j=c[g>>2]|0;k=j>>>0>i>>>0?i:j;l=i-k|0;L17402:do{if(l>>>0>e>>>0){m=i;n=k;o=l}else{p=b|0;q=b+16|0;r=i;s=k;t=l;u=j;while(1){v=c[f>>2]|0;if(!((r|0)!=(v|0)&u>>>0<r>>>0)){m=r;n=s;o=t;break L17402}w=-2-u|0;x=u+1+(w>>>0>e>>>0?e:w)|0;w=r>>>0>v>>>1>>>0?v:r<<1;if(w>>>0<x>>>0){y=v>>>0>x>>>0?x:v}else{y=w}w=c[p>>2]|0;v=(w|0)==(q|0)?0:w;w=A2(v,y)|0;if((w|0)==0){m=r;n=s;o=t;break L17402}if((v|0)==0){v=c[p>>2]|0;x=(c[g>>2]|0)+1|0;NN(w|0,v|0,x)|0}c[p>>2]=w;c[h>>2]=y;w=c[g>>2]|0;x=w>>>0>y>>>0?y:w;v=y-x|0;if(v>>>0>e>>>0){m=y;n=x;o=v;break}else{r=y;s=x;t=v;u=w}}}}while(0);if((m|0)!=(n|0)){n=o-1|0;NM((c[b>>2]|0)+(c[g>>2]|0)|0,d|0,(n>>>0<e>>>0?n:e)|0)}n=c[g>>2]|0;d=-6-n|0;o=(d>>>0<e>>>0?d:e)+n|0;c[g>>2]=o;g=c[h>>2]|0;if((g|0)==0){return}h=g-1|0;a[(c[b>>2]|0)+(o>>>0>h>>>0?h:o)|0]=0;return}function zd(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;d=(c[a+4>>2]|0)+1|0;e=a+8|0;f=c[e>>2]|0;g=d>>>0>f>>>0?f:d;d=a|0;f=c[d>>2]|0;h=(f|0)!=(a+16|0);if((b|0)==0){if(!h){i=0;c[e>>2]=g;return i|0}A6(a);i=0;c[e>>2]=g;return i|0}do{if(h){a=A2(f,g)|0;if((a|0)==0){j=c[d>>2]|0}else{j=a}c[d>>2]=0;k=0;l=j}else{a=A1(g)|0;if((a|0)==0){k=-12;l=0;break}m=c[d>>2]|0;NN(a|0,m|0,g)|0;k=0;l=a}}while(0);c[b>>2]=l;i=k;c[e>>2]=g;return i|0}function ze(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0;g=i;i=i+8|0;h=g|0;j=A7(24)|0;c[h>>2]=j;if((j|0)==0){k=0;i=g;return k|0}c[j>>2]=a;c[j+4>>2]=b;c[j+12>>2]=(d|0)!=0?d:28;c[j+16>>2]=e;c[j+8>>2]=1;if((f&1|0)!=0){f=(c[h>>2]|0)+20|0;c[f>>2]=c[f>>2]|1}f=A7(12)|0;if((f|0)==0){A6(h);k=0;i=g;return k|0}else{c[f>>2]=c[h>>2];c[f+4>>2]=a;c[f+8>>2]=b;k=f;i=g;return k|0}return 0}function zf(a,b){a=a|0;b=b|0;A4(b);return}function zg(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0;b=i;i=i+16|0;d=b|0;e=b+8|0;f=A1(a)|0;c[e>>2]=f;if((f|0)==0){g=0;i=b;return g|0}h=d;j=A7(24)|0;c[d>>2]=j;do{if((j|0)!=0){c[j>>2]=f;c[j+4>>2]=a;c[j+12>>2]=28;c[j+16>>2]=0;c[j+8>>2]=1;k=A7(12)|0;if((k|0)==0){A6(h);break}c[k>>2]=c[d>>2];c[k+4>>2]=f;c[k+8>>2]=a;g=k;i=b;return g|0}}while(0);A6(e);g=0;i=b;return g|0}function zh(a){a=a|0;var b=0,d=0;b=zg(a)|0;if((b|0)==0){d=0;return d|0}NM(c[b+4>>2]|0,0,a|0);d=b;return d|0}function zi(a){a=a|0;var b=0,d=0,e=0;b=A7(12)|0;if((b|0)==0){d=0;return d|0}e=a;c[b>>2]=c[e>>2];c[b+4>>2]=c[e+4>>2];c[b+8>>2]=c[e+8>>2];e=(c[a>>2]|0)+8|0;E=c[e>>2]|0,c[e>>2]=E+1,E;d=b;return d|0}function zj(a){a=a|0;var b=0,d=0,e=0,f=0;b=i;i=i+8|0;d=b|0;if((a|0)==0){i=b;return}e=c[a>>2]|0;if((e|0)==0){i=b;return}f=c[e>>2]|0;c[d>>2]=f;A6(a);a=f+8|0;if(((E=c[a>>2]|0,c[a>>2]=E+ -1,E)|0)!=1){i=b;return}a=c[d>>2]|0;c$[c[a+12>>2]&127](c[a+16>>2]|0,c[a>>2]|0);A6(d);i=b;return}function zk(a){a=a|0;var b=0,d=0;b=c[a>>2]|0;if((c[b+20>>2]&1|0)!=0){d=0;return d|0}d=(c[b+8>>2]|0)==1|0;return d|0}function zl(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;d=i;i=i+32|0;e=d|0;f=d+8|0;g=d+16|0;h=d+24|0;j=c[a>>2]|0;if((j|0)==0){k=A2(0,b)|0;c[g>>2]=k;if((k|0)==0){l=-12;i=d;return l|0}m=f;n=A7(24)|0;c[f>>2]=n;do{if((n|0)!=0){c[n>>2]=k;c[n+4>>2]=b;c[n+12>>2]=28;c[n+16>>2]=0;c[n+8>>2]=1;o=A7(12)|0;if((o|0)==0){A6(m);break}p=c[f>>2]|0;c[o>>2]=p;c[o+4>>2]=k;c[o+8>>2]=b;q=p+20|0;c[q>>2]=c[q>>2]|2;c[a>>2]=o;l=0;i=d;return l|0}}while(0);A6(g);l=-12;i=d;return l|0}g=j+8|0;if((c[g>>2]|0)==(b|0)){l=0;i=d;return l|0}k=j|0;f=c[k>>2]|0;do{if((c[f+20>>2]&3|0)==2){if((c[f+8>>2]|0)!=1){break}m=A2(c[c[k>>2]>>2]|0,b)|0;if((m|0)==0){l=-12;i=d;return l|0}c[j+4>>2]=m;c[c[k>>2]>>2]=m;c[g>>2]=b;c[(c[k>>2]|0)+4>>2]=b;l=0;i=d;return l|0}}while(0);c[h>>2]=0;zl(h,b)|0;k=c[h>>2]|0;if((k|0)==0){l=-12;i=d;return l|0}f=c[k+4>>2]|0;k=c[j+4>>2]|0;j=c[g>>2]|0;g=(j|0)<(b|0)?j:b;NN(f|0,k|0,g)|0;g=e;do{if((a|0)!=0){k=c[a>>2]|0;if((k|0)==0){break}f=c[k>>2]|0;c[e>>2]=f;A6(a);k=f+8|0;if(((E=c[k>>2]|0,c[k>>2]=E+ -1,E)|0)!=1){break}k=c[e>>2]|0;c$[c[k+12>>2]&127](c[k+16>>2]|0,c[k>>2]|0);A6(g)}}while(0);c[a>>2]=c[h>>2];l=0;i=d;return l|0}function zm(a,b){a=a|0;b=b|0;var d=0,e=0;d=A7(20)|0;if((d|0)==0){e=0;return e|0}c[d+12>>2]=a;c[d+16>>2]=(b|0)!=0?b:126;c[d+4>>2]=1;e=d;return e|0}function zn(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=i;i=i+16|0;d=b|0;e=b+8|0;if((a|0)==0){i=b;return}f=c[a>>2]|0;if((f|0)==0){i=b;return}c[a>>2]=0;a=f+4|0;if(((E=c[a>>2]|0,c[a>>2]=E+ -1,E)|0)!=1){i=b;return}a=d;g=e;c[d>>2]=f;h=f|0;if((c[h>>2]|0)!=0){f=h;do{h=c[f>>2]|0;c[e>>2]=h;c[c[d>>2]>>2]=c[h+16>>2];h=c[e>>2]|0;c$[c[h+8>>2]&127](c[h+4>>2]|0,c[h>>2]|0);A6(g);f=c[d>>2]|0;}while((c[f>>2]|0)!=0)}A6(a);i=b;return}function zo(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;b=i;i=i+24|0;d=b|0;e=b+8|0;f=b+16|0;g=a;h=a;j=0;k=c[g>>2]|0;while(1){l=k;if((l|0)==(j|0)){m=12959;break}n=(E=c[h>>2]|0,(c[h>>2]|0)==(k|0)?(c[h>>2]=0)|0:0,E);if((n|0)==0){m=12960;break}else{j=l;k=n}}if((m|0)==12959){if((j|0)==0){m=12960}else{o=j}}L17551:do{if((m|0)==12960){j=a+4|0;k=a+8|0;L17553:do{if((c[j>>2]|0)<=(c[k>>2]|0)){AN(0,48,1638248,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;L17555:while(1){if((c[j>>2]|0)>(c[k>>2]|0)){break L17553}n=0;l=c[g>>2]|0;while(1){p=l;if((p|0)==(n|0)){break}q=(E=c[h>>2]|0,(c[h>>2]|0)==(l|0)?(c[h>>2]=0)|0:0,E);if((q|0)==0){continue L17555}else{n=p;l=q}}if((n|0)!=0){o=n;break L17551}}}}while(0);l=c1[c[a+16>>2]&255](c[a+12>>2]|0)|0;c[e>>2]=l;if((l|0)==0){r=0;i=b;return r|0}q=A7(20)|0;if((q|0)!=0){p=l|0;c[q>>2]=c[c[p>>2]>>2];c[q+4>>2]=c[(c[p>>2]|0)+16>>2];c[q+8>>2]=c[(c[p>>2]|0)+12>>2];c[q+12>>2]=a;c[(c[p>>2]|0)+16>>2]=q;c[(c[c[e>>2]>>2]|0)+12>>2]=52;E=c[j>>2]|0,c[j>>2]=E+1,E;E=c[k>>2]|0,c[k>>2]=E+1,E;r=c[e>>2]|0;i=b;return r|0}q=c[l>>2]|0;c[d>>2]=q;A6(e);l=q+8|0;if(((E=c[l>>2]|0,c[l>>2]=E+ -1,E)|0)!=1){r=0;i=b;return r|0}l=c[d>>2]|0;c$[c[l+12>>2]&127](c[l+16>>2]|0,c[l>>2]|0);A6(d);r=0;i=b;return r|0}}while(0);d=(o|0)==0;e=o+16|0;h=c[e>>2]|0;do{if((h|0)!=0){g=c[h+12>>2]|0;m=h+16|0;if((c[m>>2]|0)==0){s=h}else{l=m;while(1){m=c[l>>2]|0;q=m+16|0;if((c[q>>2]|0)==0){s=m;break}else{l=q}}}l=g;k=h;if(((E=c[l>>2]|0,(c[l>>2]|0)==0?(c[l>>2]=k)|0:0,E)|0)==0){break}j=g;q=s;while(1){m=0;p=c[j>>2]|0;while(1){t=p;if((t|0)==(m|0)){u=m;break}v=(E=c[l>>2]|0,(c[l>>2]|0)==(p|0)?(c[l>>2]=0)|0:0,E);if((v|0)==0){u=0;break}else{m=t;p=v}}p=q+16|0;c[p>>2]=u;if((c[p>>2]|0)==0){w=q}else{m=p;while(1){p=c[m>>2]|0;v=p+16|0;if((c[v>>2]|0)==0){w=p;break}else{m=v}}}if(((E=c[l>>2]|0,(c[l>>2]|0)==0?(c[l>>2]=k)|0:0,E)|0)==0){break}else{q=w}}}}while(0);c[e>>2]=0;w=c[o>>2]|0;u=c[a+12>>2]|0;s=f;h=A7(24)|0;c[f>>2]=h;do{if((h|0)!=0){c[h>>2]=w;c[h+4>>2]=u;c[h+12>>2]=52;c[h+16>>2]=o;c[h+8>>2]=1;q=A7(12)|0;if((q|0)==0){A6(s);break}c[q>>2]=c[f>>2];c[q+4>>2]=w;c[q+8>>2]=u;k=q;q=a+4|0;E=c[q>>2]|0,c[q>>2]=E+1,E;r=k;i=b;return r|0}}while(0);if(d){r=0;i=b;return r|0}d=c[o+12>>2]|0;if((c[e>>2]|0)==0){x=o}else{a=e;while(1){e=c[a>>2]|0;u=e+16|0;if((c[u>>2]|0)==0){x=e;break}else{a=u}}}a=d;u=o;if(((E=c[a>>2]|0,(c[a>>2]|0)==0?(c[a>>2]=u)|0:0,E)|0)==0){r=0;i=b;return r|0}o=d;d=x;while(1){x=0;e=c[o>>2]|0;while(1){w=e;if((w|0)==(x|0)){y=x;break}f=(E=c[a>>2]|0,(c[a>>2]|0)==(e|0)?(c[a>>2]=0)|0:0,E);if((f|0)==0){y=0;break}else{x=w;e=f}}e=d+16|0;c[e>>2]=y;if((c[e>>2]|0)==0){z=d}else{x=e;while(1){e=c[x>>2]|0;f=e+16|0;if((c[f>>2]|0)==0){z=e;break}else{x=f}}}if(((E=c[a>>2]|0,(c[a>>2]|0)==0?(c[a>>2]=u)|0:0,E)|0)==0){r=0;break}else{d=z}}i=b;return r|0}function zp(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;b=i;i=i+16|0;d=b|0;e=b+8|0;f=c[a+12>>2]|0;do{if((a|0)!=0){g=a+16|0;if((c[g>>2]|0)==0){h=a}else{j=g;while(1){g=c[j>>2]|0;k=g+16|0;if((c[k>>2]|0)==0){h=g;break}else{j=k}}}j=f;k=a;if(((E=c[j>>2]|0,(c[j>>2]|0)==0?(c[j>>2]=k)|0:0,E)|0)==0){break}g=f;l=h;while(1){m=0;n=c[g>>2]|0;while(1){o=n;if((o|0)==(m|0)){p=m;break}q=(E=c[j>>2]|0,(c[j>>2]|0)==(n|0)?(c[j>>2]=0)|0:0,E);if((q|0)==0){p=0;break}else{m=o;n=q}}n=l+16|0;c[n>>2]=p;if((c[n>>2]|0)==0){r=l}else{m=n;while(1){n=c[m>>2]|0;q=n+16|0;if((c[q>>2]|0)==0){r=n;break}else{m=q}}}if(((E=c[j>>2]|0,(c[j>>2]|0)==0?(c[j>>2]=k)|0:0,E)|0)==0){break}else{l=r}}}}while(0);r=f+4|0;if(((E=c[r>>2]|0,c[r>>2]=E+ -1,E)|0)!=1){i=b;return}r=d;p=e;c[d>>2]=f;h=f|0;if((c[h>>2]|0)!=0){f=h;do{h=c[f>>2]|0;c[e>>2]=h;c[c[d>>2]>>2]=c[h+16>>2];h=c[e>>2]|0;c$[c[h+8>>2]&127](c[h+4>>2]|0,c[h>>2]|0);A6(p);f=c[d>>2]|0;}while((c[f>>2]|0)!=0)}A6(r);i=b;return}function zq(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;d=i;i=i+8|0;e=d|0;f=NL(b|0)|0;g=b+f|0;if((f|0)<=0){h=0;j=0;i=d;return(G=h,j)|0}f=b;b=0;k=0;L17652:while(1){L17654:do{if(f>>>0<g>>>0){l=f;while(1){m=a[l]|0;if((m<<24>>24|0)==43|(m<<24>>24|0)==124){n=l;break L17654}m=l+1|0;if(m>>>0<g>>>0){l=m}else{n=m;break}}}else{n=f}}while(0);l=f;m=n-l|0;o=0;while(1){p=c[304144+(o<<4)>>2]|0;if((NL(p|0)|0)==(m|0)){if((NO(p|0,f|0,m|0)|0)==0){q=13033;break}}p=o+1|0;if(p>>>0<27){o=p}else{r=0;q=13035;break}}L17664:do{if((q|0)==13033){q=0;p=304152+(o<<4)|0;s=c[p+4>>2]|0;t=c[p>>2]|0;q=13048}else if((q|0)==13035){L17666:while(1){q=0;do{if((r-18|0)>>>0>10){p=c[303856+(r<<3)>>2]|0;if((NL(p|0)|0)!=(m|0)){break}if((NO(p|0,f|0,m|0)|0)==0){q=13038;break L17666}}}while(0);p=r+1|0;if(p>>>0<36){r=p;q=13035}else{break}}if((q|0)==13038){q=0;p=NW(1,0,r|0)|0;u=G;v=p;break}p=aU(f|0,e|0,10)|0;w=c[e>>2]|0;do{if((w-l|0)==(m|0)){x=0}else{if((w+1-l|0)==(m|0)){if((a[w]|0)==99){x=0;break}}y=cy(f|0,e|0,0)|0;z=G;if(((c[e>>2]|0)-l|0)!=(m|0)){h=0;j=0;q=13051;break L17652}A=0;B=(z|0)>(A|0)|(z|0)==(A|0)&y>>>0>0>>>0;s=B?z:0;t=B?y:0;q=13048;break L17664}}while(0);while(1){if(x>>>0>=27){h=0;j=0;q=13052;break L17652}if((c[304148+(x<<4)>>2]|0)==(p|0)){break}else{x=x+1|0}}p=304152+(x<<4)|0;s=c[p+4>>2]|0;t=c[p>>2]|0;q=13048}}while(0);if((q|0)==13048){q=0;if((t|0)==0&(s|0)==0){h=0;j=0;q=13055;break}else{u=s;v=t}}m=v|k;l=u|b;o=n+1|0;if(o>>>0<g>>>0){f=o;b=l;k=m}else{h=l;j=m;q=13054;break}}if((q|0)==13054){i=d;return(G=h,j)|0}else if((q|0)==13051){i=d;return(G=h,j)|0}else if((q|0)==13052){i=d;return(G=h,j)|0}else if((q|0)==13055){i=d;return(G=h,j)|0}return 0}function zr(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;f=i;if((b|0)<1){g=d;h=g-(g>>>1&1431655765)|0;g=(h>>>2&858993459)+(h&858993459)|0;h=(g>>>4)+g&252645135;g=(h>>>8)+h|0;h=e-((e>>>1|0<<31)&1431655765)|0;j=(h>>>2&858993459)+(h&858993459)|0;h=(j>>>4)+j&252645135;j=(h>>>8)+h|0;k=((g>>>16)+g&63)+((j>>>16)+j&63)|0}else{k=b}b=0;while(1){if((k|0)==(c[304148+(b<<4)>>2]|0)){j=304152+(b<<4)|0;if((c[j>>2]|0)==(d|0)&(c[j+4>>2]|0)==(e|0)){l=13061;break}}j=b+1|0;if(j>>>0<27){b=j}else{break}}if((l|0)==13061){zb(a,1637792,(m=i,i=i+8|0,c[m>>2]=c[304144+(b<<4)>>2],m)|0);i=m;i=f;return}zb(a,1717888,(m=i,i=i+8|0,c[m>>2]=k,m)|0);i=m;if((d|0)==0&(e|0)==0){i=f;return}zb(a,1690504,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;k=0;b=0;while(1){l=NW(1,0,k|0)|0;if((l&d|0)==0&(G&e|0)==0){n=b}else{do{if(k>>>0<=35){l=c[303856+(k<<3)>>2]|0;if((k-18|0)>>>0<11){break}if((b|0)>0){zb(a,1670800,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m}zb(a,1637792,(m=i,i=i+8|0,c[m>>2]=l,m)|0);i=m}}while(0);n=b+1|0}l=k+1|0;if((l|0)<64){k=l;b=n}else{break}}zb(a,1656048,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;i=f;return}function zs(a,b){a=a|0;b=b|0;var c=0;c=a;a=c-(c>>>1&1431655765)|0;c=(a>>>2&858993459)+(a&858993459)|0;a=(c>>>4)+c&252645135;c=(a>>>8)+a|0;a=b-((b>>>1|0<<31)&1431655765)|0;b=(a>>>2&858993459)+(a&858993459)|0;a=(b>>>4)+b&252645135;b=(a>>>8)+a|0;return((c>>>16)+c&63)+((b>>>16)+b&63)|0}function zt(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0,g=0;f=i;i=i+1024|0;g=f|0;za(g,a,b);zr(g,c,d,e);i=f;return}function zu(a){a=a|0;var b=0,d=0,e=0,f=0;b=0;while(1){if(b>>>0>=27){d=0;e=0;f=13086;break}if((c[304148+(b<<4)>>2]|0)==(a|0)){break}else{b=b+1|0}}if((f|0)==13086){return(G=d,e)|0}f=304152+(b<<4)|0;d=c[f+4>>2]|0;e=c[f>>2]|0;return(G=d,e)|0}function zv(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0;d=a;e=d-(d>>>1&1431655765)|0;d=(e>>>2&858993459)+(e&858993459)|0;e=(d>>>4)+d&252645135;d=(e>>>8)+e|0;e=b-((b>>>1|0<<31)&1431655765)|0;f=(e>>>2&858993459)+(e&858993459)|0;e=(f>>>4)+f&252645135;f=(e>>>8)+e|0;if((((d>>>16)+d&63)+((f>>>16)+f&63)|0)==1){g=0}else{h=0;return h|0}while(1){if((g|0)>=64){h=0;i=13094;break}f=NW(1,0,g|0)|0;if((f&a|0)==0&(G&b|0)==0){g=g+1|0}else{break}}if((i|0)==13094){return h|0}if((g|0)<0|g>>>0>35){h=0;return h|0}h=c[303856+(g<<3)>>2]|0;return h|0}function zw(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0;d=a;e=d-(d>>>1&1431655765)|0;d=(e>>>2&858993459)+(e&858993459)|0;e=(d>>>4)+d&252645135;d=(e>>>8)+e|0;e=b-((b>>>1|0<<31)&1431655765)|0;f=(e>>>2&858993459)+(e&858993459)|0;e=(f>>>4)+f&252645135;f=(e>>>8)+e|0;if((((d>>>16)+d&63)+((f>>>16)+f&63)|0)==1){g=0}else{h=0;return h|0}while(1){if(g>>>0>=36){h=0;i=13104;break}f=NW(1,0,g|0)|0;if((f&a|0)==0&(G&b|0)==0){g=g+1|0}else{break}}if((i|0)==13104){return h|0}h=c[303860+(g<<3)>>2]|0;return h|0}function zx(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;if(a>>>0>26){e=-541478725;return e|0}if((b|0)!=0){f=304152+(a<<4)|0;g=c[f+4>>2]|0;c[b>>2]=c[f>>2];c[b+4>>2]=g}if((d|0)==0){e=0;return e|0}c[d>>2]=c[304144+(a<<4)>>2];e=0;return e|0}function zy(a){a=a|0;c[556294]=a;c[745338]=(a|0)!=-1;return}function zz(){if((c[745338]|0)==0){c[745338]=1}return c[556294]|0}function zA(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=i;i=i+8|0;e=d|0;c[e>>2]=346440;f=Bg(e,346344,b,a)|0;i=d;return f|0}function zB(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0;if((d-8|0)>>>0>24){g=-1;return g|0}h=0;i=NW(1,0,d|0)|0;j=G;if(!((h|0)<(j|0)|(h|0)==(j|0)&e>>>0<i>>>0)){g=-1;return g|0}if(!((f|0)==4096|(f|0)==1028)){g=-1;return g|0}i=e<<32-d;if((b|0)==0){b=0;do{d=b<<24>>31&i^b<<25;j=d>>31&i^d<<1;d=j>>31&i^j<<1;j=d>>31&i^d<<1;d=j>>31&i^j<<1;j=d>>31&i^d<<1;d=j>>31&i^j<<1;c[a+(b<<2)>>2]=cV(d>>31&i^d<<1|0)|0;b=b+1|0;}while(b>>>0<256)}else{b=0;do{i=-(b&1)&e^b>>>1;d=-(i&1)&e^i>>>1;i=-(d&1)&e^d>>>1;d=-(i&1)&e^i>>>1;i=-(d&1)&e^d>>>1;d=-(i&1)&e^i>>>1;i=-(d&1)&e^d>>>1;c[a+(b<<2)>>2]=-(i&1)&e^i>>>1;b=b+1|0;}while(b>>>0<256)}c[a+1024>>2]=1;if(f>>>0>4095){k=0}else{g=0;return g|0}while(1){f=c[a+(k<<2)>>2]|0;b=f>>>8^c[a+((f&255)<<2)>>2];c[a+(k+256<<2)>>2]=b;f=b>>>8^c[a+((b&255)<<2)>>2];c[a+(k+512<<2)>>2]=f;c[a+(k+768<<2)>>2]=f>>>8^c[a+((f&255)<<2)>>2];f=k+1|0;if(f>>>0<256){k=f}else{g=0;break}}return g|0}function zC(a){a=a|0;var b=0,e=0;b=3015648+(a<<12)|0;do{if((c[3019740+(a<<12)>>2]|0)==0){if((zB(b,d[346608+(a<<3)|0]|0,d[346609+(a<<3)|0]|0,c[346612+(a<<3)>>2]|0,4096)|0)<0){e=0}else{break}return e|0}}while(0);e=b;return e|0}function zD(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;g=e+f|0;do{if((c[a+1024>>2]|0)==0){if((e&3|0)!=0&(f|0)>0){h=e;i=b;while(1){j=h+1|0;k=c[a+(((d[h]|0)^i&255)<<2)>>2]^i>>>8;if((j&3|0)!=0&j>>>0<g>>>0){h=j;i=k}else{l=j;m=k;break}}}else{l=e;m=b}i=e+(f-3)|0;if(l>>>0<i>>>0){n=l;o=m}else{p=m;q=l;break}while(1){h=c[n>>2]^o;k=n+4|0;j=c[a+((h>>>8&255|512)<<2)>>2]^c[a+((h&255|768)<<2)>>2]^c[a+((h>>>16&255|256)<<2)>>2]^c[a+(h>>>24<<2)>>2];if(k>>>0<i>>>0){n=k;o=j}else{p=j;q=k;break}}}else{p=b;q=e}}while(0);if(q>>>0<g>>>0){r=q;s=p}else{t=p;return t|0}while(1){p=r+1|0;q=c[a+(((d[r]|0)^s&255)<<2)>>2]^s>>>8;if(p>>>0<g>>>0){r=p;s=q}else{t=q;break}}return t|0}function zE(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0;if(!((e|0)==192|(e|0)==64)){g=-1;return g|0}f=a+384|0;c[f>>2]=(e|0)>64;e=b;h=e|0;i=e+4|0;e=d[i]|d[i+1|0]<<8|d[i+2|0]<<16|d[i+3|0]<<24|0;i=cV(d[h]|d[h+1|0]<<8|d[h+2|0]<<16|d[h+3|0]<<24|0)|0;h=((e>>>16|0<<16)<<8|0>>>24)&65280|(e>>>24|0<<8)|(e<<8|0>>>24|(e>>>8|0<<24)&255)<<16|0;e=i|0;i=1969344;j=0;k=0;l=0;while(1){m=NX(h|0,e|0,d[i]|0|0)|0;n=m&1|(l<<1|0>>>31);m=G&0|(k<<1|l>>>31);o=j+1|0;if((o|0)<56){i=i+1|0;j=o;k=m;l=n}else{p=0;q=m;r=n;break}}while(1){l=(r<<1|0>>>31)&-268435458|(r>>>27|q<<5)&268435457;k=q<<1|r>>>31|0|(q>>>27|0<<5)&0;do{if((p|0)>1){if((p|0)==15|(p|0)==8){s=k;t=l;break}s=k<<1|l>>>31|0|(q>>>26|0<<6)&0;t=(l<<1|0>>>31)&-268435458|(r>>>26|q<<6)&268435457}else{s=k;t=l}}while(0);l=1969296;k=0;j=0;i=0;while(1){e=NX(t|0,s|0,d[l]|0|0)|0;u=e&1|(i<<1|0>>>31);v=G&0|(j<<1|i>>>31);e=k+1|0;if((e|0)<48){l=l+1|0;k=e;j=v;i=u}else{break}}i=a+(p<<3)|0;c[i>>2]=u;c[i+4>>2]=v;i=p+1|0;if((i|0)<16){p=i;q=s;r=t}else{break}}if((c[f>>2]|0)==0){g=0;return g|0}f=b+8|0;t=f|0;r=f+4|0;f=d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0;r=cV(d[t]|d[t+1|0]<<8|d[t+2|0]<<16|d[t+3|0]<<24|0)|0;t=((f>>>16|0<<16)<<8|0>>>24)&65280|(f>>>24|0<<8)|(f<<8|0>>>24|(f>>>8|0<<24)&255)<<16|0;f=r|0;r=1969344;s=0;q=0;p=0;while(1){v=NX(t|0,f|0,d[r]|0|0)|0;u=v&1|(p<<1|0>>>31);v=G&0|(q<<1|p>>>31);i=s+1|0;if((i|0)<56){r=r+1|0;s=i;q=v;p=u}else{w=0;x=v;y=u;break}}while(1){p=(y<<1|0>>>31)&-268435458|(y>>>27|x<<5)&268435457;q=x<<1|y>>>31|0|(x>>>27|0<<5)&0;do{if((w|0)>1){if((w|0)==15|(w|0)==8){z=q;A=p;break}z=q<<1|p>>>31|0|(x>>>26|0<<6)&0;A=(p<<1|0>>>31)&-268435458|(y>>>26|x<<6)&268435457}else{z=q;A=p}}while(0);p=1969296;q=0;s=0;r=0;while(1){f=NX(A|0,z|0,d[p]|0|0)|0;B=f&1|(r<<1|0>>>31);C=G&0|(s<<1|r>>>31);f=q+1|0;if((f|0)<48){p=p+1|0;q=f;s=C;r=B}else{break}}r=a+128+(w<<3)|0;c[r>>2]=B;c[r+4>>2]=C;r=w+1|0;if((r|0)<16){w=r;x=z;y=A}else{break}}A=b+16|0;b=A|0;y=A+4|0;A=d[y]|d[y+1|0]<<8|d[y+2|0]<<16|d[y+3|0]<<24|0;y=cV(d[b]|d[b+1|0]<<8|d[b+2|0]<<16|d[b+3|0]<<24|0)|0;b=((A>>>16|0<<16)<<8|0>>>24)&65280|(A>>>24|0<<8)|(A<<8|0>>>24|(A>>>8|0<<24)&255)<<16|0;A=y|0;y=1969344;z=0;x=0;w=0;while(1){C=NX(b|0,A|0,d[y]|0|0)|0;B=C&1|(w<<1|0>>>31);C=G&0|(x<<1|w>>>31);r=z+1|0;if((r|0)<56){y=y+1|0;z=r;x=C;w=B}else{D=0;E=C;F=B;break}}while(1){w=(F<<1|0>>>31)&-268435458|(F>>>27|E<<5)&268435457;x=E<<1|F>>>31|0|(E>>>27|0<<5)&0;do{if((D|0)>1){if((D|0)==15|(D|0)==8){H=x;I=w;break}H=x<<1|w>>>31|0|(E>>>26|0<<6)&0;I=(w<<1|0>>>31)&-268435458|(F>>>26|E<<6)&268435457}else{H=x;I=w}}while(0);w=1969296;x=0;z=0;y=0;while(1){A=NX(I|0,H|0,d[w]|0|0)|0;J=A&1|(y<<1|0>>>31);K=G&0|(z<<1|y>>>31);A=x+1|0;if((A|0)<48){w=w+1|0;x=A;z=K;y=J}else{break}}y=a+256+(D<<3)|0;c[y>>2]=J;c[y+4>>2]=K;y=D+1|0;if((y|0)<16){D=y;E=H;F=I}else{g=0;break}}return g|0}function zF(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;zG(a,b,c,d,e,f,0);return}function zG(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0;k=(h|0)!=0;if(k){l=h;m=l|0;n=l+4|0;l=d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0;n=cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0;o=n|0;p=((l>>>16|0<<16)<<8|0>>>24)&65280|(l>>>24|0<<8)|(l<<8|0>>>24|(l>>>8|0<<24)&255)<<16|0}else{o=0;p=0}l=g-1|0;L17863:do{if((g|0)>0){n=b|0;m=b+384|0;q=b+128|0;r=b+256|0;s=(j|0)==0;if((i|0)==0){t=f;u=o;v=p;w=e;x=l;while(1){if((t|0)==0){z=0;A=0}else{B=t;C=B|0;D=B+4|0;B=d[D]|d[D+1|0]<<8|d[D+2|0]<<16|d[D+3|0]<<24|0;D=cV(d[C]|d[C+1|0]<<8|d[C+2|0]<<16|d[C+3|0]<<24|0)|0;z=D|0;A=((B>>>16|0<<16)<<8|0>>>24)&65280|(B>>>24|0<<8)|(B<<8|0>>>24|(B>>>8|0<<24)&255)<<16|0}B=zH(A^v,z^u,n,0)|0;D=G;if((c[m>>2]|0)==0){E=D;F=B}else{C=zH(B,D,q,1)|0;D=zH(C,G,r,0)|0;E=G;F=D}D=k?F:0;C=k?E:0;B=cV(F|0)|0;H=w;I=H|0;y=((E>>>16|0<<16)<<8|0>>>24)&65280|(E>>>24|0<<8)|(E<<8|0>>>24|(E>>>8|0<<24)&255)<<16|0;a[I]=y&255;y=y>>8;a[I+1|0]=y&255;y=y>>8;a[I+2|0]=y&255;y=y>>8;a[I+3|0]=y&255;I=H+4|0;y=B|0;a[I]=y&255;y=y>>8;a[I+1|0]=y&255;y=y>>8;a[I+2|0]=y&255;y=y>>8;a[I+3|0]=y&255;if((x|0)>0){t=t+8|0;u=C;v=D;w=s?w+8|0:w;x=x-1|0}else{J=C;K=D;break L17863}}}if(k){x=f;w=o;v=p;u=e;t=l;while(1){if((x|0)==0){L=0;M=0}else{D=x;C=D|0;I=D+4|0;D=d[I]|d[I+1|0]<<8|d[I+2|0]<<16|d[I+3|0]<<24|0;I=cV(d[C]|d[C+1|0]<<8|d[C+2|0]<<16|d[C+3|0]<<24|0)|0;L=I|0;M=((D>>>16|0<<16)<<8|0>>>24)&65280|(D>>>24|0<<8)|(D<<8|0>>>24|(D>>>8|0<<24)&255)<<16|0}if((c[m>>2]|0)==0){N=L;O=M}else{D=zH(M,L,r,1)|0;I=zH(D,G,q,0)|0;N=G;O=I}I=zH(O,N,n,1)|0;D=G^w;C=cV(I^v|0)|0;I=u;B=I|0;y=((D>>>16|0<<16)<<8|0>>>24)&65280|(D>>>24|0<<8)|(D<<8|0>>>24|(D>>>8|0<<24)&255)<<16|0;a[B]=y&255;y=y>>8;a[B+1|0]=y&255;y=y>>8;a[B+2|0]=y&255;y=y>>8;a[B+3|0]=y&255;B=I+4|0;y=C|0;a[B]=y&255;y=y>>8;a[B+1|0]=y&255;y=y>>8;a[B+2|0]=y&255;y=y>>8;a[B+3|0]=y&255;if((t|0)>0){x=x+8|0;w=L;v=M;u=s?u+8|0:u;t=t-1|0}else{J=L;K=M;break}}}else{t=f;u=o;v=p;w=e;x=l;while(1){if((t|0)==0){P=0;Q=0}else{B=t;C=B|0;I=B+4|0;B=d[I]|d[I+1|0]<<8|d[I+2|0]<<16|d[I+3|0]<<24|0;I=cV(d[C]|d[C+1|0]<<8|d[C+2|0]<<16|d[C+3|0]<<24|0)|0;P=I|0;Q=((B>>>16|0<<16)<<8|0>>>24)&65280|(B>>>24|0<<8)|(B<<8|0>>>24|(B>>>8|0<<24)&255)<<16|0}if((c[m>>2]|0)==0){R=P;S=Q}else{B=zH(Q,P,r,1)|0;I=zH(B,G,q,0)|0;R=G;S=I}I=zH(S,R,n,1)|0;B=G^u;C=cV(I^v|0)|0;I=w;D=I|0;y=((B>>>16|0<<16)<<8|0>>>24)&65280|(B>>>24|0<<8)|(B<<8|0>>>24|(B>>>8|0<<24)&255)<<16|0;a[D]=y&255;y=y>>8;a[D+1|0]=y&255;y=y>>8;a[D+2|0]=y&255;y=y>>8;a[D+3|0]=y&255;D=I+4|0;y=C|0;a[D]=y&255;y=y>>8;a[D+1|0]=y&255;y=y>>8;a[D+2|0]=y&255;y=y>>8;a[D+3|0]=y&255;if((x|0)>0){t=t+8|0;u=0;v=0;w=s?w+8|0:w;x=x-1|0}else{J=0;K=0;break}}}}else{J=o;K=p}}while(0);if(!k){return}k=cV(K|0)|0;K=h;h=K|0;y=((J>>>16|0<<16)<<8|0>>>24)&65280|(J>>>24|0<<8)|(J<<8|0>>>24|(J>>>8|0<<24)&255)<<16|0;a[h]=y&255;y=y>>8;a[h+1|0]=y&255;y=y>>8;a[h+2|0]=y&255;y=y>>8;a[h+3|0]=y&255;h=K+4|0;y=k|0;a[h]=y&255;y=y>>8;a[h+1|0]=y&255;y=y>>8;a[h+2|0]=y&255;y=y>>8;a[h+3|0]=y&255;return}function zH(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;g=(f|0)!=0;f=1974144;h=0;i=0;j=0;while(1){k=NX(a|0,b|0,d[f]|0|0)|0;l=k&1|(j<<1|0>>>31);m=G&0|(i<<1|j>>>31);k=h+1|0;if((k|0)<64){f=f+1|0;h=k;i=m;j=l}else{break}}j=g?15:0;g=0;i=m;m=l;while(1){l=m;h=e+((g^j)<<3)|0;f=c[h>>2]|0;b=c[h+4>>2]|0;h=l<<1|l>>>31;a=l>>>3;k=a&268435440;n=(c[1965656+((((f>>>6|b<<26)^a)&255&63)<<2)>>2]|c[1965912+(((f^h)&255&63)<<2)>>2]|c[1965400+((((f>>>12|b<<20)^k>>>4)&255&63)<<2)>>2]|c[1965144+((((f>>>18|b<<14)^l>>>11)&255&63)<<2)>>2]|c[1964888+((((f>>>24|b<<8)^l>>>15)&255&63)<<2)>>2]|c[1964632+((((f>>>30|b<<2)^l>>>19)&255&63)<<2)>>2]|c[1964376+((((b>>>4|0<<28)^l>>>23)&255&63)<<2)>>2]|c[1964120+((((b>>>10|0<<22)^(h<<28|k)>>>24)&255&63)<<2)>>2])^i;o=0^m;k=g+1|0;if((k|0)<16){g=k;i=o;m=n}else{break}}m=1974207;i=n|0;n=o|0;o=0;g=0;j=0;while(1){e=NW(n&1|0,i&0|0,d[m]|0|0)|0;p=e|j;q=G|g;e=n>>>1|i<<31;k=o+1|0;if((k|0)<64){m=m-1|0;i=i>>>1|0<<31;n=e;o=k;g=q;j=p}else{break}}return(G=q,p)|0}function zI(a){a=a|0;var b=0;if((a|0)==0){b=0}else{b=c[a>>2]|0}return b|0}function zJ(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;if((b|0)==0){g=0;return g|0}if((e|0)==0){h=0}else{h=(e-(c[b+4>>2]|0)>>3)+1|0}e=c[b>>2]|0;if(h>>>0>=e>>>0){g=0;return g|0}i=c[b+4>>2]|0;b=(f&2|0)==0;L17921:do{if((f&1|0)==0){j=h;while(1){k=c[i+(j<<3)>>2]|0;l=0;while(1){m=a[k+l|0]|0;n=a[d+l|0]|0;o=n<<24>>24;if((((m-97|0)>>>0<26?m^32:m)|0)!=(((o-97|0)>>>0<26?o^32:o)|0)|n<<24>>24==0){break}else{l=l+1|0}}if(n<<24>>24==0){if((a[k+l|0]|0)==0|b^1){p=j;break L17921}}o=j+1|0;if(o>>>0<e>>>0){j=o}else{g=0;break}}return g|0}else{if(b){q=h}else{j=h;while(1){o=c[i+(j<<3)>>2]|0;m=0;while(1){r=a[d+m|0]|0;if((a[o+m|0]|0)!=r<<24>>24|r<<24>>24==0){break}else{m=m+1|0}}if(r<<24>>24==0){p=j;break L17921}m=j+1|0;if(m>>>0<e>>>0){j=m}else{g=0;break}}return g|0}while(1){j=c[i+(q<<3)>>2]|0;m=0;while(1){s=a[d+m|0]|0;if((a[j+m|0]|0)!=s<<24>>24|s<<24>>24==0){break}else{m=m+1|0}}if(s<<24>>24==0){if((a[j+m|0]|0)==0){p=q;break L17921}}o=q+1|0;if(o>>>0<e>>>0){q=o}else{g=0;break}}return g|0}}while(0);g=i+(p<<3)|0;return g|0}function zK(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;f=i;i=i+8|0;g=f|0;h=c[a>>2]|0;j=zJ(h,b,0,e)|0;c[g>>2]=0;if((h|0)==0){k=A7(8)|0;c[a>>2]=k;l=k}else{l=h}do{if((j|0)==0){h=l+4|0;k=A2(c[h>>2]|0,(c[l>>2]<<3)+8|0)|0;if((k|0)==0){m=-12;i=f;return m|0}else{c[h>>2]=k;break}}else{if((e&16|0)!=0){m=0;i=f;return m|0}k=c[j+4>>2]|0;if((e&32|0)==0){A4(k)}else{c[g>>2]=k}A4(c[j>>2]|0);k=l|0;h=(c[k>>2]|0)-1|0;c[k>>2]=h;k=(c[l+4>>2]|0)+(h<<3)|0;h=j;n=c[k+4>>2]|0;c[h>>2]=c[k>>2];c[h+4>>2]=n}}while(0);if((d|0)==0){o=c[l>>2]|0}else{if((e&4|0)==0){j=A9(b)|0;c[(c[l+4>>2]|0)+(c[l>>2]<<3)>>2]=j}else{c[(c[l+4>>2]|0)+(c[l>>2]<<3)>>2]=b}L17976:do{if((e&8|0)==0){b=c[g>>2]|0;do{if((b|0)!=0){if((e&32|0)==0){break}j=NL(b|0)|0;n=j+1+(NL(d|0)|0)|0;j=A7(n)|0;if((j|0)==0){m=-12;i=f;return m|0}else{y$(j,b,n)|0;A6(g);y$(j,d,n)|0;c[(c[l+4>>2]|0)+(c[l>>2]<<3)+4>>2]=j;break L17976}}}while(0);b=A9(d)|0;c[(c[l+4>>2]|0)+(c[l>>2]<<3)+4>>2]=b}else{c[(c[l+4>>2]|0)+(c[l>>2]<<3)+4>>2]=d}}while(0);d=l|0;g=(c[d>>2]|0)+1|0;c[d>>2]=g;o=g}if((o|0)!=0){m=0;i=f;return m|0}A4(c[l+4>>2]|0);A6(a);m=0;i=f;return m|0}function zL(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;b=c[a>>2]|0;if((b|0)==0){d=a;A6(d);return}e=b|0;f=c[e>>2]|0;g=f-1|0;c[e>>2]=g;h=b+4|0;b=c[h>>2]|0;if((f|0)==0){i=b}else{f=g;g=b;while(1){A4(c[g+(f<<3)>>2]|0);A4(c[(c[h>>2]|0)+(c[e>>2]<<3)+4>>2]|0);b=c[e>>2]|0;j=b-1|0;c[e>>2]=j;k=c[h>>2]|0;if((b|0)==0){i=k;break}else{f=j;g=k}}}A4(i);d=a;A6(d);return}function zM(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0;if((b|0)==0){return}e=b|0;f=b+4|0;b=0;while(1){if((b|0)==0){g=0}else{g=(b-(c[f>>2]|0)>>3)+1|0}if(g>>>0>=(c[e>>2]|0)>>>0){h=13291;break}i=c[f>>2]|0;j=i+(g<<3)|0;if((j|0)==0){h=13289;break}zK(a,c[i+(g<<3)>>2]|0,c[i+(g<<3)+4>>2]|0,d)|0;b=j}if((h|0)==13291){return}else if((h|0)==13289){return}}function zN(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=i;f=0;while(1){if(f>>>0>=19){break}if((c[259176+(f*12|0)>>2]|0)==(a|0)){g=13295;break}else{f=f+1|0}}if((g|0)==13295){g=c[259184+(f*12|0)>>2]|0;y_(b,g,d)|0;h=0;i=e;return h|0}g=cR(-a|0,b|0,d|0)|0;f=-g|0;if((g|0)<=0){h=f;i=e;return h|0}bc(b|0,d|0,1635736,(d=i,i=i+8|0,c[d>>2]=a,d)|0)|0;i=d;h=f;i=e;return h|0}function zO(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0.0,j=0,k=0.0,l=0,m=0,n=0.0,o=0,p=0.0,q=0,r=0,s=0.0;e=i;i=i+8|0;f=e|0;do{if((a[b]|0)==48){if((a[b+1|0]|32|0)!=120){g=13305;break}h=+((aS(b|0,f|0,16)|0)>>>0>>>0)}else{g=13305}}while(0);if((g|0)==13305){h=+NK(b,f)}j=c[f>>2]|0;do{if((j|0)==(b|0)){k=h;l=b}else{m=a[j]|0;do{if(m<<24>>24==100){if((a[j+1|0]|0)!=66){g=13311;break}n=+T(+10.0,+(h/20.0));o=j+2|0;c[f>>2]=o;p=n;q=o}else{if((m-69&255)<54){g=13311}else{p=h;q=j}}}while(0);do{if((g|0)==13311){o=a[73728+((m<<24>>24)-69)|0]|0;if(o<<24>>24==0){p=h;q=j;break}r=j+1|0;n=+(o<<24>>24|0);if((a[r]|0)==105){s=h*+b4(+(n/.3));o=j+2|0;c[f>>2]=o;p=s;q=o;break}else{s=h*+T(+10.0,+n);c[f>>2]=r;p=s;q=r;break}}}while(0);if((a[q]|0)!=66){k=p;l=q;break}m=q+1|0;c[f>>2]=m;k=p*8.0;l=m}}while(0);if((d|0)==0){i=e;return+k}c[d>>2]=l;i=e;return+k}function zP(a){a=a|0;var b=0,d=0;b=i;i=i+8|0;d=b|0;c[d>>2]=a;if((a|0)==0){i=b;return}zP(c[a+20>>2]|0);zP(c[a+24>>2]|0);zP(c[a+28>>2]|0);A6(a+32|0);A6(d);i=b;return}function zQ(b,d,e,f,g,h,j,k,l){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;m=i;i=i+64|0;n=m|0;o=m+56|0;p=n;NM(p|0,0,52);c[o>>2]=0;q=A1((NL(d|0)|0)+1|0)|0;if((q|0)==0){r=-12;i=m;return r|0}else{s=q;t=d}L18061:while(1){u=t;while(1){v=a[u]|0;if(v<<24>>24==0){break L18061}w=u+1|0;if((y7(v<<24>>24)|0)==0){break}else{u=w}}a[s]=a[u]|0;s=s+1|0;t=w}a[s]=0;c[n>>2]=259128;c[n+4>>2]=100;s=n+8|0;c[s>>2]=q;c[n+16>>2]=e;c[n+20>>2]=g;c[n+24>>2]=f;c[n+28>>2]=j;c[n+32>>2]=h;c[n+40>>2]=k;c[n+44>>2]=l;l=zR(o,n)|0;do{if((l|0)<0){x=l}else{n=c[o>>2]|0;if((a[c[s>>2]|0]|0)!=0){zP(n);AN(p,16,1635576,(k=i,i=i+16|0,c[k>>2]=c[s>>2],c[k+8>>2]=d,k)|0);i=k;x=-22;break}if((zS(n)|0)==0){zP(n);x=-22;break}else{c[n+32>>2]=A7(80)|0;c[b>>2]=n;x=l;break}}}while(0);A4(q);r=x;i=m;return r|0}function zR(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;e=i;i=i+16|0;f=e|0;g=e+8|0;j=d+4|0;k=c[j>>2]|0;if((k|0)<1){l=-22;i=e;return l|0}c[j>>2]=k-1;k=zW(f,d)|0;if((k|0)<0){l=k;i=e;return l|0}k=d+8|0;m=c[k>>2]|0;n=c[f>>2]|0;L18085:do{if((a[m]|0)==59){o=m;p=n;while(1){c[k>>2]=o+1;q=zW(g,d)|0;if((q|0)<0){r=13346;break}s=c[g>>2]|0;t=A7(40)|0;if((t|0)==0){r=13348;break}u=t;c[t>>2]=22;h[t+8>>3]=1.0;c[t+20>>2]=p;c[t+24>>2]=s;c[f>>2]=u;t=c[k>>2]|0;if((a[t]|0)==59){o=t;p=u}else{v=u;break L18085}}if((r|0)==13346){zP(p);l=q;i=e;return l|0}else if((r|0)==13348){c[f>>2]=0;zP(p);zP(s);l=-12;i=e;return l|0}}else{v=n}}while(0);c[j>>2]=(c[j>>2]|0)+1;c[b>>2]=v;l=0;i=e;return l|0}function zS(a){a=a|0;var b=0,d=0;if((a|0)==0){b=0;return b|0}switch(c[a>>2]|0){case 2:case 3:case 5:case 7:case 6:case 8:case 9:case 27:case 28:case 29:case 30:case 31:case 32:{if((zS(c[a+20>>2]|0)|0)==0){b=0;return b|0}b=(c[a+24>>2]|0)==0|0;return b|0};case 40:{if((zS(c[a+20>>2]|0)|0)==0){b=0;return b|0}if((zS(c[a+24>>2]|0)|0)==0){b=0;return b|0}else{return(zS(c[a+28>>2]|0)|0)!=0|0}break};case 35:case 36:case 25:{if((zS(c[a+20>>2]|0)|0)==0){b=0;return b|0}if((zS(c[a+24>>2]|0)|0)==0){b=0;return b|0}d=c[a+28>>2]|0;if((d|0)==0){b=1;return b|0}else{return(zS(d)|0)!=0|0}break};case 0:case 1:{b=1;return b|0};case 37:{if((zS(c[a+20>>2]|0)|0)==0){b=0;return b|0}d=c[a+24>>2]|0;if((d|0)==0){b=1;return b|0}else{return(zS(d)|0)!=0|0}break};default:{if((zS(c[a+20>>2]|0)|0)==0){b=0;return b|0}if((zS(c[a+24>>2]|0)|0)==0){b=0;return b|0}b=(c[a+28>>2]|0)==0|0;return b|0}}return 0}function zT(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0.0;e=i;i=i+56|0;f=e|0;NM(f|0,0,48);c[f+48>>2]=c[a+32>>2];c[f+12>>2]=b;c[f+36>>2]=d;g=+zU(f,a);i=e;return+g}function zU(a,b){a=a|0;b=b|0;var e=0,f=0,g=0.0,j=0,l=0,m=0.0,n=0.0,o=0.0,p=0.0,q=0.0,s=0.0,t=0,u=0.0,v=0.0,w=0.0,x=0.0,y=0.0,z=0,A=0.0,B=0.0,C=0.0,D=0.0,E=0.0,F=0.0,H=0,I=0,J=0.0,K=0.0,L=0,M=0,N=0.0,O=0.0,P=0,U=0.0,V=0.0,W=0.0,X=0;e=i;f=b|0;switch(c[f>>2]|0){case 7:{g=+h[b+8>>3];j=~~+zU(a,c[b+20>>2]|0);if((j|0)<0){l=0}else{l=(j|0)>9?9:j}m=g*+h[(c[a+48>>2]|0)+(l<<3)>>3];i=e;return+m};case 29:{g=+h[b+8>>3]*+bF(+(+zU(a,c[b+20>>2]|0)));i=e;return+g};case 26:{l=a+48|0;g=+h[c[l>>2]>>3];n=+zU(a,c[b+24>>2]|0);j=b+20|0;o=1.7976931348623157e+308;p=-1.7976931348623157e+308;q=-1.0;s=-1.0;t=-1;while(1){if((t|0)>=1024){u=o;v=p;w=q;x=s;break}do{if((t|0)<255){h[c[l>>2]>>3]=n*+(d[340928+(t&255)|0]|0|0)/255.0}else{y=n*+T(+.9,+(+(t-255|0)));h[c[l>>2]>>3]=y;if((t&1|0)!=0){z=c[l>>2]|0;h[z>>3]=+h[z>>3]*-1.0}z=c[l>>2]|0;y=+h[z>>3];if((t&2|0)==0){h[z>>3]=q+y;break}else{h[z>>3]=s+y;break}}}while(0);y=+zU(a,c[j>>2]|0);if(y<=0.0&y>p){A=y;B=+h[c[l>>2]>>3]}else{A=p;B=s}if(y>=0.0&y<o){C=y;D=+h[c[l>>2]>>3]}else{C=o;D=q}if(B<0.0|D<0.0){o=C;p=A;q=D;s=B;t=t+1|0}else{E=D;F=B;H=0;I=13454;break}}L18169:do{if((I|0)==13454){while(1){I=0;if((H|0)>=1e3){u=C;v=A;w=E;x=F;break L18169}h[c[l>>2]>>3]=(F+E)*.5;B=+h[c[l>>2]>>3];if(F==B|E==B){u=C;v=A;w=E;x=F;break L18169}B=+zU(a,c[j>>2]|0);if(B>0.0){J=F}else{J=+h[c[l>>2]>>3]}if(B<0.0){K=E}else{K=+h[c[l>>2]>>3]}h[k>>3]=B;t=c[k+4>>2]&2147483647;z=2146435072;if(t>>>0>z>>>0|t>>>0==z>>>0&(c[k>>2]|0)>>>0>0>>>0){u=C;v=A;w=B;x=B;break}else{E=K;F=J;H=H+1|0;I=13454}}}}while(0);h[c[l>>2]>>3]=g;m=u>-0.0-v?x:w;i=e;return+m};case 37:{w=+zU(a,c[b+20>>2]|0);l=c[b+24>>2]|0;if((l|0)==0){L=32}else{L=~~+zU(a,l)}AN(a,L,1717336,(L=i,i=i+8|0,h[L>>3]=w,L)|0);i=L;m=w;i=e;return+m};case 2:{w=+h[b+8>>3];L=c[b+16>>2]|0;x=+zU(a,c[b+20>>2]|0);v=w*+dd[L&31](x);i=e;return+v};case 25:{v=+zU(a,c[b+24>>2]|0);L=c[b+28>>2]|0;do{if((L|0)==0){M=0}else{l=~~+zU(a,L);if((l|0)<0){M=0;break}M=(l|0)>9?9:l}}while(0);L=a+48|0;l=c[L>>2]|0;x=+h[l+(M<<3)>>3];I=b+20|0;w=0.0;u=1.0;H=0;j=l;while(1){h[j+(M<<3)>>3]=+(H|0);g=+zU(a,c[I>>2]|0);N=w+u*g;if(w==N&g!=0.0){break}l=H+1|0;if((l|0)>=1e3){break}w=N;u=u*(v/+(l|0));H=l;j=c[L>>2]|0}h[(c[L>>2]|0)+(M<<3)>>3]=x;m=N;i=e;return+m};case 30:{N=+h[b+8>>3];x=N*+S(+(+zU(a,c[b+20>>2]|0)));i=e;return+x};case 24:{M=b+20|0;if(+zU(a,c[M>>2]|0)==0.0){m=r;i=e;return+m}L=b+24|0;while(1){x=+zU(a,c[L>>2]|0);if(+zU(a,c[M>>2]|0)==0.0){m=x;break}}i=e;return+m};case 0:{m=+h[b+8>>3];i=e;return+m};case 35:{x=+h[b+8>>3];do{if(+zU(a,c[b+20>>2]|0)!=0.0){O=+zU(a,c[b+24>>2]|0)}else{M=c[b+28>>2]|0;if((M|0)==0){O=0.0;break}O=+zU(a,M)}}while(0);m=x*O;i=e;return+m};case 5:{O=1.0/(+$(+(+zU(a,c[b+20>>2]|0)*4.0))+1.0);i=e;return+O};case 32:{M=~~+zU(a,c[b+20>>2]|0);if((M|0)<0){P=0}else{P=(M|0)>9?9:M}M=(c[a+48>>2]|0)+(P<<3)|0;O=+h[M>>3];P=c[M+4>>2]&2147483647;L=2146435072;if(P>>>0>L>>>0|P>>>0==L>>>0&(c[M>>2]|0)>>>0>0>>>0){U=1013904223.0}else{L=N2(~~O>>>0,+R(O)>=1.0?O>0.0?(ah(+Q(O/4294967296.0),4294967295.0)|0)>>>0:~~+ab((O- +(~~O>>>0))/4294967296.0)>>>0:0,1664525,0)|0;P=NU(L,G,1013904223,0)|0;U=+(P>>>0)+ +(G>>>0)*4294967296.0}h[M>>3]=U;m=U*5.421010862427522e-20*+h[b+8>>3];i=e;return+m};case 9:{U=+h[b+8>>3];O=+zU(a,c[b+20>>2]|0);h[k>>3]=O;O=U*+((c[k>>2]|0|0)==0&(c[k+4>>2]&2147483647|0)==2146435072&1|0);i=e;return+O};case 1:{m=+h[b+8>>3]*+h[(c[a+12>>2]|0)+(c[b+16>>2]<<3)>>3];i=e;return+m};case 28:{O=+h[b+8>>3];U=O*+ab(+(+zU(a,c[b+20>>2]|0)));i=e;return+U};case 31:{U=+h[b+8>>3];O=U*+(+zU(a,c[b+20>>2]|0)==0.0|0);i=e;return+O};case 36:{O=+h[b+8>>3];do{if(+zU(a,c[b+20>>2]|0)!=0.0){M=c[b+28>>2]|0;if((M|0)==0){V=0.0;break}V=+zU(a,M)}else{V=+zU(a,c[b+24>>2]|0)}}while(0);m=O*V;i=e;return+m};case 27:{V=+h[b+8>>3];O=V*+Q(+(+zU(a,c[b+20>>2]|0)));i=e;return+O};case 3:{O=+h[b+8>>3];M=c[b+16>>2]|0;P=c[a+36>>2]|0;V=+zU(a,c[b+20>>2]|0);U=O*+c7[M&1](P,V);i=e;return+U};case 4:{U=+h[b+8>>3];P=c[b+16>>2]|0;M=c[a+36>>2]|0;V=+zU(a,c[b+20>>2]|0);O=+zU(a,c[b+24>>2]|0);x=U*+cZ[P&1](M,V,O);i=e;return+x};case 40:{x=+zU(a,c[b+20>>2]|0);O=+h[b+8>>3];if(x<+zU(a,c[b+24>>2]|0)){W=0.0}else{W=+(x<=+zU(a,c[b+28>>2]|0)|0)}m=O*W;i=e;return+m};case 6:{W=+zU(a,c[b+20>>2]|0);O=+$(+(W*(-0.0-W)*.5))/2.5066282746310002;i=e;return+O};case 8:{O=+h[b+8>>3];W=+zU(a,c[b+20>>2]|0);h[k>>3]=W;M=c[k+4>>2]&2147483647;P=2146435072;W=O*+((M>>>0>P>>>0|M>>>0==P>>>0&(c[k>>2]|0)>>>0>0>>>0)&1|0);i=e;return+W};default:{W=+zU(a,c[b+20>>2]|0);O=+zU(a,c[b+24>>2]|0);switch(c[f>>2]|0){case 10:{m=+h[b+8>>3]*(W-O*+Q(+(W/O)));i=e;return+m};case 34:{x=+h[b+8>>3];m=x*(+((AU(~~W>>>0,+R(W)>=1.0?W>0.0?(ah(+Q(W/4294967296.0),4294967295.0)|0)>>>0:~~+ab((W- +(~~W>>>0))/4294967296.0)>>>0:0,~~O>>>0,+R(O)>=1.0?O>0.0?(ah(+Q(O/4294967296.0),4294967295.0)|0)>>>0:~~+ab((O- +(~~O>>>0))/4294967296.0)>>>0:0)|0)>>>0)+ +(G|0)*4294967296.0);i=e;return+m};case 11:{m=(W>O?W:O)*+h[b+8>>3];i=e;return+m};case 39:{h[k>>3]=W;f=c[k+4>>2]&2147483647;P=2146435072;if(f>>>0>P>>>0|f>>>0==P>>>0&(c[k>>2]|0)>>>0>0>>>0){m=r;i=e;return+m}h[k>>3]=O;P=c[k+4>>2]&2147483647;f=2146435072;if(P>>>0>f>>>0|P>>>0==f>>>0&(c[k>>2]|0)>>>0>0>>>0){m=r;i=e;return+m}m=+(~~O|~~W|0)*+h[b+8>>3];i=e;return+m};case 12:{m=(W<O?W:O)*+h[b+8>>3];i=e;return+m};case 13:{m=(W==O?1.0:0.0)*+h[b+8>>3];i=e;return+m};case 14:{m=(W>O?1.0:0.0)*+h[b+8>>3];i=e;return+m};case 38:{h[k>>3]=W;f=c[k+4>>2]&2147483647;P=2146435072;if(f>>>0>P>>>0|f>>>0==P>>>0&(c[k>>2]|0)>>>0>0>>>0){m=r;i=e;return+m}h[k>>3]=O;P=c[k+4>>2]&2147483647;f=2146435072;if(P>>>0>f>>>0|P>>>0==f>>>0&(c[k>>2]|0)>>>0>0>>>0){m=r;i=e;return+m}m=+(~~O&~~W|0)*+h[b+8>>3];i=e;return+m};case 18:{m=+h[b+8>>3]*+T(+W,+O);i=e;return+m};case 19:{m=W*O*+h[b+8>>3];i=e;return+m};case 20:{m=+h[b+8>>3]*(W/O);i=e;return+m};case 21:{m=(W+O)*+h[b+8>>3];i=e;return+m};case 22:{m=O*+h[b+8>>3];i=e;return+m};case 23:{x=+h[b+8>>3];f=~~W;if((f|0)<0){X=0}else{X=(f|0)>9?9:f}h[(c[a+48>>2]|0)+(X<<3)>>3]=O;m=O*x;i=e;return+m};case 33:{m=+h[b+8>>3]*+S(+(W*W+O*O));i=e;return+m};case 15:{m=(W>=O?1.0:0.0)*+h[b+8>>3];i=e;return+m};case 17:{m=(W<O?1.0:0.0)*+h[b+8>>3];i=e;return+m};case 16:{m=(W<=O?1.0:0.0)*+h[b+8>>3];i=e;return+m};default:{m=r;i=e;return+m}}}}return 0.0}function zV(a,b,d,e,f,g,j,k,l,m,n){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,s=0,t=0;o=i;i=i+64|0;p=o|0;q=o+56|0;c[q>>2]=0;s=zQ(q,b,d,f,g,j,k,m,n)|0;if((s|0)<0){h[a>>3]=r;t=s;i=o;return t|0}else{s=c[q>>2]|0;NM(p|0,0,48);c[p+48>>2]=c[s+32>>2];c[p+12>>2]=e;c[p+36>>2]=l;h[a>>3]=+zU(p,s);zP(s);s=c[a+4>>2]&2147483647;p=2146435072;t=s>>>0>p>>>0|s>>>0==p>>>0&(c[a>>2]|0)>>>0>0>>>0?-22:0;i=o;return t|0}return 0}function zW(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;e=i;i=i+16|0;f=e|0;g=e+8|0;j=zX(f,d)|0;if((j|0)<0){k=j;i=e;return k|0}j=d+8|0;l=c[f>>2]|0;while(1){m=a[c[j>>2]|0]|0;if(!((m<<24>>24|0)==45|(m<<24>>24|0)==43)){n=13537;break}o=zX(g,d)|0;if((o|0)<0){n=13533;break}p=c[g>>2]|0;m=A7(40)|0;if((m|0)==0){n=13535;break}q=m;c[m>>2]=21;h[m+8>>3]=1.0;c[m+20>>2]=l;c[m+24>>2]=p;c[f>>2]=q;l=q}if((n|0)==13537){c[b>>2]=l;k=0;i=e;return k|0}else if((n|0)==13533){zP(l);k=o;i=e;return k|0}else if((n|0)==13535){c[f>>2]=0;zP(l);zP(p);k=-12;i=e;return k|0}return 0}function zX(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;e=i;i=i+16|0;f=e|0;g=e+8|0;j=zY(f,d)|0;if((j|0)<0){k=j;i=e;return k|0}j=d+8|0;while(1){l=c[j>>2]|0;m=a[l]|0;if(!((m<<24>>24|0)==42|(m<<24>>24|0)==47)){n=13551;break}c[j>>2]=l+1;m=a[l]|0;o=c[f>>2]|0;p=zY(g,d)|0;if((p|0)<0){n=13547;break}q=c[g>>2]|0;l=A7(40)|0;if((l|0)==0){n=13549;break}c[l>>2]=m<<24>>24==42?19:20;h[l+8>>3]=1.0;c[l+20>>2]=o;c[l+24>>2]=q;c[f>>2]=l}if((n|0)==13551){c[b>>2]=c[f>>2];k=0;i=e;return k|0}else if((n|0)==13547){zP(o);k=p;i=e;return k|0}else if((n|0)==13549){c[f>>2]=0;zP(o);zP(q);k=-12;i=e;return k|0}return 0}function zY(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0.0,t=0,u=0,v=0,w=0,x=0.0,y=0,z=0;e=i;i=i+32|0;f=e|0;g=e+8|0;j=e+16|0;k=e+24|0;l=d+8|0;m=c[l>>2]|0;do{if((a[m]|0)==45){+NK(m,f);n=c[f>>2]|0;o=c[l>>2]|0;if((n|0)==(o|0)){p=n;q=13562;break}if((a[n]|0)!=100){p=o;q=13562;break}if((a[n+1|0]|0)!=66){p=o;q=13562;break}r=zZ(j,d)|0;s=1.0}else{p=m;q=13562}}while(0);if((q|0)==13562){m=a[p]|0;f=(m<<24>>24==43)-(m<<24>>24==45)|0;c[l>>2]=p+(f&1);r=zZ(j,d)|0;s=+(f|1|0)}if((r|0)<0){t=r;i=e;return t|0}r=c[l>>2]|0;f=c[j>>2]|0;L18351:do{if((a[r]|0)==94){p=r;m=f;while(1){o=p+1|0;c[l>>2]=o;do{if((a[o]|0)==45){+NK(o,g);n=c[g>>2]|0;u=c[l>>2]|0;if((n|0)==(u|0)){v=n;q=13571;break}if((a[n]|0)!=100){v=u;q=13571;break}if((a[n+1|0]|0)!=66){v=u;q=13571;break}w=zZ(k,d)|0;x=1.0}else{v=o;q=13571}}while(0);if((q|0)==13571){q=0;o=a[v]|0;u=(o<<24>>24==43)-(o<<24>>24==45)|0;c[l>>2]=v+(u&1);w=zZ(k,d)|0;x=+(u|1|0)}if((w|0)<0){q=13573;break}y=c[k>>2]|0;u=A7(40)|0;if((u|0)==0){q=13575;break}o=u;c[u>>2]=18;h[u+8>>3]=1.0;c[u+20>>2]=m;c[u+24>>2]=y;c[j>>2]=o;if((y|0)!=0){u=y+8|0;h[u>>3]=x*+h[u>>3]}u=c[l>>2]|0;if((a[u]|0)==94){p=u;m=o}else{z=o;break L18351}}if((q|0)==13573){zP(m);t=w;i=e;return t|0}else if((q|0)==13575){c[j>>2]=0;zP(m);zP(y);t=-12;i=e;return t|0}}else{z=f}}while(0);if((z|0)!=0){f=z+8|0;h[f>>3]=s*+h[f>>3]}c[b>>2]=z;t=0;i=e;return t|0}function zZ(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0.0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0;e=i;i=i+16|0;f=e|0;g=e+8|0;j=A7(40)|0;c[f>>2]=j;k=d+8|0;l=c[k>>2]|0;c[g>>2]=l;if((j|0)==0){m=-12;i=e;return m|0}n=+zO(l,g);j=c[f>>2]|0;o=j+8|0;h[o>>3]=n;p=c[g>>2]|0;if((p|0)!=(c[k>>2]|0)){c[j>>2]=0;c[k>>2]=c[g>>2];c[b>>2]=c[f>>2];m=0;i=e;return m|0}h[o>>3]=1.0;o=c[d+16>>2]|0;L18386:do{if((o|0)!=0){q=0;while(1){r=c[o+(q<<2)>>2]|0;if((r|0)==0){break L18386}s=c[k>>2]|0;t=0;while(1){u=a[r+t|0]|0;v=a[s+t|0]|0;if(u<<24>>24==0){w=13596;break}if(u<<24>>24==v<<24>>24){t=t+1|0}else{break}}if((w|0)==13596){w=0;t=v<<24>>24;if(!((t-48|0)>>>0<10|(t-97|0)>>>0<26|(t-65|0)>>>0<26|v<<24>>24==95)){break}}q=q+1|0}c[k>>2]=s+(NL(r|0)|0);c[j>>2]=1;c[(c[f>>2]|0)+16>>2]=q;c[b>>2]=c[f>>2];m=0;i=e;return m|0}}while(0);r=c[k>>2]|0;s=0;while(1){x=a[r+s|0]|0;if((s|0)==1){w=13601;break}if((a[1645240+s|0]|0)==x<<24>>24){s=s+1|0}else{y=0;w=13814;break}}if((w|0)==13601){s=x<<24>>24;if((s-48|0)>>>0<10|(s-97|0)>>>0<26|(s-65|0)>>>0<26|x<<24>>24==95){y=0;w=13814}else{z=0;A=264032}}do{if((w|0)==13814){while(1){w=0;B=a[r+y|0]|0;if((y|0)==2){w=13816;break}if((a[1643448+y|0]|0)==B<<24>>24){y=y+1|0;w=13814}else{C=0;break}}if((w|0)==13816){q=B<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|B<<24>>24==95){C=0}else{z=1;A=264048;break}}while(1){D=a[r+C|0]|0;if((C|0)==3){w=13819;break}if((a[1640944+C|0]|0)==D<<24>>24){C=C+1|0}else{break}}if((w|0)==13819){q=D<<24>>24;if(!((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|D<<24>>24==95)){z=2;A=264064;break}}q=bi(r|0,40)|0;c[k>>2]=q;if((q|0)==0){AN(d,16,1690168,(E=i,i=i+8|0,c[E>>2]=l,E)|0);i=E;c[k>>2]=c[g>>2];zP(c[f>>2]|0);m=-22;i=e;return m|0}c[k>>2]=q+1;if((a[p]|0)==40){A6(f);q=zR(f,d)|0;if((q|0)<0){m=q;i=e;return m|0}q=c[k>>2]|0;if((a[q]|0)==41){c[k>>2]=q+1;c[b>>2]=c[f>>2];m=0;i=e;return m|0}else{AN(d,16,1670560,(E=i,i=i+8|0,c[E>>2]=l,E)|0);i=E;zP(c[f>>2]|0);m=-22;i=e;return m|0}}q=zR(j+20|0,d)|0;if((q|0)<0){zP(c[f>>2]|0);m=q;i=e;return m|0}q=c[k>>2]|0;x=a[q]|0;do{if(x<<24>>24==44){c[k>>2]=q+1;s=(c[f>>2]|0)+24|0;zR(s,d)|0;s=c[k>>2]|0;v=a[s]|0;if(v<<24>>24!=44){F=s;G=v;break}c[k>>2]=s+1;s=(c[f>>2]|0)+28|0;zR(s,d)|0;s=c[k>>2]|0;F=s;G=a[s]|0}else{F=q;G=x}}while(0);if(G<<24>>24!=41){AN(d,16,1655624,(E=i,i=i+8|0,c[E>>2]=l,E)|0);i=E;zP(c[f>>2]|0);m=-22;i=e;return m|0}c[k>>2]=F+1;c[c[f>>2]>>2]=2;x=c[g>>2]|0;q=0;while(1){H=a[x+q|0]|0;if((q|0)==4){w=13619;break}if((a[1637288+q|0]|0)==H<<24>>24){q=q+1|0}else{I=0;w=13621;break}}do{if((w|0)==13619){q=H<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|H<<24>>24==95){I=0;w=13621;break}c[(c[f>>2]|0)+16>>2]=14}}while(0);L18449:do{if((w|0)==13621){while(1){w=0;J=a[x+I|0]|0;if((I|0)==4){w=13623;break}if((a[1629304+I|0]|0)==J<<24>>24){I=I+1|0;w=13621}else{K=0;break}}do{if((w|0)==13623){q=J<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|J<<24>>24==95){K=0;break}c[(c[f>>2]|0)+16>>2]=20;break L18449}}while(0);while(1){L=a[x+K|0]|0;if((K|0)==4){w=13627;break}if((a[1623144+K|0]|0)==L<<24>>24){K=K+1|0}else{M=0;break}}do{if((w|0)==13627){q=L<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|L<<24>>24==95){M=0;break}c[(c[f>>2]|0)+16>>2]=6;break L18449}}while(0);while(1){N=a[x+M|0]|0;if((M|0)==3){w=13631;break}if((a[1617520+M|0]|0)==N<<24>>24){M=M+1|0}else{O=0;break}}do{if((w|0)==13631){q=N<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|N<<24>>24==95){O=0;break}c[(c[f>>2]|0)+16>>2]=8;break L18449}}while(0);while(1){P=a[x+O|0]|0;if((O|0)==3){w=13635;break}if((a[1612640+O|0]|0)==P<<24>>24){O=O+1|0}else{Q=0;break}}do{if((w|0)==13635){q=P<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|P<<24>>24==95){Q=0;break}c[(c[f>>2]|0)+16>>2]=24;break L18449}}while(0);while(1){R=a[x+Q|0]|0;if((Q|0)==3){w=13639;break}if((a[1739640+Q|0]|0)==R<<24>>24){Q=Q+1|0}else{S=0;break}}do{if((w|0)==13639){q=R<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|R<<24>>24==95){S=0;break}c[(c[f>>2]|0)+16>>2]=12;break L18449}}while(0);while(1){T=a[x+S|0]|0;if((S|0)==4){w=13643;break}if((a[1735176+S|0]|0)==T<<24>>24){S=S+1|0}else{U=0;break}}do{if((w|0)==13643){q=T<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|T<<24>>24==95){U=0;break}c[(c[f>>2]|0)+16>>2]=22;break L18449}}while(0);while(1){V=a[x+U|0]|0;if((U|0)==4){w=13647;break}if((a[1730272+U|0]|0)==V<<24>>24){U=U+1|0}else{W=0;break}}do{if((w|0)==13647){q=V<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|V<<24>>24==95){W=0;break}c[(c[f>>2]|0)+16>>2]=10;break L18449}}while(0);while(1){X=a[x+W|0]|0;if((W|0)==4){w=13651;break}if((a[1726008+W|0]|0)==X<<24>>24){W=W+1|0}else{Y=0;break}}do{if((w|0)==13651){q=X<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|X<<24>>24==95){Y=0;break}c[(c[f>>2]|0)+16>>2]=4;break L18449}}while(0);while(1){Z=a[x+Y|0]|0;if((Y|0)==3){w=13655;break}if((a[1721776+Y|0]|0)==Z<<24>>24){Y=Y+1|0}else{_=0;break}}do{if((w|0)==13655){q=Z<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|Z<<24>>24==95){_=0;break}c[(c[f>>2]|0)+16>>2]=26;break L18449}}while(0);while(1){$=a[x+_|0]|0;if((_|0)==3){w=13659;break}if((a[1717712+_|0]|0)==$<<24>>24){_=_+1|0}else{aa=0;break}}do{if((w|0)==13659){q=$<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|$<<24>>24==95){aa=0;break}c[(c[f>>2]|0)+16>>2]=18;break L18449}}while(0);while(1){ab=a[x+aa|0]|0;if((aa|0)==3){w=13663;break}if((a[1713848+aa|0]|0)==ab<<24>>24){aa=aa+1|0}else{ac=0;break}}do{if((w|0)==13663){q=ab<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|ab<<24>>24==95){ac=0;break}c[(c[f>>2]|0)+16>>2]=2;break L18449}}while(0);while(1){ad=a[x+ac|0]|0;if((ac|0)==4){w=13667;break}if((a[1711264+ac|0]|0)==ad<<24>>24){ac=ac+1|0}else{ae=0;break}}do{if((w|0)==13667){q=ad<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|ad<<24>>24==95){ae=0;break}c[(c[f>>2]|0)+16>>2]=16;break L18449}}while(0);while(1){af=a[x+ae|0]|0;if((ae|0)==6){w=13671;break}if((a[1707880+ae|0]|0)==af<<24>>24){ae=ae+1|0}else{ag=0;break}}do{if((w|0)==13671){q=af<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|af<<24>>24==95){ag=0;break}c[c[f>>2]>>2]=5;break L18449}}while(0);while(1){ah=a[x+ag|0]|0;if((ag|0)==5){w=13675;break}if((a[1705680+ag|0]|0)==ah<<24>>24){ag=ag+1|0}else{ai=0;break}}do{if((w|0)==13675){q=ah<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|ah<<24>>24==95){ai=0;break}c[c[f>>2]>>2]=6;break L18449}}while(0);while(1){aj=a[x+ai|0]|0;if((ai|0)==3){w=13679;break}if((a[1703952+ai|0]|0)==aj<<24>>24){ai=ai+1|0}else{ak=0;break}}do{if((w|0)==13679){q=aj<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aj<<24>>24==95){ak=0;break}c[c[f>>2]>>2]=10;break L18449}}while(0);while(1){al=a[x+ak|0]|0;if((ak|0)==3){w=13683;break}if((a[1701504+ak|0]|0)==al<<24>>24){ak=ak+1|0}else{am=0;break}}do{if((w|0)==13683){q=al<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|al<<24>>24==95){am=0;break}c[c[f>>2]>>2]=11;break L18449}}while(0);while(1){an=a[x+am|0]|0;if((am|0)==3){w=13687;break}if((a[1698448+am|0]|0)==an<<24>>24){am=am+1|0}else{ao=0;break}}do{if((w|0)==13687){q=an<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|an<<24>>24==95){ao=0;break}c[c[f>>2]>>2]=12;break L18449}}while(0);while(1){ap=a[x+ao|0]|0;if((ao|0)==2){w=13691;break}if((a[1696320+ao|0]|0)==ap<<24>>24){ao=ao+1|0}else{aq=0;break}}do{if((w|0)==13691){q=ap<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|ap<<24>>24==95){aq=0;break}c[c[f>>2]>>2]=13;break L18449}}while(0);while(1){ar=a[x+aq|0]|0;if((aq|0)==3){w=13695;break}if((a[1694032+aq|0]|0)==ar<<24>>24){aq=aq+1|0}else{as=0;break}}do{if((w|0)==13695){q=ar<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|ar<<24>>24==95){as=0;break}c[c[f>>2]>>2]=15;break L18449}}while(0);while(1){at=a[x+as|0]|0;if((as|0)==2){w=13699;break}if((a[1690456+as|0]|0)==at<<24>>24){as=as+1|0}else{au=0;break}}do{if((w|0)==13699){q=at<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|at<<24>>24==95){au=0;break}c[c[f>>2]>>2]=14;break L18449}}while(0);while(1){av=a[x+au|0]|0;if((au|0)==3){w=13703;break}if((a[1687416+au|0]|0)==av<<24>>24){au=au+1|0}else{aw=0;break}}do{if((w|0)==13703){q=av<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|av<<24>>24==95){aw=0;break}c[c[f>>2]>>2]=16;break L18449}}while(0);while(1){ax=a[x+aw|0]|0;if((aw|0)==2){w=13707;break}if((a[1685808+aw|0]|0)==ax<<24>>24){aw=aw+1|0}else{ay=0;break}}do{if((w|0)==13707){q=ax<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|ax<<24>>24==95){ay=0;break}c[c[f>>2]>>2]=17;break L18449}}while(0);while(1){az=a[x+ay|0]|0;if((ay|0)==2){w=13711;break}if((a[1683856+ay|0]|0)==az<<24>>24){ay=ay+1|0}else{aA=0;break}}do{if((w|0)==13711){q=az<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|az<<24>>24==95){aA=0;break}c[c[f>>2]>>2]=7;break L18449}}while(0);while(1){aB=a[x+aA|0]|0;if((aA|0)==5){w=13715;break}if((a[1682144+aA|0]|0)==aB<<24>>24){aA=aA+1|0}else{aC=0;break}}do{if((w|0)==13715){q=aB<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aB<<24>>24==95){aC=0;break}c[c[f>>2]>>2]=8;break L18449}}while(0);while(1){aD=a[x+aC|0]|0;if((aC|0)==5){w=13719;break}if((a[1680216+aC|0]|0)==aD<<24>>24){aC=aC+1|0}else{aE=0;break}}do{if((w|0)==13719){q=aD<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aD<<24>>24==95){aE=0;break}c[c[f>>2]>>2]=9;break L18449}}while(0);while(1){aF=a[x+aE|0]|0;if((aE|0)==2){w=13723;break}if((a[1678248+aE|0]|0)==aF<<24>>24){aE=aE+1|0}else{aG=0;break}}do{if((w|0)==13723){q=aF<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aF<<24>>24==95){aG=0;break}c[c[f>>2]>>2]=23;break L18449}}while(0);while(1){aH=a[x+aG|0]|0;if((aG|0)==5){w=13727;break}if((a[1676264+aG|0]|0)==aH<<24>>24){aG=aG+1|0}else{aI=0;break}}do{if((w|0)==13727){q=aH<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aH<<24>>24==95){aI=0;break}c[c[f>>2]>>2]=24;break L18449}}while(0);while(1){aJ=a[x+aI|0]|0;if((aI|0)==6){w=13731;break}if((a[1674576+aI|0]|0)==aJ<<24>>24){aI=aI+1|0}else{aK=0;break}}do{if((w|0)==13731){q=aJ<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aJ<<24>>24==95){aK=0;break}c[c[f>>2]>>2]=25;break L18449}}while(0);while(1){aL=a[x+aK|0]|0;if((aK|0)==4){w=13735;break}if((a[1672776+aK|0]|0)==aL<<24>>24){aK=aK+1|0}else{aM=0;break}}do{if((w|0)==13735){q=aL<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aL<<24>>24==95){aM=0;break}c[c[f>>2]>>2]=26;break L18449}}while(0);while(1){aN=a[x+aM|0]|0;if((aM|0)==5){w=13739;break}if((a[1670720+aM|0]|0)==aN<<24>>24){aM=aM+1|0}else{aO=0;break}}do{if((w|0)==13739){q=aN<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aN<<24>>24==95){aO=0;break}c[c[f>>2]>>2]=27;break L18449}}while(0);while(1){aP=a[x+aO|0]|0;if((aO|0)==4){w=13743;break}if((a[1668896+aO|0]|0)==aP<<24>>24){aO=aO+1|0}else{aQ=0;break}}do{if((w|0)==13743){q=aP<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aP<<24>>24==95){aQ=0;break}c[c[f>>2]>>2]=28;break L18449}}while(0);while(1){aR=a[x+aQ|0]|0;if((aQ|0)==5){w=13747;break}if((a[1667496+aQ|0]|0)==aR<<24>>24){aQ=aQ+1|0}else{aS=0;break}}do{if((w|0)==13747){q=aR<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aR<<24>>24==95){aS=0;break}c[c[f>>2]>>2]=29;break L18449}}while(0);while(1){aT=a[x+aS|0]|0;if((aS|0)==4){w=13751;break}if((a[1666304+aS|0]|0)==aT<<24>>24){aS=aS+1|0}else{aU=0;break}}do{if((w|0)==13751){q=aT<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aT<<24>>24==95){aU=0;break}c[c[f>>2]>>2]=30;break L18449}}while(0);while(1){aV=a[x+aU|0]|0;if((aU|0)==3){w=13755;break}if((a[1664840+aU|0]|0)==aV<<24>>24){aU=aU+1|0}else{aW=0;break}}do{if((w|0)==13755){q=aV<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aV<<24>>24==95){aW=0;break}c[c[f>>2]>>2]=31;break L18449}}while(0);while(1){aX=a[x+aW|0]|0;if((aW|0)==3){w=13759;break}if((a[1663488+aW|0]|0)==aX<<24>>24){aW=aW+1|0}else{aY=0;break}}do{if((w|0)==13759){q=aX<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aX<<24>>24==95){aY=0;break}c[c[f>>2]>>2]=18;break L18449}}while(0);while(1){aZ=a[x+aY|0]|0;if((aY|0)==5){w=13763;break}if((a[1661928+aY|0]|0)==aZ<<24>>24){aY=aY+1|0}else{a_=0;break}}do{if((w|0)==13763){q=aZ<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|aZ<<24>>24==95){a_=0;break}c[c[f>>2]>>2]=37;break L18449}}while(0);while(1){a$=a[x+a_|0]|0;if((a_|0)==6){w=13767;break}if((a[1660528+a_|0]|0)==a$<<24>>24){a_=a_+1|0}else{a0=0;break}}do{if((w|0)==13767){q=a$<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|a$<<24>>24==95){a0=0;break}c[c[f>>2]>>2]=32;break L18449}}while(0);while(1){a1=a[x+a0|0]|0;if((a0|0)==5){w=13771;break}if((a[1659304+a0|0]|0)==a1<<24>>24){a0=a0+1|0}else{a2=0;break}}do{if((w|0)==13771){q=a1<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|a1<<24>>24==95){a2=0;break}c[c[f>>2]>>2]=33;break L18449}}while(0);while(1){a3=a[x+a2|0]|0;if((a2|0)==3){w=13775;break}if((a[1658088+a2|0]|0)==a3<<24>>24){a2=a2+1|0}else{a4=0;break}}do{if((w|0)==13775){q=a3<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|a3<<24>>24==95){a4=0;break}c[c[f>>2]>>2]=34;break L18449}}while(0);while(1){a5=a[x+a4|0]|0;if((a4|0)==2){w=13779;break}if((a[1655936+a4|0]|0)==a5<<24>>24){a4=a4+1|0}else{a6=0;break}}do{if((w|0)==13779){q=a5<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|a5<<24>>24==95){a6=0;break}c[c[f>>2]>>2]=35;break L18449}}while(0);while(1){a7=a[x+a6|0]|0;if((a6|0)==5){w=13783;break}if((a[1654232+a6|0]|0)==a7<<24>>24){a6=a6+1|0}else{a8=0;break}}do{if((w|0)==13783){q=a7<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|a7<<24>>24==95){a8=0;break}c[c[f>>2]>>2]=36;break L18449}}while(0);while(1){a9=a[x+a8|0]|0;if((a8|0)==6){w=13787;break}if((a[1652728+a8|0]|0)==a9<<24>>24){a8=a8+1|0}else{ba=0;break}}do{if((w|0)==13787){q=a9<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|a9<<24>>24==95){ba=0;break}c[c[f>>2]>>2]=38;break L18449}}while(0);while(1){bb=a[x+ba|0]|0;if((ba|0)==5){w=13791;break}if((a[1650912+ba|0]|0)==bb<<24>>24){ba=ba+1|0}else{bc=0;break}}do{if((w|0)==13791){q=bb<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|bb<<24>>24==95){bc=0;break}c[c[f>>2]>>2]=39;break L18449}}while(0);while(1){bd=a[x+bc|0]|0;if((bc|0)==7){w=13796;break}if((a[1649360+bc|0]|0)==bd<<24>>24){bc=bc+1|0}else{break}}do{if((w|0)==13796){q=bd<<24>>24;if((q-48|0)>>>0<10|(q-97|0)>>>0<26|(q-65|0)>>>0<26|bd<<24>>24==95){break}c[c[f>>2]>>2]=40;break L18449}}while(0);q=c[d+24>>2]|0;L18715:do{if((q|0)!=0){s=0;while(1){v=c[q+(s<<2)>>2]|0;if((v|0)==0){break L18715}else{be=0}while(1){o=a[v+be|0]|0;bf=a[x+be|0]|0;if(o<<24>>24==0){w=13802;break}if(o<<24>>24==bf<<24>>24){be=be+1|0}else{break}}if((w|0)==13802){w=0;v=bf<<24>>24;if(!((v-48|0)>>>0<10|(v-97|0)>>>0<26|(v-65|0)>>>0<26|bf<<24>>24==95)){break}}s=s+1|0}c[(c[f>>2]|0)+16>>2]=c[(c[d+20>>2]|0)+(s<<2)>>2];c[c[f>>2]>>2]=3;c[b>>2]=c[f>>2];m=0;i=e;return m|0}}while(0);q=c[d+32>>2]|0;L18727:do{if((q|0)!=0){v=0;while(1){o=c[q+(v<<2)>>2]|0;if((o|0)==0){break L18727}else{bg=0}while(1){t=a[o+bg|0]|0;bh=a[x+bg|0]|0;if(t<<24>>24==0){w=13808;break}if(t<<24>>24==bh<<24>>24){bg=bg+1|0}else{break}}if((w|0)==13808){w=0;o=bh<<24>>24;if(!((o-48|0)>>>0<10|(o-97|0)>>>0<26|(o-65|0)>>>0<26|bh<<24>>24==95)){break}}v=v+1|0}c[(c[f>>2]|0)+16>>2]=c[(c[d+28>>2]|0)+(v<<2)>>2];c[c[f>>2]>>2]=4;c[b>>2]=c[f>>2];m=0;i=e;return m|0}}while(0);AN(d,16,1646696,(E=i,i=i+8|0,c[E>>2]=l,E)|0);i=E;zP(c[f>>2]|0);m=-22;i=e;return m|0}}while(0);c[b>>2]=c[f>>2];m=0;i=e;return m|0}}while(0);c[k>>2]=r+(NL(c[A>>2]|0)|0);c[j>>2]=0;j=c[f>>2]|0;h[j+8>>3]=+h[264040+(z<<4)>>3];c[b>>2]=j;m=0;i=e;return m|0}function z_(a){a=+a;var b=0;b=Ch()|0;return+((+(b>>>0)+ +(G|0)*4294967296.0)*1.0e-6)}function z$(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=i;i=i+8|0;d=b|0;e=A7(24)|0;f=e;c[d>>2]=f;if((e|0)==0){g=0;i=b;return g|0}h=A1(a)|0;c[e>>2]=h;c[e+12>>2]=h+a;c[e+4>>2]=h;c[e+8>>2]=h;c[e+16>>2]=0;c[e+20>>2]=0;if((h|0)!=0){g=f;i=b;return g|0}A6(d);g=c[d>>2]|0;i=b;return g|0}function z0(a){a=a|0;var b=0;if((a|0)==0){return}b=a;A6(b);A4(b);return}function z1(a){a=a|0;return(c[a+20>>2]|0)-(c[a+16>>2]|0)|0}function z2(a){a=a|0;return(c[a+12>>2]|0)-(c[a>>2]|0)-(c[a+20>>2]|0)+(c[a+16>>2]|0)|0}function z3(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;d=i;i=i+8|0;e=d|0;f=a+12|0;g=a|0;if(((c[f>>2]|0)-(c[g>>2]|0)|0)>>>0>=b>>>0){h=0;i=d;return h|0}j=a+16|0;k=(c[a+20>>2]|0)-(c[j>>2]|0)|0;l=A7(24)|0;m=l;c[e>>2]=m;if((l|0)==0){h=-12;i=d;return h|0}n=A1(b)|0;c[l>>2]=n;c[l+12>>2]=n+b;c[l+4>>2]=n;c[l+8>>2]=n;c[l+16>>2]=0;c[l+20>>2]=0;if((n|0)==0){A6(e);o=c[e>>2]|0}else{o=m}if((o|0)==0){h=-12;i=d;return h|0}m=a+4|0;e=c[o>>2]|0;n=k;l=c[f>>2]|0;b=c[m>>2]|0;while(1){p=l-b|0;q=(p|0)>(n|0)?n:p;NN(e|0,b|0,q)|0;p=c[m>>2]|0;r=c[f>>2]|0;s=p+q|0;c[m>>2]=s;if(s>>>0<r>>>0){t=s}else{s=p+(q-r+(c[g>>2]|0))|0;c[m>>2]=s;t=s}c[j>>2]=(c[j>>2]|0)+q;s=n-q|0;if((s|0)>0){e=e+q|0;n=s;l=r;b=t}else{break}}t=o+8|0;c[t>>2]=(c[t>>2]|0)+k;t=o+20|0;c[t>>2]=(c[t>>2]|0)+k;A4(c[g>>2]|0);g=a;a=o;c[g>>2]=c[a>>2];c[g+4>>2]=c[a+4>>2];c[g+8>>2]=c[a+8>>2];c[g+12>>2]=c[a+12>>2];c[g+16>>2]=c[a+16>>2];c[g+20>>2]=c[a+20>>2];A4(a);h=0;i=d;return h|0}function z4(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;f=a+12|0;g=a+4|0;h=a+16|0;i=a|0;a=c[f>>2]|0;j=c[g>>2]|0;if((e|0)==0){k=b;l=d;m=a;n=j;while(1){o=m-n|0;p=(o|0)>(l|0)?l:o;NN(k|0,n|0,p)|0;o=c[g>>2]|0;q=c[f>>2]|0;r=o+p|0;c[g>>2]=r;if(r>>>0<q>>>0){s=r}else{r=o+(p-q+(c[i>>2]|0))|0;c[g>>2]=r;s=r}c[h>>2]=(c[h>>2]|0)+p;r=l-p|0;if((r|0)>0){k=k+p|0;l=r;m=q;n=s}else{break}}return 0}else{s=d;d=a;a=j;while(1){j=d-a|0;n=(j|0)>(s|0)?s:j;dm[e&127](b,a,n);j=c[g>>2]|0;m=c[f>>2]|0;l=j+n|0;c[g>>2]=l;if(l>>>0<m>>>0){t=l}else{l=j+(n-m+(c[i>>2]|0))|0;c[g>>2]=l;t=l}c[h>>2]=(c[h>>2]|0)+n;l=s-n|0;if((l|0)>0){s=l;d=m;a=t}else{break}}return 0}return 0}function z5(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;f=a+20|0;g=c[f>>2]|0;h=a+8|0;i=c[h>>2]|0;j=a+12|0;k=a|0;a=c[j>>2]|0;if((e|0)==0){l=b;m=i;n=g;o=d;p=a;while(1){q=p-m|0;r=(q|0)>(o|0)?o:q;NN(m|0,l|0,r)|0;q=c[j>>2]|0;s=m+r|0;if(s>>>0<q>>>0){t=s}else{t=c[k>>2]|0}s=r+n|0;u=o-r|0;if((u|0)>0){l=l+r|0;m=t;n=s;o=u;p=q}else{v=t;w=s;x=u;break}}c[f>>2]=w;c[h>>2]=v;y=d-x|0;return y|0}else{z=i;A=g;B=d;C=a}while(1){a=C-z|0;g=(a|0)>(B|0)?B:a;if((c6[e&255](b,z,g)|0)<1){v=z;w=A;x=B;D=13886;break}a=c[j>>2]|0;i=z+g|0;if(i>>>0<a>>>0){E=i}else{E=c[k>>2]|0}i=g+A|0;t=B-g|0;if((t|0)>0){z=E;A=i;B=t;C=a}else{v=E;w=i;x=t;D=13884;break}}if((D|0)==13886){c[f>>2]=w;c[h>>2]=v;y=d-x|0;return y|0}else if((D|0)==13884){c[f>>2]=w;c[h>>2]=v;y=d-x|0;return y|0}return 0}function z6(a,b,c){a=a|0;b=b|0;c=c|0;var d=0.0,e=0,f=0.0,h=0.0,i=0;if((c|0)>0){d=0.0;e=0}else{f=0.0;return+f}while(1){h=d+ +g[a+(e<<2)>>2]*+g[b+(e<<2)>>2];i=e+1|0;if((i|0)<(c|0)){d=h;e=i}else{f=h;break}}return+f}function z7(a,b){a=a|0;b=b|0;c[a>>2]=46;c[a+4>>2]=6;c[a+8>>2]=8;c[a+12>>2]=4;c[a+16>>2]=62;c[a+20>>2]=84;c[a+24>>2]=84;c[a+28>>2]=24;c[a+32>>2]=2;return}function z8(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;if((d|0)>0){e=0}else{return}do{g[a+(e<<2)>>2]=+g[b+(e<<2)>>2]*+g[c+(e<<2)>>2];e=e+1|0;}while((e|0)<(d|0));return}function z9(a,b,c,d){a=a|0;b=b|0;c=+c;d=d|0;var e=0,f=0;if((d|0)>0){e=0}else{return}do{f=a+(e<<2)|0;g[f>>2]=+g[f>>2]+ +g[b+(e<<2)>>2]*c;e=e+1|0;}while((e|0)<(d|0));return}function Aa(a,b,c,d){a=a|0;b=b|0;c=+c;d=d|0;var e=0;if((d|0)>0){e=0}else{return}do{g[a+(e<<2)>>2]=+g[b+(e<<2)>>2]*c;e=e+1|0;}while((e|0)<(d|0));return}function Ab(a,b,c,d){a=a|0;b=b|0;c=+c;d=d|0;var e=0;if((d|0)>0){e=0}else{return}do{h[a+(e<<3)>>3]=+h[b+(e<<3)>>3]*c;e=e+1|0;}while((e|0)<(d|0));return}function Ac(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0.0,k=0.0,l=0.0,m=0,n=0.0;if((e|0)<=0){return}f=-e|0;h=e;do{h=h-1|0;i=f+e|0;j=+g[b+(i<<2)>>2];k=+g[c+(h<<2)>>2];l=+g[d+(i<<2)>>2];m=h+e|0;n=+g[d+(m<<2)>>2];g[a+(i<<2)>>2]=j*n-k*l;g[a+(m<<2)>>2]=j*l+k*n;f=f+1|0;}while((f|0)<0);return}function Ad(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0;if((e|0)>0){f=0}else{return}do{g[a+(f<<2)>>2]=+g[b+(f<<2)>>2]*+g[c+(f<<2)>>2]+ +g[d+(f<<2)>>2];f=f+1|0;}while((f|0)<(e|0));return}function Ae(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0;e=d-1|0;if((d|0)>0){f=0}else{return}do{g[a+(f<<2)>>2]=+g[b+(f<<2)>>2]*+g[c+(e-f<<2)>>2];f=f+1|0;}while((f|0)<(d|0));return}function Af(a,b,c){a=a|0;b=b|0;c=c|0;var d=0,e=0,f=0.0,h=0,i=0.0;if((c|0)>0){d=0}else{return}do{e=a+(d<<2)|0;f=+g[e>>2];h=b+(d<<2)|0;i=+g[h>>2];g[e>>2]=f+i;g[h>>2]=f-i;d=d+1|0;}while((d|0)<(c|0));return}function Ag(a){a=a|0;var b=0;b=a+408|0;return(G=c[b+4>>2]|0,c[b>>2]|0)|0}function Ah(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=a+408|0;c[e>>2]=b;c[e+4>>2]=d;return}function Ai(a){a=a|0;var b=0;b=a+424|0;return(G=c[b+4>>2]|0,c[b>>2]|0)|0}function Aj(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=a+424|0;c[e>>2]=b;c[e+4>>2]=d;return}function Ak(a){a=a|0;var b=0;b=a+416|0;return(G=c[b+4>>2]|0,c[b>>2]|0)|0}function Al(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=a+416|0;c[e>>2]=b;c[e+4>>2]=d;return}function Am(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=a+352|0;c[e>>2]=b;c[e+4>>2]=d;return}function An(a){a=a|0;return c[a+440>>2]|0}function Ao(a,b){a=a|0;b=b|0;c[a+440>>2]=b;return}function Ap(a,b){a=a|0;b=b|0;c[a+344>>2]=b;return}function Aq(a,b){a=a|0;b=b|0;c[a+444>>2]=b;return}function Ar(a){a=a|0;return a+432|0}function As(){var a=0,b=0,d=0,e=0;a=A7(456)|0;if((a|0)==0){b=0;return b|0}d=a+64|0;e=d;c[e>>2]=0;A6(d);NM(a|0,0,456);d=a+144|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=a+152|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=a+136|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=a+408|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=a+424|0;c[d>>2]=0;c[d+4>>2]=0;d=a+416|0;c[d>>2]=-1;c[d+4>>2]=-1;c[a+444>>2]=-1;c[a+84>>2]=1;c[a+124>>2]=0;c[a+128>>2]=1;c[a+80>>2]=-1;c[e>>2]=a;b=a;return b|0}function At(a){a=a|0;var b=0;if((a|0)==0){return}b=c[a>>2]|0;if((b|0)==0){return}Au(b);A6(a);return}function Au(a){a=a|0;var b=0,d=0,e=0;b=a+404|0;d=a+400|0;if((c[b>>2]|0)>0){e=0;do{A6((c[(c[d>>2]|0)+(e<<2)>>2]|0)+4|0);zL((c[(c[d>>2]|0)+(e<<2)>>2]|0)+12|0);A6((c[d>>2]|0)+(e<<2)|0);e=e+1|0;}while((e|0)<(c[b>>2]|0))}A6(d);zj(a+360|0);zj(a+364|0);zj(a+368|0);zj(a+372|0);zj(a+376|0);zj(a+380|0);zj(a+384|0);zj(a+388|0);d=a+396|0;b=a+392|0;if((c[d>>2]|0)>0){e=0;do{zj((c[b>>2]|0)+(e<<2)|0);e=e+1|0;}while((e|0)<(c[d>>2]|0))}A6(b);zL(a+432|0);zj(a+448|0);b=a+64|0;d=a|0;if((c[b>>2]|0)!=(d|0)){A6(b)}NM(a|0,0,456);e=a+144|0;c[e>>2]=0;c[e+4>>2]=-2147483648;e=a+152|0;c[e>>2]=0;c[e+4>>2]=-2147483648;e=a+136|0;c[e>>2]=0;c[e+4>>2]=-2147483648;e=a+408|0;c[e>>2]=0;c[e+4>>2]=-2147483648;e=a+424|0;c[e>>2]=0;c[e+4>>2]=0;e=a+416|0;c[e>>2]=-1;c[e+4>>2]=-1;c[a+444>>2]=-1;c[a+84>>2]=1;c[a+124>>2]=0;c[a+128>>2]=1;c[a+80>>2]=-1;c[b>>2]=d;return}function Av(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;f=b+80|0;g=c[f>>2]|0;if((g|0)<0){h=-22;return h|0}i=b+68|0;do{if((c[i>>2]|0)>0){j=b+72|0;if((c[j>>2]|0)<=0){break}k=BP(g)|0;if((k|0)==0){h=-22;return h|0}l=AF(c[i>>2]|0,c[j>>2]|0,0,0)|0;if((l|0)<0){h=l;return h|0}l=b+32|0;m=c[l>>2]|0;do{if((m|0)==0){n=e-1|0;o=1;p=0;while(1){if((o|0)>(e|0)){q=p;break}r=AC(l,c[f>>2]|0,o-1+(c[i>>2]|0)&-o)|0;if((r|0)<0){h=r;s=14023;break}r=c[l>>2]|0;if((r&n|0)==0){q=r;break}else{o=o<<1;p=r}}if((s|0)==14023){return h|0}p=-e|0;if((q|0)==0){s=13980;break}o=q+n&p;c[l>>2]=o;r=b+36|0;t=c[r>>2]|0;do{if((t|0)!=0){c[r>>2]=t+n&p;u=b+40|0;v=c[u>>2]|0;if((v|0)==0){break}c[u>>2]=v+n&p;v=b+44|0;u=c[v>>2]|0;if((u|0)==0){break}c[v>>2]=u+n&p}}while(0);if((o|0)==0){s=13980}else{w=o;s=13978}}else{w=m;s=13978}}while(0);do{if((s|0)==13978){m=k+6|0;l=zg(ac((c[j>>2]|0)+31&-32,w)|0|16)|0;c[b+360>>2]=l;if((l|0)==0){break}c[b>>2]=c[l+4>>2];l=c[b+36>>2]|0;if((l|0)==0){s=13980;break}p=zg(16-(ac(-((c[j>>2]|0)+31&-32)>>(d[m]|0),l)|0)|0)|0;c[b+364>>2]=p;if((p|0)==0){break}c[b+4>>2]=c[p+4>>2];p=c[b+40>>2]|0;if((p|0)==0){s=13980;break}l=zg(16-(ac(-((c[j>>2]|0)+31&-32)>>(d[m]|0),p)|0)|0)|0;c[b+368>>2]=l;if((l|0)==0){break}c[b+8>>2]=c[l+4>>2];l=c[b+44>>2]|0;if((l|0)==0){s=13980;break}p=zg(ac((c[j>>2]|0)+31&-32,l)|0|16)|0;c[b+372>>2]=p;if((p|0)==0){break}c[b+12>>2]=c[p+4>>2];s=13980}}while(0);do{if((s|0)==13980){if((a[k+7|0]&66)!=0){j=b+364|0;zj(j);p=zg(1024)|0;c[j>>2]=p;if((p|0)==0){break}c[b+4>>2]=c[p+4>>2]}c[b+64>>2]=b;h=0;return h|0}}while(0);Au(b);h=-12;return h|0}}while(0);w=b+76|0;if((c[w>>2]|0)<=0){h=-22;return h|0}q=b+352|0;if((c[q>>2]|0)==0&(c[q+4>>2]|0)==0){h=-22;return h|0}q=c[b+440>>2]|0;i=(B8(g)|0)!=0?q:1;g=b+32|0;do{if((c[g>>2]|0)==0){k=B9(g,q,c[w>>2]|0,c[f>>2]|0,e)|0;if((k|0)<0){h=k}else{break}return h|0}}while(0);e=(i|0)>8;do{if(e){f=b+64|0;c[f>>2]=A7(i<<2)|0;w=i-8|0;q=A7(w<<2)|0;k=b+392|0;c[k>>2]=q;if(!((c[f>>2]|0)==0|(q|0)==0)){c[b+396>>2]=w;break}A6(f);A6(k);h=-12;return h|0}else{c[b+64>>2]=b}}while(0);k=e?8:i;L18969:do{if((k|0)>0){e=b+64|0;f=0;while(1){w=zg(c[g>>2]|0)|0;c[b+360+(f<<2)>>2]=w;if((w|0)==0){break}q=c[w+4>>2]|0;c[b+(f<<2)>>2]=q;c[(c[e>>2]|0)+(f<<2)>>2]=q;f=f+1|0;if((f|0)>=(k|0)){break L18969}}Au(b);h=-12;return h|0}}while(0);k=i-8|0;if((k|0)<=0){h=0;return h|0}i=b+392|0;f=b+64|0;e=0;while(1){q=zg(c[g>>2]|0)|0;c[(c[i>>2]|0)+(e<<2)>>2]=q;q=c[(c[i>>2]|0)+(e<<2)>>2]|0;if((q|0)==0){break}c[(c[f>>2]|0)+(e+8<<2)>>2]=c[q+4>>2];q=e+1|0;if((q|0)<(k|0)){e=q}else{h=0;s=14021;break}}if((s|0)==14021){return h|0}Au(b);h=-12;return h|0}function Aw(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;d=a+80|0;c[d>>2]=c[b+80>>2];e=a+68|0;c[e>>2]=c[b+68>>2];f=a+72|0;c[f>>2]=c[b+72>>2];g=b+440|0;c[a+440>>2]=c[g>>2];h=b+352|0;i=c[h+4>>2]|0;j=a+352|0;c[j>>2]=c[h>>2];c[j+4>>2]=i;i=b+76|0;j=a+76|0;c[j>>2]=c[i>>2];h=Ax(a,b)|0;if((h|0)<0){k=h;return k|0}h=c[b+360>>2]|0;if((h|0)==0){l=Av(a,32)|0;if((l|0)<0){k=l;return k|0}if((c[i>>2]|0)==0){AG(a|0,a+32|0,b|0,b+32|0,c[d>>2]|0,c[e>>2]|0,c[f>>2]|0);k=0;return k|0}else{f=c[g>>2]|0;e=c[a+64>>2]|0;i=c[b+64>>2]|0;l=c[j>>2]|0;j=c[d>>2]|0;Cb(e,i,0,0,l,f,j)|0;k=0;return k|0}}j=zi(h)|0;c[a+360>>2]=j;L19002:do{if((j|0)==0){m=-12}else{h=c[b+364>>2]|0;if((h|0)!=0){f=zi(h)|0;c[a+364>>2]=f;if((f|0)==0){m=-12;break}}f=c[b+368>>2]|0;if((f|0)!=0){h=zi(f)|0;c[a+368>>2]=h;if((h|0)==0){m=-12;break}}h=c[b+372>>2]|0;if((h|0)!=0){f=zi(h)|0;c[a+372>>2]=f;if((f|0)==0){m=-12;break}}f=c[b+376>>2]|0;if((f|0)!=0){h=zi(f)|0;c[a+376>>2]=h;if((h|0)==0){m=-12;break}}h=c[b+380>>2]|0;if((h|0)!=0){f=zi(h)|0;c[a+380>>2]=f;if((f|0)==0){m=-12;break}}f=c[b+384>>2]|0;if((f|0)!=0){h=zi(f)|0;c[a+384>>2]=h;if((h|0)==0){m=-12;break}}h=c[b+388>>2]|0;if((h|0)!=0){f=zi(h)|0;c[a+388>>2]=f;if((f|0)==0){m=-12;break}}f=b+392|0;L19025:do{if((c[f>>2]|0)!=0){h=b+396|0;l=A7(c[h>>2]<<2)|0;i=a+392|0;c[i>>2]=l;if((l|0)==0){m=-12;break L19002}c[a+396>>2]=c[h>>2];l=0;while(1){if((l|0)>=(c[h>>2]|0)){break L19025}e=zi(c[(c[f>>2]|0)+(l<<2)>>2]|0)|0;c[(c[i>>2]|0)+(l<<2)>>2]=e;if((c[(c[i>>2]|0)+(l<<2)>>2]|0)==0){m=-12;break L19002}else{l=l+1|0}}}}while(0);f=b+64|0;if((c[f>>2]|0)==(b|0)){c[a+64>>2]=a}else{l=c[g>>2]|0;if((l|0)==0){m=-22;break}i=l<<2;l=A1(i)|0;c[a+64>>2]=l;if((l|0)==0){m=-12;break}h=c[f>>2]|0;NN(l|0,h|0,i)|0}i=a;h=b;NN(i|0,h|0,32)|0;h=a+32|0;i=b+32|0;NN(h|0,i|0,32)|0;k=0;return k|0}}while(0);Au(a);k=m;return k|0}function Ax(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;d=i;i=i+8|0;e=d|0;c[a+84>>2]=c[b+84>>2];c[a+88>>2]=c[b+88>>2];f=b+124|0;g=a+124|0;h=c[f+4>>2]|0;c[g>>2]=c[f>>2];c[g+4>>2]=h;h=b+136|0;g=c[h+4>>2]|0;f=a+136|0;c[f>>2]=c[h>>2];c[f+4>>2]=g;c[a+292>>2]=c[b+292>>2];c[a+296>>2]=c[b+296>>2];c[a+300>>2]=c[b+300>>2];c[a+304>>2]=c[b+304>>2];c[a+344>>2]=c[b+344>>2];c[a+216>>2]=c[b+216>>2];c[a+288>>2]=c[b+288>>2];g=b+144|0;f=c[g+4>>2]|0;h=a+144|0;c[h>>2]=c[g>>2];c[h+4>>2]=f;f=b+152|0;h=c[f+4>>2]|0;g=a+152|0;c[g>>2]=c[f>>2];c[g+4>>2]=h;h=b+416|0;g=c[h+4>>2]|0;f=a+416|0;c[f>>2]=c[h>>2];c[f+4>>2]=g;c[a+444>>2]=c[b+444>>2];g=b+424|0;f=c[g+4>>2]|0;h=a+424|0;c[h>>2]=c[g>>2];c[h+4>>2]=f;f=b+320|0;h=c[f+4>>2]|0;g=a+320|0;c[g>>2]=c[f>>2];c[g+4>>2]=h;c[a+168>>2]=c[b+168>>2];h=b+408|0;g=c[h+4>>2]|0;f=a+408|0;c[f>>2]=c[h>>2];c[f+4>>2]=g;c[a+160>>2]=c[b+160>>2];c[a+164>>2]=c[b+164>>2];c[a+436>>2]=c[b+436>>2];zM(a+432|0,c[b+432>>2]|0,0);g=a+224|0;f=b+224|0;NN(g|0,f|0,64)|0;f=b+404|0;L19042:do{if((c[f>>2]|0)>0){g=b+400|0;h=e;j=a+404|0;k=a+400|0;l=0;while(1){m=c[(c[g>>2]|0)+(l<<2)>>2]|0;n=c[m>>2]|0;o=m+8|0;p=c[o>>2]|0;q=c[j>>2]|0;if(q>>>0>536870910){r=q;break}s=A2(c[k>>2]|0,(q<<2)+4|0)|0;if((s|0)==0){t=14076;break}c[k>>2]=s;s=A7(16)|0;q=s;c[e>>2]=q;if((s|0)==0){t=14076;break}u=A1(p)|0;c[s+4>>2]=u;if((u|0)==0){t=14074;break}c[s+8>>2]=p;c[s>>2]=n;n=c[j>>2]|0;c[j>>2]=n+1;c[(c[k>>2]|0)+(n<<2)>>2]=q;q=c[e>>2]|0;if((q|0)==0){t=14076;break}n=c[q+4>>2]|0;s=c[m+4>>2]|0;p=c[o>>2]|0;NN(n|0,s|0,p)|0;zM(q+12|0,c[m+12>>2]|0,0);l=l+1|0;if((l|0)>=(c[f>>2]|0)){break L19042}}if((t|0)==14074){A6(h);t=14076}if((t|0)==14076){r=c[j>>2]|0}if((r|0)>0){l=0;do{A6((c[(c[k>>2]|0)+(l<<2)>>2]|0)+4|0);A6((c[k>>2]|0)+(l<<2)|0);zL((c[(c[k>>2]|0)+(l<<2)>>2]|0)+12|0);l=l+1|0;}while((l|0)<(c[j>>2]|0))}A6(k);v=-12;i=d;return v|0}}while(0);r=a+176|0;c[r>>2]=0;t=a+180|0;c[t>>2]=0;f=a+184|0;c[f>>2]=0;e=c[b+448>>2]|0;if((e|0)==0){v=0;i=d;return v|0}j=zi(e)|0;c[a+448>>2]=j;if((j|0)==0){v=0;i=d;return v|0}c[r>>2]=c[j+4>>2];c[t>>2]=c[b+180>>2];c[f>>2]=c[b+184>>2];v=0;i=d;return v|0}function Ay(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0;b=i;i=i+8|0;d=b|0;e=A7(456)|0;if((e|0)==0){c[d>>2]=0;f=0;i=b;return f|0}g=e;h=e+64|0;j=h;c[j>>2]=0;A6(h);NM(e|0,0,456);h=e+144|0;c[h>>2]=0;c[h+4>>2]=-2147483648;h=e+152|0;c[h>>2]=0;c[h+4>>2]=-2147483648;h=e+136|0;c[h>>2]=0;c[h+4>>2]=-2147483648;h=e+408|0;c[h>>2]=0;c[h+4>>2]=-2147483648;h=e+424|0;c[h>>2]=0;c[h+4>>2]=0;h=e+416|0;c[h>>2]=-1;c[h+4>>2]=-1;c[e+444>>2]=-1;c[e+84>>2]=1;c[e+124>>2]=0;c[e+128>>2]=1;c[e+80>>2]=-1;c[j>>2]=e;c[d>>2]=g;if((Aw(g,a)|0)>=0){f=g;i=b;return f|0}Au(g);A6(d);f=c[d>>2]|0;i=b;return f|0}function Az(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=a;e=b;NN(d|0,e|0,456)|0;d=b+64|0;f=b|0;if((c[d>>2]|0)==(f|0)){c[a+64>>2]=a;NM(e|0,0,456)}else{NM(e|0,0,456)}A6(d);NM(e|0,0,456);e=b+144|0;c[e>>2]=0;c[e+4>>2]=-2147483648;e=b+152|0;c[e>>2]=0;c[e+4>>2]=-2147483648;e=b+136|0;c[e>>2]=0;c[e+4>>2]=-2147483648;e=b+408|0;c[e>>2]=0;c[e+4>>2]=-2147483648;e=b+424|0;c[e>>2]=0;c[e+4>>2]=0;e=b+416|0;c[e>>2]=-1;c[e+4>>2]=-1;c[b+444>>2]=-1;c[b+84>>2]=1;c[b+124>>2]=0;c[b+128>>2]=1;c[b+80>>2]=-1;c[d>>2]=f;return}function AA(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;b=c[a+360>>2]|0;if((b|0)==0){d=0;return d|0}e=(zk(b)|0)!=0;b=c[a+364>>2]|0;if((b|0)==0){f=e}else{f=(zk(b)|0)!=0&e}e=c[a+368>>2]|0;if((e|0)==0){g=f}else{g=(zk(e)|0)!=0&f}f=c[a+372>>2]|0;if((f|0)==0){h=g}else{h=(zk(f)|0)!=0&g}g=c[a+376>>2]|0;if((g|0)==0){i=h}else{i=(zk(g)|0)!=0&h}h=c[a+380>>2]|0;if((h|0)==0){j=i}else{j=(zk(h)|0)!=0&i}i=c[a+384>>2]|0;if((i|0)==0){k=j}else{k=(zk(i)|0)!=0&j}j=c[a+388>>2]|0;if((j|0)==0){l=k}else{l=(zk(j)|0)!=0&k}k=l&1;l=a+396|0;if((c[l>>2]|0)<=0){d=k;return d|0}j=a+392|0;a=0;i=k;while(1){k=(zk(c[(c[j>>2]|0)+(a<<2)>>2]|0)|0)!=0&i;h=a+1|0;if((h|0)<(c[l>>2]|0)){a=h;i=k}else{d=k;break}}return d|0}function AB(a,d,e){a=a|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0;NM(a|0,0,16);if((d|0)==0){f=b[e+8>>1]|0;g=((f&65535)>>>2&7)+1|0;h=a+((f&3)<<2)|0;if((g|0)>(c[h>>2]|0)){c[h>>2]=g}g=b[e+10>>1]|0;h=((g&65535)>>>2&7)+1|0;f=a+((g&3)<<2)|0;if((h|0)>(c[f>>2]|0)){c[f>>2]=h}h=b[e+12>>1]|0;f=((h&65535)>>>2&7)+1|0;g=a+((h&3)<<2)|0;if((f|0)>(c[g>>2]|0)){c[g>>2]=f}f=b[e+14>>1]|0;g=((f&65535)>>>2&7)+1|0;h=a+((f&3)<<2)|0;if((g|0)<=(c[h>>2]|0)){return}c[h>>2]=g;return}else{NM(d|0,0,16);g=e+8|0;h=b[g>>1]|0;f=((h&65535)>>>2&7)+1|0;i=a+((h&3)<<2)|0;if((f|0)>(c[i>>2]|0)){c[i>>2]=f;c[d+((b[g>>1]&3)<<2)>>2]=0}g=e+10|0;f=b[g>>1]|0;i=((f&65535)>>>2&7)+1|0;h=a+((f&3)<<2)|0;if((i|0)>(c[h>>2]|0)){c[h>>2]=i;c[d+((b[g>>1]&3)<<2)>>2]=1}g=e+12|0;i=b[g>>1]|0;h=((i&65535)>>>2&7)+1|0;f=a+((i&3)<<2)|0;if((h|0)>(c[f>>2]|0)){c[f>>2]=h;c[d+((b[g>>1]&3)<<2)>>2]=2}g=e+14|0;e=b[g>>1]|0;h=((e&65535)>>>2&7)+1|0;f=a+((e&3)<<2)|0;if((h|0)<=(c[f>>2]|0)){return}c[f>>2]=h;c[d+((b[g>>1]&3)<<2)>>2]=3;return}}function AC(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;g=i;i=i+32|0;h=g|0;j=g+16|0;k=BP(e)|0;NM(b|0,0,16);if((k|0)==0){l=-22;i=g;return l|0}e=k+7|0;if((a[e]&8)!=0){l=-22;i=g;return l|0}m=h|0;n=j|0;AB(m,n,k);o=k+5|0;k=f-1|0;if((f|0)<0){l=-22;i=g;return l|0}f=c[m>>2]|0;if(((c[n>>2]|0)-1|0)>>>0<2){p=d[o]|0}else{p=0}n=k+(1<<p)>>p;do{if((n|0)!=0){if((2147483647/(n|0)|0|0)<(f|0)){l=-22}else{break}i=g;return l|0}}while(0);p=ac(n,f)|0;if((a[e]&4)==0){q=p}else{q=p+7>>3}if((q|0)<0){l=q;i=g;return l|0}c[b>>2]=q;q=c[h+4>>2]|0;if(((c[j+4>>2]|0)-1|0)>>>0<2){r=d[o]|0}else{r=0}p=k+(1<<r)>>r;do{if((p|0)!=0){if((2147483647/(p|0)|0|0)<(q|0)){l=-22}else{break}i=g;return l|0}}while(0);r=ac(p,q)|0;if((a[e]&4)==0){s=r}else{s=r+7>>3}if((s|0)<0){l=s;i=g;return l|0}c[b+4>>2]=s;s=c[h+8>>2]|0;if(((c[j+8>>2]|0)-1|0)>>>0<2){t=d[o]|0}else{t=0}r=k+(1<<t)>>t;do{if((r|0)!=0){if((2147483647/(r|0)|0|0)<(s|0)){l=-22}else{break}i=g;return l|0}}while(0);t=ac(r,s)|0;if((a[e]&4)==0){u=t}else{u=t+7>>3}if((u|0)<0){l=u;i=g;return l|0}c[b+8>>2]=u;u=c[h+12>>2]|0;if(((c[j+12>>2]|0)-1|0)>>>0<2){v=d[o]|0}else{v=0}o=k+(1<<v)>>v;do{if((o|0)!=0){if((2147483647/(o|0)|0|0)<(u|0)){l=-22}else{break}i=g;return l|0}}while(0);v=ac(o,u)|0;if((a[e]&4)==0){w=v}else{w=v+7>>3}if((w|0)<0){l=w;i=g;return l|0}c[b+12>>2]=w;l=0;i=g;return l|0}function AD(e,f,g,h,j){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;k=i;i=i+16|0;l=k|0;NM(l|0,0,16);m=BP(f)|0;NM(e|0,0,16);if((m|0)==0){n=-22;i=k;return n|0}f=m+7|0;if((a[f]&8)!=0){n=-22;i=k;return n|0}c[e>>2]=h;o=c[j>>2]|0;if((o|0)>(2147482623/(g|0)|0|0)){n=-22;i=k;return n|0}p=ac(o,g)|0;if((a[f]&66)!=0){f=p+3&-4;c[e+4>>2]=h+f;n=f+1024|0;i=k;return n|0}c[l+((b[m+8>>1]&3)<<2)>>2]=1;c[l+((b[m+10>>1]&3)<<2)>>2]=1;c[l+((b[m+12>>1]&3)<<2)>>2]=1;c[l+((b[m+14>>1]&3)<<2)>>2]=1;f=g-1|0;o=m+6|0;if((c[l+4>>2]|0)==0){n=p;i=k;return n|0}m=d[o]|0;c[e+4>>2]=h+p;q=f+(1<<m)>>m;m=c[j+4>>2]|0;if((m|0)>(2147483647/(q|0)|0|0)){n=-22;i=k;return n|0}r=ac(q,m)|0;if((p|0)>(2147483647-r|0)){n=-22;i=k;return n|0}m=r+p|0;if((c[l+8>>2]|0)==0){n=m;i=k;return n|0}q=d[o]|0;o=p+r|0;c[e+8>>2]=h+o;r=f+(1<<q)>>q;q=c[j+8>>2]|0;if((q|0)>(2147483647/(r|0)|0|0)){n=-22;i=k;return n|0}f=ac(r,q)|0;if((m|0)>(2147483647-f|0)){n=-22;i=k;return n|0}q=f+m|0;if((c[l+12>>2]|0)==0){n=q;i=k;return n|0}c[e+12>>2]=h+(o+f);f=c[j+12>>2]|0;if((f|0)>(2147483647/(g|0)|0|0)){n=-22;i=k;return n|0}else{n=ac(f,g)|0;i=k;return((q|0)>(2147483647-n|0)?-22:n+q|0)|0}return 0}function AE(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;if((b|0)==22){d=0;while(1){c[a+(d<<2)>>2]=(ac(d>>5,2359296)|0)-16777216|((d>>>2&7)*9216|0)|((d&3)*85|0);e=d+1|0;if((e|0)<256){d=e}else{f=0;break}}return f|0}else{g=0}while(1){if((b|0)==19){h=(g>>6)*85|0;i=(g>>>3&7)*36|0;j=(g&7)*36|0}else if((b|0)==21){h=(g>>3)*255|0;i=(g>>>1&3)*85|0;j=-(g&1)&255}else if((b|0)==24){h=-(g&1)&255;i=(g>>>1&3)*85|0;j=(g>>3)*255|0}else if((b|0)==8){h=g;i=g;j=g}else{f=-22;k=14226;break}c[a+(g<<2)>>2]=(j<<16)-16777216+(i<<8)+h;d=g+1|0;if((d|0)<256){g=d}else{f=0;k=14225;break}}if((k|0)==14226){return f|0}else if((k|0)==14225){return f|0}return 0}function AF(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0;f=i;i=i+16|0;g=f|0;c[g>>2]=172960;c[g+4>>2]=d;c[g+8>>2]=e;do{if((a|0)>0&(b|0)>0){e=N2(b+128|0,0,a+128|0,0)|0;d=G;h=0;if(d>>>0<h>>>0|d>>>0==h>>>0&e>>>0<268435455>>>0){j=0}else{break}i=f;return j|0}}while(0);AN(g,16,1634824,(g=i,i=i+16|0,c[g>>2]=a,c[g+8>>2]=b,g)|0);i=g;j=-22;i=f;return j|0}function AG(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0;m=i;i=i+32|0;n=m|0;o=m+16|0;p=BP(j)|0;if((p|0)==0){i=m;return}q=d[p+7|0]|0;if((q&8|0)!=0){i=m;return}if((q&66|0)!=0){q=c[e>>2]|0;r=c[f>>2]|0;s=c[g>>2]|0;t=c[h>>2]|0;do{if(!((q|0)==0|(s|0)==0)){if((((t|0)>-1?t:-t|0)|0)<(k|0)){AN(0,0,1717128,(u=i,i=i+24|0,c[u>>2]=1690072,c[u+8>>2]=1670432,c[u+16>>2]=248,u)|0);i=u;cw()}if((((r|0)>-1?r:-r|0)|0)<(k|0)){AN(0,0,1717128,(u=i,i=i+24|0,c[u>>2]=1655472,c[u+8>>2]=1670432,c[u+16>>2]=249,u)|0);i=u;cw()}if((l|0)>0){v=s;w=q;x=l}else{break}while(1){NN(w|0,v|0,k)|0;y=x-1|0;if((y|0)>0){v=v+t|0;w=w+r|0;x=y}else{break}}}}while(0);x=c[e+4>>2]|0;r=c[g+4>>2]|0;NN(x|0,r|0,1024)|0;i=m;return}r=a[p+4|0]|0;if(r<<24>>24==0){i=m;return}x=r&255;r=0;w=0;do{t=(b[p+8+(w<<1)>>1]&3)+1|0;r=(r|0)>(t|0)?r:t;w=w+1|0;}while((w|0)<(x|0));if((r|0)<=0){i=m;return}x=j>>>0>320;w=n|0;t=o|0;v=(k|0)<0;q=k-1|0;k=-l|0;s=p+6|0;p=0;L19297:while(1){y=BP(j)|0;if(x){z=14260;break}A=y+7|0;if((a[A]&8)!=0){z=14260;break}AB(w,t,y);B=c[n+(p<<2)>>2]|0;if((y|0)==0|v){z=14260;break}if(((c[o+(p<<2)>>2]|0)-1|0)>>>0<2){C=d[y+5|0]|0}else{C=0}y=q+(1<<C)>>C;if((y|0)!=0){if((2147483647/(y|0)|0|0)<(B|0)){z=14260;break}}D=ac(y,B)|0;if((a[A]&4)==0){E=D}else{E=D+7>>3}if((E|0)<0){z=14260;break}if((p-1|0)>>>0<2){F=-(k>>(d[s]|0))|0}else{F=l}D=c[e+(p<<2)>>2]|0;A=c[f+(p<<2)>>2]|0;B=c[g+(p<<2)>>2]|0;y=c[h+(p<<2)>>2]|0;do{if(!((D|0)==0|(B|0)==0)){if((((y|0)>-1?y:-y|0)|0)<(E|0)){z=14265;break L19297}if((((A|0)>-1?A:-A|0)|0)<(E|0)){z=14268;break L19297}if((F|0)>0){G=B;H=D;I=F}else{break}while(1){NN(H|0,G|0,E)|0;J=I-1|0;if((J|0)>0){G=G+y|0;H=H+A|0;I=J}else{break}}}}while(0);A=p+1|0;if((A|0)<(r|0)){p=A}else{z=14276;break}}if((z|0)==14268){AN(0,0,1717128,(u=i,i=i+24|0,c[u>>2]=1655472,c[u+8>>2]=1670432,c[u+16>>2]=249,u)|0);i=u;cw()}else if((z|0)==14260){AN(0,16,1636472,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;i=m;return}else if((z|0)==14276){i=m;return}else if((z|0)==14265){AN(0,0,1717128,(u=i,i=i+24|0,c[u>>2]=1690072,c[u+8>>2]=1670432,c[u+16>>2]=248,u)|0);i=u;cw()}}function AH(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0;j=i;i=i+16|0;k=j|0;l=k;c[k>>2]=172960;c[k+4>>2]=0;c[k+8>>2]=0;do{if((f|0)>0&(g|0)>0){k=N2(g+128|0,0,f+128|0,0)|0;m=G;n=0;if(!(m>>>0<n>>>0|m>>>0==n>>>0&k>>>0<268435455>>>0)){break}k=AC(b,e,f)|0;if((k|0)<0){o=k;i=j;return o|0}k=h-1|0;n=-h|0;c[b>>2]=k+(c[b>>2]|0)&n;m=b+4|0;c[m>>2]=k+(c[m>>2]|0)&n;m=b+8|0;c[m>>2]=k+(c[m>>2]|0)&n;m=b+12|0;c[m>>2]=k+(c[m>>2]|0)&n;n=AD(a,e,f,0,b)|0;if((n|0)<0){o=n;i=j;return o|0}o=AD(a,e,g,d,b)|0;i=j;return o|0}}while(0);AN(l,16,1634824,(l=i,i=i+16|0,c[l>>2]=f,c[l+8>>2]=g,l)|0);i=l;o=-22;i=j;return o|0}function AI(a){a=a|0;return c[c[a>>2]>>2]|0}function AJ(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0;b=i;i=i+3072|0;j=b|0;AK(a,d,e,j|0,h,0);bc(f|0,g|0,1634568,(g=i,i=i+24|0,c[g>>2]=j,c[g+8>>2]=j+1024,c[g+16>>2]=j+2048,g)|0)|0;i=g;i=b;return}function AK(b,d,e,f,g,h){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;j=i;if((b|0)==0){k=0}else{k=c[b>>2]|0}l=f+2048|0;a[l]=0;m=f+1024|0;a[m]=0;n=f|0;a[n]=0;o=(h|0)!=0;if(o){c[h+4>>2]=16;c[h>>2]=16}L19350:do{if(!((c[g>>2]|0)==0|(k|0)==0)){p=c[k+20>>2]|0;do{if((p|0)!=0){q=c[b+p>>2]|0;if((q|0)==0){break}r=c[q>>2]|0;if((r|0)==0){break}s=q;t=c1[c[r+4>>2]&255](s)|0;bc(n|0,1024,1725920,(u=i,i=i+16|0,c[u>>2]=t,c[u+8>>2]=q,u)|0)|0;i=u;if(!o){t=c1[c[k+4>>2]&255](b)|0;bc(m|0,1024,1725920,(u=i,i=i+16|0,c[u>>2]=t,c[u+8>>2]=b,u)|0)|0;i=u;break L19350}t=c[q>>2]|0;do{if((t|0)==0){v=16}else{q=c[t+12>>2]|0;if((q&252)>>>0<100|(q|0)<3357440){v=16;break}q=c[t+32>>2]|0;if(q>>>0>10){v=16;break}r=c[t+36>>2]|0;if((r|0)==0){v=q+16|0;break}else{v=(c1[r&255](s)|0)+16|0;break}}}while(0);c[h>>2]=v}}while(0);p=c1[c[k+4>>2]&255](b)|0;bc(m|0,1024,1725920,(u=i,i=i+16|0,c[u>>2]=p,c[u+8>>2]=b,u)|0)|0;i=u;if(!o){break}p=c[b>>2]|0;do{if((p|0)==0){w=16}else{s=c[p+12>>2]|0;if((s&252)>>>0<100|(s|0)<3357440){w=16;break}s=c[p+32>>2]|0;if(s>>>0>10){w=16;break}t=c[p+36>>2]|0;if((t|0)==0){w=s+16|0;break}else{w=(c1[t&255](b)|0)+16|0;break}}}while(0);c[h+4>>2]=w}}while(0);cs(l|0,1024,d|0,e|0)|0;do{if((a[n]|0)==0){if((a[m]|0)!=0){x=14320;break}if((a[l]|0)!=0){x=14321;break}i=j;return}else{x=14320}}while(0);if((x|0)==14320){if((a[l]|0)==0){y=0}else{x=14321}}if((x|0)==14321){y=(a[(NL(l|0)|0)-1+(f+2048)|0]|0)==10|0}c[g>>2]=y;i=j;return}function AL(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,p=0,q=0;g=i;i=i+4104|0;h=g|0;j=g+4096|0;if((c[86622]|0)<(d|0)){i=g;return}k=j|0;AK(b,e,f,h|0,346496,k);f=g+3072|0;e=h|0;b=h+1024|0;l=h+2048|0;bc(f|0,1024,1634568,(h=i,i=i+24|0,c[h>>2]=e,c[h+8>>2]=b,c[h+16>>2]=l,h)|0)|0;i=h;m=c[753906]|0;if((m|0)==0){n=(b8(2)|0)!=0?1:-1;c[753906]=n;p=n}else{p=m}do{if((c[86624]|0)!=0){if((c[556292]&1|0)==0){break}m=(a8(f|0,3014600)|0)!=0;if(m|(a[f]|0)==0){break}m=(c[753908]|0)+1|0;c[753908]=m;if((p|0)!=1){i=g;return}n=c[o>>2]|0;cx(n|0,1717064,(h=i,i=i+8|0,c[h>>2]=m,h)|0)|0;i=h;i=g;return}}while(0);p=c[753908]|0;if((p|0)>0){m=c[o>>2]|0;cx(m|0,1689968,(h=i,i=i+8|0,c[h>>2]=p,h)|0)|0;i=h;c[753908]=0}NQ(3014600,f|0)|0;f=a[e]|0;if(f<<24>>24!=0){h=e;p=f;do{if((p&255)<8|(p-14&255)<18){a[h]=63}h=h+1|0;p=a[h]|0;}while(p<<24>>24!=0)}AM(c[k>>2]|0,e);e=a[b]|0;if(e<<24>>24!=0){k=b;p=e;do{if((p&255)<8|(p-14&255)<18){a[k]=63}k=k+1|0;p=a[k]|0;}while(p<<24>>24!=0)}AM(c[j+4>>2]|0,b);b=a[l]|0;if(b<<24>>24!=0){j=l;p=b;do{if((p&255)<8|(p-14&255)<18){a[j]=63}j=j+1|0;p=a[j]|0;}while(p<<24>>24!=0)}p=d>>3;if((p|0)<0){q=0}else{q=(p|0)>6?6:p}AM(q,l);i=g;return}function AM(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0;d=i;e=c[10204]|0;do{if((e|0)<0){do{if((ca(1636392)|0)==0){if((ca(1629248)|0)!=0){f=0;break}if((ca(1623088)|0)!=0){if((b8(2)|0)!=0){f=1;break}}f=(ca(1617488)|0)!=0|0}else{f=0}}while(0);c[10204]=f;if((ca(1612608)|0)==0){g=f;break}h=f<<8;c[10204]=h;g=h}else{g=e}}while(0);if((g|0)==256){e=c[o>>2]|0;f=c[265256+(a<<2)>>2]|0;h=f>>>16&255;j=f>>>8&255;cx(e|0,1735152,(k=i,i=i+16|0,c[k>>2]=h,c[k+8>>2]=j,k)|0)|0;i=k;l=e}else if((g|0)==1){g=c[o>>2]|0;e=c[265256+(a<<2)>>2]|0;cx(g|0,1739624,(k=i,i=i+16|0,c[k>>2]=e>>>4&15,c[k+8>>2]=e&15,k)|0)|0;i=k;l=g}else{l=c[o>>2]|0}aZ(b|0,l|0)|0;if((c[10204]|0)==0){i=d;return}aW(1730264,4,1,l|0)|0;i=d;return}function AN(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;f=i;i=i+16|0;g=f|0;do{if((a|0)==0){h=g;c[h>>2]=e;c[h+4>>2]=0;j=b;k=g|0;l=h}else{h=c[a>>2]|0;m=g|0;n=g;c[n>>2]=e;c[n+4>>2]=0;if((h|0)==0){j=b;k=m;l=n;break}if((c[h+12>>2]|0)<=3280641){j=b;k=m;l=n;break}o=c[h+16>>2]|0;if(!((o|0)!=0&(b|0)>7)){j=b;k=m;l=n;break}j=(c[a+o>>2]|0)+b|0;k=m;l=n}}while(0);l=c[86626]|0;if((l|0)==0){i=f;return}dq[l&127](a,j,d,k);i=f;return}function AO(){return c[86622]|0}function AP(a){a=a|0;c[86622]=a;return}function AQ(a){a=a|0;c[556292]=a;return}function AR(a){a=a|0;c[86626]=a;return}function AS(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;i=i+16|0;f=e|0;g=f;c[g>>2]=d;c[g+4>>2]=0;g=c[86626]|0;if((g|0)!=0){dq[g&127](a,24,b,f|0)}AN(a,24,1670240,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;AN(a,24,1655288,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;i=e;return}function AT(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;i=i+16|0;f=e|0;g=f;c[g>>2]=d;c[g+4>>2]=0;g=c[86626]|0;if((g|0)!=0){dq[g&127](a,24,b,f|0)}AN(a,24,1670240,(a=i,i=i+1|0,i=i+7&-8,c[a>>2]=0,a)|0);i=a;i=e;return}function AU(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0;if((c|0)==0&(d|0)==0){e=b;f=a;return(G=e,f)|0}else{g=b;h=a;i=d;j=c}while(1){c=N1(h,g,j,i)|0;d=G;if((c|0)==0&(d|0)==0){e=i;f=j;break}else{g=i;h=j;i=d;j=c}}return(G=e,f)|0}function AV(a,b,c,d,e,f,g){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;do{if((g&8192|0)==0){h=g}else{i=NU(a,b,1,-2147483648)|0;j=G;k=0;if(j>>>0<k>>>0|j>>>0==k>>>0&i>>>0<2>>>0){l=b;m=a;return(G=l,m)|0}else{h=g-8192|0;break}}}while(0);g=0;if(((b|0)<(g|0)|(b|0)==(g|0)&a>>>0<0>>>0)&((a|0)!=0|(b|0)!=(-2147483648|0))){g=NV(0,0,a,b)|0;i=AV(g,G,c,d,e,f,h>>>1&1^h)|0;g=NV(0,0,i,G)|0;return(G=G,g)|0}if((h|0)==5){g=N0(e,f,2,0)|0;n=G;o=g}else{g=(h&1|0)==0;h=NU(e,f,-1,-1)|0;n=g?0:G;o=g?0:h}h=0;g=0;if(((d|0)<(h|0)|(d|0)==(h|0)&c>>>0<-2147483648>>>0)&((f|0)<(g|0)|(f|0)==(g|0)&e>>>0<-2147483648>>>0)){g=0;if((b|0)<(g|0)|(b|0)==(g|0)&a>>>0<-2147483648>>>0){g=N2(c,d,a,b)|0;h=NU(o,n,g,G)|0;g=N0(h,G,e,f)|0;l=G;m=g;return(G=l,m)|0}else{g=N0(a,b,e,f)|0;h=N2(g,G,c,d)|0;g=G;i=N1(a,b,e,f)|0;k=N2(i,G,c,d)|0;i=NU(k,G,o,n)|0;k=N0(i,G,e,f)|0;i=NU(k,G,h,g)|0;l=G;m=i;return(G=l,m)|0}}else{i=a|0;a=b&0;g=b;h=(b|0)<0|0?-1:0;b=c|0;c=d&0;k=d;j=(d|0)<0|0?-1:0;d=N2(k,j,i,a)|0;p=G;q=N2(b,c,g,h)|0;r=NU(d,p,q,G)|0;q=G;p=N2(b,c,i,a)|0;a=NR(p|0,G|0,0,r|0)|0;p=H;i=G;c=N2(k,j,g,h)|0;h=NU(q,0,c,G)|0;c=NU(h,G,p&1,0)|0;p=G;h=NR(a|0,i|0,o|0,n|0)|0;n=h;h=G;o=NU(c,p,H&1,0)|0;p=G;c=o;o=q;q=r;r=63;while(1){i=NX(n|0,h|0,r|0)|0;a=c<<1|0>>>31|i&1;i=p<<1|c>>>31|G&0;g=q<<1|0>>>31;j=o<<1|q>>>31;if(i>>>0<f>>>0|i>>>0==f>>>0&a>>>0<e>>>0){s=j;t=g;u=i;v=a}else{k=NV(a,i,e,f)|0;s=j|0;t=g|1;u=G;v=k}if((r|0)>0){p=u;c=v;o=s;q=t;r=r-1|0}else{l=s;m=t;break}}return(G=l,m)|0}return 0}function AW(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;var g=0;g=AV(a,b,c,d,e,f,5)|0;return(G=G,g)|0}function AX(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0;g=i;h=d;d=i;i=i+8|0;c[d>>2]=c[h>>2];c[d+4>>2]=c[h+4>>2];h=e;e=i;i=i+8|0;c[e>>2]=c[h>>2];c[e+4>>2]=c[h+4>>2];h=c[d>>2]|0;j=c[e+4>>2]|0;k=N2(j,(j|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=G;j=c[e>>2]|0;e=c[d+4>>2]|0;d=N2(e,(e|0)<0|0?-1:0,j,(j|0)<0|0?-1:0)|0;j=AV(a,b,k,h,d,G,f)|0;f=G;i=g;return(G=f,j)|0}function AY(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0;f=i;g=d;d=i;i=i+8|0;c[d>>2]=c[g>>2];c[d+4>>2]=c[g+4>>2];g=e;e=i;i=i+8|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];g=c[d>>2]|0;h=c[e+4>>2]|0;j=N2(h,(h|0)<0|0?-1:0,g,(g|0)<0|0?-1:0)|0;g=G;h=c[e>>2]|0;e=c[d+4>>2]|0;d=N2(e,(e|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=AV(a,b,j,g,d,G,5)|0;d=G;i=f;return(G=d,h)|0}function AZ(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;j=d;d=i;i=i+8|0;c[d>>2]=c[j>>2];c[d+4>>2]=c[j+4>>2];j=g;g=i;i=i+8|0;c[g>>2]=c[j>>2];c[g+4>>2]=c[j+4>>2];j=c[d>>2]|0;k=c[g+4>>2]|0;l=N2(k,(k|0)<0|0?-1:0,j,(j|0)<0|0?-1:0)|0;j=G;k=c[g>>2]|0;g=c[d+4>>2]|0;d=N2(g,(g|0)<0|0?-1:0,k,(k|0)<0|0?-1:0)|0;k=G;g=-1;m=(b|0)>(g|0)|(b|0)==(g|0)&a>>>0>-1>>>0;g=NV(0,0,a,b)|0;n=m?b:G;o=-1;p=(f|0)>(o|0)|(f|0)==(o|0)&e>>>0>-1>>>0;o=NV(0,0,e,f)|0;q=(p?f:G)|n|j|k;n=0;if((q|0)<(n|0)|(q|0)==(n|0)&((p?e:o)|(m?a:g)|l|d)>>>0<-2147483648>>>0){g=N2(l,j,a,b)|0;m=G;o=N2(d,k,e,f)|0;p=G;r=(((m|0)>(p|0)|(m|0)==(p|0)&g>>>0>o>>>0)&1)-(((m|0)<(p|0)|(m|0)==(p|0)&g>>>0<o>>>0)&1)|0;i=h;return r|0}o=AV(a,b,l,j,d,k,2)|0;g=G;if((g|0)<(f|0)|(g|0)==(f|0)&o>>>0<e>>>0){r=-1;i=h;return r|0}o=AV(e,f,d,k,l,j,2)|0;j=G;r=((j|0)<(b|0)|(j|0)==(b|0)&o>>>0<a>>>0)&1;i=h;return r|0}function A_(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;var g=0;g=NV(a,b,c,d)|0;d=G;c=NU(e,f,-1,-1)|0;b=c&g;g=G&d;d=f>>>1|0<<31;c=g>>>0>d>>>0|g>>>0==d>>>0&b>>>0>(e>>>1|f<<31)>>>0;d=NV(b,g,c?e:0,c?f:0)|0;return(G=G,d)|0}function A$(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0;j=i;k=a;a=i;i=i+8|0;c[a>>2]=c[k>>2];c[a+4>>2]=c[k+4>>2];k=e;e=i;i=i+8|0;c[e>>2]=c[k>>2];c[e+4>>2]=c[k+4>>2];k=h;h=i;i=i+8|0;c[h>>2]=c[k>>2];c[h+4>>2]=c[k+4>>2];if((b|0)==0&(d|0)==(-2147483648|0)){AN(0,0,1634376,(l=i,i=i+24|0,c[l>>2]=1716976,c[l+8>>2]=1689944,c[l+16>>2]=155,l)|0);i=l;cw();return 0}if((f|0)<=-1){AN(0,0,1634376,(l=i,i=i+24|0,c[l>>2]=1670224,c[l+8>>2]=1689944,c[l+16>>2]=156,l)|0);i=l;cw();return 0}l=c[g>>2]|0;k=c[g+4>>2]|0;m=c[a>>2]|0;do{if((l|0)==0&(k|0)==(-2147483648|0)|(f|0)==0){n=c[a+4>>2]|0;o=c[h+4>>2]|0;p=c[h>>2]|0}else{q=m;r=(m|0)<0|0?-1:0;s=c[h+4>>2]|0;t=s;u=(s|0)<0|0?-1:0;v=N2(t,u,q,r)|0;w=G;x=c[h>>2]|0;y=x;z=(x|0)<0|0?-1:0;A=c[a+4>>2]|0;B=A;C=(A|0)<0|0?-1:0;D=N2(B,C,y,z)|0;E=G;if(!((w|0)>(E|0)|(w|0)==(E|0)&v>>>0>D>>>0)){n=A;o=s;p=x;break}D=b<<1|0>>>31;v=d<<1|b>>>31;E=NU(D,v,-1,-1)|0;w=G;F=c[e+4>>2]|0;H=F;I=(F|0)<0|0?-1:0;F=N2(H,I,q,r)|0;r=G;q=c[e>>2]|0;J=q;K=(q|0)<0|0?-1:0;q=N2(J,K,B,C)|0;C=G;B=AV(E,w,F,r,q,C,2)|0;w=G;E=B>>>1|w<<31;B=w>>1|((w|0)<0|0?-1:0)<<31;w=AV(D|1,v|0,F,r,q,C,3)|0;C=NU(w,G,1,0)|0;w=G;q=C>>>1|w<<31;C=w>>1|((w|0)<0|0?-1:0)<<31;w=NV(E<<1|0>>>31,B<<1|E>>>31,q,C)|0;r=G;if((k|0)<(r|0)|(k|0)==(r|0)&l>>>0<w>>>0){n=A;o=s;p=x;break}w=NV(q<<1|0>>>31,C<<1|q>>>31,E,B)|0;r=G;if((k|0)>(r|0)|(k|0)==(r|0)&l>>>0>w>>>0){n=A;o=s;p=x;break}if((k|0)<(B|0)|(k|0)==(B|0)&l>>>0<E>>>0){L=B;M=E}else{E=(k|0)>(C|0)|(k|0)==(C|0)&l>>>0>q>>>0;L=E?C:k;M=E?q:l}q=NU(M,L,f,(f|0)<0|0?-1:0)|0;c[g>>2]=q;c[g+4>>2]=G;q=N2(t,u,J,K)|0;K=G;J=N2(H,I,y,z)|0;z=AV(M,L,q,K,J,G,5)|0;N=G;O=z;i=j;return(G=N,O)|0}}while(0);L=m;M=(m|0)<0|0?-1:0;m=c[e+4>>2]|0;l=N2(m,(m|0)<0|0?-1:0,L,M)|0;m=G;k=c[e>>2]|0;e=n;a=(n|0)<0|0?-1:0;n=N2(e,a,k,(k|0)<0|0?-1:0)|0;k=AV(b,d,l,m,n,G,5)|0;n=NU(k,G,f,(f|0)<0|0?-1:0)|0;c[g>>2]=n;c[g+4>>2]=G;g=N2(o,(o|0)<0|0?-1:0,L,M)|0;M=G;L=N2(e,a,p,(p|0)<0|0?-1:0)|0;p=AV(b,d,g,M,L,G,5)|0;N=G;O=p;i=j;return(G=N,O)|0}function A0(a){a=a|0;c[32444]=a;return}function A1(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=i;i=i+16|0;d=b|0;e=b+8|0;c[e>>2]=0;f=c[32444]|0;if((f-32|0)>>>0<a>>>0){g=0;i=b;return g|0}if((a|0)!=0){if((NI(e,16,a)|0)==0){g=c[e>>2]|0;i=b;return g|0}else{c[e>>2]=0;g=0;i=b;return g|0}}c[d>>2]=0;do{if((f|0)==32){h=0}else{if((NI(d,16,1)|0)==0){h=c[d>>2]|0;break}else{c[d>>2]=0;h=0;break}}}while(0);c[e>>2]=h;g=h;i=b;return g|0}function A2(a,b){a=a|0;b=b|0;var d=0;if(((c[32444]|0)-32|0)>>>0<b>>>0){d=0;return d|0}d=NF(a,((b|0)==0)+b|0)|0;return d|0}function A3(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=ac(d,b)|0;do{if((d|b)>>>0>65535&(d|0)!=0){if(((e>>>0)/(d>>>0)|0|0)==(b|0)){break}ND(a);f=0;return f|0}}while(0);if(((c[32444]|0)-32|0)>>>0<e>>>0){g=0}else{g=NF(a,((e|0)==0)+e|0)|0}if((g|0)!=0|(e|0)==0){f=g;return f|0}ND(a);f=0;return f|0}function A4(a){a=a|0;ND(a);return}function A5(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;e=a;a=c[e>>2]|0;f=ac(d,b)|0;do{if((d|b)>>>0>65535&(d|0)!=0){if(((f>>>0)/(d>>>0)|0|0)==(b|0)){g=14484;break}ND(a);g=14488}else{g=14484}}while(0);do{if((g|0)==14484){if(((c[32444]|0)-32|0)>>>0<f>>>0){h=0}else{h=NF(a,((f|0)==0)+f|0)|0}if((h|0)!=0|(f|0)==0){c[e>>2]=h;if((h|0)==0){g=14490;break}else{break}}else{ND(a);g=14488;break}}}while(0);if((g|0)==14488){c[e>>2]=0;g=14490}do{if((g|0)==14490){if((b|0)==0|(d|0)==0){i=-12}else{break}return i|0}}while(0);i=0;return i|0}function A6(a){a=a|0;var b=0;b=a;ND(c[b>>2]|0);c[b>>2]=0;return}function A7(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0;b=i;i=i+16|0;d=b|0;e=b+8|0;c[e>>2]=0;f=c[32444]|0;if((f-32|0)>>>0<a>>>0){g=0;i=b;return g|0}do{if((a|0)==0){c[d>>2]=0;do{if((f|0)==32){h=0}else{if((NI(d,16,1)|0)==0){h=c[d>>2]|0;break}else{c[d>>2]=0;h=0;break}}}while(0);c[e>>2]=h;j=h}else{if((NI(e,16,a)|0)==0){j=c[e>>2]|0;break}c[e>>2]=0;g=0;i=b;return g|0}}while(0);if((j|0)==0){g=0;i=b;return g|0}NM(j|0,0,a|0);g=j;i=b;return g|0}function A8(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0;d=i;i=i+16|0;e=d|0;f=d+8|0;L19625:do{if((b|0)==0){g=0}else{if((2147483647/(b>>>0)|0)>>>0<=a>>>0){g=0;break}h=ac(b,a)|0;c[f>>2]=0;j=c[32444]|0;if((j-32|0)>>>0<h>>>0){g=0;break}do{if((h|0)==0){c[e>>2]=0;do{if((j|0)==32){k=0}else{if((NI(e,16,1)|0)==0){k=c[e>>2]|0;break}else{c[e>>2]=0;k=0;break}}}while(0);c[f>>2]=k;l=k}else{if((NI(f,16,h)|0)==0){l=c[f>>2]|0;break}else{c[f>>2]=0;g=0;break L19625}}}while(0);if((l|0)==0){g=0;break}NM(l|0,0,h|0);g=l}}while(0);i=d;return g|0}function A9(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0;b=i;i=i+16|0;d=b|0;e=b+8|0;L19645:do{if((a|0)==0){f=0}else{g=(NL(a|0)|0)+1|0;c[e>>2]=0;h=c[32444]|0;if((h-32|0)>>>0<g>>>0){f=0;break}do{if((g|0)==0){c[d>>2]=0;do{if((h|0)==32){j=0}else{if((NI(d,16,1)|0)==0){j=c[d>>2]|0;break}else{c[d>>2]=0;j=0;break}}}while(0);c[e>>2]=j;k=j}else{if((NI(e,16,g)|0)==0){k=c[e>>2]|0;break}else{c[e>>2]=0;f=0;break L19645}}}while(0);if((k|0)==0){f=0;break}NN(k|0,a|0,g)|0;f=k}}while(0);i=b;return f|0}function Ba(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;e=c[b>>2]|0;f=a;g=c[f>>2]|0;h=g;L19664:do{if((e-1&e|0)==0){do{if((e|0)==0){i=4;j=14546}else{if(e>>>0>268435455){k=h;break}i=e<<3;j=14546}}while(0);do{if((j|0)==14546){if(((c[32444]|0)-32|0)>>>0<i>>>0){k=h;break}l=NF(g,(i|0)==0|i)|0;m=l;if((l|0)==0){k=c[a>>2]|0;break}else{c[f>>2]=m;n=m;break L19664}}}while(0);ND(k);c[a>>2]=0;o=0;c[b>>2]=o;return}else{n=g}}while(0);c[n+(e<<2)>>2]=d;o=e+1|0;c[b>>2]=o;return}function Bb(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=c[a>>2]|0;a=(b|0)!=0;do{if(!(a|(d|0)==0)){e=c[d+8>>2]|0;if((e|0)==0){break}if((c[e>>2]|0)==0){break}else{f=e}return f|0}}while(0);do{if(a){d=b+48|0;if((c[d>>2]|0)==0){break}else{f=d}return f|0}}while(0);f=0;return f|0}function Bc(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;return Be(a,b,c,d,e,0)|0}function Bd(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;g=i;i=i+24|0;h=g|0;j=g+8|0;k=g+16|0;l=Be(b,d,0,0,f,h)|0;f=c[h>>2]|0;if((l|0)==0|(f|0)==0){m=-1414549496;i=g;return m|0}h=(e|0)!=0;d=c[l+12>>2]|0;do{if(!h){if((d|0)==5|(d|0)==1346784596|(d|0)==1397116244|(d|0)==1397316165|(d|0)==1448231252|(d|0)==1146442272|(d|0)==1129270354){break}else{m=-22}i=g;return m|0}}while(0);n=c[l+8>>2]|0;o=f+n|0;if((d|0)==1397116244){do{if(h){if((a8(e|0,1634064)|0)==0){p=-1;break}q=B3(e)|0;if((q|0)!=-1){p=q;break}q=aU(e|0,k|0,0)|0;if(!((a[c[k>>2]|0]|0)!=0|q>>>0>9)){p=q;break}AN(b,16,1655128,(r=i,i=i+8|0,c[r>>2]=e,r)|0);i=r;m=-22;i=g;return m|0}else{p=-1}}while(0);c[o>>2]=p;m=0;i=g;return m|0}else if((d|0)==1346784596){do{if(h){if((a8(e|0,1634064)|0)==0){s=-1;break}p=BM(e)|0;if((p|0)!=-1){s=p;break}p=aU(e|0,j|0,0)|0;if(!((a[c[j>>2]|0]|0)!=0|p>>>0>320)){s=p;break}AN(b,16,1670128,(r=i,i=i+8|0,c[r>>2]=e,r)|0);i=r;m=-22;i=g;return m|0}else{s=-1}}while(0);c[o>>2]=s;m=0;i=g;return m|0}else if((d|0)==1129270354){if(!h){m=0;i=g;return m|0}s=BE(o,e,-1,b)|0;if((s|0)>=0){m=s;i=g;return m|0}AN(b,16,1629192,(r=i,i=i+8|0,c[r>>2]=e,r)|0);i=r;m=s;i=g;return m|0}else if((d|0)==0|(d|0)==1|(d|0)==2|(d|0)==4|(d|0)==3|(d|0)==6){m=Bf(b,f,l,e,o)|0;i=g;return m|0}else if((d|0)==1397316165){do{if(h){if((a8(e|0,1634064)|0)==0){break}l=BC(o,f+(n+4)|0,e)|0;if((l|0)>=0){m=l;i=g;return m|0}AN(b,16,1716872,(r=i,i=i+8|0,c[r>>2]=e,r)|0);i=r;m=l;i=g;return m|0}}while(0);c[f+(n+4)>>2]=0;c[o>>2]=0;m=0;i=g;return m|0}else if((d|0)==5){A6(o);c[o>>2]=A9(e)|0;m=0;i=g;return m|0}else if((d|0)==1146442272){l=o;if(!h){c[l>>2]=0;c[l+4>>2]=0;m=0;i=g;return m|0}s=BI(l,e,1)|0;if((s|0)>=0){m=s;i=g;return m|0}AN(b,16,1636320,(r=i,i=i+8|0,c[r>>2]=e,r)|0);i=r;m=s;i=g;return m|0}else if((d|0)==1448231252){do{if(h){s=BD(o,e)|0;if((s|0)<0){t=s;break}else{m=s}i=g;return m|0}else{t=-22}}while(0);AN(b,16,1689784,(r=i,i=i+8|0,c[r>>2]=e,r)|0);i=r;m=t;i=g;return m|0}else if((d|0)==7){d=o;t=f+(n+4)|0;n=NL(e|0)|0;A6(o);c[t>>2]=0;if((n&1|0)!=0){m=-22;i=g;return m|0}o=(n|0)/2|0;n=A1(o)|0;f=a[e]|0;L19761:do{if(f<<24>>24!=0){h=n;s=e;l=f;while(1){j=s+1|0;p=l<<24>>24;do{if((l-48&255)<10){u=p-48|0}else{if((l-97&255)<6){u=p-87|0;break}else{u=(l-65&255)<6?p-55|0:-1;break}}}while(0);p=s+2|0;k=a[j]|0;q=k<<24>>24;do{if((k-48&255)<10){v=q-48|0}else{if((k-97&255)<6){v=q-87|0;break}else{v=(k-65&255)<6?q-55|0:-1;break}}}while(0);if((v|u|0)<0){break}a[h]=(v|u<<4)&255;q=a[p]|0;if(q<<24>>24==0){break L19761}else{h=h+1|0;s=p;l=q}}A4(n);m=-22;i=g;return m|0}}while(0);c[d>>2]=n;c[t>>2]=o;m=0;i=g;return m|0}else{AN(b,16,1623040,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;m=-22;i=g;return m|0}return 0}function Be(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;h=i;i=i+8|0;j=h|0;if((a|0)==0){k=0;i=h;return k|0}l=a;m=c[l>>2]|0;L19787:do{if((f&1|0)!=0){if((f&2|0)==0){n=0;o=m;while(1){p=c[o+24>>2]|0;if((p|0)==0){break L19787}q=dg[p&255](a,n)|0;if((q|0)==0){break L19787}p=Be(q,b,d,e,f,g)|0;if((p|0)!=0){k=p;break}n=q;o=c[l>>2]|0}i=h;return k|0}c[j>>2]=0;o=m+28|0;n=j;while(1){q=c[o>>2]|0;if((q|0)==0){r=14645;break}p=c1[q&255](c[j>>2]|0)|0;c[j>>2]=p;if((p|0)==0){break L19787}p=Be(n,b,d,e,f,0)|0;if((p|0)!=0){k=p;r=14677;break}}if((r|0)==14645){c[j>>2]=0;break}else if((r|0)==14677){i=h;return k|0}}}while(0);j=c[l>>2]|0;l=(j|0)==0;m=(d|0)==0;n=j+8|0;j=0;while(1){o=(j|0)!=0;do{if(o|l){r=14656}else{p=c[n>>2]|0;if((p|0)==0){r=14656;break}q=c[p>>2]|0;if((q|0)==0){r=14656}else{s=p;t=q}}}while(0);if((r|0)==14656){r=0;if(!o){k=0;r=14674;break}q=j+48|0;p=c[q>>2]|0;if((p|0)==0|(q|0)==0){k=0;r=14673;break}else{s=q;t=p}}if((a8(t|0,b|0)|0)!=0){j=s;continue}if((c[s+40>>2]&e|0)!=(e|0)){j=s;continue}p=(c[s+12>>2]|0)==128;if(m){if(p){j=s;continue}else{r=14665;break}}if(!p){j=s;continue}p=c[s+44>>2]|0;if((p|0)==0){j=s;continue}if((a8(p|0,d|0)|0)==0){r=14665;break}else{j=s}}if((r|0)==14674){i=h;return k|0}else if((r|0)==14673){i=h;return k|0}else if((r|0)==14665){if((g|0)==0){k=s;i=h;return k|0}if((f&2|0)==0){c[g>>2]=a;k=s;i=h;return k|0}else{c[g>>2]=0;k=s;i=h;return k|0}}return 0}function Bf(b,d,e,f,j){b=b|0;d=d|0;e=e|0;f=f|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,S=0,T=0,U=0.0,V=0,W=0,X=0,Y=0.0,Z=0.0,_=0.0,$=0;k=i;i=i+264|0;l=k|0;m=k+256|0;n=e+12|0;o=l|0;p=e+44|0;q=(d|0)==0;r=e+16|0;s=e+16|0;t=e+32|0;u=e+24|0;v=j;w=j;x=j;y=j;z=j+4|0;A=d;d=f;B=0;C=a[f]|0;L19834:while(1){do{if((C<<24>>24|0)==43|(C<<24>>24|0)==45){if((c[n>>2]|0)==0){D=d+1|0;E=C<<24>>24;F=0;break}if((B|0)!=0){D=d;E=0;F=0;break}a[o]=C;D=d;E=0;F=1}else{D=d;E=0;F=0}}while(0);f=F;while(1){G=a[D+f|0]|0;if((G<<24>>24|0)==0|(G<<24>>24|0)==43|(G<<24>>24|0)==45){H=f;break}a[l+f|0]=G;G=f+1|0;if(G>>>0<255){f=G}else{H=G;break}}a[l+H|0]=0;f=c[p>>2]|0;L19847:do{if(q){I=14702}else{G=c[A>>2]|0;J=(G|0)==0;K=(f|0)==0;L=G+8|0;G=0;while(1){M=(G|0)!=0;do{if(M|J){I=14692}else{N=c[L>>2]|0;if((N|0)==0){I=14692;break}O=c[N>>2]|0;if((O|0)==0){I=14692}else{P=N;S=O}}}while(0);if((I|0)==14692){I=0;if(!M){I=14702;break L19847}O=G+48|0;N=c[O>>2]|0;if((N|0)==0|(O|0)==0){I=14702;break L19847}else{P=O;S=N}}if((a8(S|0,o|0)|0)!=0){G=P;continue}T=(c[P+12>>2]|0)==128;if(K){if(T){G=P;continue}else{break}}if(!T){G=P;continue}N=c[P+44>>2]|0;if((N|0)==0){G=P;continue}if((a8(N|0,f|0)|0)==0){break}else{G=P}}if((P|0)==0|T^1){I=14702;break}G=P+16|0;h[m>>3]=+((c[G>>2]|0)>>>0)+ +(c[G+4>>2]|0)*4294967296.0}}while(0);do{if((I|0)==14702){I=0;if((a8(o|0,1623984)|0)==0){f=c[n>>2]|0;if((f|0)==2|(f|0)==128|(f|0)==0|(f|0)==1){U=+((c[s>>2]|0)>>>0)+ +(c[s+4>>2]|0)*4294967296.0}else{U=+h[r>>3]}h[m>>3]=U;break}if((a8(o|0,1623232)|0)==0){h[m>>3]=+h[t>>3];break}if((a8(o|0,1622632)|0)==0){h[m>>3]=+h[u>>3];break}if((a8(o|0,1634064)|0)==0){h[m>>3]=0.0;break}if((a8(o|0,1622224)|0)==0){h[m>>3]=-1.0;break}else{V=zV(m,o,264112,264080,0,0,0,0,0,0,b)|0;if((V|0)<0){I=14716;break L19834}else{break}}}}while(0);f=c[n>>2]|0;do{if((f|0)==1146442272|(f|0)==2){W=c[w+4>>2]|0;X=c[w>>2]|0;Y=1.0;Z=1.0;I=14727}else if((f|0)==128){W=0;X=1;Y=+h[r>>3];Z=1.0;I=14727}else if((f|0)==1346784596|(f|0)==1397116244|(f|0)==1){G=c[v>>2]|0;W=(G|0)<0|0?-1:0;X=G;Y=1.0;Z=1.0;I=14727}else if((f|0)==0){G=c[v>>2]|0;K=0;if((E|0)==43){_=+h[m>>3];h[m>>3]=+((~~_>>>0|G)>>>0)+ +((+R(_)>=1.0?_>0.0?(ah(+Q(_/4294967296.0),4294967295.0)|0)>>>0:~~+ab((_- +(~~_>>>0))/4294967296.0)>>>0:0)|K|0)*4294967296.0;break}else if((E|0)==45){_=+h[m>>3];h[m>>3]=+((G&~(~~_>>>0))>>>0)+ +(K&~(+R(_)>=1.0?_>0.0?(ah(+Q(_/4294967296.0),4294967295.0)|0)>>>0:~~+ab((_- +(~~_>>>0))/4294967296.0)>>>0:0)|0)*4294967296.0;break}else{break}}else if((f|0)==4){W=0;X=1;Y=+g[x>>2];Z=1.0;I=14727}else if((f|0)==3){W=0;X=1;Y=+h[y>>3];Z=1.0;I=14727}else if((f|0)==6){K=c[v>>2]|0;W=(K|0)<0|0?-1:0;X=K;Y=1.0;Z=+(c[z>>2]|0);I=14727}else{W=0;X=1;Y=1.0;Z=1.0;I=14727}}while(0);do{if((I|0)==14727){I=0;if((E|0)==43){h[m>>3]=+(B|0)*Y*(+(X>>>0)+ +(W|0)*4294967296.0)/Z+ +h[m>>3];break}else if((E|0)==45){h[m>>3]=+(B|0)*Y*(+(X>>>0)+ +(W|0)*4294967296.0)/Z- +h[m>>3];break}else{break}}}while(0);f=BA(b,e,j,+h[m>>3],1,1,0)|0;if((f|0)<0){$=f;I=14733;break}f=D+H|0;K=a[f]|0;if(K<<24>>24==0){$=0;I=14735;break}else{d=f;B=1;C=K}}if((I|0)==14735){i=k;return $|0}else if((I|0)==14716){AN(b,16,1621888,(b=i,i=i+8|0,c[b>>2]=D,b)|0);i=b;$=V;i=k;return $|0}else if((I|0)==14733){i=k;return $|0}return 0}function Bg(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0;do{if((b|0)==0){f=-22}else{if((c[b+12>>2]|0)!=0){f=-22;break}f=Bf(a,a,b,d,e)|0}}while(0);return f|0}function Bh(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0;g=i;i=i+8|0;h=g|0;j=Be(a,b,0,0,f,h)|0;do{if((j|0)==0){k=-1414549496}else{f=c[h>>2]|0;if((f|0)==0){k=-1414549496;break}k=BA(a,j,f+(c[j+8>>2]|0)|0,1.0,1,d,e)|0}}while(0);i=g;return k|0}function Bi(a,b,d,e){a=a|0;b=b|0;d=+d;e=e|0;var f=0,g=0,h=0,j=0;f=i;i=i+8|0;g=f|0;h=Be(a,b,0,0,e,g)|0;if((h|0)==0){j=-1414549496;i=f;return j|0}e=c[g>>2]|0;if((e|0)==0){j=-1414549496;i=f;return j|0}j=BA(a,h,e+(c[h+8>>2]|0)|0,d,1,1,0)|0;i=f;return j|0}function Bj(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0;g=i;i=i+8|0;h=g|0;j=Be(a,b,0,0,f,h)|0;f=c[h>>2]|0;if((j|0)==0|(f|0)==0){k=-1414549496;i=g;return k|0}if((c[j+12>>2]|0)!=7){k=-22;i=g;return k|0}b=(e|0)!=0;do{if(b){a=A1(e)|0;if((a|0)==0){k=-12;i=g;return k|0}else{l=a;m=c[h>>2]|0;break}}else{l=0;m=f}}while(0);f=c[j+8>>2]|0;j=m+f|0;A4(c[j>>2]|0);c[j>>2]=l;c[m+(f+4)>>2]=e;if(!b){k=0;i=g;return k|0}NN(l|0,d|0,e)|0;k=0;i=g;return k|0}function Bk(a,b,d,e,f,g,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0.0,t=0,u=0.0;k=i;i=i+8|0;l=k|0;m=Be(a,b,0,0,e,l)|0;e=c[a>>2]|0;n=c[l>>2]|0;if((m|0)==0|(n|0)==0){o=-1414549496;i=k;return o|0}if((c[m+12>>2]|0)!=(f|0)){AN(a,16,1625800,(p=i,i=i+16|0,c[p>>2]=b,c[p+8>>2]=g,p)|0);i=p;o=-22;i=k;return o|0}f=c[e+12>>2]|0;if((f|0)!=0&(f|0)<3410788){q=j-1|0;r=-1}else{s=+h[m+24>>3];if(s>-1.0){t=-1}else{t=~~s}s=+h[m+32>>3];u=+(j-1|0);q=~~(s>u?s:u);r=t}if((r|0)>(d|0)|(q|0)<(d|0)){AN(a,16,1624976,(p=i,i=i+40|0,c[p>>2]=d,c[p+8>>2]=b,c[p+16>>2]=g,c[p+24>>2]=r,c[p+32>>2]=q,p)|0);i=p;o=-34;i=k;return o|0}else{c[n+(c[m+8>>2]|0)>>2]=d;o=0;i=k;return o|0}return 0}function Bl(b,e,f,j){b=b|0;e=e|0;f=f|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;k=i;i=i+136|0;l=k|0;m=k+8|0;n=Be(b,e,0,0,f,l)|0;f=c[l>>2]|0;if((n|0)==0|(f|0)==0){o=-1414549496;i=k;return o|0}l=c[n+8>>2]|0;e=n+12|0;do{if((l|0)<1){if((c[e>>2]|0)==128){break}else{o=-1414549496}i=k;return o|0}}while(0);b=f+l|0;p=m|0;a[p]=0;m=c[e>>2]|0;if((m|0)==4){e=bc(p|0,128,1713816,(q=i,i=i+8|0,h[q>>3]=+g[b>>2],q)|0)|0;i=q;r=e}else if((m|0)==2){e=b;s=c[e+4>>2]|0;t=bc(p|0,128,1717664,(q=i,i=i+16|0,c[q>>2]=c[e>>2],c[q+8>>2]=s,q)|0)|0;i=q;r=t}else if((m|0)==1129270354){t=c[f+(l+4)>>2]|0;s=c[f+(l+8)>>2]|0;e=c[f+(l+12)>>2]|0;u=bc(p|0,128,1696296,(q=i,i=i+32|0,c[q>>2]=c[b>>2],c[q+8>>2]=t,c[q+16>>2]=s,c[q+24>>2]=e,q)|0)|0;i=q;r=u}else if((m|0)==0){u=bc(p|0,128,1725848,(q=i,i=i+8|0,c[q>>2]=c[b>>2],q)|0)|0;i=q;r=u}else if((m|0)==1346784596){u=BL(c[b>>2]|0)|0;e=bc(p|0,128,1701496,(q=i,i=i+8|0,c[q>>2]=(u|0)!=0?u:1634064,q)|0)|0;i=q;r=e}else if((m|0)==1){e=bc(p|0,128,1721744,(q=i,i=i+8|0,c[q>>2]=c[b>>2],q)|0)|0;i=q;r=e}else if((m|0)==1397116244){e=B2(c[b>>2]|0)|0;u=bc(p|0,128,1701496,(q=i,i=i+8|0,c[q>>2]=(e|0)!=0?e:1634064,q)|0)|0;i=q;r=u}else if((m|0)==128){u=bc(p|0,128,1713816,(q=i,i=i+8|0,h[q>>3]=+h[n+16>>3],q)|0)|0;i=q;r=u}else if((m|0)==1448231252|(m|0)==6){u=c[f+(l+4)>>2]|0;n=bc(p|0,128,1711216,(q=i,i=i+16|0,c[q>>2]=c[b>>2],c[q+8>>2]=u,q)|0)|0;i=q;r=n}else if((m|0)==3){n=bc(p|0,128,1713816,(q=i,i=i+8|0,h[q>>3]=+h[b>>3],q)|0)|0;i=q;r=n}else if((m|0)==1146442272){n=b;u=c[n>>2]|0;e=c[n+4>>2]|0;n=N0(u,e,-694967296,0)|0;s=G;t=N0(u,e,6e7,0)|0;v=N1(t,G,60,0)|0;t=N0(u,e,1e6,0)|0;w=N1(t,G,60,0)|0;t=N1(u,e,1e6,0)|0;e=bc(p|0,128,1698424,(q=i,i=i+40|0,c[q>>2]=n,c[q+8>>2]=s,c[q+16>>2]=v,c[q+24>>2]=w,c[q+32>>2]=t,q)|0)|0;i=q;r=e}else if((m|0)==7){e=c[f+(l+4)>>2]|0;t=e;w=((e|0)<0|0?-1:0)<<1|t>>>31;v=0;if(w>>>0>v>>>0|w>>>0==v>>>0&(t<<1|0>>>31)>>>0>2147483647>>>0){o=-22;i=k;return o|0}t=A1(e<<1|1)|0;c[j>>2]=t;if((t|0)==0){o=-12;i=k;return o|0}v=c[b>>2]|0;if((e|0)>0){x=0;y=t}else{o=0;i=k;return o|0}while(1){bc(y+(x<<1)|0,3,1707864,(q=i,i=i+8|0,c[q>>2]=d[v+x|0]|0,q)|0)|0;i=q;t=x+1|0;if((t|0)>=(e|0)){o=0;break}x=t;y=c[j>>2]|0}i=k;return o|0}else if((m|0)==1397316165){y=c[f+(l+4)>>2]|0;l=bc(p|0,128,1703944,(q=i,i=i+16|0,c[q>>2]=c[b>>2],c[q+8>>2]=y,q)|0)|0;i=q;r=l}else if((m|0)==5){m=c[b>>2]|0;if((m|0)==0){z=A9(3119144)|0}else{z=A9(m)|0}c[j>>2]=z;o=0;i=k;return o|0}else{o=-22;i=k;return o|0}if(r>>>0>127){o=-22;i=k;return o|0}c[j>>2]=A9(p)|0;o=0;i=k;return o|0}function Bm(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0.0,v=0.0,w=0,x=0,y=0.0;if((a|0)==0){e=-1;f=-1;return(G=e,f)|0}i=c[a>>2]|0;j=(i|0)==0;k=i+8|0;i=0;while(1){l=(i|0)!=0;do{if(l|j){m=14823}else{n=c[k>>2]|0;if((n|0)==0){m=14823;break}o=c[n>>2]|0;if((o|0)==0){m=14823}else{p=n;q=o}}}while(0);if((m|0)==14823){m=0;if(!l){e=-1;f=-1;m=14842;break}o=i+48|0;n=c[o>>2]|0;if((n|0)==0|(o|0)==0){e=-1;f=-1;m=14843;break}else{p=o;q=n}}if((a8(q|0,b|0)|0)!=0){i=p;continue}r=p+12|0;s=c[r>>2]|0;if((s|0)==128){i=p}else{m=14827;break}}if((m|0)==14827){if((p|0)==0){e=-1;f=-1;return(G=e,f)|0}i=c[p+8>>2]|0;b=a+i|0;if((d|0)==0){t=s}else{c[d>>2]=p;t=c[r>>2]|0}if((t|0)==6){r=c[b>>2]|0;u=+(c[a+(i+4)>>2]|0);v=1.0;w=(r|0)<0|0?-1:0;x=r}else if((t|0)==128){u=1.0;v=+h[p+16>>3];w=0;x=1}else if((t|0)==4){u=1.0;v=+g[b>>2];w=0;x=1}else if((t|0)==1346784596|(t|0)==1397116244|(t|0)==1){p=c[b>>2]|0;u=1.0;v=1.0;w=(p|0)<0|0?-1:0;x=p}else if((t|0)==1146442272|(t|0)==2){p=b;u=1.0;v=1.0;w=c[p+4>>2]|0;x=c[p>>2]|0}else if((t|0)==3){u=1.0;v=+h[b>>3];w=0;x=1}else if((t|0)==0){u=1.0;v=1.0;w=0;x=c[b>>2]|0}else{e=-1;f=-1;return(G=e,f)|0}y=v*(+(x>>>0)+ +(w|0)*4294967296.0)/u;e=+R(y)>=1.0?y>0.0?(ah(+Q(y/4294967296.0),4294967295.0)|0)>>>0:~~+ab((y- +(~~y>>>0))/4294967296.0)>>>0:0;f=~~y>>>0;return(G=e,f)|0}else if((m|0)==14842){return(G=e,f)|0}else if((m|0)==14843){return(G=e,f)|0}return 0}function Bn(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,j=0,k=0,l=0,m=0.0,n=0.0,o=0,p=0,q=0,r=0.0;f=i;i=i+8|0;j=f|0;k=Be(a,b,0,0,d,j)|0;if((k|0)==0){l=-1;i=f;return l|0}d=c[j>>2]|0;if((d|0)==0){l=-1;i=f;return l|0}j=c[k+8>>2]|0;b=d+j|0;a=c[k+12>>2]|0;if((a|0)==4){m=1.0;n=+g[b>>2];o=0;p=1}else if((a|0)==3){m=1.0;n=+h[b>>3];o=0;p=1}else if((a|0)==1346784596|(a|0)==1397116244|(a|0)==1){q=c[b>>2]|0;m=1.0;n=1.0;o=(q|0)<0|0?-1:0;p=q}else if((a|0)==1146442272|(a|0)==2){q=b;m=1.0;n=1.0;o=c[q+4>>2]|0;p=c[q>>2]|0}else if((a|0)==0){m=1.0;n=1.0;o=0;p=c[b>>2]|0}else if((a|0)==128){m=1.0;n=+h[k+16>>3];o=0;p=1}else if((a|0)==6){a=c[b>>2]|0;m=+(c[d+(j+4)>>2]|0);n=1.0;o=(a|0)<0|0?-1:0;p=a}else{l=-22;i=f;return l|0}r=n*(+(p>>>0)+ +(o|0)*4294967296.0)/m;o=+R(r)>=1.0?r>0.0?(ah(+Q(r/4294967296.0),4294967295.0)|0)>>>0:~~+ab((r- +(~~r>>>0))/4294967296.0)>>>0:0;c[e>>2]=~~r>>>0;c[e+4>>2]=o;l=0;i=f;return l|0}function Bo(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0.0,A=0.0,B=0,C=0,D=0.0;if((a|0)==0){e=0;return e|0}f=c[a>>2]|0;i=(f|0)==0;j=f+8|0;f=0;while(1){k=(f|0)!=0;do{if(k|i){l=14867}else{m=c[j>>2]|0;if((m|0)==0){l=14867;break}n=c[m>>2]|0;if((n|0)==0){l=14867}else{o=m;p=n}}}while(0);if((l|0)==14867){l=0;if(!k){q=0;r=1;break}n=f+48|0;m=c[n>>2]|0;if((m|0)==0|(n|0)==0){q=0;r=1;break}else{o=n;p=m}}if((a8(p|0,b|0)|0)!=0){f=o;continue}if((c[o+12>>2]|0)==128){f=o}else{l=14871;break}}do{if((l|0)==14871){if((o|0)==0){q=0;r=1;break}q=c[o+44>>2]|0;r=0}}while(0);o=(q|0)==0;f=0;while(1){p=(f|0)!=0;do{if(p|i){l=14877}else{m=c[j>>2]|0;if((m|0)==0){l=14877;break}n=c[m>>2]|0;if((n|0)==0){l=14877}else{s=m;t=n}}}while(0);if((l|0)==14877){l=0;if(!p){e=0;l=14906;break}k=f+48|0;n=c[k>>2]|0;if((n|0)==0|(k|0)==0){e=0;l=14911;break}else{s=k;t=n}}if((a8(t|0,d|0)|0)!=0){f=s;continue}u=c[s+12>>2]|0;n=(u|0)==128;if(o){if(n){f=s;continue}else{l=14885;break}}if(!n){f=s;continue}n=c[s+44>>2]|0;if((n|0)==0){f=s;continue}if((a8(n|0,q|0)|0)==0){l=14885;break}else{f=s}}if((l|0)==14906){return e|0}else if((l|0)==14885){if((u|0)==128&((r|(s|0)==0)^1)){v=0}else{e=0;return e|0}while(1){r=(v|0)!=0;do{if(r|i){l=14889}else{u=c[j>>2]|0;if((u|0)==0){l=14889;break}f=c[u>>2]|0;if((f|0)==0){l=14889}else{w=u;x=f}}}while(0);if((l|0)==14889){l=0;if(!r){e=0;l=14903;break}p=v+48|0;f=c[p>>2]|0;if((f|0)==0|(p|0)==0){e=0;l=14908;break}else{w=p;x=f}}if((a8(x|0,b|0)|0)!=0){v=w;continue}y=c[w+12>>2]|0;if((y|0)==128){v=w}else{l=14893;break}}if((l|0)==14908){return e|0}else if((l|0)==14893){if((w|0)==0){e=0;return e|0}v=c[w+8>>2]|0;w=a+v|0;if((y|0)==4){z=1.0;A=+g[w>>2];B=0;C=1}else if((y|0)==0){z=1.0;A=1.0;B=0;C=c[w>>2]|0}else if((y|0)==1346784596|(y|0)==1397116244|(y|0)==1){b=c[w>>2]|0;z=1.0;A=1.0;B=(b|0)<0|0?-1:0;C=b}else if((y|0)==3){z=1.0;A=+h[w>>3];B=0;C=1}else if((y|0)==6){b=c[w>>2]|0;z=+(c[a+(v+4)>>2]|0);A=1.0;B=(b|0)<0|0?-1:0;C=b}else if((y|0)==1146442272|(y|0)==2){y=w;z=1.0;A=1.0;B=c[y+4>>2]|0;C=c[y>>2]|0}else{e=0;return e|0}D=A*(+(C>>>0)+ +(B|0)*4294967296.0)/z;+R(D)>=1.0?D>0.0?(ah(+Q(D/4294967296.0),4294967295.0)|0)>>>0:~~+ab((D- +(~~D>>>0))/4294967296.0)>>>0:0;e=~~D>>>0&c[s+16>>2];return e|0}else if((l|0)==14903){return e|0}}else if((l|0)==14911){return e|0}return 0}function Bp(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0;f=i;if((a|0)==0){g=-1;i=f;return g|0}AN(b,32,1690440,(h=i,i=i+8|0,c[h>>2]=c[c[a>>2]>>2],h)|0);i=h;Bq(a,b,0,d,e);g=0;i=f;return g|0}function Bq(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;g=i;i=i+16|0;j=g|0;k=g+8|0;l=a;m=(d|0)==0;n=j;o=k|0;p=k+4|0;q=0;while(1){r=c[l>>2]|0;s=(q|0)!=0;do{if(s|(r|0)==0){t=14921}else{u=c[r+8>>2]|0;if((u|0)==0){t=14921;break}v=c[u>>2]|0;if((v|0)==0){t=14921}else{w=u;x=v}}}while(0);if((t|0)==14921){t=0;if(!s){t=14973;break}r=q+48|0;v=c[r>>2]|0;if((v|0)==0|(r|0)==0){t=14974;break}else{w=r;x=v}}v=w+40|0;r=c[v>>2]|0;if((r&e|0)==0){q=w;continue}if((r&f|0)!=0){q=w;continue}u=w+12|0;y=(c[u>>2]|0)==128;if(m){if(y){q=w;continue}AN(b,32,1667480,(z=i,i=i+16|0,c[z>>2]=(r&65536|0)!=0?3119144:1666296,c[z+8>>2]=x,z)|0);i=z}else{if(!y){q=w;continue}if((a8(d|0,c[w+44>>2]|0)|0)!=0){q=w;continue}AN(b,32,1668880,(z=i,i=i+8|0,c[z>>2]=x,z)|0);i=z}y=c[u>>2]|0;if((y|0)==1397316165){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1650896,z)|0);i=z}else if((y|0)==1346784596){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1646680,z)|0);i=z}else if((y|0)==2){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1660520,z)|0);i=z}else if((y|0)==1129270354){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1640936,z)|0);i=z}else if((y|0)==4){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1658080,z)|0);i=z}else if((y|0)==5){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1655920,z)|0);i=z}else if((y|0)==6){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1654216,z)|0);i=z}else if((y|0)==7){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1652712,z)|0);i=z}else if((y|0)==1){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1661920,z)|0);i=z}else if((y|0)==1397116244){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1645224,z)|0);i=z}else if((y|0)==1448231252){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1649344,z)|0);i=z}else if((y|0)==1146442272){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1643432,z)|0);i=z}else if((y|0)==0){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1663480,z)|0);i=z}else if((y|0)==3){AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=1659288,z)|0);i=z}else{AN(b,32,1664832,(z=i,i=i+8|0,c[z>>2]=3119144,z)|0);i=z}AN(b,32,1639336,(z=i,i=i+8|0,c[z>>2]=(c[v>>2]&1|0)!=0?69:46,z)|0);i=z;AN(b,32,1639336,(z=i,i=i+8|0,c[z>>2]=(c[v>>2]&2|0)!=0?68:46,z)|0);i=z;AN(b,32,1639336,(z=i,i=i+8|0,c[z>>2]=(c[v>>2]&65536|0)!=0?70:46,z)|0);i=z;AN(b,32,1639336,(z=i,i=i+8|0,c[z>>2]=(c[v>>2]&16|0)!=0?86:46,z)|0);i=z;AN(b,32,1639336,(z=i,i=i+8|0,c[z>>2]=(c[v>>2]&8|0)!=0?65:46,z)|0);i=z;AN(b,32,1639336,(z=i,i=i+8|0,c[z>>2]=(c[v>>2]&32|0)!=0?83:46,z)|0);i=z;v=c[w+4>>2]|0;if((v|0)!=0){AN(b,32,1637536,(z=i,i=i+8|0,c[z>>2]=v,z)|0);i=z}v=c[l>>2]|0;if((c[v+12>>2]|0)>3410688){A=c[v+40>>2]|0}else{A=0}if((dk[((A|0)==0?18:A)&63](j,a,c[w>>2]|0,2)|0)>-1){L20180:do{switch(c[u>>2]|0){case 1:case 2:case 3:case 4:case 6:{v=c[j>>2]|0;if((c[v+4>>2]|0)>0){B=0}else{C=v;break L20180}while(1){AN(b,32,1635728,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z;Bz(b,+h[(c[(c[c[j>>2]>>2]|0)+(B<<2)>>2]|0)+8>>3]);AN(b,32,1635216,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z;Bz(b,+h[(c[(c[c[j>>2]>>2]|0)+(B<<2)>>2]|0)+16>>3]);AN(b,32,1634552,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z;v=B+1|0;y=c[j>>2]|0;if((v|0)<(c[y+4>>2]|0)){B=v}else{C=y;break}}break};default:{C=c[j>>2]|0}}}while(0);s=C+4|0;if((c[s>>2]|0)>0){y=C|0;v=0;do{A6(c[(c[y>>2]|0)+(v<<2)>>2]|0);A6((c[y>>2]|0)+(v<<2)|0);v=v+1|0;}while((v|0)<(c[s>>2]|0))}A6(C);A6(n)}s=c[u>>2]|0;if((s|0)==1129270354|(s|0)==1397316165|(s|0)==5|(s|0)==1448231252){if((c[w+16>>2]|0)!=0){t=14962}}else if(!((s|0)==128|(s|0)==7)){t=14962}if((t|0)==14962){t=0;AN(b,32,1633760,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z;s=c[u>>2]|0;if((s|0)==1129270354|(s|0)==1397316165|(s|0)==5|(s|0)==1448231252){AN(b,32,1632328,(z=i,i=i+8|0,c[z>>2]=c[w+16>>2],z)|0);i=z}else if((s|0)==3|(s|0)==4){Bz(b,+h[w+16>>3])}else if((s|0)==6){B_(k,+h[w+16>>3],2147483647);v=c[p>>2]|0;AN(b,32,1711216,(z=i,i=i+16|0,c[z>>2]=c[o>>2],c[z+8>>2]=v,z)|0);i=z}else if((s|0)==1146442272|(s|0)==1|(s|0)==2){v=w+16|0;Bz(b,+((c[v>>2]|0)>>>0)+ +(c[v+4>>2]|0)*4294967296.0)}else if((s|0)==0){s=w+16|0;v=c[s+4>>2]|0;AN(b,32,1633312,(z=i,i=i+16|0,c[z>>2]=c[s>>2],c[z+8>>2]=v,z)|0);i=z}AN(b,32,1634552,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z}AN(b,32,1631400,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z;v=c[w+44>>2]|0;if((v|0)==0){q=w;continue}if((c[u>>2]|0)==128){q=w;continue}Bq(a,b,v,e,f);q=w}if((t|0)==14973){i=g;return}else if((t|0)==14974){i=g;return}}function Br(a){a=a|0;Bs(a,0,0);return}function Bs(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0.0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0;e=i;i=i+8|0;f=e|0;g=f;j=a;k=c[j>>2]|0;l=(a|0)==0;m=k+12|0;n=0;o=k;while(1){k=(n|0)!=0;p=(o|0)==0;do{if(k|p){q=14980}else{r=c[o+8>>2]|0;if((r|0)==0){q=14980;break}s=c[r>>2]|0;if((s|0)==0){q=14980}else{t=r;u=s}}}while(0);if((q|0)==14980){q=0;if(!k){q=15027;break}s=n+48|0;r=c[s>>2]|0;if((r|0)==0|(s|0)==0){q=15028;break}else{t=s;u=r}}L20220:do{if((c[t+40>>2]&b|0)==(d|0)){r=c[t+12>>2]|0;if((r|0)==1346784596){s=c[m>>2]|0;if((s|0)!=0&(s|0)<3410532){s=c[t+16>>2]|0;Bd(a,u,s,0)|0;break}else{s=c[t+16>>2]|0;Bk(a,u,s,0,1346784596,1735072,321)|0;break}}else if((r|0)==6){B_(g,+h[t+16>>3],2147483647);s=c[t>>2]|0;v=c[f+4>>2]|0;w=+(c[f>>2]|0);if(l){break}x=c[j>>2]|0;y=(x|0)==0;z=x+8|0;x=0;while(1){A=(x|0)!=0;do{if(A|y){q=15012}else{B=c[z>>2]|0;if((B|0)==0){q=15012;break}C=c[B>>2]|0;if((C|0)==0){q=15012}else{D=B;E=C}}}while(0);if((q|0)==15012){q=0;if(!A){break L20220}C=x+48|0;B=c[C>>2]|0;if((B|0)==0|(C|0)==0){break L20220}else{D=C;E=B}}if((a8(E|0,s|0)|0)!=0){x=D;continue}if((c[D+12>>2]|0)==128){x=D}else{break}}if((D|0)==0){break}BA(a,D,a+(c[D+8>>2]|0)|0,w,v,1,0)|0;break}else if((r|0)==0|(r|0)==1|(r|0)==2|(r|0)==1146442272){x=t+16|0;s=c[x>>2]|0;z=c[x+4>>2]|0;if(l){break}x=o+8|0;y=0;while(1){B=(y|0)!=0;do{if(B|p){q=14990}else{C=c[x>>2]|0;if((C|0)==0){q=14990;break}F=c[C>>2]|0;if((F|0)==0){q=14990}else{G=C;H=F}}}while(0);if((q|0)==14990){q=0;if(!B){break L20220}A=y+48|0;F=c[A>>2]|0;if((F|0)==0|(A|0)==0){break L20220}else{G=A;H=F}}if((a8(H|0,u|0)|0)!=0){y=G;continue}if((c[G+12>>2]|0)==128){y=G}else{break}}if((G|0)==0){break}BA(a,G,a+(c[G+8>>2]|0)|0,1.0,1,s,z)|0;break}else if((r|0)==1397116244){y=c[m>>2]|0;if((y|0)!=0&(y|0)<3410532){y=c[t+16>>2]|0;Bd(a,u,y,0)|0;break}else{y=c[t+16>>2]|0;Bk(a,u,y,0,1397116244,1730216,10)|0;break}}else if((r|0)==128|(r|0)==7){break}else if((r|0)==1129270354|(r|0)==5|(r|0)==1397316165|(r|0)==1448231252){Bd(a,u,c[t+16>>2]|0,0)|0;break}else if((r|0)==3|(r|0)==4){w=+h[t+16>>3];if(l){break}y=o+8|0;x=0;while(1){v=(x|0)!=0;do{if(v|p){q=15001}else{F=c[y>>2]|0;if((F|0)==0){q=15001;break}A=c[F>>2]|0;if((A|0)==0){q=15001}else{I=F;J=A}}}while(0);if((q|0)==15001){q=0;if(!v){break L20220}B=x+48|0;A=c[B>>2]|0;if((A|0)==0|(B|0)==0){break L20220}else{I=B;J=A}}if((a8(J|0,u|0)|0)!=0){x=I;continue}if((c[I+12>>2]|0)==128){x=I}else{break}}if((I|0)==0){break}BA(a,I,a+(c[I+8>>2]|0)|0,w,1,1,0)|0;break}else{AN(a,48,1687360,(x=i,i=i+16|0,c[x>>2]=r,c[x+8>>2]=u,x)|0);i=x;break}}}while(0);n=t;o=c[j>>2]|0}if((q|0)==15028){i=e;return}else if((q|0)==15027){i=e;return}}function Bt(b,d,e,f,g,h){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;j=i;i=i+8|0;k=j|0;l=c[b>>2]|0;c[k>>2]=l;m=l+(cm(l|0,1676256)|0)|0;l=m;while(1){n=a[l]|0;o=n<<24>>24;if(!(((o|32)-97|0)>>>0<26|(o-48|0)>>>0<10)){if(!((n<<24>>24|0)==95|(n<<24>>24|0)==47|(n<<24>>24|0)==45|(n<<24>>24|0)==46)){break}}l=l+1|0}n=cm(l|0,1676256)|0;o=a[l+n|0]|0;do{if(o<<24>>24==0){p=15037}else{if((bi(d|0,o<<24>>24|0)|0)==0){p=15037;break}q=l-m|0;r=A1(q+1|0)|0;if((r|0)==0){p=15037;break}s=l+(n+1)|0;NN(r|0,m|0,q)|0;a[r+q|0]=0;c[k>>2]=s;t=r}}while(0);do{if((p|0)==15037){if((f&1|0)==0){u=-22}else{t=0;break}i=j;return u|0}}while(0);f=y2(k,e)|0;if((f|0)==0){A4(t);u=-12;i=j;return u|0}else{c[b>>2]=c[k>>2];c[g>>2]=t;c[h>>2]=f;u=0;i=j;return u|0}return 0}function Bu(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0;b=a;d=0;while(1){e=c[b>>2]|0;f=(d|0)!=0;do{if(f|(e|0)==0){g=15049}else{h=c[e+8>>2]|0;if((h|0)==0){g=15049;break}if((c[h>>2]|0)==0){g=15049}else{i=h}}}while(0);if((g|0)==15049){g=0;if(!f){g=15054;break}e=d+48|0;if((c[e>>2]|0)==0|(e|0)==0){g=15055;break}else{i=e}}e=c[i+12>>2]|0;if(!((e|0)==5|(e|0)==7)){d=i;continue}A6(a+(c[i+8>>2]|0)|0);d=i}if((g|0)==15055){return}else if((g|0)==15054){return}}function Bv(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;d=i;i=i+8|0;e=d|0;c[e>>2]=0;f=zJ(c[b>>2]|0,3119144,0,2)|0;L20314:do{if((f|0)==0){g=0}else{h=f;while(1){j=h|0;k=h+4|0;l=Bd(a,c[j>>2]|0,c[k>>2]|0,0)|0;if((l|0)==-1414549496){m=c[j>>2]|0;n=c[k>>2]|0;zK(e,m,n,0)|0}else{if((l|0)<0){break}}n=zJ(c[b>>2]|0,3119144,h,2)|0;if((n|0)==0){g=0;break L20314}else{h=n}}h=c[k>>2]|0;AN(a,16,1678208,(n=i,i=i+16|0,c[n>>2]=c[j>>2],c[n+8>>2]=h,n)|0);i=n;g=l}}while(0);zL(b);c[b>>2]=c[e>>2];i=d;return g|0}function Bw(a,b){a=a|0;b=b|0;var d=0,e=0;d=c[a+28>>2]|0;if((d|0)==0){e=0;return e|0}e=c1[d&255](b)|0;return e|0}function Bx(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;e=(a|0)==0;f=a+8|0;a=0;while(1){g=(a|0)!=0;do{if(g|e){h=15072}else{i=c[f>>2]|0;if((i|0)==0){h=15072;break}j=c[i>>2]|0;if((j|0)==0){h=15072}else{k=i;l=j}}}while(0);if((h|0)==15072){h=0;if(!g){m=0;h=15082;break}j=a+48|0;i=c[j>>2]|0;if((i|0)==0|(j|0)==0){m=0;h=15081;break}else{k=j;l=i}}if((a8(l|0,d|0)|0)!=0){a=k;continue}if((c[k+12>>2]|0)==128){a=k}else{h=15076;break}}if((h|0)==15082){return m|0}else if((h|0)==15081){return m|0}else if((h|0)==15076){if((k|0)==0){m=0;return m|0}m=b+(c[k+8>>2]|0)|0;return m|0}return 0}function By(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,i=0,j=0,k=0,l=0,m=0,n=0;f=A7(8)|0;g=f;i=A7(4)|0;j=i;k=A7(48)|0;l=Be(b,d,0,0,e,0)|0;c[a>>2]=0;do{if((f|0)==0|(k|0)==0|(i|0)==0|(l|0)==0){m=-12}else{c[f>>2]=j;c[j>>2]=k;c[f+4>>2]=1;c[k+40>>2]=1;e=k+8|0;h[e>>3]=+h[l+24>>3];d=k+16|0;h[d>>3]=+h[l+32>>3];b=c[l+12>>2]|0;if((b|0)==6){h[k+24>>3]=-2147483648.0;h[k+32>>3]=2147483647.0}else if((b|0)==1397316165){h[k+24>>3]=0.0;h[k+32>>3]=2097151.0;h[e>>3]=0.0;h[d>>3]=268435455.0}else if((b|0)==5){h[k+24>>3]=0.0;h[k+32>>3]=1114111.0;h[e>>3]=-1.0;h[d>>3]=2147483647.0}else if((b|0)==1448231252){h[k+24>>3]=1.0;h[k+32>>3]=2147483647.0;h[e>>3]=1.0;h[d>>3]=2147483647.0}else if(!((b|0)==1|(b|0)==2|(b|0)==1346784596|(b|0)==1397116244|(b|0)==4|(b|0)==3|(b|0)==1146442272|(b|0)==1129270354)){m=-38;break}c[a>>2]=g;n=0;return n|0}}while(0);A4(f);A4(k);A4(i);n=m;return n|0}function Bz(a,b){a=a|0;b=+b;var d=0,e=0;d=i;if(b==2147483647.0){AN(a,32,1630856,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;i=d;return}if(b==-2147483648.0){AN(a,32,1630312,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;i=d;return}if(b==9223372036854776000.0){AN(a,32,1629400,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;i=d;return}if(b==-9223372036854776000.0){AN(a,32,1628704,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;i=d;return}if(b==3.4028234663852886e+38){AN(a,32,1628280,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;i=d;return}if(b==1.1754943508222875e-38){AN(a,32,1627800,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;i=d;return}else{AN(a,32,1627240,(e=i,i=i+8|0,h[e>>3]=b,e)|0);i=e;i=d;return}}function BA(a,b,d,e,f,j,k){a=a|0;b=b|0;d=d|0;e=+e;f=f|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0.0,p=0.0,q=0.0,r=0.0,s=0,t=0,u=0,v=0;l=i;i=i+8|0;m=l|0;n=m;o=+h[b+32>>3];p=+(f|0);q=(+(j>>>0)+ +(k|0)*4294967296.0)*e;r=+h[b+24>>3];do{if(p*o>=q){if(p*r>q){break}s=c[b+12>>2]|0;if((s|0)==4){g[d>>2]=q/p;t=0;i=l;return t|0}else if((s|0)==0|(s|0)==1346784596|(s|0)==1397116244|(s|0)==1){u=cS(+(e/p))|0;v=N2(u,G,j,k)|0;c[d>>2]=v;t=0;i=l;return t|0}else if((s|0)==6){if(+(~~e|0)==e){c[d>>2]=~~q;c[d+4>>2]=f;t=0;i=l;return t|0}else{B_(n,q/p,16777216);v=d;u=c[m+4>>2]|0;c[v>>2]=c[m>>2];c[v+4>>2]=u;t=0;i=l;return t|0}}else if((s|0)==3){h[d>>3]=q/p;t=0;i=l;return t|0}else if((s|0)==1146442272|(s|0)==2){s=cS(+(e/p))|0;u=N2(s,G,j,k)|0;s=d;c[s>>2]=u;c[s+4>>2]=G;t=0;i=l;return t|0}else{t=-22;i=l;return t|0}}}while(0);d=c[b>>2]|0;AN(a,16,1624408,(a=i,i=i+32|0,h[a>>3]=q/p,c[a+8>>2]=d,h[a+16>>3]=r,h[a+24>>3]=o,a)|0);i=a;t=-34;i=l;return t|0}function BB(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;g=i;i=i+24|0;j=g+8|0;k=g+16|0;l=a|0;m=a+4|0;n=ct(b|0,1633320,(o=i,i=i+24|0,c[o>>2]=l,c[o+8>>2]=m,c[o+16>>2]=g,o)|0)|0;i=o;if((n|0)==2){n=c[l>>2]|0;o=n;p=(n|0)<0|0?-1:0;n=c[m>>2]|0;q=n;r=(n|0)<0|0?-1:0;n=d;s=(d|0)<0|0?-1:0;BX(l,m,o,p,q,r,n,s)|0;t=0;i=g;return t|0}s=zV(j,b,0,0,0,0,0,0,0,e,f)|0;if((s|0)<0){t=s;i=g;return t|0}B_(k,+h[j>>3],d);d=a;a=c[k+4>>2]|0;c[d>>2]=c[k>>2];c[d+4>>2]=a;t=0;i=g;return t|0}function BC(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;f=i;i=i+8|0;g=f|0;h=0;while(1){if((h|0)>=43){j=15147;break}if((a8(c[33640+(h*12|0)>>2]|0,e|0)|0)==0){j=15146;break}else{h=h+1|0}}do{if((j|0)==15146){k=c[33648+(h*12|0)>>2]|0;l=c[33644+(h*12|0)>>2]|0}else if((j|0)==15147){if((h|0)!=43){m=-22;i=f;return m|0}n=aU(e|0,g|0,10)|0;o=c[g>>2]|0;if((a[o]|0)==0){p=o}else{q=o+1|0;c[g>>2]=q;p=q}q=aU(p|0,g|0,10)|0;if((a[c[g>>2]|0]|0)==0){k=q;l=n;break}else{m=-22}i=f;return m|0}}while(0);if((l|0)<1|(k|0)<1){m=-22;i=f;return m|0}c[b>>2]=l;c[d>>2]=k;m=0;i=f;return m|0}function BD(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=0;while(1){if((d|0)>=8){break}if((a8(c[34160+(d*12|0)>>2]|0,b|0)|0)==0){e=15161;break}else{d=d+1|0}}if((e|0)==15161){e=34164+(d*12|0)|0;d=a;f=c[e+4>>2]|0;c[d>>2]=c[e>>2];c[d+4>>2]=f;g=0;return g|0}f=BB(a,b,1001e3,56,0)|0;if((f|0)<0){g=f;return g|0}if((c[a>>2]|0)<1){g=-22;return g|0}else{return((c[a+4>>2]|0)<1?-22:0)|0}return 0}function BE(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0.0,z=0.0;g=i;i=i+144|0;h=g|0;j=g+136|0;if((a[d]|0)==35){k=1}else{k=(ba(d|0,1716640,2)|0)==0?2:0}if((e|0)<0){l=NL(d|0)|0}else{l=e}e=g+8|0;m=l-k+1|0;y_(e,d+k|0,m>>>0>128?128:m)|0;m=bi(e|0,64)|0;c[h>>2]=m;if((m|0)==0){n=0}else{l=m+1|0;c[h>>2]=l;a[m]=0;n=l}l=NL(e|0)|0;m=b+3|0;a[m]=-1;L20459:do{if((y4(e,1689704)|0)==0){o=15177}else{if((y4(e,1670032)|0)==0){o=15177;break}do{if((k|0)==0){if((cm(e|0,1655e3)|0)==(l|0)){break}p=bI(e|0,264136,140,8,46)|0;if((p|0)!=0){q=p+4|0;a[b]=a[q]|0;a[b+1|0]=a[q+1|0]|0;a[b+2|0]=a[q+2|0]|0;break L20459}AN(f,16,1629136,(r=i,i=i+8|0,c[r>>2]=e,r)|0);i=r;s=-22;i=g;return s|0}}while(0);q=aS(e|0,j|0,16)|0;do{if((a[c[j>>2]|0]|0)==0){if(!((l|0)==8|(l|0)==6)){break}p=q&255;t=q>>>8&255;u=q>>>16&255;if((l|0)==8){a[m]=p;v=q>>>24&255;w=u;x=t}else{v=u;w=t;x=p}a[b]=v;a[b+1|0]=w;a[b+2|0]=x;break L20459}}while(0);AN(f,16,1636232,(r=i,i=i+8|0,c[r>>2]=e,r)|0);i=r;s=-22;i=g;return s|0}}while(0);if((o|0)==15177){o=BW()|0;a[b]=o>>>24&255;a[b+1|0]=o>>>16&255;a[b+2|0]=o>>>8&255;a[m]=o&255}if((n|0)==0){s=0;i=g;return s|0}do{if((ba(n|0,1716640,2)|0)==0){y=+((aS(n|0,h|0,16)|0)>>>0>>>0)}else{z=+NK(n,h);if(z<0.0|z>1.0){y=256.0;break}y=z*255.0}}while(0);o=c[h>>2]|0;do{if((o|0)!=(n|0)){if((a[o]|0)!=0){break}if(y>255.0|y<0.0){break}a[m]=~~y;s=0;i=g;return s|0}}while(0);AN(f,16,1622976,(r=i,i=i+16|0,c[r>>2]=n,c[r+8>>2]=d,r)|0);i=r;s=-22;i=g;return s|0}function BF(a,b){a=a|0;b=b|0;return y4(a,c[b>>2]|0)|0}function BG(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0;f=e+8|0;g=e+12|0;h=e+16|0;i=e+20|0;j=e|0;k=e+4|0;e=d;d=b;L20498:while(1){if((y7(a[e]|0)|0)!=0){b=d;while(1){if((y7(a[b]|0)|0)==0){break}else{b=b+1|0}}e=e+1|0;d=b;continue}l=e+1|0;m=a[e]|0;if(m<<24>>24==0){n=d;o=15258;break}p=m<<24>>24;L20506:do{if(m<<24>>24==37){q=e+2|0;r=a[l]|0;switch(r<<24>>24|0){case 100:{s=a[d]|0;if((y6(s)|0)==0){n=0;o=15248;break L20498}t=s-48|0;s=d+1|0;u=a[s]|0;if((y6(u)|0)==0){v=t;w=s}else{v=(t*10|0)-48+u|0;w=d+2|0}if((v-1|0)>>>0>30|(v|0)==-1){n=0;o=15261;break L20498}c[g>>2]=v;e=q;d=w;continue L20498;break};case 109:{u=a[d]|0;if((y6(u)|0)==0){n=0;o=15249;break L20498}t=u-48|0;u=d+1|0;s=a[u]|0;if((y6(s)|0)==0){x=t;y=u}else{x=(t*10|0)-48+s|0;y=d+2|0}s=x-1|0;if(s>>>0>11|(x|0)==-1){n=0;o=15253;break L20498}c[h>>2]=s;e=q;d=y;continue L20498;break};case 72:case 74:{s=a[d]|0;if((y6(s)|0)==0){n=0;o=15250;break L20498}t=s-48|0;s=d+1|0;u=a[s]|0;if((y6(u)|0)==0){z=t;A=s}else{z=(t*10|0)-48+u|0;A=d+2|0}if((z|0)<0|(z|0)>((r<<24>>24==72?23:2147483647)|0)|(z|0)==-1){n=0;o=15255;break L20498}c[f>>2]=z;e=q;d=A;continue L20498;break};case 77:{r=a[d]|0;if((y6(r)|0)==0){n=0;o=15252;break L20498}u=r-48|0;r=d+1|0;t=a[r]|0;if((y6(t)|0)==0){B=u;C=r}else{B=(u*10|0)-48+t|0;C=d+2|0}if((C|0)==(d|0)|B>>>0>59){n=0;o=15256;break L20498}c[k>>2]=B;e=q;d=C;continue L20498;break};case 89:{t=a[d]|0;if((y6(t)|0)==0){n=0;o=15254;break L20498}u=t-48|0;t=d+1|0;r=a[t]|0;do{if((y6(r)|0)==0){D=u;E=t}else{s=(u*10|0)-48+r|0;F=d+2|0;G=a[F]|0;if((y6(G)|0)==0){D=s;E=F;break}F=(s*10|0)-48+G|0;G=d+3|0;s=a[G]|0;if((y6(s)|0)==0){D=F;E=G;break}D=(F*10|0)-48+s|0;E=d+4|0}}while(0);if((E|0)==(d|0)|D>>>0>9999){n=0;o=15251;break L20498}c[i>>2]=D-1900;e=q;d=E;continue L20498;break};case 83:{r=a[d]|0;if((y6(r)|0)==0){n=0;o=15259;break L20498}u=r-48|0;r=d+1|0;t=a[r]|0;if((y6(t)|0)==0){H=u;I=r}else{H=(u*10|0)-48+t|0;I=d+2|0}if((I|0)==(d|0)|H>>>0>59){n=0;o=15257;break L20498}c[j>>2]=H;e=q;d=I;continue L20498;break};case 37:{J=37;K=q;break L20506;break};default:{n=0;o=15262;break L20498}}}else{J=p;K=l}}while(0);if((J|0)!=(a[d]|0)){n=0;o=15260;break}e=K;d=d+1|0}if((o|0)==15250){return n|0}else if((o|0)==15251){return n|0}else if((o|0)==15252){return n|0}else if((o|0)==15253){return n|0}else if((o|0)==15254){return n|0}else if((o|0)==15259){return n|0}else if((o|0)==15260){return n|0}else if((o|0)==15261){return n|0}else if((o|0)==15262){return n|0}else if((o|0)==15255){return n|0}else if((o|0)==15256){return n|0}else if((o|0)==15257){return n|0}else if((o|0)==15258){return n|0}else if((o|0)==15248){return n|0}else if((o|0)==15249){return n|0}return 0}function BH(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=c[a+20>>2]|0;d=c[a+16>>2]|0;e=d+1|0;if((e|0)<3){f=b+1899|0;g=d+13|0}else{f=b+1900|0;g=e}return((c[a+8>>2]|0)*3600|0)-2032579456+(((((g*153|0)-457|0)/5|0)+(c[a+12>>2]|0)+(f*365|0)+((f|0)/4|0)+((f|0)/-100|0)+((f|0)/400|0)|0)*86400|0)+((c[a+4>>2]|0)*60|0)+(c[a>>2]|0)|0}function BI(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;f=i;i=i+96|0;g=f|0;h=f+8|0;j=f+16|0;k=f+64|0;NM(j|0,0,44);c[g>>2]=0;c[b>>2]=0;c[b+4>>2]=-2147483648;l=(e|0)!=0;L20570:do{if(l){e=(a[d]|0)==45;m=e?d+1|0:d;n=e&1;e=BG(m,1725728,j)|0;c[g>>2]=e;if((e|0)!=0){o=e;p=0;q=n;break}e=BG(m,1721680,j)|0;c[g>>2]=e;r=j+8|0;c[r>>2]=0;if((e|0)!=0){o=e;p=0;q=n;break}c[j>>2]=aU(m|0,g|0,10)|0;e=c[g>>2]|0;if((e|0)==(m|0)){s=-22;i=f;return s|0}else{c[j+4>>2]=0;c[r>>2]=0;t=n;u=0;v=e;w=15282;break}}else{c[h>>2]=cU(0)|0;if((y4(d,1730176)|0)==0){e=c[h>>2]|0;n=N2(e,(e|0)<0|0?-1:0,1e6,0)|0;c[b>>2]=n;c[b+4>>2]=G;s=0;i=f;return s|0}else{x=0}while(1){if(x>>>0>=2){w=15271;break}n=BG(d,c[346336+(x<<2)>>2]|0,j)|0;c[g>>2]=n;if((n|0)==0){x=x+1|0}else{y=0;z=n;break}}do{if((w|0)==15271){n=c[g>>2]|0;if((n|0)!=0){y=0;z=n;break}c[g>>2]=d;y=1;z=d}}while(0);n=a[z]|0;if((n<<24>>24|0)==84|(n<<24>>24|0)==116|(n<<24>>24|0)==32){A=z+1|0}else{A=z}n=0;e=z;while(1){if(n>>>0>=2){t=0;u=y;v=e;w=15282;break L20570}r=BG(A,c[346328+(n<<2)>>2]|0,j)|0;c[g>>2]=r;if((r|0)==0){n=n+1|0;e=0}else{o=r;p=y;q=0;break}}}}while(0);do{if((w|0)==15282){if((v|0)==0){s=-22}else{o=v;p=u;q=t;break}i=f;return s|0}}while(0);do{if((a[o]|0)==46){t=o+1|0;c[g>>2]=t;do{if((y6(a[t]|0)|0)==0){B=0}else{u=c[g>>2]|0;v=((a[u]|0)*1e5|0)-48e5|0;w=u+1|0;c[g>>2]=w;if((y6(a[w]|0)|0)==0){B=v;break}w=c[g>>2]|0;u=((a[w]|0)*1e4|0)-48e4+v|0;v=w+1|0;c[g>>2]=v;if((y6(a[v]|0)|0)==0){B=u;break}v=c[g>>2]|0;w=((a[v]|0)*1e3|0)-48e3+u|0;u=v+1|0;c[g>>2]=u;if((y6(a[u]|0)|0)==0){B=w;break}u=c[g>>2]|0;v=((a[u]|0)*100|0)-4800+w|0;w=u+1|0;c[g>>2]=w;if((y6(a[w]|0)|0)==0){B=v;break}w=c[g>>2]|0;u=((a[w]|0)*10|0)-480+v|0;v=w+1|0;c[g>>2]=v;if((y6(a[v]|0)|0)==0){B=u;break}v=c[g>>2]|0;w=(a[v]|0)-48+u|0;c[g>>2]=v+1;B=w}}while(0);if((y6(a[c[g>>2]|0]|0)|0)==0){C=B;break}while(1){t=(c[g>>2]|0)+1|0;c[g>>2]=t;if((y6(a[t]|0)|0)==0){C=B;break}}}else{C=0}}while(0);do{if(l){D=((c[j+4>>2]|0)*60|0)+((c[j+8>>2]|0)*3600|0)+(c[j>>2]|0)|0}else{B=c[g>>2]|0;o=a[B]|0;c[g>>2]=B+((o<<24>>24==90|o<<24>>24==122)&1);if((p|0)!=0){if((o<<24>>24|0)==122|(o<<24>>24|0)==90){B=(bz(h|0)|0)+12|0;t=k;NN(t|0,B|0,32)|0}else{B=(a0(h|0)|0)+12|0;t=k;NN(t|0,B|0,32)|0}B=j+12|0;t=k;NN(B|0,t|0,32)|0}if(!((o<<24>>24|0)==122|(o<<24>>24|0)==90)){D=b3(j|0)|0;break}o=c[j+20>>2]|0;t=c[j+16>>2]|0;B=t+1|0;if((B|0)<3){E=o+1899|0;F=t+13|0}else{E=o+1900|0;F=B}D=((c[j+8>>2]|0)*3600|0)-2032579456+(((((F*153|0)-457|0)/5|0)+(c[j+12>>2]|0)+(E*365|0)+((E|0)/4|0)+((E|0)/-100|0)+((E|0)/400|0)|0)*86400|0)+((c[j+4>>2]|0)*60|0)+(c[j>>2]|0)|0}}while(0);if((a[c[g>>2]|0]|0)!=0){s=-22;i=f;return s|0}g=N2(D,(D|0)<0|0?-1:0,1e6,0)|0;D=NU(g,G,C,(C|0)<0|0?-1:0)|0;C=G;g=(q|0)!=0;q=NV(0,0,D,C)|0;c[b>>2]=g?q:D;c[b+4>>2]=g?G:C;s=0;i=f;return s|0}function BJ(a,e,f,g,h,i,j,k,l){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;m=b[g+8+(j<<1)>>1]|0;n=m&3;o=(m&65535)>>>11&15;p=o+1|0;q=(1<<p)-1|0;r=(m&65535)>>>8&7;s=((m&65535)>>>2&7)+1|0;t=d[g+7|0]|0;if((t&4|0)!=0){g=((m&65535)>>>5&7)-1+(ac(s,h)|0)|0;u=c[e+(n<<2)>>2]|0;v=u+((ac(c[f+(n<<2)>>2]|0,i)|0)+(g>>3))|0;u=7-o-(g&7)|0;g=k-1|0;if((k|0)==0){return}o=e+4|0;if((l|0)==0){w=v;x=u;y=a;z=g;while(1){A=x-s|0;b[y>>1]=(d[w]|0)>>>(x>>>0)&q&65535;if((z|0)==0){break}else{w=w+(-(A>>3)|0)|0;x=A&7;y=y+2|0;z=z-1|0}}return}else{z=v;v=u;u=a;y=g;while(1){g=v-s|0;b[u>>1]=d[(c[o>>2]|0)+((((d[z]|0)>>>(v>>>0)&q)<<2)+j)|0]|0;if((y|0)==0){break}else{z=z+(-(g>>3)|0)|0;v=g&7;u=u+2|0;y=y-1|0}}return}}y=c[e+(n<<2)>>2]|0;u=ac(c[f+(n<<2)>>2]|0,i)|0;i=((m&65535)>>>5&7)-1+(ac(s,h)|0)+u|0;u=(p+r|0)<9;if(u){B=i+(t&1)|0}else{B=i}i=y+B|0;B=k-1|0;if((k|0)==0){return}k=(l|0)==0;l=e+4|0;e=(t&1|0)==0;if(u){if(e){u=i;t=a;y=B;while(1){p=(d[u]|0)>>>(r>>>0)&q;if(k){C=p&65535}else{C=d[(c[l>>2]|0)+((p<<2)+j)|0]|0}b[t>>1]=C;if((y|0)==0){break}else{u=u+s|0;t=t+2|0;y=y-1|0}}return}else{y=i;t=a;u=B;while(1){C=(d[y]|0)>>>(r>>>0)&q;if(k){D=C&65535}else{D=d[(c[l>>2]|0)+((C<<2)+j)|0]|0}b[t>>1]=D;if((u|0)==0){break}else{y=y+s|0;t=t+2|0;u=u-1|0}}return}}else{if(e){e=i;u=a;t=B;while(1){y=e;D=((d[y]|d[y+1|0]<<8)<<16>>16&65535)>>>(r>>>0)&q;if(k){E=D&65535}else{E=d[(c[l>>2]|0)+((D<<2)+j)|0]|0}b[u>>1]=E;if((t|0)==0){break}else{e=e+s|0;u=u+2|0;t=t-1|0}}return}else{t=i;i=a;a=B;while(1){B=t;u=((cT((d[B]|d[B+1|0]<<8)<<16>>16|0)|0)&65535)>>>(r>>>0)&q;if(k){F=u&65535}else{F=d[(c[l>>2]|0)+((u<<2)+j)|0]|0}b[i>>1]=F;if((a|0)==0){break}else{t=t+s|0;i=i+2|0;a=a-1|0}}return}}}function BK(f,g,h,i,j,k,l,m){f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;n=b[i+8+(l<<1)>>1]|0;l=n&3;o=(n&65535)>>>11&15;p=((n&65535)>>>2&7)+1|0;q=d[i+7|0]|0;if((q&4|0)!=0){i=((n&65535)>>>5&7)-1+(ac(p,j)|0)|0;if((m|0)==0){return}r=(ac(c[h+(l<<2)>>2]|0,k)|0)+(i>>3)|0;s=7-o-(i&7)|0;i=(c[g+(l<<2)>>2]|0)+r|0;r=m;t=f;while(1){u=r-1|0;a[i]=(d[i]|0|(e[t>>1]|0)<<s)&255;v=s-p|0;if((u|0)==0){break}else{s=v&7;i=i+(-(v>>3)|0)|0;r=u;t=t+2|0}}return}t=(n&65535)>>>8&7;r=c[g+(l<<2)>>2]|0;g=ac(c[h+(l<<2)>>2]|0,k)|0;k=((n&65535)>>>5&7)-1+(ac(p,j)|0)+g|0;g=r+k|0;j=(m|0)==0;if((o+1+t|0)<9){if(j){return}o=r+(k+(q&1))|0;k=m;r=f;while(1){n=k-1|0;a[o]=(d[o]|0|(e[r>>1]|0)<<t)&255;if((n|0)==0){break}else{o=o+p|0;k=n;r=r+2|0}}return}if(j){return}j=(q&1|0)==0;q=g;g=f;f=m;while(1){m=f-1|0;r=q;k=(d[r]|d[r+1|0]<<8)<<16>>16;if(j){w=((e[g>>1]|0)<<t|k&65535)&65535}else{o=(cT(k|0)|0)&65535;k=(e[g>>1]|0)<<t&65535|o;w=(k>>>8|k<<8)&65535}y=w;a[r]=y&255;y=y>>8;a[r+1|0]=y&255;if((m|0)==0){break}else{q=q+p|0;g=g+2|0;f=m}}return}function BL(a){a=a|0;var b=0;if(a>>>0>=321){b=0;return b|0}b=c[341192+(a<<4)>>2]|0;return b|0}function BM(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=i;i=i+32|0;d=b|0;if((a8(a|0,1724360)|0)==0){e=1680072}else{e=(a8(a|0,1723720)|0)==0?1683680:a}a=0;while(1){f=c[341192+(a<<4)>>2]|0;if((f|0)!=0){if((a8(f|0,e|0)|0)==0){g=15378;break}}f=a+1|0;if((f|0)<321){a=f}else{break}}do{if((g|0)==15378){if((a|0)==-1){break}else{h=a}i=b;return h|0}}while(0);a=d|0;bc(a|0,32,1723232,(d=i,i=i+16|0,c[d>>2]=e,c[d+8>>2]=1722952,d)|0)|0;i=d;d=0;while(1){e=c[341192+(d<<4)>>2]|0;if((e|0)!=0){if((a8(e|0,a|0)|0)==0){h=d;g=15385;break}}e=d+1|0;if((e|0)<321){d=e}else{h=-1;g=15386;break}}if((g|0)==15386){i=b;return h|0}else if((g|0)==15385){i=b;return h|0}return 0}function BN(b){b=b|0;var c=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;c=(d[b+6|0]|0)+(d[b+5|0]|0)|0;f=a[b+4|0]|0;if(f<<24>>24==0){g=0;h=g>>c;return h|0}i=f&255;f=0;j=0;while(1){k=(((e[b+8+(j<<1)>>1]|0)>>>11&15)+1<<((j-1|0)>>>0<2?0:c))+f|0;l=j+1|0;if((l|0)<(i|0)){f=k;j=l}else{g=k;break}}h=g>>c;return h|0}function BO(e){e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;f=i;i=i+16|0;g=f|0;h=(d[e+6|0]|0)+(d[e+5|0]|0)|0;NM(g|0,0,16);j=a[e+4|0]|0;if(j<<24>>24==0){k=0;l=0;m=0;n=0;o=l+k|0;p=m+o|0;q=n+p|0;r=e+7|0;s=a[r]|0;t=s&4;u=t<<24>>24==0;v=q<<3;w=u?v:q;x=w>>h;i=f;return x|0}y=j&255;j=0;do{z=b[e+8+(j<<1)>>1]|0;c[g+((z&3)<<2)>>2]=((z&65535)>>>2&7)+1<<((j-1|0)>>>0<2?0:h);j=j+1|0;}while((j|0)<(y|0));k=c[g>>2]|0;l=c[g+4>>2]|0;m=c[g+8>>2]|0;n=c[g+12>>2]|0;o=l+k|0;p=m+o|0;q=n+p|0;r=e+7|0;s=a[r]|0;t=s&4;u=t<<24>>24==0;v=q<<3;w=u?v:q;x=w>>h;i=f;return x|0}function BP(a){a=a|0;var b=0;if(a>>>0>320){b=0}else{b=341192+(a<<4)|0}return b|0}function BQ(a){a=a|0;var b=0,d=0,e=0;L20757:do{if((a|0)==0){b=341192}else{d=a;while(1){if(d-341192>>4>>>0>=320){b=0;break L20757}e=d+16|0;if((c[e>>2]|0)==0){d=e}else{b=e;break}}}}while(0);return b|0}function BR(a){a=a|0;var b=0;if(a>>>0<341192|a>>>0>=346328){b=-1;return b|0}b=a-341192>>4;return b|0}function BS(a,b,e){a=a|0;b=b|0;e=e|0;var f=0;if(a>>>0>320){f=-38;return f|0}c[b>>2]=d[341197+(a<<4)|0]|0;c[e>>2]=d[341198+(a<<4)|0]|0;f=0;return f|0}function BT(d){d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;e=i;i=i+16|0;f=e|0;NM(f|0,0,16);if(d>>>0>320){g=-22;i=e;return g|0}h=a[341196+(d<<4)|0]|0;if(h<<24>>24==0){j=0;k=0;l=0;m=0}else{n=h&255;h=0;do{c[f+((b[341200+(d<<4)+(h<<1)>>1]&3)<<2)>>2]=1;h=h+1|0;}while((h|0)<(n|0));j=c[f>>2]|0;k=c[f+4>>2]|0;l=c[f+8>>2]|0;m=c[f+12>>2]|0}g=m+(l+(k+j))|0;i=e;return g|0}function BU(){var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;d=i;i=i+112|0;e=d|0;f=d+72|0;g=d+88|0;h=d+104|0;j=e|0;k=f|0;l=f+4|0;m=e+17|0;n=f+8|0;o=e+34|0;p=f+12|0;f=e+51|0;e=g;q=h|0;r=g|0;g=h+2|0;h=0;L20784:while(1){s=341192+(h<<4)|0;NM(j|0,0,68);c[k>>2]=j;c[l>>2]=m;c[n>>2]=o;c[p>>2]=f;NM(e|0,0,16);t=c[s>>2]|0;u=a[341196+(h<<4)|0]|0;if((t|0)==0){if(u<<24>>24!=0){v=15433;break}if((a[341197+(h<<4)|0]|0)!=0){v=15433;break}if((a[341198+(h<<4)|0]|0)!=0){v=15433;break}if((a[341199+(h<<4)|0]|0)!=0){v=15433;break}}else{if((a[t]|0)==0){v=15433;break}if(u<<24>>24==4){w=1}else{w=(h|0)==66|0}t=a[341199+(h<<4)|0]|0;if((w|0)==((t&255)>>>7&255|0)){x=0}else{v=15437;break}do{y=341200+(h<<4)+(x<<1)|0;if((x|0)<(u&255|0)){z=b[y>>1]|0;A=(z&65535)>>>2&7;if((t&4)==0){if((((A&65535)<<3)+8|0)>>>0<(((z&65535)>>>11&15)+1|0)>>>0){v=15445;break L20784}}else{if((A&65535)<((z&65535)>>>11&15)){v=15443;break L20784}}BJ(q,k,r,s,0,0,x,2,0);if((b[q>>1]|0)!=0){v=15454;break L20784}if((b[g>>1]|0)!=0){v=15453;break L20784}A=(1<<((z&65535)>>>11&15)+1)+65535&65535;b[g>>1]=A;b[q>>1]=A;BK(q,k,r,s,0,0,x,2)}else{if((b[y>>1]&32767)!=0){v=15440;break L20784}}x=x+1|0;}while(x>>>0<4)}s=h+1|0;if(s>>>0<321){h=s}else{v=15452;break}}if((v|0)==15453){AN(0,0,1721536,(B=i,i=i+24|0,c[B>>2]=1717440,c[B+8>>2]=1721080,c[B+16>>2]=1917,B)|0);i=B;cw()}else if((v|0)==15454){AN(0,0,1721536,(B=i,i=i+24|0,c[B>>2]=1717440,c[B+8>>2]=1721080,c[B+16>>2]=1917,B)|0);i=B;cw()}else if((v|0)==15433){AN(0,0,1721536,(B=i,i=i+24|0,c[B>>2]=1720200,c[B+8>>2]=1721080,c[B+16>>2]=1901,B)|0);i=B;cw()}else if((v|0)==15440){AN(0,0,1721536,(B=i,i=i+24|0,c[B>>2]=1719104,c[B+8>>2]=1721080,c[B+16>>2]=1908,B)|0);i=B;cw()}else if((v|0)==15443){AN(0,0,1721536,(B=i,i=i+24|0,c[B>>2]=1718432,c[B+8>>2]=1721080,c[B+16>>2]=1912,B)|0);i=B;cw()}else if((v|0)==15445){AN(0,0,1721536,(B=i,i=i+24|0,c[B>>2]=1717736,c[B+8>>2]=1721080,c[B+16>>2]=1914,B)|0);i=B;cw()}else if((v|0)==15452){i=d;return}else if((v|0)==15437){AN(0,0,1721536,(B=i,i=i+24|0,c[B>>2]=1719504,c[B+8>>2]=1721080,c[B+16>>2]=1902,B)|0);i=B;cw()}}function BV(b){b=b|0;var d=0,e=0,f=0,g=0,h=0;d=i;i=i+16|0;e=d|0;if(b>>>0>320){f=-1;i=d;return f|0}g=c[341192+(b<<4)>>2]|0;if((NL(g|0)|0)>>>0<2){f=-1;i=d;return f|0}b=e|0;y_(b,g,16)|0;g=e+((NL(b|0)|0)-2)|0;do{if((a8(g|0,1717232)|0)!=0){if((a8(g|0,1722952)|0)==0){break}else{f=-1}i=d;return f|0}}while(0);a[g]=a[g]^14;g=0;while(1){e=c[341192+(g<<4)>>2]|0;if((e|0)!=0){if((a8(e|0,b|0)|0)==0){f=g;h=15466;break}}e=g+1|0;if((e|0)<321){g=e}else{f=-1;h=15464;break}}if((h|0)==15464){i=d;return f|0}else if((h|0)==15466){i=d;return f|0}return 0}function BW(){var a=0,b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;a=i;i=i+152|0;b=a|0;d=a+120|0;e=a+144|0;f=bW(1630416,0,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0)|0;i=g;do{if((f|0)!=-1){h=a1(f|0,e|0,4)|0;bh(f|0)|0;if((h|0)!=4){break}j=c[e>>2]|0;i=a;return j|0}}while(0);f=bW(1714296,0,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0)|0;i=g;do{if((f|0)!=-1){h=a1(f|0,e|0,4)|0;bh(f|0)|0;if((h|0)!=4){break}j=c[e>>2]|0;i=a;return j|0}}while(0);e=d;f=d|0;h=b;b=c[555774]|0;k=c[555775]|0;if((c[85230]|0)>>>0>=121){AN(0,0,1688776,(g=i,i=i+24|0,c[g>>2]=1669328,c[g+8>>2]=1654440,c[g+16>>2]=71,g)|0);i=g;cw();return 0}if((b|0)==0&(k|0)==0){g=0;while(1){l=bk()|0;if((g|0)==(l|0)){m=2223104+((c[555774]&511)<<2)|0;c[m>>2]=(c[m>>2]|0)+1;g=l;continue}else{m=NU(c[555774]|0,c[555775]|0,1,0)|0;n=G;c[555774]=m;c[555775]=n;o=2223104+((m&511)<<2)|0;c[o>>2]=(c[o>>2]|0)+(((l-g|0)>>>0)%3294638521|0);o=0;if(n>>>0>o>>>0|n>>>0==o>>>0&m>>>0>64>>>0){break}else{g=l;continue}}}}else{g=0;while(1){l=bk()|0;if((g|0)==(l|0)){m=2223104+((c[555774]&511)<<2)|0;c[m>>2]=(c[m>>2]|0)+1;g=l;continue}else{m=NU(c[555774]|0,c[555775]|0,1,0)|0;o=G;c[555774]=m;c[555775]=o;n=2223104+((m&511)<<2)|0;c[n>>2]=(c[n>>2]|0)+(((l-g|0)>>>0)%3294638521|0);n=NV(m,o,b,k)|0;o=G;m=0;if(o>>>0>m>>>0|o>>>0==m>>>0&n>>>0>4>>>0){break}else{g=l;continue}}}}Cc(h,160)|0;Cf(h,2223104,2048);Cg(h,e);e=cV(c[f>>2]|0)|0;j=(cV(c[d+16>>2]|0)|0)+e|0;i=a;return j|0}function BX(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;j=f^d;k=g^e;l=-1;m=(e|0)>(l|0)|(e|0)==(l|0)&d>>>0>-1>>>0;l=NV(0,0,d,e)|0;n=m?d:l;l=m?e:G;m=-1;o=(g|0)>(m|0)|(g|0)==(m|0)&f>>>0>-1>>>0;m=NV(0,0,f,g)|0;p=o?f:m;m=o?g:G;o=AU(n,l,p,m)|0;q=G;if((o|0)==0&(q|0)==0){r=e;s=d;t=g;u=f}else{f=N0(n,l,o,q)|0;l=G;n=N0(p,m,o,q)|0;r=l;s=f;t=G;u=n}L20868:do{if((r|0)>(i|0)|(r|0)==(i|0)&s>>>0>h>>>0|((t|0)>(i|0)|(t|0)==(i|0)&u>>>0>h>>>0)){if((u|0)==0&(t|0)==0){v=0;w=1;x=1;break}else{y=1;z=0;A=0;B=1;C=t;D=u;E=r;F=s}while(1){H=N0(F,E,D,C)|0;I=G;J=y;K=(y|0)<0|0?-1:0;n=N2(H,I,J,K)|0;L=A;M=(A|0)<0|0?-1:0;f=NU(n,G,L,M)|0;n=G;N=z;O=(z|0)<0|0?-1:0;l=N2(H,I,N,O)|0;P=B;Q=(B|0)<0|0?-1:0;q=NU(l,G,P,Q)|0;l=G;if((n|0)>(i|0)|(n|0)==(i|0)&f>>>0>h>>>0|((l|0)>(i|0)|(l|0)==(i|0)&q>>>0>h>>>0)){break}l=N2(H,I,D,C)|0;n=G;o=NV(F,E,l,n)|0;m=f;f=q;if((F|0)==(l|0)&(E|0)==(n|0)){v=f;w=m;x=1;break L20868}else{A=y;y=m;B=z;z=f;E=C;C=G;F=D;D=o}}if((y|0)==0){R=I;S=H}else{o=NV(h,i,L,M)|0;f=N0(o,G,J,K)|0;R=G;S=f}if((z|0)==0){T=R;U=S}else{f=NV(h,i,P,Q)|0;o=N0(f,G,N,O)|0;f=G;m=R>>>0>f>>>0|R>>>0==f>>>0&S>>>0>o>>>0;T=m?f:R;U=m?o:S}o=N2(N<<1|0>>>31,O<<1|N>>>31,U,T)|0;m=NU(o,G,P,Q)|0;o=N2(m,G,D,C)|0;m=G;f=N2(F,E,N,O)|0;n=G;if(!(m>>>0>n>>>0|m>>>0==n>>>0&o>>>0>f>>>0)){v=z;w=y;x=0;break}f=N2(U,T,J,K)|0;o=NU(f,G,L,M)|0;f=N2(U,T,N,O)|0;n=NU(f,G,P,Q)|0;v=n;w=o;x=0}else{v=u;w=s;x=1}}while(0);s=0;c[a>>2]=(k|0)<(s|0)|(k|0)==(s|0)&j>>>0<0>>>0?-w|0:w;c[b>>2]=v;return x|0}function BY(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0;e=i;f=b;b=i;i=i+8|0;c[b>>2]=c[f>>2];c[b+4>>2]=c[f+4>>2];f=d;d=i;i=i+8|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];f=b|0;g=b+4|0;h=c[f>>2]|0;j=c[d>>2]|0;k=N2(j,(j|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=G;j=c[g>>2]|0;l=c[d+4>>2]|0;d=N2(l,(l|0)<0|0?-1:0,j,(j|0)<0|0?-1:0)|0;BX(f,g,k,h,d,G,2147483647,0)|0;d=b;b=a;a=c[d+4>>2]|0;c[b>>2]=c[d>>2];c[b+4>>2]=a;i=e;return}function BZ(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0;f=i;i=i+8|0;g=b;b=i;i=i+8|0;c[b>>2]=c[g>>2];c[b+4>>2]=c[g+4>>2];g=e;e=i;i=i+8|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];g=f|0;h=c[e+4>>2]|0;j=c[e>>2]|0;e=b;b=e|0;k=d[b]|d[b+1|0]<<8|d[b+2|0]<<16|d[b+3|0]<<24|0;b=e+4|0;e=d[b]|d[b+1|0]<<8|d[b+2|0]<<16|d[b+3|0]<<24|0;c[g>>2]=k;c[g+4>>2]=e;b=k;k=N2(b,(b|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=G;b=N2(e,(e|0)<0|0?-1:0,j,(j|0)<0|0?-1:0)|0;BX(g,g+4|0,k,h,b,G,2147483647,0)|0;b=a;a=c[g+4>>2]|0;c[b>>2]=c[g>>2];c[b+4>>2]=a;i=f;return}function B_(a,b,d){a=a|0;b=+b;d=d|0;var e=0,f=0,g=0,j=0,l=0.0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;e=i;i=i+8|0;f=e|0;h[k>>3]=b;g=c[k+4>>2]&2147483647;j=2146435072;if(g>>>0>j>>>0|g>>>0==j>>>0&(c[k>>2]|0)>>>0>0>>>0){c[a>>2]=0;c[a+4>>2]=0;i=e;return}l=+R(+b);if(l>2147483650.0){c[a>>2]=b<0.0?-1:1;c[a+4>>2]=0;i=e;return}j=~~(+aa(+(l+1.0e-20))/.6931471805599453);g=(j|0)>0;m=NW(1,0,61-j|0)|0;j=g?m:0;m=g?G:536870912;g=f;n=f+4|0;l=+Q(+((+(j>>>0)+ +(m|0)*4294967296.0)*b+.5));o=~~l>>>0;p=+R(l)>=1.0?l>0.0?(ah(+Q(l/4294967296.0),4294967295.0)|0)>>>0:~~+ab((l- +(~~l>>>0))/4294967296.0)>>>0:0;BX(g,n,o,p,j,m,d,(d|0)<0|0?-1:0)|0;if((c[g>>2]|0)==0){if(b!=0.0){q=15512}}else{if((c[n>>2]|0)==0&b!=0.0){q=15512}}do{if((q|0)==15512){if(!((d|0)>0&(d|0)!=2147483647)){break}r=2147483647;s=0;BX(g,n,o,p,j,m,r,s)|0}}while(0);m=a;a=c[f+4>>2]|0;c[m>>2]=c[f>>2];c[m+4>>2]=a;i=e;return}function B$(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;d=i;e=a;a=i;i=i+8|0;c[a>>2]=c[e>>2];c[a+4>>2]=c[e+4>>2];e=c[b+4>>2]|0;if((e|0)==0){f=0;i=d;return f|0}g=c[a+4>>2]|0;h=g;j=(g|0)<0|0?-1:0;g=c[a>>2]|0;a=g;k=(g|0)<0|0?-1:0;g=0;l=0;m=e;n=e;while(1){e=c[b+(g<<3)>>2]|0;o=n;p=(n|0)<0|0?-1:0;q=N2(o,p,e,(e|0)<0|0?-1:0)|0;r=G;s=c[b+(l<<3)>>2]|0;t=m;u=(m|0)<0|0?-1:0;v=N2(t,u,s,(s|0)<0|0?-1:0)|0;w=G;x=NU(v,w,q,r)|0;y=G;z=N2(t<<1|0>>>31,u<<1|t>>>31,o,p)|0;p=G;o=AV(x,y,h,j,z,p,3)|0;t=G;u=AV(x,y,h,j,z,p,2)|0;p=G;z=(((t|0)>(k|0)|(t|0)==(k|0)&o>>>0>a>>>0)&1)-(((p|0)<(k|0)|(p|0)==(k|0)&u>>>0<a>>>0)&1)|0;do{if((v|0)==(q|0)&(w|0)==(r|0)){if(!((m|0)==0|(n|0)==0)){A=0;break}if((s|0)==0|(e|0)==0){A=-2147483648;break}A=(s>>31)-(e>>31)|0}else{NV(v,w,q,r)|0;u=G^((m^n|0)<0|0?-1:0);p=u>>31|((u|0)<0|0?-1:0)<<1;((u|0)<0|0?-1:0)>>31|((u|0)<0|0?-1:0)<<1;A=p|1}}while(0);r=(ac(A,z)|0)>0?g:l;q=g+1|0;w=c[b+(q<<3)+4>>2]|0;if((w|0)==0){f=r;break}g=q;l=r;m=w;n=c[b+(r<<3)+4>>2]|0}i=d;return f|0}function B0(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;g=f>>3;if((f&7|0)==0){h=0}else{i=-1;return i|0}while(1){a[b+h|0]=h&255;f=h+1|0;if((f|0)<256){h=f}else{j=0;k=0;l=0;break}}while(1){h=(k|0)==(g|0)?0:k;f=b+l|0;m=a[f]|0;n=(m+j&255)+(a[e+h|0]|0)&255;o=b+(n&255)|0;p=a[o]|0;a[o]=m;a[f]=p;p=l+1|0;if((p|0)<256){j=n;k=h+1|0;l=p}else{break}}c[b+256>>2]=1;c[b+260>>2]=d[b+1|0]|0;i=0;return i|0}function B1(b,d,e,f,g,h){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;h=b+256|0;g=c[h>>2]|0;i=b+260|0;j=c[i>>2]&255;k=g&255;if((f|0)<=0){l=j;m=k;c[h>>2]=m;n=l&255;c[i>>2]=n;return}o=g&255;g=(o+(f+255&255)&255)+1&255;p=o;o=j;j=e;e=d;d=f;f=k;q=a[b+k|0]|0;while(1){k=d-1|0;r=b+(o&255)|0;s=a[r]|0;t=s+q&255;a[r]=q;a[b+f|0]=s;if((j|0)==0){u=a[b+(t&255)|0]|0;v=0}else{u=a[b+(t&255)|0]^a[j];v=j+1|0}a[e]=u;t=p+1&255;s=t&255;r=a[b+s|0]|0;w=r+o&255;if((k|0)>0){p=t;o=w;j=v;e=e+1|0;d=k;f=s;q=r}else{break}}l=w;m=g&255;c[h>>2]=m;n=l&255;c[i>>2]=n;return}function B2(a){a=a|0;var b=0;if(a>>>0>9){b=0}else{b=84680+(a*20|0)|0}return b|0}function B3(a){a=a|0;var b=0,c=0,d=0;b=0;while(1){if((b|0)>=10){c=-1;d=15556;break}if((a8(84680+(b*20|0)|0,a|0)|0)==0){c=b;d=15555;break}else{b=b+1|0}}if((d|0)==15556){return c|0}else if((d|0)==15555){return c|0}return 0}function B4(a){a=a|0;var b=0;do{if(a>>>0>9){b=-1}else{if((a-5|0)>>>0>=5){b=a;break}b=c[84696+(a*20|0)>>2]|0}}while(0);return b|0}function B5(a){a=a|0;var b=0;do{if(a>>>0>9){b=-1}else{if((a-5|0)>>>0<5){b=a;break}b=c[84696+(a*20|0)>>2]|0}}while(0);return b|0}function B6(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0;e=i;i=i+24|0;f=e|0;if((d|0)<0){bc(a|0,b|0,1630248,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0)|0;i=g;i=e;return a|0}if((d|0)>=10){i=e;return a|0}h=f|0;j=84680+(d*20|0)|0;c[h>>2]=c[j>>2];c[h+4>>2]=c[j+4>>2];c[h+8>>2]=c[j+8>>2];c[h+12>>2]=c[j+12>>2];c[h+16>>2]=c[j+16>>2];j=c[f+8>>2]|0;bc(a|0,b|0,1714216,(g=i,i=i+16|0,c[g>>2]=h,c[g+8>>2]=j,g)|0)|0;i=g;i=e;return a|0}function B7(a){a=a|0;var b=0;if(a>>>0>9){b=0;return b|0}b=c[84688+(a*20|0)>>2]>>3;return b|0}function B8(a){a=a|0;var b=0;if(a>>>0>9){b=0;return b|0}b=c[84692+(a*20|0)>>2]|0;return b|0}function B9(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0;if(e>>>0>9){g=-22;return g|0}h=c[84688+(e*20|0)>>2]>>3;if((h|0)==0|(d|0)<1|(b|0)<1){g=-22;return g|0}if((f|0)==0){i=d+31&-32;j=1}else{i=d;j=f}if((2147483647/(j|0)|0|0)<(b|0)){g=-22;return g|0}f=N2(i,(i|0)<0|0?-1:0,b,(b|0)<0|0?-1:0)|0;d=G;k=(2147483647-(ac(j,b)|0)|0)/(h|0)|0;l=(k|0)<0|0?-1:0;if((d|0)>(l|0)|(d|0)==(l|0)&f>>>0>k>>>0){g=-22;return g|0}k=(e-5|0)>>>0<5;e=ac(i,h)|0;if(k){m=j-1+e&-j}else{m=j-1+(ac(e,b)|0)&-j}if((a|0)!=0){c[a>>2]=m}if(!k){g=m;return g|0}g=ac(m,b)|0;return g|0}function Ca(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;if(f>>>0>9){g=1;h=0;i=0;j=15605}else{k=(f-5|0)>>>0<5;l=k?e:1;m=ac(c[84688+(f*20|0)>>2]>>3,k?1:e)|0;e=ac(m,d)|0;if((f|0)==0){n=-128;o=l;p=m;q=e}else{g=l;h=m;i=e;j=15605}}if((j|0)==15605){n=(f|0)==5?-128:0;o=g;p=h;q=i}i=ac(p,b)|0;if((o|0)>0){r=0}else{return 0}do{NM((c[a+(r<<2)>>2]|0)+i|0,n|0,q|0);r=r+1|0;}while((r|0)<(o|0));return 0}function Cb(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;if(h>>>0>9){i=0;j=g;k=1}else{l=(h-5|0)>>>0<5;i=c[84688+(h*20|0)>>2]>>3;j=l?1:g;k=l?g:1}g=ac(i,j)|0;j=ac(g,f)|0;f=ac(g,d)|0;d=ac(g,e)|0;e=c[a>>2]|0;g=c[b>>2]|0;if(e>>>0<g>>>0){m=g-e|0}else{m=e-g|0}i=(k|0)>0;if((m|0)<(j|0)){if(i){n=1;o=e;p=g}else{return 0}while(1){NP(o+f|0,p+d|0,j|0);if((n|0)>=(k|0)){break}m=c[a+(n<<2)>>2]|0;l=c[b+(n<<2)>>2]|0;n=n+1|0;o=m;p=l}return 0}else{if(i){q=1;r=e;s=g}else{return 0}while(1){g=r+f|0;e=s+d|0;NN(g|0,e|0,j)|0;if((q|0)>=(k|0)){break}e=c[a+(q<<2)>>2]|0;g=c[b+(q<<2)>>2]|0;q=q+1|0;r=e;s=g}return 0}return 0}function Cc(b,d){b=b|0;d=d|0;var e=0;a[b|0]=d>>>5&255;if((d|0)==256){c[b+80>>2]=1779033703;c[b+84>>2]=-1150833019;c[b+88>>2]=1013904242;c[b+92>>2]=-1521486534;c[b+96>>2]=1359893119;c[b+100>>2]=-1694144372;c[b+104>>2]=528734635;c[b+108>>2]=1541459225;c[b+112>>2]=36}else if((d|0)==224){c[b+80>>2]=-1056596264;c[b+84>>2]=914150663;c[b+88>>2]=812702999;c[b+92>>2]=-150054599;c[b+96>>2]=-4191439;c[b+100>>2]=1750603025;c[b+104>>2]=1694076839;c[b+108>>2]=-1090891868;c[b+112>>2]=36}else if((d|0)==160){c[b+80>>2]=1732584193;c[b+84>>2]=-271733879;c[b+88>>2]=-1732584194;c[b+92>>2]=271733878;c[b+96>>2]=-1009589776;c[b+112>>2]=6}else{e=-1;return e|0}d=b+8|0;c[d>>2]=0;c[d+4>>2]=0;e=0;return e|0}function Cd(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0;e=i;i=i+320|0;f=e|0;g=c[a>>2]|0;h=a+4|0;j=c[h>>2]|0;k=a+8|0;l=c[k>>2]|0;m=a+12|0;n=c[m>>2]|0;o=a+16|0;p=c[o>>2]|0;q=0;r=g;s=j;t=l;u=n;v=p;w=(n^l)&j^n;do{x=b+(q<<2)|0;y=cV(d[x]|d[x+1|0]<<8|d[x+2|0]<<16|d[x+3|0]<<24|0)|0;c[f+(q<<2)>>2]=y;x=(r<<5|r>>>27)+1518500249+v+w+y|0;y=s<<30|s>>>2;z=q+1|0;A=b+(z<<2)|0;B=cV(d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24|0)|0;c[f+(z<<2)>>2]=B;z=u+1518500249+((y^t)&r^t)+B+(x<<5|x>>>27)|0;B=r<<30|r>>>2;A=q+2|0;C=b+(A<<2)|0;D=cV(d[C]|d[C+1|0]<<8|d[C+2|0]<<16|d[C+3|0]<<24|0)|0;c[f+(A<<2)>>2]=D;A=t+1518500249+(x&(y^B)^y)+D+(z<<5|z>>>27)|0;v=x<<30|x>>>2;x=q+3|0;D=b+(x<<2)|0;C=cV(d[D]|d[D+1|0]<<8|d[D+2|0]<<16|d[D+3|0]<<24|0)|0;c[f+(x<<2)>>2]=C;s=y+1518500249+C+(z&(v^B)^B)+(A<<5|A>>>27)|0;u=z<<30|z>>>2;z=q+4|0;C=b+(z<<2)|0;y=cV(d[C]|d[C+1|0]<<8|d[C+2|0]<<16|d[C+3|0]<<24|0)|0;c[f+(z<<2)>>2]=y;r=B+1518500249+y+(A&(u^v)^v)+(s<<5|s>>>27)|0;t=A<<30|A>>>2;q=q+5|0;w=(u^t)&s^u}while(q>>>0<15);q=b+60|0;b=cV(d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0)|0;c[f+60>>2]=b;q=(r<<5|r>>>27)+1518500249+v+w+b|0;w=s<<30|s>>>2;s=c[f+52>>2]|0;v=c[f+32>>2]|0;A=c[f+8>>2]|0;y=v^s^A^c[f>>2];B=y<<1|y>>>31;c[f+64>>2]=B;y=u+1518500249+((w^t)&r^t)+(q<<5|q>>>27)+B|0;u=r<<30|r>>>2;r=c[f+56>>2]|0;z=c[f+36>>2]|0;C=c[f+12>>2]|0;x=z^r^C^c[f+4>>2];D=x<<1|x>>>31;c[f+68>>2]=D;x=t+1518500249+(q&(w^u)^w)+(y<<5|y>>>27)+D|0;t=q<<30|q>>>2;q=c[f+40>>2]|0;E=c[f+16>>2]|0;F=q^b^E^A;A=F<<1|F>>>31;c[f+72>>2]=A;F=w+1518500249+(y&(t^u)^u)+(x<<5|x>>>27)+A|0;w=y<<30|y>>>2;y=c[f+20>>2]|0;G=c[f+44>>2]^B^y^C;C=G<<1|G>>>31;c[f+76>>2]=C;G=20;H=u+1518500249+(x&(w^t)^t)+(F<<5|F>>>27)+C|0;u=F;F=x<<30|x>>>2;x=w;w=t;t=D;D=E;E=A;A=c[f+28>>2]|0;I=y;y=C;C=v;v=b;b=z;z=B;B=s;s=r;r=q;while(1){q=c[f+(G-8<<2)>>2]|0;J=c[f+(G-14<<2)>>2]|0;K=q^t^J^D;L=K<<1|K>>>31;c[f+(G<<2)>>2]=L;K=(H<<5|H>>>27)+1859775393+(F^u^x)+w+L|0;M=u<<30|u>>>2;N=B^E^A^I;O=N<<1|N>>>31;c[f+(G+1<<2)>>2]=O;N=x+1859775393+(F^H^M)+(K<<5|K>>>27)+O|0;P=H<<30|H>>>2;Q=s^y^C^J;J=Q<<1|Q>>>31;c[f+(G+2<<2)>>2]=J;Q=F+1859775393+(M^P^K)+(N<<5|N>>>27)+J|0;R=K<<30|K>>>2;K=v^L^b^A;S=K<<1|K>>>31;c[f+(G+3<<2)>>2]=S;T=M+1859775393+(R^P^N)+(Q<<5|Q>>>27)+S|0;U=N<<30|N>>>2;N=z^O^r^C;M=N<<1|N>>>31;c[f+(G+4<<2)>>2]=M;V=P+1859775393+(U^R^Q)+(T<<5|T>>>27)+M|0;W=Q<<30|Q>>>2;Q=G+5|0;if(Q>>>0>=40){break}P=c[f+(G-2<<2)>>2]|0;N=c[f+(G-1<<2)>>2]|0;K=c[f+(G-5<<2)>>2]|0;G=Q;H=V;u=T;F=W;x=U;w=R;t=J;D=b;E=S;A=q;I=r;y=M;C=B;v=L;b=s;z=O;B=P;s=N;r=K}r=40;s=V;V=T;T=W;W=U;U=R;R=c[f+148>>2]|0;B=c[f+96>>2]|0;z=c[f+152>>2]|0;b=c[f+108>>2]|0;v=c[f+100>>2]|0;C=c[f+156>>2]|0;y=c[f+112>>2]|0;I=c[f+140>>2]|0;A=c[f+116>>2]|0;E=c[f+144>>2]|0;while(1){D=c[f+(r-8<<2)>>2]|0;t=c[f+(r-14<<2)>>2]|0;w=D^R^t^B;x=w<<1|w>>>31;c[f+(r<<2)>>2]=x;w=(s<<5|s>>>27)-1894007588+U+((T|V)&W|T&V)+x|0;F=V<<30|V>>>2;u=c[f+(r-7<<2)>>2]|0;H=u^z^b^v;G=H<<1|H>>>31;c[f+(r+1<<2)>>2]=G;H=W-1894007588+((F|s)&T|F&s)+(w<<5|w>>>27)+G|0;K=s<<30|s>>>2;N=c[f+(r-6<<2)>>2]|0;P=N^C^y^t;t=P<<1|P>>>31;c[f+(r+2<<2)>>2]=t;P=T-1894007588+((w|K)&F|w&K)+(H<<5|H>>>27)+t|0;O=w<<30|w>>>2;w=I^x^A^b;L=w<<1|w>>>31;c[f+(r+3<<2)>>2]=L;w=F-1894007588+((H|O)&K|H&O)+(P<<5|P>>>27)+L|0;F=H<<30|H>>>2;H=c[f+(r-10<<2)>>2]|0;M=E^G^H^y;q=M<<1|M>>>31;c[f+(r+4<<2)>>2]=q;M=K-1894007588+((P|F)&O|P&F)+(w<<5|w>>>27)+q|0;K=P<<30|P>>>2;P=r+5|0;if(P>>>0<60){r=P;s=M;V=w;T=K;W=F;U=O;R=t;B=A;z=L;b=D;v=H;C=q;y=u;I=x;A=N;E=G}else{X=60;Y=M;Z=w;_=K;$=F;aa=O;break}}do{E=c[f+(X-14<<2)>>2]|0;A=c[f+(X-8<<2)>>2]^c[f+(X-3<<2)>>2]^E^c[f+(X-16<<2)>>2];I=A<<1|A>>>31;c[f+(X<<2)>>2]=I;A=(Y<<5|Y>>>27)-899497514+(_^Z^$)+aa+I|0;y=Z<<30|Z>>>2;C=c[f+(X-13<<2)>>2]|0;v=c[f+(X-7<<2)>>2]^c[f+(X-2<<2)>>2]^C^c[f+(X-15<<2)>>2];b=v<<1|v>>>31;c[f+(X+1<<2)>>2]=b;v=$-899497514+(_^Y^y)+(A<<5|A>>>27)+b|0;z=Y<<30|Y>>>2;B=c[f+(X-12<<2)>>2]|0;R=c[f+(X-6<<2)>>2]^c[f+(X-1<<2)>>2]^B^E;E=R<<1|R>>>31;c[f+(X+2<<2)>>2]=E;R=_-899497514+(y^z^A)+(v<<5|v>>>27)+E|0;aa=A<<30|A>>>2;A=c[f+(X-5<<2)>>2]^I^c[f+(X-11<<2)>>2]^C;C=A<<1|A>>>31;c[f+(X+3<<2)>>2]=C;Z=y-899497514+(aa^z^v)+(R<<5|R>>>27)+C|0;$=v<<30|v>>>2;v=c[f+(X-4<<2)>>2]^b^c[f+(X-10<<2)>>2]^B;B=v<<1|v>>>31;c[f+(X+4<<2)>>2]=B;Y=z-899497514+($^aa^R)+(Z<<5|Z>>>27)+B|0;_=R<<30|R>>>2;X=X+5|0;}while(X>>>0<80);c[a>>2]=g+Y;c[h>>2]=j+Z;c[k>>2]=l+_;c[m>>2]=n+$;c[o>>2]=p+aa;i=e;return}function Ce(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0;e=i;i=i+256|0;f=e|0;g=a+4|0;h=a+8|0;j=a+12|0;k=a+16|0;l=a+20|0;m=a+24|0;n=a+28|0;o=0;p=c[a>>2]|0;q=c[g>>2]|0;r=c[h>>2]|0;s=c[j>>2]|0;t=c[k>>2]|0;u=c[l>>2]|0;v=c[m>>2]|0;w=c[n>>2]|0;do{x=b+(o<<2)|0;y=cV(d[x]|d[x+1|0]<<8|d[x+2|0]<<16|d[x+3|0]<<24|0)|0;c[f+(o<<2)>>2]=y;x=((t<<26|t>>>6)^(t<<21|t>>>11)^(t<<7|t>>>25))+w+((v^u)&t^v)+y+(c[1973888+(o<<2)>>2]|0)|0;y=x+s|0;z=((r|q)&p|r&q)+((p<<30|p>>>2)^(p<<19|p>>>13)^(p<<10|p>>>22))+x|0;x=o|1;A=b+(x<<2)|0;B=cV(d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24|0)|0;c[f+(x<<2)>>2]=B;A=B+v+(c[1973888+(x<<2)>>2]|0)+(y&(u^t)^u)+((y<<26|y>>>6)^(y<<21|y>>>11)^(y<<7|y>>>25))|0;x=A+r|0;B=((z<<30|z>>>2)^(z<<19|z>>>13)^(z<<10|z>>>22))+(z&(q|p)|q&p)+A|0;A=o|2;C=b+(A<<2)|0;D=cV(d[C]|d[C+1|0]<<8|d[C+2|0]<<16|d[C+3|0]<<24|0)|0;c[f+(A<<2)>>2]=D;C=D+u+(c[1973888+(A<<2)>>2]|0)+(x&(y^t)^t)+((x<<26|x>>>6)^(x<<21|x>>>11)^(x<<7|x>>>25))|0;A=C+q|0;D=((B<<30|B>>>2)^(B<<19|B>>>13)^(B<<10|B>>>22))+(B&(z|p)|z&p)+C|0;C=o|3;E=b+(C<<2)|0;F=cV(d[E]|d[E+1|0]<<8|d[E+2|0]<<16|d[E+3|0]<<24|0)|0;c[f+(C<<2)>>2]=F;E=F+t+(c[1973888+(C<<2)>>2]|0)+(A&(x^y)^y)+((A<<26|A>>>6)^(A<<21|A>>>11)^(A<<7|A>>>25))|0;C=E+p|0;F=((D<<30|D>>>2)^(D<<19|D>>>13)^(D<<10|D>>>22))+(D&(B|z)|B&z)+E|0;E=o|4;G=b+(E<<2)|0;H=cV(d[G]|d[G+1|0]<<8|d[G+2|0]<<16|d[G+3|0]<<24|0)|0;c[f+(E<<2)>>2]=H;G=H+y+(c[1973888+(E<<2)>>2]|0)+(C&(A^x)^x)+((C<<26|C>>>6)^(C<<21|C>>>11)^(C<<7|C>>>25))|0;w=G+z|0;s=((F<<30|F>>>2)^(F<<19|F>>>13)^(F<<10|F>>>22))+(F&(D|B)|D&B)+G|0;G=o|5;z=b+(G<<2)|0;E=cV(d[z]|d[z+1|0]<<8|d[z+2|0]<<16|d[z+3|0]<<24|0)|0;c[f+(G<<2)>>2]=E;z=E+x+(c[1973888+(G<<2)>>2]|0)+(w&(C^A)^A)+((w<<26|w>>>6)^(w<<21|w>>>11)^(w<<7|w>>>25))|0;v=z+B|0;r=((s<<30|s>>>2)^(s<<19|s>>>13)^(s<<10|s>>>22))+(s&(F|D)|F&D)+z|0;z=o|6;B=b+(z<<2)|0;G=cV(d[B]|d[B+1|0]<<8|d[B+2|0]<<16|d[B+3|0]<<24|0)|0;c[f+(z<<2)>>2]=G;B=G+A+(c[1973888+(z<<2)>>2]|0)+(v&(w^C)^C)+((v<<26|v>>>6)^(v<<21|v>>>11)^(v<<7|v>>>25))|0;u=B+D|0;q=((r<<30|r>>>2)^(r<<19|r>>>13)^(r<<10|r>>>22))+(r&(s|F)|s&F)+B|0;B=o|7;D=b+(B<<2)|0;z=cV(d[D]|d[D+1|0]<<8|d[D+2|0]<<16|d[D+3|0]<<24|0)|0;c[f+(B<<2)>>2]=z;D=z+C+(c[1973888+(B<<2)>>2]|0)+(u&(v^w)^w)+((u<<26|u>>>6)^(u<<21|u>>>11)^(u<<7|u>>>25))|0;t=D+F|0;p=((q<<30|q>>>2)^(q<<19|q>>>13)^(q<<10|q>>>22))+(q&(r|s)|r&s)+D|0;o=o+8|0;}while(o>>>0<9);o=16;b=p;p=q;q=r;r=s;s=t;t=u;u=v;v=w;w=c[f>>2]|0;D=c[f+8>>2]|0;F=c[f+12>>2]|0;B=c[f+16>>2]|0;C=c[f+20>>2]|0;z=c[f+24>>2]|0;A=c[f+28>>2]|0;G=c[f+32>>2]|0;while(1){x=c[f+(o-15<<2)>>2]|0;E=c[f+(o-2<<2)>>2]|0;y=(c[f+(o-7<<2)>>2]|0)+w+((x<<14|x>>>18)^x>>>3^(x<<25|x>>>7))+((E<<13|E>>>19)^E>>>10^(E<<15|E>>>17))|0;c[f+(o<<2)>>2]=y;H=((s<<26|s>>>6)^(s<<21|s>>>11)^(s<<7|s>>>25))+v+((u^t)&s^u)+(c[1973888+(o<<2)>>2]|0)+y|0;I=H+r|0;J=((q|p)&b|q&p)+((b<<30|b>>>2)^(b<<19|b>>>13)^(b<<10|b>>>22))+H|0;H=o|1;K=c[f+(o-1<<2)>>2]|0;L=c[f+(o-6<<2)>>2]|0;M=L+x+((D<<14|D>>>18)^D>>>3^(D<<25|D>>>7))+((K<<13|K>>>19)^K>>>10^(K<<15|K>>>17))|0;c[f+(H<<2)>>2]=M;x=(c[1973888+(H<<2)>>2]|0)+u+(I&(t^s)^t)+M+((I<<26|I>>>6)^(I<<21|I>>>11)^(I<<7|I>>>25))|0;H=x+q|0;N=((J<<30|J>>>2)^(J<<19|J>>>13)^(J<<10|J>>>22))+(J&(p|b)|p&b)+x|0;x=o|2;O=c[f+(o-5<<2)>>2]|0;P=O+D+((F<<14|F>>>18)^F>>>3^(F<<25|F>>>7))+((y<<13|y>>>19)^y>>>10^(y<<15|y>>>17))|0;c[f+(x<<2)>>2]=P;Q=(c[1973888+(x<<2)>>2]|0)+t+(H&(I^s)^s)+P+((H<<26|H>>>6)^(H<<21|H>>>11)^(H<<7|H>>>25))|0;x=Q+p|0;R=((N<<30|N>>>2)^(N<<19|N>>>13)^(N<<10|N>>>22))+(N&(J|b)|J&b)+Q|0;Q=o|3;S=c[f+(o-4<<2)>>2]|0;T=S+F+((B<<14|B>>>18)^B>>>3^(B<<25|B>>>7))+((M<<13|M>>>19)^M>>>10^(M<<15|M>>>17))|0;c[f+(Q<<2)>>2]=T;M=(c[1973888+(Q<<2)>>2]|0)+s+T+(x&(H^I)^I)+((x<<26|x>>>6)^(x<<21|x>>>11)^(x<<7|x>>>25))|0;Q=M+b|0;U=((R<<30|R>>>2)^(R<<19|R>>>13)^(R<<10|R>>>22))+(R&(N|J)|N&J)+M|0;M=o|4;V=c[f+(o-3<<2)>>2]|0;W=V+B+((C<<14|C>>>18)^C>>>3^(C<<25|C>>>7))+((P<<13|P>>>19)^P>>>10^(P<<15|P>>>17))|0;c[f+(M<<2)>>2]=W;P=(c[1973888+(M<<2)>>2]|0)+I+W+(Q&(x^H)^H)+((Q<<26|Q>>>6)^(Q<<21|Q>>>11)^(Q<<7|Q>>>25))|0;X=P+J|0;Y=((U<<30|U>>>2)^(U<<19|U>>>13)^(U<<10|U>>>22))+(U&(R|N)|R&N)+P|0;P=o|5;J=E+C+((z<<14|z>>>18)^z>>>3^(z<<25|z>>>7))+((T<<13|T>>>19)^T>>>10^(T<<15|T>>>17))|0;c[f+(P<<2)>>2]=J;T=(c[1973888+(P<<2)>>2]|0)+H+J+(X&(Q^x)^x)+((X<<26|X>>>6)^(X<<21|X>>>11)^(X<<7|X>>>25))|0;Z=T+N|0;_=((Y<<30|Y>>>2)^(Y<<19|Y>>>13)^(Y<<10|Y>>>22))+(Y&(U|R)|U&R)+T|0;T=o|6;N=K+z+((A<<14|A>>>18)^A>>>3^(A<<25|A>>>7))+((W<<13|W>>>19)^W>>>10^(W<<15|W>>>17))|0;c[f+(T<<2)>>2]=N;W=(c[1973888+(T<<2)>>2]|0)+x+N+(Z&(X^Q)^Q)+((Z<<26|Z>>>6)^(Z<<21|Z>>>11)^(Z<<7|Z>>>25))|0;$=W+R|0;aa=((_<<30|_>>>2)^(_<<19|_>>>13)^(_<<10|_>>>22))+(_&(Y|U)|Y&U)+W|0;W=o|7;R=y+A+((G<<14|G>>>18)^G>>>3^(G<<25|G>>>7))+((J<<13|J>>>19)^J>>>10^(J<<15|J>>>17))|0;c[f+(W<<2)>>2]=R;J=(c[1973888+(W<<2)>>2]|0)+Q+R+($&(Z^X)^X)+(($<<26|$>>>6)^($<<21|$>>>11)^($<<7|$>>>25))|0;ab=J+U|0;ac=((aa<<30|aa>>>2)^(aa<<19|aa>>>13)^(aa<<10|aa>>>22))+(aa&(_|Y)|_&Y)+J|0;J=o+8|0;if(J>>>0<57){o=J;b=ac;p=aa;q=_;r=Y;s=ab;t=$;u=Z;v=X;w=G;D=L;F=O;B=S;C=V;z=E;A=K;G=y}else{break}}G=c[g>>2]|0;A=c[h>>2]|0;z=c[j>>2]|0;C=c[k>>2]|0;B=c[l>>2]|0;F=c[m>>2]|0;D=c[n>>2]|0;c[a>>2]=(c[a>>2]|0)+ac;c[g>>2]=G+aa;c[h>>2]=A+_;c[j>>2]=z+Y;c[k>>2]=C+ab;c[l>>2]=B+$;c[m>>2]=F+Z;c[n>>2]=D+X;i=e;return}function Cf(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0;e=a+8|0;f=c[e>>2]|0;g=f&63;h=NU(f,c[e+4>>2]|0,d,0)|0;c[e>>2]=h;c[e+4>>2]=G;do{if((g+d|0)>>>0>63){e=a+16+g|0;h=64-g|0;NN(e|0,b|0,h)|0;e=a+112|0;f=a+80|0;c$[c[e>>2]&127](f,a+16|0);if((h+63|0)>>>0<d>>>0){i=h}else{j=0;k=h;break}while(1){c$[c[e>>2]&127](f,b+i|0);h=i+64|0;if((i+127|0)>>>0<d>>>0){i=h}else{j=0;k=h;break}}}else{j=g;k=0}}while(0);g=a+16+j|0;j=b+k|0;b=d-k|0;NN(g|0,j|0,b)|0;return}function Cg(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0;f=i;i=i+8|0;g=f|0;h=b+8|0;j=c[h>>2]|0;k=c[h+4>>2]|0;l=cV(j<<3|0>>>29|0)|0;c[g>>2]=((k>>>13|0<<19)<<8|0>>>24)&65280|(k>>>21|0<<11)&255|((j>>>29|k<<3)<<8|0>>>24|(k>>>5|0<<27)&255)<<16;c[g+4>>2]=l;l=j&63;m=NU(j,k,1,0)|0;c[h>>2]=m;c[h+4>>2]=G;if((l+1|0)>>>0>63){m=b+16+l|0;k=64-l|0;NN(m|0,1630056,k)|0;c$[c[b+112>>2]&127](b+80|0,b+16|0);n=0;o=k}else{n=l;o=0}l=b+16+n|0;n=1630056+o|0;k=1-o|0;NN(l|0,n|0,k)|0;k=c[h>>2]|0;n=c[h+4>>2]|0;if((k&63|0)==56&(n&0|0)==0){p=n;q=k}else{l=b+112|0;o=b+80|0;m=b+16|0;j=n;n=k;while(1){k=n&63;r=NU(n,j,1,0)|0;c[h>>2]=r;c[h+4>>2]=G;if((k+1|0)>>>0>63){r=b+16+k|0;s=64-k|0;NN(r|0,3119168,s)|0;c$[c[l>>2]&127](o,m);t=0;u=s}else{t=k;u=0}k=b+16+t|0;s=3119168+u|0;r=1-u|0;NN(k|0,s|0,r)|0;r=c[h>>2]|0;s=c[h+4>>2]|0;if((r&63|0)==56&(s&0|0)==0){p=s;q=r;break}else{j=s;n=r}}}n=g;g=q&63;j=NU(q,p,8,0)|0;c[h>>2]=j;c[h+4>>2]=G;do{if((g+8|0)>>>0>63){h=b+16+g|0;j=64-g|0;NN(h|0,n|0,j)|0;h=b+112|0;p=b+80|0;c$[c[h>>2]&127](p,b+16|0);if((j+63|0)>>>0<8){v=j}else{w=0;x=j;break}while(1){c$[c[h>>2]&127](p,n+v|0);j=v+64|0;if((v+127|0)>>>0<8){v=j}else{w=0;x=j;break}}}else{w=g;x=0}}while(0);g=b+16+w|0;w=n+x|0;n=8-x|0;NN(g|0,w|0,n)|0;n=b|0;if((a[n]|0)==0){i=f;return}else{z=0}do{w=e+(z<<2)|0;y=cV(c[b+80+(z<<2)>>2]|0)|0;a[w]=y&255;y=y>>8;a[w+1|0]=y&255;y=y>>8;a[w+2|0]=y&255;y=y>>8;a[w+3|0]=y&255;z=z+1|0;}while((z|0)<(d[n]|0));i=f;return}function Ch(){var a=0,b=0,d=0,e=0;a=i;i=i+8|0;b=a|0;cb(b|0,0)|0;d=c[b>>2]|0;e=N2(d,(d|0)<0|0?-1:0,1e6,0)|0;d=c[b+4>>2]|0;b=NU(e,G,d,(d|0)<0|0?-1:0)|0;d=G;i=a;return(G=d,b)|0}function Ci(a){a=a|0;var b=0,d=0,e=0;b=i;i=i+8|0;d=b|0;c[d>>2]=(a>>>0)/1e6|0;c[d+4>>2]=((a>>>0)%1e6|0)*1e3|0;while(1){if((bK(d|0,d|0)|0)>=0){e=15675;break}if((c[(ch()|0)>>2]|0)!=4){e=15674;break}}if((e|0)==15675){i=b;return 0}else if((e|0)==15674){i=b;return 0}return 0}function Cj(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=i;f=c[a+16>>2]|0;g=c[a+4>>2]|0;h=g&1;j=(c[a>>2]|0)+d|0;do{if((h|0)==0){k=j}else{if((f|0)==60){l=35964;m=4}else if((f|0)==30){l=17982;m=2}else{k=j;break}k=(ac(((((j|0)%(l|0)|0)-m|0)/((l>>>0)/10|0|0)|0)+(((j|0)/(l|0)|0)*9|0)|0,m)|0)+j|0}}while(0);if((k|0)<0){n=g&4;o=-k|0}else{n=0;o=k}k=(o|0)/(f*3600|0|0)|0;if((g&2|0)==0){p=k}else{p=(k|0)%24|0}bc(b|0,16,1629920,(k=i,i=i+48|0,c[k>>2]=(n|0)!=0?1714104:3119104,c[k+8>>2]=p,c[k+16>>2]=((o|0)/(f*60|0|0)|0|0)%60|0,c[k+24>>2]=((o|0)/(f|0)|0|0)%60|0,c[k+32>>2]=h|58,c[k+40>>2]=(o|0)%(f|0)|0,k)|0)|0;i=k;i=e;return b|0}function Ck(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=i;d=a;a=i;i=i+8|0;c[a>>2]=c[d>>2];c[a+4>>2]=c[d+4>>2];d=c[a>>2]|0;e=c[a+4>>2]|0;if((e|0)==0|(d|0)==0){f=-1}else{f=(((e|0)/2|0)+d|0)/(e|0)|0}e=0;while(1){if(e>>>0>=5){g=-1;h=15691;break}if((c[299800+(e<<2)>>2]|0)==(f|0)){g=0;h=15692;break}else{e=e+1|0}}if((h|0)==15692){i=b;return g|0}else if((h|0)==15691){i=b;return g|0}return 0}function Cl(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;g=i;h=b;b=i;i=i+8|0;c[b>>2]=c[h>>2];c[b+4>>2]=c[h+4>>2];NM(a|0,0,20);c[a>>2]=e;c[a+4>>2]=d;e=b;h=a+8|0;j=c[e>>2]|0;k=c[e+4>>2]|0;c[h>>2]=j;c[h+4>>2]=k;h=j;j=c[b+4>>2]|0;b=k;do{if((j|0)==0|(h|0)==0){c[a+16>>2]=-1;l=-1}else{k=(((j|0)/2|0)+h|0)/(j|0)|0;c[a+16>>2]=k;if((k|0)!=0){l=k;break}AN(f,16,1628776,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=-22;i=g;return n|0}}while(0);if((d&1|0)==0|(l|0)==30|(l|0)==60){o=0}else{AN(f,16,1622672,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=-22;i=g;return n|0}while(1){if(o>>>0>=5){break}if((c[299800+(o<<2)>>2]|0)==(l|0)){n=0;p=15706;break}else{o=o+1|0}}if((p|0)==15706){i=g;return n|0}AN(f,16,1617208,(m=i,i=i+16|0,c[m>>2]=h,c[m+8>>2]=b,m)|0);i=m;n=-1163346256;i=g;return n|0}function Cm(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;g=i;i=i+40|0;h=d;d=i;i=i+8|0;c[d>>2]=c[h>>2];c[d+4>>2]=c[h+4>>2];h=g|0;j=g+8|0;k=g+16|0;l=g+24|0;m=g+32|0;n=ct(e|0,1654352,(e=i,i=i+40|0,c[e>>2]=j,c[e+8>>2]=k,c[e+16>>2]=l,c[e+24>>2]=h,c[e+32>>2]=m,e)|0)|0;i=e;if((n|0)!=5){AN(f,16,1635800,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;o=-1094995529;i=g;return o|0}NM(b|0,0,20);n=(a[h]|0)!=58;c[b+4>>2]=n&1;h=d;p=b+8|0;q=c[h>>2]|0;r=c[h+4>>2]|0;c[p>>2]=q;c[p+4>>2]=r;p=q;q=c[d+4>>2]|0;d=r;do{if((q|0)==0|(p|0)==0){c[b+16>>2]=-1;s=-1}else{r=(((q|0)/2|0)+p|0)/(q|0)|0;c[b+16>>2]=r;if((r|0)!=0){s=r;break}AN(f,16,1628776,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;o=-22;i=g;return o|0}}while(0);do{if(n){if((s|0)==60|(s|0)==30){t=0;break}AN(f,16,1622672,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;o=-22;i=g;return o|0}else{t=0}}while(0);while(1){if(t>>>0>=5){u=15718;break}if((c[299800+(t<<2)>>2]|0)==(s|0)){break}else{t=t+1|0}}if((u|0)==15718){AN(f,16,1617208,(e=i,i=i+16|0,c[e>>2]=p,c[e+8>>2]=d,e)|0);i=e;o=-1163346256;i=g;return o|0}e=c[j>>2]|0;j=c[k>>2]|0;k=ac((j*60|0)+(e*3600|0)+(c[l>>2]|0)|0,s)|0;s=k+(c[m>>2]|0)|0;m=b|0;c[m>>2]=s;if(!n){o=0;i=g;return o|0}n=(e*60|0)+j|0;c[m>>2]=s-(n-((n|0)/10|0)<<1);o=0;i=g;return o|0}function Cn(){var b=0,d=0;b=i;if(a[337496]|0){i=b;return 3417956}d=cS(+1152921504606847000.0)|0;if(!((d|0)==0&(G|0)==268435456)){AN(0,16,1714e3,(d=i,i=i+1|0,i=i+7&-8,c[d>>2]=0,d)|0);i=d}BU();a[337496]=1;i=b;return 3417956}function Co(){return 1687536}function Cp(a){a=a|0;var b=0;switch(a|0){case 4:{b=1617160;break};case 0:{b=1654320;break};case 2:{b=1628744;break};case 1:{b=1635776;break};case 3:{b=1622648;break};default:{b=0}}return b|0}function Cq(a){a=a|0;var b=0;switch(a|0){case 4:{b=83;break};case 3:{b=66;break};case 2:{b=80;break};case 6:{b=112;break};case 1:{b=73;break};case 7:{b=98;break};case 5:{b=105;break};default:{b=63}}return b|0}function Cr(d,e,f,g){d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0;h=i;if((e|0)==0){j=0;i=h;return j|0}if((d|0)==4){k=f;l=e;m=0;while(1){if((c[l+(m<<2)>>2]|0)==(k|0)){j=m;break}else{m=m+1|0}}i=h;return j|0}else if((d|0)==8){m=e;k=0;while(1){l=m+(k<<3)|0;if((c[l>>2]|0)==(f|0)&(c[l+4>>2]|0)==(g|0)){j=k;break}else{k=k+1|0}}i=h;return j|0}else if((d|0)==1){k=f&255;g=0;while(1){if((a[e+g|0]|0)==k<<24>>24){j=g;break}else{g=g+1|0}}i=h;return j|0}else if((d|0)==2){d=e;e=f&65535;f=0;while(1){if((b[d+(f<<1)>>1]|0)==e<<16>>16){j=f;break}else{f=f+1|0}}i=h;return j|0}else{AN(0,0,1612392,(j=i,i=i+24|0,c[j>>2]=1739368,c[j+8>>2]=1734880,c[j+16>>2]=108,j)|0);i=j;cw();return 0}return 0}function Cs(a){a=a|0;var b=0;if((a|0)==0){b=0;return b|0}b=(c[a>>2]|0)==-487877|0;return b|0}function Ct(a){a=a|0;var b=0;if((a|0)==0){b=0;return b|0}b=(c[a>>2]|0)==-487877|0;return b|0}function Cu(a){a=a|0;var b=0,d=0,e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0.0,w=0.0,x=0,y=0,z=0,A=0,B=0.0,C=0,D=0,E=0,F=0,G=0.0,H=0.0,I=0.0,J=0.0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,S=0,V=0,W=0.0,X=0.0,Y=0.0,Z=0,_=0,$=0,aa=0,ab=0,ad=0.0,ae=0,af=0,ag=0,ah=0.0,ai=0.0,aj=0.0,ak=0,al=0,am=0,an=0,ao=0,ap=0.0,aq=0.0,ar=0.0,as=0.0,at=0.0,au=0.0,av=0.0,aw=0.0,ax=0.0;b=i;d=a+288|0;e=c[d>>2]|0;f=e+16|0;h=e|0;c[h>>2]=0;c[e+124>>2]=c[a+180>>2];j=c[a+32>>2]|0;c[e+140>>2]=j;if((j|0)!=0){c[a+36>>2]=0}if((c[e+85804>>2]|0)!=0){c[a+36>>2]=0}c[e+85828>>2]=c[a+272>>2];c[e+85832>>2]=c[a+276>>2];c[e+85836>>2]=c[a+280>>2];if((c[a+296>>2]|0)==0){j=e+85756|0;k=c[j>>2]&-3;c[j>>2]=k;l=k}else{k=e+85756|0;j=(Ef()|0)<<1&2;m=c[k>>2]&-3|j;c[k>>2]=m;l=m}if((c[a+292>>2]|0)==0){m=l&-2;c[e+85756>>2]=m;n=m}else{m=(Ee()|0)&1;l=e+85756|0;k=c[l>>2]&-2|m;c[l>>2]=k;n=k}if((c[a+300>>2]|0)==0){c[e+85756>>2]=n&-13}else{n=e+85756|0;k=(Eg()|0)<<2&4;c[n>>2]=c[n>>2]&-5|k;k=(Eh()|0)<<3&8;c[n>>2]=c[n>>2]&-9|k}k=e+85796|0;do{if((c[k>>2]|0)==0){n=NE(1,2772)|0;c[k>>2]=n;if((n|0)==0){o=-2}else{break}i=b;return o|0}}while(0);n=e+85676|0;do{if((c[n>>2]|0)==0){l=NE(1,134792)|0;c[n>>2]=l;if((l|0)!=0){break}D1(e);c[d>>2]=0;o=-2;i=b;return o|0}}while(0);l=e+160|0;c[l>>2]=c[a+120>>2];c[e+164>>2]=c[a+104>>2];c[e+168>>2]=c[a+108>>2];c[e+172>>2]=c[a+112>>2];c[e+176>>2]=c[a+116>>2];m=c[a+8>>2]|0;j=e+68|0;c[j>>2]=m;p=a+48|0;if((m|0)==1){c[p>>2]=3;q=1}else{q=(c[p>>2]|0)==3?1:2}p=a+48|0;m=e+72|0;c[m>>2]=q;q=a+52|0;if((c[p>>2]|0)==3){c[q>>2]=0;r=0}else{r=c[q>>2]|0}c[e+80>>2]=r;r=a+156|0;q=c[r>>2]|0;do{if((q|0)==0){s=c[a+168>>2]|0;if((s|0)==128){break}t=a+96|0;if((c[t>>2]|0)!=0){break}c[t>>2]=s;if(!((q|0)==0|(q|0)==4|(q|0)==1)){u=15807}}else if(!((q|0)==4|(q|0)==1)){u=15807}}while(0);if((u|0)==15807){c[a+56>>2]=0}q=e+152|0;c[q>>2]=c[a+56>>2];do{if((c[r>>2]|0)==0){s=a+96|0;do{if((c[s>>2]|0)==0){t=a+100|0;v=+g[t>>2];if(v==0.0){if(v!=0.0){break}}else{w=+R(+v);if(w>w*9.999999974752427e-7){break}}g[t>>2]=11.024999618530273}}while(0);t=a+100|0;w=+g[t>>2];if(w<=0.0){break}x=a+16|0;y=c[x>>2]|0;if((y|0)==0){z=D6(~~(+(c[a+12>>2]|0)*.97))|0;c[x>>2]=z;A=z;B=+g[t>>2]}else{A=y;B=w}c[s>>2]=~~(+(ac(A<<4,c[m>>2]|0)|0)/(B*1.0e3));y=f|0;c[e+20>>2]=D8(A,y)|0;if((c[q>>2]|0)!=0){break}c[s>>2]=D4(c[s>>2]|0,c[y>>2]|0,c[x>>2]|0)|0}}while(0);A=a+16|0;x=c[A>>2]|0;L21285:do{if((x|0)==0){y=c[r>>2]|0;if(!((y|0)==1|(y|0)==4)){C=0;break}y=a+164|0;t=a+160|0;B=+(c[y>>2]|0)+ +g[t>>2];z=c[a+12>>2]|0;D=2;L21295:while(1){E=c[139016+(D*24|0)>>2]|0;do{if((z|0)==(E|0)){w=+g[139020+(D*24|0)>>2];if(B>=w){break}v=B/w*+g[139028+(D*24|0)>>2];F=~~v;c[y>>2]=F;g[t>>2]=v- +(F|0)}}while(0);do{if((z|0)>=(E|0)){G=+g[139020+(D*24|0)>>2];if(G>B){break}H=+g[139024+(D*24|0)>>2];if(B<H){break L21295}}}while(0);F=D+1|0;if((F|0)<9){D=F}else{C=0;break L21285}}v=+g[139028+(D*24|0)>>2];w=v+(B-G)*(+g[139032+(D*24|0)>>2]-v)/(H-G);z=~~w;c[y>>2]=z;g[t>>2]=w- +(z|0);c[A>>2]=E;z=a+184|0;if((c[z>>2]|0)!=0){C=E;break}c[z>>2]=-1;C=E}else{if((x|0)<16e3){z=a+168|0;s=c[z>>2]|0;F=(s|0)>8?s:8;c[z>>2]=(F|0)<64?F:64;C=x;break}F=a+168|0;z=c[F>>2]|0;if((x|0)<32e3){s=(z|0)>8?z:8;c[F>>2]=(s|0)<160?s:160;C=x;break}else{s=(z|0)>32?z:32;c[F>>2]=(s|0)<320?s:320;C=x;break}}}while(0);x=a+184|0;E=c[x>>2]|0;if((E|0)==0){L21310:do{switch(c[r>>2]|0){case 0:{I=+(c[120660+((D5(c[a+96>>2]&65535)|0)<<3)>>2]|0);break};case 3:{I=+(c[120660+((D5(c[a+168>>2]&65535)|0)<<3)>>2]|0);break};case 2:{s=c[a+164>>2]|0;if(s>>>0>=10){I=19500.0;break L21310}G=+(c[138968+(s<<2)>>2]|0);I=G+(+(c[138968+(s+1<<2)>>2]|0)-G)*+g[a+160>>2];break};case 4:case 1:{s=c[a+164>>2]|0;if(s>>>0>=10){I=21500.0;break L21310}G=+(c[138920+(s<<2)>>2]|0);I=G+(+(c[138920+(s+1<<2)>>2]|0)-G)*+g[a+160>>2];break};default:{s=c[a+164>>2]|0;if(s>>>0>=10){I=19500.0;break L21310}G=+(c[138872+(s<<2)>>2]|0);I=G+(+(c[138872+(s+1<<2)>>2]|0)-G)*+g[a+160>>2]}}}while(0);do{if((c[p>>2]|0)==3){s=c[r>>2]|0;if(!((s|0)==0|(s|0)==3)){J=I;break}J=I*1.5}else{J=I}}while(0);s=~~J;c[x>>2]=s;K=c[A>>2]|0;L=s}else{K=C;L=E}if((K|0)==0){E=c[a+12>>2]|0;if((L<<1|0)>(E|0)){C=(E|0)/2|0;c[x>>2]=C;M=C}else{M=L}do{if((E|0)>47999){N=48e3}else{if((E|0)>44099){N=44100;break}if((E|0)>31999){N=32e3;break}if((E|0)>23999){N=24e3;break}if((E|0)>22049){N=22050;break}if((E|0)>15999){N=16e3;break}if((E|0)>11999){N=12e3;break}if((E|0)>11024){N=11025;break}N=(E|0)>7999?8e3:44100}}while(0);do{if((M|0)==-1){O=N}else{C=(M|0)<3971?8e3:(M|0)<4511?11025:(M|0)<5421?12e3:(M|0)<7231?16e3:(M|0)<9971?22050:(M|0)<11221?24e3:(M|0)<15251?32e3:(M|0)<15961?44100:N;if((C|0)<=(E|0)){O=C;break}if((E|0)>44100){O=48e3;break}if((E|0)>32e3){O=44100;break}if((E|0)>24e3){O=32e3;break}if((E|0)>22050){O=24e3;break}if((E|0)>16e3){O=22050;break}if((E|0)>12e3){O=16e3;break}if((E|0)>11025){O=12e3;break}O=(E|0)>8e3?11025:8e3}}while(0);c[A>>2]=O;P=M;Q=O}else{P=L;Q=K}K=c[r>>2]|0;do{if((K|0)==1|(K|0)==4){L=(P|0)>24e3?24e3:P;O=(Q|0)/2|0;c[x>>2]=(O|0)<(L|0)?O:L;if((K|0)==3){u=15877}}else{L=(P|0)>20500?20500:P;O=(Q|0)/2|0;c[x>>2]=(O|0)<(L|0)?O:L;if((K|0)==3){u=15877;break}else if((K|0)!=0){break}J=+(ac(Q<<4,c[m>>2]|0)|0);g[a+100>>2]=J/(+(c[a+96>>2]|0)*1.0e3)}}while(0);if((u|0)==15877){J=+(ac(Q<<4,c[m>>2]|0)|0);g[a+100>>2]=J/(+(c[a+168>>2]|0)*1.0e3)}Q=a+36|0;K=a+60|0;if((c[Q>>2]|0)==0){c[K>>2]=0;P=a+64|0;c[P>>2]=0;c[e+132>>2]=0;S=P}else{S=a+64|0}P=c[K>>2]|0;c[e+128>>2]=P;K=c[S>>2]|0;c[e+136>>2]=K;if((K|0)!=0){c[e+132>>2]=1}do{if((P|0)!=0){if((EP(c[n>>2]|0,c[A>>2]|0)|0)!=0){break}D1(e);c[d>>2]=0;o=-6;i=b;return o|0}}while(0);c[e+144>>2]=c[a+128>>2];n=e+52|0;c[n>>2]=c[x>>2];x=e+56|0;c[x>>2]=c[a+188>>2];c[e+60>>2]=c[a+12>>2];P=c[A>>2]|0;A=e+64|0;c[A>>2]=P;K=e+76|0;c[K>>2]=(P|0)<24001?1:2;c[e+84760>>2]=576;S=c[r>>2]|0;if((S|0)==1|(S|0)==2|(S|0)==4){g[a+100>>2]=+g[139232+(c[a+164>>2]<<2)>>2]}else if((S|0)==3){J=+(ac(P<<4,c[m>>2]|0)|0);g[a+100>>2]=J/(+(c[a+168>>2]|0)*1.0e3)}else{J=+(ac(P<<4,c[m>>2]|0)|0);g[a+100>>2]=J/(+(c[a+96>>2]|0)*1.0e3)}P=c[p>>2]|0;if((P|0)==4){c[p>>2]=1;V=1}else{V=P}P=e+180|0;c[P>>2]=V;V=c[x>>2]|0;if((V|0)>0){J=+(V|0)*2.0;x=e+256|0;g[x>>2]=J;p=c[a+196>>2]|0;if((p|0)>-1){I=+(V+p|0)*2.0;g[e+260>>2]=I;W=I}else{g[e+260>>2]=J;W=J}I=+(c[A>>2]|0);g[x>>2]=J/I;J=W/I;g[e+260>>2]=J;X=J}else{g[e+256>>2]=0.0;g[e+260>>2]=0.0;X=0.0}x=e+248|0;g[x>>2]=0.0;p=e+252|0;g[p>>2]=0.0;V=c[n>>2]|0;do{if((V|0)>0){n=c[A>>2]|0;if((V|0)>=((n|0)/2|0|0)){break}J=+(V|0)*2.0;g[p>>2]=J;S=c[a+192>>2]|0;do{if((S|0)>-1){I=+(V-S|0)*2.0;g[x>>2]=I;if(I>=0.0){Y=I;break}g[x>>2]=0.0;Y=0.0}else{g[x>>2]=J;Y=J}}while(0);B=+(n|0);I=Y/B;g[x>>2]=I;W=J/B;g[p>>2]=W;if(I>0.0){Z=0;_=999;$=32}else{break}while(1){B=+(Z|0)/31.0;if(B<W){aa=$}else{aa=($|0)<(Z|0)?$:Z}if(I<B&B<W){ab=(_|0)<(Z|0)?_:Z}else{ab=_}S=Z+1|0;if((S|0)<32){Z=S;_=ab;$=aa}else{break}}g[x>>2]=(+(((ab|0)==999?aa:ab)|0)+-.75)/31.0;g[p>>2]=+(aa|0)/31.0}}while(0);aa=e+260|0;ab=e+256|0;if(X>0.0&X<.021774193548387097){g[ab>>2]=0.0;g[aa>>2]=0.0;Ec(e,1614336,($=i,i=i+1|0,i=i+7&-8,c[$>>2]=0,$)|0);i=$;ad=+g[aa>>2]}else{ad=X}do{if(ad>0.0){X=+g[ab>>2];$=0;_=-1;Z=-1;while(1){Y=+($|0)/31.0;if(Y>X){ae=Z}else{ae=(Z|0)>($|0)?Z:$}if(X<Y&Y<ad){af=(_|0)>($|0)?_:$}else{af=_}V=$+1|0;if((V|0)<32){$=V;_=af;Z=ae}else{break}}X=+(ae|0);g[ab>>2]=X/31.0;if((af|0)==-1){Y=(X+.75)/31.0;g[aa>>2]=Y;ag=0;ah=Y;break}else{Y=(+(af|0)+.75)/31.0;g[aa>>2]=Y;ag=0;ah=Y;break}}else{ag=0;ah=ad}}while(0);while(1){ad=+(ag|0)/31.0;Y=+g[ab>>2];do{if(ah>Y){X=(ah-ad)/(ah-Y+1.0e-20);if(X>1.0){ai=0.0;break}if(X<=0.0){ai=1.0;break}ai=+U(+(X*1.5707963267948966))}else{ai=1.0}}while(0);Y=+g[p>>2];X=+g[x>>2];do{if(Y>X){W=(ad-X)/(Y-X+1.0e-20);if(W>1.0){aj=0.0;break}if(W<=0.0){aj=1.0;break}aj=+U(+(W*1.5707963267948966))}else{aj=1.0}}while(0);g[e+37040+(ag<<2)>>2]=ai*aj;af=ag+1|0;if((af|0)>=32){break}ag=af;ah=+g[aa>>2]}aa=f|0;ag=D8(c[A>>2]|0,aa)|0;x=e+20|0;c[x>>2]=ag;if((ag|0)<0){D1(e);c[d>>2]=0;o=-1;i=b;return o|0}do{if((c[r>>2]|0)==0){if((c[q>>2]|0)!=0){c[e+84744>>2]=0;break}ag=a+96|0;p=D4(c[ag>>2]|0,c[aa>>2]|0,c[A>>2]|0)|0;c[ag>>2]=p;ag=D7(p,c[aa>>2]|0,c[A>>2]|0)|0;c[e+84744>>2]=ag;if((ag|0)>=1){break}D1(e);c[d>>2]=0;o=-1;i=b;return o|0}else{c[e+84744>>2]=1}}while(0);EH(e);q=((c[aa>>2]|0)*3|0)+(c[x>>2]|0)+(-((c[A>>2]|0)<16e3|0)&6)|0;x=0;do{c[e+21360+(x<<2)>>2]=c[75416+(q*204|0)+(x<<2)>>2];x=x+1|0;}while((x|0)<23);x=c[e+21444>>2]|0;ag=((c[e+21448>>2]|0)-x|0)/6|0;c[e+21508>>2]=x;c[e+21512>>2]=ag+x;c[e+21516>>2]=(ag<<1)+x;c[e+21520>>2]=(ag*3|0)+x;c[e+21524>>2]=(ag<<2)+x;c[e+21528>>2]=(ag*5|0)+x;c[e+21532>>2]=576;c[e+21452>>2]=c[75508+(q*204|0)>>2];c[e+21456>>2]=c[75512+(q*204|0)>>2];c[e+21460>>2]=c[75516+(q*204|0)>>2];c[e+21464>>2]=c[75520+(q*204|0)>>2];c[e+21468>>2]=c[75524+(q*204|0)>>2];c[e+21472>>2]=c[75528+(q*204|0)>>2];c[e+21476>>2]=c[75532+(q*204|0)>>2];c[e+21480>>2]=c[75536+(q*204|0)>>2];c[e+21484>>2]=c[75540+(q*204|0)>>2];c[e+21488>>2]=c[75544+(q*204|0)>>2];c[e+21492>>2]=c[75548+(q*204|0)>>2];c[e+21496>>2]=c[75552+(q*204|0)>>2];x=c[75556+(q*204|0)>>2]|0;c[e+21500>>2]=x;ag=c[75560+(q*204|0)>>2]|0;c[e+21504>>2]=ag;q=(ag-x|0)/6|0;c[e+21536>>2]=x;c[e+21540>>2]=q+x;c[e+21544>>2]=(q<<1)+x;c[e+21548>>2]=(q*3|0)+x;c[e+21552>>2]=(q<<2)+x;c[e+21556>>2]=(q*5|0)+x;c[e+21560>>2]=192;x=c[K>>2]|0;q=c[m>>2]|0;ag=(q|0)==1;if((x|0)==2){p=ag?21:36;c[e+24>>2]=p;ak=p}else{p=ag?13:21;c[e+24>>2]=p;ak=p}if((c[l>>2]|0)!=0){c[e+24>>2]=ak|2}c[h>>2]=-487877;h=0;ak=x;x=q;while(1){g[e+39756+(h<<2)>>2]=+(ac(ak*700|0,x)|0);q=h+1|0;if((q|0)>=19){break}h=q;ak=c[K>>2]|0;x=c[m>>2]|0}x=a+220|0;if((c[x>>2]|0)==-1){c[x>>2]=4}K=c[r>>2]|0;do{if((K|0)==1|(K|0)==4){ak=a+124|0;if((c[ak>>2]|0)<0){c[ak>>2]=2}ak=a+244|0;if((c[ak>>2]|0)<0){c[ak>>2]=0}ak=((c[a+164>>2]|0)*-10|0)+500|0;CE(a,ak,0)|0;ak=a+44|0;h=c[ak>>2]|0;do{if((h|0)<0){c[ak>>2]=3;u=15961}else{if((h|0)<5){u=15961;break}if((h|0)<=7){break}c[ak>>2]=7}}while(0);if((u|0)==15961){c[ak>>2]=0}if((c[a+140>>2]|0)==0){c[e+85092>>2]=(c[A>>2]|0)>44e3}else{c[e+85092>>2]=0}c[e+85812>>2]=38}else if((K|0)==2){CE(a,((c[a+164>>2]|0)*-10|0)+500|0,0)|0;if((c[a+140>>2]|0)==0){c[e+85092>>2]=(c[A>>2]|0)>44e3}else{c[e+85092>>2]=0}h=a+44|0;q=c[h>>2]|0;do{if((q|0)>6){c[h>>2]=6}else{if((q|0)>=0){break}c[h>>2]=3}}while(0);c[e+85812>>2]=4}else{c[e+85092>>2]=0;h=a+44|0;if((c[h>>2]|0)<0){c[h>>2]=3}h=c[r>>2]|0;q=(h|0)==0;if(q){ak=c[a+96>>2]|0;Ds(a,ak)|0}CE(a,c[a+168>>2]|0,0)|0;c[r>>2]=h;h=e+85812|0;if(q){c[h>>2]=8;break}else{c[h>>2]=6;break}}}while(0);ah=+g[a+200>>2];K=e+84912|0;g[K>>2]=ah;aj=+g[a+204>>2];h=e+84916|0;g[h>>2]=aj;if((c[a+256>>2]|0)!=0){q=a+260|0;g[K>>2]=+g[q>>2]+ah;g[h>>2]=+g[q>>2]+aj}if((c[r>>2]|0)==0){al=a+168|0}else{q=e+112|0;c[q>>2]=1;h=e+116|0;c[h>>2]=14;K=c[A>>2]|0;if((K|0)<16e3){c[h>>2]=8}ak=a+172|0;l=c[ak>>2]|0;do{if((l|0)==0){am=1}else{p=D4(l,c[aa>>2]|0,K)|0;c[ak>>2]=p;ag=D7(p,c[aa>>2]|0,c[A>>2]|0)|0;c[q>>2]=ag;if((ag|0)<0){o=-1}else{am=ag;break}i=b;return o|0}}while(0);K=a+176|0;l=c[K>>2]|0;do{if((l|0)==0){an=am}else{ag=D4(l,c[aa>>2]|0,c[A>>2]|0)|0;c[K>>2]=ag;p=D7(ag,c[aa>>2]|0,c[A>>2]|0)|0;c[h>>2]=p;if((p|0)<0){o=-1;i=b;return o|0}else{an=c[q>>2]|0;break}}}while(0);c[ak>>2]=c[308168+(c[aa>>2]<<6)+(an<<2)>>2];c[K>>2]=c[308168+(c[aa>>2]<<6)+(c[h>>2]<<2)>>2];K=c[308168+(c[aa>>2]<<6)+(c[h>>2]<<2)>>2]|0;h=a+168|0;an=c[h>>2]|0;ak=(K|0)<(an|0)?K:an;c[h>>2]=ak;an=c[308168+(c[aa>>2]<<6)+(c[q>>2]<<2)>>2]|0;c[h>>2]=(an|0)>(ak|0)?an:ak;al=h}c[e+100>>2]=c[a+152>>2];c[e+156>>2]=c[Q>>2];Q=e+104|0;c[Q>>2]=c[r>>2];c[e+85096>>2]=c[a+80>>2];c[e+28>>2]=c[a+84>>2];c[e+32>>2]=c[a+88>>2];c[e+36>>2]=c[a+92>>2];h=e+120|0;c[h>>2]=c[a+96>>2];c[e+108>>2]=c[al>>2];g[e+244>>2]=+g[a+100>>2];al=c[d>>2]|0;ak=a+44|0;switch(c[ak>>2]|0){case 8:{c[ak>>2]=7;u=15998;break};case 7:{u=15998;break};case 6:{ak=al+28|0;if((c[ak>>2]|0)==0){c[ak>>2]=1}c[al+40>>2]=0;c[al+44>>2]=0;ak=al+32|0;if((c[ak>>2]|0)==-1){c[ak>>2]=1}c[al+36>>2]=0;c[al+48>>2]=0;break};case 5:{ak=al+28|0;if((c[ak>>2]|0)==0){c[ak>>2]=1}c[al+40>>2]=0;c[al+44>>2]=0;ak=al+32|0;if((c[ak>>2]|0)==-1){c[ak>>2]=1}c[al+36>>2]=0;c[al+48>>2]=0;break};case 4:{ak=al+28|0;if((c[ak>>2]|0)==0){c[ak>>2]=1}c[al+40>>2]=0;c[al+44>>2]=0;ak=al+32|0;if((c[ak>>2]|0)==-1){c[ak>>2]=1}c[al+36>>2]=1;c[al+48>>2]=0;break};case 3:{ak=al+28|0;if((c[ak>>2]|0)==0){c[ak>>2]=1}c[al+40>>2]=1;c[al+44>>2]=1;ak=al+32|0;if((c[ak>>2]|0)==-1){c[ak>>2]=1}c[al+36>>2]=1;c[al+48>>2]=0;break};case 2:{ak=al+28|0;if((c[ak>>2]|0)==0){c[ak>>2]=1}ak=al+85096|0;if((c[ak>>2]|0)==0){c[ak>>2]=2}c[al+40>>2]=1;c[al+44>>2]=1;ak=al+32|0;if((c[ak>>2]|0)==-1){c[ak>>2]=1}c[al+36>>2]=1;c[al+48>>2]=0;break};case 1:{ak=al+28|0;if((c[ak>>2]|0)==0){c[ak>>2]=1}ak=al+85096|0;if((c[ak>>2]|0)==0){c[ak>>2]=2}c[al+40>>2]=2;c[al+44>>2]=1;ak=al+32|0;if((c[ak>>2]|0)==-1){c[ak>>2]=1}c[al+36>>2]=1;c[al+48>>2]=0;break};case 0:{ak=al+28|0;if((c[ak>>2]|0)==0){c[ak>>2]=1}ak=al+85096|0;if((c[ak>>2]|0)==0){c[ak>>2]=2}c[al+40>>2]=2;c[al+44>>2]=1;ak=al+32|0;if((c[ak>>2]|0)==-1){c[ak>>2]=1}c[al+36>>2]=1;c[al+48>>2]=1;break};default:{c[al+28>>2]=0;NM(al+36|0,0,16)}}do{if((u|0)==15998){c[al+28>>2]=0;NM(al+36|0,0,16);ak=c[r>>2]|0;if(!((ak|0)==1|(ak|0)==4)){break}c[al+48>>2]=-1}}while(0);al=c[a+232>>2]|0;c[c[k>>2]>>2]=(al|0)<0?3:al;aj=+T(+10.0,+(+g[a+236>>2]/-10.0));g[(c[k>>2]|0)+4>>2]=aj;k=a+240|0;al=c[k>>2]|0;if((al|0)==(-1|0)){c[k>>2]=0;u=16043}else if((al|0)==0){u=16043}else{ao=al}do{if((u|0)==16043){if((c[P>>2]|0)>>>0>=2){ao=0;break}c[k>>2]=1;ao=1}}while(0);c[e+184>>2]=ao;if((Dj(a)|0)<0){Dh(a,1)|0}if((Dk(a)|0)<0){Di(a,0)|0}if(+DP(a)<0.0){DO(a,0.0)}Dm(a,Dn(a)|0|1)|0;if((c[x>>2]|0)<0){c[x>>2]=4}ao=a+224|0;if(+g[ao>>2]<0.0){g[ao>>2]=4.0}k=a+248|0;aj=+g[k>>2];if(aj<0.0){g[k>>2]=0.0;ap=0.0}else{ap=aj}k=a+244|0;if((c[k>>2]|0)<0){c[k>>2]=1}g[e+188>>2]=ap;g[e+192>>2]=+g[a+252>>2];ap=0.0- +g[a+228>>2];g[e+196>>2]=ap;g[e+200>>2]=+T(+10.0,+(ap*.10000000149011612));g[e+204>>2]=+g[ao>>2];c[e+208>>2]=c[x>>2];c[e+212>>2]=c[a+208>>2];c[e+216>>2]=c[a+212>>2];c[e+220>>2]=c[a+216>>2];c[e+84>>2]=c[a+132>>2];c[e+88>>2]=c[a+136>>2];c[e+92>>2]=c[k>>2];k=a+148|0;c[e+96>>2]=c[k>>2]&2;ap=+((c[k>>2]|0)>>>2&63|0);if(ap<32.0){aq=ap}else{aq=ap+-64.0}g[e+232>>2]=aq*.25;aq=+((c[k>>2]|0)>>>8&63|0);if(aq<32.0){ar=aq}else{ar=aq+-64.0}g[e+228>>2]=ar*.25;ar=+((c[k>>2]|0)>>>14&63|0);if(ar<32.0){as=ar}else{as=ar+-64.0}ar=as*.25;g[e+236>>2]=ar;as=+((c[k>>2]|0)>>>20&63|0);if(as<32.0){at=as}else{at=as+-64.0}g[e+240>>2]=at*.25+ar;ar=+g[a+20>>2];at=ar*0.0;as=+g[a+24>>2];aq=ar*as;ap=at*as;as=+g[a+28>>2];aj=at*as;at=ar*as;do{if((c[j>>2]|0)==2){if((c[m>>2]|0)!=1){au=aq;av=ap;aw=aj;ax=at;break}au=(aq+aj)*.5;av=(ap+at)*.5;aw=0.0;ax=0.0}else{au=aq;av=ap;aw=aj;ax=at}}while(0);g[e+264>>2]=au;g[e+268>>2]=av;g[e+272>>2]=aw;g[e+276>>2]=ax;m=e+39832|0;c[m>>2]=0;j=e+39836|0;c[j>>2]=0;if((c[Q>>2]|0)==0){Q=ac(((c[aa>>2]|0)*72e3|0)+72e3|0,c[h>>2]|0)|0;h=(Q|0)%(c[A>>2]|0)|0;c[m>>2]=h;c[j>>2]=h}do{if((a|0)!=0){if((c[a>>2]|0)!=-487877){break}h=c[d>>2]|0;if((h|0)==0){break}c[h+84748>>2]=0;if((c[a+68>>2]|0)!=0){EV(a)|0}g[h+85684>>2]=0.0;NM(h+84040|0,0,704);if((c[h+156>>2]|0)==0){break}Ev(a)|0}}while(0);CY(e);CJ(a)|0;c[e+148>>2]=Ez(f,c[a+124>>2]|0)|0;o=0;i=b;return o|0}function Cv(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;return Cw(a,b,c,d,e,f,0,1,1.0)|0}function Cw(a,b,d,e,f,h,j,k,l){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;j=j|0;k=k|0;l=+l;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0;m=i;i=i+32|0;n=m|0;o=m+8|0;p=m+16|0;q=m+24|0;if((a|0)==0){r=-3;i=m;return r|0}if((c[a>>2]|0)!=-487877){r=-3;i=m;return r|0}s=c[a+288>>2]|0;if((s|0)==0){r=-3;i=m;return r|0}a=s|0;if((c[a>>2]|0)!=-487877){r=-3;i=m;return r|0}if((e|0)==0){r=0;i=m;return r|0}t=s+52152|0;u=c[t>>2]|0;do{if((u|0)==0){v=16099}else{if((c[s+52148>>2]|0)<(e|0)){ND(u);v=16099;break}else{w=u;x=c[s+52156>>2]|0;v=16102;break}}}while(0);if((v|0)==16099){u=s+52156|0;y=c[u>>2]|0;if((y|0)!=0){ND(y)}y=NE(e,4)|0;z=y;c[t>>2]=z;A=NE(e,4)|0;c[u>>2]=A;c[s+52148>>2]=e;if((y|0)==0){B=u;C=A}else{w=z;x=A;v=16102}}do{if((v|0)==16102){A=s+52156|0;if((x|0)==0){ND(w);B=A;C=c[A>>2]|0;break}z=(b|0)==0;do{if((c[s+68>>2]|0)>1){if(z|(d|0)==0){r=0;i=m;return r|0}else{CD(s,b,d,e,j,k,l);break}}else{if(z){r=0;i=m;return r|0}else{CD(s,b,b,e,j,k,l);break}}}while(0);z=s+76|0;u=(c[z>>2]|0)*576|0;y=(c[a>>2]|0)!=-487877;if(y){r=y?-3:0;i=m;return r|0}y=EG(s,f,h,0)|0;if((y|0)<0){r=y;i=m;return r|0}D=f+y|0;E=c[t>>2]|0;F=c[A>>2]|0;G=((c[z>>2]|0)*576|0)+752|0;z=n|0;c[z>>2]=s+52160;H=n+4|0;c[H>>2]=s+68096;I=o|0;J=o+4|0;K=s+128|0;L=s+72|0;M=s+84036|0;N=s+84032|0;O=s+136|0;P=s+85676|0;if((h|0)==0){Q=E;R=F;S=e;T=D;U=y;L21726:while(1){V=Q;W=R;X=S;while(1){if((X|0)<=0){r=U;v=16160;break L21726}c[p>>2]=0;c[q>>2]=0;c[I>>2]=V;c[J>>2]=W;Ea(s,z,I,X,p,q);do{if((c[K>>2]|0)!=0){if((c[O>>2]|0)!=0){break}Y=c[M>>2]|0;if((EQ(c[P>>2]|0,(c[z>>2]|0)+(Y<<2)|0,(c[H>>2]|0)+(Y<<2)|0,c[q>>2]|0,c[L>>2]|0)|0)==0){r=-6;v=16153;break L21726}}}while(0);Y=c[p>>2]|0;Z=X-Y|0;_=V+(Y<<2)|0;if((c[L>>2]|0)==2){$=W+(Y<<2)|0}else{$=W}Y=c[q>>2]|0;aa=(c[M>>2]|0)+Y|0;c[M>>2]=aa;ab=c[N>>2]|0;if((ab|0)<1){c[N>>2]=1728;ac=1728}else{ac=ab}c[N>>2]=ac+Y;if((aa|0)<(G|0)){V=_;W=$;X=Z}else{break}}X=EK(s,c[z>>2]|0,c[H>>2]|0,T,0)|0;if((X|0)<0){r=X;v=16161;break}W=T+X|0;V=X+U|0;X=(c[M>>2]|0)-u|0;c[M>>2]=X;c[N>>2]=(c[N>>2]|0)-u;aa=c[L>>2]|0;if((aa|0)>0){ad=0;ae=X;af=aa}else{Q=_;R=$;S=Z;T=W;U=V;continue}while(1){if((ae|0)>0){aa=n+(ad<<2)|0;X=0;do{Y=c[aa>>2]|0;g[Y+(X<<2)>>2]=+g[Y+(X+u<<2)>>2];X=X+1|0;ag=c[M>>2]|0;}while((X|0)<(ag|0));ah=ag;ai=c[L>>2]|0}else{ah=ae;ai=af}X=ad+1|0;if((X|0)<(ai|0)){ad=X;ae=ah;af=ai}else{Q=_;R=$;S=Z;T=W;U=V;continue L21726}}}if((v|0)==16153){i=m;return r|0}else if((v|0)==16160){i=m;return r|0}else if((v|0)==16161){i=m;return r|0}}else{U=E;T=F;S=e;R=D;Q=y;L21697:while(1){A=U;V=T;W=S;while(1){if((W|0)<=0){r=Q;v=16154;break L21697}c[p>>2]=0;c[q>>2]=0;c[I>>2]=A;c[J>>2]=V;Ea(s,z,I,W,p,q);do{if((c[K>>2]|0)!=0){if((c[O>>2]|0)!=0){break}X=c[M>>2]|0;if((EQ(c[P>>2]|0,(c[z>>2]|0)+(X<<2)|0,(c[H>>2]|0)+(X<<2)|0,c[q>>2]|0,c[L>>2]|0)|0)==0){r=-6;v=16155;break L21697}}}while(0);X=c[p>>2]|0;aj=W-X|0;ak=A+(X<<2)|0;if((c[L>>2]|0)==2){al=V+(X<<2)|0}else{al=V}X=c[q>>2]|0;aa=(c[M>>2]|0)+X|0;c[M>>2]=aa;Y=c[N>>2]|0;if((Y|0)<1){c[N>>2]=1728;am=1728}else{am=Y}c[N>>2]=am+X;if((aa|0)<(G|0)){A=ak;V=al;W=aj}else{break}}W=EK(s,c[z>>2]|0,c[H>>2]|0,R,h-Q|0)|0;if((W|0)<0){r=W;v=16158;break}V=R+W|0;A=W+Q|0;W=(c[M>>2]|0)-u|0;c[M>>2]=W;c[N>>2]=(c[N>>2]|0)-u;aa=c[L>>2]|0;if((aa|0)>0){an=0;ao=W;ap=aa}else{U=ak;T=al;S=aj;R=V;Q=A;continue}while(1){if((ao|0)>0){aa=n+(an<<2)|0;W=0;do{X=c[aa>>2]|0;g[X+(W<<2)>>2]=+g[X+(W+u<<2)>>2];W=W+1|0;aq=c[M>>2]|0;}while((W|0)<(aq|0));ar=aq;as=c[L>>2]|0}else{ar=ao;as=ap}W=an+1|0;if((W|0)<(as|0)){an=W;ao=ar;ap=as}else{U=ak;T=al;S=aj;R=V;Q=A;continue L21697}}}if((v|0)==16154){i=m;return r|0}else if((v|0)==16155){i=m;return r|0}else if((v|0)==16158){i=m;return r|0}}}}while(0);if((C|0)!=0){ND(C)}c[t>>2]=0;c[B>>2]=0;c[s+52148>>2]=0;Ed(s,1615056,(s=i,i=i+1|0,i=i+7&-8,c[s>>2]=0,s)|0);i=s;r=-2;i=m;return r|0}function Cx(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;return Cw(a,b,c,d,e,f,3,1,1.0)|0}function Cy(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;return Cw(a,b,c,d,e,f,1,1,152587890625.0e-16)|0}function Cz(a){a=a|0;var b=0,d=0.0,e=0.0,f=0.0,h=0.0,i=0,j=0;b=a+85680|0;do{if((c[a+128>>2]|0)!=0){d=+ES(c[a+85676>>2]|0);e=d;f=+R(+d);h=+R(+(d+24601.0));if(f>24601.0){if(h>f*9.999999974752427e-7){i=16172}else{i=16173}}else{if(h>.024600999937888446){i=16172}else{i=16173}}if((i|0)==16173){c[a+85688>>2]=0;break}else if((i|0)==16172){c[a+85688>>2]=~~+Q(+(e*10.0+.5));break}}}while(0);if((c[a+132>>2]|0)==0){return}i=a+85684|0;j=~~+ab(+(+b7(+(+g[i>>2]/32767.0))*20.0*10.0));c[a+85692>>2]=j;if((j|0)>0){g[b>>2]=+Q(+(32767.0/+g[i>>2]*100.0))/100.0;return}else{g[b>>2]=-1.0;return}}function CA(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0.0,p=0,q=0.0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;e=i;i=i+4608|0;f=e|0;if((a|0)==0){g=-3;i=e;return g|0}if((c[a>>2]|0)!=-487877){g=-3;i=e;return g|0}h=c[a+288>>2]|0;if((h|0)==0){g=-3;i=e;return g|0}if((c[h>>2]|0)!=-487877){g=-3;i=e;return g|0}j=h+84032|0;k=c[j>>2]|0;if((k|0)<1){g=0;i=e;return g|0}l=(c[h+76>>2]|0)*576|0;m=l+752|0;n=k-1152|0;k=f;NM(k|0,0,4608);if((D9(h+16|0)|0)==0){o=1.0;p=n}else{q=+(c[h+60>>2]|0)/+(c[h+64>>2]|0);o=q;p=~~(+(n|0)+16.0/q)}n=l-((p|0)%(l|0)|0)|0;r=((n|0)<576?l:0)+n|0;c[h+84764>>2]=r;n=(r+p|0)/(l|0)|0;do{if((n|0)>0){l=h+84748|0;p=h+84036|0;r=(d|0)==0;s=f+2304|0;t=n;u=0;v=b;w=c[l>>2]|0;while(1){x=~~(o*+(m-(c[p>>2]|0)|0));y=(x|0)>1152?1152:x;z=Cw(a,k,s,(y|0)<1?1:y,v,r?0:d-u|0,0,1,1.0)|0;A=v+z|0;B=z+u|0;y=c[l>>2]|0;x=t-((w|0)!=(y|0))|0;if((x|0)>0&(z|0)>-1){t=x;u=B;v=A;w=y}else{break}}c[j>>2]=0;if((z|0)<0){g=z}else{C=A;D=B;break}i=e;return g|0}else{c[j>>2]=0;C=b;D=0}}while(0);b=(d|0)==0;EC(h);j=EG(h,C,b?0:d-D|0,1)|0;Cz(h);if((j|0)<0){g=j;i=e;return g|0}B=j+D|0;if((c[a+68>>2]|0)==0){g=B;i=e;return g|0}else{g=b?0:d-B|0;d=C+j|0;EX(a)|0;a=EG(h,d,g,0)|0;i=e;return((a|0)<0?0:B)+a|0}return 0}function CB(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;if((a|0)==0){b=0;return b|0}d=a|0;if((c[d>>2]|0)!=-487877){b=0;return b|0}e=a+288|0;f=c[e>>2]|0;c[d>>2]=0;if((f|0)==0){g=-3}else{d=f|0;h=(c[d>>2]|0)==-487877?0:-3;c[d>>2]=0;D1(f);c[e>>2]=0;g=h}if((c[a+284>>2]|0)==0){b=g;return b|0}ND(a);b=g;return b|0}function CC(){var a=0,b=0,d=0;Ej();a=NE(1,304)|0;if((a|0)==0){b=0;return b|0}Ei();NM(a|0,0,304);c[a>>2]=-487877;d=NE(1,85840)|0;c[a+288>>2]=d;if((d|0)==0){ND(a);b=0;return b|0}else{c[a+124>>2]=2;c[a+48>>2]=4;c[a+108>>2]=1;c[a+12>>2]=44100;c[a+8>>2]=2;c[a+4>>2]=-1;c[a+36>>2]=1;c[a+44>>2]=-1;c[a+240>>2]=-1;c[a+88>>2]=-1;c[a+184>>2]=0;c[a+188>>2]=0;c[a+192>>2]=-1;c[a+196>>2]=-1;c[a+156>>2]=0;c[a+164>>2]=4;g[a+224>>2]=-1.0;c[a+168>>2]=128;c[a+172>>2]=0;c[a+176>>2]=0;c[a+180>>2]=0;c[d+112>>2]=1;c[d+116>>2]=13;c[a+132>>2]=-1;c[a+136>>2]=-1;g[a+252>>2]=-1.0;c[d+84920>>2]=180;c[d+84924>>2]=180;c[d+84928>>2]=4;c[d+84932>>2]=4;g[d+84908>>2]=1.0;g[a+264>>2]=-1.0;g[a+268>>2]=-1.0;g[a+20>>2]=1.0;g[a+24>>2]=1.0;g[a+28>>2]=1.0;c[a+232>>2]=-1;c[a+220>>2]=-1;g[a+236>>2]=0.0;c[a+244>>2]=-1;g[a+248>>2]=-1.0;c[d+84032>>2]=1728;c[d+84764>>2]=0;c[d+84036>>2]=528;c[a+60>>2]=0;c[a+64>>2]=0;c[d+136>>2]=0;c[d+128>>2]=0;c[d+132>>2]=0;c[d+85688>>2]=0;c[d+85692>>2]=0;g[d+85680>>2]=-1.0;c[a+292>>2]=1;c[a+296>>2]=1;c[a+300>>2]=1;c[a+152>>2]=0;c[a+68>>2]=1;c[a+276>>2]=26;c[a+280>>2]=26;c[a+272>>2]=26;c[a+284>>2]=1;b=a;return b|0}return 0}function CD(a,d,e,f,i,j,k){a=a|0;d=d|0;e=e|0;f=f|0;i=i|0;j=j|0;k=+k;var l=0,m=0,n=0.0,o=0.0,p=0.0,q=0.0,r=0,s=0.0,t=0;l=c[a+52152>>2]|0;m=c[a+52156>>2]|0;n=+g[a+264>>2]*k;o=+g[a+268>>2]*k;p=+g[a+272>>2]*k;q=+g[a+276>>2]*k;switch(i|0){case 2:{if((f|0)<=0){return}i=0;a=e;r=d;while(1){k=+(c[r>>2]|0);s=+(c[a>>2]|0);g[l+(i<<2)>>2]=n*k+o*s;g[m+(i<<2)>>2]=p*k+q*s;t=i+1|0;if((t|0)<(f|0)){i=t;a=a+(j<<2)|0;r=r+(j<<2)|0}else{break}}return};case 3:{if((f|0)<=0){return}r=d;a=e;i=0;while(1){s=+g[r>>2];k=+g[a>>2];g[l+(i<<2)>>2]=n*s+o*k;g[m+(i<<2)>>2]=p*s+q*k;t=i+1|0;if((t|0)<(f|0)){r=r+(j<<2)|0;a=a+(j<<2)|0;i=t}else{break}}return};case 1:{if((f|0)<=0){return}i=0;a=e;r=d;while(1){k=+(c[r>>2]|0);s=+(c[a>>2]|0);g[l+(i<<2)>>2]=n*k+o*s;g[m+(i<<2)>>2]=p*k+q*s;t=i+1|0;if((t|0)<(f|0)){i=t;a=a+(j<<2)|0;r=r+(j<<2)|0}else{break}}return};case 0:{if((f|0)<=0){return}r=0;a=e;i=d;while(1){s=+(b[i>>1]|0);k=+(b[a>>1]|0);g[l+(r<<2)>>2]=n*s+o*k;g[m+(r<<2)>>2]=p*s+q*k;t=r+1|0;if((t|0)<(f|0)){r=t;a=a+(j<<1)|0;i=i+(j<<1)|0}else{break}}return};case 4:{if((f|0)<=0){return}i=d;d=e;e=0;while(1){k=+h[i>>3];s=+h[d>>3];g[l+(e<<2)>>2]=n*k+o*s;g[m+(e<<2)>>2]=p*k+q*s;a=e+1|0;if((a|0)<(f|0)){i=i+(j<<3)|0;d=d+(j<<3)|0;e=a}else{break}}return};default:{return}}}function CE(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;L21875:do{switch(b|0){case 1003:{c[a+152>>2]=320;CF(a,320,d)|0;Do(a,0)|0;e=320;break};case 1e3:{Do(a,4)|0;c[a+152>>2]=470;f=16266;break};case 1001:case 1004:{Do(a,4)|0;c[a+152>>2]=480;f=16267;break};case 1006:case 1007:{Do(a,4)|0;c[a+152>>2]=460;f=16265;break};case 1002:case 1005:{Do(a,4)|0;c[a+152>>2]=500;f=16269;break};default:{g=a+152|0;c[g>>2]=b;switch(b|0){case 420:{CG(a,8,d);e=420;break L21875;break};case 410:{CG(a,9,d);e=410;break L21875;break};case 430:{CG(a,7,d);e=430;break L21875;break};case 490:{CG(a,1,d);e=490;break L21875;break};case 470:{f=16266;break L21875;break};case 440:{CG(a,6,d);e=440;break L21875;break};case 500:{f=16269;break L21875;break};case 450:{CG(a,5,d);e=450;break L21875;break};case 460:{f=16265;break L21875;break};case 480:{f=16267;break L21875;break};default:{if((b-8|0)>>>0<313){e=CF(a,b,d)|0;break L21875}else{c[g>>2]=0;e=b;break L21875}}}}}}while(0);if((f|0)==16266){CG(a,3,d);e=470}else if((f|0)==16269){CG(a,0,d);e=500}else if((f|0)==16265){CG(a,4,d);e=460}else if((f|0)==16267){CG(a,2,d);e=480}return e|0}function CF(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0.0,i=0.0,j=0,k=0,l=0.0,m=0.0,n=0.0,o=0.0,p=0,q=0,r=0;e=D5(b&65535)|0;Do(a,3)|0;Ds(a,b)|0;f=Dt(a)|0;Ds(a,(f|0)<320?f:320)|0;f=Dt(a)|0;Ds(a,(f|0)>8?f:8)|0;Df(a,Dt(a)|0)|0;if((e-12|0)>>>0<5){f=Dn(a)|0|2;Dm(a,f)|0}if(e>>>0<13){DH(a,1)|0}f=(d|0)!=0;do{if(f){d=c[347836+(e*52|0)>>2]|0;Dh(a,d)|0;d=c[347840+(e*52|0)>>2]|0;Di(a,d)|0;DO(a,+g[347848+(e*52|0)>>2]);h=+g[347852+(e*52|0)>>2];DI(a,h)|0;h=+g[347856+(e*52|0)>>2];DK(a,h)|0}else{if((Dj(a)|0)==-1){d=c[347836+(e*52|0)>>2]|0;Dh(a,d)|0}if((Dk(a)|0)==-1){d=c[347840+(e*52|0)>>2]|0;Di(a,d)|0}if(+DP(a)+1.0==0.0){DO(a,+g[347848+(e*52|0)>>2])}if(+DJ(a)+1.0==0.0){h=+g[347852+(e*52|0)>>2];DI(a,h)|0}if(+DL(a)+1.0!=0.0){break}h=+g[347856+(e*52|0)>>2];DK(a,h)|0}}while(0);h=+Da(a);C9(a,h*+g[347860+(e*52|0)>>2])|0;if(f){h=+g[347864+(e*52|0)>>2];Du(a,h)|0;i=h*1.1;Dw(a,i)|0;i=+g[347868+(e*52|0)>>2];DB(a,i)|0;i=+g[347872+(e*52|0)>>2];Dz(a,i)|0;i=+g[347876+(e*52|0)>>2];DF(a,i)|0;j=347832+(e*52|0)|0;k=c[j>>2]|0;l=+(k|0);m=l/320.0;n=m*5.0;o=n;p=a+288|0;q=c[p>>2]|0;r=q+280|0;g[r>>2]=o;return b|0}if(+Dv(a)==0.0){i=+g[347864+(e*52|0)>>2];Du(a,i)|0}if(+Dx(a)==0.0){i=+g[347864+(e*52|0)>>2]*1.1;Dw(a,i)|0}if(+DC(a)==0.0){i=+g[347868+(e*52|0)>>2];DB(a,i)|0}if(+DA(a)+1.0==0.0){i=+g[347872+(e*52|0)>>2];Dz(a,i)|0}if(+DG(a)+1.0!=0.0){j=347832+(e*52|0)|0;k=c[j>>2]|0;l=+(k|0);m=l/320.0;n=m*5.0;o=n;p=a+288|0;q=c[p>>2]|0;r=q+280|0;g[r>>2]=o;return b|0}DF(a,+g[347876+(e*52|0)>>2])|0;j=347832+(e*52|0)|0;k=c[j>>2]|0;l=+(k|0);m=l/320.0;n=m*5.0;o=n;p=a+288|0;q=c[p>>2]|0;r=q+280|0;g[r>>2]=o;return b|0}function CG(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0.0,i=0,j=0,k=0,l=0.0,m=0.0,n=0.0,o=0.0,p=0.0,q=0.0,r=0.0,s=0.0,t=0,u=0,v=0.0,w=0.0,x=0.0,y=0,z=0.0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;e=Dp(a)|0;if((e|0)==4|(e|0)==1){f=37768}else{f=37016}e=a+160|0;h=+g[e>>2];i=c[f+(b*68|0)+4>>2]|0;j=c[f+(b*68|0)+8>>2]|0;k=c[f+(b*68|0)+12>>2]|0;l=+g[f+(b*68|0)+16>>2];m=+g[f+(b*68|0)+20>>2];n=+g[f+(b*68|0)+24>>2];o=+g[f+(b*68|0)+28>>2];p=+g[f+(b*68|0)+32>>2];q=+g[f+(b*68|0)+36>>2];r=+g[f+(b*68|0)+40>>2];s=+g[f+(b*68|0)+44>>2];t=c[f+(b*68|0)+48>>2]|0;u=c[f+(b*68|0)+52>>2]|0;v=+g[f+(b*68|0)+56>>2];w=+g[f+(b*68|0)+60>>2];x=+g[f+(b*68|0)+64>>2];y=b+1|0;z=l+h*(+g[f+(y*68|0)+16>>2]-l);l=m+h*(+g[f+(y*68|0)+20>>2]-m);m=n+h*(+g[f+(y*68|0)+24>>2]-n);n=o+h*(+g[f+(y*68|0)+28>>2]-o);o=p+h*(+g[f+(y*68|0)+32>>2]-p);p=q+h*(+g[f+(y*68|0)+36>>2]-q);q=r+h*(+g[f+(y*68|0)+40>>2]-r);r=s+h*(+g[f+(y*68|0)+44>>2]-s);A=~~(+(u|0)+h*+((c[f+(y*68|0)+52>>2]|0)-u|0));s=v+h*(+g[f+(y*68|0)+56>>2]-v);v=w+h*(+g[f+(y*68|0)+60>>2]-w);w=x+h*(+g[f+(y*68|0)+64>>2]-x);Dq(a,c[f+(b*68|0)>>2]|0)|0;f=(d|0)!=0;do{if(f){Dh(a,i)|0;Di(a,j)|0}else{if((Dj(a)|0)==-1){Dh(a,i)|0}if((Dk(a)|0)!=-1){break}Di(a,j)|0}}while(0);if((k|0)!=0){Dl(a,k)|0}do{if(f){DI(a,z)|0;DK(a,l)|0;Du(a,m)|0;Dw(a,n)|0}else{if(+DJ(a)+1.0==0.0){DI(a,z)|0}if(+DL(a)+1.0==0.0){DK(a,l)|0}if(+Dv(a)==0.0){Du(a,m)|0}if(+Dx(a)!=0.0){break}Dw(a,n)|0}}while(0);if((Dp(a)|0)==1){B=16328}else{if((Dp(a)|0)==4){B=16328}}if((B|0)==16328){Dy(a,5)|0}do{if(f){DB(a,o)|0;Dz(a,p)|0;DD(a,q)|0}else{if(+DC(a)==0.0){DB(a,o)|0}if(+DA(a)+1.0==0.0){Dz(a,p)|0}if(+DE(a)!=0.0){break}DD(a,q)|0}}while(0);do{if(r>0.0){if(f){DF(a,r)|0;break}if(+DG(a)+1.0!=0.0){break}DF(a,r)|0}}while(0);if((t|0)>0){t=Dn(a)|0|2;Dm(a,t)|0}do{if((A|0)>0){t=Dn(a)|0;if((t&66060288|0)!=0){break}B=t|A<<20;Dm(a,B)|0}}while(0);if(f){DO(a,s);C=a+288|0;D=c[C>>2]|0;E=D+280|0;g[E>>2]=v;F=c[C>>2]|0;G=F+224|0;g[G>>2]=w;return}if(+DP(a)+1.0==0.0){DO(a,s)}c[a+164>>2]=b;g[e>>2]=h;C=a+288|0;D=c[C>>2]|0;E=D+280|0;g[E>>2]=v;F=c[C>>2]|0;G=F+224|0;g[G>>2]=w;return}
function dr(a){a=a|0;var b=0;b=i;i=i+a|0;i=i+7&-8;return b|0}function ds(){return i|0}function dt(a){a=a|0;i=a}function du(a,b){a=a|0;b=b|0;if((t|0)==0){t=a;u=b}}function dv(b){b=b|0;a[k]=a[b];a[k+1|0]=a[b+1|0];a[k+2|0]=a[b+2|0];a[k+3|0]=a[b+3|0]}function dw(b){b=b|0;a[k]=a[b];a[k+1|0]=a[b+1|0];a[k+2|0]=a[b+2|0];a[k+3|0]=a[b+3|0];a[k+4|0]=a[b+4|0];a[k+5|0]=a[b+5|0];a[k+6|0]=a[b+6|0];a[k+7|0]=a[b+7|0]}function dx(a){a=a|0;G=a}function dy(a){a=a|0;H=a}function dz(a){a=a|0;I=a}function dA(a){a=a|0;J=a}function dB(a){a=a|0;K=a}function dC(a){a=a|0;L=a}function dD(a){a=a|0;M=a}function dE(a){a=a|0;N=a}function dF(a){a=a|0;O=a}function dG(a){a=a|0;P=a}function dH(){}function dI(b,d){b=b|0;d=d|0;var e=0,f=0,g=0;d=i;do{if((b|0)==0){e=0;f=0}else{if((a[b]|0)==0){e=0;f=0;break}if((a8(b|0,1745200)|0)==0){e=1;f=0;break}if((a8(b|0,1745192)|0)==0){e=1;f=1;break}AN(0,16,1705264,(g=i,i=i+8|0,c[g>>2]=b,g)|0);i=g;e=0;f=0}}while(0);AN(0,32,1720784,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g;AN(0,32,1716680,(g=i,i=i+8|0,c[g>>2]=90712,g)|0);i=g;AN(0,32,1724496,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g;bT(1681528,(g=i,i=i+8|0,c[g>>2]=90712,g)|0)|0;i=g;eZ(117856,1664392,2048,0,0);eZ(117856,1648688,0,59396,0);if(e){eZ(117856,1633608,4,59392,0);eZ(117856,1627120,0,2612,57344);eZ(117856,1621440,4,560,57344);eZ(117856,1616064,16,36,0);eZ(117856,1745168,20,32,0);eZ(117856,1738480,32,20,0);eZ(117856,1733776,36,16,0)}else{eZ(117856,1627120,0,2612,57344);eZ(117856,1616064,16,36,0);eZ(117856,1738480,32,20,0)}eZ(117856,1728912,512,0,0);bH(10)|0;if((f|0)==0){i=d;return}e_(ox()|0,3);e_(lW()|0,3);e_(s2()|0,3);e_(rN()|0,8);e_(f8()|0,65560);i=d;return}function dJ(){var a=0,b=0;a=i;AN(0,32,1720784,(b=i,i=i+1|0,i=i+7&-8,c[b>>2]=0,b)|0);i=b;AN(0,32,1716680,(b=i,i=i+8|0,c[b>>2]=90712,b)|0);i=b;AN(0,32,1724496,(b=i,i=i+1|0,i=i+7&-8,c[b>>2]=0,b)|0);i=b;i=a;return}function dK(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0;d=i;i=i+208|0;e=d|0;f=d+80|0;NM(e|0,0,80);g=e8(e,a,b,117856,185456,2)|0;do{if((g|0)<0){AN(0,8,1713144,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;j=g}else{b=e1(0,e|0)|0;if((b|0)<0){AN(0,8,1709352,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;j=b;break}b=e+36|0;a=dL((c[b>>2]|0)+12|0,1707360,52)|0;if((a|0)<0){AN(0,8,1705232,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;j=a;break}a=dL(c[b>>2]|0,1703568,26)|0;if((a|0)<0){AN(0,8,1700840,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;j=a;break}e7(e);k=a;i=d;return k|0}}while(0);e7(e);e=f|0;zN(j,e,128)|0;AN(0,8,1697776,(h=i,i=i+8|0,c[h>>2]=e,h)|0);i=h;k=j;i=d;return k|0}function dL(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;e=i;i=i+416|0;f=e|0;h=a+8|0;if((c[h>>2]|0)<=0){j=0;i=e;return j|0}k=a+4|0;a=f;l=f+144|0;m=f+152|0;n=f+172|0;o=f+160|0;p=f+136|0;q=f|0;r=f+104|0;s=f+100|0;t=s;u=f+108|0;v=f+196|0;w=f+128|0;x=0;while(1){y=c[k>>2]|0;z=y+(x*36|0)|0;A=a8(b|0,1707360)|0;NM(a|0,0,416);B=c[21470]|0;C=c[21471]|0;if(((B|0)!=(-1|0)|(C|0)!=2147483647)&(A|0)!=0){c[l>>2]=B;c[l+4>>2]=C;AN(0,24,1637184,(D=i,i=i+1|0,i=i+7&-8,c[D>>2]=0,D)|0);i=D}else{c[l>>2]=-1;c[l+4>>2]=2147483647}c[m>>2]=-1;c[m+4>>2]=2147483647;g[n>>2]=.699999988079071;c[o>>2]=-1;c[o+4>>2]=-1;c[p>>2]=2147483647;c[q>>2]=z;E=e1(a,z)|0;F=y+(x*36|0)+4|0;G=c[F>>2]|0;if((E|0)<0){H=34;break}AN(0,48,1637600,(D=i,i=i+16|0,c[D>>2]=b,c[D+8>>2]=G,D)|0);i=D;I=dg[d&255](f,c[F>>2]|0)|0;y=a8(b|0,1707360)|0;z=117856;do{C=c[z+8>>2]|0;B=a+C|0;A=z+4|0;J=c[A>>2]|0;do{if((J&32768|0)==0){if((J&16392|0)!=16392){break}A6(B)}else{K=B;L=a+(C+4)|0;if((c[L>>2]|0)>0){M=0;do{A6((c[K>>2]|0)+(M<<4)|0);if((c[A>>2]&8|0)!=0){A6((c[K>>2]|0)+(M<<4)+8|0)}M=M+1|0;}while((M|0)<(c[L>>2]|0))}A6(B);c[L>>2]=0}}while(0);z=z+20|0;}while((c[z>>2]|0)!=0);z=(y|0)==0;if((c[r>>2]|0)>0){B=0;do{A6((c[s>>2]|0)+(B*24|0)+20|0);B=B+1|0;}while((B|0)<(c[r>>2]|0))}A6(t);A6(u);A6(v);A6(w);B=z?c[l+4>>2]|0:2147483647;c[21470]=z?c[l>>2]|0:-1;c[21471]=B;if((I|0)<0){H=48;break}AN(0,48,1637296,(D=i,i=i+1|0,i=i+7&-8,c[D>>2]=0,D)|0);i=D;B=x+1|0;if((B|0)<(c[h>>2]|0)){x=B}else{j=0;H=54;break}}if((H|0)==34){AN(0,16,1637720,(D=i,i=i+16|0,c[D>>2]=b,c[D+8>>2]=G,D)|0);i=D;j=E;i=e;return j|0}else if((H|0)==48){E=c[F>>2]|0;AN(0,16,1637504,(D=i,i=i+16|0,c[D>>2]=b,c[D+8>>2]=E,D)|0);i=D;j=I;i=e;return j|0}else if((H|0)==54){i=e;return j|0}return 0}function dM(b,e){b=b|0;e=e|0;var f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0;f=i;i=i+176|0;g=f|0;j=f+8|0;k=f+16|0;l=f+24|0;m=f+152|0;n=f+160|0;o=f+168|0;c[n>>2]=0;p=b+16|0;q=c[p>>2]|0;do{if((q|0)==0){r=0}else{s=jj(q)|0;if((s|0)!=0){r=s;break}AN(0,8,1640504,(t=i,i=i+8|0,c[t>>2]=c[p>>2],t)|0);i=t;eW(1);r=0}}while(0);p=(a8(e|0,1658232)|0)==0?1658072:e;if((ba(p|0,1658072,5)|0)==0){u=0}else{u=(a8(p|0,1640360)|0)!=0|0}c[18294]=c[18294]&u;u=lV()|0;c[k>>2]=u;if((u|0)==0){fe(p,-12);eW(1)}u=c[b+40>>2]|0;if((u|0)!=0){e=l|0;q=c[(c[b+36>>2]|0)+(u-1<<4)+8>>2]|0;bc(e|0,128,1670504,(t=i,i=i+8|0,c[t>>2]=q,t)|0)|0;i=t;q=(c[b>>2]|0)+20|0;zK(q,1640264,e,0)|0}e=b+32|0;q=(r|0)==0;do{if(!((c[e>>2]|0)==0|q)){u=r+20|0;if((c[u>>2]|0)==0){break}if((Bc(u,1685552,0,0,2)|0)==0){break}u=l|0;s=c[(c[b+28>>2]|0)+((c[e>>2]|0)-1<<4)+8>>2]|0;bc(u|0,128,1670504,(t=i,i=i+8|0,c[t>>2]=s,t)|0)|0;i=t;s=(c[b>>2]|0)+20|0;zK(s,1685552,u,0)|0}}while(0);e=b+48|0;do{if(!((c[e>>2]|0)==0|q)){l=r+20|0;if((c[l>>2]|0)==0){break}if((Bc(l,1640128,0,0,2)|0)==0){break}l=(c[b>>2]|0)+20|0;u=c[(c[b+44>>2]|0)+((c[e>>2]|0)-1<<4)+8>>2]|0;zK(l,1640128,u,0)|0}}while(0);q=c[b+56>>2]|0;if((q|0)!=0){u=(c[b>>2]|0)+20|0;l=c[(c[b+52>>2]|0)+(q-1<<4)+8>>2]|0;zK(u,1639960,l,0)|0}l=c[b+64>>2]|0;if((l|0)!=0){u=(c[b>>2]|0)+20|0;q=c[(c[b+60>>2]|0)+(l-1<<4)+8>>2]|0;zK(u,1639816,q,0)|0}q=c[b+24>>2]|0;u=(q|0)>0;do{if(u){l=c[b+20>>2]|0;s=0;v=0;while(1){if((a8(c[l+(v<<4)>>2]|0,1636312)|0)==0){w=c[l+(v<<4)+8>>2]|0}else{w=s}x=v+1|0;if((x|0)<(q|0)){s=w;v=x}else{break}}do{if(u){v=c[b+20>>2]|0;s=0;l=0;while(1){if((a8(c[v+(l<<4)>>2]|0,1639712)|0)==0){y=c[v+(l<<4)+8>>2]|0}else{y=s}x=l+1|0;if((x|0)<(q|0)){s=y;l=x}else{break}}if(!u){z=0;A=y;break}l=c[b+20>>2]|0;s=0;v=0;while(1){if((a8(c[l+(v<<4)>>2]|0,1704760)|0)==0){B=c[l+(v<<4)+8>>2]|0}else{B=s}x=v+1|0;if((x|0)<(q|0)){s=B;v=x}else{z=B;A=y;break}}}else{z=0;A=0}}while(0);if((w|0)==0){C=0}else{C=c[(er(w,0,0)|0)+12>>2]|0}c[(c[k>>2]|0)+1112>>2]=C;if((A|0)==0){D=0}else{D=c[(er(A,1,0)|0)+12>>2]|0}c[(c[k>>2]|0)+1116>>2]=D;if((z|0)==0){E=0;break}E=c[(er(z,3,0)|0)+12>>2]|0}else{c[(c[k>>2]|0)+1112>>2]=0;c[(c[k>>2]|0)+1116>>2]=0;E=0}}while(0);c[(c[k>>2]|0)+1120>>2]=E;E=(c[k>>2]|0)+1084|0;c[E>>2]=c[E>>2]|4;E=(c[k>>2]|0)+1160|0;z=172928;D=c[z+4>>2]|0;c[E>>2]=c[z>>2];c[E+4>>2]=D;D=b|0;E=mh(k,p,r,(c[D>>2]|0)+20|0)|0;if((E|0)<0){fe(p,E);eW(1)}eF(c[(c[D>>2]|0)+20>>2]|0);E=c[k>>2]|0;if((c[E+24>>2]|0)==0){F=E}else{r=0;z=E;while(1){et(b,z,c[(c[z+28>>2]|0)+(r<<2)>>2]|0)|0;E=r+1|0;A=c[k>>2]|0;if(E>>>0<(c[A+24>>2]|0)>>>0){r=E;z=A}else{F=A;break}}}z=fC(F,c[(c[D>>2]|0)+16>>2]|0)|0;c[m>>2]=z;F=c[k>>2]|0;r=c[F+24>>2]|0;if((mF(F,z)|0)<0){AN(0,8,1639536,(t=i,i=i+8|0,c[t>>2]=p,t)|0);i=t;mJ(k);eW(1)}z=b+8|0;F=c[z>>2]|0;A=c[z+4>>2]|0;z=c[k>>2]|0;E=z+1056|0;C=c[E>>2]|0;w=c[E+4>>2]|0;E=(C|0)==0&(w|0)==(-2147483648|0);y=NU(E?0:C,E?0:w,F,A)|0;w=G;do{if(!((F|0)==0&(A|0)==0)){if((my(z,-1,0,-2147483648,y,w,y,w,0)|0)>=0){break}AN(0,24,1639296,(t=i,i=i+16|0,c[t>>2]=p,h[t+8>>3]=(+(y>>>0)+ +(w|0)*4294967296.0)/1.0e6,t)|0);i=t}}while(0);z=c[k>>2]|0;A=z+24|0;do{if((c[A>>2]|0)==0){H=z}else{F=z+28|0;E=b+88|0;C=b+232|0;B=b+360|0;q=b+304|0;u=b+300|0;v=b+44|0;s=b+400|0;l=b+396|0;x=b+368|0;I=b+376|0;J=b+372|0;K=b+364|0;L=b+356|0;M=b+228|0;N=b+84|0;O=0;P=0;L147:while(1){Q=c[(c[F>>2]|0)+(P<<2)>>2]|0;R=Q+8|0;S=c[R>>2]|0;T=A7(256)|0;U=T;if((T|0)==0){eW(1)}V=fD(c[551808]|0,4,2203656,(c[550914]|0)+1|0)|0;c[551808]=V;c[V+((c[550914]|0)-1<<2)>>2]=U;V=T+4|0;c[V>>2]=Q;c[T>>2]=c[550916];c[T+8>>2]=1;c[Q+76>>2]=48;W=T+88|0;h[W>>3]=1.0;if((c[E>>2]|0)>0){X=0;do{Y=fA(z,Q,c[(c[N>>2]|0)+(X<<4)>>2]|0)|0;do{if((Y|0)>0){h[W>>3]=+h[(c[N>>2]|0)+(X<<4)+8>>3]}else{if((Y|0)>=0){break}eW(1)}}while(0);X=X+1|0;}while((X|0)<(c[E>>2]|0))}if((c[C>>2]|0)>0){X=0;W=O;while(1){Y=fA(z,Q,c[(c[M>>2]|0)+(X<<4)>>2]|0)|0;do{if((Y|0)>0){Z=c[(c[M>>2]|0)+(X<<4)+8>>2]|0}else{if((Y|0)>=0){Z=W;break}eW(1);Z=W}}while(0);Y=X+1|0;if((Y|0)<(c[C>>2]|0)){X=Y;W=Z}else{_=Z;break}}}else{_=O}if((_|0)!=0){W=aU(_|0,j|0,0)|0;if((a[c[j>>2]|0]|0)==0){$=W}else{W=_;$=d[W]|d[W+1|0]<<8|d[W+2|0]<<16|d[W+3|0]<<24|0}c[(c[R>>2]|0)+52>>2]=$}W=et(b,z,Q)|0;X=T+16|0;c[X>>2]=W;c[T+108>>2]=fB(c[(c[D>>2]|0)+16>>2]|0,c[(c[(c[V>>2]|0)+8>>2]|0)+48>>2]|0,z,Q,W)|0;W=T+252|0;c[W>>2]=-1;if((c[B>>2]|0)>0){Y=0;do{aa=fA(z,Q,c[(c[L>>2]|0)+(Y<<4)>>2]|0)|0;do{if((aa|0)>0){c[W>>2]=c[(c[L>>2]|0)+(Y<<4)+8>>2]}else{if((aa|0)>=0){break}eW(1)}}while(0);Y=Y+1|0;}while((Y|0)<(c[B>>2]|0))}Y=T+80|0;c[Y>>2]=0;c[Y+4>>2]=-2147483648;L185:do{switch(c[S+8>>2]|0){case 0:{if((c[X>>2]|0)==0){c[X>>2]=o9(c[S+48>>2]|0)|0}if((c[S+800>>2]|0)!=0){Y=S+88|0;c[Y>>2]=c[Y>>2]|16384}c[T+128>>2]=c[S+124>>2];c[T+132>>2]=c[S+120>>2];c[T+136>>2]=c[S+140>>2];do{if((c[e>>2]|0)>0){Y=0;W=0;while(1){V=fA(z,Q,c[(c[v>>2]|0)+(Y<<4)>>2]|0)|0;do{if((V|0)>0){ab=c[(c[v>>2]|0)+(Y<<4)+8>>2]|0}else{if((V|0)>=0){ab=W;break}eW(1);ab=W}}while(0);V=Y+1|0;if((V|0)<(c[e>>2]|0)){Y=V;W=ab}else{break}}if((ab|0)==0){break}if((BD(T+112|0,ab)|0)>=0){break}AN(0,16,1638160,(t=i,i=i+8|0,c[t>>2]=ab,t)|0);i=t;eW(1)}}while(0);W=T+120|0;c[W>>2]=-1;if((c[q>>2]|0)>0){ac=0}else{break L185}do{Y=fA(z,Q,c[(c[u>>2]|0)+(ac<<4)>>2]|0)|0;do{if((Y|0)>0){c[W>>2]=c[(c[u>>2]|0)+(ac<<4)+8>>2]}else{if((Y|0)>=0){break}eW(1)}}while(0);ac=ac+1|0;}while((ac|0)<(c[q>>2]|0));break};case 1:{W=T+124|0;c[W>>2]=2147483647;if((c[s>>2]|0)>0){Y=0;do{V=fA(z,Q,c[(c[l>>2]|0)+(Y<<4)>>2]|0)|0;do{if((V|0)>0){c[W>>2]=c[(c[l>>2]|0)+(Y<<4)+8>>2]}else{if((V|0)>=0){break}eW(1)}}while(0);Y=Y+1|0;}while((Y|0)<(c[s>>2]|0))}eG(U)|0;c[T+140>>2]=c[S+420>>2];c[T+144>>2]=c[S+412>>2];c[T+148>>2]=c[S+416>>2];Y=S+448|0;W=c[Y+4>>2]|0;V=T+152|0;c[V>>2]=c[Y>>2];c[V+4>>2]=W;break};case 2:case 3:{if((c[X>>2]|0)==0){c[X>>2]=o9(c[S+48>>2]|0)|0}if((c[x>>2]|0)>0){W=T+160|0;V=0;do{Y=fA(z,Q,c[(c[K>>2]|0)+(V<<4)>>2]|0)|0;do{if((Y|0)>0){c[W>>2]=c[(c[K>>2]|0)+(V<<4)+8>>2]}else{if((Y|0)>=0){break}eW(1)}}while(0);V=V+1|0;}while((V|0)<(c[x>>2]|0))}if((c[I>>2]|0)>0){ad=0;ae=0}else{break L185}while(1){V=fA(z,Q,c[(c[J>>2]|0)+(ae<<4)>>2]|0)|0;do{if((V|0)>0){af=c[(c[J>>2]|0)+(ae<<4)+8>>2]|0}else{if((V|0)>=0){af=ad;break}eW(1);af=ad}}while(0);V=ae+1|0;if((V|0)<(c[I>>2]|0)){ad=af;ae=V}else{break}}if((af|0)==0){break L185}if((BC(S+120|0,S+124|0,af)|0)>=0){break L185}AN(0,8,1637912,(t=i,i=i+8|0,c[t>>2]=af,t)|0);i=t;eW(1);break};case 4:case-1:{break};default:{ag=178;break L147}}}while(0);S=P+1|0;if(S>>>0<(c[A>>2]|0)>>>0){O=_;P=S}else{ag=180;break}}if((ag|0)==178){cw();return 0}else if((ag|0)==180){H=c[k>>2]|0;break}}}while(0);mN(H,c[550916]|0,p,0);c[551810]=fD(c[551810]|0,4,2203664,(c[550916]|0)+1|0)|0;H=A7(48)|0;if((H|0)==0){eW(1)}c[(c[551810]|0)+((c[550916]|0)-1<<2)>>2]=H;ag=c[k>>2]|0;c[H>>2]=ag;_=ag+24|0;ag=H+12|0;c[ag>>2]=(c[550914]|0)-(c[_>>2]|0);A=b+72|0;af=(c[745332]|0)!=0;ae=NV(c[A>>2]|0,c[A+4>>2]|0,af?0:y,af?0:w)|0;w=H+16|0;c[w>>2]=ae;c[w+4>>2]=G;c[H+32>>2]=c[_>>2];c[H+40>>2]=c[b+80>>2];H=en(c[(c[D>>2]|0)+16>>2]|0)|0;c[n>>2]=H;D=c[ag>>2]|0;if((D|0)<(c[550914]|0)){ag=D;do{D=zJ(c[(c[(c[551808]|0)+(ag<<2)>>2]|0)+108>>2]|0,3119176,0,2)|0;if((D|0)!=0){_=D;do{zK(n,c[_>>2]|0,0,0)|0;_=zJ(c[(c[(c[551808]|0)+(ag<<2)>>2]|0)+108>>2]|0,3119176,_,2)|0;}while((_|0)!=0)}ag=ag+1|0;}while((ag|0)<(c[550914]|0));ah=c[n>>2]|0}else{ah=H}H=zJ(ah,3119176,0,2)|0;if((H|0)!=0){ah=o;ag=H;do{c[o>>2]=ox()|0;H=ag|0;_=Bc(ah,c[H>>2]|0,0,0,3)|0;if((_|0)!=0){if((c[_+40>>2]&2|0)==0){D=_+4|0;w=c[D>>2]|0;ae=(c[550916]|0)-1|0;AN(0,16,1639064,(t=i,i=i+32|0,c[t>>2]=c[H>>2],c[t+8>>2]=(w|0)==0?3119176:w,c[t+16>>2]=ae,c[t+24>>2]=p,t)|0);i=t;eW(1);ai=D}else{ai=_+4|0}_=c[ai>>2]|0;D=(c[550916]|0)-1|0;AN(0,24,1638728,(t=i,i=i+32|0,c[t>>2]=c[H>>2],c[t+8>>2]=(_|0)==0?3119176:_,c[t+16>>2]=D,c[t+24>>2]=p,t)|0);i=t}ag=zJ(c[n>>2]|0,3119176,ag,2)|0;}while((ag|0)!=0)}zL(n);n=b+96|0;ag=c[n>>2]|0;if((ag|0)>0){p=b+92|0;b=0;ai=c[k>>2]|0;ah=ag;while(1){if((c[ai+24>>2]|0)==0){aj=ai;ak=ah}else{ag=0;o=ai;do{D=c[(c[o+28>>2]|0)+(ag<<2)>>2]|0;do{if((fA(o,D,c[(c[p>>2]|0)+(b<<4)>>2]|0)|0)==1){_=c[(c[p>>2]|0)+(b<<4)+8>>2]|0;c[g>>2]=0;H=D+8|0;if((c[(c[H>>2]|0)+100>>2]|0)==0){ae=c[D>>2]|0;AN(0,24,1638656,(t=i,i=i+16|0,c[t>>2]=(c[550916]|0)-1,c[t+8>>2]=ae,t)|0);i=t;break}do{if((a[_]|0)==0){ae=zJ(c[D+88>>2]|0,1709336,0,0)|0;if((ae|0)==0){al=_;break}al=c[ae+4>>2]|0}else{al=_}}while(0);if((a[al]|0)==0){_=c[D>>2]|0;AN(0,8,1638496,(t=i,i=i+16|0,c[t>>2]=(c[550916]|0)-1,c[t+8>>2]=_,t)|0);i=t;eW(1)}eo(al);if((iU(g,al,2,172928,0)|0)<0){AN(0,8,1638344,(t=i,i=i+8|0,c[t>>2]=al,t)|0);i=t;eW(1)}_=c[H>>2]|0;ij(c[g>>2]|0,c[_+96>>2]|0,c[_+100>>2]|0);ik(c[g>>2]|0);_=c[g>>2]|0;iV(_)|0}}while(0);ag=ag+1|0;o=c[k>>2]|0;}while(ag>>>0<(c[o+24>>2]|0)>>>0);aj=o;ak=c[n>>2]|0}ag=b+1|0;if((ag|0)<(ak|0)){b=ag;ai=aj;ah=ak}else{break}}}if((r|0)>0){am=0}else{an=m;A6(an);i=f;return 0}do{zL((c[m>>2]|0)+(am<<2)|0);am=am+1|0;}while((am|0)<(r|0));an=m;A6(an);i=f;return 0}function dN(b,d){b=b|0;d=d|0;var e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0.0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0;e=i;i=i+200|0;f=e|0;h=e+8|0;j=e+16|0;k=e+24|0;l=e+32|0;m=e+40|0;n=e+48|0;o=e+56|0;p=e+64|0;q=e+72|0;r=e+80|0;s=e+88|0;t=e+96|0;u=e+104|0;v=e+168|0;w=e+176|0;x=e+184|0;y=e+192|0;c[r>>2]=0;z=c[550922]|0;L307:do{if((z|0)>0){A=0;B=z;while(1){C=c[(c[556308]|0)+(A<<2)>>2]|0;if((c[C+8>>2]|0)==0){if((ey(C)|0)<0){break}D=c[550922]|0}else{D=B}C=A+1|0;if((C|0)<(D|0)){A=C;B=D}else{break L307}}AN(0,8,1654520,(E=i,i=i+1|0,i=i+7&-8,c[E>>2]=0,E)|0);i=E;eW(1)}}while(0);D=b+152|0;z=c[D>>2]|0;B=c[D+4>>2]|0;do{if(!((z|0)==(-1|0)&(B|0)==2147483647)){A=b+144|0;if((c[A>>2]|0)==(-1|0)&(c[A+4>>2]|0)==2147483647){F=B;H=z}else{c[D>>2]=-1;c[D+4>>2]=2147483647;AN(0,24,1654368,(E=i,i=i+1|0,i=i+7&-8,c[E>>2]=0,E)|0);i=E;C=c[D>>2]|0;I=c[D+4>>2]|0;if((C|0)==(-1|0)&(I|0)==2147483647){break}if((c[A>>2]|0)==(-1|0)&(c[A+4>>2]|0)==2147483647){F=I;H=C}else{break}}C=b+8|0;I=c[C>>2]|0;J=c[C+4>>2]|0;if((F|0)>(J|0)|(F|0)==(J|0)&H>>>0>I>>>0){C=NV(H,F,I,J)|0;c[A>>2]=C;c[A+4>>2]=G;break}else{AN(0,24,1654168,(E=i,i=i+1|0,i=i+7&-8,c[E>>2]=0,E)|0);i=E;c[D>>2]=-1;c[D+4>>2]=2147483647;break}}}while(0);c[550878]=fD(c[550878]|0,4,2203648,(c[550912]|0)+1|0)|0;D=A7(48)|0;if((D|0)==0){eW(1)}c[(c[550878]|0)+((c[550912]|0)-1<<2)>>2]=D;F=D+8|0;c[F>>2]=c[550910];H=b+144|0;z=c[H+4>>2]|0;B=D+16|0;c[B>>2]=c[H>>2];c[B+4>>2]=z;z=b+8|0;A=c[z+4>>2]|0;C=D+24|0;c[C>>2]=c[z>>2];c[C+4>>2]=A;A=b+160|0;z=c[A+4>>2]|0;J=D+32|0;c[J>>2]=c[A>>2];c[J+4>>2]=z;c[D+40>>2]=c[b+176>>2];z=D+4|0;J=b|0;zM(z,c[(c[J>>2]|0)+20>>2]|0,0);A=(a8(d|0,1658232)|0)==0?1658072:d;d=k6(q,0,c[b+16>>2]|0,A)|0;I=c[q>>2]|0;if((I|0)==0){fe(A,d);eW(1);K=c[q>>2]|0}else{K=I}I=D;c[I>>2]=K;D=c[H>>2]|0;d=c[H+4>>2]|0;if(!((D|0)==(-1|0)&(d|0)==2147483647)){L=K+1064|0;c[L>>2]=D;c[L+4>>2]=d}d=c[K+8>>2]|0;L=K+1160|0;K=172928;D=c[K>>2]|0;M=c[K+4>>2]|0;c[L>>2]=D;c[L+4>>2]=M;L=c[550922]|0;if((L|0)>0){K=b+180|0;N=b+184|0;O=b+188|0;P=0;Q=L;while(1){L=c[(c[556308]|0)+(P<<2)>>2]|0;R=L+28|0;S=c[R>>2]|0;if((S|0)>0){T=L+24|0;L=0;U=S;while(1){S=c[(c[T>>2]|0)+(L<<2)>>2]|0;V=c[S+16>>2]|0;do{if((V|0)==0){W=U}else{if((c[V>>2]|0)!=0){W=U;break}X=f6(c[(c[V+4>>2]|0)+28>>2]|0,c[V+8>>2]|0)|0;if((X|0)==0){c[K>>2]=1}else if((X|0)==1){c[N>>2]=1}else if((X|0)==3){c[O>>2]=1}ej(S,b,c[q>>2]|0);W=c[R>>2]|0}}while(0);S=L+1|0;if((S|0)<(W|0)){L=S;U=W}else{break}}Y=c[550922]|0}else{Y=Q}U=P+1|0;if((U|0)<(Y|0)){P=U;Q=Y}else{break}}}Y=d|0;do{if((a8(c[Y>>2]|0,1654072)|0)==0){if((yY(A,1654032,0)|0)==0){break}d=e$(b,1738464,1653944,117856)|0;if((d|0)>=0){break}fe(A,d);eW(1)}}while(0);d=a8(c[Y>>2]|0,1654072)|0;do{if((c[550874]|d|0)==0){if((yY(A,1654032,0)|0)==0){Z=283;break}Y=c[q>>2]|0;Q=lV()|0;c[p>>2]=Q;P=Q+1160|0;c[P>>2]=D;c[P+4>>2]=M;P=mh(p,A,0,0)|0;do{if((P|0)<0){fe(A,P);eW(1)}else{Q=c[p>>2]|0;if((c[Q+24>>2]|0)==0){mJ(p);break}else{_=0;$=Q}do{Q=o8(c[(c[(c[(c[$+28>>2]|0)+(_<<2)>>2]|0)+8>>2]|0)+48>>2]|0)|0;W=eq(b,Y,c[Q+8>>2]|0,-1)|0;O=c[W+12>>2]|0;N=O+8|0;K=c[N>>2]|0;c[W+60>>2]=Q;U=O;L=c[(c[(c[p>>2]|0)+28>>2]|0)+(_<<2)>>2]|0;NN(U|0,L|0,480)|0;L=O+208|0;c[L>>2]=0;c[L+4>>2]=0;L=A1(88)|0;c[O+184>>2]=L;U=c[(c[(c[(c[p>>2]|0)+28>>2]|0)+(_<<2)>>2]|0)+184>>2]|0;NN(L|0,U|0,88)|0;c[N>>2]=K;ow(K,c[(c[(c[(c[p>>2]|0)+28>>2]|0)+(_<<2)>>2]|0)+8>>2]|0)|0;K=c[N>>2]|0;N=c[K+8>>2]|0;do{if((N|0)==1){if((c[W+344>>2]|0)!=0){break}ev(O,Q)}else if((N|0)==0){if((c[W+344>>2]|0)!=0){break}eu(O,Q,c[K+140>>2]|0)|0}}while(0);_=_+1|0;$=c[p>>2]|0;}while(_>>>0<(c[$+24>>2]|0)>>>0);mJ(p)}}while(0);Y=c[550910]|0;P=Y-(c[(c[q>>2]|0)+24>>2]|0)|0;if((P|0)<(Y|0)){aa=P}else{break}do{P=c[(c[550876]|0)+(aa<<2)>>2]|0;Y=c[550914]|0;K=c[551808]|0;Q=P+12|0;O=0;while(1){if((O|0)>=(Y|0)){Z=279;break}ab=c[K+(O<<2)>>2]|0;ad=ab+4|0;ae=(c[Q>>2]|0)+8|0;if((c[(c[(c[ad>>2]|0)+8>>2]|0)+8>>2]|0)==(c[(c[ae>>2]|0)+8>>2]|0)){Z=274;break}else{O=O+1|0}}if((Z|0)==274){Z=0;K=P+24|0;c[K>>2]=ab;c[P+8>>2]=O;Y=c[(c[ae>>2]|0)+8>>2]|0;if((Y|0)==1){c[P+304>>2]=A9(1653880)|0;af=c[(c[(c[Q>>2]|0)+8>>2]|0)+8>>2]|0}else{af=Y}if((af|0)==0){c[P+304>>2]=A9(1653816)|0}c[ab+8>>2]=0;c[(c[ad>>2]|0)+76>>2]=-16;ag=K}else if((Z|0)==279){Z=0;ag=P+24|0}if((c[ag>>2]|0)==0){K=Cp(c[(c[(c[Q>>2]|0)+8>>2]|0)+8>>2]|0)|0;AN(0,8,1653648,(E=i,i=i+8|0,c[E>>2]=K,E)|0);i=E;eW(1)}aa=aa+1|0;}while((aa|0)<(c[550910]|0))}else{Z=283}}while(0);L396:do{if((Z|0)==283){aa=b+104|0;ag=c[aa>>2]|0;if((ag|0)!=0){if((ag|0)<=0){break}ag=b+100|0;ad=b+188|0;ab=b+184|0;af=b+180|0;ae=b+192|0;p=0;while(1){$=c[ag>>2]|0;L403:do{if((c[$+(p*24|0)>>2]|0)==0){_=c[$+(p*24|0)+20>>2]|0;if((_|0)!=0){M=c[550922]|0;L407:do{if((M|0)>0){D=c[556308]|0;d=0;L409:while(1){K=c[D+(d<<2)>>2]|0;Y=c[K+28>>2]|0;if((Y|0)>0){W=c[K+24>>2]|0;K=0;do{ah=c[W+(K<<2)>>2]|0;N=c[ah+16>>2]|0;if((N|0)!=0){if((a8(c[N>>2]|0,_|0)|0)==0){break L409}}K=K+1|0;}while((K|0)<(Y|0))}Y=d+1|0;if((Y|0)<(M|0)){d=Y}else{Z=330;break L407}}if((ah|0)==0){Z=330}else{ai=ah}}else{Z=330}}while(0);if((Z|0)==330){Z=0;AN(0,8,1653360,(E=i,i=i+8|0,c[E>>2]=_,E)|0);i=E;eW(1);ai=0}ej(ai,b,c[q>>2]|0);break}M=c[$+(p*24|0)+4>>2]|0;d=c[$+(p*24|0)+8>>2]|0;D=d+(c[(c[(c[551810]|0)+(M<<2)>>2]|0)+12>>2]|0)|0;Y=c[(c[551808]|0)+(D<<2)>>2]|0;if((c[ad>>2]|0)!=0){if((c[(c[(c[Y+4>>2]|0)+8>>2]|0)+8>>2]|0)==3){break}}if((c[ab>>2]|0)!=0){if((c[(c[(c[Y+4>>2]|0)+8>>2]|0)+8>>2]|0)==1){break}}if((c[af>>2]|0)!=0){if((c[(c[(c[Y+4>>2]|0)+8>>2]|0)+8>>2]|0)==0){break}}K=c[(c[(c[Y+4>>2]|0)+8>>2]|0)+8>>2]|0;if((c[ae>>2]|0)!=0&(K|0)==2){break}switch(K|0){case 0:{ek(b,c[q>>2]|0,D)|0;break L403;break};case 1:{el(b,c[q>>2]|0,D)|0;break L403;break};case 3:{em(b,c[q>>2]|0,D);break L403;break};case 2:{if((c[(eq(b,c[q>>2]|0,2,D)|0)+344>>2]|0)!=0){break L403}AN(0,8,1644600,(E=i,i=i+1|0,i=i+7&-8,c[E>>2]=0,E)|0);i=E;eW(1);break L403;break};case 4:{K=eq(b,c[q>>2]|0,4,D)|0;c[K+344>>2]=1;c[K+336>>2]=1;break L403;break};default:{AN(0,8,1653136,(E=i,i=i+16|0,c[E>>2]=M,c[E+8>>2]=d,E)|0);i=E;eW(1);break L403}}}}while(0);p=p+1|0;if((p|0)>=(c[aa>>2]|0)){break L396}}}do{if((c[b+180>>2]|0)==0){aa=c[(c[q>>2]|0)+8>>2]|0;p=c[aa+20>>2]|0;if((p|0)==0){break}ae=mW(aa,p,0)|0;p=c[550914]|0;if((p|0)<=0){break}aa=c[551808]|0;af=(ae|0)==1128878145;ae=-1;ab=0;ad=0;while(1){ag=c[(c[aa+(ad<<2)>>2]|0)+4>>2]|0;$=c[ag+8>>2]|0;Q=ac(c[$+124>>2]|0,c[$+120>>2]|0)|0;if(af){aj=Q}else{aj=(c[ag+72>>2]&1024|0)==0?Q:1}do{if((c[$+8>>2]|0)==0&(aj|0)>(ab|0)){if(af){if((c[ag+72>>2]&1024|0)==0){ak=ab;al=ae;break}}ak=aj;al=ad}else{ak=ab;al=ae}}while(0);ag=ad+1|0;if((ag|0)<(p|0)){ae=al;ab=ak;ad=ag}else{break}}if((al|0)<=-1){break}ek(b,c[q>>2]|0,al)|0}}while(0);do{if((c[b+184>>2]|0)==0){ad=c[q>>2]|0;if((c[(c[ad+8>>2]|0)+16>>2]|0)==0){break}ab=c[550914]|0;if((ab|0)<=0){break}ae=c[551808]|0;p=0;af=-1;aa=0;while(1){ag=c[(c[(c[ae+(aa<<2)>>2]|0)+4>>2]|0)+8>>2]|0;if((c[ag+8>>2]|0)==1){$=c[ag+416>>2]|0;ag=($|0)>(p|0);am=ag?aa:af;an=ag?$:p}else{am=af;an=p}$=aa+1|0;if(($|0)<(ab|0)){p=an;af=am;aa=$}else{break}}if((am|0)<=-1){break}el(b,ad,am)|0}}while(0);aa=c[b+24>>2]|0;if((aa|0)>0){af=c[b+20>>2]|0;p=0;ab=0;while(1){if((a8(c[af+(p<<4)>>2]|0,1704760)|0)==0){ao=c[af+(p<<4)+8>>2]|0}else{ao=ab}ae=p+1|0;if((ae|0)<(aa|0)){p=ae;ab=ao}else{ap=ao;break}}}else{ap=0}if((c[b+188>>2]|0)!=0){break}ab=c[q>>2]|0;if((c[(c[ab+8>>2]|0)+24>>2]|0)==0&(ap|0)==0){break}p=c[550914]|0;aa=c[551808]|0;af=0;while(1){if((af|0)>=(p|0)){break L396}if((c[(c[(c[(c[aa+(af<<2)>>2]|0)+4>>2]|0)+8>>2]|0)+8>>2]|0)==3){break}else{af=af+1|0}}em(b,ab,af)}}while(0);ap=b+132|0;if((c[ap>>2]|0)>0){ao=b+128|0;am=0;do{if((iU(s,c[(c[ao>>2]|0)+(am<<2)>>2]|0,1,172928,0)|0)<0){AN(0,8,1652936,(E=i,i=i+8|0,c[E>>2]=c[(c[ao>>2]|0)+(am<<2)>>2],E)|0);i=E;eW(1)}an=ip(c[s>>2]|0)|0;al=G;ak=0;if((al|0)<(ak|0)|(al|0)==(ak|0)&an>>>0<1>>>0){AN(0,8,1652664,(E=i,i=i+8|0,c[E>>2]=c[(c[ao>>2]|0)+(am<<2)>>2],E)|0);i=E;eW(1)}ak=an;an=A1(ak)|0;if((an|0)==0){AN(0,8,1652352,(E=i,i=i+8|0,c[E>>2]=c[(c[ao>>2]|0)+(am<<2)>>2],E)|0);i=E;eW(1)}iE(c[s>>2]|0,an,ak)|0;al=eq(b,c[q>>2]|0,4,-1)|0;aj=al+336|0;c[aj>>2]=1;c[al+344>>2]=0;c[al+348>>2]=c[(c[ao>>2]|0)+(am<<2)>>2];c[aj>>2]=1;aj=al+12|0;c[(c[(c[aj>>2]|0)+8>>2]|0)+96>>2]=an;c[(c[(c[aj>>2]|0)+8>>2]|0)+100>>2]=ak;ak=c[(c[ao>>2]|0)+(am<<2)>>2]|0;an=br(ak|0,47)|0;if((an|0)==0){aq=ak}else{aq=(a[an]|0)==0?ak:an+1|0}zK((c[aj>>2]|0)+88|0,1709336,aq,16)|0;iV(c[s>>2]|0)|0;am=am+1|0;}while((am|0)<(c[ap>>2]|0))}ap=c[550910]|0;am=ap-(c[(c[q>>2]|0)+24>>2]|0)|0;if((am|0)<(ap|0)){ap=am;do{am=c[(c[550876]|0)+(ap<<2)>>2]|0;if((c[am+344>>2]|0)==0){if((c[am+348>>2]|0)!=0){Z=360}}else{Z=360}do{if((Z|0)==360){Z=0;s=zJ(c[(c[J>>2]|0)+16>>2]|0,1630640,0,2)|0;if((s|0)==0){break}aq=c[s>>2]|0;if((a[aq+5|0]|0)==0){ar=am+12|0}else{ao=am+12|0;if((fA(c[q>>2]|0,c[ao>>2]|0,aq+6|0)|0)==0){break}else{ar=ao}}if((Bd(c[(c[ar>>2]|0)+8>>2]|0,1630640,c[s+4>>2]|0,0)|0)>=0){break}eW(1)}}while(0);ap=ap+1|0;}while((ap|0)<(c[550910]|0))}ap=en(c[(c[J>>2]|0)+16>>2]|0)|0;c[r>>2]=ap;J=c[F>>2]|0;if((J|0)<(c[550910]|0)){ar=J;do{J=zJ(c[(c[(c[550876]|0)+(ar<<2)>>2]|0)+320>>2]|0,3119176,0,2)|0;if((J|0)!=0){am=J;do{zK(r,c[am>>2]|0,0,0)|0;am=zJ(c[(c[(c[550876]|0)+(ar<<2)>>2]|0)+320>>2]|0,3119176,am,2)|0;}while((am|0)!=0)}ar=ar+1|0;}while((ar|0)<(c[550910]|0));as=c[r>>2]|0}else{as=ap}ap=zJ(as,3119176,0,2)|0;if((ap|0)!=0){as=t;ar=ap;do{c[t>>2]=ox()|0;ap=ar|0;am=Bc(as,c[ap>>2]|0,0,0,3)|0;do{if((am|0)!=0){if((c[am+40>>2]&1|0)==0){J=c[am+4>>2]|0;af=(c[550912]|0)-1|0;AN(0,16,1652192,(E=i,i=i+32|0,c[E>>2]=c[ap>>2],c[E+8>>2]=(J|0)==0?3119176:J,c[E+16>>2]=af,c[E+24>>2]=A,E)|0);i=E;eW(1)}af=c[ap>>2]|0;if((a8(af|0,1668496)|0)==0){break}J=c[am+4>>2]|0;ab=(c[550912]|0)-1|0;AN(0,24,1651840,(E=i,i=i+32|0,c[E>>2]=af,c[E+8>>2]=(J|0)==0?3119176:J,c[E+16>>2]=ab,c[E+24>>2]=A,E)|0);i=E}}while(0);ar=zJ(c[r>>2]|0,3119176,ar,2)|0;}while((ar|0)!=0)}zL(r);r=c[q>>2]|0;do{if((c[(c[r+8>>2]|0)+28>>2]&2|0)!=0){if((md(r+32|0)|0)!=0){break}fe((c[q>>2]|0)+32|0,-22);eW(1)}}while(0);r=c[(c[q>>2]|0)+8>>2]|0;do{if((c[r+28>>2]&1|0)==0){eo(A);ar=c[q>>2]|0;as=iU(ar+16|0,A,2,ar+1160|0,z)|0;if((as|0)>=0){break}fe(A,as);eW(1)}else{if((a8(c[r>>2]|0,1651768)|0)!=0){break}if((md(A)|0)!=0){break}eo(A)}}while(0);at=+g[b+168>>2];if(at!=0.0){A=u|0;u=~~(at*1.0e6);bc(A|0,64,1670504,(E=i,i=i+8|0,c[E>>2]=u,E)|0)|0;i=E;zK(z,1651696,A,0)|0}c[(c[q>>2]|0)+1080>>2]=~~(+g[b+172>>2]*1.0e6);A=b+312|0;L556:do{if((c[A>>2]|0)>0){z=b+308|0;u=b+116|0;r=b+120|0;as=b+124|0;ar=0;L558:while(1){t=aU(c[(c[z>>2]|0)+(ar<<4)+8>>2]|0,v|0,0)|0;if((t|0)>=(c[550916]|0)){AN(0,8,1651504,(E=i,i=i+8|0,c[E>>2]=t,E)|0);i=E;eW(1)}am=c[(c[z>>2]|0)+(ar<<4)>>2]|0;ap=c[v>>2]|0;ab=(a[ap]|0)!=0?ap+1|0:ap;ap=c[q>>2]|0;if((t|0)>-1){au=c[c[(c[551810]|0)+(t<<2)>>2]>>2]|0}else{au=0}c[l>>2]=0;c[m>>2]=0;c[n>>2]=0;c[o>>2]=0;ep(ab,j,n,l);ep(am,k,o,m);do{if((au|0)==0){ab=a[k]|0;if(ab<<24>>24==103){Z=400}else{if((a[am]|0)==0){Z=400}}if((Z|0)==400){Z=0;c[u>>2]=1}if(ab<<24>>24==115){Z=403}else{if((a[am]|0)==0){Z=403}}if((Z|0)==403){Z=0;c[r>>2]=1}if(ab<<24>>24!=99){if((a[am]|0)!=0){break}}c[as>>2]=1}else{ab=a[j]|0;if(ab<<24>>24==103){Z=409}else{if((a[k]|0)==103){Z=409}}if((Z|0)==409){Z=0;c[u>>2]=1}t=ab<<24>>24==115;if(t){Z=412}else{if((a[k]|0)==115){Z=412}}if((Z|0)==412){Z=0;c[r>>2]=1}if(ab<<24>>24==99){Z=415}else{if((a[k]|0)==99){Z=415}}if((Z|0)==415){Z=0;c[as>>2]=1}J=ab<<24>>24;if((J|0)==103){av=au+1140|0}else if((J|0)==99){ab=c[n>>2]|0;if((ab|0)<0){Z=420}else{if(ab>>>0>=(c[au+1132>>2]|0)>>>0){Z=420}}if((Z|0)==420){Z=0;AN(0,8,1650368,(E=i,i=i+16|0,c[E>>2]=1650280,c[E+8>>2]=ab,E)|0);i=E;eW(1)}av=(c[(c[au+1136>>2]|0)+(ab<<2)>>2]|0)+32|0}else if((J|0)==112){ab=c[n>>2]|0;if((ab|0)<0){Z=424}else{if(ab>>>0>=(c[au+1104>>2]|0)>>>0){Z=424}}if((Z|0)==424){Z=0;AN(0,8,1650368,(E=i,i=i+16|0,c[E>>2]=1650240,c[E+8>>2]=ab,E)|0);i=E;eW(1)}av=(c[(c[au+1108>>2]|0)+(ab<<2)>>2]|0)+20|0}else if((J|0)==115){av=0}else{Z=426;break L558}J=a[k]|0;ab=J<<24>>24;if((ab|0)==103){aw=ap+1140|0}else if((ab|0)==99){af=c[o>>2]|0;if((af|0)<0){Z=431}else{if(af>>>0>=(c[ap+1132>>2]|0)>>>0){Z=431}}if((Z|0)==431){Z=0;AN(0,8,1650368,(E=i,i=i+16|0,c[E>>2]=1650280,c[E+8>>2]=af,E)|0);i=E;eW(1)}aw=(c[(c[ap+1136>>2]|0)+(af<<2)>>2]|0)+32|0}else if((ab|0)==112){af=c[o>>2]|0;if((af|0)<0){Z=435}else{if(af>>>0>=(c[ap+1104>>2]|0)>>>0){Z=435}}if((Z|0)==435){Z=0;AN(0,8,1650368,(E=i,i=i+16|0,c[E>>2]=1650240,c[E+8>>2]=af,E)|0);i=E;eW(1)}aw=(c[(c[ap+1108>>2]|0)+(af<<2)>>2]|0)+20|0}else if((ab|0)==115){aw=0}else{Z=437;break L558}do{if(t){ab=au+24|0;L629:do{if((c[ab>>2]|0)==0){ax=av}else{af=au+28|0;s=c[l>>2]|0;ao=0;while(1){aq=fA(au,c[(c[af>>2]|0)+(ao<<2)>>2]|0,s)|0;if((aq|0)>0){break}if((aq|0)<0){eW(1)}aq=ao+1|0;if(aq>>>0<(c[ab>>2]|0)>>>0){ao=aq}else{ax=av;break L629}}ax=(c[(c[af>>2]|0)+(ao<<2)>>2]|0)+88|0}}while(0);if((ax|0)!=0){ay=ax;break}AN(0,8,1649608,(E=i,i=i+8|0,c[E>>2]=c[l>>2],E)|0);i=E;eW(1);ay=0}else{ay=av}}while(0);if(J<<24>>24!=115){zM(aw,c[ay>>2]|0,16);break}t=ap+24|0;if((c[t>>2]|0)==0){break}ab=ap+28|0;s=c[m>>2]|0;aq=0;do{aj=fA(ap,c[(c[ab>>2]|0)+(aq<<2)>>2]|0,s)|0;do{if((aj|0)>0){zM((c[(c[ab>>2]|0)+(aq<<2)>>2]|0)+88|0,c[ay>>2]|0,16)}else{if((aj|0)>=0){break}eW(1)}}while(0);aq=aq+1|0;}while(aq>>>0<(c[t>>2]|0)>>>0)}}while(0);ar=ar+1|0;if((ar|0)>=(c[A>>2]|0)){break L556}}if((Z|0)==426){AN(0,0,165e4,(E=i,i=i+24|0,c[E>>2]=1660512,c[E+8>>2]=1649880,c[E+16>>2]=471,E)|0);i=E;cw();return 0}else if((Z|0)==437){AN(0,0,165e4,(E=i,i=i+24|0,c[E>>2]=1660512,c[E+8>>2]=1649880,c[E+16>>2]=472,E)|0);i=E;cw();return 0}}}while(0);A=b+136|0;ay=c[A>>2]|0;L657:do{if((ay|0)<(c[550916]|0)){az=ay;Z=465}else{if((ay|0)!=2147483647){AN(0,8,1651312,(E=i,i=i+8|0,c[E>>2]=ay,E)|0);i=E;eW(1);az=c[A>>2]|0;Z=465;break}c[A>>2]=-1;m=c[550916]|0;aw=c[551810]|0;av=0;while(1){if((av|0)>=(m|0)){break L657}if((c[(c[c[aw+(av<<2)>>2]>>2]|0)+1132>>2]|0)==0){av=av+1|0}else{break}}c[A>>2]=av;az=av;Z=465}}while(0);L666:do{if((Z|0)==465){if((az|0)<=-1){break}A=c[(c[551810]|0)+(az<<2)>>2]|0;ay=(c[b+124>>2]|0)==0;aw=c[A>>2]|0;m=c[I>>2]|0;l=m+1136|0;ax=aw+1132|0;au=m+1132|0;m=A3(c[l>>2]|0,(c[au>>2]|0)+(c[ax>>2]|0)|0,4)|0;if((m|0)==0){break}c[l>>2]=m;if((c[ax>>2]|0)==0){break}m=aw+1136|0;aw=A+16|0;A=f|0;o=f+4|0;k=h|0;n=h+4|0;if(ay){ay=0;do{j=c[(c[m>>2]|0)+(ay<<2)>>2]|0;v=NV(c[C>>2]|0,c[C+4>>2]|0,c[aw>>2]|0,c[aw+4>>2]|0)|0;c[A>>2]=1;c[o>>2]=1e6;ar=j+4|0;as=AY(v,G,f,ar)|0;v=G;r=c[B>>2]|0;u=c[B+4>>2]|0;if((r|0)==(-1|0)&(u|0)==2147483647){aA=2147483647;aB=-1}else{c[k>>2]=1;c[n>>2]=1e6;z=AY(r,u,h,ar)|0;aA=G;aB=z}z=j+24|0;u=c[z+4>>2]|0;if(!((u|0)<(v|0)|(u|0)==(v|0)&(c[z>>2]|0)>>>0<as>>>0)){if(!((aB|0)==(-1|0)&(aA|0)==2147483647)){u=j+16|0;r=c[u>>2]|0;ap=c[u+4>>2]|0;u=NU(aB,aA,as,v)|0;am=G;if((ap|0)>(am|0)|(ap|0)==(am|0)&r>>>0>u>>>0){break L666}}u=A7(40)|0;if((u|0)==0){break L666}c[u>>2]=c[j>>2];r=ar;ar=u+4|0;am=c[r+4>>2]|0;c[ar>>2]=c[r>>2];c[ar+4>>2]=am;am=j+16|0;ar=NV(c[am>>2]|0,c[am+4>>2]|0,as,v)|0;am=G;r=0;ap=(am|0)<(r|0)|(am|0)==(r|0)&ar>>>0<0>>>0;r=u+16|0;c[r>>2]=ap?0:ar;c[r+4>>2]=ap?0:am;am=NV(c[z>>2]|0,c[z+4>>2]|0,as,v)|0;v=G;as=(aA|0)>(v|0)|(aA|0)==(v|0)&aB>>>0>am>>>0;z=u+24|0;c[z>>2]=as?am:aB;c[z+4>>2]=as?v:aA;zM(u+32|0,c[j+32>>2]|0,0);j=c[au>>2]|0;c[au>>2]=j+1;c[(c[l>>2]|0)+(j<<2)>>2]=u}ay=ay+1|0;}while(ay>>>0<(c[ax>>2]|0)>>>0)}else{ay=0;do{av=c[(c[m>>2]|0)+(ay<<2)>>2]|0;u=NV(c[C>>2]|0,c[C+4>>2]|0,c[aw>>2]|0,c[aw+4>>2]|0)|0;c[A>>2]=1;c[o>>2]=1e6;j=av+4|0;v=AY(u,G,f,j)|0;u=G;as=c[B>>2]|0;z=c[B+4>>2]|0;if((as|0)==(-1|0)&(z|0)==2147483647){aC=2147483647;aD=-1}else{c[k>>2]=1;c[n>>2]=1e6;am=AY(as,z,h,j)|0;aC=G;aD=am}am=av+24|0;z=c[am+4>>2]|0;if(!((z|0)<(u|0)|(z|0)==(u|0)&(c[am>>2]|0)>>>0<v>>>0)){if(!((aD|0)==(-1|0)&(aC|0)==2147483647)){z=av+16|0;as=c[z>>2]|0;ap=c[z+4>>2]|0;z=NU(aD,aC,v,u)|0;r=G;if((ap|0)>(r|0)|(ap|0)==(r|0)&as>>>0>z>>>0){break L666}}z=A7(40)|0;if((z|0)==0){break L666}c[z>>2]=c[av>>2];as=j;j=z+4|0;r=c[as+4>>2]|0;c[j>>2]=c[as>>2];c[j+4>>2]=r;r=av+16|0;av=NV(c[r>>2]|0,c[r+4>>2]|0,v,u)|0;r=G;j=0;as=(r|0)<(j|0)|(r|0)==(j|0)&av>>>0<0>>>0;j=z+16|0;c[j>>2]=as?0:av;c[j+4>>2]=as?0:r;r=NV(c[am>>2]|0,c[am+4>>2]|0,v,u)|0;u=G;v=(aC|0)>(u|0)|(aC|0)==(u|0)&aD>>>0>r>>>0;am=z+24|0;c[am>>2]=v?r:aD;c[am+4>>2]=v?u:aC;u=c[au>>2]|0;c[au>>2]=u+1;c[(c[l>>2]|0)+(u<<2)>>2]=z}ay=ay+1|0;}while(ay>>>0<(c[ax>>2]|0)>>>0)}}}while(0);if((c[b+116>>2]|0)==0&(c[550916]|0)!=0){zM((c[q>>2]|0)+1140|0,c[(c[c[c[551810]>>2]>>2]|0)+1140>>2]|0,16);if(!((c[H>>2]|0)==(-1|0)&(c[H+4>>2]|0)==2147483647)){H=(c[q>>2]|0)+1140|0;zK(H,1615e3,0,0)|0}H=(c[q>>2]|0)+1140|0;zK(H,1651224,0,0)|0}do{if((c[b+120>>2]|0)==0){H=c[F>>2]|0;aC=c[550910]|0;if((H|0)<(aC|0)){aE=H;aF=aC}else{break}while(1){aC=c[(c[550876]|0)+(aE<<2)>>2]|0;H=c[aC+8>>2]|0;if((H|0)<0){aG=aF}else{zM((c[aC+12>>2]|0)+88|0,c[(c[(c[(c[551808]|0)+(H<<2)>>2]|0)+4>>2]|0)+88>>2]|0,16);aG=c[550910]|0}H=aE+1|0;if((H|0)<(aG|0)){aE=H;aF=aG}else{break}}}}while(0);aG=b+208|0;if((c[aG>>2]|0)<=0){i=e;return 0}aF=b+204|0;b=0;aE=0;while(1){c[y>>2]=0;F=c[(c[aF>>2]|0)+(b<<4)+8>>2]|0;H=bi(F|0,61)|0;if((H|0)==0){AN(0,8,1651064,(E=i,i=i+8|0,c[E>>2]=F,E)|0);i=E;eW(1)}F=H+1|0;a[H]=0;ep(c[(c[aF>>2]|0)+(b<<4)>>2]|0,w,y,x);H=a[w]|0;do{if(H<<24>>24==115){aC=c[q>>2]|0;if((c[aC+24>>2]|0)==0){aH=aE;break}aD=c[x>>2]|0;h=0;B=aC;while(1){aC=fA(B,c[(c[B+28>>2]|0)+(h<<2)>>2]|0,aD)|0;do{if((aC|0)>0){f=(c[(c[(c[q>>2]|0)+28>>2]|0)+(h<<2)>>2]|0)+88|0;C=c[(c[aF>>2]|0)+(b<<4)+8>>2]|0;aA=(a[F]|0)!=0?F:0;zK(f,C,aA,0)|0}else{if((aC|0)>=0){break}eW(1)}}while(0);aC=h+1|0;aA=c[q>>2]|0;if(aC>>>0<(c[aA+24>>2]|0)>>>0){h=aC;B=aA}else{aH=aE;break}}}else{B=H<<24>>24;if((B|0)==103){aI=(c[q>>2]|0)+1140|0}else if((B|0)==99){B=c[y>>2]|0;if((B|0)<0){Z=510}else{h=c[q>>2]|0;if(B>>>0<(c[h+1132>>2]|0)>>>0){aJ=h}else{Z=510}}if((Z|0)==510){Z=0;AN(0,8,1650840,(E=i,i=i+8|0,c[E>>2]=B,E)|0);i=E;eW(1);aJ=c[q>>2]|0}aI=(c[(c[aJ+1136>>2]|0)+(B<<2)>>2]|0)+32|0}else{AN(0,8,1650640,(E=i,i=i+8|0,c[E>>2]=c[(c[aF>>2]|0)+(b<<4)>>2],E)|0);i=E;eW(1);aI=aE}zK(aI,c[(c[aF>>2]|0)+(b<<4)+8>>2]|0,(a[F]|0)!=0?F:0,0)|0;aH=aI}}while(0);F=b+1|0;if((F|0)<(c[aG>>2]|0)){b=F;aE=aH}else{break}}i=e;return 0}function dO(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;d=i;i=i+32|0;f=d|0;g=d+8|0;h=d+16|0;j=d+24|0;k=(a[e]|0)==45;l=k?e+1|0:e;e=A9(l)|0;c[h>>2]=e;m=bi(e|0,44)|0;c[g>>2]=m;do{if((m|0)==0){n=0;o=-1}else{a[m]=0;e=aU(m+1|0,g|0,0)|0;if((e|0)>=(c[550916]|0)|(e|0)<0){AN(0,8,1655512,(p=i,i=i+8|0,c[p>>2]=e,p)|0);i=p;eW(1)}q=c[g>>2]|0;if((a[q]|0)==0){r=q}else{s=q+1|0;c[g>>2]=s;r=s}s=0;while(1){q=c[(c[551810]|0)+(e<<2)>>2]|0;t=c[q+32>>2]|0;if((s|0)>=(t|0)){u=0;v=t;break}t=c[q>>2]|0;if((fA(t,c[(c[t+28>>2]|0)+(s<<2)>>2]|0,r)|0)==1){w=526;break}else{s=s+1|0}}if((w|0)==526){u=s;v=c[(c[(c[551810]|0)+(e<<2)>>2]|0)+32>>2]|0}if((s|0)!=(v|0)){n=u;o=e;break}AN(0,8,1655216,(p=i,i=i+8|0,c[p>>2]=l,p)|0);i=p;eW(1);n=u;o=e}}while(0);u=c[h>>2]|0;L759:do{if((a[u]|0)==91){c[j>>2]=u+1;v=b+100|0;r=b+104|0;g=fD(c[v>>2]|0,24,r,(c[r>>2]|0)+1|0)|0;c[v>>2]=g;v=(c[r>>2]|0)-1|0;r=g+(v*24|0)|0;m=y2(j,1655096)|0;c[g+(v*24|0)+20>>2]=m;if((m|0)!=0){x=r;w=548;break}AN(0,16,1654968,(p=i,i=i+8|0,c[p>>2]=c[h>>2],p)|0);i=p;eW(1);x=r;w=548}else{r=aU(u|0,f|0,0)|0;if((r|0)>=(c[550916]|0)|(r|0)<0){AN(0,8,1654792,(p=i,i=i+8|0,c[p>>2]=r,p)|0);i=p;eW(1)}if(k){m=b+104|0;if((c[m>>2]|0)<=0){break}v=b+100|0;g=c[f>>2]|0;t=g+1|0;q=0;while(1){y=c[v>>2]|0;z=y+(q*24|0)|0;do{if((r|0)==(c[y+(q*24|0)+4>>2]|0)){A=c[c[(c[551810]|0)+(r<<2)>>2]>>2]|0;if((fA(A,c[(c[A+28>>2]|0)+(c[y+(q*24|0)+8>>2]<<2)>>2]|0,(a[g]|0)==58?t:g)|0)<=0){break}c[z>>2]=1}}while(0);y=q+1|0;if((y|0)<(c[m>>2]|0)){q=y}else{x=z;w=548;break L759}}}q=c[(c[551810]|0)+(r<<2)>>2]|0;if((c[q+32>>2]|0)<=0){break}m=c[f>>2]|0;g=m+1|0;t=b+100|0;v=b+104|0;e=(o|0)>-1;s=0;y=0;A=q;while(1){q=c[A>>2]|0;do{if((fA(q,c[(c[q+28>>2]|0)+(y<<2)>>2]|0,(a[m]|0)==58?g:m)|0)<1){B=s}else{C=fD(c[t>>2]|0,24,v,(c[v>>2]|0)+1|0)|0;c[t>>2]=C;D=(c[v>>2]|0)-1|0;E=C+(D*24|0)|0;c[C+(D*24|0)+4>>2]=r;c[C+(D*24|0)+8>>2]=y;F=C+(D*24|0)+12|0;if(e){c[F>>2]=o;c[C+(D*24|0)+16>>2]=n;B=E;break}else{c[F>>2]=r;c[C+(D*24|0)+16>>2]=y;B=E;break}}}while(0);q=y+1|0;z=c[(c[551810]|0)+(r<<2)>>2]|0;if((q|0)<(c[z+32>>2]|0)){s=B;y=q;A=z}else{x=B;w=548;break}}}}while(0);do{if((w|0)==548){if((x|0)==0){break}G=h;A6(G);i=d;return 0}}while(0);AN(0,8,1654608,(p=i,i=i+8|0,c[p>>2]=l,p)|0);i=p;eW(1);G=h;A6(G);i=d;return 0}function dP(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;b=i;e=a+108|0;f=a+112|0;a=fD(c[e>>2]|0,20,f,(c[f>>2]|0)+1|0)|0;c[e>>2]=a;e=(c[f>>2]|0)-1|0;f=a+(e*20|0)|0;g=a+(e*20|0)+8|0;h=a+(e*20|0)+12|0;j=a+(e*20|0)+16|0;k=ct(d|0,1657368,(l=i,i=i+24|0,c[l>>2]=g,c[l+8>>2]=h,c[l+16>>2]=j,l)|0)|0;i=l;m=(k|0)==1;do{if((k|0)==3|(k|0)==1){if((c[g>>2]|0)!=-1){break}c[a+(e*20|0)+4>>2]=-1;c[f>>2]=-1;if(!m){i=b;return 0}c[j>>2]=-1;c[h>>2]=-1;i=b;return 0}}while(0);m=f|0;f=a+(e*20|0)+4|0;e=ct(d|0,1657216,(l=i,i=i+40|0,c[l>>2]=m,c[l+8>>2]=f,c[l+16>>2]=g,c[l+24>>2]=h,c[l+32>>2]=j,l)|0)|0;i=l;if((e|0)==3){n=559}else if((e|0)!=5){AN(0,8,1656808,(l=i,i=i+1|0,i=i+7&-8,c[l>>2]=0,l)|0);i=l;eW(1);n=559}if((n|0)==559){c[j>>2]=-1;c[h>>2]=-1}h=c[m>>2]|0;if(!((h|0)>-1&(h|0)<(c[550916]|0))){AN(0,8,1656448,(l=i,i=i+8|0,c[l>>2]=h,l)|0);i=l;eW(1)}h=c[f>>2]|0;j=c[m>>2]|0;if((h|0)<0){n=564}else{e=c[(c[551810]|0)+(j<<2)>>2]|0;if((h|0)<(c[e+32>>2]|0)){o=h;p=j;q=e}else{n=564}}if((n|0)==564){AN(0,8,1656144,(l=i,i=i+16|0,c[l>>2]=j,c[l+8>>2]=h,l)|0);i=l;eW(1);h=c[m>>2]|0;o=c[f>>2]|0;p=h;q=c[(c[551810]|0)+(h<<2)>>2]|0}h=(c[(c[(c[q>>2]|0)+28>>2]|0)+(o<<2)>>2]|0)+8|0;if((c[(c[h>>2]|0)+8>>2]|0)!=1){AN(0,8,1655872,(l=i,i=i+16|0,c[l>>2]=p,c[l+8>>2]=o,l)|0);i=l;eW(1)}o=c[g>>2]|0;do{if((o|0)>=0){if((o|0)>=(c[(c[h>>2]|0)+416>>2]|0)){break}i=b;return 0}}while(0);h=c[f>>2]|0;AN(0,8,1655664,(l=i,i=i+24|0,c[l>>2]=c[m>>2],c[l+8>>2]=h,c[l+16>>2]=o,l)|0);i=l;eW(1);i=b;return 0}function dQ(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0.0;e=i;i=i+184|0;f=e|0;g=e+128|0;h=e+136|0;j=eY(b,d,0)|0;k=(+(j>>>0)+ +(G|0)*4294967296.0)/1.0e6;j=+R(k)>=1.0?k>0.0?(ah(+Q(k/4294967296.0),4294967295.0)|0)>>>0:~~+ab((k- +(~~k>>>0))/4294967296.0)>>>0:0;c[g>>2]=~~k>>>0;c[g+4>>2]=j;j=h;d=bz(g|0)|0;NN(j|0,d|0,44)|0;d=f|0;bL(d|0,128,1657672,h|0)|0;e$(a,1738464,d,117856)|0;AN(0,24,1657440,(d=i,i=i+8|0,c[d>>2]=b,d)|0);i=d;i=e;return 0}function dR(a,b,c){a=a|0;b=b|0;c=c|0;return e$(a,1657736,c,117856)|0}function dS(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0;d=i;i=i+72|0;b=d|0;c[b>>2]=0;f=(a8(e|0,1658232)|0)==0?1658072:e;e=iU(b,f,2,172928,0)|0;if((e|0)<0){g=d+8|0;NM(g|0,0,64);a[g]=0;zN(e,g,64)|0;AN(0,16,1657944,(h=i,i=i+16|0,c[h>>2]=f,c[h+8>>2]=g,h)|0);i=h;j=e;i=d;return j|0}else{c[541010]=c[b>>2];j=0;i=d;return j|0}return 0}function dT(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;b=i;L829:do{if((ba(d|0,1666152,4)|0)==0){e=0;f=d+4|0}else{if((ba(d|0,1666088,5)|0)==0){e=1;f=d+5|0;break}if((ba(d|0,1666024,5)|0)==0){e=2;f=d+5|0;break}h=c[550916]|0;L838:do{if((h|0)>0){j=c[551810]|0;k=0;L840:while(1){l=c[j+(k<<2)>>2]|0;m=c[l+32>>2]|0;if((m|0)>0){n=c[(c[l>>2]|0)+28>>2]|0;l=0;do{o=c[(c[n+(l<<2)>>2]|0)+8>>2]|0;if((c[o+8>>2]|0)==0){p=((c[o+108>>2]|0)*1e3|0|0)/(c[o+104>>2]|0)|0;if((p|0)==25e3){q=0;break L840}if((p|0)==29970|(p|0)==23976){q=1;break L840}}l=l+1|0;}while((l|0)<(m|0))}k=k+1|0;if((k|0)>=(h|0)){break L838}}AN(0,32,1665880,(r=i,i=i+8|0,c[r>>2]=(q|0)==0?1665824:1665680,r)|0);i=r;e=q;f=d;break L829}}while(0);AN(0,8,1665432,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;AN(0,8,1665248,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;AN(0,8,1664992,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;eW(1);e=3;f=d}}while(0);if((a8(f|0,1664824)|0)==0){e$(a,1668144,1664584,117856)|0;e$(a,1670216,1664368,117856)|0;e$(a,1629968,1664824,117856)|0;d=a;q=(c[d>>2]|0)+16|0;zK(q,1669600,f,0)|0;q=(e|0)==0;h=q?1664312:1664104;e$(a,1704760,h,117856)|0;h=c[120792+(e<<2)>>2]|0;e$(a,1705336,h,117856)|0;h=(c[d>>2]|0)+16|0;k=q?1663864:1663584;zK(h,1663952,k,0)|0;k=(c[d>>2]|0)+16|0;zK(k,1669600,1663472,0)|0;k=(c[d>>2]|0)+16|0;zK(k,1663360,1663472,0)|0;k=(c[d>>2]|0)+16|0;zK(k,1663264,1663472,0)|0;k=(c[d>>2]|0)+16|0;zK(k,1663184,1663088,0)|0;k=(c[d>>2]|0)+16|0;zK(k,1662880,1662632,0)|0;e$(a,1686200,1662488,117856)|0;e$(a,1685800,1662376,117856)|0;k=(c[d>>2]|0)+20|0;zK(k,1662096,1661912,0)|0;k=(c[d>>2]|0)+20|0;zK(k,1661752,1661648,0)|0;g[a+168>>2]=.4399999976158142;s=0;i=b;return s|0}if((a8(f|0,1661552)|0)==0){e$(a,1668144,1661416,117856)|0;e$(a,1670216,1664368,117856)|0;e$(a,1629968,1661552,117856)|0;k=(e|0)==0;d=k?1661384:1661208;e$(a,1704760,d,117856)|0;d=c[120792+(e<<2)>>2]|0;e$(a,1705336,d,117856)|0;e$(a,1703792,1661032,117856)|0;d=a;h=(c[d>>2]|0)+16|0;q=k?1663864:1663584;zK(h,1663952,q,0)|0;q=(c[d>>2]|0)+16|0;zK(q,1669600,1660808,0)|0;q=(c[d>>2]|0)+16|0;zK(q,1663360,1660664,0)|0;q=(c[d>>2]|0)+16|0;zK(q,1663264,1660512,0)|0;q=(c[d>>2]|0)+16|0;zK(q,1663184,1660416,0)|0;q=(c[d>>2]|0)+16|0;zK(q,1660280,1660192,0)|0;q=(c[d>>2]|0)+16|0;zK(q,1662880,1662632,0)|0;e$(a,1686200,1662488,117856)|0;q=(c[d>>2]|0)+20|0;zK(q,1662096,1661912,0)|0;s=0;i=b;return s|0}if((a8(f|0,1660048)|0)==0){e$(a,1668144,1661416,117856)|0;e$(a,1670216,1660008,117856)|0;e$(a,1629968,1660048,117856)|0;q=(e|0)==0;d=q?1659800:1659696;e$(a,1704760,d,117856)|0;d=c[120792+(e<<2)>>2]|0;e$(a,1705336,d,117856)|0;e$(a,1703792,1661032,117856)|0;d=a;h=(c[d>>2]|0)+16|0;k=q?1663864:1663584;zK(h,1663952,k,0)|0;k=(c[d>>2]|0)+16|0;zK(k,1669600,1659536,0)|0;k=(c[d>>2]|0)+16|0;zK(k,1663360,1659432,0)|0;k=(c[d>>2]|0)+16|0;zK(k,1663264,1660512,0)|0;k=(c[d>>2]|0)+16|0;zK(k,1663184,1660416,0)|0;k=(c[d>>2]|0)+20|0;zK(k,1662096,1659280,0)|0;k=(c[d>>2]|0)+20|0;zK(k,1661752,1659160,0)|0;k=(c[d>>2]|0)+16|0;zK(k,1662880,1659056,0)|0;e$(a,1686200,1658880,117856)|0;s=0;i=b;return s|0}if((ba(f|0,1658752,2)|0)!=0){AN(0,16,1658328,(r=i,i=i+8|0,c[r>>2]=f,r)|0);i=r;s=-22;i=b;return s|0}e$(a,1629968,1658752,117856)|0;r=(e|0)==0;e$(a,1704760,r?1659800:1659696,117856)|0;if((ba(f|0,1658704,4)|0)==0){t=1658528}else{t=r?1661032:1658440}e$(a,1703792,t,117856)|0;e$(a,1705336,c[120792+(e<<2)>>2]|0,117856)|0;e$(a,1686200,1658880,117856)|0;e$(a,1685800,1662376,117856)|0;s=0;i=b;return s|0}function dU(a,b,d){a=a|0;b=b|0;d=d|0;if((y4(d,1667048)|0)==0){c[8408]=1;return 0}if((y4(d,1667016)|0)==0){c[8408]=2;return 0}if((y4(d,1666824)|0)==0){c[8408]=0;return 0}if((y4(d,1666728)|0)==0){c[8408]=255;return 0}if((c[8408]|0)!=-1){return 0}c[8408]=~~+eX(1725208,d,128,-1.0,2.0);return 0}function dV(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=i;if((a8(b|0,1712520)|0)==0){AN(0,24,1667328,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;g=e$(a,1667200,d,117856)|0;i=e;return g|0}else{h=y1(1667112,(f=i,i=i+8|0,c[f>>2]=b+6,f)|0)|0;i=f;f=e$(a,h,d,117856)|0;A4(h);g=f;i=e;return g|0}return 0}function dW(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=i;if((a8(b|0,1712160)|0)==0){AN(0,24,1667656,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;f=(c[a>>2]|0)+16|0;zK(f,1667464,d,0)|0;i=e;return 0}else{f=(c[a>>2]|0)+16|0;zK(f,b,d,0)|0;i=e;return 0}return 0}function dX(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;c[556308]=fD(c[556308]|0,4,2203688,(c[550922]|0)+1|0)|0;b=A7(32)|0;c[(c[556308]|0)+((c[550922]|0)-1<<2)>>2]=b;if((b|0)==0){e=-12;return e|0}b=(c[550922]|0)-1|0;c[c[(c[556308]|0)+(b<<2)>>2]>>2]=b;b=A9(d)|0;c[(c[(c[556308]|0)+((c[550922]|0)-1<<2)>>2]|0)+4>>2]=b;e=(c[(c[(c[556308]|0)+((c[550922]|0)-1<<2)>>2]|0)+4>>2]|0)==0?-12:0;return e|0}function dY(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;b=ei(d)|0;if((b|0)==0){e=-22;return e|0}c[556308]=fD(c[556308]|0,4,2203688,(c[550922]|0)+1|0)|0;d=A7(32)|0;c[(c[556308]|0)+((c[550922]|0)-1<<2)>>2]=d;if((d|0)==0){e=-12;return e|0}d=(c[550922]|0)-1|0;c[c[(c[556308]|0)+(d<<2)>>2]>>2]=d;c[(c[(c[556308]|0)+((c[550922]|0)-1<<2)>>2]|0)+4>>2]=b;e=0;return e|0}function dZ(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;b=a+128|0;e=a+132|0;a=fD(c[b>>2]|0,4,e,(c[e>>2]|0)+1|0)|0;c[b>>2]=a;c[a+((c[e>>2]|0)-1<<2)>>2]=d;return 0}function d_(a,b,c){a=a|0;b=b|0;c=c|0;return e$(a,1668024,c,117856)|0}function d$(a,b,c){a=a|0;b=b|0;c=c|0;return e$(a,1668144,c,117856)|0}function d0(a,b,d){a=a|0;b=b|0;d=d|0;d=i;AN(0,16,1668288,(a=i,i=i+16|0,c[a>>2]=b,c[a+8>>2]=b,a)|0);i=a;i=d;return-22|0}function d1(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;b=i;e=y1(1668704,(f=i,i=i+8|0,c[f>>2]=d,f)|0)|0;i=f;if((e$(a,1668576,e,117856)|0)<=-1){A4(e);i=b;return 0}zK((c[a>>2]|0)+16|0,1668496,d,0)|0;A4(e);i=b;return 0}function d2(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;d=i;i=i+48|0;b=d+40|0;c[b>>2]=cU(0)|0;a=a0(b|0)|0;b=d|0;e=c[a+4>>2]|0;f=c[a>>2]|0;bc(b|0,40,1668768,(g=i,i=i+24|0,c[g>>2]=c[a+8>>2],c[g+8>>2]=e,c[g+16>>2]=f,g)|0)|0;i=g;A4(c[499382]|0);c[499382]=A9(b)|0;i=d;return 0}function d3(a,b,d){a=a|0;b=b|0;d=d|0;A4(c[499382]|0);c[499382]=A9(d)|0;return 0}function d4(a,b,c){a=a|0;b=b|0;c=c|0;return e$(a,1668864,c,117856)|0}function d5(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0;f=i;g=a[d]|0;h=y1(1669104,(j=i,i=i+16|0,c[j>>2]=d+1,c[j+8>>2]=g,j)|0)|0;i=j;j=e$(b,h,e,117856)|0;A4(h);i=f;return j|0}function d6(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0;f=i;i=i+16|0;g=f|0;y_(g,e,16)|0;h=bi(g|0,58)|0;if((h|0)==0){AN(0,8,1669408,(j=i,i=i+16|0,c[j>>2]=e,c[j+8>>2]=d,j)|0);i=j;eW(1)}a[h]=0;j=~~+eX(d,g,128,0.0,1023.0);g=b+196|0;c[g>>2]=fD(c[g>>2]|0,4,b+200|0,j+1|0)|0;b=~~+eX(d,h+1|0,128,0.0,2147483647.0);c[(c[g>>2]|0)+(j<<2)>>2]=b;i=f;return 0}function d7(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=i;if((a8(b|0,1687352)|0)==0){AN(0,24,1669768,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;f=(c[a>>2]|0)+16|0;zK(f,1669600,d,0)|0;i=e;return 0}else{f=(c[a>>2]|0)+16|0;zK(f,b,d,0)|0;i=e;return 0}return 0}function d8(a,b,c){a=a|0;b=b|0;c=c|0;return e$(a,1670016,c,117856)|0}function d9(a,b,c){a=a|0;b=b|0;c=c|0;return e$(a,1670064,c,117856)|0}function ea(a,b,c){a=a|0;b=b|0;c=c|0;return e$(a,1670216,c,117856)|0}function eb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;e=i;i=i+32|0;f=zq(d)|0;g=G;if((f|0)==0&(g|0)==0){AN(0,16,1670688,(h=i,i=i+8|0,c[h>>2]=d,h)|0);i=h;j=-22;i=e;return j|0}d=e|0;bc(d|0,32,1670584,(h=i,i=i+16|0,c[h>>2]=f,c[h+8>>2]=g,h)|0)|0;i=h;k=c[745334]|0;l=c[556290]|0;c[745334]=0;c[556290]=0;m=e6(0,b,d)|0;n=a;zM((c[n>>2]|0)+16|0,c[745334]|0,0);zM((c[n>>2]|0)+20|0,c[556290]|0,0);zL(2981336);zL(2225160);c[745334]=k;c[556290]=l;if((m|0)<0){j=m;i=e;return j|0}m=zs(f,g)|0;bc(d|0,32,1670504,(h=i,i=i+8|0,c[h>>2]=m,h)|0)|0;i=h;h=bi(b|0,58)|0;b=(h|0)!=0;if(b){o=(NL(h|0)|0)+3|0}else{o=3}m=A7(o)|0;if((m|0)==0){j=-12;i=e;return j|0}y_(m,1685800,3)|0;if(b){y$(m,h,o)|0}o=e$(a,m,d,117856)|0;A4(m);j=o;i=e;return j|0}function ec(a,b,c){a=a|0;b=b|0;c=c|0;return e$(a,1670872,c,117856)|0}function ed(a,b,c){a=a|0;b=b|0;c=c|0;return e$(a,1671184,c,117856)|0}function ee(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;b=i;AN(0,24,1671320,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;e=e6(a,1679864,d)|0;i=b;return e|0}function ef(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;b=i;AN(0,24,1671760,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;e=e6(a,1679224,d)|0;i=b;return e|0}function eg(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;f=i;i=i+3016|0;g=f|0;h=f+1e3|0;j=f+2e3|0;k=f+3e3|0;l=f+3008|0;m=a[d]|0;d=j|0;a[d]=m;a[j+1|0]=0;j=c[b+24>>2]|0;if((j|0)>0){n=c[b+20>>2]|0;o=0;p=0;while(1){if((a8(c[n+(o<<4)>>2]|0,d|0)|0)==0){q=c[n+(o<<4)+8>>2]|0}else{q=p}r=o+1|0;if((r|0)<(j|0)){o=r;p=q}else{s=q;break}}}else{s=0}q=g|0;g=fz(q,1e3,e,m<<24>>24==102|0,s)|0;if((g|0)==0){if((ba(e|0,1673336,16)|0)==0){AN(0,8,1673208,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t}else{AN(0,8,1672944,(t=i,i=i+8|0,c[t>>2]=e,t)|0);i=t}eW(1)}e=h|0;if((bg(e|0,1e3,g|0)|0)==0){u=aR(g|0)|0;i=f;return 0}h=b;do{do{if((bv(e|0,1672768)|0)!=0){y_(d,e,1e3)|0;if((y3(d,1672608,k)|0)==0){v=717}else{if((y3(c[k>>2]|0,1672456,l)|0)==0){v=717}}if((v|0)==717){v=0;AN(0,8,1672280,(t=i,i=i+16|0,c[t>>2]=q,c[t+8>>2]=e,t)|0);i=t;eW(1)}s=c[k>>2]|0;AN(0,48,1672128,(t=i,i=i+24|0,c[t>>2]=q,c[t+8>>2]=d,c[t+16>>2]=s,t)|0);i=t;if((a8(d|0,1685272)|0)==0){s=c[k>>2]|0;e$(b,1670216,s,117856)|0;break}if((a8(d|0,1700440)|0)==0){s=c[k>>2]|0;e$(b,1668144,s,117856)|0;break}if((a8(d|0,1681792)|0)==0){s=c[k>>2]|0;e$(b,1671184,s,117856)|0;break}s=(a8(d|0,1674288)|0)==0;m=c[k>>2]|0;if(s){e$(b,1673616,m,117856)|0;break}s=c[745334]|0;p=c[556290]|0;c[745334]=0;c[556290]=0;o=e6(0,d,m)|0;zM((c[h>>2]|0)+16|0,c[745334]|0,0);zM((c[h>>2]|0)+20|0,c[556290]|0,0);zL(2981336);zL(2225160);c[745334]=s;c[556290]=p;if((o|0)>=0){break}o=c[k>>2]|0;AN(0,8,1671960,(t=i,i=i+32|0,c[t>>2]=q,c[t+8>>2]=e,c[t+16>>2]=d,c[t+24>>2]=o,t)|0);i=t;eW(1)}}while(0);}while((bg(e|0,1e3,g|0)|0)!=0);u=aR(g|0)|0;i=f;return 0}function eh(a,b,c){a=a|0;b=b|0;c=c|0;return e$(a,1673616,c,117856)|0}function ei(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0;b=i;i=i+1048|0;d=b|0;e=b+8|0;f=b+1040|0;c[d>>2]=0;c[e>>2]=0;if((iT(d,a,1)|0)<0){AN(0,16,1667864,(g=i,i=i+8|0,c[g>>2]=a,g)|0);i=g;i=b;return 0}if((iZ(e)|0)<0){iW(d)|0;i=b;return 0}g=b+16|0;a=iE(c[d>>2]|0,g,1024)|0;h=c[e>>2]|0;if((a|0)>0){j=a;a=h;while(1){ij(a,g,j);k=iE(c[d>>2]|0,g,1024)|0;l=c[e>>2]|0;if((k|0)>0){j=k;a=l}else{m=l;break}}}else{m=h}ih(m,0);iW(d)|0;d=(i$(c[e>>2]|0,f)|0)<0;i=b;return(d?0:c[f>>2]|0)|0}function ej(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0;e=i;f=a+16|0;g=c[f>>2]|0;h=f6(c[(c[g+4>>2]|0)+28>>2]|0,c[g+8>>2]|0)|0;if((h|0)==0){j=ek(b,d,-1)|0}else if((h|0)==1){j=el(b,d,-1)|0}else{AN(0,8,1641024,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;eW(1);j=0}c[j+8>>2]=-1;c[j+300>>2]=a;c[a+4>>2]=j;if((c[j+344>>2]|0)!=0){d=c[j+4>>2]|0;AN(0,16,1640792,(k=i,i=i+16|0,c[k>>2]=c[j>>2],c[k+8>>2]=d,k)|0);i=k;eW(1)}if((ex(c[a+8>>2]|0,a,c[f>>2]|0)|0)>=0){hc(f);i=e;return}AN(0,8,1640688,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;eW(1);hc(f);i=e;return}function ek(d,e,f){d=d|0;e=e|0;f=f|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0;h=i;i=i+8|0;j=h|0;k=j;l=i;i=i+4|0;i=i+7&-8;m=i;i=i+4|0;i=i+7&-8;n=i;i=i+4|0;i=i+7&-8;o=eq(d,e,0,f)|0;f=o+12|0;p=c[f>>2]|0;q=c[p+8>>2]|0;r=d+48|0;do{if((c[r>>2]|0)>0){s=d+44|0;t=0;u=0;while(1){v=fA(e,p,c[(c[s>>2]|0)+(t<<4)>>2]|0)|0;do{if((v|0)>0){w=c[(c[s>>2]|0)+(t<<4)+8>>2]|0}else{if((v|0)>=0){w=u;break}eW(1);w=u}}while(0);v=t+1|0;if((v|0)<(c[r>>2]|0)){t=v;u=w}else{break}}if((w|0)==0){break}if((BD(o+76|0,w)|0)>=0){break}AN(0,8,1643584,(x=i,i=i+8|0,c[x>>2]=w,x)|0);i=x;eW(1)}}while(0);w=d+272|0;do{if((c[w>>2]|0)>0){r=d+268|0;u=0;t=0;while(1){s=fA(e,p,c[(c[r>>2]|0)+(u<<4)>>2]|0)|0;do{if((s|0)>0){y=c[(c[r>>2]|0)+(u<<4)+8>>2]|0}else{if((s|0)>=0){y=t;break}eW(1);y=t}}while(0);s=u+1|0;if((s|0)<(c[w>>2]|0)){u=s;t=y}else{break}}if((y|0)==0){break}do{if((BB(k,y,255,0,0)|0)<0){z=774}else{if((c[j>>2]|0)<1){z=774;break}if((c[k+4>>2]|0)<1){z=774}}}while(0);if((z|0)==774){AN(0,8,1643400,(x=i,i=i+8|0,c[x>>2]=y,x)|0);i=x;eW(1)}t=o+92|0;u=c[j+4>>2]|0;c[t>>2]=c[j>>2];c[t+4>>2]=u}}while(0);if((c[o+344>>2]|0)!=0){j=d+328|0;if((c[j>>2]|0)<=0){i=h;return o|0}y=d+324|0;z=o+352|0;k=0;do{w=fA(e,p,c[(c[y>>2]|0)+(k<<4)>>2]|0)|0;do{if((w|0)>0){c[z>>2]=c[(c[y>>2]|0)+(k<<4)+8>>2]}else{if((w|0)>=0){break}eW(1)}}while(0);k=k+1|0;}while((k|0)<(c[j>>2]|0));i=h;return o|0}j=d+56|0;do{if((c[j>>2]|0)>0){k=d+52|0;y=0;z=0;while(1){w=fA(e,p,c[(c[k>>2]|0)+(y<<4)>>2]|0)|0;do{if((w|0)>0){A=c[(c[k>>2]|0)+(y<<4)+8>>2]|0}else{if((w|0)>=0){A=z;break}eW(1);A=z}}while(0);w=y+1|0;if((w|0)<(c[j>>2]|0)){y=w;z=A}else{break}}if((A|0)==0){break}if((BC(q+120|0,q+124|0,A)|0)>=0){break}AN(0,8,1644536,(x=i,i=i+8|0,c[x>>2]=A,x)|0);i=x;eW(1)}}while(0);c[q+796>>2]=c[556288];A=d+64|0;do{if((c[A>>2]|0)>0){j=d+60|0;z=0;y=0;while(1){k=fA(e,p,c[(c[j>>2]|0)+(z<<4)>>2]|0)|0;do{if((k|0)>0){B=c[(c[j>>2]|0)+(z<<4)+8>>2]|0}else{if((k|0)>=0){B=y;break}eW(1);B=y}}while(0);k=z+1|0;if((k|0)<(c[A>>2]|0)){z=k;y=B}else{break}}if((B|0)==0){break}if((a[B]|0)==43){c[o+360>>2]=1;y=B+1|0;z=(a[y]|0)==0?0:y;if((z|0)==0){break}else{C=z}}else{C=B}z=BM(C)|0;c[q+140>>2]=z;if((z|0)!=-1){break}AN(0,8,1643232,(x=i,i=i+8|0,c[x>>2]=C,x)|0);i=x;eW(1)}}while(0);C=q+224|0;B=p+80|0;A=c[C+4>>2]|0;c[B>>2]=c[C>>2];c[B+4>>2]=A;if((c[551802]|0)!=0){c[q+136>>2]=0}A=d+288|0;do{if((c[A>>2]|0)>0){B=d+284|0;C=0;z=0;while(1){y=fA(e,p,c[(c[B>>2]|0)+(C<<4)>>2]|0)|0;do{if((y|0)>0){D=c[(c[B>>2]|0)+(C<<4)+8>>2]|0}else{if((y|0)>=0){D=z;break}eW(1);D=z}}while(0);y=C+1|0;if((y|0)<(c[A>>2]|0)){C=y;z=D}else{break}}if((D|0)==0){break}z=A7(128)|0;C=z;B=q+300|0;c[B>>2]=C;if((z|0)==0){AN(0,8,1643072,(x=i,i=i+1|0,i=i+7&-8,c[x>>2]=0,x)|0);i=x;eW(1);E=c[B>>2]|0}else{E=C}b[E>>1]=(cc(D|0)|0)&65535;C=0;B=D;do{z=bi(B|0,44)|0;if((z|0)==0){AN(0,8,1642280,(x=i,i=i+16|0,c[x>>2]=D,c[x+8>>2]=C,x)|0);i=x;eW(1)}B=z+1|0;C=C+1|0;b[E+(C<<1)>>1]=(cc(B|0)|0)&65535;}while((C|0)!=63)}}while(0);E=d+296|0;do{if((c[E>>2]|0)>0){D=d+292|0;A=0;C=0;while(1){B=fA(e,p,c[(c[D>>2]|0)+(A<<4)>>2]|0)|0;do{if((B|0)>0){F=c[(c[D>>2]|0)+(A<<4)+8>>2]|0}else{if((B|0)>=0){F=C;break}eW(1);F=C}}while(0);B=A+1|0;if((B|0)<(c[E>>2]|0)){A=B;C=F}else{break}}if((F|0)==0){break}C=A7(128)|0;A=C;D=q+304|0;c[D>>2]=A;if((C|0)==0){AN(0,8,1642928,(x=i,i=i+1|0,i=i+7&-8,c[x>>2]=0,x)|0);i=x;eW(1);G=c[D>>2]|0}else{G=A}b[G>>1]=(cc(F|0)|0)&65535;A=0;D=F;do{C=bi(D|0,44)|0;if((C|0)==0){AN(0,8,1642280,(x=i,i=i+16|0,c[x>>2]=F,c[x+8>>2]=A,x)|0);i=x;eW(1)}D=C+1|0;A=A+1|0;b[G+(A<<1)>>1]=(cc(D|0)|0)&65535;}while((A|0)!=63)}}while(0);G=d+280|0;do{if((c[G>>2]|0)>0){F=d+276|0;E=0;A=0;while(1){D=fA(e,p,c[(c[F>>2]|0)+(E<<4)>>2]|0)|0;do{if((D|0)>0){H=c[(c[F>>2]|0)+(E<<4)+8>>2]|0}else{if((D|0)>=0){H=A;break}eW(1);H=A}}while(0);D=E+1|0;if((D|0)<(c[G>>2]|0)){E=D;A=H}else{break}}if((H|0)==0){I=0;break}A=q+532|0;E=0;F=H;while(1){D=ct(F|0,1642800,(x=i,i=i+24|0,c[x>>2]=l,c[x+8>>2]=m,c[x+16>>2]=n,x)|0)|0;i=x;if((D|0)!=3){AN(0,8,1642728,(x=i,i=i+1|0,i=i+7&-8,c[x>>2]=0,x)|0);i=x;eW(1)}D=E+1|0;C=A2(c[A>>2]|0,D<<4)|0;c[A>>2]=C;c[C+(E<<4)>>2]=c[l>>2];c[(c[A>>2]|0)+(E<<4)+4>>2]=c[m>>2];C=c[n>>2]|0;B=(c[A>>2]|0)+(E<<4)+8|0;if((C|0)>0){c[B>>2]=C;g[(c[A>>2]|0)+(E<<4)+12>>2]=1.0}else{c[B>>2]=0;g[(c[A>>2]|0)+(E<<4)+12>>2]=+(-(c[n>>2]|0)|0)/100.0}B=bi(F|0,47)|0;C=(B|0)==0?0:B+1|0;if((C|0)==0){I=D;break}else{E=D;F=C}}}else{I=0}}while(0);c[q+528>>2]=I;c[q+324>>2]=(c[43230]|0)-8;if((c[744096]|0)!=0){I=q+88|0;c[I>>2]=c[I>>2]|32768}I=d+384|0;do{if((c[I>>2]|0)>0){n=d+380|0;m=0;l=0;while(1){x=fA(e,p,c[(c[n>>2]|0)+(m<<4)>>2]|0)|0;do{if((x|0)>0){J=c[(c[n>>2]|0)+(m<<4)+8>>2]|0}else{if((x|0)>=0){J=l;break}eW(1);J=l}}while(0);x=m+1|0;if((x|0)<(c[I>>2]|0)){m=x;l=J}else{break}}if((J|0)==0){break}if((J&1|0)!=0){l=q+88|0;c[l>>2]=c[l>>2]|512;l=o+320|0;zK(l,1630640,1642576,32)|0}if((J&2|0)==0){break}l=q+88|0;c[l>>2]=c[l>>2]|1024;zK(o+320|0,1630640,1642432,32)|0}}while(0);q=d+392|0;if((c[q>>2]|0)>0){J=d+388|0;I=o+292|0;l=0;while(1){m=fA(e,p,c[(c[J>>2]|0)+(l<<4)>>2]|0)|0;do{if((m|0)>0){c[I>>2]=c[(c[J>>2]|0)+(l<<4)+8>>2]}else{if((m|0)>=0){break}eW(1)}}while(0);m=l+1|0;if((m|0)<(c[q>>2]|0)){l=m}else{K=I;break}}}else{K=o+292|0}I=c[K>>2]|0;do{if((I|0)!=0){l=A9(I)|0;c[K>>2]=l;if((l|0)!=0){break}eW(1)}}while(0);K=d+256|0;if((c[K>>2]|0)>0){I=d+252|0;l=o+112|0;q=0;while(1){J=fA(e,p,c[(c[I>>2]|0)+(q<<4)>>2]|0)|0;do{if((J|0)>0){c[l>>2]=c[(c[I>>2]|0)+(q<<4)+8>>2]}else{if((J|0)>=0){break}eW(1)}}while(0);J=q+1|0;if((J|0)<(c[K>>2]|0)){q=J}else{L=l;break}}}else{L=o+112|0}l=c[L>>2]|0;if((l|0)!=0){c[L>>2]=A9(l)|0}l=d+264|0;if((c[l>>2]|0)>0){L=d+260|0;q=o+84|0;K=0;do{I=fA(e,p,c[(c[L>>2]|0)+(K<<4)>>2]|0)|0;do{if((I|0)>0){c[q>>2]=c[(c[L>>2]|0)+(K<<4)+8>>2]}else{if((I|0)>=0){break}eW(1)}}while(0);K=K+1|0;}while((K|0)<(c[l>>2]|0))}l=o+88|0;c[l>>2]=-1;K=d+304|0;if((c[K>>2]|0)>0){L=d+300|0;q=0;do{I=fA(e,p,c[(c[L>>2]|0)+(q<<4)>>2]|0)|0;do{if((I|0)>0){c[l>>2]=c[(c[L>>2]|0)+(q<<4)+8>>2]}else{if((I|0)>=0){break}eW(1)}}while(0);q=q+1|0;}while((q|0)<(c[K>>2]|0))}K=es(d,e,c[f>>2]|0)|0;c[o+304>>2]=K;if((K|0)!=0){i=h;return o|0}eW(1);i=h;return o|0}function el(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;e=i;f=eq(a,b,1,d)|0;d=f+12|0;g=c[d>>2]|0;h=c[g+8>>2]|0;c[h+8>>2]=1;if((c[f+344>>2]|0)!=0){i=e;return f|0}j=a+32|0;if((c[j>>2]|0)>0){k=a+28|0;l=h+416|0;m=0;do{n=fA(b,g,c[(c[k>>2]|0)+(m<<4)>>2]|0)|0;do{if((n|0)>0){c[l>>2]=c[(c[k>>2]|0)+(m<<4)+8>>2]}else{if((n|0)>=0){break}eW(1)}}while(0);m=m+1|0;}while((m|0)<(c[j>>2]|0))}j=a+240|0;do{if((c[j>>2]|0)>0){m=a+236|0;k=0;l=0;while(1){n=fA(b,g,c[(c[m>>2]|0)+(k<<4)>>2]|0)|0;do{if((n|0)>0){o=c[(c[m>>2]|0)+(k<<4)+8>>2]|0}else{if((n|0)>=0){o=l;break}eW(1);o=l}}while(0);n=k+1|0;if((n|0)<(c[j>>2]|0)){k=n;l=o}else{break}}if((o|0)==0){break}l=B3(o)|0;c[h+420>>2]=l;if((l|0)!=-1){break}AN(0,8,1644400,(p=i,i=i+8|0,c[p>>2]=o,p)|0);i=p;eW(1)}}while(0);o=a+40|0;if((c[o>>2]|0)>0){j=a+36|0;l=h+412|0;h=0;do{k=fA(b,g,c[(c[j>>2]|0)+(h<<4)>>2]|0)|0;do{if((k|0)>0){c[l>>2]=c[(c[j>>2]|0)+(h<<4)+8>>2]}else{if((k|0)>=0){break}eW(1)}}while(0);h=h+1|0;}while((h|0)<(c[o>>2]|0))}o=a+408|0;if((c[o>>2]|0)>0){h=a+404|0;j=f+332|0;l=0;while(1){k=fA(b,g,c[(c[h>>2]|0)+(l<<4)>>2]|0)|0;do{if((k|0)>0){c[j>>2]=c[(c[h>>2]|0)+(l<<4)+8>>2]}else{if((k|0)>=0){break}eW(1)}}while(0);k=l+1|0;if((k|0)<(c[o>>2]|0)){l=k}else{q=j;break}}}else{q=f+332|0}c[q>>2]=A9(c[q>>2]|0)|0;q=es(a,b,c[d>>2]|0)|0;c[f+304>>2]=q;if((q|0)==0){eW(1)}q=a+112|0;if((c[q>>2]|0)<=0){i=e;return f|0}b=a+108|0;a=f+8|0;j=f+288|0;l=f|0;o=0;do{h=c[b>>2]|0;g=c[(c[551808]|0)+(c[a>>2]<<2)>>2]|0;k=c[h+(o*20|0)+8>>2]|0;do{if((k|0)==-1){r=947}else{if((c[g>>2]|0)!=(c[h+(o*20|0)>>2]|0)){break}if((c[c[g+4>>2]>>2]|0)==(c[h+(o*20|0)+4>>2]|0)){r=947}}}while(0);do{if((r|0)==947){r=0;g=c[h+(o*20|0)+12>>2]|0;if((g|0)!=-1){if((c[l>>2]|0)!=(g|0)){break}}g=c[h+(o*20|0)+16>>2]|0;if((g|0)!=-1){if((c[c[d>>2]>>2]|0)!=(g|0)){break}}g=c[j>>2]|0;if(g>>>0<32){c[j>>2]=g+1;c[f+160+(g<<2)>>2]=k;break}else{g=c[c[d>>2]>>2]|0;AN(0,8,1644064,(p=i,i=i+16|0,c[p>>2]=c[l>>2],c[p+8>>2]=g,p)|0);i=p;break}}}while(0);o=o+1|0;}while((o|0)<(c[q>>2]|0));i=e;return f|0}function em(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;e=i;f=eq(a,b,3,d)|0;d=c[f+12>>2]|0;g=c[d+8>>2]|0;c[g+8>>2]=3;h=a+328|0;if((c[h>>2]|0)>0){j=a+324|0;k=f+352|0;l=0;do{m=fA(b,d,c[(c[j>>2]|0)+(l<<4)>>2]|0)|0;do{if((m|0)>0){c[k>>2]=c[(c[j>>2]|0)+(l<<4)+8>>2]}else{if((m|0)>=0){break}eW(1)}}while(0);l=l+1|0;}while((l|0)<(c[h>>2]|0))}if((c[f+344>>2]|0)!=0){i=e;return}f=a+56|0;if((c[f>>2]|0)<=0){i=e;return}h=a+52|0;a=0;l=0;while(1){j=fA(b,d,c[(c[h>>2]|0)+(a<<4)>>2]|0)|0;do{if((j|0)>0){n=c[(c[h>>2]|0)+(a<<4)+8>>2]|0}else{if((j|0)>=0){n=l;break}eW(1);n=l}}while(0);j=a+1|0;if((j|0)<(c[f>>2]|0)){a=j;l=n}else{break}}if((n|0)==0){i=e;return}if((BC(g+120|0,g+124|0,n)|0)>=0){i=e;return}AN(0,8,1644536,(g=i,i=i+8|0,c[g>>2]=n,g)|0);i=g;eW(1);i=e;return}function en(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0;d=i;i=i+8|0;e=d|0;c[e>>2]=0;f=zJ(b,3119176,0,2)|0;if((f|0)==0){g=0;i=d;return g|0}else{h=f}do{f=h|0;j=c[f>>2]|0;k=bi(j|0,58)|0;if((k|0)==0){l=c[h+4>>2]|0;zK(e,j,l,0)|0}else{a[k]=0;l=c[f>>2]|0;f=c[h+4>>2]|0;zK(e,l,f,0)|0;a[k]=58}h=zJ(b,3119176,h,2)|0;}while((h|0)!=0);g=c[e>>2]|0;i=d;return g|0}function eo(b){b=b|0;var d=0,e=0,f=0;d=i;if(!((c[560406]|0)==0|(c[550908]|0)!=0)){i=d;return}do{if((bi(b|0,58)|0)!=0){if((a[b+1|0]|0)==58){break}if((yY(b,1649496,0)|0)!=0){break}i=d;return}}while(0);if((ia(b,0)|0)!=0){i=d;return}do{if((c[18294]|0)!=0){if(!((c[550908]|0)==0|(c[560406]|0)!=0)){break}e=c[o>>2]|0;cx(e|0,1649296,(f=i,i=i+8|0,c[f>>2]=b,f)|0)|0;i=f;aT(e|0)|0;eB();a6(2,0)|0;if((fx()|0)==0){AN(0,8,1649080,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;eW(1)}eC();i=d;return}}while(0);AN(0,8,1648984,(f=i,i=i+8|0,c[f>>2]=b,f)|0);i=f;eW(1);i=d;return}function ep(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0;g=i;h=a[b]|0;if(h<<24>>24==0){a[d]=103;i=g;return}a[d]=h;h=a[b]|0;if((h|0)==115){d=b+1|0;j=a[d]|0;if((j<<24>>24|0)==0|(j<<24>>24|0)==58){k=j}else{AN(0,8,1650640,(l=i,i=i+8|0,c[l>>2]=d,l)|0);i=l;eW(1);k=a[d]|0}c[f>>2]=k<<24>>24==58?b+2|0:3119176;i=g;return}else if((h|0)==99|(h|0)==112){if((a[b+1|0]|0)!=58){i=g;return}c[e>>2]=aU(b+2|0,0,0)|0;i=g;return}else if((h|0)==103){i=g;return}else{AN(0,8,1650520,(l=i,i=i+8|0,c[l>>2]=h,l)|0);i=l;eW(1);i=g;return}}function eq(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0.0,T=0.0,U=0,V=0,W=0,X=0,Y=0,Z=0;j=i;i=i+1048|0;k=j|0;l=j+8|0;m=j+16|0;n=j+1016|0;o=j+1032|0;p=j+1040|0;q=mK(e,0)|0;r=e+24|0;s=c[r>>2]|0;if((q|0)==0){AN(0,8,1648808,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;eW(1);u=c[r>>2]|0}else{u=s}r=u-1|0;if(r>>>0<(c[b+200>>2]|0)>>>0){c[q+4>>2]=c[(c[b+196>>2]|0)+(r<<2)>>2]}c[550876]=fD(c[550876]|0,4,2203640,(c[550910]|0)+1|0)|0;r=A7(368)|0;u=r;if((r|0)==0){eW(1)}c[(c[550876]|0)+((c[550910]|0)-1<<2)>>2]=u;v=r;c[v>>2]=(c[550912]|0)-1;w=r+4|0;c[w>>2]=s-1;s=r+12|0;c[s>>2]=q;x=q+8|0;c[(c[x>>2]|0)+8>>2]=f;y=b+24|0;do{if((c[y>>2]|0)>0){z=b+20|0;A=0;B=0;while(1){C=fA(e,c[s>>2]|0,c[(c[z>>2]|0)+(A<<4)>>2]|0)|0;do{if((C|0)>0){D=c[(c[z>>2]|0)+(A<<4)+8>>2]|0}else{if((C|0)>=0){D=B;break}eW(1);D=B}}while(0);C=A+1|0;if((C|0)<(c[y>>2]|0)){A=C;B=D}else{break}}if((D|0)==0){E=1039;break}if((a8(D|0,1645504)|0)==0){c[r+344>>2]=1;F=r+60|0;break}else{B=er(D,c[(c[(c[s>>2]|0)+8>>2]|0)+8>>2]|0,1)|0;A=r+60|0;c[A>>2]=B;c[(c[(c[s>>2]|0)+8>>2]|0)+48>>2]=c[B+12>>2];F=A;break}}else{E=1039}}while(0);if((E|0)==1039){D=ji(c[e+8>>2]|0,0,e+32|0,0,c[(c[(c[s>>2]|0)+8>>2]|0)+8>>2]|0)|0;c[(c[(c[s>>2]|0)+8>>2]|0)+48>>2]=D;D=r+60|0;c[D>>2]=o8(c[(c[(c[s>>2]|0)+8>>2]|0)+48>>2]|0)|0;F=D}D=c[F>>2]|0;L1431:do{if((D|0)==0){c[r+320>>2]=fB(c[(c[b>>2]|0)+16>>2]|0,0,e,q,0)|0}else{c[p>>2]=0;s=r+320|0;c[s>>2]=fB(c[(c[b>>2]|0)+16>>2]|0,c[D+12>>2]|0,e,q,D)|0;y=b+320|0;if((c[y>>2]|0)<=0){break}A=b+316|0;B=0;z=0;while(1){C=fA(e,q,c[(c[A>>2]|0)+(B<<4)>>2]|0)|0;do{if((C|0)>0){G=c[(c[A>>2]|0)+(B<<4)+8>>2]|0}else{if((C|0)>=0){G=z;break}eW(1);G=z}}while(0);C=B+1|0;if((C|0)<(c[y>>2]|0)){B=C;z=G}else{break}}if((G|0)==0){break}z=c[c[F>>2]>>2]|0;B=m|0;y=ca(1646480)|0;c[n>>2]=y;c[n+4>>2]=ca(1646424)|0;c[n+8>>2]=1646240;L1443:do{if((z|0)==0){A=0;C=1;H=y;while(1){if((H|0)==0){I=C}else{if((C|0)==0){break L1443}J=(A|0)!=1?3119176:1645912;bc(B|0,1e3,1645672,(t=i,i=i+24|0,c[t>>2]=H,c[t+8>>2]=J,c[t+16>>2]=G,t)|0)|0;i=t;I=iU(p,B,1,172928,0)|0}J=A+1|0;if(J>>>0>2|(I|0)==0){K=I;E=1063;break L1443}A=J;C=I;H=c[n+(J<<2)>>2]|0}}else{H=0;C=1;A=y;while(1){if((A|0)==0){L=C}else{J=(H|0)!=1?3119176:1645912;bc(B|0,1e3,1646080,(t=i,i=i+32|0,c[t>>2]=A,c[t+8>>2]=J,c[t+16>>2]=z,c[t+24>>2]=G,t)|0)|0;i=t;if((iU(p,B,1,172928,0)|0)==0){break L1443}bc(B|0,1e3,1645672,(t=i,i=i+24|0,c[t>>2]=A,c[t+8>>2]=J,c[t+16>>2]=G,t)|0)|0;i=t;L=iU(p,B,1,172928,0)|0}J=H+1|0;if(J>>>0>2|(L|0)==0){K=L;E=1063;break L1443}H=J;C=L;A=c[n+(J<<2)>>2]|0}}}while(0);do{if((E|0)==1063){if((K|0)==0){break}B=c[v>>2]|0;z=c[w>>2]|0;AN(0,8,1648528,(t=i,i=i+24|0,c[t>>2]=G,c[t+8>>2]=B,c[t+16>>2]=z,t)|0);i=t;eW(1);break L1431}}while(0);z=c[p>>2]|0;do{if((iZ(k)|0)<0){AN(0,8,1646536,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;eW(1)}while(1){B=(iD(z)|0)<<24>>24;M=c[k>>2]|0;if((B|0)==10|(B|0)==0){break}ih(M,B)}ih(M,0);i$(c[k>>2]|0,l)|0;B=c[l>>2]|0;y=a[B]|0;if((y<<24>>24|0)==0|(y<<24>>24|0)==35){A4(B)}else{y=bi(B|0,61)|0;if((y|0)==0){AN(0,8,1648640,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;eW(1)}A=y+1|0;a[y]=0;zK(s,B,A,16)|0;A4(B)}z=c[p>>2]|0;}while((c[z+52>>2]|0)==0);iV(z)|0}}while(0);ou(c[x>>2]|0,c[F>>2]|0)|0;c[(c[x>>2]|0)+8>>2]=f;p=r+64|0;c[p>>2]=-1;c[p+4>>2]=2147483647;l=b+216|0;k=c[l>>2]|0;M=b+212|0;if((k|0)>0){G=0;while(1){w=fA(e,q,c[(c[M>>2]|0)+(G<<4)>>2]|0)|0;do{if((w|0)>0){v=(c[M>>2]|0)+(G<<4)+8|0;K=c[v+4>>2]|0;c[p>>2]=c[v>>2];c[p+4>>2]=K}else{if((w|0)>=0){break}eW(1)}}while(0);w=G+1|0;z=c[l>>2]|0;if((w|0)<(z|0)){G=w}else{N=z;break}}}else{N=k}L1489:do{if((f|0)==0){k=0;while(1){if((k|0)>=(N|0)){break L1489}k=k+1|0}}else{k=0;while(1){if((k|0)>=(N|0)){break L1489}if((a[c[(c[M>>2]|0)+(k<<4)>>2]|0]|0)==0){break}else{k=k+1|0}}AN(0,24,1648272,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t}}while(0);M=r+356|0;c[M>>2]=-1;N=b+336|0;if((c[N>>2]|0)>0){f=b+332|0;k=0;do{G=fA(e,q,c[(c[f>>2]|0)+(k<<4)>>2]|0)|0;do{if((G|0)>0){c[M>>2]=c[(c[f>>2]|0)+(k<<4)+8>>2]}else{if((G|0)>=0){break}eW(1)}}while(0);k=k+1|0;}while((k|0)<(c[N>>2]|0))}N=b+224|0;do{if((c[N>>2]|0)>0){k=b+220|0;f=0;M=0;while(1){G=fA(e,q,c[(c[k>>2]|0)+(f<<4)>>2]|0)|0;do{if((G|0)>0){O=c[(c[k>>2]|0)+(f<<4)+8>>2]|0}else{if((G|0)>=0){O=M;break}eW(1);O=M}}while(0);G=f+1|0;if((G|0)<(c[N>>2]|0)){f=G;M=O}else{break}}if((O|0)==0){break}M=r+56|0;f=0;k=O;while(1){G=bi(k|0,44)|0;if((G|0)==0){P=0}else{a[G]=0;P=G+1|0}G=nQ(k)|0;if((G|0)==0){AN(0,8,1648104,(t=i,i=i+8|0,c[t>>2]=k,t)|0);i=t;eW(1)}if((f|0)==0){c[M>>2]=G}else{c[f+12>>2]=G}if((P|0)==0){break}else{f=G;k=P}}c[o>>2]=0}}while(0);P=b+232|0;do{if((c[P>>2]|0)>0){t=b+228|0;O=0;N=0;while(1){k=fA(e,q,c[(c[t>>2]|0)+(O<<4)>>2]|0)|0;do{if((k|0)>0){Q=c[(c[t>>2]|0)+(O<<4)+8>>2]|0}else{if((k|0)>=0){Q=N;break}eW(1);Q=N}}while(0);k=O+1|0;if((k|0)<(c[P>>2]|0)){O=k;N=Q}else{break}}if((Q|0)==0){break}N=aU(Q|0,o|0,0)|0;if((a[c[o>>2]|0]|0)==0){R=N}else{N=Q;R=d[N]|d[N+1|0]<<8|d[N+2|0]<<16|d[N+3|0]<<24|0}c[(c[x>>2]|0)+52>>2]=R}}while(0);R=b+248|0;do{if((c[R>>2]|0)>0){Q=b+244|0;S=-1.0;o=0;while(1){P=fA(e,q,c[(c[Q>>2]|0)+(o<<4)>>2]|0)|0;do{if((P|0)>0){T=+h[(c[Q>>2]|0)+(o<<4)+8>>3]}else{if((P|0)>=0){T=S;break}eW(1);T=S}}while(0);P=o+1|0;if((P|0)<(c[R>>2]|0)){S=T;o=P}else{break}}if(T<0.0){break}o=(c[x>>2]|0)+88|0;c[o>>2]=c[o>>2]|2;c[(c[x>>2]|0)+80>>2]=~~(T*118.0)}}while(0);if((c[(c[e+8>>2]|0)+28>>2]&64|0)!=0){e=(c[x>>2]|0)+88|0;c[e>>2]=c[e>>2]|4194304}e=b|0;Bn(c[(c[e>>2]|0)+28>>2]|0,1647992,0,r+312|0)|0;b=r+324|0;zM(b,c[(c[e>>2]|0)+32>>2]|0,0);x=c[F>>2]|0;do{if((x|0)!=0){if((ps(c[x+12>>2]|0)|0)!=24){break}zK(b,1647896,1646672,0)|0}}while(0);zM(r+328|0,c[(c[e>>2]|0)+24>>2]|0,0);c[r+8>>2]=g;if((g|0)<=-1){U=r+48|0;V=U;W=0;X=-2147483648;Y=V|0;c[Y>>2]=W;Z=V+4|0;c[Z>>2]=X;i=j;return u|0}e=(c[551808]|0)+(g<<2)|0;c[r+24>>2]=c[e>>2];c[(c[e>>2]|0)+8>>2]=0;c[(c[(c[(c[551808]|0)+(g<<2)>>2]|0)+4>>2]|0)+76>>2]=-16;U=r+48|0;V=U;W=0;X=-2147483648;Y=V|0;c[Y>>2]=W;Z=V+4|0;c[Z>>2]=X;i=j;return u|0}function er(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;e=i;f=(d|0)!=0;d=f?1645368:1645216;if(f){g=pl(a)|0}else{g=pm(a)|0}L1576:do{if((g|0)==0){h=nV(a)|0;do{if((h|0)!=0){j=c[h>>2]|0;if(f){k=o8(j)|0}else{k=o9(j)|0}if((k|0)==0){break}j=c[k>>2]|0;l=c[h+8>>2]|0;AN(0,40,1644960,(m=i,i=i+24|0,c[m>>2]=d,c[m+8>>2]=j,c[m+16>>2]=l,m)|0);i=m;n=k;break L1576}}while(0);AN(0,8,1644864,(m=i,i=i+16|0,c[m>>2]=d,c[m+8>>2]=a,m)|0);i=m;eW(1);n=0}else{n=g}}while(0);if((c[n+8>>2]|0)==(b|0)){i=e;return n|0}AN(0,8,1644744,(m=i,i=i+16|0,c[m>>2]=d,c[m+8>>2]=a,m)|0);i=m;eW(1);i=e;return n|0}function es(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=i;f=a+352|0;if((c[f>>2]|0)>0){g=a+348|0;h=0;j=0;while(1){k=fA(b,d,c[(c[g>>2]|0)+(h<<4)>>2]|0)|0;do{if((k|0)>0){l=c[(c[g>>2]|0)+(h<<4)+8>>2]|0}else{if((k|0)>=0){l=j;break}eW(1);l=j}}while(0);k=h+1|0;if((k|0)<(c[f>>2]|0)){h=k;j=l}else{m=l;break}}}else{m=0}l=a+344|0;do{if((c[l>>2]|0)>0){j=a+340|0;h=0;f=0;while(1){g=fA(b,d,c[(c[j>>2]|0)+(h<<4)>>2]|0)|0;do{if((g|0)>0){n=c[(c[j>>2]|0)+(h<<4)+8>>2]|0}else{if((g|0)>=0){n=f;break}eW(1);n=f}}while(0);g=h+1|0;if((g|0)<(c[l>>2]|0)){h=g;f=n}else{break}}f=(m|0)!=0;h=(n|0)==0;if(h|f^1){if(f){o=1179;break}}else{j=c[d>>2]|0;AN(0,16,1643792,(g=i,i=i+16|0,c[g>>2]=c[550912],c[g+8>>2]=j,g)|0);i=g;eW(1);if(f){o=1179;break}}if(h){o=1182;break}p=A9(n)|0;i=e;return p|0}else{if((m|0)==0){o=1182}else{o=1179}}}while(0);if((o|0)==1182){p=A9((c[(c[d+8>>2]|0)+8>>2]|0)==0?1653816:1653880)|0;i=e;return p|0}else if((o|0)==1179){p=ei(m)|0;i=e;return p|0}return 0}function et(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;e=a+24|0;do{if((c[e>>2]|0)>0){f=a+20|0;g=0;h=0;while(1){i=fA(b,d,c[(c[f>>2]|0)+(g<<4)>>2]|0)|0;do{if((i|0)>0){j=c[(c[f>>2]|0)+(g<<4)+8>>2]|0}else{if((i|0)>=0){j=h;break}eW(1);j=h}}while(0);i=g+1|0;if((i|0)<(c[e>>2]|0)){g=i;h=j}else{break}}h=d+8|0;g=c[h>>2]|0;if((j|0)==0){k=g;break}f=er(j,c[g+8>>2]|0,0)|0;c[(c[h>>2]|0)+48>>2]=c[f+12>>2];l=f;return l|0}else{k=c[d+8>>2]|0}}while(0);l=o9(c[k+48>>2]|0)|0;return l|0}function eu(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;f=i;i=i+56|0;g=f|0;h=f+24|0;if((d|0)==0){j=e;i=f;return j|0}k=c[d+24>>2]|0;if((k|0)==0){j=e;i=f;return j|0}l=BP(e)|0;if((l|0)==0){m=0}else{m=(a[l+4|0]&1^1)&255}l=c[b+8>>2]|0;do{if((c[l+680>>2]|0)<0){b=c[l+48>>2]|0;if((b|0)==8){n=g|0;c[n>>2]=12;c[g+4>>2]=13;c[g+8>>2]=0;c[g+12>>2]=4;c[g+16>>2]=-1;o=-1;p=n;break}else if((b|0)==10){b=h|0;c[b>>2]=12;c[h+4>>2]=13;c[h+8>>2]=14;c[h+12>>2]=0;c[h+16>>2]=4;c[h+20>>2]=5;c[h+24>>2]=30;c[h+28>>2]=-1;o=-1;p=b;break}else{o=-1;p=k;break}}else{o=-1;p=k}}while(0);while(1){k=c[p>>2]|0;if((k|0)==-1){break}q=n$(o,k,e,m,0)|0;if((c[p>>2]|0)==(e|0)){r=1212;break}o=q;p=p+4|0}if((r|0)==1212){i=f;return((e|0)==-1?q:e)|0}if((e|0)==-1){j=o;i=f;return j|0}q=BL(e)|0;e=c[d>>2]|0;d=BL(o)|0;AN(0,24,1664632,(r=i,i=i+24|0,c[r>>2]=q,c[r+8>>2]=e,c[r+16>>2]=d,r)|0);i=r;j=o;i=f;return j|0}function ev(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0;d=i;if((b|0)==0){i=d;return}e=b+32|0;f=c[e>>2]|0;if((f|0)==0){i=d;return}g=a+8|0;a=f;while(1){f=c[a>>2]|0;if((f|0)==-1){break}if((f|0)==(c[(c[g>>2]|0)+420>>2]|0)){h=1232;break}else{a=a+4|0}}if((h|0)==1232){i=d;return}do{if((c[b+16>>2]|0)<0){h=B2(c[(c[g>>2]|0)+420>>2]|0)|0;if(h>>>0<=(B2(c[c[e>>2]>>2]|0)|0)>>>0){break}AN(0,16,1724512,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0);i=j}}while(0);if((B2(c[(c[g>>2]|0)+420>>2]|0)|0)!=0){h=B2(c[(c[g>>2]|0)+420>>2]|0)|0;a=c[b>>2]|0;b=B2(c[c[e>>2]>>2]|0)|0;AN(0,24,1695672,(j=i,i=i+24|0,c[j>>2]=h,c[j+8>>2]=a,c[j+16>>2]=b,j)|0);i=j}c[(c[g>>2]|0)+420>>2]=c[c[e>>2]>>2];i=d;return}function ew(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=A7(32)|0;e=d;if((d|0)==0){eW(1)}c[d>>2]=c[550922];f=d+24|0;g=d+28|0;c[f>>2]=fD(c[f>>2]|0,4,g,(c[g>>2]|0)+1|0)|0;g=A7(20)|0;c[c[f>>2]>>2]=g;if((g|0)==0){eW(1)}c[(c[c[f>>2]>>2]|0)+4>>2]=b;c[(c[c[f>>2]>>2]|0)+8>>2]=e;c[b+300>>2]=c[c[f>>2]>>2];f=d+16|0;b=d+20|0;c[f>>2]=fD(c[f>>2]|0,4,b,(c[b>>2]|0)+1|0)|0;b=A7(16)|0;c[c[f>>2]>>2]=b;if((b|0)==0){eW(1)}c[(c[c[f>>2]>>2]|0)+4>>2]=a;c[(c[c[f>>2]>>2]|0)+8>>2]=e;b=a+244|0;d=a+248|0;a=fD(c[b>>2]|0,4,d,(c[d>>2]|0)+1|0)|0;c[b>>2]=a;c[a+((c[d>>2]|0)-1<<2)>>2]=c[c[f>>2]>>2];f=fD(c[556308]|0,4,2203688,(c[550922]|0)+1|0)|0;c[556308]=f;c[f+((c[550922]|0)-1<<2)>>2]=e;return e|0}function ex(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0;f=i;i=i+2312|0;g=f|0;h=f+8|0;j=f+16|0;k=f+32|0;l=f+40|0;m=f+48|0;n=f+64|0;o=f+72|0;p=f+80|0;q=f+88|0;r=f+96|0;s=f+104|0;t=f+112|0;u=f+120|0;v=f+1144|0;w=f+1152|0;x=f+1672|0;y=f+1680|0;z=f+1688|0;A=f+1696|0;B=f+1720|0;C=f+1752|0;D=f+1760|0;E=f+1768|0;F=f+1776|0;H=f+2032|0;I=f+2288|0;J=f+2296|0;K=f+2304|0;L=d+12|0;A6(L);M=e+4|0;N=c[M>>2]|0;O=c[N+28>>2]|0;P=c[N+36>>2]|0;if((iZ(K)|0)<0){eW(1)}iX(c[K>>2]|0,1673912,(Q=i,i=i+8|0,c[Q>>2]=c[c[N+4>>2]>>2],Q)|0)|0;i=Q;if((P|0)>1){P=c[K>>2]|0;N=e+8|0;R=f5(O,c[N>>2]|0)|0;iX(P,1658744,(Q=i,i=i+8|0,c[Q>>2]=R,Q)|0)|0;i=Q;S=N}else{S=e+8|0}ih(c[K>>2]|0,0);i$(c[K>>2]|0,L)|0;L=f6(c[(c[M>>2]|0)+28>>2]|0,c[S>>2]|0)|0;if((L|0)==1){K=c[M>>2]|0;e=c[S>>2]|0;N=r;R=s;P=t;O=F|0;T=f+1160|0;U=f+1416|0;V=c[d+4>>2]|0;W=V+12|0;X=c[(c[W>>2]|0)+8>>2]|0;c[p>>2]=K;Y=V|0;Z=c[Y>>2]|0;_=c[(c[550878]|0)+(Z<<2)>>2]|0;c[q>>2]=e;$=V+4|0;aa=c[$>>2]|0;bc(O|0,255,1660760,(Q=i,i=i+16|0,c[Q>>2]=Z,c[Q+8>>2]=aa,Q)|0)|0;i=Q;aa=d|0;Z=fW(1659504)|0;ab=b+8|0;ac=gg(aa,Z,O,0,0,c[ab>>2]|0)|0;if((ac|0)<0){ad=ac;i=f;return ad|0}ac=Bh(c[aa>>2]|0,1658296,1,0,1)|0;if((ac|0)<0){ad=ac;i=f;return ad|0}ac=V+288|0;do{if((c[ac>>2]|0)==0){ae=K;af=e}else{y9(u,256,8192);Z=zu(c[ac>>2]|0)|0;ag=G;zb(u,1656328,(Q=i,i=i+16|0,c[Q>>2]=Z,c[Q+8>>2]=ag,Q)|0);i=Q;ag=c[ac>>2]|0;if((ag|0)>0){Z=0;ah=ag;while(1){ag=c[V+160+(Z<<2)>>2]|0;if((ag|0)==-1){ai=ah}else{zb(u,1654488,(Q=i,i=i+16|0,c[Q>>2]=Z,c[Q+8>>2]=ag,Q)|0);i=Q;ai=c[ac>>2]|0}ag=Z+1|0;if((ag|0)<(ai|0)){Z=ag;ah=ai}else{break}}}ah=u|0;AN(0,32,1653032,(Q=i,i=i+8|0,c[Q>>2]=c[ah>>2],Q)|0);i=Q;Z=fW(1651168)|0;ag=gg(v,Z,1651168,c[ah>>2]|0,0,c[ab>>2]|0)|0;if((ag|0)<0){ad=ag;i=f;return ad|0}ag=fN(K,e,c[v>>2]|0,0)|0;if((ag|0)<0){ad=ag;i=f;return ad|0}else{ag=c[v>>2]|0;c[p>>2]=ag;c[q>>2]=0;zd(u,0)|0;ae=ag;af=0;break}}}while(0);u=c[X+416>>2]|0;do{if((u|0)!=0){v=X+448|0;if(!((c[v>>2]|0)==0&(c[v+4>>2]|0)==0)){break}e=zu(u)|0;c[v>>2]=e;c[v+4>>2]=G}}while(0);u=c[(c[(c[W>>2]|0)+8>>2]|0)+420>>2]|0;do{if((u|0)==-1){X=V+60|0;v=c[X>>2]|0;if((v|0)==0){aj=0;break}if((c[v+32>>2]|0)==0){aj=0;break}c[n>>2]=0;if((iZ(n)|0)<0){eW(1)}v=c[(c[X>>2]|0)+32>>2]|0;X=c[v>>2]|0;if((X|0)!=-1){e=v;v=X;do{X=B2(v)|0;iX(c[n>>2]|0,1630960,(Q=i,i=i+8|0,c[Q>>2]=X,Q)|0)|0;i=Q;e=e+4|0;v=c[e>>2]|0;}while((v|0)!=-1)}v=(i$(c[n>>2]|0,o)|0)-1|0;a[(c[o>>2]|0)+v|0]=0;aj=c[o>>2]|0}else{aj=A9(B2(u)|0)|0}}while(0);c[r>>2]=aj;aj=j|0;j=m|0;m=c[(c[(c[W>>2]|0)+8>>2]|0)+412>>2]|0;do{if((m|0)==0){u=V+60|0;o=c[u>>2]|0;if((o|0)==0){ak=0;break}if((c[o+28>>2]|0)==0){ak=0;break}c[k>>2]=0;if((iZ(k)|0)<0){eW(1)}o=c[(c[u>>2]|0)+28>>2]|0;u=c[o>>2]|0;if((u|0)!=0){n=o;o=u;do{bc(j|0,16,1630408,(Q=i,i=i+8|0,c[Q>>2]=o,Q)|0)|0;i=Q;iX(c[k>>2]|0,1630960,(Q=i,i=i+8|0,c[Q>>2]=j,Q)|0)|0;i=Q;n=n+4|0;o=c[n>>2]|0;}while((o|0)!=0)}o=(i$(c[k>>2]|0,l)|0)-1|0;a[(c[l>>2]|0)+o|0]=0;ak=c[l>>2]|0}else{bc(aj|0,16,1630408,(Q=i,i=i+8|0,c[Q>>2]=m,Q)|0)|0;i=Q;ak=A9(aj)|0}}while(0);c[s>>2]=ak;ak=(c[(c[W>>2]|0)+8>>2]|0)+448|0;W=c[ak>>2]|0;m=c[ak+4>>2]|0;do{if((W|0)==0&(m|0)==0){ak=V+60|0;l=c[ak>>2]|0;if((l|0)==0){al=0;break}if((c[l+36>>2]|0)==0){al=0;break}c[g>>2]=0;if((iZ(g)|0)<0){eW(1)}l=c[(c[ak>>2]|0)+36>>2]|0;ak=c[l>>2]|0;k=c[l+4>>2]|0;if(!((ak|0)==0&(k|0)==0)){o=l;l=k;k=ak;do{bc(j|0,16,1656328,(Q=i,i=i+16|0,c[Q>>2]=k,c[Q+8>>2]=l,Q)|0)|0;i=Q;iX(c[g>>2]|0,1630960,(Q=i,i=i+8|0,c[Q>>2]=j,Q)|0)|0;i=Q;o=o+8|0;k=c[o>>2]|0;l=c[o+4>>2]|0;}while(!((k|0)==0&(l|0)==0))}l=(i$(c[g>>2]|0,h)|0)-1|0;a[(c[h>>2]|0)+l|0]=0;al=c[h>>2]|0}else{l=1656328;bc(aj|0,16,l|0,(Q=i,i=i+16|0,c[Q>>2]=W,c[Q+8>>2]=m,Q)|0)|0;i=Q;al=A9(aj)|0}}while(0);c[t>>2]=al;aj=c[r>>2]|0;do{if((aj|0)==0){r=c[s>>2]|0;if((r|0)==0&(al|0)==0){am=ae;an=af;break}a[T]=0;ao=r;ap=1322}else{a[T]=0;y0(T,256,1649544,(Q=i,i=i+8|0,c[Q>>2]=aj,Q)|0)|0;i=Q;ao=c[s>>2]|0;ap=1322}}while(0);do{if((ap|0)==1322){if((ao|0)!=0){y0(T,256,1647920,(Q=i,i=i+8|0,c[Q>>2]=ao,Q)|0)|0;i=Q}s=c[t>>2]|0;if((s|0)!=0){y0(T,256,1645392,(Q=i,i=i+8|0,c[Q>>2]=s,Q)|0)|0;i=Q}A6(N);A6(R);A6(P);s=c[Y>>2]|0;aj=c[$>>2]|0;bc(O|0,255,1643672,(Q=i,i=i+16|0,c[Q>>2]=s,c[Q+8>>2]=aj,Q)|0)|0;i=Q;aj=fW(1641112)|0;s=gg(w,aj,O,T,0,c[ab>>2]|0)|0;if((s|0)<0){ad=s;i=f;return ad|0}s=fN(ae,af,c[w>>2]|0,0)|0;if((s|0)<0){ad=s;i=f;return ad|0}else{s=c[w>>2]|0;c[p>>2]=s;c[q>>2]=0;am=s;an=0;break}}}while(0);w=c[V+332>>2]|0;L1792:do{if((w|0)!=0){if((c[_+40>>2]|0)==0){break}af=c[_>>2]|0;ae=c[af+24>>2]|0;T=af+28|0;af=0;while(1){if(af>>>0>=ae>>>0){break L1792}if((c[(c[(c[(c[T>>2]|0)+(af<<2)>>2]|0)+8>>2]|0)+8>>2]|0)==0){break}else{af=af+1|0}}bc(U|0,256,1673912,(Q=i,i=i+8|0,c[Q>>2]=w,Q)|0)|0;i=Q;AN(0,32,1639616,(Q=i,i=i+8|0,c[Q>>2]=U,Q)|0);i=Q;af=fW(1637672)|0;T=gg(x,af,1637672,U,0,c[ab>>2]|0)|0;if((T|0)<0){ad=T;i=f;return ad|0}T=fN(am,an,c[x>>2]|0,0)|0;if((T|0)<0){ad=T;i=f;return ad|0}else{c[p>>2]=c[x>>2];c[q>>2]=0;break}}}while(0);x=eA(V,p,q)|0;if((x|0)<0){ad=x;i=f;return ad|0}x=fN(c[p>>2]|0,c[q>>2]|0,c[aa>>2]|0,0)|0;ad=(x|0)<0?x:0;i=f;return ad|0}else if((L|0)==0){L=c[M>>2]|0;M=c[S>>2]|0;S=C;x=F|0;F=H|0;H=c[d+4>>2]|0;aa=H+12|0;q=c[(c[aa>>2]|0)+8>>2]|0;c[D>>2]=L;c[E>>2]=M;p=H|0;V=H+4|0;an=c[V>>2]|0;bc(x|0,255,1660760,(Q=i,i=i+16|0,c[Q>>2]=c[p>>2],c[Q+8>>2]=an,Q)|0)|0;i=Q;an=d|0;d=fW(1629520)|0;am=b+8|0;b=gg(an,d,x,0,0,c[am>>2]|0)|0;if((b|0)<0){ad=b;i=f;return ad|0}b=c[q+120>>2]|0;d=c[q+124>>2]|0;do{if((d|b|0)==0){aq=L;ar=M}else{q=c[H+312>>2]|0;bc(F|0,255,1628832,(Q=i,i=i+24|0,c[Q>>2]=b,c[Q+8>>2]=d,c[Q+16>>2]=q,Q)|0)|0;i=Q;q=c[p>>2]|0;ab=c[V>>2]|0;bc(x|0,255,1628328,(Q=i,i=i+16|0,c[Q>>2]=q,c[Q+8>>2]=ab,Q)|0)|0;i=Q;ab=fW(1627904)|0;q=gg(I,ab,x,F,0,c[am>>2]|0)|0;if((q|0)<0){ad=q;i=f;return ad|0}q=fN(L,M,c[I>>2]|0,0)|0;if((q|0)<0){ad=q;i=f;return ad|0}else{q=c[I>>2]|0;c[D>>2]=q;c[E>>2]=0;aq=q;ar=0;break}}}while(0);do{if((c[H+360>>2]|0)==0){I=c[aa>>2]|0;M=c[(c[I+8>>2]|0)+140>>2]|0;L=H+60|0;F=c[L>>2]|0;if((M|0)!=-1){as=A9(BL(eu(I,F,M)|0)|0)|0;break}if((F|0)==0){as=0;break}if((c[F+24>>2]|0)==0){as=0;break}c[y>>2]=0;if((iZ(y)|0)<0){eW(1)}F=c[(c[L>>2]|0)+24>>2]|0;L=c[(c[aa>>2]|0)+8>>2]|0;do{if((c[L+680>>2]|0)<0){M=c[L+48>>2]|0;if((M|0)==10){I=B|0;c[I>>2]=12;c[B+4>>2]=13;c[B+8>>2]=14;c[B+12>>2]=0;c[B+16>>2]=4;c[B+20>>2]=5;c[B+24>>2]=30;c[B+28>>2]=-1;at=I;break}else if((M|0)==8){M=A|0;c[M>>2]=12;c[A+4>>2]=13;c[A+8>>2]=0;c[A+12>>2]=4;c[A+16>>2]=-1;at=M;break}else{at=F;break}}else{at=F}}while(0);F=c[at>>2]|0;if((F|0)!=-1){L=at;M=F;do{F=BL(M)|0;iX(c[y>>2]|0,1630960,(Q=i,i=i+8|0,c[Q>>2]=F,Q)|0)|0;i=Q;L=L+4|0;M=c[L>>2]|0;}while((M|0)!=-1)}M=(i$(c[y>>2]|0,z)|0)-1|0;a[(c[z>>2]|0)+M|0]=0;as=c[z>>2]|0}else{M=c[H+300>>2]|0;if((M|0)!=0){gi(c[(c[M+8>>2]|0)+8>>2]|0,-1)}M=c[(c[(c[aa>>2]|0)+8>>2]|0)+140>>2]|0;if((M|0)==-1){as=0;break}as=A9(BL(M)|0)|0}}while(0);c[C>>2]=as;do{if((as|0)!=0){C=c[p>>2]|0;aa=c[V>>2]|0;bc(x|0,255,1627328,(Q=i,i=i+16|0,c[Q>>2]=C,c[Q+8>>2]=aa,Q)|0)|0;i=Q;aa=fW(1626832)|0;C=gg(J,aa,1626832,as,0,c[am>>2]|0)|0;A6(S);if((C|0)<0){ad=C;i=f;return ad|0}C=fN(aq,ar,c[J>>2]|0,0)|0;if((C|0)<0){ad=C;i=f;return ad|0}else{c[D>>2]=c[J>>2];c[E>>2]=0;break}}}while(0);J=eA(H,D,E)|0;if((J|0)<0){ad=J;i=f;return ad|0}J=fN(c[D>>2]|0,c[E>>2]|0,c[an>>2]|0,0)|0;ad=(J|0)<0?J:0;i=f;return ad|0}else{AN(0,0,1640232,(Q=i,i=i+24|0,c[Q>>2]=1630624,c[Q+8>>2]=1624232,c[Q+16>>2]=572,Q)|0);i=Q;cw();return 0}return 0}function ey(b){b=b|0;var e=0,f=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0.0,aG=0,aH=0;e=i;i=i+3400|0;f=e|0;j=e+1024|0;k=e+1280|0;l=e+1288|0;m=e+1544|0;n=e+1552|0;o=e+2576|0;p=e+2832|0;q=e+2840|0;r=e+2848|0;s=e+2856|0;t=e+2864|0;u=e+2872|0;v=e+2880|0;w=e+2888|0;x=b+8|0;y=c[x>>2]|0;z=b+4|0;A=c[z>>2]|0;B=(A|0)==0;if(B){C=c[(c[(c[c[b+24>>2]>>2]|0)+4>>2]|0)+304>>2]|0}else{C=A}gf(x);A=gd()|0;c[x>>2]=A;if((A|0)==0){D=-12;i=e;return D|0}if(B){E=b+24|0;F=c[(c[c[E>>2]>>2]|0)+4>>2]|0;G=w|0;H=c[F+312>>2]|0;bc(G|0,512,1618336,(I=i,i=i+8|0,c[I>>2]=H,I)|0)|0;i=I;H=A9(G)|0;c[(c[x>>2]|0)+12>>2]=H;a[G]=0;H=F+324|0;F=zJ(c[H>>2]|0,3118984,0,2)|0;if((F|0)!=0){J=F;do{F=c[J+4>>2]|0;y0(G,512,1740824,(I=i,i=i+16|0,c[I>>2]=c[J>>2],c[I+8>>2]=F,I)|0)|0;i=I;J=zJ(c[H>>2]|0,3118984,J,2)|0;}while((J|0)!=0)}if((a[G]|0)!=0){a[w+((NL(G|0)|0)-1)|0]=0}J=c[x>>2]|0;Bd(J,1736424,G,0)|0;a[G]=0;J=zJ(c[(c[(c[c[E>>2]>>2]|0)+4>>2]|0)+328>>2]|0,3118984,0,2)|0;if((J|0)!=0){H=J;do{J=c[H+4>>2]|0;y0(G,512,1740824,(I=i,i=i+16|0,c[I>>2]=c[H>>2],c[I+8>>2]=J,I)|0)|0;i=I;H=zJ(c[(c[(c[c[E>>2]>>2]|0)+4>>2]|0)+328>>2]|0,3118984,H,2)|0;}while((H|0)!=0)}if((a[G]|0)!=0){a[w+((NL(G|0)|0)-1)|0]=0}w=A9(G)|0;c[(c[x>>2]|0)+16>>2]=w;K=c[x>>2]|0}else{K=A}A=hd(K,C,u,v)|0;if((A|0)<0){D=A;i=e;return D|0}A=c[u>>2]|0;L1889:do{if(B){do{if((A|0)!=0){if((c[A+12>>2]|0)!=0){break}K=c[v>>2]|0;if((K|0)==0){break}if((c[K+12>>2]|0)==0){break L1889}}}while(0);AN(0,16,1731432,(I=i,i=i+8|0,c[I>>2]=C,I)|0);i=I;D=-22;i=e;return D|0}}while(0);C=B|(y|0)!=0;do{if(C|(A|0)==0){L=A}else{y=b+16|0;B=b+20|0;K=A;L1900:while(1){w=K+4|0;G=K+8|0;H=f6(c[(c[w>>2]|0)+12>>2]|0,c[G>>2]|0)|0;if(H>>>0>1){AN(0,8,1667808,(I=i,i=i+1|0,i=i+7&-8,c[I>>2]=0,I)|0);i=I;eW(1)}E=c[K>>2]|0;do{if((E|0)==0){J=c[550914]|0;L1907:do{if((J|0)>0){F=c[551808]|0;M=0;while(1){N=c[F+(M<<2)>>2]|0;if((c[(c[(c[N+4>>2]|0)+8>>2]|0)+8>>2]|0)==(H|0)){if((c[N+8>>2]|0)!=0){O=N;P=M;break L1907}}Q=M+1|0;if((Q|0)<(J|0)){M=Q}else{O=N;P=Q;break}}}else{O=0;P=0}}while(0);if((P|0)!=(J|0)){R=O;break}M=c[(c[w>>2]|0)+8>>2]|0;AN(0,8,1663776,(I=i,i=i+16|0,c[I>>2]=c[G>>2],c[I+8>>2]=M,I)|0);i=I;eW(1);R=O}else{M=aU(E|0,t|0,0)|0;if(!((M|0)>-1&(M|0)<(c[550916]|0))){F=c[z>>2]|0;AN(0,8,1666576,(I=i,i=i+16|0,c[I>>2]=M,c[I+8>>2]=F,I)|0);i=I;eW(1)}F=c[c[(c[551810]|0)+(M<<2)>>2]>>2]|0;Q=F+24|0;N=c[Q>>2]|0;if((N|0)==0){S=1391;break L1900}T=F+28|0;U=c[t>>2]|0;V=U+1|0;L1921:do{if((H|0)==0){W=0;X=N;while(1){Y=c[(c[T>>2]|0)+(W<<2)>>2]|0;Z=c[(c[Y+8>>2]|0)+8>>2]|0;if((Z|0)==0|(Z|0)==3){if((fA(F,Y,(a[U]|0)==58?V:U)|0)==1){_=W;break L1921}$=c[Q>>2]|0}else{$=X}Y=W+1|0;if(Y>>>0<$>>>0){W=Y;X=$}else{aa=U;S=1483;break L1900}}}else{X=0;W=N;while(1){Y=c[(c[T>>2]|0)+(X<<2)>>2]|0;if((c[(c[Y+8>>2]|0)+8>>2]|0)==(H|0)){if((fA(F,Y,(a[U]|0)==58?V:U)|0)==1){_=X;break L1921}ab=c[Q>>2]|0}else{ab=W}Y=X+1|0;if(Y>>>0<ab>>>0){X=Y;W=ab}else{aa=U;S=1484;break L1900}}}}while(0);Q=c[(c[T>>2]|0)+(_<<2)>>2]|0;if((Q|0)==0){aa=U;S=1485;break L1900}R=c[(c[551808]|0)+((c[Q>>2]|0)+(c[(c[(c[551810]|0)+(M<<2)>>2]|0)+12>>2]|0)<<2)>>2]|0}}while(0);if((R|0)==0){S=1410;break}c[R+8>>2]=0;H=R+12|0;c[H>>2]=(c[H>>2]|0)+1;c[(c[R+4>>2]|0)+76>>2]=-16;c[y>>2]=fD(c[y>>2]|0,4,B,(c[B>>2]|0)+1|0)|0;H=A7(16)|0;c[(c[y>>2]|0)+((c[B>>2]|0)-1<<2)>>2]=H;if((H|0)==0){eW(1)}c[(c[(c[y>>2]|0)+((c[B>>2]|0)-1<<2)>>2]|0)+4>>2]=R;c[(c[(c[y>>2]|0)+((c[B>>2]|0)-1<<2)>>2]|0)+8>>2]=b;H=R+244|0;E=R+248|0;G=fD(c[H>>2]|0,4,E,(c[E>>2]|0)+1|0)|0;c[H>>2]=G;c[G+((c[E>>2]|0)-1<<2)>>2]=c[(c[y>>2]|0)+((c[B>>2]|0)-1<<2)>>2];E=c[K+12>>2]|0;if((E|0)==0){S=1380;break}else{K=E}}if((S|0)==1483){ac=c[z>>2]|0;AN(0,8,1665160,(I=i,i=i+16|0,c[I>>2]=aa,c[I+8>>2]=ac,I)|0);i=I;eW(1);return 0}else if((S|0)==1484){ac=c[z>>2]|0;AN(0,8,1665160,(I=i,i=i+16|0,c[I>>2]=aa,c[I+8>>2]=ac,I)|0);i=I;eW(1);return 0}else if((S|0)==1485){ac=c[z>>2]|0;AN(0,8,1665160,(I=i,i=i+16|0,c[I>>2]=aa,c[I+8>>2]=ac,I)|0);i=I;eW(1);return 0}else if((S|0)==1380){L=c[u>>2]|0;break}else if((S|0)==1391){aa=c[t>>2]|0;ac=c[z>>2]|0;AN(0,8,1665160,(I=i,i=i+16|0,c[I>>2]=aa,c[I+8>>2]=ac,I)|0);i=I;eW(1);return 0}else if((S|0)==1410){AN(0,0,1640232,(I=i,i=i+24|0,c[I>>2]=1662288,c[I+8>>2]=1624232,c[I+16>>2]=263,I)|0);i=I;cw();return 0}}}while(0);L1950:do{if((L|0)!=0){ac=b+16|0;aa=o|0;z=p|0;t=p+4|0;R=b|0;_=n|0;ab=j|0;$=l|0;O=f|0;P=b+12|0;A=0;K=L;L1952:while(1){B=c[(c[ac>>2]|0)+(A<<2)>>2]|0;y=B+12|0;A6(y);E=K+4|0;G=c[E>>2]|0;H=c[G+12>>2]|0;w=c[G+20>>2]|0;if((iZ(s)|0)<0){eW(1)}iX(c[s>>2]|0,1673912,(I=i,i=i+8|0,c[I>>2]=c[c[G+4>>2]>>2],I)|0)|0;i=I;if((w|0)>1){w=c[s>>2]|0;G=K+8|0;Q=f5(H,c[G>>2]|0)|0;iX(w,1658744,(I=i,i=i+8|0,c[I>>2]=Q,I)|0)|0;i=I;ad=G}else{ad=K+8|0}ih(c[s>>2]|0,0);i$(c[s>>2]|0,y)|0;y=f6(c[(c[E>>2]|0)+12>>2]|0,c[ad>>2]|0)|0;do{if((y|0)==0){G=c[E>>2]|0;Q=fW(1684248)|0;w=c[B+4>>2]|0;H=w+112|0;V=c[H>>2]|0;if((V|0)==0){F=w+4|0;N=c[F>>2]|0;J=N+40|0;W=N+44|0;X=d[J]|d[J+1|0]<<8|d[J+2|0]<<16|d[J+3|0]<<24|0;J=d[W]|d[W+1|0]<<8|d[W+2|0]<<16|d[W+3|0]<<24|0;W=c[ad>>2]|0;mZ(p,c[c[(c[551810]|0)+(c[w>>2]<<2)>>2]>>2]|0,N,0);ae=c[z>>2]|0;af=c[t>>2]|0;ag=F;ah=X;ai=J;aj=W}else{W=w+116|0;J=d[W]|d[W+1|0]<<8|d[W+2|0]<<16|d[W+3|0]<<24|0;ae=V;af=J;ag=w+4|0;ah=J;ai=V;aj=c[ad>>2]|0}V=c[ag>>2]|0;J=c[V+8>>2]|0;if((c[J+8>>2]|0)==3){W=c[c[(c[551810]|0)+(c[w>>2]<<2)>>2]>>2]|0;X=c[J+120>>2]|0;F=c[J+124>>2]|0;if((X|0)==0|(F|0)==0){J=c[W+24>>2]|0;if((J|0)==0){ak=X;al=F}else{N=c[W+28>>2]|0;Y=0;Z=X;am=F;while(1){an=c[(c[N+(Y<<2)>>2]|0)+8>>2]|0;if((c[an+8>>2]|0)==0){ao=c[an+120>>2]|0;ap=c[an+124>>2]|0;aq=(am|0)>(ap|0)?am:ap;ar=(Z|0)>(ao|0)?Z:ao}else{aq=am;ar=Z}ao=Y+1|0;if(ao>>>0<J>>>0){Y=ao;Z=ar;am=aq}else{ak=ar;al=aq;break}}}if((ak|0)==0|(al|0)==0){as=(al|0)>576?al:576;at=(ak|0)>720?ak:720}else{as=al;at=ak}AN(W,32,1669376,(I=i,i=i+16|0,c[I>>2]=at,c[I+8>>2]=as,I)|0);i=I;au=as;av=at;aw=c[ag>>2]|0}else{au=F;av=X;aw=V}c[w+132>>2]=av;c[(c[aw+8>>2]|0)+120>>2]=av;c[w+228>>2]=av;c[w+128>>2]=au;c[(c[(c[ag>>2]|0)+8>>2]|0)+124>>2]=au;c[w+232>>2]=au;c[(c[(c[ag>>2]|0)+8>>2]|0)+140>>2]=30;c[w+136>>2]=30;am=As()|0;c[w+224>>2]=am;Z=(am|0)==0;if(Z){ax=Z?-12:0;break}ay=c[ag>>2]|0}else{ay=V}Z=ay+80|0;if((c[Z>>2]|0)==0){am=c[ay+8>>2]|0;az=am+224|0;aA=am+228|0}else{az=Z;aA=ay+84|0}Z=c[aA>>2]|0;am=(Z|0)==0;Y=am?0:c[az>>2]|0;y9(n,0,1);J=c[w+128>>2]|0;N=c[w+136>>2]|0;M=(c[(c[(c[ag>>2]|0)+8>>2]|0)+88>>2]|0)>>>4&524288|2;zb(n,1682464,(I=i,i=i+64|0,c[I>>2]=c[w+132>>2],c[I+8>>2]=J,c[I+16>>2]=N,c[I+24>>2]=ah,c[I+32>>2]=ai,c[I+40>>2]=Y,c[I+48>>2]=am?1:Z,c[I+56>>2]=M,I)|0);i=I;if(!((ae|0)==0|(af|0)==0)){zb(n,1680592,(I=i,i=i+16|0,c[I>>2]=ae,c[I+8>>2]=af,I)|0);i=I}M=c[R>>2]|0;Z=w|0;am=c[Z>>2]|0;Y=c[c[ag>>2]>>2]|0;bc(aa|0,255,1711832,(I=i,i=i+24|0,c[I>>2]=M,c[I+8>>2]=am,c[I+16>>2]=Y,I)|0)|0;i=I;Y=B|0;am=gg(Y,Q,aa,c[_>>2]|0,0,c[x>>2]|0)|0;if((am|0)<0){D=am;S=1490;break L1952}if((c[H>>2]|0)==0){aB=G;aC=aj}else{am=c[Z>>2]|0;M=c[c[ag>>2]>>2]|0;bc(aa|0,255,1678696,(I=i,i=i+16|0,c[I>>2]=am,c[I+8>>2]=M,I)|0)|0;i=I;M=fW(1676584)|0;am=gg(q,M,aa,1674960,0,c[x>>2]|0)|0;if((am|0)<0){D=am;S=1491;break L1952}am=fN(c[q>>2]|0,0,G,aj)|0;if((am|0)<0){D=am;S=1487;break L1952}aB=c[q>>2]|0;aC=0}if((c[744102]|0)==0){aD=aB;aE=aC}else{am=c[Z>>2]|0;Z=c[c[ag>>2]>>2]|0;bc(aa|0,255,1673168,(I=i,i=i+16|0,c[I>>2]=am,c[I+8>>2]=Z,I)|0)|0;i=I;Z=fW(1671160)|0;am=gg(r,Z,aa,3118984,0,c[x>>2]|0)|0;if((am|0)<0){D=am;S=1492;break L1952}am=fN(c[r>>2]|0,0,aB,aC)|0;if((am|0)<0){D=am;S=1493;break L1952}aD=c[r>>2]|0;aE=0}am=fN(c[Y>>2]|0,0,aD,aE)|0;ax=(am|0)<0?am:0}else if((y|0)==1){am=c[E>>2]|0;Y=fW(1726904)|0;Z=c[B+4>>2]|0;M=c[ad>>2]|0;y9(f,0,1);N=Z+4|0;J=c[(c[N>>2]|0)+8>>2]|0;U=c[J+412>>2]|0;T=B2(c[J+420>>2]|0)|0;zb(f,1722696,(I=i,i=i+32|0,c[I>>2]=1,c[I+8>>2]=U,c[I+16>>2]=U,c[I+24>>2]=T,I)|0);i=I;T=c[(c[N>>2]|0)+8>>2]|0;U=T+448|0;J=c[U>>2]|0;ao=c[U+4>>2]|0;if((J|0)==0&(ao|0)==0){zb(f,1714336,(I=i,i=i+8|0,c[I>>2]=c[T+416>>2],I)|0);i=I}else{zb(f,1718776,(I=i,i=i+16|0,c[I>>2]=J,c[I+8>>2]=ao,I)|0);i=I}ao=Z|0;Z=c[ao>>2]|0;J=c[c[N>>2]>>2]|0;bc(aa|0,255,1711832,(I=i,i=i+24|0,c[I>>2]=c[R>>2],c[I+8>>2]=Z,c[I+16>>2]=J,I)|0)|0;i=I;J=B|0;Z=gg(J,Y,aa,c[O>>2]|0,0,c[x>>2]|0)|0;if((Z|0)<0){D=Z;S=1494;break L1952}Z=c[767224]|0;if((Z|0)>0){NM(ab|0,0,256);y0(ab,256,1708424,(I=i,i=i+8|0,c[I>>2]=Z,I)|0)|0;i=I;aF=+g[86704];if(aF!=.1){y0(ab,256,1706272,(I=i,i=i+8|0,h[I>>3]=aF,I)|0)|0;i=I}if((c[P>>2]|0)==0){y0(ab,256,1704400,(I=i,i=i+1|0,i=i+7&-8,c[I>>2]=0,I)|0)|0;i=I}AN(0,32,1702136,(I=i,i=i+8|0,c[I>>2]=ab,I)|0);i=I;Z=c[R>>2]|0;Y=c[ao>>2]|0;T=c[c[N>>2]>>2]|0;bc(aa|0,255,1698872,(I=i,i=i+32|0,c[I>>2]=Z,c[I+8>>2]=1696856,c[I+16>>2]=Y,c[I+24>>2]=T,I)|0)|0;i=I;T=fW(1696856)|0;Y=gg(k,T,aa,ab,0,c[x>>2]|0)|0;if((Y|0)<0){D=Y;S=1495;break L1952}Y=fN(c[k>>2]|0,0,am,M)|0;if((Y|0)<0){D=Y;S=1496;break L1952}aG=c[k>>2]|0}else{aG=am}if((c[86702]|0)==256){aH=aG}else{AN(0,24,1694560,(I=i,i=i+1|0,i=i+7&-8,c[I>>2]=0,I)|0);i=I;aF=+(c[86702]|0)*.00390625;bc($|0,256,1690968,(I=i,i=i+8|0,h[I>>3]=aF,I)|0)|0;i=I;AN(0,32,1688848,(I=i,i=i+8|0,c[I>>2]=$,I)|0);i=I;am=c[R>>2]|0;Y=c[ao>>2]|0;ao=c[c[N>>2]>>2]|0;bc(aa|0,255,1698872,(I=i,i=i+32|0,c[I>>2]=am,c[I+8>>2]=1686168,c[I+16>>2]=Y,c[I+24>>2]=ao,I)|0)|0;i=I;ao=fW(1686168)|0;Y=gg(m,ao,aa,$,0,c[x>>2]|0)|0;if((Y|0)<0){D=Y;S=1497;break L1952}Y=fN(c[m>>2]|0,0,aG,M)|0;if((Y|0)<0){D=Y;S=1498;break L1952}aH=c[m>>2]|0}Y=fN(c[J>>2]|0,0,aH,M)|0;ax=(Y|0)<0?Y:0}else{S=1467;break L1952}}while(0);if((ax|0)<0){D=ax;S=1499;break}B=c[K+12>>2]|0;if((B|0)==0){break L1950}else{A=A+1|0;K=B}}if((S|0)==1487){i=e;return D|0}else if((S|0)==1490){i=e;return D|0}else if((S|0)==1491){i=e;return D|0}else if((S|0)==1492){i=e;return D|0}else if((S|0)==1467){AN(0,0,1640232,(I=i,i=i+24|0,c[I>>2]=1630624,c[I+8>>2]=1624232,c[I+16>>2]=796,I)|0);i=I;cw();return 0}else if((S|0)==1493){i=e;return D|0}else if((S|0)==1494){i=e;return D|0}else if((S|0)==1495){i=e;return D|0}else if((S|0)==1496){i=e;return D|0}else if((S|0)==1497){i=e;return D|0}else if((S|0)==1498){i=e;return D|0}else if((S|0)==1499){i=e;return D|0}}}while(0);hc(u);u=c[v>>2]|0;S=(u|0)==0;do{if(C){if(!S){I=b+24|0;ax=0;aH=u;while(1){ex(b,c[(c[I>>2]|0)+(ax<<2)>>2]|0,aH)|0;m=c[aH+12>>2]|0;if((m|0)==0){break}else{ax=ax+1|0;aH=m}}}hc(v);aH=gj(c[x>>2]|0,0)|0;if((aH|0)<0){D=aH}else{break}i=e;return D|0}else{if(S){break}aH=b+24|0;ax=b+28|0;I=u;do{c[aH>>2]=fD(c[aH>>2]|0,4,ax,(c[ax>>2]|0)+1|0)|0;m=A7(20)|0;c[(c[aH>>2]|0)+((c[ax>>2]|0)-1<<2)>>2]=m;if((m|0)==0){eW(1)}c[(c[(c[aH>>2]|0)+((c[ax>>2]|0)-1<<2)>>2]|0)+8>>2]=b;c[(c[(c[aH>>2]|0)+((c[ax>>2]|0)-1<<2)>>2]|0)+16>>2]=I;I=c[I+12>>2]|0;c[(c[(c[(c[aH>>2]|0)+((c[ax>>2]|0)-1<<2)>>2]|0)+16>>2]|0)+12>>2]=0;}while((I|0)!=0)}}while(0);c[b+12>>2]=1;D=0;i=e;return D|0}function ez(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=a+16|0;e=c[a+20>>2]|0;a=0;while(1){if((a|0)>=(e|0)){f=0;g=1507;break}if((c[(c[(c[d>>2]|0)+(a<<2)>>2]|0)+4>>2]|0)==(b|0)){f=1;g=1506;break}else{a=a+1|0}}if((g|0)==1507){return f|0}else if((g|0)==1506){return f|0}return 0}function eA(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;e=i;i=i+128|0;f=e|0;g=a|0;h=c[(c[550878]|0)+(c[g>>2]<<2)>>2]|0;j=c[(c[b>>2]|0)+48>>2]|0;k=(c[(c[(c[a+12>>2]|0)+8>>2]|0)+8>>2]|0)==0?1635928:1635296;l=h+16|0;m=h+24|0;n=c[m>>2]|0;o=c[m+4>>2]|0;do{if((c[l>>2]|0)==(-1|0)&(c[l+4>>2]|0)==2147483647){if((n|0)==0&(o|0)==0){p=0;i=e;return p|0}else{break}}else{if((n|0)==0&(o|0)==0){p=0}else{break}i=e;return p|0}}while(0);o=h+24|0;h=fW(k)|0;if((h|0)==0){AN(0,16,1634648,(q=i,i=i+8|0,c[q>>2]=k,q)|0);i=q;p=-1279870712;i=e;return p|0}n=f|0;f=c[g>>2]|0;g=c[a+4>>2]|0;bc(n|0,128,1633912,(q=i,i=i+24|0,c[q>>2]=k,c[q+8>>2]=f,c[q+16>>2]=g,q)|0)|0;i=q;g=gh(j,h,n)|0;if((g|0)==0){p=-12;i=e;return p|0}n=c[l>>2]|0;h=c[l+4>>2]|0;if((n|0)==(-1|0)&(h|0)==2147483647){r=1517}else{l=Bi(g,1633376,(+(n>>>0)+ +(h|0)*4294967296.0)/1.0e6,1)|0;if((l|0)>-1){r=1517}else{s=l}}do{if((r|0)==1517){l=c[o>>2]|0;h=c[o+4>>2]|0;if(!((l|0)==0&(h|0)==0)){n=Bi(g,1632480,(+(l>>>0)+ +(h|0)*4294967296.0)/1.0e6,1)|0;if((n|0)<0){s=n;break}}n=f2(g,0)|0;if((n|0)<0){p=n;i=e;return p|0}n=fN(c[b>>2]|0,c[d>>2]|0,g,0)|0;if((n|0)<0){p=n;i=e;return p|0}c[b>>2]=g;c[d>>2]=0;p=0;i=e;return p|0}}while(0);AN(g,16,1631552,(q=i,i=i+8|0,c[q>>2]=k,q)|0);i=q;p=s;i=e;return p|0}function eB(){var b=0,d=0;b=i;AN(0,-8,1636408,(d=i,i=i+8|0,c[d>>2]=3119008,d)|0);i=d;if(!(a[85776]|0)){i=b;return}cM(0,0,2203568)|0;i=b;return}function eC(){var b=0,d=0,e=0;b=i;i=i+64|0;d=b|0;if(!(a[85008]|0)){do{if((b8(0)|0)!=0){if((b8(2)|0)==0){break}if((bj(0,d|0)|0)!=0){break}e=d;NN(2203568,e|0,60)|0;a[85776]=1;e=d|0;c[e>>2]=c[e>>2]&-1516;e=d+4|0;c[e>>2]=c[e>>2]|1;e=d+12|0;c[e>>2]=c[e>>2]&-32843;e=d+8|0;c[e>>2]=c[e>>2]&-305|48;a[d+23|0]=1;a[d+22|0]=0;cM(0,0,d|0)|0}}while(0);a6(3,74)|0}mY()|0;a6(2,74)|0;a6(15,74)|0;a6(24,74)|0;i=b;return}function eD(b){b=b|0;var d=0;d=i;c[541004]=b;c[541006]=(c[541006]|0)+1;AN(0,-8,1636408,(b=i,i=i+8|0,c[b>>2]=3119008,b)|0);i=b;if(a[85776]|0){cM(0,0,2203568)|0}if((c[541006]|0)<=3){i=d;return}eW(123);i=d;return}function eE(a){a=a|0;return(c[541006]|0)>1|0}function eF(a){a=a|0;var b=0,d=0;b=i;d=zJ(a,3119008,0,2)|0;if((d|0)==0){i=b;return}AN(0,8,1623064,(a=i,i=i+8|0,c[a>>2]=c[d>>2],a)|0);i=a;eW(1);i=b;return}function eG(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0;b=i;i=i+256|0;d=a+4|0;e=c[(c[d>>2]|0)+8>>2]|0;f=e+448|0;if(!((c[f>>2]|0)==0&(c[f+4>>2]|0)==0)){g=1;i=b;return g|0}h=e+416|0;e=c[h>>2]|0;if((e|0)>(c[a+124>>2]|0)){g=0;i=b;return g|0}j=zu(e)|0;e=G;c[f>>2]=j;c[f+4>>2]=e;if((j|0)==0&(e|0)==0){g=0;i=b;return g|0}f=b|0;zt(f,256,c[h>>2]|0,j,e);e=c[c[d>>2]>>2]|0;AN(0,24,1617432,(d=i,i=i+24|0,c[d>>2]=c[a>>2],c[d+8>>2]=e,c[d+16>>2]=f,d)|0);i=d;g=1;i=b;return g|0}function eH(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,p=0.0,q=0,r=0,s=0;e=i;i=i+272|0;f=e|0;g=e+136|0;eV(20);cW(c[o>>2]|0,0,2,0)|0;AQ(1);e3(b,d,117856);do{if((b|0)>1){j=d+4|0;if((a8(c[j>>2]|0,1612600)|0)!=0){k=b;l=d;break}a[85008]=1;AR(44);k=b-1|0;l=j}else{k=b;l=d}}while(0);nz();fH();fK();hM();mX()|0;ff(k,l,117856);eC();if((dK(k,l)|0)<0){eW(1)}l=c[550912]|0;if((l|0)<1&(c[550916]|0)==0){dJ();AN(0,24,1739552,(m=i,i=i+8|0,c[m>>2]=90712,m)|0);i=m;eW(1);n=c[550912]|0}else{n=l}if((n|0)<1){AN(0,8,1735080,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;eW(1)}cj(0,g|0)|0;n=c[g>>2]|0;l=N2(n,(n|0)<0|0?-1:0,1e6,0)|0;n=c[g+4>>2]|0;g=NU(l,G,n,(n|0)<0|0?-1:0)|0;n=G;c[745298]=g;if((eK()|0)<0){eW(1)}cj(0,f|0)|0;l=c[f>>2]|0;k=c[f+4>>2]|0;if((c[744106]|0)!=0){f=N2(l,(l|0)<0|0?-1:0,1e6,0)|0;l=G;d=NV(k,(k|0)<0|0?-1:0,g,n)|0;n=NU(d,G,f,l)|0;p=(+(n>>>0)+ +(G|0)*4294967296.0)/1.0e6;bT(1730224,(m=i,i=i+8|0,h[m>>3]=p,m)|0)|0;i=m}n=2981168;l=c[n+4>>2]|0;f=2981176;d=c[f>>2]|0;g=c[f+4>>2]|0;AN(0,48,1725856,(m=i,i=i+32|0,c[m>>2]=c[n>>2],c[m+8>>2]=l,c[m+16>>2]=d,c[m+24>>2]=g,m)|0);i=m;m=2981168;g=c[m>>2]|0;d=c[m+4>>2]<<1|g>>>31;m=2981176;l=c[m+4>>2]|0;if(!((d|0)<(l|0)|(d|0)==(l|0)&(g<<1|0>>>31)>>>0<(c[m>>2]|0)>>>0)){q=c[541006]|0;r=(q|0)!=0;s=r?255:0;eW(s);i=e;return 0}eW(254);q=c[541006]|0;r=(q|0)!=0;s=r?255:0;eW(s);i=e;return 0}function eI(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;b=i;i=i+136|0;d=b|0;if((c[744106]|0)!=0){cj(0,d|0)|0;e=c[d+16>>2]|0;d=e;f=N0(d<<10|0>>>22,((e|0)<0|0?-1:0)<<10|d>>>22,1024,0)|0;d=f;bT(1711240,(g=i,i=i+8|0,c[g>>2]=d,g)|0)|0;i=g}if((c[550922]|0)>0){d=0;do{gf((c[(c[556308]|0)+(d<<2)>>2]|0)+8|0);f=c[(c[556308]|0)+(d<<2)>>2]|0;e=f+16|0;if((c[f+20>>2]|0)>0){f=0;h=e;while(1){A6((c[(c[h>>2]|0)+(f<<2)>>2]|0)+12|0);A6((c[(c[(c[556308]|0)+(d<<2)>>2]|0)+16>>2]|0)+(f<<2)|0);j=f+1|0;k=c[(c[556308]|0)+(d<<2)>>2]|0;l=k+16|0;if((j|0)<(c[k+20>>2]|0)){f=j;h=l}else{m=l;break}}}else{m=e}A6(m);h=c[(c[556308]|0)+(d<<2)>>2]|0;f=h+24|0;if((c[h+28>>2]|0)>0){h=0;l=f;while(1){A6((c[(c[l>>2]|0)+(h<<2)>>2]|0)+12|0);A6((c[(c[(c[556308]|0)+(d<<2)>>2]|0)+24>>2]|0)+(h<<2)|0);j=h+1|0;k=c[(c[556308]|0)+(d<<2)>>2]|0;n=k+24|0;if((j|0)<(c[k+28>>2]|0)){h=j;l=n}else{o=n;break}}}else{o=f}A6(o);A6((c[(c[556308]|0)+(d<<2)>>2]|0)+4|0);A6((c[556308]|0)+(d<<2)|0);d=d+1|0;}while((d|0)<(c[550922]|0))}A6(2225232);A6(2162256);if((c[550912]|0)>0){d=0;do{o=c[c[(c[550878]|0)+(d<<2)>>2]>>2]|0;do{if((o|0)!=0){m=c[o+8>>2]|0;if((m|0)==0){break}if((c[m+28>>2]&1|0)!=0){break}m=c[o+16>>2]|0;if((m|0)==0){break}iV(m)|0}}while(0);mi(o);zL((c[(c[550878]|0)+(d<<2)>>2]|0)+4|0);A6((c[550878]|0)+(d<<2)|0);d=d+1|0;}while((d|0)<(c[550912]|0))}if((c[550910]|0)>0){d=0;do{f=c[(c[550876]|0)+(d<<2)>>2]|0;m=c[f+56>>2]|0;if((m|0)==0){p=f}else{f=m;while(1){m=c[f+12>>2]|0;nR(f);if((m|0)==0){break}else{f=m}}p=c[(c[550876]|0)+(d<<2)>>2]|0}c[p+56>>2]=0;o1((c[(c[550876]|0)+(d<<2)>>2]|0)+72|0);A6((c[(c[550876]|0)+(d<<2)>>2]|0)+112|0);zP(c[(c[(c[550876]|0)+(d<<2)>>2]|0)+116>>2]|0);A6((c[(c[550876]|0)+(d<<2)>>2]|0)+304|0);A6((c[(c[550876]|0)+(d<<2)>>2]|0)+292|0);A6((c[550876]|0)+(d<<2)|0);d=d+1|0;}while((d|0)<(c[550910]|0))}if((c[550916]|0)>0){d=0;do{mJ(c[(c[551810]|0)+(d<<2)>>2]|0);A6((c[551810]|0)+(d<<2)|0);d=d+1|0;}while((d|0)<(c[550916]|0))}if((c[550914]|0)>0){d=0;do{At((c[(c[551808]|0)+(d<<2)>>2]|0)+20|0);At((c[(c[551808]|0)+(d<<2)>>2]|0)+24|0);zL((c[(c[551808]|0)+(d<<2)>>2]|0)+108|0);pj((c[(c[551808]|0)+(d<<2)>>2]|0)+176|0);At((c[(c[551808]|0)+(d<<2)>>2]|0)+224|0);A6((c[(c[551808]|0)+(d<<2)>>2]|0)+244|0);A6((c[551808]|0)+(d<<2)|0);d=d+1|0;}while((d|0)<(c[550914]|0))}d=c[499384]|0;if((d|0)!=0){aR(d|0)|0}A4(c[499382]|0);A6(2207232);A6(2207240);A6(2203504);A6(2203512);eT();mY()|0;if((c[541004]|0)!=0){AN(0,32,1711104,(g=i,i=i+8|0,c[g>>2]=c[541004],g)|0);i=g}AN(0,-8,1636408,(g=i,i=i+8|0,c[g>>2]=3119008,g)|0);i=g;if(!(a[85776]|0)){i=b;return}cM(0,0,2203568)|0;i=b;return}function eJ(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;return}function eK(){var b=0,e=0,f=0,j=0,k=0,l=0,m=0,n=0,q=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ad=0,ae=0,af=0,ag=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aS=0,aU=0,aV=0,aX=0,aY=0,a_=0,a$=0,a0=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a9=0,bb=0,bd=0,be=0,bf=0,bg=0,bh=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0.0,bs=0,bt=0.0,bu=0.0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0,bU=0,bW=0,bX=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b7=0,b8=0,b9=0,ca=0,cb=0,cc=0,cd=0,ce=0,cf=0,cg=0,cj=0,ck=0,cl=0,cm=0,cn=0,co=0,cp=0,cq=0,cr=0,cs=0,cu=0,cv=0,cy=0,cz=0,cA=0,cB=0,cC=0,cD=0,cE=0,cF=0,cG=0,cH=0,cI=0,cJ=0,cK=0,cL=0,cN=0,cO=0,cP=0,cQ=0,cR=0,cS=0,cT=0,cU=0,cV=0,cW=0,cX=0,cY=0,cZ=0,c_=0,c$=0,c0=0,c1=0,c2=0,c3=0,c4=0,c5=0,c6=0,c7=0,c8=0,c9=0,da=0,db=0,dc=0,dd=0,de=0,df=0,dg=0,dh=0,di=0,dj=0,dl=0,dm=0,dn=0,dp=0,dq=0,dr=0,ds=0,dt=0,du=0,dv=0,dw=0,dx=0,dy=0,dz=0,dA=0,dB=0,dC=0,dD=0,dE=0,dF=0;b=i;i=i+24608|0;e=b|0;f=b+80|0;j=b+88|0;k=b+96|0;l=b+128|0;m=b+136|0;n=b+216|0;q=b+248|0;s=b+280|0;t=b+312|0;u=b+344|0;v=b+376|0;w=b+408|0;x=b+440|0;y=b+472|0;z=b+504|0;A=b+536|0;B=b+544|0;C=b+552|0;D=b+560|0;E=b+568|0;F=b+576|0;H=b+584|0;I=b+592|0;J=b+600|0;K=b+608|0;L=b+616|0;M=b+624|0;N=b+632|0;O=b+640|0;P=b+672|0;S=b+704|0;T=b+736|0;U=b+768|0;V=b+800|0;W=b+832|0;X=b+840|0;Y=b+848|0;Z=b+856|0;_=b+864|0;$=b+992|0;aa=b+1e3|0;ad=b+1008|0;ae=b+1136|0;af=b+5232|0;ag=b+5296|0;ai=b+5552|0;aj=b+5808|0;ak=b+5816|0;al=b+5824|0;am=b+5832|0;an=b+5840|0;ao=b+5968|0;ap=b+22352|0;aq=b+23384|0;ar=b+23392|0;as=b+23400|0;at=b+23408|0;au=b+23416|0;av=b+23424|0;aw=b+23432|0;ax=b+24464|0;ay=b+24472|0;az=b+22360|0;aA=b+23440|0;aB=b+24480|0;aC=as;aD=at;aE=aw;aF=c[550916]|0;if((aF|0)>0){aG=0;aH=aF;while(1){aF=c[(c[551810]|0)+(aG<<2)>>2]|0;do{if((c[aF+40>>2]|0)==0){aI=aH}else{aJ=aF+32|0;if((c[aJ>>2]|0)<=0){aI=aH;break}aK=aF+12|0;aL=0;do{aM=Ch()|0;aN=(c[(c[551808]|0)+((c[aK>>2]|0)+aL<<2)>>2]|0)+32|0;c[aN>>2]=aM;c[aN+4>>2]=G;aL=aL+1|0;}while((aL|0)<(c[aJ>>2]|0));aI=c[550916]|0}}while(0);aF=aG+1|0;if((aF|0)<(aI|0)){aG=aF;aH=aI}else{break}}}aI=c[550912]|0;L2226:do{if((aI|0)>0){aH=c[550878]|0;aG=0;while(1){aO=c[c[aH+(aG<<2)>>2]>>2]|0;if((c[aO+24>>2]|0)==0){if((c[(c[aO+8>>2]|0)+28>>2]&4096|0)==0){break}}aF=aG+1|0;if((aF|0)<(aI|0)){aG=aF}else{aP=0;aQ=0;aS=1631;break L2226}}mN(aO,aG,aO+32|0,1);AN(0,16,1731720,(aU=i,i=i+8|0,c[aU>>2]=aG,aU)|0);i=aU;aV=-22}else{aP=0;aQ=0;aS=1631}}while(0);L2234:do{if((aS|0)==1631){while(1){aS=0;if((aQ|0)>=(c[550922]|0)){break}aO=gj(c[(c[(c[556308]|0)+(aQ<<2)>>2]|0)+8>>2]|0,0)|0;if((aO|0)<0){aV=aO;break L2234}else{aP=aO;aQ=aQ+1|0;aS=1631}}L2239:do{if((c[550910]|0)>0){aG=al|0;aO=al+4|0;aI=am|0;aH=am+4|0;aF=av|0;aJ=av+4|0;aL=au|0;aK=au+4|0;aN=ar|0;aM=ar+4|0;aX=0;aY=aP;L2241:while(1){a_=c[(c[550876]|0)+(aX<<2)>>2]|0;a$=a_|0;a0=c[c[(c[550878]|0)+(c[a$>>2]<<2)>>2]>>2]|0;a2=a_+8|0;a3=c[a2>>2]|0;if((a3|0)>-1){a4=c[(c[551808]|0)+(a3<<2)>>2]|0}else{a4=0}L2246:do{if((c[a_+348>>2]|0)==0){a5=a_+12|0;a3=c[a5>>2]|0;a6=c[a3+8>>2]|0;a7=(a4|0)!=0;if(a7){a9=c[a4+4>>2]|0;bb=c[a9+8>>2]|0;c[a3+72>>2]=c[a9+72>>2];c[a6+796>>2]=c[bb+796>>2];c[a6+400>>2]=c[bb+400>>2];bd=bb}else{bd=0}if((c[a_+344>>2]|0)!=0){if(!a7){aS=2245;break L2241}if((c[a_+300>>2]|0)!=0){aS=2246;break L2241}bb=bd+100|0;a9=c[bb>>2]|0;a3=NU(a9,(a9|0)<0|0?-1:0,16,0)|0;a9=G;be=0;if(a9>>>0>be>>>0|a9>>>0==be>>>0&a3>>>0>2147483647>>>0){aV=-22;break L2234}be=bd+48|0;a9=a6+48|0;c[a9>>2]=c[be>>2];bf=a6+8|0;c[bf>>2]=c[bd+8>>2];bg=a6+52|0;L2256:do{if((c[bg>>2]|0)==0){bh=a0+8|0;bj=c[(c[bh>>2]|0)+32>>2]|0;bk=bd+52|0;do{if((bj|0)!=0){bl=mE(bj,c[bk>>2]|0)|0;if((bl|0)==(c[a9>>2]|0)){break}if((mD(c[(c[bh>>2]|0)+32>>2]|0,c[be>>2]|0,aq)|0)!=0){break L2256}}}while(0);c[bg>>2]=c[bk>>2]}}while(0);c[a6+72>>2]=c[bd+72>>2];c[a6+540>>2]=c[bd+540>>2];c[a6+524>>2]=c[bd+524>>2];c[a6+408>>2]=c[bd+408>>2];be=A7(a3)|0;c[a6+96>>2]=be;if((be|0)==0){aV=-12;break L2234}bh=c[bd+96>>2]|0;bj=c[bb>>2]|0;NN(be|0,bh|0,bj)|0;c[a6+100>>2]=c[bb>>2];c[a6+792>>2]=c[bd+792>>2];bj=a6+104|0;bh=a4+4|0;be=(c[bh>>2]|0)+40|0;bl=bj;bm=c[be+4>>2]|0;c[bl>>2]=c[be>>2];c[bl+4>>2]=bm;bm=c[a0+8>>2]|0;be=c[bm>>2]|0;L2264:do{if((a8(be|0,1730280)|0)==0){bn=c[66006]|0;bo=(bn|0)<0;do{if(bo){bp=c[bh>>2]|0;bq=c[bp+416>>2]|0;br=+(c[bp+412>>2]|0)/+(bq|0);if(br<+(c[bp+92>>2]|0)/+(c[bp+96>>2]|0)){bs=bp;aS=1663;break}bt=.5/br;br=+(c[bp+40>>2]|0)/+(c[bp+44>>2]|0);if(bt<=br){aS=1658;break}bu=+(c[bd+104>>2]|0)/+(c[bd+108>>2]|0);if(!(bt>bu&br<.002)){aS=1658;break}if(bu<.002|(bn|0)==2){bv=bq;aS=1660}else{aS=1661}}else{aS=1658}}while(0);do{if((aS|0)==1658){aS=0;if((bn|0)!=2){aS=1661;break}bv=c[(c[bh>>2]|0)+416>>2]|0;aS=1660}}while(0);if((aS|0)==1660){aS=0;c[bj>>2]=bv;c[a6+108>>2]=c[(c[bh>>2]|0)+412>>2]<<1;c[a6+112>>2]=2;break}do{if((aS|0)==1661){aS=0;if(!bo){aS=1665;break}bs=c[bh>>2]|0;aS=1663}}while(0);do{if((aS|0)==1663){aS=0;bo=bd+112|0;bu=+(c[bs+40>>2]|0)/+(c[bs+44>>2]|0);if(+(c[bd+104>>2]|0)/+(c[bd+108>>2]|0)*+(c[bo>>2]|0)<=bu*2.0){aS=1665;break}if(bu<.002){bw=bo}else{break L2264}}}while(0);if((aS|0)==1665){aS=0;if((bn|0)!=0){break}bw=bd+112|0}bo=bd+104|0;bk=c[bo>>2]|0;bq=c[bo+4>>2]|0;c[bl>>2]=bk;c[bl+4>>2]=bq;c[bj>>2]=ac(bk,c[bw>>2]|0)|0;c[a6+108>>2]=bq<<1|0>>>31;c[a6+112>>2]=2}else{if((c[bm+28>>2]&1024|0)!=0){break}if((a8(be|0,1730016)|0)==0){break}if((a8(be|0,1729696)|0)==0){break}if((a8(be|0,1729360)|0)==0){break}if((a8(be|0,1728944)|0)==0){break}if((a8(be|0,1728768)|0)==0){break}if((a8(be|0,1727936)|0)==0){break}if((a8(be|0,1727416)|0)==0){break}bq=c[66006]|0;do{if((bq|0)<0){bk=c[bd+108>>2]|0;if((bk|0)==0){break L2264}bo=bd+112|0;bp=c[bh>>2]|0;bu=+(c[bp+40>>2]|0)/+(c[bp+44>>2]|0);if(+(c[bd+104>>2]|0)/+(bk|0)*+(c[bo>>2]|0)<=bu){aS=1680;break}if(bu<.002|(bq|0)==0){bx=bo}else{break L2264}}else{aS=1680}}while(0);if((aS|0)==1680){aS=0;if((bq|0)!=0){break}bx=bd+112|0}bn=bd+104|0;bo=c[bn>>2]|0;bk=c[bn+4>>2]|0;c[bl>>2]=bo;c[bl+4>>2]=bk;c[bj>>2]=ac(bo,c[bx>>2]|0)|0}}while(0);do{if((c[bg>>2]|0)==1684237684){be=bd+104|0;bm=c[be>>2]|0;bb=c[bd+108>>2]|0;if(!((bm|0)<(bb|0)&(bm|0)>0)){break}a3=N2(bm,(bm|0)<0|0?-1:0,121,0)|0;bm=G;bo=(bb|0)<0|0?-1:0;if(!((bm|0)>(bo|0)|(bm|0)==(bo|0)&a3>>>0>bb>>>0)){break}bb=be;be=c[bb+4>>2]|0;c[bl>>2]=c[bb>>2];c[bl+4>>2]=be}}while(0);bl=a_+76|0;bg=c[bl>>2]|0;do{if((bg|0)==0){be=a4+112|0;bb=bl;a3=c[be>>2]|0;bo=c[be+4>>2]|0;c[bb>>2]=a3;c[bb+4>>2]=bo;bb=a3;if((bb|0)==0){a3=bj|0;by=c[a3>>2]|0;bz=c[a6+108>>2]|0;bA=a3;break}else{bB=bb;bC=bo;aS=1692;break}}else{bo=a_+80|0;bB=bg;bC=d[bo]|d[bo+1|0]<<8|d[bo+2|0]<<16|d[bo+3|0]<<24|0;aS=1692}}while(0);if((aS|0)==1692){aS=0;c[a6+104>>2]=bC;c[a6+108>>2]=bB;by=bC;bz=bB;bA=bj|0}bg=a6+108|0;bl=by;bo=(by|0)<0|0?-1:0;bb=bz;a3=(bz|0)<0|0?-1:0;be=2147483647;bm=0;BX(bA,bg,bl,bo,bb,a3,be,bm)|0;switch(c[bf>>2]|0){case 2:case 4:{bD=aY;break L2246;break};case 1:{if((c[86702]|0)!=256){AN(0,8,1726640,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;eW(1)}bm=bd+448|0;be=c[bm+4>>2]|0;a3=a6+448|0;c[a3>>2]=c[bm>>2];c[a3+4>>2]=be;c[a6+412>>2]=c[bd+412>>2];c[a6+416>>2]=c[bd+416>>2];c[a6+424>>2]=c[bd+424>>2];c[a6+464>>2]=c[bd+464>>2];be=c[bd+432>>2]|0;a3=a6+432|0;c[a3>>2]=be;do{if((be|0)==1|(be|0)==1152|(be|0)==576){bm=c[a9>>2]|0;if((bm|0)!=86017){bE=bm;break}c[a3>>2]=0;bD=aY;break L2246}else{bE=c[a9>>2]|0}}while(0);if((bE|0)!=86019){bD=aY;break L2246}c[a3>>2]=0;bD=aY;break L2246;break};case 0:{c[a6+140>>2]=c[bd+140>>2];a9=a6+120|0;c[a9>>2]=c[bd+120>>2];be=c[bd+124>>2]|0;c[a6+124>>2]=be;c[a6+176>>2]=c[bd+176>>2];bf=a_+92|0;do{if((c[bf>>2]|0)==0){bj=(c[bh>>2]|0)+80|0;if((c[bj>>2]|0)==0){bm=bd+224|0;bF=c[bm+4>>2]|0;bG=c[bm>>2]|0;break}else{bm=bj;bF=c[bm+4>>2]|0;bG=c[bm>>2]|0;break}}else{c[aN>>2]=be;c[aM>>2]=c[a9>>2];BY(aC,bf,ar);bm=c[as>>2]|0;bj=c[as+4>>2]|0;AN(0,24,1725936,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;bF=bj;bG=bm}}while(0);bf=(c[a5>>2]|0)+80|0;a9=a6+224|0;c[a9>>2]=bG;c[a9+4>>2]=bF;a9=bf;c[a9>>2]=bG;c[a9+4>>2]=bF;a9=(c[bh>>2]|0)+92|0;bf=(c[a5>>2]|0)+92|0;be=c[a9+4>>2]|0;c[bf>>2]=c[a9>>2];c[bf+4>>2]=be;bD=aY;break L2246;break};case 3:{c[a6+120>>2]=c[bd+120>>2];c[a6+124>>2]=c[bd+124>>2];bD=aY;break L2246;break};default:{aS=1709;break L2241}}}be=a_+60|0;if((c[be>>2]|0)==0){bf=o8(c[a6+48>>2]|0)|0;c[be>>2]=bf;if((bf|0)==0){aS=1712;break L2241}}if(a7){bf=a4+12|0;c[bf>>2]=(c[bf>>2]|0)+1}c[a_+16>>2]=1;bf=a_+300|0;a9=a6+8|0;do{if((c[bf>>2]|0)==0){a3=c[a9>>2]|0;if(a3>>>0>=2){bH=a3;break}if((ey(ew(a4,a_)|0)|0)==0){aS=1719;break}AN(0,8,1725264,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;eW(1);aS=1719}else{aS=1719}}while(0);do{if((aS|0)==1719){aS=0;bh=c[a9>>2]|0;if((bh|0)!=0){bH=bh;break}bh=c[bf>>2]|0;do{if((bh|0)!=0){a3=a_+76|0;if((c[a3>>2]|0)!=0){break}gs(aD,c[bh>>2]|0);bm=a3;a3=c[at+4>>2]|0;c[bm>>2]=c[at>>2];c[bm+4>>2]=a3}}while(0);do{if(a7){bh=a_+76|0;if((c[bh>>2]|0)!=0){break}bq=a4+112|0;a3=bh;bh=c[bq>>2]|0;bm=c[bq+4>>2]|0;c[a3>>2]=bh;c[a3+4>>2]=bm;if((bh|0)!=0){break}bh=(c[a4+4>>2]|0)+412|0;if((c[bh>>2]|0)==0){c[a_+76>>2]=25;c[a_+80>>2]=1;break}else{bm=bh;bh=c[bm+4>>2]|0;c[a3>>2]=c[bm>>2];c[a3+4>>2]=bh;break}}}while(0);bh=c[be>>2]|0;do{if((bh|0)!=0){a3=c[bh+20>>2]|0;if((a3|0)==0){break}if((c[a_+84>>2]|0)!=0){break}bm=a_+76|0;bq=B$(bm,a3)|0;a3=(c[(c[be>>2]|0)+20>>2]|0)+(bq<<3)|0;bq=bm;bm=c[a3+4>>2]|0;c[bq>>2]=c[a3>>2];c[bq+4>>2]=bm}}while(0);bH=c[a9>>2]|0}}while(0);do{if((bH|0)==3){c[a6+104>>2]=1;c[a6+108>>2]=1e3;a9=a6+120|0;if((c[a9>>2]|0)!=0){bI=aY;break}c[a9>>2]=c[(c[(c[(c[(c[551808]|0)+(c[a2>>2]<<2)>>2]|0)+4>>2]|0)+8>>2]|0)+120>>2];c[a6+124>>2]=c[(c[(c[(c[(c[551808]|0)+(c[a2>>2]<<2)>>2]|0)+4>>2]|0)+8>>2]|0)+124>>2];bI=aY}else if((bH|0)==1){c[a6+420>>2]=c[(c[c[(c[c[bf>>2]>>2]|0)+16>>2]>>2]|0)+52>>2];a9=a6+412|0;c[a9>>2]=c[(c[c[(c[c[bf>>2]>>2]|0)+16>>2]>>2]|0)+48>>2];a7=(c[c[(c[c[bf>>2]>>2]|0)+16>>2]>>2]|0)+40|0;bh=c[a7+4>>2]|0;bm=a6+448|0;c[bm>>2]=c[a7>>2];c[bm+4>>2]=bh;c[a6+416>>2]=fO(c[c[(c[c[bf>>2]>>2]|0)+16>>2]>>2]|0)|0;bh=c[a9>>2]|0;c[a6+104>>2]=1;c[a6+108>>2]=bh;bI=aY}else if((bH|0)==0){bh=a_+76|0;a9=d[bh]|d[bh+1|0]<<8|d[bh+2|0]<<16|d[bh+3|0]<<24|0;bh=a_+80|0;bm=d[bh]|d[bh+1|0]<<8|d[bh+2|0]<<16|d[bh+3|0]<<24|0;c[a6+104>>2]=bm;c[a6+108>>2]=a9;bh=c[bf>>2]|0;a7=a6+104|0;do{if((bh|0)==0){bJ=bm;bK=a9}else{if(!((bm|0)==0|(a9|0)==0)){bJ=bm;bK=a9;break}bq=(c[c[(c[bh>>2]|0)+16>>2]>>2]|0)+56|0;a3=a7;bj=c[bq>>2]|0;bb=c[bq+4>>2]|0;c[a3>>2]=bj;c[a3+4>>2]=bb;bJ=bj;bK=bb}}while(0);bh=c[8408]|0;do{if(+(bJ|0)/+(bK|0)<.001&(bh|0)!=0){if((bh|0)==(-1|0)){if((c[(c[a0+8>>2]|0)+28>>2]&1024|0)!=0){break}}else if((bh|0)!=1){break}AN(a0,24,1724816,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU}}while(0);bh=a_+104|0;if((c[bh>>2]|0)>0){a9=a_+100|0;bm=0;do{bb=(c[a9>>2]|0)+(bm<<3)|0;bj=c[bb>>2]|0;a3=c[bb+4>>2]|0;c[aL>>2]=1;c[aK>>2]=1e6;bb=AY(bj,a3,au,a7)|0;a3=(c[a9>>2]|0)+(bm<<3)|0;c[a3>>2]=bb;c[a3+4>>2]=G;bm=bm+1|0;}while((bm|0)<(c[bh>>2]|0))}bm=a6+120|0;c[bm>>2]=c[(c[c[(c[c[bf>>2]>>2]|0)+16>>2]>>2]|0)+20>>2];a9=c[(c[c[(c[c[bf>>2]>>2]|0)+16>>2]>>2]|0)+24>>2]|0;a7=a6+124|0;c[a7>>2]=a9;a3=(c[a5>>2]|0)+80|0;bb=a_+92|0;if((c[bb>>2]|0)==0){bj=(c[c[(c[c[bf>>2]>>2]|0)+16>>2]>>2]|0)+28|0;bq=a3;bo=c[bj>>2]|0;bl=c[bj+4>>2]|0;c[bq>>2]=bo;c[bq+4>>2]=bl;bL=bl;bM=bo}else{c[aF>>2]=a9;c[aJ>>2]=c[bm>>2];BY(aE,bb,av);bb=a3;a3=c[aw>>2]|0;a9=c[aw+4>>2]|0;c[bb>>2]=a3;c[bb+4>>2]=a9;bL=a9;bM=a3}a3=a6+224|0;c[a3>>2]=bM;c[a3+4>>2]=bL;a3=a6+140|0;do{if((ba(c[c[be>>2]>>2]|0,1724552,7)|0)==0){if((c[a3>>2]|0)!=-1){break}a9=c[(c[c[(c[c[bf>>2]>>2]|0)+16>>2]>>2]|0)+52>>2]|0;if((a9|0)==0){break}bb=BL(a9)|0;AN(0,32,1724232,(aU=i,i=i+8|0,c[aU>>2]=bb,aU)|0);i=aU}}while(0);bb=c[(c[c[(c[c[bf>>2]>>2]|0)+16>>2]>>2]|0)+52>>2]|0;c[a3>>2]=bb;do{if((bd|0)==0){aS=1757}else{if((c[bm>>2]|0)!=(c[bd+120>>2]|0)){aS=1757;break}if((c[a7>>2]|0)!=(c[bd+124>>2]|0)){aS=1757;break}if((bb|0)!=(c[bd+140>>2]|0)){aS=1757}}}while(0);if((aS|0)==1757){aS=0;c[a6+796>>2]=c[556288]}bN=a_+112|0;bb=c[bN>>2]|0;if((bb|0)==0){bI=aY;break}if((ba(bb|0,1723712,5)|0)==0){bO=zQ(a_+116|0,bb+5|0,234288,0,0,0,0,0,0)|0;if((bO|0)<0){aS=1761;break L2241}NM(a_+120|0,0,16);h[a_+136>>3]=r;h[a_+144>>3]=r;bI=bO;break}a7=c[(c[a5>>2]|0)+8>>2]|0;bm=bb;a3=1;while(1){a9=a[bm]|0;if((a9<<24>>24|0)==44){bP=a3+1|0}else if((a9<<24>>24|0)==0){break}else{bP=a3}bm=bm+1|0;a3=bP}bm=A1(a3<<3)|0;a9=bm;if((bm|0)==0){AN(0,8,1712584,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;eW(1)}if((a3|0)>0){bm=a7+104|0;bo=0;bl=a3;bq=0;bj=a9;bg=bb;while(1){bk=bi(bg|0,44)|0;if((bk|0)==0){bQ=0}else{a[bk]=0;bQ=bk+1|0}do{if((NO(bg|0,1712264,8)|0)==0){bk=c[c[(c[550878]|0)+(c[a$>>2]<<2)>>2]>>2]|0;bn=bk+1132|0;bp=c[bn>>2]|0;if(bp>>>0>(2147483647-bl|0)>>>0){bR=bj;bS=bl;aS=1776}else{bU=bl-1+bp|0;bp=A3(bj,bU,8)|0;bW=bp;if((bp|0)==0){bR=bW;bS=bU;aS=1776}else{bX=bW;bZ=bU}}if((aS|0)==1776){aS=0;AN(0,8,1712584,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;eW(1);bX=bR;bZ=bS}bU=bg+8|0;if((a[bU]|0)==0){b_=0;b$=0}else{bW=eY(1712008,bU,1)|0;b_=G;b$=bW}c[aG>>2]=1;c[aO>>2]=1e6;bW=AY(b$,b_,al,bm)|0;bU=G;if((c[bn>>2]|0)==0){b0=bX;b1=bq;b2=bZ;break}bp=bk+1136|0;bk=bq;b3=0;while(1){b4=c[(c[bp>>2]|0)+(b3<<2)>>2]|0;b7=b4+16|0;b8=AY(c[b7>>2]|0,c[b7+4>>2]|0,b4+4|0,bm)|0;b4=NU(b8,G,bW,bU)|0;b8=bk+1|0;b7=bX+(bk<<3)|0;c[b7>>2]=b4;c[b7+4>>2]=G;b7=b3+1|0;if(b7>>>0<(c[bn>>2]|0)>>>0){bk=b8;b3=b7}else{b0=bX;b1=b8;b2=bZ;break}}}else{b3=eY(1712008,bg,1)|0;c[aI>>2]=1;c[aH>>2]=1e6;bk=AY(b3,G,am,bm)|0;b3=bj+(bq<<3)|0;c[b3>>2]=bk;c[b3+4>>2]=G;b0=bj;b1=bq+1|0;b2=bl}}while(0);b3=bo+1|0;if((b3|0)<(a3|0)){bo=b3;bl=b2;bq=b1;bj=b0;bg=bQ}else{b9=b2;ca=b1;cb=b0;break}}}else{b9=a3;ca=0;cb=a9}if((ca|0)!=(b9|0)){aS=1785;break L2241}b6(cb|0,ca|0,8,34);c[bh>>2]=ca;c[a_+100>>2]=cb;bI=aY}else{aS=1789;break L2241}}while(0);bf=a6+88|0;if((c[bf>>2]&1536|0)==0){bD=bI;break}bg=c[a_+292>>2]|0;bj=(bg|0)==0?1722552:bg;bc(aA|0,1024,1722936,(aU=i,i=i+16|0,c[aU>>2]=bj,c[aU+8>>2]=aX,aU)|0)|0;i=aU;if((a8(c[c[be>>2]>>2]|0,1724552)|0)==0){bj=a_+320|0;zK(bj,1721768,aA,16)|0;bD=bI;break}bj=c[bf>>2]|0;if((bj&1024|0)==0){cc=bj}else{if((fy(aA,ax,ay)|0)<0){AN(0,8,1721480,(aU=i,i=i+8|0,c[aU>>2]=aA,aU)|0);i=aU;eW(1)}c[a6+672>>2]=c[ax>>2];cc=c[bf>>2]|0}if((cc&512|0)==0){bD=bI;break}bf=bV(aA|0,1721336)|0;if((bf|0)==0){bj=ci(c[(ch()|0)>>2]|0)|0;AN(0,8,1721024,(aU=i,i=i+16|0,c[aU>>2]=aA,c[aU+8>>2]=bj,aU)|0);i=aU;eW(1)}c[a_+296>>2]=bf;bD=bI}else{bD=aY}}while(0);a0=aX+1|0;cd=c[550910]|0;if((a0|0)<(cd|0)){aX=a0;aY=bD}else{aS=1635;break}}if((aS|0)==1789){cw();return 0}else if((aS|0)==1785){AN(0,0,1616448,(aU=i,i=i+24|0,c[aU>>2]=1711672,c[aU+8>>2]=1615896,c[aU+16>>2]=2061,aU)|0);i=aU;cw();return 0}else if((aS|0)==1635){if((cd|0)>0){ce=0;cf=bD}else{cg=bD;aS=1802;break}L2464:while(1){cj=c[(c[550876]|0)+(ce<<2)>>2]|0;do{if((c[cj+16>>2]|0)==0){aY=c[(c[cj+12>>2]|0)+8>>2]|0;aX=cj+320|0;Bv(aY,aX)|0;ck=cf}else{aX=cj+60|0;aY=c[aX>>2]|0;aH=cj+8|0;aI=c[aH>>2]|0;do{if((aI|0)>-1){aO=c[(c[551808]|0)+(aI<<2)>>2]|0;if((aO|0)==0){break}aG=c[(c[aO+4>>2]|0)+8>>2]|0;if((aG|0)==0){break}aO=aG+860|0;if((c[aO>>2]|0)==0){break}aJ=aG+864|0;aG=A7((c[aJ>>2]|0)+1|0)|0;aF=cj+12|0;c[(c[(c[aF>>2]|0)+8>>2]|0)+860>>2]=aG;aG=c[(c[(c[aF>>2]|0)+8>>2]|0)+860>>2]|0;if((aG|0)==0){cl=-12;cm=1;aS=1856;break L2239}aK=c[aO>>2]|0;aO=c[aJ>>2]|0;NN(aG|0,aK|0,aO)|0;c[(c[(c[aF>>2]|0)+8>>2]|0)+864>>2]=c[aJ>>2]}}while(0);aI=cj+320|0;if((zJ(c[aI>>2]|0,1720848,0,0)|0)==0){zK(aI,1720848,1720616,0)|0}a6=cj+12|0;cn=o5(c[(c[a6>>2]|0)+8>>2]|0,aY,aI)|0;if((cn|0)<0){break L2464}be=c[aX>>2]|0;do{if((c[be+8>>2]|0)==1){if((c[be+16>>2]&65536|0)!=0){break}gr(c[c[cj+300>>2]>>2]|0,c[(c[(c[a6>>2]|0)+8>>2]|0)+424>>2]|0)}}while(0);be=zJ(c[aI>>2]|0,3119008,0,2)|0;if((be|0)!=0){AN(0,8,1623064,(aU=i,i=i+8|0,c[aU>>2]=c[be>>2],aU)|0);i=aU;eW(1)}be=c[(c[a6>>2]|0)+8>>2]|0;aX=c[be+72>>2]|0;if((aX|0)!=0&(aX|0)<1e3){AN(0,24,1719424,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;co=c[(c[a6>>2]|0)+8>>2]|0}else{co=be}be=c[co+100>>2]|0;aX=NU(c[727194]|0,c[727195]|0,be,(be|0)<0|0?-1:0)|0;c[727194]=aX;c[727195]=G;if((c[(c[(c[a6>>2]|0)+8>>2]|0)+316>>2]|0)==0){ck=cn;break}aX=(c[(c[(c[(c[551808]|0)+(c[aH>>2]<<2)>>2]|0)+4>>2]|0)+8>>2]|0)+688|0;c[aX>>2]=c[aX>>2]|32;ck=cn}}while(0);aX=ce+1|0;if((aX|0)<(c[550910]|0)){ce=aX;cf=ck}else{cg=ck;aS=1802;break L2239}}if((cn|0)==-733130664){eW(1)}aX=c[cj+4>>2]|0;bc(az|0,1024,1720064,(aU=i,i=i+16|0,c[aU>>2]=c[cj>>2],c[aU+8>>2]=aX,aU)|0)|0;i=aU;cl=cn;cm=1;aS=1856;break}else if((aS|0)==1709){cw();return 0}else if((aS|0)==1712){aX=pn(c[(c[(c[a5>>2]|0)+8>>2]|0)+48>>2]|0)|0;be=c[a$>>2]|0;aY=c[a_+4>>2]|0;bc(az|0,1024,1725504,(aU=i,i=i+24|0,c[aU>>2]=aX,c[aU+8>>2]=be,c[aU+16>>2]=aY,aU)|0)|0;i=aU;cl=-22;cm=1;aS=1856;break}else if((aS|0)==1761){AN(0,16,1723184,(aU=i,i=i+8|0,c[aU>>2]=(c[bN>>2]|0)+5,aU)|0);i=aU;aV=bO;break L2234}else if((aS|0)==2245){AN(0,0,1616448,(aU=i,i=i+24|0,c[aU>>2]=1731152,c[aU+8>>2]=1615896,c[aU+16>>2]=2139,aU)|0);i=aU;cw();return 0}else if((aS|0)==2246){AN(0,0,1616448,(aU=i,i=i+24|0,c[aU>>2]=1731152,c[aU+8>>2]=1615896,c[aU+16>>2]=2139,aU)|0);i=aU;cw();return 0}}else{cg=aP;aS=1802}}while(0);L2501:do{if((aS|0)==1802){L2503:do{if((c[550914]|0)>0){aY=an|0;be=0;L2505:while(1){cp=c[(c[551808]|0)+(be<<2)>>2]|0;do{if((c[cp+12>>2]|0)!=0){aX=c[cp+16>>2]|0;cq=cp+4|0;cr=c[(c[cq>>2]|0)+8>>2]|0;if((aX|0)==0){aS=1831;break L2505}aJ=cr;aF=1;aO=0;aK=1713200;Bh(aJ,aK,aF,aO,0)|0;aO=cp+108|0;if((zJ(c[aO>>2]|0,1720848,0,0)|0)==0){zK(aO,1720848,1720616,0)|0}cs=o5(c[(c[cq>>2]|0)+8>>2]|0,aX,aO)|0;if((cs|0)<0){aS=1835;break L2505}aX=zJ(c[aO>>2]|0,3119008,0,2)|0;if((aX|0)==0){break}AN(0,8,1623064,(aU=i,i=i+8|0,c[aU>>2]=c[aX>>2],aU)|0);i=aU;eW(1)}}while(0);aH=cp+56|0;c[aH>>2]=0;c[aH+4>>2]=-2147483648;aH=cp+40|0;c[aH>>2]=0;c[aH+4>>2]=-2147483648;c[cp+96>>2]=1;aH=be+1|0;if((aH|0)<(c[550914]|0)){be=aH}else{cu=0;break L2503}}if((aS|0)==1835){if((cs|0)==-733130664){eW(1)}zN(cs,aY,128)|0;be=c[cp>>2]|0;aH=c[c[cq>>2]>>2]|0;bc(az|0,1024,1713032,(aU=i,i=i+24|0,c[aU>>2]=be,c[aU+8>>2]=aH,c[aU+16>>2]=aY,aU)|0)|0;i=aU;cv=cs}else if((aS|0)==1831){aH=pn(c[cr+48>>2]|0)|0;be=c[cp>>2]|0;a6=c[c[cq>>2]>>2]|0;bc(az|0,1024,1713344,(aU=i,i=i+24|0,c[aU>>2]=aH,c[aU+8>>2]=be,c[aU+16>>2]=a6,aU)|0)|0;i=aU;cv=-22}if((c[550910]|0)>0){cy=0}else{cl=cv;cm=1;aS=1856;break L2501}while(1){pk(c[(c[(c[(c[550876]|0)+(cy<<2)>>2]|0)+12>>2]|0)+8>>2]|0)|0;a6=cy+1|0;if((a6|0)<(c[550910]|0)){cy=a6}else{cl=cv;cm=1;aS=1856;break L2501}}}else{cu=cg}}while(0);aY=c[550916]|0;if((aY|0)>0){a6=0;be=aY;while(1){aY=c[(c[551810]|0)+(a6<<2)>>2]|0;aH=aY|0;aI=c[aH>>2]|0;if((c[aI+1104>>2]|0)==0){cz=be}else{aX=aY+12|0;aY=0;aO=aI;do{aI=c[(c[aO+1108>>2]|0)+(aY<<2)>>2]|0;aF=c[aI+16>>2]|0;aK=aI+12|0;aJ=c[551808]|0;aG=0;while(1){if(aG>>>0>=aF>>>0){cA=48;break}if((c[(c[aJ+((c[(c[aK>>2]|0)+(aG<<2)>>2]|0)+(c[aX>>2]|0)<<2)>>2]|0)+8>>2]|0)==0){cA=0;break}else{aG=aG+1|0}}c[aI+8>>2]=cA;aY=aY+1|0;aO=c[aH>>2]|0;}while(aY>>>0<(c[aO+1104>>2]|0)>>>0);cz=c[550916]|0}aO=a6+1|0;if((aO|0)<(cz|0)){a6=aO;be=cz}else{break}}}if((c[550912]|0)<=0){cB=1;cC=cu;break}be=172928;a6=c[be>>2]|0;aO=c[be+4>>2]|0;be=1;aY=0;while(1){aH=(c[550878]|0)+(aY<<2)|0;aX=c[c[aH>>2]>>2]|0;aG=aX+1160|0;c[aG>>2]=a6;c[aG+4>>2]=aO;cD=k7(aX,(c[aH>>2]|0)+4|0)|0;if((cD|0)<0){break}aH=(a8(c[c[aX+8>>2]>>2]|0,1718424)|0)==0?be:0;aX=aY+1|0;aG=c[550912]|0;if((aX|0)<(aG|0)){be=aH;aY=aX}else{cE=cD;cF=aH;cG=aG;aS=1857;break L2501}}zN(cD,aB,128)|0;bc(az|0,1024,1719024,(aU=i,i=i+16|0,c[aU>>2]=aY,c[aU+8>>2]=aB,aU)|0)|0;i=aU;cl=-22;cm=be;aS=1856}}while(0);if((aS|0)==1856){cE=cl;cF=cm;cG=c[550912]|0;aS=1857}do{if((aS|0)==1857){if((cG|0)>0){cH=0}else{cB=cF;cC=cE;break}while(1){aO=c[c[(c[550878]|0)+(cH<<2)>>2]>>2]|0;mN(aO,cH,aO+32|0,1);aO=cH+1|0;if((aO|0)<(c[550912]|0)){cH=aO}else{cB=cF;cC=cE;break}}}}while(0);AN(0,32,1717688,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;be=c[550914]|0;if((be|0)>0){aY=0;aO=be;while(1){be=c[(c[551808]|0)+(aY<<2)>>2]|0;a6=be+248|0;aG=c[a6>>2]|0;if((aG|0)>0){aH=be+244|0;aX=be|0;aK=be+4|0;aJ=be+16|0;be=0;aF=aG;while(1){aG=c[(c[aH>>2]|0)+(be<<2)>>2]|0;if((c[(c[aG+8>>2]|0)+4>>2]|0)==0){cI=aF}else{aL=c[c[aK>>2]>>2]|0;aM=c[aJ>>2]|0;if((aM|0)==0){cJ=1717224}else{cJ=c[aM>>2]|0}aM=c[aG+12>>2]|0;AN(0,32,1717408,(aU=i,i=i+32|0,c[aU>>2]=c[aX>>2],c[aU+8>>2]=aL,c[aU+16>>2]=cJ,c[aU+24>>2]=aM,aU)|0);i=aU;if((c[550922]|0)>1){AN(0,32,1717040,(aU=i,i=i+8|0,c[aU>>2]=c[c[(c[(c[aH>>2]|0)+(be<<2)>>2]|0)+8>>2]>>2],aU)|0);i=aU}AN(0,32,1618104,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;cI=c[a6>>2]|0}aM=be+1|0;if((aM|0)<(cI|0)){be=aM;aF=cI}else{break}}cK=c[550914]|0}else{cK=aO}aF=aY+1|0;if((aF|0)<(cK|0)){aY=aF;aO=cK}else{break}}}if((c[550910]|0)>0){aO=0;do{aY=c[(c[550876]|0)+(aO<<2)>>2]|0;aF=c[aY+348>>2]|0;L2574:do{if((aF|0)==0){be=aY+300|0;a6=c[be>>2]|0;do{if((a6|0)!=0){if((c[(c[a6+8>>2]|0)+4>>2]|0)==0){break}AN(0,32,1716624,(aU=i,i=i+8|0,c[aU>>2]=c[a6+12>>2],aU)|0);i=aU;if((c[550922]|0)>1){AN(0,32,1717040,(aU=i,i=i+8|0,c[aU>>2]=c[c[(c[be>>2]|0)+8>>2]>>2],aU)|0);i=aU}aH=c[aY+4>>2]|0;aX=c[aY+60>>2]|0;if((aX|0)==0){cL=1717224}else{cL=c[aX>>2]|0}AN(0,32,1716192,(aU=i,i=i+24|0,c[aU>>2]=c[aY>>2],c[aU+8>>2]=aH,c[aU+16>>2]=cL,aU)|0);i=aU;break L2574}}while(0);be=aY+8|0;a6=c[(c[551808]|0)+(c[be>>2]<<2)>>2]|0;aH=c[c[a6+4>>2]>>2]|0;aX=c[aY>>2]|0;aJ=c[aY+4>>2]|0;AN(0,32,1715768,(aU=i,i=i+32|0,c[aU>>2]=c[a6>>2],c[aU+8>>2]=aH,c[aU+16>>2]=aX,c[aU+24>>2]=aJ,aU)|0);i=aU;aJ=c[aY+24>>2]|0;if((aJ|0)!=(c[(c[551808]|0)+(c[be>>2]<<2)>>2]|0)){aX=c[c[aJ+4>>2]>>2]|0;AN(0,32,1714520,(aU=i,i=i+16|0,c[aU>>2]=c[aJ>>2],c[aU+8>>2]=aX,aU)|0);i=aU}if((c[aY+344>>2]|0)==0){aX=c[(c[(c[551808]|0)+(c[be>>2]<<2)>>2]|0)+16>>2]|0;if((aX|0)==0){cN=1717224}else{cN=c[aX>>2]|0}aX=c[aY+60>>2]|0;if((aX|0)==0){cO=1717224}else{cO=c[aX>>2]|0}AN(0,32,1713832,(aU=i,i=i+16|0,c[aU>>2]=cN,c[aU+8>>2]=cO,aU)|0);i=aU}else{AN(0,32,1714208,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU}AN(0,32,1618104,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU}else{aX=c[aY>>2]|0;be=c[aY+4>>2]|0;AN(0,32,1716776,(aU=i,i=i+24|0,c[aU>>2]=aF,c[aU+8>>2]=aX,c[aU+16>>2]=be,aU)|0);i=aU}}while(0);aO=aO+1|0;}while((aO|0)<(c[550910]|0))}if((cC|0)!=0){AN(0,16,1713648,(aU=i,i=i+8|0,c[aU>>2]=az,aU)|0);i=aU;aV=cC;break}if((cB|0)==0){aV=0;break}aO=ao|0;aF=ap;aY=A1(c[550912]<<2)|0;c[ap>>2]=aY;if((aY|0)==0){eW(1)}aY=c[550912]|0;if((aY|0)>0){be=0;while(1){c[(c[ap>>2]|0)+(be<<2)>>2]=c[c[(c[550878]|0)+(be<<2)>>2]>>2];aX=be+1|0;aJ=c[550912]|0;if((aX|0)<(aJ|0)){be=aX}else{cP=aJ;break}}}else{cP=aY}l6(c[ap>>2]|0,cP,aO,16384)|0;bT(1713544,(aU=i,i=i+8|0,c[aU>>2]=aO,aU)|0)|0;i=aU;aT(c[p>>2]|0)|0;A6(aF);aV=0}}while(0);do{if((aV|0)<0){cQ=aV}else{if((c[18294]|0)!=0){AN(0,32,1721688,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU}cP=Ch()|0;ap=G;L2619:do{if((c[541004]|0)==0){ao=X|0;cB=X+4|0;cC=W|0;az=W+4|0;cO=n|0;cN=q|0;cL=s|0;cK=t|0;cI=u|0;cJ=v|0;cE=w|0;cF=x|0;cH=y|0;cG=z|0;cm=O|0;cl=P|0;aB=S|0;cD=T|0;cu=U|0;cz=V|0;cA=an|0;cg=l|0;cv=l+4|0;cy=m+32|0;cq=A|0;cp=A+4|0;cr=B|0;cs=B+4|0;aP=C|0;bO=C+4|0;bN=m+16|0;a_=m+8|0;a$=J|0;a5=J+4|0;cn=M|0;cj=M+4|0;ck=K|0;cf=K+4|0;ce=L|0;co=L+4|0;bD=N|0;cd=N+4|0;bI=F|0;aA=F+4|0;cc=H|0;ax=H+4|0;ay=I|0;cb=I+4|0;ca=E|0;b9=E+4|0;b0=D|0;b1=D+4|0;b2=k|0;bQ=m+64|0;am=j|0;bZ=j+4|0;bX=ae|0;al=af|0;b_=ag|0;b$=ai|0;bS=ad;bR=ad|0;bP=aa|0;bd=aa+4|0;bL=c[o>>2]|0;bM=_;aw=_|0;av=Z|0;aE=Z+4|0;L2621:while(1){au=Ch()|0;bK=G;L2623:do{if((c[18294]|0)!=0){if((c[541006]|0)!=0){break L2619}bJ=NV(au,bK,c[745340]|0,c[745341]|0)|0;bH=G;a4=0;if((bH|0)<(a4|0)|(bH|0)==(a4|0)&bJ>>>0<1e5>>>0|a[85008]){break}NM(bS|0,0,128);c[bR>>2]=1;c[bP>>2]=0;c[bd>>2]=0;if((bY(1,ad|0,0,0,aa|0)|0)<=0){c[745340]=au;c[745341]=bK;break}bJ=a1(0,$|0,1)|0;if((bJ|0)==1){cR=d[$]|0}else{cR=bJ}c[745340]=au;c[745341]=bK;switch(cR|0){case 113:{break L2619;break};case 43:{AP((AO()|0)+10|0);break L2623;break};case 45:{AP((AO()|0)-10|0);break L2623;break};case 115:{c[541008]=c[541008]^1;break L2623;break};case 104:{do{if((c[744100]|0)==0){if((c[744098]|0)==0){c[744098]=1;break}else{c[744100]=1;break}}else{c[744098]=0;c[744100]=0}}while(0);AP(48);break};default:{}}aI=(cR|0)==99;L2646:do{if((cR|0)==99|(cR|0)==67){NM(b$|0,0,256);aW(1736064,55,1,bL|0)|0;bJ=0;L2648:while(1){while(1){NM(bM|0,0,128);c[aw>>2]=1;c[av>>2]=0;c[aE>>2]=0;if((bY(1,_|0,0,0,Z|0)|0)<=0){continue}a4=a1(0,Y|0,1)|0;if((a4|0)==1){cS=d[Y]|0}else{cS=a4}if((cS|0)==10|(cS|0)==13){cT=bJ;aS=1937;break L2648}if((cS|0)>0){break}}cU=bJ+1|0;a[ae+bJ|0]=cS&255;if(cU>>>0<4095){bJ=cU}else{aS=1926;break}}do{if((aS|0)==1926){aS=0;NM(bM|0,0,128);c[aw>>2]=1;c[av>>2]=0;c[aE>>2]=0;if((bY(1,_|0,0,0,Z|0)|0)<=0){a[ae+cU|0]=0;cV=0;break}bJ=a1(0,Y|0,1)|0;if((bJ|0)==1){cW=d[Y]|0}else{cW=bJ}if((cW|0)==10|(cW|0)==13){cT=cU;aS=1937;break}a[ae+cU|0]=0;if((cW|0)>0){aS=1939}else{cV=0}}}while(0);if((aS|0)==1937){aS=0;a[ae+cT|0]=0;aS=1939}do{if((aS|0)==1939){aS=0;bJ=ct(bX|0,1735184,(aU=i,i=i+32|0,c[aU>>2]=al,c[aU+8>>2]=aj,c[aU+16>>2]=b_,c[aU+24>>2]=b$,aU)|0)|0;i=aU;if((bJ|0)<=2){cV=bJ;break}bu=+h[aj>>3];AN(0,48,1734808,(aU=i,i=i+32|0,c[aU>>2]=al,h[aU+8>>3]=bu,c[aU+16>>2]=b_,c[aU+24>>2]=b$,aU)|0);i=aU;if((c[550922]|0)<=0){break L2646}bJ=aI&1;bh=0;while(1){a9=c[(c[(c[556308]|0)+(bh<<2)>>2]|0)+8>>2]|0;do{if((a9|0)!=0){bu=+h[aj>>3];if(bu<0.0){a3=gk(a9,al,b_,b$,bX,4096,bJ)|0;cx(bL|0,1734544,(aU=i,i=i+24|0,c[aU>>2]=bh,c[aU+8>>2]=a3,c[aU+16>>2]=bX,aU)|0)|0;i=aU;break}else{gl(a9,al,b_,b$,0,bu)|0;break}}}while(0);bh=bh+1|0;if((bh|0)>=(c[550922]|0)){break L2646}}}}while(0);AN(0,16,1734224,(aU=i,i=i+16|0,c[aU>>2]=cV,c[aU+8>>2]=bX,aU)|0);i=aU}}while(0);aI=(cR|0)==68;if((cR|0)==100|(cR|0)==68){c[ak>>2]=0;do{if(aI){bh=c[(c[(c[(c[c[551808]>>2]|0)+4>>2]|0)+8>>2]|0)+688>>2]<<1;if((bh|0)==0){c[ak>>2]=1;break}c[ak>>2]=bh;if((bh&24640|0)==0){break}else{cX=bh}do{cX=cX<<1;}while((cX&24640|0)!=0);c[ak>>2]=cX}else{bh=b5(1733832,(aU=i,i=i+8|0,c[aU>>2]=ak,aU)|0)|0;i=aU;if((bh|0)==1){break}aW(1733584,26,1,bL|0)|0}}while(0);if((c[550914]|0)>0){aI=0;do{c[(c[(c[(c[(c[551808]|0)+(aI<<2)>>2]|0)+4>>2]|0)+8>>2]|0)+688>>2]=c[ak>>2];aI=aI+1|0;}while((aI|0)<(c[550914]|0))}if((c[550910]|0)>0){aI=0;do{c[(c[(c[(c[(c[550876]|0)+(aI<<2)>>2]|0)+12>>2]|0)+8>>2]|0)+688>>2]=c[ak>>2];aI=aI+1|0;}while((aI|0)<(c[550910]|0))}if((c[ak>>2]|0)==0){cY=0}else{AP(48);cY=c[ak>>2]|0}cx(bL|0,1733048,(aU=i,i=i+8|0,c[aU>>2]=cY,aU)|0)|0;i=aU;if((cR|0)!=63){break}}else if((cR|0)!=63){break}aW(1732200,265,1,bL|0)|0}}while(0);if((c[550910]|0)>0){cZ=0}else{aS=1976;break}L2709:while(1){aI=c[(c[550876]|0)+(cZ<<2)>>2]|0;bh=c[(c[550878]|0)+(c[aI>>2]<<2)>>2]|0;bJ=bh|0;do{if((c[aI+336>>2]|0)==0){a9=c[(c[bJ>>2]|0)+16>>2]|0;if((a9|0)!=0){a3=il(a9,0,0,1)|0;a9=G;a4=bh+32|0;bH=c[a4+4>>2]|0;if(!(a9>>>0<bH>>>0|a9>>>0==bH>>>0&a3>>>0<(c[a4>>2]|0)>>>0)){break}}a4=c[aI+20>>2]|0;a3=(a4|0)<0|0?-1:0;bH=aI+64|0;a9=c[bH+4>>2]|0;if((a3|0)<(a9|0)|(a3|0)==(a9|0)&a4>>>0<(c[bH>>2]|0)>>>0){break L2709}if((c[(c[bJ>>2]|0)+24>>2]|0)==0){break}bH=bh+8|0;a4=0;do{a9=c[(c[550876]|0)+((c[bH>>2]|0)+a4<<2)>>2]|0;a3=c[(c[550878]|0)+(c[a9>>2]<<2)>>2]|0;c[a9+336>>2]=1;if((c[a3+40>>2]|0)!=0){at=a9+32|0;aD=a9+40|0;bF=NV(c[at>>2]|0,c[at+4>>2]|0,c[aD>>2]|0,c[aD+4>>2]|0)|0;aD=(c[(c[a9+12>>2]|0)+8>>2]|0)+104|0;c[ao>>2]=1;c[cB>>2]=1e6;a9=AY(bF,G,aD,X)|0;aD=G;bF=a3+16|0;a3=c[bF>>2]|0;at=c[bF+4>>2]|0;bG=(at|0)>(aD|0)|(at|0)==(aD|0)&a3>>>0>a9>>>0;c[bF>>2]=bG?a9:a3;c[bF+4>>2]=bG?aD:at}a4=a4+1|0;}while(a4>>>0<(c[(c[bJ>>2]|0)+24>>2]|0)>>>0)}}while(0);bJ=cZ+1|0;if((bJ|0)<(c[550910]|0)){cZ=bJ}else{aS=1976;break L2621}}bJ=c[550910]|0;L2725:do{if((bJ|0)>0){bh=0;aI=2147483647;a4=-1;bH=0;while(1){at=c[(c[550876]|0)+(bH<<2)>>2]|0;aD=c[at+12>>2]|0;bG=aD+208|0;bF=c[bG>>2]|0;a3=c[bG+4>>2]|0;c[cC>>2]=1;c[az>>2]=1e6;bG=AY(bF,a3,aD+40|0,W)|0;aD=G;if((c[at+340>>2]|0)==0){a3=(c[at+336>>2]|0)==0&((aD|0)<(aI|0)|(aD|0)==(aI|0)&bG>>>0<a4>>>0);c_=a3?aD:aI;c$=a3?bG:a4;c0=a3?at:bh}else{c_=aI;c$=a4;c0=bh}at=bH+1|0;c1=c[550910]|0;if((at|0)<(c1|0)){bh=c0;aI=c_;a4=c$;bH=at}else{break}}if((c0|0)==0){c2=c1;aS=1982;break}bH=c[c0+300>>2]|0;L2733:do{if((bH|0)==0){a4=c[c0+8>>2]|0;if((a4|0)<=-1){aS=2013;break L2621}c3=c[(c[551808]|0)+(a4<<2)>>2]|0;aS=2015}else{a4=c[bH+8>>2]|0;aI=gn(c[a4+8>>2]|0)|0;L2735:do{if((aI|0)>-1){c4=eR()|0}else{if((aI|0)==(-541478725|0)){bh=eR()|0;at=a4+28|0;if((c[at>>2]|0)<=0){c4=bh;break}a3=a4+24|0;bG=0;while(1){aD=c[(c[(c[a3>>2]|0)+(bG<<2)>>2]|0)+4>>2]|0;bF=c[(c[550878]|0)+(c[aD>>2]<<2)>>2]|0;c[aD+336>>2]=1;if((c[bF+40>>2]|0)!=0){a9=aD+32|0;as=aD+40|0;ar=NV(c[a9>>2]|0,c[a9+4>>2]|0,c[as>>2]|0,c[as+4>>2]|0)|0;as=(c[(c[aD+12>>2]|0)+8>>2]|0)+104|0;c[am>>2]=1;c[bZ>>2]=1e6;aD=AY(ar,G,as,j)|0;as=G;ar=bF+16|0;bF=c[ar>>2]|0;a9=c[ar+4>>2]|0;aC=(a9|0)>(as|0)|(a9|0)==(as|0)&bF>>>0>aD>>>0;c[ar>>2]=aC?aD:bF;c[ar+4>>2]=aC?as:a9}a9=bG+1|0;if((a9|0)<(c[at>>2]|0)){bG=a9}else{c4=bh;break L2735}}}else if((aI|0)!=(-11|0)){c5=aI;break L2733}bh=a4+20|0;bG=c[bh>>2]|0;if((bG|0)>0){at=a4+16|0;a3=0;a9=0;as=0;aC=bG;while(1){bG=c[(c[at>>2]|0)+(a9<<2)>>2]|0;ar=c[bG+4>>2]|0;bF=c[(c[551810]|0)+(c[ar>>2]<<2)>>2]|0;do{if((c[bF+8>>2]|0)==0){if((c[bF+4>>2]|0)!=0){c6=a3;c7=as;c8=aC;break}aD=gE(c[bG>>2]|0)|0;bE=(aD|0)>(a3|0);c6=bE?aD:a3;c7=bE?ar:as;c8=c[bh>>2]|0}else{c6=a3;c7=as;c8=aC}}while(0);ar=a9+1|0;if((ar|0)<(c8|0)){a3=c6;a9=ar;as=c7;aC=c8}else{break}}if((c7|0)!=0){c3=c7;aS=2015;break L2733}}aC=a4+28|0;if((c[aC>>2]|0)<=0){aS=2190;break L2725}as=a4+24|0;a9=0;while(1){c[(c[(c[(c[as>>2]|0)+(a9<<2)>>2]|0)+4>>2]|0)+340>>2]=1;a3=a9+1|0;if((a3|0)<(c[aC>>2]|0)){a9=a3}else{aS=2190;break L2725}}}}while(0);c9=(c4|0)<0?c4:0;aS=2187}}while(0);do{if((aS|0)==2015){aS=0;bH=c3|0;a4=c[bH>>2]|0;aI=c[(c[551810]|0)+(a4<<2)>>2]|0;a9=aI|0;aC=c[a9>>2]|0;L2765:do{if((c[aI+40>>2]|0)==0){da=aC;aS=2020}else{as=aI+32|0;a3=aI+12|0;bh=0;while(1){if((bh|0)>=(c[as>>2]|0)){break}at=c[(c[551808]|0)+((c[a3>>2]|0)+bh<<2)>>2]|0;ar=at+48|0;bG=AW(c[ar>>2]|0,c[ar+4>>2]|0,1e6,0,1e6,0)|0;ar=G;bF=Ch()|0;bE=at+32|0;at=NV(bF,G,c[bE>>2]|0,c[bE+4>>2]|0)|0;bE=G;if((ar|0)>(bE|0)|(ar|0)==(bE|0)&bG>>>0>at>>>0){aS=2021;break L2765}else{bh=bh+1|0}}da=c[a9>>2]|0;aS=2020}}while(0);L2771:do{if((aS|0)==2020){aS=0;a9=mn(da,m)|0;if((a9|0)==-11){aS=2021;break}if((a9|0)<0){do{if((a9|0)!=-541478725){fe(aC+32|0,a9);if((c[744092]|0)==0){break}eW(1)}}while(0);c[aI+4>>2]=1;a9=aI+32|0;if((c[a9>>2]|0)<=0){db=-11;break}bh=aI+12|0;a3=0;while(1){as=c[(c[551808]|0)+((c[bh>>2]|0)+a3<<2)>>2]|0;if((c[as+12>>2]|0)!=0){eM(as,0)|0}if((c[550910]|0)>0){as=0;do{at=c[(c[550876]|0)+(as<<2)>>2]|0;do{if((c[at+8>>2]|0)==((c[bh>>2]|0)+a3|0)){if((c[at+344>>2]|0)==0){if((c[(c[at+60>>2]|0)+8>>2]|0)!=3){break}}bG=c[(c[550878]|0)+(c[at>>2]<<2)>>2]|0;c[at+336>>2]=1;if((c[bG+40>>2]|0)==0){break}bE=at+32|0;ar=at+40|0;bF=NV(c[bE>>2]|0,c[bE+4>>2]|0,c[ar>>2]|0,c[ar+4>>2]|0)|0;ar=(c[(c[at+12>>2]|0)+8>>2]|0)+104|0;c[cg>>2]=1;c[cv>>2]=1e6;bE=AY(bF,G,ar,l)|0;ar=G;bF=bG+16|0;bG=c[bF>>2]|0;aD=c[bF+4>>2]|0;bA=(aD|0)>(ar|0)|(aD|0)==(ar|0)&bG>>>0>bE>>>0;c[bF>>2]=bA?bE:bG;c[bF+4>>2]=bA?ar:aD}}while(0);as=as+1|0;}while((as|0)<(c[550910]|0))}as=a3+1|0;if((as|0)<(c[a9>>2]|0)){a3=as}else{db=-11;break L2771}}}if((c[550916]|0)>0){a3=0;do{c[(c[(c[551810]|0)+(a3<<2)>>2]|0)+8>>2]=0;a3=a3+1|0;}while((a3|0)<(c[550916]|0))}if((c[550910]|0)>0){a3=0;do{c[(c[(c[550876]|0)+(a3<<2)>>2]|0)+340>>2]=0;a3=a3+1|0;}while((a3|0)<(c[550910]|0))}if((c[744098]|0)!=0){mT(0,48,m,c[744100]|0,c[(c[aC+28>>2]|0)+(c[cy>>2]<<2)>>2]|0)}a3=c[cy>>2]|0;do{if((a3|0)<(c[aI+32>>2]|0)){a9=aI+12|0;bh=(c[a9>>2]|0)+a3|0;as=c[(c[551808]|0)+(bh<<2)>>2]|0;if((c[as+8>>2]|0)!=0){break}if((c[745296]|0)!=0){at=as+4|0;aD=Cp(c[(c[(c[at>>2]|0)+8>>2]|0)+8>>2]|0)|0;NM(cO|0,0,32);a[cO]=0;ar=as+40|0;bA=c[ar>>2]|0;bF=c[ar+4>>2]|0;if((bA|0)==0&(bF|0)==(-2147483648|0)){bc(cO|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bG=1667440;bc(cO|0,32,bG|0,(aU=i,i=i+16|0,c[aU>>2]=bA,c[aU+8>>2]=bF,aU)|0)|0;i=aU}NM(cN|0,0,32);a[cN]=0;bF=c[ar>>2]|0;bA=c[ar+4>>2]|0;if((bF|0)==0&(bA|0)==(-2147483648|0)){bc(cN|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bu=(+(bF>>>0)+ +(bA|0)*4294967296.0)*1.0e-6;bc(cN|0,32,1668840,(aU=i,i=i+8|0,h[aU>>3]=bu,aU)|0)|0;i=aU}NM(cL|0,0,32);a[cL]=0;bA=as+56|0;bF=c[bA>>2]|0;ar=c[bA+4>>2]|0;if((bF|0)==0&(ar|0)==(-2147483648|0)){bc(cL|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bG=1667440;bc(cL|0,32,bG|0,(aU=i,i=i+16|0,c[aU>>2]=bF,c[aU+8>>2]=ar,aU)|0)|0;i=aU}NM(cK|0,0,32);a[cK]=0;ar=c[bA>>2]|0;bF=c[bA+4>>2]|0;if((ar|0)==0&(bF|0)==(-2147483648|0)){bc(cK|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bu=(+(ar>>>0)+ +(bF|0)*4294967296.0)*1.0e-6;bc(cK|0,32,1668840,(aU=i,i=i+8|0,h[aU>>3]=bu,aU)|0)|0;i=aU}NM(cI|0,0,32);a[cI]=0;bF=c[a_>>2]|0;ar=c[a_+4>>2]|0;if((bF|0)==0&(ar|0)==(-2147483648|0)){bc(cI|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bA=1667440;bc(cI|0,32,bA|0,(aU=i,i=i+16|0,c[aU>>2]=bF,c[aU+8>>2]=ar,aU)|0)|0;i=aU}NM(cJ|0,0,32);a[cJ]=0;ar=c[a_>>2]|0;bF=c[a_+4>>2]|0;bA=c[at>>2]|0;if((ar|0)==0&(bF|0)==(-2147483648|0)){bc(cJ|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bu=(+(ar>>>0)+ +(bF|0)*4294967296.0)*(+(c[bA+40>>2]|0)/+(c[bA+44>>2]|0));bc(cJ|0,32,1668840,(aU=i,i=i+8|0,h[aU>>3]=bu,aU)|0)|0;i=aU}NM(cE|0,0,32);a[cE]=0;bA=c[bN>>2]|0;bF=c[bN+4>>2]|0;if((bA|0)==0&(bF|0)==(-2147483648|0)){bc(cE|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{ar=1667440;bc(cE|0,32,ar|0,(aU=i,i=i+16|0,c[aU>>2]=bA,c[aU+8>>2]=bF,aU)|0)|0;i=aU}NM(cF|0,0,32);a[cF]=0;bF=c[bN>>2]|0;bA=c[bN+4>>2]|0;ar=c[at>>2]|0;if((bF|0)==0&(bA|0)==(-2147483648|0)){bc(cF|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bu=(+(bF>>>0)+ +(bA|0)*4294967296.0)*(+(c[ar+40>>2]|0)/+(c[ar+44>>2]|0));bc(cF|0,32,1668840,(aU=i,i=i+8|0,h[aU>>3]=bu,aU)|0)|0;i=aU}NM(cH|0,0,32);a[cH]=0;ar=as|0;bA=(c[(c[551810]|0)+(c[ar>>2]<<2)>>2]|0)+16|0;bF=c[bA>>2]|0;at=c[bA+4>>2]|0;if((bF|0)==0&(at|0)==(-2147483648|0)){bc(cH|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bA=1667440;bc(cH|0,32,bA|0,(aU=i,i=i+16|0,c[aU>>2]=bF,c[aU+8>>2]=at,aU)|0)|0;i=aU}NM(cG|0,0,32);a[cG]=0;at=(c[(c[551810]|0)+(c[ar>>2]<<2)>>2]|0)+16|0;ar=c[at>>2]|0;bF=c[at+4>>2]|0;if((ar|0)==0&(bF|0)==(-2147483648|0)){bc(cG|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bu=(+(ar>>>0)+ +(bF|0)*4294967296.0)*1.0e-6;bc(cG|0,32,1668840,(aU=i,i=i+8|0,h[aU>>3]=bu,aU)|0)|0;i=aU}AN(0,32,1739648,(aU=i,i=i+96|0,c[aU>>2]=bh,c[aU+8>>2]=aD,c[aU+16>>2]=cO,c[aU+24>>2]=cN,c[aU+32>>2]=cL,c[aU+40>>2]=cK,c[aU+48>>2]=cI,c[aU+56>>2]=cJ,c[aU+64>>2]=cE,c[aU+72>>2]=cF,c[aU+80>>2]=cH,c[aU+88>>2]=cG,aU)|0);i=aU}aD=as+72|0;do{if((c[aD>>2]|0)==0){bh=aC+1056|0;bF=c[bh>>2]|0;ar=c[bh+4>>2]|0;if((bF|0)==0&(ar|0)==(-2147483648|0)){break}at=as+4|0;if((c[(c[at>>2]|0)+188>>2]|0)>=64){break}bA=as+40|0;do{if((c[bA>>2]|0)==0&(c[bA+4>>2]|0)==(-2147483648|0)){bG=aI+16|0;bE=c[bG>>2]|0;bz=c[bG+4>>2]|0;by=NV(0,0,bF,ar)|0;if(!((bE|0)==(by|0)&(bz|0)==(G|0))){dc=ar;dd=bF;break}if((c[(c[aC+4>>2]|0)+8>>2]&512|0)==0){dc=ar;dd=bF;break}bz=aC+24|0;if((c[bz>>2]|0)==0){de=2147483647;df=-1;dg=ar;dh=bF}else{by=aC+28|0;bE=2147483647;bB=-1;bC=0;while(1){bx=c[(c[by>>2]|0)+(bC<<2)>>2]|0;do{if((c[bx+76>>2]|0)==48){di=bE;dj=bB}else{bw=bx+48|0;bs=c[bw>>2]|0;bv=c[bw+4>>2]|0;if((bs|0)==0&(bv|0)==(-2147483648|0)){di=bE;dj=bB;break}aq=bx+40|0;c[cq>>2]=1;c[cp>>2]=1e6;aQ=AY(bs,bv,aq,A)|0;bv=G;if(!((bE|0)>(bv|0)|(bE|0)==(bv|0)&bB>>>0>aQ>>>0)){di=bE;dj=bB;break}aQ=c[bw>>2]|0;bv=c[bw+4>>2]|0;c[cr>>2]=1;c[cs>>2]=1e6;bw=AY(aQ,bv,aq,B)|0;di=G;dj=bw}}while(0);bx=bC+1|0;if(bx>>>0<(c[bz>>2]|0)>>>0){bE=di;bB=dj;bC=bx}else{break}}de=di;df=dj;dg=c[bh+4>>2]|0;dh=c[bh>>2]|0}if(!((de|0)>(dg|0)|(de|0)==(dg|0)&df>>>0>dh>>>0)){dc=dg;dd=dh;break}bC=NV(df,de,dh,dg)|0;bB=G;AN(aC,40,1739304,(aU=i,i=i+16|0,c[aU>>2]=bC,c[aU+8>>2]=bB,aU)|0);i=aU;bB=NV(0,0,df,de)|0;c[bG>>2]=bB;c[bG+4>>2]=G;dc=c[bh+4>>2]|0;dd=c[bh>>2]|0}else{dc=ar;dd=bF}}while(0);c[aP>>2]=1;c[bO>>2]=1e6;bF=AY(dd,dc,C,(c[at>>2]|0)+40|0)|0;ar=G;bh=(c[at>>2]|0)+188|0;bA=NW(1,0,c[bh>>2]|0)|0;bB=NU(bA,G,bF,ar)|0;bA=G;c[aD>>2]=1;if(!((bA|0)>(ar|0)|(bA|0)==(ar|0)&bB>>>0>bF>>>0)){break}bB=c[bN>>2]|0;bA=c[bN+4>>2]|0;do{if(!((bB|0)==0&(bA|0)==(-2147483648|0))){bC=c[bh>>2]|0;bE=NW(1,0,bC-1|0)|0;bz=NU(bE,G,bF,ar)|0;bE=G;if(!((bA|0)>(bE|0)|(bA|0)==(bE|0)&bB>>>0>bz>>>0)){break}bz=NW(1,0,bC|0)|0;bC=NV(bB,bA,bz,G)|0;c[bN>>2]=bC;c[bN+4>>2]=G;c[aD>>2]=0}}while(0);bA=c[a_>>2]|0;bB=c[a_+4>>2]|0;if((bA|0)==0&(bB|0)==(-2147483648|0)){break}bh=c[(c[at>>2]|0)+188>>2]|0;bC=NW(1,0,bh-1|0)|0;bz=NU(bC,G,bF,ar)|0;bC=G;if(!((bB|0)>(bC|0)|(bB|0)==(bC|0)&bA>>>0>bz>>>0)){break}bz=NW(1,0,bh|0)|0;bh=NV(bA,bB,bz,G)|0;c[a_>>2]=bh;c[a_+4>>2]=G;c[aD>>2]=0}}while(0);if(!((c[bN>>2]|0)==0&(c[bN+4>>2]|0)==(-2147483648|0))){aD=aI+16|0;bh=c[aD>>2]|0;bz=c[aD+4>>2]|0;c[b0>>2]=1;c[b1>>2]=1e6;aD=AY(bh,bz,D,(c[as+4>>2]|0)+40|0)|0;bz=NU(c[bN>>2]|0,c[bN+4>>2]|0,aD,G)|0;c[bN>>2]=bz;c[bN+4>>2]=G}do{if(!((c[a_>>2]|0)==0&(c[a_+4>>2]|0)==(-2147483648|0))){bz=aI+16|0;aD=c[bz>>2]|0;bh=c[bz+4>>2]|0;c[ca>>2]=1;c[b9>>2]=1e6;bz=AY(aD,bh,E,(c[as+4>>2]|0)+40|0)|0;bh=NU(c[a_>>2]|0,c[a_+4>>2]|0,bz,G)|0;bz=G;c[a_>>2]=bh;c[a_+4>>2]=bz;if((bh|0)==0&(bz|0)==(-2147483648|0)){break}bu=(+(bh>>>0)+ +(bz|0)*4294967296.0)*+h[as+88>>3];bz=+R(bu)>=1.0?bu>0.0?(ah(+Q(bu/4294967296.0),4294967295.0)|0)>>>0:~~+ab((bu- +(~~bu>>>0))/4294967296.0)>>>0:0;c[a_>>2]=~~bu>>>0;c[a_+4>>2]=bz}}while(0);bz=c[bN>>2]|0;bh=c[bN+4>>2]|0;do{if((bz|0)==0&(bh|0)==(-2147483648|0)){aS=2121}else{bu=(+(bz>>>0)+ +(bh|0)*4294967296.0)*+h[as+88>>3];aD=~~bu>>>0;bB=+R(bu)>=1.0?bu>0.0?(ah(+Q(bu/4294967296.0),4294967295.0)|0)>>>0:~~+ab((bu- +(~~bu>>>0))/4294967296.0)>>>0:0;c[bN>>2]=aD;c[bN+4>>2]=bB;if((aD|0)==0&(bB|0)==(-2147483648|0)){aS=2140;break}bA=as+40|0;if((c[bA>>2]|0)!=0|(c[bA+4>>2]|0)!=(-2147483648|0)|(c[745332]|0)!=0){aS=2121;break}if((c[(c[aC+4>>2]|0)+8>>2]&512|0)==0){aS=2121;break}bA=aI+24|0;if((c[bA>>2]|0)==0&(c[bA+4>>2]|0)==(-2147483648|0)){aS=2121;break}bC=as+4|0;bE=(c[bC>>2]|0)+40|0;c[bI>>2]=1;c[aA>>2]=1e6;by=AY(aD,bB,bE,F)|0;bE=NV(by,G,c[bA>>2]|0,c[bA+4>>2]|0)|0;bA=G;bu=+(bE>>>0)+ +(bA|0)*4294967296.0;br=+g[65166];if(bu>=br*-1.0*1.0e6){if(bu<=br*1.0e6){aS=2121;break}if((c[(c[(c[bC>>2]|0)+8>>2]|0)+8>>2]|0)==3){aS=2121;break}}by=aI+16|0;bB=NV(c[by>>2]|0,c[by+4>>2]|0,bE,bA)|0;aD=G;c[by>>2]=bB;c[by+4>>2]=aD;AN(0,48,1739016,(aU=i,i=i+32|0,c[aU>>2]=bE,c[aU+8>>2]=bA,c[aU+16>>2]=bB,c[aU+24>>2]=aD,aU)|0);i=aU;c[cc>>2]=1;c[ax>>2]=1e6;aD=AY(bE,bA,H,(c[bC>>2]|0)+40|0)|0;bB=NV(c[bN>>2]|0,c[bN+4>>2]|0,aD,G)|0;aD=G;c[bN>>2]=bB;c[bN+4>>2]=aD;if((c[a_>>2]|0)==0&(c[a_+4>>2]|0)==(-2147483648|0)){dl=aD;dm=bB;aS=2122;break}c[ay>>2]=1;c[cb>>2]=1e6;bB=AY(bE,bA,I,(c[bC>>2]|0)+40|0)|0;bC=NV(c[a_>>2]|0,c[a_+4>>2]|0,bB,G)|0;c[a_>>2]=bC;c[a_+4>>2]=G;aS=2121}}while(0);if((aS|0)==2121){aS=0;dl=c[bN+4>>2]|0;dm=c[bN>>2]|0;aS=2122}L2910:do{if((aS|0)==2122){aS=0;if((dm|0)==0&(dl|0)==(-2147483648|0)){aS=2140;break}bh=as+40|0;if((c[bh>>2]|0)==0&(c[bh+4>>2]|0)==(-2147483648|0)|(c[745332]|0)!=0){aS=2140;break}bz=as+4|0;bC=(c[bz>>2]|0)+40|0;c[a$>>2]=1;c[a5>>2]=1e6;bB=AY(dm,dl,bC,J)|0;bC=G;bA=c[bh>>2]|0;bE=c[bh+4>>2]|0;aD=NV(bB,bC,bA,bE)|0;by=G;br=+(aD>>>0)+ +(by|0)*4294967296.0;if((c[(c[aC+4>>2]|0)+8>>2]&512|0)!=0){bu=+g[65166];do{if(br>=bu*-1.0*1.0e6){if(br>bu*1.0e6){if((c[(c[(c[bz>>2]|0)+8>>2]|0)+8>>2]|0)!=3){break}}bx=NU(bB,bC,1,0)|0;bw=G;aq=as+64|0;bv=c[aq+4>>2]|0;if(!((bw|0)<(bv|0)|(bw|0)==(bv|0)&bx>>>0<(c[aq>>2]|0)>>>0)){aS=2140;break L2910}}}while(0);bC=aI+16|0;bB=NV(c[bC>>2]|0,c[bC+4>>2]|0,aD,by)|0;ar=G;c[bC>>2]=bB;c[bC+4>>2]=ar;AN(0,48,1738760,(aU=i,i=i+32|0,c[aU>>2]=aD,c[aU+8>>2]=by,c[aU+16>>2]=bB,c[aU+24>>2]=ar,aU)|0);i=aU;c[ck>>2]=1;c[cf>>2]=1e6;ar=AY(aD,by,K,(c[bz>>2]|0)+40|0)|0;bB=NV(c[bN>>2]|0,c[bN+4>>2]|0,ar,G)|0;ar=G;c[bN>>2]=bB;c[bN+4>>2]=ar;if((c[a_>>2]|0)==0&(c[a_+4>>2]|0)==(-2147483648|0)){dn=ar;dp=bB;break}c[ce>>2]=1;c[co>>2]=1e6;bB=AY(aD,by,L,(c[bz>>2]|0)+40|0)|0;ar=NV(c[a_>>2]|0,c[a_+4>>2]|0,bB,G)|0;c[a_>>2]=ar;c[a_+4>>2]=G;aS=2140;break}bu=+g[65164];do{if(br<bu*-1.0*1.0e6){aS=2134}else{if(br<=bu*1.0e6){break}if((c[(c[(c[bz>>2]|0)+8>>2]|0)+8>>2]|0)!=3){aS=2134}}}while(0);if((aS|0)==2134){aS=0;by=c[bN+4>>2]|0;aD=c[cy>>2]|0;AN(0,24,1738520,(aU=i,i=i+40|0,c[aU>>2]=c[bN>>2],c[aU+8>>2]=by,c[aU+16>>2]=bA,c[aU+24>>2]=bE,c[aU+32>>2]=aD,aU)|0);i=aU;c[bN>>2]=0;c[bN+4>>2]=-2147483648}aD=c[a_>>2]|0;by=c[a_+4>>2]|0;if((aD|0)==0&(by|0)==(-2147483648|0)){aS=2140;break}ar=(c[bz>>2]|0)+40|0;c[cn>>2]=1;c[cj>>2]=1e6;bB=AY(aD,by,ar,M)|0;ar=c[bh>>2]|0;by=c[bh+4>>2]|0;aD=NV(bB,G,ar,by)|0;bu=+(aD>>>0)+ +(G|0)*4294967296.0;br=+g[65164];if(bu>=br*-1.0*1.0e6){if(bu<=br*1.0e6){aS=2140;break}if((c[(c[(c[bz>>2]|0)+8>>2]|0)+8>>2]|0)==3){aS=2140;break}}aD=c[a_+4>>2]|0;bB=c[cy>>2]|0;AN(0,24,1738152,(aU=i,i=i+40|0,c[aU>>2]=c[a_>>2],c[aU+8>>2]=aD,c[aU+16>>2]=ar,c[aU+24>>2]=by,c[aU+32>>2]=bB,aU)|0);i=aU;c[a_>>2]=0;c[a_+4>>2]=-2147483648;aS=2140}}while(0);if((aS|0)==2140){aS=0;dn=c[bN+4>>2]|0;dp=c[bN>>2]|0}if(!((dp|0)==0&(dn|0)==(-2147483648|0))){bB=(c[as+4>>2]|0)+40|0;c[bD>>2]=1;c[cd>>2]=1e6;by=AY(dp,dn,bB,N)|0;bB=aI+24|0;c[bB>>2]=by;c[bB+4>>2]=G}if((c[745296]|0)==0){dq=as|0}else{bB=(c[cy>>2]|0)+(c[a9>>2]|0)|0;by=as+4|0;ar=Cp(c[(c[(c[by>>2]|0)+8>>2]|0)+8>>2]|0)|0;NM(cm|0,0,32);a[cm]=0;aD=c[a_>>2]|0;bC=c[a_+4>>2]|0;if((aD|0)==0&(bC|0)==(-2147483648|0)){bc(cm|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bF=1667440;bc(cm|0,32,bF|0,(aU=i,i=i+16|0,c[aU>>2]=aD,c[aU+8>>2]=bC,aU)|0)|0;i=aU}NM(cl|0,0,32);a[cl]=0;bC=c[a_>>2]|0;aD=c[a_+4>>2]|0;bF=c[by>>2]|0;if((bC|0)==0&(aD|0)==(-2147483648|0)){bc(cl|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{br=(+(bC>>>0)+ +(aD|0)*4294967296.0)*(+(c[bF+40>>2]|0)/+(c[bF+44>>2]|0));bc(cl|0,32,1668840,(aU=i,i=i+8|0,h[aU>>3]=br,aU)|0)|0;i=aU}NM(aB|0,0,32);a[aB]=0;bF=c[bN>>2]|0;aD=c[bN+4>>2]|0;if((bF|0)==0&(aD|0)==(-2147483648|0)){bc(aB|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bC=1667440;bc(aB|0,32,bC|0,(aU=i,i=i+16|0,c[aU>>2]=bF,c[aU+8>>2]=aD,aU)|0)|0;i=aU}NM(cD|0,0,32);a[cD]=0;aD=c[bN>>2]|0;bF=c[bN+4>>2]|0;bC=c[by>>2]|0;if((aD|0)==0&(bF|0)==(-2147483648|0)){bc(cD|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{br=(+(aD>>>0)+ +(bF|0)*4294967296.0)*(+(c[bC+40>>2]|0)/+(c[bC+44>>2]|0));bc(cD|0,32,1668840,(aU=i,i=i+8|0,h[aU>>3]=br,aU)|0)|0;i=aU}NM(cu|0,0,32);a[cu]=0;bC=as|0;bF=(c[(c[551810]|0)+(c[bC>>2]<<2)>>2]|0)+16|0;aD=c[bF>>2]|0;by=c[bF+4>>2]|0;if((aD|0)==0&(by|0)==(-2147483648|0)){bc(cu|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{bF=1667440;bc(cu|0,32,bF|0,(aU=i,i=i+16|0,c[aU>>2]=aD,c[aU+8>>2]=by,aU)|0)|0;i=aU}NM(cz|0,0,32);a[cz]=0;by=(c[(c[551810]|0)+(c[bC>>2]<<2)>>2]|0)+16|0;aD=c[by>>2]|0;bF=c[by+4>>2]|0;if((aD|0)==0&(bF|0)==(-2147483648|0)){bc(cz|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{br=(+(aD>>>0)+ +(bF|0)*4294967296.0)*1.0e-6;bc(cz|0,32,1668840,(aU=i,i=i+8|0,h[aU>>3]=br,aU)|0)|0;i=aU}AN(0,32,1737600,(aU=i,i=i+64|0,c[aU>>2]=bB,c[aU+8>>2]=ar,c[aU+16>>2]=cm,c[aU+24>>2]=cl,c[aU+32>>2]=aB,c[aU+40>>2]=cD,c[aU+48>>2]=cu,c[aU+56>>2]=cz,aU)|0);i=aU;dq=bC}bC=c[a_>>2]|0;ar=c[a_+4>>2]|0;bB=c[(c[551810]|0)+(c[dq>>2]<<2)>>2]|0;bF=bB+32|0;if((c[bF>>2]|0)>0){aD=bB+12|0;bB=as+4|0;by=0;do{at=c[(c[551808]|0)+((c[aD>>2]|0)+by<<2)>>2]|0;aq=at+224|0;do{if((c[aq>>2]|0)!=0){bx=AY(bC,ar,(c[bB>>2]|0)+40|0,(c[at+4>>2]|0)+40|0)|0;bv=NU(bx,G,-1,-1)|0;bx=G;bw=at+208|0;aQ=c[bw+4>>2]|0;if(!((bx|0)>(aQ|0)|(bx|0)==(aQ|0)&bv>>>0>(c[bw>>2]|0)>>>0)){break}aQ=at+216|0;bs=c[aQ+4>>2]|0;if((bx|0)<(bs|0)|(bx|0)==(bs|0)&bv>>>0<(c[aQ>>2]|0)>>>0){if((c[c[aq>>2]>>2]|0)==0){aS=2170}}else{aS=2170}if((aS|0)==2170){aS=0;eQ(at,0)}aQ=at+248|0;if((c[aQ>>2]|0)<=0){break}bs=at+244|0;be=0;aJ=0;do{be=(gE(c[c[(c[bs>>2]|0)+(aJ<<2)>>2]>>2]|0)|0)+be|0;aJ=aJ+1|0;}while((aJ|0)<(c[aQ>>2]|0));if((be|0)==0){break}aJ=c[aq>>2]|0;bG=aJ+136|0;c[bG>>2]=bv;c[bG+4>>2]=bx;c[bw>>2]=bv;c[bw+4>>2]=bx;if((c[aQ>>2]|0)>0){dr=0}else{break}do{gz(c[c[(c[bs>>2]|0)+(dr<<2)>>2]>>2]|0,aJ,12)|0;dr=dr+1|0;}while((dr|0)<(c[aQ>>2]|0))}}while(0);by=by+1|0;}while((by|0)<(c[bF>>2]|0))}bF=eM(as,m)|0;if((bF|0)>=0){break}zN(bF,cA,128)|0;bF=c[c[as+4>>2]>>2]|0;AN(0,16,1737128,(aU=i,i=i+24|0,c[aU>>2]=c[dq>>2],c[aU+8>>2]=bF,c[aU+16>>2]=cA,aU)|0);i=aU;if((c[744092]|0)==0){break}eW(1)}else{bF=c[(c[551810]|0)+(a4<<2)>>2]|0;by=c[bF>>2]|0;bB=c[(c[by+28>>2]|0)+(a3<<2)>>2]|0;ar=bF+36|0;if((a3|0)<(c[ar>>2]|0)){break}bF=Cp(c[(c[bB+8>>2]|0)+8>>2]|0)|0;bC=c[cy>>2]|0;aD=c[bQ>>2]|0;a9=c[bQ+4>>2]|0;NM(b2|0,0,32);a[b2]=0;aq=c[bN>>2]|0;at=c[bN+4>>2]|0;if((aq|0)==0&(at|0)==(-2147483648|0)){bc(b2|0,32,1670672,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0)|0;i=aU}else{br=(+(aq>>>0)+ +(at|0)*4294967296.0)*(+(c[bB+40>>2]|0)/+(c[bB+44>>2]|0));bc(b2|0,32,1668840,(aU=i,i=i+8|0,h[aU>>3]=br,aU)|0)|0;i=aU}AN(by,24,1736688,(aU=i,i=i+48|0,c[aU>>2]=bF,c[aU+8>>2]=a4,c[aU+16>>2]=bC,c[aU+24>>2]=aD,c[aU+32>>2]=a9,c[aU+40>>2]=b2,aU)|0);i=aU;c[ar>>2]=(c[cy>>2]|0)+1}}while(0);nJ(m);db=0}}while(0);if((aS|0)==2021){aS=0;c[aI+8>>2]=1;db=-11}if((db|0)==-11){if((c[(c[(c[551810]|0)+(c[bH>>2]<<2)>>2]|0)+8>>2]|0)==0){aS=2190;break L2725}c[c0+340>>2]=1;aS=2190;break L2725}else{if((db|0)<0){c5=db;break}c9=eR()|0;aS=2187;break}}}while(0);if((aS|0)==2187){aS=0;if((c9|0)<0){c5=c9}else{aS=2190;break}}if(!((c5|0)==(-11|0)|(c5|0)==(-541478725|0))){aS=2189;break L2621}}else{c2=bJ;aS=1982}}while(0);do{if((aS|0)==1982){aS=0;bJ=c[550876]|0;a4=0;while(1){if((a4|0)>=(c2|0)){aS=1991;break}if((c[(c[bJ+(a4<<2)>>2]|0)+340>>2]|0)==0){a4=a4+1|0}else{break}}if((aS|0)==1991){aS=0;AN(0,40,1616768,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;break}if((c[550916]|0)>0){a4=0;do{c[(c[(c[551810]|0)+(a4<<2)>>2]|0)+8>>2]=0;a4=a4+1|0;}while((a4|0)<(c[550916]|0));ds=c[550910]|0}else{ds=c2}if((ds|0)>0){a4=0;do{c[(c[(c[550876]|0)+(a4<<2)>>2]|0)+340>>2]=0;a4=a4+1|0;}while((a4|0)<(c[550910]|0))}Ci(1e4)|0;aS=2190}}while(0);if((aS|0)==2190){aS=0;eL(0,cP,ap,au,bK)}if((c[541004]|0)!=0){break L2619}}if((aS|0)==1976){AN(0,40,1717592,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;break}else if((aS|0)==2013){AN(0,0,1616448,(aU=i,i=i+24|0,c[aU>>2]=1616096,c[aU+8>>2]=1615896,c[aU+16>>2]=3155,aU)|0);i=aU;cw();return 0}else if((aS|0)==2189){AN(0,16,1713752,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;break}}}while(0);aF=c[550914]|0;if((aF|0)>0){aO=0;aY=aF;while(1){aF=c[(c[551808]|0)+(aO<<2)>>2]|0;do{if((c[(c[(c[551810]|0)+(c[aF>>2]<<2)>>2]|0)+4>>2]|0)==0){if((c[aF+12>>2]|0)==0){dt=aY;break}eM(aF,0)|0;dt=c[550914]|0}else{dt=aY}}while(0);aF=aO+1|0;if((aF|0)<(dt|0)){aO=aF;aY=dt}else{break}}}aY=c[550910]|0;if((aY|0)>0){aO=e+24|0;aF=e+28|0;cy=e+8|0;b2=e+16|0;bN=e+48|0;bQ=0;cA=aY;while(1){aY=c[(c[550876]|0)+(bQ<<2)>>2]|0;a_=aY+12|0;cz=c[(c[a_>>2]|0)+8>>2]|0;cu=aY|0;cD=c[c[(c[550878]|0)+(c[cu>>2]<<2)>>2]>>2]|0;L3041:do{if((c[aY+16>>2]|0)==0){du=cA}else{aB=c[cz+8>>2]|0;do{if((aB|0)==1){if((c[cz+424>>2]|0)<2){du=cA;break L3041}}else if((aB|0)==0){if((c[(c[cD+8>>2]|0)+28>>2]&32|0)==0){break}if((c[(c[cz+12>>2]|0)+12>>2]|0)==14){du=cA;break L3041}}}while(0);bK=aY+4|0;au=aY+296|0;cl=cz+104|0;cm=cz+668|0;cd=aB;while(1){if((cd|0)==0){dv=1998056;dw=1707848;dx=28}else if((cd|0)==1){dv=3068904;dw=1711184;dx=12}else{break}nB(e);c[aO>>2]=0;c[aF>>2]=0;eN(0,(aU=i,i=i+1|0,i=i+7&-8,c[aU>>2]=0,aU)|0);i=aU;bD=dk[dx&63](cz,e,0,f)|0;cj=c[cu>>2]|0;cn=c[bK>>2]|0;eN(1705632,(aU=i,i=i+24|0,c[aU>>2]=dw,c[aU+8>>2]=cj,c[aU+16>>2]=cn,aU)|0);i=aU;if((bD|0)<0){AN(0,8,1703872,(aU=i,i=i+8|0,c[aU>>2]=dw,aU)|0);i=aU;eW(1)}bD=c[aF>>2]|0;cn=NU(c[dv>>2]|0,c[dv+4>>2]|0,bD,(bD|0)<0|0?-1:0)|0;c[dv>>2]=cn;c[dv+4>>2]=G;cn=c[au>>2]|0;do{if((cn|0)!=0){bD=c[cm>>2]|0;if((bD|0)==0){break}aZ(bD|0,cn|0)|0}}while(0);if((c[f>>2]|0)==0){break}cn=c[cy>>2]|0;bD=c[cy+4>>2]|0;if(!((cn|0)==0&(bD|0)==(-2147483648|0))){cj=AY(cn,bD,cl,(c[a_>>2]|0)+40|0)|0;c[cy>>2]=cj;c[cy+4>>2]=G}cj=c[b2>>2]|0;bD=c[b2+4>>2]|0;if(!((cj|0)==0&(bD|0)==(-2147483648|0))){cn=AY(cj,bD,cl,(c[a_>>2]|0)+40|0)|0;c[b2>>2]=cn;c[b2+4>>2]=G}cn=c[bN>>2]|0;if((cn|0)>0){bD=AY(cn,(cn|0)<0|0?-1:0,cl,(c[a_>>2]|0)+40|0)|0;c[bN>>2]=bD}eO(cD,e,aY);bD=c[(c[(c[a_>>2]|0)+8>>2]|0)+8>>2]|0;if(!((bD|0)==0&(c[499382]|0)!=0)){cd=bD;continue}eP(aY,c[aF>>2]|0);cd=c[(c[(c[a_>>2]|0)+8>>2]|0)+8>>2]|0}du=c[550910]|0}}while(0);a_=bQ+1|0;if((a_|0)<(du|0)){bQ=a_;cA=du}else{break}}}AN(0,-8,1636408,(aU=i,i=i+8|0,c[aU>>2]=3119008,aU)|0);i=aU;if(a[85776]|0){cM(0,0,2203568)|0}if((c[550912]|0)>0){cA=0;do{le(c[c[(c[550878]|0)+(cA<<2)>>2]>>2]|0)|0;cA=cA+1|0;}while((cA|0)<(c[550912]|0))}cA=Ch()|0;eL(1,cP,ap,cA,G);cA=c[550910]|0;if((cA|0)>0){bQ=0;aF=cA;while(1){cA=c[(c[550876]|0)+(bQ<<2)>>2]|0;if((c[cA+16>>2]|0)==0){dy=aF}else{bN=cA+12|0;A6((c[(c[bN>>2]|0)+8>>2]|0)+672|0);cA=c[(c[bN>>2]|0)+8>>2]|0;pk(cA)|0;dy=c[550910]|0}cA=bQ+1|0;if((cA|0)<(dy|0)){bQ=cA;aF=dy}else{break}}}aF=c[550914]|0;if((aF|0)>0){dz=0;dA=aF}else{cQ=0;break}while(1){aF=c[(c[551808]|0)+(dz<<2)>>2]|0;if((c[aF+12>>2]|0)==0){dB=dA}else{bQ=c[(c[aF+4>>2]|0)+8>>2]|0;pk(bQ)|0;dB=c[550914]|0}bQ=dz+1|0;if((bQ|0)<(dB|0)){dz=bQ;dA=dB}else{cQ=0;break}}}}while(0);dB=c[550876]|0;if((dB|0)==0){i=b;return cQ|0}dA=c[550910]|0;if((dA|0)>0){dC=0;dD=dA;dE=dB}else{i=b;return cQ|0}while(1){dB=c[dE+(dC<<2)>>2]|0;if((dB|0)==0){dF=dD}else{if((c[dB+344>>2]|0)!=0){A6((c[(c[dB+12>>2]|0)+8>>2]|0)+96|0)}dA=dB+296|0;dz=c[dA>>2]|0;if((dz|0)!=0){aR(dz|0)|0;c[dA>>2]=0}A6((c[(c[dB+12>>2]|0)+8>>2]|0)+860|0);A6(dB+100|0);A6(dB+332|0);zL(dB+320|0);zL(dB+324|0);zL(dB+328|0);dF=c[550910]|0}dB=dC+1|0;if((dB|0)>=(dF|0)){break}dC=dB;dD=dF;dE=c[550876]|0}i=b;return cQ|0}function eL(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var j=0,k=0,l=0,m=0,n=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0.0,z=0,A=0,B=0,C=0,D=0.0,E=0,F=0,H=0.0,I=0,J=0,K=0,L=0.0,M=0,N=0,O=0,P=0,Q=0,R=0.0,S=0.0,T=0,U=0.0,V=0.0,W=0.0,X=0.0,Y=0.0,Z=0,_=0.0,$=0.0,ab=0.0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0.0,aj=0,ak=0;j=i;i=i+2064|0;k=j|0;l=j+1024|0;m=j+2048|0;n=j+2056|0;do{if((c[22694]|0)==0){if((b|0)!=0|(c[541010]|0)!=0){break}i=j;return}}while(0);p=(b|0)!=0;do{if(!p){q=c[22698]|0;r=c[22699]|0;if((q|0)==(-1|0)&(r|0)==(-1|0)){c[22698]=f;c[22699]=g;i=j;return}s=NV(f,g,q,r)|0;r=G;q=0;if((r|0)<(q|0)|(r|0)==(q|0)&s>>>0<5e5>>>0){i=j;return}else{c[22698]=f;c[22699]=g;break}}}while(0);s=(c[c[c[550878]>>2]>>2]|0)+16|0;q=ip(c[s>>2]|0)|0;r=G;t=0;if((r|0)<(t|0)|(r|0)==(t|0)&q>>>0<1>>>0){t=il(c[s>>2]|0,0,0,1)|0;u=G;v=t}else{u=r;v=q}q=k|0;a[q]=0;y9(l,0,1);if((c[550910]|0)>0){r=m|0;t=m+4|0;s=n|0;w=n+4|0;x=NV(f,g,d,e)|0;y=(+(x>>>0)+ +(G|0)*4294967296.0)/1.0e6;x=y>1.0;e=-2147483648;d=0;g=0;f=0;while(1){z=c[(c[550876]|0)+(g<<2)>>2]|0;A=z+12|0;B=c[(c[A>>2]|0)+8>>2]|0;do{if((c[z+344>>2]|0)==0){C=c[B+804>>2]|0;if((C|0)==0){D=-1.0;break}D=+(c[C+168>>2]|0)/118.0}else{D=-1.0}}while(0);C=(c[B+8>>2]|0)==0;do{if((f|0)==0){if(!C){E=0;break}F=c[z+20>>2]|0;if(x){H=+(F|0)/y}else{H=0.0}I=NL(q|0)|0;J=k+I|0;K=1024-I|0;I=H<9.95|0;L=D;bc(J|0,K|0,1634464,(M=i,i=i+32|0,c[M>>2]=F,c[M+8>>2]=I,h[M+16>>3]=H,h[M+24>>3]=L,M)|0)|0;i=M;zb(l,1633688,(M=i,i=i+8|0,c[M>>2]=F,M)|0);i=M;zb(l,1633240,(M=i,i=i+8|0,h[M>>3]=H,M)|0);i=M;I=z|0;K=z+4|0;J=c[K>>2]|0;zb(l,1635136,(M=i,i=i+24|0,c[M>>2]=c[I>>2],c[M+8>>2]=J,h[M+16>>3]=L,M)|0);i=M;if(p){J=NL(q|0)|0;N=k+J|0;O=1024-J|0;bc(N|0,O|0,1632160,(M=i,i=i+1|0,i=i+7&-8,c[M>>2]=0,M)|0)|0;i=M}if((c[541008]|0)!=0){O=bR(+D)|0;if((O|0)>-1&O>>>0<52){N=2164056+(O<<2)|0;c[N>>2]=(c[N>>2]|0)+1;P=0}else{P=0}do{N=NL(q|0)|0;O=bR(+(+bu(+(+((c[2164056+(P<<2)>>2]|0)+1|0)))))|0;bc(k+N|0,1024-N|0,1631360,(M=i,i=i+8|0,c[M>>2]=O,M)|0)|0;i=M;P=P+1|0;}while((P|0)<32)}if((c[B+88>>2]&32768|0)==0){E=1;break}O=B+804|0;if(!((c[O>>2]|0)!=0|p)){E=1;break}N=NL(q|0)|0;J=k+N|0;Q=1024-N|0;bc(J|0,Q|0,1630824,(M=i,i=i+1|0,i=i+7&-8,c[M>>2]=0,M)|0)|0;i=M;Q=B+120|0;J=B+124|0;L=+(F|0);if(p){N=0;R=0.0;S=0.0;while(1){T=B+720+(N<<3)|0;U=L*+(ac(c[J>>2]|0,c[Q>>2]|0)|0)*255.0*255.0;V=+((c[T>>2]|0)>>>0)+ +((c[T+4>>2]|0)>>>0)*4294967296.0;if((N|0)==0){W=U}else{W=U*.25}U=R+V;X=S+W;Y=+aa(+(V/W))*-10.0/2.302585092994046;T=NL(q|0)|0;Z=a[90784+N|0]|0;bc(k+T|0,1024-T|0,1630264,(M=i,i=i+16|0,c[M>>2]=Z,h[M+8>>3]=Y,M)|0)|0;i=M;T=c[K>>2]|0;zb(l,1629344,(M=i,i=i+32|0,c[M>>2]=c[I>>2],c[M+8>>2]=T,c[M+16>>2]=Z|32,h[M+24>>3]=Y,M)|0);i=M;Z=N+1|0;if((Z|0)<3){N=Z;R=U;S=X}else{_=U;$=X;break}}}else{N=0;S=0.0;R=0.0;while(1){F=(c[O>>2]|0)+224+(N<<3)|0;L=+(ac(c[J>>2]|0,c[Q>>2]|0)|0)*255.0*255.0;X=+((c[F>>2]|0)>>>0)+ +((c[F+4>>2]|0)>>>0)*4294967296.0;if((N|0)==0){ab=L}else{ab=L*.25}L=S+X;U=R+ab;Y=+aa(+(X/ab))*-10.0/2.302585092994046;F=NL(q|0)|0;Z=a[90784+N|0]|0;bc(k+F|0,1024-F|0,1630264,(M=i,i=i+16|0,c[M>>2]=Z,h[M+8>>3]=Y,M)|0)|0;i=M;F=c[K>>2]|0;zb(l,1629344,(M=i,i=i+32|0,c[M>>2]=c[I>>2],c[M+8>>2]=F,c[M+16>>2]=Z|32,h[M+24>>3]=Y,M)|0);i=M;Z=N+1|0;if((Z|0)<3){N=Z;S=L;R=U}else{_=L;$=U;break}}}R=+aa(+(_/$))*-10.0/2.302585092994046;N=NL(q|0)|0;Q=k+N|0;J=1024-N|0;bc(Q|0,J|0,1628672,(M=i,i=i+8|0,h[M>>3]=R,M)|0)|0;i=M;J=c[K>>2]|0;zb(l,1628224,(M=i,i=i+24|0,c[M>>2]=c[I>>2],c[M+8>>2]=J,h[M+16>>3]=R,M)|0);i=M;E=1}else{if(!C){E=f;break}J=NL(q|0)|0;Q=k+J|0;N=1024-J|0;R=D;bc(Q|0,N|0,1635656,(M=i,i=i+8|0,h[M>>3]=R,M)|0)|0;i=M;N=c[z+4>>2]|0;zb(l,1635136,(M=i,i=i+24|0,c[M>>2]=c[z>>2],c[M+8>>2]=N,h[M+16>>3]=R,M)|0);i=M;E=f}}while(0);if(p){ad=2288}else{if((c[z+336>>2]|0)==0){ad=2288}else{ae=e;af=d}}do{if((ad|0)==2288){ad=0;C=c[A>>2]|0;B=C+16|0;N=c[B>>2]|0;Q=c[B+4>>2]|0;if((N|0)==0&(Q|0)==(-2147483648|0)){ae=e;af=d;break}c[r>>2]=1;c[t>>2]=1e6;B=AY(N,Q,C+40|0,m)|0;C=G;if((e|0)>(C|0)|(e|0)==(C|0)&d>>>0>B>>>0){ae=e;af=d;break}B=c[A>>2]|0;C=B+16|0;Q=c[C>>2]|0;N=c[C+4>>2]|0;c[s>>2]=1;c[w>>2]=1e6;C=AY(Q,N,B+40|0,n)|0;ae=G;af=C}}while(0);A=g+1|0;if((A|0)<(c[550910]|0)){e=ae;d=af;g=A;f=E}else{ag=ae;ah=af;break}}}else{ag=-2147483648;ah=0}af=N0(ah,ag,1e6,0)|0;ae=af;af=N1(ah,ag,1e6,0)|0;E=af;af=(ae|0)%60|0;f=(ae|0)/3600|0;g=((ae|0)/60|0|0)%60|0;ae=-1;if(((ah|0)!=0|(ag|0)!=0)&((u|0)>(ae|0)|(u|0)==(ae|0)&v>>>0>-1>>>0)){ai=(+((v<<3|0>>>29)>>>0)+ +(u<<3|v>>>29|0)*4294967296.0)/((+(ah>>>0)+ +(ag|0)*4294967296.0)/1.0e3)}else{ai=-1.0}ae=0;d=(u|0)<(ae|0)|(u|0)==(ae|0)&v>>>0<0>>>0;ae=NL(q|0)|0;e=k+ae|0;n=1024-ae|0;if(d){bc(e|0,n|0,1627760,(M=i,i=i+1|0,i=i+7&-8,c[M>>2]=0,M)|0)|0;i=M}else{D=(+(v>>>0)+ +(u|0)*4294967296.0)*.0009765625;bc(e|0,n|0,1627192,(M=i,i=i+8|0,h[M>>3]=D,M)|0)|0;i=M}n=NL(q|0)|0;bc(k+n|0,1024-n|0,1626656,(M=i,i=i+32|0,c[M>>2]=f,c[M+8>>2]=g,c[M+16>>2]=af,c[M+24>>2]=(E*100|0|0)/1e6|0,M)|0)|0;i=M;n=NL(q|0)|0;e=k+n|0;ae=1024-n|0;if(ai<0.0){bc(e|0,ae|0,1625696,(M=i,i=i+1|0,i=i+7&-8,c[M>>2]=0,M)|0)|0;i=M}else{bc(e|0,ae|0,1624928,(M=i,i=i+8|0,h[M>>3]=ai,M)|0)|0;i=M}if(d){zb(l,1624368,(M=i,i=i+1|0,i=i+7&-8,c[M>>2]=0,M)|0);i=M}else{zb(l,1623928,(M=i,i=i+16|0,c[M>>2]=v,c[M+8>>2]=u,M)|0);i=M}zb(l,1623176,(M=i,i=i+16|0,c[M>>2]=ah,c[M+8>>2]=ag,M)|0);i=M;zb(l,1622568,(M=i,i=i+32|0,c[M>>2]=f,c[M+8>>2]=g,c[M+16>>2]=af,c[M+24>>2]=E,M)|0);i=M;E=c[550918]|0;af=c[550920]|0;if((af|E|0)==0){aj=E}else{g=NL(q|0)|0;f=k+g|0;k=1024-g|0;bc(f|0,k|0,1622176,(M=i,i=i+16|0,c[M>>2]=E,c[M+8>>2]=af,M)|0)|0;i=M;aj=c[550918]|0}zb(l,1621840,(M=i,i=i+8|0,c[M>>2]=aj,M)|0);i=M;zb(l,1621480,(M=i,i=i+8|0,c[M>>2]=c[550920],M)|0);i=M;aj=c[22694]|0;if((aj|b|0)!=0){do{if((aj|0)==1){if((AO()|0)>=32){ad=2309;break}b=c[o>>2]|0;cx(b|0,1620096,(M=i,i=i+8|0,c[M>>2]=q,M)|0)|0;i=M;ak=b}else{ad=2309}}while(0);if((ad|0)==2309){AN(0,32,1620096,(M=i,i=i+8|0,c[M>>2]=q,M)|0);i=M;ak=c[o>>2]|0}aT(ak|0)|0}do{if((c[541010]|0)==0){if(p){break}i=j;return}else{zb(l,1619224,(M=i,i=i+8|0,c[M>>2]=p?1618888:1618496,M)|0);i=M;ak=c[l+4>>2]|0;q=(c[l+8>>2]|0)-1|0;ij(c[541010]|0,c[l>>2]|0,ak>>>0>q>>>0?q:ak);ik(c[541010]|0);zd(l,0)|0;if(p){ak=c[541010]|0;iV(ak)|0;c[541010]=0;break}else{i=j;return}}}while(0);p=NU(c[499514]|0,c[499515]|0,c[767226]|0,c[767227]|0)|0;l=NU(p,G,c[540562]|0,c[540563]|0)|0;p=NU(l,G,c[727194]|0,c[727195]|0)|0;l=G;AN(0,32,1618104,(M=i,i=i+1|0,i=i+7&-8,c[M>>2]=0,M)|0);i=M;ai=(+((c[499514]|0)>>>0)+ +(c[499515]|0)*4294967296.0)*.0009765625;D=(+((c[767226]|0)>>>0)+ +(c[767227]|0)*4294967296.0)*.0009765625;$=(+((c[540562]|0)>>>0)+ +(c[540563]|0)*4294967296.0)*.0009765625;_=(+((c[727194]|0)>>>0)+ +(c[727195]|0)*4294967296.0)*.0009765625;ak=NV(v,u,p,l)|0;ab=(+(ak>>>0)+ +(G|0)*4294967296.0)*100.0/(+(p>>>0)+ +(l|0)*4294967296.0);AN(0,32,1617528,(M=i,i=i+40|0,h[M>>3]=ai,h[M+8>>3]=D,h[M+16>>3]=$,h[M+24>>3]=_,h[M+32>>3]=ab,M)|0);i=M;l=NU(c[767226]|0,c[767227]|0,c[499514]|0,c[499515]|0)|0;p=NU(l,G,c[540562]|0,c[540563]|0)|0;l=G;ak=NV(0,0,c[727194]|0,c[727195]|0)|0;if(!((p|0)==(ak|0)&(l|0)==(G|0))){i=j;return}AN(0,24,1617032,(M=i,i=i+1|0,i=i+7&-8,c[M>>2]=0,M)|0);i=M;i=j;return}function eM(b,e){b=b|0;e=e|0;var f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0.0,aa=0,ac=0,ad=0,ae=0,af=0,ag=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0;f=i;i=i+1048|0;g=f|0;j=f+8|0;k=f+16|0;l=f+96|0;m=f+104|0;n=f+112|0;o=f+120|0;p=f+128|0;q=f+136|0;r=f+168|0;s=f+200|0;t=f+208|0;u=f+216|0;v=f+248|0;w=f+280|0;x=f+312|0;y=f+568|0;z=f+576|0;A=f+640|0;B=f+704|0;C=f+712|0;D=f+720|0;E=f+728|0;F=f+736|0;H=f+744|0;I=f+752|0;J=f+816|0;K=f+896|0;L=f+904|0;M=f+912|0;N=f+920|0;O=f+1e3|0;P=f+1008|0;S=f+1016|0;T=f+1024|0;U=f+1032|0;V=f+1040|0;c[M>>2]=0;W=b+100|0;if((c[W>>2]|0)==0){X=c[b+4>>2]|0;Y=c[X+92>>2]|0;if((Y|0)==0){Z=0;_=0}else{$=+((c[(c[X+8>>2]|0)+176>>2]|0)*-1e6|0|0)/(+(Y|0)/+(c[X+96>>2]|0));Z=+R($)>=1.0?$>0.0?(ah(+Q($/4294967296.0),4294967295.0)|0)>>>0:~~+ab(($- +(~~$>>>0))/4294967296.0)>>>0:0;_=~~$>>>0}Y=b+48|0;c[Y>>2]=_;c[Y+4>>2]=Z;Z=b+64|0;c[Z>>2]=0;c[Z+4>>2]=0;do{if((e|0)!=0){_=e+8|0;aa=c[_>>2]|0;ac=c[_+4>>2]|0;if((aa|0)==0&(ac|0)==(-2147483648|0)){break}if((c[b+12>>2]|0)!=0){break}c[O>>2]=1;c[O+4>>2]=1e6;_=AY(aa,ac,X+40|0,O)|0;ac=NU(c[Y>>2]|0,c[Y+4>>2]|0,_,G)|0;_=G;c[Y>>2]=ac;c[Y+4>>2]=_;c[Z>>2]=ac;c[Z+4>>2]=_}}while(0);c[W>>2]=1}W=b+40|0;if((c[W>>2]|0)==0&(c[W+4>>2]|0)==(-2147483648|0)){Z=b+48|0;Y=c[Z+4>>2]|0;c[W>>2]=c[Z>>2];c[W+4>>2]=Y}Y=b+56|0;if((c[Y>>2]|0)==0&(c[Y+4>>2]|0)==(-2147483648|0)){Z=b+64|0;O=c[Z+4>>2]|0;c[Y>>2]=c[Z>>2];c[Y+4>>2]=O}O=(e|0)==0;do{if(O){nB(N);c[N+24>>2]=0;c[N+28>>2]=0;ad=0}else{Z=N;X=e;NN(Z|0,X|0,80)|0;X=e+16|0;Z=c[X>>2]|0;_=c[X+4>>2]|0;if((Z|0)==0&(_|0)==(-2147483648|0)){ae=2343;break}X=b+4|0;ac=(c[X>>2]|0)+40|0;c[P>>2]=1;c[P+4>>2]=1e6;aa=AY(Z,_,ac,P)|0;ac=G;_=b+48|0;c[_>>2]=aa;c[_+4>>2]=ac;c[W>>2]=aa;c[W+4>>2]=ac;if((c[(c[(c[X>>2]|0)+8>>2]|0)+8>>2]|0)==0){if((c[b+12>>2]|0)!=0){ae=2343;break}}X=b+64|0;c[X>>2]=aa;c[X+4>>2]=ac;c[Y>>2]=aa;c[Y+4>>2]=ac;ae=2343}}while(0);L3243:while(1){if((ae|0)==2343){ae=0;if((c[b+12>>2]|0)==0){ae=2510;break}P=c[N+28>>2]|0;if((P|0)>0){ad=P;continue}if(!O){break}if((c[M>>2]|0)==0){af=0;ae=2569;break}else{ad=P}}P=c[Y+4>>2]|0;ac=b+64|0;c[ac>>2]=c[Y>>2];c[ac+4>>2]=P;P=c[W+4>>2]|0;aa=b+48|0;c[aa>>2]=c[W>>2];c[aa+4>>2]=P;P=N+28|0;do{if((ad|0)!=0){if((ad|0)==(c[e+28>>2]|0)){break}X=b+104|0;AN(0,(c[X>>2]|0)!=0?40:24,1664760,(ag=i,i=i+8|0,c[ag>>2]=c[e+32>>2],ag)|0);i=ag;c[X>>2]=1}}while(0);X=b+4|0;_=c[(c[X>>2]|0)+8>>2]|0;Z=c[_+8>>2]|0;L3254:do{if((Z|0)==3){ai=q;aj=r;ak=pi(_,q,M,N)|0;al=c[M>>2]|0;am=(ak|0)<0;if((al|0)!=0|am){ae=2467}else{if((c[P>>2]|0)!=0){ae=2467}}if((ae|0)==2467){ae=0;an=2981168+(ak>>>31<<3)|0;ao=NU(c[an>>2]|0,c[an+4>>2]|0,1,0)|0;c[an>>2]=ao;c[an+4>>2]=G}if(am|(al|0)==0){if((c[P>>2]|0)!=0){ap=ak;break}am=b+248|0;if((c[am>>2]|0)<=0){ap=ak;break}an=b+244|0;ao=0;while(1){gB(c[c[(c[an>>2]|0)+(ao<<2)>>2]>>2]|0,0,0)|0;aq=ao+1|0;if((aq|0)<(c[am>>2]|0)){ao=aq}else{ap=ak;break L3254}}}if((c[b+160>>2]|0)==0){ar=ak}else{ao=b+168|0;am=c[ao>>2]|0;do{if((am|0)==0){as=0;at=al}else{an=q+24|0;aq=b+200|0;au=NV(c[an>>2]|0,c[an+4>>2]|0,c[aq>>2]|0,c[aq+4>>2]|0)|0;aq=AW(au,G,1e3,0,1e6,0)|0;au=aq;aq=b+184|0;an=c[aq>>2]|0;if(au>>>0>=an>>>0){as=am;at=al;break}AN(c[(c[X>>2]|0)+8>>2]|0,48,1663408,(ag=i,i=i+16|0,c[ag>>2]=an,c[ag+8>>2]=au,ag)|0);i=ag;c[aq>>2]=au;as=c[ao>>2]|0;at=c[M>>2]|0}}while(0);c[ao>>2]=at;c[M>>2]=as;al=b+172|0;am=c[al>>2]|0;c[al>>2]=ak;al=b+176|0;c[aj>>2]=c[al>>2];c[aj+4>>2]=c[al+4>>2];c[aj+8>>2]=c[al+8>>2];c[aj+12>>2]=c[al+12>>2];c[aj+16>>2]=c[al+16>>2];c[aj+20>>2]=c[al+20>>2];c[aj+24>>2]=c[al+24>>2];c[aj+28>>2]=c[al+28>>2];c[al>>2]=c[ai>>2];c[al+4>>2]=c[ai+4>>2];c[al+8>>2]=c[ai+8>>2];c[al+12>>2]=c[ai+12>>2];c[al+16>>2]=c[ai+16>>2];c[al+20>>2]=c[ai+20>>2];c[al+24>>2]=c[ai+24>>2];c[al+28>>2]=c[ai+28>>2];c[ai>>2]=c[aj>>2];c[ai+4>>2]=c[aj+4>>2];c[ai+8>>2]=c[aj+8>>2];c[ai+12>>2]=c[aj+12>>2];c[ai+16>>2]=c[aj+16>>2];c[ai+20>>2]=c[aj+20>>2];c[ai+24>>2]=c[aj+24>>2];c[ai+28>>2]=c[aj+28>>2];ar=am}eQ(b,q);if((c[M>>2]|0)==0){ap=ar;break}am=q+12|0;if((c[am>>2]|0)==0){ap=ar;break}if((c[550910]|0)>0){al=b|0;au=q+24|0;aq=l|0;an=l+4|0;av=q+4|0;aw=m|0;ax=m+4|0;ay=n|0;az=n+4|0;aA=q+8|0;aB=k+24|0;aC=k+28|0;aD=o|0;aE=o+4|0;aF=k+8|0;aG=p|0;aH=p+4|0;aI=k+48|0;aJ=j|0;aK=j+4|0;aL=g|0;aM=g+4|0;aN=0;do{aO=c[(c[550876]|0)+(aN<<2)>>2]|0;L3281:do{if((c[aO+8>>2]|0)==((c[c[X>>2]>>2]|0)+(c[(c[(c[551810]|0)+(c[al>>2]<<2)>>2]|0)+12>>2]|0)|0)){aP=aO|0;aQ=c[(c[550878]|0)+(c[aP>>2]<<2)>>2]|0;aR=aQ+24|0;aS=c[aR>>2]|0;aT=c[aR+4>>2]|0;if(!((aS|0)==0&(aT|0)==0)){aR=c[ac+4>>2]|0;if((aR|0)<(aT|0)|(aR|0)==(aT|0)&(c[ac>>2]|0)>>>0<aS>>>0){break}}if((c[aO+16>>2]|0)==0){break}aR=c[aQ>>2]|0;aQ=c[au>>2]|0;aU=c[au+4>>2]|0;if((aQ|0)==0&(aU|0)==(-2147483648|0)){AN(0,16,1661840,(ag=i,i=i+1|0,i=i+7&-8,c[ag>>2]=0,ag)|0);i=ag;if((c[744092]|0)==0){break}eW(1);break}aV=aO+12|0;aW=c[(c[aV>>2]|0)+8>>2]|0;if((c[540564]|0)==0){c[540564]=A1(1048576)|0;aX=(c[(c[550878]|0)+(c[aP>>2]<<2)>>2]|0)+24|0;aY=c[au+4>>2]|0;aZ=c[au>>2]|0;a_=c[aX+4>>2]|0;a$=c[aX>>2]|0}else{aY=aU;aZ=aQ;a_=aT;a$=aS}aS=aW+48|0;aT=(c[aS>>2]|0)==94209?2:1;aQ=NV(aZ,aY,a$,a_)|0;aU=G;aX=aW+104|0;a0=aO+32|0;a1=aO+40|0;a2=0;while(1){c[aq>>2]=1;c[an>>2]=1e6;a3=AY(aQ,aU,l,aX)|0;a4=G;c[a0>>2]=a3;c[a0+4>>2]=a4;a5=(c[(c[550878]|0)+(c[aP>>2]<<2)>>2]|0)+16|0;a6=c[a5>>2]|0;a7=c[a5+4>>2]|0;if(!((a6|0)==(-1|0)&(a7|0)==2147483647)){a5=NV(a3,a4,c[a1>>2]|0,c[a1+4>>2]|0)|0;a4=(c[(c[aV>>2]|0)+8>>2]|0)+104|0;c[aJ>>2]=1;c[aK>>2]=1e6;if((AZ(a5,G,a4,a6,a7,j)|0)>-1){break}}c[au>>2]=aQ;c[au+4>>2]=aU;a7=c[av>>2]|0;c[aw>>2]=1;c[ax>>2]=1e3;c[ay>>2]=1;c[az>>2]=1e6;a6=AY(a7,0,m,n)|0;a7=NU(c[au>>2]|0,c[au+4>>2]|0,a6,G)|0;c[au>>2]=a7;c[au+4>>2]=G;c[aA>>2]=(c[aA>>2]|0)-(c[av>>2]|0);c[av>>2]=0;if((a2|0)==1){c[am>>2]=0}a7=pe(aW,c[540564]|0,1048576,q)|0;if((a7|0)<0){AN(0,8,1660464,(ag=i,i=i+1|0,i=i+7&-8,c[ag>>2]=0,ag)|0);i=ag;eW(1)}nB(k);c[aB>>2]=c[540564];c[aC>>2]=a7;a7=c[au>>2]|0;a6=c[au+4>>2]|0;c[aD>>2]=1;c[aE>>2]=1e6;a4=AY(a7,a6,o,(c[aV>>2]|0)+40|0)|0;c[aF>>2]=a4;c[aF+4>>2]=G;a4=c[aA>>2]|0;c[aG>>2]=1;c[aH>>2]=1e3;a6=AY(a4,0,p,(c[aV>>2]|0)+40|0)|0;c[aI>>2]=a6;do{if((c[aS>>2]|0)==94209){if((a2|0)==0){a6=NU((c[av>>2]|0)*90|0,0,c[aF>>2]|0,c[aF+4>>2]|0)|0;c[aF>>2]=a6;c[aF+4>>2]=G;break}else{a6=NU((c[aA>>2]|0)*90|0,0,c[aF>>2]|0,c[aF+4>>2]|0)|0;c[aF>>2]=a6;c[aF+4>>2]=G;break}}}while(0);a6=c[aC>>2]|0;a4=NU(c[540562]|0,c[540563]|0,a6,(a6|0)<0|0?-1:0)|0;c[540562]=a4;c[540563]=G;eO(aR,k,aO);a2=a2+1|0;if((a2|0)>=(aT|0)){break L3281}}aT=c[(c[550878]|0)+(c[aP>>2]<<2)>>2]|0;c[aO+336>>2]=1;if((c[aT+40>>2]|0)==0){break}a2=NV(c[a0>>2]|0,c[a0+4>>2]|0,c[a1>>2]|0,c[a1+4>>2]|0)|0;aR=(c[(c[aV>>2]|0)+8>>2]|0)+104|0;c[aL>>2]=1;c[aM>>2]=1e6;aS=AY(a2,G,aR,g)|0;aR=G;a2=aT+16|0;aT=c[a2>>2]|0;aW=c[a2+4>>2]|0;aU=(aW|0)>(aR|0)|(aW|0)==(aR|0)&aT>>>0>aS>>>0;c[a2>>2]=aU?aS:aT;c[a2+4>>2]=aU?aR:aW}}while(0);aN=aN+1|0;}while((aN|0)<(c[550910]|0))}pj(q);ap=ar}else if((Z|0)==0){aN=u|0;aM=v|0;aL=w|0;aC=z|0;aF=b+20|0;aA=c[aF>>2]|0;if((aA|0)==0){av=As()|0;c[aF>>2]=av;if((av|0)==0){a8=-12}else{a9=av;ae=2405}}else{a9=aA;ae=2405}L3318:do{if((ae|0)==2405){ae=0;aA=b+24|0;if((c[aA>>2]|0)==0){av=As()|0;c[aA>>2]=av;if((av|0)==0){a8=-12;break}ba=c[aF>>2]|0}else{ba=a9}av=c[aa>>2]|0;aI=c[aa+4>>2]|0;c[s>>2]=1;c[s+4>>2]=1e6;aH=AY(av,aI,s,(c[X>>2]|0)+40|0)|0;aI=N+16|0;c[aI>>2]=aH;c[aI+4>>2]=G;eN(0,(ag=i,i=i+1|0,i=i+7&-8,c[ag>>2]=0,ag)|0);i=ag;aI=pf(c[(c[X>>2]|0)+8>>2]|0,ba,M,N)|0;aH=b|0;av=c[c[X>>2]>>2]|0;eN(1654120,(ag=i,i=i+16|0,c[ag>>2]=c[aH>>2],c[ag+8>>2]=av,ag)|0);i=ag;av=c[M>>2]|0;aG=(aI|0)<0;if((av|0)!=0|aG){ae=2410}else{if((c[P>>2]|0)!=0){ae=2410}}do{if((ae|0)==2410){ae=0;aE=2981168+(aI>>>31<<3)|0;aD=NU(c[aE>>2]|0,c[aE+4>>2]|0,1,0)|0;c[aE>>2]=aD;c[aE+4>>2]=G;if((av|0)==0|aG){if((c[P>>2]|0)==0){break}else{a8=aI;break L3318}}aE=c[b+120>>2]|0;if((aE|0)>-1){c[ba+300>>2]=aE}aE=Ag(ba)|0;aD=G;au=(aE|0)==0&(aD|0)==(-2147483648|0);if(!au){aB=ba+136|0;c[aB>>2]=aE;c[aB+4>>2]=aD;aB=(c[X>>2]|0)+40|0;c[t>>2]=1;c[t+4>>2]=1e6;am=AY(aE,aD,aB,t)|0;aB=G;c[ac>>2]=am;c[ac+4>>2]=aB;c[Y>>2]=am;c[Y+4>>2]=aB}if((c[745296]|0)!=0){aB=c[c[X>>2]>>2]|0;NM(aN|0,0,32);a[aN]=0;am=ba+136|0;az=c[am>>2]|0;ay=c[am+4>>2]|0;if((az|0)==0&(ay|0)==(-2147483648|0)){bc(aN|0,32,1670672,(ag=i,i=i+1|0,i=i+7&-8,c[ag>>2]=0,ag)|0)|0;i=ag}else{ax=1667440;bc(aN|0,32,ax|0,(ag=i,i=i+16|0,c[ag>>2]=az,c[ag+8>>2]=ay,ag)|0)|0;i=ag}NM(aM|0,0,32);a[aM]=0;ay=c[am>>2]|0;az=c[am+4>>2]|0;am=c[X>>2]|0;if((ay|0)==0&(az|0)==(-2147483648|0)){bc(aM|0,32,1670672,(ag=i,i=i+1|0,i=i+7&-8,c[ag>>2]=0,ag)|0)|0;i=ag}else{$=(+(ay>>>0)+ +(az|0)*4294967296.0)*(+(c[am+40>>2]|0)/+(c[am+44>>2]|0));bc(aM|0,32,1668840,(ag=i,i=i+8|0,h[ag>>3]=$,ag)|0)|0;i=ag}NM(aL|0,0,32);a[aL]=0;am=c[X>>2]|0;if(au){bc(aL|0,32,1670672,(ag=i,i=i+1|0,i=i+7&-8,c[ag>>2]=0,ag)|0)|0;i=ag}else{$=(+(aE>>>0)+ +(aD|0)*4294967296.0)*(+(c[am+40>>2]|0)/+(c[am+44>>2]|0));bc(aL|0,32,1668840,(ag=i,i=i+8|0,h[ag>>3]=$,ag)|0)|0;i=ag}am=c[ba+84>>2]|0;au=c[ba+88>>2]|0;AN(0,32,1652488,(ag=i,i=i+64|0,c[ag>>2]=aB,c[ag+8>>2]=aN,c[ag+16>>2]=aM,c[ag+24>>2]=aE,c[ag+32>>2]=aD,c[ag+40>>2]=aL,c[ag+48>>2]=am,c[ag+56>>2]=au,ag)|0);i=ag}c[P>>2]=0;au=(c[X>>2]|0)+80|0;if((c[au>>2]|0)!=0){am=au;au=ba+124|0;aD=c[am+4>>2]|0;c[au>>2]=c[am>>2];c[au+4>>2]=aD}aD=b+132|0;au=c[aD>>2]|0;am=ba+68|0;aE=b+128|0;aB=c[aE>>2]|0;az=ba+72|0;do{if((au|0)==(c[am>>2]|0)){if((aB|0)!=(c[az>>2]|0)){ae=2435;break}if((c[b+136>>2]|0)!=(c[ba+80>>2]|0)){ae=2435}}else{ae=2435}}while(0);do{if((ae|0)==2435){ae=0;aV=c[aH>>2]|0;a1=c[c[X>>2]>>2]|0;a0=b+136|0;aP=BL(c[a0>>2]|0)|0;ay=c[am>>2]|0;ax=c[az>>2]|0;aw=ba+80|0;aK=BL(c[aw>>2]|0)|0;AN(0,32,1650728,(ag=i,i=i+64|0,c[ag>>2]=aV,c[ag+8>>2]=a1,c[ag+16>>2]=au,c[ag+24>>2]=aB,c[ag+32>>2]=aP,c[ag+40>>2]=ay,c[ag+48>>2]=ax,c[ag+56>>2]=aK,ag)|0);i=ag;c[aD>>2]=c[am>>2];c[aE>>2]=c[az>>2];c[a0>>2]=c[aw>>2];if((c[550922]|0)<=0){break}aw=b+252|0;a0=0;do{do{if((ez(c[(c[556308]|0)+(a0<<2)>>2]|0,b)|0)!=0){if((c[aw>>2]|0)==0){break}if((ey(c[(c[556308]|0)+(a0<<2)>>2]|0)|0)>=0){break}AN(0,8,1649200,(ag=i,i=i+1|0,i=i+7&-8,c[ag>>2]=0,ag)|0);i=ag;eW(1)}}while(0);a0=a0+1|0;}while((a0|0)<(c[550922]|0))}}while(0);az=Bx(oy()|0,ba,1646616)|0;aE=b+248|0;am=c[aE>>2]|0;L3369:do{if((am|0)>0){aD=az;aB=az;au=b+244|0;a0=aI;aw=0;aK=0;ax=am;while(1){if((c[aD>>2]|0)==0){ay=(c[X>>2]|0)+80|0;aP=c[ay+4>>2]|0;c[aB>>2]=c[ay>>2];c[aB+4>>2]=aP;bb=c[aE>>2]|0}else{bb=ax}if((aK|0)<(bb-1|0)){aP=c[aA>>2]|0;ay=Aw(aP,ba)|0;if((ay|0)<0){bd=ay;be=a0;break L3369}else{bf=aP;bg=ay}}else{bf=ba;bg=aw}ay=gz(c[c[(c[au>>2]|0)+(aK<<2)>>2]>>2]|0,bf,4)|0;do{if((ay|0)==-541478725){bh=0}else{if((ay|0)>=0){bh=ay;break}NM(aC|0,0,64);a[aC]=0;zN(ay,aC,64)|0;AN(0,8,1645048,(ag=i,i=i+8|0,c[ag>>2]=aC,ag)|0);i=ag;eW(1);bh=ay}}while(0);ay=aK+1|0;aP=c[aE>>2]|0;if((ay|0)<(aP|0)){a0=bh;aw=bg;aK=ay;ax=aP}else{bd=bg;be=bh;break}}}else{bd=0;be=aI}}while(0);Au(c[aA>>2]|0);Au(ba);A4(0);a8=(bd|0)<0?bd:be;break L3318}}while(0);aA=b+248|0;if((c[aA>>2]|0)<=0){a8=aI;break}aH=b+244|0;aG=0;while(1){gB(c[c[(c[aH>>2]|0)+(aG<<2)>>2]>>2]|0,0,0)|0;av=aG+1|0;if((av|0)<(c[aA>>2]|0)){aG=av}else{a8=aI;break}}}}while(0);aC=c[N+48>>2]|0;do{if((aC|0)==0){aL=c[X>>2]|0;aM=c[aL+8>>2]|0;aN=c[aM+104>>2]|0;if((aN|0)==0){bi=-2147483648;bj=0;bk=0;break}aF=c[aM+108>>2]|0;if((aF|0)==0){bi=-2147483648;bj=0;bk=0;break}aI=c[aL+240>>2]|0;if((aI|0)==0){bl=c[aM+112>>2]|0}else{bl=(c[aI+36>>2]|0)+1|0}aI=N2(bl,(bl|0)<0|0?-1:0,1e6,0)|0;aM=N2(aI,G,aN,(aN|0)<0|0?-1:0)|0;aN=N0(aM,G,aF,(aF|0)<0|0?-1:0)|0;bm=aN;ae=2461}else{aN=(c[X>>2]|0)+40|0;c[S>>2]=1;c[S+4>>2]=1e6;aF=AY(aC,(aC|0)<0|0?-1:0,aN,S)|0;bm=aF;ae=2461}}while(0);do{if((ae|0)==2461){ae=0;if((c[aa>>2]|0)==0&(c[aa+4>>2]|0)==(-2147483648|0)|(bm|0)==0){bi=-2147483648;bj=0;bk=bm;break}aC=NU(c[W>>2]|0,c[W+4>>2]|0,bm,(bm|0)<0|0?-1:0)|0;bi=G;bj=aC;bk=bm}}while(0);c[W>>2]=bj;c[W+4>>2]=bi;if((c[M>>2]|0)==0){ap=a8;break}aC=NU(c[Y>>2]|0,c[Y+4>>2]|0,bk,(bk|0)<0|0?-1:0)|0;c[Y>>2]=aC;c[Y+4>>2]=G;ap=a8}else if((Z|0)==1){aC=z|0;aF=A|0;aN=y;aM=b+20|0;aI=c[aM>>2]|0;if((aI|0)==0){aL=o0()|0;c[aM>>2]=aL;if((aL|0)==0){af=-12;ae=2571;break L3243}else{bn=aL}}else{bn=aI}aI=b+24|0;if((c[aI>>2]|0)==0){aL=As()|0;c[aI>>2]=aL;if((aL|0)==0){af=-12;ae=2567;break L3243}bo=c[aM>>2]|0}else{bo=bn}eN(0,(ag=i,i=i+1|0,i=i+7&-8,c[ag>>2]=0,ag)|0);i=ag;aM=ph(_,bo,M,N)|0;aL=b|0;aG=c[c[X>>2]>>2]|0;eN(1643296,(ag=i,i=i+16|0,c[ag>>2]=c[aL>>2],c[ag+8>>2]=aG,ag)|0);i=ag;do{if((aM|0)>-1){aG=c[_+412>>2]|0;if((aG|0)<1){AN(_,16,1640728,(ag=i,i=i+8|0,c[ag>>2]=aG,ag)|0);i=ag;bp=-1094995529;ae=2359;break}aG=c[M>>2]|0;if((aG|0)!=0){bq=aM;br=0;bs=aG;ae=2362;break}if((c[P>>2]|0)==0){bt=aM}else{bq=aM;br=0;bs=0;ae=2362}}else{bp=aM;ae=2359}}while(0);if((ae|0)==2359){ae=0;bq=bp;br=(bp|0)<0;bs=c[M>>2]|0;ae=2362}do{if((ae|0)==2362){ae=0;aM=2981168+(bq>>>31<<3)|0;aG=NU(c[aM>>2]|0,c[aM+4>>2]|0,1,0)|0;c[aM>>2]=aG;c[aM+4>>2]=G;if((bs|0)==0|br){if((c[P>>2]|0)==0){bt=bq;break}else{ap=bq;break L3254}}aM=bo+76|0;aG=c[aM>>2]|0;aA=N2(aG,(aG|0)<0|0?-1:0,1e6,0)|0;aG=_+412|0;aH=c[aG>>2]|0;av=N0(aA,G,aH,(aH|0)<0|0?-1:0)|0;aH=NU(c[Y>>2]|0,c[Y+4>>2]|0,av,G)|0;c[Y>>2]=aH;c[Y+4>>2]=G;aH=c[aM>>2]|0;av=N2(aH,(aH|0)<0|0?-1:0,1e6,0)|0;aH=c[aG>>2]|0;aG=N0(av,G,aH,(aH|0)<0|0?-1:0)|0;aH=NU(c[W>>2]|0,c[W+4>>2]|0,aG,G)|0;c[W>>2]=aH;c[W+4>>2]=G;aH=b+140|0;aG=bo+80|0;do{if((c[aH>>2]|0)==(c[aG>>2]|0)){if((c[b+148>>2]|0)!=(c[_+416>>2]|0)){ae=2371;break}av=b+152|0;aA=bo+352|0;if(!((c[av>>2]|0)==(c[aA>>2]|0)&(c[av+4>>2]|0)==(c[aA+4>>2]|0))){ae=2371;break}if((c[b+144>>2]|0)!=(c[bo+344>>2]|0)){ae=2371}}else{ae=2371}}while(0);do{if((ae|0)==2371){ae=0;aA=x|0;av=c[X>>2]|0;aE=c[av+8>>2]|0;am=aE+448|0;L3430:do{if((c[am>>2]|0)==0&(c[am+4>>2]|0)==0){az=aE+416|0;ax=c[az>>2]|0;do{if((ax|0)>(c[b+124>>2]|0)){bu=av}else{aK=zu(ax)|0;aw=G;c[am>>2]=aK;c[am+4>>2]=aw;if((aK|0)==0&(aw|0)==0){bu=c[X>>2]|0;break}else{zt(aA,256,c[az>>2]|0,aK,aw);aw=c[c[X>>2]>>2]|0;AN(0,24,1617432,(ag=i,i=i+24|0,c[ag>>2]=c[aL>>2],c[ag+8>>2]=aw,c[ag+16>>2]=aA,ag)|0);i=ag;break L3430}}}while(0);az=c[bu>>2]|0;AN(0,8,1639176,(ag=i,i=i+16|0,c[ag>>2]=c[aL>>2],c[ag+8>>2]=az,ag)|0);i=ag;eW(1)}}while(0);aA=_+448|0;am=c[aA+4>>2]|0;av=bo+352|0;c[av>>2]=c[aA>>2];c[av+4>>2]=am;am=b+148|0;aA=b+152|0;zt(aC,64,c[am>>2]|0,c[aA>>2]|0,c[aA+4>>2]|0);aE=_+416|0;zt(aF,64,c[aE>>2]|0,c[av>>2]|0,c[av+4>>2]|0);az=c[aL>>2]|0;ax=c[c[X>>2]>>2]|0;aw=b+144|0;aK=c[aw>>2]|0;a0=B2(c[aH>>2]|0)|0;au=c[am>>2]|0;aB=bo+344|0;aD=c[aB>>2]|0;aP=B2(c[aG>>2]|0)|0;ay=c[aE>>2]|0;AN(0,32,1637336,(ag=i,i=i+80|0,c[ag>>2]=az,c[ag+8>>2]=ax,c[ag+16>>2]=aK,c[ag+24>>2]=a0,c[ag+32>>2]=au,c[ag+40>>2]=aC,c[ag+48>>2]=aD,c[ag+56>>2]=aP,c[ag+64>>2]=ay,c[ag+72>>2]=aF,ag)|0);i=ag;c[aH>>2]=c[aG>>2];c[aw>>2]=c[aB>>2];aB=c[av+4>>2]|0;c[aA>>2]=c[av>>2];c[aA+4>>2]=aB;c[am>>2]=c[aE>>2];if((c[550922]|0)>0){bv=0}else{break}do{do{if((ez(c[(c[556308]|0)+(bv<<2)>>2]|0,b)|0)!=0){aE=c[(c[556308]|0)+(bv<<2)>>2]|0;if((ey(aE)|0)<0){AN(0,8,1649200,(ag=i,i=i+1|0,i=i+7&-8,c[ag>>2]=0,ag)|0);i=ag;eW(1)}am=aE+28|0;aB=c[am>>2]|0;if((aB|0)<=0){break}aA=aE+24|0;aE=0;av=aB;while(1){aB=c[(c[(c[aA>>2]|0)+(aE<<2)>>2]|0)+4>>2]|0;aw=c[aB+60>>2]|0;do{if((c[aw+8>>2]|0)==1){if((c[aw+16>>2]&65536|0)!=0){bw=av;break}gr(c[c[aB+300>>2]>>2]|0,c[(c[(c[aB+12>>2]|0)+8>>2]|0)+424>>2]|0);bw=c[am>>2]|0}else{bw=av}}while(0);aB=aE+1|0;if((aB|0)<(bw|0)){aE=aB;av=bw}else{break}}}}while(0);bv=bv+1|0;}while((bv|0)<(c[550922]|0))}}while(0);aG=bo+136|0;aH=c[aG>>2]|0;av=c[aG+4>>2]|0;do{if((aH|0)==0&(av|0)==(-2147483648|0)){aE=bo+144|0;am=c[aE>>2]|0;aA=c[aE+4>>2]|0;if(!((am|0)==0&(aA|0)==(-2147483648|0))){c[aG>>2]=am;c[aG+4>>2]=aA;aA=N+8|0;c[aA>>2]=0;c[aA+4>>2]=-2147483648;aA=(c[X>>2]|0)+40|0;am=c[aA+4>>2]|0;c[y>>2]=c[aA>>2];c[y+4>>2]=am;break}am=N+8|0;aA=c[am>>2]|0;aE=c[am+4>>2]|0;if((aA|0)==0&(aE|0)==(-2147483648|0)){aB=c[aa+4>>2]|0;c[aG>>2]=c[aa>>2];c[aG+4>>2]=aB;c[y>>2]=1;c[aN+4>>2]=1e6;break}else{c[aG>>2]=aA;c[aG+4>>2]=aE;c[am>>2]=0;c[am+4>>2]=-2147483648;am=(c[X>>2]|0)+40|0;aE=c[am+4>>2]|0;c[y>>2]=c[am>>2];c[y+4>>2]=aE;break}}else{aE=_+104|0;c[B>>2]=1;c[B+4>>2]=1e6;am=AY(aH,av,aE,B)|0;aA=G;c[Y>>2]=am;c[Y+4>>2]=aA;c[ac>>2]=am;c[ac+4>>2]=aA;c[W>>2]=am;c[W+4>>2]=aA;c[aa>>2]=am;c[aa+4>>2]=aA;aA=aE;aE=c[aA+4>>2]|0;c[y>>2]=c[aA>>2];c[y+4>>2]=aE}}while(0);av=c[aG>>2]|0;aH=c[aG+4>>2]|0;if(!((av|0)==0&(aH|0)==(-2147483648|0))){c[C>>2]=1;aE=(c[X>>2]|0)+8|0;c[C+4>>2]=c[(c[aE>>2]|0)+412>>2];aA=c[aM>>2]|0;c[D>>2]=1;c[D+4>>2]=c[(c[aE>>2]|0)+412>>2];aE=A$(aN,av,aH,C,aA,b+80|0,D)|0;c[aG>>2]=aE;c[aG+4>>2]=G}aE=b+248|0;aA=b+244|0;aH=0;av=0;while(1){am=c[aE>>2]|0;if((aH|0)>=(am|0)){bx=av;break}if((aH|0)<(am-1|0)){am=c[aI>>2]|0;aB=Aw(am,bo)|0;if((aB|0)<0){bx=aB;break}else{by=am}}else{by=bo}am=gz(c[c[(c[aA>>2]|0)+(aH<<2)>>2]>>2]|0,by,4)|0;aB=(am|0)==-541478725?0:am;if((aB|0)<0){bx=aB;break}else{aH=aH+1|0;av=aB}}c[aG>>2]=0;c[aG+4>>2]=-2147483648;Au(c[aI>>2]|0);Au(bo);ap=(bx|0)<0?bx:bq;break L3254}}while(0);aI=b+248|0;if((c[aI>>2]|0)<=0){ap=bt;break}aN=b+244|0;aF=0;while(1){gB(c[c[(c[aN>>2]|0)+(aF<<2)>>2]>>2]|0,0,0)|0;aC=aF+1|0;if((aC|0)<(c[aI>>2]|0)){aF=aC}else{ap=bt;break}}}else{af=-1;ae=2568;break L3243}}while(0);if((ap|0)<0){af=ap;ae=2570;break}aa=N+8|0;c[aa>>2]=0;c[aa+4>>2]=-2147483648;aa=N+16|0;c[aa>>2]=0;c[aa+4>>2]=-2147483648;if(O){ae=2343;continue}aa=c[P>>2]|0;ac=(c[(c[(c[X>>2]|0)+8>>2]|0)+8>>2]|0)==1?ap:aa;_=N+24|0;c[_>>2]=(c[_>>2]|0)+ac;c[P>>2]=aa-ac;ae=2343}if((ae|0)==2510){N=c[W>>2]|0;ap=c[W+4>>2]|0;O=b+48|0;c[O>>2]=N;c[O+4>>2]=ap;bt=c[b+4>>2]|0;bq=c[bt+8>>2]|0;bx=c[bq+8>>2]|0;do{if((bx|0)==1){bo=c[bq+424>>2]|0;by=N2(bo,(bo|0)<0|0?-1:0,1e6,0)|0;bo=c[bq+412>>2]|0;D=N0(by,G,bo,(bo|0)<0|0?-1:0)|0;bo=NU(N,ap,D,G)|0;D=G;c[W>>2]=bo;c[W+4>>2]=D;bz=D;bA=bo}else if((bx|0)==0){bo=b+112|0;D=c[bo>>2]|0;if((D|0)!=0){by=b+116|0;c[T>>2]=d[by]|d[by+1|0]<<8|d[by+2|0]<<16|d[by+3|0]<<24;c[T+4>>2]=D;D=AY(N,ap,92616,T)|0;C=NU(D,G,1,0)|0;D=d[bo]|d[bo+1|0]<<8|d[bo+2|0]<<16|d[bo+3|0]<<24|0;c[U>>2]=d[by]|d[by+1|0]<<8|d[by+2|0]<<16|d[by+3|0]<<24;c[U+4>>2]=D;D=AY(C,G,U,92616)|0;C=G;c[W>>2]=D;c[W+4>>2]=C;bz=C;bA=D;break}D=c[e+48>>2]|0;if((D|0)!=0){c[V>>2]=1;c[V+4>>2]=1e6;C=AY(D,(D|0)<0|0?-1:0,bt+40|0,V)|0;D=NU(c[W>>2]|0,c[W+4>>2]|0,C,G)|0;C=G;c[W>>2]=D;c[W+4>>2]=C;bz=C;bA=D;break}D=c[bq+104>>2]|0;if((D|0)==0){bz=ap;bA=N;break}C=c[bt+240>>2]|0;if((C|0)==0){bB=c[bq+112>>2]|0}else{bB=(c[C+36>>2]|0)+1|0}C=N2(bB,(bB|0)<0|0?-1:0,1e6,0)|0;by=N2(C,G,D,(D|0)<0|0?-1:0)|0;D=c[bq+108>>2]|0;C=N0(by,G,D,(D|0)<0|0?-1:0)|0;D=NU(N,ap,C,G)|0;C=G;c[W>>2]=D;c[W+4>>2]=C;bz=C;bA=D}else{bz=ap;bA=N}}while(0);N=c[O+4>>2]|0;ap=b+64|0;c[ap>>2]=c[O>>2];c[ap+4>>2]=N;c[Y>>2]=bA;c[Y+4>>2]=bz}else if((ae|0)==2567){i=f;return af|0}else if((ae|0)==2568){i=f;return af|0}else if((ae|0)==2569){i=f;return af|0}else if((ae|0)==2570){i=f;return af|0}else if((ae|0)==2571){i=f;return af|0}if(!((e|0)!=0&(c[550910]|0)>0)){af=0;i=f;return af|0}ae=b|0;bz=b+4|0;Y=I;bA=F|0;N=F+4|0;ap=H|0;O=H+4|0;W=e+36|0;bq=e+8|0;bB=b+64|0;bt=J+8|0;V=e+16|0;U=b+48|0;T=K|0;bx=K+4|0;D=J+16|0;C=e+28|0;by=L|0;bo=L+4|0;y=b+80|0;b=e+48|0;B=J+48|0;bv=J+36|0;bw=J+24|0;ag=J+28|0;bu=e+24|0;e=J|0;x=E|0;br=E+4|0;bs=0;while(1){M=c[(c[550876]|0)+(bs<<2)>>2]|0;L3510:do{if((c[M+8>>2]|0)==((c[c[bz>>2]>>2]|0)+(c[(c[(c[551810]|0)+(c[ae>>2]<<2)>>2]|0)+12>>2]|0)|0)){bp=M|0;bn=c[(c[550878]|0)+(c[bp>>2]<<2)>>2]|0;A=bn+24|0;z=c[A>>2]|0;a8=c[A+4>>2]|0;if(!((z|0)==0&(a8|0)==0)){bk=c[bB+4>>2]|0;if((bk|0)<(a8|0)|(bk|0)==(a8|0)&(c[bB>>2]|0)>>>0<z>>>0){break}}if((c[M+16>>2]|0)!=0){break}c[bA>>2]=1;c[N>>2]=1e6;bk=M+12|0;bi=AY(z,a8,F,(c[bk>>2]|0)+40|0)|0;a8=G;z=c[A>>2]|0;bj=c[A+4>>2]|0;c[ap>>2]=1;c[O>>2]=1e6;bm=AY(z,bj,H,(c[bz>>2]|0)+40|0)|0;bj=G;nB(J);z=c[M+20>>2]|0;do{if((z|0)==0){if((c[W>>2]&1|0)!=0){break}if((c[M+352>>2]|0)==0){break L3510}}}while(0);S=c[bq>>2]|0;bl=c[bq+4>>2]|0;be=(z|0)!=0;do{if((S|0)==0&(bl|0)==(-2147483648|0)){if(be){break}bd=c[bB+4>>2]|0;ba=c[A+4>>2]|0;if(!((bd|0)<(ba|0)|(bd|0)==(ba|0)&(c[bB>>2]|0)>>>0<(c[A>>2]|0)>>>0)){break}if((c[M+356>>2]|0)==0){break L3510}}else{if(!(((bl|0)<(bj|0)|(bl|0)==(bj|0)&S>>>0<bm>>>0)&(be^1))){break}if((c[M+356>>2]|0)==0){break L3510}}}while(0);be=bn+16|0;bm=c[be>>2]|0;S=c[be+4>>2]|0;do{if(!((bm|0)==(-1|0)&(S|0)==2147483647)){be=c[bB>>2]|0;bj=c[bB+4>>2]|0;bl=NU(c[A>>2]|0,c[A+4>>2]|0,bm,S)|0;z=G;if((bj|0)<(z|0)|(bj|0)==(z|0)&be>>>0<bl>>>0){break}bl=c[(c[550878]|0)+(c[bp>>2]<<2)>>2]|0;c[M+336>>2]=1;if((c[bl+40>>2]|0)==0){break L3510}be=M+32|0;z=M+40|0;bj=NV(c[be>>2]|0,c[be+4>>2]|0,c[z>>2]|0,c[z+4>>2]|0)|0;z=(c[(c[bk>>2]|0)+8>>2]|0)+104|0;c[x>>2]=1;c[br>>2]=1e6;be=AY(bj,G,z,E)|0;z=G;bj=bl+16|0;bl=c[bj>>2]|0;ba=c[bj+4>>2]|0;bd=(ba|0)>(z|0)|(ba|0)==(z|0)&bl>>>0>be>>>0;c[bj>>2]=bd?be:bl;c[bj+4>>2]=bd?z:ba;break L3510}}while(0);bp=c[(c[(c[bk>>2]|0)+8>>2]|0)+8>>2]|0;if((bp|0)==1){S=c[C>>2]|0;bm=NU(c[767226]|0,c[767227]|0,S,(S|0)<0|0?-1:0)|0;c[767226]=bm;c[767227]=G}else if((bp|0)==0){bm=c[C>>2]|0;S=NU(c[499514]|0,c[499515]|0,bm,(bm|0)<0|0?-1:0)|0;c[499514]=S;c[499515]=G;S=M+32|0;bm=NU(c[S>>2]|0,c[S+4>>2]|0,1,0)|0;c[S>>2]=bm;c[S+4>>2]=G}else if((bp|0)==3){bp=c[C>>2]|0;S=NU(c[540562]|0,c[540563]|0,bp,(bp|0)<0|0?-1:0)|0;c[540562]=S;c[540563]=G}S=c[bq>>2]|0;bp=c[bq+4>>2]|0;if((S|0)==0&(bp|0)==(-2147483648|0)){bC=-2147483648;bD=0}else{bm=AY(S,bp,(c[bz>>2]|0)+40|0,(c[bk>>2]|0)+40|0)|0;bp=NV(bm,G,bi,a8)|0;bC=G;bD=bp}c[bt>>2]=bD;c[bt+4>>2]=bC;bp=c[V>>2]|0;bm=c[V+4>>2]|0;if((bp|0)==0&(bm|0)==(-2147483648|0)){S=c[U>>2]|0;A=c[U+4>>2]|0;c[T>>2]=1;c[bx>>2]=1e6;ba=AY(S,A,K,(c[bk>>2]|0)+40|0)|0;bE=G;bF=ba}else{ba=AY(bp,bm,(c[bz>>2]|0)+40|0,(c[bk>>2]|0)+40|0)|0;bE=G;bF=ba}ba=NV(bF,bE,bi,a8)|0;c[D>>2]=ba;c[D+4>>2]=G;ba=c[bk>>2]|0;do{if((c[(c[ba+8>>2]|0)+8>>2]|0)==1){if((c[V>>2]|0)==0&(c[V+4>>2]|0)==(-2147483648|0)){bG=ba;break}bm=pt(c[(c[bz>>2]|0)+8>>2]|0,c[C>>2]|0)|0;bp=c[bz>>2]|0;A=bp+8|0;if((bm|0)==0){bH=c[(c[A>>2]|0)+424>>2]|0}else{bH=bm}bm=c[V>>2]|0;S=c[V+4>>2]|0;c[by>>2]=1;c[bo>>2]=c[(c[A>>2]|0)+412>>2];A=A$(bp+40|0,bm,S,L,bH,y,(c[bk>>2]|0)+40|0)|0;S=NV(A,G,bi,a8)|0;A=G;c[bt>>2]=S;c[bt+4>>2]=A;c[D>>2]=S;c[D+4>>2]=A;bG=c[bk>>2]|0}else{bG=ba}}while(0);ba=c[b>>2]|0;a8=AY(ba,(ba|0)<0|0?-1:0,(c[bz>>2]|0)+40|0,bG+40|0)|0;c[B>>2]=a8;a8=c[W>>2]|0;c[bv>>2]=a8;ba=c[(c[bk>>2]|0)+8>>2]|0;bi=c[ba+48>>2]|0;do{if((bi|0)==28|(bi|0)==1|(bi|0)==2|(bi|0)==71){c[bw>>2]=c[bu>>2];c[ag>>2]=c[C>>2]}else{if((oG(c[(c[bz>>2]|0)+240>>2]|0,ba,bw,ag,c[bu>>2]|0,c[C>>2]|0,a8&1)|0)==0){break}A=ze(c[bw>>2]|0,c[ag>>2]|0,28,0,0)|0;c[e>>2]=A;if((A|0)!=0){break}eW(1)}}while(0);a8=c[(c[bk>>2]|0)+8>>2]|0;ba=bn|0;do{if((c[a8+8>>2]|0)==0){if((c[(c[(c[ba>>2]|0)+8>>2]|0)+28>>2]&32|0)==0){break}bi=c[bw>>2]|0;A=c[a8+140>>2]|0;S=c[a8+120>>2]|0;bm=c[a8+124>>2]|0;nO(I,bi,A,S,bm)|0;c[bw>>2]=Y;c[ag>>2]=64;c[bv>>2]=c[bv>>2]|1}}while(0);eO(c[ba>>2]|0,J,M);a8=(c[(c[bk>>2]|0)+8>>2]|0)+428|0;c[a8>>2]=(c[a8>>2]|0)+1}}while(0);M=bs+1|0;if((M|0)<(c[550910]|0)){bs=M}else{af=0;break}}i=f;return af|0}function eN(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0;d=i;i=i+1176|0;e=d|0;f=d+136|0;if((c[744104]|0)==0){i=d;return}cj(0,e|0)|0;g=c[e>>2]|0;h=N2(g,(g|0)<0|0?-1:0,1e6,0)|0;g=c[e+4>>2]|0;e=NU(h,G,g,(g|0)<0|0?-1:0)|0;g=G;if((a|0)!=0){h=f|0;j=f;c[j>>2]=b;c[j+4>>2]=0;j=d+152|0;cs(j|0,1024,a|0,h|0)|0;h=c[745298]|0;a=NV(e,g,h,(h|0)<0|0?-1:0)|0;h=G;g=1666224;bT(g|0,(g=i,i=i+24|0,c[g>>2]=a,c[g+8>>2]=h,c[g+16>>2]=j,g)|0)|0;i=g}c[745298]=e;i=d;return}function eO(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0.0;f=i;i=i+208|0;g=f|0;j=f+80|0;k=f+112|0;l=f+144|0;m=f+176|0;n=c[e+56>>2]|0;o=e+12|0;p=c[(c[o>>2]|0)+8>>2]|0;q=p+8|0;r=c[q>>2]|0;if((r|0)==0&(c[8408]|0)==255){s=2582}else{if((r|0)==1&(c[767224]|0)<0){s=2582}else{t=r}}if((s|0)==2582){r=d+16|0;c[r>>2]=0;c[r+4>>2]=-2147483648;r=d+8|0;c[r>>2]=0;c[r+4>>2]=-2147483648;t=c[q>>2]|0}if((t|0)==0){if((c[p+12>>2]|0)==0){s=2585}}else{s=2585}do{if((s|0)==2585){t=e+20|0;r=c[t>>2]|0;u=(r|0)<0|0?-1:0;v=e+64|0;w=c[v+4>>2]|0;if((u|0)<(w|0)|(u|0)==(w|0)&r>>>0<(c[v>>2]|0)>>>0){c[t>>2]=r+1;break}nJ(d);i=f;return}}while(0);if((n|0)!=0){r=g;t=d;v=g+24|0;w=g+28|0;u=d+24|0;x=d+28|0;y=d+36|0;z=g+52|0;A=g|0;g=d+32|0;B=p+12|0;C=n;do{NN(r|0,t|0,80)|0;n=nS(C,p,0,v,w,c[u>>2]|0,c[x>>2]|0,c[y>>2]&1)|0;do{if((n|0)==0){if((c[v>>2]|0)==(c[u>>2]|0)){break}if((c[z>>2]|0)==0){s=2595;break}D=A1((c[w>>2]|0)+16|0)|0;if((D|0)==0){E=-12;s=2599;break}F=c[v>>2]|0;H=c[w>>2]|0;NN(D|0,F|0,H)|0;NM(D+H|0,0,16);c[v>>2]=D;c[A>>2]=0;s=2596}else{s=2595}}while(0);do{if((s|0)==2595){s=0;if((n|0)>0){s=2596;break}if((n|0)<0){E=n;s=2599}}}while(0);do{if((s|0)==2596){s=0;nJ(d);n=ze(c[v>>2]|0,c[w>>2]|0,28,0,0)|0;c[A>>2]=n;if((n|0)!=0){break}eW(1)}else if((s|0)==2599){s=0;n=c[g>>2]|0;D=c[B>>2]|0;if((D|0)==0){I=1682048}else{I=c[D>>2]|0}AN(0,16,1683736,(J=i,i=i+24|0,c[J>>2]=c[c[C+4>>2]>>2],c[J+8>>2]=n,c[J+16>>2]=I,J)|0);i=J;fe(3119008,E);if((c[744092]|0)==0){break}eW(1)}}while(0);NN(t|0,r|0,80)|0;C=c[C+12>>2]|0;}while((C|0)!=0)}C=c[(c[b+8>>2]|0)+28>>2]|0;do{if((C&128|0)==0){r=c[q>>2]|0;if(r>>>0>=2){break}t=d+16|0;E=c[t>>2]|0;I=c[t+4>>2]|0;if((E|0)==0&(I|0)==(-2147483648|0)){break}B=e+48|0;g=c[B>>2]|0;s=c[B+4>>2]|0;if((g|0)==0&(s|0)==(-2147483648|0)){break}B=NU(g,s,C>>>17&1^1,0)|0;A=G;if(!((I|0)<(A|0)|(I|0)==(A|0)&E>>>0<B>>>0)){break}w=NV(B,A,E,I)|0;v=G;z=0;if((v|0)>(z|0)|(v|0)==(z|0)&w>>>0>2>>>0){K=24}else{K=(r|0)==0?24:48}r=b;w=c[c[o>>2]>>2]|0;AN(r,K,1680080,(J=i,i=i+48|0,c[J>>2]=c[e>>2],c[J+8>>2]=w,c[J+16>>2]=g,c[J+24>>2]=s,c[J+32>>2]=E,c[J+40>>2]=I,J)|0);i=J;if((c[744092]|0)!=0){AN(0,8,1678128,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;eW(1)}AN(r,K,1676144,(J=i,i=i+16|0,c[J>>2]=B,c[J+8>>2]=A,J)|0);i=J;r=d+8|0;I=c[r>>2]|0;E=c[r+4>>2]|0;s=c[t+4>>2]|0;if(!((E|0)<(s|0)|(E|0)==(s|0)&I>>>0<(c[t>>2]|0)>>>0)){s=(E|0)>(A|0)|(E|0)==(A|0)&I>>>0>B>>>0;c[r>>2]=s?I:B;c[r+4>>2]=s?E:A}c[t>>2]=B;c[t+4>>2]=A}}while(0);K=d+16|0;C=c[K+4>>2]|0;q=e+48|0;c[q>>2]=c[K>>2];c[q+4>>2]=C;c[d+32>>2]=c[e+4>>2];if((c[745296]|0)!=0){e=Cp(c[(c[(c[o>>2]|0)+8>>2]|0)+8>>2]|0)|0;C=j|0;NM(C|0,0,32);a[C]=0;j=d+8|0;q=c[j>>2]|0;A=c[j+4>>2]|0;if((q|0)==0&(A|0)==(-2147483648|0)){bc(C|0,32,1670672,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0)|0;i=J}else{t=1667440;bc(C|0,32,t|0,(J=i,i=i+16|0,c[J>>2]=q,c[J+8>>2]=A,J)|0)|0;i=J}A=k|0;NM(A|0,0,32);a[A]=0;k=c[j>>2]|0;q=c[j+4>>2]|0;j=c[o>>2]|0;if((k|0)==0&(q|0)==(-2147483648|0)){bc(A|0,32,1670672,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0)|0;i=J}else{L=(+(k>>>0)+ +(q|0)*4294967296.0)*(+(c[j+40>>2]|0)/+(c[j+44>>2]|0));bc(A|0,32,1668840,(J=i,i=i+8|0,h[J>>3]=L,J)|0)|0;i=J}j=l|0;NM(j|0,0,32);a[j]=0;l=c[K>>2]|0;q=c[K+4>>2]|0;if((l|0)==0&(q|0)==(-2147483648|0)){bc(j|0,32,1670672,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0)|0;i=J}else{k=1667440;bc(j|0,32,k|0,(J=i,i=i+16|0,c[J>>2]=l,c[J+8>>2]=q,J)|0)|0;i=J}q=m|0;NM(q|0,0,32);a[q]=0;m=c[K>>2]|0;l=c[K+4>>2]|0;K=c[o>>2]|0;if((m|0)==0&(l|0)==(-2147483648|0)){bc(q|0,32,1670672,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0)|0;i=J}else{L=(+(m>>>0)+ +(l|0)*4294967296.0)*(+(c[K+40>>2]|0)/+(c[K+44>>2]|0));bc(q|0,32,1668840,(J=i,i=i+8|0,h[J>>3]=L,J)|0)|0;i=J}K=c[d+28>>2]|0;AN(0,32,1674472,(J=i,i=i+48|0,c[J>>2]=e,c[J+8>>2]=C,c[J+16>>2]=A,c[J+24>>2]=j,c[J+32>>2]=q,c[J+40>>2]=K,J)|0);i=J}J=ld(b,d)|0;if((J|0)>=0){i=f;return}fe(1672728,J);eW(1);i=f;return}function eP(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,j=0.0,k=0,l=0,m=0.0,n=0.0;d=i;do{if((c[499384]|0)==0){e=bV(c[499382]|0,1701464)|0;c[499384]=e;if((e|0)!=0){break}a3(1698400);eW(1)}}while(0);e=a+12|0;a=c[e>>2]|0;f=c[a+8>>2]|0;if((c[f+8>>2]|0)!=0){i=d;return}g=f+804|0;j=+(c[(c[g>>2]|0)+168>>2]|0)/118.0;cx(c[499384]|0,1696248,(k=i,i=i+16|0,c[k>>2]=c[a+64>>2],h[k+8>>3]=j,k)|0)|0;i=k;if((c[f+88>>2]&32768|0)!=0){a=c[499384]|0;l=(c[g>>2]|0)+224|0;j=+((c[l>>2]|0)>>>0)+ +((c[l+4>>2]|0)>>>0)*4294967296.0;m=+aa(+(j/(+(ac(c[f+124>>2]|0,c[f+120>>2]|0)|0)*255.0*255.0)))*-10.0/2.302585092994046;cx(a|0,1694e3,(k=i,i=i+8|0,h[k>>3]=m,k)|0)|0;i=k}cx(c[499384]|0,1690400,(k=i,i=i+8|0,c[k>>2]=b,k)|0)|0;i=k;a=(c[e>>2]|0)+16|0;m=+(c[f+104>>2]|0)/+(c[f+108>>2]|0);j=(+((c[a>>2]|0)>>>0)+ +(c[a+4>>2]|0)*4294967296.0)*m;n=j<.01?.01:j;a=c[499514]|0;f=c[499515]|0;cx(c[499384]|0,1687272,(k=i,i=i+32|0,h[k>>3]=(+(a>>>0)+ +(f|0)*4294967296.0)*.0009765625,h[k+8>>3]=n,h[k+16>>3]=+(b<<3|0)/m/1.0e3,h[k+24>>3]=(+((a<<3|0>>>29)>>>0)+ +(f<<3|a>>>29|0)*4294967296.0)/n/1.0e3,k)|0)|0;i=k;a=c[499384]|0;f=(Cq(c[(c[g>>2]|0)+88>>2]|0)|0)<<24>>24;cx(a|0,1685768,(k=i,i=i+8|0,c[k>>2]=f,k)|0)|0;i=k;i=d;return}function eQ(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0;e=i;i=i+16|0;f=e|0;g=e+8|0;h=a+228|0;j=c[h>>2]|0;k=a+232|0;l=c[k>>2]|0;m=a+224|0;n=c[m>>2]|0;if((n|0)==0){i=e;return}if((b|0)==0){o=a+216|0;p=2147483647;q=-1;r=c[o+4>>2]|0;s=c[o>>2]|0;t=0;u=n}else{o=b+24|0;v=NU((c[b+4>>2]|0)*1e3|0,0,c[o>>2]|0,c[o+4>>2]|0)|0;c[f>>2]=1;c[f+4>>2]=1e6;w=a+4|0;x=AY(v,G,f,(c[w>>2]|0)+40|0)|0;f=G;v=NU((c[b+8>>2]|0)*1e3|0,0,c[o>>2]|0,c[o+4>>2]|0)|0;c[g>>2]=1;c[g+4>>2]=1e6;o=AY(v,G,g,(c[w>>2]|0)+40|0)|0;p=G;q=o;r=f;s=x;t=c[b+12>>2]|0;u=c[m>>2]|0}Au(u);c[(c[m>>2]|0)+68>>2]=c[h>>2];c[(c[m>>2]|0)+72>>2]=c[k>>2];c[(c[m>>2]|0)+80>>2]=30;if((Av(u,32)|0)<0){AN(c[(c[a+4>>2]|0)+8>>2]|0,16,1659224,(y=i,i=i+1|0,i=i+7&-8,c[y>>2]=0,y)|0);i=y;i=e;return}k=c[u>>2]|0;NM(k|0,0,ac(c[u+32>>2]|0,c[u+72>>2]|0)|0);u=c[n>>2]|0;k=c[n+32>>2]|0;if((t|0)>0){n=b+16|0;b=0;do{h=c[(c[n>>2]|0)+(b<<2)>>2]|0;L3675:do{if((c[h+84>>2]|0)==1){x=c[h>>2]|0;do{if((x|0)>=0){f=h+8|0;o=c[f>>2]|0;if((o+x|0)>(j|0)){break}w=c[h+4>>2]|0;if((w|0)<0){break}g=h+12|0;v=c[g>>2]|0;if((v+w|0)>(l|0)){break}z=c[h+24>>2]|0;if((v|0)<=0){break L3675}A=(ac(w,k)|0)+(x<<2)|0;w=h+52|0;B=u+A|0;A=c[h+20>>2]|0;C=0;D=o;o=v;while(1){if((D|0)>0){v=B;E=A;F=0;while(1){c[v>>2]=c[z+((d[E]|0)<<2)>>2];H=F+1|0;I=c[f>>2]|0;if((H|0)<(I|0)){v=v+4|0;E=E+1|0;F=H}else{break}}J=I;K=c[g>>2]|0}else{J=D;K=o}F=C+1|0;if((F|0)<(K|0)){B=B+k|0;A=A+(c[w>>2]|0)|0;C=F;D=J;o=K}else{break L3675}}}}while(0);AN(0,24,1655816,(y=i,i=i+1|0,i=i+7&-8,c[y>>2]=0,y)|0);i=y}else{AN(0,24,1658024,(y=i,i=i+1|0,i=i+7&-8,c[y>>2]=0,y)|0);i=y}}while(0);b=b+1|0;}while((b|0)<(t|0))}t=c[m>>2]|0;m=t+136|0;c[m>>2]=s;c[m+4>>2]=r;m=a+208|0;c[m>>2]=s;c[m+4>>2]=r;r=a+248|0;if((c[r>>2]|0)>0){m=a+244|0;s=0;do{gz(c[c[(c[m>>2]|0)+(s<<2)>>2]>>2]|0,t,12)|0;s=s+1|0;}while((s|0)<(c[r>>2]|0))}r=a+216|0;c[r>>2]=q;c[r+4>>2]=p;i=e;return}function eR(){var b=0,d=0,e=0,f=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0.0,aM=0,aN=0.0,aO=0.0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0.0,a8=0,a9=0,ba=0,bb=0,bd=0,be=0;b=i;i=i+408|0;d=b|0;e=b+8|0;f=b+16|0;j=b+96|0;k=b+104|0;l=b+112|0;m=b+120|0;n=b+200|0;o=b+400|0;if((c[550910]|0)<=0){p=0;i=b;return p|0}q=b+336|0;s=b+208|0;t=b+240|0;u=b+272|0;v=b+304|0;w=m+24|0;x=m+28|0;y=m+8|0;z=m+16|0;A=m+36|0;B=l|0;C=l+4|0;D=k|0;E=k+4|0;F=f+24|0;H=f+28|0;I=f+8|0;J=f+16|0;K=f+48|0;L=e|0;M=e+4|0;N=d|0;O=d+4|0;P=o|0;Q=o+4|0;R=0;L3705:while(1){S=c[(c[550876]|0)+(R<<2)>>2]|0;T=S|0;U=c[(c[550878]|0)+(c[T>>2]<<2)>>2]|0;V=S+300|0;do{if((c[V>>2]|0)!=0){W=S+72|0;X=c[W>>2]|0;if((X|0)==0){Y=o0()|0;c[W>>2]=Y;if((Y|0)==0){p=-12;Z=2817;break L3705}else{_=Y}}else{_=X}o$(_);X=c[W>>2]|0;W=gq(c[c[V>>2]>>2]|0,X,2)|0;if((W|0)<0){$=W}else{W=X+136|0;Y=S+92|0;aa=S+12|0;ab=X+124|0;ac=U|0;ad=S+8|0;ae=S+76|0;af=S+80|0;ag=S+32|0;ah=S+64|0;ai=S+20|0;aj=X+296|0;ak=X+168|0;al=X+88|0;am=S+108|0;an=S+104|0;ao=S+100|0;ap=S+116|0;aq=S+152|0;ar=S+120|0;as=S+136|0;at=S+144|0;au=S+128|0;av=S+4|0;aw=S+296|0;ax=X+300|0;ay=S+88|0;az=X;aA=S+40|0;aB=S+336|0;aC=X+76|0;aD=U+24|0;while(1){aE=c[W>>2]|0;aF=c[W+4>>2]|0;if((aE|0)==0&(aF|0)==(-2147483648|0)){aG=-2147483648;aH=0}else{aI=AY(aE,aF,(c[c[(c[c[V>>2]>>2]|0)+16>>2]>>2]|0)+56|0,(c[(c[aa>>2]|0)+8>>2]|0)+104|0)|0;aF=G;aE=c[aD>>2]|0;aJ=c[aD+4>>2]|0;c[P>>2]=1;c[Q>>2]=1e6;aK=AY(aE,aJ,o,(c[(c[aa>>2]|0)+8>>2]|0)+104|0)|0;aJ=NV(aI,aF,aK,G)|0;aK=G;c[W>>2]=aJ;c[W+4>>2]=aK;aG=aK;aH=aJ}aJ=c[(c[c[(c[c[V>>2]>>2]|0)+16>>2]>>2]|0)+16>>2]|0;L3719:do{if((aJ|0)==0){c[W>>2]=aH;c[W+4>>2]=aG;if((c[Y>>2]|0)==0){aK=(c[(c[aa>>2]|0)+8>>2]|0)+224|0;aF=c[ab+4>>2]|0;c[aK>>2]=c[ab>>2];c[aK+4>>2]=aF}aF=c[ac>>2]|0;aK=c[(c[aa>>2]|0)+8>>2]|0;aI=c[ad>>2]|0;do{if((aI|0)>-1){aE=c[(c[551808]|0)+(aI<<2)>>2]|0;if((aE|0)==0){aL=0.0;break}aM=c[aE+4>>2]|0;aE=aM+48|0;if((c[aE>>2]|0)==0&(c[aE+4>>2]|0)==(-2147483648|0)){aL=0.0;break}aE=aM+200|0;if((c[aE>>2]|0)==0&(c[aE+4>>2]|0)==(-2147483648|0)){aL=0.0;break}aE=c[ae>>2]|0;if((aE|0)==0){aL=0.0;break}aL=1.0/(+(aE|0)/+(c[af>>2]|0)*(+(c[aK+104>>2]|0)/+(c[aK+108>>2]|0)))}else{aL=0.0}}while(0);aN=+((c[W>>2]|0)>>>0)+ +(c[W+4>>2]|0)*4294967296.0;aO=aL+(aN-(+((c[ag>>2]|0)>>>0)+ +(c[ag+4>>2]|0)*4294967296.0));aI=c[8408]|0;do{if((aI|0)==-1){aE=c[(c[aF+8>>2]|0)+28>>2]|0;if((aE&1024|0)==0){Z=2701;break}aP=aE>>>6&2^2;Z=2700}else{aP=aI;Z=2700}}while(0);do{if((Z|0)==2700){Z=0;if((aP|0)==1){Z=2701;break}else if((aP|0)==255|(aP|0)==0){aI=bR(+aN)|0;c[ag>>2]=aI;c[ag+4>>2]=(aI|0)<0|0?-1:0;aQ=0;aR=1;break}else if((aP|0)!=2){Z=2708;break L3705}if(aO<=-.6){aQ=0;aR=0;break}if(aO<=.6){aQ=0;aR=1;break}aI=bR(+aN)|0;c[ag>>2]=aI;c[ag+4>>2]=(aI|0)<0|0?-1:0;aQ=0;aR=1}}while(0);do{if((Z|0)==2701){Z=0;if(aO<-1.1){aQ=0;aR=0;break}if(aO<=1.1){aQ=0;aR=1;break}aI=bR(+aO)|0;aQ=(aI|0)<0|0?-1:0;aR=aI}}while(0);aI=c[ai>>2]|0;aE=NV(c[ah>>2]|0,c[ah+4>>2]|0,aI,(aI|0)<0|0?-1:0)|0;aI=G;aM=(aQ|0)>(aI|0)|(aQ|0)==(aI|0)&aR>>>0>aE>>>0;aS=aM?aE:aR;aM?aI:aQ;aI=aS;if((aI|0)==0){c[550920]=(c[550920]|0)+1;AN(0,40,1611696,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0);i=aT;break}do{if((aI|0)>1){aS=aI-1|0;if(+(aI|0)>+g[65164]*30.0){AN(0,16,1611456,(aT=i,i=i+8|0,c[aT>>2]=aS,aT)|0);i=aT;c[550920]=(c[550920]|0)+1;break L3719}else{c[550918]=(c[550918]|0)+aS;AN(0,40,1744952,(aT=i,i=i+8|0,c[aT>>2]=aS,aT)|0);i=aT;break}}}while(0);if((aI|0)<=0){break}aS=aF+8|0;aM=aK+408|0;aE=aK+316|0;aU=aK+12|0;aV=aK+668|0;aW=aK+104|0;aX=aK+104|0;aY=aK+108|0;a_=aK+804|0;a$=0;a0=0;while(1){nB(m);c[w>>2]=0;c[x>>2]=0;a1=c[ag+4>>2]|0;c[W>>2]=c[ag>>2];c[W+4>>2]=a1;a1=(c[(c[550878]|0)+(c[T>>2]<<2)>>2]|0)+16|0;a2=c[a1>>2]|0;a3=c[a1+4>>2]|0;if(!((a2|0)==(-1|0)&(a3|0)==2147483647)){a1=NV(c[ag>>2]|0,c[ag+4>>2]|0,c[aA>>2]|0,c[aA+4>>2]|0)|0;a4=(c[(c[aa>>2]|0)+8>>2]|0)+104|0;c[B>>2]=1;c[C>>2]=1e6;if((AZ(a1,G,a4,a2,a3,l)|0)>-1){break}}do{if((c[(c[aS>>2]|0)+28>>2]&32|0)==0){Z=2726}else{if((c[(c[aU>>2]|0)+12>>2]|0)!=14){Z=2726;break}c[(c[a_>>2]|0)+296>>2]=c[aj>>2];c[(c[a_>>2]|0)+300>>2]=c[ax>>2];a3=c[a_>>2]|0;if((c[a3+296>>2]|0)==0){a5=1}else{a5=(c[a3+300>>2]|0)!=0?4:5}c[aM>>2]=a5;c[w>>2]=az;c[x>>2]=64;a3=AY(c[W>>2]|0,c[W+4>>2]|0,aW,(c[aa>>2]|0)+40|0)|0;c[y>>2]=a3;c[y+4>>2]=G;c[A>>2]=c[A>>2]|1;a3=c[x>>2]|0;a2=NU(c[499514]|0,c[499515]|0,a3,(a3|0)<0|0?-1:0)|0;c[499514]=a2;c[499515]=G;eO(aF,m,S);a6=a0}}while(0);do{if((Z|0)==2726){Z=0;do{if((c[(c[(c[aa>>2]|0)+8>>2]|0)+88>>2]&537133056|0)!=0){a2=c[ay>>2]|0;if((a2|0)<=-1){break}c[ax>>2]=(a2|0)!=0}}while(0);do{if((c[aj>>2]|0)==0){c[aM>>2]=1}else{a2=(c[ax>>2]|0)!=0;if((c[(c[aU>>2]|0)+12>>2]|0)==8){c[aM>>2]=a2?2:3;break}else{c[aM>>2]=a2?4:5;break}}}while(0);c[ak>>2]=c[(c[(c[aa>>2]|0)+8>>2]|0)+80>>2];if((c[aE>>2]|0)==0){c[al>>2]=0}a2=c[W>>2]|0;a3=c[W+4>>2]|0;if((a2|0)==0&(a3|0)==(-2147483648|0)){a7=r}else{a7=(+(a2>>>0)+ +(a3|0)*4294967296.0)*(+(c[aX>>2]|0)/+(c[aY>>2]|0))}a4=c[am>>2]|0;do{if((a4|0)<(c[an>>2]|0)){a1=(c[ao>>2]|0)+(a4<<3)|0;a8=c[a1+4>>2]|0;if((a3|0)<(a8|0)|(a3|0)==(a8|0)&a2>>>0<(c[a1>>2]|0)>>>0){Z=2741;break}c[am>>2]=a4+1;Z=2745}else{Z=2741}}while(0);do{if((Z|0)==2741){Z=0;a4=c[ap>>2]|0;if((a4|0)==0){break}h[aq>>3]=a7;a2=+zT(a4,ar,0)!=0.0;aO=+h[ar>>3];if(a2){h[as>>3]=aO;h[at>>3]=+h[aq>>3];h[au>>3]=+h[au>>3]+1.0;h[ar>>3]=aO+1.0;Z=2745;break}else{h[ar>>3]=aO+1.0;break}}}while(0);if((Z|0)==2745){Z=0;c[al>>2]=1;AN(0,48,1744160,(aT=i,i=i+8|0,h[aT>>3]=a7,aT)|0);i=aT}eN(0,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0);i=aT;a2=pd(aK,m,X,n)|0;a4=c[av>>2]|0;eN(1743600,(aT=i,i=i+16|0,c[aT>>2]=c[T>>2],c[aT+8>>2]=a4,aT)|0);i=aT;if((a2|0)<0){AN(0,8,1742088,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0);i=aT;eW(1)}if((c[n>>2]|0)==0){a6=a0;break}a2=c[y>>2]|0;a4=c[y+4>>2]|0;do{if((a2|0)==0&(a4|0)==(-2147483648|0)){if((c[(c[aU>>2]|0)+16>>2]&32|0)!=0){break}a3=c[ag>>2]|0;a1=c[ag+4>>2]|0;c[y>>2]=a3;c[y+4>>2]=a1;if(!((a3|0)==0&(a1|0)==(-2147483648|0))){a9=a1;ba=a3;Z=2752}}else{a9=a4;ba=a2;Z=2752}}while(0);if((Z|0)==2752){Z=0;a2=AY(ba,a9,aW,(c[aa>>2]|0)+40|0)|0;c[y>>2]=a2;c[y+4>>2]=G}a2=c[z>>2]|0;a4=c[z+4>>2]|0;if(!((a2|0)==0&(a4|0)==(-2147483648|0))){a3=AY(a2,a4,aW,(c[aa>>2]|0)+40|0)|0;c[z>>2]=a3;c[z+4>>2]=G}if((c[745296]|0)!=0){NM(s|0,0,32);a[s]=0;a3=c[y>>2]|0;a4=c[y+4>>2]|0;if((a3|0)==0&(a4|0)==(-2147483648|0)){bc(s|0,32,1670672,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0)|0;i=aT}else{a2=1667440;bc(s|0,32,a2|0,(aT=i,i=i+16|0,c[aT>>2]=a3,c[aT+8>>2]=a4,aT)|0)|0;i=aT}NM(t|0,0,32);a[t]=0;a4=c[y>>2]|0;a3=c[y+4>>2]|0;a2=c[aa>>2]|0;if((a4|0)==0&(a3|0)==(-2147483648|0)){bc(t|0,32,1670672,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0)|0;i=aT}else{aO=(+(a4>>>0)+ +(a3|0)*4294967296.0)*(+(c[a2+40>>2]|0)/+(c[a2+44>>2]|0));bc(t|0,32,1668840,(aT=i,i=i+8|0,h[aT>>3]=aO,aT)|0)|0;i=aT}NM(u|0,0,32);a[u]=0;a2=c[z>>2]|0;a3=c[z+4>>2]|0;if((a2|0)==0&(a3|0)==(-2147483648|0)){bc(u|0,32,1670672,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0)|0;i=aT}else{a4=1667440;bc(u|0,32,a4|0,(aT=i,i=i+16|0,c[aT>>2]=a2,c[aT+8>>2]=a3,aT)|0)|0;i=aT}NM(v|0,0,32);a[v]=0;a3=c[z>>2]|0;a2=c[z+4>>2]|0;a4=c[aa>>2]|0;if((a3|0)==0&(a2|0)==(-2147483648|0)){bc(v|0,32,1670672,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0)|0;i=aT}else{aO=(+(a3>>>0)+ +(a2|0)*4294967296.0)*(+(c[a4+40>>2]|0)/+(c[a4+44>>2]|0));bc(v|0,32,1668840,(aT=i,i=i+8|0,h[aT>>3]=aO,aT)|0)|0;i=aT}AN(0,32,1740480,(aT=i,i=i+32|0,c[aT>>2]=s,c[aT+8>>2]=t,c[aT+16>>2]=u,c[aT+24>>2]=v,aT)|0);i=aT}a4=c[x>>2]|0;a2=NU(c[499514]|0,c[499515]|0,a4,(a4|0)<0|0?-1:0)|0;c[499514]=a2;c[499515]=G;eO(aF,m,S);nJ(m);a2=c[aw>>2]|0;if((a2|0)==0){a6=a4;break}a3=c[aV>>2]|0;if((a3|0)==0){a6=a4;break}aZ(a3|0,a2|0)|0;a6=a4}}while(0);a4=NU(c[ag>>2]|0,c[ag+4>>2]|0,1,0)|0;c[ag>>2]=a4;c[ag+4>>2]=G;c[ai>>2]=(c[ai>>2]|0)+1;if(!((c[499382]|0)==0|(a6|0)==0)){eP(S,a6)}a4=a$+1|0;if((a4|0)<(aI|0)){a$=a4;a0=a6}else{break L3719}}a0=c[(c[550878]|0)+(c[T>>2]<<2)>>2]|0;c[aB>>2]=1;if((c[a0+40>>2]|0)==0){break}a$=NV(c[ag>>2]|0,c[ag+4>>2]|0,c[aA>>2]|0,c[aA+4>>2]|0)|0;aI=(c[(c[aa>>2]|0)+8>>2]|0)+104|0;c[D>>2]=1;c[E>>2]=1e6;aV=AY(a$,G,aI,k)|0;aI=G;a$=a0+16|0;a0=c[a$>>2]|0;aF=c[a$+4>>2]|0;aW=(aF|0)>(aI|0)|(aF|0)==(aI|0)&a0>>>0>aV>>>0;c[a$>>2]=aW?aV:a0;c[a$+4>>2]=aW?aI:aF}else if((aJ|0)==1){c[W>>2]=aH;c[W+4>>2]=aG;aF=c[(c[aa>>2]|0)+8>>2]|0;do{if((c[(c[aF+12>>2]|0)+16>>2]&16384|0)==0){aI=c[aF+416>>2]|0;if((aI|0)==(An(X)|0)){bb=c[(c[aa>>2]|0)+8>>2]|0;break}else{AN(0,16,1614464,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0);i=aT;break L3719}}else{bb=aF}}while(0);aF=c[ac>>2]|0;c[j>>2]=0;nB(f);c[F>>2]=0;c[H>>2]=0;aI=(c[(c[550878]|0)+(c[T>>2]<<2)>>2]|0)+16|0;aW=c[aI>>2]|0;a$=c[aI+4>>2]|0;do{if(!((aW|0)==(-1|0)&(a$|0)==2147483647)){aI=NV(c[ag>>2]|0,c[ag+4>>2]|0,c[aA>>2]|0,c[aA+4>>2]|0)|0;a0=(c[(c[aa>>2]|0)+8>>2]|0)+104|0;c[L>>2]=1;c[M>>2]=1e6;if((AZ(aI,G,a0,aW,a$,e)|0)<=-1){break}a0=c[(c[550878]|0)+(c[T>>2]<<2)>>2]|0;c[aB>>2]=1;if((c[a0+40>>2]|0)==0){break L3719}aI=NV(c[ag>>2]|0,c[ag+4>>2]|0,c[aA>>2]|0,c[aA+4>>2]|0)|0;aV=(c[(c[aa>>2]|0)+8>>2]|0)+104|0;c[N>>2]=1;c[O>>2]=1e6;aU=AY(aI,G,aV,d)|0;aV=G;aI=a0+16|0;a0=c[aI>>2]|0;aK=c[aI+4>>2]|0;aY=(aK|0)>(aV|0)|(aK|0)==(aV|0)&a0>>>0>aU>>>0;c[aI>>2]=aY?aU:a0;c[aI+4>>2]=aY?aV:aK;break L3719}}while(0);a$=c[W>>2]|0;aW=c[W+4>>2]|0;if((a$|0)==0&(aW|0)==(-2147483648|0)|(c[767224]|0)<0){aK=c[ag>>2]|0;aV=c[ag+4>>2]|0;c[W>>2]=aK;c[W+4>>2]=aV;bd=aV;be=aK}else{bd=aW;be=a$}a$=c[aC>>2]|0;aW=NU(a$,(a$|0)<0|0?-1:0,be,bd)|0;c[ag>>2]=aW;c[ag+4>>2]=G;if((c[H>>2]|0)==0){if((c[F>>2]|0)!=0){Z=2787;break L3705}}eN(0,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0);i=aT;if((pb(bb,f,X,j)|0)<0){AN(0,8,1612648,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0);i=aT;eW(1)}aW=c[av>>2]|0;eN(1612328,(aT=i,i=i+16|0,c[aT>>2]=c[T>>2],c[aT+8>>2]=aW,aT)|0);i=aT;if((c[j>>2]|0)==0){break}aW=c[I>>2]|0;a$=c[I+4>>2]|0;if(!((aW|0)==0&(a$|0)==(-2147483648|0))){aK=AY(aW,a$,bb+104|0,(c[aa>>2]|0)+40|0)|0;c[I>>2]=aK;c[I+4>>2]=G}aK=c[J>>2]|0;a$=c[J+4>>2]|0;if(!((aK|0)==0&(a$|0)==(-2147483648|0))){aW=AY(aK,a$,bb+104|0,(c[aa>>2]|0)+40|0)|0;c[J>>2]=aW;c[J+4>>2]=G}aW=c[K>>2]|0;if((aW|0)>0){a$=AY(aW,(aW|0)<0|0?-1:0,bb+104|0,(c[aa>>2]|0)+40|0)|0;c[K>>2]=a$}if((c[745296]|0)!=0){NM(s|0,0,32);a[s]=0;a$=c[I>>2]|0;aW=c[I+4>>2]|0;if((a$|0)==0&(aW|0)==(-2147483648|0)){bc(s|0,32,1670672,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0)|0;i=aT}else{aK=1667440;bc(s|0,32,aK|0,(aT=i,i=i+16|0,c[aT>>2]=a$,c[aT+8>>2]=aW,aT)|0)|0;i=aT}NM(t|0,0,32);a[t]=0;aW=c[I>>2]|0;a$=c[I+4>>2]|0;aK=c[aa>>2]|0;if((aW|0)==0&(a$|0)==(-2147483648|0)){bc(t|0,32,1670672,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0)|0;i=aT}else{aO=(+(aW>>>0)+ +(a$|0)*4294967296.0)*(+(c[aK+40>>2]|0)/+(c[aK+44>>2]|0));bc(t|0,32,1668840,(aT=i,i=i+8|0,h[aT>>3]=aO,aT)|0)|0;i=aT}NM(u|0,0,32);a[u]=0;aK=c[J>>2]|0;a$=c[J+4>>2]|0;if((aK|0)==0&(a$|0)==(-2147483648|0)){bc(u|0,32,1670672,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0)|0;i=aT}else{aW=1667440;bc(u|0,32,aW|0,(aT=i,i=i+16|0,c[aT>>2]=aK,c[aT+8>>2]=a$,aT)|0)|0;i=aT}NM(v|0,0,32);a[v]=0;a$=c[J>>2]|0;aK=c[J+4>>2]|0;aW=c[aa>>2]|0;if((a$|0)==0&(aK|0)==(-2147483648|0)){bc(v|0,32,1670672,(aT=i,i=i+1|0,i=i+7&-8,c[aT>>2]=0,aT)|0)|0;i=aT}else{aO=(+(a$>>>0)+ +(aK|0)*4294967296.0)*(+(c[aW+40>>2]|0)/+(c[aW+44>>2]|0));bc(v|0,32,1668840,(aT=i,i=i+8|0,h[aT>>3]=aO,aT)|0)|0;i=aT}AN(0,32,1611976,(aT=i,i=i+32|0,c[aT>>2]=s,c[aT+8>>2]=t,c[aT+16>>2]=u,c[aT+24>>2]=v,aT)|0);i=aT}aW=c[H>>2]|0;aK=NU(c[767226]|0,c[767227]|0,aW,(aW|0)<0|0?-1:0)|0;c[767226]=aK;c[767227]=G;eO(aF,f,S);nJ(f)}else{Z=2812;break L3705}}while(0);Au(X);aJ=gq(c[c[V>>2]>>2]|0,X,2)|0;if((aJ|0)<0){$=aJ;break}}}if(($|0)==(-11|0)|($|0)==(-541478725|0)){break}NM(q|0,0,64);a[q]=0;zN($,q,64)|0;AN(0,24,1615112,(aT=i,i=i+8|0,c[aT>>2]=q,aT)|0);i=aT}}while(0);V=R+1|0;if((V|0)<(c[550910]|0)){R=V}else{p=0;Z=2818;break}}if((Z|0)==2708){AN(0,0,1616448,(aT=i,i=i+24|0,c[aT>>2]=1614008,c[aT+8>>2]=1615896,c[aT+16>>2]=840,aT)|0);i=aT;cw();return 0}else if((Z|0)==2787){AN(0,0,1616448,(aT=i,i=i+24|0,c[aT>>2]=1613464,c[aT+8>>2]=1615896,c[aT+16>>2]=691,aT)|0);i=aT;cw();return 0}else if((Z|0)==2812){AN(0,0,1616448,(aT=i,i=i+24|0,c[aT>>2]=1614008,c[aT+8>>2]=1615896,c[aT+16>>2]=1104,aT)|0);i=aT;cw();return 0}else if((Z|0)==2817){i=b;return p|0}else if((Z|0)==2818){i=b;return p|0}return 0}function eS(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=a;a=c[d>>2]|0;e=c[d+4>>2]|0;d=b;b=c[d>>2]|0;f=c[d+4>>2]|0;if((e|0)<(f|0)|(e|0)==(f|0)&a>>>0<b>>>0){g=-1;return g|0}g=((e|0)>(f|0)|(e|0)==(f|0)&a>>>0>b>>>0)&1;return g|0}function eT(){tb(c[540558]|0);c[540558]=0;zL(2162240);zL(2225160);zL(2981336);zL(2164e3);return}function eU(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;cd(c[p>>2]|0,d|0,e|0)|0;return}function eV(a){a=a|0;c[541012]=a;return}function eW(a){a=a|0;var b=0;b=c[541012]|0;if((b|0)==0){bp(a|0)}c_[b&127](a);bp(a|0)}function eX(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=+f;g=+g;var j=0,k=0,l=0.0,m=0;j=i;i=i+8|0;k=j|0;l=+zO(d,k);do{if((a[c[k>>2]|0]|0)==0){if(l<f|l>g){m=1710864;break}if((e|0)==1024){if(+(~~l>>>0>>>0)+ +((+R(l)>=1.0?l>0.0?(ah(+Q(l/4294967296.0),4294967295.0)|0)>>>0:~~+ab((l- +(~~l>>>0))/4294967296.0)>>>0:0)|0)*4294967296.0!=l){m=1685568;break}i=j;return+l}else if((e|0)==128){if(+(~~l|0)!=l){m=1667208;break}i=j;return+l}else{i=j;return+l}}else{m=1622328}}while(0);AN(0,8,m,(m=i,i=i+32|0,c[m>>2]=b,c[m+8>>2]=d,h[m+16>>3]=f,h[m+24>>3]=g,m)|0);i=m;eW(1);return 0.0}function eY(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;i=i+8|0;f=e|0;if((BI(f,b,d)|0)<0){AN(0,8,1652280,(g=i,i=i+24|0,c[g>>2]=(d|0)!=0?1635104:1628192,c[g+8>>2]=a,c[g+16>>2]=b,g)|0);i=g;eW(1);return 0}else{i=e;return(G=c[f+4>>2]|0,c[f>>2]|0)|0}return 0}function eZ(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;g=i;i=i+64|0;h=a|0;a=c[h>>2]|0;if((a|0)==0){j=bH(10)|0;i=g;return}k=g|0;if((f|0)==0){l=1;m=h;n=a;while(1){o=c[m+4>>2]|0;do{if((o&d|0)==(d|0)){if((o&e|0)!=0){p=l;break}if((l|0)==0){q=n}else{b2(b|0)|0;q=c[m>>2]|0}y_(k,q,64)|0;r=m+16|0;if((c[r>>2]|0)!=0){y$(k,1616760,64)|0;s=c[r>>2]|0;y$(k,s,64)|0}s=c[m+12>>2]|0;bT(1611960,(t=i,i=i+16|0,c[t>>2]=k,c[t+8>>2]=s,t)|0)|0;i=t;p=0}else{p=l}}while(0);o=m+20|0;s=c[o>>2]|0;if((s|0)==0){break}else{l=p;m=o;n=s}}j=bH(10)|0;i=g;return}else{u=1;v=h;w=a}while(1){a=c[v+4>>2]|0;do{if((a&d|0)==(d|0)){if((a&f|0)==0){x=u;break}if((a&e|0)!=0){x=u;break}if((u|0)==0){y=w}else{b2(b|0)|0;y=c[v>>2]|0}y_(k,y,64)|0;h=v+16|0;if((c[h>>2]|0)!=0){y$(k,1616760,64)|0;n=c[h>>2]|0;y$(k,n,64)|0}n=c[v+12>>2]|0;bT(1611960,(t=i,i=i+16|0,c[t>>2]=k,c[t+8>>2]=n,t)|0)|0;i=t;x=0}else{x=u}}while(0);a=v+20|0;n=c[a>>2]|0;if((n|0)==0){break}else{u=x;v=a;w=n}}j=bH(10)|0;i=g;return}function e_(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=i;i=i+8|0;e=d|0;c[e>>2]=a;if((c[a+8>>2]|0)==0){f=a}else{a=e;Bp(a,0,b,0)|0;bH(10)|0;f=c[e>>2]|0}a=Bw(f,0)|0;if((a|0)==0){i=d;return}else{g=a}do{e_(g,b);g=Bw(c[e>>2]|0,g)|0;}while((g|0)!=0);i=d;return}function e$(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;g=i;h=bi(d|0,58)|0;if((h|0)==0){j=NL(d|0)|0}else{j=h-d|0}h=f|0;k=c[h>>2]|0;l=(k|0)==0;L3972:do{if(l){m=f;n=2883}else{o=f;p=k;while(1){if((ba(d|0,p|0,j|0)|0)==0){if((NL(p|0)|0)==(j|0)){break}}q=o+20|0;r=c[q>>2]|0;if((r|0)==0){m=q;n=2883;break L3972}else{o=q;p=r}}if((p|0)==0){m=o;n=2883}else{s=o;n=2894}}}while(0);L3979:do{if((n|0)==2883){if((a[d]|0)!=110){s=m;n=2894;break}if((a[d+1|0]|0)!=111){s=m;n=2894;break}j=d+2|0;r=bi(j|0,58)|0;if((r|0)==0){t=NL(j|0)|0}else{t=r-j|0}if(l){u=f;v=e;break}else{w=f;x=k}while(1){if((ba(j|0,x|0,t|0)|0)==0){if((NL(x|0)|0)==(t|0)){break}}r=w+20|0;q=c[r>>2]|0;if((q|0)==0){u=r;v=e;break L3979}else{w=r;x=q}}if((c[w>>2]|0)==0){u=w;v=e;break}u=w;v=(c[w+4>>2]&2|0)==0?e:1734536}}while(0);if((n|0)==2894){u=s;v=(c[s+4>>2]&2|0)==0?e:1729688}L3997:do{if((c[u>>2]|0)==0){e=c[h>>2]|0;L3999:do{if((e|0)!=0){s=f;n=e;while(1){if((ba(1725224,n|0,7)|0)==0){if((NL(n|0)|0)==7){break}}w=s+20|0;x=c[w>>2]|0;if((x|0)==0){break L3999}else{s=w;n=x}}if((n|0)!=0){y=s;break L3997}}}while(0);AN(0,16,1721264,(z=i,i=i+8|0,c[z>>2]=d,z)|0);i=z;A=-22;i=g;return A|0}else{y=u}}while(0);u=y+4|0;if((c[u>>2]&1|0)!=0&(v|0)==0){AN(0,16,1717184,(z=i,i=i+8|0,c[z>>2]=d,z)|0);i=z;A=-22;i=g;return A|0}z=e0(b,y,d,v)|0;if((z|0)<0){A=z;i=g;return A|0}A=c[u>>2]&1;i=g;return A|0}function e0(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0.0,E=0,F=0,G=0;j=i;i=i+104|0;k=j|0;l=j+8|0;m=j+16|0;n=j+24|0;o=j+32|0;p=j+40|0;q=d+4|0;r=c[q>>2]|0;if((r&49152|0)==0){t=c[d+8>>2]|0}else{t=b+(c[d+8>>2]|0)|0}do{if((r&32768|0)==0){u=t;v=r}else{w=t;x=bi(e|0,58)|0;y=t+4|0;z=(c[y>>2]|0)+1|0;if((z|0)>=134217727){AN(0,16,1744104,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;eW(1);return 0}B=A2(c[w>>2]|0,z<<4)|0;if((B|0)==0){AN(0,16,1743512,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;eW(1);return 0}else{C=c[y>>2]|0;NM(B+(C<<4)|0,0,z-C<<4|0);c[y>>2]=z;c[w>>2]=B;B=A9((x|0)==0?3119e3:x+1|0)|0;c[(c[w>>2]|0)+((c[y>>2]|0)-1<<4)>>2]=B;u=(c[w>>2]|0)+((c[y>>2]|0)-1<<4)+8|0;v=c[q>>2]|0;break}}}while(0);do{if((v&8|0)==0){if((v&130|0)!=0){D=+zO(f,o);if((a[c[o>>2]|0]|0)!=0){E=1622328;AN(0,8,E,(A=i,i=i+32|0,c[A>>2]=e,c[A+8>>2]=f,h[A+16>>3]=-2147483648.0,h[A+24>>3]=2147483647.0,A)|0);i=A;eW(1);return 0}if(D<-2147483648.0|D>2147483647.0){E=1710864;AN(0,8,E,(A=i,i=i+32|0,c[A>>2]=e,c[A+8>>2]=f,h[A+16>>3]=-2147483648.0,h[A+24>>3]=2147483647.0,A)|0);i=A;eW(1);return 0}if(+(~~D>>>0>>>0)+ +((+R(D)>=1.0?D>0.0?(ah(+Q(D/4294967296.0),4294967295.0)|0)>>>0:~~+ab((D- +(~~D>>>0))/4294967296.0)>>>0:0)|0)*4294967296.0!=D){E=1685568;AN(0,8,E,(A=i,i=i+32|0,c[A>>2]=e,c[A+8>>2]=f,h[A+16>>3]=-2147483648.0,h[A+24>>3]=2147483647.0,A)|0);i=A;eW(1);return 0}else{c[u>>2]=~~D;break}}if((v&1024|0)!=0){D=+zO(f,n);if((a[c[n>>2]|0]|0)!=0){F=1622328;AN(0,8,F,(A=i,i=i+32|0,c[A>>2]=e,c[A+8>>2]=f,h[A+16>>3]=-9223372036854776000.0,h[A+24>>3]=9223372036854776000.0,A)|0);i=A;eW(1);return 0}if(D<-9223372036854776000.0|D>9223372036854776000.0){F=1710864;AN(0,8,F,(A=i,i=i+32|0,c[A>>2]=e,c[A+8>>2]=f,h[A+16>>3]=-9223372036854776000.0,h[A+24>>3]=9223372036854776000.0,A)|0);i=A;eW(1);return 0}t=~~D>>>0;r=+R(D)>=1.0?D>0.0?(ah(+Q(D/4294967296.0),4294967295.0)|0)>>>0:~~+ab((D- +(~~D>>>0))/4294967296.0)>>>0:0;if(+(t>>>0)+ +(r|0)*4294967296.0!=D){F=1685568;AN(0,8,F,(A=i,i=i+32|0,c[A>>2]=e,c[A+8>>2]=f,h[A+16>>3]=-9223372036854776000.0,h[A+24>>3]=9223372036854776000.0,A)|0);i=A;eW(1);return 0}else{y=u;c[y>>2]=t;c[y+4>>2]=r;break}}if((v&65536|0)!=0){if((BI(m,f,1)|0)<0){AN(0,8,1652280,(A=i,i=i+24|0,c[A>>2]=1635104,c[A+8>>2]=e,c[A+16>>2]=f,A)|0);i=A;eW(1);return 0}else{r=c[m+4>>2]|0;y=u;c[y>>2]=c[m>>2];c[y+4>>2]=r;break}}if((v&256|0)!=0){D=+zO(f,l);if((a[c[l>>2]|0]|0)==0){g[u>>2]=D;break}else{AN(0,8,1622328,(A=i,i=i+32|0,c[A>>2]=e,c[A+8>>2]=f,h[A+16>>3]=+-s,h[A+24>>3]=+s,A)|0);i=A;eW(1);return 0}}if((v&131072|0)!=0){D=+zO(f,k);if((a[c[k>>2]|0]|0)==0){h[u>>3]=D;break}else{AN(0,8,1622328,(A=i,i=i+32|0,c[A>>2]=e,c[A+8>>2]=f,h[A+16>>3]=+-s,h[A+24>>3]=+s,A)|0);i=A;eW(1);return 0}}r=c[d+8>>2]|0;if((r|0)==0){break}y=c6[r&255](b,e,f)|0;if((y|0)>=0){break}r=p|0;NM(r|0,0,64);a[r]=0;zN(y,r,64)|0;AN(0,16,1706344,(A=i,i=i+24|0,c[A>>2]=f,c[A+8>>2]=e,c[A+16>>2]=r,A)|0);i=A;G=y;i=j;return G|0}else{y=A9(f)|0;A6(u);c[u>>2]=y}}while(0);if((c[q>>2]&2048|0)==0){G=0;i=j;return G|0}else{eW(0);return 0}return 0}function e1(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;d=i;e=b|0;f=b+4|0;g=c[f>>2]|0;AN(0,48,1713504,(h=i,i=i+16|0,c[h>>2]=c[c[e>>2]>>2],c[h+8>>2]=g,h)|0);i=h;g=b+12|0;j=b+8|0;b=0;while(1){if((b|0)>=(c[g>>2]|0)){k=2966;break}l=c[j>>2]|0;m=l+(b*12|0)|0;n=c[e>>2]|0;o=c[n+8>>2]|0;p=c[m>>2]|0;if((o|0)!=0){if((c[p+4>>2]&o|0)==0){k=2964;break}}o=l+(b*12|0)+4|0;q=c[p+12>>2]|0;r=l+(b*12|0)+8|0;s=c[r>>2]|0;AN(0,48,1707640,(h=i,i=i+24|0,c[h>>2]=c[o>>2],c[h+8>>2]=q,c[h+16>>2]=s,h)|0);i=h;s=e0(a,c[m>>2]|0,c[o>>2]|0,c[r>>2]|0)|0;if((s|0)<0){t=s;k=2968;break}else{b=b+1|0}}if((k|0)==2966){AN(0,48,1705448,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;t=0;i=d;return t|0}else if((k|0)==2968){i=d;return t|0}else if((k|0)==2964){k=c[p+12>>2]|0;p=c[n>>2]|0;n=c[f>>2]|0;AN(0,16,1710688,(h=i,i=i+32|0,c[h>>2]=c[l+(b*12|0)+4>>2],c[h+8>>2]=k,c[h+16>>2]=p,c[h+24>>2]=n,h)|0);i=h;t=-22;i=d;return t|0}return 0}function e2(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;if((b|0)<=1){g=0;return g|0}h=e|0;i=1;while(1){j=c[d+(i<<2)>>2]|0;k=j+1|0;if((a[j]|0)==45){l=bi(k|0,58)|0;if((l|0)==0){m=NL(k|0)|0}else{m=l-k|0}l=c[h>>2]|0;n=(l|0)==0;L4108:do{if(n){o=e;p=2982}else{q=e;r=l;while(1){if((ba(k|0,r|0,m|0)|0)==0){if((NL(r|0)|0)==(m|0)){break}}s=q+20|0;t=c[s>>2]|0;if((t|0)==0){o=s;p=2982;break L4108}else{q=s;r=t}}if((r|0)==0){o=q;p=2982}else{u=q}}}while(0);L4115:do{if((p|0)==2982){p=0;if((a[k]|0)!=110){u=o;break}if((a[j+2|0]|0)!=111){u=o;break}t=j+3|0;s=bi(t|0,58)|0;if((s|0)==0){v=NL(t|0)|0}else{v=s-t|0}if(n){u=e;break}else{w=e;x=l}while(1){if((ba(t|0,x|0,v|0)|0)==0){if((NL(x|0)|0)==(v|0)){u=w;break L4115}}s=w+20|0;y=c[s>>2]|0;if((y|0)==0){u=s;break}else{w=s;x=y}}}}while(0);l=c[u>>2]|0;if((l|0)==0){if((a8(k|0,f|0)|0)==0){g=i;p=3e3;break}}else{if((a8(f|0,l|0)|0)==0){g=i;p=2998;break}}z=(c[u+4>>2]&1)+i|0}else{z=i}l=z+1|0;if((l|0)<(b|0)){i=l}else{g=0;p=2997;break}}if((p|0)==2998){return g|0}else if((p|0)==2997){return g|0}else if((p|0)==3e3){return g|0}return 0}function e3(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;f=i;g=e2(b,d,e,1703736)|0;if((g|0)==0){h=e2(b,d,e,1701096)|0;if((h|0)!=0){j=h;k=3003}}else{j=g;k=3003}do{if((k|0)==3003){g=c[d+(j+1<<2)>>2]|0;if((g|0)==0){break}e4(0,0,g)|0}}while(0);j=e2(b,d,e,1698096)|0;e=ca(1695976)|0;if((e|0)==0&(j|0)==0){i=f;return}e5(e)|0;e=c[541002]|0;if((e|0)==0){i=f;return}aW(1693600,14,1,e|0)|0;if((b|0)>0){e=b-1|0;j=0;do{g=c[d+(j<<2)>>2]|0;h=a[g]|0;L4155:do{if(h<<24>>24==0){k=3010}else{l=g;m=h;while(1){if((m-43&255)>=16){if(!((m-64&255)<27|m<<24>>24==95|(m-97&255)<26)){break}}n=l+1|0;o=a[n]|0;if(o<<24>>24==0){k=3010;break L4155}else{l=n;m=o}}l=c[541002]|0;if(m<<24>>24==0){p=l;k=3015;break}bw(34,l|0)|0;l=a[g]|0;if(l<<24>>24!=0){o=g;n=l;do{l=n&255;do{if((n<<24>>24|0)==92|(n<<24>>24|0)==34|(n<<24>>24|0)==36|(n<<24>>24|0)==96){q=c[541002]|0;cx(q|0,1706816,(r=i,i=i+8|0,c[r>>2]=l,r)|0)|0;i=r}else{q=c[541002]|0;if((n-32&255)>94){cx(q|0,1706552,(r=i,i=i+8|0,c[r>>2]=l,r)|0)|0;i=r;break}else{bw(l|0,q|0)|0;break}}}while(0);o=o+1|0;n=a[o]|0;}while(n<<24>>24!=0)}bw(34,c[541002]|0)|0}}while(0);if((k|0)==3010){k=0;p=c[541002]|0;k=3015}if((k|0)==3015){k=0;aZ(g|0,p|0)|0}bw(((j|0)<(e|0)?32:10)|0,c[541002]|0)|0;j=j+1|0;}while((j|0)<(b|0))}aT(c[541002]|0)|0;i=f;return}function e4(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0;d=i;i=i+8|0;b=d|0;f=a4(e|0,1650512)|0;c[b>>2]=f;g=(f|0)==0;AQ(g&1);if((f|0)==(e|0)){h=e+((a[e+6|0]|0)==43|6)|0}else{h=e}do{if(g){j=0}else{if((a[h]|0)!=0){j=0;break}i=d;return 0}}while(0);while(1){if(j>>>0>=8){break}if((a8(c[120808+(j<<3)>>2]|0,h|0)|0)==0){k=3036;break}else{j=j+1|0}}if((k|0)==3036){AP(c[120812+(j<<3)>>2]|0);i=d;return 0}j=aU(h|0,b|0,10)|0;if((a[c[b>>2]|0]|0)!=0){AN(0,8,1648928,(b=i,i=i+8|0,c[b>>2]=h,b)|0);i=b;AN(0,8,1646472,(b=i,i=i+8|0,c[b>>2]=1661640,b)|0);i=b;AN(0,8,1646472,(b=i,i=i+8|0,c[b>>2]=1660240,b)|0);i=b;AN(0,8,1646472,(b=i,i=i+8|0,c[b>>2]=1659048,b)|0);i=b;AN(0,8,1646472,(b=i,i=i+8|0,c[b>>2]=1657728,b)|0);i=b;AN(0,8,1646472,(b=i,i=i+8|0,c[b>>2]=1655504,b)|0);i=b;AN(0,8,1646472,(b=i,i=i+8|0,c[b>>2]=1654024,b)|0);i=b;AN(0,8,1646472,(b=i,i=i+8|0,c[b>>2]=1652184,b)|0);i=b;AN(0,8,1646472,(b=i,i=i+8|0,c[b>>2]=1690128,b)|0);i=b;eW(1);return 0}AP(j);i=d;return 0}function e5(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;d=i;i=i+1120|0;e=d|0;f=d+8|0;g=d+16|0;h=d+24|0;j=d+32|0;k=d+1056|0;c[e>>2]=b;if((c[541002]|0)!=0){l=0;i=d;return l|0}cU(h|0)|0;m=a0(h|0)|0;L4203:do{if((b|0)==0){n=0}else{h=0;o=0;p=b;while(1){if((a[p]|0)==0){n=h;break L4203}q=Bt(e,1709472,1709384,0,f,g)|0;if((q|0)<0){break}r=c[e>>2]|0;if((a[r]|0)!=0){c[e>>2]=r+1}r=c[f>>2]|0;if((a8(r|0,1708936)|0)==0){A4(h);s=c[g>>2]|0;c[g>>2]=0;t=s;u=0}else{AN(0,16,1708600,(v=i,i=i+8|0,c[v>>2]=r,v)|0);i=v;t=h;u=c[g>>2]|0}A4(u);A4(c[f>>2]|0);r=c[e>>2]|0;if((r|0)==0){n=t;break L4203}else{h=t;o=o+1|0;p=r}}if((o|0)==0){n=h;break}p=k|0;NM(p|0,0,64);a[p]=0;zN(q,p,64)|0;AN(0,16,1709240,(v=i,i=i+8|0,c[v>>2]=p,v)|0);i=v;n=h}}while(0);y9(j,0,1);q=m+20|0;k=m+16|0;t=m+12|0;e=m+8|0;f=m+4|0;u=m|0;m=(n|0)!=0?n:1708480;while(1){g=m+1|0;b=a[m]|0;if((b<<24>>24|0)==0){break}else if((b<<24>>24|0)!=37){zc(j,b,1);m=g;continue}b=m+2|0;p=a[g]|0;if(p<<24>>24==0){break}g=p<<24>>24;if((g|0)==37){zc(j,p,1);m=b;continue}else if((g|0)==116){p=(c[k>>2]|0)+1|0;r=c[t>>2]|0;s=c[e>>2]|0;w=c[f>>2]|0;x=c[u>>2]|0;zb(j,1707368,(v=i,i=i+48|0,c[v>>2]=(c[q>>2]|0)+1900,c[v+8>>2]=p,c[v+16>>2]=r,c[v+24>>2]=s,c[v+32>>2]=w,c[v+40>>2]=x,v)|0);i=v;m=b;continue}else if((g|0)==112){zb(j,1707456,(v=i,i=i+8|0,c[v>>2]=90712,v)|0);i=v;m=b;continue}else{m=b;continue}}A4(n);if((c[j+4>>2]|0)>>>0>=(c[j+8>>2]|0)>>>0){AN(0,16,1708112,(v=i,i=i+1|0,i=i+7&-8,c[v>>2]=0,v)|0);i=v;l=-12;i=d;return l|0}n=j|0;m=bV(c[n>>2]|0,1707872)|0;c[541002]=m;if((m|0)==0){m=c[n>>2]|0;b=ch()|0;g=ci(c[b>>2]|0)|0;AN(0,16,1707760,(v=i,i=i+16|0,c[v>>2]=m,c[v+8>>2]=g,v)|0);i=v;l=-(c[b>>2]|0)|0;i=d;return l|0}else{AR(52);b=(c[q>>2]|0)+1900|0;q=(c[k>>2]|0)+1|0;k=c[t>>2]|0;t=c[e>>2]|0;e=c[f>>2]|0;f=c[u>>2]|0;u=c[n>>2]|0;AN(0,32,1707568,(v=i,i=i+64|0,c[v>>2]=90712,c[v+8>>2]=b,c[v+16>>2]=q,c[v+24>>2]=k,c[v+32>>2]=t,c[v+40>>2]=e,c[v+48>>2]=f,c[v+56>>2]=u,v)|0);i=v;zd(j,0)|0;l=0;i=d;return l|0}return 0}function e6(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;b=i;i=i+168|0;f=b|0;g=b+128|0;h=b+136|0;j=b+144|0;k=b+152|0;l=b+160|0;c[g>>2]=ox()|0;c[h>>2]=lW()|0;if((a8(d|0,1690128)|0)==0){m=3076}else{if((a8(d|0,1687136)|0)==0){m=3076}}if((m|0)==3076){AP(48)}n=bi(d|0,58)|0;if((n|0)==0){o=d+(NL(d|0)|0)|0}else{o=n}n=f|0;f=o-d+1|0;y_(n,d,f>>>0<128?f:128)|0;f=g;g=Bc(f,n,0,0,3)|0;do{if((g|0)==0){n=a[d]|0;if(!((n<<24>>24|0)==118|(n<<24>>24|0)==97|(n<<24>>24|0)==115)){p=0;break}n=Bc(f,d+1|0,0,0,2)|0;if((n|0)==0){p=0}else{q=n;m=3082}}else{q=g;m=3082}}while(0);if((m|0)==3082){m=(c[q+12>>2]|0)==0?32:0;zK(2981336,d,e,m)|0;p=1}m=Bc(h,d,0,0,3)|0;do{if((m|0)==0){c[j>>2]=s2()|0;if((p|0)!=0){break}if((Bc(j,d,0,0,3)|0)!=0){h=Bd(c[540558]|0,d,e,0)|0;if((h|0)>=0){break}AN(0,16,1683552,(r=i,i=i+8|0,c[r>>2]=d,r)|0);i=r;s=h;i=b;return s|0}c[k>>2]=rN()|0;h=Bc(k,d,0,0,3)|0;if((h|0)==0){s=-1414549496;i=b;return s|0}q=rO()|0;c[l>>2]=q;g=Bd(q,d,e,0)|0;rQ(l);if((g|0)<0){AN(0,16,1683552,(r=i,i=i+8|0,c[r>>2]=d,r)|0);i=r;s=g;i=b;return s|0}else{g=(c[h+12>>2]|0)==0?32:0;zK(2162240,d,e,g)|0;s=0;i=b;return s|0}}else{g=(c[m+12>>2]|0)==0?32:0;zK(2225160,d,e,g)|0;if((p|0)!=0){AN(0,40,1685496,(r=i,i=i+8|0,c[r>>2]=d,r)|0);i=r}c[j>>2]=s2()|0}}while(0);c[k>>2]=rN()|0;s=0;i=b;return s|0}function e7(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=a+40|0;d=a+36|0;if((c[b>>2]|0)>0){e=0;do{f=c[d>>2]|0;g=f+(e*12|0)+8|0;h=f+(e*12|0)+4|0;if((c[g>>2]|0)>0){f=0;do{A6((c[h>>2]|0)+(f*36|0)+8|0);zL((c[h>>2]|0)+(f*36|0)+16|0);zL((c[h>>2]|0)+(f*36|0)+20|0);zL((c[h>>2]|0)+(f*36|0)+24|0);tb(c[(c[h>>2]|0)+(f*36|0)+28>>2]|0);zL((c[h>>2]|0)+(f*36|0)+32|0);f=f+1|0;}while((f|0)<(c[g>>2]|0))}A6(h);e=e+1|0;}while((e|0)<(c[b>>2]|0))}A6(d);A6(a+52|0);A6(a+8|0);tb(c[540558]|0);c[540558]=0;zL(2162240);zL(2225160);zL(2981336);zL(2164e3);return}function e8(b,d,e,f,g,h){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0;j=i;NM(b|0,0,80);k=b+40|0;c[k>>2]=h;l=A7(h*12|0)|0;m=l;n=b+36|0;c[n>>2]=m;if((l|0)==0){eW(1);return 0}o=c[k>>2]|0;L4288:do{if((o|0)>0){c[l>>2]=g;if((o|0)>1){p=1;q=m}else{break}while(1){c[q+(p*12|0)>>2]=g+(p*12|0);r=p+1|0;if((r|0)>=(c[k>>2]|0)){break L4288}p=r;q=c[n>>2]|0}}}while(0);c[b>>2]=172936;c[b+4>>2]=3119e3;c[540558]=ta(16,16,0,16,16,0,4,0,0,0)|0;AN(0,48,1681848,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;L4294:do{if((d|0)>1){p=(h|0)>0;k=f|0;m=b+44|0;o=g|0;l=1;r=-2;L4296:while(1){s=r+1|0;t=l;L4298:while(1){u=t+1|0;v=c[e+(t<<2)>>2]|0;AN(0,48,1679920,(q=i,i=i+8|0,c[q>>2]=v,q)|0);i=q;L4300:do{if((a[v]|0)==45){w=v+1|0;x=a[w]|0;if((x<<24>>24|0)==45){if((a[v+2|0]|0)==0){break L4298}}else if((x<<24>>24|0)==0){y=3121;break}if((s|0)==(u|0)){y=3121;break}L4306:do{if(p){z=0;while(1){A=c[g+(z*12|0)+4>>2]|0;if((A|0)!=0){if((a8(A|0,w|0)|0)==0){break}}A=z+1|0;if((A|0)<(h|0)){z=A}else{break L4306}}if((z|0)<=-1){break}A=c[e+(u<<2)>>2]|0;if((A|0)==0){y=3133;break L4296}B=t+2|0;C=c[n>>2]|0;D=C+(z*12|0)+4|0;E=C+(z*12|0)+8|0;F=c[E>>2]|0;G=F+1|0;if((G|0)>=59652323){y=3135;break L4296}H=A2(c[D>>2]|0,G*36|0)|0;if((H|0)==0){y=3137;break L4296}I=c[E>>2]|0;NM(H+(I*36|0)|0,0,(G-I|0)*36|0|0);c[E>>2]=G;G=H;c[D>>2]=G;D=G+(F*36|0)|0;H=D;NN(H|0,m|0,36)|0;c[G+(F*36|0)+4>>2]=A;c[D>>2]=c[C+(z*12|0)>>2];c[G+(F*36|0)+28>>2]=c[540558];c[G+(F*36|0)+32>>2]=c[540560];c[G+(F*36|0)+16>>2]=c[745334];c[G+(F*36|0)+20>>2]=c[556290];c[G+(F*36|0)+24>>2]=c[541e3];c[745334]=0;c[556290]=0;c[541e3]=0;c[540558]=0;c[540560]=0;c[540558]=ta(16,16,0,16,16,0,4,0,0,0)|0;NM(m|0,0,36);AN(0,48,1674248,(q=i,i=i+16|0,c[q>>2]=c[g+(z*12|0)>>2],c[q+8>>2]=A,q)|0);i=q;J=B;break L4300}}while(0);B=bi(w|0,58)|0;if((B|0)==0){K=NL(w|0)|0}else{K=B-w|0}B=c[k>>2]|0;L4322:do{if((B|0)!=0){A=f;F=B;while(1){if((ba(w|0,F|0,K|0)|0)==0){if((NL(F|0)|0)==(K|0)){break}}G=A+20|0;C=c[G>>2]|0;if((C|0)==0){break L4322}else{A=G;F=C}}z=A|0;if((F|0)==0){break}C=c[A+4>>2]|0;do{if((C&2048|0)==0){if((C&1|0)==0){L=1729688;M=u;break}G=c[e+(u<<2)>>2]|0;if((G|0)==0){y=3151;break L4296}else{L=G;M=t+2|0}}else{L=c[e+(u<<2)>>2]|0;M=t+2|0}}while(0);e9(b,A,w,L);C=c[A+12>>2]|0;AN(0,48,1672400,(q=i,i=i+24|0,c[q>>2]=c[z>>2],c[q+8>>2]=C,c[q+16>>2]=L,q)|0);i=q;J=M;break L4300}}while(0);N=e+(u<<2)|0;B=c[N>>2]|0;if((B|0)==0){O=x}else{P=e6(0,w,B)|0;if((P|0)>-1){B=c[N>>2]|0;AN(0,48,1670456,(q=i,i=i+16|0,c[q>>2]=w,c[q+8>>2]=B,q)|0);i=q;J=t+2|0;break}if((P|0)!=-1414549496){y=3158;break L4296}O=a[w]|0}if(O<<24>>24!=110){y=3172;break L4296}if((a[v+2|0]|0)!=111){y=3172;break L4296}B=v+3|0;C=bi(B|0,58)|0;if((C|0)==0){Q=NL(B|0)|0}else{Q=C-B|0}C=c[k>>2]|0;L4349:do{if((C|0)==0){R=f}else{F=f;G=C;while(1){if((ba(B|0,G|0,Q|0)|0)==0){if((NL(G|0)|0)==(Q|0)){R=F;break L4349}}D=F+20|0;H=c[D>>2]|0;if((H|0)==0){R=D;break}else{F=D;G=H}}}}while(0);if((R|0)==0){y=3172;break L4296}B=R|0;if((c[B>>2]|0)==0){y=3172;break L4296}if((c[R+4>>2]&2|0)==0){y=3172;break L4296}e9(b,R,w,1734536);C=c[R+12>>2]|0;AN(0,48,1667136,(q=i,i=i+16|0,c[q>>2]=c[B>>2],c[q+8>>2]=C,q)|0);i=q;J=u}else{y=3121}}while(0);if((y|0)==3121){y=0;C=c[n>>2]|0;B=C+4|0;x=C+8|0;G=c[x>>2]|0;F=G+1|0;if((F|0)>=59652323){y=3122;break L4296}z=A2(c[B>>2]|0,F*36|0)|0;if((z|0)==0){y=3124;break L4296}A=c[x>>2]|0;NM(z+(A*36|0)|0,0,(F-A|0)*36|0|0);c[x>>2]=F;F=z;c[B>>2]=F;B=F+(G*36|0)|0;z=B;NN(z|0,m|0,36)|0;c[F+(G*36|0)+4>>2]=v;c[B>>2]=c[C>>2];c[F+(G*36|0)+28>>2]=c[540558];c[F+(G*36|0)+32>>2]=c[540560];c[F+(G*36|0)+16>>2]=c[745334];c[F+(G*36|0)+20>>2]=c[556290];c[F+(G*36|0)+24>>2]=c[541e3];c[745334]=0;c[556290]=0;c[541e3]=0;c[540558]=0;c[540560]=0;c[540558]=ta(16,16,0,16,16,0,4,0,0,0)|0;NM(m|0,0,36);AN(0,48,1677848,(q=i,i=i+8|0,c[q>>2]=c[o>>2],q)|0);i=q;J=u}if((J|0)<(d|0)){t=J}else{break L4294}}if((u|0)<(d|0)){l=u;r=u}else{break L4294}}if((y|0)==3172){AN(0,16,1666056,(q=i,i=i+8|0,c[q>>2]=w,q)|0);i=q;S=-1414549496;i=j;return S|0}else if((y|0)==3124){AN(0,16,1743512,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;eW(1);return 0}else if((y|0)==3133){AN(0,16,1675928,(q=i,i=i+8|0,c[q>>2]=w,q)|0);i=q;S=-22;i=j;return S|0}else if((y|0)==3151){AN(0,16,1675928,(q=i,i=i+8|0,c[q>>2]=w,q)|0);i=q;S=-22;i=j;return S|0}else if((y|0)==3158){r=c[N>>2]|0;AN(0,16,1668656,(q=i,i=i+16|0,c[q>>2]=w,c[q+8>>2]=r,q)|0);i=q;S=P;i=j;return S|0}else if((y|0)==3137){AN(0,16,1743512,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;eW(1);return 0}else if((y|0)==3122){AN(0,16,1744104,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;eW(1);return 0}else if((y|0)==3135){AN(0,16,1744104,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;eW(1);return 0}}}while(0);if((c[b+56>>2]|0)!=0|(c[745334]|0)!=0|(c[556290]|0)!=0|(c[541e3]|0)!=0){AN(0,24,1664528,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q}AN(0,48,1663224,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;S=0;i=j;return S|0}function e9(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0;f=i;g=(c[b+4>>2]&57344|0)==0?a|0:a+44|0;a=g+8|0;h=g+12|0;g=c[h>>2]|0;j=g+1|0;if((j|0)>=178956970){AN(0,16,1744104,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;eW(1)}l=A2(c[a>>2]|0,j*12|0)|0;if((l|0)==0){AN(0,16,1743512,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;eW(1)}else{k=c[h>>2]|0;NM(l+(k*12|0)|0,0,(j-k|0)*12|0|0);c[h>>2]=j;j=l;c[a>>2]=j;c[j+(g*12|0)>>2]=b;c[(c[a>>2]|0)+(((c[h>>2]|0)-1|0)*12|0)+4>>2]=d;c[(c[a>>2]|0)+(((c[h>>2]|0)-1|0)*12|0)+8>>2]=e;i=f;return}}function fa(a){a=a|0;return e5(0)|0}function fb(b,d,e){b=b|0;d=d|0;e=e|0;var f=0;d=i;i=i+8|0;b=d|0;f=aU(e|0,b|0,10)|0;if((a[c[b>>2]|0]|0)==0){A0(f);i=d;return 0}else{AN(0,8,1644832,(d=i,i=i+8|0,c[d>>2]=e,d)|0);i=d;eW(1);return 0}return 0}function fc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;b=i;i=i+8|0;a=b|0;c[a>>2]=zz()|0;e=zA(a,d)|0;if((e|0)<0){f=e;i=b;return f|0}zy(c[a>>2]|0);f=0;i=b;return f|0}function fd(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,j=0.0,k=0,l=0;b=i;i=i+16|0;f=b|0;g=b+8|0;j=+zO(e,f);do{if((a[c[f>>2]|0]|0)==0){if(j<0.0|j>2147483647.0){k=1710864;break}if(+(~~j>>>0>>>0)+ +((+R(j)>=1.0?j>0.0?(ah(+Q(j/4294967296.0),4294967295.0)|0)>>>0:~~+ab((j- +(~~j>>>0))/4294967296.0)>>>0:0)|0)*4294967296.0!=j){k=1685568;break}l=~~j;c[g>>2]=l;c[g+4>>2]=l+1;if((bt(0,g|0)|0)==0){i=b;return 0}a3(1643032);i=b;return 0}else{k=1622328}}while(0);AN(0,8,k,(k=i,i=i+32|0,c[k>>2]=d,c[k+8>>2]=e,h[k+16>>3]=0.0,h[k+24>>3]=2147483647.0,k)|0);i=k;eW(1);return 0}function fe(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=i;i=i+128|0;e=d|0;if((zN(b,e,128)|0)<0){f=ci(-b|0)|0}else{f=e}AN(0,16,1640496,(e=i,i=i+16|0,c[e>>2]=a,c[e+8>>2]=f,e)|0);i=e;i=d;return}function ff(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=i;if((e2(a,b,d,1638720)|0)!=0){i=e;return}AN(0,32,1712544,(d=i,i=i+8|0,c[d>>2]=90712,d)|0);i=d;AN(0,32,1712176,(d=i,i=i+16|0,c[d>>2]=c[22680],c[d+8>>2]=2013,d)|0);i=d;AN(0,32,1739008,(d=i,i=i+1|0,i=i+7&-8,c[d>>2]=0,d)|0);i=d;AN(0,32,1711912,(d=i,i=i+32|0,c[d>>2]=1716768,c[d+8>>2]=1711568,c[d+16>>2]=1711224,c[d+24>>2]=1711008,d)|0);i=d;AN(0,32,1709576,(d=i,i=i+8|0,c[d>>2]=1716768,d)|0);i=d;fg(5);fg(3);i=e;return}function fg(b){b=b|0;var d=0,e=0,f=0,g=0,h=0;d=i;e=(b&1|0)!=0?1716768:3119e3;f=(b&2|0)!=0;if(f){g=Cn()|0;AN(0,32,1716584,(h=i,i=i+64|0,c[h>>2]=e,c[h+8>>2]=1716144,c[h+16>>2]=52,c[h+24>>2]=39,c[h+32>>2]=100,c[h+40>>2]=g>>>16,c[h+48>>2]=g>>>8&255,c[h+56>>2]=g&255,h)|0);i=h}g=(b&4|0)!=0;do{if(g){b=Co()|0;if((a8(1714656,b|0)|0)==0){break}if(!(a[904]|0)){AN(0,32,1714432,(h=i,i=i+8|0,c[h>>2]=e,h)|0);i=h;a[904]=1}AN(0,32,1714136,(h=i,i=i+24|0,c[h>>2]=e,c[h+8>>2]=1716144,c[h+16>>2]=b,h)|0);i=h}}while(0);if(f){b=pq()|0;AN(0,32,1716584,(h=i,i=i+64|0,c[h>>2]=e,c[h+8>>2]=1713824,c[h+16>>2]=55,c[h+24>>2]=18,c[h+32>>2]=102,c[h+40>>2]=b>>>16,c[h+48>>2]=b>>>8&255,c[h+56>>2]=b&255,h)|0);i=h}do{if(g){b=pr()|0;if((a8(1714656,b|0)|0)==0){break}if(!(a[904]|0)){AN(0,32,1714432,(h=i,i=i+8|0,c[h>>2]=e,h)|0);i=h;a[904]=1}AN(0,32,1714136,(h=i,i=i+24|0,c[h>>2]=e,c[h+8>>2]=1713824,c[h+16>>2]=b,h)|0);i=h}}while(0);if(f){b=l8()|0;AN(0,32,1716584,(h=i,i=i+64|0,c[h>>2]=e,c[h+8>>2]=1713632,c[h+16>>2]=55,c[h+24>>2]=12,c[h+32>>2]=101,c[h+40>>2]=b>>>16,c[h+48>>2]=b>>>8&255,c[h+56>>2]=b&255,h)|0);i=h}do{if(g){b=l9()|0;if((a8(1714656,b|0)|0)==0){break}if(!(a[904]|0)){AN(0,32,1714432,(h=i,i=i+8|0,c[h>>2]=e,h)|0);i=h;a[904]=1}AN(0,32,1714136,(h=i,i=i+24|0,c[h>>2]=e,c[h+8>>2]=1713632,c[h+16>>2]=b,h)|0);i=h}}while(0);if(f){b=fI()|0;AN(0,32,1716584,(h=i,i=i+64|0,c[h>>2]=e,c[h+8>>2]=1713488,c[h+16>>2]=55,c[h+24>>2]=3,c[h+32>>2]=100,c[h+40>>2]=b>>>16,c[h+48>>2]=b>>>8&255,c[h+56>>2]=b&255,h)|0);i=h}do{if(g){b=fJ()|0;if((a8(1714656,b|0)|0)==0){break}if(!(a[904]|0)){AN(0,32,1714432,(h=i,i=i+8|0,c[h>>2]=e,h)|0);i=h;a[904]=1}AN(0,32,1714136,(h=i,i=i+24|0,c[h>>2]=e,c[h+8>>2]=1713488,c[h+16>>2]=b,h)|0);i=h}}while(0);if(f){b=fL()|0;AN(0,32,1716584,(h=i,i=i+64|0,c[h>>2]=e,c[h+8>>2]=1713328,c[h+16>>2]=3,c[h+24>>2]=80,c[h+32>>2]=100,c[h+40>>2]=b>>>16,c[h+48>>2]=b>>>8&255,c[h+56>>2]=b&255,h)|0);i=h}do{if(g){b=fM()|0;if((a8(1714656,b|0)|0)==0){break}if(!(a[904]|0)){AN(0,32,1714432,(h=i,i=i+8|0,c[h>>2]=e,h)|0);i=h;a[904]=1}AN(0,32,1714136,(h=i,i=i+24|0,c[h>>2]=e,c[h+8>>2]=1713328,c[h+16>>2]=b,h)|0);i=h}}while(0);if(f){b=s3()|0;AN(0,32,1716584,(h=i,i=i+64|0,c[h>>2]=e,c[h+8>>2]=1713184,c[h+16>>2]=2,c[h+24>>2]=3,c[h+32>>2]=100,c[h+40>>2]=b>>>16,c[h+48>>2]=b>>>8&255,c[h+56>>2]=b&255,h)|0);i=h}do{if(g){b=s4()|0;if((a8(1714656,b|0)|0)==0){break}if(!(a[904]|0)){AN(0,32,1714432,(h=i,i=i+8|0,c[h>>2]=e,h)|0);i=h;a[904]=1}AN(0,32,1714136,(h=i,i=i+24|0,c[h>>2]=e,c[h+8>>2]=1713184,c[h+16>>2]=b,h)|0);i=h}}while(0);if(f){f=rL()|0;AN(0,32,1716584,(h=i,i=i+64|0,c[h>>2]=e,c[h+8>>2]=1713016,c[h+16>>2]=0,c[h+24>>2]=17,c[h+32>>2]=102,c[h+40>>2]=f>>>16,c[h+48>>2]=f>>>8&255,c[h+56>>2]=f&255,h)|0);i=h}if(!g){i=d;return}g=rM()|0;if((a8(1714656,g|0)|0)==0){i=d;return}if(!(a[904]|0)){AN(0,32,1714432,(h=i,i=i+8|0,c[h>>2]=e,h)|0);i=h;a[904]=1}AN(0,32,1714136,(h=i,i=i+24|0,c[h>>2]=e,c[h+8>>2]=1713016,c[h+16>>2]=g,h)|0);i=h;i=d;return}function fh(a,b,d){a=a|0;b=b|0;d=d|0;d=i;AR(54);AN(0,32,1712544,(b=i,i=i+8|0,c[b>>2]=90712,b)|0);i=b;AN(0,32,1739008,(b=i,i=i+1|0,i=i+7&-8,c[b>>2]=0,b)|0);i=b;AN(0,32,1711912,(b=i,i=i+32|0,c[b>>2]=3119e3,c[b+8>>2]=1711568,c[b+16>>2]=1711224,c[b+24>>2]=1711008,b)|0);i=b;AN(0,32,1709576,(b=i,i=i+8|0,c[b>>2]=3119e3,b)|0);i=b;fg(2);i=d;return 0}function fi(a,b,d){a=a|0;b=b|0;d=d|0;d=i;bT(1636504,(b=i,i=i+24|0,c[b>>2]=90712,c[b+8>>2]=90712,c[b+16>>2]=90712,b)|0)|0;i=b;i=d;return 0}function fj(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;d=i;b2(72960)|0;b=1635080;while(1){a=jd(0)|0;L4498:do{if((a|0)==0){e=0;f=0;g=0;h=0;j=0}else{k=0;l=0;m=0;n=a;while(1){L4501:do{if((l|0)==0){o=n;while(1){p=c[o>>2]|0;if((a8(p|0,b|0)|0)>0){q=o;r=p;break L4501}p=jd(o)|0;if((p|0)==0){e=m;f=0;g=k;h=0;j=0;break L4498}else{o=p}}}else{o=n;while(1){p=c[o>>2]|0;if((a8(p|0,l|0)|0)<0){if((a8(p|0,b|0)|0)>0){q=o;r=p;break L4501}}p=jd(o)|0;if((p|0)==0){e=m;f=l;g=k;h=0;j=0;break L4498}else{o=p}}}}while(0);o=c[q+4>>2]|0;p=jd(q)|0;if((p|0)==0){e=o;f=r;g=1;h=0;j=0;break}else{k=1;l=r;m=o;n=p}}}}while(0);L4511:while(1){s=e;t=f;u=g;a=j;while(1){v=jc(a)|0;w=(t|0)==0;if((v|0)==0){break L4511}x=c[v>>2]|0;if(!w){if((a8(x|0,t|0)|0)>=0){y=u;z=t;A=s;break}}if((a8(x|0,b|0)|0)>0){B=c[v+4>>2]|0;C=x;D=0}else{B=s;C=t;D=u}if((C|0)==0){s=B;t=0;u=D;a=v}else{y=D;z=C;A=B;break}}e=A;f=z;g=y;h=(a8(x|0,z|0)|0)==0?1:h;j=v}if(w){break}bT(1634416,(a=i,i=i+32|0,c[a>>2]=(h|0)!=0?1633656:1616760,c[a+8>>2]=(u|0)!=0?1633184:1616760,c[a+16>>2]=t,c[a+24>>2]=(s|0)!=0?s:1616760,a)|0)|0;i=a;b=t}i=d;return 0}function fk(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;d=i;i=i+8|0;b=d|0;a=fl(b)|0;b2(72472)|0;e=c[b>>2]|0;if((a|0)==0){f=e;A4(f);i=d;return 0}else{g=0}do{b=c[e+(g<<2)>>2]|0;bH(32)|0;h=b|0;bT(((o9(c[h>>2]|0)|0)!=0?1633656:1631272)|0,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;bT(((o8(c[h>>2]|0)|0)!=0?1633184:1631272)|0,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;switch(c[b+4>>2]|0){case 4:{k=84;break};case 0:{k=86;break};case 2:{k=68;break};case 1:{k=65;break};case 3:{k=83;break};default:{k=63}}bH(k|0)|0;l=b+16|0;bT(((c[l>>2]&1|0)!=0?1630072:1631272)|0,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;bT(((c[l>>2]&2|0)!=0?1629288:1631272)|0,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;bT(((c[l>>2]&4|0)!=0?1628616:1631272)|0,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;l=b+8|0;m=c[b+12>>2]|0;bT(1628176,(j=i,i=i+16|0,c[j>>2]=c[l>>2],c[j+8>>2]=(m|0)==0?3119e3:m,j)|0)|0;i=j;m=0;L4538:while(1){b=c[h>>2]|0;n=oQ(m)|0;if((n|0)==0){o=0;break}else{p=n}while(1){if((c[p+12>>2]|0)==(b|0)){if((oS(p)|0)!=0){break}}n=oQ(p)|0;if((n|0)==0){o=0;break L4538}else{p=n}}if((p|0)==0){o=0;break}if((a8(c[p>>2]|0,c[l>>2]|0)|0)==0){m=p}else{q=3302;break}}if((q|0)==3302){q=0;m=c[h>>2]|0;bT(1719352,(j=i,i=i+8|0,c[j>>2]=173e4,j)|0)|0;i=j;b=oQ(0)|0;L4549:do{if((b|0)!=0){n=b;while(1){if((c[n+12>>2]|0)==(m|0)){if((oS(n)|0)!=0){break}}r=oQ(n)|0;if((r|0)==0){break L4549}else{n=r}}if((n|0)==0){break}else{s=n}while(1){bT(1718904,(j=i,i=i+8|0,c[j>>2]=c[s>>2],j)|0)|0;i=j;r=oQ(s)|0;if((r|0)==0){break L4549}else{t=r}while(1){if((c[t+12>>2]|0)==(m|0)){if((oS(t)|0)!=0){break}}r=oQ(t)|0;if((r|0)==0){break L4549}else{t=r}}if((t|0)==0){break}else{s=t}}}}while(0);bH(41)|0;o=0}L4565:while(1){m=c[h>>2]|0;b=oQ(o)|0;if((b|0)==0){break}else{u=b}while(1){if((c[u+12>>2]|0)==(m|0)){if((oR(u)|0)!=0){break}}b=oQ(u)|0;if((b|0)==0){break L4565}else{u=b}}if((u|0)==0){break}if((a8(c[u>>2]|0,c[l>>2]|0)|0)==0){o=u}else{q=3319;break}}if((q|0)==3319){q=0;l=c[h>>2]|0;bT(1719352,(j=i,i=i+8|0,c[j>>2]=1730248,j)|0)|0;i=j;m=oQ(0)|0;L4576:do{if((m|0)!=0){b=m;while(1){if((c[b+12>>2]|0)==(l|0)){if((oR(b)|0)!=0){break}}n=oQ(b)|0;if((n|0)==0){break L4576}else{b=n}}if((b|0)==0){break}else{v=b}while(1){bT(1718904,(j=i,i=i+8|0,c[j>>2]=c[v>>2],j)|0)|0;i=j;n=oQ(v)|0;if((n|0)==0){break L4576}else{w=n}while(1){if((c[w+12>>2]|0)==(l|0)){if((oR(w)|0)!=0){break}}n=oQ(w)|0;if((n|0)==0){break L4576}else{w=n}}if((w|0)==0){break}else{v=w}}}}while(0);bH(41)|0}bH(10)|0;g=g+1|0;}while(g>>>0<a>>>0);f=e;A4(f);i=d;return 0}function fl(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0;b=i;d=0;e=0;while(1){f=nU(e)|0;if((f|0)==0){break}else{d=d+1|0;e=f}}e=A8(d,4)|0;f=e;if((e|0)==0){AN(0,16,1717672,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g;eW(1);return 0}h=nU(0)|0;if((h|0)==0){j=0}else{k=0;l=h;while(1){h=k+1|0;c[f+(k<<2)>>2]=l;m=nU(l)|0;if((m|0)==0){j=h;break}else{k=h;l=m}}}if((j|0)==(d|0)){b6(e|0,d|0,4,96);c[a>>2]=f;i=b;return d|0}else{AN(0,0,1717376,(g=i,i=i+24|0,c[g>>2]=1717168,c[g+8>>2]=1717024,c[g+16>>2]=1318,g)|0);i=g;cw();return 0}return 0}function fm(a,b,c){a=a|0;b=b|0;c=c|0;fn(0);return 0}function fn(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;b=i;i=i+8|0;d=b|0;e=fl(d)|0;bT(1722136,(f=i,i=i+8|0,c[f>>2]=(a|0)!=0?1721752:1721464,f)|0)|0;i=f;g=c[d>>2]|0;if((e|0)==0){h=g;A4(h);i=b;return}d=(a|0)==0;a=0;do{j=c[g+(a<<2)>>2]|0;k=j|0;l=c[k>>2]|0;m=oQ(0)|0;L4615:do{if((m|0)!=0){n=j+4|0;o=j+8|0;p=l;q=m;while(1){r=q;L4619:while(1){do{if((c[r+12>>2]|0)==(p|0)){if(d){if((oS(r)|0)==0){break}else{break L4619}}else{if((oR(r)|0)==0){break}else{break L4619}}}}while(0);s=oQ(r)|0;if((s|0)==0){break L4615}else{r=s}}if((r|0)==0){break L4615}switch(c[n>>2]|0){case 3:{t=83;break};case 4:{t=84;break};case 0:{t=86;break};case 2:{t=68;break};case 1:{t=65;break};default:{t=63}}bT(1721256,(f=i,i=i+8|0,c[f>>2]=t,f)|0)|0;i=f;s=r+16|0;bT(((c[s>>2]&4096|0)!=0?1721016:1631272)|0,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0)|0;i=f;bT(((c[s>>2]&8192|0)!=0?1628616:1631272)|0,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0)|0;i=f;bT(((c[s>>2]&512|0)!=0?1720824:1631272)|0,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0)|0;i=f;bT(((c[s>>2]&1|0)!=0?1720608:1631272)|0,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0)|0;i=f;bT(((c[s>>2]&2|0)!=0?1633656:1631272)|0,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0)|0;i=f;s=r|0;u=c[r+4>>2]|0;bT(1628176,(f=i,i=i+16|0,c[f>>2]=c[s>>2],c[f+8>>2]=(u|0)==0?3119e3:u,f)|0)|0;i=f;u=c[o>>2]|0;if((a8(c[s>>2]|0,u|0)|0)!=0){bT(1720016,(f=i,i=i+8|0,c[f>>2]=u,f)|0)|0;i=f}bH(10)|0;p=c[k>>2]|0;q=oQ(r)|0;if((q|0)==0){break L4615}}}}while(0);a=a+1|0;}while(a>>>0<e>>>0);h=g;A4(h);i=b;return}function fo(a,b,c){a=a|0;b=b|0;c=c|0;fn(1);return 0}function fp(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;b2(72448)|0;d=nP(0)|0;if((d|0)==0){e=bH(10)|0;return 0}else{f=d}do{b2(c[f>>2]|0)|0;f=nP(f)|0;}while((f|0)!=0);e=bH(10)|0;return 0}function fq(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;d=i;i=i+8|0;b=d|0;c[b>>2]=0;b2(72408)|0;a=h1(b,0)|0;if((a|0)!=0){e=a;do{b2(e|0)|0;e=h1(b,0)|0;}while((e|0)!=0)}b2(72400)|0;e=h1(b,1)|0;if((e|0)==0){i=d;return 0}else{f=e}do{b2(f|0)|0;f=h1(b,1)|0;}while((f|0)!=0);i=d;return 0}function fr(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;e=i;i=i+64|0;d=e|0;b2(72216)|0;b=fX(0)|0;if((b|0)==0){i=e;return 0}f=d|0;g=d+60|0;d=b;do{b=d+12|0;h=d+20|0;j=c[d+8>>2]|0;if((j|0)==0){k=f;l=3393}else{m=f;n=0;while(1){if(!((c[j+(n*60|0)>>2]|0)!=0&m>>>0<g>>>0)){break}switch(c[j+(n*60|0)+4>>2]|0){case 3:{o=83;break};case 0:{o=86;break};case 4:{o=84;break};case 2:{o=68;break};case 1:{o=65;break};default:{o=63}}a[m]=o;m=m+1|0;n=n+1|0}if((n|0)==0){k=m;l=3393}else{p=m}}if((l|0)==3393){l=0;a[k]=(c[h>>2]&1|0)!=0?78:124;p=k+1|0}a[p]=45;j=p+2|0;a[p+1|0]=62;q=c[b>>2]|0;if((q|0)==0){r=j;l=3405}else{s=j;j=0;while(1){if(!((c[q+(j*60|0)>>2]|0)!=0&s>>>0<g>>>0)){break}switch(c[q+(j*60|0)+4>>2]|0){case 1:{t=65;break};case 4:{t=84;break};case 3:{t=83;break};case 0:{t=86;break};case 2:{t=68;break};default:{t=63}}a[s]=t;s=s+1|0;j=j+1|0}if((j|0)==0){r=s;l=3405}else{u=s}}if((l|0)==3405){l=0;a[r]=(c[h>>2]&2|0)!=0?78:124;u=r+1|0}a[u]=0;q=c[h>>2]|0;b=c[d>>2]|0;m=c[d+4>>2]|0;bT(1624328,(n=i,i=i+40|0,c[n>>2]=(q&196608|0)!=0?84:46,c[n+8>>2]=(q&4|0)!=0?83:46,c[n+16>>2]=b,c[n+24>>2]=f,c[n+32>>2]=m,n)|0)|0;i=n;d=fX(d)|0;}while((d|0)!=0);i=e;return 0}function fs(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0;e=i;b2(71960)|0;b=BQ(0)|0;if((b|0)==0){i=e;return 0}else{f=b}do{b=BR(f)|0;a=(s5(b)|0)!=0?73:46;g=(s6(b)|0)!=0?79:46;b=d[f+7|0]|0;h=c[f>>2]|0;j=d[f+4|0]|0;k=BN(f)|0;bT(1623096,(l=i,i=i+64|0,c[l>>2]=a,c[l+8>>2]=g,c[l+16>>2]=(b&8|0)!=0?72:46,c[l+24>>2]=(b&2|0)!=0?80:46,c[l+32>>2]=(b&4|0)!=0?66:46,c[l+40>>2]=h,c[l+48>>2]=j,c[l+56>>2]=k,l)|0)|0;i=l;f=BQ(f)|0;}while((f|0)!=0);i=e;return 0}function ft(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0;d=i;i=i+16|0;b=d|0;a=d+8|0;b2(71912)|0;e=0;do{f=NW(1,0,e|0)|0;g=G;h=zv(f,g)|0;c[a>>2]=h;if((h|0)!=0){h=zw(f,g)|0;g=c[a>>2]|0;bT(1622160,(j=i,i=i+16|0,c[j>>2]=g,c[j+8>>2]=h,j)|0)|0;i=j}e=e+1|0;}while((e|0)<63);b2(71856)|0;if((zx(0,b,a)|0)==0){k=0}else{i=d;return 0}do{e=c[a>>2]|0;if((e|0)!=0){bT(1621472,(j=i,i=i+8|0,c[j>>2]=e,j)|0)|0;i=j;e=0;h=1;while(1){g=c[b>>2]|0;f=c[b+4>>2]|0;if(!((g&h|0)==0&(f&e|0)==0)){l=NU(h,e,-1,-1)|0;m=(g&l|0)!=0|(f&G|0)!=0?1619208:3119e3;f=zv(h,e)|0;bT(1620088,(j=i,i=i+16|0,c[j>>2]=m,c[j+8>>2]=f,j)|0)|0;i=j}f=h<<1|0>>>31;m=e<<1|h>>>31;if((f|0)==0&(m|0)==0){break}else{e=m;h=f}}bH(10)|0}k=k+1|0;}while((zx(k,b,a)|0)==0);i=d;return 0}function fu(a,b,c){a=a|0;b=b|0;c=c|0;c=i;i=i+128|0;b=c|0;b2(B6(b,128,-1)|0)|0;b2(B6(b,128,0)|0)|0;b2(B6(b,128,1)|0)|0;b2(B6(b,128,2)|0)|0;b2(B6(b,128,3)|0)|0;b2(B6(b,128,4)|0)|0;b2(B6(b,128,5)|0)|0;b2(B6(b,128,6)|0)|0;b2(B6(b,128,7)|0)|0;b2(B6(b,128,8)|0)|0;b2(B6(b,128,9)|0)|0;i=c;return 0}function fv(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;d=i;i=i+8|0;b=d|0;AR(54);f=A9((e|0)!=0?e:3119e3)|0;c[b>>2]=f;e=bi(f|0,61)|0;if((e|0)==0){g=0;h=f}else{a[e]=0;g=e+1|0;h=c[b>>2]|0}if((a[h]|0)==0){dI(h,g);j=b;A6(j);i=d;return 0}if((a8(h|0,1618816)|0)==0){fw(g,0);j=b;A6(j);i=d;return 0}if((a8(h|0,1618432)|0)==0){fw(g,1);j=b;A6(j);i=d;return 0}if((a8(h|0,1617984)|0)==0){e=jj(g)|0;if((e|0)==0){AN(0,16,1735128,(k=i,i=i+8|0,c[k>>2]=g,k)|0);i=k;j=b;A6(j);i=d;return 0}f=c[e>>2]|0;l=c[e+4>>2]|0;bT(1732160,(k=i,i=i+16|0,c[k>>2]=f,c[k+8>>2]=l,k)|0)|0;i=k;l=c[e+12>>2]|0;if((l|0)!=0){bT(1734504,(k=i,i=i+8|0,c[k>>2]=l,k)|0)|0;i=k}l=c[e+20>>2]|0;if((l|0)==0){j=b;A6(j);i=d;return 0}e_(l,2);j=b;A6(j);i=d;return 0}if((a8(h|0,1617512)|0)==0){l=jh(g,0,0)|0;if((l|0)==0){AN(0,16,1735128,(k=i,i=i+8|0,c[k>>2]=g,k)|0);i=k;j=b;A6(j);i=d;return 0}e=c[l>>2]|0;f=c[l+4>>2]|0;bT(1734792,(k=i,i=i+16|0,c[k>>2]=e,c[k+8>>2]=f,k)|0)|0;i=k;f=c[l+12>>2]|0;if((f|0)!=0){bT(1734504,(k=i,i=i+8|0,c[k>>2]=f,k)|0)|0;i=k}f=c[l+8>>2]|0;if((f|0)!=0){bT(1734200,(k=i,i=i+8|0,c[k>>2]=f,k)|0)|0;i=k}f=c[l+20>>2]|0;do{if((f|0)!=0){e=nT(f)|0;if((e|0)==0){break}m=c[e+8>>2]|0;bT(1733800,(k=i,i=i+8|0,c[k>>2]=m,k)|0)|0;i=k}}while(0);f=c[l+16>>2]|0;do{if((f|0)!=0){m=nT(f)|0;if((m|0)==0){break}e=c[m+8>>2]|0;bT(1733552,(k=i,i=i+8|0,c[k>>2]=e,k)|0)|0;i=k}}while(0);f=c[l+24>>2]|0;do{if((f|0)!=0){e=nT(f)|0;if((e|0)==0){break}m=c[e+8>>2]|0;bT(1732960,(k=i,i=i+8|0,c[k>>2]=m,k)|0)|0;i=k}}while(0);f=c[l+36>>2]|0;if((f|0)==0){j=b;A6(j);i=d;return 0}e_(f,1);j=b;A6(j);i=d;return 0}if((a8(h|0,1617024)|0)!=0){dI(h,g);j=b;A6(j);i=d;return 0}h=fW(g)|0;if((g|0)==0){AN(0,16,1741976,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;j=b;A6(j);i=d;return 0}if((h|0)==0){AN(0,16,1740288,(k=i,i=i+8|0,c[k>>2]=g,k)|0);i=k;j=b;A6(j);i=d;return 0}bT(1739608,(k=i,i=i+8|0,c[k>>2]=c[h>>2],k)|0)|0;i=k;g=c[h+4>>2]|0;if((g|0)!=0){bT(1739296,(k=i,i=i+8|0,c[k>>2]=g,k)|0)|0;i=k}g=h+20|0;if((c[g>>2]&4|0)!=0){b2(72928)|0}b2(72912)|0;f=h+8|0;l=fZ(c[f>>2]|0)|0;if((l|0)>0){m=0;do{e=f5(c[f>>2]|0,m)|0;n=Cp(f6(c[f>>2]|0,m)|0)|0;bT(1738496,(k=i,i=i+24|0,c[k>>2]=m,c[k+8>>2]=e,c[k+16>>2]=n,k)|0)|0;i=k;m=m+1|0;}while((m|0)<(l|0))}do{if((c[g>>2]&1|0)==0){if((l|0)!=0){break}b2(72880)|0}else{b2(72816)|0}}while(0);b2(72864)|0;l=h+12|0;m=fZ(c[l>>2]|0)|0;if((m|0)>0){f=0;do{n=f5(c[l>>2]|0,f)|0;e=Cp(f6(c[l>>2]|0,f)|0)|0;bT(1738496,(k=i,i=i+24|0,c[k>>2]=f,c[k+8>>2]=n,c[k+16>>2]=e,k)|0)|0;i=k;f=f+1|0;}while((f|0)<(m|0))}do{if((c[g>>2]&2|0)==0){if((m|0)!=0){break}b2(72784)|0}else{b2(72816)|0}}while(0);m=c[h+16>>2]|0;if((m|0)!=0){e_(m,65560)}if((c[g>>2]&196608|0)==0){j=b;A6(j);i=d;return 0}b2(72712)|0;j=b;A6(j);i=d;return 0}function fw(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;d=i;if((a|0)==0){AN(0,16,1731616,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;i=d;return}f=(b|0)!=0;if(f){g=pl(a)|0}else{g=pm(a)|0}if((g|0)!=0){fE(g);i=d;return}g=nV(a)|0;if((g|0)==0){AN(0,16,1729640,(e=i,i=i+8|0,c[e>>2]=a,e)|0);i=e;i=d;return}h=g|0;g=c[h>>2]|0;j=oQ(0)|0;do{if((j|0)!=0){k=(b|0)==0;l=0;m=g;n=j;L4844:while(1){o=n;L4846:while(1){do{if((c[o+12>>2]|0)==(m|0)){if(k){if((oS(o)|0)==0){break}else{break L4846}}else{if((oR(o)|0)==0){break}else{break L4846}}}}while(0);p=oQ(o)|0;if((p|0)==0){break L4844}else{o=p}}if((o|0)==0){break}fE(o);p=c[h>>2]|0;q=oQ(o)|0;if((q|0)==0){r=3530;break}else{l=1;m=p;n=q}}if((r|0)==3530){i=d;return}if((l|0)==0){break}i=d;return}}while(0);AN(0,16,1730808,(e=i,i=i+16|0,c[e>>2]=a,c[e+8>>2]=f?1730248:173e4,e)|0);i=e;i=d;return}function fx(){var a=0,b=0,c=0;a=bZ()|0;b=(((a-97|0)>>>0<26?a^32:a)|0)==89|0;c=a;while(1){if((c|0)==(-1|0)|(c|0)==10){break}c=bZ()|0}return b|0}function fy(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;f=i;g=bV(b|0,1616752)|0;if((g|0)==0){h=ch()|0;j=ci(c[h>>2]|0)|0;AN(0,16,1616416,(k=i,i=i+16|0,c[k>>2]=b,c[k+8>>2]=j,k)|0);i=k;l=-(c[h>>2]|0)|0;i=f;return l|0}cG(g|0,0,2)|0;c[e>>2]=bo(g|0)|0;cG(g|0,0,0)|0;h=c[e>>2]|0;if((h|0)==-1){j=ch()|0;m=ci(c[j>>2]|0)|0;AN(0,16,1616080,(k=i,i=i+8|0,c[k>>2]=m,k)|0);i=k;aR(g|0)|0;l=-(c[j>>2]|0)|0;i=f;return l|0}j=A1(h+1|0)|0;c[d>>2]=j;if((j|0)==0){AN(0,16,1615864,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;aR(g|0)|0;l=-12;i=f;return l|0}h=cv(j|0,1,c[e>>2]|0,g|0)|0;j=c[e>>2]|0;do{if(h>>>0<j>>>0){A4(c[d>>2]|0);if((bx(g|0)|0)==0){n=-541478725;break}m=ch()|0;o=ci(c[m>>2]|0)|0;AN(0,16,1615016,(k=i,i=i+16|0,c[k>>2]=b,c[k+8>>2]=o,k)|0);i=k;n=-(c[m>>2]|0)|0}else{c[e>>2]=j+1;a[(c[d>>2]|0)+j|0]=0;n=0}}while(0);aR(g|0)|0;l=n;i=f;return l|0}function fz(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;g=i;i=i+16|0;h=g|0;j=ca(1614320)|0;c[h>>2]=j;c[h+4>>2]=ca(1613896)|0;c[h+8>>2]=1613152;if((e|0)!=0){y_(a,d,b)|0;k=bV(a|0,1612632)|0;i=g;return k|0}e=(f|0)==0;l=0;m=j;while(1){do{if((m|0)==0){n=0}else{j=(l|0)!=1?3119e3:1611944;bc(a|0,b|0,1612304,(o=i,i=i+24|0,c[o>>2]=m,c[o+8>>2]=j,c[o+16>>2]=d,o)|0)|0;i=o;p=bV(a|0,1612632)|0;if((p|0)!=0|e){n=p;break}bc(a|0,b|0,1611672,(o=i,i=i+32|0,c[o>>2]=m,c[o+8>>2]=j,c[o+16>>2]=f,c[o+24>>2]=d,o)|0)|0;i=o;n=bV(a|0,1612632)|0}}while(0);o=l+1|0;if(!((o|0)<3&(n|0)==0)){k=n;break}l=o;m=c[h+(o<<2)>>2]|0}i=g;return k|0}function fA(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=i;f=m_(a,b,d)|0;if((f|0)>=0){i=e;return f|0}AN(a,16,1611424,(a=i,i=i+8|0,c[a>>2]=d,a)|0);i=a;i=e;return f|0}function fB(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;h=i;i=i+16|0;j=h|0;k=h+8|0;c[j>>2]=0;l=e+8|0;m=(c[l>>2]|0)!=0?1:2;c[k>>2]=ox()|0;do{if((g|0)==0){if((c[l>>2]|0)==0){n=o9(d)|0;break}else{n=o8(d)|0;break}}else{n=g}}while(0);g=c[(c[f+8>>2]|0)+8>>2]|0;if((g|0)==3){o=115;p=m|32}else if((g|0)==1){o=97;p=m|8}else if((g|0)==0){o=118;p=m|16}else{o=0;p=m}m=zJ(b,3119e3,0,2)|0;L4916:do{if((m|0)!=0){g=e;d=k;l=(n|0)==0;q=n+44|0;r=q;s=m;L4918:while(1){t=s|0;u=c[t>>2]|0;v=bi(u|0,58)|0;w=(v|0)!=0;do{if(w){x=v+1|0;y=m_(e,f,x)|0;if((y|0)<0){AN(g,16,1611424,(z=i,i=i+8|0,c[z>>2]=x,z)|0);i=z}if((y|0)==0){break}else if((y|0)!=1){A=0;break L4918}a[v]=0;B=c[t>>2]|0;C=3584}else{B=u;C=3584}}while(0);do{if((C|0)==3584){C=0;L4928:do{if((Bc(d,B,0,p,2)|0)==0){do{if(!l){if((c[q>>2]|0)==0){break}if((Bc(r,c[t>>2]|0,0,p,2)|0)!=0){C=3588;break L4928}}}while(0);u=c[t>>2]|0;if((a[u]|0)!=(o|0)){break}if((Bc(d,u+1|0,0,p,2)|0)==0){break}u=(c[t>>2]|0)+1|0;y=c[s+4>>2]|0;zK(j,u,y,0)|0}else{C=3588}}while(0);if((C|0)==3588){C=0;y=c[t>>2]|0;u=c[s+4>>2]|0;zK(j,y,u,0)|0}if(!w){break}a[v]=58}}while(0);s=zJ(b,3119e3,s,2)|0;if((s|0)==0){break L4916}}i=h;return A|0}}while(0);A=c[j>>2]|0;i=h;return A|0}function fC(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0;d=i;e=a+24|0;f=c[e>>2]|0;if((f|0)==0){g=0;i=d;return g|0}h=A7(f<<2)|0;f=h;if((h|0)==0){AN(0,16,1744904,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;g=0;i=d;return g|0}if((c[e>>2]|0)==0){g=f;i=d;return g|0}h=a+28|0;j=0;while(1){k=c[(c[h>>2]|0)+(j<<2)>>2]|0;c[f+(j<<2)>>2]=fB(b,c[(c[k+8>>2]|0)+48>>2]|0,a,k,0)|0;k=j+1|0;if(k>>>0<(c[e>>2]|0)>>>0){j=k}else{g=f;break}}i=d;return g|0}function fD(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0;f=i;if((2147483647/(b|0)|0|0)<=(e|0)){AN(0,16,1744104,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g;eW(1);return 0}if((c[d>>2]|0)>=(e|0)){h=a;i=f;return h|0}j=A2(a,ac(e,b)|0)|0;if((j|0)==0){AN(0,16,1743512,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g;eW(1);return 0}g=c[d>>2]|0;a=j+(ac(g,b)|0)|0;NM(a|0,0,ac(e-g|0,b)|0);c[d>>2]=e;h=j;i=f;return h|0}function fE(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0;b=i;i=i+144|0;d=b|0;e=b+16|0;f=(oR(a)|0)!=0?1728936:1728688;g=c[a>>2]|0;h=c[a+4>>2]|0;bT(1729344,(j=i,i=i+24|0,c[j>>2]=f,c[j+8>>2]=g,c[j+16>>2]=(h|0)==0?3119e3:h,j)|0)|0;i=j;if((c[a+8>>2]|0)>>>0<2){bT(1727848,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;h=c[a+16>>2]&12288;if((h|0)==12288){bT(1727376,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j}else if((h|0)==8192){bT(1726440,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j}else if((h|0)==4096){bT(1726944,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j}else{bT(1725912,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j}bH(10)|0}h=c[a+20>>2]|0;if((h|0)!=0){bT(1725472,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;g=c[h>>2]|0;if((g|0)!=0){f=h;h=g;do{g=c[f+4>>2]|0;bT(1725216,(j=i,i=i+16|0,c[j>>2]=h,c[j+8>>2]=g,j)|0)|0;i=j;f=f+8|0;h=c[f>>2]|0;}while((h|0)!=0)}bH(10)|0}h=c[a+24>>2]|0;if((h|0)!=0){bT(1724784,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;f=c[h>>2]|0;if((f|0)!=-1){g=h;h=f;do{f=BL(h)|0;bT(1724504,(j=i,i=i+8|0,c[j>>2]=f,j)|0)|0;i=j;g=g+4|0;h=c[g>>2]|0;}while((h|0)!=-1)}bH(10)|0}h=c[a+28>>2]|0;if((h|0)!=0){bT(1724200,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;g=c[h>>2]|0;if((g|0)!=0){f=d|0;d=h;h=g;do{bc(f|0,16,1723656,(j=i,i=i+8|0,c[j>>2]=h,j)|0)|0;i=j;bT(1724504,(j=i,i=i+8|0,c[j>>2]=f,j)|0)|0;i=j;d=d+4|0;h=c[d>>2]|0;}while((h|0)!=0)}bH(10)|0}h=c[a+32>>2]|0;if((h|0)!=0){bT(1723120,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;d=c[h>>2]|0;if((d|0)!=-1){f=h;h=d;do{d=B2(h)|0;bT(1724504,(j=i,i=i+8|0,c[j>>2]=d,j)|0)|0;i=j;f=f+4|0;h=c[f>>2]|0;}while((h|0)!=-1)}bH(10)|0}h=c[a+36>>2]|0;if((h|0)!=0){bT(1722808,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0)|0;i=j;f=c[h>>2]|0;d=c[h+4>>2]|0;if(!((f|0)==0&(d|0)==0)){g=e|0;e=h;h=d;d=f;do{zt(g,128,0,d,h);bT(1724504,(j=i,i=i+8|0,c[j>>2]=g,j)|0)|0;i=j;e=e+8|0;d=c[e>>2]|0;h=c[e+4>>2]|0;}while(!((d|0)==0&(h|0)==0))}bH(10)|0}h=c[a+44>>2]|0;if((h|0)==0){i=b;return}e_(h,3);i=b;return}function fF(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=c[a>>2]|0;a=c[d+4>>2]|0;e=c[b>>2]|0;b=c[e+4>>2]|0;if((a|0)==(b|0)){f=a8(c[d+8>>2]|0,c[e+8>>2]|0)|0;return f|0}else{f=a-b|0;return f|0}return 0}function fG(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0;f=i;i=i+1040|0;g=f|0;bQ(g|0,e|0);AL(a,b,d,e);e=f+16|0;AJ(a,b,d,g|0,e,1024,130832);aZ(e|0,c[541002]|0)|0;aT(c[541002]|0)|0;i=f;return}function fH(){if(a[340888]|0){return}a[340888]=1;return}function fI(){return 3605348}function fJ(){return 1742336}function fK(){if(a[339424]|0){return}a[339424]=1;fY(340536)|0;fY(340240)|0;fY(339944)|0;fY(339888)|0;fY(339072)|0;fY(338776)|0;fY(338480)|0;fY(338184)|0;fY(337952)|0;fY(339656)|0;fY(339600)|0;fY(337896)|0;fY(339832)|0;fY(338128)|0;fY(340832)|0;fY(339368)|0;hj();return}function fL(){return 217188}function fM(){return 1740880}function fN(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;f=i;if((c[a+40>>2]|0)>>>0<=b>>>0){g=-1;i=f;return g|0}if((c[d+24>>2]|0)>>>0<=e>>>0){g=-1;i=f;return g|0}h=a+32|0;if((c[(c[h>>2]|0)+(b<<2)>>2]|0)!=0){g=-1;i=f;return g|0}j=d+16|0;if((c[(c[j>>2]|0)+(e<<2)>>2]|0)!=0){g=-1;i=f;return g|0}k=a+28|0;l=c[(c[k>>2]|0)+(b*60|0)+4>>2]|0;m=d+12|0;if((l|0)!=(c[(c[m>>2]|0)+(e*60|0)+4>>2]|0)){n=c[a+8>>2]|0;o=Cp(l)|0;l=c[d+8>>2]|0;p=Cp(c[(c[m>>2]|0)+(e*60|0)+4>>2]|0)|0;AN(a,16,1703144,(q=i,i=i+48|0,c[q>>2]=n,c[q+8>>2]=b,c[q+16>>2]=(o|0)!=0?o:1679664,c[q+24>>2]=l,c[q+32>>2]=e,c[q+40>>2]=(p|0)!=0?p:1679664,q)|0);i=q;g=-22;i=f;return g|0}q=A7(168)|0;p=q;if((q|0)==0){g=-12;i=f;return g|0}c[(c[j>>2]|0)+(e<<2)>>2]=p;c[(c[h>>2]|0)+(b<<2)>>2]=p;c[q>>2]=a;c[q+8>>2]=d;d=c[k>>2]|0;c[q+4>>2]=d+(b*60|0);c[q+12>>2]=(c[m>>2]|0)+(e*60|0);c[q+16>>2]=c[d+(b*60|0)+4>>2];c[q+52>>2]=-1;g=0;i=f;return g|0}function fO(a){a=a|0;return c[a+148>>2]|0}function fP(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;f=i;g=a+12|0;h=a+8|0;j=c[h>>2]|0;k=((c[g>>2]|0)-(c[j+12>>2]|0)|0)/60|0;l=c[(c[a>>2]|0)+8>>2]|0;m=c[j+8>>2]|0;AN(j,40,1662920,(j=i,i=i+24|0,c[j>>2]=c[b+8>>2],c[j+8>>2]=l,c[j+16>>2]=m,j)|0);i=j;c[(c[(c[h>>2]|0)+16>>2]|0)+(k<<2)>>2]=0;j=fN(b,e,c[h>>2]|0,k)|0;if((j|0)<0){c[(c[(c[h>>2]|0)+16>>2]|0)+(k<<2)>>2]=a;n=j;i=f;return n|0}c[h>>2]=b;c[g>>2]=(c[b+12>>2]|0)+(d*60|0);c[(c[b+16>>2]|0)+(d<<2)>>2]=a;d=a+68|0;if((c[d>>2]|0)!=0){g5(d,(c[(c[b+32>>2]|0)+(e<<2)>>2]|0)+68|0)}d=a+76|0;if((c[d>>2]|0)!=0){g5(d,(c[(c[b+32>>2]|0)+(e<<2)>>2]|0)+76|0)}d=a+84|0;if((c[d>>2]|0)==0){n=0;i=f;return n|0}g4(d,(c[(c[b+32>>2]|0)+(e<<2)>>2]|0)+84|0);n=0;i=f;return n|0}function fQ(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;b=i;d=a+24|0;if((c[d>>2]|0)==0){e=0;i=b;return e|0}f=a+16|0;g=0;L5082:while(1){h=c[(c[f>>2]|0)+(g<<2)>>2]|0;do{if((h|0)!=0){j=h|0;k=c[j>>2]|0;if((c[k+24>>2]|0)==0){l=0}else{l=c[c[k+16>>2]>>2]|0}m=h+104|0;c[m>>2]=0;c[m+4>>2]=-2147483648;m=h+92|0;n=c[m>>2]|0;if((n|0)==1){o=3708;break L5082}else if((n|0)!=0){break}c[m>>2]=1;n=fQ(k)|0;if((n|0)<0){e=n;o=3756;break L5082}n=c[(c[h+4>>2]|0)+48>>2]|0;if((n|0)==0){p=c[j>>2]|0;if((c[p+24>>2]|0)!=1){o=3712;break L5082}}else{q=c1[n&255](h)|0;if((q|0)<0){o=3714;break L5082}}n=c[h+16>>2]|0;L5095:do{if((n|0)==0){k=h+56|0;do{if((c[k>>2]|0)==0){r=h+60|0;if((c[r>>2]|0)!=0){break}if((l|0)==0){c[h+56>>2]=1;c[r>>2]=1e6;break}else{r=l+56|0;s=k;t=c[r+4>>2]|0;c[s>>2]=c[r>>2];c[s+4>>2]=t;break}}}while(0);k=h+28|0;do{if((c[k>>2]|0)==0){t=h+32|0;if((c[t>>2]|0)!=0){o=3726;break}if((l|0)==0){c[h+28>>2]=1;c[t>>2]=1;if((c[h+20>>2]|0)==0){o=3736;break L5082}else{o=3735;break}}else{t=l+28|0;s=k;r=c[t+4>>2]|0;c[s>>2]=c[t>>2];c[s+4>>2]=r;break}}else{o=3726}}while(0);do{if((o|0)==3726){o=0;if((l|0)!=0){break}if((c[h+20>>2]|0)==0){o=3736;break L5082}else{o=3735}}}while(0);if((o|0)==3735){o=0;if((c[h+24>>2]|0)==0){o=3736;break L5082}else{break}}k=h+116|0;do{if((c[k>>2]|0)==0){if((c[h+120>>2]|0)!=0){break}r=l+116|0;s=k;t=c[r+4>>2]|0;c[s>>2]=c[r>>2];c[s+4>>2]=t}}while(0);k=h+20|0;if((c[k>>2]|0)==0){c[k>>2]=c[l+20>>2]}k=h+24|0;if((c[k>>2]|0)!=0){break}c[k>>2]=c[l+24>>2]}else if((n|0)==1){do{if((l|0)==0){u=c[h+56>>2]|0;o=3743}else{k=h+56|0;if((c[k>>2]|0)!=0){break L5095}if((c[h+60>>2]|0)==0){t=l+56|0;s=k;k=c[t>>2]|0;r=c[t+4>>2]|0;c[s>>2]=k;c[s+4>>2]=r;u=k;o=3743;break}else{break}}}while(0);if((o|0)==3743){o=0;if((u|0)!=0){break}}k=h+60|0;if((c[k>>2]|0)!=0){break}r=c[h+48>>2]|0;c[h+56>>2]=1;c[k>>2]=r}}while(0);n=c[(c[h+12>>2]|0)+48>>2]|0;if((n|0)!=0){v=c1[n&255](h)|0;if((v|0)<0){o=3748;break L5082}}c[m>>2]=2}}while(0);n=g+1|0;if(n>>>0<(c[d>>2]|0)>>>0){g=n}else{e=0;o=3754;break}}if((o|0)==3714){g=c[j>>2]|0;AN(g,16,1626448,(w=i,i=i+8|0,c[w>>2]=c[g+8>>2],w)|0);i=w;e=q;i=b;return e|0}else if((o|0)==3736){AN(c[j>>2]|0,16,162e4,(w=i,i=i+1|0,i=i+7&-8,c[w>>2]=0,w)|0);i=w;e=-22;i=b;return e|0}else if((o|0)==3708){AN(a,32,1646152,(w=i,i=i+1|0,i=i+7&-8,c[w>>2]=0,w)|0);i=w;e=0;i=b;return e|0}else if((o|0)==3754){i=b;return e|0}else if((o|0)==3756){i=b;return e|0}else if((o|0)==3748){AN(c[j>>2]|0,16,1615768,(w=i,i=i+8|0,c[w>>2]=c[(c[h+8>>2]|0)+8>>2],w)|0);i=w;e=v;i=b;return e|0}else if((o|0)==3712){AN(p,16,1632984,(w=i,i=i+1|0,i=i+7&-8,c[w>>2]=0,w)|0);i=w;e=-22;i=b;return e|0}return 0}function fR(a,b,d){a=a|0;b=b|0;d=d|0;d=i;i=i+128|0;if((c[b+16>>2]|0)==0){i=d;return}a=b+40|0;zt(d|0,128,-1,c[a>>2]|0,c[a+4>>2]|0);i=d;return}function fS(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;b=i;i=i+128|0;if((c[a+16>>2]|0)!=0){d=a+40|0;zt(b|0,128,-1,c[d>>2]|0,c[d+4>>2]|0)}d=a+144|0;if((c[d>>2]|0)!=0){e=-541478725;i=b;return e|0}f=a+152|0;if((c[f>>2]|0)!=0){AN(0,0,1744856,(g=i,i=i+24|0,c[g>>2]=1738112,c[g+8>>2]=1733480,c[g+16>>2]=331,g)|0);i=g;cw();return 0}c[f>>2]=1;h=a+4|0;j=a|0;k=a+124|0;l=a+156|0;m=-1;while(1){n=c[(c[h>>2]|0)+44>>2]|0;do{if((n|0)==0){o=c[c[(c[j>>2]|0)+16>>2]>>2]|0;if((o|0)==0){p=m;break}p=fS(o)|0}else{p=c1[n&255](a)|0}}while(0);if((p|0)==-541478725){n=c[k>>2]|0;if((n|0)==0){q=3777;break}c[k>>2]=0;r=fT(a,n)|0}else{r=p}if((r|0)<0){q=3780;break}n=c[f>>2]|0;if((n|0)==0){e=r;q=3790;break}if((c[l>>2]&1|0)==0){q=3785;break}if((n|0)==0){e=r;q=3788;break}else{m=r}}do{if((q|0)==3777){c[f>>2]=0}else if((q|0)==3785){AN(0,0,1744856,(g=i,i=i+24|0,c[g>>2]=1728568,c[g+8>>2]=1733480,c[g+16>>2]=349,g)|0);i=g;cw();return 0}else if((q|0)==3788){i=b;return e|0}else if((q|0)==3790){i=b;return e|0}else if((q|0)==3780){c[f>>2]=0;if((r|0)==-541478725){break}else{e=r}i=b;return e|0}}while(0);c[d>>2]=1;e=-541478725;i=b;return e|0}function fT(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0.0,z=0,A=0,B=0.0,C=0.0;d=i;i=i+16|0;e=d|0;f=d+8|0;c[f>>2]=b;g=a+8|0;j=c[g>>2]|0;k=c[a+12>>2]|0;l=c[j+60>>2]|0;if((c[a+144>>2]|0)!=0){At(f);m=-541478725;i=d;return m|0}n=c[k+36>>2]|0;o=(n|0)==0?12:n;do{if((c[k+56>>2]|0)==0){p=b}else{if((AA(b)|0)!=0){p=c[f>>2]|0;break}AN(c[g>>2]|0,48,1669984,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;n=a+16|0;s=c[n>>2]|0;if((s|0)==1){t=hI(a,c[(c[f>>2]|0)+76>>2]|0)|0}else if((s|0)==0){t=hw(a,c[a+20>>2]|0,c[a+24>>2]|0)|0}else{m=-22;i=d;return m|0}if((t|0)==0){At(f);m=-12;i=d;return m|0}s=c[f>>2]|0;Ax(t,s)|0;s=c[n>>2]|0;if((s|0)==1){n=c[t+64>>2]|0;u=c[f>>2]|0;v=c[u+64>>2]|0;w=c[u+76>>2]|0;x=u+352|0;u=zs(c[x>>2]|0,c[x+4>>2]|0)|0;x=c[(c[f>>2]|0)+80>>2]|0;Cb(n,v,0,0,w,u,x)|0}else if((s|0)==0){s=c[f>>2]|0;AG(t|0,t+32|0,s|0,s+32|0,c[s+80>>2]|0,c[s+68>>2]|0,c[s+72>>2]|0)}else{m=-22;i=d;return m|0}At(f);p=t}}while(0);t=p+136|0;L5217:do{if((l|0)!=0){f=a+56|0;b=a+60|0;k=l;do{y=+h[k>>3];if(y>(+((c[t>>2]|0)>>>0)+ +(c[t+4>>2]|0)*4294967296.0)*(+(c[f>>2]|0)/+(c[b>>2]|0))){break L5217}s=k+8|0;x=c[s>>2]|0;u=k+12|0;w=c[u>>2]|0;AN(c[g>>2]|0,48,1668240,(q=i,i=i+24|0,h[q>>3]=y,c[q+8>>2]=x,c[q+16>>2]=w,q)|0);i=q;w=c[g>>2]|0;x=c[s>>2]|0;s=c[u>>2]|0;u=c[k+16>>2]|0;do{if((a8(x|0,1724168)|0)==0){v=c[c[w+4>>2]>>2]|0;n=c[w+8>>2]|0;y0(0,0,1720544,(q=i,i=i+16|0,c[q>>2]=v,c[q+8>>2]=n,q)|0)|0;i=q}else{if((a8(x|0,1716560)|0)==0){fV(w,s)|0;break}n=c[(c[w+4>>2]|0)+48>>2]|0;if((n|0)==0){break}c0[n&15](w,x,s,0,0,u)|0}}while(0);u=(c[g>>2]|0)+60|0;s=c[u>>2]|0;A6(s+12|0);A6(s+8|0);c[u>>2]=c[s+20>>2];A4(s);k=c[(c[g>>2]|0)+60>>2]|0;}while((k|0)!=0)}}while(0);g=c[t>>2]|0;q=c[t+4>>2]|0;do{if((c[j+64>>2]|0)==0){z=o;A=a+160|0}else{t=Ak(p)|0;l=G;k=a+160|0;b=j+72|0;h[(c[b>>2]|0)+8>>3]=+((c[k>>2]|0)>>>0)+ +(c[k+4>>2]|0)*4294967296.0;if((g|0)==0&(q|0)==(-2147483648|0)){B=r}else{B=(+(g>>>0)+ +(q|0)*4294967296.0)*(+(c[a+56>>2]|0)/+(c[a+60>>2]|0))}h[c[b>>2]>>3]=B;if((t|0)==(-1|0)&(l|0)==(-1|0)){C=r}else{C=+(t>>>0)+ +(l|0)*4294967296.0}h[(c[b>>2]|0)+16>>3]=C;l=+zT(c[j+68>>2]|0,c[b>>2]|0,0)==0.0;c[j+76>>2]=l&1;if(!l){z=o;A=k;break}z=(c[(c[j+4>>2]|0)+20>>2]&65536|0)==0?o:12;A=k}}while(0);o=dg[z&255](a,p)|0;p=NU(c[A>>2]|0,c[A+4>>2]|0,1,0)|0;c[A>>2]=p;c[A+4>>2]=G;c[a+152>>2]=0;if((g|0)==0&(q|0)==(-2147483648|0)){m=o;i=d;return m|0}c[e>>2]=1;c[e+4>>2]=1e6;A=AY(g,q,a+56|0,e)|0;e=a+104|0;c[e>>2]=A;c[e+4>>2]=G;e=c[a+100>>2]|0;if((e|0)==0){m=o;i=d;return m|0}if((c[a+112>>2]|0)<=-1){m=o;i=d;return m|0}gm(e,a);m=o;i=d;return m|0}function fU(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0;h=i;if((a8(b|0,1724168)|0)==0){j=c[c[a+4>>2]>>2]|0;k=c[a+8>>2]|0;y0(e,f,1720544,(l=i,i=i+16|0,c[l>>2]=j,c[l+8>>2]=k,l)|0)|0;i=l;m=0;i=h;return m|0}if((a8(b|0,1716560)|0)==0){m=fV(a,d)|0;i=h;return m|0}l=c[(c[a+4>>2]|0)+48>>2]|0;if((l|0)==0){m=-38;i=h;return m|0}m=c0[l&15](a,b,d,e,f,g)|0;i=h;return m|0}function fV(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0;d=i;e=a+68|0;f=c[e>>2]|0;g=c[a+4>>2]|0;if((c[g+20>>2]&196608|0)==0){AN(a,16,1650176,(h=i,i=i+8|0,c[h>>2]=c[g>>2],h)|0);i=h;j=-1163346256;i=d;return j|0}g=A9(b)|0;if((g|0)==0){j=-12;i=d;return j|0}b=a+72|0;do{if((c[b>>2]|0)==0){k=A8(3,8)|0;c[b>>2]=k;if((k|0)!=0){break}A4(g);j=-12;i=d;return j|0}}while(0);b=a+44|0;k=zQ(e,g,38608,0,0,0,0,0,c[b>>2]|0)|0;if((k|0)<0){AN(c[b>>2]|0,16,1648472,(h=i,i=i+8|0,c[h>>2]=g,h)|0);i=h;A4(g);j=k;i=d;return j|0}else{zP(f);f=a+64|0;A4(c[f>>2]|0);c[f>>2]=g;j=0;i=d;return j|0}return 0}function fW(a){a=a|0;var b=0,d=0,e=0,f=0;if((a|0)==0){b=0;return b|0}else{d=0}while(1){e=c[((d|0)==0?2225208:d+44|0)>>2]|0;if((e|0)==0){b=0;f=3868;break}if((a8(c[e>>2]|0,a|0)|0)==0){b=e;f=3867;break}else{d=e}}if((f|0)==3867){return b|0}else if((f|0)==3868){return b|0}return 0}function fX(a){a=a|0;return c[((a|0)==0?2225208:a+44|0)>>2]|0}function fY(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=i;if((c[a+20>>2]&196608|0)==196608){AN(0,0,1744856,(d=i,i=i+24|0,c[d>>2]=1712912,c[d+8>>2]=1733480,c[d+16>>2]=461,d)|0);i=d;cw();return 0}e=c[a+8>>2]|0;L5297:do{if((e|0)!=0){f=0;while(1){if((c[e+(f*60|0)>>2]|0)==0){break L5297}if((c[e+(f*60|0)+36>>2]|0)!=0){if((c[e+(f*60|0)+16>>2]|0)!=0){g=3883;break}if((c[e+(f*60|0)+28>>2]|0)!=0){g=3884;break}}f=f+1|0}if((g|0)==3883){AN(0,0,1744856,(d=i,i=i+24|0,c[d>>2]=1709152,c[d+8>>2]=1733480,c[d+16>>2]=466,d)|0);i=d;cw();return 0}else if((g|0)==3884){AN(0,0,1744856,(d=i,i=i+24|0,c[d>>2]=1709152,c[d+8>>2]=1733480,c[d+16>>2]=466,d)|0);i=d;cw();return 0}}}while(0);c[a+44>>2]=0;d=a;if(((E=c[556302]|0,(c[556302]|0)==0?(c[556302]=d)|0:0,E)|0)==0){i=b;return 0}else{h=2225208}do{h=(c[h>>2]|0)+44|0;a=h;}while(((E=c[a>>2]|0,(c[a>>2]|0)==0?(c[a>>2]=d)|0:0,E)|0)!=0);i=b;return 0}function fZ(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;do{if((a|0)==0){b=0}else{if((c[a>>2]|0)==0){b=0;break}else{d=a;e=0}while(1){f=d+60|0;g=e+1|0;if((c[f>>2]|0)==0){b=g;break}else{d=f;e=g}}}}while(0);return b|0}function f_(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;if((a|0)==0){d=0;return d|0}e=A7(80)|0;f=e;if((e|0)==0){d=0;return d|0}c[e>>2]=339432;c[e+4>>2]=a;if((b|0)==0){g=0}else{g=A9(b)|0}c[e+8>>2]=g;g=c[a+40>>2]|0;do{if((g|0)==0){h=3898}else{b=A7(g)|0;c[e+44>>2]=b;if((b|0)!=0){h=3898;break}i=e+56|0}}while(0);L5333:do{if((h|0)==3898){Br(e);g=c[a+16>>2]|0;if((g|0)!=0){b=e+44|0;c[c[b>>2]>>2]=g;Br(c[b>>2]|0)}b=A7(4)|0;g=e+56|0;c[g>>2]=b;if((b|0)==0){i=g;break}c[b>>2]=2;b=a+8|0;j=c[b>>2]|0;do{if((j|0)==0){h=3904}else{if((c[j>>2]|0)==0){h=3904;break}else{k=j;l=0}do{k=k+60|0;l=l+1|0;}while((c[k>>2]|0)!=0);m=e+24|0;c[m>>2]=l;if((l|0)==0){n=m;break}o=A1(l*60|0)|0;c[e+12>>2]=o;if((o|0)==0){i=g;break L5333}p=c[b>>2]|0;q=c[m>>2]|0;r=q*60|0;NN(o|0,p|0,r)|0;r=A7(q<<2)|0;c[e+16>>2]=r;if((r|0)==0){i=g;break L5333}else{n=m}}}while(0);if((h|0)==3904){b=e+24|0;c[b>>2]=0;n=b}b=a+12|0;j=c[b>>2]|0;do{if((j|0)==0){h=3911}else{if((c[j>>2]|0)==0){h=3911;break}else{s=j;t=0}do{s=s+60|0;t=t+1|0;}while((c[s>>2]|0)!=0);m=e+40|0;c[m>>2]=t;if((t|0)==0){u=0;break}r=A1(t*60|0)|0;c[e+28>>2]=r;if((r|0)==0){i=g;break L5333}q=c[b>>2]|0;p=c[m>>2]|0;o=p*60|0;NN(r|0,q|0,o)|0;o=A7(p<<2)|0;c[e+32>>2]=o;if((o|0)==0){i=g;break L5333}u=c[m>>2]|0}}while(0);if((h|0)==3911){c[e+40>>2]=0;u=0}c[e+36>>2]=u;c[e+20>>2]=c[n>>2];d=f;return d|0}}while(0);A6(e+16|0);A6(e+12|0);c[e+24>>2]=0;A6(e+32|0);A6(e+28|0);c[e+40>>2]=0;A6(e+44|0);A6(i);A4(e);d=0;return d|0}function f$(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0;if((f|0)<=0){return 0}if((e|0)==0){g=0;do{dk[b&63](a,d,g,f)|0;g=g+1|0;}while((g|0)<(f|0));return 0}else{g=0;do{c[e+(g<<2)>>2]=dk[b&63](a,d,g,f)|0;g=g+1|0;}while((g|0)<(f|0));return 0}return 0}function f0(a){a=a|0;var b=0,d=0,e=0,f=0;if((a|0)==0){return}b=c[a+48>>2]|0;if((b|0)!=0){ge(b,a)}b=a+4|0;d=c[(c[b>>2]|0)+32>>2]|0;if((d|0)!=0){c_[d&127](a)}d=a+24|0;if((c[d>>2]|0)!=0){e=a+16|0;f=0;do{f1(c[(c[e>>2]|0)+(f<<2)>>2]|0);f=f+1|0;}while(f>>>0<(c[d>>2]|0)>>>0)}d=a+40|0;if((c[d>>2]|0)!=0){f=a+32|0;e=0;do{f1(c[(c[f>>2]|0)+(e<<2)>>2]|0);e=e+1|0;}while(e>>>0<(c[d>>2]|0)>>>0)}d=a+44|0;if((c[(c[b>>2]|0)+16>>2]|0)!=0){Bu(c[d>>2]|0)}A6(a+8|0);A6(a+12|0);A6(a+28|0);A6(a+16|0);A6(a+32|0);A6(d);d=a+60|0;b=c[d>>2]|0;if((b|0)!=0){e=b;do{A6(e+12|0);A6(e+8|0);c[d>>2]=c[e+20>>2];A4(e);e=c[d>>2]|0;}while((e|0)!=0)}e=a;Bu(e);d=a+68|0;zP(c[d>>2]|0);c[d>>2]=0;A6(a+72|0);A6(a+56|0);A4(e);return}function f1(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=i;i=i+8|0;d=b|0;c[d>>2]=a;if((a|0)==0){i=b;return}e=c[a>>2]|0;if((e|0)==0){f=a}else{c[(c[e+32>>2]|0)+((((c[a+4>>2]|0)-(c[e+28>>2]|0)|0)/60|0)<<2)>>2]=0;f=c[d>>2]|0}e=c[f+8>>2]|0;if((e|0)==0){g=f}else{c[(c[e+16>>2]|0)+((((c[f+12>>2]|0)-(c[e+12>>2]|0)|0)/60|0)<<2)>>2]=0;g=c[d>>2]|0}g2(g+64|0);g2(g+68|0);g2(g+72|0);g2(g+76|0);g3(g+80|0);g3(g+84|0);if((g|0)==0){i=b;return}At(g+124|0);A6(d);i=b;return}function f2(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;e=i;i=i+16|0;f=e|0;g=e+8|0;c[f>>2]=0;L5417:do{if((d|0)==0){h=3996}else{if((a[d]|0)==0){h=3996;break}j=b+4|0;k=c[j>>2]|0;if((c[k+16>>2]|0)==0){AN(b,16,1703008,(l=i,i=i+8|0,c[l>>2]=d,l)|0);i=l;m=-22;i=e;return m|0}n=c[k>>2]|0;do{if((a8(n|0,1700432)|0)!=0){if((a8(n|0,1697632)|0)==0){break}if((a8(n|0,1695600)|0)==0){break}if((a8(n|0,1692880)|0)==0){break}if((a8(n|0,1689656)|0)==0){break}if((a8(n|0,1686920)|0)==0){break}if((a8(n|0,1685264)|0)==0){break}if((a8(n|0,1683208)|0)==0){break}k=f4(b,f,d)|0;if((k|0)<0){o=k;break L5417}else{h=3996;break L5417}}}while(0);n=A9(d)|0;c[g>>2]=n;if((n|0)==0){o=-12;break}k=c[c[j>>2]>>2]|0;do{if((a8(k|0,1695600)|0)==0){p=n;q=1}else{if((a8(k|0,1689656)|0)==0){p=n;q=1;break}p=n;q=(a8(k|0,1692880)|0)==0?3:0}}while(0);while(1){if((q|0)==0){r=p;break}j=bi(p|0,58)|0;if((j|0)==0){h=3979;break}else{p=j+1|0;q=q-1|0}}if((h|0)==3979){r=n+(NL(n|0)|0)|0}j=bi(r|0,58)|0;s=(j|0)!=0|0;t=(j|0)==0;L5444:do{if((a8(k|0,1683208)|0)==0){if(t){h=3992;break}else{u=0;v=j}while(1){w=v+1|0;if((a[w]|0)==58){x=v;break}y=bi(w|0,61)|0;z=bi(w|0,58)|0;if((y|0)!=0){if((z|0)==0|y>>>0<z>>>0){h=3987;break}}a[v]=124;z=bi(w|0,58)|0;if((z|0)==0){h=3993;break L5444}else{u=1;v=z}}if((h|0)==3987){if((w|0)==0){A=u;h=3991;break}else{x=w}}if((a[x]|0)!=58){A=u;h=3991;break}NP(x|0,x+1|0,NL(x|0)|0);h=3993}else{if(t){A=s;h=3991;break}else{B=j}while(1){a[B]=124;z=bi(B+1|0,58)|0;if((z|0)==0){A=s;h=3991;break}else{B=z}}}}while(0);if((h|0)==3991){if((A|0)==0){h=3992}else{h=3993}}if((h|0)==3992){C=b}else if((h|0)==3993){s=b;AN(s,24,1681344,(l=i,i=i+1|0,i=i+7&-8,c[l>>2]=0,l)|0);i=l;C=s}s=c[g>>2]|0;AN(C,48,1679576,(l=i,i=i+8|0,c[l>>2]=s,l)|0);i=l;j=f4(b,f,s)|0;A6(g);if((j|0)<0){o=j}else{h=3996}}}while(0);do{if((h|0)==3996){g=f3(b,f)|0;if((g|0)<0){o=g;break}C=zJ(c[f>>2]|0,3119080,0,2)|0;if((C|0)==0){o=g;break}AN(b,16,1675600,(l=i,i=i+8|0,c[l>>2]=c[C>>2],l)|0);i=l;o=-1414549496}}while(0);zL(f);m=o;i=e;return m|0}function f3(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;d=i;e=a;f=Bv(e,b)|0;if((f|0)<0){AN(e,16,1707192,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g;h=f;i=d;return h|0}j=a+4|0;k=c[j>>2]|0;l=a+52|0;do{if((c[k+20>>2]&4|0)==0){m=4009}else{n=c[a+48>>2]|0;if((c[l>>2]&1&c[n+24>>2]|0)==0){m=4009;break}o=n+32|0;if((c[(c[o>>2]|0)+4>>2]|0)==0){m=4009;break}c[l>>2]=1;c[c[a+56>>2]>>2]=c[(c[o>>2]|0)+4>>2];p=c[j>>2]|0}}while(0);if((m|0)==4009){c[l>>2]=0;p=k}do{if((c[p+16>>2]|0)==0){q=f;r=p}else{k=Bv(c[a+44>>2]|0,b)|0;if((k|0)>=0){q=k;r=c[j>>2]|0;break}AN(e,16,1705080,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g;h=k;i=d;return h|0}}while(0);g=c[r+52>>2]|0;if((g|0)!=0){h=dg[g&255](a,0)|0;i=d;return h|0}g=c[r+24>>2]|0;if((g|0)!=0){h=c1[g&255](a)|0;i=d;return h|0}g=c[r+28>>2]|0;if((g|0)==0){h=q;i=d;return h|0}h=dg[g&255](a,b)|0;i=d;return h|0}function f4(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0;f=i;i=i+88|0;g=f|0;h=f+8|0;j=f+16|0;k=f+24|0;c[g>>2]=e;if((e|0)==0){l=0;i=f;return l|0}L5506:do{if((a[e]|0)==0){m=0}else{n=b+44|0;o=b;p=0;q=0;r=-1;s=e;L5508:while(1){t=p;while(1){u=Bb(c[n>>2]|0,t)|0;if((u|0)==0){v=0;w=r;x=0;break}if((c[u+12>>2]|0)!=128){y=c[u+8>>2]|0;if((y|0)!=(r|0)){z=4035;break}}if((a[s]|0)==0){m=q;break L5506}else{t=u}}if((z|0)==4035){z=0;v=c[u>>2]|0;w=y;x=u}A=Bt(g,1667008,1665816,(v|0)!=0|0,h,j)|0;if((A|0)<0){z=4037;break}t=c[g>>2]|0;if((a[t]|0)!=0){c[g>>2]=t+1}t=c[h>>2]|0;if((t|0)==0){B=v;C=x}else{D=x;while(1){E=Bb(c[n>>2]|0,D)|0;if((E|0)==0){B=t;C=0;break}else{D=E}}}D=c[j>>2]|0;AN(o,48,1661352,(F=i,i=i+16|0,c[F>>2]=B,c[F+8>>2]=D,F)|0);i=F;D=(Bc(o,B,0,0,0)|0)==0;t=c[j>>2]|0;do{if(D){zK(d,B,t,0)|0;G=Bd(c[n>>2]|0,B,c[j>>2]|0,0)|0;if((G|0)>=0){break}if((Bc(c[n>>2]|0,B,0,0,3)|0)==0){z=4049;break L5508}}else{H=Bd(o,B,t,0)|0;if((H|0)<0){z=4046;break L5508}}}while(0);A4(c[j>>2]|0);A4(c[h>>2]|0);t=q+1|0;D=c[g>>2]|0;if((a[D]|0)==0){m=t;break L5506}else{p=C;q=t;r=w;s=D}}if((z|0)==4046){A4(c[j>>2]|0);A4(c[h>>2]|0);l=H;i=f;return l|0}else if((z|0)==4049){if((G|0)==-1414549496){AN(o,16,1659984,(F=i,i=i+8|0,c[F>>2]=B,F)|0);i=F}A4(c[j>>2]|0);A4(c[h>>2]|0);l=G;i=f;return l|0}else if((z|0)==4037){s=c[g>>2]|0;if((A|0)==-22){AN(o,16,1664280,(F=i,i=i+8|0,c[F>>2]=s,F)|0);i=F;l=-22;i=f;return l|0}else{r=k|0;NM(r|0,0,64);a[r]=0;zN(A,r,64)|0;AN(o,16,1662848,(F=i,i=i+16|0,c[F>>2]=s,c[F+8>>2]=r,F)|0);i=F;l=A;i=f;return l|0}}}}while(0);A=c[b+64>>2]|0;do{if((A|0)!=0){F=fV(b,A)|0;if((F|0)<0){l=F}else{break}i=f;return l|0}}while(0);l=m;i=f;return l|0}function f5(a,b){a=a|0;b=b|0;return c[a+(b*60|0)>>2]|0}function f6(a,b){a=a|0;b=b|0;return c[a+(b*60|0)+4>>2]|0}function f7(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0;d=i;i=i+144|0;e=d|0;f=d+8|0;g=a+16|0;do{if((c[g>>2]|0)!=0){h=a+40|0;zt(d+16|0,128,-1,c[h>>2]|0,c[h+4>>2]|0);if((c[g>>2]|0)!=1){break}h=a+132|0;j=c[h>>2]|0;if((j|0)==0){break}k=a+124|0;l=c[k>>2]|0;m=c[b+76>>2]|0;if(!((l|0)!=0|(m|0)<(j|0))){if((m|0)<=(c[a+136>>2]|0)){break}}c[e>>2]=b;j=An(b)|0;n=a+156|0;c[n>>2]=c[n>>2]|1;n=f|0;o=f+4|0;p=a+48|0;q=a+128|0;r=a+56|0;s=a+52|0;t=0;u=l;l=m;m=0;L5562:while(1){v=u;w=l;x=m;while(1){if((w|0)==0){y=4081;break L5562}if((v|0)==0){c[n>>2]=1;c[o>>2]=c[p>>2];z=hI(a,c[q>>2]|0)|0;if((z|0)==0){y=4077;break L5562}A=c[e>>2]|0;Ax(z,A)|0;A=(c[e>>2]|0)+136|0;B=c[A>>2]|0;C=c[A+4>>2]|0;A=AY(x,(x|0)<0|0?-1:0,f,r)|0;D=NU(A,G,B,C)|0;C=z+136|0;c[C>>2]=D;c[C+4>>2]=G;c[z+76>>2]=0;E=z;F=0}else{E=v;F=c[v+76>>2]|0}z=E+76|0;C=(c[q>>2]|0)-F|0;D=(w|0)>(C|0)?C:w;Cb(c[E+64>>2]|0,c[(c[e>>2]|0)+64>>2]|0,F,x,D,j,c[s>>2]|0)|0;H=D+x|0;I=w-D|0;C=(c[z>>2]|0)+D|0;c[z>>2]=C;if((C|0)<(c[h>>2]|0)){v=E;w=I;x=H}else{break}}t=fT(a,E)|0;u=0;l=I;m=H}if((y|0)==4077){AN(c[a+8>>2]|0,24,1671904,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;J=0;i=d;return J|0}else if((y|0)==4081){At(e);c[k>>2]=v;J=t;i=d;return J|0}}}while(0);J=fT(a,b)|0;i=d;return J|0}function f8(){return 339432}function f9(a,b){a=a|0;b=b|0;return f7(c[c[(c[a+8>>2]|0)+32>>2]>>2]|0,b)|0}function ga(a){a=a|0;var b=0,d=0;b=c[a+8>>2]|0;if((b|0)!=0){d=b;return d|0}d=c[c[a+4>>2]>>2]|0;return d|0}function gb(a,b){a=a|0;b=b|0;var d=0,e=0;do{if((b|0)==0){d=c[a+4>>2]|0;if((d|0)==0){break}if((c[d+16>>2]|0)==0){break}d=c[a+44>>2]|0;if((d|0)==0){break}else{e=d}return e|0}}while(0);e=0;return e|0}function gc(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;L5596:do{if((a|0)==0){b=0}else{d=0;while(1){e=c[((d|0)==0?2225208:d+44|0)>>2]|0;if((e|0)==0){f=0;break}if((c[e+16>>2]|0)==(a|0)){b=e;break L5596}else{d=e}}return f|0}}while(0);while(1){a=c[((b|0)==0?2225208:b+44|0)>>2]|0;if((a|0)==0){f=0;g=4109;break}d=c[a+16>>2]|0;if((d|0)==0){b=a}else{f=d;g=4110;break}}if((g|0)==4110){return f|0}else if((g|0)==4109){return f|0}return 0}function gd(){var a=0,b=0,d=0,e=0,f=0;a=i;i=i+8|0;b=a|0;d=A7(52)|0;c[b>>2]=d;if((d|0)==0){e=0;i=a;return e|0}f=A7(8)|0;c[d+32>>2]=f;if((f|0)==0){A6(b);e=0;i=a;return e|0}else{c[d>>2]=235512;Br(d);e=c[b>>2]|0;i=a;return e|0}return 0}function ge(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=a+20|0;e=c[d>>2]|0;f=a+8|0;a=0;while(1){if(a>>>0>=e>>>0){g=4125;break}h=c[f>>2]|0;if((c[h+(a<<2)>>2]|0)==(b|0)){break}else{a=a+1|0}}if((g|0)==4125){return}g=h+(e-1<<2)|0;e=c[g>>2]|0;c[g>>2]=b;c[(c[f>>2]|0)+(a<<2)>>2]=e;c[d>>2]=(c[d>>2]|0)-1;return}function gf(a){a=a|0;var b=0,d=0,e=0;b=c[a>>2]|0;if((b|0)==0){return}if((c[b+20>>2]|0)==0){d=b}else{e=b;while(1){f0(c[c[e+8>>2]>>2]|0);b=c[a>>2]|0;if((c[b+20>>2]|0)==0){d=b;break}else{e=b}}}A6(d+40|0);A6((c[a>>2]|0)+12|0);A6((c[a>>2]|0)+36|0);A6((c[a>>2]|0)+16|0);A6((c[a>>2]|0)+8|0);A6((c[a>>2]|0)+32|0);A6(a);return}function gg(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0;f=g+24|0;do{if((c[f>>2]|0)!=0){if((c[c[g+32>>2]>>2]|0)!=0){break}c[f>>2]=0;c[g+28>>2]=1}}while(0);f=f_(b,d)|0;do{if((f|0)!=0){d=g+8|0;b=g+20|0;h=A2(c[d>>2]|0,(c[b>>2]<<2)+4|0)|0;if((h|0)==0){f0(f);break}i=h;c[d>>2]=i;d=c[b>>2]|0;c[b>>2]=d+1;c[i+(d<<2)>>2]=f;c[g+4>>2]=c[b>>2];c[f+48>>2]=g;c[a>>2]=f;b=f2(f,e)|0;if((b|0)>=0){j=0;return j|0}d=c[a>>2]|0;if((d|0)!=0){f0(d)}c[a>>2]=0;j=b;return j|0}}while(0);c[a>>2]=0;j=-12;return j|0}function gh(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=a+24|0;do{if((c[e>>2]|0)!=0){if((c[c[a+32>>2]>>2]|0)!=0){break}c[e>>2]=0;c[a+28>>2]=1}}while(0);e=f_(b,d)|0;if((e|0)==0){f=0;return f|0}d=a+8|0;b=a+20|0;g=A2(c[d>>2]|0,(c[b>>2]<<2)+4|0)|0;if((g|0)==0){f0(e);f=0;return f|0}else{h=g;c[d>>2]=h;d=c[b>>2]|0;c[b>>2]=d+1;c[h+(d<<2)>>2]=e;c[a+4>>2]=c[b>>2];c[e+48>>2]=a;f=e;return f|0}return 0}function gi(a,b){a=a|0;b=b|0;c[a+48>>2]=b;return}function gj(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0,cb=0,cc=0,cd=0,ce=0,cf=0,cg=0,ch=0,ci=0,cj=0,ck=0,cl=0,cm=0,cn=0,co=0,cp=0,cq=0,cr=0;e=i;i=i+1360|0;f=e|0;g=e+8|0;h=e+264|0;j=e+296|0;k=e+1320|0;l=e+1328|0;m=b+20|0;n=c[m>>2]|0;o=(n|0)==0;L5667:do{if(o){p=k;q=l|0;r=4175}else{s=c[b+8>>2]|0;t=0;L5669:while(1){u=c[s+(t<<2)>>2]|0;v=c[u+24>>2]|0;w=u+16|0;x=0;while(1){if(x>>>0>=v>>>0){break}y=c[(c[w>>2]|0)+(x<<2)>>2]|0;if((y|0)==0){r=4167;break L5669}if((c[y>>2]|0)==0){r=4167;break L5669}else{x=x+1|0}}w=c[u+40>>2]|0;v=u+32|0;z=0;while(1){if(z>>>0>=w>>>0){break}y=c[(c[v>>2]|0)+(z<<2)>>2]|0;if((y|0)==0){r=4172;break L5669}if((c[y+8>>2]|0)==0){r=4172;break L5669}else{z=z+1|0}}v=t+1|0;if(v>>>0<n>>>0){t=v}else{r=4174;break}}if((r|0)==4167){t=c[u+12>>2]|0;s=c[t+(x*60|0)>>2]|0;v=Cp(c[t+(x*60|0)+4>>2]|0)|0;t=c[u+8>>2]|0;w=c[c[u+4>>2]>>2]|0;AN(d,16,1679480,(A=i,i=i+32|0,c[A>>2]=s,c[A+8>>2]=v,c[A+16>>2]=t,c[A+24>>2]=w,A)|0);i=A;B=-22;i=e;return B|0}else if((r|0)==4174){w=k;t=l|0;if(o){p=w;q=t;r=4175;break}v=b+8|0;s=0;y=0;C=n;L5686:while(1){D=c[(c[v>>2]|0)+(y<<2)>>2]|0;E=D+24|0;F=c[E>>2]|0;if((F|0)==0){G=s;H=C}else{I=D+16|0;D=0;J=s;K=F;while(1){F=c[(c[I>>2]|0)+(D<<2)>>2]|0;if((c[(c[F+12>>2]|0)+52>>2]|0)==0){L=J;M=K}else{if((c[F+16>>2]|0)==0){N=fW(1685256)|0}else{N=fW(1683200)|0}bc(t|0,32,1681320,(A=i,i=i+8|0,c[A>>2]=J,A)|0)|0;i=A;O=gg(k,N,t,0,0,b)|0;if((O|0)<0){B=O;r=4475;break L5686}O=fP(F,c[k>>2]|0,0,0)|0;if((O|0)<0){B=O;r=4478;break L5686}L=J+1|0;M=c[E>>2]|0}O=D+1|0;if(O>>>0<M>>>0){D=O;J=L;K=M}else{break}}G=L;H=c[m>>2]|0}K=y+1|0;if(K>>>0<H>>>0){s=G;y=K;C=H}else{P=H;Q=v;R=w;S=t;break L5667}}if((r|0)==4475){i=e;return B|0}else if((r|0)==4478){i=e;return B|0}}else if((r|0)==4172){t=c[u+28>>2]|0;w=c[t+(z*60|0)>>2]|0;v=Cp(c[t+(z*60|0)+4>>2]|0)|0;t=c[u+8>>2]|0;C=c[c[u+4>>2]>>2]|0;AN(d,16,1677488,(A=i,i=i+32|0,c[A>>2]=w,c[A+8>>2]=v,c[A+16>>2]=t,c[A+24>>2]=C,A)|0);i=A;B=-22;i=e;return B|0}}}while(0);if((r|0)==4175){P=0;Q=b+8|0;R=p;S=q}q=g|0;g=h|0;h=b;S=b+12|0;p=b+36|0;R=j|0;u=j+4|0;z=P;while(1){L5714:do{if((z|0)==0){T=0;r=4261}else{P=0;H=0;G=z;while(1){L=c[(c[Q>>2]|0)+(H<<2)>>2]|0;M=c[L+24>>2]|0;L5717:do{if((M|0)==0){r=4194}else{k=c[L+16>>2]|0;N=0;while(1){n=c[k+(N<<2)>>2]|0;if((c[n+68>>2]|0)==0){r=4206;break L5717}if((c[n+16>>2]|0)==1){if((c[n+76>>2]|0)==0){r=4206;break L5717}if((c[n+84>>2]|0)==0){r=4206;break L5717}}n=N+1|0;if(n>>>0<M>>>0){N=n}else{r=4194;break}}}}while(0);L5726:do{if((r|0)==4194){r=0;M=c[L+40>>2]|0;if((M|0)==0){U=P;V=G;break}N=c[L+32>>2]|0;k=0;while(1){n=c[N+(k<<2)>>2]|0;if((c[n+64>>2]|0)==0){r=4206;break L5726}if((c[n+16>>2]|0)==1){if((c[n+72>>2]|0)==0){r=4206;break L5726}if((c[n+80>>2]|0)==0){r=4206;break L5726}}n=k+1|0;if(n>>>0<M>>>0){k=n}else{U=P;V=G;break}}}}while(0);if((r|0)==4206){r=0;if((c[(c[L+4>>2]|0)+36>>2]|0)==0){W=g9(L)|0}else{W=gp(L)|0}if((W|0)<0&(W|0)!=-11){X=W;break L5714}U=(W>>>31^1)+P|0;V=c[m>>2]|0}k=H+1|0;if(k>>>0<V>>>0){P=U;H=k;G=V}else{break}}if((V|0)==0){T=U;r=4261;break}else{Y=0;Z=0;_=0;$=0;aa=0;ab=0;ac=V}L5745:while(1){G=c[(c[Q>>2]|0)+(ab<<2)>>2]|0;H=G+24|0;if((c[H>>2]|0)==0){ad=Y;ae=Z;af=_;ag=$;ah=aa;ai=ac}else{P=G+16|0;G=Y;k=Z;M=_;N=$;n=aa;o=0;while(1){aj=c[(c[P>>2]|0)+(o<<2)>>2]|0;do{if((aj|0)==0){ak=n;al=N;am=M;an=k;ao=G}else{l=c[aj+64>>2]|0;do{if((l|0)==0){r=4217}else{x=c[aj+68>>2]|0;if((x|0)==0){r=4217;break}if((l|0)==(x|0)){ap=0;aq=N;ar=M+1|0;as=k;break}else{ap=(gR(l,x,c[aj+16>>2]|0)|0)==0|0;aq=N;ar=M;as=k+1|0;break}}}while(0);if((r|0)==4217){r=0;ap=0;aq=N+1|0;ar=M;as=k}l=aj+16|0;L5762:do{if((c[l>>2]|0)==1){x=c[aj+80>>2]|0;do{if((x|0)==0){r=4224}else{C=c[aj+84>>2]|0;if((C|0)==0){r=4224;break}if((x|0)==(C|0)){at=ap;au=aq;av=ar+1|0;aw=as;break}else{at=(gT(x,C)|0)==0?1:ap;au=aq;av=ar;aw=as+1|0;break}}}while(0);if((r|0)==4224){r=0;at=ap;au=aq+1|0;av=ar;aw=as}x=c[aj+72>>2]|0;do{if((x|0)!=0){C=c[aj+76>>2]|0;if((C|0)==0){break}if((x|0)==(C|0)){ax=at;ay=au;az=av+1|0;aA=aw;break L5762}else{ax=(gS(x,C)|0)==0?1:at;ay=au;az=av;aA=aw+1|0;break L5762}}}while(0);ax=at;ay=au+1|0;az=av;aA=aw}else{ax=ap;ay=aq;az=ar;aA=as}}while(0);if((ax|0)==0){ak=n;al=ay;am=az;an=aA;ao=G;break}x=c[l>>2]|0;if((x|0)==1){C=fW(1720528)|0;if((C|0)==0){r=4240;break L5745}t=G+1|0;bc(g|0,30,1712880,(A=i,i=i+8|0,c[A>>2]=G,A)|0)|0;i=A;a[q]=0;v=c[p>>2]|0;if((v|0)==0){aB=0}else{bc(q|0,256,1709144,(A=i,i=i+8|0,c[A>>2]=v,A)|0)|0;i=A;aB=c[p>>2]|0}v=gg(f,C,g,aB,0,b)|0;if((v|0)<0){X=v;break L5714}else{aC=n;aD=t}}else if((x|0)==0){x=fW(1733472)|0;if((x|0)==0){r=4237;break L5745}bc(g|0,30,1724144,(A=i,i=i+8|0,c[A>>2]=n,A)|0)|0;i=A;t=gg(f,x,g,c[S>>2]|0,0,b)|0;if((t|0)<0){X=t;break L5714}else{aC=n+1|0;aD=G}}else{X=-22;break L5714}t=c[f>>2]|0;x=fP(aj,t,0,0)|0;if((x|0)<0){X=x;break L5714}gp(t)|0;v=c[c[t+16>>2]>>2]|0;C=c[c[t+32>>2]>>2]|0;t=v+16|0;if((gR(c[v+64>>2]|0,c[v+68>>2]|0,c[t>>2]|0)|0)==0){r=4247}else{if((gR(c[C+64>>2]|0,c[C+68>>2]|0,c[C+16>>2]|0)|0)==0){r=4247}else{aE=x}}if((r|0)==4247){r=0;aE=x|-38}do{if((c[t>>2]|0)==1){if((gS(c[v+72>>2]|0,c[v+76>>2]|0)|0)!=0){if((gT(c[v+80>>2]|0,c[v+84>>2]|0)|0)!=0){aF=aE;break}}aF=aE|-38}else{aF=aE}}while(0);if((c[C+16>>2]|0)==1){if((gS(c[C+72>>2]|0,c[C+76>>2]|0)|0)==0){r=4255;break L5745}if((gT(c[C+80>>2]|0,c[C+84>>2]|0)|0)==0){r=4255;break L5745}}if((aF|0)<0){aG=aF;break L5745}else{ak=aC;al=ay;am=az;an=aA;ao=aD}}}while(0);v=o+1|0;if(v>>>0<(c[H>>2]|0)>>>0){G=ao;k=an;M=am;N=al;n=ak;o=v}else{break}}ad=ao;ae=an;af=am;ag=al;ah=ak;ai=c[m>>2]|0}o=ab+1|0;if(o>>>0<ai>>>0){Y=ad;Z=ae;_=af;$=ag;aa=ah;ab=o;ac=ai}else{r=4262;break}}if((r|0)==4255){r=0;aG=aF|-38}else if((r|0)==4240){r=0;AN(d,16,1716496,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;X=-22;break}else if((r|0)==4262){r=0;AN(h,48,1705008,(A=i,i=i+32|0,c[A>>2]=U,c[A+8>>2]=ae,c[A+16>>2]=af,c[A+24>>2]=ag,A)|0);i=A;if((ag|0)==0){X=0;break}if((ae|U|0)!=0){X=-11;break}y9(j,0,1);o=c[m>>2]|0;if((o|0)!=0){n=0;N=o;while(1){o=c[(c[Q>>2]|0)+(n<<2)>>2]|0;M=c[o+24>>2]|0;L5819:do{if((M|0)==0){r=4267}else{k=c[o+16>>2]|0;G=0;while(1){H=c[k+(G<<2)>>2]|0;if((c[H+68>>2]|0)==0){r=4279;break L5819}if((c[H+16>>2]|0)==1){if((c[H+76>>2]|0)==0){r=4279;break L5819}if((c[H+84>>2]|0)==0){r=4279;break L5819}}H=G+1|0;if(H>>>0<M>>>0){G=H}else{r=4267;break}}}}while(0);L5828:do{if((r|0)==4267){r=0;M=c[o+40>>2]|0;if((M|0)==0){aH=N;break}G=c[o+32>>2]|0;k=0;while(1){H=c[G+(k<<2)>>2]|0;if((c[H+64>>2]|0)==0){r=4279;break L5828}if((c[H+16>>2]|0)==1){if((c[H+72>>2]|0)==0){r=4279;break L5828}if((c[H+80>>2]|0)==0){r=4279;break L5828}}H=k+1|0;if(H>>>0<M>>>0){k=H}else{aH=N;break}}}}while(0);if((r|0)==4279){r=0;k=c[o+8>>2]|0;zb(j,1703e3,(A=i,i=i+16|0,c[A>>2]=(c[u>>2]|0)!=0?1700424:3119120,c[A+8>>2]=k,A)|0);i=A;aH=c[m>>2]|0}k=n+1|0;if(k>>>0<aH>>>0){n=k;N=aH}else{break}}}AN(h,16,1695472,(A=i,i=i+8|0,c[A>>2]=c[R>>2],A)|0);i=A;X=-5;break}else if((r|0)==4237){r=0;AN(d,16,1728464,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;X=-22;break}N=c[(c[aj+8>>2]|0)+8>>2]|0;AN(d,16,1707096,(A=i,i=i+16|0,c[A>>2]=c[(c[aj>>2]|0)+8>>2],c[A+8>>2]=N,A)|0);i=A;X=aG}}while(0);if((r|0)==4261){r=0;AN(h,48,1705008,(A=i,i=i+32|0,c[A>>2]=T,c[A+8>>2]=0,c[A+16>>2]=0,c[A+24>>2]=0,A)|0);i=A;X=0}if((X|0)!=-11){break}AN(h,48,1626384,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;z=c[m>>2]|0}if((X|0)<0){B=X;i=e;return B|0}else{aI=0;aJ=0}while(1){aK=c[m>>2]|0;if(aJ>>>0>=aK>>>0){if((aI|0)==0){break}else{aI=0;aJ=0;continue}}X=c[(c[Q>>2]|0)+(aJ<<2)>>2]|0;z=X+24|0;do{if((c[z>>2]|0)==0){aL=0}else{h=X+16|0;T=X+40|0;aG=X+32|0;aj=0;d=0;while(1){R=c[(c[h>>2]|0)+(d<<2)>>2]|0;aH=c[R+68>>2]|0;L5860:do{if((aH|0)==0){aM=aj}else{if((c[aH>>2]|0)!=1){aM=aj;break}u=c[c[aH+4>>2]>>2]|0;j=c[T>>2]|0;if((j|0)==0){aM=aj;break}U=R+16|0;ae=aj;ag=0;af=j;L5864:while(1){j=c[(c[aG>>2]|0)+(ag<<2)>>2]|0;L5866:do{if((c[U>>2]|0)==(c[j+16>>2]|0)){aN=j+64|0;aF=c[aN>>2]|0;ai=aF|0;ac=c[ai>>2]|0;if((ac|0)==0){break L5864}else if((ac|0)==1){aO=ae;aP=af;break}ab=aF+4|0;aF=0;while(1){if(aF>>>0>=ac>>>0){aO=ae;aP=af;break L5866}aQ=c[ab>>2]|0;if((c[aQ+(aF<<2)>>2]|0)==(u|0)){break}else{aF=aF+1|0}}c[aQ>>2]=u;c[ai>>2]=1;aO=1;aP=c[T>>2]|0}else{aO=ae;aP=af}}while(0);j=ag+1|0;if(j>>>0<aP>>>0){ae=aO;ag=j;af=aP}else{aM=aO;break L5860}}gW(aN,u,(u|0)<0|0?-1:0)|0;aM=ae}}while(0);R=d+1|0;aR=c[z>>2]|0;if(R>>>0<aR>>>0){aj=aM;d=R}else{break}}if((aR|0)==0){aL=aM;break}else{aS=aM;aT=0}while(1){d=c[(c[h>>2]|0)+(aT<<2)>>2]|0;aj=c[d+76>>2]|0;L5879:do{if((aj|0)==0){aU=aS}else{if((c[aj>>2]|0)!=1){aU=aS;break}R=c[c[aj+4>>2]>>2]|0;aH=c[T>>2]|0;if((aH|0)==0){aU=aS;break}o=d+16|0;af=aS;ag=0;U=aH;L5883:while(1){aH=c[(c[aG>>2]|0)+(ag<<2)>>2]|0;L5885:do{if((c[o>>2]|0)==(c[aH+16>>2]|0)){aV=aH+72|0;j=c[aV>>2]|0;C=j|0;aF=c[C>>2]|0;if((aF|0)==1){aW=af;aX=U;break}else if((aF|0)==0){break L5883}ab=j+4|0;j=0;while(1){if(j>>>0>=aF>>>0){aW=af;aX=U;break L5885}aY=c[ab>>2]|0;if((c[aY+(j<<2)>>2]|0)==(R|0)){break}else{j=j+1|0}}c[aY>>2]=R;c[C>>2]=1;aW=1;aX=c[T>>2]|0}else{aW=af;aX=U}}while(0);aH=ag+1|0;if(aH>>>0<aX>>>0){af=aW;ag=aH;U=aX}else{aU=aW;break L5879}}gW(aV,R,(R|0)<0|0?-1:0)|0;aU=af}}while(0);d=aT+1|0;aZ=c[z>>2]|0;if(d>>>0<aZ>>>0){aS=aU;aT=d}else{break}}if((aZ|0)==0){aL=aU;break}else{a_=aU;a$=0}while(1){d=c[(c[h>>2]|0)+(a$<<2)>>2]|0;aj=c[d+84>>2]|0;L5898:do{if((aj|0)==0){a0=a_}else{if((c[aj+4>>2]|0)!=1){a0=a_;break}U=c[aj>>2]|0;ag=c[U>>2]|0;o=c[U+4>>2]|0;U=c[T>>2]|0;if((U|0)==0){a0=a_;break}ae=d+16|0;u=a_;aH=0;j=U;L5902:while(1){U=c[(c[aG>>2]|0)+(aH<<2)>>2]|0;a1=U+80|0;a2=c[a1>>2]|0;L5904:do{if((c[ae>>2]|0)==(c[U+16>>2]|0)){ab=a2+4|0;aF=c[ab>>2]|0;if((aF|0)==1){a3=u;a4=j;break}a5=a2+8|0;if((a[a5]|0)!=0){break L5902}ai=a2|0;ac=0;while(1){if((ac|0)>=(aF|0)){a3=u;a4=j;break L5904}a6=c[ai>>2]|0;ah=a6+(ac<<3)|0;if((c[ah>>2]|0)==(ag|0)&(c[ah+4>>2]|0)==(o|0)){break}else{ac=ac+1|0}}c[a6>>2]=ag;c[a6+4>>2]=o;c[ab>>2]=1;a3=1;a4=c[T>>2]|0}else{a3=u;a4=j}}while(0);U=aH+1|0;if(U>>>0<a4>>>0){u=a3;aH=U;j=a4}else{a0=a3;break L5898}}a[a2+9|0]=0;a[a5]=0;gX(a1,ag,o)|0;a0=u}}while(0);d=a$+1|0;if(d>>>0<(c[z>>2]|0)>>>0){a_=a0;a$=d}else{aL=a0;break}}}}while(0);aI=aL|aI;aJ=aJ+1|0}L5917:do{if((aK|0)==0){a7=0;a8=0;a9=0}else{aJ=0;L5918:while(1){aI=c[(c[Q>>2]|0)+(aJ<<2)>>2]|0;aL=c[aI+24>>2]|0;do{if((aL|0)!=0){a0=c[aI+16>>2]|0;a$=0;while(1){ba=c[a0+(a$<<2)>>2]|0;if((c[ba+16>>2]|0)==1){if((c[c[ba+68>>2]>>2]|0)==1){bb=a$;break}}a_=a$+1|0;if(a_>>>0<aL>>>0){a$=a_}else{bb=a_;break}}if((bb|0)==(aL|0)){break}a$=c[c[(c[ba+68>>2]|0)+4>>2]>>2]|0;a0=B7(a$)|0;a_=aI+40|0;a1=c[a_>>2]|0;if((a1|0)==0){break}a5=aI+32|0;if((a0|0)==4){a2=0;a3=a1;while(1){a4=c[(c[a5>>2]|0)+(a2<<2)>>2]|0;do{if((c[a4+16>>2]|0)==1){a6=a4+64|0;aU=c[a6>>2]|0;if((c[aU>>2]|0)>>>0<2){bd=a3;break}else{be=-1;bf=-2147483648;bg=0;bh=aU}while(1){aU=c[(c[bh+4>>2]|0)+(bg<<2)>>2]|0;aZ=B7(aU)|0;if((B4(aU)|0)==(a$|0)){bi=bg;break}if((B5(aU)|0)==(a$|0)|(aZ|0)==8){bi=bg;break}aU=aZ-4|0;aT=((aZ|0)<4?0:1073741823)-((aU|0)>-1?aU:4-aZ|0)|0;aZ=(aT|0)>(bf|0);aU=aZ?bg:be;aS=bg+1|0;aV=c[a6>>2]|0;if(aS>>>0<(c[aV>>2]|0)>>>0){be=aU;bf=aZ?aT:bf;bg=aS;bh=aV}else{bi=aU;break}}if((bi|0)<=-1){r=4485;break L5918}aU=c[(c[a6>>2]|0)+4>>2]|0;aV=aU+(bi<<2)|0;aS=c[aV>>2]|0;c[aV>>2]=c[aU>>2];c[c[(c[a6>>2]|0)+4>>2]>>2]=aS;bd=c[a_>>2]|0}else{bd=a3}}while(0);a4=a2+1|0;if(a4>>>0<bd>>>0){a2=a4;a3=bd}else{break}}}else{a3=0;a2=a1;while(1){a4=c[(c[a5>>2]|0)+(a3<<2)>>2]|0;do{if((c[a4+16>>2]|0)==1){u=a4+64|0;o=c[u>>2]|0;if((c[o>>2]|0)>>>0<2){bj=a2;break}else{bk=-1;bl=-2147483648;bm=0;bn=o}while(1){o=c[(c[bn+4>>2]|0)+(bm<<2)>>2]|0;ag=B7(o)|0;if((B4(o)|0)==(a$|0)){bo=bm;break}if((B5(o)|0)==(a$|0)){bo=bm;break}o=ag-a0|0;aS=((ag|0)<(a0|0)?0:1073741823)-((o|0)>-1?o:-o|0)|0;o=(aS|0)>(bl|0);ag=o?bm:bk;aU=bm+1|0;aV=c[u>>2]|0;if(aU>>>0<(c[aV>>2]|0)>>>0){bk=ag;bl=o?aS:bl;bm=aU;bn=aV}else{bo=ag;break}}if((bo|0)<=-1){r=4486;break L5918}a6=c[(c[u>>2]|0)+4>>2]|0;ag=a6+(bo<<2)|0;aV=c[ag>>2]|0;c[ag>>2]=c[a6>>2];c[c[(c[u>>2]|0)+4>>2]>>2]=aV;bj=c[a_>>2]|0}else{bj=a2}}while(0);a4=a3+1|0;if(a4>>>0<bj>>>0){a3=a4;a2=bj}else{break}}}}}while(0);aI=aJ+1|0;bp=c[m>>2]|0;if(aI>>>0<bp>>>0){aJ=aI}else{r=4359;break}}if((r|0)==4359){if((bp|0)==0){a7=0;a8=0;a9=0;break}else{bq=0;br=bp;bs=bp}while(1){aJ=c[(c[Q>>2]|0)+(bq<<2)>>2]|0;aI=c[aJ+24>>2]|0;do{if((aI|0)==0){bt=br;bu=bs}else{aL=c[aJ+16>>2]|0;a2=0;while(1){bv=c[aL+(a2<<2)>>2]|0;if((c[bv+16>>2]|0)==1){if((c[c[bv+76>>2]>>2]|0)==1){bw=a2;break}}a3=a2+1|0;if(a3>>>0<aI>>>0){a2=a3}else{bw=a3;break}}if((bw|0)==(aI|0)){bt=br;bu=bs;break}a2=c[c[(c[bv+76>>2]|0)+4>>2]>>2]|0;aL=aJ+40|0;a3=c[aL>>2]|0;if((a3|0)==0){bt=br;bu=bs;break}a_=aJ+32|0;a0=0;a$=0;a5=a3;while(1){a3=c[(c[a_>>2]|0)+(a$<<2)>>2]|0;do{if((c[a3+16>>2]|0)==1){a1=a3+72|0;a4=c[a1>>2]|0;aV=c[a4>>2]|0;if(aV>>>0<2){bx=a0;by=a5;break}a6=c[a4+4>>2]|0;a4=0;ag=a0;aU=2147483647;while(1){aS=a2-(c[a6+(a4<<2)>>2]|0)|0;o=(aS|0)>-1?aS:-aS|0;aS=(o|0)<(aU|0);bz=aS?a4:ag;aT=a4+1|0;if(aT>>>0<aV>>>0){a4=aT;ag=bz;aU=aS?o:aU}else{break}}aU=a6+(bz<<2)|0;ag=c[aU>>2]|0;c[aU>>2]=c[a6>>2];c[c[(c[a1>>2]|0)+4>>2]>>2]=ag;bx=bz;by=c[aL>>2]|0}else{bx=a0;by=a5}}while(0);a3=a$+1|0;if(a3>>>0<by>>>0){a0=bx;a$=a3;a5=by}else{break}}a5=c[m>>2]|0;bt=a5;bu=a5}}while(0);aJ=bq+1|0;if(aJ>>>0<bt>>>0){bq=aJ;br=bt;bs=bu}else{break}}if((bu|0)==0){a7=0;a8=0;a9=0;break}else{bA=0;bB=bu}L5980:while(1){aJ=c[(c[Q>>2]|0)+(bA<<2)>>2]|0;aI=c[aJ+24>>2]|0;do{if((aI|0)==0){bC=bB}else{a5=c[aJ+16>>2]|0;a$=0;while(1){bD=c[a5+(a$<<2)>>2]|0;if((c[bD+16>>2]|0)==1){if((c[(c[bD+84>>2]|0)+4>>2]|0)==1){bE=a$;break}}a0=a$+1|0;if(a0>>>0<aI>>>0){a$=a0}else{bE=a0;break}}if((bE|0)==(aI|0)){bC=bB;break}a$=aJ+40|0;a5=c[a$>>2]|0;if((a5|0)==0){bC=bB;break}a0=aJ+32|0;aL=bD+84|0;a2=0;a_=a5;while(1){a5=c[(c[a0>>2]|0)+(a2<<2)>>2]|0;do{if((c[a5+16>>2]|0)==1){a3=a5+80|0;ag=c[a3>>2]|0;if((c[ag+4>>2]|0)<2){bF=a_;break}else{bG=2147483647;bH=-2147483648;bI=-1;bJ=0;bK=ag}while(1){ag=c[c[aL>>2]>>2]|0;aU=c[ag>>2]|0;a4=c[ag+4>>2]|0;ag=(c[bK>>2]|0)+(bJ<<3)|0;aV=c[ag>>2]|0;u=c[ag+4>>2]|0;ag=zs(aU,a4)|0;o=zs(aV,u)|0;aS=o-ag|0;aT=0;aZ=(a4|0)<(aT|0)|(a4|0)==(aT|0)&aU>>>0<0>>>0;if((aU&2147483647|0)==0&(a4&0|0)==0|aZ^1){aT=-1;if((u|0)>(aT|0)|(u|0)==(aT|0)&aV>>>0>-1>>>0|(aV&2147483647|0)==0&(u&0|0)==0){bL=a4;bM=aU;bN=u;bO=aV;bP=0;bQ=1e5}else{r=4389}}else{r=4389}if((r|0)==4389){r=0;if(aZ){aZ=aU&2147483647;bR=(aZ|0)==0?ag:aZ}else{bR=ag}ag=0;if((u|0)<(ag|0)|(u|0)==(ag|0)&aV>>>0<0>>>0){ag=aV&2147483647;bS=(ag|0)==0?o:ag}else{bS=o}o=bS-bR|0;bL=0;bM=0;bN=0;bO=0;bP=0;bQ=((bR|0)>(bS|0)?8e4:9e4)-((o|0)>-1?o:-o|0)|0}while(1){o=304576+(bP<<4)|0;ag=c[o>>2]|0;aV=c[o+4>>2]|0;o=304584+(bP<<4)|0;u=c[o>>2]|0;aZ=c[o+4>>2]|0;do{if((ag&bM|0)==0&(aV&bL|0)==0){bT=bQ;bU=bN;bV=bO;bW=bL;bX=bM}else{if(!((ag&bO|0)==0&(aV&bN|0)==0)){bT=bQ;bU=bN;bV=bO;bW=bL;bX=bM;break}if((u&bO|0)==0&(aZ&bN|0)==0){bT=bQ;bU=bN;bV=bO;bW=bL;bX=bM;break}if(!((u&bM|0)==0&(aZ&bL|0)==0)){bT=bQ;bU=bN;bV=bO;bW=bL;bX=bM;break}bT=bQ-2+((zs(u,aZ)|0)*10|0)|0;bU=bN&~aZ;bV=bO&~u;bW=bL&~aV;bX=bM&~ag}}while(0);ag=bP+1|0;if(ag>>>0<21){bL=bW;bM=bX;bN=bU;bO=bV;bP=ag;bQ=bT}else{break}}if((bX&8|0)==0&(bW&0|0)==0){bY=bT}else{bY=(bV&8|0)==0&(bU&0|0)==0?bT:bT+10|0}ab=bV&-9;ag=bU|0;aV=zs(ab&bX,ag&bW)|0;u=(aV*10|0)+bY+((zs(ab&((bX|8)^-9),ag&~bW)|0)*-5|0)|0;if((u|0)>(bH|0)){r=4404}else{if((aS|0)<(bG|0)&(u|0)==(bH|0)){r=4404}else{bZ=bI;b_=bH;b$=bG}}if((r|0)==4404){r=0;bZ=bJ;b_=u;b$=aS}u=bJ+1|0;b0=c[a3>>2]|0;if((u|0)<(c[b0+4>>2]|0)){bG=b$;bH=b_;bI=bZ;bJ=u;bK=b0}else{break}}if((bZ|0)<=-1){break L5980}a1=c[b0>>2]|0;a6=a1+(bZ<<3)|0;u=c[a6>>2]|0;ag=c[a6+4>>2]|0;ab=c[a1+4>>2]|0;c[a6>>2]=c[a1>>2];c[a6+4>>2]=ab;ab=c[c[a3>>2]>>2]|0;c[ab>>2]=u;c[ab+4>>2]=ag;bF=c[a$>>2]|0}else{bF=a_}}while(0);a5=a2+1|0;if(a5>>>0<bF>>>0){a2=a5;a_=bF}else{break}}bC=c[m>>2]|0}}while(0);aJ=bA+1|0;if(aJ>>>0<bC>>>0){bA=aJ;bB=bC}else{a7=0;a8=0;a9=bC;break L5917}}AN(0,0,1662792,(A=i,i=i+24|0,c[A>>2]=1738096,c[A+8>>2]=1632928,c[A+16>>2]=856,A)|0);i=A;cw();return 0}else if((r|0)==4485){AN(0,0,1662792,(A=i,i=i+24|0,c[A>>2]=1738096,c[A+8>>2]=1632928,c[A+16>>2]=925,A)|0);i=A;cw();return 0}else if((r|0)==4486){AN(0,0,1662792,(A=i,i=i+24|0,c[A>>2]=1738096,c[A+8>>2]=1632928,c[A+16>>2]=925,A)|0);i=A;cw();return 0}}}while(0);L6033:while(1){bC=a7;bB=a8;while(1){if(bC>>>0<a9>>>0){break}if((bB|0)==0){r=4436;break L6033}else{bC=0;bB=0}}bA=c[(c[Q>>2]|0)+(bC<<2)>>2]|0;bF=bA+24|0;bZ=c[bF>>2]|0;if((bZ|0)==0){b1=bB}else{b0=bA+16|0;bK=0;bJ=bB;bI=bZ;while(1){bZ=c[(c[b0>>2]|0)+(bK<<2)>>2]|0;b_=c[bZ+64>>2]|0;do{if((b_|0)==0){b2=bJ;b3=bI}else{if((c[b_>>2]|0)!=1){b2=bJ;b3=bI;break}bH=go(bZ,0)|0;if((bH|0)<0){b4=bH;r=4444;break L6033}b2=1;b3=c[bF>>2]|0}}while(0);bZ=bK+1|0;if(bZ>>>0<b3>>>0){bK=bZ;bJ=b2;bI=b3}else{b1=b2;break}}}bI=bA+40|0;bJ=c[bI>>2]|0;do{if((bJ|0)==0){b5=b1}else{bK=bA+32|0;b0=0;bB=b1;bZ=bJ;while(1){b_=c[(c[bK>>2]|0)+(b0<<2)>>2]|0;bH=c[b_+64>>2]|0;do{if((bH|0)==0){b6=bB;b7=bZ}else{if((c[bH>>2]|0)!=1){b6=bB;b7=bZ;break}b$=go(b_,0)|0;if((b$|0)<0){b4=b$;r=4444;break L6033}b6=1;b7=c[bI>>2]|0}}while(0);b_=b0+1|0;if(b_>>>0<b7>>>0){b0=b_;bB=b6;bZ=b7}else{break}}if((c[bF>>2]|0)==0|(b7|0)==0){b5=b6;break}bZ=bA+16|0;if((c[(c[c[bZ>>2]>>2]|0)+52>>2]|0)>-1){b8=0;b9=b6;ca=b7}else{b5=b6;break}while(1){bB=c[(c[bK>>2]|0)+(b8<<2)>>2]|0;if((c[bB+52>>2]|0)<0){b0=go(bB,c[c[bZ>>2]>>2]|0)|0;if((b0|0)<0){b4=b0;r=4444;break L6033}cb=1;cc=c[bI>>2]|0}else{cb=b9;cc=ca}b0=b8+1|0;if(b0>>>0<cc>>>0){b8=b0;b9=cb;ca=cc}else{b5=cb;break}}}}while(0);a7=bC+1|0;a8=b5;a9=c[m>>2]|0}L6067:do{if((r|0)==4436){if((a9|0)==0){cd=a9;break}else{ce=0}while(1){b5=c[(c[Q>>2]|0)+(ce<<2)>>2]|0;a8=b5+24|0;a7=b5+16|0;cb=0;while(1){if(cb>>>0>=(c[a8>>2]|0)>>>0){break}cc=go(c[(c[a7>>2]|0)+(cb<<2)>>2]|0,0)|0;if((cc|0)<0){b4=cc;r=4444;break L6067}else{cb=cb+1|0}}cb=b5+40|0;a7=b5+32|0;a8=0;while(1){if(a8>>>0>=(c[cb>>2]|0)>>>0){break}cc=go(c[(c[a7>>2]|0)+(a8<<2)>>2]|0,0)|0;if((cc|0)<0){b4=cc;r=4444;break L6067}else{a8=a8+1|0}}a8=ce+1|0;a7=c[m>>2]|0;if(a8>>>0<a7>>>0){ce=a8}else{cd=a7;break}}}}while(0);do{if((r|0)==4444){ce=(b4|0)<0?b4:0;if((ce|0)==0){cd=c[m>>2]|0;break}else{B=ce;i=e;return B|0}}}while(0);L6085:do{if((cd|0)==0){cf=0}else{b4=0;ce=cd;a9=cd;while(1){bC=c[(c[Q>>2]|0)+(b4<<2)>>2]|0;if((c[bC+40>>2]|0)==0){a7=fQ(bC)|0;if((a7|0)!=0){B=a7;r=4481;break}a7=c[m>>2]|0;cg=a7;ch=a7}else{cg=ce;ch=a9}a7=b4+1|0;if(a7>>>0<cg>>>0){b4=a7;ce=cg;a9=ch}else{break}}if((r|0)==4481){i=e;return B|0}if((ch|0)==0){cf=0;break}else{ci=0;cj=0}while(1){a9=c[(c[Q>>2]|0)+(ci<<2)>>2]|0;ce=a9+24|0;if((c[ce>>2]|0)==0){ck=0}else{b4=a9+16|0;a7=0;while(1){c[(c[(c[b4>>2]|0)+(a7<<2)>>2]|0)+100>>2]=b;c[(c[(c[b4>>2]|0)+(a7<<2)>>2]|0)+112>>2]=-1;bC=a7+1|0;a8=c[ce>>2]|0;if(bC>>>0<a8>>>0){a7=bC}else{ck=a8;break}}}a7=a9+40|0;do{if((c[a7>>2]|0)==0){cl=ck;r=4460}else{b4=a9+32|0;a8=0;do{c[(c[(c[b4>>2]|0)+(a8<<2)>>2]|0)+100>>2]=b;c[(c[(c[b4>>2]|0)+(a8<<2)>>2]|0)+112>>2]=-1;a8=a8+1|0;cm=c[a7>>2]|0;}while(a8>>>0<cm>>>0);if((cm|0)!=0){cn=cj;break}cl=c[ce>>2]|0;r=4460}}while(0);if((r|0)==4460){r=0;if(cl>>>0>(2147483647-cj|0)>>>0){B=-22;break}cn=cl+cj|0}ce=ci+1|0;if(ce>>>0<(c[m>>2]|0)>>>0){ci=ce;cj=cn}else{cf=cn;break L6085}}i=e;return B|0}}while(0);cn=A8(cf,4)|0;cj=cn;if((cn|0)==0){B=-12;i=e;return B|0}cn=c[m>>2]|0;if((cn|0)==0){co=0}else{ci=0;cl=0;r=cn;while(1){cn=c[(c[Q>>2]|0)+(ci<<2)>>2]|0;do{if((c[cn+40>>2]|0)==0){cm=cn+24|0;if((c[cm>>2]|0)==0){cp=cl;cq=r;break}ck=cn+16|0;ch=0;cg=cl;while(1){c[cj+(cg<<2)>>2]=c[(c[ck>>2]|0)+(ch<<2)>>2];cr=cg+1|0;c[(c[(c[ck>>2]|0)+(ch<<2)>>2]|0)+112>>2]=cg;cd=ch+1|0;if(cd>>>0<(c[cm>>2]|0)>>>0){ch=cd;cg=cr}else{break}}cp=cr;cq=c[m>>2]|0}else{cp=cl;cq=r}}while(0);cn=ci+1|0;if(cn>>>0<cq>>>0){ci=cn;cl=cp;r=cq}else{co=cp;break}}}if((co|0)!=(cf|0)){AN(0,0,1662792,(A=i,i=i+24|0,c[A>>2]=1646048,c[A+8>>2]=1632928,c[A+16>>2]=1058,A)|0);i=A;cw();return 0}c[b+40>>2]=cj;c[b+44>>2]=cf;B=0;i=e;return B|0}function gk(b,d,e,f,g,h,i){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;if((b|0)==0){j=-38;return j|0}k=(i&1|0)!=0;do{if((i&3|0)==1){l=gk(b,d,e,f,g,h,i|2)|0;if((l|0)==-38){break}else{j=l}return j|0}}while(0);if(!((h|0)==0|(g|0)==0)){a[g]=0}l=b+20|0;m=c[l>>2]|0;if((m|0)==0){j=-38;return j|0}n=b+8|0;b=0;o=-38;p=m;while(1){m=c[(c[n>>2]|0)+(b<<2)>>2]|0;do{if((a8(d|0,1738088)|0)==0){q=4498}else{r=c[m+8>>2]|0;if((r|0)!=0){if((a8(d|0,r|0)|0)==0){q=4498;break}}if((a8(d|0,c[c[m+4>>2]>>2]|0)|0)==0){q=4498}else{s=o;t=p}}}while(0);if((q|0)==4498){q=0;r=fU(m,e,f,g,h,i)|0;if((r|0)!=-38&(k|(r|0)<0)){j=r;q=4505;break}s=r;t=c[l>>2]|0}r=b+1|0;if(r>>>0<t>>>0){b=r;o=s;p=t}else{j=s;q=4503;break}}if((q|0)==4503){return j|0}else if((q|0)==4505){return j|0}return 0}function gl(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=+g;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;if((a|0)==0){return 0}i=a+20|0;j=c[i>>2]|0;if((j|0)==0){return 0}k=a+8|0;a=(f&1|0)==0;l=0;m=j;L6167:while(1){j=c[(c[k>>2]|0)+(l<<2)>>2]|0;L6169:do{if((j|0)==0){n=m}else{do{if((a8(b|0,1738088)|0)!=0){if((a8(b|0,c[j+8>>2]|0)|0)==0){break}if((a8(b|0,c[c[j+4>>2]>>2]|0)|0)!=0){n=m;break L6169}}}while(0);o=j+60|0;while(1){p=c[o>>2]|0;if((p|0)==0){q=0;break}if(+h[p>>3]>g){q=p;break}else{o=p+20|0}}c[o>>2]=A7(24)|0;p=A9(d)|0;c[(c[o>>2]|0)+8>>2]=p;p=A9(e)|0;c[(c[o>>2]|0)+12>>2]=p;h[c[o>>2]>>3]=g;c[(c[o>>2]|0)+16>>2]=f;c[(c[o>>2]|0)+20>>2]=q;if(!a){r=4521;break L6167}n=c[i>>2]|0}}while(0);j=l+1|0;if(j>>>0<n>>>0){l=j;m=n}else{r=4524;break}}if((r|0)==4524){return 0}else if((r|0)==4521){return 0}return 0}function gm(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;d=b+112|0;e=c[d>>2]|0;f=a+40|0;g=c[f>>2]|0;L6185:do{if((e|0)==0){h=0}else{i=b+104|0;j=e;while(1){k=j-1>>1;l=c[g+(k<<2)>>2]|0;m=l+104|0;n=c[m+4>>2]|0;o=c[i+4>>2]|0;if(!((n|0)<(o|0)|(n|0)==(o|0)&(c[m>>2]|0)>>>0<(c[i>>2]|0)>>>0)){h=j;break L6185}c[g+(j<<2)>>2]=l;c[l+112>>2]=j;if((k|0)==0){h=0;break}else{j=k}}}}while(0);c[g+(h<<2)>>2]=b;c[d>>2]=h;g=c[f>>2]|0;f=h<<1|1;e=a+44|0;a=c[e>>2]|0;if((f|0)>=(a|0)){p=h;q=g+(p<<2)|0;c[q>>2]=b;c[d>>2]=p;return}j=b+104|0;i=h;h=f;f=a;while(1){a=h+1|0;if((a|0)<(f|0)){k=(c[g+(a<<2)>>2]|0)+104|0;l=c[k+4>>2]|0;m=(c[g+(h<<2)>>2]|0)+104|0;o=c[m+4>>2]|0;r=(l|0)<(o|0)|(l|0)==(o|0)&(c[k>>2]|0)>>>0<(c[m>>2]|0)>>>0?a:h}else{r=h}a=c[j+4>>2]|0;m=c[g+(r<<2)>>2]|0;k=m+104|0;o=c[k+4>>2]|0;if((a|0)<(o|0)|(a|0)==(o|0)&(c[j>>2]|0)>>>0<(c[k>>2]|0)>>>0){p=i;s=4538;break}c[g+(i<<2)>>2]=m;c[m+112>>2]=i;m=r<<1|1;k=c[e>>2]|0;if((m|0)<(k|0)){i=r;h=m;f=k}else{p=r;s=4536;break}}if((s|0)==4538){q=g+(p<<2)|0;c[q>>2]=b;c[d>>2]=p;return}else if((s|0)==4536){q=g+(p<<2)|0;c[q>>2]=b;c[d>>2]=p;return}}function gn(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;b=i;d=a+40|0;e=a+44|0;if((c[e>>2]|0)==0){f=-541478725;i=b;return f|0}while(1){a=c[c[d>>2]>>2]|0;g=fS(a)|0;if((g|0)!=-541478725){f=g;h=4556;break}g=c[a+8>>2]|0;if((g|0)==0){j=1679472}else{j=c[g+8>>2]|0}k=c[a+12>>2]|0;if((k|0)==0){l=1679472}else{l=c[k>>2]|0}AN(g,48,1702968,(g=i,i=i+16|0,c[g>>2]=j,c[g+8>>2]=l,g)|0);i=g;g=a+112|0;a=c[g>>2]|0;k=(c[e>>2]|0)-1|0;c[e>>2]=k;if((a|0)<(k|0)){a=c[d>>2]|0;m=c[a+(k<<2)>>2]|0;n=c[g>>2]|0;o=n<<1|1;L6217:do{if((o|0)<(k|0)){p=m+104|0;q=n;r=o;s=k;while(1){t=r+1|0;if((t|0)<(s|0)){u=(c[a+(t<<2)>>2]|0)+104|0;v=c[u+4>>2]|0;w=(c[a+(r<<2)>>2]|0)+104|0;x=c[w+4>>2]|0;y=(v|0)<(x|0)|(v|0)==(x|0)&(c[u>>2]|0)>>>0<(c[w>>2]|0)>>>0?t:r}else{y=r}t=c[p+4>>2]|0;w=c[a+(y<<2)>>2]|0;u=w+104|0;x=c[u+4>>2]|0;if((t|0)<(x|0)|(t|0)==(x|0)&(c[p>>2]|0)>>>0<(c[u>>2]|0)>>>0){z=q;break L6217}c[a+(q<<2)>>2]=w;c[w+112>>2]=q;w=y<<1|1;u=c[e>>2]|0;if((w|0)<(u|0)){q=y;r=w;s=u}else{z=y;break}}}else{z=n}}while(0);c[a+(z<<2)>>2]=m;c[m+112>>2]=z}c[g>>2]=-1;if((c[e>>2]|0)==0){f=-541478725;h=4557;break}}if((h|0)==4556){i=b;return f|0}else if((h|0)==4557){i=b;return f|0}return 0}function go(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;e=i;if((b|0)==0){f=0;i=e;return f|0}g=b+64|0;h=c[g>>2]|0;if((h|0)==0){f=0;i=e;return f|0}j=b+16|0;do{if((c[j>>2]|0)!=0|(d|0)==0){k=h}else{if((c[d+16>>2]|0)!=0){k=h;break}l=d+52|0;m=(a[(BP(c[l>>2]|0)|0)+4|0]&1^1)&255;n=c[g>>2]|0;if((c[n>>2]|0)==0){o=-1}else{p=-1;q=0;r=n;while(1){n=n$(p,c[(c[r+4>>2]|0)+(q<<2)>>2]|0,c[l>>2]|0,m,0)|0;s=q+1|0;t=c[g>>2]|0;if(s>>>0<(c[t>>2]|0)>>>0){p=n;q=s;r=t}else{o=n;break}}}r=c[b>>2]|0;q=BL(o)|0;p=c[c[g>>2]>>2]|0;n=BL(c[l>>2]|0)|0;AN(r,48,1619952,(u=i,i=i+32|0,c[u>>2]=q,c[u+8>>2]=p,c[u+16>>2]=n,c[u+24>>2]=m,u)|0);i=u;c[c[(c[g>>2]|0)+4>>2]>>2]=o;k=c[g>>2]|0}}while(0);c[k>>2]=1;c[b+52>>2]=c[c[(c[g>>2]|0)+4>>2]>>2];k=b+72|0;L6245:do{if((c[j>>2]|0)==1){o=c[k>>2]|0;if((c[o>>2]|0)==0){d=c[b>>2]|0;h=c[(c[b+8>>2]|0)+8>>2]|0;AN(d,16,1615696,(u=i,i=i+16|0,c[u>>2]=c[d+8>>2],c[u+8>>2]=h,u)|0);i=u;f=-22;i=e;return f|0}c[o>>2]=1;c[b+48>>2]=c[c[(c[k>>2]|0)+4>>2]>>2];o=b+80|0;h=c[o>>2]|0;if((a[h+8|0]|0)!=0){d=c[b>>2]|0;n=c[(c[b+8>>2]|0)+8>>2]|0;AN(d,16,1744784,(u=i,i=i+16|0,c[u>>2]=c[d+8>>2],c[u+8>>2]=n,u)|0);i=u;f=-22;i=e;return f|0}c[h+4>>2]=1;h=c[c[o>>2]>>2]|0;n=c[h>>2]|0;d=c[h+4>>2]|0;h=b+40|0;c[h>>2]=n;c[h+4>>2]=d;p=0;do{if((d|0)<(p|0)|(d|0)==(p|0)&n>>>0<0>>>0){q=n&2147483647;r=b+148|0;c[r>>2]=q;if((q|0)==0){v=r;break}c[h>>2]=0;c[h+4>>2]=0;w=o;break L6245}else{r=b+148|0;c[r>>2]=0;v=r}}while(0);c[v>>2]=zs(n,d)|0;w=o}else{w=b+80|0}}while(0);g2(g);g2(b+68|0);g2(k);g2(b+76|0);g3(w);g3(b+84|0);f=0;i=e;return f|0}function gp(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;d=i;i=i+64|0;e=d|0;f=b+16|0;g=c[f>>2]|0;do{if((g|0)==0){h=4586}else{j=c[g>>2]|0;if((j|0)==0){h=4586;break}k=c[j+16>>2]|0}}while(0);do{if((h|0)==4586){g=c[b+32>>2]|0;if((g|0)==0){k=0;break}j=c[g>>2]|0;if((j|0)==0){k=0;break}k=c[j+16>>2]|0}}while(0);j=c1[c[(c[b+4>>2]|0)+36>>2]&255](b)|0;if((j|0)<0){if((j|0)==-11){l=-11;i=d;return l|0}g=b;m=c[b+8>>2]|0;n=e|0;NM(n|0,0,64);a[n]=0;zN(j,n,64)|0;AN(g,16,1692840,(o=i,i=i+16|0,c[o>>2]=m,c[o+8>>2]=n,o)|0);i=o;l=j;i=d;return l|0}j=b+24|0;if((c[j>>2]|0)!=0){n=b;m=0;do{g=c[(c[(c[f>>2]|0)+(m<<2)>>2]|0)+84>>2]|0;do{if((g|0)!=0){if((c[g+4>>2]|0)==0){e=g+8|0;do{if((a[g+9|0]|0)!=0){if((a[e]|0)!=0){break}AN(n,24,1686888,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o}}while(0);a[e]=1;break}else{p=g+8|0;q=g+9|0;if((a[p]|0)==0){if((a[q]|0)!=0){h=4600}}else{h=4600}if((h|0)==4600){h=0;AN(n,24,1689616,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o}a[q]=0;a[p]=0;break}}}while(0);m=m+1|0;}while(m>>>0<(c[j>>2]|0)>>>0)}j=b+40|0;if((c[j>>2]|0)!=0){m=b;n=b+32|0;f=0;do{g=c[(c[(c[n>>2]|0)+(f<<2)>>2]|0)+80>>2]|0;do{if((g|0)!=0){if((c[g+4>>2]|0)==0){p=g+8|0;do{if((a[g+9|0]|0)!=0){if((a[p]|0)!=0){break}AN(m,24,1686888,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o}}while(0);a[p]=1;break}else{e=g+8|0;q=g+9|0;if((a[e]|0)==0){if((a[q]|0)!=0){h=4611}}else{h=4611}if((h|0)==4611){h=0;AN(m,24,1689616,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o}a[q]=0;a[e]=0;break}}}while(0);f=f+1|0;}while(f>>>0<(c[j>>2]|0)>>>0)}j=gY(k)|0;if((j|0)==0){l=-12;i=d;return l|0}g8(b,j);if((k|0)!=1){l=0;i=d;return l|0}k=gZ()|0;if((k|0)==0){l=-12;i=d;return l|0}g7(b,k);k=g_()|0;if((k|0)==0){l=-12;i=d;return l|0}g6(b,k);l=0;i=d;return l|0}function gq(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0;e=i;i=i+8|0;f=e|0;g=c[c[a+16>>2]>>2]|0;h=(c[a+44>>2]|0)+4|0;do{if((z1(c[h>>2]|0)|0)==0){if((d&2|0)!=0){j=-11;i=e;return j|0}a=fS(g)|0;if((a|0)<0){j=a}else{break}i=e;return j|0}}while(0);if((z1(c[h>>2]|0)|0)==0){j=-22;i=e;return j|0}g=c[h>>2]|0;do{if((d&1|0)==0){h=f;z4(g,h,4,0)|0;Az(b,c[f>>2]|0);At(f)}else{h=c[g>>2]|0;a=c[g+4>>2]|0;k=c[g+12>>2]|0;do{if(a>>>0<k>>>0){if(a>>>0>=h>>>0){l=a;break}l=k+(a-h)|0}else{l=h+(a-k)|0}}while(0);k=c[l>>2]|0;c[f>>2]=k;a=Aw(b,k)|0;if((a|0)<0){j=a}else{break}i=e;return j|0}}while(0);j=0;i=e;return j|0}function gr(a,b){a=a|0;b=b|0;var d=0;d=c[c[a+16>>2]>>2]|0;c[d+128>>2]=b;c[d+136>>2]=b;c[d+132>>2]=b;return}function gs(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=i;e=c[c[b+4>>2]>>2]|0;do{if((a8(e|0,1734472)|0)!=0){if((a8(e|0,1702776)|0)==0){break}AN(0,0,1662672,(f=i,i=i+24|0,c[f>>2]=1626288,c[f+8>>2]=1632864,c[f+16>>2]=357,f)|0);i=f;cw()}}while(0);e=(c[c[b+16>>2]>>2]|0)+116|0;b=a;a=c[e+4>>2]|0;c[b>>2]=c[e>>2];c[b+4>>2]=a;i=d;return}function gt(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0;b=i;i=i+8|0;d=b|0;e=c[a+44>>2]|0;a=c[e+56>>2]|0;if((a|0)!=0){yX(a)}a=e+4|0;e=c[a>>2]|0;if((e|0)==0){i=b;return}f=(z1(e)|0)>>>0>3;e=c[a>>2]|0;if(f){f=d;g=e;while(1){z4(g,f,4,0)|0;At(d);h=(z1(c[a>>2]|0)|0)>>>0>3;j=c[a>>2]|0;if(h){g=j}else{k=j;break}}}else{k=e}z0(k);c[a>>2]=0;i=b;return}function gu(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0;b=i;i=i+8|0;d=b|0;e=c[a+44>>2]|0;c[d>>2]=0;f=e+16|0;g=c[f>>2]|0;if((g&3|0)!=0){AN(a,16,1679408,(h=i,i=i+16|0,c[h>>2]=g,c[h+8>>2]=4,h)|0);i=h;j=-22;i=b;return j|0}if((g|0)==0){g9(a)|0;j=0;i=b;return j|0}h=e+12|0;e=0;k=g;while(1){if(e>>>0>=k>>>2>>>0){l=4672;break}g=c[(c[h>>2]|0)+(e<<2)>>2]|0;m=gW(d,g,(g|0)<0|0?-1:0)|0;if((m|0)<0){l=4671;break}e=e+1|0;k=c[f>>2]|0}if((l|0)==4672){g8(a,c[d>>2]|0);j=0;i=b;return j|0}else if((l|0)==4671){g2(d);j=m;i=b;return j|0}return 0}function gv(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0;d=i;e=a+44|0;f=c[e>>2]|0;do{if((b|0)==0){g=f}else{h=c[b>>2]|0;j=Cr(4,h,-1,-1)|0;if(j>>>0>536870911){k=-22;i=d;return k|0}l=Bj(f,1697608,h,j<<2,0)|0;if((l|0)<0){k=l;i=d;return k|0}else{g=c[e>>2]|0;break}}}while(0);e=z$(32)|0;c[g+4>>2]=e;if((e|0)==0){AN(a,16,1692760,(a=i,i=i+1|0,i=i+7&-8,c[a>>2]=0,a)|0);i=a;k=-12;i=d;return k|0}else{c[g+8>>2]=100;k=0;i=d;return k|0}return 0}function gw(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;b=i;i=i+16|0;d=b|0;e=b+8|0;f=c[a+44>>2]|0;c[d>>2]=0;c[e>>2]=0;g=f+24|0;h=c[g>>2]|0;if((h&3|0)!=0){AN(a,16,1689544,(j=i,i=i+16|0,c[j>>2]=h,c[j+8>>2]=4,j)|0);i=j;k=-22;i=b;return k|0}l=f+52|0;m=c[l>>2]|0;if((m&3|0)!=0){AN(a,16,1686808,(j=i,i=i+16|0,c[j>>2]=m,c[j+8>>2]=4,j)|0);i=j;k=-22;i=b;return k|0}m=f+32|0;n=c[m>>2]|0;if((n&7|0)!=0){AN(a,16,1685176,(j=i,i=i+16|0,c[j>>2]=n,c[j+8>>2]=8,j)|0);i=j;k=-22;i=b;return k|0}o=f+40|0;p=c[o>>2]|0;if((p&3|0)!=0){AN(a,16,1683096,(j=i,i=i+16|0,c[j>>2]=p,c[j+8>>2]=4,j)|0);i=j;k=-22;i=b;return k|0}do{if((h|0)==0){q=n}else{p=f+20|0;r=0;s=h;while(1){if(r>>>0>=s>>>2>>>0){t=4705;break}u=c[(c[p>>2]|0)+(r<<2)>>2]|0;v=gW(d,u,(u|0)<0|0?-1:0)|0;if((v|0)<0){break}r=r+1|0;s=c[g>>2]|0}if((t|0)==4705){g8(a,c[d>>2]|0);q=c[m>>2]|0;break}g2(d);k=v;i=b;return k|0}}while(0);do{if((q|0)==0){if((c[o>>2]|0)!=0){t=4709;break}if((c[f+44>>2]|0)!=0){t=4709}}else{t=4709}}while(0);if((t|0)==4709){v=f+28|0;g=0;h=q;while(1){if(g>>>0>=h>>>3>>>0){break}q=(c[v>>2]|0)+(g<<3)|0;w=gX(e,c[q>>2]|0,c[q+4>>2]|0)|0;if((w|0)<0){t=4714;break}g=g+1|0;h=c[m>>2]|0}if((t|0)==4714){g3(e);k=w;i=b;return k|0}w=f+36|0;m=0;while(1){if(m>>>0>=(c[o>>2]|0)>>>2>>>0){break}h=c[(c[w>>2]|0)+(m<<2)>>2]|0;x=gX(e,h|0,((h|0)<0|0?-1:0)|-2147483648)|0;if((x|0)<0){t=4718;break}m=m+1|0}if((t|0)==4718){g3(e);k=x;i=b;return k|0}do{if((c[f+44>>2]|0)!=0){if((c[e>>2]|0)!=0){AN(a,24,1681264,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0);i=j;break}x=g$()|0;c[e>>2]=x;if((x|0)==0){k=-12}else{break}i=b;return k|0}}while(0);g6(a,c[e>>2]|0)}if((c[l>>2]|0)==0){k=0;i=b;return k|0}c[d>>2]=0;e=f+48|0;f=0;while(1){if(f>>>0>=(c[l>>2]|0)>>>2>>>0){t=4729;break}j=c[(c[e>>2]|0)+(f<<2)>>2]|0;y=gW(d,j,(j|0)<0|0?-1:0)|0;if((y|0)<0){t=4728;break}else{f=f+1|0}}if((t|0)==4728){g2(d);k=y;i=b;return k|0}else if((t|0)==4729){g7(a,c[d>>2]|0);k=0;i=b;return k|0}return 0}function gx(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0;d=i;e=a+44|0;f=c[e>>2]|0;do{if((b|0)==0){g=f}else{h=c[b>>2]|0;j=Cr(4,h,-1,-1)|0;if(j>>>0>536870911){k=-22;i=d;return k|0}l=Bj(f,1744704,h,j<<2,0)|0;if((l|0)<0){k=l;i=d;return k|0}l=c[b+16>>2]|0;j=Cr(4,l,-1,-1)|0;if(j>>>0>536870911){k=-22;i=d;return k|0}h=Bj(f,1733456,l,j<<2,0)|0;if((h|0)<0){k=h;i=d;return k|0}h=c[b+4>>2]|0;j=Cr(8,h,-1,-1)|0;if(j>>>0>268435455){k=-22;i=d;return k|0}l=Bj(f,1724128,h,j<<3,0)|0;if((l|0)<0){k=l;i=d;return k|0}l=c[b+8>>2]|0;j=Cr(4,l,-1,-1)|0;if(j>>>0>536870911){k=-22;i=d;return k|0}h=Bj(f,1716480,l,j<<2,0)|0;if((h|0)<0){k=h;i=d;return k|0}h=c[b+12>>2]|0;j=Bh(f,1709120,h,(h|0)<0|0?-1:0,0)|0;if((j|0)<0){k=j;i=d;return k|0}else{g=c[e>>2]|0;break}}}while(0);e=z$(32)|0;c[g+4>>2]=e;if((e|0)==0){AN(a,16,1692760,(a=i,i=i+1|0,i=i+7&-8,c[a>>2]=0,a)|0);i=a;k=-12;i=d;return k|0}else{c[g+8>>2]=100;k=0;i=d;return k|0}return 0}function gy(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0;d=i;i=i+8|0;e=d|0;f=c[a+8>>2]|0;a=f+44|0;g=c[a>>2]|0;h=e;c[e>>2]=b;b=(c[a>>2]|0)+4|0;do{if((z2(c[b>>2]|0)|0)>>>0<4){a=c[b>>2]|0;if((z3(a,(z1(a)|0)<<1)|0)>=0){break}AN(f,16,1700320,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0);i=j;k=-12;i=d;return k|0}}while(0);z5(c[b>>2]|0,h,4,0)|0;h=g+8|0;if((c[h>>2]|0)==0){k=0;i=d;return k|0}b=(z1(c[g+4>>2]|0)|0)>>>2;g=c[h>>2]|0;if(b>>>0<g>>>0){k=0;i=d;return k|0}b=c[f+8>>2]|0;a=(b|0)!=0?b:c[c[f+4>>2]>>2]|0;AN(f,24,1702904,(j=i,i=i+16|0,c[j>>2]=g,c[j+8>>2]=a,j)|0);i=j;c[h>>2]=(c[h>>2]|0)*10|0;k=0;i=d;return k|0}function gz(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0;e=i;i=i+8|0;f=e|0;c[f>>2]=0;do{if((b|0)!=0){g=b+352|0;h=c[g>>2]|0;j=c[g+4>>2]|0;do{if(!((h|0)==0&(j|0)==0)){g=zs(h,j)|0;if((g|0)==(An(b)|0)){break}AN(0,16,1732472,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g;k=-22;i=e;return k|0}}while(0);if((d&8|0)==0){break}j=As()|0;c[f>>2]=j;if((j|0)==0){k=-12;i=e;return k|0}h=Aw(j,b)|0;if((h|0)>-1){l=gA(a,j,d)|0}else{l=h}At(f);k=l;i=e;return k|0}}while(0);k=gA(a,b,d)|0;i=e;return k|0}function gA(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;e=i;i=i+8|0;f=e|0;g=c[a+44>>2]|0;c[g+24>>2]=0;h=g+92|0;if((b|0)==0){c[h>>2]=1;j=0;i=e;return j|0}if((c[h>>2]|0)!=0){j=-22;i=e;return j|0}L6549:do{if((d&1|0)==0){h=c[(c[c[a+32>>2]>>2]|0)+16>>2]|0;if((h|0)==0){do{if((c[g+32>>2]|0)==(c[b+68>>2]|0)){if((c[g+36>>2]|0)!=(c[b+72>>2]|0)){break}if((c[g+40>>2]|0)==(c[b+80>>2]|0)){break L6549}}}while(0);AN(a,32,1675512,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;break}else if((h|0)==1){l=b+352|0;m=c[l>>2]|0;n=c[l+4>>2]|0;if((m|0)==0&(n|0)==0){o=g+80|0;p=c[o>>2]|0;q=c[o+4>>2]|0;c[l>>2]=p;c[l+4>>2]=q;r=q;s=p}else{r=n;s=m}do{if((c[g+64>>2]|0)==(c[b+80>>2]|0)){if((c[g+60>>2]|0)!=(c[b+344>>2]|0)){break}m=g+80|0;if(!((c[m>>2]|0)==(s|0)&(c[m+4>>2]|0)==(r|0))){break}m=c[g+72>>2]|0;if((m|0)==(An(b)|0)){break L6549}}}while(0);AN(a,16,1673752,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;j=-22;i=e;return j|0}else{j=-22;i=e;return j|0}}}while(0);k=g+4|0;do{if((z2(c[k>>2]|0)|0)==0){g=c[k>>2]|0;r=z3(g,(z1(g)|0)+4|0)|0;if((r|0)<0){j=r}else{break}i=e;return j|0}}while(0);r=As()|0;c[f>>2]=r;if((r|0)==0){j=-12;i=e;return j|0}Az(r,b);r=z5(c[k>>2]|0,f,4,0)|0;if((r|0)<0){Az(b,c[f>>2]|0);At(f);j=r;i=e;return j|0}do{if((d&4|0)!=0){r=c1[c[(c[a+28>>2]|0)+44>>2]&255](c[c[a+32>>2]>>2]|0)|0;if((r|0)<0){j=r}else{break}i=e;return j|0}}while(0);j=0;i=e;return j|0}function gB(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;f=i;i=i+16|0;g=f|0;h=f+8|0;j=c[a+44>>2]|0;c[g>>2]=0;c[h>>2]=0;k=j+92|0;if((b|0)==0){c[k>>2]=1;l=0;i=f;return l|0}if((c[k>>2]|0)!=0){l=-22;i=f;return l|0}k=As()|0;c[g>>2]=k;if((k|0)==0){l=-12;i=f;return l|0}k=b+100|0;j=ze(0,0,60,b,(c[k>>2]|0)>>>1&1^1)|0;c[h>>2]=j;L6596:do{if((j|0)==0){m=-12}else{n=hK(c[g>>2]|0,b)|0;if((n|0)<0){m=n;break}n=c[(c[g>>2]|0)+80>>2]|0;L6599:do{if((c[(c[c[a+32>>2]>>2]|0)+16>>2]|0)==0){o=BP(n)|0;p=BT(c[(c[g>>2]|0)+80>>2]|0)|0;if((o|0)==0|(p|0)<1){m=-22;break L6596}q=o+6|0;o=0;while(1){if((o|0)>=(p|0)){break L6599}if((o-1|0)>>>0<2){r=d[q]|0}else{r=0}s=c[g>>2]|0;t=c[s+72>>2]|0;u=c[s+32+(o<<2)>>2]|0;s=zi(c[h>>2]|0)|0;if((s|0)==0){m=-12;break L6596}v=ac(t>>r,u)|0;u=ze(c[(c[g>>2]|0)+(o<<2)>>2]|0,v,2,s,(c[k>>2]|0)>>>1&1^1)|0;c[(c[g>>2]|0)+360+(o<<2)>>2]=u;w=c[g>>2]|0;if((c[w+360+(o<<2)>>2]|0)==0){break}else{o=o+1|0}}Au(w);m=-12;break L6596}else{o=B8(n)|0;q=(c[g>>2]|0)+352|0;p=(o|0)!=0?zs(c[q>>2]|0,c[q+4>>2]|0)|0:1;q=p>>>0>8;if(q){c[(c[g>>2]|0)+396>>2]=p-8;o=A7(c[(c[g>>2]|0)+396>>2]<<2)|0;c[(c[g>>2]|0)+392>>2]=o;if((c[(c[g>>2]|0)+392>>2]|0)==0){m=-12;break L6596}}o=q?8:p;q=0;while(1){if(q>>>0>=o>>>0){break}u=zi(c[h>>2]|0)|0;if((u|0)==0){m=-12;break L6596}s=c[g>>2]|0;v=ze(c[(c[s+64>>2]|0)+(q<<2)>>2]|0,c[s+32>>2]|0,2,u,(c[k>>2]|0)>>>1&1^1)|0;c[(c[g>>2]|0)+360+(q<<2)>>2]=v;x=c[g>>2]|0;if((c[x+360+(q<<2)>>2]|0)==0){y=4850;break}else{q=q+1|0}}if((y|0)==4850){Au(x);m=-12;break L6596}q=p-8|0;o=0;while(1){if(o>>>0>=q>>>0){break L6599}v=zi(c[h>>2]|0)|0;if((v|0)==0){m=-12;break L6596}u=c[g>>2]|0;s=ze(c[(c[u+64>>2]|0)+(o+8<<2)>>2]|0,c[u+32>>2]|0,2,v,(c[k>>2]|0)>>>1&1^1)|0;c[(c[(c[g>>2]|0)+392>>2]|0)+(o<<2)>>2]=s;z=c[g>>2]|0;if((c[(c[z+392>>2]|0)+(o<<2)>>2]|0)==0){break}else{o=o+1|0}}Au(z);m=-12;break L6596}}while(0);m=gz(a,c[g>>2]|0,e)|0}}while(0);zj(h);At(g);l=m;i=f;return l|0}function gC(a,b){a=a|0;b=b|0;hL(a);return}function gD(a,b){a=a|0;b=b|0;var d=0;b=i;i=i+8|0;d=b|0;c[d>>2]=a;zj(d);i=b;return}function gE(a){a=a|0;return c[(c[a+44>>2]|0)+24>>2]|0}function gF(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;b=i;d=c[a+44>>2]|0;e=d+40|0;do{if((c[e>>2]|0)!=-1){f=d+32|0;if((c[f>>2]|0)==0){break}g=d+36|0;if((c[g>>2]|0)==0){break}h=d+8|0;j=d+12|0;if(+(c[h>>2]|0)/+(c[j>>2]|0)<=0.0){break}k=z$(4)|0;c[d+4>>2]=k;if((k|0)==0){l=-12;i=b;return l|0}k=c[f>>2]|0;f=c[g>>2]|0;g=BL(c[e>>2]|0)|0;m=c[h>>2]|0;h=c[j>>2]|0;j=c[d+16>>2]|0;n=c[d+20>>2]|0;o=c[d+48>>2]|0;p=c[d+52>>2]|0;q=c[d+56>>2]|0;AN(a,40,1712776,(r=i,i=i+80|0,c[r>>2]=k,c[r+8>>2]=f,c[r+16>>2]=g,c[r+24>>2]=m,c[r+32>>2]=h,c[r+40>>2]=j,c[r+48>>2]=n,c[r+56>>2]=o,c[r+64>>2]=p,c[r+72>>2]=(q|0)!=0?q:3119160,r)|0);i=r;c[d+28>>2]=100;l=0;i=b;return l|0}}while(0);AN(a,16,1716448,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;l=-22;i=b;return l|0}function gG(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0;b=i;i=i+8|0;d=b|0;e=(c[a+44>>2]|0)+4|0;a=c[e>>2]|0;L6645:do{if((a|0)==0){f=0}else{g=d;h=a;while(1){j=(z1(h)|0)==0;k=c[e>>2]|0;if(j){f=k;break L6645}z4(k,g,4,0)|0;At(d);k=c[e>>2]|0;if((k|0)==0){f=0;break}else{h=k}}}}while(0);z0(f);c[e>>2]=0;i=b;return}function gH(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;b=i;i=i+24|0;d=b|0;e=b+8|0;f=b+16|0;g=c[a+44>>2]|0;c[d>>2]=0;c[e>>2]=0;c[f>>2]=0;h=c[(c[c[a+32>>2]>>2]|0)+16>>2]|0;if((h|0)==0){j=c[g+40>>2]|0;k=j;l=(j|0)<0|0?-1:0;gW(e,k,l)|0;g8(a,c[e>>2]|0);m=0;i=b;return m|0}else if((h|0)==1){h=c[g+64>>2]|0;gW(e,h,(h|0)<0|0?-1:0)|0;g8(a,c[e>>2]|0);e=c[g+60>>2]|0;gW(f,e,(e|0)<0|0?-1:0)|0;g7(a,c[f>>2]|0);f=g+80|0;e=c[f>>2]|0;h=c[f+4>>2]|0;if((e|0)==0&(h|0)==0){f=c[g+72>>2]|0;n=((f|0)<0|0?-1:0)|-2147483648;o=f|0}else{n=h;o=e}gX(d,o,n)|0;g6(a,c[d>>2]|0);m=0;i=b;return m|0}else{m=-22;i=b;return m|0}return 0}function gI(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;b=i;d=c[a+44>>2]|0;e=d+68|0;f=B3(c[e>>2]|0)|0;c[d+64>>2]=f;if((f|0)==-1){AN(a,16,1645864,(g=i,i=i+8|0,c[g>>2]=c[e>>2],g)|0);i=g;h=-22;i=b;return h|0}f=d+88|0;j=c[f>>2]|0;do{if((j|0)==0){if((c[d+72>>2]|0)!=0){break}AN(a,16,1619712,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g;h=-22;i=b;return h|0}else{k=zq(j)|0;l=G;m=d+80|0;c[m>>2]=k;c[m+4>>2]=l;if((k|0)==0&(l|0)==0){AN(a,16,1632784,(g=i,i=i+8|0,c[g>>2]=c[f>>2],g)|0);i=g;h=-22;i=b;return h|0}m=zs(k,l)|0;l=d+72|0;k=c[l>>2]|0;if((k|0)==0|(m|0)==(k|0)){c[l>>2]=m;break}l=c[f>>2]|0;AN(a,16,1626160,(g=i,i=i+24|0,c[g>>2]=k,c[g+8>>2]=l,c[g+16>>2]=m,g)|0);i=g;h=-22;i=b;return h|0}}while(0);j=z$(4)|0;c[d+4>>2]=j;if((j|0)==0){h=-12;i=b;return h|0}j=d+8|0;m=c[j>>2]|0;if((m|0)==0){l=c[d+60>>2]|0;c[j>>2]=1;c[d+12>>2]=l;n=1;o=l;p=l}else{n=m;o=c[d+12>>2]|0;p=c[d+60>>2]|0}m=c[e>>2]|0;e=c[f>>2]|0;AN(a,40,1615528,(g=i,i=i+40|0,c[g>>2]=n,c[g+8>>2]=o,c[g+16>>2]=m,c[g+24>>2]=p,c[g+32>>2]=e,g)|0);i=g;c[d+28>>2]=100;h=0;i=b;return h|0}function gJ(a){a=a|0;var b=0,d=0;b=c[(c[a>>2]|0)+44>>2]|0;a=z1(c[b+4>>2]|0)|0;do{if((a|0)==0){if((c[b+92>>2]|0)==0){break}else{d=-541478725}return d|0}}while(0);d=a>>>2;return d|0}function gK(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=i;i=i+8|0;d=b|0;e=c[(c[a>>2]|0)+44>>2]|0;f=e+4|0;if((z1(c[f>>2]|0)|0)!=0){g=c[f>>2]|0;f=d;z4(g,f,4,0)|0;h=f7(a,c[d>>2]|0)|0;i=b;return h|0}if((c[e+92>>2]|0)!=0){h=-541478725;i=b;return h|0}d=e+24|0;c[d>>2]=(c[d>>2]|0)+1;h=-11;i=b;return h|0}function gL(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=c[(c[a>>2]|0)+44>>2]|0;d=c[a+16>>2]|0;do{if((d|0)==0){c[a+20>>2]=c[b+32>>2];c[a+24>>2]=c[b+36>>2];e=b+48|0;f=a+28|0;g=c[e+4>>2]|0;c[f>>2]=c[e>>2];c[f+4>>2]=g}else if((d|0)==1){g=b+80|0;if(!((c[g>>2]|0)==0&(c[g+4>>2]|0)==0)){break}f=a+40|0;e=c[f+4>>2]|0;c[g>>2]=c[f>>2];c[g+4>>2]=e}else{h=-22;return h|0}}while(0);d=b+8|0;e=a+56|0;g=c[d+4>>2]|0;c[e>>2]=c[d>>2];c[e+4>>2]=g;g=b+16|0;b=a+116|0;a=c[g+4>>2]|0;c[b>>2]=c[g>>2];c[b+4>>2]=a;h=0;return h|0}function gM(a){a=a|0;var b=0;b=c[a+44>>2]|0;c[b+8>>2]=b;return 0}function gN(a){a=a|0;var b=0,d=0,e=0,f=0;b=c[a+44>>2]|0;a=c[b+4>>2]|0;if((a|0)==0){d=b+12|0;e=d;At(e);return}else{f=a}while(1){a=c[f+4>>2]|0;At(f|0);A4(f);if((a|0)==0){break}else{f=a}}d=b+12|0;e=d;At(e);return}function gO(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;b=i;i=i+8|0;d=b|0;e=a|0;f=c[e>>2]|0;g=c[f+44>>2]|0;h=g+4|0;j=h;k=c[j>>2]|0;do{if((k|0)==0){l=fS(c[c[f+16>>2]>>2]|0)|0;if((l|0)<0){m=l;i=b;return m|0}l=c[j>>2]|0;if((l|0)!=0){n=l;break}AN(0,0,1645720,(o=i,i=i+24|0,c[o>>2]=1632720,c[o+8>>2]=1626080,c[o+16>>2]=232,o)|0);i=o;cw();return 0}else{n=k}}while(0);if((c[a+88>>2]|0)==0){k=f7(a,c[n>>2]|0)|0;n=c[j>>2]|0;a=c[n+4>>2]|0;f=g+8|0;if((c[f>>2]|0)==(n|0)){c[f>>2]=g}A6(h);c[j>>2]=a;m=k;i=b;return m|0}k=c[e>>2]|0;e=c[c[k+32>>2]>>2]|0;a=c[k+44>>2]|0;j=a+4|0;h=j;g=c[c[h>>2]>>2]|0;c[d>>2]=g;f=a+12|0;do{if((c[f>>2]|0)==0){n=e+88|0;if((c[g+76>>2]|0)<(c[n>>2]|0)){p=n;q=4961;break}if((B8(c[g+80>>2]|0)|0)==0){r=1;q=4950}else{l=g+352|0;s=zs(c[l>>2]|0,c[l+4>>2]|0)|0;if((s|0)>0){r=s;q=4950}}if((q|0)==4950){s=c[g+64>>2]|0;l=128;t=0;do{u=c[s+(t<<2)>>2]|0;v=128;while(1){if(((u|0)%(v|0)|0|0)==0){break}else{v=v>>1}}l=(v|0)<(l|0)?v:l;t=t+1|0;}while((t|0)<(r|0));if((l|0)<=31){p=n;q=4961;break}}t=c[d>>2]|0;if((c[t+76>>2]|0)==(c[n>>2]|0)){s=c[h>>2]|0;u=c[s+4>>2]|0;w=a+8|0;if((c[w>>2]|0)==(s|0)){c[w>>2]=a}A6(j);c[h>>2]=u;x=t;break}else{u=Ay(t)|0;if((u|0)==0){m=-12;i=b;return m|0}else{c[u+76>>2]=c[n>>2];gP(e,c[d>>2]|0,c[n>>2]|0);x=u;break}}}else{p=e+88|0;q=4961}}while(0);if((q|0)==4961){r=e+40|0;g=zs(c[r>>2]|0,c[r+4>>2]|0)|0;r=c[f>>2]|0;u=c[p>>2]|0;do{if((r|0)==0){t=hI(e,u)|0;c[f>>2]=t;if((t|0)==0){m=-12;i=b;return m|0}else{c[t+76>>2]=0;t=(c[d>>2]|0)+136|0;w=c[t+4>>2]|0;s=(c[f>>2]|0)+136|0;c[s>>2]=c[t>>2];c[s+4>>2]=w;w=c[p>>2]|0;c[a+16>>2]=w;y=c[f>>2]|0;z=w;break}}else{if((u|0)==(c[a+16>>2]|0)){y=r;z=u;break}AN(k,16,1619632,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;m=-22;i=b;return m|0}}while(0);o=a+16|0;L6768:do{if((c[y+76>>2]|0)<(z|0)){u=k+16|0;r=e+52|0;p=a+8|0;w=a;while(1){s=c[h>>2]|0;if((s|0)==0){t=fS(c[c[u>>2]>>2]|0)|0;if((t|0)==-541478725){q=4970;break}if((t|0)<0){m=t;q=4989;break}A=c[h>>2]|0}else{A=s}s=c[A>>2]|0;c[d>>2]=s;t=c[f>>2]|0;B=c[t+76>>2]|0;C=(c[o>>2]|0)-B|0;D=c[s+76>>2]|0;E=(C|0)>(D|0)?D:C;Cb(c[t+64>>2]|0,c[s+64>>2]|0,B,0,E,g,c[r>>2]|0)|0;B=(c[f>>2]|0)+76|0;c[B>>2]=(c[B>>2]|0)+E;B=c[d>>2]|0;if((E|0)==(c[B+76>>2]|0)){At(d);s=c[h>>2]|0;t=c[s+4>>2]|0;if((c[p>>2]|0)==(s|0)){c[p>>2]=w}A6(j);c[h>>2]=t}else{gP(e,B,E)}E=c[f>>2]|0;if((c[E+76>>2]|0)>=(c[o>>2]|0)){F=E;break L6768}}if((q|0)==4970){w=c[f>>2]|0;p=c[w+64>>2]|0;u=c[w+76>>2]|0;w=(c[o>>2]|0)-u|0;n=c[r>>2]|0;Ca(p,u,w,g,n)|0;c[(c[f>>2]|0)+76>>2]=c[o>>2];F=c[f>>2]|0;break}else if((q|0)==4989){i=b;return m|0}}else{F=y}}while(0);c[f>>2]=0;x=F}m=f7(e,x)|0;i=b;return m|0}function gP(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;e=i;i=i+8|0;f=e|0;g=a+40|0;h=zs(c[g>>2]|0,c[g+4>>2]|0)|0;g=a+52|0;j=(B8(c[g>>2]|0)|0)!=0;k=j?h:1;l=ac(B7(c[g>>2]|0)|0,j?1:h)|0;h=b+76|0;if((c[h>>2]|0)<=(d|0)){AN(0,0,1645720,(j=i,i=i+24|0,c[j>>2]=1615480,c[j+8>>2]=1626080,c[j+16>>2]=113,j)|0);i=j;cw()}if((k|0)>0){j=ac(l,d)|0;g=b+64|0;m=0;while(1){n=(c[g>>2]|0)+(m<<2)|0;c[n>>2]=(c[n>>2]|0)+j;n=m+1|0;if((n|0)<(k|0)){m=n}else{o=g;break}}}else{o=b+64|0}g=c[o>>2]|0;if((b|0)!=(g|0)){o=b;m=g;g=k>>>0>8?32:k<<2;NN(o|0,m|0,g)|0}g=ac(l,d)|0;l=b+32|0;c[l>>2]=(c[l>>2]|0)-g;c[h>>2]=(c[h>>2]|0)-d;h=b+136|0;if((c[h>>2]|0)==0&(c[h+4>>2]|0)==(-2147483648|0)){i=e;return}c[f>>2]=1;c[f+4>>2]=c[a+48>>2];b=AY(d,(d|0)<0|0?-1:0,f,a+56|0)|0;a=NU(c[h>>2]|0,c[h+4>>2]|0,b,G)|0;c[h>>2]=a;c[h+4>>2]=G;i=e;return}function gQ(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=i;i=i+8|0;e=d|0;c[e>>2]=b;b=c[(c[a+8>>2]|0)+44>>2]|0;a=A7(8)|0;f=b+8|0;c[(c[f>>2]|0)+4>>2]=a;a=c[(c[f>>2]|0)+4>>2]|0;if((a|0)==0){At(e);g=-12;i=d;return g|0}else{c[f>>2]=a;c[a>>2]=c[e>>2];g=0;i=d;return g|0}return 0}function gR(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0;g=i;i=i+24|0;h=g|0;j=g+8|0;k=g+16|0;c[h>>2]=b;c[j>>2]=e;c[k>>2]=0;if((b|0)==(e|0)){l=b;i=g;return l|0}do{if((f|0)==0){if((c[b>>2]|0)==0){m=0;n=0;o=0;p=0;q=b;r=e;break}else{s=0;t=0;u=0;v=0;w=0;x=e;y=b}while(1){if((c[x>>2]|0)==0){z=s;A=t;B=u;C=v;D=x;E=y}else{F=s;G=t;H=u;I=v;J=0;K=c[(c[y+4>>2]|0)+(w<<2)>>2]|0;while(1){L=BP(K)|0;M=BP(c[(c[(c[j>>2]|0)+4>>2]|0)+(J<<2)>>2]|0)|0;N=a[L+7|0]|0;O=a[M+7|0]&N&128|H;P=(d[L+4|0]|0)>1;if(P){Q=(d[M+4|0]|0)>1|0}else{Q=0}M=Q|G;L=c[h>>2]|0;R=c[(c[L+4>>2]|0)+(w<<2)>>2]|0;S=c[j>>2]|0;if((R|0)==(c[(c[S+4>>2]|0)+(J<<2)>>2]|0)){T=N&128|I;U=P&1|F}else{T=I;U=F}P=J+1|0;if(P>>>0<(c[S>>2]|0)>>>0){F=U;G=M;H=O;I=T;J=P;K=R}else{z=U;A=M;B=O;C=T;D=S;E=L;break}}}K=w+1|0;if(K>>>0<(c[E>>2]|0)>>>0){s=z;t=A;u=B;v=C;w=K;x=D;y=E}else{m=C;n=B;o=A;p=z;q=E;r=D;break}}}else{m=0;n=0;o=0;p=0;q=b;r=e}}while(0);if((n|0)>(m|0)|(o|0)>(p|0)){l=0;i=g;return l|0}p=c[q>>2]|0;q=c[r>>2]|0;r=p>>>0>q>>>0?q:p;p=A7(16)|0;q=p;c[k>>2]=q;L6836:do{if((p|0)==0){V=q}else{do{if((r|0)==0){W=0;X=q;Y=5036}else{o=A1(r<<2)|0;c[(c[k>>2]|0)+4>>2]=o;if((o|0)==0){break}o=c[h>>2]|0;L6841:do{if((c[o>>2]|0)==0){Z=0}else{m=0;n=0;e=c[j>>2]|0;b=o;L6843:while(1){D=c[e>>2]|0;if((D|0)==0){_=n;$=e;aa=b}else{E=0;z=n;A=e;B=D;D=b;while(1){C=c[(c[D+4>>2]|0)+(m<<2)>>2]|0;if((C|0)==(c[(c[A+4>>2]|0)+(E<<2)>>2]|0)){y=c[D>>2]|0;if(z>>>0>=(y>>>0>B>>>0?B:y)>>>0){break L6843}c[(c[(c[k>>2]|0)+4>>2]|0)+(z<<2)>>2]=C;ab=z+1|0;ac=c[j>>2]|0}else{ab=z;ac=A}C=E+1|0;y=c[ac>>2]|0;x=c[h>>2]|0;if(C>>>0<y>>>0){E=C;z=ab;A=ac;B=y;D=x}else{_=ab;$=ac;aa=x;break}}}D=m+1|0;if(D>>>0<(c[aa>>2]|0)>>>0){m=D;n=_;e=$;b=aa}else{Z=_;break L6841}}AN(0,16,1729264,(b=i,i=i+1|0,i=i+7&-8,c[b>>2]=0,b)|0);i=b;A4(c[(c[k>>2]|0)+4>>2]|0);A4(c[k>>2]|0);l=0;i=g;return l|0}}while(0);W=Z;X=c[k>>2]|0;Y=5036}}while(0);do{if((Y|0)==5036){c[X>>2]=W;o=c[k>>2]|0;if((c[o>>2]|0)==0){V=o;break L6836}b=A2(c[o+12>>2]|0,(c[(c[h>>2]|0)+8>>2]|0)+(c[o+8>>2]|0)<<2)|0;if((b|0)==0){break}c[(c[k>>2]|0)+12>>2]=b;b=c[h>>2]|0;o=b+12|0;if((c[b+8>>2]|0)==0){ad=o;ae=b}else{b=0;e=o;while(1){o=c[k>>2]|0;c[(c[o+12>>2]|0)+(c[o+8>>2]<<2)>>2]=c[(c[e>>2]|0)+(b<<2)>>2];o=c[k>>2]|0;n=o+8|0;m=c[n>>2]|0;c[n>>2]=m+1;c[c[(c[(c[k>>2]|0)+12>>2]|0)+(m<<2)>>2]>>2]=o;o=b+1|0;m=c[h>>2]|0;n=m+12|0;if(o>>>0<(c[m+8>>2]|0)>>>0){b=o;e=n}else{ad=n;ae=m;break}}}A6(ad);A6(ae+4|0);A6(h);e=c[k>>2]|0;b=A2(c[e+12>>2]|0,(c[(c[j>>2]|0)+8>>2]|0)+(c[e+8>>2]|0)<<2)|0;if((b|0)==0){break}c[(c[k>>2]|0)+12>>2]=b;b=c[j>>2]|0;e=b+12|0;if((c[b+8>>2]|0)==0){af=e;ag=b}else{b=0;m=e;while(1){e=c[k>>2]|0;c[(c[e+12>>2]|0)+(c[e+8>>2]<<2)>>2]=c[(c[m>>2]|0)+(b<<2)>>2];e=c[k>>2]|0;n=e+8|0;o=c[n>>2]|0;c[n>>2]=o+1;c[c[(c[(c[k>>2]|0)+12>>2]|0)+(o<<2)>>2]>>2]=e;e=b+1|0;o=c[j>>2]|0;n=o+12|0;if(e>>>0<(c[o+8>>2]|0)>>>0){b=e;m=n}else{af=n;ag=o;break}}}A6(af);A6(ag+4|0);A6(j);l=c[k>>2]|0;i=g;return l|0}}while(0);V=c[k>>2]|0}}while(0);if((V|0)!=0){A6(V+12|0);A6((c[k>>2]|0)+4|0)}A6(k);l=0;i=g;return l|0}function gS(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0;d=i;i=i+24|0;e=d|0;f=d+8|0;g=d+16|0;c[e>>2]=a;c[f>>2]=b;c[g>>2]=0;if((a|0)==(b|0)){h=a;i=d;return h|0}j=c[a>>2]|0;do{if((j|0)==0){k=A2(c[b+12>>2]|0,(c[a+8>>2]|0)+(c[b+8>>2]|0)<<2)|0;if((k|0)==0){l=5086;break}c[(c[f>>2]|0)+12>>2]=k;k=c[e>>2]|0;m=k+12|0;if((c[k+8>>2]|0)==0){n=m}else{k=0;o=m;while(1){m=c[f>>2]|0;c[(c[m+12>>2]|0)+(c[m+8>>2]<<2)>>2]=c[(c[o>>2]|0)+(k<<2)>>2];m=c[f>>2]|0;p=m+8|0;q=c[p>>2]|0;c[p>>2]=q+1;c[c[(c[(c[f>>2]|0)+12>>2]|0)+(q<<2)>>2]>>2]=m;m=k+1|0;q=c[e>>2]|0;p=q+12|0;if(m>>>0<(c[q+8>>2]|0)>>>0){k=m;o=p}else{n=p;break}}}A6(n);A6((c[e>>2]|0)+4|0);A6(e);o=c[f>>2]|0;c[g>>2]=o;h=o;i=d;return h|0}else{o=c[b>>2]|0;if((o|0)==0){k=A2(c[a+12>>2]|0,(c[b+8>>2]|0)+(c[a+8>>2]|0)<<2)|0;if((k|0)==0){l=5086;break}c[(c[e>>2]|0)+12>>2]=k;k=c[f>>2]|0;p=k+12|0;if((c[k+8>>2]|0)==0){r=p}else{k=0;m=p;while(1){p=c[e>>2]|0;c[(c[p+12>>2]|0)+(c[p+8>>2]<<2)>>2]=c[(c[m>>2]|0)+(k<<2)>>2];p=c[e>>2]|0;q=p+8|0;s=c[q>>2]|0;c[q>>2]=s+1;c[c[(c[(c[e>>2]|0)+12>>2]|0)+(s<<2)>>2]>>2]=p;p=k+1|0;s=c[f>>2]|0;q=s+12|0;if(p>>>0<(c[s+8>>2]|0)>>>0){k=p;m=q}else{r=q;break}}}A6(r);A6((c[f>>2]|0)+4|0);A6(f);m=c[e>>2]|0;c[g>>2]=m;h=m;i=d;return h|0}m=j>>>0>o>>>0?o:j;k=A7(16)|0;q=k;c[g>>2]=q;if((k|0)==0){t=q;break}if((m|0)==0){u=0;v=q}else{q=A1(m<<2)|0;c[(c[g>>2]|0)+4>>2]=q;if((q|0)==0){l=5086;break}q=c[e>>2]|0;L6895:do{if((c[q>>2]|0)==0){w=0}else{m=0;k=0;p=c[f>>2]|0;s=q;L6897:while(1){x=c[p>>2]|0;if((x|0)==0){y=m;z=p;A=s}else{B=m;C=0;D=p;E=x;x=s;while(1){F=c[(c[x+4>>2]|0)+(k<<2)>>2]|0;if((F|0)==(c[(c[D+4>>2]|0)+(C<<2)>>2]|0)){G=c[x>>2]|0;if(B>>>0>=(G>>>0>E>>>0?E:G)>>>0){break L6897}c[(c[(c[g>>2]|0)+4>>2]|0)+(B<<2)>>2]=F;H=B+1|0;I=c[f>>2]|0}else{H=B;I=D}F=C+1|0;G=c[I>>2]|0;J=c[e>>2]|0;if(F>>>0<G>>>0){B=H;C=F;D=I;E=G;x=J}else{y=H;z=I;A=J;break}}}x=k+1|0;if(x>>>0<(c[A>>2]|0)>>>0){m=y;k=x;p=z;s=A}else{w=y;break L6895}}AN(0,16,1729264,(s=i,i=i+1|0,i=i+7&-8,c[s>>2]=0,s)|0);i=s;A4(c[(c[g>>2]|0)+4>>2]|0);A4(c[g>>2]|0);h=0;i=d;return h|0}}while(0);u=w;v=c[g>>2]|0}c[v>>2]=u;q=c[g>>2]|0;if((c[q>>2]|0)==0){t=q;break}o=A2(c[q+12>>2]|0,(c[(c[e>>2]|0)+8>>2]|0)+(c[q+8>>2]|0)<<2)|0;if((o|0)==0){l=5086;break}c[(c[g>>2]|0)+12>>2]=o;o=c[e>>2]|0;q=o+12|0;if((c[o+8>>2]|0)==0){K=q}else{o=0;s=q;while(1){q=c[g>>2]|0;c[(c[q+12>>2]|0)+(c[q+8>>2]<<2)>>2]=c[(c[s>>2]|0)+(o<<2)>>2];q=c[g>>2]|0;p=q+8|0;k=c[p>>2]|0;c[p>>2]=k+1;c[c[(c[(c[g>>2]|0)+12>>2]|0)+(k<<2)>>2]>>2]=q;q=o+1|0;k=c[e>>2]|0;p=k+12|0;if(q>>>0<(c[k+8>>2]|0)>>>0){o=q;s=p}else{K=p;break}}}A6(K);A6((c[e>>2]|0)+4|0);A6(e);s=c[g>>2]|0;o=A2(c[s+12>>2]|0,(c[(c[f>>2]|0)+8>>2]|0)+(c[s+8>>2]|0)<<2)|0;if((o|0)==0){l=5086;break}c[(c[g>>2]|0)+12>>2]=o;o=c[f>>2]|0;s=o+12|0;if((c[o+8>>2]|0)==0){L=s}else{o=0;p=s;while(1){s=c[g>>2]|0;c[(c[s+12>>2]|0)+(c[s+8>>2]<<2)>>2]=c[(c[p>>2]|0)+(o<<2)>>2];s=c[g>>2]|0;q=s+8|0;k=c[q>>2]|0;c[q>>2]=k+1;c[c[(c[(c[g>>2]|0)+12>>2]|0)+(k<<2)>>2]>>2]=s;s=o+1|0;k=c[f>>2]|0;q=k+12|0;if(s>>>0<(c[k+8>>2]|0)>>>0){o=s;p=q}else{L=q;break}}}A6(L);A6((c[f>>2]|0)+4|0);A6(f);h=c[g>>2]|0;i=d;return h|0}}while(0);if((l|0)==5086){t=c[g>>2]|0}if((t|0)!=0){A6(t+12|0);A6((c[g>>2]|0)+4|0)}A6(g);h=0;i=d;return h|0}function gT(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0;e=i;i=i+24|0;f=e|0;g=e+8|0;h=e+16|0;c[f>>2]=b;c[g>>2]=d;c[h>>2]=0;j=(a[b+9|0]|0)+(a[b+8|0]|0)|0;k=(a[d+9|0]|0)+(a[d+8|0]|0)|0;if((d|0)==(b|0)){l=b;i=e;return l|0}if(j>>>0<k>>>0){c[g>>2]=b;c[f>>2]=d;m=k;n=j;o=b;p=d}else{m=j;n=k;o=d;p=b}do{if((m|0)==1){if((n|0)!=0){q=5108;break}b=o+4|0;d=c[b>>2]|0;if((d|0)>0){r=0;s=0;t=d}else{l=0;i=e;return l|0}while(1){d=c[o>>2]|0;k=d+(s<<3)|0;j=c[k>>2]|0;u=c[k+4>>2]|0;k=-1;if((u|0)>(k|0)|(u|0)==(k|0)&j>>>0>-1>>>0|(j&2147483647|0)==0&(u&0|0)==0){k=d+(r<<3)|0;c[k>>2]=j;c[k+4>>2]=u;v=r+1|0;w=c[b>>2]|0}else{v=r;w=t}u=s+1|0;if((u|0)<(w|0)){r=v;s=u;t=w}else{break}}if((v|0)==0){l=0;i=e;return l|0}else{c[b>>2]=v;q=5108;break}}else if((m|0)==0){u=c[p+4>>2]|0;k=c[o+4>>2]|0;j=A7(20)|0;d=j;c[h>>2]=d;if((j|0)==0){x=d;break}d=A1(k+u<<3)|0;c[c[h>>2]>>2]=d;if((d|0)==0){q=5149;break}d=c[f>>2]|0;u=d+4|0;k=c[u>>2]|0;do{if((k|0)>0){j=0;y=0;z=k;while(1){A=d|0;B=(c[A>>2]|0)+(j<<3)|0;C=c[B>>2]|0;D=c[B+4>>2]|0;B=-1;do{if((D|0)>(B|0)|(D|0)==(B|0)&C>>>0>-1>>>0|(C&2147483647|0)==0&(D&0|0)==0){E=c[g>>2]|0;F=E+4|0;G=c[F>>2]|0;if((G|0)>0){H=0;I=y;J=G;K=D;L=C}else{M=y;N=z;break}while(1){G=E|0;O=(c[G>>2]|0)+(H<<3)|0;if((L|0)==(c[O>>2]|0)&(K|0)==(c[O+4>>2]|0)){O=(c[c[h>>2]>>2]|0)+(I<<3)|0;c[O>>2]=L;c[O+4>>2]=K;O=(c[G>>2]|0)+(H<<3)|0;c[O>>2]=0;c[O+4>>2]=0;O=(c[A>>2]|0)+(j<<3)|0;c[O>>2]=0;c[O+4>>2]=0;P=I+1|0;Q=c[F>>2]|0}else{P=I;Q=J}O=H+1|0;if((O|0)>=(Q|0)){break}G=(c[A>>2]|0)+(j<<3)|0;H=O;I=P;J=Q;K=c[G+4>>2]|0;L=c[G>>2]|0}M=P;N=c[u>>2]|0}else{M=y;N=z}}while(0);A=j+1|0;if((A|0)<(N|0)){j=A;y=M;z=N}else{break}}if((N|0)>0){R=0;S=M;T=d}else{U=M;V=d;break}while(1){z=(c[T>>2]|0)+(R<<3)|0;y=c[z>>2]|0;j=c[z+4>>2]|0;do{if((y|0)==0&(j|0)==0){W=S;X=T}else{z=-1;if(!((j|0)>(z|0)|(j|0)==(z|0)&y>>>0>-1>>>0|(y&2147483647|0)==0&(j&0|0)==0)){W=S;X=T;break}z=zs(y,j)|0;A=z|0;C=((z|0)<0|0?-1:0)|-2147483648;z=c[g>>2]|0;D=z+4|0;B=c[D>>2]|0;if((B|0)>0){F=0;E=S;G=B;while(1){B=(c[z>>2]|0)+(F<<3)|0;if((c[B>>2]|0)==(A|0)&(c[B+4>>2]|0)==(C|0)){B=(c[c[f>>2]>>2]|0)+(R<<3)|0;O=c[B+4>>2]|0;Y=(c[c[h>>2]>>2]|0)+(E<<3)|0;c[Y>>2]=c[B>>2];c[Y+4>>2]=O;Z=E+1|0;_=c[D>>2]|0}else{Z=E;_=G}O=F+1|0;if((O|0)<(_|0)){F=O;E=Z;G=_}else{$=Z;break}}}else{$=S}W=$;X=c[f>>2]|0}}while(0);j=R+1|0;if((j|0)<(c[X+4>>2]|0)){R=j;S=W;T=X}else{U=W;V=X;break}}}else{U=0;V=d}}while(0);d=c[g>>2]|0;c[g>>2]=V;c[f>>2]=d;if((c[d+4>>2]|0)>0){u=0;k=U;b=d;j=V;while(1){y=(c[b>>2]|0)+(u<<3)|0;G=c[y>>2]|0;E=c[y+4>>2]|0;do{if((G|0)==0&(E|0)==0){aa=k;ab=j;ac=b}else{y=-1;if(!((E|0)>(y|0)|(E|0)==(y|0)&G>>>0>-1>>>0|(G&2147483647|0)==0&(E&0|0)==0)){aa=k;ab=j;ac=b;break}y=zs(G,E)|0;F=y|0;D=((y|0)<0|0?-1:0)|-2147483648;y=c[g>>2]|0;C=y+4|0;A=c[C>>2]|0;if((A|0)>0){z=0;O=k;Y=A;while(1){A=(c[y>>2]|0)+(z<<3)|0;if((c[A>>2]|0)==(F|0)&(c[A+4>>2]|0)==(D|0)){A=(c[c[f>>2]>>2]|0)+(u<<3)|0;B=c[A+4>>2]|0;ad=(c[c[h>>2]>>2]|0)+(O<<3)|0;c[ad>>2]=c[A>>2];c[ad+4>>2]=B;ae=O+1|0;af=c[C>>2]|0}else{ae=O;af=Y}B=z+1|0;if((B|0)<(af|0)){z=B;O=ae;Y=af}else{ag=ae;break}}}else{ag=k}aa=ag;ab=y;ac=c[f>>2]|0}}while(0);E=u+1|0;if((E|0)<(c[ac+4>>2]|0)){u=E;k=aa;b=ac;j=ab}else{ah=aa;ai=ac;aj=ab;break}}}else{ah=U;ai=d;aj=V}c[g>>2]=ai;c[f>>2]=aj;j=aj+4|0;b=c[j>>2]|0;if((b|0)>0){k=0;u=ah;E=b;while(1){b=aj|0;G=(c[b>>2]|0)+(k<<3)|0;Y=c[G>>2]|0;O=c[G+4>>2]|0;G=-1;do{if((O|0)>(G|0)|(O|0)==(G|0)&Y>>>0>-1>>>0|(Y&2147483647|0)==0&(O&0|0)==0){ak=u;al=E}else{z=ai+4|0;C=c[z>>2]|0;if((C|0)>0){am=0;an=u;ao=C;ap=O;aq=Y}else{ak=u;al=E;break}while(1){C=(c[ai>>2]|0)+(am<<3)|0;if((aq|0)==(c[C>>2]|0)&(ap|0)==(c[C+4>>2]|0)){C=(c[c[h>>2]>>2]|0)+(an<<3)|0;c[C>>2]=aq;c[C+4>>2]=ap;ar=an+1|0;as=c[z>>2]|0}else{ar=an;as=ao}C=am+1|0;if((C|0)>=(as|0)){break}D=(c[b>>2]|0)+(k<<3)|0;am=C;an=ar;ao=as;ap=c[D+4>>2]|0;aq=c[D>>2]|0}ak=ar;al=c[j>>2]|0}}while(0);b=k+1|0;if((b|0)<(al|0)){k=b;u=ak;E=al}else{at=ak;break}}}else{at=ah}c[(c[h>>2]|0)+4>>2]=at;E=c[h>>2]|0;if((c[E+4>>2]|0)==0){x=E;break}u=A2(c[E+16>>2]|0,(c[(c[f>>2]|0)+12>>2]|0)+(c[E+12>>2]|0)<<2)|0;if((u|0)==0){q=5149;break}c[(c[h>>2]|0)+16>>2]=u;u=c[f>>2]|0;E=u+16|0;if((c[u+12>>2]|0)==0){au=E}else{u=0;k=E;while(1){E=c[h>>2]|0;c[(c[E+16>>2]|0)+(c[E+12>>2]<<2)>>2]=c[(c[k>>2]|0)+(u<<2)>>2];E=c[h>>2]|0;j=E+12|0;d=c[j>>2]|0;c[j>>2]=d+1;c[c[(c[(c[h>>2]|0)+16>>2]|0)+(d<<2)>>2]>>2]=E;E=u+1|0;d=c[f>>2]|0;j=d+16|0;if(E>>>0<(c[d+12>>2]|0)>>>0){u=E;k=j}else{au=j;break}}}A6(au);A6(c[f>>2]|0);A6(f);k=c[h>>2]|0;u=A2(c[k+16>>2]|0,(c[(c[g>>2]|0)+12>>2]|0)+(c[k+12>>2]|0)<<2)|0;if((u|0)==0){q=5149;break}c[(c[h>>2]|0)+16>>2]=u;u=c[g>>2]|0;k=u+16|0;if((c[u+12>>2]|0)==0){av=k}else{u=0;j=k;while(1){k=c[h>>2]|0;c[(c[k+16>>2]|0)+(c[k+12>>2]<<2)>>2]=c[(c[j>>2]|0)+(u<<2)>>2];k=c[h>>2]|0;E=k+12|0;d=c[E>>2]|0;c[E>>2]=d+1;c[c[(c[(c[h>>2]|0)+16>>2]|0)+(d<<2)>>2]>>2]=k;k=u+1|0;d=c[g>>2]|0;E=d+16|0;if(k>>>0<(c[d+12>>2]|0)>>>0){u=k;j=E}else{av=E;break}}}A6(av);A6(c[g>>2]|0);A6(g);l=c[h>>2]|0;i=e;return l|0}else{q=5108}}while(0);do{if((q|0)==5108){av=A2(c[o+16>>2]|0,(c[p+12>>2]|0)+(c[o+12>>2]|0)<<2)|0;if((av|0)==0){q=5149;break}c[(c[g>>2]|0)+16>>2]=av;av=c[f>>2]|0;au=av+16|0;if((c[av+12>>2]|0)==0){aw=au}else{av=0;at=au;while(1){au=c[g>>2]|0;c[(c[au+16>>2]|0)+(c[au+12>>2]<<2)>>2]=c[(c[at>>2]|0)+(av<<2)>>2];au=c[g>>2]|0;ah=au+12|0;ak=c[ah>>2]|0;c[ah>>2]=ak+1;c[c[(c[(c[g>>2]|0)+16>>2]|0)+(ak<<2)>>2]>>2]=au;au=av+1|0;ak=c[f>>2]|0;ah=ak+16|0;if(au>>>0<(c[ak+12>>2]|0)>>>0){av=au;at=ah}else{aw=ah;break}}}A6(aw);A6(c[f>>2]|0);A6(f);l=c[g>>2]|0;i=e;return l|0}}while(0);if((q|0)==5149){x=c[h>>2]|0}if((x|0)!=0){A6(x+16|0);A6(c[h>>2]|0)}A6(h);l=0;i=e;return l|0}function gU(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;if((a|0)==0){b=0}else{d=0;while(1){if((c[a+(d<<2)>>2]|0)==-1){b=d;break}else{d=d+1|0}}}d=A7(16)|0;e=d;if((d|0)==0){f=0;return f|0}c[d>>2]=b;if((b|0)==0){f=e;return f|0}g=A1(b<<2)|0;c[d+4>>2]=g;if((g|0)==0){A4(d);f=0;return f|0}if((b|0)==0){f=e;return f|0}g=d+4|0;d=b;while(1){b=d-1|0;c[(c[g>>2]|0)+(b<<2)>>2]=c[a+(b<<2)>>2];if((b|0)==0){f=e;break}else{d=b}}return f|0}function gV(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;if((a|0)==0){b=0}else{d=0;while(1){e=a+(d<<3)|0;if((c[e>>2]|0)==(-1|0)&(c[e+4>>2]|0)==(-1|0)){b=d;break}else{d=d+1|0}}}d=A7(20)|0;e=d;if((d|0)==0){f=0;return f|0}c[d+4>>2]=b;if((b|0)==0){f=e;return f|0}g=b<<3;b=A1(g)|0;c[d>>2]=b;if((b|0)==0){A4(d);f=0;return f|0}else{d=a;NN(b|0,d|0,g)|0;f=e;return f|0}return 0}function gW(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;d=c[a>>2]|0;do{if((d|0)==0){e=A7(16)|0;f=e;c[a>>2]=f;if((e|0)==0){g=-12}else{h=f;break}return g|0}else{h=d}}while(0);d=A2(c[h+4>>2]|0,(c[h>>2]<<2)+4|0)|0;if((d|0)==0){g=-12;return g|0}c[(c[a>>2]|0)+4>>2]=d;d=c[a>>2]|0;h=c[d>>2]|0;c[d>>2]=h+1;c[(c[(c[a>>2]|0)+4>>2]|0)+(h<<2)>>2]=b;g=0;return g|0}function gX(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;e=c[a>>2]|0;do{if((e|0)==0){f=A7(20)|0;g=f;c[a>>2]=g;if((f|0)==0){h=-12}else{i=g;break}return h|0}else{i=e}}while(0);e=A2(c[i>>2]|0,(c[i+4>>2]<<3)+8|0)|0;if((e|0)==0){h=-12;return h|0}c[c[a>>2]>>2]=e;e=(c[a>>2]|0)+4|0;i=c[e>>2]|0;c[e>>2]=i+1;e=(c[c[a>>2]>>2]|0)+(i<<3)|0;c[e>>2]=b;c[e+4>>2]=d;h=0;return h|0}function gY(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;if((b|0)==0){d=0;e=0;while(1){do{if((a[(BP(e)|0)+7|0]&8)==0){if((d|0)==0){f=A7(16)|0;g=f;if((f|0)==0){h=g;break}else{i=g}}else{i=d}g=i+4|0;f=i|0;j=A2(c[g>>2]|0,(c[f>>2]<<2)+4|0)|0;if((j|0)==0){h=i;break}k=j;c[g>>2]=k;g=c[f>>2]|0;c[f>>2]=g+1;c[k+(g<<2)>>2]=e;h=i}else{h=d}}while(0);g=e+1|0;if((g|0)<321){d=h;e=g}else{l=h;break}}return l|0}h=(b|0)==1;b=h?10:0;if(h){m=0;n=0}else{l=0;return l|0}while(1){BP(n)|0;if((m|0)==0){h=A7(16)|0;e=h;if((h|0)==0){o=e}else{p=e;q=5222}}else{p=m;q=5222}do{if((q|0)==5222){q=0;e=p+4|0;h=p|0;d=A2(c[e>>2]|0,(c[h>>2]<<2)+4|0)|0;if((d|0)==0){o=p;break}i=d;c[e>>2]=i;e=c[h>>2]|0;c[h>>2]=e+1;c[i+(e<<2)>>2]=n;o=p}}while(0);e=n+1|0;if((e|0)<(b|0)){m=o;n=e}else{l=o;break}}return l|0}function gZ(){return A7(16)|0}function g_(){var b=0,c=0;b=A7(20)|0;if((b|0)==0){c=0;return c|0}a[b+8|0]=1;c=b;return c|0}function g$(){var b=0,c=0;b=A7(20)|0;if((b|0)==0){c=0;return c|0}a[b+9|0]=1;a[b+8|0]=1;c=b;return c|0}function g0(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;c[b>>2]=a;d=a+16|0;e=c[d>>2]|0;f=a+12|0;a=(c[f>>2]|0)+1|0;c[f>>2]=a;g=A2(e,a<<2)|0;c[d>>2]=g;c[g+((c[f>>2]|0)-1<<2)>>2]=b;return}function g1(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;c[b>>2]=a;d=a+12|0;e=c[d>>2]|0;f=a+8|0;a=(c[f>>2]|0)+1|0;c[f>>2]=a;g=A2(e,a<<2)|0;c[d>>2]=g;c[g+((c[f>>2]|0)-1<<2)>>2]=b;return}function g2(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;b=c[a>>2]|0;if((b|0)==0){return}d=c[b+8>>2]|0;e=b+12|0;f=0;while(1){if(f>>>0>=d>>>0){g=b;h=d;break}i=c[e>>2]|0;j=i+(f<<2)|0;k=f+1|0;if((c[j>>2]|0)==(a|0)){l=5246;break}else{f=k}}do{if((l|0)==5246){if((f|0)<=-1){g=b;h=d;break}NP(j|0,i+(k<<2)|0,(d-f<<2)-4|0);e=c[a>>2]|0;g=e;h=c[e+8>>2]|0}}while(0);f=h-1|0;c[g+8>>2]=f;if((f|0)==0){A4(c[(c[a>>2]|0)+4>>2]|0);A4(c[(c[a>>2]|0)+12>>2]|0);A4(c[a>>2]|0)}c[a>>2]=0;return}function g3(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;b=c[a>>2]|0;if((b|0)==0){return}d=c[b+12>>2]|0;e=b+16|0;f=0;while(1){if(f>>>0>=d>>>0){g=b;h=d;break}i=c[e>>2]|0;j=i+(f<<2)|0;k=f+1|0;if((c[j>>2]|0)==(a|0)){l=5258;break}else{f=k}}do{if((l|0)==5258){if((f|0)<=-1){g=b;h=d;break}NP(j|0,i+(k<<2)|0,(d-f<<2)-4|0);e=c[a>>2]|0;g=e;h=c[e+12>>2]|0}}while(0);f=h-1|0;c[g+12>>2]=f;if((f|0)==0){A4(c[c[a>>2]>>2]|0);A4(c[(c[a>>2]|0)+16>>2]|0);A4(c[a>>2]|0)}c[a>>2]=0;return}function g4(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=c[a>>2]|0;e=c[d+12>>2]|0;f=d+16|0;d=0;while(1){if(d>>>0>=e>>>0){g=5273;break}h=(c[f>>2]|0)+(d<<2)|0;if((c[h>>2]|0)==(a|0)){break}else{d=d+1|0}}if((g|0)==5273){return}if((d|0)<=-1){return}c[h>>2]=b;c[b>>2]=c[a>>2];c[a>>2]=0;return}function g5(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=c[a>>2]|0;e=c[d+8>>2]|0;f=d+12|0;d=0;while(1){if(d>>>0>=e>>>0){g=5282;break}h=(c[f>>2]|0)+(d<<2)|0;if((c[h>>2]|0)==(a|0)){break}else{d=d+1|0}}if((g|0)==5282){return}if((d|0)<=-1){return}c[h>>2]=b;c[b>>2]=c[a>>2];c[a>>2]=0;return}function g6(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;d=i;i=i+8|0;e=d|0;c[e>>2]=b;b=a+24|0;f=c[b>>2]|0;if((f|0)==0){g=0}else{h=a+16|0;j=0;k=0;l=f;while(1){f=c[(c[h>>2]|0)+(k<<2)>>2]|0;do{if((f|0)==0){m=j;n=l}else{o=f+84|0;if((c[o>>2]|0)!=0){m=j;n=l;break}p=c[e>>2]|0;c[o>>2]=p;q=p+16|0;r=c[q>>2]|0;s=p+12|0;p=(c[s>>2]|0)+1|0;c[s>>2]=p;t=A2(r,p<<2)|0;c[q>>2]=t;c[t+((c[s>>2]|0)-1<<2)>>2]=o;m=j+1|0;n=c[b>>2]|0}}while(0);f=k+1|0;if(f>>>0<n>>>0){j=m;k=f;l=n}else{g=m;break}}}m=a+40|0;n=c[m>>2]|0;if((n|0)==0){u=g}else{l=a+32|0;a=g;g=0;k=n;while(1){n=c[(c[l>>2]|0)+(g<<2)>>2]|0;do{if((n|0)==0){v=a;w=k}else{j=n+80|0;if((c[j>>2]|0)!=0){v=a;w=k;break}b=c[e>>2]|0;c[j>>2]=b;h=b+16|0;f=c[h>>2]|0;o=b+12|0;b=(c[o>>2]|0)+1|0;c[o>>2]=b;s=A2(f,b<<2)|0;c[h>>2]=s;c[s+((c[o>>2]|0)-1<<2)>>2]=j;v=a+1|0;w=c[m>>2]|0}}while(0);n=g+1|0;if(n>>>0<w>>>0){a=v;g=n;k=w}else{u=v;break}}}if((u|0)!=0){i=d;return}u=c[e>>2]|0;A6(u);A6(u+16|0);A6(e);i=d;return}function g7(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;d=i;i=i+8|0;e=d|0;c[e>>2]=b;b=a+24|0;f=c[b>>2]|0;if((f|0)==0){g=0}else{h=a+16|0;j=0;k=0;l=f;while(1){f=c[(c[h>>2]|0)+(k<<2)>>2]|0;do{if((f|0)==0){m=j;n=l}else{o=f+76|0;if((c[o>>2]|0)!=0){m=j;n=l;break}p=c[e>>2]|0;c[o>>2]=p;q=p+12|0;r=c[q>>2]|0;s=p+8|0;p=(c[s>>2]|0)+1|0;c[s>>2]=p;t=A2(r,p<<2)|0;c[q>>2]=t;c[t+((c[s>>2]|0)-1<<2)>>2]=o;m=j+1|0;n=c[b>>2]|0}}while(0);f=k+1|0;if(f>>>0<n>>>0){j=m;k=f;l=n}else{g=m;break}}}m=a+40|0;n=c[m>>2]|0;if((n|0)==0){u=g}else{l=a+32|0;a=g;g=0;k=n;while(1){n=c[(c[l>>2]|0)+(g<<2)>>2]|0;do{if((n|0)==0){v=a;w=k}else{j=n+72|0;if((c[j>>2]|0)!=0){v=a;w=k;break}b=c[e>>2]|0;c[j>>2]=b;h=b+12|0;f=c[h>>2]|0;o=b+8|0;b=(c[o>>2]|0)+1|0;c[o>>2]=b;s=A2(f,b<<2)|0;c[h>>2]=s;c[s+((c[o>>2]|0)-1<<2)>>2]=j;v=a+1|0;w=c[m>>2]|0}}while(0);n=g+1|0;if(n>>>0<w>>>0){a=v;g=n;k=w}else{u=v;break}}}if((u|0)!=0){i=d;return}u=c[e>>2]|0;A6(u+4|0);A6(u+12|0);A6(e);i=d;return}function g8(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;d=i;i=i+8|0;e=d|0;c[e>>2]=b;b=a+24|0;f=c[b>>2]|0;if((f|0)==0){g=0}else{h=a+16|0;j=0;k=0;l=f;while(1){f=c[(c[h>>2]|0)+(k<<2)>>2]|0;do{if((f|0)==0){m=j;n=l}else{o=f+68|0;if((c[o>>2]|0)!=0){m=j;n=l;break}p=c[e>>2]|0;c[o>>2]=p;q=p+12|0;r=c[q>>2]|0;s=p+8|0;p=(c[s>>2]|0)+1|0;c[s>>2]=p;t=A2(r,p<<2)|0;c[q>>2]=t;c[t+((c[s>>2]|0)-1<<2)>>2]=o;m=j+1|0;n=c[b>>2]|0}}while(0);f=k+1|0;if(f>>>0<n>>>0){j=m;k=f;l=n}else{g=m;break}}}m=a+40|0;n=c[m>>2]|0;if((n|0)==0){u=g}else{l=a+32|0;a=g;g=0;k=n;while(1){n=c[(c[l>>2]|0)+(g<<2)>>2]|0;do{if((n|0)==0){v=a;w=k}else{j=n+64|0;if((c[j>>2]|0)!=0){v=a;w=k;break}b=c[e>>2]|0;c[j>>2]=b;h=b+12|0;f=c[h>>2]|0;o=b+8|0;b=(c[o>>2]|0)+1|0;c[o>>2]=b;s=A2(f,b<<2)|0;c[h>>2]=s;c[s+((c[o>>2]|0)-1<<2)>>2]=j;v=a+1|0;w=c[m>>2]|0}}while(0);n=g+1|0;if(n>>>0<w>>>0){a=v;g=n;k=w}else{u=v;break}}}if((u|0)!=0){i=d;return}u=c[e>>2]|0;A6(u+4|0);A6(u+12|0);A6(e);i=d;return}function g9(b){b=b|0;var d=0,e=0,f=0,g=0,h=0;d=c[b+16>>2]|0;do{if((d|0)==0){e=5338}else{f=c[d>>2]|0;if((f|0)==0){e=5338;break}g=c[f+16>>2]|0}}while(0);do{if((e|0)==5338){d=c[b+32>>2]|0;if((d|0)==0){g=0;break}f=c[d>>2]|0;if((f|0)==0){g=0;break}g=c[f+16>>2]|0}}while(0);g8(b,gY(g)|0);if((g|0)!=1){return 0}g=A7(20)|0;if((g|0)==0){h=0}else{a[g+8|0]=1;h=g}g6(b,h);g7(b,A7(16)|0);return 0}function ha(b){b=b|0;var d=0,e=0,f=0,g=0,h=0;d=c[b+16>>2]|0;do{if((d|0)==0){e=5351}else{f=c[d>>2]|0;if((f|0)==0){e=5351;break}g=c[f+16>>2]|0}}while(0);do{if((e|0)==5351){d=c[b+32>>2]|0;if((d|0)==0){g=0;break}f=c[d>>2]|0;if((f|0)==0){g=0;break}g=c[f+16>>2]|0}}while(0);g8(b,gY(g)|0);if((g|0)!=1){return 0}g=A7(20)|0;if((g|0)==0){h=0}else{a[g+9|0]=1;a[g+8|0]=1;h=g}g6(b,h);g7(b,A7(16)|0);return 0}function hb(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0;f=i;i=i+8|0;g=f|0;h=BM(d)|0;do{if((h|0)==-1){j=aU(d|0,g|0,0)|0;if(!((a[c[g>>2]|0]|0)!=0|j>>>0>320)){k=j;break}AN(e,16,1736992,(j=i,i=i+8|0,c[j>>2]=d,j)|0);i=j;l=-22;i=f;return l|0}else{k=h}}while(0);c[b>>2]=k;l=0;i=f;return l|0}function hc(a){a=a|0;var b=0,d=0,e=0;b=c[a>>2]|0;if((b|0)==0){return}d=a;e=b;while(1){b=c[e+12>>2]|0;A6(e);A6(d);c[a>>2]=b;if((b|0)==0){break}else{e=b}}return}function hd(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;g=i;i=i+40|0;h=g|0;j=g+8|0;k=g+16|0;l=g+24|0;m=g+32|0;c[h>>2]=d;c[j>>2]=0;c[k>>2]=0;c[l>>2]=0;c[h>>2]=d+(cm(d|0,1727328)|0);d=he(h,b)|0;L7305:do{if((d|0)<0){n=d}else{o=c[h>>2]|0;c[h>>2]=o+(cm(o|0,1727328)|0);o=b;p=hf(h,j,l,o)|0;if((p|0)<0){n=p;break}else{q=0}while(1){p=hg(m,h,b,q,o)|0;if((p|0)<0){n=p;break L7305}p=hh(c[m>>2]|0,j,k,o)|0;if((p|0)<0){n=p;break L7305}p=hi(h,j,k,l,o)|0;if((p|0)<0){n=p;break L7305}p=c[h>>2]|0;r=cm(p|0,1727328)|0;c[h>>2]=p+(r+1);s=a[p+r|0]|0;do{if(s<<24>>24==59){r=c[j>>2]|0;if((r|0)==0){t=q+1|0;break}p=l;u=c[l>>2]|0;while(1){if((u|0)==0){v=5385;break}w=u+12|0;x=c[w>>2]|0;if((x|0)==0){v=5386;break}else{p=w;u=x}}if((v|0)==5386){v=0;c[w>>2]=r}else if((v|0)==5385){v=0;c[p>>2]=r}c[j>>2]=0;v=5388}else{v=5388}}while(0);if((v|0)==5388){v=0;if((s<<24>>24|0)==59|(s<<24>>24|0)==44){t=q+1|0}else if((s<<24>>24|0)==0){break}else{v=5390;break}}u=c[h>>2]|0;c[h>>2]=u+(cm(u|0,1727328)|0);u=hf(h,j,l,o)|0;if((u|0)<0){n=u;break L7305}else{q=t}}if((v|0)==5390){AN(o,16,1736872,(u=i,i=i+8|0,c[u>>2]=(c[h>>2]|0)-1,u)|0);i=u;n=-22;break}u=l;x=c[l>>2]|0;while(1){if((x|0)==0){v=5395;break}y=x+12|0;z=c[y>>2]|0;if((z|0)==0){v=5396;break}else{u=y;x=z}}if((v|0)==5395){c[u>>2]=c[j>>2]}else if((v|0)==5396){c[y>>2]=c[j>>2]}c[j>>2]=0;c[e>>2]=c[k>>2];A=0;B=c[l>>2]|0;c[f>>2]=B;i=g;return A|0}}while(0);y=b+20|0;v=b+8|0;if((c[y>>2]|0)!=0){do{f0(c[c[v>>2]>>2]|0);}while((c[y>>2]|0)!=0)}A6(v);v=c[k>>2]|0;if((v|0)!=0){y=k;b=v;while(1){v=c[b+12>>2]|0;A6(b);A6(y);c[k>>2]=v;if((v|0)==0){break}else{b=v}}}b=c[l>>2]|0;if((b|0)!=0){k=l;y=b;while(1){b=c[y+12>>2]|0;A6(y);A6(k);c[l>>2]=b;if((b|0)==0){break}else{y=b}}}y=c[j>>2]|0;if((y|0)!=0){l=j;k=y;while(1){y=c[k+12>>2]|0;A6(k);A6(l);c[j>>2]=y;if((y|0)==0){break}else{k=y}}}c[e>>2]=0;A=n;B=0;c[f>>2]=B;i=g;return A|0}function he(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=i;e=c[a>>2]|0;f=bi(e|0,59)|0;if((ba(e|0,1700272,10)|0)!=0){g=0;i=d;return g|0}if((f|0)==0){AN(b,16,1697528,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;g=-22;i=d;return g|0}c[a>>2]=e+4;e=b+12|0;A6(e);b=f+1|0;h=A7(b-(c[a>>2]|0)|0)|0;c[e>>2]=h;if((h|0)==0){g=-12;i=d;return g|0}e=c[a>>2]|0;y_(h,e,b-e|0)|0;c[a>>2]=f+1;g=0;i=d;return g|0}function hf(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;g=i;i=i+16|0;h=g|0;j=g+8|0;c[j>>2]=0;k=c[b>>2]|0;do{if((a[k]|0)==91){l=h;m=0;n=k;while(1){c[b>>2]=n+1;o=y2(b,1626024)|0;c[h>>2]=o;do{if((a[o]|0)==0){AN(f,16,1619528,(p=i,i=i+8|0,c[p>>2]=n,p)|0);i=p;q=5428}else{r=c[b>>2]|0;c[b>>2]=r+1;if((a[r]|0)==93){s=o;break}AN(f,16,1615400,(p=i,i=i+8|0,c[p>>2]=n,p)|0);i=p;q=5428}}while(0);if((q|0)==5428){q=0;A6(l);s=c[h>>2]|0}if((s|0)==0){t=-22;q=5453;break}o=c[e>>2]|0;L7384:do{if((o|0)==0){q=5436}else{r=e;u=o;while(1){v=c[u>>2]|0;if((v|0)!=0){if((a8(v|0,s|0)|0)==0){break}}v=u+12|0;w=c[v>>2]|0;if((w|0)==0){q=5436;break L7384}else{r=v;u=w}}if((u|0)==0){q=5436;break}w=u+12|0;c[r>>2]=c[w>>2];c[w>>2]=0;A4(s);x=u}}while(0);if((q|0)==5436){q=0;o=A7(16)|0;if((o|0)==0){q=5437;break}c[o>>2]=s;c[o+8>>2]=m;x=o}o=j;w=c[j>>2]|0;while(1){if((w|0)==0){q=5442;break}y=w+12|0;v=c[y>>2]|0;if((v|0)==0){q=5443;break}else{o=y;w=v}}if((q|0)==5442){q=0;c[o>>2]=x}else if((q|0)==5443){q=0;c[y>>2]=x}w=c[b>>2]|0;v=w+(cm(w|0,1727328)|0)|0;c[b>>2]=v;z=m+1|0;if((a[v]|0)==91){m=z;n=v}else{q=5445;break}}if((q|0)==5445){A=z;B=c[j>>2]|0;break}else if((q|0)==5453){i=g;return t|0}else if((q|0)==5437){A4(s);t=-12;i=g;return t|0}}else{A=0;B=0}}while(0);s=j;z=B;while(1){if((z|0)==0){q=5449;break}C=z+12|0;B=c[C>>2]|0;if((B|0)==0){q=5450;break}else{s=C;z=B}}if((q|0)==5449){c[s>>2]=c[d>>2]}else if((q|0)==5450){c[C>>2]=c[d>>2]}c[d>>2]=c[j>>2];t=A;i=g;return t|0}function hg(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;h=i;i=i+288|0;j=y2(d,1737912)|0;k=c[d>>2]|0;if((a[k]|0)==61){c[d>>2]=k+1;l=y2(d,1733392)|0}else{l=0}d=h|0;k=h+32|0;bc(d|0,30,1728296,(m=i,i=i+16|0,c[m>>2]=j,c[m+8>>2]=f,m)|0)|0;i=m;f=fW(j)|0;if((f|0)==0){AN(g,16,1724040,(m=i,i=i+8|0,c[m>>2]=j,m)|0);i=m;n=-22;A4(j);A4(l);i=h;return n|0}o=gh(e,f,d)|0;c[b>>2]=o;if((o|0)==0){AN(g,16,1720416,(m=i,i=i+8|0,c[m>>2]=j,m)|0);i=m;n=-12;A4(j);A4(l);i=h;return n|0}do{if((a8(j|0,1716424)|0)!=0|(l|0)==0){p=l;q=o}else{if((a4(l|0,1712744)|0)!=0){p=l;q=o;break}d=c[e+12>>2]|0;if((d|0)==0){p=l;q=o;break}bc(k|0,256,1709072,(m=i,i=i+16|0,c[m>>2]=l,c[m+8>>2]=d,m)|0)|0;i=m;p=k;q=c[b>>2]|0}}while(0);b=f2(q,p)|0;if((b|0)>=0){n=0;A4(j);A4(l);i=h;return n|0}AN(g,16,1706976,(m=i,i=i+8|0,c[m>>2]=j,m)|0);i=m;if((p|0)!=0){AN(g,16,1704952,(m=i,i=i+8|0,c[m>>2]=p,m)|0);i=m}AN(g,16,1702864,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=b;A4(j);A4(l);i=h;return n|0}function hh(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;f=i;g=a+24|0;h=c[b>>2]|0;j=(h|0)==0;L7442:do{if((c[g>>2]|0)==0){k=j}else{l=a+4|0;m=0;n=h;o=j;L7444:while(1){if(o){p=A7(16)|0;if((p|0)==0){q=-12;r=5498;break}else{s=p}}else{p=n+12|0;c[b>>2]=c[p>>2];c[p>>2]=0;s=n}p=s+4|0;t=c[p>>2]|0;do{if((t|0)==0){c[p>>2]=a;c[s+8>>2]=m;u=d;v=c[d>>2]|0;while(1){if((v|0)==0){r=5487;break}w=v+12|0;x=c[w>>2]|0;if((x|0)==0){r=5488;break}else{u=w;v=x}}if((r|0)==5488){r=0;c[w>>2]=s;break}else if((r|0)==5487){r=0;c[u>>2]=s;break}}else{v=c[s+8>>2]|0;x=fN(t,v,a,m)|0;if((x|0)==0){y=0}else{z=c[c[l>>2]>>2]|0;AN(e,16,1632648,(A=i,i=i+32|0,c[A>>2]=c[c[t+4>>2]>>2],c[A+8>>2]=v,c[A+16>>2]=z,c[A+24>>2]=m,A)|0);i=A;y=x}A4(c[s>>2]|0);A4(s);if((y|0)<0){q=y;r=5496;break L7444}}}while(0);t=m+1|0;p=c[b>>2]|0;x=(p|0)==0;if(t>>>0<(c[g>>2]|0)>>>0){m=t;n=p;o=x}else{k=x;break L7442}}if((r|0)==5498){i=f;return q|0}else if((r|0)==5496){i=f;return q|0}}}while(0);if(!k){AN(e,16,1744496,(A=i,i=i+8|0,c[A>>2]=c[c[a+4>>2]>>2],A)|0);i=A;q=-22;i=f;return q|0}A=c[a+40>>2]|0;if((A|0)==0){q=0;i=f;return q|0}else{B=A}while(1){A=B-1|0;e=A7(16)|0;if((e|0)==0){q=-12;r=5499;break}c[e+4>>2]=a;c[e+8>>2]=A;c[e+12>>2]=c[b>>2];c[b>>2]=e;if((A|0)==0){q=0;r=5501;break}else{B=A}}if((r|0)==5499){i=f;return q|0}else if((r|0)==5501){i=f;return q|0}return 0}function hi(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;h=i;i=i+8|0;j=h|0;k=c[b>>2]|0;if((a[k]|0)!=91){l=0;i=h;return l|0}m=j;n=0;o=k;L7483:while(1){c[b>>2]=o+1;k=y2(b,1626024)|0;c[j>>2]=k;do{if((a[k]|0)==0){AN(g,16,1619528,(p=i,i=i+8|0,c[p>>2]=o,p)|0);i=p;q=5508}else{r=c[b>>2]|0;c[b>>2]=r+1;if((a[r]|0)==93){s=k;break}AN(g,16,1615400,(p=i,i=i+8|0,c[p>>2]=o,p)|0);i=p;q=5508}}while(0);if((q|0)==5508){q=0;A6(m);s=c[j>>2]|0}k=c[d>>2]|0;if((s|0)==0){l=-22;q=5525;break}if((k|0)==0){q=5511;break}r=k+12|0;c[d>>2]=c[r>>2];t=c[e>>2]|0;L7494:do{if((t|0)==0){q=5521}else{u=e;v=t;while(1){w=v|0;x=c[w>>2]|0;if((x|0)!=0){if((a8(x|0,s|0)|0)==0){break}}x=v+12|0;y=c[x>>2]|0;if((y|0)==0){q=5521;break L7494}else{u=x;v=y}}if((v|0)==0){q=5521;break}y=v+12|0;c[u>>2]=c[y>>2];c[y>>2]=0;y=c[k+4>>2]|0;x=c[k+8>>2]|0;z=c[v+4>>2]|0;A=c[v+8>>2]|0;B=fN(y,x,z,A)|0;if((B|0)!=0){C=c[c[z+4>>2]>>2]|0;AN(g,16,1632648,(p=i,i=i+32|0,c[p>>2]=c[c[y+4>>2]>>2],c[p+8>>2]=x,c[p+16>>2]=C,c[p+24>>2]=A,p)|0);i=p;if((B|0)<0){q=5519;break L7483}}A4(c[w>>2]|0);A4(s);A4(v);A4(k)}}while(0);if((q|0)==5521){q=0;c[k>>2]=s;c[r>>2]=c[f>>2];c[f>>2]=k}t=c[b>>2]|0;A=t+(cm(t|0,1727328)|0)|0;c[b>>2]=A;t=n+1|0;if((a[A]|0)==91){n=t;o=A}else{l=t;q=5528;break}}if((q|0)==5511){AN(g,16,1645608,(p=i,i=i+8|0,c[p>>2]=s,p)|0);i=p;A4(s);l=-22;i=h;return l|0}else if((q|0)==5525){i=h;return l|0}else if((q|0)==5519){A4(s);l=B;i=h;return l|0}else if((q|0)==5528){i=h;return l|0}return 0}function hj(){return}function hk(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=i;d=c[a+44>>2]|0;e=d+4|0;f=a;a=zQ(d+8|0,c[e>>2]|0,38520,0,0,0,0,0,f)|0;if((a|0)<0){AN(f,16,1702272,(f=i,i=i+8|0,c[f>>2]=c[e>>2],f)|0);i=f;g=a;i=b;return g|0}else{h[d+32>>3]=0.0;h[d+160>>3]=0.0;h[d+64>>3]=r;h[d+72>>3]=r;h[d+80>>3]=r;h[d+88>>3]=r;h[d+112>>3]=r;h[d+120>>3]=r;g=0;i=b;return g|0}return 0}function hl(a){a=a|0;var b=0;b=(c[a+44>>2]|0)+8|0;zP(c[b>>2]|0);c[b>>2]=0;return}function hm(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,j=0,l=0,m=0,n=0,o=0,p=0,q=0,s=0,t=0,u=0,v=0.0,w=0.0,x=0,y=0,z=0.0,A=0.0,B=0.0,C=0,D=0,E=0,F=0,H=0,I=0.0,J=0.0,K=0.0,L=0.0,M=0.0,N=0,O=0,P=0,S=0,T=0;e=i;i=i+192|0;f=e|0;g=e+64|0;j=e+128|0;l=b+8|0;m=c[(c[l>>2]|0)+44>>2]|0;n=d+136|0;o=c[n>>2]|0;p=c[n+4>>2]|0;q=m+16|0;s=m+112|0;t=c[s+4>>2]&2147483647;u=2146435072;if(t>>>0>u>>>0|t>>>0==u>>>0&(c[s>>2]|0)>>>0>0>>>0){if((o|0)==0&(p|0)==(-2147483648|0)){v=r}else{v=+(o>>>0)+ +(p|0)*4294967296.0}h[s>>3]=v;s=c[n>>2]|0;u=c[n+4>>2]|0;if((s|0)==0&(u|0)==(-2147483648|0)){w=r}else{w=(+(s>>>0)+ +(u|0)*4294967296.0)*(+(c[b+56>>2]|0)/+(c[b+60>>2]|0))}h[m+120>>3]=w;x=c[n+4>>2]|0;y=c[n>>2]|0}else{x=p;y=o}if((y|0)==0&(x|0)==(-2147483648|0)){z=r}else{z=+(y>>>0)+ +(x|0)*4294967296.0}x=m+96|0;h[x>>3]=z;y=c[n>>2]|0;u=c[n+4>>2]|0;if((y|0)==0&(u|0)==(-2147483648|0)){A=r}else{A=(+(y>>>0)+ +(u|0)*4294967296.0)*(+(c[b+56>>2]|0)/+(c[b+60>>2]|0))}u=m+128|0;h[u>>3]=A;y=Ak(d)|0;if((y|0)==(-1|0)&(G|0)==(-1|0)){B=r}else{y=Ak(d)|0;B=+(y>>>0)+ +(G|0)*4294967296.0}y=m+56|0;h[y>>3]=B;s=Ch()|0;h[m+144>>3]=+(s>>>0)+ +(G|0)*4294967296.0;s=b+16|0;t=c[s>>2]|0;if((t|0)==1){C=d+76|0;h[m+160>>3]=+(c[C>>2]|0);h[m+48>>3]=+(c[C>>2]|0)}else if((t|0)==0){h[m+24>>3]=+(c[d+296>>2]|0)}B=+zT(c[m+8>>2]|0,q,0);h[k>>3]=B;q=c[k+4>>2]&2147483647;t=2146435072;C=q>>>0>t>>>0|q>>>0==t>>>0&(c[k>>2]|0)>>>0>0>>>0;if(C){D=-2147483648;E=0}else{D=+R(B)>=1.0?B>0.0?(ah(+Q(B/4294967296.0),4294967295.0)|0)>>>0:~~+ab((B- +(~~B>>>0))/4294967296.0)>>>0:0;E=~~B>>>0}c[n>>2]=E;c[n+4>>2]=D;D=c[l>>2]|0;E=m+32|0;A=+h[E>>3];t=+R(A)>=1.0?A>0.0?(ah(+Q(A/4294967296.0),4294967295.0)|0)>>>0:~~+ab((A- +(~~A>>>0))/4294967296.0)>>>0:0;q=f|0;NM(q|0,0,64);a[q]=0;z=+h[x>>3];f=c[x+4>>2]&2147483647;F=2146435072;if(f>>>0>F>>>0|f>>>0==F>>>0&(c[x>>2]|0)>>>0>0>>>0){bc(q|0,64,1679376,(H=i,i=i+1|0,i=i+7&-8,c[H>>2]=0,H)|0)|0;i=H}else{x=~~z>>>0;F=+R(z)>=1.0?z>0.0?(ah(+Q(z/4294967296.0),4294967295.0)|0)>>>0:~~+ab((z- +(~~z>>>0))/4294967296.0)>>>0:0;f=1677456;bc(q|0,64,f|0,(H=i,i=i+16|0,c[H>>2]=x,c[H+8>>2]=F,H)|0)|0;i=H}z=+h[u>>3];u=g|0;NM(u|0,0,64);a[u]=0;w=+h[y>>3];g=c[y+4>>2]&2147483647;F=2146435072;if(g>>>0>F>>>0|g>>>0==F>>>0&(c[y>>2]|0)>>>0>0>>>0){bc(u|0,64,1679376,(H=i,i=i+1|0,i=i+7&-8,c[H>>2]=0,H)|0)|0;i=H}else{y=~~w>>>0;F=+R(w)>=1.0?w>0.0?(ah(+Q(w/4294967296.0),4294967295.0)|0)>>>0:~~+ab((w- +(~~w>>>0))/4294967296.0)>>>0:0;g=1677456;bc(u|0,64,g|0,(H=i,i=i+16|0,c[H>>2]=y,c[H+8>>2]=F,H)|0)|0;i=H}AN(D,48,1686760,(H=i,i=i+40|0,c[H>>2]=~~A>>>0,c[H+8>>2]=t,c[H+16>>2]=q,h[H+24>>3]=z,c[H+32>>2]=u,H)|0);i=H;u=c[s>>2]|0;if((u|0)==0){q=c[l>>2]|0;z=+h[m+24>>3];t=+R(z)>=1.0?z>0.0?(ah(+Q(z/4294967296.0),4294967295.0)|0)>>>0:~~+ab((z- +(~~z>>>0))/4294967296.0)>>>0:0;AN(q,48,1685136,(H=i,i=i+16|0,c[H>>2]=~~z>>>0,c[H+8>>2]=t,H)|0);i=H}else if((u|0)==1){u=c[l>>2]|0;z=+h[m+48>>3];t=+R(z)>=1.0?z>0.0?(ah(+Q(z/4294967296.0),4294967295.0)|0)>>>0:~~+ab((z- +(~~z>>>0))/4294967296.0)>>>0:0;A=+h[m+40>>3];q=+R(A)>=1.0?A>0.0?(ah(+Q(A/4294967296.0),4294967295.0)|0)>>>0:~~+ab((A- +(~~A>>>0))/4294967296.0)>>>0:0;AN(u,48,1683016,(H=i,i=i+32|0,c[H>>2]=~~z>>>0,c[H+8>>2]=t,c[H+16>>2]=~~A>>>0,c[H+24>>2]=q,H)|0);i=H}q=c[l>>2]|0;t=j|0;NM(t|0,0,64);a[t]=0;if(C){bc(t|0,64,1679376,(H=i,i=i+1|0,i=i+7&-8,c[H>>2]=0,H)|0)|0;i=H}else{C=~~B>>>0;j=+R(B)>=1.0?B>0.0?(ah(+Q(B/4294967296.0),4294967295.0)|0)>>>0:~~+ab((B- +(~~B>>>0))/4294967296.0)>>>0:0;u=1677456;bc(t|0,64,u|0,(H=i,i=i+16|0,c[H>>2]=C,c[H+8>>2]=j,H)|0)|0;i=H}if(B==-9223372036854776000.0){I=r}else{I=B*(+(c[b+56>>2]|0)/+(c[b+60>>2]|0))}AN(q,48,1681216,(H=i,i=i+16|0,c[H>>2]=t,h[H+8>>3]=I,H)|0);i=H;if((c[s>>2]|0)==0){J=+h[E>>3]+1.0}else{J=+h[E>>3]+ +(c[d+76>>2]|0)}h[E>>3]=J;if((o|0)==0&(p|0)==(-2147483648|0)){h[m+64>>3]=r;K=r}else{J=+(o>>>0)+ +(p|0)*4294967296.0;h[m+64>>3]=J;K=J*(+(c[b+56>>2]|0)/+(c[b+60>>2]|0))}h[m+72>>3]=K;p=c[n>>2]|0;o=c[n+4>>2]|0;if((p|0)==0&(o|0)==(-2147483648|0)){L=r}else{L=+(p>>>0)+ +(o|0)*4294967296.0}h[m+80>>3]=L;o=c[n>>2]|0;p=c[n+4>>2]|0;if((o|0)==0&(p|0)==(-2147483648|0)){M=r}else{M=(+(o>>>0)+ +(p|0)*4294967296.0)*(+(c[b+56>>2]|0)/+(c[b+60>>2]|0))}h[m+88>>3]=M;if((c[m+176>>2]|0)!=1){N=c[l>>2]|0;O=N+32|0;P=c[O>>2]|0;S=c[P>>2]|0;T=f7(S,d)|0;i=e;return T|0}b=m+40|0;h[b>>3]=+h[b>>3]+ +(c[d+76>>2]|0);N=c[l>>2]|0;O=N+32|0;P=c[O>>2]|0;S=c[P>>2]|0;T=f7(S,d)|0;i=e;return T|0}function hn(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,j=0,k=0.0,l=0.0;b=i;d=c[(c[a+8>>2]|0)+44>>2]|0;e=d+176|0;c[e>>2]=c[a+16>>2];f=d+16|0;g=d+136|0;h[g>>3]=+(c[a+56>>2]|0)/+(c[a+60>>2]|0);j=Ch()|0;h[d+152>>3]=+(j>>>0)+ +(G|0)*4294967296.0;if((c[e>>2]|0)==1){k=+(c[a+48>>2]|0)}else{k=r}h[d+104>>3]=k;h[d+168>>3]=k;d=c[a+116>>2]|0;do{if((d|0)==0){l=r}else{e=c[a+120>>2]|0;if((e|0)==0){l=r;break}l=+(d|0)/+(e|0)}}while(0);h[f>>3]=l;AN(c[a>>2]|0,40,1689488,(a=i,i=i+24|0,h[a>>3]=+h[g>>3],h[a+8>>3]=l,h[a+16>>3]=k,a)|0);i=a;i=b;return 0}function ho(a){a=a|0;var b=0;b=(c[a+44>>2]|0)+96|0;c[b>>2]=0;c[b+4>>2]=-2147483648;return 0}function hp(a){a=a|0;var b=0;b=a+156|0;c[b>>2]=c[b>>2]|1;return 0}function hq(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;d=i;i=i+24|0;e=d|0;f=d+8|0;g=d+16|0;c[e>>2]=b;h=c[a+8>>2]|0;j=c[h+44>>2]|0;k=j+120|0;if((c[k>>2]|0)!=0){At(e);l=0;i=d;return l|0}m=b+136|0;n=c[m>>2]|0;o=c[m+4>>2]|0;if((n|0)==0&(o|0)==(-2147483648|0)){m=j+112|0;p=c[m+4>>2]|0;q=c[m>>2]|0;r=b;s=m}else{c[f>>2]=1;c[f+4>>2]=c[a+48>>2];m=AY(n,o,a+56|0,f)|0;p=G;q=m;r=c[e>>2]|0;s=j+112|0}m=r+76|0;r=c[m>>2]|0;f=NU(r,(r|0)<0|0?-1:0,q,p)|0;c[s>>2]=f;c[s+4>>2]=G;s=j+64|0;f=c[s>>2]|0;r=c[s+4>>2]|0;s=0;do{if((r|0)<(s|0)|(r|0)==(s|0)&f>>>0<0>>>0){o=j+48|0;if((c[o>>2]|0)==0&(c[o+4>>2]|0)==(-2147483648|0)){t=0;u=0;v=5605;break}o=c[m>>2]|0;w=1;x=(o|0)<0|0?-1:0;y=o;z=o;v=5601}else{o=c[m>>2]|0;n=o;b=(o|0)<0|0?-1:0;A=j+88|0;B=c[A>>2]|0;C=c[A+4>>2]|0;A=NU(B,C,n,b)|0;D=G;if(!((D|0)>(r|0)|(D|0)==(r|0)&A>>>0>f>>>0)){w=1;x=b;y=n;z=o;v=5601;break}A=NV(f,r,B,C)|0;C=G;B=(b|0)>(C|0)|(b|0)==(C|0)&n>>>0>A>>>0;w=0;x=B?C:b;y=B?A:n;z=o;v=5601}}while(0);L7610:do{if((v|0)==5601){r=j+48|0;f=c[r>>2]|0;s=c[r+4>>2]|0;do{if(!((f|0)==0&(s|0)==(-2147483648|0)|(q|0)==0&(p|0)==(-2147483648|0))){r=NU(z,(z|0)<0|0?-1:0,q,p)|0;o=G;if(!((o|0)>(s|0)|(o|0)==(s|0)&r>>>0>f>>>0)){break}r=NV(f,s,q,p)|0;o=G;n=(x|0)>(o|0)|(x|0)==(o|0)&y>>>0>r>>>0;t=n?o:x;u=n?r:y;v=5605;break L7610}}while(0);if((w|0)==0){t=x;u=y;v=5605}else{E=z}}}while(0);L7616:do{if((v|0)==5605){z=j+96|0;y=c[z>>2]|0;x=c[z+4>>2]|0;if((y|0)==0&(x|0)==(-2147483648|0)){w=NU(u,t,q,p)|0;s=G;c[z>>2]=w;c[z+4>>2]=s;F=s;H=w}else{F=x;H=y}y=j+72|0;x=c[y>>2]|0;w=c[y+4>>2]|0;do{if((x|0)==(-1|0)&(w|0)==2147483647){y=j+56|0;if(!((c[y>>2]|0)==0&(c[y+4>>2]|0)==(-2147483648|0))){I=1;J=0;K=0;v=5613;break}y=j+104|0;if(!((c[y>>2]|0)==0&(c[y+4>>2]|0)==0)){I=1;J=0;K=0;v=5613;break}y=c[m>>2]|0;L=(y|0)<0|0?-1:0;M=y}else{y=j+88|0;s=c[y>>2]|0;z=c[y+4>>2]|0;if(!((z|0)<(w|0)|(z|0)==(w|0)&s>>>0<x>>>0)){I=1;J=0;K=0;v=5613;break}y=NV(x,w,s,z)|0;z=G;s=0;f=(z|0)<(s|0)|(z|0)==(s|0)&y>>>0<0>>>0;I=0;J=f?0:z;K=f?0:y;v=5613}}while(0);L7627:do{if((v|0)==5613){w=j+56|0;x=c[w>>2]|0;y=c[w+4>>2]|0;if(((q|0)!=0|(p|0)!=(-2147483648|0))&((x|0)!=0|(y|0)!=(-2147483648|0))&((p|0)<(y|0)|(p|0)==(y|0)&q>>>0<x>>>0)){w=NV(x,y,q,p)|0;y=G;x=(J|0)>(y|0)|(J|0)==(y|0)&K>>>0>w>>>0;N=0;O=x?J:y;P=x?K:w}else{N=I;O=J;P=K}w=j+104|0;x=c[w>>2]|0;y=c[w+4>>2]|0;do{if(!((x|0)==0&(y|0)==0)){w=NV(q,p,H,F)|0;f=G;if(!((f|0)<(y|0)|(f|0)==(y|0)&w>>>0<x>>>0)){break}w=NV(x,y,q,p)|0;f=NU(w,G,H,F)|0;w=G;z=(O|0)>(w|0)|(O|0)==(w|0)&P>>>0>f>>>0;L=z?O:w;M=z?P:f;break L7627}}while(0);if((N|0)==0){L=O;M=P;break}c[a+144>>2]=1;c[k>>2]=1;E=c[(c[e>>2]|0)+76>>2]|0;break L7616}}while(0);y=c[m>>2]|0;x=j+88|0;f=NU(c[x>>2]|0,c[x+4>>2]|0,y,(y|0)<0|0?-1:0)|0;c[x>>2]=f;c[x+4>>2]=G;x=0;f=(t|0)<(x|0)|(t|0)==(x|0)&u>>>0<0>>>0;x=f?0:u;y=f?0:t;f=c[m>>2]|0;z=f;w=(f|0)<0|0?-1:0;f=(w|0)>(L|0)|(w|0)==(L|0)&z>>>0>M>>>0;s=f?M:z;z=f?L:w;if(!((y|0)<(z|0)|(y|0)==(z|0)&x>>>0<s>>>0)){AN(0,0,1709024,(w=i,i=i+24|0,c[w>>2]=1706920,c[w+8>>2]=1704904,c[w+16>>2]=307,w)|0);i=w;cw();return 0}if((x|0)==0&(y|0)==0){c[m>>2]=s;Q=c[e>>2]|0;R=h+32|0}else{w=h+32|0;f=c[c[w>>2]>>2]|0;r=NV(s,z,x,y)|0;z=hI(f,r)|0;if((z|0)==0){At(e);l=-12;i=d;return l|0}r=c[e>>2]|0;Ax(z,r)|0;r=c[z+64>>2]|0;f=c[e>>2]|0;s=c[f+64>>2]|0;n=x;o=c[z+76>>2]|0;A=f+352|0;f=zs(c[A>>2]|0,c[A+4>>2]|0)|0;A=c[(c[e>>2]|0)+80>>2]|0;Cb(r,s,0,n,o,f,A)|0;A=z+136|0;if(!((c[A>>2]|0)==0&(c[A+4>>2]|0)==(-2147483648|0))){c[g>>2]=1;c[g+4>>2]=c[z+344>>2];f=AY(x,y,g,a+56|0)|0;y=NU(c[A>>2]|0,c[A+4>>2]|0,f,G)|0;c[A>>2]=y;c[A+4>>2]=G}At(e);c[e>>2]=z;Q=z;R=w}l=f7(c[c[R>>2]>>2]|0,Q)|0;i=d;return l|0}}while(0);Q=j+88|0;j=NU(c[Q>>2]|0,c[Q+4>>2]|0,E,(E|0)<0|0?-1:0)|0;c[Q>>2]=j;c[Q+4>>2]=G;At(e);l=0;i=d;return l|0}function hr(a){a=a|0;var b=0,d=0,e=0.0,f=0,g=0.0,i=0,j=0,k=0;b=c[(c[a+8>>2]|0)+44>>2]|0;if((c[a+16>>2]|0)==0){d=a+56|0;e=+(c[d>>2]|0);f=c[d+4>>2]|0}else{e=1.0;f=c[a+48>>2]|0}g=+h[b+16>>3];do{if(g!=1.7976931348623157e+308){a=bR(+(g/(e/+(f|0))))|0;d=a;i=(a|0)<0|0?-1:0;a=b+48|0;j=c[a>>2]|0;k=c[a+4>>2]|0;if(!((j|0)==0&(k|0)==(-2147483648|0)|((i|0)<(k|0)|(i|0)==(k|0)&d>>>0<j>>>0))){break}c[a>>2]=d;c[a+4>>2]=i}}while(0);g=+h[b+24>>3];do{if(g!=1.7976931348623157e+308){i=bR(+(g/(e/+(f|0))))|0;a=i;d=(i|0)<0|0?-1:0;i=b+56|0;j=c[i>>2]|0;k=c[i+4>>2]|0;if(!((j|0)==0&(k|0)==(-2147483648|0)|((d|0)>(k|0)|(d|0)==(k|0)&a>>>0>j>>>0))){break}c[i>>2]=a;c[i+4>>2]=d}}while(0);g=+h[b+8>>3];if(g==0.0){return 0}d=bR(+(g/(e/+(f|0))))|0;f=b+104|0;c[f>>2]=d;c[f+4>>2]=(d|0)<0|0?-1:0;return 0}function hs(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;d=i;i=i+8|0;e=d|0;c[e>>2]=b;f=c[a+8>>2]|0;g=c[f+44>>2]|0;h=g+120|0;if((c[h>>2]|0)!=0){At(e);j=0;i=d;return j|0}k=g+32|0;l=c[k>>2]|0;m=c[k+4>>2]|0;k=-1;n=g+48|0;o=c[n>>2]|0;p=c[n+4>>2]|0;if((m|0)>(k|0)|(m|0)==(k|0)&l>>>0>-1>>>0){k=g+80|0;n=c[k+4>>2]|0;q=((n|0)<(m|0)|(n|0)==(m|0)&(c[k>>2]|0)>>>0<l>>>0)&1;if((o|0)==0&(p|0)==(-2147483648|0)){r=q;s=5656}else{t=q;s=5655}}else{if((o|0)==0&(p|0)==(-2147483648|0)){s=5657}else{t=1;s=5655}}if((s|0)==5655){q=b+136|0;l=c[q>>2]|0;k=c[q+4>>2]|0;r=(l|0)==0&(k|0)==(-2147483648|0)|((k|0)<(p|0)|(k|0)==(p|0)&l>>>0<o>>>0)?t:0;s=5656}if((s|0)==5656){if((r|0)==0){s=5657}}L7684:do{if((s|0)==5657){r=g+96|0;t=c[r>>2]|0;o=c[r+4>>2]|0;do{if((t|0)==0&(o|0)==(-2147483648|0)){l=b+136|0;p=c[l>>2]|0;k=c[l+4>>2]|0;if((p|0)==0&(k|0)==(-2147483648|0)){u=-2147483648;v=0;break}c[r>>2]=p;c[r+4>>2]=k;u=k;v=p}else{u=o;v=t}}while(0);t=g+40|0;o=c[t>>2]|0;r=c[t+4>>2]|0;t=g+56|0;p=c[t>>2]|0;k=c[t+4>>2]|0;do{if((o|0)==(-1|0)&(r|0)==2147483647){if(!((p|0)==0&(k|0)==(-2147483648|0))){w=1;s=5664;break}t=g+104|0;if(!((c[t>>2]|0)==0&(c[t+4>>2]|0)==0)){x=1;s=5665}}else{t=g+80|0;l=c[t+4>>2]|0;q=(l|0)>=(r|0)&((l|0)>(r|0)|(c[t>>2]|0)>>>0>=o>>>0)&1;if((p|0)==0&(k|0)==(-2147483648|0)){x=q;s=5665}else{w=q;s=5664}}}while(0);if((s|0)==5664){o=b+136|0;r=c[o>>2]|0;q=c[o+4>>2]|0;x=((r|0)!=0|(q|0)!=(-2147483648|0))&((q|0)<(k|0)|(q|0)==(k|0)&r>>>0<p>>>0)?0:w;s=5665}do{if((s|0)==5665){r=g+104|0;q=c[r>>2]|0;o=c[r+4>>2]|0;do{if((q|0)==0&(o|0)==0){y=x}else{r=b+136|0;t=c[r>>2]|0;l=c[r+4>>2]|0;if((t|0)==0&(l|0)==(-2147483648|0)){y=x;break}r=NV(t,l,v,u)|0;l=G;y=(l|0)<(o|0)|(l|0)==(o|0)&r>>>0<q>>>0?0:x}}while(0);if((y|0)==0){break}c[a+144>>2]=1;c[h>>2]=1;break L7684}}while(0);p=g+80|0;k=NU(c[p>>2]|0,c[p+4>>2]|0,1,0)|0;c[p>>2]=k;c[p+4>>2]=G;j=f7(c[c[f+32>>2]>>2]|0,b)|0;i=d;return j|0}}while(0);b=g+80|0;g=NU(c[b>>2]|0,c[b+4>>2]|0,1,0)|0;c[b>>2]=g;c[b+4>>2]=G;At(e);j=0;i=d;return j|0}function ht(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;d=i;i=i+40|0;e=d|0;f=d+32|0;g=c[b+44>>2]|0;h=c[g+4>>2]|0;if((h|0)==0){j=0;i=d;return j|0}k=e|0;l=b;b=g+8|0;g=h;while(1){h=bi(g|0,124)|0;m=(h|0)!=0;if(m){n=h-g|0}else{n=NL(g|0)|0}if((n|0)>31){o=5682;break}NN(k|0,g|0,n)|0;a[e+n|0]=0;p=hb(f,k,l)|0;if((p|0)<0){j=p;o=5686;break}c[b+(c[f>>2]<<2)>>2]=1;p=m?h+1|0:0;if((p|0)==0){j=0;o=5689;break}else{g=p}}if((o|0)==5686){i=d;return j|0}else if((o|0)==5682){AN(l,16,1702088,(l=i,i=i+1|0,i=i+7&-8,c[l>>2]=0,l)|0);i=l;j=-1;i=d;return j|0}else if((o|0)==5689){i=d;return j|0}return 0}function hu(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0;b=i;i=i+8|0;d=b|0;e=c[a+44>>2]|0;c[d>>2]=0;f=0;while(1){if((c[e+8+(f<<2)>>2]|0)==1){if((gW(d,f,(f|0)<0|0?-1:0)|0)<0){g=5693;break}}h=f+1|0;if((h|0)<321){f=h}else{g=5695;break}}if((g|0)==5695){j=c[d>>2]|0;g8(a,j);i=b;return 0}else if((g|0)==5693){g2(d);j=0;g8(a,j);i=b;return 0}return 0}function hv(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0;e=i;i=i+8|0;f=e|0;g=c[c[(c[a+8>>2]|0)+32>>2]>>2]|0;fR(0,g,0);a=c[(c[g+12>>2]|0)+20>>2]|0;do{if((a|0)!=0){h=c6[a&255](g,b,d)|0;if((h|0)==0){break}else{j=h}i=e;return j|0}}while(0);a=As()|0;c[f>>2]=a;if((a|0)==0){j=0;i=e;return j|0}c[a+68>>2]=b;c[(c[f>>2]|0)+72>>2]=d;c[(c[f>>2]|0)+80>>2]=c[g+52>>2];g=c[f>>2]|0;if((Av(g,32)|0)>=0){j=g;i=e;return j|0}At(f);j=c[f>>2]|0;i=e;return j|0}function hw(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0;e=i;i=i+8|0;f=e|0;fR(0,a,0);g=c[(c[a+12>>2]|0)+20>>2]|0;do{if((g|0)!=0){h=c6[g&255](a,b,d)|0;if((h|0)==0){break}else{j=h}i=e;return j|0}}while(0);g=As()|0;c[f>>2]=g;if((g|0)==0){j=0;i=e;return j|0}c[g+68>>2]=b;c[(c[f>>2]|0)+72>>2]=d;c[(c[f>>2]|0)+80>>2]=c[a+52>>2];a=c[f>>2]|0;if((Av(a,32)|0)>=0){j=a;i=e;return j|0}At(f);j=c[f>>2]|0;i=e;return j|0}function hx(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;d=i;e=c[b+44>>2]|0;f=c[e+16>>2]|0;L7760:do{if((f|0)!=0){if((bi(f|0,44)|0)==0){g=124}else{AN(b,24,1701936,(h=i,i=i+8|0,c[h>>2]=1678616,h)|0);i=h;g=44}j=e+4|0;k=f;while(1){l=bi(k|0,g|0)|0;if((l|0)==0){m=0}else{a[l]=0;m=l+1|0}l=B3(k)|0;if((l|0)==-1){break}gW(j,l,(l|0)<0|0?-1:0)|0;if((m|0)==0){break L7760}else{k=m}}AN(b,16,1662200,(h=i,i=i+8|0,c[h>>2]=k,h)|0);i=h;n=-22;i=d;return n|0}}while(0);m=c[e+20>>2]|0;L7774:do{if((m|0)!=0){if((bi(m|0,44)|0)==0){o=124}else{AN(b,24,1701936,(h=i,i=i+8|0,c[h>>2]=1645376,h)|0);i=h;o=44}g=e+8|0;f=m;while(1){j=bi(f|0,o|0)|0;if((j|0)==0){p=0}else{a[j]=0;p=j+1|0}j=aU(f|0,0,0)|0;l=(j|0)>0?j:0;if((l|0)==0){break}gW(g,l,(l|0)<0|0?-1:0)|0;if((p|0)==0){break L7774}else{f=p}}AN(b,16,1632448,(h=i,i=i+8|0,c[h>>2]=f,h)|0);i=h;n=-22;i=d;return n|0}}while(0);p=c[e+24>>2]|0;if((p|0)==0){n=0;i=d;return n|0}if((bi(p|0,44)|0)==0){q=124}else{AN(b,24,1701936,(h=i,i=i+8|0,c[h>>2]=1625960,h)|0);i=h;q=44}o=e+12|0;e=p;while(1){p=bi(e|0,q|0)|0;if((p|0)==0){r=0}else{a[p]=0;r=p+1|0}p=zq(e)|0;m=G;if((p|0)==0&(m|0)==0){break}gX(o,p,m)|0;if((r|0)==0){n=0;s=5747;break}else{e=r}}if((s|0)==5747){i=d;return n|0}AN(b,16,1619376,(h=i,i=i+8|0,c[h>>2]=e,h)|0);i=h;n=-22;i=d;return n|0}function hy(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=c[a+44>>2]|0;d=c[b+4>>2]|0;if((d|0)==0){e=gY(1)|0}else{e=d}g8(a,e);e=c[b+8>>2]|0;if((e|0)==0){f=gZ()|0}else{f=e}g7(a,f);f=c[b+12>>2]|0;if((f|0)!=0){g=f;g6(a,g);return 0}g=g$()|0;g6(a,g);return 0}function hz(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;d=c[a+44>>2]|0;a=d+24|0;c[a>>2]=0;c[a+4>>2]=-2147483648;a=rO()|0;e=d+16|0;c[e>>2]=a;if((a|0)==0){f=-12;return f|0}do{if((b|0)==0){g=0}else{a=0;h=0;while(1){i=zJ(c[b>>2]|0,3119032,h,2)|0;if((i|0)==0){j=5765;break}k=Bd(c[e>>2]|0,c[i>>2]|0,c[i+4>>2]|0,0)|0;if((k|0)<0){f=k;j=5771;break}else{a=k;h=i}}if((j|0)==5765){zL(b);g=a;break}else if((j|0)==5771){return f|0}}}while(0);j=c[d+4>>2]|0;if((j|0)<=0){f=g;return f|0}Bh(c[e>>2]|0,1701848,j,(j|0)<0|0?-1:0,0)|0;f=g;return f|0}function hA(a){a=a|0;rQ((c[a+44>>2]|0)+16|0);return}function hB(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;b=i;i=i+32|0;d=b|0;e=b+8|0;f=b+16|0;g=(c[a+44>>2]|0)+16|0;h=Bm(c[g>>2]|0,1701848,0)|0;j=h;h=Bm(c[g>>2]|0,1678584,0)|0;k=G;l=Bm(c[g>>2]|0,1662128,0)|0;g=l;l=c[c[a+16>>2]>>2]|0;m=c[c[a+32>>2]>>2]|0;a=gY(1)|0;n=gZ()|0;o=g$()|0;g1(a,l+68|0);g1(n,l+76|0);g0(o,l+84|0);if((j|0)>0){l=d|0;c[l>>2]=j;c[d+4>>2]=-1;p=gU(l)|0}else{p=gZ()|0}g1(p,m+72|0);if((g|0)==-1){q=gY(1)|0}else{p=e|0;c[p>>2]=g;c[e+4>>2]=-1;q=gU(p)|0}g1(q,m+64|0);if((h|0)==0&(k|0)==0){r=g$()|0;s=m+80|0;g0(r,s);i=b;return 0}else{q=f|0;c[q>>2]=h;c[q+4>>2]=k;k=f+8|0;c[k>>2]=-1;c[k+4>>2]=-1;r=gV(q)|0;s=m+80|0;g0(r,s);i=b;return 0}return 0}function hC(a,b){a=a|0;b=b|0;var d=0;if((b|0)!=0){d=0;return d|0}d=c[a+16>>2]|0;return d|0}function hD(a){a=a|0;var b=0;if((a|0)==0){b=rN()|0}else{b=0}return b|0}function hE(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;b=i;i=i+8|0;d=b|0;e=c[a>>2]|0;f=c[e+44>>2]|0;g=e+16|0;e=c[c[g>>2]>>2]|0;h=f+32|0;c[h>>2]=0;do{j=fS(c[c[g>>2]>>2]|0)|0;}while((c[h>>2]|0)==0&(j|0)>-1);if((j|0)!=-541478725){k=j;i=b;return k|0}j=hI(a,4096)|0;c[d>>2]=j;if((j|0)==0){k=-12;i=b;return k|0}h=f+16|0;f=rT(c[h>>2]|0,c[j+64>>2]|0,4096,0,0)|0;if((f|0)<1){At(d);k=(f|0)==0?-541478725:f;i=b;return k|0}else{c[j+344>>2]=c[a+48>>2];c[(c[d>>2]|0)+76>>2]=f;f=rX(c[h>>2]|0,0,-2147483648)|0;h=G;j=c[d>>2]|0;d=j+136|0;c[d>>2]=f;c[d+4>>2]=h;g=0;l=(h|0)>(g|0)|(h|0)==(g|0)&f>>>0>0>>>0;g=c[e+48>>2]|0;e=g>>1;m=e;n=(e|0)<0|0?-1:0;e=NV(0,0,m,n)|0;o=NU(l?m:e,l?n:G,f,h)|0;h=N0(o,G,g,(g|0)<0|0?-1:0)|0;c[d>>2]=h;c[d+4>>2]=G;k=f7(a,j)|0;i=b;return k|0}return 0}function hF(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;b=i;i=i+256|0;d=c[a>>2]|0;e=c[c[d+16>>2]>>2]|0;f=c[d+44>>2]|0;g=f+16|0;j=a+40|0;k=a+52|0;l=a+48|0;m=e+40|0;n=e+52|0;o=e+48|0;p=d;d=rP(c[g>>2]|0,c[j>>2]|0,c[j+4>>2]|0,c[k>>2]|0,c[l>>2]|0,c[m>>2]|0,c[m+4>>2]|0,c[n>>2]|0,c[o>>2]|0,0,p)|0;c[g>>2]=d;if((d|0)==0){q=-12;i=b;return q|0}if((c[m>>2]|0)==0&(c[m+4>>2]|0)==0){r=d;d=c[e+148>>2]|0;s=d;t=(d|0)<0|0?-1:0;d=1619368;Bh(r,d,s,t,0)|0}if((c[j>>2]|0)==0&(c[j+4>>2]|0)==0){t=c[g>>2]|0;s=c[a+148>>2]|0;d=s;r=(s|0)<0|0?-1:0;s=1615312;Bh(t,s,d,r,0)|0}r=rR(c[g>>2]|0)|0;if((r|0)<0){q=r;i=b;return q|0}r=Bm(c[g>>2]|0,1701848,0)|0;d=r;r=Bm(c[g>>2]|0,1678584,0)|0;s=G;t=Bm(c[g>>2]|0,1662128,0)|0;c[a+56>>2]=1;c[a+60>>2]=d;if((c[l>>2]|0)!=(d|0)){AN(0,0,1744312,(u=i,i=i+24|0,c[u>>2]=1737808,c[u+8>>2]=1733240,c[u+16>>2]=156,u)|0);i=u;cw();return 0}g=c[j>>2]|0;v=c[j+4>>2]|0;if(!((g|0)==(r|0)&(v|0)==(s|0)|(g|0)==0&(v|0)==0)){AN(0,0,1744312,(u=i,i=i+24|0,c[u>>2]=1728136,c[u+8>>2]=1733240,c[u+16>>2]=157,u)|0);i=u;cw();return 0}if((c[k>>2]|0)!=(t|0)){AN(0,0,1744312,(u=i,i=i+24|0,c[u>>2]=1723864,c[u+8>>2]=1733240,c[u+16>>2]=158,u)|0);i=u;cw();return 0}h[f+8>>3]=+(d|0)/+(c[o>>2]|0);d=b|0;f=e+148|0;zt(d,128,c[f>>2]|0,c[m>>2]|0,c[m+4>>2]|0);m=b+128|0;e=a+148|0;zt(m,128,c[e>>2]|0,c[j>>2]|0,c[j+4>>2]|0);j=c[f>>2]|0;f=B2(c[n>>2]|0)|0;n=c[o>>2]|0;o=c[e>>2]|0;e=B2(c[k>>2]|0)|0;k=c[l>>2]|0;AN(p,40,1720296,(u=i,i=i+64|0,c[u>>2]=j,c[u+8>>2]=d,c[u+16>>2]=f,c[u+24>>2]=n,c[u+32>>2]=o,c[u+40>>2]=m,c[u+48>>2]=e,c[u+56>>2]=k,u)|0);i=u;q=0;i=b;return q|0}function hG(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;d=i;i=i+16|0;e=d|0;f=d+8|0;c[e>>2]=b;g=c[a+8>>2]|0;j=c[g+44>>2]|0;k=c[b+76>>2]|0;l=~~(+(k|0)*+h[j+8>>3]*2.0+256.0);m=c[c[g+32>>2]>>2]|0;g=hI(m,l)|0;c[f>>2]=g;if((g|0)==0){n=-12;i=d;return n|0}Ax(g,b)|0;c[g+80>>2]=c[m+52>>2];g=c[f>>2]|0;Ao(g,c[m+148>>2]|0);b=m+40|0;o=c[b+4>>2]|0;p=g+352|0;c[p>>2]=c[b>>2];c[p+4>>2]=o;o=m+48|0;c[g+344>>2]=c[o>>2];g=c[e>>2]|0;p=g+136|0;b=c[p>>2]|0;q=c[p+4>>2]|0;if((b|0)==0&(q|0)==(-2147483648|0)){p=c[f>>2]|0;r=p+136|0;c[r>>2]=0;c[r+4>>2]=-2147483648;s=p;t=g;u=j+16|0}else{g=c[a+56>>2]|0;p=c[o>>2]|0;o=N2(p,(p|0)<0|0?-1:0,g,(g|0)<0|0?-1:0)|0;g=a+48|0;p=c[g>>2]|0;r=N2(o,G,p,(p|0)<0|0?-1:0)|0;p=c[a+60>>2]|0;a=AW(b,q,r,G,p,(p|0)<0|0?-1:0)|0;p=j+16|0;r=rX(c[p>>2]|0,a,G)|0;a=G;q=0;b=(a|0)>(q|0)|(a|0)==(q|0)&r>>>0>0>>>0;q=c[g>>2]|0;g=q>>1;o=g;v=(g|0)<0|0?-1:0;g=NV(0,0,o,v)|0;w=NU(b?o:g,b?v:G,r,a)|0;a=N0(w,G,q,(q|0)<0|0?-1:0)|0;q=G;w=c[f>>2]|0;r=w+136|0;c[r>>2]=a;c[r+4>>2]=q;r=j+24|0;c[r>>2]=a;c[r+4>>2]=q;s=w;t=c[e>>2]|0;u=p}p=rT(c[u>>2]|0,c[s+64>>2]|0,l,c[t+64>>2]|0,k)|0;if((p|0)<1){At(f);At(e);n=0;i=d;return n|0}else{c[s+76>>2]=p;p=f7(m,c[f>>2]|0)|0;c[j+32>>2]=1;At(e);n=p;i=d;return n|0}return 0}function hH(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=c[c[(c[a+8>>2]|0)+32>>2]>>2]|0;a=c[(c[d+12>>2]|0)+24>>2]|0;do{if((a|0)!=0){e=dg[a&255](d,b)|0;if((e|0)==0){break}else{f=e}return f|0}}while(0);f=hJ(d,b)|0;return f|0}function hI(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=c[(c[a+12>>2]|0)+24>>2]|0;do{if((d|0)!=0){e=dg[d&255](a,b)|0;if((e|0)==0){break}else{f=e}return f|0}}while(0);f=hJ(a,b)|0;return f|0}function hJ(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0;d=i;i=i+8|0;e=d|0;f=As()|0;c[e>>2]=f;g=a+148|0;h=c[g>>2]|0;j=a+40|0;do{if((h|0)==(zs(c[j>>2]|0,c[j+4>>2]|0)|0)){k=f}else{if((zs(c[j>>2]|0,c[j+4>>2]|0)|0)==0){k=c[e>>2]|0;break}else{AN(0,0,1720736,(l=i,i=i+24|0,c[l>>2]=1735664,c[l+8>>2]=1701744,c[l+16>>2]=47,l)|0);i=l;cw();return 0}}}while(0);if((k|0)==0){m=0;i=d;return m|0}c[k+76>>2]=b;k=a+52|0;c[(c[e>>2]|0)+80>>2]=c[k>>2];f=c[e>>2]|0;Ao(f,c[g>>2]|0);g=c[j+4>>2]|0;l=f+352|0;c[l>>2]=c[j>>2];c[l+4>>2]=g;c[f+344>>2]=c[a+48>>2];a=c[e>>2]|0;if((Av(a,0)|0)<0){At(e);m=0;i=d;return m|0}else{f=c[a+64>>2]|0;a=c[k>>2]|0;Ca(f,0,b,h,a)|0;m=c[e>>2]|0;i=d;return m|0}return 0}function hK(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0;d=i;if((a|0)==0){e=-22;i=d;return e|0}if((b|0)==0){AN(0,0,1735624,(f=i,i=i+24|0,c[f>>2]=1701704,c[f+8>>2]=1678424,c[f+16>>2]=93,f)|0);i=f;cw();return 0}g=a;h=b+4|0;NN(g|0,h|0,32)|0;h=a+32|0;g=b+40|0;NN(h|0,g|0,32)|0;g=b+80|0;h=c[g+4>>2]|0;j=a+136|0;c[j>>2]=c[g>>2];c[j+4>>2]=h;h=b+96|0;c[a+80>>2]=c[h>>2];j=b+88|0;Al(a,c[j>>2]|0,c[j+4>>2]|0);j=c[b+104>>2]|0;if((j|0)==1){g=b+76|0;k=c[g>>2]|0;if((k|0)==0){AN(0,0,1735624,(f=i,i=i+24|0,c[f>>2]=1645328,c[f+8>>2]=1678424,c[f+16>>2]=114,f)|0);i=f;cw();return 0}l=k|0;k=zs(c[l>>2]|0,c[l+4>>2]|0)|0;l=(B8(c[h>>2]|0)|0)!=0?k:1;do{if(l>>>0>8){k=l<<2;h=A7(k)|0;c[a+64>>2]=h;if((h|0)==0){e=-12;i=d;return e|0}else{m=c[b+36>>2]|0;NN(h|0,m|0,k)|0;break}}else{c[a+64>>2]=a}}while(0);c[a+76>>2]=c[(c[g>>2]|0)+8>>2];Ap(a,c[(c[g>>2]|0)+12>>2]|0);l=c[g>>2]|0;Am(a,c[l>>2]|0,c[l+4>>2]|0);Ao(a,c[(c[g>>2]|0)+16>>2]|0);e=0;i=d;return e|0}else if((j|0)==0){j=b+72|0;b=c[j>>2]|0;if((b|0)==0){AN(0,0,1735624,(f=i,i=i+24|0,c[f>>2]=1662032,c[f+8>>2]=1678424,c[f+16>>2]=104,f)|0);i=f;cw();return 0}c[a+68>>2]=c[b>>2];c[a+72>>2]=c[(c[j>>2]|0)+4>>2];b=(c[j>>2]|0)+8|0;f=a+124|0;g=c[b+4>>2]|0;c[f>>2]=c[b>>2];c[f+4>>2]=g;c[a+296>>2]=c[(c[j>>2]|0)+16>>2];c[a+300>>2]=c[(c[j>>2]|0)+20>>2];c[a+84>>2]=c[(c[j>>2]|0)+28>>2];c[a+88>>2]=c[(c[j>>2]|0)+24>>2];e=0;i=d;return e|0}else{e=-22;i=d;return e|0}return 0}function hL(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=i;if((a|0)==0){i=b;return}d=a|0;e=(c[d>>2]|0)+88|0;f=c[e>>2]|0;if((f|0)==0){AN(0,0,1719960,(g=i,i=i+24|0,c[g>>2]=1735584,c[g+8>>2]=1701672,c[g+16>>2]=99,g)|0);i=g;cw()}g=f-1|0;c[e>>2]=g;if((g|0)==0){g=c[d>>2]|0;c_[c[g+72>>2]&127](g)}g=a+36|0;if((c[g>>2]|0)!=(a+4|0)){A6(g)}g=a+72|0;d=c[g>>2]|0;if((d|0)!=0){A6(d+40|0)}A6(g);A6(a+76|0);zL(a+108|0);A4(a);i=b;return}function hM(){if(a[341184]|0){return}a[341184]=1;nz();je(257336);jf(255904);je(256352);jf(247504);je(247576);jf(247224);je(247296);jf(247152);jf(245352);je(245424);je(237672);h2(250928,68)|0;return}function hN(a){a=a|0;return((NO(c[a+4>>2]|0,256192,16)|0)==0?100:0)|0}function hO(e){e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0;f=i;i=i+1160|0;g=f|0;h=f+1040|0;j=f+1056|0;k=f+1072|0;l=f+1152|0;m=e+12|0;n=c[m>>2]|0;o=e+16|0;p=c[o>>2]|0;l0(p,j);q=j|0;if((NO(q|0,256192,16)|0)!=0){r=-1094995529;i=f;return r|0}iJ(p)|0;iI(p)|0;iD(p)|0;iD(p)|0;s=n+4|0;t=s;NM(s|0,-1|0,512);s=il(p,0,0,1)|0;u=G;l0(p,j);v=iJ(p)|0;w=G;L7986:do{if((NO(q|0,256424,16)|0)==0){x=w;y=v}else{z=e+1076|0;A=e;B=h|0;C=e+24|0;D=f+1032|0;E=f+8|0;F=g|0;H=g+4|0;I=e+1100|0;J=k+24|0;K=k+28|0;L=e+1140|0;M=u;N=s;O=w;P=v;L7988:while(1){Q=0;if((O|0)<(Q|0)|(O|0)==(Q|0)&P>>>0<24>>>0){r=-1094995529;R=6045;break}L7991:do{if((NO(q|0,256240,16)|0)==0){Q=c[m>>2]|0;S=c[o>>2]|0;l0(S,Q+156464|0);T=iJ(S)|0;U=Q+156480|0;c[U>>2]=T;c[U+4>>2]=G;U=iJ(S)|0;T=Q+156488|0;c[T>>2]=U;c[T+4>>2]=G;iJ(S)|0;T=iJ(S)|0;U=Q+156496|0;c[U>>2]=T;c[U+4>>2]=G;U=iJ(S)|0;T=Q+156504|0;c[T>>2]=U;c[T+4>>2]=G;c[Q+156512>>2]=iI(S)|0;c[Q+156516>>2]=iI(S)|0;c[Q+156520>>2]=iI(S)|0;T=Q+156524|0;c[T>>2]=iI(S)|0;U=Q+156528|0;c[U>>2]=iI(S)|0;if((c[T>>2]|0)>>>0>536870911){r=-1094995529;R=6044;break L7988}c[Q+156532>>2]=iI(S)|0;c[z>>2]=c[U>>2];R=6017}else{if((NO(q|0,255872,16)|0)==0){U=c[m>>2]|0;S=c[o>>2]|0;if((c[C>>2]|0)==127){AN(A,16,1634296,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V;R=6017;break}Q=il(S,0,0,1)|0;T=G;W=mK(e,0)|0;if((W|0)==0){R=6017;break}mL(W,32,1,1e3);X=A7(1200)|0;if((X|0)==0){R=6017;break}c[W+12>>2]=X;Y=c[U+156512>>2]|0;Z=0;b[X+128>>1]=128;L8002:do{if((c[U+156520>>2]&1|0)==0){_=ip(S)|0;$=G;aa=0;do{if(!(($|0)<(aa|0)|($|0)==(aa|0)&_>>>0<1>>>0)){ab=U+156480|0;ac=c[ab>>2]|0;ad=c[ab+4>>2]|0;ab=0;if((ad|0)<(ab|0)|(ad|0)==(ab|0)&ac>>>0<1>>>0){break}ab=NV(_,$,ac,ad)|0;ad=G;ac=-1;ae=(ad|0)>(ac|0)|(ad|0)==(ac|0)&ab>>>0>-1>>>0;ac=NV(0,0,ab,ad)|0;af=ae?ad:G;ad=0;if(!((af|0)<(ad|0)|(af|0)==(ad|0)&(ae?ab:ac)>>>0<1e4>>>0)){break L8002}}}while(0);$=U+156496|0;_=N3(c[$>>2]|0,c[$+4>>2]|0,1e4,0)|0;$=NV(_,G,Y,Z)|0;_=W+56|0;c[_>>2]=$;c[_+4>>2]=G}}while(0);l0(S,h);do{if((NO(B|0,256520,16)|0)==0){ag=1;ah=0}else{if((NO(B|0,255840,16)|0)==0){ag=0;ah=0;break}if((NO(B|0,256176,16)|0)==0){c[(c[W+8>>2]|0)+48>>2]=8;ag=0;ah=0;break}if((NO(B|0,256472,16)|0)==0){ag=2;ah=0;break}if((NO(B|0,256288,16)|0)==0){ag=-1;ah=1}else{R=6017;break L7991}}}while(0);l0(S,h);Z=8;Y=0;io(S,Z,Y)|0;Y=iI(S)|0;iI(S)|0;Z=(iG(S)|0)&127;c[W+4>>2]=Z;c[U+4+(Z<<2)>>2]=(c[C>>2]|0)-1;iI(S)|0;do{if(ah){l0(S,h);if((NO(B|0,256304,16)|0)!=0){R=5926;break}l0(S,h);iI(S)|0;iI(S)|0;iI(S)|0;l0(S,h);iI(S)|0;Z=W+8|0;c[(c[Z>>2]|0)+8>>2]=1;ai=1;aj=Z;R=5927}else{R=5926}}while(0);do{if((R|0)==5926){R=0;U=W+8|0;c[(c[U>>2]|0)+8>>2]=ag;if((ag|0)==1){ai=0;aj=U;R=5927;break}else if((ag|0)!=0){break}Z=il(S,0,0,1)|0;_=G;$=NU(P,O,-24,-1)|0;aa=NU($,G,Q,T)|0;$=NV(aa,G,Z,_)|0;_=G;Z=0;if(!((_|0)>(Z|0)|(_|0)==(Z|0)&$>>>0>50>>>0)){break}iI(S)|0;iI(S)|0;iD(S)|0;iG(S)|0;$=iI(S)|0;Z=iI(S)|0;c[(c[U>>2]|0)+120>>2]=Z;Z=iI(S)|0;c[(c[U>>2]|0)+124>>2]=Z;iG(S)|0;Z=iG(S)|0;c[(c[U>>2]|0)+792>>2]=Z;Z=iI(S)|0;_=20;aa=0;io(S,_,aa)|0;if(($|0)>40){aa=ma(S,$-40|0)|0;c[(c[U>>2]|0)+100>>2]=aa;aa=A7((c[(c[U>>2]|0)+100>>2]|0)+16|0)|0;c[(c[U>>2]|0)+96>>2]=aa;aa=c[U>>2]|0;$=c[aa+96>>2]|0;_=c[aa+100>>2]|0;iE(S,$,_)|0}_=c[U>>2]|0;$=c[_+100>>2]|0;do{if(($|0)==0){ak=_}else{if((c[_+792>>2]|0)>=9){ak=_;break}aa=X+136|0;ac=c[_+96>>2]|0;ab=($|0)>1024?1024:$;NN(aa|0,ac|0,ab)|0;c[X+132>>2]=1;ak=c[U>>2]|0}}while(0);c[ak+52>>2]=Z;$=mA(253424,Z)|0;c[(c[U>>2]|0)+48>>2]=$;if((Z|0)==542266948){c[W+236>>2]=1;c[(c[U>>2]|0)+124>>2]=0;c[(c[U>>2]|0)+120>>2]=0;A6((c[U>>2]|0)+96|0);c[(c[U>>2]|0)+100>>2]=0}if((c[(c[U>>2]|0)+48>>2]|0)!=28){break}c[W+236>>2]=4}}while(0);L8035:do{if((R|0)==5927){R=0;if((l3(S,c[aj>>2]|0,Y)|0)<0){R=6017;break L7991}if((ai|0)!=0){c[W+440>>2]=1;c[(c[aj>>2]|0)+52>>2]=0}c[W+236>>2]=(c[(c[aj>>2]|0)+48>>2]|0)!=86018;$=il(S,0,0,1)|0;_=G;ab=NV(32,0,Q,T)|0;ac=NU(ab,G,$,_)|0;_=G;if((_|0)>(O|0)|(_|0)==(O|0)&ac>>>0>P>>>0){al=X+104|0}else{ac=X+104|0;c[ac>>2]=iD(S)|0;c[X+108>>2]=iG(S)|0;c[X+112>>2]=iG(S)|0;iG(S)|0;iD(S)|0;al=ac}if((c[al>>2]|0)<=1){break}ac=c[X+112>>2]|0;do{if((ac|0)!=0){_=c[X+108>>2]|0;if(((_|0)/(ac|0)|0|0)<2){break}if(((_|0)%(ac|0)|0|0)==0){break L8035}}}while(0);c[al>>2]=0}}while(0);X=il(S,0,0,1)|0;W=G;Y=NU(P,O,-24,-1)|0;ac=NU(Y,G,Q,T)|0;Y=NV(ac,G,X,W)|0;W=G;io(S,Y,W)|0;R=6017;break}if((NO(q|0,256456,16)|0)==0){W=c[o>>2]|0;Y=iG(W)|0;X=iG(W)|0;ac=iG(W)|0;U=iG(W)|0;Z=iG(W)|0;hU(e,1644488,0,Y,32);hU(e,1635528,0,X,32);hU(e,1635e3,0,ac,32);hU(e,1642680,0,U,32);U=Z;ac=(Z|0)<0|0?-1:0;io(W,U,ac)|0;R=6017;break}if((NO(q|0,256160,16)|0)==0){ac=c[m>>2]|0;U=c[o>>2]|0;W=iG(U)|0;if((W|0)<=0){R=6017;break}Z=ac+155656|0;ac=0;while(1){X=iD(U)|0;Y=iO(U,X,D,6)|0;if(Y>>>0<X>>>0){_=X-Y|0;Y=0;io(U,_,Y)|0;}if((ac|0)<128){Y=Z+(ac*6|0)|0;y_(Y,D,6)|0}Y=ac+1|0;if((Y|0)<(W|0)){ac=Y}else{R=6017;break L7991}}}if((NO(q|0,256256,16)|0)==0){ac=c[o>>2]|0;W=c[m>>2]|0;Z=iG(ac)|0;if((Z|0)<=0){R=6017;break}U=W+154632|0;S=W+154636|0;W=0;while(1){T=iG(ac)|0;Q=(T&1)+T|0;T=iO(ac,Q,E,1024)|0;if((T|0)<(Q|0)){Y=Q-T|0;T=Y;Q=(Y|0)<0|0?-1:0;io(ac,T,Q)|0;}Q=iG(ac)|0;T=iG(ac)|0;if((Q|0)==0){am=(T&1)+T|0}else{am=T}do{if((a8(E|0,1638320)|0)==0){T=c[o>>2]|0;if((Q|0)==3){an=iI(T)|0}else if((Q|0)==4){Y=iJ(T)|0;an=Y}else if((Q|0)==5){an=iG(T)|0}else if((Q|0)==2){an=iI(T)|0}else{an=-2147483648}c[U>>2]=an}else{if((a8(E|0,1636200)|0)!=0){hU(e,E,Q,am,32);break}T=c[o>>2]|0;if((Q|0)==3){ao=iI(T)|0}else if((Q|0)==4){Y=iJ(T)|0;ao=Y}else if((Q|0)==5){ao=iG(T)|0}else if((Q|0)==2){ao=iI(T)|0}else{ao=-2147483648}c[S>>2]=ao}}while(0);Q=W+1|0;if((Q|0)<(Z|0)){W=Q}else{R=6017;break L7991}}}if((NO(q|0,255992,16)|0)==0){hT(e);R=6017;break}if((NO(q|0,255976,16)|0)==0){hT(e);R=6017;break}if((NO(q|0,256272,16)|0)==0){W=c[m>>2]|0;Z=c[o>>2]|0;iJ(Z)|0;iJ(Z)|0;S=iI(Z)|0;iI(Z)|0;iI(Z)|0;iI(Z)|0;iI(Z)|0;iI(Z)|0;iI(Z)|0;iI(Z)|0;U=iG(Z)|0;ac=iG(Z)|0;Q=U>>>0<128;if(Q){b[W+520+(U*1200|0)+128>>1]=ac&65535}iJ(Z)|0;ac=iG(Z)|0;T=iG(Z)|0;if(Q){c[W+154120+(U<<2)>>2]=S;c[W+520+(U*1200|0)+1160>>2]=0}if((ac|0)>0){S=0;do{iG(Z)|0;Y=iG(Z)|0;io(Z,Y,(Y|0)<0|0?-1:0)|0;S=S+1|0;}while((S|0)<(ac|0))}if((T|0)<=0){break}ac=W+520|0;S=ac+(U*1200|0)+1160|0;if(Q){ap=0}else{Y=0;while(1){l0(Z,h);iG(Z)|0;_=iI(Z)|0;io(Z,_,(_|0)<0|0?-1:0)|0;Y=Y+1|0;if((Y|0)>=(T|0)){break L7991}}}while(1){l0(Z,h);Y=iG(Z)|0;Q=iI(Z)|0;io(Z,Q,(Q|0)<0|0?-1:0)|0;if(ap>>>0<8){a[ac+(U*1200|0)+1164+(ap<<2)|0]=a[B]|0;b[ac+(U*1200|0)+1164+(ap<<2)+2>>1]=Y&65535;AN(A,48,1640016,(V=i,i=i+16|0,c[V>>2]=d[B]|0,c[V+8>>2]=Y&65535,V)|0);i=V;c[S>>2]=(c[S>>2]|0)+1}ap=ap+1|0;if((ap|0)>=(T|0)){break L7991}}}if((NO(q|0,256224,16)|0)==0){l0(p,j);T=6;S=0;io(p,T,S)|0;break}if((NO(q|0,256144,16)|0)==0){S=c[o>>2]|0;iJ(S)|0;iJ(S)|0;T=iI(S)|0;iG(S)|0;U=iG(S)|0;if((U|0)>0){ac=0;do{iD(S)|0;ac=ac+1|0;}while((ac|0)<(U|0))}if((T|0)>0){aq=0}else{R=6017;break}while(1){iJ(S)|0;U=iJ(S)|0;ac=G;iG(S)|0;iI(S)|0;iI(S)|0;Z=iI(S)|0;Y=iO(S,Z<<1,E,1024)|0;if((Y|0)<(Z|0)){Q=Z-Y|0;Y=Q;Z=(Q|0)<0|0?-1:0;io(S,Y,Z)|0;}c[F>>2]=1;c[H>>2]=1e7;mM(e,aq,g,U,ac,0,-2147483648,E)|0;ac=aq+1|0;if((ac|0)<(T|0)){aq=ac}else{R=6017;break L7991}}}if((iq(p)|0)!=0){r=-541478725;R=6040;break L7988}if((c[I>>2]|0)!=0){R=6017;break}if((NO(q|0,256440,16)|0)==0){AN(A,24,1679288,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V;T=iI(p)|0;AN(A,48,1677400,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V;mb(p,k,T)|0;mR(A,48,c[J>>2]|0,c[K>>2]|0);nJ(k);hU(e,1675432,-1,iI(p)|0,32);hU(e,1673720,-1,iI(p)|0,32);hU(e,1671840,-1,iI(p)|0,32);R=6017;break}if((NO(q|0,256320,16)|0)==0){AN(A,24,1669872,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V;zK(L,1668184,1666896,0)|0;R=6017;break}if((NO(q|0,256336,16)|0)!=0){R=6017;break}AN(A,32,1665720,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V;R=6017}}while(0);if((R|0)==6017){R=0;T=il(p,0,0,1)|0;S=G;ac=NU(P,O,N,M)|0;U=G;if(!((T|0)==(ac|0)&(S|0)==(U|0))){S=il(p,0,0,1)|0;T=NV(S,G,N,M)|0;S=G;AN(A,48,1664176,(V=i,i=i+32|0,c[V>>2]=T,c[V+8>>2]=S,c[V+16>>2]=P,c[V+24>>2]=O,V)|0);i=V}il(p,ac,U,0)|0;}U=il(p,0,0,1)|0;ac=G;l0(p,j);S=iJ(p)|0;T=G;if((NO(q|0,256424,16)|0)==0){x=T;y=S;break L7986}else{M=ac;N=U;O=T;P=S}}if((R|0)==6044){i=f;return r|0}else if((R|0)==6045){i=f;return r|0}else if((R|0)==6040){i=f;return r|0}}}while(0);q=il(p,0,0,1)|0;V=n+156440|0;c[V>>2]=q;c[V+4>>2]=G;V=0;if((c[n+156520>>2]&1|0)==0&((x|0)>(V|0)|(x|0)==(V|0)&y>>>0>99>>>0)){V=NU(y,x,-24,-1)|0;x=n+156448|0;c[x>>2]=V;c[x+4>>2]=G}else{x=n+156448|0;c[x>>2]=-1;c[x+4>>2]=-1}l0(p,j);iJ(p)|0;iD(p)|0;iD(p)|0;if((iq(p)|0)!=0){r=-541478725;i=f;return r|0}j=il(p,0,0,1)|0;p=n+156432|0;c[p>>2]=j;c[p+4>>2]=G;c[n+156424>>2]=0;p=e+28|0;j=n+154120|0;x=n+154632|0;V=x;y=n+520|0;q=n+155656|0;k=l|0;aq=l+1|0;g=l+2|0;l=x;x=n+154636|0;n=0;do{o=c[t+(n<<2)>>2]|0;do{if((o|0)>-1){ap=c[(c[p>>2]|0)+(o<<2)>>2]|0;h=ap+8|0;m=(c[h>>2]|0)+72|0;if((c[m>>2]|0)==0){c[m>>2]=c[j+(n<<2)>>2]}m=c[V+(n<<3)>>2]|0;do{if((m|0)>0){ao=c[V+(n<<3)+4>>2]|0;if((ao|0)<=0){R=6028;break}am=ap+80|0;an=ap+84|0;al=m;aj=(m|0)<0|0?-1:0;ai=ao;ak=(ao|0)<0|0?-1:0;ao=2147483647;ag=0;BX(am,an,al,aj,ai,ak,ao,ag)|0}else{R=6028}}while(0);do{if((R|0)==6028){R=0;m=c[l>>2]|0;if((m|0)<=0){break}ag=c[x>>2]|0;if((ag|0)<=0){break}if((c[(c[h>>2]|0)+8>>2]|0)!=0){break}ao=ap+80|0;ak=ap+84|0;ai=m;aj=(m|0)<0|0?-1:0;m=ag;al=(ag|0)<0|0?-1:0;ag=2147483647;an=0;BX(ao,ak,ai,aj,m,al,ag,an)|0}}while(0);h=b[y+(n*1200|0)+128>>1]|0;if((h&65535)>=128){break}an=h&65535;h=q+(an*6|0)|0;if((h|0)==0){break}if((NL(h|0)|0)>>>0<=1){break}a[k]=a[h]|0;a[aq]=a[q+(an*6|0)+1|0]|0;a[g]=0;an=i3(k,0)|0;if((an|0)==0){break}h=ap+88|0;zK(h,1662720,an,0)|0}}while(0);n=n+1|0;}while((n|0)<128);jx(e+1140|0,0,256008);r=0;i=f;return r|0}function hP(e,f){e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0;g=i;h=e+12|0;j=c[h>>2]|0;k=e+16|0;l=c[k>>2]|0;if((iq(l)|0)!=0){m=-541478725;i=g;return m|0}n=e;o=e+28|0;p=e+1096|0;q=e+1100|0;r=e+1076|0;s=e+1232|0;e=j+156424|0;t=j+156552|0;u=j+156604|0;v=l;l=j;L8195:while(1){j=l+156424|0;w=l+156552|0;x=l+156604|0;y=l+156616|0;z=l+156576|0;A=l+156564|0;B=l+156620|0;C=l+156560|0;D=l+156584|0;E=l+156600|0;F=l+156596|0;H=l+156592|0;I=l+156556|0;J=l+156512|0;K=l+156608|0;L=l+156572|0;while(1){M=c[j>>2]|0;if((M|0)<16){break}if((c[w>>2]|0)<1){break}L8201:do{if((c[x>>2]|0)==0){N=c[h>>2]|0;O=iD(v)|0;P=N+156552|0;c[P>>2]=(c[P>>2]|0)-1;c[N+156564>>2]=O>>7;P=O&127;O=N+156616|0;c[O>>2]=c[N+4+(P<<2)>>2];Q=N+520|0;R=N+156540|0;S=(c[R>>2]|0)>>>4&3;if((S|0)==1){c[N+156556>>2]=iD(v)|0;T=2}else if((S|0)==3){c[N+156556>>2]=iI(v)|0;T=5}else if((S|0)==2){c[N+156556>>2]=iG(v)|0;T=3}else{c[N+156556>>2]=0;T=1}S=(c[R>>2]|0)>>>2&3;if((S|0)==3){c[N+156572>>2]=iI(v)|0;U=T+4|0}else if((S|0)==2){c[N+156572>>2]=iG(v)|0;U=T+2|0}else if((S|0)==1){c[N+156572>>2]=iD(v)|0;U=T+1|0}else{c[N+156572>>2]=0;U=T}S=c[R>>2]&3;if((S|0)==3){R=iI(v)|0;c[N+156560>>2]=R;V=U+4|0;W=R}else if((S|0)==2){R=iG(v)|0;c[N+156560>>2]=R;V=U+2|0;W=R}else if((S|0)==1){S=iD(v)|0;c[N+156560>>2]=S;V=U+1|0;W=S}else{c[N+156560>>2]=0;V=U;W=0}S=N+156560|0;R=NU(V,(V|0)<0|0?-1:0,W,(W|0)<0|0?-1:0)|0;X=G;Y=N+156424|0;Z=c[Y>>2]|0;_=(Z|0)<0|0?-1:0;L8222:do{if((X|0)>(_|0)|(X|0)==(_|0)&R>>>0>Z>>>0){AN(n,16,1691576,($=i,i=i+8|0,c[$>>2]=W,$)|0);i=$}else{do{if((W|0)>7){aa=il(v,0,0,1)|0;ab=c[S>>2]|0;ad=NU(ab,(ab|0)<0|0?-1:0,aa,G)|0;aa=G;ab=iI(v)|0;ae=N+156596|0;c[ae>>2]=ab;if((ab-1|0)>>>0>16777214){AN(n,16,1689448,($=i,i=i+1|0,i=i+7&-8,c[$>>2]=0,$)|0);i=$;c[ae>>2]=0;break L8222}ae=N+156584|0;c[ae>>2]=iI(v)|0;c[ae+4>>2]=0;ab=Q+(P*1200|0)+1160|0;L8234:do{if((c[ab>>2]|0)>0){af=0;while(1){ag=b[Q+(P*1200|0)+1164+(af<<2)+2>>1]|0;if(ag<<16>>16==-1){ah=iG(v)|0}else{ah=ag&65535}ag=il(v,0,0,1)|0;ai=NU(ag,G,ah,(ah|0)<0|0?-1:0)|0;ag=G;if((ag|0)>(aa|0)|(ag|0)==(aa|0)&ai>>>0>ad>>>0){break}aj=d[Q+(P*1200|0)+1164+(af<<2)|0]|0;do{if((aj|0)==84){ak=iD(v)|0;al=iD(v)|0;if(!((ak|0)>0&(al|0)>0)){break}am=c[O>>2]|0;if((am|0)<=-1){break}an=c[(c[o>>2]|0)+(am<<2)>>2]|0;c[an+80>>2]=ak;c[an+84>>2]=al}else if((aj|0)==42){io(v,8,0)|0;al=iJ(v)|0;an=G;iJ(v)|0;if((al|0)==(-1|0)&(an|0)==(-1|0)){c[ae>>2]=0;c[ae+4>>2]=-2147483648;break}else{ak=N0(al,an,1e4,0)|0;c[ae>>2]=ak;c[ae+4>>2]=G;break}}}while(0);il(v,ai,ag,0)|0;af=af+1|0;if((af|0)>=(c[ab>>2]|0)){break L8234}}AN(n,16,1686712,($=i,i=i+1|0,i=i+7&-8,c[$>>2]=0,$)|0);i=$}}while(0);il(v,ad,aa,0)|0;ao=(c[S>>2]|0)+V|0}else{if((W|0)==1){ab=N+156572|0;c[N+156604>>2]=c[ab>>2];c[ab>>2]=0;ab=c[N+156544>>2]|0;ae=N+156584|0;c[ae>>2]=ab;c[ae+4>>2]=(ab|0)<0|0?-1:0;c[N+156600>>2]=iD(v)|0;ao=V+1|0;break}else if((W|0)==0){ao=V;break}else{AN(n,16,1685072,($=i,i=i+8|0,c[$>>2]=W,$)|0);i=$;break L8222}}}while(0);do{if((c[N+156536>>2]&1|0)==0){ab=c[Y>>2]|0;ae=ab-ao|0;c[N+156576>>2]=ae;ap=ao;aq=ae;ar=ab}else{ab=(c[N+156548>>2]|0)>>>6&3;if((ab|0)==2){ae=iG(v)|0;c[N+156576>>2]=ae;as=ao+2|0;at=ae}else if((ab|0)==3){ae=iI(v)|0;c[N+156576>>2]=ae;as=ao+4|0;at=ae}else if((ab|0)==1){ab=iD(v)|0;c[N+156576>>2]=ab;as=ao+1|0;at=ab}else{c[N+156576>>2]=0;as=ao;at=0}ab=c[Y>>2]|0;if((as|0)>(ab|0)){AN(n,16,1682976,($=i,i=i+1|0,i=i+7&-8,c[$>>2]=0,$)|0);i=$;break L8222}ae=ab-as|0;if(at>>>0<=ae>>>0){ap=as;aq=at;ar=ab;break}af=N+156568|0;aj=c[af>>2]|0;if(at>>>0>(aj+ae|0)>>>0){AN(n,16,1681160,($=i,i=i+16|0,c[$>>2]=ab,c[$+8>>2]=as,$)|0);i=$;break L8222}else{ak=at-ae|0;ae=ak+ab|0;c[Y>>2]=ae;c[af>>2]=aj-ak;ap=as;aq=at;ar=ae;break}}}while(0);if((c[S>>2]|0)==1){c[N+156592>>2]=aq;if((aq|0)>(ar|0)){break}}c[Y>>2]=ar-ap;ae=c[y>>2]|0;do{if((ae|0)>=0){ak=c[(c[o>>2]|0)+(ae<<2)>>2]|0;aj=c[ak+76>>2]|0;if((aj|0)>47){break}if((c[A>>2]|0)==0&(aj|0)>31){break}aj=c[ak+12>>2]|0;c[B>>2]=aj;au=aj;av=6117;break L8201}}while(0);c[x>>2]=0;ae=c[z>>2]|0;aj=0;io(v,ae,aj)|0;aj=c[z>>2]|0;c[j>>2]=(c[j>>2]|0)-aj;if((c[y>>2]|0)>=0){break L8201}AN(n,16,1716272,($=i,i=i+8|0,c[$>>2]=aj,$)|0);i=$;break L8201}}while(0);c[w>>2]=0}else{au=c[B>>2]|0;av=6117}}while(0);L8280:do{if((av|0)==6117){av=0;if((au|0)==0){av=6118;break L8195}do{if((c[C>>2]|0)==1){Y=c[x>>2]|0;c[D>>2]=Y;c[D+4>>2]=(Y|0)<0|0?-1:0;c[x>>2]=(c[E>>2]|0)+Y;Y=iD(v)|0;c[z>>2]=Y;c[F>>2]=Y;c[j>>2]=(c[j>>2]|0)-1;N=(c[H>>2]|0)-1|0;c[H>>2]=N;if((N|0)<(Y|0)){c[x>>2]=0;S=N;O=(N|0)<0|0?-1:0;io(v,S,O)|0;c[j>>2]=(c[j>>2]|0)-(c[H>>2]|0);break L8280}else{c[H>>2]=N-Y;aw=Y;ax=Y;break}}else{aw=c[z>>2]|0;ax=c[F>>2]|0}}while(0);ay=au+88|0;Y=c[ay>>2]|0;N=Y+aw|0;az=au+8|0;aA=au+36|0;O=c[aA>>2]|0;if(N>>>0<=O>>>0&N>>>0>ax>>>0){AN(n,32,1712656,($=i,i=i+32|0,c[$>>2]=Y,c[$+8>>2]=aw,c[$+16>>2]=ax,c[$+24>>2]=O,$)|0);i=$;Y=c[aA>>2]|0;c[F>>2]=Y;aB=c[aA>>2]|0;aC=Y}else{aB=O;aC=ax}if((aB|0)==(aC|0)){if(((c[z>>2]|0)+(c[ay>>2]|0)|0)>>>0>aC>>>0){av=6128}}else{av=6128}do{if((av|0)==6128){av=0;O=au+32|0;if((c[O>>2]|0)==0){aD=aC}else{AN(n,32,1708976,($=i,i=i+16|0,c[$>>2]=aB,c[$+8>>2]=aC,$)|0);i=$;c[ay>>2]=0;nJ(az);aD=c[F>>2]|0}nC(az,aD)|0;a[au+4|0]=c[I>>2]&255;Y=NV(c[D>>2]|0,c[D+4>>2]|0,c[J>>2]|0,0)|0;N=au+24|0;c[N>>2]=Y;c[N+4>>2]=G;c[au+40>>2]=c[y>>2];N=c[K>>2]|0;Y=c[K+4>>2]|0;S=au+120|0;c[S>>2]=N;c[S+4>>2]=Y;S=au+72|0;c[S>>2]=N;c[S+4>>2]=Y;do{if((c[O>>2]|0)!=0){Y=au+132|0;if((c[Y>>2]|0)==0){break}S=nK(az,0,1024)|0;if((S|0)==0){AN(n,16,1706880,($=i,i=i+1|0,i=i+7&-8,c[$>>2]=0,$)|0);i=$;break}else{N=au+136|0;NN(S|0,N|0,1024)|0;c[Y>>2]=0;break}}}while(0);if((c[(c[(c[(c[o>>2]|0)+(c[y>>2]<<2)>>2]|0)+8>>2]|0)+8>>2]|0)==1){c[A>>2]=1}else{if((c[A>>2]|0)==0){break}}O=au+44|0;c[O>>2]=c[O>>2]|1}}while(0);O=c[z>>2]|0;Y=(c[j>>2]|0)-O|0;c[j>>2]=Y;if((Y|0)<0){break}Y=c[L>>2]|0;N=c[aA>>2]|0;if(Y>>>0>=N>>>0|O>>>0>(N-Y|0)>>>0){AN(n,16,1704848,($=i,i=i+24|0,c[$>>2]=Y,c[$+8>>2]=O,c[$+16>>2]=N,$)|0);i=$;break}aE=au+32|0;N=iE(v,(c[aE>>2]|0)+Y|0,O)|0;O=c[z>>2]|0;do{if((N|0)==(O|0)){aF=N}else{if((N|0)<0){m=N;av=6219;break L8195}Y=(c[L>>2]|0)+N|0;if((Y|0)==0){m=-541478725;av=6220;break L8195}if((c[au+104>>2]|0)>1){NM((c[aE>>2]|0)+Y|0,0,O-N|0);aF=c[z>>2]|0;break}else{nE(az,Y);aF=N;break}}}while(0);N=c[p>>2]|0;do{if((N|0)!=0){if((c[q>>2]|0)!=20){break}nd(N,(c[aE>>2]|0)+(c[L>>2]|0)|0,aF)}}while(0);aG=(c[ay>>2]|0)+aF|0;c[ay>>2]=aG;if((aG|0)!=(c[aA>>2]|0)){break}if((c[(c[(c[(c[o>>2]|0)+(c[y>>2]<<2)>>2]|0)+8>>2]|0)+48>>2]|0)==2&(aG|0)>100){aH=0}else{av=6157;break L8195}while(1){if((aH|0)>=(aG|0)){break}if((a[(c[aE>>2]|0)+aH|0]|0)==0){aH=aH+1|0}else{break}}if((aH|0)!=(aG|0)){av=6157;break L8195}AN(n,48,1702800,($=i,i=i+1|0,i=i+7&-8,c[$>>2]=0,$)|0);i=$;c[ay>>2]=0;nJ(az)}}while(0);if((iq(v)|0)!=0){m=-541478725;av=6217;break L8195}}y=(c[l+156568>>2]|0)+M|0;if((y|0)<=-1){av=6052;break}io(v,y,(y|0)<0|0?-1:0)|0;y=il(v,0,0,1)|0;L=G;c[K>>2]=y;c[K+4>>2]=L;z=l+156448|0;j=c[z>>2]|0;A=c[z+4>>2]|0;if(!((j|0)==(-1|0)&(A|0)==(-1|0))){z=l+156440|0;J=NV(y,L,c[z>>2]|0,c[z+4>>2]|0)|0;z=G;if(!(z>>>0<A>>>0|z>>>0==A>>>0&J>>>0<j>>>0)){m=-541478725;av=6221;break}}j=c[k>>2]|0;J=c[h>>2]|0;do{if((c[J+156624>>2]|0)==0){if((c[r>>2]|0)==0){aI=32768;aJ=-1;aK=-1;aL=-1;break}A=il(j,0,0,1)|0;z=NV(A,G,c[s>>2]|0,c[s+4>>2]|0)|0;A=N1(z,G,c[r>>2]|0,0)|0;z=NU(A,G,3,0)|0;aI=z;aJ=-1;aK=-1;aL=-1}else{aI=3;aJ=-1;aK=-1;aL=-1}}while(0);while(1){if((aI|0)<=0){av=6176;break}K=iD(j)|0;if((aK|0)==130&(aJ|0)==0&(K|0)==0){aM=0;aN=0;av=6181;break}else{aI=aI-1|0;aL=aK;aK=aJ;aJ=K}}do{if((av|0)==6176){av=0;if((aL|0)==130){aM=aK;aN=aJ;av=6181;break}if((c[j+76>>2]|0)==-11){av=6212;break}if((iq(j)|0)==0){K=il(j,0,0,1)|0;z=G;AN(n,16,1744256,($=i,i=i+24|0,c[$>>2]=aL,c[$+8>>2]=K,c[$+16>>2]=z,$)|0);i=$}if((aL&143|0)==130){aM=aK;aN=aJ;av=6181;break}if((iq(j)|0)!=0){aO=aK;aP=aL;aQ=8;av=6187;break}z=-1;K=-1;il(j,z,K,1)|0;aO=aK;aP=aL;aQ=8;av=6187}}while(0);do{if((av|0)==6181){av=0;if((aM|aN|0)==0){K=iD(j)|0;aO=iD(j)|0;aP=K;aQ=11;av=6187;break}if((iq(j)|0)!=0){av=6212;break}AN(n,16,1737776,($=i,i=i+1|0,i=i+7&-8,c[$>>2]=0,$)|0);i=$;av=6212}}while(0);do{if((av|0)==6187){av=0;K=J+156536|0;c[K>>2]=aP;c[J+156540>>2]=aO;z=aP>>>5&3;if((z|0)==3){aR=aQ|4;aS=iI(j)|0}else if((z|0)==2){aR=aQ+2|0;aS=iG(j)|0}else if((z|0)==1){aR=aQ+1|0;aS=iD(j)|0}else{aR=aQ;aS=c[r>>2]|0}z=(c[K>>2]|0)>>>1&3;if((z|0)==3){iI(j)|0;aT=aR+4|0}else if((z|0)==2){iG(j)|0;aT=aR+2|0}else if((z|0)==1){iD(j)|0;aT=aR+1|0}else{aT=aR}z=(c[K>>2]|0)>>>3&3;if((z|0)==3){aU=aT+4|0;aV=iI(j)|0}else if((z|0)==2){aU=aT+2|0;aV=iG(j)|0}else if((z|0)==1){aU=aT+1|0;aV=iD(j)|0}else{aU=aT;aV=0}if((aS|0)==0|aS>>>0>536870911){z=il(j,0,0,1)|0;A=G;AN(n,16,1733160,($=i,i=i+24|0,c[$>>2]=aS,c[$+8>>2]=z,c[$+16>>2]=A,$)|0);i=$;av=6212;break}if(aV>>>0>=aS>>>0){A=il(j,0,0,1)|0;z=G;AN(n,16,1728016,($=i,i=i+24|0,c[$>>2]=aV,c[$+8>>2]=A,c[$+16>>2]=z,$)|0);i=$;av=6212;break}c[J+156544>>2]=iI(j)|0;iG(j)|0;if((c[K>>2]&1|0)==0){c[J+156552>>2]=1;c[J+156548>>2]=128;aW=aU}else{K=iD(j)|0;c[J+156548>>2]=K;c[J+156552>>2]=K&63;aW=aU+1|0}K=aS-aV|0;if(aW>>>0>K>>>0){c[J+156424>>2]=0;z=il(j,0,0,1)|0;A=G;AN(n,16,1723784,($=i,i=i+40|0,c[$>>2]=aW,c[$+8>>2]=aS,c[$+16>>2]=aV,c[$+24>>2]=z,c[$+32>>2]=A,$)|0);i=$;av=6212;break}c[J+156424>>2]=K-aW;K=c[J+156524>>2]|0;if(aS>>>0<K>>>0){aX=aV-aS+K|0}else{aX=aV}c[J+156568>>2]=aX}}while(0);do{if((av|0)==6212){av=0;if((c[e>>2]|0)<16){break}if((c[t>>2]|0)>=1){av=6214;break L8195}}}while(0);c[u>>2]=0;J=c[k>>2]|0;j=c[h>>2]|0;if((iq(J)|0)==0){v=J;l=j}else{m=-541478725;av=6218;break}}if((av|0)==6118){AN(0,0,1678312,($=i,i=i+24|0,c[$>>2]=1661960,c[$+8>>2]=1645264,c[$+16>>2]=1155,$)|0);i=$;cw();return 0}else if((av|0)==6052){cu(1720248,1645264,1124,1913512);return 0}else if((av|0)==6214){cu(1615224,1645264,1336,1913496);return 0}else if((av|0)==6157){l=au+104|0;v=c[l>>2]|0;do{if((v|0)>1){h=au+108|0;k=c[h>>2]|0;if((aG|0)!=(ac(k,v)|0)){AN(n,16,1700136,($=i,i=i+24|0,c[$>>2]=aG,c[$+8>>2]=k,c[$+16>>2]=v,$)|0);i=$;break}k=zg(aG+16|0)|0;if((k|0)==0){break}u=k+4|0;t=c[u>>2]|0;NM(t+(c[aA>>2]|0)|0,0,16);e=c[aA>>2]|0;L8410:do{if((e|0)>0){aX=au+112|0;aV=0;aS=e;aW=c[aX>>2]|0;while(1){aU=(aV|0)/(aW|0)|0;aT=c[l>>2]|0;aR=((ac(c[h>>2]|0,(aU|0)%(aT|0)|0)|0)/(aW|0)|0)+((aU|0)/(aT|0)|0)|0;if((aV+aW|0)>(aS|0)){av=6164;break}if((aR+1|0)>((aS|0)/(aW|0)|0|0)){av=6166;break}aT=t+aV|0;aU=c[aE>>2]|0;r=aU+(ac(aR,aW)|0)|0;NN(aT|0,r|0,aW)|0;aW=c[aX>>2]|0;aV=aW+aV|0;aS=c[aA>>2]|0;if((aV|0)>=(aS|0)){break L8410}}if((av|0)==6166){cu(1695208,1645264,1296,1913512);return 0}else if((av|0)==6164){cu(1697408,1645264,1295,1913512);return 0}}}while(0);t=az|0;zj(t);c[t>>2]=k;c[aE>>2]=c[u>>2]}}while(0);c[ay>>2]=0;ay=f;f=az;NN(ay|0,f|0,80)|0;c[au+60>>2]=0;c[az>>2]=0;c[aA>>2]=0;c[aE>>2]=0;c[au+52>>2]=0;c[au+48>>2]=0;m=0;i=g;return m|0}else if((av|0)==6217){i=g;return m|0}else if((av|0)==6218){i=g;return m|0}else if((av|0)==6219){i=g;return m|0}else if((av|0)==6220){i=g;return m|0}else if((av|0)==6221){i=g;return m|0}return 0}function hQ(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;d=c[b+12>>2]|0;c[d+156424>>2]=0;e=b+24|0;NM(d+156536|0,0,44);NM(d+156584|0,0,24);f=c[e>>2]|0;if((f|0)==0){g=d+156620|0;h=g;c[h>>2]=0;return 0}i=b+28|0;b=0;j=f;while(1){f=c[(c[(c[i>>2]|0)+(b<<2)>>2]|0)+12>>2]|0;if((f|0)==0){k=j}else{nJ(f+8|0);c[f+88>>2]=0;a[f+4|0]=0;k=c[e>>2]|0}f=b+1|0;if(f>>>0<k>>>0){b=f;j=k}else{break}}g=d+156620|0;h=g;c[h>>2]=0;return 0}function hR(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0;h=i;i=i+16|0;j=h|0;k=b+12|0;l=c[k>>2]|0;m=b+28|0;n=c[(c[m>>2]|0)+(d<<2)>>2]|0;o=b+1076|0;if((c[o>>2]|0)==0){p=-1;i=h;return p|0}q=b+16|0;r=c[q>>2]|0;do{if((r|0)!=0){s=iY(r,d,e,f,g)|0;t=s;if((t|0)>-1){s=c[k>>2]|0;c[s+156424>>2]=0;u=b+24|0;NM(s+156536|0,0,44);NM(s+156584|0,0,24);v=c[u>>2]|0;if((v|0)!=0){w=0;x=v;while(1){v=c[(c[(c[m>>2]|0)+(w<<2)>>2]|0)+12>>2]|0;if((v|0)==0){y=x}else{nJ(v+8|0);c[v+88>>2]=0;a[v+4|0]=0;y=c[u>>2]|0}v=w+1|0;if(v>>>0<y>>>0){w=v;x=y}else{break}}}c[s+156620>>2]=0}if((t|0)==-38){break}else{p=t}i=h;return p|0}}while(0);y=l+156456|0;l=c[y>>2]|0;if((l|0)==0){r=j|0;x=c[k>>2]|0;w=il(c[q>>2]|0,0,0,1)|0;u=G;v=c[q>>2]|0;z=x+156440|0;A=x+156448|0;B=NU(c[A>>2]|0,c[A+4>>2]|0,c[z>>2]|0,c[z+4>>2]|0)|0;z=il(v,B,G,0)|0;B=G;v=0;L8457:do{if((B|0)<(v|0)|(B|0)==(v|0)&z>>>0<0>>>0){c[x+156456>>2]=-1}else{l0(c[q>>2]|0,j);A=(NO(r|0,255888,16)|0)==0;C=iJ(c[q>>2]|0)|0;L8460:do{if(!A){D=G;E=C;while(1){F=0;if((D|0)<(F|0)|(D|0)==(F|0)&E>>>0<24>>>0){break}if((iq(c[q>>2]|0)|0)!=0){break}F=c[q>>2]|0;H=NU(E,D,-24,-1)|0;io(F,H,G)|0;l0(c[q>>2]|0,j);H=(NO(r|0,255888,16)|0)==0;F=iJ(c[q>>2]|0)|0;if(H){break L8460}else{D=G;E=F}}il(c[q>>2]|0,w,u,0)|0;c[x+156456>>2]=-1;break L8457}}while(0);l0(c[q>>2]|0,j);C=iJ(c[q>>2]|0)|0;A=G;t=iI(c[q>>2]|0)|0;s=iI(c[q>>2]|0)|0;E=b;AN(E,48,1625872,(I=i,i=i+32|0,c[I>>2]=C,c[I+8>>2]=A,c[I+16>>2]=t,c[I+24>>2]=s,I)|0);i=I;if((s|0)>0){t=b+1232|0;D=x+156512|0;F=0;H=-1;J=-1;while(1){K=iI(c[q>>2]|0)|0;L=iG(c[q>>2]|0)|0;M=c[t>>2]|0;N=c[t+4>>2]|0;O=N2(c[o>>2]|0,0,K,(K|0)<0|0?-1:0)|0;P=NU(O,G,M,N)|0;N=G;M=AW(C,A,F,(F|0)<0|0?-1:0,1e4,0)|0;O=NV(M,G,c[D>>2]|0,0)|0;M=G;Q=0;R=(M|0)>(Q|0)|(M|0)==(Q|0)&O>>>0>0>>>0;Q=R?O:0;O=R?M:0;if((P|0)==(J|0)&(N|0)==(H|0)){S=H;T=J}else{AN(E,48,1619320,(I=i,i=i+32|0,c[I>>2]=K,c[I+8>>2]=L,c[I+16>>2]=Q,c[I+24>>2]=O,I)|0);i=I;L=c[(c[m>>2]|0)+(d<<2)>>2]|0;K=c[o>>2]|0;mp(L,P,N,Q,O,K,0,1)|0;S=N;T=P}P=F+1|0;if((P|0)<(s|0)){F=P;H=S;J=T}else{break}}}c[x+156456>>2]=(s|0)>1;J=c[q>>2]|0;il(J,w,u,0)|0;}}while(0);U=c[y>>2]|0}else{U=l}do{if((U|0)>0){l=n+400|0;if((c[l>>2]|0)==0){break}y=mt(n,e,f,g)|0;if((y|0)<=-1){break}u=(c[l>>2]|0)+(y*24|0)|0;y=c[u>>2]|0;l=c[u+4>>2]|0;AN(b,48,1632344,(I=i,i=i+16|0,c[I>>2]=y,c[I+8>>2]=l,I)|0);i=I;u=il(c[q>>2]|0,y,l,0)|0;l=G;y=0;if((l|0)<(y|0)|(l|0)==(y|0)&u>>>0<0>>>0){p=-1;i=h;return p|0}u=c[k>>2]|0;c[u+156424>>2]=0;y=b+24|0;NM(u+156536|0,0,44);NM(u+156584|0,0,24);l=c[y>>2]|0;if((l|0)!=0){w=0;x=l;while(1){l=c[(c[(c[m>>2]|0)+(w<<2)>>2]|0)+12>>2]|0;if((l|0)==0){V=x}else{nJ(l+8|0);c[l+88>>2]=0;a[l+4|0]=0;V=c[y>>2]|0}l=w+1|0;if(l>>>0<V>>>0){w=l;x=V}else{break}}}c[u+156620>>2]=0;p=0;i=h;return p|0}}while(0);if((mu(b,d,e,f,g)|0)<0){p=-1;i=h;return p|0}g=c[k>>2]|0;c[g+156424>>2]=0;k=b+24|0;NM(g+156536|0,0,44);NM(g+156584|0,0,24);b=c[k>>2]|0;if((b|0)!=0){f=0;e=b;while(1){b=c[(c[(c[m>>2]|0)+(f<<2)>>2]|0)+12>>2]|0;if((b|0)==0){W=e}else{nJ(b+8|0);c[b+88>>2]=0;a[b+4|0]=0;W=c[k>>2]|0}b=f+1|0;if(b>>>0<W>>>0){f=b;e=W}else{break}}}c[g+156620>>2]=0;p=0;i=h;return p|0}function hS(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;g=i;i=i+1096|0;f=g|0;h=g+80|0;j=c[e>>2]|0;k=c[e+4>>2]|0;l=b+24|0;m=c[l>>2]|0;if((m|0)!=0){n=0;do{o=h+(n<<3)|0;c[o>>2]=j;c[o+4>>2]=k;n=n+1|0;}while(n>>>0<m>>>0)}m=c[b+1076>>2]|0;if((m|0)==0){p=k;q=j}else{n=m;m=0;o=b+1232|0;r=c[o>>2]|0;s=c[o+4>>2]|0;o=NU(j,k,-1,-1)|0;k=NU(o,G,n,m)|0;o=NV(k,G,r,s)|0;k=G;j=N1(o,k,n,m)|0;m=NV(r,s,j,G)|0;j=NU(m,G,o,k)|0;p=G;q=j}c[e>>2]=q;c[e+4>>2]=p;j=il(c[b+16>>2]|0,q,p,0)|0;p=G;q=0;if((p|0)<(q|0)|(p|0)==(q|0)&j>>>0<0>>>0){t=-2147483648;u=0;i=g;return(G=t,u)|0}j=c[b+12>>2]|0;c[j+156424>>2]=0;NM(j+156536|0,0,44);NM(j+156584|0,0,24);q=c[l>>2]|0;if((q|0)!=0){p=b+28|0;k=0;o=q;while(1){q=c[(c[(c[p>>2]|0)+(k<<2)>>2]|0)+12>>2]|0;if((q|0)==0){v=o}else{nJ(q+8|0);c[q+88>>2]=0;a[q+4|0]=0;v=c[l>>2]|0}q=k+1|0;if(q>>>0<v>>>0){k=q;o=v}else{break}}}c[j+156620>>2]=0;L8523:do{if((mn(b,f)|0)>=0){j=f+16|0;v=f+36|0;o=f+32|0;k=b+28|0;l=f+28|0;while(1){w=c[j>>2]|0;x=c[j+4>>2]|0;nJ(f);if((c[v>>2]&1|0)!=0){p=c[o>>2]|0;q=c[(c[k>>2]|0)+(p<<2)>>2]|0;m=c[q+12>>2]|0;if((m|0)==0){y=6292;break}s=m+120|0;z=c[s>>2]|0;A=c[s+4>>2]|0;m=c[l>>2]|0;r=h+(p<<3)|0;p=c[r>>2]|0;n=c[r+4>>2]|0;B=NU(z,A,1,0)|0;C=NV(B,G,p,n)|0;n=C;mp(q,z,A,w,x,m,n,1)|0;n=NU(c[s>>2]|0,c[s+4>>2]|0,1,0)|0;c[r>>2]=n;c[r+4>>2]=G;if((c[o>>2]|0)==(d|0)){break}}if((mn(b,f)|0)<0){break L8523}}if((y|0)==6292){AN(0,0,1678312,(D=i,i=i+24|0,c[D>>2]=1661960,c[D+8>>2]=1645264,c[D+16>>2]=1422,D)|0);i=D;cw();return 0}c[e>>2]=z;c[e+4>>2]=A;t=x;u=w;i=g;return(G=t,u)|0}}while(0);AN(b,32,1701560,(D=i,i=i+1|0,i=i+7&-8,c[D>>2]=0,D)|0);i=D;t=-2147483648;u=0;i=g;return(G=t,u)|0}function hT(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;b=i;i=i+1024|0;d=a+16|0;e=c[d>>2]|0;f=c[a+12>>2]|0;g=iG(e)|0;if((g|0)<=0){i=b;return}h=b|0;j=f+154632|0;f=0;do{iG(e)|0;k=iG(e)|0;l=iG(e)|0;m=iG(e)|0;n=iI(e)|0;o=iO(e,l,h,1024)|0;if((o|0)<(l|0)){p=l-o|0;o=p;l=(p|0)<0|0?-1:0;io(e,o,l)|0;}do{if((a8(h|0,1638320)|0)==0){l=c[d>>2]|0;if((m|0)==4){o=iJ(l)|0;q=o}else if((m|0)==3){q=iI(l)|0}else if((m|0)==5){q=iG(l)|0}else if((m|0)==2){q=iG(l)|0}else{q=-2147483648}if((k|0)>=128){break}c[j+(k<<3)>>2]=q}else{if((a8(h|0,1636200)|0)!=0){hU(a,h,m,n,16);break}l=c[d>>2]|0;if((m|0)==3){r=iI(l)|0}else if((m|0)==5){r=iG(l)|0}else if((m|0)==4){o=iJ(l)|0;r=o}else if((m|0)==2){r=iG(l)|0}else{r=-2147483648}if((k|0)>=128){break}c[j+(k<<3)+4>>2]=r}}while(0);f=f+1|0;}while((f|0)<(g|0));i=b;return}function hU(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0;h=i;i=i+168|0;j=h|0;k=h+8|0;l=h+88|0;m=h+152|0;n=h+160|0;o=b+16|0;p=il(c[o>>2]|0,0,0,1)|0;q=G;if(f>>>0>2147483635){i=h;return}r=f<<1;s=A1(r+22|0)|0;c[n>>2]=s;L8572:do{if((s|0)!=0){if((e|0)==1){if((a8(d|0,1661272)|0)!=0){if((a8(d|0,1659880)|0)!=0){AN(b,40,1658568,(t=i,i=i+8|0,c[t>>2]=d,t)|0);i=t;break}c[j>>2]=0;jm(b,1659880,j);if((c[j>>2]|0)!=0){jp(b,j)|0}jn(j);break}u=k;v=l|0;w=m;NM(u|0,0,80);c[m>>2]=0;if((f|0)<9){AN(b,16,1651592,(t=i,i=i+8|0,c[t>>2]=f,t)|0);i=t;break}x=iD(c[o>>2]|0)|0;if(x>>>0>20|(x|0)<0){AN(b,24,1650056,(t=i,i=i+8|0,c[t>>2]=x,t)|0);i=t;y=0}else{y=x}x=iI(c[o>>2]|0)|0;z=f-5|0;A=z-(iO(c[o>>2]|0,z,v,64)|0)|0;z=249312;while(1){B=c[z+32>>2]|0;if((B|0)==0){C=6338;break}if((ba(z|0,v|0,64)|0)==0){break}else{z=z+36|0}}if((C|0)==6338){AN(b,16,1648376,(t=i,i=i+8|0,c[t>>2]=v,t)|0);i=t;break}if((x|0)>=(A|0)){AN(b,16,1645968,(t=i,i=i+16|0,c[t>>2]=x,c[t+8>>2]=A,t)|0);i=t;break}z=A-x|0;D=z<<1|1;E=A1(D)|0;c[m>>2]=E;if((E|0)==0){break}iO(c[o>>2]|0,z,E,D)|0;do{if((mb(c[o>>2]|0,k,x)|0)>=0){D=mK(b,0)|0;if((D|0)==0){break}E=D+72|0;c[E>>2]=c[E>>2]|1024;E=D+8|0;c[(c[E>>2]|0)+8>>2]=0;c[(c[E>>2]|0)+48>>2]=B;E=D+104|0;NN(E|0,u|0,80)|0;c[D+136>>2]=c[D>>2];E=D+140|0;c[E>>2]=c[E>>2]|1;E=c[m>>2]|0;if((a[E]|0)==0){A6(w);F=D+88|0}else{z=D+88|0;zK(z,1644488,E,8)|0;F=z}z=c[249224+(y<<2)>>2]|0;zK(F,1642680,z,0)|0;break L8572}}while(0);A6(w);nJ(k);break}else if((e|0)==(-1|0)){iE(c[o>>2]|0,s,f)|0;a[(c[n>>2]|0)+f|0]=0}else if((e|0)==0){iO(c[o>>2]|0,f,s,r|1)|0}else{if((e-2|0)>>>0>=4){u=b;if((e|0)==6){AN(u,48,1654872,(t=i,i=i+8|0,c[t>>2]=d,t)|0);i=t;break}else{AN(u,48,1653744,(t=i,i=i+16|0,c[t>>2]=e,c[t+8>>2]=d,t)|0);i=t;break}}u=c[o>>2]|0;do{if((e|0)==2){if((g|0)==32){H=iI(u)|0;break}else{H=iG(u)|0;break}}else if((e|0)==4){x=iJ(u)|0;H=x}else if((e|0)==5){H=iG(u)|0}else if((e|0)==3){H=iI(u)|0}else{H=-2147483648}}while(0);bc(c[n>>2]|0,22,1657280,(t=i,i=i+16|0,c[t>>2]=H,c[t+8>>2]=(H|0)<0|0?-1:0,t)|0)|0;i=t}u=c[n>>2]|0;if((a[u]|0)==0){break}w=b+1140|0;zK(w,d,u,0)|0}}while(0);A6(n);n=c[o>>2]|0;o=NU(p,q,f,(f|0)<0|0?-1:0)|0;il(n,o,G,0)|0;i=h;return}function hV(a){a=a|0;var d=0,e=0,f=0,g=0;d=c[a+12>>2]|0;e=a+1076|0;c[e>>2]=3200;f=d+153608|0;c[f>>2]=0;c[f+4>>2]=0;c[d+156992>>2]=A1(4096)|0;c[d+156996>>2]=512;b[d+157e3>>1]=0;if((hZ(a,0,0,50,0)|0)<0){g=-1;return g|0}ik(c[a+16>>2]|0);c[d+153648>>2]=0;NM(d+153632|0,-1|0,16);ie(d+156856|0,d+153652|0,c[e>>2]|0,1,0,0,0,0)|0;e=a+1192|0;if((c[e>>2]|0)>=0){g=0;return g|0}c[e>>2]=1;g=0;return g|0}function hW(e,f){e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0;g=i;h=e+12|0;j=c[h>>2]|0;k=c[f+36>>2]|0;l=f+32|0;m=c[l>>2]|0;n=e+28|0;o=j+8|0;p=(c[(c[(c[(c[n>>2]|0)+(m<<2)>>2]|0)+8>>2]|0)+8>>2]|0)==1?k&-2:k;k=f+8|0;q=c[k>>2]|0;r=c[k+4>>2]|0;k=f+16|0;do{if((q|0)==0&(r|0)==(-2147483648|0)){s=c[k>>2]|0;t=c[k+4>>2]|0;if(!((s|0)==0&(t|0)==(-2147483648|0))){u=t;v=s;break}AN(0,0,1737744,(s=i,i=i+24|0,c[s>>2]=1716232,c[s+8>>2]=1727992,c[s+16>>2]=808,s)|0);i=s;cw();return 0}else{u=r;v=q}}while(0);q=N2(v,u,1e4,0)|0;u=G;v=j+153616|0;r=c[v>>2]|0;s=c[v+4>>2]|0;t=(c[f+48>>2]|0)*1e4|0;w=NU(t,(t|0)<0|0?-1:0,q,u)|0;t=G;x=(s|0)>(t|0)|(s|0)==(t|0)&r>>>0>w>>>0;c[v>>2]=x?r:w;c[v+4>>2]=x?s:t;t=j+153608|0;s=c[t>>2]|0;x=c[t+4>>2]|0;v=s;w=c[k>>2]|0;r=c[k+4>>2]|0;k=c[f+28>>2]|0;y=c[h>>2]|0;if((k|0)>0){z=c[f+24>>2]|0;f=y+153632|0;A=y+153628|0;B=(c[(c[n>>2]|0)+(c[l>>2]<<2)>>2]|0)+8|0;l=y+153624|0;n=NU(w,r,3100,0)|0;C=o+(m*1200|0)|0;D=(p&1|0)==0;E=o+(m*1200|0)+4|0;F=n;n=y+156856|0;H=y+153640|0;I=y+153648|0;y=z;z=0;while(1){J=k-z|0;L8648:do{if((c[f>>2]|0)==(-1|0)&(c[f+4>>2]|0)==(-1|0)){K=6389}else{while(1){L=c[A>>2]|0;M=L-29|0;if((M|0)>=(J|0)){break}if((c[(c[B>>2]|0)+8>>2]|0)!=1){break}hY(e);if((c[f>>2]|0)==(-1|0)&(c[f+4>>2]|0)==(-1|0)){K=6389;break L8648}}if((M|0)>0){N=M;K=6392;break}O=0;P=L;Q=a[l]|0}}while(0);if((K|0)==6389){K=0;R=(J|0)<3154;a[l]=R&1;c[A>>2]=3200;c[f>>2]=w;c[f+4>>2]=r;N=R?3153:3174;K=6392}if((K|0)==6392){K=0;if((J|0)>(N|0)){S=N}else{S=(J|0)==(N-1|0)?N-2|0:J}R=c[h>>2]|0;T=R+156856|0;U=c[C>>2]|0;ih(T,D?U:U|128);ih(T,d[E]|0);ir(T,z);ih(T,8);ir(T,k);ir(T,F);if((a[R+153624|0]|0)!=0){iv(T,S)}ij(n,y,S);T=a[l]|0;R=(T<<24>>24==0?-15:-17)-S+(c[A>>2]|0)|0;c[A>>2]=R;c[H>>2]=w;c[H+4>>2]=r;c[I>>2]=(c[I>>2]|0)+1;O=S;P=R;Q=T}T=O+z|0;R=y+O|0;do{if(Q<<24>>24==0){hY(e)}else{if((P|0)>=30){break}hY(e)}}while(0);if((T|0)<(k|0)){y=R;z=T}else{V=E;break}}}else{V=o+(m*1200|0)+4|0}a[V]=(a[V]|0)+1&255;V=NU(q,u,40999999,0)|0;u=N0(V,G,1e7,0)|0;V=u;if((c[j+4>>2]|0)!=0){W=j+157016|0;X=W;c[X>>2]=V;i=g;return 0}if((p&1|0)==0){W=j+157016|0;X=W;c[X>>2]=V;i=g;return 0}p=NV(c[t>>2]|0,c[t+4>>2]|0,s,x)|0;x=p&65535;p=c[h>>2]|0;h=p+157012|0;s=c[h>>2]|0;do{if((s|0)<(V|0)){if((s|0)==0){c[p+157004>>2]=v;b[p+157008>>1]=x}t=p+156996|0;if((c[t>>2]|0)>>>0<V>>>0){u=V+512&-512;c[t>>2]=u;t=p+156992|0;c[t>>2]=A2(c[t>>2]|0,u<<3)|0;Y=c[h>>2]|0}else{Y=s}if((Y|0)>=(V|0)){break}u=p+157004|0;t=p+156992|0;q=p+157008|0;m=Y;do{c[(c[t>>2]|0)+(m<<3)>>2]=c[u>>2];b[(c[t>>2]|0)+(m<<3)+4>>1]=b[q>>1]|0;m=m+1|0;}while((m|0)<(V|0))}}while(0);Y=p+157e3|0;s=b[Y>>1]|0;b[Y>>1]=(s&65535)>(x&65535)?s:x;c[p+157004>>2]=v;b[p+157008>>1]=x;c[h>>2]=V;W=j+157016|0;X=W;c[X>>2]=V;i=g;return 0}function hX(a){a=a|0;var d=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;d=a+12|0;f=c[d>>2]|0;if((c[f+156868>>2]|0)>>>0>(c[f+156860>>2]|0)>>>0){hY(a)}g=a+16|0;h=il(c[g>>2]|0,0,0,1)|0;i=G;j=f+4|0;do{if((c[j>>2]|0)==0){k=f+157012|0;if((c[k>>2]|0)==0){break}l=c[f+157016>>2]|0;m=l+1|0;n=c[d>>2]|0;o=n+157012|0;p=c[o>>2]|0;do{if((p|0)<(m|0)){if((p|0)==0){c[n+157004>>2]=0;b[n+157008>>1]=0}q=n+156996|0;if((c[q>>2]|0)>>>0<m>>>0){r=l+513&-512;c[q>>2]=r;q=n+156992|0;c[q>>2]=A2(c[q>>2]|0,r<<3)|0;s=c[o>>2]|0}else{s=p}if((s|0)>=(m|0)){break}r=n+157004|0;q=n+156992|0;t=n+157008|0;u=s;do{c[(c[q>>2]|0)+(u<<3)>>2]=c[r>>2];b[(c[q>>2]|0)+(u<<3)+4>>1]=b[t>>1]|0;u=u+1|0;}while((u|0)<(m|0))}}while(0);c[n+157004>>2]=0;b[n+157008>>1]=0;c[o>>2]=m;p=c[f+156992>>2]|0;l=b[f+157e3>>1]|0;u=c[k>>2]|0;t=c[g>>2]|0;ij(t,255888,16);iw(t,(u*6|0)+56|0,0);ij(t,2867248,16);iw(t,1e7,0);ir(t,l&65535);ir(t,u);if((u|0)==0){break}else{v=0}do{ir(t,c[p+(v<<3)>>2]|0);iv(t,e[p+(v<<3)+4>>1]|0);v=v+1|0;}while(v>>>0<u>>>0)}}while(0);ik(c[g>>2]|0);v=c[g>>2]|0;do{if((c[j>>2]|0)==0){if((c[v+88>>2]|0)==0){break}s=il(v,0,0,1)|0;u=G;p=c[g>>2]|0;t=0;k=0;il(p,t,k,0)|0;k=f+156984|0;t=NV(h,i,c[k>>2]|0,c[k+4>>2]|0)|0;k=G;hZ(a,s,u,t,k)|0;w=f+156992|0;x=w;y=c[x>>2]|0;z=y;A4(z);return 0}}while(0);a=c[d>>2]|0;iv(v,17700);iv(v,8);d=a;ir(v,c[d>>2]|0);iv(v,0);iv(v,8);c[d>>2]=(c[d>>2]|0)+1;w=f+156992|0;x=w;y=c[x>>2]|0;z=y;A4(z);return 0}function hY(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;d=i;e=b+12|0;f=c[e>>2]|0;g=f+153640|0;h=c[g>>2]|0;j=c[g+4>>2]|0;k=f+153632|0;l=k;m=c[l>>2]|0;n=c[l+4>>2]|0;if((j|0)<(n|0)|(j|0)==(n|0)&h>>>0<m>>>0){AN(0,0,1737744,(o=i,i=i+24|0,c[o>>2]=1733096,c[o+8>>2]=1727992,c[o+16>>2]=643,o)|0);i=o;cw()}if((c[f+4>>2]|0)==0){p=n;q=m;r=j;s=h;t=f;u=b+16|0}else{h=b+16|0;j=c[h>>2]|0;m=(c[b+1076>>2]|0)+8|0;iv(j,17444);iv(j,m);n=f;ir(j,c[n>>2]|0);iv(j,0);iv(j,m);c[n>>2]=(c[n>>2]|0)+1;p=c[l+4>>2]|0;q=c[l>>2]|0;r=c[g+4>>2]|0;s=c[g>>2]|0;t=c[e>>2]|0;u=h}h=q;e=NV(s,r,q,p)|0;p=e;e=f+153648|0;q=c[e>>2]|0;r=f+153628|0;s=c[r>>2]|0;g=c[u>>2]|0;l=il(g,0,0,1)|0;n=G;m=t+153624|0;t=((a[m]|0)==0?-11:-12)+s|0;if((t|0)<=-1){AN(0,0,1737744,(o=i,i=i+24|0,c[o>>2]=1720232,c[o+8>>2]=1727992,c[o+16>>2]=604,o)|0);i=o;cw()}ih(g,130);ih(g,0);ih(g,0);s=(a[m]|0)!=0|0;do{if((t|0)>0){if((t|0)<256){v=s|8;break}else{v=s|16;break}}else{v=s}}while(0);ih(g,v);ih(g,93);if((v&16|0)!=0){iv(g,t-2|0)}if((v&8|0)!=0){ih(g,t-1|0)}ir(g,h);iv(g,p);if((a[m]|0)!=0){ih(g,q|128)}q=il(g,0,0,1)|0;g=NV(q,G,l,n)|0;n=g;g=c[r>>2]|0;if((n|0)>(g|0)){AN(0,0,1737744,(o=i,i=i+24|0,c[o>>2]=1723736,c[o+8>>2]=1727992,c[o+16>>2]=655,o)|0);i=o;cw()}else{o=f+153652|0;NM(f+(156852-g)|0,0,g|0);g=b+1076|0;ij(c[u>>2]|0,o,(c[g>>2]|0)-n|0);ik(c[u>>2]|0);u=f+153608|0;n=NU(c[u>>2]|0,c[u+4>>2]|0,1,0)|0;c[u>>2]=n;c[u+4>>2]=G;c[e>>2]=0;e=f+156856|0;NM(k|0,-1|0,16);k=c[g>>2]|0;ie(e,o,k,1,0,0,0,0)|0;i=d;return}}function hZ(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0;h=i;i=i+64|0;j=h|0;k=h+8|0;l=h+16|0;m=h+24|0;n=h+32|0;o=h+40|0;p=h+48|0;q=h+56|0;r=b+12|0;s=c[r>>2]|0;t=b+16|0;u=c[t>>2]|0;v=b+1140|0;jx(v,256008,0);w=zJ(c[v>>2]|0,1661904,0,0)|0;x=zJ(c[v>>2]|0,1645208,0,0)|0;y=zJ(c[v>>2]|0,1632280,0,0)|0;z=zJ(c[v>>2]|0,1625792,0,0)|0;A=zJ(c[v>>2]|0,1619296,0,0)|0;B=s+153616|0;C=NU(c[B>>2]|0,c[B+4>>2]|0,31e6,0)|0;D=G;E=(w|0)==0;if(E&(x|0)==0&(y|0)==0){F=(A|0)!=0|(z|0)!=0}else{F=1}H=zI(c[v>>2]|0)|0;I=b+24|0;if((c[I>>2]|0)==0){J=0}else{K=b+28|0;L=0;M=0;while(1){N=c[(c[K>>2]|0)+(M<<2)>>2]|0;O=c[N+8>>2]|0;mL(N,32,1,1e3);N=(c[O+72>>2]|0)+L|0;O=M+1|0;if(O>>>0<(c[I>>2]|0)>>>0){L=N;M=O}else{J=N;break}}}M=s+4|0;if((c[M>>2]|0)!=0){L=c[r>>2]|0;r=c[t>>2]|0;iv(r,18468);iv(r,8);t=L;ir(r,c[t>>2]|0);iv(r,3072);iv(r,8);c[t>>2]=(c[t>>2]|0)+1}ij(u,256192,16);iw(u,-1,-1);t=(H|0)!=0;ir(u,(t&1)+(F?4:3)+(c[I>>2]|0)|0);ih(u,1);ih(u,2);r=il(u,0,0,1)|0;L=G;K=il(u,0,0,1)|0;N=G;ij(u,256240,16);iw(u,24,0);ij(u,2867248,16);iw(u,d,e);iw(u,-717324288,27111902);e=s+153608|0;iw(u,c[e>>2]|0,c[e+4>>2]|0);iw(u,C,D);iw(u,c[B>>2]|0,c[B+4>>2]|0);iw(u,3100,0);if((c[M>>2]|0)==0){P=(c[u+88>>2]|0)==0?3:2}else{P=3}ir(u,P);P=b+1076|0;ir(u,c[P>>2]|0);ir(u,c[P>>2]|0);ir(u,J);J=il(u,0,0,1)|0;P=G;B=NU(K,N,16,0)|0;il(u,B,G,0)|0;B=NV(J,P,K,N)|0;iw(u,B,G);il(u,J,P,0)|0;P=il(u,0,0,1)|0;J=G;ij(u,256224,16);iw(u,24,0);ij(u,256208,16);ir(u,6);iv(u,0);B=il(u,0,0,1)|0;N=G;K=NU(P,J,16,0)|0;il(u,K,G,0)|0;K=NV(B,N,P,J)|0;iw(u,K,G);il(u,B,N,0)|0;if(F){if((iZ(o)|0)<0){Q=-12;i=h;return Q|0}F=il(u,0,0,1)|0;N=G;ij(u,256456,16);iw(u,24,0);if(E){R=0}else{R=iu(c[o>>2]|0,c[w+4>>2]|0)|0}iv(u,R);if((x|0)==0){S=0}else{S=iu(c[o>>2]|0,c[x+4>>2]|0)|0}iv(u,S);if((y|0)==0){T=0}else{T=iu(c[o>>2]|0,c[y+4>>2]|0)|0}iv(u,T);if((z|0)==0){U=0}else{U=iu(c[o>>2]|0,c[z+4>>2]|0)|0}iv(u,U);if((A|0)==0){V=0}else{V=iu(c[o>>2]|0,c[A+4>>2]|0)|0}iv(u,V);V=i$(c[o>>2]|0,n)|0;ij(u,c[n>>2]|0,V);A6(n);n=il(u,0,0,1)|0;V=G;o=NU(F,N,16,0)|0;A=G;il(u,o,A,0)|0;A=NV(n,V,F,N)|0;iw(u,A,G);il(u,n,V,0)|0;}if(t){t=il(u,0,0,1)|0;V=G;ij(u,256256,16);iw(u,24,0);iv(u,H);H=zJ(c[v>>2]|0,3118992,0,2)|0;if((H|0)!=0){n=l;A=j;N=H;do{H=c[N>>2]|0;if((iZ(m)|0)>=0){F=c[m>>2]|0;iu(F,H)|0;H=i$(c[m>>2]|0,l)|0;iv(u,H);ij(u,c[l>>2]|0,H);A6(n)}iv(u,0);H=c[N+4>>2]|0;if((iZ(k)|0)>=0){F=c[k>>2]|0;iu(F,H)|0;H=i$(c[k>>2]|0,j)|0;iv(u,H);ij(u,c[j>>2]|0,H);A6(A)}N=zJ(c[v>>2]|0,3118992,N,2)|0;}while((N|0)!=0)}N=il(u,0,0,1)|0;v=G;A=NU(t,V,16,0)|0;j=G;il(u,A,j,0)|0;j=NV(N,v,t,V)|0;iw(u,j,G);il(u,N,v,0)|0;}v=b+28|0;L8799:do{if((c[I>>2]|0)!=0){b=s+8|0;N=0;while(1){j=c[(c[(c[v>>2]|0)+(N<<2)>>2]|0)+8>>2]|0;V=N+1|0;c[b+(N*1200|0)>>2]=V;a[b+(N*1200|0)+4|0]=1;t=j+8|0;if((c[t>>2]|0)==1){W=18;X=8}else{W=(c[j+100>>2]|0)+51|0;X=0}A=il(u,0,0,1)|0;k=G;ij(u,255872,16);iw(u,24,0);if((c[t>>2]|0)==1){ij(u,256520,16);ij(u,256536,16)}else{ij(u,255840,16);ij(u,255856,16)}iw(u,0,0);n=il(u,0,0,1)|0;l=G;ir(u,W);ir(u,X);iv(u,V);ir(u,0);if((c[t>>2]|0)==1){t=l1(u,j)|0;if((t|0)<0){Q=-1;break}if((t|0)!=(W|0)){m=il(u,0,0,1)|0;H=G;il(u,n,l,0)|0;ir(u,t);il(u,m,H,0)|0;}ih(u,1);do{if((c[j+48>>2]|0)==69643){Y=6494}else{H=j+432|0;m=c[H>>2]|0;if((m|0)==0){Y=6494;break}iv(u,m);iv(u,c[H>>2]|0)}}while(0);if((Y|0)==6494){Y=0;iv(u,400);iv(u,400)}iv(u,1);ih(u,0)}else{ir(u,c[j+120>>2]|0);ir(u,c[j+124>>2]|0);ih(u,2);iv(u,(c[j+100>>2]|0)+40|0);l2(u,j,253424,1)}H=il(u,0,0,1)|0;m=G;t=NU(A,k,16,0)|0;il(u,t,G,0)|0;t=NV(H,m,A,k)|0;iw(u,t,G);il(u,H,m,0)|0;if(V>>>0<(c[I>>2]|0)>>>0){N=V}else{break L8799}}i=h;return Q|0}}while(0);W=il(u,0,0,1)|0;X=G;ij(u,256488,16);iw(u,24,0);ij(u,256504,16);ir(u,c[I>>2]|0);N=p;b=0;while(1){if(b>>>0>=(c[I>>2]|0)>>>0){Y=6514;break}m=c[(c[(c[v>>2]|0)+(b<<2)>>2]|0)+8>>2]|0;H=m+48|0;t=o8(c[H>>2]|0)|0;l=m+8|0;n=c[l>>2]|0;if((n|0)==1){iv(u,2)}else if((n|0)==0){iv(u,1)}else{iv(u,-1)}do{if((c[H>>2]|0)==86024){Z=1744232}else{if((t|0)==0){Z=m+16|0;break}else{Z=c[t>>2]|0;break}}}while(0);if((iZ(q)|0)<0){Q=-12;Y=6528;break}iu(c[q>>2]|0,Z)|0;t=i$(c[q>>2]|0,p)|0;iv(u,(t|0)/2|0);ij(u,c[p>>2]|0,t);A6(N);iv(u,0);if((c[l>>2]|0)==1){iv(u,2);iv(u,c[m+52>>2]|0)}else{iv(u,4);ir(u,c[m+52>>2]|0)}if((c[m+52>>2]|0)==0){Q=-1;Y=6530;break}else{b=b+1|0}}if((Y|0)==6528){i=h;return Q|0}else if((Y|0)==6514){b=il(u,0,0,1)|0;N=G;p=NU(W,X,16,0)|0;il(u,p,G,0)|0;p=NV(b,N,W,X)|0;iw(u,p,G);il(u,b,N,0)|0;N=il(u,0,0,1)|0;b=G;p=NV(N,b,r,L)|0;X=p;if((c[M>>2]|0)!=0){M=X+88|0;p=NU(r,L,-40,-1)|0;W=G;il(u,p,W,0)|0;iv(u,M);W=NU(r,L,-32,-1)|0;p=G;il(u,W,p,0)|0;iv(u,M)}M=X+30|0;X=NU(r,L,-14,-1)|0;il(u,X,G,0)|0;iw(u,M,(M|0)<0|0?-1:0);il(u,N,b,0)|0;M=s+156984|0;c[M>>2]=N;c[M+4>>2]=b;ij(u,256424,16);iw(u,f,g);ij(u,2867248,16);iw(u,c[e>>2]|0,c[e+4>>2]|0);ih(u,1);ih(u,1);Q=0;i=h;return Q|0}else if((Y|0)==6530){i=h;return Q|0}return 0}function h_(a){a=a|0;var b=0,d=0;b=c[a+4>>2]|0;if((b|0)==0){d=1744208;return d|0}d=c[b>>2]|0;return d|0}function h$(a,b){a=a|0;b=b|0;var d=0,e=0;do{if((b|0)==0){d=c[a+8>>2]|0;if((d|0)==0){break}if((c[(c[a+4>>2]|0)+56>>2]|0)==0){break}else{e=d}return e|0}}while(0);e=0;return e|0}function h0(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;L8866:do{if((a|0)==0){b=0}else{d=0;while(1){e=c[((d|0)==0?2225184:d+28|0)>>2]|0;if((e|0)==0){b=0;break L8866}if((c[e+56>>2]|0)==(a|0)){b=e;break}else{d=e}}}}while(0);while(1){a=c[((b|0)==0?2225184:b+28|0)>>2]|0;if((a|0)==0){f=0;g=6549;break}d=c[a+56>>2]|0;if((d|0)==0){b=a}else{f=d;g=6550;break}}if((g|0)==6549){return f|0}else if((g|0)==6550){return f|0}return 0}function h1(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;d=c[a>>2]|0;e=c[((d|0)==0?2225184:d+28|0)>>2]|0;d=e;c[a>>2]=d;if((e|0)==0){f=0;return f|0}L8880:do{if((b|0)==0){g=e;h=d;while(1){if((c[g+12>>2]|0)!=0){i=g;break L8880}j=c[((h|0)==0?2225184:h+28|0)>>2]|0;k=j;c[a>>2]=k;if((j|0)==0){f=0;break}else{g=j;h=k}}return f|0}else{h=e;g=d;while(1){if((c[h+16>>2]|0)!=0){i=h;break L8880}k=c[((g|0)==0?2225184:g+28|0)>>2]|0;j=k;c[a>>2]=j;if((k|0)==0){f=0;break}else{h=k;g=j}}return f|0}}while(0);f=c[i>>2]|0;return f|0}function h2(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;if(b>>>0<68){d=A7(68)|0;e=d;f=a;NN(d|0,f|0,b)|0;g=e}else{g=a}a=2225184;while(1){e=c[a>>2]|0;if((e|0)==0){break}else{a=e+28|0}}c[a>>2]=g;c[g+28>>2]=0;return 0}function h3(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0;f=c[(c[a+4>>2]|0)+20>>2]|0;if((f|0)==0){g=-1;h=-38;return(G=g,h)|0}i=dk[f&63](a,b,d,e&-131073)|0;g=G;h=i;return(G=g,h)|0}function h4(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;g=i;i=i+256|0;h=g|0;j=g+128|0;k=cm(d|0,1701360)|0;if((c[556296]|0)==0){AN(0,24,1678056,(l=i,i=i+1|0,i=i+7&-8,c[l>>2]=0,l)|0);i=l}m=a[d+k|0]|0;if((m<<24>>24|0)==44){n=k+1|0;if((bi(d+n|0,58)|0)==0){o=6579}else{p=n;o=6578}}else if((m<<24>>24|0)==58){p=k+1|0;o=6578}else{o=6579}if((o|0)==6578){k=h|0;m=p>>>0>128?128:p;y_(k,d,m)|0}else if((o|0)==6579){m=h|0;a[m]=a[1734984]|0;a[m+1|0]=a[1734985]|0;a[m+2|0]=a[1734986]|0;a[m+3|0]=a[1734987]|0;a[m+4|0]=a[1734988]|0}m=h|0;h=bi(m|0,44)|0;if((h|0)!=0){a[h]=0}h=j|0;y_(h,m,128)|0;j=bi(h|0,43)|0;if((j|0)!=0){a[j]=0}j=c[556296]|0;L8920:do{if((j|0)!=0){k=j;while(1){p=c[k>>2]|0;if((a8(m|0,p|0)|0)==0){o=6586;break}if((c[k+60>>2]&1|0)!=0){if((a8(h|0,p|0)|0)==0){o=6590;break}}p=c[((k|0)==0?2225184:k+28|0)>>2]|0;if((p|0)==0){break L8920}else{k=p}}if((o|0)==6590){q=h5(b,k,d,e,f)|0;i=g;return q|0}else if((o|0)==6586){q=h5(b,k,d,e,f)|0;i=g;return q|0}}}while(0);c[b>>2]=0;if((a8(1661880,m|0)|0)!=0){q=-1330794744;i=g;return q|0}AN(0,24,1645120,(l=i,i=i+1|0,i=i+7&-8,c[l>>2]=0,l)|0);i=l;q=-1330794744;i=g;return q|0}function h5(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;h=i;i=i+8|0;j=h|0;do{if((f&1|0)!=0){if((c[d+12>>2]|0)!=0){break}AN(0,16,1632208,(k=i,i=i+8|0,c[k>>2]=c[d>>2],k)|0);i=k;l=-5;i=h;return l|0}}while(0);do{if((f&2|0)!=0){if((c[d+16>>2]|0)!=0){break}AN(0,16,1625728,(k=i,i=i+8|0,c[k>>2]=c[d>>2],k)|0);i=k;l=-5;i=h;return l|0}}while(0);m=A7((NL(e|0)|0)+49|0)|0;c[j>>2]=m;L8948:do{if((m|0)==0){n=-12}else{c[m>>2]=236024;o=m+48|0;p=m+12|0;c[p>>2]=o;NQ(o|0,e|0)|0;c[m+4>>2]=d;c[m+16>>2]=f;c[m+24>>2]=0;c[m+20>>2]=0;o=c[d+52>>2]|0;L8950:do{if((o|0)!=0){q=A7(o)|0;c[m+8>>2]=q;r=c[d+56>>2]|0;if((r|0)==0){break}s=d|0;t=NL(c[s>>2]|0)|0;u=bi(c[p>>2]|0,44)|0;c[q>>2]=r;r=c[j>>2]|0;Br(c[r+8>>2]|0);q=c[r+12>>2]|0;if(!((ba(c[s>>2]|0,q|0,t|0)|0)==0&(q+t|0)==(u|0))){break}t=a[u+1|0]|0;q=t<<24>>24;s=u+2|0;while(1){v=bi(s|0,q|0)|0;if(!((v|0)!=0&s>>>0<v>>>0)){w=6614;break}r=v+1|0;x=bi(r|0,q|0)|0;if((x|0)==0){w=6614;break}a[v]=0;a[x]=0;y=Bd(c[(c[j>>2]|0)+8>>2]|0,s,r,0)|0;if((y|0)==-1414549496){w=6612;break}a[v]=t;a[x]=t;if((y|0)>-1){s=x+1|0}else{break}}do{if((w|0)==6612){AN(c[j>>2]|0,16,1619248,(k=i,i=i+8|0,c[k>>2]=s,k)|0);i=k;a[v]=t;a[x]=t}else if((w|0)==6614){if((s|0)!=(v|0)){break}NP(u|0,s+1|0,NL(s|0)|0);break L8950}}while(0);s=c[j>>2]|0;AN(s,16,1615176,(k=i,i=i+8|0,c[k>>2]=u,k)|0);i=k;A6(s+8|0);A6(j);n=-22;break L8948}}while(0);p=c[j>>2]|0;if((g|0)!=0){o=g;s=p+32|0;t=c[o+4>>2]|0;c[s>>2]=c[o>>2];c[s+4>>2]=t}c[b>>2]=p;l=0;i=h;return l|0}}while(0);c[b>>2]=0;l=n;i=h;return l|0}function h6(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;g=i;i=i+8|0;h=g|0;j=h4(a,b,d,e)|0;if((j|0)!=0){k=j;i=g;return k|0}do{if((f|0)==0){l=6630}else{j=c[a>>2]|0;if((c[(c[j+4>>2]|0)+56>>2]|0)==0){l=6630;break}e=Bv(c[j+8>>2]|0,f)|0;if((e|0)<0){m=e}else{l=6630}}}while(0);do{if((l|0)==6630){e=c[a>>2]|0;j=e+4|0;d=c[j>>2]|0;b=c[d+8>>2]|0;if((b|0)==0){n=c6[c[d+4>>2]&255](e,c[e+12>>2]|0,c[e+16>>2]|0)|0}else{n=dk[b&63](e,c[e+12>>2]|0,c[e+16>>2]|0,f)|0}if((n|0)!=0){m=n;break}c[e+28>>2]=1;do{if((c[e+16>>2]&2|0)==0){if((a8(c[c[j>>2]>>2]|0,1734984)|0)==0){break}else{k=0}i=g;return k|0}}while(0);b=e+24|0;if((c[b>>2]|0)!=0){k=0;i=g;return k|0}d=c[(c[j>>2]|0)+20>>2]|0;do{if((d|0)!=0){o=dk[d&63](e,0,0,0)|0;p=G;q=0;if((p|0)<(q|0)|(p|0)==(q|0)&o>>>0<0>>>0){break}else{k=0}i=g;return k|0}}while(0);c[b>>2]=1;k=0;i=g;return k|0}}while(0);n=c[a>>2]|0;f=h;c[h>>2]=n;if((n|0)!=0){h=n+4|0;do{if((c[n+28>>2]|0)!=0){l=c[(c[h>>2]|0)+24>>2]|0;if((l|0)==0){break}c1[l&255](n)|0}}while(0);l=c[h>>2]|0;if((c[l+52>>2]|0)!=0){h=n+8|0;if((c[l+56>>2]|0)!=0){Bu(c[h>>2]|0)}A6(h)}A6(f)}c[a>>2]=0;k=m;i=g;return k|0}function h7(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=i;i=i+8|0;d=b|0;c[d>>2]=a;if((a|0)==0){e=0;i=b;return e|0}f=a+4|0;do{if((c[a+28>>2]|0)==0){g=0}else{h=c[(c[f>>2]|0)+24>>2]|0;if((h|0)==0){g=0;break}g=c1[h&255](a)|0}}while(0);h=c[f>>2]|0;if((c[h+52>>2]|0)!=0){f=a+8|0;if((c[h+56>>2]|0)!=0){Bu(c[f>>2]|0)}A6(f)}A6(d);e=g;i=b;return e|0}function h8(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0;e=a+16|0;if((c[e>>2]&1|0)==0){f=-5;return f|0}g=c[(c[a+4>>2]|0)+12>>2]|0;h=a+40|0;i=a+32|0;j=i|0;k=a+36|0;L9031:do{if((i|0)==0){l=0;m=5;n=0;o=0;p=b;q=d;L9059:while(1){do{r=c6[g&255](a,p,q)|0;}while((r|0)==-4);if((c[e>>2]&8|0)!=0){f=r;s=6703;break}do{if((r|0)==-11){if((m|0)!=0){t=m-1|0;u=0;v=n;w=o;break}do{if((c[h>>2]|0)==0&(c[h+4>>2]|0)==0){x=n;y=o}else{z=Ch()|0;A=G;if((o|0)==0&(n|0)==0){x=A;y=z;break}B=NU(c[h>>2]|0,c[h+4>>2]|0,o,n)|0;C=G;if((A|0)>(C|0)|(A|0)==(C|0)&z>>>0>B>>>0){f=-5;s=6704;break L9059}else{x=n;y=o}}}while(0);Ci(1e3)|0;t=0;u=0;v=x;w=y}else{if((r|0)<1){D=l;E=r;break L9031}t=(m|0)>2?m:2;u=r;v=n;w=o}}while(0);B=u+l|0;if((B|0)<1){l=B;m=t;n=v;o=w;p=b+B|0;q=d-B|0}else{f=B;s=6702;break}}if((s|0)==6703){return f|0}else if((s|0)==6702){return f|0}else if((s|0)==6704){return f|0}}else{q=0;p=5;o=0;n=0;m=b;l=d;L9032:while(1){do{F=c6[g&255](a,m,l)|0;}while((F|0)==-4);if((c[e>>2]&8|0)!=0){f=F;s=6706;break}do{if((F|0)==-11){if((p|0)!=0){H=p-1|0;I=0;J=o;K=n;break}do{if((c[h>>2]|0)==0&(c[h+4>>2]|0)==0){L=o;M=n}else{B=Ch()|0;z=G;if((n|0)==0&(o|0)==0){L=z;M=B;break}C=NU(c[h>>2]|0,c[h+4>>2]|0,n,o)|0;A=G;if((z|0)>(A|0)|(z|0)==(A|0)&B>>>0>C>>>0){f=-5;s=6707;break L9032}else{L=o;M=n}}}while(0);Ci(1e3)|0;H=0;I=0;J=L;K=M}else{if((F|0)<1){D=q;E=F;break L9031}H=(p|0)>2?p:2;I=F;J=o;K=n}}while(0);C=I+q|0;do{if((C|0)<(d|0)){B=c[j>>2]|0;if((B|0)==0){break}if((c1[B&255](c[k>>2]|0)|0)!=0){f=-1414092869;s=6709;break L9032}}}while(0);if((C|0)<1){q=C;p=H;o=J;n=K;m=b+C|0;l=d-C|0}else{f=C;s=6710;break}}if((s|0)==6710){return f|0}else if((s|0)==6709){return f|0}else if((s|0)==6706){return f|0}else if((s|0)==6707){return f|0}}}while(0);f=(E|0)<0&(E|0)!=-541478725?E:D;return f|0}function h9(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;e=a+16|0;if((c[e>>2]&2|0)==0){f=-5;return f|0}g=c[a+20>>2]|0;if((g|0)!=0&(g|0)<(d|0)){f=-5;return f|0}g=c[(c[a+4>>2]|0)+16>>2]|0;if((d|0)<=0){f=0;return f|0}h=a+40|0;i=a+32|0;j=(i|0)==0;k=i|0;i=a+36|0;l=0;m=5;n=0;o=0;p=b;q=d;L9093:while(1){do{r=c6[g&255](a,p,q)|0;}while((r|0)==-4);if((c[e>>2]&8|0)!=0){f=r;s=6734;break}do{if((r|0)==-11){if((m|0)!=0){t=m-1|0;u=0;v=n;w=o;break}do{if((c[h>>2]|0)==0&(c[h+4>>2]|0)==0){x=n;y=o}else{z=Ch()|0;A=G;if((o|0)==0&(n|0)==0){x=A;y=z;break}B=NU(c[h>>2]|0,c[h+4>>2]|0,o,n)|0;C=G;if((A|0)>(C|0)|(A|0)==(C|0)&z>>>0>B>>>0){f=-5;s=6739;break L9093}else{x=n;y=o}}}while(0);Ci(1e3)|0;t=0;u=0;v=x;w=y}else{if((r|0)<1){s=6726;break L9093}t=(m|0)>2?m:2;u=r;v=n;w=o}}while(0);B=u+l|0;if((B|0)>=(d|0)){f=B;s=6740;break}do{if(!j){z=c[k>>2]|0;if((z|0)==0){break}if((c1[z&255](c[i>>2]|0)|0)!=0){f=-1414092869;s=6741;break L9093}}}while(0);l=B;m=t;n=v;o=w;p=b+B|0;q=d-B|0}if((s|0)==6741){return f|0}else if((s|0)==6734){return f|0}else if((s|0)==6726){f=(r|0)<0&(r|0)!=-541478725?r:l;return f|0}else if((s|0)==6739){return f|0}else if((s|0)==6740){return f|0}return 0}function ia(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;d=i;i=i+16|0;e=d|0;f=d+8|0;g=h4(f,a,b,0)|0;if((g|0)!=0){h=g;i=d;return h|0}g=c[f>>2]|0;f=g+4|0;a=c[f>>2]|0;j=c[a+64>>2]|0;if((j|0)==0){k=c[a+8>>2]|0;if((k|0)==0){l=c6[c[a+4>>2]&255](g,c[g+12>>2]|0,c[g+16>>2]|0)|0}else{l=dk[k&63](g,c[g+12>>2]|0,c[g+16>>2]|0,0)|0}do{if((l|0)==0){c[g+28>>2]=1;if((c[g+16>>2]&2|0)==0){if((a8(c[c[f>>2]>>2]|0,1734984)|0)!=0){m=0;break}}k=g+24|0;if((c[k>>2]|0)!=0){m=0;break}a=c[(c[f>>2]|0)+20>>2]|0;if((a|0)!=0){n=dk[a&63](g,0,0,0)|0;a=G;o=0;if(!((a|0)<(o|0)|(a|0)==(o|0)&n>>>0<0>>>0)){m=0;break}}c[k>>2]=1;m=0}else{m=l}}while(0);p=(m|0)>-1?b:m}else{p=dg[j&255](g,b)|0}b=e;c[e>>2]=g;if((g|0)==0){h=p;i=d;return h|0}do{if((c[g+28>>2]|0)!=0){e=c[(c[f>>2]|0)+24>>2]|0;if((e|0)==0){break}c1[e&255](g)|0}}while(0);e=c[f>>2]|0;if((c[e+52>>2]|0)!=0){f=g+8|0;if((c[e+56>>2]|0)!=0){Bu(c[f>>2]|0)}A6(f)}A6(b);h=p;i=d;return h|0}function ib(a){a=a|0;var b=0,d=0,e=0;do{if((a|0)!=0){b=c[a>>2]|0;if((b|0)==0){break}d=c1[b&255](c[a+4>>2]|0)|0;if((d|0)==0){break}else{e=d}return e|0}}while(0);e=0;return e|0}function ic(a,b){a=a|0;b=b|0;var d=0;if((b|0)==0){d=c[a+20>>2]|0}else{d=0}return d|0}function id(a){a=a|0;return((a|0)!=0?0:236024)|0}function ie(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0;c[a+4>>2]=b;c[a+8>>2]=d;c[a+12>>2]=b;c[a+20>>2]=f;c[a+104>>2]=0;f=(e|0)!=0;if(f){c[a+16>>2]=b+d;c[a+56>>2]=1}else{c[a+16>>2]=b;c[a+56>>2]=0}c[a+28>>2]=h;c[a+24>>2]=g;c[a+32>>2]=i;i=a+40|0;c[a+76>>2]=0;NM(i|0,0,16);c[a+88>>2]=1;c[a+60>>2]=0;c[a+72>>2]=0;if((g|0)!=0|f){j=a+80|0;c[j>>2]=0;k=a+84|0;c[k>>2]=0;return 0}c[i>>2]=d;c[i+4>>2]=(d|0)<0|0?-1:0;c[a+16>>2]=b+d;j=a+80|0;c[j>>2]=0;k=a+84|0;c[k>>2]=0;return 0}function ig(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0;i=A7(128)|0;if((i|0)==0){j=0;return j|0}c[i+4>>2]=a;c[i+8>>2]=b;c[i+12>>2]=a;c[i+20>>2]=e;c[i+104>>2]=0;e=(d|0)!=0;if(e){c[i+16>>2]=a+b;c[i+56>>2]=1}else{c[i+16>>2]=a;c[i+56>>2]=0}c[i+28>>2]=g;c[i+24>>2]=f;c[i+32>>2]=h;h=i+40|0;c[i+76>>2]=0;NM(h|0,0,16);c[i+88>>2]=1;c[i+60>>2]=0;c[i+72>>2]=0;if(!((f|0)!=0|e)){e=h;c[e>>2]=b;c[e+4>>2]=(b|0)<0|0?-1:0;c[i+16>>2]=a+b}c[i+80>>2]=0;c[i+84>>2]=0;j=i;return j|0}function ih(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;e=b+12|0;f=c[e>>2]|0;c[e>>2]=f+1;a[f]=d&255;d=c[e>>2]|0;if(d>>>0<(c[b+16>>2]|0)>>>0){return}f=b+4|0;g=c[f>>2]|0;do{if(d>>>0>g>>>0){h=d-g|0;i=c[b+28>>2]|0;do{if((i|0)!=0){j=b+76|0;if((c[j>>2]|0)!=0){break}k=c6[i&255](c[b+20>>2]|0,g,h)|0;if((k|0)>=0){break}c[j>>2]=k}}while(0);i=b+124|0;c[i>>2]=(c[i>>2]|0)+1;i=b+40|0;k=NU(c[i>>2]|0,c[i+4>>2]|0,h,(h|0)<0|0?-1:0)|0;c[i>>2]=k;c[i+4>>2]=G;i=c[b+72>>2]|0;if((i|0)==0){l=c[f>>2]|0;break}else{k=b+64|0;j=b+68|0;m=c[j>>2]|0;c[k>>2]=c6[i&255](c[k>>2]|0,m,(c[e>>2]|0)-m|0)|0;m=c[f>>2]|0;c[j>>2]=m;l=m;break}}else{l=g}}while(0);c[e>>2]=l;return}function ii(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;if((d|0)<=0){return}e=a+16|0;f=a+12|0;g=b&255;b=a+4|0;h=a+28|0;i=a+124|0;j=a+40|0;k=a+72|0;l=a+64|0;m=a+68|0;n=a+76|0;o=a+20|0;a=d;d=c[f>>2]|0;while(1){p=(c[e>>2]|0)-d|0;q=(p|0)>(a|0)?a:p;NM(d|0,g|0,q|0);p=(c[f>>2]|0)+q|0;c[f>>2]=p;if(p>>>0<(c[e>>2]|0)>>>0){r=p}else{s=c[b>>2]|0;do{if(p>>>0>s>>>0){t=p-s|0;u=c[h>>2]|0;do{if((u|0)!=0){if((c[n>>2]|0)!=0){break}v=c6[u&255](c[o>>2]|0,s,t)|0;if((v|0)>=0){break}c[n>>2]=v}}while(0);c[i>>2]=(c[i>>2]|0)+1;u=NU(c[j>>2]|0,c[j+4>>2]|0,t,(t|0)<0|0?-1:0)|0;c[j>>2]=u;c[j+4>>2]=G;u=c[k>>2]|0;if((u|0)==0){w=c[b>>2]|0;break}else{v=c[m>>2]|0;c[l>>2]=c6[u&255](c[l>>2]|0,v,(c[f>>2]|0)-v|0)|0;v=c[b>>2]|0;c[m>>2]=v;w=v;break}}else{w=s}}while(0);c[f>>2]=w;r=w}s=a-q|0;if((s|0)>0){a=s;d=r}else{break}}return}function ij(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;do{if((c[a+104>>2]|0)!=0){e=a+72|0;if((c[e>>2]|0)!=0){break}f=a+12|0;g=c[f>>2]|0;h=a+4|0;i=c[h>>2]|0;do{if(g>>>0>i>>>0){j=g-i|0;k=a+28|0;l=c[k>>2]|0;do{if((l|0)!=0){m=a+76|0;if((c[m>>2]|0)!=0){break}n=c6[l&255](c[a+20>>2]|0,i,j)|0;if((n|0)>=0){break}c[m>>2]=n}}while(0);l=a+124|0;c[l>>2]=(c[l>>2]|0)+1;l=a+40|0;n=NU(c[l>>2]|0,c[l+4>>2]|0,j,(j|0)<0|0?-1:0)|0;c[l>>2]=n;c[l+4>>2]=G;l=c[e>>2]|0;if((l|0)==0){o=c[h>>2]|0;p=k;break}else{n=a+64|0;m=a+68|0;q=c[m>>2]|0;c[n>>2]=c6[l&255](c[n>>2]|0,q,(c[f>>2]|0)-q|0)|0;q=c[h>>2]|0;c[m>>2]=q;o=q;p=k;break}}else{o=i;p=a+28|0}}while(0);c[f>>2]=o;c[a+48>>2]=0;i=c[p>>2]|0;do{if((i|0)!=0){h=a+76|0;if((c[h>>2]|0)!=0){break}e=c6[i&255](c[a+20>>2]|0,b,d)|0;if((e|0)>=0){break}c[h>>2]=e}}while(0);i=a+124|0;c[i>>2]=(c[i>>2]|0)+1;i=a+40|0;f=NU(c[i>>2]|0,c[i+4>>2]|0,d,(d|0)<0|0?-1:0)|0;c[i>>2]=f;c[i+4>>2]=G;return}}while(0);if((d|0)<=0){return}p=a+16|0;o=a+12|0;i=a+4|0;f=a+28|0;e=a+124|0;h=a+40|0;g=a+72|0;q=a+64|0;m=a+68|0;n=a+76|0;l=a+20|0;a=d;d=b;b=c[o>>2]|0;while(1){r=(c[p>>2]|0)-b|0;s=(r|0)>(a|0)?a:r;NN(b|0,d|0,s)|0;r=(c[o>>2]|0)+s|0;c[o>>2]=r;if(r>>>0<(c[p>>2]|0)>>>0){t=r}else{u=c[i>>2]|0;do{if(r>>>0>u>>>0){v=r-u|0;w=c[f>>2]|0;do{if((w|0)!=0){if((c[n>>2]|0)!=0){break}x=c6[w&255](c[l>>2]|0,u,v)|0;if((x|0)>=0){break}c[n>>2]=x}}while(0);c[e>>2]=(c[e>>2]|0)+1;w=NU(c[h>>2]|0,c[h+4>>2]|0,v,(v|0)<0|0?-1:0)|0;c[h>>2]=w;c[h+4>>2]=G;w=c[g>>2]|0;if((w|0)==0){y=c[i>>2]|0;break}else{k=c[m>>2]|0;c[q>>2]=c6[w&255](c[q>>2]|0,k,(c[o>>2]|0)-k|0)|0;k=c[i>>2]|0;c[m>>2]=k;y=k;break}}else{y=u}}while(0);c[o>>2]=y;t=y}u=a-s|0;if((u|0)<=0){break}a=u;d=d+s|0;b=t}return}function ik(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;b=a+12|0;d=c[b>>2]|0;e=a+4|0;f=c[e>>2]|0;if(d>>>0<=f>>>0){g=f;c[b>>2]=g;h=a+48|0;c[h>>2]=0;return}i=d-f|0;d=c[a+28>>2]|0;do{if((d|0)!=0){j=a+76|0;if((c[j>>2]|0)!=0){break}k=c6[d&255](c[a+20>>2]|0,f,i)|0;if((k|0)>=0){break}c[j>>2]=k}}while(0);f=a+124|0;c[f>>2]=(c[f>>2]|0)+1;f=a+40|0;d=NU(c[f>>2]|0,c[f+4>>2]|0,i,(i|0)<0|0?-1:0)|0;c[f>>2]=d;c[f+4>>2]=G;f=c[a+72>>2]|0;if((f|0)==0){g=c[e>>2]|0;c[b>>2]=g;h=a+48|0;c[h>>2]=0;return}else{d=a+64|0;i=a+68|0;k=c[i>>2]|0;c[d>>2]=c6[f&255](c[d>>2]|0,k,(c[b>>2]|0)-k|0)|0;k=c[e>>2]|0;c[i>>2]=k;g=k;c[b>>2]=g;h=a+48|0;c[h>>2]=0;return}}function il(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;f=e&-131073;if((a|0)==0){g=-1;h=-22;return(G=g,h)|0}i=a+40|0;j=c[i>>2]|0;k=c[i+4>>2]|0;l=a+56|0;m=(c[l>>2]|0)==0;if(m){n=(c[a+16>>2]|0)-(c[a+4>>2]|0)|0;o=(n|0)<0|0?-1:0;p=n}else{o=0;p=0}n=NV(j,k,p,o)|0;o=G;if(f>>>0>1){g=-1;h=-22;return(G=g,h)|0}do{if((f|0)==1){p=(c[a+12>>2]|0)-(c[a+4>>2]|0)|0;q=NU(p,(p|0)<0|0?-1:0,n,o)|0;p=G;if((b|0)==0&(d|0)==0){g=p;h=q;return(G=g,h)|0}else{r=NU(q,p,b,d)|0;s=G;t=r;break}}else{s=d;t=b}}while(0);b=NV(t,s,n,o)|0;o=G;n=a+48|0;do{if((c[n>>2]|0)==0){if((c[a+104>>2]|0)==0){d=-1;if(!((o|0)>(d|0)|(o|0)==(d|0)&b>>>0>-1>>>0)){u=6886;break}}else{d=-1;if(!((c[a+32>>2]|0)==0&((o|0)>(d|0)|(o|0)==(d|0)&b>>>0>-1>>>0))){u=6886;break}}d=c[a+4>>2]|0;f=(c[a+16>>2]|0)-d|0;r=(f|0)<0|0?-1:0;if((o|0)>(r|0)|(o|0)==(r|0)&b>>>0>f>>>0){u=6886;break}c[a+12>>2]=d+b}else{u=6886}}while(0);L9310:do{if((u|0)==6886){if((c[a+88>>2]|0)==0){d=-1;if(m&((o|0)>(d|0)|(o|0)==(d|0)&b>>>0>-1>>>0)){u=6889}}else{d=(c[a+16>>2]|0)+4096-(c[a+4>>2]|0)|0;f=(d|0)<0|0?-1:0;r=-1;if((o|0)<=(f|0)&((o|0)<(f|0)|b>>>0<=d>>>0)&(m&((o|0)>(r|0)|(o|0)==(r|0)&b>>>0>-1>>>0))){u=6889}}do{if((u|0)==6889){if((c[a+104>>2]|0)==0){if((e|0)==2){break}}else{if((c[a+32>>2]|0)!=0|(e|0)==2){break}}r=a+52|0;d=j;L9321:do{if((k|0)<(s|0)|(k|0)==(s|0)&j>>>0<t>>>0){while(1){if((c[r>>2]|0)!=0){g=-1;h=-541478725;break}im(a);f=c[i>>2]|0;p=c[i+4>>2]|0;if(!((p|0)<(s|0)|(p|0)==(s|0)&f>>>0<t>>>0)){v=f;break L9321}}return(G=g,h)|0}else{v=d}}while(0);if((c[r>>2]|0)==0){c[a+12>>2]=(c[a+16>>2]|0)+(t-v);break L9310}else{g=-1;h=-541478725;return(G=g,h)|0}}}while(0);if(!m){d=a+12|0;f=c[d>>2]|0;p=a+4|0;q=c[p>>2]|0;do{if(f>>>0>q>>>0){w=f-q|0;x=c[a+28>>2]|0;do{if((x|0)!=0){y=a+76|0;if((c[y>>2]|0)!=0){break}z=c6[x&255](c[a+20>>2]|0,q,w)|0;if((z|0)>=0){break}c[y>>2]=z}}while(0);x=a+124|0;c[x>>2]=(c[x>>2]|0)+1;x=NU(c[i>>2]|0,c[i+4>>2]|0,w,(w|0)<0|0?-1:0)|0;c[i>>2]=x;c[i+4>>2]=G;x=c[a+72>>2]|0;if((x|0)==0){A=c[p>>2]|0;break}else{r=a+64|0;z=a+68|0;y=c[z>>2]|0;c[r>>2]=c6[x&255](c[r>>2]|0,y,(c[d>>2]|0)-y|0)|0;y=c[p>>2]|0;c[z>>2]=y;A=y;break}}else{A=q}}while(0);c[d>>2]=A;c[n>>2]=1}q=c[a+32>>2]|0;if((q|0)==0){g=-1;h=-32;return(G=g,h)|0}p=dk[q&63](c[a+20>>2]|0,t,s,0)|0;q=G;f=0;if((q|0)<(f|0)|(q|0)==(f|0)&p>>>0<0>>>0){g=q;h=p;return(G=g,h)|0}p=a+120|0;c[p>>2]=(c[p>>2]|0)+1;p=c[a+4>>2]|0;if((c[l>>2]|0)==0){c[a+16>>2]=p}c[a+12>>2]=p;c[i>>2]=t;c[i+4>>2]=s}}while(0);c[a+52>>2]=0;g=s;h=t;return(G=g,h)|0}function im(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;b=i;d=c[a+60>>2]|0;e=(d|0)==0?32768:d;d=a+16|0;f=c[d>>2]|0;g=a+4|0;h=c[g>>2]|0;j=f;k=h;l=a+8|0;m=c[l>>2]|0;n=(e+j-k|0)<(m|0)?f:h;o=k-n+m|0;m=a+24|0;k=c[m>>2]|0;do{if((k|0)==0){if((c[a+12>>2]|0)>>>0<f>>>0){break}c[a+52>>2]=1}}while(0);p=a+52|0;if((c[p>>2]|0)!=0){i=b;return}q=c[a+72>>2]|0;if((q|0)!=0&(n|0)==(h|0)){r=a+68|0;s=c[r>>2]|0;if(f>>>0>s>>>0){f=a+64|0;c[f>>2]=c6[q&255](c[f>>2]|0,s,j-s|0)|0;t=c[g>>2]|0;u=c[m>>2]|0}else{t=h;u=k}c[r>>2]=t;v=u;w=t}else{v=k;w=h}L9370:do{if((v|0)!=0){do{if((c[l>>2]|0)>(e|0)){if((n|0)==(w|0)){h=A1(e)|0;k=c[g>>2]|0;do{if((h|0)==0){x=k}else{A4(k);c[g>>2]=h;c[l>>2]=e;c[a+12>>2]=h;t=a+56|0;if((c[t>>2]|0)==0){c[d>>2]=h;c[t>>2]=0;x=h;break}else{c[d>>2]=h+e;c[t>>2]=1;x=h;break}}}while(0);c[a+68>>2]=x;y=x}else{y=n}if((o|0)<(e|0)){AN(0,0,1734920,(h=i,i=i+24|0,c[h>>2]=1632168,c[h+8>>2]=1677984,c[h+16>>2]=423,h)|0);i=h;cw()}else{h=c[m>>2]|0;if((h|0)==0){break L9370}else{z=h;A=y;B=e;break}}}else{z=v;A=n;B=o}}while(0);h=c6[z&255](c[a+20>>2]|0,A,B)|0;if((h|0)>=1){k=h;t=(h|0)<0|0?-1:0;u=a+40|0;r=NU(c[u>>2]|0,c[u+4>>2]|0,k,t)|0;c[u>>2]=r;c[u+4>>2]=G;c[a+12>>2]=A;c[d>>2]=A+h;u=a+112|0;r=NU(c[u>>2]|0,c[u+4>>2]|0,k,t)|0;c[u>>2]=r;c[u+4>>2]=G;i=b;return}c[p>>2]=1;if((h|0)>=0){i=b;return}c[a+76>>2]=h;i=b;return}}while(0);c[p>>2]=1;i=b;return}function io(a,b,c){a=a|0;b=b|0;c=c|0;var d=0;d=il(a,b,c,1)|0;return(G=G,d)|0}function ip(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0;if((a|0)==0){b=-1;d=-22;return(G=b,d)|0}e=a+32|0;f=c[e>>2]|0;if((f|0)==0){b=-1;d=-38;return(G=b,d)|0}g=a+20|0;h=dk[f&63](c[g>>2]|0,0,0,65536)|0;f=G;i=0;if(!((f|0)<(i|0)|(f|0)==(i|0)&h>>>0<0>>>0)){b=f;d=h;return(G=b,d)|0}h=dk[c[e>>2]&63](c[g>>2]|0,-1,-1,2)|0;f=G;i=0;if((f|0)<(i|0)|(f|0)==(i|0)&h>>>0<0>>>0){b=f;d=h;return(G=b,d)|0}i=NU(h,f,1,0)|0;f=G;h=a+40|0;dk[c[e>>2]&63](c[g>>2]|0,c[h>>2]|0,c[h+4>>2]|0,0)|0;b=f;d=i;return(G=b,d)|0}function iq(a){a=a|0;var b=0,d=0;do{if((a|0)==0){b=0}else{d=a+52|0;if((c[d>>2]|0)==0){b=0;break}c[d>>2]=0;im(a);b=c[d>>2]|0}}while(0);return b|0}function ir(a,b){a=a|0;b=b|0;ih(a,b&255);ih(a,b>>>8&255);ih(a,b>>>16&255);ih(a,b>>>24);return}function is(a,b){a=a|0;b=b|0;ih(a,b>>>24);ih(a,b>>>16&255);ih(a,b>>>8&255);ih(a,b&255);return}function it(a,b){a=a|0;b=b|0;var c=0,d=0;if((b|0)==0){ih(a,0);c=1;return c|0}else{d=(NL(b|0)|0)+1|0;ij(a,b,d);c=d;return c|0}return 0}function iu(b,c){b=b|0;c=c|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;e=a[c]|0;if(e<<24>>24==0){f=0;ih(b,0);ih(b,0);g=f+2|0;return g|0}else{h=0;i=c;j=e}while(1){e=i+1|0;c=j&255;k=c>>>1&64;if((c&192|0)==128|(j&255)>253){f=h;l=6986;break}L9431:do{if((k&c|0)==0){m=e;n=c;o=k}else{p=c;q=k;r=e;while(1){s=r+1|0;t=(d[r]|0)-128|0;if(t>>>0>63){m=s;n=p;o=q;break L9431}u=t+(p<<6)|0;t=q<<5;if((t&u|0)==0){m=s;n=u;o=t;break}else{p=u;q=t;r=s}}}}while(0);e=(o<<1)-1&n;if(e>>>0<65536){ih(b,e&255);ih(b,e>>>8&255);v=h+2|0}else{k=e-65536|0;ih(b,k>>>10&255);ih(b,k>>>18&39|216);ih(b,k&255);ih(b,k>>>8&3|220);v=h+4|0}k=a[m]|0;if(k<<24>>24==0){f=v;l=6987;break}else{h=v;i=m;j=k}}if((l|0)==6986){ih(b,0);ih(b,0);g=f+2|0;return g|0}else if((l|0)==6987){ih(b,0);ih(b,0);g=f+2|0;return g|0}return 0}function iv(a,b){a=a|0;b=b|0;ih(a,b&255);ih(a,b>>8);return}function iw(a,b,c){a=a|0;b=b|0;c=c|0;var d=0;d=b;ih(a,d&255);ih(a,d>>>8&255);ih(a,d>>>16&255);ih(a,d>>>24);ih(a,c&255);ih(a,(c>>>8|0<<24)&255);ih(a,(c>>>16|0<<16)&255);ih(a,c>>>24|0<<8);return}function ix(a,b,c){a=a|0;b=b|0;c=c|0;ih(a,c>>>24|0<<8);ih(a,(c>>>16|0<<16)&255);ih(a,(c>>>8|0<<24)&255);ih(a,c&255);c=b;ih(a,c>>>24);ih(a,c>>>16&255);ih(a,c>>>8&255);ih(a,c&255);return}function iy(a,b){a=a|0;b=b|0;ih(a,b>>8);ih(a,b&255);return}function iz(a,b){a=a|0;b=b|0;ih(a,b>>16);ih(a,b>>>8&255);ih(a,b&255);return}function iA(a,b,c){a=a|0;b=b|0;c=c|0;return zD(zC(3)|0,a,b,c)|0}function iB(a){a=a|0;var b=0,d=0,e=0,f=0;b=a+72|0;d=a+64|0;e=c[a+68>>2]|0;f=c6[c[b>>2]&255](c[d>>2]|0,e,(c[a+12>>2]|0)-e|0)|0;c[d>>2]=f;c[b>>2]=0;return f|0}function iC(a,b,d){a=a|0;b=b|0;d=d|0;c[a+72>>2]=b;if((b|0)==0){return}c[a+64>>2]=d;c[a+68>>2]=c[a+12>>2];return}function iD(a){a=a|0;var b=0,e=0,f=0,g=0,h=0,i=0,j=0;b=a+12|0;e=c[b>>2]|0;f=a+16|0;g=c[f>>2]|0;if(e>>>0<g>>>0){h=e;i=g}else{im(a);h=c[b>>2]|0;i=c[f>>2]|0}if(h>>>0>=i>>>0){j=0;return j|0}c[b>>2]=h+1;j=d[h]|0;return j|0}function iE(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;e=a+16|0;f=a+12|0;g=a+104|0;h=a+8|0;i=a+72|0;j=a+56|0;if((d|0)>0){k=a+24|0;l=a+40|0;m=a+112|0;n=a+4|0;o=a+20|0;p=b;b=d;L9466:while(1){while(1){q=c[f>>2]|0;r=(c[e>>2]|0)-q|0;s=(r|0)>(b|0)?b:r;if((s|0)!=0){if((c[j>>2]|0)==0){t=7024;break}}if((c[g>>2]|0)==0){if((b|0)>(c[h>>2]|0)){t=7015}}else{t=7015}if((t|0)==7015){t=0;if((c[i>>2]|0)==0){t=7016;break}}im(a);if((c[e>>2]|0)==(c[f>>2]|0)){u=b;break L9466}}if((t|0)==7016){t=0;r=c[k>>2]|0;if((r|0)==0){v=s}else{v=c6[r&255](c[o>>2]|0,p,b)|0}if((v|0)<1){t=7019;break}r=v;w=(v|0)<0|0?-1:0;x=NU(c[l>>2]|0,c[l+4>>2]|0,r,w)|0;c[l>>2]=x;c[l+4>>2]=G;x=NU(c[m>>2]|0,c[m+4>>2]|0,r,w)|0;c[m>>2]=x;c[m+4>>2]=G;x=c[n>>2]|0;c[f>>2]=x;c[e>>2]=x;y=v}else if((t|0)==7024){t=0;NN(p|0,q|0,s)|0;c[f>>2]=(c[f>>2]|0)+s;y=s}x=b-y|0;if((x|0)>0){p=p+y|0;b=x}else{u=x;break}}do{if((t|0)==7019){c[a+52>>2]=1;if((v|0)>=0){u=b;break}c[a+76>>2]=v;u=b}}while(0);if((u|0)==(d|0)){z=u;t=7026}else{A=u}}else{z=d;t=7026}do{if((t|0)==7026){u=c[a+76>>2]|0;if((u|0)!=0){B=u;return B|0}if((a|0)==0){A=z;break}u=a+52|0;if((c[u>>2]|0)==0){A=z;break}c[u>>2]=0;im(a);if((c[u>>2]|0)==0){A=z;break}else{B=-541478725}return B|0}}while(0);B=d-A|0;return B|0}function iF(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;if((d|0)<0){e=-1;return e|0}f=c[a+24>>2]|0;do{if((f|0)!=0){if((c[a+56>>2]|0)==0){break}g=c6[f&255](c[a+20>>2]|0,b,d)|0;if((g|0)<=0){e=g;return e|0}h=a+40|0;i=NU(c[h>>2]|0,c[h+4>>2]|0,g,(g|0)<0|0?-1:0)|0;c[h>>2]=i;c[h+4>>2]=G;e=g;return e|0}}while(0);f=a+16|0;g=c[f>>2]|0;h=a+12|0;i=c[h>>2]|0;if((g|0)==(i|0)){j=c[a+4>>2]|0;c[h>>2]=j;c[f>>2]=j;im(a);j=c[h>>2]|0;k=(c[f>>2]|0)-j|0;l=j}else{k=g-i|0;l=i}i=(k|0)>(d|0)?d:k;NN(b|0,l|0,i)|0;c[h>>2]=(c[h>>2]|0)+i;do{if((i|0)==0){h=c[a+76>>2]|0;if((h|0)!=0){e=h;return e|0}if((a|0)==0){break}h=a+52|0;if((c[h>>2]|0)==0){break}c[h>>2]=0;im(a);if((c[h>>2]|0)==0){break}else{e=-541478725}return e|0}}while(0);e=i;return e|0}function iG(a){a=a|0;var b=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;b=a+12|0;e=c[b>>2]|0;f=a+16|0;g=c[f>>2]|0;if(e>>>0<g>>>0){h=e;i=g}else{im(a);h=c[b>>2]|0;i=c[f>>2]|0}if(h>>>0<i>>>0){g=h+1|0;c[b>>2]=g;j=d[h]|0;k=g}else{j=0;k=h}if(k>>>0<i>>>0){l=k;m=i}else{im(a);l=c[b>>2]|0;m=c[f>>2]|0}if(l>>>0>=m>>>0){n=0;o=n|j;return o|0}c[b>>2]=l+1;n=(d[l]|0)<<8;o=n|j;return o|0}function iH(a){a=a|0;var b=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;b=a+12|0;e=c[b>>2]|0;f=a+16|0;g=c[f>>2]|0;if(e>>>0<g>>>0){h=e;i=g}else{im(a);h=c[b>>2]|0;i=c[f>>2]|0}if(h>>>0<i>>>0){g=h+1|0;c[b>>2]=g;j=d[h]|0;k=g}else{j=0;k=h}if(k>>>0<i>>>0){l=k;m=i}else{im(a);l=c[b>>2]|0;m=c[f>>2]|0}if(l>>>0<m>>>0){i=l+1|0;c[b>>2]=i;n=(d[l]|0)<<8;o=i}else{n=0;o=l}l=n|j;if(o>>>0<m>>>0){p=o;q=m}else{im(a);p=c[b>>2]|0;q=c[f>>2]|0}if(p>>>0>=q>>>0){r=0;s=l|r;return s|0}c[b>>2]=p+1;r=(d[p]|0)<<16;s=l|r;return s|0}function iI(a){a=a|0;var b=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;b=a+12|0;e=c[b>>2]|0;f=a+16|0;g=c[f>>2]|0;if(e>>>0<g>>>0){h=e;i=g}else{im(a);h=c[b>>2]|0;i=c[f>>2]|0}if(h>>>0<i>>>0){g=h+1|0;c[b>>2]=g;j=d[h]|0;k=g}else{j=0;k=h}if(k>>>0<i>>>0){l=k;m=i}else{im(a);l=c[b>>2]|0;m=c[f>>2]|0}if(l>>>0<m>>>0){i=l+1|0;c[b>>2]=i;n=(d[l]|0)<<8;o=i}else{n=0;o=l}l=n|j;if(o>>>0<m>>>0){p=o;q=m}else{im(a);p=c[b>>2]|0;q=c[f>>2]|0}if(p>>>0<q>>>0){m=p+1|0;c[b>>2]=m;r=d[p]|0;s=m}else{r=0;s=p}if(s>>>0<q>>>0){t=s;u=q}else{im(a);t=c[b>>2]|0;u=c[f>>2]|0}if(t>>>0>=u>>>0){v=0;w=v|r;x=w<<16;y=l|x;return y|0}c[b>>2]=t+1;v=(d[t]|0)<<8;w=v|r;x=w<<16;y=l|x;return y|0}function iJ(a){a=a|0;var b=0;b=iI(a)|0;return(G=iI(a)|0,b|0)|0}function iK(a){a=a|0;var b=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;b=a+12|0;e=c[b>>2]|0;f=a+16|0;g=c[f>>2]|0;if(e>>>0<g>>>0){h=e;i=g}else{im(a);h=c[b>>2]|0;i=c[f>>2]|0}if(h>>>0<i>>>0){g=h+1|0;c[b>>2]=g;j=(d[h]|0)<<8;k=g}else{j=0;k=h}if(k>>>0<i>>>0){l=k;m=i}else{im(a);l=c[b>>2]|0;m=c[f>>2]|0}if(l>>>0>=m>>>0){n=0;o=n|j;return o|0}c[b>>2]=l+1;n=d[l]|0;o=n|j;return o|0}function iL(a){a=a|0;var b=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;b=a+12|0;e=c[b>>2]|0;f=a+16|0;g=c[f>>2]|0;if(e>>>0<g>>>0){h=e;i=g}else{im(a);h=c[b>>2]|0;i=c[f>>2]|0}if(h>>>0<i>>>0){g=h+1|0;c[b>>2]=g;j=(d[h]|0)<<8;k=g}else{j=0;k=h}if(k>>>0<i>>>0){l=k;m=i}else{im(a);l=c[b>>2]|0;m=c[f>>2]|0}if(l>>>0<m>>>0){i=l+1|0;c[b>>2]=i;n=d[l]|0;o=i}else{n=0;o=l}l=(n|j)<<8;if(o>>>0<m>>>0){p=o;q=m}else{im(a);p=c[b>>2]|0;q=c[f>>2]|0}if(p>>>0>=q>>>0){r=0;s=r|l;return s|0}c[b>>2]=p+1;r=d[p]|0;s=r|l;return s|0}function iM(a){a=a|0;var b=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;b=a+12|0;e=c[b>>2]|0;f=a+16|0;g=c[f>>2]|0;if(e>>>0<g>>>0){h=e;i=g}else{im(a);h=c[b>>2]|0;i=c[f>>2]|0}if(h>>>0<i>>>0){g=h+1|0;c[b>>2]=g;j=(d[h]|0)<<8;k=g}else{j=0;k=h}if(k>>>0<i>>>0){l=k;m=i}else{im(a);l=c[b>>2]|0;m=c[f>>2]|0}if(l>>>0<m>>>0){i=l+1|0;c[b>>2]=i;n=d[l]|0;o=i}else{n=0;o=l}l=(n|j)<<16;if(o>>>0<m>>>0){p=o;q=m}else{im(a);p=c[b>>2]|0;q=c[f>>2]|0}if(p>>>0<q>>>0){m=p+1|0;c[b>>2]=m;r=(d[p]|0)<<8;s=m}else{r=0;s=p}if(s>>>0<q>>>0){t=s;u=q}else{im(a);t=c[b>>2]|0;u=c[f>>2]|0}if(t>>>0>=u>>>0){v=0;w=r|l;x=w|v;return x|0}c[b>>2]=t+1;v=d[t]|0;w=r|l;x=w|v;return x|0}function iN(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;if((f|0)<1){g=-22;return g|0}h=f-1|0;f=(h|0)>(d|0)?d:h;h=b+12|0;i=b+16|0;j=0;while(1){if((j|0)>=(f|0)){k=7154;break}l=c[h>>2]|0;m=c[i>>2]|0;if(l>>>0<m>>>0){n=l;o=m}else{im(b);n=c[h>>2]|0;o=c[i>>2]|0}if(n>>>0>=o>>>0){k=7152;break}c[h>>2]=n+1;m=a[n]|0;a[e+j|0]=m;l=j+1|0;if(m<<24>>24==0){g=l;k=7162;break}else{j=l}}if((k|0)==7162){return g|0}else if((k|0)==7154){a[e+j|0]=0;n=j;while(1){if((n|0)>=(d|0)){g=d;k=7166;break}o=c[h>>2]|0;f=c[i>>2]|0;if(o>>>0<f>>>0){p=o;q=f}else{im(b);p=c[h>>2]|0;q=c[i>>2]|0}if(p>>>0>=q>>>0){k=7159;break}c[h>>2]=p+1;f=n+1|0;if((a[p]|0)==0){g=f;k=7164;break}else{n=f}}if((k|0)==7164){return g|0}else if((k|0)==7166){return g|0}else if((k|0)==7159){g=n+1|0;return g|0}}else if((k|0)==7152){a[e+j|0]=0;g=j+1|0;return g|0}return 0}function iO(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0;if((g|0)<1){h=-22;return h|0}i=b+12|0;j=b+16|0;k=f;l=g-1|0;g=0;m=f;L9678:while(1){f=(m-k|0)<(l|0);do{if(f){if((g+1|0)>=(e|0)){n=g;break L9678}o=g+2|0;if((o|0)>(e|0)){n=o;break L9678}p=c[i>>2]|0;q=c[j>>2]|0;if(p>>>0<q>>>0){r=p;s=q}else{im(b);r=c[i>>2]|0;s=c[j>>2]|0}if(r>>>0<s>>>0){q=r+1|0;c[i>>2]=q;t=d[r]|0;u=q}else{t=0;u=r}if(u>>>0<s>>>0){v=u;w=s}else{im(b);v=c[i>>2]|0;w=c[j>>2]|0}if(v>>>0<w>>>0){q=v+1|0;c[i>>2]=q;x=(d[v]|0)<<8;y=q}else{x=0;y=v}q=x|t;p=q-55296|0;if(p>>>0<2048){z=g+4|0;if((z|0)>(e|0)){n=z;break L9678}if(y>>>0<w>>>0){A=y;B=w}else{im(b);A=c[i>>2]|0;B=c[j>>2]|0}if(A>>>0<B>>>0){C=A+1|0;c[i>>2]=C;D=d[A]|0;E=C}else{D=0;E=A}if(E>>>0<B>>>0){F=E;G=B}else{im(b);F=c[i>>2]|0;G=c[j>>2]|0}if(F>>>0<G>>>0){c[i>>2]=F+1;H=(d[F]|0)<<8}else{H=0}C=H|D;if(C>>>0>1023|p>>>0>1023){n=z;break L9678}I=(p<<10)+65536+C|0;J=z}else{I=q;J=o}if((I|0)==0){n=J;break L9678}if(I>>>0>=128){K=J;L=I;break}a[m]=I&255;g=J;m=m+1|0;continue L9678}else{o=g;while(1){if((o+1|0)>=(e|0)){n=o;break L9678}q=o+2|0;if((q|0)>(e|0)){n=q;break L9678}z=c[i>>2]|0;C=c[j>>2]|0;if(z>>>0<C>>>0){M=z;N=C}else{im(b);M=c[i>>2]|0;N=c[j>>2]|0}if(M>>>0<N>>>0){C=M+1|0;c[i>>2]=C;O=d[M]|0;P=C}else{O=0;P=M}if(P>>>0<N>>>0){Q=P;R=N}else{im(b);Q=c[i>>2]|0;R=c[j>>2]|0}if(Q>>>0<R>>>0){C=Q+1|0;c[i>>2]=C;S=(d[Q]|0)<<8;T=C}else{S=0;T=Q}C=S|O;z=C-55296|0;if(z>>>0<2048){p=o+4|0;if((p|0)>(e|0)){n=p;break L9678}if(T>>>0<R>>>0){U=T;V=R}else{im(b);U=c[i>>2]|0;V=c[j>>2]|0}if(U>>>0<V>>>0){W=U+1|0;c[i>>2]=W;X=d[U]|0;Y=W}else{X=0;Y=U}if(Y>>>0<V>>>0){Z=Y;_=V}else{im(b);Z=c[i>>2]|0;_=c[j>>2]|0}if(Z>>>0<_>>>0){c[i>>2]=Z+1;$=(d[Z]|0)<<8}else{$=0}W=$|X;if(W>>>0>1023|z>>>0>1023){n=p;break L9678}aa=(z<<10)+65536+W|0;ab=p}else{aa=C;ab=q}if((aa|0)==0){n=ab;break L9678}if(aa>>>0<128){o=ab}else{K=ab;L=aa;break}}}}while(0);o=L>>>0>65535;q=o?16:0;C=o?L>>>16:L;if((C&65280|0)==0){ac=C;ad=q}else{ac=C>>>8;ad=q|8}q=(ad+4+(d[247648+ac|0]|0)|0)/5|0;C=(q*6|0)-6|0;if(f){a[m]=(L>>>(C>>>0)|256-(256>>>(q>>>0)))&255;ae=C;af=m+1|0}else{ae=C;af=m}while(1){if((af-k|0)>=(l|0)){ag=ae;break}if((ae|0)<=5){g=K;m=af;continue L9678}C=ae-6|0;a[af]=(L>>>(C>>>0)&63|128)&255;ae=C;af=af+1|0}while(1){if((ag|0)<=5){g=K;m=af;continue L9678}ag=ag-6|0}}a[m]=0;h=n;return h|0}function iP(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0;if((g|0)<1){h=-22;return h|0}i=b+12|0;j=b+16|0;k=f;l=g-1|0;g=0;m=f;L9768:while(1){f=(m-k|0)<(l|0);n=g;while(1){if((n+1|0)>=(e|0)){o=n;break L9768}p=n+2|0;if((p|0)>(e|0)){o=p;break L9768}q=c[i>>2]|0;r=c[j>>2]|0;if(q>>>0<r>>>0){s=q;t=r}else{im(b);s=c[i>>2]|0;t=c[j>>2]|0}if(s>>>0<t>>>0){r=s+1|0;c[i>>2]=r;u=(d[s]|0)<<8;v=r}else{u=0;v=s}if(v>>>0<t>>>0){w=v;x=t}else{im(b);w=c[i>>2]|0;x=c[j>>2]|0}if(w>>>0<x>>>0){r=w+1|0;c[i>>2]=r;y=d[w]|0;z=r}else{y=0;z=w}r=y|u;q=r-55296|0;if(q>>>0<2048){A=n+4|0;if((A|0)>(e|0)){o=A;break L9768}if(z>>>0<x>>>0){B=z;C=x}else{im(b);B=c[i>>2]|0;C=c[j>>2]|0}if(B>>>0<C>>>0){D=B+1|0;c[i>>2]=D;E=(d[B]|0)<<8;F=D}else{E=0;F=B}if(F>>>0<C>>>0){G=F;H=C}else{im(b);G=c[i>>2]|0;H=c[j>>2]|0}if(G>>>0<H>>>0){c[i>>2]=G+1;I=d[G]|0}else{I=0}D=I|E;if(D>>>0>1023|q>>>0>1023){o=A;break L9768}J=(q<<10)+65536+D|0;K=A}else{J=r;K=p}if((J|0)==0){o=K;break L9768}if(J>>>0>=128){break}if(f){L=7261;break}else{n=K}}if((L|0)==7261){L=0;a[m]=J&255;g=K;m=m+1|0;continue}n=J>>>0>65535;p=n?16:0;r=n?J>>>16:J;if((r&65280|0)==0){M=r;N=p}else{M=r>>>8;N=p|8}p=(N+4+(d[247648+M|0]|0)|0)/5|0;r=(p*6|0)-6|0;if(f){a[m]=(J>>>(r>>>0)|256-(256>>>(p>>>0)))&255;O=r;P=m+1|0}else{O=r;P=m}while(1){if((P-k|0)>=(l|0)){Q=O;break}if((O|0)<=5){g=K;m=P;continue L9768}r=O-6|0;a[P]=(J>>>(r>>>0)&63|128)&255;O=r;P=P+1|0}while(1){if((Q|0)<=5){g=K;m=P;continue L9768}Q=Q-6|0}}a[m]=0;h=o;return h|0}function iQ(a){a=a|0;var b=0,c=0;b=iM(a)|0;c=iM(a)|0;return(G=b|0,c)|0}function iR(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;d=c[b+20>>2]|0;e=(d|0)==0?32768:d;f=A1(e)|0;if((f|0)==0){g=-12;return g|0}h=b+16|0;i=c[h>>2]|0;j=A7(128)|0;if((j|0)==0){c[a>>2]=0;A4(f);g=-12;return g|0}c[j+4>>2]=f;c[j+8>>2]=e;c[j+12>>2]=f;c[j+20>>2]=b;k=j+104|0;c[k>>2]=0;if((i&2|0)==0){c[j+16>>2]=f;c[j+56>>2]=0}else{c[j+16>>2]=f+e;c[j+56>>2]=1}c[j+28>>2]=174;c[j+24>>2]=94;c[j+32>>2]=52;c[j+76>>2]=0;NM(j+40|0,0,16);c[j+72>>2]=0;c[j+80>>2]=0;c[j+84>>2]=0;c[a>>2]=j;c[k>>2]=c[h>>2]&32768;c[j+88>>2]=(c[b+24>>2]|0)==0;c[j+60>>2]=d;d=c[b+4>>2]|0;if((d|0)!=0){c[j+80>>2]=c[d+32>>2];c[j+84>>2]=c[d+36>>2]}c[j>>2]=236136;g=0;return g|0}function iS(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;e=c[b>>2]|0;if((c[a+56>>2]|0)!=0){A6(b);f=-22;return f|0}g=a+16|0;h=a+4|0;i=(c[g>>2]|0)-(c[h>>2]|0)|0;j=a+40|0;k=NV(c[j>>2]|0,c[j+4>>2]|0,i,(i|0)<0|0?-1:0)|0;l=G;m=d;n=(d|0)<0|0?-1:0;if((l|0)>(n|0)|(l|0)==(n|0)&k>>>0>m>>>0){A6(b);f=-22;return f|0}o=NV(m,n,k,l)|0;l=o;o=i+d-l|0;k=a+8|0;n=c[k>>2]|0;m=(n|0)>(o|0)?n:o;do{if((m|0)>(d|0)){n=A3(e,1,m)|0;c[b>>2]=n;if((n|0)==0){f=-12}else{p=n;break}return f|0}else{p=e}}while(0);if((o|0)>(d|0)){e=p+d|0;b=(c[h>>2]|0)+l|0;n=i-l|0;NN(e|0,b|0,n)|0;q=o}else{q=d}A4(c[h>>2]|0);c[h>>2]=p;c[a+12>>2]=p;c[k>>2]=m;c[j>>2]=q;c[j+4>>2]=(q|0)<0|0?-1:0;c[g>>2]=p+q;c[a+52>>2]=0;c[a+48>>2]=0;f=0;return f|0}function iT(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=i;i=i+8|0;f=e|0;g=h6(f,b,d,0,0)|0;do{if((g|0)<0){h=g}else{d=iR(a,c[f>>2]|0)|0;if((d|0)>=0){h=0;break}b=c[f>>2]|0;h7(b)|0;h=d}}while(0);i=e;return h|0}function iU(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0;g=i;i=i+8|0;h=g|0;j=h6(h,b,d,e,f)|0;do{if((j|0)<0){k=j}else{f=iR(a,c[h>>2]|0)|0;if((f|0)>=0){k=0;break}e=c[h>>2]|0;h7(e)|0;k=f}}while(0);i=g;return k|0}function iV(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;b=i;if((a|0)==0){d=0;i=b;return d|0}e=a+12|0;f=c[e>>2]|0;g=a+4|0;h=c[g>>2]|0;do{if(f>>>0>h>>>0){j=f-h|0;k=c[a+28>>2]|0;do{if((k|0)!=0){l=a+76|0;if((c[l>>2]|0)!=0){break}m=c6[k&255](c[a+20>>2]|0,h,j)|0;if((m|0)>=0){break}c[l>>2]=m}}while(0);k=a+124|0;c[k>>2]=(c[k>>2]|0)+1;k=a+40|0;m=NU(c[k>>2]|0,c[k+4>>2]|0,j,(j|0)<0|0?-1:0)|0;c[k>>2]=m;c[k+4>>2]=G;k=c[a+72>>2]|0;if((k|0)==0){n=c[g>>2]|0;break}else{m=a+64|0;l=a+68|0;o=c[l>>2]|0;c[m>>2]=c6[k&255](c[m>>2]|0,o,(c[e>>2]|0)-o|0)|0;o=c[g>>2]|0;c[l>>2]=o;n=o;break}}else{n=h}}while(0);c[e>>2]=n;c[a+48>>2]=0;n=c[a+20>>2]|0;A6(g);g=a;if((c[a+56>>2]|0)==0){e=a+112|0;h=c[e+4>>2]|0;f=c[a+120>>2]|0;AN(g,48,1645008,(p=i,i=i+24|0,c[p>>2]=c[e>>2],c[p+8>>2]=h,c[p+16>>2]=f,p)|0);i=p}else{f=c[a+124>>2]|0;AN(g,48,1661776,(p=i,i=i+16|0,c[p>>2]=c[a+120>>2],c[p+8>>2]=f,p)|0);i=p}A4(g);d=h7(n)|0;i=b;return d|0}function iW(a){a=a|0;var b=0;b=iV(c[a>>2]|0)|0;c[a>>2]=0;return b|0}function iX(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;i=i+4112|0;f=e|0;g=f;c[g>>2]=d;c[g+4>>2]=0;g=e+16|0;d=cs(g|0,4096,b|0,f|0)|0;ij(a,g,NL(g|0)|0);i=e;return d|0}function iY(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0;g=c[a+84>>2]|0;if((g|0)==0){h=-1;i=-38;return(G=h,i)|0}j=c[a+20>>2]|0;k=c4[g&63](j,b,d,e,f)|0;f=G;e=-1;if(!((f|0)>(e|0)|(f|0)==(e|0)&k>>>0>-1>>>0)){h=f;i=k;return(G=h,i)|0}c[a+12>>2]=c[a+16>>2];e=dk[c[a+32>>2]&63](j,0,0,1)|0;j=G;d=-1;if(!((j|0)>(d|0)|(j|0)==(d|0)&e>>>0>-1>>>0)){d=(e|0)==(-38|0)&(j|0)==(-1|0);return(G=d?f:j,d?k:e)|0}d=a+40|0;c[d>>2]=e;c[d+4>>2]=j;h=f;i=k;return(G=h,i)|0}function iZ(a){a=a|0;var b=0,d=0,e=0,f=0;b=A7(1048)|0;if((b|0)==0){d=-12;return d|0}c[b+16>>2]=1024;e=b+20|0;f=A7(128)|0;if((f|0)==0){c[a>>2]=0;A4(b);d=-12;return d|0}else{c[f+4>>2]=e;c[f+8>>2]=1024;c[f+12>>2]=e;c[f+20>>2]=b;c[f+104>>2]=0;c[f+16>>2]=b+1044;c[f+56>>2]=1;c[f+28>>2]=108;c[f+24>>2]=0;c[f+32>>2]=40;c[f+76>>2]=0;NM(f+40|0,0,16);c[f+88>>2]=1;c[f+72>>2]=0;c[f+80>>2]=0;c[f+84>>2]=0;c[a>>2]=f;c[f+60>>2]=0;d=0;return d|0}return 0}function i_(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;if((b|0)<1){d=-1;return d|0}e=by(b|0,24)|0;if(G){d=-1;return d|0}f=A7(e)|0;if((f|0)==0){d=-12;return d|0}c[f+16>>2]=b;e=f+20|0;g=A7(128)|0;if((g|0)==0){c[a>>2]=0;A4(f);d=-12;return d|0}else{c[g+4>>2]=e;c[g+8>>2]=b;c[g+12>>2]=e;c[g+20>>2]=f;c[g+104>>2]=0;c[g+16>>2]=f+(b+20);c[g+56>>2]=1;c[g+28>>2]=92;c[g+24>>2]=0;c[g+32>>2]=0;c[g+76>>2]=0;NM(g+40|0,0,16);c[g+88>>2]=1;c[g+72>>2]=0;c[g+80>>2]=0;c[g+84>>2]=0;c[a>>2]=g;c[g+60>>2]=b;d=0;return d|0}return 0}function i$(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;d=a+20|0;e=c[d>>2]|0;if((c[a+60>>2]|0)==0){ij(a,3014552,16);f=16}else{f=0}g=a+12|0;h=c[g>>2]|0;i=a+4|0;j=c[i>>2]|0;do{if(h>>>0>j>>>0){k=h-j|0;l=c[a+28>>2]|0;do{if((l|0)!=0){m=a+76|0;if((c[m>>2]|0)!=0){break}n=c6[l&255](c[d>>2]|0,j,k)|0;if((n|0)>=0){break}c[m>>2]=n}}while(0);l=a+124|0;c[l>>2]=(c[l>>2]|0)+1;l=a+40|0;n=NU(c[l>>2]|0,c[l+4>>2]|0,k,(k|0)<0|0?-1:0)|0;c[l>>2]=n;c[l+4>>2]=G;l=c[a+72>>2]|0;if((l|0)==0){o=c[i>>2]|0;break}else{n=a+64|0;m=a+68|0;p=c[m>>2]|0;c[n>>2]=c6[l&255](c[n>>2]|0,p,(c[g>>2]|0)-p|0)|0;p=c[i>>2]|0;c[m>>2]=p;o=p;break}}else{o=j}}while(0);c[g>>2]=o;c[a+48>>2]=0;c[b>>2]=c[e+12>>2];b=c[e+4>>2]|0;A4(e);A4(a);return b-f|0}function i0(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;f=cV(e|0)|0;g=b;h=c[g>>2]|0;i=by(h|0,4)|0;j=i;i=b+8|0;k=c[i>>2]|0;if(G|j>>>0>1073741823){l=-1;return l|0}do{if(j>>>0>k>>>0){m=k;while(1){if((m|0)==0){n=j}else{n=m+1+(m>>>1)|0}if(j>>>0>n>>>0){m=n}else{break}}if(n>>>0<=k>>>0){o=7375;break}m=b+12|0;p=A3(c[m>>2]|0,1,n)|0;c[m>>2]=p;if((p|0)==0){l=-12;return l|0}else{c[i>>2]=n;q=c[g>>2]|0;r=p;break}}else{o=7375}}while(0);if((o|0)==7375){q=h;r=c[b+12>>2]|0}h=r+q|0;y=f;a[h]=y&255;y=y>>8;a[h+1|0]=y&255;y=y>>8;a[h+2|0]=y&255;y=y>>8;a[h+3|0]=y&255;c[g>>2]=j;h=b+4|0;if((j|0)>(c[h>>2]|0)){c[h>>2]=j}f=by(j|0,e|0)|0;q=f;f=c[i>>2]|0;if(G|q>>>0>1073741823){l=-1;return l|0}do{if(q>>>0>f>>>0){r=f;while(1){if((r|0)==0){s=q}else{s=r+1+(r>>>1)|0}if(q>>>0>s>>>0){r=s}else{break}}if(s>>>0<=f>>>0){o=7386;break}r=b+12|0;n=A3(c[r>>2]|0,1,s)|0;c[r>>2]=n;if((n|0)==0){l=-12;return l|0}else{c[i>>2]=s;t=c[g>>2]|0;u=n;break}}else{o=7386}}while(0);if((o|0)==7386){t=j;u=c[b+12>>2]|0}b=u+t|0;NN(b|0,d|0,e)|0;c[g>>2]=q;if((q|0)<=(c[h>>2]|0)){l=e;return l|0}c[h>>2]=q;l=e;return l|0}function i1(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=a;f=c[e>>2]|0;g=by(f|0,d|0)|0;h=g;g=a+8|0;i=c[g>>2]|0;if(G|h>>>0>1073741823){j=-1;return j|0}do{if(h>>>0>i>>>0){k=i;while(1){if((k|0)==0){l=h}else{l=k+1+(k>>>1)|0}if(h>>>0>l>>>0){k=l}else{break}}if(l>>>0<=i>>>0){m=7404;break}k=a+12|0;n=A3(c[k>>2]|0,1,l)|0;c[k>>2]=n;if((n|0)==0){j=-12;return j|0}else{c[g>>2]=l;o=c[e>>2]|0;p=n;break}}else{m=7404}}while(0);if((m|0)==7404){o=f;p=c[a+12>>2]|0}f=p+o|0;NN(f|0,b|0,d)|0;c[e>>2]=h;e=a+4|0;if((h|0)<=(c[e>>2]|0)){j=d;return j|0}c[e>>2]=h;j=d;return j|0}function i2(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0;if((e|0)==2){f=c[a+4>>2]|0;g=NU(f,(f|0)<0|0?-1:0,b,d)|0;h=G;i=g}else if((e|0)==1){e=c[a>>2]|0;g=NU(e,(e|0)<0|0?-1:0,b,d)|0;h=G;i=g}else{h=d;i=b}b=0;if(h>>>0>b>>>0|h>>>0==b>>>0&i>>>0>2147483647>>>0){j=-1;k=-1;return(G=j,k)|0}c[a>>2]=i;j=0;k=0;return(G=j,k)|0}function i3(a,c){a=a|0;c=c|0;var d=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;if(c>>>0>2){d=0;return d|0}else{f=0}do{g=bI(a|0,134736+((e[134720+(f<<1)>>1]|0)*6|0)|0,e[134728+(f<<1)>>1]|0|0,6,16)|0;h=g;f=f+1|0;i=(g|0)!=0;}while((f|0)<3&(i^1));if(!i){d=0;return d|0}i=b[134720+(c<<1)>>1]|0;f=i&65535;j=134728+(c<<1)|0;if(h>>>0<(134736+(f*6|0)|0)>>>0){k=7428}else{if(h>>>0<(134736+(((e[j>>1]|0)+f|0)*6|0)|0)>>>0){l=h}else{k=7428}}do{if((k|0)==7428){h=b[g+4>>1]|0;m=h&65535;if((h&65535)>=(i&65535)){if((m|0)<((e[j>>1]|0)+f|0)){l=134736+(m*6|0)|0;break}}h=b[134740+(m*6|0)>>1]|0;m=h&65535;if((h&65535)>=(i&65535)){if((m|0)<((e[j>>1]|0)+f|0)){l=134736+(m*6|0)|0;break}}if((c|0)==1){n=0}else{d=0;return d|0}do{o=bI(a|0,134736+((e[134720+(n<<1)>>1]|0)*6|0)|0,e[134728+(n<<1)>>1]|0|0,6,16)|0;p=o;n=n+1|0;q=(o|0)!=0;}while((n|0)<3&(q^1));if(!q){d=0;return d|0}do{if(o>>>0>=134736&o>>>0<137640){r=p}else{m=b[o+4>>1]|0;h=m&65535;if((m&65535)<484){r=134736+(h*6|0)|0;break}m=b[134740+(h*6|0)>>1]|0;if((m&65535)<484){r=134736+((m&65535)*6|0)|0;break}else{d=0}return d|0}}while(0);d=r|0;return d|0}}while(0);d=l|0;return d|0}function i4(a,b){a=a|0;b=b|0;return a8(a|0,b|0)|0}function i5(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0;e=i;i=i+88|0;f=e|0;g=e+8|0;c[f>>2]=b;h=c[a+8>>2]|0;yY(b,1677944,f)|0;b=(d&2|0)!=0;do{if((d&1|0)==0|b^1){if(!b){j=0;break}j=(c[h+8>>2]|0)==0?65:577}else{j=(c[h+8>>2]|0)==0?66:578}}while(0);b=bW(c[f>>2]|0,j|0,(j=i,i=i+8|0,c[j>>2]=438,j)|0)|0;i=j;if((b|0)==-1){k=-(c[(ch()|0)>>2]|0)|0;i=e;return k|0}c[h+4>>2]=b;if((ck(b|0,g|0)|0)==0){l=(c[g+12>>2]&61440|0)==4096|0}else{l=0}c[a+24>>2]=l;k=0;i=e;return k|0}function i6(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=a1(c[(c[a+8>>2]|0)+4>>2]|0,b|0,d|0)|0;if((e|0)!=-1){f=e;return f|0}f=-(c[(ch()|0)>>2]|0)|0;return f|0}function i7(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=cI(c[(c[a+8>>2]|0)+4>>2]|0,b|0,d|0)|0;if((e|0)!=-1){f=e;return f|0}f=-(c[(ch()|0)>>2]|0)|0;return f|0}function i8(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0;d=i;i=i+80|0;f=d|0;g=c[(c[a+8>>2]|0)+4>>2]|0;do{if((e|0)==65536){if((ck(g|0,f|0)|0)<0){h=-(c[(ch()|0)>>2]|0)|0;break}if((c[f+12>>2]&61440|0)==4096){h=0;break}h=c[f+36>>2]|0}else{a=aO(g|0,b|0,e|0)|0;if((a|0)>=0){h=a;break}h=-(c[(ch()|0)>>2]|0)|0}}while(0);i=d;return(G=(h|0)<0|0?-1:0,h)|0}function i9(a){a=a|0;return bh(c[(c[a+8>>2]|0)+4>>2]|0)|0}function ja(a){a=a|0;return c[(c[a+8>>2]|0)+4>>2]|0}function jb(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=a+12|0;if((bn(c[d>>2]|0,0)|0)<0){e=-(c[(ch()|0)>>2]|0)|0;return e|0}if((b&1|0)==0){f=0}else{f=(bn(c[d>>2]|0,4)|0)>>>31^1}if((b&2|0)==0){e=f;return e|0}else{return((bn(c[d>>2]|0,2)|0)>-1?f|2:f)|0}return 0}function jc(a){a=a|0;return c[((a|0)==0?2225200:a+24|0)>>2]|0}function jd(a){a=a|0;return c[((a|0)==0?2225192:a+40|0)>>2]|0}function je(a){a=a|0;var b=0,d=0;c[a+24>>2]=0;b=a;if(((E=c[556300]|0,(c[556300]|0)==0?(c[556300]=b)|0:0,E)|0)==0){return}else{d=2225200}do{d=(c[d>>2]|0)+24|0;a=d;}while(((E=c[a>>2]|0,(c[a>>2]|0)==0?(c[a>>2]=b)|0:0,E)|0)!=0);return}function jf(a){a=a|0;var b=0,d=0;c[a+40>>2]=0;b=a;if(((E=c[556298]|0,(c[556298]|0)==0?(c[556298]=b)|0:0,E)|0)==0){return}else{d=2225192}do{d=(c[d>>2]|0)+40|0;a=d;}while(((E=c[a>>2]|0,(c[a>>2]|0)==0?(c[a>>2]=b)|0:0,E)|0)!=0);return}function jg(b,c){b=b|0;c=c|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0;d=i;i=i+32|0;e=d|0;L10109:do{if((b|0)==0){f=0}else{g=br(b|0,46)|0;if((g|0)==0){f=0;break}h=g+1|0;g=e|0;j=e;k=g;l=c;L10112:while(1){m=a[l]|0;do{if(!((m<<24>>24|0)==0|(m<<24>>24|0)==44)){if((k-j|0)>>>0>=31){break}a[k]=m;k=k+1|0;l=l+1|0;continue L10112}}while(0);a[k]=0;if((y4(g,h)|0)==0){f=1;break L10109}if((a[l]|0)==0){f=0;break}else{k=g;l=l+1|0}}}}while(0);i=d;return f|0}function jh(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0;f=i;i=i+32|0;g=f|0;h=c[556298]|0;if((h|0)==0){j=0;i=f;return j|0}k=(e|0)==0;l=(d|0)==0;m=g|0;n=g;if((b|0)==0){g=0;o=0;p=h;while(1){q=c[p+8>>2]|0;if((q|0)==0|k){r=0}else{r=(a8(q|0,e|0)|0)==0?10:0}do{if(l){s=r}else{q=c[p+12>>2]|0;if((q|0)==0){s=r;break}t=br(d|0,46)|0;L10133:do{if((t|0)==0){u=1}else{v=t+1|0;w=m;x=q;while(1){y=a[x]|0;do{if((y<<24>>24|0)==0|(y<<24>>24|0)==44){z=7520}else{if((w-n|0)>>>0>=31){z=7520;break}a[w]=y;A=w+1|0}}while(0);if((z|0)==7520){z=0;a[w]=0;if((y4(m,v)|0)==0){u=0;break L10133}if((a[x]|0)==0){u=1;break L10133}else{A=m}}w=A;x=x+1|0}}}while(0);s=u?r:r|5}}while(0);q=(s|0)>(o|0);t=q?p:g;x=c[((p|0)==0?2225192:p+40|0)>>2]|0;if((x|0)==0){j=t;break}else{g=t;o=q?s:o;p=x}}i=f;return j|0}else{B=0;C=0;D=h}while(1){h=c[D>>2]|0;L10149:do{if((h|0)==0){E=0}else{p=NL(b|0)|0;o=h;while(1){s=bi(o|0,44)|0;if((s|0)==0){break}g=s-o|0;if((y5(b,o,(g|0)>(p|0)?g:p)|0)==0){E=100;break L10149}else{o=s+1|0}}E=(y4(b,o)|0)==0?100:0}}while(0);h=c[D+8>>2]|0;if((h|0)==0|k){F=E}else{F=(a8(h|0,e|0)|0)==0?E|10:E}do{if(l){G=F}else{h=c[D+12>>2]|0;if((h|0)==0){G=F;break}p=br(d|0,46)|0;L10162:do{if((p|0)==0){H=1}else{s=p+1|0;g=m;r=h;while(1){u=a[r]|0;do{if((u<<24>>24|0)==0|(u<<24>>24|0)==44){z=7541}else{if((g-n|0)>>>0>=31){z=7541;break}a[g]=u;I=g+1|0}}while(0);if((z|0)==7541){z=0;a[g]=0;if((y4(m,s)|0)==0){H=0;break L10162}if((a[r]|0)==0){H=1;break L10162}else{I=m}}g=I;r=r+1|0}}}while(0);G=H?F:F+5|0}}while(0);h=(G|0)>(C|0);p=h?D:B;o=c[((D|0)==0?2225192:D+40|0)>>2]|0;if((o|0)==0){j=p;break}else{B=p;C=h?G:C;D=o}}i=f;return j|0}function ji(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0;e=c[a>>2]|0;if((a8(e|0,1713120)|0)==0){g=7551}else{if((a8(e|0,1734728)|0)==0){g=7551}else{h=a}}if((g|0)==7551){h=jh(0,d,0)|0}if((f|0)==0){i=c[h+20>>2]|0;return i|0}else if((f|0)==1){i=c[h+16>>2]|0;return i|0}else if((f|0)==3){i=c[h+24>>2]|0;return i|0}else{i=0;return i|0}return 0}function jj(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0;b=c[556300]|0;if((b|0)==0){d=0;return d|0}e=(a|0)==0;f=b;L10194:while(1){b=c[f>>2]|0;if(!(e|(b|0)==0)){g=NL(a|0)|0;h=b;while(1){b=bi(h|0,44)|0;if((b|0)==0){break}i=b-h|0;if((y5(a,h,(i|0)>(g|0)?i:g)|0)==0){d=f;j=7572;break L10194}else{h=b+1|0}}if((y4(a,h)|0)==0){d=f;j=7571;break}}g=c[((f|0)==0?2225200:f+24|0)>>2]|0;if((g|0)==0){d=0;j=7573;break}else{f=g}}if((j|0)==7573){return d|0}else if((j|0)==7572){return d|0}else if((j|0)==7571){return d|0}return 0}function jk(b,c){b=b|0;c=c|0;var e=0;if((d[b]|0)!=(a[c]|0)){e=0;return e|0}if((d[b+1|0]|0)!=(a[c+1|0]|0)){e=0;return e|0}if((d[b+2|0]|0)!=(a[c+2|0]|0)){e=0;return e|0}if((a[b+3|0]|0)==-1){e=0;return e|0}if((a[b+4|0]|0)==-1){e=0;return e|0}if((a[b+6|0]|0)<=-1){e=0;return e|0}if((a[b+7|0]|0)<=-1){e=0;return e|0}if((a[b+8|0]|0)<=-1){e=0;return e|0}e=(d[b+9|0]|0)>>>7^1;return e|0}function jl(b){b=b|0;return((d[b+7|0]|0)<<14&2080768|(d[b+6|0]|0)<<21&266338304|(d[b+8|0]|0)<<7&16256|a[b+9|0]&127)+((a[b+5|0]&16)==0?10:20)|0}function jm(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0;g=i;i=i+216|0;h=g|0;j=g+8|0;k=g+136|0;l=g+144|0;m=g+152|0;n=g+160|0;o=g+168|0;p=g+176|0;q=g+200|0;r=b+16|0;s=q|0;t=q+1|0;u=e+1|0;v=q+2|0;w=e+2|0;x=q+3|0;y=q+4|0;z=q+6|0;A=q+7|0;B=q+8|0;C=q+9|0;D=q+5|0;q=h|0;E=k;F=m;H=b;I=h+4|0;J=h+3|0;K=b+1140|0;L=(f|0)==0;M=h+1|0;while(1){N=il(c[r>>2]|0,0,0,1)|0;O=G;if((iE(c[r>>2]|0,s,10)|0)!=10){P=7596;break}if((d[s]|0)!=(a[e]|0)){P=7674;break}if((d[t]|0)!=(a[u]|0)){P=7674;break}if((d[v]|0)!=(a[w]|0)){P=7674;break}h=a[x]|0;if(h<<24>>24==-1){P=7674;break}if((a[y]|0)==-1){P=7674;break}Q=a[z]|0;if(Q<<24>>24<=-1){P=7674;break}R=a[A]|0;if(R<<24>>24<=-1){P=7674;break}S=a[B]|0;if(S<<24>>24<=-1){P=7674;break}T=a[C]|0;if(T<<24>>24<=-1){P=7674;break}U=(R&255)<<14&2080768|(Q&255)<<21&266338304|(S&255)<<7&16256|T&127;T=a[D]|0;S=il(c[r>>2]|0,0,0,1)|0;Q=NU(S,G,U,0)|0;S=G;c[k>>2]=0;c[l>>2]=0;c[m>>2]=0;c[n>>2]=0;R=h&255;V=T&255;AN(H,48,1612232,(W=i,i=i+24|0,c[W>>2]=R,c[W+8>>2]=V,c[W+16>>2]=U,W)|0);i=W;do{if((R|0)==3|(R|0)==4){T=V&128;if((V&64|0)==0){X=U;Y=10;Z=1;_=T;$=1;P=7612;break}aa=c[r>>2]|0;ab=(iD(aa)|0)<<7&16256;ac=((iD(aa)|0)&127|ab)<<7;ab=(ac|(iD(aa)|0)&127)<<7;ac=ab|(iD(aa)|0)&127;aa=h<<24>>24==4?ac-4|0:ac;if((aa|0)<0){ad=1745104;P=7673;break}ac=c[r>>2]|0;ab=aa;ae=(aa|0)<0|0?-1:0;io(ac,ab,ae)|0;ae=U-4-aa|0;if((ae|0)<0){ad=1744752;P=7673}else{X=ae;Y=10;Z=1;_=T;$=1;P=7612}}else if((R|0)==2){if((V&64|0)!=0){ad=1611912;P=7673;break}X=U;Y=6;Z=0;_=V&128;$=0;P=7612}else{ad=1611656;P=7673}}while(0);do{if((P|0)==7673){P=0;AN(H,32,1737488,(W=i,i=i+16|0,c[W>>2]=R,c[W+8>>2]=ad,W)|0);i=W;af=S;ag=Q}else if((P|0)==7612){P=0;L10256:do{if((X|0)>=(Y|0)){U=h<<24>>24==3;T=0;ae=X;L10258:while(1){aa=ae;while(1){ab=c[r>>2]|0;if($){iE(ab,q,4)|0;a[I]=0;ac=c[r>>2]|0;if(U){ah=iM(ac)|0}else{ai=(iD(ac)|0)<<7&16256;aj=((iD(ac)|0)&127|ai)<<7;ai=(aj|(iD(ac)|0)&127)<<7;ah=ai|(iD(ac)|0)&127}ac=iK(c[r>>2]|0)|0;ak=ac&2;al=ac;am=ah}else{iE(ab,q,3)|0;a[J]=0;ak=0;al=0;am=iL(c[r>>2]|0)|0}if(am>>>0>268435456){break L10256}aa=aa-Y-am|0;if((aa|0)<0){break L10256}ab=il(c[r>>2]|0,0,0,1)|0;an=NU(ab,G,am,0)|0;ao=G;if((am|0)!=0){break}if((a[q]|0)!=0){AN(H,48,1744032,(W=i,i=i+8|0,c[W>>2]=q,W)|0);i=W}if((aa|0)<(Y|0)){break L10256}}if((al&1|0)==0){c[o>>2]=am;ap=am}else{if(am>>>0<4){break L10256}c[o>>2]=iM(c[r>>2]|0)|0;ap=am-4|0}ab=al&8;L10282:do{if((al&4|0)==0){ac=a[q]|0;do{if(ac<<24>>24==84){aq=T}else{if(L){ar=T}else{ai=jo(q,Z)|0;if((ai|0)==0){ar=0}else{aq=ai;break}}if(ac<<24>>24==0){break L10258}else{as=ar;break L10282}}}while(0);ac=c[r>>2]|0;ai=ak|_;aj=(ai|0)==0;at=(ab|0)==0;do{if((ai|ab|0)!=0){oP(E,l,ap);if((c[k>>2]|0)!=0){break}AN(H,16,1621336,(W=i,i=i+8|0,c[W>>2]=ap,W)|0);i=W;as=aq;break L10282}}while(0);if(aj){au=ac;av=ap}else{ai=il(c[r>>2]|0,0,0,1)|0;aw=NU(ai,G,ap,0)|0;ai=G;ax=c[k>>2]|0;ay=il(c[r>>2]|0,0,0,1)|0;az=G;L10297:do{if((az|0)<(ai|0)|(az|0)==(ai|0)&ay>>>0<aw>>>0){aA=NU(aw,ai,-1,-1)|0;aB=G;aC=ax;while(1){if((aC-(c[k>>2]|0)|0)>>>0>=ap>>>0){aD=aC;break L10297}aE=c[r>>2]|0;if((c[aE+52>>2]|0)!=0){aD=aC;break L10297}aF=iD(aE)|0;aE=aC+1|0;a[aC]=aF&255;do{if((aF&255|0)==255){aG=il(c[r>>2]|0,0,0,1)|0;aH=G;if(!((aH|0)<(aB|0)|(aH|0)==(aB|0)&aG>>>0<aA>>>0)){aI=aE;break}if((aE-(c[k>>2]|0)|0)>>>0>=ap>>>0){aI=aE;break}aG=c[r>>2]|0;if((c[aG+52>>2]|0)!=0){aI=aE;break}aH=iD(aG)|0;if((aH&255|0)==0){aJ=(iD(c[r>>2]|0)|0)&255}else{aJ=aH&255}a[aE]=aJ;aI=aC+2|0}else{aI=aE}}while(0);aE=il(c[r>>2]|0,0,0,1)|0;aF=G;if((aF|0)<(ai|0)|(aF|0)==(ai|0)&aE>>>0<aw>>>0){aC=aI}else{aD=aI;break}}}else{aD=ax}}while(0);ax=c[k>>2]|0;aw=aD;ai=aw-ax|0;ie(j,ax,ai,0,0,0,0,0)|0;au=j;av=aw-(c[k>>2]|0)|0}do{if(at){aK=au;aL=av}else{aw=c[o>>2]|0;AN(H,48,1739240,(W=i,i=i+24|0,c[W>>2]=q,c[W+8>>2]=av,c[W+16>>2]=aw,W)|0);i=W;oP(F,n,aw);ai=c[m>>2]|0;if((ai|0)==0){AN(H,16,1738912,(W=i,i=i+8|0,c[W>>2]=aw,W)|0);i=W;as=aq;break L10282}do{if(aj){aw=iE(c[r>>2]|0,c[k>>2]|0,av)|0;if((aw|0)<0){AN(H,16,1738720,(W=i,i=i+1|0,i=i+7&-8,c[W>>2]=0,W)|0);i=W;as=aq;break L10282}else{aM=aw;aN=c[m>>2]|0;break}}else{aM=av;aN=ai}}while(0);ai=bm(aN|0,o|0,c[k>>2]|0,aM|0)|0;if((ai|0)==0){aw=c[m>>2]|0;ax=c[o>>2]|0;ie(j,aw,ax,0,0,0,0,0)|0;aK=j;aL=c[o>>2]|0;break}else{AN(H,16,1738368,(W=i,i=i+8|0,c[W>>2]=ai,W)|0);i=W;as=aq;break L10282}}}while(0);if((a[q]|0)==84){jw(b,aK,aL,K,q);as=aq;break}else{cX[c[aq+8>>2]&255](b,aK,aL,q,f);as=aq;break}}else{AN(H,24,1739496,(W=i,i=i+16|0,c[W>>2]=(ab|0)==0?1742312:1740144,c[W+8>>2]=q,W)|0);i=W;aj=c[r>>2]|0;at=ap;ai=0;io(aj,at,ai)|0;as=T}}while(0);il(c[r>>2]|0,an,ao,0)|0;if((aa|0)<(Y|0)){break L10256}else{T=as;ae=aa}}if((a[M]|0)!=0){AN(H,24,1738048,(W=i,i=i+1|0,i=i+7&-8,c[W>>2]=0,W)|0);i=W}io(c[r>>2]|0,ap,0)|0;}}while(0);if(h<<24>>24!=4){af=S;ag=Q;break}ae=(V&16|0)==0;T=NU(Q,S,10,0)|0;af=ae?S:G;ag=ae?Q:T}}while(0);il(c[r>>2]|0,ag,af,0)|0;A4(c[k>>2]|0);A4(c[m>>2]|0)}if((P|0)==7674){m=c[r>>2]|0;il(m,N,O,0)|0;}else if((P|0)==7596){il(c[r>>2]|0,N,O,0)|0;}jx(K,0,249784);jx(K,0,173144);jx(K,0,249696);O=p|0;NM(O|0,0,17);N=zJ(c[K>>2]|0,1619128,0,1)|0;do{if((N|0)==0){P=7680}else{r=c[N+4>>2]|0;if((NL(r|0)|0)==4){aO=r}else{P=7680;break}while(1){aP=a[aO]|0;if((aP-48&255)<10){aO=aO+1|0}else{break}}if(aP<<24>>24==0){aQ=r}else{P=7680}}}while(0);do{if((P|0)==7680){aP=zJ(c[K>>2]|0,1618720,0,1)|0;if((aP|0)==0){i=g;return}aO=c[aP+4>>2]|0;if((NL(aO|0)|0)==4){aR=aO}else{i=g;return}while(1){aS=a[aR]|0;if((aS-48&255)<10){aR=aR+1|0}else{break}}if(aS<<24>>24==0){aQ=aO;break}i=g;return}}while(0);y_(O,aQ,5)|0;zK(K,1619128,0,0)|0;zK(K,1618720,0,0)|0;aQ=zJ(c[K>>2]|0,1618328,0,1)|0;do{if((aQ|0)==0){P=7688}else{aS=c[aQ+4>>2]|0;if((NL(aS|0)|0)==4){aT=aS}else{P=7688;break}while(1){aU=a[aT]|0;if((aU-48&255)<10){aT=aT+1|0}else{break}}if(aU<<24>>24==0){aV=aS;P=7692}else{P=7688}}}while(0);do{if((P|0)==7688){aU=zJ(c[K>>2]|0,1617896,0,1)|0;if((aU|0)==0){break}aT=c[aU+4>>2]|0;if((NL(aT|0)|0)==4){aW=aT}else{break}while(1){aX=a[aW]|0;if((aX-48&255)<10){aW=aW+1|0}else{break}}if(aX<<24>>24==0){aV=aT;P=7692}}}while(0);do{if((P|0)==7692){aX=p+4|0;aW=aV+2|0;bc(aX|0,13,1617360,(W=i,i=i+16|0,c[W>>2]=aW,c[W+8>>2]=aV,W)|0)|0;i=W;zK(K,1618328,0,0)|0;zK(K,1617896,0,0)|0;aW=zJ(c[K>>2]|0,1617e3,0,1)|0;do{if((aW|0)==0){P=7696}else{aX=c[aW+4>>2]|0;if((NL(aX|0)|0)==4){aY=aX}else{P=7696;break}while(1){aZ=a[aY]|0;if((aZ-48&255)<10){aY=aY+1|0}else{break}}if(aZ<<24>>24==0){a_=aX}else{P=7696}}}while(0);if((P|0)==7696){aW=zJ(c[K>>2]|0,1616720,0,1)|0;if((aW|0)==0){break}aT=c[aW+4>>2]|0;if((NL(aT|0)|0)==4){a$=aT}else{break}while(1){a0=a[a$]|0;if((a0-48&255)<10){a$=a$+1|0}else{break}}if(a0<<24>>24==0){a_=aT}else{break}}aW=p+10|0;aS=a_+2|0;bc(aW|0,7,1616376,(W=i,i=i+16|0,c[W>>2]=a_,c[W+8>>2]=aS,W)|0)|0;i=W;zK(K,1617e3,0,0)|0;zK(K,1616720,0,0)|0}}while(0);if((a[O]|0)==0){i=g;return}zK(K,1682968,O,0)|0;i=g;return}function jn(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0;b=i;i=i+8|0;d=b|0;e=c[a>>2]|0;c[d>>2]=e;if((e|0)==0){i=b;return}a=d;f=e;do{e=c[f>>2]|0;L10397:do{if((e|0)==0){g=f}else{do{if((NO(e|0,1631128,4)|0)==0){h=173080}else{if((NO(e|0,1633504,4)|0)==0){h=173096;break}j=(NO(e|0,1629896,4)|0)==0;if(j){h=j?173112:0}else{g=f;break L10397}}}while(0);c_[c[h+12>>2]&127](c[f+4>>2]|0);g=c[d>>2]|0}}while(0);f=c[g+8>>2]|0;A6(a);c[d>>2]=f;}while((f|0)!=0);i=b;return}function jo(a,b){a=a|0;b=b|0;var c=0,d=0;c=(b|0)!=0;b=c?4:3;if((a|0)==0){d=0;return d|0}if(!c){if((NO(a|0,1631960,b|0)|0)==0){d=173080;return d|0}if((NO(a|0,1630600,b|0)|0)==0){d=173096;return d|0}else{return((NO(a|0,1629896,b|0)|0)==0?173112:0)|0}}if((NO(a|0,1631128,b|0)|0)==0){d=173080;return d|0}if((NO(a|0,1633504,b|0)|0)==0){d=173096;return d|0}d=(NO(a|0,1629896,b|0)|0)==0?173112:0;return d|0}function jp(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0;d=c[b>>2]|0;if((d|0)==0){e=0;return e|0}else{f=d}while(1){if((a8(c[f>>2]|0,1633504)|0)==0){d=c[f+4>>2]|0;b=mK(a,0)|0;if((b|0)==0){e=-12;g=7741;break}h=b+72|0;c[h>>2]=c[h>>2]|1024;h=b+8|0;c[(c[h>>2]|0)+8>>2]=0;c[(c[h>>2]|0)+48>>2]=c[d+12>>2];h=b+88|0;i=c[d+8>>2]|0;zK(h,1737592,i,0)|0;i=c[d+4>>2]|0;zK(h,1632896,i,0)|0;i=b+104|0;nB(i);h=d;c[i>>2]=c[h>>2];c[b+128>>2]=c[(c[h>>2]|0)+4>>2];c[b+132>>2]=(c[(c[h>>2]|0)+8>>2]|0)-16;c[b+136>>2]=c[b>>2];i=b+140|0;c[i>>2]=c[i>>2]|1;c[h>>2]=0}h=c[f+8>>2]|0;if((h|0)==0){e=0;g=7743;break}else{f=h}}if((g|0)==7741){return e|0}else if((g|0)==7743){return e|0}return 0}function jq(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;g=i;i=i+8|0;h=g|0;c[h>>2]=d;if((d|0)<1){i=g;return}j=A7(20)|0;if((j|0)==0){AN(a,16,1621776,(k=i,i=i+8|0,c[k>>2]=20,k)|0);i=k;i=g;return}l=A7(12)|0;m=l;do{if((l|0)==0){AN(a,16,1621776,(k=i,i=i+8|0,c[k>>2]=12,k)|0);i=k;n=j+4|0}else{o=iD(b)|0;c[h>>2]=d-1;p=j+4|0;q=(jv(a,b,0,p,h)|0)<0;if(q|(c[h>>2]|0)<1){n=p;break}q=o<<24>>24;o=(jv(a,b,q,j+8|0,h)|0)<0;if(o|(c[h>>2]|0)<1){n=p;break}o=jv(a,b,q,j+12|0,h)|0;q=c[h>>2]|0;if((q|o|0)<0){n=p;break}if((q|0)==0){c[j+16>>2]=0;c[j>>2]=0}else{o=A1(q)|0;c[j+16>>2]=o;if((o|0)==0){AN(a,16,1621336,(k=i,i=i+8|0,c[k>>2]=q,k)|0);i=k;n=p;break}p=iE(b,o,q)|0;if(p>>>0<q>>>0){AN(a,24,1619904,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k}c[j>>2]=p}c[l>>2]=1631128;c[l+4>>2]=j;c[l+8>>2]=c[f>>2];c[f>>2]=m;i=g;return}}while(0);AN(a,16,1629080,(k=i,i=i+8|0,c[k>>2]=e,k)|0);i=k;A4(c[n>>2]|0);A4(c[j+8>>2]|0);A4(c[j+12>>2]|0);A4(c[j+16>>2]|0);A4(j);A4(l);i=g;return}function jr(a){a=a|0;A4(c[a+4>>2]|0);A4(c[a+8>>2]|0);A4(c[a+12>>2]|0);A4(c[a+16>>2]|0);A4(a);return}function js(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;e=i;i=i+88|0;g=e|0;h=e+8|0;j=e+16|0;k=e+80|0;c[h>>2]=d;c[k>>2]=0;l=il(b,0,0,1)|0;m=NU(d,(d|0)<0|0?-1:0,l,G)|0;l=G;do{if((d|0)>=5){c[k>>2]=A7(12)|0;n=A7(16)|0;o=(n|0)==0;L10468:do{if((c[k>>2]|0)==0|o){p=7782}else{q=iD(b)|0;r=d-1|0;s=j|0;t=r-(iN(b,r,s,64)|0)|0;c[h>>2]=t;r=249312;while(1){u=r+32|0;if((c[u>>2]|0)==0){break}if((y5(r|0,s,64)|0)==0){p=7772;break}else{r=r+36|0}}do{if((p|0)==7772){r=c[u>>2]|0;if((r|0)==0){break}c[n+12>>2]=r;r=iD(b)|0;c[h>>2]=t-1;if((r|0)<0|r>>>0>20){AN(a,24,1622520,(v=i,i=i+8|0,c[v>>2]=r,v)|0);i=v;w=0}else{w=r}c[n+4>>2]=c[249224+(w<<2)>>2];r=n+8|0;if((jv(a,b,q,r,h)|0)<0){AN(a,16,1622104,(v=i,i=i+1|0,i=i+7&-8,c[v>>2]=0,v)|0);i=v;p=7782;break L10468}x=c[h>>2]|0;y=zg(x+16|0)|0;z=n;c[z>>2]=y;if((y|0)==0){A=z;B=r;break L10468}if((x|0)==0){p=7782;break L10468}if((iE(b,c[y+4>>2]|0,x)|0)!=(x|0)){p=7782;break L10468}NM((c[(c[z>>2]|0)+4>>2]|0)+x|0,0,16);c[c[k>>2]>>2]=1633504;c[(c[k>>2]|0)+4>>2]=n;c[(c[k>>2]|0)+8>>2]=c[f>>2];c[f>>2]=c[k>>2];i=e;return}}while(0);AN(a,24,1622920,(v=i,i=i+8|0,c[v>>2]=s,v)|0);i=v;p=7782}}while(0);if((p|0)==7782){if(o){break}A=n;B=n+8|0}c[g>>2]=n;zj(A);A6(B);A6(g)}}while(0);A6(k);il(b,m,l,0)|0;i=e;return}function jt(a){a=a|0;var b=0,d=0;b=i;i=i+8|0;d=b|0;c[d>>2]=a;zj(a);A6(a+8|0);A6(d);i=b;return}function ju(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;g=i;i=i+24|0;f=g|0;h=g+8|0;j=g+16|0;c[f>>2]=e;c[h>>2]=0;e=(jv(b,d,0,h,f)|0)<0;k=c[f>>2]|0;if(e|(k|0)<16){i=g;return}e=iM(d)|0;l=iM(d)|0;io(d,8,0)|0;m=mM(b,(c[b+1132>>2]|0)+1|0,85888,e,0,l,0,c[h>>2]|0)|0;if((m|0)==0){A4(c[h>>2]|0);i=g;return}l=k-16|0;c[f>>2]=l;do{if((l|0)>10){k=j|0;e=j+4|0;n=m+32|0;o=l;while(1){iE(d,k,4)|0;a[e]=0;p=iM(d)|0;io(d,2,0)|0;q=o-10|0;if((p|0)<0|(p|0)>(q|0)){break}if((a[k]|0)==84){jw(b,d,p,n,k)}else{r=p;s=(p|0)<0|0?-1:0;io(d,r,s)|0;}t=q-p|0;if((t|0)>10){o=t}else{u=7802;break}}if((u|0)==7802){c[f>>2]=t;v=n;break}c[f>>2]=q;A4(c[h>>2]|0);i=g;return}else{v=m+32|0}}while(0);jx(v,0,249784);jx(v,0,249696);A4(c[h>>2]|0);i=g;return}function jv(a,b,e,f,g){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0;h=i;i=i+8|0;j=h|0;k=c[g>>2]|0;l=iZ(j)|0;if((l|0)<0){AN(a,16,1625520,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=l;i=h;return n|0}do{if((e|0)==3){if((k|0)==0){o=0;p=7843;break}else{q=k}while(1){l=iD(b)|0;ih(c[j>>2]|0,l);r=q-1|0;if((r|0)==0|(l|0)==0){s=r;t=l;p=7842;break}else{q=r}}}else if((e|0)==2){u=98;v=k;p=7813}else if((e|0)==0){if((k|0)==0){o=0;p=7843;break}else{w=k}while(1){r=iD(b)|0;do{if(r>>>0<128){ih(c[j>>2]|0,r&255)}else{l=r>>>0>65535;x=l?16:0;y=l?r>>>16:r;if((y&65280|0)==0){z=y;A=x}else{z=y>>>8;A=x|8}x=(A+4+(d[247648+z|0]|0)|0)/5|0;y=(x*6|0)-6|0;ih(c[j>>2]|0,(r>>>(y>>>0)|256-(256>>>(x>>>0)))&255);if((y|0)>5){B=y}else{break}do{B=B-6|0;ih(c[j>>2]|0,r>>>(B>>>0)&63|128);}while((B|0)>5)}}while(0);y=w-1|0;if((y|0)==0|(r|0)==0){s=y;t=r;p=7842;break}else{w=y}}}else if((e|0)==1){y=k-2|0;if((y|0)<0){AN(a,16,1624808,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;x=c[j>>2]|0;i$(x,f)|0;A6(f);n=-1094995529;i=h;return n|0}x=iK(b)|0;if((x|0)==65279){u=98;v=y;p=7813;break}else if((x|0)==65534){u=34;v=y;p=7813;break}AN(a,16,1624208,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;i$(c[j>>2]|0,f)|0;A6(f);c[g>>2]=y;n=-1094995529;i=h;return n|0}else{AN(a,24,1623656,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;o=k;p=7843}}while(0);if((p|0)==7813){L10551:do{if((v|0)<2){C=v;D=1}else{k=v;while(1){m=k-2|0;do{if((m|0)>-1){a=c1[u&255](b)|0;e=a-55296|0;if(e>>>0<2048){w=k-4|0;if((w|0)<=-1){C=w;D=-56320;break L10551}B=(c1[u&255](b)|0)-56320|0;if(B>>>0>1023|e>>>0>1023){C=w;D=B;break L10551}E=w;F=(e<<10)+65536+B|0}else{E=m;F=a}if(F>>>0<128){G=F;H=E;p=7833;break}a=F>>>0>65535;B=a?16:0;e=a?F>>>16:F;if((e&65280|0)==0){I=e;J=B}else{I=e>>>8;J=B|8}B=(J+4+(d[247648+I|0]|0)|0)/5|0;e=(B*6|0)-6|0;ih(c[j>>2]|0,(F>>>(e>>>0)|256-(256>>>(B>>>0)))&255);if((e|0)>5){K=e}else{L=E;M=F;break}while(1){e=K-6|0;ih(c[j>>2]|0,F>>>(e>>>0)&63|128);if((e|0)>5){K=e}else{L=E;M=F;break}}}else{G=0;H=m;p=7833}}while(0);if((p|0)==7833){p=0;ih(c[j>>2]|0,G&255);L=H;M=G}if((L|0)<2|(M|0)==0){C=L;D=M;break}else{k=L}}}}while(0);s=(C|0)<0?C+2|0:C;t=D;p=7842}if((p|0)==7842){if((t|0)==0){N=s}else{o=s;p=7843}}if((p|0)==7843){ih(c[j>>2]|0,0);N=o}i$(c[j>>2]|0,f)|0;c[g>>2]=N;n=0;i=h;return n|0}function jw(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;h=i;i=i+32|0;j=h|0;k=h+8|0;l=h+16|0;m=h+24|0;c[j>>2]=e;c[k>>2]=g;if((e|0)<1){i=h;return}n=iD(d)|0;c[j>>2]=e-1;if((jv(b,d,n,l,j)|0)<0){AN(b,16,1629080,(o=i,i=i+8|0,c[o>>2]=g,o)|0);i=o;i=h;return}if((a8(g|0,1661744)|0)==0){p=7855}else{if((a8(g|0,1628520)|0)==0){p=7855}else{p=7859}}do{if((p|0)==7855){g=ct(c[l>>2]|0,1628112,(o=i,i=i+8|0,c[o>>2]=m,o)|0)|0;i=o;if((g|0)==1){if((c[m>>2]|0)>>>0>=148){p=7859;break}}else{g=ct(c[l>>2]|0,1627600,(o=i,i=i+8|0,c[o>>2]=m,o)|0)|0;i=o;if(!((g|0)==1&(c[m>>2]|0)>>>0<148)){p=7859;break}}A6(l);g=A9(c[249904+(c[m>>2]<<2)>>2]|0)|0;c[l>>2]=g;q=24;r=g}}while(0);L10593:do{if((p|0)==7859){m=c[k>>2]|0;do{if((a8(m|0,1627072)|0)==0){p=7861}else{if((a8(m|0,1626376)|0)==0){p=7861;break}g=c[l>>2]|0;if((a[g]|0)!=0){q=24;r=g;break L10593}A6(l);s=24}}while(0);do{if((p|0)==7861){m=c[l>>2]|0;c[k>>2]=m;if((jv(b,d,n,l,j)|0)>=0){s=28;break}AN(b,16,1629080,(o=i,i=i+8|0,c[o>>2]=m,o)|0);i=o;A6(k);i=h;return}}while(0);q=s;r=c[l>>2]|0}}while(0);if((r|0)==0){i=h;return}zK(f,c[k>>2]|0,r,q)|0;i=h;return}function jx(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;e=i;i=i+8|0;f=e|0;c[f>>2]=0;if((b|0)==(d|0)){i=e;return}g=zJ(c[a>>2]|0,3119152,0,2)|0;L10613:do{if((g|0)!=0){h=(b|0)==0;if((d|0)==0){j=g;while(1){k=c[j>>2]|0;L10618:do{if(h){l=k}else{m=b;while(1){n=m|0;if((c[n>>2]|0)==0){l=k;break L10618}if((y4(k,c[m+4>>2]|0)|0)==0){break}else{m=m+8|0}}l=c[n>>2]|0}}while(0);zK(f,l,c[j+4>>2]|0,0)|0;j=zJ(c[a>>2]|0,3119152,j,2)|0;if((j|0)==0){break L10613}}}if(h){j=g;while(1){k=c[j>>2]|0;m=d;while(1){o=c[m>>2]|0;if((o|0)==0){p=k;break}if((y4(k,o)|0)==0){q=7885;break}else{m=m+8|0}}if((q|0)==7885){q=0;p=c[m+4>>2]|0}zK(f,p,c[j+4>>2]|0,0)|0;j=zJ(c[a>>2]|0,3119152,j,2)|0;if((j|0)==0){break L10613}}}else{r=g}do{j=c[r>>2]|0;h=d;while(1){k=c[h>>2]|0;if((k|0)==0){s=j;break}if((y4(j,k)|0)==0){q=7891;break}else{h=h+8|0}}if((q|0)==7891){q=0;s=c[h+4>>2]|0}j=b;while(1){t=j|0;if((c[t>>2]|0)==0){u=s;break}if((y4(s,c[j+4>>2]|0)|0)==0){q=7895;break}else{j=j+8|0}}if((q|0)==7895){q=0;u=c[t>>2]|0}zK(f,u,c[r+4>>2]|0,0)|0;r=zJ(c[a>>2]|0,3119152,r,2)|0;}while((r|0)!=0)}}while(0);zL(a);c[a>>2]=c[f>>2];i=e;return}function jy(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;jx(a+1140|0,b,d);e=a+24|0;if((c[e>>2]|0)!=0){f=a+28|0;g=0;do{jx((c[(c[f>>2]|0)+(g<<2)>>2]|0)+88|0,b,d);g=g+1|0;}while(g>>>0<(c[e>>2]|0)>>>0)}e=a+1132|0;if((c[e>>2]|0)!=0){g=a+1136|0;f=0;do{jx((c[(c[g>>2]|0)+(f<<2)>>2]|0)+32|0,b,d);f=f+1|0;}while(f>>>0<(c[e>>2]|0)>>>0)}e=a+1104|0;if((c[e>>2]|0)==0){return}f=a+1108|0;a=0;do{jx((c[(c[f>>2]|0)+(a<<2)>>2]|0)+20|0,b,d);a=a+1|0;}while(a>>>0<(c[e>>2]|0)>>>0);return}function jz(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;i=i+8|0;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=e|0;d=c[a+24>>2]|0;if((d|0)==0){i=e;return 0}g=c[(c[a+28>>2]|0)+(d-1<<2)>>2]|0;iM(b)|0;nt(a,b,f)|0;if((c[f>>2]|0)==3){nu(b,0)}else{iK(b)|0}nt(a,b,f)|0;if((c[f>>2]|0)!=4){i=e;return 0}nv(a,g,b)|0;i=e;return 0}function jA(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,j=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0;g=i;i=i+32|0;j=g|0;l=g+16|0;m=a+4|0;n=c[m>>2]|0;o=c[n+24>>2]|0;if((o|0)==0){p=0;i=g;return p|0}q=c[(c[n+28>>2]|0)+(o-1<<2)>>2]|0;o=q+12|0;n=c[o>>2]|0;r=e+52|0;L10683:do{if((f|0)>0){s=j|0;t=j+8|0;u=q+8|0;v=n+136|0;w=n+152|0;x=n+176|0;y=n+1200|0;z=n+140|0;A=a+40|0;B=n+128|0;C=n+124|0;D=n+64|0;E=l|0;F=l+8|0;H=n+164|0;I=n+168|0;J=0;L10685:while(1){if((c[r>>2]|0)!=0){break L10683}c[s>>2]=1685288051;c[t>>2]=0;c[t+4>>2]=0;K=il(e,0,0,1)|0;L=G;M=iM(e)|0;N=M;O=0;P=iI(e)|0;if(M>>>0>15){iM(e)|0;iK(e)|0;Q=iK(e)|0}else{if(M>>>0<8){break}else{Q=1}}M=c[(c[u>>2]|0)+52>>2]|0;L10692:do{if((M|0)==0|(M|0)==(P|0)){R=M;S=7937}else{T=c[m>>2]|0;do{if((c[T+1112>>2]|0)==0){if((M|0)==1734701162){U=1734701162;S=7939;break L10692}else{V=T}}else{W=mA(251704,P)|0;X=c[m>>2]|0;if((W|0)!=(c[X+1112>>2]|0)){V=X;break}R=c[(c[u>>2]|0)+52>>2]|0;S=7937;break L10692}}while(0);AN(V,24,1700992,(Y=i,i=i+1|0,i=i+7&-8,c[Y>>2]=0,Y)|0);i=Y;T=il(e,0,0,1)|0;X=G;W=NU(N,O,K,L)|0;Z=NV(W,G,T,X)|0;X=G;io(e,Z,X)|0;}}while(0);do{if((S|0)==7937){S=0;if((R|0)!=828601953){U=R;S=7939;break}AN(c[m>>2]|0,24,1661560,(Y=i,i=i+1|0,i=i+7&-8,c[Y>>2]=0,Y)|0);i=Y;U=c[(c[u>>2]|0)+52>>2]|0;S=7939}}while(0);do{if((S|0)==7939){S=0;c[v>>2]=(U|0)!=0?-1:J;c[w>>2]=Q;c[(c[u>>2]|0)+52>>2]=P;M=mA(253024,P)|0;do{if((M|0)==0){X=P&65535;if((X|0)==29549|(X|0)==21332){_=mA(251144,(cV(P|0)|0)&65535)|0;S=7943;break}else{$=0;aa=c[(c[u>>2]|0)+8>>2]|0;S=7945;break}}else{_=M;S=7943}}while(0);do{if((S|0)==7943){S=0;M=(c[u>>2]|0)+8|0;X=c[M>>2]|0;if((X|0)==0|(_|0)==0){$=_;aa=X;S=7945;break}c[M>>2]=1;ab=_}}while(0);L10712:do{if((S|0)==7945){S=0;if((aa|0)==1){ab=$;break}if((P|0)==1932816493|(P|0)==0){ab=$;break}M=mA(251704,P)|0;do{if((M|0)==0){X=mA(253424,P)|0;Z=c[u>>2]|0;T=Z+8|0;if((X|0)!=0){ad=X;ae=T;break}X=c[T>>2]|0;if((X|0)==3){if((c[Z+48>>2]|0)!=0){ab=0;break L10712}}else if((X|0)!=2){ab=0;break L10712}X=mA(252992,P)|0;if((X|0)==0){ab=0;break L10712}c[(c[u>>2]|0)+8>>2]=3;ab=X;break L10712}else{ad=M;ae=(c[u>>2]|0)+8|0}}while(0);c[ae>>2]=0;ab=ad}}while(0);M=c[u>>2]|0;X=c[M+8>>2]|0;do{if((X|0)==0){c[M+48>>2]=ab;iK(e)|0;iK(e)|0;iM(e)|0;iM(e)|0;iM(e)|0;Z=iK(e)|0;c[(c[u>>2]|0)+120>>2]=Z;Z=iK(e)|0;c[(c[u>>2]|0)+124>>2]=Z;iM(e)|0;iM(e)|0;iM(e)|0;iK(e)|0;Z=iD(e)|0;T=Z>>>0>31?31:Z;jB(e,T,(c[u>>2]|0)+16|0,32);if(T>>>0<31){Z=31-T|0;T=0;io(e,Z,T)|0;}T=c[u>>2]|0;if((NO(T+16|0,1644768,25)|0)==0){c[T+52>>2]=808596553;T=(c[u>>2]|0)+120|0;c[T>>2]=c[T>>2]&-2;T=(c[u>>2]|0)+124|0;c[T>>2]=c[T>>2]&-2}do{if((P|0)==859189832){T=c[u>>2]|0;if((NO(T+16|0,1632096,13)|0)!=0){break}c[T+48>>2]=22}}while(0);T=iK(e)|0;c[(c[u>>2]|0)+792>>2]=T;T=iK(e)|0;Z=(c[u>>2]|0)+792|0;W=c[Z>>2]|0;af=W&31;if(!((af|0)==8|(af|0)==4|(af|0)==2)){break}L10739:do{if((W&32|0)==0){if((T|0)==0){ag=iM(e)|0;iK(e)|0;ah=iK(e)|0;if((ah|ag)>>>0>255|ag>>>0>ah>>>0){break}else{ai=ag}while(1){ag=iD(e)|0;iD(e)|0;aj=iD(e)|0;iD(e)|0;ak=iD(e)|0;iD(e)|0;al=iD(e)|0;iD(e)|0;c[x+(ai<<2)>>2]=aj<<16&16711680|ag<<24|ak<<8&65280|al&255;ai=ai+1|0;if(ai>>>0>ah>>>0){break L10739}}}ah=1<<af;if((af|0)==4){am=244816}else if((af|0)==2){am=244032}else{am=244048}al=0;do{ak=al*3|0;c[x+(al<<2)>>2]=(d[am+ak|0]|0)<<16|(d[am+(ak+1)|0]|0)<<8|(d[am+(ak+2)|0]|0)|-16777216;al=al+1|0;}while(al>>>0<ah>>>0)}else{c[Z>>2]=af;ah=1<<af;al=ah-1|0;ak=256/(al>>>0)|0;if((ab|0)==44){ag=255;aj=0;while(1){an=al-ag&255;c[x+(aj<<2)>>2]=an|an<<8|an<<16|-16777216;an=ag-ak|0;ao=aj+1|0;if(ao>>>0<ah>>>0){ag=(an|0)<0?0:an;aj=ao}else{break}}}else{aj=255;ag=0;while(1){al=aj&255;c[x+(ag<<2)>>2]=al|al<<8|al<<16|-16777216;al=aj-ak|0;ao=ag+1|0;if(ao>>>0<ah>>>0){aj=(al|0)<0?0:al;ag=ao}else{break}}}}}while(0);c[y>>2]=1}else if((X|0)==1){af=iK(e)|0;Z=zJ(c[(c[m>>2]|0)+1140>>2]|0,1625632,0,1)|0;c[(c[u>>2]|0)+48>>2]=ab;iK(e)|0;iM(e)|0;T=iK(e)|0;c[(c[u>>2]|0)+416>>2]=T;T=iK(e)|0;c[(c[u>>2]|0)+792>>2]=T;b[z>>1]=(iK(e)|0)&65535;iK(e)|0;T=(iM(e)|0)>>>16;c[(c[u>>2]|0)+412>>2]=T;do{if((c[A>>2]|0)==0){S=7980}else{if((Z|0)==0){break}if((a4(c[Z+4>>2]|0,1619200)|0)!=0){S=7980}}}while(0);do{if((S|0)==7980){S=0;Z=af&65535;if((Z|0)==1){c[B>>2]=iM(e)|0;iM(e)|0;c[C>>2]=iM(e)|0;iM(e)|0;break}else if((Z|0)!=2){break}iM(e)|0;Z=iQ(e)|0;c[(c[u>>2]|0)+412>>2]=~~(c[k>>2]=Z,c[k+4>>2]=G,+h[k>>3]);Z=iM(e)|0;c[(c[u>>2]|0)+416>>2]=Z;iM(e)|0;Z=iM(e)|0;c[(c[u>>2]|0)+792>>2]=Z;Z=iM(e)|0;c[C>>2]=iM(e)|0;c[B>>2]=iM(e)|0;if((P|0)!=1835233388){break}T=mB(c[(c[u>>2]|0)+792>>2]|0,Z&1,Z&2,Z<<29>>31)|0;c[(c[u>>2]|0)+48>>2]=T}}while(0);af=c[u>>2]|0;T=af+48|0;Z=c[T>>2]|0;do{if((Z|0)==86026){c[B>>2]=6;c[C>>2]=c[(c[u>>2]|0)+416>>2]}else if((Z|0)==69632){c[B>>2]=64;c[C>>2]=(c[(c[u>>2]|0)+416>>2]|0)*34|0}else if((Z|0)==65540|(Z|0)==65541){if((c[af+792>>2]|0)!=16){break}c[T>>2]=65537}else if((Z|0)==65536|(Z|0)==65537){W=c[af+792>>2]|0;if((W|0)==8){c[T>>2]=65540;break}else if((W|0)==24){c[T>>2]=(Z|0)==65537?65549:65548;break}else{break}}else if((Z|0)==86034){c[B>>2]=160;c[C>>2]=33}else if((Z|0)==86025){c[B>>2]=6;c[C>>2]=c[(c[u>>2]|0)+416>>2]<<1}}while(0);Z=pc(c[(c[u>>2]|0)+48>>2]|0)|0;if((Z|0)==0){break}c[(c[u>>2]|0)+792>>2]=Z;c[D>>2]=ac(c[(c[u>>2]|0)+416>>2]|0,Z>>3)|0}else if((X|0)==3){c[E>>2]=0;Z=il(e,0,0,1)|0;T=G;af=NU(N,O,K,L)|0;W=NV(af,G,Z,T)|0;c[F>>2]=W;c[F+4>>2]=G;if((P|0)!=1932816493){jC(a,e,l)|0}c[(c[u>>2]|0)+48>>2]=ab;c[(c[u>>2]|0)+120>>2]=c[H>>2];c[(c[u>>2]|0)+124>>2]=c[I>>2]}else{if((c[M+52>>2]|0)==1684237684){W=c[o>>2]|0;iM(e)|0;T=iM(e)|0;c[W+1216>>2]=T;if((T&1|0)!=0){T=(c[u>>2]|0)+92|0;c[T>>2]=c[T>>2]|8192}iM(e)|0;iM(e)|0;T=iD(e)|0;c[(c[u>>2]|0)+108>>2]=T;c[(c[u>>2]|0)+104>>2]=1}T=il(e,0,0,1)|0;W=G;Z=NU(N,O,K,L)|0;af=NV(Z,G,T,W)|0;io(e,af,G)|0;}}while(0);M=il(e,0,0,1)|0;X=NV(K,L,M,G)|0;M=NU(X,G,N,O)|0;X=G;c[t>>2]=M;c[t+4>>2]=X;af=0;if((X|0)>(af|0)|(X|0)==(af|0)&M>>>0>8>>>0){af=jD(a,e,j)|0;if((af|0)<0){p=af;S=8041;break L10685}else{break}}af=0;if(!((X|0)>(af|0)|(X|0)==(af|0)&M>>>0>0>>>0)){break}io(e,M,X)|0;}}while(0);J=J+1|0;if((J|0)>=(f|0)){break L10683}}if((S|0)==8041){i=g;return p|0}AN(c[m>>2]|0,16,1734344,(Y=i,i=i+16|0,c[Y>>2]=N,c[Y+8>>2]=O,Y)|0);i=Y;p=-1094995529;i=g;return p|0}}while(0);if((c[r>>2]|0)!=0){p=-541478725;i=g;return p|0}r=q+8|0;Y=c[r>>2]|0;do{if((c[Y+8>>2]|0)==1){O=Y+412|0;if((c[O>>2]|0)!=0){ap=Y;break}N=c[n+92>>2]|0;if((N|0)<=1){ap=Y;break}c[O>>2]=N;ap=c[r>>2]|0}else{ap=Y}}while(0);Y=c[ap+48>>2]|0;if((Y|0)==86019){c[q+236>>2]=1;p=0;i=g;return p|0}else if((Y|0)==1){c[q+236>>2]=1;p=0;i=g;return p|0}else if((Y|0)==71){c[q+236>>2]=1;p=0;i=g;return p|0}else if((Y|0)==86016|(Y|0)==86017){c[ap+8>>2]=1;c[q+236>>2]=1;p=0;i=g;return p|0}else if((Y|0)==86034|(Y|0)==69638|(Y|0)==69633|(Y|0)==86076|(Y|0)==86025|(Y|0)==86026|(Y|0)==86035){c[ap+432>>2]=c[n+124>>2];p=0;i=g;return p|0}else if((Y|0)==86032){if((c[ap+100>>2]|0)!=36){p=0;i=g;return p|0}c[ap+416>>2]=d[(c[ap+96>>2]|0)+21|0]|0;n=c[r>>2]|0;q=(c[n+96>>2]|0)+32|0;c[n+412>>2]=cV(d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0)|0;p=0;i=g;return p|0}else if((Y|0)==73728){c[ap+416>>2]=1;c[(c[r>>2]|0)+412>>2]=8e3;p=0;i=g;return p|0}else if((Y|0)==73729){c[ap+416>>2]=1;c[(c[r>>2]|0)+412>>2]=16e3;p=0;i=g;return p|0}else if((Y|0)==86040){if((c[ap+52>>2]|0)==1886151505){aq=ap}else{c[ap+412>>2]=8e3;aq=c[r>>2]|0}c[aq+416>>2]=1;p=0;i=g;return p|0}else{p=0;i=g;return p|0}return 0}function jB(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;h=f+(g-1)|0;if((e|0)>0){i=0;j=f}else{k=f;a[k]=0;return}while(1){f=iD(b)|0;g=f&255;l=j>>>0<h>>>0;L10844:do{if(g>>>0<128&l){a[j]=f&255;m=j+1|0}else{if(!l){m=j;break}n=c[130200+(g-128<<2)>>2]|0;o=(n&65280|0)==0;p=(((d[247648+(o?n:n>>>8)|0]|0)+4+(o?0:8)|0)>>>0)/5|0;o=(p*6|0)-6|0;a[j]=(n>>>(o>>>0)|256-(256>>>(p>>>0)))&255;p=o;o=j;while(1){q=o+1|0;if(q>>>0>=h>>>0){r=p;break}if((p|0)<=5){m=q;break L10844}s=p-6|0;a[q]=(n>>>(s>>>0)&63|128)&255;p=s;o=q}while(1){if((r|0)<=5){m=q;break L10844}r=r-6|0}}}while(0);g=i+1|0;if((g|0)<(e|0)){i=g;j=m}else{k=m;break}}a[k]=0;return}function jC(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;g=c[f+24>>2]|0;if((g|0)==0){h=0;i=e;return h|0}j=c[(c[f+28>>2]|0)+(g-1<<2)>>2]|0;g=d+8|0;f=c[g>>2]|0;k=c[g+4>>2]|0;g=0;if(k>>>0>g>>>0|k>>>0==g>>>0&f>>>0>1073741824>>>0){h=-1094995529;i=e;return h|0}g=0;do{if((k|0)>(g|0)|(k|0)==(g|0)&f>>>0>9>>>0){l=iM(b)|0;m=iI(b)|0;n=-8;o=-1;il(b,n,o,1)|0;if((m|0)!=1818585446){break}if(!((l|0)==(f|0)&0==(k|0))){break}h=jD(a,b,d)|0;i=e;return h|0}}while(0);d=j+8|0;A4(c[(c[d>>2]|0)+96>>2]|0);c[(c[d>>2]|0)+100>>2]=0;j=NU(f,k,16,0)|0;k=A7(j)|0;c[(c[d>>2]|0)+96>>2]=k;k=c[d>>2]|0;if((c[k+96>>2]|0)==0){h=-12;i=e;return h|0}j=f;c[k+100>>2]=j;iE(b,c[(c[d>>2]|0)+96>>2]|0,j)|0;h=0;i=e;return h|0}function jD(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;e=i;i=i+16|0;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=e|0;g=d+8|0;h=c[g>>2]|0;j=c[g+4>>2]|0;k=0;if((j|0)<(k|0)|(j|0)==(k|0)&h>>>0<0>>>0){c[g>>2]=-1;c[g+4>>2]=2147483647;l=2147483647;m=-1;n=8074}else{g=0;if((j|0)<(g|0)|(j|0)==(g|0)&h>>>0<8>>>0){o=0;p=0;q=j;r=h}else{l=j;m=h;n=8074}}L10878:do{if((n|0)==8074){h=f+8|0;j=f|0;g=0;k=(l|0)>(g|0)|(l|0)==(g|0)&m>>>0>7>>>0;g=c[d>>2]|0;s=a+24|0;t=a+4|0;u=a+28|0;v=b+88|0;w=0;x=0;y=0;z=8;L10880:while(1){if((iq(b)|0)!=0){o=w;p=x;q=l;r=m;break L10878}c[h>>2]=m;c[h+4>>2]=l;c[j>>2]=0;do{if(k){c[h>>2]=iM(b)|0;c[h+4>>2]=0;A=iI(b)|0;c[j>>2]=A;if(!((g|0)==1953460082|(g|0)==1987014509)){if((A|0)==1801548404|(A|0)==1952539757){n=8079;break L10880}}A=c[h>>2]|0;B=c[h+4>>2]|0;if(!((A|0)==1&(B|0)==0)){C=y;D=z;E=B;F=A;break}A=iQ(b)|0;B=NU(A,G,-8,-1)|0;A=G;c[h>>2]=B;c[h+4>>2]=A;H=NU(x,w,16,0)|0;C=G;D=H;E=A;F=B}else{C=w;D=x;E=l;F=m}}while(0);if((F|0)==0&(E|0)==0){B=NV(8,0,D,C)|0;A=NU(B,G,m,l)|0;B=G;c[h>>2]=A;c[h+4>>2]=B;I=B;J=A}else{I=E;J=F}A=NU(J,I,-8,-1)|0;B=G;c[h>>2]=A;c[h+4>>2]=B;H=0;if((B|0)<(H|0)|(B|0)==(H|0)&A>>>0<0>>>0){o=C;p=D;q=l;r=m;break L10878}H=NV(m,l,D,C)|0;K=G;L=(B|0)>(K|0)|(B|0)==(K|0)&A>>>0>H>>>0;M=L?H:A;A=L?K:B;c[h>>2]=M;c[h+4>>2]=A;B=c[j>>2]|0;K=0;while(1){if((K|0)==63){n=8089;break}if((c[127008+(K<<3)>>2]|0)==(B|0)){n=8088;break}else{K=K+1|0}}do{if((n|0)==8088){n=0;N=c[127012+(K<<3)>>2]|0;n=8091}else if((n|0)==8089){n=0;if((g|0)==1635017845|(g|0)==1953721449){N=216;n=8091;break}io(b,M,A)|0;}}while(0);do{if((n|0)==8091){n=0;O=il(b,0,0,1)|0;P=G;A=c6[N&255](a,b,f)|0;if((A|0)<0){Q=A;n=8112;break L10880}do{if((c[s>>2]|0)!=0){if((c[u>>2]|0)==0){break}if((c[v>>2]|0)==0){break L10880}if((c[(c[t>>2]|0)+1084>>2]&2|0)!=0){n=8098;break L10880}A=NU(c[h>>2]|0,c[h+4>>2]|0,O,P)|0;M=G;K=ip(b)|0;if((A|0)==(K|0)&(M|0)==(G|0)){n=8097;break L10880}}}while(0);M=c[h>>2]|0;K=c[h+4>>2]|0;A=il(b,0,0,1)|0;B=NV(M,K,A,G)|0;A=NU(B,G,O,P)|0;B=G;K=0;if((B|0)>(K|0)|(B|0)==(K|0)&A>>>0>0>>>0){io(b,A,B)|0;break}K=0;if(!((B|0)<(K|0)|(B|0)==(K|0)&A>>>0<0>>>0)){break}K=c[t>>2]|0;M=NV(0,0,A,B)|0;L=G;AN(K,48,1685024,(R=i,i=i+24|0,c[R>>2]=M,c[R+8>>2]=L,c[R+16>>2]=f,R)|0);i=R;il(b,A,B,1)|0;}}while(0);B=NU(c[h>>2]|0,c[h+4>>2]|0,D,C)|0;A=G;L=NU(B,A,8,0)|0;M=G;if((M|0)>(l|0)|(M|0)==(l|0)&L>>>0>m>>>0){o=A;p=B;q=l;r=m;break L10878}else{w=A;x=B;y=M;z=L}}if((n|0)==8079){AN(c[t>>2]|0,16,1686656,(R=i,i=i+1|0,i=i+7&-8,c[R>>2]=0,R)|0);i=R;z=-8;y=-1;io(b,z,y)|0;Q=0;i=e;return Q|0}else if((n|0)==8112){i=e;return Q|0}else if((n|0)==8097){if((c[v>>2]|0)!=0){n=8098}}do{if((n|0)==8098){if((c[(c[t>>2]|0)+1084>>2]&2|0)==0){Q=0}else{break}i=e;return Q|0}}while(0);t=NU(c[h>>2]|0,c[h+4>>2]|0,O,P)|0;v=a+112|0;c[v>>2]=t;c[v+4>>2]=G;Q=0;i=e;return Q|0}}while(0);a=0;if(!(((o|0)<(q|0)|(o|0)==(q|0)&p>>>0<r>>>0)&((q|0)<(a|0)|(q|0)==(a|0)&r>>>0<524287>>>0))){Q=0;i=e;return Q|0}a=NV(r,q,p,o)|0;io(b,a,G)|0;Q=0;i=e;return Q|0}function jE(a){a=a|0;var b=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;b=i;e=c[a+8>>2]|0;if(e>>>0<8){f=0;i=b;return f|0}g=c[a+4>>2]|0;h=0;j=0;k=0;l=-1;L10934:while(1){m=h;n=j;o=k;L10936:while(1){p=m;q=n;while(1){r=q;s=g+(r+4)|0;t=d[s]|d[s+1|0]<<8|d[s+2|0]<<16|d[s+3|0]<<24|0;if((t|0)==2105508482|(t|0)==1885956979|(t|0)==1684632949|(t|0)==1818653296){u=8127;break}else if((t|0)==1987014509){u=8119;break L10936}else if((t|0)==538988650|(t|0)==1952539757|(t|0)==1953459824|(t|0)==1635017845|(t|0)==1887007846){v=l;break L10936}else if((t|0)==2003395685|(t|0)==1701079415|(t|0)==1701147238|(t|0)==1802401130|(t|0)==1952672112){u=8126;break}t=g+r|0;s=cV(d[t]|d[t+1|0]<<8|d[t+2|0]<<16|d[t+3|0]<<24|0)|0;t=s>>>0<4;w=NU(t?4:s,t?0:0,q,p)|0;t=G;s=NU(w,t,8,0)|0;x=G;y=0;if((x|0)>(y|0)|(x|0)==(y|0)&s>>>0>e>>>0){z=o;A=l;break L10934}else{p=t;q=w}}if((u|0)==8127){u=0;w=g+r|0;t=cV(d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0)|0;w=t>>>0<4;B=(o|0)>50?o:50;C=w?0:0;D=w?4:t}else if((u|0)==8126){u=0;t=g+r|0;w=cV(d[t]|d[t+1|0]<<8|d[t+2|0]<<16|d[t+3|0]<<24|0)|0;t=w>>>0<4;B=(o|0)>95?o:95;C=t?0:0;D=t?4:w}w=NU(D,C,q,p)|0;t=G;s=NU(w,t,8,0)|0;y=G;x=0;if((y|0)>(x|0)|(y|0)==(x|0)&s>>>0>e>>>0){z=B;A=l;break L10934}else{m=t;n=w;o=B}}if((u|0)==8119){u=0;n=NU(q,p,4,0)|0;v=n}n=g+r|0;m=d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0;n=cV(m|0)|0;L10948:do{if(n>>>0<8){do{if((m|0)==16777216){w=NU(q,p,12,0)|0;t=G;s=0;if((t|0)>(s|0)|(t|0)==(s|0)&w>>>0>e>>>0){break}w=g+(r+8)|0;s=w|0;t=w+4|0;w=d[t]|d[t+1|0]<<8|d[t+2|0]<<16|d[t+3|0]<<24|0;t=cV(d[s]|d[s+1|0]<<8|d[s+2|0]<<16|d[s+3|0]<<24|0)|0;if(!((((w>>>16|0<<16)<<8|0>>>24)&65280|(w>>>24|0<<8)|(w<<8|0>>>24|(w>>>8|0<<24)&255)<<16|0)==0&(t|0)==0)){E=100;break L10948}}}while(0);E=(o|0)>50?o:50}else{E=100}}while(0);o=n>>>0<4;m=NU(o?4:n,o?0:0,q,p)|0;o=G;t=NU(m,o,8,0)|0;w=G;s=0;if((w|0)>(s|0)|(w|0)==(s|0)&t>>>0>e>>>0){z=E;A=v;break}else{h=o;j=m;k=E;l=v}}if((z|0)<51|(A|0)==-1){f=z;i=b;return f|0}v=e-16|0;if((A|0)>=(v|0)){f=z;i=b;return f|0}e=c[a+4>>2]|0;a=(A|0)<0|0?-1:0;l=A;L10962:while(1){A=l;E=e+A|0;do{if((d[E]|d[E+1|0]<<8|d[E+2|0]<<16|d[E+3|0]<<24|0)==1919706216){k=e+(A+8)|0;if((d[k]|d[k+1|0]<<8|d[k+2|0]<<16|d[k+3|0]<<24|0)!=1919707245){break}k=e+(A+12)|0;if((d[k]|d[k+1|0]<<8|d[k+2|0]<<16|d[k+3|0]<<24|0)==1195724877){break L10962}}}while(0);A=NU(l,a,2,0)|0;E=G;n=(v|0)<0|0?-1:0;if((E|0)<(n|0)|(E|0)==(n|0)&A>>>0<v>>>0){a=E;l=A}else{f=z;u=8142;break}}if((u|0)==8142){i=b;return f|0}AN(0,24,1700040,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;f=5;i=b;return f|0}function jF(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0;d=i;i=i+72|0;e=d|0;f=d+24|0;g=d+40|0;h=d+48|0;j=d+56|0;k=b+12|0;l=c[k>>2]|0;m=b+16|0;n=c[m>>2]|0;c[j>>2]=1953460082;o=j+8|0;c[o>>2]=0;c[o+4>>2]=0;p=l+4|0;c[p>>2]=b;q=n+88|0;if((c[q>>2]|0)==0){r=2147483647;s=-1}else{t=ip(n)|0;r=G;s=t}c[o>>2]=s;c[o+4>>2]=r;r=jD(l,n,j)|0;if((r|0)<0){AN(b,16,1720032,(u=i,i=i+8|0,c[u>>2]=r,u)|0);i=u;jH(b)|0;v=r;i=d;return v|0}if((c[l+24>>2]|0)==0){AN(b,16,1716168,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;jH(b)|0;v=-1094995529;i=d;return v|0}do{if((c[q>>2]|0)==0){w=c[b+24>>2]|0;x=8181}else{L10986:do{if((c[l+100>>2]|0)>0){r=h;j=b+24|0;n=c[j>>2]|0;o=b+28|0;s=(c[k>>2]|0)+100|0;t=0;while(1){if(t>>>0>=n>>>0){break}z=c[(c[o>>2]|0)+(t<<2)>>2]|0;if((c[z+4>>2]|0)==(c[s>>2]|0)){x=8157;break}else{t=t+1|0}}do{if((x|0)==8157){if((z|0)==0){break}c[z+76>>2]=48;t=c[z+12>>2]|0;s=il(c[t>>2]|0,0,0,1)|0;o=G;n=z+404|0;A=c[n>>2]|0;L10995:do{if((A|0)>0){B=z+400|0;C=z+40|0;D=z+56|0;E=0;F=A;while(1){H=c[B>>2]|0;I=E+1|0;if((I|0)<(F|0)){J=H+(I*24|0)+8|0}else{J=D}K=c[J>>2]|0;L=c[J+4>>2]|0;M=H+(E*24|0)|0;N=il(c[t>>2]|0,c[M>>2]|0,c[M+4>>2]|0,0)|0;if(!((N|0)==(c[M>>2]|0)&(G|0)==(c[M+4>>2]|0))){break}M=iK(c[t>>2]|0)|0;if((M|0)<=((c[H+(E*24|0)+16>>2]>>2)-2|0)){N=M<<1|1;O=A7(N)|0;c[h>>2]=O;if((O|0)==0){break L10995}do{if((M|0)==0){a[O]=0}else{P=(iK(c[t>>2]|0)|0)&65535;if((P|0)==65534){Q=c[t>>2]|0;R=c[h>>2]|0;iO(Q,M,R,N)|0;break}else if((P|0)==65279){iP(c[t>>2]|0,M,c[h>>2]|0,N)|0;break}else{R=c[h>>2]|0;y=(P>>>8|P<<8)&65535;a[R]=y&255;y=y>>8;a[R+1|0]=y&255;R=M-1|0;if(R>>>0<2){a[(c[h>>2]|0)+M|0]=0;break}else{P=c[t>>2]|0;Q=(c[h>>2]|0)+2|0;iN(P,2147483647,Q,R)|0;break}}}}while(0);M=H+(E*24|0)+8|0;N=c[M>>2]|0;O=c[M+4>>2]|0;M=c[h>>2]|0;mM(b,E,C,N,O,K,L,M)|0;A6(r)}M=c[n>>2]|0;if((I|0)<(M|0)){E=I;F=M}else{break L10995}}AN(b,16,1702744,(u=i,i=i+8|0,c[u>>2]=E,u)|0);i=u}}while(0);n=c[t>>2]|0;il(n,s,o,0)|0;S=j;break L10986}}while(0);AN(b,16,1704800,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;S=j}else{S=b+24|0}}while(0);if((c[S>>2]|0)==0){T=0;U=b+24|0;break}r=b+28|0;n=f|0;A=g|0;F=g+4|0;C=b;D=0;while(1){B=c[(c[r>>2]|0)+(D<<2)>>2]|0;M=B+8|0;do{if((c[(c[M>>2]|0)+52>>2]|0)==1684237684){O=c[B+12>>2]|0;N=O;R=il(c[N>>2]|0,0,0,1)|0;Q=G;if((c[B+404>>2]|0)==0){break}P=c[N>>2]|0;V=c[B+400>>2]|0;W=c[V>>2]|0;X=c[V+4>>2]|0;il(P,W,X,0)|0;X=iM(c[m>>2]|0)|0;W=c[O+1216>>2]|0;O=W&1;P=(W&2|0)==0?O:O|2;O=c[M>>2]|0;c[A>>2]=c[O+108>>2];c[F>>2]=c[O+104>>2];if((Cl(e,g,(W&4|0)==0?P:P|4,0,C)|0)>=0){P=B+88|0;W=Cj(e,n,X)|0;zK(P,1708944,W,0)|0}W=c[N>>2]|0;il(W,R,Q,0)|0;}}while(0);B=D+1|0;M=c[S>>2]|0;if(B>>>0<M>>>0){D=B}else{w=M;x=8181;break}}}}while(0);do{if((x|0)==8181){S=b+24|0;if((w|0)==0){T=0;U=S;break}e=b+28|0;g=0;m=w;do{f=c[e>>2]|0;h=c[f+(g<<2)>>2]|0;J=c[(c[h+12>>2]|0)+156>>2]|0;do{if(!((J|0)<1|(m|0)==0)){z=-1;k=0;do{z=(c[(c[f+(k<<2)>>2]|0)+4>>2]|0)==(J|0)?k:z;k=k+1|0;}while(k>>>0<m>>>0);if((z|0)<0|(z|0)==(g|0)){break}k=zJ(c[(c[f+(z<<2)>>2]|0)+88>>2]|0,1708944,0,0)|0;if((k|0)==0){break}zK(h+88|0,1708944,c[k+4>>2]|0,0)|0}}while(0);g=g+1|0;m=c[S>>2]|0;}while(g>>>0<m>>>0);L11048:do{if((m|0)==0){Y=0}else{g=b+28|0;e=0;h=m;f=m;L11050:while(1){J=c[g>>2]|0;k=c[J+(e<<2)>>2]|0;o=e+1|0;L11052:do{if((c[(c[k+8>>2]|0)+52>>2]|0)==1684237684){if((h|0)!=0){s=0;do{t=c[J+(s<<2)>>2]|0;if((c[(c[t+8>>2]|0)+8>>2]|0)==0){if((c[(c[t+12>>2]|0)+156>>2]|0)==(o|0)){Z=h;_=f;break L11052}}s=s+1|0;}while(s>>>0<h>>>0)}$=zJ(c[k+88>>2]|0,1708944,0,0)|0;if(($|0)!=0){break L11050}s=c[S>>2]|0;Z=s;_=s}else{Z=h;_=f}}while(0);if(o>>>0<Z>>>0){e=o;h=Z;f=_}else{Y=_;break L11048}}zK(b+1140|0,1708944,c[$+4>>2]|0,0)|0;Y=c[S>>2]|0}}while(0);if((Y|0)==0){T=0;U=S;break}m=b+28|0;f=l+8|0;h=0;while(1){e=c[(c[m>>2]|0)+(h<<2)>>2]|0;g=c[e+12>>2]|0;k=g+92|0;do{if((c[k>>2]|0)<1){AN(c[p>>2]|0,24,1706824,(u=i,i=i+8|0,c[u>>2]=c[g+8>>2],u)|0);i=u;J=c[f>>2]|0;c[k>>2]=J;if((J|0)>=1){break}c[k>>2]=1}}while(0);k=c[e+8>>2]|0;do{if((c[k+8>>2]|0)==1){if((c[k+48>>2]|0)!=86018){break}c[e+448>>2]=c[g+1232>>2]}}while(0);g=h+1|0;e=c[S>>2]|0;if(g>>>0<e>>>0){h=g}else{T=e;U=S;break}}}}while(0);if(!((c[l+88>>2]|0)==0|(T|0)==0)){u=b+28|0;p=0;Y=T;while(1){T=c[(c[u>>2]|0)+(p<<2)>>2]|0;$=c[T+12>>2]|0;_=T+56|0;Z=c[_>>2]|0;w=c[_+4>>2]|0;if((Z|0)==0&(w|0)==0){aa=Y}else{_=$+1208|0;S=c[_>>2]|0;h=c[$+92>>2]|0;$=N2(S<<3|0>>>29,c[_+4>>2]<<3|S>>>29,h,(h|0)<0|0?-1:0)|0;h=N0($,G,Z,w)|0;c[(c[T+8>>2]|0)+72>>2]=h;aa=c[U>>2]|0}h=p+1|0;if(h>>>0<aa>>>0){p=h;Y=aa}else{break}}}aa=l+124|0;Y=c[aa>>2]|0;if((Y|0)<=0){v=0;i=d;return v|0}p=l+120|0;l=b+28|0;b=0;u=Y;while(1){if(b>>>0>=(c[U>>2]|0)>>>0){v=0;x=8229;break}Y=c[(c[p>>2]|0)+(b<<2)>>2]|0;if((Y|0)==0){ab=u}else{c[(c[(c[(c[l>>2]|0)+(b<<2)>>2]|0)+8>>2]|0)+72>>2]=Y;ab=c[aa>>2]|0}Y=b+1|0;if((Y|0)<(ab|0)){b=Y;u=ab}else{v=0;x=8228;break}}if((x|0)==8228){i=d;return v|0}else if((x|0)==8229){i=d;return v|0}return 0}function jG(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0;d=i;i=i+16|0;e=d|0;f=c[a+12>>2]|0;g=f;h=f+4|0;c[h>>2]=a;j=a+24|0;k=f+28|0;l=f+112|0;f=a+16|0;m=e|0;n=e+8|0;o=a+28|0;a=b+32|0;p=b+16|0;q=b+48|0;r=b+8|0;s=0;L11098:while(1){t=c[j>>2]|0;do{if((t|0)==0){u=s}else{v=c[o>>2]|0;w=0;x=2147483647;y=-1;z=0;A=s;while(1){B=c[v+(w<<2)>>2]|0;C=c[B+12>>2]|0;D=c[C>>2]|0;L11104:do{if((D|0)==0){E=z;F=x;H=y;I=A}else{J=c[C+120>>2]|0;if((J|0)>=(c[B+404>>2]|0)){E=z;F=x;H=y;I=A;break}K=c[B+400>>2]|0;L=K+(J*24|0)|0;M=K+(J*24|0)+8|0;J=c[C+92>>2]|0;K=AW(c[M>>2]|0,c[M+4>>2]|0,1e6,0,J,(J|0)<0|0?-1:0)|0;J=G;do{if((z|0)!=0){M=c[f>>2]|0;if((c[M+88>>2]|0)==0){N=L|0;O=c[N+4>>2]|0;P=z|0;Q=c[P+4>>2]|0;if((O|0)<(Q|0)|(O|0)==(Q|0)&(c[N>>2]|0)>>>0<(c[P>>2]|0)>>>0){break}else{E=z;F=x;H=y;I=A;break L11104}}P=(J|0)<(x|0)|(J|0)==(x|0)&K>>>0<y>>>0;if((D|0)!=(M|0)&P){break}if((D|0)!=(M|0)){E=z;F=x;H=y;I=A;break L11104}M=NV(y,x,K,J)|0;N=G;Q=-1;O=(N|0)>(Q|0)|(N|0)==(Q|0)&M>>>0>-1>>>0;Q=NV(0,0,M,N)|0;R=O?N:G;N=0;if((R|0)<(N|0)|(R|0)==(N|0)&(O?M:Q)>>>0<1000001>>>0){Q=L|0;M=c[Q+4>>2]|0;O=z|0;N=c[O+4>>2]|0;if((M|0)<(N|0)|(M|0)==(N|0)&(c[Q>>2]|0)>>>0<(c[O>>2]|0)>>>0){break}else{E=z;F=x;H=y;I=A;break L11104}}else{if(P){break}else{E=z;F=x;H=y;I=A;break L11104}}}}while(0);E=L;F=J;H=K;I=B}}while(0);B=w+1|0;if(B>>>0<t>>>0){w=B;x=F;y=H;z=E;A=I}else{break}}if((E|0)==0){u=I;break}S=c[I+12>>2]|0;A=S+120|0;c[A>>2]=(c[A>>2]|0)+1;z=c[l>>2]|0;y=c[l+4>>2]|0;if(!((z|0)==0&(y|0)==0)){x=E|0;w=c[x>>2]|0;v=c[x+4>>2]|0;B=(v|0)>(y|0)|(v|0)==(y|0)&w>>>0>z>>>0;D=B?z:w;w=B?y:v;c[x>>2]=D;c[x+4>>2]=w;x=E+16|0;v=c[x>>2]|0;y=v>>2;B=(y|0)<0|0?-1:0;z=NV(c[l>>2]|0,c[l+4>>2]|0,D,w)|0;w=G;c[x>>2]=((B|0)>(w|0)|(B|0)==(w|0)&y>>>0>z>>>0?z:y)<<2|v&3}v=I+76|0;do{if((c[v>>2]|0)!=48){y=S;z=E|0;w=il(c[y>>2]|0,c[z>>2]|0,c[z+4>>2]|0,0)|0;T=c[z>>2]|0;U=c[z+4>>2]|0;if(!((w|0)==(T|0)&(G|0)==(U|0))){V=8255;break L11098}w=mb(c[y>>2]|0,b,c[E+16>>2]>>2)|0;if((w|0)<0){W=w;V=8280;break L11098}w=S+1200|0;if((c[w>>2]|0)==0){break}y=nK(b,0,1024)|0;if((y|0)==0){AN(c[h>>2]|0,16,1723664,(X=i,i=i+1|0,i=i+7&-8,c[X>>2]=0,X)|0);i=X;break}else{z=S+176|0;NN(y|0,z|0,1024)|0;c[w>>2]=0;break}}}while(0);c[a>>2]=c[S+8>>2];w=E+8|0;z=c[w>>2]|0;y=c[w+4>>2]|0;c[p>>2]=z;c[p+4>>2]=y;w=S+36|0;B=c[w>>2]|0;do{if((B|0)==0){V=8268}else{x=S+56|0;D=c[x>>2]|0;C=S+32|0;if(D>>>0>=(c[C>>2]|0)>>>0){V=8268;break}P=c[S+172>>2]|0;O=NU(P,(P|0)<0|0?-1:0,z,y)|0;P=c[B+(D<<3)+4>>2]|0;D=NU(O,G,P,(P|0)<0|0?-1:0)|0;c[r>>2]=D;c[r+4>>2]=G;D=S+60|0;P=(c[D>>2]|0)+1|0;c[D>>2]=P;O=c[x>>2]|0;do{if(O>>>0<(c[C>>2]|0)>>>0){if((c[(c[w>>2]|0)+(O<<3)>>2]|0)!=(P|0)){break}c[x>>2]=O+1;c[D>>2]=0}}while(0);if((c[S+160>>2]|0)==0){break}c[p>>2]=0;c[p+4>>2]=-2147483648}}while(0);if((V|0)==8268){V=0;w=c[A>>2]|0;if((w|0)<(c[I+404>>2]|0)){Y=(c[I+400>>2]|0)+(w*24|0)+8|0}else{Y=I+56|0}w=NV(c[Y>>2]|0,c[Y+4>>2]|0,z,y)|0;c[q>>2]=w;c[r>>2]=z;c[r+4>>2]=y}if((c[v>>2]|0)==48){s=I;continue L11098}else{V=8273;break L11098}}}while(0);c[k>>2]=0;t=c[l>>2]|0;w=c[l+4>>2]|0;if((t|0)==0&(w|0)==0){W=-541478725;V=8277;break}il(c[f>>2]|0,t,w,0)|0;c[l>>2]=0;c[l+4>>2]=0;w=c[f>>2]|0;c[m>>2]=1953460082;c[n>>2]=-1;c[n+4>>2]=2147483647;if((jD(g,w,e)|0)<0){W=-541478725;V=8278;break}if((iq(c[f>>2]|0)|0)==0){s=u}else{W=-541478725;V=8276;break}}if((V|0)==8273){u=b+36|0;c[u>>2]=c[u>>2]|c[E+16>>2]&1;u=E|0;E=c[u+4>>2]|0;s=b+64|0;c[s>>2]=c[u>>2];c[s+4>>2]=E;W=0;i=d;return W|0}else if((V|0)==8280){i=d;return W|0}else if((V|0)==8276){i=d;return W|0}else if((V|0)==8277){i=d;return W|0}else if((V|0)==8278){i=d;return W|0}else if((V|0)==8255){AN(c[h>>2]|0,16,1727880,(X=i,i=i+24|0,c[X>>2]=c[S+8>>2],c[X+8>>2]=T,c[X+16>>2]=U,X)|0);i=X;W=-1094995529;i=d;return W|0}return 0}function jH(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;b=c[a+12>>2]|0;d=a+24|0;if((c[d>>2]|0)!=0){e=a+28|0;a=0;do{f=c[(c[(c[e>>2]|0)+(a<<2)>>2]|0)+12>>2]|0;A6(f+36|0);g=f+144|0;h=f+148|0;if((c[g>>2]|0)!=0){i=h;j=0;do{A6((c[i>>2]|0)+(j*108|0)+4|0);A6((c[i>>2]|0)+(j*108|0)+8|0);j=j+1|0;}while(j>>>0<(c[g>>2]|0)>>>0)}A6(h);g=f;if((c[f+4>>2]|0)==0){j=c[g>>2]|0;iV(j)|0}c[g>>2]=0;A6(f+20|0);A6(f+88|0);A6(f+76|0);A6(f+52|0);A6(f+44|0);A6(f+28|0);a=a+1|0;}while(a>>>0<(c[d>>2]|0)>>>0)}d=b+32|0;if((c[d>>2]|0)==0){k=b+88|0;A6(k);l=b+120|0;A6(l);return 0}a=b+36|0;e=a;g=c[e>>2]|0;if((c[g+24>>2]|0)!=0){j=0;i=g;do{A6((c[(c[i+28>>2]|0)+(j<<2)>>2]|0)+8|0);A6((c[(c[e>>2]|0)+28>>2]|0)+(j<<2)|0);j=j+1|0;i=c[e>>2]|0;}while(j>>>0<(c[i+24>>2]|0)>>>0)}A6(a);A6(d);k=b+88|0;A6(k);l=b+120|0;A6(l);return 0}function jI(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;g=a+24|0;if((c[g>>2]|0)>>>0<=b>>>0){h=-1094995529;return h|0}i=a+28|0;a=c[(c[i>>2]|0)+(b<<2)>>2]|0;j=c[a+12>>2]|0;k=mt(a,d,e,f)|0;do{if((k|0)<0){if((c[a+404>>2]|0)==0){h=-1094995529;return h|0}l=(c[a+400>>2]|0)+8|0;m=c[l+4>>2]|0;n=(m|0)>(e|0)|(m|0)==(e|0)&(c[l>>2]|0)>>>0>d>>>0?0:k;if((n|0)<0){h=-1094995529}else{o=n;break}return h|0}else{o=k}}while(0);c[j+120>>2]=o;k=c[j+36>>2]|0;do{if((k|0)!=0){n=c[j+32>>2]|0;l=0;m=0;while(1){if(l>>>0>=n>>>0){break}p=(c[k+(l<<3)>>2]|0)+m|0;if((p|0)>(o|0)){q=8304;break}else{l=l+1|0;m=p}}if((q|0)==8304){c[j+56>>2]=l;c[j+60>>2]=o-m}if((o|0)<0){h=o}else{break}return h|0}}while(0);j=(c[a+400>>2]|0)+(o*24|0)+8|0;o=c[j>>2]|0;a=c[j+4>>2]|0;if((c[g>>2]|0)==0){h=0;return h|0}j=0;if((e|0)<(j|0)|(e|0)==(j|0)&d>>>0<1>>>0){d=0;while(1){j=c[(c[i>>2]|0)+(d<<2)>>2]|0;c[j+448>>2]=c[(c[j+12>>2]|0)+1232>>2];L11224:do{if((d|0)!=(b|0)){e=AY(o,a,(c[(c[i>>2]|0)+(b<<2)>>2]|0)+40|0,j+40|0)|0;q=G;k=c[j+12>>2]|0;n=mt(j,e,q,f)|0;if((n|0)<0){if((c[j+404>>2]|0)==0){break}p=(c[j+400>>2]|0)+8|0;r=c[p+4>>2]|0;s=(r|0)>(q|0)|(r|0)==(q|0)&(c[p>>2]|0)>>>0>e>>>0?0:n;if((s|0)<0){break}else{t=s}}else{t=n}c[k+120>>2]=t;n=c[k+36>>2]|0;if((n|0)==0){break}s=c[k+32>>2]|0;e=0;p=0;while(1){if(e>>>0>=s>>>0){break L11224}q=(c[n+(e<<3)>>2]|0)+p|0;if((q|0)>(t|0)){break}else{e=e+1|0;p=q}}c[k+56>>2]=e;c[k+60>>2]=t-p}}while(0);j=d+1|0;if(j>>>0<(c[g>>2]|0)>>>0){d=j}else{h=0;break}}return h|0}else{d=0;while(1){t=c[(c[i>>2]|0)+(d<<2)>>2]|0;c[t+448>>2]=0;L11209:do{if((d|0)!=(b|0)){j=AY(o,a,(c[(c[i>>2]|0)+(b<<2)>>2]|0)+40|0,t+40|0)|0;m=G;l=c[t+12>>2]|0;n=mt(t,j,m,f)|0;if((n|0)<0){if((c[t+404>>2]|0)==0){break}s=(c[t+400>>2]|0)+8|0;q=c[s+4>>2]|0;r=(q|0)>(m|0)|(q|0)==(m|0)&(c[s>>2]|0)>>>0>j>>>0?0:n;if((r|0)<0){break}else{u=r}}else{u=n}c[l+120>>2]=u;n=c[l+36>>2]|0;if((n|0)==0){break}r=c[l+32>>2]|0;j=0;s=0;while(1){if(j>>>0>=r>>>0){break L11209}m=(c[n+(j<<3)>>2]|0)+s|0;if((m|0)>(u|0)){break}else{j=j+1|0;s=m}}c[l+56>>2]=j;c[l+60>>2]=u-s}}while(0);t=d+1|0;if(t>>>0<(c[g>>2]|0)>>>0){d=t}else{h=0;break}}return h|0}return 0}function jJ(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0;f=i;i=i+3200|0;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=f|0;h=f+2128|0;j=f+2136|0;k=f+2144|0;l=f+2152|0;m=f+3176|0;n=f+3192|0;c[n>>2]=0;o=(c[b+96>>2]|0)==0;p=c[e>>2]|0;if((p|0)==757935405&(o^1)){q=c[b+4>>2]|0;r=e+8|0;s=c[r>>2]|0;t=c[r+4>>2]|0;r=f+80|0;u=f+1104|0;NM(r|0,0,1024);NM(u|0,0,1024);v=c[q+24>>2]|0;if((v|0)==0){w=0;i=f;return w|0}x=c[(c[(c[q+28>>2]|0)+(v-1<<2)>>2]|0)+12>>2]|0;v=0;if((t|0)<(v|0)|(t|0)==(v|0)&s>>>0<9>>>0){w=0;i=f;return w|0}v=iM(d)|0;q=iI(d)|0;if((q|0)==1701667182){iM(d)|0;y=NU(s,t,-12,-1)|0;z=r;A=v-12|0;B=G;C=y;D=8343}else if((q|0)==1635017060){iM(d)|0;iM(d)|0;q=NU(s,t,-16,-1)|0;z=u;A=v-16|0;B=G;C=q;D=8343}else{E=0;F=t;H=s;I=v-8|0}if((D|0)==8343){v=A>>>0<1023;s=v?A:1023;t=v?0:0;v=(t|0)>(B|0)|(t|0)==(B|0)&s>>>0>C>>>0;q=v?C:s;v?B:t;t=q;iE(d,z,t)|0;a[z+t|0]=0;E=t;F=B;H=C;I=A}A=NV(H,F,I,(I|0)<0|0?-1:0)|0;F=G;if((I|0)>(E|0)){H=I-E|0;E=H;I=(H|0)<0|0?-1:0;io(d,E,I)|0;}I=iM(d)|0;E=iI(d)|0;if((E|0)==1701667182){iM(d)|0;H=NU(A,F,-12,-1)|0;J=r;K=I-12|0;L=G;M=H;D=8356}else if((E|0)==1635017060){iM(d)|0;iM(d)|0;E=NU(A,F,-16,-1)|0;J=u;K=I-16|0;L=G;M=E;D=8356}else{N=0;O=F;P=A;Q=I-8|0}if((D|0)==8356){I=K>>>0<1023;A=I?K:1023;F=I?0:0;I=(F|0)>(L|0)|(F|0)==(L|0)&A>>>0>M>>>0;E=I?M:A;I?L:F;F=E;iE(d,J,F)|0;a[J+F|0]=0;N=F;O=L;P=M;Q=K}K=NV(P,O,Q,(Q|0)<0|0?-1:0)|0;O=G;if((Q|0)>(N|0)){P=Q-N|0;N=P;Q=(P|0)<0|0?-1:0;io(d,N,Q)|0;}Q=iM(d)|0;N=iI(d)|0;if((N|0)==1635017060){iM(d)|0;iM(d)|0;P=NU(K,O,-16,-1)|0;R=u;S=Q-16|0;T=G;U=P;D=8363}else if((N|0)==1701667182){iM(d)|0;N=NU(K,O,-12,-1)|0;R=r;S=Q-12|0;T=G;U=N;D=8363}else{V=0;W=Q-8|0}if((D|0)==8363){Q=S>>>0<1023;N=Q?S:1023;O=Q?0:0;Q=(O|0)>(T|0)|(O|0)==(T|0)&N>>>0>U>>>0;K=Q?U:N;Q?T:O;O=K;iE(d,R,O)|0;a[R+O|0]=0;V=O;W=S}if((W|0)>(V|0)){S=W-V|0;V=S;W=(S|0)<0|0?-1:0;io(d,V,W)|0;}do{if((a[r]|0)!=0){if((a[u]|0)==0){break}do{if((a8(r|0,1629880)|0)==0){W=ct(u|0,1629064,(X=i,i=i+24|0,c[X>>2]=h,c[X+8>>2]=j,c[X+16>>2]=k,X)|0)|0;i=X;if((W|0)!=3){break}W=c[h>>2]|0;if((W-1|0)>>>0>=16383){w=1;i=f;return w|0}c[x+1232>>2]=W;w=1;i=f;return w|0}}while(0);if((a8(r|0,1628512)|0)==0){w=1}else{break}i=f;return w|0}}while(0);w=0;i=f;return w|0}if((p|0)==1886545833){Y=1673688;Z=0}else if((p|0)==1668179369){Y=1664136;Z=0}else if((p|0)==1801547177){Y=1662664;Z=0}else if((p|0)==1685024169){Y=1661248;Z=0}else if((p|0)==2054781097){Y=1659832;Z=0}else if((p|0)==1701998183){Y=1665704;Z=34}else if((p|0)==1869575337|(p|0)==1920431017){Y=1664136;Z=0}else if((p|0)==1752397428){Y=1654824;Z=0}else if((p|0)==1852143220){Y=1653704;Z=0}else if((p|0)==1953325993|(p|0)==1718512041){Y=1669848;Z=0}else if((p|0)==1835101865){Y=1682960;Z=0}else if((p|0)==1685480552){Y=1639976;Z=2}else if((p|0)==1885431664){Y=1638192;Z=2}else if((p|0)==1297240128){r=c[e+8>>2]|0;x=A1(r+1|0)|0;if((x|0)==0){w=-12;i=f;return w|0}iE(d,x,r)|0;a[x+r|0]=0;w=zK((c[b+4>>2]|0)+1140|0,1636152,x,8)|0;i=f;return w|0}else if((p|0)==1936029300){Y=1645928;Z=10}else if((p|0)==1853060724){Y=1644448;Z=10}else if((p|0)==1802073203){Y=1642584;Z=2}else if((p|0)==1668506980){Y=1658536;Z=0}else if((p|0)==1936024684){Y=1657232;Z=0}else if((p|0)==2036425897){Y=1666872;Z=0}else if((p|0)==1852139433){Y=1665704;Z=0}else if((p|0)==1852733044){Y=1651568;Z=0}else if((p|0)==1852535412){Y=1650032;Z=50}else if((p|0)==1802725732){Y=1648352;Z=50}else if((p|0)==1953849769|(p|0)==1414676905){Y=1681144;Z=0}else if((p|0)==1414676833){Y=1679248;Z=0}else if((p|0)==1920560297){Y=1671824;Z=0}else if((p|0)==1953658793){Y=1677368;Z=0}else if((p|0)==1953656931|(p|0)==2037408681){Y=1675392;Z=0}else if((p|0)==1651270057){Y=1668168;Z=0}else if((p|0)==1364348992){x=c[e+8>>2]|0;r=A1(x+1|0)|0;if((r|0)==0){w=-12;i=f;return w|0}iE(d,r,x)|0;a[r+x|0]=0;w=zK((c[b+4>>2]|0)+1140|0,1635488,r,8)|0;i=f;return w|0}else{Y=0;Z=0}r=e+8|0;x=c[r>>2]|0;h=c[r+4>>2]|0;k=0;L11332:do{if(((h|0)>(k|0)|(h|0)==(k|0)&x>>>0>8>>>0)&(o^1)){j=iM(d)|0;if((iI(d)|0)!=1635017060){w=0;i=f;return w|0}u=iM(d)|0;iM(d)|0;W=j-16|0;j=NU(x,h,-16,-1)|0;V=G;c[r>>2]=j;c[r+4>>2]=V;if((p|0)!=1920364387){_=0;$=W;aa=u;ab=V;ac=j;break}S=g;if((u|0)==14){ad=62}else if((u|0)==27){ad=79}else if((u|0)==13){ad=8}else{AN(c[b+4>>2]|0,24,1632832,(X=i,i=i+8|0,c[X>>2]=u,X)|0);i=X;io(d,W,(W|0)<0|0?-1:0)|0;_=0;$=W;aa=u;ab=V;ac=j;break}O=b+4|0;R=mK(c[O>>2]|0,0)|0;do{if((R|0)==0){ae=-12}else{K=A7(1248)|0;if((K|0)==0){ae=-12;break}c[R+12>>2]=K;K=mb(d,g,W)|0;if((K|0)<0){ae=K;break}K=R+72|0;c[K>>2]=c[K>>2]|1024;K=R+104|0;NN(K|0,S|0,80)|0;c[R+136>>2]=c[R>>2];K=R+140|0;c[K>>2]=c[K>>2]|1;K=R+8|0;c[(c[K>>2]|0)+8>>2]=0;c[(c[K>>2]|0)+48>>2]=ad;_=0;$=W;aa=u;ab=V;ac=j;break L11332}}while(0);AN(c[O>>2]|0,16,1634944,(X=i,i=i+1|0,i=i+7&-8,c[X>>2]=0,X)|0);i=X;w=ae;i=f;return w|0}else{j=0;if((h|0)<(j|0)|(h|0)==(j|0)&x>>>0<5>>>0|(Y|0)==0|o^1){_=0;$=x;aa=0;ab=h;ac=x;break}else{j=e+8|0;V=iK(d)|0;u=(iK(d)|0)&65535;W=n;ns(u,W)|0;W=NU(x,h,-4,-1)|0;R=G;c[j>>2]=W;c[j+4>>2]=R;_=u;$=V;aa=0;ab=R;ac=W;break}}}while(0);if((Y|0)==0){w=0;i=f;return w|0}h=0;if((ab|0)<(h|0)|(ab|0)==(h|0)&ac>>>0<0>>>0){w=-1094995529;i=f;return w|0}h=$>>>0<1023;x=h?$:1023;$=h?0:0;h=($|0)>(ab|0)|($|0)==(ab|0)&x>>>0>ac>>>0;h?ab:$;$=h?ac:x;if((Z|0)!=0){dk[Z&63](b,d,$,Y)|0;w=0;i=f;return w|0}if((aa|0)==0){if(_>>>0<1024|(_|0)==32767){D=8423}else{D=8424}}else if((aa|0)==3){D=8423}else{D=8424}if((D|0)==8424){aa=l|0;iE(d,aa,$)|0;a[l+$|0]=0}else if((D|0)==8423){jB(d,$,l|0,1024)}$=b+4|0;b=l|0;zK((c[$>>2]|0)+1140|0,Y,b,0)|0;l=n;if((a[l]|0)==0){w=0;i=f;return w|0}if((a8(l|0,1634256)|0)==0){w=0;i=f;return w|0}l=m|0;bc(l|0,16,1633496,(X=i,i=i+16|0,c[X>>2]=Y,c[X+8>>2]=n,X)|0)|0;i=X;zK((c[$>>2]|0)+1140|0,l,b,0)|0;w=0;i=f;return w|0}function jK(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0;d=i;i=i+24|0;iD(b)|0;f=(iD(b)|0)<<16;if((f-65536|0)>>>0>9568256){i=d;return 0}b=d|0;bc(b|0,20,1630592,(g=i,i=i+8|0,c[g>>2]=c[249904+((f>>16)-1<<2)>>2],g)|0)|0;i=g;zK((c[a+4>>2]|0)+1140|0,e,b,0)|0;i=d;return 0}function jL(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;f=i;i=i+16|0;g=f|0;iK(b)|0;h=iK(b)|0;do{if(d>>>0>5){j=(iK(b)|0)&65535;k=g|0;l=h<<16>>16;if(j<<16>>16==0){m=k;n=l;break}o=j<<16>>16;bc(k|0,16,1631120,(p=i,i=i+16|0,c[p>>2]=l,c[p+8>>2]=o,p)|0)|0;i=p;q=k;r=a+4|0;s=c[r>>2]|0;t=s+1140|0;u=zK(t,e,q,0)|0;i=f;return 0}else{m=g|0;n=h<<16>>16}}while(0);bc(m|0,16,1631952,(p=i,i=i+8|0,c[p>>2]=n,p)|0)|0;i=p;q=m;r=a+4|0;s=c[r>>2]|0;t=s+1140|0;u=zK(t,e,q,0)|0;i=f;return 0}function jM(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0;d=i;i=i+16|0;iD(b)|0;iD(b)|0;iD(b)|0;f=d|0;g=iD(b)|0;bc(f|0,16,1631952,(b=i,i=i+8|0,c[b>>2]=g,b)|0)|0;i=b;zK((c[a+4>>2]|0)+1140|0,e,f,0)|0;i=d;return 0}function jN(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0;d=i;i=i+16|0;f=d|0;g=iD(b)|0;bc(f|0,16,1631952,(b=i,i=i+8|0,c[b>>2]=g,b)|0)|0;i=b;zK((c[a+4>>2]|0)+1140|0,e,f,0)|0;i=d;return 0}function jO(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;f=i;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=c[b+4>>2]|0;b=c[e>>2]|0;h=e+8|0;e=c[h>>2]|0;j=c[h+4>>2]|0;h=c[g+24>>2]|0;if((h|0)==0){k=0;i=f;return k|0}l=(c[(c[g+28>>2]|0)+(h-1<<2)>>2]|0)+8|0;h=c[l>>2]|0;if((c[h+48>>2]|0)!=1096176969){k=0;i=f;return k|0}g=c[h+100>>2]|0;m=NU(g,(g|0)<0|0?-1:0,e,j)|0;g=G;n=NU(m,g,8,0)|0;o=NU(m,g,24,0)|0;g=G;m=0;p=0;if(g>>>0>m>>>0|g>>>0==m>>>0&o>>>0>2147483647>>>0|(j>>>0>p>>>0|j>>>0==p>>>0&e>>>0>2147483647>>>0)){k=-1094995529;i=f;return k|0}p=A2(c[h+96>>2]|0,o)|0;if((p|0)==0){k=-12;i=f;return k|0}c[(c[l>>2]|0)+96>>2]=p;o=(c[l>>2]|0)+100|0;l=c[o>>2]|0;c[o>>2]=n;n=NU(e,j,8,0)|0;j=p+l|0;y=cV(n|0)|0;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;j=p+(l+4)|0;y=b;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;iE(d,p+(l+8)|0,e)|0;k=0;i=f;return k|0}function jP(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;g=i;h=f;f=i;i=i+16|0;c[f>>2]=c[h>>2];c[f+4>>2]=c[h+4>>2];c[f+8>>2]=c[h+8>>2];c[f+12>>2]=c[h+12>>2];h=c[b+4>>2]|0;b=c[h+24>>2]|0;j=(c[(c[h+28>>2]|0)+(b-1<<2)>>2]|0)+8|0;h=c[j>>2]|0;do{if((c[h+52>>2]|0)==1852397121){if((c[h+48>>2]|0)!=28){break}k=f+8|0;l=c[k+4>>2]|0;m=0;if(!((l|0)>(m|0)|(l|0)==(m|0)&(c[k>>2]|0)>>>0>11>>>0)){break}k=10;m=0;io(e,k,m)|0;if((iK(e)|0)!=3405){n=0;i=g;return n|0}c[h+120>>2]=1440;n=0;i=g;return n|0}}while(0);m=f|0;k=d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0;m=f+8|0;f=m|0;l=d[f]|d[f+1|0]<<8|d[f+2|0]<<16|d[f+3|0]<<24|0;f=m+4|0;m=d[f]|d[f+1|0]<<8|d[f+2|0]<<16|d[f+3|0]<<24|0;if((b|0)==0){n=0;i=g;return n|0}if((c[h+48>>2]|0)!=1096176969){n=0;i=g;return n|0}b=c[h+100>>2]|0;f=NU(b,(b|0)<0|0?-1:0,l,m)|0;b=G;o=NU(f,b,8,0)|0;p=NU(f,b,24,0)|0;b=G;f=0;q=0;if(b>>>0>f>>>0|b>>>0==f>>>0&p>>>0>2147483647>>>0|(m>>>0>q>>>0|m>>>0==q>>>0&l>>>0>2147483647>>>0)){n=-1094995529;i=g;return n|0}q=A2(c[h+96>>2]|0,p)|0;if((q|0)==0){n=-12;i=g;return n|0}c[(c[j>>2]|0)+96>>2]=q;p=(c[j>>2]|0)+100|0;j=c[p>>2]|0;c[p>>2]=o;o=NU(l,m,8,0)|0;m=q+j|0;y=cV(o|0)|0;a[m]=y&255;y=y>>8;a[m+1|0]=y&255;y=y>>8;a[m+2|0]=y&255;y=y>>8;a[m+3|0]=y&255;m=q+(j+4)|0;y=k;a[m]=y&255;y=y>>8;a[m+1|0]=y&255;y=y>>8;a[m+2|0]=y&255;y=y>>8;a[m+3|0]=y&255;iE(e,q+(j+8)|0,l)|0;n=0;i=g;return n|0}function jQ(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;f=i;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=c[b+4>>2]|0;b=c[e>>2]|0;h=e+8|0;e=c[h>>2]|0;j=c[h+4>>2]|0;h=c[g+24>>2]|0;if((h|0)==0){k=0;i=f;return k|0}l=(c[(c[g+28>>2]|0)+(h-1<<2)>>2]|0)+8|0;h=c[l>>2]|0;if((c[h+48>>2]|0)!=83){k=0;i=f;return k|0}g=c[h+100>>2]|0;m=NU(g,(g|0)<0|0?-1:0,e,j)|0;g=G;n=NU(m,g,8,0)|0;o=NU(m,g,24,0)|0;g=G;m=0;p=0;if(g>>>0>m>>>0|g>>>0==m>>>0&o>>>0>2147483647>>>0|(j>>>0>p>>>0|j>>>0==p>>>0&e>>>0>2147483647>>>0)){k=-1094995529;i=f;return k|0}p=A2(c[h+96>>2]|0,o)|0;if((p|0)==0){k=-12;i=f;return k|0}c[(c[l>>2]|0)+96>>2]=p;o=(c[l>>2]|0)+100|0;l=c[o>>2]|0;c[o>>2]=n;n=NU(e,j,8,0)|0;j=p+l|0;y=cV(n|0)|0;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;j=p+(l+4)|0;y=b;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;iE(d,p+(l+8)|0,e)|0;k=0;i=f;return k|0}function jR(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;f=i;i=i+272|0;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=f|0;h=f+264|0;j=e+8|0;e=NU(c[j>>2]|0,c[j+4>>2]|0,-5,-1)|0;k=G;c[j>>2]=e;c[j+4>>2]=k;l=0;if((k|0)<(l|0)|(k|0)==(l|0)&e>>>0<0>>>0){i=f;return 0}l=iD(d)|0;iL(d)|0;if((l|0)!=0){iM(d)|0}l=iD(d)|0;if((l|0)<=0){i=f;return 0}m=g|0;n=b+4|0;b=h|0;o=h+4|0;p=0;q=k;k=e;while(1){e=0;if((q|0)<(e|0)|(q|0)==(e|0)&k>>>0<9>>>0){r=8514;break}e=iQ(d)|0;s=G;t=iD(d)|0;u=t+9|0;v=NV(k,q,u,(u|0)<0|0?-1:0)|0;u=G;c[j>>2]=v;c[j+4>>2]=u;w=0;if((u|0)<(w|0)|(u|0)==(w|0)&v>>>0<0>>>0){r=8515;break}iE(d,m,t)|0;a[g+t|0]=0;t=c[n>>2]|0;c[b>>2]=1;c[o>>2]=1e7;mM(t,p,h,e,s,0,-2147483648,m)|0;s=p+1|0;if((s|0)<(l|0)){p=s;q=u;k=v}else{r=8512;break}}if((r|0)==8512){i=f;return 0}else if((r|0)==8515){i=f;return 0}else if((r|0)==8514){i=f;return 0}return 0}function jS(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;a=c[f+24>>2]|0;if((a|0)==0){g=0;i=e;return g|0}h=c[(c[(c[f+28>>2]|0)+(a-1<<2)>>2]|0)+12>>2]|0;iD(b)|0;iL(b)|0;a=iM(b)|0;if((a|0)==0){g=0;i=e;return g|0}if(a>>>0>536870910){g=-1094995529;i=e;return g|0}f=A1(a<<3)|0;j=h+20|0;c[j>>2]=f;if((f|0)==0){g=-12;i=e;return g|0}f=h+16|0;c[f>>2]=a;h=c[d>>2]|0;L11474:do{if((h|0)==1868788851){d=b+52|0;k=0;while(1){if((c[d>>2]|0)!=0){l=k;break L11474}m=iM(b)|0;n=(c[j>>2]|0)+(k<<3)|0;c[n>>2]=m;c[n+4>>2]=0;n=k+1|0;if(n>>>0<a>>>0){k=n}else{l=n;break}}}else if((h|0)==875982691){k=b+52|0;d=0;while(1){if((c[k>>2]|0)!=0){l=d;break L11474}n=iQ(b)|0;m=(c[j>>2]|0)+(d<<3)|0;c[m>>2]=n;c[m+4>>2]=G;m=d+1|0;if(m>>>0<a>>>0){d=m}else{l=m;break}}}else{g=-1094995529;i=e;return g|0}}while(0);c[f>>2]=l;g=(c[b+52>>2]|0)==0?0:-541478725;i=e;return g|0}function jT(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=a+4|0;a=c[f>>2]|0;d=c[a+24>>2]|0;if((d|0)==0){g=0;i=e;return g|0}h=c[(c[(c[a+28>>2]|0)+(d-1<<2)>>2]|0)+12>>2]|0;iD(b)|0;iL(b)|0;d=iM(b)|0;if((d|0)==0){g=0;i=e;return g|0}if(d>>>0>536870910){g=-1094995529;i=e;return g|0}a=A1(d<<3)|0;j=h+36|0;k=j;c[k>>2]=a;if((a|0)==0){g=-12;i=e;return g|0}a=b+52|0;l=h+172|0;m=0;while(1){if((c[a>>2]|0)!=0){n=m;o=8546;break}p=iM(b)|0;q=iM(b)|0;c[(c[k>>2]|0)+(m<<3)>>2]=p;c[(c[k>>2]|0)+(m<<3)+4>>2]=q;p=-q|0;r=m+2|0;if((((q|0)>-1?q:p)|0)>268435456&r>>>0<d>>>0){o=8542;break}if(r>>>0<d>>>0&(q|0)<0){q=c[l>>2]|0;c[l>>2]=(q|0)>(p|0)?q:p}p=m+1|0;if(p>>>0<d>>>0){m=p}else{n=p;o=8546;break}}if((o|0)==8546){c[h+32>>2]=n;g=(c[a>>2]|0)==0?0:-541478725;i=e;return g|0}else if((o|0)==8542){AN(c[f>>2]|0,24,1740128,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;A6(j);c[h+32>>2]=0;g=0;i=e;return g|0}return 0}function jU(d,e,f){d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;g=i;h=f;f=i;i=i+16|0;c[f>>2]=c[h>>2];c[f+4>>2]=c[h+4>>2];c[f+8>>2]=c[h+8>>2];c[f+12>>2]=c[h+12>>2];h=d+4|0;d=c[h>>2]|0;j=c[d+24>>2]|0;if((j|0)==0){k=0;i=g;return k|0}l=c[(c[(c[d+28>>2]|0)+(j-1<<2)>>2]|0)+12>>2]|0;iM(e)|0;j=iM(e)|0;d=(j|0)<0|0?-1:0;m=f+8|0;f=NU(c[m>>2]|0,c[m+4>>2]|0,-1,-1)|0;m=N0(f,G,12,0)|0;f=NU(m,G,1,0)|0;m=G;if((d|0)>(m|0)|(d|0)==(m|0)&j>>>0>f>>>0|j>>>0>39768214){k=-1094995529;i=g;return k|0}f=l+148|0;A4(c[f>>2]|0);m=l+144|0;c[m>>2]=0;l=A7(j*108|0)|0;d=l;c[f>>2]=d;if((l|0)==0){k=-12;i=g;return k|0}c[m>>2]=j;if((j|0)==0){k=0;i=g;return k|0}else{n=0;o=d}L11523:while(1){d=iM(e)|0;j=il(e,0,0,1)|0;l=G;p=NU(d,0,-4,-1)|0;q=NU(p,G,j,l)|0;l=G;if(d>>>0<12){k=-1094995529;r=8587;break}j=o+(n*108|0)|0;c[j>>2]=iI(e)|0;iM(e)|0;L11526:do{if((c[j>>2]|0)==1936288865&d>>>0>150){p=10;s=0;io(e,p,s)|0;s=iD(e)|0;p=(s&65532)>>>0>27?27:s&65535;s=o+(n*108|0)+12|0;iE(e,s,27)|0;a[o+(n*108|0)+12+p|0]=0;AN(c[h>>2]|0,48,1611632,(t=i,i=i+16|0,c[t>>2]=s,c[t+8>>2]=p,t)|0);i=t;u=12;v=0;io(e,u,v)|0;v=iD(e)|0;u=(v&65472)>>>0>63?63:v&65535;v=o+(n*108|0)+40|0;iE(e,v,63)|0;a[o+(n*108|0)+40+u|0]=0;AN(c[h>>2]|0,48,1745080,(t=i,i=i+16|0,c[t>>2]=v,c[t+8>>2]=u,t)|0);i=t;u=16;v=0;io(e,u,v)|0;v=o+(n*108|0)+106|0;b[v>>1]=(iK(e)|0)&65535;u=(iK(e)|0)&65535;b[o+(n*108|0)+104>>1]=u;AN(c[h>>2]|0,48,1744720,(t=i,i=i+16|0,c[t>>2]=b[v>>1]|0,c[t+8>>2]=u<<16>>16,t)|0);i=t;u=16;v=0;io(e,u,v)|0;v=o+(n*108|0)+4|0;u=o+(n*108|0)+8|0;do{w=il(e,0,0,1)|0;x=G;if(!((x|0)<(l|0)|(x|0)==(l|0)&w>>>0<q>>>0)){break L11526}if((iq(e)|0)!=0){k=-541478725;r=8593;break L11523}w=(iK(e)|0)&65535;x=iK(e)|0;AN(c[h>>2]|0,48,1744008,(t=i,i=i+16|0,c[t>>2]=w<<16>>16,c[t+8>>2]=x&65535,t)|0);i=t;if((x&1|0)==0){y=x&65535}else{y=x+1&65535}if((w<<16>>16|0)==0){A4(c[u>>2]|0);x=y&65535;z=A1(x+1|0)|0;c[u>>2]=z;if((z|0)==0){k=-12;r=8588;break L11523}iE(e,z,x)|0;a[(c[u>>2]|0)+x|0]=0;if(y<<16>>16!=0){z=0;do{A=(c[u>>2]|0)+z|0;if((a[A]|0)==58){a[A]=47}z=z+1|0;}while((z|0)<(x|0))}AN(c[h>>2]|0,48,1740816,(t=i,i=i+8|0,c[t>>2]=c[u>>2],t)|0);i=t}else if((w<<16>>16|0)==2){A4(c[v>>2]|0);x=y&65535;z=A7(x+1|0)|0;c[v>>2]=z;if((z|0)==0){k=-12;r=8589;break L11523}iE(e,z,x)|0;do{if(x>>>0>p>>>0){z=c[v>>2]|0;if((ba(z|0,s|0,p|0)|0)!=0){B=y;break}A=x-p|0;C=A&65535;NP(z|0,z+p|0,C|0);a[(c[v>>2]|0)+C|0]=0;B=A&65535}else{B=y}}while(0);x=B&65535;if(B<<16>>16!=0){A=0;do{C=(c[v>>2]|0)+A|0;if((a[C]|0)==58){a[C]=47}A=A+1|0;}while((A|0)<(x|0))}AN(c[h>>2]|0,48,1742296,(t=i,i=i+8|0,c[t>>2]=c[v>>2],t)|0);i=t}else{io(e,y&65535,0)|0;}}while(w<<16>>16!=-1)}}while(0);il(e,q,l,0)|0;d=n+1|0;if(d>>>0>=(c[m>>2]|0)>>>0){k=0;r=8586;break}n=d;o=c[f>>2]|0}if((r|0)==8589){i=g;return k|0}else if((r|0)==8586){i=g;return k|0}else if((r|0)==8593){i=g;return k|0}else if((r|0)==8587){i=g;return k|0}else if((r|0)==8588){i=g;return k|0}return 0}function jV(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=a+4|0;g=c[f>>2]|0;h=c[g+24>>2]|0;if((h|0)==0){j=0;i=e;return j|0}if((c[a+108>>2]|0)!=0){j=0;i=e;return j|0}a=c[(c[(c[g+28>>2]|0)+(h-1<<2)>>2]|0)+12>>2]|0;h=iD(b)|0;iL(b)|0;g=iM(b)|0;k=N2(g,(g|0)<0|0?-1:0,12,0)|0;l=NU(k,G,8,0)|0;k=G;m=d+8|0;d=c[m+4>>2]|0;if(k>>>0>d>>>0|k>>>0==d>>>0&l>>>0>(c[m>>2]|0)>>>0){j=-1094995529;i=e;return j|0}if((g|0)<=0){j=0;i=e;return j|0}m=a+96|0;l=a+104|0;if((h|0)==1){h=0;a=0;d=0;while(1){k=iQ(b)|0;n=G;o=iQ(b)|0;p=G;iM(b)|0;do{if((h|0)==0&((o|0)==(-1|0)&(p|0)==(-1|0))){c[m>>2]=k;c[m+4>>2]=n;q=d;r=1}else{s=-1;if(!((h|0)==(a|0)&((p|0)>(s|0)|(p|0)==(s|0)&o>>>0>-1>>>0))){q=1;r=a;break}c[l>>2]=o;c[l+4>>2]=p;q=d;r=a}}while(0);p=h+1|0;if((p|0)<(g|0)){h=p;a=r;d=q}else{t=q;break}}}else{q=0;d=0;r=0;while(1){a=iM(b)|0;h=iM(b)|0;p=h;o=(h|0)<0|0?-1:0;iM(b)|0;do{if((q|0)==0&(h|0)==-1){c[m>>2]=a;c[m+4>>2]=0;u=r;v=1}else{if(!((q|0)==(d|0)&(h|0)>-1)){u=1;v=d;break}c[l>>2]=p;c[l+4>>2]=o;u=r;v=d}}while(0);o=q+1|0;if((o|0)<(g|0)){q=o;d=v;r=u}else{t=u;break}}}if((t|0)==0){j=0;i=e;return j|0}AN(c[f>>2]|0,24,1611840,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;j=0;i=e;return j|0}function jW(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;a=c[f+24>>2]|0;do{if((a|0)!=0){d=c[(c[f+28>>2]|0)+(a-1<<2)>>2]|0;if(((iK(b)|0)&255|0)!=1){break}g=(c[d+8>>2]|0)+48|0;d=c[g>>2]|0;if((d|0)==65545){c[g>>2]=65544;break}else if((d|0)==65558){c[g>>2]=65559;break}else if((d|0)==65556){c[g>>2]=65557;break}else if((d|0)==65549){c[g>>2]=65548;break}else{break}}}while(0);i=e;return 0}function jX(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;a=c[f+24>>2]|0;if((a|0)==0){g=0;i=e;return g|0}h=c[(c[f+28>>2]|0)+(a-1<<2)>>2]|0;a=d+8|0;d=c[a+4>>2]|0;f=0;if((d|0)<(f|0)|(d|0)==(f|0)&(c[a>>2]|0)>>>0<2>>>0){g=-1094995529;i=e;return g|0}a=iK(b)|0;b=a&65280;do{if((b|0)==512){f=a&255;if((f|0)==9){j=4;break}else if((f|0)==6){j=3;break}else if((f|0)==1){j=2;break}else if((f|0)==14){j=5;break}else{k=8634;break}}else if((b|0)==256){j=1}else{k=8634}}while(0);do{if((k|0)==8634){if((a|0)==0){j=0;break}AN(0,16,1612200,(b=i,i=i+8|0,c[b>>2]=a,b)|0);i=b;j=0}}while(0);c[(c[h+8>>2]|0)+408>>2]=j;g=0;i=e;return g|0}function jY(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0;f=i;i=i+32|0;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=f+16|0;h=f+24|0;NM(h|0,0,5);iE(d,h,4)|0;if((a8(h|0,1619200)|0)!=0){c[b+40>>2]=1}j=b+4|0;AN(c[j>>2]|0,48,1613760,(b=i,i=i+8|0,c[b>>2]=h,b)|0);i=b;zK((c[j>>2]|0)+1140|0,1613104,h,0)|0;h=iM(d)|0;k=f|0;bc(k|0,11,1631952,(b=i,i=i+8|0,c[b>>2]=h,b)|0)|0;i=b;zK((c[j>>2]|0)+1140|0,1612560,k,0)|0;k=e+8|0;e=NU(c[k>>2]|0,c[k+4>>2]|0,-8,0)|0;k=e;if((k|0)<0){l=-1094995529;i=f;return l|0}e=A1(k+1|0)|0;c[g>>2]=e;if((e|0)==0){l=-12;i=f;return l|0}iE(d,e,k)|0;a[e+k|0]=0;zK((c[j>>2]|0)+1140|0,1625632,e,0)|0;A6(g);l=0;i=f;return l|0}function jZ(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0;f=i;i=i+8|0;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=f|0;h=c[b+4>>2]|0;j=c[h+24>>2]|0;if((j|0)==0){k=0;i=f;return k|0}l=c[(c[h+28>>2]|0)+(j-1<<2)>>2]|0;iD(d)|0;iL(d)|0;iI(d)|0;j=iI(d)|0;if((j|0)==1853190003){c[(c[l+8>>2]|0)+8>>2]=1}else if((j|0)==543240557){c[(c[l+8>>2]|0)+48>>2]=86016}else if((j|0)==1701079414){c[(c[l+8>>2]|0)+8>>2]=0}else if((j|0)==1885564003|(j|0)==1885500787){c[(c[l+8>>2]|0)+8>>2]=3}iM(d)|0;iM(d)|0;iM(d)|0;j=e+8|0;e=NU(c[j>>2]|0,c[j+4>>2]|0,-24,0)|0;j=e;if((j|0)<=0){k=0;i=f;return k|0}e=A1(j+1|0)|0;c[g>>2]=e;if((e|0)==0){k=-12;i=f;return k|0}iE(d,e,j)|0;a[e+j|0]=0;d=a[e]|0;if(d<<24>>24!=0){h=l+88|0;if((c[b+40>>2]|0)==0){m=(d<<24>>24|0)==(j-1|0)|0}else{m=0}j=e+m|0;zK(h,1614200,j,0)|0}A6(g);k=0;i=f;return k|0}function j_(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=a+96|0;c[f>>2]=1;g=jD(a,b,d)|0;c[f>>2]=0;i=e;return g|0}function j$(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;f=i;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=c[b+4>>2]|0;b=c[e>>2]|0;h=e+8|0;e=c[h>>2]|0;j=c[h+4>>2]|0;h=c[g+24>>2]|0;if((h|0)==0){k=0;i=f;return k|0}l=(c[(c[g+28>>2]|0)+(h-1<<2)>>2]|0)+8|0;h=c[l>>2]|0;if((c[h+48>>2]|0)!=89){k=0;i=f;return k|0}g=c[h+100>>2]|0;m=NU(g,(g|0)<0|0?-1:0,e,j)|0;g=G;n=NU(m,g,8,0)|0;o=NU(m,g,24,0)|0;g=G;m=0;p=0;if(g>>>0>m>>>0|g>>>0==m>>>0&o>>>0>2147483647>>>0|(j>>>0>p>>>0|j>>>0==p>>>0&e>>>0>2147483647>>>0)){k=-1094995529;i=f;return k|0}p=A2(c[h+96>>2]|0,o)|0;if((p|0)==0){k=-12;i=f;return k|0}c[(c[l>>2]|0)+96>>2]=p;o=(c[l>>2]|0)+100|0;l=c[o>>2]|0;c[o>>2]=n;n=NU(e,j,8,0)|0;j=p+l|0;y=cV(n|0)|0;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;j=p+(l+4)|0;y=b;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;iE(d,p+(l+8)|0,e)|0;k=0;i=f;return k|0}function j0(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;b=i;e=d;d=i;i=i+16|0;c[d>>2]=c[e>>2];c[d+4>>2]=c[e+4>>2];c[d+8>>2]=c[e+8>>2];c[d+12>>2]=c[e+12>>2];e=d+8|0;if((c[e>>2]|0)==0&(c[e+4>>2]|0)==0){i=b;return 0}c[a+28>>2]=1;i=b;return 0}function j1(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;e=i;i=i+48|0;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=e+32|0;d=e+40|0;c[d>>2]=0;g=a+4|0;a=c[g>>2]|0;h=c[a+24>>2]|0;if((h|0)==0){j=0;i=e;return j|0}k=c[(c[a+28>>2]|0)+(h-1<<2)>>2]|0;h=c[k+12>>2]|0;a=iD(b)|0;if((a|0)>1){AS(c[g>>2]|0,1615592,(g=i,i=i+8|0,c[g>>2]=a,g)|0);i=g;j=-1163346256;i=e;return j|0}iL(b)|0;g=(a|0)==1;if(g){a=iQ(b)|0;l=G;iQ(b)|0;m=l;n=a}else{a=iM(b)|0;l=0;iM(b)|0;m=l;n=a}a=k+88|0;l=e|0;do{if(!((n|0)==0&(m|0)==0)){o=0;if((m|0)>(o|0)|(m|0)==(o|0)&n>>>0>2082844799>>>0){o=NU(n,m,-2082844800,0)|0;p=o}else{p=n}c[f>>2]=p;o=bz(f|0)|0;if((o|0)==0){break}bL(l|0,32,1616352,o|0)|0;zK(a,1616032,l,0)|0}}while(0);c[h+92>>2]=iM(b)|0;if(g){g=iQ(b)|0;q=G;r=g}else{q=0;r=iM(b)|0}g=k+56|0;c[g>>2]=r;c[g+4>>2]=q;q=d;if((ns(iK(b)|0,q)|0)!=0){zK(a,1614976,q,0)|0}iK(b)|0;j=0;i=e;return j|0}function j2(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=d+8|0;g=c[f+4>>2]|0;h=c[f>>2]|0;while(1){j=0;if(!((g|0)>(j|0)|(g|0)==(j|0)&h>>>0>8>>>0)){k=0;l=8711;break}j=iI(b)|0;m=NU(h,g,-4,-1)|0;n=G;c[f>>2]=m;c[f+4>>2]=n;if((j|0)==1919706216){break}else{g=n;h=m}}if((l|0)==8711){i=e;return k|0}il(b,-8,-1,1)|0;l=NU(h,g,4,0)|0;c[f>>2]=l;c[f+4>>2]=G;k=jD(a,b,d)|0;i=e;return k|0}function j3(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=il(b,0,0,1)|0;g=NU(f,G,-8,-1)|0;f=a+64|0;c[f>>2]=g;c[f+4>>2]=G;f=jD(a,b,d)|0;i=e;return f|0}function j4(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=jD(a,b,d)|0;if((f|0)<0){g=f}else{c[a+24>>2]=1;g=0}i=e;return g|0}function j5(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=i;i=i+40|0;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=e+32|0;d=iD(b)|0;iL(b)|0;g=(d|0)==1;if(g){d=iQ(b)|0;h=G;iQ(b)|0;j=h;k=d}else{d=iM(b)|0;h=0;iM(b)|0;j=h;k=d}d=a+4|0;h=(c[d>>2]|0)+1140|0;l=e|0;do{if(!((k|0)==0&(j|0)==0)){m=0;if((j|0)>(m|0)|(j|0)==(m|0)&k>>>0>2082844799>>>0){m=NU(k,j,-2082844800,0)|0;n=m}else{n=k}c[f>>2]=n;m=bz(f|0)|0;if((m|0)==0){break}bL(l|0,32,1616352,m|0)|0;zK(h,1616032,l,0)|0}}while(0);l=a+8|0;c[l>>2]=iM(b)|0;if(g){g=iQ(b)|0;o=G;p=g}else{o=0;p=iM(b)|0}g=a+16|0;c[g>>2]=p;c[g+4>>2]=o;g=c[l>>2]|0;do{if((g|0)>0){if((c[a+88>>2]|0)!=0){break}l=AW(p,o,1e6,0,g,(g|0)<0|0?-1:0)|0;h=(c[d>>2]|0)+1064|0;c[h>>2]=l;c[h+4>>2]=G}}while(0);iM(b)|0;iK(b)|0;io(b,10,0)|0;io(b,36,0)|0;iM(b)|0;iM(b)|0;iM(b)|0;iM(b)|0;iM(b)|0;iM(b)|0;iM(b)|0;i=e;return 0}function j6(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;f=i;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=c[b+4>>2]|0;b=c[e>>2]|0;h=e+8|0;e=c[h>>2]|0;j=c[h+4>>2]|0;h=c[g+24>>2]|0;if((h|0)==0){k=0;i=f;return k|0}l=(c[(c[g+28>>2]|0)+(h-1<<2)>>2]|0)+8|0;h=c[l>>2]|0;if((c[h+48>>2]|0)!=24){k=0;i=f;return k|0}g=c[h+100>>2]|0;m=NU(g,(g|0)<0|0?-1:0,e,j)|0;g=G;n=NU(m,g,8,0)|0;o=NU(m,g,24,0)|0;g=G;m=0;p=0;if(g>>>0>m>>>0|g>>>0==m>>>0&o>>>0>2147483647>>>0|(j>>>0>p>>>0|j>>>0==p>>>0&e>>>0>2147483647>>>0)){k=-1094995529;i=f;return k|0}p=A2(c[h+96>>2]|0,o)|0;if((p|0)==0){k=-12;i=f;return k|0}c[(c[l>>2]|0)+96>>2]=p;o=(c[l>>2]|0)+100|0;l=c[o>>2]|0;c[o>>2]=n;n=NU(e,j,8,0)|0;j=p+l|0;y=cV(n|0)|0;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;j=p+(l+4)|0;y=b;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;iE(d,p+(l+8)|0,e)|0;k=0;i=f;return k|0}function j7(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;f=i;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=c[b+4>>2]|0;b=c[e>>2]|0;h=e+8|0;e=c[h>>2]|0;j=c[h+4>>2]|0;h=c[g+24>>2]|0;if((h|0)==0){k=0;i=f;return k|0}l=(c[(c[g+28>>2]|0)+(h-1<<2)>>2]|0)+8|0;h=c[l>>2]|0;if((c[h+48>>2]|0)!=86032){k=0;i=f;return k|0}g=c[h+100>>2]|0;m=NU(g,(g|0)<0|0?-1:0,e,j)|0;g=G;n=NU(m,g,8,0)|0;o=NU(m,g,24,0)|0;g=G;m=0;p=0;if(g>>>0>m>>>0|g>>>0==m>>>0&o>>>0>2147483647>>>0|(j>>>0>p>>>0|j>>>0==p>>>0&e>>>0>2147483647>>>0)){k=-1094995529;i=f;return k|0}p=A2(c[h+96>>2]|0,o)|0;if((p|0)==0){k=-12;i=f;return k|0}c[(c[l>>2]|0)+96>>2]=p;o=(c[l>>2]|0)+100|0;l=c[o>>2]|0;c[o>>2]=n;n=NU(e,j,8,0)|0;j=p+l|0;y=cV(n|0)|0;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;j=p+(l+4)|0;y=b;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;iE(d,p+(l+8)|0,e)|0;k=0;i=f;return k|0}function j8(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=iM(b)|0;d=iM(b)|0;b=c[a+4>>2]|0;a=c[b+24>>2]|0;if((a|0)==0){i=e;return 0}g=c[(c[b+28>>2]|0)+(a-1<<2)>>2]|0;a=g+80|0;h=g+84|0;g=c[h>>2]|0;if((g|0)==1){if((c[a>>2]|0)!=0){j=8757}}else{j=8757}do{if((j|0)==8757){k=c[a>>2]|0;if((d|0)==(g|0)&(f|0)==(k|0)){break}AN(b,24,1616648,(l=i,i=i+32|0,c[l>>2]=k,c[l+8>>2]=g,c[l+16>>2]=f,c[l+24>>2]=d,l)|0);i=l;i=e;return 0}}while(0);if((d|0)==0){i=e;return 0}c[a>>2]=f;c[h>>2]=d;i=e;return 0}function j9(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;a=c[f+24>>2]|0;if((a|0)==0){g=0;i=e;return g|0}d=c[(c[(c[f+28>>2]|0)+(a-1<<2)>>2]|0)+12>>2]|0;iM(b)|0;a=iM(b)|0;if(a>>>0>1073741822){g=-1094995529;i=e;return g|0}f=A1(a<<2)|0;h=d+52|0;c[h>>2]=f;if((f|0)==0){g=-12;i=e;return g|0}f=b+52|0;L11796:do{if((a|0)==0){j=0}else{k=0;while(1){if((c[f>>2]|0)!=0){j=k;break L11796}l=iM(b)|0;c[(c[h>>2]|0)+(k<<2)>>2]=l;l=k+1|0;if(l>>>0<a>>>0){k=l}else{j=l;break}}}}while(0);c[d+48>>2]=j;g=(c[f>>2]|0)==0?0:-541478725;i=e;return g|0}function ka(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;a=c[f+24>>2]|0;if((a|0)==0){g=0;i=e;return g|0}h=d+8|0;d=c[h>>2]|0;j=c[h+4>>2]|0;h=0;if((j|0)<(h|0)|(j|0)==(h|0)&d>>>0<41>>>0){g=0;i=e;return g|0}h=0;if(j>>>0>h>>>0|j>>>0==h>>>0&d>>>0>1073741824>>>0){g=-1094995529;i=e;return g|0}h=(c[(c[f+28>>2]|0)+(a-1<<2)>>2]|0)+8|0;A4(c[(c[h>>2]|0)+96>>2]|0);c[(c[h>>2]|0)+100>>2]=0;a=NU(d,j,-24,0)|0;f=A7(a)|0;c[(c[h>>2]|0)+96>>2]=f;f=c[h>>2]|0;if((c[f+96>>2]|0)==0){g=-12;i=e;return g|0}a=NU(d,j,-40,0)|0;j=a;c[f+100>>2]=j;io(b,40,0)|0;iE(b,c[(c[h>>2]|0)+96>>2]|0,j)|0;g=0;i=e;return g|0}function kb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;a=c[f+24>>2]|0;if((a|0)==0){g=0;i=e;return g|0}d=c[(c[(c[f+28>>2]|0)+(a-1<<2)>>2]|0)+12>>2]|0;iD(b)|0;iL(b)|0;a=iM(b)|0;if((a|0)==0){g=0;i=e;return g|0}if(a>>>0>357913940){g=-1094995529;i=e;return g|0}f=A1(a*12|0)|0;h=d+44|0;c[h>>2]=f;if((f|0)==0){g=-12;i=e;return g|0}f=b+52|0;j=0;while(1){if((c[f>>2]|0)!=0){k=j;break}l=iM(b)|0;c[(c[h>>2]|0)+(j*12|0)>>2]=l;l=iM(b)|0;c[(c[h>>2]|0)+(j*12|0)+4>>2]=l;l=iM(b)|0;c[(c[h>>2]|0)+(j*12|0)+8>>2]=l;l=j+1|0;if(l>>>0<a>>>0){j=l}else{k=l;break}}c[d+40>>2]=k;g=(c[f>>2]|0)==0?0:-541478725;i=e;return g|0}function kc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];iD(b)|0;iL(b)|0;f=jA(a,b,iM(b)|0)|0;i=e;return f|0}function kd(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;a=c[f+24>>2]|0;if((a|0)==0){g=0;i=e;return g|0}d=c[(c[(c[f+28>>2]|0)+(a-1<<2)>>2]|0)+12>>2]|0;iD(b)|0;iL(b)|0;a=iM(b)|0;if((a|0)==0){c[d+80>>2]=1;g=0;i=e;return g|0}if(a>>>0>1073741822){g=-1094995529;i=e;return g|0}f=A1(a<<2)|0;h=d+88|0;c[h>>2]=f;if((f|0)==0){g=-12;i=e;return g|0}f=b+52|0;j=0;while(1){if((c[f>>2]|0)!=0){k=j;break}l=iM(b)|0;c[(c[h>>2]|0)+(j<<2)>>2]=l;l=j+1|0;if(l>>>0<a>>>0){j=l}else{k=l;break}}c[d+84>>2]=k;g=(c[f>>2]|0)==0?0:-541478725;i=e;return g|0}function ke(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;f=i;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=a+4|0;a=c[g>>2]|0;h=c[a+24>>2]|0;if((h|0)==0){j=0;i=f;return j|0}k=c[(c[(c[a+28>>2]|0)+(h-1<<2)>>2]|0)+12>>2]|0;iD(b)|0;iL(b)|0;do{if((c[e>>2]|0)==2054386803){h=iM(b)|0;a=k+64|0;if((c[a>>2]|0)==0){c[a>>2]=h}c[k+68>>2]=h;a=iM(b)|0;l=k+72|0;c[l>>2]=a;if((h|0)==0){m=l;n=a;o=32;break}else{j=0}i=f;return j|0}else{iL(b)|0;a=iD(b)|0;l=iM(b)|0;h=k+72|0;c[h>>2]=l;if((a|0)==32|(a|0)==16|(a|0)==8|(a|0)==4){m=h;n=l;o=a;break}AN(c[g>>2]|0,16,1616968,(l=i,i=i+8|0,c[l>>2]=a,l)|0);i=l;j=-1094995529;i=f;return j|0}}while(0);if((n|0)==0){j=0;i=f;return j|0}if(n>>>0>1073741822){j=-1094995529;i=f;return j|0}if(n>>>0>=(4294967291/(o>>>0)|0)>>>0){j=-1094995529;i=f;return j|0}g=A1(n<<2)|0;e=k+76|0;l=e;c[l>>2]=g;if((g|0)==0){j=-12;i=f;return j|0}g=((ac(n,o)|0)+4|0)>>>3;a=A1(g+16|0)|0;if((a|0)==0){A6(e);j=-12;i=f;return j|0}if((iE(b,a,g)|0)>>>0<g>>>0){A6(e);A4(a);j=-1094995529;i=f;return j|0}e=g<<3;g=e>>>0>2147483639;h=g?0:a;p=g?8:e+8|0;e=b+52|0;b=(o|0)==0;g=k+1208|0;k=(o|0)<26;q=32-o|0;r=o-16|0;s=48-o|0;t=0;u=0;while(1){if((c[e>>2]|0)!=0){v=u;break}do{if(b){w=0;x=t}else{y=h+(t>>>3)|0;z=(cV(d[y]|d[y+1|0]<<8|d[y+2|0]<<16|d[y+3|0]<<24|0)|0)<<(t&7);if(k){y=t+o|0;w=z>>>(q>>>0);x=p>>>0>y>>>0?y:p;break}else{y=t+16|0;A=p>>>0>y>>>0?y:p;y=h+(A>>>3)|0;B=A+r|0;w=(cV(d[y]|d[y+1|0]<<8|d[y+2|0]<<16|d[y+3|0]<<24|0)|0)<<(A&7)>>>(s>>>0)|z>>>16<<r;x=p>>>0>B>>>0?B:p;break}}}while(0);c[(c[l>>2]|0)+(u<<2)>>2]=w;B=c[(c[l>>2]|0)+(u<<2)>>2]|0;z=NU(c[g>>2]|0,c[g+4>>2]|0,B,(B|0)<0|0?-1:0)|0;c[g>>2]=z;c[g+4>>2]=G;z=u+1|0;if(z>>>0<n>>>0){t=x;u=z}else{v=z;break}}c[m>>2]=v;if((c[e>>2]|0)!=0){j=-541478725;i=f;return j|0}A4(a);j=0;i=f;return j|0}function kf(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=a+4|0;a=c[f>>2]|0;d=c[a+24>>2]|0;if((d|0)==0){g=0;i=e;return g|0}h=c[(c[a+28>>2]|0)+(d-1<<2)>>2]|0;d=c[h+12>>2]|0;iD(b)|0;iL(b)|0;a=iM(b)|0;if(a>>>0>536870910){g=-1;i=e;return g|0}j=A1(a<<3)|0;k=d+28|0;c[k>>2]=j;if((j|0)==0){g=-12;i=e;return g|0}j=b+52|0;L11912:do{if((a|0)==0){l=0;m=0;n=0;o=0;p=0}else{q=0;r=0;s=0;t=0;u=0;while(1){if((c[j>>2]|0)!=0){l=q;m=r;n=s;o=t;p=u;break L11912}v=iM(b)|0;w=iM(b)|0;if((w|0)<0){AN(c[f>>2]|0,16,1617328,(x=i,i=i+8|0,c[x>>2]=w,x)|0);i=x;y=1}else{y=w}c[(c[k>>2]|0)+(q<<3)>>2]=v;c[(c[k>>2]|0)+(q<<3)+4>>2]=y;w=v;x=(v|0)<0|0?-1:0;v=N2(y,(y|0)<0|0?-1:0,w,x)|0;z=NU(v,G,s,r)|0;v=G;A=NU(w,x,u,t)|0;x=G;w=q+1|0;if(w>>>0<a>>>0){q=w;r=v;s=z;t=x;u=A}else{l=w;m=v;n=z;o=x;p=A;break}}}}while(0);c[d+24>>2]=l;if((c[j>>2]|0)!=0){g=-541478725;i=e;return g|0}j=h+64|0;c[j>>2]=p;c[j+4>>2]=o;if(!((n|0)==0&(m|0)==0)){o=h+56|0;c[o>>2]=n;c[o+4>>2]=m}o=d+1224|0;c[o>>2]=n;c[o+4>>2]=m;g=0;i=e;return g|0}function kg(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;e=i;i=i+8|0;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=e|0;d=c[a+4>>2]|0;a=c[d+24>>2]|0;if((a|0)==0){i=e;return 0}g=c[(c[d+28>>2]|0)+(a-1<<2)>>2]|0;a=c[g+12>>2]|0;d=iD(b)|0;iL(b)|0;h=(d|0)==1;if(h){iQ(b)|0;iQ(b)|0;}else{iM(b)|0;iM(b)|0}c[g+4>>2]=iM(b)|0;iM(b)|0;if(h){iQ(b)|0;}else{iM(b)|0}iM(b)|0;iM(b)|0;iK(b)|0;iK(b)|0;iK(b)|0;iK(b)|0;h=iM(b)|0;d=iM(b)|0;iM(b)|0;j=iM(b)|0;k=iM(b)|0;iM(b)|0;l=iM(b)|0;m=iM(b)|0;iM(b)|0;n=iM(b)|0;o=iM(b)|0;c[a+164>>2]=n>>16;c[a+168>>2]=o>>16;if((j|0)==-65536&(d|0)==65536){a=g+88|0;zK(a,1619120,1618712,0)|0}if((h|0)==-65536&(k|0)==-65536){a=g+88|0;zK(a,1619120,1618320,0)|0}if((j|0)==65536&(d|0)==-65536){a=g+88|0;zK(a,1619120,1617888,0)|0}if((n|0)==0|(o|0)==0){i=e;return 0}if(!(((k|0)!=65536|(h|0)!=65536)&(d|0)==0&(j|0)==0&(l|0)==0&(m|0)==0)){i=e;return 0}m=N2(h,(h|0)<0|0?-1:0,n,(n|0)<0|0?-1:0)|0;h=G;l=N2(k,(k|0)<0|0?-1:0,o,(o|0)<0|0?-1:0)|0;B_(f,+(o|0)*(+(m>>>0)+ +(h|0)*4294967296.0)/(+(n|0)*(+(l>>>0)+ +(G|0)*4294967296.0)),2147483647);l=g+80|0;g=c[f+4>>2]|0;c[l>>2]=c[f>>2];c[l+4>>2]=g;i=e;return 0}function kh(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];iD(b)|0;f=iL(b)|0;d=iM(b)|0;if((d|0)==0){g=-1094995529;i=e;return g|0}c[a+48>>2]=d;h=c[a+92>>2]|0;j=a+88|0;k=0;while(1){if(k>>>0>=h>>>0){break}l=c[j>>2]|0;m=l+(k*20|0)|0;if((c[m>>2]|0)==(d|0)){n=8898;break}else{k=k+1|0}}do{if((n|0)==8898){if((m|0)==0){break}if((f&1|0)==0){d=a+64|0;o=c[d+4>>2]|0;p=c[d>>2]|0}else{d=iQ(b)|0;o=G;p=d}d=a+56|0;c[d>>2]=p;c[d+4>>2]=o;if((f&2|0)==0){q=c[l+(k*20|0)+4>>2]|0}else{q=iM(b)|0}c[a+72>>2]=q;if((f&8|0)==0){r=c[l+(k*20|0)+8>>2]|0}else{r=iM(b)|0}c[a+76>>2]=r;if((f&16|0)==0){s=c[l+(k*20|0)+12>>2]|0}else{s=iM(b)|0}c[a+80>>2]=s;if((f&32|0)==0){t=c[l+(k*20|0)+16>>2]|0}else{t=iM(b)|0}c[a+84>>2]=t;g=0;i=e;return g|0}}while(0);AN(c[a+4>>2]|0,16,1619784,(a=i,i=i+1|0,i=i+7&-8,c[a>>2]=0,a)|0);i=a;g=-1094995529;i=e;return g|0}function ki(d,e,f){d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0;g=i;i=i+1032|0;h=f;f=i;i=i+16|0;c[f>>2]=c[h>>2];c[f+4>>2]=c[h+4>>2];c[f+8>>2]=c[h+8>>2];c[f+12>>2]=c[h+12>>2];h=g|0;j=g+1024|0;k=j;l=d+4|0;m=mK(c[l>>2]|0,0)|0;if((m|0)==0){n=-12;i=g;return n|0}c[m+4>>2]=c[(c[l>>2]|0)+24>>2];o=A7(1248)|0;if((o|0)==0){n=-12;i=g;return n|0}p=m+12|0;c[p>>2]=o;q=m+8|0;c[(c[q>>2]|0)+8>>2]=2;r=m|0;s=o+8|0;c[s>>2]=c[r>>2];t=jD(d,e,f)|0;if((t|0)<0){n=t;i=g;return n|0}L11998:do{if((c[o+16>>2]|0)!=0){do{if((c[o+24>>2]|0)!=0){if((c[o+40>>2]|0)==0){break}if((c[o+64>>2]|0)!=0){break L11998}if((c[o+72>>2]|0)!=0){break L11998}}}while(0);AN(c[l>>2]|0,16,1625464,(u=i,i=i+8|0,c[u>>2]=c[r>>2],u)|0);i=u;n=0;i=g;return n|0}}while(0);t=o+92|0;f=c[t>>2]|0;do{if((f|0)<1){AN(c[l>>2]|0,24,1706824,(u=i,i=i+8|0,c[u>>2]=c[s>>2],u)|0);i=u;e=c[d+8>>2]|0;c[t>>2]=e;if((e|0)>=1){v=e;break}c[t>>2]=1;v=1}else{v=f}}while(0);mL(m,64,1,v);v=c[p>>2]|0;p=v+96|0;f=c[p>>2]|0;s=c[p+4>>2]|0;e=(f|0)==0&(s|0)==0;if(e){w=v+104|0;if((c[w>>2]|0)==0&(c[w+4>>2]|0)==0){x=0;y=0}else{z=8934}}else{z=8934}do{if((z|0)==8934){w=c[d+8>>2]|0;if((w|0)<=0){x=0;y=0;break}if(e){A=0;B=0}else{C=c[v+92>>2]|0;D=AW(f,s,C,(C|0)<0|0?-1:0,w,(w|0)<0|0?-1:0)|0;w=G;c[p>>2]=D;c[p+4>>2]=w;A=w;B=D}D=v+104|0;w=NV(c[D>>2]|0,c[D+4>>2]|0,B,A)|0;D=G;C=v+112|0;c[C>>2]=w;c[C+4>>2]=D;C=NV(0,0,w,D)|0;D=G;if((c[v+32>>2]|0)==0){x=D;y=C;break}if((c[v+24>>2]|0)==0){x=D;y=C;break}w=c[(c[v+28>>2]|0)+4>>2]|0;if(((c[(c[v+36>>2]|0)+4>>2]|0)/(((w|0)>1?w:1)|0)|0|0)<=16){x=D;y=C;break}c[v+160>>2]=1;c[(c[q>>2]|0)+176>>2]=1;x=D;y=C}}while(0);L12023:do{if((c[(c[q>>2]|0)+8>>2]|0)==1){if((c[v+24>>2]|0)!=1){z=8946;break}if((c[(c[v+28>>2]|0)+4>>2]|0)!=1){z=8946;break}A=v+40|0;B=c[A>>2]|0;L12027:do{if((B|0)==0){E=0}else{p=c[v+44>>2]|0;s=v+16|0;f=B-1|0;e=c[v+128>>2]|0;C=e>>>0>159;D=e>>>0>1;w=(e|0)==0;F=0;H=0;L12029:while(1){I=c[p+(F*12|0)+4>>2]|0;do{if((F|0)==(f|0)){z=9e3}else{if(w){z=9004;break}if(((I>>>0)%(e>>>0)|0|0)==0){z=9e3}else{break L12029}}}while(0);do{if((z|0)==9e3){z=0;if(C){J=(I>>>0)/(e>>>0)|0;break}if(!D){z=9004;break}K=1024-(1024%(e>>>0)|0)|0;J=((I-1+K|0)>>>0)/(K>>>0)|0}}while(0);if((z|0)==9004){z=0;J=(I+1023|0)>>>10}if(F>>>0<f>>>0){K=F+1|0;L=(c[p+(K*12|0)>>2]|0)-(c[p+(F*12|0)>>2]|0)|0;M=K}else{L=(c[s>>2]|0)+1-(c[p+(F*12|0)>>2]|0)|0;M=F+1|0}K=(ac(L,J)|0)+H|0;if(M>>>0<B>>>0){F=M;H=K}else{E=K;break L12027}}AN(c[l>>2]|0,16,1621752,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;break L12023}}while(0);B=m+404|0;H=c[B>>2]|0;if(E>>>0>=(178956970-H|0)>>>0){break}F=m+400|0;p=A2(c[F>>2]|0,(H+E|0)*24|0)|0;if((p|0)==0){break}c[F>>2]=p;c[m+408>>2]=((c[B>>2]|0)+E|0)*24|0;p=v+16|0;H=c[p>>2]|0;if((H|0)==0){break}s=v+20|0;f=v+44|0;e=v+128|0;D=v+124|0;C=v+64|0;w=0;K=0;N=x;O=y;P=H;L12052:while(1){H=(c[s>>2]|0)+(w<<3)|0;Q=K+1|0;if(Q>>>0<(c[A>>2]|0)>>>0){R=c[f>>2]|0;S=(w+1|0)==(c[R+(Q*12|0)>>2]|0)?Q:K;T=R}else{S=K;T=c[f>>2]|0}R=c[T+(S*12|0)+4>>2]|0;if((R|0)==0){U=N;V=O;W=P}else{Q=R;R=N;X=O;Y=c[H+4>>2]|0;Z=c[H>>2]|0;while(1){H=c[e>>2]|0;do{if(H>>>0>159){_=H;$=c[D>>2]|0}else{if(H>>>0>1){aa=1024-(1024%(H>>>0)|0)|0;ab=aa>>>0>Q>>>0?Q:aa;_=ab;$=ac(c[D>>2]|0,(ab>>>0)/(H>>>0)|0)|0;break}else{ab=Q>>>0<1024?Q:1024;_=ab;$=ac(c[C>>2]|0,ab)|0;break}}}while(0);H=c[B>>2]|0;if(H>>>0>=E>>>0){break L12052}c[B>>2]=H+1;I=c[F>>2]|0;ab=I+(H*24|0)|0;c[ab>>2]=Z;c[ab+4>>2]=Y;ab=I+(H*24|0)+8|0;c[ab>>2]=X;c[ab+4>>2]=R;c[I+(H*24|0)+20>>2]=0;c[I+(H*24|0)+16>>2]=$<<2|1;H=NU($,0,Z,Y)|0;I=G;ad=NU(_,0,X,R)|0;ae=G;if((Q|0)==(_|0)){break}else{Q=Q-_|0;R=ae;X=ad;Y=I;Z=H}}U=ae;V=ad;W=c[p>>2]|0}Z=w+1|0;if(Z>>>0<W>>>0){w=Z;K=S;N=U;O=V;P=W}else{break L12023}}AN(c[l>>2]|0,16,1621312,(u=i,i=i+8|0,c[u>>2]=E,u)|0);i=u}else{z=8946}}while(0);L12072:do{if((z|0)==8946){E=v+1236|0;if((c[E>>2]|0)==0){af=0}else{af=(c[v+1240>>2]|0)!=0}W=v+84|0;if((c[W>>2]|0)==0){z=8950}else{if((c[c[v+88>>2]>>2]|0)>0){ag=1}else{z=8950}}do{if((z|0)==8950){if((c[v+48>>2]|0)==0){ag=0;break}ag=(c[c[v+52>>2]>>2]|0)!=0|0}}while(0);V=c[v+172>>2]|0;U=NV(y,x,V,(V|0)<0|0?-1:0)|0;V=G;S=v+72|0;ad=c[S>>2]|0;if((ad|0)==0){break}ae=m+404|0;_=c[ae>>2]|0;if(!((_|0)==0&ad>>>0<(178956970-_|0)>>>0)){break}_=m+400|0;$=A2(c[_>>2]|0,ad*24|0)|0;if(($|0)==0){break}c[_>>2]=$;c[m+408>>2]=((c[S>>2]|0)+(c[ae>>2]|0)|0)*24|0;$=v+16|0;ad=c[$>>2]|0;L12086:do{if((ad|0)==0){ah=0;ai=0}else{T=v+20|0;M=v+44|0;J=v+40|0;L=v+64|0;P=v+68|0;O=v+80|0;N=v+88|0;K=v+1240|0;w=v+76|0;p=v+136|0;F=v+28|0;B=v+24|0;C=v+48|0;D=v+52|0;e=0;f=0;A=0;s=0;Z=0;Y=0;X=0;R=0;Q=0;H=0;I=0;ab=0;aa=V;aj=U;ak=ad;L12088:while(1){al=Z+1|0;am=c[T>>2]|0;if(al>>>0<ak>>>0){an=am+(al<<3)|0;ao=c[an+4>>2]|0;ap=c[an>>2]|0}else{ao=2147483647;ap=-1}an=am+(Z<<3)|0;am=c[an>>2]|0;aq=c[an+4>>2]|0;an=c[J>>2]|0;ar=I;while(1){as=ar+1|0;if(as>>>0>=an>>>0){break}if((al|0)==(c[(c[M>>2]|0)+(as*12|0)>>2]|0)){ar=as}else{break}}do{if((ao|0)>(aq|0)|(ao|0)==(aq|0)&ap>>>0>am>>>0){an=c[L>>2]|0;if((an|0)==0){z=8968;break}as=c[P>>2]|0;if(an>>>0>=as>>>0){at=as;break}an=c[(c[M>>2]|0)+(ar*12|0)+4>>2]|0;au=N2(an,(an|0)<0|0?-1:0,as,0)|0;an=G;av=NV(ap,ao,am,aq)|0;aw=G;if(!((an|0)>(aw|0)|(an|0)==(aw|0)&au>>>0>av>>>0)){z=8968;break}AN(c[l>>2]|0,24,1622864,(u=i,i=i+8|0,c[u>>2]=as,u)|0);i=u;as=c[L>>2]|0;c[P>>2]=as;at=as}else{z=8968}}while(0);if((z|0)==8968){z=0;at=c[P>>2]|0}do{if((at|0)!=0){if(at>>>0>=(c[L>>2]|0)>>>0){break}AN(c[l>>2]|0,24,1622464,(u=i,i=i+8|0,c[u>>2]=at,u)|0);i=u;c[P>>2]=c[L>>2]}}while(0);as=c[M>>2]|0;L12109:do{if((c[as+(ar*12|0)+4>>2]|0)==0){ax=e;ay=f;az=A;aA=s;aB=Y;aC=X;aD=R;aE=Q;aF=H;aG=ab;aH=aa;aI=aj}else{av=1;au=e;aw=f;an=A;aJ=s;aK=Y;aL=X;aM=R;aN=Q;aO=H;aP=ab;aQ=aa;aR=aj;aS=aq;aT=am;aU=as;while(1){if(an>>>0>=(c[S>>2]|0)>>>0){break L12088}do{if((c[O>>2]|0)==0){aV=c[W>>2]|0;if((aV|0)!=0){if((an+ag|0)!=(c[(c[N>>2]|0)+(aO<<2)>>2]|0)){z=8979;break}}aW=aO+1|0;aX=1;aY=aW>>>0<aV>>>0?aW:aO;aZ=aN}else{z=8979}}while(0);do{if((z|0)==8979){z=0;aW=c[C>>2]|0;if((aW|0)==0){aX=0;aY=aO;aZ=aN;break}if((an+ag|0)!=(c[(c[D>>2]|0)+(aN<<2)>>2]|0)){aX=0;aY=aO;aZ=aN;break}aV=aN+1|0;aX=1;aY=aO;aZ=aV>>>0<aW>>>0?aV:aN}}while(0);do{if(af){if(aL>>>0>=(c[E>>2]|0)>>>0){a_=aX;a$=aM;a0=aL;break}aV=c[K>>2]|0;aW=aM+1|0;a1=(aW|0)==(c[aV+(aL<<3)>>2]|0);a_=(c[aV+(aL<<3)+4>>2]|0)==0?aX:1;a$=a1?0:aW;a0=(a1&1)+aL|0}else{a_=aX;a$=aM;a0=aL}}while(0);a1=(a_|0)!=0;aW=a1?0:aK;aV=c[P>>2]|0;if((aV|0)==0){a2=c[(c[w>>2]|0)+(an<<2)>>2]|0}else{a2=aV}aV=c[p>>2]|0;if((aV|0)==-1){z=8989}else{if(((c[aU+(ar*12|0)+8>>2]|0)-1|0)==(aV|0)){z=8989}}if((z|0)==8989){z=0;aV=c[ae>>2]|0;c[ae>>2]=aV+1;a3=c[_>>2]|0;a4=a3+(aV*24|0)|0;c[a4>>2]=aT;c[a4+4>>2]=aS;a4=a3+(aV*24|0)+8|0;c[a4>>2]=aR;c[a4+4>>2]=aQ;c[a3+(aV*24|0)+20>>2]=aW;c[a3+(aV*24|0)+16>>2]=a2<<2|a1&1}a1=a2;aV=0;a3=NU(a1,aV,aT,aS)|0;a4=G;a5=NU(a1,aV,aw,au)|0;aV=G;a1=c[F>>2]|0;a6=c[a1+(aP<<3)+4>>2]|0;a7=NU(a6,(a6|0)<0|0?-1:0,aR,aQ)|0;a6=G;a8=aW+1|0;aW=aJ+1|0;a9=an+1|0;ba=aP+1|0;if(ba>>>0<(c[B>>2]|0)>>>0){bb=(aW|0)==(c[a1+(aP<<3)>>2]|0);bc=bb?ba:aP;bd=bb?0:aW}else{bc=aP;bd=aW}aW=c[M>>2]|0;if(av>>>0>=(c[aW+(ar*12|0)+4>>2]|0)>>>0){ax=aV;ay=a5;az=a9;aA=bd;aB=a8;aC=a0;aD=a$;aE=aZ;aF=aY;aG=bc;aH=a6;aI=a7;break L12109}av=av+1|0;au=aV;aw=a5;an=a9;aJ=bd;aK=a8;aL=a0;aM=a$;aN=aZ;aO=aY;aP=bc;aQ=a6;aR=a7;aS=a4;aT=a3;aU=aW}}}while(0);as=c[$>>2]|0;if(al>>>0<as>>>0){e=ax;f=ay;A=az;s=aA;Z=al;Y=aB;X=aC;R=aD;Q=aE;H=aF;I=ar;ab=aG;aa=aH;aj=aI;ak=as}else{ah=ax;ai=ay;break L12086}}AN(c[l>>2]|0,16,1622080,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;break L12072}}while(0);$=m+56|0;_=c[$>>2]|0;ae=c[$+4>>2]|0;$=0;if(!((ae|0)>($|0)|(ae|0)==($|0)&_>>>0>0>>>0)){break}$=c[v+92>>2]|0;E=N2(ai<<3|0>>>29,ah<<3|ai>>>29,$,($|0)<0|0?-1:0)|0;$=N3(E,G,_,ae)|0;c[(c[q>>2]|0)+72>>2]=$}}while(0);ai=(c[o+152>>2]|0)-1|0;L12144:do{if(ai>>>0<(c[o+144>>2]|0)>>>0){ah=c[o+148>>2]|0;v=ah+(ai*108|0)+4|0;ay=c[v>>2]|0;if((ay|0)==0){z=9045;break}ax=o;aI=c[l>>2]|0;aH=aI+32|0;aG=aI+1160|0;aF=c[d+104>>2]|0;aE=h|0;aD=ah+(ai*108|0)+104|0;aC=b[aD>>1]|0;do{if(aC<<16>>16>0){aB=ah+(ai*108|0)+106|0;aA=b[aB>>1]|0;if(aA<<16>>16<=0){z=9042;break}az=br(aH|0,47)|0;bc=(az|0)==0?aH:az+1|0;az=(aC<<16>>16)-1|0;aY=NL(ay|0)|0;aZ=0;L12150:while(1){be=aY;while(1){bf=be-1|0;if((bf|0)<=-1){z=9036;break L12150}if((a[ay+bf|0]|0)==47){break}else{be=bf}}if((aZ|0)==(az|0)){break}else{aY=bf;aZ=aZ+1|0}}if((z|0)==9036){if((aZ|0)!=(az|0)){break}}aY=bc-aH|0;if(aY>>>0>=1024){break}NN(aE|0,aH|0,aY)|0;a[h+aY|0]=0;if(aA<<16>>16>1){aY=1;do{y$(aE,1624200,1024)|0;aY=aY+1|0;}while((aY|0)<(b[aB>>1]|0));bg=c[v>>2]|0}else{bg=ay}y$(aE,bg+be|0,1024)|0;if((iU(ax,aE,1,aG,0)|0)==0){break L12144}}else{z=9042}}while(0);do{if((z|0)==9042){if((aF|0)==0){break}AN(aI,24,1623584,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;if((iU(ax,c[v>>2]|0,1,aG,0)|0)==0){break L12144}}}while(0);aG=c[v>>2]|0;ax=c[ah+(ai*108|0)+8>>2]|0;aI=b[ah+(ai*108|0)+106>>1]|0;aF=b[aD>>1]|0;AN(c[l>>2]|0,16,1624696,(u=i,i=i+56|0,c[u>>2]=c[r>>2],c[u+8>>2]=aG,c[u+16>>2]=ax,c[u+24>>2]=ah+(ai*108|0)+40,c[u+32>>2]=ah+(ai*108|0)+12,c[u+40>>2]=aI,c[u+48>>2]=aF,u)|0);i=u}else{z=9045}}while(0);if((z|0)==9045){c[o>>2]=c[(c[l>>2]|0)+16>>2];c[o+4>>2]=1}l=c[q>>2]|0;do{if((c[l+8>>2]|0)==0){z=m+80|0;do{if((c[z>>2]|0)==0){u=c[l+120>>2]|0;ai=c[o+164>>2]|0;r=c[l+124>>2]|0;be=c[o+168>>2]|0;if((u|0)==(ai|0)&(r|0)==(be|0)){break}B_(k,+(r|0)*+(ai|0)/(+(u|0)*+(be|0)),2147483647);be=z;u=c[j+4>>2]|0;c[be>>2]=c[j>>2];c[be+4>>2]=u}}while(0);z=m+56|0;ah=c[z>>2]|0;aD=c[z+4>>2]|0;z=0;if((aD|0)>(z|0)|(aD|0)==(z|0)&ah>>>0>0>>>0){z=m+92|0;v=m+96|0;u=c[t>>2]|0;be=m+64|0;ai=N2(u,(u|0)<0|0?-1:0,c[be>>2]|0,c[be+4>>2]|0)|0;be=G;u=2147483647;r=0;BX(z,v,ai,be,ah,aD,u,r)|0}r=c[o+24>>2]|0;if((r|0)==1){bh=c[o+28>>2]|0}else if((r|0)==2){r=c[o+28>>2]|0;if((c[r+8>>2]|0)==1){bh=r}else{break}}else{break}r=m+412|0;u=m+416|0;aD=c[t>>2]|0;ah=aD;be=(aD|0)<0|0?-1:0;aD=c[bh+4>>2]|0;ai=aD;v=(aD|0)<0|0?-1:0;aD=2147483647;z=0;BX(r,u,ah,be,ai,v,aD,z)|0}}while(0);bh=c[q>>2]|0;do{if((c[bh+100>>2]|0)==0){if((c[bh+48>>2]|0)!=28){break}if((c[bh+52>>2]|0)==828601953){break}m$(m)}}while(0);A6(o+20|0);A6(o+44|0);A6(o+76|0);A6(o+88|0);A6(o+28|0);A6(o+52|0);A6(o+1240|0);n=0;i=g;return n|0}function kj(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;a=c[f+24>>2]|0;if((a|0)==0){g=-1094995529;i=e;return g|0}d=c[(c[(c[f+28>>2]|0)+(a-1<<2)>>2]|0)+12>>2]|0;c[d+156>>2]=iM(b)|0;g=0;i=e;return g|0}function kk(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];c[a+100>>2]=iM(b)|0;i=e;return 0}function kl(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=a+92|0;d=c[f>>2]|0;g=NU(d,0,1,0)|0;h=G;j=0;if(h>>>0>j>>>0|h>>>0==j>>>0&g>>>0>214748363>>>0){k=-1094995529;i=e;return k|0}g=a+88|0;j=A2(c[g>>2]|0,(d*20|0)+20|0)|0;if((j|0)==0){k=-12;i=e;return k|0}d=j;j=(c[a+4>>2]|0)+1064|0;c[j>>2]=0;c[j+4>>2]=-2147483648;c[g>>2]=d;g=c[f>>2]|0;c[f>>2]=g+1;iD(b)|0;iL(b)|0;c[d+(g*20|0)>>2]=iM(b)|0;c[d+(g*20|0)+4>>2]=iM(b)|0;c[d+(g*20|0)+8>>2]=iM(b)|0;c[d+(g*20|0)+12>>2]=iM(b)|0;c[d+(g*20|0)+16>>2]=iM(b)|0;k=0;i=e;return k|0}function km(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=a+84|0;d=c[f>>2]|0;g=c[a+4>>2]|0;h=c[g+24>>2]|0;j=g+28|0;k=a+48|0;l=0;while(1){if(l>>>0>=h>>>0){m=9084;break}n=c[(c[j>>2]|0)+(l<<2)>>2]|0;o=c[n+4>>2]|0;if((o|0)==(c[k>>2]|0)){m=9083;break}else{l=l+1|0}}do{if((m|0)==9084){p=c[k>>2]|0}else if((m|0)==9083){if((n|0)==0){p=o;break}l=c[n+12>>2]|0;if(((c[l+136>>2]|0)+1|0)!=(c[a+72>>2]|0)){q=0;i=e;return q|0}iD(b)|0;j=iL(b)|0;h=iM(b)|0;r=l+32|0;s=c[r>>2]|0;do{if((s|0)==0){t=l+72|0;if((c[t>>2]|0)==0){u=0;break}v=A1(8)|0;w=v;if((v|0)==0){q=-12;i=e;return q|0}else{v=l+36|0;c[v>>2]=w;c[w+(c[r>>2]<<3)>>2]=c[t>>2];c[(c[v>>2]|0)+(c[r>>2]<<3)+4>>2]=0;v=(c[r>>2]|0)+1|0;c[r>>2]=v;u=v;break}}else{u=s}}while(0);s=NU(u,0,h,0)|0;v=G;t=0;if(v>>>0>t>>>0|v>>>0==t>>>0&s>>>0>536870910>>>0){q=-1094995529;i=e;return q|0}s=l+36|0;t=A2(c[s>>2]|0,u+h<<3)|0;if((t|0)==0){q=-12;i=e;return q|0}c[s>>2]=t;if((j&1|0)==0){x=0;y=0}else{t=iM(b)|0;x=(t|0)<0|0?-1:0;y=t}if((j&4|0)==0){z=d}else{z=iM(b)|0}t=l+1224|0;v=l+112|0;w=NV(c[t>>2]|0,c[t+4>>2]|0,c[v>>2]|0,c[v+4>>2]|0)|0;A=G;B=a+56|0;C=NU(c[B>>2]|0,c[B+4>>2]|0,y,x)|0;B=G;D=b+52|0;L12236:do{if((h|0)==0){E=B;F=C;H=A;I=w}else{J=a+80|0;K=a+76|0;L=(j&256|0)==0;M=(j&512|0)==0;N=(j&1024|0)==0;O=(j&2048|0)==0;P=l+172|0;Q=n+8|0;R=l+1208|0;S=B;T=C;U=A;V=w;W=0;X=0;Y=0;while(1){if((c[D>>2]|0)!=0){q=-541478725;break}Z=c[J>>2]|0;if((X|0)==0){_=z}else{_=c[f>>2]|0}if(L){$=c[K>>2]|0}else{$=iM(b)|0}if(M){aa=Z}else{aa=iM(b)|0}if(N){ab=_}else{ab=iM(b)|0}c[(c[s>>2]|0)+(c[r>>2]<<3)>>2]=1;if(O){ac=0}else{ac=iM(b)|0}c[(c[s>>2]|0)+(c[r>>2]<<3)+4>>2]=ac;Z=c[r>>2]|0;ad=c[(c[s>>2]|0)+(Z<<3)+4>>2]|0;if((ad|0)<0){ae=c[P>>2]|0;af=-ad|0;c[P>>2]=(ae|0)>(af|0)?ae:af}c[r>>2]=Z+1;do{if((c[(c[Q>>2]|0)+8>>2]|0)==1){ag=1;ah=Y}else{if((Y|0)!=0){ag=0;ah=Y;break}Z=(ab&16842752|0)==0;ag=Z;ah=Z&1}}while(0);Z=ag?0:W;mp(n,T,S,V,U,aa,Z,ag&1)|0;af=NU($,0,V,U)|0;ae=G;ad=aa;ai=0;aj=NU(ad,ai,T,S)|0;ak=G;al=NU(c[R>>2]|0,c[R+4>>2]|0,ad,ai)|0;c[R>>2]=al;c[R+4>>2]=G;al=X+1|0;if(al>>>0<h>>>0){S=ak;T=aj;U=ae;V=af;W=Z+1|0;X=al;Y=ah}else{E=ak;F=aj;H=ae;I=af;break L12236}}i=e;return q|0}}while(0);if((c[D>>2]|0)!=0){q=-541478725;i=e;return q|0}h=a+64|0;c[h>>2]=F;c[h+4>>2]=E;h=NU(c[v>>2]|0,c[v+4>>2]|0,I,H)|0;r=G;c[t>>2]=h;c[t+4>>2]=r;s=n+56|0;c[s>>2]=h;c[s+4>>2]=r;q=0;i=e;return q|0}}while(0);AN(g,16,1626240,(g=i,i=i+8|0,c[g>>2]=p,g)|0);i=g;q=-1094995529;i=e;return q|0}function kn(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;g=c[f+24>>2]|0;if((g|0)==0){h=0;i=e;return h|0}j=d+8|0;k=c[j>>2]|0;l=c[j+4>>2]|0;j=0;if(l>>>0>j>>>0|l>>>0==j>>>0&k>>>0>1073741824>>>0){h=-1094995529;i=e;return h|0}j=(c[(c[f+28>>2]|0)+(g-1<<2)>>2]|0)+8|0;g=c[j>>2]|0;f=c[g+48>>2]|0;do{if((f|0)==86035|(f|0)==86067|(f|0)==86052){A4(c[g+96>>2]|0);c[(c[j>>2]|0)+100>>2]=0;m=NU(k,l,16,0)|0;n=A7(m)|0;c[(c[j>>2]|0)+96>>2]=n;n=c[j>>2]|0;if((c[n+96>>2]|0)==0){h=-12;i=e;return h|0}else{m=k;c[n+100>>2]=m;n=c[(c[j>>2]|0)+96>>2]|0;iE(b,n,m)|0;break}}else{m=0;if(!((l|0)>(m|0)|(l|0)==(m|0)&k>>>0>8>>>0)){io(b,k,l)|0;break}m=jD(a,b,d)|0;if((m|0)<0){h=m}else{break}i=e;return h|0}}while(0);h=0;i=e;return h|0}function ko(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];jz(c[a+4>>2]|0,b,d)|0;i=e;return 0}function kp(a,b,d){a=a|0;b=b|0;d=d|0;var f=0,g=0,h=0,j=0;f=i;i=i+32|0;g=d;d=i;i=i+16|0;c[d>>2]=c[g>>2];c[d+4>>2]=c[g+4>>2];c[d+8>>2]=c[g+8>>2];c[d+12>>2]=c[g+12>>2];g=f|0;d=c[a+4>>2]|0;a=c[d+24>>2]|0;if((a|0)==0){i=f;return 0}h=c[(c[d+28>>2]|0)+(a-1<<2)>>2]|0;a=iL(b)|0;b=a>>>14&7;d=a>>>11&7;j=a>>>10&1;c[g>>2]=2;c[g+4>>2]=1;c[g+8>>2]=2;c[g+12>>2]=3;c[g+16>>2]=3;c[g+20>>2]=4;c[g+24>>2]=4;c[g+28>>2]=5;a=h+8|0;c[(c[a>>2]|0)+416>>2]=j+(c[g+(d<<2)>>2]|0);g=(c[a>>2]|0)+448|0;c[g>>2]=e[337760+(d<<1)>>1]|0;c[g+4>>2]=0;if((j|0)!=0){j=(c[a>>2]|0)+448|0;g=c[j+4>>2]|0;c[j>>2]=c[j>>2]|8;c[j+4>>2]=g}c[(c[a>>2]|0)+464>>2]=b;g=c[a>>2]|0;if(!((c[g+416>>2]|0)>1&(b|0)==7)){i=f;return 0}c[g+464>>2]=8;i=f;return 0}function kq(a,b,d){a=a|0;b=b|0;d=d|0;var f=0,g=0;f=i;g=d;d=i;i=i+16|0;c[d>>2]=c[g>>2];c[d+4>>2]=c[g+4>>2];c[d+8>>2]=c[g+8>>2];c[d+12>>2]=c[g+12>>2];g=c[a+4>>2]|0;a=c[g+24>>2]|0;if((a|0)==0){i=f;return 0}d=c[(c[g+28>>2]|0)+(a-1<<2)>>2]|0;iK(b)|0;a=iL(b)|0;b=a>>>12&31;g=d+8|0;d=(c[g>>2]|0)+448|0;c[d>>2]=e[337760+((a>>>9&7)<<1)>>1]|0;c[d+4>>2]=0;if((a&256|0)!=0){a=(c[g>>2]|0)+448|0;d=c[a+4>>2]|0;c[a>>2]=c[a>>2]|8;c[a+4>>2]=d}d=(c[g>>2]|0)+448|0;a=zs(c[d>>2]|0,c[d+4>>2]|0)|0;c[(c[g>>2]|0)+416>>2]=a;c[(c[g>>2]|0)+464>>2]=b;a=c[g>>2]|0;if(!((c[a+416>>2]|0)>1&(b|0)==7)){i=f;return 0}c[a+464>>2]=8;i=f;return 0}function kr(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=d+8|0;g=c[f>>2]|0;h=c[f+4>>2]|0;j=0;if((h|0)<(j|0)|(h|0)==(j|0)&g>>>0<8>>>0){i=e;return 0}if((iM(b)|0)!=0){j=NU(g,h,-4,-1)|0;k=G;io(b,j,k)|0;i=e;return 0}k=iI(b)|0;c[d>>2]=k;d=NU(g,h,-8,-1)|0;h=G;c[f>>2]=d;c[f+4>>2]=h;if((k|0)!=1952539757){io(b,d,h)|0;i=e;return 0}if((d|0)==0&(h|0)==0){i=e;return 0}c[a+28>>2]=1;i=e;return 0}function ks(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=a+4|0;a=c[f>>2]|0;g=c[a+24>>2]|0;if((g|0)==0){i=e;return 0}if((l3(b,c[(c[(c[a+28>>2]|0)+(g-1<<2)>>2]|0)+8>>2]|0,c[d+8>>2]|0)|0)>=0){i=e;return 0}AN(c[f>>2]|0,24,1627048,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;i=e;return 0}function kt(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;e=i;i=i+136|0;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=e|0;g=e+128|0;iM(b)|0;if((iI(b)|0)!=1836016484){h=-1094995529;i=e;return h|0}if((iI(b)|0)!=1651076218){AN(c[a+4>>2]|0,16,1627560,(j=i,i=i+1|0,i=i+7&-8,c[j>>2]=0,j)|0);i=j;h=-1094995529;i=e;return h|0}iM(b)|0;if((iI(b)|0)!=1685482851){h=-1094995529;i=e;return h|0}j=iM(b)|0;c[g>>2]=j;k=d+8|0;l=NU(c[k>>2]|0,c[k+4>>2]|0,-24,0)|0;m=l;l=A1(m)|0;if((l|0)==0){h=-12;i=e;return h|0}n=A1(j)|0;if((n|0)==0){A4(l);h=-12;i=e;return h|0}iE(b,l,m)|0;do{if((bm(n|0,g|0,l|0,m|0)|0)==0){if((ie(f,n,c[g>>2]|0,0,0,0,0,0)|0)!=0){o=-1;break}c[d>>2]=1987014509;b=c[g>>2]|0;c[k>>2]=b;c[k+4>>2]=(b|0)<0|0?-1:0;o=jD(a,f,d)|0}else{o=-1}}while(0);A4(n);A4(l);h=o;i=e;return h|0}function ku(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=a+4|0;a=c[f>>2]|0;g=c[a+24>>2]|0;if((g|0)==0){i=e;return 0}h=d+8|0;d=c[h>>2]|0;j=c[h+4>>2]|0;h=0;if((j|0)<(h|0)|(j|0)==(h|0)&d>>>0<16>>>0){i=e;return 0}h=c[(c[a+28>>2]|0)+(g-1<<2)>>2]|0;io(b,4,0)|0;g=c[f>>2]|0;f=NU(d,j,-4,-1)|0;kz(g,b,h,f,G)|0;i=e;return 0}function kv(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;a=c[f+24>>2]|0;if((a|0)==0){g=0;i=e;return g|0}h=c[(c[f+28>>2]|0)+(a-1<<2)>>2]|0;a=d+8|0;d=c[a>>2]|0;f=c[a+4>>2]|0;a=NU(d,f,-7,-1)|0;j=G;k=0;if(j>>>0>k>>>0|j>>>0==k>>>0&a>>>0>268435448>>>0){g=-1094995529;i=e;return g|0}if(((iD(b)|0)&240|0)!=192){g=0;i=e;return g|0}a=h+8|0;A4(c[(c[a>>2]|0)+96>>2]|0);c[(c[a>>2]|0)+100>>2]=0;h=NU(d,f,9,0)|0;k=A7(h)|0;c[(c[a>>2]|0)+96>>2]=k;k=c[a>>2]|0;if((c[k+96>>2]|0)==0){g=-12;i=e;return g|0}h=NU(d,f,-7,0)|0;c[k+100>>2]=h;il(b,6,0,1)|0;h=c[a>>2]|0;iE(b,c[h+96>>2]|0,c[h+100>>2]|0)|0;g=0;i=e;return g|0}function kw(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0;e=i;f=d;d=i;i=i+16|0;c[d>>2]=c[f>>2];c[d+4>>2]=c[f+4>>2];c[d+8>>2]=c[f+8>>2];c[d+12>>2]=c[f+12>>2];f=c[a+4>>2]|0;a=c[f+24>>2]|0;if((a|0)==0){g=0;i=e;return g|0}d=c[(c[(c[f+28>>2]|0)+(a-1<<2)>>2]|0)+12>>2]|0;a=iD(b)|0;iL(b)|0;if((iI(b)|0)!=544235890){g=0;i=e;return g|0}if((a&255|0)==1){iM(b)|0}a=iM(b)|0;if((a|0)==0){g=0;i=e;return g|0}if(a>>>0>536870910){g=-1094995529;i=e;return g|0}f=A1(a<<3)|0;h=d+1240|0;c[h>>2]=f;if((f|0)==0){g=-12;i=e;return g|0}f=b+52|0;j=0;while(1){if((c[f>>2]|0)!=0){k=j;break}l=iM(b)|0;c[(c[h>>2]|0)+(j<<3)>>2]=l;l=iM(b)|0;c[(c[h>>2]|0)+(j<<3)+4>>2]=l;l=j+1|0;if(l>>>0<a>>>0){j=l}else{k=l;break}}c[d+1236>>2]=k;g=(c[f>>2]|0)!=0?-541478725:0;i=e;return g|0}function kx(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;f=i;i=i+24|0;g=e;e=i;i=i+16|0;c[e>>2]=c[g>>2];c[e+4>>2]=c[g+4>>2];c[e+8>>2]=c[g+8>>2];c[e+12>>2]=c[g+12>>2];g=f+16|0;h=e+8|0;e=c[h>>2]|0;j=c[h+4>>2]|0;h=0;if((j|0)<(h|0)|(j|0)==(h|0)&e>>>0<16>>>0|(e|0)==(-1|0)&(j|0)==2147483647){k=-1094995529;i=f;return k|0}h=f|0;l=iE(d,h,16)|0;if((l|0)<0){k=l;i=f;return k|0}if((l|0)!=16){k=-1094995529;i=f;return k|0}if((NO(h|0,126200,16)|0)!=0){k=0;i=f;return k|0}h=NU(e,j,-16,0)|0;j=h;if(j>>>0<4){k=-1094995529;i=f;return k|0}io(d,4,0)|0;h=j-4|0;e=A7(j-3|0)|0;if((e|0)==0){k=-12;i=f;return k|0}j=iE(d,e,h)|0;if((j|0)<0){A4(e);k=j;i=f;return k|0}if((j|0)!=(h|0)){A4(e);k=-1094995529;i=f;return k|0}h=yZ(e,1628096)|0;L12433:do{if((h|0)!=0){j=b+124|0;d=b+120|0;l=h;while(1){m=l+15|0;n=(c[j>>2]|0)+1|0;c[j>>2]=n;o=A3(c[d>>2]|0,n,4)|0;c[d>>2]=o;if((o|0)==0){break}o=ch()|0;c[o>>2]=0;n=aU(m|0,g|0,10)|0;do{if((n|0)<0){p=9252}else{if((c[o>>2]|0)!=0){p=9252;break}if((a[c[g>>2]|0]|0)!=34){p=9252;break}c[(c[d>>2]|0)+((c[j>>2]|0)-1<<2)>>2]=n}}while(0);if((p|0)==9252){p=0;c[(c[d>>2]|0)+((c[j>>2]|0)-1<<2)>>2]=0}l=yZ(m,1628096)|0;if((l|0)==0){break L12433}}c[j>>2]=0;A4(e);k=-12;i=f;return k|0}}while(0);A4(e);k=0;i=f;return k|0}function ky(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;f=0;while(1){if((f|0)==15){break}if((c[127520+(f<<3)>>2]|0)==(a|0)){g=9270;break}else{f=f+1|0}}L12453:do{if((g|0)==9270){a=c[127524+(f<<3)>>2]|0;h=zs(b,d)|0;i=(h|0)>9?0:h;h=c[129480+(i<<2)>>2]|0;j=c[a>>2]|0;if((j|0)==0){break}k=h|0;l=0;m=j;L12456:while(1){L12458:do{if((m&65535|0)==(i|0)){j=c[k>>2]|0;if((j|0)==0){break}else{n=0;o=j}while(1){if((o|0)==(m|0)){j=h+(n<<4)+8|0;if((c[j>>2]|0)==(b|0)&(c[j+4>>2]|0)==(d|0)){break}}n=n+1|0;o=c[h+(n<<4)>>2]|0;if((o|0)==0){break L12458}}if((m|0)!=0){p=m;q=0;break L12456}}}while(0);l=l+1|0;m=c[a+(l<<2)>>2]|0;if((m|0)==0){break L12453}}c[e>>2]=q;return p|0}}while(0);o=0;n=((b|0)!=0|(d|0)!=0)&(d>>>0<o>>>0|d>>>0==o>>>0&b>>>0<262144>>>0);p=n?65536:0;q=n?b:0;c[e>>2]=q;return p|0}function kz(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;a=0;if((f|0)<(a|0)|(f|0)==(a|0)&e>>>0<12>>>0){g=-1094995529;return g|0}a=iM(b)|0;h=iM(b)|0;i=iM(b)|0;j=N2(i,0,20,0)|0;k=NU(j,G,12,0)|0;j=G;if(j>>>0>f>>>0|j>>>0==f>>>0&k>>>0>e>>>0){g=0;return g|0}L12477:do{if((i|0)==0){l=f;m=e;n=0;o=9294}else{if((a|0)==0){p=0;q=0;r=f;s=e}else{k=0;j=f;t=e;while(1){iM(b)|0;iM(b)|0;iI(b)|0;iI(b)|0;iI(b)|0;u=NU(t,j,-20,-1)|0;v=G;w=k+1|0;if(w>>>0<i>>>0){k=w;j=v;t=u}else{l=v;m=u;n=0;o=9294;break L12477}}}while(1){t=iM(b)|0;iM(b)|0;iI(b)|0;iI(b)|0;iI(b)|0;j=NU(s,r,-20,-1)|0;k=G;if((t|0)==0){x=k;y=j;break L12477}do{if(t>>>0<19){z=1<<t-1}else{if((t|0)==38){z=536870912;break}else if((t|0)!=39){l=k;m=j;n=0;o=9294;break L12477}z=1073741824}}while(0);t=z|p;u=q+1|0;if(u>>>0<i>>>0){p=t;q=u;r=k;s=j}else{l=k;m=j;n=t;o=9294;break}}}}while(0);do{if((o|0)==9294){if((a|0)==65536){s=h>>>0<262144;A=s?0:0;B=s?h:0}else if((a|0)==0){if((n|0)==0){x=l;y=m;break}s=(c[d+8>>2]|0)+448|0;c[s>>2]=n;c[s+4>>2]=0;x=l;y=m;break}else{s=a&65535;r=c[129480+((s>>>0>9?0:s)<<2)>>2]|0;s=0;while(1){q=c[r+(s<<4)>>2]|0;if((q|0)==0|(q|0)==(a|0)){break}else{s=s+1|0}}q=r+(s<<4)+8|0;A=c[q+4>>2]|0;B=c[q>>2]|0}q=(c[d+8>>2]|0)+448|0;c[q>>2]=B;c[q+4>>2]=A;x=l;y=m}}while(0);m=NU(y,x,-12,-1)|0;io(b,m,G)|0;g=0;return g|0}function kA(b,f){b=b|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;g=i;i=i+16|0;h=g|0;j=g+8|0;k=c[b+12>>2]|0;l=c[b+16>>2]|0;m=f+32|0;n=c[m>>2]|0;o=c[k+48>>2]|0;p=c[o+(n*264|0)+76>>2]|0;q=f+28|0;c[h>>2]=c[q>>2];c[j>>2]=0;r=k+52|0;do{if((c[r>>2]&2|0)==0){s=l}else{if((c[k+84>>2]|0)>0){t=o+(n*264|0)+180|0;u=c[t>>2]|0;if((u|0)!=0){s=u;break}u=iZ(t)|0;if((u|0)<0){v=u;i=g;return v|0}else{s=c[t>>2]|0;break}}else{t=k+104|0;u=c[t>>2]|0;if((u|0)!=0){s=u;break}u=iZ(t)|0;if((u|0)<0){v=u;i=g;return v|0}else{s=c[t>>2]|0;break}}}}while(0);l=p+48|0;t=c[l>>2]|0;do{if((t|0)==73728){u=c[h>>2]|0;if((u|0)<=0){w=0;break}x=c[f+24>>2]|0;y=0;z=0;do{z=(e[247472+(((d[x+z|0]|0)>>>3&15)<<1)>>1]|0)+z|0;y=y+1|0;}while((z|0)<(u|0)&y>>>0<100);if(y>>>0<=1){w=y;break}AN(b,16,1705824,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;v=-1;i=g;return v|0}else if((t|0)==69638|(t|0)==69633){w=c[p+424>>2]|0}else{u=c[o+(n*264|0)+40>>2]|0;if((u|0)==0){w=1;break}w=(c[h>>2]|0)/(u|0)|0}}while(0);u=o+(n*264|0)+80|0;do{if((c[u>>2]|0)==0){z=c[p+100>>2]|0;if((z|0)<=0){B=t;break}c[u>>2]=z;x=A1(z)|0;c[o+(n*264|0)+84>>2]=x;z=c[p+96>>2]|0;C=c[u>>2]|0;NN(x|0,z|0,C)|0;B=c[l>>2]|0}else{B=t}}while(0);do{if((B|0)==86018){if((c[q>>2]|0)<=2){D=9340;break}t=c[f+24>>2]|0;if(((cT((d[t]|d[t+1|0]<<8)<<16>>16|0)|0)&-16)<<16>>16!=-16){D=9340;break}t=(c[(c[b+28>>2]|0)+(c[m>>2]<<2)>>2]|0)+64|0;p=b;if(!((c[t>>2]|0)==0&(c[t+4>>2]|0)==0)){AN(p,24,1700880,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;E=c[l>>2]|0;D=9333;break}AN(p,16,1733872,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;v=-1;i=g;return v|0}else{E=B;D=9333}}while(0);L12542:do{if((D|0)==9333){if((E|0)!=28){D=9340;break}if((c[u>>2]|0)<=0){D=9340;break}if((a[c[o+(n*264|0)+84>>2]|0]|0)==1){D=9340;break}B=c[o+(n*264|0)+120>>2]|0;do{if((B|0)>-1){if((B|0)>=(c[k+16>>2]|0)){break}m=c[f+24>>2]|0;ng(m,j,h)|0;ij(s,c[j>>2]|0,c[h>>2]|0);break L12542}}while(0);c[h>>2]=nf(s,c[f+24>>2]|0,c[q>>2]|0)|0}}while(0);if((D|0)==9340){ij(s,c[f+24>>2]|0,c[h>>2]|0)}E=c[l>>2]|0;do{if((E|0)==100|(E|0)==86019){if((c[u>>2]|0)!=0){break}c[u>>2]=c[h>>2];B=A1(c[h>>2]|0)|0;c[o+(n*264|0)+84>>2]=B;if((B|0)==0){v=-12;i=g;return v|0}else{y=c[f+24>>2]|0;m=c[h>>2]|0;NN(B|0,y|0,m)|0;break}}}while(0);u=o+(n*264|0)+4|0;E=c[u>>2]|0;m=o+(n*264|0)+92|0;do{if(E>>>0<(c[m>>2]|0)>>>0){F=o+(n*264|0)+88|0}else{y=(E<<1)+2048|0;B=o+(n*264|0)+88|0;if((A5(B,y,40)|0)==0){c[m>>2]=y;F=B;break}else{v=-12;i=g;return v|0}}}while(0);m=il(s,0,0,1)|0;E=c[h>>2]|0;B=NV(m,G,E,(E|0)<0|0?-1:0)|0;E=(c[F>>2]|0)+((c[u>>2]|0)*40|0)|0;c[E>>2]=B;c[E+4>>2]=G;c[(c[F>>2]|0)+((c[u>>2]|0)*40|0)+20>>2]=w;c[(c[F>>2]|0)+((c[u>>2]|0)*40|0)+24>>2]=0;c[(c[F>>2]|0)+((c[u>>2]|0)*40|0)+16>>2]=c[h>>2];c[(c[F>>2]|0)+((c[u>>2]|0)*40|0)+28>>2]=w;E=f+16|0;B=c[E+4>>2]|0;m=(c[F>>2]|0)+((c[u>>2]|0)*40|0)+8|0;c[m>>2]=c[E>>2];c[m+4>>2]=B;B=o+(n*264|0)+112|0;do{if((c[u>>2]|0)==0){m=c[B>>2]|0;y=c[B+4>>2]|0;if(!((m|0)==0&(y|0)==(-2147483648|0))){p=o+(n*264|0)+24|0;t=NU(c[p>>2]|0,c[p+4>>2]|0,m,y)|0;y=(c[F>>2]|0)+8|0;c[y>>2]=t;c[y+4>>2]=G;if((c[u>>2]|0)!=0){break}if(!((c[B>>2]|0)==0&(c[B+4>>2]|0)==(-2147483648|0))){break}}y=c[k+108>>2]|0;if((y|0)<0){if((c[r>>2]&2|0)==0){break}}else{if((y|0)>0){break}}c[B>>2]=0;c[B+4>>2]=0;y=(c[F>>2]|0)+8|0;c[y>>2]=0;c[y+4>>2]=0}}while(0);r=c[B>>2]|0;y=c[B+4>>2]|0;if((r|0)==0&(y|0)==(-2147483648|0)){t=c[E>>2]|0;m=c[E+4>>2]|0;c[B>>2]=t;c[B+4>>2]=m;H=m;I=t}else{H=y;I=r}r=NV(c[E>>2]|0,c[E+4>>2]|0,I,H)|0;H=c[f+48>>2]|0;I=NU(r,G,H,(H|0)<0|0?-1:0)|0;H=o+(n*264|0)+24|0;c[H>>2]=I;c[H+4>>2]=G;c[o+(n*264|0)+32>>2]=0;H=f+8|0;I=c[H>>2]|0;r=c[H+4>>2]|0;do{if((I|0)==0&(r|0)==(-2147483648|0)){AN(b,24,1677728,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;y=c[E>>2]|0;t=c[E+4>>2]|0;c[H>>2]=y;c[H+4>>2]=t;J=t;K=y;L=t;M=y}else{y=c[E>>2]|0;t=c[E+4>>2]|0;if((y|0)==(I|0)&(t|0)==(r|0)){J=t;K=y;L=t;M=y;break}y=o+(n*264|0)+52|0;c[y>>2]=c[y>>2]|1;J=c[H+4>>2]|0;K=c[H>>2]|0;L=c[E+4>>2]|0;M=c[E>>2]|0}}while(0);E=NV(K,J,M,L)|0;c[(c[F>>2]|0)+((c[u>>2]|0)*40|0)+32>>2]=E;c[(c[F>>2]|0)+((c[u>>2]|0)*40|0)+36>>2]=0;E=c[l>>2]|0;L12587:do{if((E|0)==71){l=c[k+84>>2]|0;L=c[f+24>>2]|0;M=c[q>>2]|0;J=L+M|0;K=c[f+36>>2]&1;H=J;L12589:do{if((M|0)<4){N=0;O=0}else{r=-1;I=L;while(1){if(I>>>0>=J>>>0){N=0;O=0;break L12589}A=r<<8;if((A|0)==256){break}else{r=d[I]|A;I=I+1|0}}r=I-3|0;if(r>>>0>=J>>>0){N=0;O=0;break}A=o+(n*264|0)+256|0;y=r;r=0;t=0;while(1){m=y;B=t;L12597:while(1){p=m;while(1){C=p+4|0;L12601:do{if((H-C|0)<4){P=J}else{z=C;x=-1;while(1){if(z>>>0>=J>>>0){P=J;break L12601}Q=x<<8;R=d[z]|Q;if((Q|0)==256){break}else{z=z+1|0;x=R}}P=z-3|0}}while(0);C=p;x=cV(d[C]|d[C+1|0]<<8|d[C+2|0]<<16|d[C+3|0]<<24|0)|0;if((x|0)==271){break L12597}else if((x|0)==270){break}else if((x|0)==267){c[A>>2]=1}if(P>>>0<J>>>0){p=P}else{N=B;O=r;break L12589}}if(P>>>0<J>>>0){m=P;B=1}else{N=1;O=r;break L12589}}if(P>>>0<J>>>0){y=P;r=1;t=B}else{N=B;O=1;break}}}}while(0);J=c[u>>2]|0;H=(l|0)==0;do{if((J|l|0)==0){c[o+(n*264|0)+240>>2]=O;c[o+(n*264|0)+244>>2]=N}else{L=(O|0)!=0;if(L){if((c[o+(n*264|0)+248>>2]|0)!=0){D=9384}}else{D=9384}if((D|0)==9384){if((N|0)==0){break}if((c[o+(n*264|0)+252>>2]|0)!=0){break}}if((J|0)>0){M=0;do{t=(c[F>>2]|0)+(M*40|0)+36|0;c[t>>2]=c[t>>2]&-2;M=M+1|0;}while((M|0)<(c[u>>2]|0))}M=o+(n*264|0)+48|0;c[M>>2]=0;if(L){c[o+(n*264|0)+248>>2]=1}t=(N|0)!=0;if(t){c[o+(n*264|0)+252>>2]=1}if(!H){break}if(L){if((c[o+(n*264|0)+240>>2]|0)==0){break}}if(t){if((c[o+(n*264|0)+244>>2]|0)==0){break}}t=(c[F>>2]|0)+36|0;c[t>>2]=c[t>>2]|1;c[M>>2]=1}}while(0);H=(c[o+(n*264|0)+252>>2]|0)==0;do{if((c[o+(n*264|0)+248>>2]|0)==0){S=H?K:N}else{if(H){S=O;break}if((O|0)==0){break L12587}else{S=N}}}while(0);if((S|0)==0){break}H=(c[F>>2]|0)+((c[u>>2]|0)*40|0)+36|0;c[H>>2]=c[H>>2]|1;H=o+(n*264|0)+48|0;c[H>>2]=(c[H>>2]|0)+1}else{if((c[f+36>>2]&1|0)==0){break}H=c[u>>2]|0;K=(c[F>>2]|0)+(H*40|0)+36|0;do{if((c[k+4>>2]|0)==2&(E|0)==2&(H|0)>0){J=(c[q>>2]|0)-4|0;L12651:do{if((J|0)>0){l=c[f+24>>2]|0;M=0;t=0;r=-256;while(1){y=d[l+M|0]|r;if((y|0)==440){T=(d[l+(M+4)|0]|0)>>>6&1}else if((y|0)==256){break}else{T=t}A=M+1|0;if((A|0)<(J|0)){M=A;t=T;r=y<<8}else{break L12651}}if(((d[l+(M+2)|0]|0)>>>6|d[l+(M+1)|0]<<2|0)!=0&(t|0)==0){c[K>>2]=2;break}else{c[K>>2]=1;break}}}while(0);J=c[u>>2]|0;L=c[F>>2]|0;if((c[L+(J*40|0)+36>>2]&2|0)==0){U=J;V=L;break}r=o+(n*264|0)+52|0;c[r>>2]=c[r>>2]|2;U=J;V=L}else{c[K>>2]=1;U=c[u>>2]|0;V=c[F>>2]|0}}while(0);if((c[V+(U*40|0)+36>>2]&1|0)==0){break}K=o+(n*264|0)+48|0;c[K>>2]=(c[K>>2]|0)+1}}while(0);c[u>>2]=(c[u>>2]|0)+1;U=o+(n*264|0)+36|0;c[U>>2]=(c[U>>2]|0)+w;w=c[h>>2]|0;U=k+40|0;V=NU(c[U>>2]|0,c[U+4>>2]|0,w,(w|0)<0|0?-1:0)|0;c[U>>2]=V;c[U+4>>2]=G;ik(s);s=c[o+(n*264|0)+120>>2]|0;do{if((s|0)>-1){if((s|0)>=(c[k+16>>2]|0)){break}n=c[u>>2]|0;o=c[j>>2]|0;U=c[h>>2]|0;kV(b,f,s,n,o,U)|0}}while(0);A4(c[j>>2]|0);v=0;i=g;return v|0}function kB(b){b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0;e=i;i=i+392|0;f=e|0;g=e+152|0;h=e+160|0;j=e+168|0;k=e+192|0;l=e+272|0;m=e+280|0;n=e+288|0;o=e+368|0;p=e+376|0;q=e+384|0;r=b+16|0;s=c[r>>2]|0;t=b+12|0;u=c[t>>2]|0;v=u;w=b+1140|0;x=zJ(c[w>>2]|0,1696528,0,0)|0;z=u+88|0;do{if((c[z>>2]|0)==0){if((c[u+96>>2]|0)!=0){A=9434;break}B=c[u+52>>2]|0;if((B&44|0)==0){C=B}else{A=9434}}else{A=9434}}while(0);if((A|0)==9434){B=u+52|0;D=c[B>>2]|2;c[B>>2]=D;C=D}D=u+52|0;do{if((C&128|0)==0){E=C}else{if((C&2|0)==0){c[u+60>>2]=-1;E=C;break}else{B=C&-129;c[D>>2]=B;E=B;break}}}while(0);C=c[u+108>>2]|0;if((C|0)<0){if((E&2|0)!=0){A=9442}}else{if((C|0)<=0){A=9442}}do{if((A|0)==9442){C=b+1192|0;if((c[C>>2]|0)>=0){break}c[C>>2]=1}}while(0);do{if((c[(c[r>>2]|0)+88>>2]|0)==0){do{if((c[D>>2]&2|0)==0){C=c[b+8>>2]|0;if((C|0)==0){break}if((a8(c[C>>2]|0,1696240)|0)==0){A=9448}}else{A=9448}}while(0);if((A|0)==9448){if((c[u+100>>2]|0)==0){break}}AN(b,16,1696104,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0);i=F;H=-1;i=e;return H|0}}while(0);r=u+4|0;c[r>>2]=1;C=c[b+8>>2]|0;do{if((C|0)==0){E=b+24|0;B=c[E>>2]|0;I=u+16|0;c[I>>2]=B;J=B;K=E;L=1;M=I}else{I=c[C>>2]|0;do{if((a8(1695960,I|0)|0)==0){c[r>>2]=4}else{if((a8(1695808,I|0)|0)==0){c[r>>2]=20;break}if((a8(1661440,I|0)|0)==0){c[r>>2]=2;break}if((a8(1695640,I|0)|0)==0){c[r>>2]=8;break}if((a8(1695464,I|0)|0)==0){c[r>>2]=32;break}if((a8(1696240,I|0)|0)==0){c[r>>2]=64;break}if((a8(1695064,I|0)|0)!=0){break}c[r>>2]=128}}while(0);I=c[t>>2]|0;E=il(s,0,0,1)|0;B=G;N=b+24|0;O=c[N>>2]|0;if((O|0)==0){P=0;Q=0}else{R=c[b+28>>2]|0;S=0;T=0;U=0;while(1){V=c[(c[R+(T<<2)>>2]|0)+8>>2]|0;W=(c[V+8>>2]|0)==0?1:S;X=(c[V+48>>2]|0)==28?1:U;V=T+1|0;if(V>>>0<O>>>0){S=W;T=V;U=X}else{P=W;Q=X;break}}}is(s,0);ir(s,1887007846);U=I+4|0;T=c[U>>2]|0;L12730:do{if((T|0)==4){S=(Q|0)!=0;O=S?1681088:1680752;ir(s,(d[O+1|0]|0)<<8|(d[O]|0)|(d[O+2|0]|0)<<16|(d[O+3|0]|0)<<24);Y=S?256:512}else{if((T&16|0)!=0){S=(Q|0)!=0;O=S?1680616:1680328;ir(s,(d[O+1|0]|0)<<8|(d[O]|0)|(d[O+2|0]|0)<<16|(d[O+3|0]|0)<<24);Y=S?131072:65536;break}switch(T|0){case 128:{ir(s,544617574);Y=512;break L12730;break};case 32:{S=(P|0)!=0?1679912:1679856;ir(s,(d[S+1|0]|0)<<8|(d[S]|0)|(d[S+2|0]|0)<<16|(d[S+3|0]|0)<<24);Y=512;break L12730;break};case 64:{ir(s,1819112297);Y=512;break L12730;break};case 8:{ir(s,1447973709);Y=512;break L12730;break};case 1:{ir(s,1836020585);Y=512;break L12730;break};default:{ir(s,538997873);Y=512;break L12730}}}}while(0);is(s,Y);T=c[U>>2]|0;do{if((T|0)==2){ir(s,538997873)}else if((T|0)==64){ir(s,1717987696);ir(s,846164841)}else{ir(s,1836020585);ir(s,846164841);if((Q|0)==0){break}ir(s,828601953)}}while(0);T=c[U>>2]|0;do{if((T|0)==4){I=(Q|0)!=0?1681088:1680752;ir(s,(d[I+1|0]|0)<<8|(d[I]|0)|(d[I+2|0]|0)<<16|(d[I+3|0]|0)<<24)}else{if((T&16|0)!=0){I=(Q|0)!=0?1680616:1680328;ir(s,(d[I+1|0]|0)<<8|(d[I]|0)|(d[I+2|0]|0)<<16|(d[I+3|0]|0)<<24);break}if((T|0)==1){ir(s,825520237);break}else if((T|0)==8){ir(s,1447973709);break}else{break}}}while(0);T=il(s,0,0,1)|0;U=G;il(s,E,B,0)|0;I=NV(T,U,E,B)|0;is(s,I);il(s,T,U,0)|0;U=c[r>>2]|0;if((U|0)==8){if((c[N>>2]|0)!=2){AN(b,16,1694768,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0);i=F;H=-1;i=e;return H|0}T=c[b+28>>2]|0;I=c[(c[T>>2]|0)+8>>2]|0;S=c[(c[T+4>>2]|0)+8>>2]|0;T=c[S+412>>2]|0;O=(c[I+108>>2]<<16|0)/(c[I+104>>2]|0)|0;R=(c[S+72>>2]|0)/1e3|0;X=(c[I+72>>2]|0)/1e3|0;W=800-R|0;V=(X|0)>(W|0)?W:X;is(s,148);ir(s,1684632949);ir(s,1179603536);is(s,567431118);is(s,-1148688036);is(s,-87439552);is(s,0);is(s,3);is(s,20);ir(s,1179799622);is(s,0);is(s,0);is(s,0);is(s,44);ir(s,1179799617);is(s,0);is(s,2);ir(s,1630826605);is(s,527);is(s,0);is(s,R);is(s,R);is(s,T);is(s,c[S+416>>2]|0);is(s,52);ir(s,1179799638);is(s,0);is(s,1);if((c[I+48>>2]|0)==28){ir(s,828601953);iy(s,333);iy(s,21)}else{ir(s,1983148141);iy(s,0);iy(s,259)}is(s,0);is(s,V);is(s,V);is(s,O);is(s,O);iy(s,c[I+120>>2]|0);iy(s,c[I+124>>2]|0);is(s,65537);Z=c[r>>2]|0}else{Z=U}U=c[N>>2]|0;I=u+16|0;c[I>>2]=U;if((Z&34|0)==0){J=U;K=N;L=Z;M=I;break}if((c[b+1132>>2]|0)==0){J=U;K=N;L=Z;M=I;break}O=U+1|0;c[I>>2]=O;c[u+24>>2]=U;J=O;K=N;L=Z;M=I}}while(0);do{if((c[D>>2]&1|0)==0){_=0;$=J}else{Z=c[K>>2]|0;if((Z|0)==0){_=J;$=J;break}Q=b+28|0;Y=0;P=J;C=Z;while(1){if((c[(c[(c[(c[Q>>2]|0)+(Y<<2)>>2]|0)+8>>2]|0)+8>>2]|0)>>>0<2){Z=P+1|0;c[M>>2]=Z;aa=Z;ab=c[K>>2]|0}else{aa=P;ab=C}Z=Y+1|0;if(Z>>>0<ab>>>0){Y=Z;P=aa;C=ab}else{_=J;$=aa;break}}}}while(0);if((L|0)==2){if((c[K>>2]|0)==0){ad=0;ae=u+20|0}else{L=b+28|0;aa=(x|0)==0;J=u+20|0;ab=0;while(1){C=c[(c[L>>2]|0)+(ab<<2)>>2]|0;do{if((c[(c[C+8>>2]|0)+8>>2]|0)==0){if(aa){if((zJ(c[C+88>>2]|0,1696528,0,0)|0)==0){break}}c[J>>2]=(c[J>>2]|0)+1}}while(0);C=ab+1|0;P=c[K>>2]|0;if(C>>>0<P>>>0){ab=C}else{ad=P;ae=J;break}}}J=c[ae>>2]|0;do{if((J|0)==0){af=0}else{if((ad|0)==0){af=J;break}ab=b+28|0;aa=b;L=0;P=J;C=ad;while(1){if((c[(c[(c[(c[ab>>2]|0)+(L<<2)>>2]|0)+8>>2]|0)+52>>2]|0)==1684237684){AN(aa,24,1694464,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0);i=F;c[ae>>2]=0;ag=0;ah=c[K>>2]|0}else{ag=P;ah=C}Y=L+1|0;if(Y>>>0<ah>>>0){L=Y;P=ag;C=ah}else{af=ag;break}}}}while(0);ag=(c[M>>2]|0)+af|0;c[M>>2]=ag;ai=$;aj=ag}else{ai=0;aj=$}$=A7(aj*264|0)|0;aj=$;ag=u+48|0;af=ag;c[af>>2]=aj;if(($|0)==0){H=-12;i=e;return H|0}L12810:do{if((c[K>>2]|0)==0){A=9627}else{$=b+28|0;ah=b;ae=u+112|0;ad=b+32|0;J=0;C=aj;L12812:while(1){P=c[(c[$>>2]|0)+(J<<2)>>2]|0;L=zJ(c[P+88>>2]|0,1694184,0,0)|0;aa=P+8|0;ab=C+(J*264|0)+76|0;c[ab>>2]=c[aa>>2];if((L|0)==0){ak=1725720}else{ak=c[L+4>>2]|0}L=nr(ak,(c[r>>2]|0)!=2|0)|0;c[C+(J*264|0)+60>>2]=(L|0)<0?0:L;L=c[r>>2]|0;Y=C+(J*264|0)|0;c[Y>>2]=L;L12817:do{if((L|0)==32){Q=c[ab>>2]|0;N=c[Q+52>>2]|0;if((c[Q+8>>2]|0)==3){if((N|0)==1954047348|(N|0)==1731426420){al=N}else{A=9549}}else{A=9549}if((A|0)==9549){A=0;al=mz(290752,c[Q+48>>2]|0)|0}if((jg(ad,1682920)|0)!=0){am=al;A=9586;break}if((jg(ad,1682736)|0)!=0){am=al;A=9586;break}AN(ah,24,1682552,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0);i=F;am=al;A=9586}else if((L|0)==64){Q=c[ab>>2]|0;N=c[Q+52>>2]|0;B=(mz(246904,c[Q+48>>2]|0)|0)==0;Q=c[ab>>2]|0;E=c[Q+48>>2]|0;do{if(!B){if((E|0)==86019){an=858612577;A=9584;break L12817}else if((E|0)==28){an=828601953;A=9584;break L12817}else if((E|0)==117){an=1667330660;A=9584;break L12817}else if((E|0)==94213){an=1731426420;A=9584;break L12817}else if((E|0)==71){an=825058166;A=9584;break L12817}else{Z=c[Q+8>>2]|0;if((Z|0)==0){an=1983148141;A=9584;break L12817}I=(Z|0)==1?1630826605:N;if((I|0)==0){break}else{an=I;A=9584;break L12817}}}}while(0);am=(E|0)==86054?543255927:0;A=9586}else if((L|0)==1|(L|0)==8){N=c[ab>>2]|0;Q=c[N+52>>2]|0;if((mz(246904,c[N+48>>2]|0)|0)==0){A=9585;break L12812}N=c[ab>>2]|0;B=c[N+48>>2]|0;if((B|0)==117){an=1667330660;A=9584;break}else if((B|0)==94213){an=1731426420;A=9584;break}else if((B|0)==71){an=825058166;A=9584;break}else if((B|0)==28){an=828601953;A=9584;break}else if((B|0)==86019){an=858612577;A=9584;break}else{B=c[N+8>>2]|0;if((B|0)==0){an=1983148141;A=9584;break}am=(B|0)==1?1630826605:Q;A=9586;break}}else{if((L&4|0)!=0){am=mz(298192,c[(c[ab>>2]|0)+48>>2]|0)|0;A=9586;break}Q=c[ab>>2]|0;if((L&128|0)!=0){am=mz(290816,c[Q+48>>2]|0)|0;A=9586;break}B=c[Q+52>>2]|0;do{if((B|0)==0){ao=Q}else{if((c[Q+680>>2]|0)<=-1){an=B;A=9584;break L12817}N=c[Q+48>>2]|0;if((N|0)==25|(N|0)==14|(N|0)==5){ao=Q;break}if((pc(N)|0)==0){am=B;A=9586;break L12817}ao=c[ab>>2]|0}}while(0);Q=c[ao+48>>2]|0;if((Q|0)==25){E=c[ao+124>>2]|0;if((c[ao+120>>2]|0)!=720){if((E|0)==720){an=(c[ao+108>>2]|0)==50?1902671460:1885894244;A=9584;break}else if((E|0)==1080){an=(c[ao+108>>2]|0)==25?896038500:912815716;A=9584;break}else{A=9571;break L12812}}N=c[ao+140>>2]|0;I=(N|0)==4;if((E|0)==480){an=I?1848997476:543389284;A=9584;break}if(I){an=1882551908;A=9584;break}an=(N|0)==0?1885566564:1886418532;A=9584;break}else if((Q|0)==14){N=c[ao+52>>2]|0;I=0;E=ao;while(1){if((c[E+140>>2]|0)==(c[126216+(I*12|0)>>2]|0)){Z=c[126220+(I*12|0)>>2]|0;c[E+792>>2]=c[126224+(I*12|0)>>2];O=c[ab>>2]|0;if((c[O+52>>2]|0)==(Z|0)){am=Z;A=9586;break L12817}else{ap=Z;aq=O}}else{ap=N;aq=E}O=I+1|0;if(O>>>0<15){N=ap;I=O;E=aq}else{am=ap;A=9586;break}}}else{E=c[ao+8>>2]|0;if((E|0)==1){I=mz(253024,Q)|0;if((I|0)!=0){an=I;A=9584;break}I=mz(251144,c[(c[ab>>2]|0)+48>>2]|0)|0;if((I|0)==0){A=9585;break L12812}AN(ah,24,1683312,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0);i=F;an=I<<24|I>>>8<<16&16711680|29549;A=9584;break}else if((E|0)==3){am=mz(252992,Q)|0;A=9586;break}else if((E|0)==0){E=mz(251704,Q)|0;if((E|0)!=0){an=E;A=9584;break}E=mz(253424,c[(c[ab>>2]|0)+48>>2]|0)|0;if((E|0)==0){A=9585;break L12812}AN(ah,24,1683416,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0);i=F;an=E;A=9584;break}else{am=B;A=9586;break}}}}while(0);if((A|0)==9584){A=0;L=C+(J*264|0)+72|0;c[L>>2]=an;ar=L}else if((A|0)==9586){A=0;L=C+(J*264|0)+72|0;c[L>>2]=am;if((am|0)==0){break}else{ar=L}}c[C+(J*264|0)+120>>2]=-1;L=C+(J*264|0)+112|0;c[L>>2]=0;c[L+4>>2]=-2147483648;L=c[aa>>2]|0;E=c[L+8>>2]|0;do{if((E|0)==3){c[C+(J*264|0)+8>>2]=c[L+108>>2]}else if((E|0)==0){I=c[ar>>2]|0;if((I|0)==1882421357|(I|0)==1848866925|(I|0)==1882486893|(I|0)==1848932461|(I|0)==1882552429|(I|0)==1848997997){if((c[L+120>>2]|0)!=720){A=9592;break L12812}N=c[L+124>>2]|0;if(!((N|0)==608|(N|0)==512)){A=9592;break L12812}c[C+(J*264|0)+100>>2]=(I&-16777216|0)==1845493760?486:576}I=c[ae>>2]|0;if((I|0)==0){N=c[(c[aa>>2]|0)+108>>2]|0;while(1){if(N>>>0<1e4){N=N<<1}else{break}}c[C+(J*264|0)+8>>2]=N;as=N}else{c[C+(J*264|0)+8>>2]=I;as=I}if(!((c[Y>>2]|0)==2&as>>>0>1e5)){break}AN(ah,24,1693408,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0);i=F}else if((E|0)==1){B=C+(J*264|0)+8|0;c[B>>2]=c[L+412>>2];Q=c[aa>>2]|0;do{if((c[Q+424>>2]|0)==0){if((pc(c[Q+48>>2]|0)|0)==0){AN(ah,24,1693296,(F=i,i=i+8|0,c[F>>2]=J,F)|0);i=F;c[C+(J*264|0)+96>>2]=1;break}else{at=c[aa>>2]|0;A=9605;break}}else{at=Q;A=9605}}while(0);do{if((A|0)==9605){A=0;Q=c[at+48>>2]|0;if((Q|0)==69638|(Q|0)==69633|(Q|0)==86076){I=c[at+432>>2]|0;if((I|0)==0){A=9607;break L12812}c[C+(J*264|0)+40>>2]=I;break}if((c[at+424>>2]|0)>1){c[C+(J*264|0)+96>>2]=1;break}else{I=(pc(Q)|0)>>3;c[C+(J*264|0)+40>>2]=ac(c[(c[aa>>2]|0)+416>>2]|0,I)|0;break}}}while(0);if((c[Y>>2]|0)==2){break}au=c[ab>>2]|0;if((c[au+48>>2]|0)!=86017){break}if((c[B>>2]|0)>>>0<16e3){A=9615;break L12812}}else{c[C+(J*264|0)+8>>2]=1e3}}while(0);ab=C+(J*264|0)+100|0;if((c[ab>>2]|0)==0){c[ab>>2]=c[(c[aa>>2]|0)+124>>2]}ab=C+(J*264|0)+8|0;if((c[r>>2]|0)==64){c[ab>>2]=1e7;av=1e7}else{av=c[ab>>2]|0}mL(P,64,1,av);ab=c[(c[aa>>2]|0)+100>>2]|0;if((ab|0)!=0){Y=C+(J*264|0)+80|0;c[Y>>2]=ab;L=A1(ab)|0;c[C+(J*264|0)+84>>2]=L;ab=c[(c[aa>>2]|0)+96>>2]|0;E=c[Y>>2]|0;NN(L|0,ab|0,E)|0}E=J+1|0;if(E>>>0>=(c[K>>2]|0)>>>0){A=9627;break L12810}J=E;C=c[af>>2]|0}if((A|0)==9615){ae=c[au+412>>2]|0;AN(ah,16,1692792,(F=i,i=i+16|0,c[F>>2]=J,c[F+8>>2]=ae,F)|0);i=F;break}else if((A|0)==9592){AN(ah,16,1693792,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0);i=F;break}else if((A|0)==9607){AN(ah,16,1693032,(F=i,i=i+8|0,c[F>>2]=J,F)|0);i=F;break}else if((A|0)==9571){AN(ah,16,1683160,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0);i=F;A=9585}if((A|0)==9585){c[C+(J*264|0)+72>>2]=0}AN(ah,16,1693920,(F=i,i=i+8|0,c[F>>2]=J,F)|0);i=F}}while(0);L12945:do{if((A|0)==9627){if((c[r>>2]|0)==64){au=c[D>>2]|0;do{if((au&40|0)==0){if((c[z>>2]|0)!=0){break}if((c[u+96>>2]|0)!=0){break}c[z>>2]=5e6}}while(0);c[D>>2]=au|22}J=u+60|0;do{if((c[J>>2]|0)!=0){ah=il(s,0,0,1)|0;C=u+64|0;c[C>>2]=ah;c[C+4>>2]=G;C=c[J>>2]|0;if((C|0)<=0){break}ah=C;av=(C|0)<0|0?-1:0;io(s,ah,av)|0;}}while(0);if((c[D>>2]&2|0)==0){is(s,8);J=(c[r>>2]|0)==2?1683544:1633472;ir(s,(d[J+1|0]|0)<<8|(d[J]|0)|(d[J+2|0]|0)<<16|(d[J+3|0]|0)<<24);J=il(s,0,0,1)|0;au=u+32|0;c[au>>2]=J;c[au+4>>2]=G;is(s,0);ir(s,1952539757)}au=zJ(c[w>>2]|0,1691504,0,0)|0;if((au|0)==0){J=u+8|0;aw=c[J+4>>2]|0;ax=c[J>>2]|0}else{J=mV(c[au+4>>2]|0)|0;au=G;av=u+8|0;c[av>>2]=J;c[av+4>>2]=au;aw=au;ax=J}if(!((ax|0)==0&(aw|0)==0)){J=u+8|0;au=NU(ax,aw,2082844800,0)|0;c[J>>2]=au;c[J+4>>2]=G}J=c[u+24>>2]|0;do{if((J|0)!=0){au=o;av=c[t>>2]|0;ah=c[av+48>>2]|0;NM(n|0,0,80);c[n+32>>2]=J;c[n+36>>2]=1;c[ah+(J*264|0)>>2]=c[av+4>>2];c[ah+(J*264|0)+72>>2]=1954047348;c[ah+(J*264|0)+8>>2]=1e3;av=ov(0)|0;C=ah+(J*264|0)+76|0;c[C>>2]=av;c[av+8>>2]=3;do{if((iZ(m)|0)>-1){is(c[m>>2]|0,1);ih(c[m>>2]|0,0);ih(c[m>>2]|0,0);ih(c[m>>2]|0,0);ih(c[m>>2]|0,0);ih(c[m>>2]|0,0);ih(c[m>>2]|0,0);iy(c[m>>2]|0,0);iy(c[m>>2]|0,0);iy(c[m>>2]|0,0);iy(c[m>>2]|0,0);iy(c[m>>2]|0,0);iy(c[m>>2]|0,0);iy(c[m>>2]|0,1);ih(c[m>>2]|0,0);ih(c[m>>2]|0,0);ih(c[m>>2]|0,0);ih(c[m>>2]|0,0);ih(c[m>>2]|0,0);ih(c[m>>2]|0,0);is(c[m>>2]|0,13);ir(c[m>>2]|0,1650553958);iy(c[m>>2]|0,1);iy(c[m>>2]|0,1);ih(c[m>>2]|0,0);av=i$(c[m>>2]|0,o)|0;if((av|0)>0){c[(c[C>>2]|0)+96>>2]=c[o>>2];c[(c[C>>2]|0)+100>>2]=av;break}else{A4(au);break}}}while(0);au=b+1132|0;if((c[au>>2]|0)==0){break}C=b+1136|0;aa=p|0;P=p+4|0;av=q|0;ah=q+4|0;at=n+16|0;as=n+8|0;ar=n+48|0;am=n+28|0;an=n+24|0;ao=an;ap=0;do{aq=c[(c[C>>2]|0)+(ap<<2)>>2]|0;al=aq+24|0;ak=c[al>>2]|0;aj=c[al+4>>2]|0;al=aq+4|0;c[aa>>2]=1;c[P>>2]=1e3;ae=AY(ak,aj,al,p)|0;aj=G;ak=aq+16|0;ad=c[ak>>2]|0;$=c[ak+4>>2]|0;c[av>>2]=1;c[ah>>2]=1e3;ak=AY(ad,$,al,q)|0;al=G;c[at>>2]=ak;c[at+4>>2]=al;c[as>>2]=ak;c[as+4>>2]=al;$=NV(ae,aj,ak,al)|0;c[ar>>2]=$;$=zJ(c[aq+32>>2]|0,1624192,0,0)|0;if(($|0)!=0){aq=$+4|0;$=NL(c[aq>>2]|0)|0;al=$+2|0;c[am>>2]=al;ak=A1(al)|0;c[an>>2]=ak;al=$&65535;aj=ak;y=(al>>>8|al<<8)&65535;a[aj]=y&255;y=y>>8;a[aj+1|0]=y&255;aj=ak+2|0;ak=c[aq>>2]|0;NN(aj|0,ak|0,$)|0;kA(b,n)|0;A6(ao)}ap=ap+1|0;}while(ap>>>0<(c[au>>2]|0)>>>0)}}while(0);do{if((c[D>>2]&1|0)!=0){J=c[K>>2]|0;if((J|0)==0){break}au=b+28|0;ap=_;ao=0;an=J;while(1){if((c[(c[(c[(c[au>>2]|0)+(ao<<2)>>2]|0)+8>>2]|0)+8>>2]|0)>>>0<2){if((kU(b,ap,ao)|0)<0){break L12945}ay=ap+1|0;az=c[K>>2]|0}else{ay=ap;az=an}J=ao+1|0;if(J>>>0<az>>>0){ap=ay;ao=J;an=az}else{break}}}}while(0);do{if((c[u+20>>2]|0)!=0){if((c[K>>2]|0)==0){break}an=b+28|0;ao=(x|0)==0;ap=k;au=l;J=k+28|0;am=k+32|0;ar=k+36|0;as=l;at=au+4|0;ah=b;av=j+4|0;P=k+24|0;aa=j|0;C=ai;$=0;while(1){ak=c[(c[an>>2]|0)+($<<2)>>2]|0;do{if((c[(c[ak+8>>2]|0)+8>>2]|0)==0){if(ao){aj=zJ(c[ak+88>>2]|0,1696528,0,0)|0;if((aj|0)==0){aA=C;break}aB=aj;aC=c[(c[an>>2]|0)+($<<2)>>2]|0}else{aB=x;aC=ak}aj=c[aB+4>>2]|0;aq=c[t>>2]|0;al=aq+48|0;ae=c[al>>2]|0;NM(ap|0,0,80);c[J>>2]=4;c[am>>2]=C;c[ar>>2]=1;ad=aC+8|0;c[as>>2]=c[(c[ad>>2]|0)+108>>2];c[at>>2]=c[(c[ad>>2]|0)+104>>2];if((Ck(au)|0)<0){E=c[at>>2]|0;ab=aC+92|0;L=c[ab>>2]|0;Y=c[aC+96>>2]|0;AN(ah,48,1683688,(F=i,i=i+32|0,c[F>>2]=c[as>>2],c[F+8>>2]=E,c[F+16>>2]=L,c[F+24>>2]=Y,F)|0);i=F;Y=ab;ab=c[Y+4>>2]|0;c[l>>2]=c[Y>>2];c[l+4>>2]=ab}ab=Cm(j,au,aj,ah)|0;if((ab|0)<0){aD=ab}else{c[ae+(C*264|0)>>2]=c[aq+4>>2];aq=ae+(C*264|0)+72|0;c[aq>>2]=1684237684;c[ae+(C*264|0)+124>>2]=$;c[ae+(C*264|0)+8>>2]=c[(c[al>>2]|0)+($*264|0)+8>>2];if((c[av>>2]&1|0)!=0){al=ae+(C*264|0)+56|0;c[al>>2]=c[al>>2]|1}al=ov(0)|0;ab=ae+(C*264|0)+76|0;c[ab>>2]=al;c[al+8>>2]=2;c[(c[ab>>2]|0)+52>>2]=c[aq>>2];aq=(c[ad>>2]|0)+104|0;ad=(c[ab>>2]|0)+104|0;ab=c[aq+4>>2]|0;c[ad>>2]=c[aq>>2];c[ad+4>>2]=ab;ab=A1(c[J>>2]|0)|0;c[P>>2]=ab;ad=ab;y=cV(c[aa>>2]|0)|0;a[ad]=y&255;y=y>>8;a[ad+1|0]=y&255;y=y>>8;a[ad+2|0]=y&255;y=y>>8;a[ad+3|0]=y&255;ad=kA(b,k)|0;A4(c[P>>2]|0);aD=ad}if((aD|0)<0){break L12945}aA=C+1|0}else{aA=C}}while(0);ak=$+1|0;if(ak>>>0<(c[K>>2]|0)>>>0){C=aA;$=ak}else{break}}}}while(0);ik(s);$=c[D>>2]|0;if(($&64|0)==0){aE=$}else{$=il(s,0,0,1)|0;C=G;is(s,0);ir(s,1684632949);ij(s,126184,16);is(s,0);iX(s,1691176,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0)|0;i=F;iX(s,1690976,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0)|0;i=F;iX(s,1690592,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0)|0;i=F;iX(s,1690360,(F=i,i=i+8|0,c[F>>2]=1739224,F)|0)|0;i=F;iX(s,1690152,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0)|0;i=F;iX(s,1690104,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0)|0;i=F;iX(s,1689848,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0)|0;i=F;P=c[M>>2]|0;if((P|0)>0){aa=f|0;J=0;av=P;while(1){P=J+1|0;ah=(c[af>>2]|0)+(J*264|0)+76|0;au=c[ah>>2]|0;as=c[au+8>>2]|0;if((as|0)==1){aF=1689608;A=9682}else if((as|0)==0){aF=1689744;A=9682}else{aG=av}if((A|0)==9682){A=0;as=c[au+72>>2]|0;iX(s,1689312,(F=i,i=i+16|0,c[F>>2]=aF,c[F+8>>2]=as,F)|0)|0;i=F;as=c[(c[ah>>2]|0)+72>>2]|0;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1689032,c[F+8>>2]=as,F)|0)|0;i=F;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1688904,c[F+8>>2]=P,F)|0)|0;i=F;as=c[ah>>2]|0;au=c[as+48>>2]|0;if((c[as+8>>2]|0)==0){if((au|0)==71){iX(s,1684256,(F=i,i=i+16|0,c[F>>2]=1687264,c[F+8>>2]=1687120,F)|0)|0;i=F;at=c[ah>>2]|0;ar=c[at+96>>2]|0;am=c[at+100>>2]|0;at=am>>>0<74?am:74;mU(aa,ar,at,0)|0;a[f+(at<<1)|0]=0;iX(s,1684256,(F=i,i=i+16|0,c[F>>2]=1688624,c[F+8>>2]=aa,F)|0)|0;i=F}else if((au|0)==28){c[h>>2]=c[as+100>>2];if((ni(c[(c[ah>>2]|0)+96>>2]|0,g,h)|0)==0){as=c[g>>2]|0;if((as|0)==0){aH=c[(c[ah>>2]|0)+96>>2]|0}else{aH=as}as=c[h>>2]|0;at=as>>>0<74?as:74;mU(aa,aH,at,0)|0;a[f+(at<<1)|0]=0;iX(s,1684256,(F=i,i=i+16|0,c[F>>2]=1688624,c[F+8>>2]=aa,F)|0)|0;i=F;A4(c[g>>2]|0)}iX(s,1684256,(F=i,i=i+16|0,c[F>>2]=1687264,c[F+8>>2]=1687176,F)|0)|0;i=F}at=c[(c[ah>>2]|0)+120>>2]|0;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1687064,c[F+8>>2]=at,F)|0)|0;i=F;at=c[(c[ah>>2]|0)+124>>2]|0;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1686968,c[F+8>>2]=at,F)|0)|0;i=F;at=c[(c[ah>>2]|0)+120>>2]|0;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1686872,c[F+8>>2]=at,F)|0)|0;i=F;at=c[(c[ah>>2]|0)+124>>2]|0;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1686568,c[F+8>>2]=at,F)|0)|0;i=F}else{if((au|0)==86018){iX(s,1684256,(F=i,i=i+16|0,c[F>>2]=1687264,c[F+8>>2]=1686280,F)|0)|0;i=F}else if((au|0)==86054){iX(s,1684256,(F=i,i=i+16|0,c[F>>2]=1687264,c[F+8>>2]=1686176,F)|0)|0;i=F}au=c[ah>>2]|0;at=c[au+96>>2]|0;as=c[au+100>>2]|0;au=as>>>0<74?as:74;mU(aa,at,au,0)|0;a[f+(au<<1)|0]=0;iX(s,1684256,(F=i,i=i+16|0,c[F>>2]=1688624,c[F+8>>2]=aa,F)|0)|0;i=F;au=mz(251144,c[(c[ah>>2]|0)+48>>2]|0)|0;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1685928,c[F+8>>2]=au,F)|0)|0;i=F;au=c[(c[ah>>2]|0)+416>>2]|0;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1685752,c[F+8>>2]=au,F)|0)|0;i=F;au=c[(c[ah>>2]|0)+412>>2]|0;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1685632,c[F+8>>2]=au,F)|0)|0;i=F;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1685480,c[F+8>>2]=16,F)|0)|0;i=F;au=c[(c[ah>>2]|0)+432>>2]|0;ah=(au|0)==0?4:au;iX(s,1683952,(F=i,i=i+16|0,c[F>>2]=1685384,c[F+8>>2]=ah,F)|0)|0;i=F}iX(s,1685304,(F=i,i=i+8|0,c[F>>2]=aF,F)|0)|0;i=F;aG=c[M>>2]|0}if((P|0)<(aG|0)){J=P;av=aG}else{break}}}iX(s,1685240,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0)|0;i=F;iX(s,1684896,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0)|0;i=F;iX(s,1684664,(F=i,i=i+1|0,i=i+7&-8,c[F>>2]=0,F)|0)|0;i=F;av=il(s,0,0,1)|0;J=G;il(s,$,C,0)|0;aa=NV(av,J,$,C)|0;is(s,aa);il(s,av,J,0)|0;aE=c[D>>2]|0}if((aE&4|0)==0){H=0;i=e;return H|0}kE(s,v,b);J=u+84|0;c[J>>2]=(c[J>>2]|0)+1;H=0;i=e;return H|0}}while(0);A6(ag);H=-1;i=e;return H|0}function kC(a,d){a=a|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;e=i;i=i+88|0;f=e|0;g=e+80|0;if((d|0)==0){kF(a);h=1;i=e;return h|0}j=c[a+12>>2]|0;if((c[d+28>>2]|0)==0){h=0;i=e;return h|0}k=j+16|0;l=c[k>>2]|0;L13062:do{if((l|0)>0){m=j+48|0;n=d+16|0;o=g;p=f+28|0;q=f+24|0;r=f+8|0;s=f+16|0;t=f+48|0;u=f+32|0;v=0;w=l;L13064:while(1){x=c[m>>2]|0;do{if((c[(c[x+(v*264|0)+76>>2]|0)+48>>2]|0)==94213){y=x+(v*264|0)+24|0;z=c[y>>2]|0;A=c[y+4>>2]|0;y=c[n+4>>2]|0;if(!((A|0)<(y|0)|(A|0)==(y|0)&z>>>0<(c[n>>2]|0)>>>0)){B=w;break}if((c[x+(v*264|0)+4>>2]|0)!=0){if((c[x+(v*264|0)+32>>2]|0)!=0){B=w;break}}b[g>>1]=0;nB(f);c[p>>2]=2;c[q>>2]=o;c[r>>2]=z;c[r+4>>2]=A;c[s>>2]=z;c[s+4>>2]=A;c[t>>2]=0;c[u>>2]=v;A=kT(a,f)|0;nJ(f);if((A|0)<0){h=A;break L13064}c[x+(v*264|0)+32>>2]=1;B=c[k>>2]|0}else{B=w}}while(0);x=v+1|0;if((x|0)<(B|0)){v=x;w=B}else{break L13062}}i=e;return h|0}}while(0);h=kT(a,d)|0;i=e;return h|0}
function CH(b,e,f,j,k,l,m,n,o){b=b|0;e=e|0;f=f|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,S=0,U=0.0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0.0,aK=0,aL=0.0,aM=0.0,aN=0,aO=0.0,aP=0.0,aQ=0.0,aR=0.0,aS=0.0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0.0,a_=0.0,a$=0.0,a0=0,a1=0.0,a2=0.0,a3=0.0,a4=0.0,a5=0,a6=0.0,a7=0,a8=0,a9=0.0,ba=0.0,bb=0,bc=0,bd=0,be=0,bf=0.0,bg=0.0,bh=0,bi=0.0,bj=0.0,bk=0.0,bl=0.0,bm=0.0,bn=0.0,bo=0,bp=0,bq=0,br=0.0,bs=0.0,bt=0.0,bu=0.0,bv=0,bw=0.0,bx=0.0,by=0,bz=0.0,bA=0,bB=0,bC=0.0,bD=0,bE=0,bF=0.0,bG=0,bH=0,bI=0.0,bJ=0.0,bK=0,bL=0.0,bM=0.0,bN=0.0,bO=0,bP=0,bQ=0,bR=0,bS=0.0,bT=0.0,bU=0.0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0.0,b1=0.0,b2=0.0,b3=0.0,b4=0.0;p=i;i=i+26584|0;q=p|0;r=p+64|0;s=p+120|0;t=p+176|0;u=p+432|0;v=p+688|0;w=p+760|0;x=p+5368|0;y=p+5416|0;z=p+5464|0;A=p+5480|0;B=p+6456|0;C=p+8512|0;D=p+10064|0;E=p+18256|0;F=p+24400|0;G=p+25424|0;H=p+26448|0;I=p+26496|0;J=p+26560|0;K=p+26568|0;L=K;M=b+85800|0;N=c[M>>2]|0;O=b+140|0;P=(c[O>>2]|0)==0;if(P){Q=0}else{Q=c[b+85804>>2]|0}S=b+192|0;if(+g[S>>2]>0.0){U=+g[b+200>>2]*+g[(c[b+85796>>2]|0)+8>>2]}else{U=1.0}V=F|0;NM(I|0,0,64);W=b+180|0;X=(c[W>>2]|0)==1;if(X){Y=4}else{Y=c[b+72>>2]|0}Z=A;_=b+25660|0;NN(Z|0,_|0,976)|0;_=J|0;Z=z;if(P){$=0}else{$=c[b+85804>>2]|0}P=b+72|0;aa=c[P>>2]|0;ab=X?4:aa;NM(w|0,0,4608);if((aa|0)>0){if((ab|0)>2){X=0;do{ac=c[e+(X<<2)>>2]|0;ad=0;do{g[w+(X*2304|0)+(ad<<2)>>2]=+g[ac+(ad+407<<2)>>2]+(+g[ac+(ad+397<<2)>>2]+ +g[ac+(ad+418<<2)>>2])*-1.7303260184043527e-17+(+g[ac+(ad+399<<2)>>2]+ +g[ac+(ad+416<<2)>>2])*-1.3495279640235235e-17+(+g[ac+(ad+401<<2)>>2]+ +g[ac+(ad+414<<2)>>2])*-6.732779685849225e-17+(+g[ac+(ad+403<<2)>>2]+ +g[ac+(ad+412<<2)>>2])*-3.0835000291318875e-17+(+g[ac+(ad+405<<2)>>2]+ +g[ac+(ad+410<<2)>>2])*-1.1044240253100168e-16+((+g[ac+(ad+398<<2)>>2]+ +g[ac+(ad+417<<2)>>2])*-.017031719908118248+0.0+(+g[ac+(ad+400<<2)>>2]+ +g[ac+(ad+415<<2)>>2])*.04180720075964928+(+g[ac+(ad+402<<2)>>2]+ +g[ac+(ad+413<<2)>>2])*-.08763240277767181+(+g[ac+(ad+404<<2)>>2]+ +g[ac+(ad+411<<2)>>2])*.1863476037979126+(+g[ac+(ad+406<<2)>>2]+ +g[ac+(ad+409<<2)>>2])*-.6276379823684692);ad=ad+1|0;}while((ad|0)<576);ad=j+(f*976|0)+(X*488|0)+244|0;ac=b+26636+(X*244|0)|0;NN(ad|0,ac|0,244)|0;ac=j+(f*976|0)+(X*488|0)|0;ad=b+25660+(X*244|0)|0;NN(ac|0,ad|0,244)|0;ad=X+2|0;ac=k+(f*976|0)+(X*488|0)+244|0;ae=b+26636+(ad*244|0)|0;NN(ac|0,ae|0,244)|0;ae=k+(f*976|0)+(X*488|0)|0;ac=b+25660+(ad*244|0)|0;NN(ae|0,ac|0,244)|0;X=X+1|0;}while((X|0)<(aa|0))}else{X=0;do{ac=c[e+(X<<2)>>2]|0;ae=0;do{g[w+(X*2304|0)+(ae<<2)>>2]=+g[ac+(ae+407<<2)>>2]+(+g[ac+(ae+397<<2)>>2]+ +g[ac+(ae+418<<2)>>2])*-1.7303260184043527e-17+(+g[ac+(ae+399<<2)>>2]+ +g[ac+(ae+416<<2)>>2])*-1.3495279640235235e-17+(+g[ac+(ae+401<<2)>>2]+ +g[ac+(ae+414<<2)>>2])*-6.732779685849225e-17+(+g[ac+(ae+403<<2)>>2]+ +g[ac+(ae+412<<2)>>2])*-3.0835000291318875e-17+(+g[ac+(ae+405<<2)>>2]+ +g[ac+(ae+410<<2)>>2])*-1.1044240253100168e-16+((+g[ac+(ae+398<<2)>>2]+ +g[ac+(ae+417<<2)>>2])*-.017031719908118248+0.0+(+g[ac+(ae+400<<2)>>2]+ +g[ac+(ae+415<<2)>>2])*.04180720075964928+(+g[ac+(ae+402<<2)>>2]+ +g[ac+(ae+413<<2)>>2])*-.08763240277767181+(+g[ac+(ae+404<<2)>>2]+ +g[ac+(ae+411<<2)>>2])*.1863476037979126+(+g[ac+(ae+406<<2)>>2]+ +g[ac+(ae+409<<2)>>2])*-.6276379823684692);ae=ae+1|0;}while((ae|0)<576);ae=j+(f*976|0)+(X*488|0)+244|0;ac=b+26636+(X*244|0)|0;NN(ae|0,ac|0,244)|0;ac=j+(f*976|0)+(X*488|0)|0;ae=b+25660+(X*244|0)|0;NN(ac|0,ae|0,244)|0;X=X+1|0;}while((X|0)<(aa|0))}}if((ab|0)>0){aa=z|0;X=($|0)==0;ae=J+4|0;ac=x|0;ad=y|0;af=y+4|0;ag=x+4|0;ah=y+8|0;ai=x+8|0;aj=y+12|0;ak=y+16|0;al=y+20|0;am=y+24|0;an=y+28|0;ao=y+32|0;ap=y+36|0;aq=y+40|0;ar=y+44|0;as=z+4|0;at=z+8|0;au=z+12|0;av=x+12|0;aw=x+16|0;ax=x+20|0;ay=x+24|0;az=x+28|0;aA=x+32|0;aB=x+36|0;aC=x+40|0;aD=x+44|0;aE=0;do{NM(Z|0,0,16);aF=w+((aE&1)*2304|0)|0;if((aE|0)==2){aG=576;aH=0;while(1){aI=w+(aH<<2)|0;aJ=+g[aI>>2];aK=w+2304+(aH<<2)|0;aL=+g[aK>>2];g[aI>>2]=aJ+aL;g[aK>>2]=aJ-aL;aK=aG-1|0;if((aK|0)>0){aG=aK;aH=aH+1|0}else{break}}}aL=+g[b+27636+(aE*36|0)+24>>2];g[ad>>2]=aL;g[ac>>2]=aL/+g[b+27636+(aE*36|0)+16>>2];aJ=+g[b+27636+(aE*36|0)+28>>2];g[af>>2]=aJ;g[ag>>2]=aJ/+g[b+27636+(aE*36|0)+20>>2];aM=+g[b+27636+(aE*36|0)+32>>2];g[ah>>2]=aM;g[ai>>2]=aM/aL;g[aa>>2]=aL+0.0+aJ+aM;aH=aF;aG=0;while(1){aK=aH+256|0;aI=aH;aM=1.0;do{aJ=+R(+(+g[aI>>2]));aM=aM<aJ?aJ:aM;aI=aI+4|0;}while(aI>>>0<aK>>>0);aK=aG+3|0;g[y+(aK<<2)>>2]=aM;g[b+27636+(aE*36|0)+(aG<<2)>>2]=aM;aN=z+(((aG|0)/3|0)+1<<2)|0;g[aN>>2]=aM+ +g[aN>>2];aN=aG+1|0;aJ=+g[y+(aN<<2)>>2];do{if(aM>aJ){aO=aM/aJ}else{aL=aM*10.0;if(aJ<=aL){aO=0.0;break}aO=aJ/aL}}while(0);g[x+(aK<<2)>>2]=aO;if((aN|0)<9){aH=aI;aG=aN}else{break}}aJ=+g[ak>>2];aM=+g[al>>2];aL=+g[aj>>2]+aJ+aM;do{if(aM*6.0<aL){if(aJ*6.0>=aL){aP=.5;break}aP=.25}else{aP=1.0}}while(0);g[H+(aE*12|0)>>2]=aP;aL=+g[an>>2];aJ=+g[ao>>2];aM=+g[am>>2]+aL+aJ;do{if(aJ*6.0<aM){if(aL*6.0>=aM){aQ=.5;break}aQ=.25}else{aQ=1.0}}while(0);g[H+(aE*12|0)+4>>2]=aQ;aM=+g[aq>>2];aL=+g[ar>>2];aJ=+g[ap>>2]+aM+aL;do{if(aL*6.0<aJ){if(aM*6.0>=aJ){aR=.5;break}aR=.25}else{aR=1.0}}while(0);g[H+(aE*12|0)+8>>2]=aR;if(!X){aJ=+g[ac>>2];aM=+g[ag>>2];aL=aJ<aM?aM:aJ;aJ=+g[ai>>2];aM=aL<aJ?aJ:aL;aL=+g[av>>2];aJ=aM<aL?aL:aM;aM=+g[aw>>2];aL=aJ<aM?aM:aJ;aJ=+g[ax>>2];aM=aL<aJ?aJ:aL;aL=+g[ay>>2];aJ=aM<aL?aL:aM;aM=+g[az>>2];aL=aJ<aM?aM:aJ;aJ=+g[aA>>2];aM=aL<aJ?aJ:aL;aL=+g[aB>>2];aJ=aM<aL?aL:aM;aM=+g[aC>>2];aL=aJ<aM?aM:aJ;aJ=+g[aD>>2];aG=$+197112+(aE<<3)|0;h[$+197144+(f<<5)+(aE<<3)>>3]=+h[aG>>3];h[aG>>3]=aL<aJ?aJ:aL}aL=+g[(c[M>>2]|0)+6480+(aE<<2)>>2];aG=0;do{aH=I+(aE<<4)+(((aG|0)/3|0)<<2)|0;do{if((c[aH>>2]|0)==0){if(+g[x+(aG<<2)>>2]<=aL){break}c[aH>>2]=((aG|0)%3|0)+1}}while(0);aG=aG+1|0;}while((aG|0)<12);aG=I+(aE<<4)|0;aL=+g[aa>>2];aJ=+g[as>>2];aM=aJ*1.7000000476837158;if((aL>aJ?aL:aJ)<4.0e4&aL<aM&aJ<aL*1.7000000476837158){aH=I+(aE<<4)+4|0;if((c[aG>>2]|0)<=(c[aH>>2]|0)){c[aG>>2]=0}c[aH>>2]=0}aL=+g[at>>2];aS=aL*1.7000000476837158;if((aJ>aL?aJ:aL)<4.0e4&aJ<aS&aL<aM){c[I+(aE<<4)+8>>2]=0}aM=+g[au>>2];if((aL>aM?aL:aM)<4.0e4&aL<aM*1.7000000476837158&aM<aS){c[I+(aE<<4)+12>>2]=0}aH=c[aG>>2]|0;aN=c[b+27780+(aE<<2)>>2]|0;if((aH|0)>(aN|0)){aT=aH}else{c[aG>>2]=0;aT=0}aG=I+(aE<<4)+4|0;aH=c[aG>>2]|0;if((aN|0)==3){aU=16401}else{if((aH+aT+(c[I+(aE<<4)+8>>2]|0)|0)==(-(c[I+(aE<<4)+12>>2]|0)|0)){aV=1}else{aU=16401}}do{if((aU|0)==16401){aU=0;do{if((aH|0)==0){aW=0}else{if((aT|0)==0){aW=aH;break}c[aG>>2]=0;aW=0}}while(0);aN=I+(aE<<4)+8|0;if((c[aN>>2]|0)==0){aV=0;break}if((aW|0)!=0){c[aN>>2]=0;aV=0;break}aN=I+(aE<<4)+12|0;if((c[aN>>2]|0)==0){aV=0;break}c[aN>>2]=0;aV=0}}while(0);do{if((aE|0)<2){c[J+(aE<<2)>>2]=aV}else{if((aV|0)!=0){break}c[ae>>2]=0;c[_>>2]=0}}while(0);g[n+(aE<<2)>>2]=+g[b+27620+(aE<<2)>>2];aE=aE+1|0;}while((aE|0)<(ab|0))}ab=c[b+184>>2]|0;do{if((ab|0)==1){aE=J+4|0;if((c[_>>2]|0)!=0){if((c[aE>>2]|0)!=0){break}}c[aE>>2]=0;c[_>>2]=0}}while(0);aE=c[P>>2]|0;if((aE|0)>0){n=0;do{if((ab|0)==2){c[J+(n<<2)>>2]=1}else if((ab|0)==3){c[J+(n<<2)>>2]=0}n=n+1|0;}while((n|0)<(aE|0))}aE=(Y|0)>0;if(aE){n=B|0;ab=b+85796|0;ae=v|0;aV=t|0;aW=u|0;aT=t+4|0;au=b+84908|0;at=u+4|0;as=b+85804|0;aa=0;do{x=aa&1;$=D+(x<<12)|0;if((c[O>>2]|0)==0){aX=0}else{aX=c[as>>2]|0}aD=(aa|0)<2;do{if(aD){EM(b,$|0,aa,e)}else{if((aa|0)!=2){break}aC=x+1|0;aB=1023;while(1){aA=D+(x<<12)+(aB<<2)|0;aR=+g[aA>>2];az=D+(aC<<12)+(aB<<2)|0;aQ=+g[az>>2];g[aA>>2]=(aR+aQ)*.7071067690849304;g[az>>2]=(aR-aQ)*.7071067690849304;if((aB|0)>0){aB=aB-1|0}else{break}}}}while(0);aQ=+g[$>>2];g[n>>2]=aQ*aQ;aB=511;while(1){aC=512-aB|0;aQ=+g[D+(x<<12)+(aC<<2)>>2];aR=+g[D+(x<<12)+(aB+512<<2)>>2];g[B+(aC<<2)>>2]=(aQ*aQ+aR*aR)*.5;if((aB|0)>0){aB=aB-1|0}else{aY=11;aZ=0.0;break}}do{aZ=aZ+ +g[B+(aY<<2)>>2];aY=aY+1|0;}while((aY|0)<513);g[b+27620+(aa<<2)>>2]=aZ;if((aX|0)!=0){aB=0;do{$=aX+90936+(aa<<13)+(aB<<3)|0;h[aX+123704+(f<<15)+(aa<<13)+(aB<<3)>>3]=+h[$>>3];h[$>>3]=+g[B+(aB<<2)>>2];aB=aB+1|0;}while((aB|0)<513)}if(aD){aB=b+27612+(aa<<2)|0;g[b+27804+(f<<3)+(aa<<2)>>2]=+g[aB>>2];$=c[ab>>2]|0;aC=0;aR=0.0;do{aR=aR+ +g[B+(aC<<2)>>2]*+g[$+724+(aC<<2)>>2];aC=aC+1|0;}while((aC|0)<512);g[aB>>2]=aR*8.974871343596633e-12}aC=c[M>>2]|0;$=aC+2148|0;aD=c[$>>2]|0;az=(aD|0)>0;if(az){aA=0;ay=0;while(1){ax=c[aC+1716+(ay<<2)>>2]|0;if((ax|0)>0){aw=aA;aQ=0.0;aP=0.0;av=0;while(1){aO=+g[B+(aw<<2)>>2];a_=aQ+aO;a$=aP<aO?aO:aP;ai=av+1|0;if((ai|0)<(ax|0)){aw=aw+1|0;aQ=a_;aP=a$;av=ai}else{break}}a0=((ax|0)>1?ax:1)+aA|0;a1=a_;a2=a$}else{a0=aA;a1=0.0;a2=0.0}g[F+(aa<<8)+(ay<<2)>>2]=a1;g[t+(ay<<2)>>2]=a2;g[u+(ay<<2)>>2]=a1*+g[aC+512+(ay<<2)>>2];av=ay+1|0;if((av|0)<(aD|0)){aA=a0;ay=av}else{break}}a3=+g[aW>>2];a4=+g[at>>2]}else{a3=0.0;a4=0.0}aR=a4+a3;if(aR>0.0){aP=+g[aV>>2];aQ=+g[aT>>2];ay=~~(((aP<aQ?aQ:aP)*2.0-aR)*20.0/(aR*+((c[aC+1716>>2]|0)-1+(c[aC+1720>>2]|0)|0)));a5=(ay|0)>8?8:ay&255}else{a5=0}a[ae]=a5;ay=aD-1|0;aR=+g[at>>2];aP=a3+aR;if((ay|0)>1){aA=(ay|0)>2;aB=1;av=0;aQ=aP;aO=aR;while(1){aw=aB+1|0;aR=+g[u+(aw<<2)>>2];aS=aQ+aR;if(aS>0.0){aM=+g[t+(av<<2)>>2];aL=+g[t+(aB<<2)>>2];aJ=aM<aL?aL:aM;aM=+g[t+(aw<<2)>>2];ai=~~(((aJ<aM?aM:aJ)*3.0-aS)*20.0/(aS*+((c[aC+1716+(av<<2)>>2]|0)-1+(c[aC+1716+(aB<<2)>>2]|0)+(c[aC+1716+(aw<<2)>>2]|0)|0)));a[v+aB|0]=(ai|0)>8?8:ai&255}else{a[v+aB|0]=0}a6=aO+aR;if((aw|0)<(ay|0)){av=aB;aB=aw;aQ=a6;aO=aR}else{break}}aB=aA?ay:2;a7=aB;a8=aB-1|0;a9=a6}else{a7=1;a8=0;a9=aP}if(a9>0.0){aO=+g[t+(a8<<2)>>2];aQ=+g[t+(a7<<2)>>2];aB=~~(((aO<aQ?aQ:aO)*2.0-a9)*20.0/(a9*+((c[aC+1716+(a8<<2)>>2]|0)-1+(c[aC+1716+(a7<<2)>>2]|0)|0)));a[v+a7|0]=(aB|0)>8?8:aB&255}else{a[v+a7|0]=0}if(az){aB=aC+2156|0;av=b+27796+(x<<2)|0;aD=0;aw=0;while(1){aO=+g[aC+(aD<<2)>>2]*+g[au>>2];ai=c[aC+1204+(aD<<3)>>2]|0;ag=c[aC+1204+(aD<<3)+4>>2]|0;ac=c[64624+((d[v+aD|0]|0)<<2)>>2]|0;X=d[v+ai|0]|0;ap=c[aB>>2]|0;aQ=+g[ap+(aw<<2)>>2]*+g[F+(aa<<8)+(ai<<2)>>2]*+g[64664+(X<<2)>>2];ar=aw+1|0;aq=ai+1|0;if((aq|0)>(ag|0)){ba=aQ;bb=X;bc=2;bd=ar}else{aR=aQ;ai=X;X=2;am=ar;ar=aq;aq=ap;while(1){ap=d[v+ar|0]|0;be=ap+ai|0;aQ=+g[aq+(am<<2)>>2]*+g[F+(aa<<8)+(ar<<2)>>2]*+g[64664+(ap<<2)>>2];ap=ar-aD|0;aS=aR<0.0?0.0:aR;aJ=aQ<0.0?0.0:aQ;do{if(aS>0.0){if(aJ<=0.0){bf=aS;break}ao=aJ>aS;if(ao){bg=aJ/aS}else{bg=aS/aJ}if((((ap|0)>-1?ap:-ap|0)|0)>(ac|0)){if(bg<+g[32678]){bf=aS+aJ;break}else{bf=ao?aJ:aS;break}}else{if(bg<+g[32680]){bf=(aS+aJ)*+g[36976+(~~(+Ek(bg)*4.816479930623698)<<2)>>2];break}else{bf=aS+aJ;break}}}else{bf=aJ}}while(0);bh=am+1|0;ap=ar+1|0;if((ap|0)>(ag|0)){break}aR=bf;ai=be;X=X+1|0;am=bh;ar=ap;aq=c[aB>>2]|0}ba=bf;bb=be;bc=X<<1;bd=bh}aR=+g[64664+(((bb<<1|1|0)/(bc|0)|0)<<2)>>2]*.5;aJ=ba*aR;aq=c[av>>2]|0;do{if((aq|0)==2){aS=+g[b+21564+(aa<<8)+(aD<<2)>>2];aQ=aS*2.0;if(aQ>0.0){aM=aJ<aQ?aJ:aQ;g[G+(aa<<8)+(aD<<2)>>2]=aM;bi=aS;bj=aM;break}else{aM=aJ;aQ=+g[F+(aa<<8)+(aD<<2)>>2]*.3;aL=aM<aQ?aM:aQ;g[G+(aa<<8)+(aD<<2)>>2]=aL;bi=aS;bj=aL;break}}else{aL=+g[b+22588+(aa<<8)+(aD<<2)>>2]*16.0;aS=+g[b+21564+(aa<<8)+(aD<<2)>>2];aQ=aS*2.0;aM=aL>0.0?aL:aJ;aL=aQ>0.0?aQ:aJ;if((aq|0)==0){bk=aL<aM?aL:aM}else{bk=aL}aL=aJ<bk?aJ:bk;g[G+(aa<<8)+(aD<<2)>>2]=aL;bi=aS;bj=aL}}while(0);g[b+22588+(aa<<8)+(aD<<2)>>2]=bi;g[b+21564+(aa<<8)+(aD<<2)>>2]=aJ;aL=aR*+g[t+(aD<<2)>>2]*+g[aC+256+(aD<<2)>>2];aq=G+(aa<<8)+(aD<<2)|0;if(bj>aL){g[aq>>2]=aL;bl=aL}else{bl=bj}if(aO>1.0){aL=aO*bl;g[aq>>2]=aL;bm=aL}else{bm=bl}aL=+g[F+(aa<<8)+(aD<<2)>>2];if(bm>aL){g[aq>>2]=aL;bn=aL}else{bn=bm}if(aO<1.0){g[aq>>2]=aO*bn}bo=aD+1|0;if((bo|0)<(c[$>>2]|0)){aD=bo;aw=bd}else{break}}if((bo|0)<64){bp=bo;aU=16472}}else{bp=0;aU=16472}if((aU|0)==16472){aU=0;aw=bp+1|0;aD=((aw|0)>64?aw:64)-bp<<2;NM(F+(aa<<8)+(bp<<2)|0,0,aD|0);NM(G+(aa<<8)+(bp<<2)|0,0,aD|0)}aa=aa+1|0;}while((aa|0)<(Y|0))}do{if((c[W>>2]|0)==1){if(((c[J+4>>2]|0)+(c[_>>2]|0)|0)!=2){break}CI(V,G|0,N+768|0,(c[b+85796>>2]|0)+212|0,U,+g[S>>2],c[N+2148>>2]|0)}}while(0);aa=r|0;bp=s|0;if(aE){bo=0;do{bd=F+(bo<<8)|0;bc=G+(bo<<8)|0;CN(c[M>>2]|0,bd,bc,b+26636+(bo*244|0)|0,b+25660+(bo*244|0)|0);CN((c[M>>2]|0)+4320|0,bd,bc,aa,bp);bc=0;do{bn=+g[r+(bc<<2)>>2];bm=+g[s+(bc<<2)>>2]*.015625;g[b+26636+(bo*244|0)+88+(bc*12|0)>>2]=bn;g[b+25660+(bo*244|0)+88+(bc*12|0)>>2]=bm;g[b+26636+(bo*244|0)+88+(bc*12|0)+4>>2]=bn;g[b+25660+(bo*244|0)+88+(bc*12|0)+4>>2]=bm;g[b+26636+(bo*244|0)+88+(bc*12|0)+8>>2]=bn;g[b+25660+(bo*244|0)+88+(bc*12|0)+8>>2]=bm;bc=bc+1|0;}while((bc|0)<13);bo=bo+1|0;}while((bo|0)<(Y|0))}bo=(c[(c[M>>2]|0)+6500>>2]|0)==0;bc=t;bd=u;bb=q|0;bh=t|0;be=t+4|0;v=b+84908|0;au=u|0;a7=u+4|0;a8=c[_>>2]|0;_=-(c[J+4>>2]|0)|0;at=G|0;a5=N+2928|0;ae=b+85796|0;aT=N+4308|0;N=r+4|0;aV=s+4|0;aW=r+8|0;a0=s+8|0;B=r+12|0;ab=s+12|0;aX=r+16|0;aY=s+16|0;D=r+20|0;n=s+20|0;as=r+24|0;O=s+24|0;aD=r+28|0;aw=s+28|0;$=r+32|0;aC=s+32|0;av=r+36|0;aB=s+36|0;x=r+40|0;az=s+40|0;ay=r+44|0;aA=s+44|0;aq=r+48|0;r=s+48|0;s=0;do{if(aE){X=(s|0)==0;ar=C+(s*516|0)|0;am=0;do{ai=am&1;do{if((c[J+(ai<<2)>>2]|0)!=0&bo){if(!X){break}ag=c[(c[M>>2]|0)+4308>>2]|0;if((ag|0)>0){bq=0}else{break}do{g[b+24636+(am<<8)+(bq<<2)>>2]=+g[b+23612+(am<<8)+(bq<<2)>>2];bq=bq+1|0;}while((bq|0)<(ag|0))}else{if(X&(am|0)<2){EL(b,E+(ai*3072|0)|0,am,e)}if((am|0)==2){ag=ai+1|0;ac=255;while(1){ax=E+(ai*3072|0)+(s<<10)+(ac<<2)|0;bm=+g[ax>>2];ap=E+(ag*3072|0)+(s<<10)+(ac<<2)|0;bn=+g[ap>>2];g[ax>>2]=(bm+bn)*.7071067690849304;g[ap>>2]=(bm-bn)*.7071067690849304;if((ac|0)>0){ac=ac-1|0}else{break}}}bn=+g[E+(ai*3072|0)+(s<<10)>>2];g[ar>>2]=bn*bn;ac=127;while(1){ag=128-ac|0;bn=+g[E+(ai*3072|0)+(s<<10)+(ag<<2)>>2];bm=+g[E+(ai*3072|0)+(s<<10)+(ac+128<<2)>>2];g[C+(s*516|0)+(ag<<2)>>2]=(bn*bn+bm*bm)*.5;if((ac|0)>0){ac=ac-1|0}else{break}}ac=c[M>>2]|0;NM(bc|0,0,256);NM(bd|0,0,256);ag=ac+4308|0;ap=c[ag>>2]|0;ax=(ap|0)>0;if(ax){ao=0;an=0;while(1){aj=c[ac+3876+(ao<<2)>>2]|0;if((aj|0)>0){bm=0.0;bn=0.0;al=an;ak=0;while(1){bl=+g[C+(s*516|0)+(al<<2)>>2];br=bn+bl;bs=bm<bl?bl:bm;y=ak+1|0;if((y|0)<(aj|0)){bm=bs;bn=br;al=al+1|0;ak=y}else{break}}bt=bs;bu=br;bv=aj+an|0}else{bt=0.0;bu=0.0;bv=an}g[F+(am<<8)+(ao<<2)>>2]=bu;g[t+(ao<<2)>>2]=bt;g[u+(ao<<2)>>2]=bu*+g[ac+2672+(ao<<2)>>2];ak=ao+1|0;if((ak|0)<(ap|0)){ao=ak;an=bv}else{break}}bw=+g[au>>2];bx=+g[a7>>2]}else{bw=0.0;bx=0.0}bn=bx+bw;if(bn>0.0){bm=+g[bh>>2];bl=+g[be>>2];an=~~(((bm<bl?bl:bm)*2.0-bn)*20.0/(bn*+((c[ac+3876>>2]|0)-1+(c[ac+3880>>2]|0)|0)));by=(an|0)>8?8:an&255}else{by=0}a[bb]=by;an=ap-1|0;bn=bw+bx;if((an|0)>1){ao=(an|0)>2;ak=1;al=0;bm=bn;bl=bx;while(1){y=ak+1|0;bj=+g[u+(y<<2)>>2];bi=bm+bj;if(bi>0.0){bk=+g[t+(al<<2)>>2];ba=+g[t+(ak<<2)>>2];bf=bk<ba?ba:bk;bk=+g[t+(y<<2)>>2];z=~~(((bf<bk?bk:bf)*3.0-bi)*20.0/(bi*+((c[ac+3876+(al<<2)>>2]|0)-1+(c[ac+3876+(ak<<2)>>2]|0)+(c[ac+3876+(y<<2)>>2]|0)|0)));a[q+ak|0]=(z|0)>8?8:z&255}else{a[q+ak|0]=0}bz=bl+bj;if((y|0)<(an|0)){al=ak;ak=y;bm=bz;bl=bj}else{break}}ak=ao?an:2;bA=ak;bB=ak-1|0;bC=bz}else{bA=1;bB=0;bC=bn}if(bC>0.0){bl=+g[t+(bB<<2)>>2];bm=+g[t+(bA<<2)>>2];ak=~~(((bl<bm?bm:bl)*2.0-bC)*20.0/(bC*+((c[ac+3876+(bB<<2)>>2]|0)-1+(c[ac+3876+(bA<<2)>>2]|0)|0)));a[q+bA|0]=(ak|0)>8?8:ak&255}else{a[q+bA|0]=0}if(ax){ak=ac+4316|0;al=0;ap=0;while(1){y=c[ac+3364+(al<<3)>>2]|0;z=c[ac+3364+(al<<3)+4>>2]|0;ah=c[64624+((d[q+al|0]|0)<<2)>>2]|0;bl=+g[ac+2160+(al<<2)>>2]*+g[v>>2];af=d[q+y|0]|0;ad=c[ak>>2]|0;bm=+g[ad+(ap<<2)>>2]*+g[F+(am<<8)+(y<<2)>>2]*+g[64664+(af<<2)>>2];w=ap+1|0;Z=y+1|0;if((Z|0)>(z|0)){bD=af;bE=2;bF=bm;bG=w}else{y=af;af=2;bj=bm;aG=w;w=Z;Z=ad;while(1){ad=d[q+w|0]|0;bH=ad+y|0;bm=+g[Z+(aG<<2)>>2]*+g[F+(am<<8)+(w<<2)>>2]*+g[64664+(ad<<2)>>2];ad=w-al|0;bi=bj<0.0?0.0:bj;bf=bm<0.0?0.0:bm;do{if(bi>0.0){if(bf<=0.0){bI=bi;break}aH=bf>bi;if(aH){bJ=bf/bi}else{bJ=bi/bf}if((((ad|0)>-1?ad:-ad|0)|0)>(ah|0)){if(bJ<+g[32678]){bI=bi+bf;break}else{bI=aH?bf:bi;break}}else{if(bJ<+g[32680]){bI=(bi+bf)*+g[36976+(~~(+Ek(bJ)*4.816479930623698)<<2)>>2];break}else{bI=bi+bf;break}}}else{bI=bf}}while(0);bK=aG+1|0;ad=w+1|0;if((ad|0)>(z|0)){break}y=bH;af=af+1|0;bj=bI;aG=bK;w=ad;Z=c[ak>>2]|0}bD=bH;bE=af<<1;bF=bI;bG=bK}bj=+g[64664+(((bD<<1|1|0)/(bE|0)|0)<<2)>>2]*.5;bf=bF*bj;Z=G+(am<<8)+(al<<2)|0;g[Z>>2]=bf;w=b+23612+(am<<8)+(al<<2)|0;g[b+24636+(am<<8)+(al<<2)>>2]=+g[w>>2];g[w>>2]=bf;bi=bj*+g[t+(al<<2)>>2]*+g[ac+2416+(al<<2)>>2];if(bf>bi){g[Z>>2]=bi;bL=bi}else{bL=bf}if(bl>1.0){bf=bl*bL;g[Z>>2]=bf;bM=bf}else{bM=bL}bf=+g[F+(am<<8)+(al<<2)>>2];if(bM>bf){g[Z>>2]=bf;bN=bf}else{bN=bM}if(bl<1.0){g[Z>>2]=bl*bN}bO=al+1|0;if((bO|0)<(c[ag>>2]|0)){al=bO;ap=bG}else{break}}if((bO|0)<64){bP=bO}else{break}}else{bP=0}ap=bP+1|0;al=((ap|0)>64?ap:64)-bP<<2;NM(F+(am<<8)+(bP<<2)|0,0,al|0);NM(G+(am<<8)+(bP<<2)|0,0,al|0)}}while(0);am=am+1|0;}while((am|0)<(Y|0))}if(!((c[W>>2]|0)!=1|(a8|0)!=(_|0))){CI(V,at,a5,(c[ae>>2]|0)+468|0,U,+g[S>>2],c[aT>>2]|0)}L22344:do{if(aE){if(bo){bQ=0}else{am=0;while(1){CN((c[M>>2]|0)+2160|0,F+(am<<8)|0,G+(am<<8)|0,aa,bp);g[b+26636+(am*244|0)+88+(s<<2)>>2]=+g[aa>>2];g[b+25660+(am*244|0)+88+(s<<2)>>2]=+g[bp>>2];g[b+26636+(am*244|0)+100+(s<<2)>>2]=+g[N>>2];g[b+25660+(am*244|0)+100+(s<<2)>>2]=+g[aV>>2];g[b+26636+(am*244|0)+112+(s<<2)>>2]=+g[aW>>2];g[b+25660+(am*244|0)+112+(s<<2)>>2]=+g[a0>>2];g[b+26636+(am*244|0)+124+(s<<2)>>2]=+g[B>>2];g[b+25660+(am*244|0)+124+(s<<2)>>2]=+g[ab>>2];g[b+26636+(am*244|0)+136+(s<<2)>>2]=+g[aX>>2];g[b+25660+(am*244|0)+136+(s<<2)>>2]=+g[aY>>2];g[b+26636+(am*244|0)+148+(s<<2)>>2]=+g[D>>2];g[b+25660+(am*244|0)+148+(s<<2)>>2]=+g[n>>2];g[b+26636+(am*244|0)+160+(s<<2)>>2]=+g[as>>2];g[b+25660+(am*244|0)+160+(s<<2)>>2]=+g[O>>2];g[b+26636+(am*244|0)+172+(s<<2)>>2]=+g[aD>>2];g[b+25660+(am*244|0)+172+(s<<2)>>2]=+g[aw>>2];g[b+26636+(am*244|0)+184+(s<<2)>>2]=+g[$>>2];g[b+25660+(am*244|0)+184+(s<<2)>>2]=+g[aC>>2];g[b+26636+(am*244|0)+196+(s<<2)>>2]=+g[av>>2];g[b+25660+(am*244|0)+196+(s<<2)>>2]=+g[aB>>2];g[b+26636+(am*244|0)+208+(s<<2)>>2]=+g[x>>2];g[b+25660+(am*244|0)+208+(s<<2)>>2]=+g[az>>2];g[b+26636+(am*244|0)+220+(s<<2)>>2]=+g[ay>>2];g[b+25660+(am*244|0)+220+(s<<2)>>2]=+g[aA>>2];g[b+26636+(am*244|0)+232+(s<<2)>>2]=+g[aq>>2];g[b+25660+(am*244|0)+232+(s<<2)>>2]=+g[r>>2];am=am+1|0;if((am|0)>=(Y|0)){break L22344}}}do{if((c[J+((bQ&1)<<2)>>2]|0)==0){CN((c[M>>2]|0)+2160|0,F+(bQ<<8)|0,G+(bQ<<8)|0,aa,bp);g[b+26636+(bQ*244|0)+88+(s<<2)>>2]=+g[aa>>2];g[b+25660+(bQ*244|0)+88+(s<<2)>>2]=+g[bp>>2];g[b+26636+(bQ*244|0)+100+(s<<2)>>2]=+g[N>>2];g[b+25660+(bQ*244|0)+100+(s<<2)>>2]=+g[aV>>2];g[b+26636+(bQ*244|0)+112+(s<<2)>>2]=+g[aW>>2];g[b+25660+(bQ*244|0)+112+(s<<2)>>2]=+g[a0>>2];g[b+26636+(bQ*244|0)+124+(s<<2)>>2]=+g[B>>2];g[b+25660+(bQ*244|0)+124+(s<<2)>>2]=+g[ab>>2];g[b+26636+(bQ*244|0)+136+(s<<2)>>2]=+g[aX>>2];g[b+25660+(bQ*244|0)+136+(s<<2)>>2]=+g[aY>>2];g[b+26636+(bQ*244|0)+148+(s<<2)>>2]=+g[D>>2];g[b+25660+(bQ*244|0)+148+(s<<2)>>2]=+g[n>>2];g[b+26636+(bQ*244|0)+160+(s<<2)>>2]=+g[as>>2];g[b+25660+(bQ*244|0)+160+(s<<2)>>2]=+g[O>>2];g[b+26636+(bQ*244|0)+172+(s<<2)>>2]=+g[aD>>2];g[b+25660+(bQ*244|0)+172+(s<<2)>>2]=+g[aw>>2];g[b+26636+(bQ*244|0)+184+(s<<2)>>2]=+g[$>>2];g[b+25660+(bQ*244|0)+184+(s<<2)>>2]=+g[aC>>2];g[b+26636+(bQ*244|0)+196+(s<<2)>>2]=+g[av>>2];g[b+25660+(bQ*244|0)+196+(s<<2)>>2]=+g[aB>>2];g[b+26636+(bQ*244|0)+208+(s<<2)>>2]=+g[x>>2];g[b+25660+(bQ*244|0)+208+(s<<2)>>2]=+g[az>>2];g[b+26636+(bQ*244|0)+220+(s<<2)>>2]=+g[ay>>2];g[b+25660+(bQ*244|0)+220+(s<<2)>>2]=+g[aA>>2];g[b+26636+(bQ*244|0)+232+(s<<2)>>2]=+g[aq>>2];g[b+25660+(bQ*244|0)+232+(s<<2)>>2]=+g[r>>2]}bQ=bQ+1|0;}while((bQ|0)<(Y|0))}}while(0);s=s+1|0;}while((s|0)<3);do{if(aE){s=K|0;bQ=0;do{r=b+27780+(bQ<<2)|0;aq=0;do{aA=b+25660+(bQ*244|0)+88+(aq*12|0)|0;ay=A+(bQ*244|0)+88+(aq*12|0)+4|0;az=A+(bQ*244|0)+88+(aq*12|0)+8|0;x=0;do{U=+g[b+25660+(bQ*244|0)+88+(aq*12|0)+(x<<2)>>2]*.8;aB=(x|0)>0;if(aB){bR=K+(x-1<<2)|0}else{bR=az}bN=+g[bR>>2];av=c[I+(bQ<<4)+(x<<2)>>2]|0;if((av|0)>1){aU=16593}else{if((c[I+(bQ<<4)+(x+1<<2)>>2]|0)==1){aU=16593}else{bS=U}}do{if((aU|0)==16593){aU=0;if(U<=0.0){bS=0.0;break}bS=U*+T(+(bN/U),+.36000001430511475)}}while(0);bl=bS<U?bS:U;L22373:do{if((av|0)==1){if(bl<=0.0){bT=0.0;break}bT=bl*+T(+(bN/bl),+.18000000715255737)}else{if((x|0)==0){if((c[r>>2]|0)==3){aU=16603}else{aU=16600}}else{aU=16600}do{if((aU|0)==16600){aU=0;if(!aB){bT=U;break L22373}if((c[I+(bQ<<4)+(x-1<<2)>>2]|0)!=3){bT=U;break L22373}if((x|0)==1){bU=+g[az>>2];break}else if((x|0)==2){bU=+g[s>>2];break}else if((x|0)==0){aU=16603;break}else{bU=bN;break}}}while(0);if((aU|0)==16603){aU=0;bU=+g[ay>>2]}if(bl<=0.0){bT=0.0;break}bT=bl*+T(+(bU/bl),+.18000000715255737)}}while(0);g[K+(x<<2)>>2]=+g[H+(bQ*12|0)+(x<<2)>>2]*(bT<bl?bT:bl);x=x+1|0;}while((x|0)<3);c[aA>>2]=c[L>>2];c[aA+4>>2]=c[L+4>>2];c[aA+8>>2]=c[L+8>>2];aq=aq+1|0;}while((aq|0)<13);bQ=bQ+1|0;}while((bQ|0)<(Y|0));if(aE){bV=0}else{break}do{c[b+27780+(bV<<2)>>2]=c[I+(bV<<4)+8>>2];bV=bV+1|0;}while((bV|0)<(Y|0))}}while(0);bV=c[P>>2]|0;L22397:do{if((bV|0)>0){P=0;I=a8;while(1){L=b+27796+(P<<2)|0;H=c[L>>2]|0;do{if((I|0)==0){if((H|0)==0){c[L>>2]=1;bW=2;bX=1;break}else if((H|0)==3){c[L>>2]=2;bW=2;bX=2;break}else{bW=2;bX=H;break}}else{bW=(H|0)==2?3:0;bX=H}}while(0);c[o+(P<<2)>>2]=bX;c[L>>2]=bW;H=P+1|0;if((H|0)>=(bV|0)){break L22397}P=H;I=c[J+(H<<2)>>2]|0}}}while(0);if(!aE){i=p;return 0}aE=m-8|0;m=o+4|0;J=(Q|0)==0;bV=0;do{if((bV|0)>1){if((c[o>>2]|0)==2){aU=16625}else{if((c[m>>2]|0)==2){aU=16625}else{bY=0}}if((aU|0)==16625){aU=0;bY=2}bZ=k+(f*976|0)+((bV-2|0)*488|0)|0;b_=bY;b$=aE}else{bZ=j+(f*976|0)+(bV*488|0)|0;b_=c[o+(bV<<2)>>2]|0;b$=l}bT=+g[v>>2];if((b_|0)==2){bW=0;bU=309.07000732421875;while(1){bX=91616+(bW<<2)|0;bS=+g[bZ+88+(bW*12|0)>>2];do{if(bS>0.0){bN=bT*bS;U=+g[bZ+332+(bW*12|0)>>2];if(U<=bN){b0=bU;break}bM=+g[bX>>2];if(U>bN*1.0e10){b0=bU+bM*23.02585092994046;break}else{b0=bU+bM*+Ek(U/bN)*.30102999566398114;break}}else{b0=bU}}while(0);bS=+g[bZ+88+(bW*12|0)+4>>2];do{if(bS>0.0){bN=bT*bS;U=+g[bZ+332+(bW*12|0)+4>>2];if(U<=bN){b1=b0;break}bM=+g[bX>>2];if(U>bN*1.0e10){b1=b0+bM*23.02585092994046;break}else{b1=b0+bM*+Ek(U/bN)*.30102999566398114;break}}else{b1=b0}}while(0);bS=+g[bZ+88+(bW*12|0)+8>>2];do{if(bS>0.0){bN=bT*bS;U=+g[bZ+332+(bW*12|0)+8>>2];if(U<=bN){b2=b1;break}bM=+g[bX>>2];if(U>bN*1.0e10){b2=b1+bM*23.02585092994046;break}else{b2=b1+bM*+Ek(U/bN)*.30102999566398114;break}}else{b2=b1}}while(0);bX=bW+1|0;if(bX>>>0<12){bW=bX;bU=b2}else{break}}g[b$+(bV<<2)>>2]=b2;b3=b2}else{bW=0;bU=281.0574951171875;while(1){bS=+g[bZ+(bW<<2)>>2];do{if(bS>0.0){bN=bT*bS;U=+g[bZ+244+(bW<<2)>>2];if(U<=bN){b4=bU;break}bM=+g[91664+(bW<<2)>>2];if(U>bN*1.0e10){b4=bU+bM*23.02585092994046;break}else{b4=bU+bM*+Ek(U/bN)*.30102999566398114;break}}else{b4=bU}}while(0);bX=bW+1|0;if(bX>>>0<21){bW=bX;bU=b4}else{break}}g[b$+(bV<<2)>>2]=b4;b3=b4}if(!J){h[Q+189240+(f<<5)+(bV<<3)>>3]=b3}bV=bV+1|0;}while((bV|0)<(Y|0));i=p;return 0}function CI(a,b,c,d,e,f,h){a=a|0;b=b|0;c=c|0;d=d|0;e=+e;f=+f;h=h|0;var i=0.0,j=0,k=0,l=0.0,m=0.0,n=0.0,o=0,p=0.0,q=0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0.0,y=0.0;i=f*2.0;if((h|0)<=0){return}if(f>0.0){j=0}else{k=0;do{f=+g[a+512+(k<<2)>>2];l=+g[a+768+(k<<2)>>2];m=+g[b+(k<<2)>>2];n=+g[b+256+(k<<2)>>2];o=b+512+(k<<2)|0;p=+g[o>>2];q=b+768+(k<<2)|0;r=+g[q>>2];if(m>n*1.5800000429153442|n>m*1.5800000429153442){s=r;t=p}else{m=+g[c+(k<<2)>>2];n=l*m;u=f*m;m=r<n?r:n;n=p<u?p:u;s=r>n?r:n;t=p>m?p:m}g[o>>2]=t>f?f:t;g[q>>2]=s>l?l:s;k=k+1|0;}while((k|0)<(h|0));return}do{s=+g[a+512+(j<<2)>>2];t=+g[a+768+(j<<2)>>2];l=+g[b+(j<<2)>>2];f=+g[b+256+(j<<2)>>2];k=b+512+(j<<2)|0;m=+g[k>>2];q=b+768+(j<<2)|0;p=+g[q>>2];if(l>f*1.5800000429153442|f>l*1.5800000429153442){v=p;w=m}else{n=+g[c+(j<<2)>>2];r=t*n;u=s*n;n=p<r?p:r;r=m<u?m:u;v=p>r?p:r;w=m>n?m:n}n=+g[d+(j<<2)>>2]*e;m=l>n?l:n;l=f>n?f:n;f=w>n?w:n;r=v>n?v:n;n=f+r;do{if(n>0.0){p=i*(m<l?m:l);if(p>=n){x=f;y=r;break}u=p/n;x=f*u;y=r*u}else{x=f;y=r}}while(0);r=x<w?x:w;f=y<v?y:v;g[k>>2]=r>s?s:r;g[q>>2]=f>t?t:f;j=j+1|0;}while((j|0)<(h|0));return}function CJ(a){a=a|0;var b=0,d=0,e=0,f=0,h=0,j=0,k=0,l=0.0,m=0.0,n=0,o=0,p=0,q=0,r=0,s=0,t=0.0,u=0.0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0.0,E=0,F=0.0,G=0.0,H=0,I=0,J=0.0,K=0,L=0,M=0,N=0,O=0.0,P=0,Q=0.0,R=0.0,S=0,U=0,V=0.0,W=0,X=0,Y=0;b=i;i=i+768|0;d=b|0;e=b+256|0;f=b+512|0;h=c[a+288>>2]|0;j=h+16|0;k=h+64|0;l=+(c[k>>2]|0);m=0.0- +g[h+280>>2];n=h+85800|0;if((c[n>>2]|0)!=0){o=0;i=b;return o|0}NM(f|0,0,256);p=NE(1,6504)|0;c[n>>2]=p;c[p+6500>>2]=c[a+144>>2];c[h+27800>>2]=0;c[h+27796>>2]=0;n=0;do{q=0;while(1){g[h+21564+(n<<8)+(q<<2)>>2]=100000002004087730000.0;g[h+22588+(n<<8)+(q<<2)>>2]=100000002004087730000.0;g[h+24636+(n<<8)+(q<<2)>>2]=1.0;g[h+23612+(n<<8)+(q<<2)>>2]=1.0;r=q+1|0;if((r|0)<64){q=r}else{s=0;break}}do{g[h+26636+(n*244|0)+(s<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+(s<<2)>>2]=100000002004087730000.0;s=s+1|0;}while((s|0)<22);q=h+27780+(n<<2)|0;r=0;do{g[h+26636+(n*244|0)+88+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+88+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+100+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+100+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+112+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+112+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+124+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+124+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+136+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+136+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+148+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+148+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+160+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+160+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+172+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+172+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+184+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+184+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+196+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+196+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+208+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+208+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+220+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+220+(r<<2)>>2]=100000002004087730000.0;g[h+26636+(n*244|0)+232+(r<<2)>>2]=100000002004087730000.0;g[h+25660+(n*244|0)+232+(r<<2)>>2]=100000002004087730000.0;c[q>>2]=0;r=r+1|0;}while((r|0)<3);g[h+27636+(n*36|0)>>2]=10.0;g[h+27636+(n*36|0)+4>>2]=10.0;g[h+27636+(n*36|0)+8>>2]=10.0;g[h+27636+(n*36|0)+12>>2]=10.0;g[h+27636+(n*36|0)+16>>2]=10.0;g[h+27636+(n*36|0)+20>>2]=10.0;g[h+27636+(n*36|0)+24>>2]=10.0;g[h+27636+(n*36|0)+28>>2]=10.0;g[h+27636+(n*36|0)+32>>2]=10.0;n=n+1|0;}while((n|0)<4);g[h+27616>>2]=0.0;g[h+27612>>2]=0.0;n=p;CK(n,l,1024,576,22,h+21360|0);s=d|0;r=e|0;CL(n,l,1024,s,r);n=p+2148|0;e=c[n>>2]|0;if((e|0)>0){q=0;do{t=+g[d+(q<<2)>>2];if(t<13.0){u=0.0}else{u=(t+-13.0)*0.0/11.0+(24.0-t)*0.0/11.0}g[f+(q<<2)>>2]=+T(+10.0,+(u/10.0));q=q+1|0;}while((q|0)<(e|0))}q=p+1204|0;v=q;w=f|0;x=CM(p+2156|0,q,e,s,r,w)|0;if((x|0)!=0){o=x;i=b;return o|0}if((c[n>>2]|0)>0){x=p+1716|0;e=h+85796|0;u=m;q=p+256|0;y=0;z=0;while(1){A=x+(z<<2)|0;if((c[A>>2]|0)>0){B=0;t=1.0e+37;C=y;do{D=+T(+10.0,+((+D2(j,l*+(C|0)/1024.0e3*1.0e3)+-20.0)*.1));E=c[A>>2]|0;F=D*+(E|0);t=t>F?F:t;B=B+1|0;C=C+1|0;}while((B|0)<(E|0));G=t;H=C}else{G=9.999999933815813e+36;H=y}g[(c[e>>2]|0)+212+(z<<2)>>2]=G;F=(+g[d+(z<<2)>>2]/10.0+-1.0)*20.0;D=F>6.0?30.0:F;F=+T(+10.0,+((((c[k>>2]|0)<44e3?30.0:D<u?u:D)+-8.0)/10.0));g[q+(z<<2)>>2]=F*+(c[A>>2]|0);B=z+1|0;if((B|0)<(c[n>>2]|0)){y=H;z=B}else{break}}}z=p+2160|0;H=z;y=h+21452|0;CK(H,l,256,192,13,y);CL(H,l,256,s,r);H=p+4308|0;q=c[H>>2]|0;if((q|0)>0){e=p+3876|0;x=h+85796|0;u=m;B=p+2416|0;E=0;I=0;while(1){m=+g[d+(I<<2)>>2];if(m<13.0){J=-8.25}else{J=(m+-13.0)*-4.5/11.0+(24.0-m)*-8.25/11.0}g[f+(I<<2)>>2]=+T(+10.0,+(J/10.0));K=e+(I<<2)|0;if((c[K>>2]|0)>0){L=0;G=1.0e+37;M=E;do{F=+T(+10.0,+((+D2(j,l*+(M|0)/256.0e3*1.0e3)+-20.0)*.1));N=c[K>>2]|0;D=F*+(N|0);G=G>D?D:G;L=L+1|0;M=M+1|0;}while((L|0)<(N|0));O=G;P=M}else{O=9.999999933815813e+36;P=E}g[(c[x>>2]|0)+468+(I<<2)>>2]=O;t=(m/12.0+-1.0)*7.0;if(m>12.0){Q=t*(+aa(+(t+1.0))*3.1+1.0)}else{Q=t}if(m<12.0){R=Q*(+aa(+(1.0-Q))*2.3+1.0)}else{R=Q}t=R>6.0?30.0:R;D=+T(+10.0,+((((c[k>>2]|0)<44e3?30.0:t<u?u:t)+-8.0)/10.0));g[B+(I<<2)>>2]=D*+(c[K>>2]|0);L=I+1|0;A=c[H>>2]|0;if((L|0)<(A|0)){E=P;I=L}else{S=A;break}}}else{S=q}q=CM(p+4316|0,p+3364|0,S,s,r,w)|0;if((q|0)!=0){o=q;i=b;return o|0}g[32680]=3.6517412662506104;g[32678]=31.62277603149414;EN(h);u=l;g[p+6496>>2]=+$(+(-2.302585092994046/(u*.01/192.0)));q=h+192|0;R=+g[q>>2];g[q>>2]=R==0.0?(c[h+96>>2]|0)==0?3.5:1.0:R;q=c[n>>2]|0;if((q|0)>0){w=0;do{r=v+(w<<3)+4|0;s=q-1|0;if((c[r>>2]|0)>(s|0)){c[r>>2]=s}w=w+1|0;}while((w|0)<(q|0))}R=+T(+10.0,+(+(c[h+76>>2]|0)*576.0/u*-1.2));q=h+85796|0;g[(c[q>>2]|0)+16>>2]=R;g[(c[q>>2]|0)+8>>2]=.009999999776482582;g[(c[q>>2]|0)+12>>2]=1.0;L22548:do{if((c[h+208>>2]|0)!=-1){R=+(c[k>>2]|0)*.0009765625;u=0.0;Q=0.0;w=0;do{u=R+u;O=1.0/+T(+10.0,+(+D2(j,u)/10.0));g[(c[q>>2]|0)+724+(w<<2)>>2]=O;U=c[q>>2]|0;Q=Q+ +g[U+724+(w<<2)>>2];w=w+1|0;}while((w|0)<512);u=1.0/Q;w=511;K=U;while(1){M=K+724+(w<<2)|0;g[M>>2]=u*+g[M>>2];M=w-1|0;if((M|0)<=-1){break L22548}w=M;K=c[q>>2]|0}}}while(0);q=c[H>>2]|0;u=+g[a+264>>2];Q=+g[a+268>>2];R=u<0.0?4.400000095367432:u;g[p+6488>>2]=R;g[p+6484>>2]=R;g[p+6480>>2]=R;g[p+6492>>2]=Q<0.0?25.0:Q;U=c[a+164>>2]|0;if((U|0)<4){V=-7.400000095367432}else{Q=+g[90224+(U<<2)>>2];V=Q+ +g[a+160>>2]*(Q- +g[90224+(U+1<<2)>>2])}if((q|0)>0){U=z;a=0;j=q;do{g[U+(a<<2)>>2]=+T(+10.0,+(V*(+(j-a|0)/+(j|0))*.10000000149011612));a=a+1|0;j=c[H>>2]|0;}while((a|0)<(j|0));if((a|0)<64){W=a;X=16721}}else{W=0;X=16721}if((X|0)==16721){a=z;z=W;do{g[a+(z<<2)>>2]=1.0;z=z+1|0;}while((z|0)<64)}z=c[n>>2]|0;if((z|0)>0){a=p;W=0;j=z;do{g[a+(W<<2)>>2]=+T(+10.0,+(V*(+(j-W|0)/+(j|0))*.10000000149011612));W=W+1|0;j=c[n>>2]|0;}while((W|0)<(j|0));if((W|0)<64){Y=W;X=16727}}else{Y=0;X=16727}if((X|0)==16727){X=p;W=Y;do{g[X+(W<<2)>>2]=1.0;W=W+1|0;}while((W|0)<64)}W=p+4320|0;X=W;NN(W|0,p|0,2160)|0;CK(X,l,1024,192,13,y);o=0;i=b;return o|0}function CK(a,b,d,e,f,h){a=a|0;b=+b;d=d|0;e=e|0;f=f|0;h=h|0;var j=0,k=0,l=0,m=0.0,n=0.0,o=0.0,p=0.0,q=0,r=0,s=0,t=0,u=0,v=0.0,w=0,x=0,y=0,z=0,A=0,B=0,C=0.0;j=i;i=i+2320|0;k=j|0;l=j+264|0;m=+(e|0)*2.0;n=b/m;o=+(d|0);p=o/m;NM(l|0,0,2052);m=b/o;e=(d|0)/2|0;d=0;q=0;while(1){if((q|0)>=64){r=d;s=q;break}o=m*+(d|0);b=+D3(o);g[k+(q<<2)>>2]=o;t=d;while(1){if(+D3(m*+(t|0))-b>=.34|(t|0)>(e|0)){break}else{t=t+1|0}}u=t-d|0;c[a+1716+(q<<2)>>2]=u;if((u|0)>0){v=1.0/+(u|0)}else{v=0.0}g[a+512+(q<<2)>>2]=v;u=q+1|0;if((d|0)<(t|0)){w=d;while(1){x=w+1|0;c[l+(w<<2)>>2]=q;if((x|0)<(t|0)){w=x}else{y=t;break}}}else{y=d}if((y|0)>(e|0)){r=e;s=u;break}else{d=y;q=u}}g[k+(s<<2)>>2]=m*+(r|0);c[a+2152>>2]=f;r=a+2148|0;c[r>>2]=s;if((s|0)>0){s=0;q=0;while(1){y=c[a+1716+(q<<2)>>2]|0;v=+D3(m*+(((y|0)/2|0)+s|0));g[a+768+(q<<2)>>2]=+T(+10.0,+((1.0- +U(+((v<15.5?v:15.5)/15.5*3.141592653589793)))*1.25+ -2.5));z=q+1|0;if((z|0)<(c[r>>2]|0)){s=y+s|0;q=z}else{break}}if((z|0)<64){A=z;B=16750}}else{A=0;B=16750}if((B|0)==16750){while(1){B=0;g[a+768+(A<<2)>>2]=1.0;z=A+1|0;if((z|0)<64){A=z;B=16750}else{break}}}if((f|0)<=0){i=j;return}m=p;B=0;while(1){A=c[h+(B<<2)>>2]|0;z=B+1|0;q=c[h+(z<<2)>>2]|0;s=~~+Q(+(m*(+(A|0)+-.5)+.5));r=~~+Q(+(m*(+(q|0)+-.5)+.5));y=c[l+(((r|0)>(e|0)?e:r)<<2)>>2]|0;c[a+1972+(B<<2)>>2]=((c[l+(((s|0)<0?0:s)<<2)>>2]|0)+y|0)/2|0;c[a+2060+(B<<2)>>2]=y;p=+g[k+(y<<2)>>2];v=(n*+(q|0)-p)/(+g[k+(y+1<<2)>>2]-p);do{if(v<0.0){C=0.0}else{if(v<=1.0){C=v;break}C=1.0}}while(0);g[a+1112+(B<<2)>>2]=C;v=+D3(n*+(A|0));g[a+1024+(B<<2)>>2]=+T(+10.0,+((1.0- +U(+((v<15.5?v:15.5)/15.5*3.141592653589793)))*1.25+ -2.5));if((z|0)<(f|0)){B=z}else{break}}i=j;return}function CL(a,b,d,e,f){a=a|0;b=+b;d=d|0;e=e|0;f=f|0;var h=0,i=0.0,j=0,k=0,l=0.0,m=0;h=c[a+2148>>2]|0;i=b/+(d|0);if((h|0)<=0){return}b=i;d=0;j=0;while(1){k=c[a+1716+(j<<2)>>2]|0;l=+D3(i*+(d|0));m=k+d|0;g[e+(j<<2)>>2]=(l+ +D3(i*+(m-1|0)))*.5;l=+D3(b*(+(d|0)+-.5));g[f+(j<<2)>>2]=+D3(b*(+(m|0)+-.5))-l;k=j+1|0;if((k|0)<(h|0)){d=m;j=k}else{break}}return}function CM(a,b,d,e,f,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;var j=0,k=0,l=0,m=0,n=0.0,o=0.0,p=0,q=0.0,r=0.0,s=0.0,t=0.0,u=0.0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;j=i;i=i+16384|0;k=j|0;NM(k|0,0,16384);l=(d|0)>0;do{if(l){m=0;do{n=+g[e+(m<<2)>>2];o=+g[h+(m<<2)>>2];p=0;do{q=n- +g[e+(p<<2)>>2];if(q<0.0){r=q*1.5}else{r=q*3.0}if(r<.5|r>2.5){s=0.0}else{q=r+-.5;s=(q*q-q*2.0)*8.0}q=r+.474;t=q*7.5+15.811389- +S(+(q*q+1.0))*17.5;if(t>-60.0){u=+$(+((s+t)*.23025850929940458))/.6609193}else{u=0.0}g[k+(m<<8)+(p<<2)>>2]=u*+g[f+(p<<2)>>2]*o;p=p+1|0;}while((p|0)<(d|0));m=m+1|0;}while((m|0)<(d|0));if(l){v=0;w=0}else{x=0;break}do{m=0;while(1){if((m|0)>=(d|0)){break}if(+g[k+(v<<8)+(m<<2)>>2]>0.0){break}else{m=m+1|0}}c[b+(v<<3)>>2]=m;p=d;while(1){y=p-1|0;if((y|0)<=0){break}if(+g[k+(v<<8)+(y<<2)>>2]>0.0){break}else{p=y}}c[b+(v<<3)+4>>2]=y;w=w+p-m|0;v=v+1|0;}while((v|0)<(d|0));x=w<<2}else{x=0}}while(0);w=NC(x)|0;c[a>>2]=w;x=(w|0)==0;if(x|l^1){z=x<<31>>31;i=j;return z|0}else{A=0;B=0}while(1){x=c[b+(A<<3)>>2]|0;l=b+(A<<3)+4|0;if((x|0)>(c[l>>2]|0)){C=B}else{w=x;x=B;while(1){v=x+1|0;g[(c[a>>2]|0)+(x<<2)>>2]=+g[k+(A<<8)+(w<<2)>>2];y=w+1|0;if((y|0)>(c[l>>2]|0)){C=v;break}else{w=y;x=v}}}x=A+1|0;if((x|0)<(d|0)){A=x;B=C}else{z=0;break}}i=j;return z|0}function CN(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0.0,m=0.0,n=0,o=0,p=0,q=0,r=0,s=0.0,t=0.0,u=0,v=0.0,w=0.0,x=0.0,y=0.0,z=0.0,A=0,B=0;h=c[a+2152>>2]|0;L22661:do{if((h|0)>0){i=a+2148|0;j=0;k=0;l=0.0;m=0.0;while(1){n=c[a+2060+(k<<2)>>2]|0;o=c[i>>2]|0;p=(n|0)<(o|0)?n:o;if((j|0)<(p|0)){q=~n;n=~o;r=(q|0)>(n|0)?q:n;n=j;s=l;t=m;do{t=t+ +g[b+(n<<2)>>2];s=s+ +g[d+(n<<2)>>2];n=n+1|0;}while((n|0)<(p|0));u=~r;v=s;w=t}else{u=j;v=l;w=m}if((u|0)>=(o|0)){break}x=+g[a+1112+(k<<2)>>2];y=1.0-x;p=b+(u<<2)|0;n=d+(u<<2)|0;z=v+x*+g[n>>2];g[e+(k<<2)>>2]=w+x*+g[p>>2];g[f+(k<<2)>>2]=z;q=k+1|0;if((q|0)<(h|0)){j=u+1|0;k=q;l=y*+g[n>>2];m=y*+g[p>>2]}else{A=q;break L22661}}g[e+(k<<2)>>2]=w;g[f+(k<<2)>>2]=v;A=k+1|0}else{A=0}}while(0);if((A|0)<(h|0)){B=A}else{return}do{g[e+(B<<2)>>2]=0.0;g[f+(B<<2)>>2]=0.0;B=B+1|0;}while((B|0)<(h|0));return}function CO(a){a=a|0;c[a+85824>>2]=64;return}function CP(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0,h=0.0,i=0.0,j=0.0,k=0;g[d>>2]=0.0;if((c|0)<0){return}e=a+4764|0;f=0;h=0.0;while(1){i=+R(+(+g[a+(f<<2)>>2]));g[d>>2]=i+h;j=i;i=+S(+(j*+S(+j)));g[b+(f<<2)>>2]=i;if(i>+g[e>>2]){g[e>>2]=i}k=f+1|0;if((k|0)>(c|0)){break}f=k;h=+g[d>>2]}return}function CQ(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0.0,K=0,L=0.0,M=0,N=0,O=0,P=0.0,Q=0,R=0,S=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0;f=i;i=i+10616|0;h=f|0;j=f+5256|0;k=f+7560|0;l=f+7568|0;m=f+7576|0;n=f+7584|0;o=f+8208|0;p=f+10512|0;q=f+10576|0;r=f+10592|0;s=f+10608|0;t=a+116|0;u=a+84744|0;c[u>>2]=c[t>>2];v=C3(a,m)|0;w=a+76|0;c[m>>2]=(v|0)/(c[w>>2]|0)|0;v=a+112|0;c[u>>2]=c[v>>2];Ey(a)|0;c[u>>2]=1;c[l>>2]=Ey(a)|0;if((c[t>>2]|0)>=1){x=1;do{c[u>>2]=x;c[p+(x<<2)>>2]=C3(a,l)|0;x=x+1|0;}while((x|0)<=(c[t>>2]|0))}x=c[w>>2]|0;L22696:do{if((x|0)>0){l=a+84756|0;y=a+72|0;z=a+84916|0;A=a+84908|0;B=a+84912|0;C=0;D=1;E=0;while(1){F=r+(C<<3)|0;G=CZ(a,b,F,c[m>>2]|0,C,0)|0;if((c[l>>2]|0)==2){H=0;do{I=a+304+(C*10504|0)+(H<<2)|0;J=+g[I>>2];K=a+304+(C*10504|0)+5252+(H<<2)|0;L=+g[K>>2];g[I>>2]=(J+L)*.7071067690849304;g[K>>2]=(J-L)*.7071067690849304;H=H+1|0;}while((H|0)<576);C_(F,+g[d+(C<<2)>>2],c[m>>2]|0,G)}H=c[y>>2]|0;if((H|0)>0){K=0;I=D;M=E;while(1){N=a+304+(C*10504|0)+(K*5252|0)|0;O=(c[a+304+(C*10504|0)+(K*5252|0)+4788>>2]|0)==2;L=+$(+(3.5- +g[b+(C<<3)+(K<<2)>>2]/300.0))+1.0;if(O){P=+g[z>>2]-(2.56/L+-.14)}else{P=+g[B>>2]-(1.28/L+-.05)}g[A>>2]=+T(+10.0,+(P*.1));CU(a,N);O=(C0(a,e+(C*976|0)+(K*488|0)|0,N,n+(C*312|0)+(K*156|0)|0)|0)==0?I:0;c[q+(C<<3)+(K<<2)>>2]=126;N=(c[r+(C<<3)+(K<<2)>>2]|0)+M|0;Q=K+1|0;R=c[y>>2]|0;if((Q|0)<(R|0)){K=Q;I=O;M=N}else{S=O;U=N;V=R;break}}}else{S=D;U=E;V=H}M=C+1|0;W=c[w>>2]|0;if((M|0)<(W|0)){C=M;D=S;E=U}else{break}}if((W|0)<=0){X=S;Y=W;Z=y;break}if((U|0)>0){_=0}else{E=0;D=V;while(1){if((D|0)>0){C=0;A=D;while(1){B=q+(E<<3)+(C<<2)|0;z=c[r+(E<<3)+(C<<2)>>2]|0;if((c[B>>2]|0)>(z|0)){c[B>>2]=z;aa=V}else{aa=A}z=C+1|0;if((z|0)<(aa|0)){C=z;A=aa}else{ab=aa;break}}}else{ab=D}A=E+1|0;if((A|0)<(W|0)){E=A;D=ab}else{X=S;Y=W;Z=y;break L22696}}}while(1){if((V|0)>0){D=c[p+(c[t>>2]<<2)>>2]|0;E=(U|0)>(D|0);A=0;do{C=r+(_<<3)+(A<<2)|0;H=c[C>>2]|0;if(E){z=(ac(H,D)|0)/(U|0)|0;c[C>>2]=z;ad=z}else{ad=H}H=q+(_<<3)+(A<<2)|0;if((c[H>>2]|0)>(ad|0)){c[H>>2]=ad}A=A+1|0;}while((A|0)<(V|0))}A=_+1|0;if((A|0)<(W|0)){_=A}else{X=S;Y=W;Z=y;break}}}else{X=1;Y=x;Z=a+72|0}}while(0);x=(X|0)==0;X=a+124|0;W=o|0;S=a+85824|0;_=a+85096|0;V=h;ad=j;j=a+85092|0;U=h+2304|0;h=o;ab=0;aa=0;e=Y;L22740:while(1){if((aa|0)>=(e|0)){if(x){ae=16868}else{if((c[X>>2]|0)==0){af=1}else{ae=16868}}if((ae|0)==16868){ae=0;af=c[v>>2]|0}c[u>>2]=af;Y=c[t>>2]|0;L22749:do{if((af|0)<(Y|0)){b=af;do{if((ab|0)<=(c[p+(b<<2)>>2]|0)){break L22749}b=b+1|0;c[u>>2]=b;}while((b|0)<(Y|0))}}while(0);Y=(ab|0)>(C3(a,s)|0);ag=c[w>>2]|0;ah=(ag|0)>0;if(!Y){break}if(!ah){ab=0;aa=0;e=ag;continue}Y=0;b=c[Z>>2]|0;m=ag;while(1){if((b|0)>0){d=0;do{y=n+(Y*312|0)+(d*156|0)|0;A=a+304+(Y*10504|0)+(d*5252|0)+4856|0;if((c[A>>2]|0)>0){D=0;E=y;while(1){P=+(D|0);H=E+4|0;g[E>>2]=(P*P*.029/22.0/22.0+1.0)*+g[E>>2];z=D+1|0;if((z|0)<(c[A>>2]|0)){D=z;E=H}else{ai=H;break}}}else{ai=y}do{if((c[a+304+(Y*10504|0)+(d*5252|0)+4788>>2]|0)==2){E=c[a+304+(Y*10504|0)+(d*5252|0)+4852>>2]|0;if((E|0)<13){aj=E;ak=ai}else{break}while(1){P=+(aj|0);L=P*P*.029/13.0/13.0+1.0;E=ak+4|0;g[ak>>2]=L*+g[ak>>2];D=ak+8|0;g[E>>2]=L*+g[E>>2];g[D>>2]=L*+g[D>>2];D=aj+1|0;if((D|0)<13){aj=D;ak=ak+12|0}else{break}}}}while(0);L=+(c[q+(Y<<3)+(d<<2)>>2]|0);y=r+(Y<<3)+(d<<2)|0;P=+(c[y>>2]|0)*.9;c[y>>2]=~~(L>P?L:P);d=d+1|0;al=c[Z>>2]|0;}while((d|0)<(al|0));am=al;an=c[w>>2]|0}else{am=b;an=m}d=Y+1|0;if((d|0)<(an|0)){Y=d;b=am;m=an}else{ab=0;aa=0;e=an;continue L22740}}}if((c[Z>>2]|0)>0){m=0;b=ab;while(1){Y=a+304+(aa*10504|0)+(m*5252|0)|0;g[k>>2]=0.0;d=c[a+304+(aa*10504|0)+(m*5252|0)+5208>>2]|0;g[a+304+(aa*10504|0)+(m*5252|0)+4764>>2]=0.0;NM(o+(d<<2)|0,0,576-d<<2|0);dq[c[S>>2]&127](Y,W,d,k);do{if(+g[k>>2]>9.999999682655225e-21){d=(c[_>>2]|0)>>>1&1;y=a+304+(aa*10504|0)+(m*5252|0)+4864|0;if((c[y>>2]|0)>0){D=0;do{c[a+84936+(D<<2)>>2]=d;D=D+1|0;}while((D|0)<(c[y>>2]|0))}y=c[r+(aa<<3)+(m<<2)>>2]|0;if((y|0)==0){ao=b;break}D=n+(aa*312|0)+(m*156|0)|0;d=c[q+(aa<<3)+(m<<2)>>2]|0;E=c[j>>2]|0;NM(U|0,0,2304);A=y-42|0;H=a+304+(aa*10504|0)+(m*5252|0)+4768|0;z=Y;C=d;B=0;l=y;M=(d+y|0)/2|0;while(1){c[j>>2]=(M|0)>(A|0)?0:E;do{if((CV(a,Y,D,W,m,M)|0)<1){y=c[H>>2]|0;NN(V|0,z|0,5252)|0;NN(ad|0,h|0,2304)|0;d=y-32|0;ap=C;aq=1;ar=d;as=d-C|0;at=(d+C|0)/2|0}else{d=M+32|0;y=l-d|0;I=(l+d|0)/2|0;if((B|0)==0){ap=d;aq=0;ar=l;as=y;at=I;break}NN(z|0,V|0,5252)|0;NN(h|0,ad|0,2304)|0;ap=d;aq=2;ar=l;as=y;at=I}}while(0);if((as|0)>12){C=ap;B=aq;l=ar;M=at}else{break}}c[j>>2]=E;if((aq|0)==2){M=a+304+(aa*10504|0)+(m*5252|0)+2304|0;NN(M|0,U|0,2304)|0}if((c[_>>2]&1|0)!=0){CR(a,Y,D,W)}ao=(c[H>>2]|0)+b+(c[a+304+(aa*10504|0)+(m*5252|0)+4844>>2]|0)|0}else{NM(a+304+(aa*10504|0)+(m*5252|0)+2304|0,0,2304);ao=b}}while(0);Y=m+1|0;if((Y|0)<(c[Z>>2]|0)){m=Y;b=ao}else{break}}au=ao;av=c[w>>2]|0}else{au=ab;av=e}ab=au;aa=aa+1|0;e=av}if(!ah){aw=c[s>>2]|0;C6(a,aw);i=f;return}ah=a+304|0;av=a+36|0;e=0;aa=c[Z>>2]|0;au=ag;while(1){if((aa|0)>0){ag=0;do{ab=a+304+(e*10504|0)+(ag*5252|0)|0;DT(a,e,ag,ah);if((c[av>>2]|0)==1){DR(a,ab)}C5(a,ab);ag=ag+1|0;ax=c[Z>>2]|0;}while((ag|0)<(ax|0));ay=ax;az=c[w>>2]|0}else{ay=aa;az=au}ag=e+1|0;if((ag|0)<(az|0)){e=ag;aa=ay;au=az}else{break}}aw=c[s>>2]|0;C6(a,aw);i=f;return}function CR(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,j=0,k=0,l=0.0,m=0,n=0,o=0,p=0,q=0.0,r=0,s=0,t=0.0,u=0,v=0.0,w=0,x=0,y=0.0,z=0.0,A=0.0,B=0,C=0,D=0,E=0,F=0,G=0,H=0;f=i;i=i+184|0;h=f|0;j=f+160|0;k=c[a+85096>>2]|0;do{if((k&4|0)==0){if((c[b+4788>>2]|0)!=2){break}i=f;return}}while(0);if((k&128|0)!=0){i=f;return}C1(b,d,h|0,j,0)|0;j=0;do{if((c[b+2304+(j<<2)>>2]|0)==0){l=0.0}else{l=+R(+(+g[b+(j<<2)>>2]))}g[e+(j<<2)>>2]=l;j=j+1|0;}while((j|0)<576);j=b+4864|0;k=(c[b+4788>>2]|0)==2?6:8;m=0;while(1){n=c[b+4872+(k<<2)>>2]|0;o=n+m|0;p=h+(k<<2)|0;L22832:do{if(+g[p>>2]<1.0){b6(e+(m<<2)|0,n|0,4,24);l=+g[e+(o-1<<2)>>2];if(l==0.0){if(l==0.0){break}}else{q=+R(+l);if(q<=q*9.999999974752427e-7){break}}r=0;q=(1.0- +g[p>>2])*+g[d+(k<<2)>>2];while(1){s=r+1|0;L22840:do{if((s|0)<(n|0)){l=+g[e+(r+o-n<<2)>>2];t=+R(+l);u=r+m|0;v=t*9.999999974752427e-7;w=1;x=s;while(1){y=+g[e+(u+w<<2)>>2];z=+R(+y);A=+R(+(l-y));if(t>z){if(A>v){B=w;C=x;D=1;E=u;break L22840}}else{if(A>z*9.999999974752427e-7){B=w;C=x;D=1;E=u;break L22840}}F=w+1|0;G=F+r|0;if((G|0)<(n|0)){w=F;x=G}else{B=F;C=G;D=0;E=u;break}}}else{B=1;C=s;D=0;E=m+r|0}}while(0);v=+g[e+(E<<2)>>2];t=+(B|0)*v*v;if(q<t){break}if(D){r=C;q=q-t}else{break L22832}}if((r|0)==0){break}q=+g[e+(E-1<<2)>>2];if(q==0.0){if(q==0.0){break}else{H=n}}else{t=+R(+q);if(t>t*9.999999974752427e-7){H=n}else{break}}do{s=o-H|0;if(+R(+(+g[b+(s<<2)>>2]))<=q){c[b+2304+(s<<2)>>2]=0}H=H-1|0;}while((H|0)>0)}}while(0);n=k+1|0;if((n|0)<(c[j>>2]|0)){k=n;m=o}else{break}}c[b+4768>>2]=DQ(a,b,0)|0;i=f;return}function CS(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0.0,G=0,H=0.0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0;d=i;i=i+9952|0;f=d|0;h=d+8|0;j=d+16|0;k=d+24|0;l=d+648|0;m=d+9864|0;n=d+9928|0;o=d+9944|0;p=k|0;q=l|0;r=n|0;NM(l|0,0,9216);s=a+152|0;if((c[s>>2]|0)==0){t=a+116|0;u=a+84744|0;c[u>>2]=c[t>>2];C3(a,j)|0;v=c[a+52144>>2]|0;c[u>>2]=c[a+112>>2];Ey(a)|0;c[u>>2]=1;c[h>>2]=Ey(a)|0;w=c[t>>2]|0;if((w|0)<1){x=w}else{w=1;while(1){c[u>>2]=w;c[m+(w<<2)>>2]=C3(a,h)|0;y=w+1|0;z=c[t>>2]|0;if((y|0)>(z|0)){x=z;break}else{w=y}}}A=c[m+(x<<2)>>2]|0;B=v}else{c[a+84744>>2]=0;v=C3(a,j)|0;c[m>>2]=v;A=v;B=c[a+52144>>2]|0}v=a+76|0;x=c[v>>2]|0;if((x|0)>0){w=a+84756|0;t=a+72|0;h=a+84912|0;u=a+84908|0;y=1;z=0;C=0;while(1){CZ(a,b,n+(C<<3)|0,c[j>>2]|0,C,0)|0;if((c[w>>2]|0)==2){D=0;do{E=a+304+(C*10504|0)+(D<<2)|0;F=+g[E>>2];G=a+304+(C*10504|0)+5252+(D<<2)|0;H=+g[G>>2];g[E>>2]=(F+H)*.7071067690849304;g[G>>2]=(F-H)*.7071067690849304;D=D+1|0;}while((D|0)<576)}D=c[t>>2]|0;if((D|0)>0){G=0;E=y;I=z;while(1){J=a+304+(C*10504|0)+(G*5252|0)|0;g[u>>2]=+T(+10.0,+(+g[h>>2]*.1));CU(a,J);K=(C0(a,e+(C*976|0)+(G*488|0)|0,J,k+(C*312|0)+(G*156|0)|0)|0)==0?E:0;J=(c[n+(C<<3)+(G<<2)>>2]|0)+I|0;L=G+1|0;M=c[t>>2]|0;if((L|0)<(M|0)){G=L;E=K;I=J}else{N=K;O=J;P=M;break}}}else{N=y;O=z;P=D}I=C+1|0;Q=c[v>>2]|0;if((I|0)<(Q|0)){y=N;z=O;C=I}else{break}}if((Q|0)>0){C=(O|0)>(A|0)&(O|0)>0;z=0;y=P;while(1){if((y|0)>0){t=0;k=y;while(1){if(C){e=n+(z<<3)+(t<<2)|0;c[e>>2]=(ac(c[e>>2]|0,A)|0)/(O|0)|0;R=P}else{R=k}e=t+1|0;if((e|0)<(R|0)){t=e;k=R}else{S=R;break}}}else{S=y}k=z+1|0;if((k|0)<(Q|0)){z=k;y=S}else{break}}}if((N|0)==0){U=0;V=B;W=Q}else{X=N;Y=Q;Z=16950}}else{X=1;Y=x;Z=16950}if((Z|0)==16950){U=X;V=0;W=Y}if((W|0)>0){Y=a+72|0;X=a+85824|0;x=a+85096|0;Q=0;N=c[Y>>2]|0;B=W;while(1){if((N|0)>0){W=0;do{g[f>>2]=0.0;S=c[a+304+(Q*10504|0)+(W*5252|0)+5208>>2]|0;g[a+304+(Q*10504|0)+(W*5252|0)+4764>>2]=0.0;NM(l+(Q*4608|0)+(W*2304|0)+(S<<2)|0,0,576-S<<2|0);dq[c[X>>2]&127](a+304+(Q*10504|0)+(W*5252|0)|0,l+(Q*4608|0)+(W*2304|0)|0,S,f);do{if(+g[f>>2]>9.999999682655225e-21){S=(c[x>>2]|0)>>>1&1;y=a+304+(Q*10504|0)+(W*5252|0)+4864|0;if((c[y>>2]|0)>0){_=0}else{break}do{c[a+84936+(_<<2)>>2]=S;_=_+1|0;}while((_|0)<(c[y>>2]|0))}else{NM(a+304+(Q*10504|0)+(W*5252|0)+2304|0,0,2304);c[n+(Q<<3)+(W<<2)>>2]=0}}while(0);W=W+1|0;$=c[Y>>2]|0;}while((W|0)<($|0));aa=$;ab=c[v>>2]|0}else{aa=N;ab=B}W=Q+1|0;if((W|0)<(ab|0)){Q=W;N=aa;B=ab}else{break}}}ab=El(a,q,p,r)|0;do{if((c[s>>2]|0)==0){if((U|0)==0){Z=16964}else{if((c[a+124>>2]|0)==0){ad=1}else{Z=16964}}if((Z|0)==16964){ad=c[a+112>>2]|0}r=c[a+116>>2]|0;p=ad;while(1){if((p|0)>=(r|0)){break}if((ab|0)>(c[m+(p<<2)>>2]|0)){p=p+1|0}else{break}}q=(p|0)>(r|0)?r:p;if((V|0)>0){ae=r}else{c[a+84744>>2]=q;af=q;break}while(1){if((ae|0)<=(q|0)){break}if(((c[m+(ae<<2)>>2]|0)-ab|0)>(V|0)){ae=ae-1|0}else{break}}c[a+84744>>2]=ae;af=ae}else{c[a+84744>>2]=0;af=0}}while(0);if((ab|0)>(c[m+(af<<2)>>2]|0)){Ed(a,1618656,(af=i,i=i+1|0,i=i+7&-8,c[af>>2]=0,af)|0);i=af;bp(-1|0)}C3(a,o)|0;af=c[v>>2]|0;if((af|0)<=0){ag=c[o>>2]|0;C6(a,ag);i=d;return}m=a+72|0;ab=0;ae=c[m>>2]|0;V=af;while(1){if((ae|0)>0){af=0;do{C5(a,a+304+(ab*10504|0)+(af*5252|0)|0);af=af+1|0;ah=c[m>>2]|0;}while((af|0)<(ah|0));ai=ah;aj=c[v>>2]|0}else{ai=ae;aj=V}af=ab+1|0;if((af|0)<(aj|0)){ab=af;ae=ai;V=aj}else{break}}ag=c[o>>2]|0;C6(a,ag);i=d;return}function CT(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0.0,E=0.0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,U=0,V=0,W=0;f=i;i=i+2504|0;h=f|0;j=f+8|0;k=f+16|0;l=f+176|0;m=f+2480|0;n=f+2496|0;o=a+304|0;c[n>>2]=0;p=a+76|0;q=c[p>>2]|0;r=a+116|0;s=a+84744|0;c[s>>2]=c[r>>2];t=C3(a,j)|0;c[s>>2]=1;u=Ey(a)|0;v=c[a+24>>2]<<3;w=c[p>>2]|0;x=a+72|0;y=c[x>>2]|0;z=ac(y,w)|0;A=(u-v|0)/(z|0)|0;u=ac(q*576e3|0,c[a+108>>2]|0)|0;c[j>>2]=u;q=a+85096|0;if((c[q>>2]&1|0)==0){B=u}else{C=~~(+(u|0)*1.09);c[j>>2]=C;B=C}C=(((B|0)/(c[a+64>>2]|0)|0)-v|0)/(z|0)|0;c[j>>2]=C;D=(11.0- +g[a+244>>2])*.07/5.5+.93;E=D<.9?.8999999761581421:D;z=(w|0)>0;if(z){v=~~(+(C|0)*(E>1.0?1.0:E));B=(C|0)/2|0;u=(C*3|0|0)/2|0;F=0;do{do{if((y|0)>0){G=0;H=0;do{I=m+(F<<3)+(G<<2)|0;c[I>>2]=v;E=+g[b+(F<<3)+(G<<2)>>2];if(E>700.0){J=~~((E+-700.0)/1.4);c[I>>2]=v;if((c[a+304+(F*10504|0)+(G*5252|0)+4788>>2]|0)==2){K=(J|0)<(B|0)?B:J}else{K=J}if((K|0)>(u|0)){L=u}else{L=(K|0)<0?0:K}J=L+v|0;c[I>>2]=J;M=J}else{M=v}if((M|0)>4095){c[I>>2]=4095;N=4095}else{N=M}H=N+H|0;G=G+1|0;}while((G|0)<(y|0));if((H|0)>7680){O=0}else{break}do{G=m+(F<<3)+(O<<2)|0;c[G>>2]=((c[G>>2]|0)*7680|0|0)/(H|0)|0;O=O+1|0;}while((O|0)<(y|0))}}while(0);F=F+1|0;}while((F|0)<(w|0))}F=a+84756|0;do{if((c[F>>2]|0)==2){if(!z){break}E=+g[d>>2];C_(m|0,E,ac(C,y)|0,7680);O=c[p>>2]|0;if((O|0)>1){P=1}else{Q=O;R=17005;break}while(1){E=+g[d+(P<<2)>>2];C_(m+(P<<3)|0,E,ac(c[j>>2]|0,c[x>>2]|0)|0,7680);O=P+1|0;N=c[p>>2]|0;if((O|0)<(N|0)){P=O}else{Q=N;R=17005;break}}}else{Q=w;R=17005}}while(0);do{if((R|0)==17005){if((Q|0)<=0){break}w=c[x>>2]|0;P=0;j=0;while(1){if((w|0)>0){d=0;y=P;while(1){C=m+(j<<3)+(d<<2)|0;z=c[C>>2]|0;if((z|0)>4095){c[C>>2]=4095;S=4095}else{S=z}z=S+y|0;C=d+1|0;if((C|0)<(w|0)){d=C;y=z}else{U=z;break}}}else{U=P}y=j+1|0;if((y|0)<(Q|0)){P=U;j=y}else{break}}if((U|0)>(t|0)&(U|0)>0){j=0;do{if((w|0)>0){P=0;do{y=m+(j<<3)+(P<<2)|0;c[y>>2]=(ac(c[y>>2]|0,t)|0)/(U|0)|0;P=P+1|0;}while((P|0)<(w|0))}j=j+1|0;}while((j|0)<(Q|0))}if((Q|0)<=0){break}j=a+84912|0;w=a+84916|0;P=a+84908|0;y=l|0;d=a+85824|0;z=k|0;C=a+36|0;N=0;do{if((c[F>>2]|0)==2){O=0;do{M=a+304+(N*10504|0)+(O<<2)|0;E=+g[M>>2];v=a+304+(N*10504|0)+5252+(O<<2)|0;D=+g[v>>2];g[M>>2]=(E+D)*.7071067690849304;g[v>>2]=(E-D)*.7071067690849304;O=O+1|0;}while((O|0)<576)}if((c[x>>2]|0)>0){O=0;do{v=a+304+(N*10504|0)+(O*5252|0)|0;g[P>>2]=+T(+10.0,+(+g[((c[a+304+(N*10504|0)+(O*5252|0)+4788>>2]|0)==2?w:j)>>2]*.1));CU(a,v);g[h>>2]=0.0;M=c[a+304+(N*10504|0)+(O*5252|0)+5208>>2]|0;g[a+304+(N*10504|0)+(O*5252|0)+4764>>2]=0.0;NM(l+(M<<2)|0,0,576-M<<2|0);dq[c[d>>2]&127](v,y,M,h);if(+g[h>>2]>9.999999682655225e-21){M=(c[q>>2]|0)>>>1&1;L=a+304+(N*10504|0)+(O*5252|0)+4864|0;if((c[L>>2]|0)>0){K=0;do{c[a+84936+(K<<2)>>2]=M;K=K+1|0;}while((K|0)<(c[L>>2]|0))}L=m+(N<<3)+(O<<2)|0;if((C0(a,e+(N*976|0)+(O*488|0)|0,v,z)|0)==0){c[L>>2]=A;V=A}else{V=c[L>>2]|0}CV(a,v,z,y,O,V)|0}else{NM(a+304+(N*10504|0)+(O*5252|0)+2304|0,0,2304)}DT(a,N,O,o);if((c[C>>2]|0)==1){DR(a,v)}C5(a,v);O=O+1|0;}while((O|0)<(c[x>>2]|0))}N=N+1|0;}while((N|0)<(c[p>>2]|0))}}while(0);p=c[a+112>>2]|0;c[s>>2]=p;if((p|0)>(c[r>>2]|0)){W=c[n>>2]|0;C6(a,W);i=f;return}while(1){if((C3(a,n)|0)>-1){R=17038;break}p=(c[s>>2]|0)+1|0;c[s>>2]=p;if((p|0)>(c[r>>2]|0)){R=17040;break}}if((R|0)==17038){W=c[n>>2]|0;C6(a,W);i=f;return}else if((R|0)==17040){W=c[n>>2]|0;C6(a,W);i=f;return}}function CU(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0.0,E=0.0,F=0.0,G=0,H=0.0;d=i;i=i+2304|0;e=d|0;c[b+4768>>2]=0;c[b+4772>>2]=0;c[b+4776>>2]=0;c[b+4780>>2]=210;c[b+4784>>2]=0;f=a+64|0;NM(b+4796|0,0,52);h=b+4848|0;if((c[f>>2]|0)<8001){c[h>>2]=17;c[b+4852>>2]=9;c[b+4856>>2]=17;j=17;k=17}else{c[h>>2]=21;c[b+4852>>2]=12;l=(c[a+85092>>2]|0)!=0?22:21;c[b+4856>>2]=l;j=l;k=21}l=b+4856|0;m=b+4864|0;c[m>>2]=j;j=b+4860|0;c[j>>2]=k;k=b+4868|0;c[k>>2]=11;n=0;while(1){o=n+1|0;c[b+4872+(n<<2)>>2]=(c[a+21360+(o<<2)>>2]|0)-(c[a+21360+(n<<2)>>2]|0);c[b+5028+(n<<2)>>2]=3;if((o|0)<22){n=o}else{break}}n=b+4788|0;do{if((c[n>>2]|0)==2){o=b+4852|0;c[o>>2]=0;c[h>>2]=0;if((c[b+4792>>2]|0)==0){p=0;q=0}else{c[o>>2]=3;r=(c[a+76>>2]<<1)+4|0;c[h>>2]=r;p=r;q=3}if((c[f>>2]|0)<8001){r=((9-q|0)*3|0)+p|0;c[m>>2]=r;s=r}else{c[m>>2]=((((c[a+85092>>2]|0)!=0?13:12)-q|0)*3|0)+p;s=((12-q|0)*3|0)+p|0}c[j>>2]=s;c[k>>2]=s-18;c[l>>2]=p;r=c[a+21360+(p<<2)>>2]|0;t=e;u=b;NN(t|0,u|0,2304)|0;u=b+(r<<2)|0;r=q;while(1){t=c[a+21452+(r<<2)>>2]|0;v=r+1|0;w=c[a+21452+(v<<2)>>2]|0;if((t|0)<(w|0)){x=w-t|0;y=u;z=t;while(1){g[y>>2]=+g[e+((z*3|0)<<2)>>2];A=z+1|0;if((A|0)<(w|0)){y=y+4|0;z=A}else{break}}z=u+(x<<2)|0;y=t;while(1){g[z>>2]=+g[e+((y*3|0)+1<<2)>>2];A=y+1|0;if((A|0)<(w|0)){z=z+4|0;y=A}else{break}}y=x<<1;z=u+(y<<2)|0;A=t;while(1){g[z>>2]=+g[e+((A*3|0)+2<<2)>>2];B=A+1|0;if((B|0)<(w|0)){z=z+4|0;A=B}else{break}}C=u+(y+x<<2)|0}else{C=u}if((v|0)<13){u=C;r=v}else{break}}r=c[o>>2]|0;if((r|0)>=13){break}u=c[h>>2]|0;A=r;while(1){r=A+1|0;z=(c[a+21452+(r<<2)>>2]|0)-(c[a+21452+(A<<2)>>2]|0)|0;w=u+2|0;c[b+4872+(w<<2)>>2]=z;t=u+1|0;c[b+4872+(t<<2)>>2]=z;c[b+4872+(u<<2)>>2]=z;c[b+5028+(u<<2)>>2]=0;c[b+5028+(t<<2)>>2]=1;c[b+5028+(w<<2)>>2]=2;if((r|0)<13){u=u+3|0;A=r}else{break}}}}while(0);c[b+5184>>2]=0;c[b+5188>>2]=120992;NM(b+5192|0,0,16);c[b+5208>>2]=575;NM(b+4608|0,0,156);h=c[a+104>>2]|0;if((h|0)==1|(h|0)==4|(h|0)==3|(h|0)==0){i=d;return}h=c[a+85796>>2]|0;if((c[n>>2]|0)!=2){n=h+8|0;C=h+20|0;e=a+84852|0;q=5;L23080:while(1){p=c[a+21508+(q<<2)>>2]|0;l=c[a+21508+(q+1<<2)>>2]|0;D=+C$(+g[n>>2],+g[h+164+(q<<2)>>2],+g[C>>2],0.0);E=+g[e>>2];if(E>9.999999960041972e-13){F=D*E}else{F=D}s=l-1|0;if((s|0)>=(p|0)){l=s;do{s=b+(l<<2)|0;if(+R(+(+g[s>>2]))>=F){G=17086;break L23080}g[s>>2]=0.0;l=l-1|0;}while((l|0)>=(p|0))}if((q|0)>0){q=q-1|0}else{G=17087;break}}if((G|0)==17087){i=d;return}else if((G|0)==17086){i=d;return}}G=a+21500|0;q=a+21504|0;e=a+21536|0;C=h+8|0;n=h+20|0;p=a+84904|0;l=0;do{s=5;L23096:while(1){k=c[G>>2]|0;j=(ac((c[q>>2]|0)-k|0,l)|0)+(k*3|0)|0;k=c[a+21536+(s<<2)>>2]|0;m=j+(k-(c[e>>2]|0))|0;j=c[a+21536+(s+1<<2)>>2]|0;F=+C$(+g[C>>2],+g[h+188+(s<<2)>>2],+g[n>>2],0.0);D=+g[p>>2];if(D>9.999999960041972e-13){H=F*D}else{H=F}f=j+~k+m|0;if((f|0)>=(m|0)){k=f;do{f=b+(k<<2)|0;if(+R(+(+g[f>>2]))>=H){break L23096}g[f>>2]=0.0;k=k-1|0;}while((k|0)>=(m|0))}if((s|0)>0){s=s-1|0}else{break}}l=l+1|0;}while((l|0)<3);i=d;return}function CV(a,b,d,e,f,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0.0,ar=0,as=0,at=0.0,au=0.0,av=0.0,aw=0,ax=0.0,ay=0.0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0.0,aL=0,aM=0,aN=0,aO=0,aP=0.0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0.0,a1=0,a2=0,a3=0,a4=0.0,a5=0,a6=0,a7=0,a8=0.0,a9=0,ba=0,bb=0,bc=0.0,bd=0.0,be=0.0,bf=0.0,bg=0.0,bh=0.0,bi=0.0,bj=0,bk=0,bl=0;j=i;i=i+8248|0;k=j|0;l=j+5256|0;m=j+7560|0;n=j+7720|0;o=j+7744|0;p=j+8224|0;q=a+84928+(f<<2)|0;r=c[q>>2]|0;t=a+84920+(f<<2)|0;f=c[t>>2]|0;u=b+4780|0;c[u>>2]=f;v=h-(c[b+4844>>2]|0)|0;w=DS(a,e,b,0)|0;if((r|0)==1|(w|0)==(v|0)){x=w}else{y=r;r=0;z=0;A=w;while(1){do{if((A|0)>(v|0)){w=(z|0)==2?1:r;if((w|0)==0){B=y;C=1;D=0;E=y;break}F=(y|0)/2|0;B=F;C=1;D=w;E=F}else{F=(z|0)==1?1:r;if((F|0)==0){G=y}else{G=(y|0)/2|0}B=-G|0;C=2;D=F;E=G}}while(0);F=(c[u>>2]|0)+B|0;w=(F|0)<0;H=w?0:F;F=(H|0)>255;c[u>>2]=F?255:H;H=DS(a,e,b,0)|0;if((E|0)==1|(H|0)==(v|0)){x=H;break}else{y=E;r=F?1:w?1:D;z=C;A=H}}}L13:do{if((x|0)>(v|0)){A=x;while(1){C=c[u>>2]|0;if((C|0)>=255){I=A;break L13}c[u>>2]=C+1;C=DS(a,e,b,0)|0;if((C|0)>(v|0)){A=C}else{I=C;break}}}else{I=x}}while(0);c[q>>2]=(f-(c[u>>2]|0)|0)>3?4:2;c[t>>2]=c[u>>2];u=b+4768|0;c[u>>2]=I;I=a+28|0;if((c[I>>2]|0)==0){J=100;i=j;return J|0}NM(o|0,0,476);t=m|0;C1(b,d,t,n,o)|0;f=n+20|0;c[f>>2]=c[u>>2];q=k;x=b;NN(q|0,x|0,5252)|0;v=l;l=e;NN(v|0,l|0,2304)|0;A=a+85096|0;C=a+85092|0;z=k+4836|0;D=k+4860|0;r=a+40|0;E=k+4764|0;y=k+4844|0;B=k+4780|0;G=k+4768|0;H=n+12|0;w=p+20|0;F=b+4788|0;K=a+84|0;L=a+88|0;M=p+16|0;N=n+16|0;O=p+8|0;P=n+8|0;Q=a+48|0;T=n;U=p;V=p+12|0;W=p|0;X=n|0;Y=p+4|0;Z=n+4|0;n=k+4864|0;_=a+84936|0;$=k+4832|0;aa=k+4788|0;ab=a+32|0;ad=k+4848|0;ae=k+4868|0;af=9999999;ag=0;ah=0;while(1){ai=(ag|0)==1?2:1;aj=(ag|0)==0;ak=af;al=0;L23:while(1){am=(c[A>>2]&2|0)==0?3:20;an=c[D>>2]|0;do{if((c[C>>2]|0)!=0){if(+g[m+(an<<2)>>2]>1.0){ao=ak;ap=168;break L23}if((c[aa>>2]|0)!=2){break}if(+g[m+(an+1<<2)>>2]>1.0){ao=ak;ap=168;break L23}if(+g[m+(an+2<<2)>>2]>1.0){ao=ak;ap=168;break L23}}}while(0);aq=(c[z>>2]|0)==0?1.2968395948410034:1.6817928552627563;ar=(an|0)>0;if(ar){as=0;at=0.0;while(1){au=+g[m+(as<<2)>>2];av=at<au?au:at;aw=as+1|0;if((aw|0)<(an|0)){as=aw;at=av}else{ax=av;break}}}else{ax=0.0}as=c[r>>2]|0;aw=(as|0)==3?ai:as;do{if((aw|0)==1){at=ax;if(ax>1.0){av=+R(+(+S(+at)));ay=ax==+-s?+s:av;break}else{ay=at*.95;break}}else if((aw|0)==2){ay=ax}else{if(ax>1.0){ay=1.0;break}ay=ax*.95}}while(0);L43:do{if(ar){aw=0;az=0;aA=an;aB=as;while(1){aC=c[k+4872+(aw<<2)>>2]|0;aD=aC+az|0;if(+g[m+(aw<<2)>>2]<ay){aE=aA;aF=aB}else{do{if((c[A>>2]&2|0)!=0){aG=a+84936+(aw<<2)|0;aH=(c[aG>>2]|0)==0;c[aG>>2]=aH&1;if(aH){break}if((c[r>>2]|0)==2){aI=aA;aJ=2;break L43}}}while(0);aH=k+4608+(aw<<2)|0;c[aH>>2]=(c[aH>>2]|0)+1;if((aC|0)>0){aH=-aC|0;at=+g[E>>2];while(1){aG=e+(aH+aD<<2)|0;av=aq*+g[aG>>2];g[aG>>2]=av;if(av>at){g[E>>2]=av;aK=av}else{aK=at}aG=aH+1|0;if((aG|0)<0){aH=aG;at=aK}else{break}}}aH=c[r>>2]|0;aC=c[D>>2]|0;if((aH|0)==2){aI=aC;aJ=2;break L43}else{aE=aC;aF=aH}}aH=aw+1|0;if((aH|0)<(aE|0)){aw=aH;az=aD;aA=aE;aB=aF}else{aI=aE;aJ=aF;break}}}else{aI=an;aJ=as}}while(0);as=0;while(1){if((as|0)>=(aI|0)){aL=ak;aM=aJ;break L23}if((c[k+4608+(as<<2)>>2]|0)==(-(c[k+4808+(c[k+5028+(as<<2)>>2]<<2)>>2]|0)|0)){break}else{as=as+1|0}}as=DU(a,k)|0;if((as|0)!=0){if((c[I>>2]|0)<=1){ao=ak;ap=168;break}NM(_|0,0,156);do{if((c[z>>2]|0)==0){if((c[D>>2]|0)>0){an=0;ar=0;do{aB=c[k+4872+(an<<2)>>2]|0;aA=k+4608+(an<<2)|0;az=c[aA>>2]|0;if((c[$>>2]|0)==0){aN=az}else{aN=(c[90800+(an<<2)>>2]|0)+az|0}ar=aB+ar|0;do{if((aN&1|0)==0){aO=aN}else{az=aN+1|0;if((aB|0)<=0){aO=az;break}aw=-aB|0;aq=+g[E>>2];while(1){aH=e+(aw+ar<<2)|0;at=+g[aH>>2]*1.2968395948410034;g[aH>>2]=at;if(at>aq){g[E>>2]=at;aP=at}else{aP=aq}aH=aw+1|0;if((aH|0)<0){aw=aH;aq=aP}else{aO=az;break}}}}while(0);c[aA>>2]=aO>>1;an=an+1|0;}while((an|0)<(c[D>>2]|0))}c[$>>2]=0;c[z>>2]=1;ap=90}else{if((c[aa>>2]|0)!=2){aQ=as;break}if((c[ab>>2]|0)<=0){aQ=as;break}an=c[ad>>2]|0;ar=0;while(1){if((ar|0)>=(an|0)){aR=0;aS=an;break}if((c[k+4608+(ar<<2)>>2]|0)>15){ao=ak;ap=168;break L23}else{ar=ar+1|0}}while(1){ar=aR+aS|0;an=c[ae>>2]|0;if((ar|0)<(an|0)){aB=0;aD=ar;while(1){az=c[k+4608+(aD<<2)>>2]|0;aw=(aB|0)<(az|0)?az:aB;az=aD+3|0;if((az|0)<(an|0)){aB=aw;aD=az}else{aT=aw;aU=az;break}}}else{aT=0;aU=ar}aD=c[D>>2]|0;if((aU|0)<(aD|0)){aB=0;an=aU;do{aA=c[k+4608+(an<<2)>>2]|0;aB=(aB|0)<(aA|0)?aA:aB;an=an+3|0;}while((an|0)<(aD|0));aV=(aB|0)<8}else{aV=1}do{if((aT|0)<16&aV){aW=aR+1|0;aX=aD}else{an=k+4808+(aR<<2)|0;ar=c[an>>2]|0;if((ar|0)>6){ao=ak;ap=168;break L23}c[an>>2]=ar+1;ar=c[ad>>2]|0;an=c[a+21360+(ar<<2)>>2]|0;aA=ar+aR|0;ar=c[D>>2]|0;az=aR+1|0;if((aA|0)<(ar|0)){aw=2-aR|0;aH=an;aC=aA;while(1){aG=c[k+4872+(aC<<2)>>2]|0;aY=k+4608+(aC<<2)|0;aZ=(c[aY>>2]|0)-(4>>>((c[z>>2]|0)>>>0))|0;if((aZ|0)>-1){c[aY>>2]=aZ;a_=(aG*3|0)+aH|0}else{c[aY>>2]=0;aq=+g[2206176+((aZ<<(c[z>>2]|0)+1)+210<<2)>>2];aZ=(ac(aG,az)|0)+aH|0;if((aG|0)>0){aY=-aG|0;at=+g[E>>2];while(1){a$=e+(aY+aZ<<2)|0;av=aq*+g[a$>>2];g[a$>>2]=av;if(av>at){g[E>>2]=av;a0=av}else{a0=at}a$=aY+1|0;if((a$|0)<0){aY=a$;at=a0}else{break}}}a_=aZ+(ac(aG,aw)|0)|0}aY=aC+3|0;a$=c[D>>2]|0;if((aY|0)<(a$|0)){aH=a_;aC=aY}else{a1=a_;a2=aY;a3=a$;break}}}else{a1=an;a2=aA;a3=ar}at=+g[551746];aC=c[k+4872+(a2<<2)>>2]|0;aH=(ac(aC,az)|0)+a1|0;if((aC|0)<=0){aW=az;aX=a3;break}aw=-aC|0;aq=+g[E>>2];while(1){aC=e+(aH+aw<<2)|0;av=at*+g[aC>>2];g[aC>>2]=av;if(av>aq){g[E>>2]=av;a4=av}else{a4=aq}aC=aw+1|0;if((aC|0)<0){aw=aC;aq=a4}else{aW=az;aX=a3;break}}}}while(0);if((aW|0)>=3){a5=0;break}aR=aW;aS=c[ad>>2]|0}while(1){if((a5|0)>=(aX|0)){ao=ak;ap=168;break L23}if((c[k+4608+(a5<<2)>>2]|0)==(-(c[k+4808+(c[k+5028+(a5<<2)>>2]<<2)>>2]|0)|0)){ap=90;break}else{a5=a5+1|0}}}}while(0);if((ap|0)==90){ap=0;aQ=DU(a,k)|0}if((aQ|0)!=0){ao=ak;ap=168;break}}as=(c[z>>2]|0)==0?255:254;aD=h-(c[y>>2]|0)|0;if((aD|0)<1){ao=ak;ap=168;break}aB=DS(a,e,k,o)|0;c[G>>2]=aB;az=c[B>>2]|0;L141:do{if((aB|0)>(aD|0)){aw=az;while(1){if((aw|0)>(as|0)){a6=aw;break L141}c[B>>2]=aw+1;aH=DS(a,e,k,o)|0;c[G>>2]=aH;ar=c[B>>2]|0;if((aH|0)>(aD|0)){aw=ar}else{a6=ar;break}}}else{a6=az}}while(0);if((a6|0)>(as|0)){ao=ak;ap=168;break}if((c[H>>2]|0)==0){az=DS(a,e,k,o)|0;c[G>>2]=az;aD=c[B>>2]|0;L149:do{if((az|0)>(ak|0)){aB=aD;while(1){if((aB|0)>(as|0)){a7=aB;break L149}c[B>>2]=aB+1;aw=DS(a,e,k,o)|0;c[G>>2]=aw;ar=c[B>>2]|0;if((aw|0)>(ak|0)){aB=ar}else{a7=ar;break}}}else{a7=aD}}while(0);if((a7|0)>(as|0)){ao=ak;ap=168;break}}C1(k,d,t,p,o)|0;aD=c[G>>2]|0;c[w>>2]=aD;L155:do{switch(c[((c[F>>2]|0)==2?L:K)>>2]|0){case 1:{a8=+g[O>>2];ap=119;break};case 0:{az=c[V>>2]|0;aB=c[H>>2]|0;if((az|0)<(aB|0)){a9=1;ba=aB;break L155}if((az|0)!=(aB|0)){bb=0;ap=156;break L155}aq=+g[W>>2];at=+g[X>>2];if(aq<at){bb=1;ap=156;break L155}av=+R(+aq);au=+R(+at);bc=+R(+(aq-at));if(av>au){if(bc>av*9.999999974752427e-7){bb=0;ap=156;break L155}}else{if(bc>au*9.999999974752427e-7){bb=0;ap=156;break L155}}bb=+g[Y>>2]<+g[Z>>2]|0;ap=156;break};case 8:{if((c[n>>2]|0)>0){aB=0;au=1.0e-37;while(1){bc=+g[m+(aB<<2)>>2];av=au+ +Ek(bc*bc*bc*.632+.368)*.30102999566398114;az=aB+1|0;if((az|0)<(c[n>>2]|0)){aB=az;au=av}else{bd=av;break}}}else{bd=1.0e-37}au=bd<1.0e-20?9.999999682655225e-21:bd;g[O>>2]=au;a8=au;ap=119;break};case 2:{bb=+g[Y>>2]<+g[Z>>2]|0;ap=156;break};case 3:{if(+g[Y>>2]>=+g[Z>>2]){bb=0;ap=156;break L155}bb=+g[O>>2]<+g[P>>2]|0;ap=156;break};case 4:{au=+g[O>>2];do{if(au>0.0){be=au}else{av=+g[P>>2];bc=av;if(bc>.2){bb=1;ap=156;break L155}at=au;do{if(av<0.0){if(bc<=at+-.2){break}if(+g[Y>>2]<+g[Z>>2]){bb=1;ap=156;break L155}}}while(0);if(av<=0.0){be=at;break}if(bc<=at+-.2){be=at;break}if(+g[Y>>2]<+g[Z>>2]+ +g[X>>2]){bb=1;ap=156;break L155}else{be=at}}}while(0);if(au<=0.0){bb=0;ap=156;break L155}aq=+g[P>>2];do{if(aq>-.05){if(aq<=be+-.1){break}aB=+g[Y>>2]+ +g[W>>2]<+g[Z>>2]+ +g[X>>2];if(!aB){break}bb=aB&1;ap=156;break L155}}while(0);if(aq<=-.1){bb=0;ap=156;break L155}if(aq<=be+-.15){bb=0;ap=156;break L155}au=+g[W>>2];bf=+g[X>>2];bb=au+(+g[Y>>2]+au)<bf+(+g[Z>>2]+bf)|0;ap=156;break};case 5:{bf=+g[W>>2];au=+g[X>>2];if(bf<au){bb=1;ap=156;break L155}bg=+R(+bf);bh=+R(+au);bi=+R(+(bf-au));if(bg>bh){if(bi>bg*9.999999974752427e-7){bb=0;ap=156;break L155}}else{if(bi>bh*9.999999974752427e-7){bb=0;ap=156;break L155}}bb=+g[Y>>2]<+g[Z>>2]|0;ap=156;break};case 6:{bh=+g[W>>2];bi=+g[X>>2];if(bh<bi){bb=1;ap=156;break L155}bg=+R(+bh);au=+R(+bi);bf=+R(+(bh-bi));if(bg>au){if(bf>bg*9.999999974752427e-7){bb=0;ap=156;break L155}}else{if(bf>au*9.999999974752427e-7){bb=0;ap=156;break L155}}au=+g[O>>2];bf=+g[P>>2];if(au<bf){bb=1;ap=156;break L155}bg=+R(+au);bi=+R(+bf);bh=+R(+(au-bf));if(bg>bi){if(bh>bg*9.999999974752427e-7){bb=0;ap=156;break L155}}else{if(bh>bi*9.999999974752427e-7){bb=0;ap=156;break L155}}bb=+g[Y>>2]<=+g[Z>>2]|0;ap=156;break};case 7:{aB=c[H>>2]|0;if((c[V>>2]|0)<(aB|0)){a9=1;ba=aB;break L155}bb=+g[W>>2]<+g[X>>2]|0;ap=156;break};default:{if((c[H>>2]|0)>0){aB=c[M>>2]|0;az=c[N>>2]|0;if((aB|0)!=(az|0)){bb=(aB|0)<=(az|0)|0;ap=156;break L155}bb=(aD|0)<(c[f>>2]|0)|0;ap=156;break L155}else{bi=+g[O>>2];if(bi>=0.0){bb=0;ap=156;break L155}bb=+(aD|0)+bi*10.0<=+g[P>>2]*10.0+ +(c[f>>2]|0)|0;ap=156;break L155}}}}while(0);if((ap|0)==119){ap=0;bb=a8<+g[P>>2]|0;ap=156}if((ap|0)==156){ap=0;a9=bb;ba=c[H>>2]|0}aD=(ba|0)==0;do{if(aD){if((a9|0)==0){ap=162;break}bj=(c[w>>2]|0)<(c[f>>2]|0)|0;ap=160}else{bj=a9;ap=160}}while(0);do{if((ap|0)==160){ap=0;if((bj|0)==0){ap=162;break}as=c[u>>2]|0;c[T>>2]=c[U>>2];c[T+4>>2]=c[U+4>>2];c[T+8>>2]=c[U+8>>2];c[T+12>>2]=c[U+12>>2];c[T+16>>2]=c[U+16>>2];c[T+20>>2]=c[U+20>>2];NN(x|0,q|0,5252)|0;NN(v|0,l|0,2304)|0;bk=as;bl=0}}while(0);do{if((ap|0)==162){ap=0;if((c[Q>>2]|0)!=0){bk=ak;bl=al;break}as=al+1|0;if(!((as|0)<=(am|0)|aD^1)){ao=ak;ap=168;break L23}az=(c[r>>2]|0)!=3|aj;if((as|0)>30&(az^1)){ao=ak;ap=168;break L23}if(az){bk=ak;bl=as;break}if(((c[B>>2]|0)-ah|0)>15){ao=ak;ap=168;break L23}else{bk=ak;bl=as}}}while(0);if(((c[z>>2]|0)+(c[B>>2]|0)|0)<255){ak=bk;al=bl}else{ao=bk;ap=168;break}}if((ap|0)==168){ap=0;aL=ao;aM=c[r>>2]|0}if(!((aM|0)==3&aj)){break}NN(q|0,x|0,5252)|0;NN(l|0,v|0,2304)|0;af=aL;ag=1;ah=c[B>>2]|0}B=c[a+104>>2]|0;do{if((B|0)==2|(B|0)==4|(B|0)==1){NN(l|0,v|0,2304)|0}else{if((c[A>>2]&1|0)==0){break}CR(a,b,d,e)}}while(0);J=c[H>>2]|0;i=j;return J|0}function CW(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0.0,F=0,G=0.0,H=0,I=0;f=i;i=i+2488|0;h=f|0;j=f+168|0;k=f+2472|0;l=f+2480|0;m=a+304|0;C3(a,l)|0;n=a+76|0;if((c[n>>2]|0)<=0){o=c[l>>2]|0;C6(a,o);i=f;return}p=k|0;q=a+84756|0;r=a+72|0;s=a+84912|0;t=a+84916|0;u=a+84908|0;v=j|0;w=a+85824|0;x=a+85096|0;y=f+8|0;z=a+36|0;A=0;do{B=CZ(a,b,p,c[l>>2]|0,A,A)|0;if((c[q>>2]|0)==2){C=0;do{D=a+304+(A*10504|0)+(C<<2)|0;E=+g[D>>2];F=a+304+(A*10504|0)+5252+(C<<2)|0;G=+g[F>>2];g[D>>2]=(E+G)*.7071067690849304;g[F>>2]=(E-G)*.7071067690849304;C=C+1|0;}while((C|0)<576);C_(p,+g[d+(A<<2)>>2],c[l>>2]|0,B)}if((c[r>>2]|0)>0){C=0;do{F=a+304+(A*10504|0)+(C*5252|0)|0;g[u>>2]=+T(+10.0,+(+g[((c[a+304+(A*10504|0)+(C*5252|0)+4788>>2]|0)==2?t:s)>>2]*.1));CU(a,F);g[h>>2]=0.0;D=c[a+304+(A*10504|0)+(C*5252|0)+5208>>2]|0;g[a+304+(A*10504|0)+(C*5252|0)+4764>>2]=0.0;NM(j+(D<<2)|0,0,576-D<<2|0);dq[c[w>>2]&127](F,v,D,h);if(+g[h>>2]>9.999999682655225e-21){D=(c[x>>2]|0)>>>1&1;H=a+304+(A*10504|0)+(C*5252|0)+4864|0;if((c[H>>2]|0)>0){I=0;do{c[a+84936+(I<<2)>>2]=D;I=I+1|0;}while((I|0)<(c[H>>2]|0))}H=e+(A*976|0)+(C*488|0)|0;C0(a,H,F,y)|0;H=c[k+(C<<2)>>2]|0;CV(a,F,y,v,C,H)|0}else{NM(a+304+(A*10504|0)+(C*5252|0)+2304|0,0,2304)}DT(a,A,C,m);if((c[z>>2]|0)==1){DR(a,F)}C5(a,F);C=C+1|0;}while((C|0)<(c[r>>2]|0))}A=A+1|0;}while((A|0)<(c[n>>2]|0));o=c[l>>2]|0;C6(a,o);i=f;return}function CX(a,b){a=a|0;b=b|0;var c=0.0,d=0.0,e=0;c=+g[a>>2];d=+g[b>>2];if(c>d){e=1;return e|0}e=(c<d)<<31>>31;return e|0}function CY(a){a=a|0;var b=0,d=0,e=0,f=0.0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0.0,q=0.0,r=0.0,s=0,t=0.0,u=0,v=0,w=0.0,x=0.0,y=0,z=0,A=0,B=0.0,C=0.0,D=0.0,E=0,F=0;b=a+8|0;if((c[b>>2]|0)!=0){return}c[b>>2]=1;c[a+21312>>2]=0;b=a+16|0;d=a+85796|0;e=c[d>>2]|0;f=+(c[a+64>>2]|0);h=a+224|0;i=a+196|0;j=0;while(1){k=c[a+21360+(j<<2)>>2]|0;l=j+1|0;m=c[a+21360+(l<<2)>>2]|0;n=e+24+(j<<2)|0;g[n>>2]=9.999999933815813e+36;if((k|0)<(m|0)){o=k;do{p=+D2(b,f*+(o|0)/1152.0);q=+g[h>>2];if(q>0.0){r=p-q}else{r=p+-100.0}p=+T(+10.0,+((r+ +g[i>>2])*.10000000149011612));q=+g[n>>2];g[n>>2]=q<p?q:p;o=o+1|0;}while((o|0)<(m|0))}if((l|0)<22){j=l}else{s=0;break}}while(1){j=c[a+21508+(s<<2)>>2]|0;m=s+1|0;o=c[a+21508+(m<<2)>>2]|0;n=e+164+(s<<2)|0;g[n>>2]=9.999999933815813e+36;if((j|0)<(o|0)){k=j;do{r=+D2(b,f*+(k|0)/1152.0);p=+g[h>>2];if(p>0.0){t=r-p}else{t=r+-100.0}r=+T(+10.0,+((t+ +g[i>>2])*.10000000149011612));p=+g[n>>2];g[n>>2]=p<r?p:r;k=k+1|0;}while((k|0)<(o|0))}if((m|0)<6){s=m}else{u=0;break}}while(1){s=a+21452+(u<<2)|0;o=c[s>>2]|0;k=u+1|0;n=a+21452+(k<<2)|0;l=c[n>>2]|0;j=e+112+(u<<2)|0;g[j>>2]=9.999999933815813e+36;if((o|0)<(l|0)){v=o;while(1){t=+D2(b,f*+(v|0)/384.0);r=+g[h>>2];if(r>0.0){w=t-r}else{w=t+-100.0}t=+T(+10.0,+((w+ +g[i>>2])*.10000000149011612));r=+g[j>>2];p=r<t?r:t;g[j>>2]=p;o=v+1|0;if((o|0)<(l|0)){v=o}else{x=p;break}}}else{x=9.999999933815813e+36}g[j>>2]=x*+((c[n>>2]|0)-(c[s>>2]|0)|0);if((k|0)<13){u=k}else{break}}u=a+21504|0;v=a+21500|0;l=0;while(1){m=c[a+21536+(l<<2)>>2]|0;o=l+1|0;y=c[a+21536+(o<<2)>>2]|0;z=e+188+(l<<2)|0;g[z>>2]=9.999999933815813e+36;if((m|0)<(y|0)){A=m;while(1){x=+D2(b,f*+(A|0)/384.0);w=+g[h>>2];if(w>0.0){B=x-w}else{B=x+-100.0}x=+T(+10.0,+((B+ +g[i>>2])*.10000000149011612));w=+g[z>>2];p=w<x?w:x;g[z>>2]=p;m=A+1|0;if((m|0)<(y|0)){A=m}else{C=p;break}}}else{C=9.999999933815813e+36}g[z>>2]=C*+((c[u>>2]|0)-(c[v>>2]|0)|0);if((o|0)<6){l=o}else{break}}if((c[a+220>>2]|0)!=0){g[e+24>>2]=9.999999682655225e-21;g[e+28>>2]=9.999999682655225e-21;g[e+32>>2]=9.999999682655225e-21;g[e+36>>2]=9.999999682655225e-21;g[e+40>>2]=9.999999682655225e-21;g[e+44>>2]=9.999999682655225e-21;g[e+48>>2]=9.999999682655225e-21;g[e+52>>2]=9.999999682655225e-21;g[e+56>>2]=9.999999682655225e-21;g[e+60>>2]=9.999999682655225e-21;g[e+64>>2]=9.999999682655225e-21;g[e+68>>2]=9.999999682655225e-21;g[e+72>>2]=9.999999682655225e-21;g[e+76>>2]=9.999999682655225e-21;g[e+80>>2]=9.999999682655225e-21;g[e+84>>2]=9.999999682655225e-21;g[e+88>>2]=9.999999682655225e-21;g[e+92>>2]=9.999999682655225e-21;g[e+96>>2]=9.999999682655225e-21;g[e+100>>2]=9.999999682655225e-21;g[e+104>>2]=9.999999682655225e-21;g[e+108>>2]=9.999999682655225e-21;g[e+164>>2]=9.999999682655225e-21;g[e+168>>2]=9.999999682655225e-21;g[e+172>>2]=9.999999682655225e-21;g[e+176>>2]=9.999999682655225e-21;g[e+180>>2]=9.999999682655225e-21;g[e+184>>2]=9.999999682655225e-21;g[e+112>>2]=9.999999682655225e-21;g[e+116>>2]=9.999999682655225e-21;g[e+120>>2]=9.999999682655225e-21;g[e+124>>2]=9.999999682655225e-21;g[e+128>>2]=9.999999682655225e-21;g[e+132>>2]=9.999999682655225e-21;g[e+136>>2]=9.999999682655225e-21;g[e+140>>2]=9.999999682655225e-21;g[e+144>>2]=9.999999682655225e-21;g[e+148>>2]=9.999999682655225e-21;g[e+152>>2]=9.999999682655225e-21;g[e+156>>2]=9.999999682655225e-21;g[e+160>>2]=9.999999682655225e-21;g[e+188>>2]=9.999999682655225e-21;g[e+192>>2]=9.999999682655225e-21;g[e+196>>2]=9.999999682655225e-21;g[e+200>>2]=9.999999682655225e-21;g[e+204>>2]=9.999999682655225e-21;g[e+208>>2]=9.999999682655225e-21}C=+D2(b,-1.0);B=+g[h>>2];if(B>0.0){D=C-B}else{D=C+-100.0}C=+b7(+(+T(+10.0,+((D+ +g[i>>2])*.10000000149011612))))*10.0;g[(c[d>>2]|0)+20>>2]=C;g[541066]=0.0;d=1;do{g[2164264+(d<<2)>>2]=+T(+(+(d|0)),1.3333333333333333);d=d+1|0;}while((d|0)<8208);g[767228]=0.0;d=1;C=+g[541066];while(1){D=+g[2164264+(d<<2)>>2];g[3068912+(d<<2)>>2]=+(d|0)+-.5- +T(+((C+D)*.5),+.75);i=d+1|0;if((i|0)<8208){d=i;C=D}else{E=0;break}}while(1){g[2206176+(E<<2)>>2]=+b4(+(+(E-210|0)*-.1875));d=E+1|0;if((d|0)<257){E=d}else{F=0;break}}do{g[2197096+(F<<2)>>2]=+b4(+(+(F-326|0)*.25));F=F+1|0;}while((F|0)<374);DV(a);CO(a);F=a+232|0;C=+T(+10.0,+((+g[F>>2]+-.5)*.10000000149011612));g[a+84768>>2]=C;g[a+84772>>2]=C;g[a+84776>>2]=C;g[a+84780>>2]=C;g[a+84784>>2]=C;g[a+84788>>2]=C;g[a+84792>>2]=C;E=a+228|0;C=+T(+10.0,+((+g[E>>2]+-.25)*.10000000149011612));g[a+84796>>2]=C;g[a+84800>>2]=C;g[a+84804>>2]=C;g[a+84808>>2]=C;g[a+84812>>2]=C;g[a+84816>>2]=C;g[a+84820>>2]=C;d=a+236|0;C=+T(+10.0,+((+g[d>>2]+-.02500000037252903)*.10000000149011612));i=14;do{g[a+84768+(i<<2)>>2]=C;i=i+1|0;}while((i|0)<21);i=a+240|0;g[a+84852>>2]=+T(+10.0,+((+g[i>>2]+.5)*.10000000149011612));C=+T(+10.0,+((+g[F>>2]+-2.0)*.10000000149011612));g[a+84856>>2]=C;g[a+84860>>2]=C;g[a+84864>>2]=C;C=+T(+10.0,+((+g[E>>2]+-1.0)*.10000000149011612));g[a+84868>>2]=C;g[a+84872>>2]=C;g[a+84876>>2]=C;g[a+84880>>2]=C;C=+T(+10.0,+((+g[d>>2]+-.05000000074505806)*.10000000149011612));d=7;do{g[a+84856+(d<<2)>>2]=C;d=d+1|0;}while((d|0)<12);g[a+84904>>2]=+T(+10.0,+((+g[i>>2]+.5)*.10000000149011612));return}function CZ(a,b,d,e,f,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;j=i;i=i+24|0;k=j|0;l=j+8|0;m=j+16|0;n=m;c[k>>2]=0;c[m>>2]=0;c[m+4>>2]=0;C4(a,e,l,k,h);h=c[l>>2]|0;l=c[k>>2]|0;m=l+h|0;o=(m|0)>7680?7680:m;m=a+72|0;a=c[m>>2]|0;if((a|0)<=0){i=j;return o|0}p=(e*3|0|0)/4|0;e=0;q=0;r=a;do{a=(h|0)/(r|0)|0;s=(a|0)>4095?4095:a;c[d+(q<<2)>>2]=s;a=~~(+g[b+(f<<3)+(q<<2)>>2]*+(s|0)/700.0- +(s|0));t=(a|0)>(p|0)?p:a;a=(t|0)<0?0:t;if((a+s|0)>4095){t=4095-s|0;u=(t|0)<0?0:t}else{u=a}c[n+(q<<2)>>2]=u;e=u+e|0;q=q+1|0;r=c[m>>2]|0;}while((q|0)<(r|0));if((e|0)>(l|0)&(e|0)>0){if((r|0)>0){q=0;do{u=n+(q<<2)|0;c[u>>2]=(ac(c[u>>2]|0,l)|0)/(e|0)|0;q=q+1|0;}while((q|0)<(r|0))}else{i=j;return o|0}}if((r|0)>0){v=0;w=l}else{i=j;return o|0}do{l=c[n+(v<<2)>>2]|0;r=d+(v<<2)|0;c[r>>2]=(c[r>>2]|0)+l;w=w-l|0;v=v+1|0;x=c[m>>2]|0;}while((v|0)<(x|0));c[k>>2]=w;w=(x|0)>0;if(w){y=0;z=0}else{i=j;return o|0}do{y=(c[d+(z<<2)>>2]|0)+y|0;z=z+1|0;}while((z|0)<(x|0));if((y|0)<7681|w^1){i=j;return o|0}else{A=0}do{w=d+(A<<2)|0;c[w>>2]=((c[w>>2]|0)*7680|0|0)/(y|0)|0;A=A+1|0;}while((A|0)<(c[m>>2]|0));i=j;return o|0}function C_(a,b,d,e){a=a|0;b=+b;d=d|0;e=e|0;var f=0.0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;f=(.5-b)*.33*2.0;b=f<0.0?0.0:f;g=c[a>>2]|0;h=a+4|0;i=c[h>>2]|0;j=~~((b>.5?.25:b*.5)*+(i+g|0));k=4095-g|0;l=(j|0)>(k|0)?k:j;j=(l|0)<0?0:l;do{if((i|0)>124){l=i-j|0;if((l|0)<=125){k=g-125+i|0;c[a>>2]=k;c[h>>2]=125;m=k;n=125;break}if((g|0)<(d|0)){k=j+g|0;c[a>>2]=k;o=k}else{o=g}c[h>>2]=l;m=o;n=l}else{m=g;n=i}}while(0);i=n+m|0;if((i|0)<=(e|0)){return}c[a>>2]=(ac(m,e)|0)/(i|0)|0;c[h>>2]=(ac(n,e)|0)/(i|0)|0;return}function C$(a,b,c,d){a=+a;b=+b;c=+c;d=+d;var e=0.0,f=0.0;e=a*a;a=+Ek(b)*3.0102999566398116-c;if(e>9.999999682655225e-21){f=+Ek(e)*.03333343265598758+1.0}else{f=0.0}return+(+T(+10.0,+((c+90.30873107910156-(d<1.0?94.82444763183594:d)+a*(f<0.0?0.0:f))*.10000000149011612)))}function C0(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0.0,s=0.0,t=0.0,u=0.0,v=0,w=0,x=0.0,y=0,z=0.0,A=0.0,B=0.0,C=0.0,D=0,E=0.0,F=0.0,G=0,H=0.0,I=0.0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,S=0,U=0.0,V=0.0,W=0.0,X=0.0,Y=0.0,Z=0.0,_=0.0,$=0.0,aa=0.0,ab=0.0,ac=0.0,ad=0.0,ae=0.0,af=0,ag=0,ah=0.0,ai=0.0,aj=0.0,ak=0.0,al=0.0,am=0.0,an=0.0;h=c[b+85796>>2]|0;i=e+4856|0;if((c[i>>2]|0)>0){j=h+8|0;k=h+20|0;l=b+224|0;m=0;n=0;o=0;p=f;while(1){q=+g[j>>2];r=+g[k>>2];s=+g[l>>2];t=q*q;q=+Ek(+g[h+24+(o<<2)>>2])*3.0102999566398116-r;if(t>9.999999682655225e-21){u=+Ek(t)*.03333343265598758+1.0}else{u=0.0}t=+T(+10.0,+((r+90.30873107910156-(s<1.0?94.82444763183594:s)+q*(u<0.0?0.0:u))*.10000000149011612));q=+g[b+84768+(o<<2)>>2];s=t*q;v=c[e+4872+(o<<2)>>2]|0;t=s/+(v|0);if((v|0)>0){w=0;r=2.220446049250313e-16;x=0.0;y=n;while(1){z=+g[e+(y<<2)>>2];A=z*z;B=x+A;C=r+(A<t?A:t);D=w+1|0;if((D|0)<(v|0)){w=D;r=C;x=B;y=y+1|0}else{break}}E=C;F=B;G=n+v|0}else{E=2.220446049250313e-16;F=0.0;G=n}y=(F>s)+m|0;if(F<s){H=F}else{H=E<s?s:E}x=+g[d+244+(o<<2)>>2];do{if(x>9.999999960041972e-13){r=F*+g[d+(o<<2)>>2]/x*q;if(H>=r){I=H;break}I=r}else{I=H}}while(0);q=I>2.220446049250313e-16?I:2.220446049250313e-16;a[e+5212+o|0]=F>q+9.9999998245167e-15|0;v=p+4|0;g[p>>2]=q;w=o+1|0;if((w|0)<(c[i>>2]|0)){m=y;n=G;o=w;p=v}else{J=y;K=G;L=w;M=v;break}}}else{J=0;K=0;L=0;M=f}f=575;while(1){if((f|0)<=0){N=0;break}if(+R(+(+g[e+(f<<2)>>2]))>9.999999960041972e-13){N=f;break}else{f=f-1|0}}f=(c[e+4788>>2]|0)==2;if(f){O=N+5-((N|0)%6|0)|0}else{O=N|1}do{if((c[b+85092>>2]|0)==0){N=c[b+64>>2]|0;if((N|0)>=44e3){P=O;break}G=(N|0)<8001;if(f){Q=(c[b+21452+((G?9:12)<<2)>>2]|0)*3|0}else{Q=c[b+21360+((G?17:21)<<2)>>2]|0}G=Q-1|0;P=(O|0)>(G|0)?G:O}else{P=O}}while(0);c[e+5208>>2]=P;P=e+4864|0;if((L|0)>=(c[P>>2]|0)){S=J;return S|0}O=h+8|0;Q=h+20|0;f=b+224|0;G=b+92|0;N=b+85800|0;p=J;J=K;K=L;L=c[e+4852>>2]|0;o=M;while(1){F=+g[O>>2];I=+g[Q>>2];H=+g[f>>2];E=F*F;F=+Ek(+g[h+112+(L<<2)>>2])*3.0102999566398116-I;if(E>9.999999682655225e-21){U=+Ek(E)*.03333343265598758+1.0}else{U=0.0}E=+T(+10.0,+((I+90.30873107910156-(H<1.0?94.82444763183594:H)+F*(U<0.0?0.0:U))*.10000000149011612));M=b+84856+(L<<2)|0;F=+g[M>>2];H=E*F;n=c[e+4872+(K<<2)>>2]|0;E=H/+(n|0);if((n|0)>0){m=0;I=0.0;i=J;B=2.220446049250313e-16;while(1){C=+g[e+(i<<2)>>2];u=C*C;V=I+u;W=B+(u<E?u:E);l=m+1|0;if((l|0)<(n|0)){m=l;I=V;i=i+1|0;B=W}else{break}}i=n+J|0;m=(V>H)+p|0;if(V<H){X=V}else{X=W<H?H:W}B=+g[d+332+(L*12|0)>>2];do{if(B>9.999999960041972e-13){I=V*+g[d+88+(L*12|0)>>2]/B*F;if(X>=I){Y=X;break}Y=I}else{Y=X}}while(0);B=Y>2.220446049250313e-16?Y:2.220446049250313e-16;a[e+5212+K|0]=V>B+9.9999998245167e-15|0;y=o+4|0;g[o>>2]=B;l=0;B=0.0;k=i;I=2.220446049250313e-16;while(1){u=+g[e+(k<<2)>>2];C=u*u;Z=B+C;_=I+(C<E?C:E);j=l+1|0;if((j|0)<(n|0)){l=j;B=Z;k=k+1|0;I=_}else{break}}k=n+i|0;l=(Z>H)+m|0;if(Z<H){$=Z}else{$=_<H?H:_}I=+g[d+332+(L*12|0)+4>>2];do{if(I>9.999999960041972e-13){B=Z*+g[d+88+(L*12|0)+4>>2]/I*+g[M>>2];if($>=B){aa=$;break}aa=B}else{aa=$}}while(0);I=aa>2.220446049250313e-16?aa:2.220446049250313e-16;a[K+1+(e+5212)|0]=Z>I+9.9999998245167e-15|0;m=o+8|0;g[y>>2]=I;i=0;I=0.0;j=k;B=2.220446049250313e-16;while(1){C=+g[e+(j<<2)>>2];u=C*C;ab=I+u;ac=B+(u<E?u:E);v=i+1|0;if((v|0)<(n|0)){i=v;I=ab;j=j+1|0;B=ac}else{break}}j=n+k|0;i=(ab>H)+l|0;if(ab<H){ad=ab}else{ad=ac<H?H:ac}B=+g[d+332+(L*12|0)+8>>2];do{if(B>9.999999960041972e-13){I=ab*+g[d+88+(L*12|0)+8>>2]/B*+g[M>>2];if(ad>=I){ae=ad;break}ae=I}else{ae=ad}}while(0);B=ae>2.220446049250313e-16?ae:2.220446049250313e-16;a[K+2+(e+5212)|0]=ab>B+9.9999998245167e-15|0;g[m>>2]=B;af=i;ag=j}else{l=H<0.0|0;k=l+p|0;n=H>0.0;if(n){ah=0.0}else{ah=H>2.220446049250313e-16?H:2.220446049250313e-16}B=+g[d+332+(L*12|0)>>2];do{if(B>9.999999960041972e-13){I=+g[d+88+(L*12|0)>>2]*0.0/B*F;if(ah>=I){ai=ah;break}ai=I}else{ai=ah}}while(0);F=ai>2.220446049250313e-16?ai:2.220446049250313e-16;a[e+5212+K|0]=F+9.9999998245167e-15<0.0|0;j=o+4|0;g[o>>2]=F;i=l+k|0;if(n){aj=0.0}else{aj=H>2.220446049250313e-16?H:2.220446049250313e-16}F=+g[d+332+(L*12|0)+4>>2];do{if(F>9.999999960041972e-13){B=+g[d+88+(L*12|0)+4>>2]*0.0/F*+g[M>>2];if(aj>=B){ak=aj;break}ak=B}else{ak=aj}}while(0);F=ak>2.220446049250313e-16?ak:2.220446049250313e-16;a[K+1+(e+5212)|0]=F+9.9999998245167e-15<0.0|0;k=o+8|0;g[j>>2]=F;m=l+i|0;if(n){al=0.0}else{al=H>2.220446049250313e-16?H:2.220446049250313e-16}F=+g[d+332+(L*12|0)+8>>2];do{if(F>9.999999960041972e-13){B=+g[d+88+(L*12|0)+8>>2]*0.0/F*+g[M>>2];if(al>=B){am=al;break}am=B}else{am=al}}while(0);F=am>2.220446049250313e-16?am:2.220446049250313e-16;a[K+2+(e+5212)|0]=F+9.9999998245167e-15<0.0|0;g[k>>2]=F;af=m;ag=J}M=o+12|0;do{if((c[G>>2]|0)!=0){F=+g[o>>2];n=o+4|0;H=+g[n>>2];if(F>H){B=H+(F-H)*+g[(c[N>>2]|0)+6496>>2];g[n>>2]=B;an=B}else{an=H}n=o+8|0;H=+g[n>>2];if(an<=H){break}g[n>>2]=H+(an-H)*+g[(c[N>>2]|0)+6496>>2]}}while(0);m=K+3|0;if((m|0)<(c[P>>2]|0)){p=af;J=ag;K=m;L=L+1|0;o=M}else{S=af;break}}return S|0}function C1(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,j=0,k=0,l=0,m=0.0,n=0.0,o=0.0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0.0,G=0.0,H=0.0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0.0,Q=0,S=0.0,T=0.0,U=0,V=0.0,W=0,X=0,Y=0,Z=0,_=0.0,$=0,aa=0,ab=0.0,ad=0,ae=0.0,af=0.0,ag=0.0,ah=0,ai=0.0,aj=0,ak=0.0,al=0.0,am=0,an=0.0,ao=0,ap=0.0,aq=0,ar=0.0,as=0,at=0.0;h=i;i=i+8|0;j=h|0;k=e+16|0;c[k>>2]=0;l=a+4864|0;if((c[l>>2]|0)<=0){m=0.0;n=0.0;o=-20.0;p=0;q=e+12|0;c[q>>2]=p;r=e+4|0;g[r>>2]=n;s=e|0;g[s>>2]=m;t=e+8|0;g[t>>2]=o;i=h;return p|0}u=a+4780|0;v=a+4832|0;w=a+4836|0;x=(f|0)!=0;y=f|0;z=a+5208|0;A=a+4776|0;B=a+4772|0;C=j|0;D=j+4|0;E=0;F=0.0;G=0.0;H=-20.0;I=a+4608|0;J=0;K=0;L=d;d=b;while(1){b=I+4|0;if((c[v>>2]|0)==0){M=0}else{M=c[90800+(K<<2)>>2]|0}N=(c[u>>2]|0)-(M+(c[I>>2]|0)<<(c[w>>2]|0)+1)-(c[a+4808+(c[a+5028+(K<<2)>>2]<<2)>>2]<<3)|0;O=d+4|0;P=1.0/+g[d>>2];do{if(x){if((c[f+8+(K<<2)>>2]|0)!=(N|0)){Q=366;break}S=+g[f+320+(K<<2)>>2];T=P*+g[f+164+(K<<2)>>2];U=(c[a+4872+(K<<2)>>2]|0)+E|0;Q=384}else{Q=366}}while(0);do{if((Q|0)==366){Q=0;V=+g[2197096+(N+116<<2)>>2];W=c[a+4872+(K<<2)>>2]|0;X=c[z>>2]|0;if((W+E|0)>(X|0)){Y=X-E+1|0;Z=(Y|0)>0?Y>>1:0}else{Z=W>>1}do{if((E|0)>(c[A>>2]|0)){if((Z|0)==0){_=0.0;$=E;break}else{aa=E;ab=0.0;ad=Z}while(1){W=ad-1|0;ae=+g[a+(aa<<2)>>2];af=+g[a+(aa+1<<2)>>2];ag=ab+ae*ae+af*af;if((W|0)==0){break}else{aa=aa+2|0;ab=ag;ad=W}}_=ag;$=(Z<<1)+E|0}else{if((E|0)>(c[B>>2]|0)){g[C>>2]=0.0;g[D>>2]=V;if((Z|0)==0){_=0.0;$=E;break}else{ah=E;ai=0.0;aj=Z}while(1){W=aj-1|0;af=+R(+(+g[a+(ah<<2)>>2]));ae=af- +g[j+(c[a+2304+(ah<<2)>>2]<<2)>>2];Y=ah+1|0;af=+R(+(+g[a+(Y<<2)>>2]));ak=af- +g[j+(c[a+2304+(Y<<2)>>2]<<2)>>2];al=ai+ae*ae+ak*ak;if((W|0)==0){break}else{ah=ah+2|0;ai=al;aj=W}}_=al;$=(Z<<1)+E|0;break}else{if((Z|0)==0){_=0.0;$=E;break}else{am=E;an=0.0;ao=Z}while(1){W=ao-1|0;ak=+R(+(+g[a+(am<<2)>>2]));ae=ak-V*+g[2164264+(c[a+2304+(am<<2)>>2]<<2)>>2];Y=am+1|0;ak=+R(+(+g[a+(Y<<2)>>2]));af=ak-V*+g[2164264+(c[a+2304+(Y<<2)>>2]<<2)>>2];ap=an+ae*ae+af*af;if((W|0)==0){break}else{am=am+2|0;an=ap;ao=W}}_=ap;$=(Z<<1)+E|0;break}}}while(0);if(x){c[f+8+(K<<2)>>2]=N;g[f+164+(K<<2)>>2]=_}V=P*_;af=+Ek(V>9.999999682655225e-21?V:9.999999682655225e-21)*.30102999566398114;if(x){g[f+320+(K<<2)>>2]=af;S=af;T=V;U=$;Q=384;break}else{g[L>>2]=V;aq=$;ar=af;break}}}while(0);if((Q|0)==384){Q=0;g[L>>2]=T;c[y>>2]=c[u>>2];aq=U;ar=S}P=G+ar;if(ar>0.0){N=~~(ar*10.0+.5);W=(N|0)>1?N:1;N=ac(W,W)|0;c[k>>2]=(c[k>>2]|0)+N;as=J+1|0;at=F+ar}else{as=J;at=F}af=H>ar?H:ar;N=K+1|0;if((N|0)<(c[l>>2]|0)){E=aq;F=at;G=P;H=af;I=b;J=as;K=N;L=L+4|0;d=O}else{m=at;n=P;o=af;p=as;break}}q=e+12|0;c[q>>2]=p;r=e+4|0;g[r>>2]=n;s=e|0;g[s>>2]=m;t=e+8|0;g[t>>2]=o;i=h;return p|0}function C2(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0.0,L=0,M=0,N=0.0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0.0,V=0.0,W=0.0,X=0,Y=0,Z=0.0,_=0.0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0.0,ak=0,al=0,am=0.0,an=0.0,ao=0.0,ap=0,aq=0,ar=0,as=0;d=i;i=i+504|0;e=d|0;f=d+160|0;j=d+320|0;k=a+76|0;l=c[k>>2]|0;if((l|0)<=0){i=d;return}m=a+72|0;n=d+344|0;o=e|0;p=f|0;q=a+85804|0;r=a+212|0;s=a+85796|0;t=a+216|0;u=j+12|0;v=j+8|0;w=j|0;x=j+4|0;y=j+16|0;z=0;A=c[m>>2]|0;B=l;while(1){if((A|0)>0){l=(z|0)==1;C=0;do{D=a+304+(z*10504|0)+(C*5252|0)|0;E=a+304+(z*10504|0)+(C*5252|0)+4608|0;NN(n|0,E|0,156)|0;do{if(l){F=a+10808+(C*5252|0)+4848|0;G=c[F>>2]|0;if((G|0)>0){H=0;I=G}else{break}while(1){G=a+10808+(C*5252|0)+4608+(H<<2)|0;if((c[G>>2]|0)<0){c[G>>2]=c[a+304+(C*5252|0)+4608+(H<<2)>>2];J=c[F>>2]|0}else{J=I}G=H+1|0;if((G|0)<(J|0)){H=G;I=J}else{break}}}}while(0);K=(c[a+304+(z*10504|0)+(C*5252|0)+4836>>2]|0)==0?.5:1.0;C0(a,b+(z*976|0)+(C*488|0)|0,D,o)|0;C1(D,o,p,j,0)|0;F=c[a+304+(z*10504|0)+(C*5252|0)+4848>>2]|0;G=a+304+(z*10504|0)+(C*5252|0)+4788|0;L=c[G>>2]|0;if((L|0)==2){M=F}else{M=(c[a+304+(z*10504|0)+(C*5252|0)+4792>>2]|0)==0?22:F}if((M|0)>0){F=a+304+(z*10504|0)+(C*5252|0)+4832|0;N=-0.0-K;O=0;P=0;while(1){Q=P+1|0;R=c[a+21360+(Q<<2)>>2]|0;S=R-(c[a+21360+(P<<2)>>2]|0)|0;if((O|0)<(R|0)){T=O;U=0.0;while(1){V=+g[a+304+(z*10504|0)+(C*5252|0)+(T<<2)>>2];W=U+V*V;X=T+1|0;if((X|0)<(R|0)){T=X;U=W}else{Y=R;Z=W;break}}}else{Y=O;Z=0.0}U=+(S|0);W=Z/U;h[(c[q>>2]|0)+190712+(z*704|0)+(C*176|0)+(P<<3)>>3]=W*999999986991104.0;h[(c[q>>2]|0)+201208+(z*352|0)+(C*176|0)+(P<<3)>>3]=+g[e+(P<<2)>>2]*999999986991104.0*+g[f+(P<<2)>>2]/U;U=+g[b+(z*976|0)+(C*488|0)+244+(P<<2)>>2];do{if(U>0.0){if((c[r>>2]|0)!=0){_=0.0;break}_=W/U}else{_=0.0}}while(0);U=_*+g[b+(z*976|0)+(C*488|0)+(P<<2)>>2];W=+g[(c[s>>2]|0)+24+(P<<2)>>2];h[(c[q>>2]|0)+189304+(z*704|0)+(C*176|0)+(P<<3)>>3]=(U>W?U:W)*999999986991104.0;h[(c[q>>2]|0)+199160+(z*352|0)+(C*176|0)+(P<<3)>>3]=0.0;if((c[F>>2]|0)!=0&(P|0)>10){h[(c[q>>2]|0)+199160+(z*352|0)+(C*176|0)+(P<<3)>>3]=+(c[90800+(P<<2)>>2]|0)*N}if((P|0)<21){S=(c[q>>2]|0)+199160+(z*352|0)+(C*176|0)+(P<<3)|0;h[S>>3]=+h[S>>3]-K*+(c[a+304+(z*10504|0)+(C*5252|0)+4608+(P<<2)>>2]|0)}if((Q|0)<(M|0)){O=Y;P=Q}else{break}}$=Y;aa=M;ab=c[G>>2]|0}else{$=0;aa=0;ab=L}do{if((ab|0)==2){P=c[a+304+(z*10504|0)+(C*5252|0)+4852>>2]|0;if((P|0)<13){ac=$;ad=aa;ae=P}else{break}while(1){P=c[a+21452+(ae<<2)>>2]|0;O=ae+1|0;F=c[a+21452+(O<<2)>>2]|0;D=F-P|0;S=(P|0)<(F|0);N=+(D|0);R=ae*3|0;T=(ae|0)<12;X=ac;af=0;ag=ad;while(1){if(S){ah=X;ai=P;W=0.0;while(1){U=+g[a+304+(z*10504|0)+(C*5252|0)+(ah<<2)>>2];aj=W+U*U;ak=ai+1|0;if((ak|0)<(F|0)){ah=ah+1|0;ai=ak;W=aj}else{break}}al=X+D|0;am=aj}else{al=X;am=0.0}W=am/N;U=W>1.0e-20?W:9.999999682655225e-21;ai=af+R|0;h[(c[q>>2]|0)+194616+(z*1248|0)+(C*312|0)+(ai<<3)>>3]=U*999999986991104.0;h[(c[q>>2]|0)+201912+(z*624|0)+(C*312|0)+(ai<<3)>>3]=+g[e+(ag<<2)>>2]*999999986991104.0*+g[f+(ag<<2)>>2]/N;W=+g[b+(z*976|0)+(C*488|0)+332+(ae*12|0)+(af<<2)>>2];if(W>0.0){an=U/W}else{an=0.0}if((c[r>>2]|0)==0){if((c[t>>2]|0)==0){ao=an}else{ap=426}}else{ap=426}if((ap|0)==426){ap=0;ao=0.0}W=ao*+g[b+(z*976|0)+(C*488|0)+88+(ae*12|0)+(af<<2)>>2];U=+g[(c[s>>2]|0)+112+(ae<<2)>>2];h[(c[q>>2]|0)+192120+(z*1248|0)+(C*312|0)+(ai<<3)>>3]=(W>U?W:U)*999999986991104.0;h[(c[q>>2]|0)+199864+(z*624|0)+(C*312|0)+(ai<<3)>>3]=+(c[a+304+(z*10504|0)+(C*5252|0)+4808+(af<<2)>>2]|0)*-2.0;if(T){ah=(c[q>>2]|0)+199864+(z*624|0)+(C*312|0)+(ai<<3)|0;h[ah>>3]=+h[ah>>3]-K*+(c[a+304+(z*10504|0)+(C*5252|0)+4608+(ag<<2)>>2]|0)}ah=af+1|0;if((ah|0)<3){X=al;af=ah;ag=ag+1|0}else{break}}if((O|0)<13){ac=al;ad=ad+3|0;ae=O}else{break}}}}while(0);c[(c[q>>2]|0)+201112+(z<<3)+(C<<2)>>2]=c[a+304+(z*10504|0)+(C*5252|0)+4780>>2];L=a+304+(z*10504|0)+(C*5252|0)+4844|0;c[(c[q>>2]|0)+203400+(z<<3)+(C<<2)>>2]=(c[L>>2]|0)+(c[a+304+(z*10504|0)+(C*5252|0)+4768>>2]|0);c[(c[q>>2]|0)+203416+(z<<3)+(C<<2)>>2]=c[L>>2];c[(c[q>>2]|0)+203160+(z<<3)+(C<<2)>>2]=c[u>>2];h[(c[q>>2]|0)+203208+(z<<4)+(C<<3)>>3]=+g[v>>2]*10.0;h[(c[q>>2]|0)+203240+(z<<4)+(C<<3)>>3]=+g[w>>2]*10.0;h[(c[q>>2]|0)+203176+(z<<4)+(C<<3)>>3]=+g[x>>2]*10.0;c[(c[q>>2]|0)+203272+(z<<3)+(C<<2)>>2]=c[y>>2];NN(E|0,n|0,156)|0;C=C+1|0;aq=c[m>>2]|0;}while((C|0)<(aq|0));ar=aq;as=c[k>>2]|0}else{ar=A;as=B}C=z+1|0;if((C|0)<(as|0)){z=C;A=ar;B=as}else{break}}i=d;return}function C3(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;d=Ey(a)|0;e=c[a+76>>2]|0;f=(d-(c[a+24>>2]<<3)|0)/(e|0)|0;g=(e<<11)-8|0;h=c[a+148>>2]|0;i=h-d|0;d=a+52144|0;j=(i|0)>(g|0)?g:i;c[d>>2]=j;if((j|0)<0){k=438}else{if((c[a+144>>2]|0)==0){l=j}else{k=438}}if((k|0)==438){c[d>>2]=0;l=0}d=ac(e,f)|0;e=a+52140|0;k=c[e>>2]|0;j=((k|0)<(l|0)?k:l)+d|0;d=(j|0)>(h|0)?h:j;c[a+21320>>2]=0;j=a+85804|0;a=c[j>>2]|0;if((a|0)==0){c[b>>2]=f;return d|0}c[a+203484>>2]=(f|0)/2|0;c[(c[j>>2]|0)+203488>>2]=c[e>>2];c[b>>2]=f;return d|0}function C4(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0.0;g=c[a+52144>>2]|0;h=(c[a+52140>>2]|0)+((f|0)==0?0:b)|0;f=a+85096|0;i=c[f>>2]|0;j=(i&1|0)==0;if(j){k=g}else{k=~~(+(g|0)*.9)}l=k*9|0;do{if((h*10|0|0)>(l|0)){k=h-((l|0)/10|0)|0;c[f>>2]=i|128;m=k+b|0;n=k}else{c[f>>2]=i&127;if((c[a+144>>2]|0)!=0|j^1){m=b;n=0;break}o=+(b|0);m=~~(o-o*.1);n=0}}while(0);b=(g*6|0|0)/10|0;g=((h|0)<(b|0)?h:b)-n|0;c[d>>2]=m;c[e>>2]=(g|0)<0?0:g;return}function C5(a,b){a=a|0;b=b|0;var d=0;d=a+52140|0;c[d>>2]=(c[d>>2]|0)-((c[b+4844>>2]|0)+(c[b+4768>>2]|0));return}function C6(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0;d=ac(c[a+76>>2]|0,b)|0;b=a+52140|0;e=(c[b>>2]|0)+d|0;d=(e|0)%8|0;f=e-d-(c[a+52144>>2]|0)|0;g=((f|0)>0?f:0)+d|0;d=a+21312|0;f=c[d>>2]|0;h=f<<3;i=(((h|0)<(g|0)?h:g)|0)/8|0;h=i<<3;c[a+21320>>2]=h;j=g-h|0;c[d>>2]=f-i;c[a+21324>>2]=j;c[b>>2]=e-h-j;return}function C7(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0){d=-1;return d|0}c[a+12>>2]=b;d=0;return d|0}function C8(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0){d=-1;return d|0}if((b|0)>2|(b|0)==0){d=-1;return d|0}c[a+8>>2]=b;d=0;return d|0}function C9(a,b){a=a|0;b=+b;var c=0;if((Cs(a)|0)==0){c=-1;return c|0}g[a+20>>2]=b;c=0;return c|0}function Da(a){a=a|0;var b=0.0;if((Cs(a)|0)==0){b=0.0;return+b}b=+g[a+20>>2];return+b}function Db(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0){d=-1;return d|0}c[a+16>>2]=b;d=0;return d|0}function Dc(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0|b>>>0>1){d=-1;return d|0}c[a+36>>2]=b;d=0;return d|0}function Dd(a,b){a=a|0;b=b|0;var d=0,e=0;do{if((Cs(a)|0)==0){d=-1}else{if((b|0)<0){c[a+44>>2]=0;d=0;break}e=a+44|0;if((b|0)>9){c[e>>2]=9;d=0;break}else{c[e>>2]=b;d=0;break}}}while(0);return d|0}function De(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0|b>>>0>4){d=-1;return d|0}c[a+48>>2]=b;d=0;return d|0}function Df(a,b){a=a|0;b=b|0;var d=0;do{if((Cs(a)|0)==0){d=-1}else{c[a+96>>2]=b;if((b|0)<=320){d=0;break}c[a+128>>2]=1;d=0}}while(0);return d|0}function Dg(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0|b>>>0>1){d=-1;return d|0}c[a+128>>2]=b;d=0;return d|0}function Dh(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0){d=-1;return d|0}c[a+132>>2]=b;d=0;return d|0}function Di(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0){d=-1;return d|0}c[a+136>>2]=b;d=0;return d|0}function Dj(a){a=a|0;var b=0;if((Cs(a)|0)==0){b=0;return b|0}b=c[a+132>>2]|0;return b|0}function Dk(a){a=a|0;var b=0;if((Cs(a)|0)==0){b=0;return b|0}b=c[a+136>>2]|0;return b|0}function Dl(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0){d=-1;return d|0}c[a+140>>2]=b;d=0;return d|0}function Dm(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0){d=-1;return d|0}c[a+148>>2]=b;d=0;return d|0}function Dn(a){a=a|0;var b=0;if((Cs(a)|0)==0){b=0;return b|0}b=c[a+148>>2]|0;return b|0}function Do(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0|b>>>0>4){d=-1;return d|0}c[a+156>>2]=b;d=0;return d|0}function Dp(a){a=a|0;var b=0;if((Cs(a)|0)==0){b=0;return b|0}b=c[a+156>>2]|0;return b|0}function Dq(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;if((Cs(a)|0)==0){d=-1;return d|0}e=(b|0)<0?0:b;f=(e|0)>9;c[a+164>>2]=f?9:e;g[a+160>>2]=0.0;d=f?-1:b>>31;return d|0}function Dr(a,b){a=a|0;b=+b;var d=0,e=0,f=0.0,h=0,i=0;if((Cs(a)|0)==0){d=-1;return d|0}e=b<0.0;f=e?0.0:b;h=f>9.999;b=h?9.99899959564209:f;i=~~b;c[a+164>>2]=i;g[a+160>>2]=b- +(i|0);d=h?-1:e<<31>>31;return d|0}function Ds(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0){d=-1;return d|0}c[a+168>>2]=b;d=0;return d|0}function Dt(a){a=a|0;var b=0;if((Cs(a)|0)==0){b=0;return b|0}b=c[a+168>>2]|0;return b|0}function Du(a,b){a=a|0;b=+b;var c=0;if((Cs(a)|0)==0){c=-1;return c|0}g[a+200>>2]=b;c=0;return c|0}function Dv(a){a=a|0;var b=0.0;if((Cs(a)|0)==0){b=0.0;return+b}b=+g[a+200>>2];return+b}function Dw(a,b){a=a|0;b=+b;var c=0;if((Cs(a)|0)==0){c=-1;return c|0}g[a+204>>2]=b;c=0;return c|0}function Dx(a){a=a|0;var b=0.0;if((Cs(a)|0)==0){b=0.0;return+b}b=+g[a+204>>2];return+b}function Dy(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0){d=-1;return d|0}c[a+220>>2]=b;d=0;return d|0}function Dz(a,b){a=a|0;b=+b;var c=0;if((Cs(a)|0)==0){c=-1;return c|0}g[a+224>>2]=b;c=0;return c|0}function DA(a){a=a|0;var b=0.0;if((Cs(a)|0)==0){b=0.0;return+b}b=+g[a+224>>2];return+b}function DB(a,b){a=a|0;b=+b;var c=0;if((Cs(a)|0)==0){c=-1;return c|0}g[a+228>>2]=b;c=0;return c|0}function DC(a){a=a|0;var b=0.0;if((Cs(a)|0)==0){b=0.0;return+b}b=+g[a+228>>2];return+b}function DD(a,b){a=a|0;b=+b;var c=0;if((Cs(a)|0)==0){c=-1;return c|0}g[a+236>>2]=b;c=0;return c|0}function DE(a){a=a|0;var b=0.0;if((Cs(a)|0)==0){b=0.0;return+b}b=+g[a+236>>2];return+b}function DF(a,b){a=a|0;b=+b;var c=0;if((Cs(a)|0)==0|b<0.0|b>1.0){c=-1;return c|0}g[a+248>>2]=b;c=0;return c|0}function DG(a){a=a|0;var b=0.0;if((Cs(a)|0)==0){b=0.0;return+b}b=+g[a+248>>2];return+b}function DH(a,b){a=a|0;b=b|0;var d=0;if((Cs(a)|0)==0){d=-1;return d|0}c[a+84>>2]=(b|0)!=0?2:1;d=0;return d|0}function DI(a,b){a=a|0;b=+b;var c=0;if((Cs(a)|0)==0){c=-1;return c|0}g[a+264>>2]=b;c=0;return c|0}function DJ(a){a=a|0;var b=0.0;if((Cs(a)|0)==0){b=0.0;return+b}b=+g[a+264>>2];return+b}function DK(a,b){a=a|0;b=+b;var c=0;if((Cs(a)|0)==0){c=-1;return c|0}g[a+268>>2]=b;c=0;return c|0}function DL(a){a=a|0;var b=0.0;if((Cs(a)|0)==0){b=0.0;return+b}b=+g[a+268>>2];return+b}function DM(a){a=a|0;var b=0,d=0;if((Cs(a)|0)==0){b=0;return b|0}d=c[a+288>>2]|0;if((Ct(d)|0)==0){b=0;return b|0}b=c[d+84760>>2]|0;return b|0}function DN(a){a=a|0;var b=0,d=0;if((Cs(a)|0)==0){b=0;return b|0}d=c[a+288>>2]|0;if((Ct(d)|0)==0){b=0;return b|0}b=(c[d+76>>2]|0)*576|0;return b|0}function DO(a,b){a=a|0;b=+b;if((Cs(a)|0)==0){return}g[a+252>>2]=b;return}function DP(a){a=a|0;var b=0.0;if((Cs(a)|0)==0){b=0.0;return+b}b=+g[a+252>>2];return+b}function DQ(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;g=i;i=i+8|0;h=g|0;c[h>>2]=0;j=e+2304|0;k=(c[e+5208>>2]|0)+2&-2;l=(k|0)>576?576:k;k=(f|0)!=0;if(k){c[f+4>>2]=0;m=l}else{m=l}while(1){if((m|0)<=1){n=688;break}l=m-2|0;if((c[e+2304+(l<<2)>>2]|c[e+2304+(m-1<<2)>>2]|0)==0){m=l}else{n=690;break}}do{if((n|0)==690){c[e+4776>>2]=m;if((m|0)>3){o=0;p=0;q=m}else{n=693;break}while(1){l=q-4|0;r=c[e+2304+(l<<2)>>2]|0;s=c[e+2304+(q-3<<2)>>2]|0;t=c[e+2304+(q-2<<2)>>2]|0;u=c[e+2304+(q-1<<2)>>2]|0;if((s|r|t|u)>>>0>1){v=o;w=p;x=q;break}y=(((r<<1)+s<<1)+t<<1)+u|0;u=(d[66760+y|0]|0)+p|0;t=(d[66712+y|0]|0)+o|0;if((l|0)>3){o=t;p=u;q=l}else{v=t;w=u;x=l;break}}c[h>>2]=w;l=e+4840|0;c[l>>2]=0;if((w|0)<=(v|0)){z=w;A=x;break}c[h>>2]=v;c[l>>2]=1;z=v;A=x}else if((n|0)==688){c[e+4776>>2]=m;n=693}}while(0);if((n|0)==693){c[h>>2]=0;c[e+4840>>2]=0;z=0;A=m}c[e+5184>>2]=z;z=e+4772|0;c[z>>2]=A;if((A|0)==0){B=c[h>>2]|0;i=g;return B|0}m=e+4788|0;n=c[m>>2]|0;do{if((n|0)==2){x=(c[b+21464>>2]|0)*3|0;C=(x|0)>(A|0)?A:x;D=A}else if((n|0)==0){x=a[A-2+(b+85100)|0]|0;c[e+4824>>2]=x;v=a[A-1+(b+85100)|0]|0;c[e+4828>>2]=v;w=c[b+21360+(x+2+v<<2)>>2]|0;v=c[b+21360+(x+1<<2)>>2]|0;if((w|0)>=(A|0)){C=v;D=w;break}c[e+4804>>2]=c6[c[b+85816>>2]&255](e+2304+(w<<2)|0,e+2304+(A<<2)|0,h)|0;C=v;D=w}else{c[e+4824>>2]=7;c[e+4828>>2]=13;w=c[b+21392>>2]|0;C=(w|0)>(A|0)?A:w;D=A}}while(0);n=(C|0)<(A|0)?C:A;C=(D|0)<(A|0)?D:A;if((n|0)>0){c[e+4796>>2]=c6[c[b+85816>>2]&255](j,e+2304+(n<<2)|0,h)|0}if((n|0)<(C|0)){c[e+4800>>2]=c6[c[b+85816>>2]&255](e+2304+(n<<2)|0,e+2304+(C<<2)|0,h)|0}if((c[b+36>>2]|0)==2){C=e+4768|0;c[C>>2]=c[h>>2];DR(b,e);c[h>>2]=c[C>>2]}if(!k){B=c[h>>2]|0;i=g;return B|0}if((c[m>>2]|0)!=0){B=c[h>>2]|0;i=g;return B|0}m=c[z>>2]|0;z=0;while(1){if((c[b+21360+(z<<2)>>2]|0)<(m|0)){z=z+1|0}else{break}}c[f+4>>2]=z;B=c[h>>2]|0;i=g;return B|0}function DR(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;e=i;i=i+5672|0;f=e|0;g=e+8|0;h=e+16|0;j=e+24|0;k=e+32|0;l=e+5288|0;m=e+5384|0;n=e+5480|0;o=e+5576|0;p=b+2304|0;q=c[b+4788>>2]|0;L926:do{if((q|0)==2){if((c[a+76>>2]|0)==1){i=e;return}else{r=k;s=b;NN(r|0,s|0,5252)|0;t=r;u=s;v=722;break}}else{s=k;r=b;NN(s|0,r|0,5252)|0;if((q|0)!=0){t=s;u=r;v=722;break}w=c[b+4772>>2]|0;c[l>>2]=1e5;c[l+4>>2]=1e5;c[l+8>>2]=1e5;c[l+12>>2]=1e5;c[l+16>>2]=1e5;c[l+20>>2]=1e5;c[l+24>>2]=1e5;c[l+28>>2]=1e5;c[l+32>>2]=1e5;c[l+36>>2]=1e5;c[l+40>>2]=1e5;c[l+44>>2]=1e5;c[l+48>>2]=1e5;c[l+52>>2]=1e5;c[l+56>>2]=1e5;c[l+60>>2]=1e5;c[l+64>>2]=1e5;c[l+68>>2]=1e5;c[l+72>>2]=1e5;c[l+76>>2]=1e5;c[l+80>>2]=1e5;c[l+84>>2]=1e5;c[l+88>>2]=1e5;x=a+85816|0;y=0;while(1){z=y+1|0;A=c[a+21360+(z<<2)>>2]|0;if((A|0)>=(w|0)){break}c[j>>2]=0;B=b+2304+(A<<2)|0;A=c6[c[x>>2]&255](p,B,j)|0;C=0;do{D=C+y|0;E=c[a+21360+(D+2<<2)>>2]|0;if((E|0)>=(w|0)){break}c[h>>2]=c[j>>2];F=c6[c[x>>2]&255](B,b+2304+(E<<2)|0,h)|0;E=l+(D<<2)|0;G=c[h>>2]|0;if((c[E>>2]|0)>(G|0)){c[E>>2]=G;c[m+(D<<2)>>2]=y;c[n+(D<<2)>>2]=A;c[o+(D<<2)>>2]=F}C=C+1|0;}while((C|0)<8);if((z|0)<16){y=z}else{break}}y=k+4772|0;w=c[y>>2]|0;C=k+5184|0;A=b+4768|0;B=b+2304+(w<<2)|0;F=b+4824|0;D=b+4828|0;G=b+4796|0;E=b+4800|0;H=b+4804|0;I=2;while(1){J=c[a+21360+(I<<2)>>2]|0;if((J|0)>=(w|0)){K=y;L=s;M=r;break L926}N=I-2|0;O=(c[C>>2]|0)+(c[l+(N<<2)>>2]|0)|0;c[g>>2]=O;if((c[A>>2]|0)<=(O|0)){K=y;L=s;M=r;break L926}O=c6[c[x>>2]&255](b+2304+(J<<2)|0,B,g)|0;J=c[g>>2]|0;if((c[A>>2]|0)>(J|0)){NN(r|0,s|0,5252)|0;c[A>>2]=J;J=c[m+(N<<2)>>2]|0;c[F>>2]=J;c[D>>2]=N-J;c[G>>2]=c[n+(N<<2)>>2];c[E>>2]=c[o+(N<<2)>>2];c[H>>2]=O}O=I+1|0;if((O|0)<23){I=O}else{K=y;L=s;M=r;break}}}}while(0);if((v|0)==722){K=k+4772|0;L=t;M=u}u=c[K>>2]|0;if((u|0)==0){i=e;return}if((c[b+2304+(u-1<<2)>>2]|c[b+2304+(u-2<<2)>>2])>>>0>1){i=e;return}u=(c[b+4776>>2]|0)+2|0;if((u|0)>576){i=e;return}NN(L|0,M|0,5252)|0;c[k+4776>>2]=u;t=c[K>>2]|0;if((u|0)>(t|0)){g=0;h=0;j=u;while(1){q=j-4|0;r=(((c[b+2304+(q<<2)>>2]<<1)+(c[b+2304+(j-3<<2)>>2]|0)<<1)+(c[b+2304+(j-2<<2)>>2]|0)<<1)+(c[b+2304+(j-1<<2)>>2]|0)|0;s=(d[66760+r|0]|0)+h|0;y=(d[66712+r|0]|0)+g|0;if((q|0)>(t|0)){g=y;h=s;j=q}else{P=y;Q=s;R=q;break}}}else{P=0;Q=0;R=u}c[K>>2]=R;K=(Q|0)>(P|0);u=K?P:Q;c[k+4840>>2]=K&1;K=k+5184|0;c[K>>2]=u;if((c[k+4788>>2]|0)!=0){Q=k+4768|0;c[Q>>2]=u;u=c[a+21392>>2]|0;P=(u|0)>(R|0)?R:u;if((P|0)>0){c[k+4796>>2]=c6[c[a+85816>>2]&255](p,b+2304+(P<<2)|0,Q)|0}if((R|0)>(P|0)){c[k+4800>>2]=c6[c[a+85816>>2]&255](b+2304+(P<<2)|0,b+2304+(R<<2)|0,Q)|0}if((c[b+4768>>2]|0)<=(c[Q>>2]|0)){i=e;return}NN(M|0,L|0,5252)|0;i=e;return}Q=b+4768|0;P=a+85816|0;k=b+2304+(R<<2)|0;p=b+4824|0;u=b+4828|0;j=b+4796|0;h=b+4800|0;g=b+4804|0;t=2;while(1){q=c[a+21360+(t<<2)>>2]|0;if((q|0)>=(R|0)){v=763;break}s=t-2|0;y=(c[K>>2]|0)+(c[l+(s<<2)>>2]|0)|0;c[f>>2]=y;if((c[Q>>2]|0)<=(y|0)){v=764;break}y=c6[c[P>>2]&255](b+2304+(q<<2)|0,k,f)|0;q=c[f>>2]|0;if((c[Q>>2]|0)>(q|0)){NN(M|0,L|0,5252)|0;c[Q>>2]=q;q=c[m+(s<<2)>>2]|0;c[p>>2]=q;c[u>>2]=s-q;c[j>>2]=c[n+(s<<2)>>2];c[h>>2]=c[o+(s<<2)>>2];c[g>>2]=y}y=t+1|0;if((y|0)<23){t=y}else{v=759;break}}if((v|0)==759){i=e;return}else if((v|0)==763){i=e;return}else if((v|0)==764){i=e;return}}function DS(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,i=0.0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0;f=d+4780|0;h=c[f>>2]|0;i=+g[2206176+(h<<2)>>2];if(+g[d+4764>>2]>8206.0/i){j=1e5;return j|0}k=d+2304|0;l=(e|0)!=0;if(l){m=(h|0)==(c[e>>2]|0)}else{m=0}h=d+4788|0;n=(c[h>>2]|0)==2?38:21;o=d+4832|0;p=d+4836|0;q=.5945999622344971/i;r=d+5208|0;s=n+1|0;t=e+4|0;u=0;v=0;w=k;x=0;y=0;z=k;k=b;A=b;while(1){if(m){B=771}else{if((c[h>>2]|0)==0){B=771}else{C=-1;B=779}}do{if((B|0)==771){B=0;if((c[o>>2]|0)==0){D=0}else{D=c[90800+(v<<2)>>2]|0}E=(c[f>>2]|0)-(D+(c[d+4608+(v<<2)>>2]|0)<<(c[p>>2]|0)+1)-(c[d+4808+(c[d+5028+(v<<2)>>2]<<2)>>2]<<3)|0;if(!m){C=E;B=779;break}if((c[e+8+(v<<2)>>2]|0)!=(E|0)){C=E;B=779;break}if((x|0)!=0){D$(x,i,k,z)}if((y|0)==0){F=k;G=z;H=0;I=0;J=v;break}else{K=0}while(1){E=K|1;L=q<=+g[k+(E<<2)>>2]|0;c[z+(K<<2)>>2]=q<=+g[k+(K<<2)>>2];c[z+(E<<2)>>2]=L;L=K+2|0;if(L>>>0<y>>>0){K=L}else{F=k;G=z;H=0;I=0;J=v;break}}}}while(0);if((B|0)==779){B=0;L=c[d+4872+(v<<2)>>2]|0;E=c[r>>2]|0;if((L+u|0)>(E|0)){M=E-u+1|0;NM(d+2304+(E<<2)|0,0,576-E<<2|0);N=(M|0)<0?0:M;O=s}else{N=L;O=v}L=(x|0)!=0;M=(y|0)==0;E=M&(L^1);P=E?w:z;Q=E?A:k;do{if(l){E=c[t>>2]|0;if((E|0)<1|(O|0)<(E|0)){B=787;break}E=c[e+8+(O<<2)>>2]|0;if((E|0)<1|(C|0)<(E|0)){B=787;break}if(L){D$(x,i,Q,P);R=A;S=w}else{R=Q;S=P}T=R;U=S;V=N+y|0;W=0}else{B=787}}while(0);if((B|0)==787){B=0;if(M){X=Q;Y=P}else{L=0;while(1){E=L|1;Z=q<=+g[Q+(E<<2)>>2]|0;c[P+(L<<2)>>2]=q<=+g[Q+(L<<2)>>2];c[P+(E<<2)>>2]=Z;Z=L+2|0;if(Z>>>0<y>>>0){L=Z}else{X=A;Y=w;break}}}T=X;U=Y;V=0;W=N+x|0}if((N|0)<1){B=791;break}else{F=T;G=U;H=V;I=W;J=O}}if((J|0)>(n|0)){_=A;$=w;aa=u}else{L=c[d+4872+(J<<2)>>2]|0;_=A+(L<<2)|0;$=w+(L<<2)|0;aa=L+u|0}L=J+1|0;if((L|0)>(n|0)){B=798;break}else{u=aa;v=L;w=$;x=I;y=H;z=G;k=F;A=_}}do{if((B|0)==798){if((I|0)!=0){D$(I,i,F,G)}if((H|0)==0){break}else{ab=0}do{_=ab|1;A=q<=+g[F+(_<<2)>>2]|0;c[G+(ab<<2)>>2]=q<=+g[F+(ab<<2)>>2];c[G+(_<<2)>>2]=A;ab=ab+2|0;}while(ab>>>0<H>>>0)}else if((B|0)==791){if((V|0)!=0){A=0;do{_=A|1;k=q<=+g[T+(_<<2)>>2]|0;c[U+(A<<2)>>2]=q<=+g[T+(A<<2)>>2];c[U+(_<<2)>>2]=k;A=A+2|0;}while(A>>>0<V>>>0)}if((W|0)==0){break}D$(W,i,T,U)}}while(0);do{if((c[a+85096>>2]&2|0)!=0){i=.634521682242439/+g[2206176+((c[p>>2]|0)+(c[f>>2]|0)<<2)>>2];U=d+4860|0;T=c[U>>2]|0;if((T|0)>0){ac=0;ad=0;ae=T}else{break}while(1){T=c[d+4872+(ad<<2)>>2]|0;W=T+ac|0;if((c[a+84936+(ad<<2)>>2]|0)!=0&(T|0)>0){T=ac;do{V=d+2304+(T<<2)|0;if(+g[b+(T<<2)>>2]<i){af=0}else{af=c[V>>2]|0}c[V>>2]=af;T=T+1|0;}while((T|0)<(W|0));ag=c[U>>2]|0}else{ag=ae}T=ad+1|0;if((T|0)<(ag|0)){ac=W;ad=T;ae=ag}else{break}}}}while(0);j=DQ(a,d,e)|0;return j|0}function DT(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0;f=e+(b*10504|0)+(d*5252|0)|0;g=e+(b*10504|0)+(d*5252|0)+4860|0;h=c[g>>2]|0;if((h|0)>0){i=0;j=0;k=0;l=h;while(1){m=(c[e+(b*10504|0)+(d*5252|0)+4872+(k<<2)>>2]|0)+i|0;n=i;while(1){if((n|0)>=(m|0)){break}if((c[e+(b*10504|0)+(d*5252|0)+2304+(n<<2)>>2]|0)==0){n=n+1|0}else{break}}if((n|0)==(m|0)){c[e+(b*10504|0)+(d*5252|0)+4608+(k<<2)>>2]=-2;o=-2;p=c[g>>2]|0}else{o=j;p=l}q=k+1|0;if((q|0)<(p|0)){i=m;j=o;k=q;l=p}else{r=o;s=p;break}}}else{r=0;s=h}h=e+(b*10504|0)+(d*5252|0)+4836|0;p=e+(b*10504|0)+(d*5252|0)+4832|0;do{if((c[h>>2]|0)==0){if((c[p>>2]|0)!=0){t=r;break}o=(s|0)>0;if(o){u=0;v=0}else{t=r;break}do{l=c[e+(b*10504|0)+(d*5252|0)+4608+(u<<2)>>2]|0;v=((l|0)>0?l:0)|v;u=u+1|0;}while((u|0)<(s|0));if(!((v&1|0)==0&(v|0)!=0)){t=r;break}if(o){m=0;n=s;while(1){l=e+(b*10504|0)+(d*5252|0)+4608+(m<<2)|0;k=c[l>>2]|0;if((k|0)>0){c[l>>2]=k>>1;w=c[g>>2]|0}else{w=n}k=m+1|0;if((k|0)<(w|0)){m=k;n=w}else{break}}}c[h>>2]=1;t=1}else{t=r}}while(0);do{if((c[p>>2]|0)==0){if((c[e+(b*10504|0)+(d*5252|0)+4788>>2]|0)==2){x=t;break}if((c[a+76>>2]|0)==2){y=11}else{x=t;break}while(1){if((y|0)>=21){break}r=c[e+(b*10504|0)+(d*5252|0)+4608+(y<<2)>>2]|0;if((r|0)>=(c[90800+(y<<2)>>2]|0)|(r|0)==-2){y=y+1|0}else{break}}if((y|0)!=21){x=t;break}r=e+(b*10504|0)+(d*5252|0)+4652|0;h=c[r>>2]|0;if((h|0)>0){c[r>>2]=h-(c[22711]|0)}h=e+(b*10504|0)+(d*5252|0)+4656|0;r=c[h>>2]|0;if((r|0)>0){c[h>>2]=r-(c[22712]|0)}r=e+(b*10504|0)+(d*5252|0)+4660|0;h=c[r>>2]|0;if((h|0)>0){c[r>>2]=h-(c[22713]|0)}h=e+(b*10504|0)+(d*5252|0)+4664|0;r=c[h>>2]|0;if((r|0)>0){c[h>>2]=r-(c[22714]|0)}r=e+(b*10504|0)+(d*5252|0)+4668|0;h=c[r>>2]|0;if((h|0)>0){c[r>>2]=h-(c[22715]|0)}h=e+(b*10504|0)+(d*5252|0)+4672|0;r=c[h>>2]|0;if((r|0)>0){c[h>>2]=r-(c[22716]|0)}r=e+(b*10504|0)+(d*5252|0)+4676|0;h=c[r>>2]|0;if((h|0)>0){c[r>>2]=h-(c[22717]|0)}h=e+(b*10504|0)+(d*5252|0)+4680|0;r=c[h>>2]|0;if((r|0)>0){c[h>>2]=r-(c[22718]|0)}r=e+(b*10504|0)+(d*5252|0)+4684|0;h=c[r>>2]|0;if((h|0)>0){c[r>>2]=h-(c[22719]|0)}h=e+(b*10504|0)+(d*5252|0)+4688|0;r=c[h>>2]|0;if((r|0)>0){c[h>>2]=r-(c[22720]|0)}c[p>>2]=1;x=1}else{x=t}}while(0);NM(e+21008+((d<<2)+4<<2)|0,0,16);do{if((c[a+76>>2]|0)==2&(b|0)==1){if((c[e+(d*5252|0)+4788>>2]|0)==2){z=x;break}if((c[e+10504+(d*5252|0)+4788>>2]|0)==2){z=x;break}t=c[19374]|0;p=c[19375]|0;y=t;while(1){if((y|0)>=(p|0)){break}r=c[e+10504+(d*5252|0)+4608+(y<<2)>>2]|0;if((c[e+(d*5252|0)+4608+(y<<2)>>2]|0)!=(r|0)&(r|0)>-1){break}else{y=y+1|0}}if((y|0)==(p|0)){if((t|0)<(p|0)){r=t+1|0;NM(e+10504+(d*5252|0)+4608+(t<<2)|0,-1|0,((p|0)>(r|0)?p:r)-t<<2|0)}c[e+21024+(d<<4)>>2]=1}r=c[19376]|0;h=p;while(1){if((h|0)>=(r|0)){break}w=c[e+10504+(d*5252|0)+4608+(h<<2)>>2]|0;if((c[e+(d*5252|0)+4608+(h<<2)>>2]|0)!=(w|0)&(w|0)>-1){break}else{h=h+1|0}}if((h|0)==(r|0)){if((p|0)<(r|0)){t=p+1|0;NM(e+10504+(d*5252|0)+4608+(p<<2)|0,-1|0,((r|0)>(t|0)?r:t)-p<<2|0)}c[e+21024+(d<<4)+4>>2]=1}t=c[19377]|0;y=r;while(1){if((y|0)>=(t|0)){break}w=c[e+10504+(d*5252|0)+4608+(y<<2)>>2]|0;if((c[e+(d*5252|0)+4608+(y<<2)>>2]|0)!=(w|0)&(w|0)>-1){break}else{y=y+1|0}}if((y|0)==(t|0)){if((r|0)<(t|0)){p=r+1|0;NM(e+10504+(d*5252|0)+4608+(r<<2)|0,-1|0,((t|0)>(p|0)?t:p)-r<<2|0)}c[e+21024+(d<<4)+8>>2]=1}p=c[19378]|0;h=t;while(1){if((h|0)>=(p|0)){break}w=c[e+10504+(d*5252|0)+4608+(h<<2)>>2]|0;if((c[e+(d*5252|0)+4608+(h<<2)>>2]|0)!=(w|0)&(w|0)>-1){break}else{h=h+1|0}}if((h|0)==(p|0)){if((t|0)<(p|0)){r=t+1|0;NM(e+10504+(d*5252|0)+4608+(t<<2)|0,-1|0,((p|0)>(r|0)?p:r)-t<<2|0)}c[e+21024+(d<<4)+12>>2]=1}r=c[e+10504+(d*5252|0)+4608>>2]|0;if((r|0)==-1){A=0;B=0}else{A=(r|0)>0?r:0;B=1}r=c[e+10504+(d*5252|0)+4612>>2]|0;if((r|0)==-1){C=A;D=B}else{C=(A|0)<(r|0)?r:A;D=B+1|0}r=c[e+10504+(d*5252|0)+4616>>2]|0;if((r|0)==-1){E=C;F=D}else{E=(C|0)<(r|0)?r:C;F=D+1|0}r=c[e+10504+(d*5252|0)+4620>>2]|0;if((r|0)==-1){G=E;H=F}else{G=(E|0)<(r|0)?r:E;H=F+1|0}r=c[e+10504+(d*5252|0)+4624>>2]|0;if((r|0)==-1){I=G;J=H}else{I=(G|0)<(r|0)?r:G;J=H+1|0}r=c[e+10504+(d*5252|0)+4628>>2]|0;if((r|0)==-1){K=I;L=J}else{K=(I|0)<(r|0)?r:I;L=J+1|0}r=c[e+10504+(d*5252|0)+4632>>2]|0;if((r|0)==-1){M=K;N=L}else{M=(K|0)<(r|0)?r:K;N=L+1|0}r=c[e+10504+(d*5252|0)+4636>>2]|0;if((r|0)==-1){O=M;P=N}else{O=(M|0)<(r|0)?r:M;P=N+1|0}r=c[e+10504+(d*5252|0)+4640>>2]|0;if((r|0)==-1){Q=O;R=P}else{Q=(O|0)<(r|0)?r:O;R=P+1|0}r=c[e+10504+(d*5252|0)+4644>>2]|0;if((r|0)==-1){S=Q;T=R}else{S=(Q|0)<(r|0)?r:Q;T=R+1|0}r=c[e+10504+(d*5252|0)+4648>>2]|0;if((r|0)==-1){U=S;V=T}else{U=(S|0)<(r|0)?r:S;V=T+1|0}r=c[e+10504+(d*5252|0)+4652>>2]|0;if((r|0)==-1){W=0;X=0}else{W=(r|0)>0?r:0;X=1}r=c[e+10504+(d*5252|0)+4656>>2]|0;if((r|0)==-1){Y=W;Z=X}else{Y=(W|0)<(r|0)?r:W;Z=X+1|0}r=c[e+10504+(d*5252|0)+4660>>2]|0;if((r|0)==-1){_=Y;$=Z}else{_=(Y|0)<(r|0)?r:Y;$=Z+1|0}r=c[e+10504+(d*5252|0)+4664>>2]|0;if((r|0)==-1){aa=_;ab=$}else{aa=(_|0)<(r|0)?r:_;ab=$+1|0}r=c[e+10504+(d*5252|0)+4668>>2]|0;if((r|0)==-1){ad=aa;ae=ab}else{ad=(aa|0)<(r|0)?r:aa;ae=ab+1|0}r=c[e+10504+(d*5252|0)+4672>>2]|0;if((r|0)==-1){af=ad;ag=ae}else{af=(ad|0)<(r|0)?r:ad;ag=ae+1|0}r=c[e+10504+(d*5252|0)+4676>>2]|0;if((r|0)==-1){ah=af;ai=ag}else{ah=(af|0)<(r|0)?r:af;ai=ag+1|0}r=c[e+10504+(d*5252|0)+4680>>2]|0;if((r|0)==-1){aj=ah;ak=ai}else{aj=(ah|0)<(r|0)?r:ah;ak=ai+1|0}r=c[e+10504+(d*5252|0)+4684>>2]|0;if((r|0)==-1){al=aj;am=ak}else{al=(aj|0)<(r|0)?r:aj;am=ak+1|0}r=c[e+10504+(d*5252|0)+4688>>2]|0;if((r|0)==-1){an=am;ao=al}else{an=am+1|0;ao=(al|0)<(r|0)?r:al}r=e+10504+(d*5252|0)+4844|0;y=e+10504+(d*5252|0)+4784|0;w=0;while(1){do{if((U|0)<(c[73408+(w<<2)>>2]|0)){if((ao|0)>=(c[73280+(w<<2)>>2]|0)){break}s=ac(c[73344+(w<<2)>>2]|0,V)|0;v=(ac(c[73216+(w<<2)>>2]|0,an)|0)+s|0;if((c[r>>2]|0)<=(v|0)){break}c[r>>2]=v;c[y>>2]=w}}while(0);v=w+1|0;if(v>>>0<16){w=v}else{z=0;break}}}else{z=x}}while(0);x=c[g>>2]|0;if((x|0)>0){an=0;V=x;while(1){x=e+(b*10504|0)+(d*5252|0)+4608+(an<<2)|0;if((c[x>>2]|0)==-2){c[x>>2]=0;ap=c[g>>2]|0}else{ap=V}x=an+1|0;if((x|0)<(ap|0)){an=x;V=ap}else{break}}}if((z|0)==0){return}DU(a,f)|0;return}function DU(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0;d=i;i=i+16|0;e=d|0;if((c[a+76>>2]|0)==2){do{if((c[b+4788>>2]|0)==2){f=(c[b+4792>>2]|0)==0?77520:77584}else{a=b+4832|0;if((c[a>>2]|0)==0){g=11}else{f=77648;break}while(1){if((g|0)>=21){break}if((c[b+4608+(g<<2)>>2]|0)<(c[90800+(g<<2)>>2]|0)){break}else{g=g+1|0}}if((g|0)!=21){f=77648;break}c[a>>2]=1;h=b+4652|0;c[h>>2]=(c[h>>2]|0)-(c[22711]|0);h=b+4656|0;c[h>>2]=(c[h>>2]|0)-(c[22712]|0);h=b+4660|0;c[h>>2]=(c[h>>2]|0)-(c[22713]|0);h=b+4664|0;c[h>>2]=(c[h>>2]|0)-(c[22714]|0);h=b+4668|0;c[h>>2]=(c[h>>2]|0)-(c[22715]|0);h=b+4672|0;c[h>>2]=(c[h>>2]|0)-(c[22716]|0);h=b+4676|0;c[h>>2]=(c[h>>2]|0)-(c[22717]|0);h=b+4680|0;c[h>>2]=(c[h>>2]|0)-(c[22718]|0);h=b+4684|0;c[h>>2]=(c[h>>2]|0)-(c[22719]|0);h=b+4688|0;c[h>>2]=(c[h>>2]|0)-(c[22720]|0);f=77648}}while(0);g=c[b+4868>>2]|0;if((g|0)>0){h=0;j=0;do{k=c[b+4608+(h<<2)>>2]|0;j=(j|0)<(k|0)?k:j;h=h+1|0;}while((h|0)<(g|0));l=(g|0)>1?g:1;m=j}else{l=0;m=0}j=c[b+4860>>2]|0;if((l|0)<(j|0)){g=l;l=0;while(1){h=c[b+4608+(g<<2)>>2]|0;k=(l|0)<(h|0)?h:l;h=g+1|0;if((h|0)<(j|0)){g=h;l=k}else{n=k;break}}}else{n=0}l=b+4844|0;c[l>>2]=1e5;g=b+4784|0;j=0;k=1e5;while(1){do{if((m|0)<(c[73408+(j<<2)>>2]|0)){if((n|0)>=(c[73280+(j<<2)>>2]|0)){o=k;break}h=c[f+(j<<2)>>2]|0;if((k|0)<=(h|0)){o=k;break}c[l>>2]=h;c[g>>2]=j;o=h}else{o=k}}while(0);a=j+1|0;if((a|0)<16){j=a;k=o}else{break}}p=(o|0)==1e5|0;i=d;return p|0}o=(c[b+4832>>2]|0)==0;NM(e|0,0,16);k=o?0:2;do{if((c[b+4788>>2]|0)==2){j=0;g=0;while(1){l=c[121008+(k*48|0)+(g<<2)>>2]|0;f=(l|0)/3|0;if((l|0)>2){l=e+(g<<2)|0;n=(f|0)>1;m=0;a=j;h=c[l>>2]|0;while(1){q=a*3|0;r=c[b+4608+(q<<2)>>2]|0;if((r|0)>(h|0)){c[l>>2]=r;s=r}else{s=h}r=c[b+4608+(q+1<<2)>>2]|0;if((r|0)>(s|0)){c[l>>2]=r;t=r}else{t=s}r=c[b+4608+(q+2<<2)>>2]|0;if((r|0)>(t|0)){c[l>>2]=r;u=r}else{u=t}r=m+1|0;if((r|0)<(f|0)){m=r;a=a+1|0;h=u}else{break}}v=(n?f:1)+j|0}else{v=j}h=g+1|0;if((h|0)<4){j=v;g=h}else{break}}w=1;x=c[e>>2]|0;y=c[e+4>>2]|0;z=c[e+8>>2]|0;A=c[e+12>>2]|0}else{g=c[120992+(k*48|0)>>2]|0;if((g|0)>0){j=e|0;h=0;a=0;while(1){m=c[b+4608+(h<<2)>>2]|0;if((m|0)>(a|0)){c[j>>2]=m;B=m}else{B=a}m=h+1|0;if((m|0)<(g|0)){h=m;a=B}else{C=g;D=B;break}}}else{C=0;D=0}g=c[120996+(k*48|0)>>2]|0;if((g|0)>0){a=e+4|0;h=0;j=C;m=0;while(1){l=c[b+4608+(j<<2)>>2]|0;if((l|0)>(m|0)){c[a>>2]=l;E=l}else{E=m}l=h+1|0;if((l|0)<(g|0)){h=l;j=j+1|0;m=E}else{break}}F=g+C|0;G=E}else{F=C;G=0}m=c[121e3+(k*48|0)>>2]|0;if((m|0)>0){j=e+8|0;h=0;a=F;l=0;while(1){r=c[b+4608+(a<<2)>>2]|0;if((r|0)>(l|0)){c[j>>2]=r;H=r}else{H=l}r=h+1|0;if((r|0)<(m|0)){h=r;a=a+1|0;l=H}else{break}}I=m+F|0;J=H}else{I=F;J=0}l=c[121004+(k*48|0)>>2]|0;if((l|0)<=0){w=0;x=D;y=G;z=J;A=0;break}a=e+12|0;h=0;j=I;g=0;while(1){r=c[b+4608+(j<<2)>>2]|0;if((r|0)>(g|0)){c[a>>2]=r;K=r}else{K=g}r=h+1|0;if((r|0)<(l|0)){h=r;j=j+1|0;g=K}else{w=0;x=D;y=G;z=J;A=K;break}}}}while(0);K=((y|0)>(c[129636+(k<<4)>>2]|0))+((x|0)>(c[129632+(k<<4)>>2]|0))+((z|0)>(c[129640+(k<<4)>>2]|0))+((A|0)>(c[129644+(k<<4)>>2]|0))|0;if((K|0)!=0){p=K;i=d;return p|0}K=120992+(k*48|0)+(w<<4)|0;c[b+5188>>2]=K;J=c[121368+(x<<2)>>2]|0;c[b+5192>>2]=J;x=c[121368+(y<<2)>>2]|0;c[b+5196>>2]=x;y=c[121368+(z<<2)>>2]|0;c[b+5200>>2]=y;z=c[121368+(A<<2)>>2]|0;c[b+5204>>2]=z;if(o){c[b+4784>>2]=(y<<2)+((J*5|0)+x<<4)+z}else{c[b+4784>>2]=(J*3|0)+500+x}o=ac(c[K>>2]|0,J)|0;J=(ac(c[120992+(k*48|0)+(w<<4)+4>>2]|0,x)|0)+o|0;o=J+(ac(c[120992+(k*48|0)+(w<<4)+8>>2]|0,y)|0)|0;c[b+4844>>2]=o+(ac(c[120992+(k*48|0)+(w<<4)+12>>2]|0,z)|0);p=0;i=d;return p|0}function DV(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0;c[b+85816>>2]=68;d=2;do{e=0;do{e=e+1|0;}while((c[b+21360+(e<<2)>>2]|0)<(d|0));f=c[71672+(e<<3)>>2]|0;g=f;while(1){if((c[b+21360+(g+1<<2)>>2]|0)>(d|0)){g=g-1|0}else{break}}h=((g|0)<0?f:g)&255;a[d-2+(b+85100)|0]=h;i=c[71676+(e<<3)>>2]|0;j=h<<24>>24;h=i;while(1){if((c[b+21360+(h+2+j<<2)>>2]|0)>(d|0)){h=h-1|0}else{break}}a[d-1+(b+85100)|0]=((h|0)<0?i:h)&255;d=d+2|0;}while((d|0)<577);return}function DW(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;e=a;f=0;g=0;while(1){h=c[e>>2]|0;i=e+8|0;j=c[e+4>>2]|0;k=(f|0)<(h|0)?h:f;l=(g|0)<(j|0)?j:g;if(i>>>0<b>>>0){e=i;f=k;g=l}else{break}}g=(k|0)<(l|0)?l:k;if(g>>>0<16){m=dk[c[263928+(g<<2)>>2]&63](a,b,g,d)|0;return m|0}if(g>>>0>8206){c[d>>2]=1e5;m=-1;return m|0}k=g-15|0;g=24;while(1){if((g|0)>=32){break}if((c[174572+(g<<4)>>2]|0)>>>0<k>>>0){g=g+1|0}else{break}}l=g-8|0;while(1){if((l|0)>=24){break}if((c[174572+(l<<4)>>2]|0)>>>0<k>>>0){l=l+1|0}else{break}}k=(c[174568+(l<<4)>>2]<<16)+(c[174568+(g<<4)>>2]|0)|0;f=a;a=0;while(1){e=c[f>>2]|0;i=f+8|0;j=c[f+4>>2]|0;h=e>>>0>14;n=j>>>0>14;o=(h?k:0)+a+(c[133696+((n?15:j)+(h?240:e<<4)<<2)>>2]|0)+(n?k:0)|0;if(i>>>0<b>>>0){f=i;a=o}else{break}}a=o&65535;f=o>>>16;o=f>>>0>a>>>0;c[d>>2]=(c[d>>2]|0)+(o?a:f);m=o?g:l;return m|0}function DX(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;return 0}function DY(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0;e=c[43649]|0;g=a;a=0;while(1){h=g+8|0;i=(d[e+((c[g>>2]<<1)+(c[g+4>>2]|0))|0]|0)+a|0;if(h>>>0<b>>>0){g=h;a=i}else{break}}c[f>>2]=(c[f>>2]|0)+i;return 1}function DZ(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0;f=d-1|0;d=c[174504+(f<<2)>>2]|0;g=c[174568+(d<<4)>>2]|0;h=(f|0)==1?64584:64520;f=a;a=0;while(1){i=f+8|0;j=c[f+4>>2]|0;k=(c[h+((ac(c[f>>2]|0,g)|0)+j<<2)>>2]|0)+a|0;if(i>>>0<b>>>0){f=i;a=k}else{break}}a=k&65535;f=k>>>16;k=f>>>0>a>>>0;c[e>>2]=(c[e>>2]|0)+(k?a:f);return(k&1)+d|0}function D_(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;g=c[174504+(e-1<<2)>>2]|0;e=c[174568+(g<<4)>>2]|0;h=c[174580+(g<<4)>>2]|0;i=g+1|0;j=c[174580+(i<<4)>>2]|0;k=g+2|0;l=c[174580+(k<<4)>>2]|0;m=a;a=0;n=0;o=0;while(1){p=m+8|0;q=c[m+4>>2]|0;r=(ac(c[m>>2]|0,e)|0)+q|0;s=(d[h+r|0]|0)+a|0;t=(d[j+r|0]|0)+n|0;u=(d[l+r|0]|0)+o|0;if(p>>>0<b>>>0){m=p;a=s;n=t;o=u}else{break}}o=s>>>0>t>>>0;n=o?t:s;s=n>>>0>u>>>0;c[f>>2]=(c[f>>2]|0)+(s?u:n);return(s?k:o?i:g)|0}function D$(a,b,d,e){a=a|0;b=+b;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,l=0,m=0,n=0,o=0.0,p=0.0,q=0.0,r=0.0,s=0.0,t=0,u=0.0,v=0,w=0,x=0.0,y=0,z=0,A=0.0,B=0,C=0.0;f=e;h=a&2;i=a>>>2;if((i|0)==0){j=f;l=d}else{a=i<<2;m=e+(a<<2)|0;e=f;f=d;n=i;while(1){i=n-1|0;o=+g[f+4>>2]*b;p=+g[f+8>>2]*b;q=+g[f+12>>2]*b;r=+g[f>>2]*b+8388608.0;s=r;t=e|0;g[t>>2]=s;u=o+8388608.0;o=u;v=e+4|0;w=v|0;g[w>>2]=o;x=p+8388608.0;p=x;y=e+8|0;z=y|0;g[z>>2]=p;A=q+8388608.0;q=A;B=e+12|0;g[B>>2]=q;C=r+ +g[3068912+((g[k>>2]=s,c[k>>2]|0)-1258291200<<2)>>2];g[t>>2]=C;s=u+ +g[3068912+((g[k>>2]=o,c[k>>2]|0)-1258291200<<2)>>2];g[w>>2]=s;o=x+ +g[3068912+((g[k>>2]=p,c[k>>2]|0)-1258291200<<2)>>2];g[z>>2]=o;p=A+ +g[3068912+((g[k>>2]=q,c[k>>2]|0)-1258291200<<2)>>2];c[e>>2]=(g[k>>2]=C,c[k>>2]|0)-1258291200;c[v>>2]=(g[k>>2]=s,c[k>>2]|0)-1258291200;c[y>>2]=(g[k>>2]=o,c[k>>2]|0)-1258291200;c[B>>2]=(g[k>>2]=p,c[k>>2]|0)-1258291200;if((i|0)==0){break}else{e=e+16|0;f=f+16|0;n=i}}j=m;l=d+(a<<2)|0}if((h|0)==0){return}p=+g[l+4>>2]*b;o=+g[l>>2]*b+8388608.0;b=o;l=j|0;g[l>>2]=b;s=p+8388608.0;p=s;h=j+4|0;g[h>>2]=p;C=o+ +g[3068912+((g[k>>2]=b,c[k>>2]|0)-1258291200<<2)>>2];g[l>>2]=C;b=s+ +g[3068912+((g[k>>2]=p,c[k>>2]|0)-1258291200<<2)>>2];c[j>>2]=(g[k>>2]=C,c[k>>2]|0)-1258291200;c[h>>2]=(g[k>>2]=b,c[k>>2]|0)-1258291200;return}function D0(a){a=a|0;var b=0,d=0,e=0,f=0;b=a+85704|0;d=c[b>>2]|0;if((d|0)!=0){ND(d);c[b>>2]=0}b=a+85708|0;d=c[b>>2]|0;if((d|0)!=0){ND(d);c[b>>2]=0}b=a+85712|0;d=c[b>>2]|0;if((d|0)!=0){ND(d);c[b>>2]=0}b=a+85716|0;d=c[b>>2]|0;if((d|0)!=0){ND(d);c[b>>2]=0}b=a+85728|0;d=c[b>>2]|0;if((d|0)!=0){ND(d);c[b>>2]=0;c[a+85732>>2]=0;c[a+85740>>2]=0}b=a+85744|0;d=c[b>>2]|0;if((d|0)==0){return}else{e=d}while(1){d=c[e+24>>2]|0;f=c[e>>2]|0;ND(c[e+12>>2]|0);ND(d);ND(e);if((f|0)==0){break}else{e=f}}c[b>>2]=0;c[a+85748>>2]=0;return}function D1(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=0;do{d=a+37192+(b<<2)|0;e=c[d>>2]|0;if((e|0)!=0){ND(e);c[d>>2]=0}b=b+1|0;}while((b|0)<641);b=a+37184|0;d=c[b>>2]|0;if((d|0)!=0){ND(d);c[b>>2]=0}b=a+37188|0;d=c[b>>2]|0;if((d|0)!=0){ND(d);c[b>>2]=0}b=a+284|0;d=c[b>>2]|0;if((d|0)!=0){ND(d);c[b>>2]=0}b=a+85780|0;d=c[b>>2]|0;if((d|0)!=0){ND(d);c[b>>2]=0;c[a+85776>>2]=0}b=c[a+85796>>2]|0;if((b|0)!=0){ND(b)}b=c[a+85676>>2]|0;if((b|0)!=0){ND(b)}b=c[a+52152>>2]|0;if((b|0)!=0){ND(b)}b=c[a+52156>>2]|0;if((b|0)!=0){ND(b)}D0(a);if((a|0)==0){f=a;ND(f);return}b=a+85800|0;d=c[b>>2]|0;if((d|0)==0){f=a;ND(f);return}e=c[d+2156>>2]|0;if((e|0)==0){g=d}else{ND(e);g=c[b>>2]|0}e=c[g+4316>>2]|0;if((e|0)==0){h=g}else{ND(e);h=c[b>>2]|0}ND(h);f=a;ND(f);return}function D2(a,b){a=a|0;b=+b;var d=0.0,e=0.0,f=0.0,h=0.0,i=0.0;switch(c[a+192>>2]|0){case 5:{d=+g[a+188>>2];e=(b<-.3?3410.0:b)/1.0e3;f=e<3.4100000858306885?3.4100000858306885:e;e=f>16.100000381469727?16.100000381469727:f;f=e+ -3.4;h=e+ -8.7;i=+T(+e,+-.8)*3.64- +$(+(f*f*-.6))*6.8+ +$(+(h*h*-.15))*6.0+(d*.04+.6)*.001*+T(+e,+4.0);return+i};case 3:{e=(b<-.3?3410.0:b)/1.0e3;d=e<.10000000149011612?.10000000149011612:e;e=d>24.0?24.0:d;d=e+ -3.4;h=e+ -8.7;i=+T(+e,+-.8)*3.64- +$(+(d*d*-.6))*6.8+ +$(+(h*h*-.15))*6.0+ +T(+e,+4.0)*64.0e-5+6.0;return+i};case 1:{e=(b<-.3?3410.0:b)/1.0e3;h=e<.10000000149011612?.10000000149011612:e;e=h>24.0?24.0:h;h=e+ -3.4;d=e+ -8.7;i=+T(+e,+-.8)*3.64- +$(+(h*h*-.6))*6.8+ +$(+(d*d*-.15))*6.0+ +T(+e,+4.0)*56.0e-5;return+i};case 2:{e=(b<-.3?3410.0:b)/1.0e3;d=e<.10000000149011612?.10000000149011612:e;e=d>24.0?24.0:d;d=e+ -3.4;h=e+ -8.7;i=+T(+e,+-.8)*3.64- +$(+(d*d*-.6))*6.8+ +$(+(h*h*-.15))*6.0+ +T(+e,+4.0)*6.0e-4;return+i};case 0:{e=(b<-.3?3410.0:b)/1.0e3;h=e<.10000000149011612?.10000000149011612:e;e=h>24.0?24.0:h;h=e+ -3.4;d=e+ -8.7;i=+T(+e,+-.8)*3.64- +$(+(h*h*-.6))*6.8+ +$(+(d*d*-.15))*6.0+ +T(+e,+4.0)*96.0e-5;return+i};case 4:{e=+g[a+188>>2];d=(b<-.3?3410.0:b)/1.0e3;h=d<.10000000149011612?.10000000149011612:d;d=h>24.0?24.0:h;h=d+ -3.4;f=d+ -8.7;i=+T(+d,+-.8)*3.64- +$(+(h*h*-.6))*6.8+ +$(+(f*f*-.15))*6.0+(e*.04+.6)*.001*+T(+d,+4.0);return+i};default:{d=(b<-.3?3410.0:b)/1.0e3;b=d<.10000000149011612?.10000000149011612:d;d=b>24.0?24.0:b;b=d+ -3.4;e=d+ -8.7;i=+T(+d,+-.8)*3.64- +$(+(b*b*-.6))*6.8+ +$(+(e*e*-.15))*6.0+ +T(+d,+4.0)*6.0e-4;return+i}}return 0.0}function D3(a){a=+a;var b=0.0;b=a<0.0?0.0:a*.001;a=+Z(+(b*.76))*13.0;return+(a+ +Z(+(b*b/56.25))*3.5)}function D4(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;e=(d|0)<16e3?2:b;b=c[308172+(e<<6)>>2]|0;d=2;while(1){f=c[308168+(e<<6)+(d<<2)>>2]|0;if((f|0)>0){g=f-a|0;h=b-a|0;i=(((g|0)>0?g:-g|0)|0)<(((h|0)>0?h:-h|0)|0)?f:b}else{i=b}f=d+1|0;if((f|0)<15){b=i;d=f}else{break}}return i|0}function D5(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0;b=a&65535;a=0;while(1){if((a|0)>=16){d=320;e=16;f=320;g=16;break}h=a+1|0;i=c[121296+(h<<2)>>2]|0;if((((b|0)>(i|0)?b:i)|0)==(b|0)){a=h}else{j=1129;break}}if((j|0)==1129){d=i;e=h;f=c[121296+(a<<2)>>2]|0;g=a}return((d-b|0)>(b-f|0)?g:e)|0}function D6(a){a=a|0;var b=0;do{if((a|0)<8001){b=8e3}else{if((a|0)<11026){b=11025;break}if((a|0)<12001){b=12e3;break}if((a|0)<16001){b=16e3;break}if((a|0)<22051){b=22050;break}if((a|0)<24001){b=24e3;break}if((a|0)<32001){b=32e3;break}b=(a|0)<44101?44100:48e3}}while(0);return b|0}function D7(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=(d|0)<16e3?2:b;b=0;while(1){if((b|0)>=15){f=-1;g=1144;break}d=c[308168+(e<<6)+(b<<2)>>2]|0;if((d|0)>0&(d|0)==(a|0)){f=b;g=1145;break}else{b=b+1|0}}if((g|0)==1144){return f|0}else if((g|0)==1145){return f|0}return 0}function D8(a,b){a=a|0;b=b|0;var d=0;if((a|0)==16e3){c[b>>2]=0;d=2}else if((a|0)==44100){c[b>>2]=1;d=0}else if((a|0)==11025){c[b>>2]=0;d=0}else if((a|0)==8e3){c[b>>2]=0;d=2}else if((a|0)==22050){c[b>>2]=0;d=0}else if((a|0)==32e3){c[b>>2]=1;d=2}else if((a|0)==12e3){c[b>>2]=0;d=1}else if((a|0)==24e3){c[b>>2]=0;d=1}else if((a|0)==48e3){c[b>>2]=1;d=1}else{c[b>>2]=0;d=-1}return d|0}function D9(a){a=a|0;var b=0.0,d=0,e=0;b=+(c[a+48>>2]|0);d=c[a+44>>2]|0;if((d|0)<(~~(b*.9994999766349792)|0)){e=1;return e|0}e=(~~(b*1.000499963760376)|0)<(d|0)|0;return e|0}function Ea(a,b,d,e,f,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0.0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0.0,F=0,G=0,H=0,I=0,J=0.0,K=0.0,L=0.0,M=0.0,N=0.0,O=0,P=0.0,S=0,T=0.0,W=0,X=0.0,Y=0.0,Z=0.0,_=0.0,$=0.0,aa=0.0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0;j=c[a+84036>>2]|0;k=(c[a+76>>2]|0)*576|0;l=c[a+72>>2]|0;m=a+64|0;n=c[m>>2]|0;o=+(n|0);p=a+60|0;q=c[p>>2]|0;do{if((q|0)>=(~~(o*.9994999766349792)|0)){if((~~(o*1.000499963760376)|0)<(q|0)){break}r=(k|0)<(e|0)?k:e;s=r<<2;t=0;do{u=(c[b+(t<<2)>>2]|0)+(j<<2)|0;v=c[d+(t<<2)>>2]|0;NN(u|0,v|0,s)|0;t=t+1|0;}while((t|0)<(l|0));c[i>>2]=r;c[f>>2]=r;return}}while(0);t=a+12|0;s=a+37184|0;v=a+37188|0;u=a+37168|0;w=(k|0)>0;x=0;y=q;q=n;while(1){n=c[b+(x<<2)>>2]|0;z=c[d+(x<<2)>>2]|0;o=+(y|0)/+(q|0);if((y|0)==0){A=q}else{B=q;C=y;while(1){D=(B|0)%(C|0)|0;if((D|0)==0){A=C;break}else{B=C;C=D}}}C=(q|0)/(A|0)|0;B=(C|0)>320?320:C;E=1.0/o;r=E>1.0;D=+R(+(o- +Q(+(o+.5))))<1.0e-4?32:31;F=D+1|0;if((c[t>>2]|0)==0){c[s>>2]=NE(F,4)|0;c[v>>2]=NE(F,4)|0;G=B<<1;if((G|0)<0){NM(u|0,0,16);H=0}else{I=0;do{c[a+37192+(I<<2)>>2]=NE(F,4)|0;I=I+1|0;}while((I|0)<=(G|0));NM(u|0,0,16);J=+(B|0)*2.0;K=r?3.1415927410125732:E*3.141592653589793;L=+(D|0);M=K/3.141592653589793;N=L*K;K=+(D|0)*3.141592653589793;I=(C|0)<320;O=0;do{P=+(O-B|0)/J;S=a+37192+(O<<2)|0;T=0.0;W=0;while(1){X=(+(W|0)-P)/L;Y=X<0.0?0.0:X;X=Y>1.0?1.0:Y;Y=X+-.5;if(+R(+Y)<1.0e-9){Z=M}else{_=+U(+(X*2.0*3.141592653589793));$=.42-_*.5+ +U(+(X*4.0*3.141592653589793))*.08;Z=+V(+(N*Y))*$/(K*Y)}Y=Z;g[(c[S>>2]|0)+(W<<2)>>2]=Y;aa=T+Y;ab=W+1|0;if((ab|0)>(D|0)){ac=0;break}else{T=aa;W=ab}}do{W=(c[S>>2]|0)+(ac<<2)|0;g[W>>2]=+g[W>>2]/aa;ac=ac+1|0;}while((ac|0)<=(D|0));O=O+1|0;}while((O|0)<=(G|0));H=I?C<<1|1:641}c[t>>2]=1;ad=H}else{ad=0}G=c[a+37184+(x<<2)>>2]|0;L1577:do{if(w){O=a+37168+(x<<3)|0;r=D>>>1;S=D-r|0;K=+(D&1|0)*.5;N=+(B|0);W=0;while(1){M=o*+(W|0)- +h[O>>3];ab=~~+Q(+M);if((ab+S|0)>=(e|0)){ae=ab;af=W;ag=S;ah=O;break L1577}ai=ab-r|0;aj=c[a+37192+(~~+Q(+(N+N*(M-(K+ +(ab|0)))*2.0+.5))<<2)>>2]|0;ak=0;M=0.0;do{al=ak+ai|0;if((al|0)<0){am=G+(al+F<<2)|0}else{am=z+(al<<2)|0}M=M+ +g[am>>2]*+g[aj+(ak<<2)>>2];ak=ak+1|0;}while((ak|0)<=(D|0));g[n+(W+j<<2)>>2]=M;ak=W+1|0;if((ak|0)<(k|0)){W=ak}else{ae=ab;af=ak;ag=S;ah=O;break}}}else{ae=ad;af=0;ag=D-(D>>>1)|0;ah=a+37168+(x<<3)|0}}while(0);n=ae+ag|0;B=(n|0)>(e|0)?e:n;c[f>>2]=B;h[ah>>3]=+h[ah>>3]+(+(B|0)-o*+(af|0));B=c[f>>2]|0;do{if((B|0)<(F|0)){n=F-B|0;do{if((n|0)>0){g[G>>2]=+g[G+(B<<2)>>2];if((n|0)>1){an=1}else{ao=1;break}while(1){g[G+(an<<2)>>2]=+g[G+(an+(c[f>>2]|0)<<2)>>2];C=an+1|0;if((C|0)<(n|0)){an=C}else{ao=n;break}}}else{ao=0}}while(0);if((ao|0)<(F|0)){ap=0;aq=ao}else{break}while(1){g[G+(aq<<2)>>2]=+g[z+(ap<<2)>>2];n=aq+1|0;if((n|0)<(F|0)){ap=ap+1|0;aq=n}else{break}}}else{n=~D;g[G>>2]=+g[z+(B+n<<2)>>2];if(F>>>0>1){ar=1}else{break}do{g[G+(ar<<2)>>2]=+g[z+((c[f>>2]|0)+n+ar<<2)>>2];ar=ar+1|0;}while((ar|0)<(F|0))}}while(0);F=x+1|0;if((F|0)>=(l|0)){break}x=F;y=c[p>>2]|0;q=c[m>>2]|0}c[i>>2]=af;return}function Eb(a,b){a=a|0;b=b|0;var d=0;d=c[o>>2]|0;cd(d|0,a|0,b|0)|0;aT(d|0)|0;return}function Ec(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;i=i+16|0;f=e|0;if((a|0)==0){i=e;return}g=a+85828|0;if((c[g>>2]|0)==0){i=e;return}a=f;c[a>>2]=d;c[a+4>>2]=0;c$[c[g>>2]&127](b,f|0);i=e;return}function Ed(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;i=i+16|0;f=e|0;if((a|0)==0){i=e;return}g=a+85836|0;if((c[g>>2]|0)==0){i=e;return}a=f;c[a>>2]=d;c[a+4>>2]=0;c$[c[g>>2]&127](b,f|0);i=e;return}function Ee(){return 0}function Ef(){return 0}function Eg(){return 0}function Eh(){return 0}function Ei(){return}function Ej(){var b=0;if(a[172952]|0){a[172952]=1;return}else{b=0}do{g[2203728+(b<<2)>>2]=+aa(+(+(b|0)*.001953125+1.0))/.6931471805599453;b=b+1|0;}while((b|0)<513);a[172952]=1;return}function Ek(a){a=+a;var b=0,d=0;b=(g[k>>2]=a,c[k>>2]|0);a=+(b&16383|0)*6103515625.0e-14;d=b>>>14&511;return+(+((b>>>23&255)-127|0)+((1.0-a)*+g[2203728+(d<<2)>>2]+a*+g[2203728+(d+1<<2)>>2]))}function El(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0.0,af=0,ag=0.0,ah=0,ai=0,aj=0,ak=0.0,al=0.0,am=0.0,an=0.0,ao=0.0,ap=0,aq=0.0,ar=0.0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0.0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0.0,aX=0,aY=0,aZ=0.0,a_=0,a$=0,a0=0,a1=0.0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bq=0,br=0,bs=0,bt=0,bu=0;h=i;i=i+320|0;j=h|0;k=h+160|0;l=k;m=i;i=i+624|0;n=i;i=i+624|0;o=i;i=i+144|0;p=i;i=i+16|0;q=i;i=i+8|0;r=q;s=i;i=i+16|0;t=i;i=i+8|0;u=t;v=i;i=i+8|0;w=v;x=i;i=i+8|0;y=x;z=i;i=i+8|0;A=z;B=c[b+76>>2]|0;C=c[b+72>>2]|0;NM(p|0,0,16);c[q>>2]=0;c[q+4>>2]=0;D=s;c[D>>2]=c[488626];c[D+4>>2]=c[488627];c[D+8>>2]=c[488628];c[D+12>>2]=c[488629];c[t>>2]=7681;c[t+4>>2]=7681;D=(B|0)>0;if(!D){E=0;i=h;return E|0}F=(C|0)>0;G=b+48|0;H=C<<2;I=0;J=0;while(1){K=r+(I<<2)|0;c[K>>2]=0;if(F){NM(s+(I<<3)|0,0,H|0);L=c[G>>2]|0;M=0;N=J;O=0;do{P=c[f+(I<<3)+(M<<2)>>2]|0;c[p+(I<<3)+(M<<2)>>2]=P;O=O+P|0;N=P+N|0;c[o+(I*72|0)+(M*36|0)+4>>2]=(L|0)<0?4:2;c[o+(I*72|0)+(M*36|0)+12>>2]=b;c[o+(I*72|0)+(M*36|0)+16>>2]=b+304+(I*10504|0)+(M*5252|0);c[o+(I*72|0)+(M*36|0)+8>>2]=d+(I*4608|0)+(M*2304|0);c[o+(I*72|0)+(M*36|0)>>2]=(c[b+304+(I*10504|0)+(M*5252|0)+4788>>2]|0)==2?82:42;M=M+1|0;}while((M|0)<(C|0));c[K>>2]=O;Q=N}else{Q=J}M=I+1|0;if((M|0)<(B|0)){I=M;J=Q}else{break}}do{if(D){J=(C|0)>0;I=0;L1651:do{if(J){d=0;do{G=n+(I*312|0)+(d*156|0)|0;if((c[f+(I<<3)+(d<<2)>>2]|0)>0){H=m+(I*312|0)+(d*156|0)|0;F=o+(I*72|0)+(d*36|0)+16|0;M=c[F>>2]|0;L=c[o+(I*72|0)+(d*36|0)+8>>2]|0;P=c[M+5208>>2]|0;R=c[M+4864>>2]|0;T=o+(I*72|0)+(d*36|0)+20|0;U=o+(I*72|0)+(d*36|0)+4|0;NM(T|0,0,16);V=0;W=0;X=-1;Y=0;Z=0;while(1){_=c[M+4872+(Y<<2)>>2]|0;$=P-W+1|0;aa=_>>>0>$>>>0?$:_;$=L+(W<<2)|0;ab=aa>>>2;ad=aa&3;if((ab|0)==0){ae=0.0;af=$}else{ag=0.0;ah=ab;ai=$;while(1){aj=ah-1|0;ak=+g[ai>>2];al=ag<ak?ak:ag;ak=+g[ai+4>>2];am=al<ak?ak:al;al=+g[ai+8>>2];ak=am<al?al:am;am=+g[ai+12>>2];an=ak<am?am:ak;if((aj|0)==0){break}else{ag=an;ah=aj;ai=ai+16|0}}ae=an;af=L+((ab<<2)+W<<2)|0}do{if((ad|0)==3){ag=+g[af+8>>2];if(ae>=ag){ao=ae;ap=1256;break}ao=ag;ap=1256}else if((ad|0)==1){aq=ae;ap=1258}else if((ad|0)==2){ao=ae;ap=1256}else{ar=ae}}while(0);do{if((ap|0)==1256){ap=0;ag=+g[af+4>>2];if(ao>=ag){aq=ao;ap=1258;break}aq=ag;ap=1258}}while(0);do{if((ap|0)==1258){ap=0;ag=+g[af>>2];if(aq>=ag){ar=aq;break}ar=ag}}while(0);ad=ar*+g[551672]>8206.0;ab=ad?-64:64;ai=ar*+g[2206176+((ab&255)<<2)>>2]>8206.0;ah=(ai?32:-32)+ab&255;aj=ar*+g[2206176+((ah&255)<<2)>>2]>8206.0;as=(aj?16:-16)+ah&255;at=ar*+g[2206176+((as&255)<<2)>>2]>8206.0;au=(at?8:-8)+as&255;av=ar*+g[2206176+((au&255)<<2)>>2]>8206.0;aw=(av?4:-4)+au&255;ax=ar*+g[2206176+((aw&255)<<2)>>2]>8206.0;ay=(ax?2:-2)+aw&255;az=ar*+g[2206176+((ay&255)<<2)>>2]>8206.0;aA=(az?1:-1)+ay&255;aB=ar*+g[2206176+((aA&255)<<2)>>2]>8206.0?az?ax?av?at?aj?ai?ad?-1:-128:ab:ah:as:au:aw:ay:aA;aA=aB&255;c[n+(I*312|0)+(d*156|0)+(Y<<2)>>2]=aA;if((c[T>>2]|0)<(aA|0)){c[T>>2]=aA}ay=o+(I*72|0)+(d*36|0)+24+(V<<2)|0;if((c[ay>>2]|0)<(aA|0)){c[ay>>2]=aA}aA=V+1|0;ay=aA>>>0>2?0:aA;do{if((Y|0)<(R|0)&_>>>0>2){if((a[M+5212+Y|0]|0)==0){aC=-1;aD=-1;aE=X;break}aA=c5[c[U>>2]&7](M+(W<<2)|0,$,+g[e+(I*312|0)+(d*156|0)+(Y<<2)>>2],aa,aB)|0;aw=aA&255;aC=aA;aD=(Z&255)<(aA&255)?aA:Z;aE=(X|0)>=(aw|0)|aA<<24>>24==-1?X:aw}else{aw=(Z&255)<(aB&255)?aB:Z;aC=aw;aD=aw;aE=X}}while(0);c[m+(I*312|0)+(d*156|0)+(Y<<2)>>2]=aC&255;aF=Y+1|0;aB=_+W|0;if(aB>>>0>P>>>0){break}else{V=ay;W=aB;X=aE;Y=aF;Z=aD}}Z=o+(I*72|0)+(d*36|0)|0;if((aF|0)<39){Y=aD&255;NM(n+(I*312|0)+(d*156|0)+(aF<<2)|0,0,156-(aF<<2)|0);X=aF;do{c[m+(I*312|0)+(d*156|0)+(X<<2)>>2]=Y;X=X+1|0;}while((X|0)<39)}if((aE|0)>-1){X=0;do{Y=m+(I*312|0)+(d*156|0)+(X<<2)|0;if((c[Y>>2]|0)==255){c[Y>>2]=aE}X=X+1|0;}while((X|0)<39);aG=aE&255}else{aG=aD}dq[c[Z>>2]&127](Z,H,G,aG&255);aH=o+(I*72|0)+(d*36|0)+12|0;if((DU(c[aH>>2]|0,c[F>>2]|0)|0)!=0){ap=1276;break L1651}}d=d+1|0;}while((d|0)<(C|0))}I=I+1|0;}while((I|0)<(B|0));if((ap|0)==1276){Ed(c[aH>>2]|0,1738400,(aI=i,i=i+1|0,i=i+7&-8,c[aI>>2]=0,aI)|0);i=aI;bp(-1|0);return 0}if(!D){aJ=0;break}I=(C|0)>0;J=b+304|0;N=b+36|0;O=0;K=0;while(1){d=u+(O<<2)|0;c[d>>2]=0;if(I){X=0;Y=0;do{if((c[f+(O<<3)+(X<<2)>>2]|0)>0){W=o+(O*72|0)+(X*36|0)|0;NM((c[o+(O*72|0)+(X*36|0)+16>>2]|0)+2304|0,0,2304);Eq(W)|0}DT(b,O,X,J);if((c[N>>2]|0)==1){DR(b,b+304+(O*10504|0)+(X*5252|0)|0)}W=(c[b+304+(O*10504|0)+(X*5252|0)+4844>>2]|0)+(c[b+304+(O*10504|0)+(X*5252|0)+4768>>2]|0)|0;c[s+(O<<3)+(X<<2)>>2]=W;Y=Y+W|0;X=X+1|0;}while((X|0)<(C|0));c[d>>2]=Y;aK=Y}else{aK=0}X=aK+K|0;W=O+1|0;if((W|0)<(B|0)){O=W;K=X}else{aJ=X;break}}}else{aJ=0}}while(0);do{if((aJ|0)<=(Q|0)){if(!D){E=aJ;i=h;return E|0}aK=(C|0)>0;aH=1;aG=0;while(1){aD=(c[u+(aG<<2)>>2]|0)>7680?0:aH;if(aK){aE=aD;aF=0;while(1){aC=(c[s+(aG<<3)+(aF<<2)>>2]|0)>4095?0:aE;e=aF+1|0;if((e|0)<(C|0)){aE=aC;aF=e}else{aL=aC;break}}}else{aL=aD}aF=aG+1|0;if((aF|0)<(B|0)){aH=aL;aG=aF}else{break}}if((aL|0)==0){break}else{E=aJ}i=h;return E|0}}while(0);if(D){aJ=(C|0)>0;aL=(C|0)>1;aG=0;aH=0;while(1){aK=r+(aH<<2)|0;c[aK>>2]=0;do{if(aJ){aF=0;aE=0;do{Y=c[s+(aH<<3)+(aF<<2)>>2]|0;d=(Y|0)>4095?4095:Y;c[p+(aH<<3)+(aF<<2)>>2]=d;aE=d+aE|0;aF=aF+1|0;}while((aF|0)<(C|0));c[aK>>2]=aE;if((aE|0)<=7680){aM=aE;break}c[v>>2]=0;c[v+4>>2]=0;do{if(aJ){aF=0;ar=0.0;while(1){aD=c[p+(aH<<3)+(aF<<2)>>2]|0;if((aD|0)>0){aq=+S(+(+S(+(+(aD|0)))));g[w+(aF<<2)>>2]=aq;aN=ar+aq}else{g[w+(aF<<2)>>2]=0.0;aN=ar}aD=aF+1|0;if((aD|0)<(C|0)){aF=aD;ar=aN}else{break}}if(!aJ){break}aF=aN>0.0;F=0;do{if(aF){c[p+(aH<<3)+(F<<2)>>2]=~~(+g[w+(F<<2)>>2]*7680.0/aN)}else{c[p+(aH<<3)+(F<<2)>>2]=0}F=F+1|0;}while((F|0)<(C|0))}}while(0);do{if(aL){aE=p+(aH<<3)|0;F=c[aE>>2]|0;aF=c[s+(aH<<3)>>2]|0;G=aF+32|0;H=p+(aH<<3)+4|0;Z=c[H>>2]|0;if((F|0)>(G|0)){aD=-32-aF+(Z+F)|0;c[H>>2]=aD;c[aE>>2]=G;aO=G;aP=aD}else{aO=F;aP=Z}Z=p+(aH<<3)+4|0;F=c[s+(aH<<3)+4>>2]|0;aD=F+32|0;if((aP|0)>(aD|0)){G=-32-F+(aO+aP)|0;c[aE>>2]=G;c[Z>>2]=aD;aQ=G;aR=aD}else{aQ=aO;aR=aP}if((aQ|0)>4095){c[aE>>2]=4095}if((aR|0)<=4095){break}c[Z>>2]=4095}}while(0);c[aK>>2]=0;if(aJ){aS=0;aT=0}else{aM=0;break}do{aT=aT+(c[p+(aH<<3)+(aS<<2)>>2]|0)|0;aS=aS+1|0;}while((aS|0)<(C|0));c[aK>>2]=aT;aM=aT}else{aM=0}}while(0);aK=aM+aG|0;Z=aH+1|0;if((Z|0)<(B|0)){aG=aK;aH=Z}else{aU=aK;break}}}else{aU=0}do{if((aU|0)>(Q|0)){c[x>>2]=0;c[x+4>>2]=0;if(D){aV=0;aW=0.0}else{aX=0;aY=0;break}while(1){aH=c[r+(aV<<2)>>2]|0;if((aH|0)>0){aN=+S(+(+(aH|0)));g[y+(aV<<2)>>2]=aN;aZ=aW+aN}else{g[y+(aV<<2)>>2]=0.0;aZ=aW}aH=aV+1|0;if((aH|0)<(B|0)){aV=aH;aW=aZ}else{break}}if(!D){aX=0;aY=0;break}aH=aZ>0.0;aN=+(Q|0);aG=0;do{if(aH){c[r+(aG<<2)>>2]=~~(aN*+g[y+(aG<<2)>>2]/aZ)}else{c[r+(aG<<2)>>2]=0}aG=aG+1|0;}while((aG|0)<(B|0));if((B|0)>1){aG=q;aH=c[aG>>2]|0;aM=c[t>>2]|0;aT=aM+125|0;aS=r+4|0;aJ=c[aS>>2]|0;if((aH|0)>(aT|0)){aR=-125-aM+(aJ+aH)|0;c[aS>>2]=aR;c[aG>>2]=aT;a_=aT;a$=aR}else{a_=aH;a$=aJ}aJ=c[u+4>>2]|0;aH=aJ+125|0;if((a$|0)>(aH|0)){c[aG>>2]=-125-aJ+(a_+a$);c[r+4>>2]=aH}if(D){a0=0}else{aX=0;aY=0;break}do{aH=r+(a0<<2)|0;if((c[aH>>2]|0)>7680){c[aH>>2]=7680}a0=a0+1|0;}while((a0|0)<(B|0))}if(!D){aX=0;aY=0;break}aH=(C|0)>0;aJ=(C|0)>1;aG=0;while(1){c[z>>2]=0;c[z+4>>2]=0;do{if(aH){aR=0;aN=0.0;while(1){aT=c[p+(aG<<3)+(aR<<2)>>2]|0;if((aT|0)>0){ar=+S(+(+(aT|0)));g[A+(aR<<2)>>2]=ar;a1=aN+ar}else{g[A+(aR<<2)>>2]=0.0;a1=aN}aT=aR+1|0;if((aT|0)<(C|0)){aR=aT;aN=a1}else{break}}if(!aH){break}aR=a1>0.0;aT=r+(aG<<2)|0;aS=0;do{if(aR){c[p+(aG<<3)+(aS<<2)>>2]=~~(+(c[aT>>2]|0)*+g[A+(aS<<2)>>2]/a1)}else{c[p+(aG<<3)+(aS<<2)>>2]=0}aS=aS+1|0;}while((aS|0)<(C|0))}}while(0);L1827:do{if(aJ){aS=p+(aG<<3)|0;aT=c[aS>>2]|0;aR=c[s+(aG<<3)>>2]|0;aM=aR+32|0;aQ=p+(aG<<3)+4|0;aP=c[aQ>>2]|0;if((aT|0)>(aM|0)){aO=-32-aR+(aP+aT)|0;c[aQ>>2]=aO;c[aS>>2]=aM;a2=aM;a3=aO}else{a2=aT;a3=aP}aP=c[s+(aG<<3)+4>>2]|0;aT=aP+32|0;if((a3|0)>(aT|0)){aO=-32-aP+(a2+a3)|0;c[aS>>2]=aO;c[p+(aG<<3)+4>>2]=aT;a4=aO}else{a4=a2}if(aH){a5=0;a6=a4}else{break}while(1){if((a6|0)>4095){c[p+(aG<<3)+(a5<<2)>>2]=4095}aO=a5+1|0;if((aO|0)>=(C|0)){break L1827}a5=aO;a6=c[p+(aG<<3)+(aO<<2)>>2]|0}}}while(0);aO=aG+1|0;if((aO|0)<(B|0)){aG=aO}else{ap=1324;break}}}else{ap=1324}}while(0);do{if((ap|0)==1324){if(!D){aX=0;aY=0;break}a6=(C|0)>0;a5=0;a4=1;a2=0;do{if(a6){a3=a4;A=0;r=0;while(1){z=c[p+(a2<<3)+(A<<2)>>2]|0;a0=z+r|0;a$=(z|0)>4095?0:a3;z=A+1|0;if((z|0)<(C|0)){a3=a$;A=z;r=a0}else{a7=a$;a8=a0;break}}}else{a7=a4;a8=0}a5=a8+a5|0;a4=(a8|0)>7680?0:a7;a2=a2+1|0;}while((a2|0)<(B|0));aX=a5;aY=(a4|0)==0}}while(0);do{if(((aX|0)>(Q|0)|aY)&D){if((C|0)<=0){a9=0;break}a7=C<<2;a8=0;while(1){a2=f+(a8<<3)|0;a6=p+(a8<<3)|0;NN(a6|0,a2|0,a7)|0;a2=a8+1|0;if((a2|0)<(B|0)){a8=a2}else{ap=1378;break}}}else{ap=1378}}while(0);do{if((ap|0)==1378){aY=(C|0)>0;if(aY){ba=0}else{a9=0;break}while(1){aX=ba+1|0;NM(b+21328+(ba<<4)|0,0,16);if((aX|0)<(C|0)){ba=aX}else{a9=aY;break}}}}while(0);L1863:do{if(D){if(a9){ba=0;do{aY=0;do{c[b+304+(ba*10504|0)+(aY*5252|0)+4784>>2]=0;aY=aY+1|0;}while((aY|0)<(C|0));ba=ba+1|0;}while((ba|0)<(B|0));if(!D){bb=0;break}}ba=k|0;aY=j|0;aX=b+304|0;a8=b+36|0;a7=0;a4=0;L1873:while(1){a5=u+(a7<<2)|0;c[a5>>2]=0;if(a9){a2=0;a6=0;while(1){r=s+(a7<<3)+(a2<<2)|0;c[r>>2]=0;L1878:do{if((c[f+(a7<<3)+(a2<<2)>>2]|0)>0){A=m+(a7*312|0)+(a2*156|0)|0;a3=o+(a7*72|0)+(a2*36|0)+16|0;a0=(c[a3>>2]|0)+4780|0;a$=c[a0>>2]|0;z=39;a_=0;while(1){t=m+(a7*312|0)+(a2*156|0)+(a_<<2)|0;q=c[t>>2]|0;c[t>>2]=(q|0)<(a$|0)?q:a$;q=z-1|0;if((q|0)==0){break}else{z=q;a_=a_+1|0}}a_=o+(a7*72|0)+(a2*36|0)|0;z=c[p+(a7<<3)+(a2<<2)>>2]|0;a$=0;ay=39;_=0;while(1){q=255-(c[m+(a7*312|0)+(a2*156|0)+(a$<<2)>>2]|0)|0;bc=(_|0)<(q|0)?q:_;q=ay-1|0;if((q|0)==0){break}else{a$=a$+1|0;ay=q;_=bc}}_=n+(a7*312|0)+(a2*156|0)|0;ay=A;a$=c[a0>>2]|0;q=(bc|0)>0;t=a_|0;bd=o+(a7*72|0)+(a2*36|0)+12|0;if(q){y=bc;aV=-1;x=0;aU=bc;while(1){aG=(y|0)/2|0;aH=39;aJ=0;aO=0;while(1){aT=c[m+(a7*312|0)+(a2*156|0)+(aO<<2)>>2]|0;aS=((ac(a$-aT|0,aG)|0)/(bc|0)|0)+aT|0;if((aS|0)<0){be=0}else{be=(aS|0)>255?255:aS}c[k+(aO<<2)>>2]=be;bf=(aJ|0)<(be|0)?be:aJ;aS=aH-1|0;if((aS|0)==0){break}else{aH=aS;aJ=bf;aO=aO+1|0}}a1=+g[(c[a3>>2]|0)+4764>>2];dq[c[t>>2]&127](a_,ba,_,bf);if((DU(c[bd>>2]|0,c[a3>>2]|0)|0)!=0){ap=1462;break L1873}aO=Eq(a_)|0;aJ=c[a3>>2]|0;aH=(c[aJ+4844>>2]|0)+aO|0;g[aJ+4764>>2]=a1;aJ=(aH|0)>(z|0);aH=aJ?aU:aG-1|0;aO=aJ?aG+1|0:x;aS=aJ?aV:aG;if((aO|0)>(aH|0)){bg=aG;bh=aS;break}else{y=aO+aH|0;aV=aS;x=aO;aU=aH}}}else{aU=bc;x=-1;aV=0;y=bc;while(1){a0=(aU|0)/2|0;NN(l|0,ay|0,156)|0;A=39;aH=0;aO=0;while(1){aS=c[m+(a7*312|0)+(a2*156|0)+(aO<<2)>>2]|0;bi=(aH|0)<(aS|0)?aS:aH;aS=A-1|0;if((aS|0)==0){break}else{A=aS;aH=bi;aO=aO+1|0}}a1=+g[(c[a3>>2]|0)+4764>>2];dq[c[t>>2]&127](a_,ba,_,bi);if((DU(c[bd>>2]|0,c[a3>>2]|0)|0)!=0){ap=1461;break L1873}aO=Eq(a_)|0;aH=c[a3>>2]|0;A=(c[aH+4844>>2]|0)+aO|0;g[aH+4764>>2]=a1;aH=(A|0)>(z|0);A=aH?y:a0-1|0;aO=aH?a0+1|0:aV;aG=aH?x:a0;if((aO|0)>(A|0)){bg=a0;bh=aG;break}else{aU=aO+A|0;x=aG;aV=aO;y=A}}}if((bh|0)>-1){if((bg|0)==(bh|0)){break}if(q){y=39;aV=0;x=0;while(1){aU=c[m+(a7*312|0)+(a2*156|0)+(x<<2)>>2]|0;A=((ac(a$-aU|0,bh)|0)/(bc|0)|0)+aU|0;if((A|0)<0){bj=0}else{bj=(A|0)>255?255:A}c[k+(x<<2)>>2]=bj;A=(aV|0)<(bj|0)?bj:aV;aU=y-1|0;if((aU|0)==0){bk=A;break}else{y=aU;aV=A;x=x+1|0}}}else{NN(l|0,ay|0,156)|0;x=39;aV=0;y=0;while(1){A=c[m+(a7*312|0)+(a2*156|0)+(y<<2)>>2]|0;aU=(aV|0)<(A|0)?A:aV;A=x-1|0;if((A|0)==0){bk=aU;break}else{x=A;aV=aU;y=y+1|0}}}a1=+g[(c[a3>>2]|0)+4764>>2];dq[c[t>>2]&127](a_,ba,_,bk);if((DU(c[bd>>2]|0,c[a3>>2]|0)|0)!=0){ap=1414;break L1873}Eq(a_)|0;g[(c[a3>>2]|0)+4764>>2]=a1;break}y=255;aV=a$;x=-1;aU=a$+255|0;while(1){bl=(aU|0)/2|0;do{if(q){A=(bl|0)>255?255:bl;if((aU|0)<-1){NM(l|0,0,156);bm=0;break}else{aO=39;aG=0;aH=0;while(1){c[k+(aH<<2)>>2]=A;aS=(aG|0)<(A|0)?A:aG;aJ=aO-1|0;if((aJ|0)==0){bm=aS;break}else{aO=aJ;aG=aS;aH=aH+1|0}}}}else{NN(l|0,ay|0,156)|0;aH=39;aG=0;aO=0;while(1){A=c[m+(a7*312|0)+(a2*156|0)+(aO<<2)>>2]|0;aS=(aG|0)<(A|0)?A:aG;A=aH-1|0;if((A|0)==0){bm=aS;break}else{aH=A;aG=aS;aO=aO+1|0}}}}while(0);a1=+g[(c[a3>>2]|0)+4764>>2];dq[c[t>>2]&127](a_,ba,_,bm);if((DU(c[bd>>2]|0,c[a3>>2]|0)|0)!=0){ap=1424;break L1873}a0=Eq(a_)|0;aO=c[a3>>2]|0;aG=(c[aO+4844>>2]|0)+a0|0;g[aO+4764>>2]=a1;aO=(aG|0)>(z|0);bn=aO?x:bl;aG=aO?bl+1|0:aV;a0=aO?y:bl-1|0;if((aG|0)>(a0|0)){break}else{y=a0;aV=aG;x=bn;aU=aG+a0|0}}if((bn|0)>-1){if((bl|0)==(bn|0)){break}do{if(q){aU=(bn|0)>255?255:bn;if((bn|0)<0){NM(l|0,0,156);bo=0;break}else{x=39;aV=0;y=0;while(1){c[k+(y<<2)>>2]=aU;a$=(aV|0)<(aU|0)?aU:aV;a0=x-1|0;if((a0|0)==0){bo=a$;break}else{x=a0;aV=a$;y=y+1|0}}}}else{NN(l|0,ay|0,156)|0;y=39;aV=0;x=0;while(1){aU=c[m+(a7*312|0)+(a2*156|0)+(x<<2)>>2]|0;a$=(aV|0)<(aU|0)?aU:aV;aU=y-1|0;if((aU|0)==0){bo=a$;break}else{y=aU;aV=a$;x=x+1|0}}}}while(0);a1=+g[(c[a3>>2]|0)+4764>>2];dq[c[t>>2]&127](a_,ba,_,bo);if((DU(c[bd>>2]|0,c[a3>>2]|0)|0)!=0){ap=1435;break L1873}Eq(a_)|0;g[(c[a3>>2]|0)+4764>>2]=a1;break}ay=c[a3>>2]|0;q=c[ay+4780>>2]|0;if((q|0)>512){bq=q;br=1024}else{x=ay+4844|0;ay=1024;aV=q;y=512;while(1){a$=ay;aU=y;while(1){bs=aU+aV>>1;a0=bs-q|0;a1=+g[(c[a3>>2]|0)+4764>>2];aG=0;aO=0;do{aH=(c[k+(aG<<2)>>2]|0)+a0|0;aS=c[n+(a7*312|0)+(a2*156|0)+(aG<<2)>>2]|0;A=(aH|0)<(aS|0)?aS:aH;aH=(A|0)>255?255:A;aO=(aO|0)<(aH|0)?aH:aO;c[j+(aG<<2)>>2]=aH;aG=aG+1|0;}while((aG|0)<39);dq[c[t>>2]&127](a_,aY,_,aO);if((DU(c[bd>>2]|0,c[a3>>2]|0)|0)!=0){ap=1443;break L1873}aG=Eq(a_)|0;g[(c[a3>>2]|0)+4764>>2]=a1;if((aG|0)!=0){if(((c[x>>2]|0)+aG|0)>=(z|0)){break}}aG=bs-1|0;if((aV|0)>(aG|0)){break L1878}else{a$=bs;aU=aG}}aG=bs+1|0;a0=(a$|0)==1024?bs:a$;if((aG|0)>(aU|0)){bq=bs;br=a0;break}else{ay=a0;aV=aG;y=aU}}}if((br|0)==(bq|0)){break}y=br-q|0;aZ=+g[(c[a3>>2]|0)+4764>>2];aV=0;ay=0;do{z=(c[k+(aV<<2)>>2]|0)+y|0;x=c[n+(a7*312|0)+(a2*156|0)+(aV<<2)>>2]|0;aG=(z|0)<(x|0)?x:z;z=(aG|0)>255?255:aG;ay=(ay|0)<(z|0)?z:ay;c[j+(aV<<2)>>2]=z;aV=aV+1|0;}while((aV|0)<39);dq[c[t>>2]&127](a_,aY,_,ay);if((DU(c[bd>>2]|0,c[a3>>2]|0)|0)!=0){ap=1452;break L1873}Eq(a_)|0;g[(c[a3>>2]|0)+4764>>2]=aZ}}while(0);DT(b,a7,a2,aX);if((c[a8>>2]|0)==1){DR(b,b+304+(a7*10504|0)+(a2*5252|0)|0)}aV=(c[b+304+(a7*10504|0)+(a2*5252|0)+4844>>2]|0)+(c[b+304+(a7*10504|0)+(a2*5252|0)+4768>>2]|0)|0;c[r>>2]=aV;y=a6+aV|0;c[a5>>2]=y;aV=a2+1|0;if((aV|0)<(C|0)){a2=aV;a6=y}else{bt=y;break}}}else{bt=0}a6=bt+a4|0;a2=a7+1|0;if((a2|0)<(B|0)){a7=a2;a4=a6}else{bb=a6;break L1863}}if((ap|0)==1435){Ed(c[bd>>2]|0,1738400,(aI=i,i=i+1|0,i=i+7&-8,c[aI>>2]=0,aI)|0);i=aI;bp(-1|0);return 0}else if((ap|0)==1443){Ed(c[bd>>2]|0,1738400,(aI=i,i=i+1|0,i=i+7&-8,c[aI>>2]=0,aI)|0);i=aI;bp(-1|0);return 0}else if((ap|0)==1452){Ed(c[bd>>2]|0,1738400,(aI=i,i=i+1|0,i=i+7&-8,c[aI>>2]=0,aI)|0);i=aI;bp(-1|0);return 0}else if((ap|0)==1414){Ed(c[bd>>2]|0,1738400,(aI=i,i=i+1|0,i=i+7&-8,c[aI>>2]=0,aI)|0);i=aI;bp(-1|0);return 0}else if((ap|0)==1424){Ed(c[bd>>2]|0,1738400,(aI=i,i=i+1|0,i=i+7&-8,c[aI>>2]=0,aI)|0);i=aI;bp(-1|0);return 0}else if((ap|0)==1461){bu=c[bd>>2]|0;Ed(bu,1738400,(aI=i,i=i+1|0,i=i+7&-8,c[aI>>2]=0,aI)|0);i=aI;bp(-1|0);return 0}else if((ap|0)==1462){bu=c[bd>>2]|0;Ed(bu,1738400,(aI=i,i=i+1|0,i=i+7&-8,c[aI>>2]=0,aI)|0);i=aI;bp(-1|0);return 0}}else{bb=0}}while(0);if((bb|0)>(Q|0)){Ed(b,1744584,(aI=i,i=i+16|0,c[aI>>2]=Q,c[aI+8>>2]=bb,aI)|0);i=aI;bp(-1|0);return 0}else{E=bb;i=h;return E|0}return 0}function Em(a,b,c,d,e){a=a|0;b=b|0;c=+c;d=d|0;e=e|0;b=~~(+b7(+(c/+(d|0)))*5.799142360687256+-.5)+210|0;if((b|0)<(e&255|0)){return e|0}else{return((b|0)>254?-1:b&255)|0}return 0}function En(a,b,d,e,f){a=a|0;b=b|0;d=+d;e=e|0;f=f|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0.0,s=0.0,t=0,u=0,v=0,w=0.0,x=0.0,y=0,z=0,A=0;h=i;i=i+2048|0;j=h|0;NM(j|0,0,2048);k=-1;l=64;m=0;n=0;o=-128;while(1){p=o&255;L1991:do{if((o&255)>(f&255)){q=j+(p<<3)|0;if((c[q>>2]|0)==0){c[q>>2]=1;r=+Es(a,b,e,o);g[j+(p<<3)+4>>2]=r;s=r}else{s=+g[j+(p<<3)+4>>2]}do{if(s<=d){if(o<<24>>24==-1){t=1483}else{q=o+1&255;u=q&255;v=j+(u<<3)|0;if((c[v>>2]|0)==0){c[v>>2]=1;r=+Es(a,b,e,q);g[j+(u<<3)+4>>2]=r;w=r}else{w=+g[j+(u<<3)+4>>2]}if(w>d){break}if(o<<24>>24!=0){t=1483}}if((t|0)==1483){t=0;u=o-1&255;q=u&255;v=j+(q<<3)|0;if((c[v>>2]|0)==0){c[v>>2]=1;r=+Es(a,b,e,u);g[j+(q<<3)+4>>2]=r;x=r}else{x=+g[j+(q<<3)+4>>2]}if(x>d){break}}y=p+l&255;z=1;A=o;break L1991}}while(0);y=p-l&255;z=m;A=k}else{y=p+l&255;z=m;A=k}}while(0);p=n+1&255;if((p&255)<8){k=A;l=l>>1;m=z;n=p;o=y}else{break}}o=z<<24>>24==0?y:A;i=h;return((o&255)>(f&255)?o:f)|0}function Eo(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0;g=i;i=i+160|0;h=g|0;j=c[a+16>>2]|0;k=c[a+12>>2]|0;l=c[a+20>>2]|0;m=c[j+4864>>2]|0;if((m|0)>0){n=0;o=0;p=0;q=0;while(1){r=f-(c[b+(q<<2)>>2]|0)|0;s=(n|0)<(r|0)?r:n;t=d[129592+q|0]|0;u=r-56|0;r=u-(t<<1)|0;v=u-(t<<2)|0;t=(p|0)<(r|0)?r:p;r=(o|0)<(v|0)?v:o;v=q+1|0;if((v|0)<(m|0)){n=s;o=r;p=t;q=v}else{w=s;x=r;y=t;break}}}else{w=0;x=0;y=0}if((c[k+28>>2]|0)==2){z=(y|0)<(x|0)?y:x}else{z=y}k=f-((w|0)>(z|0)?z:w)|0;do{if((y|0)==(z|0)){c[j+4836>>2]=0}else{if((x|0)!=(z|0)){break}c[j+4836>>2]=1}}while(0);z=(k|0)<(l|0)?l:k;k=j+4780|0;c[k>>2]=z;do{if((z|0)<0){c[k>>2]=0;A=0}else{if((z|0)<=255){A=0;break}c[k>>2]=255;A=0}}while(0);do{c[h+(A<<2)>>2]=(c[b+(A<<2)>>2]|0)-z;A=A+1|0;}while((A|0)<39);A=(c[j+4836>>2]|0)==0?1:2;z=m>>>0<18?m:18;m=15<<A;b=7<<A;A=7;l=0;do{if(l>>>0<z>>>0){x=1e3;y=0;w=l;while(1){f=-(c[h+(w<<2)>>2]|0)|0;q=(y|0)<(f|0)?f:y;p=(x|0)>(f|0)?f:x;f=w+3|0;if(f>>>0<z>>>0){x=p;y=q;w=f}else{B=p;C=q;D=f;break}}}else{B=1e3;C=0;D=l}if(D>>>0<39){w=B;y=0;x=D;while(1){f=-(c[h+(x<<2)>>2]|0)|0;q=(y|0)<(f|0)?f:y;p=(w|0)>(f|0)?f:w;f=x+3|0;if(f>>>0<39){w=p;y=q;x=f}else{E=p;F=q;break}}}else{E=B;F=0}x=C-m|0;y=F-b|0;w=(x|0)>(y|0)?x:y;if((E|0)>0){y=E>>3;c[j+4808+(l<<2)>>2]=y;G=y}else{c[j+4808+(l<<2)>>2]=0;G=0}y=j+4808+(l<<2)|0;if((w|0)>0){x=w+7>>3;w=(G|0)>(x|0)?G:x;c[y>>2]=w;H=w}else{H=G}do{if((H|0)>0){w=c[a+24+(l<<2)>>2]|0;x=c[k>>2]|0;if((w|0)>(x-(H<<3)|0)){q=x-w>>3;c[y>>2]=q;I=q}else{I=H}if((I|0)<=7){J=I;break}c[y>>2]=7;J=7}else{J=H}}while(0);A=(A|0)>(J|0)?J:A;l=l+1|0;}while(l>>>0<3);l=h|0;J=j+4808|0;H=c[J>>2]|0;I=H<<3;a=j+4812|0;G=c[a>>2]|0;E=G<<3;b=j+4816|0;F=c[b>>2]|0;m=F<<3;C=0;do{B=h+(C<<2)|0;c[B>>2]=(c[B>>2]|0)+I;B=h+(C+1<<2)|0;c[B>>2]=(c[B>>2]|0)+E;B=h+(C+2<<2)|0;c[B>>2]=(c[B>>2]|0)+m;C=C+3|0;}while(C>>>0<39);if((A|0)<=0){Er(j,e,l,129592);i=g;return}c[J>>2]=H-A;c[a>>2]=G-A;c[b>>2]=F-A;c[k>>2]=(c[k>>2]|0)-(A<<3);Er(j,e,l,129592);i=g;return}function Ep(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0;g=i;i=i+160|0;h=g|0;j=c[a+16>>2]|0;k=c[a+12>>2]|0;l=c[a+20>>2]|0;a=c[j+4864>>2]|0;m=(c[k+76>>2]|0)==2?129752:129728;if((a|0)>0){n=0;o=0;p=0;q=0;r=0;s=0;while(1){t=f-(c[b+(s<<2)>>2]|0)|0;u=(n|0)<(t|0)?t:n;v=d[129752+s|0]|0;w=t-(v<<1)|0;x=t-(v<<2)|0;v=(d[m+s|0]|0)+(c[90800+(s<<2)>>2]|0)|0;y=t-(v<<1)|0;z=t-(v<<2)|0;v=(r|0)<(w|0)?w:r;w=(q|0)<(x|0)?x:q;x=(p|0)<(y|0)?y:p;y=(o|0)<(z|0)?z:o;z=s+1|0;if((z|0)<(a|0)){n=u;o=y;p=x;q=w;r=v;s=z}else{A=u;B=y;C=x;D=w;E=v;break}}}else{A=0;B=0;C=0;D=0;E=0}s=f-C|0;r=(s|0)<(l|0)?l:s;s=0;while(1){if((s|0)>=(a|0)){F=1532;break}if((r-(c[e+(s<<2)>>2]|0)-(c[90800+(s<<2)>>2]<<1)|0)<1){G=D;H=E;break}else{s=s+1|0}}L2077:do{if((F|0)==1532){s=f-B|0;r=(s|0)<(l|0)?l:s;s=0;while(1){if((s|0)>=(a|0)){G=B;H=C;break L2077}if((r-(c[e+(s<<2)>>2]|0)-(c[90800+(s<<2)>>2]<<2)|0)<1){G=D;H=C;break}else{s=s+1|0}}}}while(0);C=(c[k+28>>2]|0)==2;k=C?G:H;G=C?D:E;D=(E|0)<(H|0)?E:H;C=(D|0)<(G|0)?D:G;D=(C|0)<(k|0)?C:k;C=f-((A|0)>(D|0)?D:A)|0;A=(C|0)<(l|0)?l:C;do{if((E|0)==(D|0)){c[j+4836>>2]=0;c[j+4832>>2]=0;I=129752}else{if((H|0)==(D|0)){c[j+4836>>2]=0;c[j+4832>>2]=1;I=m;break}if((G|0)==(D|0)){c[j+4836>>2]=1;c[j+4832>>2]=0;I=129752;break}if((k|0)!=(D|0)){I=m;break}c[j+4836>>2]=1;c[j+4832>>2]=1;I=m}}while(0);m=j+4780|0;c[m>>2]=A;do{if((A|0)<0){c[m>>2]=0;J=0}else{if((A|0)<=255){J=0;break}c[m>>2]=255;J=0}}while(0);do{c[h+(J<<2)>>2]=(c[b+(J<<2)>>2]|0)-A;J=J+1|0;}while((J|0)<39);Er(j,e,h|0,I);i=g;return}function Eq(a){a=a|0;var b=0,d=0,e=0,f=0,h=0,i=0,j=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0.0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0.0,C=0.0,D=0.0,E=0.0,F=0,G=0,H=0,I=0,J=0,K=0.0,L=0.0,M=0,N=0.0,O=0.0,P=0.0,Q=0.0;b=a+16|0;d=c[b>>2]|0;e=(c[d+4836>>2]|0)==0?2:4;f=c[d+5208>>2]|0;h=d+4832|0;i=d+4780|0;j=-2-f|0;l=0;m=0;n=d+2304|0;o=c[a+8>>2]|0;while(1){if((c[h>>2]|0)==0){p=0}else{p=c[90800+(l<<2)>>2]|0}q=ac(p+(c[d+4608+(l<<2)>>2]|0)|0,e)|0;r=+g[2206176+(((c[i>>2]|0)-((c[d+4808+(c[d+5028+(l<<2)>>2]<<2)>>2]<<3)+q)&255)<<2)>>2];q=c[d+4872+(l<<2)>>2]|0;s=f-m+1|0;t=q+m|0;u=l+1|0;v=q>>>0<=s>>>0?q:s;s=v&3;w=v>>>2;if((w|0)==0){x=n;y=o}else{v=~q;q=m+j|0;z=(q>>>0<v>>>0?v:q)&-4^-4;q=n;v=w;w=o;while(1){A=v-1|0;B=r*+g[w>>2]+8388608.0;C=r*+g[w+4>>2]+8388608.0;D=r*+g[w+8>>2]+8388608.0;E=r*+g[w+12>>2]+8388608.0;F=(g[k>>2]=C+ +g[3068912+((g[k>>2]=C,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0);G=(g[k>>2]=D+ +g[3068912+((g[k>>2]=D,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0);H=(g[k>>2]=E+ +g[3068912+((g[k>>2]=E,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0);c[q>>2]=(g[k>>2]=B+ +g[3068912+((g[k>>2]=B,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200;c[q+4>>2]=F-1258291200;c[q+8>>2]=G-1258291200;c[q+12>>2]=H-1258291200;if((A|0)==0){break}else{q=q+16|0;v=A;w=w+16|0}}x=n+(z<<2)|0;y=o+(z<<2)|0}if((s|0)==0){I=y;J=x}else if((s|0)==1){K=0.0;L=0.0;M=1559}else if((s|0)==3){N=r*+g[y+8>>2];M=1558}else if((s|0)==2){N=0.0;M=1558}else{O=8388608.0;P=0.0;Q=0.0;M=1560}if((M|0)==1558){M=0;K=r*+g[y+4>>2];L=N;M=1559}if((M|0)==1559){M=0;O=r*+g[y>>2]+8388608.0;P=K;Q=L;M=1560}if((M|0)==1560){M=0;B=P+8388608.0;E=Q+8388608.0;w=(g[k>>2]=O+ +g[3068912+((g[k>>2]=O,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200|0;v=(g[k>>2]=B+ +g[3068912+((g[k>>2]=B,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200|0;if((s|0)==3){c[x+8>>2]=(g[k>>2]=E+ +g[3068912+((g[k>>2]=E,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200;M=1562}else if((s|0)==2){M=1562}else if((s|0)==1){M=1563}if((M|0)==1562){M=0;c[x+4>>2]=v;M=1563}if((M|0)==1563){M=0;c[x>>2]=w}I=y+(s<<2)|0;J=x+(s<<2)|0}if(t>>>0>f>>>0){break}else{l=u;m=t;n=J;o=I}}I=DQ(c[a+12>>2]|0,c[b>>2]|0,0)|0;c[(c[b>>2]|0)+4768>>2]=I;return c[(c[b>>2]|0)+4768>>2]|0}function Er(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;g=(c[a+4836>>2]|0)==0;h=g?2:4;i=g?1:2;g=c[a+4860>>2]|0;j=(c[a+4832>>2]|0)!=0;if(j&(g|0)>11){k=11;do{l=ac(c[90800+(k<<2)>>2]|0,h)|0;m=e+(k<<2)|0;c[m>>2]=(c[m>>2]|0)+l;k=k+1|0;}while((k|0)<(g|0))}do{if((g|0)>0){k=a+4780|0;l=h-1|0;m=0;do{n=(c[k>>2]|0)-(c[a+4808+(c[a+5028+(m<<2)>>2]<<2)>>2]<<3)|0;if(j){o=c[90800+(m<<2)>>2]|0}else{o=0}p=c[e+(m<<2)>>2]|0;do{if((p|0)<0){q=n-(ac(o,h)|0)|0;r=q-(c[b+(m<<2)>>2]|0)|0;q=l-p>>i;s=a+4608+(m<<2)|0;c[s>>2]=q;t=d[f+m|0]|0;if((q|0)>(t|0)){c[s>>2]=t;u=t}else{u=q}if(!((u|0)>0&(u<<i|0)>(r|0))){break}c[s>>2]=r>>i}else{c[a+4608+(m<<2)>>2]=0}}while(0);m=m+1|0;}while((m|0)<(g|0));if((g|0)<39){v=g;break}return}else{v=0}}while(0);NM(a+(v+1152<<2)|0,0,156-(v<<2)|0);return}function Es(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0.0,i=0.0,j=0,l=0.0,m=0,n=0,o=0,p=0.0,q=0,r=0.0,s=0.0,t=0.0,u=0.0,v=0,w=0,x=0,y=0,z=0.0,A=0.0,B=0.0,C=0,D=0.0,E=0.0,F=0.0,G=0.0,H=0.0,I=0.0,J=0.0,K=0.0,L=0.0,M=0.0,N=0.0,O=0.0;f=e&255;h=+g[2197096+(f+116<<2)>>2];i=+g[2206176+(f<<2)>>2];f=d>>>2;e=d&3;if((f|0)==0){j=b;l=0.0;m=a}else{d=f<<2;n=b+(d<<2)|0;o=b;p=0.0;b=f;f=a;while(1){q=b-1|0;r=i*+g[o>>2]+8388608.0;s=i*+g[o+4>>2]+8388608.0;t=i*+g[o+8>>2]+8388608.0;u=i*+g[o+12>>2]+8388608.0;v=(g[k>>2]=r+ +g[3068912+((g[k>>2]=r,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200|0;w=(g[k>>2]=s+ +g[3068912+((g[k>>2]=s,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200|0;x=(g[k>>2]=t+ +g[3068912+((g[k>>2]=t,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200|0;y=(g[k>>2]=u+ +g[3068912+((g[k>>2]=u,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200|0;u=+R(+(+g[f>>2]));t=u-h*+g[2164264+(v<<2)>>2];u=+R(+(+g[f+4>>2]));s=u-h*+g[2164264+(w<<2)>>2];u=+R(+(+g[f+8>>2]));r=u-h*+g[2164264+(x<<2)>>2];u=+R(+(+g[f+12>>2]));z=u-h*+g[2164264+(y<<2)>>2];A=p+(t*t+s*s+(r*r+z*z));if((q|0)==0){break}else{o=o+16|0;p=A;b=q;f=f+16|0}}j=n;l=A;m=a+(d<<2)|0}if((e|0)==2){B=0.0;C=1591}else if((e|0)==1){D=0.0;E=0.0;C=1592}else if((e|0)==3){B=i*+g[j+8>>2];C=1591}else if((e|0)==0){F=l;return+F}else{G=8388608.0;H=0.0;I=0.0}if((C|0)==1591){D=i*+g[j+4>>2];E=B;C=1592}if((C|0)==1592){G=i*+g[j>>2]+8388608.0;H=D;I=E}E=H+8388608.0;H=I+8388608.0;j=(g[k>>2]=G+ +g[3068912+((g[k>>2]=G,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200|0;d=(g[k>>2]=E+ +g[3068912+((g[k>>2]=E,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200|0;if((e|0)==3){a=(g[k>>2]=H+ +g[3068912+((g[k>>2]=H,c[k>>2]|0)-1258291200<<2)>>2],c[k>>2]|0)-1258291200|0;H=+R(+(+g[m+8>>2]));J=H-h*+g[2164264+(a<<2)>>2];C=1595}else if((e|0)==2){J=0.0;C=1595}else if((e|0)==1){K=0.0;L=0.0;C=1596}else{M=0.0;N=0.0;O=0.0}if((C|0)==1595){H=+R(+(+g[m+4>>2]));K=H-h*+g[2164264+(d<<2)>>2];L=J;C=1596}if((C|0)==1596){J=+R(+(+g[m>>2]));M=J-h*+g[2164264+(j<<2)>>2];N=K;O=L}F=l+(O*O+0.0+(N*N+M*M));return+F}function Et(){return 1744152}function Eu(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;b=c[308168+(c[a+16>>2]<<6)+(c[a+84744>>2]<<2)>>2]|0;d=a+85784|0;c[d>>2]=(c[d>>2]|0)+1;d=a+85760|0;e=(c[d>>2]|0)+b|0;c[d>>2]=e;d=a+85764|0;b=(c[d>>2]|0)+1|0;c[d>>2]=b;f=a+85768|0;if((b|0)<(c[f>>2]|0)){return}b=a+85772|0;g=c[b>>2]|0;h=a+85776|0;i=c[h>>2]|0;if((g|0)<(i|0)){c[(c[a+85780>>2]|0)+(g<<2)>>2]=e;e=(c[b>>2]|0)+1|0;c[b>>2]=e;c[d>>2]=0;j=e;k=c[h>>2]|0}else{j=g;k=i}if((j|0)!=(k|0)){return}if((k|0)>1){j=a+85780|0;a=1;do{i=c[j>>2]|0;c[i+(((a|0)/2|0)<<2)>>2]=c[i+(a<<2)>>2];a=a+2|0;}while((a|0)<(c[h>>2]|0));l=c[b>>2]|0}else{l=k}c[f>>2]=c[f>>2]<<1;c[b>>2]=(l|0)/2|0;return}function Ev(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0;d=i;i=i+2880|0;e=d|0;f=c[b+288>>2]|0;b=c[f+16>>2]|0;if((b|0)==1){g=128}else{g=(c[f+64>>2]|0)<16e3?32:64}if((c[f+104>>2]|0)==0){h=c[f+120>>2]|0}else{h=g}g=ac((b*72e3|0)+72e3|0,h)|0;h=(g|0)/(c[f+64>>2]|0)|0;g=(c[f+24>>2]|0)+156|0;b=f+85792|0;c[b>>2]=h;if((h|0)<(g|0)|(h|0)>2880){c[f+156>>2]=0;j=0;i=d;return j|0}c[f+85784>>2]=0;c[f+85788>>2]=0;c[f+85760>>2]=0;c[f+85764>>2]=0;c[f+85768>>2]=1;c[f+85772>>2]=0;h=f+85780|0;do{if((c[h>>2]|0)==0){g=NC(1600)|0;c[h>>2]=g;k=f+85776|0;if((g|0)!=0){c[k>>2]=400;break}c[k>>2]=0;Ed(f,1743424,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;c[f+156>>2]=0;j=-1;i=d;return j|0}}while(0);h=e|0;NM(h|0,0,2880);Ew(f,h);h=c[b>>2]|0;if((h|0)==0){j=0;i=d;return j|0}else{l=0}while(1){EE(f,a[e+l|0]|0,1);b=l+1|0;if(b>>>0<h>>>0){l=b}else{j=0;break}}i=d;return j|0}function Ew(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;a[e]=-1;f=e+1|0;g=a[f]<<3|7;a[f]=g;h=b+64|0;i=g<<1|(c[h>>2]|0)>15999;a[f]=i;g=b+16|0;j=((i&255)<<1&62|c[g>>2]&1)<<2|1;a[f]=j&255;i=(j<<1|(c[b+160>>2]|0)==0)&255;a[f]=i;j=e+2|0;k=(d[j]|0)<<4|c[b+84744>>2]&15;a[j]=k&255;l=k<<2&124|c[b+20>>2]&3;a[j]=l<<1&255;k=(l<<2|c[b+172>>2]&1)&255;a[j]=k;l=e+3|0;m=(d[l]|0)<<2|c[b+180>>2]&3;a[l]=m&255;n=m<<2|c[b+84756>>2]&3;a[l]=n&255;m=n<<1|c[b+164>>2]&1;a[l]=m&255;n=m<<1|c[b+168>>2]&1;a[l]=n&255;a[l]=(n<<2|c[b+176>>2]&3)&255;a[e]=-1;e=i&-15;i=c[g>>2]|0;if((i|0)==1){o=128}else{o=(c[h>>2]|0)<16e3?32:64}if((c[b+104>>2]|0)==0){p=c[b+120>>2]|0}else{p=o}if((c[b+152>>2]|0)!=0){q=0;r=i;s=k;t=(r|0)==1;u=t?10:2;v=e|u;a[f]=v;w=s&13;x=w&255;y=x|q;z=y&255;a[j]=z;return}k=(D7(p,i,c[h>>2]|0)|0)<<28>>24;q=k;r=c[g>>2]|0;s=a[j]|0;t=(r|0)==1;u=t?10:2;v=e|u;a[f]=v;w=s&13;x=w&255;y=x|q;z=y&255;a[j]=z;return}function Ex(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,h=0;if((f|0)<=0){return}g=0;h=b[a>>1]|0;do{h=((h&65535)>>>8^c[262904+(((h^(d[e+g|0]|0))&255)<<2)>>2])&65535;b[a>>1]=h;g=g+1|0;}while((g|0)<(f|0));return}function Ey(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=a+16|0;d=c[a+84744>>2]|0;if((d|0)==0){e=a+120|0;f=c[b>>2]|0}else{g=c[b>>2]|0;e=308168+(g<<6)+(d<<2)|0;f=g}g=c[a+84752>>2]|0;d=c[a+64>>2]|0;return((ac((f*72e3|0)+72e3|0,c[e>>2]|0)|0)/(d|0)|0)+g<<3|0}function Ez(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=c[a+104>>2]|0;if((d|0)>320){e=c[a>>2]|0;if((b|0)==1){f=c[a+48>>2]|0;g=((ac((e*72e3|0)+72e3|0,d)|0)/(f|0)|0)<<3;return g|0}else{g=(e*7680|0)+7680|0;return g|0}}e=c[a+48>>2]|0;f=c[a>>2]|0;if((e|0)<16e3){h=308200+(f<<6)|0}else{h=308224+(f<<6)|0}if((b|0)==2){g=(f*7680|0)+7680|0;return g|0}else if((b|0)==1){g=((ac((f*72e3|0)+72e3|0,c[h>>2]|0)|0)/(e|0)|0)<<3;return g|0}else{g=11520;return g|0}return 0}function EA(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;f=d[e+2|0]|0;g=(f&128|0)!=0?262140:196598;h=(((g^f<<10)&65536|0)==0?g:g^32773)<<1;g=(((h^f<<11)&65536|0)==0?h:h^32773)<<1;h=(((g^f<<12)&65536|0)==0?g:g^32773)<<1;g=(((h^f<<13)&65536|0)==0?h:h^32773)<<1;h=(((g^f<<14)&65536|0)==0?g:g^32773)<<1;g=(((h^f<<15)&65536|0)==0?h:h^32773)<<1;h=d[e+3|0]|0;i=(((g^f<<16)&65536|0)==0?g:g^32773)<<1;g=(((i^h<<9)&65536|0)==0?i:i^32773)<<1;i=(((g^h<<10)&65536|0)==0?g:g^32773)<<1;g=(((i^h<<11)&65536|0)==0?i:i^32773)<<1;i=(((g^h<<12)&65536|0)==0?g:g^32773)<<1;g=(((i^h<<13)&65536|0)==0?i:i^32773)<<1;i=(((g^h<<14)&65536|0)==0?g:g^32773)<<1;g=(((i^h<<15)&65536|0)==0?i:i^32773)<<1;i=((g^h<<16)&65536|0)==0?g:g^32773;g=c[b+24>>2]|0;if((g|0)>6){j=i;k=6}else{l=i>>>8&255;m=i&255;n=e+4|0;a[n]=l;o=e+5|0;a[o]=m;return}do{i=d[e+k|0]|0;b=j<<1;h=(((i<<9^b)&65536|0)==0?b:b^32773)<<1;b=(((h^i<<10)&65536|0)==0?h:h^32773)<<1;h=(((b^i<<11)&65536|0)==0?b:b^32773)<<1;b=(((h^i<<12)&65536|0)==0?h:h^32773)<<1;h=(((b^i<<13)&65536|0)==0?b:b^32773)<<1;b=(((h^i<<14)&65536|0)==0?h:h^32773)<<1;h=(((b^i<<15)&65536|0)==0?b:b^32773)<<1;j=((h^i<<16)&65536|0)==0?h:h^32773;k=k+1|0;}while((k|0)<(g|0));l=j>>>8&255;m=j&255;n=e+4|0;a[n]=l;o=e+5|0;a[o]=m;return}function EB(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0;d=i;e=c[a+52132>>2]|0;f=c[a+52128>>2]|0;g=(f|0)==0?255:f-1|0;f=(c[a+39840+(g*48|0)>>2]|0)-(c[a+292>>2]|0)|0;c[b>>2]=f;if((f|0)>-1){h=1-e+g|0;j=f-(ac(((g|0)<(e|0)?h+256|0:h)<<3,c[a+24>>2]|0)|0)|0}else{j=f}h=a+16|0;e=c[a+84744>>2]|0;if((e|0)==0){k=a+120|0;l=c[h>>2]|0}else{g=c[h>>2]|0;k=308168+(g<<6)+(e<<2)|0;l=g}g=c[a+84752>>2]|0;e=c[a+64>>2]|0;h=((ac((l*72e3|0)+72e3|0,c[k>>2]|0)|0)/(e|0)|0)+g<<3;g=h+j|0;j=h+f|0;f=((j&7|0)!=0)+((j|0)/8|0)|0;c[b>>2]=f;c[b>>2]=(c[a+296>>2]|0)+1+f;if((g|0)>=0){i=d;return g|0}Ed(a,1742144,(a=i,i=i+1|0,i=i+7&-8,c[a>>2]=0,a)|0);i=a;i=d;return g|0}function EC(a){a=a|0;var b=0,d=0;b=i;i=i+8|0;d=EB(a,b|0)|0;if((d|0)<0){i=b;return}ED(a,d);c[a+52140>>2]=0;c[a+21312>>2]=0;i=b;return}function ED(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0;do{if((e|0)>7){f=b+300|0;g=b+296|0;h=b+52132|0;i=b+292|0;j=b+284|0;k=b+24|0;l=8;do{m=c[f>>2]|0;if((m|0)==0){c[f>>2]=8;n=(c[g>>2]|0)+1|0;c[g>>2]=n;o=c[h>>2]|0;if((c[b+39840+(o*48|0)>>2]|0)==(c[i>>2]|0)){p=(c[j>>2]|0)+n|0;q=b+39840+(o*48|0)+8|0;o=c[k>>2]|0;NN(p|0,q|0,o)|0;o=c[k>>2]|0;q=(c[g>>2]|0)+o|0;c[g>>2]=q;c[i>>2]=(c[i>>2]|0)+(o<<3);c[h>>2]=(c[h>>2]|0)+1&255;r=q}else{r=n}a[(c[j>>2]|0)+r|0]=0;s=c[f>>2]|0}else{s=m}m=(l|0)<(s|0)?l:s;l=l-m|0;n=s-m|0;c[f>>2]=n;q=(c[j>>2]|0)+(c[g>>2]|0)|0;a[q]=(76>>>(l>>>0)<<n|d[q])&255;t=(c[i>>2]|0)+m|0;c[i>>2]=t;}while((l|0)>0);l=e-8|0;if((l|0)>7){u=8;v=t}else{w=l;x=1689;break}do{l=c[f>>2]|0;if((l|0)==0){c[f>>2]=8;m=(c[g>>2]|0)+1|0;c[g>>2]=m;q=c[h>>2]|0;if((c[b+39840+(q*48|0)>>2]|0)==(v|0)){n=(c[j>>2]|0)+m|0;o=b+39840+(q*48|0)+8|0;q=c[k>>2]|0;NN(n|0,o|0,q)|0;q=c[k>>2]|0;o=(c[g>>2]|0)+q|0;c[g>>2]=o;c[i>>2]=(c[i>>2]|0)+(q<<3);c[h>>2]=(c[h>>2]|0)+1&255;y=o}else{y=m}a[(c[j>>2]|0)+y|0]=0;z=c[f>>2]|0}else{z=l}l=(u|0)<(z|0)?u:z;u=u-l|0;m=z-l|0;c[f>>2]=m;o=(c[j>>2]|0)+(c[g>>2]|0)|0;a[o]=(65>>>(u>>>0)<<m|d[o])&255;v=(c[i>>2]|0)+l|0;c[i>>2]=v;}while((u|0)>0);l=e-16|0;if((l|0)>7){A=8;B=v}else{w=l;x=1689;break}do{l=c[f>>2]|0;if((l|0)==0){c[f>>2]=8;o=(c[g>>2]|0)+1|0;c[g>>2]=o;m=c[h>>2]|0;if((c[b+39840+(m*48|0)>>2]|0)==(B|0)){q=(c[j>>2]|0)+o|0;n=b+39840+(m*48|0)+8|0;m=c[k>>2]|0;NN(q|0,n|0,m)|0;m=c[k>>2]|0;n=(c[g>>2]|0)+m|0;c[g>>2]=n;c[i>>2]=(c[i>>2]|0)+(m<<3);c[h>>2]=(c[h>>2]|0)+1&255;C=n}else{C=o}a[(c[j>>2]|0)+C|0]=0;D=c[f>>2]|0}else{D=l}l=(A|0)<(D|0)?A:D;A=A-l|0;o=D-l|0;c[f>>2]=o;n=(c[j>>2]|0)+(c[g>>2]|0)|0;a[n]=(77>>>(A>>>0)<<o|d[n])&255;B=(c[i>>2]|0)+l|0;c[i>>2]=B;}while((A|0)>0);l=e-24|0;if((l|0)>7){E=8;F=B}else{w=l;x=1689;break}do{l=c[f>>2]|0;if((l|0)==0){c[f>>2]=8;n=(c[g>>2]|0)+1|0;c[g>>2]=n;o=c[h>>2]|0;if((c[b+39840+(o*48|0)>>2]|0)==(F|0)){m=(c[j>>2]|0)+n|0;q=b+39840+(o*48|0)+8|0;o=c[k>>2]|0;NN(m|0,q|0,o)|0;o=c[k>>2]|0;q=(c[g>>2]|0)+o|0;c[g>>2]=q;c[i>>2]=(c[i>>2]|0)+(o<<3);c[h>>2]=(c[h>>2]|0)+1&255;G=q}else{G=n}a[(c[j>>2]|0)+G|0]=0;H=c[f>>2]|0}else{H=l}l=(E|0)<(H|0)?E:H;E=E-l|0;n=H-l|0;c[f>>2]=n;q=(c[j>>2]|0)+(c[g>>2]|0)|0;a[q]=(69>>>(E>>>0)<<n|d[q])&255;F=(c[i>>2]|0)+l|0;c[i>>2]=F;}while((E|0)>0);l=e-32|0;if((l|0)<=31){w=l;x=1689;break}q=Et()|0;if((NL(q|0)|0)>0){I=0;J=l}else{K=l;break}while(1){l=a[q+I|0]|0;n=8;do{o=c[f>>2]|0;if((o|0)==0){c[f>>2]=8;m=(c[g>>2]|0)+1|0;c[g>>2]=m;p=c[h>>2]|0;if((c[b+39840+(p*48|0)>>2]|0)==(c[i>>2]|0)){L=(c[j>>2]|0)+m|0;M=b+39840+(p*48|0)+8|0;p=c[k>>2]|0;NN(L|0,M|0,p)|0;p=c[k>>2]|0;M=(c[g>>2]|0)+p|0;c[g>>2]=M;c[i>>2]=(c[i>>2]|0)+(p<<3);c[h>>2]=(c[h>>2]|0)+1&255;N=M}else{N=m}a[(c[j>>2]|0)+N|0]=0;O=c[f>>2]|0}else{O=o}o=(n|0)<(O|0)?n:O;n=n-o|0;m=O-o|0;c[f>>2]=m;M=(c[j>>2]|0)+(c[g>>2]|0)|0;a[M]=(l>>n<<m|d[M])&255;c[i>>2]=(c[i>>2]|0)+o;}while((n|0)>0);n=J-8|0;l=I+1|0;if((l|0)<(NL(q|0)|0)&(n|0)>7){I=l;J=n}else{w=n;x=1689;break}}}else{w=e;x=1689}}while(0);do{if((x|0)==1689){if((w|0)>0){K=w;break}return}}while(0);w=b+52136|0;x=b+300|0;e=b+296|0;J=b+52132|0;I=b+292|0;O=b+284|0;N=b+24|0;E=b+144|0;F=K;K=c[w>>2]|0;do{H=1;do{G=c[x>>2]|0;if((G|0)==0){c[x>>2]=8;B=(c[e>>2]|0)+1|0;c[e>>2]=B;A=c[J>>2]|0;if((c[b+39840+(A*48|0)>>2]|0)==(c[I>>2]|0)){D=(c[O>>2]|0)+B|0;C=b+39840+(A*48|0)+8|0;A=c[N>>2]|0;NN(D|0,C|0,A)|0;A=c[N>>2]|0;C=(c[e>>2]|0)+A|0;c[e>>2]=C;c[I>>2]=(c[I>>2]|0)+(A<<3);c[J>>2]=(c[J>>2]|0)+1&255;P=C}else{P=B}a[(c[O>>2]|0)+P|0]=0;Q=c[x>>2]|0}else{Q=G}G=(H|0)<(Q|0)?H:Q;H=H-G|0;B=Q-G|0;c[x>>2]=B;C=(c[O>>2]|0)+(c[e>>2]|0)|0;a[C]=(K>>H<<B|d[C])&255;c[I>>2]=(c[I>>2]|0)+G;}while((H|0)>0);K=(c[E>>2]|0)==0^c[w>>2];c[w>>2]=K;F=F-1|0;}while((F|0)>0);return}function EE(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;if((f|0)==0){return}g=e&255;e=b+300|0;h=b+296|0;i=b+284|0;j=b+292|0;k=f;do{f=8;while(1){l=c[e>>2]|0;if((l|0)==0){c[e>>2]=8;m=(c[h>>2]|0)+1|0;c[h>>2]=m;a[(c[i>>2]|0)+m|0]=0;n=c[e>>2]|0}else{n=l}l=(f|0)<(n|0)?f:n;m=f-l|0;o=n-l|0;c[e>>2]=o;p=(c[i>>2]|0)+(c[h>>2]|0)|0;a[p]=(g>>>(m>>>0)<<o|(d[p]|0))&255;c[j>>2]=(c[j>>2]|0)+l;if((m|0)>0){f=m}else{q=0;break}}do{f=b+39840+(q*48|0)|0;c[f>>2]=(c[f>>2]|0)+8;q=q+1|0;}while((q|0)<256);k=k-1|0;}while((k|0)!=0);return}function EF(b){b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0;e=i;i=i+8|0;f=e|0;g=b+16|0;h=b+84744|0;j=c[h>>2]|0;if((j|0)==0){k=b+120|0;l=c[g>>2]|0}else{m=c[g>>2]|0;k=308168+(m<<6)+(j<<2)|0;l=m}m=b+84752|0;j=c[m>>2]|0;g=b+64|0;n=c[g>>2]|0;o=((ac((l*72e3|0)+72e3|0,c[k>>2]|0)|0)/(n|0)|0)+j<<3;j=b+21320|0;ED(b,c[j>>2]|0);n=b+52128|0;k=c[n>>2]|0;c[b+39840+(k*48|0)+4>>2]=0;l=b+24|0;NM(b+39840+(k*48|0)+8|0,0,c[l>>2]|0);k=c[n>>2]|0;p=c[b+39840+(k*48|0)+4>>2]|0;if((c[g>>2]|0)<16e3){g=p;q=12;r=k;do{s=8-(g&7)|0;t=(q|0)<(s|0)?q:s;q=q-t|0;u=(g>>3)+(b+39840+(r*48|0)+8)|0;a[u]=(4094>>>(q>>>0)<<s-t|(d[u]|0))&255;g=t+g|0;r=c[n>>2]|0}while((q|0)>0);c[b+39840+(r*48|0)+4>>2]=g;v=r;w=g}else{g=p;p=12;r=k;do{k=8-(g&7)|0;q=(p|0)<(k|0)?p:k;p=p-q|0;t=(g>>3)+(b+39840+(r*48|0)+8)|0;a[t]=(4095>>>(p>>>0)<<k-q|(d[t]|0))&255;g=q+g|0;r=c[n>>2]|0}while((p|0)>0);c[b+39840+(r*48|0)+4>>2]=g;v=r;w=g}g=b+16|0;r=c[g>>2]|0;p=w;w=1;q=v;do{v=8-(p&7)|0;t=(w|0)<(v|0)?w:v;w=w-t|0;k=(p>>3)+(b+39840+(q*48|0)+8)|0;a[k]=(r>>w<<v-t|(d[k]|0))&255;p=t+p|0;q=c[n>>2]|0}while((w|0)>0);c[b+39840+(q*48|0)+4>>2]=p;w=p;p=2;r=q;do{q=8-(w&7)|0;t=(p|0)<(q|0)?p:q;p=p-t|0;k=(w>>3)+(b+39840+(r*48|0)+8)|0;a[k]=(1>>>(p>>>0)<<q-t|(d[k]|0))&255;w=t+w|0;r=c[n>>2]|0}while((p|0)>0);c[b+39840+(r*48|0)+4>>2]=w;p=b+160|0;t=(c[p>>2]|0)==0|0;k=w;w=1;q=r;do{r=8-(k&7)|0;v=(w|0)<(r|0)?w:r;w=w-v|0;u=(k>>3)+(b+39840+(q*48|0)+8)|0;a[u]=(t>>>(w>>>0)<<r-v|(d[u]|0))&255;k=v+k|0;q=c[n>>2]|0}while((w|0)>0);c[b+39840+(q*48|0)+4>>2]=k;w=c[h>>2]|0;h=k;k=4;t=q;do{q=8-(h&7)|0;v=(k|0)<(q|0)?k:q;k=k-v|0;u=(h>>3)+(b+39840+(t*48|0)+8)|0;a[u]=(w>>k<<q-v|(d[u]|0))&255;h=v+h|0;t=c[n>>2]|0}while((k|0)>0);c[b+39840+(t*48|0)+4>>2]=h;k=c[b+20>>2]|0;w=h;h=2;v=t;do{t=8-(w&7)|0;u=(h|0)<(t|0)?h:t;h=h-u|0;q=(w>>3)+(b+39840+(v*48|0)+8)|0;a[q]=(k>>h<<t-u|(d[q]|0))&255;w=u+w|0;v=c[n>>2]|0}while((h|0)>0);c[b+39840+(v*48|0)+4>>2]=w;h=c[m>>2]|0;m=w;w=1;k=v;do{v=8-(m&7)|0;u=(w|0)<(v|0)?w:v;w=w-u|0;q=(m>>3)+(b+39840+(k*48|0)+8)|0;a[q]=(h>>w<<v-u|(d[q]|0))&255;m=u+m|0;k=c[n>>2]|0}while((w|0)>0);c[b+39840+(k*48|0)+4>>2]=m;w=c[b+172>>2]|0;h=m;m=1;u=k;do{k=8-(h&7)|0;q=(m|0)<(k|0)?m:k;m=m-q|0;v=(h>>3)+(b+39840+(u*48|0)+8)|0;a[v]=(w>>m<<k-q|(d[v]|0))&255;h=q+h|0;u=c[n>>2]|0}while((m|0)>0);c[b+39840+(u*48|0)+4>>2]=h;m=c[b+180>>2]|0;w=h;h=2;q=u;do{u=8-(w&7)|0;v=(h|0)<(u|0)?h:u;h=h-v|0;k=(w>>3)+(b+39840+(q*48|0)+8)|0;a[k]=(m>>h<<u-v|(d[k]|0))&255;w=v+w|0;q=c[n>>2]|0}while((h|0)>0);c[b+39840+(q*48|0)+4>>2]=w;h=c[b+84756>>2]|0;m=w;w=2;v=q;do{q=8-(m&7)|0;k=(w|0)<(q|0)?w:q;w=w-k|0;u=(m>>3)+(b+39840+(v*48|0)+8)|0;a[u]=(h>>w<<q-k|(d[u]|0))&255;m=k+m|0;v=c[n>>2]|0}while((w|0)>0);c[b+39840+(v*48|0)+4>>2]=m;w=c[b+164>>2]|0;h=m;m=1;k=v;do{v=8-(h&7)|0;u=(m|0)<(v|0)?m:v;m=m-u|0;q=(h>>3)+(b+39840+(k*48|0)+8)|0;a[q]=(w>>m<<v-u|(d[q]|0))&255;h=u+h|0;k=c[n>>2]|0}while((m|0)>0);c[b+39840+(k*48|0)+4>>2]=h;m=c[b+168>>2]|0;w=h;h=1;u=k;do{k=8-(w&7)|0;q=(h|0)<(k|0)?h:k;h=h-q|0;v=(w>>3)+(b+39840+(u*48|0)+8)|0;a[v]=(m>>h<<k-q|(d[v]|0))&255;w=q+w|0;u=c[n>>2]|0}while((h|0)>0);c[b+39840+(u*48|0)+4>>2]=w;h=c[b+176>>2]|0;m=w;w=2;q=u;do{u=8-(m&7)|0;v=(w|0)<(u|0)?w:u;w=w-v|0;k=(m>>3)+(b+39840+(q*48|0)+8)|0;a[k]=(h>>w<<u-v|(d[k]|0))&255;m=v+m|0;q=c[n>>2]|0}while((w|0)>0);w=b+39840+(q*48|0)+4|0;c[w>>2]=m;if((c[p>>2]|0)==0){x=m}else{h=m;m=16;do{v=8-(h&7)|0;k=(m|0)<(v|0)?m:v;m=m-k|0;h=k+h|0;}while((m|0)>0);c[w>>2]=h;x=h}h=b+21312|0;w=c[h>>2]|0;do{if((c[g>>2]|0)==1){m=x;k=9;v=q;do{u=8-(m&7)|0;t=(k|0)<(u|0)?k:u;k=k-t|0;r=(m>>3)+(b+39840+(v*48|0)+8)|0;a[r]=(w>>k<<u-t|(d[r]|0))&255;m=t+m|0;v=c[n>>2]|0}while((k|0)>0);c[b+39840+(v*48|0)+4>>2]=m;k=b+72|0;t=c[b+21316>>2]|0;if((c[k>>2]|0)==2){r=m;u=3;s=v;do{y=8-(r&7)|0;z=(u|0)<(y|0)?u:y;u=u-z|0;A=(r>>3)+(b+39840+(s*48|0)+8)|0;a[A]=(t>>u<<y-z|(d[A]|0))&255;r=z+r|0;s=c[n>>2]|0}while((u|0)>0);c[b+39840+(s*48|0)+4>>2]=r;B=s;C=r}else{u=m;z=5;A=v;do{y=8-(u&7)|0;D=(z|0)<(y|0)?z:y;z=z-D|0;E=(u>>3)+(b+39840+(A*48|0)+8)|0;a[E]=(t>>z<<y-D|(d[E]|0))&255;u=D+u|0;A=c[n>>2]|0}while((z|0)>0);c[b+39840+(A*48|0)+4>>2]=u;B=A;C=u}z=c[k>>2]|0;if((z|0)>0){t=0;v=B;m=C;while(1){r=c[b+21328+(t<<4)>>2]|0;s=m;D=1;E=v;do{y=8-(s&7)|0;F=(D|0)<(y|0)?D:y;D=D-F|0;G=(s>>3)+(b+39840+(E*48|0)+8)|0;a[G]=(r>>D<<y-F|(d[G]|0))&255;s=F+s|0;E=c[n>>2]|0}while((D|0)>0);c[b+39840+(E*48|0)+4>>2]=s;D=c[b+21328+(t<<4)+4>>2]|0;r=s;F=1;G=E;do{y=8-(r&7)|0;H=(F|0)<(y|0)?F:y;F=F-H|0;I=(r>>3)+(b+39840+(G*48|0)+8)|0;a[I]=(D>>F<<y-H|(d[I]|0))&255;r=H+r|0;G=c[n>>2]|0}while((F|0)>0);c[b+39840+(G*48|0)+4>>2]=r;F=c[b+21328+(t<<4)+8>>2]|0;D=r;E=1;s=G;do{H=8-(D&7)|0;I=(E|0)<(H|0)?E:H;E=E-I|0;y=(D>>3)+(b+39840+(s*48|0)+8)|0;a[y]=(F>>E<<H-I|(d[y]|0))&255;D=I+D|0;s=c[n>>2]|0}while((E|0)>0);c[b+39840+(s*48|0)+4>>2]=D;E=c[b+21328+(t<<4)+12>>2]|0;F=D;G=1;r=s;do{I=8-(F&7)|0;y=(G|0)<(I|0)?G:I;G=G-y|0;H=(F>>3)+(b+39840+(r*48|0)+8)|0;a[H]=(E>>G<<I-y|(d[H]|0))&255;F=y+F|0;r=c[n>>2]|0}while((G|0)>0);c[b+39840+(r*48|0)+4>>2]=F;G=t+1|0;E=c[k>>2]|0;if((G|0)<(E|0)){t=G;v=r;m=F}else{J=0;K=E;L=r;break}}}else{J=0;K=z;L=B}while(1){if((K|0)>0){m=0;v=L;t=c[b+39840+(L*48|0)+4>>2]|0;while(1){u=(c[b+304+(J*10504|0)+(m*5252|0)+4844>>2]|0)+(c[b+304+(J*10504|0)+(m*5252|0)+4768>>2]|0)|0;A=t;E=12;G=v;do{s=8-(A&7)|0;D=(E|0)<(s|0)?E:s;E=E-D|0;y=(A>>3)+(b+39840+(G*48|0)+8)|0;a[y]=(u>>E<<s-D|(d[y]|0))&255;A=D+A|0;G=c[n>>2]|0}while((E|0)>0);c[b+39840+(G*48|0)+4>>2]=A;E=(c[b+304+(J*10504|0)+(m*5252|0)+4772>>2]|0)/2|0;u=A;D=9;y=G;do{s=8-(u&7)|0;H=(D|0)<(s|0)?D:s;D=D-H|0;I=(u>>3)+(b+39840+(y*48|0)+8)|0;a[I]=(E>>D<<s-H|(d[I]|0))&255;u=H+u|0;y=c[n>>2]|0}while((D|0)>0);c[b+39840+(y*48|0)+4>>2]=u;D=c[b+304+(J*10504|0)+(m*5252|0)+4780>>2]|0;E=u;G=8;A=y;do{H=8-(E&7)|0;I=(G|0)<(H|0)?G:H;G=G-I|0;s=(E>>3)+(b+39840+(A*48|0)+8)|0;a[s]=(D>>G<<H-I|(d[s]|0))&255;E=I+E|0;A=c[n>>2]|0}while((G|0)>0);c[b+39840+(A*48|0)+4>>2]=E;G=c[b+304+(J*10504|0)+(m*5252|0)+4784>>2]|0;D=E;y=4;u=A;do{I=8-(D&7)|0;s=(y|0)<(I|0)?y:I;y=y-s|0;H=(D>>3)+(b+39840+(u*48|0)+8)|0;a[H]=(G>>y<<I-s|(d[H]|0))&255;D=s+D|0;u=c[n>>2]|0}while((y|0)>0);y=b+39840+(u*48|0)+4|0;c[y>>2]=D;G=b+304+(J*10504|0)+(m*5252|0)+4788|0;if((c[G>>2]|0)==0){A=D;E=1;do{s=8-(A&7)|0;H=(E|0)<(s|0)?E:s;E=E-H|0;A=H+A|0;}while((E|0)>0);c[y>>2]=A;E=b+304+(J*10504|0)+(m*5252|0)+4796|0;H=c[E>>2]|0;if((H|0)==14){c[E>>2]=16;E=c[n>>2]|0;M=16;N=E;O=c[b+39840+(E*48|0)+4>>2]|0}else{M=H;N=u;O=A}H=O;E=5;s=N;do{I=8-(H&7)|0;P=(E|0)<(I|0)?E:I;E=E-P|0;Q=(H>>3)+(b+39840+(s*48|0)+8)|0;a[Q]=(M>>E<<I-P|(d[Q]|0))&255;H=P+H|0;s=c[n>>2]|0}while((E|0)>0);c[b+39840+(s*48|0)+4>>2]=H;E=b+304+(J*10504|0)+(m*5252|0)+4800|0;A=c[E>>2]|0;if((A|0)==14){c[E>>2]=16;E=c[n>>2]|0;R=16;S=E;T=c[b+39840+(E*48|0)+4>>2]|0}else{R=A;S=s;T=H}A=T;E=5;y=S;do{P=8-(A&7)|0;Q=(E|0)<(P|0)?E:P;E=E-Q|0;I=(A>>3)+(b+39840+(y*48|0)+8)|0;a[I]=(R>>E<<P-Q|(d[I]|0))&255;A=Q+A|0;y=c[n>>2]|0}while((E|0)>0);c[b+39840+(y*48|0)+4>>2]=A;E=b+304+(J*10504|0)+(m*5252|0)+4804|0;H=c[E>>2]|0;if((H|0)==14){c[E>>2]=16;E=c[n>>2]|0;U=16;V=E;W=c[b+39840+(E*48|0)+4>>2]|0}else{U=H;V=y;W=A}H=W;E=5;s=V;do{Q=8-(H&7)|0;I=(E|0)<(Q|0)?E:Q;E=E-I|0;P=(H>>3)+(b+39840+(s*48|0)+8)|0;a[P]=(U>>E<<Q-I|(d[P]|0))&255;H=I+H|0;s=c[n>>2]|0}while((E|0)>0);c[b+39840+(s*48|0)+4>>2]=H;E=c[b+304+(J*10504|0)+(m*5252|0)+4824>>2]|0;A=H;y=4;I=s;do{P=8-(A&7)|0;Q=(y|0)<(P|0)?y:P;y=y-Q|0;X=(A>>3)+(b+39840+(I*48|0)+8)|0;a[X]=(E>>y<<P-Q|(d[X]|0))&255;A=Q+A|0;I=c[n>>2]|0}while((y|0)>0);c[b+39840+(I*48|0)+4>>2]=A;y=c[b+304+(J*10504|0)+(m*5252|0)+4828>>2]|0;E=A;s=3;H=I;do{Q=8-(E&7)|0;X=(s|0)<(Q|0)?s:Q;s=s-X|0;P=(E>>3)+(b+39840+(H*48|0)+8)|0;a[P]=(y>>s<<Q-X|(d[P]|0))&255;E=X+E|0;H=c[n>>2]|0}while((s|0)>0);c[b+39840+(H*48|0)+4>>2]=E;Y=H;Z=E}else{s=D;y=1;I=u;do{A=8-(s&7)|0;X=(y|0)<(A|0)?y:A;y=y-X|0;P=(s>>3)+(b+39840+(I*48|0)+8)|0;a[P]=(1>>>(y>>>0)<<A-X|(d[P]|0))&255;s=X+s|0;I=c[n>>2]|0}while((y|0)>0);c[b+39840+(I*48|0)+4>>2]=s;y=c[G>>2]|0;u=s;D=2;E=I;do{H=8-(u&7)|0;X=(D|0)<(H|0)?D:H;D=D-X|0;P=(u>>3)+(b+39840+(E*48|0)+8)|0;a[P]=(y>>D<<H-X|(d[P]|0))&255;u=X+u|0;E=c[n>>2]|0}while((D|0)>0);c[b+39840+(E*48|0)+4>>2]=u;D=c[b+304+(J*10504|0)+(m*5252|0)+4792>>2]|0;y=u;I=1;s=E;do{G=8-(y&7)|0;X=(I|0)<(G|0)?I:G;I=I-X|0;P=(y>>3)+(b+39840+(s*48|0)+8)|0;a[P]=(D>>I<<G-X|(d[P]|0))&255;y=X+y|0;s=c[n>>2]|0}while((I|0)>0);c[b+39840+(s*48|0)+4>>2]=y;I=b+304+(J*10504|0)+(m*5252|0)+4796|0;D=c[I>>2]|0;if((D|0)==14){c[I>>2]=16;I=c[n>>2]|0;_=16;$=I;aa=c[b+39840+(I*48|0)+4>>2]|0}else{_=D;$=s;aa=y}D=aa;I=5;E=$;do{u=8-(D&7)|0;X=(I|0)<(u|0)?I:u;I=I-X|0;P=(D>>3)+(b+39840+(E*48|0)+8)|0;a[P]=(_>>I<<u-X|(d[P]|0))&255;D=X+D|0;E=c[n>>2]|0}while((I|0)>0);c[b+39840+(E*48|0)+4>>2]=D;I=b+304+(J*10504|0)+(m*5252|0)+4800|0;y=c[I>>2]|0;if((y|0)==14){c[I>>2]=16;I=c[n>>2]|0;ab=16;ad=I;ae=c[b+39840+(I*48|0)+4>>2]|0}else{ab=y;ad=E;ae=D}y=ae;I=5;s=ad;do{X=8-(y&7)|0;P=(I|0)<(X|0)?I:X;I=I-P|0;u=(y>>3)+(b+39840+(s*48|0)+8)|0;a[u]=(ab>>I<<X-P|(d[u]|0))&255;y=P+y|0;s=c[n>>2]|0}while((I|0)>0);c[b+39840+(s*48|0)+4>>2]=y;I=c[b+304+(J*10504|0)+(m*5252|0)+4808>>2]|0;D=y;E=3;P=s;do{u=8-(D&7)|0;X=(E|0)<(u|0)?E:u;E=E-X|0;G=(D>>3)+(b+39840+(P*48|0)+8)|0;a[G]=(I>>E<<u-X|(d[G]|0))&255;D=X+D|0;P=c[n>>2]|0}while((E|0)>0);c[b+39840+(P*48|0)+4>>2]=D;E=c[b+304+(J*10504|0)+(m*5252|0)+4812>>2]|0;I=D;s=3;y=P;do{X=8-(I&7)|0;G=(s|0)<(X|0)?s:X;s=s-G|0;u=(I>>3)+(b+39840+(y*48|0)+8)|0;a[u]=(E>>s<<X-G|(d[u]|0))&255;I=G+I|0;y=c[n>>2]|0}while((s|0)>0);c[b+39840+(y*48|0)+4>>2]=I;s=c[b+304+(J*10504|0)+(m*5252|0)+4816>>2]|0;E=I;P=3;D=y;do{G=8-(E&7)|0;u=(P|0)<(G|0)?P:G;P=P-u|0;X=(E>>3)+(b+39840+(D*48|0)+8)|0;a[X]=(s>>P<<G-u|(d[X]|0))&255;E=u+E|0;D=c[n>>2]|0}while((P|0)>0);c[b+39840+(D*48|0)+4>>2]=E;Y=D;Z=E}P=c[b+304+(J*10504|0)+(m*5252|0)+4832>>2]|0;s=Z;y=1;I=Y;do{u=8-(s&7)|0;X=(y|0)<(u|0)?y:u;y=y-X|0;G=(s>>3)+(b+39840+(I*48|0)+8)|0;a[G]=(P>>y<<u-X|(d[G]|0))&255;s=X+s|0;I=c[n>>2]|0}while((y|0)>0);c[b+39840+(I*48|0)+4>>2]=s;y=c[b+304+(J*10504|0)+(m*5252|0)+4836>>2]|0;P=s;E=1;D=I;do{X=8-(P&7)|0;G=(E|0)<(X|0)?E:X;E=E-G|0;u=(P>>3)+(b+39840+(D*48|0)+8)|0;a[u]=(y>>E<<X-G|(d[u]|0))&255;P=G+P|0;D=c[n>>2]|0}while((E|0)>0);c[b+39840+(D*48|0)+4>>2]=P;E=c[b+304+(J*10504|0)+(m*5252|0)+4840>>2]|0;y=P;I=1;s=D;do{G=8-(y&7)|0;u=(I|0)<(G|0)?I:G;I=I-u|0;X=(y>>3)+(b+39840+(s*48|0)+8)|0;a[X]=(E>>I<<G-u|(d[X]|0))&255;y=u+y|0;s=c[n>>2]|0}while((I|0)>0);c[b+39840+(s*48|0)+4>>2]=y;I=m+1|0;E=c[k>>2]|0;if((I|0)<(E|0)){m=I;v=s;t=y}else{af=E;ag=s;break}}}else{af=K;ag=L}t=J+1|0;if((t|0)<2){J=t;K=af;L=ag}else{ah=ag;break}}}else{k=x;z=8;t=q;do{v=8-(k&7)|0;m=(z|0)<(v|0)?z:v;z=z-m|0;r=(k>>3)+(b+39840+(t*48|0)+8)|0;a[r]=(w>>z<<v-m|(d[r]|0))&255;k=m+k|0;t=c[n>>2]|0}while((z|0)>0);z=b+39840+(t*48|0)+4|0;c[z>>2]=k;m=c[b+21316>>2]|0;r=b+72|0;v=c[r>>2]|0;if((v|0)>0){ai=k;aj=v;ak=t}else{c[z>>2]=k;ah=t;break}do{z=8-(ai&7)|0;v=(aj|0)<(z|0)?aj:z;aj=aj-v|0;F=(ai>>3)+(b+39840+(ak*48|0)+8)|0;a[F]=(m>>aj<<z-v|(d[F]|0))&255;ai=v+ai|0;ak=c[n>>2]|0}while((aj|0)>0);m=c[r>>2]|0;c[b+39840+(ak*48|0)+4>>2]=ai;if((m|0)>0){al=0;am=ak;an=ai}else{ah=ak;break}while(1){m=(c[b+304+(al*5252|0)+4844>>2]|0)+(c[b+304+(al*5252|0)+4768>>2]|0)|0;t=an;k=12;v=am;do{F=8-(t&7)|0;z=(k|0)<(F|0)?k:F;k=k-z|0;E=(t>>3)+(b+39840+(v*48|0)+8)|0;a[E]=(m>>k<<F-z|(d[E]|0))&255;t=z+t|0;v=c[n>>2]|0}while((k|0)>0);c[b+39840+(v*48|0)+4>>2]=t;k=(c[b+304+(al*5252|0)+4772>>2]|0)/2|0;m=t;z=9;E=v;do{F=8-(m&7)|0;I=(z|0)<(F|0)?z:F;z=z-I|0;D=(m>>3)+(b+39840+(E*48|0)+8)|0;a[D]=(k>>z<<F-I|(d[D]|0))&255;m=I+m|0;E=c[n>>2]|0}while((z|0)>0);c[b+39840+(E*48|0)+4>>2]=m;z=c[b+304+(al*5252|0)+4780>>2]|0;k=m;v=8;t=E;do{I=8-(k&7)|0;D=(v|0)<(I|0)?v:I;v=v-D|0;F=(k>>3)+(b+39840+(t*48|0)+8)|0;a[F]=(z>>v<<I-D|(d[F]|0))&255;k=D+k|0;t=c[n>>2]|0}while((v|0)>0);c[b+39840+(t*48|0)+4>>2]=k;v=c[b+304+(al*5252|0)+4784>>2]|0;z=k;E=9;m=t;do{D=8-(z&7)|0;F=(E|0)<(D|0)?E:D;E=E-F|0;I=(z>>3)+(b+39840+(m*48|0)+8)|0;a[I]=(v>>E<<D-F|(d[I]|0))&255;z=F+z|0;m=c[n>>2]|0}while((E|0)>0);E=b+39840+(m*48|0)+4|0;c[E>>2]=z;v=b+304+(al*5252|0)+4788|0;if((c[v>>2]|0)==0){t=z;k=1;do{F=8-(t&7)|0;I=(k|0)<(F|0)?k:F;k=k-I|0;t=I+t|0;}while((k|0)>0);c[E>>2]=t;k=b+304+(al*5252|0)+4796|0;I=c[k>>2]|0;if((I|0)==14){c[k>>2]=16;k=c[n>>2]|0;ao=16;ap=k;aq=c[b+39840+(k*48|0)+4>>2]|0}else{ao=I;ap=m;aq=t}I=aq;k=5;F=ap;do{D=8-(I&7)|0;P=(k|0)<(D|0)?k:D;k=k-P|0;u=(I>>3)+(b+39840+(F*48|0)+8)|0;a[u]=(ao>>k<<D-P|(d[u]|0))&255;I=P+I|0;F=c[n>>2]|0}while((k|0)>0);c[b+39840+(F*48|0)+4>>2]=I;k=b+304+(al*5252|0)+4800|0;t=c[k>>2]|0;if((t|0)==14){c[k>>2]=16;k=c[n>>2]|0;ar=16;as=k;at=c[b+39840+(k*48|0)+4>>2]|0}else{ar=t;as=F;at=I}t=at;k=5;E=as;do{P=8-(t&7)|0;u=(k|0)<(P|0)?k:P;k=k-u|0;D=(t>>3)+(b+39840+(E*48|0)+8)|0;a[D]=(ar>>k<<P-u|(d[D]|0))&255;t=u+t|0;E=c[n>>2]|0}while((k|0)>0);c[b+39840+(E*48|0)+4>>2]=t;k=b+304+(al*5252|0)+4804|0;I=c[k>>2]|0;if((I|0)==14){c[k>>2]=16;k=c[n>>2]|0;au=16;av=k;aw=c[b+39840+(k*48|0)+4>>2]|0}else{au=I;av=E;aw=t}I=aw;k=5;F=av;do{u=8-(I&7)|0;D=(k|0)<(u|0)?k:u;k=k-D|0;P=(I>>3)+(b+39840+(F*48|0)+8)|0;a[P]=(au>>k<<u-D|(d[P]|0))&255;I=D+I|0;F=c[n>>2]|0}while((k|0)>0);c[b+39840+(F*48|0)+4>>2]=I;k=c[b+304+(al*5252|0)+4824>>2]|0;t=I;E=4;D=F;do{P=8-(t&7)|0;u=(E|0)<(P|0)?E:P;E=E-u|0;X=(t>>3)+(b+39840+(D*48|0)+8)|0;a[X]=(k>>E<<P-u|(d[X]|0))&255;t=u+t|0;D=c[n>>2]|0}while((E|0)>0);c[b+39840+(D*48|0)+4>>2]=t;E=c[b+304+(al*5252|0)+4828>>2]|0;k=t;F=3;I=D;do{u=8-(k&7)|0;X=(F|0)<(u|0)?F:u;F=F-X|0;P=(k>>3)+(b+39840+(I*48|0)+8)|0;a[P]=(E>>F<<u-X|(d[P]|0))&255;k=X+k|0;I=c[n>>2]|0}while((F|0)>0);c[b+39840+(I*48|0)+4>>2]=k;ax=I;ay=k}else{F=z;E=1;D=m;do{t=8-(F&7)|0;X=(E|0)<(t|0)?E:t;E=E-X|0;P=(F>>3)+(b+39840+(D*48|0)+8)|0;a[P]=(1>>>(E>>>0)<<t-X|(d[P]|0))&255;F=X+F|0;D=c[n>>2]|0}while((E|0)>0);c[b+39840+(D*48|0)+4>>2]=F;E=c[v>>2]|0;m=F;z=2;k=D;do{I=8-(m&7)|0;X=(z|0)<(I|0)?z:I;z=z-X|0;P=(m>>3)+(b+39840+(k*48|0)+8)|0;a[P]=(E>>z<<I-X|(d[P]|0))&255;m=X+m|0;k=c[n>>2]|0}while((z|0)>0);c[b+39840+(k*48|0)+4>>2]=m;z=c[b+304+(al*5252|0)+4792>>2]|0;E=m;D=1;F=k;do{v=8-(E&7)|0;X=(D|0)<(v|0)?D:v;D=D-X|0;P=(E>>3)+(b+39840+(F*48|0)+8)|0;a[P]=(z>>D<<v-X|(d[P]|0))&255;E=X+E|0;F=c[n>>2]|0}while((D|0)>0);c[b+39840+(F*48|0)+4>>2]=E;D=b+304+(al*5252|0)+4796|0;z=c[D>>2]|0;if((z|0)==14){c[D>>2]=16;D=c[n>>2]|0;az=16;aA=D;aB=c[b+39840+(D*48|0)+4>>2]|0}else{az=z;aA=F;aB=E}z=aB;D=5;k=aA;do{m=8-(z&7)|0;X=(D|0)<(m|0)?D:m;D=D-X|0;P=(z>>3)+(b+39840+(k*48|0)+8)|0;a[P]=(az>>D<<m-X|(d[P]|0))&255;z=X+z|0;k=c[n>>2]|0}while((D|0)>0);c[b+39840+(k*48|0)+4>>2]=z;D=b+304+(al*5252|0)+4800|0;E=c[D>>2]|0;if((E|0)==14){c[D>>2]=16;D=c[n>>2]|0;aC=16;aD=D;aE=c[b+39840+(D*48|0)+4>>2]|0}else{aC=E;aD=k;aE=z}E=aE;D=5;F=aD;do{X=8-(E&7)|0;P=(D|0)<(X|0)?D:X;D=D-P|0;m=(E>>3)+(b+39840+(F*48|0)+8)|0;a[m]=(aC>>D<<X-P|(d[m]|0))&255;E=P+E|0;F=c[n>>2]|0}while((D|0)>0);c[b+39840+(F*48|0)+4>>2]=E;D=c[b+304+(al*5252|0)+4808>>2]|0;z=E;k=3;P=F;do{m=8-(z&7)|0;X=(k|0)<(m|0)?k:m;k=k-X|0;v=(z>>3)+(b+39840+(P*48|0)+8)|0;a[v]=(D>>k<<m-X|(d[v]|0))&255;z=X+z|0;P=c[n>>2]|0}while((k|0)>0);c[b+39840+(P*48|0)+4>>2]=z;k=c[b+304+(al*5252|0)+4812>>2]|0;D=z;F=3;E=P;do{X=8-(D&7)|0;v=(F|0)<(X|0)?F:X;F=F-v|0;m=(D>>3)+(b+39840+(E*48|0)+8)|0;a[m]=(k>>F<<X-v|(d[m]|0))&255;D=v+D|0;E=c[n>>2]|0}while((F|0)>0);c[b+39840+(E*48|0)+4>>2]=D;F=c[b+304+(al*5252|0)+4816>>2]|0;k=D;P=3;z=E;do{v=8-(k&7)|0;m=(P|0)<(v|0)?P:v;P=P-m|0;X=(k>>3)+(b+39840+(z*48|0)+8)|0;a[X]=(F>>P<<v-m|(d[X]|0))&255;k=m+k|0;z=c[n>>2]|0}while((P|0)>0);c[b+39840+(z*48|0)+4>>2]=k;ax=z;ay=k}P=c[b+304+(al*5252|0)+4836>>2]|0;F=ay;E=1;D=ax;do{m=8-(F&7)|0;X=(E|0)<(m|0)?E:m;E=E-X|0;v=(F>>3)+(b+39840+(D*48|0)+8)|0;a[v]=(P>>E<<m-X|(d[v]|0))&255;F=X+F|0;D=c[n>>2]|0}while((E|0)>0);c[b+39840+(D*48|0)+4>>2]=F;E=c[b+304+(al*5252|0)+4840>>2]|0;P=F;k=1;z=D;do{X=8-(P&7)|0;v=(k|0)<(X|0)?k:X;k=k-v|0;m=(P>>3)+(b+39840+(z*48|0)+8)|0;a[m]=(E>>k<<X-v|(d[m]|0))&255;P=v+P|0;z=c[n>>2]|0}while((k|0)>0);c[b+39840+(z*48|0)+4>>2]=P;k=al+1|0;if((k|0)<(c[r>>2]|0)){al=k;am=z;an=P}else{ah=z;break}}}}while(0);if((c[p>>2]|0)==0){aF=ah}else{EA(b,b+39840+(ah*48|0)+8|0);aF=c[n>>2]|0}ah=aF+1&255;c[n>>2]=ah;c[b+39840+(ah*48|0)>>2]=(c[b+39840+(aF*48|0)>>2]|0)+o;aF=b+52132|0;if((c[n>>2]|0)==(c[aF>>2]|0)){Ed(b,1626488,(aG=i,i=i+1|0,i=i+7&-8,c[aG>>2]=0,aG)|0);i=aG}n=c[l>>2]<<3;ah=b+72|0;do{if((c[g>>2]|0)==1){p=b+300|0;an=b+296|0;am=b+292|0;al=b+284|0;ax=b+21464|0;ay=0;aC=0;aD=c[ah>>2]|0;while(1){if((aD|0)>0){aE=ay;az=0;while(1){aA=b+304+(aC*10504|0)+(az*5252|0)|0;aB=c[b+304+(aC*10504|0)+(az*5252|0)+4784>>2]|0;au=c[73344+(aB<<2)>>2]|0;av=c[73216+(aB<<2)>>2]|0;aB=b+304+(aC*10504|0)+(az*5252|0)+4868|0;aw=c[aB>>2]|0;L2697:do{if((aw|0)>0){if((au|0)>0){aH=0;aI=0;aJ=aw}else{ar=0;as=0;while(1){at=((c[b+304+(aC*10504|0)+(az*5252|0)+4608+(as<<2)>>2]|0)==-1?0:au)+ar|0;ao=as+1|0;if((ao|0)<(aw|0)){ar=at;as=ao}else{aK=at;aL=aw;break L2697}}}while(1){as=c[b+304+(aC*10504|0)+(az*5252|0)+4608+(aI<<2)>>2]|0;if((as|0)==-1){aM=aH;aN=aJ}else{ar=au;do{at=c[p>>2]|0;if((at|0)==0){c[p>>2]=8;ao=(c[an>>2]|0)+1|0;c[an>>2]=ao;ap=c[aF>>2]|0;if((c[b+39840+(ap*48|0)>>2]|0)==(c[am>>2]|0)){aq=(c[al>>2]|0)+ao|0;ak=b+39840+(ap*48|0)+8|0;ap=c[l>>2]|0;NN(aq|0,ak|0,ap)|0;ap=c[l>>2]|0;ak=(c[an>>2]|0)+ap|0;c[an>>2]=ak;c[am>>2]=(c[am>>2]|0)+(ap<<3);c[aF>>2]=(c[aF>>2]|0)+1&255;aO=ak}else{aO=ao}a[(c[al>>2]|0)+aO|0]=0;aP=c[p>>2]|0}else{aP=at}at=(ar|0)<(aP|0)?ar:aP;ar=ar-at|0;ao=aP-at|0;c[p>>2]=ao;ak=(c[al>>2]|0)+(c[an>>2]|0)|0;a[ak]=(as>>ar<<ao|(d[ak]|0))&255;c[am>>2]=(c[am>>2]|0)+at;}while((ar|0)>0);aM=aH+au|0;aN=c[aB>>2]|0}ar=aI+1|0;if((ar|0)<(aN|0)){aH=aM;aI=ar;aJ=aN}else{aK=aM;aL=ar;break}}}else{aK=0;aL=0}}while(0);aB=b+304+(aC*10504|0)+(az*5252|0)+4860|0;au=c[aB>>2]|0;if((aL|0)<(au|0)){aw=(av|0)>0;s=aK;y=aL;ar=au;while(1){au=c[b+304+(aC*10504|0)+(az*5252|0)+4608+(y<<2)>>2]|0;if((au|0)==-1){aQ=s;aR=ar}else{if(aw){as=av;do{at=c[p>>2]|0;if((at|0)==0){c[p>>2]=8;ak=(c[an>>2]|0)+1|0;c[an>>2]=ak;ao=c[aF>>2]|0;if((c[b+39840+(ao*48|0)>>2]|0)==(c[am>>2]|0)){ap=(c[al>>2]|0)+ak|0;aq=b+39840+(ao*48|0)+8|0;ao=c[l>>2]|0;NN(ap|0,aq|0,ao)|0;ao=c[l>>2]|0;aq=(c[an>>2]|0)+ao|0;c[an>>2]=aq;c[am>>2]=(c[am>>2]|0)+(ao<<3);c[aF>>2]=(c[aF>>2]|0)+1&255;aS=aq}else{aS=ak}a[(c[al>>2]|0)+aS|0]=0;aT=c[p>>2]|0}else{aT=at}at=(as|0)<(aT|0)?as:aT;as=as-at|0;ak=aT-at|0;c[p>>2]=ak;aq=(c[al>>2]|0)+(c[an>>2]|0)|0;a[aq]=(au>>as<<ak|(d[aq]|0))&255;c[am>>2]=(c[am>>2]|0)+at;}while((as|0)>0);aU=c[aB>>2]|0}else{aU=ar}aQ=s+av|0;aR=aU}as=y+1|0;if((as|0)<(aR|0)){s=aQ;y=as;ar=aR}else{aV=aQ;break}}}else{aV=aK}if((c[b+304+(aC*10504|0)+(az*5252|0)+4788>>2]|0)==2){ar=(c[ax>>2]|0)*3|0;y=b+304+(aC*10504|0)+(az*5252|0)+4772|0;s=c[y>>2]|0;av=(ar|0)>(s|0)?s:ar;ar=EJ(b,c[b+304+(aC*10504|0)+(az*5252|0)+4796>>2]|0,0,av,aA)|0;aW=(EJ(b,c[b+304+(aC*10504|0)+(az*5252|0)+4800>>2]|0,av,c[y>>2]|0,aA)|0)+ar|0}else{ar=c[b+304+(aC*10504|0)+(az*5252|0)+4772>>2]|0;y=c[b+304+(aC*10504|0)+(az*5252|0)+4824>>2]|0;av=c[b+21360+(y+1<<2)>>2]|0;s=c[b+21360+(y+2+(c[b+304+(aC*10504|0)+(az*5252|0)+4828>>2]|0)<<2)>>2]|0;y=(av|0)>(ar|0)?ar:av;av=(s|0)>(ar|0)?ar:s;s=EJ(b,c[b+304+(aC*10504|0)+(az*5252|0)+4796>>2]|0,0,y,aA)|0;aB=(EJ(b,c[b+304+(aC*10504|0)+(az*5252|0)+4800>>2]|0,y,av,aA)|0)+s|0;aW=aB+(EJ(b,c[b+304+(aC*10504|0)+(az*5252|0)+4804>>2]|0,av,ar,aA)|0)|0}ar=aV+aE+aW+(EI(b,aA)|0)|0;av=az+1|0;aB=c[ah>>2]|0;if((av|0)<(aB|0)){aE=ar;az=av}else{aX=ar;aY=aB;break}}}else{aX=ay;aY=aD}az=aC+1|0;if((az|0)<2){ay=aX;aC=az;aD=aY}else{aZ=aX;break}}}else{if((c[ah>>2]|0)<=0){aZ=0;break}aD=b+300|0;aC=b+296|0;ay=b+292|0;ax=b+284|0;am=b+21464|0;an=0;al=0;while(1){p=b+304+(al*5252|0)|0;az=b+304+(al*5252|0)+5188|0;if((c[b+304+(al*5252|0)+4788>>2]|0)==2){aE=0;z=0;P=0;while(1){aB=c[(c[az>>2]|0)+(aE<<2)>>2]|0;ar=(aB|0)/3|0;av=c[b+304+(al*5252|0)+5192+(aE<<2)>>2]|0;if((aB|0)>2){aB=(av|0)>0;s=(ar|0)>1?ar:1;y=0;aw=P;while(1){as=aw*3|0;au=c[b+304+(al*5252|0)+4608+(as<<2)>>2]|0;at=(au|0)>0?au:0;if(aB){au=av;do{aq=c[aD>>2]|0;if((aq|0)==0){c[aD>>2]=8;ak=(c[aC>>2]|0)+1|0;c[aC>>2]=ak;ao=c[aF>>2]|0;if((c[b+39840+(ao*48|0)>>2]|0)==(c[ay>>2]|0)){ap=(c[ax>>2]|0)+ak|0;ai=b+39840+(ao*48|0)+8|0;ao=c[l>>2]|0;NN(ap|0,ai|0,ao)|0;ao=c[l>>2]|0;ai=(c[aC>>2]|0)+ao|0;c[aC>>2]=ai;c[ay>>2]=(c[ay>>2]|0)+(ao<<3);c[aF>>2]=(c[aF>>2]|0)+1&255;a_=ai}else{a_=ak}a[(c[ax>>2]|0)+a_|0]=0;a$=c[aD>>2]|0}else{a$=aq}aq=(au|0)<(a$|0)?au:a$;au=au-aq|0;ak=a$-aq|0;c[aD>>2]=ak;ai=(c[ax>>2]|0)+(c[aC>>2]|0)|0;a[ai]=(at>>au<<ak|(d[ai]|0))&255;a0=(c[ay>>2]|0)+aq|0;c[ay>>2]=a0;}while((au|0)>0);au=c[b+304+(al*5252|0)+4608+(as+1<<2)>>2]|0;at=(au|0)>0?au:0;au=av;aq=a0;do{ai=c[aD>>2]|0;if((ai|0)==0){c[aD>>2]=8;ak=(c[aC>>2]|0)+1|0;c[aC>>2]=ak;ao=c[aF>>2]|0;if((c[b+39840+(ao*48|0)>>2]|0)==(aq|0)){ap=(c[ax>>2]|0)+ak|0;aj=b+39840+(ao*48|0)+8|0;ao=c[l>>2]|0;NN(ap|0,aj|0,ao)|0;ao=c[l>>2]|0;aj=(c[aC>>2]|0)+ao|0;c[aC>>2]=aj;c[ay>>2]=(c[ay>>2]|0)+(ao<<3);c[aF>>2]=(c[aF>>2]|0)+1&255;a1=aj}else{a1=ak}a[(c[ax>>2]|0)+a1|0]=0;a2=c[aD>>2]|0}else{a2=ai}ai=(au|0)<(a2|0)?au:a2;au=au-ai|0;ak=a2-ai|0;c[aD>>2]=ak;aj=(c[ax>>2]|0)+(c[aC>>2]|0)|0;a[aj]=(at>>au<<ak|(d[aj]|0))&255;aq=(c[ay>>2]|0)+ai|0;c[ay>>2]=aq;}while((au|0)>0);au=c[b+304+(al*5252|0)+4608+(as+2<<2)>>2]|0;at=(au|0)>0?au:0;au=av;ai=aq;do{aj=c[aD>>2]|0;if((aj|0)==0){c[aD>>2]=8;ak=(c[aC>>2]|0)+1|0;c[aC>>2]=ak;ao=c[aF>>2]|0;if((c[b+39840+(ao*48|0)>>2]|0)==(ai|0)){ap=(c[ax>>2]|0)+ak|0;w=b+39840+(ao*48|0)+8|0;ao=c[l>>2]|0;NN(ap|0,w|0,ao)|0;ao=c[l>>2]|0;w=(c[aC>>2]|0)+ao|0;c[aC>>2]=w;c[ay>>2]=(c[ay>>2]|0)+(ao<<3);c[aF>>2]=(c[aF>>2]|0)+1&255;a3=w}else{a3=ak}a[(c[ax>>2]|0)+a3|0]=0;a4=c[aD>>2]|0}else{a4=aj}aj=(au|0)<(a4|0)?au:a4;au=au-aj|0;ak=a4-aj|0;c[aD>>2]=ak;w=(c[ax>>2]|0)+(c[aC>>2]|0)|0;a[w]=(at>>au<<ak|(d[w]|0))&255;ai=(c[ay>>2]|0)+aj|0;c[ay>>2]=ai;}while((au|0)>0)}au=y+1|0;if((au|0)<(ar|0)){y=au;aw=aw+1|0}else{break}}a5=(ac(av*3|0,s)|0)+z|0;a6=s+P|0}else{a5=z;a6=P}aw=aE+1|0;if((aw|0)<4){aE=aw;z=a5;P=a6}else{break}}P=(c[am>>2]|0)*3|0;z=b+304+(al*5252|0)+4772|0;aE=c[z>>2]|0;aw=(P|0)>(aE|0)?aE:P;P=EJ(b,c[b+304+(al*5252|0)+4796>>2]|0,0,aw,p)|0;a7=(EJ(b,c[b+304+(al*5252|0)+4800>>2]|0,aw,c[z>>2]|0,p)|0)+P|0;a8=a5}else{P=0;z=0;aw=0;while(1){aE=c[(c[az>>2]|0)+(P<<2)>>2]|0;y=c[b+304+(al*5252|0)+5192+(P<<2)>>2]|0;if((aE|0)>0){if((y|0)>0){ar=0;aB=aw;while(1){aA=c[b+304+(al*5252|0)+4608+(aB<<2)>>2]|0;au=(aA|0)>0?aA:0;aA=y;do{ai=c[aD>>2]|0;if((ai|0)==0){c[aD>>2]=8;at=(c[aC>>2]|0)+1|0;c[aC>>2]=at;aq=c[aF>>2]|0;if((c[b+39840+(aq*48|0)>>2]|0)==(c[ay>>2]|0)){as=(c[ax>>2]|0)+at|0;aj=b+39840+(aq*48|0)+8|0;aq=c[l>>2]|0;NN(as|0,aj|0,aq)|0;aq=c[l>>2]|0;aj=(c[aC>>2]|0)+aq|0;c[aC>>2]=aj;c[ay>>2]=(c[ay>>2]|0)+(aq<<3);c[aF>>2]=(c[aF>>2]|0)+1&255;a9=aj}else{a9=at}a[(c[ax>>2]|0)+a9|0]=0;ba=c[aD>>2]|0}else{ba=ai}ai=(aA|0)<(ba|0)?aA:ba;aA=aA-ai|0;at=ba-ai|0;c[aD>>2]=at;aj=(c[ax>>2]|0)+(c[aC>>2]|0)|0;a[aj]=(au>>aA<<at|(d[aj]|0))&255;c[ay>>2]=(c[ay>>2]|0)+ai;}while((aA|0)>0);aA=ar+1|0;if((aA|0)<(aE|0)){ar=aA;aB=aB+1|0}else{break}}}bb=(ac(y,aE)|0)+z|0;bc=aE+aw|0}else{bb=z;bc=aw}aB=P+1|0;if((aB|0)<4){P=aB;z=bb;aw=bc}else{break}}aw=c[b+304+(al*5252|0)+4772>>2]|0;z=c[b+304+(al*5252|0)+4824>>2]|0;P=c[b+21360+(z+1<<2)>>2]|0;az=c[b+21360+(z+2+(c[b+304+(al*5252|0)+4828>>2]|0)<<2)>>2]|0;z=(P|0)>(aw|0)?aw:P;P=(az|0)>(aw|0)?aw:az;az=EJ(b,c[b+304+(al*5252|0)+4796>>2]|0,0,z,p)|0;aB=(EJ(b,c[b+304+(al*5252|0)+4800>>2]|0,z,P,p)|0)+az|0;a7=aB+(EJ(b,c[b+304+(al*5252|0)+4804>>2]|0,P,aw,p)|0)|0;a8=bb}aw=a8+an+a7+(EI(b,p)|0)|0;P=al+1|0;if((P|0)<(c[ah>>2]|0)){an=aw;al=P}else{aZ=aw;break}}}}while(0);ah=b+21324|0;ED(b,c[ah>>2]|0);a7=aZ+n+(c[ah>>2]|0)|0;c[h>>2]=(c[h>>2]|0)+((o-a7|0)/8|0);n=EB(b,f)|0;f=b+52140|0;if((n|0)==(c[f>>2]|0)){bd=n}else{Ed(b,1738272,(aG=i,i=i+1|0,i=i+7&-8,c[aG>>2]=0,aG)|0);i=aG;bd=c[f>>2]|0}n=c[h>>2]<<3;if((n|0)!=(bd|0)){aZ=c[ah>>2]|0;ah=c[j>>2]|0;j=c[l>>2]<<3;Ed(b,1703248,(aG=i,i=i+72|0,c[aG>>2]=n,c[aG+8>>2]=bd,c[aG+16>>2]=aZ,c[aG+24>>2]=ah,c[aG+32>>2]=j,c[aG+40>>2]=a7-aZ-j,c[aG+48>>2]=a7,c[aG+56>>2]=(a7|0)%8|0,c[aG+64>>2]=o,aG)|0);i=aG;Ed(b,1679672,(aG=i,i=i+1|0,i=i+7&-8,c[aG>>2]=0,aG)|0);i=aG;Ed(b,1662992,(aG=i,i=i+1|0,i=i+7&-8,c[aG>>2]=0,aG)|0);i=aG;Ed(b,1646184,(aG=i,i=i+1|0,i=i+7&-8,c[aG>>2]=0,aG)|0);i=aG;Ed(b,1633088,(aG=i,i=i+1|0,i=i+7&-8,c[aG>>2]=0,aG)|0);i=aG;c[f>>2]=c[h>>2]<<3}h=b+292|0;f=c[h>>2]|0;if((f|0)>1e9){be=0}else{i=e;return 0}do{aG=b+39840+(be*48|0)|0;c[aG>>2]=(c[aG>>2]|0)-f;be=be+1|0;}while((be|0)<256);c[h>>2]=0;i=e;return 0}function EG(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0;f=a+296|0;g=c[f>>2]|0;h=g+1|0;if((g|0)<0){i=0;return i|0}if((d|0)!=0&(h|0)>(d|0)){i=-1;return i|0}d=c[a+284>>2]|0;NN(b|0,d|0,h)|0;c[f>>2]=-1;c[a+300>>2]=0;if((e|0)==0){i=h;return i|0}Ex(a+85752|0,b,h);b=a+85788|0;c[b>>2]=(c[b>>2]|0)+h;i=h;return i|0}function EH(a){a=a|0;c[a+52132>>2]=0;c[a+52128>>2]=0;c[a+39840>>2]=0;c[a+284>>2]=NC(147456)|0;c[a+288>>2]=147456;c[a+296>>2]=-1;c[a+300>>2]=0;c[a+292>>2]=0;return}function EI(b,f){b=b|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0;h=(c[f+4840>>2]|0)+32|0;i=c[f+4772>>2]|0;j=(c[f+4776>>2]|0)-i|0;if((j|0)<=3){k=0;return k|0}l=c[174576+(h<<4)>>2]|0;m=c[174580+(h<<4)>>2]|0;h=b+300|0;n=b+296|0;o=b+52132|0;p=b+292|0;q=b+284|0;r=b+24|0;s=(j|0)/4|0;j=0;t=f+2304+(i<<2)|0;u=f+(i<<2)|0;while(1){do{if((c[t>>2]|0)==0){v=0;w=0}else{if(+g[u>>2]>=0.0){v=8;w=0;break}v=8;w=1}}while(0);do{if((c[t+4>>2]|0)==0){x=v;y=w}else{i=v|4;f=w<<1;if(+g[u+4>>2]>=0.0){x=i;y=f;break}x=i;y=f|1}}while(0);do{if((c[t+8>>2]|0)==0){z=x;A=y}else{f=x+2|0;i=y<<1;if(+g[u+8>>2]>=0.0){z=f;A=i;break}z=f;A=i|1}}while(0);do{if((c[t+12>>2]|0)==0){B=z;C=A}else{i=z+1|0;f=A<<1;if(+g[u+12>>2]>=0.0){B=i;C=f;break}B=i;C=f|1}}while(0);f=t+16|0;i=u+16|0;D=(e[l+(B<<1)>>1]|0)+C|0;E=m+B|0;F=a[E]|0;if(F<<24>>24==0){G=0}else{H=F&255;do{F=c[h>>2]|0;if((F|0)==0){c[h>>2]=8;I=(c[n>>2]|0)+1|0;c[n>>2]=I;J=c[o>>2]|0;if((c[b+39840+(J*48|0)>>2]|0)==(c[p>>2]|0)){K=(c[q>>2]|0)+I|0;L=b+39840+(J*48|0)+8|0;J=c[r>>2]|0;NN(K|0,L|0,J)|0;J=c[r>>2]|0;L=(c[n>>2]|0)+J|0;c[n>>2]=L;c[p>>2]=(c[p>>2]|0)+(J<<3);c[o>>2]=(c[o>>2]|0)+1&255;M=L}else{M=I}a[(c[q>>2]|0)+M|0]=0;N=c[h>>2]|0}else{N=F}F=(H|0)<(N|0)?H:N;H=H-F|0;I=N-F|0;c[h>>2]=I;L=(c[q>>2]|0)+(c[n>>2]|0)|0;a[L]=(D>>H<<I|(d[L]|0))&255;c[p>>2]=(c[p>>2]|0)+F;}while((H|0)>0);G=d[E]|0}H=G+j|0;D=s-1|0;if((D|0)>0){s=D;j=H;t=f;u=i}else{k=H;break}}return k|0}function EJ(b,f,h,i,j){b=b|0;f=f|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0;k=c[174568+(f<<4)>>2]|0;if(!((f|0)!=0&(h|0)<(i|0))){l=0;return l|0}m=f>>>0>15;n=k&65535;o=c[174580+(f<<4)>>2]|0;p=c[174576+(f<<4)>>2]|0;f=b+300|0;q=b+296|0;r=b+52132|0;s=b+292|0;t=b+284|0;u=b+24|0;v=0;w=h;while(1){h=c[j+2304+(w<<2)>>2]|0;x=w+1|0;y=c[j+2304+(x<<2)>>2]|0;if((h|0)==0){z=0;A=0}else{z=+g[j+(w<<2)>>2]<0.0|0;A=-1}do{if(m){if(h>>>0>14){B=15;C=z|(h<<1)+131042&131070;D=n}else{B=h;C=z;D=0}if(y>>>0<=14){E=B;F=C;G=16;H=D;I=2041;break}J=(D&65535)+k&65535;K=16;L=C<<k|y+65521&65535;M=B;N=15;I=2042}else{E=h;F=z;G=k;H=0;I=2041}}while(0);if((I|0)==2041){I=0;if((y|0)==0){O=F;P=A;Q=H;R=G;S=E;T=0}else{J=H;K=G;L=F;M=E;N=y;I=2042}}if((I|0)==2042){I=0;O=+g[j+(x<<2)>>2]<0.0|L<<1;P=A-1&65535;Q=J;R=K;S=M;T=N}h=(ac(R,S)|0)+T|0;U=Q-P&65535;V=(d[o+h|0]|0)+P&65535;W=e[p+(h<<1)>>1]|0;h=V<<16>>16;if(V<<16>>16>0){V=h;do{X=c[f>>2]|0;if((X|0)==0){c[f>>2]=8;Y=(c[q>>2]|0)+1|0;c[q>>2]=Y;Z=c[r>>2]|0;if((c[b+39840+(Z*48|0)>>2]|0)==(c[s>>2]|0)){_=(c[t>>2]|0)+Y|0;$=b+39840+(Z*48|0)+8|0;Z=c[u>>2]|0;NN(_|0,$|0,Z)|0;Z=c[u>>2]|0;$=(c[q>>2]|0)+Z|0;c[q>>2]=$;c[s>>2]=(c[s>>2]|0)+(Z<<3);c[r>>2]=(c[r>>2]|0)+1&255;aa=$}else{aa=Y}a[(c[t>>2]|0)+aa|0]=0;ab=c[f>>2]|0}else{ab=X}X=(V|0)<(ab|0)?V:ab;V=V-X|0;Y=ab-X|0;c[f>>2]=Y;$=(c[t>>2]|0)+(c[q>>2]|0)|0;a[$]=(W>>>(V>>>0)<<Y|(d[$]|0))&255;c[s>>2]=(c[s>>2]|0)+X;}while((V|0)>0)}V=U&65535;if(Q<<16>>16!=P<<16>>16){W=V;do{x=c[f>>2]|0;if((x|0)==0){c[f>>2]=8;y=(c[q>>2]|0)+1|0;c[q>>2]=y;X=c[r>>2]|0;if((c[b+39840+(X*48|0)>>2]|0)==(c[s>>2]|0)){$=(c[t>>2]|0)+y|0;Y=b+39840+(X*48|0)+8|0;X=c[u>>2]|0;NN($|0,Y|0,X)|0;X=c[u>>2]|0;Y=(c[q>>2]|0)+X|0;c[q>>2]=Y;c[s>>2]=(c[s>>2]|0)+(X<<3);c[r>>2]=(c[r>>2]|0)+1&255;ad=Y}else{ad=y}a[(c[t>>2]|0)+ad|0]=0;ae=c[f>>2]|0}else{ae=x}x=(W|0)<(ae|0)?W:ae;W=W-x|0;y=ae-x|0;c[f>>2]=y;Y=(c[t>>2]|0)+(c[q>>2]|0)|0;a[Y]=(O>>W<<y|(d[Y]|0))&255;c[s>>2]=(c[s>>2]|0)+x;}while((W|0)>0)}W=V+v+h|0;U=w+2|0;if((U|0)<(i|0)){v=W;w=U}else{l=W;break}}return l|0}function EK(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0.0,M=0.0,N=0,O=0,P=0.0,Q=0.0,R=0.0,S=0.0,T=0.0,U=0,V=0,W=0.0,X=0.0,Y=0.0,Z=0.0,_=0,$=0,aa=0,ab=0.0,ad=0.0,ae=0.0,af=0.0,ag=0.0,ah=0.0,ai=0.0,aj=0,ak=0.0,al=0.0,am=0.0,an=0.0,ao=0,ap=0,aq=0,ar=0;j=i;i=i+20064|0;k=j|0;l=j+8056|0;m=j+16112|0;n=j+18064|0;o=j+20016|0;p=j+20024|0;q=j+20056|0;r=q;s=i;i=i+16|0;t=i;i=i+16|0;u=i;i=i+8|0;v=u;w=i;i=i+8|0;c[q>>2]=1056964608;c[q+4>>2]=1056964608;NM(s|0,0,16);NM(t|0,0,16);x=o|0;c[x>>2]=b;c[o+4>>2]=d;y=a+4|0;if((c[y>>2]|0)==0){z=a+76|0;A=c[z>>2]|0;B=A*576|0;c[y>>2]=1;NM(k|0,0,8056);NM(l|0,0,8056);y=B+862|0;if((y|0)>0){C=a+72|0;D=0;E=0;while(1){do{if((D|0)<(B|0)){g[k+(D<<2)>>2]=0.0;if((c[C>>2]|0)!=2){F=E;break}g[l+(D<<2)>>2]=0.0;F=E}else{g[k+(D<<2)>>2]=+g[b+(E<<2)>>2];if((c[C>>2]|0)==2){g[l+(D<<2)>>2]=+g[d+(E<<2)>>2]}F=E+1|0}}while(0);G=D+1|0;if((G|0)<(y|0)){D=G;E=F}else{break}}}if((A|0)>0){F=a+72|0;E=0;D=c[F>>2]|0;y=A;while(1){if((D|0)>0){A=0;do{c[a+304+(E*10504|0)+(A*5252|0)+4788>>2]=2;A=A+1|0;H=c[F>>2]|0;}while((A|0)<(H|0));I=H;J=c[z>>2]|0}else{I=D;J=y}A=E+1|0;if((A|0)<(J|0)){E=A;D=I;y=J}else{break}}}EY(a,k|0,l|0)}l=a+84752|0;c[l>>2]=0;k=a+39836|0;J=(c[k>>2]|0)-(c[a+39832>>2]|0)|0;c[k>>2]=J;if((J|0)<0){c[k>>2]=(c[a+64>>2]|0)+J;c[l>>2]=1}c[u>>2]=0;c[u+4>>2]=0;l=a+76|0;J=c[l>>2]|0;L2871:do{if((J|0)>0){k=a+72|0;y=u;I=m|0;D=n|0;E=w|0;z=a+180|0;H=0;F=c[k>>2]|0;while(1){if((F|0)>0){A=(H*576|0)+304|0;C=0;do{c[v+(C<<2)>>2]=(c[o+(C<<2)>>2]|0)+(A<<2);C=C+1|0;}while((C|0)<(F|0))}if((CH(a,y,H,I,D,s+(H<<3)|0,t+(H<<3)|0,p+(H<<4)|0,E)|0)!=0){K=-4;break}do{if((c[z>>2]|0)==1){L=+g[p+(H<<4)+12>>2];M=+g[p+(H<<4)+8>>2]+L;C=r+(H<<2)|0;g[C>>2]=M;if(M<=0.0){break}g[C>>2]=L/M}}while(0);C=c[k>>2]|0;if((C|0)>0){A=0;while(1){c[a+304+(H*10504|0)+(A*5252|0)+4788>>2]=c[w+(A<<2)>>2];c[a+304+(H*10504|0)+(A*5252|0)+4792>>2]=0;b=A+1|0;B=c[k>>2]|0;if((b|0)<(B|0)){A=b}else{N=B;break}}}else{N=C}A=H+1|0;B=c[l>>2]|0;if((A|0)<(B|0)){H=A;F=N}else{O=B;break L2871}}i=j;return K|0}else{O=J}}while(0);J=a+85796|0;N=c[J>>2]|0;do{if((c[N>>2]|0)==0){g[N+8>>2]=1.0}else{M=+g[a+27804>>2];L=+g[a+27812>>2];if((c[a+72>>2]|0)==2){P=+g[a+27816>>2];Q=+g[a+27808>>2]}else{P=L;Q=M}R=L+P;L=M+Q;if((O|0)==2){S=L>R?L:R}else{S=L}L=S*.5*+g[N+4>>2];if(L>.03125){w=N+8|0;R=+g[w>>2];do{if(R<1.0){M=+g[N+12>>2];if(R>=M){break}g[w>>2]=M}else{g[w>>2]=1.0}}while(0);g[(c[J>>2]|0)+12>>2]=1.0;break}R=L*31.98+625.0e-6;w=N+8|0;M=+g[w>>2];do{if(M<R){T=+g[N+12>>2];if(T>=R){g[w>>2]=R;break}if(M>=T){break}g[w>>2]=T}else{g[w>>2]=(R*.075+.925)*M;p=(c[J>>2]|0)+8|0;if(+g[p>>2]>=R){break}g[p>>2]=R}}while(0);g[(c[J>>2]|0)+12>>2]=R}}while(0);EY(a,c[x>>2]|0,d);d=a+84756|0;c[d>>2]=0;L2918:do{if((c[a+80>>2]|0)==0){if((c[a+180>>2]|0)!=1){U=0;break}x=c[l>>2]|0;L2922:do{if((x|0)>0){J=c[a+72>>2]|0;if((J|0)>0){V=0;W=0.0;X=0.0}else{N=0;while(1){N=N+1|0;if((N|0)>=(x|0)){break L2922}}}while(1){N=0;Y=W;Z=X;do{Y=Y+ +g[t+(V<<3)+(N<<2)>>2];Z=Z+ +g[s+(V<<3)+(N<<2)>>2];N=N+1|0;}while((N|0)<(J|0));N=V+1|0;if((N|0)<(x|0)){V=N;W=Y;X=Z}else{break}}if(Y>Z){U=0;break L2918}}}while(0);J=x-1|0;if((c[a+5092>>2]|0)!=(c[a+10344>>2]|0)){U=0;break}if((c[a+304+(J*10504|0)+4788>>2]|0)!=(c[a+304+(J*10504|0)+10040>>2]|0)){U=0;break}c[d>>2]=2;U=1}else{c[d>>2]=2;U=1}}while(0);V=U?t:s;s=(U?n:m)|0;m=V|0;n=a+140|0;do{if((c[n>>2]|0)!=0){U=a+85804|0;if((c[U>>2]|0)==0){break}t=c[l>>2]|0;if((t|0)<=0){break}J=a+72|0;N=0;C=c[J>>2]|0;O=t;while(1){if((C|0)>0){t=r+(N<<2)|0;w=0;do{h[(c[U>>2]|0)+90904+(N<<3)>>3]=0.0;h[(c[U>>2]|0)+90920+(N<<3)>>3]=+g[t>>2];c[(c[U>>2]|0)+203288+(N<<3)+(w<<2)>>2]=c[a+304+(N*10504|0)+(w*5252|0)+4788>>2];h[(c[U>>2]|0)+189240+(N<<5)+(w<<3)>>3]=+g[V+(N<<3)+(w<<2)>>2];p=(c[U>>2]|0)+54040+(N*9216|0)+(w*4608|0)|0;v=a+304+(N*10504|0)+(w*5252|0)|0;NN(p|0,v|0,2304)|0;if((c[d>>2]|0)==2){v=w+2|0;p=c[U>>2]|0;h[p+197144+(N<<5)+(w<<3)>>3]=+h[p+197144+(N<<5)+(v<<3)>>3];p=c[U>>2]|0;u=p+123704+(N<<15)+(w<<13)|0;F=p+123704+(N<<15)+(v<<13)|0;NN(u|0,F|0,8192)|0}w=w+1|0;_=c[J>>2]|0;}while((w|0)<(_|0));$=_;aa=c[l>>2]|0}else{$=C;aa=O}w=N+1|0;if((w|0)<(aa|0)){N=w;C=$;O=aa}else{break}}}}while(0);aa=c[a+104>>2]|0;do{if((aa|0)==0|(aa|0)==3){$=a+39760|0;Z=+g[$>>2];g[a+39756>>2]=Z;_=a+39764|0;Y=+g[_>>2];g[$>>2]=Y;$=a+39768|0;X=+g[$>>2];g[_>>2]=X;_=a+39772|0;W=+g[_>>2];g[$>>2]=W;$=a+39776|0;R=+g[$>>2];g[_>>2]=R;_=a+39780|0;S=+g[_>>2];g[$>>2]=S;$=a+39784|0;Q=+g[$>>2];g[_>>2]=Q;_=a+39788|0;P=+g[_>>2];g[$>>2]=P;$=a+39792|0;M=+g[$>>2];g[_>>2]=M;_=a+39796|0;L=+g[_>>2];g[$>>2]=L;$=a+39800|0;T=+g[$>>2];g[_>>2]=T;_=a+39804|0;ab=+g[_>>2];g[$>>2]=ab;$=a+39808|0;ad=+g[$>>2];g[_>>2]=ad;_=a+39812|0;ae=+g[_>>2];g[$>>2]=ae;$=a+39816|0;af=+g[$>>2];g[_>>2]=af;_=a+39820|0;ag=+g[_>>2];g[$>>2]=ag;$=a+39824|0;ah=+g[$>>2];g[_>>2]=ah;_=a+39828|0;ai=+g[_>>2];g[$>>2]=ai;$=c[l>>2]|0;r=($|0)>0;O=c[a+72>>2]|0;L2955:do{if(r){if((O|0)>0){aj=0;ak=0.0}else{C=0;while(1){N=C+1|0;if((N|0)<($|0)){C=N}else{al=0.0;break L2955}}}while(1){C=0;am=ak;do{am=am+ +g[V+(aj<<3)+(C<<2)>>2];C=C+1|0;}while((C|0)<(O|0));C=aj+1|0;if((C|0)<($|0)){aj=C;ak=am}else{al=am;break}}}else{al=0.0}}while(0);g[_>>2]=al;an=+(ac($*3350|0,O)|0)/(L+(Z+al)*-.10394349694252014+(Y+ai)*-.18920649588108063+(X+ah)*-.21623599529266357+(W+ag)*-.1559150069952011+(R+af)*3.8980449615198e-17+(S+ae)*.23387250304222107+(Q+ad)*.5045499801635742+(P+ab)*.7568249702453613+(M+T)*.9354900121688843);if(r&(O|0)>0){ao=0}else{break}do{C=0;do{N=V+(ao<<3)+(C<<2)|0;g[N>>2]=an*+g[N>>2];C=C+1|0;}while((C|0)<(O|0));ao=ao+1|0;}while((ao|0)<($|0))}}while(0);dq[c[a+85812>>2]&127](a,m,q,s);EF(a)|0;q=EG(a,e,f,1)|0;if((c[a+156>>2]|0)!=0){Eu(a)}do{if((c[n>>2]|0)!=0){f=a+85804|0;if((c[f>>2]|0)==0){break}e=(c[l>>2]|0)*576|0;m=a+72|0;if((c[m>>2]|0)>0){ao=0;do{V=0;do{aj=c[f>>2]|0;h[aj+24+(ao*12800|0)+(V<<3)>>3]=+h[aj+24+(ao*12800|0)+(V+e<<3)>>3];V=V+1|0;}while((V|0)<272);V=c[o+(ao<<2)>>2]|0;aj=272;do{h[(c[f>>2]|0)+24+(ao*12800|0)+(aj<<3)>>3]=+g[V+(aj-272<<2)>>2];aj=aj+1|0;}while((aj|0)<1600);ao=ao+1|0;}while((ao|0)<(c[m>>2]|0))}g[a+84908>>2]=1.0;C2(a,s)}}while(0);s=a+84748|0;c[s>>2]=(c[s>>2]|0)+1;s=a+84744|0;o=a+84040+((c[s>>2]|0)*20|0)+16|0;c[o>>2]=(c[o>>2]|0)+1;o=a+84356|0;c[o>>2]=(c[o>>2]|0)+1;o=a+72|0;if((c[o>>2]|0)==2){n=a+84040+((c[s>>2]|0)*20|0)+(c[d>>2]<<2)|0;c[n>>2]=(c[n>>2]|0)+1;n=a+84340+(c[d>>2]<<2)|0;c[n>>2]=(c[n>>2]|0)+1}n=c[l>>2]|0;if((n|0)<=0){K=q;i=j;return K|0}d=a+84740|0;m=0;ao=c[o>>2]|0;f=n;while(1){if((ao|0)>0){n=0;do{e=(c[a+304+(m*10504|0)+(n*5252|0)+4792>>2]|0)==0?c[a+304+(m*10504|0)+(n*5252|0)+4788>>2]|0:4;aj=a+84360+((c[s>>2]|0)*24|0)+(e<<2)|0;c[aj>>2]=(c[aj>>2]|0)+1;aj=a+84360+((c[s>>2]|0)*24|0)+20|0;c[aj>>2]=(c[aj>>2]|0)+1;aj=a+84720+(e<<2)|0;c[aj>>2]=(c[aj>>2]|0)+1;c[d>>2]=(c[d>>2]|0)+1;n=n+1|0;ap=c[o>>2]|0;}while((n|0)<(ap|0));aq=ap;ar=c[l>>2]|0}else{aq=ao;ar=f}n=m+1|0;if((n|0)<(ar|0)){m=n;ao=aq;f=ar}else{K=q;break}}i=j;return K|0}function EL(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0.0,q=0.0,r=0.0,s=0.0,t=0.0,u=0.0,v=0;h=f+(e<<2)|0;e=a+85820|0;a=0;f=b;while(1){i=a+1|0;j=(ac(i,12582912)|0)>>16;k=31;l=b+(a<<10)+512|0;while(1){m=d[84880+(k<<2)|0]|0;n=m+j|0;o=c[h>>2]|0;p=+g[1992912+(m<<2)>>2]*+g[o+(n<<2)>>2];q=+g[1992912+(127-m<<2)>>2]*+g[o+(n+128<<2)>>2];r=p-q;s=p+q;q=+g[1992912+(m+64<<2)>>2]*+g[o+(n+64<<2)>>2];p=+g[1992912+(63-m<<2)>>2]*+g[o+(n+192<<2)>>2];t=q-p;u=q+p;o=l-16|0;g[o>>2]=s+u;g[l-8>>2]=s-u;g[l-12>>2]=r+t;g[l-4>>2]=r-t;v=c[h>>2]|0;t=+g[1992912+(m+1<<2)>>2]*+g[v+(n+1<<2)>>2];r=+g[1992912+(126-m<<2)>>2]*+g[v+(n+129<<2)>>2];u=t-r;s=t+r;r=+g[1992912+(m+65<<2)>>2]*+g[v+(n+65<<2)>>2];t=+g[1992912+(62-m<<2)>>2]*+g[v+(n+193<<2)>>2];p=r-t;q=r+t;g[l+496>>2]=s+q;g[l+504>>2]=s-q;g[l+500>>2]=u+p;g[l+508>>2]=u-p;if((k|0)>0){k=k-1|0;l=o}else{break}}c$[c[e>>2]&127](f|0,128);if((i|0)<3){a=i;f=f+1024|0}else{break}}return}function EM(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0.0,l=0,m=0.0,n=0.0,o=0.0,p=0.0,q=0.0,r=0;h=f+(e<<2)|0;e=b+2048|0;f=127;while(1){i=d[84880+f|0]|0;j=c[h>>2]|0;k=+g[1993424+(i<<2)>>2]*+g[j+(i<<2)>>2];l=i|512;m=+g[1993424+(l<<2)>>2]*+g[j+(l<<2)>>2];n=k-m;o=k+m;l=i|256;m=+g[1993424+(l<<2)>>2]*+g[j+(l<<2)>>2];l=i|768;k=+g[1993424+(l<<2)>>2]*+g[j+(l<<2)>>2];p=m-k;q=m+k;l=e-16|0;g[l>>2]=o+q;g[e-8>>2]=o-q;g[e-12>>2]=n+p;g[e-4>>2]=n-p;j=i+1|0;r=c[h>>2]|0;p=+g[1993424+(j<<2)>>2]*+g[r+(j<<2)>>2];j=i+513|0;n=+g[1993424+(j<<2)>>2]*+g[r+(j<<2)>>2];q=p-n;o=p+n;j=i+257|0;n=+g[1993424+(j<<2)>>2]*+g[r+(j<<2)>>2];j=i+769|0;p=+g[1993424+(j<<2)>>2]*+g[r+(j<<2)>>2];k=n-p;m=n+p;g[e+2032>>2]=o+m;g[e+2040>>2]=o-m;g[e+2036>>2]=q+k;g[e+2044>>2]=q-k;if((f|0)>0){e=l;f=f-1|0}else{break}}c$[c[a+85820>>2]&127](b,512);return}function EN(a){a=a|0;var b=0,d=0.0,e=0.0,f=0,h=0;b=0;while(1){d=+(b|0)+.5;e=.42- +U(+(d*6.283185307179586*.0009765625))*.5;g[1993424+(b<<2)>>2]=e+ +U(+(d*12.566370614359172*.0009765625))*.08;f=b+1|0;if((f|0)<1024){b=f}else{h=0;break}}do{g[1992912+(h<<2)>>2]=(1.0- +U(+((+(h|0)+.5)*6.283185307179586*.00390625)))*.5;h=h+1|0;}while((h|0)<128);c[a+85820>>2]=8;return}function EO(a,b){a=a|0;b=b|0;var c=0,d=0,e=0,f=0,h=0,i=0,j=0,k=0,l=0,m=0.0,n=0,o=0.0,p=0.0,q=0.0,r=0,s=0,t=0.0,u=0.0,v=0,w=0.0,x=0.0,y=0.0,z=0.0,A=0,B=0,C=0.0,D=0.0,E=0,F=0.0,G=0,H=0.0;c=b<<1;b=a+(c<<2)|0;d=263992;e=4;while(1){f=e>>1;h=e<<1;i=h+e|0;j=e<<2;k=a;l=a+(f<<2)|0;while(1){m=+g[k>>2];n=k+(e<<2)|0;o=+g[n>>2];p=m-o;q=m+o;r=k+(h<<2)|0;o=+g[r>>2];s=k+(i<<2)|0;m=+g[s>>2];t=o-m;u=o+m;g[r>>2]=q-u;g[k>>2]=q+u;g[s>>2]=p-t;g[n>>2]=p+t;t=+g[l>>2];n=l+(e<<2)|0;p=+g[n>>2];u=t-p;q=t+p;s=l+(i<<2)|0;p=+g[s>>2]*1.4142135623730951;r=l+(h<<2)|0;t=+g[r>>2]*1.4142135623730951;g[r>>2]=q-t;g[l>>2]=q+t;g[s>>2]=u-p;g[n>>2]=u+p;n=k+(j<<2)|0;if(n>>>0<b>>>0){k=n;l=l+(j<<2)|0}else{break}}l=d+4|0;if((e|0)>2){k=1;p=+g[d>>2];u=+g[l>>2];while(1){t=u*2.0;q=1.0-u*t;m=p*t;n=a+(k<<2)|0;s=a+(e-k<<2)|0;while(1){r=n+(e<<2)|0;t=+g[r>>2];v=s+(e<<2)|0;o=+g[v>>2];w=m*t-q*o;x=q*t+m*o;o=+g[n>>2];t=o-x;y=o+x;x=+g[s>>2];o=x-w;z=x+w;A=n+(i<<2)|0;w=+g[A>>2];B=s+(i<<2)|0;x=+g[B>>2];C=m*w-q*x;D=q*w+m*x;E=n+(h<<2)|0;x=+g[E>>2];w=x-D;F=x+D;G=s+(h<<2)|0;D=+g[G>>2];x=D-C;H=D+C;C=u*F-p*x;D=p*F+u*x;g[E>>2]=y-D;g[n>>2]=y+D;g[B>>2]=o-C;g[v>>2]=o+C;C=p*H-u*w;o=u*H+p*w;g[G>>2]=z-o;g[s>>2]=z+o;g[A>>2]=t-C;g[r>>2]=t+C;r=n+(j<<2)|0;if(r>>>0<b>>>0){n=r;s=s+(j<<2)|0}else{break}}m=+g[d>>2];q=+g[l>>2];C=u*m+p*q;s=k+1|0;if((s|0)<(f|0)){k=s;p=p*m-u*q;u=C}else{break}}}if((j|0)<(c|0)){d=d+8|0;e=j}else{break}}return}function EP(a,b){a=a|0;b=b|0;var d=0;NM(a|0,0,40);NM(a+84|0,0,40);NM(a+9732|0,0,40);NM(a+19380|0,0,40);NM(a+19464|0,0,40);NM(a+29112|0,0,40);if((b|0)==12e3){c[a+38784>>2]=6}else if((b|0)==32e3){c[a+38784>>2]=2}else if((b|0)==16e3){c[a+38784>>2]=5}else if((b|0)==24e3){c[a+38784>>2]=3}else if((b|0)==44100){c[a+38784>>2]=1}else if((b|0)==11025){c[a+38784>>2]=7}else if((b|0)==8e3){c[a+38784>>2]=8}else if((b|0)==22050){c[a+38784>>2]=4}else if((b|0)==48e3){c[a+38784>>2]=0}else{d=0;return d|0}c[a+38760>>2]=(b+19|0)/20|0;NM(a+38792|0,0,48e3);NM(a+38764|0,0,20);c[a+80>>2]=a+40;c[a+19460>>2]=a+19420;c[a+9728>>2]=a+124;c[a+29108>>2]=a+19504;c[a+19376>>2]=a+9772;c[a+38756>>2]=a+29152;NM(a+86792|0,0,48e3);d=1;return d|0}function EQ(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0.0,K=0.0,L=0.0,M=0.0,N=0.0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0.0,_=0.0,$=0.0,aa=0.0,ab=0.0,ac=0.0,ad=0.0,ae=0,af=0,ag=0;if((e|0)==0){i=1;return i|0}if((f|0)==2){j=d}else if((f|0)==1){j=b}else{i=0;return i|0}f=e>>>0<10;d=a+40|0;k=b;if(f){l=e<<2;NN(d|0,k|0,l)|0;m=a+19420|0;n=j;NN(m|0,n|0,l)|0}else{NN(d|0,k|0,40)|0;d=a+19420|0;l=j;NN(d|0,l|0,40)|0}l=a+38760|0;d=a+38764|0;n=a+80|0;m=a+19460|0;o=a+9728|0;p=a+38784|0;q=a+29108|0;r=a+19376|0;s=a+38756|0;t=a+38768|0;u=a+38776|0;v=a+9732|0;w=a+29112|0;x=a+84|0;y=a+19464|0;z=0;A=e;while(1){if((A|0)<=0){break}B=c[d>>2]|0;C=(c[l>>2]|0)-B|0;D=(A|0)>(C|0)?C:A;if((z|0)<10){C=10-z|0;E=(D|0)>(C|0)?C:D;F=c[m>>2]|0;G=c[n>>2]|0}else{E=D;F=j;G=b}ER(G+(z<<2)|0,(c[o>>2]|0)+(B<<2)|0,E,1991904+((c[p>>2]|0)*84|0)|0);ER(F+(z<<2)|0,(c[q>>2]|0)+(c[d>>2]<<2)|0,E,1991904+((c[p>>2]|0)*84|0)|0);B=c[d>>2]|0;D=c[p>>2]|0;C=(E|0)==0;do{if(C){H=B;I=2220}else{J=+g[1992664+(D*20|0)>>2];K=+g[1992668+(D*20|0)>>2];L=+g[1992672+(D*20|0)>>2];M=+g[1992676+(D*20|0)>>2];N=+g[1992680+(D*20|0)>>2];O=(c[r>>2]|0)+(B<<2)|0;P=E;Q=(c[o>>2]|0)+(B<<2)|0;while(1){R=P-1|0;g[O>>2]=+g[Q>>2]*J- +g[O-4>>2]*K+ +g[Q-4>>2]*L- +g[O-8>>2]*M+ +g[Q-8>>2]*N;if((R|0)==0){break}else{O=O+4|0;P=R;Q=Q+4|0}}Q=c[d>>2]|0;P=c[p>>2]|0;if(C){H=Q;I=2220;break}N=+g[1992664+(P*20|0)>>2];M=+g[1992668+(P*20|0)>>2];L=+g[1992672+(P*20|0)>>2];K=+g[1992676+(P*20|0)>>2];J=+g[1992680+(P*20|0)>>2];P=(c[s>>2]|0)+(Q<<2)|0;O=E;R=(c[q>>2]|0)+(Q<<2)|0;while(1){Q=O-1|0;g[P>>2]=+g[R>>2]*N- +g[P-4>>2]*M+ +g[R-4>>2]*L- +g[P-8>>2]*K+ +g[R-8>>2]*J;if((Q|0)==0){break}else{P=P+4|0;O=Q;R=R+4|0}}R=c[d>>2]|0;O=c[r>>2]|0;P=O+(R<<2)|0;Q=c[s>>2]|0;S=Q+(R<<2)|0;T=(E|0)%8|0;if((T|0)==0){U=P;V=S;W=R;break}X=R+T|0;Y=Q+(X<<2)|0;Q=P;P=S;S=T;J=+h[t>>3];K=+h[u>>3];while(1){T=S-1|0;L=+g[Q>>2];M=J+L*L;h[t>>3]=M;L=+g[P>>2];N=K+L*L;h[u>>3]=N;if((T|0)==0){break}else{Q=Q+4|0;P=P+4|0;S=T;J=M;K=N}}U=O+(X<<2)|0;V=Y;W=R}}while(0);if((I|0)==2220){I=0;U=(c[r>>2]|0)+(H<<2)|0;V=(c[s>>2]|0)+(H<<2)|0;W=H}if((E+7|0)>>>0>=15){C=U;B=V;D=(E|0)/8|0;K=+h[t>>3];J=+h[u>>3];while(1){S=D-1|0;N=+g[C>>2];M=+g[C+4>>2];L=+g[C+8>>2];Z=+g[C+12>>2];_=+g[C+16>>2];$=+g[C+20>>2];aa=+g[C+24>>2];ab=+g[C+28>>2];ac=K+(N*N+M*M+L*L+Z*Z+_*_+$*$+aa*aa+ab*ab);h[t>>3]=ac;ab=+g[B>>2];aa=+g[B+4>>2];$=+g[B+8>>2];_=+g[B+12>>2];Z=+g[B+16>>2];L=+g[B+20>>2];M=+g[B+24>>2];N=+g[B+28>>2];ad=J+(ab*ab+aa*aa+$*$+_*_+Z*Z+L*L+M*M+N*N);h[u>>3]=ad;if((S|0)==0){break}else{C=C+32|0;B=B+32|0;D=S;K=ac;J=ad}}}D=A-E|0;B=E+z|0;C=W+E|0;c[d>>2]=C;S=c[l>>2]|0;if((C|0)==(S|0)){J=+b7(+((+h[t>>3]+ +h[u>>3])/+(C|0)*.5+1.0e-37))*1.0e3;if(J>0.0){ae=~~J}else{ae=0}P=a+38792+((ae>>>0>11999?11999:ae)<<2)|0;c[P>>2]=(c[P>>2]|0)+1;NM(t|0,0,16);P=c[d>>2]|0;NP(v|0,a+9732+(P<<2)|0,40);NP(w|0,a+29112+(P<<2)|0,40);NP(x|0,a+84+(P<<2)|0,40);NP(y|0,a+19464+(P<<2)|0,40);c[d>>2]=0;af=0;ag=c[l>>2]|0}else{af=C;ag=S}if((af|0)>(ag|0)){i=0;I=2237;break}else{z=B;A=D}}if((I|0)==2237){return i|0}I=a;if(f){f=10-e|0;A=f<<2;NP(I|0,a+(e<<2)|0,A|0);NP(a+19380|0,a+19380+(e<<2)|0,A|0);A=a+(f<<2)|0;z=e<<2;NN(A|0,k|0,z)|0;k=a+19380+(f<<2)|0;f=j;NN(k|0,f|0,z)|0;i=1;return i|0}else{z=e-10|0;e=b+(z<<2)|0;NN(I|0,e|0,40)|0;e=a+19380|0;a=j+(z<<2)|0;NN(e|0,a|0,40)|0;i=1;return i|0}return 0}function ER(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;if((c|0)==0){return}e=d+4|0;f=d+8|0;h=d+12|0;i=d+16|0;j=d+20|0;k=d+24|0;l=d+28|0;m=d+32|0;n=d+36|0;o=d+40|0;p=d+44|0;q=d+48|0;r=d+52|0;s=d+56|0;t=d+60|0;u=d+64|0;v=d+68|0;w=d+72|0;x=d+76|0;y=d+80|0;z=b;b=c;c=a;while(1){a=b-1|0;g[z>>2]=+g[c>>2]*+g[d>>2]+1.0e-10- +g[z-4>>2]*+g[e>>2]+ +g[c-4>>2]*+g[f>>2]- +g[z-8>>2]*+g[h>>2]+ +g[c-8>>2]*+g[i>>2]- +g[z-12>>2]*+g[j>>2]+ +g[c-12>>2]*+g[k>>2]- +g[z-16>>2]*+g[l>>2]+ +g[c-16>>2]*+g[m>>2]- +g[z-20>>2]*+g[n>>2]+ +g[c-20>>2]*+g[o>>2]- +g[z-24>>2]*+g[p>>2]+ +g[c-24>>2]*+g[q>>2]- +g[z-28>>2]*+g[r>>2]+ +g[c-28>>2]*+g[s>>2]- +g[z-32>>2]*+g[t>>2]+ +g[c-32>>2]*+g[u>>2]- +g[z-36>>2]*+g[v>>2]+ +g[c-36>>2]*+g[w>>2]- +g[z-40>>2]*+g[x>>2]+ +g[c-40>>2]*+g[y>>2];if((a|0)==0){break}else{z=z+4|0;b=a;c=c+4|0}}return}function ES(a){a=a|0;var b=0,d=0,e=0.0,f=0,g=0,h=0;b=0;d=0;do{b=(c[a+38792+(d<<2)>>2]|0)+b|0;d=d+1|0;}while(d>>>0<12e3);if((b|0)==0){e=-24601.0}else{d=~~+ab(+(+(b>>>0>>>0)*.050000000000000044));b=12e3;f=0;while(1){g=b-1|0;if((b|0)==0){break}h=(c[a+38792+(g<<2)>>2]|0)+f|0;if(h>>>0<d>>>0){b=g;f=h}else{break}}e=64.81999969482422- +(g>>>0>>>0)/100.0}g=0;do{f=a+38792+(g<<2)|0;b=a+86792+(g<<2)|0;c[b>>2]=(c[b>>2]|0)+(c[f>>2]|0);c[f>>2]=0;g=g+1|0;}while(g>>>0<12e3);NM(a|0,0,40);NM(a+84|0,0,40);NM(a+9732|0,0,40);NM(a+19380|0,0,40);NM(a+19464|0,0,40);NM(a+29112|0,0,40);NM(a+38764|0,0,20);return+e}function ET(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0;h=i;i=i+8|0;j=h|0;if((b|0)==0){k=-255;i=h;return k|0}l=c[b+288>>2]|0;if((l|0)==0){k=-255;i=h;return k|0}b=l+85744|0;m=b;while(1){n=c[m>>2]|0;if((n|0)==0){o=0;break}if((c[n+4>>2]|0)==(d|0)){o=n;break}else{m=n|0}}L3134:do{if((d|0)==1415075928|(d|0)==1465407576|(d|0)==1129270605|(d|0)==1398361172|(d|0)==1095780675|(d|0)==1195724610|(d|0)==1346588244|(d|0)==1095061059|(d|0)==1279872587|(d|0)==1162756946|(d|0)==1196575044|(d|0)==1347570006){if((o|0)==0){p=2291;break}m=j|0;n=j+1|0;q=j+2|0;r=e+1|0;s=e+2|0;t=(f|0)==0;if((e|0)==0){u=o;L3138:while(1){a[m]=88;a[n]=88;a[q]=88;v=0;while(1){if((v|0)>=3){p=2267;break}w=(NS(a[u+8+v|0]|0)|0)&255;x=(NS(a[j+v|0]|0)|0)&255;if((w<<24>>24<32?32:w)<<24>>24==(x<<24>>24<32?32:x)<<24>>24){v=v+1|0}else{y=u;break}}do{if((p|0)==2267){p=0;v=c[u+16>>2]|0;if((c[u+20>>2]|0)==1){if((v|0)==0){z=0}else{y=u;break}}else{z=v}v=u+12|0;if(t){if((z|0)==0){A=u;p=2290;break L3134}else{y=u;break}}else{B=0}while(1){if(B>>>0>=z>>>0){A=u;p=2290;break L3134}if((a[(c[v>>2]|0)+B|0]|0)==(a[f+B|0]|0)){B=B+1|0}else{y=u;break}}}}while(0);while(1){v=c[y>>2]|0;if((v|0)==0){p=2291;break L3134}if((c[v+4>>2]|0)==(d|0)){u=v;continue L3138}else{y=v}}}}else{C=o}L3156:while(1){u=a[e]|0;if(u<<24>>24==0){a[m]=88;a[n]=88;a[q]=88;D=0}else{a[m]=u;a[n]=a[r]|0;a[q]=a[s]|0;D=0}while(1){if((D|0)>=3){p=2282;break}u=(NS(a[C+8+D|0]|0)|0)&255;v=(NS(a[j+D|0]|0)|0)&255;if((u<<24>>24<32?32:u)<<24>>24==(v<<24>>24<32?32:v)<<24>>24){D=D+1|0}else{E=C;break}}do{if((p|0)==2282){p=0;v=c[C+16>>2]|0;if((c[C+20>>2]|0)==1){if((v|0)==0){F=0}else{E=C;break}}else{F=v}v=C+12|0;if(t){if((F|0)==0){A=C;p=2290;break L3134}else{E=C;break}}else{G=0}while(1){if(G>>>0>=F>>>0){A=C;p=2290;break L3134}if((a[(c[v>>2]|0)+G|0]|0)==(a[f+G|0]|0)){G=G+1|0}else{E=C;break}}}}while(0);while(1){v=c[E>>2]|0;if((v|0)==0){p=2291;break L3134}if((c[v+4>>2]|0)==(d|0)){C=v;continue L3156}else{E=v}}}}else{A=o;p=2290}}while(0);if((p|0)==2290){if((A|0)==0){p=2291}else{H=A}}if((p|0)==2291){A=NE(1,36)|0;o=A;if((A|0)==0){k=-254;i=h;return k|0}A=l+85748|0;E=c[A>>2]|0;do{if((E|0)==0){p=2294}else{if((c[b>>2]|0)==0){p=2294;break}c[E>>2]=o}}while(0);if((p|0)==2294){c[b>>2]=o}c[A>>2]=o;H=o}c[H+4>>2]=d;d=H+8|0;L3191:do{if((e|0)==0){p=2299}else{o=a[e]|0;if(o<<24>>24==0){p=2299;break}a[d]=o;do{if((a[e]|0)==0){I=1}else{a[H+9|0]=a[e+1|0]|0;if((a[e]|0)==0){I=2;break}a[H+10|0]=a[e+2|0]|0;break L3191}}while(0);NM(H+8+I|0,32,I^3|0)}}while(0);if((p|0)==2299){a[d]=88;a[H+9|0]=88;a[H+10|0]=88}d=H+12|0;do{if((d|0)==0){J=0}else{ND(c[d>>2]|0);c[d>>2]=0;if((f|0)==0){J=0;break}else{K=0}while(1){L=K+1|0;if((a[f+K|0]|0)==0){break}else{K=L}}if((K|0)==0){J=0;break}p=NE(L,1)|0;c[d>>2]=p;if((p|0)==0){J=0;break}NN(p|0,f|0,K)|0;a[p+K|0]=0;J=K}}while(0);c[H+16>>2]=J;c[H+20>>2]=0;J=H+24|0;do{if((J|0)==0){M=0}else{ND(c[J>>2]|0);c[J>>2]=0;if((g|0)==0){M=0;break}else{N=0}while(1){O=N+1|0;if((a[g+N|0]|0)==0){break}else{N=O}}if((N|0)==0){M=0;break}K=NE(O,1)|0;c[J>>2]=K;if((K|0)==0){M=0;break}NN(K|0,g|0,N)|0;a[K+N|0]=0;M=N}}while(0);c[H+28>>2]=M;c[H+32>>2]=0;H=l+85696|0;c[H>>2]=c[H>>2]|3;k=0;i=h;return k|0}function EU(d,e,f){d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0.0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0;g=i;i=i+1024|0;h=g|0;if((d|0)==0){j=0;i=g;return j|0}k=d+288|0;l=c[k>>2]|0;if((l|0)==0){j=0;i=g;return j|0}m=l+85696|0;n=c[m>>2]|0;if((n&4|0)!=0){j=0;i=g;return j|0}o=c[l+85704>>2]|0;if((o|0)==0){p=0}else{p=NL(o|0)|0}o=c[l+85708>>2]|0;if((o|0)==0){q=0}else{q=NL(o|0)|0}o=c[l+85712>>2]|0;if((o|0)==0){r=0}else{r=NL(o|0)|0}o=c[l+85716>>2]|0;if((o|0)==0){s=0}else{s=NL(o|0)|0}do{if(!(p>>>0>30|q>>>0>30|r>>>0>30|s>>>0>30)){if((s>>>0<29|(c[l+85720>>2]|0)==0)&(n&10|0)==0){j=0}else{break}i=g;return j|0}}while(0);n=c[d+4>>2]|0;do{if((n|0)!=-1){s=h|0;t=+(n>>>0>>>0)*1.0e3/+(c[l+60>>2]|0);if(t>4294967295.0){bq(s|0,1619992,(u=i,i=i+8|0,c[u>>2]=-1,u)|0)|0;i=u}else{if(t<0.0){v=0}else{v=~~t}bq(s|0,1619992,(u=i,i=i+8|0,c[u>>2]=v,u)|0)|0;i=u}r=c[k>>2]|0;if((r|0)==0){break}q=r+85696|0;r=c[q>>2]|0;ET(d,1414284622,1703120,0,s)|0;c[q>>2]=r}}while(0);d=l+85728|0;do{if((c[d>>2]|0)==0){w=0;x=10}else{k=c[l+85732>>2]|0;if((k|0)==0){w=0;x=10;break}u=c[l+85740>>2]|0;if((u|0)==2){y=1646104}else if((u|0)==3){y=1632968}else if((u|0)==1){y=1662888}else{w=0;x=10;break}w=y;x=(NL(y|0)|0)+24+k|0}}while(0);y=l+85744|0;k=c[y>>2]|0;if((k|0)==0){z=x}else{u=x;x=k;while(1){k=c[x+4>>2]|0;do{if((k|0)==1129270605|(k|0)==1431520594){v=c[x+16>>2]|0;if((c[x+20>>2]|0)==1){A=(v<<1)+16|0}else{A=v+15|0}v=c[x+28>>2]|0;if((c[x+32>>2]|0)==1){B=(v<<1)+A|0;break}else{B=v+A|0;break}}else{v=k&-16777216;if(!((v|0)==1459617792|(v|0)==0)){v=c[x+16>>2]|0;n=(v|0)!=0;if((c[x+32>>2]|0)!=1){B=(c[x+28>>2]|0)+(n?v+12|0:11)|0;break}if(n){C=(v<<1)+13|0}else{C=11}B=(c[x+28>>2]<<1)+C|0;break}v=c[x+16>>2]|0;do{if((v|0)==0){D=10}else{if((c[x+20>>2]|0)==1){D=(v<<1)+13|0;break}else{D=v+12|0;break}}}while(0);v=c[x+28>>2]|0;if((v|0)==0){B=D;break}if((c[x+32>>2]|0)==1){B=D-1+v|0;break}else{B=v+D|0;break}}}while(0);k=B+u|0;v=c[x>>2]|0;if((v|0)==0){z=k;break}else{u=k;x=v}}}if((c[m>>2]&32|0)==0){E=z}else{E=(c[l+85736>>2]|0)+z|0}if(E>>>0>f>>>0){j=E;i=g;return j|0}if((e|0)==0){j=0;i=g;return j|0}a[e]=73;a[e+1|0]=68;a[e+2|0]=51;a[e+3|0]=3;a[e+4|0]=0;a[e+5|0]=0;f=E-10|0;a[e+6|0]=f>>>21&127;a[e+7|0]=f>>>14&127;a[e+8|0]=f>>>7&127;z=e+10|0;a[e+9|0]=f&127;f=c[y>>2]|0;if((f|0)==0){F=z}else{y=z;z=f;while(1){f=c[z+4>>2]|0;do{if((f|0)==1129270605|(f|0)==1431520594){m=z+20|0;x=z+16|0;u=c[x>>2]|0;if((c[m>>2]|0)==1){G=(u<<1)+16|0}else{G=u+15|0}u=z+32|0;B=z+28|0;D=(c[B>>2]<<((c[u>>2]|0)==1))+G|0;if(D>>>0<=10){H=y;break}a[y+3|0]=f&255;a[y+2|0]=f>>>8&255;a[y+1|0]=f>>>16&255;a[y]=f>>>24&255;C=D-10|0;a[y+7|0]=C&255;a[y+6|0]=C>>>8&255;a[y+5|0]=C>>>16&255;a[y+4|0]=C>>>24&255;a[y+8|0]=0;a[y+9|0]=0;C=z+24|0;a[y+10|0]=(c[u>>2]|0)==1|0;a[y+11|0]=a[z+8|0]|0;a[y+12|0]=a[z+9|0]|0;D=y+14|0;a[y+13|0]=a[z+10|0]|0;A=z+12|0;if((c[m>>2]|0)==1){m=c[A>>2]|0;v=c[x>>2]|0;if((v|0)==0){I=D}else{k=b[m>>1]|0;n=k<<16>>16==-2;h=v;r=m;m=D;q=k;while(1){k=h-1|0;s=r+2|0;if(n){p=cT(q|0)|0;J=(p&65535)>>>8&255;K=p&255}else{J=(q&65535)>>>8&255;K=q&255}a[m]=K;a[m+1|0]=J;if((k|0)==0){break}h=k;r=s;m=m+2|0;q=b[s>>1]|0}I=y+((v<<1)+14)|0}a[I]=0;a[I+1|0]=0;L=I+2|0}else{q=c[x>>2]|0;if((q|0)==0){M=D}else{m=D;r=c[A>>2]|0;h=q;while(1){n=h-1|0;a[m]=a[r]|0;if((n|0)==0){break}else{m=m+1|0;r=r+1|0;h=n}}M=y+(q+14)|0}a[M]=0;L=M+1|0}if((c[u>>2]|0)!=1){h=c[B>>2]|0;if((h|0)==0){H=L;break}r=L;m=c[C>>2]|0;A=h;while(1){D=A-1|0;a[r]=a[m]|0;if((D|0)==0){break}else{r=r+1|0;m=m+1|0;A=D}}H=L+h|0;break}A=c[C>>2]|0;m=c[B>>2]|0;if((m|0)==0){H=L;break}r=b[A>>1]|0;u=r<<16>>16==-2;q=m;D=A;A=L;x=r;while(1){r=q-1|0;v=D+2|0;if(u){n=cT(x|0)|0;N=(n&65535)>>>8&255;O=n&255}else{N=(x&65535)>>>8&255;O=x&255}a[A]=O;a[A+1|0]=N;if((r|0)==0){break}q=r;D=v;A=A+2|0;x=b[v>>1]|0}H=L+(m<<1)|0}else{x=f&-16777216;if(!((x|0)==1459617792|(x|0)==0)){x=z+32|0;A=z+16|0;D=c[A>>2]|0;q=(D|0)!=0;if((c[x>>2]|0)==1){if(q){P=(D<<1)+13|0}else{P=11}Q=(c[z+28>>2]<<1)+P|0}else{Q=(c[z+28>>2]|0)+(q?D+12|0:11)|0}if(Q>>>0<=10){H=y;break}a[y+3|0]=f&255;a[y+2|0]=f>>>8&255;a[y+1|0]=f>>>16&255;a[y]=f>>>24&255;D=Q-10|0;a[y+7|0]=D&255;a[y+6|0]=D>>>8&255;a[y+5|0]=D>>>16&255;a[y+4|0]=D>>>24&255;a[y+8|0]=0;a[y+9|0]=0;D=z+24|0;q=y+11|0;a[y+10|0]=(c[x>>2]|0)==1|0;u=c[A>>2]|0;do{if((u|0)==0){R=q}else{A=z+12|0;if((c[z+20>>2]|0)!=1){B=q;C=c[A>>2]|0;h=u;while(1){v=h-1|0;a[B]=a[C]|0;if((v|0)==0){break}else{B=B+1|0;C=C+1|0;h=v}}a[y+(u+11)|0]=0;R=y+(u+12)|0;break}h=c[A>>2]|0;C=b[h>>1]|0;B=C<<16>>16==-2;v=u;r=h;h=q;n=C;while(1){C=v-1|0;s=r+2|0;if(B){k=cT(n|0)|0;S=(k&65535)>>>8&255;T=k&255}else{S=(n&65535)>>>8&255;T=n&255}a[h]=T;a[h+1|0]=S;if((C|0)==0){break}v=C;r=s;h=h+2|0;n=b[s>>1]|0}n=u<<1;a[y+(n+11)|0]=0;a[y+(n+12)|0]=0;R=y+(n+13)|0}}while(0);if((c[x>>2]|0)!=1){u=c[z+28>>2]|0;if((u|0)==0){H=R;break}q=R;m=c[D>>2]|0;n=u;while(1){h=n-1|0;a[q]=a[m]|0;if((h|0)==0){break}else{q=q+1|0;m=m+1|0;n=h}}H=R+u|0;break}n=c[D>>2]|0;m=c[z+28>>2]|0;if((m|0)==0){H=R;break}q=b[n>>1]|0;x=q<<16>>16==-2;h=m;r=n;n=R;v=q;while(1){q=h-1|0;B=r+2|0;if(x){A=cT(v|0)|0;U=(A&65535)>>>8&255;V=A&255}else{U=(v&65535)>>>8&255;V=v&255}a[n]=V;a[n+1|0]=U;if((q|0)==0){break}h=q;r=B;n=n+2|0;v=b[B>>1]|0}H=R+(m<<1)|0;break}v=z+16|0;n=c[v>>2]|0;do{if((n|0)==0){W=10}else{if((c[z+20>>2]|0)==1){W=(n<<1)+13|0;break}else{W=n+12|0;break}}}while(0);n=z+28|0;m=c[n>>2]|0;do{if((m|0)==0){X=W}else{if((c[z+32>>2]|0)==1){X=W-1+m|0;break}else{X=m+W|0;break}}}while(0);if(X>>>0<=10){H=y;break}a[y+3|0]=f&255;a[y+2|0]=f>>>8&255;a[y+1|0]=f>>>16&255;a[y]=f>>>24&255;m=X-10|0;a[y+7|0]=m&255;a[y+6|0]=m>>>8&255;a[y+5|0]=m>>>16&255;a[y+4|0]=m>>>24&255;a[y+8|0]=0;m=y+10|0;a[y+9|0]=0;do{if((c[v>>2]|0)==0){Y=m}else{r=z+20|0;h=y+11|0;a[m]=(c[r>>2]|0)==1|0;x=z+12|0;if((c[r>>2]|0)!=1){r=c[v>>2]|0;if((r|0)==0){Z=h}else{D=h;u=c[x>>2]|0;B=r;while(1){q=B-1|0;a[D]=a[u]|0;if((q|0)==0){break}else{D=D+1|0;u=u+1|0;B=q}}Z=y+(r+11)|0}a[Z]=0;Y=Z+1|0;break}B=c[x>>2]|0;u=c[v>>2]|0;if((u|0)==0){_=h}else{D=b[B>>1]|0;q=D<<16>>16==-2;A=u;s=B;B=h;C=D;while(1){D=A-1|0;k=s+2|0;if(q){p=cT(C|0)|0;$=(p&65535)>>>8&255;aa=p&255}else{$=(C&65535)>>>8&255;aa=C&255}a[B]=aa;a[B+1|0]=$;if((D|0)==0){break}A=D;s=k;B=B+2|0;C=b[k>>1]|0}_=y+((u<<1)+11)|0}a[_]=0;a[_+1|0]=0;Y=_+2|0}}while(0);v=z+24|0;if((c[z+32>>2]|0)!=1){m=c[n>>2]|0;if((m|0)==0){H=Y;break}C=Y;B=c[v>>2]|0;s=m;while(1){A=s-1|0;a[C]=a[B]|0;if((A|0)==0){break}else{C=C+1|0;B=B+1|0;s=A}}H=Y+m|0;break}s=c[v>>2]|0;B=c[n>>2]|0;if((B|0)==0){H=Y;break}C=b[s>>1]|0;if((C<<16>>16|0)==(-2|0)|(C<<16>>16|0)==(-257|0)){A=B-1|0;if((A|0)==0){H=Y;break}else{ab=A;ac=s+2|0}}else{ab=B;ac=s}s=C<<16>>16==-2;C=ac;B=Y;A=ab;while(1){q=A-1|0;h=b[C>>1]|0;if(s){ad=cT(h|0)|0}else{ad=h}a[B]=(ad-32&65535)>223?32:ad&255;if((q|0)==0){break}C=C+2|0;B=B+1|0;A=q}H=Y+ab|0}}while(0);f=c[z>>2]|0;if((f|0)==0){F=H;break}else{y=H;z=f}}}do{if((w|0)==0){ae=F}else{z=c[d>>2]|0;H=c[l+85732>>2]|0;if((z|0)==0|(H|0)==0){ae=F;break}a[F+3|0]=67;a[F+2|0]=73;a[F+1|0]=80;a[F]=65;y=H+4+(NL(w|0)|0)|0;a[F+7|0]=y&255;a[F+6|0]=y>>>8&255;a[F+5|0]=y>>>16&255;a[F+4|0]=y>>>24&255;a[F+8|0]=0;a[F+9|0]=0;y=F+11|0;a[F+10|0]=0;ab=a[w]|0;if(ab<<24>>24==0){af=y}else{Y=y;y=w;ad=ab;while(1){ab=y+1|0;ac=Y+1|0;a[Y]=ad;_=a[ab]|0;if(_<<24>>24==0){af=ac;break}else{Y=ac;y=ab;ad=_}}}a[af]=0;a[af+1|0]=0;a[af+2|0]=0;ad=af+3|0;y=z;Y=H;while(1){_=Y-1|0;a[ad]=a[y]|0;if((_|0)==0){break}else{ad=ad+1|0;y=y+1|0;Y=_}}ae=af+(H+3)|0}}while(0);NM(ae|0,0,E+e-ae|0);j=E;i=g;return j|0}function EV(b){b=b|0;var d=0,e=0,f=0,g=0,h=0;d=c[b+288>>2]|0;if((c[d+85696>>2]&5|0)!=1){e=0;return e|0}f=EU(b,0,0)|0;g=NE(f,1)|0;if((g|0)==0){e=-1;return e|0}h=EU(b,g,f)|0;if(h>>>0>f>>>0){ND(g);e=-1;return e|0}if((h|0)!=0){f=0;do{EE(d,a[g+f|0]|0,1);f=f+1|0;}while(f>>>0<h>>>0)}ND(g);e=h;return e|0}function EW(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;f=i;i=i+8|0;g=f|0;if((b|0)==0){h=0;i=f;return h|0}if(e>>>0<128){h=128;i=f;return h|0}e=c[b+288>>2]|0;if((e|0)==0|(d|0)==0){h=0;i=f;return h|0}b=c[e+85696>>2]|0;if((b&9|0)!=1){h=0;i=f;return h|0}a[d]=84;a[d+1|0]=65;a[d+2|0]=71;j=b<<1&32;b=c[e+85704>>2]|0;k=d+3|0;d=30;L3490:while(1){l=(b|0)==0;m=k;n=d;while(1){o=n-1|0;if(!l){p=a[b]|0;if(p<<24>>24!=0){break}}q=m+1|0;a[m]=j;if((o|0)==0){r=q;break L3490}else{m=q;n=o}}n=m+1|0;a[m]=p;if((o|0)==0){r=n;break}else{b=b+1|0;k=n;d=o}}o=c[e+85708>>2]|0;d=r;r=30;L3499:while(1){k=(o|0)==0;b=d;p=r;while(1){s=p-1|0;if(!k){t=a[o]|0;if(t<<24>>24!=0){break}}n=b+1|0;a[b]=j;if((s|0)==0){u=n;break L3499}else{b=n;p=s}}p=b+1|0;a[b]=t;if((s|0)==0){u=p;break}else{o=o+1|0;d=p;r=s}}s=c[e+85712>>2]|0;r=u;u=30;L3508:while(1){d=(s|0)==0;o=r;t=u;while(1){v=t-1|0;if(!d){w=a[s]|0;if(w<<24>>24!=0){break}}p=o+1|0;a[o]=j;if((v|0)==0){x=p;break L3508}else{o=p;t=v}}t=o+1|0;a[o]=w;if((v|0)==0){x=t;break}else{s=s+1|0;r=t;u=v}}v=g|0;g=e+85700|0;bq(v|0,1626440,(u=i,i=i+8|0,c[u>>2]=c[g>>2],u)|0)|0;i=u;u=(c[g>>2]|0)!=0?v:0;v=x;x=4;L3517:while(1){g=(u|0)==0;r=v;s=x;while(1){y=s-1|0;if(!g){z=a[u]|0;if(z<<24>>24!=0){break}}w=r+1|0;a[r]=j;if((y|0)==0){A=w;break L3517}else{r=w;s=y}}s=r+1|0;a[r]=z;if((y|0)==0){A=s;break}else{u=u+1|0;v=s;x=y}}y=e+85720|0;x=c[e+85716>>2]|0;v=A;A=(c[y>>2]|0)!=0?28:30;L3526:while(1){u=(x|0)==0;z=v;s=A;while(1){B=s-1|0;if(!u){C=a[x]|0;if(C<<24>>24!=0){break}}g=z+1|0;a[z]=j;if((B|0)==0){D=g;break L3526}else{z=g;s=B}}s=z+1|0;a[z]=C;if((B|0)==0){D=s;break}else{x=x+1|0;v=s;A=B}}if((c[y>>2]|0)==0){E=D}else{a[D]=0;a[D+1|0]=c[y>>2]&255;E=D+2|0}a[E]=c[e+85724>>2]&255;h=128;i=f;return h|0}function EX(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0;d=i;i=i+128|0;e=d|0;f=c[b+288>>2]|0;g=EW(b,e|0,128)|0;if(g>>>0>128|(g|0)==0){h=0;i=d;return h|0}else{j=0}while(1){EE(f,a[e+j|0]|0,1);b=j+1|0;if(b>>>0<g>>>0){j=b}else{h=g;break}}i=d;return h|0}function EY(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0.0,O=0,P=0.0,Q=0,R=0,S=0,T=0,U=0.0,V=0.0,W=0.0,X=0.0,Y=0.0,Z=0.0,_=0.0,$=0.0,aa=0.0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0;e=i;i=i+72|0;f=e|0;h=a+72|0;if((c[h>>2]|0)<=0){i=e;return}j=a+76|0;k=f|0;l=f+68|0;m=f+36|0;n=f+60|0;o=f+44|0;p=f+56|0;q=f+48|0;r=f+32|0;s=f+4|0;t=f+28|0;u=f+8|0;v=f+24|0;w=f+12|0;x=f+20|0;y=f+16|0;z=f+64|0;A=f+40|0;B=f+52|0;C=b;b=0;while(1){do{if((c[j>>2]|0)>0){D=C+1144|0;E=0;while(1){F=1-E|0;G=a+27824+(b*4608|0)+(F*2304|0)|0;H=D;I=0;while(1){EZ(H,G);EZ(H+128|0,G+128|0);J=G+132|0;g[J>>2]=+g[J>>2]*-1.0;J=G+140|0;g[J>>2]=+g[J>>2]*-1.0;J=G+148|0;g[J>>2]=+g[J>>2]*-1.0;J=G+156|0;g[J>>2]=+g[J>>2]*-1.0;J=G+164|0;g[J>>2]=+g[J>>2]*-1.0;J=G+172|0;g[J>>2]=+g[J>>2]*-1.0;J=G+180|0;g[J>>2]=+g[J>>2]*-1.0;J=G+188|0;g[J>>2]=+g[J>>2]*-1.0;J=G+196|0;g[J>>2]=+g[J>>2]*-1.0;J=G+204|0;g[J>>2]=+g[J>>2]*-1.0;J=G+212|0;g[J>>2]=+g[J>>2]*-1.0;J=G+220|0;g[J>>2]=+g[J>>2]*-1.0;J=G+228|0;g[J>>2]=+g[J>>2]*-1.0;J=G+236|0;g[J>>2]=+g[J>>2]*-1.0;J=G+244|0;g[J>>2]=+g[J>>2]*-1.0;J=G+252|0;g[J>>2]=+g[J>>2]*-1.0;J=I+1|0;if((J|0)<9){G=G+256|0;H=H+256|0;I=J}else{break}}I=a+304+(E*10504|0)+(b*5252|0)+4788|0;H=a+304+(E*10504|0)+(b*5252|0)+4792|0;G=a+304+(E*10504|0)+(b*5252|0)|0;J=0;while(1){K=c[92624+(J<<2)>>2]|0;L=(c[H>>2]|0)!=0&(J|0)<2?0:c[I>>2]|0;M=a+37040+(J<<2)|0;N=+g[M>>2];do{if(N<1.0e-12){NM(G|0,0,72)}else{L3563:do{if(N<1.0){O=0;P=N;while(1){Q=a+27824+(b*4608|0)+(F*2304|0)+((O<<5)+K<<2)|0;g[Q>>2]=P*+g[Q>>2];Q=O+1|0;if((Q|0)>=18){break L3563}O=Q;P=+g[M>>2]}}}while(0);if((L|0)!=2){O=-9;do{Q=O+9|0;R=(Q<<5)+K|0;S=(8-O<<5)+K|0;P=+g[240+(L*144|0)+(O+27<<2)>>2]*+g[a+27824+(b*4608|0)+(F*2304|0)+(R<<2)>>2]+ +g[240+(L*144|0)+(O+36<<2)>>2]*+g[a+27824+(b*4608|0)+(F*2304|0)+(S<<2)>>2];T=O+18|0;U=+g[240+(L*144|0)+(Q<<2)>>2]*+g[a+27824+(b*4608|0)+(E*2304|0)+(R<<2)>>2]- +g[240+(L*144|0)+(T<<2)>>2]*+g[a+27824+(b*4608|0)+(E*2304|0)+(S<<2)>>2];V=+g[528+(O+12<<2)>>2];g[f+(Q<<2)>>2]=P-V*U;g[f+(T<<2)>>2]=U+P*V;O=O+1|0;}while((O|0)<0);V=+g[l>>2]- +g[m>>2];P=+g[n>>2]- +g[o>>2];U=+g[p>>2]- +g[q>>2];W=+g[k>>2]+ +g[r>>2];X=+g[s>>2]+ +g[t>>2];Y=+g[u>>2]+ +g[v>>2];Z=+g[w>>2]+ +g[x>>2];_=W+Y-Z;g[G+68>>2]=_-(X- +g[y>>2]);$=_*.5+(X- +g[y>>2]);_=(V-P-U)*.8660253882408142;g[G+20>>2]=_+$;g[G+24>>2]=_-$;$=(+g[z>>2]- +g[A>>2])*.8660253882408142;_=X*.5+ +g[y>>2];X=U*.3420201539993286+(P*.6427876353263855+(V*.9848077297210693+$));aa=Z*.9396926164627075+(W*.1736481785774231+_-Y*-.7660444378852844);g[G+4>>2]=X+aa;g[G+8>>2]=X-aa;aa=U*.9848077297210693+(V*.6427876353263855-$-P*.3420201539993286);X=Z*-.1736481785774231+(W*.7660444378852844+_-Y*.9396926164627075);g[G+36>>2]=aa+X;g[G+40>>2]=aa-X;X=P*.9848077297210693+(V*.3420201539993286-$)-U*.6427876353263855;U=Y*-.1736481785774231+(W*.9396926164627075-_)-Z*-.7660444378852844;g[G+52>>2]=X+U;g[G+56>>2]=X-U;U=+g[r>>2]- +g[k>>2];X=+g[v>>2]- +g[u>>2];Z=+g[x>>2]- +g[w>>2];_=+g[l>>2]+ +g[m>>2];W=+g[z>>2]+ +g[A>>2];Y=+g[n>>2]+ +g[o>>2];$=+g[p>>2]+ +g[q>>2];V=_+Y+$;g[G>>2]=V+(W+ +g[B>>2]);P=V*.5-(W+ +g[B>>2]);V=(U-X+Z)*.8660253882408142;g[G+44>>2]=V+P;g[G+48>>2]=P-V;V=(+g[t>>2]- +g[s>>2])*.8660253882408142;P=+g[B>>2]-W*.5;W=$*-.7660444378852844+(Y*-.1736481785774231+(_*.9396926164627075-P));aa=Z*.6427876353263855+(X*.9848077297210693+(U*.3420201539993286+V));g[G+12>>2]=W+aa;g[G+16>>2]=W-aa;aa=_*.7660444378852844+P-Y*.9396926164627075-$*-.1736481785774231;W=U*.6427876353263855+V-X*.3420201539993286-Z*.9848077297210693;g[G+28>>2]=aa+W;g[G+32>>2]=aa-W;W=_*.1736481785774231+P-Y*-.7660444378852844-$*.9396926164627075;$=X*.6427876353263855+(U*.9848077297210693-V)-Z*.3420201539993286;g[G+60>>2]=W+$;g[G+64>>2]=W-$;break}O=K+288|0;T=K+480|0;Q=-3;while(1){S=Q+3|0;$=+g[528+(S<<2)>>2];R=Q<<5;ab=O+R|0;ac=(8-Q<<5)+K|0;ad=Q*3|0;g[G+(ad+9<<2)>>2]=$*+g[a+27824+(b*4608|0)+(E*2304|0)+(ab<<2)>>2]- +g[a+27824+(b*4608|0)+(E*2304|0)+(ac<<2)>>2];ae=a+27824+(b*4608|0)+(E*2304|0)+((14-Q<<5)+K<<2)|0;af=a+27824+(b*4608|0)+(E*2304|0)+(T+R<<2)|0;g[G+(ad+18<<2)>>2]=$*+g[ae>>2]+ +g[af>>2];g[G+(ad+10<<2)>>2]=$*+g[af>>2]- +g[ae>>2];ae=a+27824+(b*4608|0)+(F*2304|0)+((2-Q<<5)+K<<2)|0;af=a+27824+(b*4608|0)+(F*2304|0)+((S<<5)+K<<2)|0;g[G+(ad+19<<2)>>2]=$*+g[ae>>2]+ +g[af>>2];g[G+(ad+11<<2)>>2]=$*+g[af>>2]- +g[ae>>2];g[G+(ad+20<<2)>>2]=$*+g[a+27824+(b*4608|0)+(F*2304|0)+(ac<<2)>>2]+ +g[a+27824+(b*4608|0)+(F*2304|0)+(ab<<2)>>2];ab=Q+1|0;if((ab|0)<0){Q=ab}else{ag=0;ah=G;break}}while(1){Q=ah+24|0;$=+g[Q>>2];T=ah+60|0;W=+g[T>>2];Z=$*.13165250420570374-W;V=+g[ah>>2];O=ah+36|0;U=+g[O>>2];X=V*.7673270106315613-U;Y=$+W*.13165250420570374;W=V+U*.7673270106315613;U=Y+W;ab=ah+12|0;V=+g[ab>>2];ac=ah+48|0;$=+g[ac>>2];P=Z+X;_=(V*.4142135679721832-$)*2.069978111953089e-11;g[ah>>2]=P*1.90752519173728e-11+_;aa=(V+$*.4142135679721832)*2.069978111953089e-11;g[T>>2]=(-0.0-U)*1.90752519173728e-11+aa;$=(Z-X)*.8660254037844387*1.907525191737281e-11;X=U*.5*1.907525191737281e-11+aa;g[ab>>2]=$-X;g[Q>>2]=$+X;X=P*.5*1.907525191737281e-11-_;_=(W-Y)*.8660254037844387*1.907525191737281e-11;g[O>>2]=_+X;g[ac>>2]=X-_;ac=ag+1|0;if((ac|0)<3){ag=ac;ah=ah+4|0}else{break}}}}while(0);L3578:do{if(!((L|0)==2|(J|0)==0)){K=7;while(1){M=G+(K<<2)|0;N=+g[M>>2];_=+g[528+(K+20<<2)>>2];ac=G+(~K<<2)|0;X=+g[ac>>2];Y=+g[528+(K+28<<2)>>2];g[ac>>2]=N*_+X*Y;g[M>>2]=N*Y-_*X;if((K|0)<=0){break L3578}K=K-1|0}}}while(0);L=J+1|0;if((L|0)<32){G=G+72|0;J=L}else{break}}J=E+1|0;ai=c[j>>2]|0;if((J|0)<(ai|0)){D=D+2304|0;E=J}else{break}}if((ai|0)!=1){break}E=a+27824+(b*4608|0)|0;D=a+27824+(b*4608|0)+2304|0;NN(E|0,D|0,2304)|0}}while(0);D=b+1|0;if((D|0)<(c[h>>2]|0)){C=d;b=D}else{break}}i=e;return}function EZ(a,b){a=a|0;b=b|0;var c=0,d=0,e=0,f=0,h=0.0,i=0.0,j=0.0,k=0.0,l=0.0,m=0.0,n=0.0,o=0.0,p=0.0,q=0.0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0.0,y=0.0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0.0,K=0,L=0.0,M=0,N=0.0,O=0.0,P=0.0,Q=0.0,R=0,S=0,T=0,U=0,V=0.0,W=0.0,X=0,Y=0,Z=0.0,_=0.0,$=0,aa=0,ab=0.0,ac=0.0,ad=0,ae=0,af=0.0,ag=0.0,ah=0,ai=0,aj=0.0,ak=0.0,al=0.0,am=0.0,an=0.0,ao=0.0,ap=0.0,aq=0.0,ar=0.0,as=0.0;c=a-248|0;d=259448;e=-15;f=a;while(1){h=+g[d-40>>2];i=+g[d-36>>2];j=+g[d-32>>2];k=+g[d-28>>2];l=+g[d-24>>2];m=+g[d-20>>2];n=+g[d-16>>2];o=+g[d-12>>2];p=+g[d-8>>2];q=+g[d-4>>2];r=+g[d>>2];s=+g[d+4>>2];t=+g[d+8>>2];u=+g[d+12>>2];v=+g[d+16>>2];w=+g[d+20>>2];x=h*+g[f+896>>2]+i*+g[f+640>>2]+j*+g[f+384>>2]+k*+g[f+128>>2]+l*+g[f-128>>2]+m*+g[f-384>>2]+n*+g[f-640>>2]+o*+g[f-896>>2]-p*+g[c+1024>>2]-q*+g[c+768>>2]-r*+g[c+512>>2]-s*+g[c+256>>2]-t*+g[c>>2]-u*+g[c-256>>2]-v*+g[c-512>>2]-w*+g[c-768>>2];y=(h*+g[c-896>>2]+i*+g[c-640>>2]+j*+g[c-384>>2]+k*+g[c-128>>2]+l*+g[c+128>>2]+m*+g[c+384>>2]+n*+g[c+640>>2]+o*+g[c+896>>2]+p*+g[f-1024>>2]+q*+g[f-768>>2]+r*+g[f-512>>2]+s*+g[f-256>>2]+t*+g[f>>2]+u*+g[f+256>>2]+v*+g[f+512>>2]+w*+g[f+768>>2])*+g[d+24>>2];z=e<<1;g[b+(z+30<<2)>>2]=x+y;g[b+(z+31<<2)>>2]=+g[d+28>>2]*(x-y);z=e+1|0;if((z|0)<0){c=c+4|0;d=d+72|0;e=z;f=f-4|0}else{break}}y=+g[a-124>>2]*10612.802734375+(+g[a-252>>2]- +g[a+4>>2])*5302.158203125+(+g[a-380>>2]+ +g[a+132>>2])*929.7763061523438+(+g[a-508>>2]- +g[a+260>>2])*728.8010864257812+(+g[a-636>>2]+ +g[a+388>>2])*288.09765625+(+g[a-764>>2]- +g[a+516>>2])*64.91738891601562+(+g[a-892>>2]+ +g[a+644>>2])*30.125003814697266+(+g[a-1020>>2]- +g[a+772>>2])*4.101456642150879;x=+g[a-188>>2]*12804.7978515625+ +g[a-444>>2]*1945.5516357421875+ +g[a-700>>2]*313.42449951171875+ +g[a-956>>2]*20.801593780517578- +g[a+68>>2]*1995.1556396484375- +g[a+324>>2]*9.000839233398438- +g[a+580>>2]*-29.202180862426758- +g[a+836>>2];w=x-y;v=x+y;a=b+56|0;y=+g[a>>2];f=b+60|0;x=+g[f>>2]-y;u=y+v;t=w+x;s=w-x;x=v-y;e=b+112|0;y=+g[e>>2];v=+g[b>>2];w=y+v;r=(y-v)*1.9615705013275146;d=b+116|0;v=+g[d>>2];c=b+4|0;y=+g[c>>2];q=v+y;p=(v-y)*1.9615705013275146;z=b+104|0;y=+g[z>>2];A=b+8|0;v=+g[A>>2];o=y+v;n=(y-v)*1.8477590084075928;B=b+108|0;v=+g[B>>2];C=b+12|0;y=+g[C>>2];m=v+y;l=(v-y)*1.8477590084075928;D=b+96|0;y=+g[D>>2];E=b+16|0;v=+g[E>>2];k=y+v;j=(y-v)*1.662939190864563;F=b+100|0;v=+g[F>>2];G=b+20|0;y=+g[G>>2];i=v+y;h=(v-y)*1.662939190864563;H=b+88|0;y=+g[H>>2];I=b+24|0;v=+g[I>>2];J=y+v;K=b+92|0;L=+g[K>>2];M=b+28|0;N=+g[M>>2];O=L+N;P=O-J;Q=(y-v)*1.4142135623730951-P;v=(L-N)*1.4142135623730951-O-Q;O=u-J;N=J+u;u=t-P;J=P+t;t=s-Q;P=Q+s;s=x-v;Q=v+x;R=b+80|0;x=+g[R>>2];S=b+32|0;v=+g[S>>2];L=x+v;y=(x-v)*1.111140489578247;T=b+84|0;v=+g[T>>2];U=b+36|0;x=+g[U>>2];V=v+x;W=(v-x)*1.111140489578247;X=b+72|0;x=+g[X>>2];Y=b+40|0;v=+g[Y>>2];Z=x+v;_=(x-v)*.7653668522834778;$=b+76|0;v=+g[$>>2];aa=b+44|0;x=+g[aa>>2];ab=v+x;ac=(v-x)*.7653668522834778;ad=b+64|0;x=+g[ad>>2];ae=b+48|0;v=+g[ae>>2];af=x+v;ag=(x-v)*.39018064737319946;ah=b+68|0;v=+g[ah>>2];ai=b+52|0;x=+g[ai>>2];aj=v+x;ak=(v-x)*.39018064737319946;x=y+j;v=(j-y)*.7653668522834778;y=W+h;j=(h-W)*.7653668522834778;W=k+L;h=(k-L)*.7653668522834778;L=i+V;k=(i-V)*.7653668522834778;V=w+af;i=(w-af)*1.8477590084075928;af=q+aj;w=(q-aj)*1.8477590084075928;aj=ag+r;q=(ag-r)*1.8477590084075928;r=ak+p;ag=(p-ak)*1.8477590084075928;ak=o+Z;p=m+ab;al=_+n;am=ac+l;an=am-p;ao=p-ak;p=N-ak;ap=ak+N;N=(m-ab)*1.4142135623730951-an;ab=al-ao;m=J-ao;ak=ao+J;J=an-ab;an=P-ab;ao=ab+P;P=(o-Z)*1.4142135623730951-J;Z=Q-J;o=J+Q;Q=N-P;J=s-P;ab=P+s;s=(n-_)*1.4142135623730951-al-Q;al=t-Q;_=Q+t;t=(l-ac)*1.4142135623730951-am-N-s;N=u-s;am=s+u;u=O-t;s=t+O;O=V+W;t=af+L;ac=aj+x;l=r+y;Q=h+i;n=k+w;P=j+ag;aq=v-q;ar=aq-ac;as=(aj-x)*1.4142135623730951-ar;x=P-l;aj=(r-y)*1.4142135623730951-x;y=l-t;l=n-y;r=x-l;x=(af-L)*1.4142135623730951-r;L=aj-x;af=(k-w)*-1.4142135623730951-n-L;n=t-O;t=ac-n;ac=y-t;y=Q-ac;w=l-y;l=ar-w;ar=r-l;r=(V-W)*1.4142135623730951-ar;W=x-r;x=as-W;V=L-x;L=(h-i)*-1.4142135623730951-Q-V;Q=af-L;i=(v+q)*-1.4142135623730951-aq-as-Q;as=(j-ag)*-1.4142135623730951-P-aj-af-i;g[b>>2]=O+ap;g[b+124>>2]=ap-O;g[c>>2]=n+ak;g[b+120>>2]=ak-n;g[ad>>2]=t+ao;g[f>>2]=ao-t;g[ah>>2]=ac+o;g[a>>2]=o-ac;g[S>>2]=y+ab;g[K>>2]=ab-y;g[U>>2]=w+_;g[H>>2]=_-w;g[D>>2]=l+am;g[M>>2]=am-l;g[F>>2]=ar+s;g[I>>2]=s-ar;g[E>>2]=r+u;g[B>>2]=u-r;g[G>>2]=W+N;g[z>>2]=N-W;g[R>>2]=x+al;g[aa>>2]=al-x;g[T>>2]=V+J;g[Y>>2]=J-V;g[ae>>2]=L+Z;g[$>>2]=Z-L;g[ai>>2]=Q+an;g[X>>2]=an-Q;g[e>>2]=i+m;g[C>>2]=m-i;g[d>>2]=as+p;g[A>>2]=p-as;return}function E_(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=c[a+104>>2]|0;NM(a+88|0,0,16);G5(c[d+12>>2]|0);G5(c[d+16>>2]|0);G5(c[d+20>>2]|0);G5(c[d+24>>2]|0);G5(c[d+28>>2]|0);G5(c[d+32>>2]|0);G5(c[d+36>>2]|0);G5(c[d+40>>2]|0);G5(c[d+44>>2]|0);G5(c[d+48>>2]|0);G5(c[d+52>>2]|0);G5(c[d+56>>2]|0);G5(c[d+60>>2]|0);G5(c[d+64>>2]|0);G5(c[d+68>>2]|0);d=c1[c[(c[452850]|0)+12>>2]&255](a)|0;if((d|0)!=0){e=d;return e|0}if((b|0)==0){e=0;return e|0}if((E1(a)|0)!=0){e=-131;return e|0}d=a+4|0;c[b>>2]=Hb(d)|0;c[b+4>>2]=Ha(d)|0;c[b+8>>2]=0;c[b+12>>2]=c[a+44>>2];d=a+48|0;f=c[d+4>>2]|0;g=b+16|0;c[g>>2]=c[d>>2];c[g+4>>2]=f;f=a+56|0;a=c[f+4>>2]|0;g=b+24|0;c[g>>2]=c[f>>2];c[g+4>>2]=a;e=0;return e|0}function E$(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,i=0.0,j=0.0;d=c[a+28>>2]|0;e=d+3360|0;NM(b|0,0,48);if((e|0)==0){return}f=d+3372|0;if((c[f>>2]|0)<=0){return}g=c[a+8>>2]|0;a=c[d>>2]|0;c[b+24>>2]=(c[d+4>>2]|0)/(a|0)|0;c[b>>2]=1;i=+(a>>1|0);j=+(g|0);c[b+12>>2]=~~+bR(+(i*+(c[e>>2]|0)/j));c[b+16>>2]=~~+bR(+(i*+(c[d+3364>>2]|0)/j));c[b+20>>2]=~~+bR(+(i*+(c[d+3368>>2]|0)/j));h[b+32>>3]=7.0;e=~~(+(c[f>>2]|0)*+h[d+3376>>3]);c[b+8>>2]=e;c[b+4>>2]=e;return}function E0(a){a=a|0;NM(a|0,0,48);return}function E1(a){a=a|0;var b=0,d=0;b=(c[(c[a+64>>2]|0)+104>>2]|0)+80|0;do{if((b|0)!=0){if((c[b>>2]|0)==0){break}else{d=1}return d|0}}while(0);d=0;return d|0}function E2(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0.0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0.0,K=0.0,L=0.0,M=0.0,N=0.0,O=0.0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0;b=c[a+104>>2]|0;d=c[a+64>>2]|0;e=c[d+104>>2]|0;f=c[d+4>>2]|0;d=c[f+28>>2]|0;g=e+112|0;i=~~+bR(+(+h[g>>3]));j=b+12|0;b=j;k=(Ha(c[b+(i<<2)>>2]|0)|0)<<3;l=a+28|0;m=c[l>>2]|0;n=e+96|0;o=c[n>>2]|0;if((m|0)==0){p=e+100|0;q=c[p>>2]|0;r=o;s=p;t=0}else{p=c[e+104>>2]|0;u=ac(p,o)|0;v=e+100|0;q=ac(p,c[v>>2]|0)|0;r=u;s=v;t=m}m=c[d+(t<<2)>>2]>>1;t=d+3372|0;v=~~(+(c[t>>2]|0)*+h[d+3376>>3]);u=e+120|0;if((c[e+80>>2]|0)==0){if((c[u>>2]|0)!=0){w=-1;return w|0}c[u>>2]=a;w=0;return w|0}c[u>>2]=a;a=e+92|0;u=c[a>>2]|0;if((u|0)>0){if((c[l>>2]|0)==0){x=u}else{x=ac(c[e+104>>2]|0,u)|0}y=15.0/+h[d+3384>>3];d=e+84|0;u=c[d>>2]|0;p=u+(k-x)|0;L3635:do{if((p|0)>(v|0)){if((i|0)>0&(k|0)>(x|0)){z=k;A=i;B=u}else{C=i;break}while(1){if((z-x+B|0)<=(v|0)){C=A;break L3635}D=A-1|0;E=(Ha(c[b+(D<<2)>>2]|0)|0)<<3;if(!((D|0)>0&(E|0)>(x|0))){C=D;break L3635}z=E;A=D;B=c[d>>2]|0}}else{if((p|0)>=(v|0)){C=i;break}D=i+1|0;if((D|0)<15&(k|0)<(x|0)){F=k;G=i;H=D;I=u}else{C=i;break}while(1){if((F-x+I|0)>=(v|0)){C=G;break L3635}D=(Ha(c[b+(H<<2)>>2]|0)|0)<<3;E=H+1|0;if(!((E|0)<15&(D|0)<(x|0))){C=H;break L3635}F=D;G=H;H=E;I=c[d>>2]|0}}}while(0);J=+h[g>>3];K=+(m|0);L=+bR(+(+(C|0)-J))/K;M=+(c[f+8>>2]|0);N=L*M;L=-0.0-y;O=N<L?L:N;N=J+K*((O>y?y:O)/M);h[g>>3]=N;g=~~+bR(+N);f=(Ha(c[b+(g<<2)>>2]|0)|0)<<3;P=g;Q=f;R=c[n>>2]|0}else{P=i;Q=k;R=o}L3649:do{if((R|0)>0&(Q|0)<(r|0)){o=e+88|0;if((Q-r+(c[o>>2]|0)|0)<0){S=Q;T=P}else{U=P;V=Q;break}while(1){k=T+1|0;if((k|0)>14){U=k;V=S;break L3649}i=(Ha(c[b+(k<<2)>>2]|0)|0)<<3;if((i-r+(c[o>>2]|0)|0)<0){S=i;T=k}else{U=k;V=i;break}}}else{U=P;V=Q}}while(0);L3655:do{if((c[s>>2]|0)>0&(V|0)>(q|0)){Q=e+88|0;P=c[Q>>2]|0;T=c[t>>2]|0;if((V-q+P|0)>(T|0)){W=V;X=U;Y=T;Z=P}else{_=U;$=V;aa=2636;break}while(1){P=X-1|0;if((P|0)<0){ab=W;ad=Y;ae=Z;aa=2638;break L3655}T=(Ha(c[b+(P<<2)>>2]|0)|0)<<3;S=c[Q>>2]|0;R=c[t>>2]|0;if((T-q+S|0)>(R|0)){W=T;X=P;Y=R;Z=S}else{_=P;$=T;aa=2636;break}}}else{_=U;$=V;aa=2636}}while(0);do{if((aa|0)==2636){if((_|0)<0){ab=$;ad=c[t>>2]|0;ae=c[e+88>>2]|0;aa=2638;break}V=(r+7-(c[e+88>>2]|0)|0)/8|0;U=(_|0)>14?14:_;c[e+124>>2]=U;Z=b+(U<<2)|0;U=V-(Ha(c[Z>>2]|0)|0)|0;V=c[Z>>2]|0;if((U|0)>0){Y=U;U=V;while(1){X=Y-1|0;G3(U,0,8);W=c[Z>>2]|0;if((X|0)>0){Y=X;U=W}else{af=W;break}}}else{af=V}ag=(Ha(af)|0)<<3}}while(0);do{if((aa|0)==2638){af=(ad+q-ae|0)/8|0;c[e+124>>2]=0;b=j;if((Ha(c[b>>2]|0)|0)<=(af|0)){ag=ab;break}G2(c[b>>2]|0,af<<3);ag=(Ha(c[b>>2]|0)|0)<<3}}while(0);if((c[n>>2]|0)>0){aa=2645}else{if((c[s>>2]|0)>0){aa=2645}}do{if((aa|0)==2645){s=(q|0)>0;if(s&(ag|0)>(q|0)){n=e+88|0;c[n>>2]=ag-q+(c[n>>2]|0);break}n=(r|0)>0;if(n&(ag|0)<(r|0)){ab=e+88|0;c[ab>>2]=ag-r+(c[ab>>2]|0);break}ab=e+88|0;j=c[ab>>2]|0;if((j|0)>(v|0)){if(!s){c[ab>>2]=v;break}s=j+(ag-q)|0;c[ab>>2]=s;if((s|0)>=(v|0)){break}c[ab>>2]=v;break}else{if(!n){c[ab>>2]=v;break}n=j+(ag-r)|0;c[ab>>2]=n;if((n|0)<=(v|0)){break}c[ab>>2]=v;break}}}while(0);v=c[a>>2]|0;if((v|0)<=0){w=0;return w|0}if((c[l>>2]|0)==0){ah=v}else{ah=ac(c[e+104>>2]|0,v)|0}v=e+84|0;c[v>>2]=ag-ah+(c[v>>2]|0);w=0;return w|0}function E3(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0;d=c[a+104>>2]|0;a=d+120|0;e=c[a>>2]|0;if((e|0)==0){f=0;return f|0}if((b|0)!=0){g=c[e+104>>2]|0;h=(c[(c[e+64>>2]|0)+104>>2]|0)+80|0;do{if((h|0)==0){i=7}else{if((c[h>>2]|0)==0){i=7;break}i=c[d+124>>2]|0}}while(0);d=g+12+(i<<2)|0;c[b>>2]=Hb(c[d>>2]|0)|0;c[b+4>>2]=Ha(c[d>>2]|0)|0;c[b+8>>2]=0;c[b+12>>2]=c[e+44>>2];d=e+48|0;i=c[d+4>>2]|0;g=b+16|0;c[g>>2]=c[d>>2];c[g+4>>2]=i;i=e+56|0;e=c[i+4>>2]|0;g=b+24|0;c[g>>2]=c[i>>2];c[g+4>>2]=e}c[a>>2]=0;f=1;return f|0}function E4(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;NM(b|0,0,112);c[b+64>>2]=a;c[b+76>>2]=0;c[b+68>>2]=0;if((c[a>>2]|0)==0){return 0}a=NE(1,72)|0;c[b+104>>2]=a;g[a+4>>2]=-9999.0;d=b+4|0;b=a+12|0;e=0;while(1){if((e|0)==7){c[a+40>>2]=d;G1(d);e=e+1|0;continue}else{f=NE(1,20)|0;c[b+(e<<2)>>2]=f;G1(f);f=e+1|0;if((f|0)<15){e=f;continue}else{break}}}return 0}function E5(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0;d=b+7&-8;b=a+72|0;e=c[b>>2]|0;f=a+76|0;g=a+68|0;h=c[g>>2]|0;if((e+d|0)<=(c[f>>2]|0)){i=e;j=h;k=j+i|0;l=i+d|0;c[b>>2]=l;return k|0}if((h|0)!=0){m=NC(8)|0;n=a+80|0;c[n>>2]=(c[n>>2]|0)+e;e=a+84|0;c[m+4>>2]=c[e>>2];c[m>>2]=h;c[e>>2]=m}c[f>>2]=d;f=NC(d)|0;c[g>>2]=f;c[b>>2]=0;i=0;j=f;k=j+i|0;l=i+d|0;c[b>>2]=l;return k|0}function E6(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=a+84|0;d=c[b>>2]|0;if((d|0)!=0){e=d;while(1){d=c[e+4>>2]|0;ND(c[e>>2]|0);ND(e);if((d|0)==0){break}else{e=d}}}e=a+80|0;d=c[e>>2]|0;if((d|0)==0){f=a+72|0;c[f>>2]=0;c[b>>2]=0;return}g=a+68|0;h=a+76|0;c[g>>2]=NF(c[g>>2]|0,(c[h>>2]|0)+d|0)|0;c[h>>2]=(c[h>>2]|0)+(c[e>>2]|0);c[e>>2]=0;f=a+72|0;c[f>>2]=0;c[b>>2]=0;return}function E7(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;b=c[a+104>>2]|0;d=a+84|0;e=c[d>>2]|0;if((e|0)!=0){f=e;while(1){e=c[f+4>>2]|0;ND(c[f>>2]|0);ND(f);if((e|0)==0){break}else{f=e}}}f=a+80|0;e=c[f>>2]|0;g=a+68|0;h=c[g>>2]|0;if((e|0)==0){i=h}else{j=a+76|0;k=NF(h,(c[j>>2]|0)+e|0)|0;c[g>>2]=k;c[j>>2]=(c[j>>2]|0)+(c[f>>2]|0);c[f>>2]=0;i=k}c[a+72>>2]=0;c[d>>2]=0;if((i|0)!=0){ND(i)}if((b|0)==0){l=a;NM(l|0,0,112);return 0}i=b+12|0;d=0;while(1){k=i+(d<<2)|0;G4(c[k>>2]|0);if((d|0)==7){d=d+1|0;continue}else{ND(c[k>>2]|0);k=d+1|0;if((k|0)<15){d=k;continue}else{break}}}ND(b);l=a;NM(l|0,0,112);return 0}function E8(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;if((E9(a,b,1)|0)!=0){d=1;return d|0}e=c[a+104>>2]|0;c[e+60>>2]=FF(b)|0;f=NE(1,180)|0;c[e>>2]=f;Fj(f,b);E$(b,e+80|0);e=a+64|0;c[e>>2]=3;c[e+4>>2]=0;d=0;return d|0}function E9(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;e=c[b+28>>2]|0;if((e|0)==0){f=1;return f|0}g=c[e+3656>>2]|0;NM(a|0,0,112);h=NE(1,136)|0;c[a+104>>2]=h;c[a+4>>2]=b;i=c[e+8>>2]|0;j=(i|0)==0?0:i-1|0;if((j|0)==0){k=0}else{i=j;j=0;while(1){l=j+1|0;m=i>>>1;if((m|0)==0){k=l;break}else{i=m;j=l}}}c[h+44>>2]=k;k=NE(1,4)|0;c[h+12>>2]=k;j=NE(1,4)|0;i=h+16|0;c[i>>2]=j;l=NE(1,20)|0;c[k>>2]=l;c[j>>2]=NE(1,20)|0;j=e;k=e;Fz(l,c[k>>2]>>g);l=e+4|0;Fz(c[c[i>>2]>>2]|0,c[l>>2]>>g);g=c[k>>2]|0;i=(g|0)==0?0:g-1|0;if((i|0)==0){n=-6}else{g=i;i=0;while(1){m=g>>>1;if((m|0)==0){break}else{g=m;i=i+1|0}}n=i-5|0}c[h+4>>2]=n;n=c[l>>2]|0;i=(n|0)==0?0:n-1|0;if((i|0)==0){o=-6}else{n=i;i=0;while(1){g=n>>>1;if((g|0)==0){break}else{n=g;i=i+1|0}}o=i-5|0}c[h+8>>2]=o;L3785:do{if((d|0)==0){o=e+2848|0;if((c[o>>2]|0)!=0){break}i=e+24|0;c[o>>2]=NE(c[i>>2]|0,56)|0;n=c[i>>2]|0;if((n|0)<=0){break}g=e+1824|0;m=0;p=n;while(1){n=g+(m<<2)|0;q=c[n>>2]|0;if((q|0)==0){r=p;break}if((Gd((c[o>>2]|0)+(m*56|0)|0,q)|0)!=0){s=2745;break}Ga(c[n>>2]|0);c[n>>2]=0;m=m+1|0;p=c[i>>2]|0;if((m|0)>=(p|0)){break L3785}}if((s|0)==2745){r=c[i>>2]|0}if((r|0)>0){p=0;m=r;while(1){o=g+(p<<2)|0;n=c[o>>2]|0;if((n|0)==0){t=m}else{Ga(n);c[o>>2]=0;t=c[i>>2]|0}o=p+1|0;if((o|0)<(t|0)){p=o;m=t}else{break}}}Fa(a);f=-1;return f|0}else{Gg(h+20|0,c[k>>2]|0);Gg(h+32|0,c[l>>2]|0);m=e+2848|0;L3787:do{if((c[m>>2]|0)==0){p=e+24|0;i=NE(c[p>>2]|0,56)|0;c[m>>2]=i;if((c[p>>2]|0)<=0){break}g=e+1824|0;o=0;n=i;while(1){Gc(n+(o*56|0)|0,c[g+(o<<2)>>2]|0)|0;i=o+1|0;if((i|0)>=(c[p>>2]|0)){break L3787}o=i;n=c[m>>2]|0}}}while(0);m=e+28|0;n=NE(c[m>>2]|0,52)|0;o=h+56|0;c[o>>2]=n;L3794:do{if((c[m>>2]|0)>0){p=e+2852|0;g=e+2868|0;i=b+8|0;q=0;u=n;while(1){v=c[p+(q<<2)>>2]|0;FI(u+(q*52|0)|0,v,g,(c[j+(c[v>>2]<<2)>>2]|0)/2|0,c[i>>2]|0);v=q+1|0;if((v|0)>=(c[m>>2]|0)){break L3794}q=v;u=c[o>>2]|0}}}while(0);c[a>>2]=1}}while(0);j=c[l>>2]|0;k=a+16|0;c[k>>2]=j;t=b+4|0;b=NC(c[t>>2]<<2)|0;r=a+8|0;c[r>>2]=b;c[a+12>>2]=NC(c[t>>2]<<2)|0;L3819:do{if((c[t>>2]|0)>0){s=0;d=j;o=b;while(1){c[o+(s<<2)>>2]=NE(d,4)|0;m=s+1|0;if((m|0)>=(c[t>>2]|0)){break L3819}s=m;d=c[k>>2]|0;o=c[r>>2]|0}}}while(0);c[a+36>>2]=0;c[a+40>>2]=0;r=(c[l>>2]|0)/2|0;c[a+48>>2]=r;c[a+20>>2]=r;r=e+16|0;l=h+48|0;c[l>>2]=NE(c[r>>2]|0,4)|0;k=e+20|0;t=h+52|0;c[t>>2]=NE(c[k>>2]|0,4)|0;if((c[r>>2]|0)>0){h=e+800|0;b=e+1056|0;j=0;do{o=dg[c[(c[1900584+(c[h+(j<<2)>>2]<<2)>>2]|0)+8>>2]&255](a,c[b+(j<<2)>>2]|0)|0;c[(c[l>>2]|0)+(j<<2)>>2]=o;j=j+1|0;}while((j|0)<(c[r>>2]|0))}if((c[k>>2]|0)<=0){f=0;return f|0}r=e+1312|0;j=e+1568|0;e=0;while(1){l=dg[c[(c[1745208+(c[r+(e<<2)>>2]<<2)>>2]|0)+8>>2]&255](a,c[j+(e<<2)>>2]|0)|0;c[(c[t>>2]|0)+(e<<2)>>2]=l;l=e+1|0;if((l|0)<(c[k>>2]|0)){e=l}else{f=0;break}}return f|0}function Fa(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;if((a|0)==0){return}b=c[a+4>>2]|0;d=(b|0)!=0;if(d){e=c[b+28>>2]|0}else{e=0}f=c[a+104>>2]|0;g=(f|0)!=0;if(g){h=f;i=c[h>>2]|0;if((i|0)!=0){Fk(i);ND(c[h>>2]|0)}h=f+12|0;i=c[h>>2]|0;if((i|0)!=0){FA(c[i>>2]|0);ND(c[c[h>>2]>>2]|0);ND(c[h>>2]|0)}h=f+16|0;i=c[h>>2]|0;if((i|0)!=0){FA(c[i>>2]|0);ND(c[c[h>>2]>>2]|0);ND(c[h>>2]|0)}h=f+48|0;i=c[h>>2]|0;if((i|0)!=0){do{if((e|0)==0){j=i}else{k=e+16|0;if((c[k>>2]|0)>0){l=0;m=i}else{j=i;break}while(1){c_[c[(c[1900584+(c[e+800+(l<<2)>>2]<<2)>>2]|0)+16>>2]&127](c[m+(l<<2)>>2]|0);n=l+1|0;o=c[h>>2]|0;if((n|0)<(c[k>>2]|0)){l=n;m=o}else{j=o;break}}}}while(0);ND(j)}j=f+52|0;m=c[j>>2]|0;if((m|0)!=0){do{if((e|0)==0){p=m}else{l=e+20|0;if((c[l>>2]|0)>0){q=0;r=m}else{p=m;break}while(1){c_[c[(c[1745208+(c[e+1312+(q<<2)>>2]<<2)>>2]|0)+16>>2]&127](c[r+(q<<2)>>2]|0);h=q+1|0;i=c[j>>2]|0;if((h|0)<(c[l>>2]|0)){q=h;r=i}else{p=i;break}}}}while(0);ND(p)}p=f+56|0;r=c[p>>2]|0;if((r|0)!=0){do{if((e|0)==0){s=r}else{q=e+28|0;if((c[q>>2]|0)>0){t=0;u=r}else{s=r;break}while(1){FK(u+(t*52|0)|0);j=t+1|0;m=c[p>>2]|0;if((j|0)<(c[q>>2]|0)){t=j;u=m}else{s=m;break}}}}while(0);ND(s)}s=c[f+60>>2]|0;if((s|0)!=0){FG(s)}E0(f+80|0);Gh(f+20|0);Gh(f+32|0)}s=a+8|0;u=c[s>>2]|0;do{if((u|0)!=0){do{if(d){t=b+4|0;p=c[t>>2]|0;if((p|0)>0){v=0;w=p;x=u}else{y=u;break}while(1){p=c[x+(v<<2)>>2]|0;if((p|0)==0){z=w}else{ND(p);z=c[t>>2]|0}p=v+1|0;r=c[s>>2]|0;if((p|0)<(z|0)){v=p;w=z;x=r}else{y=r;break}}}else{y=u}}while(0);ND(y);t=c[a+12>>2]|0;if((t|0)==0){break}ND(t)}}while(0);if(g){g=c[f+64>>2]|0;if((g|0)!=0){ND(g)}g=c[f+68>>2]|0;if((g|0)!=0){ND(g)}g=c[f+72>>2]|0;if((g|0)!=0){ND(g)}ND(f)}NM(a|0,0,112);return}function Fb(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;d=c[a+4>>2]|0;e=c[a+104>>2]|0;f=e+64|0;g=c[f>>2]|0;if((g|0)!=0){ND(g)}c[f>>2]=0;f=e+68|0;g=c[f>>2]|0;if((g|0)!=0){ND(g)}c[f>>2]=0;f=e+72|0;e=c[f>>2]|0;if((e|0)!=0){ND(e)}c[f>>2]=0;f=a+20|0;e=c[f>>2]|0;g=a+16|0;L3916:do{if((e+b|0)<(c[g>>2]|0)){h=c[d+4>>2]|0;i=2823}else{j=e+(b<<1)|0;c[g>>2]=j;k=d+4|0;if((c[k>>2]|0)<=0){break}l=a+8|0;m=0;n=j;while(1){j=NF(c[(c[l>>2]|0)+(m<<2)>>2]|0,n<<2)|0;c[(c[l>>2]|0)+(m<<2)>>2]=j;j=m+1|0;o=c[k>>2]|0;if((j|0)>=(o|0)){h=o;i=2823;break L3916}m=j;n=c[g>>2]|0}}}while(0);do{if((i|0)==2823){g=d+4|0;if((h|0)<=0){break}b=a+8|0;e=a+12|0;n=0;while(1){c[(c[e>>2]|0)+(n<<2)>>2]=(c[(c[b>>2]|0)+(n<<2)>>2]|0)+(c[f>>2]<<2);m=n+1|0;if((m|0)<(c[g>>2]|0)){n=m}else{p=e;break}}q=c[p>>2]|0;return q|0}}while(0);p=a+12|0;q=c[p>>2]|0;return q|0}function Fc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;d=i;e=c[a+4>>2]|0;f=c[e+28>>2]|0;if((b|0)>=1){g=a+20|0;h=(c[g>>2]|0)+b|0;if((h|0)>(c[a+16>>2]|0)){j=-131;i=d;return j|0}c[g>>2]=h;if((c[a+28>>2]|0)!=0){j=0;i=d;return j|0}if((h-(c[a+48>>2]|0)|0)<=(c[f+4>>2]|0)){j=0;i=d;return j|0}Fd(a);j=0;i=d;return j|0}h=i;i=i+128|0;g=h|0;if((c[a+28>>2]|0)==0){Fd(a)}h=f+4|0;Fb(a,(c[h>>2]|0)*3|0)|0;f=a+20|0;b=c[f>>2]|0;k=a+32|0;c[k>>2]=b;c[f>>2]=b+((c[h>>2]|0)*3|0);l=e+4|0;if((c[l>>2]|0)<=0){j=0;i=d;return j|0}e=a+8|0;a=0;m=b;while(1){if((m|0)>64){b=c[h>>2]|0;n=(m|0)>(b|0)?b:m;b=(c[(c[e>>2]|0)+(a<<2)>>2]|0)+(m-n<<2)|0;+Fx(b,g,n,32);n=c[(c[e>>2]|0)+(a<<2)>>2]|0;b=c[k>>2]|0;Fy(g,n+(b-32<<2)|0,32,n+(b<<2)|0,(c[f>>2]|0)-b|0)}else{NM((c[(c[e>>2]|0)+(a<<2)>>2]|0)+(m<<2)|0,0,(c[f>>2]|0)-m<<2|0)}b=a+1|0;if((b|0)>=(c[l>>2]|0)){j=0;break}a=b;m=c[k>>2]|0}i=d;return j|0}function Fd(a){a=a|0;var b=0,d=0,e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;b=i;i=i+64|0;d=b|0;e=a+20|0;f=c[e>>2]|0;h=i;i=i+(f<<2)|0;i=i+7&-8;j=h;c[a+28>>2]=1;h=a+48|0;if((f-(c[h>>2]|0)|0)<=32){i=b;return}k=a+4|0;if((c[(c[k>>2]|0)+4>>2]|0)<=0){i=b;return}l=a+8|0;a=0;m=f;while(1){if((m|0)>0){f=c[l>>2]|0;n=0;do{g[j+(n<<2)>>2]=+g[(c[f+(a<<2)>>2]|0)+(m+~n<<2)>>2];n=n+1|0;}while((n|0)<(m|0))}+Fx(j,d,m-(c[h>>2]|0)|0,16);n=c[h>>2]|0;f=(c[e>>2]|0)-n|0;Fy(d,j+(f-16<<2)|0,16,j+(f<<2)|0,n);n=c[e>>2]|0;if((n|0)>0){f=0;o=n;while(1){g[(c[(c[l>>2]|0)+(a<<2)>>2]|0)+(o+~f<<2)>>2]=+g[j+(f<<2)>>2];p=f+1|0;q=c[e>>2]|0;if((p|0)<(q|0)){f=p;o=q}else{r=q;break}}}else{r=n}o=a+1|0;if((o|0)<(c[(c[k>>2]|0)+4>>2]|0)){a=o;m=r}else{break}}i=b;return}function Fe(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0.0,B=0.0,C=0.0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0;d=c[a+4>>2]|0;e=c[d+28>>2]|0;f=c[a+104>>2]|0;h=c[f+60>>2]|0;i=a+48|0;j=a+40|0;k=e;l=(c[i>>2]|0)-((c[k+(c[j>>2]<<2)>>2]|0)/2|0)|0;m=c[b+104>>2]|0;if((c[a+28>>2]|0)==0){n=0;return n|0}o=a+32|0;if((c[o>>2]|0)==-1){n=0;return n|0}p=Fl(a)|0;do{if((p|0)==-1){if((c[o>>2]|0)==0){n=0;return n|0}else{c[a+44>>2]=0;q=0;break}}else{r=a+44|0;if((c[e>>2]|0)==(c[e+4>>2]|0)){c[r>>2]=0;q=0;break}else{c[r>>2]=p;q=p;break}}}while(0);p=a+44|0;r=c[k+(q<<2)>>2]|0;q=((c[k+(c[j>>2]<<2)>>2]|0)/4|0)+(c[i>>2]|0)+((r|0)/4|0)|0;s=a+20|0;if((c[s>>2]|0)<(q+((r|0)/2|0)|0)){n=0;return n|0}r=b+84|0;t=c[r>>2]|0;if((t|0)!=0){u=t;while(1){t=c[u+4>>2]|0;ND(c[u>>2]|0);ND(u);if((t|0)==0){break}else{u=t}}}u=b+80|0;t=c[u>>2]|0;if((t|0)!=0){v=b+68|0;w=b+76|0;c[v>>2]=NF(c[v>>2]|0,(c[w>>2]|0)+t|0)|0;c[w>>2]=(c[w>>2]|0)+(c[u>>2]|0);c[u>>2]=0}w=b+72|0;c[w>>2]=0;c[r>>2]=0;t=a+36|0;c[b+24>>2]=c[t>>2];c[b+28>>2]=c[j>>2];c[b+32>>2]=c[p>>2];L4005:do{if((c[j>>2]|0)==0){v=m+8|0;if((Fn(a)|0)==0){c[v>>2]=1;break}else{c[v>>2]=0;break}}else{do{if((c[t>>2]|0)!=0){if((c[p>>2]|0)==0){break}c[m+8>>2]=1;break L4005}}while(0);c[m+8>>2]=0}}while(0);c[b+64>>2]=a;v=a+64|0;x=c[v>>2]|0;y=c[v+4>>2]|0;z=NU(x,y,1,0)|0;c[v>>2]=z;c[v+4>>2]=G;v=b+56|0;c[v>>2]=x;c[v+4>>2]=y;y=a+56|0;v=c[y+4>>2]|0;x=b+48|0;c[x>>2]=c[y>>2];c[x+4>>2]=v;v=b+36|0;c[v>>2]=c[k+(c[j>>2]<<2)>>2];k=m+4|0;A=+g[k>>2];x=h|0;B=+g[x>>2];if(A>B){g[x>>2]=A;C=A}else{C=B}B=+FP(C,a);g[x>>2]=B;g[k>>2]=B;k=d+4|0;d=(c[k>>2]<<2)+7&-8;x=c[w>>2]|0;h=b+76|0;z=c[h>>2]|0;D=b+68|0;E=c[D>>2]|0;if((d+x|0)>(z|0)){if((E|0)!=0){F=NC(8)|0;c[u>>2]=(c[u>>2]|0)+x;c[F+4>>2]=c[r>>2];c[F>>2]=E;c[r>>2]=F}c[h>>2]=d;F=NC(d)|0;c[D>>2]=F;c[w>>2]=0;H=0;I=F;J=d}else{H=x;I=E;J=z}z=H+d|0;c[w>>2]=z;d=b|0;c[d>>2]=I+H;H=(c[k>>2]<<2)+7&-8;if((H+z|0)>(J|0)){if((I|0)!=0){J=NC(8)|0;c[u>>2]=(c[u>>2]|0)+z;c[J+4>>2]=c[r>>2];c[J>>2]=I;c[r>>2]=J}c[h>>2]=H;J=NC(H)|0;c[D>>2]=J;c[w>>2]=0;K=0;L=J}else{K=z;L=I}c[w>>2]=K+H;H=m;c[H>>2]=L+K;if((c[k>>2]|0)>0){K=a+8|0;L=0;do{m=((c[v>>2]|0)+l<<2)+7&-8;I=c[w>>2]|0;z=c[D>>2]|0;if((m+I|0)>(c[h>>2]|0)){if((z|0)!=0){J=NC(8)|0;c[u>>2]=(c[u>>2]|0)+I;c[J+4>>2]=c[r>>2];c[J>>2]=z;c[r>>2]=J}c[h>>2]=m;J=NC(m)|0;c[D>>2]=J;c[w>>2]=0;M=0;N=J}else{M=I;N=z}c[w>>2]=M+m;c[(c[H>>2]|0)+(L<<2)>>2]=N+M;m=c[(c[H>>2]|0)+(L<<2)>>2]|0;z=c[(c[K>>2]|0)+(L<<2)>>2]|0;I=(c[v>>2]|0)+l<<2;NN(m|0,z|0,I)|0;c[(c[d>>2]|0)+(L<<2)>>2]=(c[(c[H>>2]|0)+(L<<2)>>2]|0)+(l<<2);L=L+1|0;}while((L|0)<(c[k>>2]|0))}L=c[o>>2]|0;do{if((L|0)!=0){if((c[i>>2]|0)<(L|0)){break}c[o>>2]=-1;c[b+44>>2]=1;n=1;return n|0}}while(0);b=(c[e+4>>2]|0)/2|0;e=q-b|0;if((e|0)<=0){n=1;return n|0}Fo(c[f>>2]|0,e);f=(c[s>>2]|0)-e|0;c[s>>2]=f;L4050:do{if((c[k>>2]|0)>0){q=a+8|0;L=0;l=f;while(1){H=c[(c[q>>2]|0)+(L<<2)>>2]|0;NP(H|0,H+(e<<2)|0,l<<2|0);H=L+1|0;if((H|0)>=(c[k>>2]|0)){break L4050}L=H;l=c[s>>2]|0}}}while(0);c[t>>2]=c[j>>2];c[j>>2]=c[p>>2];c[i>>2]=b;i=c[o>>2]|0;if((i|0)==0){p=NU(c[y>>2]|0,c[y+4>>2]|0,e,(e|0)<0|0?-1:0)|0;c[y>>2]=p;c[y+4>>2]=G;n=1;return n|0}p=i-e|0;i=(p|0)<1?-1:p;c[o>>2]=i;if((b|0)<(i|0)){o=NU(c[y>>2]|0,c[y+4>>2]|0,e,(e|0)<0|0?-1:0)|0;c[y>>2]=o;c[y+4>>2]=G;n=1;return n|0}else{o=i+e-b|0;b=NU(c[y>>2]|0,c[y+4>>2]|0,o,(o|0)<0|0?-1:0)|0;c[y>>2]=b;c[y+4>>2]=G;n=1;return n|0}return 0}function Ff(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;if((E9(a,b,0)|0)!=0){Fa(a);d=1;return d|0}b=c[a+4>>2]|0;e=c[a+104>>2]|0;if((e|0)==0|(b|0)==0){d=0;return d|0}f=c[b+28>>2]|0;if((f|0)==0){d=0;return d|0}b=c[f+3656>>2]|0;g=c[f+4>>2]>>b+1;c[a+48>>2]=g;c[a+20>>2]=g>>b;c[a+24>>2]=-1;NM(a+56|0,-1|0,16);c[a+32>>2]=0;a=e+128|0;c[a>>2]=-1;c[a+4>>2]=-1;d=0;return d|0}function Fg(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0;d=c[a+4>>2]|0;e=c[d+28>>2]|0;f=c[a+104>>2]|0;h=c[e+3656>>2]|0;if((b|0)==0){i=-131;return i|0}j=a+20|0;k=a+24|0;l=c[k>>2]|0;if(!((c[j>>2]|0)<=(l|0)|(l|0)==-1)){i=-131;return i|0}l=a+40|0;m=a+36|0;c[m>>2]=c[l>>2];c[l>>2]=c[b+28>>2];c[a+44>>2]=-1;n=a+64|0;o=c[n>>2]|0;p=c[n+4>>2]|0;if((o|0)==(-1|0)&(p|0)==(-1|0)){q=b+56|0;r=2942}else{s=NU(o,p,1,0)|0;p=G;o=b+56|0;if((s|0)==(c[o>>2]|0)&(p|0)==(c[o+4>>2]|0)){t=p;u=s}else{q=o;r=2942}}if((r|0)==2942){r=a+56|0;c[r>>2]=-1;c[r+4>>2]=-1;r=f+128|0;c[r>>2]=-1;c[r+4>>2]=-1;t=c[q+4>>2]|0;u=c[q>>2]|0}c[n>>2]=u;c[n+4>>2]=t;t=b|0;do{if((c[t>>2]|0)!=0){n=c[l>>2]|0;u=e;q=h+1|0;r=c[u+(n<<2)>>2]>>q;o=c[e>>2]>>q;s=c[e+4>>2]>>q;q=c[b+88>>2]|0;p=a+72|0;v=NU(c[p>>2]|0,c[p+4>>2]|0,q,(q|0)<0|0?-1:0)|0;c[p>>2]=v;c[p+4>>2]=G;p=c[b+92>>2]|0;v=a+80|0;q=NU(c[v>>2]|0,c[v+4>>2]|0,p,(p|0)<0|0?-1:0)|0;c[v>>2]=q;c[v+4>>2]=G;v=c[b+96>>2]|0;q=a+88|0;p=NU(c[q>>2]|0,c[q+4>>2]|0,v,(v|0)<0|0?-1:0)|0;c[q>>2]=p;c[q+4>>2]=G;q=c[b+100>>2]|0;p=a+96|0;v=NU(c[p>>2]|0,c[p+4>>2]|0,q,(q|0)<0|0?-1:0)|0;c[p>>2]=v;c[p+4>>2]=G;p=a+48|0;v=c[p>>2]|0;q=(v|0)==0;w=q?s:0;x=q?0:s;q=d+4|0;if((c[q>>2]|0)>0){y=f+4|0;z=a+8|0;A=(s|0)/2|0;B=(o|0)/2|0;C=A-B|0;D=(o|0)>0;E=B+A|0;B=o-1|0;F=(r|0)>0;H=f+8|0;I=(s|0)>0;J=s-1|0;K=A+w+((o|0)/-2|0)|0;A=0;L=n;while(1){n=(L|0)!=0;L4096:do{if((c[m>>2]|0)==0){M=Gm((c[y>>2]|0)-h|0)|0;N=c[(c[z>>2]|0)+(A<<2)>>2]|0;O=c[(c[t>>2]|0)+(A<<2)>>2]|0;if(!n){if(D){P=0}else{break}while(1){Q=N+(P+w<<2)|0;g[Q>>2]=+g[Q>>2]*+g[M+(B-P<<2)>>2]+ +g[O+(P<<2)>>2]*+g[M+(P<<2)>>2];P=P+1|0;if((P|0)>=(o|0)){break L4096}}}if(D){Q=0;while(1){R=N+(Q+w<<2)|0;g[R>>2]=+g[R>>2]*+g[M+(B-Q<<2)>>2]+ +g[O+(Q+C<<2)>>2]*+g[M+(Q<<2)>>2];R=Q+1|0;if((R|0)<(o|0)){Q=R}else{S=o;break}}}else{S=0}if((S|0)<(E|0)){T=S}else{break}do{g[N+(T+w<<2)>>2]=+g[O+(T+C<<2)>>2];T=T+1|0;}while((T|0)<(E|0))}else{if(n){O=Gm((c[H>>2]|0)-h|0)|0;N=c[(c[z>>2]|0)+(A<<2)>>2]|0;Q=c[(c[t>>2]|0)+(A<<2)>>2]|0;if(I){U=0}else{break}do{M=N+(U+w<<2)|0;g[M>>2]=+g[M>>2]*+g[O+(J-U<<2)>>2]+ +g[Q+(U<<2)>>2]*+g[O+(U<<2)>>2];U=U+1|0;}while((U|0)<(s|0))}else{O=Gm((c[y>>2]|0)-h|0)|0;Q=c[(c[z>>2]|0)+(A<<2)>>2]|0;N=c[(c[t>>2]|0)+(A<<2)>>2]|0;if(D){V=0}else{break}do{M=Q+(K+V<<2)|0;g[M>>2]=+g[M>>2]*+g[O+(B-V<<2)>>2]+ +g[N+(V<<2)>>2]*+g[O+(V<<2)>>2];V=V+1|0;}while((V|0)<(o|0))}}}while(0);n=c[(c[z>>2]|0)+(A<<2)>>2]|0;O=c[(c[t>>2]|0)+(A<<2)>>2]|0;if(F){N=0;do{g[n+(N+x<<2)>>2]=+g[O+(N+r<<2)>>2];N=N+1|0;}while((N|0)<(r|0))}N=A+1|0;if((N|0)>=(c[q>>2]|0)){break}A=N;L=c[l>>2]|0}W=c[p>>2]|0}else{W=v}c[p>>2]=(W|0)==0?s:0;if((c[k>>2]|0)==-1){c[k>>2]=x;c[j>>2]=x;break}else{c[k>>2]=w;c[j>>2]=(((c[u+(c[l>>2]<<2)>>2]|0)/4|0)+((c[u+(c[m>>2]<<2)>>2]|0)/4|0)>>h)+w;break}}}while(0);W=f+128|0;f=c[W>>2]|0;t=c[W+4>>2]|0;if((f|0)==(-1|0)&(t|0)==(-1|0)){X=0;Y=0}else{V=e;U=((c[V+(c[l>>2]<<2)>>2]|0)/4|0)+((c[V+(c[m>>2]<<2)>>2]|0)/4|0)|0;V=NU(U,(U|0)<0|0?-1:0,f,t)|0;X=G;Y=V}c[W>>2]=Y;c[W+4>>2]=X;X=a+56|0;Y=c[X>>2]|0;V=c[X+4>>2]|0;do{if((Y|0)==(-1|0)&(V|0)==(-1|0)){t=b+48|0;f=c[t>>2]|0;U=c[t+4>>2]|0;if((f|0)==(-1|0)&(U|0)==(-1|0)){break}c[X>>2]=f;c[X+4>>2]=U;T=c[W>>2]|0;S=c[W+4>>2]|0;if(!((S|0)>(U|0)|(S|0)==(U|0)&T>>>0>f>>>0)){break}f=NV(T,S,c[t>>2]|0,c[t+4>>2]|0)|0;t=f;f=(t|0)<0?0:t;if((c[b+44>>2]|0)!=0){t=c[j>>2]|0;S=t-(c[k>>2]|0)<<h;c[j>>2]=t-(((f|0)>(S|0)?S:f)>>h);break}S=(c[k>>2]|0)+(f>>h)|0;c[k>>2]=S;f=c[j>>2]|0;if((S|0)<=(f|0)){break}c[k>>2]=f}else{f=e;S=((c[f+(c[l>>2]<<2)>>2]|0)/4|0)+((c[f+(c[m>>2]<<2)>>2]|0)/4|0)|0;f=NU(S,(S|0)<0|0?-1:0,Y,V)|0;S=G;c[X>>2]=f;c[X+4>>2]=S;t=b+48|0;T=c[t>>2]|0;U=c[t+4>>2]|0;if((T|0)==(-1|0)&(U|0)==(-1|0)|(f|0)==(T|0)&(S|0)==(U|0)){break}do{if((S|0)>(U|0)|(S|0)==(U|0)&f>>>0>T>>>0){P=NV(f,S,T,U)|0;d=P;if((d|0)==0){Z=U;_=T;break}if((c[b+44>>2]|0)==0){Z=U;_=T;break}P=c[j>>2]|0;L=P-(c[k>>2]|0)<<h;A=(d|0)>(L|0)?L:d;c[j>>2]=P-(((A|0)<0?0:A)>>h);Z=c[t+4>>2]|0;_=c[t>>2]|0}else{Z=U;_=T}}while(0);c[X>>2]=_;c[X+4>>2]=Z}}while(0);if((c[b+44>>2]|0)==0){i=0;return i|0}c[a+32>>2]=1;i=0;return i|0}function Fh(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;d=a+24|0;e=c[d>>2]|0;if((e|0)<=-1){f=0;return f|0}g=a+20|0;h=c[g>>2]|0;if((e|0)>=(h|0)){f=0;return f|0}if((b|0)==0){i=h;j=e}else{h=(c[a+4>>2]|0)+4|0;L4161:do{if((c[h>>2]|0)>0){k=a+8|0;l=a+12|0;m=0;n=e;while(1){c[(c[l>>2]|0)+(m<<2)>>2]=(c[(c[k>>2]|0)+(m<<2)>>2]|0)+(n<<2);o=m+1|0;if((o|0)>=(c[h>>2]|0)){p=l;break L4161}m=o;n=c[d>>2]|0}}else{p=a+12|0}}while(0);c[b>>2]=c[p>>2];i=c[g>>2]|0;j=c[d>>2]|0}f=i-j|0;return f|0}function Fi(a,b){a=a|0;b=b|0;var d=0,e=0;d=c[a+24>>2]|0;do{if((b|0)!=0){if((d+b|0)>(c[a+20>>2]|0)){e=-131}else{break}return e|0}}while(0);c[a+24>>2]=d+b;e=0;return e|0}function Fj(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0.0,i=0,j=0,k=0,l=0,m=0,n=0.0,o=0,p=0.0;d=c[b+28>>2]|0;e=c[b+4>>2]|0;c[a+4>>2]=128;c[a+8>>2]=64;g[a+12>>2]=+g[d+2932>>2];c[a>>2]=e;b=a+164|0;c[b>>2]=128;c[a+176>>2]=(c[d+4>>2]|0)/2|0;d=a+36|0;c[d>>2]=NE(128,4)|0;Fz(a+16|0,128);f=0;do{h=+V(+(+(f|0)/127.0*3.141592653589793));g[(c[d>>2]|0)+(f<<2)>>2]=h;i=(c[d>>2]|0)+(f<<2)|0;h=+g[i>>2];g[i>>2]=h*h;f=f+1|0;}while((f|0)<128);c[a+40>>2]=2;c[a+44>>2]=4;c[a+56>>2]=4;c[a+60>>2]=5;c[a+72>>2]=6;c[a+76>>2]=6;c[a+88>>2]=9;c[a+92>>2]=8;c[a+104>>2]=13;c[a+108>>2]=8;c[a+120>>2]=17;c[a+124>>2]=8;c[a+136>>2]=22;c[a+140>>2]=8;f=0;d=4;while(1){i=NC(d<<2)|0;j=a+40+(f<<4)+8|0;c[j>>2]=i;if((d|0)>0){h=+(d|0);k=a+40+(f<<4)+12|0;l=0;m=i;while(1){g[m+(l<<2)>>2]=+V(+((+(l|0)+.5)/h*3.141592653589793));i=c[j>>2]|0;n=+g[i+(l<<2)>>2]+ +g[k>>2];g[k>>2]=n;o=l+1|0;if((o|0)<(d|0)){l=o;m=i}else{p=n;break}}}else{p=+g[a+40+(f<<4)+12>>2]}g[a+40+(f<<4)+12>>2]=1.0/p;m=f+1|0;if((m|0)>=7){break}f=m;d=c[a+40+(m<<4)+4>>2]|0}c[a+152>>2]=NE(e*7|0,144)|0;c[a+160>>2]=NE(c[b>>2]|0,4)|0;return}function Fk(a){a=a|0;FA(a+16|0);ND(c[a+48>>2]|0);ND(c[a+64>>2]|0);ND(c[a+80>>2]|0);ND(c[a+96>>2]|0);ND(c[a+112>>2]|0);ND(c[a+128>>2]|0);ND(c[a+144>>2]|0);ND(c[a+36>>2]|0);ND(c[a+152>>2]|0);ND(c[a+160>>2]|0);NM(a|0,0,180);return}function Fl(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;b=c[(c[a+4>>2]|0)+28>>2]|0;d=b+2868|0;e=c[c[a+104>>2]>>2]|0;f=e+168|0;g=e+8|0;h=c[g>>2]|0;i=(c[f>>2]|0)/(h|0)|0;j=(c[a+20>>2]|0)/(h|0)|0;h=j-4|0;k=(i|0)<0?0:i;i=j+2|0;j=e+164|0;if((i|0)>(c[j>>2]|0)){c[j>>2]=i;j=e+160|0;c[j>>2]=NF(c[j>>2]|0,i<<2)|0}if((k|0)<(h|0)){i=e+156|0;j=e|0;l=e+160|0;m=a+8|0;n=e+40|0;o=e+152|0;p=k;while(1){k=(c[i>>2]|0)+1|0;c[i>>2]=(k|0)>24?24:k;do{if((c[j>>2]|0)>0){k=0;q=0;do{r=c[(c[m>>2]|0)+(q<<2)>>2]|0;s=r+((ac(c[g>>2]|0,p)|0)<<2)|0;k=Fm(e,d,s,n,(c[o>>2]|0)+((q*7|0)*144|0)|0)|0|k;q=q+1|0;}while((q|0)<(c[j>>2]|0));c[(c[l>>2]|0)+(p+2<<2)>>2]=0;if((k&1|0)!=0){c[(c[l>>2]|0)+(p<<2)>>2]=1;c[(c[l>>2]|0)+(p+1<<2)>>2]=1}do{if((k&2|0)!=0){c[(c[l>>2]|0)+(p<<2)>>2]=1;if((p|0)<=0){break}c[(c[l>>2]|0)+(p-1<<2)>>2]=1}}while(0);if((k&4|0)==0){break}c[i>>2]=-1}else{c[(c[l>>2]|0)+(p+2<<2)>>2]=0}}while(0);q=p+1|0;if((q|0)<(h|0)){p=q}else{t=l;break}}}else{t=e+160|0}l=c[g>>2]|0;g=ac(l,h)|0;c[f>>2]=g;f=c[a+48>>2]|0;h=((c[b+(c[a+40>>2]<<2)>>2]|0)/4|0)+f+((c[b+4>>2]|0)/2|0)+((c[b>>2]|0)/4|0)|0;b=e+176|0;a=c[b>>2]|0;while(1){if((a|0)>=(g-l|0)){u=-1;v=3043;break}if((a|0)>=(h|0)){u=1;v=3042;break}c[b>>2]=a;if((c[(c[t>>2]|0)+(((a|0)/(l|0)|0)<<2)>>2]|0)!=0&(a|0)>(f|0)){v=3040;break}else{a=l+a|0}}if((v|0)==3042){return u|0}else if((v|0)==3043){return u|0}else if((v|0)==3040){c[e+172>>2]=a;u=0;return u|0}return 0}function Fm(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,j=0,l=0.0,m=0,n=0,o=0,p=0,q=0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0.0,y=0,z=0,A=0,B=0.0,C=0,D=0.0;h=i;j=c[a+4>>2]|0;l=+g[a+12>>2];m=i;i=i+(j<<2)|0;i=i+7&-8;n=m;o=c[a+156>>2]|0;p=(o|0)/2|0;q=(o|0)>5?p:2;r=+g[b+60>>2];s=r- +(p-2|0);t=s<0.0?0.0:s;s=t>r?r:t;if((j|0)>0){p=c[a+36>>2]|0;o=0;do{g[n+(o<<2)>>2]=+g[d+(o<<2)>>2]*+g[p+(o<<2)>>2];o=o+1|0;}while((o|0)<(j|0))}FE(a+16|0,n,n);t=+g[n>>2];r=+g[m+4>>2];u=+g[m+8>>2];v=t*t+r*r*.7+u*u*.2;m=f+140|0;a=c[m>>2]|0;if((a|0)==0){o=f+136|0;u=v+ +g[o>>2];g[f+132>>2]=u;g[o>>2]=v;w=u}else{o=f+132|0;u=v+ +g[o>>2];g[o>>2]=u;o=f+136|0;g[o>>2]=v+ +g[o>>2];w=u}o=f+72+(a<<2)|0;g[f+132>>2]=w- +g[o>>2];g[o>>2]=v;o=(c[m>>2]|0)+1|0;c[m>>2]=(o|0)>14?0:o;o=(j|0)/2|0;L4234:do{if((j|0)>1){v=(+(((g[k>>2]=w*.0625,c[k>>2]|0)&2147483647)>>>0>>>0)*7.177114298428933e-7+ -764.6162109375)*.5+-15.0;m=0;u=t;while(1){r=+g[n+((m|1)<<2)>>2];x=(+(((g[k>>2]=u*u+r*r,c[k>>2]|0)&2147483647)>>>0>>>0)*7.177114298428933e-7+ -764.6162109375)*.5;r=x<v?v:x;g[n+(m>>1<<2)>>2]=r<l?l:r;a=m+2|0;if((a|0)>=(o|0)){break L4234}v=v+-8.0;m=a;u=+g[n+(a<<2)>>2]}}}while(0);if((q|0)>0){y=0;z=0}else{o=0;j=0;while(1){m=c[e+(o<<4)+4>>2]|0;if((m|0)>0){a=c[e+(o<<4)>>2]|0;p=c[e+(o<<4)+8>>2]|0;l=0.0;d=0;while(1){t=l+ +g[n+(a+d<<2)>>2]*+g[p+(d<<2)>>2];A=d+1|0;if((A|0)<(m|0)){l=t;d=A}else{B=t;break}}}else{B=0.0}l=B*+g[e+(o<<4)+12>>2];d=f+(o*144|0)+68|0;m=c[d>>2]|0;p=m-1|0;t=+g[f+(o*144|0)+(((p|0)<0?m+16|0:p)<<2)>>2];g[f+(o*144|0)+(m<<2)>>2]=l;m=(c[d>>2]|0)+1|0;c[d>>2]=(m|0)>16?0:m;m=(l<t?t:l)+99999.0>s+ +g[b+4+(o<<2)>>2]?j|5:j;d=(l>t?t:l)+-99999.0<+g[b+32+(o<<2)>>2]-s?m|2:m;m=o+1|0;if((m|0)<7){o=m;j=d}else{C=d;break}}i=h;return C|0}while(1){j=c[e+(y<<4)+4>>2]|0;if((j|0)>0){o=c[e+(y<<4)>>2]|0;d=c[e+(y<<4)+8>>2]|0;B=0.0;m=0;while(1){l=B+ +g[n+(o+m<<2)>>2]*+g[d+(m<<2)>>2];p=m+1|0;if((p|0)<(j|0)){B=l;m=p}else{D=l;break}}}else{D=0.0}B=D*+g[e+(y<<4)+12>>2];m=f+(y*144|0)+68|0;j=c[m>>2]|0;d=j-1|0;o=(d|0)<0?j+16|0:d;l=+g[f+(y*144|0)+(o<<2)>>2];t=B<l?l:B;w=B>l?l:B;d=o;o=0;l=-99999.0;u=99999.0;do{p=d-1|0;d=(p|0)<0?d+16|0:p;v=+g[f+(y*144|0)+(d<<2)>>2];l=l<v?v:l;u=u>v?v:u;o=o+1|0;}while((o|0)<(q|0));g[f+(y*144|0)+(j<<2)>>2]=B;o=(c[m>>2]|0)+1|0;c[m>>2]=(o|0)>16?0:o;o=t-l>s+ +g[b+4+(y<<2)>>2]?z|5:z;d=w-u<+g[b+32+(y<<2)>>2]-s?o|2:o;o=y+1|0;if((o|0)<7){y=o;z=d}else{C=d;break}}i=h;return C|0}function Fn(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;b=c[c[a+104>>2]>>2]|0;d=c[(c[a+4>>2]|0)+28>>2]|0;e=c[a+48>>2]|0;f=c[a+40>>2]|0;g=d;h=(c[g+(f<<2)>>2]|0)/4|0;if((f|0)==0){f=(c[d>>2]|0)/4|0;i=f;j=f}else{i=(c[g+(c[a+36>>2]<<2)>>2]|0)/4|0;j=(c[g+(c[a+44>>2]<<2)>>2]|0)/4|0}a=e-h-i|0;i=h+e+j|0;j=c[b+172>>2]|0;if((j|0)>=(a|0)&(j|0)<(i|0)){k=1;return k|0}j=c[b+8>>2]|0;e=(i|0)/(j|0)|0;i=b+160|0;b=(a|0)/(j|0)|0;while(1){if((b|0)>=(e|0)){k=0;l=3079;break}if((c[(c[i>>2]|0)+(b<<2)>>2]|0)==0){b=b+1|0}else{k=1;l=3078;break}}if((l|0)==3078){return k|0}else if((l|0)==3079){return k|0}return 0}function Fo(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0;d=a+168|0;e=c[a+8>>2]|0;f=(b|0)/(e|0)|0;g=c[a+160>>2]|0;NP(g|0,g+(f<<2)|0,((c[d>>2]|0)/(e|0)|0)+2-f<<2|0);c[d>>2]=(c[d>>2]|0)-b;d=a+172|0;f=c[d>>2]|0;if((f|0)<=-1){h=a+176|0;i=c[h>>2]|0;j=i-b|0;c[h>>2]=j;return}c[d>>2]=f-b;h=a+176|0;i=c[h>>2]|0;j=i-b|0;c[h>>2]=j;return}function Fp(a){a=a|0;NM(a|0,0,16);return}function Fq(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0;f=i;g=NL(d|0)|0;h=g+2+(NL(e|0)|0)|0;g=i;i=i+h|0;i=i+7&-8;NQ(g|0,d|0)|0;d=g+(NL(g|0)|0)|0;y=61;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;NT(g|0,e|0)|0;e=b|0;d=b+8|0;c[e>>2]=NF(c[e>>2]|0,(c[d>>2]<<2)+8|0)|0;h=b+4|0;b=NF(c[h>>2]|0,(c[d>>2]<<2)+8|0)|0;c[h>>2]=b;h=NL(g|0)|0;j=c[d>>2]|0;c[b+(j<<2)>>2]=h;b=NC(h+1|0)|0;c[(c[e>>2]|0)+(j<<2)>>2]=b;NQ(c[(c[e>>2]|0)+(c[d>>2]<<2)>>2]|0,g|0)|0;g=(c[d>>2]|0)+1|0;c[d>>2]=g;c[(c[e>>2]|0)+(g<<2)>>2]=0;i=f;return}function Fr(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;if((a|0)==0){return}b=a|0;d=c[b>>2]|0;if((d|0)!=0){e=a+8|0;f=c[e>>2]|0;if((f|0)>0){g=0;h=d;i=f;while(1){f=c[h+(g<<2)>>2]|0;if((f|0)==0){j=i;k=h}else{ND(f);j=c[e>>2]|0;k=c[b>>2]|0}f=g+1|0;if((f|0)<(j|0)){g=f;h=k;i=j}else{l=k;break}}}else{l=d}ND(l)}l=c[a+4>>2]|0;if((l|0)!=0){ND(l)}l=c[a+12>>2]|0;if((l|0)!=0){ND(l)}NM(a|0,0,16);return}function Fs(a){a=a|0;NM(a|0,0,28);c[a+28>>2]=NE(1,3664)|0;return}function Ft(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0;b=c[a+28>>2]|0;if((b|0)==0){d=a;NM(d|0,0,32);return}e=b+8|0;f=c[e>>2]|0;if((f|0)>0){g=b+32|0;h=0;i=f;while(1){f=c[g+(h<<2)>>2]|0;if((f|0)==0){j=i}else{ND(f);j=c[e>>2]|0}f=h+1|0;if((f|0)<(j|0)){h=f;i=j}else{break}}}j=b+12|0;i=c[j>>2]|0;if((i|0)>0){h=b+544|0;e=b+288|0;g=0;f=i;while(1){i=c[h+(g<<2)>>2]|0;if((i|0)==0){k=f}else{c_[c[(c[1811400+(c[e+(g<<2)>>2]<<2)>>2]|0)+8>>2]&127](i);k=c[j>>2]|0}i=g+1|0;if((i|0)<(k|0)){g=i;f=k}else{break}}}k=b+16|0;f=c[k>>2]|0;if((f|0)>0){g=b+1056|0;j=b+800|0;e=0;h=f;while(1){f=c[g+(e<<2)>>2]|0;if((f|0)==0){l=h}else{c_[c[(c[1900584+(c[j+(e<<2)>>2]<<2)>>2]|0)+12>>2]&127](f);l=c[k>>2]|0}f=e+1|0;if((f|0)<(l|0)){e=f;h=l}else{break}}}l=b+20|0;h=c[l>>2]|0;if((h|0)>0){e=b+1568|0;k=b+1312|0;j=0;g=h;while(1){h=c[e+(j<<2)>>2]|0;if((h|0)==0){m=g}else{c_[c[(c[1745208+(c[k+(j<<2)>>2]<<2)>>2]|0)+12>>2]&127](h);m=c[l>>2]|0}h=j+1|0;if((h|0)<(m|0)){j=h;g=m}else{break}}}m=b+24|0;if((c[m>>2]|0)>0){g=b+1824|0;j=b+2848|0;l=0;while(1){k=c[g+(l<<2)>>2]|0;if((k|0)!=0){Ga(k)}k=c[j>>2]|0;if((k|0)!=0){Gb(k+(l*56|0)|0)}k=l+1|0;if((k|0)<(c[m>>2]|0)){l=k}else{n=j;break}}}else{n=b+2848|0}j=c[n>>2]|0;if((j|0)!=0){ND(j)}j=b+28|0;if((c[j>>2]|0)>0){n=b+2852|0;l=0;do{FH(c[n+(l<<2)>>2]|0);l=l+1|0;}while((l|0)<(c[j>>2]|0))}ND(b);d=a;NM(d|0,0,32);return}function Fu(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;f=i;i=i+32|0;g=f|0;h=f+24|0;if((e|0)==0){j=-133;i=f;return j|0}G6(g,c[e>>2]|0,c[e+4>>2]|0);k=G9(g,8)|0;l=h|0;NM(l|0,0,5);a[l]=(G9(g,8)|0)&255;a[h+1|0]=(G9(g,8)|0)&255;a[h+2|0]=(G9(g,8)|0)&255;a[h+3|0]=(G9(g,8)|0)&255;a[h+4|0]=(G9(g,8)|0)&255;a[h+5|0]=(G9(g,8)|0)&255;if((NO(l|0,1737040,6)|0)!=0){j=-132;i=f;return j|0}if((k|0)==3){if((c[b+8>>2]|0)==0){j=-133;i=f;return j|0}l=G9(g,32)|0;L4372:do{if((l|0)>=0){h=g+16|0;if((l|0)>((c[h>>2]|0)-8|0)){break}m=NE(l+1|0,1)|0;c[d+12>>2]=m;if((l|0)!=0){n=m;m=l;while(1){o=m-1|0;a[n]=(G9(g,8)|0)&255;if((o|0)==0){break}else{n=n+1|0;m=o}}}m=G9(g,32)|0;if((m|0)<0){break}n=c[h>>2]|0;if((m|0)>(n-(Ha(g)|0)>>2|0)){break}n=d+8|0;c[n>>2]=m;o=m+1|0;p=d|0;c[p>>2]=NE(o,4)|0;q=d+4|0;c[q>>2]=NE(o,4)|0;if((m|0)>0){m=0;do{o=G9(g,32)|0;if((o|0)<0){break L4372}r=c[h>>2]|0;if((o|0)>(r-(Ha(g)|0)|0)){break L4372}c[(c[q>>2]|0)+(m<<2)>>2]=o;r=NE(o+1|0,1)|0;c[(c[p>>2]|0)+(m<<2)>>2]=r;if((o|0)!=0){r=c[(c[p>>2]|0)+(m<<2)>>2]|0;s=o;while(1){o=s-1|0;a[r]=(G9(g,8)|0)&255;if((o|0)==0){break}else{r=r+1|0;s=o}}}m=m+1|0;}while((m|0)<(c[n>>2]|0))}if((G9(g,1)|0)==1){j=0}else{break}i=f;return j|0}}while(0);Fr(d);j=-133;i=f;return j|0}else if((k|0)==1){if((c[e+8>>2]|0)==0){j=-133;i=f;return j|0}e=b+8|0;if((c[e>>2]|0)!=0){j=-133;i=f;return j|0}l=c[b+28>>2]|0;if((l|0)==0){j=-129;i=f;return j|0}n=G9(g,32)|0;c[b>>2]=n;if((n|0)!=0){j=-134;i=f;return j|0}n=b+4|0;c[n>>2]=G9(g,8)|0;c[e>>2]=G9(g,32)|0;c[b+12>>2]=G9(g,32)|0;c[b+16>>2]=G9(g,32)|0;c[b+20>>2]=G9(g,32)|0;m=l;c[m>>2]=1<<(G9(g,4)|0);p=1<<(G9(g,4)|0);c[l+4>>2]=p;do{if((c[e>>2]|0)>=1){if((c[n>>2]|0)<1){break}l=c[m>>2]|0;if((l|0)<64|(p|0)<(l|0)|(p|0)>8192){break}if((G9(g,1)|0)==1){j=0}else{break}i=f;return j|0}}while(0);Ft(b);j=-133;i=f;return j|0}else if((k|0)==5){if((c[b+8>>2]|0)==0){j=-133;i=f;return j|0}if((c[d+12>>2]|0)==0){j=-133;i=f;return j|0}d=c[b+28>>2]|0;if((d|0)==0){j=-129;i=f;return j|0}k=G9(g,8)|0;p=k+1|0;m=d+24|0;c[m>>2]=p;L4425:do{if((k|0)>=0){n=d+1824|0;if((p|0)>0){e=0;do{l=Gp(g)|0;c[n+(e<<2)>>2]=l;if((l|0)==0){break L4425}e=e+1|0;}while((e|0)<(c[m>>2]|0))}e=G9(g,6)|0;n=e+1|0;if((e|0)<0){break}else{t=0}while(1){if((t|0)>=(n|0)){break}if((G9(g,16)|0)==0){t=t+1|0}else{break L4425}}n=G9(g,6)|0;e=n+1|0;l=d+16|0;c[l>>2]=e;if((n|0)<0){break}n=d+800|0;q=d+1056|0;if((e|0)>0){e=0;do{h=G9(g,16)|0;c[n+(e<<2)>>2]=h;if(h>>>0>1){break L4425}s=dg[c[(c[1900584+(h<<2)>>2]|0)+4>>2]&255](b,g)|0;c[q+(e<<2)>>2]=s;if((s|0)==0){break L4425}e=e+1|0;}while((e|0)<(c[l>>2]|0))}l=G9(g,6)|0;e=l+1|0;q=d+20|0;c[q>>2]=e;if((l|0)<0){break}l=d+1312|0;n=d+1568|0;if((e|0)>0){e=0;do{s=G9(g,16)|0;c[l+(e<<2)>>2]=s;if(s>>>0>2){break L4425}h=dg[c[(c[1745208+(s<<2)>>2]|0)+4>>2]&255](b,g)|0;c[n+(e<<2)>>2]=h;if((h|0)==0){break L4425}e=e+1|0;}while((e|0)<(c[q>>2]|0))}q=G9(g,6)|0;e=q+1|0;n=d+12|0;c[n>>2]=e;if((q|0)<0){break}q=d+288|0;l=d+544|0;if((e|0)>0){e=(c[452850]|0)+4|0;h=0;do{s=G9(g,16)|0;c[q+(h<<2)>>2]=s;if((s|0)!=0){break L4425}s=dg[c[e>>2]&255](b,g)|0;c[l+(h<<2)>>2]=s;if((s|0)==0){break L4425}h=h+1|0;}while((h|0)<(c[n>>2]|0))}h=G9(g,6)|0;l=h+1|0;e=d+8|0;c[e>>2]=l;if((h|0)<0){break}h=d+32|0;if((l|0)>0){l=0;do{q=h+(l<<2)|0;c[q>>2]=NE(1,16)|0;s=G9(g,1)|0;c[c[q>>2]>>2]=s;s=G9(g,16)|0;c[(c[q>>2]|0)+4>>2]=s;s=G9(g,16)|0;c[(c[q>>2]|0)+8>>2]=s;s=G9(g,8)|0;c[(c[q>>2]|0)+12>>2]=s;s=c[q>>2]|0;if((c[s+4>>2]|0)>0){break L4425}if((c[s+8>>2]|0)>0){break L4425}q=c[s+12>>2]|0;if((q|0)>=(c[n>>2]|0)|(q|0)<0){break L4425}l=l+1|0;}while((l|0)<(c[e>>2]|0))}if((G9(g,1)|0)==1){j=0}else{break}i=f;return j|0}}while(0);Ft(b);j=-133;i=f;return j|0}else{j=-133;i=f;return j|0}return 0}function Fv(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0;G3(b,3,8);G3(b,118,8);G3(b,111,8);G3(b,114,8);G3(b,98,8);G3(b,105,8);G3(b,115,8);G3(b,43,32);e=1678992;f=43;while(1){g=f-1|0;G3(b,a[e]|0,8);if((g|0)==0){break}else{e=e+1|0;f=g}}f=d+8|0;G3(b,c[f>>2]|0,32);if((c[f>>2]|0)<=0){G3(b,1,1);return}e=d|0;g=d+4|0;d=0;do{do{if((c[(c[e>>2]|0)+(d<<2)>>2]|0)==0){G3(b,0,32)}else{G3(b,c[(c[g>>2]|0)+(d<<2)>>2]|0,32);h=c[(c[g>>2]|0)+(d<<2)>>2]|0;if((h|0)==0){break}i=c[(c[e>>2]|0)+(d<<2)>>2]|0;j=h;while(1){h=j-1|0;G3(b,a[i]|0,8);if((h|0)==0){break}else{i=i+1|0;j=h}}}}while(0);d=d+1|0;}while((d|0)<(c[f>>2]|0));G3(b,1,1);return}function Fw(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;g=i;i=i+24|0;h=g|0;j=c[a+4>>2]|0;k=c[a+104>>2]|0;if((k|0)==0){NM(d|0,0,32);NM(e|0,0,32);NM(f|0,0,32);l=-129;i=g;return l|0}G1(h);a=j+28|0;m=c[a>>2]|0;if((m|0)==0){NM(d|0,0,32);NM(e|0,0,32);NM(f|0,0,32);n=k+64|0}else{G3(h,1,8);G3(h,118,8);G3(h,111,8);G3(h,114,8);G3(h,98,8);G3(h,105,8);G3(h,115,8);G3(h,0,32);G3(h,c[j+4>>2]|0,8);G3(h,c[j+8>>2]|0,32);G3(h,c[j+12>>2]|0,32);G3(h,c[j+16>>2]|0,32);G3(h,c[j+20>>2]|0,32);o=c[m>>2]|0;p=(o|0)==0?0:o-1|0;if((p|0)==0){q=0}else{o=p;p=0;while(1){r=p+1|0;s=o>>>1;if((s|0)==0){q=r;break}else{o=s;p=r}}}G3(h,q,4);q=c[m+4>>2]|0;m=(q|0)==0?0:q-1|0;if((m|0)==0){t=0}else{q=m;m=0;while(1){p=m+1|0;o=q>>>1;if((o|0)==0){t=p;break}else{q=o;m=p}}}G3(h,t,4);G3(h,1,1);t=k+64|0;m=c[t>>2]|0;if((m|0)!=0){ND(m)}m=NC(Ha(h)|0)|0;c[t>>2]=m;q=h+8|0;p=c[q>>2]|0;o=Ha(h)|0;NN(m|0,p|0,o)|0;c[d>>2]=c[t>>2];c[d+4>>2]=Ha(h)|0;c[d+8>>2]=1;NM(d+12|0,0,20);G5(h);Fv(h,b);b=k+68|0;o=c[b>>2]|0;if((o|0)!=0){ND(o)}o=NC(Ha(h)|0)|0;c[b>>2]=o;p=c[q>>2]|0;m=Ha(h)|0;NN(o|0,p|0,m)|0;c[e>>2]=c[b>>2];c[e+4>>2]=Ha(h)|0;b=e+24|0;NM(e+8|0,0,16);c[b>>2]=1;c[b+4>>2]=0;G5(h);b=c[a>>2]|0;L4509:do{if((b|0)!=0){G3(h,5,8);G3(h,118,8);G3(h,111,8);G3(h,114,8);G3(h,98,8);G3(h,105,8);G3(h,115,8);a=b+24|0;G3(h,(c[a>>2]|0)-1|0,8);m=b+1824|0;p=0;while(1){if((p|0)>=(c[a>>2]|0)){break}if((Go(c[m+(p<<2)>>2]|0,h)|0)==0){p=p+1|0}else{break L4509}}G3(h,0,6);G3(h,0,16);p=b+16|0;G3(h,(c[p>>2]|0)-1|0,6);if((c[p>>2]|0)>0){m=b+800|0;a=b+1056|0;o=0;do{r=m+(o<<2)|0;G3(h,c[r>>2]|0,16);s=c[c[1900584+(c[r>>2]<<2)>>2]>>2]|0;if((s|0)==0){break L4509}c$[s&127](c[a+(o<<2)>>2]|0,h);o=o+1|0;}while((o|0)<(c[p>>2]|0))}p=b+20|0;G3(h,(c[p>>2]|0)-1|0,6);if((c[p>>2]|0)>0){o=b+1312|0;a=b+1568|0;m=0;do{s=o+(m<<2)|0;G3(h,c[s>>2]|0,16);c$[c[c[1745208+(c[s>>2]<<2)>>2]>>2]&127](c[a+(m<<2)>>2]|0,h);m=m+1|0;}while((m|0)<(c[p>>2]|0))}p=b+12|0;G3(h,(c[p>>2]|0)-1|0,6);if((c[p>>2]|0)>0){m=b+288|0;a=b+544|0;o=0;do{s=m+(o<<2)|0;G3(h,c[s>>2]|0,16);dm[c[c[1811400+(c[s>>2]<<2)>>2]>>2]&127](j,c[a+(o<<2)>>2]|0,h);o=o+1|0;}while((o|0)<(c[p>>2]|0))}p=b+8|0;G3(h,(c[p>>2]|0)-1|0,6);if((c[p>>2]|0)>0){o=b+32|0;a=0;do{m=o+(a<<2)|0;G3(h,c[c[m>>2]>>2]|0,1);G3(h,c[(c[m>>2]|0)+4>>2]|0,16);G3(h,c[(c[m>>2]|0)+8>>2]|0,16);G3(h,c[(c[m>>2]|0)+12>>2]|0,8);a=a+1|0;}while((a|0)<(c[p>>2]|0))}G3(h,1,1);p=k+72|0;a=c[p>>2]|0;if((a|0)!=0){ND(a)}a=NC(Ha(h)|0)|0;c[p>>2]=a;o=c[q>>2]|0;m=Ha(h)|0;NN(a|0,o|0,m)|0;c[f>>2]=c[p>>2];c[f+4>>2]=Ha(h)|0;p=f+24|0;NM(f+8|0,0,16);c[p>>2]=2;c[p+4>>2]=0;G4(h);l=0;i=g;return l|0}}while(0);NM(d|0,0,32);NM(e|0,0,32);NM(f|0,0,32);n=t}G4(h);h=c[n>>2]|0;if((h|0)!=0){ND(h)}h=k+68|0;t=c[h>>2]|0;if((t|0)!=0){ND(t)}t=k+72|0;k=c[t>>2]|0;if((k|0)!=0){ND(k)}c[n>>2]=0;c[h>>2]=0;c[t>>2]=0;l=-130;i=g;return l|0}function Fx(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0,j=0,k=0,l=0,m=0.0,n=0.0,o=0.0,p=0,q=0.0,r=0,s=0.0,t=0.0,u=0.0,v=0,w=0,x=0.0,y=0,z=0,A=0.0,B=0,C=0.0,D=0,E=0.0,F=0;e=i;f=d+1|0;j=i;i=i+(f<<3)|0;i=i+7&-8;k=j;j=i;i=i+(d<<3)|0;i=i+7&-8;l=j;if((f|0)==0){m=0.0}else{f=d;while(1){if((f|0)<(c|0)){n=0.0;j=f;while(1){o=n+ +g[a+(j<<2)>>2]*+g[a+(j-f<<2)>>2];p=j+1|0;if((p|0)<(c|0)){n=o;j=p}else{q=o;break}}}else{q=0.0}h[k+(f<<3)>>3]=q;if((f|0)==0){break}else{f=f-1|0}}m=+h[k>>3]}q=m*1.0000000001;n=m*1.0e-9+1.0e-10;f=(d|0)>0;if(f){r=0;s=q}else{t=q;u=t;i=e;return+u}while(1){c=r+1|0;if(s<n){v=3285;break}q=-0.0- +h[k+(c<<3)>>3];do{if((r|0)>0){a=0;m=q;do{m=m- +h[l+(a<<3)>>3]*+h[k+(r-a<<3)>>3];a=a+1|0;}while((a|0)<(r|0));o=m/s;h[l+(r<<3)>>3]=o;a=(r|0)/2|0;if((r|0)<=1){w=0;x=o;break}j=r-1|0;p=(a|0)>1;y=0;do{z=l+(y<<3)|0;A=+h[z>>3];B=l+(j-y<<3)|0;h[z>>3]=A+o*+h[B>>3];h[B>>3]=o*A+ +h[B>>3];y=y+1|0;}while((y|0)<(a|0));w=p?a:1;x=o}else{m=q/s;h[l+(r<<3)>>3]=m;w=0;x=m}}while(0);if((r&1|0)!=0){y=l+(w<<3)|0;q=+h[y>>3];h[y>>3]=q+x*q}q=s*(1.0-x*x);if((c|0)<(d|0)){r=c;s=q}else{C=q;break}}if((v|0)==3285){NM(l+(r<<3)|0,0,d-r<<3|0);C=s}if(f){D=0;E=.99}else{t=C;u=t;i=e;return+u}while(1){r=l+(D<<3)|0;h[r>>3]=E*+h[r>>3];r=D+1|0;if((r|0)>=(d|0)){break}D=r;E=E*.99}if(f){F=0}else{t=C;u=t;i=e;return+u}while(1){g[b+(F<<2)>>2]=+h[l+(F<<3)>>3];f=F+1|0;if((f|0)<(d|0)){F=f}else{t=C;break}}u=t;i=e;return+u}function Fy(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0.0;f=i;h=d;j=b;k=i;i=i+(e+c<<2)|0;i=i+7&-8;l=k;m=(c|0)>0;do{if((b|0)==0){if(!m){break}NM(k|0,0,c<<2|0)}else{if(!m){break}n=c<<2;NN(k|0,j|0,n)|0}}while(0);if((e|0)<=0){i=f;return}if((c|0)>0){o=0;p=c}else{j=e<<2;NM(k|0,0,j|0);NM(h|0,0,j|0);i=f;return}while(1){j=0;h=o;k=c;q=0.0;while(1){m=k-1|0;r=q- +g[l+(h<<2)>>2]*+g[a+(m<<2)>>2];b=j+1|0;if((b|0)<(c|0)){j=b;h=h+1|0;k=m;q=r}else{break}}g[l+(p<<2)>>2]=r;g[d+(o<<2)>>2]=r;k=o+1|0;if((k|0)<(e|0)){o=k;p=p+1|0}else{break}}i=f;return}function Fz(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0,i=0.0,j=0,k=0.0,l=0,m=0.0,n=0.0,o=0,p=0.0,q=0,r=0,s=0,t=0;d=(b|0)/4|0;e=NC(d<<2)|0;f=NC(d+b<<2)|0;h=b>>1;i=+(b|0);j=~~+bR(+(+aa(+i)/.6931471805599453));c[a+4>>2]=j;c[a>>2]=b;c[a+8>>2]=f;c[a+12>>2]=e;if((b|0)<=3){k=4.0/i;l=a+16|0;g[l>>2]=k;return}m=3.141592653589793/+(b|0);n=3.141592653589793/+(b<<1|0);o=0;do{p=+(o<<2|0)*m;q=o<<1;g[f+(q<<2)>>2]=+U(+p);r=q|1;g[f+(r<<2)>>2]=-0.0- +V(+p);p=+(r|0)*n;r=q+h|0;g[f+(r<<2)>>2]=+U(+p);g[f+(r+1<<2)>>2]=+V(+p);o=o+1|0;}while((o|0)<(d|0));d=(b|0)/8|0;o=(b|0)>7;if(!o){k=4.0/i;l=a+16|0;g[l>>2]=k;return}n=3.141592653589793/+(b|0);h=0;do{m=+(h<<2|2|0)*n;r=(h<<1)+b|0;g[f+(r<<2)>>2]=+U(+m)*.5;g[f+(r+1<<2)>>2]=+V(+m)*-.5;h=h+1|0;}while((h|0)<(d|0));h=(1<<j-1)-1|0;f=1<<j-2;if(o){s=0}else{k=4.0/i;l=a+16|0;g[l>>2]=k;return}do{o=0;j=0;b=f;while(1){if((b&s|0)==0){t=j}else{t=j|1<<o}r=o+1|0;q=f>>r;if((q|0)==0){break}else{o=r;j=t;b=q}}b=s<<1;c[e+(b<<2)>>2]=(h&~t)-1;c[e+((b|1)<<2)>>2]=t;s=s+1|0;}while((s|0)<(d|0));k=4.0/i;l=a+16|0;g[l>>2]=k;return}function FA(a){a=a|0;var b=0;if((a|0)==0){return}b=c[a+8>>2]|0;if((b|0)!=0){ND(b)}b=c[a+12>>2]|0;if((b|0)!=0){ND(b)}NM(a|0,0,20);return}function FB(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0.0,u=0,v=0;e=a|0;f=c[e>>2]|0;h=f>>1;i=f>>2;f=d+(h+i<<2)|0;j=a+8|0;k=(c[j>>2]|0)+(i<<2)|0;l=f;m=b+(h-7<<2)|0;while(1){n=l-16|0;o=m+8|0;p=k+12|0;q=k+8|0;g[n>>2]=+g[p>>2]*(-0.0- +g[o>>2])- +g[m>>2]*+g[q>>2];g[l-12>>2]=+g[m>>2]*+g[p>>2]- +g[o>>2]*+g[q>>2];q=m+24|0;o=k+4|0;p=m+16|0;g[l-8>>2]=+g[o>>2]*(-0.0- +g[q>>2])- +g[p>>2]*+g[k>>2];g[l-4>>2]=+g[p>>2]*+g[o>>2]- +g[q>>2]*+g[k>>2];q=m-32|0;if(q>>>0<b>>>0){break}else{k=k+16|0;l=n;m=q}}m=d+(h<<2)|0;l=(c[j>>2]|0)+(i<<2)|0;k=f;q=b+(h-8<<2)|0;while(1){n=l-16|0;o=q+16|0;p=l-4|0;r=q+24|0;s=l-8|0;g[k>>2]=+g[o>>2]*+g[p>>2]+ +g[r>>2]*+g[s>>2];g[k+4>>2]=+g[o>>2]*+g[s>>2]- +g[r>>2]*+g[p>>2];p=l-12|0;r=q+8|0;g[k+8>>2]=+g[q>>2]*+g[p>>2]+ +g[r>>2]*+g[n>>2];g[k+12>>2]=+g[q>>2]*+g[n>>2]- +g[r>>2]*+g[p>>2];p=q-32|0;if(p>>>0<b>>>0){break}else{l=n;k=k+16|0;q=p}}FC(c[a+4>>2]|0,c[j>>2]|0,m,h);FD(c[e>>2]|0,c[j>>2]|0,c[a+12>>2]|0,d);a=d;e=f;q=f;k=(c[j>>2]|0)+(h<<2)|0;while(1){h=q-16|0;j=k+4|0;l=a+4|0;g[q-4>>2]=+g[a>>2]*+g[j>>2]- +g[l>>2]*+g[k>>2];g[e>>2]=-0.0-(+g[a>>2]*+g[k>>2]+ +g[l>>2]*+g[j>>2]);j=a+8|0;l=k+12|0;b=a+12|0;p=k+8|0;g[q-8>>2]=+g[j>>2]*+g[l>>2]- +g[b>>2]*+g[p>>2];g[e+4>>2]=-0.0-(+g[j>>2]*+g[p>>2]+ +g[b>>2]*+g[l>>2]);l=a+16|0;b=k+20|0;p=a+20|0;j=k+16|0;g[q-12>>2]=+g[l>>2]*+g[b>>2]- +g[p>>2]*+g[j>>2];g[e+8>>2]=-0.0-(+g[l>>2]*+g[j>>2]+ +g[p>>2]*+g[b>>2]);b=a+24|0;p=k+28|0;j=a+28|0;l=k+24|0;g[h>>2]=+g[b>>2]*+g[p>>2]- +g[j>>2]*+g[l>>2];g[e+12>>2]=-0.0-(+g[b>>2]*+g[l>>2]+ +g[j>>2]*+g[p>>2]);p=a+32|0;if(p>>>0<h>>>0){a=p;e=e+16|0;q=h;k=k+32|0}else{break}}k=d+(i<<2)|0;i=f;d=k;q=k;while(1){k=q-16|0;e=i-16|0;t=+g[i-4>>2];g[q-4>>2]=t;g[d>>2]=-0.0-t;t=+g[i-8>>2];g[q-8>>2]=t;g[d+4>>2]=-0.0-t;t=+g[i-12>>2];g[q-12>>2]=t;g[d+8>>2]=-0.0-t;t=+g[e>>2];g[k>>2]=t;g[d+12>>2]=-0.0-t;a=d+16|0;if(a>>>0<e>>>0){i=e;d=a;q=k}else{u=f;v=f;break}}while(1){f=v-16|0;g[f>>2]=+g[u+12>>2];g[v-12>>2]=+g[u+8>>2];g[v-8>>2]=+g[u+4>>2];g[v-4>>2]=+g[u>>2];if(f>>>0>m>>>0){u=u+16|0;v=f}else{break}}return}function FC(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0,h=0,i=0,j=0.0,k=0,l=0.0,m=0.0,n=0,o=0.0,p=0,q=0.0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0.0,I=0.0,J=0.0,K=0.0,L=0.0,M=0.0,N=0.0,O=0.0,P=0.0,Q=0.0,R=0.0,S=0.0,T=0.0,U=0.0,V=0.0,W=0.0,X=0.0,Y=0.0,Z=0.0,_=0,$=0.0,aa=0.0,ab=0.0,ad=0,ae=0,af=0,ag=0.0,ah=0,ai=0.0,aj=0.0,ak=0.0,al=0,am=0,an=0.0,ao=0,ap=0.0,aq=0.0,ar=0.0;if((a-6|0)>0){e=b;f=c+(d-8<<2)|0;h=c+((d>>1)-8<<2)|0;while(1){i=f+24|0;j=+g[i>>2];k=h+24|0;l=+g[k>>2];m=j-l;n=f+28|0;o=+g[n>>2];p=h+28|0;q=o- +g[p>>2];g[i>>2]=j+l;g[n>>2]=o+ +g[p>>2];n=e+4|0;g[k>>2]=q*+g[n>>2]+m*+g[e>>2];g[p>>2]=q*+g[e>>2]-m*+g[n>>2];n=f+16|0;m=+g[n>>2];p=h+16|0;q=+g[p>>2];o=m-q;k=f+20|0;l=+g[k>>2];i=h+20|0;j=l- +g[i>>2];g[n>>2]=m+q;g[k>>2]=l+ +g[i>>2];k=e+20|0;n=e+16|0;g[p>>2]=j*+g[k>>2]+o*+g[n>>2];g[i>>2]=j*+g[n>>2]-o*+g[k>>2];k=f+8|0;o=+g[k>>2];n=h+8|0;j=+g[n>>2];l=o-j;i=f+12|0;q=+g[i>>2];p=h+12|0;m=q- +g[p>>2];g[k>>2]=o+j;g[i>>2]=q+ +g[p>>2];i=e+36|0;k=e+32|0;g[n>>2]=m*+g[i>>2]+l*+g[k>>2];g[p>>2]=m*+g[k>>2]-l*+g[i>>2];l=+g[f>>2];m=+g[h>>2];q=l-m;i=f+4|0;j=+g[i>>2];k=h+4|0;o=j- +g[k>>2];g[f>>2]=l+m;g[i>>2]=j+ +g[k>>2];i=e+52|0;p=e+48|0;g[h>>2]=o*+g[i>>2]+q*+g[p>>2];g[k>>2]=o*+g[p>>2]-q*+g[i>>2];i=h-32|0;if(i>>>0<c>>>0){break}else{e=e+64|0;f=f-32|0;h=i}}}h=a-7|0;if((h|0)>0){a=1;f=h;while(1){h=1<<a;if((h|0)>0){e=d>>a;i=4<<a;p=e-8|0;k=(e>>1)-8|0;n=i+1|0;r=i<<1;s=r|1;t=r+i|0;u=t+1|0;v=t+i|0;w=0;do{x=ac(w,e)|0;y=c+(x<<2)|0;z=b;A=c+(p+x<<2)|0;B=c+(k+x<<2)|0;while(1){x=A+24|0;q=+g[x>>2];C=B+24|0;o=+g[C>>2];j=q-o;D=A+28|0;m=+g[D>>2];E=B+28|0;l=m- +g[E>>2];g[x>>2]=q+o;g[D>>2]=m+ +g[E>>2];D=z+4|0;g[C>>2]=l*+g[D>>2]+j*+g[z>>2];g[E>>2]=l*+g[z>>2]-j*+g[D>>2];D=z+(i<<2)|0;E=A+16|0;j=+g[E>>2];C=B+16|0;l=+g[C>>2];m=j-l;x=A+20|0;o=+g[x>>2];F=B+20|0;q=o- +g[F>>2];g[E>>2]=j+l;g[x>>2]=o+ +g[F>>2];x=z+(n<<2)|0;g[C>>2]=q*+g[x>>2]+m*+g[D>>2];g[F>>2]=q*+g[D>>2]-m*+g[x>>2];x=z+(r<<2)|0;D=A+8|0;m=+g[D>>2];F=B+8|0;q=+g[F>>2];o=m-q;C=A+12|0;l=+g[C>>2];E=B+12|0;j=l- +g[E>>2];g[D>>2]=m+q;g[C>>2]=l+ +g[E>>2];C=z+(s<<2)|0;g[F>>2]=j*+g[C>>2]+o*+g[x>>2];g[E>>2]=j*+g[x>>2]-o*+g[C>>2];C=z+(t<<2)|0;o=+g[A>>2];j=+g[B>>2];l=o-j;x=A+4|0;q=+g[x>>2];E=B+4|0;m=q- +g[E>>2];g[A>>2]=o+j;g[x>>2]=q+ +g[E>>2];x=z+(u<<2)|0;g[B>>2]=m*+g[x>>2]+l*+g[C>>2];g[E>>2]=m*+g[C>>2]-l*+g[x>>2];x=B-32|0;if(x>>>0<y>>>0){break}else{z=z+(v<<2)|0;A=A-32|0;B=x}}w=w+1|0;}while((w|0)<(h|0))}h=f-1|0;if((h|0)>0){a=a+1|0;f=h}else{break}}}if((d|0)>0){G=0}else{return}do{f=c+(G<<2)|0;a=c+((G|30)<<2)|0;l=+g[a>>2];b=c+((G|14)<<2)|0;m=+g[b>>2];q=l-m;h=c+((G|31)<<2)|0;j=+g[h>>2];w=c+((G|15)<<2)|0;o=+g[w>>2];H=j-o;I=l+m;m=j+o;v=c+((G|28)<<2)|0;o=+g[v>>2];u=c+((G|12)<<2)|0;j=+g[u>>2];l=o-j;t=c+((G|29)<<2)|0;J=+g[t>>2];s=c+((G|13)<<2)|0;K=+g[s>>2];L=J-K;M=o+j;j=J+K;K=l*.9238795042037964-L*.3826834261417389;J=l*.3826834261417389+L*.9238795042037964;r=c+((G|26)<<2)|0;L=+g[r>>2];n=c+((G|10)<<2)|0;l=+g[n>>2];o=L-l;i=c+((G|27)<<2)|0;N=+g[i>>2];k=c+((G|11)<<2)|0;O=+g[k>>2];P=N-O;Q=L+l;l=N+O;O=(o-P)*.7071067690849304;N=(o+P)*.7071067690849304;p=c+((G|24)<<2)|0;P=+g[p>>2];e=c+((G|8)<<2)|0;o=+g[e>>2];L=P-o;B=c+((G|25)<<2)|0;R=+g[B>>2];A=c+((G|9)<<2)|0;S=+g[A>>2];T=R-S;U=P+o;o=R+S;S=L*.3826834261417389-T*.9238795042037964;R=L*.9238795042037964+T*.3826834261417389;z=c+((G|22)<<2)|0;T=+g[z>>2];y=c+((G|6)<<2)|0;L=+g[y>>2];P=T-L;x=c+((G|7)<<2)|0;V=+g[x>>2];C=c+((G|23)<<2)|0;W=+g[C>>2];X=V-W;Y=T+L;L=V+W;E=c+((G|4)<<2)|0;W=+g[E>>2];F=c+((G|20)<<2)|0;V=+g[F>>2];T=W-V;D=c+((G|5)<<2)|0;Z=+g[D>>2];_=c+((G|21)<<2)|0;$=+g[_>>2];aa=Z-$;ab=W+V;V=Z+$;$=T*.3826834261417389+aa*.9238795042037964;Z=aa*.3826834261417389-T*.9238795042037964;ad=c+((G|2)<<2)|0;T=+g[ad>>2];ae=c+((G|18)<<2)|0;aa=+g[ae>>2];W=T-aa;af=c+((G|3)<<2)|0;ag=+g[af>>2];ah=c+((G|19)<<2)|0;ai=+g[ah>>2];aj=ag-ai;ak=T+aa;aa=ag+ai;ai=(W+aj)*.7071067690849304;ag=(aj-W)*.7071067690849304;W=+g[f>>2];al=c+((G|16)<<2)|0;aj=+g[al>>2];T=W-aj;am=c+((G|1)<<2)|0;an=+g[am>>2];ao=c+((G|17)<<2)|0;ap=+g[ao>>2];aq=an-ap;ar=W+aj;aj=an+ap;ap=T*.9238795042037964+aq*.3826834261417389;an=aq*.9238795042037964-T*.3826834261417389;T=an-R;aq=ap-S;W=S+ap;ap=R+an;an=(T+aq)*.7071067690849304;R=(T-aq)*.7071067690849304;aq=ag-N;T=O-ai;S=O+ai;ai=N+ag;ag=K-$;N=J-Z;O=K+$;$=J+Z;Z=(ag-N)*.7071067690849304;J=(ag+N)*.7071067690849304;N=q-X;ag=H-P;K=q+X;X=H+P;P=N+aq;H=N-aq;aq=Z+an;N=Z-an;g[y>>2]=P+aq;g[E>>2]=P-aq;aq=J-R;P=ag-T;g[f>>2]=H+aq;g[ad>>2]=H-aq;aq=J+R;R=ag+T;g[af>>2]=P+N;g[am>>2]=P-N;g[x>>2]=R+aq;g[D>>2]=R-aq;aq=K+S;R=K-S;S=O+W;K=O-W;g[b>>2]=aq+S;g[u>>2]=aq-S;S=$-ap;aq=X-ai;g[e>>2]=R+S;g[n>>2]=R-S;S=$+ap;ap=X+ai;g[k>>2]=aq+K;g[A>>2]=aq-K;g[w>>2]=ap+S;g[s>>2]=ap-S;S=aj-o;ap=ar-U;K=U+ar;ar=o+aj;aj=(ap+S)*.7071067690849304;o=(S-ap)*.7071067690849304;ap=aa-l;S=Q-ak;U=Q+ak;ak=l+aa;aa=M-ab;l=j-V;Q=M+ab;ab=j+V;V=(aa-l)*.7071067690849304;j=(aa+l)*.7071067690849304;l=I-Y;aa=m-L;M=I+Y;Y=m+L;L=l+ap;m=l-ap;ap=V+aj;l=V-aj;g[z>>2]=L+ap;g[F>>2]=L-ap;ap=j-o;L=aa-S;g[al>>2]=m+ap;g[ae>>2]=m-ap;ap=j+o;o=aa+S;g[ah>>2]=L+l;g[ao>>2]=L-l;g[C>>2]=o+ap;g[_>>2]=o-ap;ap=M+U;o=M-U;U=Q+K;M=Q-K;g[a>>2]=ap+U;g[v>>2]=ap-U;U=ab-ar;ap=Y-ak;g[p>>2]=o+U;g[r>>2]=o-U;U=ab+ar;ar=Y+ak;g[i>>2]=ap+M;g[B>>2]=ap-M;g[h>>2]=ar+U;g[t>>2]=ar-U;G=G+32|0;}while((G|0)<(d|0));return}function FD(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0.0,l=0.0,m=0.0,n=0.0,o=0.0,p=0.0,q=0.0,r=0.0,s=0.0,t=0.0,u=0;f=a>>1;h=d;d=e;i=e+(f<<2)|0;j=b+(a<<2)|0;while(1){a=(c[h>>2]|0)+f|0;b=(c[h+4>>2]|0)+f|0;k=+g[e+(a+1<<2)>>2];l=+g[e+(b+1<<2)>>2];m=k-l;n=+g[e+(a<<2)>>2];o=+g[e+(b<<2)>>2];p=n+o;q=+g[j>>2];r=+g[j+4>>2];s=p*q+m*r;t=p*r-m*q;b=i-16|0;q=(k+l)*.5;l=(n-o)*.5;g[d>>2]=q+s;g[i-8>>2]=q-s;g[d+4>>2]=l+t;g[i-4>>2]=t-l;a=(c[h+8>>2]|0)+f|0;u=(c[h+12>>2]|0)+f|0;l=+g[e+(a+1<<2)>>2];t=+g[e+(u+1<<2)>>2];s=l-t;q=+g[e+(a<<2)>>2];o=+g[e+(u<<2)>>2];n=q+o;k=+g[j+8>>2];m=+g[j+12>>2];r=n*k+s*m;p=n*m-s*k;k=(l+t)*.5;t=(q-o)*.5;g[d+8>>2]=k+r;g[b>>2]=k-r;g[d+12>>2]=t+p;g[i-12>>2]=p-t;u=d+16|0;if(u>>>0<b>>>0){h=h+16|0;d=u;i=b;j=j+16|0}else{break}}return}function FE(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0.0,C=0.0,D=0,E=0,F=0,G=0,H=0,I=0,J=0;e=i;f=a|0;h=c[f>>2]|0;j=h>>1;k=h>>2;l=h>>3;m=i;i=i+(h<<2)|0;i=i+7&-8;n=m;m=n+(j<<2)|0;o=j+k|0;p=b+(o<<2)|0;q=a+8|0;r=c[q>>2]|0;s=r+(j<<2)|0;if((l|0)>0){t=(l-1|0)>>>1;u=t<<1;v=j-2-u|0;w=o-4-(t<<2)|0;t=p;x=b+(o+1<<2)|0;o=s;y=0;while(1){z=t-16|0;A=o-8|0;B=+g[t-8>>2]+ +g[x>>2];C=+g[z>>2]+ +g[x+8>>2];D=o-4|0;g[n+(y+j<<2)>>2]=C*+g[D>>2]+B*+g[A>>2];g[n+((y|1)+j<<2)>>2]=C*+g[A>>2]-B*+g[D>>2];D=y+2|0;if((D|0)<(l|0)){t=z;x=x+16|0;o=A;y=D}else{break}}E=b+(w<<2)|0;F=r+(v<<2)|0;G=u+2|0}else{E=p;F=s;G=0}s=b+4|0;p=j-l|0;if((G|0)<(p|0)){l=E;E=s;u=F;v=G;while(1){w=u-8|0;y=l-16|0;B=+g[l-8>>2]- +g[E>>2];C=+g[y>>2]- +g[E+8>>2];o=u-4|0;g[n+(v+j<<2)>>2]=C*+g[o>>2]+B*+g[w>>2];g[n+((v|1)+j<<2)>>2]=C*+g[w>>2]-B*+g[o>>2];o=E+16|0;x=v+2|0;if((x|0)<(p|0)){l=y;E=o;u=w;v=x}else{H=o;I=w;J=x;break}}}else{H=s;I=F;J=G}if((J|0)<(j|0)){G=b+(h<<2)|0;h=H;H=I;I=J;while(1){J=H-8|0;b=G-16|0;B=-0.0- +g[G-8>>2]- +g[h>>2];C=-0.0- +g[b>>2]- +g[h+8>>2];F=H-4|0;g[n+(I+j<<2)>>2]=C*+g[F>>2]+B*+g[J>>2];g[n+((I|1)+j<<2)>>2]=C*+g[J>>2]-B*+g[F>>2];F=I+2|0;if((F|0)<(j|0)){G=b;h=h+16|0;H=J;I=F}else{break}}}FC(c[a+4>>2]|0,r,m,j);FD(c[f>>2]|0,c[q>>2]|0,c[a+12>>2]|0,n);if((k|0)<=0){i=e;return}f=a+16|0;a=n;n=d+(j<<2)|0;m=(c[q>>2]|0)+(j<<2)|0;j=0;while(1){q=n-4|0;r=a+4|0;I=m+4|0;g[d+(j<<2)>>2]=+g[f>>2]*(+g[a>>2]*+g[m>>2]+ +g[r>>2]*+g[I>>2]);g[q>>2]=+g[f>>2]*(+g[a>>2]*+g[I>>2]- +g[r>>2]*+g[m>>2]);r=j+1|0;if((r|0)<(k|0)){a=a+8|0;n=q;m=m+8|0;j=r}else{break}}i=e;return}function FF(a){a=a|0;var b=0,d=0;b=(c[a+28>>2]|0)+2868|0;d=NE(1,36)|0;c[d+4>>2]=c[a+4>>2];g[d>>2]=-9999.0;c[d+8>>2]=b;return d|0}function FG(a){a=a|0;if((a|0)==0){return}ND(a);return}function FH(a){a=a|0;if((a|0)==0){return}ND(a);return}function FI(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0.0,k=0.0,l=0.0,m=0.0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0.0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0.0,E=0,F=0,G=0;NM(a|0,0,52);h=d|0;c[a+36>>2]=c[h>>2];d=~~(+bR(+(+aa(+(+(c[h>>2]|0)*8.0))/.6931471805599453))+-1.0);i=a+32|0;c[i>>2]=d;j=+(f|0);k=+(e|0);l=+(1<<d+1|0);m=(+aa(+(j*.25*.5/k))*1.4426950216293335+ -5.965784072875977)*l;d=~~(m- +(c[h>>2]|0));c[a+28>>2]=d;c[a+40>>2]=1-d+~~((+aa(+((+(e|0)+.25)*j*.5/k))*1.4426950216293335+ -5.965784072875977)*l+.5);d=e<<2;h=a+16|0;c[h>>2]=NC(d)|0;n=a+20|0;c[n>>2]=NC(d)|0;o=a+24|0;c[o>>2]=NC(d)|0;p=a+4|0;c[p>>2]=b;c[a>>2]=e;c[a+44>>2]=f;q=a+48|0;g[q>>2]=1.0;do{if((f|0)<26e3){g[q>>2]=0.0}else{if((f|0)<38e3){g[q>>2]=.9399999976158142;break}if((f|0)<=46e3){break}g[q>>2]=1.274999976158142}}while(0);l=+(f|0);q=0;r=0;L4732:while(1){s=q;while(1){if((s|0)>=87){break L4732}t=s+1|0;u=~~+bR(+(k*+$(+((+(t|0)*.125+-2.0+5.965784072875977)*.6931470036506653))*2.0/l));v=+g[1991552+(s<<2)>>2];if((r|0)<(u|0)){break}else{s=t}}j=(+g[1991552+(t<<2)>>2]-v)/+(u-r|0);if((r|0)>=(e|0)){q=t;r=r;continue}s=r-u|0;w=r-e|0;x=s>>>0>w>>>0?s:w;m=v;w=r;while(1){g[(c[h>>2]|0)+(w<<2)>>2]=m+100.0;s=w+1|0;if(!((s|0)<(u|0)&(s|0)<(e|0))){break}m=j+m;w=s}q=t;r=r-x|0}if((r|0)<(e|0)){t=r;do{r=c[h>>2]|0;g[r+(t<<2)>>2]=+g[r+(t-1<<2)>>2];t=t+1|0;}while((t|0)<(e|0))}t=(e|0)>0;do{if(t){h=(f|0)/(e<<1|0)|0;r=b+120|0;q=b+112|0;u=b+124|0;w=b+116|0;s=1;y=-99;z=0;while(1){A=ac(h,z)|0;v=+(A|0);m=+Z(+(v*.0007399999885819852))*13.100000381469727;j=m+ +Z(+(+(ac(A,A)|0)*1.8499999754340024e-8))*2.240000009536743+v*9999999747378752.0e-20;A=c[r>>2]|0;B=y;while(1){if((A+B|0)>=(z|0)){break}C=ac(B,h)|0;v=+(C|0);m=+Z(+(v*.0007399999885819852))*13.100000381469727;D=v*9999999747378752.0e-20+(m+ +Z(+(+(ac(C,C)|0)*1.8499999754340024e-8))*2.240000009536743);if(D<j- +g[q>>2]){B=B+1|0}else{break}}L4756:do{if((s|0)>(e|0)){E=s}else{A=(c[u>>2]|0)+z|0;C=s;while(1){if((C|0)>=(A|0)){F=ac(C,h)|0;D=+(F|0);m=+Z(+(D*.0007399999885819852))*13.100000381469727;v=D*9999999747378752.0e-20+(m+ +Z(+(+(ac(F,F)|0)*1.8499999754340024e-8))*2.240000009536743);if(v>=j+ +g[w>>2]){E=C;break L4756}}F=C+1|0;if((F|0)>(e|0)){E=F;break}else{C=F}}}}while(0);c[(c[o>>2]|0)+(z<<2)>>2]=(B<<16)-65537+E;C=z+1|0;if((C|0)<(e|0)){s=E;y=B;z=C}else{break}}if(t){G=0}else{break}do{j=+aa(+(l*(+(G|0)+.25)*.5/k))*1.4426950216293335+ -5.965784072875977;c[(c[n>>2]|0)+(G<<2)>>2]=~~(j*+(1<<(c[i>>2]|0)+1|0)+.5);G=G+1|0;}while((G|0)<(e|0))}}while(0);c[a+8>>2]=FJ(b+36|0,l*.5/k,e,+g[b+24>>2],+g[b+28>>2])|0;b=NC(12)|0;G=b;i=a+12|0;c[i>>2]=G;a=NC(d)|0;c[G>>2]=a;c[b+4>>2]=NC(d)|0;c[b+8>>2]=NC(d)|0;if(!t){return}j=k*2.0;t=0;d=a;while(1){k=(+aa(+(l*(+(t|0)+.5)/j))*1.4426950216293335+ -5.965784072875977)*2.0;v=k<0.0?0.0:k;k=v<16.0?v:16.0;a=~~k;v=k- +(a|0);k=1.0-v;b=a+1|0;G=c[p>>2]|0;g[d+(t<<2)>>2]=k*+g[G+132+(a<<2)>>2]+v*+g[G+132+(b<<2)>>2];G=c[p>>2]|0;g[(c[(c[i>>2]|0)+4>>2]|0)+(t<<2)>>2]=k*+g[G+200+(a<<2)>>2]+v*+g[G+200+(b<<2)>>2];G=c[p>>2]|0;g[(c[(c[i>>2]|0)+8>>2]|0)+(t<<2)>>2]=k*+g[G+268+(a<<2)>>2]+v*+g[G+268+(b<<2)>>2];b=t+1|0;if((b|0)>=(e|0)){break}t=b;d=c[c[i>>2]>>2]|0}return}function FJ(a,b,d,e,f){a=a|0;b=+b;d=d|0;e=+e;f=+f;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0.0,w=0,x=0.0,y=0.0,z=0.0,A=0.0,B=0.0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0;h=i;i=i+32480|0;j=h|0;k=h+224|0;l=h+30688|0;m=i;i=i+(d<<2)|0;i=i+7&-8;n=NC(68)|0;NM(k|0,0,30464);o=e>0.0;p=e<0.0;q=j;r=0;do{s=r<<2;t=0;do{u=t+s|0;if((u|0)<88){v=+g[1991552+(u<<2)>>2]}else{v=-30.0}w=u+1|0;do{if((w|0)<88){x=+g[1991552+(w<<2)>>2];if(v<=x){y=v;break}y=x}else{if(v<=-30.0){y=v;break}y=-30.0}}while(0);w=u+2|0;do{if((w|0)<88){x=+g[1991552+(w<<2)>>2];if(y<=x){z=y;break}z=x}else{if(y<=-30.0){z=y;break}z=-30.0}}while(0);w=u+3|0;do{if((w|0)<88){x=+g[1991552+(w<<2)>>2];if(z<=x){A=z;break}A=x}else{if(z<=-30.0){A=z;break}A=-30.0}}while(0);g[j+(t<<2)>>2]=A;t=t+1|0;}while((t|0)<56);t=k+(r*1792|0)+448|0;s=41496+(r*1344|0)|0;NN(t|0,s|0,224)|0;t=k+(r*1792|0)+672|0;w=41720+(r*1344|0)|0;NN(t|0,w|0,224)|0;w=k+(r*1792|0)+896|0;t=41944+(r*1344|0)|0;NN(w|0,t|0,224)|0;t=k+(r*1792|0)+1120|0;w=42168+(r*1344|0)|0;NN(t|0,w|0,224)|0;w=k+(r*1792|0)+1344|0;t=42392+(r*1344|0)|0;NN(w|0,t|0,224)|0;t=k+(r*1792|0)+1568|0;w=42616+(r*1344|0)|0;NN(t|0,w|0,224)|0;w=k+(r*1792|0)|0;NN(w|0,s|0,224)|0;w=k+(r*1792|0)+224|0;NN(w|0,s|0,224)|0;if(o){s=0;do{if(p){w=0;do{t=16-w|0;x=+(((t|0)>-1?t:-t|0)|0)*f+e;B=x<0.0?0.0:x;t=k+(r*1792|0)+(s*224|0)+(w<<2)|0;g[t>>2]=(B>0.0?0.0:B)+ +g[t>>2];w=w+1|0;}while((w|0)<56)}else{w=0;do{t=16-w|0;B=+(((t|0)>-1?t:-t|0)|0)*f+e;t=k+(r*1792|0)+(s*224|0)+(w<<2)|0;g[t>>2]=(B<0.0?0.0:B)+ +g[t>>2];w=w+1|0;}while((w|0)<56)}s=s+1|0;}while((s|0)<8)}else{s=0;do{if(p){w=0;do{t=16-w|0;B=+(((t|0)>-1?t:-t|0)|0)*f+e;t=k+(r*1792|0)+(s*224|0)+(w<<2)|0;g[t>>2]=(B>0.0?0.0:B)+ +g[t>>2];w=w+1|0;}while((w|0)<56)}else{w=0;do{t=16-w|0;u=k+(r*1792|0)+(s*224|0)+(w<<2)|0;g[u>>2]=+(((t|0)>-1?t:-t|0)|0)*f+e+ +g[u>>2];w=w+1|0;}while((w|0)<56)}s=s+1|0;}while((s|0)<8)}B=+g[a+(r<<2)>>2]+100.0;s=0;while(1){x=B-((s|0)<2?20.0:+(s|0)*10.0)+-30.0;w=0;do{u=k+(r*1792|0)+(s*224|0)+(w<<2)|0;g[u>>2]=x+ +g[u>>2];w=w+1|0;}while((w|0)<56);w=l+(s*224|0)|0;NN(w|0,q|0,224)|0;x=100.0- +(s|0)*10.0+-30.0;w=0;while(1){u=l+(s*224|0)+(w<<2)|0;g[u>>2]=x+ +g[u>>2];u=w+1|0;if((u|0)<56){w=u}else{C=0;break}}do{x=+g[k+(r*1792|0)+(s*224|0)+(C<<2)>>2];w=l+(s*224|0)+(C<<2)|0;if(x>+g[w>>2]){g[w>>2]=x}C=C+1|0;}while((C|0)<56);w=s+1|0;if((w|0)<8){s=w}else{D=1;break}}do{s=D-1|0;w=0;while(1){B=+g[l+(s*224|0)+(w<<2)>>2];u=l+(D*224|0)+(w<<2)|0;if(B<+g[u>>2]){g[u>>2]=B}u=w+1|0;if((u|0)<56){w=u}else{E=0;break}}do{B=+g[l+(D*224|0)+(E<<2)>>2];w=k+(r*1792|0)+(D*224|0)+(E<<2)|0;if(B<+g[w>>2]){g[w>>2]=B}E=E+1|0;}while((E|0)<56);D=D+1|0;}while((D|0)<8);r=r+1|0;}while((r|0)<17);r=m;m=n;e=b;n=(d|0)>0;D=~d;E=0;do{l=NC(32)|0;c[m+(E<<2)>>2]=l;f=+(E|0)*.5;C=~~+Q(+(+$(+((f+5.965784072875977)*.6931470036506653))/e));q=~~+ab(+((+aa(+(+(C|0)*b+1.0))*1.4426950216293335+ -5.965784072875977)*2.0));a=~~+Q(+((+aa(+(+(C+1|0)*b))*1.4426950216293335+ -5.965784072875977)*2.0));C=(q|0)>(E|0)?E:q;q=(C|0)<0?0:C;C=(a|0)>16?16:a;a=(q|0)>(C|0);E=E+1|0;p=(E|0)<17;o=0;do{j=l+(o<<2)|0;c[j>>2]=NC(232)|0;if(n){w=0;do{g[r+(w<<2)>>2]=999.0;w=w+1|0;}while((w|0)<(d|0))}if(!a){w=q;do{A=+(w|0)*.5;s=0;u=0;while(1){z=A+ +(s|0)*.125;t=~~(+$(+((z+ -2.0625+5.965784072875977)*.6931470036506653))/e);F=~~(+$(+((z+ -1.9375+5.965784072875977)*.6931470036506653))/e+1.0);G=(t|0)<0?0:t;H=(G|0)>(d|0)?d:G;G=(H|0)<(u|0)?H:u;H=(F|0)<0?0:F;I=(H|0)>(d|0)?d:H;if((G|0)<(I|0)&(G|0)<(d|0)){z=+g[k+(w*1792|0)+(o*224|0)+(s<<2)>>2];H=~u;J=(H|0)>(D|0)?H:D;H=(t|0)>0?~t:-1;t=(J|0)>(H|0)?J:H;H=(F|0)>0?~F:-1;F=((H|0)<(D|0)?D:H)-t|0;H=~(t+d);J=F>>>0>H>>>0?F:H;H=G;do{F=r+(H<<2)|0;if(+g[F>>2]>z){g[F>>2]=z}H=H+1|0;}while((H|0)<(I|0)&(H|0)<(d|0));K=~t-J|0}else{K=G}H=s+1|0;if((H|0)<56){s=H;u=K}else{break}}if((K|0)<(d|0)){A=+g[k+(w*1792|0)+(o*224|0)+220>>2];u=K;do{s=r+(u<<2)|0;if(+g[s>>2]>A){g[s>>2]=A}u=u+1|0;}while((u|0)<(d|0))}w=w+1|0;}while((w|0)<=(C|0))}do{if(p){w=0;u=0;while(1){A=f+ +(w|0)*.125;s=~~(+$(+((A+ -2.0625+5.965784072875977)*.6931470036506653))/e);H=~~(+$(+((A+ -1.9375+5.965784072875977)*.6931470036506653))/e+1.0);I=(s|0)<0?0:s;F=(I|0)>(d|0)?d:I;I=(F|0)<(u|0)?F:u;F=(H|0)<0?0:H;L=(F|0)>(d|0)?d:F;if((I|0)<(L|0)&(I|0)<(d|0)){A=+g[k+(E*1792|0)+(o*224|0)+(w<<2)>>2];F=~u;M=(F|0)>(D|0)?F:D;F=(s|0)>0?~s:-1;s=(M|0)>(F|0)?M:F;F=(H|0)>0?~H:-1;H=((F|0)<(D|0)?D:F)-s|0;F=~(s+d);M=H>>>0>F>>>0?H:F;F=I;do{H=r+(F<<2)|0;if(+g[H>>2]>A){g[H>>2]=A}F=F+1|0;}while((F|0)<(L|0)&(F|0)<(d|0));N=~s-M|0}else{N=I}F=w+1|0;if((F|0)<56){w=F;u=N}else{break}}if((N|0)>=(d|0)){O=0;break}A=+g[k+(E*1792|0)+(o*224|0)+220>>2];u=N;while(1){w=r+(u<<2)|0;if(+g[w>>2]>A){g[w>>2]=A}w=u+1|0;if((w|0)<(d|0)){u=w}else{O=0;break}}}else{O=0}}while(0);while(1){u=~~(+$(+((f+ +(O|0)*.125+-2.0+5.965784072875977)*.6931470036506653))/e);do{if((u|0)<0){g[(c[l+(o<<2)>>2]|0)+(O+2<<2)>>2]=-999.0}else{if((u|0)<(d|0)){g[(c[j>>2]|0)+(O+2<<2)>>2]=+g[r+(u<<2)>>2];break}else{g[(c[j>>2]|0)+(O+2<<2)>>2]=-999.0;break}}}while(0);u=O+1|0;if((u|0)<56){O=u}else{P=0;break}}while(1){if((P|0)>=16){break}if(+g[(c[l+(o<<2)>>2]|0)+(P+2<<2)>>2]>-200.0){break}else{P=P+1|0}}g[c[j>>2]>>2]=+(P|0);u=55;while(1){if((u|0)<=17){break}if(+g[(c[l+(o<<2)>>2]|0)+(u+2<<2)>>2]>-200.0){break}else{u=u-1|0}}g[(c[j>>2]|0)+4>>2]=+(u|0);o=o+1|0;}while((o|0)<8)}while(p);i=h;return m|0}function FK(a){a=a|0;var b=0,d=0,e=0,f=0;if((a|0)==0){return}b=c[a+16>>2]|0;if((b|0)!=0){ND(b)}b=c[a+20>>2]|0;if((b|0)!=0){ND(b)}b=c[a+24>>2]|0;if((b|0)!=0){ND(b)}b=a+8|0;d=c[b>>2]|0;if((d|0)!=0){e=0;f=d;do{ND(c[c[f+(e<<2)>>2]>>2]|0);ND(c[(c[(c[b>>2]|0)+(e<<2)>>2]|0)+4>>2]|0);ND(c[(c[(c[b>>2]|0)+(e<<2)>>2]|0)+8>>2]|0);ND(c[(c[(c[b>>2]|0)+(e<<2)>>2]|0)+12>>2]|0);ND(c[(c[(c[b>>2]|0)+(e<<2)>>2]|0)+16>>2]|0);ND(c[(c[(c[b>>2]|0)+(e<<2)>>2]|0)+20>>2]|0);ND(c[(c[(c[b>>2]|0)+(e<<2)>>2]|0)+24>>2]|0);ND(c[(c[(c[b>>2]|0)+(e<<2)>>2]|0)+28>>2]|0);ND(c[(c[b>>2]|0)+(e<<2)>>2]|0);e=e+1|0;f=c[b>>2]|0}while((e|0)<17);ND(f)}f=a+12|0;e=c[f>>2]|0;if((e|0)!=0){ND(c[e>>2]|0);ND(c[(c[f>>2]|0)+4>>2]|0);ND(c[(c[f>>2]|0)+8>>2]|0);ND(c[f>>2]|0)}NM(a|0,0,52);return}function FL(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0;e=i;f=c[a>>2]|0;h=i;i=i+(f<<2)|0;i=i+7&-8;j=h;h=a+24|0;FM(f,c[h>>2]|0,b,d,140.0,-1);k=(f|0)>0;if(k){l=0;do{g[j+(l<<2)>>2]=+g[b+(l<<2)>>2]- +g[d+(l<<2)>>2];l=l+1|0;}while((l|0)<(f|0))}l=a+4|0;FM(f,c[h>>2]|0,j,d,0.0,c[(c[l>>2]|0)+128>>2]|0);if(k){m=0}else{i=e;return}do{h=j+(m<<2)|0;g[h>>2]=+g[b+(m<<2)>>2]- +g[h>>2];m=m+1|0;}while((m|0)<(f|0));if(k){n=0}else{i=e;return}do{k=d+(n<<2)|0;m=~~(+g[k>>2]+.5);b=(m|0)>39?39:m;g[k>>2]=+g[j+(n<<2)>>2]+ +g[(c[l>>2]|0)+336+(((b|0)<0?0:b)<<2)>>2];n=n+1|0;}while((n|0)<(f|0));i=e;return}function FM(a,b,d,e,f,h){a=a|0;b=b|0;d=d|0;e=e|0;f=+f;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0.0,y=0.0,z=0.0,A=0.0,B=0.0,C=0.0,D=0,E=0,F=0.0,G=0.0,H=0.0,I=0.0,J=0,K=0,L=0,M=0,N=0,O=0.0,P=0.0,Q=0.0,R=0.0,S=0,T=0.0,U=0.0,V=0.0,W=0.0,X=0.0,Y=0.0,Z=0.0,_=0.0,$=0.0,aa=0.0,ab=0,ac=0.0,ad=0.0,ae=0.0,af=0.0,ag=0,ah=0.0;j=i;k=a<<2;l=i;i=i+k|0;i=i+7&-8;m=l;l=i;i=i+k|0;i=i+7&-8;n=l;l=i;i=i+k|0;i=i+7&-8;o=l;l=i;i=i+k|0;i=i+7&-8;p=l;l=i;i=i+k|0;i=i+7&-8;k=l;q=+g[d>>2]+f;r=q<1.0?1.0:q;q=r*r*.5;s=q+0.0;t=r*q+0.0;g[m>>2]=s;g[n>>2]=s;g[o>>2]=0.0;g[p>>2]=t;g[k>>2]=0.0;if((a|0)>1){q=s;r=s;s=0.0;u=t;t=0.0;l=1;v=1.0;while(1){w=+g[d+(l<<2)>>2]+f;x=w<1.0?1.0:w;w=x*x;y=q+w;z=v*w;A=r+z;B=s+v*z;C=u+x*w;w=t+x*z;g[m+(l<<2)>>2]=y;g[n+(l<<2)>>2]=A;g[o+(l<<2)>>2]=B;g[p+(l<<2)>>2]=C;g[k+(l<<2)>>2]=w;D=l+1|0;if((D|0)<(a|0)){q=y;r=A;s=B;u=C;t=w;l=D;v=v+1.0}else{break}}}l=c[b>>2]|0;d=l>>16;if((d|0)>-1){E=0;F=0.0;G=0.0;H=1.0;I=0.0;J=l}else{D=0;v=0.0;K=l;l=d;while(1){d=K&65535;L=-l|0;t=+g[m+(d<<2)>>2]+ +g[m+(L<<2)>>2];u=+g[n+(d<<2)>>2]- +g[n+(L<<2)>>2];s=+g[o+(d<<2)>>2]+ +g[o+(L<<2)>>2];r=+g[p+(d<<2)>>2]+ +g[p+(L<<2)>>2];q=+g[k+(d<<2)>>2]- +g[k+(L<<2)>>2];w=s*r-u*q;C=t*q-u*r;r=t*s-u*u;u=(w+v*C)/r;g[e+(D<<2)>>2]=(u<0.0?0.0:u)-f;L=D+1|0;u=v+1.0;d=c[b+(L<<2)>>2]|0;M=d>>16;if((M|0)>-1){E=L;F=w;G=C;H=r;I=u;J=d;break}else{D=L;v=u;K=d;l=M}}}l=J&65535;if((l|0)<(a|0)){K=E;v=I;D=J;J=l;while(1){l=D>>16;u=+g[m+(J<<2)>>2]- +g[m+(l<<2)>>2];r=+g[n+(J<<2)>>2]- +g[n+(l<<2)>>2];C=+g[o+(J<<2)>>2]- +g[o+(l<<2)>>2];w=+g[p+(J<<2)>>2]- +g[p+(l<<2)>>2];s=+g[k+(J<<2)>>2]- +g[k+(l<<2)>>2];t=C*w-r*s;q=u*s-r*w;w=u*C-r*r;r=(t+v*q)/w;g[e+(K<<2)>>2]=(r<0.0?0.0:r)-f;l=K+1|0;r=v+1.0;M=c[b+(l<<2)>>2]|0;d=M&65535;if((d|0)<(a|0)){K=l;v=r;D=M;J=d}else{N=l;O=t;P=q;Q=w;R=r;break}}}else{N=E;O=F;P=G;Q=H;R=I}if((N|0)<(a|0)){E=N;I=R;while(1){R=(O+P*I)/Q;g[e+(E<<2)>>2]=(R<0.0?0.0:R)-f;N=E+1|0;if((N|0)<(a|0)){E=N;I=I+1.0}else{break}}}if((h|0)<1){i=j;return}E=(h|0)/2|0;N=E-h|0;if((N|0)>-1){S=0;T=O;U=P;V=Q;W=0.0}else{J=0;Q=0.0;D=E;K=N;do{N=-K|0;P=+g[m+(D<<2)>>2]+ +g[m+(N<<2)>>2];O=+g[n+(D<<2)>>2]- +g[n+(N<<2)>>2];I=+g[o+(D<<2)>>2]+ +g[o+(N<<2)>>2];R=+g[p+(D<<2)>>2]+ +g[p+(N<<2)>>2];H=+g[k+(D<<2)>>2]- +g[k+(N<<2)>>2];X=I*R-O*H;Y=P*H-O*R;Z=P*I-O*O;O=(X+Q*Y)/Z-f;N=e+(J<<2)|0;if(O<+g[N>>2]){g[N>>2]=O}J=J+1|0;Q=Q+1.0;D=E+J|0;K=D-h|0;}while((K|0)<=-1);S=h-E|0;T=X;U=Y;V=Z;W=Q}K=S+E|0;if((K|0)<(a|0)){D=S;Q=W;J=K;do{K=J-h|0;Z=+g[m+(J<<2)>>2]- +g[m+(K<<2)>>2];Y=+g[n+(J<<2)>>2]- +g[n+(K<<2)>>2];X=+g[o+(J<<2)>>2]- +g[o+(K<<2)>>2];O=+g[p+(J<<2)>>2]- +g[p+(K<<2)>>2];I=+g[k+(J<<2)>>2]- +g[k+(K<<2)>>2];_=X*O-Y*I;$=Z*I-Y*O;aa=Z*X-Y*Y;Y=(_+Q*$)/aa-f;K=e+(D<<2)|0;if(Y<+g[K>>2]){g[K>>2]=Y}D=D+1|0;Q=Q+1.0;J=D+E|0;}while((J|0)<(a|0));ab=a-E|0;ac=_;ad=$;ae=aa;af=Q}else{ab=S;ac=T;ad=U;ae=V;af=W}if((ab|0)<(a|0)){ag=ab;ah=af}else{i=j;return}while(1){af=(ac+ad*ah)/ae-f;ab=e+(ag<<2)|0;if(af<+g[ab>>2]){g[ab>>2]=af}ab=ag+1|0;if((ab|0)<(a|0)){ag=ab;ah=ah+1.0}else{break}}i=j;return}function FN(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=+e;f=+f;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0,s=0.0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0.0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0.0,Q=0.0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0;h=i;j=a|0;k=c[j>>2]|0;l=a+40|0;m=c[l>>2]|0;n=i;i=i+(m<<2)|0;i=i+7&-8;o=n;n=a+4|0;p=c[n>>2]|0;q=+g[p+4>>2]+f;if((m|0)>0){r=0;do{g[o+(r<<2)>>2]=-9999.0;r=r+1|0;}while((r|0)<(m|0))}f=+g[p+8>>2];s=q<f?f:q;do{if((k|0)>0){p=a+16|0;m=0;do{g[d+(m<<2)>>2]=s+ +g[(c[p>>2]|0)+(m<<2)>>2];m=m+1|0;}while((m|0)<(k|0));m=c[j>>2]|0;p=c[a+8>>2]|0;q=+g[(c[n>>2]|0)+496>>2]-e;if((m|0)<=0){t=3590;break}r=a+20|0;u=a+32|0;v=a+28|0;w=a+36|0;x=c[r>>2]|0;y=0;while(1){z=c[x+(y<<2)>>2]|0;f=+g[b+(y<<2)>>2];A=y;L5012:while(1){B=A;while(1){C=B+1|0;if((C|0)>=(m|0)){D=0;break L5012}if((c[x+(C<<2)>>2]|0)!=(z|0)){D=1;break L5012}E=+g[b+(C<<2)>>2];if(E>f){f=E;A=C;continue L5012}else{B=C}}}L5019:do{if(f+6.0>+g[d+(B<<2)>>2]){A=z>>c[u>>2];F=(A|0)>16?16:A;A=c[l>>2]|0;G=c[w>>2]|0;H=~~((q+f+-30.0)*.10000000149011612);I=(H|0)<0?0:H;H=c[(c[p+(((F|0)<0?0:F)<<2)>>2]|0)+(((I|0)>7?7:I)<<2)>>2]|0;I=~~+g[H+4>>2];E=+g[H>>2];F=~~E;J=~~(+((c[x+(B<<2)>>2]|0)-(c[v>>2]|0)|0)+ +(G|0)*(E+-16.0)- +(G>>1|0));while(1){if((F|0)>=(I|0)){break L5019}do{if((J|0)>0){E=f+ +g[H+(F+2<<2)>>2];K=o+(J<<2)|0;if(+g[K>>2]>=E){break}g[K>>2]=E}}while(0);K=J+G|0;if((K|0)<(A|0)){F=F+1|0;J=K}else{break}}}}while(0);if(D){y=C}else{L=w;M=r;N=v;break}}}else{t=3590}}while(0);if((t|0)==3590){L=a+36|0;M=a+20|0;N=a+28|0}a=c[L>>2]|0;FT(o,a,c[l>>2]|0);L=c[j>>2]|0;L5032:do{if((L|0)>1){t=c[M>>2]|0;C=c[t>>2]|0;D=c[N>>2]|0;B=C-(a>>1)-D|0;b=0;k=1;v=L;r=t;t=C;C=D;while(1){e=+g[o+(B<<2)>>2];D=((c[r+(k<<2)>>2]|0)+t>>1)-C|0;s=+g[(c[n>>2]|0)+32>>2];q=e>s?s:e;w=B+1|0;L5036:do{if((w|0)>(D|0)){O=B;P=q}else{e=q;y=w;while(1){x=e==-9999.0;p=y;while(1){Q=+g[o+(p<<2)>>2];if(Q>-9999.0){if(Q<e|x){break}}else{if(x){break}}u=p+1|0;if((u|0)>(D|0)){O=p;P=e;break L5036}else{p=u}}x=p+1|0;if((x|0)>(D|0)){O=p;P=Q;break}else{e=Q;y=x}}}}while(0);D=O+C|0;L5047:do{if((b|0)>=(v|0)|(t|0)>(D|0)){R=b;S=v}else{w=v;y=b;while(1){x=d+(y<<2)|0;if(+g[x>>2]<P){g[x>>2]=P;T=c[j>>2]|0}else{T=w}x=y+1|0;if((x|0)>=(T|0)){R=x;S=T;break L5047}if((c[(c[M>>2]|0)+(x<<2)>>2]|0)>(D|0)){R=x;S=T;break}else{w=T;y=x}}}}while(0);D=R+1|0;if((D|0)>=(S|0)){U=R;V=S;break L5032}y=c[M>>2]|0;B=O;b=R;k=D;v=S;r=y;t=c[y+(R<<2)>>2]|0;C=c[N>>2]|0}}else{U=0;V=L}}while(0);P=+g[o+((c[l>>2]|0)-1<<2)>>2];if((U|0)<(V|0)){W=U;X=V}else{i=h;return}while(1){V=d+(W<<2)|0;if(+g[V>>2]<P){g[V>>2]=P;Y=c[j>>2]|0}else{Y=X}V=W+1|0;if((V|0)<(Y|0)){W=V;X=Y}else{break}}i=h;return}function FO(a,b,d,e,f,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0.0,n=0,o=0,p=0.0,q=0,r=0.0,s=0.0,t=0.0,u=0.0;j=c[a>>2]|0;k=a+4|0;l=c[k>>2]|0;m=+g[l+12+(e<<2)>>2];if((j|0)<=0){return}n=a+12|0;o=(e|0)==1;p=+g[a+48>>2];a=0;q=l;while(1){r=+g[b+(a<<2)>>2]+ +g[(c[(c[n>>2]|0)+(e<<2)>>2]|0)+(a<<2)>>2];s=+g[q+108>>2];t=r>s?s:r;r=m+ +g[d+(a<<2)>>2];g[f+(a<<2)>>2]=t<r?r:t;if(o){r=t- +g[i+(a<<2)>>2];t=r+17.200000762939453;do{if(r>-17.200000762939453){s=1.0-p*t*.005;if(s>=0.0){u=s;break}u=9999999747378752.0e-20}else{u=1.0-p*t*3.0e-4}}while(0);l=h+(a<<2)|0;g[l>>2]=u*+g[l>>2]}l=a+1|0;if((l|0)>=(j|0)){break}a=l;q=c[k>>2]|0}return}function FP(a,b){a=+a;b=b|0;var d=0,e=0,f=0.0;d=c[b+4>>2]|0;e=c[d+28>>2]|0;f=+((c[e+(c[b+40>>2]<<2)>>2]|0)/2|0|0)/+(c[d+8>>2]|0)*+g[e+2936>>2]+a;return+(f<-9999.0?-9999.0:f)}function FQ(a,b,d,e,f,j,k,l,m){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0.0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0.0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,S=0,T=0.0,U=0,V=0.0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0.0,ak=0.0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0;n=i;o=c[d>>2]|0;p=d+4|0;d=c[p>>2]|0;if((c[d+500>>2]|0)==0){q=16}else{q=c[d+508>>2]|0}r=c[b+132+((c[d>>2]|0)*60|0)+(a<<2)>>2]|0;s=+h[73104+(c[b+252+(a<<2)>>2]<<3)>>3];d=m<<2;t=i;i=i+d|0;i=i+7&-8;u=t;t=i;i=i+d|0;i=i+7&-8;v=t;t=i;i=i+d|0;i=i+7&-8;w=t;t=i;i=i+d|0;i=i+7&-8;x=t;t=i;i=i+d|0;i=i+7&-8;y=t;z=e+1156|0;A=+h[((o|0)>1e3?73032:73104)+(c[b+312+(a<<2)>>2]<<3)>>3];a=ac(d,q)|0;b=i;i=i+a|0;i=i+7&-8;B=b;c[u>>2]=B;b=i;i=i+a|0;i=i+7&-8;C=b;c[v>>2]=C;b=i;i=i+a|0;i=i+7&-8;D=b;c[w>>2]=D;b=i;i=i+a|0;i=i+7&-8;E=b;c[x>>2]=E;L5086:do{if((m|0)>1){b=1;F=B;G=C;H=D;I=E;while(1){J=ac(b,q)|0;c[u+(b<<2)>>2]=F+(J<<2);c[v+(b<<2)>>2]=G+(J<<2);c[w+(b<<2)>>2]=H+(J<<2);c[x+(b<<2)>>2]=I+(J<<2);J=b+1|0;if((J|0)>=(m|0)){break L5086}b=J;F=c[u>>2]|0;G=c[v>>2]|0;H=c[w>>2]|0;I=c[x>>2]|0}}}while(0);E=c[z>>2]|0;if((o|0)>0){D=k;C=c[x>>2]|0;B=(m|0)>0;I=0;while(1){H=o-I|0;G=(q|0)>(H|0)?H:q;NN(t|0,D|0,d)|0;NM(C|0,0,a|0);if(B){H=(G|0)>0;F=r-I|0;b=0;do{J=c[j+(b<<2)>>2]|0;K=J+(I<<2)|0;do{if((c[y+(b<<2)>>2]|0)==0){if(!H){break}L=c[w+(b<<2)>>2]|0;M=c[u+(b<<2)>>2]|0;N=c[v+(b<<2)>>2]|0;O=c[x+(b<<2)>>2]|0;P=0;do{g[L+(P<<2)>>2]=1.000000013351432e-10;g[M+(P<<2)>>2]=0.0;g[N+(P<<2)>>2]=0.0;c[O+(P<<2)>>2]=0;c[J+(P+I<<2)>>2]=0;P=P+1|0;}while((P|0)<(G|0))}else{P=c[w+(b<<2)>>2]|0;do{if(H){O=0;do{g[P+(O<<2)>>2]=+g[1975232+(c[J+(O+I<<2)>>2]<<2)>>2];O=O+1|0;}while((O|0)<(G|0));O=f+(b<<2)|0;N=c[O>>2]|0;M=c[x+(b<<2)>>2]|0;if(H){Q=0}else{S=3660;break}do{T=+R(+(+g[N+(Q+I<<2)>>2]));c[M+(Q<<2)>>2]=T/+g[P+(Q<<2)>>2]>=((Q|0)>=(F|0)?A:s);Q=Q+1|0;}while((Q|0)<(G|0));if(!H){S=3660;break}M=c[u+(b<<2)>>2]|0;N=c[v+(b<<2)>>2]|0;L=0;while(1){U=L+I|0;T=+g[(c[O>>2]|0)+(U<<2)>>2];V=T*T;W=M+(L<<2)|0;g[W>>2]=V;g[N+(L<<2)>>2]=V;if(+g[(c[O>>2]|0)+(U<<2)>>2]<0.0){g[W>>2]=+g[W>>2]*-1.0}W=P+(L<<2)|0;V=+g[W>>2];g[W>>2]=V*V;W=L+1|0;if((W|0)<(G|0)){L=W}else{X=M;Y=N;break}}}else{S=3660}}while(0);if((S|0)==3660){S=0;X=c[u+(b<<2)>>2]|0;Y=c[v+(b<<2)>>2]|0}+FR(c[p>>2]|0,r,X,Y,P,0,I,G,K)}}while(0);b=b+1|0;}while((b|0)<(m|0))}b=c[z>>2]|0;if((b|0)>0){H=(G|0)>0;F=l-I|0;K=r-I|0;J=0;N=b;while(1){M=c[e+1160+(J<<2)>>2]|0;L=c[e+2184+(J<<2)>>2]|0;O=c[j+(M<<2)>>2]|0;W=O+(I<<2)|0;U=c[j+(L<<2)>>2]|0;Z=c[u+(M<<2)>>2]|0;_=c[u+(L<<2)>>2]|0;$=c[v+(M<<2)>>2]|0;aa=c[v+(L<<2)>>2]|0;ab=c[w+(M<<2)>>2]|0;ad=c[w+(L<<2)>>2]|0;ae=c[x+(M<<2)>>2]|0;af=c[x+(L<<2)>>2]|0;ag=y+(M<<2)|0;M=y+(L<<2)|0;if((c[ag>>2]|0)==0){if((c[M>>2]|0)==0){ah=N}else{S=3672}}else{S=3672}if((S|0)==3672){S=0;c[M>>2]=1;c[ag>>2]=1;if(H){ag=0;do{L5134:do{if((ag|0)<(F|0)){M=ae+(ag<<2)|0;L=af+(ag<<2)|0;do{if((c[M>>2]|0)==0){if((c[L>>2]|0)!=0){break}do{if((ag|0)<(K|0)){ai=Z+(ag<<2)|0;V=+g[_+(ag<<2)>>2]+ +g[ai>>2];g[ai>>2]=V;g[$+(ag<<2)>>2]=+R(+V)}else{ai=Z+(ag<<2)|0;V=+g[ai>>2];T=+g[_+(ag<<2)>>2];aj=+R(+V);ak=aj+ +R(+T);g[$+(ag<<2)>>2]=ak;if(V+T<0.0){g[ai>>2]=-0.0-ak;break}else{g[ai>>2]=ak;break}}}while(0);g[aa+(ag<<2)>>2]=0.0;g[_+(ag<<2)>>2]=0.0;c[L>>2]=1;c[U+(ag+I<<2)>>2]=0;break L5134}}while(0);ai=Z+(ag<<2)|0;ak=+R(+(+g[ai>>2]));g[ai>>2]=ak+ +R(+(+g[_+(ag<<2)>>2]));ai=$+(ag<<2)|0;g[ai>>2]=+g[ai>>2]+ +g[aa+(ag<<2)>>2];c[L>>2]=1;c[M>>2]=1;ai=ag+I|0;al=O+(ai<<2)|0;am=c[al>>2]|0;an=U+(ai<<2)|0;ai=c[an>>2]|0;if((((am|0)>-1?am:-am|0)|0)>(((ai|0)>-1?ai:-ai|0)|0)){ao=(am|0)>0?am-ai|0:ai-am|0;c[an>>2]=ao;ap=ao;aq=c[al>>2]|0}else{c[an>>2]=(ai|0)>0?am-ai|0:ai-am|0;c[al>>2]=ai;ap=c[an>>2]|0;aq=ai}if((ap|0)<(((aq|0)>-1?aq:-aq|0)<<1|0)){break}c[an>>2]=-ap;c[al>>2]=-(c[al>>2]|0)}}while(0);P=ab+(ag<<2)|0;al=ad+(ag<<2)|0;ak=+g[P>>2]+ +g[al>>2];g[al>>2]=ak;g[P>>2]=ak;ag=ag+1|0;}while((ag|0)<(G|0))}ag=c[p>>2]|0;+FR(ag,r,Z,$,ab,ae,I,G,W);ah=c[z>>2]|0}ag=J+1|0;if((ag|0)<(ah|0)){J=ag;N=ah}else{ar=ah;break}}}else{ar=b}N=I+q|0;if((N|0)<(o|0)){I=N}else{as=ar;break}}}else{as=E}if((as|0)>0){at=0;au=as}else{i=n;return}while(1){as=k+(c[e+1160+(at<<2)>>2]<<2)|0;E=e+2184+(at<<2)|0;if((c[as>>2]|0)==0){if((c[k+(c[E>>2]<<2)>>2]|0)==0){av=au}else{S=3693}}else{S=3693}if((S|0)==3693){S=0;c[as>>2]=1;c[k+(c[E>>2]<<2)>>2]=1;av=c[z>>2]|0}E=at+1|0;if((E|0)<(av|0)){at=E;au=av}else{break}}i=n;return}function FR(a,b,d,e,f,j,l,m,n){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;j=j|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0.0,y=0,z=0.0,A=0.0,B=0,C=0.0,D=0.0,E=0,F=0.0;o=i;p=i;i=i+(m<<2)|0;i=i+7&-8;q=p;if((c[a+500>>2]|0)==0){r=m}else{r=(c[a+504>>2]|0)-l|0}s=(r|0)>(m|0)?m:r;if((s|0)>0){t=(j|0)==0;u=~r;r=~m;v=(u|0)>(r|0)?u:r;r=0;do{if(t){w=3707}else{if((c[j+(r<<2)>>2]|0)==0){w=3707}}do{if((w|0)==3707){w=0;u=+g[d+(r<<2)>>2]<0.0;x=+bR(+(+S(+(+g[e+(r<<2)>>2]/+g[f+(r<<2)>>2]))));if(u){c[n+(r<<2)>>2]=~~(-0.0-x);break}else{c[n+(r<<2)>>2]=~~x;break}}}while(0);r=r+1|0;}while((r|0)<(s|0));y=~v}else{y=0}if((y|0)>=(m|0)){z=0.0;i=o;return+z}v=(j|0)!=0;s=b-l|0;l=0;b=y;x=0.0;while(1){if(v){if((c[j+(b<<2)>>2]|0)==0){w=3713}else{A=x;B=l}}else{w=3713}L5192:do{if((w|0)==3713){w=0;y=e+(b<<2)|0;r=f+(b<<2)|0;C=+g[y>>2]/+g[r>>2];do{if(C<.25){if(v&(b|0)<(s|0)){break}c[q+(l<<2)>>2]=y;A=x+C;B=l+1|0;break L5192}}while(0);t=+g[d+(b<<2)>>2]<0.0;D=+bR(+(+S(+C)));if(t){t=~~(-0.0-D);c[n+(b<<2)>>2]=t;E=t}else{t=~~D;c[n+(b<<2)>>2]=t;E=t}D=+(ac(E,E)|0);g[y>>2]=+g[r>>2]*D;A=x;B=l}}while(0);t=b+1|0;if((t|0)<(m|0)){l=B;b=t;x=A}else{break}}if((B|0)==0){z=A;i=o;return+z}b6(p|0,B|0,4,136);if((B|0)<=0){z=A;i=o;return+z}p=e;b=a+512|0;a=0;x=A;while(1){l=(c[q+(a<<2)>>2]|0)-p>>2;if(x<+h[b>>3]){c[n+(l<<2)>>2]=0;g[e+(l<<2)>>2]=0.0;F=x}else{c[n+(l<<2)>>2]=~~(c[k>>2]=(g[k>>2]=+g[d+(l<<2)>>2],c[k>>2]|0)&-2147483648|1065353216,+g[k>>2]);g[e+(l<<2)>>2]=+g[f+(l<<2)>>2];F=x+-1.0}l=a+1|0;if((l|0)<(B|0)){a=l;x=F}else{z=F;break}}i=o;return+z}function FS(a,b){a=a|0;b=b|0;var d=0.0,e=0.0;d=+g[c[a>>2]>>2];e=+g[c[b>>2]>>2];return(d<e)-(d>e)|0}function FT(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0.0,o=0,p=0.0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;e=i;f=d<<2;h=i;i=i+f|0;i=i+7&-8;j=h;h=i;i=i+f|0;i=i+7&-8;f=h;if((d|0)>0){k=0;l=0}else{i=e;return}do{do{if((k|0)<2){c[j+(k<<2)>>2]=l;g[f+(k<<2)>>2]=+g[a+(l<<2)>>2];m=k}else{n=+g[a+(l<<2)>>2];h=k;while(1){o=h-1|0;p=+g[f+(o<<2)>>2];if(n<p){q=3741;break}if(!((l|0)<((c[j+(o<<2)>>2]|0)+b|0)&(h|0)>1)){q=3745;break}r=h-2|0;if(p>+g[f+(r<<2)>>2]){q=3745;break}if((l|0)<((c[j+(r<<2)>>2]|0)+b|0)){h=o}else{q=3745;break}}if((q|0)==3741){q=0;c[j+(h<<2)>>2]=l;g[f+(h<<2)>>2]=n;m=h;break}else if((q|0)==3745){q=0;c[j+(h<<2)>>2]=l;g[f+(h<<2)>>2]=n;m=h;break}}}while(0);k=m+1|0;l=l+1|0;}while((l|0)<(d|0));if((k|0)<=0){i=e;return}l=b+1|0;b=~d;o=0;r=0;while(1){do{if((r|0)<(m|0)){s=r+1|0;if(+g[f+(s<<2)>>2]<=+g[f+(r<<2)>>2]){q=3750;break}t=c[j+(s<<2)>>2]|0}else{q=3750}}while(0);if((q|0)==3750){q=0;t=l+(c[j+(r<<2)>>2]|0)|0}s=(t|0)>(d|0)?d:t;if((o|0)<(s|0)){p=+g[f+(r<<2)>>2];u=~t;v=(u|0)>(b|0)?u:b;u=o;do{g[a+(u<<2)>>2]=p;u=u+1|0;}while((u|0)<(s|0));w=~v}else{w=o}s=r+1|0;if((s|0)<(k|0)){o=w;r=s}else{break}}i=e;return}function FU(a){a=a|0;if((a|0)!=0){ND(a)}return}function FV(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;if((a|0)==0){return}b=a+4|0;d=c[b>>2]|0;e=a+20|0;f=c[e>>2]|0;if((d|0)>0){g=0;h=f;i=d;while(1){d=c[h+(g<<2)>>2]|0;if((d|0)==0){j=i;k=h}else{ND(d);j=c[b>>2]|0;k=c[e>>2]|0}d=g+1|0;if((d|0)<(j|0)){g=d;h=k;i=j}else{l=k;break}}}else{l=f}ND(l);l=a+24|0;f=a+28|0;k=c[f>>2]|0;if((c[l>>2]|0)>0){j=0;i=k;while(1){ND(c[i+(j<<2)>>2]|0);h=j+1|0;g=c[f>>2]|0;if((h|0)<(c[l>>2]|0)){j=h;i=g}else{m=g;break}}}else{m=k}ND(m);ND(a);return}function FW(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;G3(b,c[a>>2]|0,24);G3(b,c[a+4>>2]|0,24);G3(b,(c[a+8>>2]|0)-1|0,24);d=a+12|0;G3(b,(c[d>>2]|0)-1|0,6);G3(b,c[a+20>>2]|0,8);if((c[d>>2]|0)<=0){return}e=a+24|0;f=0;g=0;do{h=e+(f<<2)|0;i=c[h>>2]|0;do{if((i|0)==0){j=0;k=3782}else{l=i;m=0;do{m=m+1|0;l=l>>>1;}while((l|0)!=0);if((m|0)<=3){j=i;k=3782;break}G3(b,i,3);G3(b,1,1);G3(b,c[h>>2]>>3,5)}}while(0);if((k|0)==3782){k=0;G3(b,j,4)}i=c[h>>2]|0;if((i|0)==0){n=0}else{l=i;i=0;while(1){o=(l&1)+i|0;p=l>>>1;if((p|0)==0){n=o;break}else{l=p;i=o}}}g=n+g|0;f=f+1|0;}while((f|0)<(c[d>>2]|0));if((g|0)<=0){return}d=a+280|0;a=0;do{G3(b,c[d+(a<<2)>>2]|0,8);a=a+1|0;}while((a|0)<(g|0));return}function FX(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;d=NE(1,2840)|0;e=c[a+28>>2]|0;c[d>>2]=G9(b,24)|0;c[d+4>>2]=G9(b,24)|0;c[d+8>>2]=(G9(b,24)|0)+1;a=(G9(b,6)|0)+1|0;f=d+12|0;c[f>>2]=a;g=G9(b,8)|0;h=d+20|0;c[h>>2]=g;L5300:do{if((g|0)<0){i=3814}else{do{if((a|0)>0){j=d+24|0;k=0;l=0;do{m=G9(b,3)|0;n=G9(b,1)|0;if((n|0)<0){i=3814;break L5300}if((n|0)==0){o=m}else{n=G9(b,5)|0;if((n|0)<0){i=3814;break L5300}o=n<<3|m}c[j+(l<<2)>>2]=o;if((o|0)==0){p=0}else{m=o;n=0;while(1){q=(m&1)+n|0;r=m>>>1;if((r|0)==0){p=q;break}else{m=r;n=q}}}k=p+k|0;l=l+1|0;}while((l|0)<(c[f>>2]|0));if((k|0)<=0){s=k;break}l=d+280|0;j=0;while(1){n=G9(b,8)|0;if((n|0)<0){i=3814;break L5300}c[l+(j<<2)>>2]=n;n=j+1|0;if((n|0)<(k|0)){j=n}else{s=k;break}}}else{s=0}}while(0);k=c[h>>2]|0;j=c[e+24>>2]|0;if((k|0)>=(j|0)){break}l=d+280|0;n=e+1824|0;m=0;while(1){if((m|0)>=(s|0)){break}q=c[l+(m<<2)>>2]|0;if((q|0)>=(j|0)){i=3814;break L5300}if((c[(c[n+(q<<2)>>2]|0)+12>>2]|0)==0){i=3814;break L5300}else{m=m+1|0}}m=c[n+(k<<2)>>2]|0;j=c[m+4>>2]|0;l=c[m>>2]|0;if((l|0)<1){break}else{t=1;u=l}while(1){if((u|0)<=0){break}l=ac(c[f>>2]|0,t)|0;if((l|0)>(j|0)){i=3814;break L5300}else{t=l;u=u-1|0}}c[d+16>>2]=t;v=d;return v|0}}while(0);do{if((i|0)==3814){if((d|0)==0){v=0}else{break}return v|0}}while(0);ND(d);v=0;return v|0}function FY(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;d=NE(1,44)|0;e=c[(c[a+4>>2]|0)+28>>2]|0;c[d>>2]=b;a=c[b+12>>2]|0;c[d+4>>2]=a;f=e+2848|0;e=c[f>>2]|0;c[d+12>>2]=e;g=e+((c[b+20>>2]|0)*56|0)|0;c[d+16>>2]=g;e=c[g>>2]|0;g=NE(a,4)|0;c[d+20>>2]=g;if((a|0)>0){h=b+24|0;i=b+280|0;b=0;j=0;k=0;while(1){l=h+(k<<2)|0;m=c[l>>2]|0;do{if((m|0)==0){n=j;o=b}else{p=m;q=0;do{q=q+1|0;p=p>>>1;}while((p|0)!=0);if((q|0)==0){n=j;o=b;break}p=(q|0)>(j|0)?q:j;r=g+(k<<2)|0;c[r>>2]=NE(q,4)|0;if((q|0)>0){s=b;t=0}else{n=p;o=b;break}while(1){if((c[l>>2]&1<<t|0)==0){u=s}else{c[(c[r>>2]|0)+(t<<2)>>2]=(c[f>>2]|0)+((c[i+(s<<2)>>2]|0)*56|0);u=s+1|0}v=t+1|0;if((v|0)<(q|0)){s=u;t=v}else{n=p;o=u;break}}}}while(0);l=k+1|0;if((l|0)<(a|0)){b=o;j=n;k=l}else{w=n;break}}}else{w=0}n=d+24|0;c[n>>2]=1;k=(e|0)>0;if(k){j=0;o=1;do{o=ac(o,a)|0;j=j+1|0;}while((j|0)<(e|0));c[n>>2]=o;x=o}else{x=1}c[d+8>>2]=w;w=NC(x<<2)|0;c[d+28>>2]=w;if((x|0)<=0){return d|0}o=e<<2;if(k){y=0}else{k=0;do{c[w+(k<<2)>>2]=NC(o)|0;k=k+1|0;}while((k|0)<(x|0));return d|0}do{k=NC(o)|0;n=w+(y<<2)|0;c[n>>2]=k;j=0;b=y;u=x;t=k;while(1){k=(u|0)/(a|0)|0;s=(b|0)/(k|0)|0;c[t+(j<<2)>>2]=s;i=j+1|0;if((i|0)>=(e|0)){break}f=b-(ac(s,k)|0)|0;j=i;b=f;u=k;t=c[n>>2]|0}y=y+1|0;}while((y|0)<(x|0));return d|0}function FZ(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0;if((f|0)>0){g=0;h=0}else{return 0}while(1){if((c[e+(g<<2)>>2]|0)==0){i=h}else{c[d+(h<<2)>>2]=c[d+(g<<2)>>2];i=h+1|0}j=g+1|0;if((j|0)<(f|0)){g=j;h=i}else{break}}if((i|0)==0){return 0}F_(a,b,d,i,4);return 0}function F_(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0;g=i;h=c[b>>2]|0;j=c[h+8>>2]|0;k=b+16|0;l=c[c[k>>2]>>2]|0;m=c[a+36>>2]>>1;n=c[h+4>>2]|0;o=h|0;p=((n|0)<(m|0)?n:m)-(c[o>>2]|0)|0;if((p|0)<=0){i=g;return}m=(p|0)/(j|0)|0;p=i;i=i+(e<<2)|0;i=i+7&-8;n=p;p=(e|0)>0;if(p){q=((l-1+m|0)/(l|0)|0)<<2;r=0;do{c[n+(r<<2)>>2]=E5(a,q)|0;r=r+1|0;}while((r|0)<(e|0))}r=b+8|0;q=c[r>>2]|0;if((q|0)<=0){i=g;return}s=(m|0)>0;t=a+4|0;a=h+16|0;u=b+28|0;v=(l|0)>0;w=b+20|0;b=0;x=q;L5398:while(1){if(s){q=(b|0)==0;y=1<<b;z=0;A=0;while(1){L5404:do{if(q){B=0;while(1){if((B|0)>=(e|0)){break L5404}C=Gr(c[k>>2]|0,t)|0;if((C|0)==-1){D=3884;break L5398}if((C|0)>=(c[a>>2]|0)){D=3881;break L5398}E=c[(c[u>>2]|0)+(C<<2)>>2]|0;c[(c[n+(B<<2)>>2]|0)+(z<<2)>>2]=E;if((E|0)==0){D=3883;break L5398}else{B=B+1|0}}}}while(0);L5411:do{if(v&(A|0)<(m|0)){if(p){F=0;G=A}else{B=0;E=A;while(1){C=B+1|0;H=E+1|0;if((C|0)<(l|0)&(H|0)<(m|0)){B=C;E=H}else{I=H;break L5411}}}while(1){E=ac(G,j)|0;B=0;do{H=(c[o>>2]|0)+E|0;C=c[(c[(c[n+(B<<2)>>2]|0)+(z<<2)>>2]|0)+(F<<2)>>2]|0;do{if((c[h+24+(C<<2)>>2]&y|0)!=0){J=c[(c[(c[w>>2]|0)+(C<<2)>>2]|0)+(b<<2)>>2]|0;if((J|0)==0){break}if((dk[f&63](J,(c[d+(B<<2)>>2]|0)+(H<<2)|0,t,j)|0)==-1){D=3882;break L5398}}}while(0);B=B+1|0;}while((B|0)<(e|0));B=F+1|0;E=G+1|0;if((B|0)<(l|0)&(E|0)<(m|0)){F=B;G=E}else{I=E;break}}}else{I=A}}while(0);if((I|0)<(m|0)){z=z+1|0;A=I}else{break}}K=c[r>>2]|0}else{K=x}A=b+1|0;if((A|0)<(K|0)){b=A;x=K}else{D=3886;break}}if((D|0)==3886){i=g;return}else if((D|0)==3883){i=g;return}else if((D|0)==3884){i=g;return}else if((D|0)==3881){i=g;return}else if((D|0)==3882){i=g;return}}function F$(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0;if((g|0)>0){j=0;k=0}else{return 0}while(1){if((c[f+(j<<2)>>2]|0)==0){l=k}else{c[e+(k<<2)>>2]=c[e+(j<<2)>>2];l=k+1|0}i=j+1|0;if((i|0)<(g|0)){j=i;k=l}else{break}}if((l|0)==0){return 0}F0(a,d,e,l,h);return 0}function F0(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0;g=i;i=i+1088|0;h=g|0;j=g+32|0;k=g+64|0;l=g+576|0;m=c[b>>2]|0;n=c[m+8>>2]|0;o=c[m+12>>2]|0;p=b+16|0;q=c[c[p>>2]>>2]|0;r=m|0;s=((c[m+4>>2]|0)-(c[r>>2]|0)|0)/(n|0)|0;NM(k|0,0,512);NM(l|0,0,512);t=b+8|0;u=c[t>>2]|0;if((u|0)<=0){i=g;return}v=(s|0)>0;w=(e|0)>0;x=(q|0)>1;y=b+36|0;z=(q|0)>0;A=b+20|0;B=h;C=j;D=j|0;E=b+32|0;b=-q|0;F=0;G=u;while(1){if(v){u=(F|0)==0;H=1<<F;I=0;while(1){L5457:do{if(u&w){if(x){J=0}else{K=0;while(1){L=c[(c[f+(K<<2)>>2]|0)+(I<<2)>>2]|0;M=c[p>>2]|0;if((L|0)<(c[M+4>>2]|0)){N=Gq(M,L,a)|0;c[y>>2]=(c[y>>2]|0)+N}K=K+1|0;if((K|0)>=(e|0)){break L5457}}}do{K=c[f+(J<<2)>>2]|0;N=c[K+(I<<2)>>2]|0;L=1;while(1){M=ac(N,o)|0;O=L+I|0;if((O|0)<(s|0)){P=(c[K+(O<<2)>>2]|0)+M|0}else{P=M}M=L+1|0;if((M|0)<(q|0)){N=P;L=M}else{break}}L=c[p>>2]|0;if((P|0)<(c[L+4>>2]|0)){N=Gq(L,P,a)|0;c[y>>2]=(c[y>>2]|0)+N}J=J+1|0;}while((J|0)<(e|0))}}while(0);if(z&(I|0)<(s|0)){N=I-s|0;L=N>>>0<b>>>0?b:N;N=1;K=I;while(1){M=ac(K,n)|0;O=(c[r>>2]|0)+M|0;if(w){M=0;do{Q=c[(c[f+(M<<2)>>2]|0)+(K<<2)>>2]|0;if(u){R=l+(Q<<2)|0;c[R>>2]=(c[R>>2]|0)+n}R=f+(M<<2)|0;do{if((c[m+24+(Q<<2)>>2]&H|0)!=0){S=c[(c[(c[A>>2]|0)+(Q<<2)>>2]|0)+(F<<2)>>2]|0;if((S|0)==0){break}T=c[d+(M<<2)>>2]|0;U=S|0;V=c[U>>2]|0;W=(n|0)/(V|0)|0;L5490:do{if((W|0)>0){X=S+48|0;Y=S+52|0;Z=S+44|0;_=S+12|0;$=S+4|0;aa=0;ab=0;ad=V;while(1){ae=(ac(ab,V)|0)+O|0;af=T+(ae<<2)|0;ag=c[X>>2]|0;ah=c[Y>>2]|0;ai=c[Z>>2]|0;aj=ai>>1;NM(B|0,0,32);ak=(ad|0)>0;do{if((ah|0)==1){if(!ak){al=0;break}am=ai-1|0;an=0;ao=ad;ap=0;while(1){aq=ao-1|0;ar=c[T+(ae+aq<<2)>>2]|0;as=ar-ag|0;if((as|0)<(aj|0)){at=(aj-as<<1)-1|0}else{at=as-aj<<1}as=ac(an,ai)|0;if((at|0)<0){au=0}else{au=(at|0)<(ai|0)?at:am}av=au+as|0;c[h+(aq<<2)>>2]=ar;ar=ap+1|0;if((ar|0)<(ad|0)){an=av;ao=aq;ap=ar}else{al=av;break}}}else{if(!ak){al=0;break}ap=(ah>>1)-ag|0;ao=ai-1|0;an=0;am=ad;av=0;while(1){ar=am-1|0;aq=(ap+(c[T+(ae+ar<<2)>>2]|0)|0)/(ah|0)|0;if((aq|0)<(aj|0)){aw=(aj-aq<<1)-1|0}else{aw=aq-aj<<1}as=ac(an,ai)|0;if((aw|0)<0){ax=0}else{ax=(aw|0)<(ai|0)?aw:ao}ay=ax+as|0;c[h+(ar<<2)>>2]=(ac(aq,ah)|0)+ag;aq=av+1|0;if((aq|0)<(ad|0)){an=ay;am=ar;av=aq}else{al=ay;break}}}}while(0);aj=c[(c[_>>2]|0)+8>>2]|0;do{if((c[aj+(al<<2)>>2]|0)<1){NM(C|0,0,32);av=(ac(ai-1|0,ah)|0)+ag|0;if((c[$>>2]|0)>0){az=al;aA=-1;aB=0}else{aC=al;break}while(1){do{if((c[aj+(aB<<2)>>2]|0)>0){if(ak){am=0;an=0;while(1){ao=(c[j+(am<<2)>>2]|0)-(c[T+(ae+am<<2)>>2]|0)|0;ap=(ac(ao,ao)|0)+an|0;ao=am+1|0;if((ao|0)<(ad|0)){am=ao;an=ap}else{aD=ap;break}}}else{aD=0}if(!((aA|0)==-1|(aD|0)<(aA|0))){aE=aA;aF=az;break}NN(B|0,C|0,32)|0;aE=aD;aF=aB}else{aE=aA;aF=az}}while(0);an=c[D>>2]|0;if((an|0)<(av|0)){aG=D;aH=an}else{an=0;am=D;while(1){ap=an+1|0;c[am>>2]=0;ao=j+(ap<<2)|0;ay=c[ao>>2]|0;if((ay|0)<(av|0)){aG=ao;aH=ay;break}else{an=ap;am=ao}}}if((aH|0)>-1){am=(c[Y>>2]|0)+aH|0;c[aG>>2]=am;aI=am}else{aI=aH}c[aG>>2]=-aI;am=aB+1|0;if((am|0)<(c[$>>2]|0)){az=aF;aA=aE;aB=am}else{aC=aF;break}}}else{aC=al}}while(0);if((aC|0)>-1&ak){ae=0;aj=af;while(1){c[aj>>2]=(c[aj>>2]|0)-(c[h+(ae<<2)>>2]|0);ag=ae+1|0;if((ag|0)<(ad|0)){ae=ag;aj=aj+4|0}else{break}}}aj=(Gq(S,aC,a)|0)+aa|0;ae=ab+1|0;if((ae|0)>=(W|0)){aJ=aj;break L5490}aa=aj;ab=ae;ad=c[U>>2]|0}}else{aJ=0}}while(0);c[E>>2]=(c[E>>2]|0)+aJ;U=k+(c[(c[R>>2]|0)+(K<<2)>>2]<<2)|0;c[U>>2]=(c[U>>2]|0)+aJ}}while(0);M=M+1|0;}while((M|0)<(e|0))}M=K+1|0;if(!((N|0)<(q|0)&(M|0)<(s|0))){break}N=N+1|0;K=M}aK=I-L|0}else{aK=I}if((aK|0)<(s|0)){I=aK}else{break}}aL=c[t>>2]|0}else{aL=G}I=F+1|0;if((I|0)<(aL|0)){F=I;G=aL}else{break}}i=g;return}function F1(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0.0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0.0,A=0;if((f|0)>0){g=0;h=0}else{i=0;return i|0}while(1){if((c[e+(g<<2)>>2]|0)==0){j=h}else{c[d+(h<<2)>>2]=c[d+(g<<2)>>2];j=h+1|0}k=g+1|0;if((k|0)<(f|0)){g=k;h=j}else{break}}if((j|0)==0){i=0;return i|0}h=c[b>>2]|0;g=c[h+8>>2]|0;f=c[h+12>>2]|0;e=h|0;k=((c[h+4>>2]|0)-(c[e>>2]|0)|0)/(g|0)|0;l=E5(a,j<<2)|0;m=100.0/+(g|0);n=(j|0)>0;if(n){o=k<<2;p=0;do{q=E5(a,o)|0;c[l+(p<<2)>>2]=q;NM(q|0,0,o|0);p=p+1|0;}while((p|0)<(j|0))}if((k|0)>0){p=(g|0)>0;o=f-1|0;f=(o|0)>0;a=0;do{q=ac(a,g)|0;r=(c[e>>2]|0)+q|0;if(n){q=0;do{if(p){s=c[d+(q<<2)>>2]|0;t=0;u=0;v=0;do{w=c[s+(r+t<<2)>>2]|0;x=(w|0)>-1?w:-w|0;u=(x|0)>(u|0)?x:u;v=x+v|0;t=t+1|0;}while((t|0)<(g|0));y=u;z=+(v|0)}else{y=0;z=0.0}t=~~(m*z);L5582:do{if(f){s=0;while(1){if((y|0)<=(c[h+2328+(s<<2)>>2]|0)){x=c[h+2584+(s<<2)>>2]|0;if((x|0)<0|(t|0)<(x|0)){A=s;break L5582}}x=s+1|0;if((x|0)<(o|0)){s=x}else{A=x;break}}}else{A=0}}while(0);c[(c[l+(q<<2)>>2]|0)+(a<<2)>>2]=A;q=q+1|0;}while((q|0)<(j|0))}a=a+1|0;}while((a|0)<(k|0))}k=b+40|0;c[k>>2]=(c[k>>2]|0)+1;i=l;return i|0}function F2(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0;if((f|0)>0){g=0;h=0}else{return 0}while(1){if((c[e+(g<<2)>>2]|0)==0){i=h}else{c[d+(h<<2)>>2]=c[d+(g<<2)>>2];i=h+1|0}j=g+1|0;if((j|0)<(f|0)){g=j;h=i}else{break}}if((i|0)==0){return 0}F_(a,b,d,i,8);return 0}function F3(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;if((f|0)>0){g=0;h=0}else{i=0;return i|0}do{h=((c[e+(g<<2)>>2]|0)!=0)+h|0;g=g+1|0;}while((g|0)<(f|0));if((h|0)==0){i=0;return i|0}h=c[b>>2]|0;g=c[h+8>>2]|0;e=c[h+12>>2]|0;j=h|0;k=((c[h+4>>2]|0)-(c[j>>2]|0)|0)/(g|0)|0;l=E5(a,4)|0;m=k<<2;n=E5(a,m)|0;c[l>>2]=n;NM(n|0,0,m|0);if((k|0)>0){m=(g|0)>0;n=e-1|0;e=(n|0)>0;a=(f|0)>1;o=(c[j>>2]|0)/(f|0)|0;j=0;while(1){if(m){p=c[d>>2]|0;q=o;r=0;s=0;t=0;while(1){u=c[p+(q<<2)>>2]|0;v=(u|0)>-1?u:-u|0;u=(v|0)>(s|0)?v:s;if(a){v=1;w=t;while(1){x=c[(c[d+(v<<2)>>2]|0)+(q<<2)>>2]|0;y=(x|0)>-1?x:-x|0;x=(y|0)>(w|0)?y:w;y=v+1|0;if((y|0)<(f|0)){v=y;w=x}else{z=x;break}}}else{z=t}w=q+1|0;v=r+f|0;if((v|0)<(g|0)){q=w;r=v;s=u;t=z}else{A=w;B=u;C=z;break}}}else{A=o;B=0;C=0}L5627:do{if(e){t=0;while(1){if((B|0)<=(c[h+2328+(t<<2)>>2]|0)){if((C|0)<=(c[h+2584+(t<<2)>>2]|0)){D=t;break L5627}}s=t+1|0;if((s|0)<(n|0)){t=s}else{D=s;break}}}else{D=0}}while(0);c[(c[l>>2]|0)+(j<<2)>>2]=D;t=j+1|0;if((t|0)<(k|0)){o=A;j=t}else{break}}}j=b+40|0;c[j>>2]=(c[j>>2]|0)+1;i=l;return i|0}function F4(a,b,d,e,f,g,h,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;j=i;i=i+8|0;k=j|0;l=c[b+36>>2]|0;m=(l|0)/2|0;n=E5(b,ac(g<<2,m)|0)|0;c[k>>2]=n;if((g|0)<=0){i=j;return 0}b=(l|0)>1;l=0;o=0;do{p=c[e+(l<<2)>>2]|0;o=((c[f+(l<<2)>>2]|0)!=0)+o|0;if(b){q=0;r=l;while(1){c[n+(r<<2)>>2]=c[p+(q<<2)>>2];s=q+1|0;if((s|0)<(m|0)){q=s;r=r+g|0}else{break}}}l=l+1|0;}while((l|0)<(g|0));if((o|0)==0){i=j;return 0}F0(a,d,k,1,h);i=j;return 0}function F5(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;g=c[b>>2]|0;h=c[g+8>>2]|0;i=b+16|0;j=c[c[i>>2]>>2]|0;k=(ac(c[a+36>>2]|0,f)|0)>>1;l=c[g+4>>2]|0;m=g|0;n=((l|0)<(k|0)?l:k)-(c[m>>2]|0)|0;if((n|0)<=0){return 0}k=(n|0)/(h|0)|0;n=E5(a,((j-1+k|0)/(j|0)|0)<<2)|0;l=0;while(1){if((l|0)>=(f|0)){break}if((c[e+(l<<2)>>2]|0)==0){l=l+1|0}else{break}}if((l|0)==(f|0)){return 0}l=b+8|0;e=c[l>>2]|0;if((e|0)<=0){return 0}o=(k|0)>0;p=a+4|0;a=g+16|0;q=b+28|0;r=(j|0)>0;s=b+20|0;b=0;t=e;L5665:while(1){if(o){e=(b|0)==0;u=1<<b;v=0;w=0;while(1){if(e){x=Gr(c[i>>2]|0,p)|0;if((x|0)==-1){y=4062;break L5665}if((x|0)>=(c[a>>2]|0)){y=4061;break L5665}z=c[(c[q>>2]|0)+(x<<2)>>2]|0;c[n+(v<<2)>>2]=z;if((z|0)==0){y=4060;break L5665}}if(r&(w|0)<(k|0)){z=n+(v<<2)|0;x=0;A=w;while(1){B=c[(c[z>>2]|0)+(x<<2)>>2]|0;do{if((c[g+24+(B<<2)>>2]&u|0)!=0){C=c[(c[(c[s>>2]|0)+(B<<2)>>2]|0)+(b<<2)>>2]|0;if((C|0)==0){break}D=ac(A,h)|0;if((Gw(C,d,(c[m>>2]|0)+D|0,f,p,h)|0)==-1){y=4058;break L5665}}}while(0);B=x+1|0;D=A+1|0;if((B|0)<(j|0)&(D|0)<(k|0)){x=B;A=D}else{E=D;break}}}else{E=w}if((E|0)<(k|0)){v=v+1|0;w=E}else{break}}F=c[l>>2]|0}else{F=t}w=b+1|0;if((w|0)<(F|0)){b=w;t=F}else{y=4059;break}}if((y|0)==4061){return 0}else if((y|0)==4059){return 0}else if((y|0)==4060){return 0}else if((y|0)==4058){return 0}else if((y|0)==4062){return 0}return 0}function F6(a){a=a|0;var b=0,c=0,d=0,e=0;if((a|0)==0){b=0}else{c=a;a=0;while(1){d=a+1|0;e=c>>>1;if((e|0)==0){b=d;break}else{c=e;a=d}}}return b|0}function F7(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;e=i;i=i+136|0;f=e|0;g=(d|0)!=0;h=NC((g?d:b)<<2)|0;j=h;NM(f|0,0,132);k=(b|0)>0;L5699:do{if(k){l=f+4|0;m=(d|0)==0|0;n=0;o=0;L5701:while(1){p=c[a+(o<<2)>>2]|0;L5703:do{if((p|0)>0){q=c[f+(p<<2)>>2]|0;if((p|0)<32){if((q>>>(p>>>0)|0)!=0){break L5701}}c[j+(n<<2)>>2]=q;r=p;s=q;while(1){t=f+(r<<2)|0;if((s&1|0)!=0){u=4075;break}c[t>>2]=s+1;v=r-1|0;if((v|0)<=0){break}r=v;s=c[f+(v<<2)>>2]|0}do{if((u|0)==4075){u=0;if((r|0)==1){c[l>>2]=(c[l>>2]|0)+1;break}else{c[t>>2]=c[f+(r-1<<2)>>2]<<1;break}}}while(0);r=p+1|0;if((r|0)<33){w=p;x=q;y=r}else{z=1;break}while(1){r=f+(y<<2)|0;s=c[r>>2]|0;if((s>>>1|0)!=(x|0)){z=1;break L5703}c[r>>2]=c[f+(w<<2)>>2]<<1;r=y+1|0;if((r|0)<33){w=y;x=s;y=r}else{z=1;break}}}else{z=m}}while(0);p=o+1|0;if((p|0)<(b|0)){n=n+z|0;o=p}else{break L5699}}ND(h);A=0;i=e;return A|0}}while(0);L5725:do{if((d|0)!=1){z=1;while(1){if((z|0)>=33){break L5725}if((c[f+(z<<2)>>2]&-1>>>((32-z|0)>>>0)|0)==0){z=z+1|0}else{break}}ND(h);A=0;i=e;return A|0}}while(0);if(!k){A=j;i=e;return A|0}if(g){B=0;C=0}else{g=0;k=0;while(1){h=c[a+(k<<2)>>2]|0;if((h|0)>0){f=c[j+(g<<2)>>2]|0;d=0;z=0;while(1){y=f>>>(d>>>0)&1|z<<1;x=d+1|0;if((x|0)<(h|0)){d=x;z=y}else{D=y;break}}}else{D=0}c[j+(g<<2)>>2]=D;z=k+1|0;if((z|0)<(b|0)){g=g+1|0;k=z}else{A=j;break}}i=e;return A|0}while(1){k=c[a+(C<<2)>>2]|0;if((k|0)>0){g=c[j+(B<<2)>>2]|0;D=0;z=0;while(1){d=g>>>(D>>>0)&1|z<<1;h=D+1|0;if((h|0)<(k|0)){D=h;z=d}else{E=d;break}}}else{E=0}if((k|0)==0){F=B}else{c[j+(B<<2)>>2]=E;F=B+1|0}z=C+1|0;if((z|0)<(b|0)){B=F;C=z}else{A=j;break}}i=e;return A|0}function F8(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=c[a+4>>2]|0;d=c[a>>2]|0;a=~~+Q(+(+T(+(+(b|0)),+(1.0/+(d|0)))));if((d|0)>0){e=a}else{while(1){}}while(1){a=e+1|0;f=1;g=1;h=0;do{f=ac(f,e)|0;g=ac(g,a)|0;h=h+1|0;}while((h|0)<(d|0));if((f|0)<=(b|0)&(g|0)>(b|0)){break}if((f|0)>(b|0)){e=e-1|0;continue}else{e=e+1|0;continue}}return e|0}function F9(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,i=0.0,j=0.0,k=0.0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0.0,B=0.0,C=0,D=0,E=0,F=0,G=0;e=a+12|0;if(((c[e>>2]|0)-1|0)>>>0>=2){f=0;return f|0}h=c[a+16>>2]|0;i=+(h&2097151|0);if((h|0)<0){j=-0.0-i}else{j=i}i=+bS(+j,(h>>>21&1023)-788|0);h=c[a+20>>2]|0;j=+(h&2097151|0);if((h|0)<0){k=-0.0-j}else{k=j}j=+bS(+k,(h>>>21&1023)-788|0);h=c[a>>2]|0;l=NE(ac(h,b)|0,4)|0;b=c[e>>2]|0;if((b|0)==2){e=c[a+4>>2]|0;if((e|0)<=0){f=l;return f|0}m=(d|0)!=0;n=a+8|0;o=m^1;p=a+32|0;k=j;q=i;r=a+28|0;s=0;t=0;while(1){if(m){if((c[(c[n>>2]|0)+(t<<2)>>2]|0)!=0|o){u=4143}else{v=s}}else{u=4143}if((u|0)==4143){u=0;if((h|0)>0){w=c[p>>2]|0;x=(c[r>>2]|0)==0;y=d+(s<<2)|0;z=0;A=0.0;while(1){B=A+(q+k*+R(+(+(c[w+((ac(h,t)|0)+z<<2)>>2]|0))));if(m){g[l+((ac(c[y>>2]|0,h)|0)+z<<2)>>2]=B}else{g[l+((ac(h,s)|0)+z<<2)>>2]=B}C=z+1|0;if((C|0)<(h|0)){z=C;A=x?A:B}else{break}}}v=s+1|0}x=t+1|0;if((x|0)<(e|0)){s=v;t=x}else{f=l;break}}return f|0}else if((b|0)==1){b=c[a+4>>2]|0;if((h|0)<=0){while(1){}}t=~~+Q(+(+T(+(+(b|0)),+(1.0/+(h|0)))));while(1){v=t+1|0;s=1;e=1;m=0;do{s=ac(s,t)|0;e=ac(e,v)|0;m=m+1|0;}while((m|0)<(h|0));if((s|0)<=(b|0)&(e|0)>(b|0)){break}t=(s|0)>(b|0)?t-1|0:v}if((b|0)<=0){f=l;return f|0}m=(d|0)!=0;r=a+8|0;p=m^1;o=a+32|0;k=j;j=i;n=a+28|0;a=0;x=0;while(1){do{if(m){if(!((c[(c[r>>2]|0)+(x<<2)>>2]|0)!=0|p)){D=a;break}z=c[o>>2]|0;y=(c[n>>2]|0)==0;w=d+(a<<2)|0;C=0;i=0.0;E=1;while(1){q=i+(j+k*+R(+(+(c[z+((((x|0)/(E|0)|0|0)%(t|0)|0)<<2)>>2]|0))));g[l+((ac(c[w>>2]|0,h)|0)+C<<2)>>2]=q;F=ac(E,t)|0;G=C+1|0;if((G|0)<(h|0)){C=G;i=y?i:q;E=F}else{u=4139;break}}}else{E=c[o>>2]|0;if((c[n>>2]|0)==0){y=0;C=1;while(1){i=j+k*+R(+(+(c[E+((((x|0)/(C|0)|0|0)%(t|0)|0)<<2)>>2]|0)))+0.0;g[l+((ac(h,a)|0)+y<<2)>>2]=i;w=ac(C,t)|0;z=y+1|0;if((z|0)<(h|0)){y=z;C=w}else{u=4139;break}}}else{C=0;i=0.0;y=1;while(1){q=i+(j+k*+R(+(+(c[E+((((x|0)/(y|0)|0|0)%(t|0)|0)<<2)>>2]|0))));g[l+((ac(h,a)|0)+C<<2)>>2]=q;w=ac(y,t)|0;z=C+1|0;if((z|0)<(h|0)){C=z;i=q;y=w}else{u=4139;break}}}}}while(0);if((u|0)==4139){u=0;D=a+1|0}v=x+1|0;if((v|0)<(b|0)){a=D;x=v}else{f=l;break}}return f|0}else{f=l;return f|0}return 0}function Ga(a){a=a|0;var b=0;if((c[a+36>>2]|0)==0){return}b=c[a+32>>2]|0;if((b|0)!=0){ND(b)}b=c[a+8>>2]|0;if((b|0)!=0){ND(b)}ND(a);return}function Gb(a){a=a|0;var b=0,d=0;b=c[a+16>>2]|0;if((b|0)!=0){ND(b)}b=c[a+20>>2]|0;if((b|0)!=0){ND(b)}b=c[a+24>>2]|0;if((b|0)!=0){ND(b)}b=c[a+28>>2]|0;if((b|0)!=0){ND(b)}b=c[a+32>>2]|0;if((b|0)==0){d=a;NM(d|0,0,56);return}ND(b);d=a;NM(d|0,0,56);return}function Gc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0.0,l=0.0,m=0.0,n=0,o=0.0,p=0.0,q=0.0,r=0,s=0;NM(a|0,0,56);c[a+12>>2]=b;d=b+4|0;c[a+4>>2]=c[d>>2];c[a+8>>2]=c[d>>2];e=b|0;c[a>>2]=c[e>>2];c[a+20>>2]=F7(c[b+8>>2]|0,c[d>>2]|0,0)|0;f=c[d>>2]|0;d=c[e>>2]|0;if((d|0)<=0){while(1){}}e=~~+Q(+(+T(+(+(f|0)),+(1.0/+(d|0)))));while(1){g=e+1|0;h=1;i=1;j=0;do{h=ac(h,e)|0;i=ac(i,g)|0;j=j+1|0;}while((j|0)<(d|0));if((h|0)<=(f|0)&(i|0)>(f|0)){break}e=(h|0)>(f|0)?e-1|0:g}c[a+44>>2]=e;e=c[b+16>>2]|0;k=+(e&2097151|0);if((e|0)<0){l=-0.0-k}else{l=k}c[a+48>>2]=~~+bR(+(+bS(+l,(e>>>21&1023)-788|0)));e=c[b+20>>2]|0;l=+(e&2097151|0);b=e>>>21&1023;if((e|0)>=0){m=l;n=b-788|0;o=+bS(+m,n|0);p=o;q=+bR(+p);r=~~q;s=a+52|0;c[s>>2]=r;return 0}m=-0.0-l;n=b-788|0;o=+bS(+m,n|0);p=o;q=+bR(+p);r=~~q;s=a+52|0;c[s>>2]=r;return 0}function Gd(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0;e=i;NM(b|0,0,56);f=d+4|0;g=c[f>>2]|0;if((g|0)>0){h=c[d+8>>2]|0;j=0;k=0;while(1){l=((c[h+(k<<2)>>2]|0)>0)+j|0;m=k+1|0;if((m|0)<(g|0)){j=l;k=m}else{n=l;break}}}else{n=0}c[b+4>>2]=g;g=b+8|0;c[g>>2]=n;c[b>>2]=c[d>>2];if((n|0)<=0){o=0;i=e;return o|0}k=d+8|0;j=F7(c[k>>2]|0,c[f>>2]|0,n)|0;h=n<<2;l=i;i=i+h|0;i=i+7&-8;m=l;if((j|0)==0){Gb(b);o=-1;i=e;return o|0}else{p=0}do{q=j+(p<<2)|0;r=c[q>>2]|0;s=r>>>16|r<<16;r=s>>>8&16711935|s<<8&-16711936;s=r>>>4&252645135|r<<4&-252645136;r=s>>>2&858993459|s<<2&-858993460;c[q>>2]=r>>>1&1431655765|r<<1&-1431655766;c[m+(p<<2)>>2]=q;p=p+1|0;}while((p|0)<(n|0));b6(l|0,n|0,4,84);l=i;i=i+h|0;i=i+7&-8;p=l;l=NC(h)|0;q=b+20|0;c[q>>2]=l;r=j;s=0;while(1){c[p+((c[m+(s<<2)>>2]|0)-r>>2<<2)>>2]=s;t=s+1|0;if((t|0)<(n|0)){s=t}else{u=0;v=l;break}}while(1){c[v+(c[p+(u<<2)>>2]<<2)>>2]=c[j+(u<<2)>>2];l=u+1|0;if((l|0)>=(n|0)){break}u=l;v=c[q>>2]|0}ND(j);c[b+16>>2]=F9(d,n,p)|0;n=b+24|0;c[n>>2]=NC(h)|0;h=c[f>>2]|0;if((h|0)>0){d=0;j=0;v=h;while(1){if((c[(c[k>>2]|0)+(j<<2)>>2]|0)>0){c[(c[n>>2]|0)+(c[p+(d<<2)>>2]<<2)>>2]=j;w=d+1|0;x=c[f>>2]|0}else{w=d;x=v}h=j+1|0;if((h|0)<(x|0)){d=w;j=h;v=x}else{y=w;break}}}else{y=0}w=b+28|0;c[w>>2]=NC(y)|0;y=c[f>>2]|0;if((y|0)>0){x=0;v=0;j=y;while(1){y=c[(c[k>>2]|0)+(v<<2)>>2]|0;if((y|0)>0){a[(c[w>>2]|0)+(c[p+(x<<2)>>2]|0)|0]=y&255;z=x+1|0;A=c[f>>2]|0}else{z=x;A=j}y=v+1|0;if((y|0)<(A|0)){x=z;v=y;j=A}else{B=z;break}}}else{B=0}z=c[g>>2]|0;if((z|0)==0){C=-4}else{g=z;z=0;while(1){A=g>>>1;if((A|0)==0){break}else{g=A;z=z+1|0}}C=z-3|0}z=b+36|0;g=(C|0)<5?5:C;C=(g|0)>8?8:g;c[z>>2]=C;g=1<<C;A=b+32|0;c[A>>2]=NE(g,4)|0;j=b+40|0;c[j>>2]=0;L5921:do{if((B|0)>0){b=0;v=c[w>>2]|0;x=C;f=0;while(1){p=v+b|0;k=a[p]|0;y=k<<24>>24;if((f|0)<(y|0)){c[j>>2]=y;D=a[p]|0}else{D=k}k=D<<24>>24;do{if((k|0)>(x|0)){E=x;F=v}else{p=c[(c[q>>2]|0)+(b<<2)>>2]|0;y=p>>>16|p<<16;p=y>>>8&16711935|y<<8&-16711936;y=p>>>4&252645135|p<<4&-252645136;p=y>>>2&858993459|y<<2&-858993460;y=p>>>1&1431655765|p<<1&-1431655766;if((1<<x-k|0)<=0){E=x;F=v;break}p=b+1|0;d=0;n=k;while(1){c[(c[A>>2]|0)+((y|d<<n)<<2)>>2]=p;h=d+1|0;u=c[z>>2]|0;l=c[w>>2]|0;s=a[l+b|0]|0;if((h|0)<(1<<u-s|0)){d=h;n=s}else{E=u;F=l;break}}}}while(0);k=b+1|0;if((k|0)>=(B|0)){G=E;break L5921}b=k;v=F;x=E;f=c[j>>2]|0}}else{G=C}}while(0);C=-2<<31-G;if((g|0)>0){H=0;I=0;J=0;K=G}else{o=0;i=e;return o|0}while(1){G=H<<32-K;j=G>>>16|G<<16;E=j>>>8&16711935|j<<8&-16711936;j=E>>>4&252645135|E<<4&-252645136;E=j>>>2&858993459|j<<2&-858993460;j=(c[A>>2]|0)+((E>>>1&1431655765|E<<1&-1431655766)<<2)|0;if((c[j>>2]|0)==0){E=I;while(1){F=E+1|0;if((F|0)>=(B|0)){L=J;break}if((c[(c[q>>2]|0)+(F<<2)>>2]|0)>>>0>G>>>0){L=J;break}else{E=F}}while(1){if((L|0)>=(B|0)){break}if(G>>>0<(c[(c[q>>2]|0)+(L<<2)>>2]&C)>>>0){break}else{L=L+1|0}}G=B-L|0;c[j>>2]=(E>>>0>32767?-1073774592:E<<15|-2147483648)|(G>>>0>32767?32767:G);M=L;N=E}else{M=J;N=I}G=H+1|0;if((G|0)>=(g|0)){o=0;break}H=G;I=N;J=M;K=c[z>>2]|0}i=e;return o|0}function Ge(a,b){a=a|0;b=b|0;var d=0;d=c[c[a>>2]>>2]|0;a=c[c[b>>2]>>2]|0;return(d>>>0>a>>>0)-(d>>>0<a>>>0)|0}function Gf(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;d=c[a>>2]|0;if((d|0)==1){return}e=c[a+4>>2]|0;f=c[a+8>>2]|0;a=c[f+4>>2]|0;if((a|0)<=0){return}h=a+1|0;i=d-1|0;j=d;k=1;l=d;m=0;while(1){n=c[f+(h-m<<2)>>2]|0;o=(l|0)/(n|0)|0;p=(d|0)/(l|0)|0;q=ac(p,o)|0;r=j-(ac(p,n-1|0)|0)|0;s=1-k|0;do{if((n|0)==2){t=e+(i+r<<2)|0;if((k|0)==1){Gj(p,o,b,e,t);u=s;break}else{Gj(p,o,e,b,t);u=s;break}}else if((n|0)==4){t=r+p|0;v=e+(i+r<<2)|0;w=e+(i+t<<2)|0;x=e+(i+p+t<<2)|0;if((k|0)==1){Gi(p,o,b,e,v,w,x);u=s;break}else{Gi(p,o,e,b,v,w,x);u=s;break}}else{x=e+(i+r<<2)|0;if((((p|0)==1?k:s)|0)==0){Gk(p,n,o,q,b,b,b,e,e,x);u=1;break}else{Gk(p,n,o,q,e,e,e,b,b,x);u=0;break}}}while(0);q=m+1|0;if((q|0)<(a|0)){j=r;k=u;l=o;m=q}else{break}}if((u|0)!=1&(d|0)>0){y=0}else{return}do{g[b+(y<<2)>>2]=+g[e+(y<<2)>>2];y=y+1|0;}while((y|0)<(d|0));return}function Gg(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0.0,v=0.0,w=0,x=0,y=0.0,z=0.0,A=0.0,B=0,C=0;c[a>>2]=b;d=NE(b*3|0,4)|0;c[a+4>>2]=d;e=NE(32,4)|0;f=e;c[a+8>>2]=f;if((b|0)==1){return}a=e+8|0;h=0;i=b;j=0;k=0;L5985:while(1){if((k|0)<4){l=c[260672+(k<<2)>>2]|0}else{l=j+2|0}m=(l|0)!=2;n=h;o=i;while(1){p=(o|0)/(l|0)|0;if((o|0)!=(ac(p,l)|0)){break}q=n+1|0;c[f+(n+2<<2)>>2]=l;r=(n|0)==0;if(!(m|r)){if((n|0)>0){s=1;do{t=q-s|0;c[f+(t+2<<2)>>2]=c[f+(t+1<<2)>>2];s=s+1|0;}while((s|0)<(q|0))}c[a>>2]=2}if((p|0)==1){break L5985}else{n=q;o=p}}h=n;i=o;j=l;k=k+1|0}c[f>>2]=b;c[e+4>>2]=q;u=6.2831854820251465/+(b|0);if(!((n|0)>0&(r^1))){return}r=b+1|0;q=1;e=0;k=0;while(1){l=c[f+(k+2<<2)>>2]|0;j=ac(l,q)|0;i=(b|0)/(j|0)|0;h=l-1|0;if((h|0)>0){l=(i|0)>2;a=0;m=e;s=0;while(1){t=a+q|0;v=u*+(t|0);if(l){w=2;x=m;y=0.0;while(1){z=y+1.0;A=v*z;g[d+(x+b<<2)>>2]=+U(+A);g[d+(r+x<<2)>>2]=+V(+A);B=w+2|0;if((B|0)<(i|0)){w=B;x=x+2|0;y=z}else{break}}}x=s+1|0;if((x|0)<(h|0)){a=t;m=m+i|0;s=x}else{break}}C=(ac(i,h)|0)+e|0}else{C=e}s=k+1|0;if((s|0)<(n|0)){q=j;e=C;k=s}else{break}}return}function Gh(a){a=a|0;var b=0;if((a|0)==0){return}b=c[a+4>>2]|0;if((b|0)!=0){ND(b)}b=c[a+8>>2]|0;if((b|0)!=0){ND(b)}NM(a|0,0,12);return}function Gi(a,b,c,d,e,f,h){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0.0,v=0,w=0,x=0.0,y=0,z=0,A=0,B=0,C=0.0,D=0.0,E=0.0,F=0.0,G=0.0,H=0.0,I=0.0,J=0.0;i=ac(b,a)|0;j=i<<1;k=(b|0)>0;if(k){l=(a<<2)-1|0;m=a<<1;n=j;o=0;p=j+i|0;q=i;r=0;while(1){s=c+(q<<2)|0;t=c+(p<<2)|0;u=+g[s>>2]+ +g[t>>2];v=c+(o<<2)|0;w=c+(n<<2)|0;x=+g[v>>2]+ +g[w>>2];y=o<<2;g[d+(y<<2)>>2]=u+x;g[d+(l+y<<2)>>2]=x-u;z=y+m|0;g[d+(z-1<<2)>>2]=+g[v>>2]- +g[w>>2];g[d+(z<<2)>>2]=+g[t>>2]- +g[s>>2];s=r+1|0;if((s|0)<(b|0)){n=n+a|0;o=o+a|0;p=p+a|0;q=q+a|0;r=s}else{break}}}if((a|0)<2){return}do{if((a|0)!=2){do{if(k){r=a<<1;if((a|0)>2){A=0;B=0}else{break}while(1){q=A<<2;p=q+r|0;o=q;q=A;n=2;while(1){m=q+2|0;l=o+2|0;s=p-2|0;t=m+i|0;z=n-2|0;u=+g[e+(z<<2)>>2];x=+g[c+(t-1<<2)>>2];w=n-1|0;C=+g[e+(w<<2)>>2];D=+g[c+(t<<2)>>2];E=u*x+C*D;F=u*D-x*C;v=t+i|0;C=+g[f+(z<<2)>>2];x=+g[c+(v-1<<2)>>2];D=+g[f+(w<<2)>>2];u=+g[c+(v<<2)>>2];G=C*x+D*u;H=C*u-x*D;t=v+i|0;D=+g[h+(z<<2)>>2];x=+g[c+(t-1<<2)>>2];u=+g[h+(w<<2)>>2];C=+g[c+(t<<2)>>2];I=D*x+u*C;J=D*C-x*u;u=E+I;x=I-E;E=F+J;I=F-J;J=+g[c+(m<<2)>>2];F=H+J;C=J-H;H=+g[c+(q+1<<2)>>2];J=G+H;D=H-G;g[d+((o|1)<<2)>>2]=u+J;g[d+(l<<2)>>2]=F+E;g[d+(p-3<<2)>>2]=D-I;g[d+(s<<2)>>2]=x-C;t=l+r|0;g[d+(t-1<<2)>>2]=I+D;g[d+(t<<2)>>2]=C+x;t=s+r|0;g[d+(t-1<<2)>>2]=J-u;g[d+(t<<2)>>2]=E-F;t=n+2|0;if((t|0)<(a|0)){p=s;o=l;q=m;n=t}else{break}}n=B+1|0;if((n|0)<(b|0)){A=A+a|0;B=n}else{break}}}}while(0);if((a&1|0)==0){break}return}}while(0);B=a-1+i|0;A=a<<2;h=a<<1;if(!k){return}k=a;f=a;e=B+j|0;j=B;B=0;while(1){F=+g[c+(j<<2)>>2];E=+g[c+(e<<2)>>2];u=(F+E)*-.7071067690849304;J=(F-E)*.7071067690849304;r=c+(k-1<<2)|0;g[d+(f-1<<2)>>2]=+g[r>>2]+J;n=f+h|0;g[d+(n-1<<2)>>2]=+g[r>>2]-J;r=c+(j+i<<2)|0;g[d+(f<<2)>>2]=u- +g[r>>2];g[d+(n<<2)>>2]=u+ +g[r>>2];r=B+1|0;if((r|0)<(b|0)){k=k+a|0;f=f+A|0;e=e+a|0;j=j+a|0;B=r}else{break}}return}function Gj(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0.0,w=0.0,x=0.0,y=0.0,z=0.0,A=0.0,B=0;f=ac(b,a)|0;h=a<<1;i=(b|0)>0;if(i){j=h-1|0;k=0;l=0;m=f;while(1){n=c+(l<<2)|0;o=c+(m<<2)|0;p=l<<1;g[d+(p<<2)>>2]=+g[n>>2]+ +g[o>>2];g[d+(j+p<<2)>>2]=+g[n>>2]- +g[o>>2];o=k+1|0;if((o|0)<(b|0)){k=o;l=l+a|0;m=m+a|0}else{break}}}if((a|0)<2){return}do{if((a|0)!=2){if(i&(a|0)>2){m=0;l=0;k=f;while(1){j=l<<1;o=2;n=k;p=j+h|0;q=l;r=j;while(1){j=n+2|0;s=p-2|0;t=q+2|0;u=r+2|0;v=+g[e+(o-2<<2)>>2];w=+g[c+(n+1<<2)>>2];x=+g[e+(o-1<<2)>>2];y=+g[c+(j<<2)>>2];z=v*w+x*y;A=v*y-w*x;B=c+(t<<2)|0;g[d+(u<<2)>>2]=+g[B>>2]+A;g[d+(s<<2)>>2]=A- +g[B>>2];B=c+(q+1<<2)|0;g[d+((r|1)<<2)>>2]=z+ +g[B>>2];g[d+(p-3<<2)>>2]=+g[B>>2]-z;B=o+2|0;if((B|0)<(a|0)){o=B;n=j;p=s;q=t;r=u}else{break}}r=m+1|0;if((r|0)<(b|0)){m=r;l=l+a|0;k=k+a|0}else{break}}}if(((a|0)%2|0|0)!=1){break}return}}while(0);e=a-1|0;if(!i){return}i=0;k=a;l=f+e|0;f=e;while(1){g[d+(k<<2)>>2]=-0.0- +g[c+(l<<2)>>2];g[d+(k-1<<2)>>2]=+g[c+(f<<2)>>2];e=i+1|0;if((e|0)<(b|0)){i=e;k=k+h|0;l=l+a|0;f=f+a|0}else{break}}return}function Gk(a,b,c,d,e,f,h,i,j,k){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0.0,m=0.0,n=0.0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0.0,Q=0.0,R=0.0,S=0.0,T=0.0,W=0.0,X=0.0,Y=0,Z=0,_=0;l=6.2831854820251465/+(b|0);m=+U(+l);n=+V(+l);o=b+1>>1;p=a-1>>1;q=ac(c,a)|0;r=ac(b,a)|0;s=(a|0)==1;L6084:do{if(!s){if((d|0)>0){t=0;do{g[j+(t<<2)>>2]=+g[h+(t<<2)>>2];t=t+1|0;}while((t|0)<(d|0))}t=(b|0)>1;if(t){u=(c|0)>0;v=0;w=1;do{v=v+q|0;if(u){x=v;y=0;while(1){g[i+(x<<2)>>2]=+g[f+(x<<2)>>2];z=y+1|0;if((z|0)<(c|0)){x=x+a|0;y=z}else{break}}}w=w+1|0;}while((w|0)<(b|0))}w=-a|0;do{if((p|0)>(c|0)){if(!t){break}v=(c|0)>0;u=(a|0)>2;y=0;x=w;z=1;do{y=y+q|0;x=x+a|0;if(v){A=x-1|0;B=y-a|0;C=0;do{B=B+a|0;if(u){D=B;E=2;F=A;while(1){G=F+2|0;H=D+2|0;I=k+(F+1<<2)|0;J=D+1|0;K=f+(J<<2)|0;L=k+(G<<2)|0;M=f+(H<<2)|0;g[i+(J<<2)>>2]=+g[I>>2]*+g[K>>2]+ +g[L>>2]*+g[M>>2];g[i+(H<<2)>>2]=+g[I>>2]*+g[M>>2]- +g[L>>2]*+g[K>>2];K=E+2|0;if((K|0)<(a|0)){D=H;E=K;F=G}else{break}}}C=C+1|0;}while((C|0)<(c|0))}z=z+1|0;}while((z|0)<(b|0))}else{if(!t){break}z=(a|0)>2;u=(c|0)>0;y=0;x=w;v=1;do{x=x+a|0;y=y+q|0;if(z){C=y;A=2;B=x-1|0;while(1){F=B+2|0;E=C+2|0;if(u){D=k+(B+1<<2)|0;G=k+(F<<2)|0;K=E;H=0;while(1){L=K-1|0;M=f+(L<<2)|0;I=f+(K<<2)|0;g[i+(L<<2)>>2]=+g[D>>2]*+g[M>>2]+ +g[G>>2]*+g[I>>2];g[i+(K<<2)>>2]=+g[D>>2]*+g[I>>2]- +g[G>>2]*+g[M>>2];M=H+1|0;if((M|0)<(c|0)){K=K+a|0;H=M}else{break}}}H=A+2|0;if((H|0)<(a|0)){C=E;A=H;B=F}else{break}}}v=v+1|0;}while((v|0)<(b|0))}}while(0);w=ac(q,b)|0;t=(o|0)>1;if((p|0)>=(c|0)){if(!t){break}v=(c|0)>0;u=(a|0)>2;x=w;y=0;z=1;while(1){y=y+q|0;x=x-q|0;if(v){B=x;A=y;C=0;while(1){if(u){H=A;K=B;G=2;while(1){D=H+2|0;M=K+2|0;I=H+1|0;L=i+(I<<2)|0;J=K+1|0;N=i+(J<<2)|0;g[f+(I<<2)>>2]=+g[L>>2]+ +g[N>>2];I=i+(D<<2)|0;O=i+(M<<2)|0;g[f+(J<<2)>>2]=+g[I>>2]- +g[O>>2];g[f+(D<<2)>>2]=+g[I>>2]+ +g[O>>2];g[f+(M<<2)>>2]=+g[N>>2]- +g[L>>2];L=G+2|0;if((L|0)<(a|0)){H=D;K=M;G=L}else{break}}}G=C+1|0;if((G|0)<(c|0)){B=B+a|0;A=A+a|0;C=G}else{break}}}z=z+1|0;if((z|0)>=(o|0)){break L6084}}}if(!t){break}z=(a|0)>2;u=(c|0)>0;y=w;x=0;v=1;do{x=x+q|0;y=y-q|0;if(z){C=y;A=x;B=2;do{A=A+2|0;C=C+2|0;if(u){G=A-a|0;K=C-a|0;H=0;do{G=G+a|0;K=K+a|0;L=G-1|0;M=i+(L<<2)|0;D=K-1|0;N=i+(D<<2)|0;g[f+(L<<2)>>2]=+g[M>>2]+ +g[N>>2];L=i+(G<<2)|0;O=i+(K<<2)|0;g[f+(D<<2)>>2]=+g[L>>2]- +g[O>>2];g[f+(G<<2)>>2]=+g[L>>2]+ +g[O>>2];g[f+(K<<2)>>2]=+g[N>>2]- +g[M>>2];H=H+1|0;}while((H|0)<(c|0))}B=B+2|0;}while((B|0)<(a|0))}v=v+1|0;}while((v|0)<(o|0))}}while(0);k=(d|0)>0;if(k){v=0;do{g[h+(v<<2)>>2]=+g[j+(v<<2)>>2];v=v+1|0;}while((v|0)<(d|0))}v=ac(d,b)|0;u=(o|0)>1;do{if(u){x=(c|0)>0;y=v;z=0;w=1;do{z=z+q|0;y=y-q|0;if(x){t=y-a|0;B=z-a|0;C=0;do{B=B+a|0;t=t+a|0;A=i+(B<<2)|0;H=i+(t<<2)|0;g[f+(B<<2)>>2]=+g[A>>2]+ +g[H>>2];g[f+(t<<2)>>2]=+g[H>>2]- +g[A>>2];C=C+1|0;}while((C|0)<(c|0))}w=w+1|0;}while((w|0)<(o|0));w=ac(b-1|0,d)|0;if(!u){break}z=(o|0)>2;l=0.0;P=1.0;y=v;x=0;C=1;while(1){t=x+d|0;B=y-d|0;Q=m*P-n*l;R=m*l+n*P;if(k){A=B;H=w;K=d;G=t;M=0;while(1){g[j+(G<<2)>>2]=+g[h+(M<<2)>>2]+Q*+g[h+(K<<2)>>2];g[j+(A<<2)>>2]=R*+g[h+(H<<2)>>2];N=M+1|0;if((N|0)<(d|0)){A=A+1|0;H=H+1|0;K=K+1|0;G=G+1|0;M=N}else{break}}}if(z){M=w;G=d;S=R;T=Q;K=2;while(1){H=G+d|0;A=M-d|0;W=Q*T-R*S;X=Q*S+R*T;if(k){N=t;O=B;L=H;D=A;I=0;while(1){J=j+(N<<2)|0;g[J>>2]=+g[J>>2]+W*+g[h+(L<<2)>>2];J=j+(O<<2)|0;g[J>>2]=+g[J>>2]+X*+g[h+(D<<2)>>2];J=I+1|0;if((J|0)<(d|0)){N=N+1|0;O=O+1|0;L=L+1|0;D=D+1|0;I=J}else{break}}}I=K+1|0;if((I|0)<(o|0)){M=A;G=H;S=X;T=W;K=I}else{break}}}K=C+1|0;if((K|0)<(o|0)){l=R;P=Q;y=B;x=t;C=K}else{break}}if(u){Y=0;Z=1}else{break}do{Y=Y+d|0;if(k){C=Y;x=0;while(1){y=j+(x<<2)|0;g[y>>2]=+g[h+(C<<2)>>2]+ +g[y>>2];y=x+1|0;if((y|0)<(d|0)){C=C+1|0;x=y}else{break}}}Z=Z+1|0;}while((Z|0)<(o|0))}}while(0);do{if((a|0)<(c|0)){if((a|0)<=0){break}Z=(c|0)>0;d=0;do{if(Z){h=d;j=d;Y=0;while(1){g[e+(h<<2)>>2]=+g[i+(j<<2)>>2];k=Y+1|0;if((k|0)<(c|0)){h=h+r|0;j=j+a|0;Y=k}else{break}}}d=d+1|0;}while((d|0)<(a|0))}else{if((c|0)<=0){break}d=(a|0)>0;Z=0;Y=0;j=0;while(1){if(d){h=Z;k=Y;v=0;while(1){g[e+(h<<2)>>2]=+g[i+(k<<2)>>2];f=v+1|0;if((f|0)<(a|0)){h=h+1|0;k=k+1|0;v=f}else{break}}}v=j+1|0;if((v|0)<(c|0)){Z=Z+r|0;Y=Y+a|0;j=v}else{break}}}}while(0);j=a<<1;Y=ac(q,b)|0;if(u){b=(c|0)>0;Z=Y;d=0;v=0;k=1;do{v=v+j|0;d=d+q|0;Z=Z-q|0;if(b){h=v;f=d;x=Z;C=0;while(1){g[e+(h-1<<2)>>2]=+g[i+(f<<2)>>2];g[e+(h<<2)>>2]=+g[i+(x<<2)>>2];t=C+1|0;if((t|0)<(c|0)){h=h+r|0;f=f+a|0;x=x+a|0;C=t}else{break}}}k=k+1|0;}while((k|0)<(o|0))}if(s){return}s=-a|0;if((p|0)>=(c|0)){if(!u){return}p=Y;k=0;Z=0;d=s;v=1;do{d=d+j|0;Z=Z+j|0;k=k+q|0;p=p-q|0;if(!((c|0)<1|(a|0)<3)){b=d;C=Z;x=k;f=p;h=0;while(1){t=2;do{B=t+x|0;y=i+(B-1<<2)|0;w=t+f|0;z=i+(w-1<<2)|0;K=t+C|0;g[e+(K-1<<2)>>2]=+g[y>>2]+ +g[z>>2];G=a-t+b|0;g[e+(G-1<<2)>>2]=+g[y>>2]- +g[z>>2];z=i+(B<<2)|0;B=i+(w<<2)|0;g[e+(K<<2)>>2]=+g[z>>2]+ +g[B>>2];g[e+(G<<2)>>2]=+g[B>>2]- +g[z>>2];t=t+2|0;}while((t|0)<(a|0));t=h+1|0;if((t|0)<(c|0)){b=b+r|0;C=C+r|0;x=x+a|0;f=f+a|0;h=t}else{break}}}v=v+1|0;}while((v|0)<(o|0));return}if(!u){return}u=(a|0)>2;v=(c|0)>0;p=Y;Y=0;k=0;Z=s;s=1;do{Z=Z+j|0;k=k+j|0;Y=Y+q|0;p=p-q|0;L6243:do{if(u){d=Z+a|0;if(v){_=2}else{h=2;while(1){h=h+2|0;if((h|0)>=(a|0)){break L6243}}}do{h=d-_|0;f=_+k|0;x=_+Y|0;C=_+p|0;b=0;while(1){t=i+(x-1<<2)|0;z=i+(C-1<<2)|0;g[e+(f-1<<2)>>2]=+g[t>>2]+ +g[z>>2];g[e+(h-1<<2)>>2]=+g[t>>2]- +g[z>>2];z=i+(x<<2)|0;t=i+(C<<2)|0;g[e+(f<<2)>>2]=+g[z>>2]+ +g[t>>2];g[e+(h<<2)>>2]=+g[t>>2]- +g[z>>2];z=b+1|0;if((z|0)<(c|0)){h=h+r|0;f=f+r|0;x=x+a|0;C=C+a|0;b=z}else{break}}_=_+2|0;}while((_|0)<(a|0))}}while(0);s=s+1|0;}while((s|0)<(o|0));return}function Gl(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0;d=(a|0)!=0;do{if(d){e=c[a+64>>2]|0;if((e|0)==0){f=0;g=0;h=1;i=0;j=1;break}k=c[e+104>>2]|0;l=c[e+4>>2]|0;if((l|0)==0){f=0;g=k;h=0;i=0;j=1;break}f=c[l+28>>2]|0;g=k;h=0;i=l;j=0}else{f=0;g=0;h=1;i=0;j=1}}while(0);l=d?a+4|0:0;if(h|(g|0)==0|j|(f|0)==0|(l|0)==0){m=-136;return m|0}E6(a);G6(l,c[b>>2]|0,c[b+4>>2]|0);if((G9(l,1)|0)!=0){m=-135;return m|0}j=G9(l,c[g+44>>2]|0)|0;if((j|0)==-1){m=-136;return m|0}c[a+40>>2]=j;g=f+32+(j<<2)|0;j=c[g>>2]|0;if((j|0)==0){m=-136;return m|0}h=c[j>>2]|0;j=a+28|0;c[j>>2]=h;do{if((h|0)==0){c[a+24>>2]=0;c[a+32>>2]=0;n=0}else{c[a+24>>2]=G9(l,1)|0;d=G9(l,1)|0;c[a+32>>2]=d;if((d|0)==-1){m=-136;return m|0}else{n=c[j>>2]|0;break}}}while(0);j=b+16|0;l=c[j+4>>2]|0;h=a+48|0;c[h>>2]=c[j>>2];c[h+4>>2]=l;l=b+24|0;h=c[l+4>>2]|0;j=a+56|0;c[j>>2]=c[l>>2];c[j+4>>2]=h;c[a+44>>2]=c[b+12>>2];b=a+36|0;c[b>>2]=c[f+(n<<2)>>2];n=i+4|0;i=a|0;c[i>>2]=E5(a,c[n>>2]<<2)|0;if((c[n>>2]|0)>0){h=0;do{j=E5(a,c[b>>2]<<2)|0;c[(c[i>>2]|0)+(h<<2)>>2]=j;h=h+1|0;}while((h|0)<(c[n>>2]|0))}n=c[(c[g>>2]|0)+12>>2]|0;m=dg[c[(c[1811400+(c[f+288+(n<<2)>>2]<<2)>>2]|0)+16>>2]&255](a,c[f+544+(n<<2)>>2]|0)|0;return m|0}function Gm(a){a=a|0;return c[33552+(a<<2)>>2]|0}function Gn(a,b,d,e,f,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0;i=(f|0)!=0;j=i?e:0;e=i?h:0;h=c[33552+(c[b+(j<<2)>>2]<<2)>>2]|0;i=c[33552+(c[b+(e<<2)>>2]<<2)>>2]|0;b=c[d+(f<<2)>>2]|0;f=c[d+(j<<2)>>2]|0;j=c[d+(e<<2)>>2]|0;e=(b|0)/4|0;d=e-((f|0)/4|0)|0;k=d+((f|0)/2|0)|0;f=((b|0)/2|0)+e+((j|0)/-4|0)|0;e=(j|0)/2|0;l=f+e|0;if((d|0)>0){NM(a|0,0,d<<2|0);m=d}else{m=0}if((m|0)<(k|0)){d=m;m=0;while(1){n=a+(d<<2)|0;g[n>>2]=+g[h+(m<<2)>>2]*+g[n>>2];n=d+1|0;if((n|0)<(k|0)){d=n;m=m+1|0}else{break}}}if((j|0)>1){j=f+1|0;m=(l|0)>(j|0);d=f;k=e;do{k=k-1|0;e=a+(d<<2)|0;g[e>>2]=+g[i+(k<<2)>>2]*+g[e>>2];d=d+1|0;}while((d|0)<(l|0));o=m?l:j}else{o=f}if((o|0)>=(b|0)){return}NM(a+(o<<2)|0,0,b-o<<2|0);return}function Go(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;G3(b,5653314,24);d=a|0;G3(b,c[d>>2]|0,16);e=a+4|0;G3(b,c[e>>2]|0,24);f=c[e>>2]|0;g=a+8|0;h=1;while(1){if((h|0)>=(f|0)){break}i=c[g>>2]|0;j=c[i+(h-1<<2)>>2]|0;if((j|0)==0){break}if((c[i+(h<<2)>>2]|0)<(j|0)){break}else{h=h+1|0}}L6310:do{if((h|0)==(f|0)){G3(b,1,1);G3(b,(c[c[g>>2]>>2]|0)-1|0,5);j=c[e>>2]|0;if((j|0)>1){i=0;k=1;l=j;while(1){m=c[g>>2]|0;n=c[m+(k<<2)>>2]|0;o=c[m+(k-1<<2)>>2]|0;if((n|0)>(o|0)){m=o;o=i;p=l;while(1){G3(b,k-o|0,F6(p-o|0)|0);q=m+1|0;r=c[e>>2]|0;if((q|0)<(n|0)){m=q;o=k;p=r}else{s=k;t=r;break}}}else{s=i;t=l}p=k+1|0;if((p|0)<(t|0)){i=s;k=p;l=t}else{u=s;v=p;w=t;break}}}else{u=0;v=1;w=j}G3(b,v-u|0,F6(w-u|0)|0)}else{G3(b,0,1);l=c[e>>2]|0;k=0;while(1){if((k|0)>=(l|0)){break}if((c[(c[g>>2]|0)+(k<<2)>>2]|0)==0){break}else{k=k+1|0}}if((k|0)==(l|0)){G3(b,0,1);if((c[e>>2]|0)>0){x=0}else{break}while(1){G3(b,(c[(c[g>>2]|0)+(x<<2)>>2]|0)-1|0,5);x=x+1|0;if((x|0)>=(c[e>>2]|0)){break L6310}}}G3(b,1,1);if((c[e>>2]|0)>0){y=0}else{break}do{if((c[(c[g>>2]|0)+(y<<2)>>2]|0)==0){G3(b,0,1)}else{G3(b,1,1);G3(b,(c[(c[g>>2]|0)+(y<<2)>>2]|0)-1|0,5)}y=y+1|0;}while((y|0)<(c[e>>2]|0))}}while(0);y=a+12|0;G3(b,c[y>>2]|0,4);g=c[y>>2]|0;if((g|0)==1|(g|0)==2){z=4495}else if((g|0)!=0){A=-1;return A|0}do{if((z|0)==4495){g=a+32|0;if((c[g>>2]|0)==0){A=-1;return A|0}G3(b,c[a+16>>2]|0,32);G3(b,c[a+20>>2]|0,32);x=a+24|0;G3(b,(c[x>>2]|0)-1|0,4);G3(b,c[a+28>>2]|0,1);u=c[y>>2]|0;if((u|0)==1){B=F8(a)|0}else if((u|0)==2){B=ac(c[d>>2]|0,c[e>>2]|0)|0}else{break}if((B|0)>0){C=0}else{break}do{u=c[(c[g>>2]|0)+(C<<2)>>2]|0;G3(b,(u|0)>-1?u:-u|0,c[x>>2]|0);C=C+1|0;}while((C|0)<(B|0))}}while(0);A=0;return A|0}function Gp(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;b=NE(1,40)|0;d=b;c[b+36>>2]=1;L6353:do{if((G9(a,24)|0)==5653314){e=b;c[e>>2]=G9(a,16)|0;f=G9(a,24)|0;g=b+4|0;c[g>>2]=f;if((f|0)==-1){break}f=F6(c[e>>2]|0)|0;if(((F6(c[g>>2]|0)|0)+f|0)>24){break}f=G9(a,1)|0;L6357:do{if((f|0)==1){h=(G9(a,5)|0)+1|0;if((h|0)==0){break L6353}i=c[g>>2]|0;j=b+8|0;c[j>>2]=NC(i<<2)|0;if((i|0)>0){k=0;l=h;m=i}else{break}while(1){i=G9(a,F6(m-k|0)|0)|0;if((i|0)==-1|(l|0)>32){break L6353}h=c[g>>2]|0;if((i|0)>(h-k|0)){break L6353}if((i|0)>0){if((i-1>>l-1|0)>1){break L6353}else{n=0;o=k}while(1){c[(c[j>>2]|0)+(o<<2)>>2]=l;p=n+1|0;if((p|0)<(i|0)){n=p;o=o+1|0}else{break}}q=i+k|0;r=c[g>>2]|0}else{q=k;r=h}if((q|0)<(r|0)){k=q;l=l+1|0;m=r}else{break}}}else if((f|0)==0){j=G9(a,1)|0;p=(j|0)!=0;j=(ac(p?1:5,c[g>>2]|0)|0)+7>>3;s=c[a+16>>2]|0;if((j|0)>(s-(Ha(a)|0)|0)){break L6353}s=c[g>>2]|0;j=b+8|0;c[j>>2]=NC(s<<2)|0;t=(s|0)>0;if(!p){if(t){u=0}else{break}while(1){p=G9(a,5)|0;if((p|0)==-1){break L6353}c[(c[j>>2]|0)+(u<<2)>>2]=p+1;u=u+1|0;if((u|0)>=(c[g>>2]|0)){break L6357}}}if(t){v=0}else{break}do{if((G9(a,1)|0)==0){c[(c[j>>2]|0)+(v<<2)>>2]=0}else{p=G9(a,5)|0;if((p|0)==-1){break L6353}c[(c[j>>2]|0)+(v<<2)>>2]=p+1}v=v+1|0;}while((v|0)<(c[g>>2]|0))}else{break L6353}}while(0);f=G9(a,4)|0;j=b+12|0;c[j>>2]=f;if((f|0)==0){w=d;return w|0}else if(!((f|0)==1|(f|0)==2)){break}c[b+16>>2]=G9(a,32)|0;c[b+20>>2]=G9(a,32)|0;f=b+24|0;c[f>>2]=(G9(a,4)|0)+1;t=G9(a,1)|0;c[b+28>>2]=t;if((t|0)==-1){break}t=c[j>>2]|0;do{if((t|0)==2){x=ac(c[e>>2]|0,c[g>>2]|0)|0}else if((t|0)==1){if((c[e>>2]|0)==0){x=0;break}x=F8(d)|0}else{x=0}}while(0);e=(ac(c[f>>2]|0,x)|0)+7>>3;t=c[a+16>>2]|0;if((e|0)>(t-(Ha(a)|0)|0)){break}t=b+32|0;c[t>>2]=NC(x<<2)|0;if((x|0)>0){e=0;do{g=G9(a,c[f>>2]|0)|0;c[(c[t>>2]|0)+(e<<2)>>2]=g;e=e+1|0;}while((e|0)<(x|0))}if((x|0)==0){w=d;return w|0}if((c[(c[t>>2]|0)+(x-1<<2)>>2]|0)==-1){break}else{w=d}return w|0}}while(0);Ga(d);w=0;return w|0}function Gq(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;if((b|0)<0){e=0;return e|0}f=a+12|0;g=c[f>>2]|0;if((c[g+4>>2]|0)<=(b|0)){e=0;return e|0}G3(d,c[(c[a+20>>2]|0)+(b<<2)>>2]|0,c[(c[g+8>>2]|0)+(b<<2)>>2]|0);e=c[(c[(c[f>>2]|0)+8>>2]|0)+(b<<2)>>2]|0;return e|0}function Gr(a,b){a=a|0;b=b|0;var d=0,e=0;if((c[a+8>>2]|0)<=0){d=-1;return d|0}e=Gs(a,b)|0;if((e|0)<=-1){d=-1;return d|0}d=c[(c[a+24>>2]|0)+(e<<2)>>2]|0;return d|0}function Gs(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;e=c[b+40>>2]|0;f=G7(d,c[b+36>>2]|0)|0;do{if((f|0)>-1){g=c[(c[b+32>>2]|0)+(f<<2)>>2]|0;if((g|0)<0){h=g>>>15&32767;i=(c[b+8>>2]|0)-(g&32767)|0;break}j=g-1|0;G8(d,a[(c[b+28>>2]|0)+j|0]|0);k=j;return k|0}else{h=0;i=c[b+8>>2]|0}}while(0);f=G7(d,e)|0;j=(f|0)<0;if(j&(e|0)>1){g=e;while(1){l=g-1|0;m=G7(d,l)|0;n=(m|0)<0;if(n&(l|0)>1){g=l}else{o=m;p=l;q=n;break}}}else{o=f;p=e;q=j}if(q){k=-1;return k|0}q=o>>>16|o<<16;o=q>>>8&16711935|q<<8&-16711936;q=o>>>4&252645135|o<<4&-252645136;o=q>>>2&858993459|q<<2&-858993460;q=o>>>1&1431655765|o<<1&-1431655766;o=i-h|0;if((o|0)>1){j=c[b+20>>2]|0;e=i;i=h;f=o;while(1){o=f>>1;g=(c[j+(o+i<<2)>>2]|0)>>>0>q>>>0;n=((g^1)<<31>>31&o)+i|0;l=e-(o&-(g&1))|0;g=l-n|0;if((g|0)>1){e=l;i=n;f=g}else{r=n;break}}}else{r=h}h=a[(c[b+28>>2]|0)+r|0]|0;if((h|0)>(p|0)){G8(d,p);k=-1;return k|0}else{G8(d,h);k=r;return k|0}return 0}function Gt(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;f=i;if((c[a+8>>2]|0)<=0){h=0;i=f;return h|0}j=a|0;k=c[j>>2]|0;l=(e|0)/(k|0)|0;e=i;i=i+(l<<2)|0;i=i+7&-8;m=e;e=(l|0)>0;L6452:do{if(e){n=a+16|0;o=0;while(1){p=Gs(a,d)|0;if((p|0)==-1){h=-1;break}q=c[n>>2]|0;r=c[j>>2]|0;c[m+(o<<2)>>2]=q+((ac(r,p)|0)<<2);p=o+1|0;if((p|0)<(l|0)){o=p}else{s=r;break L6452}}i=f;return h|0}else{s=k}}while(0);if((s|0)<=0){h=0;i=f;return h|0}if(e){t=0;u=0}else{e=0;while(1){k=e+1|0;if((k|0)<(s|0)){e=k}else{h=0;break}}i=f;return h|0}while(1){e=0;do{s=b+(e+u<<2)|0;g[s>>2]=+g[(c[m+(e<<2)>>2]|0)+(t<<2)>>2]+ +g[s>>2];e=e+1|0;}while((e|0)<(l|0));e=t+1|0;if((e|0)<(c[j>>2]|0)){t=e;u=u+l|0}else{h=0;break}}i=f;return h|0}function Gu(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0;if((c[a+8>>2]|0)<=0){f=0;return f|0}h=a|0;if((c[h>>2]|0)>8){if((e|0)<=0){f=0;return f|0}i=a+16|0;j=0;while(1){k=Gs(a,d)|0;if((k|0)==-1){f=-1;l=4623;break}m=c[i>>2]|0;n=c[h>>2]|0;o=ac(n,k)|0;if((n|0)>0){n=j;k=0;while(1){p=k+1|0;q=n+1|0;r=b+(n<<2)|0;g[r>>2]=+g[m+(k+o<<2)>>2]+ +g[r>>2];if((p|0)<(c[h>>2]|0)){n=q;k=p}else{s=q;break}}}else{s=j}if((s|0)<(e|0)){j=s}else{f=0;l=4618;break}}if((l|0)==4618){return f|0}else if((l|0)==4623){return f|0}}s=a+16|0;j=0;L6492:while(1){if((j|0)>=(e|0)){f=0;l=4619;break}L6494:while(1){i=Gs(a,d)|0;if((i|0)==-1){f=-1;l=4621;break L6492}t=c[s>>2]|0;k=c[h>>2]|0;u=ac(k,i)|0;switch(k|0){case 3:{v=0;w=j;l=4611;break L6494;break};case 8:{l=4606;break L6494;break};case 4:{x=0;y=j;l=4610;break L6494;break};case 7:{z=0;A=j;l=4607;break L6494;break};case 2:{B=0;C=j;l=4612;break L6494;break};case 1:{D=0;E=j;break L6494;break};case 5:{F=0;G=j;l=4609;break L6494;break};case 6:{H=0;I=j;l=4608;break L6494;break};default:{}}}if((l|0)==4606){l=0;k=b+(j<<2)|0;g[k>>2]=+g[t+(u<<2)>>2]+ +g[k>>2];z=1;A=j+1|0;l=4607}if((l|0)==4607){l=0;k=b+(A<<2)|0;g[k>>2]=+g[t+(z+u<<2)>>2]+ +g[k>>2];H=z+1|0;I=A+1|0;l=4608}if((l|0)==4608){l=0;k=b+(I<<2)|0;g[k>>2]=+g[t+(H+u<<2)>>2]+ +g[k>>2];F=H+1|0;G=I+1|0;l=4609}if((l|0)==4609){l=0;k=b+(G<<2)|0;g[k>>2]=+g[t+(F+u<<2)>>2]+ +g[k>>2];x=F+1|0;y=G+1|0;l=4610}if((l|0)==4610){l=0;k=b+(y<<2)|0;g[k>>2]=+g[t+(x+u<<2)>>2]+ +g[k>>2];v=x+1|0;w=y+1|0;l=4611}if((l|0)==4611){l=0;k=b+(w<<2)|0;g[k>>2]=+g[t+(v+u<<2)>>2]+ +g[k>>2];B=v+1|0;C=w+1|0;l=4612}if((l|0)==4612){l=0;k=b+(C<<2)|0;g[k>>2]=+g[t+(B+u<<2)>>2]+ +g[k>>2];D=B+1|0;E=C+1|0}k=b+(E<<2)|0;g[k>>2]=+g[t+(D+u<<2)>>2]+ +g[k>>2];j=E+1|0}if((l|0)==4621){return f|0}else if((l|0)==4619){return f|0}return 0}function Gv(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;f=b;h=(e|0)>0;if((c[a+8>>2]|0)<=0){if(!h){i=0;return i|0}NM(f|0,0,e<<2|0);i=0;return i|0}if(!h){i=0;return i|0}h=a+16|0;f=a|0;j=0;while(1){k=Gs(a,d)|0;if((k|0)==-1){i=-1;l=4640;break}m=c[h>>2]|0;n=c[f>>2]|0;o=ac(n,k)|0;L6529:do{if((j|0)<(e|0)){k=0;p=j;q=n;while(1){if((k|0)>=(q|0)){r=p;break L6529}s=p+1|0;g[b+(p<<2)>>2]=+g[m+(k+o<<2)>>2];if((s|0)>=(e|0)){r=s;break L6529}k=k+1|0;p=s;q=c[f>>2]|0}}else{r=j}}while(0);if((r|0)<(e|0)){j=r}else{i=0;l=4639;break}}if((l|0)==4640){return i|0}else if((l|0)==4639){return i|0}return 0}function Gw(a,b,d,e,f,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;if((c[a+8>>2]|0)<=0){i=0;return i|0}j=(d|0)/(e|0)|0;k=(h+d|0)/(e|0)|0;if((j|0)>=(k|0)){i=0;return i|0}d=a+16|0;h=a|0;l=j;j=0;while(1){m=Gs(a,f)|0;if((m|0)==-1){i=-1;n=4650;break}o=c[d>>2]|0;p=c[h>>2]|0;q=ac(p,m)|0;if((p|0)>0){p=0;m=l;r=j;while(1){s=r+1|0;t=(c[b+(r<<2)>>2]|0)+(m<<2)|0;g[t>>2]=+g[o+(p+q<<2)>>2]+ +g[t>>2];t=(s|0)==(e|0);u=(t&1)+m|0;v=t?0:s;s=p+1|0;if((s|0)<(c[h>>2]|0)){p=s;m=u;r=v}else{w=u;x=v;break}}}else{w=l;x=j}if((w|0)<(k|0)){l=w;j=x}else{i=0;n=4651;break}}if((n|0)==4651){return i|0}else if((n|0)==4650){return i|0}return 0}function Gx(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;d=c[a+28>>2]|0;a=NC(96)|0;e=G9(b,8)|0;c[a>>2]=e;f=G9(b,16)|0;c[a+4>>2]=f;g=G9(b,16)|0;c[a+8>>2]=g;c[a+12>>2]=G9(b,6)|0;c[a+16>>2]=G9(b,8)|0;h=G9(b,4)|0;i=h+1|0;j=a+20|0;c[j>>2]=i;L6556:do{if(!((e|0)<1|(f|0)<1)){if((g|0)<1|(h|0)<0){break}k=a+24|0;l=d+24|0;m=d+1824|0;n=0;o=i;while(1){if((n|0)>=(o|0)){p=a;break}q=G9(b,8)|0;c[k+(n<<2)>>2]=q;if((q|0)<0){break L6556}if((q|0)>=(c[l>>2]|0)){break L6556}r=c[m+(q<<2)>>2]|0;if((c[r+12>>2]|0)==0){break L6556}if((c[r>>2]|0)<1){break L6556}n=n+1|0;o=c[j>>2]|0}return p|0}}while(0);if((a|0)==0){p=0;return p|0}ND(a);p=0;return p|0}function Gy(a,b){a=a|0;b=b|0;a=NE(1,32)|0;c[a+4>>2]=c[b>>2];c[a>>2]=c[b+8>>2];c[a+20>>2]=b;c[a+8>>2]=NE(2,4)|0;return a|0}function Gz(a){a=a|0;if((a|0)!=0){ND(a)}return}function GA(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;if((a|0)==0){return}b=a+8|0;d=c[b>>2]|0;if((d|0)!=0){e=c[d>>2]|0;if((e|0)==0){f=d}else{ND(e);f=c[b>>2]|0}e=c[f+4>>2]|0;if((e|0)==0){g=f}else{ND(e);g=c[b>>2]|0}ND(g)}ND(a);return}function GB(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0,i=0,j=0.0,k=0.0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;d=c[b+20>>2]|0;e=a+4|0;f=d+12|0;h=G9(e,c[f>>2]|0)|0;if((h|0)<=0){i=0;return i|0}j=+(h|0)/+((1<<c[f>>2])-1|0)*+(c[d+16>>2]|0);f=d+20|0;h=G9(e,F6(c[f>>2]|0)|0)|0;if((h|0)==-1){i=0;return i|0}if((h|0)>=(c[f>>2]|0)){i=0;return i|0}f=(c[(c[(c[(c[a+64>>2]|0)+4>>2]|0)+28>>2]|0)+2848>>2]|0)+((c[d+24+(h<<2)>>2]|0)*56|0)|0;h=b+4|0;b=f|0;d=E5(a,((c[b>>2]|0)+(c[h>>2]|0)<<2)+4|0)|0;a=d;if((Gv(f,a,e,c[h>>2]|0)|0)==-1){i=0;return i|0}e=c[h>>2]|0;if((e|0)>0){f=0;k=0.0;l=e;while(1){L6607:do{if((f|0)<(l|0)){m=0;n=f;o=l;while(1){if((m|0)>=(c[b>>2]|0)){p=n;q=o;break L6607}r=a+(n<<2)|0;g[r>>2]=k+ +g[r>>2];r=n+1|0;s=c[h>>2]|0;if((r|0)>=(s|0)){p=r;q=s;break L6607}m=m+1|0;n=r;o=s}}else{p=f;q=l}}while(0);if((p|0)<(q|0)){f=p;k=+g[a+(p-1<<2)>>2];l=q}else{t=q;break}}}else{t=e}g[a+(t<<2)>>2]=j;i=d;return i|0}function GC(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0.0,s=0.0,t=0.0,u=0,v=0;f=b;h=c[b+20>>2]|0;i=a+28|0;j=c[i>>2]|0;k=b+8|0;l=(c[k>>2]|0)+(j<<2)|0;if((c[l>>2]|0)==0){m=c[(c[(c[(c[a+64>>2]|0)+4>>2]|0)+28>>2]|0)+(j<<2)>>2]|0;a=(m|0)/2|0;n=b;o=c[n>>2]|0;p=h+4|0;q=+(c[p>>2]|0)*.5;c[l>>2]=NC((a<<2)+4|0)|0;if((m|0)>1){r=+Z(+(q*q*1.8499999754340024e-8));s=+(a|0);t=+(o|0)/(q*9999999747378752.0e-20+(r*2.240000009536743+ +Z(+(q*.0007399999885819852))*13.100000381469727));o=0;do{q=+(o|0)*(+(c[p>>2]|0)*.5/s);r=+Z(+(q*.0007399999885819852))*13.100000381469727;m=~~+Q(+(t*(q*9999999747378752.0e-20+(r+ +Z(+(q*q*1.8499999754340024e-8))*2.240000009536743))));l=c[n>>2]|0;c[(c[(c[k>>2]|0)+(j<<2)>>2]|0)+(o<<2)>>2]=(m|0)<(l|0)?m:l-1|0;o=o+1|0;}while((o|0)<(a|0));u=(a|0)>1?a:1}else{u=0}c[(c[(c[k>>2]|0)+(j<<2)>>2]|0)+(u<<2)>>2]=-1;c[f+12+(j<<2)>>2]=a}if((d|0)==0){NM(e|0,0,c[b+12+(c[i>>2]<<2)>>2]<<2|0);v=0;return v|0}else{a=d;d=c[b+4>>2]|0;j=c[i>>2]|0;GP(e,c[(c[k>>2]|0)+(j<<2)>>2]|0,c[b+12+(j<<2)>>2]|0,c[b>>2]|0,a,d,+g[a+(d<<2)>>2],+(c[h+16>>2]|0));v=1;return v|0}return 0}function GD(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0.0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0.0,aT=0.0,aU=0.0,aV=0,aW=0,aX=0,aY=0.0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0;f=i;i=i+3584|0;h=f|0;j=h;k=i;i=i+260|0;i=i+7&-8;l=i;i=i+260|0;i=i+7&-8;m=i;i=i+260|0;i=i+7&-8;n=m;o=i;i=i+260|0;i=i+7&-8;p=i;i=i+260|0;i=i+7&-8;q=p;r=i;i=i+4|0;i=i+7&-8;s=i;i=i+4|0;i=i+7&-8;t=i;i=i+4|0;i=i+7&-8;u=i;i=i+4|0;i=i+7&-8;v=i;i=i+4|0;i=i+7&-8;w=i;i=i+4|0;i=i+7&-8;x=c[b+1296>>2]|0;y=c[b+1288>>2]|0;z=c[b+1284>>2]|0;A=(z|0)>0;do{if(A){B=0;do{c[k+(B<<2)>>2]=-200;B=B+1|0;}while((B|0)<(z|0));if(A){C=0}else{break}do{c[l+(C<<2)>>2]=-200;C=C+1|0;}while((C|0)<(z|0));if(!A){break}NM(n|0,0,z<<2|0);B=0;do{c[o+(B<<2)>>2]=1;B=B+1|0;}while((B|0)<(z|0));if(!A){break}NM(q|0,-1|0,z<<2|0)}}while(0);if((z|0)==0){NM(j|0,0,56);c[h>>2]=0;c[h+4>>2]=y;j=y-1|0;if((j|0)<0){D=0;E=0;F=0;G=0;H=0;I=0;J=0;K=0;L=0;M=0;N=0;O=0}else{q=x+1112|0;A=0;n=0;C=0;B=0;P=0;Q=0;R=0;S=0;T=0;U=0;V=0;W=0;X=0;while(1){Y=+g[e+(A<<2)>>2];Z=~~(Y*7.314285755157471+1023.5);if((Z|0)>1023){_=1023;$=4728}else{aa=(Z|0)<0?0:Z;if((aa|0)==0){ab=X;ad=W;ae=V;af=U;ag=T;ah=S;ai=R;aj=Q;ak=P;al=B;am=C;an=n}else{_=aa;$=4728}}do{if(($|0)==4728){$=0;if(+g[d+(A<<2)>>2]+ +g[q>>2]<Y){aa=(ac(A,A)|0)+U|0;Z=(ac(_,_)|0)+V|0;ab=X+1|0;ad=(ac(_,A)|0)+W|0;ae=Z;af=aa;ag=_+T|0;ah=A+S|0;ai=R;aj=Q;ak=P;al=B;am=C;an=n;break}else{aa=(ac(A,A)|0)+B|0;Z=(ac(_,_)|0)+P|0;ab=X;ad=W;ae=V;af=U;ag=T;ah=S;ai=R+1|0;aj=(ac(_,A)|0)+Q|0;ak=Z;al=aa;am=_+C|0;an=A+n|0;break}}}while(0);aa=A+1|0;if((aa|0)>(j|0)){D=an;E=am;F=al;G=ak;H=aj;I=ai;J=ah;K=ag;L=af;M=ae;N=ad;O=ab;break}else{A=aa;n=an;C=am;B=al;P=ak;Q=aj;R=ai;S=ah;T=ag;U=af;V=ae;W=ad;X=ab}}}c[h+8>>2]=D;c[h+12>>2]=E;c[h+16>>2]=F;c[h+20>>2]=G;c[h+24>>2]=H;c[h+28>>2]=I;c[h+32>>2]=J;c[h+36>>2]=K;c[h+40>>2]=L;c[h+44>>2]=M;c[h+48>>2]=N;c[h+52>>2]=O;ao=I}else{I=z-1|0;if((I|0)<=0){ap=0;i=f;return ap|0}O=y-1|0;N=x+1112|0;M=0;L=0;K=c[b>>2]|0;while(1){J=L+1|0;H=c[b+(J<<2)>>2]|0;G=h+(L*56|0)|0;NM(G|0,0,56);c[G>>2]=K;c[h+(L*56|0)+4>>2]=H;G=(H|0)<(y|0)?H:O;if((G|0)<(K|0)){aq=0;ar=0;as=0;at=0;au=0;av=0;aw=0;ax=0;ay=0;az=0;aA=0;aB=0}else{F=K;E=0;D=0;ab=0;X=0;ad=0;W=0;ae=0;V=0;af=0;U=0;ag=0;T=0;while(1){Y=+g[e+(F<<2)>>2];ah=~~(Y*7.314285755157471+1023.5);if((ah|0)>1023){aC=1023;$=4736}else{S=(ah|0)<0?0:ah;if((S|0)==0){aD=T;aE=ag;aF=U;aG=af;aH=V;aI=ae;aJ=W;aK=ad;aL=X;aM=ab;aN=D;aO=E}else{aC=S;$=4736}}do{if(($|0)==4736){$=0;if(+g[d+(F<<2)>>2]+ +g[N>>2]<Y){S=(ac(F,F)|0)+af|0;ah=(ac(aC,aC)|0)+U|0;aD=T+1|0;aE=(ac(aC,F)|0)+ag|0;aF=ah;aG=S;aH=aC+V|0;aI=F+ae|0;aJ=W;aK=ad;aL=X;aM=ab;aN=D;aO=E;break}else{S=(ac(F,F)|0)+ab|0;ah=(ac(aC,aC)|0)+X|0;aD=T;aE=ag;aF=U;aG=af;aH=V;aI=ae;aJ=W+1|0;aK=(ac(aC,F)|0)+ad|0;aL=ah;aM=S;aN=aC+D|0;aO=F+E|0;break}}}while(0);S=F+1|0;if((S|0)>(G|0)){aq=aO;ar=aN;as=aM;at=aL;au=aK;av=aJ;aw=aI;ax=aH;ay=aG;az=aF;aA=aE;aB=aD;break}else{F=S;E=aO;D=aN;ab=aM;X=aL;ad=aK;W=aJ;ae=aI;V=aH;af=aG;U=aF;ag=aE;T=aD}}}c[h+(L*56|0)+8>>2]=aq;c[h+(L*56|0)+12>>2]=ar;c[h+(L*56|0)+16>>2]=as;c[h+(L*56|0)+20>>2]=at;c[h+(L*56|0)+24>>2]=au;c[h+(L*56|0)+28>>2]=av;c[h+(L*56|0)+32>>2]=aw;c[h+(L*56|0)+36>>2]=ax;c[h+(L*56|0)+40>>2]=ay;c[h+(L*56|0)+44>>2]=az;c[h+(L*56|0)+48>>2]=aA;c[h+(L*56|0)+52>>2]=aB;T=av+M|0;if((J|0)<(I|0)){M=T;L=J;K=H}else{ao=T;break}}}if((ao|0)==0){ap=0;i=f;return ap|0}c[r>>2]=-200;c[s>>2]=-200;GE(h|0,z-1|0,r,s,x)|0;ao=c[r>>2]|0;r=k|0;c[r>>2]=ao;K=l|0;c[K>>2]=ao;L=c[s>>2]|0;s=l+4|0;c[s>>2]=L;M=k+4|0;c[M>>2]=L;L=(z|0)>2;do{if(L){I=x+1112|0;av=x+1096|0;aB=x+1100|0;aA=x+1104|0;az=2;L6683:while(1){ay=c[b+520+(az<<2)>>2]|0;ax=c[m+(ay<<2)>>2]|0;aw=c[o+(ay<<2)>>2]|0;au=p+(ax<<2)|0;L6685:do{if((c[au>>2]|0)!=(aw|0)){at=c[b+520+(ax<<2)>>2]|0;as=c[b+520+(aw<<2)>>2]|0;c[au>>2]=aw;ar=c[x+836+(ax<<2)>>2]|0;aq=c[x+836+(aw<<2)>>2]|0;aD=c[k+(ax<<2)>>2]|0;aE=l+(ax<<2)|0;aF=c[aE>>2]|0;do{if((aD|0)<0){aP=aF}else{if((aF|0)<0){aP=aD;break}aP=aF+aD>>1}}while(0);aD=k+(aw<<2)|0;aF=c[aD>>2]|0;aG=c[l+(aw<<2)>>2]|0;do{if((aF|0)<0){aQ=aG}else{if((aG|0)<0){aQ=aF;break}aQ=aG+aF>>1}}while(0);if((aP|0)==-1|(aQ|0)==-1){$=4752;break L6683}aF=aQ-aP|0;aG=aq-ar|0;aH=(aF|0)/(aG|0)|0;aI=aF>>31|1;Y=+g[e+(ar<<2)>>2];aJ=~~(Y*7.314285755157471+1023.5);if((aJ|0)>1023){aR=1023}else{aR=(aJ|0)<0?0:aJ}aJ=ac(aH,aG)|0;aK=((aF|0)>-1?aF:-aF|0)-((aJ|0)>-1?aJ:-aJ|0)|0;aJ=aP-aR|0;aF=ac(aJ,aJ)|0;aS=+g[I>>2];do{if(+g[d+(ar<<2)>>2]+aS<Y){$=4758}else{aT=+(aP|0);aU=+(aR|0);if(aT+ +g[av>>2]<aU){break}if(aT- +g[aB>>2]<=aU){$=4758}}}while(0);L6702:do{if(($|0)==4758){$=0;aJ=ar+1|0;if((aJ|0)<(aq|0)){aL=aP;aM=0;aN=aF;aO=1;aC=aJ;while(1){aJ=aM+aK|0;N=(aJ|0)<(aG|0);O=aJ-(N?0:aG)|0;aJ=aL+aH+(N?0:aI)|0;Y=+g[e+(aC<<2)>>2];N=~~(Y*7.314285755157471+1023.5);if((N|0)>1023){aV=1023}else{aV=(N|0)<0?0:N}N=aJ-aV|0;y=(ac(N,N)|0)+aN|0;N=aO+1|0;if(!(aS+ +g[d+(aC<<2)>>2]<Y|(aV|0)==0)){Y=+(aJ|0);aU=+(aV|0);if(Y+ +g[av>>2]<aU){break L6702}if(Y- +g[aB>>2]>aU){break L6702}}T=aC+1|0;if((T|0)<(aq|0)){aL=aJ;aM=O;aN=y;aO=N;aC=T}else{aW=y;aX=N;break}}}else{aW=aF;aX=1}aU=+g[av>>2];Y=+(aX|0);aT=+g[aA>>2];do{if(aU*aU/Y<=aT){aY=+g[aB>>2];if(aY*aY/Y>aT){break}if(+((aW|0)/(aX|0)|0|0)>aT){break L6702}}}while(0);c[k+(az<<2)>>2]=-200;c[l+(az<<2)>>2]=-200;break L6685}}while(0);c[t>>2]=-200;c[u>>2]=-200;c[v>>2]=-200;c[w>>2]=-200;aF=GE(h+(at*56|0)|0,ay-at|0,t,u,x)|0;aq=GE(h+(ay*56|0)|0,as-ay|0,v,w,x)|0;aI=(aF|0)!=0;if(aI){c[t>>2]=aP;c[u>>2]=c[v>>2]}do{if((aq|0)!=0){c[v>>2]=c[u>>2];c[w>>2]=aQ;if(!aI){break}c[k+(az<<2)>>2]=-200;c[l+(az<<2)>>2]=-200;break L6685}}while(0);aI=c[t>>2]|0;c[aE>>2]=aI;if((ax|0)==0){c[r>>2]=aI}aI=c[u>>2]|0;c[k+(az<<2)>>2]=aI;aq=c[v>>2]|0;c[l+(az<<2)>>2]=aq;as=c[w>>2]|0;c[aD>>2]=as;if((aw|0)==1){c[s>>2]=as}if((aI&aq|0)<=-1){break}L6734:do{if((ay|0)>0){aq=ay;do{aq=aq-1|0;aI=o+(aq<<2)|0;if((c[aI>>2]|0)!=(aw|0)){break L6734}c[aI>>2]=az;}while((aq|0)>0)}}while(0);aD=ay+1|0;if((aD|0)<(z|0)){aZ=aD}else{break}do{aD=m+(aZ<<2)|0;if((c[aD>>2]|0)!=(ax|0)){break L6685}c[aD>>2]=az;aZ=aZ+1|0;}while((aZ|0)<(z|0))}}while(0);ax=az+1|0;if((ax|0)<(z|0)){az=ax}else{$=4786;break}}if(($|0)==4752){bp(1);return 0}else if(($|0)==4786){a_=c[r>>2]|0;a$=c[K>>2]|0;break}}else{a_=ao;a$=ao}}while(0);ao=E5(a,z<<2)|0;a=ao;do{if((a_|0)<0){a0=a$}else{if((a$|0)<0){a0=a_;break}a0=a$+a_>>1}}while(0);c[a>>2]=a0;a0=c[M>>2]|0;M=c[s>>2]|0;do{if((a0|0)<0){a1=M}else{if((M|0)<0){a1=a0;break}a1=M+a0>>1}}while(0);c[ao+4>>2]=a1;if(L){a2=2}else{ap=a;i=f;return ap|0}while(1){L=a2-2|0;a1=c[b+1032+(L<<2)>>2]|0;ao=c[b+780+(L<<2)>>2]|0;L=c[x+836+(a1<<2)>>2]|0;a0=c[a+(a1<<2)>>2]&32767;a1=(c[a+(ao<<2)>>2]&32767)-a0|0;M=(c[x+836+(ao<<2)>>2]|0)-L|0;ao=(ac((a1|0)>-1?a1:-a1|0,(c[x+836+(a2<<2)>>2]|0)-L|0)|0)/(M|0)|0;M=((a1|0)<0?-ao|0:ao)+a0|0;a0=c[k+(a2<<2)>>2]|0;ao=c[l+(a2<<2)>>2]|0;do{if((a0|0)<0){a3=ao}else{if((ao|0)<0){a3=a0;break}a3=ao+a0>>1}}while(0);if((a3|0)<0|(M|0)==(a3|0)){c[a+(a2<<2)>>2]=M|32768}else{c[a+(a2<<2)>>2]=a3}a0=a2+1|0;if((a0|0)<(z|0)){a2=a0}else{ap=a;break}}i=f;return ap|0}function GE(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0.0,k=0.0,l=0.0,m=0.0,n=0.0,o=0.0,p=0,q=0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0.0,y=0.0,z=0.0,A=0.0,B=0.0,C=0.0,D=0.0,E=0.0,F=0.0,G=0.0,H=0.0,I=0.0,J=0.0,K=0.0,L=0.0,M=0,N=0,O=0,P=0,Q=0,R=0;h=c[a>>2]|0;i=c[a+((b-1|0)*56|0)+4>>2]|0;if((b|0)>0){j=+g[f+1108>>2];k=0.0;f=0;l=0.0;m=0.0;n=0.0;o=0.0;while(1){p=c[a+(f*56|0)+52>>2]|0;q=c[a+(f*56|0)+28>>2]|0;r=j*+(q+p|0)/+(q+1|0)+1.0;s=o+(+(c[a+(f*56|0)+32>>2]|0)+r*+(c[a+(f*56|0)+8>>2]|0));t=n+(+(c[a+(f*56|0)+36>>2]|0)+r*+(c[a+(f*56|0)+12>>2]|0));u=m+(+(c[a+(f*56|0)+40>>2]|0)+r*+(c[a+(f*56|0)+16>>2]|0));v=l+(+(c[a+(f*56|0)+48>>2]|0)+r*+(c[a+(f*56|0)+24>>2]|0));w=k+(+(p|0)+ +(q|0)*r);q=f+1|0;if((q|0)<(b|0)){k=w;f=q;l=v;m=u;n=t;o=s}else{x=w;y=v;z=u;A=t;B=s;break}}}else{x=0.0;y=0.0;z=0.0;A=0.0;B=0.0}f=c[d>>2]|0;if((f|0)>-1){o=+(ac(h,h)|0)+z;C=+(h|0)+B;D=A+ +(f|0);E=o;F=y+ +(ac(f,h)|0);G=x+1.0}else{C=B;D=A;E=z;F=y;G=x}f=c[e>>2]|0;if((f|0)>-1){x=+(ac(i,i)|0)+E;H=+(i|0)+C;I=D+ +(f|0);J=x;K=F+ +(ac(f,i)|0);L=G+1.0}else{H=C;I=D;J=E;K=F;L=G}G=L*J-H*H;if(G<=0.0){c[d>>2]=0;c[e>>2]=0;M=1;return M|0}F=(J*I-K*H)/G;J=(L*K-I*H)/G;c[d>>2]=~~+bR(+(F+ +(h|0)*J));h=~~+bR(+(F+ +(i|0)*J));c[e>>2]=h;i=c[d>>2]|0;if((i|0)>1023){c[d>>2]=1023;N=c[e>>2]|0;O=1023}else{N=h;O=i}if((N|0)>1023){c[e>>2]=1023;P=c[d>>2]|0;Q=1023}else{P=O;Q=N}if((P|0)<0){c[d>>2]=0;R=c[e>>2]|0}else{R=Q}if((R|0)>=0){M=0;return M|0}c[e>>2]=0;M=0;return M|0}function GF(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0;g=c[b+1284>>2]|0;if((d|0)==0|(e|0)==0){h=0;return h|0}b=E5(a,g<<2)|0;if((g|0)<=0){h=b;return h|0}a=65536-f|0;i=0;while(1){j=d+(i<<2)|0;k=ac(c[j>>2]&32767,a)|0;l=e+(i<<2)|0;m=k+32768+(ac(c[l>>2]&32767,f)|0)>>16;k=b+(i<<2)|0;c[k>>2]=m;do{if((c[j>>2]&32768|0)!=0){if((c[l>>2]&32768|0)==0){break}c[k>>2]=m|32768}}while(0);m=i+1|0;if((m|0)<(g|0)){i=m}else{h=b;break}}return h|0}function GG(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;g=i;i=i+328|0;h=g|0;j=g+264|0;k=g+296|0;l=c[d+1296>>2]|0;m=d+1284|0;n=c[m>>2]|0;o=c[(c[(c[b+64>>2]|0)+4>>2]|0)+28>>2]|0;p=o+1824|0;q=c[o+2848>>2]|0;if((e|0)==0){G3(a,0,1);NM(f|0,0,((c[b+36>>2]|0)/2|0)<<2|0);r=0;i=g;return r|0}if((n|0)>0){s=l+832|0;t=0;do{u=e+(t<<2)|0;v=c[u>>2]|0;w=v&32767;x=c[s>>2]|0;if((x|0)==4){y=w>>>4}else if((x|0)==1){y=w>>>2}else if((x|0)==2){y=w>>>3}else if((x|0)==3){y=(w>>>0)/12|0}else{y=w}c[u>>2]=v&32768|y;t=t+1|0;}while((t|0)<(n|0))}t=h|0;c[t>>2]=c[e>>2];y=h+4|0;c[y>>2]=c[e+4>>2];s=d+1292|0;if((n|0)>2){v=2;do{u=v-2|0;w=c[d+1032+(u<<2)>>2]|0;x=c[d+780+(u<<2)>>2]|0;u=c[l+836+(w<<2)>>2]|0;z=e+(w<<2)|0;w=e+(x<<2)|0;A=c[z>>2]&32767;B=(c[w>>2]&32767)-A|0;C=(c[l+836+(x<<2)>>2]|0)-u|0;x=(ac((B|0)>-1?B:-B|0,(c[l+836+(v<<2)>>2]|0)-u|0)|0)/(C|0)|0;C=((B|0)<0?-x|0:x)+A|0;x=e+(v<<2)|0;B=c[x>>2]|0;if((B&32768|0)!=0|(C|0)==(B|0)){c[x>>2]=C|32768;c[h+(v<<2)>>2]=0}else{x=(c[s>>2]|0)-C|0;u=(x|0)<(C|0)?x:C;x=B-C|0;do{if((x|0)<0){if((x|0)<(-u|0)){D=u+~x|0;break}else{D=~(x<<1);break}}else{if((x|0)<(u|0)){D=x<<1;break}else{D=u+x|0;break}}}while(0);c[h+(v<<2)>>2]=D;c[z>>2]=A;c[w>>2]=c[w>>2]&32767}v=v+1|0;}while((v|0)<(n|0))}G3(a,1,1);n=d+1308|0;c[n>>2]=(c[n>>2]|0)+1;n=(c[s>>2]|0)-1|0;v=(n|0)==0;do{if(v){E=0;F=d+1304|0;G=c[t>>2]|0}else{D=n;x=0;do{x=x+1|0;D=D>>>1;}while((D|0)!=0);D=d+1304|0;c[D>>2]=(c[D>>2]|0)+(x<<1);w=c[t>>2]|0;if(v){E=0;F=D;G=w;break}else{H=n;I=0}while(1){A=I+1|0;z=H>>>1;if((z|0)==0){E=A;F=D;G=w;break}else{H=z;I=A}}}}while(0);G3(a,G,E);E=c[y>>2]|0;y=(c[s>>2]|0)-1|0;if((y|0)==0){J=0}else{s=y;y=0;while(1){G=y+1|0;I=s>>>1;if((I|0)==0){J=G;break}else{s=I;y=G}}}G3(a,E,J);J=l|0;if((c[J>>2]|0)>0){E=j;y=d+1300|0;s=2;G=0;while(1){I=c[l+4+(G<<2)>>2]|0;H=c[l+128+(I<<2)>>2]|0;n=c[l+192+(I<<2)>>2]|0;v=1<<n;NM(E|0,0,32);if((n|0)!=0){if((v|0)>0){t=0;do{w=c[l+320+(I<<5)+(t<<2)>>2]|0;if((w|0)<0){c[k+(t<<2)>>2]=1}else{c[k+(t<<2)>>2]=c[(c[p+(w<<2)>>2]|0)+4>>2]}t=t+1|0;}while((t|0)<(v|0))}if((H|0)>0){t=0;w=0;D=0;while(1){x=h+(D+s<<2)|0;A=0;while(1){if((A|0)>=(v|0)){K=4879;break}if((c[x>>2]|0)<(c[k+(A<<2)>>2]|0)){K=4878;break}else{A=A+1|0}}if((K|0)==4878){K=0;x=j+(D<<2)|0;c[x>>2]=A;L=x}else if((K|0)==4879){K=0;L=j+(D<<2)|0}x=c[L>>2]<<w|t;z=D+1|0;if((z|0)<(H|0)){t=x;w=w+n|0;D=z}else{M=x;break}}}else{M=0}D=Gq(q+((c[l+256+(I<<2)>>2]|0)*56|0)|0,M,a)|0;c[y>>2]=(c[y>>2]|0)+D}if((H|0)>0){D=0;do{n=c[l+320+(I<<5)+(c[j+(D<<2)>>2]<<2)>>2]|0;do{if((n|0)>-1){w=c[h+(D+s<<2)>>2]|0;if((w|0)>=(c[q+(n*56|0)+4>>2]|0)){break}t=Gq(q+(n*56|0)|0,w,a)|0;c[F>>2]=(c[F>>2]|0)+t}}while(0);D=D+1|0;}while((D|0)<(H|0))}D=G+1|0;if((D|0)<(c[J>>2]|0)){s=H+s|0;G=D}else{break}}}G=l+832|0;s=ac(c[G>>2]|0,c[e>>2]|0)|0;J=(c[o+(c[b+28>>2]<<2)>>2]|0)/2|0;if((c[m>>2]|0)>1){o=1;F=0;a=0;q=s;while(1){h=c[d+260+(o<<2)>>2]|0;j=c[e+(h<<2)>>2]|0;do{if((j&32767|0)==(j|0)){y=ac(c[G>>2]|0,j)|0;M=c[l+836+(h<<2)>>2]|0;L=y-q|0;K=M-a|0;k=(L|0)/(K|0)|0;p=L>>31|1;E=ac(k,K)|0;D=((L|0)>-1?L:-L|0)-((E|0)>-1?E:-E|0)|0;E=(J|0)>(M|0)?M:J;if((E|0)>(a|0)){c[f+(a<<2)>>2]=q}L=a+1|0;if((L|0)<(E|0)){N=q;O=0;P=L}else{Q=y;R=M;S=M;break}while(1){L=O+D|0;I=(L|0)<(K|0);n=N+k+(I?0:p)|0;c[f+(P<<2)>>2]=n;A=P+1|0;if((A|0)<(E|0)){N=n;O=L-(I?0:K)|0;P=A}else{Q=y;R=M;S=M;break}}}else{Q=q;R=a;S=F}}while(0);h=o+1|0;if((h|0)<(c[m>>2]|0)){o=h;F=S;a=R;q=Q}else{T=S;U=Q;break}}}else{T=0;U=s}s=b+36|0;if((T|0)<((c[s>>2]|0)/2|0|0)){V=T}else{r=1;i=g;return r|0}while(1){c[f+(V<<2)>>2]=U;T=V+1|0;if((T|0)<((c[s>>2]|0)/2|0|0)){V=T}else{r=1;break}}i=g;return r|0}function GH(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;d=a+836|0;e=c[a+840>>2]|0;f=a;G3(b,c[f>>2]|0,5);do{if((c[f>>2]|0)>0){g=a+4|0;h=0;i=-1;do{j=g+(h<<2)|0;G3(b,c[j>>2]|0,4);k=c[j>>2]|0;i=(i|0)<(k|0)?k:i;h=h+1|0;}while((h|0)<(c[f>>2]|0));h=i+1|0;if((h|0)<=0){break}g=a+128|0;k=a+192|0;j=a+256|0;l=a+320|0;m=0;do{G3(b,(c[g+(m<<2)>>2]|0)-1|0,3);n=k+(m<<2)|0;G3(b,c[n>>2]|0,2);if((c[n>>2]|0)==0){o=0}else{G3(b,c[j+(m<<2)>>2]|0,8);o=c[n>>2]|0}if((1<<o|0)>0){p=0;do{G3(b,(c[l+(m<<5)+(p<<2)>>2]|0)+1|0,8);p=p+1|0;}while((p|0)<(1<<c[n>>2]|0))}m=m+1|0;}while((m|0)<(h|0))}}while(0);G3(b,(c[a+832>>2]|0)-1|0,2);o=(e|0)==0?0:e-1|0;if((o|0)==0){G3(b,0,4);q=0}else{e=o;h=0;do{h=h+1|0;e=e>>>1;}while((e|0)!=0);G3(b,h,4);h=o;o=0;while(1){e=o+1|0;m=h>>>1;if((m|0)==0){q=e;break}else{h=m;o=e}}}o=c[f>>2]|0;if((o|0)<=0){return}h=a+4|0;e=a+128|0;a=0;m=0;l=0;j=o;while(1){o=(c[e+(c[h+(l<<2)>>2]<<2)>>2]|0)+a|0;if((m|0)<(o|0)){k=m;do{G3(b,c[d+(k+2<<2)>>2]|0,q);k=k+1|0;}while((k|0)<(o|0));r=o;s=c[f>>2]|0}else{r=m;s=j}k=l+1|0;if((k|0)<(s|0)){a=o;m=r;l=k;j=s}else{break}}return}function GI(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;d=i;i=i+264|0;e=d|0;f=c[a+28>>2]|0;a=NE(1,1120)|0;g=G9(b,5)|0;h=a;c[h>>2]=g;L6946:do{if((g|0)>0){j=a+4|0;k=-1;l=0;do{m=G9(b,4)|0;c[j+(l<<2)>>2]=m;if((m|0)<0){break L6946}k=(k|0)<(m|0)?m:k;l=l+1|0;}while((l|0)<(c[h>>2]|0));l=k+1|0;if((l|0)<=0){n=4943;break}j=a+128|0;m=a+192|0;o=a+256|0;p=f+24|0;q=a+320|0;r=0;while(1){c[j+(r<<2)>>2]=(G9(b,3)|0)+1;s=G9(b,2)|0;t=m+(r<<2)|0;c[t>>2]=s;if((s|0)<0){break L6946}if((s|0)==0){u=c[o+(r<<2)>>2]|0}else{s=G9(b,8)|0;c[o+(r<<2)>>2]=s;u=s}if((u|0)<0){n=4957;break L6946}if((u|0)<(c[p>>2]|0)){v=0}else{n=4957;break L6946}while(1){if((v|0)>=(1<<c[t>>2]|0)){break}s=G9(b,8)|0;w=s-1|0;c[q+(r<<5)+(v<<2)>>2]=w;if((s|0)<0){break L6946}if((w|0)>=(c[p>>2]|0)){n=4957;break L6946}v=v+1|0}t=r+1|0;if((t|0)<(l|0)){r=t}else{n=4943;break}}}else{n=4943}}while(0);L6967:do{if((n|0)==4943){c[a+832>>2]=(G9(b,2)|0)+1;v=G9(b,4)|0;if((v|0)<0){break}if((c[h>>2]|0)>0){u=a+4|0;f=a+128|0;g=a+836|0;r=g;l=1<<v;p=0;q=0;o=0;while(1){x=(c[f+(c[u+(o<<2)>>2]<<2)>>2]|0)+p|0;if((x|0)>63){n=4957;break L6967}else{y=q}while(1){if((y|0)>=(x|0)){break}m=G9(b,v)|0;c[r+(y+2<<2)>>2]=m;if((m|0)>-1&(m|0)<(l|0)){y=y+1|0}else{break L6967}}m=o+1|0;if((m|0)<(c[h>>2]|0)){p=x;q=y;o=m}else{break}}o=x+2|0;q=g;c[q>>2]=0;c[a+840>>2]=l;if((o|0)>0){z=o;A=q;n=4952}else{B=o}}else{o=a+836|0;c[o>>2]=0;c[a+840>>2]=1<<v;z=2;A=o;n=4952}if((n|0)==4952){o=0;while(1){c[e+(o<<2)>>2]=A+(o<<2);q=o+1|0;if((q|0)<(z|0)){o=q}else{B=z;break}}}b6(e|0,B|0,4,10);o=1;while(1){if((o|0)>=(B|0)){C=a;break}if((c[c[e+(o-1<<2)>>2]>>2]|0)==(c[c[e+(o<<2)>>2]>>2]|0)){n=4957;break L6967}else{o=o+1|0}}i=d;return C|0}}while(0);do{if((n|0)==4957){if((a|0)==0){C=0}else{break}i=d;return C|0}}while(0);ND(a);C=0;i=d;return C|0}function GJ(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;a=i;i=i+264|0;d=a|0;e=NE(1,1312)|0;c[e+1296>>2]=b;f=b+836|0;g=f;h=e+1288|0;c[h>>2]=c[b+840>>2];j=c[b>>2]|0;do{if((j|0)>0){k=b+4|0;l=b+128|0;m=0;n=0;do{m=(c[l+(c[k+(n<<2)>>2]<<2)>>2]|0)+m|0;n=n+1|0;}while((n|0)<(j|0));n=m+2|0;c[e+1284>>2]=n;if((n|0)>0){o=m;p=n;q=4969;break}b6(d|0,n|0,4,10);r=m}else{c[e+1284>>2]=2;o=0;p=2;q=4969}}while(0);if((q|0)==4969){q=f;j=0;do{c[d+(j<<2)>>2]=q+(j<<2);j=j+1|0;}while((j|0)<(p|0));b6(d|0,p|0,4,10);j=f;f=e+260|0;q=0;do{c[f+(q<<2)>>2]=(c[d+(q<<2)>>2]|0)-j>>2;q=q+1|0;}while((q|0)<(p|0));q=e+260|0;j=e+520|0;d=0;do{c[j+(c[q+(d<<2)>>2]<<2)>>2]=d;d=d+1|0;}while((d|0)<(p|0));d=e+260|0;q=e;j=0;while(1){c[q+(j<<2)>>2]=c[g+(c[d+(j<<2)>>2]<<2)>>2];f=j+1|0;if((f|0)<(p|0)){j=f}else{r=o;break}}}o=c[b+832>>2]|0;if((o|0)==4){c[e+1292>>2]=64}else if((o|0)==3){c[e+1292>>2]=86}else if((o|0)==2){c[e+1292>>2]=128}else if((o|0)==1){c[e+1292>>2]=256}if((r|0)<=0){i=a;return e|0}o=e+1032|0;b=e+780|0;j=0;do{p=j+2|0;d=c[g+(p<<2)>>2]|0;if((p|0)>0){q=0;f=1;n=0;k=c[h>>2]|0;l=0;while(1){s=c[g+(l<<2)>>2]|0;t=(s|0)>(n|0)&(s|0)<(d|0);u=t?l:q;v=(s|0)<(k|0)&(s|0)>(d|0);w=v?l:f;x=l+1|0;if((x|0)<(p|0)){q=u;f=w;n=t?s:n;k=v?s:k;l=x}else{y=u;z=w;break}}}else{y=0;z=1}c[o+(j<<2)>>2]=y;c[b+(j<<2)>>2]=z;j=j+1|0;}while((j|0)<(r|0));i=a;return e|0}function GK(a){a=a|0;if((a|0)!=0){ND(a)}return}function GL(a){a=a|0;if((a|0)!=0){ND(a)}return}function GM(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;d=c[b+1296>>2]|0;e=c[(c[(c[(c[a+64>>2]|0)+4>>2]|0)+28>>2]|0)+2848>>2]|0;f=a+4|0;if((G9(f,1)|0)!=1){g=0;return g|0}h=b+1284|0;i=E5(a,c[h>>2]<<2)|0;a=i;j=b+1292|0;k=(c[j>>2]|0)-1|0;if((k|0)==0){l=0}else{m=k;k=0;while(1){n=k+1|0;o=m>>>1;if((o|0)==0){l=n;break}else{m=o;k=n}}}c[a>>2]=G9(f,l)|0;l=(c[j>>2]|0)-1|0;if((l|0)==0){p=0}else{k=l;l=0;while(1){m=l+1|0;n=k>>>1;if((n|0)==0){p=m;break}else{k=n;l=m}}}c[i+4>>2]=G9(f,p)|0;p=d|0;L7053:do{if((c[p>>2]|0)>0){l=2;k=0;L7054:while(1){m=c[d+4+(k<<2)>>2]|0;n=c[d+128+(m<<2)>>2]|0;o=c[d+192+(m<<2)>>2]|0;q=1<<o;if((o|0)==0){r=0}else{s=Gr(e+((c[d+256+(m<<2)>>2]|0)*56|0)|0,f)|0;if((s|0)==-1){g=0;t=5027;break}else{r=s}}if((n|0)>0){s=q-1|0;q=r;u=0;do{v=c[d+320+(m<<5)+((q&s)<<2)>>2]|0;q=q>>o;if((v|0)>-1){w=Gr(e+(v*56|0)|0,f)|0;c[a+(u+l<<2)>>2]=w;if((w|0)==-1){g=0;t=5030;break L7054}}else{c[a+(u+l<<2)>>2]=0}u=u+1|0;}while((u|0)<(n|0))}u=k+1|0;if((u|0)<(c[p>>2]|0)){l=n+l|0;k=u}else{break L7053}}if((t|0)==5030){return g|0}else if((t|0)==5027){return g|0}}}while(0);if((c[h>>2]|0)<=2){g=i;return g|0}t=b+1032|0;p=b+780|0;b=2;while(1){f=b-2|0;e=t+(f<<2)|0;r=c[e>>2]|0;k=c[d+836+(r<<2)>>2]|0;l=p+(f<<2)|0;f=c[l>>2]|0;u=c[a+(r<<2)>>2]&32767;r=(c[a+(f<<2)>>2]&32767)-u|0;o=(c[d+836+(f<<2)>>2]|0)-k|0;f=(ac((r|0)>-1?r:-r|0,(c[d+836+(b<<2)>>2]|0)-k|0)|0)/(o|0)|0;o=((r|0)<0?-f|0:f)+u|0;u=(c[j>>2]|0)-o|0;f=a+(b<<2)|0;r=c[f>>2]|0;if((r|0)==0){c[f>>2]=o|32768}else{do{if((r|0)<(((u|0)<(o|0)?u:o)<<1|0)){if((r&1|0)==0){x=r>>1;break}else{x=-(r+1>>1)|0;break}}else{if((u|0)>(o|0)){x=r-o|0;break}else{x=~(r-u);break}}}while(0);c[f>>2]=x+o&32767;u=a+(c[e>>2]<<2)|0;c[u>>2]=c[u>>2]&32767;u=a+(c[l>>2]<<2)|0;c[u>>2]=c[u>>2]&32767}u=b+1|0;if((u|0)<(c[h>>2]|0)){b=u}else{g=i;break}}return g|0}function GN(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0.0;f=c[b+1296>>2]|0;h=(c[(c[(c[(c[a+64>>2]|0)+4>>2]|0)+28>>2]|0)+(c[a+28>>2]<<2)>>2]|0)/2|0;if((d|0)==0){NM(e|0,0,h<<2|0);i=0;return i|0}a=d;d=f+832|0;j=ac(c[d>>2]|0,c[a>>2]|0)|0;if((j|0)<0){k=0}else{k=(j|0)>255?255:j}j=b+1284|0;if((c[j>>2]|0)>1){l=b+260|0;b=1;m=0;n=0;o=k;while(1){p=c[l+(b<<2)>>2]|0;q=c[a+(p<<2)>>2]|0;do{if((q&32767|0)==(q|0)){r=c[f+836+(p<<2)>>2]|0;s=ac(c[d>>2]|0,q)|0;if((s|0)<0){t=0}else{t=(s|0)>255?255:s}s=t-o|0;u=r-n|0;v=(s|0)/(u|0)|0;w=s>>31|1;x=ac(v,u)|0;y=((s|0)>-1?s:-s|0)-((x|0)>-1?x:-x|0)|0;x=(h|0)>(r|0)?r:h;if((x|0)>(n|0)){s=e+(n<<2)|0;g[s>>2]=+g[1974208+(o<<2)>>2]*+g[s>>2]}s=n+1|0;if((s|0)<(x|0)){z=o;A=0;B=s}else{C=t;D=r;E=r;break}while(1){s=A+y|0;F=(s|0)<(u|0);G=z+v+(F?0:w)|0;H=e+(B<<2)|0;g[H>>2]=+g[1974208+(G<<2)>>2]*+g[H>>2];H=B+1|0;if((H|0)<(x|0)){z=G;A=s-(F?0:u)|0;B=H}else{C=t;D=r;E=r;break}}}else{C=o;D=n;E=m}}while(0);q=b+1|0;if((q|0)<(c[j>>2]|0)){b=q;m=E;n=D;o=C}else{I=E;J=C;break}}}else{I=0;J=k}if((I|0)>=(h|0)){i=1;return i|0}K=+g[1974208+(J<<2)>>2];J=I;while(1){I=e+(J<<2)|0;g[I>>2]=K*+g[I>>2];I=J+1|0;if((I|0)<(h|0)){J=I}else{i=1;break}}return i|0}function GO(a,b){a=a|0;b=b|0;return(c[c[a>>2]>>2]|0)-(c[c[b>>2]>>2]|0)|0}function GP(a,b,d,e,f,h,i,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;i=+i;j=+j;var k=0.0,l=0,m=0.0,n=0,o=0.0,p=0.0,q=0.0,r=0,s=0,t=0,u=0,v=0.0,w=0.0,x=0;k=3.141592653589793/+(e|0);if((h|0)>0){e=0;do{l=f+(e<<2)|0;g[l>>2]=+U(+(+g[l>>2]))*2.0;e=e+1|0;}while((e|0)<(h|0))}if((d|0)<=0){return}m=i;i=j;if((h|0)>1){n=0}else{e=0;while(1){l=c[b+(e<<2)>>2]|0;j=+U(+(k*+(l|0)))*2.0;if((h|0)==1){o=(j- +g[f>>2])*.5;p=o*o;q=4.0-j*j}else{p=(j+2.0)*.5*.5;q=2.0-j}j=+$(+((m/+S(+(p+q*.5*.5))-i)*.1151292473077774));r=a+(e<<2)|0;g[r>>2]=+g[r>>2]*j;r=e+1|0;if((c[b+(r<<2)>>2]|0)==(l|0)){s=r;while(1){t=a+(s<<2)|0;g[t>>2]=j*+g[t>>2];t=s+1|0;if((c[b+(t<<2)>>2]|0)==(l|0)){s=t}else{u=t;break}}}else{u=r}if((u|0)<(d|0)){e=u}else{break}}return}while(1){u=c[b+(n<<2)>>2]|0;q=+U(+(k*+(u|0)))*2.0;e=1;p=.5;j=.5;do{j=j*(q- +g[f+(e-1<<2)>>2]);p=p*(q- +g[f+(e<<2)>>2]);e=e+2|0;}while((e|0)<(h|0));if((e|0)==(h|0)){o=j*(q- +g[f+(h-1<<2)>>2]);v=o*o;w=4.0-q*q}else{v=j*(q+2.0)*j;w=2.0-q}o=+$(+((m/+S(+(v+p*p*w))-i)*.1151292473077774));r=a+(n<<2)|0;g[r>>2]=+g[r>>2]*o;r=n+1|0;if((c[b+(r<<2)>>2]|0)==(u|0)){s=r;while(1){l=a+(s<<2)|0;g[l>>2]=o*+g[l>>2];l=s+1|0;if((c[b+(l<<2)>>2]|0)==(u|0)){s=l}else{x=l;break}}}else{x=r}if((x|0)<(d|0)){n=x}else{break}}return}function GQ(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;e=b;if((c[e>>2]|0)>1){G3(d,1,1);G3(d,(c[e>>2]|0)-1|0,4)}else{G3(d,0,1)}f=b+1156|0;do{if((c[f>>2]|0)>0){G3(d,1,1);G3(d,(c[f>>2]|0)-1|0,8);if((c[f>>2]|0)<=0){break}g=b+1160|0;h=a+4|0;i=b+2184|0;j=0;do{k=c[g+(j<<2)>>2]|0;l=c[h>>2]|0;m=(l|0)==0?0:l-1|0;if((m|0)==0){n=0}else{l=m;m=0;while(1){o=m+1|0;p=l>>>1;if((p|0)==0){n=o;break}else{l=p;m=o}}}G3(d,k,n);m=c[i+(j<<2)>>2]|0;l=c[h>>2]|0;o=(l|0)==0?0:l-1|0;if((o|0)==0){q=0}else{l=o;o=0;while(1){p=o+1|0;r=l>>>1;if((r|0)==0){q=p;break}else{l=r;o=p}}}G3(d,m,q);j=j+1|0;}while((j|0)<(c[f>>2]|0))}else{G3(d,0,1)}}while(0);G3(d,0,2);f=c[e>>2]|0;do{if((f|0)>1){q=a+4|0;if((c[q>>2]|0)<=0){break}n=b+4|0;j=0;do{G3(d,c[n+(j<<2)>>2]|0,4);j=j+1|0;}while((j|0)<(c[q>>2]|0));s=c[e>>2]|0;t=5091}else{s=f;t=5091}}while(0);do{if((t|0)==5091){if((s|0)>0){break}return}}while(0);s=b+1028|0;t=b+1092|0;b=0;do{G3(d,0,8);G3(d,c[s+(b<<2)>>2]|0,8);G3(d,c[t+(b<<2)>>2]|0,8);b=b+1|0;}while((b|0)<(c[e>>2]|0));return}function GR(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;d=NE(1,3208)|0;e=c[a+28>>2]|0;NM(d|0,0,3208);f=G9(b,1)|0;L7191:do{if((f|0)<0){g=5124}else{if((f|0)==0){c[d>>2]=1}else{h=G9(b,4)|0;c[d>>2]=h+1;if((h|0)<0){break}}h=G9(b,1)|0;if((h|0)<0){g=5124;break}L7198:do{if((h|0)!=0){i=G9(b,8)|0;j=i+1|0;k=d+1156|0;c[k>>2]=j;if((i|0)<0){g=5124;break L7191}i=a+4|0;l=d+1160|0;m=d+2184|0;n=0;o=j;while(1){if((n|0)>=(o|0)){break L7198}j=c[i>>2]|0;p=(j|0)==0?0:j-1|0;if((p|0)==0){q=0}else{j=p;p=0;while(1){r=p+1|0;s=j>>>1;if((s|0)==0){q=r;break}else{j=s;p=r}}}p=G9(b,q)|0;c[l+(n<<2)>>2]=p;j=c[i>>2]|0;r=(j|0)==0?0:j-1|0;if((r|0)==0){t=0}else{j=r;r=0;while(1){s=r+1|0;u=j>>>1;if((u|0)==0){t=s;break}else{j=u;r=s}}}r=G9(b,t)|0;c[m+(n<<2)>>2]=r;if((r|p|0)<0|(p|0)==(r|0)){g=5124;break L7191}j=c[i>>2]|0;if(!((p|0)<(j|0)&(r|0)<(j|0))){g=5124;break L7191}n=n+1|0;o=c[k>>2]|0}}}while(0);if((G9(b,2)|0)!=0){g=5124;break}h=d;k=c[h>>2]|0;L7216:do{if((k|0)>1){o=a+4|0;n=d+4|0;i=0;m=k;while(1){if((i|0)>=(c[o>>2]|0)){v=m;break L7216}l=G9(b,4)|0;c[n+(i<<2)>>2]=l;j=c[h>>2]|0;if((l|0)>=(j|0)|(l|0)<0){break L7191}else{i=i+1|0;m=j}}}else{v=k}}while(0);k=d+1028|0;m=e+16|0;i=d+1092|0;n=e+20|0;o=0;j=v;while(1){if((o|0)>=(j|0)){w=d;break}G9(b,8)|0;l=G9(b,8)|0;c[k+(o<<2)>>2]=l;if((l|0)>=(c[m>>2]|0)|(l|0)<0){g=5124;break L7191}l=G9(b,8)|0;c[i+(o<<2)>>2]=l;if((l|0)>=(c[n>>2]|0)|(l|0)<0){g=5124;break L7191}o=o+1|0;j=c[h>>2]|0}return w|0}}while(0);do{if((g|0)==5124){if((d|0)==0){w=0}else{break}return w|0}}while(0);ND(d);w=0;return w|0}function GS(a){a=a|0;if((a|0)!=0){ND(a)}return}function GT(a){a=a|0;var b=0,d=0,e=0,f=0,h=0,j=0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0.0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0.0,L=0,M=0,N=0,O=0.0,P=0,Q=0.0,R=0.0,S=0,T=0.0,U=0.0,V=0.0,W=0.0,X=0.0,Y=0.0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0;b=i;d=c[a+64>>2]|0;e=c[d+4>>2]|0;f=c[e+28>>2]|0;h=c[d+104>>2]|0;d=c[a+104>>2]|0;j=c[a+36>>2]|0;l=e+4|0;e=c[l>>2]<<2;m=i;i=i+e|0;i=i+7&-8;n=m;m=E5(a,e)|0;e=E5(a,c[l>>2]<<2)|0;o=E5(a,c[l>>2]<<2)|0;p=d+4|0;q=+g[p>>2];r=i;i=i+(c[l>>2]<<2)|0;i=i+7&-8;s=r;r=a+28|0;t=c[r>>2]|0;u=c[f+544+(t<<2)>>2]|0;v=u;w=(c[h+56>>2]|0)+((((t|0)!=0?2:0)+(c[d+8>>2]|0)|0)*52|0)|0;x=a+40|0;c[x>>2]=t;if((c[l>>2]|0)>0){y=a|0;z=(j|0)/2|0;A=z<<2;B=+(((g[k>>2]=4.0/+(j|0),c[k>>2]|0)&2147483647)>>>0>>>0)*7.177114298428933e-7+ -764.6162109375+.345;C=h+4|0;D=f;E=a+24|0;F=a+32|0;G=h+12|0;H=h+20|0;I=j-1|0;J=(I|0)>1;K=q;L=0;while(1){M=c[(c[y>>2]|0)+(L<<2)>>2]|0;c[e+(L<<2)>>2]=E5(a,A)|0;N=m+(L<<2)|0;c[N>>2]=E5(a,A)|0;Gn(M,C,D,c[E>>2]|0,c[r>>2]|0,c[F>>2]|0);FE(c[c[G+(c[r>>2]<<2)>>2]>>2]|0,M,c[N>>2]|0);Gf(H+((c[r>>2]|0)*12|0)|0,M);O=B+(+(((g[k>>2]=+g[M>>2],c[k>>2]|0)&2147483647)>>>0>>>0)*7.177114298428933e-7+ -764.6162109375)+.345;g[M>>2]=O;N=s+(L<<2)|0;g[N>>2]=O;if(J){P=1;Q=O;while(1){R=+g[M+(P<<2)>>2];S=P+1|0;T=+g[M+(S<<2)>>2];U=B+(+(((g[k>>2]=R*R+T*T,c[k>>2]|0)&2147483647)>>>0>>>0)*7.177114298428933e-7+ -764.6162109375)*.5+.345;g[M+(S>>1<<2)>>2]=U;if(U>Q){g[N>>2]=U;V=U}else{V=Q}S=P+2|0;if((S|0)<(I|0)){P=S;Q=V}else{W=V;break}}}else{W=O}if(W>0.0){g[N>>2]=0.0;X=0.0}else{X=W}Q=X>K?X:K;P=L+1|0;if((P|0)<(c[l>>2]|0)){K=Q;L=P}else{Y=Q;Z=z;_=A;break}}}else{A=(j|0)/2|0;Y=q;Z=A;_=A<<2}A=E5(a,_)|0;z=E5(a,_)|0;_=u+4|0;L7254:do{if((c[l>>2]|0)>0){L=a|0;I=u+1028|0;J=f+800|0;H=h+48|0;if((j|0)>1){$=0}else{G=0;while(1){F=c[_+(G<<2)>>2]|0;E=c[m+(G<<2)>>2]|0;D=c[(c[L>>2]|0)+(G<<2)>>2]|0;C=D+(Z<<2)|0;c[x>>2]=t;y=E5(a,60)|0;P=o+(G<<2)|0;c[P>>2]=y;NM(y|0,0,60);FL(w,C,A);FN(w,D,z,Y,+g[s+(G<<2)>>2]);FO(w,A,z,1,D,E,C);y=I+(F<<2)|0;F=c[y>>2]|0;if((c[J+(F<<2)>>2]|0)!=1){aa=-1;break}M=GD(a,c[(c[H>>2]|0)+(F<<2)>>2]|0,C,D)|0;c[(c[P>>2]|0)+28>>2]=M;do{if((E1(a)|0)!=0){if((c[(c[P>>2]|0)+28>>2]|0)==0){break}FO(w,A,z,2,D,E,C);M=GD(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,C,D)|0;c[(c[P>>2]|0)+56>>2]=M;FO(w,A,z,0,D,E,C);M=GD(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,C,D)|0;c[c[P>>2]>>2]=M;M=c[P>>2]|0;F=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[M>>2]|0,c[M+28>>2]|0,9362)|0;c[(c[P>>2]|0)+4>>2]=F;F=c[P>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[F>>2]|0,c[F+28>>2]|0,18724)|0;c[(c[P>>2]|0)+8>>2]=M;M=c[P>>2]|0;F=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[M>>2]|0,c[M+28>>2]|0,28086)|0;c[(c[P>>2]|0)+12>>2]=F;F=c[P>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[F>>2]|0,c[F+28>>2]|0,37449)|0;c[(c[P>>2]|0)+16>>2]=M;M=c[P>>2]|0;F=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[M>>2]|0,c[M+28>>2]|0,46811)|0;c[(c[P>>2]|0)+20>>2]=F;F=c[P>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[F>>2]|0,c[F+28>>2]|0,56173)|0;c[(c[P>>2]|0)+24>>2]=M;M=c[P>>2]|0;F=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[M+28>>2]|0,c[M+56>>2]|0,9362)|0;c[(c[P>>2]|0)+32>>2]=F;F=c[P>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[F+28>>2]|0,c[F+56>>2]|0,18724)|0;c[(c[P>>2]|0)+36>>2]=M;M=c[P>>2]|0;F=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[M+28>>2]|0,c[M+56>>2]|0,28086)|0;c[(c[P>>2]|0)+40>>2]=F;F=c[P>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[F+28>>2]|0,c[F+56>>2]|0,37449)|0;c[(c[P>>2]|0)+44>>2]=M;M=c[P>>2]|0;F=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[M+28>>2]|0,c[M+56>>2]|0,46811)|0;c[(c[P>>2]|0)+48>>2]=F;F=c[P>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[y>>2]<<2)>>2]|0,c[F+28>>2]|0,c[F+56>>2]|0,56173)|0;c[(c[P>>2]|0)+52>>2]=M}}while(0);P=G+1|0;if((P|0)<(c[l>>2]|0)){G=P}else{ab=I;ac=H;break L7254}}i=b;return aa|0}while(1){G=c[_+($<<2)>>2]|0;N=c[m+($<<2)>>2]|0;P=c[(c[L>>2]|0)+($<<2)>>2]|0;y=P+(Z<<2)|0;c[x>>2]=t;D=E5(a,60)|0;C=o+($<<2)|0;c[C>>2]=D;NM(D|0,0,60);D=0;do{g[P+(D+Z<<2)>>2]=+(((g[k>>2]=+g[N+(D<<2)>>2],c[k>>2]|0)&2147483647)>>>0>>>0)*7.177114298428933e-7+ -764.6162109375+.345;D=D+1|0;}while((D|0)<(Z|0));FL(w,y,A);FN(w,P,z,Y,+g[s+($<<2)>>2]);FO(w,A,z,1,P,N,y);D=I+(G<<2)|0;E=c[D>>2]|0;if((c[J+(E<<2)>>2]|0)!=1){aa=-1;break}M=GD(a,c[(c[H>>2]|0)+(E<<2)>>2]|0,y,P)|0;c[(c[C>>2]|0)+28>>2]=M;do{if((E1(a)|0)!=0){if((c[(c[C>>2]|0)+28>>2]|0)==0){break}FO(w,A,z,2,P,N,y);M=GD(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,y,P)|0;c[(c[C>>2]|0)+56>>2]=M;FO(w,A,z,0,P,N,y);M=GD(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,y,P)|0;c[c[C>>2]>>2]=M;M=c[C>>2]|0;E=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[M>>2]|0,c[M+28>>2]|0,9362)|0;c[(c[C>>2]|0)+4>>2]=E;E=c[C>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[E>>2]|0,c[E+28>>2]|0,18724)|0;c[(c[C>>2]|0)+8>>2]=M;M=c[C>>2]|0;E=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[M>>2]|0,c[M+28>>2]|0,28086)|0;c[(c[C>>2]|0)+12>>2]=E;E=c[C>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[E>>2]|0,c[E+28>>2]|0,37449)|0;c[(c[C>>2]|0)+16>>2]=M;M=c[C>>2]|0;E=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[M>>2]|0,c[M+28>>2]|0,46811)|0;c[(c[C>>2]|0)+20>>2]=E;E=c[C>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[E>>2]|0,c[E+28>>2]|0,56173)|0;c[(c[C>>2]|0)+24>>2]=M;M=c[C>>2]|0;E=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[M+28>>2]|0,c[M+56>>2]|0,9362)|0;c[(c[C>>2]|0)+32>>2]=E;E=c[C>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[E+28>>2]|0,c[E+56>>2]|0,18724)|0;c[(c[C>>2]|0)+36>>2]=M;M=c[C>>2]|0;E=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[M+28>>2]|0,c[M+56>>2]|0,28086)|0;c[(c[C>>2]|0)+40>>2]=E;E=c[C>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[E+28>>2]|0,c[E+56>>2]|0,37449)|0;c[(c[C>>2]|0)+44>>2]=M;M=c[C>>2]|0;E=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[M+28>>2]|0,c[M+56>>2]|0,46811)|0;c[(c[C>>2]|0)+48>>2]=E;E=c[C>>2]|0;M=GF(a,c[(c[H>>2]|0)+(c[D>>2]<<2)>>2]|0,c[E+28>>2]|0,c[E+56>>2]|0,56173)|0;c[(c[C>>2]|0)+52>>2]=M}}while(0);C=$+1|0;if((C|0)<(c[l>>2]|0)){$=C}else{ab=I;ac=H;break L7254}}i=b;return aa|0}else{ab=u+1028|0;ac=h+48|0}}while(0);g[p>>2]=Y;p=c[l>>2]<<2;$=i;i=i+p|0;i=i+7&-8;z=$;$=i;i=i+p|0;i=i+7&-8;p=$;$=(E1(a)|0)!=0?0:7;E1(a)|0;A=d+12|0;d=h+44|0;s=a+24|0;Z=a+32|0;x=f+2868|0;j=f+3240|0;H=u;I=u+1092|0;u=f+1312|0;f=h+52|0;h=$;while(1){$=c[A+(h<<2)>>2]|0;G3($,0,1);G3($,t,c[d>>2]|0);if((c[r>>2]|0)!=0){G3($,c[s>>2]|0,1);G3($,c[Z>>2]|0,1)}J=c[l>>2]|0;if((J|0)>0){L=0;while(1){c[n+(L<<2)>>2]=GG($,a,c[(c[ac>>2]|0)+(c[ab+(c[_+(L<<2)>>2]<<2)>>2]<<2)>>2]|0,c[(c[o+(L<<2)>>2]|0)+(h<<2)>>2]|0,c[e+(L<<2)>>2]|0)|0;C=L+1|0;D=c[l>>2]|0;if((C|0)<(D|0)){L=C}else{ad=D;break}}}else{ad=J}FQ(h,x,w,v,m,e,n,c[j+((c[r>>2]|0)*60|0)+(h<<2)>>2]|0,ad);if((c[H>>2]|0)>0){L=0;do{D=c[I+(L<<2)>>2]|0;C=c[l>>2]|0;if((C|0)>0){P=0;y=0;N=C;while(1){if((c[_+(P<<2)>>2]|0)==(L|0)){C=p+(y<<2)|0;c[C>>2]=0;if((c[n+(P<<2)>>2]|0)!=0){c[C>>2]=1}c[z+(y<<2)>>2]=c[e+(P<<2)>>2];ae=y+1|0;af=c[l>>2]|0}else{ae=y;af=N}C=P+1|0;if((C|0)<(af|0)){P=C;y=ae;N=af}else{ag=ae;break}}}else{ag=0}N=u+(D<<2)|0;y=c4[c[(c[1745208+(c[N>>2]<<2)>>2]|0)+20>>2]&63](a,c[(c[f>>2]|0)+(D<<2)>>2]|0,z,p,ag)|0;P=c[l>>2]|0;if((P|0)>0){C=0;G=0;M=P;while(1){if((c[_+(C<<2)>>2]|0)==(L|0)){c[z+(G<<2)>>2]=c[e+(C<<2)>>2];ah=G+1|0;ai=c[l>>2]|0}else{ah=G;ai=M}P=C+1|0;if((P|0)<(ai|0)){C=P;G=ah;M=ai}else{aj=ah;break}}}else{aj=0}dp[c[(c[1745208+(c[N>>2]<<2)>>2]|0)+24>>2]&7]($,a,c[(c[f>>2]|0)+(D<<2)>>2]|0,z,p,aj,y,L)|0;L=L+1|0;}while((L|0)<(c[H>>2]|0))}L=h+1|0;if((L|0)>(((E1(a)|0)!=0?14:7)|0)){aa=0;break}else{h=L}}i=b;return aa|0}function GU(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0.0,I=0.0,J=0;d=i;e=c[a+64>>2]|0;f=c[e+4>>2]|0;h=c[f+28>>2]|0;j=c[e+104>>2]|0;e=a+28|0;k=c[h+(c[e>>2]<<2)>>2]|0;c[a+36>>2]=k;l=f+4|0;f=c[l>>2]<<2;m=i;i=i+f|0;i=i+7&-8;n=m;m=i;i=i+f|0;i=i+7&-8;o=m;m=i;i=i+f|0;i=i+7&-8;p=m;m=i;i=i+f|0;i=i+7&-8;f=m;m=c[l>>2]|0;if((m|0)>0){q=b+4|0;r=b+1028|0;s=h+800|0;t=j+48|0;u=a|0;v=k<<1&2147483646;w=0;while(1){x=c[r+(c[q+(w<<2)>>2]<<2)>>2]|0;y=dg[c[(c[1900584+(c[s+(x<<2)>>2]<<2)>>2]|0)+20>>2]&255](a,c[(c[t>>2]|0)+(x<<2)>>2]|0)|0;c[f+(w<<2)>>2]=y;c[p+(w<<2)>>2]=(y|0)!=0;NM(c[(c[u>>2]|0)+(w<<2)>>2]|0,0,v|0);y=w+1|0;x=c[l>>2]|0;if((y|0)<(x|0)){w=y}else{z=x;break}}}else{z=m}m=b+1156|0;w=c[m>>2]|0;if((w|0)>0){v=b+1160|0;u=b+2184|0;t=0;do{s=p+(c[v+(t<<2)>>2]<<2)|0;q=c[u+(t<<2)>>2]|0;if((c[s>>2]|0)==0){if((c[p+(q<<2)>>2]|0)!=0){A=5189}}else{A=5189}if((A|0)==5189){A=0;c[s>>2]=1;c[p+(q<<2)>>2]=1}t=t+1|0;}while((t|0)<(w|0))}t=b;if((c[t>>2]|0)>0){A=b+1092|0;u=h+1312|0;v=j+52|0;q=b+4|0;s=a|0;r=0;x=z;while(1){if((x|0)>0){z=0;y=0;B=x;while(1){if((c[q+(y<<2)>>2]|0)==(r|0)){c[o+(z<<2)>>2]=(c[p+(y<<2)>>2]|0)!=0;c[n+(z<<2)>>2]=c[(c[s>>2]|0)+(y<<2)>>2];C=z+1|0;D=c[l>>2]|0}else{C=z;D=B}E=y+1|0;if((E|0)<(D|0)){z=C;y=E;B=D}else{F=C;break}}}else{F=0}B=c[A+(r<<2)>>2]|0;c4[c[(c[1745208+(c[u+(B<<2)>>2]<<2)>>2]|0)+28>>2]&63](a,c[(c[v>>2]|0)+(B<<2)>>2]|0,n,o,F)|0;B=r+1|0;if((B|0)>=(c[t>>2]|0)){break}r=B;x=c[l>>2]|0}G=c[m>>2]|0}else{G=w}if((G|0)>0){w=b+1160|0;m=a|0;x=b+2184|0;r=(k|0)/2|0;t=(k|0)>1;k=G;do{k=k-1|0;G=c[m>>2]|0;F=c[G+(c[w+(k<<2)>>2]<<2)>>2]|0;o=c[G+(c[x+(k<<2)>>2]<<2)>>2]|0;if(t){G=0;do{n=F+(G<<2)|0;H=+g[n>>2];v=o+(G<<2)|0;I=+g[v>>2];u=I>0.0;do{if(H>0.0){if(u){g[n>>2]=H;g[v>>2]=H-I;break}else{g[v>>2]=H;g[n>>2]=H+I;break}}else{if(u){g[n>>2]=H;g[v>>2]=H+I;break}else{g[v>>2]=H;g[n>>2]=H-I;break}}}while(0);G=G+1|0;}while((G|0)<(r|0))}}while((k|0)>0)}if((c[l>>2]|0)<=0){i=d;return 0}k=a|0;r=b+4|0;t=b+1028|0;b=h+800|0;h=j+48|0;x=0;do{w=c[t+(c[r+(x<<2)>>2]<<2)>>2]|0;dk[c[(c[1900584+(c[b+(w<<2)>>2]<<2)>>2]|0)+24>>2]&63](a,c[(c[h>>2]|0)+(w<<2)>>2]|0,c[f+(x<<2)>>2]|0,c[(c[k>>2]|0)+(x<<2)>>2]|0)|0;x=x+1|0;J=c[l>>2]|0;}while((x|0)<(J|0));if((J|0)<=0){i=d;return 0}J=a|0;a=j+12|0;j=0;do{x=c[(c[J>>2]|0)+(j<<2)>>2]|0;FB(c[c[a+(c[e>>2]<<2)>>2]>>2]|0,x,x);j=j+1|0;}while((j|0)<(c[l>>2]|0));i=d;return 0}function GV(a){a=a|0;var b=0,d=0,e=0,f=0,i=0,j=0,k=0,l=0.0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0.0,R=0.0,S=0.0,T=0,U=0,V=0,W=0,X=0.0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0.0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0;b=a+28|0;d=c[b>>2]|0;e=d+3392|0;if((d|0)==0){f=-131;return f|0}i=(c[d+3456>>2]|0)==0;j=i&1;k=d+3496|0;l=+h[k>>3];do{if(l>-80.0){h[k>>3]=-80.0}else{if(l>=-200.0){break}h[k>>3]=-200.0}}while(0);k=d+3512|0;l=+h[k>>3];do{if(l>0.0){h[k>>3]=0.0}else{if(l>=-99999.0){break}h[k>>3]=-99999.0}}while(0);k=c[d+3396>>2]|0;if((k|0)==0){f=-131;return f|0}c[e>>2]=1;e=d+3400|0;m=c[b>>2]|0;n=~~+h[e>>3];o=c[(c[k+28>>2]|0)+(n<<2)>>2]|0;c[m>>2]=c[(c[k+24>>2]|0)+(n<<2)>>2];c[m+4>>2]=o;o=(c[d>>2]|0)==(c[d+4>>2]|0);m=k+144|0;if((c[m>>2]|0)>0){n=k+136|0;p=k+140|0;q=k+148|0;r=0;do{s=~~+h[e>>3];t=c[n>>2]|0;u=c[p>>2]|0;v=c[(c[q>>2]|0)+(r<<2)>>2]|0;w=NE(1,1120)|0;x=c[b>>2]|0;y=v+(s<<2)|0;s=c[y>>2]|0;v=u+(s*1120|0)|0;NN(w|0,v|0,1120)|0;v=c[w>>2]|0;do{if((v|0)>0){u=w+4|0;z=0;A=-1;do{B=c[u+(z<<2)>>2]|0;A=(B|0)>(A|0)?B:A;z=z+1|0;}while((z|0)<(v|0));if((A|0)<0){break}z=w+256|0;u=x+24|0;B=w+192|0;C=w+320|0;D=0;E=-1;while(1){F=z+(D<<2)|0;G=c[F>>2]|0;H=(G|0)>(E|0)?G:E;c[F>>2]=(c[u>>2]|0)+G;G=B+(D<<2)|0;F=c[G>>2]|0;if((1<<F|0)>0){I=0;J=H;K=F;while(1){F=C+(D<<5)+(I<<2)|0;L=c[F>>2]|0;M=(L|0)>(J|0)?L:J;if((L|0)>-1){c[F>>2]=(c[u>>2]|0)+L;N=c[G>>2]|0}else{N=K}L=I+1|0;if((L|0)<(1<<N|0)){I=L;J=M;K=N}else{O=M;break}}}else{O=H}K=D+1|0;if((K|0)>(A|0)){break}else{D=K;E=O}}if((O|0)<0){break}E=x+1824|0;D=c[c[t+(s<<2)>>2]>>2]|0;A=c[u>>2]|0;c[u>>2]=A+1;c[E+(A<<2)>>2]=D;if((O|0)<1){break}else{P=1}do{D=c[(c[t+(c[y>>2]<<2)>>2]|0)+(P<<2)>>2]|0;A=c[u>>2]|0;c[u>>2]=A+1;c[E+(A<<2)>>2]=D;P=P+1|0;}while((P|0)<=(O|0))}}while(0);y=x+16|0;c[x+800+(c[y>>2]<<2)>>2]=1;c[x+1056+(c[y>>2]<<2)>>2]=w;c[y>>2]=(c[y>>2]|0)+1;r=r+1|0;}while((r|0)<(c[m>>2]|0))}l=+h[d+3520>>3];m=c[k+124>>2]|0;r=c[k+128>>2]|0;O=c[b>>2]|0;P=~~l;Q=l- +(P|0);N=O+2868|0;q=r+(P<<3)|0;p=m+(~~+h[q>>3]*492|0)|0;NN(N|0,p|0,492)|0;l=(1.0-Q)*+h[q>>3]+Q*+h[r+(P+1<<3)>>3];P=~~l;Q=l- +(P|0);r=Q==0.0&(P|0)>0;l=r?1.0:Q;q=(r<<31>>31)+P|0;Q=1.0-l;P=q+1|0;g[O+2872>>2]=+g[m+(P*492|0)+4>>2]*l+ +g[m+(q*492|0)+4>>2]*Q;g[O+2900>>2]=+g[m+(P*492|0)+32>>2]*l+ +g[m+(q*492|0)+32>>2]*Q;g[O+2876>>2]=+g[m+(P*492|0)+8>>2]*l+ +g[m+(q*492|0)+8>>2]*Q;g[O+2904>>2]=+g[m+(q*492|0)+36>>2]*Q+l*+g[m+(P*492|0)+36>>2];g[O+2880>>2]=Q*+g[m+(q*492|0)+12>>2]+l*+g[m+(P*492|0)+12>>2];g[O+2908>>2]=Q*+g[m+(q*492|0)+40>>2]+l*+g[m+(P*492|0)+40>>2];g[O+2884>>2]=Q*+g[m+(q*492|0)+16>>2]+l*+g[m+(P*492|0)+16>>2];g[O+2912>>2]=Q*+g[m+(q*492|0)+44>>2]+l*+g[m+(P*492|0)+44>>2];g[O+2936>>2]=+h[O+3512>>3];O=c[k+132>>2]|0;l=+h[d+3472>>3];P=~~l;Q=l- +(P|0);m=c[b>>2]|0;L7412:do{if((O|0)==0){q=c[m>>2]|0;c[m+3240>>2]=q;r=c[m+4>>2]|0;c[m+3300>>2]=r;c[m+3244>>2]=q;c[m+3304>>2]=r;c[m+3248>>2]=q;c[m+3308>>2]=r;c[m+3252>>2]=q;c[m+3312>>2]=r;c[m+3256>>2]=q;c[m+3316>>2]=r;c[m+3260>>2]=q;c[m+3320>>2]=r;c[m+3264>>2]=q;c[m+3324>>2]=r;c[m+3268>>2]=q;c[m+3328>>2]=r;c[m+3272>>2]=q;c[m+3332>>2]=r;c[m+3276>>2]=q;c[m+3336>>2]=r;c[m+3280>>2]=q;c[m+3340>>2]=r;c[m+3284>>2]=q;c[m+3344>>2]=r;c[m+3288>>2]=q;c[m+3348>>2]=r;c[m+3292>>2]=q;c[m+3352>>2]=r;c[m+3296>>2]=q;c[m+3356>>2]=r}else{r=m+3120|0;q=O+(P*240|0)|0;NN(r|0,q|0,60)|0;q=m+3180|0;r=O+(P*240|0)+60|0;NN(q|0,r|0,60)|0;if((c[d+3420>>2]|0)!=0){l=1.0-Q;r=P+1|0;q=a+8|0;p=m;N=m+3e3|0;n=m+4|0;y=m+3060|0;t=m+2940|0;s=m+3240|0;v=m+3300|0;E=0;while(1){R=l*+g[O+(P*240|0)+120+(E<<2)>>2]+Q*+g[O+(r*240|0)+120+(E<<2)>>2];S=R*1.0e3;c[N+(E<<2)>>2]=~~(S/+(c[q>>2]|0)*+(c[p>>2]|0));c[y+(E<<2)>>2]=~~(S/+(c[q>>2]|0)*+(c[n>>2]|0));c[t+(E<<2)>>2]=~~R;R=(l*+g[O+(P*240|0)+180+(E<<2)>>2]+Q*+g[O+(r*240|0)+180+(E<<2)>>2])*1.0e3;c[s+(E<<2)>>2]=~~(R/+(c[q>>2]|0)*+(c[p>>2]|0));c[v+(E<<2)>>2]=~~(R/+(c[q>>2]|0)*+(c[n>>2]|0));E=E+1|0;if((E|0)>=15){break L7412}}}l=1.0-Q;E=P+1|0;R=l*+g[O+(P*240|0)+148>>2]+Q*+g[O+(E*240|0)+148>>2];S=R*1.0e3;n=a+8|0;q=m;v=m+3e3|0;p=m+4|0;s=m+3060|0;r=~~R;t=m+2940|0;y=0;do{c[v+(y<<2)>>2]=~~(S/+(c[n>>2]|0)*+(c[q>>2]|0));c[s+(y<<2)>>2]=~~(S/+(c[n>>2]|0)*+(c[p>>2]|0));c[t+(y<<2)>>2]=r;y=y+1|0;}while((y|0)<15);S=(l*+g[O+(P*240|0)+208>>2]+Q*+g[O+(E*240|0)+208>>2])*1.0e3;y=m+3240|0;r=m+3300|0;t=0;do{c[y+(t<<2)>>2]=~~(S/+(c[n>>2]|0)*+(c[q>>2]|0));c[r+(t<<2)>>2]=~~(S/+(c[n>>2]|0)*+(c[p>>2]|0));t=t+1|0;}while((t|0)<15)}}while(0);m=k+92|0;O=c[m>>2]|0;P=k+100|0;t=c[P>>2]|0;p=k+108|0;n=c[p>>2]|0;r=c[b>>2]|0;q=r+2852|0;y=c[q>>2]|0;E=~~+h[e>>3];s=r+28|0;if((c[s>>2]|0)<=0){c[s>>2]=1}if((y|0)==0){s=NE(1,520)|0;c[q>>2]=s;T=s}else{T=y}y=T;NN(y|0,1805952,520)|0;c[T>>2]=0;if((c[r+3460>>2]|0)!=0){c[T+500>>2]=1;c[T+504>>2]=c[O+(E<<2)>>2];c[T+508>>2]=c[t+(E<<2)>>2];h[T+512>>3]=+h[n+(E<<3)>>3]}E=c[m>>2]|0;m=c[P>>2]|0;P=c[p>>2]|0;n=c[b>>2]|0;T=n+2856|0;t=c[T>>2]|0;O=~~+h[e>>3];r=n+28|0;if((c[r>>2]|0)<=1){c[r>>2]=2}if((t|0)==0){r=NE(1,520)|0;c[T>>2]=r;U=r}else{U=t}t=U;NN(t|0,1805952,520)|0;c[U>>2]=0;if((c[n+3460>>2]|0)!=0){c[U+500>>2]=1;c[U+504>>2]=c[E+(O<<2)>>2];c[U+508>>2]=c[m+(O<<2)>>2];h[U+512>>3]=+h[P+(O<<3)>>3]}do{if(!o){O=k+96|0;P=c[O>>2]|0;U=k+104|0;m=c[U>>2]|0;E=c[p>>2]|0;n=c[b>>2]|0;t=n+2860|0;r=c[t>>2]|0;T=~~+h[e>>3];y=n+28|0;if((c[y>>2]|0)<=2){c[y>>2]=3}if((r|0)==0){y=NE(1,520)|0;c[t>>2]=y;V=y}else{V=r}r=V;NN(r|0,1805952,520)|0;c[V>>2]=1;if((c[n+3460>>2]|0)!=0){c[V+500>>2]=1;c[V+504>>2]=c[P+(T<<2)>>2];c[V+508>>2]=c[m+(T<<2)>>2];h[V+512>>3]=+h[E+(T<<3)>>3]}T=c[O>>2]|0;O=c[U>>2]|0;U=c[p>>2]|0;E=c[b>>2]|0;m=E+2864|0;P=c[m>>2]|0;n=~~+h[e>>3];r=E+28|0;if((c[r>>2]|0)<=3){c[r>>2]=4}if((P|0)==0){r=NE(1,520)|0;c[m>>2]=r;W=r}else{W=P}P=W;NN(P|0,1805952,520)|0;c[W>>2]=1;if((c[E+3460>>2]|0)==0){break}c[W+500>>2]=1;c[W+504>>2]=c[T+(n<<2)>>2];c[W+508>>2]=c[O+(n<<2)>>2];h[W+512>>3]=+h[U+(n<<3)>>3]}}while(0);W=d+3528|0;p=k+32|0;V=k+36|0;GW(c[b>>2]|0,+h[W+(j<<5)>>3],0,c[p>>2]|0,c[V>>2]|0,c[k+44>>2]|0);n=k+52|0;GW(c[b>>2]|0,+h[d+3560>>3],1,c[p>>2]|0,c[V>>2]|0,c[n>>2]|0);if(!o){GW(c[b>>2]|0,+h[d+3592>>3],2,c[p>>2]|0,c[V>>2]|0,c[n>>2]|0);GW(c[b>>2]|0,+h[d+3624>>3],3,c[p>>2]|0,c[V>>2]|0,c[k+48>>2]|0)}Q=+h[W+(j<<5)+24>>3];V=k+80|0;p=c[V>>2]|0;n=k+84|0;U=c[n>>2]|0;O=~~Q;S=Q- +(O|0);T=c[(c[b>>2]|0)+2852>>2]|0;Q=(1.0-S)*+h[U+(O<<3)>>3]+S*+h[U+(O+1<<3)>>3];O=~~Q;S=Q- +(O|0);U=S==0.0&(O|0)>0;Q=U?1.0:S;E=(U<<31>>31)+O|0;S=1.0-Q;O=E+1|0;U=0;do{g[T+336+(U<<2)>>2]=S*+(c[p+(E*160|0)+(U<<2)>>2]|0)+Q*+(c[p+(O*160|0)+(U<<2)>>2]|0);U=U+1|0;}while((U|0)<40);Q=+h[d+3584>>3];U=c[V>>2]|0;O=c[n>>2]|0;n=~~Q;S=Q- +(n|0);p=c[(c[b>>2]|0)+2856>>2]|0;Q=(1.0-S)*+h[O+(n<<3)>>3]+S*+h[O+(n+1<<3)>>3];n=~~Q;S=Q- +(n|0);O=S==0.0&(n|0)>0;Q=O?1.0:S;E=(O<<31>>31)+n|0;S=1.0-Q;n=E+1|0;O=0;do{g[p+336+(O<<2)>>2]=S*+(c[U+(E*160|0)+(O<<2)>>2]|0)+Q*+(c[U+(n*160|0)+(O<<2)>>2]|0);O=O+1|0;}while((O|0)<40);if(!o){Q=+h[d+3616>>3];O=c[V>>2]|0;n=k+88|0;U=c[n>>2]|0;E=~~Q;S=Q- +(E|0);p=c[(c[b>>2]|0)+2860>>2]|0;Q=(1.0-S)*+h[U+(E<<3)>>3]+S*+h[U+(E+1<<3)>>3];E=~~Q;S=Q- +(E|0);U=S==0.0&(E|0)>0;Q=U?1.0:S;T=(U<<31>>31)+E|0;S=1.0-Q;E=T+1|0;U=0;do{g[p+336+(U<<2)>>2]=S*+(c[O+(T*160|0)+(U<<2)>>2]|0)+Q*+(c[O+(E*160|0)+(U<<2)>>2]|0);U=U+1|0;}while((U|0)<40);Q=+h[d+3648>>3];U=c[V>>2]|0;V=c[n>>2]|0;n=~~Q;S=Q- +(n|0);E=c[(c[b>>2]|0)+2864>>2]|0;Q=(1.0-S)*+h[V+(n<<3)>>3]+S*+h[V+(n+1<<3)>>3];n=~~Q;S=Q- +(n|0);V=S==0.0&(n|0)>0;Q=V?1.0:S;O=(V<<31>>31)+n|0;S=1.0-Q;n=O+1|0;V=0;do{g[E+336+(V<<2)>>2]=S*+(c[U+(O*160|0)+(V<<2)>>2]|0)+Q*+(c[U+(n*160|0)+(V<<2)>>2]|0);V=V+1|0;}while((V|0)<40)}Q=+h[W+(j<<5)+8>>3];V=k+40|0;n=c[V>>2]|0;U=~~Q;S=Q- +(U|0);g[(c[(c[b>>2]|0)+2852>>2]|0)+32>>2]=(1.0-S)*+(c[n+(U<<2)>>2]|0)+S*+(c[n+(U+1<<2)>>2]|0);S=+h[d+3568>>3];U=c[V>>2]|0;n=~~S;Q=S- +(n|0);g[(c[(c[b>>2]|0)+2856>>2]|0)+32>>2]=(1.0-Q)*+(c[U+(n<<2)>>2]|0)+Q*+(c[U+(n+1<<2)>>2]|0);if(!o){Q=+h[d+3600>>3];n=c[V>>2]|0;U=~~Q;S=Q- +(U|0);g[(c[(c[b>>2]|0)+2860>>2]|0)+32>>2]=(1.0-S)*+(c[n+(U<<2)>>2]|0)+S*+(c[n+(U+1<<2)>>2]|0);S=+h[d+3632>>3];U=c[V>>2]|0;V=~~S;Q=S- +(V|0);g[(c[(c[b>>2]|0)+2864>>2]|0)+32>>2]=(1.0-Q)*+(c[U+(V<<2)>>2]|0)+Q*+(c[U+(V+1<<2)>>2]|0)}V=k+76|0;U=k+56|0;if(i){X=0.0}else{X=+h[d+3408>>3]}GX(c[b>>2]|0,+h[W+(j<<5)+16>>3],0,c[V>>2]|0,c[k+60>>2]|0,c[U>>2]|0,X);GX(c[b>>2]|0,+h[d+3576>>3],1,c[V>>2]|0,c[k+64>>2]|0,c[U>>2]|0,0.0);if(o){o=c[b>>2]|0;j=c[o+2852>>2]|0;g[j+4>>2]=+h[o+3496>>3];g[j+8>>2]=+h[o+3504>>3];o=c[b>>2]|0;j=c[o+2856>>2]|0;g[j+4>>2]=+h[o+3496>>3];g[j+8>>2]=+h[o+3504>>3]}else{GX(c[b>>2]|0,+h[d+3608>>3],2,c[V>>2]|0,c[k+68>>2]|0,c[U>>2]|0,0.0);GX(c[b>>2]|0,+h[d+3640>>3],3,c[V>>2]|0,c[k+72>>2]|0,c[U>>2]|0,0.0);U=c[b>>2]|0;V=c[U+2852>>2]|0;g[V+4>>2]=+h[U+3496>>3];g[V+8>>2]=+h[U+3504>>3];U=c[b>>2]|0;V=c[U+2856>>2]|0;g[V+4>>2]=+h[U+3496>>3];g[V+8>>2]=+h[U+3504>>3];U=c[b>>2]|0;V=c[U+2860>>2]|0;g[V+4>>2]=+h[U+3496>>3];g[V+8>>2]=+h[U+3504>>3];U=c[b>>2]|0;V=c[U+2864>>2]|0;g[V+4>>2]=+h[U+3496>>3];g[V+8>>2]=+h[U+3504>>3]}U=c[k+152>>2]|0;k=c[b>>2]|0;V=~~+h[e>>3];e=c[U+(V<<3)>>2]|0;o=c[U+(V<<3)+4>>2]|0;V=(c[k>>2]|0)==(c[k+4>>2]|0)?1:2;U=k+544|0;j=k+32|0;W=k+8|0;i=k+288|0;n=k+12|0;k=a+8|0;O=a+4|0;E=0;do{T=U+(E<<2)|0;c[T>>2]=NE(1,3208)|0;p=NE(1,16)|0;c[j+(E<<2)>>2]=p;P=1811024+(E<<4)|0;c[p>>2]=c[P>>2];c[p+4>>2]=c[P+4>>2];c[p+8>>2]=c[P+8>>2];c[p+12>>2]=c[P+12>>2];if((E|0)>=(c[W>>2]|0)){c[W>>2]=E+1}c[i+(E<<2)>>2]=0;P=c[T>>2]|0;T=e+(E*3208|0)|0;p=T;NN(P|0,p|0,3208)|0;if((E|0)>=(c[n>>2]|0)){c[n>>2]=E+1}p=T|0;if((c[p>>2]|0)>0){T=0;do{P=c[e+(E*3208|0)+1092+(T<<2)>>2]|0;r=c[b>>2]|0;m=r;y=NC(2840)|0;c[r+1568+(P<<2)>>2]=y;t=c[o+(P<<5)+12>>2]|0;NN(y|0,t|0,2840)|0;t=r+20|0;if((c[t>>2]|0)<=(P|0)){c[t>>2]=P+1}t=y+8|0;c[t>>2]=c[o+(P<<5)+8>>2];s=r+1312+(P<<2)|0;c[s>>2]=c[o+(P<<5)>>2];q=r+3420|0;v=y+12|0;N=(c[v>>2]|0)>0;do{if((c[q>>2]|0)==0){if(N){w=y+24|0;x=c[o+(P<<5)+24>>2]|0;u=0;do{D=w+(u<<2)|0;if((c[x+(u<<4)>>2]|0)!=0){c[D>>2]=c[D>>2]|1}if((c[x+(u<<4)+4>>2]|0)!=0){c[D>>2]=c[D>>2]|2}if((c[x+(u<<4)+8>>2]|0)!=0){c[D>>2]=c[D>>2]|4}if((c[x+(u<<4)+12>>2]|0)!=0){c[D>>2]=c[D>>2]|8}u=u+1|0;}while((u|0)<(c[v>>2]|0))}u=o+(P<<5)+16|0;x=c[u>>2]|0;w=r+24|0;H=c[w>>2]|0;D=0;while(1){if((D|0)>=(H|0)){Y=5333;break}if((c[m+1824+(D<<2)>>2]|0)==(x|0)){Z=D;_=x;break}else{D=D+1|0}}if((Y|0)==5333){Y=0;c[w>>2]=H+1;Z=H;_=c[u>>2]|0}c[y+20>>2]=Z;D=r+1824|0;c[D+(Z<<2)>>2]=_;if((c[v>>2]|0)<=0){break}x=o+(P<<5)+24|0;A=y+280|0;C=0;B=0;while(1){z=c[x>>2]|0;K=c[z+(B<<4)>>2]|0;if((K|0)==0){$=C;aa=z}else{J=c[w>>2]|0;I=0;while(1){if((I|0)>=(J|0)){Y=5340;break}if((c[m+1824+(I<<2)>>2]|0)==(K|0)){ab=I;ad=z;break}else{I=I+1|0}}if((Y|0)==5340){Y=0;c[w>>2]=J+1;ab=J;ad=c[x>>2]|0}c[A+(C<<2)>>2]=ab;c[D+(ab<<2)>>2]=c[ad+(B<<4)>>2];$=C+1|0;aa=c[x>>2]|0}I=c[aa+(B<<4)+4>>2]|0;if((I|0)==0){ae=$;af=aa}else{z=c[w>>2]|0;K=0;while(1){if((K|0)>=(z|0)){Y=5365;break}if((c[m+1824+(K<<2)>>2]|0)==(I|0)){ag=K;ah=aa;break}else{K=K+1|0}}if((Y|0)==5365){Y=0;c[w>>2]=z+1;ag=z;ah=c[x>>2]|0}c[A+($<<2)>>2]=ag;c[D+(ag<<2)>>2]=c[ah+(B<<4)+4>>2];ae=$+1|0;af=c[x>>2]|0}K=c[af+(B<<4)+8>>2]|0;if((K|0)==0){ai=ae;aj=af}else{I=c[w>>2]|0;J=0;while(1){if((J|0)>=(I|0)){Y=5371;break}if((c[m+1824+(J<<2)>>2]|0)==(K|0)){ak=J;al=af;break}else{J=J+1|0}}if((Y|0)==5371){Y=0;c[w>>2]=I+1;ak=I;al=c[x>>2]|0}c[A+(ae<<2)>>2]=ak;c[D+(ak<<2)>>2]=c[al+(B<<4)+8>>2];ai=ae+1|0;aj=c[x>>2]|0}J=c[aj+(B<<4)+12>>2]|0;if((J|0)==0){am=ai}else{K=c[w>>2]|0;z=0;while(1){if((z|0)>=(K|0)){Y=5377;break}if((c[m+1824+(z<<2)>>2]|0)==(J|0)){an=z;ao=aj;break}else{z=z+1|0}}if((Y|0)==5377){Y=0;c[w>>2]=K+1;an=K;ao=c[x>>2]|0}c[A+(ai<<2)>>2]=an;c[D+(an<<2)>>2]=c[ao+(B<<4)+12>>2];am=ai+1|0}z=B+1|0;if((z|0)<(c[v>>2]|0)){C=am;B=z}else{break}}}else{if(N){B=y+24|0;C=c[o+(P<<5)+28>>2]|0;D=0;do{A=B+(D<<2)|0;if((c[C+(D<<4)>>2]|0)!=0){c[A>>2]=c[A>>2]|1}if((c[C+(D<<4)+4>>2]|0)!=0){c[A>>2]=c[A>>2]|2}if((c[C+(D<<4)+8>>2]|0)!=0){c[A>>2]=c[A>>2]|4}if((c[C+(D<<4)+12>>2]|0)!=0){c[A>>2]=c[A>>2]|8}D=D+1|0;}while((D|0)<(c[v>>2]|0))}D=o+(P<<5)+20|0;C=c[D>>2]|0;B=r+24|0;A=c[B>>2]|0;x=0;while(1){if((x|0)>=(A|0)){Y=5317;break}if((c[m+1824+(x<<2)>>2]|0)==(C|0)){ap=x;aq=C;break}else{x=x+1|0}}if((Y|0)==5317){Y=0;c[B>>2]=A+1;ap=A;aq=c[D>>2]|0}c[y+20>>2]=ap;x=r+1824|0;c[x+(ap<<2)>>2]=aq;if((c[v>>2]|0)<=0){break}C=o+(P<<5)+28|0;w=y+280|0;u=0;H=0;while(1){z=c[C>>2]|0;J=c[z+(H<<4)>>2]|0;if((J|0)==0){ar=u;as=z}else{I=c[B>>2]|0;G=0;while(1){if((G|0)>=(I|0)){Y=5324;break}if((c[m+1824+(G<<2)>>2]|0)==(J|0)){at=G;au=z;break}else{G=G+1|0}}if((Y|0)==5324){Y=0;c[B>>2]=I+1;at=I;au=c[C>>2]|0}c[w+(u<<2)>>2]=at;c[x+(at<<2)>>2]=c[au+(H<<4)>>2];ar=u+1|0;as=c[C>>2]|0}G=c[as+(H<<4)+4>>2]|0;if((G|0)==0){av=ar;aw=as}else{z=c[B>>2]|0;J=0;while(1){if((J|0)>=(z|0)){Y=5389;break}if((c[m+1824+(J<<2)>>2]|0)==(G|0)){ax=J;ay=as;break}else{J=J+1|0}}if((Y|0)==5389){Y=0;c[B>>2]=z+1;ax=z;ay=c[C>>2]|0}c[w+(ar<<2)>>2]=ax;c[x+(ax<<2)>>2]=c[ay+(H<<4)+4>>2];av=ar+1|0;aw=c[C>>2]|0}J=c[aw+(H<<4)+8>>2]|0;if((J|0)==0){az=av;aA=aw}else{G=c[B>>2]|0;I=0;while(1){if((I|0)>=(G|0)){Y=5395;break}if((c[m+1824+(I<<2)>>2]|0)==(J|0)){aB=I;aC=aw;break}else{I=I+1|0}}if((Y|0)==5395){Y=0;c[B>>2]=G+1;aB=G;aC=c[C>>2]|0}c[w+(av<<2)>>2]=aB;c[x+(aB<<2)>>2]=c[aC+(H<<4)+8>>2];az=av+1|0;aA=c[C>>2]|0}I=c[aA+(H<<4)+12>>2]|0;if((I|0)==0){aD=az}else{J=c[B>>2]|0;z=0;while(1){if((z|0)>=(J|0)){Y=5401;break}if((c[m+1824+(z<<2)>>2]|0)==(I|0)){aE=z;aF=aA;break}else{z=z+1|0}}if((Y|0)==5401){Y=0;c[B>>2]=J+1;aE=J;aF=c[C>>2]|0}c[w+(az<<2)>>2]=aE;c[x+(aE<<2)>>2]=c[aF+(H<<4)+12>>2];aD=az+1|0}z=H+1|0;if((z|0)<(c[v>>2]|0)){u=aD;H=z}else{break}}}}while(0);X=+h[r+3480>>3]*1.0e3;Q=+(c[k>>2]|0)*.5;v=c[r+(E<<2)>>2]>>1;S=X>Q?Q:X;X=+(v|0);c[(c[r+1056+(E<<2)>>2]|0)+1116>>2]=~~(S/Q*X);m=c[o+(P<<5)+4>>2]|0;do{if((m|0)==1){l=+(c[((c[q>>2]|0)==0?r+2968|0:r+2996|0)>>2]|0)*1.0e3;if(l<=Q){aG=l;break}aG=Q}else if((m|0)==2){aG=250.0}else{aG=S}}while(0);do{if((c[s>>2]|0)==2){m=c[r+12>>2]|0;if((m|0)>0){q=r+544|0;N=0;while(1){H=c[q+(N<<2)>>2]|0;u=c[H>>2]|0;if((u|0)>0){x=H+1092|0;w=H+4|0;H=0;while(1){do{if((c[x+(H<<2)>>2]|0)==(P|0)){C=c[O>>2]|0;if((C|0)>0){aH=0;aI=0}else{aJ=0;break}while(1){B=((c[w+(aH<<2)>>2]|0)==(H|0))+aI|0;D=aH+1|0;if((D|0)<(C|0)){aH=D;aI=B}else{aJ=B;break}}}else{aJ=0}}while(0);C=H+1|0;if((C|0)<(u|0)&(aJ|0)==0){H=C}else{aK=aJ;break}}}else{aK=0}H=N+1|0;if((H|0)<(m|0)&(aK|0)==0){N=H}else{aL=aK;break}}}else{aL=0}N=c[t>>2]|0;m=ac(~~(+(aL|0)*X*(aG/Q)/+(N|0)+.9),N)|0;q=y+4|0;c[q>>2]=m;H=ac(aL,v)|0;if((m|0)<=(H|0)){aM=m;aN=N;break}m=H-((H|0)%(N|0)|0)|0;c[q>>2]=m;aM=m;aN=N}else{N=c[t>>2]|0;m=ac(~~(X*(aG/Q)/+(N|0)+.9),N)|0;q=y+4|0;c[q>>2]=m;if((m|0)<=(v|0)){aM=m;aN=N;break}m=v-((v|0)%(N|0)|0)|0;c[q>>2]=m;aM=m;aN=N}}while(0);if((aM|0)==0){c[y+4>>2]=aN}T=T+1|0;}while((T|0)<(c[p>>2]|0))}E=E+1|0;}while((E|0)<(V|0));V=d+3428|0;E=c[V>>2]|0;if((E|0)>0){c[a+16>>2]=E}else{E=c[b>>2]|0;aG=+h[E+3400>>3];b=~~aG;Q=aG- +(b|0);aN=c[(c[E+3396>>2]|0)+4>>2]|0;if((aN|0)==0){aO=-1}else{aO=~~(+(c[O>>2]|0)*((1.0-Q)*+h[aN+(b<<3)>>3]+Q*+h[aN+(b+1<<3)>>3]))}c[a+16>>2]=aO}aO=d+3424|0;c[a+20>>2]=c[aO>>2];b=d+3440|0;c[a+12>>2]=c[b>>2];aN=c[V>>2]|0;if((aN|0)==0){c[a+24>>2]=0}else{c[a+24>>2]=~~(+(c[d+3444>>2]|0)/+(aN|0))}if((c[d+3420>>2]|0)==0){f=0;return f|0}c[d+3360>>2]=c[V>>2];c[d+3364>>2]=c[aO>>2];c[d+3368>>2]=c[b>>2];c[d+3372>>2]=c[d+3444>>2];h[d+3376>>3]=+h[d+3448>>3];h[d+3384>>3]=+h[d+3432>>3];f=0;return f|0}function GW(a,b,d,e,f,h){a=a|0;b=+b;d=d|0;e=e|0;f=f|0;h=h|0;var i=0,j=0.0,k=0;i=~~b;j=b- +(i|0);k=c[a+2852+(d<<2)>>2]|0;b=1.0-j;d=i+1|0;g[k+12>>2]=b*+(c[e+(i*20|0)>>2]|0)+j*+(c[e+(d*20|0)>>2]|0);g[k+16>>2]=b*+(c[e+(i*20|0)+4>>2]|0)+j*+(c[e+(d*20|0)+4>>2]|0);g[k+20>>2]=b*+(c[e+(i*20|0)+8>>2]|0)+j*+(c[e+(d*20|0)+8>>2]|0);g[k+24>>2]=b*+g[e+(i*20|0)+12>>2]+j*+g[e+(d*20|0)+12>>2];g[k+28>>2]=b*+g[e+(i*20|0)+16>>2]+j*+g[e+(d*20|0)+16>>2];g[k+496>>2]=b*+(c[f+(i<<2)>>2]|0)+j*+(c[f+(d<<2)>>2]|0);f=0;do{g[k+36+(f<<2)>>2]=b*+(c[h+(i*68|0)+(f<<2)>>2]|0)+j*+(c[h+(d*68|0)+(f<<2)>>2]|0);f=f+1|0;}while((f|0)<17);return}function GX(a,b,d,e,f,h,i){a=a|0;b=+b;d=d|0;e=e|0;f=f|0;h=h|0;i=+i;var j=0,k=0.0,l=0,m=0,n=0,o=0.0;j=~~b;k=b- +(j|0);l=c[a+2852+(d<<2)>>2]|0;b=1.0-k;a=j+1|0;g[l+108>>2]=b*+(c[e+(j<<2)>>2]|0)+k*+(c[e+(a<<2)>>2]|0);c[l+120>>2]=c[h+(d*12|0)>>2];c[l+124>>2]=c[h+(d*12|0)+4>>2];c[l+128>>2]=c[h+(d*12|0)+8>>2];d=0;while(1){g[l+132+(d<<2)>>2]=b*+(c[f+(j*204|0)+(d<<2)>>2]|0)+k*+(c[f+(a*204|0)+(d<<2)>>2]|0);h=d+1|0;if((h|0)<17){d=h}else{m=0;break}}while(1){g[l+200+(m<<2)>>2]=b*+(c[f+(j*204|0)+68+(m<<2)>>2]|0)+k*+(c[f+(a*204|0)+68+(m<<2)>>2]|0);d=m+1|0;if((d|0)<17){m=d}else{n=0;break}}do{g[l+268+(n<<2)>>2]=b*+(c[f+(j*204|0)+136+(n<<2)>>2]|0)+k*+(c[f+(a*204|0)+136+(n<<2)>>2]|0);n=n+1|0;}while((n|0)<17);k=+g[l+132>>2];b=k+6.0;n=0;o=k;while(1){k=o+i;g[l+132+(n<<2)>>2]=k<b?b:k;a=n+1|0;if((a|0)>=17){break}n=a;o=+g[l+132+(a<<2)>>2]}o=+g[l+200>>2];b=o+6.0;n=0;k=o;while(1){o=k+i;g[l+200+(n<<2)>>2]=o<b?b:o;a=n+1|0;if((a|0)>=17){break}n=a;k=+g[l+200+(a<<2)>>2]}k=+g[l+268>>2];b=k+6.0;n=0;o=k;while(1){k=o+i;g[l+268+(n<<2)>>2]=k<b?b:k;a=n+1|0;if((a|0)>=17){break}n=a;o=+g[l+268+(a<<2)>>2]}return}function GY(a,b,d,e){a=a|0;b=b|0;d=d|0;e=+e;var f=0,h=0.0,i=0,j=0;f=c[a+28>>2]|0;h=e+1.0e-7;e=h<1.0?h:.9998999834060669;g[f+3416>>2]=e;i=GZ(b,d,e,0,f+3400|0)|0;c[f+3396>>2]=i;if((i|0)==0){j=-130;return j|0}G_(a,b,d);c[f+3420>>2]=0;c[f+3464>>2]=1;j=0;return j|0}function GZ(a,b,d,e,f){a=a|0;b=b|0;d=+d;e=e|0;f=f|0;var g=0,i=0,j=0,k=0,l=0,m=0,n=0.0,o=0,p=0,q=0,r=0.0,s=0.0,t=0,u=0.0,v=0,w=0.0;L7708:do{if((e|0)==0){g=0;i=77256;while(1){j=c[i>>2]|0;k=c[j+12>>2]|0;do{if((k|0)==-1|(k|0)==(a|0)){if((c[j+16>>2]|0)>(b|0)){break}if((c[j+20>>2]|0)<(b|0)){break}l=c[j>>2]|0;m=c[j+8>>2]|0;n=+h[m>>3];if(n>d){break}if(+h[m+(l<<3)>>3]>=d){o=i;p=l;q=m;r=n;s=d;break L7708}}}while(0);j=g+1|0;if((j|0)==17){t=0;break}else{g=j;i=77256+(j<<2)|0}}return t|0}else{n=d/+(a|0);i=0;g=77256;while(1){j=c[g>>2]|0;k=c[j+12>>2]|0;do{if((k|0)==-1|(k|0)==(a|0)){if((c[j+16>>2]|0)>(b|0)){break}if((c[j+20>>2]|0)<(b|0)){break}m=c[j>>2]|0;l=c[j+4>>2]|0;u=+h[l>>3];if(n<u){break}if(n<=+h[l+(m<<3)>>3]){o=g;p=m;q=l;r=u;s=n;break L7708}}}while(0);j=i+1|0;if((j|0)==17){t=0;break}else{i=j;g=77256+(j<<2)|0}}return t|0}}while(0);L7729:do{if((p|0)>0){b=0;d=r;while(1){a=b+1|0;if(s>=d){if(s<+h[q+(a<<3)>>3]){v=b;break L7729}}if((a|0)>=(p|0)){v=a;break L7729}b=a;d=+h[q+(a<<3)>>3]}}else{v=0}}while(0);if((v|0)==(p|0)){w=+(p|0)+-.001}else{r=+h[q+(v<<3)>>3];w=+(v|0)+(s-r)/(+h[q+(v+1<<3)>>3]-r)}h[f>>3]=w;t=c[o>>2]|0;return t|0}function G_(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0.0,i=0.0,j=0.0,k=0.0,l=0;e=c[a+28>>2]|0;f=c[e+3396>>2]|0;c[a>>2]=0;c[a+4>>2]=b;c[a+8>>2]=d;c[e+3456>>2]=1;c[e+3460>>2]=1;g=+h[e+3400>>3];d=~~g;i=g- +(d|0);h[e+3472>>3]=g;if((c[e+3488>>2]|0)==0){a=c[f+120>>2]|0;j=1.0-i;b=d+1|0;h[e+3480>>3]=j*+h[a+(d<<3)>>3]+i*+h[a+(b<<3)>>3];k=j;l=b}else{k=1.0-i;l=d+1|0}b=c[f+112>>2]|0;h[e+3496>>3]=k*+(c[b+(d<<2)>>2]|0)+i*+(c[b+(l<<2)>>2]|0);b=c[f+116>>2]|0;h[e+3504>>3]=k*+(c[b+(d<<2)>>2]|0)+i*+(c[b+(l<<2)>>2]|0);h[e+3512>>3]=-6.0;h[e+3520>>3]=g;h[e+3528>>3]=g;h[e+3536>>3]=g;h[e+3544>>3]=g;h[e+3552>>3]=g;h[e+3560>>3]=g;h[e+3568>>3]=g;h[e+3576>>3]=g;h[e+3584>>3]=g;h[e+3592>>3]=g;h[e+3600>>3]=g;h[e+3608>>3]=g;h[e+3616>>3]=g;h[e+3624>>3]=g;h[e+3632>>3]=g;h[e+3640>>3]=g;h[e+3648>>3]=g;return}function G$(a,b,d,e,f,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;i=i|0;var j=0,k=0.0,l=0,m=0,n=0;j=c[a+28>>2]|0;do{if((f|0)<1){k=+(e|0);l=(i|0)>0;if((e|0)<=0){if(l){m=i;break}else{n=-131}return n|0}if(l){m=~~(+(i+e|0)*.5);break}else{m=~~(k*.875);break}}else{m=f}}while(0);g[j+3416>>2]=+(m|0);l=GZ(b,d,+(m|0),1,j+3400|0)|0;c[j+3396>>2]=l;if((l|0)==0){n=-130;return n|0}G_(a,b,d);c[j+3464>>2]=1;c[j+3420>>2]=1;c[j+3424>>2]=i;c[j+3440>>2]=e;c[j+3428>>2]=f;h[j+3432>>3]=1.5;c[j+3444>>2]=m<<1;h[j+3448>>3]=.1;n=0;return n|0}function G0(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,j=0,k=0,l=0,m=0.0,n=0,o=0,p=0,q=0,r=0,s=0.0,t=0;e=i;i=i+8|0;f=e|0;if((a|0)==0){j=-131;i=e;return j|0}k=a+28|0;l=c[k>>2]|0;do{if((b&15|0)!=0){if((c[l+3392>>2]|0)==0){break}else{j=-131}i=e;return j|0}}while(0);switch(b|0){case 18:{if((d|0)==0){c[l+3428>>2]=0;j=0;i=e;return j|0}else{c[l+3428>>2]=~~(+((c[d+28>>2]|0)+(c[d+24>>2]|0)|0)*.5);j=0;i=e;return j|0}break};case 16:{c[d>>2]=c[l+3420>>2];m=+(c[l+3444>>2]|0)/+(c[a+8>>2]|0);h[d+32>>3]=m;h[d+16>>3]=m;h[d+40>>3]=1.0;c[d+4>>2]=c[l+3424>>2];c[d+8>>2]=c[l+3440>>2];b=l+3428|0;c[d+24>>2]=c[b>>2];c[d+28>>2]=c[b>>2];j=0;i=e;return j|0};case 17:{if((d|0)==0){c[l+3420>>2]=0;j=0;i=e;return j|0}c[l+3420>>2]=c[d>>2];b=c[k>>2]|0;if((c[b+3392>>2]|0)==0){c[b+3428>>2]=~~(+((c[d+28>>2]|0)+(c[d+24>>2]|0)|0)*.5)}b=c[k>>2]|0;if((c[b+3392>>2]|0)!=0){j=0;i=e;return j|0}k=c[d+4>>2]|0;c[b+3424>>2]=k;n=c[d+8>>2]|0;c[b+3440>>2]=n;o=~~(+h[d+16>>3]*+(k+n|0)*.5);c[b+3444>>2]=o;if((o|0)>=128){j=0;i=e;return j|0}c[b+3444>>2]=128;j=0;i=e;return j|0};case 20:{if((d|0)==0){j=-131;i=e;return j|0}c[d>>2]=c[l+3420>>2];c[d+4>>2]=(c[l+3424>>2]|0)/1e3|0;c[d+8>>2]=(c[l+3440>>2]|0)/1e3|0;c[d+24>>2]=(c[l+3428>>2]|0)/1e3|0;h[d+32>>3]=+h[l+3432>>3];c[d+12>>2]=c[l+3444>>2];h[d+16>>3]=+h[l+3448>>3];j=0;i=e;return j|0};case 19:{if((d|0)==0){c[l+3424>>2]=0;c[l+3440>>2]=0;p=c[l+3444>>2]|0}else{b=c[d+4>>2]|0;c[l+3424>>2]=b;o=c[d+8>>2]|0;c[l+3440>>2]=o;n=~~(+h[d+16>>3]*+(b+o|0)*.5);c[l+3444>>2]=n;p=n}if((p|0)>=128){j=0;i=e;return j|0}c[l+3444>>2]=128;j=0;i=e;return j|0};case 21:{if((d|0)==0){c[l+3420>>2]=0;j=0;i=e;return j|0}p=d+4|0;n=c[p>>2]|0;o=(n|0)>0;do{if(o){b=c[d+24>>2]|0;if((b|0)>0&(n|0)>(b|0)){j=-131}else{break}i=e;return j|0}}while(0);b=d+8|0;k=c[b>>2]|0;q=(k|0)>0;do{if(q){r=c[d+24>>2]|0;if((r|0)>0&(k|0)<(r|0)){j=-131}else{break}i=e;return j|0}}while(0);do{if(o){if(q&(n|0)>(k|0)){j=-131}else{break}i=e;return j|0}}while(0);k=d+32|0;if(+h[k>>3]<=0.0){j=-131;i=e;return j|0}n=d+12|0;if((c[n>>2]|0)<0){j=-131;i=e;return j|0}q=d+16|0;m=+h[q>>3];if(m<0.0|m>1.0){j=-131;i=e;return j|0}c[l+3420>>2]=c[d>>2];c[l+3424>>2]=(c[p>>2]|0)*1e3|0;c[l+3440>>2]=(c[b>>2]|0)*1e3|0;c[l+3428>>2]=(c[d+24>>2]|0)*1e3|0;h[l+3432>>3]=+h[k>>3];c[l+3444>>2]=c[n>>2];h[l+3448>>3]=+h[q>>3];j=0;i=e;return j|0};case 32:{h[d>>3]=+h[l+3480>>3];j=0;i=e;return j|0};case 33:{m=+h[d>>3];s=m<2.0?2.0:m;h[l+3480>>3]=s>99.0?99.0:s;c[l+3488>>2]=1;j=0;i=e;return j|0};case 48:{h[d>>3]=+h[l+3408>>3];j=0;i=e;return j|0};case 49:{s=+h[d>>3];m=s>0.0?0.0:s;h[l+3408>>3]=m<-15.0?-15.0:m;j=0;i=e;return j|0};case 64:{c[d>>2]=c[l+3464>>2];j=0;i=e;return j|0};case 65:{h[f>>3]=0.0;q=(c[d>>2]|0)!=0;c[l+3464>>2]=q&1;if(q){t=c[a+4>>2]|0}else{t=-1}q=a+8|0;d=GZ(t,c[q>>2]|0,+g[l+3416>>2],c[l+3420>>2]|0,f)|0;t=l+3396|0;if((c[t>>2]|0)==0){j=-130;i=e;return j|0}c[t>>2]=d;h[l+3400>>3]=+h[f>>3];G_(a,c[a+4>>2]|0,c[q>>2]|0);j=0;i=e;return j|0};default:{j=-130;i=e;return j|0}}return 0}function G1(b){b=b|0;var d=0;NM(b|0,0,16);d=NC(256)|0;c[b+8>>2]=d;c[b+12>>2]=d;a[d]=0;c[b+16>>2]=256;return}function G2(b,e){b=b|0;e=e|0;var f=0,g=0,h=0;f=e>>3;g=b+12|0;if((c[g>>2]|0)==0){return}h=e-(f<<3)|0;e=(c[b+8>>2]|0)+f|0;c[g>>2]=e;c[b+4>>2]=h;c[b>>2]=f;a[e]=(d[e]|0)&c[129784+(h<<2)>>2]&255;return}function G3(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0;do{if(f>>>0<=32){g=b|0;h=b+16|0;i=c[h>>2]|0;j=b+12|0;k=c[j>>2]|0;if((c[g>>2]|0)<(i-4|0)){l=k}else{if((k|0)==0){return}if((i|0)>2147483391){break}k=b+8|0;m=NF(c[k>>2]|0,i+256|0)|0;if((m|0)==0){break}c[k>>2]=m;c[h>>2]=(c[h>>2]|0)+256;h=m+(c[g>>2]|0)|0;c[j>>2]=h;l=h}h=c[129784+(f<<2)>>2]&e;j=b+4|0;m=c[j>>2]|0;k=m+f|0;i=b+12|0;a[l]=(d[l]|0|h<<m)&255;do{if((k|0)>7){a[(c[i>>2]|0)+1|0]=h>>>((8-(c[j>>2]|0)|0)>>>0)&255;if((k|0)<=15){break}a[(c[i>>2]|0)+2|0]=h>>>((16-(c[j>>2]|0)|0)>>>0)&255;if((k|0)<=23){break}a[(c[i>>2]|0)+3|0]=h>>>((24-(c[j>>2]|0)|0)>>>0)&255;if((k|0)<=31){break}m=c[j>>2]|0;if((m|0)==0){a[(c[i>>2]|0)+4|0]=0;break}else{a[(c[i>>2]|0)+4|0]=h>>>((32-m|0)>>>0)&255;break}}}while(0);h=(k|0)/8|0;c[g>>2]=(c[g>>2]|0)+h;c[i>>2]=(c[i>>2]|0)+h;c[j>>2]=k&7;return}}while(0);l=c[b+8>>2]|0;if((l|0)!=0){ND(l)}NM(b|0,0,20);return}function G4(a){a=a|0;var b=0;b=c[a+8>>2]|0;if((b|0)!=0){ND(b)}NM(a|0,0,20);return}function G5(b){b=b|0;var d=0,e=0;d=b+12|0;if((c[d>>2]|0)==0){return}e=c[b+8>>2]|0;c[d>>2]=e;a[e]=0;c[b>>2]=0;c[b+4>>2]=0;return}function G6(a,b,d){a=a|0;b=b|0;d=d|0;NM(a|0,0,16);c[a+12>>2]=b;c[a+8>>2]=b;c[a+16>>2]=d;return}function G7(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0;if(b>>>0>32){e=-1;return e|0}f=c[129784+(b<<2)>>2]|0;g=c[a+4>>2]|0;h=g+b|0;b=c[a>>2]|0;i=c[a+16>>2]|0;do{if((b|0)>=(i-4|0)){if((b|0)>(i-(h+7>>3)|0)){e=-1;return e|0}if((h|0)==0){e=0}else{break}return e|0}}while(0);i=c[a+12>>2]|0;a=(d[i]|0)>>>(g>>>0);do{if((h|0)>8){b=(d[i+1|0]|0)<<8-g|a;if((h|0)<=16){j=b;break}k=(d[i+2|0]|0)<<16-g|b;if((h|0)<=24){j=k;break}b=(d[i+3|0]|0)<<24-g|k;if((h|0)<33|(g|0)==0){j=b;break}j=(d[i+4|0]|0)<<32-g|b}else{j=a}}while(0);e=j&f;return e|0}function G8(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0;d=a+4|0;e=(c[d>>2]|0)+b|0;b=a|0;f=c[b>>2]|0;g=c[a+16>>2]|0;if((f|0)>(g-(e+7>>3)|0)){c[a+12>>2]=0;c[b>>2]=g;h=1;c[d>>2]=h;return}else{g=(e|0)/8|0;i=a+12|0;c[i>>2]=(c[i>>2]|0)+g;c[b>>2]=f+g;h=e&7;c[d>>2]=h;return}}function G9(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;L7921:do{if(b>>>0>32){e=c[a+16>>2]|0;f=a|0;g=a+4|0}else{h=c[129784+(b<<2)>>2]|0;i=a+4|0;j=c[i>>2]|0;k=j+b|0;l=a|0;m=c[l>>2]|0;n=c[a+16>>2]|0;do{if((m|0)>=(n-4|0)){if((m|0)>(n-(k+7>>3)|0)){e=n;f=l;g=i;break L7921}if((k|0)==0){o=0}else{break}return o|0}}while(0);n=a+12|0;p=c[n>>2]|0;q=(d[p]|0)>>>(j>>>0);do{if((k|0)>8){r=(d[p+1|0]|0)<<8-j|q;if((k|0)<=16){s=r;break}t=(d[p+2|0]|0)<<16-j|r;if((k|0)<=24){s=t;break}r=(d[p+3|0]|0)<<24-j|t;if((k|0)<33|(j|0)==0){s=r;break}s=(d[p+4|0]|0)<<32-j|r}else{s=q}}while(0);q=(k|0)/8|0;c[n>>2]=p+q;c[l>>2]=m+q;c[i>>2]=k&7;o=s&h;return o|0}}while(0);c[a+12>>2]=0;c[f>>2]=e;c[g>>2]=1;o=-1;return o|0}function Ha(a){a=a|0;return(((c[a+4>>2]|0)+7|0)/8|0)+(c[a>>2]|0)|0}function Hb(a){a=a|0;return c[a+8>>2]|0}function Hc(b,e){b=b|0;e=e|0;var f=0,g=0;Kj(b);I_(e,64);c[e+8>>2]=c[b+28>>2];c[b+16>>2]=2;c[e>>2]=2;c[e+4>>2]=c[b>>2];c[e+12>>2]=c[b+40>>2];c[e+16>>2]=c[b+4>>2];c[e+20>>2]=c[b+24>>2];c[e+24>>2]=c[b+36>>2];a[e+40|0]=a[b+76|0]|0;a[e+41|0]=a[b+77|0]|0;a[e+42|0]=a[b+78|0]|0;c[e+28>>2]=d[b+79|0]|0;c[e+32>>2]=-1;f=b+48|0;b=c[f>>2]|0;if((b&1|0)==0){g=b}else{b=e+53|0;a[b]=a[b]|1;g=c[f>>2]|0}if((g&4|0)!=0){g=e+53|0;a[g]=a[g]|2}c[e+44>>2]=-1;a[e+50|0]=0;a[e+48|0]=0;a[e+49|0]=1;a[e+51|0]=0;a[e+52|0]=-1;c[e+56>>2]=0;a[e+60|0]=0;c[e+36>>2]=0;return 0}function Hd(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;f=i;i=i+1800|0;g=f|0;h=f+8|0;c[g>>2]=0;do{if((b|0)==0){j=32}else{k=IQ(1,2004)|0;c[g>>2]=k;if((k|0)==0){j=33;break}I_(k,2004);k=(c[g>>2]|0)+1996|0;if((d|0)==0){c[k>>2]=1;l=(c[g>>2]|0)+1996|0;c[l>>2]=c[l>>2]|2;l=(c[g>>2]|0)+1996|0;c[l>>2]=c[l>>2]|4;l=(c[g>>2]|0)+1996|0;c[l>>2]=c[l>>2]|16}else{c[k>>2]=d}if((e|0)==0){c[(c[g>>2]|0)+1980>>2]=6;c[(c[g>>2]|0)+1988>>2]=6;k=c[g>>2]|0;m=k;n=c[k+1980>>2]|0}else{c[(c[g>>2]|0)+1980>>2]=e&255;k=c[g>>2]|0;if((c[k+1996>>2]&2|0)==0){o=k}else{if((e&65280|0)==0){p=c[k+1980>>2]|0}else{p=e>>>8}c[k+1988>>2]=p;o=c[g>>2]|0}k=c[o+1980>>2]|0;if((k|0)>6){j=35;break}if((c[o+1988>>2]|0)>6){j=35;break}else{m=o;n=k}}c[m+1976>>2]=(n|0)>6?6:n;k=c[g>>2]|0;l=c[k+1988>>2]|0;c[k+1984>>2]=(l|0)>6?6:l;c[(c[g>>2]|0)+1992>>2]=1;l=IQ((c[(c[g>>2]|0)+1980>>2]|0)*3685|0,2)|0;c[(c[g>>2]|0)+228>>2]=l;l=c[g>>2]|0;k=c[l+1996>>2]|0;if((k&2|0)==0){q=l}else{if((Io(l+204|0,c[l+1984>>2]|0,c[l+1988>>2]|0,k>>>2&1)|0)!=0){j=33;break}q=c[g>>2]|0}if((Kk(q+200|0,c[q+1976>>2]|0,c[q+1980>>2]|0,1)|0)!=0){j=33;break}k=c[g>>2]|0;l=(ac((c[k+1992>>2]|0)*6144|0,c[k+1980>>2]|0)|0)>>>3;r=1;while(1){s=1<<r;if(s>>>0<l>>>0){r=r+1|0}else{break}}c[k+224>>2]=s;r=JU(0)|0;c[(c[g>>2]|0)+220>>2]=r;r=c[g>>2]|0;if((c[r+224>>2]|0)>8192){j=33;break}if((c[r+1996>>2]&16|0)==0){t=r}else{if((Ho(r+208|0)|0)!=0){j=33;break}t=c[g>>2]|0}if((HU(t+216|0)|0)==0){u=0}else{j=33;break}do{c[h+(u*56|0)+12>>2]=0;u=u+1|0;}while((u|0)<32);H4(h|0)|0;k=0;while(1){if((k|0)>=32){v=0;break}if((c[h+(k*56|0)+12>>2]|0)==8){w=5675;break}else{k=k+1|0}}if((w|0)==5675){v=c[h+(k*56|0)+20>>2]|0}c[(c[g>>2]|0)+2e3>>2]=v;r=c[g>>2]|0;if((HX(c[r+216>>2]|0,4,r)|0)!=0){j=67;break}r=c[g>>2]|0;Hc(r+120|0,r|0)|0;r=c[g>>2]|0;a[r+117|0]=a[r+52|0]|0;c[(c[g>>2]|0)+1972>>2]=65535;c[b>>2]=c[g>>2];x=0;i=f;return x|0}}while(0);Hf(g)|0;x=j;i=f;return x|0}function He(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;H9(c[a+204>>2]|0,b,i,0);return 0}function Hf(a){a=a|0;var b=0,d=0,e=0,f=0;if((a|0)==0){b=32;return b|0}d=c[a>>2]|0;if((d|0)==0){b=0;return b|0}e=d+228|0;f=c[e>>2]|0;if((f|0)!=0){IR(f);c[e>>2]=0}e=d+220|0;if((c[e>>2]|0)!=0){JV(e)}e=d+204|0;if((c[e>>2]|0)!=0){Il(e)}e=d+200|0;if((c[e>>2]|0)!=0){Kn(e)}H0(d+216|0);e=d+208|0;if((c[e>>2]|0)!=0){Hp(e)|0}IR(c[a>>2]|0);c[a>>2]=0;b=0;return b|0}function Hg(e,f,g,h,j){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0;k=i;i=i+576|0;l=k|0;m=k+8|0;n=k+16|0;o=k+176|0;p=k+272|0;q=k+280|0;r=k+288|0;s=k+384|0;t=k+544|0;u=k+552|0;v=k+560|0;w=k+568|0;if((e|0)==0){x=32;i=k;return x|0}y=e+1972|0;z=c[y>>2]|0;do{if((z|0)!=0){c[p>>2]=0;A=e+204|0;B=e+120|0;c[e+244>>2]=0;C=e+164|0;D=c[C>>2]|0;c[q>>2]=D;L8023:do{if((z&1|0)==0){E=z;F=D;G=5786}else{H=e+160|0;I=c[H>>2]|0;J=e+8|0;K=c[J>>2]|0;L=e+148|0;c[L>>2]=K;M=c[e+4>>2]|0;N=e+120|0;c[N>>2]=M;O=e+196|0;a[O]=a[e+40|0]|0;P=e+197|0;a[P]=a[e+41|0]|0;a[e+198|0]=a[e+42|0]|0;Q=c[e+16>>2]|0;R=e+124|0;c[R>>2]=Q;S=c[e+12>>2]|0;c[H>>2]=S;T=e+20|0;U=e+144|0;c[U>>2]=c[T>>2];c[e+156>>2]=c[e+24>>2];a[e+199|0]=c[e+28>>2]&255;V=c[e>>2]|0;W=e+136|0;c[W>>2]=V;c[e+176>>2]=c[e+36>>2];X=e+168|0;c[X>>2]=0;Y=e+172|0;a[Y]=-1;switch(V|0){case 23:{a[Y]=0;Z=a[e+53|0]|0;_=((Z&2)!=0?4:0)|(Z&1|96)|((Z&4)!=0?2:0);c[X>>2]=_;Z=e+44|0;$=c[Z>>2]|0;c[Z>>2]=($|0)==-1?10:$;$=c[e+32>>2]|0;Z=($|0)==-1?512:$;c[C>>2]=Z;if((Z|0)==512){aa=512;ab=_;G=5711}else if((Z|0)==480){ad=480;ae=_;G=5710}else{af=35;break L8023}break};case 17:{a[Y]=0;_=a[e+53|0]|0;Z=((_&2)!=0?4:0)|(_&1|64);c[X>>2]=Z;_=e+44|0;$=c[_>>2]|0;c[_>>2]=($|0)==-1?10:$;$=c[e+32>>2]|0;_=($|0)==-1?1024:$;c[C>>2]=_;if((_|0)==1024){aa=1024;ab=Z;G=5711}else if((_|0)==960){ad=960;ae=Z;G=5710}else{af=35;break L8023}break};case 2:case 5:case 29:{G=5705;break};case 39:{a[Y]=0;Z=a[e+53|0]|0;_=((Z&2)!=0?4:0)|(Z&1|80)|((Z&4)!=0?2:0)|((a[e+61|0]|0)!=0?8192:0);c[X>>2]=_;Z=e+44|0;$=c[Z>>2]|0;c[Z>>2]=($|0)==-1?10:$;$=c[e+32>>2]|0;Z=($|0)==-1?512:$;c[C>>2]=Z;if((Z|0)==512){aa=512;ab=_;G=5711}else if((Z|0)==480){ad=480;ae=_;G=5710}else{af=35;break L8023}break};case 129:case 132:case 156:{a[P]=0;if((a[e+48|0]|0)==0){G=5705}else{af=35;break L8023}break};default:{ag=D;G=5712}}do{if((G|0)==5711){ah=0;ai=ab;aj=aa;ak=c[e+132>>2]|0}else if((G|0)==5710){ah=0;ai=ae;aj=ad;ak=c[e+132>>2]|0}else if((G|0)==5705){_=e+44|0;Z=c[_>>2]|0;c[_>>2]=(Z|0)==-1?2:Z;Z=c[e+32>>2]|0;_=(Z|0)==-1?1024:Z;c[C>>2]=_;if((_|0)==960){ag=960;G=5712;break}else if((_|0)!=1024){af=35;break L8023}ag=1024;G=5712}}while(0);L8035:do{if((G|0)==5712){_=c[e+132>>2]|0;switch(V|0){case 5:case 29:case 132:case 156:case 136:case 137:case 144:case 145:{break};default:{ah=0;ai=0;aj=ag;ak=_;break L8035}}ah=1;ai=0;aj=ag;ak=_}}while(0);if((V|0)==39){al=(ai&8192|0)==0?ah:1}else{al=ah}Hl(S,0,n)|0;_=(al|0)!=0;if(_){am=(M|0)/2|0}else{am=M}if((V|0)==39|(V|0)==23){Z=K*36e4|0;$=(Z|0)<(Q|0)?Z:Q;Z=K*8e3|0;an=(Z|0)>($|0)?Z:$}else{an=Q}if((V|0)==29|(V|0)==5|(V|0)==2){$=K*576e3|0;ao=($|0)<(an|0)?$:an}else{ao=an}$=Kh(0,am,aj,K,c[n+8>>2]|0,ao,-1,0,-1,ak)|0;L8051:do{if(_){Z=n+12|0;do{if((c[Z>>2]|0)>0){ap=$;aq=0;do{a[o+(aq<<4)+14|0]=c[n+16+(aq*24|0)+12>>2]&255;a[o+(aq<<4)+15|0]=c[n+16+(aq*24|0)+16>>2]&255;c[o+(aq<<4)>>2]=c[n+16+(aq*24|0)>>2];ar=HP(c[n+16+(aq*24|0)+20>>2]|0,$,m)|0;as=c[m>>2]|0;L8057:do{if((as|0)>0){do{if((ar|0)!=0){at=ar>>31^ar;if((at|0)>-1){au=~at;at=0;while(1){av=au<<1;if((av|0)<0){au=av;at=at+1|0}else{aw=at;break}}}else{aw=-1}if((aw|0)>=(as|0)){break}ax=(ar|0)>0?2147483647:-2147483648;break L8057}}while(0);ax=ar<<as}else{if((as|0)<-31){ax=0;break}ax=ar>>-as}}while(0);c[o+(aq<<4)+4>>2]=ax;c[o+(aq<<4)+8>>2]=c[n+16+(aq*24|0)+4>>2];a[o+(aq<<4)+13|0]=c[n+16+(aq*24|0)+8>>2]&255;ap=ap-ax|0;aq=aq+1|0;ay=c[Z>>2]|0;}while((aq|0)<(ay|0));aq=o+4|0;c[aq>>2]=(c[aq>>2]|0)+ap;if((ay|0)>0){az=0;aA=0;aB=-1}else{aC=0;aD=-1;break}while(1){do{if((c[n+16+(aA*24|0)>>2]|0)>>>0<2){aq=c[o+(aA<<4)+4>>2]|0;as=Ip(aq,c[n+16+(aA*24|0)+8>>2]|0,am,V)|0;if((as|0)==0){aE=0;break L8051}if((aq|0)<(as|0)){aF=0;aG=(aA|0)==0?0:az}else{aF=aB;aG=az}if((aq|0)>(as|0)){aH=(aA|0)==0?5e6:aG;aI=1}else{if((aF|0)==-1){aJ=-1;aK=aG;break}else{aH=aG;aI=aF}}aq=HR(as,c[n+16+(aA*24|0)+20>>2]|0)|0;if((aI|0)==0){as=aq+16|0;aJ=0;aK=(aH|0)>(as|0)?aH:as;break}else{as=aq-16|0;aJ=aI;aK=(aH|0)<(as|0)?aH:as;break}}else{aJ=aB;aK=az}}while(0);as=aA+1|0;if((as|0)<(c[Z>>2]|0)){az=aK;aA=as;aB=aJ}else{aC=aK;aD=aJ;break}}}else{aC=0;aD=-1}}while(0);aE=(aD|0)==-1?$:aC}else{aE=$}}while(0);c[R>>2]=aE;$=c[W>>2]|0;do{if(($|0)==23|($|0)==39){V=c[T>>2]|0;if((V|0)==0){c[e+192>>2]=(c[J>>2]|0)*50|0}else if((V|0)==8){c[U>>2]=0;break}if((c[U>>2]|0)!=0){af=35;break L8023}}}while(0);if((a[Y]|0)>-1){c[X>>2]=c[X>>2]|64;if(((c[H>>2]|0)-1|0)>>>0>6){af=35;break}}if((Hj(H,c[L>>2]|0)|0)!=0){af=35;break}if((c[L>>2]|0)>(c[e+1980>>2]|0)){af=35;break}J=c[(Hk(c[H>>2]|0)|0)+8>>2]|0;if((J|0)>(c[e+1988>>2]|0)){J=c[W>>2]|0;switch(J|0){case 5:case 29:case 132:case 156:case 136:case 137:case 144:case 145:{aL=1;break};default:{aL=0}}if((J|0)==39){aM=(c[X>>2]&8192|0)==0?aL:1}else{aM=aL}if((aM|0)!=0){af=35;break}}J=c[U>>2]|0;if((J-1|0)>>>0<5){T=Ki(J,c[H>>2]|0)|0;c[R>>2]=T;aN=T}else{aN=c[R>>2]|0}do{if((aN|0)==-1){T=c[(Hk(c[H>>2]|0)|0)+8>>2]|0;J=ac(c[N>>2]|0,T)|0;T=c[W>>2]|0;if((T|0)==2){c[R>>2]=(J>>1)+J;break}else if((T|0)==5){c[R>>2]=(J>>2)+J>>1;break}else if((T|0)==29){c[R>>2]=J>>1;break}else{c[R>>2]=J;break}}}while(0);if(((c[U>>2]|0)-1|0)>>>0<5){G=5775}else{if((a[O]|0)==0){G=5775}}if((G|0)==5775){a[P]=0}R=c[W>>2]|0;do{if((R|0)==2|(R|0)==5){c[e+212>>2]=1;N=c[H>>2]|0;if((N-1|0)>>>0<=6){aO=N;break}a[e+60|0]=0;aO=N}else{c[e+212>>2]=0;aO=c[H>>2]|0}}while(0);H=c[C>>2]|0;c[q>>2]=H;if((I|0)==(aO|0)){E=z;F=H;G=5786;break}switch(R|0){case 5:case 29:case 132:case 156:case 136:case 137:case 144:case 145:{aP=1;break};default:{aP=0}}if((R|0)==39){aQ=(c[X>>2]&8192|0)==0?aP:1}else{aQ=aP}E=(aQ|0)==0?z:z|2;F=H;G=5786}}while(0);L8140:do{if((G|0)==5786){do{if((E|0)==65535){I_(c[e+228>>2]|0,(c[e+1980>>2]|0)*7370|0);aR=c[C>>2]|0;G=5790}else{if((E&1|0)!=0){aR=F;G=5790;break}aS=0;aT=c[e+136>>2]|0}}while(0);if((G|0)==5790){c[p>>2]=0;X=c[e+136>>2]|0;if((X|0)==39){c[e+248>>2]=((aR|0)/2|0)+aR}else{c[e+248>>2]=((aR|0)/16|0)+aR+(((aR|0)/8|0)<<2)}c[e+128>>2]=0;aS=1;aT=X}X=e+136|0;switch(aT|0){case 5:case 29:case 132:case 156:case 136:case 137:case 144:case 145:{aU=1;break};default:{aU=0}}if((aT|0)==39){aV=(c[e+168>>2]&8192|0)==0?aU:1}else{aV=aU}do{if((aV|0)!=0){if((E&2|0)==0&(aS^1)){break}R=e+160|0;if((Hl(c[R>>2]|0,c[e+152>>2]|0,s)|0)!=0){af=64;break L8140}I=s+12|0;D=c[I>>2]|0;if((D|0)>6){af=64;break L8140}H=r|0;W=c[e+124>>2]|0;if((D|0)>0){P=W;O=0;while(1){a[r+(O<<4)+14|0]=c[s+16+(O*24|0)+12>>2]&255;a[r+(O<<4)+15|0]=c[s+16+(O*24|0)+16>>2]&255;c[r+(O<<4)>>2]=c[s+16+(O*24|0)>>2];U=HP(c[s+16+(O*24|0)+20>>2]|0,W,l)|0;N=c[l>>2]|0;L8168:do{if((N|0)>0){do{if((U|0)!=0){L=U>>31^U;if((L|0)>-1){Y=~L;L=0;while(1){J=Y<<1;if((J|0)<0){Y=J;L=L+1|0}else{aW=L;break}}}else{aW=-1}if((aW|0)>=(N|0)){break}aX=(U|0)>0?2147483647:-2147483648;break L8168}}while(0);aX=U<<N}else{if((N|0)<-31){aX=0;break}aX=U>>-N}}while(0);c[r+(O<<4)+4>>2]=aX;c[r+(O<<4)+8>>2]=c[s+16+(O*24|0)+4>>2];a[r+(O<<4)+13|0]=c[s+16+(O*24|0)+8>>2]&255;N=P-aX|0;U=O+1|0;L=c[I>>2]|0;if((U|0)<(L|0)){P=N;O=U}else{aY=N;aZ=L;break}}}else{aY=W;aZ=D}O=r+4|0;c[O>>2]=(c[O>>2]|0)+aY;O=e+148|0;if((Is(c[A>>2]|0,H,aZ,c[e+228>>2]|0,e+156|0,p,O,B|0,q,X,e+248|0,(c[X>>2]|0)==39?1:8,E>>>1&1)|0)!=0){af=66;break L8140}if((c[X>>2]|0)!=(aT|0)){af=66;break L8140}if((c[O>>2]|0)==1){c[R>>2]=1}O=e+197|0;if((a[O]|0)!=0){a[O]=0}c[e+128>>2]=Iv(c[A>>2]|0)|0}}while(0);if((E&4096|0)!=0){O=e+64|0;I_(O,56);P=e+112|0;c[P>>2]=0;I=c[X>>2]|0;if((I|0)==129){a_=2;a$=0;a0=129;G=5830}else if((I|0)==156){a1=29;G=5825}else if((I|0)==132){a1=5;G=5825}else{a2=I;a3=0;G=5826}if((G|0)==5825){c[P>>2]=8388608;a2=a1;a3=8388608;G=5826}do{if((G|0)==5826){if((I|0)==39){L=c[e+168>>2]<<10&8388608|a3;c[P>>2]=L;a4=L}else{a4=a3}if(!((a2|0)==29|(a2|0)==5)){a_=a2;a$=a4;a0=I;G=5830;break}c[O>>2]=2;a5=a2;a6=a4;a7=I}}while(0);if((G|0)==5830){c[O>>2]=a_;a5=a_;a6=a$;a7=a0}I=a[e+48|0]|0;do{if((I<<24>>24|0)==0){c[e+68>>2]=0}else if((I<<24>>24|0)==1){if(!((a5|0)==29|(a5|0)==5)){G=5835;break}c[e+68>>2]=5}else{G=5835}}while(0);if((G|0)==5835){c[e+68>>2]=a5}c[e+80>>2]=c[e+4>>2];c[e+84>>2]=c[e+124>>2];c[e+92>>2]=c[e+148>>2];I=c[e+160>>2]|0;c[e+72>>2]=I;L=c[e+132>>2]|0;N=a[e+49|0]|0;c[e+100>>2]=(L|0)>1&N<<24>>24==1?L:N&255;N=((a[e+51|0]|0)!=0?4194304:0)|(a6|2097152);c[P>>2]=N;L=a[e+52|0]|0;do{if(L<<24>>24==-1){U=c[e+44>>2]|0;if((U|0)==2|(U|0)==10|(U|0)==6){a[e+117|0]=10;break}else{a[e+117|0]=0;break}}else{a[e+117|0]=L}}while(0);c[e+88>>2]=c[C>>2];c[e+76>>2]=c[B>>2];if((a7|0)==129|(a7|0)==132|(a7|0)==156){L=N&-1048577;c[P>>2]=L;c[e+68>>2]=0;a8=L}else{L=N|1048576;c[P>>2]=L;a8=L}L=c[e+168>>2]|0;U=L<<25&33554432|a8|L<<24&67108864|L<<23&16777216;c[P>>2]=U;L=c[e+56>>2]|0;do{if((L&1|0)==0){G=5848}else{if((a[e+172|0]|0)!=-1){G=5848;break}if((I-5|0)>>>0>=2){G=5848;break}a[e+116|0]=(L>>>1&3)+1&255;c[P>>2]=L<<24&134217728|U}}while(0);if((G|0)==5848){a[e+116|0]=0}U=(a[e+50|0]|0)==1?2:0;L=e+220|0;P=e+224|0;I_(c[L>>2]|0,c[P>>2]|0);if((HV(c[e+216>>2]|0,c[L>>2]|0,c[P>>2]|0,c[e+44>>2]|0,O,U)|0)!=0){af=67;break}}U=(E&2|0)==0&(aS^1);if(!U){if((Kl(c[e+200>>2]|0,B,c[e+216>>2]|0,E>>>1&1)|0)!=0){af=65;break}}P=e+208|0;L=c[P>>2]|0;do{if((L|0)!=0){if((c[e+212>>2]|0)==0|U){break}I=c[C>>2]|0;N=((I|0)/16|0)+I+(((I|0)/8|0)<<2)|0;I=c[X>>2]|0;switch(I|0){case 5:case 29:case 132:case 156:case 136:case 137:case 144:case 145:{a9=1;break};default:{a9=0}}if((I|0)==39){ba=(c[e+168>>2]&8192|0)==0?a9:1}else{ba=a9}if((ba|0)==0|(A|0)==0){bb=N;bc=L}else{I=(Iw(c[A>>2]|0)|0)+(N<<1)|0;bb=I;bc=c[P>>2]|0}if((Hq(bc,E>>>1&1,d[e+60|0]|0,bb,c[q>>2]|0,c[e+4>>2]|0,c[e+8>>2]|0,c[e+12>>2]|0,c[e+152>>2]|0)|0)!=0){af=68;break L8140}I=Hs(c[P>>2]|0)|0;N=e+248|0;c[N>>2]=(c[N>>2]|0)+I}}while(0);if(!aS){af=0;break}c[e+232>>2]=c[p>>2];P=c[e+8>>2]|0;c[e+236>>2]=ac(P,c[q>>2]|0)|0;L=e+248|0;c[L>>2]=ac(c[L>>2]|0,P)|0;af=0}}while(0);if((af|0)==0){c[y>>2]=0;break}else{x=af;i=k;return x|0}}}while(0);af=(j|0)==0;if(!af){I_(j,12)}q=(g|0)==0;do{if(q){G=5874}else{p=g|0;aS=c[p>>2]|0;if((aS|0)<=0){break}bb=g+4|0;E=g+12|0;bc=0;ba=aS;while(1){aS=c[(c[bb>>2]|0)+(bc<<2)>>2]|0;if((aS|0)==0){bd=ba}else{I_(aS,c[(c[E>>2]|0)+(bc<<2)>>2]|0);bd=c[p>>2]|0}aS=bc+1|0;if((aS|0)<(bd|0)){bc=aS;ba=bd}else{G=5874;break}}}}while(0);do{if((G|0)==5874){if((f|0)==0&q&(h|0)==0&af){x=0}else{break}i=k;return x|0}}while(0);af=t|0;q=e+132|0;I_(t,c[q>>2]<<2);bd=h|0;ba=c[bd>>2]|0;L8271:do{if((ba|0)>0){bc=f+8|0;p=c[f>>2]|0;E=0;while(1){if((E|0)>=(p|0)){break L8271}be=c[bc>>2]|0;if((c[be+(E<<2)>>2]|0)==0){break}else{E=E+1|0}}if((E|0)==-1){break}else{bf=0}while(1){if((bf|0)>=(p|0)){bg=-1;break}if((c[be+(bf<<2)>>2]|0)==0){bg=bf;break}else{bf=bf+1|0}}p=c[e+236>>2]|0;E=e+240|0;bc=c[E>>2]|0;bb=p-bc|0;aS=(ba|0)<(bb|0)?ba:bb;bb=(aS|0)<0?0:aS;aS=c[e+228>>2]|0;a9=(c[e+232>>2]|0)+bc|0;a8=c[(c[f+16>>2]|0)+(bg<<2)>>2]|0;do{if((a8|0)==2){IY(aS+(a9<<1)|0,c[(c[f+4>>2]|0)+(bg<<2)>>2]|0,bb<<1)}else{a7=(bb|0)>0;if((a8|0)<=2){if(!a7){break}a6=bc-1-p|0;a5=~ba;a0=(a6|0)>(a5|0)?a6:a5;NM(aS+(a9<<1)|0,0,((a0|0)<-1?a0<<1^-2:0)|0);break}if(!a7){break}a7=f+4|0;a0=0;do{b[aS+(a0+a9<<1)>>1]=(c[(c[(c[a7>>2]|0)+(bg<<2)>>2]|0)+(a0<<2)>>2]|0)>>>16&65535;a0=a0+1|0;}while((a0|0)<(bb|0))}}while(0);c[E>>2]=(c[E>>2]|0)+bb;c[j+4>>2]=bb}}while(0);bg=e+240|0;ba=c[bg>>2]|0;bf=e+236|0;be=c[bf>>2]|0;do{if((ba|0)<(be|0)){if((c[bd>>2]|0)!=-1){x=0;i=k;return x|0}a9=e+244|0;if((c[a9>>2]|0)>=(c[e+248>>2]|0)){x=128;i=k;return x|0}aS=be-ba|0;if((be|0)==(ba|0)){break}I_((c[e+228>>2]|0)+((c[e+232>>2]|0)+ba<<1)|0,aS<<1);c[a9>>2]=(c[a9>>2]|0)+aS;c[bg>>2]=c[bf>>2]}}while(0);ba=e+252|0;I_(ba,160);c[e+264>>2]=-1;c[e+280>>2]=-1;c[e+296>>2]=-1;c[e+312>>2]=-1;c[e+328>>2]=-1;c[e+344>>2]=-1;c[e+360>>2]=-1;c[e+376>>2]=-1;c[e+392>>2]=-1;c[e+408>>2]=-1;be=ba|0;ba=e+412|0;I_(ba|0,1536);bd=e+1948|0;I_(bd,24);aS=c[e+208>>2]|0;do{if((aS|0)==0){bh=0}else{if((c[e+212>>2]|0)==0){bh=0;break}c[u>>2]=0;c[v>>2]=0;c[w>>2]=0;a9=f+8|0;p=c[f>>2]|0;bc=0;while(1){if((bc|0)>=(p|0)){bi=0;break}bj=c[a9>>2]|0;if((c[bj+(bc<<2)>>2]|0)==2){G=5901;break}else{bc=bc+1|0}}do{if((G|0)==5901){if((bc|0)==-1){bi=0;break}else{bk=0}while(1){if((bk|0)>=(p|0)){bl=-1;break}if((c[bj+(bk<<2)>>2]|0)==2){bl=bk;break}else{bk=bk+1|0}}bi=c[(c[f+4>>2]|0)+(bl<<2)>>2]|0}}while(0);Hr(aS,(c[e+228>>2]|0)+(c[e+232>>2]<<1)|0,c[bg>>2]|0,bi,u,v,w)|0;p=c[v>>2]|0;if((p|0)>0){bc=c[u>>2]|0;a9=(p|0)>1;bb=0;while(1){E=bb+1|0;a8=e+252+(bb<<4)|0;a0=bc+(bb<<4)|0;c[a8>>2]=c[a0>>2];c[a8+4>>2]=c[a0+4>>2];c[a8+8>>2]=c[a0+8>>2];c[a8+12>>2]=c[a0+12>>2];if((E|0)<(p|0)){bb=E}else{break}}bm=a9?p:1}else{bm=0}bb=c[w>>2]|0;if((bb|0)==-1){bh=bm;break}bc=bb<<1&4|9;bb=e+56|0;if((c[bb>>2]|0)==(bc|0)){bh=bm;break}c[bb>>2]=bc;c[y>>2]=c[y>>2]|4096;bh=bm}}while(0);bm=e+136|0;w=c[bm>>2]|0;switch(w|0){case 5:case 29:case 132:case 156:case 136:case 137:case 144:case 145:{bn=1;break};default:{bn=0}}if((w|0)==39){bo=(c[e+168>>2]&8192|0)==0?bn:1}else{bo=bn}do{if((bo|0)==0){bp=bh;G=5921}else{if((It(c[e+204>>2]|0,c[e+228>>2]|0,c[e+8>>2]|0,bd|0,ba|0)|0)!=0){break}bn=e+1948|0;if((c[bn>>2]|0)==0){bq=bh}else{c[e+252+(bh<<4)>>2]=e+412;c[e+252+(bh<<4)+4>>2]=c[bn>>2];c[e+252+(bh<<4)+12>>2]=0;c[e+252+(bh<<4)+8>>2]=13;bq=bh+1|0}bn=c[e+1952>>2]|0;if((bn|0)==0){br=bq}else{c[e+252+(bq<<4)>>2]=e+668;c[e+252+(bq<<4)+4>>2]=bn;c[e+252+(bq<<4)+12>>2]=1;c[e+252+(bq<<4)+8>>2]=13;br=bq+1|0}bn=c[e+1956>>2]|0;if((bn|0)==0){bs=br}else{c[e+252+(br<<4)>>2]=e+924;c[e+252+(br<<4)+4>>2]=bn;c[e+252+(br<<4)+12>>2]=2;c[e+252+(br<<4)+8>>2]=13;bs=br+1|0}bn=c[e+1960>>2]|0;if((bn|0)==0){bt=bs}else{c[e+252+(bs<<4)>>2]=e+1180;c[e+252+(bs<<4)+4>>2]=bn;c[e+252+(bs<<4)+12>>2]=3;c[e+252+(bs<<4)+8>>2]=13;bt=bs+1|0}bn=e+1964|0;if((c[bn>>2]|0)==0){bu=bt}else{c[e+252+(bt<<4)>>2]=e+1436;c[e+252+(bt<<4)+4>>2]=c[bn>>2];c[e+252+(bt<<4)+12>>2]=4;c[e+252+(bt<<4)+8>>2]=13;bu=bt+1|0}bn=c[e+1968>>2]|0;if((bn|0)==0){bp=bu;G=5921;break}c[e+252+(bu<<4)>>2]=e+1692;c[e+252+(bu<<4)+4>>2]=bn;c[e+252+(bu<<4)+12>>2]=5;c[e+252+(bu<<4)+8>>2]=13;bp=bu+1|0;G=5921}}while(0);do{if((G|0)==5921){bu=h+4|0;bt=c[bu>>2]|0;L8352:do{if((bt|0)>0){bs=f+8|0;br=c[f>>2]|0;bq=0;while(1){if((bq|0)>=(br|0)){bv=-1;break L8352}bw=c[bs>>2]|0;if((c[bw+(bq<<2)>>2]|0)==1){break}else{bq=bq+1|0}}if((bq|0)==-1){bv=-1;break}else{bx=0}while(1){if((bx|0)>=(br|0)){by=-1;break}if((c[bw+(bx<<2)>>2]|0)==1){by=bx;break}else{bx=bx+1|0}}c[e+252+(bp<<4)+4>>2]=bt<<3;c[e+252+(bp<<4)>>2]=c[(c[f+4>>2]|0)+(by<<2)>>2];c[e+252+(bp<<4)+8>>2]=2;c[e+252+(bp<<4)+12>>2]=-1;bv=bp}else{bv=-1}}while(0);bt=e+228|0;if((Km(c[e+200>>2]|0,c[e+216>>2]|0,c[bt>>2]|0,af,be)|0)!=0){break}if((bv|0)>-1){c[j+8>>2]=(c[bu>>2]|0)-((c[e+252+(bv<<4)+4>>2]|0)>>>3)}c[bg>>2]=(c[bg>>2]|0)-(c[bf>>2]|0);p=c[bm>>2]|0;switch(p|0){case 5:case 29:case 132:case 156:case 136:case 137:case 144:case 145:{bz=1;break};default:{bz=0}}if((p|0)==39){bA=(c[e+168>>2]&8192|0)==0?bz:1}else{bA=bz}if((bA|0)!=0){p=c[e+204>>2]|0;a9=c[bt>>2]|0;Iu(p,a9)|0}a9=c[g>>2]|0;if((a9|0)<=0){x=0;i=k;return x|0}p=g+8|0;bt=0;while(1){if((bt|0)>=(a9|0)){bB=-1;break}if((c[(c[p>>2]|0)+(bt<<2)>>2]|0)==3){bB=bt;break}else{bt=bt+1|0}}bt=0;while(1){if((bt|0)>=(a9|0)){bC=-1;break}if((c[(c[p>>2]|0)+(bt<<2)>>2]|0)==4){bC=bt;break}else{bt=bt+1|0}}bt=c[q>>2]|0;if((bt|0)>0){p=g+4|0;if((bC|0)==-1){a9=0;bu=0;while(1){br=(c[t+(bu<<2)>>2]|0)+a9|0;bq=bu+1|0;if((bq|0)<(bt|0)){a9=br;bu=bq}else{bD=br;break}}}else{bu=0;a9=0;while(1){bt=c[t+(a9<<2)>>2]|0;br=bt+bu|0;c[(c[(c[p>>2]|0)+(bC<<2)>>2]|0)+(a9<<2)>>2]=bt;bt=a9+1|0;if((bt|0)<(c[q>>2]|0)){bu=br;a9=bt}else{bD=br;break}}}}else{bD=0}if((bB|0)==-1){break}if((c[(c[g+12>>2]|0)+(bB<<2)>>2]|0)<(bD|0)){break}IY(c[(c[g+4>>2]|0)+(bB<<2)>>2]|0,c[e+220>>2]|0,bD);c[j>>2]=bD;x=0;i=k;return x|0}}while(0);c[y>>2]=65535;x=96;i=k;return x|0}function Hh(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0;L8401:do{if((b|0)==0){g=32}else{if((e|0)==1280){h=b+36|0;if((c[h>>2]|0)==(f|0)){g=0;break}c[h>>2]=f;g=0;break}else if((e|0)==770){h=b+48|0;if((d[h]|0|0)==(f|0)){g=0;break}if(f>>>0>=3){g=35;break}a[h]=f&255;h=b+1972|0;c[h>>2]=c[h>>2]|4096;g=0;break}else if((e|0)==65280){h=b+1972|0;if((c[h>>2]|0)==(f|0)){g=0;break}if((f&8192|0)!=0){c[b+240>>2]=0}c[h>>2]=f;g=0;break}else if((e|0)==258){h=b+20|0;if((c[h>>2]|0)==(f|0)){g=0;break}switch(f|0){case 0:case 1:case 2:case 3:case 4:case 5:case 8:{break};default:{g=35;break L8401}}c[h>>2]=f;h=b+1972|0;c[h>>2]=c[h>>2]|4097;g=0;break}else if((e|0)==262){h=b+12|0;if((c[h>>2]|0)==(f|0)){g=0;break}i=Hk(f)|0;if((i|0)==0){g=35;break}if((c[i+12>>2]|0)>(c[b+1976>>2]|0)){g=35;break}if((c[i+8>>2]|0)>(c[b+1980>>2]|0)){g=35;break}if(!((f|0)!=0&f>>>0<7)){g=35;break}c[h>>2]=f;c[b+8>>2]=c[i+4>>2];c[b+240>>2]=0;i=b+1972|0;c[i>>2]=c[i>>2]|4097;g=0;break}else if((e|0)==769){i=b+52|0;if((d[i]|0|0)==(f|0)){g=0;break}a[i]=f&255;i=b+1972|0;c[i>>2]=c[i>>2]|4096;g=0;break}else if((e|0)==768){i=b+44|0;if((c[i>>2]|0)==(f|0)){g=0;break}h=c[b+2e3>>2]|0;switch(f|0){case 6:{if((h&20|0)!=20){g=35;break L8401}break};case 10:{if((h&8|0)==0){g=35;break L8401}break};case 0:{if((h&16|0)==0){g=35;break L8401}break};case 2:{if((h&1|0)==0){g=35;break L8401}break};case 7:{if((h&20|0)!=20){g=35;break L8401}break};case 1:{if((h&2|0)==0){g=35;break L8401}break};default:{g=35;break L8401}}c[i>>2]=f;i=b+1972|0;c[i>>2]=c[i>>2]|4096;g=0;break}else if((e|0)==256){i=b|0;if((c[i>>2]|0)==(f|0)){g=0;break}switch(f|0){case 29:case 156:{if((c[b+1996>>2]&4|0)==0){g=35;break L8401}else{j=5976}break};case 2:case 129:case 17:case 23:case 39:{break};case 5:case 132:{j=5976;break};default:{g=35;break L8401}}if((j|0)==5976){if((c[b+1996>>2]&2|0)==0){g=35;break}}if((c[b+1996>>2]&1|0)==0){g=35;break}c[i>>2]=f;i=b+1972|0;c[i>>2]=c[i>>2]|4099;g=0;break}else if((e|0)==774){i=b+51|0;if((d[i]|0|0)==(f|0)){g=0;break}if(f>>>0>=2){g=35;break}a[i]=f&255;i=b+1972|0;c[i>>2]=c[i>>2]|4096;g=0;break}else if((e|0)==257){i=b+16|0;if((c[i>>2]|0)==(f|0)){g=0;break}c[i>>2]=f;i=b+1972|0;c[i>>2]=c[i>>2]|4097;g=0;break}else if((e|0)==260){i=b+61|0;if((d[i]|0|0)==(f|0)){g=0;break}a[i]=f&255;i=b+1972|0;c[i>>2]=c[i>>2]|4099;g=0;break}else if((e|0)==261){i=b+32|0;if((c[i>>2]|0)==(f|0)){g=0;break}if(!((f|0)==1024|(f|0)==512|(f|0)==480)){g=35;break}c[i>>2]=f;i=b+1972|0;c[i>>2]=c[i>>2]|4097;g=0;break}else if((e|0)==512){i=b+28|0;if((c[i>>2]|0)==(f|0)){g=0;break}if(f>>>0>=2){g=35;break}c[i>>2]=f;i=b+1972|0;c[i>>2]=c[i>>2]|1;g=0;break}else if((e|0)==771){i=b+49|0;if((d[i]|0|0)==(f|0)){g=0;break}if(!((f|0)!=0&f>>>0<5)){g=35;break}a[i]=f&255;i=b+1972|0;c[i>>2]=c[i>>2]|4096;g=0;break}else if((e|0)==263){i=b+152|0;if((c[i>>2]|0)==(f|0)){g=0;break}if(f>>>0>=3){g=35;break}c[i>>2]=f;c[b+240>>2]=0;i=b+1972|0;c[i>>2]=c[i>>2]|4099;g=0;break}else if((e|0)==259){i=b+4|0;if((c[i>>2]|0)==(f|0)){g=0;break}if(!((f|0)==96e3|(f|0)==88200|(f|0)==64e3|(f|0)==48e3|(f|0)==44100|(f|0)==32e3|(f|0)==24e3|(f|0)==22050|(f|0)==16e3|(f|0)==12e3|(f|0)==11025|(f|0)==8e3)){g=35;break}c[i>>2]=f;c[b+240>>2]=0;i=b+1972|0;c[i>>2]=c[i>>2]|4099;g=0;break}else if((e|0)==1536){i=b+60|0;if((d[i]|0|0)==(f|0)){g=0;break}if(f>>>0>=3){g=35;break}a[i]=f&255;i=b+1972|0;c[i>>2]=c[i>>2]|1;g=0;break}else if((e|0)==515){i=b+24|0;if((c[i>>2]|0)==(f|0)){g=0;break}c[i>>2]=f;i=b+1972|0;c[i>>2]=c[i>>2]|1;g=0;break}else{g=34;break}}}while(0);return g|0}function Hi(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0;d=i;i=i+120|0;e=d|0;I_(b,92);f=b+88|0;c[f>>2]=64;c[b>>2]=((c[a+1980>>2]|0)*6144|0)>>3;c[b+4>>2]=c[a+180>>2];g=a+8|0;c[b+8>>2]=((c[a+240>>2]|0)>>>0)/((c[g>>2]|0)>>>0)|0;c[b+12>>2]=c[g>>2];c[b+16>>2]=((c[a+236>>2]|0)>>>0)/((c[g>>2]|0)>>>0)|0;c[b+20>>2]=((c[a+248>>2]|0)>>>0)/((c[g>>2]|0)>>>0)|0;g=e+8|0;Ht(g,d+56|0,64,0);h=e+4|0;c[h>>2]=0;j=e|0;c[j>>2]=0;k=e+40|0;c[k>>2]=1;l=H3(c[a+216>>2]|0,a+64|0,e,d+48|0)|0;if((c[k>>2]|0)==0){Hy(g,c[h>>2]|0,0)}else{Hw(g,c[j>>2]|0,c[h>>2]|0)}c[h>>2]=0;c[j>>2]=0;e=(c[k>>2]|0)==0;a=HA(g)|0;if(e){Hz(g,a&7,c[k>>2]&255)}else{Hw(g,0,-a&7)}if((c[k>>2]|0)==0){Hy(g,c[h>>2]|0,0)}else{Hw(g,c[j>>2]|0,c[h>>2]|0)}c[h>>2]=0;c[j>>2]=0;a=HA(g)|0;if(a>>>0>c[f>>2]<<3>>>0){m=64;i=d;return m|0}if((c[k>>2]|0)==0){Hy(g,c[h>>2]|0,0)}else{Hw(g,c[j>>2]|0,c[h>>2]|0)}c[h>>2]=0;c[j>>2]=0;HB(g,b+24|0,f);m=(l|0)==0?0:64;i=d;return m|0}function Hj(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;d=c[a>>2]|0;if((d|0)==0){e=0}else{f=0;while(1){if((c[1921848+(f<<4)>>2]|0)==(d|0)){break}else{f=f+1|0}}g=(c[1921852+(f<<4)>>2]|0)==(b|0)?d:-1;h=(g|0)==-1;i=h?12512:0;return i|0}while(1){if((e|0)>=7){j=-1;break}if((c[1921852+(e<<4)>>2]|0)==(b|0)){k=6056;break}else{e=e+1|0}}if((k|0)==6056){j=c[1921848+(e<<4)>>2]|0}c[a>>2]=j;g=j;h=(g|0)==-1;i=h?12512:0;return i|0}function Hk(a){a=a|0;var b=0,d=0,e=0,f=0;b=0;while(1){if((b|0)>=7){d=0;e=6068;break}f=1921848+(b<<4)|0;if((c[f>>2]|0)==(a|0)){d=f;e=6067;break}else{b=b+1|0}}if((e|0)==6068){return d|0}else if((e|0)==6067){return d|0}return 0}function Hl(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;I_(d,160);e=0;while(1){if((e|0)>=7){break}if((c[1921848+(e<<4)>>2]|0)==(a|0)){f=6072;break}else{e=e+1|0}}if((f|0)==6072){c[d>>2]=a;c[d+4>>2]=c[1921852+(e<<4)>>2];c[d+8>>2]=c[1921856+(e<<4)>>2];c[d+12>>2]=c[1921860+(e<<4)>>2]}switch(a|0){case 2:{if((b|0)==0){g=1917872}else if((b|0)==1){g=1918984}else{g=1918568}a=7;while(1){if((a|0)<=0){break}if((c[g+(a*52|0)>>2]|0)==2){break}else{a=a-1|0}}c[d+16>>2]=1;c[d+36>>2]=2147483647;c[d+24>>2]=2;c[d+28>>2]=c[g+(a*52|0)+4>>2];c[d+32>>2]=c[g+(a*52|0)+8>>2];c[d+20>>2]=0;h=0;return h|0};case 5:{if((b|0)==0){i=1917872}else if((b|0)==1){i=1918984}else{i=1918568}a=7;while(1){if((a|0)<=0){break}if((c[i+(a*52|0)>>2]|0)==5){break}else{a=a-1|0}}c[d+16>>2]=0;c[d+36>>2]=558345728;c[d+24>>2]=1;c[d+28>>2]=c[i+(a*52|0)+4>>2];c[d+20>>2]=0;if((b|0)==0){j=1917872}else if((b|0)==1){j=1918984}else{j=1918568}a=7;while(1){if((a|0)<=0){break}if((c[j+(a*52|0)>>2]|0)==5){break}else{a=a-1|0}}c[d+40>>2]=1;c[d+60>>2]=794568960;c[d+48>>2]=2;c[d+52>>2]=c[j+(a*52|0)+8>>2];c[d+56>>2]=c[j+(a*52|0)+12>>2];c[d+44>>2]=0;if((b|0)==1){k=1918984}else if((b|0)==0){k=1917872}else{k=1918568}a=7;while(1){if((a|0)<=0){break}if((c[k+(a*52|0)>>2]|0)==5){break}else{a=a-1|0}}c[d+64>>2]=1;c[d+84>>2]=794568960;c[d+72>>2]=2;c[d+76>>2]=c[k+(a*52|0)+16>>2];c[d+80>>2]=c[k+(a*52|0)+20>>2];c[d+68>>2]=1;h=0;return h|0};case 4:{if((b|0)==1){l=1918984}else if((b|0)==0){l=1917872}else{l=1918568}a=7;while(1){if((a|0)<=0){break}if((c[l+(a*52|0)>>2]|0)==4){break}else{a=a-1|0}}c[d+16>>2]=0;c[d+36>>2]=644245120;c[d+24>>2]=1;c[d+28>>2]=c[l+(a*52|0)+4>>2];c[d+20>>2]=0;if((b|0)==0){m=1917872}else if((b|0)==1){m=1918984}else{m=1918568}a=7;while(1){if((a|0)<=0){break}if((c[m+(a*52|0)>>2]|0)==4){break}else{a=a-1|0}}c[d+40>>2]=1;c[d+60>>2]=858993472;c[d+48>>2]=2;c[d+52>>2]=c[m+(a*52|0)+8>>2];c[d+56>>2]=c[m+(a*52|0)+12>>2];c[d+44>>2]=0;if((b|0)==1){n=1918984}else if((b|0)==0){n=1917872}else{n=1918568}a=7;while(1){if((a|0)<=0){break}if((c[n+(a*52|0)>>2]|0)==4){break}else{a=a-1|0}}c[d+64>>2]=0;c[d+84>>2]=644245120;c[d+72>>2]=1;c[d+76>>2]=c[n+(a*52|0)+16>>2];c[d+68>>2]=1;h=0;return h|0};case 6:{if((b|0)==0){o=1917872}else if((b|0)==1){o=1918984}else{o=1918568}a=7;while(1){if((a|0)<=0){break}if((c[o+(a*52|0)>>2]|0)==6){break}else{a=a-1|0}}c[d+16>>2]=0;c[d+36>>2]=515396064;c[d+24>>2]=1;c[d+28>>2]=c[o+(a*52|0)+4>>2];c[d+20>>2]=0;if((b|0)==1){p=1918984}else if((b|0)==0){p=1917872}else{p=1918568}a=7;while(1){if((a|0)<=0){break}if((c[p+(a*52|0)>>2]|0)==6){break}else{a=a-1|0}}c[d+40>>2]=1;c[d+60>>2]=751619264;c[d+48>>2]=2;c[d+52>>2]=c[p+(a*52|0)+8>>2];c[d+56>>2]=c[p+(a*52|0)+12>>2];c[d+44>>2]=0;if((b|0)==1){q=1918984}else if((b|0)==0){q=1917872}else{q=1918568}a=7;while(1){if((a|0)<=0){break}if((c[q+(a*52|0)>>2]|0)==6){break}else{a=a-1|0}}c[d+64>>2]=1;c[d+84>>2]=751619264;c[d+72>>2]=2;c[d+76>>2]=c[q+(a*52|0)+16>>2];c[d+80>>2]=c[q+(a*52|0)+20>>2];c[d+68>>2]=1;if((b|0)==0){r=1917872}else if((b|0)==1){r=1918984}else{r=1918568}a=7;while(1){if((a|0)<=0){break}if((c[r+(a*52|0)>>2]|0)==6){break}else{a=a-1|0}}c[d+88>>2]=3;c[d+108>>2]=128849016;c[d+96>>2]=1;c[d+100>>2]=c[r+(a*52|0)+24>>2];c[d+92>>2]=0;h=0;return h|0};case 7:{if((b|0)==1){s=1918984}else if((b|0)==0){s=1917872}else{s=1918568}a=7;while(1){if((a|0)<=0){break}if((c[s+(a*52|0)>>2]|0)==7){break}else{a=a-1|0}}c[d+16>>2]=0;c[d+36>>2]=386547072;c[d+24>>2]=1;c[d+28>>2]=c[s+(a*52|0)+4>>2];c[d+20>>2]=0;if((b|0)==1){t=1918984}else if((b|0)==0){t=1917872}else{t=1918568}a=7;while(1){if((a|0)<=0){break}if((c[t+(a*52|0)>>2]|0)==7){break}else{a=a-1|0}}c[d+40>>2]=1;c[d+60>>2]=558345728;c[d+48>>2]=2;c[d+52>>2]=c[t+(a*52|0)+8>>2];c[d+56>>2]=c[t+(a*52|0)+12>>2];c[d+44>>2]=0;if((b|0)==1){u=1918984}else if((b|0)==0){u=1917872}else{u=1918568}a=7;while(1){if((a|0)<=0){break}if((c[u+(a*52|0)>>2]|0)==7){break}else{a=a-1|0}}c[d+64>>2]=1;c[d+84>>2]=558345728;c[d+72>>2]=2;c[d+76>>2]=c[u+(a*52|0)+16>>2];c[d+80>>2]=c[u+(a*52|0)+20>>2];c[d+68>>2]=1;if((b|0)==1){v=1918984}else if((b|0)==0){v=1917872}else{v=1918568}a=7;while(1){if((a|0)<=0){break}if((c[v+(a*52|0)>>2]|0)==7){break}else{a=a-1|0}}c[d+88>>2]=1;c[d+108>>2]=558345728;c[d+96>>2]=2;c[d+100>>2]=c[v+(a*52|0)+24>>2];c[d+104>>2]=c[v+(a*52|0)+28>>2];c[d+92>>2]=2;if((b|0)==1){w=1918984}else if((b|0)==0){w=1917872}else{w=1918568}a=7;while(1){if((a|0)<=0){break}if((c[w+(a*52|0)>>2]|0)==7){break}else{a=a-1|0}}c[d+112>>2]=3;c[d+132>>2]=85899344;c[d+120>>2]=1;c[d+124>>2]=c[w+(a*52|0)+32>>2];c[d+116>>2]=0;h=0;return h|0};case 1:{if((b|0)==1){x=1918984}else if((b|0)==0){x=1917872}else{x=1918568}a=7;while(1){if((a|0)<=0){break}if((c[x+(a*52|0)>>2]|0)==1){break}else{a=a-1|0}}c[d+16>>2]=0;c[d+36>>2]=2147483647;c[d+24>>2]=1;c[d+28>>2]=c[x+(a*52|0)+4>>2];c[d+20>>2]=0;h=0;return h|0};case 3:{if((b|0)==0){y=1917872}else if((b|0)==1){y=1918984}else{y=1918568}a=7;while(1){if((a|0)<=0){break}if((c[y+(a*52|0)>>2]|0)==3){break}else{a=a-1|0}}c[d+16>>2]=0;c[d+36>>2]=858993472;c[d+24>>2]=1;c[d+28>>2]=c[y+(a*52|0)+4>>2];c[d+20>>2]=0;if((b|0)==1){z=1918984}else if((b|0)==0){z=1917872}else{z=1918568}b=7;while(1){if((b|0)<=0){break}if((c[z+(b*52|0)>>2]|0)==3){break}else{b=b-1|0}}c[d+40>>2]=1;c[d+60>>2]=1288490240;c[d+48>>2]=2;c[d+52>>2]=c[z+(b*52|0)+8>>2];c[d+56>>2]=c[z+(b*52|0)+12>>2];c[d+44>>2]=0;h=0;return h|0};default:{h=12512;return h|0}}return 0}function Hm(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;do{if((d|0)==0){g=0}else{h=d>>31^d;if((h|0)<=-1){g=-1;break}i=~h;h=0;while(1){j=i<<1;if((j|0)<0){i=j;h=h+1|0}else{g=h;break}}}}while(0);switch(c[b>>2]|0){case 2:{h=a+52|0;c[c[h>>2]>>2]=d>>1;c[(c[h>>2]|0)+4>>2]=f<<1;c[(c[h>>2]|0)+16>>2]=c[b+36>>2];k=0;return k|0};case 5:{h=b+36|0;i=a+52|0;c[(c[i>>2]|0)+16>>2]=c[h>>2];j=b+60|0;l=a+56|0;c[(c[l>>2]|0)+16>>2]=c[j>>2];m=b+84|0;n=a+60|0;c[(c[n>>2]|0)+16>>2]=c[m>>2];o=c[h>>2]|0;h=c[j>>2]|0;j=c[m>>2]|0;m=d<<g;p=m;q=(m|0)<0|0?-1:0;N2(o,(o|0)<0|0?-1:0,p,q)|0;c[c[i>>2]>>2]=(G<<1|0>>>31)>>g;N2(h,(h|0)<0|0?-1:0,p,q)|0;h=g+1|0;c[c[l>>2]>>2]=(G<<1|0>>>31)>>h;N2(j,(j|0)<0|0?-1:0,p,q)|0;c[c[n>>2]>>2]=(G<<1|0>>>31)>>h;c[(c[i>>2]|0)+4>>2]=f;i=f<<1;c[(c[l>>2]|0)+4>>2]=i;c[(c[n>>2]|0)+4>>2]=i;k=0;return k|0};case 3:{i=b+36|0;n=a+52|0;c[(c[n>>2]|0)+16>>2]=c[i>>2];l=b+60|0;h=a+56|0;c[(c[h>>2]|0)+16>>2]=c[l>>2];q=c[i>>2]|0;i=c[l>>2]|0;l=d<<g;p=l;j=(l|0)<0|0?-1:0;N2(q,(q|0)<0|0?-1:0,p,j)|0;c[c[n>>2]>>2]=(G<<1|0>>>31)>>g;N2(i,(i|0)<0|0?-1:0,p,j)|0;c[c[h>>2]>>2]=(G<<1|0>>>31)>>g+1;c[(c[n>>2]|0)+4>>2]=f;c[(c[h>>2]|0)+4>>2]=f<<1;k=0;return k|0};case 4:{h=b+36|0;n=a+52|0;c[(c[n>>2]|0)+16>>2]=c[h>>2];j=b+60|0;p=a+56|0;c[(c[p>>2]|0)+16>>2]=c[j>>2];i=b+84|0;q=a+60|0;c[(c[q>>2]|0)+16>>2]=c[i>>2];l=c[h>>2]|0;h=c[j>>2]|0;j=c[i>>2]|0;i=d<<g;o=i;m=(i|0)<0|0?-1:0;N2(l,(l|0)<0|0?-1:0,o,m)|0;c[c[n>>2]>>2]=(G<<1|0>>>31)>>g;N2(h,(h|0)<0|0?-1:0,o,m)|0;c[c[p>>2]>>2]=(G<<1|0>>>31)>>g+1;N2(j,(j|0)<0|0?-1:0,o,m)|0;c[c[q>>2]>>2]=(G<<1|0>>>31)>>g;c[(c[n>>2]|0)+4>>2]=f;c[(c[p>>2]|0)+4>>2]=f<<1;c[(c[q>>2]|0)+4>>2]=f;k=0;return k|0};case 6:{q=b+36|0;p=a+52|0;c[(c[p>>2]|0)+16>>2]=c[q>>2];n=b+60|0;m=a+56|0;c[(c[m>>2]|0)+16>>2]=c[n>>2];o=b+84|0;j=a+60|0;c[(c[j>>2]|0)+16>>2]=c[o>>2];h=b+108|0;l=a+64|0;c[(c[l>>2]|0)+16>>2]=c[h>>2];i=c[q>>2]|0;q=c[n>>2]|0;n=c[o>>2]|0;o=c[h>>2]|0;h=f*5|0;r=(f|0)>(e|0)?f:e;do{if((r|0)==0){s=0}else{t=r>>31^r;if((t|0)<=-1){s=-1;break}u=~t;t=0;while(1){v=u<<1;if((v|0)<0){u=v;t=t+1|0}else{s=t;break}}}}while(0);r=f<<s;t=o;u=(o|0)<0|0?-1:0;N2(r,(r|0)<0|0?-1:0,t,u)|0;r=(G<<1|0>>>31)>>s<<1;o=e<<s;N2(o,(o|0)<0|0?-1:0,t,u)|0;o=G<<1|0>>>31;N2(o,(o|0)<0|0?-1:0,1181116032,0)|0;o=(G<<2|0>>>30)>>s;s=(r|0)>(o|0)?r:o;o=h-s|0;do{if((h|0)==(s|0)){w=0}else{r=o>>31^o;if((r|0)<=-1){w=-1;break}v=~r;r=0;while(1){x=v<<1;if((x|0)<0){v=x;r=r+1|0}else{w=r;break}}}}while(0);h=o<<w;o=c[43185]|0;N2(o,(o|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=(G<<1|0>>>31)>>w;w=d<<g;o=w;r=(w|0)<0|0?-1:0;N2(i,(i|0)<0|0?-1:0,o,r)|0;c[c[p>>2]>>2]=(G<<1|0>>>31)>>g;N2(q,(q|0)<0|0?-1:0,o,r)|0;q=g+1|0;c[c[m>>2]>>2]=(G<<1|0>>>31)>>q;N2(n,(n|0)<0|0?-1:0,o,r)|0;c[c[j>>2]>>2]=(G<<1|0>>>31)>>q;N2(t,u,o,r)|0;c[c[l>>2]>>2]=(G<<1|0>>>31)>>g;c[(c[p>>2]|0)+4>>2]=h;p=h<<1;c[(c[m>>2]|0)+4>>2]=p;c[(c[j>>2]|0)+4>>2]=p;c[(c[l>>2]|0)+4>>2]=s;k=0;return k|0};case 7:{s=b+36|0;l=a+52|0;c[(c[l>>2]|0)+16>>2]=c[s>>2];p=b+60|0;j=a+56|0;c[(c[j>>2]|0)+16>>2]=c[p>>2];m=b+84|0;h=a+60|0;c[(c[h>>2]|0)+16>>2]=c[m>>2];r=b+108|0;o=a+64|0;c[(c[o>>2]|0)+16>>2]=c[r>>2];u=b+132|0;t=a+68|0;c[(c[t>>2]|0)+16>>2]=c[u>>2];q=c[s>>2]|0;s=c[p>>2]|0;p=c[m>>2]|0;m=c[r>>2]|0;r=c[u>>2]|0;u=f*7|0;n=(f|0)>(e|0)?f:e;do{if((n|0)==0){y=0}else{i=n>>31^n;if((i|0)<=-1){y=-1;break}w=~i;i=0;while(1){v=w<<1;if((v|0)<0){w=v;i=i+1|0}else{y=i;break}}}}while(0);n=f<<y;i=r;w=(r|0)<0|0?-1:0;N2(n,(n|0)<0|0?-1:0,i,w)|0;n=(G<<1|0>>>31)>>y<<1;r=e<<y;N2(r,(r|0)<0|0?-1:0,i,w)|0;r=G<<1|0>>>31;N2(r,(r|0)<0|0?-1:0,1181116032,0)|0;r=(G<<2|0>>>30)>>y;y=(n|0)>(r|0)?n:r;r=(u-y|0)/7|0;u=d<<g;n=u;e=(u|0)<0|0?-1:0;N2(q,(q|0)<0|0?-1:0,n,e)|0;c[c[l>>2]>>2]=(G<<1|0>>>31)>>g;N2(s,(s|0)<0|0?-1:0,n,e)|0;s=g+1|0;c[c[j>>2]>>2]=(G<<1|0>>>31)>>s;N2(p,(p|0)<0|0?-1:0,n,e)|0;c[c[h>>2]>>2]=(G<<1|0>>>31)>>s;N2(m,(m|0)<0|0?-1:0,n,e)|0;c[c[o>>2]>>2]=(G<<1|0>>>31)>>s;N2(i,w,n,e)|0;c[c[t>>2]>>2]=(G<<1|0>>>31)>>g;c[(c[l>>2]|0)+4>>2]=r;l=r<<1;c[(c[j>>2]|0)+4>>2]=l;c[(c[h>>2]|0)+4>>2]=l;c[(c[o>>2]|0)+4>>2]=l;c[(c[t>>2]|0)+4>>2]=y;k=0;return k|0};case 1:{y=a+52|0;c[c[y>>2]>>2]=d;c[(c[y>>2]|0)+4>>2]=f;c[(c[y>>2]|0)+16>>2]=c[b+36>>2];k=0;return k|0};default:{k=12512;return k|0}}return 0}function Hn(a){a=a|0;var b=0;switch(a|0){case 1:{b=1;break};case 2:case 3:case 4:case 5:case 6:case 7:{b=2;break};default:{b=0}}return b|0}function Ho(a){a=a|0;var b=0,d=0,e=0;if((a|0)==0){b=32;return b|0}d=IQ(1,33180)|0;if((d|0)==0){b=33;return b|0}I_(d,33180);e=d+4|0;if((Kt(e)|0)==0){c[a>>2]=d;b=0;return b|0}else{Ku(e)|0;IR(d);b=33;return b|0}return 0}function Hp(a){a=a|0;var b=0,d=0;if((a|0)==0){b=32;return b|0}d=c[a>>2]|0;if((d|0)==0){b=0;return b|0}Ku(d+4|0)|0;IR(c[a>>2]|0);c[a>>2]=0;b=0;return b|0}function Hq(b,d,e,f,g,h,i,j,k){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0;if((b|0)==0){l=32;return l|0}else{m=f;n=0}while(1){o=m-g|0;if((o|0)>0){m=o;n=n+1|0}else{break}}m=b+48|0;if((c[m>>2]|0)>8){l=64;return l|0}f=-o|0;if((f|0)>2048){l=64;return l|0}o=b+8|0;IY(o,1918408,32);p=b+33176|0;c[p>>2]=0;q=b+40|0;do{if((d|0)==0){if((c[q>>2]|0)!=(f|0)){r=6265;break}if((c[m>>2]|0)!=(i|0)){r=6265;break}s=b|0;t=c[s>>2]|0;u=(e|0)==0;if((t|0)!=0|u){v=t}else{t=o|0;w=b+12|0;x=b+16|0;y=b+20|0;z=b+24|0;A=b+28|0;B=b+34|0;C=b+35|0;D=b+32|0;E=b+33|0;F=b+36|0;do{if((o|0)==0){G=b+32824|0;if((G|0)!=0){I_(G|0,96);a[b+32917|0]=0}G=b+32920|0;if((G|0)!=0){I_(G|0,96);a[b+33013|0]=0}G=b+33016|0;if((G|0)==0){break}I_(G|0,96);a[b+33109|0]=0}else{if((i|0)==2){G=0;do{H=b+32824+(G*96|0)|0;if((H|0)!=0){I=H|0;I_(I,96);c[b+32824+(G*96|0)+56>>2]=c[t>>2];c[b+32824+(G*96|0)+72>>2]=c[w>>2];c[b+32824+(G*96|0)+60>>2]=c[x>>2];c[b+32824+(G*96|0)+76>>2]=c[y>>2];a[I]=c[z>>2]&255;I=8192-(c[A>>2]|0)>>14;if((I|0)<0){J=0}else{J=(I|0)<127?I&255:127}a[b+32824+(G*96|0)+1|0]=J;a[b+32824+(G*96|0)+88|0]=a[B]|0;a[b+32824+(G*96|0)+89|0]=a[C]|0;a[b+32824+(G*96|0)+90|0]=a[D]|0;a[b+32824+(G*96|0)+91|0]=a[E]|0;a[b+32824+(G*96|0)+68|0]=1;a[b+32824+(G*96|0)+92|0]=a[F]|0;c[b+32824+(G*96|0)+80>>2]=0;c[b+32824+(G*96|0)+84>>2]=0;a[b+32824+(G*96|0)+93|0]=0}G=G+1|0;}while((G|0)<3)}else{G=0;do{I=b+32824+(G*96|0)|0;if((I|0)!=0){H=I|0;I_(H,96);c[b+32824+(G*96|0)+56>>2]=c[t>>2];c[b+32824+(G*96|0)+72>>2]=c[w>>2];c[b+32824+(G*96|0)+60>>2]=c[x>>2];c[b+32824+(G*96|0)+76>>2]=c[y>>2];a[H]=c[z>>2]&255;H=8192-(c[A>>2]|0)>>14;if((H|0)<0){K=0}else{K=(H|0)<127?H&255:127}a[b+32824+(G*96|0)+1|0]=K;a[b+32824+(G*96|0)+88|0]=a[B]|0;a[b+32824+(G*96|0)+89|0]=a[C]|0;a[b+32824+(G*96|0)+90|0]=a[D]|0;a[b+32824+(G*96|0)+91|0]=a[E]|0;a[b+32824+(G*96|0)+68|0]=1;a[b+32824+(G*96|0)+92|0]=0;c[b+32824+(G*96|0)+80>>2]=0;c[b+32824+(G*96|0)+84>>2]=0;a[b+32824+(G*96|0)+93|0]=0}G=G+1|0;}while((G|0)<3)}}}while(0);v=c[s>>2]|0}if(!((v|0)!=0&u)){L=s;break}c[p>>2]=v;L=s}else{r=6265}}while(0);if((r|0)==6265){I_(b+52|0,32768);I_(b+32824|0,288);c[b+32820>>2]=0;c[b+33112>>2]=0;L=b|0}c[q>>2]=f;c[b+44>>2]=n;c[m>>2]=i;c[L>>2]=e;if((e|0)==0){l=0;return l|0}else{return((Kv(c[b+4>>2]|0,0,0,g,h,j,k,1)|0)==0?0:64)|0}return 0}function Hr(e,f,g,h,j,k,l){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0;m=i;i=i+112|0;n=m|0;o=m+48|0;p=m+96|0;q=m+104|0;r=e+33112|0;s=c[r>>2]|0;t=e|0;u=c[t>>2]|0;v=s+1|0;c[r>>2]=v;if((v|0)>(c[e+44>>2]|0)){c[r>>2]=0;w=0}else{w=v}if((h|0)!=0){IY(e+8|0,h,32)}h=e+33176|0;v=u&255;do{if((c[h>>2]|0)==0){x=v}else{if((c[t>>2]|0)!=0){x=v;break}IY(e+8|0,1918408,32);u=c[h>>2]|0;c[h>>2]=0;x=u&255}}while(0);h=e+8|0;v=e+48|0;t=c[v>>2]|0;u=e+32824+(s*96|0)|0;do{if((u|0)!=0){r=u|0;I_(r,96);y=(h|0)==0;L8820:do{if(y){a[e+32824+(s*96|0)+93|0]=0}else{c[e+32824+(s*96|0)+56>>2]=c[h>>2];c[e+32824+(s*96|0)+72>>2]=c[e+12>>2];c[e+32824+(s*96|0)+60>>2]=c[e+16>>2];c[e+32824+(s*96|0)+76>>2]=c[e+20>>2];a[r]=c[e+24>>2]&255;z=8192-(c[e+28>>2]|0)>>14;if((z|0)<0){A=0}else{A=(z|0)<127?z&255:127}a[e+32824+(s*96|0)+1|0]=A;a[e+32824+(s*96|0)+88|0]=a[e+34|0]|0;a[e+32824+(s*96|0)+89|0]=a[e+35|0]|0;a[e+32824+(s*96|0)+90|0]=a[e+32|0]|0;a[e+32824+(s*96|0)+91|0]=a[e+33|0]|0;a[e+32824+(s*96|0)+68|0]=1;if((t|0)==2){a[e+32824+(s*96|0)+92|0]=a[e+36|0]|0}else{a[e+32824+(s*96|0)+92|0]=0}c[e+32824+(s*96|0)+80>>2]=0;c[e+32824+(s*96|0)+84>>2]=0;a[e+32824+(s*96|0)+93|0]=x;z=e+32824+(s*96|0)+93|0;if(x<<24>>24==0){break}B=c[e+4>>2]|0;C=e+32824+(s*96|0)+56|0;D=c[C>>2]|0;E=D>>>0<6?D:0;D=e+32824+(s*96|0)+72|0;F=c[D>>2]|0;G=F>>>0<6?F:0;do{if((B|0)!=0){if((E|0)==(Kz(B)|0)){if((G|0)!=(KA(B)|0)){H=6314}}else{H=6314}if((H|0)==6314){Kw(B,E,G)|0}F=e+32824+(s*96|0)+69|0;if((G|0)==0){a[F]=-128;I=-128}else{I=a[F]|0}J=e+32824+(s*96|0)+18|0;K=e+32824+(s*96|0)+2|0;L=d[J]<<14;c[p>>2]=(a[K]|0)==0?L:-L|0;L=ac(8-((I&255)>>>4&255)|0,101008904)|0;c[q>>2]=L+(ac(I&15,-6734375)|0)>>8;if((Ky(B,f,-(d[e+32824+(s*96|0)+1|0]<<14)|0,c[e+32824+(s*96|0)+60>>2]|0,c[e+32824+(s*96|0)+76>>2]|0,c[1914704+(a[e+32824+(s*96|0)+88|0]<<2)>>2]|0,c[1914704+(a[e+32824+(s*96|0)+89|0]<<2)>>2]|0,p,q)|0)!=0){break}a[e+32824+(s*96|0)+35|0]=0;L=c[p>>2]|0;if((L|0)<0){a[K]=1;M=-L|0}else{a[K]=0;M=L}a[J]=(M|0)<2080768?(M+8192|0)>>>14&255:127;J=((3156476-(c[q>>2]|0)|0)*15|0)+197283|0;L=(J|0)/394566|0;do{if((J|0)>94695839){N=-1}else{if((J|0)<-394565){N=0;break}N=(((J|0)/5918490|0)<<4|((L|0)%15|0))&255}}while(0);a[F]=N;break L8820}}while(0);B=c[v>>2]|0;I_(r,96);if(y){a[z]=0;break}c[C>>2]=c[h>>2];c[D>>2]=c[e+12>>2];c[e+32824+(s*96|0)+60>>2]=c[e+16>>2];c[e+32824+(s*96|0)+76>>2]=c[e+20>>2];a[r]=c[e+24>>2]&255;G=8192-(c[e+28>>2]|0)>>14;if((G|0)<0){O=0}else{O=(G|0)<127?G&255:127}a[e+32824+(s*96|0)+1|0]=O;a[e+32824+(s*96|0)+88|0]=a[e+34|0]|0;a[e+32824+(s*96|0)+89|0]=a[e+35|0]|0;a[e+32824+(s*96|0)+90|0]=a[e+32|0]|0;a[e+32824+(s*96|0)+91|0]=a[e+33|0]|0;a[e+32824+(s*96|0)+68|0]=1;if((B|0)==2){a[e+32824+(s*96|0)+92|0]=a[e+36|0]|0}else{a[e+32824+(s*96|0)+92|0]=0}c[e+32824+(s*96|0)+80>>2]=0;c[e+32824+(s*96|0)+84>>2]=0;a[z]=x}}while(0);r=e+32824+(w*96|0)|0;if((e|0)==0|(r|0)==0){break}y=e+33172|0;c[y>>2]=0;B=e+33136|0;c[B>>2]=-1;G=e+32824+(w*96|0)+93|0;do{if((a[G]|0)!=0){E=e+33116|0;c[e+33140>>2]=E;c[e+33148>>2]=11;c[e+33152>>2]=-1;L=o+8|0;Ht(L,E,16,0);E=o+4|0;J=o|0;K=o+40|0;c[K>>2]=1;P=c[497855]|0;Q=(a[e+32824+(w*96|0)+64|0]|0)!=0&P;R=a[e+32824+(w*96|0)+34|0]|0;c[E>>2]=3;S=Q<<1|R<<24>>24!=0&P;c[J>>2]=S;do{if(R<<24>>24==0){Q=a[r|0]|0;T=S;U=1;V=Q;W=Q&255;X=4;H=6344}else{Q=c[497858]|0;Y=d[e+32824+(w*96|0)+35|0]|0;Z=d[e+32824+(w*96|0)+52|0]|0;c[E>>2]=11;_=(Y&Q|S<<4)<<4|Z&Q;c[J>>2]=_;Q=Y+1|0;Y=c[497862]|0;Z=0;$=11;aa=_;while(1){_=d[e+32824+(w*96|0)+36+Z|0]|0;ab=$+8|0;if(ab>>>0<32){c[E>>2]=ab;ad=_&Y|aa<<8;ae=ab}else{Hw(L,aa,$);c[E>>2]=8;ad=_&Y;ae=8}c[J>>2]=ad;_=Z+1|0;if((_|0)<(Q|0)){Z=_;$=ae;aa=ad}else{break}}aa=r|0;$=a[aa]|0;Z=$&255;Y=ae+1|0;if(Y>>>0<32){T=ad;U=Q;V=$;W=Z;X=Y;H=6344;break}Hw(L,ad,ae);c[E>>2]=1;af=Z&P;ag=a[aa]|0;ah=1;ai=Q}}while(0);if((H|0)==6344){c[E>>2]=X;af=T<<1|W&P;ag=V;ah=X;ai=U}c[J>>2]=af;L8880:do{if(ag<<24>>24==0){aj=ah;ak=af;al=c[497861]|0}else{S=a[e+32824+(w*96|0)+1|0]|0;R=c[497861]|0;z=ah+7|0;do{if(z>>>0<32){c[E>>2]=z;D=R&S|af<<7;c[J>>2]=D;C=ah+8|0;if(C>>>0<32){am=D;an=C;break}Hw(L,D,z);c[E>>2]=1;c[J>>2]=0;aj=1;ak=0;al=R;break L8880}else{Hw(L,af,ah);c[E>>2]=7;D=R&S;c[J>>2]=D;am=D;an=8}}while(0);c[E>>2]=an;S=am<<1;c[J>>2]=S;aj=an;ak=S;al=R}}while(0);S=0;z=aj;Q=ak;while(1){D=(a[e+32824+(w*96|0)+2+S|0]|0)!=0|0;C=z+1|0;do{if(C>>>0<32){c[E>>2]=C;aa=D&P|Q<<1;c[J>>2]=aa;Z=d[e+32824+(w*96|0)+18+S|0]|0;Y=z+8|0;if(Y>>>0<32){ao=aa;ap=Z;aq=Y;H=6357;break}Hw(L,aa,C);c[E>>2]=7;ar=Z&al;as=7}else{Hw(L,Q,z);c[E>>2]=1;Z=D&P;c[J>>2]=Z;ao=Z;ap=d[e+32824+(w*96|0)+18+S|0]|0;aq=8;H=6357}}while(0);if((H|0)==6357){H=0;c[E>>2]=aq;ar=ao<<7|ap&al;as=aq}c[J>>2]=ar;D=S+1|0;if((D|0)<(ai|0)){S=D;z=as;Q=ar}else{break}}if((c[K>>2]|0)==0){Hy(L,as,0)}else{Hw(L,ar,as)}c[E>>2]=0;c[J>>2]=0;Q=HA(L)|0;z=c[y>>2]|0;c[e+33140+(z<<4)+4>>2]=Q;Q=z+1|0;c[y>>2]=Q;if((a[e+32824+(w*96|0)+90|0]|0)!=0){c[B>>2]=d[1915240+(a[e+32824+(w*96|0)+89|0]|0)|0]|0}if((a[G]|0)!=2){break}c[e+33140+(Q<<4)>>2]=e+33128;Q=c[y>>2]|0;c[e+33140+(Q<<4)+8>>2]=2;c[e+33140+(Q<<4)+12>>2]=-1;z=n+8|0;Ht(z,c[e+33140+(Q<<4)>>2]|0,16,0);Q=n+4|0;S=n|0;D=n+40|0;c[D>>2]=1;C=c[497862]|0;R=c[497856]|0;Z=a[e+32824+(w*96|0)+91|0]|0;aa=e+32824+(w*96|0)+68|0;Y=a[aa]|0;$=e+32824+(w*96|0)+80|0;F=((Z<<24>>24!=0&P|((C<<2&752|R&3)<<2|d[e+32824+(w*96|0)+92|0]&R)<<8)<<2|Y<<24>>24!=0&P)<<1|(c[$>>2]|0)!=0&P;R=e+32824+(w*96|0)+84|0;_=(c[R>>2]|0)!=0|0;c[Q>>2]=24;ab=F<<1|_&P;c[S>>2]=ab;if(Z<<24>>24==0){at=24;au=ab;av=Y}else{Y=(a[e+32824+(w*96|0)+89|0]|a[e+32824+(w*96|0)+88|0]<<4|-120)<<24>>24;Hw(z,ab,24);c[Q>>2]=8;ab=Y&C;c[S>>2]=ab;at=8;au=ab;av=a[aa]|0}L8910:do{if(av<<24>>24==0){aw=at;ax=au}else{aa=at+8|0;do{if(aa>>>0<32){c[Q>>2]=aa;ab=au<<8|C&1;c[S>>2]=ab;Y=d[e+32824+(w*96|0)+69|0]|0;Z=at+16|0;if(Z>>>0<32){ay=ab;az=Y;aA=Z;break}Hw(z,ab,aa);c[Q>>2]=8;ab=Y&C;c[S>>2]=ab;aw=8;ax=ab;break L8910}else{Hw(z,au,at);c[Q>>2]=8;ab=C&1;c[S>>2]=ab;ay=ab;az=d[e+32824+(w*96|0)+69|0]|0;aA=16}}while(0);c[Q>>2]=aA;aa=ay<<8|az&C;c[S>>2]=aa;aw=aA;ax=aa}}while(0);do{if((c[$>>2]|0)==0){aB=aw;aC=ax}else{C=aw+16|0;if(C>>>0<32){c[Q>>2]=C;P=ax<<16;c[S>>2]=P;aB=C;aC=P;break}else{Hw(z,ax,aw);c[Q>>2]=16;c[S>>2]=0;aB=16;aC=0;break}}}while(0);do{if((c[R>>2]|0)==0){aD=aB;aE=aC}else{$=aB+16|0;if($>>>0<32){c[Q>>2]=$;P=aC<<16;c[S>>2]=P;aD=$;aE=P;break}else{Hw(z,aC,aB);c[Q>>2]=16;c[S>>2]=0;aD=16;aE=0;break}}}while(0);if((c[D>>2]|0)==0){Hy(z,aD,0)}else{Hw(z,aE,aD)}c[Q>>2]=0;c[S>>2]=0;R=HA(z)|0;P=c[y>>2]|0;c[e+33140+(P<<4)+4>>2]=R;c[y>>2]=P+1}}while(0);c[j>>2]=e+33140;c[k>>2]=c[y>>2];c[l>>2]=c[B>>2]}}while(0);l=c[e+40>>2]|0;if((l|0)==0){i=m;return 0}k=ac(c[v>>2]|0,l)|0;if((g|0)<=0){i=m;return 0}l=e+32820|0;v=0;j=c[l>>2]|0;do{aD=f+(v<<1)|0;aE=b[aD>>1]|0;b[aD>>1]=b[e+52+(j<<1)>>1]|0;b[e+52+(c[l>>2]<<1)>>1]=aE;aE=(c[l>>2]|0)+1|0;j=(aE|0)<(k|0)?aE:0;c[l>>2]=j;v=v+1|0;}while((v|0)<(g|0));i=m;return 0}function Hs(a){a=a|0;var b=0;if((a|0)==0){b=0}else{b=c[a+40>>2]|0}return b|0}function Ht(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;c[a>>2]=e;NM(a+4|0,0,16);c[a+20>>2]=b;c[a+24>>2]=d;c[a+28>>2]=d<<3;return}function Hu(a){a=a|0;NM(a|0,0,20);return}function Hv(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0;e=a+16|0;f=c[e>>2]|0;g=f>>>3;h=f&7;c[e>>2]=(c[a+28>>2]|0)-1&f+b;f=a+12|0;c[f>>2]=(c[f>>2]|0)+b;f=a|0;c[f>>2]=(c[f>>2]|0)-b;f=(c[a+24>>2]|0)-1|0;e=c[a+20>>2]|0;a=(d[e+(f&g+1)|0]|0)<<16|(d[e+(f&g)|0]|0)<<24|(d[e+(f&g+2)|0]|0)<<8|(d[e+(f&g+3)|0]|0);if((h|0)==0){i=a;j=32-b|0;k=i>>>(j>>>0);return k|0}i=(d[e+(f&g+4)|0]|0)>>>((8-h|0)>>>0)|a<<h;j=32-b|0;k=i>>>(j>>>0);return k|0}function Hw(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0;g=b+16|0;h=c[g>>2]|0;i=h>>>3;j=h&7;c[g>>2]=(c[b+28>>2]|0)-1&h+f;h=b+12|0;c[h>>2]=(c[h>>2]|0)+f;h=b|0;c[h>>2]=(c[h>>2]|0)+f;h=(c[b+24>>2]|0)-1|0;g=32-f|0;k=e<<g>>>(j>>>0);l=(c[1991416+(g<<2)>>2]|0)>>>(j>>>0)|~c[1991416+(32-j<<2)>>2];g=b+20|0;b=(c[g>>2]|0)+(h&i)|0;a[b]=((d[b]|0)&l>>>24|k>>>24)&255;b=(c[g>>2]|0)+(h&i+1)|0;a[b]=((d[b]|0)&l>>>16|k>>>16)&255;b=(c[g>>2]|0)+(h&i+2)|0;a[b]=((d[b]|0)&l>>>8|k>>>8)&255;b=(c[g>>2]|0)+(h&i+3)|0;a[b]=((d[b]|0)&l|k)&255;if(!((j|0)!=0&f>>>0>24)){return}k=40-f|0;f=(c[g>>2]|0)+(h&i+4)|0;a[f]=((d[f]|0)&c[1991416+(k-j<<2)>>2]|e<<k>>>(j>>>0))&255;return}function Hx(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0;g=b+16|0;h=c[g>>2]|0;i=h&7^7;j=c[b+24>>2]|0;k=c[1991416+(f<<2)>>2]<<i;c[g>>2]=(c[b+28>>2]|0)-1&h-f;g=b+12|0;c[g>>2]=(c[g>>2]|0)-f;g=b|0;c[g>>2]=(c[g>>2]|0)-f;g=0;l=0;do{m=31-(g<<1)|0;l=(1<<g&e)<<m|l|(-2147483648>>>(g>>>0)&e)>>>(m>>>0);g=g+1|0;}while((g|0)<16);g=h>>>3;h=j-1|0;j=~k;k=32-f|0;e=l>>>(k>>>0)<<i;m=b+20|0;b=(c[m>>2]|0)+(h&g)|0;a[b]=((d[b]|0)&j|e)&255;b=(c[m>>2]|0)+(g-1&h)|0;a[b]=((d[b]|0)&j>>>8|e>>>8)&255;b=(c[m>>2]|0)+(g-2&h)|0;a[b]=((d[b]|0)&j>>>16|e>>>16)&255;b=(c[m>>2]|0)+(g-3&h)|0;a[b]=((d[b]|0)&j>>>24|e>>>24)&255;if((i+f|0)>>>0<=32){return}e=(c[m>>2]|0)+(g-4&h)|0;a[e]=(((c[1991416+(i<<2)>>2]|0)>>>(k>>>0)^255)&(d[e]|0)|l>>>((64-f-i|0)>>>0))&255;return}function Hy(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=a+12|0;c[e>>2]=(c[e>>2]|0)-b;e=a|0;c[e>>2]=(c[e>>2]|0)+(d<<24>>24==0?b:-b|0);d=a+16|0;c[d>>2]=(c[a+28>>2]|0)-1&(c[d>>2]|0)-b;return}function Hz(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=a+12|0;c[e>>2]=(c[e>>2]|0)+b;e=a|0;c[e>>2]=(c[e>>2]|0)-(d<<24>>24==0?b:-b|0);d=a+16|0;c[d>>2]=(c[a+28>>2]|0)-1&(c[d>>2]|0)+b;return}function HA(a){a=a|0;return c[a>>2]|0}function HB(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=a|0;f=(c[e>>2]|0)>>>3;g=c[d>>2]|0;h=f>>>0<g>>>0?f:g;if((h|0)==0){i=0;c[d>>2]=i;return}g=a+24|0;f=a+8|0;j=a+20|0;a=b;b=0;k=h;h=c[g>>2]|0;l=c[f>>2]|0;while(1){m=h-l|0;n=m>>>0<k>>>0?m:k;IY(a,(c[j>>2]|0)+l|0,n);c[e>>2]=(c[e>>2]|0)-(n<<3);m=n+b|0;o=c[g>>2]|0;p=o-1&(c[f>>2]|0)+n;c[f>>2]=p;if((k|0)==(n|0)){i=m;break}else{a=a+n|0;b=m;k=k-n|0;h=o;l=p}}c[d>>2]=i;return}function HC(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0;if((a|0)>-1){d=~a;e=0;while(1){f=e+1|0;g=d<<1;if((g|0)<0){d=g;e=f}else{h=f;break}}}else{h=0}e=31-h|0;h=e-1|0;d=a>>e-3;if((d|0)==8){i=e-2|0;j=0}else if((d|0)==12){i=h;j=2}else if((d|0)==15){i=h;j=1}else{i=h;j=0}return c[24+((b&1)*108|0)+(j*36|0)+(((b|0)==1?i-5|0:i)<<2)>>2]|0}function HD(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;L8991:do{switch(a|0){case 39:{if(c<<24>>24==1){e=1921168;break L8991}e=b<<24>>24<1?1921200:1921184;break};case 17:case 23:{d=b<<24>>24==0;if(c<<24>>24==1){e=d?1921232:1921216;break L8991}else{e=d?1921264:1921248;break L8991}break};case 2:case 5:case 29:{e=c<<24>>24==1?1927368:1927384;break};case 256:{e=c<<24>>24==1?1921136:1921152;break};default:{e=0}}}while(0);return e|0}function HE(a){a=a|0;return HF(a,0)|0}function HF(a,b){a=a|0;b=b|0;var c=0,d=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;if((a|0)<1){c=-2147483648;return c|0}d=~a;f=0;while(1){g=d<<1;if((g|0)<0){d=g;f=f+1|0}else{break}}d=-(a<<f^-2147483648)|0;a=d;g=(d|0)<0|0?-1:0;h=0;i=d;d=0;while(1){j=(e[1914792+(d<<1)>>1]|0)<<16;k=i;l=(i|0)<0|0?-1:0;N2(j,(j|0)<0|0?-1:0,k,l)|0;m=G+h|0;N2(k,l,a,g)|0;l=d+1|0;if((l|0)<10){h=m;i=G<<1|0>>>31;d=l}else{break}}d=b-f|0;N2(m,(m|0)<0|0?-1:0,1901360723,0)|0;i=G+m|0;do{if((f|0)==(b|0)){n=i;o=-5}else{m=d>>31^d;if((m|0)>-1){h=~m;m=0;while(1){g=h<<1;if((g|0)<0){h=g;m=m+1|0}else{p=m;break}}}else{p=-1}m=32-p|0;h=(i>>m-1)+(d<<31-m)|0;g=m-6|0;if((g|0)<=0){n=h;o=g;break}c=h<<g;return c|0}}while(0);c=n>>-o;return c|0}function HG(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;if((d|0)>0){e=0}else{return}do{c[b+(e<<2)>>2]=HF(c[a+(e<<2)>>2]|0,0)|0;e=e+1|0;}while((e|0)<(d|0));return}function HH(a){a=a|0;var b=0,c=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;if((a|0)==0){b=2147483647;return b|0}c=a>>25;d=a-(c<<25)<<6;e=(d|0)>1073741824;if((a|0)<0){if(e){f=d^-2147483648;g=c+1|0}else{f=d;g=c}if((f|0)<-1073741824){h=f^-2147483648;i=g-1|0}else{h=f;i=g}g=h;f=(h|0)<0|0?-1:0;h=1488519168;a=0;N2(g,f,h,a)|0;a=G+1073741824|0;N2(g,f,g,f)|0;h=G<<1|0>>>31;j=h;k=(h|0)<0|0?-1:0;h=515899392;l=0;N2(j,k,h,l)|0;l=a+G|0;N2(j,k,g,f)|0;k=G<<1|0>>>31;j=k;a=(k|0)<0|0?-1:0;k=119209984;h=0;N2(j,a,k,h)|0;h=l+G|0;N2(j,a,g,f)|0;a=G<<1|0>>>31;j=a;l=(a|0)<0|0?-1:0;a=20643840;k=0;N2(j,l,a,k)|0;k=h+G|0;N2(j,l,g,f)|0;f=G<<1|0>>>31;g=f;l=(f|0)<0|0?-1:0;f=2883584;j=0;N2(g,l,f,j)|0;j=k+G|0;k=i+1|0;i=(k|0)<-31?-31:k;k=(i|0)>31?31:i;if((k|0)>0){b=j<<k;return b|0}else{b=j>>-k;return b|0}}if(e){m=d^-2147483648;n=c+1|0}else{m=d;n=c}if((m|0)<-1073741824){o=m^-2147483648;p=n-1|0}else{o=m;p=n}n=o;m=(o|0)<0|0?-1:0;N2(n,m,1488519168,0)|0;o=G+1073741824|0;N2(n,m,n,m)|0;c=G<<1|0>>>31;d=c;e=(c|0)<0|0?-1:0;N2(d,e,515899392,0)|0;c=o+G|0;N2(d,e,n,m)|0;e=G<<1|0>>>31;d=e;o=(e|0)<0|0?-1:0;N2(d,o,119209984,0)|0;e=c+G|0;N2(d,o,n,m)|0;o=G<<1|0>>>31;d=o;c=(o|0)<0|0?-1:0;N2(d,c,20643840,0)|0;o=e+G|0;N2(d,c,n,m)|0;m=G<<1|0>>>31;N2(m,(m|0)<0|0?-1:0,2883584,0)|0;m=o+G|0;o=p-29|0;p=(o|0)>-31?o:-31;o=(p|0)<31?p:31;p=(o|0)>0;if(p&(m|0)>(2147483647>>>(o>>>0)|0)){b=2147483647;return b|0}if(p){q=m<<o}else{q=m>>-o}b=q+1>>1;return b|0}function HI(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;if((b|0)>0){e=31-b|0;f=a>>e;g=a-(f<<e)<<b;h=f}else{g=a>>-b;h=0}if((g|0)>1073741824){i=g^-2147483648;j=h+1|0}else{i=g;j=h}if((i|0)<-1073741824){k=i^-2147483648;l=j-1|0}else{k=i;l=j}j=k;i=(k|0)<0|0?-1:0;N2(j,i,1488519168,0)|0;k=G+1073741824|0;N2(j,i,j,i)|0;h=G<<1|0>>>31;g=h;b=(h|0)<0|0?-1:0;N2(g,b,515899392,0)|0;h=G+k|0;N2(g,b,j,i)|0;b=G<<1|0>>>31;g=b;k=(b|0)<0|0?-1:0;N2(g,k,119209984,0)|0;b=G+h|0;N2(g,k,j,i)|0;k=G<<1|0>>>31;g=k;h=(k|0)<0|0?-1:0;N2(g,h,20643840,0)|0;k=G+b|0;N2(g,h,j,i)|0;i=G<<1|0>>>31;N2(i,(i|0)<0|0?-1:0,2883584,0)|0;c[d>>2]=l+1;return G+k|0}function HJ(){return}function HK(a){a=a|0;var b=0;if((a-1|0)>>>0>=192){b=0;return b|0}b=c[1928784+(a<<2)>>2]|0;return b|0}function HL(){return}function HM(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;if((a|0)==0){d=1;e=1;c[b>>2]=e;return d|0}if((a|0)>-1){f=~a;g=0;while(1){h=g+1|0;i=f<<1;if((i|0)<0){f=i;g=h}else{j=h;break}}}else{j=0}g=a<<j-1;a=j+1|0;c[b>>2]=a;j=c[1929560+((g>>>23&127)<<2)>>2]|0;f=j;h=(j|0)<0|0?-1:0;N2(f,h,f,h)|0;N2(G,0,g,(g|0)<0|0?-1:0)|0;g=134217728-G|0;N2(g,(g|0)<0|0?-1:0,f,h)|0;h=(G<<4|0>>>28)+j|0;if((a&1|0)==0){k=h}else{j=h;f=(h|0)<0|0?-1:0;h=1518500224;g=0;N2(j,f,h,g)|0;k=G<<2|0>>>30}d=k;e=a>>1;c[b>>2]=e;return d|0}function HN(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0;if((a|0)==0){b=1;d=0}else{if((a|0)>-1){e=~a;f=0;while(1){g=f+1|0;h=e<<1;if((h|0)<0){e=h;f=g}else{i=g;break}}}else{i=0}f=a<<i-1;e=i+1|0;i=c[1929560+((f>>>23&127)<<2)>>2]|0;g=i;h=(i|0)<0|0?-1:0;N2(g,h,g,h)|0;j=G;k=0;l=f;m=(f|0)<0|0?-1:0;N2(j,k,l,m)|0;m=134217728-G|0;l=m;k=(m|0)<0|0?-1:0;N2(l,k,g,h)|0;h=(G<<4|0>>>28)+i|0;if((e&1|0)==0){n=h}else{i=h;g=(h|0)<0|0?-1:0;h=1518500224;k=0;N2(i,g,h,k)|0;n=G<<2|0>>>30}b=n;d=(e>>1)-1|0}e=a<<d;N2(b,(b|0)<0|0?-1:0,e,(e|0)<0|0?-1:0)|0;return G<<2|0>>>30|0}function HO(a,b,c){a=a|0;b=b|0;c=c|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;d=a>>1;a=b>>1;do{if((d|0)==0){e=0}else{b=c-1|0;if((b|0)==0){e=0;break}else{f=d;g=0;h=b}while(1){b=g<<1;i=f<<1;if((i|0)<(a|0)){j=b;k=i}else{j=b|1;k=i-a|0}i=h-1|0;if((i|0)==0){e=j;break}else{f=k;g=j;h=i}}}}while(0);return e<<32-c|0}function HP(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0;if((a|0)==0|(b|0)==0){e=0;f=0;c[d>>2]=f;return e|0}g=a>>31^a;if((g|0)>-1){h=~g;g=0;while(1){i=h<<1;if((i|0)<0){h=i;g=g+1|0}else{j=g;break}}}else{j=-1}g=a<<j;a=b>>31^b;if((a|0)>-1){h=~a;a=0;while(1){i=h<<1;if((i|0)<0){h=i;a=a+1|0}else{k=a;break}}}else{k=-1}a=b<<k;N2(a,(a|0)<0|0?-1:0,g,(g|0)<0|0?-1:0)|0;e=G<<1|0>>>31;f=-(j+k|0)|0;c[d>>2]=f;return e|0}function HQ(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0;if((a|0)==0){c[d>>2]=0;e=0;return e|0}f=a>>31^a;if((f|0)>-1){g=~f;f=0;while(1){h=g<<1;if((h|0)<0){g=h;f=f+1|0}else{i=f;break}}}else{i=-1}f=a<<i;a=1-i|0;c[d>>2]=a;do{if((b|0)==0){j=0}else{i=b>>31^b;if((i|0)<=-1){j=-1;break}g=~i;i=0;while(1){h=g<<1;if((h|0)<0){g=h;i=i+1|0}else{j=i;break}}}}while(0);c[d>>2]=a+j;a=f>>2;f=b<<j>>1;do{if((a|0)==0){k=0}else{j=a<<1;b=(j|0)<(f|0);d=b?0:2;i=j-(b?0:f)<<1;if((i|0)<(f|0)){l=d;m=i}else{l=d|1;m=i-f|0}i=l<<1;d=m<<1;if((d|0)<(f|0)){n=i;o=d}else{n=i|1;o=d-f|0}d=n<<1;i=o<<1;if((i|0)<(f|0)){p=d;q=i}else{p=d|1;q=i-f|0}i=p<<1;d=q<<1;if((d|0)<(f|0)){r=i;s=d}else{r=i|1;s=d-f|0}d=r<<1;i=s<<1;if((i|0)<(f|0)){t=d;u=i}else{t=d|1;u=i-f|0}i=t<<1;d=u<<1;if((d|0)<(f|0)){v=i;w=d}else{v=i|1;w=d-f|0}d=v<<1;i=w<<1;if((i|0)<(f|0)){x=d;y=i}else{x=d|1;y=i-f|0}i=x<<1;d=y<<1;if((d|0)<(f|0)){z=i;A=d}else{z=i|1;A=d-f|0}d=z<<1;i=A<<1;if((i|0)<(f|0)){B=d;C=i}else{B=d|1;C=i-f|0}i=B<<1;d=C<<1;if((d|0)<(f|0)){D=i;E=d}else{D=i|1;E=d-f|0}d=D<<1;i=E<<1;if((i|0)<(f|0)){F=d;G=i}else{F=d|1;G=i-f|0}i=F<<1;d=G<<1;if((d|0)<(f|0)){H=i;I=d}else{H=i|1;I=d-f|0}d=H<<1;i=I<<1;if((i|0)<(f|0)){J=d;K=i}else{J=d|1;K=i-f|0}i=J<<1;if((K<<1|0)<(f|0)){k=i;break}k=i|1}}while(0);e=k<<16;return e|0}function HR(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=i;i=i+8|0;e=d|0;f=HQ(a,b,e)|0;b=c[e>>2]|0;do{if((f|0)==1073741824&(b|0)==1){g=2147483647}else{if((b|0)>0){g=f<<b;break}else{g=f>>-b;break}}}while(0);i=d;return g|0}function HS(a,b,d){a=a|0;b=b|0;d=d|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;if((a|0)<1){f=-2147483648;g=31;c[d>>2]=g;return f|0}h=~a;i=0;while(1){j=h<<1;if((j|0)<0){h=j;i=i+1|0}else{break}}h=-(a<<i^-2147483648)|0;a=h;j=(h|0)<0|0?-1:0;k=0;l=h;h=0;while(1){m=(e[1914792+(h<<1)>>1]|0)<<16;n=l;o=(l|0)<0|0?-1:0;N2(m,(m|0)<0|0?-1:0,n,o)|0;p=G+k|0;N2(n,o,a,j)|0;o=h+1|0;if((o|0)<10){k=p;l=G<<1|0>>>31;h=o}else{break}}h=b-i|0;N2(p,(p|0)<0|0?-1:0,1901360723,0)|0;l=G+p|0;if((i|0)==(b|0)){f=l;g=1;c[d>>2]=g;return f|0}b=h>>31^h;if((b|0)>-1){i=~b;b=0;while(1){p=i<<1;if((p|0)<0){i=p;b=b+1|0}else{q=b;break}}}else{q=-1}b=32-q|0;f=(l>>b-1)+(h<<31-b)|0;g=b;c[d>>2]=g;return f|0}function HT(a,b,c,d,f){a=a|0;b=b|0;c=c|0;d=d|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;do{if((a|0)<1){g=-2147483648;h=31}else{i=~a;j=0;while(1){k=i<<1;if((k|0)<0){i=k;j=j+1|0}else{break}}i=-(a<<j^-2147483648)|0;k=i;l=(i|0)<0|0?-1:0;m=0;n=i;i=0;while(1){o=(e[1914792+(i<<1)>>1]|0)<<16;p=n;q=(n|0)<0|0?-1:0;N2(o,(o|0)<0|0?-1:0,p,q)|0;r=G+m|0;N2(p,q,k,l)|0;q=i+1|0;if((q|0)<10){m=r;n=G<<1|0>>>31;i=q}else{break}}i=b-j|0;N2(r,(r|0)<0|0?-1:0,1901360723,0)|0;n=G+r|0;if((j|0)==(b|0)){g=n;h=1;break}m=i>>31^i;if((m|0)>-1){l=~m;m=0;while(1){k=l<<1;if((k|0)<0){l=k;m=m+1|0}else{s=m;break}}}else{s=-1}m=32-s|0;g=(n>>m-1)+(i<<31-m)|0;h=m}}while(0);s=(c|0)>0?c:-c|0;do{if((s|0)==0){t=0}else{b=s>>31^s;if((b|0)<=-1){t=-1;break}r=~b;b=0;while(1){a=r<<1;if((a|0)<0){r=a;b=b+1|0}else{t=b;break}}}}while(0);s=c<<t;N2(s,(s|0)<0|0?-1:0,g,(g|0)<0|0?-1:0)|0;return HI(G<<1|0>>>31,h+d-t|0,f)|0}function HU(a){a=a|0;var b=0,d=0;b=IQ(1,248)|0;if((b|0)==0){d=2;return d|0}c[a>>2]=b;d=0;return d|0}function HV(b,d,e,f,g,h){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0;i=b|0;IY(b,g,56);c[b+56>>2]=f;c[b+104>>2]=d;c[b+108>>2]=e;g=b+60|0;Ht(b+68|0,d,e,0);c[b+64>>2]=0;c[g>>2]=0;c[b+100>>2]=1;L9242:do{switch(f|0){case 0:{c[b+116>>2]=0;c[b+120>>2]=c[b+36>>2];break};case 1:{if((c[b>>2]|0)!=2){j=2;return j|0}if((c[b+24>>2]|0)==1024){c[b+140>>2]=0;c[b+120>>2]=c[b+12>>2];c[b+124>>2]=c[b+20>>2];c[b+128>>2]=1;c[b+116>>2]=c[b+8>>2];c[b+132>>2]=0;c[b+136>>2]=0;break L9242}else{j=2;return j|0}break};case 10:case 7:case 6:{e=LH(b+116|0,g,i,h&2,f,b+224|0)|0;if((e|0)==0){break L9242}else{j=e}return j|0};case 2:{if((c[b>>2]|0)!=2){j=2;return j|0}if((c[b+24>>2]|0)!=1024){j=2;return j|0}if((Lu(b+116|0,i)|0)==0){break L9242}else{j=2}return j|0};default:{j=2;return j|0}}}while(0);i=Lx(c[b+8>>2]|0)|0;h=a[b+53|0]|0;g=h&255;e=a[b+52|0]|0;L9263:do{if(h<<24>>24==0){k=-1}else{if((i|0)==0){if((f|0)==2|(f|0)==7|(f|0)==0){k=g;break}k=-1;break}else if(!((i|0)==5|(i|0)==6)){k=-1;break}if(e<<24>>24==0){k=-1;break}switch(f|0){case 1:case 2:case 10:case 6:case 7:case 0:{k=g;break L9263;break};default:{}}k=-1}}while(0);c[b+112>>2]=k;j=0;return j|0}function HW(a){a=a|0;return a+60|0}function HX(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;if((a|0)==0){e=-1;return e|0}c[a+240>>2]=b;c[a+244>>2]=d;e=0;return e|0}function HY(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;if((b|0)==0){h=2;return h|0}i=b+60|0;j=b+112|0;k=b+53|0;if((c[j>>2]|0)<(d[k]|0)){l=e}else{l=(Lz(c[b+8>>2]|0,d[b+52|0]|0,3)|0)+e|0}e=b+56|0;switch(c[e>>2]|0){case 1:{Ht(b+68|0,c[b+104>>2]|0,c[b+108>>2]|0,0);c[b+64>>2]=0;c[i>>2]=0;c[b+100>>2]=1;Lp(b+116|0,i,f)|0;m=0;break};case 0:{n=b+116|0;do{if((c[n>>2]|0)<(c[b+120>>2]|0)){o=b+68|0;if((c[b+100>>2]|0)!=0){p=o;q=6663;break}Hy(o,c[b+64>>2]|0,0);r=i|0;s=o}else{c[n>>2]=0;o=b+68|0;Ht(o,c[b+104>>2]|0,c[b+108>>2]|0,0);c[b+64>>2]=0;c[i>>2]=0;c[b+100>>2]=1;p=o;q=6663}}while(0);if((q|0)==6663){q=i|0;Hw(p,c[q>>2]|0,c[b+64>>2]|0);r=q;s=p}c[b+64>>2]=0;c[r>>2]=0;c[b+124>>2]=HA(s)|0;m=0;break};case 2:{s=((f|0)/(g|0)|0|0)/32|0;Lv(b+116|0,i,(s|0)>2047?2047:s,l)|0;m=0;break};case 10:case 7:case 6:{s=((f|0)/(g|0)|0|0)/32|0;LD(b+116|0,i,l,(s|0)>255?255:s,b+224|0)|0;m=0;break};default:{m=4}}s=c[j>>2]|0;do{if((s|0)<(d[k]|0)){if((s|0)==-1){h=m}else{t=s;break}return h|0}else{l=b+68|0;if((c[b+100>>2]|0)==0){Hy(l,c[b+64>>2]|0,0);u=i|0}else{g=i|0;Hw(l,c[g>>2]|0,c[b+64>>2]|0);u=g}g=b+64|0;c[g>>2]=0;c[u>>2]=0;f=HA(l)|0;r=c[497857]|0;p=c[g>>2]|0;q=p+3|0;if(q>>>0<32){c[g>>2]=q;q=i|0;c[q>>2]=c[q>>2]<<3|r&5}else{q=i|0;Hw(l,c[q>>2]|0,p);c[g>>2]=3;c[q>>2]=r&5}do{if((c[e>>2]|0)==2){if((a[b+127|0]|0)!=0){v=0;break}v=Lr(b+116|0,i,0)|0}else{v=0}}while(0);r=c[b+8>>2]|0;q=c[b+12>>2]|0;g=d[b+52|0]|0;p=c[b+48>>2]&134217728;Ly(i,r,q,0,1,g,p,f)|0;do{if((c[e>>2]|0)==2){if((a[b+127|0]|0)!=0){break}Ls(b+116|0,i,v)}}while(0);c[j>>2]=0;t=0}}while(0);c[j>>2]=t+1;h=m;return h|0}function HZ(a,b){a=a|0;b=b|0;var d=0;switch(c[a+56>>2]|0){case 1:{d=Lq(a+116|0)|0;c[b>>2]=(c[b>>2]|0)-d;c[a+140>>2]=1;return 0};case 7:case 6:case 10:{LF(a+116|0,b);return 0};case 0:{c[b>>2]=(c[b>>2]|0)-(c[a+124>>2]|0);return 0};case 2:{Lw(a+116|0,a+60|0,b);return 0};default:{return 0}}return 0}function H_(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;e=a+60|0;switch(c[a+56>>2]|0){case 7:case 6:case 10:{c[b>>2]=c[a+108>>2];LG(a+116|0,e,b);return 0};case 0:{f=a+100|0;g=a+68|0;if((c[f>>2]|0)==0){Hy(g,c[a+64>>2]|0,0);h=e|0}else{i=e|0;Hw(g,c[i>>2]|0,c[a+64>>2]|0);h=i}i=a+64|0;c[i>>2]=0;c[h>>2]=0;h=a+116|0;c[h>>2]=(c[h>>2]|0)+1;if((c[f>>2]|0)==0){Hy(g,c[i>>2]|0,0);j=e|0}else{f=e|0;Hw(g,c[f>>2]|0,c[i>>2]|0);j=f}c[i>>2]=0;c[j>>2]=0;c[b>>2]=((HA(g)|0)+7-(c[a+124>>2]|0)|0)>>>3;return 0};case 1:{g=a+68|0;if((c[a+100>>2]|0)==0){Hy(g,c[a+64>>2]|0,0);k=e|0}else{j=e|0;Hw(g,c[j>>2]|0,c[a+64>>2]|0);k=j}c[a+64>>2]=0;c[k>>2]=0;c[b>>2]=((HA(g)|0)+7|0)>>>3;return 0};case 2:{g=a+144|0;if((c[g>>2]|0)<((d[a+138|0]|0)+1|0)){c[b>>2]=0;return 0}k=a+68|0;if((c[a+100>>2]|0)==0){Hy(k,c[a+64>>2]|0,0);l=e|0}else{j=e|0;Hw(k,c[j>>2]|0,c[a+64>>2]|0);l=j}c[a+64>>2]=0;c[l>>2]=0;c[b>>2]=((HA(k)|0)+7|0)>>>3;c[g>>2]=0;return 0};default:{return 0}}return 0}function H$(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,i=0;if((c[a+112>>2]|0)<(d[a+53|0]|0|0)){e=0;f=b}else{g=Lz(c[a+8>>2]|0,d[a+52|0]|0,3)|0;e=g;f=g+b|0}b=c[a+56>>2]|0;if((b|0)==2){h=Lt(a+116|0)|0;i=h+e|0;return i|0}else if((b|0)==6|(b|0)==7|(b|0)==10){h=LC(a+116|0,f)|0;i=h+e|0;return i|0}else{h=0;i=h+e|0;return i|0}return 0}function H0(a){a=a|0;var b=0;if((a|0)==0){return}b=c[a>>2]|0;if((b|0)==0){return}IR(b);c[a>>2]=0;return}function H1(a,b){a=a|0;b=b|0;var d=0;if((c[a+56>>2]|0)!=2){d=0;return d|0}d=Lr(a+116|0,a+60|0,b)|0;return d|0}function H2(a,b){a=a|0;b=b|0;if((c[a+56>>2]|0)!=2){return}Ls(a+116|0,a+60|0,b);return}function H3(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0;c[e>>2]=0;f=c[a+56>>2]|0;if((f|0)==7|(f|0)==6|(f|0)==10){f=LB(a+116|0,d,0,a+224|0)|0;c[e>>2]=1;return f|0}else{return(LA(d,b,a+224|0)|0)!=0|0}return 0}function H4(a){a=a|0;var b=0,d=0,e=0,f=0;b=i;if((a|0)==0){d=2;i=b;return d|0}else{e=0}while(1){if((e|0)>=32){break}if((c[a+(e*56|0)+12>>2]|0)==0){break}else{e=e+1|0}}if((e|0)==32){d=1;i=b;return d|0}c[a+(e*56|0)+12>>2]=8;c[a+(e*56|0)+16>>2]=33685760;IP(a+(e*56|0)+24|0,1721600,(f=i,i=i+24|0,c[f>>2]=2,c[f+8>>2]=2,c[f+16>>2]=1,f)|0)|0;i=f;c[a+(e*56|0)+4>>2]=1736120;c[a+(e*56|0)+8>>2]=1701864;c[a+(e*56|0)>>2]=1678592;c[a+(e*56|0)+20>>2]=31;d=0;i=b;return d|0}function H5(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;h=f|0;c[h>>2]=0;c[f+4>>2]=0;if((e|0)==0){i=0;return i|0}j=f+12|0;do{if((c[d+12>>2]|0)==0){k=f+16|0;l=c[k>>2]|0;m=l+1|0;if(m>>>0<32){c[k>>2]=m;m=j|0;c[m>>2]=c[m>>2]<<1;n=1;break}else{m=j|0;Hw(f+20|0,c[m>>2]|0,l);c[k>>2]=1;c[m>>2]=0;n=1;break}}else{m=c[497855]|0;k=f+16|0;l=c[k>>2]|0;o=l+1|0;if(o>>>0<32){c[k>>2]=o;o=j|0;c[o>>2]=c[o>>2]<<1|m&1}else{o=j|0;Hw(f+20|0,c[o>>2]|0,l);c[k>>2]=1;c[o>>2]=m&1}n=(Ia(a,j)|0)+1|0}}while(0);c[h>>2]=n;i=(H6(e,0,b,f,1,0,g)|0)+n|0;return i|0}function H6(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0;if((h|0)==2){k=g+12|0;l=g+16|0;m=c[l>>2]|0;n=m+1|0;do{if(n>>>0<32){c[l>>2]=n;o=k|0;p=c[o>>2]<<1;c[o>>2]=p;q=c[497855]|0;r=m+2|0;if(r>>>0<32){s=p;t=q;u=r;v=6817;break}Hw(g+20|0,p,n);c[l>>2]=1;c[o>>2]=q&i;w=q}else{q=k|0;Hw(g+20|0,c[q>>2]|0,m);c[l>>2]=1;c[q>>2]=0;s=0;t=c[497855]|0;u=2;v=6817}}while(0);if((v|0)==6817){c[l>>2]=u;c[k>>2]=s<<1|t&i;w=t}t=(a[b+1606|0]|0)!=0;if((i|0)==0){do{if(t){v=6874}else{if((a[e+1606|0]|0)!=0){v=6874;break}i=(Ib(b,k)|0)+2|0;x=i+(Ib(e,k)|0)|0}}while(0);do{if((v|0)==6874){if((c[(c[b+1120>>2]|0)+8>>2]|0)==4){y=Ic(b,k)|0}else{y=Ib(b,k)|0}i=y+2|0;if((c[(c[e+1120>>2]|0)+8>>2]|0)==4){x=(Ic(e,k)|0)+i|0;break}else{x=(Ib(e,k)|0)+i|0;break}}}while(0);y=b+1196|0;v=c[y>>2]|0;i=(v|0)>1?2:1;if((v|0)>0){v=k|0;s=g+20|0;u=0;m=c[l>>2]|0;while(1){n=c[b+1220+(u<<2)>>2]|0;q=m+1|0;if(q>>>0<32){c[l>>2]=q;z=q;A=c[v>>2]<<1|n&w}else{Hw(s,c[v>>2]|0,m);c[l>>2]=1;z=1;A=n&w}c[v>>2]=A;n=u+1|0;if((n|0)<(c[y>>2]|0)){u=n;m=z}else{B=n;C=z;D=v;E=s;break}}}else{B=0;C=c[l>>2]|0;D=k|0;E=g+20|0}s=0;v=C;while(1){C=c[b+1240+(s<<2)>>2]|0;z=v+1|0;if(z>>>0<32){c[l>>2]=z;F=z;G=c[D>>2]<<1|C&w}else{Hw(E,c[D>>2]|0,v);c[l>>2]=1;F=1;G=C&w}c[D>>2]=G;C=s+1|0;if((C|0)<(i|0)){s=C;v=F}else{break}}v=e+1196|0;s=c[v>>2]|0;C=(s|0)>1?2:1;if((s|0)>0){s=0;z=F;m=G;while(1){u=c[e+1220+(s<<2)>>2]|0;y=z+1|0;if(y>>>0<32){c[l>>2]=y;H=y;I=u&w|m<<1}else{Hw(E,m,z);c[l>>2]=1;H=1;I=u&w}c[D>>2]=I;u=s+1|0;if((u|0)<(c[v>>2]|0)){s=u;z=H;m=I}else{J=u;K=H;L=I;break}}}else{J=0;K=F;L=G}G=0;F=K;K=L;while(1){L=c[e+1240+(G<<2)>>2]|0;I=F+1|0;if(I>>>0<32){c[l>>2]=I;M=I;N=L&w|K<<1}else{Hw(E,K,F);c[l>>2]=1;M=1;N=L&w}c[D>>2]=N;L=G+1|0;if((L|0)<(C|0)){G=L;F=M;K=N}else{break}}K=i+x+B+C+J|0;J=b+1308|0;if((c[J>>2]|0)>0){C=c[497856]|0;B=K;x=0;i=M;F=N;while(1){G=c[b+12+(x<<2)>>2]|0;L=i+2|0;if(L>>>0<32){c[l>>2]=L;O=L;P=G&C|F<<2}else{Hw(E,F,i);c[l>>2]=2;O=2;P=G&C}c[D>>2]=P;G=B+2|0;L=x+1|0;if((L|0)<(c[J>>2]|0)){B=G;x=L;i=O;F=P}else{Q=G;R=O;S=P;break}}}else{Q=K;R=M;S=N}N=e+1308|0;if((c[N>>2]|0)>0){M=c[497856]|0;K=Q;P=0;O=R;R=S;while(1){S=c[e+12+(P<<2)>>2]|0;F=O+2|0;if(F>>>0<32){c[l>>2]=F;T=F;U=S&M|R<<2}else{Hw(E,R,O);c[l>>2]=2;T=2;U=S&M}c[D>>2]=U;S=K+2|0;F=P+1|0;if((F|0)<(c[N>>2]|0)){K=S;P=F;O=T;R=U}else{V=S;break}}}else{V=Q}Q=(Id(b,k,0)|0)+V|0;V=Q+(Id(e,k,0)|0)|0;Q=V+(Ie(b,k,0)|0)|0;V=Q+(Ie(e,k,0)|0)|0;Q=b+1128|0;U=c[Q>>2]|0;R=c[l>>2]|0;T=R+1|0;if(T>>>0<32){c[l>>2]=T;W=T;X=c[D>>2]<<1|U&w}else{Hw(E,c[D>>2]|0,R);c[l>>2]=1;W=1;X=U&w}c[D>>2]=X;do{if((c[Q>>2]|0)==0){Y=1;Z=W;_=X}else{U=b+1124|0;if((c[U>>2]|0)>0){$=0;aa=1;ab=W;ac=X}else{Y=1;Z=W;_=X;break}while(1){R=d[b+1132+$|0]|0;T=ab+1|0;if(T>>>0<32){c[l>>2]=T;ad=T;ae=R&w|ac<<1}else{Hw(E,ac,ab);c[l>>2]=1;ad=1;ae=R&w}c[D>>2]=ae;R=aa+1|0;T=$+1|0;if((T|0)<(c[U>>2]|0)){$=T;aa=R;ab=ad;ac=ae}else{Y=R;Z=ad;_=ae;break}}}}while(0);ae=V+Y|0;Y=e+1128|0;V=c[Y>>2]|0;ad=Z+1|0;if(ad>>>0<32){c[l>>2]=ad;af=ad;ag=V&w|_<<1}else{Hw(E,_,Z);c[l>>2]=1;af=1;ag=V&w}c[D>>2]=ag;do{if((c[Y>>2]|0)==0){ah=1}else{V=e+1124|0;if((c[V>>2]|0)>0){ai=0;aj=1;ak=af;al=ag}else{ah=1;break}while(1){Z=d[e+1132+ai|0]|0;_=ak+1|0;if(_>>>0<32){c[l>>2]=_;am=_;an=Z&w|al<<1}else{Hw(E,al,ak);c[l>>2]=1;am=1;an=Z&w}c[D>>2]=an;Z=aj+1|0;_=ai+1|0;if((_|0)<(c[V>>2]|0)){ai=_;aj=Z;ak=am;al=an}else{ah=Z;break}}}}while(0);ao=ae+ah|0}else{do{if(t){if((c[(c[b+1120>>2]|0)+8>>2]|0)==4){ap=Ic(b,k)|0;break}else{ap=Ib(b,k)|0;break}}else{ap=Ib(b,k)|0}}while(0);t=b+1196|0;ah=c[t>>2]|0;ae=(ah|0)>1?2:1;if((ah|0)>0){ah=k|0;an=g+20|0;al=0;am=c[l>>2]|0;while(1){ak=c[b+1220+(al<<2)>>2]|0;aj=am+1|0;if(aj>>>0<32){c[l>>2]=aj;aq=aj;ar=c[ah>>2]<<1|ak&w}else{Hw(an,c[ah>>2]|0,am);c[l>>2]=1;aq=1;ar=ak&w}c[ah>>2]=ar;ak=al+1|0;if((ak|0)<(c[t>>2]|0)){al=ak;am=aq}else{as=ak;at=aq;au=ah;av=an;break}}}else{as=0;at=c[l>>2]|0;au=k|0;av=g+20|0}an=0;ah=at;while(1){at=c[b+1240+(an<<2)>>2]|0;aq=ah+1|0;if(aq>>>0<32){c[l>>2]=aq;aw=aq;ax=c[au>>2]<<1|at&w}else{Hw(av,c[au>>2]|0,ah);c[l>>2]=1;aw=1;ax=at&w}c[au>>2]=ax;at=an+1|0;if((at|0)<(ae|0)){an=at;ah=aw}else{break}}ah=e+1196|0;an=c[ah>>2]|0;at=(an|0)>1?2:1;if((an|0)>0){an=0;aq=aw;am=ax;while(1){al=c[e+1220+(an<<2)>>2]|0;t=aq+1|0;if(t>>>0<32){c[l>>2]=t;ay=t;az=al&w|am<<1}else{Hw(av,am,aq);c[l>>2]=1;ay=1;az=al&w}c[au>>2]=az;al=an+1|0;if((al|0)<(c[ah>>2]|0)){an=al;aq=ay;am=az}else{aA=al;aB=ay;aC=az;break}}}else{aA=0;aB=aw;aC=ax}ax=0;aw=aB;aB=aC;while(1){aC=c[e+1240+(ax<<2)>>2]|0;az=aw+1|0;if(az>>>0<32){c[l>>2]=az;aD=az;aE=aC&w|aB<<1}else{Hw(av,aB,aw);c[l>>2]=1;aD=1;aE=aC&w}c[au>>2]=aE;aC=ax+1|0;if((aC|0)<(at|0)){ax=aC;aw=aD;aB=aE}else{break}}aB=ap+2+ae+as+at+aA|0;aA=b+1308|0;if((c[aA>>2]|0)>0){at=c[497856]|0;as=aB;ae=0;ap=aD;aD=aE;while(1){aE=c[b+12+(ae<<2)>>2]|0;aw=ap+2|0;if(aw>>>0<32){c[l>>2]=aw;aF=aw;aG=aE&at|aD<<2}else{Hw(av,aD,ap);c[l>>2]=2;aF=2;aG=aE&at}c[au>>2]=aG;aE=as+2|0;aw=ae+1|0;if((aw|0)<(c[aA>>2]|0)){as=aE;ae=aw;ap=aF;aD=aG}else{aH=aE;break}}}else{aH=aB}aB=(Id(b,k,1)|0)+aH|0;aH=aB+(Ie(b,k,1)|0)|0;aB=aH+(Id(e,k,1)|0)|0;aH=aB+(Ie(e,k,1)|0)|0;aB=b+1128|0;aG=c[aB>>2]|0;aD=c[l>>2]|0;aF=aD+1|0;if(aF>>>0<32){c[l>>2]=aF;aI=aF;aJ=c[au>>2]<<1|aG&w}else{Hw(av,c[au>>2]|0,aD);c[l>>2]=1;aI=1;aJ=aG&w}c[au>>2]=aJ;do{if((c[aB>>2]|0)==0){aK=1;aL=aI;aM=aJ}else{aG=b+1124|0;if((c[aG>>2]|0)>0){aN=0;aO=1;aP=aI;aQ=aJ}else{aK=1;aL=aI;aM=aJ;break}while(1){aD=d[b+1132+aN|0]|0;aF=aP+1|0;if(aF>>>0<32){c[l>>2]=aF;aR=aF;aS=aD&w|aQ<<1}else{Hw(av,aQ,aP);c[l>>2]=1;aR=1;aS=aD&w}c[au>>2]=aS;aD=aO+1|0;aF=aN+1|0;if((aF|0)<(c[aG>>2]|0)){aN=aF;aO=aD;aP=aR;aQ=aS}else{aK=aD;aL=aR;aM=aS;break}}}}while(0);aS=aH+aK|0;aK=e+1128|0;aH=c[aK>>2]|0;aR=aL+1|0;if(aR>>>0<32){c[l>>2]=aR;aT=aR;aU=aH&w|aM<<1}else{Hw(av,aM,aL);c[l>>2]=1;aT=1;aU=aH&w}c[au>>2]=aU;do{if((c[aK>>2]|0)==0){aV=1}else{aH=e+1124|0;if((c[aH>>2]|0)>0){aW=0;aX=1;aY=aT;aZ=aU}else{aV=1;break}while(1){aL=d[e+1132+aW|0]|0;aM=aY+1|0;if(aM>>>0<32){c[l>>2]=aM;a_=aM;a$=aL&w|aZ<<1}else{Hw(av,aZ,aY);c[l>>2]=1;a_=1;a$=aL&w}c[au>>2]=a$;aL=aX+1|0;aM=aW+1|0;if((aM|0)<(c[aH>>2]|0)){aW=aM;aX=aL;aY=a_;aZ=a$}else{aV=aL;break}}}}while(0);ao=aS+aV|0}a0=(If(f,k)|0)+ao|0;a1=g+4|0;c[a1>>2]=a0;return a0|0}else if((h|0)==1){h=g+12|0;ao=g+16|0;k=c[ao>>2]|0;aV=k+1|0;if(aV>>>0<32){c[ao>>2]=aV;aS=h|0;a$=c[aS>>2]<<1;c[aS>>2]=a$;a2=aV;a3=a$}else{a$=h|0;Hw(g+20|0,c[a$>>2]|0,k);c[ao>>2]=1;c[a$>>2]=0;a2=1;a3=0}do{if((a[b+1606|0]|0)==0){do{if((j&2|0)==0){a4=1}else{a$=c[497855]|0;k=a2+1|0;if(k>>>0<32){c[ao>>2]=k;c[h>>2]=a$&1|a3<<1;a4=2;break}else{Hw(g+20|0,a3,a2);c[ao>>2]=1;c[h>>2]=a$&1;a4=2;break}}}while(0);a5=(Ib(b,h)|0)+a4|0}else{if((c[(c[b+1120>>2]|0)+8>>2]|0)==4){a5=(Ic(b,h)|0)+1|0;break}else{a5=(Ib(b,h)|0)+1|0;break}}}while(0);a4=b+1196|0;a2=c[a4>>2]|0;a3=(a2|0)>1?2:1;j=c[497855]|0;if((a2|0)>0){a2=h|0;a$=g+20|0;k=0;aV=c[ao>>2]|0;while(1){aS=c[b+1220+(k<<2)>>2]|0;aZ=aV+1|0;if(aZ>>>0<32){c[ao>>2]=aZ;a6=aZ;a7=c[a2>>2]<<1|aS&j}else{Hw(a$,c[a2>>2]|0,aV);c[ao>>2]=1;a6=1;a7=aS&j}c[a2>>2]=a7;aS=k+1|0;if((aS|0)<(c[a4>>2]|0)){k=aS;aV=a6}else{a8=aS;a9=a6;ba=a2;bb=a$;break}}}else{a8=0;a9=c[ao>>2]|0;ba=h|0;bb=g+20|0}a$=0;a2=a9;while(1){a9=c[b+1240+(a$<<2)>>2]|0;a6=a2+1|0;if(a6>>>0<32){c[ao>>2]=a6;bc=a6;bd=c[ba>>2]<<1|a9&j}else{Hw(bb,c[ba>>2]|0,a2);c[ao>>2]=1;bc=1;bd=a9&j}c[ba>>2]=bd;a9=a$+1|0;if((a9|0)<(a3|0)){a$=a9;a2=bc}else{break}}a2=a3+a5+a8|0;a8=b+1308|0;if((c[a8>>2]|0)>0){a5=c[497856]|0;a3=0;a$=a2;a9=bc;bc=bd;while(1){bd=c[b+12+(a3<<2)>>2]|0;a6=a9+2|0;if(a6>>>0<32){c[ao>>2]=a6;be=a6;bf=bd&a5|bc<<2}else{Hw(bb,bc,a9);c[ao>>2]=2;be=2;bf=bd&a5}c[ba>>2]=bf;bd=a$+2|0;a6=a3+1|0;if((a6|0)<(c[a8>>2]|0)){a3=a6;a$=bd;a9=be;bc=bf}else{bg=bd;break}}}else{bg=a2}a2=(Id(b,h,0)|0)+bg|0;bg=a2+(Ie(b,h,0)|0)|0;a2=b+1128|0;bf=c[a2>>2]|0;bc=c[ao>>2]|0;be=bc+1|0;if(be>>>0<32){c[ao>>2]=be;bh=be;bi=c[ba>>2]<<1|bf&j}else{Hw(bb,c[ba>>2]|0,bc);c[ao>>2]=1;bh=1;bi=bf&j}c[ba>>2]=bi;do{if((c[a2>>2]|0)==0){bj=1}else{bf=b+1124|0;if((c[bf>>2]|0)>0){bk=0;bl=1;bm=bh;bn=bi}else{bj=1;break}while(1){bc=d[b+1132+bk|0]|0;be=bm+1|0;if(be>>>0<32){c[ao>>2]=be;bo=be;bp=bc&j|bn<<1}else{Hw(bb,bn,bm);c[ao>>2]=1;bo=1;bp=bc&j}c[ba>>2]=bp;bc=bl+1|0;be=bk+1|0;if((be|0)<(c[bf>>2]|0)){bk=be;bl=bc;bm=bo;bn=bp}else{bj=bc;break}}}}while(0);a0=bg+bj+(If(f,h)|0)|0;a1=g+4|0;c[a1>>2]=a0;return a0|0}else{a0=0;a1=g+4|0;c[a1>>2]=a0;return a0|0}return 0}function H7(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;i=g|0;c[i>>2]=0;c[g+4>>2]=0;if((e|0)==0|(f|0)==0){j=0;return j|0}k=g+12|0;do{if((c[d+12>>2]|0)==0){l=g+16|0;m=c[l>>2]|0;n=m+1|0;if(n>>>0<32){c[l>>2]=n;n=k|0;c[n>>2]=c[n>>2]<<1;o=1;break}else{n=k|0;Hw(g+20|0,c[n>>2]|0,m);c[l>>2]=1;c[n>>2]=0;o=1;break}}else{n=c[497855]|0;l=g+16|0;m=c[l>>2]|0;p=m+1|0;if(p>>>0<32){c[l>>2]=p;p=k|0;c[p>>2]=c[p>>2]<<1|n&1}else{p=k|0;Hw(g+20|0,c[p>>2]|0,m);c[l>>2]=1;c[p>>2]=n&1}o=(Ia(a,k)|0)+1|0}}while(0);c[i>>2]=o;j=(H6(e,f,b,g,2,c[a+64>>2]|0,h)|0)+o|0;return j|0}function H8(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;i=g+12|0;j=g+52|0;k=g+20|0;if((c[j>>2]|0)==0){Hy(k,c[g+16>>2]|0,0);l=i|0}else{m=i|0;Hw(k,c[m>>2]|0,c[g+16>>2]|0);l=m}m=g+16|0;c[m>>2]=0;c[l>>2]=0;l=HA(k)|0;n=H7(a,b,d,e,f,g,h)|0;if((c[j>>2]|0)==0){Hy(k,c[m>>2]|0,0);o=i|0}else{h=i|0;Hw(k,c[h>>2]|0,c[m>>2]|0);o=h}c[m>>2]=0;c[o>>2]=0;o=(HA(k)|0)-l|0;l=c[m>>2]|0;h=l+o|0;g=(c[j>>2]|0)==0;do{if(h>>>0<32){if(!g){p=6963;break}c[m>>2]=h;Hy(k,h,0);c[m>>2]=0;c[i>>2]=0;return n|0}else{if(!g){p=6963;break}Hy(k,l,0);q=i|0}}while(0);if((p|0)==6963){p=i|0;Hw(k,c[p>>2]|0,l);q=p}c[m>>2]=0;c[q>>2]=0;Hy(k,o,c[j>>2]&255);return n|0}function H9(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0;f=a+(d<<2)|0;Ia((c[f>>2]|0)+100|0,b)|0;if((e|0)!=0){return}c[(c[f>>2]|0)+188>>2]=-1;c[(c[f>>2]|0)+184>>2]=0;c[(c[f>>2]|0)+192>>2]=-1;return}function Ia(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0;if((a|0)==0){d=0;return d|0}e=c[a>>2]|0;f=c[497855]|0;g=b+4|0;h=c[g>>2]|0;i=h+1|0;do{if(i>>>0<32){c[g>>2]=i;j=b|0;k=c[j>>2]<<1|f&e;c[j>>2]=k;j=c[a+4>>2]|0;l=c[497858]|0;m=h+5|0;if(m>>>0<32){n=k;o=j;p=l;q=m;r=6978;break}Hw(b+8|0,k,i);c[g>>2]=4;k=l&j;c[b>>2]=k;s=l;t=k;u=c[a+8>>2]|0;v=8;r=6980}else{k=b|0;Hw(b+8|0,c[k>>2]|0,h);c[g>>2]=1;l=f&e;c[k>>2]=l;n=l;o=c[a+4>>2]|0;p=c[497858]|0;q=5;r=6978}}while(0);do{if((r|0)==6978){c[g>>2]=q;e=n<<4|p&o;c[b>>2]=e;h=c[a+8>>2]|0;i=q+4|0;if(i>>>0<32){s=p;t=e;u=h;v=i;r=6980;break}Hw(b+8|0,e,q);c[g>>2]=4;e=p&h;c[b>>2]=e;w=e;x=c[a+12>>2]|0;y=c[497857]|0;z=7;r=6982}}while(0);do{if((r|0)==6980){c[g>>2]=v;p=t<<4|s&u;c[b>>2]=p;q=c[a+12>>2]|0;o=c[497857]|0;n=v+3|0;if(n>>>0<32){w=p;x=q;y=o;z=n;r=6982;break}Hw(b+8|0,p,v);c[g>>2]=3;p=o&q;c[b>>2]=p;A=p;B=5;r=6984}}while(0);do{if((r|0)==6982){c[g>>2]=z;v=w<<3|y&x;c[b>>2]=v;u=z+2|0;if(u>>>0<32){A=v;B=u;r=6984;break}Hw(b+8|0,v,z);c[g>>2]=2;c[b>>2]=0;v=a+24|0;C=0;D=v;E=c[v>>2]|0;F=3;r=6986}}while(0);do{if((r|0)==6984){c[g>>2]=B;z=A<<2;c[b>>2]=z;x=a+24|0;y=c[x>>2]|0;w=B+1|0;if(w>>>0<32){C=z;D=x;E=y;F=w;r=6986;break}Hw(b+8|0,z,B);c[g>>2]=1;z=f&y;c[b>>2]=z;y=a+28|0;G=x;H=z;I=y;J=c[y>>2]|0;K=2;r=6987}}while(0);do{if((r|0)==6986){c[g>>2]=F;B=C<<1|f&E;c[b>>2]=B;A=a+28|0;y=c[A>>2]|0;z=F+1|0;if(z>>>0<32){G=D;H=B;I=A;J=y;K=z;r=6987;break}Hw(b+8|0,B,F);c[g>>2]=1;B=f&y;c[b>>2]=B;L=1;M=B;N=D;O=A}}while(0);if((r|0)==6987){c[g>>2]=K;D=H<<1|f&J;c[b>>2]=D;L=K;M=D;N=G;O=I}L9725:do{if((c[N>>2]|0)==0){P=16;Q=L;R=M}else{I=c[a+56>>2]|0;G=c[497856]|0;D=L+2|0;do{if(D>>>0<32){c[g>>2]=D;K=M<<2|G&I;c[b>>2]=K;J=c[a+52>>2]|0;H=L+3|0;if(H>>>0<32){S=K;T=J;U=H;r=6994;break}Hw(b+8|0,K,D);c[g>>2]=1;K=f&J;c[b>>2]=K;V=K;W=c[a+16>>2]|0;X=3}else{Hw(b+8|0,M,L);c[g>>2]=2;K=G&I;c[b>>2]=K;S=K;T=c[a+52>>2]|0;U=3;r=6994}}while(0);do{if((r|0)==6994){c[g>>2]=U;I=S<<1|f&T;c[b>>2]=I;D=c[a+16>>2]|0;K=U+2|0;if(K>>>0<32){V=I;W=D;X=K;break}Hw(b+8|0,I,U);c[g>>2]=2;I=G&D;c[b>>2]=I;P=21;Q=2;R=I;break L9725}}while(0);c[g>>2]=X;I=V<<2|G&W;c[b>>2]=I;P=21;Q=X;R=I}}while(0);if((c[O>>2]|0)==0){d=P;return d|0}O=c[a+36>>2]|0;X=c[497856]|0;W=Q+2|0;do{if(W>>>0<32){c[g>>2]=W;V=R<<2|X&O;c[b>>2]=V;U=P|2;T=c[a+40>>2]|0;S=Q+4|0;if(S>>>0<32){Y=V;Z=U;_=T;$=S;r=7002;break}Hw(b+8|0,V,W);c[g>>2]=2;V=X&T;c[b>>2]=V;aa=U;ab=V;ac=c[a+44>>2]|0;ad=3;r=7004}else{Hw(b+8|0,R,Q);c[g>>2]=2;V=X&O;c[b>>2]=V;Y=V;Z=P|2;_=c[a+40>>2]|0;$=4;r=7002}}while(0);do{if((r|0)==7002){c[g>>2]=$;P=Y<<2|X&_;c[b>>2]=P;O=c[a+44>>2]|0;Q=$+1|0;if(Q>>>0<32){aa=Z;ab=P;ac=O;ad=Q;r=7004;break}Hw(b+8|0,P,$);c[g>>2]=1;P=f&O;c[b>>2]=P;ae=Z;af=P;ag=c[a+48>>2]|0;ah=2;r=7005}}while(0);do{if((r|0)==7004){c[g>>2]=ad;Z=ab<<1|f&ac;c[b>>2]=Z;$=c[a+48>>2]|0;_=ad+1|0;if(_>>>0<32){ae=aa;af=Z;ag=$;ah=_;r=7005;break}Hw(b+8|0,Z,ad);c[g>>2]=1;c[b>>2]=f&$;ai=aa}}while(0);if((r|0)==7005){c[g>>2]=ah;c[b>>2]=af<<1|f&ag;ai=ae}d=ai+4|0;return d|0}function Ib(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0;e=b+1120|0;f=c[e>>2]|0;g=c[f>>2]|0;h=c[f+4>>2]|0;i=b+1606|0;j=c[f+8>>2]|0;do{if((a[i]|0)==0){f=c[497856]|0;k=d+4|0;l=c[k>>2]|0;m=l+2|0;if(m>>>0<32){c[k>>2]=m;n=d|0;o=c[n>>2]<<2|f&j;c[n>>2]=o;p=2;q=m;r=o;break}else{o=d|0;Hw(d+8|0,c[o>>2]|0,l);c[k>>2]=2;k=f&j;c[o>>2]=k;p=2;q=2;r=k;break}}else{k=c[497855]|0;o=d+4|0;f=c[o>>2]|0;l=f+1|0;if(l>>>0<32){c[o>>2]=l;m=d|0;n=c[m>>2]<<1|k&j;c[m>>2]=n;p=1;q=l;r=n;break}else{n=d|0;Hw(d+8|0,c[n>>2]|0,f);c[o>>2]=1;o=k&j;c[n>>2]=o;p=1;q=1;r=o;break}}}while(0);j=c[e>>2]|0;o=c[j+8>>2]|0;if((o|0)==3){n=(c[j+56>>2]|0)-g|0;k=c[497856]|0;f=d+4|0;l=q+2|0;if(l>>>0<32){c[f>>2]=l;m=r<<2|k&n;c[d>>2]=m;s=l;t=m}else{Hw(d+8|0,r,q);c[f>>2]=2;m=k&n;c[d>>2]=m;s=2;t=m}m=(c[(c[e>>2]|0)+60>>2]|0)-(h+g)|0;n=s+2|0;do{if(n>>>0<32){c[f>>2]=n;l=t<<2|k&m;c[d>>2]=l;u=p|4;v=c[(c[e>>2]|0)+64>>2]|0;w=s+4|0;if(w>>>0<32){x=l;y=u;z=v;A=w;B=7064;break}Hw(d+8|0,l,n);c[f>>2]=2;l=k&v;c[d>>2]=l;C=u;D=l;E=c[(c[e>>2]|0)+68>>2]|0;F=4;B=7065}else{Hw(d+8|0,t,s);c[f>>2]=2;l=k&m;c[d>>2]=l;x=l;y=p|4;z=c[(c[e>>2]|0)+64>>2]|0;A=4;B=7064}}while(0);do{if((B|0)==7064){c[f>>2]=A;m=x<<2|k&z;c[d>>2]=m;s=c[(c[e>>2]|0)+68>>2]|0;t=A+2|0;if(t>>>0<32){C=y;D=m;E=s;F=t;B=7065;break}Hw(d+8|0,m,A);c[f>>2]=2;m=k&s;c[d>>2]=m;G=2;H=m;I=y}}while(0);if((B|0)==7065){c[f>>2]=F;y=D<<2|k&E;c[d>>2]=y;G=F;H=y;I=C}C=I+4|0;I=c[e>>2]|0;if((c[I+64>>2]|0)>0){y=d|0;F=d+8|0;E=0;D=C;A=I;z=G;x=H;while(1){m=(c[A+72+(E<<2)>>2]|0)-2>>1;s=z+2|0;if(s>>>0<32){c[f>>2]=s;J=s;K=x<<2|m&k}else{Hw(F,x,z);c[f>>2]=2;J=2;K=m&k}c[y>>2]=K;m=D+2|0;s=E+1|0;t=c[e>>2]|0;if((s|0)<(c[t+64>>2]|0)){E=s;D=m;A=t;z=J;x=K}else{L=m;M=t;N=J;O=K;break}}}else{L=C;M=I;N=G;O=H}H=c[M+68>>2]|0;if((H|0)>0){G=d|0;I=d+8|0;C=0;K=L;J=M;x=N;z=O;while(1){A=(c[J+84+(C<<2)>>2]|0)-2>>1;D=x+2|0;if(D>>>0<32){c[f>>2]=D;P=D;Q=z<<2|A&k}else{Hw(I,z,x);c[f>>2]=2;P=2;Q=A&k}c[G>>2]=Q;A=K+2|0;D=C+1|0;E=c[e>>2]|0;y=c[E+68>>2]|0;if((D|0)<(y|0)){C=D;K=A;J=E;x=P;z=Q}else{R=A;S=y;T=E;U=P;V=Q;break}}}else{R=L;S=H;T=M;U=N;V=O}O=S+2+(c[T+64>>2]|0)|0;S=-1;N=0;while(1){W=S+1|0;if((1<<W|0)<(O|0)){S=W;N=N+1&255}else{break}}S=c[T+24>>2]|0;T=c[1991416+(W<<2)>>2]|0;O=U+W|0;if(O>>>0<32){c[f>>2]=O;M=V<<W|T&S;c[d>>2]=M;X=O;Y=M}else{Hw(d+8|0,V,U);c[f>>2]=W;U=T&S;c[d>>2]=U;X=W;Y=U}U=c[e>>2]|0;S=c[U+64>>2]|0;T=c[U+68>>2]|0;V=S+1+T|0;if((V|0)<=0){Z=(W&255)+R|0;return Z|0}W=c[497855]|0;M=d|0;O=d+8|0;H=R+S+T|0;T=N&255;N=0;S=X;X=Y;Y=U;while(1){U=c[Y+96+(N<<2)>>2]|0;R=S+1|0;if(R>>>0<32){c[f>>2]=R;_=R;$=X<<1|W&U}else{Hw(O,X,S);c[f>>2]=1;_=1;$=W&U}c[M>>2]=$;U=N+1|0;if((U|0)>=(V|0)){break}N=U;S=_;X=$;Y=c[e>>2]|0}Z=H+1+T|0;return Z|0}else if((o|0)==1|(o|0)==2){T=(c[j+16>>2]|0)-g+((o|0)==1?-h|0:0)|0;h=c[497856]|0;g=d+4|0;H=q+2|0;do{if(H>>>0<32){c[g>>2]=H;Y=r<<2|T&h;c[d>>2]=Y;$=c[(c[e>>2]|0)+20>>2]|0;X=q+4|0;if(X>>>0<32){aa=Y;ab=$;ac=X;B=7038;break}Hw(d+8|0,Y,H);c[g>>2]=2;Y=h&$;c[d>>2]=Y;ad=2;ae=Y}else{Hw(d+8|0,r,q);c[g>>2]=2;Y=T&h;c[d>>2]=Y;aa=Y;ab=c[(c[e>>2]|0)+20>>2]|0;ac=4;B=7038}}while(0);if((B|0)==7038){c[g>>2]=ac;B=aa<<2|h&ab;c[d>>2]=B;ad=ac;ae=B}B=p|4;ac=c[e>>2]|0;ab=c[ac+20>>2]|0;if((ab|0)>0){aa=d|0;T=d+8|0;H=0;Y=B;$=ac;X=ad;_=ae;while(1){S=(c[$+28+(H<<2)>>2]|0)-2>>1;N=X+2|0;if(N>>>0<32){c[g>>2]=N;af=N;ag=_<<2|S&h}else{Hw(T,_,X);c[g>>2]=2;af=2;ag=S&h}c[aa>>2]=ag;S=Y+2|0;N=H+1|0;V=c[e>>2]|0;M=c[V+20>>2]|0;if((N|0)<(M|0)){H=N;Y=S;$=V;X=af;_=ag}else{ah=S;ai=M;aj=V;ak=af;al=ag;break}}}else{ah=B;ai=ab;aj=ac;ak=ad;al=ae}ae=ai+2|0;ai=-1;do{ai=ai+1|0;}while((1<<ai|0)<(ae|0));ae=c[aj+24>>2]|0;aj=c[1991416+(ai<<2)>>2]|0;ad=ak+ai|0;if(ad>>>0<32){c[g>>2]=ad;ac=al<<ai|aj&ae;c[d>>2]=ac;am=ad;an=ac}else{Hw(d+8|0,al,ak);c[g>>2]=ai;ak=aj&ae;c[d>>2]=ak;am=ai;an=ak}ak=(ai&255)+ah|0;ah=c[e>>2]|0;if(((c[ah+20>>2]|0)+1|0)<=0){Z=ak;return Z|0}ai=c[497855]|0;ae=d|0;aj=d+8|0;al=0;ac=ak;ak=ah;ah=am;am=an;while(1){an=c[ak+40+(al<<2)>>2]|0;ad=ah+1|0;if(ad>>>0<32){c[g>>2]=ad;ao=ad;ap=am<<1|ai&an}else{Hw(aj,am,ah);c[g>>2]=1;ao=1;ap=ai&an}c[ae>>2]=ap;an=ac+1|0;ad=al+1|0;ab=c[e>>2]|0;if((ad|0)<((c[ab+20>>2]|0)+1|0)){al=ad;ac=an;ak=ab;ah=ao;am=ap}else{Z=an;break}}return Z|0}else if((o|0)==0){o=c[j+12>>2]|0;j=-1;do{j=j+1|0;}while((1<<j|0)<(o|0));o=c[497856]|0;ap=d+4|0;am=q+2|0;if(am>>>0<32){c[ap>>2]=am;ao=r<<2|o&j;c[d>>2]=ao;aq=am;ar=ao}else{Hw(d+8|0,r,q);c[ap>>2]=2;q=o&j;c[d>>2]=q;aq=2;ar=q}q=p+2|0;do{if((a[i]|0)==0){as=q;at=aq;au=ar}else{if((c[(c[e>>2]|0)+12>>2]|0)!=1){as=q;at=aq;au=ar;break}j=c[b+1320>>2]|0;o=c[497855]|0;r=aq+1|0;if(r>>>0<32){c[ap>>2]=r;ao=ar<<1|o&j;c[d>>2]=ao;av=r;aw=ao}else{Hw(d+8|0,ar,aq);c[ap>>2]=1;ao=o&j;c[d>>2]=ao;av=1;aw=ao}as=p+3|0;at=av;au=aw}}while(0);aw=c[(c[e>>2]|0)+40>>2]|0;e=c[497855]|0;av=at+1|0;if(av>>>0<32){c[ap>>2]=av;c[d>>2]=au<<1|e&aw}else{Hw(d+8|0,au,at);c[ap>>2]=1;c[d>>2]=e&aw}Z=as+1|0;return Z|0}else{Z=p;return Z|0}return 0}function Ic(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;d=c[497855]|0;e=b+4|0;f=c[e>>2]|0;g=f+1|0;if(g>>>0<32){c[e>>2]=g;h=b|0;i=c[h>>2]<<1|d&1;c[h>>2]=i;j=g;k=i}else{i=b|0;Hw(b+8|0,c[i>>2]|0,f);c[e>>2]=1;f=d&1;c[i>>2]=f;j=1;k=f}f=a+1120|0;a=c[f>>2]|0;i=c[a+16>>2]|0;do{if((c[a+4>>2]|0)==8){g=c[497857]|0;h=j+3|0;if(h>>>0<32){c[e>>2]=h;l=k<<3|g&i;c[b>>2]=l;m=4;n=h;o=l;break}else{Hw(b+8|0,k,j);c[e>>2]=3;l=g&i;c[b>>2]=l;m=4;n=3;o=l;break}}else{l=c[497858]|0;g=j+4|0;if(g>>>0<32){c[e>>2]=g;h=k<<4|l&i;c[b>>2]=h;m=5;n=g;o=h;break}else{Hw(b+8|0,k,j);c[e>>2]=4;h=l&i;c[b>>2]=h;m=5;n=4;o=h;break}}}while(0);i=c[f>>2]|0;if((c[i+12>>2]|0)<=0){p=m;return p|0}j=b|0;k=b+8|0;b=m;m=0;a=i;i=n;n=o;while(1){o=c[a+40+(m<<2)>>2]|0;h=i+1|0;if(h>>>0<32){c[e>>2]=h;q=h;r=n<<1|d&o}else{Hw(k,n,i);c[e>>2]=1;q=1;r=d&o}c[j>>2]=r;o=b+1|0;h=m+1|0;l=c[f>>2]|0;if((h|0)<(c[l+12>>2]|0)){b=o;m=h;a=l;i=q;n=r}else{p=o;break}}return p|0}function Id(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0;f=b+1196|0;if((c[f>>2]|0)<=0){g=0;return g|0}h=(e|0)==0;e=b+1184|0;i=d+4|0;j=d|0;k=d+8|0;d=b+1312|0;l=b+1180|0;m=b+1020|0;n=b+1028|0;o=b+1036|0;p=b+1024|0;q=b+1032|0;r=b+1040|0;s=b+1056|0;t=b+1048|0;u=b+1064|0;v=b+1016|0;w=b+1044|0;x=b+1060|0;y=b+1052|0;z=b+1068|0;A=0;B=0;while(1){C=b+1220+(A<<2)|0;L9904:do{if((c[C>>2]|0)==0){do{if(!h){if((c[d>>2]|0)==0){break}D=c[b+56+(A*192|0)>>2]|0;E=c[l>>2]|0;F=c[1991416+(E<<2)>>2]|0;G=c[i>>2]|0;H=G+E|0;if(H>>>0<32){c[i>>2]=H;I=c[j>>2]<<E|F&D}else{Hw(k,c[j>>2]|0,G);c[i>>2]=E;I=F&D}c[j>>2]=I;J=(E&255)+B|0;break L9904}}while(0);E=c[b+56+(A*192|0)>>2]|0;D=c[e>>2]|0;F=c[1991416+(D<<2)>>2]|0;G=c[i>>2]|0;H=G+D|0;if(H>>>0<32){c[i>>2]=H;K=c[j>>2]<<D|F&E}else{Hw(k,c[j>>2]|0,G);c[i>>2]=D;K=F&E}c[j>>2]=K;J=(D&255)+B|0}else{J=B}}while(0);D=1-(c[C>>2]|0)|0;E=b+1200+(A<<2)|0;if((D|0)<(c[E>>2]|0)){F=D;D=J;while(1){G=c[b+56+(A*192|0)+(F<<2)>>2]|0;do{if(h){H=(c[m>>2]|0)+G|0;if((c[C>>2]|0)==0){L=c[(c[n>>2]|0)+(H<<2)>>2]|0;M=a[(c[o>>2]|0)+H|0]|0;N=M&255;O=c[1991416+(N<<2)>>2]|0;P=c[i>>2]|0;Q=P+N|0;if(Q>>>0<32){c[i>>2]=Q;c[j>>2]=c[j>>2]<<N|O&L;R=M;break}else{Hw(k,c[j>>2]|0,P);c[i>>2]=N;c[j>>2]=O&L;R=M;break}}else{M=c[(c[p>>2]|0)+(H<<2)>>2]|0;L=a[(c[q>>2]|0)+H|0]|0;H=L&255;O=c[1991416+(H<<2)>>2]|0;N=c[i>>2]|0;P=N+H|0;if(P>>>0<32){c[i>>2]=P;c[j>>2]=c[j>>2]<<H|O&M;R=L;break}else{Hw(k,c[j>>2]|0,N);c[i>>2]=H;c[j>>2]=O&M;R=L;break}}}else{L=(c[C>>2]|0)!=0;if((c[d>>2]|0)==0){M=(c[m>>2]|0)+G|0;if(L){O=c[(c[r>>2]|0)+(M<<2)>>2]|0;H=a[(c[s>>2]|0)+M|0]|0;N=H&255;P=c[1991416+(N<<2)>>2]|0;Q=c[i>>2]|0;S=Q+N|0;if(S>>>0<32){c[i>>2]=S;c[j>>2]=c[j>>2]<<N|P&O;R=H;break}else{Hw(k,c[j>>2]|0,Q);c[i>>2]=N;c[j>>2]=P&O;R=H;break}}else{H=c[(c[t>>2]|0)+(M<<2)>>2]|0;O=a[(c[u>>2]|0)+M|0]|0;M=O&255;P=c[1991416+(M<<2)>>2]|0;N=c[i>>2]|0;Q=N+M|0;if(Q>>>0<32){c[i>>2]=Q;c[j>>2]=c[j>>2]<<M|P&H;R=O;break}else{Hw(k,c[j>>2]|0,N);c[i>>2]=M;c[j>>2]=P&H;R=O;break}}}else{O=(c[v>>2]|0)+G|0;if(L){L=c[(c[w>>2]|0)+(O<<2)>>2]|0;H=a[(c[x>>2]|0)+O|0]|0;P=H&255;M=c[1991416+(P<<2)>>2]|0;N=c[i>>2]|0;Q=N+P|0;if(Q>>>0<32){c[i>>2]=Q;c[j>>2]=c[j>>2]<<P|M&L;R=H;break}else{Hw(k,c[j>>2]|0,N);c[i>>2]=P;c[j>>2]=M&L;R=H;break}}else{H=c[(c[y>>2]|0)+(O<<2)>>2]|0;L=a[(c[z>>2]|0)+O|0]|0;O=L&255;M=c[1991416+(O<<2)>>2]|0;P=c[i>>2]|0;N=P+O|0;if(N>>>0<32){c[i>>2]=N;c[j>>2]=c[j>>2]<<O|M&H;R=L;break}else{Hw(k,c[j>>2]|0,P);c[i>>2]=O;c[j>>2]=M&H;R=L;break}}}}}while(0);G=(R&255)+D|0;L=F+1|0;if((L|0)<(c[E>>2]|0)){F=L;D=G}else{T=G;break}}}else{T=J}D=A+1|0;if((D|0)<(c[f>>2]|0)){A=D;B=T}else{g=T;break}}return g|0}function Ie(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0;f=(c[b+1196>>2]|0)>1?2:1;g=(e|0)!=0;h=b+1312|0;i=b+1308|0;j=b+1188|0;k=d+4|0;l=d|0;m=d+8|0;d=b+1192|0;n=b+1108|0;o=b+1104|0;p=b+1116|0;q=b+1112|0;r=b+1084|0;s=b+1080|0;t=(e|0)==0;e=b+1092|0;u=b+1088|0;v=b+1100|0;w=b+1096|0;x=0;y=0;while(1){z=c[b+1240+(x<<2)>>2]|0;L9961:do{if((z|0)==1){A=c[i>>2]|0;B=ac(A,x)|0;C=x+1|0;if((B|0)>=(ac(A,C)|0)){D=y;E=C;break}if(t){A=y;F=B;while(1){G=(a[b+1260+F|0]|0)+31|0;H=c[(c[e>>2]|0)+(G<<2)>>2]|0;I=a[(c[u>>2]|0)+G|0]|0;G=I&255;J=c[1991416+(G<<2)>>2]|0;K=c[k>>2]|0;L=K+G|0;if(L>>>0<32){c[k>>2]=L;M=c[l>>2]<<G|J&H}else{Hw(m,c[l>>2]|0,K);c[k>>2]=G;M=J&H}c[l>>2]=M;H=(I&255)+A|0;I=F+1|0;if((I|0)<(ac(c[i>>2]|0,C)|0)){A=H;F=I}else{D=H;E=C;break L9961}}}else{N=y;O=B}while(1){F=a[b+1260+O|0]|0;do{if((c[h>>2]|0)==0){A=F+31|0;H=c[(c[e>>2]|0)+(A<<2)>>2]|0;I=a[(c[u>>2]|0)+A|0]|0;A=I&255;J=c[1991416+(A<<2)>>2]|0;G=c[k>>2]|0;K=G+A|0;if(K>>>0<32){c[k>>2]=K;c[l>>2]=c[l>>2]<<A|J&H;P=I;break}else{Hw(m,c[l>>2]|0,G);c[k>>2]=A;c[l>>2]=J&H;P=I;break}}else{I=F+12|0;H=c[(c[v>>2]|0)+(I<<2)>>2]|0;J=a[(c[w>>2]|0)+I|0]|0;I=J&255;A=c[1991416+(I<<2)>>2]|0;G=c[k>>2]|0;K=G+I|0;if(K>>>0<32){c[k>>2]=K;c[l>>2]=c[l>>2]<<I|A&H;P=J;break}else{Hw(m,c[l>>2]|0,G);c[k>>2]=I;c[l>>2]=A&H;P=J;break}}}while(0);F=(P&255)+N|0;J=O+1|0;if((J|0)<(ac(c[i>>2]|0,C)|0)){N=F;O=J}else{D=F;E=C;break}}}else if((z|0)==0){do{if(g){if((c[h>>2]|0)==0){Q=7168;break}C=a[(ac(c[i>>2]|0,x)|0)+(b+1260)|0]|0;B=c[j>>2]|0;F=c[1991416+(B<<2)>>2]|0;J=c[k>>2]|0;H=J+B|0;if(H>>>0<32){c[k>>2]=H;A=c[l>>2]<<B|F&C;c[l>>2]=A;R=B&255;S=H;T=A;break}else{Hw(m,c[l>>2]|0,J);c[k>>2]=B;J=F&C;c[l>>2]=J;R=B&255;S=B;T=J;break}}else{Q=7168}}while(0);do{if((Q|0)==7168){Q=0;J=a[(ac(c[i>>2]|0,x)|0)+(b+1260)|0]|0;B=c[d>>2]|0;C=c[1991416+(B<<2)>>2]|0;F=c[k>>2]|0;A=F+B|0;if(A>>>0<32){c[k>>2]=A;H=c[l>>2]<<B|C&J;c[l>>2]=H;R=B&255;S=A;T=H;break}else{Hw(m,c[l>>2]|0,F);c[k>>2]=B;F=C&J;c[l>>2]=F;R=B&255;S=B;T=F;break}}}while(0);F=(R&255)+y|0;B=c[i>>2]|0;J=(ac(B,x)|0)+1|0;C=x+1|0;if((J|0)<(ac(B,C)|0)){U=F;V=J;W=S;X=T}else{D=F;E=C;break}while(1){do{if(g){F=a[b+1260+V|0]|0;if((c[h>>2]|0)==0){J=F+31|0;B=c[(c[n>>2]|0)+(J<<2)>>2]|0;H=a[(c[o>>2]|0)+J|0]|0;J=H&255;A=c[1991416+(J<<2)>>2]|0;I=W+J|0;if(I>>>0<32){c[k>>2]=I;G=X<<J|A&B;c[l>>2]=G;Y=H;Z=I;_=G;break}else{Hw(m,X,W);c[k>>2]=J;G=A&B;c[l>>2]=G;Y=H;Z=J;_=G;break}}else{G=F+12|0;F=c[(c[p>>2]|0)+(G<<2)>>2]|0;J=a[(c[q>>2]|0)+G|0]|0;G=J&255;H=c[1991416+(G<<2)>>2]|0;B=W+G|0;if(B>>>0<32){c[k>>2]=B;A=X<<G|H&F;c[l>>2]=A;Y=J;Z=B;_=A;break}else{Hw(m,X,W);c[k>>2]=G;A=H&F;c[l>>2]=A;Y=J;Z=G;_=A;break}}}else{A=(a[b+1260+V|0]|0)+31|0;G=c[(c[r>>2]|0)+(A<<2)>>2]|0;J=a[(c[s>>2]|0)+A|0]|0;A=J&255;F=c[1991416+(A<<2)>>2]|0;H=W+A|0;if(H>>>0<32){c[k>>2]=H;B=X<<A|F&G;c[l>>2]=B;Y=J;Z=H;_=B;break}else{Hw(m,X,W);c[k>>2]=A;B=F&G;c[l>>2]=B;Y=J;Z=A;_=B;break}}}while(0);B=(Y&255)+U|0;A=V+1|0;if((A|0)<(ac(c[i>>2]|0,C)|0)){U=B;V=A;W=Z;X=_}else{D=B;E=C;break}}}else{D=y;E=x+1|0}}while(0);if((E|0)<(f|0)){x=E;y=D}else{break}}return D|0}function If(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;do{if((a|0)!=0){d=(Ij(a,0)|0)+9>>3;if((d|0)==0){break}e=c[497855]|0;f=b+4|0;g=c[f>>2]|0;h=g+1|0;if(h>>>0<32){c[f>>2]=h;i=b|0;j=c[i>>2]<<1|e&1;c[i>>2]=j;k=h;l=j}else{j=b|0;Hw(b+8|0,c[j>>2]|0,g);c[f>>2]=1;g=e&1;c[j>>2]=g;k=1;l=g}g=c[497858]|0;j=k+4|0;e=j>>>0<32;L10025:do{if((d|0)<15){if(e){c[f>>2]=j;h=l<<4|g&d;c[b>>2]=h;m=5;n=j;o=h;p=7215;break}else{Hw(b+8|0,l,k);c[f>>2]=4;h=g&d;c[b>>2]=h;q=5;r=6;s=h;p=7214;break}}else{do{if(e){c[f>>2]=j;h=l<<4|g&15;c[b>>2]=h;i=d-15|0;t=c[497862]|0;u=k+12|0;if(u>>>0<32){v=h;w=i;x=t;y=u;break}Hw(b+8|0,h,j);c[f>>2]=8;h=t&i;c[b>>2]=h;q=13;r=10;s=h;p=7214;break L10025}else{Hw(b+8|0,l,k);c[f>>2]=4;h=g&15;c[b>>2]=h;v=h;w=d-15|0;x=c[497862]|0;y=12}}while(0);c[f>>2]=y;h=v<<8|x&w;c[b>>2]=h;m=13;n=y;o=h;p=7215}}while(0);do{if((p|0)==7215){d=c[497856]|0;g=n+2|0;if(g>>>0<32){z=o;A=m;B=d;C=g;p=7216;break}Hw(b+8|0,o,n);c[f>>2]=2;c[b>>2]=d&2;D=m}else if((p|0)==7214){z=s;A=q;B=c[497856]|0;C=r;p=7216}}while(0);if((p|0)==7216){c[f>>2]=C;c[b>>2]=z<<2|B&2;D=A}d=(Ij(a,b)|0)+2|0;g=d+D|0;j=(d|0)%8|0;if((j|0)==0){E=g;return E|0}d=8-j|0;j=c[f>>2]|0;e=j+d|0;if(e>>>0<32){c[f>>2]=e;e=b|0;c[e>>2]=c[e>>2]<<d}else{e=b|0;Hw(b+8|0,c[e>>2]|0,j);c[f>>2]=d;c[e>>2]=0}E=(d&255)+g|0;return E|0}}while(0);D=b+4|0;a=c[D>>2]|0;A=a+1|0;if(A>>>0<32){c[D>>2]=A;A=b|0;c[A>>2]=c[A>>2]<<1;E=1;return E|0}else{A=b|0;Hw(b+8|0,c[A>>2]|0,a);c[D>>2]=1;c[A>>2]=0;E=1;return E|0}return 0}function Ig(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;if((a|0)==0){b=32;return b|0}d=JA(0)|0;if((d|0)==0){b=33;return b|0}I_(d,30608);e=Mg(d|0)|0;if((e|0)==0){f=0}else{b=e;return b|0}while(1){if((f|0)>=2){break}if((K2(d+30260+(f*168|0)|0,d+23780+(f*312|0)|0,312,d+24404+(f*2928|0)|0,2928)|0)==0){f=f+1|0}else{b=33;g=7241;break}}if((g|0)==7241){return b|0}c[a>>2]=d;b=0;return b|0}function Ih(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0;if((a|0)==0|(b|0)==0){g=32;return g|0}c[a+23768>>2]=1;c[a+23772>>2]=d;d=a+23776|0;c[d>>2]=e;I_(a+15460|0,8192);c[a+23652>>2]=15;K3(a+30260|0,0,64,64,1)|0;K3(a+30428|0,0,64,64,1)|0;K5(a+30596|0,0,64,64)|0;c[a+23656>>2]=(c[d>>2]|0)*6|0;d=b+16|0;e=c[d>>2]|0;if((e-1|0)>>>0>3){c[d>>2]=2;h=2}else{h=e}c[a+23660>>2]=h;h=Mh(c[a>>2]|0,c[b+12>>2]|0,c[b+20>>2]|0)|0;if((h|0)==0){i=0}else{g=h;return g|0}do{h=JF(i,f)|0;b=JG(i,f)|0;e=0;do{d=e*71|0;j=e+10|0;c[a+14788+(j<<4)+(i<<3)>>2]=h+(d<<2);c[a+14788+(j<<4)+(i<<3)+4>>2]=b+(d<<2);e=e+1|0;}while((e|0)<32);c[a+14788+(i<<3)>>2]=a+3428+(i*568|0);c[a+14788+(i<<3)+4>>2]=a+3428+(i*568|0)+284;c[a+14804+(i<<3)>>2]=a+4564+(i*568|0);c[a+14804+(i<<3)+4>>2]=a+4564+(i*568|0)+284;c[a+14820+(i<<3)>>2]=a+5700+(i*568|0);c[a+14820+(i<<3)+4>>2]=a+5700+(i*568|0)+284;c[a+14836+(i<<3)>>2]=a+6836+(i*568|0);c[a+14836+(i<<3)+4>>2]=a+6836+(i*568|0)+284;c[a+14852+(i<<3)>>2]=a+7972+(i*568|0);c[a+14852+(i<<3)+4>>2]=a+7972+(i*568|0)+284;c[a+14868+(i<<3)>>2]=a+9108+(i*568|0);c[a+14868+(i<<3)+4>>2]=a+9108+(i*568|0)+284;c[a+14884+(i<<3)>>2]=a+10244+(i*568|0);c[a+14884+(i<<3)+4>>2]=a+10244+(i*568|0)+284;c[a+14900+(i<<3)>>2]=a+11380+(i*568|0);c[a+14900+(i<<3)+4>>2]=a+11380+(i*568|0)+284;c[a+14916+(i<<3)>>2]=a+12516+(i*568|0);c[a+14916+(i<<3)+4>>2]=a+12516+(i*568|0)+284;c[a+14932+(i<<3)>>2]=a+13652+(i*568|0);c[a+14932+(i<<3)+4>>2]=a+13652+(i*568|0)+284;i=i+1|0;}while((i|0)<2);I_(a+3428|0,11360);I_(a+4|0,3424);I_(a+23664|0,20);I_(a+23684|0,80);g=0;return g|0}function Ii(a){a=a|0;var b=0;if((a|0)==0){return 0}b=c[a>>2]|0;if((b|0)==0){return 0}Mi(b|0)|0;JB(a);return 0}function Ij(a,b){a=a|0;b=b|0;var c=0;if((a|0)==0){c=0;return c|0}c=Me(a+4|0,b)|0;return c|0}function Ik(b,d,e,f,g,h,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0;n=i;i=i+2776|0;o=n|0;p=n+512|0;q=n+1080|0;r=n+1592|0;s=n+1752|0;t=s+512|0;u=s+768|0;v=s|0;s=c[f>>2]|0;if((c[s+28>>2]|0)>0){w=b+30260|0;x=0;y=s;while(1){z=c[d>>2]|0;A=c[y+24>>2]|0;Li(y,t,u,z+((ac(ac(x,e)|0,A)|0)<<1)|0,e,v);A=x+10|0;K4(w,t,u,c[b+14788+(A<<4)>>2]|0,c[b+14788+(A<<4)+4>>2]|0)|0;A=x+1|0;z=c[f>>2]|0;if((A|0)<(c[z+28>>2]|0)){x=A;y=z}else{B=z;break}}}else{B=s}s=c[B+40>>2]|0;B=f+4|0;f=c[B>>2]|0;if((c[f+28>>2]|0)>0){y=d+4|0;d=b+30428|0;x=0;w=f;do{f=c[y>>2]|0;z=c[w+24>>2]|0;Li(w,t,u,f+((ac(ac(x,e)|0,z)|0)<<1)|0,e,v);z=x+10|0;K4(d,t,u,c[b+14788+(z<<4)+8>>2]|0,c[b+14788+(z<<4)+12>>2]|0)|0;x=x+1|0;w=c[B>>2]|0;}while((x|0)<(c[w+28>>2]|0))}w=b+23664|0;x=b+23764|0;B=b|0;u=c[B>>2]|0;t=b+23772|0;d=c[t>>2]|0;v=u+884|0;e=c[v>>2]|0;y=(c[u+892>>2]|0)+(c[u+888>>2]|0)|0;I_(r,160);if((y|0)>0){z=d-10|0;f=(z|0)>0;A=(c[v>>2]|0)==10|0;v=0;while(1){C=c[u+1092+(v<<2)>>2]>>A;D=r+(C<<2)|0;E=c[D>>2]|0;F=u+896+(v<<2)|0;do{if(f){H=c[F>>2]|0;I=v+1|0;J=u+896+(I<<2)|0;K=c[J>>2]|0;if((H|0)<(K|0)){L=0;M=E}else{N=E;O=I;P=J;break}while(1){Q=L+10|0;R=c[b+14788+(Q<<4)>>2]|0;S=c[b+14788+(Q<<4)+4>>2]|0;T=c[b+14788+(Q<<4)+8>>2]|0;U=c[b+14788+(Q<<4)+12>>2]|0;Q=M;V=H;do{W=c[R+(V<<2)>>2]|0;X=(W|0)>0?W:-W|0;W=(Q|0)>(X|0)?Q:X;X=c[S+(V<<2)>>2]|0;Y=(X|0)>0?X:-X|0;X=(W|0)>(Y|0)?W:Y;Y=c[T+(V<<2)>>2]|0;W=(Y|0)>0?Y:-Y|0;Y=(X|0)>(W|0)?X:W;W=c[U+(V<<2)>>2]|0;X=(W|0)>0?W:-W|0;Q=(Y|0)>(X|0)?Y:X;V=V+1|0;}while((V|0)<(K|0));V=L+1|0;if((V|0)<(z|0)){L=V;M=Q}else{N=Q;O=I;P=J;break}}}else{J=v+1|0;N=E;O=J;P=u+896+(J<<2)|0}}while(0);c[D>>2]=N;E=r+80+(C<<2)|0;J=c[E>>2]|0;I=c[F>>2]|0;K=c[P>>2]|0;if((I|0)<(K|0)){H=z;V=J;while(1){U=H+10|0;T=c[b+14788+(U<<4)>>2]|0;S=c[b+14788+(U<<4)+4>>2]|0;R=c[b+14788+(U<<4)+8>>2]|0;X=c[b+14788+(U<<4)+12>>2]|0;U=V;Y=I;do{W=c[T+(Y<<2)>>2]|0;Z=(W|0)>0?W:-W|0;W=(U|0)>(Z|0)?U:Z;Z=c[S+(Y<<2)>>2]|0;_=(Z|0)>0?Z:-Z|0;Z=(W|0)>(_|0)?W:_;_=c[R+(Y<<2)>>2]|0;W=(_|0)>0?_:-_|0;_=(Z|0)>(W|0)?Z:W;W=c[X+(Y<<2)>>2]|0;Z=(W|0)>0?W:-W|0;U=(_|0)>(Z|0)?_:Z;Y=Y+1|0;}while((Y|0)<(K|0));Y=H+1|0;if((Y|0)<(d|0)){H=Y;V=U}else{$=U;break}}}else{$=J}c[E>>2]=$;if((O|0)<(y|0)){v=O}else{break}}}do{if((e|0)>0){O=0;v=0;do{y=c[r+(O<<2)>>2]|0;$=b+23684+(O<<2)|0;d=c[$>>2]|0;z=(y|0)>(d|0)?y:d;do{if((z|0)==0){aa=0}else{d=z>>31^z;if((d|0)<=-1){aa=-1;break}P=~d;d=0;while(1){N=P<<1;if((N|0)<0){P=N;d=d+1|0}else{break}}aa=d&255}}while(0);a[b+23664+O|0]=aa;z=c[r+80+(O<<2)>>2]|0;U=(y|0)>(z|0)?y:z;v=(v|0)>(U|0)?v:U;c[$>>2]=U;O=O+1|0;}while((O|0)<(e|0));if((v|0)==0){ab=0;break}O=v>>31^v;if((O|0)<=-1){ab=-1;break}E=~O;O=0;while(1){J=E<<1;if((J|0)<0){E=J;O=O+1|0}else{ab=O;break}}}else{ab=0}}while(0);a[x]=(ab|0)>32?32:ab&255;ab=b+14788|0;if((b|0)==0){ad=32;i=n;return ad|0}e=b+23768|0;r=b+1716|0;if((c[e>>2]|0)==0){ae=r}else{aa=r;O=b+4|0;NN(aa|0,O|0,1712)|0;ae=aa}aa=b+4|0;NN(aa|0,ae|0,1712)|0;O=Mj(c[B>>2]|0,r,w,c[b+23660>>2]|0,ab,c[t>>2]|0,m)|0;if((O|0)!=0){ad=O;i=n;return ad|0}if((c[e>>2]|0)==0){af=0}else{NN(aa|0,ae|0,1712)|0;c[e>>2]=0;af=0}do{e=af+32|0;IY(c[b+14788+(af<<4)>>2]|0,c[b+14788+(e<<4)>>2]|0,284);IY(c[b+14788+(af<<4)+4>>2]|0,c[b+14788+(e<<4)+4>>2]|0,284);IY(c[b+14788+(af<<4)+8>>2]|0,c[b+14788+(e<<4)+8>>2]|0,284);IY(c[b+14788+(af<<4)+12>>2]|0,c[b+14788+(e<<4)+12>>2]|0,284);af=af+1|0;}while((af|0)<10);af=c[t>>2]|0;e=q;ae=(a[x]|0)-1|0;x=(ae|0)<0?0:ae;ae=s-x|0;if((af|0)>0){s=b+30596|0;aa=p|0;O=p+284|0;m=ae-7|0;ab=k+24|0;w=o|0;o=0;do{r=o+10|0;B=b+14788+(r<<4)|0;E=b+14788+(r<<4)+4|0;v=b+14788+(r<<4)+8|0;J=b+14788+(r<<4)+12|0;r=0;do{U=c[(c[B>>2]|0)+(r<<2)>>2]|0;z=c[(c[E>>2]|0)+(r<<2)>>2]|0;P=c[(c[v>>2]|0)+(r<<2)>>2]|0;Q=c[(c[J>>2]|0)+(r<<2)>>2]|0;N=(U|0)>0?U:-U|0;u=(z|0)>0?z:-z|0;M=(N|0)>(u|0)?N:u;u=(P|0)>0?P:-P|0;N=(Q|0)>0?Q:-Q|0;L=(u|0)>(N|0)?u:N;N=(M|0)>(L|0)?M:L;if((N|0)>-1){L=~N;N=0;while(1){M=L<<1;if((M|0)<0){L=M;N=N+1|0}else{break}}ag=N-1|0}else{ag=-2}L=(ag|0)<0?0:ag;$=U<<L;y=z<<L;M=P<<L;u=Q<<L;f=L-x|0;L=(f|0)<31?f:31;f=$;A=($|0)<0|0?-1:0;N2(f,A,f,A)|0;V=G;H=y;K=(y|0)<0|0?-1:0;N2(H,K,H,K)|0;I=V+G|0;V=M;F=(M|0)<0|0?-1:0;N2(V,F,V,F)|0;C=I+G|0;I=u;D=(u|0)<0|0?-1:0;N2(I,D,I,D)|0;Y=C+G|0;N2(V,F,f,A)|0;A=G<<1|0>>>31;N2(I,D,H,K)|0;K=(G<<1|0>>>31)+A+Y|0;A=(K|0)>0?K:-K|0;N2(A,(A|0)<0|0?-1:0,2147483647,0)|0;if((Y>>>1|0)<(G<<1|0>>>31|0)){do{if((Y|0)==0){ah=0}else{K=Y>>>31^Y;if((K|0)<=-1){ah=-1;break}H=~K;K=0;while(1){D=H<<1;if((D|0)<0){H=D;K=K+1|0}else{ah=K;break}}}}while(0);do{if((A|0)==0){ai=0}else{Q=A>>31^A;if((Q|0)<=-1){ai=-1;break}P=~Q;Q=0;while(1){z=P<<1;if((z|0)<0){P=z;Q=Q+1|0}else{ai=Q;break}}}}while(0);Q=ai-ah|0;P=HO(Y<<ah>>1,A<<ai,16)|0;if((Q&1|0)==0){aj=Q;ak=P}else{aj=Q+1|0;ak=P>>1}P=(HN(ak)|0)<<(aj>>1);al=(P|0)<0|0?-1:0;am=P}else{al=0;am=2147483647}P=$+M|0;N2(am,al,P,(P|0)<0|0?-1:0)|0;c[p+(r<<2)>>2]=G>>L;P=y+u|0;N2(am,al,P,(P|0)<0|0?-1:0)|0;c[p+284+(r<<2)>>2]=G>>L;r=r+1|0;}while((r|0)<71);r=g+(o<<2)|0;J=h+(o<<2)|0;K6(s,aa,O,c[r>>2]|0,c[J>>2]|0)|0;v=c[r>>2]|0;r=c[J>>2]|0;Lk(k,v,r,m,m,j+((ac(c[ab>>2]|0,o)|0)<<1)|0,1,w);o=o+1|0;}while((o|0)<(af|0))}af=7-ae&255;a[l]=af;ae=c[t>>2]>>1;t=c[b+23776>>2]|0;o=(ae|0)>0;if(o){w=t<<2;ab=q+256|0;q=0;do{j=b+15460+(q<<8)|0;IY(e,j,w);m=b+19556+(q<<8)|0;IY(ab,m,w);k=q+ae|0;O=g+(k<<2)|0;IY(j,c[O>>2]|0,w);j=h+(k<<2)|0;IY(m,c[j>>2]|0,w);m=g+(q<<2)|0;IY(c[O>>2]|0,c[m>>2]|0,w);O=h+(q<<2)|0;IY(c[j>>2]|0,c[O>>2]|0,w);IY(c[m>>2]|0,e,w);IY(c[O>>2]|0,ab,w);q=q+1|0;}while((q|0)<(ae|0));an=a[l]|0}else{an=af}af=b+23652|0;b=c[af>>2]|0;q=an<<24>>24;w=(b|0)>(q|0);ab=w?b-q|0:q-b|0;q=w?0:ae;if(o&(t|0)>0){o=0;do{w=o+q|0;e=g+(w<<2)|0;O=h+(w<<2)|0;w=0;do{m=(c[e>>2]|0)+(w<<2)|0;c[m>>2]=c[m>>2]>>ab;m=(c[O>>2]|0)+(w<<2)|0;c[m>>2]=c[m>>2]>>ab;w=w+1|0;}while((w|0)<(t|0));o=o+1|0;}while((o|0)<(ae|0));ao=a[l]|0;ap=c[af>>2]|0}else{ao=an;ap=b}b=ao<<24>>24;a[l]=(b|0)<(ap|0)?ao:ap&255;c[af>>2]=b;ad=0;i=n;return ad|0}function Il(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=c[a>>2]|0;if((b|0)==0){return}else{d=0}do{e=b+(d<<2)|0;f=c[e>>2]|0;if((f|0)!=0){g=f+68|0;if((c[g>>2]|0)!=0){Jj(g)}g=f+60|0;if((c[g>>2]|0)!=0){Jf(g)}g=f+64|0;if((c[g>>2]|0)!=0){Jh(g)}I7(e)}d=d+1|0;}while((d|0)<6);d=b+24|0;e=c[d>>2]|0;if((e|0)!=0){IL(e+1620|0);LU(e+760|0);I5(d)}d=b+52|0;if((c[d>>2]|0)!=0){I9(d)}d=b+28|0;e=c[d>>2]|0;if((e|0)!=0){IL(e+1620|0);LU(e+760|0);I5(d)}d=b+108|0;if((c[d>>2]|0)!=0){I9(d)}d=b+32|0;e=c[d>>2]|0;if((e|0)!=0){IL(e+1620|0);LU(e+760|0);I5(d)}d=b+164|0;if((c[d>>2]|0)!=0){I9(d)}d=b+36|0;e=c[d>>2]|0;if((e|0)!=0){IL(e+1620|0);LU(e+760|0);I5(d)}d=b+220|0;if((c[d>>2]|0)!=0){I9(d)}d=b+40|0;e=c[d>>2]|0;if((e|0)!=0){IL(e+1620|0);LU(e+760|0);I5(d)}d=b+276|0;if((c[d>>2]|0)!=0){I9(d)}d=b+44|0;e=c[d>>2]|0;if((e|0)!=0){IL(e+1620|0);LU(e+760|0);I5(d)}d=b+332|0;if((c[d>>2]|0)!=0){I9(d)}d=b+596|0;if((c[d>>2]|0)!=0){Ii(d)|0}d=b+604|0;if((c[d>>2]|0)!=0){Jx(d)}I1(b+592|0);I3(a);return}function Im(b,e,f,g,h,j,k){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0;l=i;i=i+1768|0;m=l|0;n=l+72|0;o=l+120|0;p=l+1216|0;q=l+1728|0;r=l+1744|0;s=l+1752|0;t=l+1760|0;if((b|0)==0){u=-1;i=l;return u|0}v=c[b+(e<<2)>>2]|0;if((v|0)==0){u=-1;i=l;return u|0}e=v+172|0;w=v+184|0;c[w>>2]=0;x=v+192|0;y=c[x>>2]|0;z=c[v+188>>2]|0;A=(y|0)==(z-1|0)|0;if((y|0)==0){c[w>>2]=1}do{if((z|0)==0){c[x>>2]=1}else{if((y|0)<=-1){break}c[x>>2]=(y+1|0)%(z|0)|0}}while(0);z=v+196|0;L10288:do{if((c[v+300>>2]|0)==0){B=A;C=v+16|0}else{y=v+324|0;c[v+328>>2]=c[y>>2];x=v+320|0;c[y>>2]=c[x>>2];y=v+316|0;D=c[y>>2]|0;c[x>>2]=D;x=v+312|0;E=c[x>>2]|0;c[y>>2]=E;c[x>>2]=c[v+308>>2];x=(E|0)>(D|0)?D:E;E=v+16|0;D=v+48|0;if((c[D>>2]|0)==(x|0)){B=A;C=E;break}y=v+52|0;F=c[y>>2]|0;G=ac(x<<2,F)|0;H=v+36|0;I=((G|0)/(c[H>>2]|0)|0)+1>>1;G=v+68|0;J=v+32|0;K=c[J>>2]|0;L=0;while(1){if((L|0)>=(K|0)){break}if((I|0)==(d[(c[G>>2]|0)+L|0]|0)){break}else{L=L+1|0}}c[D>>2]=x;c[v+112>>2]=L;c[w>>2]=1;if((In(E,v+100|0,F)|0)!=0){u=1;i=l;return u|0}I=c[v+20>>2]|0;if((I|0)<=0){B=1;C=E;break}K=v+116|0;M=v+88|0;N=v+64|0;O=v+60|0;P=v+24|0;Q=v+28|0;R=0;while(1){S=c[v+(R<<2)>>2]|0;c[S+3744>>2]=c[K>>2];if((IK(S+1620|0,c[M>>2]|0,d[c[N>>2]|0]|0,c[G>>2]|0,c[J>>2]|0,c[H>>2]|0,O,P,c[y>>2]|0)|0)!=0){u=1;break}T=c[S+3740>>2]|0;c[S+632>>2]=T;c[S+636>>2]=T;c[S+496>>2]=c[P>>2];c[S+500>>2]=c[Q>>2];c[S+5860>>2]=c[Q>>2];c[S+492>>2]=0;c[S+628>>2]=0;S=R+1|0;if((S|0)<(I|0)){R=S}else{B=1;C=E;break L10288}}i=l;return u|0}}while(0);w=b+576|0;A=C|0;E=LL(z,v+348+(c[w>>2]<<8)|0,256,m,c[A>>2]|0)|0;I_(o,548);I_(o+548|0,548);I_(n,44);c[n>>2]=1;c[n+4>>2]=1;c[n+8>>2]=1;c[n+12>>2]=1;c[n+16>>2]=1;c[n+20>>2]=1;c[n+24>>2]=1;c[n+28>>2]=1;c[n+32>>2]=1;c[n+36>>2]=1;R=o|0;do{if((k|0)==0){I=v+20|0;if((c[I>>2]|0)<=0){break}Q=v+344|0;P=p|0;y=q|0;O=s|0;H=v+346|0;J=s+4|0;G=v+347|0;N=b+596|0;M=v+8|0;K=b+600|0;F=v+100|0;L=0;x=0;while(1){D=c[v+(x<<2)>>2]|0;S=D+760|0;T=a[Q]|0;if(T<<24>>24==0){Lj(c[v+8+(x<<2)>>2]|0,S|0,D+888|0,q,f+(d[v+346+x|0]<<1)|0,g,P);c[D+6344>>2]=(c[y>>2]|0)+7;U=a[Q]|0}else{U=T}if(U<<24>>24!=0&(L|0)==0){c[O>>2]=f+(d[H]<<1);c[J>>2]=f+(d[G]<<1);T=Ik(c[N>>2]|0,O,g,M,S|0,D+888|0,f+(d[v+346+x|0]<<1)|0,K,r,B)|0;c[D+6344>>2]=a[r]|0;V=T}else{V=L}LN(C,F,e,D|0,z,o+(x*548|0)|0,n);D=x+1|0;if((D|0)<(c[I>>2]|0)){L=V;x=D}else{break}}}}while(0);if((a[v+344|0]|0)==0){W=0}else{W=c[b+596>>2]|0}LO(C,v+100|0,W,e,c[v>>2]|0,c[v+4>>2]|0,z,R,n,k);LM(z,m,E,c[A>>2]|0);A=v+208|0;E=v+216|0;if((c[v+248>>2]|0)==0){Hy(E,c[v+212>>2]|0,0);X=A|0}else{m=A|0;Hw(E,c[m>>2]|0,c[v+212>>2]|0);X=m}c[v+212>>2]=0;c[X>>2]=0;X=HA(E)|0;c[v+1116+(c[w>>2]<<2)>>2]=X;X=v+1116+(c[w>>2]<<2)|0;if((c[X>>2]|0)>>>0>2048){c[X>>2]=0}if((j|0)!=0){X=v+1116|0;c[h>>2]=c[X>>2];IY(j,v+348|0,((c[X>>2]|0)+7|0)>>>3)}if((c[b+572>>2]|0)==0){u=0;i=l;return u|0}X=c[v+20>>2]|0;if((X|0)<=0){u=0;i=l;return u|0}j=b+560|0;h=v+40|0;w=b+552|0;b=0;while(1){E=d[v+346+b|0]|0;Mm((c[v+(b<<2)>>2]|0)+6352|0,f+(E+(c[j>>2]|0)<<1)|0,c[h>>2]|0,g,f+(E<<1)|0,t,c[w>>2]|0)|0;E=b+1|0;if((E|0)<(X|0)){b=E}else{u=0;break}}i=l;return u|0}function In(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;f=i;i=i+16|0;g=f|0;h=f+8|0;j=a+20|0;k=b+60|0;if((Iz(c[j>>2]|0,e,c[b+4>>2]|0,c[b+8>>2]|0,c[k>>2]|0,g,h)|0)!=0){l=1;i=f;return l|0}m=a+52|0;n=a+16|0;if((IA(c[m>>2]|0,n,c[g>>2]|0,c[h>>2]|0,c[b+56>>2]|0,c[b+52>>2]|0)|0)!=0){l=1;i=f;return l|0}h=b+12|0;c[h>>2]=0;b=a+48|0;g=a+12|0;if((IC(c[b>>2]|0,g,c[m>>2]|0,c[n>>2]|0,h,c[k>>2]|0,e)|0)!=0){l=1;i=f;return l|0}k=a+44|0;ID(c[k>>2]|0,a+8|0,c[b>>2]|0,c[g>>2]|0);c[a+28>>2]=((ac(d[c[k>>2]|0]|0,c[j>>2]|0)|0)/(e|0)|0)+1>>1;l=0;i=f;return l|0}function Io(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;f=i;i=i+8|0;g=f|0;c[g>>2]=0;L10351:do{if((a|0)!=0){h=I2(0)|0;c[g>>2]=h;if((h|0)==0){break}I_(h,668);h=I0(0)|0;c[(c[g>>2]|0)+592>>2]=h;h=c[g>>2]|0;c[h+588>>2]=c[h+592>>2];h=0;while(1){if((h|0)>=(b|0)){j=0;break}k=I6(h)|0;c[(c[g>>2]|0)+(h<<2)>>2]=k;k=c[(c[g>>2]|0)+(h<<2)>>2]|0;if((k|0)==0){break L10351}I_(k,1128);k=Je(h)|0;c[(c[(c[g>>2]|0)+(h<<2)>>2]|0)+60>>2]=k;k=Jg(h)|0;c[(c[(c[g>>2]|0)+(h<<2)>>2]|0)+64>>2]=k;k=Ji(h)|0;c[(c[(c[g>>2]|0)+(h<<2)>>2]|0)+68>>2]=k;k=c[(c[g>>2]|0)+(h<<2)>>2]|0;if((c[k+60>>2]|0)==0){break L10351}if((c[k+64>>2]|0)==0){break L10351}if((c[k+68>>2]|0)==0){break L10351}else{h=h+1|0}}while(1){if((j|0)>=(d|0)){break}h=I4(j)|0;c[(c[g>>2]|0)+24+(j<<2)>>2]=h;h=c[g>>2]|0;k=c[h+24+(j<<2)>>2]|0;if((k|0)==0){break L10351}l=c[h+588>>2]|0;I_(k,6352);if((IH(k+1620|0,j)|0)!=0){break L10351}if((LS(k+760|0,j,0,l)|0)==0){j=j+1|0}else{break L10351}}l=(e|0)!=0;k=l?2:0;h=(k|0)<(d|0)?d:k;k=0;while(1){if((k|0)>=(h|0)){break}m=I8(k)|0;c[(c[g>>2]|0)+48+(k*56|0)+4>>2]=m;if((c[(c[g>>2]|0)+48+(k*56|0)+4>>2]|0)==0){break L10351}else{k=k+1|0}}k=c[g>>2]|0;if(l){if((Ig(k+596|0)|0)!=0){break}h=Jw(0)|0;c[(c[g>>2]|0)+604>>2]=h;h=c[g>>2]|0;if((c[h+604>>2]|0)==0){break}else{n=h}}else{n=k}c[a>>2]=n;o=0;i=f;return o|0}}while(0);Il(g);o=1;i=f;return o|0}function Ip(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0;f=i;i=i+8|0;g=f|0;do{if((e|0)==29){if((b|0)==2){h=(Iq(a,g)|0)==-1;j=h?c[g>>2]|0:a;k=1;break}else{l=0;i=f;return l|0}}else{j=a;k=b}}while(0);if((Ir(j,k,d,e,g)|0)==-1){l=c[g>>2]|0;i=f;return l|0}else{c[g>>2]=j;l=j;i=f;return l|0}return 0}function Iq(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;d=c[22658]|0;e=d>>>0>a>>>0;f=c[22659]|0;if(f>>>0>a>>>0&(e^1)){g=0;return g|0}h=e&d>>>0<5e6;e=h?d:5e6;d=f>>>0<=a>>>0;i=(f|0)!=0;j=d&i?f-1|0:0;f=c[22663]|0;k=f>>>0>a>>>0;l=c[22664]|0;if(l>>>0>a>>>0&(k^1)){g=1;return g|0}m=k&f>>>0<e>>>0;k=m?f:e;e=l>>>0<=a>>>0&l>>>0>j>>>0;f=e?l-1|0:j;j=c[22668]|0;l=j>>>0>a>>>0;n=c[22669]|0;if(n>>>0>a>>>0&(l^1)){g=2;return g|0}o=l&j>>>0<k>>>0;l=o?j:k;k=n>>>0<=a>>>0&n>>>0>f>>>0;j=k?n-1|0:f;f=c[22673]|0;n=f>>>0>a>>>0;p=c[22674]|0;if(p>>>0>a>>>0&(n^1)){g=3;return g|0}q=n&f>>>0<l>>>0;n=q?3:o?2:m?1:(h^1)<<31>>31;h=p>>>0<=a>>>0&p>>>0>j>>>0;m=h?3:k?2:e?1:(d&i^1)<<31>>31;if((b|0)==0){g=-1;return g|0}if((n|0)>-1){r=(c[80480+(n*28|0)>>2]|0)-a|0}else{r=5e6}if((m|0)>-1){s=a-(c[80484+(m*28|0)>>2]|0)|0}else{s=5e6}if((s|0)<(r|0)){c[b>>2]=h?p-1|0:j;g=-1;return g|0}else{c[b>>2]=q?f:l;g=-1;return g|0}return 0}function Ir(a,b,f,g,h){a=a|0;b=b|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;i=(g|0)==39;g=i?150:124;j=i?124:0;i=-1;k=-1;l=0;m=0;n=5e6;L10417:while(1){do{if((d[80490+(j*28|0)|0]|0|0)==(b|0)){if((e[80488+(j*28|0)>>1]|0|0)!=(f|0)){o=n;p=m;q=l;r=k;s=i;break}t=c[80480+(j*28|0)>>2]|0;u=t>>>0>a>>>0;v=c[80484+(j*28|0)>>2]|0;if(v>>>0>a>>>0&(u^1)){w=j;x=7511;break L10417}y=u&t>>>0<n>>>0;u=v>>>0<=a>>>0&v>>>0>m>>>0;o=y?t:n;p=u?v-1|0:m;q=1;r=u?j:k;s=y?j:i}else{o=n;p=m;q=l;r=k;s=i}}while(0);y=j+1|0;if((y|0)<(g|0)){j=y;i=s;k=r;l=q;m=p;n=o}else{break}}if((x|0)==7511){return w|0}if((h|0)==0){w=-1;return w|0}if((q|0)==0){c[h>>2]=0;w=-1;return w|0}if((s|0)>-1){z=(c[80480+(s*28|0)>>2]|0)-a|0}else{z=5e6}if((r|0)>-1){A=a-(c[80484+(r*28|0)>>2]|0)|0}else{A=5e6}if((A|0)<(z|0)){c[h>>2]=p;w=-1;return w|0}else{c[h>>2]=o;w=-1;return w|0}return 0}function Is(b,f,g,h,j,k,l,m,n,o,p,q,r){b=b|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;r=r|0;var s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0;s=i;i=i+1160|0;t=s|0;u=s+8|0;v=s+1136|0;w=c[m>>2]|0;x=c[n>>2]|0;y=c[j>>2]|0;z=c[l>>2]|0;A=c[o>>2]|0;if((A|0)==29|(A|0)==156|(A|0)==137|(A|0)==145){B=1}else{B=0}if((A|0)==39){C=1;D=7518}else{if((A|0)==23){E=1}else{C=0;D=7518}}L10451:do{if((D|0)==7518){do{if((B|0)==0){F=0}else{if((z|0)==2&(g|0)==1){c[f>>2]=0;a[f+12|0]=1;a[f+13|0]=1;c[l>>2]=1;F=B;break}if((A|0)==137){c[o>>2]=136;F=0;break}else if((A|0)==156){c[o>>2]=132;F=0;break}else if((A|0)==145){c[o>>2]=144;F=0;break}else{c[o>>2]=5;F=0;break}}}while(0);G=(g|0)>0;L10464:do{if(G){H=w>>>1;L10466:do{if(w>>>0<16e3){I=0;while(1){if((c[f+(I<<4)>>2]|0)>>>0<2){break}I=I+1|0;if((I|0)>=(g|0)){break L10464}}J=c[o>>2]|0}else{I=0;while(1){if((c[f+(I<<4)>>2]|0)>>>0<2){K=c[f+(I<<4)+4>>2]|0;L=a[f+(I<<4)+13|0]|0;M=c[o>>2]|0;if((K|0)==0){N=(L&255)*24e3|0}else{N=K}K=(M|0)==39;O=K?150:124;P=K?124:0;L10478:while(1){do{if((a[80490+(P*28|0)|0]|0)==L<<24>>24){if((e[80488+(P*28|0)>>1]|0)!=(H|0)){break}if((c[80484+(P*28|0)>>2]|0)>>>0>N>>>0&(c[80480+(P*28|0)>>2]|0)>>>0<=N>>>0){break L10478}}}while(0);K=P+1|0;if((K|0)<(O|0)){P=K}else{J=M;break L10466}}if((P|0)==-1){J=M;break L10466}}I=I+1|0;if((I|0)>=(g|0)){break L10464}}}}while(0);switch(J|0){case 136:case 137:{c[o>>2]=135;E=1;break L10451;break};case 144:case 145:{c[o>>2]=143;E=1;break L10451;break};case 39:{E=1;break L10451;break};case 132:case 156:{c[o>>2]=129;E=1;break L10451;break};default:{c[o>>2]=2;E=1;break L10451}}}}while(0);c[m>>2]=(c[m>>2]|0)/2|0;do{if((C|0)==0){H=c[p>>2]<<1;I=x<<1;O=((I|0)/64|0)<<6;if((F|0)==0){Q=H+385-O|0;R=O+577|0;D=7550;break}L=O+1953|0;K=-31-O+H|0;c[p>>2]=L;if((K|0)>0){S=0;T=K;U=L}else{V=0;W=K;X=0;Y=L;D=7553;break}do{U=U+I|0;T=T-I|0;S=S+1|0;}while((T|0)>0);Z=T;_=S;$=0;aa=U}else{I=((x|0)/2|0)+x<<1;Q=(c[p>>2]<<1|1)-I|0;R=I+63|0;D=7550}}while(0);if((D|0)==7550){c[p>>2]=R;V=1;W=Q;X=(Q|0)>0;Y=R;D=7553}if((D|0)==7553){Z=W;_=0;$=V;aa=Y+(X?W:-W|0)|0}c[p>>2]=aa;if((Z|0)<0){ab=_;ad=0;ae=(ac(c[l>>2]|0,-Z|0)|0)/2|0}else{I=x<<1;L=(Z|0)>(I|0);ab=L?1:_;ad=ac(c[l>>2]|0,Z-(L?I:0)|0)|0;ae=0}I=b+576|0;c[I>>2]=ab;c[b+552>>2]=c[l>>2];L=b+556|0;c[L>>2]=c[n>>2]<<1;K=b+572|0;c[K>>2]=$;H=b+580|0;c[H>>2]=0;O=b+584|0;c[O>>2]=0;do{if(G){af=(F|0)!=0|0;ag=x<<1;ah=ag>>>0>2048;ai=9999;aj=9999;ak=-1;al=0;while(1){if((c[f+(al<<4)>>2]|0)>>>0<2){am=ak+1|0;a[f+(al<<4)+12|0]=af;if(!ah){c[u+(am*188|0)+20>>2]=1e3;c[u+(am*188|0)+24>>2]=0;c[u+(am*188|0)+28>>2]=0;c[u+(am*188|0)+32>>2]=1;c[u+(am*188|0)+52>>2]=13e3;c[u+(am*188|0)+36>>2]=1;c[u+(am*188|0)+64>>2]=ag;c[u+(am*188|0)+68>>2]=0;c[u+(am*188|0)+72>>2]=1;c[u+(am*188|0)+80>>2]=0;c[u+(am*188|0)+84>>2]=1;c[u+(am*188|0)+88>>2]=1;c[u+(am*188|0)+92>>2]=0;c[u+(am*188|0)+96>>2]=1;c[u+(am*188|0)+104>>2]=1;c[u+(am*188|0)+108>>2]=644245120;c[u+(am*188|0)+112>>2]=644245120;c[u+(am*188|0)+116>>2]=4;c[u+(am*188|0)+120>>2]=2;c[u+(am*188|0)+124>>2]=2;c[u+(am*188|0)+128>>2]=0;c[u+(am*188|0)+140>>2]=0;c[u+(am*188|0)+144>>2]=0;c[u+(am*188|0)+152>>2]=0;c[u+(am*188|0)+100>>2]=3;c[u+(am*188|0)+76>>2]=6;c[u+(am*188|0)+56>>2]=0;c[u+(am*188|0)+132>>2]=5;c[u+(am*188|0)+136>>2]=9;a[u+(am*188|0)+156|0]=2;c[u+(am*188|0)+160>>2]=1;c[u+(am*188|0)+164>>2]=2;c[u+(am*188|0)+168>>2]=2;c[u+(am*188|0)+172>>2]=2;c[u+(am*188|0)+176>>2]=1;c[u+(am*188|0)+180>>2]=0}an=c[f+(al<<4)+4>>2]|0;ao=a[f+(al<<4)+13|0]|0;ap=ao&255;aq=c[m>>2]|0;ar=c[o>>2]|0;c[u+(am*188|0)>>2]=an;c[u+(am*188|0)+4>>2]=ap;c[u+(am*188|0)+8>>2]=aq;c[u+(am*188|0)+12>>2]=q;c[u+(am*188|0)+16>>2]=24e3;as=aq<<1;do{if((an|0)==0){at=ap*24e3|0;if(ao<<24>>24!=1){au=at;break}if(!((as|0)==48e3|(as|0)==44100)){au=at;break}au=32e3}else{au=an}}while(0);an=(ar|0)==39;as=an?150:124;ap=an?124:0;L10526:while(1){do{if((a[80490+(ap*28|0)|0]|0)==ao<<24>>24){if((e[80488+(ap*28|0)>>1]|0)!=(aq|0)){break}if((c[80484+(ap*28|0)>>2]|0)>>>0>au>>>0&(c[80480+(ap*28|0)>>2]|0)>>>0<=au>>>0){D=7573;break L10526}}}while(0);M=ap+1|0;if((M|0)<(as|0)){ap=M}else{break}}do{if((D|0)==7573){D=0;if((ap|0)==-1){break}c[u+(am*188|0)+132>>2]=d[80491+(ap*28|0)|0]|0;c[u+(am*188|0)+136>>2]=d[80493+(ap*28|0)|0]|0;c[u+(am*188|0)+164>>2]=d[80495+(ap*28|0)|0]|0;if(an){a[u+(am*188|0)+184|0]=0}c[u+(am*188|0)+56>>2]=d[80496+(ap*28|0)|0]|0;c[u+(am*188|0)+76>>2]=a[80497+(ap*28|0)|0]|0;c[u+(am*188|0)+100>>2]=c[80500+(ap*28|0)>>2];a[u+(am*188|0)+156|0]=a[80504+(ap*28|0)|0]|0;if(an){if(au>>>0<28e3){a[u+(am*188|0)+184|0]=1}c[u+(am*188|0)+20>>2]=-1}do{if(ao<<24>>24==1){if(au>>>0>=16e3){break}c[u+(am*188|0)+36>>2]=0}else{if(au>>>0>=2e4){break}c[u+(am*188|0)+36>>2]=0}}while(0);c[u+(am*188|0)+60>>2]=0;c[u+(am*188|0)+148>>2]=F}}while(0);ao=c[u+(am*188|0)+132>>2]|0;an=c[u+(am*188|0)+136>>2]|0;av=am;aw=(aj|0)<(an|0)?aj:an;ax=(ai|0)<(ao|0)?ai:ao}else{av=ak;aw=aj;ax=ai}ao=al+1|0;if((ao|0)<(g|0)){ai=ax;aj=aw;ak=av;al=ao}else{break}}al=av+1|0;ak=b+548|0;c[ak>>2]=al;aj=b+544|0;c[aj>>2]=-1;if(G){ai=0;ag=-1;ah=0;af=0;while(1){ao=f+(ai<<4)|0;an=c[ao>>2]|0;if((an|0)==3){c[aj>>2]=d[f+(ai<<4)+14|0]|0;ay=af;az=ah;aA=ag}else if((an|0)==0|(an|0)==1){an=ag+1|0;ap=c[b+(an<<2)>>2]|0;as=f+(ai<<4)+13|0;if((a[as]|0)==0){aB=af}else{aq=af;ar=0;while(1){c[ap+(ar<<2)>>2]=c[b+24+(aq<<2)>>2];M=aq+1|0;P=ar+1|0;if((P|0)<(d[as]|0)){aq=M;ar=P}else{aB=M;break}}}ar=f+(ai<<4)+12|0;aq=0;am=ah;while(1){if((a[ar]|0)==0){aC=d[as]|0}else{aC=2}if((aq|0)>=(aC|0)){break}a[ap+346+aq|0]=a[f+(ai<<4)+14+aq|0]|0;c[ap+8+(aq<<2)>>2]=b+48+(am*56|0);aq=aq+1|0;am=am+1|0}c[ap+332>>2]=c[ao>>2];c[ap+340>>2]=c[f+(ai<<4)+8>>2];a[ap+345|0]=a[as]|0;a[ap+344|0]=a[ar]|0;ay=aB;az=am;aA=an}else{ay=af;az=ah;aA=ag}aq=ai+1|0;if((aq|0)<(g|0)){ai=aq;ag=aA;ah=az;af=ay}else{break}}aD=c[ak>>2]|0}else{aD=al}if((aD|0)<=0){aE=99999;aF=aD;aG=ak;break}af=b+588|0;ah=t;ag=t+4|0;ai=(r|0)==0;aj=99999;aq=0;while(1){M=u+(aq*188|0)|0;P=u+(aq*188|0)+132|0;c[P>>2]=ax;at=u+(aq*188|0)+136|0;c[at>>2]=aw;aH=c[b+(aq<<2)>>2]|0;aI=c[af>>2]|0;aJ=u+(aq*188|0)+4|0;if(((c[aJ>>2]|0)-1|0)>>>0>1){E=1;break L10451}aK=c[o>>2]|0;aL=aH+348+(ab<<8)|0;aM=aH+16|0;aN=aM|0;c[aN>>2]=0;if((aK|0)==145|(aK|0)==144){c[aN>>2]=14;aO=14}else if((aK|0)==39){c[aN>>2]=1;aO=5}else{aO=4}if((c[u+(aq*188|0)+28>>2]|0)!=0){c[aN>>2]=aO}aK=aH+52|0;c[aK>>2]=64;aP=u+(aq*188|0)+64|0;aQ=aH+56|0;c[aQ>>2]=(c[aP>>2]|0)/64|0;Ht(aH+216|0,aL,256,0);c[aH+212>>2]=0;c[aH+208>>2]=0;c[aH+248>>2]=1;aL=aH+20|0;c[aL>>2]=c[aJ>>2];if((c[aJ>>2]|0)==2){c[aH+72>>2]=c[u+(aq*188|0)+100>>2]}else{c[aH+72>>2]=0}aJ=c[aP>>2]|0;aR=aH+40|0;c[aR>>2]=aJ;aS=aH+100|0;c[aH+160>>2]=1;aT=c[u+(aq*188|0)+8>>2]<<1;aU=aH+36|0;c[aU>>2]=aT;c[aH+192>>2]=0;aV=c[u+(aq*188|0)+20>>2]|0;if((aV|0)>0){aW=(ac(aT,aV)|0)/(aJ*1e3|0|0)|0;c[aH+188>>2]=(aW|0)>1?aW:1}else{c[aH+188>>2]=0}aW=aH+120|0;c[aW>>2]=c[u+(aq*188|0)+68>>2];c[aH+184>>2]=0;aJ=c[P>>2]|0;P=aH+104|0;c[P>>2]=aJ;c[aH+108>>2]=c[at>>2];c[aH+112>>2]=0;c[aH+132>>2]=0;at=u+(aq*188|0)+124|0;if((c[at>>2]|0)!=2){c[aW>>2]=1}aW=aS|0;c[aW>>2]=c[u+(aq*188|0)+72>>2];aV=u+(aq*188|0)+156|0;c[aH+156>>2]=d[aV]|0;aX=u+(aq*188|0)+160|0;c[aH+152>>2]=c[aX>>2];aY=u+(aq*188|0)+164|0;aZ=aH+116|0;c[aZ>>2]=c[aY>>2];a_=aH+124|0;c[a_>>2]=0;do{if((a[aV]|0)==2){if((c[aX>>2]|0)!=1){D=7620;break}if((c[aY>>2]|0)!=2){D=7620}}else{D=7620}}while(0);if((D|0)==7620){D=0;c[a_>>2]=1}aY=u+(aq*188|0)+168|0;c[aH+136>>2]=c[aY>>2];aX=u+(aq*188|0)+172|0;c[aH+140>>2]=(aT|0)>48e3&(aJ|0)>8?3:c[aX>>2]|0;aV=u+(aq*188|0)+176|0;c[aH+144>>2]=c[aV>>2];an=u+(aq*188|0)+180|0;c[aH+148>>2]=c[an>>2];am=aH+128|0;c[am>>2]=0;do{if((c[aY>>2]|0)==2){if((c[aX>>2]|0)!=2){D=7625;break}if((c[aV>>2]|0)!=1){D=7625;break}if((c[an>>2]|0)!=0){D=7625}}else{D=7625}}while(0);if((D|0)==7625){D=0;c[am>>2]=1}c[aH+80>>2]=c[u+(aq*188|0)+24>>2];an=u+(aq*188|0)+36|0;c[aH+84>>2]=c[an>>2];if((In(aM,aS,c[aK>>2]|0)|0)!=0){E=1;break L10451}L10609:do{if((c[aL>>2]|0)>0){aV=u+(aq*188|0)+96|0;aX=u+(aq*188|0)+120|0;aY=aH+92|0;aJ=u+(aq*188|0)+76|0;aT=u+(aq*188|0)+56|0;a_=u+(aq*188|0)+60|0;ar=u+(aq*188|0)+116|0;ap=u+(aq*188|0)+80|0;as=aH+24|0;ao=u+(aq*188|0)+104|0;a$=u+(aq*188|0)+108|0;a0=u+(aq*188|0)+112|0;a1=u+(aq*188|0)+184|0;a2=aH+96|0;a3=u+(aq*188|0)+88|0;a4=u+(aq*188|0)+92|0;a5=aH+88|0;a6=aH+28|0;a7=0;while(1){a8=c[aH+(a7<<2)>>2]|0;c[t>>2]=3;c[t+4>>2]=3;a9=1<<c[aV>>2];ba=a8+4736|0;bb=a8+4740|0;c[bb>>2]=1;a[a8+6348|0]=0;bc=a8+6342|0;a[bc]=c[aN>>2]&1;bd=c[aX>>2]|0;be=a8+4788|0;c[be>>2]=bd;if((bd|0)==4){c[aY>>2]=1;c[be>>2]=0}else{c[aY>>2]=0}c[ba>>2]=c[at>>2];c[a8+6060>>2]=(c[an>>2]|0)!=0;c[a8+6064>>2]=0;be=640-(c[aK>>2]|0)|0;bd=c[aP>>2]|0;switch(bd|0){case 2048:case 1024:{bf=16;break};case 1920:case 960:{bf=15;break};case 1152:{bf=9;break};case 2304:{bf=18;break};default:{E=1;break L10451}}bg=(c[aQ>>2]|0)/(bf|0)|0;bh=a8+1620|0;if((II(bd,bh,aM,bf,c[at>>2]|0,c[aJ>>2]|0,c[aZ>>2]|0,c[aT>>2]|0,c[a_>>2]|0)|0)!=0){E=1;break L10451}bd=c[a8+3740>>2]|0;c[a8+6044>>2]=bd;c[ah>>2]=bd;c[ag>>2]=bd;bd=c[ar>>2]|0;bi=a8+4744|0;c[bi>>2]=bd;if((bd|0)==4){c[bi>>2]=2;c[bh>>2]=1}else{c[bh>>2]=0}bh=c[ap>>2]|0;if((bh|0)==0){bi=Ix(c[P>>2]|0,64,c[aU>>2]|0)|0;bj=(bi|0)>5e3?5e3:bi}else{bj=bh}bh=c[aK>>2]|0;bi=ac(bj<<2,bh)|0;bd=((bi|0)/(c[aU>>2]|0)|0)+1>>1;bi=c[aN>>2]|0;do{if((bi&1|0)==0){bk=c[aP>>2]|0;if((bk|0)==2048){bl=0;bm=(bg<<2)+8|0;break}else if((bk|0)==1920){bl=0;bm=(bg<<2)+7|0;break}else{E=1;break L10451}}else{bl=3;bm=3}}while(0);if((LT(a8+760|0,c[aQ>>2]|0,bh,be,bf,bg,bm,r,a7,aI,bi)|0)!=0){E=1;break L10451}bk=a8+488|0;if((LK(bk,as,c[ao>>2]|0,c[a$>>2]|0,c[a0>>2]|0)|0)!=0){E=1;break L10451}bn=a8+624|0;if((LK(bn,ah,c[ao>>2]|0,0,0)|0)!=0){E=1;break L10451}a[a2]=a[a1]|0;if((LI(ba,bk,bn,c[aW>>2]|0)|0)!=0){E=1;break L10451}LZ(a8+1240|0,c[a3>>2]|0,a9,c[a4>>2]|0,bf,c[bb>>2]|0,d[bc]|0);if((IO(a8|0,c[aR>>2]|0,c[aU>>2]|0,M,bd,c[aQ>>2]|0,c[aK>>2]|0,c[a8+1208>>2]|0,c[a8+1212>>2]|0,bl,bm)|0)!=0){E=1;break L10451}c[a5>>2]=c[at>>2];c[a8+5860>>2]=c[a6>>2];c[a8+5864>>2]=0;a7=a7+1|0;if((a7|0)>=(c[aL>>2]|0)){break L10609}}}}while(0);at=aH+344|0;L10642:do{if(ai){M=0;while(1){if((a[at]|0)==0){bo=c[aL>>2]|0}else{bo=2}if((M|0)>=(bo|0)){break L10642}aR=c[aH+8+(M<<2)>>2]|0;aW=c[aK>>2]|0;Ll(aR,c[aR+4>>2]|0,c[aQ>>2]|0,aW,aW,aW,c[aN>>2]<<2&4|8)|0;M=M+1|0}}else{M=0;while(1){if((a[at]|0)==0){bp=c[aL>>2]|0}else{bp=2}if((M|0)>=(bp|0)){break L10642}aW=c[aH+8+(M<<2)>>2]|0;aR=c[aK>>2]|0;Ll(aW,c[aW+4>>2]|0,c[aQ>>2]|0,aR,aR,aR,c[aN>>2]<<2&4)|0;M=M+1|0}}}while(0);aN=c[aH+44>>2]|0;c[aH+296>>2]=aN;if((c[u+(aq*188|0)+32>>2]|0)==0){bq=0}else{bq=(c[u+(aq*188|0)+144>>2]|0)!=0|0}c[aH+300>>2]=bq;aQ=c[aH+68>>2]|0;at=c[aK>>2]|0;M=ac(aN<<2,at)|0;aR=c[aU>>2]|0;aW=((M|0)/(aR|0)|0)+1>>1;M=c[aH+32>>2]|0;aI=0;aP=aW;while(1){if((aI|0)>=(M|0)){br=aI;break}P=(d[aQ+aI|0]|0)-aW|0;aZ=(P|0)>0?P:-P|0;if((aZ|0)<(aP|0)){aI=aI+1|0;aP=aZ}else{D=7666;break}}if((D|0)==7666){D=0;br=aI-1|0}aP=((ac(d[aQ+br|0]|0,aR)|0)/(at|0)|0)+1>>1;c[aH+308>>2]=aP;c[aH+312>>2]=aP;c[aH+316>>2]=aP;c[aH+320>>2]=aP;c[aH+324>>2]=aP;c[aH+328>>2]=aP;c[aH+304>>2]=c[aL>>2];c[aH+48>>2]=aN;aP=(aj|0)<(aN|0)?aj:aN;aW=aq+1|0;M=c[ak>>2]|0;if((aW|0)<(M|0)){aj=aP;aq=aW}else{aE=aP;aF=M;aG=ak;break}}}else{ak=b+548|0;c[ak>>2]=0;c[b+544>>2]=-1;aE=99999;aF=0;aG=ak}}while(0);if((c[K>>2]|0)==0){bs=ad;bt=ae}else{if((aF|0)>0){G=(aE*2e3|0|0)/(w|0)|0;ak=0;aq=aF;while(1){bu=c[b+(ak<<2)>>2]|0;aj=bu+345|0;if((a[aj]|0)==0){bv=aq}else{ai=0;do{Ml((c[bu+(ai<<2)>>2]|0)+6352|0,G,2)|0;ai=ai+1|0;}while((ai|0)<(d[aj]|0));bv=c[aG>>2]|0}aj=ak+1|0;if((aj|0)<(bv|0)){ak=aj;aq=bv}else{break}}bw=c[(c[bu>>2]|0)+6504>>2]|0}else{bw=0}aq=b+384|0;Ml(aq,0,2)|0;aq=c[l>>2]|0;do{if((ad|0)>0){D=7678}else{ak=(ac(aq,bw)|0)/2|0;if((ae|0)<=(ak|0)){D=7678;break}bx=0;by=ae-ak|0}}while(0);if((D|0)==7678){ak=bw-ae|0;G=(ac(aq,ak)|0)+ad|0;c[p>>2]=(c[p>>2]|0)+ak;bx=G;by=0}c[O>>2]=bw;bs=bx;bt=by}c[j>>2]=aE;c[H>>2]=(c[H>>2]|0)+((c[l>>2]|0)*2500|0);if((F|0)!=0){G=Iq(c[f+4>>2]|0,0)|0;c[v>>2]=c[n>>2];c[v+4>>2]=0;ak=v+8|0;c[ak>>2]=0;if((G|0)==-1){E=1;break}K=c[90640+(G*20|0)>>2]|0;c[v+12>>2]=K;aj=c[90644+(G*20|0)>>2]|0;c[v+16>>2]=aj;c[v+20>>2]=c[90648+(G*20|0)>>2];G=ac(ac((c[m>>2]|0)*5|0,K)|0,aj)|0;c[H>>2]=(c[H>>2]|0)+((G|0)/(c[L>>2]|0)|0);G=b+600|0;aj=c[b+604>>2]|0;K=b|0;ai=c[K>>2]|0;ah=c[ai+56>>2]|0;ag=c[ai+52>>2]>>1;ai=(r|0)!=0?0:8;Ln(G,aj,ah,ag,ag,ag,ai)|0;c[ak>>2]=LV((c[c[K>>2]>>2]|0)+760|0)|0;ak=c[b+596>>2]|0;ai=c[K>>2]|0;K=c[ai+56>>2]|0;ag=c[ai+52>>2]|0;ai=c[b+588>>2]|0;Ih(ak,v,K,ag,ai)|0}ai=b+564|0;c[ai>>2]=bt;c[b+568>>2]=ac(c[l>>2]|0,x)|0;c[b+560>>2]=bs;ag=c[I>>2]|0;if((ag|0)>0){K=ag;ag=c[aG>>2]|0;while(1){if((ag|0)>0){ak=0;while(1){if((Im(b,ak,h+(c[ai>>2]<<1)|0,c[(c[b+(ak<<2)>>2]|0)+20>>2]|0,0,0,1)|0)!=0){E=-1;break L10451}Iu(b,h)|0;ah=ak+1|0;aj=c[aG>>2]|0;if((ah|0)<(aj|0)){ak=ah}else{bz=aj;break}}}else{bz=ag}ak=K-1|0;if((ak|0)>0){K=ak;ag=bz}else{break}}}c[n>>2]=x<<1;c[k>>2]=(bs|0)>(bt|0)?bs:bt;bA=0;i=s;return bA|0}}while(0);c[m>>2]=w;c[n>>2]=x;c[l>>2]=z;c[j>>2]=y;bA=E;i=s;return bA|0}function It(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;g=i;i=i+8|0;h=g|0;j=a+548|0;k=c[j>>2]|0;L10704:do{if((k|0)>0){l=a+564|0;m=0;n=k;while(1){if((c[a+(m<<2)>>2]|0)==0){o=n}else{p=Im(a,m,b+(c[l>>2]<<1)|0,d,e+(m<<2)|0,f+(m<<8)|0,0)|0;if((p|0)!=0){q=p;break}o=c[j>>2]|0}p=m+1|0;if((p|0)<(o|0)){m=p;n=o}else{break L10704}}i=g;return q|0}}while(0);o=c[a+544>>2]|0;if((o|0)==-1){q=0;i=g;return q|0}if((c[a+572>>2]|0)==0){q=0;i=g;return q|0}j=(c[a+564>>2]|0)+o|0;Mm(a+384|0,b+(j+(c[a+560>>2]|0)<<1)|0,c[a+556>>2]|0,d,b+(j<<1)|0,h,c[a+552>>2]|0)|0;q=0;i=g;return q|0}function Iu(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=c[a+564>>2]|0;e=b;if((d|0)>0){IY(e,b+(c[a+568>>2]<<1)|0,d<<1)}else{d=b+((ac(c[a+556>>2]|0,c[a+552>>2]|0)|0)<<1)|0;IY(e,d,c[a+560>>2]<<1)}d=a+576|0;e=c[d>>2]|0;if((e|0)<=0){return 0}b=a+548|0;if((c[b>>2]|0)>0){f=0;g=e}else{return 0}while(1){e=a+(f<<2)|0;h=c[e>>2]|0;IZ(h+348|0,h+604|0,g<<8);h=c[e>>2]|0;IZ(h+1116|0,h+1120|0,c[d>>2]<<2);h=f+1|0;if((h|0)>=(c[b>>2]|0)){break}f=h;g=c[d>>2]|0}return 0}function Iv(a){a=a|0;var b=0;if((a|0)==0){b=0}else{b=c[a+580>>2]|0}return b|0}function Iw(a){a=a|0;var b=0;if((a|0)==0){b=-1}else{b=c[a+584>>2]|0}return b|0}function Ix(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;if(a>>>0>15){e=-1;return e|0}if((d|0)==88200|(d|0)==96e3){f=1913624+(a<<2)|0;g=7}else if((d|0)==24e3){f=1913816+(a<<2)|0;g=16}else if((d|0)==44100){h=12;i=7734}else if((d|0)==64e3){h=10;i=7734}else if((d|0)==32e3){f=1913752+(a<<2)|0;g=16}else if((d|0)==48e3){h=11;i=7734}else if((d|0)==16e3){f=1913944+(a<<2)|0;g=24}else if((d|0)==22050){f=1913880+(a<<2)|0;g=17}else{f=1913560+(a<<2)|0;g=11}if((i|0)==7734){f=1913688+(a<<2)|0;g=h}e=((ac((c[f>>2]|0)+g|0,d)|0)/(b|0)|0)+1>>1;return e|0}function Iy(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;d=i;i=i+560|0;e=d|0;f=d+56|0;g=d+112|0;h=d+168|0;j=d+224|0;k=d+280|0;l=d+336|0;m=d+392|0;n=d+448|0;o=d+504|0;p=f;NN(p|0,1914400,56)|0;p=g;NN(p|0,1914344,56)|0;p=h;NN(p|0,1914288,56)|0;p=j;NN(p|0,1914288,56)|0;p=k;NN(p|0,1914232,56)|0;p=l;NN(p|0,1914176,56)|0;p=m;NN(p|0,1914120,56)|0;p=n;NN(p|0,1914064,56)|0;p=o;NN(p|0,1914008,56)|0;if((a|0)==44100){q=k|0;r=23}else if((a|0)==88200){q=n|0;r=15}else if((a|0)==48e3){q=l|0;r=21}else if((a|0)==16e3){q=f|0;r=48}else if((a|0)==24e3){q=h|0;r=32}else if((a|0)==64e3){q=m|0;r=20}else if((a|0)==22050){q=g|0;r=35}else if((a|0)==96e3){q=o|0;r=13}else if((a|0)==32e3){q=j|0;r=32}else{q=0;r=21}j=c[q+4>>2]|0;a=e|0;c[a>>2]=j-(c[q>>2]|0);o=c[q+8>>2]|0;c[e+4>>2]=o-j;j=c[q+12>>2]|0;c[e+8>>2]=j-o;o=c[q+16>>2]|0;c[e+12>>2]=o-j;j=c[q+20>>2]|0;c[e+16>>2]=j-o;o=c[q+24>>2]|0;c[e+20>>2]=o-j;j=c[q+28>>2]|0;c[e+24>>2]=j-o;o=c[q+32>>2]|0;c[e+28>>2]=o-j;j=c[q+36>>2]|0;c[e+32>>2]=j-o;o=c[q+40>>2]|0;c[e+36>>2]=o-j;j=c[q+44>>2]|0;c[e+40>>2]=j-o;o=c[q+48>>2]|0;c[e+44>>2]=o-j;c[e+48>>2]=(c[q+52>>2]|0)-o;Mo(a,13);if((b|0)>0){s=r;t=0}else{u=r;i=d;return u|0}while(1){r=(c[e+(t<<2)>>2]|0)+s|0;a=t+1|0;if((a|0)<(b|0)){s=r;t=a}else{u=r;break}}i=d;return u|0}function Iz(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;if((a|0)==32e3){i=1913752+(d<<2)|0;j=16}else if((a|0)==88200|(a|0)==96e3){i=1913624+(d<<2)|0;j=7}else if((a|0)==16e3){i=1913944+(d<<2)|0;j=24}else if((a|0)==44100){k=12;l=7763}else if((a|0)==48e3){k=11;l=7763}else if((a|0)==64e3){k=10;l=7763}else if((a|0)==22050){i=1913880+(d<<2)|0;j=17}else if((a|0)==24e3){i=1913816+(d<<2)|0;j=16}else{i=1913560+(d<<2)|0;j=11}if((l|0)==7763){i=1913688+(d<<2)|0;j=k}k=(c[i>>2]|0)+j|0;c[g>>2]=k;do{if((f|0)==1){j=ac(b,a)|0;if((j|0)>=(ac(a<<1,k)|0)){break}return 1}}while(0);do{if((e|0)<14){f=Iy(a,e)|0;c[h>>2]=f;m=f}else{if((e|0)==14){f=k<<1;c[h>>2]=f;m=f;break}else{f=k*3|0;c[h>>2]=f;m=f;break}}}while(0);if((m|0)>(b|0)){c[h>>2]=b;n=b}else{n=m}if((a|0)==44100){m=c[g>>2]|0;if((n-m|0)>35){return 1}else{o=m;p=n-o|0;q=(p|0)>48;r=p>>>31;s=q?1:r;return s|0}}m=c[g>>2]|0;if((a|0)<=47999){o=m;p=n-o|0;q=(p|0)>48;r=p>>>31;s=q?1:r;return s|0}if((n-m|0)>32){return 1}else{o=m;p=n-o|0;q=(p|0)>48;r=p>>>31;s=q?1:r;return s|0}return 0}function IA(b,d,e,f,g,h){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;j=i;i=i+320|0;k=j|0;l=k|0;m=k+116|0;n=(g|0)==3?8:(g|0)==2?10:(g|0)==1?12:0;o=(h|0)==0;do{if((g|0)>0){if((f<<2|0)<(e*9|0|0)){h=HK(f)|0;p=h-(HK(e)|0)|0;h=(ac((p>>>2|((p|0)<0|0?-1:0)<<30)<<1|0>>>31,n)|0)+16777216>>25<<1;IB(l,e,f,h);Mo(l,h);p=c[l>>2]|0;if((p|0)==0){q=1;i=j;return q|0}r=e&255;a[b]=r;L10824:do{if((h|0)>=1){s=1;t=r;u=p;while(1){v=u+(t&255)&255;a[b+s|0]=v;w=s+1|0;if((w|0)>(h|0)){break L10824}x=c[k+(s<<2)>>2]|0;s=w;t=v;u=x}}}while(0);c[d>>2]=h;y=h;break}p=e<<1;r=HK(p)|0;u=r-(HK(e)|0)|0;r=(ac((u>>>2|((u|0)<0|0?-1:0)<<30)<<1|0>>>31,n)|0)+16777216>>25<<1;u=HK(f)|0;t=u-(HK(p)|0)|0;u=t;s=(t|0)<0|0?-1:0;t=o?1073741824:825955264;x=o?0:0;N2(u,s,t,x)|0;x=(ac(G<<1|0>>>31,n)|0)+16777216>>25<<1;IB(l,e,p,r);Mo(l,r);t=c[l>>2]|0;if((t|0)==0){q=1;i=j;return q|0}s=e&255;a[b]=s;L10833:do{if((r|0)>=1){u=1;v=s;w=t;while(1){z=w+(v&255)&255;a[b+u|0]=z;A=u+1|0;if((A|0)>(r|0)){break L10833}B=c[k+(u<<2)>>2]|0;u=A;v=z;w=B}}}while(0);IB(m,p,f,x);Mo(m,x);t=c[k+(r-1<<2)>>2]|0;s=c[m>>2]|0;if((t|0)>(s|0)){h=t-s|0;t=k+(x+28<<2)|0;w=c[t>>2]|0;v=(w-s|0)/2|0;u=(h|0)>(v|0)?v:h;c[m>>2]=u+s;c[t>>2]=w-u;Mo(m,x)}u=p&255;a[b+r|0]=u;if((x|0)>=1){w=1;t=u;do{t=(c[k+(w+28<<2)>>2]|0)+(t&255)&255;a[b+(w+r)|0]=t;w=w+1|0;}while((w|0)<=(x|0))}w=x+r|0;c[d>>2]=w;y=w}else{w=f-e|0;if(o){C=1;D=w}else{C=2;D=((w|0)/2|0)+1|0}t=(D|0)/2|0;p=t<<1;u=ac(p,C)|0;s=w-u|0;if((p|0)>0){h=0;do{c[k+(h<<2)>>2]=C;h=h+1|0;}while((h|0)<(p|0));E=t<<1}else{E=0}h=(s|0)>0;r=h?-1:s>>>31;if((w|0)!=(u|0)){x=h?p-1|0:(s|0)<0?0:E;h=s;while(1){v=k+(x<<2)|0;c[v>>2]=(c[v>>2]|0)-r;v=h+r|0;if((v|0)==0){break}else{x=x+r|0;h=v}}}h=e&255;a[b]=h;if((p|0)>=1){r=1;x=h;do{x=(c[k+(r-1<<2)>>2]|0)+(x&255)&255;a[b+r|0]=x;r=r+1|0;}while((r|0)<=(p|0))}c[d>>2]=p;y=p}}while(0);q=(y|0)<1|0;i=j;return q|0}function IB(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;f=i;i=i+24|0;g=f|0;h=f+8|0;j=f+16|0;if((e|0)<1){i=f;return}k=b<<24;l=k;m=(k|0)<0|0?-1:0;k=b;n=1;while(1){o=HQ(d,b,g)|0;p=HQ(n,e,h)|0;q=HT(o,c[g>>2]|0,p,c[h>>2]|0,j)|0;N2(q,(q|0)<0|0?-1:0,l,m)|0;q=G<<1|0>>>31;p=c[j>>2]|0;o=p-23|0;if((o|0)>0){r=q<<o}else{r=q>>23-p}p=r+1>>1;c[a+(n-1<<2)>>2]=p-k;q=n+1|0;if((q|0)>(e|0)){break}else{k=p;n=q}}i=f;return}function IC(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0;k=c[h>>2]|0;l=(j|0)/(((i|0)==1?2:1)|0)|0;if((d[f+k|0]|0|0)>(l|0)|(k|0)>(g|0)){i=0;while(1){j=i+1|0;if((d[f+j|0]|0|0)<(l|0)&(j|0)<(g|0)){i=j}else{break}}c[h>>2]=i;m=i}else{m=k}c[e>>2]=g-m;m=c[h>>2]|0;if((m|0)>(g|0)){return 0}else{n=m;o=m}while(1){a[b+(n-o)|0]=a[f+n|0]|0;m=n+1|0;if((m|0)>(g|0)){break}n=m;o=c[h>>2]|0}return 0}function ID(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0;if((f&1|0)==0){c[d>>2]=(f|0)/2|0;if((f|0)<-1){return}else{g=0}do{a[b+g|0]=a[e+(g<<1)|0]|0;g=g+1|0;}while((g|0)<=(c[d>>2]|0));return}else{c[d>>2]=(f+1|0)/2|0;a[b]=a[e]|0;if((c[d>>2]|0)<1){return}else{h=1}do{a[b+h|0]=a[e+((h<<1)-1)|0]|0;h=h+1|0;}while((h|0)<=(c[d>>2]|0));return}}function IE(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0;g=i;i=i+2096|0;h=g|0;j=g+48|0;k=c[a+44>>2]|0;l=c[a+16>>2]|0;m=c[a+20>>2]|0;n=c[a+36>>2]|0;o=c[a+8>>2]|0;p=c[a+12>>2]|0;q=a+24|0;r=a+96|0;s=a+112|0;t=j|0;u=j+1024|0;v=c[a+4>>2]<<2;if((n|0)>0){w=0;do{x=w+m|0;IY(c[a+80+(w<<2)>>2]|0,c[a+80+(x<<2)>>2]|0,v);IY(c[a+64+(w<<2)>>2]|0,c[a+64+(x<<2)>>2]|0,v);w=w+1|0;}while((w|0)<(n|0))}IZ(r,a+96+(m<<2)|0,n<<2);I_(a+96+(k<<2)|0,l-k<<2);I_(s,v);if((e|0)<=0){i=g;return}v=a+32|0;s=(o|0)>0;l=h|0;n=h+36|0;m=h+12|0;r=h+24|0;w=h|0;x=h+4|0;y=a+28|0;z=h+16|0;A=h+28|0;B=h+20|0;C=h+32|0;D=h+40|0;h=t;E=u;u=0;while(1){F=c[v>>2]|0;do{if((h|0)==(t|0)){H=j+896|0;I=E+896|0;if(s){J=0}else{K=I;L=H;break}while(1){M=c[b+(J<<2)>>2]|0;c[j+(J+224<<2)>>2]=c[M+(u<<2)>>2];N=c[d+(J<<2)>>2]|0;c[j+(J+480<<2)>>2]=c[N+(u<<2)>>2];O=u+1|0;c[j+(J+192<<2)>>2]=c[M+(O<<2)>>2];c[j+(J+448<<2)>>2]=c[N+(O<<2)>>2];O=u+2|0;c[j+(J+160<<2)>>2]=c[M+(O<<2)>>2];c[j+(J+416<<2)>>2]=c[N+(O<<2)>>2];O=u+3|0;c[j+(J+128<<2)>>2]=c[M+(O<<2)>>2];c[j+(J+384<<2)>>2]=c[N+(O<<2)>>2];O=u+4|0;c[j+(J+96<<2)>>2]=c[M+(O<<2)>>2];c[j+(J+352<<2)>>2]=c[N+(O<<2)>>2];O=u+5|0;c[j+(J+64<<2)>>2]=c[M+(O<<2)>>2];c[j+(J+320<<2)>>2]=c[N+(O<<2)>>2];O=u+6|0;c[j+(J+32<<2)>>2]=c[M+(O<<2)>>2];c[j+(J+288<<2)>>2]=c[N+(O<<2)>>2];O=u+7|0;c[j+(J<<2)>>2]=c[M+(O<<2)>>2];c[j+(J+256<<2)>>2]=c[N+(O<<2)>>2];O=J+1|0;if((O|0)<(o|0)){J=O}else{K=I;L=H;break}}}else{K=E-128|0;L=h-128|0}}while(0);H=c[q>>2]|0;if((F|0)>(o-H|0)){P=u+1|0}else{I=a+112+(u<<2)|0;O=u+1|0;N=k;M=F;Q=H;while(1){H=M-2|0;R=L+(H<<2)|0;S=Q+2|0;T=(S|0)==0;if(T){U=0;V=7861}else{W=S;X=R;Y=0;while(1){Z=c[X>>2]|0;_=Z>>31^Z|Y;Z=W-1|0;if((Z|0)==0){break}else{W=Z;X=X+4|0;Y=_}}if((_|0)>-1){U=_;V=7861}else{$=-1}}if((V|0)==7861){V=0;Y=~U;X=0;while(1){W=Y<<1;if((W|0)<0){Y=W;X=X+1|0}else{$=X;break}}}X=($|0)<0?0:$;Y=K+(H<<2)|0;if(T){aa=0;V=7866}else{W=S;Z=Y;ab=0;while(1){ac=c[Z>>2]|0;ad=ac>>31^ac|ab;ac=W-1|0;if((ac|0)==0){break}else{W=ac;Z=Z+4|0;ab=ad}}if((ad|0)>-1){aa=ad;V=7866}else{ae=-1}}if((V|0)==7866){V=0;ab=~aa;Z=0;while(1){W=ab<<1;if((W|0)<0){ab=W;Z=Z+1|0}else{ae=Z;break}}}Z=(ae|0)<0?0:ae;ab=((X|0)<(Z|0)?X:Z)-1|0;Z=(ab|0)<0?0:ab;IF(R,S,Z);IF(Y,S,Z);ab=K7(l,L+(M<<2)|0,K+(M<<2)|0,Q)|0;W=c[n>>2]|0;T=c[m>>2]|0;if((W|0)==0){H=T>>2;ac=c[r>>2]|0;af=ac>>2;ag=c[w>>2]|0;ah=c[x>>2]|0;ai=ag;aj=(ag|0)<0|0?-1:0;ag=ah;ak=(ah|0)<0|0?-1:0;N2(ag,ak,ai,aj)|0;al=G>>1;am=af;an=0;ao=0;ap=H;aq=0;ar=0;as=ac;at=ah}else{ah=c[B>>2]|0;ac=T;H=(T|0)<0|0?-1:0;af=ah;aj=(ah|0)<0|0?-1:0;N2(af,aj,ac,H)|0;ah=G>>1;ai=c[r>>2]|0;ak=c[C>>2]|0;ag=ai;au=(ai|0)<0|0?-1:0;av=ak;aw=(ak|0)<0|0?-1:0;N2(av,aw,ag,au)|0;ak=ah-(G>>1)|0;ah=c[z>>2]|0;ax=c[x>>2]|0;ay=ah;az=(ah|0)<0|0?-1:0;ah=ax;aA=(ax|0)<0|0?-1:0;N2(ah,aA,ay,az)|0;az=ak-(G>>1)|0;N2(ag,au,af,aj)|0;ak=G>>1;N2(av,aw,ac,H)|0;ay=(G>>1)+ak|0;ak=c[A>>2]|0;aB=ak;aC=(ak|0)<0|0?-1:0;N2(aB,aC,ah,aA)|0;aC=ay-(G>>1)|0;ay=W;aB=(W|0)<0|0?-1:0;N2(ac,H,ay,aB)|0;H=(c[D>>2]|0)+1|0;ac=G>>H;W=az;ak=(az|0)<0|0?-1:0;N2(W,ak,af,aj)|0;az=(G<<1|0>>>31)+ac|0;ac=aC;aD=(aC|0)<0|0?-1:0;N2(ac,aD,av,aw)|0;aC=az+(G<<1|0>>>31)|0;N2(ag,au,ay,aB)|0;au=G>>H;N2(ac,aD,af,aj)|0;aj=G<<1|0>>>31;N2(W,ak,av,aw)|0;aw=au-(G<<1|0>>>31)+aj|0;aj=c[w>>2]|0;N2(ah,aA,ay,aB)|0;aB=G<<1|0>>>31;ay=aj;aA=(aj|0)<0|0?-1:0;aj=aB;ah=(aB|0)<0|0?-1:0;N2(aj,ah,ay,aA)|0;al=G>>H;am=aw;an=aD;ao=ac;ap=aC;aq=ak;ar=W;as=ai;at=ax}if((al|0)==0){c[(c[a+80+(N<<2)>>2]|0)+(u<<2)>>2]=0;c[(c[a+64+(N<<2)>>2]|0)+(u<<2)>>2]=0}else{ax=ap;ai=(ap|0)<0|0?-1:0;W=T;ak=(T|0)<0|0?-1:0;N2(W,ak,ax,ai)|0;ai=G;ax=am;ak=(am|0)<0|0?-1:0;W=as;T=(as|0)<0|0?-1:0;N2(W,T,ax,ak)|0;ak=G+ai|0;ai=c[z>>2]|0;ax=ai;T=(ai|0)<0|0?-1:0;ai=at;W=(at|0)<0|0?-1:0;N2(ai,W,ax,T)|0;T=G<<1|0>>>31;ax=T;aC=(T|0)<0|0?-1:0;N2(ax,aC,ar,aq)|0;aC=ak-G|0;ak=c[A>>2]|0;ax=ak;T=(ak|0)<0|0?-1:0;N2(ax,T,ai,W)|0;W=G<<1|0>>>31;ai=W;T=(W|0)<0|0?-1:0;N2(ai,T,ao,an)|0;T=aC-G|0;aC=(T|0)>0?T:-T|0;T=al>>1;ai=al;W=(al|0)<0|0?-1:0;ax=1125899904;ak=0;N2(ai,W,ax,ak)|0;ak=(G>>19)+T-aC|0;T=(ak|0)>0?ak:-ak|0;ak=aC;ax=(aC|0)<0|0?-1:0;aC=1125899904;W=0;N2(ak,ax,aC,W)|0;W=G<<1|0>>>31;do{if((W|0)==0){aE=-2;V=7879}else{aC=W>>31^W;if((aC|0)<=-1){aE=-3;V=7879;break}ax=~aC;aC=0;while(1){ak=ax<<1;if((ak|0)<0){ax=ak;aC=aC+1|0}else{break}}ax=aC-2|0;if((ax|0)<=0){aE=ax;V=7879;break}aF=W<<ax;aG=ax}}while(0);if((V|0)==7879){V=0;aF=W>>-aE;aG=aE}do{if((T|0)==0){aH=0}else{S=T>>31^T;if((S|0)<=-1){aH=-1;break}Y=~S;S=0;while(1){R=Y<<1;if((R|0)<0){Y=R;S=S+1|0}else{aH=S;break}}}}while(0);W=T<<aH;do{if((aF|0)<1|(W|0)==0){c[(c[a+80+(N<<2)>>2]|0)+(u<<2)>>2]=0}else{S=aG-aH+19|0;Y=(S|0)<31?S:31;if((Y|0)>=0){S=(HO(aF,W,16)|0)>>Y;c[(c[a+80+(N<<2)>>2]|0)+(u<<2)>>2]=S;break}S=-Y|0;Y=HO(aF,W,16)|0;do{if((Y|0)==0){aI=0}else{aC=Y>>31^Y;if((aC|0)<=-1){aI=-1;break}R=~aC;aC=0;while(1){X=R<<1;if((X|0)<0){R=X;aC=aC+1|0}else{aI=aC;break}}}}while(0);c[(c[a+80+(N<<2)>>2]|0)+(u<<2)>>2]=Y<<((aI|0)>(S|0)?S:aI)}}while(0);W=c[x>>2]|0;if((W|0)==0){V=7895}else{T=c[m>>2]|0;if((T|W|0)>-1|(T&W|0)<0){V=7895}else{aJ=u}}if((V|0)==7895){V=0;aJ=O}c[(c[a+64+(N<<2)>>2]|0)+(u<<2)>>2]=1-(aJ<<1&2)}W=ab+4+(Z+f<<1)|0;T=c[w>>2]>>((W|0)>31?31:W);W=a+96+(N<<2)|0;c[W>>2]=(c[W>>2]|0)+T;c[I>>2]=T+(c[I>>2]|0);T=M+p|0;W=c[y>>2]|0;if((T|0)>(o-W|0)){P=O;break}else{N=N+1|0;M=T;Q=W}}}if((P|0)<(e|0)){h=L;E=K;u=P}else{break}}i=g;return}function IF(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;if((d|0)==0){return}if((d|0)>0){e=(d|0)<31?d:31;f=b&3;if((f|0)==0){g=a}else{h=b&3;i=f;f=a;while(1){j=i-1|0;c[f>>2]=c[f>>2]<<e;if((j|0)==0){break}else{i=j;f=f+4|0}}g=a+(h<<2)|0}h=b>>2;if((h|0)==0){return}else{k=h;l=g}while(1){g=k-1|0;h=l+4|0;c[l>>2]=c[l>>2]<<e;f=l+8|0;c[h>>2]=c[h>>2]<<e;h=l+12|0;c[f>>2]=c[f>>2]<<e;c[h>>2]=c[h>>2]<<e;if((g|0)==0){break}else{k=g;l=l+16|0}}return}else{l=-d|0;d=(l|0)<31?l:31;l=b&3;if((l|0)==0){m=a}else{k=b&3;e=l;l=a;while(1){g=e-1|0;c[l>>2]=c[l>>2]>>d;if((g|0)==0){break}else{e=g;l=l+4|0}}m=a+(k<<2)|0}k=b>>2;if((k|0)==0){return}else{n=k;o=m}while(1){m=n-1|0;k=o+4|0;c[o>>2]=c[o>>2]>>d;b=o+8|0;c[k>>2]=c[k>>2]>>d;k=o+12|0;c[b>>2]=c[b>>2]>>d;c[k>>2]=c[k>>2]>>d;if((m|0)==0){break}else{n=m;o=o+16|0}}return}}function IG(b,e,f,g,h,i,j,k,l,m,n,o){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0;p=a[k+1|0]|0;q=d[k]|0;r=b+56|0;do{if((c[r>>2]|0)==0){if(p<<24>>24==0){s=0;break}if(((c[b+60>>2]|0)+q|0)<(c[j+4+(c[j>>2]<<2)>>2]|0)){c[r>>2]=0;s=1;break}else{c[r>>2]=1;s=0;break}}else{c[r>>2]=0;if(p<<24>>24==0){s=1;break}if(((c[b+60>>2]|0)+q|0)<(c[j+4+(c[j>>2]<<2)>>2]|0)){s=1;break}c[r>>2]=1;s=1}}while(0);if((c[b>>2]|0)!=0){r=c[b+48>>2]|0;L_(b+2136|0,b+80|0,b+96|0,b+368|0,r,(c[b+20>>2]|0)+r|0,s,e)}if((n|0)==2){n=b+80|0;r=b+368|0;L1(b+588|0,n,b+64|0,r,j,k,g,h,l,m,i,b+112|0);t=n;u=r}else{c[g>>2]=0;I_(h,m);t=b+80|0;u=b+368|0}L7(b+1864|0,j,f,t,u,c[g>>2]|0,c[b+40>>2]|0,c[b+20>>2]|0,s,b+2272|0,o);o=b+2260|0;if((c[o>>2]|0)>0){v=0}else{return}do{c[b+2272+(v<<2)>>2]=c[e+(v<<2)>>2];v=v+1|0;}while((v|0)<(c[o>>2]|0));return}function IH(a,b){a=a|0;b=b|0;var d=0,e=0;d=Ja(b)|0;e=Jc(b)|0;I_(a,3116);c[a+80>>2]=d;c[a+64>>2]=e;c[a+84>>2]=d+256;c[a+68>>2]=e+256;c[a+88>>2]=d+512;c[a+72>>2]=e+512;c[a+92>>2]=d+768;c[a+76>>2]=e+768;L2(a+588|0,b)|0;return 0}function II(a,b,e,f,g,h,i,j,k){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;l=e+40|0;m=c[l>>2]|0;n=c[e+20>>2]|0;o=c[e+36>>2]|0;p=e+44|0;q=d[c[p>>2]|0]|0;r=c[e+52>>2]|0;s=c[e+16>>2]|0;t=e+8|0;u=e|0;do{if((c[u>>2]&1|0)==0){if((f|0)==16){c[b+24>>2]=14;c[b+28>>2]=14;c[b+16>>2]=4;v=(c[l>>2]|0)/16|0;c[b+20>>2]=v;c[b+48>>2]=0;c[b+60>>2]=4;w=16;x=4;y=v;break}else if((f|0)==15){c[b+24>>2]=13;c[b+28>>2]=13;c[b+16>>2]=4;v=(c[l>>2]|0)/15|0;c[b+20>>2]=v;c[b+48>>2]=0;c[b+60>>2]=4;w=15;x=4;y=v;break}else{z=-1;return z|0}}else{if((f|0)==15){c[b+24>>2]=6;c[b+28>>2]=5;c[b+16>>2]=3;v=(c[l>>2]|0)/7|0;c[b+20>>2]=v;c[b+48>>2]=0;c[b+60>>2]=0;w=8;x=3;y=v;break}else if((f|0)==16){c[b+24>>2]=6;c[b+28>>2]=6;c[b+16>>2]=3;v=(c[l>>2]|0)/8|0;c[b+20>>2]=v;c[b+48>>2]=0;c[b+60>>2]=0;w=8;x=3;y=v;break}else{w=(c[b+24>>2]|0)+2|0;x=c[b+16>>2]|0;y=c[b+20>>2]|0;break}}}while(0);c[b+8>>2]=m;c[b+12>>2]=w;c[b+32>>2]=2;w=b+16|0;m=b+20|0;l=x-y|0;y=b+36|0;c[y>>2]=l;c[b+44>>2]=l;c[b+40>>2]=0;c[b+52>>2]=0;c[b+56>>2]=0;c[b+4>>2]=o;if((x|0)>0){x=o<<2;l=0;do{I_(c[b+80+(l<<2)>>2]|0,x);I_(c[b+64+(l<<2)>>2]|0,x);l=l+1|0;}while((l|0)<(c[w>>2]|0))}c[b+576>>2]=0;c[b+580>>2]=1;if((IJ(b,g,q,r,s,n,o)|0)!=0){z=1;return z|0}if((L8(b+1864|0,h,c[p>>2]|0,c[t>>2]|0,i,j,f,k)|0)!=0){z=1;return z|0}if((L$(b+2136|0,b+2076|0,c[b+2120>>2]|0,k)|0)!=0){z=1;return z|0}z=(L3(b+588|0,n,a,c[e+12>>2]|0,o,c[w>>2]|0,c[y>>2]|0,c[m>>2]|0,c[u>>2]|0)|0)!=0|0;return z|0}function IJ(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0;k=c[b+576>>2]|0;l=d[g]|0;m=g+h|0;n=d[m]|0;o=(e|0)==1;e=o?f:l;p=o?0:f-l|0;f=((j*32e3|0)+(i>>1)|0)/(i|0)|0;do{if((l|0)<(f|0)){if((n|0)>(f|0)){q=1}else{r=n;break}while(1){i=d[g+q|0]|0;if((i|0)<(f|0)){q=q+1|0}else{r=i;break}}}else{r=l}}while(0);l=b+580|0;q=p+e|0;L11072:do{if((q|0)<(n|0)){f=1-e|0;i=(c[l>>2]|0)+p|0;j=q;o=r;s=0;while(1){if((s|0)>5){t=1;break}c[b+432+(s*24|0)+8>>2]=j;u=j+k|0;c[b+432+(s*24|0)+12>>2]=u;v=o-u|0;if((v|0)<(e-i|0)){w=v}else{v=(u-i&-2)+e|0;x=d[g]|0;do{if((x|0)<(v|0)){y=d[m]|0;if((y|0)>(v|0)){z=h}else{A=y;break}while(1){y=z-1|0;B=d[g+y|0]|0;if((B|0)>(v|0)){z=y}else{A=B;break}}}else{A=x}}while(0);w=A-u|0}x=f+u+w&-2;if((w|0)<1){C=s-1|0;D=u}else{v=u-x|0;c[b+432+(s*24|0)>>2]=v;c[b+432+(s*24|0)+16>>2]=x;c[b+432+(s*24|0)+20>>2]=w;c[b+432+(s*24|0)+4>>2]=v+w;C=s;D=w+u|0}v=D-o|0;x=C+1|0;if((D|0)<(n|0)){i=c[l>>2]|0;j=D;o=(((v|0)>0?v:-v|0)|0)<3?n:o;s=x}else{E=x;break L11072}}return t|0}else{E=0}}while(0);n=E-1|0;D=((c[b+432+(n*24|0)+20>>2]|0)<3&(n|0)>0?E-2|0:n)+1|0;n=b+584|0;c[n>>2]=D;E=b+440|0;l=c[E>>2]|0;if((l|0)>0){C=0;do{a[b+368+C|0]=C&255;C=C+1|0;F=c[E>>2]|0;}while((C|0)<(F|0));G=c[n>>2]|0;H=F}else{G=D;H=l}if((G|0)>0){I=0;J=H}else{t=0;return t|0}while(1){H=c[b+432+(I*24|0)>>2]|0;G=c[b+432+(I*24|0)+12>>2]|0;l=c[b+432+(I*24|0)+20>>2]|0;D=G-J|0;if((D|0)>0){NM(b+368+J|0,-1|0,D|0)}if((l|0)>0){D=0;do{a[D+G+(b+368)|0]=D+H&255;D=D+1|0;}while((D|0)<(l|0))}l=I+1|0;if((l|0)>=(c[n>>2]|0)){t=0;break}I=l;J=c[b+432+(l*24|0)+8>>2]|0}return t|0}function IK(a,b,d,e,f,g,h,i,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0;c[a+576>>2]=0;c[a+580>>2]=1;if((IJ(a,b,d,e,f,g,j)|0)!=0){k=1;return k|0}if((L9(a+1864|0,c[h>>2]|0,c[i>>2]|0)|0)!=0){k=1;return k|0}if((L0(a+2136|0,a+2076|0,c[a+2120>>2]|0)|0)!=0){k=1;return k|0}k=(L5(a+588|0,c[i+4>>2]|0)|0)!=0|0;return k|0}function IL(a){a=a|0;if((a|0)==0){return}Jb(a+80|0);Jd(a+64|0);L4(a+588|0);return}function IM(b,d,e,f,g,h,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0;n=i;i=i+3456|0;o=n|0;if((a[g+1|0]|0)!=0){i=n;return}p=c[172720+(l<<2)>>2]|0;q=m<<16;N2(p,(p|0)<0|0?-1:0,q,(q|0)<0|0?-1:0)|0;q=G+16384>>15;p=c[e+484>>2]>>j;do{if((p|0)<(h|0)){r=a[f]|0;if(r<<24>>24==0){s=0;t=h;break}u=r&255;r=0;v=p;while(1){w=c[b+(v<<2)>>2]|0;x=r;y=0;do{x=(c[w+(y<<2)>>2]>>6)+x|0;y=y+1|0;}while((y|0)<(u|0));y=v+1|0;if((y|0)<(h|0)){r=x;v=y}else{s=x;t=h;break}}}else{s=0;t=p}}while(0);h=p+(m>>j)|0;do{if((t|0)<(h|0)){j=a[f]|0;if(j<<24>>24==0){z=0;break}m=j&255;j=0;p=t;while(1){v=c[b+(p<<2)>>2]|0;r=j;u=0;do{r=(c[v+(u<<2)>>2]>>6)+r|0;u=u+1|0;}while((u|0)<(m|0));u=p+1|0;if((u|0)<(h|0)){j=r;p=u}else{z=r;break}}}else{z=0}}while(0);h=c[d>>2]|0;t=c[d+4>>2]|0;p=(z>>((t|0)<31?t:31))+(s>>((h|0)<31?h:31))<<2;s=(q|0)>0;if(s){t=(k|0)>0;z=(l|0)<1;j=k<<2;m=0;do{u=o+(m*192|0)|0;v=m<<1;do{if(t){if(z){NM(u|0,0,j|0);break}x=0;y=a[f]|0;while(1){w=x+1|0;A=a[f+w|0]|0;B=A&255;if((y&255)<(A&255)){C=y&255;D=0;while(1){E=0;F=D;do{F=(c[(c[b+(E+v>>1<<2)>>2]|0)+(C<<2)>>2]>>5)+F|0;E=E+1|0;}while((E|0)<(l|0));E=C+1|0;if((E|0)<(B|0)){C=E;D=F}else{H=F;break}}}else{H=0}c[o+(m*192|0)+(x<<2)>>2]=H;if((w|0)<(k|0)){x=w;y=A}else{break}}}}while(0);m=m+1|0;}while((m|0)<(q|0))}m=(h|0)>8?8:h;if((h-1|0)>31){I=0}else{if(s&(k|0)>0){s=0;H=0;while(1){l=0;b=H;do{b=(c[o+(s*192|0)+(l<<2)>>2]>>m)+b|0;l=l+1|0;}while((l|0)<(k|0));l=s+1|0;if((l|0)<(q|0)){s=l;H=b}else{J=b;break}}}else{J=0}I=J>>h-m}do{if((c[e+480>>2]|0)==0){m=((c[e+456>>2]|0)+p>>1)+I|0;J=q+1>>1;H=q-J|0;s=1073741824-(ac(c[172720+(q<<2)>>2]|0,J)|0)|0;l=s;f=(s|0)<0|0?-1:0;N2(l,f,l,f)|0;f=2147483647-(G<<3|0>>>29)|0;if((k|0)>0){l=(J|0)>0;s=(J|0)<(q|0);j=(m|0)==0;z=(m>>3)+1|0;m=0;t=0;v=h;while(1){u=(v|0)<25?31<<v>>3:130023424;if(l){y=u;x=0;while(1){r=(c[o+(x*192|0)+(m<<2)>>2]>>3)+y|0;D=x+1|0;if((D|0)<(J|0)){y=r;x=D}else{K=r;break}}}else{K=u}if(s){x=u;y=J;while(1){r=(c[o+(y*192|0)+(m<<2)>>2]>>3)+x|0;D=y+1|0;if((D|0)<(q|0)){x=r;y=D}else{L=r;break}}}else{L=u}y=HE(L)|0;x=y-(HE(K)|0)|0;y=x+(HE(J)|0)|0;x=y-(HE(H)|0)|0;N2(x,(x|0)<0|0?-1:0,1488522240,0)|0;x=G<<1|0>>>31;y=(x|0)>0?x:-x|0;if(j){M=0}else{x=c[d>>2]|0;M=Ms(L+K|0,z,(x|0)<31?2147483647>>>(x>>>0):0)|0}x=HN(M)|0;N2(x,(x|0)<0|0?-1:0,y,(y|0)<0|0?-1:0)|0;N=(G<<1|0>>>31)+t|0;y=m+1|0;if((y|0)>=(k|0)){break}m=y;t=N;v=c[d>>2]|0}O=(N|0)<0|0?-1:0;P=N}else{O=0;P=0}v=f;t=(f|0)<0|0?-1:0;N2(P,O,v,t)|0;if((G<<1|0>>>31|0)>(c[e+452>>2]>>6|0)){a[g]=1;break}else{a[g]=0;break}}else{a[g]=0}}while(0);c[e+456>>2]=p;c[e+460>>2]=I;i=n;return}function IN(b,d,e,f,g,h,j,k){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0;l=i;i=i+256|0;m=l|0;n=b+468|0;o=c[n>>2]|0;p=ac(k,j)|0;k=(p|0)<1|0;if((j|0)==2){q=1}else if((j|0)==4){q=2}else{q=0}j=b+472|0;r=c[j>>2]|0;s=b+484|0;t=c[s>>2]|0;u=c[e>>2]|0;v=e+4|0;w=c[v>>2]|0;x=t+o|0;y=(u|0)<(w|0)?u:w;z=u-y|0;A=(z|0)<31?z:31;z=w-y|0;B=(z|0)<31?z:31;if((r|0)>0){z=c[172720+(x-1<<2)>>2]<<h;C=c[172720+(x<<2)>>2]<<h;x=t>>h;D=t+(o>>h)|0;E=(x|0)<(g|0);F=(y|0)<32;H=z;I=(z|0)<0|0?-1:0;z=C;J=(C|0)<0|0?-1:0;C=0;do{if(E){K=x;L=0;while(1){M=c[(c[d+(K<<2)>>2]|0)+(C<<2)>>2]|0;N2(M,(M|0)<0|0?-1:0,z,J)|0;M=(G<<1|0>>>31)+L|0;N=K+1|0;if((N|0)<(g|0)){K=N;L=M}else{O=g;P=M;break}}}else{O=x;P=0}if((O|0)<(D|0)){L=0;K=O;while(1){M=c[(c[d+(K<<2)>>2]|0)+(C<<2)>>2]|0;N2(M,(M|0)<0|0?-1:0,z,J)|0;M=(G<<1|0>>>31)+L|0;N=K+1|0;if((N|0)<(D|0)){L=M;K=N}else{Q=M;break}}}else{Q=0}K=(Q>>B)+(P>>A)|0;do{if((K|0)==0){R=0}else{L=K>>31^K;if((L|0)<=-1){R=-1;break}M=~L;L=0;while(1){N=M<<1;if((N|0)<0){M=N;L=L+1|0}else{R=L;break}}}}while(0);L=R-6|0;M=(L|0)<0?0:L;if(E){L=x;N=0;while(1){S=K-(c[(c[d+(L<<2)>>2]|0)+(C<<2)>>2]>>A)<<M;T=S;U=(S|0)<0|0?-1:0;N2(T,U,T,U)|0;U=G<<1|0>>>31;N2(U,(U|0)<0|0?-1:0,H,I)|0;U=(G<<1|0>>>31)+N|0;T=L+1|0;if((T|0)<(g|0)){L=T;N=U}else{V=g;W=U;break}}}else{V=x;W=0}if((V|0)<(D|0)){N=V;L=W;while(1){U=K-(c[(c[d+(N<<2)>>2]|0)+(C<<2)>>2]>>B)<<M;T=U;S=(U|0)<0|0?-1:0;N2(T,S,T,S)|0;S=G<<1|0>>>31;N2(S,(S|0)<0|0?-1:0,H,I)|0;S=(G<<1|0>>>31)+L|0;T=N+1|0;if((T|0)<(D|0)){N=T;L=S}else{X=S;break}}}else{X=W}L=HN(X)|0;if(F){N=L>>M;L=b+192+(C<<2)|0;K=c[L>>2]|0;S=K;T=(K|0)<0|0?-1:0;K=1417339264;U=0;N2(S,T,K,U)|0;U=G<<1|0>>>31;K=N;T=(N|0)<0|0?-1:0;N=730144448;S=0;N2(K,T,N,S)|0;Y=U+((G<<1|0>>>31)>>y)|0;Z=L}else{Y=0;Z=b+192+(C<<2)|0}c[Z>>2]=(Y|0)<16?16:Y;C=C+1|0;}while((C|0)<(r|0));_=c[n>>2]|0;$=c[j>>2]|0;aa=c[s>>2]|0;ab=c[e>>2]|0;ad=c[v>>2]|0}else{_=o;$=r;aa=t;ab=u;ad=w}w=(ab|0)<31?ab:31;ab=(ad|0)<31?ad:31;ad=aa+k|0;IZ(b,b+(_-k<<2)|0,ad<<2);I_(b+(ad<<2)|0,_<<2);u=aa-3>>h;t=(g<<h)-1+_>>h;if(($|0)>0){r=(h|0)==1;h=(u|0)<(g|0);v=(_|0)==0;e=0;do{s=c[b+192+(e<<2)>>2]|0;if((s|0)>255){j=(2147483647/(s+1|0)|0)<<8;ae=(j|0)<0|0?-1:0;af=j}else{ae=0;af=2147483647}do{if(r){if(h){j=u;while(1){n=c[(c[d+(j<<2)>>2]|0)+(e<<2)>>2]>>w;C=j<<1;c[m+(C<<2)>>2]=n;c[m+((C|1)<<2)>>2]=n;n=j+1|0;if((n|0)<(g|0)){j=n}else{ag=g;break}}}else{ag=u}if((ag|0)>(t|0)){break}else{ah=ag}do{j=c[(c[d+(ah<<2)>>2]|0)+(e<<2)>>2]>>ab;n=ah<<1;c[m+(n<<2)>>2]=j;c[m+((n|1)<<2)>>2]=j;ah=ah+1|0;}while((ah|0)<=(t|0))}else{if(h){j=u;while(1){c[m+(j<<2)>>2]=c[(c[d+(j<<2)>>2]|0)+(e<<2)>>2]>>w;n=j+1|0;if((n|0)<(g|0)){j=n}else{ai=g;break}}}else{ai=u}if((ai|0)>(t|0)){break}else{aj=ai}do{c[m+(aj<<2)>>2]=c[(c[d+(aj<<2)>>2]|0)+(e<<2)>>2]>>ab;aj=aj+1|0;}while((aj|0)<=(t|0))}}while(0);if(!v){M=aa;j=ad;n=_;while(1){C=M+1|0;Y=(c[m+(C<<2)>>2]|0)-s-(c[m+(M-1<<2)>>2]|0)|0;if((Y|0)>0){Z=Y;y=(Y|0)<0|0?-1:0;N2(Z,y,af,ae)|0;ak=G<<1|0>>>31}else{ak=0}y=Y-s+(c[m+(M+2<<2)>>2]|0)-(c[m+(M-2<<2)>>2]|0)|0;if((y|0)>0){Y=y;Z=(y|0)<0|0?-1:0;N2(Y,Z,af,ae)|0;al=(G<<1|0>>>31)+ak|0}else{al=ak}Z=y-s+(c[m+(M+3<<2)>>2]|0)-(c[m+(M-3<<2)>>2]|0)|0;if((Z|0)>0){y=Z;Y=(Z|0)<0|0?-1:0;N2(y,Y,af,ae)|0;am=(G<<1|0>>>31)+al|0}else{am=al}Y=n-1|0;y=b+(j<<2)|0;c[y>>2]=(c[y>>2]|0)+am;if((Y|0)==0){break}else{M=C;j=j+1|0;n=Y}}}e=e+1|0;}while((e|0)<($|0))}a[f]=0;$=f+1|0;a[$]=0;e=f+2|0;a[e]=0;am=k+p|0;p=o+am|0;L11278:do{if((o|0)>0){k=b+448|0;al=am;while(1){ae=c[b+(al<<2)>>2]|0;af=c[b+(al-1<<2)>>2]|0;N2(af,(af|0)<0|0?-1:0,1932735232,0)|0;if((ae|0)<(G<<1|0>>>31|0)){if((af|0)>(c[k>>2]|0)){break}}af=al+1|0;if((af|0)<(p|0)){al=af}else{an=0;break L11278}}a[f]=al-am>>q&255;a[$]=1;an=1}else{an=0}}while(0);$=c[b+480>>2]|0;if(($|0)<=0){i=l;return}am=b+448|0;f=p;while(1){o=c[b+(f<<2)>>2]|0;k=c[b+(f-1<<2)>>2]|0;N2(k,(k|0)<0|0?-1:0,1932735232,0)|0;if((o|0)<(G<<1|0>>>31|0)){if((k|0)>(c[am>>2]|0)){break}}k=f+1|0;if((k|0)<($+p|0)){f=k}else{ao=8118;break}}if((ao|0)==8118){i=l;return}if(!((f-p>>q|0)<3&an<<24>>24==0)){i=l;return}a[e]=1;i=l;return}function IO(a,b,d,e,f,g,h,j,k,l,m){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;k=i;i=i+16|0;j=k|0;n=k+8|0;o=c[e+16>>2]|0;p=c[e+4>>2]|0;q=c[e>>2]|0;I_(a,488);c[a+480>>2]=l;c[a+484>>2]=m;if((q|0)==0){c[j>>2]=0;r=0;s=536870912}else{m=HQ(ac(p,o)|0,q<<2,j)|0;r=(m|0)<0|0?-1:0;s=m}m=(HR(b,d)|0)-21474836|0;d=(m|0)>214748;b=d?m:214748;q=d?(m|0)<0|0?-1:0:0;N2(b,q,b,q)|0;q=HQ(161061,G<<1|0>>>31,n)|0;b=-2-(c[n>>2]|0)-(c[j>>2]|0)|0;c[n>>2]=b;c[a+468>>2]=g;c[a+448>>2]=(c[e+52>>2]<<7|0)/(h|0)|0;c[a+464>>2]=f;N2(q,(q|0)<0|0?-1:0,s,r)|0;r=G<<1|0>>>31;if((b|0)>-1){c[a+452>>2]=r>>b;t=a+472|0;c[t>>2]=h;u=e+84|0;v=c[u>>2]|0;w=a+476|0;c[w>>2]=v;x=a+456|0;c[x>>2]=0;i=k;return 0}else{c[a+452>>2]=r<<-b;t=a+472|0;c[t>>2]=h;u=e+84|0;v=c[u>>2]|0;w=a+476|0;c[w>>2]=v;x=a+456|0;c[x>>2]=0;i=k;return 0}return 0}function IP(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=i;i=i+16|0;f=e|0;g=f;c[g>>2]=d;c[g+4>>2]=0;g=cq(a|0,b|0,f|0)|0;i=e;return g|0}function IQ(a,b){a=a|0;b=b|0;return NE(a,b)|0}function IR(a){a=a|0;ND(a);return}function IS(a,b){a=a|0;b=b|0;var d=0,e=0;d=NE(1,a+4+b|0)|0;if((d|0)==0){e=0;return e|0}b=-(d+4|0)&7;c[d+b>>2]=d;e=d+(b+4)|0;return e|0}function IT(a){a=a|0;ND(c[a-4>>2]|0);return}function IU(a,b,c){a=a|0;b=b|0;c=c|0;return NE(a,b)|0}function IV(a){a=a|0;ND(a);return}function IW(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;d=NE(1,a+4+b|0)|0;if((d|0)==0){e=0;return e|0}b=-(d+4|0)&7;c[d+b>>2]=d;e=d+(b+4)|0;return e|0}function IX(a){a=a|0;ND(c[a-4>>2]|0);return}function IY(a,b,c){a=a|0;b=b|0;c=c|0;NN(a|0,b|0,c)|0;return}function IZ(a,b,c){a=a|0;b=b|0;c=c|0;NP(a|0,b|0,c|0);return}function I_(a,b){a=a|0;b=b|0;NM(a|0,0,b|0);return}function I$(a){a=a|0;return((a|0)>-1?a:-a|0)|0}function I0(a){a=a|0;return IQ(9088,4)|0}function I1(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function I2(a){a=a|0;return IQ(1,668)|0}function I3(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function I4(a){a=a|0;return IQ(1,6512)|0}function I5(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function I6(a){a=a|0;return IQ(1,1128)|0}function I7(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function I8(a){a=a|0;return IW(1280,8,8192)|0}function I9(a){a=a|0;if((a|0)==0){return}IX(c[a>>2]|0);c[a>>2]=0;return}function Ja(a){a=a|0;return IU(256,4,8192)|0}function Jb(a){a=a|0;if((a|0)==0){return}IV(c[a>>2]|0);c[a>>2]=0;return}function Jc(a){a=a|0;return IQ(256,4)|0}function Jd(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Je(a){a=a|0;return IQ(25,1)|0}function Jf(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Jg(a){a=a|0;return IQ(49,1)|0}function Jh(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Ji(a){a=a|0;return IQ(49,1)|0}function Jj(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Jk(a){a=a|0;return IQ(192,1)|0}function Jl(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Jm(a){a=a|0;return IQ(48,1)|0}function Jn(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Jo(a){a=a|0;return IQ(48,1)|0}function Jp(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Jq(a){a=a|0;return IQ(192,1)|0}function Jr(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Js(a){a=a|0;return IQ(192,4)|0}function Jt(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Ju(a){a=a|0;return IQ(192,4)|0}function Jv(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Jw(a){a=a|0;return IW(1280,8,8192)|0}function Jx(a){a=a|0;if((a|0)==0){return}IX(c[a>>2]|0);c[a>>2]=0;return}function Jy(a){a=a|0;return IU(1,1356,8192)|0}function Jz(a){a=a|0;if((a|0)==0){return}IV(c[a>>2]|0);c[a>>2]=0;return}function JA(a){a=a|0;return IQ(1,30608)|0}function JB(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function JC(a){a=a|0;return IQ(1024,4)|0}function JD(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function JE(a,b){a=a|0;b=b|0;return b+((a*9088|0)+18176)|0}function JF(a,b){a=a|0;b=b|0;return b+(a*18176|0)|0}function JG(a,b){a=a|0;b=b|0;return b+((a*18176|0)+9088)|0}function JH(a){a=a|0;return IS(66384,8)|0}function JI(a){a=a|0;if((a|0)==0){return}IT(c[a>>2]|0);c[a>>2]=0;return}function JJ(a){a=a|0;return IQ(1,220)|0}function JK(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function JL(a){a=a|0;return IQ(1,8)|0}function JM(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function JN(a){a=a|0;return IQ(1,3744)|0}function JO(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function JP(a){a=a|0;return IQ(1,4524)|0}function JQ(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function JR(a){a=a|0;return IQ(2048,2)|0}function JS(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function JT(a,b){a=a|0;b=b|0;return b+((a*9072|0)+57312)|0}function JU(a){a=a|0;return IQ(8192,1)|0}function JV(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function JW(a){a=a|0;return IQ(1,48)|0}function JX(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function JY(a){a=a|0;return IQ(1,256)|0}function JZ(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function J_(a){a=a|0;return IQ(1,2120)|0}function J$(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function J0(a){a=a|0;return IQ(1,84)|0}function J1(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function J2(a){a=a|0;return IQ(1,88)|0}function J3(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function J4(a){a=a|0;return IQ(1,92)|0}function J5(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function J6(a){a=a|0;return IQ(1,20)|0}function J7(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function J8(a){a=a|0;return IQ(1,8)|0}function J9(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Ka(a,b){a=a|0;b=b|0;return b+57312|0}function Kb(a,b){a=a|0;b=b|0;return b+59760|0}function Kc(a){a=a|0;return IQ(1,144)|0}function Kd(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Ke(a){a=a|0;return IQ(1,2008)|0}function Kf(a){a=a|0;if((a|0)==0){return}IR(c[a>>2]|0);c[a>>2]=0;return}function Kg(a,b){a=a|0;b=b|0;return b+(a*9552|0)|0}function Kh(a,b,d,e,f,g,h,i,j,k){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0;j=0;while(1){h=j+1|0;l=-1<<h;if((l&d|0)!=(d|0)){break}if((l&b|0)==(b|0)){j=h}else{break}}h=d>>j;l=b>>j;j=(a|0)==0;m=e*40|0;e=(ac(f*6144|0,l)|0)/(h|0)|0;if((i|0)==0){if(!j){f=(ac((H$(a,((ac(h,g)|0)/(l|0)|0|0)/(k|0)|0)|0)+m|0,b)|0)/(d|0)|0;n=(f|0)<(g|0)?g:f;f=(n|0)<(e|0)?n:e;if((f|0)==(g|0)){o=g;return o|0}n=(ac((H$(a,((ac(f,h)|0)/(l|0)|0|0)/(k|0)|0)|0)+m|0,b)|0)/(d|0)|0;p=(f|0)>(n|0)?f:n;n=(p|0)<(e|0)?p:e;if((f|0)==(n|0)){o=f;return o|0}f=(ac((H$(a,((ac(n,h)|0)/(l|0)|0|0)/(k|0)|0)|0)+m|0,b)|0)/(d|0)|0;p=(n|0)>(f|0)?n:f;f=(p|0)<(e|0)?p:e;if((n|0)==(f|0)){o=n;return o|0}else{n=(ac((H$(a,((ac(f,h)|0)/(l|0)|0|0)/(k|0)|0)|0)+m|0,b)|0)/(d|0)|0;p=(f|0)>(n|0)?f:n;return((p|0)<(e|0)?p:e)|0}}p=(ac(m+208|0,b)|0)/(d|0)|0;n=(p|0)<(g|0)?g:p;f=(n|0)<(e|0)?n:e;if((f|0)==(g|0)){o=g;return o|0}n=(f|0)>(p|0)?f:p;q=(n|0)<(e|0)?n:e;if((f|0)==(q|0)){o=f;return o|0}f=(q|0)>(p|0)?q:p;n=(f|0)<(e|0)?f:e;if((q|0)==(n|0)){o=q;return o|0}q=(n|0)>(p|0)?n:p;o=(q|0)<(e|0)?q:e;return o|0}q=((ac(h,g)|0)/(l|0)|0|0)/(k|0)|0;c[i>>2]=q;if(j){j=(ac(m+208|0,b)|0)/(d|0)|0;p=(j|0)<(g|0)?g:j;n=(p|0)<(e|0)?p:e;if((n|0)==(g|0)){o=g;return o|0}c[i>>2]=((ac(n,h)|0)/(l|0)|0|0)/(k|0)|0;p=(n|0)>(j|0)?n:j;f=(p|0)<(e|0)?p:e;if((n|0)==(f|0)){o=n;return o|0}c[i>>2]=((ac(f,h)|0)/(l|0)|0|0)/(k|0)|0;n=(f|0)>(j|0)?f:j;p=(n|0)<(e|0)?n:e;if((f|0)==(p|0)){o=f;return o|0}c[i>>2]=((ac(p,h)|0)/(l|0)|0|0)/(k|0)|0;f=(p|0)>(j|0)?p:j;o=(f|0)<(e|0)?f:e;return o|0}else{f=(ac((H$(a,q)|0)+m|0,b)|0)/(d|0)|0;q=(f|0)<(g|0)?g:f;f=(q|0)<(e|0)?q:e;if((f|0)==(g|0)){o=g;return o|0}g=((ac(f,h)|0)/(l|0)|0|0)/(k|0)|0;c[i>>2]=g;q=(ac((H$(a,g)|0)+m|0,b)|0)/(d|0)|0;g=(f|0)>(q|0)?f:q;q=(g|0)<(e|0)?g:e;if((f|0)==(q|0)){o=f;return o|0}f=((ac(q,h)|0)/(l|0)|0|0)/(k|0)|0;c[i>>2]=f;g=(ac((H$(a,f)|0)+m|0,b)|0)/(d|0)|0;f=(q|0)>(g|0)?q:g;g=(f|0)<(e|0)?f:e;if((q|0)==(g|0)){o=q;return o|0}q=((ac(g,h)|0)/(l|0)|0|0)/(k|0)|0;c[i>>2]=q;i=(ac((H$(a,q)|0)+m|0,b)|0)/(d|0)|0;d=(g|0)>(i|0)?g:i;o=(d|0)<(e|0)?d:e;return o|0}return 0}function Ki(a,b){a=a|0;b=b|0;var d=0,e=0;d=Hn(b)|0;if((a-1|0)>>>0<5){e=c[1927404+(a*12|0)+(((d|0)==2)<<2)>>2]|0}else{e=0}return ac(c[(Hk(b)|0)+8>>2]|0,e)|0}function Kj(b){b=b|0;I_(b,80);c[b+56>>2]=0;c[b+8>>2]=0;c[b+4>>2]=-1;c[b+20>>2]=-1;c[b+24>>2]=0;c[b+36>>2]=0;a[b+76|0]=15;a[b+77|0]=1;a[b+78|0]=1;c[b+44>>2]=-1;c[b+48>>2]=0;a[b+52|0]=-1;c[b+12>>2]=1;c[b+32>>2]=0;c[b+40>>2]=0;c[b+64>>2]=-1;c[b+68>>2]=-1;c[b+72>>2]=-1;HJ();HL();return}function Kk(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0;if((a|0)==0){f=8224;return f|0}g=JJ(0)|0;do{if((g|0)==0){h=12576}else{I_(g,220);i=JH(0)|0;c[g+200>>2]=i;j=i;i=KB(g+180|0,b,d,j)|0;if((i|0)!=0){h=i;break}i=KD(g+176|0,b,d,e,j)|0;if((i|0)!=0){h=i;break}i=KH(g+172|0,b,d,e,j)|0;if((i|0)!=0){h=i;break}i=KJ(g+168|0,b,j)|0;if((i|0)!=0){h=i;break}c[g+204>>2]=d;c[g+208>>2]=b;c[g+212>>2]=e;h=0}}while(0);c[a>>2]=g;f=h;return f|0}function Kl(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0;h=i;i=i+96|0;j=h|0;k=h+8|0;l=h+16|0;m=h+80|0;n=h+88|0;c[m>>2]=0;if((e|0)==0){o=8224;i=h;return o|0}p=e+28|0;q=c[p>>2]|0;if((q-1|0)>>>0>5){o=12512;i=h;return o|0}r=e|0;s=c[r>>2]|0;if(!((s|0)==8e3|(s|0)==11025|(s|0)==12e3|(s|0)==16e3|(s|0)==22050|(s|0)==24e3|(s|0)==32e3|(s|0)==44100|(s|0)==48e3|(s|0)==64e3|(s|0)==88200|(s|0)==96e3)){o=12544;i=h;return o|0}t=e+4|0;if((c[t>>2]|0)==-1){o=12320;i=h;return o|0}u=e+44|0;v=c[u>>2]|0;w=e+40|0;x=c[(Hk(c[w>>2]|0)|0)+8>>2]|0;y=e+24|0;z=e+12|0;A=Kh(f,s,v,q,x,c[t>>2]|0,0,m,0,c[z>>2]|0)|0;if((A|0)!=(c[t>>2]|0)){o=12320;i=h;return o|0}x=e+48|0;if((c[x>>2]&5|0)!=0){o=12448;i=h;return o|0}q=c[u>>2]|0;do{if((q|0)==512|(q|0)==480){v=c[e+16>>2]|0;if((v|0)==23|(v|0)==39){break}else{o=8320}i=h;return o|0}else if((q|0)==1024){v=c[e+16>>2]|0;if((v|0)==23|(v|0)==39){o=8320}else{break}i=h;return o|0}else{o=8320;i=h;return o|0}}while(0);v=c[e+56>>2]|0;if((v|0)==0){B=q;C=c[r>>2]|0}else{s=b+4|0;D=c[r>>2]|0;if((v|0)<-1){o=12352;i=h;return o|0}do{if((v|0)==0){E=0}else if((v|0)==(-1|0)){if((A|0)>191999){E=19199;break}E=(A|0)/10|0}else{if((v|0)>19199){o=12352;i=h;return o|0}if((v*20|0|0)>(A*3|0|0)){o=12352}else{E=v;break}i=h;return o|0}}while(0);v=(ac(E,q)|0)/(D|0)|0;D=v-((v|0)%8|0)|0;c[s>>2]=D;s=c[r>>2]|0;v=ac(D,s)|0;D=c[u>>2]|0;q=e+8|0;c[q>>2]=(c[q>>2]|0)+((v|0)/(D|0)|0);B=D;C=s}s=HQ(B,C,n)|0;C=((c[p>>2]|0)*-8e3|0)+(c[t>>2]|0)|0;do{if((C|0)==0){F=0}else{p=C>>31^C;if((p|0)<=-1){F=-1;break}B=~p;p=0;while(1){D=B<<1;if((D|0)<0){B=D;p=p+1|0}else{F=p;break}}}}while(0);p=C<<F;N2(p,(p|0)<0|0?-1:0,s,(s|0)<0|0?-1:0)|0;s=G;if((s|0)<0){H=0}else{p=s>>F+2-(c[n>>2]|0);H=(p|0)>256?256:p}c[e+60>>2]=H;H=b|0;c[H>>2]=e;p=b+192|0;c[p>>2]=c[y>>2];n=c[w>>2]|0;w=b+184|0;c[w>>2]=n;F=b+8|0;s=Hl(n,c[e+32>>2]|0,F)|0;if((s|0)!=0){o=s;i=h;return o|0}s=e+8|0;n=Ko((c[H>>2]|0)+36|0,c[e+36>>2]|0,(c[t>>2]|0)-(c[s>>2]|0)|0,c[p>>2]|0,c[r>>2]|0,c[u>>2]|0,F,c[w>>2]|0)|0;if((n|0)!=0){o=n;i=h;return o|0}n=b+188|0;c[n>>2]=c[(c[H>>2]|0)+36>>2];w=a[e+76|0]|0;p=c[t>>2]|0;C=c[s>>2]|0;B=b+180|0;D=b+212|0;v=e+16|0;q=KE(c[B>>2]|0,b+176|0,c[D>>2]|0,c[b+204>>2]|0,c[v>>2]|0,F)|0;if((q|0)!=0){o=q;i=h;return o|0}q=KF(c[B>>2]|0,c[v>>2]|0,F,c[r>>2]|0,c[u>>2]|0,p-C|0,w<<24>>24!=0?15:0,c[n>>2]|0,d[e+77|0]|0,d[e+78|0]|0,c[x>>2]|0,g)|0;if((q|0)!=0){o=q;i=h;return o|0}q=KI(b+172|0,c[D>>2]|0,F)|0;if((q|0)!=0){o=q;i=h;return o|0}c[l>>2]=F;c[l+4>>2]=0;F=c[m>>2]|0;m=F+7&-8;q=l+12|0;c[q>>2]=m;D=b+16|0;g=(c[D>>2]|0)*6144|0;x=g-m|0;w=c[e+72>>2]|0;if((w|0)==-1){I=x}else{I=(w|0)<(x|0)?w:x}c[l+16>>2]=I;x=m+I|0;m=(g|0)<(x|0)?g:x;g=l+8|0;c[g>>2]=m;w=c[e+68>>2]|0;if((w|0)==-1){J=m}else{J=(m|0)<(w|0)?m:w}c[g>>2]=J;J=(F-1&-8)-I-(H$(f,x)|0)|0;x=(J|0)<0?0:J;J=l+56|0;c[J>>2]=x;I=c[e+64>>2]|0;if((I|0)==-1){K=x}else{K=(x|0)>(I|0)?x:I}c[J>>2]=K;K=l+52|0;c[K>>2]=c[z>>2];z=c[r>>2]|0;c[l+60>>2]=z;r=HQ(c[n>>2]|0,z>>1,k)|0;z=31-(c[k>>2]|0)|0;c[k>>2]=z;k=(c[u>>2]|0)*655360|0;N2(k,(k|0)<0|0?-1:0,r,(r|0)<0|0?-1:0)|0;c[l+28>>2]=(G<<1|0>>>31)>>z-15;z=HQ((c[D>>2]|0)*5400|0,(c[q>>2]|0)/(c[K>>2]|0)|0,j)|0;D=31-(c[j>>2]|0)|0;c[j>>2]=D;if((D|0)>24){L=z>>D-24}else{L=z<<24-D}c[l+44>>2]=L;switch(c[y>>2]|0){case 0:{c[l+24>>2]=0;break};case 5:{c[l+24>>2]=5;break};case 7:{c[l+24>>2]=7;break};case 6:{c[l+24>>2]=6;break};case 1:{c[l+24>>2]=1;break};case 2:{c[l+24>>2]=2;break};case 3:{c[l+24>>2]=3;break};case 4:{c[l+24>>2]=4;break};default:{o=12328;i=h;return o|0}}c[l+36>>2]=(a[e+79|0]|0)!=0?2:0;e=c[v>>2]|0;if((e|0)==23|(e|0)==39){c[l+40>>2]=2}else{c[l+40>>2]=5}c[l+48>>2]=(c[t>>2]|0)-(c[s>>2]|0);c[l+20>>2]=H$(f,(c[q>>2]|0)/(c[K>>2]|0)|0)|0;K=KL(c[b+168>>2]|0,l)|0;if((K|0)!=0){o=K;i=h;return o|0}K=c[(c[H>>2]|0)+16>>2]|0;switch(K|0){case 129:case 135:{c[b+216>>2]=2;o=0;i=h;return o|0};case 132:case 136:{c[b+216>>2]=5;o=0;i=h;return o|0};case 156:case 137:{c[b+216>>2]=29;o=0;i=h;return o|0};default:{c[b+216>>2]=K;o=0;i=h;return o|0}}return 0}
function kD(a){a=a|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0;d=i;i=i+152|0;e=d|0;f=d+8|0;g=d+16|0;h=d+24|0;j=d+32|0;k=d+40|0;l=d+48|0;m=d+56|0;n=d+136|0;o=d+144|0;p=a+12|0;q=c[p>>2]|0;r=q;s=a+16|0;t=c[s>>2]|0;u=q+16|0;v=c[u>>2]|0;if((v|0)>0){w=q+48|0;x=n;y=m+28|0;z=m+24|0;A=m+8|0;B=m+16|0;C=m+48|0;D=m+32|0;E=0;F=v;while(1){v=c[w>>2]|0;do{if((c[(c[v+(E*264|0)+76>>2]|0)+48>>2]|0)==94213){H=v+(E*264|0)+32|0;if((c[H>>2]|0)!=0){I=F;break}J=v+(E*264|0)+24|0;K=c[J>>2]|0;L=c[J+4>>2]|0;b[n>>1]=0;nB(m);c[y>>2]=2;c[z>>2]=x;c[A>>2]=K;c[A+4>>2]=L;c[B>>2]=K;c[B+4>>2]=L;c[C>>2]=0;c[D>>2]=E;kT(a,m)|0;nJ(m);c[H>>2]=1;I=c[u>>2]|0}else{I=F}}while(0);v=E+1|0;if((v|0)<(I|0)){E=v;F=I}else{break}}}I=il(t,0,0,1)|0;F=G;E=q+52|0;do{if((c[E>>2]&2|0)==0){m=q+40|0;D=NU(c[m>>2]|0,c[m+4>>2]|0,8,0)|0;C=G;B=1;A=q+32|0;x=c[A>>2]|0;z=c[A+4>>2]|0;if(C>>>0<B>>>0|C>>>0==B>>>0&D>>>0<0>>>0){il(t,x,z,0)|0;D=NU(c[m>>2]|0,c[m+4>>2]|0,8,0)|0;is(t,D)}else{D=NU(x,z,-8,-1)|0;z=G;il(t,D,z,0)|0;is(t,1);ir(t,1952539757);z=NU(c[m>>2]|0,c[m+4>>2]|0,16,0)|0;ix(t,z,G)}z=q+60|0;if((c[z>>2]|0)>0){m=q+64|0;M=c[m+4>>2]|0;N=c[m>>2]|0}else{M=F;N=I}il(t,N,M,0)|0;m=c[z>>2]|0;if((m|0)!=-1){kE(t,r,a);if((m|0)<=0){O=0;break}m=c[z>>2]|0;z=il(t,0,0,1)|0;D=q+64|0;x=NV(c[D>>2]|0,c[D+4>>2]|0,z,G)|0;z=NU(x,G,m,(m|0)<0|0?-1:0)|0;m=G;x=0;if((m|0)<(x|0)|(m|0)==(x|0)&z>>>0<8>>>0){x=NV(8,0,z,m)|0;D=G;AN(a,16,1634168,(P=i,i=i+16|0,c[P>>2]=x,c[P+8>>2]=D,P)|0);i=P;Q=-1;i=d;return Q|0}is(t,z);ir(t,1701147238);D=0;do{ih(t,0);D=D+1|0;x=(D|0)<0|0?-1:0;}while((x|0)<(m|0)|(x|0)==(m|0)&D>>>0<z>>>0);il(t,I,F,0)|0;O=0;break}z=a;AN(z,32,1634880,(P=i,i=i+1|0,i=i+7&-8,c[P>>2]=0,P)|0);i=P;D=c[p>>2]|0;m=c[s>>2]|0;x=0;B=0;il(m,x,B,1)|0;B=c[p>>2]|0;x=iZ(h)|0;do{if((x|0)<0){R=x}else{kE(c[h>>2]|0,B,a);m=i$(c[h>>2]|0,g)|0;A4(c[g>>2]|0);if((m|0)<0){R=m;break}C=B+16|0;if((c[C>>2]|0)>0){A=m;y=(m|0)<0|0?-1:0;n=B+48|0;w=0;do{v=(c[n>>2]|0)+(w*264|0)+192|0;H=NU(c[v>>2]|0,c[v+4>>2]|0,A,y)|0;c[v>>2]=H;c[v+4>>2]=G;w=w+1|0;}while((w|0)<(c[C>>2]|0))}w=c[p>>2]|0;y=iZ(f)|0;if((y|0)<0){S=y}else{kE(c[f>>2]|0,w,a);w=i$(c[f>>2]|0,e)|0;A4(c[e>>2]|0);S=w}w=(S|0)<0;do{if(!(w|(S|0)==(m|0))){if((c[C>>2]|0)<=0){break}y=S-m|0;A=y;n=(y|0)<0|0?-1:0;y=B+48|0;v=0;do{H=(c[y>>2]|0)+(v*264|0)+192|0;L=NU(c[H>>2]|0,c[H+4>>2]|0,A,n)|0;c[H>>2]=L;c[H+4>>2]=G;v=v+1|0;}while((v|0)<(c[C>>2]|0))}}while(0);if(w){R=S;break}C=A1(S<<1)|0;if((C|0)==0){R=-12;break}c[j>>2]=C;m=C+S|0;c[j+4>>2]=m;ik(c[s>>2]|0);v=a+32|0;n=iT(l,v,1)|0;if((n|0)<0){AN(z,16,1696784,(P=i,i=i+8|0,c[P>>2]=v,P)|0);i=P}else{v=il(c[s>>2]|0,0,0,1)|0;A=G;y=c[s>>2]|0;H=D+64|0;L=NU(c[H>>2]|0,c[H+4>>2]|0,S,(S|0)<0|0?-1:0)|0;K=G;il(y,L,K,0)|0;K=c[l>>2]|0;L=c[H>>2]|0;y=c[H+4>>2]|0;il(K,L,y,0)|0;y=il(c[l>>2]|0,0,0,1)|0;L=G;c[k>>2]=iE(c[l>>2]|0,C,S)|0;K=1;H=L;L=y;y=m;do{c[k+(K<<2)>>2]=iE(c[l>>2]|0,y,S)|0;K=K^1;m=c[k+(K<<2)>>2]|0;if((m|0)<1){break}y=c[j+(K<<2)>>2]|0;ij(c[s>>2]|0,y,m);L=NU(m,(m|0)<0|0?-1:0,L,H)|0;H=G;}while((H|0)<(A|0)|(H|0)==(A|0)&L>>>0<v>>>0);iV(c[l>>2]|0)|0}A4(C);R=n}}while(0);if((R|0)!=0){O=R;break}D=c[s>>2]|0;z=q+64|0;B=c[z>>2]|0;x=c[z+4>>2]|0;il(D,B,x,0)|0;kE(t,r,a);O=0}else{kF(a);x=il(t,0,0,1)|0;B=G;is(t,0);ir(t,1634887277);if((c[E>>2]&64|0)!=0){D=il(t,0,0,1)|0;z=G;il(t,x,B,0)|0;v=NV(D,z,x,B)|0;is(t,v);il(t,D,z,0)|0;O=0;break}z=c[u>>2]|0;if((z|0)>0){D=q+48|0;v=0;L=z;while(1){z=c[D>>2]|0;A=z+(v*264|0)+216|0;if((c[A>>2]|0)==0){T=L}else{H=il(t,0,0,1)|0;y=G;is(t,0);ir(t,1634887284);ih(t,1);iz(t,0);is(t,c[z+(v*264|0)+68>>2]|0);is(t,0);is(t,c[A>>2]|0);if((c[A>>2]|0)>0){K=z+(v*264|0)+220|0;z=0;do{w=(c[K>>2]|0)+(z<<5)+8|0;ix(t,c[w>>2]|0,c[w+4>>2]|0);w=(c[K>>2]|0)+(z<<5)|0;ix(t,c[w>>2]|0,c[w+4>>2]|0);ih(t,1);ih(t,1);ih(t,1);z=z+1|0;}while((z|0)<(c[A>>2]|0))}A=il(t,0,0,1)|0;z=G;il(t,H,y,0)|0;K=NV(A,z,H,y)|0;is(t,K);il(t,A,z,0)|0;T=c[u>>2]|0}z=v+1|0;if((z|0)<(T|0)){v=z;L=T}else{break}}}is(t,16);ir(t,1869768301);is(t,0);L=il(t,0,0,1)|0;v=G;D=NV(4,0,x,B)|0;z=NU(D,G,L,v)|0;is(t,z);z=il(t,0,0,1)|0;v=G;il(t,x,B,0)|0;L=NV(z,v,x,B)|0;is(t,L);il(t,z,v,0)|0;O=0}}while(0);T=c[q+24>>2]|0;a=q+48|0;if((T|0)!=0){A6((c[a>>2]|0)+(T*264|0)+76|0)}if((c[u>>2]|0)>0){T=a;r=o|0;o=q+20|0;q=0;do{R=c[T>>2]|0;l=c[R+(q*264|0)+72>>2]|0;do{if((l|0)==544240754){kW(R+(q*264|0)|0)}else if((l|0)==1684237684){if((c[o>>2]|0)==0){break}A6(R+(q*264|0)+76|0)}}while(0);do{if((c[E>>2]&2|0)!=0){R=(c[T>>2]|0)+(q*264|0)+232|0;if((c[R>>2]|0)==0&(c[R+4>>2]|0)==0){break}if((c[(c[s>>2]|0)+88>>2]|0)==0){break}R=il(t,0,0,1)|0;l=G;if((kG((c[T>>2]|0)+(q*264|0)|0,r)|0)<=-1){break}B=(c[T>>2]|0)+(q*264|0)+232|0;x=c[B>>2]|0;j=c[B+4>>2]|0;il(t,x,j,0)|0;ij(t,r,7);il(t,R,l,0)|0;}}while(0);A6((c[T>>2]|0)+(q*264|0)+88|0);A6((c[T>>2]|0)+(q*264|0)+220|0);l=c[T>>2]|0;if((c[l+(q*264|0)+80>>2]|0)!=0){A4(c[l+(q*264|0)+84>>2]|0)}q=q+1|0;}while((q|0)<(c[u>>2]|0))}A6(a);Q=O;i=d;return Q|0}function kE(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0.0,an=0.0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0;e=i;i=i+1048|0;f=e|0;g=e+8|0;h=e+16|0;j=e+24|0;k=e+1024|0;l=il(a,0,0,1)|0;m=G;NM(k|0,0,20);is(a,0);ir(a,1987014509);n=b+16|0;o=c[n>>2]|0;if((o|0)>0){p=b+48|0;q=b+52|0;r=b+8|0;s=0;while(1){t=c[p>>2]|0;do{if((c[t+(s*264|0)+4>>2]|0)<1){if((c[q>>2]&2|0)!=0){u=9802;break}v=s+1|0}else{u=9802}}while(0);do{if((u|0)==9802){u=0;w=c[r+4>>2]|0;x=t+(s*264|0)+16|0;c[x>>2]=c[r>>2];c[x+4>>2]=w;w=s+1|0;c[(c[p>>2]|0)+(s*264|0)+68>>2]=w;x=c[p>>2]|0;y=x+(s*264|0)+4|0;if((c[y>>2]|0)==0){v=w;break}z=x+(s*264|0)+88|0;A=c[z>>2]|0;B=c[A+16>>2]|0;c[A+24>>2]=1;C=x+(s*264|0)+44|0;if((c[C>>2]|0)!=0){v=w;break}c[C>>2]=1;if((c[y>>2]|0)<=1){v=w;break}x=1;D=A;A=0;E=B;while(1){B=D|0;F=NU(c[B>>2]|0,c[B+4>>2]|0,E,A)|0;B=c[z>>2]|0;H=B+(x*40|0)|0;do{if((F|0)==(c[H>>2]|0)&(G|0)==(c[H+4>>2]|0)){I=NU(c[B+(x*40|0)+16>>2]|0,0,E,A)|0;J=G;K=0;if(!(J>>>0<K>>>0|J>>>0==K>>>0&I>>>0<1048576>>>0)){u=9809;break}K=D+20|0;c[K>>2]=(c[K>>2]|0)+(c[B+(x*40|0)+28>>2]|0);L=J;M=I;N=D}else{u=9809}}while(0);if((u|0)==9809){u=0;c[B+(x*40|0)+24>>2]=(c[D+24>>2]|0)+1;H=c[z>>2]|0;F=c[H+(x*40|0)+16>>2]|0;c[C>>2]=(c[C>>2]|0)+1;L=0;M=F;N=H+(x*40|0)|0}H=x+1|0;if((H|0)<(c[y>>2]|0)){x=H;D=N;A=L;E=M}else{v=w;break}}}}while(0);t=c[n>>2]|0;if((v|0)<(t|0)){s=v}else{O=t;break}}}else{O=o}o=b+24|0;do{if((c[o>>2]|0)==0){P=O}else{v=d+24|0;if((c[v>>2]|0)==0){P=O;break}s=b+48|0;M=0;do{c[(c[s>>2]|0)+(M*264|0)+104>>2]=1885431907;L=c[s>>2]|0;c[L+(M*264|0)+108>>2]=c[L+((c[o>>2]|0)*264|0)+68>>2];M=M+1|0;}while(M>>>0<(c[v>>2]|0)>>>0);P=c[n>>2]|0}}while(0);do{if((P|0)>0){o=b+48|0;O=0;v=P;while(1){M=c[o>>2]|0;if((c[M+(O*264|0)+72>>2]|0)==544240754){c[M+(O*264|0)+104>>2]=1953393e3;M=c[o>>2]|0;c[M+(O*264|0)+108>>2]=c[M+((c[M+(O*264|0)+124>>2]|0)*264|0)+68>>2];Q=c[n>>2]|0}else{Q=v}M=O+1|0;if((M|0)<(Q|0)){O=M;v=Q}else{break}}if((Q|0)<=0){R=1;S=0;T=0;U=0;u=9834;break}v=b+48|0;O=0;o=Q;while(1){M=c[v>>2]|0;if((c[M+(O*264|0)+72>>2]|0)==1684237684){s=c[M+(O*264|0)+124>>2]|0;c[M+(s*264|0)+104>>2]=1684237684;M=c[v>>2]|0;c[M+(s*264|0)+108>>2]=c[M+(O*264|0)+68>>2];M=c[v>>2]|0;L=M+(s*264|0)+24|0;s=c[L+4>>2]|0;N=M+(O*264|0)+24|0;c[N>>2]=c[L>>2];c[N+4>>2]=s;V=c[n>>2]|0}else{V=o}s=O+1|0;if((s|0)<(V|0)){O=s;o=V}else{break}}if((V|0)<=0){R=1;S=0;T=0;U=0;u=9834;break}o=c[b+48>>2]|0;O=0;v=0;s=0;N=1;while(1){if((c[o+(O*264|0)+4>>2]|0)>0){L=o+(O*264|0)+24|0;M=AV(c[L>>2]|0,c[L+4>>2]|0,1e3,0,c[o+(O*264|0)+8>>2]|0,0,3)|0;L=G;p=(v|0)<(L|0)|(v|0)==(L|0)&s>>>0<M>>>0;r=c[o+(O*264|0)+68>>2]|0;W=(N|0)<(r|0)?r:N;X=p?L:v;Y=p?M:s}else{W=N;X=v;Y=s}M=O+1|0;if((M|0)<(V|0)){O=M;v=X;s=Y;N=W}else{break}}N=0;s=(X|0)>(N|0)|(X|0)==(N|0)&Y>>>0>-2>>>0;N=s&1;if(!s){R=W;S=X;T=Y;U=N;u=9834;break}is(a,120);Z=W;_=X;$=Y;aa=1;ab=N}else{R=1;S=0;T=0;U=0;u=9834}}while(0);if((u|0)==9834){is(a,108);Z=R;_=S;$=T;aa=0;ab=U}ir(a,1684567661);ih(a,ab);iz(a,0);ab=b+8|0;U=c[ab>>2]|0;if(aa){ix(a,U,c[ab+4>>2]|0);ix(a,c[ab>>2]|0,c[ab+4>>2]|0);is(a,1e3);ix(a,$,_)}else{is(a,U);is(a,c[ab>>2]|0);is(a,1e3);is(a,$)}is(a,65536);iy(a,256);iy(a,0);is(a,0);is(a,0);is(a,65536);is(a,0);is(a,0);is(a,0);is(a,65536);is(a,0);is(a,0);is(a,0);is(a,1073741824);is(a,0);is(a,0);is(a,0);is(a,0);is(a,0);is(a,0);is(a,Z+1|0);Z=b+4|0;do{if((c[Z>>2]|0)!=2){if((c[b+72>>2]|0)!=0){break}$=il(a,0,0,1)|0;ab=G;U=c[b+80>>2]|0;_=c[b+76>>2]|0;aa=c[n>>2]|0;if((aa|0)>0){T=c[b+48>>2]|0;S=0;R=0;Y=0;while(1){if((c[T+(Y*264|0)+4>>2]|0)>0){X=c[(c[T+(Y*264|0)+76>>2]|0)+8>>2]|0;ad=(X|0)==0|R;ae=(X|0)==1|S}else{ad=R;ae=S}X=Y+1|0;if((X|0)<(aa|0)){S=ae;R=ad;Y=X}else{af=ae;ag=ad;break}}}else{af=0;ag=0}is(a,0);ir(a,1935961961);is(a,0);ih(a,16);ih(a,128);ih(a,128);ih(a,128);ih(a,7);iy(a,79);ih(a,255);ih(a,255);ih(a,(U|0)<0?255-af|0:U);ih(a,(_|0)<0?255-ag|0:_);ih(a,255);Y=il(a,0,0,1)|0;R=G;il(a,$,ab,0)|0;S=NV(Y,R,$,ab)|0;is(a,S);il(a,Y,R,0)|0;}}while(0);ag=c[n>>2]|0;L13249:do{if((ag|0)>0){af=b+48|0;ad=d+24|0;ae=d+28|0;R=b+108|0;Y=b+52|0;S=j|0;aa=0;T=ag;while(1){X=c[af>>2]|0;if((c[X+(aa*264|0)+4>>2]|0)>0){u=9851}else{if((c[Y>>2]&2|0)==0){ah=T}else{u=9851}}if((u|0)==9851){u=0;W=c[ad>>2]|0;do{if(aa>>>0<W>>>0){V=c[(c[(c[(c[ae>>2]|0)+(aa<<2)>>2]|0)+8>>2]|0)+8>>2]|0;if(!((V|0)==3|(V|0)==1)){ai=X;aj=W;break}Q=k+(V<<2)|0;c[X+(aa*264|0)+64>>2]=c[Q>>2];c[Q>>2]=1;ai=c[af>>2]|0;aj=c[ad>>2]|0}else{ai=X;aj=W}}while(0);W=ai+(aa*264|0)|0;if(aa>>>0<aj>>>0){ak=c[(c[ae>>2]|0)+(aa<<2)>>2]|0}else{ak=0}X=il(a,0,0,1)|0;Q=G;is(a,0);ir(a,1801548404);V=ai+(aa*264|0)+24|0;P=ai+(aa*264|0)+8|0;N=AV(c[V>>2]|0,c[V+4>>2]|0,1e3,0,c[P>>2]|0,0,3)|0;s=G;v=0;O=W|0;o=(c[O>>2]|0)==64?1:((s|0)>(v|0)|(s|0)==(v|0)&N>>>0>2147483646>>>0)&1;v=(o|0)==1;if(v){is(a,104)}else{is(a,92)}ir(a,1684564852);ih(a,o);iz(a,(c[ai+(aa*264|0)+64>>2]|0)!=0?2:15);o=ai+(aa*264|0)+16|0;M=c[o>>2]|0;if(v){ix(a,M,c[o+4>>2]|0);ix(a,c[o>>2]|0,c[o+4>>2]|0)}else{is(a,M);is(a,c[o>>2]|0)}M=ai+(aa*264|0)+68|0;is(a,c[M>>2]|0);is(a,0);p=ai+(aa*264|0)+4|0;do{if((c[p>>2]|0)==0){if(v){ix(a,-1,-1);break}else{is(a,-1);break}}else{if(v){ix(a,N,s);break}else{is(a,N);break}}}while(0);is(a,0);is(a,0);iy(a,0);N=(ak|0)!=0;if(N){al=c[(c[ak+8>>2]|0)+8>>2]|0}else{al=0}iy(a,al);s=ai+(aa*264|0)+76|0;if((c[(c[s>>2]|0)+8>>2]|0)==1){iy(a,256)}else{iy(a,0)}iy(a,0);do{if(N){v=c[ak+88>>2]|0;do{if((v|0)==0){u=9882}else{L=zJ(v,1697624,0,0)|0;if((L|0)==0){u=9882;break}r=c[L+4>>2]|0;if((r|0)==0){u=9882;break}L=cc(r|0)|0;if((L|0)==90){r=c[(c[s>>2]|0)+124>>2]|0;is(a,0);is(a,65536);is(a,0);is(a,-65536);is(a,0);is(a,0);is(a,r<<16);is(a,0);is(a,1073741824);break}else if((L|0)==180){r=c[s>>2]|0;q=c[r+120>>2]|0;t=c[r+124>>2]|0;is(a,-65536);is(a,0);is(a,0);is(a,0);is(a,-65536);is(a,0);is(a,q<<16);is(a,t<<16);is(a,1073741824);break}else if((L|0)==270){L=c[(c[s>>2]|0)+120>>2]|0;is(a,0);is(a,-65536);is(a,0);is(a,65536);is(a,0);is(a,0);is(a,0);is(a,L<<16);is(a,1073741824);break}else{u=9882;break}}}while(0);if((u|0)==9882){u=0;is(a,65536);is(a,0);is(a,0);is(a,0);is(a,65536);is(a,0);is(a,0);is(a,0);is(a,1073741824)}v=c[s>>2]|0;B=c[v+8>>2]|0;if(!((B|0)==0|(B|0)==3)){u=9891;break}if((c[O>>2]|0)==2){is(a,c[v+120>>2]<<16);is(a,c[ai+(aa*264|0)+100>>2]<<16);break}am=+(c[ak+80>>2]|0)/+(c[ak+84>>2]|0);B=ai+(aa*264|0)+100|0;if(am!=0.0){if((c[B>>2]|0)==(c[v+124>>2]|0)){an=am}else{u=9888}}else{u=9888}if((u|0)==9888){u=0;an=1.0}is(a,~~(an*+(c[v+120>>2]|0)*65536.0));is(a,c[B>>2]<<16)}else{is(a,65536);is(a,0);is(a,0);is(a,0);is(a,65536);is(a,0);is(a,0);is(a,0);is(a,1073741824);u=9891}}while(0);if((u|0)==9891){u=0;is(a,0);is(a,0)}N=c[R>>2]|0;if((N|0)<0){if((c[Y>>2]&2|0)==0){u=9895}}else{if((N|0)>0){u=9895}}if((u|0)==9895){u=0;N=c[P>>2]|0;B=0;v=AV(c[V>>2]|0,c[V+4>>2]|0,1e3,0,N,B,3)|0;L=G;t=0;q=ai+(aa*264|0)+88|0;r=c[q>>2]|0;w=c[r+32>>2]|0;E=w;A=(w|0)<0|0?-1:0;w=r+8|0;r=NU(c[w>>2]|0,c[w+4>>2]|0,E,A)|0;w=AV(r,G,1e3,0,N,B,2)|0;B=G;N=0;r=((B|0)>(N|0)|(B|0)==(N|0)&w>>>0>2147483646>>>0)&1|((L|0)>(t|0)|(L|0)==(t|0)&v>>>0>2147483646>>>0)&1;t=(r|0)==1;N=0;D=(B|0)>(N|0)|(B|0)==(N|0)&w>>>0>0>>>0;N=D?2:1;x=ac(t?20:12,N)|0;is(a,x+24|0);ir(a,1937007717);is(a,x+16|0);ir(a,1953721445);ih(a,r);iz(a,0);is(a,N);if(D){if(t){ix(a,w,B);ix(a,-1,-1)}else{is(a,w);is(a,-1)}is(a,65536);ao=A;ap=E;aq=L;ar=v}else{E=(c[q>>2]|0)+8|0;q=c[E>>2]|0;A=c[E+4>>2]|0;E=AV(q,A,1e3,0,c[P>>2]|0,0,2)|0;D=G;N=0;if(!((D|0)<(N|0)|(D|0)==(N|0)&E>>>0<1>>>0)){break}E=0;N=(A|0)>(E|0)|(A|0)==(E|0)&q>>>0>0>>>0;E=NV(0,0,q,A)|0;A=N?0:G;q=NU(w,B,v,L)|0;ao=A;ap=N?0:E;aq=G;ar=q}if(t){ix(a,ar,aq);ix(a,ap,ao)}else{is(a,ar);is(a,ap)}is(a,65536)}t=ai+(aa*264|0)+104|0;if((c[t>>2]|0)!=0){is(a,20);ir(a,1717924468);is(a,12);ir(a,c[t>>2]|0);is(a,c[ai+(aa*264|0)+108>>2]|0)}t=il(a,0,0,1)|0;q=G;is(a,0);ir(a,1634296941);E=c[V+4>>2]|0;N=0;A=(c[O>>2]|0)==64?1:((E|0)>(N|0)|(E|0)==(N|0)&(c[V>>2]|0)>>>0>2147483646>>>0)&1;N=(A|0)==1;if(N){is(a,44)}else{is(a,32)}ir(a,1684563053);ih(a,A);iz(a,0);E=c[o>>2]|0;if(N){ix(a,E,c[o+4>>2]|0);ix(a,c[o>>2]|0,c[o+4>>2]|0)}else{is(a,E);is(a,c[o>>2]|0)}is(a,c[P>>2]|0);do{if((c[p>>2]|0)==0){if(N){ix(a,-1,-1);break}else{is(a,-1);break}}else{E=c[V>>2]|0;if(N){ix(a,E,c[V+4>>2]|0);break}else{is(a,E);break}}}while(0);iy(a,c[ai+(aa*264|0)+60>>2]|0);iy(a,0);do{if((A|0)!=0){if((c[O>>2]|0)!=2){break}AN(0,16,1699272,(as=i,i=i+1|0,i=i+7&-8,c[as>>2]=0,as)|0);i=as}}while(0);kM(a,W);A=il(a,0,0,1)|0;V=G;is(a,0);ir(a,1718511981);N=c[(c[s>>2]|0)+8>>2]|0;do{if((N|0)==1){is(a,16);ir(a,1684565363);is(a,0);iy(a,0);iy(a,0)}else if((N|0)==0){is(a,20);ir(a,1684565366);is(a,1);ix(a,0,0)}else{p=c[ai+(aa*264|0)+72>>2]|0;if((N|0)==3){if((p|0)==1954047348|(p|0)==942683747){kN(a,W);break}else{is(a,12);ir(a,1684565358);is(a,0);break}}else{if((p|0)==1684237684){kN(a,W);break}else if((p|0)==544240754){is(a,28);ir(a,1684565352);is(a,0);iy(a,0);iy(a,0);is(a,0);is(a,0);is(a,0);break}else{break}}}}while(0);if((c[O>>2]|0)==2){kM(a,0)}N=il(a,0,0,1)|0;p=G;is(a,0);ir(a,1718511972);is(a,28);ir(a,1717924452);is(a,0);is(a,1);is(a,12);ir(a,543978101);is(a,1);P=il(a,0,0,1)|0;o=G;il(a,N,p,0)|0;E=NV(P,o,N,p)|0;is(a,E);il(a,P,o,0)|0;kO(a,W);o=il(a,0,0,1)|0;P=G;il(a,A,V,0)|0;E=NV(o,P,A,V)|0;is(a,E);il(a,o,P,0)|0;P=il(a,0,0,1)|0;o=G;il(a,t,q,0)|0;E=NV(P,o,t,q)|0;is(a,E);il(a,P,o,0)|0;if((c[O>>2]|0)==8){is(a,52);ir(a,1684632949);ir(a,1414353749);is(a,567431118);is(a,-1148688036);is(a,-87439552);is(a,28);ir(a,1413764173);is(a,65554);is(a,10);is(a,1438908416);is(a,1);is(a,0)}if((c[ai+(aa*264|0)+72>>2]|0)==544240754){o=c[ai+(aa*264|0)+128>>2]|0;NM(S|0,0,1e3);l7(S,1e3,c[c[o+28>>2]>>2]|0,c[ai+(aa*264|0)+124>>2]|0,0,0,0,0,o);o=c[M>>2]|0;y0(S,1e3,1719288,(as=i,i=i+8|0,c[as>>2]=o,as)|0)|0;i=as;o=NL(S|0)|0;is(a,o+24|0);ir(a,1635017845);is(a,o+16|0);ir(a,1769238120);is(a,o+8|0);ir(a,544236659);ij(a,S,o)}o=c[s>>2]|0;do{if((c[o+8>>2]|0)==0){if((c[O>>2]|0)!=2){break}P=c[ak+80>>2]|0;if(!((P|0)!=0&+(P|0)/+(c[ak+84>>2]|0)!=1.0)){break}P=c[o+224>>2]|0;E=c[o+120>>2]|0;p=c[o+228>>2]|0;N=AW(P,(P|0)<0|0?-1:0,E,(E|0)<0|0?-1:0,p,(p|0)<0|0?-1:0)|0;p=il(a,0,0,1)|0;E=G;is(a,0);ir(a,1953522036);is(a,20);ir(a,1717922915);is(a,0);P=N<<16;is(a,P);is(a,c[(c[s>>2]|0)+124>>2]<<16);is(a,20);ir(a,1718579824);is(a,0);is(a,P);is(a,c[(c[s>>2]|0)+124>>2]<<16);is(a,20);ir(a,1718578789);is(a,0);is(a,c[(c[s>>2]|0)+120>>2]<<16);is(a,c[(c[s>>2]|0)+124>>2]<<16);P=il(a,0,0,1)|0;N=G;il(a,p,E,0)|0;L=NV(P,N,p,E)|0;is(a,L);il(a,P,N,0)|0;}}while(0);s=il(a,0,0,1)|0;o=G;il(a,X,Q,0)|0;O=NV(s,o,X,Q)|0;is(a,O);il(a,s,o,0)|0;ah=c[n>>2]|0}o=aa+1|0;if((o|0)<(ah|0)){aa=o;T=ah}else{at=Y;break L13249}}AN(0,0,1698272,(as=i,i=i+24|0,c[as>>2]=1697960,c[as+8>>2]=1697840,c[as+16>>2]=1729,as)|0);i=as;cw()}else{at=b+52|0}}while(0);if((c[at>>2]&2|0)!=0){at=il(a,0,0,1)|0;as=G;is(a,0);ir(a,2019915373);if((c[n>>2]|0)>0){ah=b+48|0;ak=0;do{ai=c[ah>>2]|0;is(a,32);ir(a,2019914356);is(a,0);is(a,c[ai+(ak*264|0)+68>>2]|0);is(a,1);is(a,0);is(a,0);is(a,0);ak=ak+1|0;}while((ak|0)<(c[n>>2]|0))}n=il(a,0,0,1)|0;ak=G;il(a,at,as,0)|0;ah=NV(n,ak,at,as)|0;is(a,ah);il(a,n,ak,0)|0;}if((c[Z>>2]|0)==8){ak=zJ(c[d+1140>>2]|0,1624192,0,0)|0;if((ak|0)==0){au=0;av=0;aw=il(a,au,av,1)|0;ax=G;ay=il(a,l,m,0)|0;az=G;aA=NV(aw,ax,l,m)|0;aB=G;aC=aA;aD=aC;is(a,aD);aE=il(a,aw,ax,0)|0;aF=G;i=e;return}n=il(a,0,0,1)|0;ah=G;is(a,0);ir(a,1684632949);ir(a,1414353749);is(a,567431118);is(a,-1148688036);is(a,-87439552);as=il(a,0,0,1)|0;at=G;is(a,0);ir(a,1413764173);iy(a,4);iy(a,12);is(a,11);iy(a,21956);iy(a,0);iy(a,540);kL(a,1722672,1724120,4);kL(a,c[ak+4>>2]|0,1724120,1);kL(a,1722048,1725720,3);ak=il(a,0,0,1)|0;ai=G;il(a,as,at,0)|0;ap=NV(ak,ai,as,at)|0;is(a,ap);il(a,ak,ai,0)|0;ai=il(a,0,0,1)|0;ak=G;il(a,n,ah,0)|0;ap=NV(ai,ak,n,ah)|0;is(a,ap);il(a,ai,ak,0)|0;au=0;av=0;aw=il(a,au,av,1)|0;ax=G;ay=il(a,l,m,0)|0;az=G;aA=NV(aw,ax,l,m)|0;aB=G;aC=aA;aD=aC;is(a,aD);aE=il(a,aw,ax,0)|0;aF=G;i=e;return}ak=c[d+24>>2]|0;ai=b+48|0;b=0;while(1){if(b>>>0>=ak>>>0){break}if((c[(c[(c[ai>>2]|0)+(b*264|0)+76>>2]|0)+88>>2]&8388608|0)==0){b=b+1|0}else{u=9985;break}}if((u|0)==9985){au=0;av=0;aw=il(a,au,av,1)|0;ax=G;ay=il(a,l,m,0)|0;az=G;aA=NV(aw,ax,l,m)|0;aB=G;aC=aA;aD=aC;is(a,aD);aE=il(a,aw,ax,0)|0;aF=G;i=e;return}if((iZ(g)|0)<0){au=0;av=0;aw=il(a,au,av,1)|0;ax=G;ay=il(a,l,m,0)|0;az=G;aA=NV(aw,ax,l,m)|0;aB=G;aC=aA;aD=aC;is(a,aD);aE=il(a,aw,ax,0)|0;aF=G;i=e;return}u=c[Z>>2]|0;do{if((u&4|0)==0){Z=c[g>>2]|0;if((u|0)==2){kI(d,Z,1616960,1625440,0);kI(d,c[g>>2]|0,1616640,1624192,0);kI(d,c[g>>2]|0,1616344,1622856,0);kI(d,c[g>>2]|0,1616024,1619104,0);kI(d,c[g>>2]|0,1615584,1617320,0);kI(d,c[g>>2]|0,1614944,1614192,0);kI(d,c[g>>2]|0,1613752,1621304,0);kI(d,c[g>>2]|0,1613096,1621304,0);kI(d,c[g>>2]|0,1612552,1622072,0);kI(d,c[g>>2]|0,1612192,1618304,0);break}b=il(Z,0,0,1)|0;ai=G;is(Z,0);ir(Z,1635018093);is(Z,0);is(Z,33);ir(Z,1919706216);is(Z,0);is(Z,0);ir(Z,1919509613);ir(Z,1819308129);is(Z,0);is(Z,0);ih(Z,0);ak=il(Z,0,0,1)|0;ap=G;is(Z,0);ir(Z,1953721449);kI(d,Z,1616640,1624192,1);kI(d,Z,1616960,1625440,1);kI(d,Z,1744e3,1742280,1);kI(d,Z,1740808,1740112,1);kI(d,Z,1616024,1619104,1);kI(d,Z,1615584,1617320,1);kJ(Z,1739488,1739224,0,1)|0;kI(d,Z,1613096,1621304,1);kI(d,Z,1612552,1622072,1);kI(d,Z,1612192,1618304,1);kI(d,Z,1738904,1738704,1);kI(d,Z,1738360,1738e3,1);kI(d,Z,1737480,1736944,1);kI(d,Z,1736416,1735608,1);kI(d,Z,1734992,1734720,1);kI(d,Z,1734464,1734152,1);kI(d,Z,1733752,1733448,1);kK(d,Z,1732896,1732072,4);kK(d,Z,1731424,1730712,4);kK(d,Z,1730168,1729976,1);kK(d,Z,1729520,1729192,1);kK(d,Z,1728896,1728408,1);ah=d+1140|0;n=zJ(c[ah>>2]|0,1726896,0,0)|0;do{if((n|0)!=0){at=cc(c[n+4>>2]|0)|0;if((at|0)==0){break}is(Z,32);ir(Z,1852535412);is(Z,24);ir(Z,1635017060);is(Z,0);is(Z,0);iy(Z,0);iy(Z,at);iy(Z,0);iy(Z,0)}}while(0);n=zJ(c[ah>>2]|0,1727824,0,0)|0;do{if((n|0)!=0){at=cc(c[n+4>>2]|0)|0;if((at|0)==0){break}is(Z,26);ir(Z,1869639028);is(Z,18);ir(Z,1635017060);is(Z,21);is(Z,0);iy(Z,at)}}while(0);n=il(Z,0,0,1)|0;ah=G;il(Z,ak,ap,0)|0;at=NV(n,ah,ak,ap)|0;is(Z,at);il(Z,n,ah,0)|0;ah=il(Z,0,0,1)|0;n=G;il(Z,b,ai,0)|0;at=NV(ah,n,b,ai)|0;is(Z,at);il(Z,ah,n,0)|0;}else{kH(c[g>>2]|0,d,1626224,1625440);kH(c[g>>2]|0,d,1624688,1624192);kH(c[g>>2]|0,d,1623576,1622856);kH(c[g>>2]|0,d,1622456,1622072);kH(c[g>>2]|0,d,1621744,1621304);kH(c[g>>2]|0,d,1619776,1619104);kH(c[g>>2]|0,d,1618648,1618304);kH(c[g>>2]|0,d,1617880,1617320)}}while(0);u=d+1132|0;if((c[u>>2]|0)!=0){n=c[g>>2]|0;ah=il(n,0,0,1)|0;at=G;as=c[u>>2]|0;u=as>>>0>255?255:as;is(n,0);ir(n,1819306083);is(n,16777216);is(n,0);ih(n,u);if((u|0)>0){as=d+1136|0;d=f|0;ar=f+4|0;ao=0;do{aq=c[(c[as>>2]|0)+(ao<<2)>>2]|0;al=aq+16|0;aj=c[al>>2]|0;k=c[al+4>>2]|0;c[d>>2]=1;c[ar>>2]=1e7;al=AY(aj,k,aq+4|0,f)|0;ix(n,al,G);al=zJ(c[aq+32>>2]|0,1624192,0,0)|0;if((al|0)==0){ih(n,0)}else{aq=al+4|0;al=NL(c[aq>>2]|0)|0;k=al>>>0>255?255:al;ih(n,k);ij(n,c[aq>>2]|0,k)}ao=ao+1|0;}while((ao|0)<(u|0))}u=il(n,0,0,1)|0;ao=G;il(n,ah,at,0)|0;f=NV(u,ao,ah,at)|0;is(n,f);il(n,u,ao,0)|0;}ao=i$(c[g>>2]|0,h)|0;if((ao|0)>0){is(a,ao+8|0);ir(a,1635017845);ij(a,c[h>>2]|0,ao)}A4(c[h>>2]|0);au=0;av=0;aw=il(a,au,av,1)|0;ax=G;ay=il(a,l,m,0)|0;az=G;aA=NV(aw,ax,l,m)|0;aB=G;aC=aA;aD=aC;is(a,aD);aE=il(a,aw,ax,0)|0;aF=G;i=e;return}function kF(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0;b=i;i=i+32|0;d=b|0;e=b+8|0;f=b+16|0;g=b+24|0;h=a+12|0;j=c[h>>2]|0;k=j;l=j+52|0;m=c[l>>2]|0;if((m&2|0)==0){i=b;return}do{if((m&4|0)==0){n=j+84|0;if((c[n>>2]|0)!=0){break}o=a+16|0;p=il(c[o>>2]|0,0,0,1)|0;q=G;r=j+48|0;s=j+16|0;t=c[s>>2]|0;u=0;while(1){if((u|0)>=(t|0)){break}if((c[(c[r>>2]|0)+(u*264|0)+4>>2]|0)==0){v=10100;break}else{u=u+1|0}}if((v|0)==10100){i=b;return}u=c[h>>2]|0;t=iZ(e)|0;if((t|0)<0){w=t}else{kE(c[e>>2]|0,u,a);u=i$(c[e>>2]|0,d)|0;A4(c[d>>2]|0);w=u}if((c[s>>2]|0)>0){u=NU(p,q,8,0)|0;t=NU(u,G,w,(w|0)<0|0?-1:0)|0;u=G;x=0;do{y=(c[r>>2]|0)+(x*264|0)+192|0;c[y>>2]=t;c[y+4>>2]=u;x=x+1|0;}while((x|0)<(c[s>>2]|0))}kE(c[o>>2]|0,k,a);x=j+104|0;u=i$(c[x>>2]|0,f)|0;c[x>>2]=0;is(c[o>>2]|0,u+8|0);ir(c[o>>2]|0,1952539757);ij(c[o>>2]|0,c[f>>2]|0,u);A4(c[f>>2]|0);c[n>>2]=(c[n>>2]|0)+1;u=j+40|0;c[u>>2]=0;c[u+4>>2]=0;if((c[s>>2]|0)>0){u=0;do{x=c[r>>2]|0;if((c[x+(u*264|0)+4>>2]|0)==0){z=x}else{t=x+(u*264|0)+112|0;q=x+(u*264|0)+24|0;p=NU(c[q>>2]|0,c[q+4>>2]|0,c[t>>2]|0,c[t+4>>2]|0)|0;t=(c[x+(u*264|0)+88>>2]|0)+8|0;q=NV(p,G,c[t>>2]|0,c[t+4>>2]|0)|0;t=x+(u*264|0)+200|0;x=NU(q,G,c[t>>2]|0,c[t+4>>2]|0)|0;c[t>>2]=x;c[t+4>>2]=G;z=c[r>>2]|0}c[z+(u*264|0)+4>>2]=0;u=u+1|0;}while((u|0)<(c[s>>2]|0))}ik(c[o>>2]|0);i=b;return}}while(0);z=j+16|0;if((c[z>>2]|0)<=0){i=b;return}f=j+48|0;k=0;w=0;d=-1;e=0;h=m;while(1){m=c[f>>2]|0;s=(h&16|0)==0;u=m+(e*264|0)+192|0;c[u>>2]=s?w:0;c[u+4>>2]=s?k:0;s=c[m+(e*264|0)+180>>2]|0;if((s|0)==0){A=d;B=k;C=w}else{m=il(s,0,0,1)|0;s=NU(m,G,w,k)|0;A=(d|0)<0?e:d;B=G;C=s}s=e+1|0;D=c[z>>2]|0;if((s|0)>=(D|0)){break}k=B;w=C;d=A;e=s;h=c[l>>2]|0}if((C|0)==0&(B|0)==0){i=b;return}L13489:do{if((D|0)>0){h=j+48|0;e=a+16|0;d=j+100|0;w=j+84|0;k=j+4|0;f=B;s=C;m=0;while(1){u=c[h>>2]|0;r=u+(m*264|0)+4|0;n=c[r>>2]|0;if((n|0)==0){E=0;F=0}else{t=u+(m*264|0)+112|0;x=u+(m*264|0)+24|0;q=NU(c[x>>2]|0,c[x+4>>2]|0,c[t>>2]|0,c[t+4>>2]|0)|0;t=(c[u+(m*264|0)+88>>2]|0)+8|0;x=NV(q,G,c[t>>2]|0,c[t+4>>2]|0)|0;E=G;F=x}do{if((c[l>>2]&16|0)==0){if((m|0)==(A|0)){H=f;I=s;J=-1;v=10019}else{K=f;L=s;M=n;v=10091}}else{x=c[u+(m*264|0)+180>>2]|0;if((x|0)==0){N=f;O=s;break}t=il(x,0,0,1)|0;H=G;I=t;J=m;v=10019}}while(0);if((v|0)==10019){v=0;ik(c[e>>2]|0);n=u+(m*264|0)+216|0;t=c[n>>2]|0;x=t+1|0;c[n>>2]=x;q=u+(m*264|0)+224|0;if(x>>>0<(c[q>>2]|0)>>>0){P=x;Q=u+(m*264|0)+220|0}else{x=t+65|0;t=u+(m*264|0)+220|0;if((A5(t,x,32)|0)!=0){break}c[q>>2]=x;P=c[n>>2]|0;Q=t}t=P-1|0;x=c[Q>>2]|0;q=il(c[e>>2]|0,0,0,1)|0;p=x+(t<<5)|0;c[p>>2]=q;c[p+4>>2]=G;p=(c[h>>2]|0)+(m*264|0)+200|0;q=c[p+4>>2]|0;y=x+(t<<5)+8|0;c[y>>2]=c[p>>2];c[y+4>>2]=q;q=x+(t<<5)+16|0;c[q>>2]=F;c[q+4>>2]=E;q=c[e>>2]|0;y=il(q,0,0,1)|0;p=G;R=c[d>>2]|0;if((R|0)>0){S=0;T=R;while(1){R=c[n>>2]|0;U=-2-S+R|0;V=R+~U|0;do{if((U|0)<0){W=T}else{R=(V<<4)+29|0;X=(c[Q>>2]|0)+(U<<5)+24|0;Y=c[X>>2]|0;Z=c[X+4>>2]|0;il(q,Y,Z,0)|0;is(q,R);ir(q,1684632949);ij(q,126168,16);ih(q,1);iz(q,0);ih(q,V);if((V|0)>0){R=U+1|0;Z=0;do{Y=R+Z|0;X=(c[Q>>2]|0)+(Y<<5)+8|0;ix(q,c[X>>2]|0,c[X+4>>2]|0);X=(c[Q>>2]|0)+(Y<<5)+16|0;ix(q,c[X>>2]|0,c[X+4>>2]|0);Z=Z+1|0;}while((Z|0)<(V|0))}Z=c[d>>2]|0;if((V|0)>=(Z|0)){W=Z;break}R=Z-V<<4;is(q,R);ir(q,1701147238);ii(q,0,R-8|0);W=c[d>>2]|0}}while(0);V=S+1|0;if((V|0)<(W|0)){S=V;T=W}else{break}}}il(q,y,p,0)|0;T=c[e>>2]|0;S=il(T,0,0,1)|0;n=G;is(T,0);ir(T,1718579053);is(T,16);ir(T,1684563565);is(T,0);is(T,c[w>>2]|0);V=c[z>>2]|0;if((V|0)>0){U=(J|0)<0;R=0;Z=V;while(1){V=c[h>>2]|0;do{if(U|(R|0)==(J|0)){X=V+(R*264|0)+4|0;if((c[X>>2]|0)==0){_=Z;break}Y=il(T,0,0,1)|0;$=G;is(T,0);ir(T,1717662324);aa=il(T,0,0,1)|0;ab=G;ac=(c[X>>2]|0)==0?65561:57;ad=(c[V+(R*264|0)>>2]|0)==64?ac&65568:ac;is(T,0);ir(T,1684563572);ih(T,0);iz(T,ad);is(T,c[V+(R*264|0)+68>>2]|0);if((ad&1|0)!=0){ix(T,S,n)}if((ad&8|0)!=0){ac=c[X>>2]|0;if((ac|0)>0){if((ac|0)==1){ac=V+(R*264|0)+24|0;ae=V+(R*264|0)+112|0;af=NU(c[ae>>2]|0,c[ae+4>>2]|0,c[ac>>2]|0,c[ac+4>>2]|0)|0;ag=G;ah=af;ai=c[V+(R*264|0)+88>>2]|0}else{af=c[V+(R*264|0)+88>>2]|0;ac=af+48|0;ag=c[ac+4>>2]|0;ah=c[ac>>2]|0;ai=af}af=ai+8|0;ac=NV(ah,ag,c[af>>2]|0,c[af+4>>2]|0)|0;aj=ac}else{aj=0}ac=V+(R*264|0)+152|0;c[ac>>2]=aj;c[ac+4>>2]=(aj|0)<0|0?-1:0;is(T,aj)}if((ad&16|0)==0){c[V+(R*264|0)+164>>2]=-1}else{if((c[X>>2]|0)==0){ak=1}else{ak=c[(c[V+(R*264|0)+88>>2]|0)+16>>2]|0}c[V+(R*264|0)+164>>2]=ak;is(T,ak)}if((ad&32|0)!=0){ad=(c[(c[V+(R*264|0)+76>>2]|0)+8>>2]|0)==0?16842752:33554432;c[V+(R*264|0)+160>>2]=ad;is(T,ad)}ad=il(T,0,0,1)|0;ac=G;il(T,aa,ab,0)|0;af=NV(ad,ac,aa,ab)|0;is(T,af);il(T,ad,ac,0)|0;ac=il(T,0,0,1)|0;ad=G;af=c[X>>2]|0;if((af|0)>0){ab=V+(R*264|0)+24|0;aa=V+(R*264|0)+112|0;ae=V+(R*264|0)+152|0;al=c[ae>>2]|0;am=c[ae+4>>2]|0;ae=c[V+(R*264|0)+88>>2]|0;an=c[V+(R*264|0)+164>>2]|0;ao=V+(R*264|0)+160|0;ap=1;aq=0;while(1){ar=aq+1|0;if((ar|0)==(af|0)){as=NU(c[aa>>2]|0,c[aa+4>>2]|0,c[ab>>2]|0,c[ab+4>>2]|0)|0;at=G;au=as}else{as=ae+(ar*40|0)+8|0;at=c[as+4>>2]|0;au=c[as>>2]|0}as=ae+(aq*40|0)+8|0;av=NV(au,at,c[as>>2]|0,c[as+4>>2]|0)|0;as=av;av=(as|0)==(al|0)&(((as|0)<0|0?-1:0)|0)==(am|0)?ap:ap|256;as=(c[ae+(aq*40|0)+16>>2]|0)==(an|0)?av:av|512;if((aq|0)>0){aw=(((c[ae+(aq*40|0)+36>>2]&1|0)!=0?33554432:16842752)|0)==(c[ao>>2]|0)?as:as|1024}else{aw=as}if((ar|0)<(af|0)){ap=aw;aq=ar}else{ax=aw;break}}}else{ax=1}aq=(ax&1024|0)==0?ax|4:ax;ap=(c[V+(R*264|0)+52>>2]&1|0)==0?aq:aq|2048;is(T,0);ir(T,1853190772);ih(T,0);iz(T,ap);is(T,c[X>>2]|0);aq=il(T,0,0,1)|0;af=V+(R*264|0)+184|0;c[af>>2]=aq;c[af+4>>2]=G;is(T,0);if((ap&4|0)!=0){is(T,(c[(c[V+(R*264|0)+88>>2]|0)+36>>2]&1|0)!=0?33554432:16842752)}af=c[X>>2]|0;if((af|0)>0){aq=(ap&256|0)==0;ao=(ap&512|0)==0;ae=(ap&1024|0)==0;an=(ap&2048|0)==0;ap=V+(R*264|0)+88|0;am=V+(R*264|0)+24|0;al=V+(R*264|0)+112|0;ab=0;aa=af;do{if(!aq){af=ab+1|0;if((af|0)==(aa|0)){ar=NU(c[al>>2]|0,c[al+4>>2]|0,c[am>>2]|0,c[am+4>>2]|0)|0;ay=G;az=ar;aA=c[ap>>2]|0}else{ar=c[ap>>2]|0;as=ar+(af*40|0)+8|0;ay=c[as+4>>2]|0;az=c[as>>2]|0;aA=ar}ar=aA+(ab*40|0)+8|0;as=NV(az,ay,c[ar>>2]|0,c[ar+4>>2]|0)|0;is(T,as)}if(!ao){is(T,c[(c[ap>>2]|0)+(ab*40|0)+16>>2]|0)}if(!ae){is(T,(c[(c[ap>>2]|0)+(ab*40|0)+36>>2]&1|0)!=0?33554432:16842752)}if(!an){is(T,c[(c[ap>>2]|0)+(ab*40|0)+32>>2]|0)}ab=ab+1|0;aa=c[X>>2]|0;}while((ab|0)<(aa|0))}aa=il(T,0,0,1)|0;ab=G;il(T,ac,ad,0)|0;X=NV(aa,ab,ac,ad)|0;is(T,X);il(T,aa,ab,0)|0;do{if((c[k>>2]|0)==64){ab=il(T,0,0,1)|0;aa=G;is(T,0);ir(T,1684632949);ij(T,126152,16);ih(T,1);iz(T,0);X=V+(R*264|0)+200|0;ix(T,c[X>>2]|0,c[X+4>>2]|0);X=V+(R*264|0)+112|0;ap=V+(R*264|0)+24|0;an=NU(c[ap>>2]|0,c[ap+4>>2]|0,c[X>>2]|0,c[X+4>>2]|0)|0;X=(c[V+(R*264|0)+88>>2]|0)+8|0;ap=NV(an,G,c[X>>2]|0,c[X+4>>2]|0)|0;ix(T,ap,G);ap=il(T,0,0,1)|0;X=G;il(T,ab,aa,0)|0;an=NV(ap,X,ab,aa)|0;is(T,an);il(T,ap,X,0)|0;X=c[d>>2]|0;if((X|0)==0){break}ap=X<<4;X=ap+21|0;an=il(T,0,0,1)|0;aa=V+(R*264|0)+208|0;c[aa>>2]=an;c[aa+4>>2]=G;is(T,ap+29|0);ir(T,1701147238);if((X|0)>0){aB=0}else{break}do{ih(T,0);aB=aB+1|0;}while((aB|0)<(X|0))}}while(0);ad=il(T,0,0,1)|0;ac=G;il(T,Y,$,0)|0;X=NV(ad,ac,Y,$)|0;is(T,X);il(T,ad,ac,0)|0;_=c[z>>2]|0}else{_=Z}}while(0);V=R+1|0;if((V|0)<(_|0)){R=V;Z=_}else{break}}}Z=il(T,0,0,1)|0;R=G;U=c[z>>2]|0;L13595:do{if((U|0)>0){p=NV(Z,R,S,n)|0;y=p+8|0;p=0;if((J|0)<0){q=0;V=U;while(1){ac=c[h>>2]|0;if((c[ac+(q*264|0)+4>>2]|0)==0){aC=V}else{ad=ac+(q*264|0)+184|0;ac=c[ad>>2]|0;X=c[ad+4>>2]|0;il(T,ac,X,0)|0;X=(c[h>>2]|0)+(q*264|0)+192|0;ac=NU(c[X>>2]|0,c[X+4>>2]|0,y,p)|0;is(T,ac);aC=c[z>>2]|0}ac=q+1|0;if((ac|0)<(aC|0)){q=ac;V=aC}else{break L13595}}}else{aD=0;aE=U}while(1){V=c[h>>2]|0;do{if((aD|0)==(J|0)){if((c[V+(J*264|0)+4>>2]|0)==0){aF=aE;break}q=V+(J*264|0)+184|0;ac=c[q>>2]|0;X=c[q+4>>2]|0;il(T,ac,X,0)|0;X=(c[h>>2]|0)+(J*264|0)+192|0;ac=NU(c[X>>2]|0,c[X+4>>2]|0,y,p)|0;is(T,ac);aF=c[z>>2]|0}else{aF=aE}}while(0);V=aD+1|0;if((V|0)<(aF|0)){aD=V;aE=aF}else{break}}}}while(0);il(T,Z,R,0)|0;U=il(T,0,0,1)|0;p=G;il(T,S,n,0)|0;y=NV(U,p,S,n)|0;is(T,y);il(T,U,p,0)|0;p=u+(m*264|0)+208|0;U=c[p+4>>2]|0;y=x+(t<<5)+24|0;c[y>>2]=c[p>>2];c[y+4>>2]=U;c[w>>2]=(c[w>>2]|0)+1;U=c[e>>2]|0;y=NU(I,H,8,0)|0;is(U,y);ir(c[e>>2]|0,1952539757);K=H;L=I;M=c[r>>2]|0;v=10091}do{if((v|0)==10091){v=0;if((M|0)!=0){y=u+(m*264|0)+200|0;U=NU(c[y>>2]|0,c[y+4>>2]|0,F,E)|0;c[y>>2]=U;c[y+4>>2]=G}c[r>>2]=0;y=u+(m*264|0)+180|0;U=c[y>>2]|0;if((U|0)==0){N=K;O=L;break}p=i$(U,g)|0;c[y>>2]=0;ij(c[e>>2]|0,c[g>>2]|0,p);A4(c[g>>2]|0);N=K;O=L}}while(0);u=m+1|0;if((u|0)<(c[z>>2]|0)){f=N;s=O;m=u}else{aG=e;break L13489}}i=b;return}else{aG=a+16|0}}while(0);a=j+40|0;c[a>>2]=0;c[a+4>>2]=0;ik(c[aG>>2]|0);i=b;return}function kG(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0;f=b+84|0;g=c[b+80>>2]|0;h=(c[f>>2]|0)+g|0;i=c[b+248>>2]|0;j=c[b+252>>2]|0;k=c[b+256>>2]|0;l=b+112|0;m=c[l>>2]|0;n=c[l+4>>2]|0;l=A7(g+16|0)|0;if((l|0)==0){o=-12;return o|0}g=c[f>>2]|0;f=h;L13624:do{if((f-g|0)>=4){p=-1;q=g;while(1){if(q>>>0>=h>>>0){break L13624}r=p<<8;if((r|0)==256){break}else{p=d[q]|r;q=q+1|0}}p=q-3|0;if(p>>>0<h>>>0){s=p;t=0;u=0;v=0}else{break}L13629:while(1){p=s;while(1){r=p+4|0;L13633:do{if((f-r|0)<4){w=h}else{x=r;z=-1;while(1){if(x>>>0>=h>>>0){w=h;break L13633}A=z<<8;B=d[x]|A;if((A|0)==256){break}else{x=x+1|0;z=B}}w=x-3|0}}while(0);z=w-p|0;B=z-4|0;if((B|0)>=1){if((B|0)<4){A=0;C=l;D=r;while(1){a[C]=a[D]|0;E=A+1|0;if((E|0)<(B|0)){A=E;C=C+1|0;D=D+1|0}else{F=B;break}}}else{D=z-5|0;C=0;A=0;E=r;while(1){G=a[E]|0;do{if(G<<24>>24==3&(C|0)>1){if((a[E-1|0]|0)!=0){H=10124;break}if(!((a[E-2|0]|0)==0&(C|0)<(D|0))){H=10124;break}I=E+1|0;J=a[I]|0;if((J&255)>=4){H=10124;break}a[l+A|0]=J;K=I;L=C+1|0}else{H=10124}}while(0);if((H|0)==10124){H=0;a[l+A|0]=G;K=E;L=C}x=A+1|0;I=L+1|0;if((I|0)<(B|0)){C=I;A=x;E=K+1|0}else{F=x;break}}}M=F<<3;N=M>>>0>2147483639;E=p;if((d[E]|d[E+1|0]<<8|d[E+2|0]<<16|d[E+3|0]<<24|0)==251723776){break}}if(w>>>0<h>>>0){p=w}else{H=10133;break L13629}}p=N?0:l;E=N?8:M+8|0;A=p;C=E>>>0>2?2:E;if(((d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24)&192|0)!=192){H=10129;break}A=p+(C>>>3)|0;B=(cV(d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24|0)|0)<<(C&2)>>>29;A=C+3|0;C=E>>>0>A>>>0?A:E;A=-C|0;D=E-C|0;if((A|0)>35){O=A}else{O=(D|0)<35?D:35}D=C+1+O|0;C=E>>>0>D>>>0?D:E;E=d[p+(C>>>3)|0]<<(C&7)>>>7&1;if(w>>>0<h>>>0){s=w;t=1;u=B;v=E}else{P=E;Q=B;break}}if((H|0)==10129){A4(l);o=-38;return o|0}else if((H|0)==10133){if((t|0)==0){break}else{P=v;Q=u}}q=(m|0)!=0|(n|0)!=(-2147483648|0);B=((((P|((Q<<4|Q)<<8|393216))<<1|(i|0)==0&q&1)<<1|(j|0)==0&q&1)<<1|(k|0)==0)<<2^32;q=c[(c[b+76>>2]|0)+108>>2]|0;E=e;y=B&65280|B>>>16&255|(B&252|q>>>24<<8)<<16;a[E]=y&255;y=y>>8;a[E+1|0]=y&255;y=y>>8;a[E+2|0]=y&255;y=y>>8;a[E+3|0]=y&255;E=q<<8;B=q>>>16&255;q=8;C=e+4|0;while(1){a[C]=B;p=E;E=p<<8;D=q+8|0;if((D|0)<32){B=p>>>16&255;q=D;C=C+1|0}else{break}}E=0;A4(l);o=0;return o|0}}while(0);A4(l);o=-38;return o|0}function kH(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;h=il(b,0,0,1)|0;i=G;j=e+1140|0;e=zJ(c[j>>2]|0,g,0,0)|0;if((e|0)==0){return}g=e+4|0;e=c[g>>2]|0;k=a[e]|0;if(k<<24>>24==0){return}else{l=e;m=0;n=k}L13679:while(1){k=n&255;e=k>>>1&64;if((k&192|0)==128|(n&255)>253){break}o=l+1|0;if((e&k|0)==0){p=o}else{q=k;k=e;e=o;while(1){o=(d[e]|0)-128|0;if(o>>>0>63){break L13679}r=o+(q<<6)|0;o=k<<5;s=e+1|0;if((r&o|0)==0){p=s;break}else{q=r;k=o;e=s}}}t=m+1|0;e=a[p]|0;if(e<<24>>24==0){u=10150;break}else{l=p;m=t;n=e}}do{if((u|0)==10150){if((t|0)!=0){break}return}}while(0);is(b,0);ir(b,(d[f+1|0]|0)<<8|(d[f]|0)|(d[f+2|0]|0)<<16|(d[f+3|0]|0)<<24);is(b,0);do{if((a8(f|0,1617880)|0)==0){iy(b,cc(c[g>>2]|0)|0)}else{iy(b,5575);t=c[g>>2]|0;ij(b,t,(NL(t|0)|0)+1|0);if((a8(f|0,1619776)|0)!=0){break}t=zJ(c[j>>2]|0,1726896,0,0)|0;if((t|0)==0){break}ih(b,cc(c[t+4>>2]|0)|0)}}while(0);j=il(b,0,0,1)|0;f=G;il(b,h,i,0)|0;g=NV(j,f,h,i)|0;is(b,g);il(b,j,f,0)|0;return}function kI(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;g=i;i=i+16|0;h=a+1140|0;a=zJ(c[h>>2]|0,e,0,0)|0;if((a|0)==0){i=g;return}j=NL(c[a>>2]|0)|0;k=g|0;bc(k|0,16,1724472,(l=i,i=i+8|0,c[l>>2]=e,l)|0)|0;i=l;l=zJ(c[h>>2]|0,k,0,2)|0;L13702:do{if((l|0)==0){m=0;n=a+4|0}else{e=j+4|0;o=a+4|0;p=l;while(1){q=c[p>>2]|0;do{if((NL(q|0)|0)==(e|0)){if((a8(c[o>>2]|0,c[p+4>>2]|0)|0)!=0){break}r=nr(q+(j+1)|0,1)|0;if((r|0)>-1){m=r;n=o;break L13702}}}while(0);q=zJ(c[h>>2]|0,k,p,2)|0;if((q|0)==0){m=0;n=o;break}else{p=q}}}}while(0);kJ(b,d,c[n>>2]|0,m,f)|0;i=g;return}function kJ(b,c,e,f,g){b=b|0;c=c|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0;if((e|0)==0){h=0;return h|0}if((a[e]|0)==0){h=0;return h|0}i=il(b,0,0,1)|0;j=G;is(b,0);ir(b,d[c+1|0]<<8|d[c]|d[c+2|0]<<16|d[c+3|0]<<24);if((g|0)==0){if((f|0)==0){k=nr(1725720,1)|0}else{k=f}iy(b,NL(e|0)|0);iy(b,k);ij(b,e,NL(e|0)|0)}else{is(b,(NL(e|0)|0)+16|0);ir(b,1635017060);is(b,1);is(b,0);ij(b,e,NL(e|0)|0)}e=il(b,0,0,1)|0;k=G;il(b,i,j,0)|0;f=NV(e,k,i,j)|0;j=f;is(b,j);il(b,e,k,0)|0;h=j;return h|0}function kK(a,b,e,f,g){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;var h=0;if(!((g|0)==4|(g|0)==1)){return}h=zJ(c[a+1140>>2]|0,f,0,0)|0;if((h|0)==0){return}f=cc(c[h+4>>2]|0)|0;is(b,g+24|0);ir(b,(d[e+1|0]|0)<<8|(d[e]|0)|(d[e+2|0]|0)<<16|(d[e+3|0]|0)<<24);is(b,g+16|0);ir(b,1635017060);is(b,21);is(b,0);e=f&255;if((g|0)==4){is(b,e);return}else{ih(b,e);return}}function kL(b,c,e,f){b=b|0;c=c|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;g=a[c]|0;do{if(g<<24>>24==0){h=0}else{i=c;j=0;k=g;L13742:while(1){l=k&255;m=l>>>1&64;if((l&192|0)==128|(k&255)>253){n=10214;break}o=i+1|0;if((m&l|0)==0){p=o}else{q=l;l=m;m=o;while(1){o=(d[m]|0)-128|0;if(o>>>0>63){n=10213;break L13742}r=o+(q<<6)|0;o=l<<5;s=m+1|0;if((r&o|0)==0){p=s;break}else{q=r;l=o;m=s}}}t=j+1|0;m=a[p]|0;if(m<<24>>24==0){n=10202;break}else{i=p;j=t;k=m}}if((n|0)==10202){if((t|0)>=0){h=t;break}return}else if((n|0)==10214){return}else if((n|0)==10213){return}}}while(0);iy(b,(h<<1)+12|0);is(b,f);iy(b,((d[e+1|0]|0)<<5&992|(d[e]|0)<<10&31744|a[e+2|0]&31)&65535);iy(b,1);e=a[c]|0;L13756:do{if(e<<24>>24!=0){f=c;h=e;L13757:while(1){t=h&255;p=t>>>1&64;if((t&192|0)==128|(h&255)>253){n=10212;break}g=f+1|0;if((p&t|0)==0){u=t;v=p;w=g}else{k=t;t=p;p=g;while(1){g=(d[p]|0)-128|0;if(g>>>0>63){n=10216;break L13757}j=g+(k<<6)|0;g=t<<5;i=p+1|0;if((j&g|0)==0){u=j;v=g;w=i;break}else{k=j;t=g;p=i}}}iy(b,(v<<1)-1&u);p=a[w]|0;if(p<<24>>24==0){break L13756}else{f=w;h=p}}if((n|0)==10212){return}else if((n|0)==10216){return}}}while(0);iy(b,0);return}function kM(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=il(a,0,0,1)|0;f=G;g=(b|0)!=0;do{if(g){h=(c[b>>2]|0)==2?1703608:3119136;i=c[b+76>>2]|0;j=c[i+8>>2]|0;if((j|0)==0){k=h;l=1702960;m=1702704;break}else if((j|0)==3){n=c[b+72>>2]|0;if((n|0)==942683747){k=h;l=1701696;m=1701432;break}else if((n|0)==1731426420){o=1701224}else{o=1704752}k=h;l=o;m=1701064;break}else if((j|0)==1){k=h;l=1702344;m=1702120;break}else{j=c[i+52>>2]|0;if((j|0)==1684237684){k=h;l=1713304;m=1700928;break}else if((j|0)==544240754){k=h;l=1700816;m=1700408;break}else{k=1703784;l=1705560;m=1703696;break}}}else{k=1703784;l=1705560;m=1703696}}while(0);is(a,0);ir(a,1919706216);is(a,0);ij(a,k,4);ir(a,(d[l+1|0]|0)<<8|(d[l]|0)|(d[l+2|0]|0)<<16|(d[l+3|0]|0)<<24);is(a,0);is(a,0);is(a,0);if(g){if((c[b>>2]|0)==2){p=10228}}else{p=10228}if((p|0)==10228){ih(a,NL(m|0)|0)}ij(a,m,NL(m|0)|0);do{if(g){if((c[b>>2]|0)==2){break}ih(a,0)}}while(0);b=il(a,0,0,1)|0;g=G;il(a,e,f,0)|0;m=NV(b,g,e,f)|0;is(a,m);il(a,b,g,0)|0;return}function kN(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;d=il(a,0,0,1)|0;e=G;is(a,0);ir(a,1684565351);is(a,24);ir(a,1852403047);is(a,0);iy(a,64);iy(a,32768);iy(a,32768);iy(a,32768);iy(a,0);iy(a,0);if((c[b+72>>2]|0)!=942683747){is(a,44);ir(a,1954047348);iy(a,1);is(a,0);is(a,0);is(a,0);is(a,1);is(a,0);is(a,0);is(a,0);is(a,16384);iy(a,0)}if((c[(c[b+76>>2]|0)+52>>2]|0)!=1684237684){f=0;g=0;h=il(a,f,g,1)|0;i=G;j=il(a,d,e,0)|0;k=G;l=NV(h,i,d,e)|0;m=G;n=l;o=n;is(a,o);p=il(a,h,i,0)|0;q=G;return}b=il(a,0,0,1)|0;r=G;is(a,0);ir(a,1684237684);s=il(a,0,0,1)|0;t=G;is(a,0);ir(a,1768776564);is(a,0);iy(a,0);iy(a,0);iy(a,12);iy(a,0);iy(a,0);iy(a,0);iy(a,0);iy(a,65535);iy(a,65535);iy(a,65535);ih(a,13);ij(a,1704512,13);u=il(a,0,0,1)|0;v=G;il(a,s,t,0)|0;w=NV(u,v,s,t)|0;is(a,w);il(a,u,v,0)|0;v=il(a,0,0,1)|0;u=G;il(a,b,r,0)|0;w=NV(v,u,b,r)|0;is(a,w);il(a,v,u,0)|0;f=0;g=0;h=il(a,f,g,1)|0;i=G;j=il(a,d,e,0)|0;k=G;l=NV(h,i,d,e)|0;m=G;n=l;o=n;is(a,o);p=il(a,h,i,0)|0;q=G;return}function kO(b,f){b=b|0;f=f|0;var g=0,j=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0;g=i;i=i+56|0;j=g|0;l=g+8|0;m=g+16|0;n=g+24|0;o=il(b,0,0,1)|0;p=G;is(b,0);ir(b,1818391667);q=il(b,0,0,1)|0;r=G;is(b,0);ir(b,1685288051);is(b,0);is(b,1);s=f+76|0;t=c[s>>2]|0;u=c[t+8>>2]|0;do{if((u|0)==1){v=il(b,0,0,1)|0;w=G;x=f+72|0;y=c[x>>2]|0;z=f|0;L13804:do{if((c[z>>2]|0)==2){if((c[f+8>>2]|0)>>>0>65535){switch(c[(c[s>>2]|0)+48>>2]|0){case 65556:case 65558:case 65557:case 65559:case 65541:case 65537:case 65549:case 65545:case 65540:case 65536:case 65548:case 65544:{A=1835233388;B=2;break L13804;break};default:{}}A=y;B=2;break}if((c[f+96>>2]|0)==0){C=c[(c[s>>2]|0)+48>>2]|0;if(!((C|0)==65557|(C|0)==65548|(C|0)==65544|(C|0)==65559|(C|0)==65556|(C|0)==65549|(C|0)==65545|(C|0)==65558|(C|0)==69638|(C|0)==69633|(C|0)==86035)){A=y;B=0;break}}A=y;B=1}else{A=y;B=0}}while(0);is(b,0);ir(b,A);is(b,0);iy(b,0);iy(b,1);iy(b,B);iy(b,0);is(b,0);if((B|0)==2){iy(b,3);iy(b,16);iy(b,65534);iy(b,0);is(b,65536);is(b,72);h[k>>3]=+(c[(c[s>>2]|0)+412>>2]|0);ix(b,c[k>>2]|0,c[k+4>>2]|0);is(b,c[(c[s>>2]|0)+416>>2]|0);is(b,2130706432);is(b,pc(c[(c[s>>2]|0)+48>>2]|0)|0);switch(c[(c[s>>2]|0)+48>>2]|0){case 65537:case 65549:case 65545:{D=14;break};case 65540:case 65536:case 65548:case 65544:{D=12;break};case 65556:case 65558:{D=11;break};case 65557:case 65559:{D=9;break};case 65541:{D=10;break};default:{D=0}}is(b,D);is(b,c[f+40>>2]|0);L13832:do{if((c[f+96>>2]|0)==0){E=1}else{y=c[f+4>>2]|0;if((y|0)==0){E=0;break}if((y|0)>0){if((y|0)==1){C=f+24|0;F=f+112|0;H=NU(c[F>>2]|0,c[F+4>>2]|0,c[C>>2]|0,c[C+4>>2]|0)|0;I=G;J=H;K=c[f+88>>2]|0}else{H=c[f+88>>2]|0;C=H+48|0;I=c[C+4>>2]|0;J=c[C>>2]|0;K=H}H=K+8|0;C=NV(J,I,c[H>>2]|0,c[H+4>>2]|0)|0;L=C}else{L=0}C=f+24|0;H=f+112|0;F=f+88|0;M=1;while(1){if((M|0)>=(y|0)){E=L;break L13832}N=M+1|0;if((N|0)==(y|0)){O=NU(c[H>>2]|0,c[H+4>>2]|0,c[C>>2]|0,c[C+4>>2]|0)|0;P=G;Q=O;R=c[F>>2]|0}else{O=c[F>>2]|0;S=O+(N*40|0)+8|0;P=c[S+4>>2]|0;Q=c[S>>2]|0;R=O}O=R+(M*40|0)+8|0;S=NV(Q,P,c[O>>2]|0,c[O+4>>2]|0)|0;if((S|0)==(L|0)){M=N}else{E=0;break}}}}while(0);is(b,E)}else{if((c[z>>2]|0)==2){iy(b,c[(c[s>>2]|0)+416>>2]|0);if(((c[(c[s>>2]|0)+48>>2]|0)-65540|0)>>>0<2){iy(b,8)}else{iy(b,16)}iy(b,(c[f+96>>2]|0)!=0?-2:0)}else{iy(b,2);iy(b,16);iy(b,0)}iy(b,0);M=c[(c[s>>2]|0)+412>>2]|0;iy(b,(M|0)<65536?M:0);iy(b,0)}M=(B|0)==1;if(M){F=c[s>>2]|0;switch(c[F+48>>2]|0){case 65557:case 65548:case 65544:case 65559:case 65556:case 65549:case 65545:case 65558:{is(b,1);break};default:{is(b,c[F+424>>2]|0)}}F=f+40|0;is(b,(c[F>>2]|0)/(c[(c[s>>2]|0)+416>>2]|0)|0);is(b,c[F>>2]|0);is(b,2)}do{if((c[z>>2]|0)==2){F=c[(c[s>>2]|0)+48>>2]|0;if((F|0)==65557|(F|0)==65548|(F|0)==65544|(F|0)==65559){if(!M){T=10333;break}}else if((F|0)==65556|(F|0)==65549|(F|0)==65545|(F|0)==65558){if(!M){T=10333;break}}else if(!((F|0)==86018|(F|0)==86019|(F|0)==73728|(F|0)==86032|(F|0)==69638|(F|0)==69633|(F|0)==86035)){T=10333;break}F=il(b,0,0,1)|0;C=G;is(b,0);ir(b,1702257015);H=c[s>>2]|0;do{if((c[H+48>>2]|0)==86035){U=H;T=10330}else{is(b,12);ir(b,1634562662);ir(b,c[x>>2]|0);y=c[s>>2]|0;N=c[y+48>>2]|0;if((N|0)==69638|(N|0)==69633){S=il(b,0,0,1)|0;O=G;is(b,0);ir(b,c[x>>2]|0);V=c[x>>2]|0;c[(c[s>>2]|0)+52>>2]=V>>>16<<8&65280|V>>>24;V=c[s>>2]|0;l1(b,V)|0;V=il(b,0,0,1)|0;W=G;il(b,S,O,0)|0;X=NV(V,W,S,O)|0;is(b,X);il(b,V,W,0)|0;break}else if((N|0)==65557|(N|0)==65548|(N|0)==65544|(N|0)==65559){is(b,10);ir(b,1633971813);iy(b,1);break}else if((N|0)==65556|(N|0)==65549|(N|0)==65545|(N|0)==65558){is(b,10);ir(b,1633971813);iy(b,0);break}else if((N|0)==73728){kR(b,f);break}else if((N|0)==86019){kS(b,f);break}else if((N|0)==86032|(N|0)==86035){U=y;T=10330;break}else if((N|0)==86018){is(b,12);ir(b,1630826605);is(b,0);kQ(b,f);break}else{break}}}while(0);if((T|0)==10330){ij(b,c[U+96>>2]|0,c[U+100>>2]|0)}is(b,8);is(b,0);H=il(b,0,0,1)|0;N=G;il(b,F,C,0)|0;y=NV(H,N,F,C)|0;is(b,y);il(b,H,N,0)|0;}else{T=10333}}while(0);do{if((T|0)==10333){if((c[x>>2]|0)==1630826605){kQ(b,f);break}M=c[s>>2]|0;N=c[M+48>>2]|0;if((N|0)==73728){kR(b,f);break}else if((N|0)==86019){kS(b,f);break}else if((N|0)==86032){ij(b,c[M+96>>2]|0,c[M+100>>2]|0);break}else if((N|0)==86054){N=il(b,0,0,1)|0;M=G;is(b,0);ir(b,2019911287);l1(b,c[s>>2]|0)|0;H=il(b,0,0,1)|0;y=G;il(b,N,M,0)|0;W=NV(H,y,N,M)|0;is(b,W);il(b,H,y,0)|0;break}else{y=f+80|0;H=c[y>>2]|0;if((H|0)<=0){break}is(b,H+8|0);ir(b,1818389607);ij(b,c[f+84>>2]|0,c[y>>2]|0);break}}}while(0);do{if((c[z>>2]|0)==2){if((c[(c[s>>2]|0)+8>>2]|0)!=1){break}x=il(b,0,0,1)|0;y=G;H=c[s>>2]|0;W=H+448|0;M=ky(c[H+48>>2]|0,c[W>>2]|0,c[W+4>>2]|0,j)|0;if((M|0)==0){AN(c[s>>2]|0,24,1712080,(Y=i,i=i+1|0,i=i+7&-8,c[Y>>2]=0,Y)|0);i=Y;break}else{is(b,0);ir(b,1851877475);ih(b,0);iz(b,0);is(b,M);is(b,c[j>>2]|0);is(b,0);M=il(b,0,0,1)|0;W=G;il(b,x,y,0)|0;H=NV(M,W,x,y)|0;is(b,H);il(b,M,W,0)|0;break}}}while(0);z=il(b,0,0,1)|0;W=G;il(b,v,w,0)|0;M=NV(z,W,v,w)|0;is(b,M);il(b,z,W,0)|0;}else if((u|0)==0){W=n|0;z=il(b,0,0,1)|0;M=G;NM(W|0,0,32);is(b,0);H=f+72|0;ir(b,c[H>>2]|0);is(b,0);iy(b,0);iy(b,1);iy(b,0);iy(b,0);y=f|0;do{if((c[y>>2]|0)==2){ir(b,1347241542);if((c[(c[s>>2]|0)+48>>2]|0)==14){is(b,0);is(b,1024);break}else{is(b,512);is(b,512);break}}else{is(b,0);is(b,0);is(b,0)}}while(0);iy(b,c[(c[s>>2]|0)+120>>2]|0);iy(b,c[f+100>>2]|0);is(b,4718592);is(b,4718592);is(b,0);iy(b,1);do{if((c[y>>2]|0)==2){w=c[(c[s>>2]|0)+12>>2]|0;if((w|0)==0){break}v=c[w>>2]|0;if((v|0)==0){break}y_(W,v,32)|0}}while(0);ih(b,NL(W|0)|0);ij(b,W,31);do{if((c[y>>2]|0)==2){v=c[(c[s>>2]|0)+792>>2]|0;if((v|0)==0){T=10253;break}iy(b,v)}else{T=10253}}while(0);if((T|0)==10253){iy(b,24)}iy(b,65535);do{if((c[H>>2]|0)==1983148141){kQ(b,f)}else{W=c[s>>2]|0;v=c[W+48>>2]|0;if((v|0)==1096176969|(v|0)==24){ij(b,c[W+96>>2]|0,c[W+100>>2]|0);is(b,0);break}else if((v|0)==100){is(b,24);ir(b,1380729665);ir(b,1380729665);ir(b,825241648);is(b,2);is(b,0);is(b,24);ir(b,1196576833);ir(b,1196576833);ir(b,825241648);is(b,1);is(b,0);is(b,120);ir(b,1397051969);ir(b,1397051969);ir(b,825241648);W=f+84|0;w=(c[W>>2]|0)+40|0;is(b,cV(d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0)|0);is(b,c[(c[s>>2]|0)+120>>2]|0);w=c[(c[s>>2]|0)+124>>2]|0;do{if((a[(c[W>>2]|0)+5|0]&2)==0){is(b,w);is(b,1);is(b,0);if((c[(c[s>>2]|0)+124>>2]|0)==1080){is(b,5);break}else{is(b,6);break}}else{is(b,(w|0)/2|0);is(b,2);is(b,0);is(b,4)}}while(0);ix(b,0,0);ix(b,0,0);ix(b,0,0);ix(b,0,0);ix(b,0,0);ix(b,0,0);ix(b,0,0);ix(b,0,0);ix(b,0,0);ix(b,0,0);is(b,0);break}else if((v|0)==71){w=f+80|0;if((c[w>>2]|0)<=0){break}W=m|0;NM(W|0,0,7);if((kG(f,W)|0)<0){break}is(b,(c[w>>2]|0)+15|0);ir(b,828601956);C=il(b,0,0,1)|0;F=f+232|0;c[F>>2]=C;c[F+4>>2]=G;ij(b,W,7);ij(b,c[f+84>>2]|0,c[w>>2]|0);break}else if((v|0)==5){is(b,15);ir(b,859189860);ir(b,1347241542);ih(b,0);ih(b,10);ih(b,0);break}else if((v|0)==28){w=il(b,0,0,1)|0;W=G;is(b,0);ir(b,1130591841);nh(b,c[f+84>>2]|0,c[f+80>>2]|0)|0;F=il(b,0,0,1)|0;C=G;il(b,w,W,0)|0;x=NV(F,C,w,W)|0;is(b,x);il(b,F,C,0)|0;if((c[y>>2]|0)!=32){break}is(b,28);ir(b,1684632949);is(b,1801994482);is(b,1596215237);is(b,-1170627301);is(b,-821877773);is(b,0);break}else{C=f+80|0;F=c[C>>2]|0;if((F|0)<=0){break}is(b,F+8|0);ir(b,1818389607);ij(b,c[f+84>>2]|0,c[C>>2]|0);break}}}while(0);y=c[s>>2]|0;H=c[y+48>>2]|0;do{if((H|0)==28|(H|0)==13|(H|0)==100){Z=y}else{C=c[y+408>>2]|0;if(!((C|0)!=0&C>>>0<6)){Z=y;break}F=e[236008+(C<<1)>>1]|0;is(b,10);ir(b,1818585446);iy(b,F);Z=c[s>>2]|0}}while(0);y=c[Z+228>>2]|0;do{if((y|0)!=0){H=c[Z+224>>2]|0;if((H|0)==0|(y|0)==(H|0)){break}F=l|0;C=l+4|0;x=H;W=(H|0)<0|0?-1:0;H=y;w=(y|0)<0|0?-1:0;N=2147483647;V=0;BX(F,C,x,W,H,w,N,V)|0;is(b,16);ir(b,1886609776);is(b,c[F>>2]|0);is(b,c[C>>2]|0)}}while(0);y=il(b,0,0,1)|0;C=G;il(b,z,M,0)|0;F=NV(y,C,z,M)|0;is(b,F);il(b,y,C,0)|0;}else if((u|0)==3){C=il(b,0,0,1)|0;y=G;is(b,0);ir(b,c[f+72>>2]|0);is(b,0);iy(b,0);iy(b,1);F=c[s>>2]|0;V=c[F+100>>2]|0;if((V|0)!=0){ij(b,c[F+96>>2]|0,V)}V=il(b,0,0,1)|0;F=G;il(b,C,y,0)|0;N=NV(V,F,C,y)|0;is(b,N);il(b,V,F,0)|0;}else{F=c[t+52>>2]|0;if((F|0)==544240754){V=il(b,0,0,1)|0;N=G;is(b,0);ir(b,544240754);is(b,0);iy(b,0);iy(b,1);iy(b,1);iy(b,1);is(b,c[f+144>>2]|0);is(b,12);ir(b,1936550260);is(b,c[f+8>>2]|0);y=il(b,0,0,1)|0;C=G;il(b,V,N,0)|0;w=NV(y,C,V,N)|0;is(b,w);il(b,y,C,0)|0;break}else if((F|0)!=1684237684){break}F=il(b,0,0,1)|0;C=G;y=f+8|0;w=c[s>>2]|0;N=c[w+104>>2]|0;V=c[w+108>>2]|0;w=~~(1.0/(+(N|0)/+(V|0))+.5);if((w|0)>255){AN(0,16,1713440,(Y=i,i=i+8|0,c[Y>>2]=w,Y)|0);i=Y;break}else{H=AW(c[y>>2]|0,0,N,(N|0)<0|0?-1:0,V,(V|0)<0|0?-1:0)|0;is(b,0);ir(b,1684237684);is(b,0);is(b,1);is(b,0);is(b,c[f+56>>2]|0);is(b,c[y>>2]|0);is(b,H);ih(b,w);iz(b,0);w=il(b,0,0,1)|0;H=G;il(b,F,C,0)|0;y=NV(w,H,F,C)|0;is(b,y);il(b,w,H,0)|0;break}}}while(0);Y=il(b,0,0,1)|0;t=G;il(b,q,r,0)|0;u=NV(Y,t,q,r)|0;is(b,u);il(b,Y,t,0)|0;do{if((c[(c[s>>2]|0)+8>>2]|0)==1){if((c[f+96>>2]|0)!=0){T=10359;break}t=A1(8)|0;c[t>>2]=c[f+36>>2];c[t+4>>2]=1;_=t;$=1}else{T=10359}}while(0);if((T|0)==10359){t=f+4|0;Y=c[t>>2]|0;do{if((Y|0)==0){aa=-1;ab=0}else{u=A1(Y<<3)|0;r=c[t>>2]|0;if((r|0)<=0){aa=-1;ab=u;break}q=f+24|0;l=f+112|0;Z=f+88|0;m=-1;n=0;while(1){j=n+1|0;if((j|0)==(r|0)){U=NU(c[l>>2]|0,c[l+4>>2]|0,c[q>>2]|0,c[q+4>>2]|0)|0;ac=G;ad=U;ae=c[Z>>2]|0}else{U=c[Z>>2]|0;B=U+(j*40|0)+8|0;ac=c[B+4>>2]|0;ad=c[B>>2]|0;ae=U}U=ae+(n*40|0)+8|0;B=NV(ad,ac,c[U>>2]|0,c[U+4>>2]|0)|0;U=B;do{if((n|0)==0){T=10368}else{if((U|0)!=(c[u+(m<<3)+4>>2]|0)){T=10368;break}B=u+(m<<3)|0;c[B>>2]=(c[B>>2]|0)+1;af=m}}while(0);if((T|0)==10368){T=0;v=m+1|0;c[u+(v<<3)+4>>2]=U;c[u+(v<<3)>>2]=1;af=v}if((j|0)<(r|0)){m=af;n=j}else{aa=af;ab=u;break}}}}while(0);_=ab;$=aa+1|0}is(b,($<<3)+16|0);ir(b,1937011827);is(b,0);is(b,$);if(($|0)!=0){aa=0;do{is(b,c[_+(aa<<3)>>2]|0);is(b,c[_+(aa<<3)+4>>2]|0);aa=aa+1|0;}while(aa>>>0<$>>>0)}A4(_);_=c[s>>2]|0;if((c[_+8>>2]|0)==0){T=10375}else{if((c[_+52>>2]|0)==544240754){T=10375}}do{if((T|0)==10375){_=c[f+48>>2]|0;if((_|0)==0){break}if((_|0)>=(c[f+4>>2]|0)){break}kP(b,f,1)}}while(0);do{if((c[f>>2]|0)==2){if((c[f+52>>2]&2|0)==0){break}kP(b,f,2)}}while(0);do{if((c[(c[s>>2]|0)+8>>2]|0)==0){if((c[f+52>>2]&1|0)==0){break}T=f+4|0;_=A1((c[T>>2]<<3)+8|0)|0;$=_;c[_>>2]=1;aa=c[f+88>>2]|0;c[_+4>>2]=c[aa+32>>2];ab=c[T>>2]|0;if((ab|0)>1){T=0;af=1;while(1){ac=c[aa+(af*40|0)+32>>2]|0;if((ac|0)==(c[$+(T<<3)+4>>2]|0)){ad=$+(T<<3)|0;c[ad>>2]=(c[ad>>2]|0)+1;ag=T}else{ad=T+1|0;c[$+(ad<<3)+4>>2]=ac;c[$+(ad<<3)>>2]=1;ag=ad}ad=af+1|0;if((ad|0)<(ab|0)){T=ag;af=ad}else{ah=ag;break}}}else{ah=0}af=ah+1|0;is(b,(af<<3)+16|0);ir(b,1937011811);is(b,0);is(b,af);if((af|0)!=0){T=0;do{is(b,c[$+(T<<3)>>2]|0);is(b,c[$+(T<<3)+4>>2]|0);T=T+1|0;}while(T>>>0<af>>>0)}A4(_)}}while(0);ah=il(b,0,0,1)|0;ag=G;is(b,0);ir(b,1668510835);is(b,0);s=il(b,0,0,1)|0;af=G;T=f+44|0;is(b,c[T>>2]|0);$=f+4|0;ab=c[$>>2]|0;if((ab|0)>0){aa=f+88|0;ad=0;ac=-1;ae=0;t=c[aa>>2]|0;Y=ab;while(1){do{if((ac|0)==(c[t+(ad*40|0)+20>>2]|0)){ai=ae;aj=ac;ak=t;al=Y}else{ab=c[t+(ad*40|0)+24>>2]|0;if((ab|0)==0){ai=ae;aj=ac;ak=t;al=Y;break}is(b,ab);is(b,c[(c[aa>>2]|0)+(ad*40|0)+20>>2]|0);is(b,1);ab=c[aa>>2]|0;ai=ae+1|0;aj=c[ab+(ad*40|0)+20>>2]|0;ak=ab;al=c[$>>2]|0}}while(0);_=ad+1|0;if((_|0)<(al|0)){ad=_;ac=aj;ae=ai;t=ak;Y=al}else{am=ai;break}}}else{am=0}ai=il(b,0,0,1)|0;al=G;il(b,s,af,0)|0;is(b,am);il(b,ai,al,0)|0;al=il(b,0,0,1)|0;ai=G;il(b,ah,ag,0)|0;am=NV(al,ai,ah,ag)|0;is(b,am);il(b,al,ai,0)|0;ai=il(b,0,0,1)|0;al=G;is(b,0);ir(b,2054386803);is(b,0);am=c[$>>2]|0;if((am|0)>0){ag=c[f+88>>2]|0;ah=0;af=-1;s=0;Y=1;while(1){ak=c[ag+(s*40|0)+28>>2]|0;t=((c[ag+(s*40|0)+16>>2]|0)>>>0)/(ak>>>0)|0;an=(af|0)==-1|(t|0)==(af|0)?Y:0;ao=ak+ah|0;ak=s+1|0;if((ak|0)<(am|0)){ah=ao;af=t;s=ak;Y=an}else{break}}ap=ao;aq=(an|0)!=0}else{ap=0;aq=1}do{if(aq&(am|0)!=0){an=c[f+88>>2]|0;ao=((c[an+16>>2]|0)>>>0)/((c[an+28>>2]|0)>>>0)|0;is(b,(ao|0)<1?1:ao);is(b,ap)}else{is(b,0);is(b,ap);ao=c[$>>2]|0;if((ao|0)<=0){break}an=f+88|0;Y=0;s=c[an>>2]|0;af=ao;while(1){ao=c[s+(Y*40|0)+28>>2]|0;if((ao|0)==0){ar=s;as=af}else{ah=0;ag=s;ak=ao;do{is(b,((c[ag+(Y*40|0)+16>>2]|0)>>>0)/(ak>>>0)|0);ah=ah+1|0;ag=c[an>>2]|0;ak=c[ag+(Y*40|0)+28>>2]|0;}while(ah>>>0<ak>>>0);ar=ag;as=c[$>>2]|0}ak=Y+1|0;if((ak|0)<(as|0)){Y=ak;s=ar;af=as}else{break}}}}while(0);as=il(b,0,0,1)|0;ar=G;il(b,ai,al,0)|0;ap=NV(as,ar,ai,al)|0;is(b,ap);il(b,as,ar,0)|0;ar=c[$>>2]|0;L14037:do{if((ar|0)>0){as=c[f+88>>2]|0;ap=f+192|0;al=0;while(1){if((c[as+(al*40|0)+24>>2]|0)!=0){ai=as+(al*40|0)|0;am=NU(c[ap>>2]|0,c[ap+4>>2]|0,c[ai>>2]|0,c[ai+4>>2]|0)|0;ai=G;aq=0;if(ai>>>0>aq>>>0|ai>>>0==aq>>>0&am>>>0>-1>>>0){at=1;break L14037}}am=al+1|0;if((am|0)<(ar|0)){al=am}else{at=0;break}}}else{at=0}}while(0);ar=il(b,0,0,1)|0;al=G;is(b,0);if((at|0)==0){ir(b,1868788851)}else{ir(b,875982691)}is(b,0);is(b,c[T>>2]|0);if((c[$>>2]|0)<=0){au=0;av=0;aw=il(b,au,av,1)|0;ax=G;ay=il(b,ar,al,0)|0;az=G;aA=NV(aw,ax,ar,al)|0;aB=G;aC=aA;aD=aC;is(b,aD);aE=il(b,aw,ax,0)|0;aF=G;aG=0;aH=0;aI=il(b,aG,aH,1)|0;aJ=G;aK=il(b,o,p,0)|0;aL=G;aM=NV(aI,aJ,o,p)|0;aN=G;aO=aM;aP=aO;is(b,aP);aQ=il(b,aI,aJ,0)|0;aR=G;i=g;return}T=f+88|0;ap=(at|0)==1;at=f+192|0;f=0;do{as=c[T>>2]|0;do{if((c[as+(f*40|0)+24>>2]|0)!=0){am=as+(f*40|0)|0;aq=NU(c[at>>2]|0,c[at+4>>2]|0,c[am>>2]|0,c[am+4>>2]|0)|0;if(ap){ix(b,aq,G);break}else{is(b,aq);break}}}while(0);f=f+1|0;}while((f|0)<(c[$>>2]|0));au=0;av=0;aw=il(b,au,av,1)|0;ax=G;ay=il(b,ar,al,0)|0;az=G;aA=NV(aw,ax,ar,al)|0;aB=G;aC=aA;aD=aC;is(b,aD);aE=il(b,aw,ax,0)|0;aF=G;aG=0;aH=0;aI=il(b,aG,aH,1)|0;aJ=G;aK=il(b,o,p,0)|0;aL=G;aM=NV(aI,aJ,o,p)|0;aN=G;aO=aM;aP=aO;is(b,aP);aQ=il(b,aI,aJ,0)|0;aR=G;i=g;return}function kP(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;f=il(a,0,0,1)|0;g=G;is(a,0);h=(e|0)==1?1714328:1714048;ir(a,(d[h+1|0]|0)<<8|(d[h]|0)|(d[h+2|0]|0)<<16|(d[h+3|0]|0)<<24);is(a,0);h=il(a,0,0,1)|0;i=G;j=b+4|0;is(a,c[j>>2]|0);k=c[j>>2]|0;if((k|0)>0){l=b+88|0;b=0;m=0;n=k;while(1){k=m+1|0;if((c[(c[l>>2]|0)+(m*40|0)+36>>2]&e|0)==0){o=b;p=n}else{is(a,k);o=b+1|0;p=c[j>>2]|0}if((k|0)<(p|0)){b=o;m=k;n=p}else{q=o;break}}}else{q=0}o=il(a,0,0,1)|0;p=G;il(a,h,i,0)|0;is(a,q);il(a,o,p,0)|0;p=il(a,0,0,1)|0;o=G;il(a,f,g,0)|0;q=NV(p,o,f,g)|0;is(a,q);il(a,p,o,0)|0;return}function kQ(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;d=il(a,0,0,1)|0;e=G;f=b+80|0;g=c[f>>2]|0;h=(g|0)==0?0:g+5|0;is(a,0);ir(a,1935962981);is(a,0);g=h+27|0;ih(a,3);ih(a,g>>>21|128);ih(a,g>>>14|128);ih(a,g>>>7|128);ih(a,g&127);iy(a,c[b+68>>2]|0);ih(a,0);g=h+13|0;ih(a,4);ih(a,g>>>21|128);ih(a,g>>>14|128);ih(a,g>>>7|128);ih(a,g&127);g=b+76|0;h=c[g>>2]|0;i=c[h+48>>2]|0;do{if((i-86016|0)>>>0<2){if((c[h+412>>2]|0)<=24e3){j=10436;break}ih(a,107)}else{j=10436}}while(0);if((j|0)==10436){ih(a,mz(246904,i)|0)}if((c[(c[g>>2]|0)+8>>2]|0)==1){ih(a,21)}else{ih(a,17)}iz(a,c[(c[g>>2]|0)+524>>2]>>3);i=b+24|0;j=c[i>>2]|0;h=c[i+4>>2]|0;if((j|0)==0&(h|0)==0){k=0}else{i=c[b+4>>2]|0;if((i|0)>0){l=c[b+88>>2]|0;m=0;n=0;o=0;do{n=NU(c[l+(o*40|0)+16>>2]|0,0,n,m)|0;m=G;o=o+1|0;}while((o|0)<(i|0));p=m<<3|n>>>29;q=n<<3|0>>>29}else{p=0;q=0}n=N2(c[b+8>>2]|0,0,q,p)|0;p=N3(n,G,j,h)|0;k=p}p=c[g>>2]|0;g=c[p+72>>2]|0;h=c[p+540>>2]|0;p=(g|0)>(h|0)?g:h;is(a,p>>>0>k>>>0?p:k);is(a,k);k=c[f>>2]|0;if((k|0)==0){ih(a,6);ih(a,128);ih(a,128);ih(a,128);ih(a,1);ih(a,2);r=0;s=0;t=il(a,r,s,1)|0;u=G;v=il(a,d,e,0)|0;w=G;x=NV(t,u,d,e)|0;y=G;z=x;A=z;is(a,A);B=il(a,t,u,0)|0;C=G;return}ih(a,5);ih(a,k>>>21|128);ih(a,k>>>14|128);ih(a,k>>>7|128);ih(a,k&127);ij(a,c[b+84>>2]|0,c[f>>2]|0);ih(a,6);ih(a,128);ih(a,128);ih(a,128);ih(a,1);ih(a,2);r=0;s=0;t=il(a,r,s,1)|0;u=G;v=il(a,d,e,0)|0;w=G;x=NV(t,u,d,e)|0;y=G;z=x;A=z;is(a,A);B=il(a,t,u,0)|0;C=G;return}function kR(a,b){a=a|0;b=b|0;is(a,17);if((c[b>>2]|0)==2){ir(a,1919770995)}else{ir(a,1919770980)}ir(a,1347241542);ih(a,0);iy(a,33279);ih(a,0);ih(a,1);return}function kS(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;f=i;i=i+8|0;g=f|0;h=e+80|0;if((c[h>>2]|0)<7){i=f;return}is(b,11);ir(b,862151012);j=(c[e+84>>2]|0)+4|0;e=c[h>>2]<<3;h=(e-32|0)>>>0>2147483639|(j|0)==0;k=h?0:j;j=h?8:e-24|0;e=k;h=(d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24|0)>>>6;e=j>>>0>2?2:j;l=k+(e>>>3)|0;m=(cV(d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0)|0)<<(e&2)>>>27;l=e+6|0;e=j>>>0>l>>>0?l:j;l=k+(e>>>3)|0;n=(cV(d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0)|0)<<(e&7)>>>27;l=e+5|0;e=j>>>0>l>>>0?l:j;l=k+(e>>>3)|0;o=(cV(d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0)|0)<<(e&7)>>>29;l=e+3|0;e=j>>>0>l>>>0?l:j;l=k+(e>>>3)|0;p=(cV(d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0)|0)<<(e&7)>>>29;l=e+3|0;e=j>>>0>l>>>0?l:j;do{if((p|0)==2){l=e+2|0;q=j>>>0>l>>>0?l:j}else{if((p&1|0)!=0&(p|0)!=1){l=e+2|0;r=j>>>0>l>>>0?l:j}else{r=e}if((p&4|0)==0){q=r;break}l=r+2|0;q=j>>>0>l>>>0?l:j}}while(0);j=g|0;g=o|(n|h<<5&96)<<3;h=(((d[k+(q>>>3)|0]|0)<<(q&7)>>>7&1|(p|g<<3)<<1)<<5|m)<<13;m=g>>>2&255;g=8;p=j;while(1){a[p]=m;q=h;h=q<<8;k=g+8|0;if((k|0)<32){m=q>>>16&255;g=k;p=p+1|0}else{break}}h=0;ij(b,j,3);i=f;return}function kT(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;d=i;i=i+8|0;e=d|0;f=c[a+12>>2]|0;g=c[b+32>>2]|0;h=c[f+48>>2]|0;j=c[h+(g*264|0)+76>>2]|0;k=c[b+28>>2]|0;if((k|0)==0){l=0;i=d;return l|0}m=h+(g*264|0)+4|0;do{if((c[m>>2]|0)==0){n=0;o=0}else{if(g>>>0>=(c[a+24>>2]|0)>>>0){n=0;o=0;break}p=b+16|0;q=(c[h+(g*264|0)+88>>2]|0)+8|0;r=NV(c[p>>2]|0,c[p+4>>2]|0,c[q>>2]|0,c[q+4>>2]|0)|0;q=(c[(c[a+28>>2]|0)+(g<<2)>>2]|0)+40|0;c[e>>2]=1;c[e+4>>2]=1e6;p=AY(r,G,q,e)|0;n=G;o=p}}while(0);e=c[f+88>>2]|0;g=(e|0)<0|0?-1:0;do{if((e|0)==0|((n|0)<(g|0)|(n|0)==(g|0)&o>>>0<e>>>0)){h=c[f+96>>2]|0;if((h|0)!=0){p=f+40|0;q=NU(c[p>>2]|0,c[p+4>>2]|0,k,(k|0)<0|0?-1:0)|0;p=G;r=(h|0)<0|0?-1:0;if(!(p>>>0<r>>>0|p>>>0==r>>>0&q>>>0<h>>>0)){s=10479;break}}if((c[f+52>>2]&8|0)==0){break}if((c[j+8>>2]|0)!=0){break}if((c[m>>2]|0)==0){break}if((c[b+36>>2]&1|0)!=0){s=10479}}else{s=10479}}while(0);do{if((s|0)==10479){m=c[f+92>>2]|0;j=(m|0)<0|0?-1:0;if((n|0)<(j|0)|(n|0)==(j|0)&o>>>0<m>>>0){break}kF(a)}}while(0);l=kA(a,b)|0;i=d;return l|0}function kU(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;e=i;f=c[(c[a+12>>2]|0)+48>>2]|0;g=c[(c[a+28>>2]|0)+(d<<2)>>2]|0;h=f+(b*264|0)+72|0;c[h>>2]=544240754;c[f+(b*264|0)+124>>2]=d;j=ov(0)|0;k=f+(b*264|0)+76|0;c[k>>2]=j;do{if((j|0)==0){l=-12}else{c[j+8>>2]=2;c[(c[k>>2]|0)+52>>2]=c[h>>2];m=f+(b*264|0)+128|0;n=l5(m,a,g,0,1450,d)|0;if((n|0)<0){l=n;break}c[f+(b*264|0)+8>>2]=c[(c[c[(c[m>>2]|0)+28>>2]>>2]|0)+44>>2];c[f+(d*264|0)+120>>2]=b;o=0;i=e;return o|0}}while(0);AN(a,24,1701104,(a=i,i=i+8|0,c[a>>2]=d,a)|0);i=a;A6(k);c[f+(b*264|0)+8>>2]=9e4;o=l;i=e;return o|0}function kV(b,e,f,g,h,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0;k=i;i=i+96|0;l=k|0;m=k+8|0;n=k+16|0;o=c[(c[b+12>>2]|0)+48>>2]|0;p=c[o+(f*264|0)+128>>2]|0;c[l>>2]=0;c[m>>2]=0;if((p|0)==0){q=-2;i=k;return q|0}r=p+16|0;if((c[r>>2]|0)==0){q=-12;i=k;return q|0}s=o+(f*264|0)+168|0;do{if((h|0)==0){t=c[e+24>>2]|0;u=c[e+28>>2]|0;if((u|0)<15){break}v=o+(f*264|0)+176|0;w=c[v>>2]|0;if((w|0)==0){x=c[s>>2]|0;y=10506}else{z=c[o+(f*264|0)+172>>2]|0;A=c[s>>2]|0;if((z|0)<(A|0)){B=z;C=w}else{x=A;y=10506}}if((y|0)==10506){A=x+10|0;c[s>>2]=A;z=A2(w,A*20|0)|0;if((z|0)==0){break}A=z;c[v>>2]=A;B=c[o+(f*264|0)+172>>2]|0;C=A}A=o+(f*264|0)+172|0;c[C+(B*20|0)>>2]=t;c[(c[v>>2]|0)+((c[A>>2]|0)*20|0)+4>>2]=u;c[(c[v>>2]|0)+((c[A>>2]|0)*20|0)+8>>2]=g;c[(c[v>>2]|0)+((c[A>>2]|0)*20|0)+12>>2]=0;c[(c[v>>2]|0)+((c[A>>2]|0)*20|0)+16>>2]=0;c[A>>2]=(c[A>>2]|0)+1}else{if((j|0)<15){break}A=o+(f*264|0)+176|0;v=c[A>>2]|0;if((v|0)==0){D=c[s>>2]|0;y=10499}else{u=c[o+(f*264|0)+172>>2]|0;t=c[s>>2]|0;if((u|0)<(t|0)){E=u;F=v}else{D=t;y=10499}}if((y|0)==10499){t=D+10|0;c[s>>2]=t;u=A2(v,t*20|0)|0;if((u|0)==0){break}t=u;c[A>>2]=t;E=c[o+(f*264|0)+172>>2]|0;F=t}t=o+(f*264|0)+172|0;c[F+(E*20|0)>>2]=h;c[(c[A>>2]|0)+((c[t>>2]|0)*20|0)+4>>2]=j;c[(c[A>>2]|0)+((c[t>>2]|0)*20|0)+8>>2]=g;c[(c[A>>2]|0)+((c[t>>2]|0)*20|0)+12>>2]=0;c[(c[A>>2]|0)+((c[t>>2]|0)*20|0)+16>>2]=0;c[t>>2]=(c[t>>2]|0)+1}}while(0);lf(p,0,e,b)|0;p=i$(c[r>>2]|0,l)|0;g=i_(r,1450)|0;do{if((g|0)<0|(p|0)<1){H=g}else{r=iZ(m)|0;if((r|0)<0){H=r;break}nB(n);j=c[m>>2]|0;h=c[l>>2]|0;E=n+16|0;F=il(j,0,0,1)|0;s=G;iy(j,0);iy(j,0);L14173:do{if((p|0)>4){D=o+(f*264|0)+144|0;B=o+(f*264|0)+132|0;C=o+(f*264|0)+136|0;x=o+(f*264|0)+172|0;t=o+(f*264|0)+176|0;A=0;u=p;v=h;while(1){z=u;w=v;while(1){I=w;J=cV(d[I]|d[I+1|0]<<8|d[I+2|0]<<16|d[I+3|0]<<24|0)|0;I=z-4|0;if(J>>>0>I>>>0|J>>>0<13){K=A;break L14173}L=w+4|0;M=a[w+5|0]|0;if(!((M+64&255)<4|(M+56&255)<11)){break}M=I-J|0;if((M|0)>4){z=M;w=w+(J+4)|0}else{K=A;break L14173}}if(J>>>0>(c[D>>2]|0)>>>0){c[D>>2]=J}M=w+6|0;I=cT((d[M]|d[M+1|0]<<8)<<16>>16|0)|0;M=w+8|0;N=cV(d[M]|d[M+1|0]<<8|d[M+2|0]<<16|d[M+3|0]<<24|0)|0;M=c[B>>2]|0;if((M|0)==0){c[B>>2]=N;O=N}else{O=M}M=N-O|0;if((M|0)>0){P=NU(c[C>>2]|0,c[C+4>>2]|0,M,(M|0)<0|0?-1:0)|0;c[C>>2]=P;c[C+4>>2]=G;c[B>>2]=N;Q=0}else{Q=M}if((c[E>>2]|0)==0&(c[E+4>>2]|0)==(-2147483648|0)){M=c[C+4>>2]|0;c[E>>2]=c[C>>2];c[E+4>>2]=M}M=A+1|0;is(j,0);ij(j,L,2);iy(j,I&65535);I=(Q|0)!=0;iy(j,I?4:0);N=il(j,0,0,1)|0;P=G;iy(j,0);if(I){is(j,16);is(j,12);ij(j,1733032,4);is(j,Q)}I=z-16|0;R=J-12|0;L14197:do{if((R|0)>0){S=R;T=w+16|0;U=0;L14199:while(1){if((c[x>>2]|0)<=0){V=S;W=T;X=U;break}L14201:while(1){Y=c[t>>2]|0;Z=Y+12|0;_=c[Z>>2]|0;$=c[Y+4>>2]|0;do{if((_|0)==0){if(($|0)<=5){aa=0;break}c[Z>>2]=5;aa=5}else{aa=_}}while(0);_=c[Y>>2]|0;ab=0;do{ac=0;while(1){ad=ac+ab|0;if((ad|0)>=(S|0)){break}ae=ac+aa|0;if((ae|0)>=($|0)){break}if((a[_+ae|0]|0)==(a[T+ad|0]|0)){ac=ac+1|0}else{break}}if((ac|0)>=9){af=aa;ag=ab;ah=ac;while(1){ad=(ag|0)>0;if(!((af|0)>0&ad)){ai=ad;break}ad=af-1|0;ae=ag-1|0;if((a[_+ad|0]|0)==(a[T+ae|0]|0)){af=ad;ag=ae;ah=ah+1|0}else{ai=1;break}}if((ah|0)>=15){break L14201}}ab=ab+1|0;}while((ab|0)<(S|0));if((aa|0)<10&($|0)>20){c[Z>>2]=($|0)/2|0}else{ab=c[x>>2]|0;if((ab|0)<1){V=S;W=T;X=U;break L14199}_=c[t>>2]|0;if((c[_+16>>2]|0)==0){aj=ab;ak=_}else{A4(c[_>>2]|0);aj=c[x>>2]|0;ak=c[t>>2]|0}_=aj-1|0;c[x>>2]=_;NP(ak|0,ak+20|0,_*20|0|0)}if((c[x>>2]|0)<=0){V=S;W=T;X=U;break L14199}}_=c[Y+8>>2]|0;ab=af+ah|0;c[Z>>2]=ab+5;do{if((ab+15|0)>=($|0)){ac=c[x>>2]|0;if((ac|0)<1){break}ae=c[t>>2]|0;if((c[ae+16>>2]|0)==0){al=ac;am=ae}else{A4(c[ae>>2]|0);al=c[x>>2]|0;am=c[t>>2]|0}ae=al-1|0;c[x>>2]=ae;NP(am|0,am+20|0,ae*20|0|0)}}while(0);if(ai){ab=ag;ae=T;ac=U;while(1){ad=(ab|0)>14?14:ab;ih(j,1);ih(j,ad);ij(j,ae,ad);an=ae+ad|0;ao=ab-ad|0;if((ad|0)<14){ap=ad;do{ih(j,0);ap=ap+1|0;}while((ap|0)<14)}ap=ac+1|0;if((ao|0)>0){ab=ao;ae=an;ac=ap}else{aq=ap;break}}}else{aq=U}ih(j,2);ih(j,0);iy(j,ah);is(j,_);is(j,af);iy(j,1);iy(j,1);ac=aq+1|0;ae=S-ah-ag|0;if((ae|0)>0){S=ae;T=T+(ag+ah)|0;U=ac}else{ar=ac;break L14197}}while(1){U=(V|0)>14?14:V;ih(j,1);ih(j,U);ij(j,W,U);T=W+U|0;S=V-U|0;if((U|0)<14){ac=U;do{ih(j,0);ac=ac+1|0;}while((ac|0)<14)}ac=X+1|0;if((S|0)>0){V=S;W=T;X=ac}else{ar=ac;break}}}else{ar=0}}while(0);R=I+(12-J)|0;z=il(j,0,0,1)|0;ac=G;il(j,N,P,0)|0;iy(j,ar);il(j,z,ac,0)|0;if((R|0)>4){A=M;u=R;v=w+(J+4)|0}else{K=M;break}}}else{K=0}}while(0);h=il(j,0,0,1)|0;v=G;il(j,F,s,0)|0;iy(j,K);il(j,h,v,0)|0;A6(l);c[n+28>>2]=i$(c[m>>2]|0,l)|0;c[n+24>>2]=c[l>>2];v=c[E+4>>2]|0;h=n+8|0;c[h>>2]=c[E>>2];c[h+4>>2]=v;c[n+32>>2]=f;if((c[e+36>>2]&1|0)!=0){v=n+36|0;c[v>>2]=c[v>>2]|1}if((K|0)<=0){H=r;break}kA(b,n)|0;H=r}}while(0);A4(c[l>>2]|0);l=o+(f*264|0)+172|0;n=c[l>>2]|0;if((n|0)<=0){q=H;i=k;return q|0}b=o+(f*264|0)+176|0;f=0;o=n;L14261:while(1){n=f+1|0;K=o;while(1){as=c[b>>2]|0;at=as+(f*20|0)+16|0;if((c[at>>2]|0)!=0){au=K;break}av=as+(f*20|0)+4|0;aw=A1(c[av>>2]|0)|0;if((aw|0)!=0){y=10574;break}e=c[b>>2]|0;NP(e+(f*20|0)|0,e+(n*20|0)|0,(((c[l>>2]|0)-f|0)*20|0)-20|0);e=(c[l>>2]|0)-1|0;c[l>>2]=e;if((f|0)<(e|0)){K=e}else{q=H;y=10577;break L14261}}if((y|0)==10574){y=0;K=as+(f*20|0)|0;r=c[K>>2]|0;E=c[av>>2]|0;NN(aw|0,r|0,E)|0;c[K>>2]=aw;c[at>>2]=1;au=c[l>>2]|0}if((n|0)<(au|0)){f=n;o=au}else{q=H;y=10581;break}}if((y|0)==10581){i=k;return q|0}else if((y|0)==10577){i=k;return q|0}return 0}function kW(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0;b=i;i=i+8|0;d=b|0;e=c[a+128>>2]|0;A6(a+76|0);f=a+172|0;g=c[f>>2]|0;h=a+176|0;if((g|0)>0){j=0;k=g;while(1){g=c[h>>2]|0;if((c[g+(j*20|0)+16>>2]|0)==0){l=k}else{A4(c[g+(j*20|0)>>2]|0);l=c[f>>2]|0}g=j+1|0;if((g|0)<(l|0)){j=g;k=l}else{break}}}A6(h);c[f>>2]=0;c[a+168>>2]=0;if((e|0)==0){i=b;return}a=e+16|0;if((c[a>>2]|0)!=0){le(e)|0;f=c[a>>2]|0;i$(f,d)|0;A4(c[d>>2]|0)}mi(e);i=b;return}function kX(b){b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;e=i;i=i+952|0;f=e|0;g=e+8|0;h=c[b+4>>2]|0;j=b+8|0;b=h+((c[j>>2]|0)-4)|0;k=h;while(1){if(k>>>0>=b>>>0){l=0;break}if((a[k]|0)==0){k=k+1|0}else{m=0;n=0;o=k;p=10596;break}}do{if((p|0)==10596){while(1){p=0;L14295:do{if(o>>>0<b>>>0){h=0;q=o;while(1){r=q;s=ok(g,cV(d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0)|0,f,f,f,f)|0;if((s|0)<0){t=h;u=q;break L14295}r=q+s|0;s=h+1|0;if(r>>>0<b>>>0){h=s;q=r}else{t=s;u=r;break}}}else{t=0;u=o}}while(0);v=(n|0)>(t|0)?n:t;w=(o|0)==(k|0)?t:m;q=u+1|0;if(q>>>0<b>>>0){m=w;n=v;o=q;p=10596}else{break}}if((w|0)>3){x=51;i=e;return x|0}if((v|0)>200){x=50;i=e;return x|0}if((v|0)>3){x=25}else{l=v;break}i=e;return x|0}}while(0);do{if((jk(k,1625456)|0)!=0){v=(jl(k)|0)<<1;if((v|0)<(c[j>>2]|0)){break}else{x=12}i=e;return x|0}}while(0);x=(l|0)>0|0;i=e;return x|0}function kY(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0;b=i;i=i+56|0;d=b|0;e=b+40|0;f=b+48|0;g=a+12|0;h=c[g>>2]|0;j=mK(a,0)|0;if((j|0)==0){k=-12;i=b;return k|0}l=j+8|0;c[(c[l>>2]|0)+8>>2]=1;c[(c[l>>2]|0)+48>>2]=86017;c[j+236>>2]=1463898624;m=j+48|0;c[m>>2]=0;c[m+4>>2]=0;mL(j,64,1,14112e3);m=a+16|0;n=(c[m>>2]|0)+96|0;c[n>>2]=-1;c[n+4>>2]=-1;n=il(c[m>>2]|0,0,0,1)|0;o=G;if((zJ(c[a+1140>>2]|0,3119056,0,2)|0)==0){nk(a)}p=c[m>>2]|0;if((c[p+88>>2]|0)==0){q=p}else{r=ip(p)|0;p=h+8|0;c[p>>2]=r;c[p+4>>2]=G;q=c[m>>2]|0}p=c[g>>2]|0;r=iM(q)|0;do{if(!((r&-2097152|0)!=-2097152|(r&393216|0)==0|(r&61440|0)==61440|(r&3072|0)==3072)){if((oj(d,r)|0)==0){q=c[d>>2]|0;s=(q|0)<0|0?-1:0;t=q}else{s=0;t=0}if((c[d+8>>2]|0)!=3){break}q=c[d+36>>2]|0;h=(q|0)!=0?576:1152;u=c[m>>2]|0;v=126072+(((q|0)==1)<<4)+(((c[d+24>>2]|0)==1)<<3)|0;q=c[v>>2]|0;w=c[v+4>>2]|0;io(u,q,w)|0;w=iM(c[m>>2]|0)|0;q=(w|0)==1231971951;u=q&1;do{if((w|0)==1483304551|(w|0)==1231971951){v=iM(c[m>>2]|0)|0;if((v&1|0)==0){x=0}else{x=iM(c[m>>2]|0)|0}if((v&2|0)==0){y=0}else{y=iM(c[m>>2]|0)|0}L14338:do{if((v&4|0)!=0){z=y;A=0;c[e>>2]=h;c[e+4>>2]=c[d+12>>2];B=AY(x,0,e,j+40|0)|0;C=G;D=c[g>>2]|0;if((c[D+36>>2]|0)==0){E=0}else{F=0;E=((C|0)>(F|0)|(C|0)==(F|0)&B>>>0>0>>>0)&1}do{if((y|0)==0){F=ip(c[m>>2]|0)|0;H=G;if(!((F|0)==0&(H|0)==0)){I=H;J=F;K=10632;break}AN(a,24,1631896,(L=i,i=i+1|0,i=i+7&-8,c[L>>2]=0,L)|0);i=L;M=0}else{I=A;J=z;K=10632}}while(0);do{if((K|0)==10632){z=a+28|0;if((E|0)==0){M=0;break}else{N=0}do{A=iD(c[m>>2]|0)|0;F=c[c[z>>2]>>2]|0;H=AW(A&255,0,J,I,256,0)|0;A=G;O=AW(N,(N|0)<0|0?-1:0,B,C,100,0)|0;mp(F,H,A,O,G,0,0,1)|0;N=N+1|0;}while((N|0)<100);c[D+24>>2]=1;break L14338}}while(0);do{iD(c[m>>2]|0)|0;M=M+1|0;}while((M|0)<100)}}while(0);if((v&8|0)!=0){D=c[m>>2]|0;C=4;B=0;io(D,C,B)|0;}B=iM(c[m>>2]|0)|0;if(!((B|0)==1281455718|(B|0)==1279348037)){P=y;Q=x;break}B=c[m>>2]|0;C=17;D=0;io(B,C,D)|0;D=iL(c[m>>2]|0)|0;C=D>>>12;B=p+28|0;c[B>>2]=C;z=p+32|0;c[z>>2]=D&4095;c[j+448>>2]=C+529;C=c[z>>2]|0;AN(a,48,1644432,(L=i,i=i+16|0,c[L>>2]=c[B>>2],c[L+8>>2]=C,L)|0);i=L;P=y;Q=x}else{P=0;Q=0}}while(0);w=c[m>>2]|0;C=NU(n,o,36,0)|0;B=G;il(w,C,B,0)|0;do{if((iM(c[m>>2]|0)|0)==1447187017){if((iK(c[m>>2]|0)|0)!=1){R=P;S=Q;break}B=c[m>>2]|0;C=4;w=0;io(B,C,w)|0;w=iM(c[m>>2]|0)|0;R=w;S=iM(c[m>>2]|0)|0}else{R=P;S=Q}}while(0);w=(S|0)!=0;C=w^1;B=(R|0)==0;if(B&C){break}z=c[m>>2]|0;D=NU(t,s,n,o)|0;O=G;il(z,D,O,0)|0;if(w){c[f>>2]=h;c[f+4>>2]=c[d+12>>2];w=AY(S,0,f,j+40|0)|0;O=j+56|0;c[O>>2]=w;c[O+4>>2]=G}O=R;w=0;if(!(B|C|q)){C=c[d+12>>2]<<3;B=N2(S,0,h,0)|0;D=AW(O,w,C,(C|0)<0|0?-1:0,B,G)|0;c[(c[l>>2]|0)+72>>2]=D}c[p+40>>2]=u;D=p+16|0;c[D>>2]=O;c[D+4>>2]=w;k=0;i=b;return k|0}}while(0);il(c[m>>2]|0,n,o,0)|0;k=0;i=b;return k|0}function kZ(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;d=c[a+12>>2]|0;e=a+16|0;a=il(c[e>>2]|0,0,0,1)|0;f=G;g=d+8|0;d=c[g>>2]|0;h=c[g+4>>2]|0;g=0;if(((h|0)>(g|0)|(h|0)==(g|0)&d>>>0>128>>>0)&((f|0)<(h|0)|(f|0)==(h|0)&a>>>0<d>>>0)){g=NV(d,h,a,f)|0;f=G;a=0;i=(f|0)<(a|0)|(f|0)==(a|0)&g>>>0<1024>>>0?g:1024}else{i=1024}g=mb(c[e>>2]|0,b,i)|0;if((g|0)<1){j=(g|0)<0?g:-541478725;return j|0}i=b+36|0;c[i>>2]=c[i>>2]&-3;c[b+32>>2]=0;if((g|0)>127){i=g-128|0;k=(NO((c[b+24>>2]|0)+i|0,1677360,3)|0)==0?i:g}else{k=g}c[b+28>>2]=k;j=k;return j|0}function k_(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0;b=i;i=i+64|0;g=b|0;h=b+40|0;j=c[a+12>>2]|0;k=c[c[a+28>>2]>>2]|0;l=mt(k,d,e,f)|0;do{if((c[j+40>>2]|0)==0){m=10670}else{f=k+56|0;n=c[f+4>>2]|0;o=0;if(!((n|0)>(o|0)|(n|0)==(o|0)&(c[f>>2]|0)>>>0>0>>>0)){m=10670;break}o=j+16|0;n=c[o+4>>2]|0;p=a+1232|0;q=c[p+4>>2]|0;if(!((n|0)>(q|0)|(n|0)==(q|0)&(c[o>>2]|0)>>>0>(c[p>>2]|0)>>>0)){m=10670;break}q=a+16|0;n=ip(c[q>>2]|0)|0;r=G;s=c[p>>2]|0;t=c[p+4>>2]|0;p=(r|0)>(t|0)|(r|0)==(t|0)&n>>>0>s>>>0;u=c[o>>2]|0;v=c[o+4>>2]|0;o=NV(p?n:u,p?r:v,s,t)|0;r=G;p=c[f>>2]|0;n=c[f+4>>2]|0;f=NV(u,v,s,t)|0;v=AW(p,n,o,r,f,G)|0;f=G;n=0;if((e|0)<(n|0)|(e|0)==(n|0)&d>>>0<0>>>0){w=0;x=0}else{n=(f|0)<(e|0)|(f|0)==(e|0)&v>>>0<d>>>0;w=n?f:e;x=n?v:d}n=h+8|0;c[n>>2]=x;c[n+4>>2]=w;n=AW(x,w,o,r,v,f)|0;f=NU(s,t,n,G)|0;n=G;t=h|0;c[t>>2]=f;c[t+4>>2]=n;y=h;z=n;A=f;B=q}}while(0);do{if((m|0)==10670){if((c[j+24>>2]|0)!=0){if((l|0)<0){C=l;i=b;return C|0}else{h=(c[k+400>>2]|0)+(l*24|0)|0;w=h|0;y=h;z=c[w+4>>2]|0;A=c[w>>2]|0;B=a+16|0;break}}w=0;if((e|0)<(w|0)|(e|0)==(w|0)&d>>>0<1>>>0){D=(c[j+28>>2]|0)+529|0}else{D=0}c[k+448>>2]=D;C=-1;i=b;return C|0}}while(0);D=y|0;d=il(c[B>>2]|0,A,z,0)|0;z=G;A=0;if((z|0)<(A|0)|(z|0)==(A|0)&d>>>0<0>>>0){C=d;i=b;return C|0}d=g|0;A=0;z=0;L14410:while(1){if((z|0)>=4096){m=10680;break}e=z;l=(z|0)<0|0?-1:0;w=NU(c[D>>2]|0,c[D+4>>2]|0,e,l)|0;h=G;x=il(c[B>>2]|0,w,h,0)|0;q=G;f=0;do{if((q|0)<(f|0)|(q|0)==(f|0)&x>>>0<0>>>0){E=x;m=10686}else{n=iM(c[B>>2]|0)|0;if((n&-2097152|0)!=-2097152|(n&393216|0)==0|(n&61440|0)==61440|(n&3072|0)==3072){F=0;break}if((oj(g,n)|0)==1){F=0;break}E=c[d>>2]|0;m=10686}}while(0);do{if((m|0)==10686){m=0;if((E|0)<0){F=0;break}x=NU(E,(E|0)<0|0?-1:0,w,h)|0;f=G;q=il(c[B>>2]|0,x,f,0)|0;n=G;t=0;if((n|0)<(t|0)|(n|0)==(t|0)&q>>>0<0>>>0){H=q}else{q=iM(c[B>>2]|0)|0;if((q&-2097152|0)!=-2097152|(q&393216|0)==0|(q&61440|0)==61440|(q&3072|0)==3072){F=1;break}if((oj(g,q)|0)==1){F=1;break}H=c[d>>2]|0}if((H|0)<0){F=1;break}q=NU(H,(H|0)<0|0?-1:0,x,f)|0;f=il(c[B>>2]|0,q,G,0)|0;q=G;x=0;if((q|0)<(x|0)|(q|0)==(x|0)&f>>>0<0>>>0){I=f}else{f=iM(c[B>>2]|0)|0;if((f&-2097152|0)!=-2097152|(f&393216|0)==0|(f&61440|0)==61440|(f&3072|0)==3072){F=2;break}if((oj(g,f)|0)==1){F=2;break}I=c[d>>2]|0}if((I|0)<0){F=2}else{J=3;K=l;L=e;break L14410}}}while(0);A=F;z=z+1|0}if((m|0)==10680){J=A;K=(z|0)<0|0?-1:0;L=z}z=(J|0)==3;J=c[B>>2]|0;B=NU(c[D>>2]|0,c[D+4>>2]|0,z?L:0,z?K:0)|0;K=il(J,B,G,0)|0;B=G;J=0;if((B|0)<(J|0)|(B|0)==(J|0)&K>>>0<0>>>0){C=K;i=b;return C|0}K=y+8|0;mr(a,k,c[K>>2]|0,c[K+4>>2]|0);a=c[K+4>>2]|0;y=0;if((a|0)<(y|0)|(a|0)==(y|0)&(c[K>>2]|0)>>>0<1>>>0){M=(c[j+28>>2]|0)+529|0}else{M=0}c[k+448>>2]=M;C=0;i=b;return C|0}function k$(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;b=i;d=c[a+12>>2]|0;e=d+3280|0;c[e>>2]=-1;f=a+24|0;g=c[f>>2]|0;do{if((g|0)!=0){h=a+28|0;j=0;k=-1;l=g;while(1){m=c[(c[(c[h>>2]|0)+(j<<2)>>2]|0)+8>>2]|0;n=c[m+8>>2]|0;if((n|0)==0){o=k;p=l}else if((n|0)==1){if((k|0)>-1){q=10716;break}if((c[m+48>>2]|0)!=86017){q=10716;break}c[e>>2]=j;o=j;p=c[f>>2]|0}else{q=10718;break}m=j+1|0;if(m>>>0<p>>>0){j=m;k=o;l=p}else{q=10720;break}}if((q|0)==10716){AN(a,16,1699856,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;s=-22;i=b;return s|0}else if((q|0)==10718){AN(a,16,1697232,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;s=-22;i=b;return s|0}else if((q|0)==10720){if((o|0)<0){break}l=d+3284|0;c[l>>2]=p-1;k=d+8|0;j=a+16|0;nl(k,c[j>>2]|0,c[d+32>>2]|0,1691496);h=nm(a,k)|0;if((h|0)<0){s=h;i=b;return s|0}if((c[l>>2]|0)!=0){s=0;i=b;return s|0}np(k,c[j>>2]|0);k4(a);s=0;i=b;return s|0}}}while(0);AN(a,16,1695032,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;s=-22;i=b;return s|0}function k0(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;d=i;i=i+8|0;e=d|0;f=c[a+12>>2]|0;g=b+32|0;h=c[g>>2]|0;do{if((h|0)==(c[f+3280>>2]|0)){if((c[f+3284>>2]|0)==0){j=k5(a,b)|0;i=d;return j|0}k=A7(88)|0;l=k;c[e>>2]=l;if((k|0)==0){j=-12;i=d;return j|0}m=b;NN(k|0,m|0,80)|0;m=zi(c[b>>2]|0)|0;c[k>>2]=m;if((m|0)==0){A6(e);j=-12;i=d;return j|0}m=f+3292|0;k=c[m>>2]|0;if((k|0)==0){c[f+3288>>2]=l}else{c[k+80>>2]=l}c[m>>2]=c[e>>2]}else{m=a+28|0;l=(c[(c[m>>2]|0)+(h<<2)>>2]|0)+64|0;if((c[l>>2]|0)==1&(c[l+4>>2]|0)==0){AN(a,24,1702648,(l=i,i=i+8|0,c[l>>2]=h,l)|0);i=l}l=f+3284|0;if((c[l>>2]|0)==0){j=0;i=d;return j|0}k=(c[(c[m>>2]|0)+(c[g>>2]<<2)>>2]|0)+64|0;m=c[k+4>>2]|0;n=0;if((m|0)>(n|0)|(m|0)==(n|0)&(c[k>>2]|0)>>>0>0>>>0){j=0;i=d;return j|0}k=no(a,f+8|0,b)|0;if((k|0)<0){j=k;i=d;return j|0}k=(c[l>>2]|0)-1|0;c[l>>2]=k;if((k|0)!=0){break}k=k3(a)|0;if((k|0)<0){j=k}else{break}i=d;return j|0}}while(0);j=0;i=d;return j|0}function k1(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;d=i;i=i+128|0;e=d|0;f=b+12|0;g=c[f>>2]|0;if((c[g+3284>>2]|0)!=0){AN(b,24,1677224,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;k3(b)|0}do{if((c[g+36>>2]|0)!=0){h=e|0;NM(h|0,0,128);a[h]=84;a[e+1|0]=65;a[e+2|0]=71;j=b+1140|0;k=zJ(c[j>>2]|0,1644344,0,0)|0;l=(k|0)!=0;if(l){m=e+3|0;n=c[k+4>>2]|0;y_(m,n,31)|0}n=zJ(c[j>>2]|0,1631880,0,0)|0;m=(n|0)!=0;if(m){k=e+33|0;o=c[n+4>>2]|0;y_(k,o,31)|0}o=zJ(c[j>>2]|0,1625432,0,0)|0;k=(o|0)!=0;if(k){n=e+63|0;p=c[o+4>>2]|0;y_(n,p,31)|0}p=zJ(c[j>>2]|0,1619096,0,0)|0;n=(p|0)!=0;if(n){o=e+93|0;q=c[p+4>>2]|0;y_(o,q,5)|0}q=zJ(c[j>>2]|0,1614936,0,0)|0;o=(q|0)!=0;if(o){p=e+97|0;r=c[q+4>>2]|0;y_(p,r,31)|0}r=(m&1)+(l&1)+(k&1)+(n&1)+(o&1)|0;o=zJ(c[j>>2]|0,1743992,0,0)|0;if((o|0)==0){s=r}else{a[e+125|0]=0;a[e+126|0]=(cc(c[o+4>>2]|0)|0)&255;s=r+1|0}r=e+127|0;a[r]=-1;o=zJ(c[j>>2]|0,1737472,0,0)|0;L14531:do{if((o|0)==0){t=s}else{j=o+4|0;n=0;while(1){if((n|0)>=148){t=s;break L14531}if((y4(c[j>>2]|0,c[249904+(n<<2)>>2]|0)|0)==0){break}else{n=n+1|0}}a[r]=n&255;t=s+1|0}}while(0);if((t|0)<=0){break}ij(c[b+16>>2]|0,h,128)}}while(0);t=g+40|0;if((c[t>>2]|0)==0&(c[t+4>>2]|0)==0){i=d;return 0}t=c[f>>2]|0;f=b+16|0;if((c[t+3276>>2]|0)==0){b=c[f>>2]|0;g=t+40|0;s=c[g>>2]|0;e=c[g+4>>2]|0;il(b,s,e,0)|0;ir(c[f>>2]|0,1868983881);u=g}else{u=t+40|0}g=c[f>>2]|0;e=NU(c[u>>2]|0,c[u+4>>2]|0,8,0)|0;il(g,e,G,0)|0;is(c[f>>2]|0,c[t+48>>2]|0);e=t+52|0;is(c[f>>2]|0,c[e>>2]|0);ih(c[f>>2]|0,0);g=t+64|0;u=t+72|0;t=1;do{s=u+((((ac(c[g>>2]|0,t)|0)>>>0)/100|0)<<3)|0;b=c[s>>2]|0;r=c[e>>2]|0;o=N3(b<<8|0>>>24,c[s+4>>2]<<8|b>>>24,r,(r|0)<0|0?-1:0)|0;r=o;ih(c[f>>2]|0,(r|0)>255?255:r);t=t+1|0;}while((t|0)<100);il(c[f>>2]|0,0,0,2)|0;i=d;return 0}function k2(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;b=249312;while(1){d=c[b+32>>2]|0;if((d|0)==0){e=-1;f=10796;break}if((d|0)==(a|0)){e=1128878145;f=10795;break}else{b=b+36|0}}if((f|0)==10796){return e|0}else if((f|0)==10795){return e|0}return 0}function k3(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;b=i;i=i+8|0;d=b|0;e=c[a+12>>2]|0;np(e+8|0,c[a+16>>2]|0);k4(a);f=e+3288|0;g=c[f>>2]|0;c[d>>2]=g;if((g|0)==0){h=0;j=e+3292|0;k=j;c[k>>2]=0;i=b;return h|0}l=d;m=0;n=1;o=g;while(1){if((n|0)==0){p=0;q=m;r=o}else{g=k5(a,o|0)|0;p=(g|0)<0?0:n;q=g;r=c[d>>2]|0}nJ(r|0);c[f>>2]=c[r+80>>2];A6(l);g=c[f>>2]|0;c[d>>2]=g;if((g|0)==0){h=q;break}else{m=q;n=p;o=g}}j=e+3292|0;k=j;c[k>>2]=0;i=b;return h|0}function k4(b){b=b|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;f=i;i=i+40|0;g=f|0;h=c[b+12>>2]|0;j=c[(c[(c[b+28>>2]|0)+(c[h+3280>>2]<<2)>>2]|0)+8>>2]|0;k=(c[j+88>>2]&8388608|0)!=0?1727816:1723592;l=b+16|0;if((c[(c[l>>2]|0)+88>>2]|0)==0){i=f;return}m=j+412|0;n=0;while(1){if(n>>>0>=3){o=0;break}p=c[m>>2]|0;q=e[337568+(n<<1)>>1]|0;if((p|0)==(q|0)){o=1572864;break}if((p|0)==(q>>>1|0)){o=1048576;break}if((p|0)==(q>>>2|0)){o=0;break}else{n=n+1|0}}if((n|0)==3){AN(b,24,1719904,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;i=f;return}m=c[j+416>>2]|0;if((m|0)==1){s=192}else if((m|0)==2){s=0}else{AN(b,24,1715992,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;i=f;return}b=n<<10|o|s|-1900544;s=g+20|0;o=j+72|0;n=2147483647;m=-1;q=1;while(1){oj(g,q<<12|b)|0;p=(c[s>>2]|0)-(c[o>>2]|0)|0;t=(p|0)>-1?p:-p|0;p=(t|0)<(n|0);u=p?q:m;v=q+1|0;if((v|0)<15){n=p?t:n;m=u;q=v}else{break}}if((u|0)<=-1){AN(0,0,1712480,(r=i,i=i+24|0,c[r>>2]=1708800,c[r+8>>2]=1706736,c[r+16>>2]=174,r)|0);i=r;cw()}if((u|0)==15){i=f;return}r=g+24|0;q=g+36|0;m=g|0;n=b;b=u;while(1){u=b<<12;w=u|n;oj(g,w)|0;x=d[((c[r>>2]|0)==1)+(8+(((c[q>>2]|0)==1)<<1))|0]|0;y=x+144|0;if((y|0)<=(c[m>>2]|0)){break}o=b+1|0;if((o|0)==15){z=10836;break}else{n=w^u;b=o}}if((z|0)==10836){i=f;return}is(c[l>>2]|0,w);ii(c[l>>2]|0,0,x);x=il(c[l>>2]|0,0,0,1)|0;w=h+40|0;c[w>>2]=x;c[w+4>>2]=G;ir(c[l>>2]|0,1735289176);is(c[l>>2]|0,7);c[h+52>>2]=c[m>>2];c[h+56>>2]=1;c[h+60>>2]=0;c[h+64>>2]=0;is(c[l>>2]|0,0);is(c[l>>2]|0,0);h=0;do{ih(c[l>>2]|0,((h*255|0|0)/100|0)&255);h=h+1|0;}while((h|0)<100);h=a[k]|0;if(h<<24>>24==0){A=0;B=c[l>>2]|0;z=10831}else{w=1;x=h;while(1){ih(c[l>>2]|0,x<<24>>24);if(w>>>0>=(NL(k|0)|0)>>>0){break}h=a[k+w|0]|0;w=w+1|0;x=h}x=c[l>>2]|0;if((w|0)<21){A=w;B=x;z=10831}else{C=x}}if((z|0)==10831){while(1){z=0;ih(B,0);x=A+1|0;w=c[l>>2]|0;if((x|0)<21){A=x;B=w;z=10831}else{C=w;break}}}z=(c[j+116>>2]|0)-529|0;iz(C,(z|0)>0?z<<12:0);ii(c[l>>2]|0,0,(c[m>>2]|0)-y|0);i=f;return}function k5(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;e=i;i=i+40|0;f=e|0;g=c[a+12>>2]|0;h=g;j=c[b+24>>2]|0;do{if((j|0)!=0){k=b+28|0;l=c[k>>2]|0;if((l|0)<=3){break}m=j;n=cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0;if((n&-2097152|0)!=-2097152|(n&393216|0)==0|(n&61440|0)==61440|(n&3072|0)==3072){AN(a,24,1732816,(m=i,i=i+16|0,c[m>>2]=l,c[m+8>>2]=n,m)|0);i=m;o=l$(a,b)|0;i=e;return o|0}oj(f,n)|0;n=g+3272|0;m=c[n>>2]|0;l=f+20|0;if((m|0)==0){p=c[l>>2]|0;c[n>>2]=p;q=p}else{q=m}m=c[l>>2]|0;if(!((m|0)!=0&(q|0)==(m|0))){c[g+3276>>2]=1}m=g+40|0;if((c[m>>2]|0)==0&(c[m+4>>2]|0)==0){break}m=g+48|0;c[m>>2]=(c[m>>2]|0)+1;m=g+60|0;l=(c[m>>2]|0)+1|0;c[m>>2]=l;p=g+52|0;n=(c[p>>2]|0)+(c[k>>2]|0)|0;c[p>>2]=n;p=g+56|0;if((c[p>>2]|0)!=(l|0)){break}l=g+64|0;k=h+72+(c[l>>2]<<3)|0;c[k>>2]=n;c[k+4>>2]=(n|0)<0|0?-1:0;n=(c[l>>2]|0)+1|0;c[l>>2]=n;if((n|0)==400){n=1;do{k=h+72+(n<<3)|0;r=c[k+4>>2]|0;s=h+72+(n>>1<<3)|0;c[s>>2]=c[k>>2];c[s+4>>2]=r;n=n+2|0;}while((n|0)<400);c[p>>2]=c[p>>2]<<1;c[l>>2]=200}c[m>>2]=0}}while(0);o=l$(a,b)|0;i=e;return o|0}function k6(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;f=i;g=lV()|0;c[a>>2]=0;L14640:do{if((g|0)==0){h=10873}else{do{if((b|0)==0){if((d|0)==0){j=jh(0,e,0)|0;if((j|0)!=0){k=j;break}AN(g,16,1732656,(l=i,i=i+8|0,c[l>>2]=e,l)|0);i=l;m=-22;break L14640}else{j=jh(d,0,0)|0;if((j|0)!=0){k=j;break}AN(g,16,1699464,(l=i,i=i+8|0,c[l>>2]=d,l)|0);i=l;m=-22;break L14640}}else{k=b}}while(0);j=g+8|0;c[j>>2]=k;n=c[k+44>>2]|0;do{if((n|0)>0){o=A7(n)|0;p=g+12|0;c[p>>2]=o;if((o|0)==0){h=10873;break L14640}q=c[(c[j>>2]|0)+36>>2]|0;if((q|0)==0){break}c[o>>2]=q;Br(c[p>>2]|0)}else{c[g+12>>2]=0}}while(0);if((e|0)!=0){j=g+32|0;y_(j,e,1024)|0}c[a>>2]=g;r=0;i=f;return r|0}}while(0);if((h|0)==10873){AN(g,16,1699808,(l=i,i=i+1|0,i=i+7&-8,c[l>>2]=0,l)|0);i=l;m=-12}mi(g);r=m;i=f;return r|0}function k7(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0.0,I=0.0,J=0.0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;d=i;i=i+72|0;e=d|0;f=d+8|0;g=d+40|0;c[e>>2]=0;h=a+8|0;j=c[h>>2]|0;k=(b|0)!=0;if(k){zM(e,c[b>>2]|0,0)}l=a;m=Bv(l,e)|0;L14668:do{if((m|0)<0){n=m;o=10934}else{p=a+12|0;q=c[p>>2]|0;do{if((q|0)!=0){r=c[(c[h>>2]|0)+36>>2]|0;if((r|0)==0){break}if((c[q>>2]|0)!=(r|0)){break}r=Bv(q,e)|0;if((r|0)<0){n=r;o=10934;break L14668}}}while(0);q=a+24|0;L14675:do{if((c[q>>2]|0)==0){if((c[j+28>>2]&4096|0)!=0){break}AN(l,16,1614920,(s=i,i=i+1|0,i=i+7&-8,c[s>>2]=0,s)|0);i=s;n=-22;o=10934;break L14668}else{r=a+28|0;t=j+32|0;u=j+28|0;v=0;L14677:while(1){w=c[(c[r>>2]|0)+(v<<2)>>2]|0;x=c[w+8>>2]|0;y=c[x+8>>2]|0;L14679:do{if((y|0)==1){if((c[x+412>>2]|0)<1){o=10891;break L14677}z=x+432|0;if((c[z>>2]|0)!=0){break}A=c[x+416>>2]|0;c[z>>2]=(ac(pc(c[x+48>>2]|0)|0,A)|0)>>3}else if((y|0)==0){if((c[x+104>>2]|0)<1){o=10896;break L14677}if((c[x+108>>2]|0)<1){o=10896;break L14677}if((c[x+120>>2]|0)<1){o=10899}else{if((c[x+124>>2]|0)<1){o=10899}}if((o|0)==10899){o=0;if((c[u>>2]&2048|0)==0){o=10900;break L14677}}B=c[w+80>>2]|0;C=c[w+84>>2]|0;D=c[x+224>>2]|0;E=c[x+228>>2]|0;A=N2(E,(E|0)<0|0?-1:0,B,(B|0)<0|0?-1:0)|0;z=G;F=N2(D,(D|0)<0|0?-1:0,C,(C|0)<0|0?-1:0)|0;do{if((A|0)==(F|0)&(z|0)==(G|0)){if(!((E|0)==0|(C|0)==0)){break L14679}if((B|0)==0|(D|0)==0){break}if((B>>31|0)==(D>>31|0)){break L14679}}}while(0);H=+(B|0)/+(C|0);I=H- +(D|0)/+(E|0);if(I<0.0){J=-0.0-I}else{J=I}if(!(J<=H*.004|(B|0)==0|(C|0)==0|(E|0)==0)){o=10908;break L14677}}}while(0);y=c[t>>2]|0;L14700:do{if((y|0)!=0){K=x+52|0;L=x+48|0;L14702:do{if((c[K>>2]|0)!=0){do{if((c[L>>2]|0)==14){if((mC(y,14)|0)!=0){if((mC(c[t>>2]|0,c[L>>2]|0)|0)!=544694642){break}}if((lg(a,w)|0)!=0){break}c[K>>2]=0;break L14702}}while(0);if((c[K>>2]|0)==0){break}if((lg(a,w)|0)==0){o=10918;break L14677}else{break L14700}}}while(0);c[K>>2]=mC(c[t>>2]|0,c[L>>2]|0)|0}}while(0);do{if((c[u>>2]&64|0)!=0){if((c[x+88>>2]&4194304|0)!=0){break}AN(l,24,1719808,(s=i,i=i+8|0,c[s>>2]=v,s)|0);i=s}}while(0);v=v+1|0;if(v>>>0>=(c[q>>2]|0)>>>0){break L14675}}if((o|0)==10908){AN(l,16,1727744,(s=i,i=i+32|0,c[s>>2]=B,c[s+8>>2]=C,c[s+16>>2]=D,c[s+24>>2]=E,s)|0);i=s;n=-22;o=10934;break L14668}else if((o|0)==10900){AN(l,16,1732792,(s=i,i=i+1|0,i=i+7&-8,c[s>>2]=0,s)|0);i=s;n=-22;o=10934;break L14668}else if((o|0)==10891){AN(l,16,1743968,(s=i,i=i+1|0,i=i+7&-8,c[s>>2]=0,s)|0);i=s;n=-22;o=10934;break L14668}else if((o|0)==10896){AN(l,16,1737448,(s=i,i=i+1|0,i=i+7&-8,c[s>>2]=0,s)|0);i=s;n=-22;o=10934;break L14668}else if((o|0)==10918){po(f,32,c[K>>2]|0)|0;po(g,32,mC(c[(c[h>>2]|0)+32>>2]|0,c[L>>2]|0)|0)|0;v=c[K>>2]|0;u=c[L>>2]|0;AN(l,16,1723528,(s=i,i=i+32|0,c[s>>2]=f,c[s+8>>2]=v,c[s+16>>2]=u,c[s+24>>2]=g,s)|0);i=s;n=-1094995529;o=10934;break L14668}}}while(0);do{if((c[p>>2]|0)==0){u=c[j+44>>2]|0;if((u|0)<=0){break}v=A7(u)|0;c[p>>2]=v;if((v|0)==0){n=-12;o=10934;break L14668}u=c[j+36>>2]|0;if((u|0)==0){break}c[v>>2]=u;Br(v);v=Bv(c[p>>2]|0,e)|0;if((v|0)<0){n=v;o=10934;break L14668}}}while(0);do{if((c[q>>2]|0)!=0){if((c[(c[(c[c[a+28>>2]>>2]|0)+8>>2]|0)+88>>2]&8388608|0)!=0){break}p=a+1140|0;zK(p,1715984,1712464,0)|0}}while(0);if(!k){M=0;break}zL(b);c[b>>2]=c[e>>2];M=0}}while(0);if((o|0)==10934){zL(e);M=n}if((M|0)!=0){N=M;i=d;return N|0}M=c[(c[h>>2]|0)+48>>2]|0;do{if((M|0)!=0){n=c1[M&255](a)|0;if((n|0)<=-1){N=n;i=d;return N|0}e=c[a+16>>2]|0;if((e|0)==0){break}b=c[e+76>>2]|0;e=(b|0)<0?b:n;if((e|0)<0){N=e}else{break}i=d;return N|0}}while(0);M=a+24|0;e=c[M>>2]|0;L14752:do{if((e|0)!=0){n=a+28|0;b=0;k=e;L14754:while(1){j=c[(c[n>>2]|0)+(b<<2)>>2]|0;s=c[j+8>>2]|0;g=c[s+8>>2]|0;if((g|0)==1){f=c[j+40>>2]|0;l=c[s+412>>2]|0;L=N2(l,(l|0)<0|0?-1:0,f,(f|0)<0|0?-1:0)|0;O=G;P=L;o=10945}else if((g|0)==0){g=c[j+40>>2]|0;L=c[s+108>>2]|0;s=N2(L,(L|0)<0|0?-1:0,g,(g|0)<0|0?-1:0)|0;O=G;P=s;o=10945}else{Q=k}do{if((o|0)==10945){o=0;if((P|0)==0&(O|0)==(-2147483648|0)){Q=k;break}s=0;if((O|0)<(s|0)|(O|0)==(s|0)&P>>>0<1>>>0){N=-1094995529;break L14754}s=P>>>1|O<<31;g=O>>1|((O|0)<0|0?-1:0)<<31;if((g|0)<(O|0)|(g|0)==(O|0)&s>>>0<P>>>0){R=0;S=0;T=g;U=s}else{L=N0(s,g,P,O)|0;f=G;l=N1(s,g,P,O)|0;R=f;S=L;T=G;U=l}l=j+16|0;c[l>>2]=S;c[l+4>>2]=R;l=j+24|0;c[l>>2]=U;c[l+4>>2]=T;l=j+32|0;c[l>>2]=P;c[l+4>>2]=O;Q=c[M>>2]|0}}while(0);j=b+1|0;if(j>>>0<Q>>>0){b=j;k=Q}else{break L14752}}i=d;return N|0}}while(0);Q=a+1192|0;if((c[Q>>2]|0)>=0){N=0;i=d;return N|0}if((c[(c[h>>2]|0)+28>>2]&262272|0)==0){c[Q>>2]=1;N=0;i=d;return N|0}else{c[Q>>2]=0;N=0;i=d;return N|0}return 0}function k8(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0;f=i;i=i+144|0;g=f|0;h=f+8|0;j=f+16|0;k=f+48|0;l=f+80|0;m=f+112|0;n=d+8|0;o=c[n>>2]|0;p=c[o+176>>2]|0;q=(c[o+156>>2]|0)>0|0;o=(p|0)>(q|0)?p:q;q=e+48|0;do{if((c[q>>2]|0)==0){mm(g,h,d,0,e);p=c[h>>2]|0;if((p|0)==0){break}r=c[g>>2]|0;if((r|0)==0){break}s=c[d+44>>2]|0;t=N2(s,(s|0)<0|0?-1:0,r,(r|0)<0|0?-1:0)|0;r=c[(c[n>>2]|0)+112>>2]|0;s=N2(t,G,r,(r|0)<0|0?-1:0)|0;r=G;t=c[d+40>>2]|0;u=N2(t,(t|0)<0|0?-1:0,p,(p|0)<0|0?-1:0)|0;p=AW(1,0,s,r,u,G)|0;c[q>>2]=p}}while(0);g=e+8|0;h=c[g>>2]|0;p=c[g+4>>2]|0;do{if((h|0)==0&(p|0)==(-2147483648|0)){u=e+16|0;r=c[u>>2]|0;s=c[u+4>>2]|0;if(!(((r|0)!=0|(s|0)!=(-2147483648|0))&(o|0)==0)){v=-2147483648;w=0;x=10971;break}c[g>>2]=r;c[g+4>>2]=s;y=s;z=r;x=10970}else{y=p;z=h;x=10970}}while(0);if((x|0)==10970){if((z|0)==0&(y|0)==0|(z|0)==0&(y|0)==(-2147483648|0)){v=y;w=z;x=10971}else{A=y;B=z;x=10976}}if((x|0)==10971){z=e+16|0;if((c[z>>2]|0)==0&(c[z+4>>2]|0)==(-2147483648|0)&(o|0)==0){if(!(a[264128]|0)){AN(b,24,1661152,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;a[264128]=1}y=d+16|0;h=c[y>>2]|0;p=c[y+4>>2]|0;c[g>>2]=h;c[g+4>>2]=p;c[z>>2]=h;c[z+4>>2]=p;D=p;E=h}else{D=v;E=w}if(!((E|0)==0&(D|0)==(-2147483648|0))){A=D;B=E;x=10976}}do{if((x|0)==10976){E=e+16|0;if(!((c[E>>2]|0)==0&(c[E+4>>2]|0)==(-2147483648|0)&(o|0)<17)){break}D=d+264|0;c[D>>2]=B;c[D+4>>2]=A;w=o+1|0;v=(o|0)>0;L14799:do{if(v){h=~o;p=1;do{z=d+264+(p<<3)|0;if(!((c[z>>2]|0)==0&(c[z+4>>2]|0)==(-2147483648|0))){break}y=c[g>>2]|0;r=c[g+4>>2]|0;s=ac(c[q>>2]|0,p+h|0)|0;u=NU(s,(s|0)<0|0?-1:0,y,r)|0;c[z>>2]=u;c[z+4>>2]=G;p=p+1|0;}while((p|0)<(w|0));if(!v){break}p=c[D>>2]|0;h=c[D+4>>2]|0;z=0;while(1){u=z+1|0;r=d+264+(u<<3)|0;y=c[r>>2]|0;s=c[r+4>>2]|0;if(!((h|0)>(s|0)|(h|0)==(s|0)&p>>>0>y>>>0)){break L14799}t=d+264+(z<<3)|0;c[r>>2]=p;c[r+4>>2]=h;c[t>>2]=y;c[t+4>>2]=s;if((u|0)<(o|0)){z=u}else{break}}}}while(0);v=c[D+4>>2]|0;c[E>>2]=c[D>>2];c[E+4>>2]=v}}while(0);o=d+208|0;q=c[o>>2]|0;A=c[o+4>>2]|0;do{if(!((q|0)==0&(A|0)==0|(q|0)==0&(A|0)==(-2147483648|0))){B=e+16|0;x=c[B>>2]|0;v=c[B+4>>2]|0;if(!((A|0)>=(v|0)&((A|0)>(v|0)|q>>>0>=x>>>0)&(c[(c[b+8>>2]|0)+28>>2]&131072|0)==0|((A|0)>(v|0)|(A|0)==(v|0)&q>>>0>x>>>0))){break}x=c[d>>2]|0;v=j|0;NM(v|0,0,32);a[v]=0;if((q|0)==0&(A|0)==(-2147483648|0)){bc(v|0,32,1625424,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0)|0;i=C}else{B=1619088;bc(v|0,32,B|0,(C=i,i=i+16|0,c[C>>2]=q,c[C+8>>2]=A,C)|0)|0;i=C}B=k|0;NM(B|0,0,32);a[B]=0;w=e+16|0;z=c[w>>2]|0;h=c[w+4>>2]|0;if((z|0)==0&(h|0)==(-2147483648|0)){bc(B|0,32,1625424,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0)|0;i=C}else{w=1619088;bc(B|0,32,w|0,(C=i,i=i+16|0,c[C>>2]=z,c[C+8>>2]=h,C)|0)|0;i=C}AN(b,16,1644248,(C=i,i=i+24|0,c[C>>2]=x,c[C+8>>2]=v,c[C+16>>2]=B,C)|0);i=C;F=-22;i=f;return F|0}}while(0);k=e+16|0;A=c[k>>2]|0;q=c[k+4>>2]|0;do{if(!((A|0)==0&(q|0)==(-2147483648|0))){j=c[g>>2]|0;B=c[g+4>>2]|0;if(!(((j|0)!=0|(B|0)!=(-2147483648|0))&((B|0)<(q|0)|(B|0)==(q|0)&j>>>0<A>>>0))){break}v=b;x=l|0;NM(x|0,0,32);a[x]=0;h=1619088;bc(x|0,32,h|0,(C=i,i=i+16|0,c[C>>2]=j,c[C+8>>2]=B,C)|0)|0;i=C;B=m|0;NM(B|0,0,32);a[B]=0;j=c[k>>2]|0;h=c[k+4>>2]|0;if((j|0)==0&(h|0)==(-2147483648|0)){bc(B|0,32,1625424,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0)|0;i=C}else{z=1619088;bc(B|0,32,z|0,(C=i,i=i+16|0,c[C>>2]=j,c[C+8>>2]=h,C)|0)|0;i=C}h=c[d>>2]|0;AN(v,16,1631840,(C=i,i=i+24|0,c[C>>2]=x,c[C+8>>2]=B,c[C+16>>2]=h,C)|0);i=C;F=-22;i=f;return F|0}}while(0);c[o>>2]=A;c[o+4>>2]=q;q=c[k>>2]|0;o=c[k+4>>2]|0;k=d+16|0;c[k>>2]=q;c[k+4>>2]=o;A=c[n>>2]|0;n=c[A+8>>2]|0;if((n|0)==0){C=c[d+44>>2]|0;m=c[A+104>>2]|0;l=N2(m,(m|0)<0|0?-1:0,C,(C|0)<0|0?-1:0)|0;C=d+24|0;m=NU(l,G,c[C>>2]|0,c[C+4>>2]|0)|0;l=G;b=d+32|0;g=c[b>>2]|0;h=c[b+4>>2]|0;b=0;do{if((l|0)<(b|0)|(l|0)==(b|0)&m>>>0<0>>>0){B=N0(m,l,g,h)|0;x=NU(B,G,q,o)|0;B=G;c[k>>2]=x;c[k+4>>2]=B;v=N1(m,l,g,h)|0;j=G;z=0;if(!((j|0)<(z|0)|(j|0)==(z|0)&v>>>0<0>>>0)){H=j;I=v;break}z=NU(v,j,g,h)|0;j=G;v=NU(x,B,-1,-1)|0;c[k>>2]=v;c[k+4>>2]=G;H=j;I=z}else{if((l|0)<(h|0)|(l|0)==(h|0)&m>>>0<g>>>0){H=l;I=m;break}z=N0(m,l,g,h)|0;j=NU(z,G,q,o)|0;c[k>>2]=j;c[k+4>>2]=G;j=N1(m,l,g,h)|0;H=G;I=j}}while(0);c[C>>2]=I;c[C+4>>2]=H;F=0;i=f;return F|0}else if((n|0)==1){n=e+28|0;e=ml(A,c[n>>2]|0,1)|0;if((e|0)<=-1){F=0;i=f;return F|0}A=d+24|0;H=c[A>>2]|0;C=c[A+4>>2]|0;A=d+32|0;I=c[A>>2]|0;h=c[A+4>>2]|0;do{if((c[n>>2]|0)==0){if(!((H|0)==(I>>>1|h<<31|0)&(C|0)==(h>>1|((h|0)<0|0?-1:0)<<31|0))){break}if((c[k>>2]|0)==0&(c[k+4>>2]|0)==0){F=0}else{break}i=f;return F|0}}while(0);n=c[d+44>>2]|0;A=N2(n,(n|0)<0|0?-1:0,e,(e|0)<0|0?-1:0)|0;e=d+24|0;d=NU(A,G,H,C)|0;C=G;H=0;do{if((C|0)<(H|0)|(C|0)==(H|0)&d>>>0<0>>>0){A=N0(d,C,I,h)|0;n=NU(c[k>>2]|0,c[k+4>>2]|0,A,G)|0;A=G;c[k>>2]=n;c[k+4>>2]=A;g=N1(d,C,I,h)|0;l=G;m=0;if(!((l|0)<(m|0)|(l|0)==(m|0)&g>>>0<0>>>0)){J=l;K=g;break}m=NU(g,l,I,h)|0;l=G;g=NU(n,A,-1,-1)|0;c[k>>2]=g;c[k+4>>2]=G;J=l;K=m}else{if((C|0)<(h|0)|(C|0)==(h|0)&d>>>0<I>>>0){J=C;K=d;break}m=N0(d,C,I,h)|0;l=NU(c[k>>2]|0,c[k+4>>2]|0,m,G)|0;c[k>>2]=l;c[k+4>>2]=G;l=N1(d,C,I,h)|0;J=G;K=l}}while(0);c[e>>2]=K;c[e+4>>2]=J;F=0;i=f;return F|0}else{F=0;i=f;return F|0}return 0}function k9(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;do{if((c[a+1192>>2]|0)>0){d=c[(c[a+28>>2]|0)+(c[b+32>>2]<<2)>>2]|0;e=d+456|0;f=c[e>>2]|0;g=c[e+4>>2]|0;h=b+16|0;i=c[h>>2]|0;j=c[h+4>>2]|0;k=a+1264|0;l=c[k>>2]|0;m=c[k+4>>2]|0;n=0;if(((i|0)!=0|(j|0)!=(-2147483648|0))&((j|0)<(n|0)|(j|0)==(n|0)&i>>>0<0>>>0)&((l|0)==0&(m|0)==0)){n=NV(0,0,i,j)|0;j=G;c[k>>2]=n;c[k+4>>2]=j;k=d+40|0;i=a+1272|0;o=c[k+4>>2]|0;c[i>>2]=c[k>>2];c[i+4>>2]=o;p=j;q=n}else{p=m;q=l}if(((q|0)!=0|(p|0)!=0)&((f|0)==0&(g|0)==0)){l=AX(q,p,a+1272|0,d+40|0,3)|0;d=G;c[e>>2]=l;c[e+4>>2]=d;r=d;s=l}else{r=g;s=f}f=c[h>>2]|0;g=c[h+4>>2]|0;if(!((f|0)==0&(g|0)==(-2147483648|0))){l=NU(f,g,s,r)|0;c[h>>2]=l;c[h+4>>2]=G}h=b+8|0;l=c[h>>2]|0;g=c[h+4>>2]|0;if((l|0)==0&(g|0)==(-2147483648|0)){break}f=NU(l,g,s,r)|0;c[h>>2]=f;c[h+4>>2]=G}}while(0);r=nN(b)|0;s=dg[c[(c[a+8>>2]|0)+52>>2]&255](a,b)|0;do{if((c[a+1216>>2]|0)!=0){p=c[a+16>>2]|0;if((p|0)==0){break}if((c[p+76>>2]|0)<=-1){break}ik(p)}}while(0);if((r|0)==0){return s|0}nM(b)|0;return s|0}function la(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0;e=i;i=i+8|0;f=e|0;g=b+32|0;h=a+28|0;j=c[(c[h>>2]|0)+(c[g>>2]<<2)>>2]|0;k=a+1184|0;if((c[k>>2]|0)==0){l=(c[a+1180>>2]|0)!=0}else{l=1}m=A7(88)|0;n=m;if((m|0)==0){o=-12;i=e;return o|0}p=m;q=b;NN(m|0,q|0,80)|0;c[b+52>>2]=0;c[b>>2]=0;nH(p)|0;nG(p,p)|0;if((c[(c[(c[h>>2]|0)+(c[g>>2]<<2)>>2]|0)+244>>2]|0)==0){r=a+1220|0}else{r=(c[j+244>>2]|0)+80|0}L14893:do{if(l){p=c[a+1180>>2]|0;c[f>>2]=1;c[f+4>>2]=1e6;q=AX(p,(p|0)<0|0?-1:0,f,j+40|0,3)|0;p=G;s=c[b+28>>2]|0;t=j+424|0;u=NU(c[t>>2]|0,c[t+4>>2]|0,s,(s|0)<0|0?-1:0)|0;s=G;c[t>>2]=u;c[t+4>>2]=s;v=c[b+48>>2]|0;w=j+432|0;x=NU(c[w>>2]|0,c[w+4>>2]|0,v,(v|0)<0|0?-1:0)|0;v=G;c[w>>2]=x;c[w+4>>2]=v;y=c[k>>2]|0;z=(y|0)<0|0?-1:0;if(!((y|0)!=0&((s|0)>(z|0)|(s|0)==(z|0)&u>>>0>y>>>0))){if(!(((q|0)!=0|(p|0)!=0)&(v>>>0>p>>>0|v>>>0==p>>>0&x>>>0>q>>>0))){break}}c[t>>2]=0;c[t+4>>2]=0;t=m+36|0;c[t>>2]=c[t>>2]|4096;do{if(!((q|0)==0&(p|0)==0)){t=c[w>>2]|0;x=c[w+4>>2]|0;if(!(x>>>0>p>>>0|x>>>0==p>>>0&t>>>0>q>>>0)){break}v=NV(0,0,(c[(c[j+8>>2]|0)+8>>2]|0)==0|0,0)|0;y=p&G;u=(q&v)>>>1|y<<31;v=y>>>1|0<<31;y=b+16|0;z=c[y>>2]|0;s=c[y+4>>2]|0;y=NU(u,v,z,s)|0;A=AW(y,G,1,0,q,p)|0;y=N2(A,G,q,p)|0;A=NV(z,s,y,G)|0;y=NU(A,G,u,v)|0;v=N0(y,G,8,0)|0;y=G;u=NV(t,x,q,p)|0;x=NU(u,G,v,y)|0;c[w>>2]=x;c[w+4>>2]=G;break L14893}}while(0);c[w>>2]=0;c[w+4>>2]=0}}while(0);j=c[r>>2]|0;L14903:do{if((j|0)==0){B=r;C=a+1224|0;D=11071}else{if(l){if((c[m+36>>2]&4096|0)==0){E=r;F=j;break}}k=a+1224|0;if((c6[d&255](a,c[k>>2]|0,b)|0)==0){B=(c[k>>2]|0)+80|0;C=k;D=11071;break}f=c[r>>2]|0;if((f|0)==0){B=r;C=k;D=11071;break}L14913:do{if(l){p=r;q=f;while(1){if((c[q+36>>2]&4096|0)==0){H=q}else{x=(c6[d&255](a,q|0,b)|0)==0;y=c[p>>2]|0;if(x){H=y}else{I=p;J=y;break L14913}}y=H+80|0;x=c[y>>2]|0;if((x|0)==0){B=y;C=k;D=11071;break L14903}else{p=y;q=x}}}else{q=r;p=f;while(1){x=(c6[d&255](a,p|0,b)|0)==0;y=c[q>>2]|0;if(!x){I=q;J=y;break L14913}x=y+80|0;y=c[x>>2]|0;if((y|0)==0){B=x;C=k;D=11071;break L14903}else{q=x;p=y}}}}while(0);if((J|0)==0){B=I;C=k;D=11071}else{E=I;F=J}}}while(0);if((D|0)==11071){c[C>>2]=n;E=B;F=c[B>>2]|0}c[m+80>>2]=F;c[E>>2]=n;c[(c[(c[h>>2]|0)+(c[g>>2]<<2)>>2]|0)+244>>2]=n;o=0;i=e;return o|0}function lb(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0;f=i;i=i+24|0;g=f|0;h=f+8|0;j=f+16|0;do{if((d|0)!=0){k=la(a,d,42)|0;if((k|0)<0){l=k}else{break}i=f;return l|0}}while(0);d=a+24|0;k=c[d>>2]|0;L14932:do{if((k|0)!=0){m=c[a+28>>2]|0;n=0;o=0;p=0;while(1){q=c[m+(o<<2)>>2]|0;if((c[q+244>>2]|0)==0){r=p;s=((c[(c[q+8>>2]|0)+8>>2]|0)==3)+n|0}else{r=p+1|0;s=n}q=o+1|0;if(q>>>0<k>>>0){n=s;o=q;p=r}else{break}}do{if((k|0)==(r|0)){t=1}else{if((e|0)!=0){t=e;break}if((k|0)==0){break L14932}p=a+28|0;o=h|0;n=h+4|0;m=a+1220|0;q=j|0;u=j+4|0;v=0;w=0;x=0;y=k;while(1){z=c[(c[p>>2]|0)+(x<<2)>>2]|0;A=c[z+244>>2]|0;if((A|0)==0){B=v;C=w;D=y}else{E=A+16|0;A=c[E>>2]|0;F=c[E+4>>2]|0;c[o>>2]=1;c[n>>2]=1e6;E=AY(A,F,z+40|0,h)|0;z=G;F=c[m>>2]|0;A=F+16|0;H=c[A>>2]|0;I=c[A+4>>2]|0;A=(c[(c[p>>2]|0)+(c[F+32>>2]<<2)>>2]|0)+40|0;c[q>>2]=1;c[u>>2]=1e6;F=AY(H,I,A,j)|0;A=NV(E,z,F,G)|0;F=G;z=(v|0)>(F|0)|(v|0)==(F|0)&w>>>0>A>>>0;B=z?v:F;C=z?w:A;D=c[d>>2]|0}A=x+1|0;if(A>>>0<D>>>0){v=B;w=C;x=A;y=D}else{break}}y=0;if(!((D|0)==(r+s|0)&((B|0)>(y|0)|(B|0)==(y|0)&C>>>0>2e7>>>0))){break L14932}AN(a,48,1677184,(y=i,i=i+8|0,c[y>>2]=s,y)|0);i=y;t=1}}while(0);if((r|0)==0|(t|0)==0){break}y=a+1220|0;x=c[y>>2]|0;c[g>>2]=x;w=b;v=x;NN(w|0,v|0,80)|0;v=c[(c[a+28>>2]|0)+(c[b+32>>2]<<2)>>2]|0;w=c[x+80>>2]|0;c[y>>2]=w;if((w|0)==0){c[a+1224>>2]=0}w=v+244|0;if((c[w>>2]|0)==(x|0)){c[w>>2]=0}A6(g);l=1;i=f;return l|0}}while(0);nB(b);l=0;i=f;return l|0}function lc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0;e=i;i=i+16|0;f=e|0;g=e+8|0;h=d+32|0;j=c[a+28>>2]|0;k=c[j+(c[h>>2]<<2)>>2]|0;l=b+32|0;m=c[j+(c[l>>2]<<2)>>2]|0;j=b+16|0;b=m+40|0;n=d+16|0;d=k+40|0;o=AZ(c[j>>2]|0,c[j+4>>2]|0,b,c[n>>2]|0,c[n+4>>2]|0,d)|0;p=a+1176|0;do{if((c[p>>2]|0)==0){q=o}else{a=k+8|0;r=m+8|0;if(!((c[(c[a>>2]|0)+8>>2]|0)==1^(c[(c[r>>2]|0)+8>>2]|0)==1)){q=o;break}s=c[n>>2]|0;t=c[n+4>>2]|0;c[f>>2]=1;c[f+4>>2]=1e6;u=AY(s,t,d,f)|0;t=c[p>>2]&-((c[(c[a>>2]|0)+8>>2]|0)==1|0);s=NV(u,G,t,(t|0)<0|0?-1:0)|0;t=G;u=c[j>>2]|0;v=c[j+4>>2]|0;c[g>>2]=1;c[g+4>>2]=1e6;w=AY(u,v,b,g)|0;v=c[p>>2]|0;u=(c[(c[r>>2]|0)+8>>2]|0)==1;r=v&-(u&1);x=NV(w,G,r,(r|0)<0|0?-1:0)|0;r=G;if((s|0)==(x|0)&(t|0)==(r|0)){w=c[d>>2]|0;y=N2(c[n>>2]|0,c[n+4>>2]|0,1e6,0)|0;z=N2(y,G,w,(w|0)<0|0?-1:0)|0;w=G;y=v;A=(v|0)<0|0?-1:0;v=NV(0,0,(c[(c[a>>2]|0)+8>>2]|0)==1|0,0)|0;a=c[k+44>>2]|0;B=a;C=(a|0)<0|0?-1:0;a=N2(y&v,A&G,B,C)|0;v=NV(z,w,a,G)|0;a=c[m+44>>2]|0;w=a;z=(a|0)<0|0?-1:0;a=N2(v,G,w,z)|0;v=G;D=c[b>>2]|0;E=N2(c[j>>2]|0,c[j+4>>2]|0,1e6,0)|0;F=N2(E,G,D,(D|0)<0|0?-1:0)|0;D=G;E=NV(0,0,u&1,0)|0;u=N2(w,z,y&E,A&G)|0;A=NV(F,D,u,G)|0;u=N2(A,G,B,C)|0;C=NV(a,v,u,G)|0;H=0;I=0;J=G;K=C}else{H=r;I=x;J=t;K=s}q=(((J|0)>(H|0)|(J|0)==(H|0)&K>>>0>I>>>0)&1)-(((J|0)<(H|0)|(J|0)==(H|0)&K>>>0<I>>>0)&1)|0}}while(0);if((q|0)==0){L=(c[h>>2]|0)<(c[l>>2]|0);M=L&1;i=e;return M|0}else{L=(q|0)>0;M=L&1;i=e;return M|0}return 0}function ld(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;d=i;i=i+80|0;e=d|0;do{if((b|0)==0){f=1;g=0;h=a+28|0}else{j=a+28|0;k=c[(c[j>>2]|0)+(c[b+32>>2]<<2)>>2]|0;do{if((c[(c[k+8>>2]|0)+8>>2]|0)==1){if((c[b+28>>2]|0)==0){l=0}else{break}i=d;return l|0}}while(0);m=k8(a,k,b)|0;do{if((m|0)<0){if((c[(c[a+8>>2]|0)+28>>2]&128|0)==0){l=m}else{break}i=d;return l|0}}while(0);m=b+16|0;if(!((c[m>>2]|0)==0&(c[m+4>>2]|0)==(-2147483648|0))){f=0;g=b;h=j;break}if((c[(c[a+8>>2]|0)+28>>2]&128|0)==0){l=-22}else{f=0;g=b;h=j;break}i=d;return l|0}}while(0);b=a+8|0;m=e+32|0;k=a+16|0;n=g;while(1){g=c[(c[b>>2]|0)+60>>2]|0;do{if((g|0)==0){o=lb(a,e,n,f)|0}else{p=dk[g&63](a,e,n,f)|0;if((n|0)==0){o=p;break}nJ(n);o=p}}while(0);if((o|0)<1){l=o;q=11138;break}r=k9(a,e)|0;if((r|0)<=-1){q=11130;break}g=(c[(c[h>>2]|0)+(c[m>>2]<<2)>>2]|0)+64|0;j=NU(c[g>>2]|0,c[g+4>>2]|0,1,0)|0;c[g>>2]=j;c[g+4>>2]=G;nJ(e);g=c[k>>2]|0;if((g|0)==0){n=0;continue}j=c[g+76>>2]|0;if((j|0)==0){n=0}else{l=j;q=11136;break}}if((q|0)==11138){i=d;return l|0}else if((q|0)==11130){nJ(e);l=r;i=d;return l|0}else if((q|0)==11136){i=d;return l|0}return 0}function le(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;b=i;i=i+80|0;d=b|0;e=a+8|0;f=d+32|0;g=a+28|0;h=a+16|0;while(1){j=c[(c[e>>2]|0)+60>>2]|0;if((j|0)==0){k=lb(a,d,0,1)|0}else{k=dk[j&63](a,d,0,1)|0}if((k|0)<0){l=k;break}if((k|0)==0){m=11150;break}n=k9(a,d)|0;if((n|0)<=-1){m=11147;break}j=(c[(c[g>>2]|0)+(c[f>>2]<<2)>>2]|0)+64|0;o=NU(c[j>>2]|0,c[j+4>>2]|0,1,0)|0;c[j>>2]=o;c[j+4>>2]=G;nJ(d);j=c[h>>2]|0;if((j|0)==0){continue}if((c[j+76>>2]|0)!=0){l=n;break}}do{if((m|0)==11147){nJ(d);l=n}else if((m|0)==11150){f=c[(c[e>>2]|0)+56>>2]|0;if((f|0)==0){l=0;break}l=c1[f&255](a)|0}}while(0);m=c[h>>2]|0;if((m|0)!=0){ik(m)}do{if((l|0)==0){m=c[h>>2]|0;if((m|0)==0){p=0;break}p=c[m+76>>2]|0}else{p=l}}while(0);l=a+24|0;if((c[l>>2]|0)!=0){h=0;do{A6((c[(c[g>>2]|0)+(h<<2)>>2]|0)+12|0);A6((c[(c[g>>2]|0)+(h<<2)>>2]|0)+400|0);h=h+1|0;}while(h>>>0<(c[l>>2]|0)>>>0)}l=a+12|0;if((c[(c[e>>2]|0)+36>>2]|0)==0){q=l;A6(q);i=b;return p|0}Bu(c[l>>2]|0);q=l;A6(q);i=b;return p|0}function lf(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;f=i;i=i+80|0;g=f|0;h=g;j=d;NN(h|0,j|0,80)|0;j=g+32|0;c[j>>2]=b;h=d+8|0;k=c[h>>2]|0;l=c[h+4>>2]|0;if(!((k|0)==0&(l|0)==(-2147483648|0))){h=AY(k,l,(c[(c[e+28>>2]|0)+(c[d+32>>2]<<2)>>2]|0)+40|0,(c[(c[a+28>>2]|0)+(b<<2)>>2]|0)+40|0)|0;l=g+8|0;c[l>>2]=h;c[l+4>>2]=G}l=d+16|0;h=c[l>>2]|0;k=c[l+4>>2]|0;if(!((h|0)==0&(k|0)==(-2147483648|0))){l=AY(h,k,(c[(c[e+28>>2]|0)+(c[d+32>>2]<<2)>>2]|0)+40|0,(c[(c[a+28>>2]|0)+(b<<2)>>2]|0)+40|0)|0;k=g+16|0;c[k>>2]=l;c[k+4>>2]=G}k=c[d+48>>2]|0;if((k|0)==0){m=a+28|0}else{l=a+28|0;h=AY(k,(k|0)<0|0?-1:0,(c[(c[e+28>>2]|0)+(c[d+32>>2]<<2)>>2]|0)+40|0,(c[(c[l>>2]|0)+(b<<2)>>2]|0)+40|0)|0;c[g+48>>2]=h;m=l}l=k8(a,c[(c[m>>2]|0)+(c[j>>2]<<2)>>2]|0,g)|0;do{if((l|0)<0){if((c[(c[a+8>>2]|0)+28>>2]&128|0)==0){n=l}else{break}i=f;return n|0}}while(0);l=k9(a,g)|0;if((l|0)<=-1){n=l;i=f;return n|0}g=c[a+16>>2]|0;do{if((g|0)==0){o=l}else{a=c[g+76>>2]|0;h=(a|0)<0?a:l;if((h|0)>-1){o=h;break}else{n=h}i=f;return n|0}}while(0);l=(c[(c[m>>2]|0)+(c[j>>2]<<2)>>2]|0)+64|0;j=NU(c[l>>2]|0,c[l+4>>2]|0,1,0)|0;c[l>>2]=j;c[l+4>>2]=G;n=o;i=f;return n|0}function lg(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;d=a+8|0;a=c[d>>2]|0;e=c[c[a+32>>2]>>2]|0;do{if((e|0)!=0){f=b+8|0;g=0;h=0;i=0;j=e;k=a;L15063:while(1){l=j|0;if((c[l>>2]|0)==0){m=h;n=i;o=k}else{p=h;q=i;r=l;while(1){l=r+4|0;s=pv(c[l>>2]|0)|0;t=(s|0)==(pv(c[(c[f>>2]|0)+52>>2]|0)|0);s=c[r>>2]|0;u=c[(c[f>>2]|0)+48>>2]|0;if(t){if((s|0)==(u|0)){v=1;w=11197;break L15063}else{x=s}}else{x=p}if((s|0)==(u|0)){y=c[l>>2]|0}else{y=q}l=r+8|0;if((c[l>>2]|0)==0){break}else{p=x;q=y;r=l}}m=x;n=y;o=c[d>>2]|0}r=g+1|0;q=c[(c[o+32>>2]|0)+(r<<2)>>2]|0;if((q|0)==0){break}else{g=r;h=m;i=n;j=q;k=o}}if((w|0)==11197){return v|0}if((m|0)!=0){v=0;return v|0}if((n|0)==0){break}if((c[(c[b+8>>2]|0)+680>>2]|0)>-1){v=0}else{break}return v|0}}while(0);v=1;return v|0}function lh(a){a=a|0;var b=0,e=0;b=c[a+4>>2]|0;do{if((NO(1708792,b|0,5)|0)==0){if((d[b+5|0]|0)<8){e=100}else{break}return e|0}}while(0);e=0;return e|0}function li(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0;b=i;i=i+8|0;d=b|0;e=a+12|0;f=c[e>>2]|0;c[f+12>>2]=-1;g=f+8|0;do{h=ln(a,0,0,0,0)|0;if((h|0)<0){j=11209;break}}while((c[g>>2]|0)==0);if((j|0)==11209){g=c[e>>2]|0;k=g+4|0;l=g;g=c[l>>2]|0;if((c[k>>2]|0)>0){m=0;n=g;while(1){A4(c[n+(m*384|0)>>2]|0);o=c[l>>2]|0;p=c[o+(m*384|0)+84>>2]|0;do{if((p|0)==0){q=o}else{r=c[p+32>>2]|0;if((r|0)==0){q=o;break}c$[r&127](a,m);q=c[l>>2]|0}}while(0);A4(c[q+(m*384|0)+380>>2]|0);o=m+1|0;p=c[l>>2]|0;if((o|0)<(c[k>>2]|0)){m=o;n=p}else{s=p;break}}}else{s=g}A4(s);t=h;i=b;return t|0}h=f+4|0;L15108:do{if((c[h>>2]|0)>0){s=f;g=a;n=a+28|0;m=a+1156|0;k=0;L15110:while(1){l=c[s>>2]|0;do{if((c[l+(k*384|0)+88>>2]|0)<0){AN(g,16,1715944,(u=i,i=i+8|0,c[u>>2]=k,u)|0);i=u;c[(c[s>>2]|0)+(k*384|0)+84>>2]=0}else{q=c[l+(k*384|0)+84>>2]|0;if((q|0)==0){break}p=c[l+(k*384|0)+376>>2]|0;o=c[q+28>>2]|0;if((p|0)>=(o|0)){break}AN(g,24,1712400,(u=i,i=i+24|0,c[u>>2]=k,c[u+8>>2]=o,c[u+16>>2]=p,u)|0);i=u;if((c[m>>2]&8|0)!=0){t=-1094995529;break L15110}}}while(0);p=l+(k*384|0)+40|0;o=c[p>>2]|0;q=c[p+4>>2]|0;if(!((o|0)==(-1|0)&(q|0)==(-1|0))){p=c[(c[c[e>>2]>>2]|0)+(k*384|0)+84>>2]|0;do{if((p|0)==0){v=q;w=o}else{r=c[p+20>>2]|0;if((r|0)==0){v=q;w=o;break}x=c4[r&63](a,k,o,q,0)|0;v=G;w=x}}while(0);q=(c[(c[n>>2]|0)+(k<<2)>>2]|0)+48|0;c[q>>2]=w;c[q+4>>2]=v;q=l+(k*384|0)+48|0;c[q>>2]=w;c[q+4>>2]=v}k=k+1|0;if((k|0)>=(c[h>>2]|0)){break L15108}}i=b;return t|0}}while(0);h=c[e>>2]|0;v=a+16|0;w=c[v>>2]|0;if((c[w+88>>2]|0)==0){t=0;i=b;return t|0}u=a+1064|0;if(!((c[u>>2]|0)==0&(c[u+4>>2]|0)==(-2147483648|0))){t=0;i=b;return t|0}u=ip(w)|0;w=G;f=0;if((w|0)<(f|0)|(w|0)==(f|0)&u>>>0<0>>>0){t=0;i=b;return t|0}f=0;k=(w|0)>(f|0)|(w|0)==(f|0)&u>>>0>65307>>>0;f=NU(u,w,-65307,-1)|0;w=k?G:0;lr(a);il(c[v>>2]|0,k?f:0,w,0)|0;w=h+16|0;c[w>>2]=-1;c[w+4>>2]=-1;L15136:do{if((lp(a,d)|0)==0){w=h;f=a+28|0;k=0;while(1){L15140:while(1){u=c[d>>2]|0;n=c[w>>2]|0;m=n+(u*384|0)+32|0;g=c[m>>2]|0;s=c[m+4>>2]|0;do{if(!((g|0)==(-1|0)&(s|0)==(-1|0)|(g|0)==0&(s|0)==0)){if((c[n+(u*384|0)+84>>2]|0)==0){break}m=c[(c[c[e>>2]>>2]|0)+(u*384|0)+84>>2]|0;do{if((m|0)==0){y=s;z=g;A=u}else{q=c[m+20>>2]|0;if((q|0)==0){y=s;z=g;A=u;break}o=c4[q&63](a,u,g,s,0)|0;y=G;z=o;A=c[d>>2]|0}}while(0);m=(c[(c[f>>2]|0)+(A<<2)>>2]|0)+56|0;c[m>>2]=z;c[m+4>>2]=y;B=c[(c[f>>2]|0)+(A<<2)>>2]|0;m=B+48|0;C=c[m>>2]|0;D=c[m+4>>2]|0;if(!((C|0)==0&(D|0)==(-2147483648|0))){j=11242;break L15140}E=(c[w>>2]|0)+(A*384|0)+368|0;if((c[E>>2]|0)==0){j=11245;break L15140}}}while(0);if((lp(a,d)|0)!=0){F=k;break L15136}}if((j|0)==11242){j=0;l=B+56|0;s=NV(c[l>>2]|0,c[l+4>>2]|0,C,D)|0;c[l>>2]=s;c[l+4>>2]=G;l=(c[w>>2]|0)+(A*384|0)+368|0;s=k-((c[l>>2]|0)==-1)|0;c[l>>2]=1;H=s}else if((j|0)==11245){j=0;c[E>>2]=-1;H=k+1|0}if((lp(a,d)|0)==0){k=H}else{F=H;break}}}else{F=0}}while(0);ls(c[e>>2]|0,c[v>>2]|0);lr(a);H=a+1232|0;il(c[v>>2]|0,c[H>>2]|0,c[H+4>>2]|0,0)|0;E=c[e>>2]|0;A=il(c[v>>2]|0,0,0,1)|0;D=G;C=E+4|0;if((c[C>>2]|0)>0){B=E;y=0;do{z=c[B>>2]|0;c[z+(y*384|0)+8>>2]=0;c[z+(y*384|0)+12>>2]=0;c[z+(y*384|0)+16>>2]=0;k=z+(y*384|0)+32|0;c[k>>2]=-1;c[k+4>>2]=-1;k=z+(y*384|0)+48|0;c[k>>2]=0;c[k+4>>2]=-2147483648;w=z+(y*384|0)+56|0;c[w>>2]=0;c[w+4>>2]=-2147483648;w=z+(y*384|0)+64|0;c[w>>2]=-1;c[w+4>>2]=-1;w=z+(y*384|0)+72|0;c[w>>2]=0;c[w+4>>2]=0;c[z+(y*384|0)+92>>2]=0;c[z+(y*384|0)+96>>2]=0;c[z+(y*384|0)+356>>2]=0;c[z+(y*384|0)+372>>2]=0;z=c[H+4>>2]|0;if(!((D|0)>(z|0)|(D|0)==(z|0)&A>>>0>(c[H>>2]|0)>>>0)){c[k>>2]=0;c[k+4>>2]=0}y=y+1|0;}while((y|0)<(c[C>>2]|0))}C=E+16|0;c[C>>2]=-1;c[C+4>>2]=-1;c[E+12>>2]=-1;L15164:do{if((F|0)>0){E=a+28|0;C=h;y=F;do{if((ln(a,d,0,0,0)|0)!=0){break L15164}L15168:while(1){H=c[d>>2]|0;L15170:do{if((H|0)>=0){I=lo(a,H,0)|0;J=G;K=c[d>>2]|0;L=c[(c[E>>2]|0)+(K<<2)>>2]|0;A=L+48|0;D=(c[A>>2]|0)==0&(c[A+4>>2]|0)==(-2147483648|0);do{if((I|0)==0&(J|0)==(-2147483648|0)){if(D){break L15170}}else{if(!D){break}if((c[(c[C>>2]|0)+(K*384|0)+368>>2]|0)==0){j=11261;break L15168}else{break L15170}}}while(0);M=(c[C>>2]|0)+(K*384|0)+368|0;if((c[M>>2]|0)==0){j=11262;break L15168}}}while(0);if((ln(a,d,0,0,0)|0)!=0){break L15164}}if((j|0)==11261){j=0;H=L+56|0;D=NV(c[H>>2]|0,c[H+4>>2]|0,I,J)|0;c[H>>2]=D;c[H+4>>2]=G;c[(c[C>>2]|0)+(K*384|0)+368>>2]=1}else if((j|0)==11262){j=0;c[M>>2]=1}y=y-1|0;}while((y|0)>0)}}while(0);ls(c[e>>2]|0,c[v>>2]|0);t=0;i=b;return t|0}function lj(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;e=i;i=i+40|0;f=e|0;g=e+8|0;h=e+16|0;j=e+24|0;k=e+32|0;l=a+1280|0;if((c[l>>2]|0)!=0){m=c[a+12>>2]|0;n=il(c[a+16>>2]|0,0,0,1)|0;o=G;p=m+4|0;if((c[p>>2]|0)>0){q=m;r=a+1232|0;s=0;do{t=c[q>>2]|0;c[t+(s*384|0)+8>>2]=0;c[t+(s*384|0)+12>>2]=0;c[t+(s*384|0)+16>>2]=0;u=t+(s*384|0)+32|0;c[u>>2]=-1;c[u+4>>2]=-1;u=t+(s*384|0)+48|0;c[u>>2]=0;c[u+4>>2]=-2147483648;v=t+(s*384|0)+56|0;c[v>>2]=0;c[v+4>>2]=-2147483648;v=t+(s*384|0)+64|0;c[v>>2]=-1;c[v+4>>2]=-1;v=t+(s*384|0)+72|0;c[v>>2]=0;c[v+4>>2]=0;c[t+(s*384|0)+92>>2]=0;c[t+(s*384|0)+96>>2]=0;c[t+(s*384|0)+356>>2]=0;c[t+(s*384|0)+372>>2]=0;t=c[r+4>>2]|0;if(!((o|0)>(t|0)|(o|0)==(t|0)&n>>>0>(c[r>>2]|0)>>>0)){c[u>>2]=0;c[u+4>>2]=0}s=s+1|0;}while((s|0)<(c[p>>2]|0))}p=m+16|0;c[p>>2]=-1;c[p+4>>2]=-1;c[m+12>>2]=-1;c[l>>2]=0}l=ln(a,f,g,h,j)|0;if((l|0)<0){w=l;i=e;return w|0}l=a+28|0;m=a+12|0;p=a;L15199:while(1){x=c[f>>2]|0;do{if((x|0)>=0){if((c[(c[l>>2]|0)+(x<<2)>>2]|0)==0){break}y=c[c[m>>2]>>2]|0;z=lo(a,x,k)|0;A=G;s=c[f>>2]|0;r=c[g>>2]|0;n=c[c[m>>2]>>2]|0;do{if((c[h>>2]|0)!=0){if((c[(c[(c[(c[l>>2]|0)+(s<<2)>>2]|0)+8>>2]|0)+48>>2]|0)!=31){break}o=n+(s*384|0)+20|0;q=c[o>>2]|0;if((q&1|0)==(((d[(c[n+(s*384|0)>>2]|0)+r|0]|0)>>>6&1^1)&255|0)){break}u=q^1;c[o>>2]=u;AN(p,24,1699592,(o=i,i=i+8|0,c[o>>2]=(u&1|0)!=0?3119072:1661048,o)|0);i=o}}while(0);B=y+(x*384|0)+364|0;if((c[B>>2]|0)==0){break L15199}if((c[y+(x*384|0)+20>>2]&1|0)!=0){break L15199}}}while(0);r=ln(a,f,g,h,j)|0;if((r|0)<0){w=r;C=11293;break}}if((C|0)==11293){i=e;return w|0}c[B>>2]=0;B=nC(b,c[h>>2]|0)|0;if((B|0)<0){w=B;i=e;return w|0}c[b+32>>2]=c[f>>2];f=c[b+24>>2]|0;B=(c[y+(x*384|0)>>2]|0)+(c[g>>2]|0)|0;g=c[h>>2]|0;NN(f|0,B|0,g)|0;g=b+8|0;c[g>>2]=z;c[g+4>>2]=A;A=c[k+4>>2]|0;g=b+16|0;c[g>>2]=c[k>>2];c[g+4>>2]=A;c[b+36>>2]=c[y+(x*384|0)+20>>2];c[b+48>>2]=c[y+(x*384|0)+24>>2];x=c[j+4>>2]|0;y=b+64|0;c[y>>2]=c[j>>2];c[y+4>>2]=x;w=c[h>>2]|0;i=e;return w|0}function lk(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;b=c[a+12>>2]|0;d=b+4|0;e=b;b=c[e>>2]|0;if((c[d>>2]|0)>0){f=0;g=b}else{h=b;i=h;A4(i);return 0}while(1){A4(c[g+(f*384|0)>>2]|0);b=c[e>>2]|0;j=c[b+(f*384|0)+84>>2]|0;do{if((j|0)==0){k=b}else{l=c[j+32>>2]|0;if((l|0)==0){k=b;break}c$[l&127](a,f);k=c[e>>2]|0}}while(0);A4(c[k+(f*384|0)+380>>2]|0);b=f+1|0;j=c[e>>2]|0;if((b|0)<(c[d>>2]|0)){f=b;g=j}else{h=j;break}}i=h;A4(i);return 0}function ll(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;g=i;h=c[a+12>>2]|0;j=h;k=c[j>>2]|0;l=h+4|0;if((c[l>>2]|0)<=(b|0)){AN(0,0,1727584,(m=i,i=i+24|0,c[m>>2]=1723480,c[m+8>>2]=1719776,c[m+16>>2]=826,m)|0);i=m;cw();return 0}m=il(c[a+16>>2]|0,0,0,1)|0;n=G;if((c[l>>2]|0)>0){o=a+1232|0;p=0;do{q=c[j>>2]|0;c[q+(p*384|0)+8>>2]=0;c[q+(p*384|0)+12>>2]=0;c[q+(p*384|0)+16>>2]=0;r=q+(p*384|0)+32|0;c[r>>2]=-1;c[r+4>>2]=-1;r=q+(p*384|0)+48|0;c[r>>2]=0;c[r+4>>2]=-2147483648;s=q+(p*384|0)+56|0;c[s>>2]=0;c[s+4>>2]=-2147483648;s=q+(p*384|0)+64|0;c[s>>2]=-1;c[s+4>>2]=-1;s=q+(p*384|0)+72|0;c[s>>2]=0;c[s+4>>2]=0;c[q+(p*384|0)+92>>2]=0;c[q+(p*384|0)+96>>2]=0;c[q+(p*384|0)+356>>2]=0;c[q+(p*384|0)+372>>2]=0;q=c[o+4>>2]|0;if(!((n|0)>(q|0)|(n|0)==(q|0)&m>>>0>(c[o>>2]|0)>>>0)){c[r>>2]=0;c[r+4>>2]=0}p=p+1|0;}while((p|0)<(c[l>>2]|0))}l=h+16|0;c[l>>2]=-1;c[l+4>>2]=-1;c[h+12>>2]=-1;do{if((c[(c[(c[(c[a+28>>2]|0)+(b<<2)>>2]|0)+8>>2]|0)+8>>2]|0)==0){if((f&4|0)!=0){break}c[k+(b*384|0)+364>>2]=1}}while(0);k=mu(a,b,d,e,f)|0;if((k|0)>=0){i=g;return k|0}c[(c[j>>2]|0)+(b*384|0)+364>>2]=0;i=g;return k|0}function lm(a,b,e,f,g){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0;h=i;i=i+24|0;j=h|0;k=h+8|0;l=h+16|0;m=a+12|0;n=c[m>>2]|0;o=a+16|0;p=c[o>>2]|0;il(p,c[e>>2]|0,c[e+4>>2]|0,0)|0;q=c[m>>2]|0;r=il(c[o>>2]|0,0,0,1)|0;s=G;t=q+4|0;if((c[t>>2]|0)>0){u=q;v=a+1232|0;w=0;do{x=c[u>>2]|0;c[x+(w*384|0)+8>>2]=0;c[x+(w*384|0)+12>>2]=0;c[x+(w*384|0)+16>>2]=0;y=x+(w*384|0)+32|0;c[y>>2]=-1;c[y+4>>2]=-1;y=x+(w*384|0)+48|0;c[y>>2]=0;c[y+4>>2]=-2147483648;z=x+(w*384|0)+56|0;c[z>>2]=0;c[z+4>>2]=-2147483648;z=x+(w*384|0)+64|0;c[z>>2]=-1;c[z+4>>2]=-1;z=x+(w*384|0)+72|0;c[z>>2]=0;c[z+4>>2]=0;c[x+(w*384|0)+92>>2]=0;c[x+(w*384|0)+96>>2]=0;c[x+(w*384|0)+356>>2]=0;c[x+(w*384|0)+372>>2]=0;x=c[v+4>>2]|0;if(!((s|0)>(x|0)|(s|0)==(x|0)&r>>>0>(c[v>>2]|0)>>>0)){c[y>>2]=0;c[y+4>>2]=0}w=w+1|0;}while((w|0)<(c[t>>2]|0))}t=q+16|0;c[t>>2]=-1;c[t+4>>2]=-1;c[q+12>>2]=-1;q=n;n=a+28|0;t=a;w=-1;v=-1;L15257:while(1){r=-1;L15259:do{if((w|0)>(r|0)|(w|0)==(r|0)&v>>>0>-1>>>0){s=il(p,0,0,1)|0;u=G;if((u|0)>(g|0)|(u|0)==(g|0)&s>>>0>f>>>0){A=-2147483648;B=0;break L15257}if((ln(a,j,k,l,e)|0)!=0){A=-2147483648;B=0;break L15257}if((c[j>>2]|0)!=(b|0)){w=w;v=v;continue L15257}s=c[q>>2]|0;u=lo(a,b,0)|0;y=G;x=c[j>>2]|0;z=c[k>>2]|0;C=c[c[m>>2]>>2]|0;do{if((c[l>>2]|0)!=0){if((c[(c[(c[(c[n>>2]|0)+(x<<2)>>2]|0)+8>>2]|0)+48>>2]|0)!=31){break}D=C+(x*384|0)+20|0;E=c[D>>2]|0;if((E&1|0)==(((d[(c[C+(x*384|0)>>2]|0)+z|0]|0)>>>6&1^1)&255|0)){break}F=E^1;c[D>>2]=F;AN(t,24,1699592,(H=i,i=i+8|0,c[H>>2]=(F&1|0)!=0?3119072:1661048,H)|0);i=H}}while(0);if((c[s+(b*384|0)+20>>2]&1|0)!=0){I=y;J=u;K=11344;break}if((c[s+(b*384|0)+364>>2]|0)==0){L=y;M=u;N=w;O=v;break}c[e>>2]=v;c[e+4>>2]=w;L=y;M=u;N=w;O=v}else{while(1){z=il(p,0,0,1)|0;x=G;if((x|0)>(g|0)|(x|0)==(g|0)&z>>>0>f>>>0){A=-2147483648;B=0;break L15257}if((ln(a,j,k,l,e)|0)!=0){A=-2147483648;B=0;break L15257}if((c[j>>2]|0)!=(b|0)){w=w;v=v;continue L15257}z=c[q>>2]|0;x=lo(a,b,0)|0;C=G;F=c[j>>2]|0;D=c[k>>2]|0;E=c[c[m>>2]>>2]|0;do{if((c[l>>2]|0)!=0){if((c[(c[(c[(c[n>>2]|0)+(F<<2)>>2]|0)+8>>2]|0)+48>>2]|0)!=31){break}P=E+(F*384|0)+20|0;Q=c[P>>2]|0;if((Q&1|0)==(((d[(c[E+(F*384|0)>>2]|0)+D|0]|0)>>>6&1^1)&255|0)){break}R=Q^1;c[P>>2]=R;AN(t,24,1699592,(H=i,i=i+8|0,c[H>>2]=(R&1|0)!=0?3119072:1661048,H)|0);i=H}}while(0);if((c[z+(b*384|0)+20>>2]&1|0)!=0){I=C;J=x;K=11344;break L15259}if((c[z+(b*384|0)+364>>2]|0)==0){L=C;M=x;N=w;O=v;break}}}}while(0);if((K|0)==11344){K=0;L=I;M=J;N=c[e+4>>2]|0;O=c[e>>2]|0}if((M|0)==0&(L|0)==(-2147483648|0)){w=N;v=O}else{A=L;B=M;break}}M=c[m>>2]|0;m=il(c[o>>2]|0,0,0,1)|0;o=G;L=M+4|0;if((c[L>>2]|0)<=0){S=M+16|0;T=S;U=-1;V=-1;W=T|0;c[W>>2]=U;X=T+4|0;c[X>>2]=V;Y=M+12|0;Z=Y;c[Z>>2]=-1;i=h;return(G=A,B)|0}O=M;v=a+1232|0;a=0;do{N=c[O>>2]|0;c[N+(a*384|0)+8>>2]=0;c[N+(a*384|0)+12>>2]=0;c[N+(a*384|0)+16>>2]=0;w=N+(a*384|0)+32|0;c[w>>2]=-1;c[w+4>>2]=-1;w=N+(a*384|0)+48|0;c[w>>2]=0;c[w+4>>2]=-2147483648;e=N+(a*384|0)+56|0;c[e>>2]=0;c[e+4>>2]=-2147483648;e=N+(a*384|0)+64|0;c[e>>2]=-1;c[e+4>>2]=-1;e=N+(a*384|0)+72|0;c[e>>2]=0;c[e+4>>2]=0;c[N+(a*384|0)+92>>2]=0;c[N+(a*384|0)+96>>2]=0;c[N+(a*384|0)+356>>2]=0;c[N+(a*384|0)+372>>2]=0;N=c[v+4>>2]|0;if(!((o|0)>(N|0)|(o|0)==(N|0)&m>>>0>(c[v>>2]|0)>>>0)){c[w>>2]=0;c[w+4>>2]=0}a=a+1|0;}while((a|0)<(c[L>>2]|0));S=M+16|0;T=S;U=-1;V=-1;W=T|0;c[W>>2]=U;X=T+4|0;c[X>>2]=V;Y=M+12|0;Z=Y;c[Z>>2]=-1;i=h;return(G=A,B)|0}function ln(b,e,f,g,h){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0;j=i;i=i+8|0;k=j|0;l=c[b+12>>2]|0;m=(e|0)!=0;if(m){c[e>>2]=-1}n=l+12|0;o=l;L15299:while(1){p=c[n>>2]|0;c[k>>2]=p;if((p|0)<0){while(1){q=lp(b,k)|0;if((q|0)<0){r=q;s=11409;break L15299}q=c[k>>2]|0;if((q|0)>=0){t=q;break}}}else{t=p}u=c[o>>2]|0;v=u+(t*384|0)+84|0;if((c[v>>2]|0)==0){w=u+(t*384|0)+88|0;if((c[w>>2]|0)>=0){r=0;s=11410;break}q=c[u+(t*384|0)>>2]|0;x=c[u+(t*384|0)+8>>2]|0;y=0;z=238112;while(1){A=d[z+4|0]|0;if((A|0)<=(x|0)){if((NO(q|0,c[z>>2]|0,A|0)|0)==0){break}}A=y+1|0;if((A|0)==14){s=11368;break L15299}else{y=A;z=c[120920+(A<<2)>>2]|0}}c[v>>2]=z;if((z|0)==0){break}}B=u+(t*384|0)+96|0;C=c[B>>2]|0;D=u+(t*384|0)+16|0;E=c[D>>2]|0;F=u+(t*384|0)+92|0;y=c[F>>2]|0;q=C;x=E;while(1){if((q|0)>=(y|0)){break}p=q+1|0;c[B>>2]=p;A=a[u+(t*384|0)+100+q|0]|0;G=x+(A&255)|0;c[D>>2]=G;if(A<<24>>24==-1){q=p;x=G}else{s=11376;break L15299}}if((q|0)!=(y|0)){continue}c[n>>2]=-1;c[u+(t*384|0)+356>>2]=(c[D>>2]|0)!=0}if((s|0)==11376){x=u+(t*384|0)+32|0;if((c[x>>2]|0)==(-1|0)&(c[x+4>>2]|0)==(-1|0)){x=u+(t*384|0)+72|0;z=c[x+4>>2]|0;AN(b,24,1631784,(H=i,i=i+16|0,c[H>>2]=c[x>>2],c[H+8>>2]=z,H)|0);i=H}c[n>>2]=c[k>>2];c[u+(t*384|0)+356>>2]=0;z=u+(t*384|0)+88|0;do{if((c[z>>2]|0)==0){c[u+(t*384|0)+20>>2]=0;c[u+(t*384|0)+24>>2]=0;x=c[v>>2]|0;do{if((x|0)!=0){G=c[x+16>>2]|0;if((G|0)==0){break}p=c[k>>2]|0;dg[G&255](b,p)|0}}while(0);if(m){c[e>>2]=c[k>>2]}if((f|0)!=0){c[f>>2]=c[u+(t*384|0)+12>>2]}if((g|0)!=0){c[g>>2]=c[D>>2]}if((h|0)!=0){x=u+(t*384|0)+64|0;y=c[x+4>>2]|0;c[h>>2]=c[x>>2];c[h+4>>2]=y}y=u+(t*384|0)+12|0;x=(c[y>>2]|0)+(c[D>>2]|0)|0;c[y>>2]=x;c[D>>2]=0;q=u+(t*384|0)+8|0;if((x|0)==(c[q>>2]|0)){c[y>>2]=0;c[q>>2]=0}q=u+(t*384|0)+72|0;y=c[q+4>>2]|0;x=u+(t*384|0)+64|0;c[x>>2]=c[q>>2];c[x+4>>2]=y}else{y=dg[c[(c[v>>2]|0)+12>>2]&255](b,c[k>>2]|0)|0;c[z>>2]=y;if((y|0)!=0){y=u+(t*384|0)+376|0;c[y>>2]=(c[y>>2]|0)+1;y=u+(t*384|0)+12|0;c[y>>2]=(c[y>>2]|0)+(c[D>>2]|0);c[D>>2]=0;break}c[B>>2]=C;c[D>>2]=E;c[l+8>>2]=1;y=b+1232|0;x=c[y>>2]|0;q=c[y+4>>2]|0;if((x|0)==0&(q|0)==0){p=u+(t*384|0)+64|0;G=c[p>>2]|0;A=c[p+4>>2]|0;c[y>>2]=G;c[y+4>>2]=A;I=A;J=G}else{I=q;J=x}x=l+4|0;q=c[x>>2]|0;if((q|0)>0){K=0;L=q;M=I;N=J}else{break}while(1){q=c[o>>2]|0;if((c[q+(K*384|0)+356>>2]|0)==0){O=L;P=M;Q=N}else{G=q+(K*384|0)+64|0;q=c[G>>2]|0;A=c[G+4>>2]|0;G=(M|0)>(A|0)|(M|0)==(A|0)&N>>>0>q>>>0;p=G?q:N;q=G?A:M;c[y>>2]=p;c[y+4>>2]=q;O=c[x>>2]|0;P=q;Q=p}p=K+1|0;if((p|0)<(O|0)){K=p;L=O;M=P;N=Q}else{break}}}}while(0);Q=u+(t*384|0)+360|0;c[Q>>2]=1;N=c[B>>2]|0;B=c[F>>2]|0;F=N;while(1){if((F|0)>=(B|0)){break}if((a[u+(t*384|0)+100+F|0]|0)==-1){F=F+1|0}else{s=11404;break}}if((s|0)==11404){c[Q>>2]=0}if((N|0)!=(B|0)){r=0;i=j;return r|0}c[n>>2]=-1;r=0;i=j;return r|0}else if((s|0)==11368){c[v>>2]=0}else if((s|0)==11410){i=j;return r|0}else if((s|0)==11409){i=j;return r|0}AN(b,24,1644168,(H=i,i=i+1|0,i=i+7&-8,c[H>>2]=0,H)|0);i=H;c[w>>2]=0;r=0;i=j;return r|0}function lo(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;e=a+12|0;f=c[c[e>>2]>>2]|0;g=(d|0)!=0;if(g){c[d>>2]=0;c[d+4>>2]=-2147483648}h=f+(b*384|0)+48|0;i=c[h>>2]|0;j=c[h+4>>2]|0;if((i|0)==0&(j|0)==(-2147483648|0)){k=-2147483648;l=0}else{c[h>>2]=0;c[h+4>>2]=-2147483648;k=j;l=i}i=f+(b*384|0)+56|0;j=c[i>>2]|0;m=c[i+4>>2]|0;if(!((j|0)==0&(m|0)==(-2147483648|0))){if(g){c[d>>2]=j;c[d+4>>2]=m}c[i>>2]=0;c[i+4>>2]=-2147483648}if((c[f+(b*384|0)+360>>2]|0)==0){n=k;o=l;return(G=n,o)|0}m=f+(b*384|0)+32|0;j=c[m>>2]|0;g=c[m+4>>2]|0;if((j|0)==(-1|0)&(g|0)==(-1|0)){n=k;o=l;return(G=n,o)|0}p=c[f+(b*384|0)+84>>2]|0;L15394:do{if((p|0)==0){q=11430}else{if((c[p+24>>2]|0)==0){q=11430;break}f=c[(c[c[e>>2]>>2]|0)+(b*384|0)+84>>2]|0;do{if((f|0)!=0){r=c[f+20>>2]|0;if((r|0)==0){break}s=c4[r&63](a,b,j,g,d)|0;t=G;u=s;break L15394}}while(0);if((d|0)==0){t=g;u=j;break}c[d>>2]=j;c[d+4>>2]=g;t=g;u=j}}while(0);if((q|0)==11430){d=c[(c[c[e>>2]>>2]|0)+(b*384|0)+84>>2]|0;do{if((d|0)==0){q=11433}else{e=c[d+20>>2]|0;if((e|0)==0){q=11433;break}p=c4[e&63](a,b,j,g,i)|0;v=G;w=p}}while(0);do{if((q|0)==11433){if((i|0)==0){v=g;w=j;break}c[i>>2]=j;c[i+4>>2]=g;v=g;w=j}}while(0);c[h>>2]=w;c[h+4>>2]=v;t=k;u=l}c[m>>2]=-1;c[m+4>>2]=-1;n=t;o=u;return(G=n,o)|0}function lp(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0;f=i;i=i+16|0;g=f|0;h=f+8|0;j=h;k=b+16|0;l=c[k>>2]|0;m=b+12|0;n=c[m>>2]|0;o=iE(l,h,4)|0;if((o|0)<4){p=(o|0)<0?o:-541478725;i=f;return p|0}o=l+88|0;q=n+16|0;r=0;L15418:while(1){s=j+(r&3)|0;do{if((a[s]|0)==79){if((a[j+(r+1&3)|0]|0)!=103){break}if((a[j+(r+2&3)|0]|0)!=103){break}if((a[j+(r+3&3)|0]|0)==83){t=r;break L15418}}}while(0);do{if((r|0)==0){if((c[o>>2]|0)==0){break}u=c[q>>2]|0;v=c[q+4>>2]|0;w=0;if(!((v|0)>(w|0)|(v|0)==(w|0)&u>>>0>0>>>0)){break}c[h>>2]=0;w=NU(u,v,4,0)|0;v=G;il(l,w,v,0)|0;c[q>>2]=-1;c[q+4>>2]=-1}}while(0);v=iD(l)|0;if((iq(l)|0)!=0){p=-541478725;x=11511;break}w=r+1|0;a[s]=v&255;if((r|0)<65307){r=w}else{t=w;break}}if((x|0)==11511){i=f;return p|0}if((t|0)>65306){AN(b,32,1625352,(y=i,i=i+1|0,i=i+7&-8,c[y>>2]=0,y)|0);i=y;p=-1094995529;i=f;return p|0}if((iD(l)|0)!=0){AN(b,16,1619048,(y=i,i=i+1|0,i=i+7&-8,c[y>>2]=0,y)|0);i=y;p=-1094995529;i=f;return p|0}t=iD(l)|0;r=iJ(l)|0;h=G;o=iI(l)|0;io(l,8,0)|0;j=iD(l)|0;w=n;v=c[n+4>>2]|0;n=0;while(1){if((n|0)>=(v|0)){z=0;x=11462;break}A=c[w>>2]|0;if((c[A+(n*384|0)+28>>2]|0)==(o|0)){x=11461;break}else{n=n+1|0}}if((x|0)==11461){if((n|0)<0){z=0;x=11462}else{B=n;C=A}}L15447:do{if((x|0)==11462){while(1){x=0;if((z|0)>=(v|0)){x=11479;break}if((c[(c[w>>2]|0)+(z*384|0)+372>>2]|0)==0){z=z+1|0;x=11462}else{x=11464;break}}L15451:do{if((x|0)==11464){s=g|0;A=c[m>>2]|0;n=c[k>>2]|0;L15453:do{if((c[n+88>>2]|0)==0){if((c[A+4>>2]|0)==1){D=0;E=c[A>>2]|0;break}else{AT(b,1732720,(y=i,i=i+1|0,i=i+7&-8,c[y>>2]=0,y)|0);i=y;F=-1163346256;break L15451}}else{u=il(n,0,0,1)|0;H=G;I=c[k>>2]|0;J=j;K=(j|0)<0|0?-1:0;io(I,J,K)|0;K=c[k>>2]|0;iE(K,s,8)|0;K=c[k>>2]|0;il(K,u,H,0)|0;H=0;u=238112;while(1){K=a[u+4|0]|0;if((K&255)<=8){if((NO(s|0,c[u>>2]|0,K&255|0)|0)==0){x=11469;break}}K=H+1|0;if((K|0)==14){break}else{H=K;u=c[120920+(K<<2)>>2]|0}}do{if((x|0)==11469){if((u|0)==0){break}H=A;K=c[A+4>>2]|0;J=0;while(1){if((J|0)>=(K|0)){break}I=c[H>>2]|0;if((c[I+(J*384|0)+84>>2]|0)==(u|0)){D=J;E=I;break L15453}else{J=J+1|0}}L=lq(b,o)|0;x=11480;break L15451}}while(0);AN(b,16,1737400,(y=i,i=i+1|0,i=i+7&-8,c[y>>2]=0,y)|0);i=y;F=-1094995529;break L15451}}while(0);c[E+(D*384|0)+28>>2]=o;L=D;x=11480}else if((x|0)==11479){L=lq(b,o)|0;x=11480}}while(0);do{if((x|0)==11480){if((L|0)<0){F=L;break}B=L;C=c[w>>2]|0;break L15447}}while(0);AN(b,16,1614872,(y=i,i=i+1|0,i=i+7&-8,c[y>>2]=0,y)|0);i=y;p=F;i=f;return p|0}}while(0);F=C+(B*384|0)|0;y=il(l,0,0,1)|0;b=NU(y,G,-27,-1)|0;y=G;L=C+(B*384|0)+72|0;c[L>>2]=b;c[L+4>>2]=y;c[q>>2]=b;c[q+4>>2]=y;y=C+(B*384|0)+16|0;if((c[y>>2]|0)!=0){q=c[w>>2]|0;w=A1((c[q+(B*384|0)+4>>2]|0)+16|0)|0;b=q+(B*384|0)+8|0;o=q+(B*384|0)+12|0;D=c[o>>2]|0;E=(c[b>>2]|0)-D|0;k=q+(B*384|0)|0;q=c[k>>2]|0;if((q|0)!=0){m=q+D|0;NN(w|0,m|0,E)|0;A4(q)}c[k>>2]=w;c[b>>2]=E;c[o>>2]=0}o=iE(l,C+(B*384|0)+100|0,j)|0;if((o|0)<(j|0)){p=(o|0)<0?o:-541478725;i=f;return p|0}c[C+(B*384|0)+92>>2]=j;o=C+(B*384|0)+96|0;c[o>>2]=0;if((j|0)>0){E=0;b=0;while(1){w=(d[C+(B*384|0)+100+b|0]|0)+E|0;k=b+1|0;if((k|0)<(j|0)){E=w;b=k}else{M=w;break}}}else{M=0}if((t&2|0)==0){c[C+(B*384|0)+372>>2]=1}do{if((t&1|0)==0){if((c[C+(B*384|0)+356>>2]|0)!=0){x=11495;break}c[y>>2]=0;b=c[L+4>>2]|0;E=C+(B*384|0)+64|0;c[E>>2]=c[L>>2];c[E+4>>2]=b}else{x=11495}}while(0);do{if((x|0)==11495){if((c[y>>2]|0)!=0){break}b=C+(B*384|0)+12|0;E=0;while(1){if((E|0)>=(j|0)){break}w=E+1|0;c[o>>2]=w;k=a[C+(B*384|0)+100+E|0]|0;c[b>>2]=(c[b>>2]|0)+(k&255);if(k<<24>>24==-1){E=w}else{break}}E=c[L+4>>2]|0;b=C+(B*384|0)+64|0;c[b>>2]=c[L>>2];c[b+4>>2]=E}}while(0);L=C+(B*384|0)+4|0;o=c[L>>2]|0;j=C+(B*384|0)+8|0;y=c[j>>2]|0;do{if((o-y|0)>>>0<M>>>0){x=o<<1;c[L>>2]=x;E=A1(x+16|0)|0;if((E|0)==0){p=-12;i=f;return p|0}else{x=F|0;b=c[x>>2]|0;w=c[j>>2]|0;NN(E|0,b|0,w)|0;A4(c[x>>2]|0);c[x>>2]=E;N=c[j>>2]|0;O=E;break}}else{N=y;O=c[F>>2]|0}}while(0);y=iE(l,O+N|0,M)|0;if((y|0)<(M|0)){p=(y|0)<0?y:-541478725;i=f;return p|0}y=(c[j>>2]|0)+M|0;c[j>>2]=y;j=C+(B*384|0)+32|0;c[j>>2]=r;c[j+4>>2]=h;c[C+(B*384|0)+80>>2]=t;NM((c[F>>2]|0)+y|0,0,16);if((e|0)==0){p=0;i=f;return p|0}c[e>>2]=B;p=0;i=f;return p|0}function lq(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0;d=i;e=c[a+12>>2]|0;f=e+4|0;g=c[f>>2]|0;if((c[e+24>>2]|0)!=0){AN(a,16,1743856,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;j=-558323010;i=d;return j|0}h=g+1|0;k=h*384|0;do{if(h>>>0>65535){if(((k>>>0)/(h>>>0)|0|0)==384){break}else{j=-12}i=d;return j|0}}while(0);h=e;e=A2(c[h>>2]|0,k)|0;k=e;if((e|0)==0){j=-12;i=d;return j|0}c[h>>2]=k;h=k+(g*384|0)|0;e=h;NM(e|0,0,384);c[k+(g*384|0)+28>>2]=b;c[k+(g*384|0)+4>>2]=65307;b=A1(65323)|0;c[h>>2]=b;c[k+(g*384|0)+88>>2]=-1;h=k+(g*384|0)+40|0;c[h>>2]=-1;c[h+4>>2]=-1;if((b|0)==0){j=-12;i=d;return j|0}b=mK(a,0)|0;if((b|0)==0){A6(e);j=-12;i=d;return j|0}else{c[b+4>>2]=g;mL(b,64,1,1e6);c[f>>2]=(c[f>>2]|0)+1;j=g;i=d;return j|0}return 0}function lr(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;b=c[a+12>>2]|0;d=b+4|0;e=A1((c[d>>2]|0)*384|0|24)|0;f=e;g=il(c[a+16>>2]|0,0,0,1)|0;a=e;c[a>>2]=g;c[a+4>>2]=G;c[e+8>>2]=c[b+12>>2];a=b+24|0;c[e+12>>2]=c[a>>2];c[e+16>>2]=c[d>>2];g=e+24|0;e=g;h=b;b=c[h>>2]|0;i=(c[d>>2]|0)*384|0;NN(g|0,b|0,i)|0;if((c[d>>2]|0)>0){j=0}else{c[a>>2]=f;return}do{i=c[h>>2]|0;b=A7((c[i+(j*384|0)+4>>2]|0)+16|0)|0;c[i+(j*384|0)>>2]=b;g=c[e+(j*384|0)>>2]|0;k=c[i+(j*384|0)+8>>2]|0;NN(b|0,g|0,k)|0;j=j+1|0;}while((j|0)<(c[d>>2]|0));c[a>>2]=f;return}function ls(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0;d=a+24|0;e=c[d>>2]|0;if((e|0)==0){return}c[d>>2]=c[e+12>>2];d=a;f=c[d>>2]|0;g=a+4|0;L15553:do{if((c[g>>2]|0)>0){h=0;i=f;while(1){A4(c[i+(h*384|0)>>2]|0);j=h+1|0;if((j|0)>=(c[g>>2]|0)){break L15553}h=j;i=c[d>>2]|0}}}while(0);i=e|0;il(b,c[i>>2]|0,c[i+4>>2]|0,0)|0;i=a+16|0;c[i>>2]=-1;c[i+4>>2]=-1;c[a+12>>2]=c[e+8>>2];a=e+16|0;i=c[a>>2]|0;c[g>>2]=i;b=A2(c[d>>2]|0,i*384|0)|0;c[d>>2]=b;if((b|0)==0){A4(f);c[g>>2]=0}else{g=e+24|0;f=(c[a>>2]|0)*384|0;NN(b|0,g|0,f)|0}A4(e);return}function lt(b){b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0;e=i;i=i+80|0;f=e|0;g=e+8|0;h=e+16|0;j=e+24|0;k=e+32|0;l=e+40|0;m=e+48|0;n=e+56|0;o=e+64|0;p=e+72|0;if((c[(c[b+12>>2]|0)+8>>2]|0)!=0){AN(b,24,1644128,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q}r=b+24|0;s=b+28|0;t=b+1140|0;u=b;v=0;L15567:while(1){w=c[(c[s>>2]|0)+(v<<2)>>2]|0;x=w+8|0;z=c[x>>2]|0;A=c[z+8>>2]|0;do{if((A|0)==1){if((c[z+48>>2]|0)==1330664787){mL(w,64,1,48e3);break}else{mL(w,64,1,c[z+412>>2]|0);break}}else if((A|0)==0){mL(w,64,c[z+104>>2]|0,c[z+108>>2]|0)}}while(0);z=c[x>>2]|0;A=c[z+48>>2]|0;if(!((A|0)==86021|(A|0)==31|(A|0)==86052|(A|0)==86028|(A|0)==1330664787)){B=11563;break}if((c[z+96>>2]|0)==0){B=11566;break}if((c[z+100>>2]|0)==0){B=11566;break}C=A7(65376)|0;c[C+72>>2]=v;L15579:do{if((c[(c[x>>2]|0)+88>>2]&8388608|0)==0){L15580:while(1){z=BW()|0;A=0;while(1){if((A|0)>=(v|0)){D=z;break L15579}if((z|0)==(c[(c[(c[(c[s>>2]|0)+(A<<2)>>2]|0)+12>>2]|0)+65360>>2]|0)){continue L15580}else{A=A+1|0}}}}else{D=v}}while(0);c[C+65360>>2]=D;E=w+12|0;c[E>>2]=C;A=c[x>>2]|0;z=c[A+48>>2]|0;do{if((z|0)==86052){F=c[A+88>>2]&8388608;if((c[A+100>>2]|0)<80){G=-1;B=11581;break L15567}H=A7(80)|0;if((H|0)==0){G=-12;B=11581;break L15567}c[C+4>>2]=H;c[C+16>>2]=80;I=c[A+96>>2]|0;NN(H|0,I|0,80)|0;I=H+68|0;y=0;a[I]=y&255;y=y>>8;a[I+1|0]=y&255;y=y>>8;a[I+2|0]=y&255;y=y>>8;a[I+3|0]=y&255;I=(F|0)!=0?1719768:1715928;jx(t,237744,0);F=m4(c[t>>2]|0,I,l)|0;H=A7(F)|0;c[k>>2]=H;if((H|0)==0){G=-12;B=11581;break L15567}J=C+20|0;c[k>>2]=H;K=c[l>>2]|0;m5(k,t,I,K)|0;c[J>>2]=F;c[C+8>>2]=H}else if((z|0)==1330664787){H=c[A+88>>2]&8388608;F=A+100|0;J=c[F>>2]|0;if((J|0)<19){L=-1;B=11586;break L15567}K=A7(J)|0;if((K|0)==0){L=-12;B=11586;break L15567}c[C+4>>2]=K;c[C+16>>2]=c[F>>2];J=c[A+96>>2]|0;I=c[F>>2]|0;NN(K|0,J|0,I)|0;I=(H|0)!=0?1719768:1715928;jx(t,237744,0);H=(m4(c[t>>2]|0,I,j)|0)+8|0;J=A7(H)|0;c[h>>2]=J;if((J|0)==0){L=-12;B=11586;break L15567}c[h>>2]=J+8;m5(h,t,I,c[j>>2]|0)|0;c[C+20>>2]=H;c[C+8>>2]=J;H=J;J=H|0;y=1937076303;a[J]=y&255;y=y>>8;a[J+1|0]=y&255;y=y>>8;a[J+2|0]=y&255;y=y>>8;a[J+3|0]=y&255;J=H+4|0;y=1936154964;a[J]=y&255;y=y>>8;a[J+1|0]=y&255;y=y>>8;a[J+2|0]=y&255;y=y>>8;a[J+3|0]=y&255}else if((z|0)==86028){J=c[A+88>>2]&8388608;if((nX(A,o,p)|0)==0){M=-1;B=11576;break L15567}c[C+16>>2]=51;H=A7(51)|0;c[C+4>>2]=H;if((H|0)==0){M=-12;B=11576;break L15567}a[H]=127;I=H+1|0;y=1128352838;a[I]=y&255;y=y>>8;a[I+1|0]=y&255;y=y>>8;a[I+2|0]=y&255;y=y>>8;a[I+3|0]=y&255;a[H+5|0]=1;a[H+6|0]=0;I=H+7|0;y=256;a[I]=y&255;y=y>>8;a[I+1|0]=y&255;I=H+9|0;y=1130450022;a[I]=y&255;y=y>>8;a[I+1|0]=y&255;y=y>>8;a[I+2|0]=y&255;y=y>>8;a[I+3|0]=y&255;a[H+13|0]=0;a[H+16|0]=34;a[H+15|0]=0;a[H+14|0]=0;I=H+17|0;H=c[p>>2]|0;NN(I|0,H|0,34)|0;H=(J|0)!=0?1719768:1715928;jx(t,237744,0);J=m4(c[t>>2]|0,H,n)|0;I=J+4|0;K=A7(I)|0;c[m>>2]=K;if((K|0)==0){M=-12;B=11576;break L15567}c[m>>2]=K+4;m5(m,t,H,c[n>>2]|0)|0;c[C+20>>2]=I;c[C+8>>2]=K;a[K]=-124;a[K+3|0]=J&255;a[K+2|0]=J>>>8&255;a[K+1|0]=J>>>16&255}else{J=(z|0)==86021;K=J?1737384:1732712;I=C+4|0;if((pI(c[A+96>>2]|0,c[A+100>>2]|0,J?30:42,I,C+16|0)|0)<0){B=11588;break L15567}H=C+20|0;F=(c[(c[x>>2]|0)+88>>2]&8388608|0)!=0?1719768:1715928;jx(t,237744,0);N=(m4(c[t>>2]|0,F,g)|0)+(J?8:7)|0;O=A7(N)|0;c[f>>2]=O;if((O|0)==0){P=0}else{c[f>>2]=O+7;Q=c[g>>2]|0;m5(f,t,F,Q)|0;if(J){a[c[f>>2]|0]=1;c[f>>2]=(c[f>>2]|0)+1}c[H>>2]=N;P=O}c[C+8>>2]=P;if((P|0)==0){R=-12;B=11612;break L15567}a[P]=J?3:-127;J=P+1|0;a[J]=a[K]|0;a[J+1|0]=a[K+1|0]|0;a[J+2|0]=a[K+2|0]|0;a[J+3|0]=a[K+3|0]|0;a[J+4|0]=a[K+4|0]|0;a[J+5|0]=a[K+5|0]|0;if((c[(c[x>>2]|0)+48>>2]|0)!=31){break}K=c[I>>2]|0;I=(d[K+40|0]|0)<<3&24|(d[K+41|0]|0)>>>5;c[C+28>>2]=I;J=d[K+9|0]|0;c[C+40>>2]=J;AN(u,48,1723448,(q=i,i=i+16|0,c[q>>2]=I,c[q+8>>2]=J,q)|0);i=q}}while(0);x=v+1|0;S=c[r>>2]|0;if(x>>>0<S>>>0){v=x}else{B=11555;break}}if((B|0)==11576){AN(u,16,1619016,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;A6(E);R=M;i=e;return R|0}else if((B|0)==11612){i=e;return R|0}else if((B|0)==11563){AN(u,16,1631744,(q=i,i=i+8|0,c[q>>2]=v,q)|0);i=q;R=-1;i=e;return R|0}else if((B|0)==11588){AN(u,16,1727560,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;A6(E);R=-1;i=e;return R|0}else if((B|0)==11566){AN(u,16,1625328,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;R=-1;i=e;return R|0}else if((B|0)==11555){do{if((S|0)!=0){v=b+28|0;M=0;do{P=c[(c[v>>2]|0)+(M<<2)>>2]|0;f=c[P+12>>2]|0;ly(b,c[(c[P+8>>2]|0)+48>>2]|0,f,c[f+4>>2]|0,c[f+16>>2]|0,0,0,1);P=f+76|0;a[P]=a[P]|2;lw(b,f);M=M+1|0;T=c[r>>2]|0;}while(M>>>0<T>>>0);if((T|0)==0){break}M=b+28|0;v=0;do{f=c[(c[M>>2]|0)+(v<<2)>>2]|0;P=f+12|0;t=c[P>>2]|0;g=f+8|0;f=c[t+20>>2]|0;if((f|0)!=0){ly(b,c[(c[g>>2]|0)+48>>2]|0,t,c[t+8>>2]|0,f,0,0,1)}f=c[t+24>>2]|0;if((f|0)!=0){ly(b,c[(c[g>>2]|0)+48>>2]|0,c[P>>2]|0,c[t+12>>2]|0,f,0,0,1)}lw(b,t);v=v+1|0;}while(v>>>0<(c[r>>2]|0)>>>0)}}while(0);r=C+56|0;c[r>>2]=0;c[r+4>>2]=-2147483648;R=0;i=e;return R|0}else if((B|0)==11581){AN(u,16,1614840,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;A6(E);R=G;i=e;return R|0}else if((B|0)==11586){AN(u,16,1743824,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;A6(E);R=L;i=e;return R|0}return 0}function lu(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0;d=i;i=i+8|0;e=d|0;f=c[(c[a+28>>2]|0)+(c[b+32>>2]<<2)>>2]|0;g=f+12|0;h=c[g>>2]|0;j=f+8|0;k=c[j>>2]|0;l=c[k+48>>2]|0;do{if((l|0)==31){m=b+8|0;n=c[m>>2]|0;o=c[m+4>>2]|0;if((c[h+40>>2]|0)<1){p=o;q=n}else{m=c[b+48>>2]|0;r=NU(m,(m|0)<0|0?-1:0,n,o)|0;p=G;q=r}r=h+32|0;if((c[b+36>>2]&1|0)==0){s=c[r+4>>2]|0;t=c[r>>2]|0}else{c[r>>2]=q;c[r+4>>2]=p;s=p;t=q}r=NV(q,p,t,s)|0;o=r;r=c[h+28>>2]|0;if((o|0)<(1<<r|0)){u=o;v=s;w=t}else{n=h+32|0;m=NU(o,(o|0)<0|0?-1:0,t,s)|0;o=G;c[n>>2]=m;c[n+4>>2]=o;u=0;v=o;w=m}m=NW(w|0,v|0,r|0)|0;x=G|((u|0)<0|0?-1:0);y=m|u}else{m=b+8|0;r=c[b+48>>2]|0;o=NU(r,(r|0)<0|0?-1:0,c[m>>2]|0,c[m+4>>2]|0)|0;m=G;if((l|0)!=1330664787){x=m;y=o;break}r=c[k+116>>2]|0;c[e>>2]=1;c[e+4>>2]=c[(c[j>>2]|0)+412>>2];n=AY(r,(r|0)<0|0?-1:0,e,f+40|0)|0;r=NU(n,G,o,m)|0;x=G;y=r}}while(0);f=h+56|0;if(!((c[f>>2]|0)==0&(c[f+4>>2]|0)==(-2147483648|0))){z=b+24|0;A=c[z>>2]|0;B=b+28|0;C=c[B>>2]|0;D=c[j>>2]|0;E=c[g>>2]|0;F=D+48|0;H=c[F>>2]|0;ly(a,H,E,A,C,y,x,0);lx(a,0);I=h+65368|0;J=I;K=J|0;c[K>>2]=y;L=J+4|0;c[L>>2]=x;i=d;return 0}e=b+8|0;k=c[e+4>>2]|0;c[f>>2]=c[e>>2];c[f+4>>2]=k;z=b+24|0;A=c[z>>2]|0;B=b+28|0;C=c[B>>2]|0;D=c[j>>2]|0;E=c[g>>2]|0;F=D+48|0;H=c[F>>2]|0;ly(a,H,E,A,C,y,x,0);lx(a,0);I=h+65368|0;J=I;K=J|0;c[K>>2]=y;L=J+4|0;c[L>>2]=x;i=d;return 0}function lv(a){a=a|0;var d=0,e=0,f=0,g=0,h=0,i=0;d=a+24|0;e=c[d>>2]|0;if((e|0)!=0){f=a+28|0;g=0;h=e;while(1){e=c[(c[(c[f>>2]|0)+(g<<2)>>2]|0)+12>>2]|0;if((b[e+65358>>1]|0)==0){i=h}else{lw(a,e);i=c[d>>2]|0}e=g+1|0;if(e>>>0<i>>>0){g=e;h=i}else{break}}}lx(a,1);if((c[d>>2]|0)==0){return 0}i=a+28|0;a=0;do{h=c[(c[i>>2]|0)+(a<<2)>>2]|0;g=h+12|0;f=c[g>>2]|0;e=c[(c[h+8>>2]|0)+48>>2]|0;if((e|0)==86028|(e|0)==86052|(e|0)==1330664787){A6(f+4|0)}A6(f+8|0);A6(g);a=a+1|0;}while(a>>>0<(c[d>>2]|0)>>>0);return 0}function lw(d,e){d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0;f=i;i=i+16|0;g=f|0;h=f+8|0;j=c[d+12>>2]|0;k=A7(65312)|0;l=k;if((k|0)==0){i=f;return}m=j+4|0;j=e+56|0;n=j;NN(k|0,n|0,65304)|0;n=e+64|0;o=c[n+4>>2]|0;p=j|0;c[p>>2]=c[n>>2];c[p+4>>2]=o;o=e+48|0;c[o>>2]=(c[o>>2]|0)+1;c[n>>2]=-1;c[n+4>>2]=-1;a[e+76|0]=0;a[e+77|0]=0;b[e+65358>>1]=0;e=c[m>>2]|0;L15683:do{if((e|0)==0){q=m;r=0}else{n=d+28|0;o=k+16|0;p=k+8|0;j=g|0;s=g+4|0;t=h|0;u=h+4|0;v=m;w=e;while(1){x=c[n>>2]|0;y=w+8|0;z=c[y>>2]|0;A=c[y+4>>2]|0;y=c[x+(c[w+16>>2]<<2)>>2]|0;B=c[x+(c[o>>2]<<2)>>2]|0;do{if((z|0)==(-1|0)&(A|0)==(-1|0)){C=w}else{if((c[p>>2]|0)==(-1|0)&(c[p+4>>2]|0)==(-1|0)){C=w;break}x=c[(c[y+12>>2]|0)+28>>2]|0;if((x|0)==0){D=A;E=z}else{F=NY(z|0,A|0,x|0)|0;H=(1<<x)-1|0;x=NU(H&z,((H|0)<0|0?-1:0)&A,F,G)|0;D=G;E=x}c[j>>2]=1;c[s>>2]=1e6;x=AY(E,D,y+40|0,g)|0;F=G;H=c[p>>2]|0;I=c[p+4>>2]|0;J=c[(c[B+12>>2]|0)+28>>2]|0;if((J|0)==0){K=I;L=H}else{M=NY(H|0,I|0,J|0)|0;N=(1<<J)-1|0;J=NU(N&H,((N|0)<0|0?-1:0)&I,M,G)|0;K=G;L=J}c[t>>2]=1;c[u>>2]=1e6;J=AY(L,K,B+40|0,h)|0;M=G;I=c[v>>2]|0;if((F|0)>(M|0)|(F|0)==(M|0)&x>>>0>J>>>0){q=v;r=I;break L15683}else{C=I}}}while(0);B=C+65304|0;y=c[B>>2]|0;if((y|0)==0){q=B;r=0;break}else{v=B;w=y}}}}while(0);c[k+65304>>2]=r;c[q>>2]=l;i=f;return}function lx(a,b){a=a|0;b=b|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;f=i;i=i+24|0;g=f|0;h=f+8|0;j=f+16|0;k=(c[a+12>>2]|0)+4|0;l=c[k>>2]|0;if((l|0)==0){i=f;return}c[j>>2]=l;m=a+28|0;n=j;o=a+16|0;L15703:do{if((b|0)==0){a=l;while(1){p=c[(c[(c[m>>2]|0)+(c[a+16>>2]<<2)>>2]|0)+12>>2]|0;q=p+48|0;if((c[q>>2]|0)>>>0<2){r=a;break L15703}do{if((iZ(g)|0)>=0){iC(c[g>>2]|0,134,0);ir(c[g>>2]|0,1399285583);ih(c[g>>2]|0,0);ih(c[g>>2]|0,d[a+20|0]|0);s=a+8|0;iw(c[g>>2]|0,c[s>>2]|0,c[s+4>>2]|0);ir(c[g>>2]|0,c[p+65360>>2]|0);s=c[g>>2]|0;t=p;u=c[t>>2]|0;c[t>>2]=u+1;ir(s,u);u=il(c[g>>2]|0,0,0,1)|0;s=G;ir(c[g>>2]|0,0);t=a+21|0;ih(c[g>>2]|0,d[t]|0);ij(c[g>>2]|0,a+22|0,d[t]|0);ij(c[g>>2]|0,a+277|0,e[a+65302>>1]|0);t=c[g>>2]|0;v=il(t,0,0,1)|0;w=G;x=iB(t)|0;il(t,u,s,0)|0;is(t,x);il(t,v,w,0)|0;ik(c[g>>2]|0);w=i$(c[g>>2]|0,h)|0;if((w|0)<0){break}ij(c[o>>2]|0,c[h>>2]|0,w);ik(c[o>>2]|0);A4(c[h>>2]|0);c[q>>2]=(c[q>>2]|0)-1}}while(0);q=c[(c[j>>2]|0)+65304>>2]|0;A6(n);c[j>>2]=q;if((q|0)==0){r=0;break}else{a=q}}}else{a=l;while(1){q=c[(c[(c[m>>2]|0)+(c[a+16>>2]<<2)>>2]|0)+12>>2]|0;p=q+48|0;w=c[p>>2]|0;do{if((iZ(g)|0)>=0){iC(c[g>>2]|0,134,0);ir(c[g>>2]|0,1399285583);ih(c[g>>2]|0,0);ih(c[g>>2]|0,d[a+20|0]|0|((w|0)==1?4:0));v=a+8|0;iw(c[g>>2]|0,c[v>>2]|0,c[v+4>>2]|0);ir(c[g>>2]|0,c[q+65360>>2]|0);v=c[g>>2]|0;t=q;x=c[t>>2]|0;c[t>>2]=x+1;ir(v,x);x=il(c[g>>2]|0,0,0,1)|0;v=G;ir(c[g>>2]|0,0);t=a+21|0;ih(c[g>>2]|0,d[t]|0);ij(c[g>>2]|0,a+22|0,d[t]|0);ij(c[g>>2]|0,a+277|0,e[a+65302>>1]|0);t=c[g>>2]|0;s=il(t,0,0,1)|0;u=G;y=iB(t)|0;il(t,x,v,0)|0;is(t,y);il(t,s,u,0)|0;ik(c[g>>2]|0);u=i$(c[g>>2]|0,h)|0;if((u|0)<0){break}ij(c[o>>2]|0,c[h>>2]|0,u);ik(c[o>>2]|0);A4(c[h>>2]|0);c[p>>2]=(c[p>>2]|0)-1}}while(0);p=c[(c[j>>2]|0)+65304>>2]|0;A6(n);c[j>>2]=p;if((p|0)==0){r=0;break}else{a=p}}}}while(0);c[k>>2]=r;i=f;return}function ly(f,g,h,j,k,l,m,n){f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0;o=i;i=i+16|0;p=o|0;q=o+8|0;r=h;s=c[f+12>>2]|0;t=((k>>>0)/255|0)+1|0;do{if((g|0)==31&(n|0)==0){u=c[h+28>>2]|0;if((u|0)==0){v=h+65368|0;w=c[v+4>>2]|0;x=c[v>>2]|0;y=m;z=l;A=0}else{v=u;B=NY(l|0,m|0,v|0)|0;C=(1<<u)-1|0;D=C;E=(C|0)<0|0?-1:0;C=NU(D&l,E&m,B,G)|0;B=G;F=h+65368|0;H=c[F>>2]|0;I=c[F+4>>2]|0;F=NY(H|0,I|0,v|0)|0;v=NU(D&H,E&I,F,G)|0;w=G;x=v;y=B;z=C;A=u}u=NU(x,w,1,0)|0;C=G;if(!((y|0)>(C|0)|(y|0)==(C|0)&z>>>0>u>>>0)){if((A|0)==0){J=0;break}u=(1<<A)-1|0;if(!((u&l|0)==0&(((u|0)<0|0?-1:0)&m|0)==0)){J=0;break}}u=h+64|0;if((c[u>>2]|0)==(-1|0)&(c[u+4>>2]|0)==(-1|0)){J=1;break}lw(f,r);J=1}else{J=0}}while(0);A=(n|0)!=0;do{if(!A){n=b[h+65358>>1]|0;if(n<<16>>16==0){break}if((65025-(n&65535)|0)>>>0>=k>>>0){break}lw(f,r)}}while(0);n=h+77|0;z=h+76|0;y=h+65358|0;w=h+64|0;x=h+72|0;g=f+28|0;u=h+56|0;C=p|0;B=p+4|0;v=q|0;F=q+4|0;I=s+8|0;E=s+16|0;if(A){A=0;s=j;H=k;while(1){D=t-A|0;K=a[n]|0;L=K&255;M=L^255;N=(D|0)>(M|0)?M:D;if((A|0)!=0&K<<24>>24==0){a[z]=a[z]|1}K=N-1|0;NM(h+(L+78)|0,-1|0,K|0);L=(d[n]|0)+K|0;D=N*255|0;M=H>>>0>D>>>0?D:H;a[n]=(L&255)+1&255;a[h+((L&255)+78)|0]=M+(K*-255|0)&255;K=h+((e[y>>1]|0)+333)|0;NN(K|0,s|0,M)|0;K=N+A|0;b[y>>1]=(e[y>>1]|0)+M&65535;if((K|0)==(t|0)){c[w>>2]=l;c[w+4>>2]=m}if((K|0)<(t|0)){A=K;s=s+M|0;H=H-M|0}else{break}}}else{H=0;s=j;j=k;do{k=t-H|0;A=a[n]|0;M=A&255;K=M^255;N=(k|0)>(K|0)?K:k;if((H|0)!=0&A<<24>>24==0){a[z]=a[z]|1}A=N-1|0;NM(h+(M+78)|0,-1|0,A|0);M=(d[n]|0)+A|0;k=N*255|0;K=j>>>0>k>>>0?k:j;a[n]=(M&255)+1&255;a[h+((M&255)+78)|0]=K+(A*-255|0)&255;A=h+((e[y>>1]|0)+333)|0;NN(A|0,s|0,K)|0;s=s+K|0;j=j-K|0;H=N+H|0;b[y>>1]=(e[y>>1]|0)+K&65535;if((H|0)==(t|0)){c[w>>2]=l;c[w+4>>2]=m}K=c[u>>2]|0;N=c[u+4>>2]|0;A=(c[(c[g>>2]|0)+(c[x>>2]<<2)>>2]|0)+40|0;c[C>>2]=1;c[B>>2]=1e6;M=AY(K,N,A,p)|0;N=G;K=c[w>>2]|0;k=c[w+4>>2]|0;c[v>>2]=1;c[F>>2]=1e6;L=AY(K,k,A,q)|0;A=G;do{if((a[n]|0)==-1){O=11703}else{k=c[I>>2]|0;if((k|0)>0){if((e[y>>1]|0)>=(k|0)){O=11703;break}}k=c[E>>2]|0;K=c[E+4>>2]|0;D=0;if(!((K|0)>(D|0)|(K|0)==(D|0)&k>>>0>0>>>0)){break}D=NV(L,A,M,N)|0;P=G;if(!((P|0)<(K|0)|(P|0)==(K|0)&D>>>0<k>>>0)){O=11703}}}while(0);if((O|0)==11703){O=0;lw(f,r)}}while((H|0)<(t|0))}if((J|0)==0){i=o;return}J=h+64|0;if((c[J>>2]|0)==(-1|0)&(c[J+4>>2]|0)==(-1|0)){i=o;return}lw(f,r);i=o;return}function lz(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;f=c[c[b+12>>2]>>2]|0;g=c[(c[b+28>>2]|0)+(e<<2)>>2]|0;h=f+(e*384|0)+380|0;i=c[h>>2]|0;j=c[f+(e*384|0)>>2]|0;k=c[f+(e*384|0)+12>>2]|0;l=j+k|0;m=c[f+(e*384|0)+16>>2]|0;do{if((m|0)==60){if((NO(l|0,1697720,8)|0)!=0){break}e=A1(24)|0;f=A1(4)|0;if((e|0)==0|(f|0)==0){A4(e);A4(f);n=-12;return n|0}o=j+(k+28)|0;p=d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0;o=j+(k+36)|0;q=d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0;o=j+(k+40)|0;r=d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0;o=j+(k+44)|0;s=d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0;o=j+(k+48)|0;t=d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0;o=j+(k+56)|0;u=d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0;o=g+8|0;c[(c[o>>2]|0)+8>>2]=1;c[(c[o>>2]|0)+48>>2]=86068;c[(c[o>>2]|0)+412>>2]=q;c[(c[o>>2]|0)+416>>2]=r;c[(c[o>>2]|0)+424>>2]=s;A4(c[(c[o>>2]|0)+96>>2]|0);c[(c[o>>2]|0)+96>>2]=e;c[(c[o>>2]|0)+100>>2]=8;if((q|0)!=0){mL(g,64,1,q)}c[f>>2]=u+1;A4(c[h>>2]|0);c[h>>2]=f;f=e;y=t;a[f]=y&255;y=y>>8;a[f+1|0]=y&255;y=y>>8;a[f+2|0]=y&255;y=y>>8;a[f+3|0]=y&255;f=e+4|0;y=p;a[f]=y&255;y=y>>8;a[f+1|0]=y&255;y=y>>8;a[f+2|0]=y&255;y=y>>8;a[f+3|0]=y&255;n=1;return n|0}}while(0);if((i|0)==0){n=0;return n|0}h=i;if((c[h>>2]|0)==0){n=0;return n|0}lR(b,g+88|0,l,m)|0;c[h>>2]=(c[h>>2]|0)-1;n=1;return n|0}function lA(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0;d=i;i=i+48|0;e=d+24|0;f=c[c[a+12>>2]>>2]|0;g=c[(c[a+28>>2]|0)+(b<<2)>>2]|0;a=g+8|0;if((c[(c[a>>2]|0)+48>>2]|0)==117){h=0;i=d;return h|0}j=(c[f+(b*384|0)>>2]|0)+((c[f+(b*384|0)+12>>2]|0)+13)|0;k=(c[f+(b*384|0)+16>>2]<<3)-104|0;b=k>>>0>2147483639|(j|0)==0;f=b?0:j;j=b?0:k;c[e>>2]=f;c[e+12>>2]=j;c[e+16>>2]=j+8;c[e+4>>2]=f+(j>>3);c[e+8>>2]=0;if((nW(c[a>>2]|0,e,d|0)|0)<0){h=-1;i=d;return h|0}c[(c[a>>2]|0)+8>>2]=0;c[(c[a>>2]|0)+48>>2]=117;e=c[a>>2]|0;mL(g,64,c[e+104>>2]|0,c[e+108>>2]<<1);h=1;i=d;return h|0}function lB(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0;g=d>>>31|e<<1;h=e>>31|((e|0)<0|0?-1:0)<<1;i=NU((d>>>9|e<<23)&8191,(e>>>9|0<<23)&0,g,h)|0;j=G;if(((d>>>14|e<<18)&65280|d&255|0)==0&((e>>>14|0<<18)&0|e&0|0)==0){e=(c[c[a+12>>2]>>2]|0)+(b*384|0)+20|0;c[e>>2]=c[e>>2]|1}if((f|0)==0){return(G=j,i)|0}c[f>>2]=g;c[f+4>>2]=h;return(G=j,i)|0}function lC(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0;f=c[c[b+12>>2]>>2]|0;g=c[(c[b+28>>2]|0)+(e<<2)>>2]|0;b=c[f+(e*384|0)>>2]|0;h=c[f+(e*384|0)+12>>2]|0;if((a[b+h|0]|0)!=75){i=0;return i|0}e=g+8|0;c[(c[e>>2]|0)+8>>2]=0;c[(c[e>>2]|0)+48>>2]=117;e=b+(h+12)|0;f=cV(d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24|0)|0;e=b+(h+8)|0;mL(g,64,f,cV(d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24|0)|0);i=1;return i|0}function lD(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0;f=d>>>30|e<<2;g=e>>>30|0<<2;h=d&1073741823;d=e&0;if(!((h|0)==0&(d|0)==0)){i=NU(f,g,h,d)|0;j=G;return(G=j,i)|0}e=(c[c[a+12>>2]>>2]|0)+(b*384|0)+20|0;c[e>>2]=c[e>>2]|1;i=NU(f,g,h,d)|0;j=G;return(G=j,i)|0}function lE(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;f=i;i=i+32|0;g=f|0;h=c[c[b+12>>2]>>2]|0;j=c[(c[b+28>>2]|0)+(e<<2)>>2]|0;k=c[h+(e*384|0)+12>>2]|0;l=c[h+(e*384|0)>>2]|0;m=l+k|0;if((a[m]|0)==-1){n=0;i=f;return n|0}o=c[h+(e*384|0)+16>>2]|0;e=o<<3;h=e>>>0>2147483639|(m|0)==0;p=h?0:m;m=h?8:e+8|0;e=m>>>0>1?1:m;h=p+(e>>>3)|0;q=(cV(d[h]|d[h+1|0]<<8|d[h+2|0]<<16|d[h+3|0]<<24|0)|0)<<(e&1)>>>25;h=e+7|0;e=m>>>0>h>>>0?h:m;if((q|0)==127){h=l+(k+17)|0;r=-e|0;s=m-e|0;if((r|0)>32){t=r}else{t=(s|0)<32?s:32}s=t+e|0;e=p+(s>>>3)|0;t=s+8|0;r=m>>>0>t>>>0?t:m;if(((cV(d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24|0)|0)<<(s&7)&-16777216|0)!=16777216){n=-1;i=f;return n|0}s=-r|0;e=m-r|0;if((s|0)>24){u=s}else{u=(e|0)<24?e:24}e=u+r|0;r=-e|0;u=m-e|0;if((r|0)>32){v=r}else{v=(u|0)<32?u:32}u=v+e|0;e=p+(u>>>3)|0;v=(cV(d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24|0)|0)<<(u&7)&-65536;e=u+16|0;u=m>>>0>e>>>0?e:m;m=p+(u>>>3)|0;if(((cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0)<<(u&7)>>>16|v|0)!=34){n=-1;i=f;return n|0}v=j+8|0;nY(c[v>>2]|0,g,h);c[(c[v>>2]|0)+8>>2]=1;c[(c[v>>2]|0)+48>>2]=86028;c[j+236>>2]=2;g=A1(50)|0;c[(c[v>>2]|0)+96>>2]=g;g=c[(c[v>>2]|0)+96>>2]|0;NN(g|0,h|0,34)|0;c[(c[v>>2]|0)+100>>2]=34;mL(j,64,1,c[(c[v>>2]|0)+412>>2]|0);n=1;i=f;return n|0}else if((q|0)==4){lR(b,j+88|0,l+(k+4)|0,o-4|0)|0;n=1;i=f;return n|0}else{n=1;i=f;return n|0}return 0}function lF(a,b){a=a|0;b=b|0;var d=0;d=(c[(c[a+28>>2]|0)+(b<<2)>>2]|0)+8|0;c[(c[d>>2]|0)+8>>2]=1;c[(c[d>>2]|0)+48>>2]=86028;return 0}function lG(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;f=i;i=i+8|0;g=f|0;h=c[c[b+12>>2]>>2]|0;j=c[(c[b+28>>2]|0)+(e<<2)>>2]|0;k=c[h+(e*384|0)>>2]|0;l=c[h+(e*384|0)+12>>2]|0;m=a[k+l|0]|0;if((m&1)==0){n=0;i=f;return n|0}if((m<<24>>24|0)==1){o=a[k+(l+1)|0]|0;do{if(o<<24>>24==118){p=j+8|0;c[(c[p>>2]|0)+8>>2]=0;q=k+(l+9)|0;r=d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0;q=mA(253424,r)|0;c[(c[p>>2]|0)+48>>2]=q;c[(c[p>>2]|0)+52>>2]=r;s=k+(l+13)|0}else{r=j+8|0;p=(c[r>>2]|0)+8|0;if(o<<24>>24==116){c[p>>2]=3;c[(c[r>>2]|0)+48>>2]=94210;s=k+(l+13)|0;break}c[p>>2]=1;p=k+(l+9)|0;c[g>>2]=d[p]|d[p+1|0]<<8|d[p+2|0]<<16|d[p+3|0]<<24;p=k+(l+13)|0;a[g+4|0]=0;q=mA(251144,aU(g|0,0,16)|0)|0;c[(c[r>>2]|0)+48>>2]=q;if((c[(c[r>>2]|0)+48>>2]|0)==86018){s=p;break}c[j+236>>2]=1;s=p}}while(0);g=s;o=d[g]|d[g+1|0]<<8|d[g+2|0]<<16|d[g+3|0]<<24|0;g=c[h+(e*384|0)+16>>2]|0;p=o>>>0>g>>>0?g:o;o=s+4|0;g=o|0;r=d[g]|d[g+1|0]<<8|d[g+2|0]<<16|d[g+3|0]<<24|0;g=o+4|0;o=d[g]|d[g+1|0]<<8|d[g+2|0]<<16|d[g+3|0]<<24|0;g=s+12|0;q=g|0;t=d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0;q=g+4|0;g=d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0;q=j+8|0;u=c[q>>2]|0;v=s+32|0;if((c[u+8>>2]|0)==0){w=v;c[u+120>>2]=d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24;w=s+36|0;c[(c[q>>2]|0)+124>>2]=d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24;w=N2(t,g,1e7,0)|0;mL(j,64,r,w);n=1;i=f;return n|0}w=v;c[u+416>>2]=(d[w]|d[w+1|0]<<8)<<16>>16&65535;w=s+40|0;u=s+36|0;c[(c[q>>2]|0)+72>>2]=(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24)<<3;if((r|0)==0&(o|0)==0){x=0}else{u=N2(t,g,1e7,0)|0;g=N3(u,G,r,o)|0;x=g}c[(c[q>>2]|0)+412>>2]=x;mL(j,64,1,c[(c[q>>2]|0)+412>>2]|0);do{if(p>>>0>55){if((c[(c[q>>2]|0)+48>>2]|0)!=86018){y=p;z=w;break}y=p-4|0;z=s+44|0}else{y=p;z=w}}while(0);if(y>>>0<=52){n=1;i=f;return n|0}w=y-52|0;c[(c[q>>2]|0)+100>>2]=w;p=A1(y-36|0)|0;c[(c[q>>2]|0)+96>>2]=p;p=c[(c[q>>2]|0)+96>>2]|0;NN(p|0,z|0,w)|0;n=1;i=f;return n|0}else if((m<<24>>24|0)==3){m=c[h+(e*384|0)+16>>2]|0;if(m>>>0<=8){n=1;i=f;return n|0}lR(b,j+88|0,k+(l+7)|0,m-8|0)|0;n=1;i=f;return n|0}else{n=1;i=f;return n|0}return 0}function lH(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0;f=c[c[b+12>>2]>>2]|0;b=c[f+(e*384|0)>>2]|0;g=f+(e*384|0)+12|0;h=c[g>>2]|0;i=b+h|0;j=a[i]|0;if((j&8)==0){k=j}else{j=f+(e*384|0)+20|0;c[j>>2]=c[j>>2]|1;k=a[i]|0}i=k&255;k=i<<1&4|i>>>6;i=k+1|0;c[g>>2]=i+h;g=f+(e*384|0)+16|0;c[g>>2]=(c[g>>2]|0)-i;if((k|0)==0){return 0}i=f+(e*384|0)+24|0;e=k;k=c[i>>2]|0;while(1){f=e-1|0;g=((d[b+(e+h)|0]|0)<<(f<<3))+k|0;c[i>>2]=g;if((f|0)==0){break}else{e=f;k=g}}return 0}function lI(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0;f=c[c[b+12>>2]>>2]|0;g=c[(c[b+28>>2]|0)+(e<<2)>>2]|0;b=c[f+(e*384|0)>>2]|0;h=c[f+(e*384|0)+12>>2]|0;e=a[b+h|0]|0;if((e&1)==0){i=0;return i|0}if(e<<24>>24!=1){i=1;return i|0}e=b+(h+96)|0;f=d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24|0;if((f|0)==89694081){e=g+8|0;c[(c[e>>2]|0)+8>>2]=1;j=b+(h+124)|0;k=mA(251144,(d[j]|d[j+1|0]<<8)<<16>>16&65535)|0;c[(c[e>>2]|0)+48>>2]=k;k=b+(h+126)|0;c[(c[e>>2]|0)+416>>2]=(d[k]|d[k+1|0]<<8)<<16>>16&65535;k=b+(h+128)|0;c[(c[e>>2]|0)+412>>2]=d[k]|d[k+1|0]<<8|d[k+2|0]<<16|d[k+3|0]<<24;k=b+(h+132)|0;c[(c[e>>2]|0)+72>>2]=(d[k]|d[k+1|0]<<8|d[k+2|0]<<16|d[k+3|0]<<24)<<3;i=1;return i|0}else if((f|0)==89694080){f=g+8|0;c[(c[f>>2]|0)+8>>2]=0;k=b+(h+68)|0;e=mA(253424,d[k]|d[k+1|0]<<8|d[k+2|0]<<16|d[k+3|0]<<24|0)|0;c[(c[f>>2]|0)+48>>2]=e;e=b+(h+164)|0;mL(g,64,d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24|0,1e7);e=b+(h+176)|0;c[(c[f>>2]|0)+120>>2]=d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24;e=b+(h+180)|0;c[(c[f>>2]|0)+124>>2]=d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24;i=1;return i|0}else{i=1;return i|0}return 0}function lJ(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=c[c[a+12>>2]>>2]|0;f=c[(c[a+28>>2]|0)+(b<<2)>>2]|0;g=e+(b*384|0)+380|0;h=c[g>>2]|0;i=c[e+(b*384|0)>>2]|0;j=c[e+(b*384|0)+12>>2]|0;k=i+j|0;do{if((h|0)==0){l=A7(16)|0;c[g>>2]=l;if((l|0)==0){m=-12}else{n=l;break}return m|0}else{n=h}}while(0);if((c[e+(b*384|0)+80>>2]&2|0)==0){h=n;if((c[h>>2]|0)==0){m=0;return m|0}g=c[e+(b*384|0)+16>>2]|0;if(g>>>0<8){m=-1094995529;return m|0}if((NO(k|0,1676848,8)|0)!=0){m=-1094995529;return m|0}l=f+88|0;o=i+(j+8)|0;p=g-8|0;lR(a,l,o,p)|0;c[h>>2]=(c[h>>2]|0)-1;m=1;return m|0}else{h=e+(b*384|0)+16|0;if((c[h>>2]|0)>>>0<19){m=-1094995529;return m|0}if((d[i+(j+8)|0]|0)>15){m=-1094995529;return m|0}b=f+8|0;c[(c[b>>2]|0)+8>>2]=1;c[(c[b>>2]|0)+48>>2]=1330664787;c[(c[b>>2]|0)+416>>2]=d[i+(j+9)|0]|0;e=i+(j+10)|0;c[n+4>>2]=(d[e]|d[e+1|0]<<8)<<16>>16&65535;e=A1((c[h>>2]|0)+16|0)|0;if((e|0)==0){m=-12;return m|0}j=c[h>>2]|0;NN(e|0,k|0,j)|0;c[(c[b>>2]|0)+96>>2]=e;c[(c[b>>2]|0)+100>>2]=c[h>>2];c[(c[b>>2]|0)+412>>2]=48e3;mL(f,64,1,48e3);c[n>>2]=1;m=1;return m|0}return 0}function lK(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;e=i;f=c[c[b+12>>2]>>2]|0;g=c[(c[b+28>>2]|0)+(d<<2)>>2]|0;h=c[f+(d*384|0)+380>>2]|0;j=c[f+(d*384|0)>>2]|0;k=c[f+(d*384|0)+12>>2]|0;l=c[f+(d*384|0)+16>>2]|0;if((l|0)==0){m=-1094995529;i=e;return m|0}n=a[j+k|0]|0;o=n&255;p=o>>>3;q=o&3;do{if((n&255)<96){o=(p&3)*960|0;r=o>>>0<480?480:o}else{if(n<<24>>24>-1){r=480<<(p&1);break}else{r=120<<(p&3);break}}}while(0);do{if((q|0)==0){s=1}else if((q|0)==3){if(l>>>0<2){m=-1094995529;i=e;return m|0}else{s=a[j+(k+1)|0]&63;break}}else{s=2}}while(0);k=ac(s,r)|0;r=f+(d*384|0)+24|0;c[r>>2]=k;s=f+(d*384|0)+48|0;j=c[s>>2]|0;l=c[s+4>>2]|0;if((j|0)==0&(l|0)==(-2147483648|0)){q=h+8|0;t=k;u=c[q+4>>2]|0;v=c[q>>2]|0}else{q=g+48|0;if((c[q>>2]|0)==0&(c[q+4>>2]|0)==(-2147483648|0)){c[q>>2]=j;c[q+4>>2]=l;w=c[s+4>>2]|0;x=c[s>>2]|0}else{w=l;x=j}j=NV(x,w,c[h+4>>2]|0,0)|0;w=G;c[s>>2]=j;c[s+4>>2]=w;s=f+(d*384|0)+56|0;c[s>>2]=j;c[s+4>>2]=w;s=h+8|0;c[s>>2]=j;c[s+4>>2]=w;t=c[r>>2]|0;u=w;v=j}j=h+8|0;w=NU(v,u,t,0)|0;t=G;c[j>>2]=w;c[j+4>>2]=t;if((c[f+(d*384|0)+80>>2]&4|0)==0){m=0;i=e;return m|0}j=f+(d*384|0)+32|0;d=NV(w,t,c[j>>2]|0,c[j+4>>2]|0)|0;j=NU(c[h+4>>2]|0,0,d,G)|0;d=G;h=c[r>>2]|0;t=0;w=(d|0)>(t|0)|(d|0)==(t|0)&j>>>0>h>>>0;f=w?h:j;j=w?t:d;d=0;if(!((j|0)>(d|0)|(j|0)==(d|0)&f>>>0>0>>>0)){m=0;i=e;return m|0}if((j|0)<(t|0)|(j|0)==(t|0)&f>>>0<h>>>0){d=NV(h,t,f,j)|0;y=d}else{y=1}c[r>>2]=y;AN(b,24,1699184,(b=i,i=i+8|0,c[b>>2]=y,b)|0);i=b;m=0;i=e;return m|0}function lL(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;e=i;i=i+16|0;f=e|0;g=e+8|0;h=c[a+12>>2]|0;j=h;k=c[j>>2]|0;l=c[(c[a+28>>2]|0)+(b<<2)>>2]|0;m=c[k+(b*384|0)>>2]|0;n=c[k+(b*384|0)+12>>2]|0;o=m+n|0;p=l+8|0;q=(c[p>>2]|0)+16|0;NN(q|0,1731976,9)|0;c[(c[p>>2]|0)+8>>2]=2;p=c[k+(b*384|0)+16>>2]|0;if((c[k+(b*384|0)+80>>2]&4|0)!=0&(p|0)==0){r=1;i=e;return r|0}if(p>>>0<8){r=-1;i=e;return r|0}if((ba(o|0,1696952,8)|0)==0){if(p>>>0<64){r=-1;i=e;return r|0}q=m+(n+8)|0;s=(d[q]|d[q+1|0]<<8)<<16>>16;if((s-3&65535)>1){q=m+(n+10)|0;t=(d[q]|d[q+1|0]<<8)<<16>>16&65535;AN(a,24,1699152,(u=i,i=i+16|0,c[u>>2]=s&65535,c[u+8>>2]=t,u)|0);i=u;r=-1;i=e;return r|0}t=m+(n+12)|0;s=t|0;q=d[s]|d[s+1|0]<<8|d[s+2|0]<<16|d[s+3|0]<<24|0;s=t+4|0;t=d[s]|d[s+1|0]<<8|d[s+2|0]<<16|d[s+3|0]<<24|0;s=m+(n+20)|0;v=s|0;w=d[v]|d[v+1|0]<<8|d[v+2|0]<<16|d[v+3|0]<<24|0;v=s+4|0;s=d[v]|d[v+1|0]<<8|d[v+2|0]<<16|d[v+3|0]<<24|0;v=0;x=0;if(!(((s|0)>(v|0)|(s|0)==(v|0)&w>>>0>0>>>0)&((t|0)>(x|0)|(t|0)==(x|0)&q>>>0>0>>>0))){r=1;i=e;return r|0}x=2147483647;v=0;BX(f,g,q,t,w,s,x,v)|0;mL(l,64,1,c[g>>2]|0);g=c[f>>2]|0;f=g;v=(g|0)<0|0?-1:0;g=l+48|0;c[g>>2]=f;c[g+4>>2]=v;g=k+(b*384|0)+48|0;c[g>>2]=f;c[g+4>>2]=v;r=1;i=e;return r|0}if((ba(o|0,1676840,8)|0)!=0){r=1;i=e;return r|0}if(p>>>0<52){r=-1;i=e;return r|0}p=m+(n+12)|0;o=d[p]|d[p+1|0]<<8|d[p+2|0]<<16|d[p+3|0]<<24|0;p=c[h+4>>2]|0;h=0;while(1){if((h|0)>=(p|0)){break}y=c[j>>2]|0;if((c[y+(h*384|0)+28>>2]|0)==(o|0)){z=11877;break}else{h=h+1|0}}do{if((z|0)==11877){o=m+(n+36)|0;j=o|0;p=d[j]|d[j+1|0]<<8|d[j+2|0]<<16|d[j+3|0]<<24|0;j=o+4|0;o=d[j]|d[j+1|0]<<8|d[j+2|0]<<16|d[j+3|0]<<24|0;if((h|0)<0){break}j=y+(h*384|0)+40|0;if(!((c[j>>2]|0)==(-1|0)&(c[j+4>>2]|0)==(-1|0))){AN(a,24,1643960,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;r=1;i=e;return r|0}if((p|0)==(-1|0)&(o|0)==(-1|0)){r=1;i=e;return r|0}c[j>>2]=p;c[j+4>>2]=o;r=1;i=e;return r|0}}while(0);AN(a,24,1660928,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;r=1;i=e;return r|0}function lM(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;e=i;f=c[c[a+12>>2]>>2]|0;g=f+(b*384|0)+380|0;h=c[g>>2]|0;j=c[(c[a+28>>2]|0)+(b<<2)>>2]|0;k=c[f+(b*384|0)>>2]|0;l=c[f+(b*384|0)+12>>2]|0;m=k+l|0;if((h|0)==0){n=A7(12)|0;c[g>>2]=n;o=n}else{o=h}h=o+8|0;n=c[h>>2]|0;if((n|0)>1){p=0;i=e;return p|0}if((n|0)==0){n=j+8|0;c[(c[n>>2]|0)+8>>2]=1;c[(c[n>>2]|0)+48>>2]=86052;g=f+(b*384|0)+16|0;if((c[g>>2]|0)>>>0<68){AN(a,16,1731944,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;p=-1094995529;i=e;return p|0}r=k+(l+36)|0;c[(c[n>>2]|0)+412>>2]=d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24;r=k+(l+48)|0;c[(c[n>>2]|0)+416>>2]=d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24;r=c[n>>2]|0;s=c[r+416>>2]|0;if((s-1|0)>>>0>1){AN(a,16,1699088,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;p=-1094995529;i=e;return p|0}q=(s|0)==1;s=r+448|0;c[s>>2]=q?4:3;c[s+4>>2]=q?0:0;q=k+(l+56)|0;s=d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0;q=o;c[q>>2]=s;o=k+(l+64)|0;l=d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0;if((l|0)!=0){c[q>>2]=ac(l,s)|0}c[(c[n>>2]|0)+100>>2]=c[g>>2];g=A1((c[(c[n>>2]|0)+100>>2]|0)+16|0)|0;c[(c[n>>2]|0)+96>>2]=g;g=c[n>>2]|0;s=c[g+96>>2]|0;l=c[g+100>>2]|0;NN(s|0,m|0,l)|0;mL(j,64,1,c[(c[n>>2]|0)+412>>2]|0)}else{n=j+88|0;j=c[f+(b*384|0)+16>>2]|0;lR(a,n,m,j)|0}c[h>>2]=(c[h>>2]|0)+1;p=1;i=e;return p|0}function lN(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;e=c[c[b+12>>2]>>2]|0;b=c[e+(d*384|0)+380>>2]|0;f=c[b>>2]|0;g=e+(d*384|0)+80|0;h=e+(d*384|0)+48|0;do{if((c[g>>2]&4|0)!=0){i=c[h>>2]|0;j=c[h+4>>2]|0;if((i|0)==0&(j|0)==(-2147483648|0)){break}k=e+(d*384|0)+32|0;l=c[k>>2]|0;m=c[k+4>>2]|0;if((l|0)==0&(m|0)==0){break}k=NV(l,m,i,j)|0;j=G;i=c[e+(d*384|0)+92>>2]|0;if((i|0)>0){m=0;l=0;do{l=((a[e+(d*384|0)+100+m|0]|0)!=-1)+l|0;m=m+1|0;}while((m|0)<(i|0));n=l-1|0}else{n=-1}i=NV(k,j,ac(n,f)|0,0)|0;c[b+4>>2]=i}}while(0);do{if((c[h>>2]|0)==0&(c[h+4>>2]|0)==0){n=e+(d*384|0)+32|0;i=c[n>>2]|0;m=c[n+4>>2]|0;if((i|0)==0&(m|0)==0){break}n=c[e+(d*384|0)+92>>2]|0;if((n|0)>0){o=0;p=0;while(1){q=((a[e+(d*384|0)+100+o|0]|0)!=-1)+p|0;r=o+1|0;if((r|0)<(n|0)){o=r;p=q}else{s=q;break}}}else{s=0}p=ac(s,f)|0;o=NV(i,m,p,(p|0)<0|0?-1:0)|0;p=G;n=e+(d*384|0)+56|0;c[n>>2]=o;c[n+4>>2]=p;c[h>>2]=o;c[h+4>>2]=p}}while(0);do{if((c[g>>2]&4|0)!=0){if((c[e+(d*384|0)+96>>2]|0)!=(c[e+(d*384|0)+92>>2]|0)){break}h=c[b+4>>2]|0;if((h|0)==0){break}c[e+(d*384|0)+24>>2]=h;return 0}}while(0);c[e+(d*384|0)+24>>2]=f;return 0}function lO(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;f=i;g=c[c[b+12>>2]>>2]|0;h=c[(c[b+28>>2]|0)+(e<<2)>>2]|0;j=g+(e*384|0)+380|0;k=c[j>>2]|0;l=h+8|0;m=g+(e*384|0)+16|0;n=(c[m>>2]|0)+(c[(c[l>>2]|0)+100>>2]|0)|0;o=n+2|0;p=g+(e*384|0)+12|0;q=c[p>>2]|0;r=g+(e*384|0)|0;e=c[r>>2]|0;g=a[e+q|0]|0;if(g<<24>>24>=0){s=0;i=f;return s|0}if((k|0)==0){t=A7(12)|0;c[j>>2]=t;j=c[p>>2]|0;u=c[r>>2]|0;v=t;w=j;x=u;y=a[u+j|0]|0}else{v=k;w=q;x=e;y=g}g=x+w|0;e=y&255;if((e|0)==129){y=h+88|0;q=x+(w+7)|0;w=(c[m>>2]|0)-7|0;lR(b,y,q,w)|0;z=11956}else if((e|0)==130){z=11956}else if((e|0)==128){w=c[m>>2]<<3;q=w>>>0>2147483639|(g|0)==0;y=q?0:g;g=q?8:w+8|0;w=(g|0)<56?g:56;q=y+(w>>>3)|0;x=cV(d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0)|0;q=x>>>8;k=w+24|0;w=g>>>0>k>>>0?k:g;k=v+8|0;c[k>>2]=q;if(x>>>0<50397184){AN(b,16,1731816,(A=i,i=i+8|0,c[A>>2]=q,A)|0);i=A;s=-1;i=f;return s|0}q=y+(w>>>3)|0;x=w+16|0;j=g>>>0>x>>>0?x:g;x=(cV(d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0)|0)<<(w&7)>>>16<<4;w=y+(j>>>3)|0;q=j+16|0;u=g>>>0>q>>>0?q:g;q=(cV(d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0)|0)<<(j&7)>>>16<<4;oU(c[l>>2]|0,x,q);q=c[k>>2]|0;if(q>>>0>197631){x=u+100|0;B=g>>>0>x>>>0?x:g;z=11943}else{if(q>>>0>197119){B=u;z=11943}else{C=u}}if((z|0)==11943){u=y+(B>>>3)|0;q=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(B&7)>>>8;u=B+24|0;B=g>>>0>u>>>0?u:g;u=y+(B>>>3)|0;x=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(B&7)>>>8;u=B+24|0;B=g>>>0>u>>>0?u:g;u=c[l>>2]|0;j=c[u+120>>2]|0;do{if((q|0)<=(j|0)&(q|0)>(j-16|0)){w=c[u+124>>2]|0;if(!((x|0)<=(w|0)&(x|0)>(w-16|0))){break}oU(u,q,x)}}while(0);x=B+16|0;C=g>>>0>x>>>0?x:g}x=y+(C>>>3)|0;B=(cV(d[x]|d[x+1|0]<<8|d[x+2|0]<<16|d[x+3|0]<<24|0)|0)<<(C&7)&-65536;x=C+16|0;C=g>>>0>x>>>0?x:g;x=y+(C>>>3)|0;q=C+16|0;u=g>>>0>q>>>0?q:g;q=(cV(d[x]|d[x+1|0]<<8|d[x+2|0]<<16|d[x+3|0]<<24|0)|0)<<(C&7)>>>16|B;B=y+(u>>>3)|0;C=(cV(d[B]|d[B+1|0]<<8|d[B+2|0]<<16|d[B+3|0]<<24|0)|0)<<(u&7)&-65536;B=u+16|0;u=g>>>0>B>>>0?B:g;B=y+(u>>>3)|0;x=u+16|0;j=g>>>0>x>>>0?x:g;x=(cV(d[B]|d[B+1|0]<<8|d[B+2|0]<<16|d[B+3|0]<<24|0)|0)<<(u&7)>>>16|C;if((x|0)>0&(q|0)>0){D=x;E=q}else{AN(b,24,1699032,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;D=1;E=25}mL(h,64,D,E);E=y+(j>>>3)|0;D=j+24|0;q=g>>>0>D>>>0?D:g;c[h+80>>2]=(cV(d[E]|d[E+1|0]<<8|d[E+2|0]<<16|d[E+3|0]<<24|0)|0)<<(j&7)>>>8;j=y+(q>>>3)|0;E=q+24|0;D=g>>>0>E>>>0?E:g;c[h+84>>2]=(cV(d[j]|d[j+1|0]<<8|d[j+2|0]<<16|d[j+3|0]<<24|0)|0)<<(q&7)>>>8;q=c[k>>2]|0;do{if(q>>>0>197119){k=-D|0;j=g-D|0;if((k|0)>38){F=k}else{F=(j|0)<38?j:38}j=F+D|0;if(q>>>0<=3162111){G=j;break}k=j+2|0;G=g>>>0>k>>>0?k:g}else{G=D}}while(0);D=y+(G>>>3)|0;y=(cV(d[D]|d[D+1|0]<<8|d[D+2|0]<<16|d[D+3|0]<<24|0)|0)<<(G&7)>>>27;c[v>>2]=y;c[v+4>>2]=(1<<y)-1;c[(c[l>>2]|0)+8>>2]=0;c[(c[l>>2]|0)+48>>2]=31;c[h+236>>2]=2}else{AN(b,16,1676776,(A=i,i=i+8|0,c[A>>2]=e,A)|0);i=A;s=-1;i=f;return s|0}do{if((z|0)==11956){if((c[v+8>>2]|0)==0){s=-1}else{break}i=f;return s|0}}while(0);v=A2(c[(c[l>>2]|0)+96>>2]|0,n+18|0)|0;c[(c[l>>2]|0)+96>>2]=v;v=c[l>>2]|0;n=c[v+96>>2]|0;z=c[v+100>>2]|0;a[n+z|0]=(c[m>>2]|0)>>>8&255;v=n+(z+2)|0;a[n+(z+1)|0]=c[m>>2]&255;z=(c[r>>2]|0)+(c[p>>2]|0)|0;p=c[m>>2]|0;NN(v|0,z|0,p)|0;c[(c[l>>2]|0)+100>>2]=o;s=1;i=f;return s|0}function lP(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;e=c[c[b+12>>2]>>2]|0;f=e+(d*384|0)+48|0;g=c[f>>2]|0;h=c[f+4>>2]|0;do{if((g|0)==0&(h|0)==0|(g|0)==0&(h|0)==(-2147483648|0)){if((c[e+(d*384|0)+80>>2]&4|0)!=0){break}i=c[e+(d*384|0)+96>>2]|0;j=c[e+(d*384|0)+92>>2]|0;if((i|0)<(j|0)){k=1;l=i;do{k=((a[e+(d*384|0)+100+l|0]|0)!=-1)+k|0;l=l+1|0;}while((l|0)<(j|0));m=(k|0)<0|0?-1:0;n=k}else{m=0;n=1}j=e+(d*384|0)+32|0;l=c[j>>2]|0;i=c[j+4>>2]|0;j=c[e+(d*384|0)+380>>2]|0;if((j|0)==0){o=-2147483648;p=0}else{q=NX(l|0,i|0,c[j>>2]|0)|0;r=c[j+4>>2]|0;s=r&l;l=((r|0)<0|0?-1:0)&i;i=(c[j+8>>2]|0)>>>0<197121|0;if((s|0)==0&(l|0)==0){j=e+(d*384|0)+20|0;c[j>>2]=c[j>>2]|1}j=NU(s,l,q,G)|0;q=NU(j,G,i,0)|0;o=G;p=q}q=NV(p,o,n,m)|0;i=G;j=e+(d*384|0)+56|0;c[j>>2]=q;c[j+4>>2]=i;c[f>>2]=q;c[f+4>>2]=i;j=b+28|0;l=(c[(c[j>>2]|0)+(d<<2)>>2]|0)+48|0;if(!((c[l>>2]|0)==0&(c[l+4>>2]|0)==(-2147483648|0))){break}c[l>>2]=q;c[l+4>>2]=i;i=c[(c[j>>2]|0)+(d<<2)>>2]|0;j=i+56|0;l=c[j>>2]|0;q=c[j+4>>2]|0;if((l|0)==0&(q|0)==0){break}s=i+48|0;i=NV(l,q,c[s>>2]|0,c[s+4>>2]|0)|0;c[j>>2]=i;c[j+4>>2]=G}}while(0);if((c[e+(d*384|0)+16>>2]|0)==0){return 0}c[e+(d*384|0)+24>>2]=1;return 0}function lQ(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0;g=c[c[a+12>>2]>>2]|0;a=c[g+(b*384|0)+380>>2]|0;if((a|0)==0){h=-2147483648;i=0;return(G=h,i)|0}j=NX(d|0,e|0,c[a>>2]|0)|0;k=c[a+4>>2]|0;l=k&d;d=((k|0)<0|0?-1:0)&e;e=NU((c[a+8>>2]|0)>>>0<197121|0,0,j,G)|0;if((l|0)==0&(d|0)==0){j=g+(b*384|0)+20|0;c[j>>2]=c[j>>2]|1}j=NU(e,G,l,d)|0;d=G;if((f|0)==0){h=d;i=j;return(G=h,i)|0}c[f>>2]=j;c[f+4>>2]=d;h=d;i=j;return(G=h,i)|0}function lR(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0;h=i;i=i+72|0;j=h|0;k=h+8|0;l=h+16|0;m=h+24|0;n=h+32|0;o=h+40|0;p=h+48|0;q=h+56|0;r=h+64|0;s=f+g|0;if((g|0)<8){t=-1;i=h;return t|0}g=f;u=d[g]|d[g+1|0]<<8|d[g+2|0]<<16|d[g+3|0]<<24|0;g=s;if((-4-(f+4)+g|0)<(u|0)|(u|0)<0){t=-1;i=h;return t|0}v=f+(u+8)|0;w=f+(u+4)|0;u=d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0;w=(u|0)==0;L16137:do{if((g-v|0)<4|w){x=v;y=u;z=w}else{f=p;A=q;B=b;C=r;D=o|0;E=o+4|0;F=b+1132|0;G=b+1136|0;H=v;I=u;while(1){J=H+4|0;K=H;L=d[K]|d[K+1|0]<<8|d[K+2|0]<<16|d[K+3|0]<<24|0;K=J;if((g-K|0)<(L|0)|(L|0)<0){x=J;y=I;z=0;break L16137}M=H+(L+4)|0;N=I-1|0;O=a9(J|0,61,L|0)|0;L16142:do{if((O|0)!=0){P=O-K|0;Q=L-P|0;R=Q-1|0;S=O+1|0;if((O|0)==(J|0)|(R|0)==0){break}T=A1(P+1|0)|0;c[p>>2]=T;U=A1(Q)|0;c[q>>2]=U;if((T|0)==0|(U|0)==0){A6(f);A6(A);AN(B,24,1696376,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V;break}else{W=0;X=T}do{T=a[H+(W+4)|0]|0;a[X+W|0]=((T<<24>>24)-97|0)>>>0<26?T^32:T;W=W+1|0;X=c[p>>2]|0}while(W>>>0<P>>>0);a[X+P|0]=0;T=c[q>>2]|0;NN(T|0,S|0,R)|0;a[(c[q>>2]|0)+R|0]=0;T=c[p>>2]|0;if((a8(T|0,1731792)|0)==0){U=A1(R)|0;c[r>>2]=U;if((U|0)==0){AN(B,24,1698984,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V;break}Q=y8(U,c[q>>2]|0,R)|0;if((Q|0)>0){Y=nj(b,U,Q)|0}else{Y=Q}A6(C);if((Y|0)>=0){break}AN(B,24,1676736,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V;break}Q=c[q>>2]|0;U=NL(T|0)|0;L16160:do{if((U|0)>=9){Z=ct(T|0,1618976,(V=i,i=i+8|0,c[V>>2]=j,V)|0)|0;i=V;if((Z|0)!=1){break}if((U|0)==9){Z=ct(Q|0,1614776,(V=i,i=i+32|0,c[V>>2]=k,c[V+8>>2]=l,c[V+16>>2]=m,c[V+24>>2]=n,V)|0)|0;i=V;if((Z|0)<4){break}Z=c[j>>2]|0;c[D>>2]=1;c[E>>2]=1e3;_=((((((c[k>>2]|0)*60|0)+(c[l>>2]|0)|0)*60|0)+(c[m>>2]|0)|0)*1e3|0)+(c[n>>2]|0)|0;$=_;aa=(_|0)<0|0?-1:0;_=0;ab=-2147483648;mM(b,Z,o,$,aa,_,ab,0)|0;A4(Q)}else{if((a8(T+9|0,1743792)|0)!=0){break}ab=c[F>>2]|0;_=c[j>>2]|0;aa=0;while(1){if(aa>>>0>=ab>>>0){break L16160}ac=c[(c[G>>2]|0)+(aa<<2)>>2]|0;if((c[ac>>2]|0)==(_|0)){break}else{aa=aa+1|0}}if((ac|0)==0){break}zK(ac+32|0,1737360,Q,8)|0}A4(T);break L16142}}while(0);zK(e,c[p>>2]|0,c[q>>2]|0,12)|0}}while(0);J=(N|0)==0;if((g-M|0)<4|J){x=M;y=N;z=J;break}else{H=M;I=N}}}}while(0);if((x|0)!=(s|0)){AN(b,32,1660880,(V=i,i=i+8|0,c[V>>2]=g-x,V)|0);i=V}if(!z){AN(b,32,1643896,(V=i,i=i+8|0,c[V>>2]=y,V)|0);i=V}jx(e,0,237744);t=0;i=h;return t|0}function lS(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;f=i;g=c[c[b+12>>2]>>2]|0;h=c[(c[b+28>>2]|0)+(e<<2)>>2]|0;j=g+(e*384|0)+12|0;k=g+(e*384|0)|0;l=a[(c[k>>2]|0)+(c[j>>2]|0)|0]|0;m=l&255;n=g+(e*384|0)+380|0;o=c[n>>2]|0;p=(o|0)!=0;if((m&1|0)==0){q=(p^1)<<31>>31;i=f;return q|0}do{if(p){r=o}else{s=A7(336)|0;c[n>>2]=s;if((s|0)==0){q=-1}else{r=s;break}i=f;return q|0}}while(0);n=g+(e*384|0)+16|0;e=c[n>>2]|0;if((e|0)==0|(l&255)>5){q=-1;i=f;return q|0}g=m>>>1;m=r+12|0;o=m+(g<<2)|0;if((c[o>>2]|0)!=0){q=-1;i=f;return q|0}do{if((l&255)>1){if((c[m>>2]|0)==0){q=-1;i=f;return q|0}if((l&255)<=3){break}if((c[r+16>>2]|0)==0){q=-1}else{break}i=f;return q|0}}while(0);c[r+(g<<2)>>2]=e;e=A7(c[n>>2]|0)|0;c[o>>2]=e;if((e|0)==0){q=-12;i=f;return q|0}o=(c[k>>2]|0)+(c[j>>2]|0)|0;g=c[n>>2]|0;NN(e|0,o|0,g)|0;g=c[j>>2]|0;j=c[k>>2]|0;k=a[j+g|0]|0;if((k<<24>>24|0)==3){o=c[n>>2]|0;if(o>>>0<=8){q=1;i=f;return q|0}if((lR(b,h+88|0,j+(g+7)|0,o-8|0)|0)<=-1){q=1;i=f;return q|0}o=r+16|0;e=c[o>>2]|0;l=e+7|0;p=d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0;l=p+16|0;if(p>>>0>=4294967280){q=1;i=f;return q|0}if(l>>>0>=(c[n>>2]|0)>>>0){q=1;i=f;return q|0}s=e+(p+11)|0;y=0;a[s]=y&255;y=y>>8;a[s+1|0]=y&255;y=y>>8;a[s+2|0]=y&255;y=y>>8;a[s+3|0]=y&255;a[(c[o>>2]|0)+(p+15)|0]=1;c[r+4>>2]=l;q=1;i=f;return q|0}else if((k<<24>>24|0)==1){if((c[n>>2]|0)!=30){q=-1;i=f;return q|0}n=j+(g+7)|0;if((d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0)!=0){q=-1;i=f;return q|0}n=d[j+(g+11)|0]|0;k=h+8|0;l=(c[k>>2]|0)+416|0;p=c[l>>2]|0;if(!((p|0)==0|(n|0)==(p|0))){AN(b,16,1625224,(b=i,i=i+1|0,i=i+7&-8,c[b>>2]=0,b)|0);i=b;q=-1163346256;i=f;return q|0}c[l>>2]=n;n=j+(g+12)|0;l=d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0;n=j+(g+20)|0;c[(c[k>>2]|0)+72>>2]=d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24;n=a[j+(g+28)|0]|0;b=n&255;p=b&15;if(p>>>0>b>>>4>>>0){q=-1;i=f;return q|0}if(p>>>0<6|(n&255)>223){q=-1;i=f;return q|0}if((a[j+(g+29)|0]|0)!=1){q=-1;i=f;return q|0}c[(c[k>>2]|0)+8>>2]=1;c[(c[k>>2]|0)+48>>2]=86021;if((l|0)<=0){q=1;i=f;return q|0}c[(c[k>>2]|0)+412>>2]=l;mL(h,64,1,l);q=1;i=f;return q|0}else{l=h+8|0;h=(c[l>>2]|0)+96|0;k=r;g=r+4|0;j=r+8|0;n=(c[g>>2]|0)+(c[k>>2]|0)+(c[j>>2]|0)|0;p=((n|0)/255|0)+64+n|0;n=A2(0,p)|0;c[h>>2]=n;if((n|0)==0){t=0}else{NM(n|0,0,p|0);a[n]=2;p=(pu(n+1|0,c[k>>2]|0)|0)+1|0;b=(pu(n+p|0,c[g>>2]|0)|0)+p|0;p=n+b|0;o=c[m>>2]|0;s=c[k>>2]|0;NN(p|0,o|0,s)|0;s=(c[k>>2]|0)+b|0;A6(m);m=n+s|0;b=r+16|0;k=c[b>>2]|0;o=c[g>>2]|0;NN(m|0,k|0,o)|0;o=(c[g>>2]|0)+s|0;A6(b);b=n+o|0;n=r+20|0;s=c[n>>2]|0;g=c[j>>2]|0;NN(b|0,s|0,g)|0;g=(c[j>>2]|0)+o|0;A6(n);c[h>>2]=A2(c[h>>2]|0,g+16|0)|0;t=g}c[(c[l>>2]|0)+100>>2]=t;t=py(c[l>>2]|0,r+24|0)|0;if((t|0)==0){q=1;i=f;return q|0}A6((c[l>>2]|0)+96|0);c[(c[l>>2]|0)+100>>2]=0;q=t;i=f;return q|0}return 0}function lT(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0;e=c[c[b+12>>2]>>2]|0;f=e+(d*384|0)|0;g=c[e+(d*384|0)+380>>2]|0;h=e+(d*384|0)+48|0;i=c[h>>2]|0;j=c[h+4>>2]|0;do{if((i|0)==0&(j|0)==0|(i|0)==0&(j|0)==(-2147483648|0)){if((c[e+(d*384|0)+80>>2]&4|0)!=0){break}k=c[f>>2]|0;l=c[e+(d*384|0)+12>>2]|0;m=g+24|0;pA(m);n=c[e+(d*384|0)+96>>2]|0;o=pz(m,k+l|0,1)|0;if((o|0)<0){p=e+(d*384|0)+20|0;c[p>>2]=c[p>>2]|2;return 0}p=k+((c[e+(d*384|0)+16>>2]|0)+l)|0;l=e+(d*384|0)+92|0;k=c[l>>2]|0;L16265:do{if((n|0)<(k|0)){q=n;r=o;s=p;t=p;u=k;while(1){v=e+(d*384|0)+100+q|0;if((a[v]|0)==-1){w=s;x=r;y=255;z=u}else{A=pz(m,s,1)|0;if((A|0)<0){break}B=a[v]|0;w=t+(B&255)|0;x=A+r|0;y=B&255;z=c[l>>2]|0}B=q+1|0;if((B|0)<(z|0)){q=B;r=x;s=w;t=t+y|0;u=z}else{C=x;D=12094;break L16265}}u=e+(d*384|0)+32|0;t=c[u>>2]|0;E=t;F=c[u+4>>2]|0;H=t}else{C=o;D=12094}}while(0);if((D|0)==12094){o=e+(d*384|0)+32|0;E=C;F=c[o+4>>2]|0;H=c[o>>2]|0}o=NV(H,F,E,(E|0)<0|0?-1:0)|0;l=G;k=e+(d*384|0)+56|0;c[k>>2]=o;c[k+4>>2]=l;c[h>>2]=o;c[h+4>>2]=l;k=b+28|0;p=(c[(c[k>>2]|0)+(d<<2)>>2]|0)+48|0;do{if((c[p>>2]|0)==0&(c[p+4>>2]|0)==(-2147483648|0)){n=0;t=(l|0)>(n|0)|(l|0)==(n|0)&o>>>0>0>>>0;c[p>>2]=t?o:0;c[p+4>>2]=t?l:0;t=c[(c[k>>2]|0)+(d<<2)>>2]|0;n=t+56|0;u=c[n>>2]|0;s=c[n+4>>2]|0;if((u|0)==0&(s|0)==0){break}r=t+48|0;t=NV(u,s,c[r>>2]|0,c[r+4>>2]|0)|0;c[n>>2]=t;c[n+4>>2]=G}}while(0);k=g+320|0;c[k>>2]=0;c[k+4>>2]=-2147483648;pA(m)}}while(0);do{if((c[e+(d*384|0)+16>>2]|0)!=0){b=pz(g+24|0,(c[f>>2]|0)+(c[e+(d*384|0)+12>>2]|0)|0,1)|0;if((b|0)>=0){c[e+(d*384|0)+24>>2]=b;break}b=e+(d*384|0)+20|0;c[b>>2]=c[b>>2]|2;return 0}}while(0);if((c[e+(d*384|0)+80>>2]&4|0)==0){return 0}f=c[h>>2]|0;b=c[h+4>>2]|0;if(!((f|0)==0&(b|0)==(-2147483648|0))){h=g+320|0;c[h>>2]=f;c[h+4>>2]=b;c[g+328>>2]=0}if((c[e+(d*384|0)+96>>2]|0)==(c[e+(d*384|0)+92>>2]|0)){b=e+(d*384|0)+32|0;h=g+320|0;f=NV(c[b>>2]|0,c[b+4>>2]|0,c[h>>2]|0,c[h+4>>2]|0)|0;h=g+328|0;b=NV(f,G,c[h>>2]|0,0)|0;f=b;c[e+(d*384|0)+24>>2]=f;I=f;J=h}else{I=c[e+(d*384|0)+24>>2]|0;J=g+328|0}c[J>>2]=(c[J>>2]|0)+I;return 0}function lU(a,b){a=a|0;b=b|0;var d=0;d=c[(c[c[a+12>>2]>>2]|0)+(b*384|0)+380>>2]|0;if((d|0)==0){return}A6(d+12|0);A6(d+16|0);A6(d+20|0);return}function lV(){var a=0,b=0;a=A1(1288)|0;b=a;if((a|0)==0){return b|0}NM(a|0,0,1288);c[a>>2]=346560;Br(a);return b|0}function lW(){return 346560}function lX(a){a=a|0;var b=0,d=0;b=c[a+4>>2]|0;if((b|0)!=0){d=c[b>>2]|0;return d|0}b=c[a+8>>2]|0;if((b|0)==0){d=1621296;return d|0}d=c[b>>2]|0;return d|0}function lY(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;do{if((b|0)==0){d=c[a+12>>2]|0;if((d|0)==0){break}e=c[a+4>>2]|0;do{if((e|0)!=0){if((c[e+20>>2]|0)==0){break}else{f=d}return f|0}}while(0);e=c[a+8>>2]|0;if((e|0)==0){break}if((c[e+36>>2]|0)==0){break}else{f=d}return f|0}}while(0);e=c[a+16>>2]|0;do{if((e|0)!=0){if((c[e>>2]|0)==0){break}a=e;if((a|0)==(b|0)){break}else{f=a}return f|0}}while(0);f=0;return f|0}function lZ(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0;if((a|0)==0){b=236136;return b|0}else{d=0}while(1){e=jc(d)|0;if((e|0)==0){f=0;g=12152;break}if((c[e+20>>2]|0)==(a|0)){h=e;g=12154;break}else{d=e}}L16342:do{if((g|0)==12152){while(1){g=0;d=jd(f)|0;if((d|0)==0){h=0;g=12154;break L16342}if((c[d+36>>2]|0)==(a|0)){i=d;break}else{f=d;g=12152}}}}while(0);L16346:do{if((g|0)==12154){while(1){g=0;f=jc(h)|0;if((f|0)==0){i=0;break L16346}a=c[f+20>>2]|0;if((a|0)==0){h=f;g=12154}else{b=a;break}}return b|0}}while(0);while(1){h=jd(i)|0;if((h|0)==0){b=0;g=12159;break}a=c[h+36>>2]|0;if((a|0)==0){i=h}else{b=a;g=12161;break}}if((g|0)==12161){return b|0}else if((g|0)==12159){return b|0}return 0}function l_(a){a=a|0;return((c[a+4>>2]|0)==0?3:4)|0}function l$(a,b){a=a|0;b=b|0;ij(c[a+16>>2]|0,c[b+24>>2]|0,c[b+28>>2]|0);return 0}function l0(a,b){a=a|0;b=b|0;var c=0;c=b|0;if((iE(a,c,16)|0)>=16){return}NM(c|0,0,16);return}function l1(a,d){a=a|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;e=i;i=i+256|0;f=e|0;g=f|0;h=f;j=f;k=f;l=d+52|0;m=c[l>>2]|0;if((m|0)==0|m>>>0>65535){n=-1;i=e;return n|0}m=pt(d,0)|0;if((m|0)==0){o=c[d+424>>2]|0}else{o=m}m=d+416|0;do{if((c[m>>2]|0)>2){p=d+448|0;if((c[p>>2]|0)==0&(c[p+4>>2]|0)==0){q=12176;break}r=d+412|0;q=12178}else{q=12176}}while(0);do{if((q|0)==12176){p=d+412|0;if((c[p>>2]|0)>48e3){r=p;q=12178;break}s=d+48|0;if((pc(c[s>>2]|0)|0)>16){r=p;q=12178;break}iv(a,c[l>>2]|0);t=0;u=p;v=s}}while(0);if((q|0)==12178){iv(a,65534);t=1;u=r;v=d+48|0}iv(a,c[m>>2]|0);ir(a,c[u>>2]|0);r=c[v>>2]|0;L16381:do{switch(r|0){case 86047:case 86069:case 86016:case 86017:case 86046:{w=0;break};default:{s=pc(r)|0;if((s|0)!=0){w=s;break L16381}s=c[d+792>>2]|0;w=(s|0)==0?16:s}}}while(0);r=c[d+792>>2]|0;if(!((w|0)==(r|0)|(r|0)==0)){AN(d,24,1686424,(s=i,i=i+16|0,c[s>>2]=r,c[s+8>>2]=w,s)|0);i=s}s=c[v>>2]|0;L16388:do{switch(s|0){case 86019:{x=3840;q=12190;break};case 86069:{x=24;q=12190;break};case 86018:{x=(c[m>>2]|0)*768|0;q=12190;break};case 86016:case 86017:{z=o;break};default:{r=c[d+432>>2]|0;if((r|0)!=0){x=r;q=12190;break L16388}r=ac(c[m>>2]|0,w)|0;x=N0(r,(r|0)<0|0?-1:0,AU(8,0,w,(w|0)<0|0?-1:0)|0,G)|0;q=12190}}}while(0);if((q|0)==12190){z=x}if((s|0)==65541|(s|0)==65548|(s|0)==65544|(s|0)==65557|(s|0)==65559|(s|0)==65536){A=ac(c[u>>2]|0,z)|0}else if((s|0)==86069){A=800}else{A=(c[d+72>>2]|0)/8|0}ir(a,A);iv(a,z);iv(a,w);z=c[v>>2]|0;do{if((z|0)==86016){b[j>>1]=2;v=k+2|0;y=c[d+72>>2]|0;b[v>>1]=y&65535;b[v+2>>1]=y>>16;b[k+6>>1]=(c[m>>2]|0)==2?1:8;v=f+8|0;b[v>>1]=0;b[v+2>>1]=1;v=f+12|0;A=v;b[v>>1]=16;v=A+2|0;y=0;b[v>>1]=y&65535;b[v+2>>1]=y>>16;v=A+6|0;y=0;b[v>>1]=y&65535;b[v+2>>1]=y>>16;B=k;C=40;D=A+10|0}else if((z|0)==86017){b[j>>1]=1;A=k+2|0;y=2;b[A>>1]=y&65535;b[A+2>>1]=y>>16;b[k+6>>1]=1152;A=f+8|0;b[A>>1]=1;b[A+2>>1]=1393;B=k;C=30;D=f+12|0}else if((z|0)==86069){c[g>>2]=-1697775614;c[f+4>>2]=-1365051598;A=f+8|0;b[A>>1]=-21282;B=k;C=38;D=A+2|0}else if((z|0)==86046|(z|0)==69633){b[j>>1]=o&65535;B=k;C=20;D=h+2|0}else{A=c[d+100>>2]|0;if((A|0)==0){B=k;C=18;D=k;break}v=c[d+96>>2]|0;B=v;C=A+18|0;D=v+A|0}}while(0);if(t){t=D;k=B;iv(a,t+22-k|0);iv(a,w);ir(a,c[d+448>>2]|0);ir(a,c[l>>2]|0);ir(a,1048576);ir(a,-1442840448);ir(a,1905997824);E=C+22|0;F=t-k|0}else{k=D-B|0;iv(a,k);E=C;F=k}ij(a,B,F);if((E&1|0)==0){n=E;i=e;return n|0}ih(a,0);n=E+1|0;i=e;return n|0}function l2(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0;d=b+100|0;ir(a,(c[d>>2]|0)+40|0);f=b+120|0;ir(a,c[f>>2]|0);g=b+52|0;h=b+124|0;i=c[h>>2]|0;ir(a,(c[g>>2]|0)!=0?i:-i|0);iv(a,1);i=b+792|0;j=c[i>>2]|0;iv(a,(j|0)==0?24:j);ir(a,c[g>>2]|0);g=ac(c[h>>2]|0,c[f>>2]|0)|0;f=c[i>>2]|0;ir(a,((ac(g,(f|0)==0?24:f)|0)+7|0)/8|0);ir(a,0);ir(a,0);ir(a,0);ir(a,0);ij(a,c[b+96>>2]|0,c[d>>2]|0);if((e|0)!=0){return}if((c[d>>2]&1|0)==0){return}ih(a,0);return}function l3(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;g=i;i=i+32|0;h=g|0;j=g+16|0;k=iG(b)|0;c[e+8>>2]=1;l=e+416|0;c[l>>2]=iG(b)|0;m=e+412|0;c[m>>2]=iI(b)|0;n=e+72|0;c[n>>2]=(iI(b)|0)<<3;c[e+432>>2]=iG(b)|0;if((f|0)==14){c[e+792>>2]=8;o=8}else{p=iG(b)|0;c[e+792>>2]=p;o=p}p=(k|0)==65534;q=e+52|0;if(p){c[q>>2]=0}else{c[q>>2]=k;r=mA(251144,k)|0;if((r|0)==65557){s=mB(o,1,0,0)|0;t=12225}else if((r|0)==0){u=0}else if((r|0)==65536){s=mB(o,0,0,-2)|0;t=12225}else{s=r;t=12225}if((t|0)==12225){u=(s|0)==69633&(o|0)==8?65553:s}c[e+48>>2]=u}do{if((f|0)>17){u=iG(b)|0;s=f-18|0;o=(s|0)>(u|0)?u:s;if((o|0)<22|p^1){v=o;w=s}else{s=h;u=j|0;r=h;k=iG(b)|0;if((k|0)!=0){c[e+792>>2]=k}k=e+448|0;c[k>>2]=iI(b)|0;c[k+4>>2]=0;if((iE(b,s,16)|0)<16){NM(s|0,0,16)}k=h+4|0;a[u]=0;a[j+1|0]=0;a[j+2|0]=16;a[j+3|0]=0;a[j+4|0]=-128;a[j+5|0]=0;a[j+6|0]=0;a[j+7|0]=-86;a[j+8|0]=0;a[j+9|0]=56;a[j+10|0]=-101;a[j+11|0]=113;do{if((NO(k|0,u|0,12)|0)==0){x=c[h>>2]|0;c[q>>2]=x;y=c[e+792>>2]|0;z=mA(251144,x)|0;if((z|0)==65557){A=mB(y,1,0,0)|0;t=12237}else if((z|0)==0){B=0}else if((z|0)==65536){A=mB(y,0,0,-2)|0;t=12237}else{A=z;t=12237}if((t|0)==12237){B=(A|0)==69633&(y|0)==8?65553:A}c[e+48>>2]=B}else{y=0;while(1){if((y|0)==4){t=12242;break}if((NO(251604+(y*20|0)|0,s|0,16)|0)==0){t=12241;break}else{y=y+1|0}}if((t|0)==12241){c[e+48>>2]=c[251600+(y*20|0)>>2];break}else if((t|0)==12242){c[e+48>>2]=0;z=d[r+1|0]|0;x=d[r+2|0]|0;C=d[r+3|0]|0;D=d[k]|0;E=d[r+5|0]|0;F=d[r+6|0]|0;G=d[r+7|0]|0;H=d[h+8|0]|0;I=d[r+9|0]|0;J=d[r+10|0]|0;K=d[r+11|0]|0;L=d[h+12|0]|0;M=d[r+13|0]|0;N=d[r+14|0]|0;O=d[r+15|0]|0;AN(e,24,1671600,(P=i,i=i+128|0,c[P>>2]=d[s]|0,c[P+8>>2]=z,c[P+16>>2]=x,c[P+24>>2]=C,c[P+32>>2]=D,c[P+40>>2]=E,c[P+48>>2]=F,c[P+56>>2]=G,c[P+64>>2]=H,c[P+72>>2]=I,c[P+80>>2]=J,c[P+88>>2]=K,c[P+96>>2]=L,c[P+104>>2]=M,c[P+112>>2]=N,c[P+120>>2]=O,P)|0);i=P;break}}}while(0);v=o-22|0;w=f-40|0}s=e+100|0;c[s>>2]=v;do{if((v|0)>0){r=e+96|0;A4(c[r>>2]|0);k=A7((c[s>>2]|0)+16|0)|0;c[r>>2]=k;if((k|0)==0){Q=-12;i=g;return Q|0}else{r=c[s>>2]|0;iE(b,k,r)|0;R=w-v|0;break}}else{R=w}}while(0);if((R|0)<=0){break}s=R;o=(R|0)<0|0?-1:0;io(b,s,o)|0;}}while(0);b=c[e+48>>2]|0;if((b|0)==86066){c[l>>2]=0;c[m>>2]=0;Q=0;i=g;return Q|0}else if((b|0)==69643){b=c[m>>2]|0;if((b|0)==0){Q=0;i=g;return Q|0}c[e+792>>2]=(c[n>>2]|0)/(b|0)|0;Q=0;i=g;return Q|0}else{Q=0;i=g;return Q|0}return 0}function l4(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;e=i;i=i+8|0;f=e|0;g=c[a+16>>2]|0;h=il(g,0,0,1)|0;j=NU(h,G,b,d)|0;d=G;b=il(g,0,0,1)|0;h=G;k=-1;if(!((h|0)>(k|0)|(h|0)==(k|0)&b>>>0>-1>>>0)){l=0;i=e;return l|0}k=NU(j,d,-8,-1)|0;m=G;n=f|0;o=g+52|0;p=f;f=a+1140|0;q=a;a=h;h=b;L16486:while(1){if((a|0)>(m|0)|(a|0)==(m|0)&h>>>0>k>>>0){l=0;r=12285;break}NM(n|0,0,5);s=iI(g)|0;t=iI(g)|0;b=t;u=0;if((iq(g)|0)!=0){r=12263;break}if((u|0)>(d|0)|(u|0)==(d|0)&b>>>0>j>>>0){r=12267}else{v=NV(j,d,b,u)|0;w=G;if((w|0)<(a|0)|(w|0)==(a|0)&v>>>0<h>>>0|(t|0)==-1){r=12267}else{x=u;y=b;z=s}}if((r|0)==12267){r=0;b=-9;u=-1;il(g,b,u,1)|0;u=iI(g)|0;b=iI(g)|0;v=b;w=0;if((w|0)>(d|0)|(w|0)==(d|0)&v>>>0>j>>>0){r=12269;break}A=NV(j,d,v,w)|0;B=G;if((B|0)<(a|0)|(B|0)==(a|0)&A>>>0<h>>>0|(b|0)==-1){r=12269;break}else{x=w;y=v;z=u}}u=NU(y&1,x&0,y,x)|0;v=G;do{if((z|0)==0){if((u|0)==0&(v|0)==0){if((c[o>>2]|0)==0){break}else{r=12274;break L16486}}else{io(g,u,v)|0;break}}else{w=NU(u,v,1,0)|0;b=A7(w)|0;if((b|0)==0){r=12276;break L16486}c[p>>2]=z;w=iE(g,b,u)|0;if(!((w|0)==(u|0)&(((w|0)<0|0?-1:0)|0)==(v|0))){AN(q,24,1673472,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C}zK(f,n,b,8)|0}}while(0);v=il(g,0,0,1)|0;u=G;b=-1;if((u|0)>(b|0)|(u|0)==(b|0)&v>>>0>-1>>>0){a=u;h=v}else{l=0;r=12289;break}}if((r|0)==12263){if((t|s|0)==0){l=-541478725;i=e;return l|0}AN(q,24,1680848,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;l=-1094995529;i=e;return l|0}else if((r|0)==12274){AN(q,24,1677072,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;l=-541478725;i=e;return l|0}else if((r|0)==12269){AN(q,24,1679128,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;l=-1094995529;i=e;return l|0}else if((r|0)==12285){i=e;return l|0}else if((r|0)==12289){i=e;return l|0}else if((r|0)==12276){AN(q,16,1675264,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;l=-12;i=e;return l|0}return 0}function l5(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;h=i;i=i+24|0;j=h|0;k=h+8|0;l=h+16|0;m=jh(1692904,0,0)|0;c[k>>2]=0;do{if((m|0)==0){n=-38;o=0}else{p=lV()|0;if((p|0)==0){n=-12;o=0;break}c[p+8>>2]=m;if((mK(p,0)|0)==0){n=-12;o=p;break}q=b+1160|0;r=p+1160|0;s=c[q+4>>2]|0;c[r>>2]=c[q>>2];c[r+4>>2]=s;c[p+1080>>2]=c[b+1080>>2];s=p+28|0;r=d+80|0;q=(c[c[s>>2]>>2]|0)+80|0;t=c[r+4>>2]|0;c[q>>2]=c[r>>2];c[q+4>>2]=t;t=p+1084|0;c[t>>2]=c[t>>2]|c[b+1084>>2]&32768;t=c[d+4>>2]|0;if((t|0)<96){q=ny(b,c[d+8>>2]|0,g)|0;c[(c[c[s>>2]>>2]|0)+4>>2]=q}else{c[(c[c[s>>2]>>2]|0)+4>>2]=t}if((Bl(b,1730728,1,j)|0)>-1){t=c[j>>2]|0;zK(k,1730728,t,8)|0}t=b+1144|0;q=c[t+4>>2]|0;r=p+1144|0;c[r>>2]=c[t>>2];c[r+4>>2]=q;q=c[(c[c[s>>2]>>2]|0)+8>>2]|0;s=c[d+8>>2]|0;ow(q,s)|0;s=(e|0)!=0;q=p+16|0;if(s){iR(q,e)|0}else{i_(q,f)|0}r=k7(p,k)|0;zL(k);if((r|0)==0){c[a>>2]=p;u=0;i=h;return u|0}t=c[q>>2]|0;if(s){iV(t)|0}else{i$(t,l)|0;A4(c[l>>2]|0)}mi(p);u=r;i=h;return u|0}}while(0);A4(o);if((e|0)==0){u=n;i=h;return u|0}h7(e)|0;u=n;i=h;return u|0}function l6(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;return-38|0}function l7(a,b,c,d,e,f,g,h,i){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;return}function l8(){return 3607653}function l9(){return 1691632}function ma(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;d=i;e=a+96|0;f=c[e>>2]|0;g=c[e+4>>2]|0;h=-1;if(!((g|0)>(h|0)|(g|0)==(h|0)&f>>>0>-1>>>0)){j=b;i=d;return j|0}h=il(a,0,0,1)|0;k=NV(f,g,h,G)|0;h=G;g=b;f=(b|0)<0|0?-1:0;if((h|0)<(f|0)|(h|0)==(f|0)&k>>>0<g>>>0){l=ip(a)|0;m=G;n=c[e>>2]|0;o=c[e+4>>2]|0;if((n|0)==0&(o|0)==0|((o|0)<(m|0)|(o|0)==(m|0)&n>>>0<l>>>0)){p=NV(l,m,(l|0)==0&(m|0)==0&1,0)|0;m=G;c[e>>2]=p;c[e+4>>2]=m;q=m;r=p}else{q=o;r=n}n=il(a,0,0,1)|0;a=NV(r,q,n,G)|0;n=G;q=0;r=(n|0)>(q|0)|(n|0)==(q|0)&a>>>0>0>>>0;s=r?n:0;t=r?a:0}else{s=h;t=k}k=c[e+4>>2]|0;h=-1;if(!((k|0)>(h|0)|(k|0)==(h|0)&(c[e>>2]|0)>>>0>-1>>>0)){j=b;i=d;return j|0}e=NU(t,s,1,0)|0;h=G;if(!((h|0)<(f|0)|(h|0)==(f|0)&e>>>0<g>>>0)){j=b;i=d;return j|0}AN(0,(t|0)!=0|(s|0)!=0?16:48,1698568,(s=i,i=i+24|0,c[s>>2]=b,c[s+8>>2]=e,c[s+16>>2]=h,s)|0);i=s;j=e;i=d;return j|0}function mb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;nB(b);c[b+24>>2]=0;c[b+28>>2]=0;e=il(a,0,0,1)|0;f=b+64|0;c[f>>2]=e;c[f+4>>2]=G;return mc(a,b,d)|0}function mc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;e=b+64|0;f=c[e>>2]|0;g=c[e+4>>2]|0;h=b+28|0;i=c[h>>2]|0;j=a+96|0;k=b+24|0;l=d;d=i;while(1){do{if((l|0)>5e6){m=ma(a,l)|0;n=c[j+4>>2]|0;o=0;if(!((n|0)<(o|0)|(n|0)==(o|0)&(c[j>>2]|0)>>>0<0>>>0)){p=m;break}p=(m|0)>5e7?5e7:m}else{p=l}}while(0);m=nF(b,p)|0;if((m|0)<0){q=m;r=12341;break}s=iE(a,(c[k>>2]|0)+d|0,p)|0;if((s|0)!=(p|0)){r=12338;break}m=l-p|0;if((m|0)<=0){t=p;break}l=m;d=c[h>>2]|0}if((r|0)==12338){nE(b,((s|0)>0?s:0)+d|0);q=s;r=12341}do{if((r|0)==12341){if((l|0)<=0){t=q;break}s=b+36|0;c[s>>2]=c[s>>2]|2;t=q}}while(0);c[e>>2]=f;c[e+4>>2]=g;g=c[h>>2]|0;if((g|0)!=0){u=g;v=(u|0)>(i|0);w=u-i|0;x=v?w:t;return x|0}nJ(b);u=c[h>>2]|0;v=(u|0)>(i|0);w=u-i|0;x=v?w:t;return x|0}function md(a){a=a|0;var b=0,c=0;b=i;i=i+1024|0;if((a|0)==0){c=0;i=b;return c|0}c=(me(b|0,1024,a,1)|0)>>>31^1;i=b;return c|0}function me(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;g=i;i=i+24|0;h=b;j=d-1|0;d=g|0;k=0;l=b;b=e;L16599:while(1){m=l;e=b;while(1){n=m-h|0;if((n|0)>=(j|0)){o=e;p=12361;break}q=e+1|0;r=a[e]|0;if((r<<24>>24|0)==0){p=12370;break L16599}else if((r<<24>>24|0)==37){s=0;t=q;while(1){if((y6(a[t]|0)|0)==0){u=t+1|0;v=a[t]|0;w=v<<24>>24;if((y6(w)|0)==0){break}else{s=0;t=u;continue}}else{s=(s*10|0)-48+(a[t]|0)|0;t=t+1|0;continue}}if((w|0)==100){x=u;y=s;break}else if((w|0)==37){z=u;A=v}else{break L16599}}else{z=q;A=r}a[m]=A;m=m+1|0;e=z}if((p|0)==12361){while(1){p=0;e=o+1|0;t=a[o]|0;if((t<<24>>24|0)==0){p=12370;break L16599}else if((t<<24>>24|0)==37){B=0;C=e}else{o=e;p=12361;continue}while(1){if((y6(a[C]|0)|0)==0){D=C+1|0;E=a[C]|0;if((y6(E)|0)==0){break}else{B=0;C=D;continue}}else{B=(B*10|0)-48+(a[C]|0)|0;C=C+1|0;continue}}if((E|0)==100){x=D;y=B;break}else if((E|0)==37){o=D;p=12361}else{break L16599}}}if((k|0)!=0){break}bc(d|0,20,1628992,(r=i,i=i+16|0,c[r>>2]=y,c[r+8>>2]=f,r)|0)|0;i=r;r=NL(d|0)|0;if((n+r|0)>(j|0)){break}NN(m|0,d|0,r)|0;k=1;l=m+r|0;b=x}do{if((p|0)==12370){if((k|0)==0){break}else{F=0}a[m]=0;i=g;return F|0}}while(0);F=-1;a[m]=0;i=g;return F|0}function mf(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;e=i;i=i+16|0;f=e|0;g=f;h=a;c[g>>2]=c[h>>2];c[g+4>>2]=c[h+4>>2];c[g+8>>2]=c[h+8>>2];h=f+4|0;g=c[h>>2]|0;if((g|0)==0){c[h>>2]=3014568;j=3014568}else{j=g}g=f+8|0;do{if((c[g>>2]|0)>10){if((jk(j,1676384)|0)==0){k=0;break}a=jl(c[h>>2]|0)|0;l=c[g>>2]|0;if((l|0)<=(a+16|0)){k=1;break}c[h>>2]=(c[h>>2]|0)+a;c[g>>2]=l-a;k=0}else{k=0}}while(0);g=(b|0)==0|0;b=f|0;h=(k|0)!=0?24:1;j=0;a=0;L16638:while(1){l=a;m=a;while(1){n=l;do{n=jc(n)|0;if((n|0)==0){break L16638}}while((g|0)==(c[n+8>>2]&1^1|0));o=c[n+36>>2]|0;do{if((o|0)==0){p=c[n+12>>2]|0;if((p|0)==0){q=0;break}q=(jg(c[b>>2]|0,p)|0)==0?0:50}else{p=c1[o&255](f)|0;r=c[n+12>>2]|0;if((r|0)==0){q=p;break}if((jg(c[b>>2]|0,r)|0)==0){q=p;break}q=(p|0)>(h|0)?p:h}}while(0);if((q|0)>(j|0)){j=q;a=n;continue L16638}l=n;m=(q|0)==(j|0)?0:m}}if((k|0)==0){s=j;c[d>>2]=s;i=e;return m|0}s=(j|0)<24?j:24;c[d>>2]=s;i=e;return m|0}function mg(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;h=i;i=i+40|0;j=h|0;k=h+8|0;l=h+24|0;m=h+32|0;c[k>>2]=(d|0)!=0?d:3119048;d=k+4|0;c[d>>2]=0;n=k+8|0;c[n>>2]=-f;c[l>>2]=0;do{if((g|0)==0|g>>>0>1048576){o=1048576}else{if(g>>>0>=2048){o=g;break}AN(e,16,1643520,(p=i,i=i+16|0,c[p>>2]=g,c[p+8>>2]=2048,p)|0);i=p;q=-22;i=h;return q|0}}while(0);if(o>>>0<=f>>>0){q=-22;i=h;return q|0}do{if((c[b>>2]|0)==0){if((c[a>>2]|0)==0){break}g=(Bl(a,1631472,1,m)|0)>-1;r=c[m>>2]|0;if(!(g&(r|0)!=0)){break}if((y4(r,1625048)|0)==0){c[b>>2]=jj(1618920)|0}A6(m)}}while(0);m=(c[b>>2]|0)==0;L16676:do{if(o>>>0<2048){s=m;t=12425}else{r=2048;g=0;u=m;L16678:while(1){if(!u){break L16676}v=r>>>0<o>>>0?25:0;do{if(r>>>0<f>>>0){w=g}else{x=A2(c[l>>2]|0,r+32|0)|0;if((x|0)==0){t=12414;break L16678}c[l>>2]=x;y=iE(a,x+g|0,r-g|0)|0;if((y|0)<0){if((y|0)==-541478725){z=0;A=0}else{t=12417;break L16678}}else{z=y;A=v}x=z+g|0;c[n>>2]=x;B=c[l>>2]|0;c[d>>2]=B+f;NM(B+(x+f)|0,0,32);B=mf(k,1,j)|0;C=c[j>>2]|0;if((C|0)<=(A|0)){c[b>>2]=0;w=x;break}c[b>>2]=B;if((B|0)==0){w=x;break}D=c[B>>2]|0;if((C|0)<26){AN(e,24,1614592,(p=i,i=i+16|0,c[p>>2]=D,c[p+8>>2]=C,p)|0);i=p;w=x;break}else{AN(e,48,1743664,(p=i,i=i+24|0,c[p>>2]=D,c[p+8>>2]=r,c[p+16>>2]=C,p)|0);i=p;w=x;break}}}while(0);v=r<<1;x=r+1|0;C=o>>>0>x>>>0?o:x;x=v>>>0>C>>>0?C:v;v=(c[b>>2]|0)==0;if(x>>>0>o>>>0){s=v;t=12425;break L16676}else{r=x;g=w;u=v}}if((t|0)==12414){A4(c[l>>2]|0);q=-12;i=h;return q|0}else if((t|0)==12417){A4(c[l>>2]|0);q=y;i=h;return q|0}}}while(0);do{if((t|0)==12425){if(!s){break}A4(c[l>>2]|0);q=-1094995529;i=h;return q|0}}while(0);q=iS(a,l,c[n>>2]|0)|0;i=h;return q|0}function mh(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0;f=i;i=i+1144|0;g=f|0;h=f+80|0;j=f+1104|0;k=f+1112|0;l=f+1128|0;m=f+1136|0;n=c[a>>2]|0;c[l>>2]=0;c[m>>2]=0;do{if((n|0)==0){o=lV()|0;if((o|0)==0){p=-12}else{q=o;break}i=f;return p|0}else{q=n}}while(0);if((c[q>>2]|0)==0){AN(0,16,1737192,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;p=-22;i=f;return p|0}if((d|0)!=0){c[q+4>>2]=d}d=(e|0)!=0;if(d){zM(l,c[e>>2]|0,0)}n=q;o=Bv(n,l)|0;L16721:do{if((o|0)<0){s=o;t=q+16|0}else{c[k>>2]=b;c[k+4>>2]=0;c[k+8>>2]=0;u=q+16|0;v=c[u>>2]|0;L16724:do{if((v|0)==0){w=q+4|0;x=c[w>>2]|0;do{if((x|0)==0){y=mf(k,0,j)|0;if((c[j>>2]|0)>25){c[w>>2]=y;if((y|0)==0){break}else{z=w;break L16724}}else{c[w>>2]=0;break}}else{if((c[x+8>>2]&1|0)!=0){z=w;break L16724}}}while(0);x=iU(u,b,c[q+1196>>2]|1,q+1160|0,l)|0;if((x|0)<0){s=x;t=u;break L16721}if((c[w>>2]|0)!=0){z=w;break}A=mg(c[u>>2]|0,w,b,n,0,c[q+1088>>2]|0)|0;B=12458}else{x=q+1084|0;c[x>>2]=c[x>>2]|128;x=q+4|0;y=c[x>>2]|0;if((y|0)==0){A=mg(v,x,b,n,0,c[q+1088>>2]|0)|0;B=12458;break}if((c[y+8>>2]&1|0)==0){z=x;break}AN(n,24,1717968,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;z=x}}while(0);if((B|0)==12458){if((A|0)<0){s=A;t=u;break}z=q+4|0}v=c[u>>2]|0;x=c[q+1204>>2]|0;y=0;io(v,x,y)|0;if((c[(c[z>>2]|0)+8>>2]&2|0)!=0){if((b|0)==0){s=-22;t=u;break}if((me(h|0,1024,b,1)|0)<0){s=-22;t=u;break}}y=q+1056|0;c[y>>2]=0;c[y+4>>2]=-2147483648;y=q+1064|0;c[y>>2]=0;c[y+4>>2]=-2147483648;y=q+32|0;x=(b|0)!=0?b:3119048;y_(y,x,1024)|0;x=c[(c[z>>2]|0)+32>>2]|0;do{if((x|0)>0){y=A7(x)|0;v=q+12|0;c[v>>2]=y;if((y|0)==0){s=-12;t=u;break L16721}C=c[(c[z>>2]|0)+20>>2]|0;if((C|0)==0){break}c[y>>2]=C;Br(c[v>>2]|0);C=Bv(c[v>>2]|0,l)|0;if((C|0)<0){s=C;t=u;break L16721}}}while(0);if((c[u>>2]|0)!=0){jm(q,1676384,m)}x=q+1084|0;do{if((c[x>>2]&131072|0)==0){C=c[(c[z>>2]|0)+40>>2]|0;if((C|0)==0){break}v=c1[C&255](q)|0;if((v|0)<0){s=v;t=u;break L16721}}}while(0);L16760:do{if((c[m>>2]|0)!=0){v=c[c[z>>2]>>2]|0;do{if((a8(v|0,1732584)|0)!=0){if((a8(v|0,1618920)|0)==0){break}if((a8(v|0,1727448)|0)==0){break}AN(n,48,1723256,(r=i,i=i+1|0,i=i+7&-8,c[r>>2]=0,r)|0);i=r;break L16760}}while(0);v=jp(q,m)|0;if((v|0)<0){s=v;t=u;break L16721}}}while(0);jn(m);v=g|0;w=q+24|0;if((c[w>>2]|0)!=0){C=q+28|0;y=q+1240|0;D=q+1244|0;E=0;do{F=c[(c[C>>2]|0)+(E<<2)>>2]|0;do{if((c[F+72>>2]&1024|0)!=0){if((c[F+76>>2]|0)>=48){break}H=c[F+104>>2]|0;I=F+108|0;NN(v|0,I|0,76)|0;I=zi(H)|0;if((I|0)==0){s=-12;t=u;break L16721}H=A7(88)|0;J=H;if((H|0)==0){break}if((c[y>>2]|0)==0){c[y>>2]=J}else{c[(c[D>>2]|0)+80>>2]=J}c[D>>2]=J;c[H>>2]=I;I=H+4|0;NN(I|0,v|0,76)|0}}while(0);E=E+1|0;}while(E>>>0<(c[w>>2]|0)>>>0)}do{if((c[x>>2]&131072|0)==0){w=c[u>>2]|0;if((w|0)==0){break}E=q+1232|0;if(!((c[E>>2]|0)==0&(c[E+4>>2]|0)==0)){break}v=il(w,0,0,1)|0;c[E>>2]=v;c[E+4>>2]=G}}while(0);c[q+1256>>2]=25e5;if(d){zL(e);c[e>>2]=c[l>>2]}c[a>>2]=q;p=0;i=f;return p|0}}while(0);jn(m);zL(l);l=c[t>>2]|0;do{if((l|0)!=0){if((c[q+1084>>2]&128|0)!=0){break}iV(l)|0}}while(0);mi(q);c[a>>2]=0;p=s;i=f;return p|0}function mi(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;if((a|0)==0){return}b=a;Bu(b);d=c[a+4>>2]|0;do{if((d|0)!=0){if((c[d+20>>2]|0)==0){break}e=c[a+12>>2]|0;if((e|0)==0){break}Bu(e)}}while(0);d=(c[a+24>>2]|0)-1|0;if((d|0)>-1){e=a+28|0;f=d;do{mI(a,c[(c[e>>2]|0)+(f<<2)>>2]|0);f=f-1|0;}while((f|0)>-1)}f=(c[a+1104>>2]|0)-1|0;e=a+1108|0;if((f|0)>-1){d=f;do{zL((c[(c[e>>2]|0)+(d<<2)>>2]|0)+20|0);A6((c[(c[e>>2]|0)+(d<<2)>>2]|0)+12|0);A6((c[e>>2]|0)+(d<<2)|0);d=d-1|0;}while((d|0)>-1)}A6(e);A6(a+12|0);e=a+1132|0;d=c[e>>2]|0;f=d-1|0;c[e>>2]=f;g=a+1136|0;if((d|0)!=0){d=f;do{zL((c[(c[g>>2]|0)+(d<<2)>>2]|0)+32|0);A6((c[g>>2]|0)+(c[e>>2]<<2)|0);f=c[e>>2]|0;d=f-1|0;c[e>>2]=d;}while((f|0)!=0)}A6(g);zL(a+1140|0);A6(a+28|0);A4(b);return}function mj(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0;d=i;i=i+8|0;e=d|0;f=a+1240|0;g=b;h=b+32|0;j=a+28|0;k=a+1256|0;l=b+24|0;m=b+28|0;n=a+4|0;o=a+24|0;p=a+1084|0;q=b+16|0;r=b+8|0;s=a+1188|0;t=a+1244|0;u=e|0;v=e+4|0;w=a+1112|0;x=a+1116|0;y=a+1120|0;z=a;A=b+36|0;L16822:while(1){B=c[f>>2]|0;C=(B|0)!=0;if(C){D=B;NN(g|0,D|0,80)|0;E=c[(c[j>>2]|0)+(c[h>>2]<<2)>>2]|0;if((c[k>>2]|0)<1){mk(a,E,0)}if((c[E+440>>2]|0)<1){F=12526;break}}c[l>>2]=0;c[m>>2]=0;nB(b);E=dg[c[(c[n>>2]|0)+44>>2]&255](a,b)|0;if((E|0)<0){if((E|0)==-11|C^1){H=E;F=12573;break}else{I=0}while(1){if(I>>>0>=(c[o>>2]|0)>>>0){continue L16822}J=c[(c[j>>2]|0)+(I<<2)>>2]|0;if((c[J+228>>2]|0)!=0){mk(a,J,0)}if((c[J+440>>2]|0)>=1){F=12534;break L16822}I=I+1|0}}J=c[p>>2]|0;do{if((J&256|0)!=0){if((c[A>>2]&2|0)==0){break}AN(z,24,1708680,(K=i,i=i+8|0,c[K>>2]=c[h>>2],K)|0);i=K;nJ(b);continue L16822}}while(0);if((J&262144|0)==0){nM(b)|0}L=c[h>>2]|0;if(L>>>0>=(c[o>>2]|0)>>>0){AN(z,16,1706624,(K=i,i=i+8|0,c[K>>2]=L,K)|0);i=K;continue}M=c[(c[j>>2]|0)+(L<<2)>>2]|0;L=c[q>>2]|0;N=c[q+4>>2]|0;O=M+472|0;P=c[O>>2]|0;do{if((P|0)==0){Q=N;R=L}else{S=M+464|0;T=c[S>>2]|0;U=c[S+4>>2]|0;if((T|0)==0&(U|0)==(-2147483648|0)|(L|0)==0&(N|0)==(-2147483648|0)){Q=N;R=L;break}S=(U|0)>(N|0)|(U|0)==(N|0)&T>>>0>L>>>0;if((P|0)==1&S){T=NW(1,0,c[M+188>>2]|0)|0;U=NU(T,G,L,N)|0;Q=G;R=U;break}if((P|0)!=-1|S){Q=N;R=L;break}S=NW(1,0,c[M+188>>2]|0)|0;U=NV(L,N,S,G)|0;Q=G;R=U}}while(0);c[q>>2]=R;c[q+4>>2]=Q;N=c[r>>2]|0;L=c[r+4>>2]|0;P=c[O>>2]|0;do{if((P|0)==0){V=L;W=N}else{J=M+464|0;U=c[J>>2]|0;S=c[J+4>>2]|0;if((U|0)==0&(S|0)==(-2147483648|0)|(N|0)==0&(L|0)==(-2147483648|0)){V=L;W=N;break}J=(S|0)>(L|0)|(S|0)==(L|0)&U>>>0>N>>>0;if((P|0)==1&J){U=NW(1,0,c[M+188>>2]|0)|0;S=NU(U,G,N,L)|0;V=G;W=S;break}if((P|0)!=-1|J){V=L;W=N;break}J=NW(1,0,c[M+188>>2]|0)|0;S=NV(N,L,J,G)|0;V=G;W=S}}while(0);c[r>>2]=W;c[r+4>>2]=V;L=c[M+8>>2]|0;N=c[L+8>>2]|0;do{if((N|0)==0){P=c[w>>2]|0;if((P|0)==0){break}c[L+48>>2]=P}else if((N|0)==3){P=c[y>>2]|0;if((P|0)==0){break}c[L+48>>2]=P}else if((N|0)==1){P=c[x>>2]|0;if((P|0)==0){break}c[L+48>>2]=P}}while(0);if((c[s>>2]|0)!=0){L=Ch()|0;c[u>>2]=1;c[v>>2]=1e6;N=AY(L,G,e,M+40|0)|0;L=G;c[r>>2]=N;c[r+4>>2]=L;c[q>>2]=N;c[q+4>>2]=L}if(!C){if((c[M+440>>2]|0)<1){H=E;F=12574;break}}L=A7(88)|0;N=L;if((L|0)!=0){if((c[f>>2]|0)==0){c[f>>2]=N}else{c[(c[t>>2]|0)+80>>2]=N}c[t>>2]=N;NN(L|0,g|0,80)|0}c[k>>2]=(c[k>>2]|0)-(c[m>>2]|0);mk(a,M,b)}if((F|0)==12526){c[f>>2]=c[B+80>>2];c[k>>2]=(c[k>>2]|0)+(c[m>>2]|0);A4(D);H=0;i=d;return H|0}else if((F|0)==12573){i=d;return H|0}else if((F|0)==12574){i=d;return H|0}else if((F|0)==12534){AN(0,0,1719600,(K=i,i=i+24|0,c[K>>2]=1715824,c[K+8>>2]=1712288,c[K+16>>2]=651,K)|0);i=K;cw();return 0}return 0}function mk(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;f=i;i=i+8|0;g=f|0;h=b+440|0;if((c[h>>2]|0)<=0){i=f;return}j=b+248|0;k=a;l=b|0;m=b+228|0;n=c[m>>2]|0;AN(k,48,1723040,(o=i,i=i+16|0,c[o>>2]=c[l>>2],c[o+8>>2]=n,o)|0);i=o;c[m>>2]=(c[m>>2]|0)-1;do{if((e|0)==0){p=b+256|0;q=12580}else{n=b+252|0;r=b+256|0;s=e+28|0;t=A2(c[n>>2]|0,(c[r>>2]|0)+32+(c[s>>2]|0)|0)|0;if((t|0)==0){p=r;q=12580;break}c[n>>2]=t;u=t+(c[r>>2]|0)|0;t=c[e+24>>2]|0;v=c[s>>2]|0;NN(u|0,t|0,v)|0;v=(c[r>>2]|0)+(c[s>>2]|0)|0;c[r>>2]=v;NM((c[n>>2]|0)+v|0,0,32)}}while(0);do{if((q|0)==12580){c[m>>2]=0;if((c[p>>2]|0)!=0){break}AN(k,24,1722640,(o=i,i=i+8|0,c[o>>2]=c[l>>2],o)|0);i=o}}while(0);do{if((c[a+1256>>2]|0)<1){w=1}else{if((c[m>>2]|0)<1){w=1;break}p=c[b+256>>2]|0;q=p>>>0>65535;v=q?16:0;n=q?p>>>16:p;if((n&65280|0)==0){x=n;y=v}else{x=n>>>8;y=v|8}v=p-(c[e+28>>2]|0)|0;p=v>>>0>65535;n=p?16:0;q=p?v>>>16:v;if((q&65280|0)==0){z=q;A=n}else{z=q>>>8;A=n|8}if(((d[247648+x|0]|0)+y|0)!=((d[247648+z|0]|0)+A|0)){w=0;break}i=f;return}}while(0);A=mf(j,1,g)|0;L16916:do{if((A|0)==0){B=c[g>>2]|0}else{j=c[g>>2]|0;if((c[h>>2]|0)>(j|0)){B=j;break}z=2500-(c[m>>2]|0)|0;y=A|0;x=c[y>>2]|0;AN(k,48,1718704,(o=i,i=i+32|0,c[o>>2]=c[b+256>>2],c[o+8>>2]=z,c[o+16>>2]=x,c[o+24>>2]=j,o)|0);i=o;x=0;while(1){if((x|0)==9){B=j;break L16916}if((a8(c[y>>2]|0,c[77376+(x*12|0)>>2]|0)|0)==0){break}else{x=x+1|0}}y=b+8|0;c[(c[y>>2]|0)+48>>2]=c[77380+(x*12|0)>>2];c[(c[y>>2]|0)+8>>2]=c[77384+(x*12|0)>>2];B=j}}while(0);A=b+8|0;do{if((c[(c[A>>2]|0)+48>>2]|0)!=0&(B|0)>25|w){c[b+256>>2]=0;A6(b+252|0);c[h>>2]=-1;m=c[l>>2]|0;if((c[(c[A>>2]|0)+48>>2]|0)==0){AN(k,24,1721648,(o=i,i=i+8|0,c[o>>2]=m,o)|0);i=o;break}else{AN(k,48,1721992,(o=i,i=i+8|0,c[o>>2]=m,o)|0);i=o;break}}}while(0);o=c[A>>2]|0;A=c[o+8>>2]|0;if((A|0)==3){k=c[a+1120>>2]|0;if((k|0)==0){i=f;return}c[o+48>>2]=k;i=f;return}else if((A|0)==0){k=c[a+1112>>2]|0;if((k|0)==0){i=f;return}c[o+48>>2]=k;i=f;return}else if((A|0)==1){A=c[a+1116>>2]|0;if((A|0)==0){i=f;return}c[o+48>>2]=A;i=f;return}else{i=f;return}}function ml(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;e=(d|0)!=0;do{if(!e){d=c[a+424>>2]|0;if((d|0)>1){f=d}else{break}return f|0}}while(0);d=pt(a,b)|0;if((d|0)>0){f=d;return f|0}d=c[a+424>>2]|0;g=(d|0)>1;if(g|e){f=g?d:-1;return f|0}d=c[a+72>>2]|0;if(!((d|0)>0&(b|0)>0)){f=-1;return f|0}g=c[a+412>>2]|0;if((g|0)<=0){f=-1;return f|0}if((c[a+432>>2]|0)<=1){f=-1;return f|0}if(((c[a+48>>2]|0)-86023|0)>>>0>=2){f=-1;return f|0}a=b;e=N2(a<<3|0>>>29,((b|0)<0|0?-1:0)<<3|a>>>29,g,(g|0)<0|0?-1:0)|0;g=N0(e,G,d,(d|0)<0|0?-1:0)|0;f=g;return f|0}function mm(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;c[a>>2]=0;c[b>>2]=0;g=d+8|0;h=c[g>>2]|0;i=c[h+8>>2]|0;if((i|0)==0){j=d+412|0;if((c[j>>2]|0)!=0&(e|0)==0){c[a>>2]=c[d+416>>2];c[b>>2]=c[j>>2];return}j=c[d+40>>2]|0;k=N2(j,(j|0)<0|0?-1:0,1e3,0)|0;l=G;m=d+44|0;d=c[m>>2]|0;n=(d|0)<0|0?-1:0;if((l|0)>(n|0)|(l|0)==(n|0)&k>>>0>d>>>0){c[a>>2]=j;c[b>>2]=c[m>>2];return}m=c[h+104>>2]|0;j=N2(m,(m|0)<0|0?-1:0,1e3,0)|0;d=G;k=c[h+108>>2]|0;n=(k|0)<0|0?-1:0;if(!((d|0)>(n|0)|(d|0)==(n|0)&j>>>0>k>>>0)){return}c[a>>2]=m;m=c[(c[g>>2]|0)+108>>2]|0;c[b>>2]=m;k=(e|0)!=0;do{if(k){j=c[e+36>>2]|0;if((j|0)==0){return}n=c[a>>2]|0;d=j+1|0;if((n|0)>(2147483647/(d|0)|0|0)){c[b>>2]=(m|0)/(d|0)|0;break}c[a>>2]=ac(n,d)|0;return}}while(0);if((c[(c[g>>2]|0)+112>>2]|0)<2|k){return}c[b>>2]=0;c[a>>2]=0;return}else if((i|0)==1){i=c[f+28>>2]|0;f=h+424|0;k=c[f>>2]|0;do{if((k|0)>1){o=k}else{m=pt(h,i)|0;if((m|0)>0){o=m;break}m=c[f>>2]|0;e=(m|0)>1;do{if(e){p=e?m:-1}else{d=c[h+72>>2]|0;if(!((d|0)>0&(i|0)>0)){return}n=c[h+412>>2]|0;if((n|0)<=0){return}if((c[h+432>>2]|0)<=1){return}if(((c[h+48>>2]|0)-86023|0)>>>0<2){j=i;l=N2(j<<3|0>>>29,((i|0)<0|0?-1:0)<<3|j>>>29,n,(n|0)<0|0?-1:0)|0;n=N0(l,G,d,(d|0)<0|0?-1:0)|0;p=n;break}else{return}}}while(0);if((p|0)>=1){o=p;break}return}}while(0);if((c[(c[g>>2]|0)+412>>2]|0)<1){return}c[a>>2]=o;c[b>>2]=c[(c[g>>2]|0)+412>>2];return}else{return}}function mn(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;e=i;i=i+16|0;f=e|0;g=e+8|0;h=b+1220|0;do{if((c[b+1084>>2]&1|0)==0){j=c[h>>2]|0;if((j|0)==0){k=mo(b,d)|0;if((k|0)<0){l=k}else{m=k;break}i=e;return l|0}k=g;c[g>>2]=j;n=d;o=j;NN(n|0,o|0,80)|0;o=j+80|0;c[h>>2]=c[o>>2];if((c[o>>2]|0)==0){c[b+1224>>2]=0}A6(k);m=0}else{k=b+1224|0;o=d;j=b+28|0;n=1;L17029:while(1){while(1){p=c[h>>2]|0;if((p|0)==0){q=0}else{r=p+16|0;s=c[r>>2]|0;t=c[r+4>>2]|0;if((s|0)==0&(t|0)==(-2147483648|0)){u=p;v=p+8|0}else{w=p+32|0;x=p+8|0;z=NW(2,0,(c[(c[(c[j>>2]|0)+(c[w>>2]<<2)>>2]|0)+188>>2]|0)-1|0)|0;A=G;B=p;C=t;t=s;while(1){if(!((c[x>>2]|0)==0&(c[x+4>>2]|0)==(-2147483648|0))){D=C;E=t;break}do{if((c[B+32>>2]|0)==(c[w>>2]|0)){s=B+16|0;F=A_(c[r>>2]|0,c[r+4>>2]|0,c[s>>2]|0,c[s+4>>2]|0,z,A)|0;H=G;I=0;if(!((H|0)<(I|0)|(H|0)==(I|0)&F>>>0<0>>>0)){J=C;K=t;break}F=B+8|0;I=A_(c[F>>2]|0,c[F+4>>2]|0,c[s>>2]|0,c[s+4>>2]|0,z,A)|0;if(!((I|0)==0&(G|0)==0)){I=c[s+4>>2]|0;c[x>>2]=c[s>>2];c[x+4>>2]=I}if((t|0)==0&(C|0)==(-2147483648|0)){J=-2147483648;K=0;break}J=c[s+4>>2]|0;K=c[s>>2]|0}else{J=C;K=t}}while(0);s=c[B+80>>2]|0;if((s|0)==0){D=J;E=K;break}else{B=s;C=J;t=K}}do{if(!n){if((c[x>>2]|0)!=0|(c[x+4>>2]|0)!=(-2147483648|0)|(E|0)==0&(D|0)==(-2147483648|0)){break}t=c[p+48>>2]|0;C=NU(t,(t|0)<0|0?-1:0,E,D)|0;c[x>>2]=C;c[x+4>>2]=G}}while(0);u=c[h>>2]|0;v=x}if(!((c[v>>2]|0)==0&(c[v+4>>2]|0)==(-2147483648|0))){L=12700;break L17029}if(((c[r>>2]|0)!=0|(c[r+4>>2]|0)!=(-2147483648|0))&n){q=u}else{L=12700;break L17029}}M=mo(b,d)|0;if((M|0)<0){break}p=A7(88)|0;C=p;if((p|0)==0){N=0}else{if((c[h>>2]|0)==0){c[h>>2]=C}else{c[(c[k>>2]|0)+80>>2]=C}c[k>>2]=C;C=p;NN(p|0,o|0,80)|0;N=C}if((nH(N)|0)<0){l=-12;L=12736;break L17029}}if((q|0)==0|(M|0)==-11){l=M;L=12738;break}else{n=0}}if((L|0)==12736){i=e;return l|0}else if((L|0)==12738){i=e;return l|0}else if((L|0)==12700){n=f;if((u|0)==0){AN(0,0,1719600,(O=i,i=i+24|0,c[O>>2]=1723512,c[O+8>>2]=1712288,c[O+16>>2]=1273,O)|0);i=O;cw();return 0}c[f>>2]=u;j=u;NN(o|0,j|0,80)|0;j=u+80|0;c[h>>2]=c[j>>2];if((c[j>>2]|0)==0){c[k>>2]=0}A6(n);m=0;break}}}while(0);h=b+28|0;u=c[(c[h>>2]|0)+(c[d+32>>2]<<2)>>2]|0;f=u+448|0;if((c[f>>2]|0)!=0){L=nK(d,70,10)|0;M=L;y=c[f>>2]|0;a[M]=y&255;y=y>>8;a[M+1|0]=y&255;y=y>>8;a[M+2|0]=y&255;y=y>>8;a[M+3|0]=y&255;AN(b,48,1704616,(O=i,i=i+8|0,c[O>>2]=c[f>>2],O)|0);i=O;c[f>>2]=0}do{if((c[(c[b+4>>2]|0)+8>>2]&256|0)!=0){if((c[d+36>>2]&1|0)==0){break}f=c[(c[h>>2]|0)+(c[u>>2]<<2)>>2]|0;O=f+404|0;M=c[O>>2]|0;if(M>>>0>=(((c[b+1124>>2]|0)>>>0)/24|0)>>>0){if((M|0)>0){M=f+400|0;f=0;L=0;while(1){q=c[M>>2]|0;N=q+(f*24|0)|0;v=q+(L*24|0)|0;c[N>>2]=c[v>>2];c[N+4>>2]=c[v+4>>2];c[N+8>>2]=c[v+8>>2];c[N+12>>2]=c[v+12>>2];c[N+16>>2]=c[v+16>>2];c[N+20>>2]=c[v+20>>2];v=f+1|0;N=v<<1;if((N|0)<(c[O>>2]|0)){f=v;L=N}else{P=v;break}}}else{P=0}c[O>>2]=P}L=d+64|0;f=c[L>>2]|0;M=c[L+4>>2]|0;L=d+16|0;k=c[L>>2]|0;o=c[L+4>>2]|0;L=c[u+472>>2]|0;do{if((L|0)==0){Q=o;R=k}else{v=u+464|0;N=c[v>>2]|0;q=c[v+4>>2]|0;if((N|0)==0&(q|0)==(-2147483648|0)|(k|0)==0&(o|0)==(-2147483648|0)){Q=o;R=k;break}v=(q|0)>(o|0)|(q|0)==(o|0)&N>>>0>k>>>0;if((L|0)==1&v){N=NW(1,0,c[u+188>>2]|0)|0;q=NU(N,G,k,o)|0;Q=G;R=q;break}if((L|0)!=-1|v){Q=o;R=k;break}v=NW(1,0,c[u+188>>2]|0)|0;q=NV(k,o,v,G)|0;Q=G;R=q}}while(0);o=u+400|0;k=u+404|0;L=u+408|0;ms(o,k,L,f,M,R,Q,0,0,1)|0}}while(0);Q=d+16|0;R=c[Q>>2]|0;u=c[Q+4>>2]|0;P=2147352575;if((u|0)>(P|0)|(u|0)==(P|0)&R>>>0>-1>>>0){P=NU(R,u,1,-2147418112)|0;c[Q>>2]=P;c[Q+4>>2]=G}Q=d+8|0;d=c[Q>>2]|0;P=c[Q+4>>2]|0;u=2147352575;if(!((P|0)>(u|0)|(P|0)==(u|0)&d>>>0>-1>>>0)){l=m;i=e;return l|0}u=NU(d,P,1,-2147418112)|0;c[Q>>2]=u;c[Q+4>>2]=G;l=m;i=e;return l|0}function mo(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0;e=i;i=i+280|0;f=e|0;g=e+8|0;h=e+216|0;j=e+248|0;nB(d);k=b+1248|0;l=g+32|0;m=b+28|0;n=g+8|0;o=b+1168|0;p=d;q=g;r=b+4|0;s=d+36|0;t=d+16|0;u=b+1124|0;v=d+64|0;w=b+1084|0;x=b;y=e+152|0;z=e+184|0;A=g+16|0;B=g+28|0;C=g+48|0;D=g+36|0;E=e+88|0;F=e+120|0;L17116:while(1){H=c[k>>2]|0;if((H|0)!=0){I=H;J=12800;break}K=mj(b,g)|0;if((K|0)<0){J=12743;break}H=c[l>>2]|0;L=c[(c[m>>2]|0)+(H<<2)>>2]|0;M=c[n>>2]|0;N=c[n+4>>2]|0;do{if(!((M|0)==0&(N|0)==(-2147483648|0))){O=c[A>>2]|0;P=c[A+4>>2]|0;if(!(((O|0)!=0|(P|0)!=(-2147483648|0))&((N|0)<(P|0)|(N|0)==(P|0)&M>>>0<O>>>0))){break}NM(E|0,0,32);a[E]=0;O=1724096;bc(E|0,32,O|0,(Q=i,i=i+16|0,c[Q>>2]=M,c[Q+8>>2]=N,Q)|0)|0;i=Q;NM(F|0,0,32);a[F]=0;O=c[A>>2]|0;P=c[A+4>>2]|0;if((O|0)==0&(P|0)==(-2147483648|0)){bc(F|0,32,1724464,(Q=i,i=i+1|0,i=i+7&-8,c[Q>>2]=0,Q)|0)|0;i=Q}else{R=1724096;bc(F|0,32,R|0,(Q=i,i=i+16|0,c[Q>>2]=O,c[Q+8>>2]=P,Q)|0)|0;i=Q}P=c[B>>2]|0;AN(x,24,1727256,(Q=i,i=i+32|0,c[Q>>2]=H,c[Q+8>>2]=E,c[Q+16>>2]=F,c[Q+24>>2]=P,Q)|0);i=Q}}while(0);if((c[o>>2]&1|0)!=0){H=c[l>>2]|0;NM(y|0,0,32);a[y]=0;N=c[n>>2]|0;M=c[n+4>>2]|0;if((N|0)==0&(M|0)==(-2147483648|0)){bc(y|0,32,1724464,(Q=i,i=i+1|0,i=i+7&-8,c[Q>>2]=0,Q)|0)|0;i=Q}else{P=1724096;bc(y|0,32,P|0,(Q=i,i=i+16|0,c[Q>>2]=N,c[Q+8>>2]=M,Q)|0)|0;i=Q}NM(z|0,0,32);a[z]=0;M=c[A>>2]|0;N=c[A+4>>2]|0;if((M|0)==0&(N|0)==(-2147483648|0)){bc(z|0,32,1724464,(Q=i,i=i+1|0,i=i+7&-8,c[Q>>2]=0,Q)|0)|0;i=Q}else{P=1724096;bc(z|0,32,P|0,(Q=i,i=i+16|0,c[Q>>2]=M,c[Q+8>>2]=N,Q)|0)|0;i=Q}N=c[B>>2]|0;M=c[C>>2]|0;P=c[D>>2]|0;AN(x,48,1726816,(Q=i,i=i+48|0,c[Q>>2]=H,c[Q+8>>2]=y,c[Q+16>>2]=z,c[Q+24>>2]=N,c[Q+32>>2]=M,c[Q+40>>2]=P,Q)|0);i=Q}P=L+236|0;L17139:do{if((c[P>>2]|0)==0){J=12775}else{M=L+240|0;do{if((c[M>>2]|0)==0){if((c[w>>2]&32|0)!=0){J=12772;break}N=L+8|0;H=oE(c[(c[N>>2]|0)+48>>2]|0)|0;c[M>>2]=H;if((H|0)==0){O=pn(c[(c[N>>2]|0)+48>>2]|0)|0;AN(x,40,1686352,(Q=i,i=i+8|0,c[Q>>2]=O,Q)|0);i=Q;c[P>>2]=0;J=12775;break L17139}O=c[P>>2]|0;if((O|0)==2){N=H+176|0;c[N>>2]=c[N>>2]|1;J=12772;break}else if((O|0)==4){N=H+176|0;c[N>>2]=c[N>>2]|2;J=12772;break}else if((O|0)==1463898624){N=H+176|0;c[N>>2]=c[N>>2]|4096;J=12772;break}else{S=O;break}}else{J=12772}}while(0);if((J|0)==12772){J=0;S=c[P>>2]|0}if((S|0)==0){J=12775;break}if((c[M>>2]|0)==0){J=12775;break}if((c[L+76>>2]|0)<48){O=m1(b,g,c[l>>2]|0)|0;if((O|0)<0){T=O;J=12814;break L17116}else{U=0;V=O;break}}else{nJ(g);U=0;V=0;break}}}while(0);do{if((J|0)==12775){J=0;NN(p|0,q|0,80)|0;m2(b,L,0,d);if((c[(c[r>>2]|0)+8>>2]&256|0)==0){U=1;V=0;break}if((c[s>>2]&1|0)==0){U=1;V=0;break}P=c[t>>2]|0;O=c[t+4>>2]|0;if((P|0)==0&(O|0)==(-2147483648|0)){U=1;V=0;break}N=c[(c[m>>2]|0)+(c[L>>2]<<2)>>2]|0;H=N+404|0;R=c[H>>2]|0;if(R>>>0<(((c[u>>2]|0)>>>0)/24|0)>>>0){W=O;X=P}else{if((R|0)>0){R=N+400|0;N=0;P=0;while(1){O=c[R>>2]|0;Y=O+(N*24|0)|0;Z=O+(P*24|0)|0;c[Y>>2]=c[Z>>2];c[Y+4>>2]=c[Z+4>>2];c[Y+8>>2]=c[Z+8>>2];c[Y+12>>2]=c[Z+12>>2];c[Y+16>>2]=c[Z+16>>2];c[Y+20>>2]=c[Z+20>>2];Z=N+1|0;Y=Z<<1;if((Y|0)<(c[H>>2]|0)){N=Z;P=Y}else{_=Z;break}}}else{_=0}c[H>>2]=_;W=c[t+4>>2]|0;X=c[t>>2]|0}P=c[v>>2]|0;N=c[v+4>>2]|0;R=c[L+472>>2]|0;do{if((R|0)==0){$=W;aa=X}else{M=L+464|0;Z=c[M>>2]|0;Y=c[M+4>>2]|0;if((Z|0)==0&(Y|0)==(-2147483648|0)|(X|0)==0&(W|0)==(-2147483648|0)){$=W;aa=X;break}M=(Y|0)>(W|0)|(Y|0)==(W|0)&Z>>>0>X>>>0;if((R|0)==1&M){Z=NW(1,0,c[L+188>>2]|0)|0;Y=NU(Z,G,X,W)|0;$=G;aa=Y;break}if((R|0)!=-1|M){$=W;aa=X;break}M=NW(1,0,c[L+188>>2]|0)|0;Y=NV(X,W,M,G)|0;$=G;aa=Y}}while(0);R=L+400|0;H=L+404|0;Y=L+408|0;ms(R,H,Y,P,N,aa,$,0,0,1)|0;U=1;V=0}}while(0);Y=L+444|0;do{if((c[s>>2]&1|0)==0){if((c[Y>>2]|0)==0){break}nJ(g);if((U|0)==0){continue L17116}NN(p|0,q|0,80)|0;continue L17116}else{c[Y>>2]=0}}while(0);if((U|0)!=0){ab=V;break}}if((J|0)==12814){i=e;return T|0}if((J|0)==12743){if((K|0)==-11){T=-11;i=e;return T|0}V=b+24|0;U=c[V>>2]|0;if((U|0)!=0){q=0;g=U;while(1){U=c[(c[m>>2]|0)+(q<<2)>>2]|0;do{if((c[U+240>>2]|0)==0){ac=g}else{if((c[U+236>>2]|0)==0){ac=g;break}$=c[U>>2]|0;m1(b,0,$)|0;ac=c[V>>2]|0}}while(0);U=q+1|0;if(U>>>0<ac>>>0){q=U;g=ac}else{break}}}ac=c[k>>2]|0;if((ac|0)==0){ab=K}else{I=ac;J=12800}}if((J|0)==12800){J=f;c[f>>2]=I;f=I;NN(p|0,f|0,80)|0;f=I+80|0;c[k>>2]=c[f>>2];if((c[f>>2]|0)==0){c[b+1252>>2]=0}A6(J);ab=0}if((c[o>>2]&1|0)==0){T=ab;i=e;return T|0}o=c[d+32>>2]|0;J=h|0;NM(J|0,0,32);a[J]=0;h=d+8|0;b=c[h>>2]|0;f=c[h+4>>2]|0;if((b|0)==0&(f|0)==(-2147483648|0)){bc(J|0,32,1724464,(Q=i,i=i+1|0,i=i+7&-8,c[Q>>2]=0,Q)|0)|0;i=Q}else{h=1724096;bc(J|0,32,h|0,(Q=i,i=i+16|0,c[Q>>2]=b,c[Q+8>>2]=f,Q)|0)|0;i=Q}f=j|0;NM(f|0,0,32);a[f]=0;j=c[t>>2]|0;b=c[t+4>>2]|0;if((j|0)==0&(b|0)==(-2147483648|0)){bc(f|0,32,1724464,(Q=i,i=i+1|0,i=i+7&-8,c[Q>>2]=0,Q)|0)|0;i=Q}else{t=1724096;bc(f|0,32,t|0,(Q=i,i=i+16|0,c[Q>>2]=j,c[Q+8>>2]=b,Q)|0)|0;i=Q}b=c[d+28>>2]|0;j=c[d+48>>2]|0;d=c[s>>2]|0;AN(x,48,1726304,(Q=i,i=i+48|0,c[Q>>2]=o,c[Q+8>>2]=J,c[Q+16>>2]=f,c[Q+24>>2]=b,c[Q+32>>2]=j,c[Q+40>>2]=d,Q)|0);i=Q;T=ab;i=e;return T|0}function mp(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0;j=c[a+472>>2]|0;do{if((j|0)==0){k=f;l=e}else{m=a+464|0;n=c[m>>2]|0;o=c[m+4>>2]|0;if((n|0)==0&(o|0)==(-2147483648|0)|(e|0)==0&(f|0)==(-2147483648|0)){k=f;l=e;break}m=(o|0)>(f|0)|(o|0)==(f|0)&n>>>0>e>>>0;if((j|0)==1&m){n=NW(1,0,c[a+188>>2]|0)|0;o=NU(n,G,e,f)|0;k=G;l=o;break}if((j|0)!=-1|m){k=f;l=e;break}m=NW(1,0,c[a+188>>2]|0)|0;o=NV(e,f,m,G)|0;k=G;l=o}}while(0);return ms(a+400|0,a+404|0,a+408|0,b,d,l,k,g,h,i)|0}function mq(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0;b=i;i=i+24|0;d=b|0;e=b+8|0;f=b+16|0;g=a+1248|0;h=a+1252|0;j=f;k=c[g>>2]|0;if((k|0)!=0){l=k;do{c[f>>2]=l;c[g>>2]=c[l+80>>2];nJ(l|0);A6(j);l=c[g>>2]|0;}while((l|0)!=0)}c[h>>2]=0;h=a+1220|0;l=a+1224|0;g=e;j=c[h>>2]|0;if((j|0)!=0){f=j;do{c[e>>2]=f;c[h>>2]=c[f+80>>2];nJ(f|0);A6(g);f=c[h>>2]|0;}while((f|0)!=0)}c[l>>2]=0;l=a+1240|0;f=a+1244|0;h=d;g=c[l>>2]|0;if((g|0)!=0){e=g;do{c[d>>2]=e;c[l>>2]=c[e+80>>2];nJ(e|0);A6(h);e=c[l>>2]|0;}while((e|0)!=0)}c[f>>2]=0;c[a+1256>>2]=25e5;f=a+24|0;if((c[f>>2]|0)==0){i=b;return}e=a+28|0;a=0;do{l=c[(c[e>>2]|0)+(a<<2)>>2]|0;h=l+240|0;d=c[h>>2]|0;if((d|0)!=0){oH(d);c[h>>2]=0}h=l+216|0;c[h>>2]=0;c[h+4>>2]=-2147483648;h=l+200|0;d=(c[h>>2]|0)==0&(c[h+4>>2]|0)==(-2147483648|0);h=l+208|0;c[h>>2]=d?-1:0;c[h+4>>2]=d?2147418111:-2147483648;d=l+192|0;c[d>>2]=0;c[d+4>>2]=-2147483648;c[l+228>>2]=2500;d=l+264|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+272|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+280|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+288|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+296|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+304|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+312|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+320|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+328|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+336|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+344|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+352|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+360|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+368|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+376|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+384|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=l+392|0;c[d>>2]=0;c[d+4>>2]=-2147483648;a=a+1|0;}while(a>>>0<(c[f>>2]|0)>>>0);i=b;return}function mr(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0;f=a+24|0;if((c[f>>2]|0)==0){return}g=a+28|0;a=b+40|0;h=b+44|0;b=0;do{i=c[(c[g>>2]|0)+(b<<2)>>2]|0;j=c[i+44>>2]|0;k=c[a>>2]|0;l=N2(k,(k|0)<0|0?-1:0,j,(j|0)<0|0?-1:0)|0;j=G;k=c[i+40>>2]|0;m=c[h>>2]|0;n=N2(m,(m|0)<0|0?-1:0,k,(k|0)<0|0?-1:0)|0;k=AW(d,e,l,j,n,G)|0;n=i+208|0;c[n>>2]=k;c[n+4>>2]=G;b=b+1|0;}while(b>>>0<(c[f>>2]|0)>>>0);return}function ms(a,b,d,e,f,g,h,j,k,l){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;m=i;n=c[b>>2]|0;if((n+1|0)>>>0>178956969){o=-1;i=m;return o|0}if((g|0)==0&(h|0)==(-2147483648|0)){o=-22;i=m;return o|0}p=2147352575;q=(h|0)>(p|0)|(h|0)==(p|0)&g>>>0>-1>>>0;p=NU(g,h,1,-2147418112)|0;r=q?p:g;g=q?G:h;h=oO(c[a>>2]|0,d,(n*24|0)+24|0)|0;n=h;if((h|0)==0){o=-1;i=m;return o|0}c[a>>2]=n;a=c[b>>2]|0;h=(a|0)==0;if(h){s=0;t=-1}else{d=a-1|0;q=n+(d*24|0)+8|0;p=c[q+4>>2]|0;s=a;t=(p|0)<(g|0)|(p|0)==(g|0)&(c[q>>2]|0)>>>0<r>>>0?d:-1}if((s-t|0)>1){d=t;t=s;while(1){q=d+t>>1;p=n+(q*24|0)+8|0;u=c[p>>2]|0;v=c[p+4>>2]|0;p=(v|0)<(g|0)|(v|0)==(g|0)&u>>>0<r>>>0?t:q;w=(v|0)>(g|0)|(v|0)==(g|0)&u>>>0>r>>>0?d:q;if((p-w|0)>1){d=w;t=p}else{x=p;break}}}else{x=s}s=(x|0)==(a|0)?-1:x;do{if((s|0)<0){c[b>>2]=a+1;x=n+(a*24|0)|0;if(h){y=0;z=x;A=k;break}t=n+((a-1|0)*24|0)+8|0;d=c[t+4>>2]|0;if((d|0)<(g|0)|(d|0)==(g|0)&(c[t>>2]|0)>>>0<r>>>0){y=a;z=x;A=k;break}AN(0,0,1719600,(x=i,i=i+24|0,c[x>>2]=1702440,c[x+8>>2]=1712288,c[x+16>>2]=1606,x)|0);i=x;cw();return 0}else{x=n+(s*24|0)|0;t=n+(s*24|0)+8|0;d=c[t>>2]|0;p=c[t+4>>2]|0;if((d|0)==(r|0)&(p|0)==(g|0)){t=x|0;if(!((c[t>>2]|0)==(e|0)&(c[t+4>>2]|0)==(f|0))){y=s;z=x;A=k;break}t=c[n+(s*24|0)+20>>2]|0;y=s;z=x;A=(t|0)>(k|0)?t:k;break}if((p|0)>(g|0)|(p|0)==(g|0)&d>>>0>r>>>0){NP(n+((s+1|0)*24|0)|0,x|0,(a-s|0)*24|0|0);c[b>>2]=(c[b>>2]|0)+1;y=s;z=x;A=k;break}else{o=-1;i=m;return o|0}}}while(0);k=z|0;c[k>>2]=e;c[k+4>>2]=f;f=z+8|0;c[f>>2]=r;c[f+4>>2]=g;c[z+20>>2]=A;c[z+16>>2]=l&3|j<<2;o=y;i=m;return o|0}function mt(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;f=c[a+400>>2]|0;g=c[a+404>>2]|0;if((g|0)==0){h=0;i=-1}else{a=g-1|0;j=f+(a*24|0)+8|0;k=c[j+4>>2]|0;h=g;i=(k|0)<(d|0)|(k|0)==(d|0)&(c[j>>2]|0)>>>0<b>>>0?a:-1}if((h-i|0)>1){a=i;j=h;while(1){k=a+j>>1;l=f+(k*24|0)+8|0;m=c[l>>2]|0;n=c[l+4>>2]|0;l=(n|0)<(d|0)|(n|0)==(d|0)&m>>>0<b>>>0?j:k;o=(n|0)>(d|0)|(n|0)==(d|0)&m>>>0>b>>>0?a:k;if((l-o|0)>1){a=o;j=l}else{p=o;q=l;break}}}else{p=i;q=h}h=e&1;i=(h|0)!=0?p:q;if((e&4|0)!=0){r=i;s=(r|0)==(g|0);t=s?-1:r;return t|0}if(!((i|0)>-1&(i|0)<(g|0))){r=i;s=(r|0)==(g|0);t=s?-1:r;return t|0}e=(h<<1^2)-1|0;h=i;while(1){if((c[f+(h*24|0)+16>>2]&1|0)!=0){r=h;u=12876;break}i=e+h|0;if((i|0)>-1&(i|0)<(g|0)){h=i}else{r=i;u=12879;break}}if((u|0)==12879){s=(r|0)==(g|0);t=s?-1:r;return t|0}else if((u|0)==12876){s=(r|0)==(g|0);t=s?-1:r;return t|0}return 0}function mu(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0;g=i;i=i+8|0;h=g|0;j=c[a+4>>2]|0;if((b|0)<0){k=-1;i=g;return k|0}l=a+28|0;m=c[(c[l>>2]|0)+(b<<2)>>2]|0;n=c[m+400>>2]|0;do{if((n|0)==0){o=0;p=0;q=-2147483648;r=0;s=-2147483648;t=0;u=-1;v=-1;w=0;x=0}else{y=c[m+404>>2]|0;z=(y|0)==0;if(z){A=0;B=-1}else{C=y-1|0;D=n+(C*24|0)+8|0;E=c[D+4>>2]|0;A=y;B=(E|0)<(e|0)|(E|0)==(e|0)&(c[D>>2]|0)>>>0<d>>>0?C:-1}if((A-B|0)>1){C=B;D=A;while(1){E=C+D>>1;F=n+(E*24|0)+8|0;H=c[F>>2]|0;I=c[F+4>>2]|0;F=(I|0)<(e|0)|(I|0)==(e|0)&H>>>0<d>>>0?D:E;J=(I|0)>(e|0)|(I|0)==(e|0)&H>>>0>d>>>0?C:E;if((F-J|0)>1){C=J;D=F}else{K=J;break}}}else{K=B}D=(f&4|0)==0;L17322:do{if(D){if((K|0)>-1&(K|0)<(y|0)){L=K}else{M=K;break}while(1){if((c[n+(L*24|0)+16>>2]&1|0)!=0){M=L;break L17322}C=L-1|0;if((L|0)>0&(C|0)<(y|0)){L=C}else{M=C;break}}}else{M=K}}while(0);C=(M|0)==(y|0)?-1:M;J=(C|0)>0?C:0;C=n+(J*24|0)+8|0;F=c[C>>2]|0;E=c[C+4>>2]|0;C=n+(J*24|0)|0;H=c[C>>2]|0;I=c[C+4>>2]|0;if((E|0)>(e|0)|(E|0)==(e|0)&F>>>0>d>>>0){C=c[n+(J*24|0)+20>>2]|0;if((H|0)==(C|0)&(I|0)==(((C|0)<0|0?-1:0)|0)){N=12892}else{O=0;P=0;Q=-2147483648;R=0}}else{N=12892}if((N|0)==12892){O=I;P=H;Q=E;R=F}if(z){S=0;T=-1}else{F=y-1|0;E=n+(F*24|0)+8|0;H=c[E+4>>2]|0;S=y;T=(H|0)<(e|0)|(H|0)==(e|0)&(c[E>>2]|0)>>>0<d>>>0?F:-1}if((S-T|0)>1){F=T;E=S;while(1){H=F+E>>1;I=n+(H*24|0)+8|0;C=c[I>>2]|0;J=c[I+4>>2]|0;I=(J|0)<(e|0)|(J|0)==(e|0)&C>>>0<d>>>0?E:H;U=(J|0)>(e|0)|(J|0)==(e|0)&C>>>0>d>>>0?F:H;if((I-U|0)>1){F=U;E=I}else{V=I;break}}}else{V=S}L17340:do{if(D){if((V|0)>-1&(V|0)<(y|0)){W=V}else{X=V;break}while(1){if((c[n+(W*24|0)+16>>2]&1|0)!=0){X=W;break L17340}E=W+1|0;if((E|0)>-1&(E|0)<(y|0)){W=E}else{X=E;break}}}else{X=V}}while(0);D=(X|0)==(y|0)?-1:X;if((D|0)>=(y|0)){AN(0,0,1719600,(E=i,i=i+24|0,c[E>>2]=1699640,c[E+8>>2]=1712288,c[E+16>>2]=1722,E)|0);i=E;cw();return 0}if((D|0)<=-1){o=O;p=P;q=-2147483648;r=0;s=Q;t=R;u=-1;v=-1;w=0;x=0;break}E=n+(D*24|0)|0;F=c[E>>2]|0;z=c[E+4>>2]|0;E=n+(D*24|0)+8|0;I=c[E>>2]|0;U=c[E+4>>2]|0;E=c[n+(D*24|0)+20>>2]|0;D=NV(F,z,E,(E|0)<0|0?-1:0)|0;o=O;p=P;q=U;r=I;s=Q;t=R;u=G;v=D;w=z;x=F}}while(0);R=mv(a,b,d,e,p,o,x,w,v,u,t,s,r,q,f,h,c[j+56>>2]|0)|0;j=G;f=0;if((j|0)<(f|0)|(j|0)==(f|0)&R>>>0<0>>>0){k=-1;i=g;return k|0}f=il(c[a+16>>2]|0,R,j,0)|0;j=G;R=0;if((j|0)<(R|0)|(j|0)==(R|0)&f>>>0<0>>>0){k=f;i=g;return k|0}mq(a);f=c[h>>2]|0;R=c[h+4>>2]|0;h=a+24|0;if((c[h>>2]|0)==0){k=0;i=g;return k|0}a=m+40|0;j=m+44|0;m=0;while(1){q=c[(c[l>>2]|0)+(m<<2)>>2]|0;r=c[q+44>>2]|0;s=c[a>>2]|0;t=N2(s,(s|0)<0|0?-1:0,r,(r|0)<0|0?-1:0)|0;r=G;s=c[q+40>>2]|0;u=c[j>>2]|0;v=N2(u,(u|0)<0|0?-1:0,s,(s|0)<0|0?-1:0)|0;s=AW(f,R,t,r,v,G)|0;v=q+208|0;c[v>>2]=s;c[v+4>>2]=G;v=m+1|0;if(v>>>0<(c[h>>2]|0)>>>0){m=v}else{k=0;break}}i=g;return k|0}function mv(a,b,d,e,f,g,h,j,k,l,m,n,o,p,q,r,s){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;r=r|0;s=s|0;var t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0;t=i;i=i+32|0;u=t|0;v=t+8|0;w=t+16|0;x=t+24|0;c[u>>2]=f;c[u+4>>2]=g;c[v>>2]=h;c[v+4>>2]=j;c[w>>2]=o;c[w+4>>2]=p;do{if((m|0)==0&(n|0)==(-2147483648|0)){p=a+1232|0;o=c[p+4>>2]|0;c[u>>2]=c[p>>2];c[u+4>>2]=o;o=c4[s&63](a,b,u,-1,2147483647)|0;p=G;do{if((b|0)>-1){j=c[(c[a+28>>2]|0)+(b<<2)>>2]|0;h=c[j+472>>2]|0;if((h|0)==0){y=p;z=o;break}g=j+464|0;f=c[g>>2]|0;A=c[g+4>>2]|0;if((f|0)==0&(A|0)==(-2147483648|0)|(o|0)==0&(p|0)==(-2147483648|0)){y=p;z=o;break}g=(A|0)>(p|0)|(A|0)==(p|0)&f>>>0>o>>>0;if((h|0)==1&g){f=NW(1,0,c[j+188>>2]|0)|0;A=NU(f,G,o,p)|0;y=G;z=A;break}if((h|0)!=-1|g){y=p;z=o;break}g=NW(1,0,c[j+188>>2]|0)|0;j=NV(o,p,g,G)|0;y=G;z=j}else{y=p;z=o}}while(0);if((z|0)==0&(y|0)==(-2147483648|0)){B=-1;C=-1}else{D=y;E=z;break}i=t;return(G=B,C)|0}else{D=n;E=m}}while(0);if(!((D|0)<(e|0)|(D|0)==(e|0)&E>>>0<d>>>0)){c[r>>2]=E;c[r+4>>2]=D;B=c[u+4>>2]|0;C=c[u>>2]|0;i=t;return(G=B,C)|0}m=c[w>>2]|0;n=c[w+4>>2]|0;do{if((m|0)==0&(n|0)==(-2147483648|0)){z=mw(a,b,w,v,s)|0;if((z|0)>=0){F=c[v+4>>2]|0;H=c[v>>2]|0;I=c[w+4>>2]|0;J=c[w>>2]|0;break}B=(z|0)<0|0?-1:0;C=z;i=t;return(G=B,C)|0}else{F=l;H=k;I=n;J=m}}while(0);if(!((I|0)>(e|0)|(I|0)==(e|0)&J>>>0>d>>>0)){c[r>>2]=J;c[r+4>>2]=I;B=c[v+4>>2]|0;C=c[v>>2]|0;i=t;return(G=B,C)|0}if((D|0)>(I|0)|(D|0)==(I|0)&E>>>0>J>>>0){B=-1;C=-1;i=t;return(G=B,C)|0}m=(E|0)==(J|0)&(D|0)==(I|0);I=c[u>>2]|0;J=c[u+4>>2]|0;n=(b|0)>-1;k=a+28|0;l=m?J:F;F=m?I:H;H=0;m=D;D=E;E=J;J=I;L17396:while(1){I=NV(d,e,D,m)|0;z=G;y=l;o=F;p=H;K=E;L=J;while(1){if(!((K|0)<(y|0)|(K|0)==(y|0)&L>>>0<o>>>0)){M=12962;break L17396}j=c[v>>2]|0;g=c[v+4>>2]|0;if((y|0)>(g|0)|(y|0)==(g|0)&o>>>0>j>>>0){M=12939;break L17396}if((p|0)==0){h=NV(j,g,L,K)|0;A=G;f=NV(c[w>>2]|0,c[w+4>>2]|0,D,m)|0;N=AW(I,z,h,A,f,G)|0;f=G;A=NU(L,K,o,y)|0;h=NV(A,G,j,g)|0;g=NU(h,G,N,f)|0;f=G;O=f;P=g;Q=f;R=g;M=12944}else if((p|0)==1){g=NU(L,K,o,y)|0;f=G;N=g>>>1|f<<31;g=f>>1|((f|0)<0|0?-1:0)<<31;O=g;P=N;Q=g;R=N;M=12944}else{c[x>>2]=L;c[x+4>>2]=K;M=12945}do{if((M|0)==12944){M=0;c[x>>2]=R;c[x+4>>2]=Q;if(!((O|0)>(K|0)|(O|0)==(K|0)&P>>>0>L>>>0)){M=12945;break}if(!((O|0)>(y|0)|(O|0)==(y|0)&P>>>0>o>>>0)){S=O;T=P;break}c[x>>2]=o;c[x+4>>2]=y;S=y;T=o}}while(0);if((M|0)==12945){M=0;N=NU(L,K,1,0)|0;g=G;c[x>>2]=N;c[x+4>>2]=g;S=g;T=N}N=c4[s&63](a,b,x,-1,2147483647)|0;g=G;do{if(n){f=c[(c[k>>2]|0)+(b<<2)>>2]|0;h=c[f+472>>2]|0;if((h|0)==0){U=g;V=N;break}j=f+464|0;A=c[j>>2]|0;W=c[j+4>>2]|0;if((A|0)==0&(W|0)==(-2147483648|0)|(N|0)==0&(g|0)==(-2147483648|0)){U=g;V=N;break}j=(W|0)>(g|0)|(W|0)==(g|0)&A>>>0>N>>>0;if((h|0)==1&j){A=NW(1,0,c[f+188>>2]|0)|0;W=NU(A,G,N,g)|0;U=G;V=W;break}if((h|0)!=-1|j){U=g;V=N;break}j=NW(1,0,c[f+188>>2]|0)|0;f=NV(N,g,j,G)|0;U=G;V=f}else{U=g;V=N}}while(0);X=c[x>>2]|0;Y=c[x+4>>2]|0;Z=(X|0)==(c[v>>2]|0)&(Y|0)==(c[v+4>>2]|0)?p+1|0:0;if((V|0)==0&(U|0)==(-2147483648|0)){M=12956;break L17396}if((U|0)<(e|0)|(U|0)==(e|0)&V>>>0<d>>>0){_=y;$=o}else{N=NU(T,S,-1,-1)|0;c[v>>2]=X;c[v+4>>2]=Y;c[w>>2]=V;c[w+4>>2]=U;_=G;$=N}if(!((U|0)>(e|0)|(U|0)==(e|0)&V>>>0>d>>>0)){break}y=_;o=$;p=Z;K=c[u+4>>2]|0;L=c[u>>2]|0}c[u>>2]=X;c[u+4>>2]=Y;l=_;F=$;H=Z;m=U;D=V;E=Y;J=X}if((M|0)==12962){X=(q&1|0)!=0;q=X?L:c[v>>2]|0;L=X?K:c[v+4>>2]|0;c[x>>2]=q;c[x+4>>2]=L;x=X?m:c[w+4>>2]|0;c[r>>2]=X?D:c[w>>2]|0;c[r+4>>2]=x;B=L;C=q;i=t;return(G=B,C)|0}else if((M|0)==12939){cu(1697112,1712288,1828,1913480);return 0}else if((M|0)==12956){AN(a,16,1694880,(a=i,i=i+1|0,i=i+7&-8,c[a>>2]=0,a)|0);i=a;B=-1;C=-1;i=t;return(G=B,C)|0}return 0}function mw(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0;g=i;i=i+16|0;h=g|0;j=g+8|0;k=ip(c[a+16>>2]|0)|0;l=G;m=NU(k,l,-1,-1)|0;n=G;c[h>>2]=m;c[h+4>>2]=n;o=(b|0)>-1;p=a+28|0;q=0;r=1024;s=n;n=m;while(1){m=NV(n,s,r,q)|0;t=G;u=0;v=(t|0)<(u|0)|(t|0)==(u|0)&m>>>0<0>>>0;c[h>>2]=v?0:m;c[h+4>>2]=v?0:t;t=c4[f&63](a,b,h,n,s)|0;v=G;do{if(o){m=c[(c[p>>2]|0)+(b<<2)>>2]|0;u=c[m+472>>2]|0;if((u|0)==0){w=v;x=t;break}y=m+464|0;z=c[y>>2]|0;A=c[y+4>>2]|0;if((z|0)==0&(A|0)==(-2147483648|0)|(t|0)==0&(v|0)==(-2147483648|0)){w=v;x=t;break}y=(A|0)>(v|0)|(A|0)==(v|0)&z>>>0>t>>>0;if((u|0)==1&y){z=NW(1,0,c[m+188>>2]|0)|0;A=NU(z,G,t,v)|0;w=G;x=A;break}if((u|0)!=-1|y){w=v;x=t;break}y=NW(1,0,c[m+188>>2]|0)|0;m=NV(t,v,y,G)|0;w=G;x=m}else{w=v;x=t}}while(0);t=r<<1|0>>>31;v=q<<1|r>>>31;B=(x|0)==0&(w|0)==(-2147483648|0);m=s<<1|n>>>31;if(!(B&((m|0)>(v|0)|(m|0)==(v|0)&(n<<1|0>>>31)>>>0>t>>>0))){break}q=v;r=t;s=c[h+4>>2]|0;n=c[h>>2]|0}if(B){C=-1;i=g;return C|0}B=c[h>>2]|0;n=c[h+4>>2]|0;L17451:do{if(o){s=w;r=x;q=n;t=B;while(1){v=NU(t,q,1,0)|0;c[j>>2]=v;c[j+4>>2]=G;v=c4[f&63](a,b,j,-1,2147483647)|0;m=G;y=c[(c[p>>2]|0)+(b<<2)>>2]|0;u=c[y+472>>2]|0;do{if((u|0)==0){D=m;E=v}else{A=y+464|0;z=c[A>>2]|0;F=c[A+4>>2]|0;if((z|0)==0&(F|0)==(-2147483648|0)|(v|0)==0&(m|0)==(-2147483648|0)){D=m;E=v;break}A=(F|0)>(m|0)|(F|0)==(m|0)&z>>>0>v>>>0;if((u|0)==1&A){z=NW(1,0,c[y+188>>2]|0)|0;F=NU(z,G,v,m)|0;D=G;E=F;break}if((u|0)!=-1|A){D=m;E=v;break}A=NW(1,0,c[y+188>>2]|0)|0;F=NV(v,m,A,G)|0;D=G;E=F}}while(0);if((E|0)==0&(D|0)==(-2147483648|0)){H=s;I=r;break L17451}m=c[j>>2]|0;v=c[j+4>>2]|0;c[h>>2]=m;c[h+4>>2]=v;if((v|0)<(l|0)|(v|0)==(l|0)&m>>>0<k>>>0){s=D;r=E;q=v;t=m}else{H=D;I=E;break}}}else{t=w;q=x;r=n;s=B;while(1){m=NU(s,r,1,0)|0;c[j>>2]=m;c[j+4>>2]=G;m=c4[f&63](a,b,j,-1,2147483647)|0;v=G;if((m|0)==0&(v|0)==(-2147483648|0)){H=t;I=q;break L17451}y=c[j>>2]|0;u=c[j+4>>2]|0;c[h>>2]=y;c[h+4>>2]=u;if((u|0)<(l|0)|(u|0)==(l|0)&y>>>0<k>>>0){t=v;q=m;r=u;s=y}else{H=v;I=m;break}}}}while(0);if((d|0)!=0){c[d>>2]=I;c[d+4>>2]=H}if((e|0)==0){C=0;i=g;return C|0}H=c[h+4>>2]|0;c[e>>2]=c[h>>2];c[e+4>>2]=H;C=0;i=g;return C|0}function mx(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0;g=i;i=i+160|0;h=g|0;j=g+80|0;k=a+4|0;l=c[k>>2]|0;do{if((c[l+68>>2]|0)!=0){if((c[l+52>>2]|0)!=0){break}m=(f&1|0)==0;n=my(a,b,m?d:0,m?e:-2147483648,d,e,m?-1:d,m?2147483647:e,f&-2)|0;i=g;return n|0}}while(0);L17480:do{if((f&2|0)==0){if((b|0)<0){m=c[a+24>>2]|0;if((m|0)==0){n=-1;i=g;return n|0}o=c[a+28>>2]|0;p=-1;q=0;while(1){r=c[o+(q<<2)>>2]|0;s=c[(c[r+8>>2]|0)+8>>2]|0;if((s|0)==0){if((c[r+72>>2]&1024|0)==0){t=13018;break}}if((p|0)<0){u=(s|0)==1?q:p}else{u=p}s=q+1|0;if(s>>>0<m>>>0){p=u;q=s}else{t=13017;break}}do{if((t|0)==13018){if((q|0)<0){n=-1}else{v=q;w=r;break}i=g;return n|0}else if((t|0)==13017){p=(u|0)<0?0:u;v=p;w=c[o+(p<<2)>>2]|0}}while(0);o=c[w+44>>2]|0;q=c[w+40>>2]|0;p=N2(q,(q|0)<0|0?-1:0,1e6,0)|0;q=AW(d,e,o,(o|0)<0|0?-1:0,p,G)|0;x=v;y=G;z=q}else{x=b;y=e;z=d}if((c[l+52>>2]|0)==0){A=l}else{mq(a);if((c4[c[(c[k>>2]|0)+52>>2]&63](a,x,z,y,f)|0)>-1){break}A=c[k>>2]|0}q=c[A+8>>2]|0;do{if((c[A+56>>2]|0)==0){t=13026}else{if((q&8192|0)!=0){t=13026;break}mq(a);B=mu(a,x,z,y,f)|0}}while(0);L17515:do{if((t|0)==13026){if((q&16384|0)!=0){n=-1;i=g;return n|0}mq(a);p=a+28|0;o=c[(c[p>>2]|0)+(x<<2)>>2]|0;m=o+400|0;s=c[m>>2]|0;C=o+404|0;D=c[C>>2]|0;E=(D|0)==0;if(E){F=0;H=-1}else{I=D-1|0;J=s+(I*24|0)+8|0;K=c[J+4>>2]|0;F=D;H=(K|0)<(y|0)|(K|0)==(y|0)&(c[J>>2]|0)>>>0<z>>>0?I:-1}if((F-H|0)>1){I=H;J=F;while(1){K=I+J>>1;L=s+(K*24|0)+8|0;M=c[L>>2]|0;N=c[L+4>>2]|0;L=(N|0)<(y|0)|(N|0)==(y|0)&M>>>0<z>>>0?J:K;O=(N|0)>(y|0)|(N|0)==(y|0)&M>>>0>z>>>0?I:K;if((L-O|0)>1){I=O;J=L}else{P=O;Q=L;break}}}else{P=H;Q=F}J=f&1;I=(J|0)!=0;L=I?P:Q;O=(f&4|0)==0;L17527:do{if(O){if(!((L|0)>-1&(L|0)<(D|0))){R=L;break}K=(J<<1^2)-1|0;M=L;while(1){if((c[s+(M*24|0)+16>>2]&1|0)!=0){R=M;break L17527}N=K+M|0;if((N|0)>-1&(N|0)<(D|0)){M=N}else{R=N;break}}}else{R=L}}while(0);L=(R|0)==(D|0)?-1:R;do{if((L|0)<0){if(E){t=13049;break}M=s+8|0;K=c[M+4>>2]|0;if((K|0)>(y|0)|(K|0)==(y|0)&(c[M>>2]|0)>>>0>z>>>0){n=-1}else{t=13041;break}i=g;return n|0}else{if((L|0)!=(D-1|0)){S=L;break}if(E){t=13049}else{t=13041}}}while(0);do{if((t|0)==13041){if((s|0)==0){AN(0,0,1719600,(T=i,i=i+24|0,c[T>>2]=1728352,c[T+8>>2]=1712288,c[T+16>>2]=1924,T)|0);i=T;cw();return 0}E=D-1|0;L=s+(E*24|0)|0;M=il(c[a+16>>2]|0,c[L>>2]|0,c[L+4>>2]|0,0)|0;L=G;K=0;if((L|0)<(K|0)|(L|0)==(K|0)&M>>>0<0>>>0){B=M;break L17515}M=s+(E*24|0)+8|0;E=c[M>>2]|0;K=c[M+4>>2]|0;M=a+24|0;if((c[M>>2]|0)==0){t=13048;break}L=o+40|0;N=o+44|0;U=0;while(1){V=c[(c[p>>2]|0)+(U<<2)>>2]|0;W=c[V+44>>2]|0;X=c[L>>2]|0;Y=N2(X,(X|0)<0|0?-1:0,W,(W|0)<0|0?-1:0)|0;W=G;X=c[V+40>>2]|0;Z=c[N>>2]|0;_=N2(Z,(Z|0)<0|0?-1:0,X,(X|0)<0|0?-1:0)|0;X=AW(E,K,Y,W,_,G)|0;_=V+208|0;c[_>>2]=X;c[_+4>>2]=G;_=U+1|0;if(_>>>0<(c[M>>2]|0)>>>0){U=_}else{t=13048;break}}}else if((t|0)==13049){U=a+1232|0;M=il(c[a+16>>2]|0,c[U>>2]|0,c[U+4>>2]|0,0)|0;U=G;K=0;if(!((U|0)<(K|0)|(U|0)==(K|0)&M>>>0<0>>>0)){t=13048;break}B=M;break L17515}}while(0);do{if((t|0)==13048){s=h+32|0;D=h+16|0;M=h+36|0;K=o+8|0;U=0;L17555:while(1){while(1){E=mn(a,h)|0;if((E|0)==-11){continue}if((E|0)<0){break L17555}nJ(h);if((c[s>>2]|0)!=(x|0)){continue}E=c[D+4>>2]|0;if((E|0)>(y|0)|(E|0)==(y|0)&(c[D>>2]|0)>>>0>z>>>0){break}}if((c[M>>2]&1|0)!=0){break}$=U+1|0;if((U|0)<=1e3){U=$;continue}if((c[(c[K>>2]|0)+48>>2]|0)==133){U=$}else{t=13059;break}}if((t|0)==13059){AN(a,16,1727616,(T=i,i=i+8|0,c[T>>2]=$,T)|0);i=T}U=c[m>>2]|0;K=c[C>>2]|0;if((K|0)==0){aa=0;ab=-1}else{M=K-1|0;D=U+(M*24|0)+8|0;s=c[D+4>>2]|0;aa=K;ab=(s|0)<(y|0)|(s|0)==(y|0)&(c[D>>2]|0)>>>0<z>>>0?M:-1}if((aa-ab|0)>1){M=ab;D=aa;while(1){s=M+D>>1;E=U+(s*24|0)+8|0;N=c[E>>2]|0;L=c[E+4>>2]|0;E=(L|0)<(y|0)|(L|0)==(y|0)&N>>>0<z>>>0?D:s;_=(L|0)>(y|0)|(L|0)==(y|0)&N>>>0>z>>>0?M:s;if((E-_|0)>1){M=_;D=E}else{ac=_;ad=E;break}}}else{ac=ab;ad=aa}D=I?ac:ad;L17575:do{if(O){if(!((D|0)>-1&(D|0)<(K|0))){ae=D;break}M=(J<<1^2)-1|0;E=D;while(1){if((c[U+(E*24|0)+16>>2]&1|0)!=0){ae=E;break L17575}_=M+E|0;if((_|0)>-1&(_|0)<(K|0)){E=_}else{ae=_;break}}}else{ae=D}}while(0);D=(ae|0)==(K|0)?-1:ae;if((D|0)<0){n=-1}else{S=D;break}i=g;return n|0}}while(0);mq(a);J=c[(c[k>>2]|0)+52>>2]|0;if((J|0)!=0){if((c4[J&63](a,x,z,y,f)|0)>-1){break L17480}}J=c[m>>2]|0;O=J+(S*24|0)|0;I=il(c[a+16>>2]|0,c[O>>2]|0,c[O+4>>2]|0,0)|0;O=G;C=0;if((O|0)<(C|0)|(O|0)==(C|0)&I>>>0<0>>>0){B=I;break}I=J+(S*24|0)+8|0;J=c[I>>2]|0;C=c[I+4>>2]|0;I=a+24|0;if((c[I>>2]|0)==0){break L17480}O=o+40|0;D=o+44|0;U=0;while(1){E=c[(c[p>>2]|0)+(U<<2)>>2]|0;M=c[E+44>>2]|0;_=c[O>>2]|0;s=N2(_,(_|0)<0|0?-1:0,M,(M|0)<0|0?-1:0)|0;M=G;_=c[E+40>>2]|0;N=c[D>>2]|0;L=N2(N,(N|0)<0|0?-1:0,_,(_|0)<0|0?-1:0)|0;_=AW(J,C,s,M,L,G)|0;L=E+208|0;c[L>>2]=_;c[L+4>>2]=G;U=U+1|0;if(U>>>0>=(c[I>>2]|0)>>>0){break L17480}}}}while(0);if((B|0)>-1){break}else{n=B}i=g;return n|0}else{if((c[l+8>>2]&32768|0)!=0){n=-1;i=g;return n|0}mq(a);q=a+1232|0;I=c[q>>2]|0;U=c[q+4>>2]|0;q=a+16|0;C=ip(c[q>>2]|0)|0;J=NU(C,G,-1,-1)|0;C=G;if((U|0)>(e|0)|(U|0)==(e|0)&I>>>0>d>>>0){af=U;ag=I}else{I=(C|0)<(e|0)|(C|0)==(e|0)&J>>>0<d>>>0;af=I?C:e;ag=I?J:d}J=c[q>>2]|0;il(J,ag,af,0)|0;c[a+1280>>2]=1}}while(0);af=j|0;j=a+24|0;if((c[j>>2]|0)==0){n=0;i=g;return n|0}ag=a+28|0;d=a+1240|0;e=a+1244|0;a=0;L17599:while(1){l=c[(c[ag>>2]|0)+(a<<2)>>2]|0;do{if((c[l+72>>2]&1024|0)!=0){if((c[l+76>>2]|0)>=48){break}B=c[l+104>>2]|0;S=l+108|0;NN(af|0,S|0,76)|0;S=zi(B)|0;if((S|0)==0){n=-12;t=13090;break L17599}B=A7(88)|0;f=B;if((B|0)==0){break}if((c[d>>2]|0)==0){c[d>>2]=f}else{c[(c[e>>2]|0)+80>>2]=f}c[e>>2]=f;c[B>>2]=S;S=B+4|0;NN(S|0,af|0,76)|0}}while(0);l=a+1|0;if(l>>>0<(c[j>>2]|0)>>>0){a=l}else{n=0;t=13094;break}}if((t|0)==13090){i=g;return n|0}else if((t|0)==13094){i=g;return n|0}return 0}function my(a,b,d,e,f,g,h,j,k){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0;l=i;i=i+88|0;m=l|0;n=l+80|0;o=n;p=i;i=i+8|0;if((e|0)>(g|0)|(e|0)==(g|0)&d>>>0>f>>>0|((j|0)<(g|0)|(j|0)==(g|0)&h>>>0<f>>>0)){q=-1;i=l;return q|0}if((b|0)<-1){q=-22;i=l;return q|0}r=a+24|0;if((c[r>>2]|0)<=(b|0)){q=-22;i=l;return q|0}s=((c[a+1212>>2]|0)>0?k|4:k)&-2;k=a+4|0;if((c[(c[k>>2]|0)+68>>2]|0)==0){t=NV(f,g,d,e)|0;u=G;v=NV(h,j,f,g)|0;w=G;x=u>>>0>w>>>0|u>>>0==w>>>0&t>>>0>v>>>0;v=s|x&1;t=mx(a,b,f,g,v)|0;if((t|0)>-1|(f|0)==(d|0)&(g|0)==(e|0)|(h|0)==(f|0)&(j|0)==(g|0)){q=t;i=l;return q|0}t=mx(a,b,x?h:d,x?j:e,v)|0;if((t|0)<=-1){q=t;i=l;return q|0}q=mx(a,b,f,g,v^1)|0;i=l;return q|0}mq(a);do{if((b|0)==-1){if((c[r>>2]|0)!=1){y=j;z=h;A=g;B=f;C=e;D=d;break}v=(c[c[a+28>>2]>>2]|0)+40|0;t=c[v+4>>2]|0;c[n>>2]=c[v>>2];c[n+4>>2]=t;c[p>>2]=1;c[p+4>>2]=1e6;t=AY(f,g,p,o)|0;v=G;x=c[o+4>>2]|0;w=x;u=(x|0)<0|0?-1:0;x=c[n>>2]|0;E=N2(x,(x|0)<0|0?-1:0,1e6,0)|0;x=G;F=AV(d,e,w,u,E,x,8195)|0;H=G;I=AV(h,j,w,u,E,x,8194)|0;y=G;z=I;A=v;B=t;C=H;D=F}else{y=j;z=h;A=g;B=f;C=e;D=d}}while(0);d=db[c[(c[k>>2]|0)+68>>2]&1](a,b,D,C,B,A,z,y,s)|0;if((d|0)<=-1){q=d;i=l;return q|0}d=m|0;if((c[r>>2]|0)==0){q=0;i=l;return q|0}m=a+28|0;s=a+1240|0;y=a+1244|0;a=0;L17644:while(1){z=c[(c[m>>2]|0)+(a<<2)>>2]|0;do{if((c[z+72>>2]&1024|0)!=0){if((c[z+76>>2]|0)>=48){break}A=c[z+104>>2]|0;B=z+108|0;NN(d|0,B|0,76)|0;B=zi(A)|0;if((B|0)==0){q=-12;J=13125;break L17644}A=A7(88)|0;C=A;if((A|0)==0){break}if((c[s>>2]|0)==0){c[s>>2]=C}else{c[(c[y>>2]|0)+80>>2]=C}c[y>>2]=C;c[A>>2]=B;B=A+4|0;NN(B|0,d|0,76)|0}}while(0);z=a+1|0;if(z>>>0<(c[r>>2]|0)>>>0){a=z}else{q=0;J=13124;break}}if((J|0)==13125){i=l;return q|0}else if((J|0)==13124){i=l;return q|0}return 0}function mz(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=a;while(1){a=c[d>>2]|0;if((a|0)==0){e=0;f=13140;break}if((a|0)==(b|0)){break}else{d=d+8|0}}if((f|0)==13140){return e|0}e=c[d+4>>2]|0;return e|0}function mA(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0;d=0;while(1){e=c[a+(d<<3)>>2]|0;if((e|0)==0){f=0;break}if((c[a+(d<<3)+4>>2]|0)==(b|0)){g=e;h=13149;break}else{d=d+1|0}}if((h|0)==13149){return g|0}while(1){i=a+(f<<3)|0;if((c[i>>2]|0)==0){g=0;h=13148;break}d=pv(b)|0;if((d|0)==(pv(c[a+(f<<3)+4>>2]|0)|0)){break}else{f=f+1|0}}if((h|0)==13148){return g|0}g=c[i>>2]|0;return g|0}function mB(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0;do{if((b|0)==0){e=a+7>>3;if((1<<e-1&d|0)==0){if((e|0)==3){f=(c|0)!=0?65551:65550;break}else if((e|0)==4){f=(c|0)!=0?65547:65546;break}else if((e|0)==2){f=(c|0)!=0?65539:65538;break}else if((e|0)==1){f=65541;break}else{f=0;break}}else{if((e|0)==2){f=(c|0)!=0?65537:65536;break}else if((e|0)==3){f=(c|0)!=0?65549:65548;break}else if((e|0)==4){f=(c|0)!=0?65545:65544;break}else if((e|0)==1){f=65540;break}else{f=0;break}}}else{if((a|0)==32){f=(c|0)!=0?65556:65557;break}else if((a|0)==64){f=(c|0)!=0?65558:65559;break}else{f=0;break}}}while(0);return f|0}function mC(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0;if((a|0)==0){d=0;return d|0}e=c[a>>2]|0;if((e|0)==0){d=0;return d|0}else{f=0;g=e}L17707:while(1){h=g;while(1){e=c[h>>2]|0;if((e|0)==0){break}if((e|0)==(b|0)){break L17707}else{h=h+8|0}}e=f+1|0;i=c[a+(e<<2)>>2]|0;if((i|0)==0){d=0;j=13177;break}else{f=e;g=i}}if((j|0)==13177){return d|0}d=c[h+4>>2]|0;return d|0}function mD(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;if((a|0)==0){e=0;return e|0}else{f=0}L17720:while(1){g=c[a+(f<<2)>>2]|0;if((g|0)==0){e=0;h=13187;break}else{i=g}while(1){g=c[i>>2]|0;if((g|0)==0){break}if((g|0)==(b|0)){break L17720}else{i=i+8|0}}f=f+1|0}if((h|0)==13187){return e|0}c[d>>2]=c[i+4>>2];e=1;return e|0}function mE(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;if((a|0)==0){d=0;return d|0}else{e=0}while(1){f=c[a+(e<<2)>>2]|0;if((f|0)==0){d=0;g=13199;break}else{h=0}while(1){i=c[f+(h<<3)>>2]|0;if((i|0)==0){j=0;g=13193;break}if((c[f+(h<<3)+4>>2]|0)==(b|0)){k=i;g=13196;break}else{h=h+1|0}}L17738:do{if((g|0)==13193){while(1){g=0;l=f+(j<<3)|0;if((c[l>>2]|0)==0){break L17738}i=pv(b)|0;if((i|0)==(pv(c[f+(j<<3)+4>>2]|0)|0)){break}else{j=j+1|0;g=13193}}k=c[l>>2]|0;g=13196}}while(0);if((g|0)==13196){g=0;if((k|0)!=0){d=k;g=13201;break}}e=e+1|0}if((g|0)==13201){return d|0}else if((g|0)==13199){return d|0}return 0}function mF(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a9=0,ba=0,bb=0.0,bc=0,bd=0,be=0,bf=0.0,bg=0.0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0.0,bF=0,bG=0,bH=0.0,bI=0.0,bJ=0,bK=0,bL=0,bM=0,bN=0.0,bO=0.0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0,cb=0,cc=0,cd=0,ce=0,cf=0;e=i;i=i+656|0;f=e|0;g=e+24|0;j=e+32|0;k=e+40|0;l=e+48|0;m=e+56|0;n=e+64|0;o=e+72|0;p=e+80|0;q=e+160|0;r=e+168|0;s=e+248|0;t=e+256|0;u=e+264|0;v=e+272|0;w=e+280|0;x=e+288|0;y=e+296|0;z=e+304|0;A=e+312|0;B=e+392|0;C=e+400|0;D=a+16|0;E=il(c[D>>2]|0,0,0,1)|0;F=G;H=a+24|0;I=c[H>>2]|0;J=a+1088|0;K=(c[J>>2]|0)==0;L=c[D>>2]|0;if((L|0)==0){M=I}else{N=il(L,0,0,1)|0;L=G;AN(a,48,1689112,(O=i,i=i+16|0,c[O>>2]=N,c[O+8>>2]=L,O)|0);i=O;M=c[H>>2]|0}do{if((M|0)==0){P=13207}else{L=a+28|0;N=a+1084|0;Q=a;S=(b|0)!=0;T=0;do{c[s>>2]=0;U=c[(c[L>>2]|0)+(T<<2)>>2]|0;V=U+8|0;W=c[V>>2]|0;X=c[W+8>>2]|0;do{if((X|0)==0|(X|0)==3){Y=W+104|0;if((c[Y>>2]|0)!=0){break}Z=U+40|0;_=Y;Y=c[Z+4>>2]|0;c[_>>2]=c[Z>>2];c[_+4>>2]=Y}}while(0);W=U+240|0;do{if((c[W>>2]|0)==0){if((c[N>>2]&32|0)!=0){break}X=oE(c[(c[V>>2]|0)+48>>2]|0)|0;c[W>>2]=X;Y=c[U+236>>2]|0;if((X|0)==0){if((Y|0)==0){break}_=pn(c[(c[V>>2]|0)+48>>2]|0)|0;AN(Q,40,1686352,(O=i,i=i+8|0,c[O>>2]=_,O)|0);i=O;break}if((Y|0)==2){_=X+176|0;c[_>>2]=c[_>>2]|1;break}else if((Y|0)==1463898624){Y=X+176|0;c[Y>>2]=c[Y>>2]|4096;break}else{break}}}while(0);W=c[V>>2]|0;Y=c[W+12>>2]|0;if((Y|0)==0){$=o9(c[W+48>>2]|0)|0}else{$=Y}if(S){aa=b+(T<<2)|0}else{aa=s}zK(aa,1684736,1682848,0)|0;Y=c[V>>2]|0;W=($|0)==0;do{if(!((c[Y+8>>2]|0)!=3|W)){if((c[Y+12>>2]|0)!=0){break}if(S){ab=b+(T<<2)|0}else{ab=s}o5(Y,$,ab)|0}}while(0);do{if((mG(U,0)|0)==0){if((c[U+440>>2]|0)>0|W){break}Y=c[V>>2]|0;if((c[Y+12>>2]|0)!=0){break}if(S){ad=b+(T<<2)|0}else{ad=s}o5(Y,$,ad)|0}}while(0);if(!S){zL(s)}T=T+1|0;ae=c[H>>2]|0;}while(T>>>0<ae>>>0);if((ae|0)==0){P=13207;break}T=a+28|0;S=0;while(1){Q=c[(c[(c[T>>2]|0)+(S<<2)>>2]|0)+184>>2]|0;c[Q>>2]=0;c[Q+4>>2]=-2147483648;Q=(c[(c[(c[T>>2]|0)+(S<<2)>>2]|0)+184>>2]|0)+56|0;c[Q>>2]=0;c[Q+4>>2]=-2147483648;Q=(c[(c[(c[T>>2]|0)+(S<<2)>>2]|0)+184>>2]|0)+72|0;c[Q>>2]=0;c[Q+4>>2]=-2147483648;Q=S+1|0;if(Q>>>0<(c[H>>2]|0)>>>0){S=Q}else{af=T;break}}}}while(0);if((P|0)==13207){af=a+28|0}ae=a+1160|0;s=a+20|0;ad=a+1152|0;$=a+1084|0;ab=a+1220|0;aa=a+1224|0;M=r;T=t|0;S=t+4|0;Q=u|0;N=u+4|0;L=v|0;V=v+4|0;W=w|0;U=w+4|0;Y=x|0;X=x+4|0;_=y|0;Z=y+4|0;ag=z|0;ah=z+4|0;ai=a+1092|0;aj=f|0;ak=f+4|0;al=f+8|0;am=f+12|0;an=f+16|0;ao=f+20|0;ap=(b|0)!=0;aq=a;ar=0;as=0;at=0;L17803:while(1){do{if((ib(ae)|0)!=0){P=13242;break L17803}if((c[H>>2]|0)==0){au=0;P=13259}else{av=0;L17809:while(1){aw=c[(c[af>>2]|0)+(av<<2)>>2]|0;if((mG(aw,0)|0)==0){P=13257;break}ax=c[ad>>2]|0;ay=(c[aw+72>>2]&1024|0)==0?(ax|0)>-1?ax:+(c[aw+40>>2]|0)/+(c[aw+44>>2]|0)>5.0e-4?40:20:0;ax=c[aw+8>>2]|0;az=c[ax+108>>2]|0;aA=c[ax+104>>2]|0;do{if((az|0)>=(aA*101|0|0)|(az|0)<(aA*5|0|0)){P=13247}else{if((c[ax+52>>2]|0)==1983148141){P=13247;break}aB=c[ax+48>>2]|0;if((aB|0)==2|(aB|0)==28){P=13247}}}while(0);do{if((P|0)==13247){P=0;if((c[aw+412>>2]|0)!=0){if((c[aw+92>>2]|0)!=0){break}}if((c[(c[aw+184>>2]|0)+16>>2]|0)>=(ay|0)){break}if((c[ax+8>>2]|0)==0){P=13257;break L17809}}}while(0);ay=c[aw+240>>2]|0;do{if((ay|0)!=0){if((c[(c[ay+4>>2]|0)+36>>2]|0)==0){break}if((c[ax+96>>2]|0)==0){P=13257;break L17809}}}while(0);ay=aw+200|0;if((c[ay>>2]|0)==0&(c[ay+4>>2]|0)==(-2147483648|0)){if((c[ax+8>>2]|0)>>>0<2){P=13257;break}}ay=av+1|0;aA=c[H>>2]|0;if(ay>>>0<aA>>>0){av=ay}else{aC=ay;aD=aA;break}}if((P|0)==13257){P=0;aC=av;aD=c[H>>2]|0}if((aC|0)==(aD|0)){au=aC;P=13259}else{aE=aC}}if((P|0)==13259){P=0;if((c[s>>2]&1|0)==0){P=13260;break L17803}else{aE=au}}aF=c[J>>2]|0;aA=0;if(!((as|0)<(aA|0)|(as|0)==(aA|0)&at>>>0<aF>>>0)){P=13262;break L17803}aG=mo(a,r)|0;}while((aG|0)==-11);if((aG|0)<0){aH=aG;P=13328;break}if((c[$>>2]&64|0)==0){aA=A7(88)|0;ay=aA;if((aA|0)==0){aI=-12;break}if((c[ab>>2]|0)==0){c[ab>>2]=ay}else{c[(c[aa>>2]|0)+80>>2]=ay}c[aa>>2]=ay;ay=aA;NN(aA|0,M|0,80)|0;aA=nH(ay)|0;if((aA|0)<0){aI=aA;break}else{aJ=aA;aK=ay}}else{aJ=aG;aK=r}ay=aK+28|0;aA=c[ay>>2]|0;az=NU(aA,(aA|0)<0|0?-1:0,at,as)|0;aA=G;aB=c[(c[af>>2]|0)+(c[aK+32>>2]<<2)>>2]|0;aL=aK+16|0;aM=c[aL>>2]|0;aN=c[aL+4>>2]|0;aO=aB+232|0;do{if(!((aM|0)==0&(aN|0)==(-2147483648|0))){aP=c[aO>>2]|0;if((aP|0)<=1){break}aQ=aB+184|0;aR=c[aQ>>2]|0;aS=aR+72|0;aT=c[aS>>2]|0;aU=c[aS+4>>2]|0;if((aT|0)==0&(aU|0)==(-2147483648|0)|((aU|0)<(aN|0)|(aU|0)==(aN|0)&aT>>>0<aM>>>0)){aV=aR;aW=aU;aX=aT}else{aS=c[aR+80>>2]|0;AN(aq,48,1673384,(O=i,i=i+56|0,c[O>>2]=c[aB>>2],c[O+8>>2]=aS,c[O+16>>2]=aT,c[O+24>>2]=aU,c[O+32>>2]=aP,c[O+40>>2]=aM,c[O+48>>2]=aN,O)|0);i=O;aP=(c[aQ>>2]|0)+72|0;c[aP>>2]=0;c[aP+4>>2]=-2147483648;aP=(c[aQ>>2]|0)+56|0;c[aP>>2]=0;c[aP+4>>2]=-2147483648;aP=c[aQ>>2]|0;aU=aP+72|0;aV=aP;aW=c[aU+4>>2]|0;aX=c[aU>>2]|0}do{if((aX|0)==0&(aW|0)==(-2147483648|0)){aY=aV}else{aU=c[aV+80>>2]|0;aP=c[aV+64>>2]|0;if((aU|0)<=(aP|0)){aY=aV;break}aT=c[aL>>2]|0;aS=c[aL+4>>2]|0;aR=NV(aT,aS,aX,aW)|0;aZ=N0(aR,G,1e3,0)|0;aR=G;a_=aV+56|0;a$=NV(aX,aW,c[a_>>2]|0,c[a_+4>>2]|0)|0;a_=aU-aP|0;aP=N0(a$,G,a_,(a_|0)<0|0?-1:0)|0;a_=G;if(!((aR|0)>(a_|0)|(aR|0)==(a_|0)&aZ>>>0>aP>>>0)){aY=aV;break}aP=c[aO>>2]|0;AN(aq,24,1671512,(O=i,i=i+56|0,c[O>>2]=c[aB>>2],c[O+8>>2]=aU,c[O+16>>2]=aX,c[O+24>>2]=aW,c[O+32>>2]=aP,c[O+40>>2]=aT,c[O+48>>2]=aS,O)|0);i=O;aS=(c[aQ>>2]|0)+72|0;c[aS>>2]=0;c[aS+4>>2]=-2147483648;aS=(c[aQ>>2]|0)+56|0;c[aS>>2]=0;c[aS+4>>2]=-2147483648;aY=c[aQ>>2]|0}}while(0);av=aY+56|0;if((c[av>>2]|0)==0&(c[av+4>>2]|0)==(-2147483648|0)){aS=c[aL+4>>2]|0;c[av>>2]=c[aL>>2];c[av+4>>2]=aS;c[(c[aQ>>2]|0)+64>>2]=c[aO>>2];a0=c[aQ>>2]|0}else{a0=aY}aS=c[aL+4>>2]|0;av=a0+72|0;c[av>>2]=c[aL>>2];c[av+4>>2]=aS;c[(c[aQ>>2]|0)+80>>2]=c[aO>>2]}}while(0);do{if((c[aO>>2]|0)>1){aN=aB+40|0;if((c[aB+44>>2]|0)>0){aM=(c[aB+184>>2]|0)+24|0;aS=c[aM>>2]|0;av=c[aM+4>>2]|0;c[T>>2]=1;c[S>>2]=1e6;aM=AY(aS,av,aN,t)|0;a1=G;a2=aM}else{a1=0;a2=0}aM=aB+92|0;av=c[aM>>2]|0;do{if((av|0)>0){aS=c[aO>>2]|0;aT=aB+96|0;c[Q>>2]=d[aT]|d[aT+1|0]<<8|d[aT+2|0]<<16|d[aT+3|0]<<24;c[N>>2]=av;c[L>>2]=1;c[V>>2]=1e6;aP=AY(aS,(aS|0)<0|0?-1:0,u,v)|0;aS=G;if((a1|0)>(aS|0)|(a1|0)==(aS|0)&a2>>>0>aP>>>0){a3=a1;a4=a2;break}aP=c[aO>>2]|0;aS=d[aM]|d[aM+1|0]<<8|d[aM+2|0]<<16|d[aM+3|0]<<24|0;c[W>>2]=d[aT]|d[aT+1|0]<<8|d[aT+2|0]<<16|d[aT+3|0]<<24;c[U>>2]=aS;c[Y>>2]=1;c[X>>2]=1e6;aS=AY(aP,(aP|0)<0|0?-1:0,w,x)|0;a3=G;a4=aS}else{a3=a1;a4=a2}}while(0);do{if((a4|0)==0&(a3|0)==0){if((c[aO>>2]|0)<=30){a5=0;a6=0;break}aM=aB+184|0;av=c[aM>>2]|0;aQ=av+56|0;aS=c[aQ>>2]|0;aP=c[aQ+4>>2]|0;if((aS|0)==0&(aP|0)==(-2147483648|0)){a5=0;a6=0;break}aQ=av+72|0;av=c[aQ>>2]|0;aT=c[aQ+4>>2]|0;if((av|0)==0&(aT|0)==(-2147483648|0)){a5=0;a6=0;break}aQ=NV(av,aT,aS,aP)|0;c[_>>2]=1;c[Z>>2]=1e6;aP=AY(aQ,G,aN,y)|0;aQ=G;aS=0;if((aQ|0)<(aS|0)|(aQ|0)==(aS|0)&aP>>>0<0>>>0){a5=0;a6=0;break}aP=c[aM>>2]|0;aM=aP+72|0;aS=aP+56|0;aP=NV(c[aM>>2]|0,c[aM+4>>2]|0,c[aS>>2]|0,c[aS+4>>2]|0)|0;c[ag>>2]=1;c[ah>>2]=1e6;aS=AY(aP,G,aN,z)|0;a5=G;a6=aS}else{a5=a3;a6=a4}}while(0);a7=c[ai>>2]|0;aN=(a7|0)<0|0?-1:0;if(!((a5|0)<(aN|0)|(a5|0)==(aN|0)&a6>>>0<a7>>>0)){P=13299;break L17803}aN=c[aK+48>>2]|0;if((aN|0)==0){break}aS=aB+184|0;aP=(c[aS>>2]|0)+24|0;aM=NU(c[aP>>2]|0,c[aP+4>>2]|0,aN,(aN|0)<0|0?-1:0)|0;c[aP>>2]=aM;c[aP+4>>2]=G;aP=c[aB+240>>2]|0;do{if((aP|0)==0){a9=0;ba=2}else{if((c[(c[aB+8>>2]|0)+112>>2]|0)!=2){a9=0;ba=2;break}aM=(c[aP+36>>2]|0)+1|0;a9=(aM|0)<0|0?-1:0;ba=aM}}while(0);aP=(c[aS>>2]|0)+32|0;aM=NU(c[aP>>2]|0,c[aP+4>>2]|0,ba,a9)|0;c[aP>>2]=aM;c[aP+4>>2]=G}}while(0);aP=aB+184|0;aM=c[aP>>2]|0;aN=aM|0;aQ=c[aN>>2]|0;aT=c[aN+4>>2]|0;aN=c[aL>>2]|0;av=c[aL+4>>2]|0;aU=NV(aN,av,aQ,aT)|0;aZ=G;a_=2147483647;do{if(((aQ|0)!=0|(aT|0)!=(-2147483648|0))&((aN|0)!=0|(av|0)!=(-2147483648|0))&((av|0)>(aT|0)|(av|0)==(aT|0)&aN>>>0>aQ>>>0)&(aZ>>>0<a_>>>0|aZ>>>0==a_>>>0&aU>>>0<-1>>>0)){aR=2147352575;a$=(av|0)>(aR|0)|(av|0)==(aR|0)&aN>>>0>-1>>>0;aR=NU(aN,av,1,-2147418112)|0;bb=(+((a$?aR:aN)>>>0)+ +((a$?G:av)|0)*4294967296.0)*(+(c[aB+40>>2]|0)/+(c[aB+44>>2]|0));if((c[aM+20>>2]|0)==0){a$=A7(23232)|0;c[(c[aP>>2]|0)+20>>2]=a$;if((c[(c[aP>>2]|0)+20>>2]|0)==0){bc=-12;P=13529;break L17803}else{bd=0}}else{bd=0}do{if((bd|0)<720){be=(bd*1001|0)+1001|0}else{c[aj>>2]=24;c[ak>>2]=30;c[al>>2]=60;c[am>>2]=12;c[an>>2]=15;c[ao>>2]=48;be=(c[f+(bd-720<<2)>>2]|0)*12e3|0}bf=bb*+(be|0)/12012.0;a$=cS(+(bf+0.0))|0;bg=bf-(+(a$>>>0)+ +(G|0)*4294967296.0)+0.0;a$=(c[(c[aP>>2]|0)+20>>2]|0)+(bd<<3)|0;h[a$>>3]=+h[a$>>3]+bg;a$=(c[(c[aP>>2]|0)+20>>2]|0)+5808+(bd<<3)|0;h[a$>>3]=bg*bg+ +h[a$>>3];a$=cS(+(bf+.5))|0;bg=bf-(+(a$>>>0)+ +(G|0)*4294967296.0)+.5;a$=(c[(c[aP>>2]|0)+20>>2]|0)+11616+(bd<<3)|0;h[a$>>3]=+h[a$>>3]+bg;a$=(c[(c[aP>>2]|0)+20>>2]|0)+17424+(bd<<3)|0;h[a$>>3]=bg*bg+ +h[a$>>3];bd=bd+1|0;}while((bd|0)<726);aS=(c[aP>>2]|0)+16|0;c[aS>>2]=(c[aS>>2]|0)+1;aS=c[aP>>2]|0;if((c[aS+16>>2]|0)<=3){bh=726;P=13315;break}a$=c[aL>>2]|0;aR=c[aL+4>>2]|0;bi=2147352575;bj=2147352575;if(((aR|0)>(bi|0)|(aR|0)==(bi|0)&a$>>>0>-1>>>0)^((aT|0)>(bj|0)|(aT|0)==(bj|0)&aQ>>>0>-1>>>0)){bk=726;bl=aR;bm=a$;break}a$=aS+8|0;aS=AU(c[a$>>2]|0,c[a$+4>>2]|0,aU,aZ)|0;c[a$>>2]=aS;c[a$+4>>2]=G;bh=726;P=13315}else{bh=aE;P=13315}}while(0);if((P|0)==13315){P=0;bk=bh;bl=c[aL+4>>2]|0;bm=c[aL>>2]|0}if(!((bm|0)==0&(bl|0)==(-2147483648|0))){aZ=c[aP>>2]|0;c[aZ>>2]=bm;c[aZ+4>>2]=bl}aZ=c[aB+240>>2]|0;do{if((aZ|0)!=0){aU=c[(c[aZ+4>>2]|0)+36>>2]|0;if((aU|0)==0){break}aQ=aB+8|0;aT=c[aQ>>2]|0;if((c[aT+96>>2]|0)!=0){break}aM=aK+24|0;av=c6[aU&255](aT,c[aM>>2]|0,c[ay>>2]|0)|0;if((av-1|0)>>>0>=268435439){break}c[(c[aQ>>2]|0)+100>>2]=av;aT=A1((c[(c[aQ>>2]|0)+100>>2]|0)+16|0)|0;c[(c[aQ>>2]|0)+96>>2]=aT;aT=c[aQ>>2]|0;aU=c[aT+96>>2]|0;if((aU|0)==0){bc=-12;P=13526;break L17803}aN=c[aM>>2]|0;aM=c[aT+100>>2]|0;NN(aU|0,aN|0,aM)|0;NM((c[(c[aQ>>2]|0)+96>>2]|0)+av|0,0,16)}}while(0);if(ap&(bk|0)<(I|0)){bn=b+(bk<<2)|0}else{bn=0}mH(aB,aK,bn)|0;c[aO>>2]=(c[aO>>2]|0)+1;ar=ar+1|0;as=aA;at=az}do{if((P|0)==13526){i=e;return bc|0}else if((P|0)==13529){i=e;return bc|0}else if((P|0)==13260){AN(aq,48,1679112,(O=i,i=i+1|0,i=i+7&-8,c[O>>2]=0,O)|0);i=O;bo=ar;P=13329}else if((P|0)==13262){AN(aq,48,1677024,(O=i,i=i+8|0,c[O>>2]=aF,O)|0);i=O;at=c[H>>2]|0;if((at|0)==0){aH=ar;P=13328;break}else{bp=0;bq=at}while(1){at=c[(c[af>>2]|0)+(bp<<2)>>2]|0;do{if((c[at+412>>2]|0)==0){if((c[(c[at+184>>2]|0)+16>>2]|0)>=2){br=bq;break}AN(aq,24,1675184,(O=i,i=i+8|0,c[O>>2]=bp,O)|0);i=O;br=c[H>>2]|0}else{br=bq}}while(0);at=bp+1|0;if(at>>>0<br>>>0){bp=at;bq=br}else{aH=ar;P=13328;break}}}else if((P|0)==13242){AN(aq,48,1680832,(O=i,i=i+1|0,i=i+7&-8,c[O>>2]=0,O)|0);i=O;aH=-1414092869;P=13328}else if((P|0)==13299){AN(aq,40,1669624,(O=i,i=i+24|0,c[O>>2]=a7,c[O+8>>2]=a6,c[O+16>>2]=a5,O)|0);i=O;aH=aJ;P=13328}}while(0);do{if((P|0)==13328){if(K){bo=aH;P=13329;break}NM(A|0,0,80);nB(A);if((c[H>>2]|0)==0){bs=aH;P=13408;break}else{bt=0}while(1){aJ=c[(c[af>>2]|0)+(bt<<2)>>2]|0;L17931:do{if((c[(c[aJ+184>>2]|0)+40>>2]|0)==1){a5=ap&(bt|0)<(I|0);a6=b+(bt<<2)|0;while(1){bu=mH(aJ,A,a5?a6:0)|0;if((bu|0)<=0){break}if((mG(aJ,0)|0)!=0){break L17931}}if((bu|0)>=0){break}AN(aq,32,1668056,(O=i,i=i+8|0,c[O>>2]=c[aJ>>2],O)|0);i=O}}while(0);aJ=bt+1|0;a6=c[H>>2]|0;if(aJ>>>0<a6>>>0){bt=aJ}else{bv=aH;bw=a6;P=13330;break}}}}while(0);if((P|0)==13329){bv=bo;bw=c[H>>2]|0;P=13330}do{if((P|0)==13330){if((bw|0)==0){bs=bv;P=13408;break}else{bx=0}do{pk(c[(c[(c[af>>2]|0)+(bx<<2)>>2]|0)+8>>2]|0)|0;bx=bx+1|0;by=c[H>>2]|0;}while(bx>>>0<by>>>0);if((by|0)==0){bs=bv;P=13408;break}else{bz=0}while(1){bo=c[(c[af>>2]|0)+(bz<<2)>>2]|0;aH=bo+8|0;bt=c[aH>>2]|0;bu=c[bt+8>>2]|0;L17948:do{if((bu|0)==1){if((c[bt+792>>2]|0)==0){A=pc(c[bt+48>>2]|0)|0;c[(c[aH>>2]|0)+792>>2]=A;bA=c[aH>>2]|0}else{bA=bt}switch(c[bA+464>>2]|0){case 1:{c[bo+72>>2]=512;break L17948;break};case 2:{c[bo+72>>2]=256;break L17948;break};case 3:{c[bo+72>>2]=128;break L17948;break};case 5:{c[bo+72>>2]=8;break L17948;break};case 8:{c[bo+72>>2]=32;break L17948;break};default:{break L17948}}}else if((bu|0)==0){do{if((c[bt+48>>2]|0)==14){if((c[bt+52>>2]|0)!=0){break}if((c[bt+792>>2]|0)!=0){break}A=oM(c[bt+140>>2]|0)|0;b=oN(242800,A)|0;I=c[aH>>2]|0;if((b|0)!=(c[I+140>>2]|0)){break}c[I+52>>2]=A}}while(0);A=bo+184|0;I=c[A>>2]|0;b=I+32|0;ap=c[b>>2]|0;K=c[b+4>>2]|0;do{if(!((ap|0)==0&(K|0)==0)){b=bo+92|0;if((c[b>>2]|0)!=0){break}a6=I+24|0;aJ=c[a6>>2]|0;a5=c[a6+4>>2]|0;if((aJ|0)==0&(a5|0)==0){break}a6=bo+96|0;ax=c[bo+44>>2]|0;aw=N2(ax,(ax|0)<0|0?-1:0,ap,K)|0;ax=G;a7=c[bo+40>>2]|0;ar=N2(aJ<<1|0>>>31,a5<<1|aJ>>>31,a7,(a7|0)<0|0?-1:0)|0;a7=G;aJ=6e4;a5=0;BX(b,a6,aw,ax,ar,a7,aJ,a5)|0;a5=1;aJ=0;bb=.01;while(1){if((a5|0)<720){bB=(a5*1001|0)+1001|0}else{c[aj>>2]=24;c[ak>>2]=30;c[al>>2]=60;c[am>>2]=12;c[an>>2]=15;c[ao>>2]=48;bB=(c[f+(a5-720<<2)>>2]|0)*12e3|0}bg=+R(+(+(c[b>>2]|0)/+(c[a6>>2]|0)/(+(bB|0)/12012.0)+-1.0));a7=bg<bb;bC=a7?bB:aJ;ar=a5+1|0;if((ar|0)<726){a5=ar;aJ=bC;bb=a7?bg:bb}else{break}}if((bC|0)==0){break}BX(b,a6,bC,(bC|0)<0|0?-1:0,12012,0,2147483647,0)|0}}while(0);K=c[aH>>2]|0;ap=c[K+108>>2]|0;I=c[K+104>>2]|0;do{if((ap|0)>=(I*101|0|0)|(ap|0)<(I*5|0|0)){P=13360}else{if((c[K+52>>2]|0)==1983148141){P=13360;break}aJ=c[K+48>>2]|0;if((aJ|0)==2|(aJ|0)==28){P=13360}}}while(0);do{if((P|0)==13360){P=0;K=c[A>>2]|0;if((c[K+16>>2]|0)<=15){break}I=K+8|0;K=c[I>>2]|0;ap=c[I+4>>2]|0;I=c[bo+44>>2]|0;aJ=I;a5=(I|0)<0|0?-1:0;I=c[bo+40>>2]|0;a7=I;ar=(I|0)<0|0?-1:0;I=N2(a7,ar,500,0)|0;ax=N0(aJ,a5,I,G)|0;I=G;aw=0;br=(I|0)<(aw|0)|(I|0)==(aw|0)&ax>>>0<1>>>0;aw=br?0:I;if(!((ap|0)>(aw|0)|(ap|0)==(aw|0)&K>>>0>(br?1:ax)>>>0)){break}ax=bo+412|0;if((c[ax>>2]|0)!=0){break}br=bo+416|0;aw=N2(a7,ar,K,ap)|0;ap=G;K=2147483647;ar=0;BX(ax,br,aJ,a5,aw,ap,K,ar)|0}}while(0);ar=c[A>>2]|0;L17988:do{if((c[ar+16>>2]|0)>1){K=bo+412|0;if((c[K>>2]|0)!=0){break}ap=c[aH>>2]|0;aw=c[ap+108>>2]|0;a5=c[ap+104>>2]|0;do{if(!((aw|0)>=(a5*101|0|0)|(aw|0)<(a5*5|0|0))){if((c[ap+52>>2]|0)==1983148141){break}aJ=c[ap+48>>2]|0;if(!((aJ|0)==2|(aJ|0)==28)){break L17988}}}while(0);ap=bo+40|0;a5=bo+44|0;aw=0;a6=0;bb=.01;b=ar;while(1){aJ=b+24|0;br=c[aJ>>2]|0;ax=c[aJ+4>>2]|0;aJ=(br|0)==0&(ax|0)==0;do{if(aJ){P=13376}else{bg=(+(br>>>0)+ +(ax|0)*4294967296.0)*(+(c[ap>>2]|0)/+(c[a5>>2]|0));if((aw|0)<720){bD=(aw*1001|0)+1001|0}else{c[aj>>2]=24;c[ak>>2]=30;c[al>>2]=60;c[am>>2]=12;c[an>>2]=15;c[ao>>2]=48;bD=(c[f+(aw-720<<2)>>2]|0)*12e3|0}if(bg<12012.0/+(bD|0)){bE=bb;bF=a6;break}if(aJ){P=13376}else{P=13380}}}while(0);if((P|0)==13376){P=0;if((aw|0)<720){bG=(aw*1001|0)+1001|0}else{c[aj>>2]=24;c[ak>>2]=30;c[al>>2]=60;c[am>>2]=12;c[an>>2]=15;c[ao>>2]=48;bG=(c[f+(aw-720<<2)>>2]|0)*12e3|0}if(12012.0/+(bG|0)>1.0){bE=bb;bF=a6}else{P=13380}}do{if((P|0)==13380){P=0;aJ=(aw|0)<720;ax=(aw*1001|0)+1001|0;br=f+(aw-720<<2)|0;a7=c[b+16>>2]|0;I=c[b+20>>2]|0;bg=+(a7|0);bf=+h[I+(aw<<3)>>3]/bg;bH=+h[I+5808+(aw<<3)>>3]/bg-bf*bf;do{if(bH<bb&bb>1.0e-9){if(aJ){bI=bH;bJ=ax;break}c[aj>>2]=24;c[ak>>2]=30;c[al>>2]=60;c[am>>2]=12;c[an>>2]=15;c[ao>>2]=48;bI=bH;bJ=(c[br>>2]|0)*12e3|0}else{bI=bb;bJ=a6}}while(0);if(bH<.02){if(aJ){bK=ax}else{c[aj>>2]=24;c[ak>>2]=30;c[al>>2]=60;c[am>>2]=12;c[an>>2]=15;c[ao>>2]=48;bK=(c[br>>2]|0)*12e3|0}AN(0,48,1666744,(O=i,i=i+16|0,h[O>>3]=+(bK|0)/12.0/1001.0,h[O+8>>3]=bH,O)|0);i=O;bq=c[A>>2]|0;bL=c[bq+16>>2]|0;bM=c[bq+20>>2]|0}else{bL=a7;bM=I}bf=+(bL|0);bg=+h[bM+11616+(aw<<3)>>3]/bf;bN=+h[bM+17424+(aw<<3)>>3]/bf-bg*bg;do{if(bN<bI&bI>1.0e-9){if(aJ){bO=bN;bP=ax;break}c[aj>>2]=24;c[ak>>2]=30;c[al>>2]=60;c[am>>2]=12;c[an>>2]=15;c[ao>>2]=48;bO=bN;bP=(c[br>>2]|0)*12e3|0}else{bO=bI;bP=bJ}}while(0);if(bN>=.02){bE=bO;bF=bP;break}if(aJ){bQ=ax}else{c[aj>>2]=24;c[ak>>2]=30;c[al>>2]=60;c[am>>2]=12;c[an>>2]=15;c[ao>>2]=48;bQ=(c[br>>2]|0)*12e3|0}AN(0,48,1666744,(O=i,i=i+16|0,h[O>>3]=+(bQ|0)/12.0/1001.0,h[O+8>>3]=bN,O)|0);i=O;bE=bO;bF=bP}}while(0);I=aw+1|0;if((I|0)>=726){break}aw=I;a6=bF;bb=bE;b=c[A>>2]|0}if((bF|0)==0){break}b=c[K>>2]|0;if((b|0)==0){bR=bo+416|0}else{a6=bo+416|0;if(+(bF|0)/12012.0<+(b|0)/+(c[a6>>2]|0)*1.01){bR=a6}else{break}}BX(K,bR,bF,(bF|0)<0|0?-1:0,12012,0,2147483647,0)|0}}while(0);A=bo+412|0;if((c[A>>2]|0)!=0){break}ar=c[aH>>2]|0;a6=c[ar+108>>2]|0;b=c[bo+40>>2]|0;aw=N2(b,(b|0)<0|0?-1:0,a6,(a6|0)<0|0?-1:0)|0;a5=G;ap=ar+104|0;I=ar+112|0;ar=ac(c[I>>2]|0,c[ap>>2]|0)|0;a7=c[bo+44>>2]|0;bq=N2(ar,(ar|0)<0|0?-1:0,a7,(a7|0)<0|0?-1:0)|0;ar=G;if((a5|0)>(ar|0)|(a5|0)==(ar|0)&aw>>>0>bq>>>0){c[A>>2]=a7;c[bo+416>>2]=b;break}else{c[A>>2]=a6;c[bo+416>>2]=ac(c[I>>2]|0,c[ap>>2]|0)|0;break}}}while(0);bo=bz+1|0;if(bo>>>0<(c[H>>2]|0)>>>0){bz=bo}else{bs=bv;P=13408;break}}}}while(0);do{if((P|0)==13408){if((c[J>>2]|0)!=0){bv=a+4|0;bz=c[bv>>2]|0;if((c[bz+8>>2]&1|0)==0){bF=ip(c[D>>2]|0)|0;bR=G;bP=0;bQ=(bR|0)<(bP|0)|(bR|0)==(bP|0)&bF>>>0<0>>>0;bS=bQ?0:bR;bT=bQ?0:bF;bU=c[bv>>2]|0}else{bS=0;bT=0;bU=bz}bz=c[bU>>2]|0;if((a8(bz|0,1734976)|0)==0){if((bT|0)==0&(bS|0)==0){P=13458}else{P=13414}}else{if((a8(bz|0,1734704)|0)!=0|(bT|0)==0&(bS|0)==0){P=13458}else{P=13414}}do{if((P|0)==13414){if((c[(c[D>>2]|0)+88>>2]|0)==0){P=13458;break}bz=a+1248|0;bv=a+1252|0;bF=o;bQ=c[bz>>2]|0;if((bQ|0)!=0){bR=bQ;do{c[o>>2]=bR;c[bz>>2]=c[bR+80>>2];nJ(bR|0);A6(bF);bR=c[bz>>2]|0;}while((bR|0)!=0)}c[bv>>2]=0;bR=l;bz=c[ab>>2]|0;if((bz|0)!=0){bF=bz;do{c[l>>2]=bF;c[ab>>2]=c[bF+80>>2];nJ(bF|0);A6(bR);bF=c[ab>>2]|0;}while((bF|0)!=0)}c[aa>>2]=0;bF=a+1240|0;bR=a+1244|0;bv=k;bz=c[bF>>2]|0;if((bz|0)!=0){bQ=bz;do{c[k>>2]=bQ;c[bF>>2]=c[bQ+80>>2];nJ(bQ|0);A6(bv);bQ=c[bF>>2]|0;}while((bQ|0)!=0)}c[bR>>2]=0;c[a+1256>>2]=25e5;if((c[H>>2]|0)!=0){bQ=0;do{bF=c[(c[af>>2]|0)+(bQ<<2)>>2]|0;bv=bF+48|0;do{if((c[bv>>2]|0)==0&(c[bv+4>>2]|0)==(-2147483648|0)){bz=bF+200|0;if(!((c[bz>>2]|0)==0&(c[bz+4>>2]|0)==(-2147483648|0))){break}AN(c[bF+8>>2]|0,24,1733696,(O=i,i=i+1|0,i=i+7&-8,c[O>>2]=0,O)|0);i=O}}while(0);bv=bF+240|0;bz=c[bv>>2]|0;if((bz|0)!=0){oH(bz);c[bv>>2]=0}bQ=bQ+1|0;}while(bQ>>>0<(c[H>>2]|0)>>>0)}bQ=c[D>>2]|0;if((bQ|0)==0){bV=0;bW=0}else{bR=ip(bQ)|0;bV=G;bW=bR}bR=p+28|0;bQ=p+32|0;bv=p+8|0;bz=0;do{bP=NW(25e4,0,bz|0)|0;ao=G;an=NV(bW,bV,bP,ao)|0;am=G;al=0;ak=(am|0)<(al|0)|(am|0)==(al|0)&an>>>0<0>>>0;il(c[D>>2]|0,ak?0:an,ak?0:am,0)|0;am=bz-1|0;ak=(am|0)>0;an=NW(25e4,0,am|0)|0;am=ak?an:25e4;an=ak?G:0;ak=0;L18085:do{if((an|0)>(ak|0)|(an|0)==(ak|0)&am>>>0>0>>>0){al=0;aj=-2147483648;bJ=0;while(1){while(1){bM=mj(a,p)|0;if((bM|0)==0){break}else if((bM|0)!=(-11|0)){bX=aj;bY=bJ;break L18085}}bM=(c[bR>>2]|0)+al|0;bL=c[(c[af>>2]|0)+(c[bQ>>2]<<2)>>2]|0;bK=c[bv>>2]|0;f=c[bv+4>>2]|0;do{if((bK|0)==0&(f|0)==(-2147483648|0)){bZ=aj;b_=bJ}else{bG=bL+48|0;bD=c[bG>>2]|0;bC=c[bG+4>>2]|0;if((bD|0)==0&(bC|0)==(-2147483648|0)){bG=bL+200|0;bB=c[bG>>2]|0;bA=c[bG+4>>2]|0;if((bB|0)==0&(bA|0)==(-2147483648|0)){bZ=aj;b_=bJ;break}else{b$=bA;b0=bB}}else{b$=bC;b0=bD}bD=NV(bK,f,b0,b$)|0;bC=G;bB=0;if(!((bC|0)>(bB|0)|(bC|0)==(bB|0)&bD>>>0>0>>>0)){bZ=f;b_=bK;break}bB=bL+56|0;bA=c[bB>>2]|0;bG=c[bB+4>>2]|0;by=(c[bL+184>>2]|0)+48|0;do{if((bA|0)==0&(bG|0)==(-2147483648|0)){P=13442}else{bx=c[by>>2]|0;bw=c[by+4>>2]|0;bo=0;if((bw|0)<(bo|0)|(bw|0)==(bo|0)&bx>>>0<1>>>0){P=13442;break}if(!((bG|0)<(bC|0)|(bG|0)==(bC|0)&bA>>>0<bD>>>0)){break}bo=NV(bD,bC,bx,bw)|0;bw=G;bx=-1;aH=(bw|0)>(bx|0)|(bw|0)==(bx|0)&bo>>>0>-1>>>0;bx=NV(0,0,bo,bw)|0;bt=aH?bw:G;bw=c[bL+44>>2]|0;bu=N2(bw,(bw|0)<0|0?-1:0,60,0)|0;bw=c[bL+40>>2]|0;ap=N0(bu,G,bw,(bw|0)<0|0?-1:0)|0;bw=G;if((bt|0)<(bw|0)|(bt|0)==(bw|0)&(aH?bo:bx)>>>0<ap>>>0){P=13442}}}while(0);if((P|0)==13442){P=0;c[bB>>2]=bD;c[bB+4>>2]=bC}c[by>>2]=bD;c[by+4>>2]=bC;bZ=f;b_=bK}}while(0);nJ(p);bK=(bM|0)<0|0?-1:0;if((bK|0)<(an|0)|(bK|0)==(an|0)&bM>>>0<am>>>0){al=bM;aj=bZ;bJ=b_}else{bX=bZ;bY=b_;break}}}else{bX=-2147483648;bY=0}}while(0);if(!((bY|0)==0&(bX|0)==(-2147483648|0)&((bV|0)>(ao|0)|(bV|0)==(ao|0)&bW>>>0>bP>>>0))){break}bz=bz+1|0;}while((bz|0)<5);m0(a);if((c[H>>2]|0)!=0){bz=a+1056|0;bv=a+1064|0;bQ=n|0;bR=n+4|0;am=m|0;an=m+4|0;ak=0;do{bF=c[(c[af>>2]|0)+(ak<<2)>>2]|0;bJ=bF+48|0;do{if((c[bJ>>2]|0)==0&(c[bJ+4>>2]|0)==(-2147483648|0)){aj=c[bz>>2]|0;al=c[bz+4>>2]|0;if(!((aj|0)==0&(al|0)==(-2147483648|0))){c[am>>2]=1;c[an>>2]=1e6;K=AY(aj,al,m,bF+40|0)|0;c[bJ>>2]=K;c[bJ+4>>2]=G}K=c[bv>>2]|0;al=c[bv+4>>2]|0;if((K|0)==0&(al|0)==(-2147483648|0)){break}c[bQ>>2]=1;c[bR>>2]=1e6;aj=AY(K,al,n,bF+40|0)|0;al=bF+56|0;c[al>>2]=aj;c[al+4>>2]=G}}while(0);ak=ak+1|0;}while(ak>>>0<(c[H>>2]|0)>>>0)}il(c[D>>2]|0,E,F,0)|0;if((c[H>>2]|0)!=0){ak=0;do{bR=c[(c[af>>2]|0)+(ak<<2)>>2]|0;bQ=bR+200|0;bv=c[bQ+4>>2]|0;an=bR+208|0;c[an>>2]=c[bQ>>2];c[an+4>>2]=bv;bv=bR+216|0;c[bv>>2]=0;c[bv+4>>2]=-2147483648;bv=bR+192|0;c[bv>>2]=0;c[bv+4>>2]=-2147483648;ak=ak+1|0;}while(ak>>>0<(c[H>>2]|0)>>>0)}c[a+1200>>2]=0}}while(0);L18124:do{if((P|0)==13458){ak=c[H>>2]|0;bv=0;while(1){if(bv>>>0>=ak>>>0){P=13461;break}bR=(c[(c[af>>2]|0)+(bv<<2)>>2]|0)+56|0;if((c[bR>>2]|0)==0&(c[bR+4>>2]|0)==(-2147483648|0)){bv=bv+1|0}else{break}}do{if((P|0)==13461){bv=a+1064|0;if(!((c[bv>>2]|0)==0&(c[bv+4>>2]|0)==(-2147483648|0))){break}bv=a+1072|0;do{if((c[bv>>2]|0)<1){if((ak|0)==0){c[bv>>2]=0;break}bR=c[af>>2]|0;an=0;bQ=0;do{am=c[(c[(c[bR+(bQ<<2)>>2]|0)+8>>2]|0)+72>>2]|0;an=((am|0)>0?am:0)+an|0;bQ=bQ+1|0;}while(bQ>>>0<ak>>>0);c[bv>>2]=an;if((an|0)!=0){P=13477}}else{P=13477}}while(0);do{if((P|0)==13477){bQ=c[D>>2]|0;if((bQ|0)==0){break}bR=ip(bQ)|0;bQ=G;am=0;if(!((bQ|0)>(am|0)|(bQ|0)==(am|0)&bR>>>0>0>>>0)){break}am=c[H>>2]|0;if((am|0)==0){break}bz=bR<<3|0>>>29;bF=bQ<<3|bR>>>29;bR=0;bQ=0;bJ=am;while(1){am=c[(c[af>>2]|0)+(bR<<2)>>2]|0;bP=c[am+40>>2]|0;ao=bP;al=(bP|0)<0|0?-1:0;bP=c[bv>>2]|0;aj=bP;K=(bP|0)<0|0?-1:0;bP=N0(-1,2147483647,aj,K)|0;bK=G;do{if((al|0)>(bK|0)|(al|0)==(bK|0)&ao>>>0>bP>>>0){b1=bQ;b2=bJ}else{f=am+56|0;if(!((c[f>>2]|0)==0&(c[f+4>>2]|0)==(-2147483648|0))){b1=bQ;b2=bJ;break}bL=c[am+44>>2]|0;bA=N2(aj,K,ao,al)|0;bG=AW(bz,bF,bL,(bL|0)<0|0?-1:0,bA,G)|0;c[f>>2]=bG;c[f+4>>2]=G;b1=1;b2=c[H>>2]|0}}while(0);al=bR+1|0;if(al>>>0<b2>>>0){bR=al;bQ=b1;bJ=b2}else{break}}if((b1|0)==0){break}AN(aq,24,1734088,(O=i,i=i+1|0,i=i+7&-8,c[O>>2]=0,O)|0);i=O}}while(0);c[a+1200>>2]=2;break L18124}}while(0);m0(a);if((c[H>>2]|0)!=0){ak=a+1056|0;bv=a+1064|0;bJ=j|0;bQ=j+4|0;bR=g|0;bF=g+4|0;bz=0;do{an=c[(c[af>>2]|0)+(bz<<2)>>2]|0;al=an+48|0;do{if((c[al>>2]|0)==0&(c[al+4>>2]|0)==(-2147483648|0)){ao=c[ak>>2]|0;K=c[ak+4>>2]|0;if(!((ao|0)==0&(K|0)==(-2147483648|0))){c[bR>>2]=1;c[bF>>2]=1e6;aj=AY(ao,K,g,an+40|0)|0;c[al>>2]=aj;c[al+4>>2]=G}aj=c[bv>>2]|0;K=c[bv+4>>2]|0;if((aj|0)==0&(K|0)==(-2147483648|0)){break}c[bJ>>2]=1;c[bQ>>2]=1e6;ao=AY(aj,K,j,an+40|0)|0;K=an+56|0;c[K>>2]=ao;c[K+4>>2]=G}}while(0);bz=bz+1|0;}while(bz>>>0<(c[H>>2]|0)>>>0)}c[a+1200>>2]=1}}while(0);m0(a)}bz=c[H>>2]|0;if((bs|0)>-1){b3=(bz|0)==0?bs:-1}else{b3=bs}if((bz|0)==0){b4=b3}else{bz=C|0;bQ=b3;bJ=0;while(1){bv=c[(c[af>>2]|0)+(bJ<<2)>>2]|0;if((mG(bv,B)|0)==0){pp(bz,256,c[bv+8>>2]|0,0);bv=c[B>>2]|0;AN(aq,24,1665504,(O=i,i=i+24|0,c[O>>2]=bJ,c[O+8>>2]=bz,c[O+16>>2]=bv,O)|0);i=O;b5=bQ}else{b5=0}bv=bJ+1|0;if(bv>>>0<(c[H>>2]|0)>>>0){bQ=b5;bJ=bv}else{b4=b5;break}}}bJ=a+1064|0;bQ=a+1056|0;bz=c[bQ>>2]|0;bv=c[bQ+4>>2]|0;bQ=(bz|0)==0&(bv|0)==(-2147483648|0);bF=NU(bQ?0:bz,bQ?0:bv,c[bJ>>2]|0,c[bJ+4>>2]|0)|0;bJ=G;bv=a+1132|0;bQ=c[bv>>2]|0;if((bQ|0)==0){aI=b4;break}bz=a+1136|0;bR=(bF|0)==0&(bJ|0)==0;ak=q|0;an=q+4|0;al=0;K=bQ;while(1){bQ=c[(c[bz>>2]|0)+(al<<2)>>2]|0;ao=bQ+24|0;if((c[ao>>2]|0)==0&(c[ao+4>>2]|0)==(-2147483648|0)){if(bR){b6=2147483647;b7=-1;b8=K}else{c[ak>>2]=1;c[an>>2]=1e6;aj=AY(bF,bJ,q,bQ+4|0)|0;b6=G;b7=aj;b8=c[bv>>2]|0}if((b8|0)==0){b9=b6;ca=b7}else{aj=bQ+4|0;am=bQ+16|0;bP=b6;bK=b7;bM=0;while(1){f=c[(c[bz>>2]|0)+(bM<<2)>>2]|0;bG=f+16|0;bA=AY(c[bG>>2]|0,c[bG+4>>2]|0,f+4|0,aj)|0;f=G;if((bM|0)==(al|0)){cb=bP;cc=bK}else{bG=c[am+4>>2]|0;bL=((f|0)>(bG|0)|(f|0)==(bG|0)&bA>>>0>(c[am>>2]|0)>>>0)&((f|0)<(bP|0)|(f|0)==(bP|0)&bA>>>0<bK>>>0);cb=bL?f:bP;cc=bL?bA:bK}bA=bM+1|0;if(bA>>>0<(c[bv>>2]|0)>>>0){bP=cb;bK=cc;bM=bA}else{b9=cb;ca=cc;break}}}if((ca|0)==(-1|0)&(b9|0)==2147483647){bM=bQ+16|0;cd=c[bM+4>>2]|0;ce=c[bM>>2]|0}else{cd=b9;ce=ca}c[ao>>2]=ce;c[ao+4>>2]=cd;cf=c[bv>>2]|0}else{cf=K}bM=al+1|0;if(bM>>>0<cf>>>0){al=bM;K=cf}else{aI=b4;break}}}}while(0);if((c[H>>2]|0)!=0){b4=0;do{cf=c[(c[af>>2]|0)+(b4<<2)>>2]|0;cd=c[cf+8>>2]|0;do{if((cd|0)!=0){if((c[cd+8>>2]|0)==1){break}c[cd+808>>2]=0}}while(0);cd=c[cf+184>>2]|0;if((cd|0)!=0){A6(cd+20|0)}A6((c[(c[af>>2]|0)+(b4<<2)>>2]|0)+184|0);b4=b4+1|0;}while(b4>>>0<(c[H>>2]|0)>>>0)}H=c[D>>2]|0;if((H|0)==0){bc=aI;i=e;return bc|0}D=il(H,0,0,1)|0;H=G;AN(aq,48,1663976,(O=i,i=i+16|0,c[O>>2]=D,c[O+8>>2]=H,O)|0);i=O;bc=aI;i=e;return bc|0}function mG(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=c[a+8>>2]|0;e=c[d+8>>2]|0;do{if((e|0)==1){do{if((c[d+424>>2]|0)==0){f=c[d+48>>2]|0;if(!((f|0)==86059|(f|0)==86016|(f|0)==86017)){break}if((b|0)==0){g=0;return g|0}c[b>>2]=1732768;g=0;return g|0}}while(0);f=(c[(c[a+184>>2]|0)+40>>2]|0)>-1;do{if(f){if((c[d+420>>2]|0)!=-1){break}if((b|0)==0){g=0;return g|0}c[b>>2]=1732032;g=0;return g|0}}while(0);if((c[d+412>>2]|0)==0){if((b|0)==0){g=0;return g|0}c[b>>2]=1731400;g=0;return g|0}if((c[d+416>>2]|0)==0){if((b|0)==0){g=0;return g|0}c[b>>2]=1730680;g=0;return g|0}if(!f){break}if((c[a+452>>2]|0)!=0){break}if((c[d+48>>2]|0)!=86020){break}if((b|0)==0){g=0;return g|0}c[b>>2]=1730144;g=0;return g|0}else if((e|0)==3){if((c[d+48>>2]|0)!=94214){break}if((c[d+120>>2]|0)!=0){break}if((b|0)==0){g=0;return g|0}c[b>>2]=1729952;g=0;return g|0}else if((e|0)==2){if((c[d+48>>2]|0)==0){g=1}else{break}return g|0}else if((e|0)==0){if((c[d+120>>2]|0)==0){if((b|0)==0){g=0;return g|0}c[b>>2]=1729952;g=0;return g|0}do{if((c[(c[a+184>>2]|0)+40>>2]|0)>-1){if((c[d+140>>2]|0)!=-1){break}if((b|0)==0){g=0;return g|0}c[b>>2]=1729488;g=0;return g|0}}while(0);if(((c[d+48>>2]|0)-69|0)>>>0>=2){break}if((c[a+80>>2]|0)!=0){break}if((c[d+224>>2]|0)!=0){break}if((c[a+232>>2]|0)!=0){break}if((b|0)==0){g=0;return g|0}c[b>>2]=1729144;g=0;return g|0}}while(0);if((c[d+48>>2]|0)!=0){g=1;return g|0}if((b|0)==0){g=0;return g|0}c[b>>2]=1728880;g=0;return g|0}function mH(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;e=i;i=i+136|0;f=e|0;g=e+8|0;h=e+16|0;j=e+48|0;k=e+128|0;c[f>>2]=1;l=o0()|0;c[g>>2]=l;m=j;n=b;NN(m|0,n|0,80)|0;if((l|0)==0){o=-12;i=e;return o|0}l=a+8|0;n=(o7(c[l>>2]|0)|0)==0;m=a+184|0;b=c[m>>2]|0;p=(c[b+40>>2]|0)==0;L18301:do{if(n){if(!p){q=0;r=13607;break}c[k>>2]=0;s=c[l>>2]|0;t=c[s+12>>2]|0;do{if((t|0)==0){u=o9(c[s+48>>2]|0)|0;if((u|0)!=0){v=u;break}c[(c[m>>2]|0)+40>>2]=-1;w=-1;break L18301}else{v=t}}while(0);t=(d|0)!=0;s=t?d:k;zK(s,1684736,1682848,0)|0;u=o5(c[l>>2]|0,v,s)|0;if(!t){zL(k)}t=(c[m>>2]|0)+40|0;if((u|0)<0){c[t>>2]=-1;w=u;break}else{c[t>>2]=1;q=u;r=13607;break}}else{if(!p){q=0;r=13607;break}c[b+40>>2]=1;q=0;r=13607}}while(0);L18316:do{if((r|0)==13607){b=a+184|0;if((c[(c[b>>2]|0)+40>>2]|0)<0){w=-1;break}p=j+24|0;m=j+28|0;k=a+452|0;v=a+232|0;d=q;L18319:while(1){x=d;n=c[m>>2]|0;while(1){if((n|0)>0){if((x|0)<=-1){break L18319}}else{if((c[p>>2]|0)!=0){w=x;break L18316}if(!((c[f>>2]|0)!=0&(x|0)>-1)){break L18319}}L18328:do{if((mG(a,0)|0)!=0){u=c[l>>2]|0;do{if((c[u+48>>2]|0)==28){if((c[b>>2]|0)==0){break}t=c[u+176>>2]|0;if((t|0)<3){if((c[k>>2]|0)>6){break}else{break L18328}}s=c[k>>2]|0;if((t|0)<4){if((s|0)>17){break}else{break L18328}}else{if((s|0)>19){break}else{break L18328}}}}while(0);if((c[v>>2]|0)!=0){break L18319}if((c[(c[u+12>>2]|0)+16>>2]&1024|0)==0){break L18319}}}while(0);c[f>>2]=0;o$(c[g>>2]|0);s=c[l>>2]|0;t=c[s+8>>2]|0;if((t|0)==3){pi(s,h,f,j)|0;y=c[m>>2]|0}else if((t|0)==1){y=ph(s,c[g>>2]|0,f,j)|0}else if((t|0)==0){y=pf(s,c[g>>2]|0,f,j)|0}else{y=x}if((y|0)<=-1){d=y;continue L18319}s=c[f>>2]|0;if((s|0)!=0){c[k>>2]=(c[k>>2]|0)+1}c[p>>2]=(c[p>>2]|0)+y;t=(c[m>>2]|0)-y|0;c[m>>2]=t;x=s;n=t}}if((c[p>>2]|0)!=0){w=x;break}w=(c[f>>2]|0)==0?-1:x}}while(0);o1(g);o=w;i=e;return o|0}function mI(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;d=i;e=a+24|0;f=c[e>>2]|0;if((f|0)==0){AN(0,0,1719600,(g=i,i=i+24|0,c[g>>2]=1662512,c[g+8>>2]=1712288,c[g+16>>2]=3175,g)|0);i=g;cw()}h=a+28|0;if((c[(c[h>>2]|0)+(f-1<<2)>>2]|0)!=(b|0)){AN(0,0,1719600,(g=i,i=i+24|0,c[g>>2]=1661056,c[g+8>>2]=1712288,c[g+16>>2]=3176,g)|0);i=g;cw()}g=c[b+240>>2]|0;if((g|0)!=0){oH(g)}if((c[b+128>>2]|0)!=0){nJ(b+104|0)}zL(b+88|0);A6(b+252|0);A6(b+400|0);g=b+8|0;A6((c[g>>2]|0)+96|0);A6((c[g>>2]|0)+860|0);A6(g);A6(b+12|0);g=b+184|0;b=c[g>>2]|0;if((b|0)==0){j=g;A6(j);k=c[e>>2]|0;l=k-1|0;c[e>>2]=l;m=c[h>>2]|0;n=m+(l<<2)|0;o=n;A6(o);i=d;return}A6(b+20|0);j=g;A6(j);k=c[e>>2]|0;l=k-1|0;c[e>>2]=l;m=c[h>>2]|0;n=m+(l<<2)|0;o=n;A6(o);i=d;return}function mJ(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;b=i;i=i+24|0;d=b|0;e=b+8|0;f=b+16|0;g=c[a>>2]|0;h=c[g+16>>2]|0;j=g+4|0;k=c[j>>2]|0;if((k|0)==0){l=13653}else{if((c[k+8>>2]&1|0)==0){l=13653}else{l=13654}}if((l|0)==13653){if((c[g+1084>>2]&128|0)==0){m=h}else{l=13654}}if((l|0)==13654){m=0}l=g+1248|0;h=g+1252|0;k=f;n=c[l>>2]|0;if((n|0)!=0){o=n;do{c[f>>2]=o;c[l>>2]=c[o+80>>2];nJ(o|0);A6(k);o=c[l>>2]|0;}while((o|0)!=0)}c[h>>2]=0;h=g+1220|0;o=g+1224|0;l=e;k=c[h>>2]|0;if((k|0)!=0){f=k;do{c[e>>2]=f;c[h>>2]=c[f+80>>2];nJ(f|0);A6(l);f=c[h>>2]|0;}while((f|0)!=0)}c[o>>2]=0;o=g+1240|0;f=g+1244|0;h=d;l=c[o>>2]|0;if((l|0)!=0){e=l;do{c[d>>2]=e;c[o>>2]=c[e+80>>2];nJ(e|0);A6(h);e=c[o>>2]|0;}while((e|0)!=0)}c[f>>2]=0;c[g+1256>>2]=25e5;f=c[j>>2]|0;if((f|0)==0){mi(g);c[a>>2]=0;p=iV(m)|0;i=b;return}j=c[f+48>>2]|0;if((j|0)==0){mi(g);c[a>>2]=0;p=iV(m)|0;i=b;return}c1[j&255](g)|0;mi(g);c[a>>2]=0;p=iV(m)|0;i=b;return}function mK(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;d=a+24|0;e=c[d>>2]|0;if(e>>>0>536870910){f=0;return f|0}g=a+28|0;h=A2(c[g>>2]|0,(e<<2)+4|0)|0;if((h|0)==0){f=0;return f|0}c[g>>2]=h;h=A7(480)|0;e=h;if((h|0)==0){f=0;return f|0}i=A7(88)|0;j=h+184|0;c[j>>2]=i;if((i|0)==0){A4(h);f=0;return f|0}k=i;c[k>>2]=0;c[k+4>>2]=-2147483648;k=ov(b)|0;c[h+8>>2]=k;b=a+4|0;if((c[b>>2]|0)==0){l=0;m=0}else{c[k+72>>2]=0;k=(c[b>>2]|0)!=0;l=k?2147418111:0;m=k?-1:0}c[h>>2]=c[d>>2];k=h+48|0;c[k>>2]=0;c[k+4>>2]=-2147483648;k=h+56|0;c[k>>2]=0;c[k+4>>2]=-2147483648;k=h+208|0;c[k>>2]=m;c[k+4>>2]=l;l=h+200|0;c[l>>2]=0;c[l+4>>2]=-2147483648;c[h+228>>2]=2500;l=h+464|0;c[l>>2]=0;c[l+4>>2]=-2147483648;c[h+472>>2]=0;mL(e,33,1,9e4);l=h+216|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+264|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+272|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+280|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+288|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+296|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+304|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+312|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+320|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+328|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+336|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+344|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+352|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+360|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+368|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+376|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+384|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+392|0;c[l>>2]=0;c[l+4>>2]=-2147483648;l=h+192|0;c[l>>2]=0;c[l+4>>2]=-2147483648;c[h+80>>2]=0;c[h+84>>2]=1;h=c[j>>2]|0;c[h>>2]=0;c[h+4>>2]=-2147483648;h=(c[j>>2]|0)+56|0;c[h>>2]=0;c[h+4>>2]=-2147483648;h=(c[j>>2]|0)+72|0;c[h>>2]=0;c[h+4>>2]=-2147483648;h=c[d>>2]|0;c[d>>2]=h+1;c[(c[g>>2]|0)+(h<<2)>>2]=e;f=e;return f|0}function mL(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;f=i;i=i+8|0;g=f|0;h=g;j=g;k=h+4|0;do{if((BX(j,k,d,0,e,0,2147483647,0)|0)==0){AN(0,24,1627448,(l=i,i=i+8|0,c[l>>2]=c[a>>2],l)|0);i=l;m=13686}else{n=c[j>>2]|0;if((n|0)==(d|0)){o=d;break}AN(0,48,1628024,(l=i,i=i+16|0,c[l>>2]=c[a>>2],c[l+8>>2]=(d>>>0)/(n>>>0)|0,l)|0);i=l;m=13686}}while(0);if((m|0)==13686){o=c[j>>2]|0}j=c[k>>2]|0;if((o|0)<1|(j|0)<1){k=c[a>>2]|0;AN(0,16,1626944,(l=i,i=i+24|0,c[l>>2]=o,c[l+8>>2]=j,c[l+16>>2]=k,l)|0);i=l;i=f;return}else{l=a+40|0;k=c[g+4>>2]|0;c[l>>2]=c[g>>2];c[l+4>>2]=k;o2(c[a+8>>2]|0,h);c[a+188>>2]=b;i=f;return}}function mM(a,b,d,e,f,g,h,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;k=i;l=d;d=i;i=i+8|0;c[d>>2]=c[l>>2];c[d+4>>2]=c[l+4>>2];l=a+1132|0;m=c[l>>2]|0;if((m|0)==0){n=13697}else{o=c[a+1136>>2]|0;p=0;q=0;do{r=c[o+(p<<2)>>2]|0;q=(c[r>>2]|0)==(b|0)?r:q;p=p+1|0;}while(p>>>0<m>>>0);if((q|0)==0){n=13697}else{s=q}}do{if((n|0)==13697){q=A7(40)|0;if((q|0)==0){t=0;i=k;return t|0}else{Ba(a+1136|0,l,q);s=q;break}}}while(0);zK(s+32|0,1659712,j,0)|0;c[s>>2]=b;b=d;d=s+4|0;j=c[b+4>>2]|0;c[d>>2]=c[b>>2];c[d+4>>2]=j;j=s+16|0;c[j>>2]=e;c[j+4>>2]=f;f=s+24|0;c[f>>2]=g;c[f+4>>2]=h;t=s;i=k;return t|0}function mN(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;g=i;j=b+24|0;k=c[j>>2]|0;do{if((k|0)==0){l=0}else{m=A7(k)|0;if(!((c[j>>2]|0)!=0&(m|0)==0)){l=m;break}i=g;return}}while(0);k=(f|0)!=0;if(k){n=c[b+8>>2]|0}else{n=c[b+4>>2]|0}m=c[n>>2]|0;AN(0,32,1657e3,(n=i,i=i+40|0,c[n>>2]=k?1654680:1653488,c[n+8>>2]=d,c[n+16>>2]=m,c[n+24>>2]=k?1651376:1649904,c[n+32>>2]=e,n)|0);i=n;mO(c[b+1140>>2]|0,1648152);if(!k){AN(0,32,1645752,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;k=b+1064|0;e=c[k>>2]|0;m=c[k+4>>2]|0;if((e|0)==0&(m|0)==(-2147483648|0)){AN(0,32,1642456,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n}else{k=NU(e,m,5e3,0)|0;m=G;e=N0(k,m,1e6,0)|0;o=e;e=N1(k,m,1e6,0)|0;AN(0,32,1644192,(n=i,i=i+32|0,c[n>>2]=(o|0)/3600|0,c[n+8>>2]=((o|0)/60|0|0)%60|0,c[n+16>>2]=(o|0)%60|0,c[n+24>>2]=(e*100|0|0)/1e6|0,n)|0);i=n}e=b+1056|0;if(!((c[e>>2]|0)==0&(c[e+4>>2]|0)==(-2147483648|0))){AN(0,32,1639848,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;o=c[e>>2]|0;m=c[e+4>>2]|0;e=N0(o,m,1e6,0)|0;k=N1(o,m,1e6,0)|0;m=k;k=(m|0)>-1?m:-m|0;m=AW(k,(k|0)<0|0?-1:0,1e6,0,1e6,0)|0;AN(0,32,1638016,(n=i,i=i+16|0,c[n>>2]=e,c[n+8>>2]=m,n)|0);i=n}AN(0,32,1636064,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;m=c[b+1072>>2]|0;if((m|0)==0){AN(0,32,1642456,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n}else{AN(0,32,1635416,(n=i,i=i+8|0,c[n>>2]=(m|0)/1e3|0,n)|0);i=n}AN(0,32,1634856,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n}m=b+1132|0;if((c[m>>2]|0)!=0){e=b+1136|0;k=0;do{o=c[(c[e>>2]|0)+(k<<2)>>2]|0;AN(0,32,1634096,(n=i,i=i+16|0,c[n>>2]=d,c[n+8>>2]=k,n)|0);i=n;p=o+16|0;q=o+4|0;r=o+8|0;AN(0,32,1633448,(n=i,i=i+8|0,h[n>>3]=(+((c[p>>2]|0)>>>0)+ +(c[p+4>>2]|0)*4294967296.0)*(+(c[q>>2]|0)/+(c[r>>2]|0)),n)|0);i=n;p=o+24|0;AN(0,32,1632736,(n=i,i=i+8|0,h[n>>3]=(+((c[p>>2]|0)>>>0)+ +(c[p+4>>2]|0)*4294967296.0)*(+(c[q>>2]|0)/+(c[r>>2]|0)),n)|0);i=n;mO(c[o+32>>2]|0,1631824);k=k+1|0;}while(k>>>0<(c[m>>2]|0)>>>0)}m=b+1104|0;do{if((c[m>>2]|0)==0){s=13730}else{k=b+1108|0;e=0;o=0;r=c[k>>2]|0;while(1){q=zJ(c[(c[r+(e<<2)>>2]|0)+20>>2]|0,1631024,0,0)|0;if((q|0)==0){t=3119048}else{t=c[q+4>>2]|0}AN(0,32,1630536,(n=i,i=i+16|0,c[n>>2]=c[c[(c[k>>2]|0)+(e<<2)>>2]>>2],c[n+8>>2]=t,n)|0);i=n;mO(c[(c[(c[k>>2]|0)+(e<<2)>>2]|0)+20>>2]|0,1631824);q=c[k>>2]|0;p=c[q+(e<<2)>>2]|0;if((c[p+16>>2]|0)==0){u=0;v=q}else{q=0;w=p;while(1){mP(b,c[(c[w+12>>2]|0)+(q<<2)>>2]|0,d,f);p=c[k>>2]|0;x=p+(e<<2)|0;a[l+(c[(c[(c[x>>2]|0)+12>>2]|0)+(q<<2)>>2]|0)|0]=1;y=q+1|0;z=c[x>>2]|0;x=c[z+16>>2]|0;if(y>>>0<x>>>0){q=y;w=z}else{u=x;v=p;break}}}A=u+o|0;w=e+1|0;if(w>>>0<(c[m>>2]|0)>>>0){e=w;o=A;r=v}else{break}}r=c[j>>2]|0;if(A>>>0>=r>>>0){B=r;break}AN(0,32,1629736,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;s=13730}}while(0);if((s|0)==13730){B=c[j>>2]|0}if((B|0)!=0){s=0;n=B;while(1){if((a[l+s|0]|0)==0){mP(b,s,d,f);C=c[j>>2]|0}else{C=n}B=s+1|0;if(B>>>0<C>>>0){s=B;n=C}else{break}}}A4(l);i=g;return}function mO(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;e=i;i=i+256|0;f=e|0;if((b|0)==0){i=e;return}do{if((zI(b)|0)==1){if((zJ(b,1616944,0,0)|0)==0){break}i=e;return}}while(0);AN(0,32,1737968,(g=i,i=i+8|0,c[g>>2]=d,g)|0);i=g;h=zJ(b,3119048,0,2)|0;if((h|0)==0){i=e;return}j=f|0;f=h;do{h=c[f>>2]|0;if((a8(1616944,h|0)|0)!=0){k=c[f+4>>2]|0;AN(0,32,1737432,(g=i,i=i+16|0,c[g>>2]=d,c[g+8>>2]=h,g)|0);i=g;L18510:do{if((a[k]|0)!=0){h=k;do{l=bv(h|0,1736936)|0;m=l+1|0;y_(j,h,m>>>0<256?m:256)|0;AN(0,32,1736408,(g=i,i=i+8|0,c[g>>2]=j,g)|0);i=g;n=h+l|0;l=a[n]|0;if(l<<24>>24==13){AN(0,32,1617872,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g;o=a[n]|0}else{o=l}if(o<<24>>24==10){AN(0,32,1735568,(g=i,i=i+16|0,c[g>>2]=d,c[g+8>>2]=3119048,g)|0);i=g;p=a[n]|0}else{p=o}if(p<<24>>24==0){break L18510}h=h+m|0;}while((a[h]|0)!=0)}}while(0);AN(0,32,1634856,(g=i,i=i+1|0,i=i+7&-8,c[g>>2]=0,g)|0);i=g}f=zJ(b,3119048,f,2)|0;}while((f|0)!=0);i=e;return}function mP(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0.0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0;f=i;i=i+264|0;g=f+256|0;if((e|0)==0){j=(c[a+4>>2]|0)+8|0}else{j=(c[a+8>>2]|0)+28|0}k=c[j>>2]|0;j=c[(c[a+28>>2]|0)+(b<<2)>>2]|0;a=j+40|0;l=c[a>>2]|0;m=j+44|0;n=c[m>>2]|0;o=AU(l,(l|0)<0|0?-1:0,n,(n|0)<0|0?-1:0)|0;n=o;o=j+88|0;l=zJ(c[o>>2]|0,1616944,0,0)|0;p=f|0;q=j+8|0;pp(p,256,c[q>>2]|0,e);AN(0,32,1616616,(e=i,i=i+16|0,c[e>>2]=d,c[e+8>>2]=b,e)|0);i=e;if((k&8|0)!=0){AN(0,32,1616336,(e=i,i=i+8|0,c[e>>2]=c[j+4>>2],e)|0);i=e}if((l|0)!=0){AN(0,32,1616016,(e=i,i=i+8|0,c[e>>2]=c[l+4>>2],e)|0);i=e}l=(c[a>>2]|0)/(n|0)|0;k=(c[m>>2]|0)/(n|0)|0;AN(0,48,1615512,(e=i,i=i+24|0,c[e>>2]=c[j+232>>2],c[e+8>>2]=l,c[e+16>>2]=k,e)|0);i=e;AN(0,32,1614912,(e=i,i=i+8|0,c[e>>2]=p,e)|0);i=e;p=j+80|0;k=c[p>>2]|0;L18534:do{if((k|0)!=0){l=c[q>>2]|0;n=j+84|0;b=c[n>>2]|0;d=c[l+224>>2]|0;r=c[l+228>>2]|0;s=N2(r,(r|0)<0|0?-1:0,k,(k|0)<0|0?-1:0)|0;t=G;u=N2(d,(d|0)<0|0?-1:0,b,(b|0)<0|0?-1:0)|0;do{if((s|0)==(u|0)&(t|0)==(G|0)){if(!((r|0)==0|(b|0)==0)){break L18534}if((d|0)==0){break}if((k>>31|0)==(d>>31|0)){break L18534}}}while(0);d=g|0;r=g+4|0;t=ac(k,c[l+120>>2]|0)|0;u=t;s=(t|0)<0|0?-1:0;t=ac(b,c[l+124>>2]|0)|0;v=t;w=(t|0)<0|0?-1:0;t=1048576;x=0;BX(d,r,u,s,v,w,t,x)|0;x=c[n>>2]|0;t=c[d>>2]|0;d=c[r>>2]|0;AN(0,32,1614168,(e=i,i=i+32|0,c[e>>2]=c[p>>2],c[e+8>>2]=x,c[e+16>>2]=t,c[e+24>>2]=d,e)|0);i=e}}while(0);do{if((c[(c[q>>2]|0)+8>>2]|0)==0){p=c[j+96>>2]|0;do{if((p|0)!=0){k=c[j+92>>2]|0;if((k|0)==0){break}y=+(k|0)/+(p|0);k=bR(+(y*100.0))|0;g=k;d=(k|0)<0|0?-1:0;k=N4(g,d,100,0)|0;if(!((k|0)==0&(G|0)==0)){AN(0,32,1738888,(e=i,i=i+16|0,h[e>>3]=y,c[e+8>>2]=1613744,e)|0);i=e;break}k=N4(g,d,1e5,0)|0;if((k|0)==0&(G|0)==0){AN(0,32,1738344,(e=i,i=i+16|0,h[e>>3]=y/1.0e3,c[e+8>>2]=1613744,e)|0);i=e;break}else{AN(0,32,1738688,(e=i,i=i+16|0,h[e>>3]=y,c[e+8>>2]=1613744,e)|0);i=e;break}}}while(0);p=c[j+416>>2]|0;do{if((p|0)!=0){n=c[j+412>>2]|0;if((n|0)==0){break}y=+(n|0)/+(p|0);n=bR(+(y*100.0))|0;l=n;b=(n|0)<0|0?-1:0;n=N4(l,b,100,0)|0;if(!((n|0)==0&(G|0)==0)){AN(0,32,1738888,(e=i,i=i+16|0,h[e>>3]=y,c[e+8>>2]=1613088,e)|0);i=e;break}n=N4(l,b,1e5,0)|0;if((n|0)==0&(G|0)==0){AN(0,32,1738344,(e=i,i=i+16|0,h[e>>3]=y/1.0e3,c[e+8>>2]=1613088,e)|0);i=e;break}else{AN(0,32,1738688,(e=i,i=i+16|0,h[e>>3]=y,c[e+8>>2]=1613088,e)|0);i=e;break}}}while(0);p=c[m>>2]|0;do{if((p|0)!=0){n=c[a>>2]|0;if((n|0)==0){break}y=1.0/(+(n|0)/+(p|0));n=bR(+(y*100.0))|0;b=n;l=(n|0)<0|0?-1:0;n=N4(b,l,100,0)|0;if(!((n|0)==0&(G|0)==0)){AN(0,32,1738888,(e=i,i=i+16|0,h[e>>3]=y,c[e+8>>2]=1612544,e)|0);i=e;break}n=N4(b,l,1e5,0)|0;if((n|0)==0&(G|0)==0){AN(0,32,1738344,(e=i,i=i+16|0,h[e>>3]=y/1.0e3,c[e+8>>2]=1612544,e)|0);i=e;break}else{AN(0,32,1738688,(e=i,i=i+16|0,h[e>>3]=y,c[e+8>>2]=1612544,e)|0);i=e;break}}}while(0);p=c[q>>2]|0;n=c[p+108>>2]|0;if((n|0)==0){break}l=c[p+104>>2]|0;if((l|0)==0){break}y=1.0/(+(l|0)/+(n|0));n=bR(+(y*100.0))|0;l=n;p=(n|0)<0|0?-1:0;n=N4(l,p,100,0)|0;if(!((n|0)==0&(G|0)==0)){AN(0,32,1738888,(e=i,i=i+16|0,h[e>>3]=y,c[e+8>>2]=1612184,e)|0);i=e;break}n=N4(l,p,1e5,0)|0;if((n|0)==0&(G|0)==0){AN(0,32,1738344,(e=i,i=i+16|0,h[e>>3]=y/1.0e3,c[e+8>>2]=1612184,e)|0);i=e;break}else{AN(0,32,1738688,(e=i,i=i+16|0,h[e>>3]=y,c[e+8>>2]=1612184,e)|0);i=e;break}}}while(0);q=j+72|0;j=c[q>>2]|0;if((j&1|0)==0){z=j}else{AN(0,32,1611824,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;z=c[q>>2]|0}if((z&2|0)==0){A=z}else{AN(0,32,1611624,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;A=c[q>>2]|0}if((A&4|0)==0){B=A}else{AN(0,32,1745064,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;B=c[q>>2]|0}if((B&8|0)==0){C=B}else{AN(0,32,1744672,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;C=c[q>>2]|0}if((C&16|0)==0){D=C}else{AN(0,32,1743952,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;D=c[q>>2]|0}if((D&32|0)==0){E=D}else{AN(0,32,1742264,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;E=c[q>>2]|0}if((E&64|0)==0){F=E}else{AN(0,32,1740792,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;F=c[q>>2]|0}if((F&128|0)==0){H=F}else{AN(0,32,1740088,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;H=c[q>>2]|0}if((H&256|0)==0){I=H}else{AN(0,32,1739464,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;I=c[q>>2]|0}if((I&512|0)==0){AN(0,32,1634856,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;J=c[o>>2]|0;mO(J,1631824);i=f;return}AN(0,32,1739200,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;AN(0,32,1634856,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;J=c[o>>2]|0;mO(J,1631824);i=f;return}function mQ(b,e,f,g,h){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;j=i;if((h|0)<=0){i=j;return}k=(e|0)!=0;l=0;do{m=h-l|0;n=(m|0)>16?16:m;if(k){cx(e|0,1619080,(o=i,i=i+8|0,c[o>>2]=l,o)|0)|0;i=o;p=0}else{AN(b,f,1619080,(o=i,i=i+8|0,c[o>>2]=l,o)|0);i=o;p=0}do{do{if((p|0)<(n|0)){m=d[g+(p+l)|0]|0;if(k){cx(e|0,1618640,(o=i,i=i+8|0,c[o>>2]=m,o)|0)|0;i=o;break}else{AN(b,f,1618640,(o=i,i=i+8|0,c[o>>2]=m,o)|0);i=o;break}}else{if(k){aW(1618296,3,1,e|0)|0;break}else{AN(b,f,1618296,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;break}}}while(0);p=p+1|0;}while((p|0)<16);if(k){bw(32,e|0)|0}else{AN(b,f,1617872,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o}if((n|0)>0){m=0;do{q=a[g+(m+l)|0]|0;r=(q-32&255)>94?46:q&255;if(k){bw(r|0,e|0)|0}else{AN(b,f,1617312,(o=i,i=i+8|0,c[o>>2]=r,o)|0);i=o}m=m+1|0;}while((m|0)<(n|0))}if(k){bw(10,e|0)|0}else{AN(b,f,1634856,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o}l=l+16|0;}while((l|0)<(h|0));i=j;return}function mR(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;mQ(a,0,b,c,d);return}function mS(a,b,d,e,f,g,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0.0,q=0.0,r=0,s=0;k=i;l=(b|0)!=0;m=c[e+32>>2]|0;if(l){cx(b|0,1623536,(n=i,i=i+8|0,c[n>>2]=m,n)|0)|0;i=n;o=c[e+36>>2]&1;cx(b|0,1622816,(n=i,i=i+8|0,c[n>>2]=o,n)|0)|0;i=n}else{AN(a,d,1623536,(n=i,i=i+8|0,c[n>>2]=m,n)|0);i=n;AN(a,d,1622816,(n=i,i=i+8|0,c[n>>2]=c[e+36>>2]&1,n)|0);i=n}p=+(g|0)/+(j|0);q=+(c[e+48>>2]|0)*p;if(l){cx(b|0,1622392,(n=i,i=i+8|0,h[n>>3]=q,n)|0)|0;i=n;aW(1622048,6,1,b|0)|0}else{AN(a,d,1622392,(n=i,i=i+8|0,h[n>>3]=q,n)|0);i=n;AN(a,d,1622048,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n}j=e+16|0;g=c[j>>2]|0;m=c[j+4>>2]|0;do{if((g|0)==0&(m|0)==(-2147483648|0)){if(l){aW(1642456,3,1,b|0)|0;r=13867;break}else{AN(a,d,1642456,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;r=13866;break}}else{q=p*(+(g>>>0)+ +(m|0)*4294967296.0);if(l){cx(b|0,1621680,(n=i,i=i+8|0,h[n>>3]=q,n)|0)|0;i=n;r=13867;break}else{AN(a,d,1621680,(n=i,i=i+8|0,h[n>>3]=q,n)|0);i=n;r=13866;break}}}while(0);if((r|0)==13866){AN(a,d,1621288,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n}else if((r|0)==13867){aW(1621288,6,1,b|0)|0}m=e+8|0;g=c[m>>2]|0;j=c[m+4>>2]|0;do{if((g|0)==0&(j|0)==(-2147483648|0)){if(l){aW(1642456,3,1,b|0)|0;r=13876;break}else{AN(a,d,1642456,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;r=13875;break}}else{q=p*(+(g>>>0)+ +(j|0)*4294967296.0);if(l){cx(b|0,1621680,(n=i,i=i+8|0,h[n>>3]=q,n)|0)|0;i=n;r=13876;break}else{AN(a,d,1621680,(n=i,i=i+8|0,h[n>>3]=q,n)|0);i=n;r=13875;break}}}while(0);if((r|0)==13875){AN(a,d,1634856,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;l=e+28|0;AN(a,d,1619696,(n=i,i=i+8|0,c[n>>2]=c[l>>2],n)|0);i=n;s=l}else if((r|0)==13876){bw(10,b|0)|0;r=e+28|0;cx(b|0,1619696,(n=i,i=i+8|0,c[n>>2]=c[r>>2],n)|0)|0;i=n;s=r}if((f|0)==0){i=k;return}mQ(0,b,0,c[e+24>>2]|0,c[s>>2]|0);i=k;return}function mT(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;mS(a,0,b,d,e,c[f+40>>2]|0,c[f+44>>2]|0);return}function mU(b,c,e,f){b=b|0;c=c|0;e=e|0;f=f|0;var g=0,h=0,i=0;g=(f|0)!=0?251056:251040;if((e|0)>0){h=0}else{return b|0}do{f=c+h|0;i=h<<1;a[b+i|0]=a[g+((d[f]|0)>>>4)|0]|0;a[b+(i|1)|0]=a[g+(a[f]&15)|0]|0;h=h+1|0;}while((h|0)<(e|0));return b|0}function mV(a){a=a|0;var b=0,c=0,d=0,e=0,f=0;b=i;i=i+96|0;c=b|0;d=b+48|0;NM(c|0,0,44);NM(d|0,0,44);e=BG(a,1626104,c)|0;if((BG(a,1625376,d)|0)!=0&(e|0)==0){f=BH(d)|0}else{f=BH(c)|0}i=b;return(G=(f|0)<0|0?-1:0,f)|0}function mW(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;do{if((a|0)!=0){e=c[a+64>>2]|0;if((e|0)!=0){f=dg[e&255](b,d)|0;return f|0}e=c[a+32>>2]|0;if((e|0)==0){if((c[a+20>>2]|0)==(b|0)){f=1;return f|0}if((c[a+16>>2]|0)==(b|0)){f=1;return f|0}if((c[a+24>>2]|0)==(b|0)){f=1}else{break}return f|0}g=c[e>>2]|0;if((g|0)==0){f=0;return f|0}else{h=0;i=g}L18724:while(1){j=i;while(1){g=c[j>>2]|0;if((g|0)==0){break}if((g|0)==(b|0)){break L18724}else{j=j+8|0}}g=h+1|0;k=c[e+(g<<2)>>2]|0;if((k|0)==0){f=0;l=13908;break}else{h=g;i=k}}if((l|0)==13908){return f|0}f=(c[j+4>>2]|0)!=0|0;return f|0}}while(0);f=-1163346256;return f|0}function mX(){return 0}function mY(){return 0}function mZ(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;f=i;i=i+8|0;b=f|0;g=b;h=i;i=i+8|0;j=h;k=i;i=i+8|0;l=e+412|0;m=c[l>>2]|0;n=c[l+4>>2]|0;c[b>>2]=m;c[b+4>>2]=n;l=c[e+8>>2]|0;o=l+112|0;p=m;q=n;do{if((c[o>>2]|0)>1){r=l+104|0;s=d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0;r=l+108|0;t=d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0;r=e+92|0;u=c[r+4>>2]|0;c[h>>2]=c[r>>2];c[h+4>>2]=u;u=ac(c[o>>2]|0,s)|0;if(!((t|0)>0&(u|0)>0)){v=n;w=m;break}if(+(t|0)/+(u|0)>=+(p|0)/+(q|0)*.7){v=n;w=m;break}BZ(k,j,g);if(+R(+(1.0- +(c[k>>2]|0)/+(c[k+4>>2]|0)))>.1){s=t|0;t=u|0;c[b>>2]=s;c[b+4>>2]=t;v=t;w=s;break}else{v=c[b+4>>2]|0;w=c[b>>2]|0;break}}else{v=n;w=m}}while(0);m=a;c[m>>2]=w;c[m+4>>2]=v;i=f;return}function m_(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;f=i;i=i+16|0;g=f|0;h=f+8|0;j=a[e]|0;if((j-48&255)<10){k=aU(e|0,0,0)|0;l=(k|0)==(c[d>>2]|0)|0;i=f;return l|0}L18752:do{switch(j<<24>>24){case 112:{if((a[e+1|0]|0)!=58){break L18752}k=aU(e+2|0,g|0,0)|0;m=c[b+1104>>2]|0;if((m|0)==0){l=0;i=f;return l|0}n=b+1108|0;o=c[n>>2]|0;p=d|0;q=0;L18758:while(1){r=c[o+(q<<2)>>2]|0;L18760:do{if((c[r>>2]|0)==(k|0)){s=c[g>>2]|0;t=s+1|0;c[g>>2]=t;if((a[s]|0)==58){u=13947;break L18758}s=c[r+16>>2]|0;v=r+12|0;w=0;while(1){if(w>>>0>=s>>>0){break L18760}if((c[p>>2]|0)==(c[(c[v>>2]|0)+(w<<2)>>2]|0)){l=1;u=13963;break L18758}else{w=w+1|0}}}}while(0);r=q+1|0;if(r>>>0<m>>>0){q=r}else{l=0;u=13962;break}}if((u|0)==13962){i=f;return l|0}else if((u|0)==13963){i=f;return l|0}else if((u|0)==13947){m=aU(t|0,0,0)|0;if((m|0)<=-1){l=0;i=f;return l|0}k=c[(c[n>>2]|0)+(q<<2)>>2]|0;if(m>>>0>=(c[k+16>>2]|0)>>>0){l=0;i=f;return l|0}l=(c[p>>2]|0)==(c[(c[k+12>>2]|0)+(m<<2)>>2]|0)|0;i=f;return l|0}break};case 35:{m=aU(e+1|0,h|0,0)|0;if((a[c[h>>2]|0]|0)!=0){break L18752}l=(c[d+4>>2]|0)==(m|0)|0;i=f;return l|0};case 0:{l=1;i=f;return l|0};case 118:case 97:case 115:case 100:case 116:{switch(j<<24>>24|0){case 97:{x=1;break};case 115:{x=3;break};case 116:{x=4;break};case 118:{x=0;break};case 100:{x=2;break};default:{AN(0,0,1719600,(y=i,i=i+24|0,c[y>>2]=1624552,c[y+8>>2]=1712288,c[y+16>>2]=4083,y)|0);i=y;cw();return 0}}if((x|0)!=(c[(c[d+8>>2]|0)+8>>2]|0)){l=0;i=f;return l|0}if((a[e+1|0]|0)!=58){l=1;i=f;return l|0}m=aU(e+2|0,0,0)|0;k=c[b+24>>2]|0;if((k|0)==0){l=0;i=f;return l|0}o=c[b+28>>2]|0;r=0;w=m;while(1){if((c[(c[(c[o+(r<<2)>>2]|0)+8>>2]|0)+8>>2]|0)==(x|0)){if((w|0)==0){break}else{z=w-1|0}}else{z=w}m=r+1|0;if(m>>>0<k>>>0){r=m;w=z}else{l=0;u=13960;break}}if((u|0)==13960){i=f;return l|0}l=(r|0)==(c[d>>2]|0)|0;i=f;return l|0};default:{}}}while(0);AN(b,16,1624136,(y=i,i=i+8|0,c[y>>2]=e,y)|0);i=y;l=-22;i=f;return l|0}function m$(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=a+8|0;a=c[b>>2]|0;d=c[a+120>>2]|0;do{if((d|0)==1920){e=(c[a+408>>2]|0)==1;f=e?81:97;g=e?250696:250784}else if((d|0)==1440){f=97;g=250496}else{e=(d|0)==1280;if(e){f=e?89:0;g=e?250600:0;break}return}}while(0);A6(a+96|0);c[(c[b>>2]|0)+100>>2]=0;a=A7(f+16|0)|0;c[(c[b>>2]|0)+96>>2]=a;a=c[(c[b>>2]|0)+96>>2]|0;if((a|0)==0){return}NN(a|0,g|0,f)|0;c[(c[b>>2]|0)+100>>2]=f;return}function m0(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0.0;b=i;i=i+24|0;d=b|0;e=b+8|0;f=b+16|0;g=a+24|0;do{if((c[g>>2]|0)!=0){j=a+28|0;k=f|0;l=f+4|0;m=d|0;n=d+4|0;o=a+1104|0;p=a+1108|0;q=e|0;r=e+4|0;s=-2147483648;t=0;u=-2147483648;v=0;w=0;x=2147483647;y=-1;z=2147483647;A=-1;while(1){B=c[(c[j>>2]|0)+(w<<2)>>2]|0;C=B+48|0;D=c[C>>2]|0;E=c[C+4>>2]|0;L18825:do{if((D|0)==0&(E|0)==(-2147483648|0)){F=z;H=A;I=x;J=y;K=u;L=v}else{C=B+40|0;if((c[B+44>>2]|0)==0){F=z;H=A;I=x;J=y;K=u;L=v;break}c[m>>2]=1;c[n>>2]=1e6;M=AY(D,E,C,d)|0;N=G;if(((c[(c[B+8>>2]|0)+8>>2]|0)-2|0)>>>0<2){O=(N|0)<(x|0)|(N|0)==(x|0)&M>>>0<y>>>0;P=z;Q=A;R=O?N:x;S=O?M:y}else{O=(z|0)>(N|0)|(z|0)==(N|0)&A>>>0>M>>>0;P=O?N:z;Q=O?M:A;R=x;S=y}O=B+56|0;T=c[O>>2]|0;U=c[O+4>>2]|0;if((T|0)==0&(U|0)==(-2147483648|0)){V=u;W=v;X=-2147483648;Y=0}else{c[q>>2]=1;c[r>>2]=1e6;O=AY(T,U,C,e)|0;C=NU(O,G,M,N)|0;O=G;U=(u|0)>(O|0)|(u|0)==(O|0)&v>>>0>C>>>0;V=U?u:O;W=U?v:C;X=O;Y=C}C=c[o>>2]|0;if((C|0)==0){F=P;H=Q;I=R;J=S;K=V;L=W;break}else{Z=0;_=C}while(1){C=c[p>>2]|0;O=Z;U=0;L18837:while(1){$=c[C+(U<<2)>>2]|0;L18839:do{if(($|0)==(O|0)){aa=0}else{if((O|0)!=0){aa=O;break}T=c[$+16>>2]|0;ab=$+12|0;ac=0;while(1){if(ac>>>0>=T>>>0){aa=0;break L18839}if((c[(c[ab>>2]|0)+(ac<<2)>>2]|0)==(w|0)){break L18837}else{ac=ac+1|0}}}}while(0);ac=U+1|0;if(ac>>>0<_>>>0){O=aa;U=ac}else{F=P;H=Q;I=R;J=S;K=V;L=W;break L18825}}if(($|0)==0){F=P;H=Q;I=R;J=S;K=V;L=W;break L18825}U=$+40|0;O=c[U>>2]|0;C=c[U+4>>2]|0;if((O|0)==0&(C|0)==(-2147483648|0)|((C|0)>(N|0)|(C|0)==(N|0)&O>>>0>M>>>0)){c[U>>2]=M;c[U+4>>2]=N}U=$+48|0;O=c[U+4>>2]|0;if((O|0)<(X|0)|(O|0)==(X|0)&(c[U>>2]|0)>>>0<Y>>>0){c[U>>2]=Y;c[U+4>>2]=X}U=c[o>>2]|0;if((U|0)==0){F=P;H=Q;I=R;J=S;K=V;L=W;break}else{Z=$;_=U}}}}while(0);E=B+56|0;D=c[E>>2]|0;N=c[E+4>>2]|0;if((D|0)==0&(N|0)==(-2147483648|0)){ad=s;ae=t}else{c[k>>2]=1;c[l>>2]=1e6;E=AY(D,N,B+40|0,f)|0;N=G;D=(s|0)>(N|0)|(s|0)==(N|0)&t>>>0>E>>>0;ad=D?s:N;ae=D?t:E}E=w+1|0;if(E>>>0<(c[g>>2]|0)>>>0){s=ad;t=ae;u=K;v=L;w=E;x=I;y=J;z=F;A=H}else{break}}do{if((H|0)==(-1|0)&(F|0)==2147483647){af=I;ag=J}else{A=(F|0)>(I|0)|(F|0)==(I|0)&H>>>0>J>>>0;z=NV(H,F,J,I)|0;y=G;x=0;w=A&((y|0)<(x|0)|(y|0)==(x|0)&z>>>0<1e6>>>0);if(w|A^1){af=w?I:F;ag=w?J:H;break}AN(a,40,1734408,(w=i,i=i+8|0,h[w>>3]=(+(J>>>0)+ +(I|0)*4294967296.0)/1.0e6,w)|0);i=w;af=F;ag=H}}while(0);do{if((ag|0)==(-1|0)&(af|0)==2147483647){ah=ad;ai=ae}else{w=a+1056|0;c[w>>2]=ag;c[w+4>>2]=af;if((L|0)==0&(K|0)==(-2147483648|0)){ah=ad;ai=ae;break}w=c[a+1104>>2]|0;if((w|0)==0){A=NV(L,K,ag,af)|0;z=G;x=(ad|0)>(z|0)|(ad|0)==(z|0)&ae>>>0>A>>>0;ah=x?ad:z;ai=x?ae:A;break}A=c[a+1108>>2]|0;x=ad;z=ae;y=0;while(1){v=c[A+(y<<2)>>2]|0;u=v+40|0;t=c[u>>2]|0;s=c[u+4>>2]|0;do{if((t|0)==0&(s|0)==(-2147483648|0)){aj=x;ak=z}else{u=v+48|0;l=c[u>>2]|0;k=c[u+4>>2]|0;if(!((k|0)>(s|0)|(k|0)==(s|0)&l>>>0>t>>>0)){aj=x;ak=z;break}u=NV(l,k,t,s)|0;k=G;l=(x|0)>(k|0)|(x|0)==(k|0)&z>>>0>u>>>0;aj=l?x:k;ak=l?z:u}}while(0);s=y+1|0;if(s>>>0<w>>>0){x=aj;z=ak;y=s}else{ah=aj;ai=ak;break}}}}while(0);y=0;if(!((ah|0)>(y|0)|(ah|0)==(y|0)&ai>>>0>0>>>0)){break}y=a+1064|0;if(!((c[y>>2]|0)==0&(c[y+4>>2]|0)==(-2147483648|0))){break}c[y>>2]=ai;c[y+4>>2]=ah}}while(0);ah=c[a+16>>2]|0;if((ah|0)==0){i=b;return}ai=ip(ah)|0;ah=G;ak=0;if(!((ah|0)>(ak|0)|(ah|0)==(ak|0)&ai>>>0>0>>>0)){i=b;return}ak=a+1064|0;aj=c[ak>>2]|0;ae=c[ak+4>>2]|0;if((aj|0)==0&(ae|0)==(-2147483648|0)){i=b;return}al=(+(ai>>>0)+ +(ah|0)*4294967296.0)*8.0*1.0e6/(+(aj>>>0)+ +(ae|0)*4294967296.0);if(!(al>=0.0&al<=2147483647.0)){i=b;return}c[a+1072>>2]=~~al;i=b;return}function m1(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0;e=i;i=i+168|0;f=e|0;g=e+80|0;h=e+160|0;j=f;NM(j|0,0,80);NM(g|0,0,80);k=c[(c[a+28>>2]|0)+(d<<2)>>2]|0;do{if((b|0)==0){nB(g);l=g;m=1;n=0;o=0}else{d=c[b+24>>2]|0;p=c[b+28>>2]|0;if((p|0)!=0){l=b;m=0;n=p;o=d;break}p=c[k+240>>2]|0;if((c[p+176>>2]&1|0)==0){l=b;m=0;n=0;o=d;break}m2(a,k,p,b);l=b;m=0;n=0;o=d}}while(0);b=k+240|0;d=k+8|0;p=f+24|0;q=f+28|0;r=l+8|0;s=l+16|0;t=l+64|0;u=(l|0)==(g|0);g=l+40|0;v=f+48|0;w=h|0;x=h+4|0;y=k+40|0;z=k|0;A=f+32|0;B=f+8|0;C=f+16|0;D=f+64|0;E=k+236|0;F=l+36|0;G=f+36|0;H=l+24|0;I=l+28|0;J=l|0;K=f|0;L=l+52|0;M=f+52|0;N=a+1248|0;O=a+1252|0;P=f+40|0;Q=l+44|0;R=f+44|0;S=m;m=0;T=n;n=o;L18899:while(1){if(u){o=S;U=T;V=n;while(1){if((U|0)<1&(o|0)==0){W=14070;break L18899}nB(f);X=oF(c[b>>2]|0,c[d>>2]|0,p,q,V,U,c[r>>2]|0,c[r+4>>2]|0,c[s>>2]|0,c[s+4>>2]|0,c[t>>2]|0,c[t+4>>2]|0)|0;c[s>>2]=0;c[s+4>>2]=-2147483648;c[r>>2]=0;c[r+4>>2]=-2147483648;c[t>>2]=-1;c[t+4>>2]=-1;Y=V+X|0;Z=U-X|0;X=(c[q>>2]|0)!=0;_=X&1;if(X){$=Y;aa=Z;ab=_;break}else{o=_;U=Z;V=Y}}}else{V=T;U=n;while(1){if((V|0)<=0){W=14070;break L18899}nB(f);o=oF(c[b>>2]|0,c[d>>2]|0,p,q,U,V,c[r>>2]|0,c[r+4>>2]|0,c[s>>2]|0,c[s+4>>2]|0,c[t>>2]|0,c[t+4>>2]|0)|0;c[s>>2]=0;c[s+4>>2]=-2147483648;c[r>>2]=0;c[r+4>>2]=-2147483648;c[t>>2]=-1;c[t+4>>2]=-1;Y=U+o|0;Z=V-o|0;if((c[q>>2]|0)==0){V=Z;U=Y}else{$=Y;aa=Z;ab=1;break}}}U=c[g>>2]|0;if((U|0)!=0){c[P>>2]=U;c[R>>2]=c[Q>>2];c[g>>2]=0;c[Q>>2]=0}c[v>>2]=0;U=c[d>>2]|0;do{if((c[U+8>>2]|0)==1){if((c[U+412>>2]|0)<=0){break}V=c[(c[b>>2]|0)+304>>2]|0;c[w>>2]=1;c[x>>2]=c[(c[d>>2]|0)+412>>2];Z=AX(V,(V|0)<0|0?-1:0,h,y,2)|0;c[v>>2]=Z}else{Z=U+104|0;if((c[Z>>2]|0)==0){break}if((c[U+108>>2]|0)==0){break}V=c[(c[b>>2]|0)+304>>2]|0;Y=AX(V,(V|0)<0|0?-1:0,Z,y,2)|0;c[v>>2]=Y}}while(0);c[A>>2]=c[z>>2];U=(c[b>>2]|0)+40|0;Y=c[U+4>>2]|0;c[B>>2]=c[U>>2];c[B+4>>2]=Y;Y=(c[b>>2]|0)+48|0;U=c[Y+4>>2]|0;c[C>>2]=c[Y>>2];c[C+4>>2]=U;U=(c[b>>2]|0)+288|0;Y=c[U+4>>2]|0;c[D>>2]=c[U>>2];c[D+4>>2]=Y;if((c[E>>2]|0)==1463898624){Y=(c[b>>2]|0)+8|0;U=c[Y+4>>2]|0;c[D>>2]=c[Y>>2];c[D+4>>2]=U}U=c[b>>2]|0;Y=c[U+224>>2]|0;if((Y|0)==(-1|0)){if((c[U+32>>2]|0)==1){W=14056}else{ac=U;W=14057}}else if((Y|0)==1){W=14056}else{ad=U}if((W|0)==14056){W=0;c[G>>2]=c[G>>2]|1;U=c[b>>2]|0;if((c[U+224>>2]|0)==-1){ac=U;W=14057}else{ad=U}}do{if((W|0)==14057){W=0;if((c[ac+32>>2]|0)!=0){ad=ac;break}if((c[F>>2]&1|0)==0){ad=ac;break}c[G>>2]=c[G>>2]|1;ad=c[b>>2]|0}}while(0);m2(a,k,ad,f);do{if((c[p>>2]|0)==(c[H>>2]|0)){if((c[q>>2]|0)!=(c[I>>2]|0)){break}c[K>>2]=c[J>>2];c[J>>2]=0;c[M>>2]=c[L>>2];c[L>>2]=0}}while(0);U=nH(f)|0;if((U|0)<0){ae=U;W=14074;break}Y=A7(88)|0;Z=Y;if((Y|0)==0){W=14069;break}if((c[N>>2]|0)==0){c[N>>2]=Z}else{c[(c[O>>2]|0)+80>>2]=Z}c[O>>2]=Z;NN(Y|0,j|0,80)|0;S=ab;m=U;T=aa;n=$}if((W|0)==14070){if(!u){ae=m;nJ(l);i=e;return ae|0}oH(c[b>>2]|0);c[b>>2]=0;ae=m;nJ(l);i=e;return ae|0}else if((W|0)==14074){nJ(l);i=e;return ae|0}else if((W|0)==14069){nJ(f);ae=-12;nJ(l);i=e;return ae|0}return 0}function m2(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0;g=i;i=i+112|0;h=g|0;j=g+32|0;k=g+64|0;l=g+96|0;m=g+104|0;n=c[b+1084>>2]|0;if((n&16|0)!=0){i=g;return}do{if((n&8|0)!=0){o=f+8|0;if((c[o>>2]|0)==0&(c[o+4>>2]|0)==(-2147483648|0)){break}o=f+16|0;c[o>>2]=0;c[o+4>>2]=-2147483648}}while(0);n=d+8|0;o=c[n>>2]|0;p=(e|0)==0;do{if((c[o+48>>2]|0)==28|p){q=o}else{if((c[e+32>>2]|0)!=3){q=o;break}c[o+176>>2]=1;q=c[n>>2]|0}}while(0);o=c[q+176>>2]|0;q=(o|0)==0;if(q|p){r=0}else{r=(c[e+32>>2]|0)!=3|0}p=f+8|0;s=c[p>>2]|0;t=c[p+4>>2]|0;do{if((s|0)==0&(t|0)==(-2147483648|0)){u=-2147483648;v=0}else{w=f+16|0;x=c[w>>2]|0;y=c[w+4>>2]|0;if((x|0)==0&(y|0)==(-2147483648|0)){u=t;v=s;break}z=c[d+188>>2]|0;if((z|0)>=63){u=t;v=s;break}A=NW(1,0,z-1|0)|0;B=NV(x,y,A,G)|0;A=G;if(!((A|0)>(t|0)|(A|0)==(t|0)&B>>>0>s>>>0)){u=t;v=s;break}C=d+208|0;D=c[C>>2]|0;E=c[C+4>>2]|0;C=2147352575;F=NW(1,0,z|0)|0;z=G;if((E|0)>(C|0)|(E|0)==(C|0)&D>>>0>-1>>>0|((A|0)>(E|0)|(A|0)==(E|0)&B>>>0>D>>>0)){D=NV(x,y,F,z)|0;c[w>>2]=D;c[w+4>>2]=G;u=t;v=s;break}else{w=NU(F,z,s,t)|0;z=G;c[p>>2]=w;c[p+4>>2]=z;u=z;v=w;break}}}while(0);t=(o|0)==1;do{if(t){s=f+16|0;w=c[s>>2]|0;z=c[s+4>>2]|0;if((w|0)!=(v|0)|(z|0)!=(u|0)|(w|0)==0&(z|0)==(-2147483648|0)|(r|0)==0){break}AN(b,48,1725680,(H=i,i=i+16|0,c[H>>2]=v,c[H+8>>2]=u,H)|0);i=H;if((a8(c[c[b+4>>2]>>2]|0,1725440)|0)==0){break}c[s>>2]=0;c[s+4>>2]=-2147483648}}while(0);u=f+48|0;v=c[u>>2]|0;if((v|0)==0){mm(l,m,d,e,f);s=c[m>>2]|0;do{if((s|0)==0){I=14101}else{m=c[l>>2]|0;if((m|0)==0){I=14101;break}z=c[d+44>>2]|0;w=N2(z,(z|0)<0|0?-1:0,m,(m|0)<0|0?-1:0)|0;m=G;z=c[d+40>>2]|0;F=N2(z,(z|0)<0|0?-1:0,s,(s|0)<0|0?-1:0)|0;z=AV(1,0,w,m,F,G,2)|0;F=z;c[u>>2]=F;J=F}}while(0);if((I|0)==14101){J=c[u>>2]|0}if((J|0)!=0){K=J;I=14103}}else{K=v;I=14103}L18987:do{if((I|0)==14103){v=b+1220|0;J=c[v>>2]|0;s=(J|0)==0;l=c[b+1248>>2]|0;if(s&(l|0)==0){break}F=c[f+32>>2]|0;z=h|0;m=j|0;w=k|0;D=(l|0)==0;y=D?J:l;x=d+200|0;B=c[x>>2]|0;E=c[x+4>>2]|0;L18990:do{if((B|0)==0&(E|0)==(-2147483648|0)){A=d+208|0;if((c[A>>2]|0)==(-1|0)&(c[A+4>>2]|0)==2147418111){L=2147418111;M=-1;N=y}else{break L18987}}else{L18993:do{if((y|0)!=0){A=K;C=(K|0)<0|0?-1:0;O=b+1252|0;P=y;Q=E;R=B;while(1){if((c[P+32>>2]|0)==(F|0)){S=P+8|0;T=P+16|0;U=c[T>>2]|0;V=c[T+4>>2]|0;if(!((c[S>>2]|0)==(U|0)&(c[S+4>>2]|0)==(V|0)&((U|0)==0&(V|0)==(-2147483648|0)))){W=V;X=U;break}if((c[P+48>>2]|0)!=0){W=-2147483648;X=0;break}U=NV(R,Q,A,C)|0;Y=G;Z=U}else{Y=Q;Z=R}U=c[P+80>>2]|0;if((U|0)!=0){P=U;Q=Y;R=Z;continue}if((c[O>>2]|0)!=(P|0)|s){break L18993}else{P=J;Q=Y;R=Z}}if((X|0)==(B|0)&(W|0)==(E|0)){c[x>>2]=R;c[x+4>>2]=Q;L=Q;M=R;N=D?J:l;break L18990}NM(z|0,0,32);a[z]=0;bc(z|0,32,1724096,(H=i,i=i+16|0,c[H>>2]=B,c[H+8>>2]=E,H)|0)|0;i=H;NM(m|0,0,32);a[m]=0;P=c[T>>2]|0;O=c[T+4>>2]|0;if((P|0)==0&(O|0)==(-2147483648|0)){bc(m|0,32,1724464,(H=i,i=i+1|0,i=i+7&-8,c[H>>2]=0,H)|0)|0;i=H}else{C=1724096;bc(m|0,32,C|0,(H=i,i=i+16|0,c[H>>2]=P,c[H+8>>2]=O,H)|0)|0;i=H}AN(b,48,1725152,(H=i,i=i+16|0,c[H>>2]=z,c[H+8>>2]=m,H)|0);i=H;break L18987}}while(0);O=b;NM(w|0,0,32);a[w]=0;P=1724096;bc(w|0,32,P|0,(H=i,i=i+16|0,c[H>>2]=B,c[H+8>>2]=E,H)|0)|0;i=H;AN(O,48,1724672,(H=i,i=i+8|0,c[H>>2]=w,H)|0);i=H;break L18987}}while(0);L19013:do{if((N|0)==0){_=L;$=M}else{w=K;E=(K|0)<0|0?-1:0;B=b+1252|0;m=N;z=L;l=M;while(1){if((c[m+32>>2]|0)==(F|0)){J=m+8|0;D=c[J>>2]|0;s=c[J+4>>2]|0;y=m+16|0;if(!((D|0)==(c[y>>2]|0)&(s|0)==(c[y+4>>2]|0))){break L18987}if(!((D|0)==0&(s|0)==(-2147483648|0))){if(!((D|0)==(c[x>>2]|0)&(s|0)==(c[x+4>>2]|0))){break L18987}}s=m+48|0;if((c[s>>2]|0)!=0){break L18987}c[y>>2]=l;c[y+4>>2]=z;if((c[(c[n>>2]|0)+176>>2]|0)==0){c[J>>2]=l;c[J+4>>2]=z}c[s>>2]=K;s=NU(l,z,w,E)|0;aa=G;ab=s}else{aa=z;ab=l}s=c[m+80>>2]|0;if((s|0)!=0){m=s;z=aa;l=ab;continue}if((c[B>>2]|0)!=(m|0)){_=aa;$=ab;break L19013}s=c[v>>2]|0;if((s|0)==0){_=aa;$=ab;break}else{m=s;z=aa;l=ab}}}}while(0);v=d+208|0;c[v>>2]=$;c[v+4>>2]=_}}while(0);_=(e|0)!=0;do{if(_){do{if((c[d+236>>2]|0)==3){$=c[f+28>>2]|0;if(($|0)==0){break}ab=e+184|0;aa=c[u>>2]|0;K=AW(c[ab>>2]|0,c[ab+4>>2]|0,aa,(aa|0)<0|0?-1:0,$,($|0)<0|0?-1:0)|0;$=G;aa=c[p>>2]|0;ab=c[p+4>>2]|0;if(!((aa|0)==0&(ab|0)==(-2147483648|0))){M=NU(aa,ab,K,$)|0;c[p>>2]=M;c[p+4>>2]=G}M=f+16|0;ab=c[M>>2]|0;aa=c[M+4>>2]|0;if((ab|0)==0&(aa|0)==(-2147483648|0)){break}L=NU(ab,aa,K,$)|0;c[M>>2]=L;c[M+4>>2]=G}}while(0);M=e+240|0;if((c[M>>2]|0)<=-1){break}L=c[n>>2]|0;$=c[L+108>>2]|0;K=c[d+40>>2]|0;aa=N2(K,(K|0)<0|0?-1:0,$,($|0)<0|0?-1:0)|0;$=G;K=0;if(!(($|0)>(K|0)|($|0)==(K|0)&aa>>>0>0>>>0)){break}K=c[L+104>>2]|0;L=c[d+44>>2]|0;ab=N2(L,(L|0)<0|0?-1:0,K,(K|0)<0|0?-1:0)|0;K=G;L=f+16|0;N=c[L>>2]|0;H=c[L+4>>2]|0;do{if((N|0)==0&(H|0)==(-2147483648|0)){T=d+192|0;W=c[T>>2]|0;X=c[T+4>>2]|0;if((W|0)==0&(X|0)==(-2147483648|0)){ac=-2147483648;ad=0;break}T=c[e+244>>2]|0;Z=N2(T,(T|0)<0|0?-1:0,ab,K)|0;T=N0(Z,G,aa,$)|0;Z=NU(T,G,W,X)|0;X=G;c[L>>2]=Z;c[L+4>>2]=X;W=c[e+248>>2]|0;T=N2(W,(W|0)<0|0?-1:0,ab,K)|0;W=N0(T,G,aa,$)|0;T=NU(W,G,Z,X)|0;c[p>>2]=T;c[p+4>>2]=G;ac=X;ad=Z}else{Z=c[e+244>>2]|0;X=N2(Z,(Z|0)<0|0?-1:0,ab,K)|0;Z=N0(X,G,aa,$)|0;X=NV(N,H,Z,G)|0;Z=d+192|0;c[Z>>2]=X;c[Z+4>>2]=G;Z=c[L>>2]|0;X=c[L+4>>2]|0;T=c[e+248>>2]|0;W=N2(T,(T|0)<0|0?-1:0,ab,K)|0;T=N0(W,G,aa,$)|0;W=NU(T,G,Z,X)|0;c[p>>2]=W;c[p+4>>2]=G;ac=X;ad=Z}}while(0);if((c[M>>2]|0)<=0){break}$=d+192|0;c[$>>2]=ad;c[$+4>>2]=ac}}while(0);ac=f+16|0;ad=c[ac>>2]|0;$=c[ac+4>>2]|0;aa=(ad|0)==0&($|0)==(-2147483648|0);if(aa){ae=r}else{K=c[p>>2]|0;ab=c[p+4>>2]|0;ae=((K|0)!=0|(ab|0)!=(-2147483648|0))&((ab|0)>($|0)|(ab|0)==($|0)&K>>>0>ad>>>0)?1:r}do{if(q|t&_){if((c[(c[n>>2]|0)+48>>2]|0)==28){break}if((ae|0)==0){r=c[p>>2]|0;K=c[p+4>>2]|0;ab=c[u>>2]|0;if((r|0)==0&(K|0)==(-2147483648|0)){if(aa&(ab|0)==0){break}c[p>>2]=ad;c[p+4>>2]=$;af=$;ag=ad}else{af=K;ag=r}m3(b,c[f+32>>2]|0,ag,af,ag,af,f);r=c[p>>2]|0;K=c[p+4>>2]|0;if((r|0)==0&(K|0)==(-2147483648|0)){L=d+208|0;H=c[L>>2]|0;N=c[L+4>>2]|0;c[p>>2]=H;c[p+4>>2]=N;c[ac>>2]=H;c[ac+4>>2]=N;if((H|0)==0&(N|0)==(-2147483648|0)){break}else{ah=N;ai=H}}else{c[ac>>2]=r;c[ac+4>>2]=K;ah=K;ai=r}r=NU(ai,ah,ab,(ab|0)<0|0?-1:0)|0;ab=d+208|0;c[ab>>2]=r;c[ab+4>>2]=G;break}if(aa){ab=d+216|0;r=c[ab>>2]|0;K=c[ab+4>>2]|0;c[ac>>2]=r;c[ac+4>>2]=K;aj=K;ak=r}else{aj=$;ak=ad}m3(b,c[f+32>>2]|0,ak,aj,c[p>>2]|0,c[p+4>>2]|0,f);r=c[ac>>2]|0;K=c[ac+4>>2]|0;if((r|0)==0&(K|0)==(-2147483648|0)){ab=d+208|0;H=c[ab>>2]|0;N=c[ab+4>>2]|0;c[ac>>2]=H;c[ac+4>>2]=N;al=N;am=H}else{al=K;am=r}r=d+224|0;K=c[r>>2]|0;if((K|0)==0){H=c[u>>2]|0;c[r>>2]=H;an=c[ac+4>>2]|0;ao=c[ac>>2]|0;ap=H}else{an=al;ao=am;ap=K}if(!((ao|0)==0&(an|0)==(-2147483648|0))){K=NU(ap,(ap|0)<0|0?-1:0,ao,an)|0;H=d+208|0;c[H>>2]=K;c[H+4>>2]=G}c[r>>2]=c[u>>2];r=c[p+4>>2]|0;H=d+216|0;c[H>>2]=c[p>>2];c[H+4>>2]=r}}while(0);u=c[p>>2]|0;an=c[p+4>>2]|0;L19081:do{if(((u|0)!=0|(an|0)!=(-2147483648|0))&(o|0)<17){ao=c[n>>2]|0;do{if((c[ao+48>>2]|0)==28){if((c[d+184>>2]|0)==0){break}ap=c[ao+176>>2]|0;if((ap|0)<3){if((c[d+452>>2]|0)>6){break}else{break L19081}}am=c[d+452>>2]|0;if((ap|0)<4){if((am|0)>17){break}else{break L19081}}else{if((am|0)>19){break}else{break L19081}}}}while(0);ao=d+264|0;c[ao>>2]=u;c[ao+4>>2]=an;L19093:do{if((o|0)>0){M=0;while(1){am=M+1|0;ap=d+264+(am<<3)|0;al=c[ap>>2]|0;aj=c[ap+4>>2]|0;if(!((an|0)>(aj|0)|(an|0)==(aj|0)&u>>>0>al>>>0)){break L19093}ak=d+264+(M<<3)|0;c[ap>>2]=u;c[ap+4>>2]=an;c[ak>>2]=al;c[ak+4>>2]=aj;if((am|0)<(o|0)){M=am}else{break}}}}while(0);if(!((c[ac>>2]|0)==0&(c[ac+4>>2]|0)==(-2147483648|0))){break}M=c[ao+4>>2]|0;c[ac>>2]=c[ao>>2];c[ac+4>>2]=M}}while(0);if((c[(c[n>>2]|0)+48>>2]|0)==28){m3(b,c[f+32>>2]|0,c[ac>>2]|0,c[ac+4>>2]|0,c[p>>2]|0,c[p+4>>2]|0,f)}p=c[ac>>2]|0;b=c[ac+4>>2]|0;ac=d+208|0;d=c[ac+4>>2]|0;if((b|0)>(d|0)|(b|0)==(d|0)&p>>>0>(c[ac>>2]|0)>>>0){c[ac>>2]=p;c[ac+4>>2]=b}b=c[n>>2]|0;do{if((c[b+8>>2]|0)==0){n=o3(b)|0;if((n|0)==0){ac=nT(c[b+48>>2]|0)|0;o4(b,ac);if((ac|0)==0){break}else{aq=ac}}else{aq=n}if((c[aq+16>>2]&1|0)!=0){I=14192}}else{I=14192}}while(0);if((I|0)==14192){I=f+36|0;c[I>>2]=c[I>>2]|1}if(!_){i=g;return}_=e+232|0;e=c[_+4>>2]|0;I=f+72|0;c[I>>2]=c[_>>2];c[I+4>>2]=e;i=g;return}function m3(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0;j=i;i=i+136|0;k=j|0;l=a+28|0;m=c[(c[l>>2]|0)+(b<<2)>>2]|0;n=c[a+1248>>2]|0;if((n|0)==0){o=c[a+1220>>2]|0}else{o=n}n=m+200|0;if((c[n>>2]|0)!=0|(c[n+4>>2]|0)!=(-2147483648|0)|(d|0)==0&(e|0)==(-2147483648|0)){i=j;return}p=m+208|0;q=c[p>>2]|0;r=c[p+4>>2]|0;s=2147352575;if((q|0)==0&(r|0)==(-2147483648|0)|((e|0)>(s|0)|(e|0)==(s|0)&d>>>0>-1>>>0)){i=j;return}s=m+8|0;t=c[(c[s>>2]|0)+176>>2]|0;u=NU(d,e,-1,2147418111)|0;v=NV(u,G,q,r)|0;r=G;c[n>>2]=v;c[n+4>>2]=r;c[p>>2]=d;c[p+4>>2]=e;e=k|0;c[e>>2]=0;c[e+4>>2]=-2147483648;d=k+8|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+16|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+24|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+32|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+40|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+48|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+56|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+64|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+72|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+80|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+88|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+96|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+104|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+112|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+120|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=k+128|0;c[d>>2]=0;c[d+4>>2]=-2147483648;d=NU(v,r,1,-2147418112)|0;r=G;v=2147352576;q=(g|0)<(v|0)|(g|0)==(v|0)&f>>>0<0>>>0;v=NU(q?0:d,q?0:r,f,g)|0;g=G;L19128:do{if((o|0)!=0){f=m+48|0;q=(t|0)<17;u=m+184|0;w=m+452|0;x=(t|0)>0;y=a+1252|0;z=a+1220|0;A=o;while(1){L19132:do{if((c[A+32>>2]|0)==(b|0)){B=A+8|0;C=c[B>>2]|0;D=c[B+4>>2]|0;E=2147352575;if((D|0)>(E|0)|(D|0)==(E|0)&C>>>0>-1>>>0){E=NU(C,D,d,r)|0;F=G;c[B>>2]=E;c[B+4>>2]=F;H=F;I=E}else{H=D;I=C}C=A+16|0;D=c[C>>2]|0;E=c[C+4>>2]|0;F=2147352575;if((E|0)>(F|0)|(E|0)==(F|0)&D>>>0>-1>>>0){F=NU(D,E,d,r)|0;c[C>>2]=F;c[C+4>>2]=G}if((c[f>>2]|0)==0&(c[f+4>>2]|0)==(-2147483648|0)){if((I|0)==0&(H|0)==(-2147483648|0)){break}c[f>>2]=I;c[f+4>>2]=H;J=c[B+4>>2]|0;K=c[B>>2]|0}else{J=H;K=I}if(!(((K|0)!=0|(J|0)!=(-2147483648|0))&q)){break}B=c[s>>2]|0;do{if((c[B+48>>2]|0)==28){if((c[u>>2]|0)==0){break}F=c[B+176>>2]|0;if((F|0)<3){if((c[w>>2]|0)>6){break}else{break L19132}}E=c[w>>2]|0;if((F|0)<4){if((E|0)>17){break}else{break L19132}}else{if((E|0)>19){break}else{break L19132}}}}while(0);c[e>>2]=K;c[e+4>>2]=J;L19155:do{if(x){B=0;while(1){E=B+1|0;F=k+(E<<3)|0;D=c[F>>2]|0;L=c[F+4>>2]|0;if(!((J|0)>(L|0)|(J|0)==(L|0)&K>>>0>D>>>0)){break L19155}M=k+(B<<3)|0;c[F>>2]=K;c[F+4>>2]=J;c[M>>2]=D;c[M+4>>2]=L;if((E|0)<(t|0)){B=E}else{break}}}}while(0);if(!((c[C>>2]|0)==0&(c[C+4>>2]|0)==(-2147483648|0))){break}B=c[e+4>>2]|0;c[C>>2]=c[e>>2];c[C+4>>2]=B}}while(0);B=c[A+80>>2]|0;if((B|0)!=0){A=B;continue}if((c[y>>2]|0)!=(A|0)){break L19128}A=c[z>>2]|0;if((A|0)==0){break}}}}while(0);do{if((c[a+1208>>2]|0)==0){N=g;O=v}else{e=m+188|0;t=c[e>>2]|0;if((t|0)>=63){N=g;O=v;break}J=m+464|0;if(!((c[J>>2]|0)==0&(c[J+4>>2]|0)==(-2147483648|0))){N=g;O=v;break}K=c[n>>2]|0;k=c[n+4>>2]|0;if((K|0)==0&(k|0)==(-2147483648|0)){N=g;O=v;break}s=c[m+44>>2]|0;I=c[m+40>>2]|0;H=AW(60,0,s,(s|0)<0|0?-1:0,I,(I|0)<0|0?-1:0)|0;I=G;s=NV(K,k,H,I)|0;r=G;d=NW(1,0,t|0)|0;o=G;A=NW(1,0,t-3|0)|0;t=NV(d,o,A,G)|0;A=G;if((k|0)<(A|0)|(k|0)==(A|0)&K>>>0<t>>>0){P=1}else{t=NV(d,o,H,I)|0;I=G;P=(k|0)<(I|0)|(k|0)==(I|0)&K>>>0<t>>>0?1:-1}t=a+1104|0;K=c[t>>2]|0;L19173:do{if((K|0)==0){Q=14242}else{I=a+1108|0;k=c[I>>2]|0;H=0;L19175:while(1){R=c[k+(H<<2)>>2]|0;L19177:do{if((R|0)!=0){S=c[R+16>>2]|0;o=R+12|0;d=0;while(1){if(d>>>0>=S>>>0){break L19177}T=c[o>>2]|0;if((c[T+(d<<2)>>2]|0)==(b|0)){U=R;break L19175}else{d=d+1|0}}}}while(0);C=H+1|0;if(C>>>0<K>>>0){H=C}else{Q=14242;break L19173}}L19183:while(1){H=U+56|0;V=c[H>>2]|0;W=c[H+4>>2]|0;if((V|0)==0&(W|0)==(-2147483648|0)){X=U;Y=0}else{Q=14256;break}L19185:while(1){Z=c[k+(Y<<2)>>2]|0;L19187:do{if((Z|0)==(X|0)){_=0}else{if((X|0)!=0){_=X;break}H=c[Z+16>>2]|0;C=Z+12|0;d=0;while(1){if(d>>>0>=H>>>0){_=0;break L19187}if((c[(c[C>>2]|0)+(d<<2)>>2]|0)==(b|0)){break L19185}else{d=d+1|0}}}}while(0);d=Y+1|0;if(d>>>0<K>>>0){X=_;Y=d}else{$=P;aa=r;ab=s;break L19183}}if((Z|0)==0){$=P;aa=r;ab=s;break}else{U=Z}}if((Q|0)==14256){$=c[U+64>>2]|0;aa=W;ab=V}k=R;d=K;C=S;H=T;while(1){o=k+56|0;if((c[o>>2]|0)==(ab|0)&(c[o+4>>2]|0)==(aa|0)){ac=d}else{A=k+16|0;L19202:do{if((C|0)!=0){z=k+12|0;y=0;x=H;while(1){w=(c[(c[l>>2]|0)+(c[x+(y<<2)>>2]<<2)>>2]|0)+464|0;c[w>>2]=ab;c[w+4>>2]=aa;c[(c[(c[l>>2]|0)+(c[(c[z>>2]|0)+(y<<2)>>2]<<2)>>2]|0)+472>>2]=$;w=y+1|0;if(w>>>0>=(c[A>>2]|0)>>>0){break L19202}y=w;x=c[z>>2]|0}}}while(0);c[o>>2]=ab;c[o+4>>2]=aa;c[k+64>>2]=$;ac=c[t>>2]|0}if((ac|0)==0){break L19173}A=c[I>>2]|0;z=k;x=0;L19210:while(1){ad=c[A+(x<<2)>>2]|0;L19212:do{if((ad|0)==(z|0)){ae=0}else{if((z|0)!=0){ae=z;break}af=c[ad+16>>2]|0;y=ad+12|0;w=0;while(1){if(w>>>0>=af>>>0){ae=0;break L19212}ag=c[y>>2]|0;if((c[ag+(w<<2)>>2]|0)==(b|0)){break L19210}else{w=w+1|0}}}}while(0);w=x+1|0;if(w>>>0<ac>>>0){z=ae;x=w}else{break L19173}}if((ad|0)==0){break}else{k=ad;d=ac;C=af;H=ag}}}}while(0);do{if((Q|0)==14242){t=a+24|0;K=c[t>>2]|0;H=(K|0)==0;L19222:do{if(H){ah=-1}else{C=c[l>>2]|0;d=-1;k=0;while(1){I=c[C+(k<<2)>>2]|0;x=c[(c[I+8>>2]|0)+8>>2]|0;if((x|0)==0){if((c[I+72>>2]&1024|0)==0){ah=k;break L19222}}if((d|0)<0){ai=(x|0)==1?k:d}else{ai=d}x=k+1|0;if(x>>>0<K>>>0){d=ai;k=x}else{break}}ah=(ai|0)<0?0:ai}}while(0);K=c[l>>2]|0;k=(c[K+(ah<<2)>>2]|0)+464|0;d=c[k>>2]|0;C=c[k+4>>2]|0;if(!((d|0)==0&(C|0)==(-2147483648|0))){c[J>>2]=d;c[J+4>>2]=C;c[m+472>>2]=c[(c[(c[l>>2]|0)+(ah<<2)>>2]|0)+472>>2];break}if(H){break}C=(c[K>>2]|0)+464|0;c[C>>2]=s;c[C+4>>2]=r;c[(c[c[l>>2]>>2]|0)+472>>2]=P;if((c[t>>2]|0)>>>0>1){aj=1}else{break}do{C=(c[(c[l>>2]|0)+(aj<<2)>>2]|0)+464|0;c[C>>2]=s;c[C+4>>2]=r;c[(c[(c[l>>2]|0)+(aj<<2)>>2]|0)+472>>2]=P;aj=aj+1|0;}while(aj>>>0<(c[t>>2]|0)>>>0)}}while(0);r=m+472|0;if((c[r>>2]|0)!=-1){N=g;O=v;break}s=c[n>>2]|0;t=c[n+4>>2]|0;H=c[J>>2]|0;C=c[J+4>>2]|0;K=(H|0)==0&(C|0)==(-2147483648|0);if(K|(s|0)==0&(t|0)==(-2147483648|0)|((C|0)>(t|0)|(C|0)==(t|0)&H>>>0>s>>>0)){ak=t;al=s}else{d=NW(1,0,c[e>>2]|0)|0;k=NV(s,t,d,G)|0;ak=G;al=k}c[n>>2]=al;c[n+4>>2]=ak;k=c[p>>2]|0;d=c[p+4>>2]|0;if(K|(k|0)==0&(d|0)==(-2147483648|0)|((C|0)>(d|0)|(C|0)==(d|0)&H>>>0>k>>>0)){am=d;an=k}else{t=NW(1,0,c[e>>2]|0)|0;s=NV(k,d,t,G)|0;am=G;an=s}c[p>>2]=an;c[p+4>>2]=am;s=h+16|0;t=c[s>>2]|0;d=c[s+4>>2]|0;if(K|(t|0)==0&(d|0)==(-2147483648|0)|((C|0)>(d|0)|(C|0)==(d|0)&H>>>0>t>>>0)){ao=d;ap=t}else{H=NW(1,0,c[e>>2]|0)|0;C=NV(t,d,H,G)|0;ao=G;ap=C}c[s>>2]=ap;c[s+4>>2]=ao;s=h+8|0;C=c[s>>2]|0;H=c[s+4>>2]|0;d=c[r>>2]|0;do{if((d|0)==0){aq=H;ar=C}else{t=c[J>>2]|0;K=c[J+4>>2]|0;if((t|0)==0&(K|0)==(-2147483648|0)|(C|0)==0&(H|0)==(-2147483648|0)){aq=H;ar=C;break}k=(K|0)>(H|0)|(K|0)==(H|0)&t>>>0>C>>>0;if((d|0)==1&k){t=NW(1,0,c[e>>2]|0)|0;K=NU(t,G,C,H)|0;aq=G;ar=K;break}if((d|0)!=-1|k){aq=H;ar=C;break}k=NW(1,0,c[e>>2]|0)|0;K=NV(C,H,k,G)|0;aq=G;ar=K}}while(0);c[s>>2]=ar;c[s+4>>2]=aq;H=c[r>>2]|0;if((H|0)==0){N=g;O=v;break}C=c[J>>2]|0;d=c[J+4>>2]|0;if((C|0)==0&(d|0)==(-2147483648|0)|(v|0)==0&(g|0)==(-2147483648|0)){N=g;O=v;break}K=(d|0)>(g|0)|(d|0)==(g|0)&C>>>0>v>>>0;if((H|0)==1&K){C=NW(1,0,c[e>>2]|0)|0;d=NU(C,G,v,g)|0;N=G;O=d;break}if((H|0)!=-1|K){N=g;O=v;break}K=NW(1,0,c[e>>2]|0)|0;H=NV(v,g,K,G)|0;N=G;O=H}}while(0);g=m+48|0;if(!((c[g>>2]|0)==0&(c[g+4>>2]|0)==(-2147483648|0))){i=j;return}c[g>>2]=O;c[g+4>>2]=N;i=j;return}function m4(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=(NL(b|0)|0)+8|0;c[d>>2]=0;if((a|0)==0){f=e;return f|0}b=zJ(a,3119024,0,2)|0;if((b|0)==0){f=e;return f|0}else{g=e;h=b}while(1){b=NL(c[h>>2]|0)|0;e=g+5+b+(NL(c[h+4>>2]|0)|0)|0;c[d>>2]=(c[d>>2]|0)+1;b=zJ(a,3119024,h,2)|0;if((b|0)==0){f=e;break}else{g=e;h=b}}return f|0}function m5(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0;g=NL(e|0)|0;h=c[b>>2]|0;y=g;a[h]=y&255;y=y>>8;a[h+1|0]=y&255;y=y>>8;a[h+2|0]=y&255;y=y>>8;a[h+3|0]=y&255;h=(c[b>>2]|0)+4|0;c[b>>2]=h;g=NL(e|0)|0;NN(h|0,e|0,g)|0;e=(c[b>>2]|0)+g|0;c[b>>2]=e;g=e;if((c[d>>2]|0)==0){y=0;a[g]=y&255;y=y>>8;a[g+1|0]=y&255;y=y>>8;a[g+2|0]=y&255;y=y>>8;a[g+3|0]=y&255;c[b>>2]=(c[b>>2]|0)+4;return 0}y=f;a[g]=y&255;y=y>>8;a[g+1|0]=y&255;y=y>>8;a[g+2|0]=y&255;y=y>>8;a[g+3|0]=y&255;c[b>>2]=(c[b>>2]|0)+4;g=zJ(c[d>>2]|0,3119024,0,2)|0;if((g|0)==0){return 0}else{i=g}do{g=i|0;f=NL(c[g>>2]|0)|0;e=i+4|0;h=NL(c[e>>2]|0)|0;j=c[b>>2]|0;y=f+1+h|0;a[j]=y&255;y=y>>8;a[j+1|0]=y&255;y=y>>8;a[j+2|0]=y&255;y=y>>8;a[j+3|0]=y&255;j=(c[b>>2]|0)+4|0;c[b>>2]=j;k=c[g>>2]|0;NN(j|0,k|0,f)|0;k=(c[b>>2]|0)+f|0;c[b>>2]=k;a[k]=61;k=(c[b>>2]|0)+1|0;c[b>>2]=k;f=c[e>>2]|0;NN(k|0,f|0,h)|0;c[b>>2]=(c[b>>2]|0)+h;i=zJ(c[d>>2]|0,3119024,i,2)|0;}while((i|0)!=0);return 0}function m6(a){a=a|0;var b=0,d=0;if((c[a+8>>2]|0)<33){b=0;return b|0}d=c[a+4>>2]|0;do{if((NO(d+8|0,1682840,4)|0)==0){if((NO(d|0,1680824,4)|0)==0){b=99;return b|0}if((NO(d|0,1679104,4)|0)!=0){break}if((NO(d+12|0,1677016,4)|0)==0){b=100}else{break}return b|0}}while(0);b=0;return b|0}function m7(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0;d=i;i=i+400|0;e=d|0;f=d+264|0;g=b+16|0;h=c[g>>2]|0;j=c[b+12>>2]|0;k=j+24|0;c[k>>2]=-1;c[k+4>>2]=-1;l=iI(h)|0;m=(l|0)==875972178;if(!((l|0)==1179011410|(l|0)==875972178)){n=-1094995529;i=d;return n|0}iI(h)|0;if((iI(h)|0)!=1163280727){n=-1094995529;i=d;return n|0}do{if(m){if((iI(h)|0)!=875983716){n=-1094995529;i=d;return n|0}l=iI(h)|0;o=l;p=0;if(l>>>0<24){n=-1094995529;i=d;return n|0}iJ(h)|0;l=iJ(h)|0;q=G;r=iJ(h)|0;s=G;t=s|q;u=0;if(!((t|0)<(u|0)|(t|0)==(u|0)&(r|l)>>>0<0>>>0)){u=NU(o,p,-24,-1)|0;p=G;io(h,u,p)|0;v=q;w=l;x=s;z=r;break}AN(b,16,1698112,(A=i,i=i+32|0,c[A>>2]=l,c[A+8>>2]=q,c[A+16>>2]=r,c[A+24>>2]=s,A)|0);i=A;n=-1094995529;i=d;return n|0}else{v=0;w=0;x=0;z=0}}while(0);s=b;r=j+8|0;q=h+88|0;l=m^1;p=f|0;f=e|0;u=e+256|0;o=e+32|0;t=e+10|0;B=e+8|0;e=b+1140|0;C=-1;D=-1;E=0;F=v;v=w;w=x;x=z;z=0;L19322:while(1){H=iI(h)|0;I=iI(h)|0;J=I;K=0;L=il(h,0,0,1)|0;M=NU(L,G,J,K)|0;L=G;if((iq(h)|0)!=0){N=C;O=D;P=F;Q=v;R=w;S=x;T=z;break}do{if((H|0)==1952670054){if(!((x|0)==0&(w|0)==0)){U=C;V=D;W=L;X=M;Y=E;Z=F;_=v;$=w;aa=x;ab=z;break}U=C;V=D;W=L;X=M;Y=E;Z=F;_=v;$=0;aa=iI(h)|0;ab=z}else if((H|0)==544501094){if((E|0)!=0){AN(s,24,1676e3,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;U=C;V=D;W=L;X=M;Y=1;Z=F;_=v;$=w;aa=x;ab=z;break}ac=c[g>>2]|0;ad=mK(b,0)|0;if((ad|0)==0){n=-12;ae=14437;break L19322}af=ad+8|0;ag=l3(ac,c[af>>2]|0,I)|0;if((ag|0)<0){n=ag;ae=14421;break L19322}ag=c[af>>2]|0;if((c[ag+48>>2]|0)==65536){c[ad+440>>2]=50;af=ad+228|0;ac=c[af>>2]|0;c[af>>2]=(ac|0)>4?4:ac}c[ad+236>>2]=1463898624;mL(ad,64,1,c[ag+412>>2]|0);U=C;V=D;W=L;X=M;Y=1;Z=F;_=v;$=w;aa=x;ab=ad}else if((H|0)==1954047330){ad=iE(c[g>>2]|0,f,256)|0;if((ad|0)<0){n=ad;ae=14428;break L19322}a[u]=0;if((a[f]|0)!=0){ad=zK(e,1737048,f,0)|0;if((ad|0)<0){n=ad;ae=14432;break L19322}}ad=iE(c[g>>2]|0,f,32)|0;if((ad|0)<0){n=ad;ae=14423;break L19322}a[o]=0;if((a[f]|0)!=0){ad=zK(e,1727400,f,0)|0;if((ad|0)<0){n=ad;ae=14426;break L19322}}ad=iE(c[g>>2]|0,f,32)|0;if((ad|0)<0){n=ad;ae=14434;break L19322}a[o]=0;if((a[f]|0)!=0){ad=zK(e,1706592,f,0)|0;if((ad|0)<0){n=ad;ae=14427;break L19322}}ad=iE(c[g>>2]|0,f,10)|0;if((ad|0)<0){n=ad;ae=14439;break L19322}a[t]=0;if((a[f]|0)!=0){ad=zK(e,1719400,f,0)|0;if((ad|0)<0){n=ad;ae=14419;break L19322}}ad=iE(c[g>>2]|0,f,8)|0;if((ad|0)<0){n=ad;ae=14420;break L19322}a[B]=0;if((a[f]|0)!=0){ad=zK(e,1712240,f,0)|0;if((ad|0)<0){n=ad;ae=14418;break L19322}}ad=iJ(c[g>>2]|0)|0;ag=G;bc(p|0,131,1704608,(A=i,i=i+16|0,c[A>>2]=ad,c[A+8>>2]=ag,A)|0)|0;i=A;ag=zK(e,1702424,p,0)|0;if((ag|0)<0){n=ag;ae=14435;break L19322}ag=(iG(c[g>>2]|0)|0)==0;ad=c[g>>2]|0;if(ag){ag=254;ac=0;io(ad,ag,ac)|0;}else{ac=iQ(ad)|0;ad=G;ag=iQ(c[g>>2]|0)|0;af=G;ah=iQ(c[g>>2]|0)|0;ai=G;aj=iQ(c[g>>2]|0)|0;ak=G;al=iQ(c[g>>2]|0)|0;am=G;an=iQ(c[g>>2]|0)|0;ao=G;ap=iQ(c[g>>2]|0)|0;aq=G;ar=iQ(c[g>>2]|0)|0;as=G;if(!((ag|ac|ah|aj|al|an|ap|ar|0)==0&(af|ad|ai|ak|am|ao|aq|as|0)==0)){do{if((an|al|0)==0&(ao|am|0)==0){if(!((ar|ap|0)==0&(as|aq|0)==0)){at=0;au=0;ae=14380;break}av=1699560;bc(p|0,131,av|0,(A=i,i=i+64|0,c[A>>2]=ac,c[A+8>>2]=ad,c[A+16>>2]=ag,c[A+24>>2]=af,c[A+32>>2]=ah,c[A+40>>2]=ai,c[A+48>>2]=aj,c[A+56>>2]=ak,A)|0)|0;i=A}else{at=ao;au=an;ae=14380}}while(0);if((ae|0)==14380){ae=0;an=1697048;bc(p|0,131,an|0,(A=i,i=i+128|0,c[A>>2]=ac,c[A+8>>2]=ad,c[A+16>>2]=ag,c[A+24>>2]=af,c[A+32>>2]=ah,c[A+40>>2]=ai,c[A+48>>2]=aj,c[A+56>>2]=ak,c[A+64>>2]=al,c[A+72>>2]=am,c[A+80>>2]=au,c[A+88>>2]=at,c[A+96>>2]=ap,c[A+104>>2]=aq,c[A+112>>2]=ar,c[A+120>>2]=as,A)|0)|0;i=A}an=zK(e,1694872,p,0)|0;if((an|0)<0){n=an;ae=14429;break L19322}}an=c[g>>2]|0;ao=190;av=0;io(an,ao,av)|0;}if(I>>>0<=602){U=C;V=D;W=L;X=M;Y=E;Z=F;_=v;$=w;aa=x;ab=z;break}av=A1(I-601|0)|0;if((av|0)==0){n=-12;ae=14416;break L19322}ao=I-602|0;an=iE(c[g>>2]|0,av,ao)|0;if((an|0)<0){n=an;ae=14436;break L19322}a[av+ao|0]=0;ao=zK(e,1691328,av,8)|0;if((ao|0)<0){n=ao;ae=14430;break L19322}else{U=C;V=D;W=L;X=M;Y=E;Z=F;_=v;$=w;aa=x;ab=z}}else if((H|0)==810962259){ae=14388;break L19322}else if((H|0)==1414744396){if(I>>>0<4){ae=14397;break L19322}if((iI(h)|0)!=1330007625){U=C;V=D;W=L;X=M;Y=E;Z=F;_=v;$=w;aa=x;ab=z;break}ao=NU(J,K,-4,-1)|0;l4(b,ao,G)|0;U=C;V=D;W=L;X=M;Y=E;Z=F;_=v;$=w;aa=x;ab=z}else if((H|0)==1635017060){if((E|0)==0){ae=14351;break L19322}if(m){ao=il(h,0,0,1)|0;av=NU(ao,G,v,F)|0;aw=F;ax=v;ay=G;az=av}else{av=(I|0)!=0;aw=K;ax=J;ay=av?L:2147483647;az=av?M:-1}c[r>>2]=az;c[r+4>>2]=ay;av=il(h,0,0,1)|0;ao=G;if((c[q>>2]|0)==0){N=ao;O=av;P=aw;Q=ax;R=w;S=x;T=z;break L19322}if((I|0)==0&l){N=ao;O=av;P=aw;Q=ax;R=w;S=x;T=z;break L19322}else{U=ao;V=av;W=ay;X=az;Y=E;Z=aw;_=ax;$=w;aa=x;ab=z}}else{U=C;V=D;W=L;X=M;Y=E;Z=F;_=v;$=w;aa=x;ab=z}}while(0);M=ip(h)|0;L=G;J=0;if((L|0)>(J|0)|(L|0)==(J|0)&M>>>0>0>>>0){M=ip(h)|0;J=G;if(!((W|0)<(J|0)|(W|0)==(J|0)&X>>>0<M>>>0)){N=U;O=V;P=Z;Q=_;R=$;S=aa;T=ab;break}}M=(X|0)==(-1|0)&(W|0)==2147483647;J=NU(M?0:X&1,M?0:W&0,X,W)|0;M=il(h,J,G,0)|0;J=G;L=0;if((J|0)<(L|0)|(J|0)==(L|0)&M>>>0<0>>>0){N=U;O=V;P=Z;Q=_;R=$;S=aa;T=ab;break}else{C=U;D=V;E=Y;F=Z;v=_;w=$;x=aa;z=ab}}do{if((ae|0)==14418){i=d;return n|0}else if((ae|0)==14419){i=d;return n|0}else if((ae|0)==14420){i=d;return n|0}else if((ae|0)==14421){i=d;return n|0}else if((ae|0)==14423){i=d;return n|0}else if((ae|0)==14416){i=d;return n|0}else if((ae|0)==14434){i=d;return n|0}else if((ae|0)==14435){i=d;return n|0}else if((ae|0)==14436){i=d;return n|0}else if((ae|0)==14437){i=d;return n|0}else if((ae|0)==14439){i=d;return n|0}else if((ae|0)==14426){i=d;return n|0}else if((ae|0)==14427){i=d;return n|0}else if((ae|0)==14428){i=d;return n|0}else if((ae|0)==14429){i=d;return n|0}else if((ae|0)==14430){i=d;return n|0}else if((ae|0)==14432){i=d;return n|0}else if((ae|0)==14351){AN(s,16,1660296,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;n=-1094995529;i=d;return n|0}else if((ae|0)==14388){if((E|0)==0){AN(s,16,1643152,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;n=-1094995529;i=d;return n|0}if((I|0)!=808464944){AN(s,16,1631304,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;N=C;O=D;P=F;Q=v;R=w;S=x;T=z;break}AN(s,48,1624912,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;c[j+68>>2]=0;ab=mK(b,0)|0;if((ab|0)==0){n=-12;i=d;return n|0}iD(h)|0;c[ab+4>>2]=1;aa=ab+8|0;c[(c[aa>>2]|0)+8>>2]=0;c[(c[aa>>2]|0)+48>>2]=1397577290;$=iH(h)|0;c[(c[aa>>2]|0)+120>>2]=$;$=iH(h)|0;c[(c[aa>>2]|0)+124>>2]=$;c[(c[aa>>2]|0)+100>>2]=4;$=A1((c[(c[aa>>2]|0)+100>>2]|0)+16|0)|0;c[(c[aa>>2]|0)+96>>2]=$;if((c[(c[aa>>2]|0)+96>>2]|0)!=0){$=iH(h)|0;_=il(h,0,0,1)|0;Z=G;Y=N2($,0,3,0)|0;$=G;V=NU(_,Z,-15,-1)|0;Z=NU(V,G,Y,$)|0;c[k>>2]=Z;c[k+4>>2]=G;iH(h)|0;c[j+32>>2]=iH(h)|0;mL(ab,32,1,iH(h)|0);Z=ab+56|0;c[Z>>2]=iH(h)|0;c[Z+4>>2]=0;iH(h)|0;iH(h)|0;Z=iH(h)|0;c[j+36>>2]=Z;ab=c[(c[aa>>2]|0)+96>>2]|0;y=Z;a[ab]=y&255;y=y>>8;a[ab+1|0]=y&255;y=y>>8;a[ab+2|0]=y&255;y=y>>8;a[ab+3|0]=y&255;c[j+64>>2]=0;N=C;O=D;P=F;Q=v;R=w;S=x;T=z;break}AN(s,16,1618856,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;n=-12;i=d;return n|0}else if((ae|0)==14397){AN(s,16,1614440,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;n=-1094995529;i=d;return n|0}}while(0);z=0;if((N|0)<(z|0)|(N|0)==(z|0)&O>>>0<0>>>0){AN(s,16,1743576,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;n=-1094995529;i=d;return n|0}il(h,O,N,0)|0;do{if((S|0)==0&(R|0)==0){N=T+8|0;O=c[N>>2]|0;if((c[O+416>>2]|0)==0){break}if((pc(c[O+48>>2]|0)|0)==0){break}O=c[r>>2]|0;A=c[r+4>>2]|0;s=ip(h)|0;z=G;if((A|0)>(z|0)|(A|0)==(z|0)&O>>>0>s>>>0){break}s=c[N>>2]|0;N=c[s+416>>2]|0;O=pc(c[s+48>>2]|0)|0;s=N2(O,(O|0)<0|0?-1:0,N,(N|0)<0|0?-1:0)|0;N=N3(Q<<3|0>>>29,P<<3|Q>>>29,s,G)|0;s=G;if(!((N|0)==0&(s|0)==0)){aA=s;aB=N;ae=14410}}else{aA=R;aB=S;ae=14410}}while(0);if((ae|0)==14410){ae=T+56|0;c[ae>>2]=aB;c[ae+4>>2]=aA}jy(b,0,816);jy(b,0,242440);n=0;i=d;return n|0}function m8(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0;d=c[a+12>>2]|0;e=d+24|0;f=c[e+4>>2]|0;g=0;h=a+28|0;if((f|0)>(g|0)|(f|0)==(g|0)&(c[e>>2]|0)>>>0>0>>>0){i=d+52|0;j=d+48|0;k=a+16|0}else{l=a+16|0;m=14462}L19444:while(1){if((m|0)==14462){m=0;n=c[c[h>>2]>>2]|0;o=d+8|0;a=c[o>>2]|0;g=c[o+4>>2]|0;f=il(c[l>>2]|0,0,0,1)|0;p=NV(a,g,f,G)|0;f=(c[d+56>>2]|0)==0;g=f?p:2147483647;p=f?G:0;f=0;if(!((p|0)<(f|0)|(p|0)==(f|0)&g>>>0<1>>>0)){q=p;r=g;break}g=c[l>>2]|0;if((iq(g)|0)==0){do{p=iI(g)|0;s=iI(g)|0;t=0;if((p|0)==1635017060){m=14468;break L19444}p=NU(s&1,t&0,s,t)|0;il(g,p,G,1)|0;}while((iq(g)|0)==0)}g=d+52|0;c[g>>2]=1;p=c[e+4>>2]|0;f=0;if(!((p|0)>(f|0)|(p|0)==(f|0)&(c[e>>2]|0)>>>0>0>>>0)){u=-541478725;m=14475;break}f=d+48|0;if((c[f>>2]|0)==0){i=g;j=f;k=l}else{u=-541478725;m=14477;break}}f=d+44|0;g=d+40|0;p=d+32|0;a=b+64|0;v=d+36|0;w=d+64|0;x=b+8|0;y=b+32|0;z=d+68|0;while(1){A=c[h>>2]|0;B=c[A>>2]|0;C=B+208|0;D=c[C>>2]|0;E=c[C+4>>2]|0;C=c[A+4>>2]|0;A=C+208|0;F=c[A>>2]|0;H=c[A+4>>2]|0;if((D|0)==0&(E|0)==(-2147483648|0)|(F|0)==0&(H|0)==(-2147483648|0)){I=c[f>>2]|0}else{if((c[z>>2]|0)==0){J=0}else{J=(AZ(F,H,C+40|0,D,E,B+40|0)|0)>0|0}c[f>>2]=J;c[z>>2]=1;I=J}B=(I|0)==0|0;c[f>>2]=B;E=B|c[i>>2];c[f>>2]=E;B=(c[j>>2]|0)==0&E;c[f>>2]=B;if((B|0)==0){l=k;m=14462;continue L19444}B=il(c[k>>2]|0,0,0,1)|0;E=G;D=c[e>>2]|0;C=c[e+4>>2]|0;H=ac(c[p>>2]|0,c[g>>2]|0)|0;F=NU(H,(H|0)<0|0?-1:0,D,C)|0;C=il(c[k>>2]|0,F,G,0)|0;F=G;D=0;H=c[k>>2]|0;if((F|0)<(D|0)|(F|0)==(D|0)&C>>>0<0>>>0){il(H,B,E,0)|0;}else{C=iH(H)|0;H=mb(c[k>>2]|0,b,C)|0;if((H|0)>=0){C=NU(c[a>>2]|0,c[a+4>>2]|0,-3,-1)|0;c[a>>2]=C;c[a+4>>2]=G;C=ac(c[v>>2]|0,c[g>>2]|0)|0;D=C+(c[w>>2]|0)|0;c[x>>2]=D;c[x+4>>2]=(D|0)<0|0?-1:0;D=(c[w>>2]|0)+1|0;c[w>>2]=D;C=c[v>>2]|0;if((C|0)>0){F=(D|0)%(C|0)|0;c[w>>2]=F;K=F}else{K=D}if((K|0)==0){c[g>>2]=(c[g>>2]|0)+1}c[y>>2]=1}D=c[k>>2]|0;il(D,B,E,0)|0;if((H|0)!=-541478725){u=H;m=14476;break L19444}}c[j>>2]=1}}if((m|0)==14468){j=il(c[l>>2]|0,0,0,1)|0;k=NU(j,G,s,t)|0;c[o>>2]=k;c[o+4>>2]=G;q=t;r=s}else if((m|0)==14475){return u|0}else if((m|0)==14477){return u|0}else if((m|0)==14476){return u|0}m=c[(c[n+8>>2]|0)+432>>2]|0;if((m|0)>1){n=(m|0)>4096?m:4096;s=n-((n|0)%(m|0)|0)|0;L=(s|0)<0|0?-1:0;M=s}else{L=0;M=4096}s=(L|0)>(q|0)|(L|0)==(q|0)&M>>>0>r>>>0;s?q:L;L=mb(c[l>>2]|0,b,s?r:M)|0;if((L|0)<0){u=L;return u|0}c[b+32>>2]=0;u=L;return u|0}function m9(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;g=c[a+12>>2]|0;c[g+48>>2]=0;c[g+52>>2]=0;h=g+24|0;i=c[h+4>>2]|0;j=0;do{if((i|0)>(j|0)|(i|0)==(j|0)&(c[h>>2]|0)>>>0>0>>>0){k=a+28|0;l=c[k>>2]|0;if((b|0)==0){m=AY(d,e,(c[l>>2]|0)+40|0,(c[l+4>>2]|0)+40|0)|0;n=G;o=m;p=e;q=d}else{m=AY(d,e,(c[l+4>>2]|0)+40|0,(c[l>>2]|0)+40|0)|0;n=e;o=d;p=G;q=m}m=c[g+36>>2]|0;if((m|0)<=0){r=p;s=q;t=k;break}l=m;u=(m|0)<0|0?-1:0;m=N0(o,n,l,u)|0;c[g+40>>2]=m;m=N1(o,n,l,u)|0;c[g+64>>2]=m;r=p;s=q;t=k}else{r=e;s=d;t=a+28|0}}while(0);d=c[(c[(c[c[t>>2]>>2]|0)+8>>2]|0)+48>>2]|0;if((d|0)==86016|(d|0)==86017|(d|0)==86019|(d|0)==86020){v=-1;return v|0}v=nw(a,b,s,r,f)|0;return v|0}function na(a){a=a|0;var b=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;b=c[a+4>>2]|0;e=(c[a+8>>2]|0)-7|0;a=b+e|0;do{if((e|0)>0){f=a;g=0;h=b;i=0;do{L19510:do{if(h>>>0<a>>>0){j=0;k=h;while(1){l=k;if(((cT((d[l]|d[l+1|0]<<8)<<16>>16|0)|0)&-10)<<16>>16!=-16){m=j;n=k;break L19510}l=k+3|0;o=(cV(d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0)|0)>>>13&8191;if(o>>>0<7){m=j;n=k;break L19510}l=f-k|0;p=k+((o|0)>(l|0)?l:o)|0;o=j+1|0;if(p>>>0<a>>>0){j=o;k=p}else{m=o;n=p;break}}}else{m=0;n=h}}while(0);i=(i|0)>(m|0)?i:m;g=(h|0)==(b|0)?m:g;h=n+1|0;}while(h>>>0<a>>>0);if((g|0)>2){q=51;return q|0}if((i|0)>500){q=50;return q|0}if((i|0)>2){q=25}else{r=i;break}return q|0}else{r=0}}while(0);q=(r|0)>0|0;return q|0}function nb(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;b=mK(a,0)|0;if((b|0)==0){d=-12;return d|0}e=b+8|0;c[(c[e>>2]|0)+8>>2]=1;c[(c[e>>2]|0)+48>>2]=c[(c[a+4>>2]|0)+28>>2];c[b+236>>2]=1463898624;nk(a);e=a+16|0;do{if((c[(c[e>>2]|0)+88>>2]|0)!=0){if((zJ(c[a+1140>>2]|0,3119128,0,2)|0)!=0){break}f=il(c[e>>2]|0,0,0,1)|0;g=G;nc(a)|0;h=c[e>>2]|0;il(h,f,g,0)|0;}}while(0);mL(b,64,1,28224e3);d=0;return d|0}function nc(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;d=i;i=i+2136|0;e=d|0;f=d+2048|0;g=b+16|0;h=c[g>>2]|0;j=ip(h)|0;k=G;l=0;if((k|0)<(l|0)|(k|0)==(l|0)&j>>>0<32>>>0){m=0;n=0;i=d;return(G=m,n)|0}l=NU(j,k,-32,-1)|0;o=G;il(h,l,o,0)|0;p=d+2128|0;iE(h,p,8)|0;if((ba(p|0,1685336,8)|0)!=0){m=0;n=0;i=d;return(G=m,n)|0}if((iI(h)|0)>>>0>2e3){AN(b,16,1729560,(q=i,i=i+8|0,c[q>>2]=2e3,q)|0);i=q;m=0;n=0;i=d;return(G=m,n)|0}p=iI(h)|0;if((p-32|0)>>>0>16777216){AN(b,16,1698064,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;m=0;n=0;i=d;return(G=m,n)|0}r=p;s=0;if((s|0)>(o|0)|(s|0)==(o|0)&r>>>0>l>>>0){AN(b,16,1675968,(q=i,i=i+8|0,c[q>>2]=p,q)|0);i=q;m=0;n=0;i=d;return(G=m,n)|0}p=NV(j,k,r,s)|0;s=G;r=NU(p,s,-32,-1)|0;k=G;j=iI(h)|0;if(j>>>0>65536){AN(b,16,1660248,(q=i,i=i+8|0,c[q>>2]=j,q)|0);i=q;m=0;n=0;i=d;return(G=m,n)|0}if(((iI(h)|0)&536870912|0)!=0){AN(b,16,1643048,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;m=0;n=0;i=d;return(G=m,n)|0}il(h,p,s,0)|0;s=e|0;p=d+1024|0;h=f;l=b+1140|0;o=b;t=0;while(1){if(t>>>0>=j>>>0){m=k;n=r;u=14559;break}v=c[g>>2]|0;w=iI(v)|0;x=iI(v)|0;y=0;while(1){z=iD(v)|0;if((z-32|0)>>>0>94){A=y;break}a[e+y|0]=z&255;B=y+1|0;if(B>>>0<1023){y=B}else{A=B;break}}a[e+A|0]=0;do{if((z|0)==0){if((w|0)==-1){C=-1;break}if((x&2|0)==0){y=A1(w+1|0)|0;if((y|0)==0){C=-12;break}B=iE(v,y,w)|0;if((B|0)<0){A4(y);C=B;break}else{a[y+B|0]=0;zK(l,s,y,8)|0;C=0;break}}y=mK(b,0)|0;if((y|0)==0){C=-12;break}B=iN(v,w,p,1024)|0;D=w-B|0;if((w|0)==(B|0)){AN(o,24,1614408,(q=i,i=i+8|0,c[q>>2]=s,q)|0);i=q;C=0;break}B=y+88|0;zK(B,s,p,0)|0;B=cL(p|0)|0;if((B|0)!=0){E=mb(c[g>>2]|0,f,D)|0;if((E|0)<0){AN(o,16,1743544,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;C=E;break}else{E=y+72|0;c[E>>2]=c[E>>2]|1024;E=y+8|0;c[(c[E>>2]|0)+8>>2]=0;c[(c[E>>2]|0)+48>>2]=B;B=y+104|0;NN(B|0,h|0,80)|0;c[y+136>>2]=c[y>>2];B=y+140|0;c[B>>2]=c[B>>2]|1;C=0;break}}B=A1(D+16|0)|0;E=y+8|0;c[(c[E>>2]|0)+96>>2]=B;B=c[(c[E>>2]|0)+96>>2]|0;if((B|0)==0){C=-12;break}y=(iE(v,B,D)|0)==(D|0);B=c[E>>2]|0;if(y){c[B+100>>2]=D;c[(c[E>>2]|0)+8>>2]=4;C=0;break}else{A6(B+96|0);C=-5;break}}else{AN(o,24,1618824,(q=i,i=i+8|0,c[q>>2]=s,q)|0);i=q;C=-1}}while(0);if((C|0)<0){m=k;n=r;u=14554;break}else{t=t+1|0}}if((u|0)==14559){i=d;return(G=m,n)|0}else if((u|0)==14554){i=d;return(G=m,n)|0}return 0}function nd(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0;g=i;i=i+728|0;h=g|0;j=g+392|0;k=g+656|0;l=g+720|0;m=f>>3;n=k;NM(n|0,0,64);if((f|0)<16){if((f|0)>0){o=0}else{i=g;return}do{p=e+o|0;a[p]=a[p]^a[b+o|0];o=o+1|0;}while((o|0)<(f|0));i=g;return}B0(j,b,96,1)|0;B1(j,n,0,64,0,1);o=c[k>>2]|1;p=c[n+4>>2]|1;q=c[k+8>>2]|1;r=c[n+12>>2]|1;s=c[k+16>>2]|1;t=c[n+20>>2]|1;u=c[k+24>>2]|1;v=c[n+28>>2]|1;w=c[k+32>>2]|1;x=c[n+36>>2]|1;z=c[k+40>>2]|1;A=c[n+44>>2]|1;n=e+((m<<3)-8)|0;B=n|0;C=n+4|0;n=k+56|0;D=c[n+4>>2]^(d[C]|d[C+1|0]<<8|d[C+2|0]<<16|d[C+3|0]<<24);c[l>>2]=c[n>>2]^(d[B]|d[B+1|0]<<8|d[B+2|0]<<16|d[B+3|0]<<24);c[l+4>>2]=D;zE(h,b+12|0,64,1)|0;b=l;zF(h,b,b,1,0,1);h=k+48|0;k=c[l+4>>2]^c[h+4>>2];c[l>>2]=c[l>>2]^c[h>>2];c[l+4>>2]=k;B0(j,b,64,1)|0;B1(j,e,e,f,0,1);f=m-1|0;if((f|0)>0){j=(m<<3)-8|0;m=e;b=0;k=0;h=0;while(1){D=m;B=D|0;n=D+4|0;D=d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0;n=NU(d[B]|d[B+1|0]<<8|d[B+2|0]<<16|d[B+3|0]<<24|0,D,k,b)|0;B=ac(n,o)|0;n=ac(B>>>16|B<<16,p)|0;B=ac(n>>>16|n<<16,q)|0;n=ac(B>>>16|B<<16,r)|0;B=(ac(n>>>16|n<<16,s)|0)+t|0;n=NU(B,0,b,0)|0;C=ac(B+D|0,u)|0;D=ac(C>>>16|C<<16,v)|0;C=ac(D>>>16|D<<16,w)|0;D=ac(C>>>16|C<<16,x)|0;C=(ac(D>>>16|D<<16,z)|0)+A|0;E=C|0;F=C+n|0;n=h+1|0;if((n|0)<(f|0)){m=m+8|0;b=F;k=E;h=n}else{break}}H=e+j|0;I=F;J=E}else{H=e;I=0;J=0}e=ac(ac(o,o)|0,o)|0;E=ac(2-(ac(e,o)|0)|0,e)|0;e=ac(2-(ac(E,o)|0)|0,E)|0;E=ac(2-(ac(e,o)|0)|0,e)|0;e=ac(ac(p,p)|0,p)|0;o=ac(2-(ac(e,p)|0)|0,e)|0;e=ac(2-(ac(o,p)|0)|0,o)|0;o=ac(2-(ac(e,p)|0)|0,e)|0;e=ac(ac(q,q)|0,q)|0;p=ac(2-(ac(e,q)|0)|0,e)|0;e=ac(2-(ac(p,q)|0)|0,p)|0;p=ac(2-(ac(e,q)|0)|0,e)|0;e=ac(ac(r,r)|0,r)|0;q=ac(2-(ac(e,r)|0)|0,e)|0;e=ac(2-(ac(q,r)|0)|0,q)|0;q=ac(2-(ac(e,r)|0)|0,e)|0;e=ac(ac(s,s)|0,s)|0;r=ac(2-(ac(e,s)|0)|0,e)|0;e=ac(2-(ac(r,s)|0)|0,r)|0;r=ac(2-(ac(e,s)|0)|0,e)|0;e=ac(ac(u,u)|0,u)|0;s=ac(2-(ac(e,u)|0)|0,e)|0;e=ac(2-(ac(s,u)|0)|0,s)|0;s=ac(2-(ac(e,u)|0)|0,e)|0;e=ac(ac(v,v)|0,v)|0;u=ac(2-(ac(e,v)|0)|0,e)|0;e=ac(2-(ac(u,v)|0)|0,u)|0;u=ac(2-(ac(e,v)|0)|0,e)|0;e=ac(ac(w,w)|0,w)|0;v=ac(2-(ac(e,w)|0)|0,e)|0;e=ac(2-(ac(v,w)|0)|0,v)|0;v=ac(2-(ac(e,w)|0)|0,e)|0;e=ac(ac(x,x)|0,x)|0;w=ac(2-(ac(e,x)|0)|0,e)|0;e=ac(2-(ac(w,x)|0)|0,w)|0;w=ac(2-(ac(e,x)|0)|0,e)|0;e=ac(ac(z,z)|0,z)|0;x=ac(2-(ac(e,z)|0)|0,e)|0;e=ac(2-(ac(x,z)|0)|0,x)|0;x=ac(2-(ac(e,z)|0)|0,e)|0;e=c[l+4>>2]|0;z=(c[l>>2]|0)-e|0;F=ac(x,e-A|0)|0;A=ac(w,F>>>16|F<<16)|0;F=ac(v,A>>>16|A<<16)|0;A=ac(u,F>>>16|F<<16)|0;F=ac(s,A>>>16|A<<16)|0;A=NV(z,0,I,0)|0;z=A;A=ac(r,z-t|0)|0;t=ac(q,A>>>16|A<<16)|0;A=ac(p,t>>>16|t<<16)|0;t=ac(o,A>>>16|A<<16)|0;A=NV(ac(E,t>>>16|t<<16)|0,0,J,I)|0;I=A|0|0;A=G&0|F-z;c[l>>2]=I;c[l+4>>2]=A;l=H;H=l|0;y=I;a[H]=y&255;y=y>>8;a[H+1|0]=y&255;y=y>>8;a[H+2|0]=y&255;y=y>>8;a[H+3|0]=y&255;H=l+4|0;y=A;a[H]=y&255;y=y>>8;a[H+1|0]=y&255;y=y>>8;a[H+2|0]=y&255;y=y>>8;a[H+3|0]=y&255;i=g;return}function ne(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;e=b+(4-(b&3))|0;f=d-3|0;L19615:do{if(f>>>0>b>>>0){g=b;while(1){h=g+1|0;do{if((a[g]|0)==0){if((a[h]|0)!=0){break}if((a[g+2|0]|0)==1){i=g;break L19615}}}while(0);if(h>>>0<e>>>0&h>>>0<f>>>0){g=h}else{j=h;k=14579;break}}}else{j=b;k=14579}}while(0);L19622:do{if((k|0)==14579){e=d-6|0;if(j>>>0<e>>>0){g=j;while(1){l=c[g>>2]|0;m=l&255;do{if(((l&-2139062144^-2139062144)&l-16843009|0)!=0){n=g+1|0;do{if((a[n]|0)==0){o=a[g+2|0]|0;if(m<<24>>24==0&o<<24>>24==1){i=g;break L19622}if(o<<24>>24!=0){break}if((a[g+3|0]|0)==1){i=n;break L19622}}}while(0);n=g+3|0;if((a[n]|0)!=0){break}o=g+2|0;p=a[g+4|0]|0;if((a[o]|0)==0&p<<24>>24==1){i=o;break L19622}if(p<<24>>24!=0){break}if((a[g+5|0]|0)==1){i=n;break L19622}}}while(0);m=g+4|0;if(m>>>0<e>>>0){g=m}else{q=m;break}}}else{q=j}if(q>>>0<f>>>0){r=q}else{s=d;return s|0}while(1){g=r+1|0;do{if((a[r]|0)==0){if((a[g]|0)!=0){break}if((a[r+2|0]|0)==1){i=r;break L19622}}}while(0);if(g>>>0<f>>>0){r=g}else{s=d;break}}return s|0}}while(0);if(i>>>0>b>>>0&i>>>0<d>>>0){d=i-1|0;return((a[d]|0)==0?d:i)|0}else{s=i;return s|0}return 0}function nf(b,c,d){b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;e=c+d|0;d=0;f=ne(c,e)|0;while(1){c=f;while(1){if(c>>>0>=e>>>0){g=c;break}h=c+1|0;if((a[c]|0)==0){c=h}else{g=h;break}}if((g|0)==(e|0)){break}c=ne(g,e)|0;h=g;i=c-h|0;is(b,i);ij(b,g,i);d=d-h+(c+4)|0;f=c}return d|0}function ng(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;f=i;i=i+8|0;g=f|0;h=iZ(g)|0;if((h|0)<0){j=h;i=f;return j|0}h=c[g>>2]|0;k=b+(c[e>>2]|0)|0;l=ne(b,k)|0;while(1){if(l>>>0<k>>>0){b=l+1|0;if((a[l]|0)==0){l=b;continue}else{m=b}}else{m=l}if((m|0)==(k|0)){break}b=ne(m,k)|0;n=b-m|0;is(h,n);ij(h,m,n);l=b}A6(d);c[e>>2]=i$(c[g>>2]|0,d)|0;j=0;i=f;return j|0}function nh(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;g=i;i=i+16|0;h=g|0;j=g+8|0;c[h>>2]=f;if((f|0)<=6){k=0;i=g;return k|0}l=e;m=d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0;do{if((m|0)!=16777216){if(((d[e+1|0]|0)<<8|m<<16&16711680|(d[e+2|0]|0)|0)==1){break}ij(b,e,f);k=0;i=g;return k|0}}while(0);c[j>>2]=0;f=ng(e,j,h)|0;if((f|0)<0){k=f;i=g;return k|0}f=c[j>>2]|0;e=c[h>>2]|0;h=f+e|0;if((e|0)>4){n=0;o=0;p=0;q=0;r=f;s=e}else{k=-1094995529;i=g;return k|0}while(1){e=r;m=cV(d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24|0)|0;e=s-4|0;l=m>>>0>e>>>0?e:m;m=r+4|0;c[j>>2]=m;e=a[m]&31;if(e<<24>>24==7){t=l;u=p;v=o;w=m}else{x=e<<24>>24==8;t=q;u=x?l:p;v=x?m:o;w=n}m=r+(l+4)|0;c[j>>2]=m;l=h-m|0;if((l|0)>4){n=w;o=v;p=u;q=t;r=m;s=l}else{break}}if((w|0)==0|(v|0)==0){k=-1094995529;i=g;return k|0}if((t-4|0)>>>0>65531|u>>>0>65535){k=-1094995529;i=g;return k|0}ih(b,1);ih(b,d[w+1|0]|0);ih(b,d[w+2|0]|0);ih(b,d[w+3|0]|0);ih(b,255);ih(b,225);iy(b,t);ij(b,w,t);ih(b,1);iy(b,u);ij(b,v,u);A4(f);k=0;i=g;return k|0}function ni(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;c[e>>2]=0;g=c[f>>2]|0;if((g|0)<=3){h=-1094995529;return h|0}i=b;j=d[i]|d[i+1|0]<<8|d[i+2|0]<<16|d[i+3|0]<<24|0;if((j|0)==16777216){h=0;return h|0}if(((d[b+1|0]|0)<<8|j<<16&16711680|(d[b+2|0]|0)|0)==1){h=0;return h|0}if(!((g|0)>10&(j&255)<<24>>24==1)){h=-1094995529;return h|0}j=b+6|0;i=(cT((d[j]|d[j+1|0]<<8)<<16>>16|0)|0)&65535;j=i+11|0;if((j|0)>(g|0)){h=-1094995529;return h|0}k=b+(i+9)|0;l=(cT((d[k]|d[k+1|0]<<8)<<16>>16|0)|0)&65535;if((l+j|0)>(g|0)){h=-1094995529;return h|0}g=i+8|0;k=g+l|0;m=A7(k)|0;if((m|0)==0){h=-12;return h|0}n=m;y=16777216;a[n]=y&255;y=y>>8;a[n+1|0]=y&255;y=y>>8;a[n+2|0]=y&255;y=y>>8;a[n+3|0]=y&255;n=m+4|0;o=b+8|0;NN(n|0,o|0,i)|0;o=m+(i+4)|0;y=16777216;a[o]=y&255;y=y>>8;a[o+1|0]=y&255;y=y>>8;a[o+2|0]=y&255;y=y>>8;a[o+3|0]=y&255;o=m+g|0;g=b+j|0;NN(o|0,g|0,l)|0;c[e>>2]=m;c[f>>2]=k;h=0;return h|0}function nj(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;f=i;i=i+88|0;g=f|0;h=f+8|0;j=f+72|0;k=f+80|0;c[g>>2]=0;c[j>>2]=0;l=ig(d,e,0,0,0,0,0)|0;c[k>>2]=l;if((l|0)==0){m=-12;i=f;return m|0}e=iM(l)|0;do{if(e>>>0>20|(e|0)<0){AN(b,16,1683224,(n=i,i=i+8|0,c[n>>2]=e,n)|0);i=n;if((c[b+1156>>2]&8|0)!=0){o=-1094995529;break}p=0;q=c[k>>2]|0;r=14659}else{p=e;q=l;r=14659}}while(0);L19730:do{if((r|0)==14659){l=iM(q)|0;do{if((l|0)>=1){e=h|0;if((iE(c[k>>2]|0,e,l>>>0>63?63:l)|0)!=(l|0)){break}a[h+l|0]=0;d=249312;while(1){s=c[d+32>>2]|0;if((s|0)==0){r=14665;break}if((ba(d|0,e|0,64)|0)==0){break}else{d=d+36|0}}if((r|0)==14665){AN(b,16,1697864,(n=i,i=i+8|0,c[n>>2]=e,n)|0);i=n;o=(c[b+1156>>2]&8|0)==0?0:-1094995529;break L19730}d=iM(c[k>>2]|0)|0;do{if((d|0)>0){t=A1(d+1|0)|0;c[j>>2]=t;if((t|0)==0){o=-12;break L19730}if((iE(c[k>>2]|0,t,d)|0)==(d|0)){a[(c[j>>2]|0)+d|0]=0;break}else{AN(b,16,1675784,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;o=(c[b+1156>>2]&8|0)==0?0:-5;break L19730}}}while(0);d=iM(c[k>>2]|0)|0;e=iM(c[k>>2]|0)|0;io(c[k>>2]|0,8,0)|0;t=iM(c[k>>2]|0)|0;if((t|0)<1){AN(b,16,1660152,(n=i,i=i+8|0,c[n>>2]=t,n)|0);i=n;o=(c[b+1156>>2]&8|0)==0?0:-1094995529;break L19730}u=zg(t)|0;c[g>>2]=u;if((u|0)==0){o=-12;break L19730}if((iE(c[k>>2]|0,c[u+4>>2]|0,t)|0)!=(t|0)){AN(b,16,1642888,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;o=(c[b+1156>>2]&8|0)==0?0:-5;break L19730}u=mK(b,0)|0;if((u|0)==0){o=-12;break L19730}v=u+104|0;nB(v);c[v>>2]=c[g>>2];c[u+128>>2]=c[(c[g>>2]|0)+4>>2];c[u+132>>2]=t;c[u+136>>2]=c[u>>2];t=u+140|0;c[t>>2]=c[t>>2]|1;t=u+72|0;c[t>>2]=c[t>>2]|1024;t=u+8|0;c[(c[t>>2]|0)+8>>2]=0;c[(c[t>>2]|0)+48>>2]=s;c[(c[t>>2]|0)+120>>2]=d;c[(c[t>>2]|0)+124>>2]=e;e=u+88|0;zK(e,1631232,c[249224+(p<<2)>>2]|0,0)|0;u=c[j>>2]|0;if((u|0)!=0){zK(e,1624896,u,8)|0}A6(k);m=0;i=f;return m|0}}while(0);AN(b,16,1729208,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;o=(c[b+1156>>2]&8|0)==0?0:-1094995529}}while(0);zj(g);A6(j);A6(k);m=o;i=f;return m|0}function nk(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;d=i;i=i+648|0;e=d|0;f=d+520|0;g=b+16|0;h=il(c[g>>2]|0,0,0,1)|0;j=G;k=c[g>>2]|0;if((c[k+88>>2]|0)==0){i=d;return}l=ip(k)|0;k=G;m=0;if(!((k|0)>(m|0)|(k|0)==(m|0)&l>>>0>128>>>0)){i=d;return}m=c[g>>2]|0;n=NU(l,k,-128,-1)|0;il(m,n,G,0)|0;n=f|0;do{if((iE(c[g>>2]|0,n,128)|0)==128){m=d+512|0;if((a[n]|0)!=84){break}if((a[f+1|0]|0)!=65){break}if((a[f+2|0]|0)!=71){break}k=e|0;l=e;o=0;p=k;while(1){q=a[f+(o+3)|0]|0;if(q<<24>>24==0){r=p;break}if((p-l|0)>>>0>510){r=p;break}s=p+1|0;a[p]=q;q=o+1|0;if((q|0)<30){o=q;p=s}else{r=s;break}}a[r]=0;if((a[k]|0)==0){t=0;u=k}else{p=b+1140|0;zK(p,1721160,k,0)|0;t=0;u=k}while(1){p=a[f+(t+33)|0]|0;if(p<<24>>24==0){v=u;break}if((u-l|0)>>>0>510){v=u;break}o=u+1|0;a[u]=p;p=t+1|0;if((p|0)<30){t=p;u=o}else{v=o;break}}a[v]=0;if((a[k]|0)==0){w=0;x=k}else{o=b+1140|0;zK(o,1720992,k,0)|0;w=0;x=k}while(1){o=a[f+(w+63)|0]|0;if(o<<24>>24==0){y=x;break}if((x-l|0)>>>0>510){y=x;break}p=x+1|0;a[x]=o;o=w+1|0;if((o|0)<30){w=o;x=p}else{y=p;break}}a[y]=0;if((a[k]|0)!=0){p=b+1140|0;zK(p,1720696,k,0)|0}p=a[f+93|0]|0;do{if(p<<24>>24==0){z=k}else{o=e+1|0;a[k]=p;s=a[f+94|0]|0;if(s<<24>>24==0){z=o;break}q=e+2|0;a[o]=s;s=a[f+95|0]|0;if(s<<24>>24==0){z=q;break}o=e+3|0;a[q]=s;s=a[f+96|0]|0;if(s<<24>>24==0){z=o;break}a[o]=s;z=e+4|0}}while(0);a[z]=0;if((a[k]|0)==0){A=0;B=k}else{p=b+1140|0;zK(p,1720464,k,0)|0;A=0;B=k}while(1){p=a[f+(A+97)|0]|0;if(p<<24>>24==0){C=B;break}if((B-l|0)>>>0>510){C=B;break}s=B+1|0;a[B]=p;p=A+1|0;if((p|0)<30){A=p;B=s}else{C=s;break}}a[C]=0;if((a[k]|0)!=0){l=b+1140|0;zK(l,1719760,k,0)|0}do{if((a[f+125|0]|0)==0){l=a[f+126|0]|0;if(l<<24>>24==0){break}s=l&255;bc(m|0,5,1719264,(l=i,i=i+8|0,c[l>>2]=s,l)|0)|0;i=l;l=b+1140|0;zK(l,1718696,m,0)|0}}while(0);m=a[f+127|0]|0;if((m&255)>=148){break}zK(b+1140|0,1717944,c[249904+((m&255)<<2)>>2]|0,0)|0}}while(0);il(c[g>>2]|0,h,j,0)|0;i=d;return}function nl(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;c[b>>2]=f;is(e,a[g+2|0]<<8|f|a[g+1|0]<<16|d[g]<<24);ih(e,0);ih(e,0);g=il(e,0,0,1)|0;f=b+8|0;c[f>>2]=g;c[f+4>>2]=G;is(e,0);return}function nm(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;d=i;i=i+144|0;e=d|0;f=d+8|0;g=(c[b>>2]|0)==3?1:3;h=a+16|0;j=nn(c[h>>2]|0,a+1140|0,b,g)|0;if((j|0)<0){k=j;i=d;return k|0}j=a+1132|0;l=e;m=d+16|0;n=a+1136|0;a=b+16|0;o=0;while(1){if(o>>>0>=(c[j>>2]|0)>>>0){k=0;p=14743;break}q=c[(c[n>>2]|0)+(o<<2)>>2]|0;c[e>>2]=0;c[f>>2]=0;r=iZ(f)|0;do{if((r|0)<0){s=r}else{t=q+16|0;u=q+4|0;v=AY(c[t>>2]|0,c[t+4>>2]|0,u,16)|0;t=v;v=q+24|0;w=AY(c[v>>2]|0,c[v+4>>2]|0,u,16)|0;u=w;bc(m|0,122,1660016,(w=i,i=i+8|0,c[w>>2]=o,w)|0)|0;i=w;w=it(c[f>>2]|0,m)|0;c[a>>2]=(c[a>>2]|0)+w;is(c[f>>2]|0,t);is(c[f>>2]|0,u);is(c[f>>2]|0,-1);is(c[f>>2]|0,-1);u=nn(c[f>>2]|0,q+32|0,b,g)|0;if((u|0)<0){s=u;break}t=i$(c[f>>2]|0,e)|0;c[a>>2]=(c[a>>2]|0)+26;is(c[h>>2]|0,1128808784);is(c[h>>2]|0,t);iy(c[h>>2]|0,0);ij(c[h>>2]|0,c[e>>2]|0,t);s=u}}while(0);q=c[f>>2]|0;do{if((q|0)!=0){if((c[e>>2]|0)!=0){break}i$(q,e)|0}}while(0);A6(l);if((s|0)<0){k=s;p=14742;break}else{o=o+1|0}}if((p|0)==14742){i=d;return k|0}else if((p|0)==14743){i=d;return k|0}return 0}function nn(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;h=i;i=i+24|0;j=h|0;jx(e,249784,0);k=f|0;l=c[k>>2]|0;if((l|0)==3){m=h+8|0;n=h+16|0;c[j>>2]=0;NM(m|0,0,5);NM(n|0,0,5);o=zJ(c[e>>2]|0,3119088,0,2)|0;if((o|0)!=0){p=o;do{o=c[p>>2]|0;q=(y4(o,1642760)|0)==0;r=c[p+4>>2]|0;do{if(q){s=0;while(1){t=r+s|0;u=a[t]|0;v=s+1|0;if((u-48&255)<10){s=v}else{break}}if(!((u<<24>>24|0)==0|(u<<24>>24|0)==45)){zK(j,o,r,0)|0;break}y_(m,r,5)|0;zK(j,1631160,m,0)|0;if((a[t]|0)!=45){break}w=r+v|0;if(((a[w]|0)-48&255)>=2){break}if(((a[r+(s+2)|0]|0)-48&255)>=10){break}if((a[r+(s+3)|0]|0)!=45){break}x=r+(s+4)|0;if(((a[x]|0)-48&255)>=4){break}if(((a[r+(s+5)|0]|0)-48&255)>=10){break}y=a[r+(s+6)|0]|0;if(!((y<<24>>24|0)==0|(y<<24>>24|0)==32)){break}bc(n|0,5,1624856,(y=i,i=i+16|0,c[y>>2]=x,c[y+8>>2]=w,y)|0)|0;i=y;zK(j,1618736,n,0)|0}else{zK(j,o,r,0)|0}}while(0);p=zJ(c[e>>2]|0,3119088,p,2)|0;}while((p|0)!=0)}zL(e);c[e>>2]=c[j>>2]}else if((l|0)==4){jx(e,249696,0)}l=zJ(c[e>>2]|0,3119088,0,2)|0;if((l|0)==0){z=0;i=h;return z|0}j=f+16|0;p=l;while(1){l=p|0;n=c[l>>2]|0;L19869:do{if((a[n]|0)==84){if((NL(n|0)|0)==4){v=n;t=d[v]|d[v+1|0]<<8|d[v+2|0]<<16|d[v+3|0]<<24|0;v=cV(t|0)|0;m=t&255;t=0;while(1){u=249088+(t<<2)|0;if((a[u]|0)==0){A=n;B=m;break}r=u;if((v|0)==(cV(d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0)|0)){C=14770;break}else{t=t+1|0}}do{if((C|0)==14770){C=0;t=nq(f,b,c[p+4>>2]|0,0,v,g)|0;if((t|0)>0){c[j>>2]=(c[j>>2]|0)+t;break L19869}else{t=c[l>>2]|0;A=t;B=a[t]|0;break}}}while(0);if(B<<24>>24==84){D=A;E=(c[k>>2]|0)==3?249752:249640}else{F=A;C=14782;break}}else{D=n;E=(c[k>>2]|0)==3?249752:249640}if((NL(D|0)|0)!=4){F=D;C=14782;break}v=D;t=cV(d[v]|d[v+1|0]<<8|d[v+2|0]<<16|d[v+3|0]<<24|0)|0;v=0;while(1){m=E+(v<<2)|0;if((a[m]|0)==0){F=D;C=14782;break L19869}s=m;if((t|0)==(cV(d[s]|d[s+1|0]<<8|d[s+2|0]<<16|d[s+3|0]<<24|0)|0)){break}else{v=v+1|0}}v=nq(f,b,c[p+4>>2]|0,0,t,g)|0;if((v|0)>0){c[j>>2]=(c[j>>2]|0)+v;break}else{F=c[l>>2]|0;C=14782;break}}else{F=n;C=14782}}while(0);if((C|0)==14782){C=0;n=nq(f,b,F,c[p+4>>2]|0,1415075928,g)|0;if((n|0)<0){z=n;C=14786;break}c[j>>2]=(c[j>>2]|0)+n}n=zJ(c[e>>2]|0,3119088,p,2)|0;if((n|0)==0){z=0;C=14785;break}else{p=n}}if((C|0)==14785){i=h;return z|0}else if((C|0)==14786){i=h;return z|0}return 0}function no(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;e=i;i=i+16|0;f=e|0;g=e+8|0;h=c[(c[a+28>>2]|0)+(c[d+32>>2]<<2)>>2]|0;j=b|0;k=(c[j>>2]|0)==3;l=k?1:3;m=h+8|0;n=249312;while(1){o=c[n+32>>2]|0;if((o|0)==0){break}if((o|0)==(c[(c[m>>2]|0)+48>>2]|0)){p=14791;break}else{n=n+36|0}}do{if((p|0)==14791){m=n|0;if((n|0)==0){break}o=h+88|0;q=zJ(c[o>>2]|0,1697664,0,0)|0;r=q+4|0;L19906:do{if((q|0)==0){s=0}else{t=0;while(1){if(t>>>0>=21){s=0;break L19906}u=c[249224+(t<<2)>>2]|0;v=c[r>>2]|0;if((ba(u|0,v|0,NL(v|0)|0)|0)==0){s=t;break}else{t=t+1|0}}}}while(0);r=zJ(c[o>>2]|0,1675672,0,0)|0;if((r|0)==0){w=3119088}else{w=c[r+4>>2]|0}if((iZ(f)|0)<0){x=-12;i=e;return x|0}ih(c[f>>2]|0,l);r=c[f>>2]|0;it(r,m)|0;ih(c[f>>2]|0,s);r=c[f>>2]|0;if(k){iv(r,65279);y=102}else{y=142}dg[y&255](r,w)|0;ij(c[f>>2]|0,c[d+24>>2]|0,c[d+28>>2]|0);r=i$(c[f>>2]|0,g)|0;q=a+16|0;is(c[q>>2]|0,1095780675);t=c[q>>2]|0;if((c[j>>2]|0)==3){is(t,r)}else{ih(t,r>>>21&127);ih(t,r>>>14&127);ih(t,r>>>7&127);ih(t,r&127)}iy(c[q>>2]|0,0);ij(c[q>>2]|0,c[g>>2]|0,r);A6(g);q=b+16|0;c[q>>2]=r+10+(c[q>>2]|0);x=0;i=e;return x|0}}while(0);AN(a,16,1728696,(a=i,i=i+8|0,c[a>>2]=c[h>>2],a)|0);i=a;x=-22;i=e;return x|0}function np(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;ii(b,0,10);d=a+16|0;c[d>>2]=(c[d>>2]|0)+10;e=il(b,0,0,1)|0;f=G;g=a+8|0;il(b,c[g>>2]|0,c[g+4>>2]|0,0)|0;g=c[d>>2]|0;ih(b,g>>>21&127);ih(b,g>>>14&127);ih(b,g>>>7&127);ih(b,g&127);il(b,e,f,0)|0;return}function nq(b,d,e,f,g,h){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;j=i;i=i+16|0;k=j|0;l=j+8|0;if((iZ(l)|0)<0){m=-12;i=j;return m|0}do{if((h|0)==1){n=e;while(1){o=a[n]|0;p=o<<24>>24==0;if(o<<24>>24>-1&(p^1)){n=n+1|0}else{break}}if(!p){q=1;break}if((f|0)!=0){n=f;while(1){o=a[n]|0;r=o<<24>>24==0;if(o<<24>>24>-1&(r^1)){n=n+1|0}else{break}}if(!r){q=1;break}}q=0}else{q=h}}while(0);ih(c[l>>2]|0,q);h=c[l>>2]|0;r=(q|0)==1;if(r){iv(h,65279);s=102}else{s=142}dg[s&255](h,e)|0;if((f|0)!=0){e=c[l>>2]|0;if(r){iv(e,65279);t=102}else{t=142}dg[t&255](e,f)|0}f=i$(c[l>>2]|0,k)|0;is(d,g);if((c[b>>2]|0)==3){is(d,f)}else{ih(d,f>>>21&127);ih(d,f>>>14&127);ih(d,f>>>7&127);ih(d,f&127)}iy(d,0);ij(d,c[k>>2]|0,f);A6(k);m=f+10|0;i=j;return m|0}function nr(b,c){b=b|0;c=c|0;var d=0,e=0,f=0;d=a[b]|0;e=(c|0)==0;L19958:do{if(d<<24>>24!=0&e){c=0;while(1){if(c>>>0>=139){break L19958}if((a8(b|0,126448+(c<<2)|0)|0)==0){f=c;break}else{c=c+1|0}}return f|0}}while(0);if(e){f=-1;return f|0}e=d<<24>>24==0?1679640:b;b=(a[e]|0)-96&255;if((b&255)>31){f=-1;return f|0}d=(a[e+1|0]|0)-96&255;if((d&255)>31){f=-1;return f|0}c=(a[e+2|0]|0)-96&255;if((c&255)>31){f=-1;return f|0}f=c&255|(d&255|(b&255)<<5)<<5;return f|0}function ns(b,c){b=b|0;c=c|0;var e=0,f=0;e=c;y=0;a[e]=y&255;y=y>>8;a[e+1|0]=y&255;y=y>>8;a[e+2|0]=y&255;y=y>>8;a[e+3|0]=y&255;if(b>>>0>1023&(b|0)!=32767){a[c+2|0]=(b&31|96)&255;a[c+1|0]=(b>>>5&31|96)&255;a[c]=(b>>>10&31|96)&255;f=1;return f|0}if(b>>>0>138){f=0;return f|0}c=126448+(b<<2)|0;if((a[c|0]|0)==0){f=0;return f|0}b=c;y=d[b]|d[b+1|0]<<8|d[b+2|0]<<16|d[b+3|0]<<24|0;a[e]=y&255;y=y>>8;a[e+1|0]=y&255;y=y>>8;a[e+2|0]=y&255;y=y>>8;a[e+3|0]=y&255;f=1;return f|0}function nt(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;c[d>>2]=iD(b)|0;d=0;a=4;while(1){if((a|0)==0){e=d;f=14860;break}g=iD(b)|0;h=g&127|d<<7;if((g&128|0)==0){e=h;f=14861;break}else{d=h;a=a-1|0}}if((f|0)==14860){return e|0}else if((f|0)==14861){return e|0}return 0}function nu(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=iK(a)|0;if((b|0)!=0){c[b>>2]=d}d=iD(a)|0;if((d&128|0)!=0){iK(a)|0}if((d&64|0)!=0){b=iD(a)|0;e=b;f=(b|0)<0|0?-1:0;io(a,e,f)|0;}if((d&32|0)==0){return}iK(a)|0;return}function nv(a,b,d){a=a|0;b=b|0;d=d|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;f=i;i=i+48|0;g=f|0;h=iD(d)|0;iD(d)|0;iL(d)|0;iM(d)|0;iM(d)|0;j=b+8|0;if((o7(c[j>>2]|0)|0)!=0){AN(a,48,1728528,(a=i,i=i+1|0,i=i+7&-8,c[a>>2]=0,a)|0);i=a;k=-1;i=f;return k|0}a=mA(246904,h)|0;c[(c[j>>2]|0)+48>>2]=a;a=iD(d)|0;h=0;b=4;while(1){if((b|0)==0){l=h;break}m=iD(d)|0;n=m&127|h<<7;if((m&128|0)==0){l=n;break}else{h=n;b=b-1|0}}if((a|0)!=5){k=0;i=f;return k|0}if((l|0)==0|l>>>0>1073741824){k=-1;i=f;return k|0}A4(c[(c[j>>2]|0)+96>>2]|0);a=A7(l+16|0)|0;c[(c[j>>2]|0)+96>>2]=a;a=c[(c[j>>2]|0)+96>>2]|0;if((a|0)==0){k=-12;i=f;return k|0}iE(d,a,l)|0;c[(c[j>>2]|0)+100>>2]=l;l=c[j>>2]|0;if((c[l+48>>2]|0)!=86018){k=0;i=f;return k|0}oc(g,c[l+96>>2]|0,c[l+100>>2]<<3,1)|0;c[(c[j>>2]|0)+416>>2]=c[g+36>>2];l=g|0;do{if((c[l>>2]|0)==29){a=c[g+4>>2]|0;if((a|0)>=3){o=14885;break}c[(c[j>>2]|0)+412>>2]=e[337568+(a<<1)>>1]|0}else{o=14885}}while(0);do{if((o|0)==14885){a=c[g+28>>2]|0;if((a|0)==0){c[(c[j>>2]|0)+412>>2]=c[g+8>>2];break}else{c[(c[j>>2]|0)+412>>2]=a;break}}}while(0);g=mA(126024,c[l>>2]|0)|0;c[(c[j>>2]|0)+48>>2]=g;if((g|0)!=0){k=0;i=f;return k|0}c[(c[j>>2]|0)+48>>2]=86018;k=0;i=f;return k|0}function nw(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;b=c[c[a+28>>2]>>2]|0;g=b+8|0;h=c[g>>2]|0;i=c[h+432>>2]|0;if((i|0)==0){j=pc(c[h+48>>2]|0)|0;k=c[g>>2]|0;l=(ac(c[k+416>>2]|0,j)|0)>>3;m=k}else{l=i;m=h}h=c[m+72>>2]|0;if((h|0)==0){n=ac(c[m+412>>2]|0,l)|0}else{n=h>>3}if((l|0)<1|(n|0)<1){return-1|0}else{h=0;m=(e|0)<(h|0)|(e|0)==(h|0)&d>>>0<0>>>0;h=n;i=(n|0)<0|0?-1:0;n=N2(h,i,m?0:d,m?0:e)|0;e=G;m=c[b+40>>2]|0;d=m;k=(m|0)<0|0?-1:0;m=c[b+44>>2]|0;j=m;g=(m|0)<0|0?-1:0;m=l;o=(l|0)<0|0?-1:0;l=N2(j,g,m,o)|0;p=AV(n,e,d,k,l,G,f&1^3)|0;f=N2(p,G,m,o)|0;o=G;m=N2(d,k,h,i)|0;i=AW(f,o,j,g,m,G)|0;m=b+208|0;c[m>>2]=i;c[m+4>>2]=G;m=c[a+16>>2]|0;i=a+1232|0;a=NU(c[i>>2]|0,c[i+4>>2]|0,f,o)|0;o=il(m,a,G,0)|0;a=G;m=0;return((a|0)<(m|0)|(a|0)==(m|0)&o>>>0<0>>>0?o:0)|0}return 0}function nx(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;if((nC(b,1024)|0)<0){d=-12;return d|0}e=a+16|0;a=il(c[e>>2]|0,0,0,1)|0;f=b+64|0;c[f>>2]=a;c[f+4>>2]=G;c[b+32>>2]=0;f=iF(c[e>>2]|0,c[b+24>>2]|0,1024)|0;if((f|0)<0){nJ(b);d=f;return d|0}else{nE(b,f);d=f;return d|0}return 0}function ny(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;e=i;i=i+8|0;f=e|0;g=(a|0)!=0;L20065:do{if(g){h=a+8|0;j=c[h>>2]|0;if((j|0)==0){k=b+48|0;l=b+412|0;m=b+416|0;n=b+8|0}else{do{if((c[j+36>>2]|0)!=0){o=c[a+12>>2]|0;if((o|0)==0){break}p=(Bn(o,1679176,0,f)|0)>-1;o=c[f>>2]|0;q=c[f+4>>2]|0;r=-1;if(!(p&((q|0)>(r|0)|(q|0)==(r|0)&o>>>0>-1>>>0))){break}s=o;i=e;return s|0}}while(0);j=b+48|0;o=b+412|0;r=b+416|0;q=b+8|0;if(g){k=j;l=o;m=r;n=q}else{t=j;u=o;v=r;w=q;x=14917;break}}q=a+12|0;r=0;o=85016;while(1){j=c[85032+(r*28|0)>>2]|0;do{if((j|0)==(c[k>>2]|0)){if((r|0)==25){p=c[h>>2]|0;if((p|0)==0){y=n;x=14955;break L20065}if((c[p+36>>2]|0)==0){y=n;x=14955;break L20065}p=c[q>>2]|0;if((p|0)==0){y=n;x=14955;break L20065}if((Bo(p,1728376,1697584)|0)==0){y=n;x=14955;break L20065}z=c[k>>2]|0}else{z=j}do{if((z|0)==69660){if((c[l>>2]|0)!=16e3){break}if((c[m>>2]|0)==1){A=o;x=14947;break L20065}}}while(0);if((c[n>>2]|0)!=1){B=o;x=14953;break L20065}if((67096575>>>(r>>>0)&1|0)!=0){if((c[l>>2]|0)!=(c[85036+(r*28|0)>>2]|0)){break}}if((249855>>>(r>>>0)&1|0)==0){B=o;x=14953;break L20065}if((c[m>>2]|0)==(c[85040+(r*28|0)>>2]|0)){B=o;x=14953;break L20065}}}while(0);j=r+1|0;if((j|0)==26){y=n;x=14955;break L20065}r=j;o=85016+(j*28|0)|0}}else{t=b+48|0;u=b+412|0;v=b+416|0;w=b+8|0;x=14917}}while(0);L20100:do{if((x|0)==14917){b=c[t>>2]|0;n=0;m=85016;l=b;while(1){do{if((c[85032+(n*28|0)>>2]|0)==(l|0)){if((l|0)==5){C=5;break}do{if((b|0)==69660){if((c[u>>2]|0)!=16e3){break}if((c[v>>2]|0)==1){A=m;x=14947;break L20100}}}while(0);if((c[w>>2]|0)!=1){B=m;x=14953;break L20100}if((67096575>>>(n>>>0)&1|0)!=0){if((c[u>>2]|0)!=(c[85036+(n*28|0)>>2]|0)){C=b;break}}if((249855>>>(n>>>0)&1|0)==0){B=m;x=14953;break L20100}if((c[v>>2]|0)==(c[85040+(n*28|0)>>2]|0)){B=m;x=14953;break L20100}else{C=b}}else{C=l}}while(0);z=n+1|0;if((z|0)==26){y=w;x=14955;break}else{n=z;m=85016+(z*28|0)|0;l=C}}}}while(0);if((x|0)==14955){if((d|0)<0){D=(c[y>>2]|0)==1|0}else{D=d}s=D+96|0;i=e;return s|0}else if((x|0)==14947){s=c[A>>2]|0;i=e;return s|0}else if((x|0)==14953){s=c[B>>2]|0;i=e;return s|0}return 0}function nz(){if(a[340896]|0){return}a[340896]=1;oT(257408);oT(247368);oT(236648);oT(236752);oT(244968);oT(244864);oT(248216);oT(248112);oT(247904);oT(248008);oD(245496);return}function nA(a){a=a|0;var b=0,d=0,e=0,f=0;b=a+44|0;d=a+40|0;if((c[b>>2]|0)>0){e=0}else{f=d;A6(f);c[b>>2]=0;return}do{A4(c[(c[d>>2]|0)+(e*12|0)>>2]|0);e=e+1|0;}while((e|0)<(c[b>>2]|0));f=d;A6(f);c[b>>2]=0;return}function nB(a){a=a|0;var b=0;b=a+8|0;c[b>>2]=0;c[b+4>>2]=-2147483648;b=a+16|0;c[b>>2]=0;c[b+4>>2]=-2147483648;b=a+64|0;c[b>>2]=-1;c[b+4>>2]=-1;b=a+72|0;c[b>>2]=0;c[b+4>>2]=0;c[a>>2]=0;NM(a+32|0,0,24);return}function nC(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=i;i=i+8|0;e=d|0;c[e>>2]=0;if(b>>>0>4294967279){f=-22;i=d;return f|0}zl(e,b+16|0)|0;g=c[e>>2]|0;if((g|0)==0){f=-12;i=d;return f|0}e=g+4|0;NM((c[e>>2]|0)+b|0,0,16);h=a+8|0;c[h>>2]=0;c[h+4>>2]=-2147483648;h=a+16|0;c[h>>2]=0;c[h+4>>2]=-2147483648;h=a+64|0;c[h>>2]=-1;c[h+4>>2]=-1;h=a+72|0;c[h>>2]=0;c[h+4>>2]=0;NM(a+32|0,0,24);c[a>>2]=g;c[a+24>>2]=c[e>>2];c[a+28>>2]=b;c[a+52>>2]=54;f=0;i=d;return f|0}function nD(a){a=a|0;AN(0,0,1678880,(a=i,i=i+24|0,c[a>>2]=1642696,c[a+8>>2]=1697504,c[a+16>>2]=52,a)|0);i=a;cw()}function nE(a,b){a=a|0;b=b|0;var d=0;d=a+28|0;if((c[d>>2]|0)<=(b|0)){return}c[d>>2]=b;NM((c[a+24>>2]|0)+b|0,0,16);return}function nF(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;d=i;i=i+8|0;e=d|0;f=a+28|0;g=c[f>>2]|0;if(g>>>0>=2147483632){AN(0,0,1678880,(h=i,i=i+24|0,c[h>>2]=1728256,c[h+8>>2]=1697504,c[h+16>>2]=108,h)|0);i=h;cw();return 0}if((g|0)==0){c[e>>2]=0;if(b>>>0>4294967279){j=-22;i=d;return j|0}h=b+16|0;zl(e,h)|0;h=c[e>>2]|0;if((h|0)==0){j=-12;i=d;return j|0}e=h+4|0;NM((c[e>>2]|0)+b|0,0,16);k=a+8|0;c[k>>2]=0;c[k+4>>2]=-2147483648;k=a+16|0;c[k>>2]=0;c[k+4>>2]=-2147483648;k=a+64|0;c[k>>2]=-1;c[k+4>>2]=-1;k=a+72|0;c[k>>2]=0;c[k+4>>2]=0;NM(a+32|0,0,24);c[a>>2]=h;c[a+24>>2]=c[e>>2];c[f>>2]=b;c[a+52>>2]=54;j=0;i=d;return j|0}if((2147483631-g|0)>>>0<b>>>0){j=-1;i=d;return j|0}e=b+16+g|0;g=a|0;do{if((c[g>>2]|0)==0){h=zg(e)|0;c[g>>2]=h;if((h|0)==0){j=-12;i=d;return j|0}else{k=c[h+4>>2]|0;h=a+24|0;l=c[h>>2]|0;m=(c[f>>2]|0)+((b|0)<0?b:0)|0;NN(k|0,l|0,m)|0;c[a+52>>2]=54;n=h;break}}else{h=zl(g,e)|0;if((h|0)<0){j=h;i=d;return j|0}else{n=a+24|0;break}}}while(0);a=c[(c[g>>2]|0)+4>>2]|0;c[n>>2]=a;n=(c[f>>2]|0)+b|0;c[f>>2]=n;NM(a+n|0,0,16);j=0;i=d;return j|0}function nG(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;d=b+44|0;e=c[d>>2]|0;if((e|0)==0){f=0;return f|0}g=A1(e*12|0)|0;L20185:do{if((g|0)!=0){e=b+40|0;h=c[e>>2]|0;i=(c[d>>2]|0)*12|0;NN(g|0,h|0,i)|0;i=a+40|0;c[i>>2]=g;h=c[d>>2]|0;NM(g|0,0,h*12|0|0);if((h|0)>0){j=0}else{f=0;return f|0}while(1){h=c[(c[e>>2]|0)+(j*12|0)+4>>2]|0;if(h>>>0>4294967279){break L20185}k=A1(h+16|0)|0;if((k|0)==0){break L20185}h=c[e>>2]|0;l=c[h+(j*12|0)>>2]|0;m=h+(j*12|0)+4|0;h=c[m>>2]|0;NN(k|0,l|0,h)|0;NM(k+(c[m>>2]|0)|0,0,16);c[(c[i>>2]|0)+(j*12|0)>>2]=k;c[(c[i>>2]|0)+(j*12|0)+4>>2]=c[(c[e>>2]|0)+(j*12|0)+4>>2];c[(c[i>>2]|0)+(j*12|0)+8>>2]=c[(c[e>>2]|0)+(j*12|0)+8>>2];k=j+1|0;if((k|0)<(c[d>>2]|0)){j=k}else{f=0;break}}return f|0}}while(0);j=a+24|0;A4(c[j>>2]|0);c[j>>2]=0;c[a+28>>2]=0;f=-12;return f|0}function nH(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=i;i=i+80|0;d=b|0;do{if((c[a>>2]|0)==0){if((c[a+24>>2]|0)==0){e=0;break}if((c[a+52>>2]|0)!=0){e=0;break}f=d;g=a;NN(f|0,g|0,80)|0;e=nI(a,d,1)|0}else{e=0}}while(0);i=b;return e|0}function nI(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0;e=a+24|0;c[e>>2]=0;f=a+40|0;c[f>>2]=0;g=a|0;L20203:do{if((c[g>>2]|0)==0){h=a+28|0;i=c[h>>2]|0;do{if(i>>>0<=4294967279){j=i+16|0;zl(g,j)|0;j=c[g>>2]|0;if((j|0)==0){break}k=c[j+4>>2]|0;if((k|0)==0){break}j=c[b+24>>2]|0;l=c[h>>2]|0;NN(k|0,j|0,l)|0;NM(k+(c[h>>2]|0)|0,0,16);m=k;break L20203}}while(0);A4(c[e>>2]|0);c[e>>2]=0;c[h>>2]=0;n=-12;return n|0}else{i=zi(c[b>>2]|0)|0;if((i|0)==0){n=-12;return n|0}else{c[g>>2]=i;m=c[i+4>>2]|0;break}}}while(0);c[e>>2]=m;c[a+52>>2]=54;m=c[a+44>>2]|0;e=(d|0)==0;if(!((m|0)==0|e)){c[f>>2]=c[b+40>>2]}if(!((m|0)!=0&e)){n=0;return n|0}n=nG(a,b)|0;return n|0}function nJ(a){a=a|0;var b=0,d=0,e=0,f=0;if((a|0)==0){return}b=a|0;do{if((c[b>>2]|0)==0){d=a+52|0;e=c[d>>2]|0;if((e|0)==0){f=d;break}c_[e&127](a);f=d}else{zj(b);f=a+52|0}}while(0);c[f>>2]=0;c[a+24>>2]=0;c[a+28>>2]=0;f=a+44|0;b=a+40|0;if((c[f>>2]|0)>0){a=0;do{A4(c[(c[b>>2]|0)+(a*12|0)>>2]|0);a=a+1|0;}while((a|0)<(c[f>>2]|0))}A6(b);c[f>>2]=0;return}function nK(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=a+44|0;f=c[e>>2]|0;if((f+1|0)>>>0>178956970|d>>>0>2147483631){g=0;return g|0}h=a+40|0;a=A2(c[h>>2]|0,(f*12|0)+12|0)|0;c[h>>2]=a;if((a|0)==0){g=0;return g|0}a=A1(d+16|0)|0;c[(c[h>>2]|0)+(f*12|0)>>2]=a;a=c[h>>2]|0;if((c[a+(f*12|0)>>2]|0)==0){g=0;return g|0}c[a+(f*12|0)+4>>2]=d;c[(c[h>>2]|0)+(f*12|0)+8>>2]=b;c[e>>2]=(c[e>>2]|0)+1;g=c[(c[h>>2]|0)+(f*12|0)>>2]|0;return g|0}function nL(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0;e=c[a+44>>2]|0;f=a+40|0;a=0;while(1){if((a|0)>=(e|0)){g=0;h=15069;break}i=c[f>>2]|0;if((c[i+(a*12|0)+8>>2]|0)==(b|0)){break}else{a=a+1|0}}if((h|0)==15069){return g|0}if((d|0)==0){j=i}else{c[d>>2]=c[i+(a*12|0)+4>>2];j=c[f>>2]|0}g=c[j+(a*12|0)>>2]|0;return g|0}function nM(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;d=i;i=i+80|0;e=d|0;f=b+44|0;if((c[f>>2]|0)==0){g=0;i=d;return g|0}h=b+28|0;j=c[h>>2]|0;k=NU(j,(j|0)<0|0?-1:0,24,0)|0;j=G;l=e;m=b;NN(l|0,m|0,80)|0;m=e+44|0;l=c[m>>2]|0;if((l|0)>0){n=c[e+40>>2]|0;o=0;p=j;q=k;while(1){r=c[n+(o*12|0)+4>>2]|0;s=NU(q,p,5,0)|0;t=NU(s,G,r,(r|0)<0|0?-1:0)|0;r=G;s=o+1|0;if((s|0)<(l|0)){o=s;p=r;q=t}else{u=r;v=t;break}}}else{u=j;v=k}k=0;if(u>>>0>k>>>0|u>>>0==k>>>0&v>>>0>2147483647>>>0){g=-22;i=d;return g|0}k=zg(v)|0;if((k|0)==0){g=-12;i=d;return g|0}c[b>>2]=k;j=c[k+4>>2]|0;k=b+24|0;c[k>>2]=j;c[b+52>>2]=54;q=NU(v,u,-16,0)|0;c[h>>2]=q;q=e+24|0;u=c[q>>2]|0;v=e+28|0;p=c[v>>2]|0;NN(j|0,u|0,p)|0;u=j+p|0;p=c[m>>2]|0;if((p|0)>0){j=e+40|0;o=u;l=p;while(1){p=l-1|0;n=c[j>>2]|0;t=c[n+(p*12|0)>>2]|0;r=c[n+(p*12|0)+4>>2]|0;NN(o|0,t|0,r)|0;t=o+r|0;y=cV(c[(c[j>>2]|0)+(p*12|0)+4>>2]|0)|0;a[t]=y&255;y=y>>8;a[t+1|0]=y&255;y=y>>8;a[t+2|0]=y&255;y=y>>8;a[t+3|0]=y&255;t=o+(r+5)|0;a[o+(r+4)|0]=(((p|0)==((c[m>>2]|0)-1|0))<<7|c[(c[j>>2]|0)+(p*12|0)+8>>2])&255;if((p|0)>0){o=t;l=p}else{w=t;break}}}else{w=u}u=w;l=u|0;y=278744460;a[l]=y&255;y=y>>8;a[l+1|0]=y&255;y=y>>8;a[l+2|0]=y&255;y=y>>8;a[l+3|0]=y&255;l=u+4|0;y=-18274930;a[l]=y&255;y=y>>8;a[l+1|0]=y&255;y=y>>8;a[l+2|0]=y&255;y=y>>8;a[l+3|0]=y&255;l=w+8|0;if((l-(c[k>>2]|0)|0)!=(c[h>>2]|0)){AN(0,0,1678880,(h=i,i=i+24|0,c[h>>2]=1675472,c[h+8>>2]=1697504,c[h+16>>2]=346,h)|0);i=h;cw();return 0}NM(l|0,0,16);l=e|0;do{if((c[l>>2]|0)==0){h=e+52|0;k=c[h>>2]|0;if((k|0)==0){x=h;break}c_[k&127](e);x=h}else{zj(l);x=e+52|0}}while(0);c[x>>2]=0;c[q>>2]=0;c[v>>2]=0;v=e+40|0;if((c[m>>2]|0)>0){e=0;do{A4(c[(c[v>>2]|0)+(e*12|0)>>2]|0);e=e+1|0;}while((e|0)<(c[m>>2]|0))}A6(v);c[m>>2]=0;c[f>>2]=0;c[b+40>>2]=0;g=1;i=d;return g|0}function nN(b){b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;e=i;f=b+44|0;if((c[f>>2]|0)!=0){g=0;i=e;return g|0}h=b+28|0;j=c[h>>2]|0;if((j|0)<=12){g=0;i=e;return g|0}k=b+24|0;l=c[k>>2]|0;m=l+(j-8)|0;n=m|0;o=m+4|0;m=d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0;o=cV(d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0)|0;if(!((((m>>>16|0<<16)<<8|0>>>24)&65280|(m>>>24|0<<8)|(m<<8|0>>>24|(m>>>8|0<<24)&255)<<16|0)==(-1910117890|0)&(o|0)==(-1941070576|0))){g=0;i=e;return g|0}o=l+(j-13)|0;m=o;n=cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0;if((n|0)<0){g=0;i=e;return g|0}m=l;l=1;p=o;o=n;while(1){if((p-m|0)>>>0<o>>>0){g=0;q=15118;break}if((a[p+4|0]|0)<0){q=15102;break}n=p+(-5-o)|0;r=n;s=cV(d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0)|0;if((s|0)<0){g=0;q=15119;break}else{l=l+1|0;p=n;o=s}}if((q|0)==15119){i=e;return g|0}else if((q|0)==15118){i=e;return g|0}else if((q|0)==15102){o=A1(l*12|0)|0;l=b+40|0;c[l>>2]=o;if((o|0)==0){g=-12;i=e;return g|0}o=c[k>>2]|0;b=o+((c[h>>2]|0)-13)|0;p=b;m=cV(d[p]|d[p+1|0]<<8|d[p+2|0]<<16|d[p+3|0]<<24|0)|0;if((m|0)>-1){t=0;u=b;v=m;w=o}else{AN(0,0,1678880,(x=i,i=i+24|0,c[x>>2]=1659904,c[x+8>>2]=1697504,c[x+16>>2]=379,x)|0);i=x;cw();return 0}while(1){if((u-w|0)>>>0<v>>>0){q=15114;break}o=A1(v+16|0)|0;c[(c[l>>2]|0)+(t*12|0)>>2]=o;c[(c[l>>2]|0)+(t*12|0)+4>>2]=v;o=u+4|0;c[(c[l>>2]|0)+(t*12|0)+8>>2]=a[o]&127;m=c[(c[l>>2]|0)+(t*12|0)>>2]|0;if((m|0)==0){g=-12;q=15116;break}b=u+(-v|0)|0;NN(m|0,b|0,v)|0;y=-5-v+(c[h>>2]|0)|0;c[h>>2]=y;if((a[o]|0)<0){q=15110;break}o=u+(-5-v)|0;b=o;m=cV(d[b]|d[b+1|0]<<8|d[b+2|0]<<16|d[b+3|0]<<24|0)|0;if((m|0)<=-1){q=15112;break}t=t+1|0;u=o;v=m;w=c[k>>2]|0}if((q|0)==15112){AN(0,0,1678880,(x=i,i=i+24|0,c[x>>2]=1659904,c[x+8>>2]=1697504,c[x+16>>2]=379,x)|0);i=x;cw();return 0}else if((q|0)==15114){AN(0,0,1678880,(x=i,i=i+24|0,c[x>>2]=1659904,c[x+8>>2]=1697504,c[x+16>>2]=379,x)|0);i=x;cw();return 0}else if((q|0)==15116){i=e;return g|0}else if((q|0)==15110){q=y-8|0;c[h>>2]=q;h=j-q|0;NM((c[k>>2]|0)+q|0,0,(h>>>0>16?16:h)|0);c[f>>2]=t+1;g=1;i=e;return g|0}}return 0}function nO(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;return AH(a|0,a+32|0,b,c,d,e,1)|0}function nP(a){a=a|0;return c[((a|0)==0?2225216:a+16|0)>>2]|0}function nQ(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=2225216;while(1){d=c[b>>2]|0;if((d|0)==0){e=0;f=15133;break}if((a8(a|0,c[d>>2]|0)|0)==0){break}else{b=d+16|0}}if((f|0)==15133){return e|0}f=A7(16)|0;c[f+4>>2]=d;b=c[d+4>>2]|0;if((b|0)==0){g=0}else{g=A7(b)|0}c[f>>2]=g;e=f;return e|0}function nR(a){a=a|0;var b=0;if((a|0)==0){return}b=c[(c[a+4>>2]|0)+12>>2]|0;if((b|0)!=0){c_[b&127](a)}b=a;A6(b);oH(c[a+8>>2]|0);A4(b);return}function nS(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;c[e>>2]=g;c[f>>2]=h;return dp[c[(c[a+4>>2]|0)+8>>2]&7](a,b,d,e,f,g,h,i)|0}function nT(a){a=a|0;var b=0,d=0,e=0,f=0;b=0;while(1){if(b>>>0>=365){d=0;e=15147;break}f=290856+(b*20|0)|0;if((c[f>>2]|0)==(a|0)){d=f;e=15148;break}else{b=b+1|0}}if((e|0)==15148){return d|0}else if((e|0)==15147){return d|0}return 0}function nU(a){a=a|0;if((a|0)==0){return 290856}else{return(((a-290856|0)/20|0)>>>0<364?a+20|0:0)|0}return 0}function nV(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=0;while(1){if((b|0)==0){d=290856}else{e=((b-290856|0)/20|0)>>>0<364?b+20|0:0;if((e|0)==0){f=0;g=15157;break}else{d=e}}if((a8(c[d+8>>2]|0,a|0)|0)==0){f=d;g=15158;break}else{b=d}}if((g|0)==15158){return f|0}else if((g|0)==15157){return f|0}return 0}function nW(e,f,g){e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0,cb=0,cc=0,cd=0,ce=0,cf=0,cg=0,ch=0;h=i;i=i+16|0;j=h|0;k=h+8|0;l=f+8|0;m=c[l>>2]|0;n=f+16|0;o=c[n>>2]|0;p=f|0;f=c[p>>2]|0;q=f+(m>>>3)|0;r=(cV(d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0)|0)<<(m&7);if((r&-1434451968|0)==0){q=1;s=m;t=r;while(1){u=t>>>24;v=a[248576+u|0]|0;w=v&255;x=((v&255)>8?8:w)+s|0;y=o>>>0>x>>>0;z=y?x:o;if(v<<24>>24!=9){A=15162;break}v=d[248832+u|0]|q<<4;if(v>>>0>=134217728){B=v;break}x=f+(z>>>3)|0;C=(cV(d[x]|d[x+1|0]<<8|d[x+2|0]<<16|d[x+3|0]<<24|0)|0)<<(z&7);if(y){q=v;s=z;t=C}else{B=v;break}}if((A|0)==15162){B=d[248832+u|0]|q<<(w-1>>1)}c[l>>2]=z;D=B-1|0;E=z}else{z=r>>>24;r=(d[248576+z|0]|0)+m|0;m=o>>>0>r>>>0?r:o;c[l>>2]=m;D=d[248320+z|0]|0;E=m}m=f+(E>>>3)|0;z=(cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0)<<(E&7);L20383:do{if((z&-1434451968|0)==0){m=1;r=E;B=z;while(1){w=B>>>24;q=a[248576+w|0]|0;u=((q&255)>8?8:q&255)+r|0;t=o>>>0>u>>>0;s=t?u:o;if(q<<24>>24!=9){F=s;break L20383}q=d[248832+w|0]|m<<4;if(q>>>0>=134217728){F=s;break L20383}w=f+(s>>>3)|0;u=(cV(d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0)|0)<<(s&7);if(t){m=q;r=s;B=u}else{F=s;break}}}else{B=(d[248576+(z>>>24)|0]|0)+E|0;F=o>>>0>B>>>0?B:o}}while(0);c[l>>2]=F;E=f+(F>>>3)|0;z=(cV(d[E]|d[E+1|0]<<8|d[E+2|0]<<16|d[E+3|0]<<24|0)|0)<<(F&7);if((z&-1434451968|0)==0){E=1;B=F;r=z;while(1){G=r>>>24;m=a[248576+G|0]|0;H=m&255;s=((m&255)>8?8:H)+B|0;u=o>>>0>s>>>0;I=u?s:o;if(m<<24>>24!=9){A=15174;break}m=d[248832+G|0]|E<<4;if(m>>>0>=134217728){J=m;break}s=f+(I>>>3)|0;q=(cV(d[s]|d[s+1|0]<<8|d[s+2|0]<<16|d[s+3|0]<<24|0)|0)<<(I&7);if(u){E=m;B=I;r=q}else{J=m;break}}if((A|0)==15174){J=d[248832+G|0]|E<<(H-1>>1)}c[l>>2]=I;K=J-1|0}else{J=z>>>24;z=(d[248576+J|0]|0)+F|0;c[l>>2]=o>>>0>z>>>0?z:o;K=d[248320+J|0]|0}c[e+840>>2]=K;K=c[l>>2]|0;J=c[n>>2]|0;o=c[p>>2]|0;z=o+(K>>>3)|0;F=(cV(d[z]|d[z+1|0]<<8|d[z+2|0]<<16|d[z+3|0]<<24|0)|0)<<(K&7);if((F&-1434451968|0)==0){z=1;I=K;H=F;while(1){L=H>>>24;E=a[248576+L|0]|0;M=E&255;G=((E&255)>8?8:M)+I|0;r=J>>>0>G>>>0;N=r?G:J;if(E<<24>>24!=9){A=15181;break}E=d[248832+L|0]|z<<4;if(E>>>0>=134217728){O=E;break}G=o+(N>>>3)|0;B=(cV(d[G]|d[G+1|0]<<8|d[G+2|0]<<16|d[G+3|0]<<24|0)|0)<<(N&7);if(r){z=E;I=N;H=B}else{O=E;break}}if((A|0)==15181){O=d[248832+L|0]|z<<(M-1>>1)}c[l>>2]=N;P=O-1|0}else{O=F>>>24;F=(d[248576+O|0]|0)+K|0;c[l>>2]=J>>>0>F>>>0?F:J;P=d[248320+O|0]|0}c[e+844>>2]=P;P=c[l>>2]|0;O=c[n>>2]|0;J=c[p>>2]|0;F=J+(P>>>3)|0;K=(cV(d[F]|d[F+1|0]<<8|d[F+2|0]<<16|d[F+3|0]<<24|0)|0)<<(P&7);if((K&-1434451968|0)==0){F=1;N=P;M=K;while(1){Q=M>>>24;z=a[248576+Q|0]|0;R=z&255;L=((z&255)>8?8:R)+N|0;H=O>>>0>L>>>0;S=H?L:O;if(z<<24>>24!=9){A=15188;break}z=d[248832+Q|0]|F<<4;if(z>>>0>=134217728){T=z;break}L=J+(S>>>3)|0;I=(cV(d[L]|d[L+1|0]<<8|d[L+2|0]<<16|d[L+3|0]<<24|0)|0)<<(S&7);if(H){F=z;N=S;M=I}else{T=z;break}}if((A|0)==15188){T=d[248832+Q|0]|F<<(R-1>>1)}c[l>>2]=S;U=T-1|0}else{T=K>>>24;K=(d[248576+T|0]|0)+P|0;c[l>>2]=O>>>0>K>>>0?K:O;U=d[248320+T|0]|0}do{if(D>>>0<2){AN(e,24,1667904,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V}else{if(D>>>0<=2){break}AN(e,24,1725360,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V}}while(0);if(U>>>0>20){W=-1094995529;i=h;return W|0}D=g;T=262064+(U*24|0)|0;c[D>>2]=c[T>>2];c[D+4>>2]=c[T+4>>2];c[D+8>>2]=c[T+8>>2];c[D+12>>2]=c[T+12>>2];c[D+16>>2]=c[T+16>>2];c[D+20>>2]=c[T+20>>2];T=c[l>>2]|0;D=c[p>>2]|0;U=d[D+(T>>>3)|0]|0;O=c[n>>2]|0;K=((T|0)<(O|0))+T|0;c[l>>2]=K;if((U&128>>>((T&7)>>>0)|0)==0){X=K;Y=D;Z=O}else{T=D+(K>>>3)|0;U=(cV(d[T]|d[T+1|0]<<8|d[T+2|0]<<16|d[T+3|0]<<24|0)|0)<<(K&7);if((U&-1434451968|0)==0){T=1;P=K;S=U;while(1){_=S>>>24;R=a[248576+_|0]|0;$=R&255;F=((R&255)>8?8:$)+P|0;Q=O>>>0>F>>>0;aa=Q?F:O;if(R<<24>>24!=9){A=15201;break}R=d[248832+_|0]|T<<4;if(R>>>0>=134217728){ab=R;break}F=D+(aa>>>3)|0;M=(cV(d[F]|d[F+1|0]<<8|d[F+2|0]<<16|d[F+3|0]<<24|0)|0)<<(aa&7);if(Q){T=R;P=aa;S=M}else{ab=R;break}}if((A|0)==15201){ab=d[248832+_|0]|T<<($-1>>1)}c[l>>2]=aa;ac=ab-1|0}else{ab=U>>>24;U=(d[248576+ab|0]|0)+K|0;c[l>>2]=O>>>0>U>>>0?U:O;ac=d[248320+ab|0]|0}c[g>>2]=ac;ac=c[l>>2]|0;ab=c[n>>2]|0;O=c[p>>2]|0;U=O+(ac>>>3)|0;K=(cV(d[U]|d[U+1|0]<<8|d[U+2|0]<<16|d[U+3|0]<<24|0)|0)<<(ac&7);if((K&-1434451968|0)==0){U=1;aa=ac;$=K;while(1){ad=$>>>24;T=a[248576+ad|0]|0;ae=T&255;_=((T&255)>8?8:ae)+aa|0;S=ab>>>0>_>>>0;af=S?_:ab;if(T<<24>>24!=9){A=15208;break}T=d[248832+ad|0]|U<<4;if(T>>>0>=134217728){ag=T;break}_=O+(af>>>3)|0;P=(cV(d[_]|d[_+1|0]<<8|d[_+2|0]<<16|d[_+3|0]<<24|0)|0)<<(af&7);if(S){U=T;aa=af;$=P}else{ag=T;break}}if((A|0)==15208){ag=d[248832+ad|0]|U<<(ae-1>>1)}c[l>>2]=af;ah=ag-1|0}else{ag=K>>>24;K=(d[248576+ag|0]|0)+ac|0;c[l>>2]=ab>>>0>K>>>0?K:ab;ah=d[248320+ag|0]|0}c[g+4>>2]=ah;X=c[l>>2]|0;Y=c[p>>2]|0;Z=c[n>>2]|0}ah=d[Y+(X>>>3)|0]|0;ag=((X|0)<(Z|0))+X|0;c[l>>2]=ag;if((ah&128>>>((X&7)>>>0)|0)==0){ai=a[g+8|0]|0}else{X=Y+(ag>>>3)|0;ah=(cV(d[X]|d[X+1|0]<<8|d[X+2|0]<<16|d[X+3|0]<<24|0)|0)<<(ag&7);if((ah&-1434451968|0)==0){X=1;ab=ag;K=ah;while(1){aj=K>>>24;ac=a[248576+aj|0]|0;ak=ac&255;af=((ac&255)>8?8:ak)+ab|0;ae=Z>>>0>af>>>0;al=ae?af:Z;if(ac<<24>>24!=9){A=15218;break}ac=d[248832+aj|0]|X<<4;if(ac>>>0>=134217728){am=ac;break}af=Y+(al>>>3)|0;U=(cV(d[af]|d[af+1|0]<<8|d[af+2|0]<<16|d[af+3|0]<<24|0)|0)<<(al&7);if(ae){X=ac;ab=al;K=U}else{am=ac;break}}if((A|0)==15218){am=d[248832+aj|0]|X<<(ak-1>>1)}c[l>>2]=al;an=am+255&255}else{am=ah>>>24;ah=(d[248576+am|0]|0)+ag|0;c[l>>2]=Z>>>0>ah>>>0?ah:Z;an=a[248320+am|0]|0}a[g+8|0]=an;ai=an}an=g+8|0;if((ai&255)>2){AN(e,16,1674312,(V=i,i=i+8|0,c[V>>2]=ai&255,V)|0);i=V;W=-1094995529;i=h;return W|0}ai=c[l>>2]|0;am=c[p>>2]|0;Z=d[am+(ai>>>3)|0]|0;ah=c[n>>2]|0;ag=((ai|0)<(ah|0))+ai|0;c[l>>2]=ag;if((Z&128>>>((ai&7)>>>0)|0)==0){ao=a[g+9|0]|0}else{ai=am+(ag>>>3)|0;Z=(cV(d[ai]|d[ai+1|0]<<8|d[ai+2|0]<<16|d[ai+3|0]<<24|0)|0)<<(ag&7);if((Z&-1434451968|0)==0){ai=1;al=ag;ak=Z;while(1){ap=ak>>>24;X=a[248576+ap|0]|0;aq=X&255;aj=((X&255)>8?8:aq)+al|0;K=ah>>>0>aj>>>0;ar=K?aj:ah;if(X<<24>>24!=9){A=15230;break}X=d[248832+ap|0]|ai<<4;if(X>>>0>=134217728){as=X;break}aj=am+(ar>>>3)|0;ab=(cV(d[aj]|d[aj+1|0]<<8|d[aj+2|0]<<16|d[aj+3|0]<<24|0)|0)<<(ar&7);if(K){ai=X;al=ar;ak=ab}else{as=X;break}}if((A|0)==15230){as=d[248832+ap|0]|ai<<(aq-1>>1)}c[l>>2]=ar;at=as+255&255}else{as=Z>>>24;Z=(d[248576+as|0]|0)+ag|0;c[l>>2]=ah>>>0>Z>>>0?Z:ah;at=a[248320+as|0]|0}a[g+9|0]=at;ao=at}if((ao&255)>1){W=-1094995529;i=h;return W|0}ao=c[l>>2]|0;at=c[p>>2]|0;as=d[at+(ao>>>3)|0]|0;ah=c[n>>2]|0;Z=((ao|0)<(ah|0))+ao|0;c[l>>2]=Z;do{if((as&128>>>((ao&7)>>>0)|0)==0){au=0;av=0;aw=g+11|0}else{ag=at+(Z>>>3)|0;ar=(cV(d[ag]|d[ag+1|0]<<8|d[ag+2|0]<<16|d[ag+3|0]<<24|0)|0)<<(Z&7);if((ar&-1434451968|0)==0){ag=1;aq=Z;ai=ar;while(1){ax=ai>>>24;ap=a[248576+ax|0]|0;ay=ap&255;ak=((ap&255)>8?8:ay)+aq|0;al=ah>>>0>ak>>>0;az=al?ak:ah;if(ap<<24>>24!=9){A=15241;break}ap=d[248832+ax|0]|ag<<4;if(ap>>>0>=134217728){aA=ap;break}ak=at+(az>>>3)|0;am=(cV(d[ak]|d[ak+1|0]<<8|d[ak+2|0]<<16|d[ak+3|0]<<24|0)|0)<<(az&7);if(al){ag=ap;aq=az;ai=am}else{aA=ap;break}}if((A|0)==15241){aA=d[248832+ax|0]|ag<<(ay-1>>1)}c[l>>2]=az;aB=aA-1|0}else{ai=ar>>>24;aq=(d[248576+ai|0]|0)+Z|0;c[l>>2]=ah>>>0>aq>>>0?aq:ah;aB=d[248320+ai|0]|0}ai=aB&255;aq=g+11|0;a[aq]=ai;if((aB&255)>>>0>10){W=-1094995529;i=h;return W|0}if(ai<<24>>24!=0){au=0;av=0;aw=aq;break}ai=c[l>>2]|0;ap=c[n>>2]|0;am=c[p>>2]|0;al=am+(ai>>>3)|0;ak=(cV(d[al]|d[al+1|0]<<8|d[al+2|0]<<16|d[al+3|0]<<24|0)|0)<<(ai&7);if((ak&-1434451968|0)==0){al=1;X=ai;ab=ak;while(1){aC=ab>>>24;K=a[248576+aC|0]|0;aD=K&255;aj=((K&255)>8?8:aD)+X|0;Y=ap>>>0>aj>>>0;aE=Y?aj:ap;if(K<<24>>24!=9){A=15250;break}K=d[248832+aC|0]|al<<4;if(K>>>0>=134217728){aF=K;break}aj=am+(aE>>>3)|0;ac=(cV(d[aj]|d[aj+1|0]<<8|d[aj+2|0]<<16|d[aj+3|0]<<24|0)|0)<<(aE&7);if(Y){al=K;X=aE;ab=ac}else{aF=K;break}}if((A|0)==15250){aF=d[248832+aC|0]|al<<(aD-1>>1)}c[l>>2]=aE;aG=aF-1|0;aH=aE}else{ab=ak>>>24;X=(d[248576+ab|0]|0)+ai|0;ar=ap>>>0>X>>>0?X:ap;c[l>>2]=ar;aG=d[248320+ab|0]|0;aH=ar}ar=am+(aH>>>3)|0;ab=(cV(d[ar]|d[ar+1|0]<<8|d[ar+2|0]<<16|d[ar+3|0]<<24|0)|0)<<(aH&7);if((ab&-1434451968|0)==0){aI=1;aJ=aH;aK=ab}else{ar=ab>>>24;ab=(d[248576+ar|0]|0)+aH|0;c[l>>2]=ap>>>0>ab>>>0?ab:ap;au=aG;av=d[248320+ar|0]|0;aw=aq;break}while(1){aL=aK>>>24;ar=a[248576+aL|0]|0;aM=ar&255;ab=((ar&255)>8?8:aM)+aJ|0;X=ap>>>0>ab>>>0;aN=X?ab:ap;if(ar<<24>>24!=9){A=15257;break}ar=d[248832+aL|0]|aI<<4;if(ar>>>0>=134217728){aO=ar;break}ab=am+(aN>>>3)|0;ag=(cV(d[ab]|d[ab+1|0]<<8|d[ab+2|0]<<16|d[ab+3|0]<<24|0)|0)<<(aN&7);if(X){aI=ar;aJ=aN;aK=ag}else{aO=ar;break}}if((A|0)==15257){aO=d[248832+aL|0]|aI<<(aM-1>>1)}c[l>>2]=aN;au=aG;av=aO-1|0;aw=aq}}while(0);aO=a[aw]|0;aw=aO&255;do{if(aO<<24>>24==0){aP=au;aQ=av}else{if((aO&255)<9){aP=c[245552+(aw<<3)>>2]|0;aQ=c[245556+(aw<<3)>>2]|0;break}else{aG=aw-9|0;aP=c[262656+(aG<<3)>>2]|0;aQ=c[262660+(aG<<3)>>2]|0;break}}}while(0);BX(e+104|0,e+108|0,aQ,(aQ|0)<0|0?-1:0,aP,(aP|0)<0|0?-1:0,1073741824,0)|0;aP=c[l>>2]|0;aQ=c[p>>2]|0;aw=d[aQ+(aP>>>3)|0]|0;aO=c[n>>2]|0;av=((aP|0)<(aO|0))+aP|0;c[l>>2]=av;do{if((aw&128>>>((aP&7)>>>0)|0)==0){aR=g+12|0}else{au=aQ+(av>>>3)|0;aG=(cV(d[au]|d[au+1|0]<<8|d[au+2|0]<<16|d[au+3|0]<<24|0)|0)<<(av&7);if((aG&-1434451968|0)==0){au=1;aN=av;aM=aG;while(1){aS=aM>>>24;aI=a[248576+aS|0]|0;aT=aI&255;aL=((aI&255)>8?8:aT)+aN|0;aK=aO>>>0>aL>>>0;aU=aK?aL:aO;if(aI<<24>>24!=9){A=15270;break}aI=d[248832+aS|0]|au<<4;if(aI>>>0>=134217728){aV=aI;break}aL=aQ+(aU>>>3)|0;aJ=(cV(d[aL]|d[aL+1|0]<<8|d[aL+2|0]<<16|d[aL+3|0]<<24|0)|0)<<(aU&7);if(aK){au=aI;aN=aU;aM=aJ}else{aV=aI;break}}if((A|0)==15270){aV=d[248832+aS|0]|au<<(aT-1>>1)}c[l>>2]=aU;aW=aV-1|0}else{aM=aG>>>24;aN=(d[248576+aM|0]|0)+av|0;c[l>>2]=aO>>>0>aN>>>0?aN:aO;aW=d[248320+aM|0]|0}aM=aW&255;aN=g+12|0;a[aN]=aM;if((aW&255)>>>0>6){W=-1094995529;i=h;return W|0}if(aM<<24>>24!=0){aR=aN;break}aM=c[l>>2]|0;aq=c[n>>2]|0;aI=c[p>>2]|0;aJ=aI+(aM>>>3)|0;aK=(cV(d[aJ]|d[aJ+1|0]<<8|d[aJ+2|0]<<16|d[aJ+3|0]<<24|0)|0)<<(aM&7);if((aK&-1434451968|0)==0){aJ=1;aL=aM;aH=aK;while(1){aX=aH>>>24;aE=a[248576+aX|0]|0;aY=aE&255;aF=((aE&255)>8?8:aY)+aL|0;aD=aq>>>0>aF>>>0;aZ=aD?aF:aq;if(aE<<24>>24!=9){A=15279;break}aE=d[248832+aX|0]|aJ<<4;if(aE>>>0>=134217728){a_=aE;break}aF=aI+(aZ>>>3)|0;aC=(cV(d[aF]|d[aF+1|0]<<8|d[aF+2|0]<<16|d[aF+3|0]<<24|0)|0)<<(aZ&7);if(aD){aJ=aE;aL=aZ;aH=aC}else{a_=aE;break}}if((A|0)==15279){a_=d[248832+aX|0]|aJ<<(aY-1>>1)}c[l>>2]=aZ;a$=a_-1|0}else{aH=aK>>>24;aL=(d[248576+aH|0]|0)+aM|0;c[l>>2]=aq>>>0>aL>>>0?aL:aq;a$=d[248320+aH|0]|0}c[e+224>>2]=a$;aH=c[l>>2]|0;aL=c[n>>2]|0;aI=c[p>>2]|0;aG=aI+(aH>>>3)|0;au=(cV(d[aG]|d[aG+1|0]<<8|d[aG+2|0]<<16|d[aG+3|0]<<24|0)|0)<<(aH&7);if((au&-1434451968|0)==0){aG=1;aE=aH;aC=au;while(1){a0=aC>>>24;aD=a[248576+a0|0]|0;a1=aD&255;aF=((aD&255)>8?8:a1)+aE|0;aB=aL>>>0>aF>>>0;a2=aB?aF:aL;if(aD<<24>>24!=9){A=15286;break}aD=d[248832+a0|0]|aG<<4;if(aD>>>0>=134217728){a3=aD;break}aF=aI+(a2>>>3)|0;ah=(cV(d[aF]|d[aF+1|0]<<8|d[aF+2|0]<<16|d[aF+3|0]<<24|0)|0)<<(a2&7);if(aB){aG=aD;aE=a2;aC=ah}else{a3=aD;break}}if((A|0)==15286){a3=d[248832+a0|0]|aG<<(a1-1>>1)}c[l>>2]=a2;a4=a3-1|0}else{aC=au>>>24;aE=(d[248576+aC|0]|0)+aH|0;c[l>>2]=aL>>>0>aE>>>0?aE:aL;a4=d[248320+aC|0]|0}c[e+228>>2]=a4;aR=aN}}while(0);a4=a[aR]|0;if(a4<<24>>24!=0){aR=262584+((a4&255)-1<<3)|0;a4=e+224|0;a3=c[aR+4>>2]|0;c[a4>>2]=c[aR>>2];c[a4+4>>2]=a3}a3=c[l>>2]|0;a4=c[p>>2]|0;aR=d[a4+(a3>>>3)|0]|0;a2=c[n>>2]|0;a1=((a3|0)<(a2|0))+a3|0;c[l>>2]=a1;if((aR&128>>>((a3&7)>>>0)|0)==0){a5=a1;a6=a4;a7=a2}else{a3=a4+(a1>>>3)|0;aR=(cV(d[a3]|d[a3+1|0]<<8|d[a3+2|0]<<16|d[a3+3|0]<<24|0)|0)<<(a1&7);if((aR&-1434451968|0)==0){a3=1;a0=a1;a$=aR;while(1){a8=a$>>>24;a_=a[248576+a8|0]|0;a9=a_&255;aZ=((a_&255)>8?8:a9)+a0|0;aY=a2>>>0>aZ>>>0;ba=aY?aZ:a2;if(a_<<24>>24!=9){A=15297;break}a_=d[248832+a8|0]|a3<<4;if(a_>>>0>=134217728){bb=a_;break}aZ=a4+(ba>>>3)|0;aX=(cV(d[aZ]|d[aZ+1|0]<<8|d[aZ+2|0]<<16|d[aZ+3|0]<<24|0)|0)<<(ba&7);if(aY){a3=a_;a0=ba;a$=aX}else{bb=a_;break}}if((A|0)==15297){bb=d[248832+a8|0]|a3<<(a9-1>>1)}c[l>>2]=ba;bc=bb+65535&65535}else{bb=aR>>>24;aR=(d[248576+bb|0]|0)+a1|0;c[l>>2]=a2>>>0>aR>>>0?aR:a2;bc=d[248320+bb|0]|0}b[g+14>>1]=bc;bc=c[l>>2]|0;bb=c[n>>2]|0;a2=c[p>>2]|0;aR=a2+(bc>>>3)|0;a1=(cV(d[aR]|d[aR+1|0]<<8|d[aR+2|0]<<16|d[aR+3|0]<<24|0)|0)<<(bc&7);if((a1&-1434451968|0)==0){aR=1;ba=bc;a9=a1;while(1){bd=a9>>>24;a3=a[248576+bd|0]|0;be=a3&255;a8=((a3&255)>8?8:be)+ba|0;a$=bb>>>0>a8>>>0;bf=a$?a8:bb;if(a3<<24>>24!=9){A=15304;break}a3=d[248832+bd|0]|aR<<4;if(a3>>>0>=134217728){bg=a3;break}a8=a2+(bf>>>3)|0;a0=(cV(d[a8]|d[a8+1|0]<<8|d[a8+2|0]<<16|d[a8+3|0]<<24|0)|0)<<(bf&7);if(a$){aR=a3;ba=bf;a9=a0}else{bg=a3;break}}if((A|0)==15304){bg=d[248832+bd|0]|aR<<(be-1>>1)}c[l>>2]=bf;bh=bg+65535&65535}else{bg=a1>>>24;a1=(d[248576+bg|0]|0)+bc|0;c[l>>2]=bb>>>0>a1>>>0?a1:bb;bh=d[248320+bg|0]|0}b[g+16>>1]=bh;bh=c[l>>2]|0;bg=c[n>>2]|0;bb=c[p>>2]|0;a1=bb+(bh>>>3)|0;bc=(cV(d[a1]|d[a1+1|0]<<8|d[a1+2|0]<<16|d[a1+3|0]<<24|0)|0)<<(bh&7);if((bc&-1434451968|0)==0){a1=1;bf=bh;be=bc;while(1){bi=be>>>24;aR=a[248576+bi|0]|0;bj=aR&255;bd=((aR&255)>8?8:bj)+bf|0;a9=bg>>>0>bd>>>0;bk=a9?bd:bg;if(aR<<24>>24!=9){A=15311;break}aR=d[248832+bi|0]|a1<<4;if(aR>>>0>=134217728){bl=aR;break}bd=bb+(bk>>>3)|0;ba=(cV(d[bd]|d[bd+1|0]<<8|d[bd+2|0]<<16|d[bd+3|0]<<24|0)|0)<<(bk&7);if(a9){a1=aR;bf=bk;be=ba}else{bl=aR;break}}if((A|0)==15311){bl=d[248832+bi|0]|a1<<(bj-1>>1)}c[l>>2]=bk;bm=bl+65535&65535}else{bl=bc>>>24;bc=(d[248576+bl|0]|0)+bh|0;c[l>>2]=bg>>>0>bc>>>0?bc:bg;bm=d[248320+bl|0]|0}b[g+18>>1]=bm;bm=c[l>>2]|0;bl=c[n>>2]|0;bg=c[p>>2]|0;bc=bg+(bm>>>3)|0;bh=(cV(d[bc]|d[bc+1|0]<<8|d[bc+2|0]<<16|d[bc+3|0]<<24|0)|0)<<(bm&7);if((bh&-1434451968|0)==0){bc=1;bk=bm;bj=bh;while(1){bn=bj>>>24;a1=a[248576+bn|0]|0;bo=a1&255;bi=((a1&255)>8?8:bo)+bk|0;be=bl>>>0>bi>>>0;bp=be?bi:bl;if(a1<<24>>24!=9){A=15318;break}a1=d[248832+bn|0]|bc<<4;if(a1>>>0>=134217728){bq=a1;break}bi=bg+(bp>>>3)|0;bf=(cV(d[bi]|d[bi+1|0]<<8|d[bi+2|0]<<16|d[bi+3|0]<<24|0)|0)<<(bp&7);if(be){bc=a1;bk=bp;bj=bf}else{bq=a1;break}}if((A|0)==15318){bq=d[248832+bn|0]|bc<<(bo-1>>1)}c[l>>2]=bp;br=bq+65535&65535}else{bq=bh>>>24;bh=(d[248576+bq|0]|0)+bm|0;c[l>>2]=bl>>>0>bh>>>0?bh:bl;br=d[248320+bq|0]|0}b[g+20>>1]=br;a5=c[l>>2]|0;a6=c[p>>2]|0;a7=c[n>>2]|0}br=d[a6+(a5>>>3)|0]|0;bq=((a5|0)<(a7|0))+a5|0;c[l>>2]=bq;do{if((br&128>>>((a5&7)>>>0)|0)==0){bs=16;bt=8;bu=g+22|0}else{bl=a6+(bq>>>3)|0;bh=(cV(d[bl]|d[bl+1|0]<<8|d[bl+2|0]<<16|d[bl+3|0]<<24|0)|0)<<(bq&7);if((bh&-1434451968|0)==0){bl=1;bm=bq;bp=bh;while(1){bv=bp>>>24;bo=a[248576+bv|0]|0;bw=bo&255;bc=((bo&255)>8?8:bw)+bm|0;bn=a7>>>0>bc>>>0;bx=bn?bc:a7;if(bo<<24>>24!=9){A=15328;break}bo=d[248832+bv|0]|bl<<4;if(bo>>>0>=134217728){by=bo;break}bc=a6+(bx>>>3)|0;bj=(cV(d[bc]|d[bc+1|0]<<8|d[bc+2|0]<<16|d[bc+3|0]<<24|0)|0)<<(bx&7);if(bn){bl=bo;bm=bx;bp=bj}else{by=bo;break}}if((A|0)==15328){by=d[248832+bv|0]|bl<<(bw-1>>1)}c[l>>2]=bx;bz=by-1|0}else{bp=bh>>>24;bm=(d[248576+bp|0]|0)+bq|0;c[l>>2]=a7>>>0>bm>>>0?bm:a7;bz=d[248320+bp|0]|0}bp=bz&255;bm=g+22|0;a[bm]=bp;if((bz&255)>>>0>4){W=-1094995529;i=h;return W|0}if(bp<<24>>24!=0){bs=16;bt=8;bu=bm;break}bp=c[l>>2]|0;aN=c[n>>2]|0;aL=c[p>>2]|0;aH=aL+(bp>>>3)|0;au=(cV(d[aH]|d[aH+1|0]<<8|d[aH+2|0]<<16|d[aH+3|0]<<24|0)|0)<<(bp&7);if((au&-1434451968|0)==0){aH=1;aG=bp;bo=au;while(1){bA=bo>>>24;bj=a[248576+bA|0]|0;bB=bj&255;bn=((bj&255)>8?8:bB)+aG|0;bc=aN>>>0>bn>>>0;bC=bc?bn:aN;if(bj<<24>>24!=9){A=15337;break}bj=d[248832+bA|0]|aH<<4;if(bj>>>0>=134217728){bD=bj;break}bn=aL+(bC>>>3)|0;bk=(cV(d[bn]|d[bn+1|0]<<8|d[bn+2|0]<<16|d[bn+3|0]<<24|0)|0)<<(bC&7);if(bc){aH=bj;aG=bC;bo=bk}else{bD=bj;break}}if((A|0)==15337){bD=d[248832+bA|0]|aH<<(bB-1>>1)}c[l>>2]=bC;bE=bD-1|0;bF=bC}else{bo=au>>>24;aG=(d[248576+bo|0]|0)+bp|0;bh=aN>>>0>aG>>>0?aG:aN;c[l>>2]=bh;bE=d[248320+bo|0]|0;bF=bh}bh=aL+(bF>>>3)|0;bo=(cV(d[bh]|d[bh+1|0]<<8|d[bh+2|0]<<16|d[bh+3|0]<<24|0)|0)<<(bF&7);if((bo&-1434451968|0)==0){bh=1;aG=bF;bl=bo;while(1){bG=bl>>>24;bj=a[248576+bG|0]|0;bH=bj&255;bk=((bj&255)>8?8:bH)+aG|0;bc=aN>>>0>bk>>>0;bI=bc?bk:aN;if(bj<<24>>24!=9){A=15344;break}bj=d[248832+bG|0]|bh<<4;if(bj>>>0>=134217728){bJ=bj;break}bk=aL+(bI>>>3)|0;bn=(cV(d[bk]|d[bk+1|0]<<8|d[bk+2|0]<<16|d[bk+3|0]<<24|0)|0)<<(bI&7);if(bc){bh=bj;aG=bI;bl=bn}else{bJ=bj;break}}if((A|0)==15344){bJ=d[248832+bG|0]|bh<<(bH-1>>1)}c[l>>2]=bI;bK=bJ-1|0;bL=bI}else{bl=bo>>>24;aG=(d[248576+bl|0]|0)+bF|0;bp=aN>>>0>aG>>>0?aG:aN;c[l>>2]=bp;bK=d[248320+bl|0]|0;bL=bp}bp=bK>>>0>65535;bl=bp?16:0;aG=bp?bK>>>16:bK;if((aG&65280|0)==0){bM=aG;bN=bl}else{bM=aG>>>8;bN=bl|8}bl=bN+1+(d[247648+bM|0]|0)|0;aG=aL+(bL>>>3)|0;bp=(cV(d[aG]|d[aG+1|0]<<8|d[aG+2|0]<<16|d[aG+3|0]<<24|0)|0)<<(bL&7);L20653:do{if((bp&-1434451968|0)==0){aG=1;au=bL;aH=bp;while(1){bj=aH>>>24;bn=a[248576+bj|0]|0;bc=((bn&255)>8?8:bn&255)+au|0;bk=aN>>>0>bc>>>0;bg=bk?bc:aN;if(bn<<24>>24!=9){bO=bg;break L20653}bn=d[248832+bj|0]|aG<<4;if(bn>>>0>=134217728){bO=bg;break L20653}bj=aL+(bg>>>3)|0;bc=(cV(d[bj]|d[bj+1|0]<<8|d[bj+2|0]<<16|d[bj+3|0]<<24|0)|0)<<(bg&7);if(bk){aG=bn;au=bg;aH=bc}else{bO=bg;break}}}else{aH=(d[248576+(bp>>>24)|0]|0)+bL|0;bO=aN>>>0>aH>>>0?aH:aN}}while(0);c[l>>2]=bO;bp=aL+(bO>>>3)|0;bo=(cV(d[bp]|d[bp+1|0]<<8|d[bp+2|0]<<16|d[bp+3|0]<<24|0)|0)<<(bO&7);L20660:do{if((bo&-1434451968|0)==0){bp=1;bh=bO;aH=bo;while(1){au=aH>>>24;aG=a[248576+au|0]|0;bg=((aG&255)>8?8:aG&255)+bh|0;bc=aN>>>0>bg>>>0;bn=bc?bg:aN;if(aG<<24>>24!=9){bP=bn;break L20660}aG=d[248832+au|0]|bp<<4;if(aG>>>0>=134217728){bP=bn;break L20660}au=aL+(bn>>>3)|0;bg=(cV(d[au]|d[au+1|0]<<8|d[au+2|0]<<16|d[au+3|0]<<24|0)|0)<<(bn&7);if(bc){bp=aG;bh=bn;aH=bg}else{bP=bn;break}}}else{aH=(d[248576+(bo>>>24)|0]|0)+bO|0;bP=aN>>>0>aH>>>0?aH:aN}}while(0);c[l>>2]=bP;c[e+396>>2]=(bE|0)!=0?1:2;bs=bE;bt=bl;bu=bm}}while(0);bE=a[bu]|0;if(bE<<24>>24==0){bQ=bt}else{bt=(bE&255)-1|0;bE=d[91584+(bt<<3)|0]|0;c[e+396>>2]=c[91588+(bt<<3)>>2];bQ=bE}if(bQ>>>0>8){AN(e,24,1659088,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V}bQ=c[262632+(((bs|0)==0|0)*12|0)+(d[an]<<2)>>2]|0;c[e+140>>2]=bQ;nZ(bQ,j,k);bQ=g|0;do{if(((1<<c[j>>2])-1&c[bQ>>2]|0)==0){an=g+4|0;if(((1<<c[k>>2])-1&c[an>>2]|0)!=0){break}bs=c[l>>2]|0;bE=c[p>>2]|0;bt=d[bE+(bs>>>3)|0]|0;bu=c[n>>2]|0;bP=((bs|0)<(bu|0))+bs|0;c[l>>2]=bP;do{if((bt&128>>>((bs&7)>>>0)|0)==0){bO=d[g+23|0]|0;c[e+384>>2]=c[262672+(bO*12|0)>>2];c[e+392>>2]=c[262676+(bO*12|0)>>2];c[e+388>>2]=c[262680+(bO*12|0)>>2]}else{bO=bE+(bP>>>3)|0;bL=(cV(d[bO]|d[bO+1|0]<<8|d[bO+2|0]<<16|d[bO+3|0]<<24|0)|0)<<(bP&7);if((bL&-1434451968|0)==0){bO=1;bM=bP;bN=bL;while(1){bR=bN>>>24;bK=a[248576+bR|0]|0;bS=bK&255;bF=((bK&255)>8?8:bS)+bM|0;bI=bu>>>0>bF>>>0;bT=bI?bF:bu;if(bK<<24>>24!=9){A=15372;break}bK=d[248832+bR|0]|bO<<4;if(bK>>>0>=134217728){bU=bK;break}bF=bE+(bT>>>3)|0;bJ=(cV(d[bF]|d[bF+1|0]<<8|d[bF+2|0]<<16|d[bF+3|0]<<24|0)|0)<<(bT&7);if(bI){bO=bK;bM=bT;bN=bJ}else{bU=bK;break}}if((A|0)==15372){bU=d[248832+bR|0]|bO<<(bS-1>>1)}c[l>>2]=bT;bV=bU-1|0}else{bN=bL>>>24;bM=(d[248576+bN|0]|0)+bP|0;c[l>>2]=bu>>>0>bM>>>0?bM:bu;bV=d[248320+bN|0]|0}bN=g+23|0;a[bN]=bV&255;bM=bV&255;if(bM>>>0>4){W=-1094995529;i=h;return W|0}bK=e+384|0;c[bK>>2]=c[262672+(bM*12|0)>>2];bJ=e+392|0;c[bJ>>2]=c[262676+(bM*12|0)>>2];bI=e+388|0;c[bI>>2]=c[262680+(bM*12|0)>>2];if((a[bN]|0)!=0){break}bN=c[l>>2]|0;bM=c[p>>2]|0;bF=d[bM+(bN>>>3)|0]|0;bH=c[n>>2]|0;bG=((bN|0)<(bH|0))+bN|0;c[l>>2]=bG;do{if((bF&128>>>((bN&7)>>>0)|0)==0){bW=bG;bX=bM;bY=bH}else{bC=bM+(bG>>>3)|0;bD=(cV(d[bC]|d[bC+1|0]<<8|d[bC+2|0]<<16|d[bC+3|0]<<24|0)|0)<<(bG&7);if((bD&-1434451968|0)==0){bC=1;bB=bG;bA=bD;while(1){bZ=bA>>>24;bz=a[248576+bZ|0]|0;b_=bz&255;a7=((bz&255)>8?8:b_)+bB|0;bq=bH>>>0>a7>>>0;b$=bq?a7:bH;if(bz<<24>>24!=9){A=15382;break}bz=d[248832+bZ|0]|bC<<4;if(bz>>>0>=134217728){b0=bz;break}a7=bM+(b$>>>3)|0;by=(cV(d[a7]|d[a7+1|0]<<8|d[a7+2|0]<<16|d[a7+3|0]<<24|0)|0)<<(b$&7);if(bq){bC=bz;bB=b$;bA=by}else{b0=bz;break}}if((A|0)==15382){b0=d[248832+bZ|0]|bC<<(b_-1>>1)}c[l>>2]=b$;b1=b0-1|0;b2=b$}else{bA=bD>>>24;bB=(d[248576+bA|0]|0)+bG|0;bz=bH>>>0>bB>>>0?bB:bH;c[l>>2]=bz;b1=d[248320+bA|0]|0;b2=bz}if(b1>>>0>=3){bW=b2;bX=bM;bY=bH;break}c[bK>>2]=c[262568+(b1<<2)>>2];bW=c[l>>2]|0;bX=c[p>>2]|0;bY=c[n>>2]|0}}while(0);bK=d[bX+(bW>>>3)|0]|0;bH=((bW|0)<(bY|0))+bW|0;c[l>>2]=bH;do{if((bK&128>>>((bW&7)>>>0)|0)!=0){bM=bX+(bH>>>3)|0;bG=(cV(d[bM]|d[bM+1|0]<<8|d[bM+2|0]<<16|d[bM+3|0]<<24|0)|0)<<(bH&7);if((bG&-1434451968|0)==0){bM=1;bN=bH;bF=bG;while(1){b3=bF>>>24;bL=a[248576+b3|0]|0;b4=bL&255;bO=((bL&255)>8?8:b4)+bN|0;bz=bY>>>0>bO>>>0;b5=bz?bO:bY;if(bL<<24>>24!=9){A=15392;break}bL=d[248832+b3|0]|bM<<4;if(bL>>>0>=134217728){b6=bL;break}bO=bX+(b5>>>3)|0;bA=(cV(d[bO]|d[bO+1|0]<<8|d[bO+2|0]<<16|d[bO+3|0]<<24|0)|0)<<(b5&7);if(bz){bM=bL;bN=b5;bF=bA}else{b6=bL;break}}if((A|0)==15392){b6=d[248832+b3|0]|bM<<(b4-1>>1)}c[l>>2]=b5;b7=b6-1|0}else{bF=bG>>>24;bN=(d[248576+bF|0]|0)+bH|0;c[l>>2]=bY>>>0>bN>>>0?bN:bY;b7=d[248320+bF|0]|0}if((b7|0)==0){c[bJ>>2]=1;break}else if((b7|0)==1){c[bJ>>2]=5;break}else{break}}}while(0);bJ=c[l>>2]|0;bH=c[p>>2]|0;bK=d[bH+(bJ>>>3)|0]|0;bF=c[n>>2]|0;bN=((bJ|0)<(bF|0))+bJ|0;c[l>>2]=bN;if((bK&128>>>((bJ&7)>>>0)|0)==0){break}bJ=bH+(bN>>>3)|0;bK=(cV(d[bJ]|d[bJ+1|0]<<8|d[bJ+2|0]<<16|d[bJ+3|0]<<24|0)|0)<<(bN&7);if((bK&-1434451968|0)==0){bJ=1;bD=bN;bC=bK;while(1){b8=bC>>>24;bL=a[248576+b8|0]|0;b9=bL&255;bA=((bL&255)>8?8:b9)+bD|0;bz=bF>>>0>bA>>>0;ca=bz?bA:bF;if(bL<<24>>24!=9){A=15403;break}bL=d[248832+b8|0]|bJ<<4;if(bL>>>0>=134217728){cb=bL;break}bA=bH+(ca>>>3)|0;bO=(cV(d[bA]|d[bA+1|0]<<8|d[bA+2|0]<<16|d[bA+3|0]<<24|0)|0)<<(ca&7);if(bz){bJ=bL;bD=ca;bC=bO}else{cb=bL;break}}if((A|0)==15403){cb=d[248832+b8|0]|bJ<<(b9-1>>1)}c[l>>2]=ca;cc=cb-1|0}else{bC=bK>>>24;bD=(d[248576+bC|0]|0)+bN|0;c[l>>2]=bF>>>0>bD>>>0?bD:bF;cc=d[248320+bC|0]|0}if((cc|0)!=0){break}c[bI>>2]=1}}while(0);bu=e;bP=AF(c[bQ>>2]|0,c[an>>2]|0,0,bu)|0;if((bP|0)!=0){W=bP;i=h;return W|0}oU(e,c[bQ>>2]|0,c[an>>2]|0);bP=c[l>>2]|0;bE=c[n>>2]|0;bs=c[p>>2]|0;bt=bs+(bP>>>3)|0;bm=(cV(d[bt]|d[bt+1|0]<<8|d[bt+2|0]<<16|d[bt+3|0]<<24|0)|0)<<(bP&7);if((bm&-1434451968|0)==0){bt=1;bl=bP;bC=bm;while(1){cd=bC>>>24;bD=a[248576+cd|0]|0;ce=bD&255;bH=((bD&255)>8?8:ce)+bl|0;bL=bE>>>0>bH>>>0;cf=bL?bH:bE;if(bD<<24>>24!=9){A=15414;break}bD=d[248832+cd|0]|bt<<4;if(bD>>>0>=134217728){cg=bD;break}bH=bs+(cf>>>3)|0;bO=(cV(d[bH]|d[bH+1|0]<<8|d[bH+2|0]<<16|d[bH+3|0]<<24|0)|0)<<(cf&7);if(bL){bt=bD;bl=cf;bC=bO}else{cg=bD;break}}if((A|0)==15414){cg=d[248832+cd|0]|bt<<(ce-1>>1)}c[l>>2]=cf;ch=cg-1|0}else{bC=bm>>>24;bl=(d[248576+bC|0]|0)+bP|0;c[l>>2]=bE>>>0>bl>>>0?bl:bE;ch=d[248320+bC|0]|0}if((ch|0)==0){W=0;i=h;return W|0}AN(bu,16,1696056,(V=i,i=i+8|0,c[V>>2]=ch,V)|0);i=V;W=-1094995529;i=h;return W|0}}while(0);AN(e,16,1640584,(V=i,i=i+1|0,i=i+7&-8,c[V>>2]=0,V)|0);i=V;W=-1094995529;i=h;return W|0}function nX(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;f=i;g=a+96|0;h=c[g>>2]|0;do{if((h|0)!=0){j=c[a+100>>2]|0;if((j|0)<34){break}k=h;do{if((d[k]|d[k+1|0]<<8|d[k+2|0]<<16|d[k+3|0]<<24|0)==1130450022){if((j|0)>=42){c[b>>2]=1;l=(c[g>>2]|0)+8|0;break}AN(a,16,1742064,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=0;i=f;return n|0}else{if((j|0)!=34){AN(a,24,1613968,(m=i,i=i+8|0,c[m>>2]=34-j,m)|0);i=m}c[b>>2]=0;l=c[g>>2]|0}}while(0);c[e>>2]=l;n=1;i=f;return n|0}}while(0);AN(a,16,1618464,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=0;i=f;return n|0}function nY(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;f=i;g=(e|0)==0?8:280;h=g>>>0>16?16:g;j=e+(h>>>3)|0;k=cV(d[j]|d[j+1|0]<<8|d[j+2|0]<<16|d[j+3|0]<<24|0)|0;j=k>>>16;l=h+16|0;h=b+12|0;c[h>>2]=j;if(k>>>0<1048576){AN(a,24,1736656,(k=i,i=i+8|0,c[k>>2]=j,k)|0);i=k;c[h>>2]=16}h=(g>>>0>l>>>0?l:g)+24|0;l=g>>>0>h>>>0?h:g;h=e+(l>>>3)|0;k=l+24|0;j=g>>>0>k>>>0?k:g;c[b+16>>2]=(cV(d[h]|d[h+1|0]<<8|d[h+2|0]<<16|d[h+3|0]<<24|0)|0)<<(l&7)>>>8;l=e+(j>>>3)|0;h=j+20|0;k=g>>>0>h>>>0?h:g;h=b|0;c[h>>2]=(cV(d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0)|0)<<(j&7)>>>12;j=e+(k>>>3)|0;l=(cV(d[j]|d[j+1|0]<<8|d[j+2|0]<<16|d[j+3|0]<<24|0)|0)<<(k&7)>>>29;j=k+3|0;k=g>>>0>j>>>0?j:g;j=l+1|0;c[b+4>>2]=j;m=e+(k>>>3)|0;n=k+5|0;o=g>>>0>n>>>0?n:g;n=b+8|0;c[n>>2]=((cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0)<<(k&7)>>>27)+1;c[a+416>>2]=j;c[a+412>>2]=c[h>>2];c[a+796>>2]=c[n>>2];if(j>>>0<9){j=234776+(l<<3)|0;l=c[j+4>>2]|0;n=a+448|0;c[n>>2]=c[j>>2];c[n+4>>2]=l}else{l=a+448|0;c[l>>2]=0;c[l+4>>2]=0}l=e+(o>>>3)|0;a=(cV(d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0)|0)<<(o&7)>>>28;l=o+4|0;o=g>>>0>l>>>0?l:g;l=e+(o>>>3)|0;n=(cV(d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0)|0)<<(o&7)&-65536;l=o+16|0;o=g>>>0>l>>>0?l:g;g=e+(o>>>3)|0;e=b+24|0;c[e>>2]=(cV(d[g]|d[g+1|0]<<8|d[g+2|0]<<16|d[g+3|0]<<24|0)|0)<<(o&7)>>>16|n;c[e+4>>2]=a;i=f;return}function nZ(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0;f=i;g=BP(a)|0;if((g|0)==0){AN(0,0,1667296,(a=i,i=i+24|0,c[a>>2]=1725256,c[a+8>>2]=1696024,c[a+16>>2]=68,a)|0);i=a;cw()}else{c[b>>2]=d[g+5|0]|0;c[e>>2]=d[g+6|0]|0;i=f;return}}function n_(d,f,g,h){d=d|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;i=BP(f)|0;j=BP(d)|0;if(d>>>0>320){k=-1;return k|0}c[g>>2]=0;if((d|0)==(f|0)){k=2147483647;return k|0}l=BP(f)|0;if((l|0)==0){k=-22;return k|0}if((a[l+4|0]|0)==0){k=-22;return k|0}l=BP(d)|0;if((l|0)==0){k=-22;return k|0}if((a[l+4|0]|0)==0){k=-22;return k|0}l=a[i+4|0]|0;do{if((l-1&255)<2){m=1}else{n=c[i>>2]|0;if((n|0)!=0){if((ba(n|0,1674296,4)|0)==0){m=3;break}}if((a[i+7|0]&32)!=0){m=0;break}m=l<<24>>24==0?-1:2}}while(0);n=a[j+4|0]|0;do{if((n-1&255)<2){o=1}else{p=c[j>>2]|0;if((p|0)!=0){if((ba(p|0,1674296,4)|0)==0){o=3;break}}if((a[j+7|0]&32)!=0){o=0;break}o=n<<24>>24==0?-1:2}}while(0);p=(l&255)>(n&255)?n:l;q=p&255;do{if(p<<24>>24==0){r=0;s=2147483646}else{if((h&2|0)==0){r=0;s=2147483646;break}else{t=0;u=0;v=2147483646}while(1){w=(e[j+8+(u<<1)>>1]|0)>>>11&15;if(((e[i+8+(u<<1)>>1]|0)>>>11&15)>(w&65535)){x=v-(65536>>>((w&65535)>>>0))|0;y=t|2}else{x=v;y=t}w=u+1|0;if((w|0)<(q|0)){t=y;u=w;v=x}else{r=y;s=x;break}}}}while(0);do{if((h&1|0)==0){z=s;A=r}else{x=a[j+5|0]|0;y=a[i+5|0]|0;if((x&255)>(y&255)){B=s-(256<<(x&255))|0;C=r|1}else{B=s;C=r}v=a[j+6|0]|0;u=a[i+6|0]|0;if((v&255)>(u&255)){D=B-(256<<(v&255))|0;E=C|1}else{D=B;E=C}if(!(x<<24>>24==1&y<<24>>24==0&v<<24>>24==1)){z=D;A=E;break}z=u<<24>>24==0?D+512|0:D;A=E}}while(0);do{if((h&4|0)==0){F=A}else{if((o|0)==0){F=m>>>0>1?A|4:A;break}else if((o|0)==2){F=(m|0)==2?A:A|4;break}else if((o|0)==3){F=(m-2|0)>>>0>1&(m|0)!=1?A|4:A;break}else if((o|0)==1){F=(m|0)==1?A:A|4;break}else{F=(m|0)==(o|0)?A:A|4;break}}}while(0);if((F&4|0)==0){G=z}else{A=b[j+8>>1]|0;E=b[i+8>>1]|0;G=z-(q<<16>>>((((((A&65535)>>>11&15)>((E&65535)>>>11&15)?E:A)&65535)>>>11&15)>>>0))|0}A=(m|0)==1;do{if((o|0)!=1|A){H=G;I=F}else{if((h&32|0)==0){H=G;I=F;break}H=G-131072|0;I=F|32}}while(0);do{if((n<<24>>24|0)==2|(n<<24>>24|0)==4){J=H;K=I}else{if((a[j+7|0]&2)!=0){J=H;K=I;break}if(!((l<<24>>24|0)==2|(l<<24>>24|0)==4)){if((a[i+7|0]&2)==0){J=H;K=I;break}}if((h&8|0)==0){J=H;K=I;break}J=H-65536|0;K=I|8}}while(0);do{if((d|0)==11){if((h&16|0)==0|(f|0)==11){L=J;M=K;break}if(A){if(!((l<<24>>24|0)==2|(l<<24>>24|0)==4)){if((a[i+7|0]&2)==0){L=J;M=K;break}}if((h&8|0)==0){L=J;M=K;break}}L=J-65536|0;M=K|16}else{L=J;M=K}}while(0);c[g>>2]=M;k=L;return k|0}function n$(a,b,e,f,g){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;h=i;i=i+24|0;j=h|0;k=BP(a)|0;l=BP(b)|0;m=(g|0)!=0;if(m){n=~c[g>>2]}else{n=-1}o=(f|0)==0?n&-9:n;n=n_(a,e,h+8|0,o)|0;p=n_(b,e,h+16|0,o)|0;do{if((n|0)==(p|0)){o=BO(l)|0;if((o|0)==(BO(k)|0)){q=(d[l+4|0]|0)<(d[k+4|0]|0)?b:a;break}else{o=BO(l)|0;q=(o|0)<(BO(k)|0)?b:a;break}}else{q=(n|0)<(p|0)?b:a}}while(0);if(!m){i=h;return q|0}m=n_(q,e,j,(f|0)!=0?-1:-9)|0;c[g>>2]=(m|0)<0?m:c[j>>2]|0;i=h;return q|0}function n0(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;b=i;i=i+96|0;d=b|0;e=a+60|0;f=c[e>>2]|0;NM(d|0,0,92);g=f+4|0;h=a+416|0;j=Hd(g,0,c[h>>2]|0)|0;L20888:do{if((j|0)==0){k=a+840|0;l=c[k>>2]|0;m=(l|0)==-99?2:l+1|0;l=Hh(c[g>>2]|0,256,m)|0;if((l|0)!=0){switch(l|0){case 68:{n=1731664;break};case 35:{n=1624352;break};case 33:{n=1640536;break};case 67:{n=1736616;break};case 128:{n=1722920;break};case 96:{n=1727e3;break};case 65:{n=1613928;break};case 34:{n=1630792;break};case 66:{n=1742024;break};case 64:{n=1618440;break};case 32:{n=1659064;break};default:{n=1719008}}AN(a,16,1711976,(o=i,i=i+16|0,c[o>>2]=m,c[o+8>>2]=n,o)|0);i=o;p=-22;break}do{if((m|0)==39){if((c[f+12>>2]|0)==0){break}l=Hh(c[g>>2]|0,260,1)|0;if((l|0)==0){break}switch(l|0){case 32:{q=1659064;break};case 67:{q=1736616;break};case 68:{q=1731664;break};case 35:{q=1624352;break};case 64:{q=1618440;break};case 96:{q=1727e3;break};case 128:{q=1722920;break};case 33:{q=1640536;break};case 34:{q=1630792;break};case 65:{q=1613928;break};case 66:{q=1742024;break};default:{q=1719008}}AN(a,16,1708504,(o=i,i=i+8|0,c[o>>2]=q,o)|0);i=o;p=-22;break L20888}}while(0);m=a+412|0;l=Hh(c[g>>2]|0,259,c[m>>2]|0)|0;if((l|0)!=0){switch(l|0){case 65:{r=1613928;break};case 66:{r=1742024;break};case 67:{r=1736616;break};case 68:{r=1731664;break};case 96:{r=1727e3;break};case 128:{r=1722920;break};case 33:{r=1640536;break};case 34:{r=1630792;break};case 32:{r=1659064;break};case 35:{r=1624352;break};case 64:{r=1618440;break};default:{r=1719008}}AN(a,16,1706424,(o=i,i=i+16|0,c[o>>2]=c[m>>2],c[o+8>>2]=r,o)|0);i=o;p=-22;break}l=c[h>>2]|0;switch(l|0){case 2:{s=128;t=0;break};case 3:{s=128;t=96;break};case 4:{s=128;t=192;break};case 5:{s=256;t=96;break};case 6:{s=256;t=192;break};case 1:{s=0;t=96;break};default:{AN(a,16,1704432,(o=i,i=i+8|0,c[o>>2]=l,o)|0);i=o;p=-22;break L20888}}u=Hh(c[g>>2]|0,262,l)|0;if((u|0)!=0){switch(u|0){case 68:{v=1731664;break};case 96:{v=1727e3;break};case 128:{v=1722920;break};case 34:{v=1630792;break};case 35:{v=1624352;break};case 64:{v=1618440;break};case 32:{v=1659064;break};case 33:{v=1640536;break};case 65:{v=1613928;break};case 66:{v=1742024;break};case 67:{v=1736616;break};default:{v=1719008}}AN(a,16,1702232,(o=i,i=i+16|0,c[o>>2]=l,c[o+8>>2]=v,o)|0);i=o;p=-22;break}u=Hh(c[g>>2]|0,263,1)|0;if((u|0)!=0){switch(u|0){case 33:{w=1640536;break};case 34:{w=1630792;break};case 35:{w=1624352;break};case 64:{w=1618440;break};case 65:{w=1613928;break};case 66:{w=1742024;break};case 67:{w=1736616;break};case 68:{w=1731664;break};case 96:{w=1727e3;break};case 128:{w=1722920;break};case 32:{w=1659064;break};default:{w=1719008}}AN(a,16,1698936,(o=i,i=i+16|0,c[o>>2]=l,c[o+8>>2]=w,o)|0);i=o;p=-22;break}l=a+88|0;u=c[f+28>>2]|0;x=(u|0)==0;do{if((c[l>>2]&2|0)==0){if(!x){y=u;z=15627;break}A=a+72|0;B=c[A>>2]|0;do{if((B|0)<1){C=c[k>>2]|0;D=(C|0)==28;E=ac((D?96:t)+(D?0:s)|0,c[m>>2]|0)|0;D=(E|0)/44|0;c[A>>2]=D;if(!((C|0)==4|(C|0)==28|(C|0)==131)){if((c[f+12>>2]|0)==0){F=D;break}}D=(E|0)/88|0;c[A>>2]=D;F=D}else{F=B}}while(0);B=Hh(c[g>>2]|0,257,F)|0;if((B|0)==0){break}switch(B|0){case 128:{G=1722920;break};case 32:{G=1659064;break};case 33:{G=1640536;break};case 34:{G=1630792;break};case 35:{G=1624352;break};case 64:{G=1618440;break};case 65:{G=1613928;break};case 66:{G=1742024;break};case 67:{G=1736616;break};case 68:{G=1731664;break};case 96:{G=1727e3;break};default:{G=1719008}}AN(a,16,1688968,(o=i,i=i+16|0,c[o>>2]=c[A>>2],c[o+8>>2]=G,o)|0);i=o;p=-22;break L20888}else{if(!x){y=u;z=15627;break}y=c[a+80>>2]|0;z=15627}}while(0);L21005:do{if((z|0)==15627){u=(y|0)>5;do{if((y-1|0)>>>0>4){x=a;AN(x,24,1696904,(o=i,i=i+8|0,c[o>>2]=y,o)|0);i=o;if((y|0)<1){H=1;I=x;break}H=u?5:y;I=x}else{H=y;I=a}}while(0);AN(I,24,1694648,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;switch(Hh(c[g>>2]|0,258,H)|0){case 32:{J=1659064;break};case 0:{break L21005;break};case 33:{J=1640536;break};case 34:{J=1630792;break};case 35:{J=1624352;break};case 64:{J=1618440;break};case 65:{J=1613928;break};case 66:{J=1742024;break};case 67:{J=1736616;break};case 68:{J=1731664;break};case 96:{J=1727e3;break};case 128:{J=1722920;break};default:{J=1719008}}AN(I,16,1691096,(o=i,i=i+16|0,c[o>>2]=H,c[o+8>>2]=J,o)|0);i=o;p=-22;break L20888}}while(0);if((c[l>>2]&4194304|0)==0){K=(c[f+20>>2]|0)!=0?10:2}else{K=0}u=Hh(c[g>>2]|0,768,K)|0;if((u|0)!=0){switch(u|0){case 33:{L=1640536;break};case 34:{L=1630792;break};case 35:{L=1624352;break};case 64:{L=1618440;break};case 65:{L=1613928;break};case 32:{L=1659064;break};case 66:{L=1742024;break};case 67:{L=1736616;break};case 68:{L=1731664;break};case 96:{L=1727e3;break};case 128:{L=1722920;break};default:{L=1719008}}AN(a,16,1686216,(o=i,i=i+8|0,c[o>>2]=L,o)|0);i=o;p=-22;break}do{if((c[f+20>>2]|0)!=0){u=c[f+24>>2]|0;if((u|0)==0){break}A=Hh(c[g>>2]|0,769,u)|0;if((A|0)==0){break}switch(A|0){case 64:{M=1618440;break};case 65:{M=1613928;break};case 66:{M=1742024;break};case 67:{M=1736616;break};case 68:{M=1731664;break};case 96:{M=1727e3;break};case 128:{M=1722920;break};case 32:{M=1659064;break};case 33:{M=1640536;break};case 34:{M=1630792;break};case 35:{M=1624352;break};default:{M=1719008}}AN(a,16,1684608,(o=i,i=i+8|0,c[o>>2]=M,o)|0);i=o;p=-22;break L20888}}while(0);A=f+16|0;u=c[A>>2]|0;if((u|0)<0){x=(c[l>>2]|0)>>>21&2;c[A>>2]=x;N=x}else{N=u}u=Hh(c[g>>2]|0,770,N)|0;if((u|0)!=0){switch(u|0){case 33:{O=1640536;break};case 34:{O=1630792;break};case 35:{O=1624352;break};case 64:{O=1618440;break};case 65:{O=1613928;break};case 66:{O=1742024;break};case 67:{O=1736616;break};case 68:{O=1731664;break};case 96:{O=1727e3;break};case 128:{O=1722920;break};case 32:{O=1659064;break};default:{O=1719008}}AN(a,16,1682672,(o=i,i=i+16|0,c[o>>2]=c[A>>2],c[o+8>>2]=O,o)|0);i=o;p=-22;break}A=f+8|0;u=Hh(c[g>>2]|0,512,c[A>>2]|0)|0;if((u|0)!=0){switch(u|0){case 33:{P=1640536;break};case 34:{P=1630792;break};case 35:{P=1624352;break};case 64:{P=1618440;break};case 65:{P=1613928;break};case 66:{P=1742024;break};case 67:{P=1736616;break};case 68:{P=1731664;break};case 96:{P=1727e3;break};case 128:{P=1722920;break};case 32:{P=1659064;break};default:{P=1719008}}AN(a,16,1680696,(o=i,i=i+16|0,c[o>>2]=c[A>>2],c[o+8>>2]=P,o)|0);i=o;p=-22;break}A=a+436|0;u=c[A>>2]|0;do{if((u|0)>0){x=(c[m>>2]|0)+255>>8;if((u|0)<(x|0)|(u|0)>2e4){AN(a,16,1678800,(o=i,i=i+8|0,c[o>>2]=x,o)|0);i=o;p=-22;break L20888}x=Hh(c[g>>2]|0,515,u)|0;if((x|0)==0){break}switch(x|0){case 33:{Q=1640536;break};case 34:{Q=1630792;break};case 35:{Q=1624352;break};case 64:{Q=1618440;break};case 65:{Q=1613928;break};case 66:{Q=1742024;break};case 67:{Q=1736616;break};case 68:{Q=1731664;break};case 96:{Q=1727e3;break};case 128:{Q=1722920;break};case 32:{Q=1659064;break};default:{Q=1719008}}AN(a,16,1676664,(o=i,i=i+16|0,c[o>>2]=c[A>>2],c[o+8>>2]=Q,o)|0);i=o;p=-22;break L20888}}while(0);A=Hg(c[g>>2]|0,0,0,0,0)|0;if((A|0)!=0){switch(A|0){case 33:{R=1640536;break};case 34:{R=1630792;break};case 35:{R=1624352;break};case 64:{R=1618440;break};case 65:{R=1613928;break};case 66:{R=1742024;break};case 67:{R=1736616;break};case 68:{R=1731664;break};case 96:{R=1727e3;break};case 128:{R=1722920;break};case 32:{R=1659064;break};default:{R=1719008}}AN(a,16,1674984,(o=i,i=i+8|0,c[o>>2]=R,o)|0);i=o;S=-22;i=b;return S|0}A=Hi(c[g>>2]|0,d)|0;if((A|0)==0){c[a+424>>2]=c[d+16>>2];c[a+116>>2]=c[d+20>>2];qf(a,f+32|0);if((c[l>>2]&4194304|0)==0){S=0;i=b;return S|0}u=d+88|0;m=c[u>>2]|0;c[a+100>>2]=m;x=A7(m+16|0)|0;c[a+96>>2]=x;if((x|0)==0){p=-12;break}m=d+24|0;k=c[u>>2]|0;NN(x|0,m|0,k)|0;S=0;i=b;return S|0}switch(A|0){case 35:{T=1624352;break};case 64:{T=1618440;break};case 65:{T=1613928;break};case 66:{T=1742024;break};case 67:{T=1736616;break};case 68:{T=1731664;break};case 96:{T=1727e3;break};case 128:{T=1722920;break};case 32:{T=1659064;break};case 33:{T=1640536;break};case 34:{T=1630792;break};default:{T=1719008}}AN(a,16,1673280,(o=i,i=i+8|0,c[o>>2]=T,o)|0);i=o;p=-22}else{switch(j|0){case 68:{U=1731664;break};case 66:{U=1742024;break};case 32:{U=1659064;break};case 35:{U=1624352;break};case 64:{U=1618440;break};case 65:{U=1613928;break};case 128:{U=1722920;break};case 34:{U=1630792;break};case 33:{U=1640536;break};case 67:{U=1736616;break};case 96:{U=1727e3;break};default:{U=1719008}}AN(a,16,1714488,(o=i,i=i+8|0,c[o>>2]=U,o)|0);i=o;p=-22}}while(0);o=c[e>>2]|0;e=o+4|0;if((c[e>>2]|0)!=0){Hf(e)|0}A6(a+96|0);qg(o+32|0);S=p;i=b;return S|0}function n1(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;f=i;i=i+56|0;g=f|0;h=f+24|0;j=f+48|0;k=j;l=i;i=i+12|0;i=i+7&-8;m=i;i=i+4|0;i=i+7&-8;n=i;i=i+4|0;i=i+7&-8;o=i;i=i+4|0;i=i+7&-8;p=i;i=i+4|0;i=i+7&-8;q=i;i=i+4|0;i=i+7&-8;r=i;i=i+4|0;i=i+7&-8;s=i;i=i+4|0;i=i+7&-8;t=i;i=i+4|0;i=i+7&-8;u=c[a+60>>2]|0;NM(g|0,0,20);NM(h|0,0,20);c[j>>2]=0;c[j+4>>2]=0;NM(l|0,0,12);c[m>>2]=0;c[p>>2]=3;v=(d|0)!=0;do{if(v){c[s>>2]=c[d>>2];w=a+416|0;x=c[w>>2]|0;y=c[d+76>>2]|0;c[n>>2]=ac(x<<1,y)|0;c[o>>2]=2;c[j>>2]=ac(y,x)|0;c[g>>2]=1;c[g+4>>2]=s;c[g+8>>2]=m;c[g+12>>2]=n;c[g+16>>2]=o;x=qh(u+32|0,d)|0;if((x|0)<0){z=x}else{A=w;break}i=f;return z|0}else{c[j>>2]=-1;A=a+416|0}}while(0);j=(c[A>>2]|0)*768|0;A=pa(a,b,(j|0)<8192?8192:j)|0;if((A|0)<0){z=A;i=f;return z|0}c[t>>2]=c[b+24>>2];A=b+28|0;c[q>>2]=c[A>>2];c[r>>2]=1;c[h>>2]=1;c[h+4>>2]=t;c[h+8>>2]=p;c[h+12>>2]=q;c[h+16>>2]=r;r=Hg(c[u+4>>2]|0,g,h,k,l)|0;if((r|0)==0){k=l|0;if((c[k>>2]|0)==0){z=0;i=f;return z|0}qi(u+32|0,c[a+424>>2]|0,b+8|0,b+48|0);c[A>>2]=c[k>>2];c[e>>2]=1;z=0;i=f;return z|0}if((r|0)==128&(v^1)){z=0;i=f;return z|0}switch(r|0){case 34:{B=1630792;break};case 64:{B=1618440;break};case 32:{B=1659064;break};case 68:{B=1731664;break};case 67:{B=1736616;break};case 65:{B=1613928;break};case 96:{B=1727e3;break};case 66:{B=1742024;break};case 128:{B=1722920;break};case 35:{B=1624352;break};case 33:{B=1640536;break};default:{B=1719008}}AN(a,16,1695992,(a=i,i=i+8|0,c[a>>2]=B,a)|0);i=a;z=-22;i=f;return z|0}function n2(a){a=a|0;var b=0,d=0;b=c[a+60>>2]|0;d=b+4|0;if((c[d>>2]|0)!=0){Hf(d)|0}A6(a+96|0);qg(b+32|0);return 0}function n3(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0.0,m=0,n=0,o=0,p=0,q=0;b=a+60|0;d=c[b>>2]|0;c[d+4>>2]=a;e=CC()|0;f=d+8|0;c[f>>2]=e;if((e|0)==0){g=-12;return g|0}h=a+416|0;C8(e,c[h>>2]|0)|0;if((c[h>>2]|0)>1){i=(c[d+28>>2]|0)!=0|0}else{i=3}De(c[f>>2]|0,i)|0;i=a+412|0;C7(c[f>>2]|0,c[i>>2]|0)|0;Db(c[f>>2]|0,c[i>>2]|0)|0;i=c[a+84>>2]|0;e=c[f>>2]|0;if((i|0)==-1){Dd(e,5)|0}else{Dd(e,i)|0}i=a+88|0;do{if((c[i>>2]&2|0)==0){e=c[a+72>>2]|0;if((e|0)==0){break}j=c[f>>2]|0;k=(e|0)/1e3|0;Df(j,k)|0}else{k=c[f>>2]|0;Do(k,4)|0;k=c[f>>2]|0;l=+(c[a+80>>2]|0)/118.0;Dr(k,l)|0}}while(0);Dc(c[f>>2]|0,0)|0;Dg(c[f>>2]|0,(c[d+24>>2]|0)==0|0)|0;L21228:do{if((Cu(c[f>>2]|0)|0)<0){m=-1}else{c[a+116>>2]=(DM(c[f>>2]|0)|0)+529;qf(a,d+40|0);k=a+424|0;c[k>>2]=DN(c[f>>2]|0)|0;L21230:do{if((c[a+420>>2]|0)==8){j=d+32|0;e=0;while(1){if((e|0)>=(c[h>>2]|0)){break L21230}n=A1(c[k>>2]<<2)|0;c[j+(e<<2)>>2]=n;if((n|0)==0){m=-12;break L21228}else{e=e+1|0}}}}while(0);k=d+12|0;e=k;j=c[e>>2]|0;if((j|0)==0){o=c[d+16>>2]|0;p=15829}else{n=c[d+16>>2]|0;if(((c[d+20>>2]|0)-n|0)<10792){o=n;p=15829}}do{if((p|0)==15829){n=o+21584|0;q=A2(j,n)|0;if((q|0)==0){A6(k);c[d+16>>2]=0;c[d+20>>2]=0;m=-12;break L21228}else{c[e>>2]=q;c[d+20>>2]=n;break}}}while(0);z7(d+64|0,c[i>>2]&8388608);g=0;return g|0}}while(0);i=c[b>>2]|0;A6(i+32|0);A6(i+36|0);A6(i+12|0);qg(i+40|0);CB(c[i+8>>2]|0)|0;g=m;return g|0}function n4(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;g=i;i=i+40|0;h=g|0;j=c[a+60>>2]|0;k=(e|0)!=0;do{if(k){l=c[a+420>>2]|0;if((l|0)==8){m=e+76|0;n=c[m>>2]|0;if((c[e+32>>2]|0)<((n<<2)+28&-32|0)){AN(a,16,1674168,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;p=-22;i=g;return p|0}q=a+416|0;if((c[q>>2]|0)>0){r=j+72|0;s=j+32|0;t=s;u=0;v=n;while(1){cY[c[r>>2]&15](c[t+(u<<2)>>2]|0,c[e+(u<<2)>>2]|0,32768.0,v+7&-8);w=u+1|0;x=c[m>>2]|0;if((w|0)<(c[q>>2]|0)){u=w;v=x}else{y=x;z=s;break}}}else{y=n;z=j+32|0}s=c[j+16>>2]|0;A=Cx(c[j+8>>2]|0,c[z>>2]|0,c[j+36>>2]|0,y,(c[j+12>>2]|0)+s|0,(c[j+20>>2]|0)-s|0)|0;break}else if((l|0)==7){s=c[j+16>>2]|0;A=Cy(c[j+8>>2]|0,c[e>>2]|0,c[e+4>>2]|0,c[e+76>>2]|0,(c[j+12>>2]|0)+s|0,(c[j+20>>2]|0)-s|0)|0;break}else if((l|0)==6){s=c[j+16>>2]|0;A=Cv(c[j+8>>2]|0,c[e>>2]|0,c[e+4>>2]|0,c[e+76>>2]|0,(c[j+12>>2]|0)+s|0,(c[j+20>>2]|0)-s|0)|0;break}else{p=-558323010;i=g;return p|0}}else{s=c[j+16>>2]|0;A=CA(c[j+8>>2]|0,(c[j+12>>2]|0)+s|0,(c[j+20>>2]|0)-s|0)|0}}while(0);if((A|0)<0){if((A|0)!=-1){p=-1;i=g;return p|0}y=c[j+16>>2]|0;z=(c[j+20>>2]|0)-y|0;AN(a,16,1658952,(o=i,i=i+16|0,c[o>>2]=y,c[o+8>>2]=z,o)|0);i=o;p=-1;i=g;return p|0}z=j+16|0;y=(c[z>>2]|0)+A|0;c[z>>2]=y;A=j+12|0;s=A;v=c[s>>2]|0;if((v|0)==0){B=15855}else{if(((c[j+20>>2]|0)-y|0)<10792){B=15855}}do{if((B|0)==15855){u=y+21584|0;q=A2(v,u)|0;if((q|0)!=0){c[s>>2]=q;c[j+20>>2]=u;break}A6(A);c[z>>2]=0;c[j+20>>2]=0;AN(a,16,1640400,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;p=-12;i=g;return p|0}}while(0);do{if(k){A=qh(j+40|0,e)|0;if((A|0)<0){p=A}else{break}i=g;return p|0}}while(0);if((c[z>>2]|0)<4){p=0;i=g;return p|0}e=c[s>>2]|0;if((oj(h,cV(d[e]|d[e+1|0]<<8|d[e+2|0]<<16|d[e+3|0]<<24|0)|0)|0)!=0){AN(a,16,1630744,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;p=-1;i=g;return p|0}o=c[h>>2]|0;if((o|0)>(c[z>>2]|0)){p=0;i=g;return p|0}h=pa(a,b,o)|0;if((h|0)<0){p=h;i=g;return p|0}h=c[b+24>>2]|0;e=c[s>>2]|0;NN(h|0,e|0,o)|0;e=(c[z>>2]|0)-o|0;c[z>>2]=e;z=c[s>>2]|0;NP(z|0,z+o|0,e|0);qi(j+40|0,c[a+424>>2]|0,b+8|0,b+48|0);c[b+28>>2]=o;c[f>>2]=1;p=0;i=g;return p|0}function n5(a){a=a|0;var b=0;b=c[a+60>>2]|0;A6(b+32|0);A6(b+36|0);A6(b+12|0);qg(b+40|0);CB(c[b+8>>2]|0)|0;return 0}function n6(b){b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;e=i;i=i+32|0;f=e|0;g=e+16|0;h=c[b+60>>2]|0;j=c[b+96>>2]|0;k=h;Fs(k);l=h+256|0;Fp(l);m=c[b+100>>2]|0;if((m|0)==0|(j|0)==0){AN(b,16,1725016,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;o=-1;i=e;return o|0}p=a[j]|0;L21308:do{if((p<<24>>24|0)==0){if((a[j+1|0]|0)!=30){q=15891;break}r=j;s=(cT((d[r]|d[r+1|0]<<8)<<16>>16|0)|0)&65535;c[f>>2]=s;c[g>>2]=j+2;r=s+2|0;t=j+r|0;u=(cT((d[t]|d[t+1|0]<<8)<<16>>16|0)|0)&65535;c[f+4>>2]=u;c[g+4>>2]=j+(s+4);s=r+(u+2)|0;u=j+s|0;c[f+8>>2]=(cT((d[u]|d[u+1|0]<<8)<<16>>16|0)|0)&65535;c[g+8>>2]=j+(s+2)}else if((p<<24>>24|0)==2){s=f|0;c[s>>2]=0;u=j+1|0;r=a[u]|0;L21312:do{if(r<<24>>24==-1){t=1;v=u;w=0;while(1){if(t>>>0>=m>>>0){x=t;y=v;z=-1;A=w;break L21312}B=w+255|0;c[s>>2]=B;C=t+1|0;D=v+1|0;E=a[D]|0;if(E<<24>>24==-1){t=C;v=D;w=B}else{x=C;y=D;z=E;A=B;break}}}else{x=1;y=u;z=r;A=0}}while(0);r=m-1|0;do{if(x>>>0<r>>>0){u=A+(z&255)|0;c[s>>2]=u;w=x+1|0;v=f+4|0;c[v>>2]=0;t=y+1|0;B=a[t]|0;L21319:do{if(B<<24>>24==-1){E=w;D=t;C=0;while(1){if(E>>>0>=m>>>0){F=E;G=-1;H=C;break L21319}I=C+255|0;c[v>>2]=I;J=E+1|0;K=D+1|0;L=a[K]|0;if(L<<24>>24==-1){E=J;D=K;C=I}else{F=J;G=L;H=I;break}}}else{F=w;G=B;H=0}}while(0);if(F>>>0>=r>>>0){break}B=H+(G&255)|0;c[v>>2]=B;w=F+1|0;c[f+8>>2]=m+~F-u-B;c[g>>2]=j+w;t=u+w|0;c[g+4>>2]=j+t;c[g+8>>2]=j+(t+B);break L21308}}while(0);AN(b,16,1695888,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;o=-1;i=e;return o|0}else{q=15891}}while(0);if((q|0)==15891){AN(b,16,1674128,(n=i,i=i+8|0,c[n>>2]=p&255,n)|0);i=n;o=-1;i=e;return o|0}p=h+272|0;j=p;F=h+280|0;m=h+276|0;G=p;p=0;while(1){if((p|0)>=3){q=15895;break}c[F>>2]=(p|0)==0;c[m>>2]=c[f+(p<<2)>>2];c[G>>2]=c[g+(p<<2)>>2];M=p+1|0;if((Fu(k,l,j)|0)<0){q=15894;break}else{p=M}}if((q|0)==15894){AN(b,16,1658920,(n=i,i=i+8|0,c[n>>2]=M,n)|0);i=n;o=-1;i=e;return o|0}else if((q|0)==15895){c[b+416>>2]=c[h+4>>2];q=c[h+8>>2]|0;c[b+412>>2]=q;c[b+420>>2]=1;c[b+104>>2]=1;c[b+108>>2]=q;q=h+32|0;Ff(q,k)|0;E4(q,h+144|0)|0;o=0;i=e;return o|0}return 0}function n7(a,d,e,f){a=a|0;d=d|0;e=e|0;f=f|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;h=i;i=i+8|0;j=h|0;k=c[a+60>>2]|0;l=k+272|0;m=f+28|0;if((c[m>>2]|0)==0){n=0;i=h;return n|0}o=d+76|0;c[o>>2]=32768;p=oX(a,d,0)|0;if((p|0)<0){n=p;i=h;return n|0}p=c[d>>2]|0;c[l>>2]=c[f+24>>2];c[k+276>>2]=c[m>>2];f=k+144|0;d=k+32|0;if((Gl(f,l)|0)==0){Fg(d,f)|0}f=Fh(d,j)|0;if((f|0)>0){l=k+4|0;k=0;a=0;q=f;while(1){f=c[j>>2]|0;r=c[l>>2]|0;s=p+k|0;if((r|0)>0){t=0;do{u=c[f+(t<<2)>>2]|0;v=0;w=s+(t<<1)|0;while(1){x=~~(+g[u+(v<<2)>>2]*32767.0);if((x+32768|0)>>>0>65535){y=(x>>31^32767)&65535}else{y=x&65535}b[w>>1]=y;x=v+1|0;if((x|0)<(q|0)){v=x;w=w+(r<<1)|0}else{break}}t=t+1|0;}while((t|0)<(r|0));z=c[l>>2]|0}else{z=r}t=(ac(q<<1,z)|0)+k|0;s=q+a|0;Fi(d,q)|0;f=Fh(d,j)|0;if((f|0)>0){k=t;a=s;q=f}else{A=s;break}}}else{A=0}c[o>>2]=A;c[e>>2]=1;n=c[m>>2]|0;i=h;return n|0}function n8(a){a=a|0;var b=0;b=c[a+60>>2]|0;Ft(b);Fr(b+256|0);return 0}function n9(b){b=b|0;var d=0,e=0,f=0,g=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0;d=i;i=i+136|0;e=d|0;f=d+40|0;g=d+72|0;j=d+104|0;k=b+60|0;l=c[k>>2]|0;m=l+464|0;Fs(m);n=e;o=d+8|0;p=c[k>>2]|0;k=b+88|0;q=(c[k>>2]&2|0)==0;do{if(q){r=c[b+72>>2]|0;if((r|0)==0){s=15928;break}t=c[b+544>>2]|0;u=(t|0)>0?t:-1;t=c[b+540>>2]|0;v=(t|0)>0?t:-1;t=G$(m,c[b+416>>2]|0,c[b+412>>2]|0,v,r,u)|0;if((t|0)!=0){w=t;s=15947;break}if(!((u|0)==-1&(v|0)==-1)){s=15932;break}v=G0(m,21,0)|0;if((v|0)==0){s=15932}else{w=v;s=15947}}else{s=15928}}while(0);if((s|0)==15928){v=GY(m,c[b+416>>2]|0,c[b+412>>2]|0,(q?3.0:+(c[b+80>>2]|0)/118.0)/10.0)|0;if((v|0)==0){s=15932}else{w=v;s=15947}}do{if((s|0)==15932){v=c[b+436>>2]|0;if((v|0)>0){h[e>>3]=+(v|0)/1.0e3;v=G0(m,33,n)|0;if((v|0)!=0){w=v;s=15947;break}}v=p+752|0;if(+h[v>>3]!=0.0){q=G0(m,49,v)|0;if((q|0)!=0){w=q;s=15947;break}}q=c[b+416>>2]|0;switch(q|0){case 6:{v=b+448|0;u=c[v>>2]|0;t=c[v+4>>2]|0;if(!((u|0)==1551&(t|0)==0|(u|0)==63&(t|0)==0)){x=t;y=u;s=15943}break};case 4:{u=b+448|0;t=c[u>>2]|0;v=c[u+4>>2]|0;if(!((t|0)==1539&(v|0)==0|(t|0)==51&(v|0)==0)){x=v;y=t;s=15943}break};case 3:{t=b+448|0;v=c[t>>2]|0;u=c[t+4>>2]|0;if(!((v|0)==7&(u|0)==0)){x=u;y=v;s=15943}break};case 5:{v=b+448|0;u=c[v>>2]|0;t=c[v+4>>2]|0;if(!((u|0)==1543&(t|0)==0|(u|0)==55&(t|0)==0)){x=t;y=u;s=15943}break};case 8:{u=b+448|0;t=c[u>>2]|0;v=c[u+4>>2]|0;if(!((t|0)==1599&(v|0)==0)){x=v;y=t;s=15943}break};case 7:{t=b+448|0;v=c[t>>2]|0;u=c[t+4>>2]|0;if(!((v|0)==1807&(u|0)==0)){x=u;y=v;s=15943}break};default:{}}do{if((s|0)==15943){if((y|0)==0&(x|0)==0){AN(b,24,1718912,(z=i,i=i+8|0,c[z>>2]=q,z)|0);i=z;break}else{zt(o,32,q,y,x);AN(b,16,1722840,(z=i,i=i+8|0,c[z>>2]=o,z)|0);i=z;break}}}while(0);q=GV(m)|0;if((q|0)!=0){w=q;s=15947;break}q=l+496|0;v=E8(q,m)|0;if((v|0)!=0){AN(b,16,1618400,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z;if((v|0)==(-129|0)){A=-558323010;break}else if((v|0)==(-131|0)){A=-22;break}else if((v|0)==(-130|0)){A=-22;break}else{A=-1313558101;break}}c[l+728>>2]=1;v=E4(q,l+608|0)|0;if((v|0)!=0){AN(b,16,1613872,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z;if((v|0)==(-131|0)){A=-22;break}else if((v|0)==(-130|0)){A=-22;break}else if((v|0)==(-129|0)){A=-558323010;break}else{A=-1313558101;break}}v=l+732|0;Fp(v);if((c[k>>2]&8388608|0)==0){Fq(v,1741968,1736576)}u=Fw(q,v,f,g,j)|0;if((u|0)==(-129|0)){A=-558323010;break}else if((u|0)==(-130|0)){A=-22;break}else if((u|0)==0){q=f+4|0;t=c[q>>2]|0;r=g+4|0;B=c[r>>2]|0;C=j+4|0;D=t+3+((t|0)/255|0)+B+((B|0)/255|0)+(c[C>>2]|0)|0;B=b+100|0;c[B>>2]=D;t=A1(D+16|0)|0;c[b+96>>2]=t;if((t|0)==0){A=-12;break}a[t]=2;D=(pu(t+1|0,c[q>>2]|0)|0)+1|0;E=(pu(t+D|0,c[r>>2]|0)|0)+D|0;D=t+E|0;F=c[f>>2]|0;G=c[q>>2]|0;NN(D|0,F|0,G)|0;F=G+E|0;E=t+F|0;G=c[g>>2]|0;D=c[r>>2]|0;NN(E|0,G|0,D)|0;G=D+F|0;F=t+G|0;t=c[j>>2]|0;D=c[C>>2]|0;NN(F|0,t|0,D)|0;if((D+G|0)!=(c[B>>2]|0)){AN(0,0,1658888,(z=i,i=i+24|0,c[z>>2]=1731584,c[z+8>>2]=1630712,c[z+16>>2]=249,z)|0);i=z;cw();return 0}B=py(b,l+760|0)|0;if((B|0)<0){AN(b,16,1726952,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z;H=B;i=d;return H|0}else{Fr(v);c[b+424>>2]=64;qf(b,l+1052|0);v=z$(65536)|0;c[l+720>>2]=v;if((v|0)==0){A=-12;break}else{H=0}i=d;return H|0}}else if((u|0)==(-131|0)){A=-22;break}else{A=-1313558101;break}}}while(0);if((s|0)==15947){if((w|0)==(-131|0)){I=-22}else if((w|0)==(-130|0)){I=-22}else if((w|0)==(-129|0)){I=-558323010}else{I=-1313558101}AN(b,16,1624296,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z;A=I}ob(b)|0;H=A;i=d;return H|0}function oa(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;g=i;i=i+40|0;h=g|0;j=g+8|0;k=c[a+60>>2]|0;do{if((e|0)==0){l=k+724|0;m=k+496|0;do{if((c[l>>2]|0)==0){n=Fc(m,0)|0;if((n|0)>=0){break}AN(a,16,1724960,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;if((n|0)==(-129|0)){p=-558323010;i=g;return p|0}else if((n|0)==(-131|0)){p=-22;i=g;return p|0}else if((n|0)==(-130|0)){p=-22;i=g;return p|0}else{p=-1313558101;i=g;return p|0}}}while(0);c[l>>2]=1;q=m}else{n=c[e+76>>2]|0;r=c[k+468>>2]|0;s=k+496|0;t=Fb(s,n)|0;if((r|0)>0){u=e+64|0;v=n<<2;w=r-1|0;if((r|0)>8){x=0;do{y=c[t+(x<<2)>>2]|0;z=c[(c[u>>2]|0)+(x<<2)>>2]|0;NN(y|0,z|0,v)|0;x=x+1|0;}while((x|0)<(r|0))}else{x=0;do{m=c[t+(x<<2)>>2]|0;l=c[(c[u>>2]|0)+((d[237776+(w<<3)+x|0]|0)<<2)>>2]|0;NN(m|0,l|0,v)|0;x=x+1|0;}while((x|0)<(r|0))}}r=Fc(s,n)|0;if((r|0)>=0){x=qh(k+1052|0,e)|0;if((x|0)<0){p=x}else{q=s;break}i=g;return p|0}AN(a,16,1724960,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;if((r|0)==(-129|0)){p=-558323010;i=g;return p|0}else if((r|0)==(-131|0)){p=-22;i=g;return p|0}else if((r|0)==(-130|0)){p=-22;i=g;return p|0}else{p=-1313558101;i=g;return p|0}}}while(0);e=k+608|0;r=k+720|0;x=j+4|0;v=j;w=j|0;L21474:while(1){u=Fe(q,e)|0;if((u|0)!=1){A=u;B=16007;break}u=E_(e,0)|0;if((u|0)<0){C=u;break}u=E2(e)|0;if((u|0)<0){C=u;break}u=E3(q,j)|0;if((u|0)==1){while(1){t=z2(c[r>>2]|0)|0;if(t>>>0<((c[x>>2]|0)+32|0)>>>0){B=16003;break L21474}z5(c[r>>2]|0,v,32,0)|0;z5(c[r>>2]|0,c[w>>2]|0,c[x>>2]|0,0)|0;t=E3(q,j)|0;if((t|0)!=1){D=t;break}}}else{D=u}if((D|0)<0){B=16006;break}}if((B|0)==16003){AN(a,16,1695856,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;p=-558323010;i=g;return p|0}else if((B|0)==16006){AN(a,16,1674088,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;A=D;B=16007}do{if((B|0)==16007){if((A|0)<0){C=A;break}if((z1(c[r>>2]|0)|0)>>>0<32){p=0;i=g;return p|0}D=c[r>>2]|0;z4(D,v,32,0)|0;D=pa(a,b,c[x>>2]|0)|0;if((D|0)<0){p=D;i=g;return p|0}D=c[r>>2]|0;q=b+24|0;w=c[q>>2]|0;e=c[x>>2]|0;z4(D,w,e,0)|0;e=j+16|0;w=c[e>>2]|0;D=c[e+4>>2]|0;if((w|0)==0&(D|0)==(-2147483648|0)){E=-2147483648;F=0}else{c[h>>2]=1;c[h+4>>2]=c[a+412>>2];e=AY(w,D,h,a+104|0)|0;E=G;F=e}e=b+8|0;c[e>>2]=F;c[e+4>>2]=E;D=pz(k+760|0,c[q>>2]|0,c[b+28>>2]|0)|0;if((D|0)>0){q=a+116|0;do{if((c[q>>2]|0)==0){w=k+1064|0;if((c[w>>2]|0)==0){break}c[q>>2]=D;if((c[k+1056>>2]|0)==0){s=(c[w>>2]|0)+8|0;c[s>>2]=(c[s>>2]|0)+D;s=c[w>>2]|0;w=NV(c[s>>2]|0,c[s+4>>2]|0,D,(D|0)<0|0?-1:0)|0;c[s>>2]=w;c[s+4>>2]=G;s=k+1060|0;c[s>>2]=(c[s>>2]|0)+D;break}else{AN(0,0,1658888,(o=i,i=i+24|0,c[o>>2]=1640376,c[o+8>>2]=1630712,c[o+16>>2]=352,o)|0);i=o;cw();return 0}}}while(0);qi(k+1052|0,D,e,b+48|0)}c[f>>2]=1;p=0;i=g;return p|0}}while(0);AN(a,16,1674088,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;if((C|0)==(-129|0)){p=-558323010;i=g;return p|0}else if((C|0)==(-131|0)){p=-22;i=g;return p|0}else if((C|0)==(-130|0)){p=-22;i=g;return p|0}else{p=-1313558101;i=g;return p|0}return 0}function ob(a){a=a|0;var b=0,d=0;b=c[a+60>>2]|0;d=b+496|0;if((c[b+728>>2]|0)!=0){Fc(d,0)|0}E7(b+608|0)|0;Fa(d);Ft(b+464|0);z0(c[b+720>>2]|0);qg(b+1052|0);A6(a+96|0);return 0}function oc(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0;if((e|0)<1){g=-1094995529;return g|0}h=e>>>0>2147483639|(b|0)==0;i=h?0:b;b=h?0:e;e=b+8|0;if(h){g=-1094995529;return g|0}h=i;j=(d[h]|d[h+1|0]<<8|d[h+2|0]<<16|d[h+3|0]<<24|0)>>>3&31;h=e>>>0>5?5:e;if((j|0)==31){k=i+(h>>>3)|0;l=h+6|0;m=((cV(d[k]|d[k+1|0]<<8|d[k+2|0]<<16|d[k+3|0]<<24|0)|0)<<(h&7)>>>26)+32|0;n=e>>>0>l>>>0?l:e}else{m=j;n=h}h=a|0;c[h>>2]=m;j=i+(n>>>3)|0;l=(cV(d[j]|d[j+1|0]<<8|d[j+2|0]<<16|d[j+3|0]<<24|0)|0)<<(n&7)>>>28;j=n+4|0;n=e>>>0>j>>>0?j:e;c[a+4>>2]=l;if((l|0)==15){j=i+(n>>>3)|0;k=n+24|0;o=(cV(d[j]|d[j+1|0]<<8|d[j+2|0]<<16|d[j+3|0]<<24|0)|0)<<(n&7)>>>8;p=e>>>0>k>>>0?k:e}else{o=c[337504+(l<<2)>>2]|0;p=n}n=a+8|0;c[n>>2]=o;l=i+(p>>>3)|0;k=(cV(d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0)|0)<<(p&7);l=k>>>28;j=p+4|0;p=e>>>0>j>>>0?j:e;j=a+12|0;c[j>>2]=l;if((k|0)>-1){c[a+36>>2]=d[245544+l|0]|0}l=a+16|0;c[l>>2]=-1;k=a+40|0;c[k>>2]=-1;do{if((m|0)==5){q=-1;r=i+(p>>>3)|0;s=p&7;t=16058}else if((m|0)==29){u=i+(p>>>3)|0;v=p&7;w=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<v;if((w&1610612736|0)!=0&(w&528482304|0)==0){t=16065;break}c[k>>2]=1;q=1;r=u;s=v;t=16058}else{t=16065}}while(0);do{if((t|0)==16058){c[a+20>>2]=5;c[l>>2]=1;v=(cV(d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0)|0)<<s>>>28;u=p+4|0;w=e>>>0>u>>>0?u:e;c[a+24>>2]=v;if((v|0)==15){u=i+(w>>>3)|0;x=w+24|0;y=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(w&7)>>>8;z=e>>>0>x>>>0?x:e}else{y=c[337504+(v<<2)>>2]|0;z=w}c[a+28>>2]=y;w=i+(z>>>3)|0;v=(cV(d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0)|0)<<(z&7)>>>27;w=z+5|0;x=e>>>0>w>>>0?w:e;if((v|0)==31){w=i+(x>>>3)|0;u=x+6|0;A=((cV(d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0)|0)<<(x&7)>>>26)+32|0;B=e>>>0>u>>>0?u:e}else{A=v;B=x}c[h>>2]=A;if((A|0)!=22){C=B;D=A;E=5;F=1;G=q;t=16066;break}x=i+(B>>>3)|0;v=B+4|0;c[a+32>>2]=(cV(d[x]|d[x+1|0]<<8|d[x+2|0]<<16|d[x+3|0]<<24|0)|0)<<(B&7)>>>28;H=e>>>0>v>>>0?v:e;I=22;J=q;t=16094}else if((t|0)==16065){c[a+20>>2]=0;c[a+28>>2]=0;C=p;D=m;E=0;F=-1;G=-1;t=16066}}while(0);L21556:do{if((t|0)==16066){if((D|0)==36){m=C+5|0;p=e>>>0>m>>>0?m:e;m=i+(p>>>3)|0;if(((cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0)<<(p&7)&-256|0)==1095521024){K=p}else{m=-p|0;q=e-p|0;if((m|0)>24){L=m}else{L=(q|0)<24?q:24}K=L+p|0}if((b-K|0)<112){g=-1;return g|0}p=i+(K>>>3)|0;q=(cV(d[p]|d[p+1|0]<<8|d[p+2|0]<<16|d[p+3|0]<<24|0)|0)<<(K&7)&-65536;p=K+16|0;m=e>>>0>p>>>0?p:e;p=i+(m>>>3)|0;B=m+16|0;A=e>>>0>B>>>0?B:e;if(((cV(d[p]|d[p+1|0]<<8|d[p+2|0]<<16|d[p+3|0]<<24|0)|0)<<(m&7)>>>16|q|0)!=1095521024){g=-1;return g|0}q=i+(A>>>3)|0;m=(cV(d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0)|0)<<(A&7)&-65536;q=A+16|0;A=e>>>0>q>>>0?q:e;q=i+(A>>>3)|0;p=A+16|0;B=e>>>0>p>>>0?p:e;p=(cV(d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0)|0)<<(A&7)>>>16|m;c[n>>2]=p;m=-B|0;A=e-B|0;if((m|0)>32){M=m}else{M=(A|0)<32?A:32}A=M+B|0;c[j>>2]=0;B=i+(A>>>3)|0;m=A+16|0;c[a+36>>2]=((cV(d[B]|d[B+1|0]<<8|d[B+2|0]<<16|d[B+3|0]<<24|0)|0)<<(A&7)>>>16)+1;N=K;O=e>>>0>m>>>0?m:e;P=p}else{N=C;O=C;P=o}p=a+20|0;if((E|0)==5|(f|0)==0){H=N;I=D;J=G;t=16094;break}if((b-O|0)>15){Q=O}else{H=N;I=D;J=G;t=16094;break}while(1){m=i+(Q>>>3)|0;if(((cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0)<<(Q&7)&-2097152|0)==1457520640){break}m=((Q|0)<(e|0))+Q|0;if((b-m|0)>15){Q=m}else{H=N;I=D;J=G;t=16094;break L21556}}m=Q+11|0;A=e>>>0>m>>>0?m:e;m=i+(A>>>3)|0;B=(cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0)<<(A&7)>>>27;m=A+5|0;A=e>>>0>m>>>0?m:e;if((B|0)==31){m=i+(A>>>3)|0;q=A+6|0;R=((cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0)<<(A&7)>>>26)+32|0;S=e>>>0>q>>>0?q:e}else{R=B;S=A}c[p>>2]=R;do{if((R|0)==5){A=((S|0)<(e|0))+S|0;B=(d[i+(S>>>3)|0]|0)<<(S&7)>>>7&1;c[l>>2]=B;if((B|0)==0){T=A;U=0;break}q=i+(A>>>3)|0;m=(cV(d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0)|0)<<(A&7)>>>28;q=A+4|0;A=e>>>0>q>>>0?q:e;c[a+24>>2]=m;if((m|0)==15){q=i+(A>>>3)|0;h=A+24|0;V=(cV(d[q]|d[q+1|0]<<8|d[q+2|0]<<16|d[q+3|0]<<24|0)|0)<<(A&7)>>>8;W=e>>>0>h>>>0?h:e}else{V=c[337504+(m<<2)>>2]|0;W=A}c[a+28>>2]=V;if((V|0)!=(P|0)){T=W;U=B;break}c[l>>2]=-1;T=W;U=-1}else{T=S;U=F}}while(0);do{if((b-T|0)>11){p=i+(T>>>3)|0;B=T+11|0;A=e>>>0>B>>>0?B:e;if(((cV(d[p]|d[p+1|0]<<8|d[p+2|0]<<16|d[p+3|0]<<24|0)|0)<<(T&7)&-2097152|0)!=-1459617792){X=G;break}p=(d[i+(A>>>3)|0]|0)<<(A&7)>>>7&1;c[k>>2]=p;X=p}else{X=G}}while(0);if((U|0)!=0){H=N;I=D;J=X;t=16094;break}c[k>>2]=0;Y=N;t=16095}}while(0);if((t|0)==16094){if((J|0)!=-1|(I|0)==2){Y=H;t=16095}else{Z=H}}do{if((t|0)==16095){if((c[a+36>>2]|0)>>>0>1){Z=Y;break}else{g=Y}return g|0}}while(0);c[k>>2]=0;g=Z;return g|0}function od(a,b,e,f,g,h){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;j=i;i=i+48|0;k=j|0;l=j+8|0;m=j+16|0;n=j+24|0;o=j+32|0;p=j+40|0;c[k>>2]=g;c[l>>2]=h;g=c[a>>2]|0;q=g;r=g+16|0;s=c[r>>2]|0;L21606:do{if((h|0)>0){t=g+40|0;u=g+48|0;v=g+44|0;w=b+412|0;x=b+416|0;y=a+304|0;z=g+52|0;A=b+72|0;B=0;C=s;D=h;while(1){E=c[t>>2]|0;L21610:do{if((E|0)==0){if((B|0)<(D|0)){F=B;G=C}else{H=C;I=B;break}while(1){J=G<<8;K=F+1|0;L=d[(c[k>>2]|0)+F|0]|0|J;M=ok(b,L,m,n,p,o)|0;if((M|0)>=4){break}if((K|0)>4){c[u>>2]=-2}if((K|0)<(c[l>>2]|0)){F=K;G=L}else{H=L;I=K;break L21610}}N=c[v>>2]|0;if(!(((N^J)&-128e3|0)==0|(N|0)==0)){c[v>>2]=L;c[u>>2]=-2;c[t>>2]=M-4;H=L;I=K;break}N=(c[u>>2]|0)+1|0;c[v>>2]=L;c[u>>2]=N;c[t>>2]=M-4;if((N|0)<=0){H=L;I=K;break}c[w>>2]=c[m>>2];c[x>>2]=c[n>>2];c[y>>2]=c[p>>2];if((c[z>>2]|0)==0){if((c[A>>2]|0)!=0){H=L;I=K;break}}c[z>>2]=1;N=c[A>>2]|0;c[A>>2]=(((c[o>>2]|0)-N|0)/(c[u>>2]|0)|0)+N;H=L;I=K}else{N=D-B|0;O=(N|0)>(E|0)?E:N;N=O+B|0;c[t>>2]=E-O;if((E|0)==(O|0)){P=0;Q=N;break L21606}else{H=0;I=N}}}while(0);E=c[l>>2]|0;if((I|0)<(E|0)){B=I;C=H;D=E}else{P=H;Q=-100;break}}}else{P=s;Q=-100}}while(0);c[r>>2]=P;if((oI(q,Q,k,l)|0)<0){c[e>>2]=0;c[f>>2]=0;R=c[l>>2]|0;i=j;return R|0}else{c[e>>2]=c[k>>2];c[f>>2]=c[l>>2];R=Q;i=j;return R|0}return 0}function oe(e){e=e|0;var f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0.0,B=0.0,C=0,D=0,E=0,F=0,H=0;f=i;i=i+1544|0;h=f|0;j=f+8|0;k=f+520|0;l=c[e+60>>2]|0;if(!(a[262896]|0)){m=j|0;n=k;o=0;while(1){b[2162456+(o<<1)>>1]=((o|0)%3|0|((o|0)/3|0)<<2)&65535;p=o+1|0;if((p|0)<64){o=p}else{q=0;break}}do{o=q+2|0;p=NW(8388608,0,o|0)|0;r=(1<<o)-1|0;o=N0(p,G,r,(r|0)<0|0?-1:0)|0;r=o;p=r;s=(r|0)<0|0?-1:0;c[2162272+(q*12|0)>>2]=o<<1|0>>>31;o=N2(p,s,13316085,0)|0;c[2162276+(q*12|0)>>2]=o>>>23|G<<9;o=N2(p,s,10568984,0)|0;c[2162280+(q*12|0)>>2]=o>>>23|G<<9;q=q+1|0;}while((q|0)<15);oo(2599496);q=0;o=1;while(1){NM(m|0,0,512);NM(n|0,0,1024);s=c[125720+(o*12|0)>>2]|0;p=(s|0)>1?s:1;r=c[125724+(o*12|0)>>2]|0;t=c[125728+(o*12|0)>>2]|0;u=0;v=0;while(1){w=u<<5;if((u|0)==0){x=0;y=v;while(1){z=x|w;a[j+z|0]=a[r+y|0]|0;b[k+(z<<1)>>1]=b[t+(y<<1)>>1]|0;z=x+1|0;if((z|0)<(s|0)){x=z;y=y+1|0}else{break}}}else{y=0;x=v;while(1){z=y|w|((y|0)!=0)<<4;a[j+z|0]=a[r+x|0]|0;b[k+(z<<1)>>1]=b[t+(x<<1)>>1]|0;z=y+1|0;if((z|0)<(s|0)){y=z;x=x+1|0}else{break}}}x=u+1|0;if((x|0)<(s|0)){u=x;v=v+p|0}else{break}}c[2222236+(o<<4)>>2]=2207248+(q<<2);p=c[173440+(o<<2)>>2]|0;c[2222244+(o<<4)>>2]=p;qk(2222232+(o<<4)|0,7,512,m,1,1,n,2,2,0,0,0,4)|0;v=o+1|0;if((v|0)<16){q=p+q|0;o=v}else{break}}c[555767]=2222488;c[555769]=128;qk(2223064,7,16,121464,1,1,121432,1,1,0,0,0,4)|0;c[555771]=2223e3;c[555773]=16;qk(2223080,4,16,121480,1,1,121448,1,1,0,0,0,4)|0;o=0;q=0;n=0;while(1){b[3014136+(q<<1)>>1]=n;m=(d[337232+q|0]|0)+o|0;k=q+1|0;if((k|0)<22){o=m;q=k;n=m&65535}else{break}}b[1507090]=576;n=0;q=0;o=0;while(1){b[3014182+(q<<1)>>1]=o;m=(d[337254+q|0]|0)+n|0;k=q+1|0;if((k|0)<22){n=m;q=k;o=m&65535}else{break}}b[1507113]=576;o=0;q=0;n=0;while(1){b[3014228+(q<<1)>>1]=n;m=(d[337276+q|0]|0)+o|0;k=q+1|0;if((k|0)<22){o=m;q=k;n=m&65535}else{break}}b[1507136]=576;n=0;q=0;o=0;while(1){b[3014274+(q<<1)>>1]=o;m=(d[337298+q|0]|0)+n|0;k=q+1|0;if((k|0)<22){n=m;q=k;o=m&65535}else{break}}b[1507159]=576;o=0;q=0;n=0;while(1){b[3014320+(q<<1)>>1]=n;m=(d[337320+q|0]|0)+o|0;k=q+1|0;if((k|0)<22){o=m;q=k;n=m&65535}else{break}}b[1507182]=576;n=0;q=0;o=0;while(1){b[3014366+(q<<1)>>1]=o;m=(d[337342+q|0]|0)+n|0;k=q+1|0;if((k|0)<22){n=m;q=k;o=m&65535}else{break}}b[1507205]=576;o=0;q=0;n=0;while(1){b[3014412+(q<<1)>>1]=n;m=(d[337364+q|0]|0)+o|0;k=q+1|0;if((k|0)<22){o=m;q=k;n=m&65535}else{break}}b[1507228]=576;n=0;q=0;o=0;while(1){b[3014458+(q<<1)>>1]=o;m=(d[337386+q|0]|0)+n|0;k=q+1|0;if((k|0)<22){n=m;q=k;o=m&65535}else{break}}b[1507251]=576;o=0;q=0;n=0;while(1){b[3014504+(q<<1)>>1]=n;m=(d[337408+q|0]|0)+o|0;k=q+1|0;if((k|0)<22){o=m;q=k;n=m&65535}else{break}}b[1507274]=576;n=1;while(1){A=+((n|0)/4|0|0);B=A/1.759*+a_(+A);q=~~(+aY(+(+b4(+(+(n&3|0)*.25))*B),h|0)*2147483648.0+.5);o=c[h>>2]|0;c[h>>2]=o-103;c[1998088+(n<<2)>>2]=q;a[2129400+n|0]=103-o&255;o=n+1|0;if((o|0)<32828){n=o}else{C=0;break}}do{B=+b4(+(+(C-400|0)*.25+23.0+5.0));n=0;do{A=B*+(n|0)*+a_(+(+(n|0)))/1.759;h=cS(+A)|0;c[2941552+(C<<6)+(n<<2)>>2]=h;g[2908784+(C<<6)+(n<<2)>>2]=A;n=n+1|0;}while((n|0)<16);c[2974320+(C<<2)>>2]=c[2941556+(C<<6)>>2];C=C+1|0;}while((C|0)<512);C=c[61444]|0;do{if((C|0)<0){n=1<<1-C;if((n|0)<=0){break}h=c[61426]|0;o=0;do{q=(o|0)/(h|0)|0;b[2981040+(o<<1)>>1]=(((q|0)%(h|0)|0)<<4)+((o|0)%(h|0)|0)+(((q|0)/(h|0)|0)<<8)&65535;o=o+1|0;}while((o|0)<(n|0))}}while(0);C=c[61445]|0;do{if((C|0)<0){n=1<<1-C;if((n|0)<=0){break}o=c[61427]|0;h=0;do{q=(h|0)/(o|0)|0;b[2980528+(h<<1)>>1]=(((q|0)%(o|0)|0)<<4)+((h|0)%(o|0)|0)+(((q|0)/(o|0)|0)<<8)&65535;h=h+1|0;}while((h|0)<(n|0))}}while(0);C=c[61446]|0;do{if((C|0)<0){n=1<<1-C;if((n|0)<=0){break}h=c[61428]|0;o=0;do{q=(o|0)/(h|0)|0;b[o<<1>>1]=(((q|0)%(h|0)|0)<<4)+((o|0)%(h|0)|0)+(((q|0)/(h|0)|0)<<8)&65535;o=o+1|0;}while((o|0)<(n|0))}}while(0);C=c[61447]|0;do{if((C|0)<0){n=1<<1-C;if((n|0)<=0){D=0;break}o=c[61429]|0;h=0;while(1){q=(h|0)/(o|0)|0;b[2976432+(h<<1)>>1]=(((q|0)%(o|0)|0)<<4)+((h|0)%(o|0)|0)+(((q|0)/(o|0)|0)<<8)&65535;q=h+1|0;if((q|0)<(n|0)){h=q}else{D=0;break}}}else{D=0}}while(0);do{if((D|0)==6){E=8388608}else{B=+W(+(+(D|0)*3.141592653589793/12.0));E=~~(B/(B+1.0)*8388608.0+.5)}c[2206048+(D<<2)>>2]=E;c[2206112+(6-D<<2)>>2]=E;D=D+1|0;}while((D|0)<7);NM(2206076,0,36);NM(2206140,0,36);D=0;while(1){E=D+1|0;C=E>>1;h=D&1;n=h^1;c[2205792+(n<<6)+(D<<2)>>2]=~~(+b4(+(+(-C|0)*.25))*8388608.0+.5);c[2205792+(h<<6)+(D<<2)>>2]=8388608;c[2205920+(n<<6)+(D<<2)>>2]=~~(+b4(+(+(C*-2|0|0)*.25))*8388608.0+.5);c[2205920+(h<<6)+(D<<2)>>2]=8388608;if((E|0)<16){D=E}else{F=0;break}}do{B=+g[299768+(F<<2)>>2];A=1.0/+S(+(B*B+1.0));D=~~(A*.25*4294967296.0+.5);c[2981200+(F<<4)>>2]=D;E=~~(B*A*.25*4294967296.0+.5);c[2981204+(F<<4)>>2]=E;c[2981208+(F<<4)>>2]=E+D;c[2981212+(F<<4)>>2]=E-D;F=F+1|0;}while((F|0)<8);a[262896]=1}c[l+32940>>2]=e;z7(l+32968|0,c[e+88>>2]&8388608);ol(l+32944|0);F=e+48|0;do{if((c[e+468>>2]|0)==1){if((c[F>>2]|0)==86030){H=16186;break}c[e+420>>2]=1}else{H=16186}}while(0);if((H|0)==16186){c[e+420>>2]=6}c[l+32936>>2]=c[e+696>>2];if((c[F>>2]|0)!=86029){i=f;return 0}c[l+32928>>2]=1;i=f;return 0}function of(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;j=g+28|0;k=c[j>>2]|0;l=c[b+60>>2]|0;m=l;if((k|0)==0){n=-1094995529;i=h;return n|0}o=k;k=c[g+24>>2]|0;while(1){if((a[k]|0)!=0){break}g=o-1|0;if((g|0)==0){n=-1094995529;p=16215;break}else{o=g;k=k+1|0}}if((p|0)==16215){i=h;return n|0}if((o|0)<4){n=-1094995529;i=h;return n|0}p=k;g=cV(d[p]|d[p+1|0]<<8|d[p+2|0]<<16|d[p+3|0]<<24|0)|0;if((g&-256|0)==1413564160){AN(b,48,1674016,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;n=o;i=h;return n|0}if((g&-2097152|0)!=-2097152|(g&393216|0)==0|(g&61440|0)==61440|(g&3072|0)==3072){AN(b,16,1658864,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;n=-1094995529;i=h;return n|0}if((oj(l,g)|0)==1){c[l>>2]=-1;n=-1094995529;i=h;return n|0}g=l+24|0;c[b+416>>2]=c[g>>2];p=(c[g>>2]|0)==1;g=b+448|0;c[g>>2]=p?4:3;c[g+4>>2]=p?0:0;p=b+72|0;if((c[p>>2]|0)==0){c[p>>2]=c[l+20>>2]}p=l;g=c[p>>2]|0;if((g|0)<1|(g|0)>(o|0)){AN(b,16,1640336,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;n=-1094995529;i=h;return n|0}if((g|0)<(o|0)){AN(b,48,1630648,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;r=c[p>>2]|0}else{r=o}o=l+33004|0;c[o>>2]=e;e=oh(m,k,r)|0;do{if((e|0)>-1){c[(c[o>>2]|0)+76>>2]=c[b+424>>2];c[f>>2]=1;c[b+412>>2]=c[l+12>>2]}else{AN(b,16,1624256,(q=i,i=i+1|0,i=i+7&-8,c[q>>2]=0,q)|0);i=q;c[f>>2]=0;if((r|0)!=(c[j>>2]|0)&(e|0)==-1094995529){break}else{n=e}i=h;return n|0}}while(0);c[p>>2]=0;n=r;i=h;return n|0}function og(a){a=a|0;var b=0;b=c[a+60>>2]|0;NM(b+1152|0,0,8192);c[b+1088>>2]=0;return}function oh(f,g,h){f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0,cb=0,cc=0,cd=0,ce=0,cf=0,cg=0,ch=0,ci=0,cj=0,ck=0,cl=0,cm=0,cn=0,co=0,cp=0,cq=0,cr=0,cs=0,ct=0,cu=0,cv=0,cx=0,cy=0;j=i;i=i+3808|0;k=j|0;l=j+192|0;m=j+256|0;n=j+320|0;o=j+336|0;p=j+2640|0;q=j+3792|0;r=f+1096|0;s=g+4|0;g=h-4|0;h=g<<3;t=h>>>0>2147483639|(s|0)==0;u=t?0:s;s=t?0:h;h=r|0;c[h>>2]=u;t=f+1108|0;c[t>>2]=s;v=s+8|0;w=f+1112|0;c[w>>2]=v;x=f+1100|0;c[x>>2]=u+(s>>3);s=f+1104|0;c[s>>2]=0;if((c[f+4>>2]|0)==0){y=0}else{z=v>>>0>16?16:v;c[s>>2]=z;y=z}z=c[f+8>>2]|0;if((z|0)==2){c[(c[f+32940>>2]|0)+424>>2]=1152;A=f+24|0;B=q2((c[f+20>>2]|0)/1e3|0,c[A>>2]|0,c[f+12>>2]|0,c[f+36>>2]|0)|0;C=c[245680+(B<<2)>>2]|0;D=c[246880+(B<<2)>>2]|0;if((c[f+28>>2]|0)==1){E=(c[f+32>>2]<<2)+4|0}else{E=C}B=(E|0)>(C|0)?C:E;E=(B|0)>0;if(E){F=f+1096|0;H=c[A>>2]|0;I=(H|0)>0;J=0;K=0;while(1){L=d[D+J|0]|0;if(I){M=32-L|0;N=c[w>>2]|0;O=c[F>>2]|0;P=0;Q=c[s>>2]|0;do{R=O+(Q>>>3)|0;S=(cV(d[R]|d[R+1|0]<<8|d[R+2|0]<<16|d[R+3|0]<<24|0)|0)<<(Q&7)>>>(M>>>0);R=Q+L|0;Q=N>>>0>R>>>0?R:N;c[s>>2]=Q;a[l+(P<<5)+K|0]=S&255;P=P+1|0;}while((P|0)<(H|0))}P=(1<<L)+J|0;Q=K+1|0;if((Q|0)<(B|0)){J=P;K=Q}else{T=P;break}}}else{T=0}K=(B|0)<(C|0);if(K){J=c[w>>2]|0;H=c[f+1096>>2]|0;F=T;T=B;I=c[s>>2]|0;while(1){P=d[D+F|0]|0;Q=H+(I>>>3)|0;N=(cV(d[Q]|d[Q+1|0]<<8|d[Q+2|0]<<16|d[Q+3|0]<<24|0)|0)<<(I&7)>>>((32-P|0)>>>0);Q=P+I|0;M=J>>>0>Q>>>0?Q:J;c[s>>2]=M;Q=N&255;a[l+T|0]=Q;a[l+32+T|0]=Q;Q=T+1|0;if((Q|0)<(C|0)){F=(1<<P)+F|0;T=Q;I=M}else{break}}}I=f+1096|0;if((C|0)>0){T=c[A>>2]|0;F=(T|0)>0;J=0;while(1){if(F){H=0;do{if((a[l+(H<<5)+J|0]|0)!=0){M=c[s>>2]|0;Q=c[w>>2]|0;P=(c[I>>2]|0)+(M>>>3)|0;N=(cV(d[P]|d[P+1|0]<<8|d[P+2|0]<<16|d[P+3|0]<<24|0)|0)<<(M&7)>>>30;P=M+2|0;c[s>>2]=Q>>>0>P>>>0?P:Q;a[m+(H<<5)+J|0]=N&255}H=H+1|0;}while((H|0)<(T|0))}H=J+1|0;if((H|0)<(C|0)){J=H}else{U=0;break}}do{if(F){J=0;do{do{if((a[l+(J<<5)+U|0]|0)!=0){H=k+(J*96|0)+(U*3|0)|0;L=d[m+(J<<5)+U|0]|0;if((L|0)==2){N=c[s>>2]|0;Q=c[w>>2]|0;P=(c[I>>2]|0)+(N>>>3)|0;M=(cV(d[P]|d[P+1|0]<<8|d[P+2|0]<<16|d[P+3|0]<<24|0)|0)<<(N&7)>>>26;P=N+6|0;c[s>>2]=Q>>>0>P>>>0?P:Q;Q=M&255;a[H]=Q;a[k+(J*96|0)+(U*3|0)+1|0]=Q;a[k+(J*96|0)+(U*3|0)+2|0]=Q;break}else if((L|0)==1){Q=c[s>>2]|0;M=c[w>>2]|0;P=c[I>>2]|0;N=P+(Q>>>3)|0;O=(cV(d[N]|d[N+1|0]<<8|d[N+2|0]<<16|d[N+3|0]<<24|0)|0)<<(Q&7)>>>26;N=Q+6|0;Q=M>>>0>N>>>0?N:M;c[s>>2]=Q;N=O&255;a[H]=N;O=P+(Q>>>3)|0;P=(cV(d[O]|d[O+1|0]<<8|d[O+2|0]<<16|d[O+3|0]<<24|0)|0)<<(Q&7)>>>26;O=Q+6|0;c[s>>2]=M>>>0>O>>>0?O:M;a[k+(J*96|0)+(U*3|0)+2|0]=P&255;a[k+(J*96|0)+(U*3|0)+1|0]=N;break}else if((L|0)==3){L=c[s>>2]|0;N=c[w>>2]|0;P=c[I>>2]|0;M=P+(L>>>3)|0;O=(cV(d[M]|d[M+1|0]<<8|d[M+2|0]<<16|d[M+3|0]<<24|0)|0)<<(L&7)>>>26;M=L+6|0;L=N>>>0>M>>>0?M:N;c[s>>2]=L;a[H]=O&255;O=P+(L>>>3)|0;P=(cV(d[O]|d[O+1|0]<<8|d[O+2|0]<<16|d[O+3|0]<<24|0)|0)<<(L&7)>>>26;O=L+6|0;c[s>>2]=N>>>0>O>>>0?O:N;N=P&255;a[k+(J*96|0)+(U*3|0)+2|0]=N;a[k+(J*96|0)+(U*3|0)+1|0]=N;break}else{N=c[s>>2]|0;P=c[w>>2]|0;O=c[I>>2]|0;L=O+(N>>>3)|0;M=(cV(d[L]|d[L+1|0]<<8|d[L+2|0]<<16|d[L+3|0]<<24|0)|0)<<(N&7)>>>26;L=N+6|0;N=P>>>0>L>>>0?L:P;c[s>>2]=N;a[H]=M&255;M=O+(N>>>3)|0;H=(cV(d[M]|d[M+1|0]<<8|d[M+2|0]<<16|d[M+3|0]<<24|0)|0)<<(N&7)>>>26;M=N+6|0;N=P>>>0>M>>>0?M:P;c[s>>2]=N;a[k+(J*96|0)+(U*3|0)+1|0]=H&255;H=O+(N>>>3)|0;O=(cV(d[H]|d[H+1|0]<<8|d[H+2|0]<<16|d[H+3|0]<<24|0)|0)<<(N&7)>>>26;H=N+6|0;c[s>>2]=P>>>0>H>>>0?H:P;a[k+(J*96|0)+(U*3|0)+2|0]=O&255;break}}}while(0);J=J+1|0;}while((J|0)<(T|0))}U=U+1|0;}while((U|0)<(C|0))}U=(C|0)<32;T=0;while(1){F=T*12|0;J=0;do{if(E){O=J+F|0;P=O+1|0;H=O+2|0;N=0;M=0;L=c[A>>2]|0;while(1){Q=d[D+N|0]|0;if((L|0)>0){S=0;while(1){R=a[l+(S<<5)+M|0]|0;do{if(R<<24>>24==0){c[f+9376+(S*4608|0)+(O<<7)+(M<<2)>>2]=0;c[f+9376+(S*4608|0)+(P<<7)+(M<<2)>>2]=0;c[f+9376+(S*4608|0)+(H<<7)+(M<<2)>>2]=0}else{V=d[k+(S*96|0)+(M*3|0)+T|0]|0;W=d[D+((R&255)+N)|0]|0;X=c[245776+(W<<2)>>2]|0;if((X|0)>=0){Y=32-X|0;Z=X-1|0;_=2162456+(V<<1)|0;$=(-1<<Z)+1|0;aa=X-2|0;ab=c[s>>2]|0;ad=c[w>>2]|0;ae=(c[I>>2]|0)+(ab>>>3)|0;af=(cV(d[ae]|d[ae+1|0]<<8|d[ae+2|0]<<16|d[ae+3|0]<<24|0)|0)<<(ab&7)>>>(Y>>>0);ae=ab+X|0;c[s>>2]=ad>>>0>ae>>>0?ae:ad;ad=e[_>>1]|0;ae=af+$|0;af=c[2162272+(aa*12|0)+((ad&3)<<2)>>2]|0;ab=N2(ae,(ae|0)<0|0?-1:0,af,(af|0)<0|0?-1:0)|0;af=G;ae=(ad>>>2)+Z|0;ad=NW(1,0,ae-1|0)|0;ag=NU(ad,G,ab,af)|0;af=NY(ag|0,G|0,ae|0)|0;c[f+9376+(S*4608|0)+(O<<7)+(M<<2)>>2]=af;af=c[s>>2]|0;ae=c[w>>2]|0;ag=(c[I>>2]|0)+(af>>>3)|0;ab=(cV(d[ag]|d[ag+1|0]<<8|d[ag+2|0]<<16|d[ag+3|0]<<24|0)|0)<<(af&7)>>>(Y>>>0);ag=af+X|0;c[s>>2]=ae>>>0>ag>>>0?ag:ae;ae=e[_>>1]|0;ag=ab+$|0;ab=c[2162272+(aa*12|0)+((ae&3)<<2)>>2]|0;af=N2(ag,(ag|0)<0|0?-1:0,ab,(ab|0)<0|0?-1:0)|0;ab=G;ag=(ae>>>2)+Z|0;ae=NW(1,0,ag-1|0)|0;ad=NU(ae,G,af,ab)|0;ab=NY(ad|0,G|0,ag|0)|0;c[f+9376+(S*4608|0)+(P<<7)+(M<<2)>>2]=ab;ab=c[s>>2]|0;ag=c[w>>2]|0;ad=(c[I>>2]|0)+(ab>>>3)|0;af=(cV(d[ad]|d[ad+1|0]<<8|d[ad+2|0]<<16|d[ad+3|0]<<24|0)|0)<<(ab&7)>>>(Y>>>0);Y=ab+X|0;c[s>>2]=ag>>>0>Y>>>0?Y:ag;ag=e[_>>1]|0;_=af+$|0;$=c[2162272+(aa*12|0)+((ag&3)<<2)>>2]|0;aa=N2(_,(_|0)<0|0?-1:0,$,($|0)<0|0?-1:0)|0;$=G;_=(ag>>>2)+Z|0;Z=NW(1,0,_-1|0)|0;ag=NU(Z,G,aa,$)|0;$=NY(ag|0,G|0,_|0)|0;c[f+9376+(S*4608|0)+(H<<7)+(M<<2)>>2]=$;break}$=c[s>>2]|0;_=c[w>>2]|0;ag=(c[I>>2]|0)+($>>>3)|0;aa=(cV(d[ag]|d[ag+1|0]<<8|d[ag+2|0]<<16|d[ag+3|0]<<24|0)|0)<<($&7)>>>((X+32|0)>>>0);ag=$-X|0;c[s>>2]=_>>>0>ag>>>0?ag:_;_=b[(c[260688+(W<<2)>>2]|0)+(aa<<1)>>1]|0;aa=c[245704+(W<<2)>>2]|0;W=2162456+(V<<1)|0;V=e[W>>1]|0;ag=V>>>2;X=aa>>1;$=aa>>2;aa=ac(c[77712+($*12|0)+((V&3)<<2)>>2]|0,(_&15)-X|0)|0;if((ag|0)==0){ah=aa}else{ah=(1<<ag-1)+aa>>ag}c[f+9376+(S*4608|0)+(O<<7)+(M<<2)>>2]=ah;ag=e[W>>1]|0;aa=ag>>>2;V=ac(c[77712+($*12|0)+((ag&3)<<2)>>2]|0,(_>>>4&15)-X|0)|0;if((aa|0)==0){ai=V}else{ai=(1<<aa-1)+V>>aa}c[f+9376+(S*4608|0)+(P<<7)+(M<<2)>>2]=ai;aa=e[W>>1]|0;W=aa>>>2;V=ac(c[77712+($*12|0)+((aa&3)<<2)>>2]|0,(_>>8)-X|0)|0;if((W|0)==0){aj=V}else{aj=(1<<W-1)+V>>W}c[f+9376+(S*4608|0)+(H<<7)+(M<<2)>>2]=aj}}while(0);R=S+1|0;W=c[A>>2]|0;if((R|0)<(W|0)){S=R}else{ak=W;break}}}else{ak=L}S=(1<<Q)+N|0;W=M+1|0;if((W|0)<(B|0)){N=S;M=W;L=ak}else{al=S;break}}}else{al=0}if(K){L=J+F|0;M=L+1|0;N=L+2|0;H=al;P=B;while(1){O=d[D+H|0]|0;S=a[l+P|0]|0;L21859:do{if(S<<24>>24==0){c[f+9376+(L<<7)+(P<<2)>>2]=0;c[f+9376+(M<<7)+(P<<2)>>2]=0;c[f+9376+(N<<7)+(P<<2)>>2]=0;c[f+13984+(L<<7)+(P<<2)>>2]=0;c[f+13984+(M<<7)+(P<<2)>>2]=0;c[f+13984+(N<<7)+(P<<2)>>2]=0}else{W=d[k+(P*3|0)+T|0]|0;R=d[k+96+(P*3|0)+T|0]|0;V=d[D+((S&255)+H)|0]|0;X=c[245776+(V<<2)>>2]|0;if((X|0)>=0){_=32-X|0;aa=X-1|0;$=2162456+(W<<1)|0;ag=(-1<<aa)+1|0;Z=X-2|0;af=2162456+(R<<1)|0;Y=0;while(1){ab=c[s>>2]|0;ad=c[w>>2]|0;ae=(c[I>>2]|0)+(ab>>>3)|0;am=(cV(d[ae]|d[ae+1|0]<<8|d[ae+2|0]<<16|d[ae+3|0]<<24|0)|0)<<(ab&7)>>>(_>>>0);ae=ab+X|0;c[s>>2]=ad>>>0>ae>>>0?ae:ad;ad=e[$>>1]|0;ae=ag+am|0;am=ae;ab=(ae|0)<0|0?-1:0;ae=c[2162272+(Z*12|0)+((ad&3)<<2)>>2]|0;an=N2(am,ab,ae,(ae|0)<0|0?-1:0)|0;ae=G;ao=(ad>>>2)+aa|0;ad=NW(1,0,ao-1|0)|0;ap=NU(ad,G,an,ae)|0;ae=NY(ap|0,G|0,ao|0)|0;ao=Y+L|0;c[f+9376+(ao<<7)+(P<<2)>>2]=ae;ae=e[af>>1]|0;ap=c[2162272+(Z*12|0)+((ae&3)<<2)>>2]|0;an=N2(ap,(ap|0)<0|0?-1:0,am,ab)|0;ab=G;am=(ae>>>2)+aa|0;ae=NW(1,0,am-1|0)|0;ap=NU(ae,G,an,ab)|0;ab=NY(ap|0,G|0,am|0)|0;c[f+13984+(ao<<7)+(P<<2)>>2]=ab;Y=Y+1|0;if((Y|0)>=3){break L21859}}}Y=c[s>>2]|0;aa=c[w>>2]|0;Z=(c[I>>2]|0)+(Y>>>3)|0;af=(cV(d[Z]|d[Z+1|0]<<8|d[Z+2|0]<<16|d[Z+3|0]<<24|0)|0)<<(Y&7)>>>((X+32|0)>>>0);Z=Y-X|0;c[s>>2]=aa>>>0>Z>>>0?Z:aa;aa=c[245704+(V<<2)>>2]|0;Z=(af|0)/(aa|0)|0;Y=2162456+(W<<1)|0;ag=e[Y>>1]|0;$=ag>>>2;_=aa>>1;ab=((af|0)%(aa|0)|0)-_|0;af=aa>>2;ao=ac(c[77712+(af*12|0)+((ag&3)<<2)>>2]|0,ab)|0;if(($|0)==0){aq=ao}else{aq=(1<<$-1)+ao>>$}c[f+9376+(L<<7)+(P<<2)>>2]=aq;$=2162456+(R<<1)|0;ao=e[$>>1]|0;ag=ao>>>2;am=ac(c[77712+(af*12|0)+((ao&3)<<2)>>2]|0,ab)|0;if((ag|0)==0){ar=am}else{ar=(1<<ag-1)+am>>ag}c[f+13984+(L<<7)+(P<<2)>>2]=ar;ag=e[Y>>1]|0;am=ag>>>2;ab=((Z|0)%(aa|0)|0)-_|0;ao=ac(c[77712+(af*12|0)+((ag&3)<<2)>>2]|0,ab)|0;if((am|0)==0){as=ao}else{as=(1<<am-1)+ao>>am}c[f+9376+(M<<7)+(P<<2)>>2]=as;am=e[$>>1]|0;ao=am>>>2;ag=ac(c[77712+(af*12|0)+((am&3)<<2)>>2]|0,ab)|0;if((ao|0)==0){at=ag}else{at=(1<<ao-1)+ag>>ao}c[f+13984+(M<<7)+(P<<2)>>2]=at;ao=e[Y>>1]|0;Y=ao>>>2;ag=((Z|0)/(aa|0)|0)-_|0;_=ac(c[77712+(af*12|0)+((ao&3)<<2)>>2]|0,ag)|0;if((Y|0)==0){au=_}else{au=(1<<Y-1)+_>>Y}c[f+9376+(N<<7)+(P<<2)>>2]=au;Y=e[$>>1]|0;$=Y>>>2;_=ac(c[77712+(af*12|0)+((Y&3)<<2)>>2]|0,ag)|0;if(($|0)==0){av=_}else{av=(1<<$-1)+_>>$}c[f+13984+(N<<7)+(P<<2)>>2]=av}}while(0);S=P+1|0;if((S|0)<(C|0)){H=(1<<O)+H|0;P=S}else{break}}}if(U){P=J+F|0;H=P+1|0;N=P+2|0;M=C;L=c[A>>2]|0;while(1){if((L|0)>0){S=0;while(1){c[f+9376+(S*4608|0)+(P<<7)+(M<<2)>>2]=0;c[f+9376+(S*4608|0)+(H<<7)+(M<<2)>>2]=0;c[f+9376+(S*4608|0)+(N<<7)+(M<<2)>>2]=0;Q=S+1|0;$=c[A>>2]|0;if((Q|0)<($|0)){S=Q}else{aw=$;break}}}else{aw=L}S=M+1|0;if((S|0)<32){M=S;L=aw}else{break}}}J=J+3|0;}while((J|0)<12);J=T+1|0;if((J|0)<3){T=J}else{ax=36;break}}}else if((z|0)==1){c[(c[f+32940>>2]|0)+424>>2]=384;do{if((c[f+28>>2]|0)==1){T=(c[f+32>>2]<<2)+4|0;if((T|0)>0){ay=T;az=16231;break}aA=0;aB=T;aC=f+1096|0;az=16235}else{ay=32;az=16231}}while(0);if((az|0)==16231){T=f+1096|0;aw=c[f+24>>2]|0;A=(aw|0)>0;C=0;do{if(A){U=c[w>>2]|0;av=c[T>>2]|0;au=0;at=c[s>>2]|0;do{as=av+(at>>>3)|0;ar=(cV(d[as]|d[as+1|0]<<8|d[as+2|0]<<16|d[as+3|0]<<24|0)|0)<<(at&7)>>>28;as=at+4|0;at=U>>>0>as>>>0?as:U;c[s>>2]=at;a[l+(au<<5)+C|0]=ar&255;au=au+1|0;}while((au|0)<(aw|0))}C=C+1|0;}while((C|0)<(ay|0));if((ay|0)<32){aA=1;aB=ay;aC=T;az=16235}else{aD=0;aE=ay;aF=aw;az=16241}}do{if((az|0)==16235){aw=c[w>>2]|0;ay=c[aC>>2]|0;T=aB;C=c[s>>2]|0;do{A=ay+(C>>>3)|0;au=(cV(d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24|0)|0)<<(C&7)>>>28;A=C+4|0;C=aw>>>0>A>>>0?A:aw;c[s>>2]=C;a[l+T|0]=au&255;T=T+1|0;}while((T|0)<32);if(!aA){aG=0;aH=aB;aI=aC;az=16240;break}aD=1;aE=aB;aF=c[f+24>>2]|0;az=16241}}while(0);if((az|0)==16241){aB=f+1096|0;aC=(aF|0)>0;aA=0;do{if(aC){T=0;do{if((a[l+(T<<5)+aA|0]|0)!=0){C=c[s>>2]|0;aw=c[w>>2]|0;ay=(c[aB>>2]|0)+(C>>>3)|0;au=(cV(d[ay]|d[ay+1|0]<<8|d[ay+2|0]<<16|d[ay+3|0]<<24|0)|0)<<(C&7)>>>26;ay=C+6|0;c[s>>2]=aw>>>0>ay>>>0?ay:aw;a[m+(T<<5)+aA|0]=au&255}T=T+1|0;}while((T|0)<(aF|0))}aA=aA+1|0;}while((aA|0)<(aE|0));if(aD){aG=1;aH=aE;aI=aB;az=16240}else{aJ=1;aK=aE;aL=0;aM=aB}}if((az|0)==16240){aB=aH;while(1){if((a[l+aB|0]|0)!=0){aE=c[s>>2]|0;aD=c[w>>2]|0;aA=c[aI>>2]|0;aF=aA+(aE>>>3)|0;aC=(cV(d[aF]|d[aF+1|0]<<8|d[aF+2|0]<<16|d[aF+3|0]<<24|0)|0)<<(aE&7)>>>26;aF=aE+6|0;aE=aD>>>0>aF>>>0?aF:aD;c[s>>2]=aE;a[m+aB|0]=aC&255;aC=aA+(aE>>>3)|0;aA=(cV(d[aC]|d[aC+1|0]<<8|d[aC+2|0]<<16|d[aC+3|0]<<24|0)|0)<<(aE&7)>>>26;aC=aE+6|0;c[s>>2]=aD>>>0>aC>>>0?aC:aD;a[m+32+aB|0]=aA&255}aA=aB+1|0;if((aA|0)<32){aB=aA}else{aJ=aG;aK=aH;aL=1;aM=aI;break}}}aI=f+24|0;aH=0;while(1){if(aJ){aG=0;aB=c[aI>>2]|0;while(1){if((aB|0)>0){aA=0;while(1){aD=a[l+(aA<<5)+aG|0]|0;aC=aD&255;if(aD<<24>>24==0){aN=0}else{aD=c[s>>2]|0;aE=c[w>>2]|0;aF=(c[aM>>2]|0)+(aD>>>3)|0;T=(cV(d[aF]|d[aF+1|0]<<8|d[aF+2|0]<<16|d[aF+3|0]<<24|0)|0)<<(aD&7)>>>((31-aC|0)>>>0);aF=aC+1+aD|0;c[s>>2]=aE>>>0>aF>>>0?aF:aE;aE=e[2162456+(d[m+(aA<<5)+aG|0]<<1)>>1]|0;aF=(-1<<aC)+1+T|0;T=c[2162272+((aC-1|0)*12|0)+((aE&3)<<2)>>2]|0;aD=N2(T,(T|0)<0|0?-1:0,aF,(aF|0)<0|0?-1:0)|0;aF=G;T=(aE>>>2)+aC|0;aC=NW(1,0,T-1|0)|0;aE=NU(aC,G,aD,aF)|0;aF=NY(aE|0,G|0,T|0)|0;aN=aF}c[f+9376+(aA*4608|0)+(aH<<7)+(aG<<2)>>2]=aN;aF=aA+1|0;T=c[aI>>2]|0;if((aF|0)<(T|0)){aA=aF}else{aO=T;break}}}else{aO=aB}aA=aG+1|0;if((aA|0)<(aK|0)){aG=aA;aB=aO}else{break}}}if(aL){aB=aK;do{aG=a[l+aB|0]|0;aA=aG&255;if(aG<<24>>24==0){c[f+9376+(aH<<7)+(aB<<2)>>2]=0;c[f+13984+(aH<<7)+(aB<<2)>>2]=0}else{aG=c[s>>2]|0;T=c[w>>2]|0;aF=(c[aM>>2]|0)+(aG>>>3)|0;aE=(cV(d[aF]|d[aF+1|0]<<8|d[aF+2|0]<<16|d[aF+3|0]<<24|0)|0)<<(aG&7)>>>((31-aA|0)>>>0);aF=aA+1+aG|0;c[s>>2]=T>>>0>aF>>>0?aF:T;T=e[2162456+(d[m+aB|0]<<1)>>1]|0;aF=(-1<<aA)+1+aE|0;aE=aF;aG=(aF|0)<0|0?-1:0;aF=aA-1|0;aD=c[2162272+(aF*12|0)+((T&3)<<2)>>2]|0;aC=N2(aD,(aD|0)<0|0?-1:0,aE,aG)|0;aD=G;au=(T>>>2)+aA|0;T=NW(1,0,au-1|0)|0;aw=NU(T,G,aC,aD)|0;aD=NY(aw|0,G|0,au|0)|0;c[f+9376+(aH<<7)+(aB<<2)>>2]=aD;aD=e[2162456+(d[m+32+aB|0]<<1)>>1]|0;au=c[2162272+(aF*12|0)+((aD&3)<<2)>>2]|0;aF=N2(au,(au|0)<0|0?-1:0,aE,aG)|0;aG=G;aE=(aD>>>2)+aA|0;aA=NW(1,0,aE-1|0)|0;aD=NU(aA,G,aF,aG)|0;aG=NY(aD|0,G|0,aE|0)|0;c[f+13984+(aH<<7)+(aB<<2)>>2]=aG}aB=aB+1|0;}while((aB|0)<32)}aB=aH+1|0;if((aB|0)<12){aH=aB}else{ax=12;break}}}else if((z|0)==3){z=f+36|0;c[(c[f+32940>>2]|0)+424>>2]=(c[z>>2]|0)!=0?576:1152;aP=c[s>>2]|0;aQ=c[w>>2]|0;aR=c[h>>2]|0;aS=z;az=16340}else{aP=y;aQ=v;aR=u;aS=f+36|0;az=16340}do{if((az|0)==16340){u=(c[aS>>2]|0)==0;v=aR+(aP>>>3)|0;y=(cV(d[v]|d[v+1|0]<<8|d[v+2|0]<<16|d[v+3|0]<<24|0)|0)<<(aP&7);do{if(u){v=y>>>23;z=aP+9|0;aH=aQ>>>0>z>>>0?z:aQ;c[s>>2]=aH;z=c[f+24>>2]|0;if((z|0)==2){m=aH+3|0;aM=aQ>>>0>m>>>0?m:aQ;c[s>>2]=aM;aT=0;aU=aM}else{aM=aH+5|0;aH=aQ>>>0>aM>>>0?aM:aQ;c[s>>2]=aH;if((z|0)>0){aT=0;aU=aH}else{aV=2;aW=v;aX=z;break}}while(1){a[f+23200+(aT*4864|0)|0]=0;aH=aR+(aU>>>3)|0;aM=(cV(d[aH]|d[aH+1|0]<<8|d[aH+2|0]<<16|d[aH+3|0]<<24|0)|0)<<(aU&7)>>>28;aH=aU+4|0;m=aQ>>>0>aH>>>0?aH:aQ;c[s>>2]=m;a[f+23200+(aT*4864|0)+2432|0]=aM&255;aM=aT+1|0;if((aM|0)<(z|0)){aT=aM;aU=m}else{aV=2;aW=v;aX=z;break}}}else{z=aP+8|0;v=c[f+24>>2]|0;m=v+(aQ>>>0>z>>>0?z:aQ)|0;c[s>>2]=aQ>>>0>m>>>0?m:aQ;aV=1;aW=y>>>24;aX=v}}while(0);y=f+24|0;u=f+32|0;v=f+16|0;m=f+32940|0;z=0;aM=aX;aH=aQ;l=aR;L21972:while(1){if((aM|0)>0){aK=0;aL=aH;aO=l;while(1){aI=c[s>>2]|0;aN=aO+(aI>>>3)|0;aJ=(cV(d[aN]|d[aN+1|0]<<8|d[aN+2|0]<<16|d[aN+3|0]<<24|0)|0)<<(aI&7)>>>20;aN=aI+12|0;aI=aL>>>0>aN>>>0?aN:aL;c[s>>2]=aI;c[f+23200+(aK*4864|0)+(z*2432|0)+4>>2]=aJ;aJ=aO+(aI>>>3)|0;aN=(cV(d[aJ]|d[aJ+1|0]<<8|d[aJ+2|0]<<16|d[aJ+3|0]<<24|0)|0)<<(aI&7);aJ=aN>>>23;aB=aI+9|0;aI=aL>>>0>aB>>>0?aB:aL;c[s>>2]=aI;c[f+23200+(aK*4864|0)+(z*2432|0)+8>>2]=aJ;if(aN>>>0>2424307711){az=16349;break L21972}aN=aO+(aI>>>3)|0;aB=(cV(d[aN]|d[aN+1|0]<<8|d[aN+2|0]<<16|d[aN+3|0]<<24|0)|0)<<(aI&7)>>>24;aN=aI+8|0;aI=aL>>>0>aN>>>0?aN:aL;c[s>>2]=aI;aN=f+23200+(aK*4864|0)+(z*2432|0)+12|0;c[aN>>2]=aB;if((c[u>>2]&3|0)==2){c[aN>>2]=aB-2}aB=c[aS>>2]|0;aN=aO+(aI>>>3)|0;aG=(cV(d[aN]|d[aN+1|0]<<8|d[aN+2|0]<<16|d[aN+3|0]<<24|0)|0)<<(aI&7);if((aB|0)==0){aN=aI+4|0;aE=aL>>>0>aN>>>0?aN:aL;c[s>>2]=aE;c[f+23200+(aK*4864|0)+(z*2432|0)+16>>2]=aG>>>28;aY=aE}else{aE=aI+9|0;aI=aL>>>0>aE>>>0?aE:aL;c[s>>2]=aI;c[f+23200+(aK*4864|0)+(z*2432|0)+16>>2]=aG>>>23;aY=aI}aI=d[aO+(aY>>>3)|0]|0;aG=((aY|0)<(aL|0))+aY|0;c[s>>2]=aG;if((aI&128>>>((aY&7)>>>0)|0)==0){a[f+23200+(aK*4864|0)+(z*2432|0)+20|0]=0;a[f+23200+(aK*4864|0)+(z*2432|0)+21|0]=0;aI=aO+(aG>>>3)|0;aE=(cV(d[aI]|d[aI+1|0]<<8|d[aI+2|0]<<16|d[aI+3|0]<<24|0)|0)<<(aG&7)>>>27;aI=aG+5|0;aN=aL>>>0>aI>>>0?aI:aL;c[s>>2]=aN;c[f+23200+(aK*4864|0)+(z*2432|0)+24>>2]=aE;aE=aO+(aN>>>3)|0;aI=(cV(d[aE]|d[aE+1|0]<<8|d[aE+2|0]<<16|d[aE+3|0]<<24|0)|0)<<(aN&7)>>>27;aE=aN+5|0;aN=aL>>>0>aE>>>0?aE:aL;c[s>>2]=aN;c[f+23200+(aK*4864|0)+(z*2432|0)+28>>2]=aI;aI=aO+(aN>>>3)|0;aE=(cV(d[aI]|d[aI+1|0]<<8|d[aI+2|0]<<16|d[aI+3|0]<<24|0)|0)<<(aN&7)>>>27;aI=aN+5|0;aN=aL>>>0>aI>>>0?aI:aL;c[s>>2]=aN;c[f+23200+(aK*4864|0)+(z*2432|0)+32>>2]=aE;aE=aO+(aN>>>3)|0;aI=(cV(d[aE]|d[aE+1|0]<<8|d[aE+2|0]<<16|d[aE+3|0]<<24|0)|0)<<(aN&7)>>>28;aE=aN+4|0;aN=aL>>>0>aE>>>0?aE:aL;c[s>>2]=aN;aE=aO+(aN>>>3)|0;aD=(cV(d[aE]|d[aE+1|0]<<8|d[aE+2|0]<<16|d[aE+3|0]<<24|0)|0)<<(aN&7)>>>29;aE=aN+3|0;c[s>>2]=aL>>>0>aE>>>0?aE:aL;aE=c[v>>2]|0;aN=(e[3014136+(aE*46|0)+(aI+1<<1)>>1]|0)>>>1;aF=f+23200+(aK*4864|0)+(z*2432|0)+52|0;c[aF>>2]=aN;aA=aI+2+aD|0;aD=(e[3014136+(aE*46|0)+(((aA|0)>22?22:aA)<<1)>>1]|0)>>>1;c[f+23200+(aK*4864|0)+(z*2432|0)+56>>2]=aD;aZ=aN;a_=aD;a$=0;a0=0;a1=aE;a2=aF}else{aF=aO+(aG>>>3)|0;aE=(cV(d[aF]|d[aF+1|0]<<8|d[aF+2|0]<<16|d[aF+3|0]<<24|0)|0)<<(aG&7)>>>30;aF=aG+2|0;aG=aL>>>0>aF>>>0?aF:aL;c[s>>2]=aG;aF=aE&255;a[f+23200+(aK*4864|0)+(z*2432|0)+20|0]=aF;if((aE|0)==0){az=16357;break L21972}aE=d[aO+(aG>>>3)|0]<<(aG&7)>>>7;aD=((aG|0)<(aL|0))+aG|0;c[s>>2]=aD;aG=aE&1;a[f+23200+(aK*4864|0)+(z*2432|0)+21|0]=aG;aE=aO+(aD>>>3)|0;aN=(cV(d[aE]|d[aE+1|0]<<8|d[aE+2|0]<<16|d[aE+3|0]<<24|0)|0)<<(aD&7)>>>27;aE=aD+5|0;aD=aL>>>0>aE>>>0?aE:aL;c[s>>2]=aD;c[f+23200+(aK*4864|0)+(z*2432|0)+24>>2]=aN;aN=aO+(aD>>>3)|0;aE=(cV(d[aN]|d[aN+1|0]<<8|d[aN+2|0]<<16|d[aN+3|0]<<24|0)|0)<<(aD&7)>>>27;aN=aD+5|0;aD=aL>>>0>aN>>>0?aN:aL;c[s>>2]=aD;c[f+23200+(aK*4864|0)+(z*2432|0)+28>>2]=aE;aE=aO+(aD>>>3)|0;aN=(cV(d[aE]|d[aE+1|0]<<8|d[aE+2|0]<<16|d[aE+3|0]<<24|0)|0)<<(aD&7)>>>29;aE=aD+3|0;aD=aL>>>0>aE>>>0?aE:aL;c[s>>2]=aD;c[f+23200+(aK*4864|0)+(z*2432|0)+36>>2]=aN;aN=aO+(aD>>>3)|0;aE=(cV(d[aN]|d[aN+1|0]<<8|d[aN+2|0]<<16|d[aN+3|0]<<24|0)|0)<<(aD&7)>>>29;aN=aD+3|0;aD=aL>>>0>aN>>>0?aN:aL;c[s>>2]=aD;c[f+23200+(aK*4864|0)+(z*2432|0)+40>>2]=aE;aE=(c[h>>2]|0)+(aD>>>3)|0;aN=(cV(d[aE]|d[aE+1|0]<<8|d[aE+2|0]<<16|d[aE+3|0]<<24|0)|0)<<(aD&7)>>>29;aE=aD+3|0;c[s>>2]=aL>>>0>aE>>>0?aE:aL;c[f+23200+(aK*4864|0)+(z*2432|0)+44>>2]=aN;aN=c[v>>2]|0;aE=aF<<24>>24==2;do{if(aE){aF=f+23200+(aK*4864|0)+(z*2432|0)+52|0;if((aN|0)==8){c[aF>>2]=36;a3=36;break}else{c[aF>>2]=18;a3=18;break}}else{if((aN|0)<3){c[f+23200+(aK*4864|0)+(z*2432|0)+52>>2]=18;a3=18;break}aF=f+23200+(aK*4864|0)+(z*2432|0)+52|0;if((aN|0)==8){c[aF>>2]=54;a3=54;break}else{c[aF>>2]=27;a3=27;break}}}while(0);c[f+23200+(aK*4864|0)+(z*2432|0)+56>>2]=288;aZ=a3;a_=288;a$=aE;a0=aG;a1=aN;a2=f+23200+(aK*4864|0)+(z*2432|0)+52|0}aF=(aZ|0)>(aJ|0)?aJ:aZ;c[a2>>2]=aF;aD=a_>>>0>aJ>>>0?aJ:a_;c[f+23200+(aK*4864|0)+(z*2432|0)+56>>2]=aD-aF;c[f+23200+(aK*4864|0)+(z*2432|0)+60>>2]=(aJ>>>0<288?aJ:288)-aD;do{if(a$){if(a0<<24>>24==0){c[f+23200+(aK*4864|0)+(z*2432|0)+72>>2]=0;c[f+23200+(aK*4864|0)+(z*2432|0)+68>>2]=0;a4=aB;break}if((a1|0)==8){AS(c[m>>2]|0,1711944,(a5=i,i=i+1|0,i=i+7&-8,c[a5>>2]=0,a5)|0);i=a5;a6=c[v>>2]|0;a7=c[aS>>2]|0}else{a6=a1;a7=aB}c[f+23200+(aK*4864|0)+(z*2432|0)+72>>2]=(a6|0)<3?8:6;c[f+23200+(aK*4864|0)+(z*2432|0)+68>>2]=3;a4=a7}else{c[f+23200+(aK*4864|0)+(z*2432|0)+68>>2]=13;c[f+23200+(aK*4864|0)+(z*2432|0)+72>>2]=22;a4=aB}}while(0);aB=f+23200+(aK*4864|0)+(z*2432|0)+64|0;c[aB>>2]=0;aJ=c[s>>2]|0;if((a4|0)==0){aN=c[h>>2]|0;aG=d[aN+(aJ>>>3)|0]<<(aJ&7)>>>7;aE=c[w>>2]|0;aD=((aJ|0)<(aE|0))+aJ|0;c[s>>2]=aD;c[aB>>2]=aG&1;a8=aD;a9=aN;ba=aE}else{a8=aJ;a9=c[h>>2]|0;ba=c[w>>2]|0}aJ=d[a9+(a8>>>3)|0]<<(a8&7)>>>7;c[s>>2]=((a8|0)<(ba|0))+a8;a[f+23200+(aK*4864|0)+(z*2432|0)+48|0]=aJ&1;aJ=c[s>>2]|0;aE=d[a9+(aJ>>>3)|0]<<(aJ&7)>>>7;c[s>>2]=((aJ|0)<(ba|0))+aJ;a[f+23200+(aK*4864|0)+(z*2432|0)+49|0]=aE&1;aE=aK+1|0;aJ=c[y>>2]|0;if((aE|0)<(aJ|0)){aK=aE;aL=ba;aO=a9}else{bb=aJ;bc=ba;bd=a9;break}}}else{bb=aM;bc=aH;bd=l}aO=z+1|0;if((aO|0)<(aV|0)){z=aO;aM=bb;aH=bc;l=bd}else{az=16381;break}}if((az|0)==16357){AN(c[m>>2]|0,16,1722784,(a5=i,i=i+1|0,i=i+7&-8,c[a5>>2]=0,a5)|0);i=a5;be=-1094995529}else if((az|0)==16381){do{if((c[f+32928>>2]|0)==0){l=c[s>>2]|0;aH=bd+(l>>3)|0;aM=(c[t>>2]|0)-l>>3;if((aM|0)<0){bf=0}else{bf=(aM|0)>24?24:aM}aM=f+40|0;l=f+1088|0;z=(c[l>>2]|0)+(f+40)|0;NN(z|0,aH|0,bf)|0;aH=f+1116|0;z=aH;aO=r;c[z>>2]=c[aO>>2];c[z+4>>2]=c[aO+4>>2];c[z+8>>2]=c[aO+8>>2];c[z+12>>2]=c[aO+12>>2];c[z+16>>2]=c[aO+16>>2];aL=c[l>>2]|0;aK=aL<<3;aJ=aK>>>0>2147483639|(aM|0)==0;aE=aJ?0:aM;aM=aJ?0:aK;c[h>>2]=aE;c[t>>2]=aM;c[x>>2]=aE+(aM>>3);c[s>>2]=0;aE=1048-aL|0;c[w>>2]=aM+8+(((bf|0)>(aE|0)?bf:aE)<<3);c[l>>2]=aK;aE=0;aM=aK;while(1){if((aM>>3|0)>=(aW|0)){bg=aE;bh=aM;break}if((c[y>>2]|0)>0){aK=aE*18|0;aL=0;aJ=aM;while(1){c[l>>2]=(c[f+23200+(aL*4864|0)+(aE*2432|0)+4>>2]|0)+aJ;NM(f+23200+(aL*4864|0)+(aE*2432|0)+128|0,0,2304);oi(f,f+23200+(aL*4864|0)+(aE*2432|0)|0,f+9376+(aL*4608|0)+(aK<<7)|0,f+18592+(aL*2304|0)|0);aN=aL+1|0;aD=c[l>>2]|0;if((aN|0)<(c[y>>2]|0)){aL=aN;aJ=aD}else{bi=aD;break}}}else{bi=aM}aJ=aE+1|0;if((aJ|0)<(aV|0)){aE=aJ;aM=bi}else{bg=aJ;bh=bi;break}}aM=bh-(aW<<3)|0;aE=c[t>>2]|0;do{if((aM|0)<(aE|0)){az=16396}else{l=aH|0;if((c[l>>2]|0)==0){az=16396;break}aJ=aM-aE|0;aL=f+1124|0;aK=c[aL>>2]|0;aD=-aK|0;aN=(c[f+1132>>2]|0)-aK|0;if((aJ|0)<(aD|0)){bj=aD}else{bj=(aN|0)<(aJ|0)?aN:aJ}c[aL>>2]=bj+aK;c[aO>>2]=c[z>>2];c[aO+4>>2]=c[z+4>>2];c[aO+8>>2]=c[z+8>>2];c[aO+12>>2]=c[z+12>>2];c[aO+16>>2]=c[z+16>>2];c[l>>2]=0}}while(0);if((az|0)==16396){aE=c[s>>2]|0;l=-aE|0;aK=(c[w>>2]|0)-aE|0;if((aM|0)<(l|0)){bk=l}else{bk=(aK|0)<(aM|0)?aK:aM}c[s>>2]=bk+aE}if((bg|0)>=(aV|0)){break}bl=bg;bm=c[y>>2]|0;bn=aH;bo=aO;bp=z;az=16401}else{aE=f+1116|0;bl=0;bm=bb;bn=aE;bo=r;bp=aE;az=16401}}while(0);if((az|0)==16401){aE=f+28|0;aK=n|0;l=n+4|0;aL=n+8|0;aJ=o;aN=o|0;aD=p|0;aG=bn|0;aB=f+32936|0;aF=q|0;aA=q+12|0;aI=q+8|0;au=q+4|0;aw=bl;aC=0;T=bm;while(1){ay=aC+bl|0;if((T|0)>0){C=0;A=c[s>>2]|0;while(1){at=f+23200+(C*4864|0)+(ay*2432|0)+76|0;do{if((c[aS>>2]|0)==0){U=c[f+23200+(C*4864|0)+(aw*2432|0)+16>>2]|0;av=a[73184+U|0]|0;ar=av&255;as=a[73200+U|0]|0;U=as&255;if((a[f+23200+(C*4864|0)+(aw*2432|0)+20|0]|0)==2){aq=(a[f+23200+(C*4864|0)+(aw*2432|0)+21|0]|0)!=0?17:18;L22111:do{if(av<<24>>24==0){NM(at|0,0,aq|0)}else{I=32-ar|0;D=0;k=A;while(1){B=c[w>>2]|0;al=(c[h>>2]|0)+(k>>>3)|0;K=(cV(d[al]|d[al+1|0]<<8|d[al+2|0]<<16|d[al+3|0]<<24|0)|0)<<(k&7)>>>(I>>>0);al=k+ar|0;c[s>>2]=B>>>0>al>>>0?al:B;B=D+1|0;a[f+23200+(C*4864|0)+(aw*2432|0)+76+D|0]=K&255;if((B|0)>=(aq|0)){break L22111}D=B;k=c[s>>2]|0}}}while(0);if(as<<24>>24==0){NM(f+23200+(C*4864|0)+(ay*2432|0)+76+aq|0,0,21);break}av=32-U|0;R=0;W=aq;while(1){V=c[s>>2]|0;X=c[w>>2]|0;k=(c[h>>2]|0)+(V>>>3)|0;D=(cV(d[k]|d[k+1|0]<<8|d[k+2|0]<<16|d[k+3|0]<<24|0)|0)<<(V&7)>>>(av>>>0);k=V+U|0;c[s>>2]=X>>>0>k>>>0?k:X;a[f+23200+(C*4864|0)+(aw*2432|0)+76+W|0]=D&255;D=R+1|0;if((D|0)<18){R=D;W=W+1|0}else{break}}NM(aq+18+(f+23200+(C*4864|0)+(ay*2432|0)+76)|0,0,3);break}else{W=f+23200+(C*4864|0)+(aw*2432|0)|0;R=0;av=0;while(1){as=(av|0)==0?6:5;L22098:do{if((d[W]&8>>>(av>>>0)|0)==0){D=(av|0)<2?ar:U;if((D|0)==0){NM(f+23200+(C*4864|0)+(ay*2432|0)+76+R|0,0,as|0);break}X=32-D|0;k=1;V=R;while(1){I=c[s>>2]|0;B=c[w>>2]|0;K=(c[h>>2]|0)+(I>>>3)|0;al=(cV(d[K]|d[K+1|0]<<8|d[K+2|0]<<16|d[K+3|0]<<24|0)|0)<<(I&7)>>>(X>>>0);K=I+D|0;c[s>>2]=B>>>0>K>>>0?K:B;a[f+23200+(C*4864|0)+(aw*2432|0)+76+V|0]=al&255;if((k|0)>=(as|0)){break L22098}k=k+1|0;V=V+1|0}}else{V=0;k=R;while(1){a[f+23200+(C*4864|0)+(aw*2432|0)+76+k|0]=a[f+23200+(C*4864|0)+76+k|0]|0;D=V+1|0;if((D|0)<(as|0)){V=D;k=k+1|0}else{break}}}}while(0);k=av+1|0;if((k|0)<4){R=as+R|0;av=k}else{break}}a[f+23200+(C*4864|0)+(aw*2432|0)+97|0]=0;break}}else{if((a[f+23200+(C*4864|0)+(aw*2432|0)+20|0]|0)==2){bq=(a[f+23200+(C*4864|0)+(aw*2432|0)+21|0]|0)!=0?2:1}else{bq=0}av=c[f+23200+(C*4864|0)+(aw*2432|0)+16>>2]|0;do{if((c[u>>2]&1|0)!=0&(C|0)==1){R=av>>1;if((R|0)<180){c[aA>>2]=0;U=(R*171|0)>>10;c[aI>>2]=(U*-6|0)+R;ar=(U*171|0)>>10;c[au>>2]=(ar*-6|0)+U;c[aF>>2]=ar;br=3;bs=ar;break}if((R|0)<244){ar=R-180|0;c[aA>>2]=0;c[aI>>2]=ar&3;c[au>>2]=ar>>>2&3;U=ar>>4;c[aF>>2]=U;br=4;bs=U;break}else{U=R-244|0;c[aA>>2]=0;c[aI>>2]=0;R=(U*171|0)>>9;c[au>>2]=(R*-3|0)+U;c[aF>>2]=R;br=5;bs=R;break}}else{if((av|0)<400){c[aA>>2]=av&3;c[aI>>2]=av>>>2&3;R=av>>4;U=(R*205|0)>>10;c[au>>2]=(U*-5|0)+R;c[aF>>2]=U;br=0;bs=U;break}if((av|0)<500){U=av-400|0;c[aA>>2]=0;c[aI>>2]=U&3;R=U>>2;U=(R*205|0)>>10;c[au>>2]=(U*-5|0)+R;c[aF>>2]=U;br=1;bs=U;break}else{U=av-500|0;c[aA>>2]=0;c[aI>>2]=0;R=(U*171|0)>>9;c[au>>2]=(R*-3|0)+U;c[aF>>2]=R;c[f+23200+(C*4864|0)+(aw*2432|0)+64>>2]=1;br=2;bs=R;break}}}while(0);av=0;R=0;U=bs;while(1){ar=a[130760+(br*12|0)+(bq<<2)+R|0]|0;W=ar&255;aq=ar<<24>>24==0;do{if((U|0)==0){if(aq){bt=av;break}ar=W>>>0>1?W:1;NM(f+23200+(C*4864|0)+(ay*2432|0)+76+av|0,0,ar|0);bt=ar+av|0}else{if(aq){bt=av;break}ar=32-U|0;k=W>>>0>1;V=1;D=av;while(1){X=c[s>>2]|0;al=c[w>>2]|0;B=(c[h>>2]|0)+(X>>>3)|0;K=(cV(d[B]|d[B+1|0]<<8|d[B+2|0]<<16|d[B+3|0]<<24|0)|0)<<(X&7)>>>(ar>>>0);B=X+U|0;c[s>>2]=al>>>0>B>>>0?B:al;a[f+23200+(C*4864|0)+(aw*2432|0)+76+D|0]=K&255;if((V|0)>=(W|0)){break}V=V+1|0;D=D+1|0}bt=(k?W:1)+av|0}}while(0);W=R+1|0;if((W|0)>=4){break}av=bt;R=W;U=c[q+(W<<2)>>2]|0}if((bt|0)>=40){break}NM(f+23200+(C*4864|0)+(ay*2432|0)+76+bt|0,0,40-bt|0)}}while(0);at=c[f+23200+(C*4864|0)+(aw*2432|0)+12>>2]|0;U=at-210|0;R=(d[f+23200+(C*4864|0)+(aw*2432|0)+48|0]|0)+1|0;av=c[v>>2]|0;W=c[f+23200+(C*4864|0)+(aw*2432|0)+64>>2]|0;aq=f+23200+(C*4864|0)+(aw*2432|0)+72|0;as=c[aq>>2]|0;if((as|0)>0){D=at+190|0;at=0;V=aD;ar=as;while(1){K=a[337232+(av*22|0)+at|0]|0;if(K<<24>>24==0){bu=V;bv=ar}else{al=K&255;K=D-((d[f+23200+(C*4864|0)+(aw*2432|0)+76+at|0]|0)+(d[121496+(W*22|0)+at|0]|0)<<R)&65535;B=-al|0;X=al;I=V;while(1){b[I>>1]=K;ak=X-1|0;if((ak|0)>0){X=ak;I=I+2|0}else{break}}bu=V+(al+1+((B|0)>-1?B:-1)<<1)|0;bv=c[aq>>2]|0}I=at+1|0;if((I|0)<(bv|0)){at=I;V=bu;ar=bv}else{bw=bu;bx=bv;break}}}else{bw=aD;bx=as}ar=c[f+23200+(C*4864|0)+(aw*2432|0)+68>>2]|0;if((ar|0)<13){V=c[v>>2]|0;c[aK>>2]=U-(c[f+23200+(C*4864|0)+(aw*2432|0)+36>>2]<<3);c[l>>2]=U-(c[f+23200+(C*4864|0)+(aw*2432|0)+40>>2]<<3);c[aL>>2]=U-(c[f+23200+(C*4864|0)+(aw*2432|0)+44>>2]<<3);at=ar;ar=bx;aq=bw;while(1){W=a[337112+(V*13|0)+at|0]|0;D=W&255;av=-D|0;I=(av|0)>-1?av:-1;av=ar+1|0;if(W<<24>>24==0){by=aq}else{W=(c[aK>>2]|0)+400-(d[f+23200+(C*4864|0)+(aw*2432|0)+76+ar|0]<<R)&65535;X=D;K=aq;while(1){b[K>>1]=W;ak=X-1|0;if((ak|0)>0){X=ak;K=K+2|0}else{break}}K=D+1+I|0;X=(c[l>>2]|0)+400-(d[f+23200+(C*4864|0)+(aw*2432|0)+76+av|0]<<R)&65535;W=D;B=aq+(K<<1)|0;while(1){b[B>>1]=X;al=W-1|0;if((al|0)>0){W=al;B=B+2|0}else{break}}B=K<<1;W=(c[aL>>2]|0)+400-(d[ar+2+(f+23200+(C*4864|0)+(aw*2432|0)+76)|0]<<R)&65535;X=D;av=aq+(B<<1)|0;while(1){b[av>>1]=W;I=X-1|0;if((I|0)>0){X=I;av=av+2|0}else{break}}by=aq+(B+K<<1)|0}av=at+1|0;if((av|0)<13){at=av;ar=ar+3|0;aq=by}else{break}}}aq=(c[f+23200+(C*4864|0)+(aw*2432|0)+4>>2]|0)+A|0;ar=c[t>>2]|0;at=aq;R=(ar|0)<(aq|0)?ar:aq;aq=0;ar=0;while(1){V=c[f+23200+(C*4864|0)+(aw*2432|0)+52+(aq<<2)>>2]|0;L22155:do{if((V|0)==0){bz=ar;bA=R;bB=at}else{U=c[f+23200+(C*4864|0)+(aw*2432|0)+24+(aq<<2)>>2]|0;as=d[125912+(U<<1)|0]|0;av=d[125913+(U<<1)|0]|0;if((16401>>>(U>>>0)&1|0)!=0){NM(f+23200+(C*4864|0)+(aw*2432|0)+128+(ar<<2)|0,0,V<<3|0);bz=(V<<1)+ar|0;bA=R;bB=at;break}if((V|0)<=0){bz=ar;bA=R;bB=at;break}X=2222236+(as<<4)|0;as=U>>>0<16;U=32-av|0;W=at;D=R;I=ar;al=V;while(1){ak=c[s>>2]|0;if((ak|0)<(D|0)){bC=D;bD=W;bE=ak}else{do{if((c[aG>>2]|0)==0){bF=ak;bG=D;bH=W}else{if((ak|0)<(c[t>>2]|0)){bF=ak;bG=D;bH=W;break}c[bo>>2]=c[bp>>2];c[bo+4>>2]=c[bp+4>>2];c[bo+8>>2]=c[bp+8>>2];c[bo+12>>2]=c[bp+12>>2];c[bo+16>>2]=c[bp+16>>2];c[aG>>2]=0;aj=ak-D|0;ai=c[s>>2]|0;ah=-ai|0;E=(c[w>>2]|0)-ai|0;if((aj|0)<(ah|0)){bI=ah}else{bI=(E|0)<(aj|0)?E:aj}aj=bI+ai|0;c[s>>2]=aj;ai=aj+(W-ak)|0;bF=aj;bG=ai;bH=ai}}while(0);if((bF|0)<(bG|0)){bC=bG;bD=bH;bE=bF}else{bz=I;bA=bG;bB=bH;break L22155}}ak=c[X>>2]|0;k=c[w>>2]|0;ai=c[h>>2]|0;aj=ai+(bE>>>3)|0;E=(cV(d[aj]|d[aj+1|0]<<8|d[aj+2|0]<<16|d[aj+3|0]<<24|0)|0)<<(bE&7)>>>25;aj=b[ak+(E<<2)>>1]|0;ah=b[ak+(E<<2)+2>>1]|0;E=ah<<16>>16;do{if(ah<<16>>16<0){J=bE+7|0;F=k>>>0>J>>>0?J:k;J=ai+(F>>>3)|0;L=((cV(d[J]|d[J+1|0]<<8|d[J+2|0]<<16|d[J+3|0]<<24|0)|0)<<(F&7)>>>((E+32|0)>>>0))+aj|0;J=b[ak+(L<<2)>>1]|0;M=b[ak+(L<<2)+2>>1]|0;L=M<<16>>16;if(M<<16>>16>=0){bJ=J;bK=F;bL=L;break}M=F-E|0;F=k>>>0>M>>>0?M:k;M=ai+(F>>>3)|0;N=((cV(d[M]|d[M+1|0]<<8|d[M+2|0]<<16|d[M+3|0]<<24|0)|0)<<(F&7)>>>((L+32|0)>>>0))+J|0;bJ=b[ak+(N<<2)>>1]|0;bK=F;bL=b[ak+(N<<2)+2>>1]|0}else{bJ=aj;bK=bE;bL=E}}while(0);E=bK+bL|0;aj=k>>>0>E>>>0?E:k;c[s>>2]=aj;do{if((bJ|0)==0){c[f+23200+(C*4864|0)+(aw*2432|0)+128+(I+1<<2)>>2]=0;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(I<<2)>>2]=0}else{E=b[p+(I<<1)>>1]|0;ak=bJ>>5;ah=bJ&15;if((bJ&16|0)==0){N=ak+ah|0;if((N|0)<15){F=d[ai+(aj>>>3)|0]<<(aj&7)>>>7;c[s>>2]=((aj|0)<(k|0))+aj;J=F&1;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(((ah|0)!=0)+I<<2)>>2]=(c[2941552+(E<<6)+(N<<2)>>2]^-J)+J}else{if(as){bM=0;bN=aj}else{J=ai+(aj>>>3)|0;F=(cV(d[J]|d[J+1|0]<<8|d[J+2|0]<<16|d[J+3|0]<<24|0)|0)<<(aj&7)>>>(U>>>0);J=aj+av|0;L=k>>>0>J>>>0?J:k;c[s>>2]=L;bM=F;bN=L}L=bM+N<<2|E&3;N=(a[2129400+L|0]|0)-(E>>2)|0;if((N|0)>31){bO=0}else{bO=((c[1998088+(L<<2)>>2]|0)+(1<<N-1)|0)>>>(N>>>0)}N=d[ai+(bN>>>3)|0]|0;c[s>>2]=((bN|0)<(k|0))+bN;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(((ah|0)!=0)+I<<2)>>2]=(N&128>>>((bN&7)>>>0)|0)==0?bO:-bO|0}c[f+23200+(C*4864|0)+(aw*2432|0)+128+(((ah|0)==0)+I<<2)>>2]=0;break}if((ak|0)<15){N=d[ai+(aj>>>3)|0]<<(aj&7)>>>7;c[s>>2]=((aj|0)<(k|0))+aj;L=N&1;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(I<<2)>>2]=(c[2941552+(E<<6)+(ak<<2)>>2]^-L)+L}else{if(as){bP=0;bQ=aj}else{L=ai+(aj>>>3)|0;N=(cV(d[L]|d[L+1|0]<<8|d[L+2|0]<<16|d[L+3|0]<<24|0)|0)<<(aj&7)>>>(U>>>0);L=aj+av|0;F=k>>>0>L>>>0?L:k;c[s>>2]=F;bP=N;bQ=F}F=bP+ak<<2|E&3;ak=(a[2129400+F|0]|0)-(E>>2)|0;if((ak|0)>31){bR=0}else{bR=((c[1998088+(F<<2)>>2]|0)+(1<<ak-1)|0)>>>(ak>>>0)}ak=d[ai+(bQ>>>3)|0]|0;c[s>>2]=((bQ|0)<(k|0))+bQ;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(I<<2)>>2]=(ak&128>>>((bQ&7)>>>0)|0)==0?bR:-bR|0}if((ah|0)!=15){ak=c[s>>2]|0;F=d[(c[h>>2]|0)+(ak>>>3)|0]<<(ak&7)>>>7;c[s>>2]=((ak|0)<(c[w>>2]|0))+ak;ak=F&1;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(I+1<<2)>>2]=(c[2941552+(E<<6)+(ah<<2)>>2]^-ak)+ak;break}if(as){bS=60}else{ak=c[s>>2]|0;ah=c[w>>2]|0;F=(c[h>>2]|0)+(ak>>>3)|0;N=(cV(d[F]|d[F+1|0]<<8|d[F+2|0]<<16|d[F+3|0]<<24|0)|0)<<(ak&7)>>>(U>>>0);F=ak+av|0;c[s>>2]=ah>>>0>F>>>0?F:ah;bS=(N<<2)+60|0}N=bS|E&3;ah=(a[2129400+N|0]|0)-(E>>2)|0;if((ah|0)>31){bT=0}else{bT=((c[1998088+(N<<2)>>2]|0)+(1<<ah-1)|0)>>>(ah>>>0)}ah=c[s>>2]|0;N=d[(c[h>>2]|0)+(ah>>>3)|0]|0;c[s>>2]=((ah|0)<(c[w>>2]|0))+ah;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(I+1<<2)>>2]=(N&128>>>((ah&7)>>>0)|0)==0?bT:-bT|0}}while(0);k=I+2|0;ai=al-1|0;if((ai|0)>0){W=bD;D=bC;I=k;al=ai}else{bz=k;bA=bC;bB=bD;break}}}}while(0);V=aq+1|0;if((V|0)<3){at=bB;R=bA;aq=V;ar=bz}else{break}}ar=d[f+23200+(C*4864|0)+(aw*2432|0)+49|0]|0;L22215:do{if((bz|0)<573){aq=2223068+(ar<<4)|0;R=2223064+(ar<<4)|0;at=bB;V=bA;K=bz;B=0;while(1){bU=c[s>>2]|0;if((bU|0)<(V|0)){bV=bU;bW=V;bX=at}else{if(!((bU|0)<=(at|0)|(B|0)==0)){break}do{if((c[aG>>2]|0)==0){bY=bU;bZ=V;b_=at}else{if((bU|0)<(c[t>>2]|0)){bY=bU;bZ=V;b_=at;break}c[bo>>2]=c[bp>>2];c[bo+4>>2]=c[bp+4>>2];c[bo+8>>2]=c[bp+8>>2];c[bo+12>>2]=c[bp+12>>2];c[bo+16>>2]=c[bp+16>>2];c[aG>>2]=0;al=bU-V|0;I=c[s>>2]|0;D=-I|0;W=(c[w>>2]|0)-I|0;if((al|0)<(D|0)){b$=D}else{b$=(W|0)<(al|0)?W:al}al=b$+I|0;c[s>>2]=al;I=al+(at-bU)|0;bY=al;bZ=I;b_=I}}while(0);if((bY|0)<(bZ|0)){bV=bY;bW=bZ;bX=b_}else{b0=K;b1=bZ;b2=b_;break L22215}}I=c[aq>>2]|0;al=c[R>>2]|0;W=c[w>>2]|0;D=(c[h>>2]|0)+(bV>>>3)|0;av=(cV(d[D]|d[D+1|0]<<8|d[D+2|0]<<16|d[D+3|0]<<24|0)|0)<<(bV&7)>>>((32-al|0)>>>0);al=b[I+(av<<2)>>1]|0;D=(b[I+(av<<2)+2>>1]|0)+bV|0;c[s>>2]=W>>>0>D>>>0?D:W;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(K+3<<2)>>2]=0;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(K+2<<2)>>2]=0;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(K+1<<2)>>2]=0;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(K<<2)>>2]=0;if(al<<16>>16!=0){W=al<<16>>16;while(1){al=c[173376+(W<<2)>>2]|0;D=al+K|0;av=8>>>(al>>>0);al=c[s>>2]|0;I=d[(c[h>>2]|0)+(al>>>3)|0]<<(al&7)>>>7;c[s>>2]=((al|0)<(c[w>>2]|0))+al;al=I&1;c[f+23200+(C*4864|0)+(aw*2432|0)+128+(D<<2)>>2]=(c[2974320+(b[p+(D<<1)>>1]<<2)>>2]^-al)+al;if((av|0)==(W|0)){break}else{W=av^W}}}W=K+4|0;if((W|0)<573){at=bX;V=bW;K=W;B=bV}else{b0=W;b1=bW;b2=bX;break L22215}}R=B-bU|0;aq=-bU|0;W=(c[w>>2]|0)-bU|0;if((R|0)<(aq|0)){b3=aq}else{b3=(W|0)<(R|0)?W:R}c[s>>2]=b3+bU;AN(c[m>>2]|0,32,1718864,(a5=i,i=i+24|0,c[a5>>2]=R,c[a5+8>>2]=V-bU,c[a5+16>>2]=at-bU,a5)|0);i=a5;b0=(c[aB>>2]&131074|0)==0?K-4|0:0;b1=V;b2=at}else{b0=bz;b1=bA;b2=bB}}while(0);ar=b2-(c[s>>2]|0)|0;do{if((ar|0)<0){if((c[aB>>2]&131076|0)==0){az=16533;break}AN(c[m>>2]|0,16,1714416,(a5=i,i=i+8|0,c[a5>>2]=ar,a5)|0);i=a5;b4=0}else{az=16533}}while(0);do{if((az|0)==16533){az=0;if((ar|0)<=0){b4=b0;break}if((c[aB>>2]&262148|0)==0){b4=b0;break}AN(c[m>>2]|0,16,1714416,(a5=i,i=i+8|0,c[a5>>2]=ar,a5)|0);i=a5;b4=0}}while(0);NM(f+23200+(C*4864|0)+(aw*2432|0)+128+(b4<<2)|0,0,576-b4<<2|0);R=c[s>>2]|0;W=-R|0;aq=(c[w>>2]|0)-R|0;if((ar|0)<(W|0)){b5=W}else{b5=(aq|0)<(ar|0)?aq:ar}aq=b5+R|0;c[s>>2]=aq;do{if((c[aG>>2]|0)==0){b6=aq}else{if((aq|0)<(c[t>>2]|0)){b6=aq;break}c[bo>>2]=c[bp>>2];c[bo+4>>2]=c[bp+4>>2];c[bo+8>>2]=c[bp+8>>2];c[bo+12>>2]=c[bp+12>>2];c[bo+16>>2]=c[bp+16>>2];c[aG>>2]=0;R=aq-b1|0;W=c[s>>2]|0;av=-W|0;al=(c[w>>2]|0)-W|0;if((R|0)<(av|0)){b7=av}else{b7=(al|0)<(R|0)?al:R}R=b7+W|0;c[s>>2]=R;b6=R}}while(0);aq=C+1|0;if((aq|0)<(c[y>>2]|0)){C=aq;A=b6}else{break}}}L22259:do{if((c[aE>>2]|0)==1){A=c[u>>2]|0;if((A&1|0)==0){if((A&2|0)==0){break}else{b8=0}while(1){A=f+23200+(aw*2432|0)+128+(b8<<2)|0;C=c[A>>2]|0;ay=f+28064+(aw*2432|0)+128+(b8<<2)|0;z=c[ay>>2]|0;c[A>>2]=z+C;c[ay>>2]=C-z;b8=b8+1|0;if((b8|0)>=576){break L22259}}}if((c[aS>>2]|0)==0){b9=2206048;ca=7}else{b9=2205792+((c[f+28064+(aw*2432|0)+16>>2]&1)<<7)|0;ca=16}z=f+23200+(aw*2432|0)+2432|0;C=f+28064+(aw*2432|0)+2432|0;c[aK>>2]=0;c[l>>2]=0;c[aL>>2]=0;ay=f+28064+(aw*2432|0)+68|0;A=c[ay>>2]|0;aO=f+28064+(aw*2432|0)+72|0;if((A|0)>12){cb=z;cc=C}else{aH=12;aM=((13-A|0)*3|0)-3+(c[aO>>2]|0)|0;A=z;z=C;while(1){C=(aH|0)==11?aM:aM-3|0;aq=a[337112+((c[v>>2]|0)*13|0)+aH|0]|0;ar=aq&255;R=-ar|0;W=aq<<24>>24==0;aq=2;al=A;av=z;while(1){D=al+(R<<2)|0;I=av+(R<<2)|0;U=n+(aq<<2)|0;do{if((c[U>>2]|0)==0){as=0;while(1){if((as|0)>=(ar|0)){break}if((c[av+(as-ar<<2)>>2]|0)==0){as=as+1|0}else{az=16554;break}}if((az|0)==16554){az=0;c[U>>2]=1;az=16559;break}as=d[aq+C+(f+28064+(aw*2432|0)+76)|0]|0;if(as>>>0>=ca>>>0){az=16559;break}if(W){break}X=c[b9+64+(as<<2)>>2]|0;k=c[b9+(as<<2)>>2]|0;as=k;ai=(k|0)<0|0?-1:0;k=X;aj=(X|0)<0|0?-1:0;X=0;do{ah=X-ar|0;N=al+(ah<<2)|0;E=c[N>>2]|0;F=E;ak=(E|0)<0|0?-1:0;E=N2(F,ak,as,ai)|0;c[N>>2]=E>>>23|G<<9;E=N2(F,ak,k,aj)|0;c[av+(ah<<2)>>2]=E>>>23|G<<9;X=X+1|0;}while((X|0)<(ar|0))}else{az=16559}}while(0);do{if((az|0)==16559){az=0;if((c[u>>2]&2|0)==0|W){break}else{cd=0}do{U=cd-ar|0;X=al+(U<<2)|0;aj=c[X>>2]|0;k=av+(U<<2)|0;U=c[k>>2]|0;ai=U+aj|0;as=N2(ai,(ai|0)<0|0?-1:0,5931642,0)|0;c[X>>2]=as>>>23|G<<9;as=aj-U|0;U=N2(as,(as|0)<0|0?-1:0,5931642,0)|0;c[k>>2]=U>>>23|G<<9;cd=cd+1|0;}while((cd|0)<(ar|0))}}while(0);if((aq|0)<=0){break}aq=aq-1|0;al=D;av=I}av=ar*-3|0;al=z+(av<<2)|0;aq=A+(av<<2)|0;av=aH-1|0;if((av|0)<(c[ay>>2]|0)){cb=aq;cc=al;break}else{aH=av;aM=C;A=aq;z=al}}}z=c[aO>>2]|0;if((z|0)<=0){break}A=c[aK>>2]|c[l>>2]|c[aL>>2];aM=cb;aH=cc;ay=z;while(1){z=(A|0)==0;al=aM;aq=aH;av=ay;L22297:while(1){ce=av-1|0;cf=a[337232+((c[v>>2]|0)*22|0)+ce|0]|0;cg=cf&255;W=-cg|0;ch=al+(W<<2)|0;ci=aq+(W<<2)|0;if(z){cj=0}else{ck=A;break}while(1){if((cj|0)>=(cg|0)){break}if((c[aq+(cj-cg<<2)>>2]|0)==0){cj=cj+1|0}else{ck=1;break L22297}}I=d[((ce|0)==21?20:ce)+(f+28064+(aw*2432|0)+76)|0]|0;if(I>>>0>=ca>>>0){ck=A;break}if(cf<<24>>24!=0){D=c[b9+64+(I<<2)>>2]|0;W=c[b9+(I<<2)>>2]|0;I=W;R=(W|0)<0|0?-1:0;W=D;at=(D|0)<0|0?-1:0;D=0;do{V=D-cg|0;K=al+(V<<2)|0;B=c[K>>2]|0;U=B;k=(B|0)<0|0?-1:0;B=N2(U,k,I,R)|0;c[K>>2]=B>>>23|G<<9;B=N2(U,k,W,at)|0;c[aq+(V<<2)>>2]=B>>>23|G<<9;D=D+1|0;}while((D|0)<(cg|0))}if((ce|0)>0){al=ch;aq=ci;av=ce}else{break L22259}}if(!((c[u>>2]&2|0)==0|cf<<24>>24==0)){av=0;do{z=av-cg|0;C=al+(z<<2)|0;ar=c[C>>2]|0;D=aq+(z<<2)|0;z=c[D>>2]|0;at=z+ar|0;W=N2(at,(at|0)<0|0?-1:0,5931642,0)|0;c[C>>2]=W>>>23|G<<9;W=ar-z|0;z=N2(W,(W|0)<0|0?-1:0,5931642,0)|0;c[D>>2]=z>>>23|G<<9;av=av+1|0;}while((av|0)<(cg|0))}if((ce|0)>0){A=ck;aM=ch;aH=ci;ay=ce}else{break}}}}while(0);ay=c[y>>2]|0;if((ay|0)>0){aH=aw*18|0;aM=0;while(1){A=f+23200+(aM*4864|0)+(aw*2432|0)|0;aO=f+23200+(aM*4864|0)+(aw*2432|0)+20|0;do{if((a[aO]|0)==2){av=f+23200+(aM*4864|0)+(aw*2432|0)+21|0;aq=a[av]|0;do{if(aq<<24>>24==0){cl=f+23200+(aM*4864|0)+(aw*2432|0)+128|0}else{if((c[v>>2]|0)==8){cl=f+23200+(aM*4864|0)+(aw*2432|0)+416|0;break}else{cl=f+23200+(aM*4864|0)+(aw*2432|0)+272|0;break}}}while(0);al=c[f+23200+(aM*4864|0)+(aw*2432|0)+68>>2]|0;if((al|0)<13){z=cl;D=al;while(1){al=a[337112+((c[v>>2]|0)*13|0)+D|0]|0;W=al&255;ar=W<<1;if(al<<24>>24==0){cm=z}else{al=-W|0;C=W;at=z;R=aN;while(1){c[R>>2]=c[at>>2];c[R+4>>2]=c[at+(W<<2)>>2];c[R+8>>2]=c[at+(ar<<2)>>2];I=C-1|0;if((I|0)>0){C=I;at=at+4|0;R=R+12|0}else{break}}cm=z+(W+1+((al|0)>-1?al:-1)<<2)|0}R=z;at=W*12|0;NN(R|0,aJ|0,at)|0;at=D+1|0;if((at|0)>=13){break}z=cm+(ar<<2)|0;D=at}if((a[aO]|0)!=2){cn=31;az=16598;break}co=a[av]|0}else{co=aq}if(co<<24>>24!=0){cn=1;az=16598}}else{cn=31;az=16598}}while(0);if((az|0)==16598){az=0;aO=cn;D=f+23200+(aM*4864|0)+(aw*2432|0)+200|0;while(1){z=D-4|0;at=c[z>>2]|0;R=c[D>>2]|0;C=R+at|0;I=c[745300]|0;N2(C,(C|0)<0|0?-1:0,I,(I|0)<0|0?-1:0)|0;I=G;C=c[745302]|0;N2(C,(C|0)<0|0?-1:0,R,(R|0)<0|0?-1:0)|0;c[z>>2]=I-G<<2;z=c[745303]|0;N2(z,(z|0)<0|0?-1:0,at,(at|0)<0|0?-1:0)|0;c[D>>2]=G+I<<2;I=D-8|0;at=c[I>>2]|0;z=D+4|0;R=c[z>>2]|0;C=R+at|0;B=c[745304]|0;N2(C,(C|0)<0|0?-1:0,B,(B|0)<0|0?-1:0)|0;B=G;C=c[745306]|0;N2(C,(C|0)<0|0?-1:0,R,(R|0)<0|0?-1:0)|0;c[I>>2]=B-G<<2;I=c[745307]|0;N2(I,(I|0)<0|0?-1:0,at,(at|0)<0|0?-1:0)|0;c[z>>2]=G+B<<2;B=D-12|0;z=c[B>>2]|0;at=D+8|0;I=c[at>>2]|0;R=I+z|0;C=c[745308]|0;N2(R,(R|0)<0|0?-1:0,C,(C|0)<0|0?-1:0)|0;C=G;R=c[745310]|0;N2(R,(R|0)<0|0?-1:0,I,(I|0)<0|0?-1:0)|0;c[B>>2]=C-G<<2;B=c[745311]|0;N2(B,(B|0)<0|0?-1:0,z,(z|0)<0|0?-1:0)|0;c[at>>2]=G+C<<2;C=D-16|0;at=c[C>>2]|0;z=D+12|0;B=c[z>>2]|0;I=B+at|0;R=c[745312]|0;N2(I,(I|0)<0|0?-1:0,R,(R|0)<0|0?-1:0)|0;R=G;I=c[745314]|0;N2(I,(I|0)<0|0?-1:0,B,(B|0)<0|0?-1:0)|0;c[C>>2]=R-G<<2;C=c[745315]|0;N2(C,(C|0)<0|0?-1:0,at,(at|0)<0|0?-1:0)|0;c[z>>2]=G+R<<2;R=D-20|0;z=c[R>>2]|0;at=D+16|0;C=c[at>>2]|0;B=C+z|0;I=c[745316]|0;N2(B,(B|0)<0|0?-1:0,I,(I|0)<0|0?-1:0)|0;I=G;B=c[745318]|0;N2(B,(B|0)<0|0?-1:0,C,(C|0)<0|0?-1:0)|0;c[R>>2]=I-G<<2;R=c[745319]|0;N2(R,(R|0)<0|0?-1:0,z,(z|0)<0|0?-1:0)|0;c[at>>2]=G+I<<2;I=D-24|0;at=c[I>>2]|0;z=D+20|0;R=c[z>>2]|0;C=R+at|0;B=c[745320]|0;N2(C,(C|0)<0|0?-1:0,B,(B|0)<0|0?-1:0)|0;B=G;C=c[745322]|0;N2(C,(C|0)<0|0?-1:0,R,(R|0)<0|0?-1:0)|0;c[I>>2]=B-G<<2;I=c[745323]|0;N2(I,(I|0)<0|0?-1:0,at,(at|0)<0|0?-1:0)|0;c[z>>2]=G+B<<2;B=D-28|0;z=c[B>>2]|0;at=D+24|0;I=c[at>>2]|0;R=I+z|0;C=c[745324]|0;N2(R,(R|0)<0|0?-1:0,C,(C|0)<0|0?-1:0)|0;C=G;R=c[745326]|0;N2(R,(R|0)<0|0?-1:0,I,(I|0)<0|0?-1:0)|0;c[B>>2]=C-G<<2;B=c[745327]|0;N2(B,(B|0)<0|0?-1:0,z,(z|0)<0|0?-1:0)|0;c[at>>2]=G+C<<2;C=D-32|0;at=c[C>>2]|0;z=D+28|0;B=c[z>>2]|0;I=B+at|0;R=c[745328]|0;N2(I,(I|0)<0|0?-1:0,R,(R|0)<0|0?-1:0)|0;R=G;I=c[745330]|0;N2(I,(I|0)<0|0?-1:0,B,(B|0)<0|0?-1:0)|0;c[C>>2]=R-G<<2;C=c[745331]|0;N2(C,(C|0)<0|0?-1:0,at,(at|0)<0|0?-1:0)|0;c[z>>2]=G+R<<2;R=aO-1|0;if((R|0)>0){aO=R;D=D+72|0}else{break}}}oi(f,A,f+9376+(aM*4608|0)+(aH<<7)|0,f+18592+(aM*2304|0)|0);D=aM+1|0;aO=c[y>>2]|0;if((D|0)<(aO|0)){aM=D}else{cp=aO;break}}}else{cp=ay}aM=aw+1|0;if((aM|0)<(aV|0)){aw=aM;aC=aC+1|0;T=cp}else{break}}}T=c[s>>2]|0;if((T|0)<0){aC=-T|0;aw=(c[w>>2]|0)-T|0;c[s>>2]=((aw|0)<(aC|0)?aw:aC)+T}be=aV*18|0}else if((az|0)==16349){AN(c[m>>2]|0,16,1726920,(a5=i,i=i+1|0,i=i+7&-8,c[a5>>2]=0,a5)|0);i=a5;be=-1094995529}T=f+1088|0;c[T>>2]=0;aC=f+1116|0;aw=aC|0;if((c[aw>>2]|0)!=0){y=c[s>>2]|0;aJ=-y&7;if((aJ|0)==0){cq=y}else{aN=c[w>>2]|0;v=aJ+y|0;y=aN>>>0>v>>>0?v:aN;c[s>>2]=y;cq=y}y=(c[t>>2]|0)-cq>>3;if(y>>>0<513){NP(f+40|0,(c[h>>2]|0)+(cq>>3)|0,y|0);c[T>>2]=y}else{AN(c[m>>2]|0,16,1618368,(a5=i,i=i+8|0,c[a5>>2]=y,a5)|0);i=a5}y=r;aN=aC;c[y>>2]=c[aN>>2];c[y+4>>2]=c[aN+4>>2];c[y+8>>2]=c[aN+8>>2];c[y+12>>2]=c[aN+12>>2];c[y+16>>2]=c[aN+16>>2];c[aw>>2]=0}aw=c[s>>2]|0;aN=-aw&7;if((aN|0)==0){cr=aw}else{y=c[w>>2]|0;aC=aN+aw|0;aw=y>>>0>aC>>>0?aC:y;c[s>>2]=aw;cr=aw}aw=(c[t>>2]|0)-cr>>3;y=(be|0)<0;if(aw>>>0>512|y){if((aw|0)<0){AN(c[m>>2]|0,16,1613840,(a5=i,i=i+8|0,c[a5>>2]=aw,a5)|0);i=a5}cs=(g|0)<512?g:512}else{cs=aw}aw=(c[T>>2]|0)+(f+40)|0;aC=(c[h>>2]|0)+(g-cs)|0;NN(aw|0,aC|0,cs)|0;c[T>>2]=(c[T>>2]|0)+cs;if(y){ct=be}else{ax=be;break}i=j;return ct|0}}while(0);be=f+33004|0;cs=c[be>>2]|0;if((cs|0)==0){AN(0,0,1740848,(a5=i,i=i+24|0,c[a5>>2]=1736560,c[a5+8>>2]=1731552,c[a5+16>>2]=1631,a5)|0);i=a5;cw();return 0}a5=f+32940|0;c[cs+76>>2]=c[(c[a5>>2]|0)+424>>2];cs=oX(c[a5>>2]|0,c[be>>2]|0,0)|0;if((cs|0)<0){ct=cs;i=j;return ct|0}cs=c[(c[be>>2]|0)+64>>2]|0;be=f+24|0;g=c[be>>2]|0;if((g|0)>0){h=(ax|0)>0;cr=f+32944|0;t=f+32932|0;s=0;w=g;while(1){if((c[(c[a5>>2]|0)+420>>2]|0)==6){cu=1;cv=c[cs+(s<<2)>>2]|0}else{cu=w;cv=(c[cs>>2]|0)+(s<<1)|0}if(h){r=f+1152+(s<<12)|0;cq=f+9344+(s<<2)|0;az=cu<<5;aV=0;cp=cv;while(1){on(cr,r,cq,2599496,t,cp,cu,f+9376+(s*4608|0)+(aV<<7)|0);cn=aV+1|0;if((cn|0)<(ax|0)){aV=cn;cp=cp+(az<<1)|0}else{break}}cx=c[be>>2]|0}else{cx=w}az=s+1|0;if((az|0)<(cx|0)){s=az;w=cx}else{cy=cx;break}}}else{cy=g}ct=ac(ax<<6,cy)|0;i=j;return ct|0}function oi(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0;h=i;i=i+48|0;j=h|0;k=e+128|0;l=e+272|0;m=e+2432|0;while(1){if(m>>>0<l>>>0){n=m;break}o=m-24|0;if((c[m-20>>2]|c[o>>2]|c[m-16>>2]|c[m-12>>2]|c[m-8>>2]|c[m-4>>2]|0)==0){m=o}else{n=o;break}}m=((n-k>>2|0)/18|0)+1|0;n=a[e+20|0]|0;l=e+21|0;if(n<<24>>24==2){p=(a[l]|0)==0?0:2}else{p=m}c9[c[b+32964>>2]&127](f,g,k,p,d[l]|0,n&255);n=g+(((p>>2)*72|0|p&3)<<2)|0;if((p|0)<(m|0)){g=j|0;l=j+40|0;k=j+28|0;b=j+16|0;o=j+4|0;q=j+36|0;r=j+32|0;s=j+12|0;t=j+8|0;u=j+20|0;v=j+44|0;w=j+24|0;x=n;y=p;z=e+128+((p*18|0)<<2)|0;e=f+(p+192<<2)|0;A=f+(p+384<<2)|0;while(1){c[f+(y<<2)>>2]=c[x>>2];c[f+(y+32<<2)>>2]=c[x+16>>2];c[f+(y+64<<2)>>2]=c[x+32>>2];c[f+(y+96<<2)>>2]=c[x+48>>2];c[f+(y+128<<2)>>2]=c[x+64>>2];c[f+(y+160<<2)>>2]=c[x+80>>2];p=-(y&1)&4|2;B=c[z>>2]|0;C=c[z+12>>2]|0;D=C+B|0;E=c[z+24>>2]|0;F=c[z+36>>2]|0;H=F+E|0;I=c[z+48>>2]|0;J=I+F|0;F=H+I+(c[z+60>>2]|0)|0;I=E+C<<1;N2(I,(I|0)<0|0?-1:0,1859775393,0)|0;I=G;C=H+D<<2;N2(C,(C|0)<0|0?-1:0,1859775393,0)|0;C=G;H=B-J|0;E=D-F<<1;N2(E,(E|0)<0|0?-1:0,1518500250,0)|0;E=G;K=E+H|0;c[l>>2]=K;c[k>>2]=K;K=H-E|0;c[b>>2]=K;c[o>>2]=K;K=(J>>1)+B|0;B=I+K|0;J=F+(D<<1)|0;D=C+J|0;N2(D,(D|0)<0|0?-1:0,1111619334,0)|0;D=G;F=D+B|0;c[q>>2]=F;c[r>>2]=F;F=B-D|0;c[s>>2]=F;c[t>>2]=F;F=K-I|0;I=J-C<<1;N2(I,(I|0)<0|0?-1:0,2074309917,0)|0;I=G;C=F-I|0;c[u>>2]=C;c[g>>2]=C;J=I+F|0;c[v>>2]=J;c[w>>2]=J;J=0;F=e;I=C;while(1){C=c[2603848+(p*160|0)+(J<<2)>>2]|0;N2(C,(C|0)<0|0?-1:0,I,(I|0)<0|0?-1:0)|0;C=J+6|0;c[F>>2]=G+(c[x+(C<<2<<2)>>2]|0);K=c[j+(C<<2)>>2]|0;D=c[2603848+(p*160|0)+(C<<2)>>2]|0;N2(D,(D|0)<0|0?-1:0,K,(K|0)<0|0?-1:0)|0;c[x+((J<<2)+48<<2)>>2]=G;K=J+1|0;if((K|0)>=6){break}J=K;F=F+128|0;I=c[j+(K<<2)>>2]|0}I=c[z+4>>2]|0;F=c[z+16>>2]|0;J=F+I|0;K=c[z+28>>2]|0;D=c[z+40>>2]|0;C=D+K|0;B=c[z+52>>2]|0;E=B+D|0;D=C+B+(c[z+64>>2]|0)|0;B=K+F<<1;N2(B,(B|0)<0|0?-1:0,1859775393,0)|0;B=G;F=C+J<<2;N2(F,(F|0)<0|0?-1:0,1859775393,0)|0;F=G;C=I-E|0;K=J-D<<1;N2(K,(K|0)<0|0?-1:0,1518500250,0)|0;K=G;H=K+C|0;c[l>>2]=H;c[k>>2]=H;H=C-K|0;c[b>>2]=H;c[o>>2]=H;H=(E>>1)+I|0;I=B+H|0;E=D+(J<<1)|0;J=F+E|0;N2(J,(J|0)<0|0?-1:0,1111619334,0)|0;J=G;D=J+I|0;c[q>>2]=D;c[r>>2]=D;D=I-J|0;c[s>>2]=D;c[t>>2]=D;D=H-B|0;B=E-F<<1;N2(B,(B|0)<0|0?-1:0,2074309917,0)|0;B=G;F=D-B|0;c[u>>2]=F;c[g>>2]=F;E=B+D|0;c[v>>2]=E;c[w>>2]=E;E=0;D=A;B=F;while(1){F=c[2603848+(p*160|0)+(E<<2)>>2]|0;N2(F,(F|0)<0|0?-1:0,B,(B|0)<0|0?-1:0)|0;F=E<<2;c[D>>2]=G+(c[x+(F+48<<2)>>2]|0);H=E+6|0;J=c[j+(H<<2)>>2]|0;I=c[2603848+(p*160|0)+(H<<2)>>2]|0;N2(I,(I|0)<0|0?-1:0,J,(J|0)<0|0?-1:0)|0;c[x+(F<<2)>>2]=G;F=E+1|0;if((F|0)>=6){break}E=F;D=D+128|0;B=c[j+(F<<2)>>2]|0}B=c[z+8>>2]|0;D=c[z+20>>2]|0;E=D+B|0;F=c[z+32>>2]|0;J=c[z+44>>2]|0;I=J+F|0;H=c[z+56>>2]|0;K=H+J|0;J=I+H+(c[z+68>>2]|0)|0;H=F+D<<1;N2(H,(H|0)<0|0?-1:0,1859775393,0)|0;H=G;D=I+E<<2;N2(D,(D|0)<0|0?-1:0,1859775393,0)|0;D=G;I=B-K|0;F=E-J<<1;N2(F,(F|0)<0|0?-1:0,1518500250,0)|0;F=G;C=F+I|0;c[l>>2]=C;c[k>>2]=C;C=I-F|0;c[b>>2]=C;c[o>>2]=C;C=(K>>1)+B|0;B=H+C|0;K=J+(E<<1)|0;E=D+K|0;N2(E,(E|0)<0|0?-1:0,1111619334,0)|0;E=G;J=E+B|0;c[q>>2]=J;c[r>>2]=J;J=B-E|0;c[s>>2]=J;c[t>>2]=J;J=C-H|0;H=K-D<<1;N2(H,(H|0)<0|0?-1:0,2074309917,0)|0;H=G;D=J-H|0;c[u>>2]=D;c[g>>2]=D;K=H+J|0;c[v>>2]=K;c[w>>2]=K;K=0;J=D;while(1){D=c[2603848+(p*160|0)+(K<<2)>>2]|0;N2(D,(D|0)<0|0?-1:0,J,(J|0)<0|0?-1:0)|0;D=K<<2;H=x+(D<<2)|0;c[H>>2]=G+(c[H>>2]|0);H=K+6|0;C=c[j+(H<<2)>>2]|0;E=c[2603848+(p*160|0)+(H<<2)>>2]|0;N2(E,(E|0)<0|0?-1:0,C,(C|0)<0|0?-1:0)|0;c[x+(H<<2<<2)>>2]=G;c[x+(D+48<<2)>>2]=0;D=K+1|0;if((D|0)>=6){break}K=D;J=c[j+(D<<2)>>2]|0}J=x+(((y&3|0)!=3?1:69)<<2)|0;K=y+1|0;if((K|0)<(m|0)){x=J;y=K;z=z+72|0;e=e+4|0;A=A+4|0}else{L=J;break}}}else{L=n}if((m|0)<32){M=L;N=m}else{i=h;return}while(1){c[f+(N<<2)>>2]=c[M>>2];c[M>>2]=0;m=M+16|0;c[f+(N+32<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+32|0;c[f+(N+64<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+48|0;c[f+(N+96<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+64|0;c[f+(N+128<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+80|0;c[f+(N+160<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+96|0;c[f+(N+192<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+112|0;c[f+(N+224<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+128|0;c[f+(N+256<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+144|0;c[f+(N+288<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+160|0;c[f+(N+320<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+176|0;c[f+(N+352<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+192|0;c[f+(N+384<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+208|0;c[f+(N+416<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+224|0;c[f+(N+448<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+240|0;c[f+(N+480<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+256|0;c[f+(N+512<<2)>>2]=c[m>>2];c[m>>2]=0;m=M+272|0;c[f+(N+544<<2)>>2]=c[m>>2];c[m>>2]=0;m=N+1|0;if((m|0)<32){M=M+(((N&3|0)!=3?1:69)<<2)|0;N=m}else{break}}i=h;return}function oj(a,b){a=a|0;b=b|0;var d=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;if((b&1048576|0)==0){c[a+36>>2]=1;d=1;f=1}else{g=b>>>19&1^1;c[a+36>>2]=g;d=0;f=g}g=4-(b>>>17&3)|0;c[a+8>>2]=g;h=b>>>10&3;i=f+d|0;d=(e[337568+(h<<1)>>1]|0)>>>(i>>>0);c[a+16>>2]=(i*3|0)+h;c[a+4>>2]=b>>>16&1^1;c[a+12>>2]=d;h=b>>>12&15;i=b>>>9&1;j=b>>>6&3;c[a+28>>2]=j;c[a+32>>2]=b>>>4&3;c[a+24>>2]=(j|0)==3?1:2;if((h|0)==0){k=1;return k|0}j=e[337576+(f*90|0)+((g-1|0)*30|0)+(h<<1)>>1]|0;c[a+20>>2]=j*1e3|0;if((g|0)==1){l=((j*12e3|0|0)/(d|0)|0)+i<<2}else if((g|0)==2){l=((j*144e3|0|0)/(d|0)|0)+i|0}else{l=((j*144e3|0|0)/(d<<f|0)|0)+i|0}c[a>>2]=l;k=0;return k|0}function ok(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0;h=i;i=i+40|0;j=h|0;if((b&-2097152|0)!=-2097152|(b&393216|0)==0|(b&61440|0)==61440|(b&3072|0)==3072){k=-1;i=h;return k|0}if((oj(j,b)|0)!=0){k=-1;i=h;return k|0}b=c[j+8>>2]|0;do{if((b|0)==2){c[a+48>>2]=86016;c[f>>2]=1152}else if((b|0)==1){c[a+48>>2]=86059;c[f>>2]=384}else{c[a+48>>2]=86017;if((c[j+36>>2]|0)==0){c[f>>2]=1152;break}else{c[f>>2]=576;break}}}while(0);c[d>>2]=c[j+12>>2];c[e>>2]=c[j+24>>2];c[g>>2]=c[j+20>>2];k=c[j>>2]|0;i=h;return k|0}function ol(a){a=a|0;var b=0,d=0;b=i;i=i+112|0;d=b|0;qo(d,5,0)|0;os();op();c[a>>2]=12;c[a+4>>2]=102;c[a+8>>2]=c[d+108>>2];c[a+12>>2]=14;c[a+16>>2]=28;c[a+20>>2]=44;i=b;return}function om(a,d,e,f,g){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0;h=a+2048|0;i=a;NN(h|0,i|0,128)|0;i=c[e>>2]|0;h=c[d>>2]|0;j=c[a+64>>2]|0;k=N2(j,(j|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=NU(k,G,i,(i|0)<0|0?-1:0)|0;i=G;k=c[d+256>>2]|0;j=c[a+320>>2]|0;l=N2(j,(j|0)<0|0?-1:0,k,(k|0)<0|0?-1:0)|0;k=NU(h,i,l,G)|0;l=G;i=c[d+512>>2]|0;h=c[a+576>>2]|0;j=N2(h,(h|0)<0|0?-1:0,i,(i|0)<0|0?-1:0)|0;i=NU(k,l,j,G)|0;j=G;l=c[d+768>>2]|0;k=c[a+832>>2]|0;h=N2(k,(k|0)<0|0?-1:0,l,(l|0)<0|0?-1:0)|0;l=NU(i,j,h,G)|0;h=G;j=c[d+1024>>2]|0;i=c[a+1088>>2]|0;k=N2(i,(i|0)<0|0?-1:0,j,(j|0)<0|0?-1:0)|0;j=NU(l,h,k,G)|0;k=G;h=c[d+1280>>2]|0;l=c[a+1344>>2]|0;i=N2(l,(l|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=NU(j,k,i,G)|0;i=G;k=c[d+1536>>2]|0;j=c[a+1600>>2]|0;l=N2(j,(j|0)<0|0?-1:0,k,(k|0)<0|0?-1:0)|0;k=NU(h,i,l,G)|0;l=G;i=c[d+1792>>2]|0;h=c[a+1856>>2]|0;j=N2(h,(h|0)<0|0?-1:0,i,(i|0)<0|0?-1:0)|0;i=NU(k,l,j,G)|0;j=G;l=c[d+128>>2]|0;k=c[a+192>>2]|0;h=N2(k,(k|0)<0|0?-1:0,l,(l|0)<0|0?-1:0)|0;l=NV(i,j,h,G)|0;h=G;j=c[d+384>>2]|0;i=c[a+448>>2]|0;k=N2(i,(i|0)<0|0?-1:0,j,(j|0)<0|0?-1:0)|0;j=NV(l,h,k,G)|0;k=G;h=c[d+640>>2]|0;l=c[a+704>>2]|0;i=N2(l,(l|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=NV(j,k,i,G)|0;i=G;k=c[d+896>>2]|0;j=c[a+960>>2]|0;l=N2(j,(j|0)<0|0?-1:0,k,(k|0)<0|0?-1:0)|0;k=NV(h,i,l,G)|0;l=G;i=c[d+1152>>2]|0;h=c[a+1216>>2]|0;j=N2(h,(h|0)<0|0?-1:0,i,(i|0)<0|0?-1:0)|0;i=NV(k,l,j,G)|0;j=G;l=c[d+1408>>2]|0;k=c[a+1472>>2]|0;h=N2(k,(k|0)<0|0?-1:0,l,(l|0)<0|0?-1:0)|0;l=NV(i,j,h,G)|0;h=G;j=c[d+1664>>2]|0;i=c[a+1728>>2]|0;k=N2(i,(i|0)<0|0?-1:0,j,(j|0)<0|0?-1:0)|0;j=NV(l,h,k,G)|0;k=G;h=c[d+1920>>2]|0;l=c[a+1984>>2]|0;i=N2(l,(l|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=NV(j,k,i,G)|0;i=G;k=h>>>24|i<<8;j=k;if((j+32768|0)>>>0>65535){m=(j>>31^32767)&65535}else{m=k&65535}b[f>>1]=m;m=-g|0;k=f+(g<<4<<1)|0;j=1;l=f+((g*31|0)<<1)|0;n=d+124|0;o=i&0;i=h&16777215;h=f;f=d;while(1){p=h+(g<<1)|0;q=f+4|0;r=c[a+(j+16<<2)>>2]|0;s=c[q>>2]|0;t=r;u=(r|0)<0|0?-1:0;r=N2(s,(s|0)<0|0?-1:0,t,u)|0;s=NU(r,G,i,o)|0;r=G;v=c[n>>2]|0;w=NV(0,0,t,u)|0;u=N2(v,(v|0)<0|0?-1:0,w,G)|0;w=G;v=c[a+(j+80<<2)>>2]|0;t=c[f+260>>2]|0;x=v;y=(v|0)<0|0?-1:0;v=N2(t,(t|0)<0|0?-1:0,x,y)|0;t=NU(s,r,v,G)|0;v=G;r=c[n+256>>2]|0;s=N2(r,(r|0)<0|0?-1:0,x,y)|0;y=NV(u,w,s,G)|0;s=G;w=c[a+(j+144<<2)>>2]|0;u=c[f+516>>2]|0;x=w;r=(w|0)<0|0?-1:0;w=N2(u,(u|0)<0|0?-1:0,x,r)|0;u=NU(t,v,w,G)|0;w=G;v=c[n+512>>2]|0;t=N2(v,(v|0)<0|0?-1:0,x,r)|0;r=NV(y,s,t,G)|0;t=G;s=c[a+(j+208<<2)>>2]|0;y=c[f+772>>2]|0;x=s;v=(s|0)<0|0?-1:0;s=N2(y,(y|0)<0|0?-1:0,x,v)|0;y=NU(u,w,s,G)|0;s=G;w=c[n+768>>2]|0;u=N2(w,(w|0)<0|0?-1:0,x,v)|0;v=NV(r,t,u,G)|0;u=G;t=c[a+(j+272<<2)>>2]|0;r=c[f+1028>>2]|0;x=t;w=(t|0)<0|0?-1:0;t=N2(r,(r|0)<0|0?-1:0,x,w)|0;r=NU(y,s,t,G)|0;t=G;s=c[n+1024>>2]|0;y=N2(s,(s|0)<0|0?-1:0,x,w)|0;w=NV(v,u,y,G)|0;y=G;u=c[a+(j+336<<2)>>2]|0;v=c[f+1284>>2]|0;x=u;s=(u|0)<0|0?-1:0;u=N2(v,(v|0)<0|0?-1:0,x,s)|0;v=NU(r,t,u,G)|0;u=G;t=c[n+1280>>2]|0;r=N2(t,(t|0)<0|0?-1:0,x,s)|0;s=NV(w,y,r,G)|0;r=G;y=c[a+(j+400<<2)>>2]|0;w=c[f+1540>>2]|0;x=y;t=(y|0)<0|0?-1:0;y=N2(w,(w|0)<0|0?-1:0,x,t)|0;w=NU(v,u,y,G)|0;y=G;u=c[n+1536>>2]|0;v=N2(u,(u|0)<0|0?-1:0,x,t)|0;t=NV(s,r,v,G)|0;v=G;r=c[a+(j+464<<2)>>2]|0;s=c[f+1796>>2]|0;x=r;u=(r|0)<0|0?-1:0;r=N2(s,(s|0)<0|0?-1:0,x,u)|0;s=NU(w,y,r,G)|0;r=G;y=c[n+1792>>2]|0;w=N2(y,(y|0)<0|0?-1:0,x,u)|0;u=NV(t,v,w,G)|0;w=G;v=48-j|0;t=c[a+(v<<2)>>2]|0;x=c[f+132>>2]|0;y=t;z=(t|0)<0|0?-1:0;t=N2(x,(x|0)<0|0?-1:0,y,z)|0;x=NV(s,r,t,G)|0;t=G;r=c[n+128>>2]|0;s=N2(r,(r|0)<0|0?-1:0,y,z)|0;z=NV(u,w,s,G)|0;s=G;w=c[a+(v+64<<2)>>2]|0;u=c[f+388>>2]|0;y=w;r=(w|0)<0|0?-1:0;w=N2(u,(u|0)<0|0?-1:0,y,r)|0;u=NV(x,t,w,G)|0;w=G;t=c[n+384>>2]|0;x=N2(t,(t|0)<0|0?-1:0,y,r)|0;r=NV(z,s,x,G)|0;x=G;s=c[a+(v+128<<2)>>2]|0;z=c[f+644>>2]|0;y=s;t=(s|0)<0|0?-1:0;s=N2(z,(z|0)<0|0?-1:0,y,t)|0;z=NV(u,w,s,G)|0;s=G;w=c[n+640>>2]|0;u=N2(w,(w|0)<0|0?-1:0,y,t)|0;t=NV(r,x,u,G)|0;u=G;x=c[a+(v+192<<2)>>2]|0;r=c[f+900>>2]|0;y=x;w=(x|0)<0|0?-1:0;x=N2(r,(r|0)<0|0?-1:0,y,w)|0;r=NV(z,s,x,G)|0;x=G;s=c[n+896>>2]|0;z=N2(s,(s|0)<0|0?-1:0,y,w)|0;w=NV(t,u,z,G)|0;z=G;u=c[a+(v+256<<2)>>2]|0;t=c[f+1156>>2]|0;y=u;s=(u|0)<0|0?-1:0;u=N2(t,(t|0)<0|0?-1:0,y,s)|0;t=NV(r,x,u,G)|0;u=G;x=c[n+1152>>2]|0;r=N2(x,(x|0)<0|0?-1:0,y,s)|0;s=NV(w,z,r,G)|0;r=G;z=c[a+(v+320<<2)>>2]|0;w=c[f+1412>>2]|0;y=z;x=(z|0)<0|0?-1:0;z=N2(w,(w|0)<0|0?-1:0,y,x)|0;w=NV(t,u,z,G)|0;z=G;u=c[n+1408>>2]|0;t=N2(u,(u|0)<0|0?-1:0,y,x)|0;x=NV(s,r,t,G)|0;t=G;r=c[a+(v+384<<2)>>2]|0;s=c[f+1668>>2]|0;y=r;u=(r|0)<0|0?-1:0;r=N2(s,(s|0)<0|0?-1:0,y,u)|0;s=NV(w,z,r,G)|0;r=G;z=c[n+1664>>2]|0;w=N2(z,(z|0)<0|0?-1:0,y,u)|0;u=NV(x,t,w,G)|0;w=G;t=c[a+(v+448<<2)>>2]|0;v=c[f+1924>>2]|0;x=t;y=(t|0)<0|0?-1:0;t=N2(v,(v|0)<0|0?-1:0,x,y)|0;v=NV(s,r,t,G)|0;t=G;r=c[n+1920>>2]|0;s=N2(r,(r|0)<0|0?-1:0,x,y)|0;y=NV(u,w,s,G)|0;s=G;w=v>>>24|t<<8;u=w;if((u+32768|0)>>>0>65535){A=(u>>31^32767)&65535}else{A=w&65535}b[p>>1]=A;w=NU(y,s,v&16777215,t&0)|0;u=w>>>24|G<<8;w=u;if((w+32768|0)>>>0>65535){B=(w>>31^32767)&65535}else{B=u&65535}b[l>>1]=B;u=j+1|0;w=NU(v,t,y,s)|0;C=w&16777215;D=G&0;if((u|0)<16){j=u;l=l+(m<<1)|0;n=n-4|0;o=D;i=C;h=p;f=q}else{break}}f=c[d+192>>2]|0;h=c[a+128>>2]|0;i=N2(h,(h|0)<0|0?-1:0,f,(f|0)<0|0?-1:0)|0;f=NV(C,D,i,G)|0;i=G;D=c[d+448>>2]|0;C=c[a+384>>2]|0;h=N2(C,(C|0)<0|0?-1:0,D,(D|0)<0|0?-1:0)|0;D=NV(f,i,h,G)|0;h=G;i=c[d+704>>2]|0;f=c[a+640>>2]|0;C=N2(f,(f|0)<0|0?-1:0,i,(i|0)<0|0?-1:0)|0;i=NV(D,h,C,G)|0;C=G;h=c[d+960>>2]|0;D=c[a+896>>2]|0;f=N2(D,(D|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=NV(i,C,f,G)|0;f=G;C=c[d+1216>>2]|0;i=c[a+1152>>2]|0;D=N2(i,(i|0)<0|0?-1:0,C,(C|0)<0|0?-1:0)|0;C=NV(h,f,D,G)|0;D=G;f=c[d+1472>>2]|0;h=c[a+1408>>2]|0;i=N2(h,(h|0)<0|0?-1:0,f,(f|0)<0|0?-1:0)|0;f=NV(C,D,i,G)|0;i=G;D=c[d+1728>>2]|0;C=c[a+1664>>2]|0;h=N2(C,(C|0)<0|0?-1:0,D,(D|0)<0|0?-1:0)|0;D=NV(f,i,h,G)|0;h=G;i=c[d+1984>>2]|0;d=c[a+1920>>2]|0;a=N2(d,(d|0)<0|0?-1:0,i,(i|0)<0|0?-1:0)|0;i=NV(D,h,a,G)|0;a=i>>>24|G<<8;h=a;if((h+32768|0)>>>0>65535){E=(h>>31^32767)&65535;b[k>>1]=E;F=i;H=F;I=H&16777215;c[e>>2]=I;return}else{E=a&65535;b[k>>1]=E;F=i;H=F;I=H&16777215;c[e>>2]=I;return}}function on(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0;j=c[d>>2]|0;k=b+(j<<2)|0;c$[c[a+12>>2]&127](k,i);cX[c[a+4>>2]&255](k,e,f,g,h);c[d>>2]=j+480&511;return}function oo(a){a=a|0;var b=0,d=0,e=0,f=0;b=0;while(1){d=c[245848+(b<<2)>>2]|0;c[a+(b<<2)>>2]=d;if((b|0)==0){b=b+1|0;continue}else{c[a+(512-b<<2)>>2]=(b&63|0)==0?d:-d|0;d=b+1|0;if((d|0)<257){b=d;continue}else{e=0;break}}}while(1){b=e<<6|32;d=e<<4;c[a+(d+512<<2)>>2]=c[a+(b<<2)>>2];c[a+(d+513<<2)>>2]=c[a+(b-1<<2)>>2];c[a+(d+514<<2)>>2]=c[a+(b-2<<2)>>2];c[a+(d+515<<2)>>2]=c[a+(b-3<<2)>>2];c[a+(d+516<<2)>>2]=c[a+(b-4<<2)>>2];c[a+(d+517<<2)>>2]=c[a+(b-5<<2)>>2];c[a+(d+518<<2)>>2]=c[a+(b-6<<2)>>2];c[a+(d+519<<2)>>2]=c[a+(b-7<<2)>>2];c[a+(d+520<<2)>>2]=c[a+(b-8<<2)>>2];c[a+(d+521<<2)>>2]=c[a+(b-9<<2)>>2];c[a+(d+522<<2)>>2]=c[a+(b-10<<2)>>2];c[a+(d+523<<2)>>2]=c[a+(b-11<<2)>>2];c[a+(d+524<<2)>>2]=c[a+(b-12<<2)>>2];c[a+(d+525<<2)>>2]=c[a+(b-13<<2)>>2];c[a+(d+526<<2)>>2]=c[a+(b-14<<2)>>2];c[a+(d+527<<2)>>2]=c[a+(b-15<<2)>>2];b=e+1|0;if((b|0)<8){e=b}else{f=0;break}}do{e=f<<6|48;b=f<<4;c[a+(b+640<<2)>>2]=c[a+(e<<2)>>2];c[a+(b+641<<2)>>2]=c[a+(e-1<<2)>>2];c[a+(b+642<<2)>>2]=c[a+(e-2<<2)>>2];c[a+(b+643<<2)>>2]=c[a+(e-3<<2)>>2];c[a+(b+644<<2)>>2]=c[a+(e-4<<2)>>2];c[a+(b+645<<2)>>2]=c[a+(e-5<<2)>>2];c[a+(b+646<<2)>>2]=c[a+(e-6<<2)>>2];c[a+(b+647<<2)>>2]=c[a+(e-7<<2)>>2];c[a+(b+648<<2)>>2]=c[a+(e-8<<2)>>2];c[a+(b+649<<2)>>2]=c[a+(e-9<<2)>>2];c[a+(b+650<<2)>>2]=c[a+(e-10<<2)>>2];c[a+(b+651<<2)>>2]=c[a+(e-11<<2)>>2];c[a+(b+652<<2)>>2]=c[a+(e-12<<2)>>2];c[a+(b+653<<2)>>2]=c[a+(e-13<<2)>>2];c[a+(b+654<<2)>>2]=c[a+(e-14<<2)>>2];c[a+(b+655<<2)>>2]=c[a+(e-15<<2)>>2];f=f+1|0;}while((f|0)<8);return}function op(){var a=0,b=0,d=0.0,e=0.0,f=0,g=0,h=0,i=0,j=0,k=0.0,l=0,m=0.0,n=0,o=0,p=0,q=0.0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0.0,y=0.0,z=0.0,A=0,B=0,C=0,D=0;a=0;while(1){b=((a|0)%3|0|0)==1;d=(+(a|0)+.5)*3.141592653589793/36.0;e=+((a<<1)+19|0)*3.141592653589793/72.0;f=(a|0)/3|0;g=(a|0)<18;h=g?a:a+2|0;i=(a|0)>29;j=(a|0)>23;k=(+(a-18|0)+.5)*3.141592653589793/12.0;l=(a|0)>17;m=(+(a-6|0)+.5)*3.141592653589793/12.0;if((a|0)<6){n=0;while(1){o=(n|0)==2;L22496:do{if(o){if(!b){p=16725;break}q=+V(+d);p=16723}else{r=+V(+d);if((n|0)==1){p=16716}else if((n|0)==3){s=.8795/+U(+e)*0.0;p=16726;break}else{t=r}do{if((p|0)==16716){p=0;if(i){t=0.0;break}if(j){q=+V(+k);p=16723;break L22496}else{q=l?1.0:r;p=16723;break L22496}}}while(0);s=t*(.8795/+U(+e));p=16726}}while(0);do{if((p|0)==16723){p=0;r=q*(.8795/+U(+e));if(!o){s=r;p=16726;break}c[2604168+(f<<2)>>2]=~~(r*.03125*4294967296.0+.5);p=16725}}while(0);if((p|0)==16726){p=0;c[2603848+(n*160|0)+(h<<2)>>2]=~~(s*.03125*4294967296.0+.5);o=n+1|0;if((o|0)<4){n=o;continue}else{break}}else if((p|0)==16725){p=0;n=n+1|0;continue}}}else{if((a|0)<12){n=0;while(1){o=(n|0)==2;do{if(o){if(!b){p=16739;break}u=+V(+d);p=16737}else{r=+V(+d);if((n|0)==3){v=+V(+m)}else if((n|0)==1){if(i){u=0.0;p=16737;break}if(j){u=+V(+k);p=16737;break}else{u=l?1.0:r;p=16737;break}}else{v=r}w=v*(.8795/+U(+e));p=16740}}while(0);do{if((p|0)==16737){p=0;r=u*(.8795/+U(+e));if(!o){w=r;p=16740;break}c[2604168+(f<<2)>>2]=~~(r*.03125*4294967296.0+.5);p=16739}}while(0);if((p|0)==16739){p=0;n=n+1|0;continue}else if((p|0)==16740){p=0;c[2603848+(n*160|0)+(h<<2)>>2]=~~(w*.03125*4294967296.0+.5);o=n+1|0;if((o|0)<4){n=o;continue}else{break}}}}else{n=0;while(1){o=(n|0)==2;L22519:do{if(o){if(!b){p=16753;break}x=+V(+d);p=16751}else{m=+V(+d);if((n|0)==3){x=g?1.0:m;p=16751;break}else if((n|0)==1){p=16745}else{y=m}do{if((p|0)==16745){p=0;if(i){y=0.0;break}if(j){x=+V(+k);p=16751;break L22519}else{x=l?1.0:m;p=16751;break L22519}}}while(0);z=y*(.8795/+U(+e));p=16754}}while(0);do{if((p|0)==16751){p=0;m=x*(.8795/+U(+e));if(!o){z=m;p=16754;break}c[2604168+(f<<2)>>2]=~~(m*.03125*4294967296.0+.5);p=16753}}while(0);if((p|0)==16753){p=0;n=n+1|0;continue}else if((p|0)==16754){p=0;c[2603848+(n*160|0)+(h<<2)>>2]=~~(z*.03125*4294967296.0+.5);o=n+1|0;if((o|0)<4){n=o;continue}else{break}}}}}n=a+1|0;if((n|0)<36){a=n}else{A=0;break}}while(1){c[2604488+(A<<2)>>2]=c[2603848+(A<<2)>>2];a=A|1;c[2604488+(a<<2)>>2]=-(c[2603848+(a<<2)>>2]|0);a=A+2|0;if((a|0)<40){A=a}else{B=0;break}}while(1){c[2604648+(B<<2)>>2]=c[2604008+(B<<2)>>2];A=B|1;c[2604648+(A<<2)>>2]=-(c[2604008+(A<<2)>>2]|0);A=B+2|0;if((A|0)<40){B=A}else{C=0;break}}while(1){c[2604808+(C<<2)>>2]=c[2604168+(C<<2)>>2];B=C|1;c[2604808+(B<<2)>>2]=-(c[2604168+(B<<2)>>2]|0);B=C+2|0;if((B|0)<40){C=B}else{D=0;break}}do{c[2604968+(D<<2)>>2]=c[2604328+(D<<2)>>2];C=D|1;c[2604968+(C<<2)>>2]=-(c[2604328+(C<<2)>>2]|0);D=D+2|0;}while((D|0)<40);return}function oq(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0;h=i;i=i+72|0;j=h|0;if((e|0)<=0){i=h;return}k=(f|0)!=0;f=j+64|0;l=j+68|0;m=d;d=0;n=b;b=a;while(1){a=-(d&1)&4;o=m+68|0;p=m+64|0;q=c[p>>2]|0;r=q+(c[o>>2]|0)|0;s=m+60|0;t=c[s>>2]|0;c[p>>2]=t+q;q=m+56|0;p=c[q>>2]|0;u=p+t|0;t=m+52|0;v=c[t>>2]|0;c[q>>2]=v+p;p=m+48|0;q=c[p>>2]|0;w=q+v|0;v=m+44|0;x=c[v>>2]|0;c[p>>2]=x+q;q=m+40|0;p=c[q>>2]|0;y=p+x|0;x=m+36|0;z=c[x>>2]|0;c[q>>2]=z+p;p=m+32|0;q=c[p>>2]|0;A=q+z|0;z=m+28|0;B=c[z>>2]|0;c[p>>2]=B+q;q=m+24|0;p=c[q>>2]|0;C=p+B|0;B=m+20|0;D=c[B>>2]|0;c[q>>2]=D+p;p=m+16|0;q=c[p>>2]|0;E=q+D|0;D=m+12|0;F=c[D>>2]|0;c[p>>2]=F+q;q=m+8|0;p=c[q>>2]|0;H=p+F|0;F=m+4|0;I=c[F>>2]|0;c[q>>2]=I+p;p=c[m>>2]|0;q=p+I|0;c[F>>2]=q;F=k&(d|0)<2?0:g;c[o>>2]=r+u;c[s>>2]=w+u;c[t>>2]=y+w;c[v>>2]=A+y;c[x>>2]=C+A;c[z>>2]=E+C;c[B>>2]=H+E;c[D>>2]=q+H;H=0;q=p;while(1){p=H+8|0;D=c[m+(p<<2)>>2]|0;E=H+16|0;B=c[m+(E<<2)>>2]|0;C=H+4|0;z=c[m+(C<<2)>>2]|0;A=B+D-z|0;x=H+12|0;y=c[m+(x<<2)>>2]|0;v=(y>>1)+q|0;w=q-y|0;y=H+6|0;c[j+(y<<2)>>2]=w-(A>>1);c[j+(E<<2)>>2]=w+A;A=z+D<<1;N2(A,(A|0)<0|0?-1:0,2017974537,0)|0;A=G;w=D-B|0;N2(w,(w|0)<0|0?-1:0,-745813244,-1)|0;w=G;D=z+B<<1;N2(D,(D|0)<0|0?-1:0,-1645067915,-1)|0;D=G;B=H+10|0;c[j+(B<<2)>>2]=v-A-D;z=H+2|0;c[j+(z<<2)>>2]=w+v+A;A=H+14|0;c[j+(A<<2)>>2]=v-w+D;D=c[m+(B<<2)>>2]|0;B=c[m+(A<<2)>>2]|0;A=c[m+(z<<2)>>2]|0;z=B+D-A<<1;N2(z,(z|0)<0|0?-1:0,-1859775393,-1)|0;c[j+(C<<2)>>2]=G;C=A+D<<1;N2(C,(C|0)<0|0?-1:0,2114858546,0)|0;C=G;z=D-B|0;N2(z,(z|0)<0|0?-1:0,-1468965330,-1)|0;z=G;D=c[m+(y<<2)>>2]<<1;N2(D,(D|0)<0|0?-1:0,1859775393,0)|0;D=G;y=A+B<<1;N2(y,(y|0)<0|0?-1:0,-1380375881,-1)|0;y=G;c[j+(H<<2)>>2]=C+z+D;c[j+(x<<2)>>2]=y+C-D;c[j+(p<<2)>>2]=z-y-D;D=H+1|0;if((D|0)>=2){break}H=D;q=c[m+(D<<2)>>2]|0}q=F+a|0;H=0;D=0;while(1){y=c[j+(D<<2)>>2]|0;z=c[j+((D|2)<<2)>>2]|0;p=z+y|0;C=z-y|0;y=c[j+((D|1)<<2)>>2]|0;z=c[j+((D|3)<<2)>>2]|0;x=z+y<<1;B=c[173256+(H<<2)>>2]|0;N2(x,(x|0)<0|0?-1:0,B,(B|0)<0|0?-1:0)|0;B=G;x=z-y|0;y=8-H|0;z=c[173336+(y<<2)>>2]|0;A=N2(z,(z|0)<0|0?-1:0,x,(x|0)<0|0?-1:0)|0;x=A>>>23|G<<9;A=B+p|0;z=p-B|0;B=H+9|0;p=c[2603848+(q*160|0)+(B<<2)>>2]|0;w=z;v=(z|0)<0|0?-1:0;N2(w,v,p,(p|0)<0|0?-1:0)|0;p=n+(B<<2<<2)|0;c[b+(B<<5<<2)>>2]=G+(c[p>>2]|0);B=c[2603848+(q*160|0)+(y<<2)>>2]|0;N2(w,v,B,(B|0)<0|0?-1:0)|0;B=n+(y<<2<<2)|0;c[b+(y<<5<<2)>>2]=G+(c[B>>2]|0);y=c[2603848+(q*160|0)+(H+29<<2)>>2]|0;v=A;w=(A|0)<0|0?-1:0;N2(v,w,y,(y|0)<0|0?-1:0)|0;c[p>>2]=G;p=c[2603848+(q*160|0)+(28-H<<2)>>2]|0;N2(p,(p|0)<0|0?-1:0,v,w)|0;c[B>>2]=G;B=x+C|0;w=C-x|0;x=17-H|0;C=c[2603848+(q*160|0)+(x<<2)>>2]|0;v=w;p=(w|0)<0|0?-1:0;N2(C,(C|0)<0|0?-1:0,v,p)|0;C=n+(x<<2<<2)|0;c[b+(x<<5<<2)>>2]=G+(c[C>>2]|0);x=c[2603848+(q*160|0)+(H<<2)>>2]|0;N2(x,(x|0)<0|0?-1:0,v,p)|0;p=n+(H<<2<<2)|0;c[b+(H<<5<<2)>>2]=G+(c[p>>2]|0);v=c[2603848+(q*160|0)+(37-H<<2)>>2]|0;x=B;w=(B|0)<0|0?-1:0;N2(v,(v|0)<0|0?-1:0,x,w)|0;c[C>>2]=G;C=c[2603848+(q*160|0)+(H+20<<2)>>2]|0;N2(C,(C|0)<0|0?-1:0,x,w)|0;c[p>>2]=G;p=H+1|0;if((p|0)<4){H=p;D=D+4|0}else{break}}D=c[f>>2]|0;H=c[l>>2]<<1;N2(H,(H|0)<0|0?-1:0,1518500250,0)|0;H=G;a=H+D|0;F=D-H|0;H=c[2603900+(q*160|0)>>2]|0;D=F;p=(F|0)<0|0?-1:0;N2(D,p,H,(H|0)<0|0?-1:0)|0;H=n+208|0;c[b+1664>>2]=G+(c[H>>2]|0);F=c[2603864+(q*160|0)>>2]|0;N2(D,p,F,(F|0)<0|0?-1:0)|0;F=n+64|0;c[b+512>>2]=G+(c[F>>2]|0);p=c[2603980+(q*160|0)>>2]|0;D=a;w=(a|0)<0|0?-1:0;N2(D,w,p,(p|0)<0|0?-1:0)|0;c[H>>2]=G;H=c[2603944+(q*160|0)>>2]|0;N2(D,w,H,(H|0)<0|0?-1:0)|0;c[F>>2]=G;F=n+(((d&3|0)!=3?1:69)<<2)|0;H=d+1|0;if((H|0)<(e|0)){m=m+72|0;d=H;n=F;b=b+4|0}else{break}}i=h;return}function or(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0.0,q=0.0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0,y=0.0,z=0.0,A=0.0,B=0.0,C=0.0,D=0.0,E=0.0,F=0.0,G=0.0;h=a+2048|0;i=a;NN(h|0,i|0,128)|0;g[e>>2]=+(c[d>>2]|0)+ +g[b>>2]*+g[a+64>>2]+ +g[b+256>>2]*+g[a+320>>2]+ +g[b+512>>2]*+g[a+576>>2]+ +g[b+768>>2]*+g[a+832>>2]+ +g[b+1024>>2]*+g[a+1088>>2]+ +g[b+1280>>2]*+g[a+1344>>2]+ +g[b+1536>>2]*+g[a+1600>>2]+ +g[b+1792>>2]*+g[a+1856>>2]- +g[b+128>>2]*+g[a+192>>2]- +g[b+384>>2]*+g[a+448>>2]- +g[b+640>>2]*+g[a+704>>2]- +g[b+896>>2]*+g[a+960>>2]- +g[b+1152>>2]*+g[a+1216>>2]- +g[b+1408>>2]*+g[a+1472>>2]- +g[b+1664>>2]*+g[a+1728>>2]- +g[b+1920>>2]*+g[a+1984>>2];i=-f|0;h=e+(f<<4<<2)|0;j=1;k=e+((f*31|0)<<2)|0;l=b+124|0;m=e;e=b;while(1){n=m+(f<<2)|0;o=e+4|0;p=+g[a+(j+16<<2)>>2];q=+g[a+(j+80<<2)>>2];r=+g[a+(j+144<<2)>>2];s=+g[a+(j+208<<2)>>2];t=+g[a+(j+272<<2)>>2];u=+g[a+(j+336<<2)>>2];v=+g[a+(j+400<<2)>>2];w=+g[a+(j+464<<2)>>2];x=48-j|0;y=+g[a+(x<<2)>>2];z=+g[a+(x+64<<2)>>2];A=+g[a+(x+128<<2)>>2];B=+g[a+(x+192<<2)>>2];C=+g[a+(x+256<<2)>>2];D=+g[a+(x+320<<2)>>2];E=+g[a+(x+384<<2)>>2];F=+g[a+(x+448<<2)>>2];G=0.0-p*+g[l>>2]-q*+g[l+256>>2]-r*+g[l+512>>2]-s*+g[l+768>>2]-t*+g[l+1024>>2]-u*+g[l+1280>>2]-v*+g[l+1536>>2]-w*+g[l+1792>>2]-y*+g[l+128>>2]-z*+g[l+384>>2]-A*+g[l+640>>2]-B*+g[l+896>>2]-C*+g[l+1152>>2]-D*+g[l+1408>>2]-E*+g[l+1664>>2]-F*+g[l+1920>>2];g[n>>2]=p*+g[o>>2]+0.0+q*+g[e+260>>2]+r*+g[e+516>>2]+s*+g[e+772>>2]+t*+g[e+1028>>2]+u*+g[e+1284>>2]+v*+g[e+1540>>2]+w*+g[e+1796>>2]-y*+g[e+132>>2]-z*+g[e+388>>2]-A*+g[e+644>>2]-B*+g[e+900>>2]-C*+g[e+1156>>2]-D*+g[e+1412>>2]-E*+g[e+1668>>2]-F*+g[e+1924>>2];g[k>>2]=G+0.0;x=j+1|0;if((x|0)<16){j=x;k=k+(i<<2)|0;l=l-4|0;m=n;e=o}else{break}}g[h>>2]=0.0- +g[b+192>>2]*+g[a+128>>2]- +g[b+448>>2]*+g[a+384>>2]- +g[b+704>>2]*+g[a+640>>2]- +g[b+960>>2]*+g[a+896>>2]- +g[b+1216>>2]*+g[a+1152>>2]- +g[b+1472>>2]*+g[a+1408>>2]- +g[b+1728>>2]*+g[a+1664>>2]- +g[b+1984>>2]*+g[a+1920>>2];c[d>>2]=0;return}function os(){var a=0,b=0,c=0.0,d=0.0,e=0,f=0,h=0,i=0,j=0,k=0.0,l=0,m=0.0,n=0.0,o=0.0,p=0.0,q=0,r=0,s=0,t=0;a=0;while(1){b=((a|0)%3|0|0)==1;c=(+(a|0)+.5)*3.141592653589793/36.0;d=+((a<<1)+19|0)*3.141592653589793/72.0;e=(a|0)/3|0;f=(a|0)<18;h=f?a:a+2|0;i=(a|0)>29;j=(a|0)>23;k=(+(a-18|0)+.5)*3.141592653589793/12.0;l=(a|0)>17;m=(+(a-6|0)+.5)*3.141592653589793/12.0;n=+V(+c);g[2602568+(h<<2)>>2]=n*(.8795/+U(+d))*.03125;n=+V(+c);do{if((a|0)<6){g[2602728+(h<<2)>>2]=(l?1.0:n)*(.8795/+U(+d))*.03125;if(b){o=+V(+c);g[2602888+(e<<2)>>2]=o*(.8795/+U(+d))*.03125}g[2603048+(h<<2)>>2]=.8795/+U(+d)*0.0*.03125}else{if((a|0)<12){g[2602728+(h<<2)>>2]=(l?1.0:n)*(.8795/+U(+d))*.03125;if(b){o=+V(+c);g[2602888+(e<<2)>>2]=o*(.8795/+U(+d))*.03125}o=+V(+m);g[2603048+(h<<2)>>2]=o*(.8795/+U(+d))*.03125;break}do{if(i){p=0.0}else{if(j){p=+V(+k);break}else{p=l?1.0:n;break}}}while(0);g[2602728+(h<<2)>>2]=p*(.8795/+U(+d))*.03125;if(b){o=+V(+c);g[2602888+(e<<2)>>2]=o*(.8795/+U(+d))*.03125}o=f?1.0:+V(+c);g[2603048+(h<<2)>>2]=o*(.8795/+U(+d))*.03125}}while(0);h=a+1|0;if((h|0)<36){a=h}else{q=0;break}}while(1){g[2603208+(q<<2)>>2]=+g[2602568+(q<<2)>>2];a=q|1;g[2603208+(a<<2)>>2]=-0.0- +g[2602568+(a<<2)>>2];a=q+2|0;if((a|0)<40){q=a}else{r=0;break}}while(1){g[2603368+(r<<2)>>2]=+g[2602728+(r<<2)>>2];q=r|1;g[2603368+(q<<2)>>2]=-0.0- +g[2602728+(q<<2)>>2];q=r+2|0;if((q|0)<40){r=q}else{s=0;break}}while(1){g[2603528+(s<<2)>>2]=+g[2602888+(s<<2)>>2];r=s|1;g[2603528+(r<<2)>>2]=-0.0- +g[2602888+(r<<2)>>2];r=s+2|0;if((r|0)<40){s=r}else{t=0;break}}do{g[2603688+(t<<2)>>2]=+g[2603048+(t<<2)>>2];s=t|1;g[2603688+(s<<2)>>2]=-0.0- +g[2603048+(s<<2)>>2];t=t+2|0;}while((t|0)<40);return}function ot(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0.0,s=0,t=0.0,u=0.0,v=0,w=0.0,x=0,y=0.0,z=0,A=0.0,B=0,C=0.0,D=0,E=0.0,F=0,G=0.0,H=0,I=0.0;h=i;i=i+72|0;j=h|0;if((d|0)<=0){i=h;return}k=(e|0)!=0;e=j+64|0;l=j+68|0;m=c;c=0;n=b;b=a;while(1){a=-(c&1)&4;o=m+68|0;p=m+64|0;q=+g[p>>2];r=+g[o>>2]+q;s=m+60|0;t=+g[s>>2];g[p>>2]=q+t;p=m+56|0;q=+g[p>>2];u=t+q;v=m+52|0;t=+g[v>>2];g[p>>2]=q+t;p=m+48|0;q=+g[p>>2];w=t+q;x=m+44|0;t=+g[x>>2];g[p>>2]=q+t;p=m+40|0;q=+g[p>>2];y=t+q;z=m+36|0;t=+g[z>>2];g[p>>2]=q+t;p=m+32|0;q=+g[p>>2];A=t+q;B=m+28|0;t=+g[B>>2];g[p>>2]=q+t;p=m+24|0;q=+g[p>>2];C=t+q;D=m+20|0;t=+g[D>>2];g[p>>2]=q+t;p=m+16|0;q=+g[p>>2];E=t+q;F=m+12|0;t=+g[F>>2];g[p>>2]=q+t;p=m+8|0;q=+g[p>>2];G=t+q;H=m+4|0;t=+g[H>>2];g[p>>2]=q+t;q=+g[m>>2];I=t+q;g[H>>2]=I;H=k&(c|0)<2?0:f;g[o>>2]=u+r;g[s>>2]=u+w;g[v>>2]=w+y;g[x>>2]=y+A;g[z>>2]=A+C;g[B>>2]=C+E;g[D>>2]=E+G;g[F>>2]=G+I;F=0;I=q;while(1){D=F+8|0;q=+g[m+(D<<2)>>2];B=F+16|0;G=+g[m+(B<<2)>>2];z=F+4|0;E=+g[m+(z<<2)>>2];C=q+G-E;x=F+12|0;A=+g[m+(x<<2)>>2];y=I+A*.5;w=I-A;v=F+6|0;g[j+(v<<2)>>2]=w-C*.5;g[j+(B<<2)>>2]=C+w;w=(q+E)*.9396926164627075;C=(q-G)*-.1736481785774231;q=(G+E)*-.7660444378852844;B=F+10|0;g[j+(B<<2)>>2]=y-w-q;s=F+2|0;g[j+(s<<2)>>2]=C+(w+y);o=F+14|0;g[j+(o<<2)>>2]=q+y-C;C=+g[m+(B<<2)>>2];y=+g[m+(o<<2)>>2];q=+g[m+(s<<2)>>2];g[j+(z<<2)>>2]=(C+y-q)*-.8660253882408142;w=(C+q)*.9848077297210693;E=(C-y)*-.3420201539993286;C=+g[m+(v<<2)>>2]*.8660253882408142;G=(y+q)*-.6427876353263855;g[j+(F<<2)>>2]=C+(E+w);g[j+(x<<2)>>2]=w+G-C;g[j+(D<<2)>>2]=E-G-C;D=F+1|0;if((D|0)>=2){break}F=D;I=+g[m+(D<<2)>>2]}F=H+a|0;D=0;x=0;while(1){I=+g[j+(x<<2)>>2];C=+g[j+((x|2)<<2)>>2];G=I+C;E=C-I;I=+g[j+((x|1)<<2)>>2];C=+g[j+((x|3)<<2)>>2];w=(I+C)*+g[173216+(D<<2)>>2]*2.0;v=8-D|0;q=(C-I)*+g[173296+(v<<2)>>2];I=G+w;C=G-w;z=D+9|0;s=n+(z<<2<<2)|0;g[b+(z<<5<<2)>>2]=+g[s>>2]+ +g[2602568+(F*160|0)+(z<<2)>>2]*C;z=n+(v<<2<<2)|0;g[b+(v<<5<<2)>>2]=C*+g[2602568+(F*160|0)+(v<<2)>>2]+ +g[z>>2];g[s>>2]=I*+g[2602568+(F*160|0)+(D+29<<2)>>2];g[z>>2]=I*+g[2602568+(F*160|0)+(28-D<<2)>>2];I=E+q;C=E-q;z=17-D|0;s=n+(z<<2<<2)|0;g[b+(z<<5<<2)>>2]=C*+g[2602568+(F*160|0)+(z<<2)>>2]+ +g[s>>2];z=n+(D<<2<<2)|0;g[b+(D<<5<<2)>>2]=C*+g[2602568+(F*160|0)+(D<<2)>>2]+ +g[z>>2];g[s>>2]=I*+g[2602568+(F*160|0)+(37-D<<2)>>2];g[z>>2]=I*+g[2602568+(F*160|0)+(D+20<<2)>>2];z=D+1|0;if((z|0)<4){D=z;x=x+4|0}else{break}}I=+g[e>>2];C=+g[l>>2]*.7071067690849304;q=I+C;E=I-C;x=n+208|0;g[b+1664>>2]=+g[x>>2]+ +g[2602620+(F*160|0)>>2]*E;D=n+64|0;g[b+512>>2]=E*+g[2602584+(F*160|0)>>2]+ +g[D>>2];g[x>>2]=q*+g[2602700+(F*160|0)>>2];g[D>>2]=q*+g[2602664+(F*160|0)>>2];D=n+(((c&3|0)!=3?1:69)<<2)|0;x=c+1|0;if((x|0)<(d|0)){m=m+72|0;c=x;n=D;b=b+4|0}else{break}}i=h;return}function ou(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;d=i;e=a;NM(e|0,0,944);c[a>>2]=346712;f=(b|0)!=0;do{if(f){g=c[b+8>>2]|0;c[a+8>>2]=g;c[a+48>>2]=c[b+12>>2];if((g|0)==1){h=8;break}else if((g|0)!=0){j=g;k=16812;break}h=16}else{c[a+8>>2]=-1;j=-1;k=16812}}while(0);if((k|0)==16812){h=(j|0)==3?32:0}Bs(e,h,h);c[a+104>>2]=0;c[a+108>>2]=1;c[a+484>>2]=78;c[a+152>>2]=74;c[a+824>>2]=8;c[a+828>>2]=32;c[a+224>>2]=0;c[a+228>>2]=1;c[a+140>>2]=-1;c[a+420>>2]=-1;h=a+616|0;c[h>>2]=-1;c[h+4>>2]=-1;h=a+704|0;c[h>>2]=0;c[h+4>>2]=-2147483648;if(!f){l=0;i=d;return l|0}f=c[b+52>>2]|0;do{if((f|0)!=0){h=a+60|0;j=c[h>>2]|0;do{if((j|0)==0){g=A7(f)|0;c[h>>2]=g;if((g|0)==0){l=-12}else{m=g;break}i=d;return l|0}else{m=j}}while(0);j=c[b+44>>2]|0;if((j|0)==0){break}c[m>>2]=j;Br(c[h>>2]|0)}}while(0);m=c[b+68>>2]|0;if((m|0)==0){l=0;i=d;return l|0}else{n=m}while(1){m=c[n>>2]|0;if((m|0)==0){l=0;k=16825;break}if((Bd(e,m,c[n+4>>2]|0,0)|0)>-1){n=n+8|0}else{k=16822;break}}if((k|0)==16825){i=d;return l|0}else if((k|0)==16822){AN(0,0,1664960,(k=i,i=i+24|0,c[k>>2]=1724608,c[k+8>>2]=1695752,c[k+16>>2]=144,k)|0);i=k;cw();return 0}return 0}function ov(a){a=a|0;var b=0,c=0,d=0;b=A1(944)|0;c=b;do{if((b|0)==0){d=0}else{if((ou(c,a)|0)>=0){d=c;break}A4(b);d=0}}while(0);return d|0}function ow(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;e=i;f=b;if((o7(b)|0)!=0){AN(f,16,1673936,(g=i,i=i+16|0,c[g>>2]=d,c[g+8>>2]=b,g)|0);i=g;h=-22;i=e;return h|0}g=d;NN(f|0,g|0,944)|0;c[b+60>>2]=0;c[b+12>>2]=0;c[b+220>>2]=0;c[b+712>>2]=0;c[b+832>>2]=0;c[b+64>>2]=0;g=b+536|0;c[g>>2]=0;f=b+96|0;c[f>>2]=0;j=b+300|0;c[j>>2]=0;k=b+304|0;c[k>>2]=0;l=b+532|0;c[l>>2]=0;m=c[d+536>>2]|0;do{if((m|0)!=0){n=A9(m)|0;c[g>>2]=n;if((n|0)==0){h=-12}else{break}i=e;return h|0}}while(0);m=d+96|0;do{if((c[m>>2]|0)==0){o=16840}else{n=d+100|0;p=c[n>>2]|0;if((p|0)<=0){o=16840;break}q=A1(p+16|0)|0;c[f>>2]=q;if((q|0)==0){break}p=c[m>>2]|0;r=c[n>>2]|0;NN(q|0,p|0,r)|0;NM(q+r|0,0,16);o=16840}}while(0);L22679:do{if((o|0)==16840){m=d+300|0;if((c[m>>2]|0)!=0){r=A1(128)|0;c[j>>2]=r;if((r|0)==0){break}q=c[m>>2]|0;NN(r|0,q|0,128)|0}q=d+304|0;if((c[q>>2]|0)!=0){r=A1(128)|0;c[k>>2]=r;if((r|0)==0){break}m=c[q>>2]|0;NN(r|0,m|0,128)|0}m=d+532|0;do{if((c[m>>2]|0)!=0){r=d+528|0;q=c[r>>2]<<4;if((q|0)==0){break}p=A1(q)|0;c[l>>2]=p;if((p|0)==0){break L22679}q=c[m>>2]|0;n=c[r>>2]<<4;NN(p|0,q|0,n)|0}}while(0);m=d+860|0;if((c[m>>2]|0)==0){h=0;i=e;return h|0}n=d+864|0;q=c[n>>2]|0;if((q|0)<=0){h=0;i=e;return h|0}p=A1(q+1|0)|0;c[b+860>>2]=p;if((p|0)==0){break}q=c[m>>2]|0;m=c[n>>2]|0;NN(p|0,q|0,m)|0;a[p+m|0]=0;h=0;i=e;return h|0}}while(0);A6(l);A6(j);A6(k);A6(f);A6(g);h=-12;i=e;return h|0}function ox(){return 346712}function oy(){return 346512}function oz(a){a=a|0;var b=0,d=0,e=0;do{if((a|0)!=0){b=c[a+12>>2]|0;if((b|0)==0){break}d=c[b>>2]|0;if((d|0)==0){break}else{e=d}return e|0}}while(0);e=1639048;return e|0}function oA(a,b){a=a|0;b=b|0;var d=0,e=0;do{if((b|0)==0){d=c[a+12>>2]|0;if((d|0)==0){break}if((c[d+44>>2]|0)==0){break}d=c[a+60>>2]|0;if((d|0)==0){break}else{e=d}return e|0}}while(0);e=0;return e|0}function oB(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;L22722:do{if((a|0)==0){b=0}else{d=0;while(1){e=oQ(d)|0;if((e|0)==0){b=0;break L22722}if((c[e+44>>2]|0)==(a|0)){b=e;break}else{d=e}}}}while(0);while(1){a=oQ(b)|0;if((a|0)==0){f=0;g=16886;break}d=c[a+44>>2]|0;if((d|0)==0){b=a}else{f=d;g=16885;break}}if((g|0)==16886){return f|0}else if((g|0)==16885){return f|0}return 0}function oC(a){a=a|0;var b=0,d=0;b=c[a+12>>2]|0;do{if((b|0)!=0){if((c[b+88>>2]|0)==0){break}else{d=6}return d|0}}while(0);d=5;return d|0}function oD(a){a=a|0;var b=0,d=0;b=a+40|0;d=a;do{a=c[753910]|0;c[b>>2]=a;}while((a|0)!=((E=c[753910]|0,(c[753910]|0)==(a|0)?(c[753910]=d)|0:0,E)|0));return}function oE(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;if((a|0)==0){b=0;return b|0}else{d=3015640}while(1){e=c[d>>2]|0;if((e|0)==0){b=0;f=16914;break}if((c[e>>2]|0)==(a|0)){break}if((c[e+4>>2]|0)==(a|0)){break}if((c[e+8>>2]|0)==(a|0)){break}if((c[e+12>>2]|0)==(a|0)){break}if((c[e+16>>2]|0)==(a|0)){break}else{d=e+40|0}}if((f|0)==16914){return b|0}f=A7(320)|0;d=f;if((f|0)!=0){c[f+4>>2]=e;a=A7(c[e+20>>2]|0)|0;c[f>>2]=a;do{if((a|0)!=0){c[f+72>>2]=1;c[f+32>>2]=1;g=c[e+24>>2]|0;if((g|0)!=0){if((c1[g&255](d)|0)!=0){break}}c[f+224>>2]=-1;g=f+232|0;c[g>>2]=0;c[g+4>>2]=0;c[f+240>>2]=-2147483648;c[f+244>>2]=-2147483648;c[f+248>>2]=-2147483648;b=d;return b|0}}while(0);A6(f)}A4(f);b=0;return b|0}function oF(a,b,d,e,f,g,h,j,k,l,m,n){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0;o=i;i=i+16|0;p=a+176|0;q=c[p>>2]|0;if((q&4|0)==0){r=a+16|0;c[r>>2]=m;c[r+4>>2]=n;r=a+24|0;c[r>>2]=m;c[r+4>>2]=n;c[p>>2]=q|4}do{if((g|0)==0){q=o|0;NM(q|0,0,16);s=q}else{q=a+16|0;p=c[q>>2]|0;r=c[q+4>>2]|0;t=g;u=(g|0)<0|0?-1:0;v=NU(p,r,t,u)|0;w=a+76|0;x=c[w>>2]|0;y=a+192+(x<<3)|0;if((v|0)==(c[y>>2]|0)&(G|0)==(c[y+4>>2]|0)){s=f;break}y=x+1&3;c[w>>2]=y;w=a+80+(y<<3)|0;c[w>>2]=p;c[w+4>>2]=r;r=NU(c[q>>2]|0,c[q+4>>2]|0,t,u)|0;u=a+192+(y<<3)|0;c[u>>2]=r;c[u+4>>2]=G;u=a+112+(y<<3)|0;c[u>>2]=h;c[u+4>>2]=j;u=a+144+(y<<3)|0;c[u>>2]=k;c[u+4>>2]=l;u=a+256+(y<<3)|0;c[u>>2]=m;c[u+4>>2]=n;s=f}}while(0);f=a+72|0;L22775:do{if((c[f>>2]|0)!=0){c[f>>2]=0;n=a+40|0;m=c[n+4>>2]|0;l=a+56|0;c[l>>2]=c[n>>2];c[l+4>>2]=m;m=a+48|0;l=c[m+4>>2]|0;k=a+64|0;c[k>>2]=c[m>>2];c[k+4>>2]=l;l=a+288|0;k=c[l+4>>2]|0;j=a+296|0;c[j>>2]=c[l>>2];c[j+4>>2]=k;c[n>>2]=0;c[n+4>>2]=-2147483648;c[m>>2]=0;c[m+4>>2]=-2147483648;c[l>>2]=-1;c[l+4>>2]=-1;k=a+184|0;c[k>>2]=0;c[k+4>>2]=0;j=a+16|0;h=a+8|0;u=a+24|0;y=c[j>>2]|0;r=c[j+4>>2]|0;j=a+80|0;t=c[j>>2]|0;q=c[j+4>>2]|0;do{if(!((r|0)<(q|0)|(r|0)==(q|0)&y>>>0<t>>>0)){j=c[h>>2]|0;w=c[h+4>>2]|0;if(!((w|0)<(q|0)|(w|0)==(q|0)&j>>>0<t>>>0)){if(!((j|0)==0&(w|0)==0)){break}if(!((c[u>>2]|0)==0&(c[u+4>>2]|0)==0)){break}}w=a+192|0;j=c[w>>2]|0;p=c[w+4>>2]|0;if((j|0)==0&(p|0)==0){break}w=a+144|0;x=c[w+4>>2]|0;c[m>>2]=c[w>>2];c[m+4>>2]=x;x=a+112|0;w=c[x+4>>2]|0;c[n>>2]=c[x>>2];c[n+4>>2]=w;w=a+256|0;x=c[w+4>>2]|0;c[l>>2]=c[w>>2];c[l+4>>2]=x;x=NV(c[u>>2]|0,c[u+4>>2]|0,t,q)|0;c[k>>2]=x;c[k+4>>2]=G;if((r|0)<(p|0)|(r|0)==(p|0)&y>>>0<j>>>0){break L22775}}}while(0);q=a+88|0;t=c[q>>2]|0;j=c[q+4>>2]|0;do{if(!((r|0)<(j|0)|(r|0)==(j|0)&y>>>0<t>>>0)){q=c[h>>2]|0;p=c[h+4>>2]|0;if(!((p|0)<(j|0)|(p|0)==(j|0)&q>>>0<t>>>0)){if(!((q|0)==0&(p|0)==0)){break}if(!((c[u>>2]|0)==0&(c[u+4>>2]|0)==0)){break}}p=a+200|0;q=c[p>>2]|0;x=c[p+4>>2]|0;if((q|0)==0&(x|0)==0){break}p=a+152|0;w=c[p+4>>2]|0;c[m>>2]=c[p>>2];c[m+4>>2]=w;w=a+120|0;p=c[w+4>>2]|0;c[n>>2]=c[w>>2];c[n+4>>2]=p;p=a+264|0;w=c[p+4>>2]|0;c[l>>2]=c[p>>2];c[l+4>>2]=w;w=NV(c[u>>2]|0,c[u+4>>2]|0,t,j)|0;c[k>>2]=w;c[k+4>>2]=G;if((r|0)<(x|0)|(r|0)==(x|0)&y>>>0<q>>>0){break L22775}}}while(0);j=a+96|0;t=c[j>>2]|0;q=c[j+4>>2]|0;do{if(!((r|0)<(q|0)|(r|0)==(q|0)&y>>>0<t>>>0)){j=c[h>>2]|0;x=c[h+4>>2]|0;if(!((x|0)<(q|0)|(x|0)==(q|0)&j>>>0<t>>>0)){if(!((j|0)==0&(x|0)==0)){break}if(!((c[u>>2]|0)==0&(c[u+4>>2]|0)==0)){break}}x=a+208|0;j=c[x>>2]|0;w=c[x+4>>2]|0;if((j|0)==0&(w|0)==0){break}x=a+160|0;p=c[x+4>>2]|0;c[m>>2]=c[x>>2];c[m+4>>2]=p;p=a+128|0;x=c[p+4>>2]|0;c[n>>2]=c[p>>2];c[n+4>>2]=x;x=a+272|0;p=c[x+4>>2]|0;c[l>>2]=c[x>>2];c[l+4>>2]=p;p=NV(c[u>>2]|0,c[u+4>>2]|0,t,q)|0;c[k>>2]=p;c[k+4>>2]=G;if((r|0)<(w|0)|(r|0)==(w|0)&y>>>0<j>>>0){break L22775}}}while(0);q=a+104|0;t=c[q>>2]|0;j=c[q+4>>2]|0;if((r|0)<(j|0)|(r|0)==(j|0)&y>>>0<t>>>0){break}q=c[h>>2]|0;w=c[h+4>>2]|0;if(!((w|0)<(j|0)|(w|0)==(j|0)&q>>>0<t>>>0)){if(!((q|0)==0&(w|0)==0)){break}if(!((c[u>>2]|0)==0&(c[u+4>>2]|0)==0)){break}}w=a+216|0;if((c[w>>2]|0)==0&(c[w+4>>2]|0)==0){break}w=a+168|0;q=c[w+4>>2]|0;c[m>>2]=c[w>>2];c[m+4>>2]=q;q=a+136|0;w=c[q+4>>2]|0;c[n>>2]=c[q>>2];c[n+4>>2]=w;w=a+280|0;q=c[w+4>>2]|0;c[l>>2]=c[w>>2];c[l+4>>2]=q;q=NV(c[u>>2]|0,c[u+4>>2]|0,t,j)|0;c[k>>2]=q;c[k+4>>2]=G}}while(0);q=c0[c[(c[a+4>>2]|0)+28>>2]&15](a,b,d,e,s,g)|0;if((c[e>>2]|0)==0){e=a+16|0;z=c[e+4>>2]|0;A=c[e>>2]|0;B=(q|0)<0;C=B?0:q;D=C;E=(C|0)<0|0?-1:0;F=a+16|0;H=NU(A,z,D,E)|0;I=G;J=F|0;c[J>>2]=H;K=F+4|0;c[K>>2]=I;i=o;return C|0}else{e=a+24|0;g=c[e+4>>2]|0;s=a+8|0;c[s>>2]=c[e>>2];c[s+4>>2]=g;g=a+16|0;s=c[g>>2]|0;d=c[g+4>>2]|0;g=NU(s,d,q,(q|0)<0|0?-1:0)|0;c[e>>2]=g;c[e+4>>2]=G;c[f>>2]=1;z=d;A=s;B=(q|0)<0;C=B?0:q;D=C;E=(C|0)<0|0?-1:0;F=a+16|0;H=NU(A,z,D,E)|0;I=G;J=F|0;c[J>>2]=H;K=F+4|0;c[K>>2]=I;i=o;return C|0}return 0}function oG(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0;do{if((a|0)==0){i=f;j=g}else{k=c[(c[a+4>>2]|0)+36>>2]|0;if((k|0)==0){i=f;j=g;break}if((c[b+88>>2]&4194304|0)==0){if((c[b+92>>2]&8|0)==0){i=f;j=g;break}}l=c6[k&255](b,f,g)|0;i=f+l|0;j=g-l|0}}while(0);c[d>>2]=i;c[e>>2]=j;g=b+96|0;if((c[g>>2]|0)==0|(h|0)==0){m=0;return m|0}if((c[b+92>>2]&8|0)==0){m=0;return m|0}h=b+100|0;b=(c[h>>2]|0)+j|0;c[e>>2]=b;e=A1(b+16|0)|0;c[d>>2]=e;d=c[g>>2]|0;g=c[h>>2]|0;NN(e|0,d|0,g)|0;d=e+g|0;g=j+16|0;NN(d|0,i|0,g)|0;m=1;return m|0}function oH(a){a=a|0;var b=0;if((a|0)==0){return}b=c[(c[a+4>>2]|0)+32>>2]|0;if((b|0)!=0){c_[b&127](a)}A4(c[a>>2]|0);A4(a);return}function oI(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;h=b+24|0;if((c[h>>2]|0)>0){i=b+28|0;j=b|0;k=b+4|0;while(1){l=c[i>>2]|0;c[i>>2]=l+1;m=c[j>>2]|0;n=a[m+l|0]|0;l=c[k>>2]|0;c[k>>2]=l+1;a[m+l|0]=n;n=(c[h>>2]|0)-1|0;c[h>>2]=n;if((n|0)<=0){o=k;break}}}else{o=b+4|0}k=(c[g>>2]|0)==0&(e|0)==-100?0:e;e=c[o>>2]|0;j=b+8|0;c[j>>2]=e;if((k|0)==-100){i=b|0;n=oO(c[i>>2]|0,b+12|0,(c[g>>2]|0)+16+e|0)|0;if((n|0)==0){p=-12;return p|0}c[i>>2]=n;i=n+(c[o>>2]|0)|0;n=c[f>>2]|0;l=c[g>>2]|0;NN(i|0,n|0,l)|0;c[o>>2]=(c[o>>2]|0)+(c[g>>2]|0);p=-1;return p|0}l=e+k|0;c[b+28>>2]=l;c[g>>2]=l;l=c[o>>2]|0;if((l|0)!=0){g=b|0;e=k+16|0;n=oO(c[g>>2]|0,b+12|0,e+l|0)|0;if((n|0)==0){p=-12;return p|0}c[g>>2]=n;if((k|0)>-16){l=n+(c[o>>2]|0)|0;i=c[f>>2]|0;NN(l|0,i|0,e)|0;q=c[g>>2]|0}else{q=n}c[o>>2]=0;c[f>>2]=q}if((k|0)>=0){p=0;return p|0}q=b+16|0;f=b+32|0;o=c[j>>2]|0;j=c[b>>2]|0;b=k;k=c[q>>2]|0;n=c[f+4>>2]|0;g=c[f>>2]|0;e=c[h>>2]|0;while(1){i=j+(o+b)|0;l=d[i]|0|k<<8;c[q>>2]=l;m=d[i]|0|(g<<8|0>>>24);i=n<<8|g>>>24|0;c[f>>2]=m;c[f+4>>2]=i;r=e+1|0;c[h>>2]=r;s=b+1|0;if((s|0)<0){b=s;k=l;n=i;g=m;e=r}else{p=0;break}}return p|0}function oJ(a){a=a|0;A6(c[a>>2]|0);return}function oK(a){a=a|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;d=i;e=c[a+60>>2]|0;if((c[a+416>>2]|0)<1){AN(a,16,1740672,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;g=-22;i=d;return g|0}f=a+48|0;h=c[f>>2]|0;if((h|0)==65543){j=e;k=0;do{l=(k&255^85)&255;m=l>>>4&7;n=l<<1&30;if((m|0)==0){o=n<<3|8}else{o=(n|33)<<m+2}b[j+(k<<1)>>1]=((l&128|0)!=0?o:-o|0)&65535;k=k+1|0;}while((k|0)<256)}else if((h|0)==65542){h=e;e=0;do{k=~(e&255)&255;o=(k<<3&120|132)<<(k>>>4&7);b[h+(e<<1)>>1]=((k&128|0)!=0?132-o|0:o+65404|0)&65535;e=e+1|0;}while((e|0)<256)}e=c[c[(c[a+12>>2]|0)+32>>2]>>2]|0;c[a+420>>2]=e;if((e|0)!=2){g=0;i=d;return g|0}c[a+796>>2]=pc(c[f>>2]|0)|0;g=0;i=d;return g|0}function oL(e,f,g,h){e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0;j=i;k=c[h+24>>2]|0;l=c[h+28>>2]|0;h=c[e+60>>2]|0;m=f;n=e+48|0;o=pc(c[n>>2]|0)|0;p=c[n>>2]|0;do{if((p|0)==65555){q=c[e+792>>2]|0;if((q|0)==20|(q|0)==24){r=(q<<1|0)/8|0;s=2;break}AN(e,16,1639448,(t=i,i=i+8|0,c[t>>2]=q,t)|0);i=t;u=-22;i=j;return u|0}else{q=(p|0)==65561;r=q?5:(o|0)/8|0;s=q?2:1}}while(0);if((r|0)==0){AN(e,16,1630352,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;u=-22;i=j;return u|0}o=e+416|0;q=c[o>>2]|0;if((q|0)==0){AN(e,16,1624e3,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;u=-22;i=j;return u|0}if((p|0)!=(c[(c[e+12>>2]|0)+12>>2]|0)){AN(e,16,1618144,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;u=-22;i=j;return u|0}p=ac(q,r)|0;do{if((p|0)==0){v=l}else{w=(l|0)%(p|0)|0;if((w|0)==0){v=l;break}if((l|0)>=(p|0)){v=l-w|0;break}AN(e,16,1613552,(t=i,i=i+16|0,c[t>>2]=l,c[t+8>>2]=p,t)|0);i=t;u=-1094995529;i=j;return u|0}}while(0);t=(v|0)/(r|0)|0;c[f+76>>2]=(ac(t,s)|0)/(q|0)|0;q=oX(e,m,0)|0;if((q|0)<0){u=q;i=j;return u|0}q=c[f>>2]|0;m=c[n>>2]|0;L22919:do{if((m|0)==65559|(m|0)==65557|(m|0)==65544|(m|0)==65536|(m|0)==65541){n=ac(t,r)|0;NN(q|0,k|0,n)|0}else if((m|0)==65554|(m|0)==542135120){n=c[o>>2]|0;if((n|0)<=0){break}s=f+64|0;p=ac((t|0)/(n|0)|0,r)|0;n=k;l=0;while(1){w=c[(c[s>>2]|0)+(l<<2)>>2]|0;NN(w|0,n|0,p)|0;w=l+1|0;if((w|0)<(c[o>>2]|0)){n=n+p|0;l=w}else{break}}}else if((m|0)==65539){if((t|0)>0){x=k;y=q;z=t}else{break}while(1){l=x;b[y>>1]=(cT((d[l]|d[l+1|0]<<8)<<16>>16|0)|0)^-32768;l=z-1|0;if((l|0)>0){x=x+2|0;y=y+2|0;z=l}else{break}}}else if((m|0)==65538){if((t|0)>0){A=k;B=q;C=t}else{break}while(1){l=A;b[B>>1]=(d[l]|d[l+1|0]<<8)<<16>>16^-32768;l=C-1|0;if((l|0)>0){A=A+2|0;B=B+2|0;C=l}else{break}}}else if((m|0)==65555){l=q;p=c[o>>2]|0;n=(t|0)/(p|0)|0;s=c[e+792>>2]|0;if((s|0)==20){if((n|0)==0){break}else{D=k;E=l;F=n;G=p}while(1){w=F-1|0;H=D+(G<<2)|0;if((G|0)==0){I=E;J=H}else{K=G<<1;L=G*5|0;M=D;N=E;O=G;P=H;while(1){H=O-1|0;Q=M;R=((cT((d[Q]|d[Q+1|0]<<8)<<16>>16|0)|0)&65535)<<16;c[N>>2]=(d[P]|0)<<8&61440|R;R=M+2|0;Q=((cT((d[R]|d[R+1|0]<<8)<<16>>16|0)|0)&65535)<<16;c[N+4>>2]=(d[P]|0)<<12&61440|Q;if((H|0)==0){break}else{M=M+4|0;N=N+8|0;O=H;P=P+1|0}}I=E+(K<<2)|0;J=D+L|0}if((w|0)==0){break L22919}D=J;E=I;F=w;G=c[o>>2]|0}}else if((s|0)==24){if((n|0)==0){break}else{S=k;T=l;U=n;V=p}while(1){P=U-1|0;O=S+(V<<2)|0;if((V|0)==0){W=T;X=O}else{N=V<<1;M=V*6|0;H=S;Q=T;R=V;Y=O;while(1){O=R-1|0;Z=H;_=((cT((d[Z]|d[Z+1|0]<<8)<<16>>16|0)|0)&65535)<<16;c[Q>>2]=(d[Y]|0)<<8|_;_=H+2|0;Z=((cT((d[_]|d[_+1|0]<<8)<<16>>16|0)|0)&65535)<<16;c[Q+4>>2]=(d[Y+1|0]|0)<<8|Z;if((O|0)==0){break}else{H=H+4|0;Q=Q+8|0;R=O;Y=Y+2|0}}W=T+(N<<2)|0;X=S+M|0}if((P|0)==0){break L22919}S=X;T=W;U=P;V=c[o>>2]|0}}else{break}}else if((m|0)==65563){p=c[o>>2]|0;n=(t|0)/(p|0)|0;if((p|0)<=0){break}l=f+64|0;if((n|0)>0){$=k;aa=0}else{s=0;while(1){s=s+1|0;if((s|0)>=(p|0)){break L22919}}}while(1){p=$;s=c[(c[l>>2]|0)+(aa<<2)>>2]|0;Y=n;while(1){a[s]=a[p]^-128;R=Y-1|0;if((R|0)>0){p=p+1|0;s=s+1|0;Y=R}else{break}}Y=aa+1|0;if((Y|0)<(c[o>>2]|0)){$=$+n|0;aa=Y}else{break}}}else if((m|0)==65550){if((t|0)>0){ab=k;ad=q;ae=t}else{break}while(1){c[ad>>2]=((d[ab+1|0]|0)<<8|(d[ab+2|0]|0)<<16|(d[ab]|0))<<8^-2147483648;n=ae-1|0;if((n|0)>0){ab=ab+3|0;ad=ad+4|0;ae=n}else{break}}}else if((m|0)==65549){if((t|0)>0){af=k;ag=q;ah=t}else{break}while(1){c[ag>>2]=((d[af+1|0]|0)<<8|(d[af]|0)<<16|(d[af+2|0]|0))<<8;n=ah-1|0;if((n|0)>0){af=af+3|0;ag=ag+4|0;ah=n}else{break}}}else if((m|0)==65546){if((t|0)>0){ai=k;aj=q;ak=t}else{break}while(1){n=ai;c[aj>>2]=(d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24)^-2147483648;n=ak-1|0;if((n|0)>0){ai=ai+4|0;aj=aj+4|0;ak=n}else{break}}}else if((m|0)==1347637264){n=c[o>>2]|0;l=(t|0)/(n|0)|0;if((n|0)<=0){break}Y=f+64|0;if((l|0)<=0){s=0;while(1){s=s+1|0;if((s|0)>=(n|0)){break L22919}}}n=l<<1;s=k;p=0;while(1){P=s;M=c[(c[Y>>2]|0)+(p<<2)>>2]|0;N=l;while(1){R=P;b[M>>1]=cT((d[R]|d[R+1|0]<<8)<<16>>16|0)|0;R=N-1|0;if((R|0)>0){P=P+2|0;M=M+2|0;N=R}else{break}}N=p+1|0;if((N|0)<(c[o>>2]|0)){s=s+n|0;p=N}else{break}}}else if((m|0)==65561){p=c[o>>2]|0;n=(t|0)/(p|0)|0;if((p|0)<=0){break}s=f+64|0;l=(n|0)>0;Y=n*5|0;N=k;M=0;P=p;while(1){if(l){p=N;R=c[(c[s>>2]|0)+(M<<2)>>2]|0;Q=0;while(1){H=p+2|0;w=d[H]|0;L=d[p+1|0]|0;c[R>>2]=L|w<<28|L<<20|w<<8&3840|(d[p]|0)<<12;w=d[p+4|0]|0;L=d[p+3|0]|0;c[R+4>>2]=w<<4|w<<24|L<<16|L>>>4|(d[H]|0)<<8&61440;H=Q+1|0;if((H|0)<(n|0)){p=p+5|0;R=R+8|0;Q=H}else{break}}al=N+Y|0;am=c[o>>2]|0}else{al=N;am=P}Q=M+1|0;if((Q|0)<(am|0)){N=al;M=Q;P=am}else{break}}}else if((m|0)==65553){if((t|0)>0){an=k;ao=q;ap=t}else{break}while(1){P=a[an]|0;a[ao]=P<<24>>24>-1?-128-P&255:P;P=ap-1|0;if((P|0)>0){an=an+1|0;ao=ao+1|0;ap=P}else{break}}}else if((m|0)==65558){if((t|0)>0){aq=k;ar=q;as=t}else{break}while(1){P=aq;M=P|0;N=P+4|0;P=d[N]|d[N+1|0]<<8|d[N+2|0]<<16|d[N+3|0]<<24|0;N=cV(d[M]|d[M+1|0]<<8|d[M+2|0]<<16|d[M+3|0]<<24|0)|0;M=ar;c[M>>2]=((P>>>16|0<<16)<<8|0>>>24)&65280|(P>>>24|0<<8)|(P<<8|0>>>24|(P>>>8|0<<24)&255)<<16;c[M+4>>2]=N;N=as-1|0;if((N|0)>0){aq=aq+8|0;ar=ar+8|0;as=N}else{break}}}else if((m|0)==65556|(m|0)==65545){if((t|0)>0){at=k;au=q;av=t}else{break}while(1){N=at;c[au>>2]=cV(d[N]|d[N+1|0]<<8|d[N+2|0]<<16|d[N+3|0]<<24|0)|0;N=av-1|0;if((N|0)>0){at=at+4|0;au=au+4|0;av=N}else{break}}}else if((m|0)==65537){if((t|0)>0){aw=k;ax=q;ay=t}else{break}while(1){N=aw;b[ax>>1]=cT((d[N]|d[N+1|0]<<8)<<16>>16|0)|0;N=ay-1|0;if((N|0)>0){aw=aw+2|0;ax=ax+2|0;ay=N}else{break}}}else if((m|0)==407917392){N=c[o>>2]|0;M=(t|0)/(N|0)|0;if((N|0)<=0){break}P=f+64|0;if((M|0)<=0){Y=0;while(1){Y=Y+1|0;if((Y|0)>=(N|0)){break L22919}}}N=M*3|0;Y=k;n=0;while(1){s=Y;l=M;Q=c[(c[P>>2]|0)+(n<<2)>>2]|0;while(1){c[Q>>2]=((d[s+1|0]|0)<<8|(d[s+2|0]|0)<<16|(d[s]|0))<<8;R=l-1|0;if((R|0)>0){s=s+3|0;l=R;Q=Q+4|0}else{break}}Q=n+1|0;if((Q|0)<(c[o>>2]|0)){Y=Y+N|0;n=Q}else{break}}}else if((m|0)==65548){if((t|0)>0){az=k;aA=q;aB=t}else{break}while(1){c[aA>>2]=((d[az+1|0]|0)<<8|(d[az+2|0]|0)<<16|(d[az]|0))<<8;n=aB-1|0;if((n|0)>0){az=az+3|0;aA=aA+4|0;aB=n}else{break}}}else if((m|0)==65547){if((t|0)>0){aC=k;aD=q;aE=t}else{break}while(1){n=aC;c[aD>>2]=(cV(d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0)|0)^-2147483648;n=aE-1|0;if((n|0)>0){aC=aC+4|0;aD=aD+4|0;aE=n}else{break}}}else if((m|0)==65543|(m|0)==65542){if((t|0)<=0){break}n=h;N=k;Y=q;P=t;while(1){b[Y>>1]=b[n+((d[N]|0)<<1)>>1]|0;M=P-1|0;if((M|0)>0){N=N+1|0;Y=Y+2|0;P=M}else{break}}}else if((m|0)==65552){if((t|0)>0){aF=k;aG=q;aH=t}else{break}while(1){P=(d[aF+1|0]|0)<<8;b[aG>>1]=(d[242544+((P|(d[aF+2|0]|0))>>>4&255)|0]|0)<<8|(d[242544+((P|(d[aF]|0)<<16)>>>12&255)|0]|0);P=aH-1|0;if((P|0)>0){aF=aF+3|0;aG=aG+2|0;aH=P}else{break}}}else if((m|0)==65551){if((t|0)>0){aI=k;aJ=q;aK=t}else{break}while(1){c[aJ>>2]=((d[aI+1|0]|0)<<8|(d[aI]|0)<<16|(d[aI+2|0]|0))<<8^-2147483648;P=aK-1|0;if((P|0)>0){aI=aI+3|0;aJ=aJ+4|0;aK=P}else{break}}}else if((m|0)==65540){if((t|0)>0){aL=k;aM=q;aN=t}else{break}while(1){a[aM]=a[aL]^-128;P=aN-1|0;if((P|0)>0){aL=aL+1|0;aM=aM+1|0;aN=P}else{break}}}else{u=-1;i=j;return u|0}}while(0);c[g>>2]=1;u=v;i=j;return u|0}function oM(a){a=a|0;var b=0,d=0,e=0,f=0;b=242800;while(1){d=c[b>>2]|0;if((d|0)<=-1){e=0;f=17120;break}if((d|0)==(a|0)){break}else{b=b+8|0}}if((f|0)==17120){return e|0}e=c[b+4>>2]|0;return e|0}function oN(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;d=a;while(1){a=c[d>>2]|0;if((a|0)<=-1){e=-1;f=17125;break}if((c[d+4>>2]|0)==(b|0)){e=a;f=17126;break}else{d=d+8|0}}if((f|0)==17125){return e|0}else if((f|0)==17126){return e|0}return 0}function oO(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;if((c[b>>2]|0)>>>0>d>>>0){e=a;return e|0}f=((d*17|0)>>>4)+32|0;g=f>>>0>d>>>0?f:d;d=A2(a,g)|0;c[b>>2]=(d|0)==0?0:g;e=d;return e|0}function oP(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=a;if((c[b>>2]|0)>>>0>d>>>0){return}a=((d*17|0)>>>4)+32|0;f=a>>>0>d>>>0?a:d;A4(c[e>>2]|0);d=A1(f)|0;c[e>>2]=d;c[b>>2]=(d|0)==0?0:f;return}function oQ(a){a=a|0;return c[((a|0)==0?2225224:a+56|0)>>2]|0}function oR(a){a=a|0;var b=0;do{if((a|0)==0){b=0}else{if((c[a+80>>2]|0)!=0){b=1;break}b=(c[a+84>>2]|0)!=0|0}}while(0);return b|0}function oS(a){a=a|0;var b=0;if((a|0)==0){b=0;return b|0}b=(c[a+88>>2]|0)!=0|0;return b|0}function oT(b){b=b|0;var d=0,e=0,f=0;if(!(a[340904]|0)){a[340904]=1}c[b+56>>2]=0;d=b;if(((E=c[556306]|0,(c[556306]|0)==0?(c[556306]=d)|0:0,E)|0)!=0){e=2225224;do{e=(c[e>>2]|0)+56|0;f=e;}while(((E=c[f>>2]|0,(c[f>>2]|0)==0?(c[f>>2]=d)|0:0,E)|0)!=0)}d=c[b+72>>2]|0;if((d|0)==0){return}c_[d&127](b);return}function oU(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;c[a+128>>2]=b;c[a+132>>2]=d;e=c[a+800>>2]|0;c[a+120>>2]=-(-b>>e);c[a+124>>2]=-(-d>>e);return}function oV(a,b,d){a=a|0;b=b|0;d=d|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;d=i;i=i+96|0;f=d|0;g=d+8|0;h=d+16|0;j=d+80|0;k=j;l=a+64|0;m=c[(c[l>>2]|0)+480>>2]|0;n=a+8|0;o=c[n>>2]|0;L23088:do{if((o|0)==1){p=An(b)|0;q=b+80|0;r=(B8(c[q>>2]|0)|0)!=0?p:1;s=m+16|0;do{if((c[s>>2]|0)==(c[q>>2]|0)){if((c[m+76>>2]|0)!=(r|0)){break}if((c[m+80>>2]|0)!=(p|0)){break}if((c[b+76>>2]|0)==(c[m+84>>2]|0)){break L23088}}}while(0);t=m|0;zn(t);u=m+60|0;v=b+76|0;w=B9(u,p,c[v>>2]|0,c[q>>2]|0,0)|0;if((w|0)<0){x=w;y=17194;break}w=zm(c[u>>2]|0,0)|0;c[t>>2]=w;if((w|0)==0){x=-12;y=17194;break}c[s>>2]=c[q>>2];c[m+76>>2]=r;c[m+80>>2]=p;c[m+84>>2]=c[v>>2]}else if((o|0)==0){NM(k|0,0,16);v=b+68|0;w=c[v>>2]|0;t=b+72|0;u=c[t>>2]|0;z=m+16|0;A=b+80|0;do{if((c[z>>2]|0)==(c[A>>2]|0)){if((c[m+20>>2]|0)!=(w|0)){break}if((c[m+24>>2]|0)==(u|0)){break L23088}}}while(0);p=m+28|0;r=a+140|0;L23103:do{switch(c[r>>2]|0){case 46:{q=(c[a+48>>2]|0)==43?4:1;B=q;C=q;break};case 6:{q=(c[a+48>>2]|0)==23?64:1;B=q;C=q;break};case 11:case 19:case 22:{q=c[a+48>>2]|0;if(!((q|0)==50|(q|0)==44)){B=1;C=1;break L23103}B=4;C=4;break};case 3:{q=((c[a+48>>2]|0)-54|0)>>>0<2?4:1;B=q;C=q;break};case 2:{q=(c[a+48>>2]|0)==44?4:1;B=q;C=q;break};case 7:case 320:case 18:{B=8;C=32;break};case 0:case 1:case 17:case 4:case 33:case 5:case 317:case 82:case 8:case 31:case 32:case 12:case 13:case 34:case 14:case 35:case 300:case 299:case 70:case 69:case 72:case 71:case 302:case 301:case 304:case 303:case 80:case 79:case 74:case 73:case 306:case 305:case 308:case 307:case 76:case 75:case 78:case 77:case 310:case 309:case 312:case 311:case 92:case 91:case 98:case 97:case 94:case 93:case 100:case 99:case 96:case 95:case 102:case 101:case 84:case 83:case 86:case 85:case 314:case 313:case 316:case 315:{B=32;C=16;break};default:{B=1;C=1}}}while(0);q=c[a+48>>2]|0;if((q-137|0)>>>0<2){D=C>>>0>8?C:8}else{D=C}s=w-1+D&-D;E=u-1+B&-B;if((q|0)==28){y=17174}else{if((c[a+800>>2]|0)==0){F=E}else{y=17174}}if((y|0)==17174){F=E+2|0}c[p>>2]=8;E=m+32|0;c[E>>2]=8;q=m+36|0;c[q>>2]=8;G=m+40|0;c[G>>2]=8;if((c[a+88>>2]&16384|0)==0){H=F+32|0;I=s+32|0}else{H=F;I=s}s=h+32|0;J=h+36|0;K=h+40|0;L=h+44|0;M=I;while(1){AC(s,c[r>>2]|0,M)|0;if(((c[J>>2]|0)%(c[E>>2]|0)|0|((c[s>>2]|0)%(c[p>>2]|0)|0)|((c[K>>2]|0)%(c[q>>2]|0)|0)|((c[L>>2]|0)%(c[G>>2]|0)|0)|0)==0){break}else{M=(M&-M)+M|0}}M=h|0;G=AD(M,c[r>>2]|0,H,0,s)|0;if((G|0)<0){N=-1;i=d;return N|0}q=c[h+4>>2]|0;p=c[M>>2]|0;do{if((q|0)==0){O=0;P=j|0;Q=p}else{M=q;E=p;u=j|0;c[u>>2]=M-E;w=c[h+8>>2]|0;if((w|0)==0){O=1;P=u;Q=E;break}R=w;c[j+4>>2]=R-M;M=c[h+12>>2]|0;if((M|0)==0){O=2;P=u;Q=E;break}c[j+8>>2]=M-R;O=3;P=u;Q=E}}while(0);c[j+(O<<2)>>2]=Q+G-(c[h+(O<<2)>>2]|0);p=m|0;zn(p);c[m+60>>2]=c[s>>2];q=c[P>>2]|0;if((q|0)!=0){r=zm(q+23|0,72)|0;c[p>>2]=r;if((r|0)==0){x=-12;y=17194;break}}r=m+4|0;zn(r);c[m+64>>2]=c[J>>2];p=c[j+4>>2]|0;if((p|0)!=0){q=zm(p+23|0,72)|0;c[r>>2]=q;if((q|0)==0){x=-12;y=17194;break}}q=m+8|0;zn(q);c[m+68>>2]=c[K>>2];r=c[j+8>>2]|0;if((r|0)!=0){p=zm(r+23|0,72)|0;c[q>>2]=p;if((p|0)==0){x=-12;y=17194;break}}p=m+12|0;zn(p);c[m+72>>2]=c[L>>2];q=c[j+12>>2]|0;if((q|0)!=0){r=zm(q+23|0,72)|0;c[p>>2]=r;if((r|0)==0){x=-12;y=17194;break}}c[z>>2]=c[A>>2];c[m+20>>2]=c[v>>2];c[m+24>>2]=c[t>>2]}else{AN(0,0,1655944,(S=i,i=i+24|0,c[S>>2]=1618136,c[S+8>>2]=1694056,c[S+16>>2]=469,S)|0);i=S;cw();return 0}}while(0);if((y|0)==17194){zn(m|0);zn(m+4|0);zn(m+8|0);zn(m+12|0);c[m+16>>2]=-1;c[m+84>>2]=0;c[m+80>>2]=0;c[m+76>>2]=0;c[m+24>>2]=0;c[m+20>>2]=0;N=x;i=d;return N|0}c[b+288>>2]=1;x=c[n>>2]|0;if((x|0)==1){n=c[(c[l>>2]|0)+480>>2]|0;m=c[n+76>>2]|0;c[b+32>>2]=c[n+60>>2];do{if((m|0)>8){j=b+64|0;c[j>>2]=A7(m<<2)|0;P=m-8|0;c[b+396>>2]=P;O=A7(P<<2)|0;P=b+392|0;c[P>>2]=O;if(!((c[j>>2]|0)==0|(O|0)==0)){T=8;y=17227;break}A6(j);A6(P);N=-12;i=d;return N|0}else{c[b+64>>2]=b;if((c[b+396>>2]|0)==0){if((m|0)>0){T=m;y=17227;break}else{break}}else{AN(0,0,1655944,(S=i,i=i+24|0,c[S>>2]=1614048,c[S+8>>2]=1694056,c[S+16>>2]=501,S)|0);i=S;cw();return 0}}}while(0);L23162:do{if((y|0)==17227){m=n|0;P=b+64|0;j=0;while(1){O=zo(c[m>>2]|0)|0;c[b+360+(j<<2)>>2]=O;if((O|0)==0){break}h=c[O+4>>2]|0;c[b+(j<<2)>>2]=h;c[(c[P>>2]|0)+(j<<2)>>2]=h;h=j+1|0;if((h|0)<(T|0)){j=h}else{y=17229;break}}L23167:do{if((y|0)==17229){j=b+396|0;if((c[b+396>>2]|0)<=0){break L23162}P=n|0;m=b+392|0;t=b+64|0;v=0;while(1){A=zo(c[P>>2]|0)|0;c[(c[m>>2]|0)+(v<<2)>>2]=A;A=c[(c[m>>2]|0)+(v<<2)>>2]|0;if((A|0)==0){break L23167}c[(c[t>>2]|0)+(v+8<<2)>>2]=c[A+4>>2];v=v+1|0;if((v|0)>=(c[j>>2]|0)){break L23162}}}}while(0);Au(b);N=-12;i=d;return N|0}}while(0);if((c[a+688>>2]&32768|0)==0){N=0;i=d;return N|0}AN(a,48,1613672,(S=i,i=i+8|0,c[S>>2]=b,S)|0);i=S;N=0;i=d;return N|0}else if((x|0)==0){x=c[(c[l>>2]|0)+480>>2]|0;l=BP(c[b+80>>2]|0)|0;n=b|0;if((c[n>>2]|0)!=0){AN(a,16,1612768,(S=i,i=i+1|0,i=i+7&-8,c[S>>2]=0,S)|0);i=S;N=-1;i=d;return N|0}T=(e[l+8>>1]|0)>>>2&7;NM(b|0,0,32);c[b+64>>2]=n;n=a+140|0;BS(c[n>>2]|0,f,g)|0;l=a+88|0;j=x+8|0;v=(T<<4)+16|0;T=0;while(1){t=x+(T<<2)|0;if((c[t>>2]|0)==0){U=T;y=17215;break}m=(T|0)==0;P=m?0:c[f>>2]|0;A=m?0:c[g>>2]|0;m=b+32+(T<<2)|0;c[m>>2]=c[x+60+(T<<2)>>2];z=zo(c[t>>2]|0)|0;c[b+360+(T<<2)>>2]=z;if((z|0)==0){y=17221;break}do{if((c[l>>2]&16384|0)==0){if((c[j>>2]|0)==0){y=17212;break}t=c[x+28+(T<<2)>>2]|0;c[b+(T<<2)>>2]=(c[z+4>>2]|0)+((v>>P)-1+t+(c[m>>2]<<4>>A)&-t)}else{y=17212}}while(0);if((y|0)==17212){y=0;c[b+(T<<2)>>2]=c[z+4>>2]}V=T+1|0;if((V|0)<4){T=V}else{y=17208;break}}if((y|0)==17221){Au(b);N=-12;i=d;return N|0}else if((y|0)==17208){if((V|0)<8){U=V;y=17215}}if((y|0)==17215){while(1){y=0;c[b+(U<<2)>>2]=0;c[b+32+(U<<2)>>2]=0;V=U+1|0;if((V|0)<8){U=V;y=17215}else{break}}}y=c[b+4>>2]|0;do{if((y|0)!=0){if((c[b+8>>2]|0)!=0){break}U=y;V=c[n>>2]|0;AE(U,V)|0}}while(0);if((c[a+688>>2]&32768|0)==0){N=0;i=d;return N|0}AN(a,48,1612464,(S=i,i=i+8|0,c[S>>2]=b,S)|0);i=S;N=0;i=d;return N|0}else{N=-1;i=d;return N|0}return 0}function oW(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;d=i;e=a+872|0;f=c[e>>2]|0;if((f|0)==0){g=b+144|0;c[g>>2]=0;c[g+4>>2]=-2147483648;Al(b,-1,-1);Aj(b,0,0);Aq(b,-1)}else{g=f+8|0;f=c[g+4>>2]|0;h=b+144|0;c[h>>2]=c[g>>2];c[h+4>>2]=f;f=(c[e>>2]|0)+64|0;Al(b,c[f>>2]|0,c[f+4>>2]|0);f=c[(c[e>>2]|0)+48>>2]|0;Aj(b,f,(f|0)<0|0?-1:0);Aq(b,c[(c[e>>2]|0)+28>>2]|0)}e=a+704|0;f=c[e+4>>2]|0;h=b+320|0;c[h>>2]=c[e>>2];c[h+4>>2]=f;f=c[(c[a+12>>2]|0)+8>>2]|0;if((f|0)==1){h=b+344|0;if((c[h>>2]|0)==0){c[h>>2]=c[a+412>>2]}h=b+80|0;if((c[h>>2]|0)<0){c[h>>2]=c[a+420>>2]}h=b+352|0;do{if((c[h>>2]|0)==0&(c[h+4>>2]|0)==0){e=a+448|0;g=c[e>>2]|0;j=c[e+4>>2]|0;if((g|0)==0&(j|0)==0){k=a+416|0;l=c[k>>2]|0;if(l>>>0<=63){m=k;break}AN(a,16,1658096,(n=i,i=i+8|0,c[n>>2]=l,n)|0);i=n;o=-38;i=d;return o|0}l=zs(g,j)|0;j=a+416|0;if((l|0)==(c[j>>2]|0)){l=c[e+4>>2]|0;c[h>>2]=c[e>>2];c[h+4>>2]=l;m=j;break}AN(a,16,1672792,(n=i,i=i+1|0,i=i+7&-8,c[n>>2]=0,n)|0);i=n;o=-22;i=d;return o|0}else{m=a+416|0}}while(0);Ao(b,c[m>>2]|0);o=0;i=d;return o|0}else if((f|0)==0){f=c[a+120>>2]|0;m=a+800|0;n=-(-(c[a+128>>2]|0)>>c[m>>2])|0;c[b+68>>2]=(f|0)>(n|0)?f:n;n=c[a+124>>2]|0;f=-(-(c[a+132>>2]|0)>>c[m>>2])|0;c[b+72>>2]=(n|0)>(f|0)?n:f;f=b+80|0;if((c[f>>2]|0)<0){c[f>>2]=c[a+140>>2]}f=b+124|0;if((c[f>>2]|0)!=0){o=0;i=d;return o|0}b=a+224|0;a=f;f=c[b+4>>2]|0;c[a>>2]=c[b>>2];c[a+4>>2]=f;o=0;i=d;return o|0}else{o=0;i=d;return o|0}return 0}function oX(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;g=i;i=i+16|0;h=g|0;j=g+8|0;k=h;l=b+8|0;do{if((c[l>>2]|0)==0){m=b;if((AF(c[b+120>>2]|0,c[b+124>>2]|0,0,m)|0)>=0){if((c[b+140>>2]|0)>=0){n=17282;break}}AN(m,16,1614712,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;p=-22}else{n=17282}}while(0);L23254:do{if((n|0)==17282){m=oW(b,e)|0;if((m|0)<0){p=m;break}m=b+472|0;q=c[m>>2]|0;if((q|0)==0){r=c6[c[b+484>>2]&255](b,e,f)|0;if((c[l>>2]|0)==0){c[e+68>>2]=c[b+120>>2];c[e+72>>2]=c[b+124>>2]}if((r|0)<0){p=r;break}else{s=r}i=g;return s|0}c[h>>2]=0;c[j>>2]=0;if((f&1|0)==0){t=q}else{c[e+172>>2]=1;t=c[m>>2]|0}m=dg[t&255](b,e)|0;if((m|0)<0){p=m;break}L23268:do{if((c[e+360>>2]|0)==0){m=A7(1400)|0;c[h>>2]=m;L23270:do{if((m|0)==0){u=-12}else{q=b;NN(m|0,q|0,944)|0;q=m+944|0;r=e;NN(q|0,r|0,456)|0;r=ze(0,0,30,m,0)|0;c[j>>2]=r;if((r|0)==0){u=-12;break}r=e+80|0;q=c[r>>2]|0;L23273:do{if((c[l>>2]|0)==0){v=BP(q)|0;w=BT(c[r>>2]|0)|0;if((v|0)==0){u=-22;break L23270}x=(a[v+7|0]&8)==0?w:1;if((x|0)<1){u=-22;break L23270}w=v+6|0;v=e+72|0;y=0;while(1){if((y|0)>=(x|0)){break L23273}if((y-1|0)>>>0<2){z=d[w]|0}else{z=0}A=c[v>>2]|0;B=c[e+32+(y<<2)>>2]|0;C=zi(c[j>>2]|0)|0;if((C|0)==0){u=-12;break L23270}D=ac(A>>z,B)|0;B=ze(c[e+(y<<2)>>2]|0,D,48,C,0)|0;c[e+360+(y<<2)>>2]=B;if((B|0)==0){break}else{y=y+1|0}}Au(e);u=-12;break L23270}else{do{if((B8(q)|0)==0){E=1}else{y=c[b+416>>2]|0;if(y>>>0<=8){E=y;break}v=y-8|0;c[e+396>>2]=v;y=A1(v<<2)|0;c[e+392>>2]=y;if((y|0)==0){u=-12;break L23270}else{E=8}}}while(0);y=e+64|0;v=e+32|0;w=0;while(1){if(w>>>0>=E>>>0){break}x=zi(c[j>>2]|0)|0;if((x|0)==0){u=-12;break L23270}B=ze(c[(c[y>>2]|0)+(w<<2)>>2]|0,c[v>>2]|0,48,x,0)|0;c[e+360+(w<<2)>>2]=B;if((B|0)==0){n=17308;break}else{w=w+1|0}}if((n|0)==17308){Au(e);u=-12;break L23270}w=e+396|0;B=e+392|0;x=0;while(1){if((x|0)>=(c[w>>2]|0)){break L23273}C=zi(c[j>>2]|0)|0;if((C|0)==0){u=-12;break L23270}D=ze(c[(c[y>>2]|0)+(x+8<<2)>>2]|0,c[v>>2]|0,48,C,0)|0;c[(c[B>>2]|0)+(x<<2)>>2]=D;if((c[(c[B>>2]|0)+(x<<2)>>2]|0)==0){break}else{x=x+1|0}}Au(e);u=-12;break L23270}}while(0);zj(j);break L23268}}while(0);c$[c[b+476>>2]&127](b,e);A6(k);zj(j);p=u;break L23254}}while(0);c[e+68>>2]=c[b+120>>2];c[e+72>>2]=c[b+124>>2];s=0;i=g;return s|0}}while(0);AN(b,16,1639384,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;s=p;i=g;return s|0}function oY(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;if((f|0)<=0){return 0}if((e|0)==0){h=0;do{i=d+(ac(h,g)|0)|0;dg[b&255](a,i)|0;h=h+1|0;}while((h|0)<(f|0));return 0}else{h=0;do{i=d+(ac(h,g)|0)|0;c[e+(h<<2)>>2]=dg[b&255](a,i)|0;h=h+1|0;}while((h|0)<(f|0));return 0}return 0}function oZ(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0;if((f|0)<=0){return 0}if((e|0)==0){g=0;do{dk[b&63](a,d,g,0)|0;g=g+1|0;}while((g|0)<(f|0));return 0}else{g=0;do{c[e+(g<<2)>>2]=dk[b&63](a,d,g,0)|0;g=g+1|0;}while((g|0)<(f|0));return 0}return 0}function o_(b,d){b=b|0;d=d|0;var e=0,f=0;b=d;while(1){d=c[b>>2]|0;if((d|0)==-1){e=-1;f=17345;break}if((a[(BP(d)|0)+7|0]&8)==0){break}else{b=b+4|0}}if((f|0)==17345){return e|0}e=c[b>>2]|0;return e|0}function o$(a){a=a|0;var b=0;NM(a|0,0,456);b=a+144|0;c[b>>2]=0;c[b+4>>2]=-2147483648;b=a+152|0;c[b>>2]=0;c[b+4>>2]=-2147483648;b=a+136|0;c[b>>2]=0;c[b+4>>2]=-2147483648;Ah(a,0,-2147483648);Aj(a,0,0);Al(a,-1,-1);Aq(a,-1);c[a+84>>2]=1;c[a+124>>2]=0;c[a+128>>2]=1;c[a+80>>2]=-1;c[a+64>>2]=a;return}function o0(){var a=0,b=0,d=0,e=0;a=A1(456)|0;b=a;if((a|0)==0){d=0;return d|0}e=a+144|0;NM(a|0,0,456);c[e>>2]=0;c[e+4>>2]=-2147483648;e=a+152|0;c[e>>2]=0;c[e+4>>2]=-2147483648;e=a+136|0;c[e>>2]=0;c[e+4>>2]=-2147483648;Ah(b,0,-2147483648);Aj(b,0,0);Al(b,-1,-1);Aq(b,-1);c[a+84>>2]=1;c[a+124>>2]=0;c[a+128>>2]=1;c[a+80>>2]=-1;c[a+64>>2]=a;d=b;return d|0}function o1(a){a=a|0;var b=0,d=0;if((a|0)==0){return}b=c[a>>2]|0;if((b|0)==0){return}d=b+64|0;if((c[d>>2]|0)!=(b|0)){A6(d)}A6(a);return}function o2(a,b){a=a|0;b=b|0;var d=0,e=0;d=i;e=b;b=i;i=i+8|0;c[b>>2]=c[e>>2];c[b+4>>2]=c[e+4>>2];e=b;b=a+888|0;a=c[e+4>>2]|0;c[b>>2]=c[e>>2];c[b+4>>2]=a;i=d;return}function o3(a){a=a|0;return c[a+896>>2]|0}function o4(a,b){a=a|0;b=b|0;c[a+896>>2]=b;return}function o5(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0;f=i;i=i+1800|0;g=f|0;h=f+8|0;j=f+136|0;k=f+264|0;l=f+776|0;m=f+1288|0;c[g>>2]=0;n=a+64|0;if((c[n>>2]|0)!=0){o=0;i=f;return o|0}p=a+12|0;q=c[p>>2]|0;r=(q|0)==0;do{if((b|0)==0){if(!r){s=q;break}AN(a,16,1613512,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;o=-22;i=f;return o|0}else{if(r|(q|0)==(b|0)){s=b;break}u=c[b>>2]|0;AN(a,16,1740592,(t=i,i=i+16|0,c[t>>2]=c[q>>2],c[t+8>>2]=u,t)|0);i=t;o=-22;i=f;return o|0}}while(0);if((c[a+100>>2]|0)>>>0>268435439){o=-22;i=f;return o|0}q=(e|0)!=0;if(q){zM(g,c[e>>2]|0,0)}b=o6(a)|0;if((b|0)<0){o=b;i=f;return o|0}b=A7(504)|0;c[n>>2]=b;L23379:do{if((b|0)==0){v=-12}else{r=A7(88)|0;c[(c[n>>2]|0)+480>>2]=r;L23381:do{if((c[(c[n>>2]|0)+480>>2]|0)==0){w=-12;x=a+60|0}else{r=c[s+52>>2]|0;u=a+60|0;do{if((r|0)>0){do{if((c[u>>2]|0)==0){y=A7(r)|0;c[u>>2]=y;if((y|0)==0){v=-12;break L23379}z=s+44|0;A=c[z>>2]|0;if((A|0)==0){B=z;break}c[y>>2]=A;Br(y);B=z}else{B=s+44|0}}while(0);if((c[B>>2]|0)==0){break}z=Bv(c[u>>2]|0,g)|0;if((z|0)<0){w=z;x=u;break L23381}}else{c[u>>2]=0}}while(0);r=a;z=Bv(r,g)|0;if((z|0)<0){w=z;x=u;break}z=a+128|0;y=c[z>>2]|0;L23395:do{if((y|0)==0){C=0;D=17394}else{A=a+132|0;E=c[A>>2]|0;if((E|0)==0){C=y;D=17394;break}F=c[a+120>>2]|0;do{if((F|0)!=0){if((c[a+124>>2]|0)==0){break}G=c[a+48>>2]|0;if((G|0)==28|(G|0)==93){H=y;I=F;break L23395}}}while(0);c[z>>2]=y;c[A>>2]=E;F=c[a+800>>2]|0;G=-(-y>>F)|0;c[a+120>>2]=G;c[a+124>>2]=-(-E>>F);H=y;I=G}}while(0);do{if((D|0)==17394){y=a+120|0;G=c[y>>2]|0;if((G|0)==0){H=C;I=0;break}F=a+124|0;J=c[F>>2]|0;if((J|0)==0){H=C;I=G;break}c[z>>2]=G;c[a+132>>2]=J;K=c[a+800>>2]|0;L=-(-G>>K)|0;c[y>>2]=L;c[F>>2]=-(-J>>K);H=G;I=L}}while(0);L=c[a+132>>2]|0;do{if((H|L|0)==0){if((I|0)!=0){M=0;D=17400;break}if((c[a+124>>2]|0)!=0){M=0;D=17400}}else{M=L;D=17400}}while(0);do{if((D|0)==17400){L=a+132|0;G=a+120|0;if((AF(H,M,0,r)|0)<0){N=a+124|0}else{K=a+124|0;if((AF(c[G>>2]|0,c[K>>2]|0,0,r)|0)<0){N=K}else{break}}AN(r,24,1736152,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;c[z>>2]=0;c[L>>2]=0;c[G>>2]=0;c[N>>2]=0}}while(0);z=(s|0)==0;do{if(!z){if((c[s+88>>2]|0)==0){break}A6(a+860|0)}}while(0);G=a+416|0;if((c[G>>2]|0)>>>0>63){w=-22;x=u;break}c[p>>2]=s;L=a+8|0;K=c[L>>2]|0;do{if((K|0)==-1){D=17411}else{if((K|0)==(c[s+8>>2]|0)){D=17411;break}O=K;P=c[a+48>>2]|0}}while(0);do{if((D|0)==17411){J=a+48|0;F=c[J>>2]|0;if((F|0)!=0){O=K;P=F;break}F=c[s+8>>2]|0;c[L>>2]=F;y=c[s+12>>2]|0;c[J>>2]=y;O=F;P=y}}while(0);K=a+48|0;y=s+12|0;do{if((P|0)==(c[y>>2]|0)){if(!((O|0)==(c[s+8>>2]|0)|(O|0)==4)){break}c[a+428>>2]=0;F=a+896|0;c[F>>2]=nT(P)|0;do{if((c[(c[p>>2]|0)+16>>2]&512|0)!=0){if((c[a+680>>2]|0)<=-2){break}L23435:do{if(z){J=c[s>>2]|0;AN(r,16,1718496,(t=i,i=i+24|0,c[t>>2]=1722576,c[t+8>>2]=J,c[t+16>>2]=-2,t)|0);i=t;Q=c[y>>2]|0;R=1722576;D=17436}else{if((c[s+80>>2]|0)==0){S=(c[s+84>>2]|0)!=0?1726744:1722576}else{S=1726744}J=c[s>>2]|0;AN(r,16,1718496,(t=i,i=i+24|0,c[t>>2]=S,c[t+8>>2]=J,c[t+16>>2]=-2,t)|0);i=t;if((c[s+80>>2]|0)==0){J=c[y>>2]|0;if((c[s+84>>2]|0)==0){Q=J;R=S;D=17436;break}else{T=J}}else{T=c[y>>2]|0}J=c[556306]|0;if((T|0)==86079){U=1950507339}else if((T|0)==171){U=1160852272}else if((T|0)==172){U=4665933}else if((T|0)==86077){U=1330664787}else{U=T}if((J|0)==0){V=0;W=S;break}else{X=J;Y=0}while(1){if((c[X+80>>2]|0)==0){if((c[X+84>>2]|0)==0){Z=Y}else{D=17433}}else{D=17433}do{if((D|0)==17433){D=0;if((c[X+12>>2]|0)!=(U|0)){Z=Y;break}if((c[X+16>>2]&512|0)!=0&(Y|0)==0){Z=X}else{V=X;W=S;break L23435}}}while(0);J=c[X+56>>2]|0;if((J|0)==0){V=Z;W=S;break}else{X=J;Y=Z}}}}while(0);L23459:do{if((D|0)==17436){J=c[556306]|0;if((Q|0)==86079){_=1950507339}else if((Q|0)==171){_=1160852272}else if((Q|0)==172){_=4665933}else if((Q|0)==86077){_=1330664787}else{_=Q}if((J|0)==0){V=0;W=R;break}else{$=J;aa=0}while(1){do{if((c[$+88>>2]|0)==0){ab=aa}else{if((c[$+12>>2]|0)!=(_|0)){ab=aa;break}if((c[$+16>>2]&512|0)!=0&(aa|0)==0){ab=$}else{V=$;W=R;break L23459}}}while(0);J=c[$+56>>2]|0;if((J|0)==0){V=ab;W=R;break}else{$=J;aa=ab}}}}while(0);if((c[V+16>>2]&512|0)!=0){w=-733130664;x=u;break L23381}J=c[V>>2]|0;AN(r,16,1714240,(t=i,i=i+16|0,c[t>>2]=W,c[t+8>>2]=J,t)|0);i=t;w=-733130664;x=u;break L23381}}while(0);do{if((c[L>>2]|0)==1){E=a+104|0;A=a+108|0;if((c[E>>2]|0)!=0){if((c[A>>2]|0)!=0){break}}c[E>>2]=1;c[A>>2]=c[a+412>>2]}}while(0);AN(r,24,1711696,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;if((c[s+16>>2]&32768|0)==0){c[a+808>>2]=1}A=c[p>>2]|0;E=d[A+40|0]|0;J=c[a+800>>2]|0;if((E|0)<(J|0)|(J|0)<0){AN(r,16,1708264,(t=i,i=i+8|0,c[t>>2]=E,t)|0);i=t;w=-22;x=u;break L23381}do{if((A|0)!=0){if((c[A+80>>2]|0)==0){if((c[A+84>>2]|0)==0){break}}E=c[A+32>>2]|0;do{if((E|0)==0){ad=A}else{J=c[E>>2]|0;L23495:do{if((J|0)==-1){ae=0;af=A}else{ag=a+420|0;ah=0;ai=J;aj=A;while(1){ak=c[ag>>2]|0;if((ak|0)==(ai|0)){ae=ah;af=aj;break L23495}if((c[G>>2]|0)==1){al=B5(ak)|0;ak=(al|0)==(B5(c[(c[(c[p>>2]|0)+32>>2]|0)+(ah<<2)>>2]|0)|0);am=c[p>>2]|0;if(ak){break}else{an=am}}else{an=aj}ak=ah+1|0;al=c[(c[an+32>>2]|0)+(ak<<2)>>2]|0;if((al|0)==-1){ae=ak;af=an;break L23495}else{ah=ak;ai=al;aj=an}}c[ag>>2]=c[(c[am+32>>2]|0)+(ah<<2)>>2];ae=ah;af=am}}while(0);if((c[(c[af+32>>2]|0)+(ae<<2)>>2]|0)!=-1){ad=af;break}J=h|0;aj=a+420|0;ai=c[aj>>2]|0;bc(J|0,128,1706224,(t=i,i=i+8|0,c[t>>2]=ai,t)|0)|0;i=t;ai=B2(c[aj>>2]|0)|0;AN(r,16,1704248,(t=i,i=i+8|0,c[t>>2]=(ai|0)!=0?ai:J,t)|0);i=t;w=-22;x=u;break L23381}}while(0);E=c[ad+24>>2]|0;L23507:do{if((E|0)!=0){J=a+140|0;ai=0;while(1){aj=c[E+(ai<<2)>>2]|0;if((aj|0)==-1){break}if((c[J>>2]|0)==(aj|0)){break L23507}else{ai=ai+1|0}}ai=c[K>>2]|0;if((ai|0)==8|(ai|0)==10){if((c[a+680>>2]|0)<0){break}}ai=j|0;bc(ai|0,128,1706224,(t=i,i=i+8|0,c[t>>2]=c[J>>2],t)|0)|0;i=t;aj=BL(c[J>>2]|0)|0;AN(r,16,1701880,(t=i,i=i+8|0,c[t>>2]=(aj|0)!=0?aj:ai,t)|0);i=t;w=-22;x=u;break L23381}}while(0);E=c[ad+28>>2]|0;L23517:do{if((E|0)!=0){ai=a+412|0;aj=0;while(1){al=c[E+(aj<<2)>>2]|0;ao=c[ai>>2]|0;if((al|0)==0){break}if((ao|0)==(al|0)){break L23517}else{aj=aj+1|0}}AN(r,16,1698696,(t=i,i=i+8|0,c[t>>2]=ao,t)|0);i=t;w=-22;x=u;break L23381}}while(0);E=c[ad+36>>2]|0;aj=a+448|0;L23524:do{if((E|0)!=0){ai=c[aj>>2]|0;J=c[aj+4>>2]|0;if((ai|0)==0&(J|0)==0){AN(r,24,1696680,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;break}else{ap=0}while(1){al=E+(ap<<3)|0;ak=c[al>>2]|0;aq=c[al+4>>2]|0;if((ak|0)==0&(aq|0)==0){break}if((ai|0)==(ak|0)&(J|0)==(aq|0)){break L23524}else{ap=ap+1|0}}aq=k|0;zt(aq,512,-1,ai,J);AN(r,16,1694400,(t=i,i=i+8|0,c[t>>2]=aq,t)|0);i=t;w=-22;x=u;break L23381}}while(0);E=c[aj>>2]|0;aq=c[aj+4>>2]|0;do{if(!((E|0)==0&(aq|0)==0)){ak=(c[G>>2]|0)==0;al=zs(E,aq)|0;if(ak){c[G>>2]=al;break}if((al|0)==(c[G>>2]|0)){break}ak=l|0;zt(ak,512,-1,c[aj>>2]|0,c[aj+4>>2]|0);ar=c[G>>2]|0;AN(r,16,1690800,(t=i,i=i+24|0,c[t>>2]=ak,c[t+8>>2]=al,c[t+16>>2]=ar,t)|0);i=t;w=-22;x=u;break L23381}}while(0);aj=c[L>>2]|0;do{if((aj|0)==0){if((c[K>>2]|0)==62){D=17498;break}if((c[a+120>>2]|0)>=1){if((c[a+124>>2]|0)>=1){D=17497;break}}AN(r,16,1688752,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;w=-22;x=u;break L23381}else{D=17497}}while(0);if((D|0)==17497){if(aj>>>0<2){D=17498}}do{if((D|0)==17498){aq=c[a+72>>2]|0;if((aq-1|0)>>>0>=999){break}AN(r,24,1686064,(t=i,i=i+16|0,c[t>>2]=aq,c[t+8>>2]=aq,t)|0);i=t}}while(0);aj=a+564|0;if((c[aj>>2]|0)!=0){break}c[aj>>2]=((c[a+524>>2]|0)*3|0|0)/4|0}}while(0);A=a+928|0;NM(a+904|0,0,16);c[A>>2]=0;c[A+4>>2]=-2147483648;A=a+920|0;c[A>>2]=0;c[A+4>>2]=-2147483648;A=c[p>>2]|0;aj=c[A+76>>2]|0;do{if((aj|0)==0){as=A}else{if((c[a+816>>2]&1|0)!=0){if((c[(c[n>>2]|0)+492>>2]|0)==0){as=A;break}}aq=c1[aj&255](a)|0;if((aq|0)<0){w=aq;x=u;break L23381}as=c[p>>2]|0}}while(0);if((as|0)==0){v=0;break L23379}if((c[as+88>>2]|0)==0){v=0;break L23379}aj=a+72|0;if((c[aj>>2]|0)==0){L23565:do{if((c[L>>2]|0)==1){A=c[K>>2]|0;L23567:do{switch(A|0){case 69649:{at=2;break};case 69648:{at=3;break};case 69647:case 69633:case 69632:case 69645:case 69638:{at=4;break};default:{if((A|0)==65543|(A|0)==65542|(A|0)==65540|(A|0)==65563|(A|0)==65541|(A|0)==65553){at=8;break L23567}else if((A|0)==65537|(A|0)==1347637264|(A|0)==65536|(A|0)==65554|(A|0)==65539|(A|0)==65538){at=16;break L23567}else if((A|0)==65552|(A|0)==65549|(A|0)==65548|(A|0)==407917392|(A|0)==65551|(A|0)==65550){at=24;break L23567}else if((A|0)==65545|(A|0)==65544|(A|0)==542135120|(A|0)==65547|(A|0)==65546|(A|0)==65556|(A|0)==65557){at=32;break L23567}else if((A|0)==65558|(A|0)==65559){at=64;break L23567}else if((A|0)==86071|(A|0)==86072|(A|0)==69644|(A|0)==69661|(A|0)==69655|(A|0)==1330333984|(A|0)==69636|(A|0)==69660|(A|0)==69646){at=4;break L23567}else{au=0;break L23565}}}}while(0);A=c[G>>2]|0;au=ac(ac(c[a+412>>2]|0,at)|0,A)|0}else{au=0}}while(0);c[aj>>2]=au}A=a+448|0;aq=c[A>>2]|0;E=c[A+4>>2]|0;do{if((aq|0)==0&(E|0)==0){D=17527}else{ar=zs(aq,E)|0;al=c[G>>2]|0;if((al|0)==0){c[G>>2]=ar;av=ar;break}if((ar|0)==(al|0)){D=17527;break}al=m|0;zt(al,512,-1,c[A>>2]|0,c[A+4>>2]|0);ak=c[G>>2]|0;AN(r,24,1684112,(t=i,i=i+24|0,c[t>>2]=al,c[t+8>>2]=ar,c[t+16>>2]=ak,t)|0);i=t;c[A>>2]=0;c[A+4>>2]=0;D=17527}}while(0);if((D|0)==17527){av=c[G>>2]|0}if((av|0)<0|av>>>0>63){w=-22;x=u;break L23381}A=a+936|0;E=c[A>>2]|0;if((E|0)==0){v=0;break L23379}if((c[L>>2]|0)!=3){AN(r,16,1682336,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;w=-22;x=u;break L23381}aq=c[F>>2]|0;if((c[aq+16>>2]&65536|0)!=0){AN(r,24,1680464,(t=i,i=i+8|0,c[t>>2]=c[aq+8>>2],t)|0);i=t;c[a+940>>2]=-1;v=0;break L23379}aq=a+940|0;aj=c[aq>>2]|0;if((aj|0)==0){c[aq>>2]=1}else if((aj|0)!=1){v=0;break L23379}aj=Nv(1678608,E)|0;if((aj|0)==-1){AN(r,16,1676496,(t=i,i=i+8|0,c[t>>2]=c[A>>2],t)|0);i=t;w=-(c[(ch()|0)>>2]|0)|0;x=u;break L23381}else{Nx(aj)|0;v=0;break L23379}}}while(0);AN(r,16,1731208,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;w=-22;x=u}}while(0);zL(g);A6(x);L=c[n>>2]|0;if((L|0)!=0){A6(L+480|0)}A6(n);c[p>>2]=0;v=w}}while(0);if((c[716810]|0)==0){AN(0,0,1655944,(t=i,i=i+24|0,c[t>>2]=1617256,c[t+8>>2]=1694056,c[t+16>>2]=3059,t)|0);i=t;cw();return 0}c[716810]=0;c[744094]=(c[744094]|0)-1;t=c[551446]|0;if((t|0)!=0){dg[t&255](2981344,2)|0}if(!q){o=v;i=f;return o|0}zL(e);c[e>>2]=c[g>>2];o=v;i=f;return o|0}function o6(a){a=a|0;var b=0,d=0,e=0,f=0;b=i;d=c[551446]|0;do{if((d|0)!=0){if((dg[d&255](2981344,1)|0)==0){break}else{e=-1}i=b;return e|0}}while(0);c[744094]=(c[744094]|0)+1;if((c[744094]|0)==1){if((c[716810]|0)!=0){AN(0,0,1655944,(f=i,i=i+24|0,c[f>>2]=1617704,c[f+8>>2]=1694056,c[f+16>>2]=3052,f)|0);i=f;cw();return 0}c[716810]=1;e=0;i=b;return e|0}AN(a,16,1618192,(f=i,i=i+1|0,i=i+7&-8,c[f>>2]=0,f)|0);i=f;c[716810]=1;if((c[716810]|0)==0){AN(0,0,1655944,(f=i,i=i+24|0,c[f>>2]=1617256,c[f+8>>2]=1694056,c[f+16>>2]=3059,f)|0);i=f;cw();return 0}c[716810]=0;c[744094]=(c[744094]|0)-1;f=c[551446]|0;if((f|0)==0){e=-22;i=b;return e|0}dg[f&255](2981344,2)|0;e=-22;i=b;return e|0}function o7(a){a=a|0;return(c[a+64>>2]|0)!=0|0}function o8(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0;b=c[556306]|0;if((a|0)==86079){d=1950507339}else if((a|0)==172){d=4665933}else if((a|0)==86077){d=1330664787}else if((a|0)==171){d=1160852272}else{d=a}if((b|0)==0){e=0;return e|0}else{f=b;g=0}L23649:while(1){if((c[f+80>>2]|0)==0){if((c[f+84>>2]|0)==0){h=g}else{i=17580}}else{i=17580}do{if((i|0)==17580){i=0;if((c[f+12>>2]|0)!=(d|0)){h=g;break}if((c[f+16>>2]&512|0)!=0&(g|0)==0){h=f}else{e=f;i=17586;break L23649}}}while(0);b=c[f+56>>2]|0;if((b|0)==0){e=h;i=17585;break}else{f=b;g=h}}if((i|0)==17585){return e|0}else if((i|0)==17586){return e|0}return 0}function o9(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0;b=c[556306]|0;if((a|0)==172){d=4665933}else if((a|0)==86079){d=1950507339}else if((a|0)==171){d=1160852272}else if((a|0)==86077){d=1330664787}else{d=a}if((b|0)==0){e=0;return e|0}else{f=b;g=0}L23669:while(1){do{if((c[f+88>>2]|0)==0){h=g}else{if((c[f+12>>2]|0)!=(d|0)){h=g;break}if((c[f+16>>2]&512|0)!=0&(g|0)==0){h=f}else{e=f;i=17599;break L23669}}}while(0);b=c[f+56>>2]|0;if((b|0)==0){e=h;i=17598;break}else{f=b;g=h}}if((i|0)==17599){return e|0}else if((i|0)==17598){return e|0}return 0}function pa(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;e=i;f=b+28|0;g=c[f>>2]|0;if((g|0)<0){AN(a,16,1674840,(h=i,i=i+8|0,c[h>>2]=g,h)|0);i=h;j=-22;i=e;return j|0}if(d>>>0>2147483631){AN(a,16,1673056,(h=i,i=i+16|0,c[h>>2]=d,c[h+8>>2]=2147483631,h)|0);i=h;j=-22;i=e;return j|0}k=b+24|0;l=c[k>>2]|0;L23687:do{if((a|0)==0){m=g;n=l}else{o=c[a+64>>2]|0;do{if((l|0)!=0){if((l|0)==(c[o+484>>2]|0)){AN(0,0,1655944,(h=i,i=i+24|0,c[h>>2]=1671032,c[h+8>>2]=1694056,c[h+16>>2]=1409,h)|0);i=h;cw();return 0}else{if((g|0)<(d|0)){break}else{m=g;n=l;break L23687}}}}while(0);p=a+64|0;q=o+484|0;r=o+488|0;do{if(d>>>0>4294967279){A6(q);c[r>>2]=0}else{s=d+16|0;if((c[r>>2]|0)>>>0>s>>>0){NM((c[q>>2]|0)+d|0,0,16);break}else{t=((s*17|0)>>>4)+32|0;u=t>>>0>s>>>0?t:s;A4(c[q>>2]|0);s=A7(u)|0;c[q>>2]=s;c[r>>2]=(s|0)==0?0:u;break}}}while(0);r=c[(c[p>>2]|0)+484>>2]|0;c[k>>2]=r;q=c[(c[p>>2]|0)+488>>2]|0;c[f>>2]=q;c[b+52>>2]=0;m=q;n=r}}while(0);if((n|0)==0){n=nC(b,d)|0;if((n|0)>=0){j=n;i=e;return j|0}AN(a,16,1667752,(h=i,i=i+8|0,c[h>>2]=d,h)|0);i=h;j=n;i=e;return j|0}n=b|0;k=b+52|0;if((m|0)<(d|0)){AN(a,16,1669288,(h=i,i=i+16|0,c[h>>2]=m,c[h+8>>2]=d,h)|0);i=h;j=-22;i=e;return j|0}else{h=c[k>>2]|0;m=c[n>>2]|0;nB(b);c[k>>2]=h;c[n>>2]=m;c[f>>2]=d;j=0;i=e;return j|0}return 0}function pb(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0;f=i;i=i+480|0;g=f|0;h=f+8|0;j=f+16|0;k=f+472|0;c[k>>2]=0;l=b|0;m=c[l>>2]|0;n=b+24|0;o=c[n>>2]|0;p=b+28|0;q=c[p>>2]|0;r=b+52|0;s=c[r>>2]|0;t=(o|0)!=0;u=t&1^1;c[e>>2]=0;v=a+12|0;w=c[v>>2]|0;x=c[w+16>>2]|0;y=(d|0)==0;if((x&32|0)==0&y){nJ(b);nB(b);z=0;i=f;return z|0}do{if(y){A=0;B=w;C=17654}else{if((c[d+64>>2]|0)==0){do{if((B8(c[a+420>>2]|0)|0)!=0){if((c[a+416>>2]|0)<=8){break}AN(a,16,1666448,(D=i,i=i+8|0,c[D>>2]=8,D)|0);i=D;z=-22;i=f;return z|0}}while(0);AN(a,24,1665104,(D=i,i=i+1|0,i=i+7&-8,c[D>>2]=0,D)|0);i=D;E=j;F=d;NN(E|0,F|0,456)|0;c[j+64>>2]=j;F=c[v>>2]|0;G=j;H=F;I=c[F+16>>2]|0}else{G=d;H=w;I=x}if((I&64|0)!=0){if((c[G+76>>2]|0)<=(c[a+424>>2]|0)){A=G;B=H;C=17654;break}AN(a,16,1663640,(D=i,i=i+1|0,i=i+7&-8,c[D>>2]=0,D)|0);i=D;z=-22;i=f;return z|0}if((I&65536|0)!=0){A=G;B=H;C=17654;break}F=G+76|0;E=c[F>>2]|0;J=a+424|0;K=c[J>>2]|0;L23736:do{if((E|0)<(K|0)){L=a+64|0;if((c[(c[L>>2]|0)+16>>2]|0)!=0){M=G;N=E;O=K;break}c[g>>2]=0;P=A1(456)|0;Q=P;if((P|0)==0){c[g>>2]=0;z=-12;i=f;return z|0}R=P+144|0;NM(P|0,0,456);c[R>>2]=0;c[R+4>>2]=-2147483648;R=P+152|0;c[R>>2]=0;c[R+4>>2]=-2147483648;R=P+136|0;c[R>>2]=0;c[R+4>>2]=-2147483648;Ah(Q,0,-2147483648);Aj(Q,0,0);Al(Q,-1,-1);Aq(Q,-1);c[P+84>>2]=1;c[P+124>>2]=0;c[P+128>>2]=1;c[P+64>>2]=P;c[g>>2]=Q;c[P+80>>2]=c[G+80>>2];R=G+352|0;S=c[R+4>>2]|0;T=P+352|0;c[T>>2]=c[R>>2];c[T+4>>2]=S;Ao(Q,An(G)|0);c[P+76>>2]=c[J>>2];P=Av(Q,32)|0;do{if((P|0)<0){U=P}else{S=Ax(Q,G)|0;if((S|0)<0){U=S;break}S=a+416|0;T=a+420|0;R=Cb(c[(c[g>>2]|0)+64>>2]|0,c[G+64>>2]|0,0,0,c[F>>2]|0,c[S>>2]|0,c[T>>2]|0)|0;if((R|0)<0){U=R;break}R=c[g>>2]|0;V=c[F>>2]|0;W=Ca(c[R+64>>2]|0,V,(c[R+76>>2]|0)-V|0,c[S>>2]|0,c[T>>2]|0)|0;if((W|0)<0){U=W;break}W=c[g>>2]|0;c[k>>2]=W;c[(c[L>>2]|0)+16>>2]=1;M=W;N=c[W+76>>2]|0;O=c[J>>2]|0;break L23736}}while(0);At(g);z=U;i=f;return z|0}else{M=G;N=E;O=K}}while(0);if((N|0)==(O|0)){A=M;B=c[v>>2]|0;C=17654;break}else{AN(a,16,1662136,(D=i,i=i+16|0,c[D>>2]=N,c[D+8>>2]=O,D)|0);i=D;X=-22;break}}}while(0);L23754:do{if((C|0)==17654){O=dk[c[B+84>>2]&63](a,b,A,e)|0;do{if((O|0)==0){if((c[e>>2]|0)==0){c[p>>2]=0;break}N=b+8|0;do{if((c[(c[v>>2]|0)+16>>2]&32|0)==0){if((c[N>>2]|0)==0&(c[N+4>>2]|0)==(-2147483648|0)){M=A+136|0;G=c[M+4>>2]|0;c[N>>2]=c[M>>2];c[N+4>>2]=G}G=b+48|0;if((c[G>>2]|0)!=0){break}M=c[A+76>>2]|0;c[h>>2]=1;c[h+4>>2]=c[a+412>>2];U=AY(M,(M|0)<0|0?-1:0,h,a+104|0)|0;c[G>>2]=U}}while(0);U=c[N+4>>2]|0;G=b+16|0;c[G>>2]=c[N>>2];c[G+4>>2]=U}}while(0);U=c[n>>2]|0;do{if((U|0)==0){Y=u;Z=O}else{if((U|0)!=(c[(c[a+64>>2]|0)+484>>2]|0)){Y=u;Z=O;break}if(!t){Y=0;Z=(nH(b)|0)<0?-12:O;break}G=c[p>>2]|0;if((q|0)<(G|0)){AN(a,16,1660704,(D=i,i=i+8|0,c[D>>2]=G,D)|0);i=D;c[p>>2]=q;_=-1}else{NN(o|0,U|0,G)|0;_=O}c[l>>2]=m;c[n>>2]=o;c[r>>2]=s;Y=0;Z=_}}while(0);if((Z|0)==0){do{if((Y|0)==0){$=0}else{if((c[n>>2]|0)==0){$=0;break}O=zl(l,(c[p>>2]|0)+16|0)|0;if((O|0)<=-1){$=O;break}c[n>>2]=c[(c[l>>2]|0)+4>>2];$=O}}while(0);O=a+428|0;c[O>>2]=(c[O>>2]|0)+1;aa=$}else{aa=Z}do{if((aa|0)>=0){if((c[e>>2]|0)==0){break}O=b+36|0;c[O>>2]=c[O>>2]|1;X=aa;break L23754}}while(0);nJ(b);nB(b);X=aa}}while(0);At(k);z=X;i=f;return z|0}function pc(a){a=a|0;var b=0;L23795:do{switch(a|0){case 69649:{b=2;break};case 69647:case 69633:case 69632:case 69645:case 69638:{b=4;break};case 69648:{b=3;break};default:{if((a|0)==86071|(a|0)==86072|(a|0)==69644|(a|0)==69661|(a|0)==69655|(a|0)==1330333984|(a|0)==69636|(a|0)==69660|(a|0)==69646){b=4;break L23795}else if((a|0)==65558|(a|0)==65559){b=64;break L23795}else if((a|0)==65545|(a|0)==65544|(a|0)==542135120|(a|0)==65547|(a|0)==65546|(a|0)==65556|(a|0)==65557){b=32;break L23795}else if((a|0)==65537|(a|0)==1347637264|(a|0)==65536|(a|0)==65554|(a|0)==65539|(a|0)==65538){b=16;break L23795}else if((a|0)==65543|(a|0)==65542|(a|0)==65540|(a|0)==65563|(a|0)==65541|(a|0)==65553){b=8;break L23795}else if((a|0)==65552|(a|0)==65549|(a|0)==65548|(a|0)==407917392|(a|0)==65551|(a|0)==65550){b=24;break L23795}else{b=0;break L23795}}}}while(0);return b|0}function pd(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;g=i;h=d|0;j=c[h>>2]|0;k=d+24|0;l=c[k>>2]|0;m=d+28|0;n=c[m>>2]|0;o=d+52|0;p=c[o>>2]|0;q=(l|0)!=0;r=q&1^1;c[f>>2]=0;do{if((c[b+88>>2]&512|0)!=0){s=c[b+668>>2]|0;if((s|0)==0){break}a[s]=0}}while(0);s=b+12|0;if((c[(c[s>>2]|0)+16>>2]&32|0)==0&(e|0)==0){nJ(d);nB(d);c[m>>2]=0;t=0;i=g;return t|0}u=b;if((AF(c[b+120>>2]|0,c[b+124>>2]|0,0,u)|0)!=0){t=-22;i=g;return t|0}v=c[(c[s>>2]|0)+84>>2]|0;if((v|0)==0){AN(0,0,1655944,(w=i,i=i+24|0,c[w>>2]=1656248,c[w+8>>2]=1694056,c[w+16>>2]=1754,w)|0);i=w;cw();return 0}x=dk[v&63](b,d,e,f)|0;if((x|0)>=1){AN(0,0,1655944,(w=i,i=i+24|0,c[w>>2]=1654424,c[w+8>>2]=1694056,c[w+16>>2]=1757,w)|0);i=w;cw();return 0}v=c[k>>2]|0;do{if((v|0)==0){y=r;z=x}else{if((v|0)!=(c[(c[b+64>>2]|0)+484>>2]|0)){y=r;z=x;break}if(!q){y=0;z=(nH(d)|0)<0?-12:x;break}A=c[m>>2]|0;if((n|0)<(A|0)){AN(u,16,1660704,(w=i,i=i+8|0,c[w>>2]=A,w)|0);i=w;c[m>>2]=n;B=-1}else{NN(l|0,v|0,A)|0;B=x}c[h>>2]=j;c[k>>2]=l;c[o>>2]=p;y=0;z=B}}while(0);if((z|0)==0){do{if((c[f>>2]|0)==0){c[m>>2]=0}else{if((c[(c[s>>2]|0)+16>>2]&32|0)!=0){break}B=e+136|0;p=c[B>>2]|0;o=c[B+4>>2]|0;B=d+16|0;c[B>>2]=p;c[B+4>>2]=o;B=d+8|0;c[B>>2]=p;c[B+4>>2]=o}}while(0);do{if((y|0)==0){C=0}else{if((c[k>>2]|0)==0){C=0;break}e=zl(h,(c[m>>2]|0)+16|0)|0;if((e|0)<=-1){C=e;break}c[k>>2]=c[(c[h>>2]|0)+4>>2];C=e}}while(0);h=b+428|0;c[h>>2]=(c[h>>2]|0)+1;D=C}else{D=z}do{if((D|0)>=0){if((c[f>>2]|0)==0){break}nM(d)|0;t=D;i=g;return t|0}}while(0);nJ(d);t=D;i=g;return t|0}function pe(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0;f=i;if((c[e+4>>2]|0)==0){g=dk[c[(c[a+12>>2]|0)+80>>2]&63](a,b,d,e)|0;e=a+428|0;c[e>>2]=(c[e>>2]|0)+1;h=g;i=f;return h|0}else{AN(a,16,1652976,(a=i,i=i+1|0,i=i+7&-8,c[a>>2]=0,a)|0);i=a;h=-1;i=f;return h|0}return 0}function pf(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0;f=i;i=i+88|0;g=f|0;h=f+8|0;j=c[a+64>>2]|0;k=h;l=e;NN(k|0,l|0,80)|0;l=a+12|0;k=c[l>>2]|0;if((k|0)==0){m=-22;i=f;return m|0}if((c[k+8>>2]|0)!=0){AN(a,16,1651136,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;m=-22;i=f;return m|0}c[d>>2]=0;k=c[a+128>>2]|0;n=c[a+132>>2]|0;do{if((k|n|0)!=0){if((AF(k,n,0,a)|0)==0){break}else{m=-22}i=f;return m|0}}while(0);n=b+64|0;k=b|0;o=b;NM(o|0,0,456);p=b+144|0;c[p>>2]=0;c[p+4>>2]=-2147483648;q=b+152|0;c[q>>2]=0;c[q+4>>2]=-2147483648;r=b+136|0;c[r>>2]=0;c[r+4>>2]=-2147483648;Ah(b,0,-2147483648);Aj(b,0,0);Al(b,-1,-1);Aq(b,-1);c[b+84>>2]=1;c[b+124>>2]=0;c[b+128>>2]=1;r=b+80|0;c[r>>2]=-1;c[n>>2]=k;s=a+488|0;if((c[s>>2]|0)==0){Au(j+24|0)}do{if((c[(c[l>>2]|0)+16>>2]&32|0)==0){if((c[e+28>>2]|0)!=0){t=17753;break}if((c[a+816>>2]&1|0)==0){u=0}else{t=17753}}else{t=17753}}while(0);do{if((t|0)==17753){v=nN(h)|0;pg(a,h);w=a+872|0;c[w>>2]=h;x=dk[c[(c[l>>2]|0)+88>>2]&63](a,o,d,h)|0;y=e+16|0;z=c[y+4>>2]|0;c[q>>2]=c[y>>2];c[q+4>>2]=z;if((c[a+176>>2]|0)==0){z=e+64|0;Al(b,c[z>>2]|0,c[z+4>>2]|0)}do{if((c[(c[l>>2]|0)+16>>2]&2|0)==0){z=b+124|0;if((c[z>>2]|0)==0){y=a+224|0;A=z;z=c[y+4>>2]|0;c[A>>2]=c[y>>2];c[A+4>>2]=z}z=b+68|0;if((c[z>>2]|0)==0){c[z>>2]=c[a+120>>2]}z=b+72|0;if((c[z>>2]|0)==0){c[z>>2]=c[a+124>>2]}if((c[r>>2]|0)!=-1){break}c[r>>2]=c[a+140>>2]}}while(0);z=nL(c[w>>2]|0,72,g)|0;L23897:do{if((z|0)!=0){A=c[g>>2]|0;y=z+A|0;if((A|0)>0){B=z}else{break}do{A=NL(B|0)|0;C=B+(A+1)|0;if((zK(Ar(b)|0,B,C,0)|0)<0){break L23897}B=B+(A+2+(NL(C|0)|0))|0;}while(B>>>0<y>>>0)}}while(0);c[w>>2]=0;do{if((v|0)==0){D=x}else{nA(h);if((x|0)!=(c[h+28>>2]|0)){D=x;break}D=c[e+28>>2]|0}}while(0);do{if((D|0)<0){if((c[k>>2]|0)==0){break}Au(b)}}while(0);if((c[d>>2]|0)==0){u=D;break}if((c[s>>2]|0)==0){x=j+24|0;v=x;NN(v|0,o|0,456)|0;c[j+88>>2]=x;NM(b+360|0,0,32)}x=a+428|0;c[x>>2]=(c[x>>2]|0)+1;x=c[p>>2]|0;v=c[p+4>>2]|0;w=c[q>>2]|0;z=c[q+4>>2]|0;y=(w|0)==0&(z|0)==(-2147483648|0);if(!y){C=a+928|0;A=c[C+4>>2]|0;E=a+912|0;F=NU((A|0)>=(z|0)&((A|0)>(z|0)|(c[C>>2]|0)>>>0>=w>>>0)&1,0,c[E>>2]|0,c[E+4>>2]|0)|0;c[E>>2]=F;c[E+4>>2]=G;c[C>>2]=w;c[C+4>>2]=z}C=(x|0)!=0|(v|0)!=(-2147483648|0);if(C){E=a+920|0;F=c[E+4>>2]|0;A=a+904|0;H=NU((F|0)>=(v|0)&((F|0)>(v|0)|(c[E>>2]|0)>>>0>=x>>>0)&1,0,c[A>>2]|0,c[A+4>>2]|0)|0;F=G;c[A>>2]=H;c[A+4>>2]=F;c[E>>2]=x;c[E+4>>2]=v;I=F;J=H}else{H=a+904|0;I=c[H+4>>2]|0;J=c[H>>2]|0}H=a+912|0;F=c[H+4>>2]|0;E=((I|0)<=(F|0)&((I|0)<(F|0)|J>>>0<=(c[H>>2]|0)>>>0)|y)&C;Ah(b,E?x:w,E?v:z);u=D}}while(0);c[n>>2]=k;m=u;i=f;return m|0}function pg(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;e=i;i=i+8|0;f=e|0;c[f>>2]=0;if((c[(c[a+12>>2]|0)+16>>2]&16384|0)==0){i=e;return}g=nL(b,2,f)|0;if((g|0)==0){i=e;return}b=c[f>>2]|0;if((b|0)<4){i=e;return}h=g;j=d[h]|d[h+1|0]<<8|d[h+2|0]<<16|d[h+3|0]<<24|0;h=b-4|0;c[f>>2]=h;if((h|0)<4){i=e;return}k=g+4|0;if((j&1|0)==0){l=k;m=h}else{h=k;c[a+416>>2]=d[h]|d[h+1|0]<<8|d[h+2|0]<<16|d[h+3|0]<<24;h=b-8|0;c[f>>2]=h;l=g+8|0;m=h}do{if((j&2|0)==0){n=l;o=m}else{if((m|0)<8){i=e;return}else{h=l;g=h|0;b=h+4|0;h=d[b]|d[b+1|0]<<8|d[b+2|0]<<16|d[b+3|0]<<24|0;b=a+448|0;c[b>>2]=d[g]|d[g+1|0]<<8|d[g+2|0]<<16|d[g+3|0]<<24;c[b+4>>2]=h;h=m-8|0;c[f>>2]=h;n=l+8|0;o=h;break}}}while(0);if((o|0)<4){i=e;return}if((j&4|0)==0){p=n;q=o}else{l=n;c[a+412>>2]=d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24;l=o-4|0;c[f>>2]=l;p=n+4|0;q=l}if((j&8|0)==0|(q|0)<8){i=e;return}j=p;l=d[j]|d[j+1|0]<<8|d[j+2|0]<<16|d[j+3|0]<<24|0;j=a+120|0;c[j>>2]=l;n=p+4|0;p=d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0;c[a+128>>2]=l;c[a+132>>2]=p;n=c[a+800>>2]|0;c[j>>2]=-(-l>>n);c[a+124>>2]=-(-p>>n);c[f>>2]=q-8;i=e;return}function ph(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;g=i;i=i+104|0;h=g|0;j=g+8|0;k=g+16|0;l=g+96|0;m=a+64|0;n=c[m>>2]|0;c[e>>2]=0;do{if((c[f+24>>2]|0)==0){if((c[f+28>>2]|0)==0){break}AN(a,16,1643632,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;p=-22;i=g;return p|0}}while(0);q=a+12|0;r=c[q>>2]|0;if((r|0)==0){p=-22;i=g;return p|0}if((c[r+8>>2]|0)!=1){AN(a,16,1641080,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;p=-22;i=g;return p|0}r=b+64|0;s=b|0;t=b;NM(t|0,0,456);u=b+144|0;c[u>>2]=0;c[u+4>>2]=-2147483648;v=b+152|0;c[v>>2]=0;c[v+4>>2]=-2147483648;w=b+136|0;c[w>>2]=0;c[w+4>>2]=-2147483648;Ah(b,0,-2147483648);Aj(b,0,0);Al(b,-1,-1);Aq(b,-1);c[b+84>>2]=1;c[b+124>>2]=0;c[b+128>>2]=1;w=b+80|0;c[w>>2]=-1;c[r>>2]=s;x=a+488|0;if((c[x>>2]|0)==0){Au(n+24|0)}do{if((c[(c[q>>2]|0)+16>>2]&32|0)==0){if((c[f+28>>2]|0)!=0){y=17823;break}if((c[a+816>>2]&1|0)==0){z=0}else{y=17823}}else{y=17823}}while(0);L23975:do{if((y|0)==17823){A=k;B=f;NN(A|0,B|0,80)|0;B=nN(k)|0;pg(a,k);A=a+872|0;c[A>>2]=k;C=dk[c[(c[q>>2]|0)+88>>2]&63](a,t,e,k)|0;D=f+16|0;E=c[D+4>>2]|0;c[v>>2]=c[D>>2];c[v+4>>2]=E;do{if((C|0)>-1){if((c[e>>2]|0)==0){break}E=nL(c[A>>2]|0,72,h)|0;L23980:do{if((E|0)!=0){D=c[h>>2]|0;F=E+D|0;if((D|0)>0){H=E}else{break}do{D=NL(H|0)|0;I=H+(D+1)|0;if((zK(Ar(b)|0,H,I,0)|0)<0){break L23980}H=H+(D+2+(NL(I|0)|0))|0;}while(H>>>0<F>>>0)}}while(0);E=a+428|0;c[E>>2]=(c[E>>2]|0)+1;E=c[u>>2]|0;F=c[u+4>>2]|0;I=c[v>>2]|0;D=c[v+4>>2]|0;J=(I|0)==0&(D|0)==(-2147483648|0);if(!J){K=a+928|0;L=c[K+4>>2]|0;M=a+912|0;N=NU((L|0)>=(D|0)&((L|0)>(D|0)|(c[K>>2]|0)>>>0>=I>>>0)&1,0,c[M>>2]|0,c[M+4>>2]|0)|0;c[M>>2]=N;c[M+4>>2]=G;c[K>>2]=I;c[K+4>>2]=D}K=(E|0)!=0|(F|0)!=(-2147483648|0);if(K){M=a+920|0;N=c[M+4>>2]|0;L=a+904|0;O=NU((N|0)>=(F|0)&((N|0)>(F|0)|(c[M>>2]|0)>>>0>=E>>>0)&1,0,c[L>>2]|0,c[L+4>>2]|0)|0;N=G;c[L>>2]=O;c[L+4>>2]=N;c[M>>2]=E;c[M+4>>2]=F;P=N;Q=O}else{O=a+904|0;P=c[O+4>>2]|0;Q=c[O>>2]|0}O=a+912|0;N=c[O+4>>2]|0;M=((P|0)<=(N|0)&((P|0)<(N|0)|Q>>>0<=(c[O>>2]|0)>>>0)|J)&K;Ah(b,M?E:I,M?F:D);if((c[w>>2]|0)==-1){c[w>>2]=c[a+420>>2]}D=b+352|0;if((c[D>>2]|0)==0&(c[D+4>>2]|0)==0){F=a+448|0;M=c[F+4>>2]|0;c[D>>2]=c[F>>2];c[D+4>>2]=M}if((An(b)|0)==0){Ao(b,c[a+416>>2]|0)}M=b+344|0;if((c[M>>2]|0)!=0){break}c[M>>2]=c[a+412>>2]}}while(0);M=nL(c[A>>2]|0,70,j)|0;if((M|0)!=0&(c[j>>2]|0)>9){D=M;c[(c[m>>2]|0)+496>>2]=d[D]|d[D+1|0]<<8|d[D+2|0]<<16|d[D+3|0]<<24;AN(a,48,1639576,(o=i,i=i+8|0,c[o>>2]=c[(c[m>>2]|0)+496>>2],o)|0);i=o}D=c[(c[m>>2]|0)+496>>2]|0;do{if((D|0)!=0){if((c[e>>2]|0)==0){break}M=b+76|0;F=c[M>>2]|0;if((F|0)<=(D|0)){c[e>>2]=0;I=(c[m>>2]|0)+496|0;c[I>>2]=(c[I>>2]|0)-(c[M>>2]|0);AN(a,48,1637632,(o=i,i=i+8|0,c[o>>2]=c[(c[m>>2]|0)+496>>2],o)|0);i=o;break}I=c[r>>2]|0;E=F-D|0;F=c[a+416>>2]|0;K=c[w>>2]|0;Cb(I,I,0,D,E,F,K)|0;K=a+888|0;do{if((c[K>>2]|0)==0){y=17856}else{F=c[a+412>>2]|0;if((F|0)==0){y=17856;break}E=c[(c[m>>2]|0)+496>>2]|0;c[l>>2]=1;c[l+4>>2]=F;F=AY(E,(E|0)<0|0?-1:0,l,K)|0;E=G;I=c[u>>2]|0;J=c[u+4>>2]|0;if(!((I|0)==0&(J|0)==(-2147483648|0))){O=NU(I,J,F,E)|0;c[u>>2]=O;c[u+4>>2]=G}O=c[v>>2]|0;J=c[v+4>>2]|0;if(!((O|0)==0&(J|0)==(-2147483648|0))){I=NU(O,J,F,E)|0;c[v>>2]=I;c[v+4>>2]=G}I=Ai(b)|0;J=G;if((J|0)<(E|0)|(J|0)==(E|0)&I>>>0<F>>>0){break}I=Ai(b)|0;J=NV(I,G,F,E)|0;Aj(b,J,G)}}while(0);if((y|0)==17856){AN(a,24,1635872,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o}K=c[M>>2]|0;AN(a,48,1635272,(o=i,i=i+16|0,c[o>>2]=c[(c[m>>2]|0)+496>>2],c[o+8>>2]=K,o)|0);i=o;c[M>>2]=(c[M>>2]|0)-(c[(c[m>>2]|0)+496>>2]|0);c[(c[m>>2]|0)+496>>2]=0}}while(0);c[A>>2]=0;do{if((B|0)==0){R=C}else{nA(k);if((C|0)!=(c[k+28>>2]|0)){R=C;break}R=c[f+28>>2]|0}}while(0);do{if((R|0)>-1){if((c[e>>2]|0)==0){break}if((c[x>>2]|0)!=0){z=R;break L23975}C=n+24|0;B=C;NN(B|0,t|0,456)|0;c[n+88>>2]=C;NM(b+360|0,0,40);z=R;break L23975}}while(0);if((c[s>>2]|0)==0){z=R;break}Au(b);z=R}}while(0);if((c[e>>2]|0)==0){c[r>>2]=0;p=z;i=g;return p|0}e=B8(c[w>>2]|0)|0;if((e|0)!=0&(An(b)|0)>8){p=z;i=g;return p|0}c[r>>2]=s;p=z;i=g;return p|0}function pi(e,f,g,h){e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0;j=i;i=i+280|0;k=j|0;l=j+8|0;m=j+16|0;n=j+24|0;o=j+32|0;p=j+112|0;q=j+192|0;r=j+272|0;s=e+12|0;if((c[(c[s>>2]|0)+8>>2]|0)!=3){AN(e,16,1634608,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;u=-22;i=j;return u|0}c[g>>2]=0;v=f;NM(v|0,0,24);w=f+24|0;c[w>>2]=0;c[w+4>>2]=-2147483648;x=h+28|0;if((c[x>>2]|0)==0){u=0;i=j;return u|0}y=q;z=h;NN(y|0,z|0,80)|0;z=nN(q)|0;A=p;NN(A|0,y|0,80)|0;if((c[e+940>>2]|0)==1){y=e+936|0;A=Nv(1678608,c[y>>2]|0)|0;if((A|0)==-1){AN(0,0,1655944,(t=i,i=i+24|0,c[t>>2]=1616864,c[t+8>>2]=1694056,c[t+16>>2]=2194,t)|0);i=t;cw();return 0}B=q+24|0;c[k>>2]=c[B>>2];C=c[q+28>>2]|0;c[m>>2]=C;L24059:do{if(C>>>0>536870894){AN(e,16,1616544,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;D=-12}else{E=nC(o,C<<2)|0;if((E|0)<0){D=E;break}c[p>>2]=c[o>>2];F=c[o+24>>2]|0;H=p+24|0;c[H>>2]=F;I=c[o+28>>2]|0;J=p+28|0;c[J>>2]=I;c[l>>2]=F;c[n>>2]=I;do{if((Ny(A,k,m,l,n)|0)!=-1){if((Ny(A,0,0,l,n)|0)==-1){break}I=c[n>>2]|0;F=c[J>>2]|0;if(I>>>0>=F>>>0){break}if((c[m>>2]|0)!=0){break}K=F-I|0;c[J>>2]=K;NM((c[H>>2]|0)+K|0,0,I|0);D=E;break L24059}}while(0);E=c[y>>2]|0;AN(e,16,1616168,(t=i,i=i+16|0,c[t>>2]=c[B>>2],c[t+8>>2]=E,t)|0);i=t;nJ(o);D=-(c[(ch()|0)>>2]|0)|0}}while(0);Nx(A)|0;L=D}else{L=0}if((L|0)<0){c[g>>2]=0;M=L}else{L=e+872|0;c[L>>2]=p;D=e+888|0;do{if((c[e+892>>2]|0)!=0){A=h+8|0;o=c[A>>2]|0;B=c[A+4>>2]|0;if((o|0)==0&(B|0)==(-2147483648|0)){break}c[r>>2]=1;c[r+4>>2]=1e6;A=AY(o,B,D,r)|0;c[w>>2]=A;c[w+4>>2]=G}}while(0);w=dk[c[(c[s>>2]|0)+88>>2]&63](e,v,g,p)|0;v=f+12|0;s=c[v>>2]|0;L24078:do{if((s|0)!=0){r=f+8|0;if((c[r>>2]|0)==0){A=c[h+48>>2]|0;do{if((A|0)==0){N=s}else{if((c[D>>2]|0)==0){N=s;break}B=AY(A,(A|0)<0|0?-1:0,D,340912)|0;c[r>>2]=B;N=c[v>>2]|0}}while(0);if((N|0)==0){break}else{O=N}}else{O=s}r=c[f+16>>2]|0;A=0;L24087:while(1){B=c[(c[r+(A<<2)>>2]|0)+92>>2]|0;L24089:do{if((B|0)!=0){o=B;while(1){y=a[o]|0;if(y<<24>>24==0){break L24089}m=y&255;n=m>>>1&64;if((m&192|0)==128|(y&255)>253){break L24087}y=o+1|0;if((n&m|0)==0){P=m;Q=n;R=y}else{l=m;m=n;n=y;while(1){y=(d[n]|0)-128|0;if(y>>>0>63){break L24087}k=y+(l<<6)|0;y=m<<5;C=n+1|0;if((k&y|0)==0){P=k;Q=y;R=C;break}else{l=k;m=y;n=C}}}n=(Q<<1)-1&P;m=R-o|0;do{if((m|0)==1){S=0}else{if((m|0)==2){S=128;break}S=1<<(m*5|0)-4}}while(0);if(n>>>0<S>>>0|n>>>0>1114111|(n|0)==65534|(n-55296|0)>>>0<2048){break L24087}else{o=R}}}}while(0);A=A+1|0;if(A>>>0>=O>>>0){break L24078}}AN(e,16,1633832,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;pj(f);u=-1094995529;i=j;return u|0}}while(0);if((c[q+24>>2]|0)!=(c[p+24>>2]|0)){c[p+40>>2]=0;c[p+44>>2]=0;nJ(p)}b[f>>1]=((c[(c[e+896>>2]|0)+16>>2]|0)>>>16&1^1)&65535;c[L>>2]=0;M=w}do{if((z|0)==0){T=M}else{nA(q);if((M|0)!=(c[q+28>>2]|0)){T=M;break}T=c[x>>2]|0}}while(0);if((c[g>>2]|0)==0){u=T;i=j;return u|0}g=e+428|0;c[g>>2]=(c[g>>2]|0)+1;u=T;i=j;return u|0}function pj(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=a+12|0;d=a+16|0;if((c[b>>2]|0)==0){e=d;A6(e);f=a;NM(f|0,0,32);return}else{g=0}do{A6((c[(c[d>>2]|0)+(g<<2)>>2]|0)+20|0);A6((c[(c[d>>2]|0)+(g<<2)>>2]|0)+24|0);A6((c[(c[d>>2]|0)+(g<<2)>>2]|0)+28|0);A6((c[(c[d>>2]|0)+(g<<2)>>2]|0)+32|0);A6((c[(c[d>>2]|0)+(g<<2)>>2]|0)+88|0);A6((c[(c[d>>2]|0)+(g<<2)>>2]|0)+92|0);A6((c[d>>2]|0)+(g<<2)|0);g=g+1|0;}while(g>>>0<(c[b>>2]|0)>>>0);e=d;A6(e);f=a;NM(f|0,0,32);return}function pk(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0;b=i;d=o6(a)|0;if((d|0)<0){e=d;i=b;return e|0}d=a+64|0;f=c[d>>2]|0;if((f|0)!=0){g=c[f+480>>2]|0;h=c[a+12>>2]|0;do{if((h|0)==0){j=f}else{k=c[h+92>>2]|0;if((k|0)==0){j=f;break}c1[k&255](a)|0;j=c[d>>2]|0}}while(0);c[a+804>>2]=0;c[j+488>>2]=0;A6((c[d>>2]|0)+484|0);if((c[a+488>>2]|0)==0){Au((c[d>>2]|0)+24|0)}zn(g|0);zn(g+4|0);zn(g+8|0);zn(g+12|0);A6((c[d>>2]|0)+480|0);A6(d)}d=a+60|0;g=c[d>>2]|0;j=a+12|0;do{if((g|0)!=0){f=c[j>>2]|0;if((f|0)==0){break}if((c[f+44>>2]|0)==0){break}Bu(g)}}while(0);Bu(a);A6(d);d=c[j>>2]|0;do{if((d|0)!=0){if((c[d+80>>2]|0)==0){if((c[d+84>>2]|0)==0){break}}A6(a+96|0)}}while(0);c[j>>2]=0;c[a+816>>2]=0;if((c[716810]|0)==0){AN(0,0,1655944,(a=i,i=i+24|0,c[a>>2]=1617256,c[a+8>>2]=1694056,c[a+16>>2]=3059,a)|0);i=a;cw();return 0}c[716810]=0;c[744094]=(c[744094]|0)-1;a=c[551446]|0;if((a|0)==0){e=0;i=b;return e|0}dg[a&255](2981344,2)|0;e=0;i=b;return e|0}function pl(a){a=a|0;var b=0,d=0,e=0,f=0;if((a|0)==0){b=0;return b|0}d=c[556306]|0;if((d|0)==0){b=0;return b|0}else{e=d}while(1){if((c[e+80>>2]|0)==0){if((c[e+84>>2]|0)!=0){f=17965}}else{f=17965}if((f|0)==17965){f=0;if((a8(a|0,c[e>>2]|0)|0)==0){b=e;f=17969;break}}d=c[e+56>>2]|0;if((d|0)==0){b=0;f=17970;break}else{e=d}}if((f|0)==17969){return b|0}else if((f|0)==17970){return b|0}return 0}function pm(a){a=a|0;var b=0,d=0,e=0,f=0;if((a|0)==0){b=0;return b|0}d=c[556306]|0;if((d|0)==0){b=0;return b|0}else{e=d}while(1){if((c[e+88>>2]|0)!=0){if((a8(a|0,c[e>>2]|0)|0)==0){b=e;f=17978;break}}d=c[e+56>>2]|0;if((d|0)==0){b=0;f=17981;break}else{e=d}}if((f|0)==17981){return b|0}else if((f|0)==17978){return b|0}return 0}function pn(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;b=i;if((a|0)==0){d=1633368;i=b;return d|0}e=nT(a)|0;if((e|0)!=0){d=c[e+8>>2]|0;i=b;return d|0}AN(0,24,1632408,(e=i,i=i+8|0,c[e>>2]=a,e)|0);i=e;e=c[556306]|0;if((a|0)==172){f=4665933}else if((a|0)==86077){f=1330664787}else if((a|0)==86079){f=1950507339}else if((a|0)==171){f=1160852272}else{f=a}g=(e|0)==0;do{if(!g){h=e;j=0;L24202:while(1){do{if((c[h+88>>2]|0)==0){k=j}else{if((c[h+12>>2]|0)!=(f|0)){k=j;break}if((c[h+16>>2]&512|0)!=0&(j|0)==0){k=h}else{l=h;break L24202}}}while(0);m=c[h+56>>2]|0;if((m|0)==0){l=k;break}else{h=m;j=k}}if((l|0)==0){break}d=c[l>>2]|0;i=b;return d|0}}while(0);if((a|0)==86079){n=1950507339}else if((a|0)==86077){n=1330664787}else if((a|0)==172){n=4665933}else if((a|0)==171){n=1160852272}else{n=a}if(g){d=1631536;i=b;return d|0}else{o=e;p=0}L24220:while(1){if((c[o+80>>2]|0)==0){if((c[o+84>>2]|0)==0){q=p}else{r=18005}}else{r=18005}do{if((r|0)==18005){r=0;if((c[o+12>>2]|0)!=(n|0)){q=p;break}if((c[o+16>>2]&512|0)!=0&(p|0)==0){q=o}else{s=o;break L24220}}}while(0);e=c[o+56>>2]|0;if((e|0)==0){s=q;break}else{o=e;p=q}}if((s|0)==0){d=1631536;i=b;return d|0}d=c[s>>2]|0;i=b;return d|0}function po(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0;e=i;f=d&255;do{if((f-48|0)>>>0<10|(f-97|0)>>>0<26|(f-65|0)>>>0<26){g=1630952}else{if((f|0)==46|(f|0)==45|(f|0)==32){g=1630952;break}g=(f|0)==95?1630952:1630400}}while(0);h=bc(a|0,b|0,g|0,(g=i,i=i+8|0,c[g>>2]=f,g)|0)|0;i=g;f=a+h|0;j=h>>>0<b>>>0?b-h|0:0;b=d>>>8&255;do{if((b-48|0)>>>0<10|(b-97|0)>>>0<26|(b-65|0)>>>0<26){k=1630952}else{if((b|0)==46|(b|0)==45|(b|0)==32){k=1630952;break}k=(b|0)==95?1630952:1630400}}while(0);l=bc(f|0,j|0,k|0,(g=i,i=i+8|0,c[g>>2]=b,g)|0)|0;i=g;b=h+l|0;k=a+b|0;f=j>>>0>l>>>0?j-l|0:0;j=l+h|0;h=d>>>16&255;do{if((h-48|0)>>>0<10|(h-97|0)>>>0<26|(h-65|0)>>>0<26){m=1630952}else{if((h|0)==46|(h|0)==45|(h|0)==32){m=1630952;break}m=(h|0)==95?1630952:1630400}}while(0);l=bc(k|0,f|0,m|0,(g=i,i=i+8|0,c[g>>2]=h,g)|0)|0;i=g;h=a+(b+l)|0;b=f>>>0>l>>>0?f-l|0:0;f=l+j|0;j=d>>>24;do{if((j-48|0)>>>0<10|(j-97|0)>>>0<26|(j-65|0)>>>0<26){n=1630952}else{if((j|0)==46|(j|0)==45|(j|0)==32){n=1630952;break}n=(j|0)==95?1630952:1630400}}while(0);d=bc(h|0,b|0,n|0,(g=i,i=i+8|0,c[g>>2]=j,g)|0)|0;i=g;i=e;return d+f|0}function pp(b,d,f,g){b=b|0;d=d|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0;h=i;i=i+40|0;j=h|0;k=h+8|0;if((b|0)==0|(d|0)<1){i=h;return}l=f+8|0;m=Cp(c[l>>2]|0)|0;n=f+48|0;o=pn(c[n>>2]|0)|0;p=c[f+840>>2]|0;q=f+12|0;L4:do{if((p|0)==-99){r=0}else{s=c[q>>2]|0;if((s|0)==0){t=c[n>>2]|0;u=c[556306]|0;L8:do{if((g|0)==0){if((t|0)==86079){v=1950507339}else if((t|0)==171){v=1160852272}else if((t|0)==172){v=4665933}else if((t|0)==86077){v=1330664787}else{v=t}if((u|0)==0){r=0;break L4}else{w=u;x=0}while(1){do{if((c[w+88>>2]|0)==0){y=x}else{if((c[w+12>>2]|0)!=(v|0)){y=x;break}if((c[w+16>>2]&512|0)!=0&(x|0)==0){y=w}else{z=w;break L8}}}while(0);A=c[w+56>>2]|0;if((A|0)==0){z=y;break}else{w=A;x=y}}}else{if((t|0)==86079){B=1950507339}else if((t|0)==171){B=1160852272}else if((t|0)==172){B=4665933}else if((t|0)==86077){B=1330664787}else{B=t}if((u|0)==0){r=0;break L4}else{C=u;D=0}while(1){if((c[C+80>>2]|0)==0){if((c[C+84>>2]|0)==0){E=D}else{F=13}}else{F=13}do{if((F|0)==13){F=0;if((c[C+12>>2]|0)!=(B|0)){E=D;break}if((c[C+16>>2]&512|0)!=0&(D|0)==0){E=C}else{z=C;break L8}}}while(0);A=c[C+56>>2]|0;if((A|0)==0){z=E;break}else{C=A;D=E}}}}while(0);if((z|0)==0){r=0;break}else{G=z}}else{G=s}u=c[G+48>>2]|0;if((u|0)==0){r=0;break}else{H=u}while(1){u=c[H>>2]|0;if((u|0)==-99){r=0;break L4}if((u|0)==(p|0)){break}else{H=H+8|0}}r=c[H+4>>2]|0}}while(0);bc(b|0,d|0,1629512,(H=i,i=i+16|0,c[H>>2]=(m|0)!=0?m:1628848,c[H+8>>2]=o,H)|0)|0;i=H;a[b]=a[b]^32;m=c[q>>2]|0;do{if((m|0)!=0){q=c[m>>2]|0;if((a8(q|0,o|0)|0)==0){break}p=NL(b|0)|0;G=b+p|0;z=d-p|0;bc(G|0,z|0,1628360,(H=i,i=i+8|0,c[H>>2]=q,H)|0)|0;i=H}}while(0);if((r|0)!=0){o=NL(b|0)|0;m=b+o|0;q=d-o|0;bc(m|0,q|0,1628360,(H=i,i=i+8|0,c[H>>2]=r,H)|0)|0;i=H}r=f+52|0;q=c[r>>2]|0;if((q|0)!=0){m=k|0;po(m,32,q)|0;q=NL(b|0)|0;k=b+q|0;o=d-q|0;q=c[r>>2]|0;bc(k|0,o|0,1627912,(H=i,i=i+16|0,c[H>>2]=m,c[H+8>>2]=q,H)|0)|0;i=H}q=c[l>>2]|0;do{if((q|0)==0){m=f+140|0;o=c[m>>2]|0;do{if((o|0)!=-1){k=NL(b|0)|0;r=b+k|0;z=d-k|0;k=BL(o)|0;bc(r|0,z|0,1627368,(H=i,i=i+8|0,c[H>>2]=k,H)|0)|0;i=H;k=f+796|0;z=c[k>>2]|0;if((z|0)==0){break}if((z|0)>((e[(BP(c[m>>2]|0)|0)+8>>1]|0)>>>11&15|0)){break}z=NL(b|0)|0;r=b+z|0;G=d-z|0;z=c[k>>2]|0;bc(r|0,G|0,1626840,(H=i,i=i+8|0,c[H>>2]=z,H)|0)|0;i=H}}while(0);m=f+120|0;o=c[m>>2]|0;do{if((o|0)!=0){z=NL(b|0)|0;G=b+z|0;r=d-z|0;z=f+124|0;k=c[z>>2]|0;bc(G|0,r|0,1625952,(H=i,i=i+16|0,c[H>>2]=o,c[H+8>>2]=k,H)|0)|0;i=H;k=f+224|0;r=c[k>>2]|0;if((r|0)!=0){G=j|0;p=j+4|0;E=ac(c[m>>2]|0,r)|0;r=E;D=(E|0)<0|0?-1:0;E=f+228|0;C=ac(c[E>>2]|0,c[z>>2]|0)|0;z=C;B=(C|0)<0|0?-1:0;C=1048576;y=0;BX(G,p,r,D,z,B,C,y)|0;y=NL(b|0)|0;C=b+y|0;B=d-y|0;y=c[k>>2]|0;k=c[E>>2]|0;E=c[G>>2]|0;G=c[p>>2]|0;bc(C|0,B|0,1625128,(H=i,i=i+32|0,c[H>>2]=y,c[H+8>>2]=k,c[H+16>>2]=E,c[H+24>>2]=G,H)|0)|0;i=H}if((AO()|0)<=47){break}G=c[f+104>>2]|0;E=c[f+108>>2]|0;k=AU(G,(G|0)<0|0?-1:0,E,(E|0)<0|0?-1:0)|0;y=k;k=NL(b|0)|0;B=b+k|0;C=d-k|0;k=(G|0)/(y|0)|0;G=(E|0)/(y|0)|0;bc(B|0,C|0,1624496,(H=i,i=i+16|0,c[H>>2]=k,c[H+8>>2]=G,H)|0)|0;i=H}}while(0);if((g|0)==0){break}m=NL(b|0)|0;o=b+m|0;G=d-m|0;m=c[f+500>>2]|0;k=c[f+504>>2]|0;bc(o|0,G|0,1624088,(H=i,i=i+16|0,c[H>>2]=m,c[H+8>>2]=k,H)|0)|0;i=H;F=57}else if((q|0)==1){k=c[f+412>>2]|0;if((k|0)!=0){m=NL(b|0)|0;G=b+m|0;o=d-m|0;bc(G|0,o|0,1623288,(H=i,i=i+8|0,c[H>>2]=k,H)|0)|0;i=H}y$(b,1622768,d)|0;k=NL(b|0)|0;o=f+448|0;zt(b+k|0,d-k|0,c[f+416>>2]|0,c[o>>2]|0,c[o+4>>2]|0);o=c[f+420>>2]|0;if((o|0)==-1){F=57;break}k=NL(b|0)|0;G=B2(o)|0;bc(b+k|0,d-k|0,1627368,(H=i,i=i+8|0,c[H>>2]=G,H)|0)|0;i=H;F=57}else if((q|0)==2){if((AO()|0)<=47){F=57;break}G=c[f+104>>2]|0;k=c[f+108>>2]|0;o=AU(G,(G|0)<0|0?-1:0,k,(k|0)<0|0?-1:0)|0;m=o;if((m|0)==0){F=57;break}o=NL(b|0)|0;bc(b+o|0,d-o|0,1624496,(H=i,i=i+16|0,c[H>>2]=(G|0)/(m|0)|0,c[H+8>>2]=(k|0)/(m|0)|0,H)|0)|0;i=H;F=57}else{i=h;return}}while(0);do{if((F|0)==57){if((g|0)==0){break}q=f+88|0;j=c[q>>2]|0;if((j&512|0)==0){I=j}else{j=NL(b|0)|0;m=b+j|0;k=d-j|0;bc(m|0,k|0,1622272,(H=i,i=i+1|0,i=i+7&-8,c[H>>2]=0,H)|0)|0;i=H;I=c[q>>2]|0}if((I&1024|0)==0){break}q=NL(b|0)|0;k=b+q|0;m=d-q|0;bc(k|0,m|0,1621984,(H=i,i=i+1|0,i=i+7&-8,c[H>>2]=0,H)|0)|0;i=H}}while(0);L86:do{switch(c[l>>2]|0){case 0:case 2:case 3:case 4:{J=c[f+72>>2]|0;break};case 1:{I=c[n>>2]|0;L89:do{switch(I|0){case 69648:{K=3;break};case 69647:case 69633:case 69632:case 69645:case 69638:{K=4;break};case 69649:{K=2;break};default:{if((I|0)==65543|(I|0)==65542|(I|0)==65540|(I|0)==65563|(I|0)==65541|(I|0)==65553){K=8;break L89}else if((I|0)==65537|(I|0)==1347637264|(I|0)==65536|(I|0)==65554|(I|0)==65539|(I|0)==65538){K=16;break L89}else if((I|0)==65552|(I|0)==65549|(I|0)==65548|(I|0)==407917392|(I|0)==65551|(I|0)==65550){K=24;break L89}else if((I|0)==65545|(I|0)==65544|(I|0)==542135120|(I|0)==65547|(I|0)==65546|(I|0)==65556|(I|0)==65557){K=32;break L89}else if((I|0)==65558|(I|0)==65559){K=64;break L89}else if((I|0)==86071|(I|0)==86072|(I|0)==69644|(I|0)==69661|(I|0)==69655|(I|0)==1330333984|(I|0)==69636|(I|0)==69660|(I|0)==69646){K=4;break L89}else{J=c[f+72>>2]|0;break L86}}}}while(0);I=c[f+416>>2]|0;J=ac(ac(c[f+412>>2]|0,K)|0,I)|0;break};default:{i=h;return}}}while(0);if((J|0)==0){i=h;return}K=NL(b|0)|0;bc(b+K|0,d-K|0,1621560,(H=i,i=i+8|0,c[H>>2]=(J|0)/1e3|0,H)|0)|0;i=H;i=h;return}function pq(){return 3609190}function pr(){return 1620176}function ps(a){a=a|0;var b=0;if((a|0)==65543|(a|0)==65542|(a|0)==65540|(a|0)==65563|(a|0)==65541|(a|0)==65553){b=8}else if((a|0)==65537|(a|0)==1347637264|(a|0)==65536|(a|0)==65554|(a|0)==65539|(a|0)==65538){b=16}else if((a|0)==65552|(a|0)==65549|(a|0)==65548|(a|0)==407917392|(a|0)==65551|(a|0)==65550){b=24}else if((a|0)==65545|(a|0)==65544|(a|0)==542135120|(a|0)==65547|(a|0)==65546|(a|0)==65556|(a|0)==65557){b=32}else if((a|0)==65558|(a|0)==65559){b=64}else if((a|0)==86071|(a|0)==86072|(a|0)==69644|(a|0)==69661|(a|0)==69655|(a|0)==1330333984|(a|0)==69636|(a|0)==69660|(a|0)==69646){b=4}else{b=0}return b|0}function pt(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;d=c[a+48>>2]|0;e=c[a+412>>2]|0;f=c[a+416>>2]|0;g=c[a+432>>2]|0;h=c[a+52>>2]|0;if((d|0)==65543|(d|0)==65542|(d|0)==65540|(d|0)==65563|(d|0)==65541|(d|0)==65553){i=8}else if((d|0)==65537|(d|0)==1347637264|(d|0)==65536|(d|0)==65554|(d|0)==65539|(d|0)==65538){i=16}else if((d|0)==65552|(d|0)==65549|(d|0)==65548|(d|0)==407917392|(d|0)==65551|(d|0)==65550){i=24}else if((d|0)==65545|(d|0)==65544|(d|0)==542135120|(d|0)==65547|(d|0)==65546|(d|0)==65556|(d|0)==65557){i=32}else if((d|0)==65558|(d|0)==65559){i=64}else if((d|0)==86071|(d|0)==86072|(d|0)==69644|(d|0)==69661|(d|0)==69655|(d|0)==1330333984|(d|0)==69636|(d|0)==69660|(d|0)==69646){i=4}else{i=0}j=(f|0)>0;k=(b|0)>0;L127:do{if((i|0)!=0&j&k&(f|0)<32768){l=b;m=ac(i,f)|0;n=N0(l<<3|0>>>29,((b|0)<0|0?-1:0)<<3|l>>>29,m,(m|0)<0|0?-1:0)|0;o=n}else{n=c[a+792>>2]|0;if((d|0)==69641){o=32;break}else if((d|0)==69632){o=64;break}else if((d|0)==69657){o=128;break}else if((d|0)==73728|(d|0)==1936029283|(d|0)==86034|(d|0)==86040|(d|0)==77825){o=160;break}else if((d|0)==73729|(d|0)==86046){o=320;break}else if((d|0)==86059){o=384;break}else if((d|0)==86063){o=512;break}else if((d|0)==86047){o=1024;break}else if((d|0)==86016|(d|0)==86044){o=1152;break}else if((d|0)==86019){o=1536;break}else{do{if((e|0)>0){if((d|0)==86038){o=(e<<8|0)/245|0;break L127}if(!(j&(d|0)==86065)){break}o=(480<<((e|0)/22050|0)|0)/(f|0)|0;break L127}}while(0);m=(g|0)>0;do{if(m){if((d|0)==86076){if((g|0)==38){o=160;break L127}else if((g|0)!=50){break}o=240;break L127}else if((d|0)!=86058){break}if((g|0)==20){o=160;break L127}else if((g|0)==19){o=144;break L127}else if((g|0)==29){o=288;break L127}else if((g|0)==37){o=480;break L127}else{break}}}while(0);if(!k){o=0;break}if((d|0)==86037){o=((b|0)/32|0)*240|0;break}else if((d|0)==86050){o=((b|0)/64|0)<<8;break}else if((d|0)==77824){o=((b|0)/20|0)*160|0;break}else if((d|0)==86069){o=((b|0)/24|0)*240|0;break}else{l=(n|0)>0;if(l&(d|0)==69643){o=(b<<3|0)/(n|0)|0;break}if(!j){o=0;break}if((d|0)==1095123744){o=((b|0)/(f*9|0|0)|0)<<4;break}else if((d|0)==1146374944){o=((b|0)/(f<<4|0)|0)*28|0;break}else if((d|0)==69639|(d|0)==69659){o=(b-(f<<2)<<1|0)/(f|0)|0;break}else if((d|0)==69637){o=((b<<1)-8|0)/(f|0)|0;break}else if((d|0)==69651){o=((b<<1)-16|0)/(f|0)|0;break}else if((d|0)==69640){o=(((b|0)/128|0)*224|0|0)/(f|0)|0;break}else if((d|0)==81921){o=(b-6-f|0)/(f|0)|0;break}else if((d|0)==81920){o=(b-8|0)/(f|0)|0;break}else if((d|0)==81922){o=(b-(f<<1)|0)/(f|0)|0;break}else if((d|0)==86025){o=(b*3|0|0)/(f|0)|0;break}else if((d|0)==86026){o=(b*6|0|0)/(f|0)|0;break}else if((d|0)==65561){o=((b|0)/(f*5|0|0)|0)<<1;break}else if((d|0)==86075|(d|0)==86043){o=(b<<2|0)/(f|0)|0;break}else{if((h|0)!=0&(d|0)==81923){if((h|0)==3){o=(b|0)/(f|0)|0;break}else{o=(b<<1|0)/(f|0)|0;break}}do{if(m){p=(b|0)/(g|0)|0;if((d|0)==69633){if((n-2|0)>>>0>3){o=0;break L127}o=ac(((g-(f<<2)|0)/(ac(n,f)|0)|0)<<3|1,p)|0;break L127}else if((d|0)==69634){o=ac(((((g<<1)-32|0)/3|0)<<2|0)/(f|0)|0,p)|0;break L127}else if((d|0)==69635){o=ac(((g-(f<<2)<<1|0)/(f|0)|0)+1|0,p)|0;break L127}else if((d|0)==1380008992){o=ac((g-(f<<2)<<1|0)/(f|0)|0,p)|0;break L127}else if((d|0)==69638){o=ac(((g+(f*-7|0)<<1|0)/(f|0)|0)+2|0,p)|0;break L127}else{break}}}while(0);if(!l){o=0;break}if((d|0)==65555){if((n|0)<4){o=0;break}o=((b|0)/(ac((n<<1|0)/8|0,f)|0)|0)<<1;break}else if((d|0)==65560){if((n|0)<4){o=0;break}o=(b|0)/((ac(n,f+1&-2)|0)/8|0|0)|0;break}else if((d|0)==65562){o=(((b|0)/((n+4|0)/4|0|0)|0)<<1|0)/(f|0)|0;break}else{o=0;break}}}}}}while(0);return o|0}function pu(b,c){b=b|0;c=c|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;d=c&255;if(c>>>0>254){e=b;f=c;c=1;while(1){g=e+1|0;a[e]=-1;h=f-255|0;i=c+1|0;if(h>>>0>254){e=g;f=h;c=i}else{break}}j=g;k=h&255;l=i}else{j=b;k=d;l=1}a[j]=k;return l|0}function pv(a){a=a|0;var b=0,c=0,d=0,e=0;b=a&255;c=a>>>8&255;d=a>>>16&255;e=a>>>24;return((e-97|0)>>>0<26?e^32:e)<<24|((b-97|0)>>>0<26?b^32:b)|((c-97|0)>>>0<26?c^32:c)<<8|((d-97|0)>>>0<26?d^32:d)<<16|0}function pw(a,b){a=a|0;b=b|0;var d=0,e=0,f=0;b=i;i=i+8|0;d=b|0;c[d>>2]=a;e=c[a+476>>2]|0;if((e|0)==0){f=d;A6(f);i=b;return}c$[e&127](a,a+944|0);f=d;A6(f);i=b;return}function px(a,b){a=a|0;b=b|0;var d=0;b=i;i=i+8|0;d=b|0;c[d>>2]=a;zj(d);i=b;return}function py(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;f=i;i=i+32|0;g=f|0;h=f+16|0;c[e>>2]=b;c[e+4>>2]=1;j=g|0;k=h|0;l=pI(c[b+96>>2]|0,c[b+100>>2]|0,30,j,k)|0;if((l|0)<0){AN(b,16,1653112,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=l;i=f;return n|0}l=c[j>>2]|0;if((c[k>>2]|0)<30){AN(b,16,1735992,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=-1094995529;i=f;return n|0}if((a[l]|0)!=1){AN(b,16,1731120,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=-1094995529;i=f;return n|0}if((NO(l+1|0,1639040,6)|0)!=0){AN(b,16,1726600,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=-1094995529;i=f;return n|0}if((a[l+29|0]&1)==0){AN(b,16,1722512,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=-1094995529;i=f;return n|0}k=l+28|0;c[e+12>>2]=1<<(a[k]&15);c[e+16>>2]=1<<((d[k]|0)>>>4);k=c[g+8>>2]|0;g=c[h+8>>2]|0;if((g|0)<7){AN(b,16,1672520,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=-1094995529;i=f;return n|0}if((a[k]|0)!=5){AN(b,16,1657848,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=-1094995529;i=f;return n|0}if((NO(k+1|0,1639040,6)|0)!=0){AN(b,16,1630192,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=-1094995529;i=f;return n|0}h=A1(g)|0;if((h|0)==0){AN(b,16,1623912,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m;n=-12;i=f;return n|0}if((g|0)>0){l=g-1|0;j=0;do{a[h+j|0]=a[k+(l-j)|0]|0;j=j+1|0;}while((j|0)<(g|0))}j=g<<3;g=j>>>0>2147483639;l=g?0:h;k=g?0:j;j=k+8|0;g=0;while(1){if((k-g|0)<=97){o=205;break}p=((g|0)<(j|0))+g|0;if((d[l+(g>>>3)|0]&128>>>((g&7)>>>0)|0)==0){g=p}else{o=203;break}}L267:do{if((o|0)==203){if((p|0)==0){o=205;break}do{if((k-p|0)>96){g=p;q=1;r=0;s=0;while(1){t=l+(g>>>3)|0;u=g+8|0;v=j>>>0>u>>>0?u:j;if((cV(d[t]|d[t+1|0]<<8|d[t+2|0]<<16|d[t+3|0]<<24|0)|0)<<(g&7)>>>0>1073741823){w=r;x=s;break}t=l+(v>>>3)|0;u=v+16|0;y=j>>>0>u>>>0?u:j;if(((cV(d[t]|d[t+1|0]<<8|d[t+2|0]<<16|d[t+3|0]<<24|0)|0)&-65536>>>((v&7)>>>0)|0)!=0){w=r;x=s;break}v=l+(y>>>3)|0;t=y+16|0;if(((cV(d[v]|d[v+1|0]<<8|d[v+2|0]<<16|d[v+3|0]<<24|0)|0)&-65536>>>((y&7)>>>0)|0)!=0){w=r;x=s;break}y=(j>>>0>t>>>0?t:j)+1|0;t=j>>>0>y>>>0?y:j;if((q|0)>64){w=r;x=s;break}y=l+(t>>>3)|0;v=(((cV(d[y]|d[y+1|0]<<8|d[y+2|0]<<16|d[y+3|0]<<24|0)|0)<<(t&7)>>>26)+1|0)==(q|0);y=v?1:r;u=v?q:s;if((k-t|0)>96){g=t;q=q+1|0;r=y;s=u}else{w=y;x=u;break}}if((w|0)==0){break}do{if((x|0)>2){s=b;AS(s,1613392,(m=i,i=i+8|0,c[m>>2]=x,m)|0);i=m;if((x|0)<=63){break}AN(s,16,1740448,(m=i,i=i+8|0,c[m>>2]=x,m)|0);i=m;break L267}}while(0);c[e+280>>2]=x;s=x-1|0;r=s>>>0>65535;q=r?16:0;g=r?s>>>16:s;if((g&65280|0)==0){z=g;A=q}else{z=g>>>8;A=q|8}q=(2<<A+1+(d[247648+z|0]|0))-2|0;c[e+284>>2]=q;c[e+288>>2]=(q|1)+1;if((p|0)<0){B=0}else{B=(j|0)<(p|0)?j:p}if((x|0)>0){q=B;g=s;while(1){s=-q|0;r=j-q|0;if((s|0)>40){C=s}else{C=(r|0)<40?r:40}r=C+q|0;c[e+24+(g<<2)>>2]=d[l+(r>>>3)|0]<<(r&7)>>>7&1;if((g|0)>0){q=((r|0)<(j|0))+r|0;g=g-1|0}else{break}}}A4(h);c[e+8>>2]=1;c[e+20>>2]=c[e+12+(c[e+24>>2]<<2)>>2];n=0;i=f;return n|0}}while(0);AN(b,16,1618080,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m}}while(0);if((o|0)==205){AN(b,16,1618080,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m}A4(h);n=-1094995529;i=f;return n|0}function pz(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0;f=i;if(!((c[a+8>>2]|0)!=0&(e|0)>0)){g=0;i=f;return g|0}e=a+20|0;h=c[e>>2]|0;j=d[b]|0;if((j&1|0)!=0){AN(c[a>>2]|0,16,1721392,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;g=-1094995529;i=f;return g|0}b=c[a+280>>2]|0;if((b|0)==1){l=0}else{l=(c[a+284>>2]&j)>>>1}if((l|0)>=(b|0)){AN(c[a>>2]|0,16,1693720,(k=i,i=i+1|0,i=i+7&-8,c[k>>2]=0,k)|0);i=k;g=-1094995529;i=f;return g|0}k=c[a+24+(l<<2)>>2]|0;if((k|0)==0){m=h}else{m=c[a+12+(((j&c[a+288>>2]|0)!=0)<<2)>>2]|0}j=c[a+12+(k<<2)>>2]|0;c[e>>2]=j;g=j+m>>2;i=f;return g|0}function pA(a){a=a|0;if((c[a+8>>2]|0)==0){return}c[a+20>>2]=c[a+12>>2];return}function pB(a){a=a|0;var b=0,e=0,f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0.0,s=0,t=0,u=0.0;b=i;e=c[a+60>>2]|0;f=e;if((c[a+432>>2]|0)==0){AN(a,16,1722488,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;j=-22;i=b;return j|0}c[e>>2]=a;k=c[a+96>>2]|0;l=a+12|0;m=c[(c[l>>2]|0)+12>>2]|0;do{if((m|0)==86024){if((c[a+100>>2]|0)<=5){n=0;break}o=k+4|0;n=(d[o]|d[o+1|0]<<8)<<16>>16&65535}else if((m|0)==86023){if((c[a+100>>2]|0)<=3){n=0;break}o=k+2|0;n=(d[o]|d[o+1|0]<<8)<<16>>16&65535}else{n=0}}while(0);m=e+60|0;c[m>>2]=n&1;c[e+52>>2]=n&2;o=n&4;p=e+56|0;c[p>>2]=o;do{if((c[(c[l>>2]|0)+12>>2]|0)==86024){if((c[a+100>>2]|0)<=7){break}q=k+4|0;if((d[q]|d[q+1|0]<<8)<<16>>16<<16>>16!=13|(o|0)==0){break}AN(a,24,1718312,(h=i,i=i+1|0,i=i+7&-8,c[h>>2]=0,h)|0);i=h;c[p>>2]=0}}while(0);if((rl(a,n)|0)<0){j=-1;i=b;return j|0}n=e+1088|0;if((c[n>>2]|0)>0){p=e+66720|0;h=e+1084|0;o=0;do{qZ(p+(o<<6)|0,1-o+(c[h>>2]|0)|0,1,30517578125.0e-15)|0;o=o+1|0;}while((o|0)<(c[n>>2]|0))}if((c[e+64>>2]|0)!=0){n=e+744|0;qk(n,9,37,237576,1,1,237496,2,2,0,0,0,0)|0}if((c[m>>2]|0)==0){m=c[e+1080>>2]|0;r=3.141592653589793/+(m|0);if((m|0)>0){n=0;while(1){g[f+149024+(n<<2)>>2]=+U(+(r*+(n|0)))*2.0;o=n+1|0;if((o|0)<(m|0)){n=o}else{s=0;break}}}else{s=0}while(1){g[f+157216+(s<<2)>>2]=+b4(+(+(s-126|0)*-.25));n=s+1|0;if((n|0)<256){s=n}else{t=127;u=1.0;break}}while(1){r=+T(+(+(t+128|0)*.00390625),+-.25);g[f+158240+(t<<2)>>2]=r*2.0-u;g[f+158752+(t<<2)>>2]=u-r;if((t|0)>0){t=t-1|0;u=r}else{break}}}else{qk(e+72|0,8,121,257160,1,1,256672,4,4,0,0,0,0)|0}c[a+420>>2]=8;j=0;i=b;return j|0}function pC(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0;h=i;j=e;k=c[g+24>>2]|0;l=c[g+28>>2]|0;g=c[b+60>>2]|0;m=g;if((l|0)==0){c[g+116244>>2]=0;n=0;i=h;return n|0}o=c[b+432>>2]|0;if((l|0)<(o|0)){AN(b,16,1693680,(p=i,i=i+16|0,c[p>>2]=l,c[p+8>>2]=o,p)|0);i=p;n=-1094995529;i=h;return n|0}q=(o|0)==0?l:o;o=q<<3;l=o>>>0>2147483639|(k|0)==0;r=l?0:k;s=l?0:o;l=g+4|0;c[l>>2]=r;t=g+16|0;c[t>>2]=s;u=s+8|0;v=g+20|0;c[v>>2]=u;w=g+8|0;c[w>>2]=r+(s>>3);s=g+12|0;c[s>>2]=0;x=g+52|0;do{if((c[x>>2]|0)==0){y=1}else{z=u>>>0>4?4:u;c[s>>2]=z;A=r+(z>>>3)|0;B=(cV(d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24|0)|0)<<(z&4)>>>28;A=z+4|0;c[s>>2]=u>>>0>A>>>0?A:u;A=B-((c[g+116244>>2]|0)<1)|0;if((A|0)>=1){y=A;break}AN(b,16,1672496,(p=i,i=i+8|0,c[p>>2]=A,p)|0);i=p;n=-1094995529;i=h;return n|0}}while(0);u=g+1080|0;c[e+76>>2]=ac(c[u>>2]|0,y)|0;r=oX(b,j,0)|0;if((r|0)<0){n=r;i=h;return n|0}r=c[e+64>>2]|0;L382:do{if((c[x>>2]|0)==0){if((pE(m,r,0)|0)<0){C=322}else{C=321}}else{e=g+68|0;j=c[e>>2]|0;A=c[s>>2]|0;B=c[v>>2]|0;z=c[l>>2]|0;D=z+(A>>>3)|0;E=(cV(d[D]|d[D+1|0]<<8|d[D+2|0]<<16|d[D+3|0]<<24|0)|0)<<(A&7)>>>((29-j|0)>>>0);D=j+3+A|0;A=B>>>0>D>>>0?D:B;c[s>>2]=A;D=(c[t>>2]|0)-A|0;if((E|0)>(D|0)){AN(b,16,1657792,(p=i,i=i+24|0,c[p>>2]=E,c[p+8>>2]=D,c[p+16>>2]=q,p)|0);i=p;C=322;break}D=g+116244|0;F=c[D>>2]|0;if((F|0)>0){if((F+(E+7>>3)|0)>16384){C=322;break}G=g+99840|0;H=g+(F+99840)|0;if((E|0)>7){I=E-8|0;J=I>>>3;K=J<<3;L=F+J+99841|0;J=E;F=H;M=A;A=B;B=z;while(1){z=B+(M>>>3)|0;N=(cV(d[z]|d[z+1|0]<<8|d[z+2|0]<<16|d[z+3|0]<<24|0)|0)<<(M&7)>>>24;z=M+8|0;c[s>>2]=A>>>0>z>>>0?z:A;a[F]=N&255;N=J-8|0;if((N|0)<=7){break}J=N;F=F+1|0;M=c[s>>2]|0;A=c[v>>2]|0;B=c[l>>2]|0}O=I-K|0;P=g+L|0}else{O=E;P=H}if((O|0)>0){B=c[s>>2]|0;A=c[v>>2]|0;M=(c[l>>2]|0)+(B>>>3)|0;F=(cV(d[M]|d[M+1|0]<<8|d[M+2|0]<<16|d[M+3|0]<<24|0)|0)<<(B&7)>>>((32-O|0)>>>0);M=B+O|0;c[s>>2]=A>>>0>M>>>0?M:A;a[P]=F<<8-O&255;Q=P+1|0}else{Q=P}NM(Q|0,0,16);F=(c[D>>2]<<3)+E|0;A=F>>>0>2147483639|(G|0)==0;M=A?0:G;B=A?0:F;c[l>>2]=M;c[t>>2]=B;F=B+8|0;c[v>>2]=F;c[w>>2]=M+(B+7>>3);c[s>>2]=0;B=c[g+116240>>2]|0;if((B|0)>0){c[s>>2]=F>>>0>B>>>0?B:F}if((pE(m,r,0)|0)<0){C=322;break}R=c[u>>2]|0;S=y-1|0;T=c[e>>2]|0}else{R=0;S=y;T=j}F=E+11|0;B=F+T|0;if((B|0)>131071|(B|0)>(o|0)){n=-1094995529;i=h;return n|0}M=B>>3;A=k+M|0;J=q-M<<3;M=J>>>0>2147483639|(A|0)==0;N=M?0:A;A=M?0:J;c[l>>2]=N;c[t>>2]=A;J=A+8|0;c[v>>2]=J;c[w>>2]=N+(A>>3);A=B&7;if((A|0)==0){U=0}else{U=J>>>0>A>>>0?A:J}c[s>>2]=U;c[g+1092>>2]=1;if((S|0)>0){J=0;A=R;while(1){if((pE(m,r,A)|0)<0){C=322;break L382}B=J+1|0;if((B|0)<(S|0)){J=B;A=(c[u>>2]|0)+A|0}else{break}}V=c[s>>2]|0;W=c[e>>2]|0}else{V=U;W=T}A=(F+W&-8)+V|0;c[g+116240>>2]=A&7;J=A>>3;A=q-J|0;if(A>>>0>16384){AN(c[g>>2]|0,16,1639024,(p=i,i=i+8|0,c[p>>2]=A,p)|0);i=p;C=322;break}else{c[D>>2]=A;E=g+99840|0;j=k+J|0;NN(E|0,j|0,A)|0;C=321;break}}}while(0);if((C|0)==321){c[f>>2]=1;n=q;i=h;return n|0}else if((C|0)==322){c[g+116244>>2]=0;n=-1;i=h;return n|0}return 0}function pD(a){a=a|0;var b=0;b=c[a+60>>2]|0;c[b+116244>>2]=0;c[b+116240>>2]=0;return}
function pE(f,h,j){f=f|0;h=h|0;j=j|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0.0,br=0.0,bs=0.0,bt=0.0,bu=0.0,bv=0.0,bw=0.0,bx=0.0,by=0.0,bz=0.0,bA=0.0,bB=0.0,bC=0.0,bD=0.0,bE=0.0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0.0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0.0,b$=0.0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0.0,cb=0,cc=0,cd=0,ce=0,cf=0,cg=0,ch=0,ci=0,cj=0.0,ck=0,cl=0,cm=0,cn=0,co=0,cp=0.0,cq=0,cr=0,cs=0,ct=0,cu=0,cv=0,cw=0,cx=0,cy=0,cz=0,cA=0,cB=0,cC=0,cD=0,cE=0,cF=0,cG=0,cH=0,cI=0,cJ=0;l=i;i=i+112|0;m=l|0;n=l+40|0;o=l+48|0;p=f+1112|0;c[p>>2]=0;q=f+1116|0;c[q>>2]=0;r=f+56|0;s=f+1084|0;t=f+1100|0;u=f+1104|0;v=f+1096|0;w=f+1088|0;x=f+1108|0;y=f+1080|0;z=f|0;A=f+12|0;B=f+4|0;C=f+20|0;D=f+1120|0;E=f+4|0;F=E|0;G=f+380|0;H=f+60|0;I=m|0;J=m+4|0;K=m+8|0;L=m+12|0;M=m+16|0;N=m+20|0;O=m+24|0;P=m+28|0;Q=m+32|0;R=m+36|0;U=f+48|0;V=f+76|0;W=f+149016|0;X=f+149020|0;Y=f+1122|0;Z=f+1121|0;_=f+33952|0;$=f+159308|0;aa=f+33952|0;ab=f+42144|0;ac=f+64|0;ad=f+748|0;ae=f+50336|0;af=f+159300|0;ag=f+159304|0;ah=f+50336|0;ai=f+1092|0;aj=0;L428:while(1){if((c[r>>2]|0)==0){ak=c[s>>2]|0;c[t>>2]=ak;c[u>>2]=ak;c[v>>2]=ak;al=ak;am=ak;an=c[w>>2]|0}else{ak=c[w>>2]|0;ao=ak-1|0;ap=ao>>>0>65535;aq=ap?16:0;ar=ap?ao>>>16:ao;if((ar&65280|0)==0){as=ar;at=aq}else{as=ar>>>8;at=aq|8}aq=(d[247648+as|0]|0)+at|0;ar=aq+1|0;if((c[ai>>2]|0)==0){c[u>>2]=c[v>>2];ao=c[t>>2]|0;c[v>>2]=ao;au=ao;av=c[A>>2]|0;aw=c[C>>2]|0;ax=c[B>>2]|0;ay=31-aq|0}else{c[ai>>2]=0;ao=c[A>>2]|0;ap=c[C>>2]|0;az=c[B>>2]|0;aA=az+(ao>>>3)|0;aB=31-aq|0;aC=(cV(d[aA]|d[aA+1|0]<<8|d[aA+2|0]<<16|d[aA+3|0]<<24|0)|0)<<(ao&7)>>>(aB>>>0);aA=ao+ar|0;ao=ap>>>0>aA>>>0?aA:ap;c[A>>2]=ao;if((aC|0)>=(ak|0)){aD=338;break}aE=c[s>>2]|0;c[u>>2]=aE-aC;aA=az+(ao>>>3)|0;aF=(cV(d[aA]|d[aA+1|0]<<8|d[aA+2|0]<<16|d[aA+3|0]<<24|0)|0)<<(ao&7)>>>(aB>>>0);aA=ao+ar|0;ao=ap>>>0>aA>>>0?aA:ap;c[A>>2]=ao;if((aF|0)>=(ak|0)){aD=340;break}aA=aE-aF|0;c[v>>2]=aA;au=aA;av=ao;aw=ap;ax=az;ay=aB}aB=ax+(av>>>3)|0;aG=(cV(d[aB]|d[aB+1|0]<<8|d[aB+2|0]<<16|d[aB+3|0]<<24|0)|0)<<(av&7)>>>(ay>>>0);aB=av+ar|0;c[A>>2]=aw>>>0>aB>>>0?aB:aw;if((aG|0)>=(ak|0)){aD=344;break}aB=c[s>>2]|0;c[t>>2]=aB-aG;al=aB;am=au;an=ak}ak=al-am|0;if((ak|0)>=(an|0)){aD=348;break}aB=1<<am;c[x>>2]=aB;aH=c[z>>2]|0;if((aj+aB|0)>(c[y>>2]|0)){aD=350;break}aB=aH+416|0;ar=c[aB>>2]|0;if((ar|0)==2){az=c[A>>2]|0;ap=d[(c[B>>2]|0)+(az>>>3)|0]<<(az&7)>>>7;c[A>>2]=((az|0)<(c[C>>2]|0))+az;a[D]=ap&1;aI=c[aB>>2]|0}else{aI=ar}do{if((aI|0)>0){ar=0;aB=0;do{ap=c[A>>2]|0;az=d[(c[B>>2]|0)+(ap>>>3)|0]<<(ap&7)>>>7;c[A>>2]=((ap|0)<(c[C>>2]|0))+ap;ap=az&1;a[f+1121+ar|0]=ap&255;aB=ap|aB;ar=ar+1|0;aJ=c[z>>2]|0;}while((ar|0)<(c[aJ+416>>2]|0));ar=(c[s>>2]|0)-(c[v>>2]|0)|0;if((aB|0)==0){aK=aJ;aL=ar;aM=aJ;break}ap=c[C>>2]|0;az=c[F>>2]|0;ao=1;aA=c[A>>2]|0;do{aq=az+(aA>>>3)|0;aN=(cV(d[aq]|d[aq+1|0]<<8|d[aq+2|0]<<16|d[aq+3|0]<<24|0)|0)<<(aA&7)>>>25;aq=aA+7|0;aA=ap>>>0>aq>>>0?aq:ap;c[A>>2]=aA;ao=aN+ao|0;}while((aN|0)==127);aA=rn(ao)|0;ap=f+384+(ar<<2)|0;az=(c[ap>>2]|0)-(c[G>>2]|0)|0;aB=c[z>>2]|0;aN=c[aB+416>>2]|0;do{if((aN|0)>0){aq=0;do{c[n+(aq<<2)>>2]=az;aq=aq+1|0;}while((aq|0)<(aN|0));if((c[ac>>2]|0)==0){aO=aB;aP=aB;aQ=aN;break}aq=f+404+(ar<<2)|0;aR=0;aS=aB;while(1){do{if((a[f+1121+aR|0]|0)==0){aT=aS}else{aU=c[aq>>2]|0;if((aU|0)<=0){aT=aS;break}aV=n+(aR<<2)|0;aW=0;do{aX=c[A>>2]|0;aY=d[(c[F>>2]|0)+(aX>>>3)|0]<<(aX&7)>>>7;c[A>>2]=((aX|0)<(c[C>>2]|0))+aX;aX=aY&1;c[f+760+(aR<<6)+(aW<<2)>>2]=aX;if((aX|0)!=0){c[aV>>2]=(c[aV>>2]|0)-(c[f+424+(ar<<6)+(aW<<2)>>2]|0)}aW=aW+1|0;}while((aW|0)<(aU|0));aT=c[z>>2]|0}}while(0);aU=aR+1|0;aZ=c[aT+416>>2]|0;if((aU|0)<(aZ|0)){aR=aU;aS=aT}else{break}}if((aZ|0)>0){a_=0;a$=aT;a0=aT}else{aO=aT;aP=aT;aQ=aZ;break}while(1){do{if((a[f+1121+a_|0]|0)==0){a1=a$;a2=a0}else{aS=c[aq>>2]|0;if((aS|0)>0){a3=0;a4=-2147483648}else{a1=a$;a2=a0;break}while(1){if((c[f+760+(a_<<6)+(a3<<2)>>2]|0)==0){a5=a4}else{if((a4|0)==-2147483648){aR=c[A>>2]|0;aU=c[C>>2]|0;aW=(c[F>>2]|0)+(aR>>>3)|0;aV=(cV(d[aW]|d[aW+1|0]<<8|d[aW+2|0]<<16|d[aW+3|0]<<24|0)|0)<<(aR&7)>>>25;aW=aR+7|0;c[A>>2]=aU>>>0>aW>>>0?aW:aU;a6=aV-19|0}else{aV=c[ad>>2]|0;aU=c[A>>2]|0;aW=c[C>>2]|0;aR=c[F>>2]|0;aX=aR+(aU>>>3)|0;aY=(cV(d[aX]|d[aX+1|0]<<8|d[aX+2|0]<<16|d[aX+3|0]<<24|0)|0)<<(aU&7)>>>23;aX=b[aV+(aY<<2)>>1]|0;a7=b[aV+(aY<<2)+2>>1]|0;aY=a7<<16>>16;if(a7<<16>>16<0){a7=aU+9|0;a8=aW>>>0>a7>>>0?a7:aW;a7=aR+(a8>>>3)|0;aR=((cV(d[a7]|d[a7+1|0]<<8|d[a7+2|0]<<16|d[a7+3|0]<<24|0)|0)<<(a8&7)>>>((aY+32|0)>>>0))+aX|0;a9=b[aV+(aR<<2)>>1]|0;ba=a8;bb=b[aV+(aR<<2)+2>>1]|0}else{a9=aX;ba=aU;bb=aY}aY=ba+bb|0;c[A>>2]=aW>>>0>aY>>>0?aY:aW;if((a9|0)<0){aD=379;break L428}a6=a4-18+a9|0}c[f+888+(a_<<6)+(a3<<2)>>2]=a6;a5=a6}aW=a3+1|0;if((aW|0)<(aS|0)){a3=aW;a4=a5}else{break}}aS=c[z>>2]|0;a1=aS;a2=aS}}while(0);aS=a_+1|0;aW=c[a1+416>>2]|0;if((aS|0)<(aW|0)){a_=aS;a$=a1;a0=a2}else{aO=a1;aP=a2;aQ=aW;break}}}else{aO=aB;aP=aB;aQ=aN}}while(0);if((c[v>>2]|0)==(c[s>>2]|0)){bc=aQ;aD=387}else{aN=c[A>>2]|0;aB=d[(c[F>>2]|0)+(aN>>>3)|0]|0;c[A>>2]=((aN|0)<(c[C>>2]|0))+aN;az=c[aO+416>>2]|0;if((aB&128>>>((aN&7)>>>0)|0)==0){bd=aO;be=aP;bf=az;aD=388}else{bc=az;aD=387}}do{if((aD|0)==387){aD=0;if((bc|0)>0){bg=0;bh=aO;bi=aP}else{bj=bc;bk=aP;bl=aO;aD=389;break}while(1){if((a[f+1121+bg|0]|0)==0){bm=bh;bn=bi}else{if((c[H>>2]|0)==0){az=c[C>>2]|0;aN=c[B>>2]|0;aB=0;aq=c[A>>2]|0;while(1){aW=aN+(aq>>>3)|0;aS=(cV(d[aW]|d[aW+1|0]<<8|d[aW+2|0]<<16|d[aW+3|0]<<24|0)|0)<<(aq&7);if((aB|0)==0|(aB|0)>7){aW=aq+3|0;bo=aS>>>29;bp=az>>>0>aW>>>0?aW:az}else{aW=aq+4|0;bo=aS>>>28;bp=az>>>0>aW>>>0?aW:az}c[A>>2]=bp;g[m+(aB<<2)>>2]=+g[236856+(aB<<6)+(bo<<2)>>2];aW=aB+1|0;if((aW|0)<10){aB=aW;aq=bp}else{break}}aq=f+17536+(bg<<2)|0;aB=c[x>>2]|0;if((aB|0)>0){bq=+g[I>>2];br=+g[J>>2];bs=+g[K>>2];bt=+g[L>>2];bu=+g[M>>2];bv=+g[N>>2];bw=+g[O>>2];bx=+g[P>>2];by=+g[Q>>2];bz=+g[R>>2];az=0;bA=0.0;while(1){bB=+g[f+149024+(az<<2)>>2];bC=(bB-by)*(bB-bw)*(bB-bu)*(bB-bs)*(bB-bq)*.5;bD=(bB-bz)*(bB-bx)*(bB-bv)*(bB-bt)*(bB-br)*.5;aN=(g[k>>2]=bC*(bB+2.0)*bC+bD*(2.0-bB)*bD,c[k>>2]|0);aW=aN>>>16&127;bD=+g[f+157216+(aN>>>23<<2)>>2]*(+g[f+158240+(aW<<2)>>2]+ +g[f+158752+(aW<<2)>>2]*(c[k>>2]=aN<<7&8388480|1065353216,+g[k>>2]));bB=bD>bA?bD:bA;g[f+1152+(bg<<13)+(az<<2)>>2]=bD;aN=az+1|0;if((aN|0)<(aB|0)){az=aN;bA=bB}else{bE=bB;break}}}else{bE=0.0}g[aq>>2]=bE}else{az=(c[s>>2]|0)-(c[v>>2]|0)|0;aB=f+108+(az*50|0)|0;aN=f+1152+(bg<<13)|0;aW=f+1152+(bg<<13)+(c[x>>2]<<2)|0;do{if((c[U>>2]|0)==1){aS=c[A>>2]|0;aY=c[C>>2]|0;aU=(c[B>>2]|0)+(aS>>>3)|0;aX=(cV(d[aU]|d[aU+1|0]<<8|d[aU+2|0]<<16|d[aU+3|0]<<24|0)|0)<<(aS&7)>>>27;aU=aS+5|0;c[A>>2]=aY>>>0>aU>>>0?aU:aY;aY=aX+10|0;bA=+g[90960+(aX+70<<2)>>2];br=+bA;aX=f+108+(az*50|0)+2|0;aU=e[aB>>1]|0;aS=aU&3;if((aS|0)==2){bF=aN;bG=aU;aD=396}else if((aS|0)==1){bH=aN;bI=aU;aD=397}else if((aS|0)==3){bJ=aN;bK=aU;aD=395}else if((aS|0)==0){bL=aN;bM=aU;aD=394}else{bN=aN;bO=bA;bP=aX;bQ=aY;break}while(1){if((aD|0)==396){aD=0;g[bF>>2]=br;bH=bF+4|0;bI=bG;aD=397;continue}else if((aD|0)==397){aD=0;aU=bH+4|0;g[bH>>2]=br;aS=bI-4|0;if((aS|0)>0){bL=aU;bM=aS;aD=394;continue}else{bN=aU;bO=bA;bP=aX;bQ=aY;break}}else if((aD|0)==395){aD=0;g[bJ>>2]=br;bF=bJ+4|0;bG=bK;aD=396;continue}else if((aD|0)==394){aD=0;g[bL>>2]=br;bJ=bL+4|0;bK=bM;aD=395;continue}}}else{bN=aN;bO=0.0;bP=aB;bQ=36}}while(0);if(bN>>>0<aW>>>0){aB=bQ;aN=bP;br=bO;az=bN;while(1){aq=c[V>>2]|0;aY=c[C>>2]|0;aX=c[B>>2]|0;bA=br;aU=aN;aS=aB;aR=c[A>>2]|0;while(1){aV=aX+(aR>>>3)|0;a8=(cV(d[aV]|d[aV+1|0]<<8|d[aV+2|0]<<16|d[aV+3|0]<<24|0)|0)<<(aR&7)>>>24;aV=b[aq+(a8<<2)>>1]|0;a7=b[aq+(a8<<2)+2>>1]|0;a8=a7<<16>>16;do{if(a7<<16>>16<0){bR=aR+8|0;bS=aY>>>0>bR>>>0?bR:aY;bR=aX+(bS>>>3)|0;bT=((cV(d[bR]|d[bR+1|0]<<8|d[bR+2|0]<<16|d[bR+3|0]<<24|0)|0)<<(bS&7)>>>((a8+32|0)>>>0))+aV|0;bR=b[aq+(bT<<2)>>1]|0;bU=b[aq+(bT<<2)+2>>1]|0;bT=bU<<16>>16;if(bU<<16>>16>=0){bV=bR;bW=bS;bX=bT;break}bU=bS-a8|0;bS=aY>>>0>bU>>>0?bU:aY;bU=aX+(bS>>>3)|0;bY=((cV(d[bU]|d[bU+1|0]<<8|d[bU+2|0]<<16|d[bU+3|0]<<24|0)|0)<<(bS&7)>>>((bT+32|0)>>>0))+bR|0;bV=b[aq+(bY<<2)>>1]|0;bW=bS;bX=b[aq+(bY<<2)+2>>1]|0}else{bV=aV;bW=aR;bX=a8}}while(0);a8=bW+bX|0;aV=aY>>>0>a8>>>0?a8:aY;c[A>>2]=aV;if((bV|0)<0){aD=407;break L428}bZ=aS-60+bV|0;a8=bV+aS|0;if(a8>>>0>155){aD=408;break L428}bt=+g[90960+(a8<<2)>>2];b_=+bt;b$=bt>bA?bt:bA;b0=aU+2|0;a8=e[aU>>1]|0;a7=a8&3;if((a7|0)==0){b1=az;b2=a8;aD=409;break}else if((a7|0)==3){b3=az;b4=a8;aD=410;break}else if((a7|0)==2){b5=az;b6=a8;aD=411;break}else if((a7|0)==1){b7=az;b8=a8;aD=412;break}else{bA=b$;aU=b0;aS=bZ;aR=aV}}while(1){if((aD|0)==409){aD=0;g[b1>>2]=b_;b3=b1+4|0;b4=b2;aD=410;continue}else if((aD|0)==410){aD=0;g[b3>>2]=b_;b5=b3+4|0;b6=b4;aD=411;continue}else if((aD|0)==411){aD=0;g[b5>>2]=b_;b7=b5+4|0;b8=b6;aD=412;continue}else if((aD|0)==412){aD=0;b9=b7+4|0;g[b7>>2]=b_;aR=b8-4|0;if((aR|0)>0){b1=b9;b2=aR;aD=409;continue}else{break}}}if(b9>>>0<aW>>>0){aB=bZ;aN=b0;br=b$;az=b9}else{ca=b$;break}}}else{ca=bO}g[f+17536+(bg<<2)>>2]=ca}c[f+1124+(bg<<2)>>2]=ar;az=c[z>>2]|0;bm=az;bn=az}az=bg+1|0;aN=c[bm+416>>2]|0;if((az|0)<(aN|0)){bg=az;bh=bm;bi=bn}else{bd=bm;be=bn;bf=aN;aD=388;break}}}}while(0);do{if((aD|0)==388){aD=0;if((bf|0)>0){cb=0;cc=be}else{bj=bf;bk=be;bl=bd;aD=389;break}while(1){if((a[f+1121+cb|0]|0)==0){cd=cc}else{aN=f+17544+(cb<<13)|0;if((cb|0)==1){ce=(a[D]|0)!=0|0}else{ce=0}NM(aN|0,0,c[x>>2]<<2|0);az=c[z>>2]|0;aB=f+1016+(ce<<4)|0;aW=c[f+1056+(ce<<2)>>2]|0;aR=c[f+1048+(ce<<2)>>2]|0;aS=c[n+(cb<<2)>>2]|0;aU=c[x>>2]|0;aY=c[s>>2]|0;rq(az,E,aB,aW,aR,0,aN,0,aS,aU,aY,aA)|0;cd=c[z>>2]|0}aY=c[U>>2]|0;aU=cd+416|0;do{if((aY|0)==1){if((c[aU>>2]|0)<=1){break}aS=c[A>>2]|0;aN=-aS&7;if((aN|0)==0){break}aR=c[C>>2]|0;aW=aN+aS|0;c[A>>2]=aR>>>0>aW>>>0?aW:aR}}while(0);aR=cb+1|0;aW=c[aU>>2]|0;if((aR|0)<(aW|0)){cb=aR;cc=cd}else{cf=cd;cg=aW;ch=aY;ci=cd;break}}}}while(0);if((aD|0)==389){aD=0;cf=bl;cg=bj;ch=c[U>>2]|0;ci=bk}aA=(c[x>>2]|0)/2|0;br=1.0/+(aA|0);if((ch|0)==1){cj=br*+S(+(+(aA|0)))}else{cj=br}if((cg|0)>0){br=+(ao|0)*.05;aA=f+404+(ar<<2)|0;aW=f+360+(ar<<2)|0;aR=-1<<ar;aS=0;while(1){do{if((a[f+1121+aS|0]|0)!=0){aN=f+17544+(aS<<13)|0;aB=c[f+1124+(aS<<2)>>2]|0;bA=+T(+10.0,+br);az=f+17536+(aS<<2)|0;bt=cj*(bA/+g[az>>2]);aq=f+33952+(aS<<13)|0;aX=c[G>>2]|0;aV=(aX|0)>0;if((c[ac>>2]|0)==0){if(aV){a8=aq;a7=0;while(1){bY=a8+4|0;g[a8>>2]=0.0;bS=a7+1|0;if((bS|0)<(c[G>>2]|0)){a8=bY;a7=bS}else{ck=bY;break}}}else{ck=aq}a7=c[n+(aS<<2)>>2]|0;if((a7|0)>0){a8=ck;aY=0;while(1){g[a8>>2]=bt*+g[f+17544+(aS<<13)+(aY<<2)>>2]*+g[f+1152+(aS<<13)+(aY<<ar>>aB<<2)>>2];aU=aY+1|0;if((aU|0)<(a7|0)){a8=a8+4|0;aY=aU}else{break}}cl=ck+(a7<<2)|0}else{cl=ck}aY=(c[x>>2]|0)-(c[ap>>2]|0)|0;if((aY|0)<=0){break}NM(cl|0,0,aY<<2|0);break}if(aV){aY=aq;a8=0;aU=c[W>>2]|0;while(1){bY=aY+4|0;g[aY>>2]=bt*+g[f+116248+(aU<<2)>>2]*+g[f+1152+(aS<<13)+(a8<<ar>>aB<<2)>>2];bS=(c[W>>2]|0)+1&8191;c[W>>2]=bS;bR=a8+1|0;bT=c[G>>2]|0;if((bR|0)<(bT|0)){aY=bY;a8=bR;aU=bS}else{cm=bY;cn=bT;break}}}else{cm=aq;cn=aX}aU=c[aA>>2]|0;if((aU|0)>0){a8=(c[s>>2]|0)-(c[v>>2]|0)|0;aY=f+1152+(aS<<13)+(c[aW>>2]<<ar>>aB<<2)|0;aV=0;a7=0;while(1){bT=c[f+424+(a8<<6)+(aV<<2)>>2]|0;if((c[f+760+(aS<<6)+(aV<<2)>>2]|0)==0){co=a7}else{if((bT|0)>0){bY=0;bA=0.0;while(1){bv=+g[aY+(bY<<ar>>aB<<2)>>2];bx=bA+bv*bv;bS=bY+1|0;if((bS|0)<(bT|0)){bY=bS;bA=bx}else{cp=bx;break}}}else{cp=0.0}g[o+(aV<<2)>>2]=cp/+(bT|0);co=aV}bY=aV+1|0;if((bY|0)<(aU|0)){aY=aY+(bT<<ar>>aB<<2)|0;aV=bY;a7=co}else{cq=co;break}}}else{cq=0}a7=f+1152+(aS<<13)+(cn<<ar>>aB<<2)|0;if((aU|0)>-1){aV=o+(cq<<2)|0;aY=aN;a8=cm;aX=a7;aq=-1;while(1){do{if((aq|0)<0){cr=(c[aW>>2]|0)-(c[G>>2]|0)|0;aD=455}else{bY=c[f+424+((c[s>>2]|0)-(c[v>>2]|0)<<6)+(aq<<2)>>2]|0;if((c[f+760+(aS<<6)+(aq<<2)>>2]|0)==0){cr=bY;aD=455;break}bA=+S(+(+g[o+(aq<<2)>>2]/+g[aV>>2]));bx=bA*+T(+10.0,+(+(c[f+888+(aS<<6)+(aq<<2)>>2]|0)*.05));bA=cj*(bx/(+g[az>>2]*+g[X>>2]));if((bY|0)>0){cs=a8;ct=0}else{cu=a8;cv=aY;cw=bY;break}while(1){bS=c[W>>2]|0;bx=+g[f+116248+(bS<<2)>>2];c[W>>2]=bS+1&8191;g[cs>>2]=bA*bx*+g[aX+(ct<<ar>>aB<<2)>>2];bS=ct+1|0;if((bS|0)>=(bY|0)){break}cs=cs+4|0;ct=bS}cu=a8+(bY<<2)|0;cv=aY;cw=bY}}while(0);do{if((aD|0)==455){aD=0;if((cr|0)>0){cx=aY;cy=a8;cz=0}else{cu=a8;cv=aY;cw=cr;break}while(1){bT=c[W>>2]|0;bA=+g[f+116248+(bT<<2)>>2];c[W>>2]=bT+1&8191;g[cy>>2]=bt*(bA+ +g[cx>>2])*+g[aX+(cz<<ar>>aB<<2)>>2];bT=cz+1|0;if((bT|0)>=(cr|0)){break}cx=cx+4|0;cy=cy+4|0;cz=bT}cu=a8+(cr<<2)|0;cv=aY+(cr<<2)|0;cw=cr}}while(0);bY=aX+(cw<<ar>>aB<<2)|0;bT=aq+1|0;if((bT|0)<(aU|0)){aY=cv;a8=cu;aX=bY;aq=bT}else{cA=cu;cB=bY;break}}}else{cA=cm;cB=a7}aq=(c[x>>2]|0)-(c[ap>>2]|0)|0;bA=bt*+g[cB+(aR>>aB<<2)>>2];if((aq|0)<=0){break}aX=cA;a8=0;aY=c[W>>2]|0;while(1){g[aX>>2]=bA*+g[f+116248+(aY<<2)>>2];aU=(c[W>>2]|0)+1&8191;c[W>>2]=aU;az=a8+1|0;if((az|0)<(aq|0)){aX=aX+4|0;a8=az;aY=aU}else{break}}}}while(0);aY=aS+1|0;a8=c[z>>2]|0;if((aY|0)<(c[a8+416>>2]|0)){aS=aY}else{cC=a8;cD=a8;break}}}else{cC=cf;cD=ci}if((a[D]|0)==0){aK=cC;aL=ar;aM=cD;break}if((a[Y]|0)==0){aK=cC;aL=ar;aM=cD;break}aS=c[x>>2]|0;if((a[Z]|0)==0){NM(_|0,0,aS<<2|0);a[Z]=1}dm[c[$>>2]&127](aa,ab,aS);aS=c[z>>2]|0;aK=aS;aL=ar;aM=aS}else{aK=aH;aL=ak;aM=aH}}while(0);ak=f+66720+(aL<<6)|0;if((c[aK+416>>2]|0)>0){aS=f+66720+(aL<<6)+40|0;aR=0;while(1){ap=c[x>>2]|0;do{if((a[f+1121+aR|0]|0)==0){if((a[D]|0)!=0&(aR|0)==1){break}NM(ae|0,0,16384)}else{dm[c[aS>>2]&127](ak,ah,f+33952+(aR<<13)|0)}}while(0);ar=(c[q>>2]|0)+((c[y>>2]|0)/2|0)+((ap|0)/-2|0)|0;aW=f+67072+(aR<<14)+(ar<<2)|0;aA=c[v>>2]|0;ao=c[u>>2]|0;if((aA|0)>(ao|0)){a8=1<<ao;aY=((c[x>>2]|0)-a8|0)/2|0;aX=f+67072+(aR<<14)+(aY+ar<<2)|0;cX[c[af>>2]&255](aX,f+50336+(aY<<2)|0,c[f+67040+((c[s>>2]|0)-ao<<2)>>2]|0,aX,a8);aX=aY+a8|0;a8=f+67072+(aR<<14)+(aX+ar<<2)|0;ao=f+50336+(aX<<2)|0;aX=aY<<2;NN(a8|0,ao|0,aX)|0}else{cX[c[af>>2]&255](aW,ah,c[f+67040+((c[s>>2]|0)-aA<<2)>>2]|0,aW,c[x>>2]|0)}aW=c[x>>2]|0;aA=f+67072+(aR<<14)+(aW+ar<<2)|0;aX=f+50336+(aW<<2)|0;ao=c[v>>2]|0;a8=c[t>>2]|0;if((ao|0)>(a8|0)){aY=1<<a8;aq=(aW-aY|0)/2|0;aB=(c[s>>2]|0)-a8|0;a8=aA;a7=aX;aU=aq<<2;NN(a8|0,a7|0,aU)|0;a7=aq+aW|0;dq[c[ag>>2]&127](f+67072+(aR<<14)+(a7+ar<<2)|0,f+50336+(a7<<2)|0,c[f+67040+(aB<<2)>>2]|0,aY);NM(f+67072+(aR<<14)+(aY+ar+a7<<2)|0,0,aU|0)}else{dq[c[ag>>2]&127](aA,aX,c[f+67040+((c[s>>2]|0)-ao<<2)>>2]|0,aW)}aW=aR+1|0;ao=c[z>>2]|0;if((aW|0)<(c[ao+416>>2]|0)){aR=aW}else{cE=ao;break}}}else{cE=aM}c[p>>2]=(c[p>>2]|0)+1;aR=(c[q>>2]|0)+(c[x>>2]|0)|0;c[q>>2]=aR;cF=c[y>>2]|0;if((aR|0)<(cF|0)){aj=aR}else{aD=494;break}}if((aD|0)==379){AN(c[z>>2]|0,16,1735968,(cG=i,i=i+1|0,i=i+7&-8,c[cG>>2]=0,cG)|0);i=cG;cH=-1;i=l;return cH|0}else if((aD|0)==407){AN(c[z>>2]|0,16,1731096,(cG=i,i=i+1|0,i=i+7&-8,c[cG>>2]=0,cG)|0);i=cG;cH=-1;i=l;return cH|0}else if((aD|0)==408){AN(c[z>>2]|0,16,1726568,(cG=i,i=i+8|0,c[cG>>2]=bZ,cG)|0);i=cG;cH=-1;i=l;return cH|0}else if((aD|0)==338){AN(c[z>>2]|0,16,1630152,(cG=i,i=i+8|0,c[cG>>2]=(c[s>>2]|0)-aC,cG)|0);i=cG;cH=-1;i=l;return cH|0}else if((aD|0)==340){AN(c[z>>2]|0,16,1623880,(cG=i,i=i+8|0,c[cG>>2]=aE-aF,cG)|0);i=cG;cH=-1;i=l;return cH|0}else if((aD|0)==344){AN(c[z>>2]|0,16,1618040,(cG=i,i=i+8|0,c[cG>>2]=(c[s>>2]|0)-aG,cG)|0);i=cG;cH=-1;i=l;return cH|0}else if((aD|0)==348){AN(c[z>>2]|0,16,1613336,(cG=i,i=i+1|0,i=i+7&-8,c[cG>>2]=0,cG)|0);i=cG;cH=-1;i=l;return cH|0}else if((aD|0)==350){AN(aH,16,1740424,(cG=i,i=i+1|0,i=i+7&-8,c[cG>>2]=0,cG)|0);i=cG;cH=-1;i=l;return cH|0}else if((aD|0)==494){if((c[cE+416>>2]|0)>0){cI=0;cJ=cF}else{cH=0;i=l;return cH|0}while(1){cF=(c[h+(cI<<2)>>2]|0)+(j<<2)|0;cE=f+67072+(cI<<14)|0;aD=cJ<<2;NN(cF|0,cE|0,aD)|0;aD=c[y>>2]|0;NP(cE|0,f+67072+(cI<<14)+(aD<<2)|0,aD<<2|0);aD=cI+1|0;if((aD|0)>=(c[(c[z>>2]|0)+416>>2]|0)){cH=0;break}cI=aD;cJ=c[y>>2]|0}i=l;return cH|0}return 0}function pF(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;d=i;e=c[b+60>>2]|0;c[e>>2]=b;f=b+416|0;g=c[f>>2]|0;if((g|0)>2){AN(b,16,1740376,(h=i,i=i+16|0,c[h>>2]=g,c[h+8>>2]=2,h)|0);i=h;j=-22;i=d;return j|0}g=b+412|0;k=c[g>>2]|0;if((k|0)>48e3){AN(b,16,1735904,(h=i,i=i+8|0,c[h>>2]=k,h)|0);i=h;j=-22;i=d;return j|0}k=b+72|0;l=c[k>>2]|0;if((l|0)<24e3){AN(b,16,1731048,(h=i,i=i+8|0,c[h>>2]=l,h)|0);i=h;j=-22;i=d;return j|0}l=c[(c[b+12>>2]|0)+12>>2]|0;if((l|0)==86024){m=A7(10)|0;c[b+100>>2]=10;n=m;y=0;a[n]=y&255;y=y>>8;a[n+1|0]=y&255;y=y>>8;a[n+2|0]=y&255;y=y>>8;a[n+3|0]=y&255;n=m+4|0;y=1;a[n]=y&255;y=y>>8;a[n+1|0]=y&255;o=m}else if((l|0)==86023){l=A1(4)|0;c[b+100>>2]=4;m=l;y=0;a[m]=y&255;y=y>>8;a[m+1|0]=y&255;m=l+2|0;y=1;a[m]=y&255;y=y>>8;a[m+1|0]=y&255;o=l}else{AN(0,0,1672464,(h=i,i=i+24|0,c[h>>2]=1618032,c[h+8>>2]=1639e3,c[h+16>>2]=69,h)|0);i=h;cw();return 0}c[b+96>>2]=o;c[e+60>>2]=1;c[e+52>>2]=0;c[e+56>>2]=0;if((c[f>>2]|0)==2){a[e+1120|0]=1}rl(b,1)|0;f=e+1088|0;if((c[f>>2]|0)>0){o=e+66720|0;h=e+1084|0;l=0;do{qZ(o+(l<<6)|0,1-l+(c[h>>2]|0)|0,0,1.0)|0;l=l+1|0;}while((l|0)<(c[f>>2]|0))}f=c[k>>2]|0;k=e+1080|0;e=c[k>>2]|0;l=N2(e,(e|0)<0|0?-1:0,f,(f|0)<0|0?-1:0)|0;f=c[g>>2]<<3;g=N0(l,G,f,(f|0)<0|0?-1:0)|0;f=g;c[b+432>>2]=(f|0)>16384?16384:f;f=c[k>>2]|0;c[b+116>>2]=f;c[b+424>>2]=f;j=0;i=d;return j|0}function pG(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0.0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0.0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0;h=i;i=i+8|0;j=h|0;k=b+60|0;l=c[k>>2]|0;m=l;n=c[l+1084>>2]|0;c[l+1096>>2]=n;o=l+1108|0;c[o>>2]=1<<n;n=c[e+64>>2]|0;p=c[e+76>>2]|0;q=c[k>>2]|0;k=c[q+1096>>2]|0;r=(c[q+1084>>2]|0)-k|0;s=q+66720|0;t=s+(r<<6)|0;u=c[q+67040+(r<<2)>>2]|0;v=1<<k;w=65536.0/+(v|0);k=b+416|0;if((c[k>>2]|0)>0){x=q+50336|0;z=q+67072|0;A=v<<2;B=q+159288|0;C=q+159304|0;D=x+(v<<2)|0;v=q+159280|0;E=s+(r<<6)+48|0;r=q+33952|0;q=x;s=0;do{F=z+(s<<14)|0;H=F;NN(x|0,H|0,A)|0;H=F|0;cY[c[B>>2]&15](H,c[n+(s<<2)>>2]|0,w,p);dq[c[C>>2]&127](D,H,u,p);dq[c[v>>2]&127](H,H,u,p);dm[c[E>>2]&127](t,r+(s<<13)|0,q);s=s+1|0;}while((s|0)<(c[k>>2]|0))}do{if((a[l+1120|0]|0)!=0){if((c[o>>2]|0)<=0){break}k=l+33952|0;s=l+42144|0;q=0;do{r=k+(q<<2)|0;w=+g[r>>2]*.5;t=s+(q<<2)|0;I=+g[t>>2]*.5;g[r>>2]=w+I;g[t>>2]=w-I;q=q+1|0;}while((q|0)<(c[o>>2]|0))}}while(0);o=pa(b,d,32768)|0;if((o|0)<0){J=o;i=h;return J|0}o=l+33952|0;q=d+24|0;s=d+28|0;k=(pH(m,o,c[q>>2]|0,c[s>>2]|0,64)|0)<1?64:128;t=k-32|0;r=(pH(m,o,c[q>>2]|0,c[s>>2]|0,t)|0)<1?t:k;k=r-16|0;t=(pH(m,o,c[q>>2]|0,c[s>>2]|0,k)|0)<1?k:r;r=t-8|0;k=(pH(m,o,c[q>>2]|0,c[s>>2]|0,r)|0)<1?r:t;t=k-4|0;r=(pH(m,o,c[q>>2]|0,c[s>>2]|0,t)|0)<1?t:k;k=r-2|0;t=(pH(m,o,c[q>>2]|0,c[s>>2]|0,k)|0)<1?k:r;r=t-1|0;k=pH(m,o,c[q>>2]|0,c[s>>2]|0,r)|0;E=(k|0)<1?r:t;t=(k|0)>0;if((E|0)<129&t){k=E;while(1){E=k+1|0;r=(pH(m,o,c[q>>2]|0,c[s>>2]|0,k)|0)>0;if((E|0)<129&r){k=E}else{K=r;break}}}else{K=t}if(K){AN(b,16,1693616,(L=i,i=i+1|0,i=i+7&-8,c[L>>2]=0,L)|0);i=L;c[s>>2]=0;J=-22;i=h;return J|0}K=l+28|0;t=c[K>>2]|0;k=l+32|0;q=l+36|0;o=c[q>>2]|0;m=(o-(c[k>>2]|0)<<3)+(32-t)|0;if((m&7|0)!=0){AN(0,0,1672464,(L=i,i=i+24|0,c[L>>2]=1657752,c[L+8>>2]=1639e3,c[L+16>>2]=384,L)|0);i=L;cw();return 0}r=b+432|0;E=c[r>>2]|0;p=(m+7|0)/8|0;m=E-p|0;if((m|0)<=-1){AN(0,0,1672464,(L=i,i=i+24|0,c[L>>2]=1630144,c[L+8>>2]=1639e3,c[L+16>>2]=386,L)|0);i=L;cw();return 0}u=l+24|0;if((E|0)==(p|0)){M=t;N=o}else{p=m;m=o;o=c[u>>2]|0;E=t;while(1){t=p-1|0;if((E|0)>8){O=E-8|0;P=o<<8|78;Q=m}else{l=78>>>((8-E|0)>>>0)|o<<E;v=m;y=l>>>16<<8&65280|l>>>24|(l>>>8&255|l<<8)<<16;a[v]=y&255;y=y>>8;a[v+1|0]=y&255;y=y>>8;a[v+2|0]=y&255;y=y>>8;a[v+3|0]=y&255;v=(c[q>>2]|0)+4|0;c[q>>2]=v;O=E+24|0;P=78;Q=v}c[u>>2]=P;c[K>>2]=O;if((t|0)==0){M=O;N=Q;break}else{p=t;m=Q;o=P;E=O}}}if((M|0)<32){O=c[u>>2]<<M;c[u>>2]=O;M=O>>>24&255;O=N;while(1){c[q>>2]=O+1;a[O]=M;E=c[u>>2]|0;c[u>>2]=E<<8;P=(c[K>>2]|0)+8|0;c[K>>2]=P;if((P|0)>=32){break}M=E>>>16&255;O=c[q>>2]|0}R=c[q>>2]|0}else{R=N}c[K>>2]=32;c[u>>2]=0;u=R-(c[k>>2]|0)|0;if((u|0)!=(c[r>>2]|0)){AN(0,0,1672464,(L=i,i=i+24|0,c[L>>2]=1623824,c[L+8>>2]=1639e3,c[L+16>>2]=391,L)|0);i=L;cw();return 0}L=e+136|0;e=c[L>>2]|0;k=c[L+4>>2]|0;if((e|0)==0&(k|0)==(-2147483648|0)){S=u}else{u=c[b+116>>2]|0;c[j>>2]=1;c[j+4>>2]=c[b+412>>2];L=AY(u,(u|0)<0|0?-1:0,j,b+104|0)|0;b=NV(e,k,L,G)|0;L=d+8|0;c[L>>2]=b;c[L+4>>2]=G;S=c[r>>2]|0}c[s>>2]=S;c[f>>2]=1;J=0;i=h;return J|0}function pH(b,f,h,j,k){b=b|0;f=f|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0.0,D=0.0,E=0,F=0,G=0.0,H=0,I=0,J=0,K=0,L=0.0,M=0.0,N=0,O=0,P=0,Q=0.0,R=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0;l=i;i=i+8|0;m=l|0;n=b+24|0;o=(j|0)<0;p=o?0:h;h=o?0:j;c[b+44>>2]=h<<3;j=b+32|0;c[j>>2]=p;c[b+40>>2]=p+h;h=b+36|0;c[h>>2]=p;p=b+28|0;c[p>>2]=32;o=n|0;c[o>>2]=0;if((c[b+52>>2]|0)!=0){AN(0,0,1672464,(q=i,i=i+24|0,c[q>>2]=1618032,c[q+8>>2]=1639e3,c[q+16>>2]=332,q)|0);i=q;cw();return 0}if((c[b+56>>2]|0)!=0){AN(0,0,1672464,(q=i,i=i+24|0,c[q>>2]=1618032,c[q+8>>2]=1639e3,c[q+16>>2]=171,q)|0);i=q;cw();return 0}r=b+1084|0;s=c[r>>2]|0;c[b+1100>>2]=s;c[b+1104>>2]=s;t=b+1096|0;c[t>>2]=s;u=1<<s;s=b+1108|0;c[s>>2]=u;v=b+380|0;w=(c[b+384>>2]|0)-(c[v>>2]|0)|0;x=b|0;z=c[x>>2]|0;A=c[z+416>>2]|0;if((A|0)>0){B=0;do{c[m+(B<<2)>>2]=w;B=B+1|0;}while((B|0)<(A|0))}B=(u|0)/2|0;C=1.0/+(B|0);u=b+48|0;if((c[u>>2]|0)==1){D=C*+S(+(+(B|0)))}else{D=C}if((A|0)==2){c[b+24>>2]=(a[b+1120|0]|0)!=0;c[p>>2]=31;E=c[z+416>>2]|0}else{E=A}L774:do{if((E|0)>0){A=0;do{a[b+1121+A|0]=1;z=c[s>>2]|0;B=b+1152+(A<<13)+(z<<2)|0;if((z|0)>0){z=b+108+(((c[r>>2]|0)-(c[t>>2]|0)|0)*50|0)|0;w=b+1152+(A<<13)|0;C=0.0;F=260552;while(1){G=+T(+10.0,+(+(c[F>>2]|0)*.0625));H=e[z>>1]|0;I=w;J=H;while(1){g[I>>2]=G;K=J-1|0;if((K|0)==0){break}else{I=I+4|0;J=K}}L=C>G?C:G;J=w+(H<<2)|0;if(J>>>0<B>>>0){z=z+2|0;w=J;C=L;F=F+4|0}else{M=L;break}}}else{M=0.0}g[b+17536+(A<<2)>>2]=M;A=A+1|0;N=c[x>>2]|0;O=c[N+416>>2]|0;}while((A|0)<(O|0));if((O|0)<=0){break}C=+(k|0)*.05;A=0;F=N;L787:while(1){do{if((a[b+1121+A|0]|0)==0){P=F}else{L=+T(+10.0,+C);Q=D*(L/+g[b+17536+(A<<2)>>2]);w=c[m+(A<<2)>>2]|0;if((w|0)<=0){P=F;break}z=0;B=f+(A<<13)+(c[v>>2]<<2)|0;while(1){L=+g[B>>2]/(Q*+g[b+1152+(A<<13)+(z<<2)>>2]);if(L<-32768.0|L>32767.0){R=2147483647;U=683;break L787}g[b+17544+(A<<13)+(z<<2)>>2]=+(bR(+L)|0);J=z+1|0;if((J|0)<(w|0)){z=J;B=B+4|0}else{break}}P=c[x>>2]|0}}while(0);B=A+1|0;V=c[P+416>>2]|0;if((B|0)<(V|0)){A=B;F=P}else{break}}if((U|0)==683){i=l;return R|0}if((V|0)<=0){break}F=b+24|0;A=0;B=0;z=c[F>>2]|0;w=c[p>>2]|0;H=P;while(1){J=d[b+1121+A|0]|0;if((w|0)>1){W=w-1|0;X=J|z<<1;Y=H}else{I=J>>>((1-w|0)>>>0)|z<<w;K=c[h>>2]|0;y=I>>>16<<8&65280|I>>>24|(I>>>8&255|I<<8)<<16;a[K]=y&255;y=y>>8;a[K+1|0]=y&255;y=y>>8;a[K+2|0]=y&255;y=y>>8;a[K+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;W=w+31|0;X=J;Y=c[x>>2]|0}c[F>>2]=X;c[p>>2]=W;Z=J|B;J=A+1|0;if((J|0)<(c[Y+416>>2]|0)){A=J;B=Z;z=X;w=W;H=Y}else{break}}if((Z|0)==0){break}H=k-1|0;w=(W|0)>7;if((H|0)>126){z=k-128|0;B=H;A=X;J=W;K=w;while(1){if(K){_=J-7|0;$=A<<7|127}else{I=127>>>((7-J|0)>>>0)|A<<J;aa=c[h>>2]|0;y=I>>>16<<8&65280|I>>>24|(I>>>8&255|I<<8)<<16;a[aa]=y&255;y=y>>8;a[aa+1|0]=y&255;y=y>>8;a[aa+2|0]=y&255;y=y>>8;a[aa+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;_=J+25|0;$=127}c[o>>2]=$;c[p>>2]=_;aa=B-127|0;ab=(_|0)>7;if((aa|0)>126){B=aa;A=$;J=_;K=ab}else{break}}K=(z>>>0)%127|0;if(ab){ac=_;ad=$;ae=K;U=606}else{af=_;ag=$;ah=K;U=607}}else{if(w){ac=W;ad=X;ae=H;U=606}else{af=W;ag=X;ah=H;U=607}}if((U|0)==607){K=ah>>>((7-af|0)>>>0)|ag<<af;J=c[h>>2]|0;y=K>>>16<<8&65280|K>>>24|(K>>>8&255|K<<8)<<16;a[J]=y&255;y=y>>8;a[J+1|0]=y&255;y=y>>8;a[J+2|0]=y&255;y=y>>8;a[J+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;ai=af+25|0;aj=ah}else if((U|0)==606){ai=ac-7|0;aj=ad<<7|ae}c[o>>2]=aj;c[p>>2]=ai;J=rn(k)|0;do{if((c[b+64>>2]|0)!=0){K=c[x>>2]|0;if((c[K+416>>2]|0)<=0){break}A=b+404|0;B=0;aa=K;while(1){do{if((a[b+1121+B|0]|0)==0){ak=aa}else{K=c[A>>2]|0;if((K|0)>0){al=0}else{ak=aa;break}do{c[b+760+(B<<6)+(al<<2)>>2]=0;I=c[o>>2]|0;am=c[p>>2]|0;if((am|0)>1){an=am-1|0;ao=I<<1}else{ap=cV(I<<am|0)|0;I=c[h>>2]|0;y=ap;a[I]=y&255;y=y>>8;a[I+1|0]=y&255;y=y>>8;a[I+2|0]=y&255;y=y>>8;a[I+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;an=am+31|0;ao=0}c[o>>2]=ao;c[p>>2]=an;al=al+1|0;}while((al|0)<(K|0));ak=c[x>>2]|0}}while(0);K=B+1|0;if((K|0)<(c[ak+416>>2]|0)){B=K;aa=ak}else{break}}}}while(0);if((c[t>>2]|0)!=(c[r>>2]|0)){H=c[o>>2]|0;w=c[p>>2]|0;if((w|0)>1){aq=w-1|0;ar=H<<1|1}else{z=1>>>((1-w|0)>>>0)|H<<w;H=c[h>>2]|0;y=z>>>16<<8&65280|z>>>24|(z>>>8&255|z<<8)<<16;a[H]=y&255;y=y>>8;a[H+1|0]=y&255;y=y>>8;a[H+2|0]=y&255;y=y>>8;a[H+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;aq=w+31|0;ar=1}c[o>>2]=ar;c[p>>2]=aq}if((c[(c[x>>2]|0)+416>>2]|0)<=0){break}w=b+60|0;H=0;L847:do{do{if((a[b+1121+H|0]|0)!=0){if((c[w>>2]|0)==0){U=642;break L847}z=(c[r>>2]|0)-(c[t>>2]|0)|0;aa=b+108+(z*50|0)|0;B=b+1152+(H<<13)+(c[s>>2]<<2)|0;if((c[u>>2]|0)==1){A=c[F>>2]|0;K=c[p>>2]|0;if((K|0)>5){as=K-5|0;at=A<<5|10}else{am=10>>>((5-K|0)>>>0)|A<<K;A=c[h>>2]|0;y=am>>>16<<8&65280|am>>>24|(am>>>8&255|am<<8)<<16;a[A]=y&255;y=y>>8;a[A+1|0]=y&255;y=y>>8;a[A+2|0]=y&255;y=y>>8;a[A+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;as=K+27|0;at=10}c[F>>2]=at;c[p>>2]=as;au=260556;av=20;aw=b+108+(z*50|0)+2|0;ax=b+1152+(H<<13)+(e[aa>>1]<<2)|0}else{au=260552;av=36;aw=aa;ax=b+1152+(H<<13)|0}if(ax>>>0>=B>>>0){break}aa=ax;z=aw;K=av;A=au;am=c[F>>2]|0;I=c[p>>2]|0;while(1){ap=c[A>>2]|0;ay=ap-K+60|0;az=d[257160+ay|0]|0;aA=c[256672+(ay<<2)>>2]|0;if((I|0)>(az|0)){aB=I;aC=am<<az|aA}else{ay=aA>>>((az-I|0)>>>0)|am<<I;aD=c[h>>2]|0;y=ay>>>16<<8&65280|ay>>>24|(ay>>>8&255|ay<<8)<<16;a[aD]=y&255;y=y>>8;a[aD+1|0]=y&255;y=y>>8;a[aD+2|0]=y&255;y=y>>8;a[aD+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;aB=I+32|0;aC=aA}aA=aB-az|0;c[F>>2]=aC;c[p>>2]=aA;az=aa+(e[z>>1]<<2)|0;if(az>>>0<B>>>0){aa=az;z=z+2|0;K=ap;A=A+4|0;am=aC;I=aA}else{break}}}}while(0);H=H+1|0;aE=c[(c[x>>2]|0)+416>>2]|0;}while((H|0)<(aE|0));if((U|0)==642){AN(0,0,1672464,(q=i,i=i+24|0,c[q>>2]=1618032,c[q+8>>2]=1639e3,c[q+16>>2]=274,q)|0);i=q;cw();return 0}if((aE|0)<=0){break}H=b+1120|0;F=1<<J;w=32-J|0;I=0;L872:while(1){do{if((a[b+1121+I|0]|0)!=0){if((I|0)==1){aF=(a[H]|0)!=0|0}else{aF=0}am=c[m+(I<<2)>>2]|0;A=b+17544+(I<<13)+(am<<2)|0;if((am|0)<=0){break}am=b+1072+(aF<<2)|0;K=b+1064+(aF<<2)|0;z=0;aa=b+17544+(I<<13)|0;while(1){C=+g[aa>>2];if(C!=0.0){B=~~C;aA=(B|0)>-1?B:-B|0;ap=c[am>>2]|0;do{if((aA|0)>(c[ap+4>>2]|0)){aG=0}else{az=aA-1|0;if((z|0)>=(e[(c[ap+16>>2]|0)+(az<<1)>>1]|0)){aG=0;break}aG=(e[(c[K>>2]|0)+(az<<1)>>1]|0)+z|0}}while(0);az=d[(c[ap+12>>2]|0)+aG|0]|0;aD=c[(c[ap+8>>2]|0)+(aG<<2)>>2]|0;ay=c[o>>2]|0;aH=c[p>>2]|0;if((aH|0)>(az|0)){aI=aH-az|0;aJ=ay<<az|aD}else{aK=aD>>>((az-aH|0)>>>0)|ay<<aH;ay=c[h>>2]|0;y=aK>>>16<<8&65280|aK>>>24|(aK>>>8&255|aK<<8)<<16;a[ay]=y&255;y=y>>8;a[ay+1|0]=y&255;y=y>>8;a[ay+2|0]=y&255;y=y>>8;a[ay+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;aI=32-az+aH|0;aJ=aD}c[o>>2]=aJ;c[p>>2]=aI;if((aG|0)==0){if((F|0)<=(aA|0)){R=2147483647;break L872}if((aI|0)>(J|0)){aL=aI-J|0;aM=aJ<<J|aA}else{aD=aA>>>((J-aI|0)>>>0)|aJ<<aI;aH=c[h>>2]|0;y=aD>>>16<<8&65280|aD>>>24|(aD>>>8&255|aD<<8)<<16;a[aH]=y&255;y=y>>8;a[aH+1|0]=y&255;y=y>>8;a[aH+2|0]=y&255;y=y>>8;a[aH+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;aL=w+aI|0;aM=aA}c[o>>2]=aM;c[p>>2]=aL;aH=c[r>>2]|0;if((aL|0)>(aH|0)){aN=aL;aO=aM<<aH|z}else{aD=z>>>((aH-aL|0)>>>0)|aM<<aL;az=c[h>>2]|0;y=aD>>>16<<8&65280|aD>>>24|(aD>>>8&255|aD<<8)<<16;a[az]=y&255;y=y>>8;a[az+1|0]=y&255;y=y>>8;a[az+2|0]=y&255;y=y>>8;a[az+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;aN=aL+32|0;aO=z}az=aN-aH|0;c[o>>2]=aO;c[p>>2]=az;aP=az;aQ=aO}else{aP=aI;aQ=aJ}az=B>>>31;if((aP|0)>1){aR=aP-1|0;aS=aQ<<1|az}else{aH=az>>>((1-aP|0)>>>0)|aQ<<aP;aD=c[h>>2]|0;y=aH>>>16<<8&65280|aH>>>24|(aH>>>8&255|aH<<8)<<16;a[aD]=y&255;y=y>>8;a[aD+1|0]=y&255;y=y>>8;a[aD+2|0]=y&255;y=y>>8;a[aD+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;aR=aP+31|0;aS=az}c[o>>2]=aS;c[p>>2]=aR;aT=0}else{aT=z+1|0}az=aa+4|0;if(az>>>0<A>>>0){z=aT;aa=az}else{break}}if((aT|0)==0){break}aa=c[am>>2]|0;z=d[(c[aa+12>>2]|0)+1|0]|0;A=c[(c[aa+8>>2]|0)+4>>2]|0;aa=c[o>>2]|0;K=c[p>>2]|0;if((K|0)>(z|0)){aU=K-z|0;aV=aa<<z|A}else{az=A>>>((z-K|0)>>>0)|aa<<K;aa=c[h>>2]|0;y=az>>>16<<8&65280|az>>>24|(az>>>8&255|az<<8)<<16;a[aa]=y&255;y=y>>8;a[aa+1|0]=y&255;y=y>>8;a[aa+2|0]=y&255;y=y>>8;a[aa+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;aU=32-z+K|0;aV=A}c[o>>2]=aV;c[p>>2]=aU}}while(0);do{if((c[u>>2]|0)==1){if((c[(c[x>>2]|0)+416>>2]|0)<=1){break}qj(n)}}while(0);I=I+1|0;if((I|0)>=(c[(c[x>>2]|0)+416>>2]|0)){break L774}}i=l;return R|0}}while(0);qj(n);R=((32-(c[p>>2]|0)+((c[h>>2]|0)-(c[j>>2]|0)<<3)|0)/8|0)-(c[(c[x>>2]|0)+432>>2]|0)|0;i=l;return R|0}function pI(b,e,f,g,h){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;do{if((e|0)>5){i=b;if(((cT((d[i]|d[i+1|0]<<8)<<16>>16|0)|0)&65535|0)!=(f|0)){break}c[h>>2]=f;c[g>>2]=b+2;i=c[h>>2]|0;if((e-i|0)<6){j=-1;return j|0}k=i+2|0;l=i+6|0;m=b+k|0;n=h+4|0;c[n>>2]=(cT((d[m]|d[m+1|0]<<8)<<16>>16|0)|0)&65535;c[g+4>>2]=b+(i+4);i=c[n>>2]|0;if((l|0)>(e-i|0)){j=-1;return j|0}else{n=k+(i+2)|0;k=b+n|0;m=h+8|0;c[m>>2]=(cT((d[k]|d[k+1|0]<<8)<<16>>16|0)|0)&65535;c[g+8>>2]=b+(n+2);return((i+l|0)>(e-(c[m>>2]|0)|0))<<31>>31|0}}}while(0);if((e-3|0)>>>0>=2147483133){j=-1;return j|0}if((a[b]|0)==2){o=3;p=0;q=b}else{j=-1;return j|0}while(1){r=q+1|0;if((p|0)>=2){break}b=h+(p<<2)|0;c[b>>2]=0;L943:do{if((o|0)<(e|0)){f=r;m=o;l=0;while(1){if((a[f]|0)!=-1){s=f;t=m;u=l;break L943}i=l+255|0;c[b>>2]=i;n=m+256|0;k=f+1|0;if((n|0)<(e|0)){f=k;m=n;l=i}else{s=k;t=n;u=i;break}}}else{s=r;t=o;u=0}}while(0);c[b>>2]=u+(d[s]|0);l=(d[s]|0)+t|0;if((l|0)>(e|0)){j=-1;v=703;break}else{o=l;p=p+1|0;q=s}}if((v|0)==703){return j|0}c[h+8>>2]=e-o;c[g>>2]=r;r=(c[h>>2]|0)+1|0;c[g+4>>2]=q+r;c[g+8>>2]=q+(r+(c[h+4>>2]|0));j=0;return j|0}function pJ(b){b=b|0;var f=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0.0,C=0.0;f=i;i=i+192|0;h=f|0;j=c[b+60>>2]|0;k=j;l=j+4|0;c[l>>2]=b;m=b+412|0;c[j+6416>>2]=c[m>>2];c[j+6676>>2]=54;c[j+6680>>2]=50;c[j+6684>>2]=22;c[j+6688>>2]=26;c[j+6692>>2]=10;c[b+420>>2]=8;n=c[b+100>>2]|0;do{if((n|0)>0){o=pN(k,c[l>>2]|0,j+6408|0,c[b+96>>2]|0,n<<3,1)|0;if((o|0)<0){p=o;i=f;return p|0}else{q=b+416|0;break}}else{o=c[m>>2]|0;do{if((o|0)>92016){r=0}else{if((o|0)>75131){r=1;break}if((o|0)>55425){r=2;break}if((o|0)>46008){r=3;break}if((o|0)>37565){r=4;break}if((o|0)>27712){r=5;break}if((o|0)>23003){r=6;break}if((o|0)>18782){r=7;break}if((o|0)>13855){r=8;break}if((o|0)>11501){r=9;break}r=(o|0)>9390?10:11}}while(0);c[j+6412>>2]=r;o=b+416|0;c[j+6444>>2]=c[o>>2];c[j+6424>>2]=-1;c[j+6448>>2]=-1;s=0;while(1){if(s>>>0>=8){break}if((d[245544+s|0]|0)==(c[o>>2]|0)){break}else{s=s+1|0}}t=(s|0)==8?0:s;c[j+6420>>2]=t;if((t|0)==0){q=o;break}u=t-1|0;if(u>>>0<=6){v=h|0;w=a[64504+t|0]|0;x=h|0;y=350944+(u*15|0)|0;u=w*3|0;NN(x|0,y|0,u)|0;pS(k,v,w,3,0)|0;q=o;break}AN(b,16,1659440,(z=i,i=i+8|0,c[z>>2]=t,z)|0);i=z;if((c[b+696>>2]&8|0)==0){q=o;break}else{p=-1094995529}i=f;return p|0}}while(0);if((c[q>>2]|0)>64){AN(b,16,1708232,(z=i,i=i+1|0,i=i+7&-8,c[z>>2]=0,z)|0);i=z;p=-1094995529;i=f;return p|0}c[499387]=3117768;c[499389]=304;qk(1997544,8,e[128276]|0,c[64156]|0,1,1,c[64144]|0,2,2,0,0,0,4)|0;c[499391]=3116688;c[499393]=270;qk(1997560,8,e[128277]|0,c[64157]|0,1,1,c[64145]|0,2,2,0,0,0,4)|0;c[499395]=3114488;c[499397]=550;qk(1997576,8,e[128278]|0,c[64158]|0,1,1,c[64146]|0,2,2,0,0,0,4)|0;c[499399]=3113288;c[499401]=300;qk(1997592,8,e[128279]|0,c[64159]|0,1,1,c[64147]|0,2,2,0,0,0,4)|0;c[499403]=3111976;c[499405]=328;qk(1997608,8,e[128280]|0,c[64160]|0,1,1,c[64148]|0,2,2,0,0,0,4)|0;c[499407]=3110800;c[499409]=294;qk(1997624,8,e[128281]|0,c[64161]|0,1,1,c[64149]|0,2,2,0,0,0,4)|0;c[499411]=3109576;c[499413]=306;qk(1997640,8,e[128282]|0,c[64162]|0,1,1,c[64150]|0,2,2,0,0,0,4)|0;c[499415]=3108504;c[499417]=268;qk(1997656,8,e[128283]|0,c[64163]|0,1,1,c[64151]|0,2,2,0,0,0,4)|0;c[499419]=3106464;c[499421]=510;qk(1997672,8,e[128284]|0,c[64164]|0,1,1,c[64152]|0,2,2,0,0,0,4)|0;c[499423]=3105e3;c[499425]=366;qk(1997688,8,e[128285]|0,c[64165]|0,1,1,c[64153]|0,2,2,0,0,0,4)|0;c[499427]=3103152;c[499429]=462;qk(1997704,8,e[128286]|0,c[64166]|0,1,1,c[64154]|0,2,2,0,0,0,4)|0;p$();qU(j+5312|0,b);z7(j+5328|0,c[b+88>>2]&8388608);c[j+5364>>2]=523124044;qe();c[499431]=3101744;c[499433]=352;qk(1997720,7,121,257160,1,1,256672,4,4,0,0,0,4)|0;qZ(j+5120|0,11,1,2.9802322387695312e-8)|0;qZ(j+5184|0,8,1,2.384185791015625e-7)|0;qZ(j+5248|0,11,0,-65536.0)|0;qY(2902632,4.0,1024);qY(2902120,6.0,128);rk(10);rk(7);if((c[753533]|0)==0){A=0}else{p=0;i=f;return p|0}while(1){B=+(A|0);C=B*+a_(+B);g[2981368+(A<<2)>>2]=C;j=A+1|0;if((j|0)<8192){A=j}else{p=0;break}}i=f;return p|0}function pK(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=i;i=i+40|0;j=h|0;k=h+32|0;l=c[b+60>>2]|0;m=c[g+24>>2]|0;n=c[g+28>>2]|0;nL(g,1,h+24|0)|0;o=nL(g,71,k)|0;p=l+5628|0;c[p>>2]=0;if((o|0)!=0&(c[k>>2]|0)>0){c[p>>2]=(d[o]|0)+1}o=c[l+5624>>2]|0;if((o|0)>-1){c[p>>2]=o}if((n|0)>268435454){q=-1094995529;i=h;return q|0}o=n<<3;p=o>>>0>2147483639|(m|0)==0;l=p?0:m;k=p?0:o;c[j>>2]=l;c[j+12>>2]=k;c[j+16>>2]=k+8;c[j+4>>2]=l+(k>>3);k=j+8|0;c[k>>2]=0;l=pO(b,e,f,j,g)|0;if((l|0)<0){q=l;i=h;return q|0}l=(c[k>>2]|0)+7>>3;k=l;while(1){if((k|0)>=(n|0)){q=n;r=748;break}if((a[m+k|0]|0)==0){k=k+1|0}else{q=l;r=749;break}}if((r|0)==748){i=h;return q|0}else if((r|0)==749){i=h;return q|0}return 0}function pL(a){a=a|0;var b=0,d=0,e=0,f=0;b=c[a+60>>2]|0;a=b+492|0;d=0;do{e=a+(d<<2)|0;f=c[e>>2]|0;if((f|0)!=0){p1(f+108256|0)}A6(e);e=a+64+(d<<2)|0;f=c[e>>2]|0;if((f|0)!=0){p1(f+108256|0)}A6(e);e=a+128+(d<<2)|0;f=c[e>>2]|0;if((f|0)!=0){p1(f+108256|0)}A6(e);e=a+192+(d<<2)|0;f=c[e>>2]|0;if((f|0)!=0){p1(f+108256|0)}A6(e);d=d+1|0;}while((d|0)<16);q_(b+5120|0);q_(b+5184|0);q_(b+5248|0);return 0}function pM(a){a=a|0;var b=0,d=0,e=0,f=0,g=0;b=(c[a+60>>2]|0)+492|0;a=0;while(1){d=c[b+192+(a<<2)>>2]|0;if((d|0)!=0){NM(d+9568|0,0,4096);NM(d+59712|0,0,4096)}d=a+1|0;if((d|0)<16){a=d}else{e=0;break}}while(1){a=c[b+128+(e<<2)>>2]|0;if((a|0)!=0){NM(a+9568|0,0,4096);NM(a+59712|0,0,4096)}a=e+1|0;if((a|0)<16){e=a}else{f=0;break}}while(1){e=c[b+64+(f<<2)>>2]|0;if((e|0)!=0){NM(e+9568|0,0,4096);NM(e+59712|0,0,4096)}e=f+1|0;if((e|0)<16){f=e}else{g=0;break}}do{f=c[b+(g<<2)>>2]|0;if((f|0)!=0){NM(f+9568|0,0,4096);NM(f+59712|0,0,4096)}g=g+1|0;}while((g|0)<16);return}function pN(b,e,f,g,h,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;k=i;i=i+216|0;l=k|0;m=k+192|0;n=h>>>0>2147483639|(g|0)==0;o=n?0:g;p=n?0:h;c[m>>2]=o;c[m+12>>2]=p;q=p+8|0;r=m+16|0;c[r>>2]=q;c[m+4>>2]=o+(p+7>>3);p=m+8|0;c[p>>2]=0;if(n){s=n?-1094995529:0;i=k;return s|0}n=oc(f,g,h,j)|0;if((n|0)<0){s=-1094995529;i=k;return s|0}j=c[f+4>>2]|0;if((j|0)>12){AN(e,16,1623792,(t=i,i=i+8|0,c[t>>2]=j,t)|0);i=t;s=-1094995529;i=k;return s|0}j=(q|0)<(n|0)?q:n;c[p>>2]=j;n=f|0;h=c[n>>2]|0;if(!((h|0)==1|(h|0)==2|(h|0)==4)){AN(e,16,1656200,(t=i,i=i+16|0,c[t>>2]=(c[f+16>>2]|0)==1?1654416:3119040,c[t+8>>2]=h,t)|0);i=t;s=-38;i=k;return s|0}g=c[f+12>>2]|0;u=l|0;v=d[o+(j>>>3)|0]|0;w=((j|0)<(q|0))+j|0;c[p>>2]=w;if((v&128>>>((j&7)>>>0)|0)!=0){AS(e,1651112,(t=i,i=i+1|0,i=i+7&-8,c[t>>2]=0,t)|0);i=t;s=-1163346256;i=k;return s|0}j=((w|0)<(q|0))+w|0;if((d[o+(w>>>3)|0]&128>>>((w&7)>>>0)|0)==0){x=j}else{w=j+14|0;x=q>>>0>w>>>0?w:q}w=x&7;j=d[o+(x>>>3)|0]|0;o=((x|0)<(q|0))+x|0;c[p>>2]=o;if((h|0)==6|(h|0)==20){h=o+3|0;x=q>>>0>h>>>0?h:q;c[p>>2]=x;y=x}else{y=o}do{if((g|0)==0){o=y+4|0;c[p>>2]=q>>>0>o>>>0?o:q;o=l|0;x=pR(e,f,o,m)|0;if((x|0)<0){s=x}else{z=x;A=o;break}i=k;return s|0}else{o=g-1|0;if(o>>>0<=6){x=a[64504+g|0]|0;h=350944+(o*15|0)|0;o=x*3|0;NN(u|0,h|0,o)|0;z=x;A=l|0;break}AN(e,16,1659440,(t=i,i=i+8|0,c[t>>2]=g,t)|0);i=t;s=-1094995529;i=k;return s|0}}while(0);do{if((z|0)>0){t=0;g=0;do{e=a[l+(g*3|0)+2|0]|0;if(e<<24>>24==0){B=0}else{B=e<<24>>24!=5|0}t=(((a[l+(g*3|0)|0]|0)==1?2:1)&-B)+t|0;g=g+1|0;}while((g|0)<(z|0));if((t|0)<=1){C=797;break}c[f+40>>2]=0}else{C=797}}while(0);do{if((C|0)==797){if((c[f+16>>2]|0)!=1){break}B=f+40|0;if((c[B>>2]|0)!=-1){break}c[B>>2]=1}}while(0);do{if((b|0)==0){C=802}else{f=pS(b,A,z,3,0)|0;if((f|0)==0){C=802;break}if((f|0)<0){s=f}else{break}i=k;return s|0}}while(0);do{if((C|0)==802){if((j&128>>>(w>>>0)|0)==0){break}switch(c[n>>2]|0){case 22:{z=c[r>>2]|0;A=(c[p>>2]|0)+5|0;b=(z>>>0>A>>>0?A:z)+11|0;A=z>>>0>b>>>0?b:z;c[p>>2]=A;D=A;E=z;break};case 17:case 19:case 20:case 23:{z=c[r>>2]|0;A=(c[p>>2]|0)+3|0;b=z>>>0>A>>>0?A:z;c[p>>2]=b;D=b;E=z;break};default:{D=c[p>>2]|0;E=c[r>>2]|0}}z=D+1|0;c[p>>2]=E>>>0>z>>>0?z:E}}while(0);s=c[p>>2]|0;i=k;return s|0}function pO(f,h,j,k,l){f=f|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0.0,bG=0.0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0.0,bT=0,bU=0,bV=0.0,bW=0,bX=0,bY=0,bZ=0.0,b_=0.0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0,cb=0.0,cc=0,cd=0.0,ce=0,cf=0,cg=0,ch=0,ci=0,cj=0,ck=0,cl=0,cm=0,cn=0,co=0,cp=0,cq=0,cr=0,cs=0,cu=0,cv=0,cw=0,cx=0,cy=0,cz=0,cA=0,cB=0,cC=0,cD=0,cE=0,cF=0,cG=0,cH=0,cI=0,cJ=0,cK=0,cL=0,cM=0,cN=0,cO=0,cP=0,cQ=0,cR=0,cS=0,cT=0;m=i;i=i+880|0;n=m|0;o=m+256|0;p=m+264|0;q=m+272|0;r=m+464|0;s=m+488|0;t=m+680|0;u=m+872|0;v=c[f+60>>2]|0;w=v;x=v+8|0;c[x>>2]=h;z=k|0;A=k+8|0;B=c[A>>2]|0;C=(c[z>>2]|0)+(B>>>3)|0;L1109:do{if(((cV(d[C]|d[C+1|0]<<8|d[C+2|0]<<16|d[C+3|0]<<24|0)|0)<<(B&7)&-1048576|0)==-1048576){D=s|0;E=rs(k,r)|0;L1111:do{if((E|0)>0){F=v+6672|0;do{if((c[F>>2]|0)==0){if((a[r+16|0]|0)==1){break}AT(c[v+4>>2]|0,1658240,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;c[F>>2]=1}}while(0);F=v+6664|0;if((c[F>>2]|0)==4){H=v+6408|0;I=v+6144|0;NN(I|0,H|0,264)|0}c[F>>2]=0;F=r+15|0;H=a[F]|0;do{if(H<<24>>24==0){c[v+6420>>2]=0;if((c[v+5628>>2]|0)==0){break}if((c[v+6400>>2]|0)!=0){break}a[s+3|0]=0;a[D]=0;a[s+5|0]=1;a[s+2|0]=1;a[s+1|0]=0;a[s+4|0]=1;if((pS(w,s|0,2,2,0)|0)!=0){break L1111}}else{I=H&255;c[v+6420>>2]=I;J=I-1|0;if(J>>>0>6){AN(c[v+4>>2]|0,16,1659440,(G=i,i=i+8|0,c[G>>2]=I,G)|0);i=G;break L1111}else{K=s|0;L=a[64504+I|0]|0;I=350944+(J*15|0)|0;J=L*3|0;NN(D|0,I|0,J)|0;if((pS(w,K,L,2,0)|0)==0){break}else{break L1111}}}}while(0);H=c[r>>2]|0;c[v+6416>>2]=H;c[v+6412>>2]=d[r+14|0]|0;c[v+6408>>2]=d[r+13|0]|0;do{if((c[v+6400>>2]|0)==4){if((c[v+6156>>2]|0)!=(d[F]|0)){M=838;break}if((c[v+6152>>2]|0)!=(H|0)){M=838}}else{M=838}}while(0);if((M|0)==838){c[v+6424>>2]=-1;c[v+6448>>2]=-1}if((a[r+12|0]|0)!=0){M=841;break}H=c[k+16>>2]|0;F=(c[A>>2]|0)+16|0;c[A>>2]=H>>>0>F>>>0?F:H;M=841}else{M=841}}while(0);do{if((M|0)==841){if((E|0)<0){break}D=c[v+6412>>2]|0;if((D|0)<=12){M=845;break L1109}AN(c[v+4>>2]|0,16,1623792,(G=i,i=i+8|0,c[G>>2]=D,G)|0);i=G;N=-1;break L1109}}while(0);AN(f,16,1630080,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;N=-1}else{M=845}}while(0);L1141:do{if((M|0)==845){if((pP(f)|0)<0){N=-1;break}r=v+1004|0;c[r>>2]=0;B=k+16|0;C=s|0;E=q|0;D=v+6420|0;H=v+6664|0;F=v+6408|0;L=v+6144|0;K=v+4|0;J=s|0;I=v+6448|0;O=q|0;P=v+6424|0;Q=v+492|0;R=v+564|0;S=v+684|0;U=v+560|0;V=v+496|0;W=v+556|0;X=k+12|0;Y=F;Z=v+5356|0;_=v+5336|0;$=F;aa=t|0;ab=f;ad=v+6452|0;ae=v+6644|0;af=v+488|0;ag=v+412|0;ah=v+416|0;ai=v+16|0;aj=n|0;ak=0;al=0;am=0;an=0;ao=7;ap=0;aq=c[A>>2]|0;L1144:while(1){ar=c[B>>2]|0;as=c[z>>2]|0;at=as+(aq>>>3)|0;au=(cV(d[at]|d[at+1|0]<<8|d[at+2|0]<<16|d[at+3|0]<<24|0)|0)<<(aq&7);av=au>>>29;at=aq+3|0;aw=ar>>>0>at>>>0?at:ar;c[A>>2]=aw;if((av|0)==7){M=1054;break}at=as+(aw>>>3)|0;ax=(cV(d[at]|d[at+1|0]<<8|d[at+2|0]<<16|d[at+3|0]<<24|0)|0)<<(aw&7)>>>28;at=aw+4|0;c[A>>2]=ar>>>0>at>>>0?at:ar;if((au|0)>-1){au=c[D>>2]|0;L1149:do{if((au|0)==0){ay=c[w+748+(av<<6)+(ax<<2)>>2]|0}else{ar=c[r>>2]|0;at=(av|0)==1;if((ar|0)==0&at&(au|0)==1){if((c[H>>2]|0)==4){NN(L|0,F|0,264)|0}c[H>>2]=0;AN(c[K>>2]|0,48,1662112,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;a[C]=a[350959]|0;a[C+1|0]=a[350960]|0;a[C+2|0]=a[350961]|0;if((pS(w,J,1,2,1)|0)<0){M=879;break L1144}c[D>>2]=2;c[I>>2]=0;az=c[r>>2]|0;aA=2}else{az=ar;aA=au}ar=(av|0)==0;L1159:do{if((az|av|0)==0&(aA|0)==2){if((c[H>>2]|0)==4){NN(L|0,F|0,264)|0}c[H>>2]=0;AN(c[K>>2]|0,48,1660672,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;a[E]=a[350944]|0;a[E+1|0]=a[350945]|0;a[E+2|0]=a[350946]|0;if((pS(w,O,1,2,1)|0)<0){M=879;break L1144}c[D>>2]=1;if((c[P>>2]|0)==0){break}c[I>>2]=-1}else{L1167:do{switch(aA|0){case 3:case 2:{aB=aA;M=873;break};case 1:{break L1159;break};case 5:{aC=5;M=868;break};case 4:{aD=4;M=870;break};case 7:{if(!((az|0)==3&at)){M=865;break L1167}c[r>>2]=4;aw=c[R>>2]|0;c[w+812+(ax<<2)>>2]=aw;ay=aw;break L1149;break};case 6:{M=865;break};default:{M=879;break L1144}}}while(0);do{if((M|0)==865){M=0;aw=a[64504+aA|0]|0;if((az|0)!=(aw-1|0)){aC=aA;M=868;break}if(!((av|0)==3|(av|0)==0)){aC=aA;M=868;break}c[r>>2]=aw;aw=c[S>>2]|0;c[w+748+(av<<6)+(ax<<2)>>2]=aw;ay=aw;break L1149}}while(0);do{if((M|0)==868){M=0;if(!((az|0)==2&at)){aD=aC;M=870;break}c[r>>2]=3;aw=c[U>>2]|0;c[w+812+(ax<<2)>>2]=aw;ay=aw;break L1149}}while(0);do{if((M|0)==870){M=0;if((az|0)!=2){aB=aD;M=873;break}if(!((aD|0)==4&ar)){aE=aD;break}c[r>>2]=3;aw=c[V>>2]|0;c[w+748+(ax<<2)>>2]=aw;ay=aw;break L1149}}while(0);do{if((M|0)==873){M=0;if(!((az|0)==((aB|0)!=2|0)&at)){aE=aB;break}c[r>>2]=az+1;aw=c[W>>2]|0;c[w+812+(ax<<2)>>2]=aw;ay=aw;break L1149}}while(0);if((aE|0)==2){M=879;break L1144}}}while(0);at=c[r>>2]|0;if((at|av|0)!=0){M=879;break L1144}c[r>>2]=at+1;at=c[Q>>2]|0;c[w+748+(ax<<2)>>2]=at;ay=at}}while(0);if((ay|0)==0){M=879;break}else{aF=ay;aG=1024}}else{aF=al;aG=ap}L1189:do{switch(av|0){case 3:{aH=ak;aI=pQ(w,aF+160|0,k,0)|0;M=1051;break};case 4:{au=c[A>>2]|0;at=c[z>>2]|0;ar=d[at+(au>>>3)|0]|0;aw=c[B>>2]|0;as=((au|0)<(aw|0))+au|0;c[A>>2]=as;aJ=at+(as>>>3)|0;aK=(cV(d[aJ]|d[aJ+1|0]<<8|d[aJ+2|0]<<16|d[aJ+3|0]<<24|0)|0)<<(as&7)>>>24;aJ=as+8|0;as=aw>>>0>aJ>>>0?aJ:aw;c[A>>2]=as;if((aK|0)==255){aJ=at+(as>>>3)|0;at=(cV(d[aJ]|d[aJ+1|0]<<8|d[aJ+2|0]<<16|d[aJ+3|0]<<24|0)|0)<<(as&7)>>>24;aJ=as+8|0;aL=aw>>>0>aJ>>>0?aJ:aw;c[A>>2]=aL;aM=at+255|0;aN=aL}else{aM=aK;aN=as}do{if((ar&128>>>((au&7)>>>0)|0)==0){aO=aN}else{as=-aN&7;if((as|0)==0){aO=aN;break}aK=as+aN|0;as=aw>>>0>aK>>>0?aK:aw;c[A>>2]=as;aO=as}}while(0);au=aM<<3;if(((c[X>>2]|0)-aO|0)<(au|0)){M=978;break L1144}ar=-aO|0;as=aw-aO|0;if((au|0)<(ar|0)){aP=ar}else{aP=(as|0)<(au|0)?as:au}c[A>>2]=aP+aO;aQ=an;aR=am;aS=ak;break};case 5:{if((c[H>>2]|0)==4){NN(L|0,F|0,264)|0}c[H>>2]=0;au=pR(f,$,aa,k)|0;if((au|0)<0){N=au;break L1141}if((am|0)!=0){AN(ab,16,1613248,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;aQ=an;aR=am;aS=ak;break L1189}as=pS(w,aa,au,1,1)|0;if((as|0)!=0){N=as;break L1141}c[D>>2]=0;aQ=an;aR=1;aS=ak;break};case 6:{as=c[A>>2]|0;if((ax|0)==15){au=c[B>>2]|0;ar=(c[z>>2]|0)+(as>>>3)|0;aK=(cV(d[ar]|d[ar+1|0]<<8|d[ar+2|0]<<16|d[ar+3|0]<<24|0)|0)<<(as&7)>>>24;ar=as+8|0;aL=au>>>0>ar>>>0?ar:au;c[A>>2]=aL;aT=aK+14|0;aU=aL}else{aT=ax;aU=as}if(((c[X>>2]|0)-aU|0)<(aT<<3|0)){M=994;break L1144}if((aT|0)<=0){aQ=an;aR=am;aS=ak;break L1189}as=(al|0)==0;aL=al+108256|0;aK=aT;au=aU;while(1){ar=c[B>>2]|0;at=c[z>>2]|0;aJ=at+(au>>>3)|0;aV=(cV(d[aJ]|d[aJ+1|0]<<8|d[aJ+2|0]<<16|d[aJ+3|0]<<24|0)|0)<<(au&7)>>>28;aJ=au+4|0;aW=ar>>>0>aJ>>>0?aJ:ar;c[A>>2]=aW;do{if((aV|0)==0){aJ=aK<<3;aX=aJ-4|0;do{if((aX|0)<69){aY=aX}else{aZ=aW+13|0;a_=ar>>>0>aZ>>>0?aZ:ar;c[A>>2]=a_;aZ=aJ-17|0;if((aZ|0)>7){a$=~((aJ-25|0)>>>3);a0=a$>>>0>4294967041;a1=aZ;a2=0;a3=1;a4=a_;while(1){a_=at+(a4>>>3)|0;a5=(cV(d[a_]|d[a_+1|0]<<8|d[a_+2|0]<<16|d[a_+3|0]<<24|0)|0)<<(a4&7)>>>24;a_=a4+8|0;a6=ar>>>0>a_>>>0?a_:ar;c[A>>2]=a6;a[n+a2|0]=a5&255;a5=a1-8|0;a_=a3+1|0;if(a_>>>0<256&(a5|0)>7){a1=a5;a2=a3;a3=a_;a4=a6}else{break}}a4=a0?a$:-255;a7=(a4<<3)+aZ|0;a8=-a4|0}else{a7=aZ;a8=0}a[n+a8|0]=0;a4=c[K>>2]|0;if((c[a4+688>>2]&1|0)!=0){AN(a4,48,1714192,(G=i,i=i+8|0,c[G>>2]=aj,G)|0);i=G}a4=ct(aj|0,1711656,(G=i,i=i+16|0,c[G>>2]=o,c[G+8>>2]=p,G)|0)|0;i=G;if((a4|0)!=2){aY=a7;break}c[(c[(c[K>>2]|0)+64>>2]|0)+496>>2]=1024;aY=a7}}while(0);aJ=c[A>>2]|0;aX=-aJ|0;a4=(c[B>>2]|0)-aJ|0;if((aY|0)<(aX|0)){a9=aX}else{a9=(a4|0)<(aY|0)?a4:aY}c[A>>2]=a9+aJ;ba=aK}else if((aV|0)==14){bb=1;M=997}else if((aV|0)==13){bb=0;M=997}else if((aV|0)==11){aJ=d[at+(aW>>>3)|0]|0;a4=((aW|0)<(ar|0))+aW|0;c[A>>2]=a4;if((aJ&128>>>((aW&7)>>>0)|0)==0){bc=1;bd=a4;be=at;bf=ar}else{aJ=at+(a4>>>3)|0;aX=(cV(d[aJ]|d[aJ+1|0]<<8|d[aJ+2|0]<<16|d[aJ+3|0]<<24|0)|0)<<(a4&7)>>>28;aJ=a4+4|0;c[A>>2]=ar>>>0>aJ>>>0?aJ:ar;c[ai>>2]=aX;aX=c[B>>2]|0;aJ=(c[A>>2]|0)+4|0;a4=aX>>>0>aJ>>>0?aJ:aX;c[A>>2]=a4;bc=2;bd=a4;be=c[z>>2]|0;bf=aX}aX=d[be+(bd>>>3)|0]|0;a4=((bd|0)<(bf|0))+bd|0;c[A>>2]=a4;if((aX&128>>>((bd&7)>>>0)|0)==0){bg=bc;bh=a4;bi=be;bj=bf}else{a4=0;aX=0;L1248:while(1){L1250:do{if((a4|0)<57){aJ=aX;while(1){if((aJ|0)<7){bk=aJ;break L1250}a3=c[A>>2]|0;a2=c[z>>2]|0;a1=d[a2+(a3>>>3)|0]|0;a6=c[B>>2]|0;a_=((a3|0)<(a6|0))+a3|0;c[A>>2]=a_;if((a1&128>>>((a3&7)>>>0)|0)==0){bl=a_;bm=a2;bn=a6;break L1248}else{aJ=0}}}else{if((aX|0)<7){bk=aX}else{M=1021;break L1248}}}while(0);aZ=c[A>>2]|0;a$=d[(c[z>>2]|0)+(aZ>>>3)|0]<<(aZ&7)>>>7;c[A>>2]=((aZ|0)<(c[B>>2]|0))+aZ;c[w+156+(a4<<2)>>2]=a$&1;a4=a4+1|0;aX=bk+1|0}if((M|0)==1021){M=0;bl=c[A>>2]|0;bm=c[z>>2]|0;bn=c[B>>2]|0}bg=((a4|0)/7|0)+bc|0;bh=bl;bi=bm;bj=bn}aX=d[bi+(bh>>>3)|0]|0;a$=((bh|0)<(bj|0))+bh|0;c[A>>2]=a$;if((aX&128>>>((bh&7)>>>0)|0)==0){bo=1;bp=bg;bq=a$;br=bi;bs=bj}else{aX=bi+(a$>>>3)|0;aZ=(cV(d[aX]|d[aX+1|0]<<8|d[aX+2|0]<<16|d[aX+3|0]<<24|0)|0)<<(a$&7)>>>28;aX=a$+4|0;c[A>>2]=bj>>>0>aX>>>0?aX:bj;c[ag>>2]=aZ;aZ=c[A>>2]|0;aX=c[B>>2]|0;a$=(c[z>>2]|0)+(aZ>>>3)|0;a0=(cV(d[a$]|d[a$+1|0]<<8|d[a$+2|0]<<16|d[a$+3|0]<<24|0)|0)<<(aZ&7)>>>28;a$=aZ+4|0;c[A>>2]=aX>>>0>a$>>>0?a$:aX;c[ah>>2]=a0;a0=c[ag>>2]|0;aX=a0+1|0;a$=bg+1|0;if((aX|0)>0){aZ=0;do{aJ=c[A>>2]|0;a6=c[B>>2]|0;a2=(c[z>>2]|0)+(aJ>>>3)|0;a_=(cV(d[a2]|d[a2+1|0]<<8|d[a2+2|0]<<16|d[a2+3|0]<<24|0)|0)<<(aJ&7)>>>24;a2=aJ+8|0;c[A>>2]=a6>>>0>a2>>>0?a2:a6;c[w+420+(aZ<<2)>>2]=a_;aZ=aZ+1|0;}while((aZ|0)<(aX|0));bt=bg+2+a0|0}else{bt=a$}bo=aX;bp=bt;bq=c[A>>2]|0;br=c[z>>2]|0;bs=c[B>>2]|0}aZ=d[br+(bq>>>3)|0]|0;a4=((bq|0)<(bs|0))+bq|0;c[A>>2]=a4;if((aZ&128>>>((bq&7)>>>0)|0)==0){bu=bp;bv=a4;bw=bs}else{aZ=br+(a4>>>3)|0;a_=(cV(d[aZ]|d[aZ+1|0]<<8|d[aZ+2|0]<<16|d[aZ+3|0]<<24|0)|0)<<(a4&7)>>>25;aZ=a4+7|0;c[A>>2]=bs>>>0>aZ>>>0?aZ:bs;c[af>>2]=a_;a_=c[B>>2]|0;aZ=(c[A>>2]|0)+1|0;a4=a_>>>0>aZ>>>0?aZ:a_;c[A>>2]=a4;bu=bp+1|0;bv=a4;bw=a_}if((bo|0)>0){bx=0;by=bv;bz=bw}else{ba=bu;break}while(1){a_=d[(c[z>>2]|0)+(by>>>3)|0]<<(by&7)>>>7;c[A>>2]=((by|0)<(bz|0))+by;c[w+20+(bx<<2)>>2]=a_&1;a_=c[A>>2]|0;a4=c[B>>2]|0;aZ=(c[z>>2]|0)+(a_>>>3)|0;a6=(cV(d[aZ]|d[aZ+1|0]<<8|d[aZ+2|0]<<16|d[aZ+3|0]<<24|0)|0)<<(a_&7)>>>25;aZ=a_+7|0;c[A>>2]=a4>>>0>aZ>>>0?aZ:a4;c[w+88+(bx<<2)>>2]=a6;a6=bx+1|0;if((a6|0)>=(bo|0)){break}bx=a6;by=c[A>>2]|0;bz=c[B>>2]|0}ba=bu+bo|0}else{aX=(aK<<3)-4|0;a$=-aW|0;a0=ar-aW|0;if((aX|0)<(a$|0)){bA=a$}else{bA=(a0|0)<(aX|0)?a0:aX}c[A>>2]=bA+aW;ba=aK}}while(0);L1275:do{if((M|0)==997){M=0;if(as){AN(c[K>>2]|0,16,1726512,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;ba=aK;break}aW=c[P>>2]|0;if((aW|0)==0){AN(c[K>>2]|0,16,1722424,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;ar=(aK<<3)-4|0;at=c[A>>2]|0;aV=-at|0;aX=(c[B>>2]|0)-at|0;if((ar|0)<(aV|0)){bB=aV}else{bB=(aX|0)<(ar|0)?aX:ar}c[A>>2]=bB+at;ba=aK;break}else if((aW|0)==(-1|0)){M=1003}do{if((M|0)==1003){M=0;if((c[H>>2]|0)!=4){break}AN(c[K>>2]|0,16,1718240,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;aW=(aK<<3)-4|0;at=c[A>>2]|0;ar=-at|0;aX=(c[B>>2]|0)-at|0;if((aW|0)<(ar|0)){bC=ar}else{bC=(aX|0)<(aW|0)?aX:aW}c[A>>2]=bC+at;ba=aK;break L1275}}while(0);do{if((c[I>>2]|0)==-1){at=c[H>>2]|0;if(at>>>0>=4){M=1011;break}if((c[(c[K>>2]|0)+416>>2]|0)!=1){M=1011;break}c[P>>2]=1;c[I>>2]=1;aW=c[ae>>2]|0;pS(w,ad,aW,at,1)|0}else{M=1011}}while(0);if((M|0)==1011){M=0;c[P>>2]=1}ba=p2(w,aL,k,bb,aK,ao)|0}}while(0);at=aK-ba|0;if((at|0)<=0){aQ=an;aR=am;aS=ak;break L1189}aK=at;au=c[A>>2]|0}break};case 0:{aH=ak+1|0;aI=pQ(w,aF+160|0,k,0)|0;M=1051;break};case 2:{au=c[A>>2]|0;aK=d[(c[z>>2]|0)+(au>>>3)|0]<<(au&7)>>>7;c[A>>2]=((au|0)<(c[B>>2]|0))+au;au=aF+100448|0;c[au>>2]=aK<<1&2;aK=c[A>>2]|0;aL=c[B>>2]|0;as=(c[z>>2]|0)+(aK>>>3)|0;aw=(cV(d[as]|d[as+1|0]<<8|d[as+2|0]<<16|d[as+3|0]<<24|0)|0)<<(aK&7)>>>29;as=aK+3|0;c[A>>2]=aL>>>0>as>>>0?as:aL;aL=aF+100452|0;c[aL>>2]=aw;aw=0;as=0;while(1){aK=aw+1|0;at=c[A>>2]|0;aW=d[(c[z>>2]|0)+(at>>>3)|0]<<(at&7);c[A>>2]=((at|0)<(c[B>>2]|0))+at;at=aF+100456+(as<<2)|0;c[at>>2]=aW>>>7&1;aW=c[A>>2]|0;aX=c[B>>2]|0;ar=(c[z>>2]|0)+(aW>>>3)|0;aV=(cV(d[ar]|d[ar+1|0]<<8|d[ar+2|0]<<16|d[ar+3|0]<<24|0)|0)<<(aW&7)>>>28;ar=aW+4|0;c[A>>2]=aX>>>0>ar>>>0?ar:aX;c[aF+100488+(as<<2)>>2]=aV;if((c[at>>2]|0)==1){at=c[A>>2]|0;aV=c[B>>2]|0;aX=(c[z>>2]|0)+(at>>>3)|0;ar=(cV(d[aX]|d[aX+1|0]<<8|d[aX+2|0]<<16|d[aX+3|0]<<24|0)|0)<<(at&7)>>>30;aX=at+2|0;c[A>>2]=aV>>>0>aX>>>0?aX:aV;c[aF+100520+(as<<2)>>2]=ar;bD=(ar|0)==3?aw+2|0:aK}else{c[aF+100520+(as<<2)>>2]=2;bD=aK}aK=as+1|0;if((aK|0)>(c[aL>>2]|0)){break}else{aw=bD;as=aK}}as=aF+160|0;aw=c[A>>2]|0;aL=d[(c[z>>2]|0)+(aw>>>3)|0]|0;c[A>>2]=((aw|0)<(c[B>>2]|0))+aw;aK=c[au>>2]|0;if((aL&128>>>((aw&7)>>>0)|0)==0){bE=aK>>>0>1|0}else{bE=1}c[au>>2]=bE+aK;aK=c[A>>2]|0;aw=c[B>>2]|0;aL=c[z>>2]|0;ar=aL+(aK>>>3)|0;aV=cV(d[ar]|d[ar+1|0]<<8|d[ar+2|0]<<16|d[ar+3|0]<<24|0)|0;ar=aK+1|0;aX=aw>>>0>ar>>>0?ar:aw;c[A>>2]=aX;ar=aL+(aX>>>3)|0;aL=(cV(d[ar]|d[ar+1|0]<<8|d[ar+2|0]<<16|d[ar+3|0]<<24|0)|0)<<(aX&7)>>>30;ar=aX+2|0;c[A>>2]=aw>>>0>ar>>>0?ar:aw;bF=+g[304920+(aL<<2)>>2];aL=pQ(w,as,k,0)|0;if((aL|0)!=0){N=aL;break L1141}if((bD|0)<=0){aQ=an;aR=am;aS=ak;break L1189}aL=aF+176|0;aw=as|0;as=(aV&-2147483648>>>((aK&7)>>>0)|0)==0;aK=0;while(1){if((aK|0)==0){bG=1.0;bH=0;bI=1}else{aV=c[A>>2]|0;if((c[au>>2]|0)==3){bJ=1;bK=aV;bL=c[B>>2]|0;bM=c[z>>2]|0;M=947}else{ar=c[z>>2]|0;aX=d[ar+(aV>>>3)|0]<<(aV&7)>>>7;at=c[B>>2]|0;aW=((aV|0)<(at|0))+aV|0;c[A>>2]=aW;aV=aX&1;if((aV|0)==0){bN=0;bO=0}else{bJ=aV;bK=aW;bL=at;bM=ar;M=947}}if((M|0)==947){M=0;ar=c[499431]|0;at=bM+(bK>>>3)|0;aW=(cV(d[at]|d[at+1|0]<<8|d[at+2|0]<<16|d[at+3|0]<<24|0)|0)<<(bK&7)>>>25;at=b[ar+(aW<<2)>>1]|0;aV=b[ar+(aW<<2)+2>>1]|0;aW=aV<<16>>16;do{if(aV<<16>>16<0){aX=bK+7|0;a0=bL>>>0>aX>>>0?aX:bL;aX=bM+(a0>>>3)|0;a$=((cV(d[aX]|d[aX+1|0]<<8|d[aX+2|0]<<16|d[aX+3|0]<<24|0)|0)<<(a0&7)>>>((aW+32|0)>>>0))+at|0;aX=b[ar+(a$<<2)>>1]|0;a6=b[ar+(a$<<2)+2>>1]|0;a$=a6<<16>>16;if(a6<<16>>16>=0){bP=aX;bQ=a0;bR=a$;break}a6=a0-aW|0;a0=bL>>>0>a6>>>0?a6:bL;a6=bM+(a0>>>3)|0;a4=((cV(d[a6]|d[a6+1|0]<<8|d[a6+2|0]<<16|d[a6+3|0]<<24|0)|0)<<(a0&7)>>>((a$+32|0)>>>0))+aX|0;bP=b[ar+(a4<<2)>>1]|0;bQ=a0;bR=b[ar+(a4<<2)+2>>1]|0}else{bP=at;bQ=bK;bR=aW}}while(0);aW=bQ+bR|0;c[A>>2]=bL>>>0>aW>>>0?aW:bL;bN=bP-60|0;bO=bJ}bG=+T(+bF,+(+(-bN|0)));bH=bN;bI=bO}do{if((c[au>>2]|0)==3){g[aF+100552+(aK*480|0)>>2]=bG}else{aW=c[aL>>2]|0;if((aW|0)<=0){break}at=(bI|0)==0;ar=a[aw]|0;aV=0;a4=0;a0=bH;bS=bG;aX=ar;a$=aW;aW=ar;while(1){if(aX<<24>>24==0){bT=a4;bU=a0;bV=bS;bW=0;bX=a$;bY=aW}else{ar=0;a6=a4;aZ=a0;bZ=bS;a_=aW;while(1){if((c[aF+3332+(a6<<2)>>2]|0)==0){b_=bZ;b$=aZ;b0=a_}else{do{if(at){a2=c[499431]|0;aJ=c[A>>2]|0;a3=c[B>>2]|0;a1=c[z>>2]|0;a5=a1+(aJ>>>3)|0;b1=(cV(d[a5]|d[a5+1|0]<<8|d[a5+2|0]<<16|d[a5+3|0]<<24|0)|0)<<(aJ&7)>>>25;a5=b[a2+(b1<<2)>>1]|0;b2=b[a2+(b1<<2)+2>>1]|0;b1=b2<<16>>16;do{if(b2<<16>>16<0){b3=aJ+7|0;b4=a3>>>0>b3>>>0?b3:a3;b3=a1+(b4>>>3)|0;b5=((cV(d[b3]|d[b3+1|0]<<8|d[b3+2|0]<<16|d[b3+3|0]<<24|0)|0)<<(b4&7)>>>((b1+32|0)>>>0))+a5|0;b3=b[a2+(b5<<2)>>1]|0;b6=b[a2+(b5<<2)+2>>1]|0;b5=b6<<16>>16;if(b6<<16>>16>=0){b7=b3;b8=b4;b9=b5;break}b6=b4-b1|0;b4=a3>>>0>b6>>>0?b6:a3;b6=a1+(b4>>>3)|0;ca=((cV(d[b6]|d[b6+1|0]<<8|d[b6+2|0]<<16|d[b6+3|0]<<24|0)|0)<<(b4&7)>>>((b5+32|0)>>>0))+b3|0;b7=b[a2+(ca<<2)>>1]|0;b8=b4;b9=b[a2+(ca<<2)+2>>1]|0}else{b7=a5;b8=aJ;b9=b1}}while(0);b1=b8+b9|0;c[A>>2]=a3>>>0>b1>>>0?b1:a3;b1=b7-60|0;if((b1|0)==0){cb=bZ;cc=aZ;break}aJ=b1+aZ|0;if(as){cd=1.0;ce=aJ}else{cd=+(1-(aJ<<1&2)|0);ce=aJ>>1}cb=cd*+T(+bF,+(+(-ce|0)));cc=aJ}else{cb=bZ;cc=aZ}}while(0);g[aF+100552+(aK*480|0)+(a6<<2)>>2]=cb;b_=cb;b$=cc;b0=a[aw]|0}aJ=ar+1|0;cf=a6+1|0;if((aJ|0)<(b0&255|0)){ar=aJ;a6=cf;aZ=b$;bZ=b_;a_=b0}else{break}}bT=cf;bU=b$;bV=b_;bW=b0;bX=c[aL>>2]|0;bY=b0}a_=aV+1|0;if((a_|0)<(bX|0)){aV=a_;a4=bT;a0=bU;bS=bV;aX=bW;a$=bX;aW=bY}else{break}}}}while(0);aW=aK+1|0;if((aW|0)<(bD|0)){aK=aW}else{aQ=an;aR=am;aS=ak;break}}break};case 1:{aK=c[A>>2]|0;aL=d[(c[z>>2]|0)+(aK>>>3)|0]<<(aK&7)>>>7;c[A>>2]=((aK|0)<(c[B>>2]|0))+aK;aK=aL&1;aL=(aK|0)!=0;L1357:do{if(aL){aw=aF+160|0;if((pU(w,aw,k)|0)!=0){N=-1094995529;break L1141}as=a[aF+50316|0]|0;au=aF+50304|0;aW=aw|0;NN(au|0,aW|0,152)|0;a[aF+50317|0]=as;do{if((c[aF+50400>>2]|0)!=0){if((c[Y>>2]|0)==1){break}as=c[A>>2]|0;aW=c[B>>2]|0;aw=(c[z>>2]|0)+(as>>>3)|0;a$=(cV(d[aw]|d[aw+1|0]<<8|d[aw+2|0]<<16|d[aw+3|0]<<24|0)|0)<<(as&7)>>>31;aw=as+1|0;c[A>>2]=aW>>>0>aw>>>0?aw:aW;aW=a$&255;a[aF+50332|0]=aW;if(aW<<24>>24==0){break}aW=a[au]|0;a$=c[A>>2]|0;aw=c[B>>2]|0;as=(c[z>>2]|0)+(a$>>>3)|0;aX=(cV(d[as]|d[as+1|0]<<8|d[as+2|0]<<16|d[as+3|0]<<24|0)|0)<<(a$&7)>>>21;as=a$+11|0;c[A>>2]=aw>>>0>as>>>0?as:aw;b[aF+50334>>1]=aX&65535;aX=c[A>>2]|0;aw=c[B>>2]|0;as=(c[z>>2]|0)+(aX>>>3)|0;a$=(cV(d[as]|d[as+1|0]<<8|d[as+2|0]<<16|d[as+3|0]<<24|0)|0)<<(aX&7)>>>29;as=aX+3|0;c[A>>2]=aw>>>0>as>>>0?as:aw;g[aF+50336>>2]=+g[130728+(a$<<2)>>2];a$=(aW&255)>40?40:aW&255;if((a$|0)==0){break}else{cg=0}do{aW=c[A>>2]|0;aw=d[(c[z>>2]|0)+(aW>>>3)|0]<<(aW&7)>>>7;c[A>>2]=((aW|0)<(c[B>>2]|0))+aW;a[aF+50340+cg|0]=aw&1;cg=cg+1|0;}while((cg|0)<(a$|0))}}while(0);au=c[A>>2]|0;a$=c[B>>2]|0;aw=c[z>>2]|0;aW=aw+(au>>>3)|0;as=(cV(d[aW]|d[aW+1|0]<<8|d[aW+2|0]<<16|d[aW+3|0]<<24|0)|0)<<(au&7)>>>30;aW=au+2|0;au=a$>>>0>aW>>>0?aW:a$;c[A>>2]=au;if((as|0)==0){ch=0;break}else if((as|0)==3){M=890;break L1144}if((as|0)==2){NM(aF+8|0,1,ac(d[aF+160|0]|0,c[aF+176>>2]|0)|0);ch=2;break}else if((as|0)!=1){ch=as;break}as=aF+176|0;aW=aF+160|0;if((ac(d[aW]|0,c[as>>2]|0)|0)>0){ci=0;cj=au;ck=aw;cl=a$}else{ch=1;break}while(1){a$=d[ck+(cj>>>3)|0]<<(cj&7)>>>7;c[A>>2]=((cj|0)<(cl|0))+cj;a[aF+8+ci|0]=a$&1;a$=ci+1|0;if((a$|0)>=(ac(d[aW]|0,c[as>>2]|0)|0)){ch=1;break L1357}ci=a$;cj=c[A>>2]|0;ck=c[z>>2]|0;cl=c[B>>2]|0}}else{ch=0}}while(0);as=aF+160|0;aW=pQ(w,as,k,aK)|0;if((aW|0)!=0){N=aW;break L1141}aW=aF+50304|0;a$=pQ(w,aW,k,aK)|0;if((a$|0)!=0){N=a$;break L1141}do{if(aL){do{if((ch|0)!=0){a$=c[aF+236>>2]|0;aw=aF+176|0;au=c[aw>>2]|0;if((au|0)<=0){break}aX=aF+160|0;a0=a[aX]|0;a4=aF+55616|0;aV=0;at=0;a_=aF+5472|0;aZ=a0;a6=au;au=a0;while(1){a0=aF+180+aV|0;if(aZ<<24>>24==0){cm=at;cn=0;co=a6;cp=au}else{ar=0;aJ=at;b1=au;while(1){do{if((a[aF+8+aJ|0]|0)==0){cq=b1}else{if((c[aF+3332+(aJ<<2)>>2]|0)>>>0>=13){cq=b1;break}if((c[aF+53476+(aJ<<2)>>2]|0)>>>0>=13){cq=b1;break}if((a[a0]|0)==0){cq=b1;break}a5=a$+(ar<<1)|0;a2=a$+(ar+1<<1)|0;a1=0;do{b2=e[a5>>1]|0;ca=b2+(a1<<7)|0;dm[c[Z>>2]&127](a_+(ca<<2)|0,a4+(ca<<2)|0,(e[a2>>1]|0)-b2|0);a1=a1+1|0;}while((a1|0)<(d[a0]|0));cq=a[aX]|0}}while(0);a1=ar+1|0;cr=aJ+1|0;if((a1|0)<(cq&255|0)){ar=a1;aJ=cr;b1=cq}else{break}}cm=cr;cn=cq;co=c[aw>>2]|0;cp=cq}b1=d[a0]<<7;aJ=aV+1|0;if((aJ|0)<(co|0)){a4=a4+(b1<<2)|0;aV=aJ;at=cm;a_=a_+(b1<<2)|0;aZ=cn;a6=co;au=cp}else{break}}}}while(0);if((c[Y>>2]|0)!=1){break}pV(w,as);pV(w,aW)}}while(0);aW=c[aF+50380>>2]|0;as=aF+50320|0;aL=c[as>>2]|0;if((aL|0)<=0){aQ=1;aR=am;aS=ak;break L1189}aK=aF+50304|0;au=(ch|0)==0;a6=a[aK]|0;aZ=0;a_=0;at=aF+55616|0;aV=aF+5472|0;a4=a6;aw=aL;aL=a6;while(1){a6=aF+50324+a_|0;if(a4<<24>>24==0){cs=aZ;cu=0;cv=aw;cw=aL}else{aX=0;a$=aZ;b1=aL;while(1){aJ=c[aF+53476+(a$<<2)>>2]|0;ar=c[aF+53988+(a$<<2)>>2]|0;do{if((aJ-14|0)>>>0<2){if((aX|0)<(ar|0)){cx=aX;cy=a$;cz=aJ}else{cA=a$;cB=aX;cC=b1;break}while(1){a1=(cz<<1)-29|0;if(au){cD=a1}else{cD=ac(1-(d[aF+8+cy|0]<<1)|0,a1)|0}bF=+(cD|0)*+g[aF+54468+(cy<<2)>>2];if((a[a6]|0)==0){cE=cx+1|0}else{a1=aW+(cx<<1)|0;a2=cx+1|0;a5=aW+(a2<<1)|0;a3=0;while(1){b2=e[a1>>1]|0;ca=b2+(a3<<7)|0;cY[c[_>>2]&15](at+(ca<<2)|0,aV+(ca<<2)|0,bF,(e[a5>>1]|0)-b2|0);b2=a3+1|0;if((b2|0)<(d[a6]|0)){a3=b2}else{cE=a2;break}}}a2=cy+1|0;if((cE|0)>=(ar|0)){break}cx=cE;cy=a2;cz=c[aF+53476+(a2<<2)>>2]|0}cA=a$-aX+ar|0;cB=ar;cC=a[aK]|0}else{cA=a$-aX+ar|0;cB=ar;cC=b1}}while(0);if((cB|0)<(cC&255|0)){aX=cB;a$=cA;b1=cC}else{break}}cs=cA;cu=cC;cv=c[as>>2]|0;cw=cC}b1=d[a6]<<7;a$=a_+1|0;if((a$|0)<(cv|0)){aZ=cs;a_=a$;at=at+(b1<<2)|0;aV=aV+(b1<<2)|0;a4=cu;aw=cv;aL=cw}else{aQ=1;aR=am;aS=ak;break}}break};default:{N=-1;break L1141}}}while(0);if((M|0)==1051){M=0;if((aI|0)==0){aQ=1;aR=am;aS=aH}else{N=aI;break L1141}}aL=c[A>>2]|0;if(((c[X>>2]|0)-aL|0)<3){M=1053;break}else{ak=aS;al=aF;am=aR;an=aQ;ao=av;ap=aG;aq=aL}}if((M|0)==978){AN(c[K>>2]|0,16,1694320,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;N=-1094995529;break}else if((M|0)==879){AN(c[K>>2]|0,16,1617992,(G=i,i=i+16|0,c[G>>2]=av,c[G+8>>2]=ax,G)|0);i=G;N=-1;break}else if((M|0)==890){AN(c[K>>2]|0,16,1690736,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;N=-1094995529;break}else if((M|0)==1053){AN(ab,16,1735848,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;N=-1;break}else if((M|0)==1054){aq=v+6680|0;ao=v+6676|0;am=v+6692|0;al=v+6684|0;X=3;while(1){_=(X|0)<2;Z=(X|0)==1;B=(X|0)==2;ad=(X|0)<3;ae=0;do{I=c[w+492+(X<<6)+(ae<<2)>>2]|0;do{if((I|0)!=0){if(_){af=I+50304|0;ag=I+160|0;ah=0;do{ai=c[w+620+(ah<<2)>>2]|0;do{if((ai|0)!=0){if((c[ai+100448>>2]|0)!=0){break}aj=ai+100452|0;if((c[aj>>2]|0)<0){break}else{cF=0;cG=0}while(1){do{if((c[ai+100456+(cF<<2)>>2]|0)==(X|0)){if((c[ai+100488+(cF<<2)>>2]|0)!=(ae|0)){M=1067;break}D=ai+100520+(cF<<2)|0;if((c[D>>2]|0)==1){cH=cG}else{pT(w,ag,ai,cG);aa=c[D>>2]|0;D=((aa|0)!=0)+cG|0;if((aa|0)==2){cI=D;break}else{cH=D}}pT(w,af,ai,cH);cI=cH+1|0}else{M=1067}}while(0);if((M|0)==1067){M=0;cI=((c[ai+100520+(cF<<2)>>2]|0)==3?2:1)+cG|0}D=cF+1|0;if((D|0)>(c[aj>>2]|0)){break}else{cF=D;cG=cI}}}}while(0);ah=ah+1|0;}while((ah|0)<16)}do{if((c[Y>>2]|0)==4){if((c[I+256>>2]|0)==0){break}if((a[I+188|0]|0)!=0){c$[c[aq>>2]&127](w,I+160|0)}if(!((a[I+50332|0]|0)!=0&Z)){break}c$[c[aq>>2]&127](w,I+50304|0)}}while(0);ah=I+160|0;af=ah|0;ag=I+312|0;if((c[ag>>2]|0)!=0){dq[c[al>>2]&127](I+5472|0,ag,ah|0,1)}ah=I+50304|0;ag=I+50456|0;if((c[ag>>2]|0)!=0){dq[c[al>>2]&127](I+55616|0,ag,ah|0,1)}if(_){ag=I+160|0;a6=0;do{ai=c[w+620+(a6<<2)>>2]|0;do{if((ai|0)!=0){if((c[ai+100448>>2]|0)!=1){break}aj=ai+100452|0;if((c[aj>>2]|0)<0){break}else{cJ=0;cK=0}while(1){do{if((c[ai+100456+(cJ<<2)>>2]|0)==(X|0)){if((c[ai+100488+(cJ<<2)>>2]|0)!=(ae|0)){M=1090;break}D=ai+100520+(cJ<<2)|0;if((c[D>>2]|0)==1){cL=cK}else{pT(w,ag,ai,cK);aa=c[D>>2]|0;D=((aa|0)!=0)+cK|0;if((aa|0)==2){cM=D;break}else{cL=D}}pT(w,ah,ai,cL);cM=cL+1|0}else{M=1090}}while(0);if((M|0)==1090){M=0;cM=((c[ai+100520+(cJ<<2)>>2]|0)==3?2:1)+cK|0}D=cJ+1|0;if((D|0)>(c[aj>>2]|0)){break}else{cJ=D;cK=cM}}}}while(0);a6=a6+1|0;}while((a6|0)<16)}if(B){if((c[I+100448>>2]|0)==3){M=1095}}else{M=1095}if((M|0)==1095){M=0;c$[c[ao>>2]&127](w,af);if((c[Y>>2]|0)==4){c$[c[am>>2]&127](w,af)}do{if(Z){c$[c[ao>>2]&127](w,ah);if((c[Y>>2]|0)!=4){break}c$[c[am>>2]&127](w,ah)}}while(0);if((c[P>>2]|0)>0){p3(w,I+108256|0,X,c[I+50272>>2]|0,c[I+100416>>2]|0)}if(!ad){break}}ah=I+100416|0;af=I+50272|0;a6=0;do{ag=c[w+620+(a6<<2)>>2]|0;do{if((ag|0)!=0){if((c[ag+100448>>2]|0)!=3){break}ai=ag+100452|0;if((c[ai>>2]|0)<0){break}aj=ag+50272|0;D=0;aa=0;while(1){do{if((c[ag+100456+(D<<2)>>2]|0)==(X|0)){if((c[ag+100488+(D<<2)>>2]|0)!=(ae|0)){M=1117;break}$=ag+100520+(D<<2)|0;F=c[$>>2]|0;if((F|0)==1){cN=aa}else{bF=+g[ag+100552+(aa*480|0)>>2];L=c[aj>>2]|0;Q=c[af>>2]|0;r=1024<<((c[P>>2]|0)==1);if((r|0)>0){W=0;do{V=Q+(W<<2)|0;g[V>>2]=+g[V>>2]+bF*+g[L+(W<<2)>>2];W=W+1|0;}while((W|0)<(r|0));cO=c[$>>2]|0}else{cO=F}r=((cO|0)!=0)+aa|0;if((cO|0)==2){cP=r;break}else{cN=r}}r=cN+1|0;bF=+g[ag+100552+(cN*480|0)>>2];W=c[aj>>2]|0;L=c[ah>>2]|0;Q=1024<<((c[P>>2]|0)==1);if((Q|0)>0){cQ=0}else{cP=r;break}while(1){V=L+(cQ<<2)|0;g[V>>2]=+g[V>>2]+bF*+g[W+(cQ<<2)>>2];V=cQ+1|0;if((V|0)<(Q|0)){cQ=V}else{cP=r;break}}}else{M=1117}}while(0);if((M|0)==1117){M=0;cP=((c[ag+100520+(D<<2)>>2]|0)==3?2:1)+aa|0}r=D+1|0;if((r|0)>(c[ai>>2]|0)){break}else{D=r;aa=cP}}}}while(0);a6=a6+1|0;}while((a6|0)<16)}}while(0);ae=ae+1|0;}while((ae|0)<16);if((X|0)>0){X=X-1|0}else{break}}if((c[P>>2]|0)==1){cR=(c[v+6436>>2]|0)>(c[v+6416>>2]|0)|0}else{cR=0}X=ap<<cR;am=v+5628|0;if((c[am>>2]|0)!=0&(ak|0)==2){Y=v+6656|0;cS=(c[Y>>2]|0)==3&(c[Y+4>>2]|0)==0}else{cS=0}Y=(X|0)!=0;ao=c[x>>2]|0;if(Y){c[ao+76>>2]=X}else{Au(ao)}c[j>>2]=Y&1;do{if(cS){Y=c[am>>2]|0;if((Y|0)==1){c[h+4>>2]=c[h>>2];break}else if((Y|0)==2){c[h>>2]=c[h+4>>2];break}else{break}}}while(0);if(!((c[H>>2]|0)==0|(an|0)==0)){c[f+412>>2]=c[v+6416>>2]<<cR;c[f+424>>2]=X;c[H>>2]=4}if((cR|0)==0){cT=0;i=m;return cT|0}am=nL(l,70,u)|0;if(!((am|0)!=0&(c[u>>2]|0)>3)){cT=0;i=m;return cT|0}ak=am;y=(d[ak]|d[ak+1|0]<<8|d[ak+2|0]<<16|d[ak+3|0]<<24)<<1;a[ak]=y&255;y=y>>8;a[ak+1|0]=y&255;y=y>>8;a[ak+2|0]=y&255;y=y>>8;a[ak+3|0]=y&255;cT=0;i=m;return cT|0}else if((M|0)==994){AN(ab,16,1740312,(G=i,i=i+1|0,i=i+7&-8,c[G>>2]=0,G)|0);i=G;N=-1;break}}}while(0);G=v+6408|0;M=v+6664|0;if((c[M>>2]|0)==4){cT=N;i=m;return cT|0}if((c[v+6400>>2]|0)==0){cT=N;i=m;return cT|0}u=v+6144|0;NN(G|0,u|0,264)|0;u=v+4|0;c[(c[u>>2]|0)+416>>2]=c[v+6648>>2];G=v+6656|0;l=c[G+4>>2]|0;cR=(c[u>>2]|0)+448|0;c[cR>>2]=c[G>>2];c[cR+4>>2]=l;pS(w,v+6452|0,c[v+6644>>2]|0,c[M>>2]|0,0)|0;cT=N;i=m;return cT|0}function pP(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0;b=c[a+60>>2]|0;d=b+492|0;e=0;while(1){f=c[d+(e<<2)>>2]|0;if((f|0)!=0){c[f+50272>>2]=f+13664;c[f+100416>>2]=f+63808}f=e+1|0;if((f|0)<16){e=f}else{g=0;break}}while(1){e=c[d+64+(g<<2)>>2]|0;if((e|0)!=0){c[e+50272>>2]=e+13664;c[e+100416>>2]=e+63808}e=g+1|0;if((e|0)<16){g=e}else{h=0;break}}while(1){g=c[d+128+(h<<2)>>2]|0;if((g|0)!=0){c[g+50272>>2]=g+13664;c[g+100416>>2]=g+63808}g=h+1|0;if((g|0)<16){h=g}else{i=0;break}}do{h=c[d+192+(i<<2)>>2]|0;if((h|0)!=0){c[h+50272>>2]=h+13664;c[h+100416>>2]=h+63808}i=i+1|0;}while((i|0)<16);i=b+8|0;Au(c[i>>2]|0);c[(c[i>>2]|0)+76>>2]=2048;d=oX(a,c[i>>2]|0,0)|0;if((d|0)<0){j=d;return j|0}d=a+416|0;a=c[d>>2]|0;if((a|0)<=0){j=0;return j|0}h=b+5368|0;b=0;g=a;while(1){a=c[h+(b<<2)>>2]|0;if((a|0)==0){k=g}else{c[a+50112>>2]=c[(c[(c[i>>2]|0)+64>>2]|0)+(b<<2)>>2];k=c[d>>2]|0}a=b+1|0;if((a|0)<(k|0)){b=a;g=k}else{j=0;break}}return j|0}function pQ(f,h,j,l){f=f|0;h=h|0;j=j|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0.0,bw=0.0,bx=0.0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0.0,b7=0.0,b8=0.0;m=i;i=i+40|0;n=m|0;o=h+5312|0;p=n|0;c[p>>2]=0;q=j+8|0;r=c[q>>2]|0;s=j+16|0;t=c[s>>2]|0;u=j|0;v=(c[u>>2]|0)+(r>>>3)|0;w=(cV(d[v]|d[v+1|0]<<8|d[v+2|0]<<16|d[v+3|0]<<24|0)|0)<<(r&7)>>>24;v=r+8|0;c[q>>2]=t>>>0>v>>>0?v:t;t=(l|0)!=0;do{if(!t){if((pU(f,h|0,j)|0)<0){x=-1094995529}else{break}i=m;return x|0}}while(0);l=h+4|0;v=(c[l>>2]|0)==2?3:5;r=h+16|0;y=c[r>>2]|0;L1612:do{if((y|0)>0){z=h|0;A=32-v|0;B=j+12|0;C=(1<<v)-1|0;D=a[z]|0;E=0;F=0;G=D;H=y;I=D;L1615:while(1){if(G<<24>>24==0){J=E;K=0;L=H;M=I}else{D=E;N=0;while(1){O=c[q>>2]|0;P=c[s>>2]|0;Q=c[u>>2]|0;T=Q+(O>>>3)|0;U=(cV(d[T]|d[T+1|0]<<8|d[T+2|0]<<16|d[T+3|0]<<24|0)|0)<<(O&7)>>>28;T=O+4|0;O=P>>>0>T>>>0?T:P;c[q>>2]=O;if((U|0)==12){V=1181;break L1615}T=c[B>>2]|0;W=N&255;X=O;while(1){O=Q+(X>>>3)|0;Y=(cV(d[O]|d[O+1|0]<<8|d[O+2|0]<<16|d[O+3|0]<<24|0)|0)<<(X&7)>>>(A>>>0);O=X+v|0;Z=P>>>0>O>>>0?O:P;c[q>>2]=Z;O=Y+(W&255)|0;if((T-Z|0)<0){V=1183;break L1615}_=O&255;$=a[z]|0;aa=$&255;if(_>>>0>aa>>>0){V=1185;break L1615}if((Y|0)==(C|0)){W=O&255;X=Z}else{break}}if((N|0)<(_|0)){X=D;W=N;while(1){c[h+3172+(X<<2)>>2]=U;c[h+3684+(X<<2)>>2]=_;T=W+1|0;if((T|0)<(_|0)){X=X+1|0;W=T}else{break}}ab=D-N+_|0;ad=_;ae=a[z]|0}else{ab=D;ad=N;ae=$}if((ad|0)<(ae&255|0)){D=ab;N=ad}else{break}}J=ab;K=ae;L=c[r>>2]|0;M=ae}N=F+1|0;if((N|0)<(L|0)){E=J;F=N;G=K;H=L;I=M}else{V=1191;break}}if((V|0)==1181){AN(c[f+4>>2]|0,16,1666424,(af=i,i=i+1|0,i=i+7&-8,c[af>>2]=0,af)|0);i=af;x=-1094995529;i=m;return x|0}else if((V|0)==1183){AN(c[f+4>>2]|0,16,1665032,(af=i,i=i+1|0,i=i+7&-8,c[af>>2]=0,af)|0);i=af;x=-1094995529;i=m;return x|0}else if((V|0)==1185){AN(c[f+4>>2]|0,16,1663592,(af=i,i=i+16|0,c[af>>2]=_,c[af+8>>2]=aa,af)|0);i=af;x=-1094995529;i=m;return x|0}else if((V|0)==1191){I=h+4164|0;if((L|0)<=0){ag=I;break}H=f+4|0;G=0;F=0;E=0;C=w-90|0;A=w;B=1;N=M;D=M;L1642:while(1){L1644:do{if(N<<24>>24==0){ah=F;ai=A;aj=C;ak=B;al=E;am=0;an=D}else{W=0;X=F;U=E;T=C;P=A;Q=B;Z=D;while(1){O=W;Y=X;ao=T;ap=P;aq=Q;ar=Z;L1647:while(1){as=O;at=Y;au=ap;av=ar;L1649:while(1){aw=as;ax=at;ay=av;while(1){az=c[h+3684+(ax<<2)>>2]|0;aA=c[h+3172+(ax<<2)>>2]|0;if((aA|0)==15|(aA|0)==14){break L1647}else if((aA|0)==13){break L1649}else if((aA|0)!=0){break}if((aw|0)<(az|0)){NM(h+4164+(ax<<2)|0,0,az-aw<<2|0);aB=az;aC=ax-aw+az|0;aD=a[z]|0}else{aB=aw;aC=ax;aD=ay}if((aB|0)<(aD&255|0)){aw=aB;ax=aC;ay=aD}else{ah=aC;ai=au;aj=ao;ak=aq;al=U;am=aD;an=aD;break L1644}}if((aw|0)<(az|0)){aA=aw;aE=ax;aF=au;do{aG=c[499431]|0;aH=c[q>>2]|0;aI=c[s>>2]|0;aJ=c[u>>2]|0;aK=aJ+(aH>>>3)|0;aL=(cV(d[aK]|d[aK+1|0]<<8|d[aK+2|0]<<16|d[aK+3|0]<<24|0)|0)<<(aH&7)>>>25;aK=b[aG+(aL<<2)>>1]|0;aM=b[aG+(aL<<2)+2>>1]|0;aL=aM<<16>>16;do{if(aM<<16>>16<0){aN=aH+7|0;aO=aI>>>0>aN>>>0?aN:aI;aN=aJ+(aO>>>3)|0;aP=((cV(d[aN]|d[aN+1|0]<<8|d[aN+2|0]<<16|d[aN+3|0]<<24|0)|0)<<(aO&7)>>>((aL+32|0)>>>0))+aK|0;aN=b[aG+(aP<<2)>>1]|0;aQ=b[aG+(aP<<2)+2>>1]|0;aP=aQ<<16>>16;if(aQ<<16>>16>=0){aR=aN;aS=aO;aT=aP;break}aQ=aO-aL|0;aO=aI>>>0>aQ>>>0?aQ:aI;aQ=aJ+(aO>>>3)|0;aU=((cV(d[aQ]|d[aQ+1|0]<<8|d[aQ+2|0]<<16|d[aQ+3|0]<<24|0)|0)<<(aO&7)>>>((aP+32|0)>>>0))+aN|0;aR=b[aG+(aU<<2)>>1]|0;aS=aO;aT=b[aG+(aU<<2)+2>>1]|0}else{aR=aK;aS=aH;aT=aL}}while(0);aL=aS+aT|0;c[q>>2]=aI>>>0>aL>>>0?aL:aI;aF=aF-60+aR|0;if(aF>>>0>255){break L1642}g[h+4164+(aE<<2)>>2]=-0.0- +g[2900408+(aF+100<<2)>>2];aA=aA+1|0;aE=aE+1|0;}while((aA|0)<(az|0));aV=aA;aW=aE;aX=aF;aY=a[z]|0}else{aV=aw;aW=ax;aX=au;aY=ay}if((aV|0)<(aY&255|0)){as=aV;at=aW;au=aX;av=aY}else{ah=aW;ai=aX;aj=ao;ak=aq;al=U;am=aY;an=aY;break L1644}}if((aw|0)<(az|0)){av=aw;at=ax;as=ao;aL=aq;while(1){aH=aL-1|0;if((aL|0)>0){aK=c[q>>2]|0;aG=c[s>>2]|0;aJ=(c[u>>2]|0)+(aK>>>3)|0;aM=(cV(d[aJ]|d[aJ+1|0]<<8|d[aJ+2|0]<<16|d[aJ+3|0]<<24|0)|0)<<(aK&7)>>>23;aJ=aK+9|0;c[q>>2]=aG>>>0>aJ>>>0?aJ:aG;aZ=aM-256|0}else{aM=c[499431]|0;aG=c[q>>2]|0;aJ=c[s>>2]|0;aK=c[u>>2]|0;aU=aK+(aG>>>3)|0;aO=(cV(d[aU]|d[aU+1|0]<<8|d[aU+2|0]<<16|d[aU+3|0]<<24|0)|0)<<(aG&7)>>>25;aU=b[aM+(aO<<2)>>1]|0;aN=b[aM+(aO<<2)+2>>1]|0;aO=aN<<16>>16;do{if(aN<<16>>16<0){aP=aG+7|0;aQ=aJ>>>0>aP>>>0?aP:aJ;aP=aK+(aQ>>>3)|0;a$=((cV(d[aP]|d[aP+1|0]<<8|d[aP+2|0]<<16|d[aP+3|0]<<24|0)|0)<<(aQ&7)>>>((aO+32|0)>>>0))+aU|0;aP=b[aM+(a$<<2)>>1]|0;a0=b[aM+(a$<<2)+2>>1]|0;a$=a0<<16>>16;if(a0<<16>>16>=0){a1=aP;a2=aQ;a3=a$;break}a0=aQ-aO|0;aQ=aJ>>>0>a0>>>0?a0:aJ;a0=aK+(aQ>>>3)|0;a4=((cV(d[a0]|d[a0+1|0]<<8|d[a0+2|0]<<16|d[a0+3|0]<<24|0)|0)<<(aQ&7)>>>((a$+32|0)>>>0))+aP|0;a1=b[aM+(a4<<2)>>1]|0;a2=aQ;a3=b[aM+(a4<<2)+2>>1]|0}else{a1=aU;a2=aG;a3=aO}}while(0);aO=a2+a3|0;c[q>>2]=aJ>>>0>aO>>>0?aO:aJ;aZ=a1-60|0}a5=aZ+as|0;if((a5|0)<-100){a6=-100}else{a6=(a5|0)>155?155:a5}if((a5|0)!=(a6|0)){AS(c[H>>2]|0,1669184,(af=i,i=i+16|0,c[af>>2]=a5,c[af+8>>2]=a6,af)|0);i=af}g[h+4164+(at<<2)>>2]=-0.0- +g[2900408+(a6+200<<2)>>2];aO=av+1|0;if((aO|0)<(az|0)){av=aO;at=at+1|0;as=a5;aL=aH}else{break}}a7=az;a8=ax-aw+az|0;a9=a5;ba=aw+aq-az|0;bb=a[z]|0}else{a7=aw;a8=ax;a9=ao;ba=aq;bb=ay}if((a7|0)<(bb&255|0)){O=a7;Y=a8;ao=a9;ap=au;aq=ba;ar=bb}else{ah=a8;ai=au;aj=a9;ak=ba;al=U;am=bb;an=bb;break L1644}}if((aw|0)<(az|0)){ar=aw;ap=ax;Y=U;while(1){O=c[499431]|0;aL=c[q>>2]|0;as=c[s>>2]|0;at=c[u>>2]|0;av=at+(aL>>>3)|0;aO=(cV(d[av]|d[av+1|0]<<8|d[av+2|0]<<16|d[av+3|0]<<24|0)|0)<<(aL&7)>>>25;av=b[O+(aO<<2)>>1]|0;aG=b[O+(aO<<2)+2>>1]|0;aO=aG<<16>>16;do{if(aG<<16>>16<0){aU=aL+7|0;aM=as>>>0>aU>>>0?aU:as;aU=at+(aM>>>3)|0;aK=((cV(d[aU]|d[aU+1|0]<<8|d[aU+2|0]<<16|d[aU+3|0]<<24|0)|0)<<(aM&7)>>>((aO+32|0)>>>0))+av|0;aU=b[O+(aK<<2)>>1]|0;aN=b[O+(aK<<2)+2>>1]|0;aK=aN<<16>>16;if(aN<<16>>16>=0){bc=aK;bd=aM;be=aU;break}aN=aM-aO|0;aM=as>>>0>aN>>>0?aN:as;aN=at+(aM>>>3)|0;aE=((cV(d[aN]|d[aN+1|0]<<8|d[aN+2|0]<<16|d[aN+3|0]<<24|0)|0)<<(aM&7)>>>((aK+32|0)>>>0))+aU|0;bc=b[O+(aE<<2)+2>>1]|0;bd=aM;be=b[O+(aE<<2)>>1]|0}else{bc=aO;bd=aL;be=av}}while(0);av=bc+bd|0;c[q>>2]=as>>>0>av>>>0?av:as;bf=Y-60+be|0;if((bf|0)<-155){bg=-155}else{bg=(bf|0)>100?100:bf}if((bf|0)!=(bg|0)){AS(c[H>>2]|0,1670912,(af=i,i=i+16|0,c[af>>2]=bf,c[af+8>>2]=bg,af)|0);i=af}g[h+4164+(ap<<2)>>2]=+g[2900408+(200-bg<<2)>>2];av=ar+1|0;if((av|0)>=(az|0)){break}ar=av;ap=ap+1|0;Y=bf}bh=az;bi=ax-aw+az|0;bj=bf;bk=a[z]|0}else{bh=aw;bi=ax;bj=U;bk=ay}if((bh|0)<(bk&255|0)){W=bh;X=bi;U=bj;T=ao;P=au;Q=aq;Z=bk}else{ah=bi;ai=au;aj=ao;ak=aq;al=bj;am=bk;an=bk;break}}}}while(0);Z=G+1|0;if((Z|0)<(c[r>>2]|0)){G=Z;F=ah;E=al;C=aj;A=ai;B=ak;N=am;D=an}else{ag=I;break L1612}}AN(c[H>>2]|0,16,1667720,(af=i,i=i+8|0,c[af>>2]=aF,af)|0);i=af;x=-1094995529;i=m;return x|0}}else{ag=h+4164|0}}while(0);aF=c[q>>2]|0;an=c[u>>2]|0;am=d[an+(aF>>>3)|0]|0;ak=c[s>>2]|0;ai=((aF|0)<(ak|0))+aF|0;c[q>>2]=ai;aj=(am&128>>>((aF&7)>>>0)|0)==0;L1709:do{if(aj){bl=ai}else{if((c[l>>2]|0)==2){AN(c[f+4>>2]|0,16,1680408,(af=i,i=i+1|0,i=i+7&-8,c[af>>2]=0,af)|0);i=af;x=-1094995529;i=m;return x|0}aF=c[h+76>>2]|0;am=c[h+84>>2]|0;al=an+(ai>>>3)|0;ah=(cV(d[al]|d[al+1|0]<<8|d[al+2|0]<<16|d[al+3|0]<<24|0)|0)<<(ai&7)>>>30;al=ai+2|0;bk=ak>>>0>al>>>0?al:ak;c[q>>2]=bk;al=ah+1|0;c[p>>2]=al;ah=an+(bk>>>3)|0;bj=(cV(d[ah]|d[ah+1|0]<<8|d[ah+2|0]<<16|d[ah+3|0]<<24|0)|0)<<(bk&7)>>>26;ah=bk+6|0;bk=ak>>>0>ah>>>0?ah:ak;c[q>>2]=bk;L1715:do{if((bj|0)<(am|0)){ah=e[aF+(bj<<1)>>1]|0;au=an+(bk>>>3)|0;bi=(cV(d[au]|d[au+1|0]<<8|d[au+2|0]<<16|d[au+3|0]<<24|0)|0)<<(bk&7)>>>27;au=bk+5|0;bh=ak>>>0>au>>>0?au:ak;c[q>>2]=bh;au=bi+ah|0;c[n+8>>2]=au;if((au|0)>1023){break}ah=an+(bh>>>3)|0;bi=(cV(d[ah]|d[ah+1|0]<<8|d[ah+2|0]<<16|d[ah+3|0]<<24|0)|0)<<(bh&7)>>>28;ah=bh+4|0;bh=ak>>>0>ah>>>0?ah:ak;c[q>>2]=bh;c[n+24>>2]=bi;if(al>>>0>1){bm=1;bn=bh;bo=au}else{bl=bh;break L1709}while(1){bh=an+(bn>>>3)|0;au=(cV(d[bh]|d[bh+1|0]<<8|d[bh+2|0]<<16|d[bh+3|0]<<24|0)|0)<<(bn&7)>>>27;bh=bn+5|0;bi=ak>>>0>bh>>>0?bh:ak;c[q>>2]=bi;bh=au+bo|0;c[n+8+(bm<<2)>>2]=bh;if((bh|0)>1023){break L1715}au=an+(bi>>>3)|0;ah=(cV(d[au]|d[au+1|0]<<8|d[au+2|0]<<16|d[au+3|0]<<24|0)|0)<<(bi&7)>>>28;au=bi+4|0;bi=ak>>>0>au>>>0?au:ak;c[q>>2]=bi;c[n+24+(bm<<2)>>2]=ah;ah=bm+1|0;if((ah|0)<(c[p>>2]|0)){bm=ah;bn=bi;bo=bh}else{bl=bi;break L1709}}}}while(0);AN(c[f+4>>2]|0,16,1678552,(af=i,i=i+1|0,i=i+7&-8,c[af>>2]=0,af)|0);i=af;x=-1094995529;i=m;return x|0}}while(0);bo=d[an+(bl>>>3)|0]<<(bl&7)>>>7;c[q>>2]=((bl|0)<(ak|0))+bl;bl=bo&1;c[h+152>>2]=bl;L1724:do{if((bl|0)!=0){bo=(c[l>>2]|0)==2;ak=bo&1;if(bo){bp=7}else{bp=(c[f+6408>>2]|0)==1?20:12}bo=h+88|0;if((c[bo>>2]|0)<=0){break}an=2-ak|0;bn=32-an|0;bm=ak<<1;ak=6-bm|0;ai=32-ak|0;al=5-bm|0;bm=32-al|0;bk=0;L1730:while(1){bj=c[q>>2]|0;aF=c[s>>2]|0;am=(c[u>>2]|0)+(bj>>>3)|0;H=(cV(d[am]|d[am+1|0]<<8|d[am+2|0]<<16|d[am+3|0]<<24|0)|0)<<(bj&7)>>>(bn>>>0);am=bj+an|0;c[q>>2]=aF>>>0>am>>>0?am:aF;aF=h+156+(bk<<2)|0;c[aF>>2]=H;L1732:do{if((H|0)!=0){am=c[q>>2]|0;bj=c[u>>2]|0;bi=d[bj+(am>>>3)|0]<<(am&7)>>>7;bh=c[s>>2]|0;ah=((am|0)<(bh|0))+am|0;c[q>>2]=ah;am=bi&1;if((c[aF>>2]|0)<=0){break}bi=am+3|0;au=0;ay=ah;ah=bh;bh=bj;while(1){bj=bh+(ay>>>3)|0;ax=(cV(d[bj]|d[bj+1|0]<<8|d[bj+2|0]<<16|d[bj+3|0]<<24|0)|0)<<(ay&7)>>>(ai>>>0);bj=ay+ak|0;c[q>>2]=ah>>>0>bj>>>0?bj:ah;c[h+188+(bk<<4)+(au<<2)>>2]=ax;ax=c[q>>2]|0;bj=c[s>>2]|0;aw=(c[u>>2]|0)+(ax>>>3)|0;bq=(cV(d[aw]|d[aw+1|0]<<8|d[aw+2|0]<<16|d[aw+3|0]<<24|0)|0)<<(ax&7)>>>(bm>>>0);aw=ax+al|0;c[q>>2]=bj>>>0>aw>>>0?aw:bj;br=h+444+(bk<<4)+(au<<2)|0;c[br>>2]=bq;if((bq|0)>(bp|0)){break L1730}L1738:do{if((bq|0)!=0){bj=c[q>>2]|0;aw=d[(c[u>>2]|0)+(bj>>>3)|0]<<(bj&7)>>>7;c[q>>2]=((bj|0)<(c[s>>2]|0))+bj;c[h+316+(bk<<4)+(au<<2)>>2]=aw&1;aw=c[q>>2]|0;bj=c[u>>2]|0;ax=d[bj+(aw>>>3)|0]<<(aw&7)>>>7;bf=c[s>>2]|0;az=((aw|0)<(bf|0))+aw|0;c[q>>2]=az;aw=ax&1;ax=bi-aw|0;if((c[br>>2]|0)<=0){break}bg=32-ax|0;be=c[64488+((aw<<1|am)<<2)>>2]|0;aw=0;bd=az;az=bf;bf=bj;while(1){bj=bf+(bd>>>3)|0;bc=(cV(d[bj]|d[bj+1|0]<<8|d[bj+2|0]<<16|d[bj+3|0]<<24|0)|0)<<(bd&7)>>>(bg>>>0);bj=bd+ax|0;c[q>>2]=az>>>0>bj>>>0?bj:az;g[h+572+(bk*320|0)+(au*80|0)+(aw<<2)>>2]=+g[be+(bc<<2)>>2];bc=aw+1|0;if((bc|0)>=(c[br>>2]|0)){break L1738}aw=bc;bd=c[q>>2]|0;az=c[s>>2]|0;bf=c[u>>2]|0}}}while(0);aq=au+1|0;if((aq|0)>=(c[aF>>2]|0)){break L1732}au=aq;ay=c[q>>2]|0;ah=c[s>>2]|0;bh=c[u>>2]|0}}}while(0);bk=bk+1|0;if((bk|0)>=(c[bo>>2]|0)){break L1724}}AN(c[f+4>>2]|0,16,1673e3,(af=i,i=i+16|0,c[af>>2]=bq,c[af+8>>2]=bp,af)|0);i=af;c[br>>2]=0;x=-1094995529;i=m;return x|0}}while(0);br=c[q>>2]|0;bp=d[(c[u>>2]|0)+(br>>>3)|0]|0;c[q>>2]=((br|0)<(c[s>>2]|0))+br;if((bp&128>>>((br&7)>>>0)|0)!=0){AS(c[f+4>>2]|0,1676480,(af=i,i=i+1|0,i=i+7&-8,c[af>>2]=0,af)|0);i=af;x=-1163346256;i=m;return x|0}br=h+88|0;bp=c[br>>2]|0;bq=1024/(bp|0)|0;l=c[h+76>>2]|0;if((bp|0)>0){bp=h|0;bl=0;do{bo=e[l+(d[bp]<<1)>>1]|0;NM(h+5312+(bo+(bl<<7)<<2)|0,0,bq-bo<<2|0);bl=bl+1|0;}while((bl|0)<(c[br>>2]|0))}br=c[r>>2]|0;L1759:do{if((br|0)>0){bl=h|0;bq=f+5336|0;bp=f+5360|0;bo=f+5364|0;bk=0;al=0;bm=o;ak=a[bl]|0;ai=br;L1761:while(1){an=a[h+20+al|0]|0;bn=an&255;if(ak<<24>>24==0){bs=bk;bt=0;bu=ai}else{aF=an<<24>>24==0;an=bk;H=0;do{bh=(c[h+3172+(an<<2)>>2]|0)-1|0;ah=e[l+(H<<1)>>1]|0;ay=bm+(ah<<2)|0;H=H+1|0;au=(e[l+(H<<1)>>1]|0)-ah|0;L1767:do{if(bh>>>0>12){if(aF){break}ah=au<<2;am=0;bi=ay;while(1){NM(bi|0,0,ah|0);aq=am+1|0;if(aq>>>0<bn>>>0){am=aq;bi=bi+512|0}else{break}}}else{if((bh|0)==12){if(aF){break}bi=(au|0)>0;am=ag+(an<<2)|0;ah=0;aq=ay;while(1){if(bi){ao=0;do{bf=(ac(c[bo>>2]|0,1664525)|0)+1013904223|0;c[bo>>2]=bf;g[aq+(ao<<2)>>2]=+(bf|0);ao=ao+1|0;}while((ao|0)<(au|0))}bv=+da[c[bp>>2]&3](aq,aq,au);bw=+g[am>>2];bx=bw/+S(+bv);cY[c[bq>>2]&15](aq,aq,bx,au);ao=ah+1|0;if(ao>>>0<bn>>>0){ah=ao;aq=aq+512|0}else{break L1767}}}aq=c[257512+(bh<<2)>>2]|0;ah=c[257560+(bh<<2)>>2]|0;am=c[1997548+(bh<<4)>>2]|0;bi=c[q>>2]|0;ao=c[s>>2]|0;L1783:do{switch(bh>>>1|0){case 0:{if(aF){by=bi;break L1783}bf=ag+(an<<2)|0;az=bi;bd=0;aw=ay;while(1){be=az;ax=aw;bg=au;while(1){as=c[u>>2]|0;bc=as+(be>>>3)|0;bj=(cV(d[bc]|d[bc+1|0]<<8|d[bc+2|0]<<16|d[bc+3|0]<<24|0)|0)<<(be&7)>>>24;bc=b[am+(bj<<2)>>1]|0;bb=b[am+(bj<<2)+2>>1]|0;bj=bb<<16>>16;if(bb<<16>>16<0){bb=be+8|0;ba=ao>>>0>bb>>>0?bb:ao;bb=as+(ba>>>3)|0;as=((cV(d[bb]|d[bb+1|0]<<8|d[bb+2|0]<<16|d[bb+3|0]<<24|0)|0)<<(ba&7)>>>((bj+32|0)>>>0))+bc|0;bz=ba;bA=b[am+(as<<2)>>1]|0;bB=b[am+(as<<2)+2>>1]|0}else{bz=be;bA=bc;bB=bj}bj=bz+bB|0;bC=ao>>>0>bj>>>0?bj:ao;bj=e[ah+(bA<<1)>>1]|0;bx=+g[bf>>2];g[ax>>2]=bx*+g[aq+((bj&3)<<2)>>2];g[ax+4>>2]=bx*+g[aq+((bj>>>2&3)<<2)>>2];g[ax+8>>2]=bx*+g[aq+((bj>>>4&3)<<2)>>2];g[ax+12>>2]=bx*+g[aq+((bj>>>6&3)<<2)>>2];bj=bg-4|0;if((bj|0)==0){break}be=bC;ax=ax+16|0;bg=bj}bg=bd+1|0;if(bg>>>0<bn>>>0){az=bC;bd=bg;aw=aw+512|0}else{by=bC;break}}break};case 1:{if(aF){by=bi;break L1783}aw=ag+(an<<2)|0;bd=bi;az=0;bf=ay;while(1){bg=bd;ax=au;be=bf;while(1){aH=c[u>>2]|0;aJ=aH+(bg>>>3)|0;bj=(cV(d[aJ]|d[aJ+1|0]<<8|d[aJ+2|0]<<16|d[aJ+3|0]<<24|0)|0)<<(bg&7);aJ=bj>>>24;bc=b[am+(aJ<<2)>>1]|0;as=b[am+(aJ<<2)+2>>1]|0;aJ=as<<16>>16;if(as<<16>>16<0){as=bg+8|0;ba=ao>>>0>as>>>0?as:ao;as=aH+(ba>>>3)|0;aH=(cV(d[as]|d[as+1|0]<<8|d[as+2|0]<<16|d[as+3|0]<<24|0)|0)<<(ba&7);as=(aH>>>((aJ+32|0)>>>0))+bc|0;bD=ba;bE=aH;bF=b[am+(as<<2)+2>>1]|0;bG=b[am+(as<<2)>>1]|0}else{bD=bg;bE=bj;bF=aJ;bG=bc}bc=bD+bF|0;aJ=e[ah+(bG<<1)>>1]|0;bj=aJ>>>8&15;as=(bj|0)!=0?bE<<bF:0;aH=bj+(ao>>>0>bc>>>0?bc:ao)|0;bH=ao>>>0>aH>>>0?aH:ao;aH=(g[k>>2]=+g[aw>>2],c[k>>2]|0);g[be>>2]=+g[aq+((aJ&3)<<2)>>2]*(c[k>>2]=as&-2147483648^aH,+g[k>>2]);bc=as<<(aJ>>>12&1);g[be+4>>2]=+g[aq+((aJ>>>2&3)<<2)>>2]*(c[k>>2]=bc&-2147483648^aH,+g[k>>2]);as=bc<<(aJ>>>13&1);g[be+8>>2]=+g[aq+((aJ>>>4&3)<<2)>>2]*(c[k>>2]=as&-2147483648^aH,+g[k>>2]);g[be+12>>2]=+g[aq+((aJ>>>6&3)<<2)>>2]*(c[k>>2]=as<<(aJ>>>14&1)&-2147483648^aH,+g[k>>2]);aH=ax-4|0;if((aH|0)==0){break}bg=bH;ax=aH;be=be+16|0}be=az+1|0;if(be>>>0<bn>>>0){bd=bH;az=be;bf=bf+512|0}else{by=bH;break}}break};case 2:{if(aF){by=bi;break L1783}bf=ag+(an<<2)|0;az=bi;bd=0;aw=ay;while(1){be=az;ax=au;bg=aw;while(1){aH=c[u>>2]|0;aJ=aH+(be>>>3)|0;as=(cV(d[aJ]|d[aJ+1|0]<<8|d[aJ+2|0]<<16|d[aJ+3|0]<<24|0)|0)<<(be&7)>>>24;aJ=b[am+(as<<2)>>1]|0;bc=b[am+(as<<2)+2>>1]|0;as=bc<<16>>16;if(bc<<16>>16<0){bc=be+8|0;bj=ao>>>0>bc>>>0?bc:ao;bc=aH+(bj>>>3)|0;aH=((cV(d[bc]|d[bc+1|0]<<8|d[bc+2|0]<<16|d[bc+3|0]<<24|0)|0)<<(bj&7)>>>((as+32|0)>>>0))+aJ|0;bI=bj;bJ=b[am+(aH<<2)+2>>1]|0;bK=b[am+(aH<<2)>>1]|0}else{bI=be;bJ=as;bK=aJ}aJ=bI+bJ|0;bL=ao>>>0>aJ>>>0?aJ:ao;aJ=e[ah+(bK<<1)>>1]|0;bx=+g[bf>>2];g[bg>>2]=bx*+g[aq+((aJ&15)<<2)>>2];g[bg+4>>2]=bx*+g[aq+((aJ>>>4&15)<<2)>>2];aJ=ax-2|0;if((aJ|0)==0){break}be=bL;ax=aJ;bg=bg+8|0}bg=bd+1|0;if(bg>>>0<bn>>>0){az=bL;bd=bg;aw=aw+512|0}else{by=bL;break}}break};case 3:case 4:{if(aF){by=bi;break L1783}aw=ag+(an<<2)|0;bd=bi;az=0;bf=ay;while(1){bg=au;ax=bf;be=bd;while(1){aJ=c[u>>2]|0;as=aJ+(be>>>3)|0;aH=(cV(d[as]|d[as+1|0]<<8|d[as+2|0]<<16|d[as+3|0]<<24|0)|0)<<(be&7);as=aH>>>24;bj=b[am+(as<<2)>>1]|0;bc=b[am+(as<<2)+2>>1]|0;as=bc<<16>>16;if(bc<<16>>16<0){bc=be+8|0;ba=ao>>>0>bc>>>0?bc:ao;bc=aJ+(ba>>>3)|0;aJ=(cV(d[bc]|d[bc+1|0]<<8|d[bc+2|0]<<16|d[bc+3|0]<<24|0)|0)<<(ba&7);bc=(aJ>>>((as+32|0)>>>0))+bj|0;bM=b[am+(bc<<2)+2>>1]|0;bN=b[am+(bc<<2)>>1]|0;bO=ba;bP=aJ}else{bM=as;bN=bj;bO=be;bP=aH}aH=bM+bO|0;bj=e[ah+(bN<<1)>>1]|0;as=bj>>>8&15;if((as|0)==0){bQ=0}else{bQ=bP<<bM>>>((32-as|0)>>>0)<<(bj>>>12)}aJ=as+(ao>>>0>aH>>>0?aH:ao)|0;bR=ao>>>0>aJ>>>0?aJ:ao;aJ=(g[k>>2]=+g[aw>>2],c[k>>2]|0);g[ax>>2]=+g[aq+((bj&15)<<2)>>2]*(c[k>>2]=aJ^bQ>>>1<<31,+g[k>>2]);g[ax+4>>2]=+g[aq+((bj>>>4&15)<<2)>>2]*(c[k>>2]=aJ^bQ<<31,+g[k>>2]);aJ=bg-2|0;if((aJ|0)==0){break}bg=aJ;ax=ax+8|0;be=bR}be=az+1|0;if(be>>>0<bn>>>0){bd=bR;az=be;bf=bf+512|0}else{by=bR;break}}break};default:{if(aF){by=bi;break L1783}bf=ag+(an<<2)|0;az=bi;bd=0;aw=ay;while(1){be=au;ax=aw;bg=az;while(1){aJ=c[u>>2]|0;bj=aJ+(bg>>>3)|0;aH=(cV(d[bj]|d[bj+1|0]<<8|d[bj+2|0]<<16|d[bj+3|0]<<24|0)|0)<<(bg&7);bj=aH>>>24;as=b[am+(bj<<2)>>1]|0;ba=b[am+(bj<<2)+2>>1]|0;bj=ba<<16>>16;if(ba<<16>>16<0){ba=bg+8|0;bc=ao>>>0>ba>>>0?ba:ao;ba=aJ+(bc>>>3)|0;bb=(cV(d[ba]|d[ba+1|0]<<8|d[ba+2|0]<<16|d[ba+3|0]<<24|0)|0)<<(bc&7);ba=(bb>>>((bj+32|0)>>>0))+as|0;bS=b[am+(ba<<2)+2>>1]|0;bT=b[am+(ba<<2)>>1]|0;bU=bc;bV=bb}else{bS=bj;bT=as;bU=bg;bV=aH}aH=bS+bU|0;as=ao>>>0>aH>>>0?aH:ao;if((bT|0)==0){c[ax>>2]=0;c[ax+4>>2]=0;bW=as}else{aH=e[ah+(bT<<1)>>1]|0;bj=aH>>>12;bb=aH>>>8;bc=32-bj|0;ba=bV<<bS>>>(bc>>>0)<<bc;bc=bj+as|0;as=ao>>>0>bc>>>0?bc:ao;if((bb&1|0)==0){bc=c[aq+((aH&15)<<2)>>2]|0;c[ax>>2]=bc|ba&-2147483648;bX=ba<<((bc|0)!=0);bY=as}else{bc=aJ+(as>>>3)|0;aJ=(cV(d[bc]|d[bc+1|0]<<8|d[bc+2|0]<<16|d[bc+3|0]<<24|0)|0)<<(as&7);bc=~aJ;bj=aJ>>>0<4294901760;a9=bj?16:0;a8=bj?bc>>>16:bc;if((a8&65280|0)==0){bZ=a8;b_=a9}else{bZ=a8>>>8;b_=a9|8}a9=31-b_-(d[247648+bZ|0]|0)|0;if(a9>>>0>8){break L1761}a8=a9+1|0;bc=a8+as|0;as=a9+4|0;bj=(ao>>>0>bc>>>0?bc:ao)+as|0;c[ax>>2]=c[2981368+((aJ<<a8>>>((28-a9|0)>>>0))+(1<<as)<<2)>>2]|ba&-2147483648;bX=ba<<1;bY=ao>>>0>bj>>>0?bj:ao}bj=ax+4|0;if((bb&2|0)==0){b$=bY;b0=c[aq+((aH>>>4&15)<<2)>>2]|bX&-2147483648}else{aH=(c[u>>2]|0)+(bY>>>3)|0;bb=(cV(d[aH]|d[aH+1|0]<<8|d[aH+2|0]<<16|d[aH+3|0]<<24|0)|0)<<(bY&7);aH=~bb;ba=bb>>>0<4294901760;as=ba?16:0;a9=ba?aH>>>16:aH;if((a9&65280|0)==0){b1=a9;b2=as}else{b1=a9>>>8;b2=as|8}as=31-b2-(d[247648+b1|0]|0)|0;if(as>>>0>8){break L1761}a9=as+1|0;aH=a9+bY|0;ba=as+4|0;a8=(ao>>>0>aH>>>0?aH:ao)+ba|0;b$=ao>>>0>a8>>>0?a8:ao;b0=c[2981368+((bb<<a9>>>((28-as|0)>>>0))+(1<<ba)<<2)>>2]|bX&-2147483648}c[bj>>2]=b0;bW=b$}bj=be-2|0;if((bj|0)==0){break}be=bj;ax=ax+8|0;bg=bW}cY[c[bq>>2]&15](aw,aw,+g[bf>>2],au);bg=bd+1|0;if(bg>>>0<bn>>>0){az=bW;bd=bg;aw=aw+512|0}else{by=bW;break}}}}}while(0);c[q>>2]=by}}while(0);an=an+1|0;b3=a[bl]|0;}while((H|0)<(b3&255|0));bs=an;bt=b3;bu=c[r>>2]|0}H=al+1|0;if((H|0)<(bu|0)){bk=bs;al=H;bm=bm+(bn<<7<<2)|0;ak=bt;ai=bu}else{break L1759}}AN(c[f+4>>2]|0,16,1674800,(af=i,i=i+1|0,i=i+7&-8,c[af>>2]=0,af)|0);i=af;x=-1094995529;i=m;return x|0}}while(0);do{if(!aj){af=c[p>>2]|0;if((af|0)>0){b4=0;b5=0}else{break}while(1){bu=c[n+8+(b5<<2)>>2]|0;bt=h+5312+(bu<<2)|0;bx=+g[bt>>2];bs=b4;while(1){r=bs+1|0;if((e[l+(r<<1)>>1]|0)>(bu|0)){break}else{bs=r}}do{if((c[h+3172+(bs<<2)>>2]|0)!=13){bu=ag+(bs<<2)|0;bv=+g[bu>>2];if(bv==0.0){break}bw=+(-(c[n+24+(b5<<2)>>2]|0)|0);if(bx!=0.0){b6=bx/bv;bv=b6/+S(+(+S(+(+R(+b6)))));if(b6>0.0){b7=-0.0-bw}else{b7=bw}b8=bv+b7}else{b8=bw}bw=b8*+a_(+(+R(+b8)));g[bt>>2]=bw*+g[bu>>2]}}while(0);bt=b5+1|0;if((bt|0)<(af|0)){b4=bs;b5=bt}else{break}}}}while(0);if((c[f+6408>>2]|0)!=1|t){x=0;i=m;return x|0}pV(f,h);x=0;i=m;return x|0}function pR(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;h=i;j=g+8|0;k=g+16|0;l=c[k>>2]|0;m=(c[j>>2]|0)+2|0;n=l>>>0>m>>>0?m:l;c[j>>2]=n;m=g|0;o=c[m>>2]|0;p=o+(n>>>3)|0;q=(cV(d[p]|d[p+1|0]<<8|d[p+2|0]<<16|d[p+3|0]<<24|0)|0)<<(n&7)>>>28;p=n+4|0;n=l>>>0>p>>>0?p:l;c[j>>2]=n;if((c[e+4>>2]|0)==(q|0)){r=n;s=l;t=o}else{AN(b,24,1706104,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;r=c[j>>2]|0;s=c[k>>2]|0;t=c[m>>2]|0}o=t+(r>>>3)|0;l=(cV(d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0)|0)<<(r&7)>>>28;o=r+4|0;r=s>>>0>o>>>0?o:s;c[j>>2]=r;o=t+(r>>>3)|0;n=(cV(d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0)|0)<<(r&7)>>>28;o=r+4|0;r=s>>>0>o>>>0?o:s;c[j>>2]=r;o=t+(r>>>3)|0;q=(cV(d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0)|0)<<(r&7)>>>28;o=r+4|0;r=s>>>0>o>>>0?o:s;c[j>>2]=r;o=t+(r>>>3)|0;e=(cV(d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0)|0)<<(r&7)>>>30;o=r+2|0;r=s>>>0>o>>>0?o:s;c[j>>2]=r;o=t+(r>>>3)|0;p=(cV(d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0)|0)<<(r&7)>>>29;o=r+3|0;r=s>>>0>o>>>0?o:s;c[j>>2]=r;o=t+(r>>>3)|0;v=(cV(d[o]|d[o+1|0]<<8|d[o+2|0]<<16|d[o+3|0]<<24|0)|0)<<(r&7)>>>28;o=r+4|0;r=s>>>0>o>>>0?o:s;c[j>>2]=r;o=((r|0)<(s|0))+r|0;if(((d[t+(r>>>3)|0]|0)&128>>>((r&7)>>>0)|0)==0){w=o}else{r=o+4|0;w=s>>>0>r>>>0?r:s}c[j>>2]=w;r=((w|0)<(s|0))+w|0;if(((d[t+(w>>>3)|0]|0)&128>>>((w&7)>>>0)|0)==0){x=r}else{w=r+4|0;x=s>>>0>w>>>0?w:s}c[j>>2]=x;w=((x|0)<(s|0))+x|0;if(((d[t+(x>>>3)|0]|0)&128>>>((x&7)>>>0)|0)==0){y=w}else{x=w+3|0;y=s>>>0>x>>>0?x:s}c[j>>2]=y;x=g+12|0;g=n+l|0;w=q+g|0;r=e+w|0;if(((c[x>>2]|0)-y|0)<(p+r+v<<2|0)){AN(b,16,1704184,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;z=-1;i=h;return z|0}L1908:do{if((l|0)!=0){o=f;A=l;B=y;C=t;D=s;while(1){E=A-1|0;F=(d[C+(B>>>3)|0]|0)<<(B&7)>>>7;c[j>>2]=((B|0)<(D|0))+B;a[o|0]=F&1;F=c[j>>2]|0;G=c[k>>2]|0;H=(c[m>>2]|0)+(F>>>3)|0;I=(cV(d[H]|d[H+1|0]<<8|d[H+2|0]<<16|d[H+3|0]<<24|0)|0)<<(F&7)>>>28;H=F+4|0;c[j>>2]=G>>>0>H>>>0?H:G;a[o+1|0]=I&255;a[o+2|0]=1;if((E|0)==0){break L1908}o=o+3|0;A=E;B=c[j>>2]|0;C=c[m>>2]|0;D=c[k>>2]|0}}}while(0);L1913:do{if((n|0)!=0){s=f+(l*3|0)|0;t=n;while(1){y=t-1|0;D=c[j>>2]|0;C=(d[(c[m>>2]|0)+(D>>>3)|0]|0)<<(D&7)>>>7;c[j>>2]=((D|0)<(c[k>>2]|0))+D;a[s|0]=C&1;C=c[j>>2]|0;D=c[k>>2]|0;B=(c[m>>2]|0)+(C>>>3)|0;A=(cV(d[B]|d[B+1|0]<<8|d[B+2|0]<<16|d[B+3|0]<<24|0)|0)<<(C&7)>>>28;B=C+4|0;c[j>>2]=D>>>0>B>>>0?B:D;a[s+1|0]=A&255;a[s+2|0]=2;if((y|0)==0){break L1913}s=s+3|0;t=y}}}while(0);L1919:do{if((q|0)!=0){n=f+(g*3|0)|0;l=q;while(1){t=l-1|0;s=c[j>>2]|0;y=(d[(c[m>>2]|0)+(s>>>3)|0]|0)<<(s&7)>>>7;c[j>>2]=((s|0)<(c[k>>2]|0))+s;a[n|0]=y&1;y=c[j>>2]|0;s=c[k>>2]|0;A=(c[m>>2]|0)+(y>>>3)|0;D=(cV(d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24|0)|0)<<(y&7)>>>28;A=y+4|0;c[j>>2]=s>>>0>A>>>0?A:s;a[n+1|0]=D&255;a[n+2|0]=3;if((t|0)==0){break L1919}n=n+3|0;l=t}}}while(0);if((e|0)!=0){q=f+(w*3|0)|0;w=e;while(1){e=w-1|0;a[q|0]=3;g=c[j>>2]|0;l=c[k>>2]|0;n=(c[m>>2]|0)+(g>>>3)|0;t=(cV(d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0)|0)<<(g&7)>>>28;n=g+4|0;c[j>>2]=l>>>0>n>>>0?n:l;a[q+1|0]=t&255;a[q+2|0]=4;if((e|0)==0){break}else{q=q+3|0;w=e}}}w=p<<2;p=c[j>>2]|0;q=-p|0;e=c[k>>2]|0;t=e-p|0;if((w|0)<(q|0)){J=q}else{J=(t|0)<(w|0)?t:w}w=J+p|0;c[j>>2]=w;if((v|0)==0){K=w}else{p=f+(r*3|0)|0;f=v;J=w;w=e;while(1){e=f-1|0;t=J+1|0;c[j>>2]=w>>>0>t>>>0?t:w;a[p|0]=2;t=c[j>>2]|0;q=c[k>>2]|0;l=(c[m>>2]|0)+(t>>>3)|0;n=(cV(d[l]|d[l+1|0]<<8|d[l+2|0]<<16|d[l+3|0]<<24|0)|0)<<(t&7)>>>28;l=t+4|0;c[j>>2]=q>>>0>l>>>0?l:q;a[p+1|0]=n&255;a[p+2|0]=5;if((e|0)==0){break}p=p+3|0;f=e;J=c[j>>2]|0;w=c[k>>2]|0}K=c[j>>2]|0}w=-K&7;J=c[k>>2]|0;if((w|0)==0){L=K}else{k=w+K|0;K=J>>>0>k>>>0?k:J;c[j>>2]=K;L=K}K=(c[m>>2]|0)+(L>>>3)|0;m=(cV(d[K]|d[K+1|0]<<8|d[K+2|0]<<16|d[K+3|0]<<24|0)|0)<<(L&7)>>>24;K=L+8|0;L=J>>>0>K>>>0?K:J;c[j>>2]=L;K=m<<3;if(((c[x>>2]|0)-L|0)<(K|0)){AN(b,16,1704184,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;z=-1094995529;i=h;return z|0}u=-L|0;b=J-L|0;if((K|0)<(u|0)){M=u}else{M=(b|0)<(K|0)?b:K}c[j>>2]=M+L;z=v+r|0;i=h;return z|0}function pS(b,e,f,g,h){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0;j=i;i=i+1096|0;k=j|0;l=j+8|0;m=j+16|0;n=j+24|0;o=b+4|0;p=c[o>>2]|0;q=b+6452|0;if((q|0)!=(e|0)){r=q|0;q=e|0;s=f*3|0;NN(r|0,q|0,s)|0;c[b+6644>>2]=f}s=p+456|0;L1955:do{if((c[s>>2]|0)==0&(c[s+4>>2]|0)==(-2147483648|0)){t=0;u=0}else{q=n;r=j+1048|0;v=j+1056|0;w=j+1064|0;x=j+1072|0;y=j+1080|0;NM(q|0,0,1024);if(f>>>0>64){t=0;u=0;break}if((f|0)>0){z=0;A=0;B=0;C=0;while(1){if((a[e+(C*3|0)+2|0]|0)!=1){D=z;E=A;F=B;G=C;break}if((a[e+(C*3|0)|0]|0)==1){if(!((B|0)==0|(A|0)==0)){t=0;u=0;break L1955}H=0;I=1;J=z+2|0}else{H=B^1;I=A;J=z+1|0}K=C+1|0;if((K|0)<(f|0)){z=J;A=I;B=H;C=K}else{D=J;E=I;F=H;G=K;break}}if((F|0)!=0&(E|0)!=0|(D|0)<0){t=0;u=0;break}else{L=D;M=G}}else{L=0;M=0}if((M|0)<(f|0)){C=0;B=0;A=M;while(1){if((a[e+(A*3|0)+2|0]|0)!=2){N=C;O=B;P=A;break}if((a[e+(A*3|0)|0]|0)==1){if((B|0)!=0){t=0;u=0;break L1955}Q=0;R=C+2|0}else{Q=B^1;R=C+1|0}z=A+1|0;if((z|0)<(f|0)){C=R;B=Q;A=z}else{N=R;O=Q;P=z;break}}if((O|0)!=0|(N|0)<0){t=0;u=0;break}else{S=N;T=P}}else{S=0;T=M}if((T|0)<(f|0)){A=0;B=0;C=T;while(1){if((a[e+(C*3|0)+2|0]|0)!=3){U=A;break}if((a[e+(C*3|0)|0]|0)==1){if((B|0)!=0){t=0;u=0;break L1955}V=0;W=A+2|0}else{V=B^1;W=A+1|0}z=C+1|0;if((z|0)<(f|0)){A=W;B=V;C=z}else{U=W;break}}if((U|0)<0){t=0;u=0;break}else{X=U}}else{X=0}if((L&1|0)==0){Y=L;Z=0}else{C=a[e+1|0]|0;B=n|0;c[B>>2]=4;c[B+4>>2]=0;a[n+8|0]=0;a[n+9|0]=C;a[n+10|0]=1;C=q+11|0;a[C]=a[r]|0;a[C+1|0]=a[r+1|0]|0;a[C+2|0]=a[r+2|0]|0;a[C+3|0]=a[r+3|0]|0;a[C+4|0]=a[r+4|0]|0;Y=L-1|0;Z=1}if((Y|0)>3){C=k|0;B=l|0;A=m|0;z=n+(Z<<4)|0;K=a[e+(Z*3|0)+1|0]|0;_=z|0;if((a[e+(Z*3|0)|0]|0)==1){c[_>>2]=192;c[_+4>>2]=0;a[n+(Z<<4)+8|0]=1;a[n+(Z<<4)+9|0]=K;a[n+(Z<<4)+10|0]=1;$=z+11|0;a[$]=a[C]|0;a[$+1|0]=a[C+1|0]|0;a[$+2|0]=a[C+2|0]|0;a[$+3|0]=a[C+3|0]|0;a[$+4|0]=a[C+4|0]|0;aa=1}else{c[_>>2]=64;c[_+4>>2]=0;a[n+(Z<<4)+8|0]=0;a[n+(Z<<4)+9|0]=K;a[n+(Z<<4)+10|0]=1;K=z+11|0;a[K]=a[B]|0;a[K+1|0]=a[B+1|0]|0;a[K+2|0]=a[B+2|0]|0;a[K+3|0]=a[B+3|0]|0;a[K+4|0]=a[B+4|0]|0;B=Z+1|0;K=n+(B<<4)|0;z=a[e+(B*3|0)+1|0]|0;_=K|0;c[_>>2]=128;c[_+4>>2]=0;a[n+(B<<4)+8|0]=0;a[n+(B<<4)+9|0]=z;a[n+(B<<4)+10|0]=1;B=K+11|0;a[B]=a[A]|0;a[B+1|0]=a[A+1|0]|0;a[B+2|0]=a[A+2|0]|0;a[B+3|0]=a[A+3|0]|0;a[B+4|0]=a[A+4|0]|0;aa=2}ab=Y-2|0;ac=aa+Z|0}else{ab=Y;ac=Z}do{if((ab|0)>1){A=k|0;B=l|0;K=m|0;z=n+(ac<<4)|0;_=a[e+(ac*3|0)+1|0]|0;C=z|0;if((a[e+(ac*3|0)|0]|0)==1){c[C>>2]=3;c[C+4>>2]=0;a[n+(ac<<4)+8|0]=1;a[n+(ac<<4)+9|0]=_;a[n+(ac<<4)+10|0]=1;$=z+11|0;a[$]=a[A]|0;a[$+1|0]=a[A+1|0]|0;a[$+2|0]=a[A+2|0]|0;a[$+3|0]=a[A+3|0]|0;a[$+4|0]=a[A+4|0]|0;ad=1}else{c[C>>2]=1;c[C+4>>2]=0;a[n+(ac<<4)+8|0]=0;a[n+(ac<<4)+9|0]=_;a[n+(ac<<4)+10|0]=1;_=z+11|0;a[_]=a[B]|0;a[_+1|0]=a[B+1|0]|0;a[_+2|0]=a[B+2|0]|0;a[_+3|0]=a[B+3|0]|0;a[_+4|0]=a[B+4|0]|0;_=ac+1|0;z=n+(_<<4)|0;C=a[e+(_*3|0)+1|0]|0;$=z|0;c[$>>2]=2;c[$+4>>2]=0;a[n+(_<<4)+8|0]=0;a[n+(_<<4)+9|0]=C;a[n+(_<<4)+10|0]=1;_=z+11|0;a[_]=a[K]|0;a[_+1|0]=a[K+1|0]|0;a[_+2|0]=a[K+2|0]|0;a[_+3|0]=a[K+3|0]|0;a[_+4|0]=a[K+4|0]|0;ad=2}_=ad+ac|0;z=ab-2|0;if((z|0)>1){ae=_;af=z}else{ag=_;break}while(1){_=(a[e+(ae*3|0)|0]|0)==1;z=n+(ae<<4)|0;C=a[e+(ae*3|0)+1|0]|0;$=z|0;c[$>>2]=-1;c[$+4>>2]=-1;$=n+(ae<<4)+8|0;if(_){a[$]=1;a[n+(ae<<4)+9|0]=C;a[n+(ae<<4)+10|0]=1;_=z+11|0;a[_]=a[A]|0;a[_+1|0]=a[A+1|0]|0;a[_+2|0]=a[A+2|0]|0;a[_+3|0]=a[A+3|0]|0;a[_+4|0]=a[A+4|0]|0;ah=1}else{a[$]=0;a[n+(ae<<4)+9|0]=C;a[n+(ae<<4)+10|0]=1;C=z+11|0;a[C]=a[B]|0;a[C+1|0]=a[B+1|0]|0;a[C+2|0]=a[B+2|0]|0;a[C+3|0]=a[B+3|0]|0;a[C+4|0]=a[B+4|0]|0;C=ae+1|0;z=n+(C<<4)|0;$=a[e+(C*3|0)+1|0]|0;_=z|0;c[_>>2]=-1;c[_+4>>2]=-1;a[n+(C<<4)+8|0]=0;a[n+(C<<4)+9|0]=$;a[n+(C<<4)+10|0]=1;C=z+11|0;a[C]=a[K]|0;a[C+1|0]=a[K+1|0]|0;a[C+2|0]=a[K+2|0]|0;a[C+3|0]=a[K+3|0]|0;a[C+4|0]=a[K+4|0]|0;ah=2}C=ah+ae|0;z=af-2|0;if((z|0)>1){ae=C;af=z}else{ag=C;break}}}else{ag=ac}}while(0);do{if((S|0)>1){r=k|0;q=l|0;K=m|0;B=n+(ag<<4)|0;A=a[e+(ag*3|0)+1|0]|0;C=B|0;if((a[e+(ag*3|0)|0]|0)==1){c[C>>2]=1536;c[C+4>>2]=0;a[n+(ag<<4)+8|0]=1;a[n+(ag<<4)+9|0]=A;a[n+(ag<<4)+10|0]=1;z=B+11|0;a[z]=a[r]|0;a[z+1|0]=a[r+1|0]|0;a[z+2|0]=a[r+2|0]|0;a[z+3|0]=a[r+3|0]|0;a[z+4|0]=a[r+4|0]|0;ai=1}else{c[C>>2]=512;c[C+4>>2]=0;a[n+(ag<<4)+8|0]=0;a[n+(ag<<4)+9|0]=A;a[n+(ag<<4)+10|0]=1;A=B+11|0;a[A]=a[q]|0;a[A+1|0]=a[q+1|0]|0;a[A+2|0]=a[q+2|0]|0;a[A+3|0]=a[q+3|0]|0;a[A+4|0]=a[q+4|0]|0;A=ag+1|0;B=n+(A<<4)|0;C=a[e+(A*3|0)+1|0]|0;z=B|0;c[z>>2]=1024;c[z+4>>2]=0;a[n+(A<<4)+8|0]=0;a[n+(A<<4)+9|0]=C;a[n+(A<<4)+10|0]=1;A=B+11|0;a[A]=a[K]|0;a[A+1|0]=a[K+1|0]|0;a[A+2|0]=a[K+2|0]|0;a[A+3|0]=a[K+3|0]|0;a[A+4|0]=a[K+4|0]|0;ai=2}A=ai+ag|0;B=S-2|0;if((B|0)>1){aj=A;ak=B}else{al=A;break}while(1){A=(a[e+(aj*3|0)|0]|0)==1;B=n+(aj<<4)|0;C=a[e+(aj*3|0)+1|0]|0;z=B|0;c[z>>2]=-1;c[z+4>>2]=-1;z=n+(aj<<4)+8|0;if(A){a[z]=1;a[n+(aj<<4)+9|0]=C;a[n+(aj<<4)+10|0]=2;A=B+11|0;a[A]=a[r]|0;a[A+1|0]=a[r+1|0]|0;a[A+2|0]=a[r+2|0]|0;a[A+3|0]=a[r+3|0]|0;a[A+4|0]=a[r+4|0]|0;am=1}else{a[z]=0;a[n+(aj<<4)+9|0]=C;a[n+(aj<<4)+10|0]=2;C=B+11|0;a[C]=a[q]|0;a[C+1|0]=a[q+1|0]|0;a[C+2|0]=a[q+2|0]|0;a[C+3|0]=a[q+3|0]|0;a[C+4|0]=a[q+4|0]|0;C=aj+1|0;B=n+(C<<4)|0;z=a[e+(C*3|0)+1|0]|0;A=B|0;c[A>>2]=-1;c[A+4>>2]=-1;a[n+(C<<4)+8|0]=0;a[n+(C<<4)+9|0]=z;a[n+(C<<4)+10|0]=2;C=B+11|0;a[C]=a[K]|0;a[C+1|0]=a[K+1|0]|0;a[C+2|0]=a[K+2|0]|0;a[C+3|0]=a[K+3|0]|0;a[C+4|0]=a[K+4|0]|0;am=2}C=am+aj|0;B=ak-2|0;if((B|0)>1){aj=C;ak=B}else{al=C;break}}}else{al=ag}}while(0);if((X|0)>3){K=k|0;q=l|0;r=m|0;C=X-4|0;B=al;z=X;do{A=(a[e+(B*3|0)|0]|0)==1;$=n+(B<<4)|0;_=a[e+(B*3|0)+1|0]|0;an=$|0;c[an>>2]=-1;c[an+4>>2]=-1;an=n+(B<<4)+8|0;if(A){a[an]=1;a[n+(B<<4)+9|0]=_;a[n+(B<<4)+10|0]=3;A=$+11|0;a[A]=a[K]|0;a[A+1|0]=a[K+1|0]|0;a[A+2|0]=a[K+2|0]|0;a[A+3|0]=a[K+3|0]|0;a[A+4|0]=a[K+4|0]|0;ao=1}else{a[an]=0;a[n+(B<<4)+9|0]=_;a[n+(B<<4)+10|0]=3;_=$+11|0;a[_]=a[q]|0;a[_+1|0]=a[q+1|0]|0;a[_+2|0]=a[q+2|0]|0;a[_+3|0]=a[q+3|0]|0;a[_+4|0]=a[q+4|0]|0;_=B+1|0;$=n+(_<<4)|0;an=a[e+(_*3|0)+1|0]|0;A=$|0;c[A>>2]=-1;c[A+4>>2]=-1;a[n+(_<<4)+8|0]=0;a[n+(_<<4)+9|0]=an;a[n+(_<<4)+10|0]=3;_=$+11|0;a[_]=a[r]|0;a[_+1|0]=a[r+1|0]|0;a[_+2|0]=a[r+2|0]|0;a[_+3|0]=a[r+3|0]|0;a[_+4|0]=a[r+4|0]|0;ao=2}B=ao+B|0;z=z-2|0;}while((z|0)>3);ap=B;aq=X-2-(C&-2)|0}else{ap=al;aq=X}if((aq|0)>1){z=k|0;r=l|0;q=m|0;K=n+(ap<<4)|0;_=a[e+(ap*3|0)+1|0]|0;$=K|0;if((a[e+(ap*3|0)|0]|0)==1){c[$>>2]=48;c[$+4>>2]=0;a[n+(ap<<4)+8|0]=1;a[n+(ap<<4)+9|0]=_;a[n+(ap<<4)+10|0]=3;an=K+11|0;a[an]=a[z]|0;a[an+1|0]=a[z+1|0]|0;a[an+2|0]=a[z+2|0]|0;a[an+3|0]=a[z+3|0]|0;a[an+4|0]=a[z+4|0]|0;ar=1}else{c[$>>2]=16;c[$+4>>2]=0;a[n+(ap<<4)+8|0]=0;a[n+(ap<<4)+9|0]=_;a[n+(ap<<4)+10|0]=3;_=K+11|0;a[_]=a[r]|0;a[_+1|0]=a[r+1|0]|0;a[_+2|0]=a[r+2|0]|0;a[_+3|0]=a[r+3|0]|0;a[_+4|0]=a[r+4|0]|0;r=ap+1|0;_=n+(r<<4)|0;K=a[e+(r*3|0)+1|0]|0;$=_|0;c[$>>2]=32;c[$+4>>2]=0;a[n+(r<<4)+8|0]=0;a[n+(r<<4)+9|0]=K;a[n+(r<<4)+10|0]=3;r=_+11|0;a[r]=a[q]|0;a[r+1|0]=a[q+1|0]|0;a[r+2|0]=a[q+2|0]|0;a[r+3|0]=a[q+3|0]|0;a[r+4|0]=a[q+4|0]|0;ar=2}as=aq-2|0;at=ar+ap|0}else{as=aq;at=ap}if((as|0)==0){au=at}else{q=n+(at<<4)|0;r=a[e+(at*3|0)+1|0]|0;_=q|0;c[_>>2]=256;c[_+4>>2]=0;a[n+(at<<4)+8|0]=0;a[n+(at<<4)+9|0]=r;a[n+(at<<4)+10|0]=3;r=q+11|0;a[r]=a[v]|0;a[r+1|0]=a[v+1|0]|0;a[r+2|0]=a[v+2|0]|0;a[r+3|0]=a[v+3|0]|0;a[r+4|0]=a[v+4|0]|0;au=at+1|0}do{if((au|0)<(f|0)){if((a[e+(au*3|0)+2|0]|0)!=4){av=au;break}r=n+(au<<4)|0;q=a[e+(au*3|0)+1|0]|0;_=r|0;c[_>>2]=8;c[_+4>>2]=0;a[n+(au<<4)+8|0]=3;a[n+(au<<4)+9|0]=q;a[n+(au<<4)+10|0]=4;q=r+11|0;a[q]=a[w]|0;a[q+1|0]=a[w+1|0]|0;a[q+2|0]=a[w+2|0]|0;a[q+3|0]=a[w+3|0]|0;a[q+4|0]=a[w+4|0]|0;av=au+1|0}else{av=au}}while(0);L2051:do{if((av|0)<(f|0)){w=av;while(1){if((a[e+(w*3|0)+2|0]|0)!=4){aw=w;break L2051}v=n+(w<<4)|0;C=a[e+(w*3|0)+1|0]|0;B=v|0;c[B>>2]=-1;c[B+4>>2]=-1;a[n+(w<<4)+8|0]=3;a[n+(w<<4)+9|0]=C;a[n+(w<<4)+10|0]=4;C=v+11|0;a[C]=a[x]|0;a[C+1|0]=a[x+1|0]|0;a[C+2|0]=a[x+2|0]|0;a[C+3|0]=a[x+3|0]|0;a[C+4|0]=a[x+4|0]|0;C=w+1|0;if((C|0)<(f|0)){w=C}else{aw=C;break}}}else{aw=av}}while(0);x=aw;while(1){if((x|0)>1){ax=1;ay=0}else{break}while(1){w=ax-1|0;C=n+(w<<4)|0;v=c[C+4>>2]|0;B=n+(ax<<4)|0;q=B|0;r=c[q+4>>2]|0;if(v>>>0>r>>>0|v>>>0==r>>>0&(c[C>>2]|0)>>>0>(c[q>>2]|0)>>>0){q=B;c[y>>2]=c[q>>2];c[y+4>>2]=c[q+4>>2];c[y+8>>2]=c[q+8>>2];c[y+12>>2]=c[q+12>>2];B=n+(w<<4)|0;c[q>>2]=c[B>>2];c[q+4>>2]=c[B+4>>2];c[q+8>>2]=c[B+8>>2];c[q+12>>2]=c[B+12>>2];c[B>>2]=c[y>>2];c[B+4>>2]=c[y+4>>2];c[B+8>>2]=c[y+8>>2];c[B+12>>2]=c[y+12>>2];az=ax}else{az=ay}B=ax+1|0;if((B|0)<(x|0)){ax=B;ay=az}else{break}}if((az|0)>0){x=az}else{break}}if((aw|0)>0){aA=0;aB=0;aC=0}else{t=0;u=0;break}while(1){a[e+(aA*3|0)|0]=a[n+(aA<<4)+8|0]|0;a[e+(aA*3|0)+1|0]=a[n+(aA<<4)+9|0]|0;a[e+(aA*3|0)+2|0]=a[n+(aA<<4)+10|0]|0;x=n+(aA<<4)|0;y=c[x>>2]|0;B=c[x+4>>2]|0;x=(y|0)==(-1|0)&(B|0)==(-1|0);q=(x?0:y)|aC;y=(x?0:B)|aB;B=aA+1|0;if((B|0)<(aw|0)){aA=B;aB=y;aC=q}else{t=y;u=q;break}}}}while(0);do{if((f|0)>0){aC=b+6448|0;aB=0;aA=0;L2070:while(1){aw=a[e+(aA*3|0)|0]|0;if((aB|0)>63){aD=-1094995529;aE=1525;break}n=b+492+((aw&255)<<6)+(d[e+(aA*3|0)+1|0]<<2)|0;az=c[n>>2]|0;ay=(az|0)!=0;do{if((a[e+(aA*3|0)+2|0]|0)==0){if(ay){p1(az+108256|0)}A6(n);aF=aB}else{if(!ay){ax=A7(452064)|0;c[n>>2]=ax;if((ax|0)==0){aD=-12;aE=1526;break L2070}p0(b,ax+108256|0)}if(aw<<24>>24==2){aF=aB;break}ax=aw<<24>>24==1;do{if(ax){aG=1}else{if(aw<<24>>24!=0){aG=0;break}aG=(c[aC>>2]|0)==1|0}}while(0);if((aB|0)>=(64-aG|0)){aE=1508;break L2070}av=aB+1|0;c[b+5368+(aB<<2)>>2]=(c[n>>2]|0)+160;if(!ax){if(aw<<24>>24!=0){aF=av;break}if((c[aC>>2]|0)!=1){aF=av;break}}c[b+5368+(av<<2)>>2]=(c[n>>2]|0)+50304;aF=aB+2|0}}while(0);n=aA+1|0;if((n|0)<(f|0)){aB=aF;aA=n}else{aE=1517;break}}if((aE|0)==1508){AN(c[o>>2]|0,16,1708232,(aA=i,i=i+1|0,i=i+7&-8,c[aA>>2]=0,aA)|0);i=aA;aD=-1094995529;i=j;return aD|0}else if((aE|0)==1517){if(!((c[aC>>2]|0)==1&(aF|0)==2)){aH=t;aI=u;aJ=aF;break}aA=(u|0)==4&(t|0)==0;aH=aA?0:0;aI=aA?3:0;aJ=aF;break}else if((aE|0)==1525){i=j;return aD|0}else if((aE|0)==1526){i=j;return aD|0}}else{aH=t;aI=u;aJ=0}}while(0);u=b+748|0;t=b+492|0;NN(u|0,t|0,256)|0;if(!((aI|0)==0&(aH|0)==0)){t=p+448|0;c[t>>2]=aI;c[t+4>>2]=aH}t=b+6656|0;c[t>>2]=aI;c[t+4>>2]=aH;c[b+6648>>2]=aJ;c[p+416>>2]=aJ;c[b+6664>>2]=g;do{if((h|0)!=0){g=pP(c[o>>2]|0)|0;if((g|0)<0){aD=g}else{break}i=j;return aD|0}}while(0);aD=0;i=j;return aD|0}function pT(e,f,h,j){e=e|0;f=f|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0.0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0;k=i;l=c[h+236>>2]|0;if((c[e+6408>>2]|0)==4){AN(c[e+4>>2]|0,16,1730992,(e=i,i=i+1|0,i=i+7&-8,c[e>>2]=0,e)|0);i=e;i=k;return}e=h+176|0;m=c[e>>2]|0;if((m|0)<=0){i=k;return}n=h+160|0;o=a[n]|0;p=h+5472|0;q=0;r=0;s=f+5312|0;f=o;t=m;m=o;while(1){o=h+180+q|0;if(f<<24>>24==0){u=r;v=0;w=t;x=m}else{y=0;z=r;A=m;while(1){do{if((c[h+3332+(z<<2)>>2]|0)==0){B=A}else{C=+g[h+100552+(j*480|0)+(z<<2)>>2];D=a[o]|0;if(D<<24>>24==0){B=A;break}E=l+(y<<1)|0;F=l+(y+1<<1)|0;G=0;H=b[F>>1]|0;I=D;while(1){D=b[E>>1]|0;if((D&65535)<(H&65535)){J=G<<7;K=D&65535;do{D=K+J|0;L=s+(D<<2)|0;g[L>>2]=+g[L>>2]+C*+g[p+(D<<2)>>2];K=K+1|0;M=b[F>>1]|0;}while((K|0)<(M&65535|0));N=M;O=a[o]|0}else{N=H;O=I}K=G+1|0;if((K|0)<(O&255|0)){G=K;H=N;I=O}else{break}}B=a[n]|0}}while(0);I=y+1|0;P=z+1|0;if((I|0)<(B&255|0)){y=I;z=P;A=B}else{break}}u=P;v=B;w=c[e>>2]|0;x=B}A=(d[o]|0)<<7;z=q+1|0;if((z|0)<(w|0)){p=p+(A<<2)|0;q=z;r=u;s=s+(A<<2)|0;f=v;t=w;m=x}else{break}}i=k;return}function pU(e,f,h){e=e|0;f=f|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;j=i;k=h+8|0;l=c[k>>2]|0;m=h|0;n=d[(c[m>>2]|0)+(l>>>3)|0]|0;o=h+16|0;c[k>>2]=((l|0)<(c[o>>2]|0))+l;if((n&128>>>((l&7)>>>0)|0)!=0){AN(c[e+4>>2]|0,16,1688728,(p=i,i=i+1|0,i=i+7&-8,c[p>>2]=0,p)|0);i=p;q=-1094995529;i=j;return q|0}l=f+4|0;c[f+8>>2]=c[l>>2];n=c[k>>2]|0;h=c[o>>2]|0;r=(c[m>>2]|0)+(n>>>3)|0;s=(cV(d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0)|0)<<(n&7)>>>30;r=n+2|0;c[k>>2]=h>>>0>r>>>0?r:h;c[l>>2]=s;s=f+12|0;a[f+13|0]=a[s]|0;h=c[k>>2]|0;r=(d[(c[m>>2]|0)+(h>>>3)|0]|0)<<(h&7)>>>7;c[k>>2]=((h|0)<(c[o>>2]|0))+h;a[s]=r&1;r=f+16|0;c[r>>2]=1;a[f+20|0]=1;s=(c[l>>2]|0)==2;l=c[k>>2]|0;h=c[o>>2]|0;n=(c[m>>2]|0)+(l>>>3)|0;t=(cV(d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0)|0)<<(l&7);L2144:do{if(s){n=l+4|0;c[k>>2]=h>>>0>n>>>0?n:h;a[f|0]=t>>>28&255;n=0;do{u=c[k>>2]|0;v=d[(c[m>>2]|0)+(u>>>3)|0]|0;c[k>>2]=((u|0)<(c[o>>2]|0))+u;w=c[r>>2]|0;if((v&128>>>((u&7)>>>0)|0)==0){c[r>>2]=w+1;a[f+20+w|0]=1}else{u=w-1+(f+20)|0;a[u]=(a[u]|0)+1&255}n=n+1|0;}while((n|0)<7);c[f+88>>2]=8;n=e+6412|0;c[f+76>>2]=c[237960+(c[n>>2]<<2)>>2];c[f+84>>2]=d[257304+(c[n>>2]|0)|0]|0;c[f+92>>2]=d[237880+(c[n>>2]|0)|0]|0;c[f+96>>2]=0;x=1571}else{n=l+6|0;c[k>>2]=h>>>0>n>>>0?n:h;n=f|0;a[n]=t>>>26&255;c[f+88>>2]=1;u=e+6412|0;c[f+76>>2]=c[238016+(c[u>>2]<<2)>>2];c[f+84>>2]=d[257320+(c[u>>2]|0)|0]|0;c[f+92>>2]=d[237896+(c[u>>2]|0)|0]|0;w=c[k>>2]|0;v=(d[(c[m>>2]|0)+(w>>>3)|0]|0)<<(w&7)>>>7;c[k>>2]=((w|0)<(c[o>>2]|0))+w;w=v&1;c[f+96>>2]=w;v=f+104|0;c[v>>2]=0;if((w|0)==0){x=1571;break}w=c[e+6408>>2]|0;if((w|0)==1){y=c[k>>2]|0;z=c[m>>2]|0;A=d[z+(y>>>3)|0]|0;B=c[o>>2]|0;C=((y|0)<(B|0))+y|0;c[k>>2]=C;do{if((A&128>>>((y&7)>>>0)|0)!=0){D=z+(C>>>3)|0;E=(cV(d[D]|d[D+1|0]<<8|d[D+2|0]<<16|d[D+3|0]<<24|0)|0)<<(C&7)>>>27;D=C+5|0;c[k>>2]=B>>>0>D>>>0?D:B;c[v>>2]=E;if(!((E|0)==31|(E|0)==0)){break}AN(c[e+4>>2]|0,16,1682304,(p=i,i=i+1|0,i=i+7&-8,c[p>>2]=0,p)|0);i=p;break L2144}}while(0);v=a[n]|0;B=a[257288+(c[u>>2]|0)|0]|0;if(((v&255)>(B&255)?B:v)<<24>>24==0){x=1571;break}else{F=0}while(1){v=c[k>>2]|0;B=(d[(c[m>>2]|0)+(v>>>3)|0]|0)<<(v&7)>>>7;c[k>>2]=((v|0)<(c[o>>2]|0))+v;a[f+108+F|0]=B&1;B=F+1|0;v=a[n]|0;C=a[257288+(c[u>>2]|0)|0]|0;if((B|0)<(((v&255)>(C&255)?C:v)&255|0)){F=B}else{x=1571;break}}}else if((w|0)==2){AN(c[e+4>>2]|0,16,1686024,(p=i,i=i+1|0,i=i+7&-8,c[p>>2]=0,p)|0);i=p;break}else{u=c[k>>2]|0;B=c[o>>2]|0;v=(c[m>>2]|0)+(u>>>3)|0;C=(cV(d[v]|d[v+1|0]<<8|d[v+2|0]<<16|d[v+3|0]<<24|0)|0)<<(u&7)>>>31;v=u+1|0;c[k>>2]=B>>>0>v>>>0?v:B;B=C&255;a[f+28|0]=B;if(B<<24>>24==0){x=1571;break}B=a[n]|0;C=c[k>>2]|0;v=c[o>>2]|0;u=(c[m>>2]|0)+(C>>>3)|0;z=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(C&7)>>>21;u=C+11|0;c[k>>2]=v>>>0>u>>>0?u:v;b[f+30>>1]=z&65535;z=c[k>>2]|0;v=c[o>>2]|0;u=(c[m>>2]|0)+(z>>>3)|0;C=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(z&7)>>>29;u=z+3|0;c[k>>2]=v>>>0>u>>>0?u:v;g[f+32>>2]=+g[130728+(C<<2)>>2];C=(B&255)>40?40:B&255;if((C|0)==0){x=1571;break}else{G=0}while(1){B=c[k>>2]|0;v=(d[(c[m>>2]|0)+(B>>>3)|0]|0)<<(B&7)>>>7;c[k>>2]=((B|0)<(c[o>>2]|0))+B;a[f+36+G|0]=v&1;v=G+1|0;if((v|0)<(C|0)){G=v}else{x=1571;break}}}}}while(0);do{if((x|0)==1571){G=d[f|0]|0;o=c[f+84>>2]|0;if((G|0)>(o|0)){AN(c[e+4>>2]|0,16,1684048,(p=i,i=i+16|0,c[p>>2]=G,c[p+8>>2]=o,p)|0);i=p;break}else{q=0;i=j;return q|0}}}while(0);a[f|0]=0;q=-1094995529;i=j;return q|0}function pV(f,h){f=f|0;h=h|0;var i=0,j=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0.0,v=0,w=0.0,x=0,y=0.0,z=0,A=0.0,B=0,C=0.0,D=0,E=0.0,F=0,G=0.0,H=0.0,I=0.0,J=0.0,K=0.0,L=0,M=0,N=0,O=0;i=h+100|0;if((c[i>>2]|0)==0){j=0;do{g[h+33984+(j*24|0)+16>>2]=0.0;g[h+33984+(j*24|0)+20>>2]=0.0;g[h+33984+(j*24|0)>>2]=0.0;g[h+33984+(j*24|0)+4>>2]=0.0;g[h+33984+(j*24|0)+8>>2]=1.0;g[h+33984+(j*24|0)+12>>2]=1.0;j=j+1|0;}while((j|0)<672);c[i>>2]=1}if((c[h+4>>2]|0)==2){i=0;do{g[h+33984+(i*24|0)+16>>2]=0.0;g[h+33984+(i*24|0)+20>>2]=0.0;g[h+33984+(i*24|0)>>2]=0.0;g[h+33984+(i*24|0)+4>>2]=0.0;g[h+33984+(i*24|0)+8>>2]=1.0;g[h+33984+(i*24|0)+12>>2]=1.0;i=i+1|0;}while((i|0)<672);return}i=f+6412|0;f=c[i>>2]|0;if((a[257288+f|0]|0)!=0){j=h+76|0;l=h+96|0;m=0;n=c[j>>2]|0;o=f;while(1){f=b[n+(m<<1)>>1]|0;p=m+1|0;if((f&65535)<(e[n+(p<<1)>>1]|0)){q=h+108+m|0;r=f&65535;do{f=h+5312+(r<<2)|0;if((c[l>>2]|0)==0){s=0}else{s=(a[q]|0)!=0}t=h+33984+(r*24|0)+16|0;u=+g[t>>2];v=h+33984+(r*24|0)+20|0;w=+g[v>>2];x=h+33984+(r*24|0)|0;y=+g[x>>2];z=h+33984+(r*24|0)+4|0;A=+g[z>>2];B=h+33984+(r*24|0)+8|0;C=+g[B>>2];D=h+33984+(r*24|0)+12|0;E=+g[D>>2];if(C>1.0){F=(g[k>>2]=.953125/C,c[k>>2]|0);G=y*(c[k>>2]=F+32767+(F&1)&-65536,+g[k>>2])}else{G=0.0}if(E>1.0){F=(g[k>>2]=.953125/E,c[k>>2]|0);H=A*(c[k>>2]=F+32767+(F&1)&-65536,+g[k>>2])}else{H=0.0}I=u*G;if(s){J=(c[k>>2]=(g[k>>2]=I+w*H,c[k>>2]|0)+32768&-65536,+g[k>>2])+ +g[f>>2];g[f>>2]=J;K=J}else{K=+g[f>>2]}J=K-I;g[z>>2]=(c[k>>2]=(g[k>>2]=A*.90625+w*J,c[k>>2]|0)&-65536,+g[k>>2]);g[D>>2]=(c[k>>2]=(g[k>>2]=E*.90625+(w*w+J*J)*.5,c[k>>2]|0)&-65536,+g[k>>2]);g[x>>2]=(c[k>>2]=(g[k>>2]=y*.90625+u*K,c[k>>2]|0)&-65536,+g[k>>2]);g[B>>2]=(c[k>>2]=(g[k>>2]=C*.90625+(u*u+K*K)*.5,c[k>>2]|0)&-65536,+g[k>>2]);g[v>>2]=(c[k>>2]=(g[k>>2]=(u-G*K)*.953125,c[k>>2]|0)&-65536,+g[k>>2]);g[t>>2]=(c[k>>2]=(g[k>>2]=K*.953125,c[k>>2]|0)&-65536,+g[k>>2]);r=r+1|0;L=c[j>>2]|0;}while((r|0)<(e[L+(p<<1)>>1]|0));M=L;N=c[i>>2]|0}else{M=n;N=o}if((p|0)<(d[257288+N|0]|0)){m=p;n=M;o=N}else{break}}}N=c[h+104>>2]|0;if((N|0)==0){return}o=N-1|0;if((o|0)<672){O=o}else{return}do{g[h+33984+(O*24|0)+16>>2]=0.0;g[h+33984+(O*24|0)+20>>2]=0.0;g[h+33984+(O*24|0)>>2]=0.0;g[h+33984+(O*24|0)+4>>2]=0.0;g[h+33984+(O*24|0)+8>>2]=1.0;g[h+33984+(O*24|0)+12>>2]=1.0;O=O+30|0;}while((O|0)<672);return}function pW(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;e=c[d+50112>>2]|0;f=d+9408|0;g=(a[d+12|0]|0)!=0?2902120:2302480;h=(a[d+13|0]|0)!=0;i=h?2902632:2302992;j=h?2902120:2302480;h=b+1024|0;k=b+5632|0;l=d+4|0;if((c[l>>2]|0)==2){m=b+5184|0;n=b+5228|0;dm[c[n>>2]&127](m,h,d+5312|0);dm[c[n>>2]&127](m,b+1536|0,d+5824|0);dm[c[n>>2]&127](m,b+2048|0,d+6336|0);dm[c[n>>2]&127](m,b+2560|0,d+6848|0);dm[c[n>>2]&127](m,b+3072|0,d+7360|0);dm[c[n>>2]&127](m,b+3584|0,d+7872|0);dm[c[n>>2]&127](m,b+4096|0,d+8384|0);dm[c[n>>2]&127](m,b+4608|0,d+8896|0)}else{dm[c[b+5164>>2]&127](b+5120|0,h,d+5312|0)}m=c[d+8>>2]|0;do{if((m|0)==0|(m|0)==3){if((c[l>>2]|0)>>>0>=2){o=1613;break}cX[c[b+5344>>2]&255](e,f,h,i,512)}else{o=1613}}while(0);do{if((o|0)==1613){i=e;m=f;NN(i|0,m|0,1792)|0;m=(c[l>>2]|0)==2;i=b+5344|0;cX[c[i>>2]&255](e+1792|0,d+11200|0,h,j,64);if(m){cX[c[i>>2]&255](e+2304|0,b+1280|0,b+1536|0,g,64);cX[c[i>>2]&255](e+2816|0,b+1792|0,b+2048|0,g,64);cX[c[i>>2]&255](e+3328|0,b+2304|0,b+2560|0,g,64);cX[c[i>>2]&255](k,b+2816|0,b+3072|0,g,64);i=e+3840|0;m=k;NN(i|0,m|0,256)|0;break}else{m=e+2304|0;i=b+1280|0;NN(m|0,i|0,1792)|0;break}}}while(0);e=c[l>>2]|0;if((e|0)==2){l=f;k=b+5888|0;NN(l|0,k|0,256)|0;k=b+5344|0;cX[c[k>>2]&255](d+9664|0,b+3328|0,b+3584|0,g,64);cX[c[k>>2]&255](d+10176|0,b+3840|0,b+4096|0,g,64);cX[c[k>>2]&255](d+10688|0,b+4352|0,b+4608|0,g,64);g=d+11200|0;k=b+4864|0;NN(g|0,k|0,256)|0;return}k=f;f=b+3072|0;if((e|0)==1){NN(k|0,f|0,1792)|0;e=d+11200|0;d=b+4864|0;NN(e|0,d|0,256)|0;return}else{NN(k|0,f|0,2048)|0;return}}function pX(d,f){d=d|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;h=f|0;i=c[f+76>>2]|0;if((c[f+4>>2]|0)==2){return}j=c[f+50112>>2]|0;k=d+1024|0;l=f+30|0;m=b[l>>1]|0;if(m<<16>>16<1024){n=((m&65535)<<16)+67108864|0;if((n|0)>0){o=n>>16;p=1628}else{q=0}}else{o=2048;p=1628}L2247:do{if((p|0)==1628){n=f+32|0;r=0;s=m;while(1){g[j+(r<<2)>>2]=+g[f+21696+(r+2048-(s<<16>>16)<<2)>>2]*+g[n>>2];t=r+1|0;if((t|0)>=(o|0)){q=o;break L2247}r=t;s=b[l>>1]|0}}}while(0);NM(j+(q<<2)|0,0,2048-q<<2|0);dq[c[d+6688>>2]&127](d,k,j,h);j=f+152|0;if((c[j>>2]|0)!=0){dq[c[d+6684>>2]&127](k,j,h,0)}h=f|0;j=a[h]|0;if(j<<24>>24==0){return}else{u=0;v=j}while(1){do{if((a[f+36+u|0]|0)==0){w=v;x=u+1|0}else{j=b[i+(u<<1)>>1]|0;k=u+1|0;q=i+(k<<1)|0;if((j&65535)>=(e[q>>1]|0)){w=v;x=k;break}l=j&65535;do{j=f+5312+(l<<2)|0;g[j>>2]=+g[d+1024+(l<<2)>>2]+ +g[j>>2];l=l+1|0;}while((l|0)<(e[q>>1]|0));w=a[h]|0;x=k}}while(0);if((x|0)<(((w&255)>40?40:w&255)|0)){u=x;v=w}else{break}}return}function pY(a,b,f,h){a=a|0;b=b|0;f=f|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0.0,B=0,C=0,D=0,E=0,F=0,G=0.0,H=0,I=0.0,J=0,K=0,L=0,M=0,N=0,O=0;j=i;i=i+168|0;k=j|0;l=j+80|0;m=c[f+92>>2]|0;n=d[f|0]|0;o=(m|0)>(n|0)?n:m;m=f+88|0;n=c[m>>2]|0;if((n|0)<=0){i=j;return}p=f+84|0;q=f+76|0;f=(h|0)==0;h=l|0;r=0;s=n;while(1){n=b+4+(r<<2)|0;if((c[n>>2]|0)>0){t=r<<7;u=c[p>>2]|0;v=0;while(1){w=u-(c[b+36+(r<<4)+(v<<2)>>2]|0)|0;x=(w|0)<0?0:w;w=c[b+292+(r<<4)+(v<<2)>>2]|0;L2279:do{if((w|0)!=0){y=(w|0)>0;if(y){z=0;while(1){A=-0.0- +g[b+420+(r*320|0)+(v*80|0)+(z<<2)>>2];g[k+(z<<2)>>2]=A;B=z+1|0;C=B>>1;if((C|0)>0){D=z-1|0;E=0;do{F=k+(E<<2)|0;G=+g[F>>2];H=k+(D-E<<2)|0;I=+g[H>>2];g[F>>2]=G+I*A;g[H>>2]=I+G*A;E=E+1|0;}while((E|0)<(C|0))}if((B|0)<(w|0)){z=B}else{break}}}z=c[q>>2]|0;C=e[z+(((x|0)>(o|0)?o:x)<<1)>>1]|0;E=e[z+(((u|0)>(o|0)?o:u)<<1)>>1]|0;z=E-C|0;if((z|0)<1){break}D=(c[b+164+(r<<4)+(v<<2)>>2]|0)==0;H=D?1:-1;F=(D?C:E-1|0)+t|0;E=(z|0)>0;if(!f){if(E){J=0;K=F}else{break}while(1){C=(J|0)>(w|0)?w:J;if((C|0)>=1){D=a+(K<<2)|0;L=1;A=+g[D>>2];do{G=+g[a+(K-(ac(L,H)|0)<<2)>>2];A=A-G*+g[k+(L-1<<2)>>2];g[D>>2]=A;L=L+1|0;}while((L|0)<=(C|0))}C=J+1|0;if((C|0)<(z|0)){J=C;K=K+H|0}else{break L2279}}}if(E){M=0;N=F}else{break}while(1){C=a+(N<<2)|0;A=+g[C>>2];g[h>>2]=A;L=(M|0)>(w|0)?w:M;if((L|0)>=1){D=1;G=A;do{G=G+ +g[l+(D<<2)>>2]*+g[k+(D-1<<2)>>2];D=D+1|0;}while((D|0)<=(L|0));g[C>>2]=G}if(y){L=w;while(1){D=L-1|0;g[l+(L<<2)>>2]=+g[l+(D<<2)>>2];if((D|0)>0){L=D}else{break}}}L=M+1|0;if((L|0)<(z|0)){M=L;N=N+H|0}else{break}}}}while(0);w=v+1|0;if((w|0)<(c[n>>2]|0)){u=x;v=w}else{break}}O=c[m>>2]|0}else{O=s}v=r+1|0;if((v|0)<(O|0)){r=v;s=O}else{break}}i=j;return}function pZ(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0;g=(a[f+12|0]|0)!=0;h=(a[f+13|0]|0)!=0;i=f+4|0;if((c[i>>2]|0)==3){NM(e|0,0,1792);f=e+1792|0;dq[c[b+5328>>2]&127](f,f,h?2902120:2302480,128)}else{dq[c[b+5328>>2]&127](e,e,h?2902632:2302992,1024)}h=c[b+5352>>2]|0;f=e+4096|0;if((c[i>>2]|0)==1){i=e+5888|0;dq[h&127](i,i,g?2902120:2302480,128);NM(e+6400|0,0,1792);j=b+5248|0;k=b+5296|0;l=c[k>>2]|0;dm[l&127](j,d,e);return}else{dq[h&127](f,f,g?2902632:2302992,1024);j=b+5248|0;k=b+5296|0;l=c[k>>2]|0;dm[l&127](j,d,e);return}}function p_(b,d){b=b|0;d=d|0;var e=0,f=0,h=0,i=0,j=0,k=0,l=0,m=0;e=d+5312|0;f=(a[d+12|0]|0)!=0;h=f?2902632:2302992;i=f?2902120:2302480;f=c[d+4>>2]|0;if((f|0)==1){j=e;k=b+3072|0;NN(j|0,k|0,1792)|0;NM(d+7616|0,0,1792);dq[c[b+5352>>2]&127](d+7104|0,b+4864|0,i+256|0,64);k=0;while(1){g[d+5312+(k+512<<2)>>2]=+g[b+1024+(1023-k<<2)>>2]*+g[i+(63-k<<2)>>2];l=k+1|0;if((l|0)<64){k=l}else{m=j;break}}}else if((f|0)==2){f=e;j=d+9408|0;NN(f|0,j|0,2048)|0;NM(d+7616|0,0,1792);dq[c[b+5352>>2]&127](d+7104|0,b+4864|0,i+256|0,64);j=0;while(1){g[d+5312+(j+512<<2)>>2]=+g[b+1024+(1023-j<<2)>>2]*+g[i+(63-j<<2)>>2];k=j+1|0;if((k|0)<64){j=k}else{m=f;break}}}else{dq[c[b+5352>>2]&127](e,b+3072|0,h+2048|0,512);f=0;do{g[d+5312+(f+512<<2)>>2]=+g[b+1024+(1023-f<<2)>>2]*+g[h+(511-f<<2)>>2];f=f+1|0;}while((f|0)<512);m=e}e=d+21696|0;f=d+25792|0;NN(e|0,f|0,4096)|0;e=c[d+50112>>2]|0;NN(f|0,e|0,4096)|0;e=d+29888|0;NN(e|0,m|0,4096)|0;return}function p$(){var a=0;c[499435]=2896016;c[499437]=1098;qk(1997736,9,121,66112,1,1,65624,4,4,0,0,0,4)|0;c[499439]=2891648;c[499441]=1092;qk(1997752,9,121,258760,1,1,258272,4,4,0,0,0,4)|0;c[499443]=2888576;c[499445]=768;qk(1997768,9,49,65248,1,1,65048,4,4,0,0,0,4)|0;c[499447]=2884472;c[499449]=1026;qk(1997784,9,49,257896,1,1,257696,4,4,0,0,0,4)|0;c[499451]=2880240;c[499453]=1058;qk(1997800,9,63,65560,1,1,65304,4,4,0,0,0,4)|0;c[499455]=2876032;c[499457]=1052;qk(1997816,9,63,258208,1,1,257952,4,4,0,0,0,4)|0;c[499459]=2873856;c[499461]=544;qk(1997832,9,25,65016,1,1,64960,2,2,0,0,0,4)|0;c[499463]=2871680;c[499465]=544;qk(1997848,9,25,257664,1,1,257608,2,2,0,0,0,4)|0;c[499467]=2869312;c[499469]=592;qk(1997864,9,63,64896,1,1,64768,2,2,0,0,0,4)|0;c[499471]=2867264;c[499473]=512;qk(1997880,9,25,64736,1,1,64704,1,1,0,0,0,4)|0;a=1;do{g[77752+(a+320<<2)>>2]=+g[77752+(320-a<<2)>>2];a=a+1|0;}while((a|0)<320);g[19822]=-0.0- +g[19822];g[19950]=-0.0- +g[19950];a=0;do{g[2162584+(a<<2)>>2]=+g[77752+(a<<1<<2)>>2];a=a+1|0;}while((a|0)<320);rw();return}function p0(a,b){a=a|0;b=b|0;if((c[b+343668>>2]|0)!=0){return}a=b+68|0;c[b+64>>2]=c[a>>2];c[b+4>>2]=0;c[a>>2]=32;c[b+76>>2]=0;c[b+102312>>2]=-1;c[b+14664>>2]=-1;NM(b+12|0,-1|0,6);c[b+14656>>2]=1152;c[b+102304>>2]=1152;qZ(b+343648|0,7,1,4.76837158203125e-7)|0;qZ(b+343584|0,7,1,-65536.0)|0;rx(b+175392|0);q5(b+343712|0);c[b+343768>>2]=24;c[b+343772>>2]=22;c[b+343776>>2]=6;c[b+343780>>2]=18;return}function p1(a){a=a|0;q_(a+343648|0);q_(a+343584|0);return}function p2(f,g,h,j,k,l){f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,U=0,V=0.0,W=0.0,X=0.0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0.0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0;m=i;i=i+272|0;n=m|0;o=m+32|0;p=m+136|0;q=m+248|0;r=q;s=h;c[r>>2]=c[s>>2];c[r+4>>2]=c[s+4>>2];c[r+8>>2]=c[s+8>>2];c[r+12>>2]=c[s+12>>2];c[r+16>>2]=c[s+16>>2];s=(k<<3)-4|0;r=h+8|0;t=c[r>>2]|0;u=-t|0;v=(c[h+16>>2]|0)-t|0;if((s|0)<(u|0)){w=u}else{w=(v|0)<(s|0)?v:s}c[r>>2]=w+t;t=g+8|0;c[t>>2]=0;w=g|0;if((c[w>>2]|0)==0){c[w>>2]=c[f+6416>>2]<<1}r=f+6436|0;if((c[r>>2]|0)==0){c[r>>2]=c[f+6416>>2]<<1}r=q+8|0;s=c[r>>2]|0;v=c[q+16>>2]|0;if((j|0)==0){x=1;z=s}else{j=s+10|0;s=v>>>0>j>>>0?j:v;c[r>>2]=s;x=11;z=s}s=g+68|0;c[g+64>>2]=c[s>>2];r=g+76|0;c[g+72>>2]=c[r>>2];c[g+80>>2]=1;j=q+8|0;u=q|0;h=c[u>>2]|0;A=d[h+(z>>>3)|0]|0;B=q+16|0;C=((z|0)<(v|0))+z|0;c[j>>2]=C;if((A&128>>>((z&7)>>>0)|0)==0){D=x}else{z=m+240|0;A=g+24|0;E=c[A>>2]|0;c[g+4>>2]=1;F=g+12|0;a[z]=a[F]|0;a[z+1|0]=a[F+1|0]|0;a[z+2|0]=a[F+2|0]|0;a[z+3|0]=a[F+3|0]|0;a[z+4|0]=a[F+4|0]|0;a[z+5|0]=a[F+5|0]|0;G=d[h+(C>>>3)|0]<<(C&7)>>>7;H=((C|0)<(v|0))+C|0;c[j>>2]=H;c[g+20>>2]=G&1;G=h+(H>>>3)|0;I=(cV(d[G]|d[G+1|0]<<8|d[G+2|0]<<16|d[G+3|0]<<24|0)|0)<<(H&7)>>>28;G=H+4|0;H=v>>>0>G>>>0?G:v;c[j>>2]=H;a[F]=I&255;I=h+(H>>>3)|0;G=(cV(d[I]|d[I+1|0]<<8|d[I+2|0]<<16|d[I+3|0]<<24|0)|0)<<(H&7)>>>28;I=H+4|0;H=v>>>0>I>>>0?I:v;c[j>>2]=H;a[g+13|0]=G&255;G=h+(H>>>3)|0;I=H+3|0;a[g+14|0]=(cV(d[G]|d[G+1|0]<<8|d[G+2|0]<<16|d[G+3|0]<<24|0)|0)<<(H&7)>>>29&255;H=(v>>>0>I>>>0?I:v)+2|0;I=v>>>0>H>>>0?H:v;c[j>>2]=I;H=d[h+(I>>>3)|0]|0;G=((I|0)<(v|0))+I|0;c[j>>2]=G;J=d[h+(G>>>3)|0]|0;K=((G|0)<(v|0))+G|0;c[j>>2]=K;if((128>>>((I&7)>>>0)&H|0)==0){a[g+15|0]=2;a[g+16|0]=1;a[g+17|0]=2;L=K}else{H=h+(K>>>3)|0;I=(cV(d[H]|d[H+1|0]<<8|d[H+2|0]<<16|d[H+3|0]<<24|0)|0)<<(K&7)>>>30;H=K+2|0;K=v>>>0>H>>>0?H:v;c[j>>2]=K;a[g+15|0]=I&255;I=d[h+(K>>>3)|0]<<(K&7)>>>7;H=((K|0)<(v|0))+K|0;c[j>>2]=H;a[g+16|0]=I&1;I=h+(H>>>3)|0;K=(cV(d[I]|d[I+1|0]<<8|d[I+2|0]<<16|d[I+3|0]<<24|0)|0)<<(H&7)>>>30;I=H+2|0;H=v>>>0>I>>>0?I:v;c[j>>2]=H;a[g+17|0]=K&255;L=H}if((NO(z|0,F|0,6)|0)!=0){c[t>>2]=1}if((128>>>((G&7)>>>0)&J|0)==0){c[A>>2]=2;c[g+28>>2]=2;c[g+32>>2]=1;c[g+36>>2]=1;M=2;N=L}else{J=h+(L>>>3)|0;G=(cV(d[J]|d[J+1|0]<<8|d[J+2|0]<<16|d[J+3|0]<<24|0)|0)<<(L&7)>>>30;J=L+2|0;L=v>>>0>J>>>0?J:v;c[j>>2]=L;c[A>>2]=G;A=h+(L>>>3)|0;J=(cV(d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24|0)|0)<<(L&7)>>>30;A=L+2|0;L=v>>>0>A>>>0?A:v;c[j>>2]=L;c[g+28>>2]=J;J=d[h+(L>>>3)|0]<<(L&7)>>>7;A=((L|0)<(v|0))+L|0;c[j>>2]=A;c[g+32>>2]=J&1;J=d[h+(A>>>3)|0]<<(A&7)>>>7;h=((A|0)<(v|0))+A|0;c[j>>2]=h;c[g+36>>2]=J&1;M=G;N=h}do{if((M|0)==(E|0)){O=N}else{if((c[t>>2]|0)!=0){O=N;break}qc(g);O=c[j>>2]|0}}while(0);D=x-C+O|0}L2383:do{if((c[t>>2]|0)!=0){O=n;C=o;x=c[w>>2]|0;if((x|0)<32e3){P=3e3}else{P=(x|0)<64e3?4e3:5e3}if((x|0)==22050){Q=80328;R=1735}else if((x|0)==24e3){Q=80344;R=1735}else if((x|0)==32e3){Q=80360;R=1735}else if((x|0)==44100|(x|0)==48e3|(x|0)==64e3){Q=80376;R=1735}else if((x|0)==88200|(x|0)==96e3|(x|0)==128e3|(x|0)==176400|(x|0)==192e3){Q=80392;R=1735}else if((x|0)==16e3){Q=80312;R=1735}else{AN(c[f+4>>2]|0,16,1688688,(S=i,i=i+8|0,c[S>>2]=x,S)|0);i=S;U=g+12|0}L2395:do{if((R|0)==1735){N=x>>1;E=(((P<<8)+N|0)>>>0)/(x>>>0)|0;M=g+12|0;h=(a[Q+(d[M]|0)|0]|0)+((((P<<7)+N|0)>>>0)/(x>>>0)|0)|0;N=g+44|0;c[N>>2]=h;G=g+13|0;J=a[G]|0;A=J&255;do{if((J&255)<14){v=g+52|0;c[v>>2]=E;V=+(E|0);W=+T(+(64.0/V),+.07692307978868484);X=V*W;L=bR(+X)|0;b[n>>1]=L-E&65535;V=W*X;F=bR(+V)|0;b[n+2>>1]=F-L&65535;X=W*V;L=bR(+X)|0;b[n+4>>1]=L-F&65535;V=W*X;F=bR(+V)|0;b[n+6>>1]=F-L&65535;X=W*V;L=bR(+X)|0;b[n+8>>1]=L-F&65535;V=W*X;F=bR(+V)|0;b[n+10>>1]=F-L&65535;X=W*V;L=bR(+X)|0;b[n+12>>1]=L-F&65535;V=W*X;F=bR(+V)|0;b[n+14>>1]=F-L&65535;X=W*V;L=bR(+X)|0;b[n+16>>1]=L-F&65535;V=W*X;F=bR(+V)|0;b[n+18>>1]=F-L&65535;X=W*V;L=bR(+X)|0;b[n+20>>1]=L-F&65535;F=bR(+(W*X))|0;b[n+22>>1]=F-L&65535;b[n+24>>1]=64-F&65535;b6(O|0,13,2,100);F=a[G]|0;L=c[v>>2]|0;if(F<<24>>24==0){Y=L}else{z=F&255;F=0;H=L;do{H=(b[n+(F<<1)>>1]|0)+H|0;F=F+1|0;}while((F|0)<(z|0));c[v>>2]=H;Y=H}Z=Y;_=c[w>>2]|0}else{if((J<<24>>24|0)==14){z=h<<1;c[g+52>>2]=z;Z=z;_=x;break}else if((J<<24>>24|0)==15){z=h*3|0;c[g+52>>2]=z;Z=z;_=x;break}else{AN(c[f+4>>2]|0,16,1685992,(S=i,i=i+8|0,c[S>>2]=A,S)|0);i=S;U=M;break L2395}}}while(0);A=g+52|0;h=Z>>>0<64?Z:64;c[A>>2]=h;do{if((_|0)<32001){$=48}else{if((_|0)==44100){$=35;break}if((_|0)>47999){$=32;break}AN(0,0,1684016,(S=i,i=i+24|0,c[S>>2]=1682296,c[S+8>>2]=1680384,c[S+16>>2]=403,S)|0);i=S;cw();return 0}}while(0);J=c[N>>2]|0;G=h-J|0;if(G>>>0>$>>>0){AN(c[f+4>>2]|0,16,1678504,(S=i,i=i+8|0,c[S>>2]=G,S)|0);i=S;U=M;break}E=a[g+15|0]|0;do{if(E<<24>>24==0){z=(d[g+16|0]|0)+1|0;F=((z&2)+G|0)>>>(z>>>0)<<1;L=g+84|0;c[L>>2]=F;K=c[f+4>>2]|0;I=d[g+14|0]|0;if((F|0)<1){AN(K,16,1670888,(S=i,i=i+8|0,c[S>>2]=F,S)|0);i=S;U=M;break L2395}if((I|0)>=(F|0)){AN(K,16,1669112,(S=i,i=i+8|0,c[S>>2]=I,S)|0);i=S;U=M;break L2395}if((F|0)==0){aa=h;ab=J;ad=0}else{F=z&65535;I=1;do{b[g+262896+(I<<1)>>1]=F;I=I+1|0;ae=c[L>>2]|0;}while(I>>>0<=ae>>>0);aa=c[A>>2]|0;ab=c[N>>2]|0;ad=ae}I=aa-ab|0;F=ac(ad,z)|0;H=I-F|0;do{if((H|0)<0){v=g+262898|0;b[v>>1]=(b[v>>1]|0)-1&65535;v=g+262900|0;b[v>>1]=(b[v>>1]|0)-((H|0)<-1)&65535;af=ab&65535;ag=ad}else{if((I|0)==(F|0)){af=ab&65535;ag=ad;break}v=g+262896+(ad<<1)|0;b[v>>1]=(b[v>>1]|0)+1&65535;af=c[N>>2]&65535;ag=c[L>>2]|0}}while(0);b[g+262896>>1]=af;if((ag|0)==0){ah=0;break}else{ai=1;aj=af}while(1){F=g+262896+(ai<<1)|0;I=(b[F>>1]|0)+aj&65535;b[F>>1]=I;F=ai+1|0;H=c[L>>2]|0;if(F>>>0>H>>>0){ah=H;break}else{ai=F;aj=I}}}else{if((h*49|0)>>>0>(J*110|0)>>>0){L=J<<1;c[g+48>>2]=L;ak=1;al=L}else{c[g+48>>2]=h;ak=0;al=h}X=+(7-(E&255)|0);L=g+48|0;I=bR(+(X*+bu(+(+(al>>>0>>>0)/+(J>>>0>>>0)))))|0;F=I<<1;if((F|0)<1){AN(c[f+4>>2]|0,16,1676448,(S=i,i=i+8|0,c[S>>2]=F,S)|0);i=S;U=M;break L2395}H=o|0;b[H>>1]=0;z=o+2|0;v=c[N>>2]|0;K=c[L>>2]|0;W=+(v|0);V=+T(+(+(K|0)/W),+(1.0/+(F|0)));am=F-1|0;if((am|0)>0){an=v;ao=0;ap=W;while(1){W=V*ap;aq=bR(+W)|0;ar=ao+1|0;b[o+(ar<<1)>>1]=aq-an&65535;if((ar|0)<(am|0)){an=aq;ao=ar;ap=W}else{as=aq;break}}}else{as=v}ao=o+(F<<1)|0;b[ao>>1]=K-as&65535;b6(z|0,F|0,2,100);an=b[ao>>1]|0;ao=c[N>>2]&65535;b[H>>1]=ao;am=1;aq=ao;while(1){ao=o+(am<<1)|0;at=b[ao>>1]|0;if(at<<16>>16<1){R=1775;break}ar=at+aq&65535;b[ao>>1]=ar;ao=am+1|0;if((ao|0)>(F|0)){break}else{am=ao;aq=ar}}if((R|0)==1775){AN(c[f+4>>2]|0,16,1674776,(S=i,i=i+16|0,c[S>>2]=am,c[S+8>>2]=at<<16>>16,S)|0);i=S;U=M;break L2395}if((ak|0)==0){c[g+84>>2]=F;aq=d[g+14|0]|0;if((aq|0)<(F|0)){H=g+262896|0;z=I<<2|2;NN(H|0,C|0,z)|0;ah=F;break}else{AN(c[f+4>>2]|0,16,1669112,(S=i,i=i+8|0,c[S>>2]=aq,S)|0);i=S;U=M;break L2395}}ap=X*((a[g+16|0]|0)!=0?.7692307829856873:1.0);aq=bR(+(ap*+bu(+(+((c[A>>2]|0)>>>0>>>0)/+((c[L>>2]|0)>>>0>>>0)))))|0;z=aq<<1;H=p|0;K=p+2|0;v=c[L>>2]|0;ar=c[A>>2]|0;ap=+(v|0);V=+T(+(+(ar|0)/ap),+(1.0/+(z|0)));ao=z-1|0;if((ao|0)>0){au=v;av=0;W=ap;while(1){ap=V*W;aw=bR(+ap)|0;ax=av+1|0;b[p+(ax<<1)>>1]=aw-au&65535;if((ax|0)<(ao|0)){au=aw;av=ax;W=ap}else{ay=aw;break}}}else{ay=v}av=p+(z<<1)|0;b[av>>1]=ar-ay&65535;au=b[K>>1]|0;if((z|0)>1){ao=1;I=au;while(1){am=ao+1|0;aw=b[p+(am<<1)>>1]|0;ax=(aw|0)>(I|0)?I:aw;if((am|0)<(z|0)){ao=am;I=ax}else{az=ax;break}}}else{az=au}I=K;if((az|0)<(an|0)){b6(I|0,z|0,2,100);ao=b[K>>1]|0;ar=ao<<16>>16;v=an-ar|0;ax=b[av>>1]|0;am=(ax<<16>>16)-ar>>1;ar=(v|0)>(am|0)?am:v;b[K>>1]=ar+(ao&65535)&65535;b[av>>1]=(ax&65535)-ar&65535}b6(I|0,z|0,2,100);ar=c[L>>2]&65535;b[H>>1]=ar;L2475:do{if((z|0)>=1){ax=1;ao=ar;while(1){v=p+(ax<<1)|0;aA=b[v>>1]|0;if(aA<<16>>16<1){break}am=aA+ao&65535;b[v>>1]=am;v=ax+1|0;if((v|0)>(z|0)){break L2475}else{ax=v;ao=am}}AN(c[f+4>>2]|0,16,1672976,(S=i,i=i+16|0,c[S>>2]=ax,c[S+8>>2]=aA<<16>>16,S)|0);i=S;U=M;break L2395}}while(0);ar=z+F|0;H=g+84|0;c[H>>2]=ar;L=c[f+4>>2]|0;av=d[g+14|0]|0;if((ar|0)<1){AN(L,16,1670888,(S=i,i=i+8|0,c[S>>2]=ar,S)|0);i=S;U=M;break L2395}if((av|0)<(ar|0)){ar=g+262896|0;K=F|1;an=K<<1;NN(ar|0,C|0,an)|0;an=g+262896+(K<<1)|0;K=aq<<2;NN(an|0,I|0,K)|0;ah=c[H>>2]|0;break}else{AN(L,16,1669112,(S=i,i=i+8|0,c[S>>2]=av,S)|0);i=S;U=M;break L2395}}}while(0);J=g+84|0;E=d[g+14|0]|0;h=ah-E|0;c[g+262884>>2]=h;G=(h+1|0)>>>1;av=g+262880|0;c[av>>2]=G;L=g+263044|0;H=L;K=g+262896+(E<<1)|0;E=(h<<1)+2|0;NN(H|0,K|0,E)|0;E=b[g+263044+(h<<1)>>1]|0;K=E&65535;H=b[L>>1]|0;L=H&65535;c[r>>2]=K-L;c[s>>2]=L;if((E&65535)>64){AN(c[f+4>>2]|0,16,1698648,(S=i,i=i+8|0,c[S>>2]=K,S)|0);i=S;U=M;break}if((H&65535)>32){AN(c[f+4>>2]|0,16,1696640,(S=i,i=i+8|0,c[S>>2]=L,S)|0);i=S;U=M;break}K=g+262994|0;b[K>>1]=H;H=h&1;if((G|0)==0){aB=L}else{L=1;do{b[g+262994+(L<<1)>>1]=b[g+263044+((L<<1)-H<<1)>>1]|0;L=L+1|0;}while(L>>>0<=(c[av>>2]|0)>>>0);aB=c[s>>2]|0}L=g+17|0;W=+(d[L]|0);do{if((bR(+(W*+bu(+(+((c[A>>2]|0)>>>0>>>0)/+(aB>>>0>>>0)))))|0)<1){H=g+262888|0;c[H>>2]=1;aC=H}else{V=+(d[L]|0);H=bR(+(V*+bu(+(+((c[A>>2]|0)>>>0>>>0)/+((c[s>>2]|0)>>>0>>>0)))))|0;G=g+262888|0;c[G>>2]=H;if(H>>>0<=5){aC=G;break}AN(c[f+4>>2]|0,16,1694280,(S=i,i=i+8|0,c[S>>2]=H,S)|0);i=S;U=M;break L2395}}while(0);b[g+263142>>1]=b[K>>1]|0;A=c[aC>>2]|0;if((A|0)!=0){L=1;H=0;G=A;do{H=((((c[av>>2]|0)-H|0)>>>0)/((G+1-L|0)>>>0)|0)+H|0;b[g+263142+(L<<1)>>1]=b[g+262994+(H<<1)>>1]|0;L=L+1|0;G=c[aC>>2]|0;}while(L>>>0<=G>>>0)}G=c[N>>2]|0;L=c[s>>2]|0;H=c[w>>2]|0;av=((H>>1)+2048e3|0)/(H|0)|0;H=g+263216|0;c[H>>2]=0;if(av>>>0<((c[r>>2]|0)+L|0)>>>0){K=0;while(1){if((e[g+262896+(K<<1)>>1]|0)<(av|0)){K=K+1|0}else{aD=L;aE=G;aF=0;aG=K;aH=0;aI=G;break}}}else{aD=L;aE=G;aF=0;aG=c[J>>2]|0;aH=0;aI=G}while(1){K=aI-1+aE|0;av=0;A=aF;h=aG;while(1){if((h|0)!=(aG|0)){if(A>>>0<=(K-av|0)>>>0){break}}E=e[g+262896+(h<<1)>>1]|0;av=E+aI&1;A=E;h=h-1|0}if(aH>>>0>5){R=1827;break}h=A-aD|0;a[g+263220+aH|0]=(h|0)>0?h&255:0;h=c[H>>2]|0;a[g+263226+h|0]=(c[N>>2]|0)-av-(d[g+263220+h|0]|0)&255;h=c[H>>2]|0;if((a[g+263220+h|0]|0)==0){aJ=aD;aK=c[s>>2]|0;aL=h}else{K=h+1|0;c[H>>2]=K;aJ=A;aK=A;aL=K}if(((e[g+262896+(aG<<1)>>1]|0)-A|0)<3){aM=c[J>>2]|0}else{aM=aG}if((A|0)==((c[r>>2]|0)+(c[s>>2]|0)|0)){break}aD=aJ;aE=aK;aF=A;aG=aM;aH=aL;aI=c[N>>2]|0}if((R|0)==1827){AN(c[f+4>>2]|0,16,1690712,(S=i,i=i+8|0,c[S>>2]=aH,S)|0);i=S;U=M;break}do{if(aL>>>0>1){N=aL-1|0;if((d[g+263220+N|0]|0)>=3){break}c[H>>2]=N}}while(0);qc(g);c[g+87736>>2]=0;c[g+175384>>2]=0;break L2383}}while(0);AN(c[f+4>>2]|0,16,1701784,(S=i,i=i+1|0,i=i+7&-8,c[S>>2]=0,S)|0);i=S;c[g+4>>2]=0;c[s>>2]=32;c[r>>2]=0;c[g+102312>>2]=-1;c[g+14664>>2]=-1;NM(U|0,-1|0,6)}}while(0);U=g+4|0;if((c[U>>2]|0)==0){aN=D}else{aL=c[j>>2]|0;do{if((l|0)==1){aH=c[u>>2]|0;aI=c[B>>2]|0;aM=((aL|0)<(aI|0))+aL|0;if((d[aH+(aL>>>3)|0]&128>>>((aL&7)>>>0)|0)==0){aO=aM}else{aG=aM+8|0;aO=aI>>>0>aG>>>0?aG:aI}c[j>>2]=aO;aG=d[aH+(aO>>>3)|0]<<(aO&7)>>>7;c[j>>2]=((aO|0)<(aI|0))+aO;aI=aG&1;c[g+40>>2]=aI;aG=g+96|0;aH=(p9(f,g,q,aG)|0)!=0;do{if((aI|0)==0){if(aH){R=1896;break}aM=g+87744|0;if((p9(f,g,q,aM)|0)!=0){R=1896;break}aF=c[g+104>>2]|0;if((aF|0)>0){aK=c[u>>2]|0;aE=c[B>>2]|0;aJ=0;aD=c[j>>2]|0;do{w=d[aK+(aD>>>3)|0]<<(aD&7)>>>7;aD=((aD|0)<(aE|0))+aD|0;c[j>>2]=aD;a[g+120+aJ|0]=w&1;aJ=aJ+1|0;}while((aJ|0)<(aF|0))}aF=c[g+116>>2]|0;if((aF|0)>0){aJ=c[u>>2]|0;aD=c[B>>2]|0;aE=0;aK=c[j>>2]|0;do{w=d[aJ+(aK>>>3)|0]<<(aK&7)>>>7;aK=((aK|0)<(aD|0))+aK|0;c[j>>2]=aK;a[g+125+aE|0]=w&1;aE=aE+1|0;}while((aE|0)<(aF|0))}aF=c[g+87752>>2]|0;if((aF|0)>0){aE=c[u>>2]|0;aK=c[B>>2]|0;aD=0;aJ=c[j>>2]|0;do{w=d[aE+(aJ>>>3)|0]<<(aJ&7)>>>7;aJ=((aJ|0)<(aK|0))+aJ|0;c[j>>2]=aJ;a[g+87768+aD|0]=w&1;aD=aD+1|0;}while((aD|0)<(aF|0))}aF=c[g+87764>>2]|0;if((aF|0)>0){aD=c[u>>2]|0;aJ=c[B>>2]|0;aK=0;aE=c[j>>2]|0;do{w=d[aD+(aE>>>3)|0]<<(aE&7)>>>7;aE=((aE|0)<(aJ|0))+aE|0;c[j>>2]=aE;a[g+87773+aK|0]=w&1;aK=aK+1|0;}while((aK|0)<(aF|0))}aF=g+132|0;aK=g+127|0;a[aF]=a[aK]|0;a[aF+1|0]=a[aK+1|0]|0;a[aF+2|0]=a[aK+2|0]|0;a[aF+3|0]=a[aK+3|0]|0;a[aF+4|0]=a[aK+4|0]|0;aK=g+262888|0;do{if((c[aK>>2]|0)==0){aF=g+87780|0;aE=g+87775|0;a[aF]=a[aE]|0;a[aF+1|0]=a[aE+1|0]|0;a[aF+2|0]=a[aE+2|0]|0;a[aF+3|0]=a[aE+3|0]|0;a[aF+4|0]=a[aE+4|0]|0}else{aE=c[B>>2]|0;aF=c[u>>2]|0;aJ=0;aD=c[j>>2]|0;do{w=aF+(aD>>>3)|0;aC=(cV(d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0)|0)<<(aD&7)>>>30;w=aD+2|0;aD=aE>>>0>w>>>0?w:aE;c[j>>2]=aD;a[g+127+aJ|0]=aC&255;aJ=aJ+1|0;aP=c[aK>>2]|0;}while(aJ>>>0<aP>>>0);aJ=g+87780|0;A=g+87775|0;a[aJ]=a[A]|0;a[aJ+1|0]=a[A+1|0]|0;a[aJ+2|0]=a[A+2|0]|0;a[aJ+3|0]=a[A+3|0]|0;a[aJ+4|0]=a[A+4|0]|0;if((aP|0)==0){break}else{aQ=0;aR=aD}do{A=aF+(aR>>>3)|0;aJ=(cV(d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24|0)|0)<<(aR&7)>>>30;A=aR+2|0;aR=aE>>>0>A>>>0?A:aE;c[j>>2]=aR;a[g+87775+aQ|0]=aJ&255;aQ=aQ+1|0;}while(aQ>>>0<(c[aK>>2]|0)>>>0)}}while(0);qa(g,q,aG,0);qa(g,q,aM,1);qb(g,q,aG,0);qb(g,q,aM,1)}else{if(aH){R=1896;break}aK=g+87744|0;aE=g+87752|0;aF=c[aE>>2]|0;a[g+87756|0]=a[g+87756+aF|0]|0;a[g+175380|0]=a[g+175372+aF|0]|0;aD=g+102312|0;c[g+102308>>2]=-((c[aD>>2]|0)!=(aF|0)|0);aF=g+87757|0;aJ=g+109|0;a[aF]=a[aJ]|0;a[aF+1|0]=a[aJ+1|0]|0;a[aF+2|0]=a[aJ+2|0]|0;a[aF+3|0]=a[aJ+3|0]|0;a[aF+4|0]=a[aJ+4|0]|0;a[aF+5|0]=a[aJ+5|0]|0;aJ=g+87724|0;aF=g+175372|0;A=aJ|0;av=aJ+4|0;aJ=d[av]|d[av+1|0]<<8|d[av+2|0]<<16|d[av+3|0]<<24|0;av=aF|0;y=d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24|0;a[av]=y&255;y=y>>8;a[av+1|0]=y&255;y=y>>8;a[av+2|0]=y&255;y=y>>8;a[av+3|0]=y&255;av=aF+4|0;y=aJ;a[av]=y&255;y=y>>8;a[av+1|0]=y&255;y=y>>8;a[av+2|0]=y&255;y=y>>8;a[av+3|0]=y&255;av=g+175381|0;aJ=g+87733|0;a[av]=a[aJ]|0;a[av+1|0]=a[aJ+1|0]|0;a[av+2|0]=a[aJ+2|0]|0;aJ=c[g+104>>2]|0;c[aE>>2]=aJ;c[g+87836>>2]=c[g+188>>2];av=g+116|0;aF=c[av>>2]|0;A=g+87764|0;c[A>>2]=aF;c[aK>>2]=c[aG>>2];c[aD>>2]=c[g+14664>>2];if((aJ|0)>0){aD=c[u>>2]|0;aC=c[B>>2]|0;w=0;aB=c[j>>2]|0;do{ah=d[aD+(aB>>>3)|0]<<(aB&7)>>>7;aB=((aB|0)<(aC|0))+aB|0;c[j>>2]=aB;a[g+120+w|0]=ah&1;w=w+1|0;}while((w|0)<(aJ|0));aS=c[av>>2]|0}else{aS=aF}if((aS|0)>0){aJ=c[u>>2]|0;w=c[B>>2]|0;aB=0;aC=c[j>>2]|0;do{aD=d[aJ+(aC>>>3)|0]<<(aC&7)>>>7;aC=((aC|0)<(w|0))+aC|0;c[j>>2]=aC;a[g+125+aB|0]=aD&1;aB=aB+1|0;}while((aB|0)<(aS|0))}aB=c[aE>>2]|0;if((aB|0)>0){aC=c[u>>2]|0;w=c[B>>2]|0;aJ=0;aF=c[j>>2]|0;do{av=d[aC+(aF>>>3)|0]<<(aF&7)>>>7;aF=((aF|0)<(w|0))+aF|0;c[j>>2]=aF;a[g+87768+aJ|0]=av&1;aJ=aJ+1|0;}while((aJ|0)<(aB|0))}aB=c[A>>2]|0;if((aB|0)>0){aJ=c[u>>2]|0;aF=c[B>>2]|0;w=0;aC=c[j>>2]|0;do{aE=d[aJ+(aC>>>3)|0]<<(aC&7)>>>7;aC=((aC|0)<(aF|0))+aC|0;c[j>>2]=aC;a[g+87773+w|0]=aE&1;w=w+1|0;}while((w|0)<(aB|0))}aB=g+132|0;w=g+127|0;a[aB]=a[w]|0;a[aB+1|0]=a[w+1|0]|0;a[aB+2|0]=a[w+2|0]|0;a[aB+3|0]=a[w+3|0]|0;a[aB+4|0]=a[w+4|0]|0;aB=g+262888|0;if((c[aB>>2]|0)!=0){aC=c[B>>2]|0;aF=c[u>>2]|0;aJ=0;A=c[j>>2]|0;do{aE=aF+(A>>>3)|0;av=(cV(d[aE]|d[aE+1|0]<<8|d[aE+2|0]<<16|d[aE+3|0]<<24|0)|0)<<(A&7)>>>30;aE=A+2|0;A=aC>>>0>aE>>>0?aE:aC;c[j>>2]=A;a[g+127+aJ|0]=av&255;aJ=aJ+1|0;}while(aJ>>>0<(c[aB>>2]|0)>>>0)}aB=g+87780|0;aJ=g+87775|0;a[aB]=a[aJ]|0;a[aB+1|0]=a[aJ+1|0]|0;a[aB+2|0]=a[aJ+2|0]|0;a[aB+3|0]=a[aJ+3|0]|0;a[aB+4|0]=a[aJ+4|0]|0;a[aJ]=a[w]|0;a[aJ+1|0]=a[w+1|0]|0;a[aJ+2|0]=a[w+2|0]|0;a[aJ+3|0]=a[w+3|0]|0;a[aJ+4|0]=a[w+4|0]|0;qa(g,q,aG,0);qb(g,q,aG,0);qa(g,q,aK,1);qb(g,q,aK,1)}}while(0);if((R|0)==1896){c[U>>2]=0;c[s>>2]=32;c[r>>2]=0;c[g+102312>>2]=-1;c[g+14664>>2]=-1;NM(g+12|0,-1|0,6);break}aG=c[j>>2]|0;aH=c[u>>2]|0;aI=d[aH+(aG>>>3)|0]<<(aG&7)>>>7;aJ=c[B>>2]|0;aB=((aG|0)<(aJ|0))+aG|0;c[j>>2]=aB;aG=aI&1;c[g+100>>2]=aG;do{if((aG|0)==0){aT=aB}else{aI=c[g+262884>>2]|0;if((aI|0)>0){aU=0;aV=aB}else{aT=aB;break}while(1){A=d[aH+(aV>>>3)|0]<<(aV&7)>>>7;aC=((aV|0)<(aJ|0))+aV|0;c[j>>2]=aC;a[g+137+aU|0]=A&1;A=aU+1|0;if((A|0)<(aI|0)){aU=A;aV=aC}else{aT=aC;break}}}}while(0);aB=d[aH+(aT>>>3)|0]<<(aT&7)>>>7;aG=((aT|0)<(aJ|0))+aT|0;c[j>>2]=aG;aI=aB&1;c[g+87748>>2]=aI;if((aI|0)==0){aW=aG;aX=aH;aY=aJ;R=1898;break}aI=c[g+262884>>2]|0;if((aI|0)>0){aZ=0;a_=aG}else{aW=aG;aX=aH;aY=aJ;R=1898;break}while(1){aG=d[aH+(a_>>>3)|0]<<(a_&7)>>>7;aB=((a_|0)<(aJ|0))+a_|0;c[j>>2]=aB;a[g+87785+aZ|0]=aG&1;aG=aZ+1|0;if((aG|0)<(aI|0)){aZ=aG;a_=aB}else{aW=aB;aX=aH;aY=aJ;R=1898;break}}}else if((l|0)==2|(l|0)==0){aJ=c[B>>2]|0;aH=((aL|0)<(aJ|0))+aL|0;if((d[(c[u>>2]|0)+(aL>>>3)|0]&128>>>((aL&7)>>>0)|0)==0){a$=aH}else{aI=aH+4|0;a$=aJ>>>0>aI>>>0?aI:aJ}c[j>>2]=a$;aJ=g+96|0;if((p9(f,g,q,aJ)|0)!=0){c[U>>2]=0;c[s>>2]=32;c[r>>2]=0;c[g+102312>>2]=-1;c[g+14664>>2]=-1;NM(g+12|0,-1|0,6);break}aI=c[g+104>>2]|0;if((aI|0)>0){aH=c[u>>2]|0;aB=c[B>>2]|0;aG=0;aK=c[j>>2]|0;do{w=d[aH+(aK>>>3)|0]<<(aK&7)>>>7;aK=((aK|0)<(aB|0))+aK|0;c[j>>2]=aK;a[g+120+aG|0]=w&1;aG=aG+1|0;}while((aG|0)<(aI|0))}aI=c[g+116>>2]|0;if((aI|0)>0){aG=c[u>>2]|0;aK=c[B>>2]|0;aB=0;aH=c[j>>2]|0;do{w=d[aG+(aH>>>3)|0]<<(aH&7)>>>7;aH=((aH|0)<(aK|0))+aH|0;c[j>>2]=aH;a[g+125+aB|0]=w&1;aB=aB+1|0;}while((aB|0)<(aI|0))}aI=g+132|0;aB=g+127|0;a[aI]=a[aB]|0;a[aI+1|0]=a[aB+1|0]|0;a[aI+2|0]=a[aB+2|0]|0;a[aI+3|0]=a[aB+3|0]|0;a[aI+4|0]=a[aB+4|0]|0;aB=g+262888|0;if((c[aB>>2]|0)!=0){aI=c[B>>2]|0;aH=c[u>>2]|0;aK=0;aG=c[j>>2]|0;do{w=aH+(aG>>>3)|0;aC=(cV(d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0)|0)<<(aG&7)>>>30;w=aG+2|0;aG=aI>>>0>w>>>0?w:aI;c[j>>2]=aG;a[g+127+aK|0]=aC&255;aK=aK+1|0;}while(aK>>>0<(c[aB>>2]|0)>>>0)}qa(g,q,aJ,0);qb(g,q,aJ,0);aB=c[j>>2]|0;aK=c[u>>2]|0;aG=d[aK+(aB>>>3)|0]<<(aB&7)>>>7;aI=c[B>>2]|0;aH=((aB|0)<(aI|0))+aB|0;c[j>>2]=aH;aB=aG&1;c[g+100>>2]=aB;if((aB|0)==0){aW=aH;aX=aK;aY=aI;R=1898;break}aB=c[g+262884>>2]|0;if((aB|0)>0){a0=0;a1=aH}else{aW=aH;aX=aK;aY=aI;R=1898;break}while(1){aH=d[aK+(a1>>>3)|0]<<(a1&7)>>>7;aG=((a1|0)<(aI|0))+a1|0;c[j>>2]=aG;a[g+137+a0|0]=aH&1;aH=a0+1|0;if((aH|0)<(aB|0)){a0=aH;a1=aG}else{aW=aG;aX=aK;aY=aI;R=1898;break}}}else{AN(c[f+4>>2]|0,16,1726448,(S=i,i=i+8|0,c[S>>2]=l,S)|0);i=S;c[U>>2]=0;c[s>>2]=32;c[r>>2]=0;c[g+102312>>2]=-1;c[g+14664>>2]=-1;NM(g+12|0,-1|0,6)}}while(0);L2643:do{if((R|0)==1898){r=d[aX+(aW>>>3)|0]|0;s=((aW|0)<(aY|0))+aW|0;c[j>>2]=s;if((r&128>>>((aW&7)>>>0)|0)==0){break}r=aX+(s>>>3)|0;U=(cV(d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0)|0)<<(s&7)>>>28;r=s+4|0;s=aY>>>0>r>>>0?r:aY;c[j>>2]=s;if((U|0)==15){r=aX+(s>>>3)|0;l=(cV(d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0)|0)<<(s&7)>>>24;r=s+8|0;a1=aY>>>0>r>>>0?r:aY;c[j>>2]=a1;a2=l+15|0;a3=a1}else{a2=U;a3=s}s=a2<<3;L2649:do{if((s|0)>7){U=f+4|0;a1=f+6448|0;l=g+175392|0;r=s;a0=a3;a$=aY;a_=aX;while(1){a4=r-2|0;aZ=a_+(a0>>>3)|0;aT=(cV(d[aZ]|d[aZ+1|0]<<8|d[aZ+2|0]<<16|d[aZ+3|0]<<24|0)|0)<<(a0&7)>>>30;aZ=a0+2|0;a5=a$>>>0>aZ>>>0?aZ:a$;c[j>>2]=a5;if((aT|0)==0){R=1910;break}else if((aT|0)!=2){R=1912;break}a6=c[U>>2]|0;if((c[a1>>2]|0)==0){R=1905;break}aT=a4-(rt(a6,q,l,a4)|0)|0;if((aT|0)<=7){a7=aT;break L2649}r=aT;a0=c[j>>2]|0;a$=c[B>>2]|0;a_=c[u>>2]|0}do{if((R|0)==1905){AN(a6,16,1718160,(S=i,i=i+1|0,i=i+7&-8,c[S>>2]=0,S)|0);i=S;a0=c[j>>2]|0;l=-a0|0;a1=(c[B>>2]|0)-a0|0;if((a4|0)<(l|0)){a8=l}else{a8=(a1|0)<(a4|0)?a1:a4}c[j>>2]=a8+a0;break L2643}else if((R|0)==1910){if((a4|0)>16){R=1912;break}a0=a_+(a5>>>3)|0;if(((cV(d[a0]|d[a0+1|0]<<8|d[a0+2|0]<<16|d[a0+3|0]<<24|0)|0)<<(a5&7)>>>((34-r|0)>>>0)|0)==0){a9=a5;ba=a$}else{R=1912}}}while(0);if((R|0)==1912){AS(c[U>>2]|0,1714168,(S=i,i=i+1|0,i=i+7&-8,c[S>>2]=0,S)|0);i=S;a9=c[j>>2]|0;ba=c[B>>2]|0}a$=-a9|0;r=ba-a9|0;if((a4|0)<(a$|0)){bb=a$}else{bb=(r|0)<(a4|0)?r:a4}c[j>>2]=bb+a9;break L2643}else{a7=s}}while(0);if((a7|0)<0){AN(c[f+4>>2]|0,16,1722392,(S=i,i=i+1|0,i=i+7&-8,c[S>>2]=0,S)|0);i=S}if((a7|0)<=0){break}s=c[B>>2]|0;r=(c[j>>2]|0)+a7|0;c[j>>2]=s>>>0>r>>>0?r:s}}while(0);aN=D-aL+(c[j>>2]|0)|0}j=(aN+4+(4-aN&7)|0)>>>3;if(j>>>0<=k>>>0){i=m;return k|0}AN(c[f+4>>2]|0,16,1650584,(S=i,i=i+16|0,c[S>>2]=k,c[S+8>>2]=j,S)|0);i=S;i=m;return k|0}function p3(f,h,j,k,l){f=f|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0.0,s=0.0,t=0,u=0,v=0,w=0,x=0,y=0,z=0.0,A=0,B=0.0,C=0,D=0.0,E=0,F=0,G=0,H=0.0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0.0,ao=0.0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0.0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0.0,bk=0.0,bl=0.0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0.0,bs=0.0,bt=0.0,bu=0,bv=0,bw=0;m=i;n=(c[f+6436>>2]|0)<(c[h>>2]|0)|0;o=(j|0)==1;j=o?2:1;p=h+80|0;if((c[p>>2]|0)==0){c[h+64>>2]=c[h+68>>2];c[h+72>>2]=c[h+76>>2]}else{c[p>>2]=0}p=h+4|0;L2685:do{if((c[p>>2]|0)!=0){do{if(o){if((c[h+40>>2]|0)==0){break}q=(c[h+188>>2]|0)!=0;r=q?1.0:.5;s=q?12.0:24.0;q=h+104|0;t=c[q>>2]|0;if((t|0)!=0){u=1;v=t;while(1){t=h+108+u|0;if((c[h+262880+(d[t]<<2)>>2]|0)==0){w=v}else{x=0;do{y=h+86512+(u*192|0)+(x<<2)|0;z=+b4(+(r*+g[y>>2]+7.0));A=h+174160+(u*192|0)+(x<<2)|0;B=+b4(+(r*(s- +g[A>>2])));if(z>1.0e20){AN(0,16,1730944,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;D=1.0}else{D=z}z=D/(B+1.0);g[y>>2]=z;g[A>>2]=B*z;x=x+1|0;}while(x>>>0<(c[h+262880+(d[t]<<2)>>2]|0)>>>0);w=c[q>>2]|0}t=u+1|0;if(t>>>0>w>>>0){break}else{u=t;v=w}}}v=h+116|0;u=c[v>>2]|0;if((u|0)==0){break L2685}q=h+262888|0;t=1;x=c[q>>2]|0;A=u;while(1){if((x|0)==0){E=0;F=A}else{u=0;do{y=h+87664+(t*20|0)+(u<<2)|0;s=+b4(+(6.0- +g[y>>2]+1.0));G=h+175312+(t*20|0)+(u<<2)|0;r=+b4(+(12.0- +g[G>>2]));if(s>1.0e20){AN(0,16,1730944,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;H=1.0}else{H=s}s=H/(r+1.0);g[y>>2]=s;g[G>>2]=r*s;u=u+1|0;I=c[q>>2]|0;}while(u>>>0<I>>>0);E=I;F=c[v>>2]|0}u=t+1|0;if(u>>>0>F>>>0){break L2685}else{t=u;x=E;A=F}}}}while(0);A=h+262888|0;x=0;do{s=(c[h+96+(x*87648|0)+92>>2]|0)!=0?1.0:.5;t=h+96+(x*87648|0)+8|0;v=c[t>>2]|0;if((v|0)!=0){q=1;u=v;while(1){v=h+96+(x*87648|0)+12+q|0;if((c[h+262880+(d[v]<<2)>>2]|0)==0){J=u}else{G=0;do{y=h+96+(x*87648|0)+86416+(q*192|0)+(G<<2)|0;r=+b4(+(s*+g[y>>2]+6.0));g[y>>2]=r;if(r>1.0e20){AN(0,16,1730944,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;g[y>>2]=1.0}G=G+1|0;}while(G>>>0<(c[h+262880+(d[v]<<2)>>2]|0)>>>0);J=c[t>>2]|0}v=q+1|0;if(v>>>0>J>>>0){break}else{q=v;u=J}}}u=h+96+(x*87648|0)+20|0;q=c[u>>2]|0;if((q|0)!=0){t=1;v=c[A>>2]|0;G=q;while(1){if((v|0)==0){K=0;L=G}else{q=0;do{y=h+96+(x*87648|0)+87568+(t*20|0)+(q<<2)|0;g[y>>2]=+b4(+(6.0- +g[y>>2]));q=q+1|0;M=c[A>>2]|0;}while(q>>>0<M>>>0);K=M;L=c[u>>2]|0}q=t+1|0;if(q>>>0>L>>>0){break}else{t=q;v=K;G=L}}}x=x+1|0;}while((x|0)<(j|0))}}while(0);L=h+343584|0;K=h+343712|0;M=h+342304|0;J=f+5352|0;F=K|0;E=h+343724|0;I=h+343628|0;w=h+342560|0;o=h+343728|0;x=h+343768|0;A=h+263232|0;G=h+343776|0;v=h+343780|0;t=h+332864|0;u=h+333376|0;q=h+44|0;y=h+262888|0;N=h+273472|0;O=h+68|0;P=h+263216|0;Q=h+76|0;R=h+32|0;T=h+263044|0;U=h+262994|0;V=h+343716|0;W=h+343772|0;X=h+262892|0;Y=h+28|0;Z=h+263154|0;_=h+262884|0;$=h+263142|0;aa=f+4|0;ab=h+102312|0;ad=h+14664|0;ae=h+12|0;af=h+343744|0;ag=0;do{ah=(ag|0)!=0?l:k;ai=h+96+(ag*87648|0)+9312|0;aj=h+96+(ag*87648|0)+30976|0;ak=c[aj>>2]|0;al=ai;am=h+96+(ag*87648|0)+13408|0;NN(al|0,am|0,1152)|0;am=h+96+(ag*87648|0)+10464|0;al=ah;NN(am|0,al|0,4096)|0;al=ai;ai=0;while(1){dq[c[J>>2]&127](M,2162584,al,320);c_[c[F>>2]&127](M);c_[c[E>>2]&127](M);dm[c[I>>2]&127](L,M,w);c$[c[o>>2]&127](h+96+(ag*87648|0)+14592+(ak<<13)+(ai<<8)|0,M);am=ai+1|0;if((am|0)<32){al=al+128|0;ai=am}else{break}}ai=h+96+(ag*87648|0)|0;c4[c[x>>2]&63](f,h,A,h+96+(ag*87648|0)+14592|0,c[aj>>2]|0)|0;c[aj>>2]=c[aj>>2]^1;L2744:do{if((c[p>>2]|0)!=0){cX[c[v>>2]&255](K,t,u,A,c[q>>2]|0);if((c[y>>2]|0)!=0){al=0;do{ak=d[h+96+(ag*87648|0)+31+al|0]|0;if(((d[h+96+(ag*87648|0)+36+al|0]|0)+ak|0)==1){an=.6000000238418579}else{an=+g[80464+(ak<<2)>>2]}ak=h+96+(ag*87648|0)+14572+(al<<2)|0;H=+g[ak>>2];if(an<H){ao=an*.75+H*.25}else{ao=an*.90625+H*.09375}g[ak>>2]=ao<.015625?0.0:ao;al=al+1|0;}while(al>>>0<(c[y>>2]|0)>>>0)}al=h+96+(ag*87648|0)+87628|0;ak=c[O>>2]|0;am=c[P>>2]|0;do{if((am|0)==0){ap=ak;aq=ak;ar=1986}else{ah=(c[h+96+(ag*87648|0)+8>>2]|0)+(h+96+(ag*87648|0)+87628)|0;as=0;at=0;au=ak;av=am;L2759:while(1){aw=h+263220+as|0;if((a[aw]|0)==0){ax=at;ay=au;az=av}else{aA=h+263226+as|0;aB=0;aC=at;aD=au;while(1){aE=(d[aA]|0)+aB|0;aF=c[y>>2]|0;aG=aC;while(1){if(aG>>>0>aF>>>0){break}if((aD|0)<(e[h+263142+(aG<<1)>>1]|0)){break}else{aG=aG+1|0}}aH=aG-1|0;if((aH|0)<0){ar=1981;break L2759}dl[c[af>>2]&3](h+273472+(aD*320|0)+16|0,h+263232+(aE*320|0)+16|0,h+332864+(aE<<3)|0,h+333376+(aE<<3)|0,+g[h+96+(ag*87648|0)+14572+(aH<<2)>>2],d[al]<<1,d[ah]<<1);aF=aB+1|0;aI=aD+1|0;if((aF|0)<(d[aw]|0)){aB=aF;aC=aH;aD=aI}else{break}}ax=aH;ay=aI;az=c[P>>2]|0}aC=as+1|0;if(aC>>>0<az>>>0){as=aC;at=ax;au=ay;av=az}else{ar=1985;break}}if((ar|0)==1981){ar=0;AN(c[aa>>2]|0,16,1735800,(C=i,i=i+8|0,c[C>>2]=aD,C)|0);i=C;break}else if((ar|0)==1985){ar=0;ap=ay;aq=c[O>>2]|0;ar=1986;break}}}while(0);do{if((ar|0)==1986){ar=0;al=(c[Q>>2]|0)+aq|0;if(ap>>>0>=al>>>0){break}NM(h+273472+(ap*320|0)|0,0,(al-ap|0)*320|0|0)}}while(0);al=h+96+(ag*87648|0)+14564|0;NM(h+96+(ag*87648|0)+86080|0,0,336);am=h+96+(ag*87648|0)+8|0;L2779:do{if((c[am>>2]|0)==0){aJ=0}else{ak=h+96+(ag*87648|0)+20|0;av=h+96+(ag*87648|0)+87638|0;au=h+96+(ag*87648|0)+4|0;at=h+96+(ag*87648|0)+14568|0;as=0;while(1){ah=as+1|0;aC=a[h+96+(ag*87648|0)+12+ah|0]|0;aB=c[h+262880+((aC&255)<<2)>>2]|0;aw=aC<<24>>24==0?U:T;aC=b[aw>>1]|0;if((c[O>>2]|0)!=(aC&65535|0)){break}aA=(aB|0)==0;if(!aA){aF=0;aK=aC;while(1){aC=aF+1|0;aL=aw+(aC<<1)|0;aM=b[aL>>1]|0;if((aK&65535)<(aM&65535)){aN=h+96+(ag*87648|0)+86416+(ah*192|0)+(aF<<2)|0;aO=aK&65535;while(1){g[h+333888+(as*192|0)+(aO-(c[O>>2]|0)<<2)>>2]=+g[aN>>2];aP=aO+1|0;aQ=b[aL>>1]|0;if((aP|0)<(aQ&65535|0)){aO=aP}else{aR=aQ;break}}}else{aR=aM}if(aC>>>0<aB>>>0){aF=aC;aK=aR}else{break}}}if((c[ak>>2]|0)>>>0>1){aS=(d[h+96+(ag*87648|0)+87628+as|0]|0)>=(d[av]|0)?2:1}else{aS=1}aK=c[y>>2]|0;if((aK|0)!=0){aF=0;aO=b[$>>1]|0;aL=aK;while(1){aK=aF+1|0;aN=h+263142+(aK<<1)|0;aE=b[aN>>1]|0;if((aO&65535)<(aE&65535)){aG=h+96+(ag*87648|0)+87568+(aS*20|0)+(aF<<2)|0;aQ=aO&65535;do{g[h+335232+(as*192|0)+(aQ-(c[O>>2]|0)<<2)>>2]=+g[aG>>2];aQ=aQ+1|0;aT=b[aN>>1]|0;}while((aQ|0)<(aT&65535|0));aU=aT;aV=c[y>>2]|0}else{aU=aE;aV=aL}if(aK>>>0<aV>>>0){aF=aK;aO=aU;aL=aV}else{break}}}aL=c[_>>2]|0;if((aL|0)!=0){aO=0;aF=aL;while(1){if((c[au>>2]|0)==0){aW=aF;aX=aO+1|0}else{aL=aO+1|0;aQ=((e[h+263044+(aL<<1)>>1]|0)+(e[h+263044+(aO<<1)>>1]|0)|0)>>>1;aN=c[O>>2]|0;if((as|0)<(c[at>>2]|0)){aY=(a[aQ-aN+(h+96+(ag*87648|0)+86032)|0]|0)==1|0}else{aY=1}a[aQ-aN+(h+96+(ag*87648|0)+86032+(ah*48|0))|0]=a[h+96+(ag*87648|0)+41+aO|0]&(-aY&255);aW=c[_>>2]|0;aX=aL}if(aX>>>0<aW>>>0){aO=aX;aF=aW}else{break}}}if(!aA){aF=0;do{aO=e[aw+(aF<<1)>>1]|0;aF=aF+1|0;aL=e[aw+(aF<<1)>>1]|0;aN=c[O>>2]|0;aQ=aO;while(1){if((aQ|0)>=(aL|0)){aZ=0;break}if((a[aQ-aN+(h+96+(ag*87648|0)+86032+(ah*48|0))|0]|0)==0){aQ=aQ+1|0}else{aZ=1;break}}NM(aO-aN+(h+336576+(as*48|0))|0,aZ|0,aL-aO|0);}while(aF>>>0<aB>>>0)}aB=c[am>>2]|0;if(ah>>>0<aB>>>0){as=ah}else{aJ=aB;break L2779}}AN(c[aa>>2]|0,16,1740208,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;c[p>>2]=0;c[O>>2]=32;c[Q>>2]=0;c[ab>>2]=-1;c[ad>>2]=-1;NM(ae|0,-1|0,6);break L2744}}while(0);as=h+96+(ag*87648|0)+86032|0;at=h+96+(ag*87648|0)+86032+(aJ*48|0)|0;NN(as|0,at|0,48)|0;at=c[O>>2]|0;as=(aJ|0)==0;do{if((c[R>>2]|0)==0){if(as){break}else{a_=0;a$=aJ}while(1){au=a_+1|0;av=d[h+96+(ag*87648|0)+87628+au|0]|0;ak=d[h+96+(ag*87648|0)+87628+a_|0]|0;aB=av-ak<<1;aF=ak<<1;ak=aF+2|0;aw=h+96+(ag*87648|0)+12+au|0;aA=a[aw]|0;aQ=aA<<24>>24==0?U:T;if((c[h+262880+((aA&255)<<2)>>2]|0)==0){a0=a$}else{aA=(av<<1)-aF|0;aF=0;av=b[aQ>>1]|0;while(1){aK=aF+1|0;aE=aQ+(aK<<1)|0;aG=b[aE>>1]|0;aC=aQ+(aF<<1)|0;aM=av&65535;aP=ac(aB,(aG&65535)-aM|0)|0;if((av&65535)<(aG&65535)){a1=aM;H=0.0;do{H=H+ +dh[c[V>>2]&3](h+273472+(a1*320|0)+(ak<<3)|0,aA);a1=a1+1|0;a2=b[aE>>1]|0;}while((a1|0)<(a2&65535|0));a3=H;a4=b[aC>>1]|0;a5=a2}else{a3=0.0;a4=av;a5=aG}D=a3/+(aP|0);if((a4&65535)<(a5&65535)){a1=a4&65535;while(1){g[h+336912+(a_*192|0)+(a1-at<<2)>>2]=D;aO=a1+1|0;aL=b[aE>>1]|0;if((aO|0)<(aL&65535|0)){a1=aO}else{a6=aL;break}}}else{a6=a5}if(aK>>>0<(c[h+262880+(d[aw]<<2)>>2]|0)>>>0){aF=aK;av=a6}else{break}}a0=c[am>>2]|0}if(au>>>0<a0>>>0){a_=au;a$=a0}else{a7=a0;ar=2039;break}}}else{if(as){break}av=0;aF=c[Q>>2]|0;aw=aJ;while(1){aA=av+1|0;ak=d[h+96+(ag*87648|0)+87628+aA|0]|0;aB=d[h+96+(ag*87648|0)+87628+av|0]|0;D=.5/+(ak-aB|0);aQ=aB<<1;aB=aQ+2|0;if((aF|0)==0){a8=0;a9=aw}else{ah=(ak<<1)-aQ|0;aQ=0;do{g[h+336912+(av*192|0)+(aQ<<2)>>2]=D*+dh[c[V>>2]&3](h+273472+((aQ+at|0)*320|0)+(aB<<3)|0,ah);aQ=aQ+1|0;ba=c[Q>>2]|0;}while(aQ>>>0<ba>>>0);a8=ba;a9=c[am>>2]|0}if(aA>>>0<a9>>>0){av=aA;aF=a8;aw=a9}else{a7=a9;ar=2039;break}}}}while(0);do{if((ar|0)==2039){ar=0;if((a7|0)==0){break}at=h+96+(ag*87648|0)+14568|0;as=0;aw=a7;while(1){if((as|0)==(c[at>>2]|0)){bb=0}else{bb=(as|0)!=(c[al>>2]|0)}if((c[X>>2]|0)==0){bc=aw;bd=as+1|0}else{aF=as+1|0;D=+(bb&1|0);av=0;aQ=b[Z>>1]|0;ah=c[O>>2]|0;while(1){aB=h+263154+(av<<1)|0;au=(aQ&65535)-ah|0;ak=av+1|0;a1=h+263154+(ak<<1)|0;aE=b[a1>>1]|0;if(au>>>0<((aE&65535)-ah|0)>>>0){aP=au;do{H=+g[h+333888+(as*192|0)+(aP<<2)>>2];s=+g[h+335232+(as*192|0)+(aP<<2)>>2];r=s+1.0;z=H/r;g[h+338256+(as*192|0)+(aP<<2)>>2]=+S(+(s*z));g[h+339600+(as*192|0)+(aP<<2)>>2]=+S(+(z*+(d[h+96+(ag*87648|0)+86032+(aF*48|0)+aP|0]|0)));if((a[h+336576+(as*48|0)+aP|0]|0)==0){g[h+340944+(as*192|0)+(aP<<2)>>2]=+S(+(H/((D*s+1.0)*(+g[h+336912+(as*192|0)+(aP<<2)>>2]+1.0))))}else{g[h+340944+(as*192|0)+(aP<<2)>>2]=+S(+(H*s/(r*(+g[h+336912+(as*192|0)+(aP<<2)>>2]+1.0))))}aP=aP+1|0;be=b[a1>>1]|0;bf=c[O>>2]|0;}while(aP>>>0<((be&65535)-bf|0)>>>0);bg=bf;bh=b[aB>>1]|0;bi=be}else{bg=ah;bh=aQ;bi=aE}aP=(bh&65535)-bg|0;aK=(bi&65535)-bg|0;au=aP>>>0<aK>>>0;if(au){aG=aP;r=0.0;s=0.0;do{s=s+ +g[h+333888+(as*192|0)+(aG<<2)>>2];r=r+ +g[h+336912+(as*192|0)+(aG<<2)>>2];aG=aG+1|0;}while(aG>>>0<aK>>>0);bj=r+1.1920928955078125e-7;bk=s+1.1920928955078125e-7}else{bj=1.1920928955078125e-7;bk=1.1920928955078125e-7}H=+g[80448+(c[Y>>2]<<2)>>2];z=H*+S(+(bk/bj));H=z<1.0e5?z:1.0e5;if(au){aK=aP;do{aG=h+338256+(as*192|0)+(aK<<2)|0;z=+g[aG>>2];aE=h+340944+(as*192|0)+(aK<<2)|0;B=+g[aE>>2];bl=H*z/B;g[aG>>2]=z>bl?bl:z;g[aE>>2]=B>H?H:B;aK=aK+1|0;bm=b[a1>>1]|0;bn=c[O>>2]|0;}while(aK>>>0<((bm&65535)-bn|0)>>>0);bo=bn;bp=b[aB>>1]|0;bq=bm}else{bo=bg;bp=bh;bq=bi}aK=(bp&65535)-bo|0;aP=(bq&65535)-bo|0;au=aK>>>0<aP>>>0;if(au){if(bb){aE=aK;H=0.0;s=0.0;while(1){r=s+ +g[h+333888+(as*192|0)+(aE<<2)>>2];B=+g[h+340944+(as*192|0)+(aE<<2)>>2];z=+g[h+339600+(as*192|0)+(aE<<2)>>2];bl=+g[h+338256+(as*192|0)+(aE<<2)>>2];br=H+(B*+g[h+336912+(as*192|0)+(aE<<2)>>2]*B+z*z+bl*bl*+(z==0.0|0));aG=aE+1|0;if(aG>>>0<aP>>>0){aE=aG;H=br;s=r}else{bs=br;bt=r;break}}}else{aE=aK;s=0.0;H=0.0;while(1){r=H+ +g[h+333888+(as*192|0)+(aE<<2)>>2];br=+g[h+340944+(as*192|0)+(aE<<2)>>2];z=+g[h+339600+(as*192|0)+(aE<<2)>>2];bl=+g[h+338256+(as*192|0)+(aE<<2)>>2];B=s+(br*+g[h+336912+(as*192|0)+(aE<<2)>>2]*br+z*z+bl*bl*0.0);aB=aE+1|0;if(aB>>>0<aP>>>0){aE=aB;s=B;H=r}else{bs=B;bt=r;break}}}}else{bs=0.0;bt=0.0}H=+S(+((bt+1.1920928955078125e-7)/(bs+1.1920928955078125e-7)));s=H<1.5848932266235352?H:1.5848932266235352;if(au){aE=aK;while(1){aP=h+340944+(as*192|0)+(aE<<2)|0;g[aP>>2]=s*+g[aP>>2];aP=h+338256+(as*192|0)+(aE<<2)|0;g[aP>>2]=s*+g[aP>>2];aP=h+339600+(as*192|0)+(aE<<2)|0;g[aP>>2]=s*+g[aP>>2];aP=aE+1|0;aB=b[a1>>1]|0;aG=c[O>>2]|0;if(aP>>>0<((aB&65535)-aG|0)>>>0){aE=aP}else{bu=aB;bv=aG;break}}}else{bu=bq;bv=bo}if(ak>>>0<(c[X>>2]|0)>>>0){av=ak;aQ=bu;ah=bv}else{break}}bc=c[am>>2]|0;bd=aF}if(bd>>>0<bc>>>0){as=bd;aw=bc}else{break}}}}while(0);cX[c[W>>2]&255](h+96+(ag*87648|0)+30992+((c[aj>>2]|0)*19456|0)|0,N,h,ai,al)}}while(0);ai=c[aj>>2]|0;c0[c[G>>2]&15](h,h+293952+(ag*19456|0)|0,h+96+(ag*87648|0)+30992+((1-ai|0)*19456|0)|0,h+96+(ag*87648|0)+30992+(ai*19456|0)|0,A,ag)|0;ag=ag+1|0;}while((ag|0)<(j|0));do{if((c[f+6448>>2]|0)==1){ag=h+175392|0;if((c[ag>>2]|0)==0){A=h+313408|0;G=h+293952|0;NN(A|0,G|0,19456)|0;bw=2;break}else{G=c[aa>>2]|0;A=h+293952|0;N=h+313408|0;W=(c[Q>>2]|0)+(c[O>>2]|0)|0;ru(G,ag,A,N,W)|0;bw=2;break}}else{bw=j}}while(0);j=h+343648|0;O=f+5328|0;f=h+342304|0;p4(j,K,O,k,h+293952|0,f,h+192|0,h+14656|0,n);if((bw|0)!=2){i=m;return}p4(j,K,O,l,h+313408|0,f,h+87840|0,h+102304|0,n);i=m;return}function p4(a,b,d,e,f,h,i,j,k){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;l=(k|0)!=0;m=l?2162584:77752;n=128>>>(k>>>0);o=1152>>>(k>>>0);p=2304-o|0;q=i+(p<<2)|0;r=i;s=o<<2;o=d|0;t=64>>>(k>>>0);u=d+20|0;d=192>>>(k>>>0);v=m+(t<<2)|0;w=256>>>(k>>>0);x=m+(n<<2)|0;y=448>>>(k>>>0);z=m+(d<<2)|0;A=512>>>(k>>>0);B=m+(w<<2)|0;C=704>>>(k>>>0);D=m+(320>>>(k>>>0)<<2)|0;E=768>>>(k>>>0);F=m+(384>>>(k>>>0)<<2)|0;G=960>>>(k>>>0);H=m+(y<<2)|0;I=1024>>>(k>>>0);J=m+(A<<2)|0;K=1216>>>(k>>>0);L=m+(576>>>(k>>>0)<<2)|0;k=b+8|0;M=a+44|0;N=h|0;O=h+256|0;h=b+24|0;P=b+20|0;if(l){Q=0;R=e}else{l=0;b=e;while(1){e=c[j>>2]|0;if((e|0)<(n|0)){NN(q|0,r|0,s)|0;S=p}else{S=e}e=S-n|0;c[j>>2]=e;T=i+(e<<2)|0;U=f+9728+(l<<8)|0;c_[c[k>>2]&127](U);dm[c[M>>2]&127](a,N,f+(l<<8)|0);dm[c[M>>2]&127](a,O,U);dm[c[h>>2]&127](T,O,N);dq[c[o>>2]&127](b,T,m,t);cX[c[u>>2]&255](b,i+(e+d<<2)|0,v,b,t);cX[c[u>>2]&255](b,i+(e+w<<2)|0,x,b,t);cX[c[u>>2]&255](b,i+(e+y<<2)|0,z,b,t);cX[c[u>>2]&255](b,i+(e+A<<2)|0,B,b,t);cX[c[u>>2]&255](b,i+(e+C<<2)|0,D,b,t);cX[c[u>>2]&255](b,i+(e+E<<2)|0,F,b,t);cX[c[u>>2]&255](b,i+(e+G<<2)|0,H,b,t);cX[c[u>>2]&255](b,i+(e+I<<2)|0,J,b,t);cX[c[u>>2]&255](b,i+(e+K<<2)|0,L,b,t);e=l+1|0;if((e|0)<32){l=e;b=b+(t<<2)|0}else{break}}return}while(1){b=c[j>>2]|0;if((b|0)<(n|0)){NN(q|0,r|0,s)|0;V=p}else{V=b}b=V-n|0;c[j>>2]=b;l=i+(b<<2)|0;O=0;do{h=f+(Q<<8)+(O<<2)|0;g[h>>2]=-0.0- +g[h>>2];g[f+(Q<<8)+(O+32<<2)>>2]=+g[f+9728+(Q<<8)+(31-O<<2)>>2];O=O+1|0;}while((O|0)<32);dm[c[M>>2]&127](a,N,f+(Q<<8)|0);c$[c[P>>2]&127](l,N);dq[c[o>>2]&127](R,l,m,t);cX[c[u>>2]&255](R,i+(b+d<<2)|0,v,R,t);cX[c[u>>2]&255](R,i+(b+w<<2)|0,x,R,t);cX[c[u>>2]&255](R,i+(b+y<<2)|0,z,R,t);cX[c[u>>2]&255](R,i+(b+A<<2)|0,B,R,t);cX[c[u>>2]&255](R,i+(b+C<<2)|0,D,R,t);cX[c[u>>2]&255](R,i+(b+E<<2)|0,F,R,t);cX[c[u>>2]&255](R,i+(b+G<<2)|0,H,R,t);cX[c[u>>2]&255](R,i+(b+I<<2)|0,J,R,t);cX[c[u>>2]&255](R,i+(b+K<<2)|0,L,R,t);O=Q+1|0;if((O|0)<32){Q=O;R=R+(t<<2)|0}else{break}}return}function p5(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0;NM(d|0,0,10240);a=b+68|0;if((c[a>>2]|0)!=0){h=0;do{i=8;do{j=i-8|0;g[d+(h*320|0)+(i<<3)>>2]=+g[e+(f<<13)+(j<<8)+(h<<3)>>2];g[d+(h*320|0)+(i<<3)+4>>2]=+g[e+(f<<13)+(j<<8)+(h<<3)+4>>2];i=i+1|0;}while((i|0)<40);h=h+1|0;}while(h>>>0<(c[a>>2]|0)>>>0)}a=1-f|0;f=b+64|0;if((c[f>>2]|0)==0){return 0}else{k=0}do{g[d+(k*320|0)>>2]=+g[e+(a<<13)+6144+(k<<3)>>2];g[d+(k*320|0)+4>>2]=+g[e+(a<<13)+6144+(k<<3)+4>>2];g[d+(k*320|0)+8>>2]=+g[e+(a<<13)+6400+(k<<3)>>2];g[d+(k*320|0)+12>>2]=+g[e+(a<<13)+6400+(k<<3)+4>>2];g[d+(k*320|0)+16>>2]=+g[e+(a<<13)+6656+(k<<3)>>2];g[d+(k*320|0)+20>>2]=+g[e+(a<<13)+6656+(k<<3)+4>>2];g[d+(k*320|0)+24>>2]=+g[e+(a<<13)+6912+(k<<3)>>2];g[d+(k*320|0)+28>>2]=+g[e+(a<<13)+6912+(k<<3)+4>>2];g[d+(k*320|0)+32>>2]=+g[e+(a<<13)+7168+(k<<3)>>2];g[d+(k*320|0)+36>>2]=+g[e+(a<<13)+7168+(k<<3)+4>>2];g[d+(k*320|0)+40>>2]=+g[e+(a<<13)+7424+(k<<3)>>2];g[d+(k*320|0)+44>>2]=+g[e+(a<<13)+7424+(k<<3)+4>>2];g[d+(k*320|0)+48>>2]=+g[e+(a<<13)+7680+(k<<3)>>2];g[d+(k*320|0)+52>>2]=+g[e+(a<<13)+7680+(k<<3)+4>>2];g[d+(k*320|0)+56>>2]=+g[e+(a<<13)+7936+(k<<3)>>2];g[d+(k*320|0)+60>>2]=+g[e+(a<<13)+7936+(k<<3)+4>>2];k=k+1|0;}while(k>>>0<(c[f>>2]|0)>>>0);return 0}function p6(b,e,f,h,j){b=b|0;e=e|0;f=f|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0.0,$=0.0,aa=0,ab=0.0,ac=0.0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0;k=i;i=i+384|0;l=k|0;m=k+192|0;n=(c[f+36>>2]|0)==0;o=(n&1)<<2;p=c[f+68>>2]|0;q=c[f+76>>2]|0;r=h+87640|0;s=c[r>>2]|0;t=h+87644|0;u=c[t>>2]|0;do{if((c[f+8>>2]|0)==0){if(!n){break}v=h+87628|0;w=h+69904+(((d[v]|0)<<1)*192|0)|0;x=h+87636|0;y=h+69904+(((d[x]|0)<<1)*192|0)|0;NN(w|0,y|0,768)|0;y=h+77968+(((d[v]|0)<<1)*192|0)|0;v=h+77968+(((d[x]|0)<<1)*192|0)|0;NN(y|0,v|0,768)|0}else{if(!n){break}v=h+87628|0;y=f+340944|0;x=q<<2;w=f+338256|0;z=0;do{A=h+69904+((((d[v]|0)<<1)+z|0)*192|0)|0;NN(A|0,y|0,x)|0;A=h+77968+((((d[v]|0)<<1)+z|0)*192|0)|0;NN(A|0,w|0,x)|0;z=z+1|0;}while((z|0)<(o|0))}}while(0);z=h+8|0;x=c[z>>2]|0;if((x|0)==0){B=u;C=s;c[r>>2]=C;c[t>>2]=B;i=k;return}w=q<<2;v=0;y=a[h+87628|0]|0;A=x;while(1){x=(y&255)<<1;D=v+1|0;E=h+87628+D|0;F=a[E]|0;if(x>>>0<(F&255)<<1>>>0){G=f+340944+(v*192|0)|0;H=f+338256+(v*192|0)|0;I=x;do{x=I+o|0;J=h+69904+(x*192|0)|0;NN(J|0,G|0,w)|0;J=h+77968+(x*192|0)|0;NN(J|0,H|0,w)|0;I=I+1|0;K=a[E]|0;}while((I|0)<((K&255)<<1|0));L=K;M=c[z>>2]|0}else{L=F;M=A}if(D>>>0<M>>>0){v=D;y=L;A=M}else{break}}if((M|0)==0){B=u;C=s;c[r>>2]=C;c[t>>2]=B;i=k;return}A=l|0;L=m|0;y=j+4|0;v=(q|0)>0;K=f+343748|0;w=e+(p*320|0)|0;e=p&1;I=(q|0)>1;E=(q&1|0)==0;H=u;u=s;s=0;G=a[h+87628|0]|0;J=M;while(1){M=(G&255)<<1;x=s+1|0;N=h+87628+x|0;O=a[N]|0;if(M>>>0<(O&255)<<1>>>0){P=f+339600+(s*192|0)|0;Q=H;R=u;S=M;do{do{if(n){if((s|0)==(c[j>>2]|0)){T=2126;break}if((s|0)==(c[y>>2]|0)){T=2126;break}if(!v){U=L;V=A;break}M=S+o|0;W=0;while(1){X=l+(W<<2)|0;g[X>>2]=0.0;Y=m+(W<<2)|0;g[Y>>2]=0.0;Z=0;_=0.0;$=0.0;do{aa=M-Z|0;ab=+g[80424+(Z<<2)>>2];_=_+ +g[h+69904+(aa*192|0)+(W<<2)>>2]*ab;$=$+ +g[h+77968+(aa*192|0)+(W<<2)>>2]*ab;Z=Z+1|0;}while((Z|0)<=(o|0));g[X>>2]=_;g[Y>>2]=$;Z=W+1|0;if((Z|0)<(q|0)){W=Z}else{U=L;V=A;break}}}else{T=2126}}while(0);if((T|0)==2126){T=0;U=h+77968+(S*192|0)|0;V=h+69904+((S+o|0)*192|0)|0}W=b+(S<<9)+(p<<3)|0;cX[c[K>>2]&255](W,w,V,q,S+2|0);do{if((s|0)==(c[j>>2]|0)){T=2130}else{if((s|0)==(c[y>>2]|0)){T=2130;break}c9[c[f+343752+(Q<<2)>>2]&127](W,P,U,R,p,q)}}while(0);do{if((T|0)==2130){T=0;W=Q&1;M=1-(Q+e&2)|0;if(I){ab=+(M|0);ac=+((M^-W)+W|0);Z=0;aa=1;while(1){ad=Z<<1|W;ae=b+(S<<9)+(p<<3)+(ad<<2)|0;g[ae>>2]=+g[ae>>2]+ab*+g[f+339600+(s*192|0)+(Z<<2)>>2];ae=b+(S<<9)+(p<<3)+((ad|2)<<2)|0;g[ae>>2]=+g[ae>>2]+ac*+g[f+339600+(s*192|0)+(aa<<2)>>2];ae=Z+2|0;ad=ae|1;if((ad|0)<(q|0)){Z=ae;aa=ad}else{af=ae;break}}}else{af=0}if(E){break}aa=b+(S<<9)+(p<<3)+((af<<1|W)<<2)|0;g[aa>>2]=+g[aa>>2]+ +(M|0)*+g[f+339600+(s*192|0)+(af<<2)>>2]}}while(0);R=R+q&511;Q=Q+1&3;S=S+1|0;ag=a[N]|0;}while((S|0)<((ag&255)<<1|0));ah=Q;ai=R;aj=ag;ak=c[z>>2]|0}else{ah=H;ai=u;aj=O;ak=J}if(x>>>0<ak>>>0){H=ah;u=ai;s=x;G=aj;J=ak}else{B=ah;C=ai;break}}c[r>>2]=C;c[t>>2]=B;i=k;return}function p7(a,b,e,f,h,i){a=a|0;b=b|0;e=e|0;f=f|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;j=((d[a+96+(i*87648|0)+87636|0]|0)<<1)-32|0;i=(j|0)>0;k=i?j:0;NM(b|0,0,19456);j=a+64|0;l=c[j>>2]|0;if((l|0)==0){m=0;n=0}else{o=0;p=l;while(1){if(i){l=0;do{q=l+2|0;g[b+(l<<8)+(o<<2)>>2]=+g[h+(o*320|0)+(q<<3)>>2];g[b+9728+(l<<8)+(o<<2)>>2]=+g[h+(o*320|0)+(q<<3)+4>>2];l=l+1|0;}while((l|0)<(k|0));r=c[j>>2]|0}else{r=p}l=o+1|0;if(l>>>0<r>>>0){o=l;p=r}else{m=l;n=r;break}}}r=a+72|0;p=c[r>>2]|0;if(m>>>0<(p+n|0)>>>0){o=m;m=n;n=p;while(1){if(i){p=0;do{l=p+32|0;g[b+(p<<8)+(o<<2)>>2]=+g[e+(l<<9)+(o<<3)>>2];g[b+9728+(p<<8)+(o<<2)>>2]=+g[e+(l<<9)+(o<<3)+4>>2];p=p+1|0;}while((p|0)<(k|0));s=c[j>>2]|0;t=c[r>>2]|0}else{s=m;t=n}p=o+1|0;if(p>>>0<(t+s|0)>>>0){o=p;m=s;n=t}else{break}}}t=a+68|0;n=c[t>>2]|0;L3024:do{if((n|0)==0){u=0;v=0}else{if((k|0)<38){w=0}else{s=0;while(1){m=s+1|0;if(m>>>0<n>>>0){s=m}else{u=m;v=n;break L3024}}}while(1){s=k;do{m=s+2|0;g[b+(s<<8)+(w<<2)>>2]=+g[h+(w*320|0)+(m<<3)>>2];g[b+9728+(s<<8)+(w<<2)>>2]=+g[h+(w*320|0)+(m<<3)+4>>2];s=s+1|0;}while((s|0)<38);s=c[t>>2]|0;m=w+1|0;if(m>>>0<s>>>0){w=m}else{u=m;v=s;break}}}}while(0);w=a+76|0;a=(c[w>>2]|0)+v|0;if(u>>>0>=a>>>0){return 0}if((k|0)<32){x=u}else{v=u;do{v=v+1|0;}while(v>>>0<a>>>0);return 0}do{a=k;do{g[b+(a<<8)+(x<<2)>>2]=+g[f+(a<<9)+(x<<3)>>2];g[b+9728+(a<<8)+(x<<2)>>2]=+g[f+(a<<9)+(x<<3)+4>>2];a=a+1|0;}while((a|0)<32);x=x+1|0;}while(x>>>0<((c[w>>2]|0)+(c[t>>2]|0)|0)>>>0);return 0}function p8(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0.0,u=0.0,v=0.0,w=0.0,x=0.0,y=0.0,z=0.0,A=0.0,B=0.0,C=0.0,D=0.0,E=0.0,F=0,G=0,H=0;h=i;i=i+48|0;j=h|0;if((f|0)<=0){i=h;return}k=j|0;l=a+28|0;a=j+40|0;m=j+16|0;n=j+24|0;o=j+28|0;p=j|0;q=j+4|0;r=j+8|0;s=j+12|0;j=0;do{c$[c[l>>2]&127](e+(j*320|0)|0,k);t=+g[m>>2];u=+g[n>>2];v=+g[o>>2];w=+g[a>>2]*t-(u*u+v*v)/1.0000009536743164;if(w!=0.0){x=+g[p>>2];y=+g[q>>2];z=v*x+u*y-t*+g[s>>2];A=(u*x-v*y-t*+g[r>>2])/w;g[d+(j<<3)>>2]=A;y=z/w;g[d+(j<<3)+4>>2]=y;B=A;C=y}else{g[d+(j<<3)>>2]=0.0;g[d+(j<<3)+4>>2]=0.0;B=0.0;C=0.0}if(t!=0.0){y=u*C+ +g[q>>2]-B*v;A=(-0.0-(+g[p>>2]+B*u+C*v))/t;g[b+(j<<3)>>2]=A;v=(-0.0-y)/t;g[b+(j<<3)+4>>2]=v;D=A;E=v}else{g[b+(j<<3)>>2]=0.0;g[b+(j<<3)+4>>2]=0.0;D=0.0;E=0.0}F=d+(j<<3)|0;G=d+(j<<3)+4|0;v=+g[F>>2];A=+g[G>>2];if(v*v+A*A<16.0){if(D*D+E*E>=16.0){H=2176}}else{H=2176}if((H|0)==2176){H=0;g[F>>2]=0.0;g[G>>2]=0.0;g[b+(j<<3)>>2]=0.0;g[b+(j<<3)+4>>2]=0.0}j=j+1|0;}while((j|0)<(f|0));i=h;return}function p9(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;h=i;j=g+8|0;k=c[j>>2]|0;a[g+12|0]=a[g+12+k|0]|0;l=g+92|0;c[l>>2]=c[e+20>>2];e=g+87628|0;a[g+87636|0]=a[g+87628+k|0]|0;m=f+8|0;n=c[m>>2]|0;o=f+16|0;p=c[o>>2]|0;q=f|0;f=(c[q>>2]|0)+(n>>>3)|0;r=(cV(d[f]|d[f+1|0]<<8|d[f+2|0]<<16|d[f+3|0]<<24|0)|0)<<(n&7)>>>30;f=n+2|0;c[m>>2]=p>>>0>f>>>0?f:p;p=g|0;c[p>>2]=r;L3069:do{if((r|0)==1){f=c[m>>2]|0;n=c[o>>2]|0;s=c[q>>2]|0;t=s+(f>>>3)|0;u=(cV(d[t]|d[t+1|0]<<8|d[t+2|0]<<16|d[t+3|0]<<24|0)|0)<<(f&7)>>>30;t=f+2|0;f=n>>>0>t>>>0?t:n;c[m>>2]=f;t=s+(f>>>3)|0;s=(cV(d[t]|d[t+1|0]<<8|d[t+2|0]<<16|d[t+3|0]<<24|0)|0)<<(f&7)>>>30;t=f+2|0;c[m>>2]=n>>>0>t>>>0?t:n;n=s+1|0;c[j>>2]=n;a[e|0]=0;a[g+87628+n|0]=(u|16)&255;u=c[j>>2]|0;if((s|0)==0){v=u}else{n=0;t=u;while(1){u=d[t-n+(g+87628)|0]|0;f=c[m>>2]|0;w=c[o>>2]|0;x=(c[q>>2]|0)+(f>>>3)|0;y=(cV(d[x]|d[x+1|0]<<8|d[x+2|0]<<16|d[x+3|0]<<24|0)|0)<<(f&7)>>>30;x=f+2|0;c[m>>2]=w>>>0>x>>>0?x:w;a[(c[j>>2]|0)+~n+(g+87628)|0]=u+254-(y<<1)&255;y=n+1|0;u=c[j>>2]|0;if((y|0)<(s|0)){n=y;t=u}else{v=u;break}}}t=a[304912+v|0]|0;n=c[m>>2]|0;s=c[o>>2]|0;u=c[q>>2]|0;y=u+(n>>>3)|0;w=(cV(d[y]|d[y+1|0]<<8|d[y+2|0]<<16|d[y+3|0]<<24|0)|0)<<(n&7)>>>((32-t|0)>>>0);y=n+t|0;t=s>>>0>y>>>0?y:s;c[m>>2]=t;if((c[j>>2]|0)==0){z=w;A=2215;break}else{B=0;C=t;D=u;E=s}while(1){s=d[D+(C>>>3)|0]<<(C&7)>>>7;c[m>>2]=((C|0)<(E|0))+C;a[(c[j>>2]|0)-B+(g+12)|0]=s&1;s=B+1|0;if(s>>>0>=(c[j>>2]|0)>>>0){z=w;A=2215;break L3069}B=s;C=c[m>>2]|0;D=c[q>>2]|0;E=c[o>>2]|0}}else if((r|0)==2){w=c[m>>2]|0;s=c[o>>2]|0;u=(c[q>>2]|0)+(w>>>3)|0;t=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(w&7)>>>30;u=w+2|0;c[m>>2]=s>>>0>u>>>0?u:s;a[e|0]=t&255;t=c[m>>2]|0;s=c[o>>2]|0;u=(c[q>>2]|0)+(t>>>3)|0;w=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(t&7)>>>30;u=t+2|0;c[m>>2]=s>>>0>u>>>0?u:s;s=w+1|0;c[j>>2]=s;a[g+87628+s|0]=16;if((w|0)!=0){s=0;u=a[g+87628|0]|0;do{t=c[m>>2]|0;y=c[o>>2]|0;n=(c[q>>2]|0)+(t>>>3)|0;x=(cV(d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0)|0)<<(t&7)>>>30;n=t+2|0;c[m>>2]=y>>>0>n>>>0?n:y;u=(u&255)+2+(x<<1)&255;s=s+1|0;a[g+87628+s|0]=u;}while((s|0)<(w|0))}w=a[304912+(c[j>>2]|0)|0]|0;s=c[m>>2]|0;u=c[o>>2]|0;x=c[q>>2]|0;y=x+(s>>>3)|0;n=(cV(d[y]|d[y+1|0]<<8|d[y+2|0]<<16|d[y+3|0]<<24|0)|0)<<(s&7)>>>((32-w|0)>>>0);y=s+w|0;w=u>>>0>y>>>0?y:u;c[m>>2]=w;y=c[j>>2]|0;if((y|0)>0){F=1;G=w;H=x;I=u}else{z=n;A=2215;break}while(1){u=d[H+(G>>>3)|0]<<(G&7)>>>7;c[m>>2]=((G|0)<(I|0))+G;a[g+12+F|0]=u&1;if((F|0)>=(y|0)){z=n;A=2215;break L3069}F=F+1|0;G=c[m>>2]|0;H=c[q>>2]|0;I=c[o>>2]|0}}else if((r|0)==0){n=c[m>>2]|0;y=c[o>>2]|0;u=(c[q>>2]|0)+(n>>>3)|0;x=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(n&7)>>>30;u=n+2|0;c[m>>2]=y>>>0>u>>>0?u:y;y=1<<x;c[j>>2]=y;x=y-1|0;do{if((y|0)==1){c[l>>2]=0}else{if(y>>>0<=4){break}AN(c[b+4>>2]|0,16,1711584,(J=i,i=i+8|0,c[J>>2]=y,J)|0);i=J;K=-1;i=h;return K|0}}while(0);a[e|0]=0;a[g+87628+y|0]=16;u=c[j>>2]|0;n=(((u>>>1)+16|0)>>>0)/(u>>>0)|0;if((x|0)>0){u=0;w=a[g+87628|0]|0;do{w=(w&255)+n&255;u=u+1|0;a[g+87628+u|0]=w;}while((u|0)<(x|0))}x=c[m>>2]|0;u=d[(c[q>>2]|0)+(x>>>3)|0]<<(x&7)>>>7;c[m>>2]=((x|0)<(c[o>>2]|0))+x;x=u&1;u=g+13|0;a[u]=x;w=c[j>>2]|0;if(w>>>0>1){L=2;M=x}else{N=0;O=w;break}while(1){a[g+12+L|0]=M;w=c[j>>2]|0;if(L>>>0>=w>>>0){N=0;O=w;break L3069}L=L+1|0;M=a[u]|0}}else if((r|0)==3){u=c[m>>2]|0;w=c[o>>2]|0;x=(c[q>>2]|0)+(u>>>3)|0;n=(cV(d[x]|d[x+1|0]<<8|d[x+2|0]<<16|d[x+3|0]<<24|0)|0)<<(u&7)>>>30;x=u+2|0;c[m>>2]=w>>>0>x>>>0?x:w;a[e|0]=n&255;n=c[m>>2]|0;w=c[o>>2]|0;x=c[q>>2]|0;u=x+(n>>>3)|0;y=d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0;u=n+2|0;s=w>>>0>u>>>0?u:w;c[m>>2]=s;u=x+(s>>>3)|0;t=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(s&7)>>>30;u=s+2|0;s=w>>>0>u>>>0?u:w;c[m>>2]=s;u=x+(s>>>3)|0;x=(cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(s&7)>>>30;u=s+2|0;c[m>>2]=w>>>0>u>>>0?u:w;w=t+1+x|0;c[j>>2]=w;if(w>>>0>5){AN(c[b+4>>2]|0,16,1708160,(J=i,i=i+8|0,c[J>>2]=w,J)|0);i=J;K=-1;i=h;return K|0}a[g+87628+w|0]=((cV(y|0)|0)<<(n&7)>>>30|16)&255;if((t|0)!=0){n=0;y=a[g+87628|0]|0;do{w=c[m>>2]|0;u=c[o>>2]|0;s=(c[q>>2]|0)+(w>>>3)|0;f=(cV(d[s]|d[s+1|0]<<8|d[s+2|0]<<16|d[s+3|0]<<24|0)|0)<<(w&7)>>>30;s=w+2|0;c[m>>2]=u>>>0>s>>>0?s:u;y=(y&255)+2+(f<<1)&255;n=n+1|0;a[g+87628+n|0]=y;}while((n|0)<(t|0))}t=c[j>>2]|0;if((x|0)==0){P=t}else{n=0;y=t;while(1){t=d[y-n+(g+87628)|0]|0;f=c[m>>2]|0;u=c[o>>2]|0;s=(c[q>>2]|0)+(f>>>3)|0;w=(cV(d[s]|d[s+1|0]<<8|d[s+2|0]<<16|d[s+3|0]<<24|0)|0)<<(f&7)>>>30;s=f+2|0;c[m>>2]=u>>>0>s>>>0?s:u;a[(c[j>>2]|0)+~n+(g+87628)|0]=t+254-(w<<1)&255;w=n+1|0;t=c[j>>2]|0;if((w|0)<(x|0)){n=w;y=t}else{P=t;break}}}y=a[304912+P|0]|0;n=c[m>>2]|0;x=c[o>>2]|0;t=c[q>>2]|0;w=t+(n>>>3)|0;u=(cV(d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0)|0)<<(n&7)>>>((32-y|0)>>>0);w=n+y|0;y=x>>>0>w>>>0?w:x;c[m>>2]=y;w=c[j>>2]|0;if((w|0)>0){Q=1;R=y;S=t;T=x}else{z=u;A=2215;break}while(1){x=d[S+(R>>>3)|0]<<(R&7)>>>7;c[m>>2]=((R|0)<(T|0))+R;a[g+12+Q|0]=x&1;if((Q|0)>=(w|0)){z=u;A=2215;break L3069}Q=Q+1|0;R=c[m>>2]|0;S=c[q>>2]|0;T=c[o>>2]|0}}else{N=0;O=c[j>>2]|0}}while(0);do{if((A|0)==2215){o=c[j>>2]|0;if(z>>>0<=(o+1|0)>>>0){N=z;O=o;break}AN(c[b+4>>2]|0,16,1706e3,(J=i,i=i+8|0,c[J>>2]=z,J)|0);i=J;K=-1;i=h;return K|0}}while(0);z=1;while(1){if(z>>>0>O>>>0){break}if((d[z-1+(g+87628)|0]|0)>(d[g+87628+z|0]|0)){A=2219;break}else{z=z+1|0}}if((A|0)==2219){AN(c[b+4>>2]|0,16,1704152,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;K=-1;i=h;return K|0}J=O>>>0>1?2:1;b=g+20|0;c[b>>2]=J;a[g+87637|0]=a[e|0]|0;a[g+87637+J|0]=a[g+87628+O|0]|0;O=c[p>>2]|0;if((c[b>>2]|0)>>>0>1){do{if((O|0)==0){U=(c[j>>2]|0)>>>1}else{if((O&1|0)!=0){b=N-1|0;U=(c[j>>2]|0)-((b|0)>1?b:1)|0;break}if((N|0)==0){U=1;break}else if((N|0)==1){U=(c[j>>2]|0)-1|0;break}else{U=N-1|0;break}}}while(0);a[g+87638|0]=a[g+87628+U|0]|0}U=g+14568|0;c[g+14564>>2]=-((c[U>>2]|0)!=(k|0)|0);c[U>>2]=-1;if(!((O&1|0)==0|(N|0)==0)){c[U>>2]=1-N+(c[j>>2]|0);K=0;i=h;return K|0}if(!((O|0)==2&N>>>0>1)){K=0;i=h;return K|0}c[U>>2]=N-1;K=0;i=h;return K|0}function qa(e,f,h,i){e=e|0;f=f|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0.0,D=0,E=0.0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0.0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0;j=c[e+40>>2]|0;if((i|0)==1){k=(j|0)==1?2:1}else{k=1}l=c[e+262884>>2]&1;m=(c[h+92>>2]|0)==0;if((j|0)==0|(i|0)==0){n=m?60:31;o=m?1997756:1997820;p=m?1997740:1997804;q=m?7:6}else{n=m?24:12;o=m?1997788:1997852;p=m?1997772:1997836;q=m?6:5}m=c[o>>2]|0;o=c[p>>2]|0;p=h+8|0;if((c[p>>2]|0)==0){r=0;s=h+86416|0;t=s;u=h+86416+(r*192|0)|0;v=u;NN(t|0,v|0,192)|0;return}i=f+8|0;j=f+16|0;w=f|0;f=32-q|0;x=0;while(1){L3166:do{if((a[h+24+x|0]|0)==0){y=c[i>>2]|0;z=c[j>>2]|0;A=(c[w>>2]|0)+(y>>>3)|0;B=(cV(d[A]|d[A+1|0]<<8|d[A+2|0]<<16|d[A+3|0]<<24|0)|0)<<(y&7)>>>(f>>>0);A=y+q|0;c[i>>2]=z>>>0>A>>>0?A:z;C=+((ac(B,k)|0)>>>0>>>0);B=x+1|0;g[h+86416+(B*192|0)>>2]=C;z=h+12+B|0;if((c[e+262880+(d[z]<<2)>>2]|0)>>>0>1){D=1;E=C}else{break}do{A=c[i>>2]|0;y=c[j>>2]|0;F=c[w>>2]|0;G=F+(A>>>3)|0;H=(cV(d[G]|d[G+1|0]<<8|d[G+2|0]<<16|d[G+3|0]<<24|0)|0)<<(A&7)>>>23;G=b[m+(H<<2)>>1]|0;I=b[m+(H<<2)+2>>1]|0;H=I<<16>>16;do{if(I<<16>>16<0){J=A+9|0;K=y>>>0>J>>>0?J:y;J=F+(K>>>3)|0;L=((cV(d[J]|d[J+1|0]<<8|d[J+2|0]<<16|d[J+3|0]<<24|0)|0)<<(K&7)>>>((H+32|0)>>>0))+G|0;J=b[m+(L<<2)>>1]|0;M=b[m+(L<<2)+2>>1]|0;L=M<<16>>16;if(M<<16>>16>=0){N=J;O=K;P=L;break}M=K-H|0;K=y>>>0>M>>>0?M:y;M=F+(K>>>3)|0;Q=((cV(d[M]|d[M+1|0]<<8|d[M+2|0]<<16|d[M+3|0]<<24|0)|0)<<(K&7)>>>((L+32|0)>>>0))+J|0;N=b[m+(Q<<2)>>1]|0;O=K;P=b[m+(Q<<2)+2>>1]|0}else{N=G;O=A;P=H}}while(0);H=O+P|0;c[i>>2]=y>>>0>H>>>0?H:y;E=E+ +(ac(N-n|0,k)|0);g[h+86416+(B*192|0)+(D<<2)>>2]=E;D=D+1|0;}while(D>>>0<(c[e+262880+(d[z]<<2)>>2]|0)>>>0)}else{z=x+1|0;B=h+12+z|0;H=a[B]|0;if(H<<24>>24==(a[h+12+x|0]|0)){if((c[e+262880+((H&255)<<2)>>2]|0)==0){break}else{R=0}while(1){C=+g[h+86416+(x*192|0)+(R<<2)>>2];A=c[i>>2]|0;G=c[j>>2]|0;F=c[w>>2]|0;I=F+(A>>>3)|0;Q=(cV(d[I]|d[I+1|0]<<8|d[I+2|0]<<16|d[I+3|0]<<24|0)|0)<<(A&7)>>>23;I=b[o+(Q<<2)>>1]|0;K=b[o+(Q<<2)+2>>1]|0;Q=K<<16>>16;do{if(K<<16>>16<0){J=A+9|0;L=G>>>0>J>>>0?J:G;J=F+(L>>>3)|0;M=((cV(d[J]|d[J+1|0]<<8|d[J+2|0]<<16|d[J+3|0]<<24|0)|0)<<(L&7)>>>((Q+32|0)>>>0))+I|0;J=b[o+(M<<2)>>1]|0;S=b[o+(M<<2)+2>>1]|0;M=S<<16>>16;if(S<<16>>16>=0){T=M;U=L;V=J;break}S=L-Q|0;L=G>>>0>S>>>0?S:G;S=F+(L>>>3)|0;W=((cV(d[S]|d[S+1|0]<<8|d[S+2|0]<<16|d[S+3|0]<<24|0)|0)<<(L&7)>>>((M+32|0)>>>0))+J|0;T=b[o+(W<<2)+2>>1]|0;U=L;V=b[o+(W<<2)>>1]|0}else{T=Q;U=A;V=I}}while(0);I=T+U|0;c[i>>2]=G>>>0>I>>>0?I:G;g[h+86416+(z*192|0)+(R<<2)>>2]=C+ +(ac(V-n|0,k)|0);R=R+1|0;if(R>>>0>=(c[e+262880+(d[B]<<2)>>2]|0)>>>0){break L3166}}}if(H<<24>>24!=0){if((c[e+262880+((H&255)<<2)>>2]|0)==0){break}else{X=0}while(1){Y=+g[h+86416+(x*192|0)+(X+l>>1<<2)>>2];I=c[i>>2]|0;A=c[j>>2]|0;Q=c[w>>2]|0;F=Q+(I>>>3)|0;K=(cV(d[F]|d[F+1|0]<<8|d[F+2|0]<<16|d[F+3|0]<<24|0)|0)<<(I&7)>>>23;F=b[o+(K<<2)>>1]|0;y=b[o+(K<<2)+2>>1]|0;K=y<<16>>16;do{if(y<<16>>16<0){W=I+9|0;L=A>>>0>W>>>0?W:A;W=Q+(L>>>3)|0;J=((cV(d[W]|d[W+1|0]<<8|d[W+2|0]<<16|d[W+3|0]<<24|0)|0)<<(L&7)>>>((K+32|0)>>>0))+F|0;W=b[o+(J<<2)>>1]|0;M=b[o+(J<<2)+2>>1]|0;J=M<<16>>16;if(M<<16>>16>=0){Z=W;_=L;$=J;break}M=L-K|0;L=A>>>0>M>>>0?M:A;M=Q+(L>>>3)|0;S=((cV(d[M]|d[M+1|0]<<8|d[M+2|0]<<16|d[M+3|0]<<24|0)|0)<<(L&7)>>>((J+32|0)>>>0))+W|0;Z=b[o+(S<<2)>>1]|0;_=L;$=b[o+(S<<2)+2>>1]|0}else{Z=F;_=I;$=K}}while(0);K=_+$|0;c[i>>2]=A>>>0>K>>>0?K:A;g[h+86416+(z*192|0)+(X<<2)>>2]=Y+ +(ac(Z-n|0,k)|0);X=X+1|0;if(X>>>0>=(c[e+262880+(d[B]<<2)>>2]|0)>>>0){break L3166}}}if((c[e+262880>>2]|0)==0){break}else{aa=0}do{if((aa|0)==0){ab=0}else{ab=(aa<<1)-l|0}C=+g[h+86416+(x*192|0)+(ab<<2)>>2];H=c[i>>2]|0;K=c[j>>2]|0;I=c[w>>2]|0;F=I+(H>>>3)|0;Q=(cV(d[F]|d[F+1|0]<<8|d[F+2|0]<<16|d[F+3|0]<<24|0)|0)<<(H&7)>>>23;F=b[o+(Q<<2)>>1]|0;y=b[o+(Q<<2)+2>>1]|0;Q=y<<16>>16;do{if(y<<16>>16<0){G=H+9|0;S=K>>>0>G>>>0?G:K;G=I+(S>>>3)|0;L=((cV(d[G]|d[G+1|0]<<8|d[G+2|0]<<16|d[G+3|0]<<24|0)|0)<<(S&7)>>>((Q+32|0)>>>0))+F|0;G=b[o+(L<<2)>>1]|0;W=b[o+(L<<2)+2>>1]|0;L=W<<16>>16;if(W<<16>>16>=0){ad=G;ae=S;af=L;break}W=S-Q|0;S=K>>>0>W>>>0?W:K;W=I+(S>>>3)|0;J=((cV(d[W]|d[W+1|0]<<8|d[W+2|0]<<16|d[W+3|0]<<24|0)|0)<<(S&7)>>>((L+32|0)>>>0))+G|0;ad=b[o+(J<<2)>>1]|0;ae=S;af=b[o+(J<<2)+2>>1]|0}else{ad=F;ae=H;af=Q}}while(0);Q=ae+af|0;c[i>>2]=K>>>0>Q>>>0?Q:K;g[h+86416+(z*192|0)+(aa<<2)>>2]=C+ +(ac(ad-n|0,k)|0);aa=aa+1|0;}while(aa>>>0<(c[e+262880+(d[B]<<2)>>2]|0)>>>0)}}while(0);B=x+1|0;z=c[p>>2]|0;if(B>>>0<z>>>0){x=B}else{r=z;break}}s=h+86416|0;t=s;u=h+86416+(r*192|0)|0;v=u;NN(t|0,v|0,192)|0;return}function qb(e,f,h,i){e=e|0;f=f|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0.0,A=0,B=0.0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;j=c[e+40>>2]|0;if((i|0)==1){k=(j|0)==1?2:1}else{k=1}l=(j|0)==0|(i|0)==0;i=l?31:12;j=l?c[499455]|0:c[499463]|0;m=l?c[499467]|0:c[499471]|0;l=h+20|0;if((c[l>>2]|0)==0){n=0;o=h+87568|0;p=o;q=h+87568+(n*20|0)|0;r=q;c[p>>2]=c[r>>2];c[p+4>>2]=c[r+4>>2];c[p+8>>2]=c[r+8>>2];c[p+12>>2]=c[r+12>>2];c[p+16>>2]=c[r+16>>2];return}s=f+8|0;t=f+16|0;u=f|0;f=e+262888|0;e=0;while(1){do{if((a[h+29+e|0]|0)==0){v=c[s>>2]|0;w=c[t>>2]|0;x=(c[u>>2]|0)+(v>>>3)|0;y=(cV(d[x]|d[x+1|0]<<8|d[x+2|0]<<16|d[x+3|0]<<24|0)|0)<<(v&7)>>>27;x=v+5|0;c[s>>2]=w>>>0>x>>>0?x:w;z=+((ac(y,k)|0)>>>0>>>0);y=e+1|0;g[h+87568+(y*20|0)>>2]=z;if((c[f>>2]|0)>>>0>1){A=1;B=z}else{C=y;break}while(1){w=c[s>>2]|0;x=c[t>>2]|0;v=c[u>>2]|0;D=v+(w>>>3)|0;E=(cV(d[D]|d[D+1|0]<<8|d[D+2|0]<<16|d[D+3|0]<<24|0)|0)<<(w&7)>>>23;D=b[j+(E<<2)>>1]|0;F=b[j+(E<<2)+2>>1]|0;E=F<<16>>16;do{if(F<<16>>16<0){G=w+9|0;H=x>>>0>G>>>0?G:x;G=v+(H>>>3)|0;I=((cV(d[G]|d[G+1|0]<<8|d[G+2|0]<<16|d[G+3|0]<<24|0)|0)<<(H&7)>>>((E+32|0)>>>0))+D|0;G=b[j+(I<<2)>>1]|0;J=b[j+(I<<2)+2>>1]|0;I=J<<16>>16;if(J<<16>>16>=0){K=G;L=H;M=I;break}J=H-E|0;H=x>>>0>J>>>0?J:x;J=v+(H>>>3)|0;N=((cV(d[J]|d[J+1|0]<<8|d[J+2|0]<<16|d[J+3|0]<<24|0)|0)<<(H&7)>>>((I+32|0)>>>0))+G|0;K=b[j+(N<<2)>>1]|0;L=H;M=b[j+(N<<2)+2>>1]|0}else{K=D;L=w;M=E}}while(0);E=L+M|0;c[s>>2]=x>>>0>E>>>0?E:x;z=B+ +(ac(K-i|0,k)|0);g[h+87568+(y*20|0)+(A<<2)>>2]=z;E=A+1|0;if(E>>>0<(c[f>>2]|0)>>>0){A=E;B=z}else{C=y;break}}}else{y=e+1|0;if((c[f>>2]|0)==0){C=y;break}else{O=0}while(1){z=+g[h+87568+(e*20|0)+(O<<2)>>2];E=c[s>>2]|0;w=c[t>>2]|0;D=c[u>>2]|0;v=D+(E>>>3)|0;F=(cV(d[v]|d[v+1|0]<<8|d[v+2|0]<<16|d[v+3|0]<<24|0)|0)<<(E&7)>>>23;v=b[m+(F<<2)>>1]|0;N=b[m+(F<<2)+2>>1]|0;F=N<<16>>16;if(N<<16>>16<0){N=E+9|0;H=w>>>0>N>>>0?N:w;N=D+(H>>>3)|0;D=((cV(d[N]|d[N+1|0]<<8|d[N+2|0]<<16|d[N+3|0]<<24|0)|0)<<(H&7)>>>((F+32|0)>>>0))+v|0;P=b[m+(D<<2)+2>>1]|0;Q=H;R=b[m+(D<<2)>>1]|0}else{P=F;Q=E;R=v}v=P+Q|0;c[s>>2]=w>>>0>v>>>0?v:w;g[h+87568+(y*20|0)+(O<<2)>>2]=z+ +(ac(R-i|0,k)|0);w=O+1|0;if(w>>>0<(c[f>>2]|0)>>>0){O=w}else{C=y;break}}}}while(0);y=c[l>>2]|0;if(C>>>0<y>>>0){e=C}else{n=y;break}}o=h+87568|0;p=o;q=h+87568+(n*20|0)|0;r=q;c[p>>2]=c[r>>2];c[p+4>>2]=c[r+4>>2];c[p+8>>2]=c[r+8>>2];c[p+12>>2]=c[r+12>>2];c[p+16>>2]=c[r+16>>2];return}function qc(a){a=a|0;var e=0,f=0,h=0,j=0.0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;e=i;i=i+16|0;f=e|0;h=c[a+24>>2]|0;if((h|0)==0){b[a+263154>>1]=b[a+262994>>1]|0;b[a+263156>>1]=b[a+262994+(c[a+262880>>2]<<1)>>1]|0;c[a+262892>>2]=1;i=e;return}j=+g[80408+(h-1<<2)>>2];h=a+263154|0;k=h|0;l=a+263156|0;m=c[a+68>>2]&65535;b[f>>1]=m;n=a+263216|0;o=c[n>>2]|0;if((o|0)!=0){p=1;q=m;do{q=(d[p-1+(a+263220)|0]|0)+q&65535;b[f+(p<<1)>>1]=q;p=p+1|0;}while(p>>>0<=o>>>0)}o=h;h=a+262994|0;p=a+262880|0;q=c[p>>2]|0;m=(q<<1)+2|0;NN(o|0,h|0,m)|0;m=c[n>>2]|0;if(m>>>0>1){h=a+263154+(q+1<<1)|0;r=f+2|0;s=(m<<1)-2|0;NN(h|0,r|0,s)|0;t=c[n>>2]|0;u=c[p>>2]|0}else{t=m;u=q}b6(o|0,u+t|0,2,100);t=(c[p>>2]|0)-1+(c[n>>2]|0)|0;p=a+262892|0;c[p>>2]=t;if(k>>>0<(a+263154+(t<<1)|0)>>>0){v=l;w=k;x=t}else{i=e;return}L3243:while(1){t=v;k=x;L3245:while(1){y=b[t>>1]|0;l=b[w>>1]|0;if(+(y&65535|0)>=j*+(l&65535|0)){z=2303;break}L3248:do{if(y<<16>>16==l<<16>>16){A=k}else{u=c[n>>2]|0;o=0;while(1){if((o|0)>(u|0)){A=k;break L3248}if((b[f+(o<<1)>>1]|0)==y<<16>>16){B=0;break}else{o=o+1|0}}while(1){if((B|0)>(u|0)){break}if((b[f+(B<<1)>>1]|0)==l<<16>>16){z=2314;break L3245}else{B=B+1|0}}b[w>>1]=y;A=c[p>>2]|0}}while(0);l=A-1|0;c[p>>2]=l;if(w>>>0>=(a+263154+(l<<1)|0)>>>0){z=2320;break L3243}t=t+2|0;k=l}if((z|0)==2303){z=0;k=w+2|0;b[k>>1]=y;C=k}else if((z|0)==2314){z=0;k=w+2|0;b[k>>1]=y;C=k}k=c[p>>2]|0;if(C>>>0<(a+263154+(k<<1)|0)>>>0){v=t+2|0;w=C;x=k}else{z=2317;break}}if((z|0)==2320){i=e;return}else if((z|0)==2317){i=e;return}}function qd(a,c){a=a|0;c=c|0;return(b[a>>1]|0)-(b[c>>1]|0)|0}function qe(){var a=0;a=0;do{g[2900408+(a<<2)>>2]=+b4(+(+(a-200|0)*.25));a=a+1|0;}while((a|0)<428);return}function qf(a,b){a=a|0;b=b|0;var d=0;c[b>>2]=a;d=a+116|0;c[b+4>>2]=c[d>>2];c[b+8>>2]=c[d>>2];c[b+16>>2]=0;return}function qg(a){a=a|0;var b=0,d=0,e=0;b=i;d=c[a+16>>2]|0;if((d|0)!=0){AN(c[a>>2]|0,24,1649504,(e=i,i=i+8|0,c[e>>2]=d,e)|0);i=e}A6(a+12|0);NM(a|0,0,24);i=b;return}function qh(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;d=i;i=i+8|0;e=d|0;f=a+12|0;g=a+16|0;h=oO(c[f>>2]|0,a+20|0,(c[g>>2]<<4)+16|0)|0;j=h;if((h|0)==0){k=-12;i=d;return k|0}c[f>>2]=j;f=c[g>>2]|0;h=j+(f<<4)|0;l=b+76|0;m=c[l>>2]|0;n=j+(f<<4)+8|0;c[n>>2]=m;o=a+4|0;c[n>>2]=(c[o>>2]|0)+m;m=b+136|0;b=c[m>>2]|0;n=c[m+4>>2]|0;do{if((b|0)==0&(n|0)==(-2147483648|0)){m=h|0;c[m>>2]=0;c[m+4>>2]=-2147483648}else{m=a|0;p=c[m>>2]|0;c[e>>2]=1;c[e+4>>2]=c[p+412>>2];q=AY(b,n,p+104|0,e)|0;p=G;r=h|0;c[r>>2]=q;c[r+4>>2]=p;s=c[o>>2]|0;t=NV(q,p,s,(s|0)<0|0?-1:0)|0;s=G;c[r>>2]=t;c[r+4>>2]=s;if((c[g>>2]|0)==0){break}r=j+(f-1<<4)|0;p=c[r+4>>2]|0;if((p|0)<(s|0)|(p|0)==(s|0)&(c[r>>2]|0)>>>0<t>>>0){break}AN(c[m>>2]|0,24,1720920,(m=i,i=i+1|0,i=i+7&-8,c[m>>2]=0,m)|0);i=m}}while(0);c[o>>2]=0;o=a+8|0;c[o>>2]=(c[o>>2]|0)+(c[l>>2]|0);c[g>>2]=(c[g>>2]|0)+1;k=0;i=d;return k|0}function qi(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0;f=i;i=i+16|0;g=f|0;h=f+8|0;j=a+16|0;if((c[j>>2]|0)==0){if((c[a+20>>2]|0)==0){k=-2147483648;l=0}else{m=c[a+12>>2]|0;k=c[m+4>>2]|0;l=c[m>>2]|0}AN(c[a>>2]|0,24,1693192,(n=i,i=i+8|0,c[n>>2]=b,n)|0);i=n;o=k;p=l}else{l=c[a+12>>2]|0;o=c[l+4>>2]|0;p=c[l>>2]|0}if((d|0)!=0){l=c[a>>2]|0;if((p|0)==0&(o|0)==(-2147483648|0)){q=-2147483648;r=0}else{c[h>>2]=1;c[h+4>>2]=c[l+412>>2];k=AY(p,o,h,l+104|0)|0;q=G;r=k}c[d>>2]=r;c[d+4>>2]=q}do{if((b|0)==0){s=0;t=0;u=0;v=a+8|0;w=2356}else{q=a+12|0;d=b;r=0;k=0;while(1){if(k>>>0>=(c[j>>2]|0)>>>0){x=d;y=r;z=k;A=1;break}l=(c[q>>2]|0)+(k<<4)+8|0;h=c[l>>2]|0;o=(h|0)>(d|0)?d:h;c[l>>2]=h-o;h=d-o|0;l=o+r|0;p=(c[q>>2]|0)+(k<<4)|0;m=c[p>>2]|0;B=c[p+4>>2]|0;if(!((m|0)==0&(B|0)==(-2147483648|0))){C=NU(o,(o|0)<0|0?-1:0,m,B)|0;c[p>>2]=C;c[p+4>>2]=G}p=k+1|0;if((d|0)==(o|0)){x=h;y=l;z=p;A=0;break}else{d=h;r=l;k=p}}k=a+8|0;c[k>>2]=(c[k>>2]|0)-y;if((z|0)==0){s=A;t=y;u=x;v=k;w=2356;break}r=c[a+12>>2]|0;D=(c[r+(z-1<<4)+8>>2]|0)!=0|0;E=r;F=A;H=z;I=y;J=x;K=k}}while(0);if((w|0)==2356){D=0;E=c[a+12>>2]|0;F=s;H=0;I=t;J=u;K=v}v=H-D|0;D=a+12|0;NP(E|0,E+(v<<4)|0,(c[j>>2]|0)-v<<4|0);E=c[j>>2]|0;c[j>>2]=E-v;if(F){if((E|0)!=(v|0)){AN(0,0,1672232,(n=i,i=i+24|0,c[n>>2]=1657568,c[n+8>>2]=1638616,c[n+16>>2]=105,n)|0);i=n;cw()}if((c[K>>2]|0)!=(c[a+4>>2]|0)){AN(0,0,1672232,(n=i,i=i+24|0,c[n>>2]=1630008,c[n+8>>2]=1638616,c[n+16>>2]=106,n)|0);i=n;cw()}K=c[D>>2]|0;do{if((K|0)!=0){D=K|0;v=c[D>>2]|0;E=c[D+4>>2]|0;if((v|0)==0&(E|0)==(-2147483648|0)){break}F=NU(v,E,J,(J|0)<0|0?-1:0)|0;c[D>>2]=F;c[D+4>>2]=G}}while(0);AN(c[a>>2]|0,48,1623728,(n=i,i=i+8|0,c[n>>2]=J,n)|0);i=n}if((e|0)==0){i=f;return}n=c[a>>2]|0;c[g>>2]=1;c[g+4>>2]=c[n+412>>2];a=AY(I,(I|0)<0|0?-1:0,g,n+104|0)|0;c[e>>2]=a;i=f;return}function qj(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;d=b+4|0;e=c[d>>2]|0;f=e&7;g=b|0;h=c[g>>2]|0;if((e|0)>(f|0)){i=e;j=h<<f;k=i-f|0;c[g>>2]=j;c[d>>2]=k;return}else{l=cV(h<<e|0)|0;h=b+12|0;b=c[h>>2]|0;y=l;a[b]=y&255;y=y>>8;a[b+1|0]=y&255;y=y>>8;a[b+2|0]=y&255;y=y>>8;a[b+3|0]=y&255;c[h>>2]=(c[h>>2]|0)+4;i=e+32|0;j=0;k=i-f|0;c[g>>2]=j;c[d>>2]=k;return}}function qk(f,g,h,j,k,l,m,n,o,p,q,r,s){f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;r=r|0;s=s|0;var t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;t=i;i=i+16|0;u=t|0;c[f>>2]=g;if((s&4|0)!=0){v=u;w=f;c[v>>2]=c[w>>2];c[v+4>>2]=c[w+4>>2];c[v+8>>2]=c[w+8>>2];c[v+12>>2]=c[w+12>>2];w=f+8|0;if((c[w>>2]|0)!=0){x=0;i=t;return x|0}if((qk(u,g,h,j,k,l,m,n,o,p,q,r,s&-5)|0)<=-1){AN(0,0,1649248,(y=i,i=i+24|0,c[y>>2]=1720856,c[y+8>>2]=1693168,c[y+16>>2]=285,y)|0);i=y;cw();return 0}v=u+8|0;z=c[v>>2]|0;A=c[f+12>>2]|0;if((z|0)>(A|0)){AN(0,0,1649248,(y=i,i=i+24|0,c[y>>2]=1672184,c[y+8>>2]=1693168,c[y+16>>2]=286,y)|0);i=y;cw();return 0}if((z|0)<(A|0)){AN(0,16,1657544,(y=i,i=i+16|0,c[y>>2]=z,c[y+8>>2]=A,y)|0);i=y;B=c[v>>2]|0}else{B=z}z=c[f+4>>2]|0;A=u+4|0;u=c[A>>2]|0;C=B<<2;NN(z|0,u|0,C)|0;c[w>>2]=c[v>>2];A6(A);x=0;i=t;return x|0}A=f+4|0;c[A>>2]=0;c[f+12>>2]=0;c[f+8>>2]=0;v=A1((h<<3)+8|0)|0;w=v;if((v|0)==0){x=-12;i=t;return x|0}C=(p|0)==0;if(!((r|0)<3|C)){AN(0,0,1649248,(y=i,i=i+24|0,c[y>>2]=1638584,c[y+8>>2]=1693168,c[y+16>>2]=305,y)|0);i=y;cw();return 0}u=(h|0)>0;L3361:do{if(u){z=g*3|0;B=(s&2|0)==0;D=0;E=0;while(1){F=j+(ac(E,k)|0)|0;if((l|0)==1){H=a[F]|0;a[w+(D<<3)|0]=H;I=H}else if((l|0)==2){H=b[F>>1]&255;a[w+(D<<3)|0]=H;I=H}else{H=c[F>>2]&255;a[w+(D<<3)|0]=H;I=H}J=I&255;if((J|0)>(g|0)){if((J|0)>(z|0)|(I&255)>32){K=2399;break}H=m+(ac(E,n)|0)|0;if((o|0)==1){F=d[H]|0;c[w+(D<<3)+4>>2]=F;L=F}else if((o|0)==2){F=e[H>>1]|0;c[w+(D<<3)+4>>2]=F;L=F}else{F=c[H>>2]|0;c[w+(D<<3)+4>>2]=F;L=F}F=0;H=NW(1,0,I&255|0)|0;M=G;if(!((F|0)<(M|0)|(F|0)==(M|0)&L>>>0<H>>>0)){K=2405;break}if(B){N=L<<32-J}else{N=(d[242544+(L>>>8&255)|0]|0)<<16|(d[242544+(L&255)|0]|0)<<24|(d[242544+(L>>>16&255)|0]|0)<<8|(d[242544+(L>>>24)|0]|0)}c[w+(D<<3)+4>>2]=N;do{if(C){b[w+(D<<3)+2>>1]=E&65535}else{H=p+(ac(E,q)|0)|0;if((r|0)==2){b[w+(D<<3)+2>>1]=b[H>>1]|0;break}else if((r|0)==1){b[w+(D<<3)+2>>1]=d[H]|0;break}else{b[w+(D<<3)+2>>1]=c[H>>2]&65535;break}}}while(0);O=D+1|0}else{O=D}H=E+1|0;if((H|0)<(h|0)){D=O;E=H}else{K=2417;break}}if((K|0)==2399){AN(0,16,1629976,(y=i,i=i+8|0,c[y>>2]=J,y)|0);i=y;A4(v);x=-1;i=t;return x|0}else if((K|0)==2417){b6(v|0,O|0,8,56);if(!u){P=O;break}E=g*3|0;D=(s&2|0)==0;B=O;z=0;while(1){H=j+(ac(z,k)|0)|0;if((l|0)==1){M=a[H]|0;a[w+(B<<3)|0]=M;Q=M}else if((l|0)==2){M=b[H>>1]&255;a[w+(B<<3)|0]=M;Q=M}else{M=c[H>>2]&255;a[w+(B<<3)|0]=M;Q=M}R=Q&255;if(Q<<24>>24==0|(R|0)>(g|0)){S=B}else{if((R|0)>(E|0)|(Q&255)>32){K=2425;break}M=m+(ac(z,n)|0)|0;if((o|0)==1){H=d[M]|0;c[w+(B<<3)+4>>2]=H;T=H}else if((o|0)==2){H=e[M>>1]|0;c[w+(B<<3)+4>>2]=H;T=H}else{H=c[M>>2]|0;c[w+(B<<3)+4>>2]=H;T=H}H=0;M=NW(1,0,Q&255|0)|0;F=G;if(!((H|0)<(F|0)|(H|0)==(F|0)&T>>>0<M>>>0)){K=2431;break}if(D){U=T<<32-R}else{U=(d[242544+(T>>>8&255)|0]|0)<<16|(d[242544+(T&255)|0]|0)<<24|(d[242544+(T>>>16&255)|0]|0)<<8|(d[242544+(T>>>24)|0]|0)}c[w+(B<<3)+4>>2]=U;do{if(C){b[w+(B<<3)+2>>1]=z&65535}else{M=p+(ac(z,q)|0)|0;if((r|0)==1){b[w+(B<<3)+2>>1]=d[M]|0;break}else if((r|0)==2){b[w+(B<<3)+2>>1]=b[M>>1]|0;break}else{b[w+(B<<3)+2>>1]=c[M>>2]&65535;break}}}while(0);S=B+1|0}M=z+1|0;if((M|0)<(h|0)){B=S;z=M}else{P=S;break L3361}}if((K|0)==2431){AN(0,16,1623696,(y=i,i=i+1|0,i=i+7&-8,c[y>>2]=0,y)|0);i=y;A4(v);x=-1;i=t;return x|0}else if((K|0)==2425){AN(0,16,1629976,(y=i,i=i+8|0,c[y>>2]=R,y)|0);i=y;A4(v);x=-1;i=t;return x|0}}else if((K|0)==2405){AN(0,16,1623696,(y=i,i=i+1|0,i=i+7&-8,c[y>>2]=0,y)|0);i=y;A4(v);x=-1;i=t;return x|0}}else{b6(v|0,0,8,56);P=0}}while(0);y=qn(f,g,P,w,s)|0;A4(v);if((y|0)>=0){x=0;i=t;return x|0}A6(A);x=y;i=t;return x|0}function ql(a){a=a|0;A6(a+4|0);return}function qm(a,b){a=a|0;b=b|0;return((c[a+4>>2]|0)>>>1)-((c[b+4>>2]|0)>>>1)|0}function qn(e,f,g,h,j){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;k=i;l=1<<f;if((f|0)>30){m=-1;i=k;return m|0}n=e+8|0;o=c[n>>2]|0;p=o+l|0;c[n>>2]=p;n=e+12|0;q=c[n>>2]|0;do{if((p|0)>(q|0)){if((j&4|0)!=0){cw();return 0}r=(1<<c[e>>2])+q|0;c[n>>2]=r;s=e+4|0;t=A3(c[s>>2]|0,r,4)|0;c[s>>2]=t;if((t|0)==0){m=-12}else{break}i=k;return m|0}}while(0);if((o|0)<0){m=o;i=k;return m|0}n=e+4|0;q=c[n>>2]|0;p=q+(o<<2)|0;if((l|0)>0){t=0;do{s=t+o|0;b[q+(s<<2)+2>>1]=0;b[q+(s<<2)>>1]=-1;t=t+1|0;}while((t|0)<(l|0))}if((g|0)<=0){m=o;i=k;return m|0}l=32-f|0;t=(j&2|0)==0;q=0;s=p;L3462:while(1){p=h+(q<<3)|0;r=p|0;u=a[r]|0;v=u&255;w=h+(q<<3)+4|0;x=c[w>>2]|0;y=b[h+(q<<3)+2>>1]|0;do{if((v|0)>(f|0)){z=v-f|0;A=x>>>(l>>>0);a[r]=z&255;c[w>>2]=x<<f;B=q+1|0;L3466:do{if((B|0)<(g|0)){C=z;D=q;E=B;while(1){F=h+(E<<3)|0;G=(d[F]|0)-f|0;if((G|0)<1){H=C;I=D;J=E;break L3466}K=h+(E<<3)+4|0;L=c[K>>2]|0;if((L>>>(l>>>0)|0)!=(A|0)){H=C;I=D;J=E;break L3466}a[F]=G&255;c[K>>2]=L<<f;L=(C|0)>(G|0)?C:G;G=E+1|0;if((G|0)<(g|0)){C=L;D=E;E=G}else{H=L;I=E;J=G;break}}}else{H=z;I=q;J=B}}while(0);B=(H|0)>(f|0)?f:H;if(t){M=A}else{M=(d[242544+(A>>>8&255)|0]<<16|d[242544+(A&255)|0]<<24|d[242544+(A>>>16&255)|0]<<8|d[242544+(A>>>24)|0])>>>(l>>>0)}b[s+(M<<2)+2>>1]=-B&65535;z=qn(e,B,J-q|0,p,j)|0;if((z|0)<0){m=z;N=2486;break L3462}B=c[n>>2]|0;b[B+(M+o<<2)>>1]=z&65535;O=B+(o<<2)|0;P=I}else{B=1<<f-v;if(t){Q=1;R=x>>>(l>>>0)}else{Q=1<<v;R=d[242544+(x>>>8&255)|0]<<16|d[242544+(x&255)|0]<<24|d[242544+(x>>>16&255)|0]<<8|d[242544+(x>>>24)|0]}if((B|0)<=0){O=s;P=q;break}z=u&255;E=R;D=0;while(1){C=s+(E<<2)+2|0;if((b[C>>1]|0)!=0){N=2473;break L3462}b[C>>1]=z;b[s+(E<<2)>>1]=y;C=D+1|0;if((C|0)<(B|0)){E=E+Q|0;D=C}else{O=s;P=q;break}}}}while(0);y=P+1|0;if((y|0)<(g|0)){q=y;s=O}else{m=o;N=2487;break}}if((N|0)==2473){AN(0,16,1617960,(o=i,i=i+1|0,i=i+7&-8,c[o>>2]=0,o)|0);i=o;m=-1094995529;i=k;return m|0}else if((N|0)==2486){i=k;return m|0}else if((N|0)==2487){i=k;return m|0}return 0}function qo(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,i=0,j=0.0,k=0,l=0.0;e=1<<b;NM(a|0,0,112);c[a>>2]=b;c[a+4>>2]=d;do{if((d|0)==0&(b|0)==5){c[a+104>>2]=18}else{f=b+2|0;qw(f);c[a+96>>2]=c[251072+(f<<2)>>2];f=(e|0)/2|0;h=a+100|0;c[h>>2]=A1(f<<2)|0;if((q3(a+8|0,b,(d|0)==1|0)|0)<0){A4(c[h>>2]|0);i=-1;return i|0}if((e|0)>1){j=3.141592653589793/+(e<<1|0);k=0;do{l=.5/+V(+(+(k<<1|1|0)*j));g[(c[h>>2]|0)+(k<<2)>>2]=l;k=k+1|0;}while((k|0)<(f|0))}if((d|0)==3){c[a+104>>2]=16;break}else if((d|0)==1){c[a+104>>2]=4;break}else if((d|0)==2){c[a+104>>2]=56;break}else if((d|0)==0){c[a+104>>2]=12;break}else{break}}}while(0);c[a+108>>2]=38;i=0;return i|0}function qp(a,b){a=a|0;b=b|0;c$[c[a+108>>2]&127](b,b);return}function qq(a,b){a=a|0;b=b|0;var d=0,e=0.0,f=0,h=0.0,i=0,j=0,k=0.0,l=0,m=0.0,n=0,o=0.0,p=0,q=0,r=0.0,s=0.0,t=0.0,u=0.0,v=0,w=0.0;d=1<<c[a>>2];e=+g[b>>2];f=b+(d<<2)|0;h=(e- +g[f>>2])*-.5;i=(d|0)/2|0;L3512:do{if((d|0)>1){j=a+96|0;k=h;l=0;m=e;while(1){n=b+(d-l<<2)|0;o=+g[n>>2];p=l<<1;q=c[j>>2]|0;r=m-o;s=r*+g[q+(d-p<<2)>>2];t=k+r*+g[q+(p<<2)>>2];r=(m+o)*.5;g[b+(l<<2)>>2]=r-s;g[n>>2]=r+s;n=l+1|0;if((n|0)>=(i|0)){u=t;break L3512}k=t;l=n;m=+g[b+(n<<2)>>2]}}else{u=h}}while(0);c$[c[a+92>>2]&127](a+8|0,b);a=b+4|0;g[f>>2]=+g[a>>2];g[a>>2]=u;if((d|0)<3){return}else{v=3;w=u}do{a=b+(v<<2)|0;w=w- +g[a>>2];g[a>>2]=w;v=v+2|0;}while((v|0)<=(d|0));return}function qr(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0,i=0,j=0,k=0.0,l=0,m=0.0,n=0.0,o=0.0,p=0.0;d=1<<c[a>>2];e=(d|0)/2|0;if((d|0)>1){f=d-1|0;h=a+96|0;i=0;do{j=b+(i<<2)|0;k=+g[j>>2];l=b+(f-i<<2)|0;m=+g[l>>2];n=(k-m)*+g[(c[h>>2]|0)+(d-(i<<1|1)<<2)>>2];o=(k+m)*.5;g[j>>2]=o+n;g[l>>2]=o-n;i=i+1|0;}while((i|0)<(e|0))}c$[c[a+92>>2]&127](a+8|0,b);e=b+4|0;n=+g[e>>2];g[e>>2]=n*-1.0;e=d-2|0;if((e|0)<=-1){return}i=a+96|0;o=n*.5;a=d;h=e;while(1){e=b+(h<<2)|0;n=+g[e>>2];f=b+(a-1<<2)|0;m=+g[f>>2];l=c[i>>2]|0;k=+g[l+(h<<2)>>2];p=+g[l+(d-h<<2)>>2];g[e>>2]=n*k+m*p;g[f>>2]=o;f=h-2|0;if((f|0)>-1){o=o+(n*p-m*k);a=h;h=f}else{break}}return}function qs(a,b){a=a|0;b=b|0;var d=0,e=0,f=0.0,h=0.0,i=0,j=0,k=0,l=0,m=0.0,n=0,o=0.0,p=0,q=0.0,r=0.0;d=1<<c[a>>2];e=d-1|0;f=+g[b+(e<<2)>>2];h=1.0/+(d|0);i=d-2|0;if((i|0)>1){j=a+96|0;k=d;l=i;while(1){i=b+(l<<2)|0;m=+g[i>>2];n=b+(k-1<<2)|0;o=+g[b+(k-3<<2)>>2]- +g[n>>2];p=c[j>>2]|0;q=+g[p+(l<<2)>>2];r=+g[p+(d-l<<2)>>2];g[i>>2]=m*q+o*r;g[n>>2]=m*r-o*q;n=l-2|0;if((n|0)>1){k=l;l=n}else{break}}}g[b+4>>2]=f*2.0;c$[c[a+92>>2]&127](a+8|0,b);l=(d|0)/2|0;if((d|0)<=1){return}d=a+100|0;a=0;do{k=b+(a<<2)|0;f=h*+g[k>>2];j=b+(e-a<<2)|0;q=h*+g[j>>2];o=+g[(c[d>>2]|0)+(a<<2)>>2]*(f-q);r=f+q;g[k>>2]=r+o;g[j>>2]=r-o;a=a+1|0;}while((a|0)<(l|0));return}function qt(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0,i=0,j=0.0,k=0,l=0.0,m=0.0,n=0.0,o=0,p=0.0,q=0,r=0;d=1<<c[a>>2];g[b>>2]=0.0;e=(d|0)/2|0;if((d|0)>3){f=a+96|0;h=1;do{i=b+(h<<2)|0;j=+g[i>>2];k=b+(d-h<<2)|0;l=+g[k>>2];m=(j+l)*+g[(c[f>>2]|0)+(d-(h<<1)<<2)>>2];n=(j-l)*.5;g[i>>2]=n+m;g[k>>2]=m-n;h=h+1|0;}while((h|0)<(e|0))}h=b+(e<<2)|0;g[h>>2]=+g[h>>2]*2.0;c$[c[a+92>>2]&127](a+8|0,b);n=+g[b>>2]*.5;g[b>>2]=n;a=d-2|0;if((a|0)>1){o=1;p=n}else{q=d-1|0;r=b+(q<<2)|0;g[r>>2]=0.0;return}while(1){h=b+(o+1<<2)|0;n=p+ +g[h>>2];g[h>>2]=n;h=o+2|0;g[b+(o<<2)>>2]=-0.0- +g[b+(h<<2)>>2];if((h|0)<(a|0)){o=h;p=n}else{break}}q=d-1|0;r=b+(q<<2)|0;g[r>>2]=0.0;return}function qu(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;d=c[b>>2]|0;e=c[b+124>>2]|0;f=e+d|0;g=d-e<<1;N2(g,(g|0)<0|0?-1:0,1075036753,0)|0;g=G;e=c[b+60>>2]|0;d=c[b+64>>2]|0;h=d+e|0;i=e-d<<5;N2(i,(i|0)<0|0?-1:0,1367679739,0)|0;i=G;d=h+f|0;e=f-h<<1;N2(e,(e|0)<0|0?-1:0,1078937202,0)|0;e=G;h=i+g|0;f=i-g<<1;N2(f,(f|0)<0|0?-1:0,-1078937202,-1)|0;f=G;g=c[b+28>>2]|0;i=c[b+96>>2]|0;j=i+g|0;k=g-i<<1;N2(k,(k|0)<0|0?-1:0,1449139879,0)|0;k=G;i=c[b+32>>2]|0;g=c[b+92>>2]|0;l=g+i|0;m=i-g<<1;N2(m,(m|0)<0|0?-1:0,1598879467,0)|0;m=G;g=l+j|0;i=j-l<<4;N2(i,(i|0)<0|0?-1:0,1369329156,0)|0;i=G;l=m+k|0;j=m-k<<4;N2(j,(j|0)<0|0?-1:0,-1369329156,-1)|0;j=G;k=g+d|0;m=d-g<<1;N2(m,(m|0)<0|0?-1:0,1094777670,0)|0;m=G;g=i+e|0;d=i-e<<1;N2(d,(d|0)<0|0?-1:0,-1094777670,-1)|0;d=G;e=l+h|0;i=h-l<<1;N2(i,(i|0)<0|0?-1:0,1094777670,0)|0;i=G;l=j+f|0;h=j-f<<1;N2(h,(h|0)<0|0?-1:0,-1094777670,-1)|0;h=G;f=c[b+12>>2]|0;j=c[b+112>>2]|0;n=j+f|0;o=f-j<<1;N2(o,(o|0)<0|0?-1:0,1140405281,0)|0;o=G;j=c[b+48>>2]|0;f=c[b+76>>2]|0;p=f+j|0;q=j-f<<2;N2(q,(q|0)<0|0?-1:0,1593609622,0)|0;q=G;f=p+n|0;j=n-p<<1;N2(j,(j|0)<0|0?-1:0,1389039203,0)|0;j=G;p=q+o|0;n=q-o<<1;N2(n,(n|0)<0|0?-1:0,-1389039203,-1)|0;n=G;o=c[b+16>>2]|0;q=c[b+108>>2]|0;r=q+o|0;s=o-q<<1;N2(s,(s|0)<0|0?-1:0,1187781572,0)|0;s=G;q=c[b+44>>2]|0;o=c[b+80>>2]|0;t=o+q|0;u=q-o<<2;N2(u,(u|0)<0|0?-1:0,1255676567,0)|0;u=G;o=t+r|0;q=r-t<<1;N2(q,(q|0)<0|0?-1:0,1692549166,0)|0;q=G;t=u+s|0;r=u-s<<1;N2(r,(r|0)<0|0?-1:0,-1692549166,-1)|0;r=G;s=o+f|0;u=f-o<<3;N2(u,(u|0)<0|0?-1:0,1375954754,0)|0;u=G;o=q+j|0;f=q-j<<3;N2(f,(f|0)<0|0?-1:0,-1375954754,-1)|0;f=G;j=t+p|0;q=p-t<<3;N2(q,(q|0)<0|0?-1:0,1375954754,0)|0;q=G;t=r+n|0;p=r-n<<3;N2(p,(p|0)<0|0?-1:0,-1375954754,-1)|0;p=G;n=s+k|0;r=k-s<<1;N2(r,(r|0)<0|0?-1:0,1162209775,0)|0;r=G;s=u+m|0;k=u-m<<1;N2(k,(k|0)<0|0?-1:0,-1162209775,-1)|0;k=G;m=o+g|0;u=g-o<<1;N2(u,(u|0)<0|0?-1:0,1162209775,0)|0;u=G;o=f+d|0;g=f-d<<1;N2(g,(g|0)<0|0?-1:0,-1162209775,-1)|0;g=G;d=j+e|0;f=e-j<<1;N2(f,(f|0)<0|0?-1:0,1162209775,0)|0;f=G;j=q+i|0;e=q-i<<1;N2(e,(e|0)<0|0?-1:0,-1162209775,-1)|0;e=G;i=t+l|0;q=l-t<<1;N2(q,(q|0)<0|0?-1:0,1162209775,0)|0;q=G;t=p+h|0;l=p-h<<1;N2(l,(l|0)<0|0?-1:0,-1162209775,-1)|0;l=G;h=c[b+4>>2]|0;p=c[b+120>>2]|0;v=p+h|0;w=h-p<<1;N2(w,(w|0)<0|0?-1:0,1085490621,0)|0;w=G;p=c[b+56>>2]|0;h=c[b+68>>2]|0;x=h+p|0;y=p-h<<3;N2(y,(y|0)<0|0?-1:0,1829445839,0)|0;y=G;h=x+v|0;p=v-x<<1;N2(p,(p|0)<0|0?-1:0,1122057232,0)|0;p=G;x=y+w|0;v=y-w<<1;N2(v,(v|0)<0|0?-1:0,-1122057232,-1)|0;v=G;w=c[b+24>>2]|0;y=c[b+100>>2]|0;z=y+w|0;A=w-y<<1;N2(A,(A|0)<0|0?-1:0,1336817425,0)|0;A=G;y=c[b+36>>2]|0;w=c[b+88>>2]|0;B=w+y|0;C=y-w<<1;N2(C,(C|0)<0|0?-1:0,1802489638,0)|0;C=G;w=B+z|0;y=z-B<<2;N2(y,(y|0)<0|0?-1:0,1849463489,0)|0;y=G;B=C+A|0;z=C-A<<2;N2(z,(z|0)<0|0?-1:0,-1849463489,-1)|0;z=G;A=w+h|0;C=h-w<<1;N2(C,(C|0)<0|0?-1:0,1291378312,0)|0;C=G;w=y+p|0;h=y-p<<1;N2(h,(h|0)<0|0?-1:0,-1291378312,-1)|0;h=G;p=B+x|0;y=x-B<<1;N2(y,(y|0)<0|0?-1:0,1291378312,0)|0;y=G;B=z+v|0;x=z-v<<1;N2(x,(x|0)<0|0?-1:0,-1291378312,-1)|0;x=G;v=c[b+8>>2]|0;z=c[b+116>>2]|0;D=z+v|0;E=v-z<<1;N2(E,(E|0)<0|0?-1:0,1106914669,0)|0;E=G;z=c[b+52>>2]|0;v=c[b+72>>2]|0;F=v+z|0;H=z-v<<3;N2(H,(H|0)<0|0?-1:0,1104762768,0)|0;H=G;v=F+D|0;z=D-F<<1;N2(z,(z|0)<0|0?-1:0,1217503044,0)|0;z=G;F=H+E|0;D=H-E<<1;N2(D,(D|0)<0|0?-1:0,-1217503044,-1)|0;D=G;E=c[b+20>>2]|0;H=c[b+104>>2]|0;I=H+E|0;J=E-H<<1;N2(J,(J|0)<0|0?-1:0,1251843312,0)|0;J=G;H=c[b+40>>2]|0;E=c[b+84>>2]|0;b=E+H|0;K=H-E<<1;N2(K,(K|0)<0|0?-1:0,2088574387,0)|0;K=G;E=b+I|0;H=I-b<<2;N2(H,(H|0)<0|0?-1:0,1138893993,0)|0;H=G;b=K+J|0;I=K-J<<2;N2(I,(I|0)<0|0?-1:0,-1138893993,-1)|0;I=G;J=E+v|0;K=v-E<<1;N2(K,(K|0)<0|0?-1:0,1932684223,0)|0;K=G;E=H+z|0;v=H-z<<1;N2(v,(v|0)<0|0?-1:0,-1932684223,-1)|0;v=G;z=b+F|0;H=F-b<<1;N2(H,(H|0)<0|0?-1:0,1932684223,0)|0;H=G;b=I+D|0;F=I-D<<1;N2(F,(F|0)<0|0?-1:0,-1932684223,-1)|0;F=G;D=J+A|0;I=A-J<<2;N2(I,(I|0)<0|0?-1:0,1402911301,0)|0;I=G;J=K+C|0;A=K-C<<2;N2(A,(A|0)<0|0?-1:0,-1402911301,-1)|0;A=G;C=E+w|0;K=w-E<<2;N2(K,(K|0)<0|0?-1:0,1402911301,0)|0;K=G;E=v+h|0;w=v-h<<2;N2(w,(w|0)<0|0?-1:0,-1402911301,-1)|0;w=G;h=z+p|0;v=p-z<<2;N2(v,(v|0)<0|0?-1:0,1402911301,0)|0;v=G;z=H+y|0;p=H-y<<2;N2(p,(p|0)<0|0?-1:0,-1402911301,-1)|0;p=G;y=b+B|0;H=B-b<<2;N2(H,(H|0)<0|0?-1:0,1402911301,0)|0;H=G;b=F+x|0;B=F-x<<2;N2(B,(B|0)<0|0?-1:0,-1402911301,-1)|0;B=G;x=n-D<<1;N2(x,(x|0)<0|0?-1:0,1518500250,0)|0;x=G;F=I-r<<1;N2(F,(F|0)<0|0?-1:0,-1518500250,-1)|0;F=G;L=s-J<<1;N2(L,(L|0)<0|0?-1:0,1518500250,0)|0;L=G;M=A-k<<1;N2(M,(M|0)<0|0?-1:0,-1518500250,-1)|0;M=G;N=A+k+M|0;k=m-C<<1;N2(k,(k|0)<0|0?-1:0,1518500250,0)|0;k=G;A=K-u<<1;N2(A,(A|0)<0|0?-1:0,-1518500250,-1)|0;A=G;O=K+u+A|0;u=o-E<<1;N2(u,(u|0)<0|0?-1:0,1518500250,0)|0;u=G;K=w-g<<1;N2(K,(K|0)<0|0?-1:0,-1518500250,-1)|0;K=G;P=w+g+K|0;g=E+o+P|0;o=P+u|0;P=K+u|0;u=d-h<<1;N2(u,(u|0)<0|0?-1:0,1518500250,0)|0;u=G;E=v-f<<1;N2(E,(E|0)<0|0?-1:0,-1518500250,-1)|0;E=G;w=v+f+E|0;f=j-z<<1;N2(f,(f|0)<0|0?-1:0,1518500250,0)|0;f=G;v=p-e<<1;N2(v,(v|0)<0|0?-1:0,-1518500250,-1)|0;v=G;Q=p+e+v|0;e=z+j+Q|0;j=Q+f|0;Q=v+f|0;f=i-y<<1;N2(f,(f|0)<0|0?-1:0,1518500250,0)|0;f=G;z=H-q<<1;N2(z,(z|0)<0|0?-1:0,-1518500250,-1)|0;z=G;p=H+q+z|0;q=t-b<<1;N2(q,(q|0)<0|0?-1:0,1518500250,0)|0;q=G;H=B-l<<1;N2(H,(H|0)<0|0?-1:0,-1518500250,-1)|0;H=G;R=B+l+H|0;l=b+t+R|0;t=R+q|0;R=H+q|0;c[a>>2]=D+n;c[a+64>>2]=x;c[a+32>>2]=I+r+F;c[a+96>>2]=F;c[a+16>>2]=J+s+N;c[a+80>>2]=M+L;c[a+48>>2]=N+L;c[a+112>>2]=M;c[a+8>>2]=C+m+g;c[a+72>>2]=P+k;c[a+40>>2]=o+O;c[a+104>>2]=K+A;c[a+24>>2]=g+O;c[a+88>>2]=P+A;c[a+56>>2]=o+k;c[a+120>>2]=K;K=y+i+l|0;i=l+p|0;l=t+p|0;p=t+f|0;t=R+f|0;f=R+z|0;R=H+z|0;c[a+4>>2]=h+d+K;c[a+68>>2]=t+u;c[a+36>>2]=l+w;c[a+100>>2]=R+E;c[a+20>>2]=i+e;c[a+84>>2]=f+Q;c[a+52>>2]=p+j;c[a+116>>2]=H+v;c[a+12>>2]=K+e;c[a+76>>2]=t+Q;c[a+44>>2]=l+j;c[a+108>>2]=R+v;c[a+28>>2]=i+w;c[a+92>>2]=f+E;c[a+60>>2]=p+u;c[a+124>>2]=H;return}function qv(a,b){a=a|0;b=b|0;var c=0.0,d=0.0,e=0.0,f=0.0,h=0.0,i=0.0,j=0.0,k=0.0,l=0.0,m=0.0,n=0.0,o=0.0,p=0.0,q=0.0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0.0,y=0.0,z=0.0,A=0.0,B=0.0,C=0.0,D=0.0,E=0.0,F=0.0,G=0.0,H=0.0,I=0.0,J=0.0,K=0.0,L=0.0,M=0.0,N=0.0,O=0.0,P=0.0,Q=0.0;c=+g[b>>2];d=+g[b+124>>2];e=c+d;f=(c-d)*.5006030201911926;d=+g[b+60>>2];c=+g[b+64>>2];h=d+c;i=(d-c)*10.190008163452148;c=e+h;d=(e-h)*.5024192929267883;h=f+i;e=(i-f)*-.5024192929267883;f=+g[b+28>>2];i=+g[b+96>>2];j=f+i;k=(f-i)*.6748083233833313;i=+g[b+32>>2];f=+g[b+92>>2];l=i+f;m=(i-f)*.744536280632019;f=j+l;i=(j-l)*5.10114860534668;l=k+m;j=(m-k)*-5.10114860534668;k=c+f;m=(c-f)*.509795606136322;f=d+i;c=(i-d)*-.509795606136322;d=h+l;i=(h-l)*.509795606136322;l=e+j;h=(j-e)*-.509795606136322;e=+g[b+12>>2];j=+g[b+112>>2];n=e+j;o=(e-j)*.5310425758361816;j=+g[b+48>>2];e=+g[b+76>>2];p=j+e;q=(j-e)*1.4841645956039429;e=n+p;j=(n-p)*.6468217968940735;p=o+q;n=(q-o)*-.6468217968940735;o=+g[b+16>>2];q=+g[b+108>>2];r=o+q;s=(o-q)*.5531039237976074;q=+g[b+44>>2];o=+g[b+80>>2];t=q+o;u=(q-o)*1.1694399118423462;o=r+t;q=(r-t)*.7881546020507812;t=s+u;r=(u-s)*-.7881546020507812;s=e+o;u=(e-o)*2.562915563583374;o=j+q;e=(q-j)*-2.562915563583374;j=p+t;q=(p-t)*2.562915563583374;t=n+r;p=(r-n)*-2.562915563583374;n=k+s;r=(k-s)*.5411961078643799;s=m+u;k=(u-m)*-.5411961078643799;m=f+o;u=(f-o)*.5411961078643799;o=c+e;f=(e-c)*-.5411961078643799;c=d+j;e=(d-j)*.5411961078643799;j=i+q;d=(q-i)*-.5411961078643799;i=l+t;q=(l-t)*.5411961078643799;t=h+p;l=(p-h)*-.5411961078643799;h=+g[b+4>>2];p=+g[b+120>>2];v=h+p;w=(h-p)*.5054709315299988;p=+g[b+56>>2];h=+g[b+68>>2];x=p+h;y=(p-h)*3.4076085090637207;h=v+x;p=(v-x)*.522498607635498;x=w+y;v=(y-w)*-.522498607635498;w=+g[b+24>>2];y=+g[b+100>>2];z=w+y;A=(w-y)*.6225041151046753;y=+g[b+36>>2];w=+g[b+88>>2];B=y+w;C=(y-w)*.839349627494812;w=z+B;y=(z-B)*1.722447156906128;B=A+C;z=(C-A)*-1.722447156906128;A=h+w;C=(h-w)*.601344883441925;w=p+y;h=(y-p)*-.601344883441925;p=x+B;y=(x-B)*.601344883441925;B=v+z;x=(z-v)*-.601344883441925;v=+g[b+8>>2];z=+g[b+116>>2];D=v+z;E=(v-z)*.5154473185539246;z=+g[b+52>>2];v=+g[b+72>>2];F=z+v;G=(z-v)*2.0577809810638428;v=D+F;z=(D-F)*.5669440627098083;F=E+G;D=(G-E)*-.5669440627098083;E=+g[b+20>>2];G=+g[b+104>>2];H=E+G;I=(E-G)*.5829349756240845;G=+g[b+40>>2];E=+g[b+84>>2];J=G+E;K=(G-E)*.9725682139396667;E=H+J;G=(H-J)*1.0606776475906372;J=I+K;H=(K-I)*-1.0606776475906372;I=v+E;K=(v-E)*.8999761939048767;E=z+G;v=(G-z)*-.8999761939048767;z=F+J;G=(F-J)*.8999761939048767;J=D+H;F=(H-D)*-.8999761939048767;D=A+I;H=(A-I)*1.3065630197525024;I=C+K;A=(K-C)*-1.3065630197525024;C=w+E;K=(w-E)*1.3065630197525024;E=h+v;w=(v-h)*-1.3065630197525024;h=p+z;v=(p-z)*1.3065630197525024;z=y+G;p=(G-y)*-1.3065630197525024;y=B+J;G=(B-J)*1.3065630197525024;J=x+F;B=(F-x)*-1.3065630197525024;x=(H-r)*-.7071067690849304;F=(s-I)*.7071067690849304;L=(A-k)*-.7071067690849304;M=k+A+L;A=(m-C)*.7071067690849304;k=(K-u)*-.7071067690849304;N=u+K+k;K=(o-E)*.7071067690849304;u=(w-f)*-.7071067690849304;O=f+w+u;w=o+E+O;E=K+O;O=K+u;K=(c-h)*.7071067690849304;o=(v-e)*-.7071067690849304;f=e+v+o;v=(j-z)*.7071067690849304;e=(p-d)*-.7071067690849304;P=d+p+e;p=j+z+P;z=v+P;P=v+e;v=(i-y)*.7071067690849304;j=(G-q)*-.7071067690849304;d=q+G+j;G=(t-J)*.7071067690849304;q=(B-l)*-.7071067690849304;Q=l+B+q;B=t+J+Q;J=G+Q;Q=G+q;g[a>>2]=n+D;g[a+64>>2]=(n-D)*.7071067690849304;g[a+32>>2]=r+H+x;g[a+96>>2]=x;g[a+16>>2]=s+I+M;g[a+80>>2]=F+L;g[a+48>>2]=F+M;g[a+112>>2]=L;g[a+8>>2]=m+C+w;g[a+72>>2]=A+O;g[a+40>>2]=N+E;g[a+104>>2]=k+u;g[a+24>>2]=N+w;g[a+88>>2]=k+O;g[a+56>>2]=A+E;g[a+120>>2]=u;u=i+y+B;y=d+B;B=d+J;d=v+J;J=v+Q;v=j+Q;Q=j+q;g[a+4>>2]=c+h+u;g[a+68>>2]=K+J;g[a+36>>2]=f+B;g[a+100>>2]=o+Q;g[a+20>>2]=p+y;g[a+84>>2]=P+v;g[a+52>>2]=z+d;g[a+116>>2]=e+q;g[a+12>>2]=p+u;g[a+76>>2]=P+J;g[a+44>>2]=z+B;g[a+108>>2]=e+Q;g[a+28>>2]=f+y;g[a+92>>2]=o+v;g[a+60>>2]=K+d;g[a+124>>2]=q;return}function qw(a){a=a|0;var b=0,d=0.0,e=0,f=0;b=1<<a;d=6.283185307179586/+(b|0);e=c[251072+(a<<2)>>2]|0;a=(b|0)/4|0;if((b|0)<-3){return}else{f=0}do{g[e+(f<<2)>>2]=+U(+(d*+(f|0)));f=f+1|0;}while((f|0)<=(a|0));if((b|0)<=7){return}f=(b|0)/2|0;b=1;do{g[e+(f-b<<2)>>2]=+g[e+(b<<2)>>2];b=b+1|0;}while((b|0)<(a|0));return}function qx(a,d,e){a=a|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0.0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0;do{if((d-2|0)>>>0>14){f=a+8|0}else{h=a|0;c[h>>2]=d;i=1<<d;j=A1(i<<1)|0;k=a+8|0;c[k>>2]=j;if((j|0)==0){f=k;break}j=A1(i<<3)|0;c[a+12>>2]=j;if((j|0)==0){f=k;break}j=a+4|0;c[j>>2]=e;l=a+56|0;c[l>>2]=0;c[a+32>>2]=32;c[a+36>>2]=44;c[a+40>>2]=62;c[a+44>>2]=8;c[a+48>>2]=30;c[a+52>>2]=30;do{if((d|0)<4){m=0}else{n=4;do{o=1<<n;p=6.283185307179586/+(o|0);q=c[251072+(n<<2)>>2]|0;r=(o|0)/4|0;do{if((o|0)>=-3){s=0;do{g[q+(s<<2)>>2]=+U(+(p*+(s|0)));s=s+1|0;}while((s|0)<=(r|0));if((o|0)<=7){break}s=(o|0)/2|0;t=1;do{g[q+(s-t<<2)>>2]=+g[q+(t<<2)>>2];t=t+1|0;}while((t|0)<(r|0))}}while(0);n=n+1|0;}while((n|0)<=(d|0));n=c[l>>2]|0;if((n|0)!=2){m=n;break}n=1<<c[h>>2];if((n|0)<=0){u=0;return u|0}r=n-1|0;q=0;while(1){o=n;t=q;L3595:while(1){s=o;while(1){if((s|0)<33){break L3595}v=(s|0)/2|0;if((v|0)>(t|0)){s=v}else{break}}w=(s*3|0|0)/4|0;o=(s|0)/4|0;t=t-((w|0)>(t|0)?v:w)|0}if((t|0)>15){o=0;do{w=(c[337432+(o<<2)>>2]|0)+q&65535;x=r&-(qA(o+q|0,n,c[j>>2]|0)|0);b[(c[k>>2]|0)+(x<<1)>>1]=w;o=o+1|0;}while((o|0)<16)}else{o=0;do{t=o+q|0;w=r&-(qA(t,n,c[j>>2]|0)|0);b[(c[k>>2]|0)+(w<<1)>>1]=(t>>>1&3|t&65528|t<<2&4)&65535;o=o+1|0;}while((o|0)<16)}o=q+16|0;if((o|0)<(n|0)){q=o}else{u=0;break}}return u|0}}while(0);if((i|0)<=0){u=0;return u|0}h=i-1|0;q=0;n=m;while(1){if((n|0)==1){y=(q>>>1&1|q&65532|q<<1&2)&65535}else{y=q&65535}r=h&-(qA(q,i,c[j>>2]|0)|0);b[(c[k>>2]|0)+(r<<1)>>1]=y;r=q+1|0;if((r|0)>=(i|0)){u=0;break}q=r;n=c[l>>2]|0}return u|0}}while(0);A6(f);A6(a+12|0);u=-1;return u|0}function qy(a,b){a=a|0;b=b|0;var d=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0;d=c[a+8>>2]|0;f=1<<c[a>>2];g=a+12|0;if((f|0)>0){h=0}else{i=b;j=c[g>>2]|0;k=j;l=f<<3;NN(i|0,k|0,l)|0;return}do{a=b+(h<<3)|0;m=(c[g>>2]|0)+((e[d+(h<<1)>>1]|0)<<3)|0;n=c[a+4>>2]|0;c[m>>2]=c[a>>2];c[m+4>>2]=n;h=h+1|0;}while((h|0)<(f|0));i=b;j=c[g>>2]|0;k=j;l=f<<3;NN(i|0,k|0,l)|0;return}function qz(a,b){a=a|0;b=b|0;c_[c[236072+((c[a>>2]|0)-2<<2)>>2]&127](b);return}function qA(a,b,c){a=a|0;b=b|0;c=c|0;var d=0,e=0;if((b|0)<3){d=a&1;return d|0}e=b>>1;if((e&a|0)==0){return(qA(a,e,c)|0)<<1|0}e=b>>2;b=(qA(a,e,c)|0)<<2;if(((e&a|0)==0|0)==(c|0)){d=b|1;return d|0}else{d=b-1|0;return d|0}return 0}function qB(a){a=a|0;A6(a+8|0);A6(a+12|0);return}function qC(a){a=a|0;var b=0,c=0.0,d=0,e=0.0,f=0.0,h=0.0,i=0,j=0,k=0.0,l=0.0,m=0,n=0,o=0.0,p=0.0;b=a|0;c=+g[b>>2];d=a+8|0;e=+g[d>>2];f=c-e;h=c+e;i=a+24|0;e=+g[i>>2];j=a+16|0;c=+g[j>>2];k=e-c;l=e+c;g[j>>2]=h-l;g[b>>2]=h+l;b=a+4|0;l=+g[b>>2];j=a+12|0;h=+g[j>>2];c=l-h;e=l+h;m=a+20|0;h=+g[m>>2];n=a+28|0;l=+g[n>>2];o=h-l;p=h+l;g[n>>2]=c-k;g[j>>2]=k+c;g[i>>2]=f-o;g[d>>2]=f+o;g[m>>2]=e-p;g[b>>2]=e+p;return}function qD(a){a=a|0;var b=0,c=0.0,d=0,e=0.0,f=0.0,h=0.0,i=0,j=0,k=0.0,l=0.0,m=0,n=0,o=0.0,p=0.0,q=0,r=0,s=0.0,t=0.0,u=0,v=0,w=0.0,x=0.0,y=0,z=0,A=0.0,B=0.0,C=0,D=0,E=0.0,F=0,G=0.0,H=0,I=0.0,J=0.0,K=0.0,L=0.0;b=a|0;c=+g[b>>2];d=a+8|0;e=+g[d>>2];f=c-e;h=c+e;i=a+24|0;e=+g[i>>2];j=a+16|0;c=+g[j>>2];k=e-c;l=e+c;c=h-l;e=h+l;m=a+4|0;l=+g[m>>2];n=a+12|0;h=+g[n>>2];o=l-h;p=l+h;q=a+20|0;h=+g[q>>2];r=a+28|0;l=+g[r>>2];s=h-l;t=h+l;l=o-k;h=k+o;o=f-s;k=f+s;s=p-t;f=p+t;u=a+32|0;t=+g[u>>2];v=a+40|0;p=+g[v>>2];w=t+p;x=t-p;y=a+36|0;p=+g[y>>2];z=a+44|0;t=+g[z>>2];A=p+t;B=p-t;C=a+48|0;t=+g[C>>2];D=a+56|0;p=+g[D>>2];E=t+p;F=a+52|0;G=+g[F>>2];H=a+60|0;I=+g[H>>2];J=G+I;K=E-w;L=w+E;g[u>>2]=e-L;g[b>>2]=L+e;g[F>>2]=s-K;g[q>>2]=K+s;s=A-J;K=A+J;g[C>>2]=c-s;g[j>>2]=s+c;g[y>>2]=f-K;g[m>>2]=f+K;K=x*.7071067690849304-B*-.7071067690849304;f=x*-.7071067690849304+B*.7071067690849304;B=(t-p)*.7071067690849304;p=(G-I)*.7071067690849304;I=B-p;G=B+p;p=I-K;B=K+I;g[v>>2]=k-B;g[d>>2]=k+B;g[H>>2]=l-p;g[r>>2]=p+l;l=f-G;p=f+G;g[D>>2]=o-l;g[i>>2]=o+l;g[z>>2]=h-p;g[n>>2]=h+p;return}function qE(a){a=a|0;var b=0.0,c=0.0,d=0,e=0.0,f=0,h=0.0,i=0.0,j=0.0,k=0,l=0,m=0.0,n=0.0,o=0,p=0,q=0.0,r=0.0,s=0,t=0,u=0.0,v=0.0,w=0,x=0,y=0.0,z=0.0,A=0,B=0,C=0.0,D=0.0,E=0,F=0,G=0.0,H=0.0,I=0.0,J=0,K=0,L=0.0,M=0.0,N=0,O=0,P=0.0,Q=0,R=0.0,S=0,T=0.0,U=0.0,V=0.0;b=+g[716227];c=+g[716229];qD(a);d=a+64|0;e=+g[d>>2];f=a+72|0;h=+g[f>>2];i=e-h;j=e+h;k=a+88|0;h=+g[k>>2];l=a+80|0;e=+g[l>>2];m=h-e;n=h+e;e=j-n;h=j+n;o=a+68|0;n=+g[o>>2];p=a+76|0;j=+g[p>>2];q=n-j;r=n+j;s=a+84|0;j=+g[s>>2];t=a+92|0;n=+g[t>>2];u=j-n;v=j+n;n=q-m;j=m+q;q=i-u;m=i+u;u=r-v;i=r+v;w=a+96|0;v=+g[w>>2];x=a+104|0;r=+g[x>>2];y=v-r;z=v+r;A=a+120|0;r=+g[A>>2];B=a+112|0;v=+g[B>>2];C=r-v;D=r+v;v=z+D;E=a+100|0;r=+g[E>>2];F=a+108|0;G=+g[F>>2];H=r-G;I=r+G;J=a+116|0;G=+g[J>>2];K=a+124|0;r=+g[K>>2];L=G-r;M=G+r;r=H-C;G=C+H;H=y-L;C=y+L;L=I+M;N=a|0;y=+g[N>>2];O=a+4|0;P=+g[O>>2];Q=a+32|0;R=+g[Q>>2];S=a+36|0;T=+g[S>>2];U=v-h;V=h+v;g[d>>2]=y-V;g[N>>2]=y+V;g[E>>2]=T-U;g[S>>2]=U+T;T=i-L;U=i+L;g[w>>2]=R-T;g[Q>>2]=T+R;g[o>>2]=P-U;g[O>>2]=U+P;P=e*.7071067690849304-u*-.7071067690849304;U=e*-.7071067690849304+u*.7071067690849304;u=(z-D)*.7071067690849304;D=(I-M)*.7071067690849304;M=u-D;I=u+D;O=a+16|0;D=+g[O>>2];o=a+20|0;u=+g[o>>2];Q=a+48|0;z=+g[Q>>2];w=a+52|0;e=+g[w>>2];R=M-P;T=P+M;g[l>>2]=D-T;g[O>>2]=D+T;g[J>>2]=e-R;g[w>>2]=R+e;e=U-I;R=U+I;g[B>>2]=z-e;g[Q>>2]=z+e;g[s>>2]=u-R;g[o>>2]=u+R;R=-0.0-c;u=b*m-j*R;e=m*R+b*j;j=b*C-c*G;R=c*C+b*G;o=a+8|0;G=+g[o>>2];s=a+12|0;C=+g[s>>2];Q=a+40|0;m=+g[Q>>2];B=a+44|0;z=+g[B>>2];I=j-u;U=u+j;g[f>>2]=G-U;g[o>>2]=G+U;g[F>>2]=z-I;g[B>>2]=I+z;z=e-R;I=e+R;g[x>>2]=m-z;g[Q>>2]=m+z;g[p>>2]=C-I;g[s>>2]=C+I;I=-0.0-b;C=c*q-n*I;z=q*I+c*n;n=c*H-b*r;I=b*H+c*r;s=a+24|0;r=+g[s>>2];p=a+28|0;c=+g[p>>2];Q=a+56|0;H=+g[Q>>2];x=a+60|0;b=+g[x>>2];q=n-C;m=C+n;g[k>>2]=r-m;g[s>>2]=r+m;g[K>>2]=b-q;g[x>>2]=q+b;b=z-I;q=z+I;g[A>>2]=H-b;g[Q>>2]=H+b;g[t>>2]=c-q;g[p>>2]=c+q;return}function qF(a){a=a|0;qE(a);qD(a+128|0);qD(a+192|0);qS(a,2827464,4);return}function qG(a){a=a|0;qE(a);qD(a+128|0);qD(a+192|0);qS(a,2827464,4);qE(a+256|0);qE(a+384|0);qS(a,2752584,8);return}function qH(a){a=a|0;var b=0;qE(a);qD(a+128|0);qD(a+192|0);qS(a,2827464,4);qE(a+256|0);qE(a+384|0);qS(a,2752584,8);b=a+512|0;qE(b);qD(a+640|0);qD(a+704|0);qS(b,2827464,4);b=a+768|0;qE(b);qD(a+896|0);qD(a+960|0);qS(b,2827464,4);qS(a,2864936,16);return}function qI(a){a=a|0;var b=0;qH(a);b=a+1024|0;qE(b);qD(a+1152|0);qD(a+1216|0);qS(b,2827464,4);qE(a+1280|0);qE(a+1408|0);qS(b,2752584,8);b=a+1536|0;qE(b);qD(a+1664|0);qD(a+1728|0);qS(b,2827464,4);qE(a+1792|0);qE(a+1920|0);qS(b,2752584,8);qS(a,2827528,32);return}function qJ(a){a=a|0;qI(a);qH(a+2048|0);qH(a+3072|0);qS(a,2752712,64);return}function qK(a){a=a|0;qI(a);qH(a+2048|0);qH(a+3072|0);qS(a,2752712,64);qI(a+4096|0);qI(a+6144|0);qR(a,2865192,128);return}function qL(a){a=a|0;var b=0;qI(a);qH(a+2048|0);qH(a+3072|0);qS(a,2752712,64);qI(a+4096|0);qI(a+6144|0);qR(a,2865192,128);b=a+8192|0;qI(b);qH(a+10240|0);qH(a+11264|0);qS(b,2752712,64);b=a+12288|0;qI(b);qH(a+14336|0);qH(a+15360|0);qS(b,2752712,64);qR(a,2828040,256);return}function qM(a){a=a|0;var b=0;qL(a);b=a+16384|0;qI(b);qH(a+18432|0);qH(a+19456|0);qS(b,2752712,64);qI(a+20480|0);qI(a+22528|0);qR(b,2865192,128);b=a+24576|0;qI(b);qH(a+26624|0);qH(a+27648|0);qS(b,2752712,64);qI(a+28672|0);qI(a+30720|0);qR(b,2865192,128);qR(a,2753736,512);return}function qN(a){a=a|0;qM(a);qL(a+32768|0);qL(a+49152|0);qR(a,2605128,1024);return}function qO(a){a=a|0;qM(a);qL(a+32768|0);qL(a+49152|0);qR(a,2605128,1024);qM(a+65536|0);qM(a+98304|0);qR(a,2832136,2048);return}function qP(a){a=a|0;var b=0;qM(a);qL(a+32768|0);qL(a+49152|0);qR(a,2605128,1024);qM(a+65536|0);qM(a+98304|0);qR(a,2832136,2048);b=a+131072|0;qM(b);qL(a+163840|0);qL(a+180224|0);qR(b,2605128,1024);b=a+196608|0;qM(b);qL(a+229376|0);qL(a+245760|0);qR(b,2605128,1024);qR(a,2761928,4096);return}function qQ(a){a=a|0;var b=0;qP(a);b=a+262144|0;qM(b);qL(a+294912|0);qL(a+311296|0);qR(b,2605128,1024);qM(a+327680|0);qM(a+360448|0);qR(b,2832136,2048);b=a+393216|0;qM(b);qL(a+425984|0);qL(a+442368|0);qR(b,2605128,1024);qM(a+458752|0);qM(a+491520|0);qR(b,2832136,2048);qR(a,2621512,8192);return}function qR(a,b,c){a=a|0;b=b|0;c=c|0;var d=0,e=0,f=0,h=0,i=0.0,j=0,k=0.0,l=0,m=0.0,n=0,o=0.0,p=0,q=0.0,r=0,s=0.0,t=0,u=0.0,v=0,w=0.0,x=0.0,y=0.0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;d=c<<1;e=c<<2;f=c*6|0;h=a+(e<<3)|0;i=+g[h>>2];j=a+(e<<3)+4|0;k=+g[j>>2];l=a+(f<<3)|0;m=+g[l>>2];n=a+(f<<3)+4|0;o=+g[n>>2];p=a|0;q=+g[p>>2];r=a+4|0;s=+g[r>>2];t=a+(d<<3)|0;u=+g[t>>2];v=a+(d<<3)+4|0;w=+g[v>>2];x=m-i;y=i+m;g[h>>2]=q-y;g[p>>2]=y+q;g[n>>2]=w-x;g[v>>2]=x+w;w=k-o;x=k+o;g[l>>2]=u-w;g[t>>2]=w+u;g[j>>2]=s-x;g[r>>2]=x+s;r=e|1;j=a+(r<<3)|0;s=+g[j>>2];x=+g[b+4>>2];t=a+(r<<3)+4|0;u=+g[t>>2];w=+g[b+(d-1<<2)>>2];o=-0.0-w;k=s*x-u*o;q=x*u+s*o;r=f|1;l=a+(r<<3)|0;o=+g[l>>2];v=a+(r<<3)+4|0;s=+g[v>>2];u=x*o-w*s;y=w*o+x*s;n=a+8|0;s=+g[n>>2];p=a+12|0;x=+g[p>>2];h=d|1;z=a+(h<<3)|0;o=+g[z>>2];A=a+(h<<3)+4|0;w=+g[A>>2];m=u-k;i=k+u;g[j>>2]=s-i;g[n>>2]=s+i;g[v>>2]=w-m;g[A>>2]=m+w;w=q-y;m=q+y;g[l>>2]=o-w;g[z>>2]=o+w;g[t>>2]=x-m;g[p>>2]=x+m;p=e|2;t=f+2|0;f=d+2|0;z=e|3;e=r+2|0;r=h+2|0;h=a;a=b;l=c-1|0;c=b+(d<<2)|0;while(1){d=h+16|0;b=a+8|0;A=c-8|0;v=h+(p<<3)|0;m=+g[v>>2];x=+g[b>>2];n=h+(p<<3)+4|0;w=+g[n>>2];o=+g[A>>2];y=-0.0-o;q=m*x-w*y;i=x*w+m*y;j=h+(t<<3)|0;y=+g[j>>2];B=h+(t<<3)+4|0;m=+g[B>>2];w=x*y-o*m;s=o*y+x*m;C=d|0;m=+g[C>>2];D=h+20|0;x=+g[D>>2];E=h+(f<<3)|0;y=+g[E>>2];F=h+(f<<3)+4|0;o=+g[F>>2];u=w-q;k=q+w;g[v>>2]=m-k;g[C>>2]=m+k;g[B>>2]=o-u;g[F>>2]=u+o;o=i-s;u=i+s;g[j>>2]=y-o;g[E>>2]=y+o;g[n>>2]=x-u;g[D>>2]=x+u;D=h+(z<<3)|0;u=+g[D>>2];x=+g[a+12>>2];n=h+(z<<3)+4|0;o=+g[n>>2];y=+g[c-12>>2];s=-0.0-y;i=u*x-o*s;k=x*o+u*s;E=h+(e<<3)|0;s=+g[E>>2];j=h+(e<<3)+4|0;u=+g[j>>2];o=x*s-y*u;m=y*s+x*u;F=h+24|0;u=+g[F>>2];B=h+28|0;x=+g[B>>2];C=h+(r<<3)|0;s=+g[C>>2];v=h+(r<<3)+4|0;y=+g[v>>2];w=o-i;q=i+o;g[D>>2]=u-q;g[F>>2]=u+q;g[j>>2]=y-w;g[v>>2]=w+y;y=k-m;w=k+m;g[E>>2]=s-y;g[C>>2]=s+y;g[n>>2]=x-w;g[B>>2]=x+w;B=l-1|0;if((B|0)==0){break}else{h=d;a=b;l=B;c=A}}return}function qS(a,b,c){a=a|0;b=b|0;c=c|0;var d=0,e=0,f=0,h=0,i=0.0,j=0,k=0.0,l=0,m=0.0,n=0,o=0.0,p=0.0,q=0.0,r=0,s=0.0,t=0.0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;d=c<<1;e=c<<2;f=c*6|0;h=a+(e<<3)|0;i=+g[h>>2];j=a+(e<<3)+4|0;k=+g[j>>2];l=a+(f<<3)|0;m=+g[l>>2];n=a+(f<<3)+4|0;o=+g[n>>2];p=m-i;q=i+m;r=a|0;g[h>>2]=+g[r>>2]-q;g[r>>2]=q+ +g[r>>2];r=a+(d<<3)+4|0;g[n>>2]=+g[r>>2]-p;g[r>>2]=p+ +g[r>>2];p=k-o;q=k+o;r=a+(d<<3)|0;g[l>>2]=+g[r>>2]-p;g[r>>2]=p+ +g[r>>2];r=a+4|0;g[j>>2]=+g[r>>2]-q;g[r>>2]=q+ +g[r>>2];r=e|1;j=a+(r<<3)|0;q=+g[j>>2];p=+g[b+4>>2];l=a+(r<<3)+4|0;o=+g[l>>2];k=+g[b+(d-1<<2)>>2];m=-0.0-k;i=q*p-o*m;s=p*o+q*m;r=f|1;n=a+(r<<3)|0;m=+g[n>>2];h=a+(r<<3)+4|0;q=+g[h>>2];o=p*m-k*q;t=k*m+p*q;q=o-i;p=i+o;u=a+8|0;g[j>>2]=+g[u>>2]-p;g[u>>2]=+g[u>>2]+p;u=d|1;j=a+(u<<3)+4|0;g[h>>2]=+g[j>>2]-q;g[j>>2]=q+ +g[j>>2];q=s-t;p=s+t;j=a+(u<<3)|0;g[n>>2]=+g[j>>2]-q;g[j>>2]=q+ +g[j>>2];j=a+12|0;g[l>>2]=+g[j>>2]-p;g[j>>2]=p+ +g[j>>2];j=e|2;l=f+2|0;f=d+2|0;n=e|3;e=r+2|0;r=u+2|0;u=b+(d<<2)|0;d=a;a=b;b=c-1|0;while(1){c=d+16|0;h=a+8|0;v=u-8|0;w=d+(j<<3)|0;p=+g[w>>2];q=+g[h>>2];x=d+(j<<3)+4|0;t=+g[x>>2];s=+g[v>>2];o=-0.0-s;i=p*q-t*o;m=q*t+p*o;y=d+(l<<3)|0;o=+g[y>>2];z=d+(l<<3)+4|0;p=+g[z>>2];t=q*o-s*p;k=s*o+q*p;p=t-i;q=i+t;A=c|0;g[w>>2]=+g[A>>2]-q;g[A>>2]=+g[A>>2]+q;A=d+(f<<3)+4|0;g[z>>2]=+g[A>>2]-p;g[A>>2]=p+ +g[A>>2];p=m-k;q=m+k;A=d+(f<<3)|0;g[y>>2]=+g[A>>2]-p;g[A>>2]=p+ +g[A>>2];A=d+20|0;g[x>>2]=+g[A>>2]-q;g[A>>2]=q+ +g[A>>2];A=d+(n<<3)|0;q=+g[A>>2];p=+g[a+12>>2];x=d+(n<<3)+4|0;k=+g[x>>2];m=+g[u-12>>2];t=-0.0-m;i=q*p-k*t;o=p*k+q*t;y=d+(e<<3)|0;t=+g[y>>2];z=d+(e<<3)+4|0;q=+g[z>>2];k=p*t-m*q;s=m*t+p*q;q=k-i;p=i+k;w=d+24|0;g[A>>2]=+g[w>>2]-p;g[w>>2]=+g[w>>2]+p;w=d+(r<<3)+4|0;g[z>>2]=+g[w>>2]-q;g[w>>2]=q+ +g[w>>2];q=o-s;p=o+s;w=d+(r<<3)|0;g[y>>2]=+g[w>>2]-q;g[w>>2]=q+ +g[w>>2];w=d+28|0;g[x>>2]=+g[w>>2]-p;g[w>>2]=p+ +g[w>>2];w=b-1|0;if((w|0)==0){break}else{u=v;d=c;a=h;b=w}}return}function qT(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0;do{if((e|0)==2){if((d|0)==0){return}f=b+4|0;h=0;do{i=h<<1;g[a+(i<<2)>>2]=+g[(c[b>>2]|0)+(h<<2)>>2];g[a+((i|1)<<2)>>2]=+g[(c[f>>2]|0)+(h<<2)>>2];h=h+1|0;}while(h>>>0<d>>>0);return}else if((e|0)==1){if(d>>>0>=536870911){j=0;break}h=a;f=c[b>>2]|0;i=d<<2;NN(h|0,f|0,i)|0;return}else{if(!((e|0)<1|(d|0)==0)){j=0;break}return}}while(0);do{i=b+(j<<2)|0;f=j;h=0;while(1){g[a+(f<<2)>>2]=+g[(c[i>>2]|0)+(h<<2)>>2];k=h+1|0;if(k>>>0<d>>>0){f=f+e|0;h=k}else{break}}j=j+1|0;}while((j|0)<(e|0));return}function qU(a,b){a=a|0;b=b|0;c[a>>2]=2;c[a+4>>2]=26;c[a+8>>2]=40;c[a+12>>2]=24;return}function qV(a,b,d,e){a=a|0;b=b|0;d=+d;e=e|0;var f=0;if((e|0)>0){f=0}else{return}do{g[a+(f<<2)>>2]=+(c[b+(f<<2)>>2]|0)*d;f=f+1|0;}while((f|0)<(e|0));return}function qW(a,c,d){a=a|0;c=c|0;d=d|0;var e=0,f=0,h=0;if((d|0)>0){e=0}else{return}do{f=bR(+(+g[c+(e<<2)>>2]))|0;if((f+32768|0)>>>0>65535){h=(f>>31^32767)&65535}else{h=f&65535}b[a+(e<<1)>>1]=h;e=e+1|0;}while((e|0)<(d|0));return}function qX(a,d,e,f){a=a|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;if((f|0)!=2){if((f|0)>0&(e|0)>0){h=0}else{return}do{i=d+(h<<2)|0;j=0;k=h;while(1){l=bR(+(+g[(c[i>>2]|0)+(j<<2)>>2]))|0;if((l+32768|0)>>>0>65535){m=(l>>31^32767)&65535}else{m=l&65535}b[a+(k<<1)>>1]=m;l=j+1|0;if((l|0)<(e|0)){j=l;k=k+f|0}else{break}}h=h+1|0;}while((h|0)<(f|0));return}if((e|0)<=0){return}f=d+4|0;h=0;do{m=bR(+(+g[(c[d>>2]|0)+(h<<2)>>2]))|0;if((m+32768|0)>>>0>65535){n=(m>>31^32767)&65535}else{n=m&65535}m=h<<1;b[a+(m<<1)>>1]=n;k=bR(+(+g[(c[f>>2]|0)+(h<<2)>>2]))|0;if((k+32768|0)>>>0>65535){o=(k>>31^32767)&65535}else{o=k&65535}b[a+((m|1)<<1)>>1]=o;h=h+1|0;}while((h|0)<(e|0));return}function qY(a,b,d){a=a|0;b=+b;d=d|0;var e=0,f=0,j=0.0,k=0,l=0.0,m=0,n=0.0,o=0,p=0;e=i;i=i+8192|0;f=e|0;j=b*3.141592653589793/+(d|0);b=j*j;if((d|0)>=1025){AN(0,0,1648840,(k=i,i=i+24|0,c[k>>2]=1720832,c[k+8>>2]=1693144,c[k+16>>2]=33,k)|0);i=k;cw()}k=(d|0)>0;if(k){l=0.0;m=0}else{i=e;return}do{j=b*+(ac(d-m|0,m)|0);n=1.0;o=50;do{n=j*n/+(ac(o,o)|0)+1.0;o=o-1|0;}while((o|0)>0);l=l+n;h[f+(m<<3)>>3]=l;m=m+1|0;}while((m|0)<(d|0));b=l+1.0;if(k){p=0}else{i=e;return}do{g[a+(p<<2)>>2]=+S(+(+h[f+(p<<3)>>3]/b));p=p+1|0;}while((p|0)<(d|0));i=e;return}function qZ(a,b,d,e){a=a|0;b=b|0;d=d|0;e=+e;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0.0,q=0.0,r=0,s=0.0,t=0,u=0.0,v=0.0;NM(a|0,0,60);f=1<<b;c[a+20>>2]=b;c[a+16>>2]=f;h=f>>2;i=a+60|0;c[i>>2]=0;do{if((qx(a,b-2|0,d)|0)<0){j=a+24|0}else{k=A1(((f|0)/2|0)<<2)|0;l=k;m=a+24|0;c[m>>2]=l;if((k|0)==0){j=m;break}n=c[i>>2]|0;if((n|0)==1){c[a+28>>2]=k+4;o=2}else if((n|0)==0){c[a+28>>2]=l+(h<<2);o=1}else{j=m;break}p=(e<0.0?+(h|0):0.0)+.125;q=+S(+(+R(+e)));if((h|0)<=0){r=0;return r|0}s=+(f|0);n=a+28|0;k=0;t=l;while(1){u=(p+ +(k|0))*6.283185307179586/s;v=q*(-0.0- +U(+u));l=ac(k,o)|0;g[t+(l<<2)>>2]=v;v=q*(-0.0- +V(+u));g[(c[n>>2]|0)+(l<<2)>>2]=v;l=k+1|0;if((l|0)>=(h|0)){r=0;break}k=l;t=c[m>>2]|0}return r|0}}while(0);A6(j);qB(a);r=-1;return r|0}function q_(a){a=a|0;A6(a+24|0);qB(a);return}function q$(a,b,d){a=a|0;b=b|0;d=d|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0.0,y=0.0;f=c[a+8>>2]|0;h=c[a+24>>2]|0;i=c[a+28>>2]|0;j=b;b=1<<c[a+20>>2];k=b>>2;l=b>>3;if((k|0)>0){m=d+((b>>1)-1<<2)|0;b=d;d=0;while(1){n=e[f+(d<<1)>>1]|0;o=h+(d<<2)|0;p=i+(d<<2)|0;g[j+(n<<3)>>2]=+g[m>>2]*+g[o>>2]- +g[b>>2]*+g[p>>2];g[j+(n<<3)+4>>2]=+g[m>>2]*+g[p>>2]+ +g[b>>2]*+g[o>>2];o=d+1|0;if((o|0)<(k|0)){m=m-8|0;b=b+8|0;d=o}else{break}}}c$[c[a+36>>2]&127](a,j);if((l|0)>0){q=0}else{return}do{a=l-q-1|0;d=j+(a<<3)+4|0;r=+g[d>>2];s=+g[i+(a<<2)>>2];b=j+(a<<3)|0;t=+g[b>>2];u=+g[h+(a<<2)>>2];a=q+l|0;m=j+(a<<3)+4|0;v=+g[m>>2];w=+g[i+(a<<2)>>2];k=j+(a<<3)|0;x=+g[k>>2];y=+g[h+(a<<2)>>2];g[b>>2]=r*s-t*u;g[d>>2]=w*x+v*y;g[k>>2]=v*w-x*y;g[m>>2]=s*t+r*u;q=q+1|0;}while((q|0)<(l|0));return}function q0(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,i=0,j=0,k=0;e=1<<c[a+20>>2];f=e>>1;h=e>>2;q$(a,b+(h<<2)|0,d);if((h|0)<=0){return}d=f-1|0;a=e-1|0;e=0;i=0;while(1){g[b+(e<<2)>>2]=-0.0- +g[b+(d+i<<2)>>2];g[b+(a+i<<2)>>2]=+g[b+(e+f<<2)>>2];j=e+1|0;k=~e;if((j|0)<(h|0)){e=j;i=k}else{break}}return}function q1(a,b,d){a=a|0;b=b|0;d=d|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0.0,v=0.0,w=0,x=0,y=0,z=0,A=0.0,B=0.0,C=0.0,D=0.0,E=0.0,F=0.0;f=c[a+8>>2]|0;h=c[a+24>>2]|0;i=c[a+28>>2]|0;j=b;b=1<<c[a+20>>2];k=b>>1;l=b>>2;m=b>>3;n=l*3|0;o=(m|0)>0;if(!o){c$[c[a+36>>2]&127](a,j);return}p=n-1|0;q=l-1|0;r=k-1|0;s=b-1|0;b=0;do{t=b<<1;u=-0.0- +g[d+(t+n<<2)>>2]- +g[d+(p-t<<2)>>2];v=+g[d+(q-t<<2)>>2]- +g[d+(t+l<<2)>>2];w=e[f+(b<<1)>>1]|0;x=h+(b<<2)|0;y=i+(b<<2)|0;g[j+(w<<3)>>2]=u*(-0.0- +g[x>>2])-v*+g[y>>2];g[j+(w<<3)+4>>2]=u*+g[y>>2]+v*(-0.0- +g[x>>2]);v=+g[d+(t<<2)>>2]- +g[d+(r-t<<2)>>2];u=-0.0- +g[d+(t+k<<2)>>2]- +g[d+(s-t<<2)>>2];t=b+m|0;x=e[f+(t<<1)>>1]|0;y=h+(t<<2)|0;w=i+(t<<2)|0;g[j+(x<<3)>>2]=v*(-0.0- +g[y>>2])-u*+g[w>>2];g[j+(x<<3)+4>>2]=v*+g[w>>2]+u*(-0.0- +g[y>>2]);b=b+1|0;}while((b|0)<(m|0));c$[c[a+36>>2]&127](a,j);if(o){z=0}else{return}do{o=m-z-1|0;a=j+(o<<3)|0;u=+g[a>>2];v=-0.0- +g[i+(o<<2)>>2];b=j+(o<<3)+4|0;A=+g[b>>2];B=-0.0- +g[h+(o<<2)>>2];o=z+m|0;f=j+(o<<3)|0;C=+g[f>>2];D=-0.0- +g[i+(o<<2)>>2];s=j+(o<<3)+4|0;E=+g[s>>2];F=-0.0- +g[h+(o<<2)>>2];g[a>>2]=A*v+u*B;g[b>>2]=C*D-E*F;g[f>>2]=E*D+C*F;g[s>>2]=u*v-A*B;z=z+1|0;}while((z|0)<(m|0));return}function q2(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0;e=(a|0)/(b|0)|0;if((d|0)!=0){f=4;return f|0}if((c|0)==48e3&(e|0)>55|(e-56|0)>>>0<25){f=0;return f|0}if((c|0)!=48e3&(e|0)>95){f=1;return f|0}else{return((c|0)!=32e3&(e|0)<49?2:3)|0}return 0}function q3(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,h=0,i=0.0,j=0,k=0,l=0.0,m=0,n=0,o=0.0,p=0,q=0,r=0;e=1<<b;f=(d|0)==0;do{if(f){c[a>>2]=b;c[a+4>>2]=0;h=0;i=-6.283185307179586/+(e|0);j=2743}else{k=(d|0)==3;l=(k?-6.283185307179586:6.283185307179586)/+(e|0);c[a>>2]=b;if((d|0)==1){c[a+4>>2]=1;h=1;i=l;j=2743;break}else{c[a+4>>2]=k&1;if((d|0)==2){m=1;n=0;o=l;p=1;break}else{h=0;i=l;j=2743;break}}}}while(0);if((j|0)==2743){m=(d|0)==3?1:-1;n=h;o=i;p=0}c[a+8>>2]=m;if((b-4|0)>>>0>12){q=-1;return q|0}if((qx(a+20|0,b-1|0,n?1:p)|0)<0){q=-1;return q|0}qw(b);c[a+12>>2]=c[251072+(b<<2)>>2];if(f){r=1}else{r=(d|0)==3|0}d=e>>2;e=(c[238208+(b<<2)>>2]|0)+((d&-r)<<2)|0;r=a+16|0;c[r>>2]=e;L3826:do{if((d|0)>0){b=0;f=e;while(1){g[f+(b<<2)>>2]=+V(+(o*+(b|0)));p=b+1|0;if((p|0)>=(d|0)){break L3826}b=p;f=c[r>>2]|0}}}while(0);c[a+84>>2]=22;q=0;return q|0}function q4(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,h=0.0,i=0,j=0,k=0,l=0.0,m=0.0,n=0,o=0,p=0,q=0,r=0,s=0.0,t=0.0,u=0.0,v=0,w=0.0,x=0.0,y=0,z=0;d=1<<c[a>>2];e=a+4|0;f=c[e>>2]|0;h=.5- +(f|0);i=c[a+12>>2]|0;j=c[a+16>>2]|0;if((f|0)==0){f=a+20|0;k=b;c$[c[a+52>>2]&127](f,k);c$[c[a+56>>2]&127](f,k)}l=+g[b>>2];k=b+4|0;m=+g[k>>2];g[b>>2]=l+m;g[k>>2]=l-m;f=d>>2;if((f|0)>1){m=-0.0-h;n=1;do{o=n<<1;p=d-o|0;q=b+(o<<2)|0;l=+g[q>>2];r=b+(p<<2)|0;s=+g[r>>2];t=(l+s)*.5;u=(l-s)*m;v=b+((o|1)<<2)|0;s=+g[v>>2];o=b+(p+1<<2)|0;l=+g[o>>2];w=(s-l)*.5;x=h*(s+l);p=i+(n<<2)|0;y=j+(n<<2)|0;g[q>>2]=t+ +g[p>>2]*x-u*+g[y>>2];g[v>>2]=w+u*+g[p>>2]+x*+g[y>>2];g[r>>2]=t-x*+g[p>>2]+u*+g[y>>2];g[o>>2]=u*+g[p>>2]-w+x*+g[y>>2];n=n+1|0;}while((n|0)<(f|0));z=f<<1|1}else{z=3}f=b+(z<<2)|0;g[f>>2]=+(c[a+8>>2]|0)*+g[f>>2];if((c[e>>2]|0)==0){return}g[b>>2]=+g[b>>2]*.5;g[k>>2]=+g[k>>2]*.5;k=a+20|0;e=b;c$[c[a+52>>2]&127](k,e);c$[c[a+56>>2]&127](k,e);return}function q5(a){a=a|0;c[a>>2]=14;c[a+4>>2]=2;c[a+8>>2]=76;c[a+12>>2]=30;c[a+16>>2]=34;c[a+20>>2]=24;c[a+24>>2]=94;c[a+28>>2]=42;c[a+32>>2]=2;c[a+36>>2]=56;c[a+40>>2]=22;c[a+44>>2]=20;c[a+48>>2]=26;c[a+52>>2]=24;return}function q6(a){a=a|0;var b=0,c=0;b=0;do{c=a+(b<<2)|0;g[c>>2]=+g[c>>2]+ +g[a+(b+64<<2)>>2]+ +g[a+(b+128<<2)>>2]+ +g[a+(b+192<<2)>>2]+ +g[a+(b+256<<2)>>2];b=b+1|0;}while((b|0)<64);return}function q7(a,b){a=a|0;b=b|0;var c=0.0,d=0.0,e=0,f=0.0,h=0.0,i=0.0,j=0.0,k=0.0,l=0,m=0.0,n=0.0;if((b|0)>0){c=0.0;d=0.0;e=0}else{f=0.0;h=0.0;i=f+h;return+i}while(1){j=+g[a+(e<<3)>>2];k=+g[a+(e<<3)+4>>2];l=e|1;m=+g[a+(l<<3)>>2];n=c+j*j+m*m;m=+g[a+(l<<3)+4>>2];j=d+k*k+m*m;l=e+2|0;if((l|0)<(b|0)){c=n;d=j;e=l}else{f=n;h=j;break}}i=f+h;return+i}function q8(a){a=a|0;var b=0,d=0;b=1;do{d=a+(b<<2)|0;c[d>>2]=c[d>>2]^-2147483648;d=a+(b+2<<2)|0;c[d>>2]=c[d>>2]^-2147483648;b=b+4|0;}while((b|0)<64);return}function q9(a){a=a|0;var b=0,d=0;c[a+256>>2]=c[a>>2];c[a+260>>2]=c[a+4>>2];b=1;do{d=b<<1;c[a+(d+64<<2)>>2]=c[a+(64-b<<2)>>2]^-2147483648;c[a+(d+65<<2)>>2]=c[a+(b+1<<2)>>2];c[a+(d+66<<2)>>2]=c[a+(63-b<<2)>>2]^-2147483648;b=b+2|0;c[a+(d+67<<2)>>2]=c[a+(b<<2)>>2];}while((b|0)<31);c[a+504>>2]=c[a+132>>2]^-2147483648;c[a+508>>2]=c[a+128>>2];return}function ra(a,b){a=a|0;b=b|0;var d=0,e=0;d=0;do{e=d<<1;c[a+(e<<2)>>2]=c[b+(63-d<<2)>>2]^-2147483648;c[a+((e|1)<<2)>>2]=c[b+(d<<2)>>2];c[a+((e|2)<<2)>>2]=c[b+(62-d<<2)>>2]^-2147483648;c[a+((e|3)<<2)>>2]=c[b+((d|1)<<2)>>2];d=d+2|0;}while((d|0)<32);return}function rb(a,b){a=a|0;b=b|0;var d=0,e=0;d=0;do{e=63-(d<<1)|0;c[a+(d<<2)>>2]=c[b+(e<<2)>>2];c[a+(63-d<<2)>>2]=c[b+(e-1<<2)>>2]^-2147483648;d=d+1|0;}while((d|0)<32);return}function rc(a,b,c){a=a|0;b=b|0;c=c|0;var d=0,e=0,f=0;d=0;do{e=b+(d<<2)|0;f=c+(63-d<<2)|0;g[a+(d<<2)>>2]=+g[e>>2]- +g[f>>2];g[a+(127-d<<2)>>2]=+g[e>>2]+ +g[f>>2];d=d+1|0;}while((d|0)<64);return}function rd(a,b){a=a|0;b=b|0;var c=0,d=0.0,e=0.0,f=0,h=0.0,i=0.0,j=0,k=0,l=0.0,m=0.0,n=0.0,o=0,p=0.0,q=0.0,r=0.0,s=0.0,t=0,u=0.0,v=0.0,w=0,x=0.0,y=0.0,z=0.0;c=a|0;d=+g[c>>2];e=+g[a+16>>2];f=a+4|0;h=+g[f>>2];i=+g[a+20>>2];j=a+8|0;k=a+12|0;l=d*e+h*i;m=d*i-e*h;h=0.0;d=0.0;n=0.0;o=1;p=+g[j>>2];q=+g[k>>2];r=e;e=i;while(1){s=n+(p*p+q*q);t=o+1|0;u=h+(p*r+q*e);v=d+(p*e-q*r);w=o+2|0;i=+g[a+(w<<3)>>2];x=+g[a+(w<<3)+4>>2];y=l+(p*i+q*x);z=m+(p*x-q*i);if((t|0)<38){l=y;m=z;h=u;d=v;n=s;o=t;p=r;q=e;r=i;e=x}else{break}}g[b+8>>2]=y;g[b+12>>2]=z;z=+g[c>>2];y=+g[f>>2];g[b+40>>2]=s+z*z+y*y;o=a+304|0;y=+g[o>>2];t=a+308|0;z=+g[t>>2];g[b+16>>2]=s+y*y+z*z;g[b+24>>2]=u+ +g[c>>2]*+g[j>>2]+ +g[f>>2]*+g[k>>2];g[b+28>>2]=v+ +g[c>>2]*+g[k>>2]- +g[f>>2]*+g[j>>2];j=a+312|0;f=a+316|0;g[b>>2]=u+ +g[o>>2]*+g[j>>2]+ +g[t>>2]*+g[f>>2];g[b+4>>2]=v+ +g[o>>2]*+g[f>>2]- +g[t>>2]*+g[j>>2];return}function re(a,b,c,d,e,f,h){a=a|0;b=b|0;c=c|0;d=d|0;e=+e;f=f|0;h=h|0;var i=0.0,j=0.0,k=0.0,l=0.0,m=0,n=0,o=0;i=+g[d>>2]*e*e;j=+g[d+4>>2]*e*e;k=+g[c>>2]*e;l=+g[c+4>>2]*e;if((f|0)<(h|0)){m=f}else{return}do{f=m-2|0;c=b+(f<<3)|0;d=b+(f<<3)+4|0;f=m-1|0;n=b+(f<<3)|0;o=b+(f<<3)+4|0;g[a+(m<<3)>>2]=+g[b+(m<<3)>>2]+(i*+g[c>>2]-j*+g[d>>2]+k*+g[n>>2]-l*+g[o>>2]);g[a+(m<<3)+4>>2]=+g[b+(m<<3)+4>>2]+(i*+g[d>>2]+j*+g[c>>2]+k*+g[o>>2]+l*+g[n>>2]);m=m+1|0;}while((m|0)<(h|0));return}function rf(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0,h=0;if((d|0)>0){f=0}else{return}do{h=c+(f<<2)|0;g[a+(f<<3)>>2]=+g[b+(f*320|0)+(e<<3)>>2]*+g[h>>2];g[a+(f<<3)+4>>2]=+g[b+(f*320|0)+(e<<3)+4>>2]*+g[h>>2];f=f+1|0;}while((f|0)<(d|0));return}function rg(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;var h=0,i=0.0,j=0,k=0,l=0.0,m=0.0,n=0.0;if((f|0)>0){h=0;i=0.0;j=d}else{return}while(1){d=a+(h<<3)|0;e=a+(h<<3)+4|0;k=j+1&511;l=+g[b+(h<<2)>>2];if(l!=0.0){m=l;n=i*l}else{l=+g[c+(h<<2)>>2];m=l*+g[238280+(k<<3)>>2];n=l*+g[238284+(k<<3)>>2]}l=+g[e>>2]+n;g[d>>2]=+g[d>>2]+m;g[e>>2]=l;e=h+1|0;if((e|0)<(f|0)){h=e;i=-0.0-i;j=k}else{break}}return}function rh(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;var h=0,i=0.0,j=0,k=0,l=0.0,m=0.0,n=0.0;if((f|0)<=0){return}h=0;i=+(1-(e<<1&2)|0);e=d;while(1){d=a+(h<<3)|0;j=a+(h<<3)+4|0;k=e+1&511;l=+g[b+(h<<2)>>2];if(l!=0.0){m=l*0.0;n=i*l}else{l=+g[c+(h<<2)>>2];m=l*+g[238280+(k<<3)>>2];n=l*+g[238284+(k<<3)>>2]}l=+g[j>>2]+n;g[d>>2]=+g[d>>2]+m;g[j>>2]=l;j=h+1|0;if((j|0)<(f|0)){h=j;i=-0.0-i;e=k}else{break}}return}function ri(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;var h=0,i=0.0,j=0,k=0,l=0.0,m=0.0,n=0.0;if((f|0)>0){h=0;i=0.0;j=d}else{return}while(1){d=a+(h<<3)|0;e=a+(h<<3)+4|0;k=j+1&511;l=+g[b+(h<<2)>>2];if(l!=0.0){m=l*-1.0;n=i*l}else{l=+g[c+(h<<2)>>2];m=l*+g[238280+(k<<3)>>2];n=l*+g[238284+(k<<3)>>2]}l=+g[e>>2]+n;g[d>>2]=+g[d>>2]+m;g[e>>2]=l;e=h+1|0;if((e|0)<(f|0)){h=e;i=-0.0-i;j=k}else{break}}return}function rj(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0.0,k=0.0,l=0.0,m=0.0;if((f|0)<=0){return}h=0;i=d;j=+(1-(e<<1&2)|0);do{j=-0.0-j;e=a+(h<<3)|0;d=a+(h<<3)+4|0;i=i+1&511;k=+g[b+(h<<2)>>2];if(k!=0.0){l=k*0.0;m=k*j}else{k=+g[c+(h<<2)>>2];l=k*+g[238280+(i<<3)>>2];m=k*+g[238284+(i<<3)>>2]}k=+g[d>>2]+m;g[e>>2]=+g[e>>2]+l;g[d>>2]=k;h=h+1|0;}while((h|0)<(f|0));return}function rk(a){a=a|0;var b=0,d=0,e=0.0;b=c[238152+(a<<2)>>2]|0;d=1<<a;if((d|0)<=0){return}e=3.141592653589793/(+(d|0)*2.0);a=0;do{g[b+(a<<2)>>2]=+V(+(e*(+(a|0)+.5)));a=a+1|0;}while((a|0)<(d|0));return}function rl(f,h){f=f|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0.0,v=0,w=0.0,x=0.0,y=0,z=0,A=0,B=0.0,C=0.0,D=0.0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0;j=i;k=c[f+60>>2]|0;l=f+412|0;if(((c[l>>2]|0)-1|0)>>>0>49999){m=-1;i=j;return m|0}n=f+416|0;if(((c[n>>2]|0)-1|0)>>>0>1){m=-1;i=j;return m|0}o=f+72|0;if((c[o>>2]|0)<1){m=-1;i=j;return m|0}qU(k+159264|0,f);z7(k+159280|0,c[f+88>>2]&8388608);p=k+48|0;q=(c[(c[f+12>>2]|0)+12>>2]|0)==86023?1:2;c[p>>2]=q;r=rr(c[l>>2]|0,q,0)|0;q=k+1084|0;c[q>>2]=r;c[k+1100>>2]=r;c[k+1104>>2]=r;c[k+1096>>2]=r;s=k+1080|0;c[s>>2]=1<<r;if((c[k+56>>2]|0)==0){c[k+1088>>2]=1}else{t=(((c[o>>2]|0)/(c[n>>2]|0)|0|0)>31999?3:1)+(h>>>3&3)|0;h=r-7|0;c[k+1088>>2]=((t|0)>(h|0)?h:t)+1}t=k+64|0;c[t>>2]=1;h=c[l>>2]|0;u=+(h|0)*.5;do{if((c[p>>2]|0)==2){if((h|0)>44099){v=44100;break}if((h|0)>22049){v=22050;break}if((h|0)>15999){v=16e3;break}if((h|0)>11024){v=11025;break}v=(h|0)<8e3?h:8e3}else{v=h}}while(0);w=+(c[o>>2]|0);x=w/+(ac(h,c[n>>2]|0)|0);h=~~(x*+(c[s>>2]|0)*.125+.5);o=h>>>0>65535;r=o?16:0;y=o?h>>>16:h;if((y&65280|0)==0){z=y;A=r}else{z=y>>>8;A=r|8}r=(d[247648+z|0]|0)+A|0;A=r+2|0;c[k+68>>2]=A;if((r+5|0)>25){AN(f,16,1648192,(f=i,i=i+8|0,c[f>>2]=A,f)|0);i=f;m=-1163346256;i=j;return m|0}if((c[n>>2]|0)==2){B=x*1.6}else{B=x}do{if((v|0)==16e3){if(x>.5){C=u*.5;break}else{C=u*.3;break}}else if((v|0)==22050){w=B;if(w>=1.16){c[t>>2]=0;C=u;break}D=u;if(w<.72){C=D*.6;break}else{C=D*.7;break}}else if((v|0)==11025){C=u*.7}else if((v|0)==44100){if(B<.61){C=u*.4;break}else{c[t>>2]=0;C=u;break}}else{D=x;if((v|0)==8e3){if(D<=.625){C=u*.5;break}if(x>.75){c[t>>2]=0;C=u;break}else{C=u*.65;break}}else{if(D>=.8){C=u*.75;break}if(x<.6){C=u*.5;break}else{C=u*.6;break}}}}while(0);v=c[p>>2]|0;c[k+380>>2]=(v|0)==1?3:0;n=k+1088|0;do{if((c[n>>2]|0)>0){f=k+384|0;A=k+360|0;r=k+88|0;z=r;y=k+404|0;h=k+108|0;o=h;E=k+424|0;F=h;h=r;r=0;G=v;while(1){H=c[s>>2]>>r;L4011:do{if((G|0)==1){I=H<<1;J=0;K=0;while(1){if((J|0)>=25){L=J;break}M=c[l>>2]|0;N=((ac(I,e[237616+(J<<1)>>1]|0)|0)+(M>>1)|0)/(M|0)|0;M=(N|0)>(H|0)?H:N;b[F+(J<<1)>>1]=M-K&65535;N=J+1|0;if((M|0)<(H|0)){J=N;K=M}else{L=N;break}}c[h>>2]=L;O=c[z+(r<<2)>>2]|0}else{K=-7-r+(c[q>>2]|0)|0;L4018:do{if((K|0)<3){J=c[l>>2]|0;do{if((J|0)>44099){P=258888+(K*25|0)|0}else{if((J|0)>31999){P=258968+(K*25|0)|0;break}if((J|0)<=22049){Q=0;R=0;S=0;break L4018}P=259048+(K*25|0)|0}}while(0);J=a[P]|0;I=J&255;if(J<<24>>24!=0){J=0;while(1){N=J+1|0;b[o+(r*50|0)+(J<<1)>>1]=d[P+N|0]|0;if((N|0)<(I|0)){J=N}else{break}}}c[z+(r<<2)>>2]=I;O=I;break L4011}else{Q=0;R=0;S=0}}while(0);while(1){if((R|0)>=25){T=Q;break}K=c[l>>2]|0;J=(((ac(e[237616+(R<<1)>>1]|0,H)|0)+K<<1|0)/(K<<2|0)|0)<<2;K=(J|0)>(H|0)?H:J;if((K|0)>(S|0)){b[o+(r*50|0)+(Q<<1)>>1]=K-S&65535;U=Q+1|0}else{U=Q}if((K|0)<(H|0)){Q=U;R=R+1|0;S=K}else{T=U;break}}c[z+(r<<2)>>2]=T;O=T}}while(0);K=c[s>>2]|0;J=K-((K*9|0|0)/100|0)>>r;K=f+(r<<2)|0;c[K>>2]=J;N=~~(C*+(H<<1|0)/+(c[l>>2]|0)+.5);M=A+(r<<2)|0;c[M>>2]=N;L4040:do{if((O|0)>0){V=0;W=0;X=0;Y=N;Z=J;while(1){_=(e[o+(r*50|0)+(W<<1)>>1]|0)+V|0;$=(V|0)<(Y|0)?Y:V;aa=(_|0)>(Z|0)?Z:_;if((aa|0)>($|0)){c[E+(r<<6)+(X<<2)>>2]=aa-$;ab=X+1|0}else{ab=X}$=W+1|0;if(($|0)>=(O|0)){ad=ab;break L4040}V=_;W=$;X=ab;Y=c[M>>2]|0;Z=c[K>>2]|0}}else{ad=0}}while(0);c[y+(r<<2)>>2]=ad;K=r+1|0;ae=c[n>>2]|0;if((K|0)>=(ae|0)){break}r=K;G=c[p>>2]|0}if((ae|0)<=0){break}G=k+67040|0;r=0;do{rk((c[q>>2]|0)-r|0);c[G+(r<<2)>>2]=c[238152+((c[q>>2]|0)-r<<2)>>2];r=r+1|0;}while((r|0)<(c[n>>2]|0))}}while(0);c[k+1092>>2]=1;if((c[t>>2]|0)!=0){C=(c[k+60>>2]|0)==0?.03999999910593033:.019999999552965164;g[k+149020>>2]=C;u=C*8.065490087349327e-10;t=k+116248|0;n=0;q=1;do{q=(q*314159|0)+1|0;g[t+(n<<2)>>2]=u*+(q|0);n=n+1|0;}while((n|0)<8192)}do{if((c[l>>2]|0)>31999){u=B;if(u<.72){af=0;break}if(u>=1.16){af=4;break}af=2}else{af=4}}while(0);l=265368+(af*20|0)|0;c[k+1072>>2]=l;n=k+1076|0;c[n>>2]=265368+((af|1)*20|0);rm(k+1016|0,k+1048|0,k+1056|0,k+1064|0,l);rm(k+1032|0,k+1052|0,k+1060|0,k+1068|0,c[n>>2]|0);m=0;i=j;return m|0}function rm(a,d,e,f,h){a=a|0;d=d|0;e=e|0;f=f|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0.0,v=0,w=0,x=0,y=0;i=c[h>>2]|0;j=c[h+16>>2]|0;qk(a,9,i,c[h+12>>2]|0,1,1,c[h+8>>2]|0,4,4,0,0,0,0)|0;h=i<<1;a=A1(h)|0;k=A1(h)|0;l=k;m=A1(i<<2)|0;n=A1(h)|0;if((i|0)>2){o=2;p=0;q=1}else{c[d>>2]=a;c[e>>2]=m;c[f>>2]=n;A4(k);return}while(1){b[n+(p<<1)>>1]=o&65535;h=p+1|0;r=b[j+(p<<1)>>1]|0;s=r&65535;if(r<<16>>16==0){t=o}else{r=q&65535;u=+(q|0);v=s>>>0>1?s:1;w=o;x=0;while(1){b[a+(w<<1)>>1]=x&65535;b[l+(w<<1)>>1]=r;g[m+(w<<2)>>2]=u;y=x+1|0;if((y|0)<(s|0)){w=w+1|0;x=y}else{break}}t=o+v|0}if((t|0)<(i|0)){o=t;p=h;q=q+1|0}else{break}}c[d>>2]=a;c[e>>2]=m;c[f>>2]=n;A4(k);return}function rn(a){a=a|0;var b=0;do{if((a|0)<15){b=13}else{if((a|0)<32){b=12;break}if((a|0)<40){b=11;break}b=(a|0)<45?10:9}}while(0);return b|0}function ro(a){a=a|0;var b=0,d=0,e=0;b=c[a+60>>2]|0;a=b+1088|0;if((c[a>>2]|0)>0){d=b+66720|0;e=0;do{q_(d+(e<<6)|0);e=e+1|0;}while((e|0)<(c[a>>2]|0))}if((c[b+60>>2]|0)!=0){ql(b+72|0)}if((c[b+64>>2]|0)!=0){ql(b+744|0)}ql(b+1016|0);A4(c[b+1048>>2]|0);A4(c[b+1056>>2]|0);A4(c[b+1064>>2]|0);ql(b+1032|0);A4(c[b+1052>>2]|0);A4(c[b+1060>>2]|0);A4(c[b+1068>>2]|0);return 0}function rp(a){a=a|0;var b=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0;b=a+8|0;e=c[b>>2]|0;f=c[a>>2]|0;g=d[f+(e>>>3)|0]|0;h=c[a+16>>2]|0;a=((e|0)<(h|0))+e|0;c[b>>2]=a;do{if((g&128>>>((e&7)>>>0)|0)==0){i=8;j=a}else{k=d[f+(a>>>3)|0]|0;l=((a|0)<(h|0))+a|0;c[b>>2]=l;if((k&128>>>((a&7)>>>0)|0)==0){i=16;j=l;break}k=d[f+(l>>>3)|0]|0;m=((l|0)<(h|0))+l|0;c[b>>2]=m;i=(k&128>>>((l&7)>>>0)|0)==0?24:31;j=m}}while(0);a=f+(j>>>3)|0;e=(cV(d[a]|d[a+1|0]<<8|d[a+2|0]<<16|d[a+3|0]<<24|0)|0)<<(j&7);if(i>>>0<26){a=j+i|0;c[b>>2]=h>>>0>a>>>0?a:h;n=e>>>((32-i|0)>>>0);return n|0}else{a=j+16|0;j=h>>>0>a>>>0?a:h;c[b>>2]=j;a=i-16|0;g=f+(j>>>3)|0;f=(cV(d[g]|d[g+1|0]<<8|d[g+2|0]<<16|d[g+3|0]<<24|0)|0)<<(j&7)>>>((48-i|0)>>>0);i=j+a|0;c[b>>2]=h>>>0>i>>>0?i:h;n=f|e>>>16<<a;return n|0}return 0}function rq(a,f,h,j,k,l,m,n,o,p,q,r){a=a|0;f=f|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;r=r|0;var s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0;s=i;t=p-1|0;L4106:do{if((n|0)<(o|0)){p=h+4|0;u=f+8|0;v=f+16|0;w=f|0;x=32-r|0;y=32-q|0;if((l|0)==0){z=n;while(1){A=c[p>>2]|0;B=c[u>>2]|0;C=c[v>>2]|0;D=c[w>>2]|0;E=D+(B>>>3)|0;F=(cV(d[E]|d[E+1|0]<<8|d[E+2|0]<<16|d[E+3|0]<<24|0)|0)<<(B&7)>>>23;E=b[A+(F<<2)>>1]|0;G=b[A+(F<<2)+2>>1]|0;F=G<<16>>16;do{if(G<<16>>16<0){H=B+9|0;I=C>>>0>H>>>0?H:C;H=D+(I>>>3)|0;J=((cV(d[H]|d[H+1|0]<<8|d[H+2|0]<<16|d[H+3|0]<<24|0)|0)<<(I&7)>>>((F+32|0)>>>0))+E|0;H=b[A+(J<<2)>>1]|0;K=b[A+(J<<2)+2>>1]|0;J=K<<16>>16;if(K<<16>>16>=0){L=H;M=I;N=J;break}K=I-F|0;I=C>>>0>K>>>0?K:C;K=D+(I>>>3)|0;O=((cV(d[K]|d[K+1|0]<<8|d[K+2|0]<<16|d[K+3|0]<<24|0)|0)<<(I&7)>>>((J+32|0)>>>0))+H|0;L=b[A+(O<<2)>>1]|0;M=I;N=b[A+(O<<2)+2>>1]|0}else{L=E;M=B;N=F}}while(0);F=M+N|0;B=C>>>0>F>>>0?F:C;c[u>>2]=B;if((L|0)>1){F=(e[k+(L<<1)>>1]|0)+z|0;E=d[D+(B>>>3)|0]<<(B&7)>>>7;c[u>>2]=((B|0)<(C|0))+B;c[m+((F&t)<<2)>>2]=c[j+(L<<2)>>2]^-2147483648^E<<31;P=F}else{if((L|0)==1){Q=z;break L4106}F=D+(B>>>3)|0;E=(cV(d[F]|d[F+1|0]<<8|d[F+2|0]<<16|d[F+3|0]<<24|0)|0)<<(B&7)>>>(x>>>0);F=B+r|0;B=C>>>0>F>>>0?F:C;c[u>>2]=B;F=D+(B>>>3)|0;A=(cV(d[F]|d[F+1|0]<<8|d[F+2|0]<<16|d[F+3|0]<<24|0)|0)<<(B&7)>>>(y>>>0);F=B+q|0;B=C>>>0>F>>>0?F:C;c[u>>2]=B;F=A+z|0;A=d[D+(B>>>3)|0]<<(B&7)>>>7;c[u>>2]=((B|0)<(C|0))+B;B=(A&1)-1|0;g[m+((F&t)<<2)>>2]=+((B^E)-B|0);P=F}F=P+1|0;if((F|0)<(o|0)){z=F}else{Q=F;break L4106}}}else{R=n}L4120:while(1){z=c[p>>2]|0;x=c[u>>2]|0;F=c[v>>2]|0;B=c[w>>2]|0;E=B+(x>>>3)|0;A=(cV(d[E]|d[E+1|0]<<8|d[E+2|0]<<16|d[E+3|0]<<24|0)|0)<<(x&7)>>>23;E=b[z+(A<<2)>>1]|0;G=b[z+(A<<2)+2>>1]|0;A=G<<16>>16;do{if(G<<16>>16<0){O=x+9|0;I=F>>>0>O>>>0?O:F;O=B+(I>>>3)|0;H=((cV(d[O]|d[O+1|0]<<8|d[O+2|0]<<16|d[O+3|0]<<24|0)|0)<<(I&7)>>>((A+32|0)>>>0))+E|0;O=b[z+(H<<2)>>1]|0;J=b[z+(H<<2)+2>>1]|0;H=J<<16>>16;if(J<<16>>16>=0){S=O;T=I;U=H;break}J=I-A|0;I=F>>>0>J>>>0?J:F;J=B+(I>>>3)|0;K=((cV(d[J]|d[J+1|0]<<8|d[J+2|0]<<16|d[J+3|0]<<24|0)|0)<<(I&7)>>>((H+32|0)>>>0))+O|0;S=b[z+(K<<2)>>1]|0;T=I;U=b[z+(K<<2)+2>>1]|0}else{S=E;T=x;U=A}}while(0);A=T+U|0;x=F>>>0>A>>>0?A:F;c[u>>2]=x;if((S|0)>1){A=(e[k+(S<<1)>>1]|0)+R|0;E=d[B+(x>>>3)|0]<<(x&7)>>>7;c[u>>2]=((x|0)<(F|0))+x;c[m+((A&t)<<2)>>2]=c[j+(S<<2)>>2]^-2147483648^E<<31;V=A}else{if((S|0)==1){Q=R;break L4106}A=rp(f)|0;E=c[u>>2]|0;x=c[w>>2]|0;z=d[x+(E>>>3)|0]|0;G=c[v>>2]|0;C=((E|0)<(G|0))+E|0;c[u>>2]=C;do{if((z&128>>>((E&7)>>>0)|0)==0){W=R;X=C}else{D=d[x+(C>>>3)|0]|0;K=((C|0)<(G|0))+C|0;c[u>>2]=K;I=x+(K>>>3)|0;if((D&128>>>((C&7)>>>0)|0)==0){D=I;O=(cV(d[D]|d[D+1|0]<<8|d[D+2|0]<<16|d[D+3|0]<<24|0)|0)<<(K&7)>>>30;D=K+2|0;H=G>>>0>D>>>0?D:G;c[u>>2]=H;W=R+1+O|0;X=H;break}H=d[I]|0;I=((K|0)<(G|0))+K|0;c[u>>2]=I;if((H&128>>>((K&7)>>>0)|0)!=0){break L4120}K=x+(I>>>3)|0;H=(cV(d[K]|d[K+1|0]<<8|d[K+2|0]<<16|d[K+3|0]<<24|0)|0)<<(I&7)>>>(y>>>0);K=I+q|0;I=G>>>0>K>>>0?K:G;c[u>>2]=I;W=R+4+H|0;X=I}}while(0);C=d[x+(X>>>3)|0]<<(X&7)>>>7;c[u>>2]=((X|0)<(G|0))+X;E=(C&1)-1|0;g[m+((W&t)<<2)>>2]=+((E^A)-E|0);V=W}E=V+1|0;if((E|0)<(o|0)){R=E}else{Q=E;break L4106}}AN(a,16,1720712,(Y=i,i=i+1|0,i=i+7&-8,c[Y>>2]=0,Y)|0);i=Y;Z=-1;i=s;return Z|0}else{Q=n}}while(0);if((Q|0)<=(o|0)){Z=0;i=s;return Z|0}AN(a,16,1692992,(Y=i,i=i+1|0,i=i+7&-8,c[Y>>2]=0,Y)|0);i=Y;Z=-1;i=s;return Z|0}function rr(a,b,c){a=a|0;b=b|0;c=c|0;var d=0,e=0;do{if((a|0)<16001){d=9}else{if((a|0)<22051){d=10;break}if((a|0)<32001&(b|0)==1){d=10;break}if((a|0)<48001|(b|0)<3){d=11;break}d=(a|0)<96001?12:13}}while(0);do{if((b|0)==3){a=c&6;if((a|0)==2){e=d+1|0;break}else if((a|0)==6){e=d-2|0;break}else if((a|0)==4){e=d-1|0;break}else{e=d;break}}else{e=d}}while(0);return e|0}function rs(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;f=b+8|0;g=c[f>>2]|0;h=c[b+16>>2]|0;i=c[b>>2]|0;b=i+(g>>>3)|0;j=(cV(d[b]|d[b+1|0]<<8|d[b+2|0]<<16|d[b+3|0]<<24|0)|0)<<(g&7);b=g+12|0;g=h>>>0>b>>>0?b:h;c[f>>2]=g;if((j&-1048576|0)!=-1048576){k=-1;return k|0}j=g+1|0;g=(h>>>0>j>>>0?j:h)+2|0;j=h>>>0>g>>>0?g:h;c[f>>2]=j;g=(d[i+(j>>>3)|0]|0)<<(j&7)>>>7;b=((j|0)<(h|0))+j|0;c[f>>2]=b;j=i+(b>>>3)|0;l=(cV(d[j]|d[j+1|0]<<8|d[j+2|0]<<16|d[j+3|0]<<24|0)|0)<<(b&7)>>>30;j=b+2|0;b=h>>>0>j>>>0?j:h;c[f>>2]=b;j=i+(b>>>3)|0;m=(cV(d[j]|d[j+1|0]<<8|d[j+2|0]<<16|d[j+3|0]<<24|0)|0)<<(b&7)>>>28;j=b+4|0;b=h>>>0>j>>>0?j:h;c[f>>2]=b;j=c[337504+(m<<2)>>2]|0;if((j|0)==0){k=-3;return k|0}n=b+1|0;b=h>>>0>n>>>0?n:h;c[f>>2]=b;n=i+(b>>>3)|0;o=d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0;n=b+3|0;p=(h>>>0>n>>>0?n:h)+1|0;n=(h>>>0>p>>>0?p:h)+1|0;p=(h>>>0>n>>>0?n:h)+1|0;n=(h>>>0>p>>>0?p:h)+1|0;p=h>>>0>n>>>0?n:h;c[f>>2]=p;n=i+(p>>>3)|0;q=(cV(d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0)|0)<<(p&7);n=q>>>19;r=p+13|0;p=h>>>0>r>>>0?r:h;c[f>>2]=p;if(q>>>0<3670016){k=-4;return k|0}q=(cV(o|0)|0)<<(b&7)>>>29;b=p+11|0;p=h>>>0>b>>>0?b:h;c[f>>2]=p;b=i+(p>>>3)|0;i=(cV(d[b]|d[b+1|0]<<8|d[b+2|0]<<16|d[b+3|0]<<24|0)|0)<<(p&7)>>>30;b=p+2|0;c[f>>2]=h>>>0>b>>>0?b:h;a[e+13|0]=l+1&255;a[e+15|0]=q&255;a[e+12|0]=g&1;g=i+1|0;a[e+16|0]=g&255;a[e+14|0]=m&255;c[e>>2]=j;m=g<<10;c[e+4>>2]=m;c[e+8>>2]=((ac(n<<3,j)|0)>>>0)/(m>>>0)|0;k=n;return k|0}function rt(e,f,g,h){e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0;j=i;i=i+24|0;k=j|0;l=f+8|0;m=c[l>>2]|0;n=k;o=f;c[n>>2]=c[o>>2];c[n+4>>2]=c[o+4>>2];c[n+8>>2]=c[o+8>>2];c[n+12>>2]=c[o+12>>2];c[n+16>>2]=c[o+16>>2];o=k+8|0;n=c[o>>2]|0;p=k|0;q=c[p>>2]|0;r=d[q+(n>>>3)|0]|0;s=k+16|0;t=c[s>>2]|0;u=((n|0)<(t|0))+n|0;c[o>>2]=u;v=(r&128>>>((n&7)>>>0)|0)!=0;L4171:do{if(v){n=d[q+(u>>>3)|0]<<(u&7)>>>7;r=((u|0)<(t|0))+u|0;c[o>>2]=r;w=n&1;c[g+4>>2]=w;do{if((w|0)==0){x=r}else{n=q+(r>>>3)|0;y=(cV(d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0)|0)<<(r&7);n=y>>>29;z=r+3|0;A=t>>>0>z>>>0?z:t;c[o>>2]=A;if(y>>>0>3221225471){AN(e,16,1648008,(B=i,i=i+8|0,c[B>>2]=n,B)|0);i=B;break L4171}else{c[g+12>>2]=a[121288+n|0]|0;c[g+8>>2]=y>>>0>1610612735;c[g+16>>2]=a[121280+n|0]|0;x=A;break}}}while(0);r=d[q+(x>>>3)|0]<<(x&7)>>>7;w=((x|0)<(t|0))+x|0;c[o>>2]=w;A=r&1;c[g+20>>2]=A;do{if((A|0)==0){C=w}else{r=q+(w>>>3)|0;n=(cV(d[r]|d[r+1|0]<<8|d[r+2|0]<<16|d[r+3|0]<<24|0)|0)<<(w&7);r=n>>>29;y=w+3|0;z=t>>>0>y>>>0?y:t;c[o>>2]=z;c[g+24>>2]=r;if(n>>>0>3221225471){AN(e,16,1720656,(B=i,i=i+8|0,c[B>>2]=r,B)|0);i=B;break L4171}else{c[g+28>>2]=a[121288+r|0]|0;C=z;break}}}while(0);w=d[q+(C>>>3)|0]<<(C&7)>>>7;A=((C|0)<(t|0))+C|0;c[o>>2]=A;c[g+32>>2]=w&1;D=A;E=3023}else{D=u;E=3023}}while(0);L4185:do{if((E|0)==3023){u=((D|0)<(t|0))+D|0;C=d[q+(D>>>3)|0]<<(D&7)>>>7&1;c[g+36>>2]=C;x=g+44|0;A=g+40|0;c[A>>2]=c[x>>2];w=q+(u>>>3)|0;z=(cV(d[w]|d[w+1|0]<<8|d[w+2|0]<<16|d[w+3|0]<<24|0)|0)<<(u&7)>>>30;w=u+2|0;u=t>>>0>w>>>0?w:t;c[o>>2]=u;w=a[120984+(C<<2)+z|0]|0;z=w<<24>>24;c[x>>2]=z;c[g+52>>2]=-1;r=w<<24>>24<1;do{if((C|0)==0){if(r){F=z;G=u;break}else{H=1;I=z}while(1){c[g+52+(H<<2)>>2]=(H<<5>>d[247648+I|0])-1;w=H+1|0;n=c[x>>2]|0;if((w|0)>(n|0)){F=n;G=u;break}else{H=w;I=n}}}else{if(r){F=z;G=u;break}else{J=1;K=u}while(1){n=q+(K>>>3)|0;w=K+5|0;L=t>>>0>w>>>0?w:t;c[g+52+(J<<2)>>2]=(cV(d[n]|d[n+1|0]<<8|d[n+2|0]<<16|d[n+3|0]<<24|0)|0)<<(K&7)>>>27;n=J+1|0;M=c[x>>2]|0;if((n|0)>(M|0)){break}else{J=n;K=L}}c[o>>2]=L;F=M;G=L}}while(0);u=g+4|0;do{if((c[u>>2]|0)==0){NM(g+76|0,0,170);N=F;O=G}else{if((F|0)<=0){N=F;O=G;break}z=g+8|0;r=g+12|0;C=0;n=G;w=c[z>>2]|0;L4200:while(1){y=((n|0)<(t|0))+n|0;P=d[q+(n>>>3)|0]<<(n&7)>>>7&1;Q=c[174392+((P<<1)+w<<2)>>2]|0;R=c[r>>2]|0;S=c[1997900+(Q<<4)>>2]|0;L4202:do{if((P|0)==0){T=173536+Q|0;U=0;V=0;W=y;X=w;while(1){if((V|0)>=(R|0)){Y=W;Z=X;break L4202}_=q+(W>>>3)|0;$=(cV(d[_]|d[_+1|0]<<8|d[_+2|0]<<16|d[_+3|0]<<24|0)|0)<<(W&7)>>>23;_=b[S+($<<2)>>1]|0;aa=b[S+($<<2)+2>>1]|0;$=aa<<16>>16;do{if(aa<<16>>16<0){ab=W+9|0;ac=t>>>0>ab>>>0?ab:t;ab=q+(ac>>>3)|0;ad=((cV(d[ab]|d[ab+1|0]<<8|d[ab+2|0]<<16|d[ab+3|0]<<24|0)|0)<<(ac&7)>>>(($+32|0)>>>0))+_|0;ab=b[S+(ad<<2)>>1]|0;ae=b[S+(ad<<2)+2>>1]|0;ad=ae<<16>>16;if(ae<<16>>16>=0){af=ab;ag=ac;ah=ad;break}ae=ac-$|0;ac=t>>>0>ae>>>0?ae:t;ae=q+(ac>>>3)|0;ai=((cV(d[ae]|d[ae+1|0]<<8|d[ae+2|0]<<16|d[ae+3|0]<<24|0)|0)<<(ac&7)>>>((ad+32|0)>>>0))+ab|0;af=b[S+(ai<<2)>>1]|0;ag=ac;ah=b[S+(ai<<2)+2>>1]|0}else{af=_;ag=W;ah=$}}while(0);$=ag+ah|0;_=t>>>0>$>>>0?$:t;$=af-(a[T]|0)+U|0;aa=$&255;a[g+76+(C*34|0)+V|0]=aa;ai=aa<<24>>24;ac=c[z>>2]|0;if(((aa<<24>>24>-1?ai:-ai|0)|0)>(ac<<3|7|0)){aj=_;E=3047;break L4200}else{U=$;V=V+1|0;W=_;X=ac}}}else{if((C|0)==0){ak=c[A>>2]|0}else{ak=C}X=ak-1|0;W=(X|0)>0?X:0;X=173536+Q|0;V=0;U=y;T=w;while(1){if((V|0)>=(R|0)){Y=U;Z=T;break L4202}ac=d[g+76+(W*34|0)+V|0]|0;_=q+(U>>>3)|0;$=(cV(d[_]|d[_+1|0]<<8|d[_+2|0]<<16|d[_+3|0]<<24|0)|0)<<(U&7)>>>23;_=b[S+($<<2)>>1]|0;ai=b[S+($<<2)+2>>1]|0;$=ai<<16>>16;do{if(ai<<16>>16<0){aa=U+9|0;ab=t>>>0>aa>>>0?aa:t;aa=q+(ab>>>3)|0;ad=((cV(d[aa]|d[aa+1|0]<<8|d[aa+2|0]<<16|d[aa+3|0]<<24|0)|0)<<(ab&7)>>>(($+32|0)>>>0))+_|0;aa=b[S+(ad<<2)>>1]|0;ae=b[S+(ad<<2)+2>>1]|0;ad=ae<<16>>16;if(ae<<16>>16>=0){al=aa;am=ab;an=ad;break}ae=ab-$|0;ab=t>>>0>ae>>>0?ae:t;ae=q+(ab>>>3)|0;ao=((cV(d[ae]|d[ae+1|0]<<8|d[ae+2|0]<<16|d[ae+3|0]<<24|0)|0)<<(ab&7)>>>((ad+32|0)>>>0))+aa|0;al=b[S+(ao<<2)>>1]|0;am=ab;an=b[S+(ao<<2)+2>>1]|0}else{al=_;am=U;an=$}}while(0);$=am+an|0;_=t>>>0>$>>>0?$:t;$=al+ac-(d[X]|0)&255;a[g+76+(C*34|0)+V|0]=$;ai=$<<24>>24;ao=c[z>>2]|0;if((($<<24>>24>-1?ai:-ai|0)|0)>(ao<<3|7|0)){aj=_;E=3047;break L4200}else{V=V+1|0;U=_;T=ao}}}}while(0);S=C+1|0;ap=c[x>>2]|0;if((S|0)<(ap|0)){C=S;n=Y;w=Z}else{E=3050;break}}if((E|0)==3047){c[o>>2]=aj;AN(e,16,1711544,(B=i,i=i+1|0,i=i+7&-8,c[B>>2]=0,B)|0);i=B;break L4185}else if((E|0)==3050){c[o>>2]=Y;N=ap;O=Y;break}}}while(0);w=g+20|0;do{if((c[w>>2]|0)==0){NM(g+246|0,0,170);aq=N;ar=O}else{if((N|0)<=0){aq=N;ar=O;break}n=g+28|0;C=0;z=O;L4231:while(1){r=((z|0)<(t|0))+z|0;S=d[q+(z>>>3)|0]<<(z&7)>>>7&1;R=S|4;y=c[n>>2]|0;Q=c[1997900+(R<<4)>>2]|0;L4233:do{if((S|0)==0){P=173536+R|0;T=0;U=0;V=r;while(1){if((U|0)>=(y|0)){as=V;break L4233}X=q+(V>>>3)|0;W=(cV(d[X]|d[X+1|0]<<8|d[X+2|0]<<16|d[X+3|0]<<24|0)|0)<<(V&7)>>>23;X=b[Q+(W<<2)>>1]|0;ao=b[Q+(W<<2)+2>>1]|0;W=ao<<16>>16;do{if(ao<<16>>16<0){_=V+9|0;ai=t>>>0>_>>>0?_:t;_=q+(ai>>>3)|0;$=((cV(d[_]|d[_+1|0]<<8|d[_+2|0]<<16|d[_+3|0]<<24|0)|0)<<(ai&7)>>>((W+32|0)>>>0))+X|0;_=b[Q+($<<2)>>1]|0;ab=b[Q+($<<2)+2>>1]|0;$=ab<<16>>16;if(ab<<16>>16>=0){at=_;au=ai;av=$;break}ab=ai-W|0;ai=t>>>0>ab>>>0?ab:t;ab=q+(ai>>>3)|0;aa=((cV(d[ab]|d[ab+1|0]<<8|d[ab+2|0]<<16|d[ab+3|0]<<24|0)|0)<<(ai&7)>>>(($+32|0)>>>0))+_|0;at=b[Q+(aa<<2)>>1]|0;au=ai;av=b[Q+(aa<<2)+2>>1]|0}else{at=X;au=V;av=W}}while(0);W=au+av|0;X=t>>>0>W>>>0?W:t;W=at-(a[P]|0)+T|0;ao=W&255;a[g+246+(C*34|0)+U|0]=ao;if((ao&255)>7){aw=X;E=3069;break L4231}else{T=W;U=U+1|0;V=X}}}else{if((C|0)==0){ax=c[A>>2]|0}else{ax=C}V=ax-1|0;U=(V|0)>0?V:0;V=173536+R|0;T=0;P=r;while(1){if((T|0)>=(y|0)){as=P;break L4233}X=d[g+246+(U*34|0)+T|0]|0;W=q+(P>>>3)|0;ao=(cV(d[W]|d[W+1|0]<<8|d[W+2|0]<<16|d[W+3|0]<<24|0)|0)<<(P&7)>>>23;W=b[Q+(ao<<2)>>1]|0;ac=b[Q+(ao<<2)+2>>1]|0;ao=ac<<16>>16;do{if(ac<<16>>16<0){aa=P+9|0;ai=t>>>0>aa>>>0?aa:t;aa=q+(ai>>>3)|0;_=((cV(d[aa]|d[aa+1|0]<<8|d[aa+2|0]<<16|d[aa+3|0]<<24|0)|0)<<(ai&7)>>>((ao+32|0)>>>0))+W|0;aa=b[Q+(_<<2)>>1]|0;$=b[Q+(_<<2)+2>>1]|0;_=$<<16>>16;if($<<16>>16>=0){ay=aa;az=ai;aA=_;break}$=ai-ao|0;ai=t>>>0>$>>>0?$:t;$=q+(ai>>>3)|0;ab=((cV(d[$]|d[$+1|0]<<8|d[$+2|0]<<16|d[$+3|0]<<24|0)|0)<<(ai&7)>>>((_+32|0)>>>0))+aa|0;ay=b[Q+(ab<<2)>>1]|0;az=ai;aA=b[Q+(ab<<2)+2>>1]|0}else{ay=W;az=P;aA=ao}}while(0);ao=az+aA|0;W=t>>>0>ao>>>0?ao:t;ao=ay+X-(d[V]|0)&255;a[g+246+(C*34|0)+T|0]=ao;if((ao&255)>7){aw=W;E=3069;break L4231}else{T=T+1|0;P=W}}}}while(0);Q=C+1|0;aB=c[x>>2]|0;if((Q|0)<(aB|0)){C=Q;z=as}else{E=3072;break}}if((E|0)==3072){c[o>>2]=as;aq=aB;ar=as;break}else if((E|0)==3069){c[o>>2]=aw;AN(e,16,1714112,(B=i,i=i+1|0,i=i+7&-8,c[B>>2]=0,B)|0);i=B;break L4185}}}while(0);do{if((c[g+32>>2]|0)==0){aC=aq;aD=ar}else{z=q+(ar>>>3)|0;C=(cV(d[z]|d[z+1|0]<<8|d[z+2|0]<<16|d[z+3|0]<<24|0)|0)<<(ar&7)>>>28;z=ar+4|0;n=t>>>0>z>>>0?z:t;c[o>>2]=n;if((C|0)==15){z=q+(n>>>3)|0;Q=(cV(d[z]|d[z+1|0]<<8|d[z+2|0]<<16|d[z+3|0]<<24|0)|0)<<(n&7)>>>24;z=n+8|0;y=t>>>0>z>>>0?z:t;c[o>>2]=y;aE=Q+15|0;aF=y}else{aE=C;aF=n}n=aE<<3;L4263:do{if((n|0)>7){C=g+48|0;y=g+416|0;Q=g+586|0;z=n;r=aq;R=aF;S=t;P=q;T=t;while(1){V=P+(R>>>3)|0;U=cV(d[V]|d[V+1|0]<<8|d[V+2|0]<<16|d[V+3|0]<<24|0)|0;V=R+2|0;W=S>>>0>V>>>0?V:S;c[o>>2]=W;if((U&-1073741824>>>((R&7)>>>0)|0)==0){U=d[P+(W>>>3)|0]<<(W&7)>>>7;V=((W|0)<(S|0))+W|0;c[o>>2]=V;ao=U&1;c[C>>2]=ao;if((ao|0)!=0&(r|0)>0){ao=0;U=V;ac=P;ab=T;while(1){ai=d[ac+(U>>>3)|0]<<(U&7)>>>7;c[o>>2]=((U|0)<(ab|0))+U;aa=ai&1;rB(k,g,y,aa|6,ao,aa);aa=c[o>>2]|0;ai=d[(c[p>>2]|0)+(aa>>>3)|0]<<(aa&7)>>>7;c[o>>2]=((aa|0)<(c[s>>2]|0))+aa;aa=ai&1;rB(k,g,Q,aa|8,ao,aa);aa=ao+1|0;aG=c[x>>2]|0;aH=c[o>>2]|0;if((aa|0)>=(aG|0)){break}ao=aa;U=aH;ac=c[p>>2]|0;ab=c[s>>2]|0}ab=c[s>>2]|0;aI=aG;aJ=aH;aK=ab;aL=ab}else{aI=r;aJ=V;aK=S;aL=T}ab=aJ+1|0;ac=aK>>>0>ab>>>0?ab:aK;c[o>>2]=ac;aM=2-W+ac|0;aN=aI;aO=ac;aP=aK;aQ=aL}else{aM=2;aN=r;aO=W;aP=S;aQ=T}ac=z-aM|0;if((ac|0)<=7){aR=ac;aS=aN;aT=aO;aU=aP;break L4263}z=ac;r=aN;R=aO;S=aP;P=c[p>>2]|0;T=aQ}}else{aR=n;aS=aq;aT=aF;aU=t}}while(0);if((aR|0)<0){AN(e,16,1692952,(B=i,i=i+8|0,c[B>>2]=aR,B)|0);i=B;break L4185}else{n=aT+aR|0;T=aU>>>0>n>>>0?n:aU;c[o>>2]=T;aC=aS;aD=T;break}}}while(0);T=g+48|0;c[T>>2]=c[T>>2]&1;if((aC|0)==0){aV=c[A>>2]|0;aW=0;E=3092}else{if((c[g+52+(aC<<2)>>2]|0)<31){aV=aC;aW=aC;E=3092}}if((E|0)==3092){n=aV-1|0;do{if(!((aV|0)<1|(n|0)==(aW|0))){if((c[u>>2]|0)!=0){P=g+76+(aW*34|0)|0;S=g+76+(n*34|0)|0;NN(P|0,S|0,34)|0}if((c[w>>2]|0)!=0){S=g+246+((c[x>>2]|0)*34|0)|0;P=g+246+(n*34|0)|0;NN(S|0,P|0,34)|0}if((c[T>>2]|0)==0){break}P=g+416+((c[x>>2]|0)*34|0)|0;S=g+416+(n*34|0)|0;NN(P|0,S|0,34)|0;S=g+586+((c[x>>2]|0)*34|0)|0;P=g+586+(n*34|0)|0;NN(S|0,P|0,34)|0}}while(0);L4297:do{if((c[u>>2]|0)!=0){n=c[g+12>>2]|0;A=g+8|0;P=0;while(1){if((P|0)>=(n|0)){break L4297}S=a[g+76+((c[x>>2]|0)*34|0)+P|0]|0;R=S<<24>>24;if(((S<<24>>24>-1?R:-R|0)|0)>(c[A>>2]<<3|7|0)){break}else{P=P+1|0}}AN(e,16,1672072,(B=i,i=i+1|0,i=i+7&-8,c[B>>2]=0,B)|0);i=B;break L4185}}while(0);L4304:do{if((c[w>>2]|0)!=0){P=c[g+12>>2]|0;A=0;while(1){if((A|0)>=(P|0)){break L4304}if((d[g+246+((c[x>>2]|0)*34|0)+A|0]|0)>7){break}else{A=A+1|0}}AN(e,16,1657400,(B=i,i=i+1|0,i=i+7&-8,c[B>>2]=0,B)|0);i=B;break L4185}}while(0);A=(c[x>>2]|0)+1|0;c[x>>2]=A;c[g+52+(A<<2)>>2]=31}A=g+756|0;c[g+760>>2]=c[A>>2];do{if((c[u>>2]|0)==0){if((c[w>>2]|0)!=0){E=3114}}else{if((c[g+12>>2]|0)==34){aX=1;E=3115;break}if((c[w>>2]|0)==0){aX=0;E=3115}else{E=3114}}}while(0);if((E|0)==3114){aX=(c[g+28>>2]|0)==34|0;E=3115}if((E|0)==3115){c[A>>2]=aX}if((c[T>>2]|0)==0){NM(g+416|0,0,340)}if(v){c[g>>2]=1}w=aD-m|0;if((w|0)>(h|0)){AN(e,16,1638392,(B=i,i=i+16|0,c[B>>2]=h,c[B+8>>2]=w,B)|0);i=B;break}u=c[l>>2]|0;x=-u|0;P=(c[f+16>>2]|0)-u|0;if((w|0)<(x|0)){aY=x}else{aY=(P|0)<(w|0)?P:w}c[l>>2]=aY+u;aZ=w;i=j;return aZ|0}}while(0);c[g>>2]=0;aY=c[l>>2]|0;B=-aY|0;e=(c[f+16>>2]|0)-aY|0;if((h|0)<(B|0)){a_=B}else{a_=(e|0)<(h|0)?e:h}c[l>>2]=a_+aY;NM(g+76|0,0,680);aZ=h;i=j;return aZ|0}function ru(b,d,e,f,h){b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0.0,$=0.0,aa=0.0,ab=0.0,ac=0.0,ad=0.0,ae=0.0,af=0,ag=0,ah=0.0,ai=0.0,aj=0.0,ak=0.0,al=0,am=0.0,an=0.0,ao=0.0,ap=0.0,aq=0.0,ar=0.0,as=0,at=0,au=0;b=i;i=i+56160|0;j=b|0;k=b+176|0;l=b+352|0;m=b+528|0;n=b+704|0;o=b+712|0;p=b+720|0;q=b+728|0;r=b+736|0;s=b+768|0;t=b+800|0;u=b+5152|0;v=b+9504|0;w=b+9568|0;x=b+32864|0;y=w|0;z=x|0;A=c[d+756>>2]|0;B=1969408+(A<<2)|0;C=c[B>>2]|0;D=h-64+C|0;NM(d+2528+(D*368|0)|0,0,(64-h|0)*368|0|0);h=c[1969416+(A<<2)>>2]|0;if((D|0)<(h|0)){NM(d+36016+(D*888|0)|0,0,(h-D|0)*888|0|0)}D=d+87444|0;E=0;do{F=E+6|0;g[d+768+(F<<3)>>2]=+g[e+(E<<8)>>2];g[d+768+(F<<3)+4>>2]=+g[e+9728+(E<<8)>>2];E=E+1|0;}while((E|0)<38);E=d+768|0;F=0;while(1){G=F+6|0;g[d+1120+(G<<3)>>2]=+g[e+(F<<8)+4>>2];g[d+1120+(G<<3)+4>>2]=+g[e+9728+(F<<8)+4>>2];G=F+1|0;if((G|0)<38){F=G}else{H=0;break}}while(1){F=H+6|0;g[d+1472+(F<<3)>>2]=+g[e+(H<<8)+8>>2];g[d+1472+(F<<3)+4>>2]=+g[e+9728+(H<<8)+8>>2];F=H+1|0;if((F|0)<38){H=F}else{I=0;break}}while(1){H=I+6|0;g[d+1824+(H<<3)>>2]=+g[e+(I<<8)+12>>2];g[d+1824+(H<<3)+4>>2]=+g[e+9728+(I<<8)+12>>2];H=I+1|0;if((H|0)<38){I=H}else{J=0;break}}do{I=J+6|0;g[d+2176+(I<<3)>>2]=+g[e+(J<<8)+16>>2];g[d+2176+(I<<3)+4>>2]=+g[e+9728+(J<<8)+16>>2];J=J+1|0;}while((J|0)<38);J=E|0;if((A|0)==0){I=v|0;H=d+87452|0;F=v+48|0;G=v+52|0;K=v+56|0;L=v+60|0;M=v|0;N=v+4|0;O=v+8|0;P=v+12|0;Q=v+16|0;R=v+40|0;S=v+20|0;T=v+44|0;U=v+24|0;V=v+32|0;W=v+28|0;X=v+36|0;v=0;Y=J;while(1){cX[c[H>>2]&255](I,Y,2908264,1,8);g[w+(v<<3)>>2]=+g[F>>2];g[w+(v<<3)+4>>2]=+g[G>>2];g[w+256+(v<<3)>>2]=+g[K>>2];g[w+256+(v<<3)+4>>2]=+g[L>>2];g[w+512+(v<<3)>>2]=+g[M>>2];g[w+512+(v<<3)+4>>2]=+g[N>>2];g[w+768+(v<<3)>>2]=+g[O>>2];g[w+768+(v<<3)+4>>2]=+g[P>>2];g[w+1024+(v<<3)>>2]=+g[Q>>2]+ +g[R>>2];g[w+1024+(v<<3)+4>>2]=+g[S>>2]+ +g[T>>2];g[w+1280+(v<<3)>>2]=+g[U>>2]+ +g[V>>2];g[w+1280+(v<<3)+4>>2]=+g[W>>2]+ +g[X>>2];Z=v+1|0;if((Z|0)<32){v=Z;Y=Y+8|0}else{break}}Y=0;v=d+1120|0;while(1){_=(+g[v+8>>2]+ +g[v+88>>2])*.018994875252246857+0.0+(+g[v+24>>2]+ +g[v+72>>2])*-.07293139398097992+(+g[v+40>>2]+ +g[v+56>>2])*.305966317653656;$=(+g[v+12>>2]+ +g[v+92>>2])*.018994875252246857+0.0+(+g[v+28>>2]+ +g[v+76>>2])*-.07293139398097992+(+g[v+44>>2]+ +g[v+60>>2])*.305966317653656;aa=+g[v+48>>2]*.5;ab=+g[v+52>>2]*.5;g[w+1792+(Y<<3)>>2]=aa+_;g[w+1792+(Y<<3)+4>>2]=ab+$;g[w+1536+(Y<<3)>>2]=aa-_;g[w+1536+(Y<<3)+4>>2]=ab-$;X=Y+1|0;if((X|0)<32){Y=X;v=v+8|0}else{break}}v=0;Y=d+1472|0;while(1){$=(+g[Y+8>>2]+ +g[Y+88>>2])*.018994875252246857+0.0+(+g[Y+24>>2]+ +g[Y+72>>2])*-.07293139398097992+(+g[Y+40>>2]+ +g[Y+56>>2])*.305966317653656;ab=(+g[Y+12>>2]+ +g[Y+92>>2])*.018994875252246857+0.0+(+g[Y+28>>2]+ +g[Y+76>>2])*-.07293139398097992+(+g[Y+44>>2]+ +g[Y+60>>2])*.305966317653656;_=+g[Y+48>>2]*.5;aa=+g[Y+52>>2]*.5;g[w+2048+(v<<3)>>2]=_+$;g[w+2048+(v<<3)+4>>2]=aa+ab;g[w+2304+(v<<3)>>2]=_-$;g[w+2304+(v<<3)+4>>2]=aa-ab;X=v+1|0;if((X|0)<32){v=X;Y=Y+8|0}else{break}}dq[c[d+87456>>2]&127](w+1792|0,e,3,32)}else{Y=d+87452|0;v=J;J=0;while(1){cX[c[Y>>2]&255](w+(J<<3)|0,v,2907496,32,12);X=J+1|0;if((X|0)<32){v=v+8|0;J=X}else{break}}J=d+1120|0;v=0;while(1){cX[c[Y>>2]&255](w+3072+(v<<3)|0,J,2906984,32,8);X=v+1|0;if((X|0)<32){J=J+8|0;v=X}else{break}}v=d+1472|0;J=0;while(1){cX[c[Y>>2]&255](w+5120+(J<<3)|0,v,2906728,32,4);X=J+1|0;if((X|0)<32){v=v+8|0;J=X}else{break}}J=d+1824|0;v=0;while(1){cX[c[Y>>2]&255](w+6144+(v<<3)|0,J,2906728,32,4);X=v+1|0;if((X|0)<32){J=J+8|0;v=X}else{break}}v=d+2176|0;J=0;while(1){cX[c[Y>>2]&255](w+7168+(J<<3)|0,v,2906728,32,4);X=J+1|0;if((X|0)<32){v=v+8|0;J=X}else{break}}dq[c[d+87456>>2]&127](w+6912|0,e,5,32)}J=E;E=d+1024|0;NN(J|0,E|0,48)|0;E=d+1120|0;J=d+1376|0;NN(E|0,J|0,48)|0;J=d+1472|0;E=d+1728|0;NN(J|0,E|0,48)|0;E=d+1824|0;J=d+2080|0;NN(E|0,J|0,48)|0;J=d+2176|0;E=d+2432|0;NN(J|0,E|0,48)|0;E=(A|0)!=0;J=E?156760:156856;NM(t|0,0,4352);v=d+760|0;if((c[v>>2]|0)!=(A|0)){NM(d+80560|0,0,136);NM(d+80704|0,0,136);NM(d+2528|0,0,78024)}Y=A>>>0<2;if(Y){X=d+87444|0;W=0;do{dm[c[X>>2]&127](t+(a[J+W|0]<<7)|0,w+(W<<8)|0,32);W=W+1|0;}while((W|0)<(C|0));W=c[1969400+(A<<2)>>2]|0;X=0;do{V=d+80416+(X<<2)|0;U=d+80560+(X<<2)|0;T=d+80704+(X<<2)|0;S=0;ab=+g[V>>2];aa=+g[U>>2];$=+g[T>>2];do{_=ab*.765928328037262;ac=+g[t+(X<<7)+(S<<2)>>2];ab=_>ac?_:ac;aa=aa+(ac-aa)*.25;$=$+(ab-ac-$)*.25;ac=$*1.5;if(ac>aa){ad=aa/ac}else{ad=1.0}g[u+(X<<7)+(S<<2)>>2]=ad;S=S+1|0;}while((S|0)<32);g[V>>2]=ab;g[U>>2]=aa;g[T>>2]=$;X=X+1|0;}while((X|0)<(W|0));W=c[1991408+(A<<2)>>2]|0;X=d+87464|0;t=0;do{S=a[J+t|0]|0;ad=1.0- +(t-W|0)*.05000000074505806;if(ad<0.0){ae=0.0}else{ae=ad>1.0?1.0:ad}R=d+2528+(t*368|0)|0;Q=d+2528+(t*368|0)+256|0;NN(R|0,Q|0,112)|0;Q=d+2528+(t*368|0)+112|0;R=w+(t<<8)|0;NN(Q|0,R|0,256)|0;R=d+36016+(t*888|0)|0;Q=R;P=d+36016+(t*888|0)+256|0;NN(Q|0,P|0,40)|0;P=d+36016+(t*888|0)+296|0;Q=d+36016+(t*888|0)+552|0;NN(P|0,Q|0,40)|0;Q=d+36016+(t*888|0)+592|0;P=d+36016+(t*888|0)+848|0;NN(Q|0,P|0,40)|0;c2[c[X>>2]&3](x+(t<<8)|0,d+2528+(t*368|0)+96|0,R,2198600+(A*400|0)+(t<<3)|0,3119664+(A*1200|0)+(t*24|0)|0,u+(S<<7)|0,ae,32);t=t+1|0;}while((t|0)<(h|0));af=(h|0)>1?h:1}else{af=0}h=c[1966168+(A<<2)>>2]|0;if((af|0)<(h|0)){t=d+87448|0;X=af+1|0;W=af;do{S=a[J+W|0]|0;R=d+2528+(W*368|0)|0;P=d+2528+(W*368|0)+256|0;NN(R|0,P|0,112)|0;P=d+2528+(W*368|0)+112|0;R=w+(W<<8)|0;NN(P|0,R|0,256)|0;dq[c[t>>2]&127](x+(W<<8)|0,d+2528+(W*368|0)|0,u+(S<<7)|0,32);W=W+1|0;}while((W|0)<(h|0));ag=(h|0)>(X|0)?h:X}else{ag=af}if((ag|0)<(C|0)){af=d+87448|0;X=ag;do{ag=a[J+X|0]|0;h=d+2528+(X*368|0)|0;W=d+2528+(X*368|0)+256|0;NN(h|0,W|0,112)|0;W=d+2528+(X*368|0)+112|0;h=w+(X<<8)|0;NN(W|0,h|0,256)|0;dq[c[af>>2]&127](x+(X<<8)|0,d+2528+(X*368|0)+104|0,u+(ag<<7)|0,32);X=X+1|0;}while((X|0)<(C|0))}C=d+80848|0;X=d+82480|0;u=d+84112|0;af=d+85744|0;ag=d+87376|0;h=d+87410|0;c[n>>2]=j;c[o>>2]=k;c[p>>2]=l;c[q>>2]=m;m=(c[d+24>>2]|0)<3?3127952:3122064;l=c[d+40>>2]|0;if((l|0)!=0){k=C;j=d+80848+(l*136|0)|0;NN(k|0,j|0,136)|0;j=d+81664|0;k=d+81664+(l*136|0)|0;NN(j|0,k|0,136)|0;k=X;j=d+82480+(l*136|0)|0;NN(k|0,j|0,136)|0;j=d+83296|0;k=d+83296+(l*136|0)|0;NN(j|0,k|0,136)|0;k=u;j=d+84112+(l*136|0)|0;NN(k|0,j|0,136)|0;j=d+84928|0;k=d+84928+(l*136|0)|0;NN(j|0,k|0,136)|0;k=af;j=d+85744+(l*136|0)|0;NN(k|0,j|0,136)|0;j=d+86560|0;k=d+86560+(l*136|0)|0;NN(j|0,k|0,136)|0}k=d+76|0;j=c[d+12>>2]|0;l=d+44|0;W=c[l>>2]|0;do{if(E){ry(n,k,j,W,1);ry(o,d+246|0,c[d+28>>2]|0,c[l>>2]|0,1);if((c[d+48>>2]|0)!=0){t=d+16|0;ry(p,d+416|0,c[t>>2]|0,c[l>>2]|0,0);ry(q,d+586|0,c[t>>2]|0,c[l>>2]|0,0)}if((c[v>>2]|0)!=0){break}t=d+80924|0;ae=+g[t>>2];g[d+80980>>2]=ae;g[d+80976>>2]=ae;S=d+80920|0;ae=+g[S>>2];g[d+80972>>2]=ae;g[d+80968>>2]=ae;g[d+80964>>2]=ae;g[d+80960>>2]=ae;R=d+80916|0;ae=+g[R>>2];g[d+80956>>2]=ae;g[d+80952>>2]=ae;P=d+80912|0;ae=+g[P>>2];g[d+80948>>2]=ae;g[d+80944>>2]=ae;Q=d+80908|0;ae=+g[Q>>2];g[d+80940>>2]=ae;g[d+80936>>2]=ae;O=d+80904|0;ae=+g[O>>2];g[d+80932>>2]=ae;g[d+80928>>2]=ae;N=d+80900|0;g[t>>2]=+g[N>>2];t=d+80896|0;g[S>>2]=+g[t>>2];S=d+80892|0;g[R>>2]=+g[S>>2];R=d+80888|0;g[P>>2]=+g[R>>2];P=d+80884|0;ae=+g[P>>2];g[Q>>2]=ae;g[O>>2]=ae;O=d+80880|0;ae=+g[O>>2];g[N>>2]=ae;g[t>>2]=ae;t=d+80876|0;g[S>>2]=+g[t>>2];S=d+80872|0;g[R>>2]=+g[S>>2];R=d+80868|0;ae=+g[R>>2];g[P>>2]=ae;g[O>>2]=ae;O=d+80864|0;ae=+g[O>>2];g[t>>2]=ae;g[S>>2]=ae;S=d+80860|0;ae=+g[S>>2];g[R>>2]=ae;R=d+80856|0;ad=+g[R>>2];g[O>>2]=(ae+ad)*.5;g[S>>2]=ad;S=d+80852|0;ad=+g[S>>2];g[R>>2]=ad;g[S>>2]=(ad+ +g[C>>2])*.5;S=d+81740|0;ad=+g[S>>2];g[d+81796>>2]=ad;g[d+81792>>2]=ad;R=d+81736|0;ad=+g[R>>2];g[d+81788>>2]=ad;g[d+81784>>2]=ad;g[d+81780>>2]=ad;g[d+81776>>2]=ad;O=d+81732|0;ad=+g[O>>2];g[d+81772>>2]=ad;g[d+81768>>2]=ad;t=d+81728|0;ad=+g[t>>2];g[d+81764>>2]=ad;g[d+81760>>2]=ad;P=d+81724|0;ad=+g[P>>2];g[d+81756>>2]=ad;g[d+81752>>2]=ad;N=d+81720|0;ad=+g[N>>2];g[d+81748>>2]=ad;g[d+81744>>2]=ad;Q=d+81716|0;g[S>>2]=+g[Q>>2];S=d+81712|0;g[R>>2]=+g[S>>2];R=d+81708|0;g[O>>2]=+g[R>>2];O=d+81704|0;g[t>>2]=+g[O>>2];t=d+81700|0;ad=+g[t>>2];g[P>>2]=ad;g[N>>2]=ad;N=d+81696|0;ad=+g[N>>2];g[Q>>2]=ad;g[S>>2]=ad;S=d+81692|0;g[R>>2]=+g[S>>2];R=d+81688|0;g[O>>2]=+g[R>>2];O=d+81684|0;ad=+g[O>>2];g[t>>2]=ad;g[N>>2]=ad;N=d+81680|0;ad=+g[N>>2];g[S>>2]=ad;g[R>>2]=ad;R=d+81676|0;ad=+g[R>>2];g[O>>2]=ad;O=d+81672|0;ae=+g[O>>2];g[N>>2]=(ad+ae)*.5;g[R>>2]=ae;R=d+81668|0;ae=+g[R>>2];g[O>>2]=ae;g[R>>2]=(ae+ +g[d+81664>>2])*.5;R=d+82556|0;ae=+g[R>>2];g[d+82612>>2]=ae;g[d+82608>>2]=ae;O=d+82552|0;ae=+g[O>>2];g[d+82604>>2]=ae;g[d+82600>>2]=ae;g[d+82596>>2]=ae;g[d+82592>>2]=ae;N=d+82548|0;ae=+g[N>>2];g[d+82588>>2]=ae;g[d+82584>>2]=ae;S=d+82544|0;ae=+g[S>>2];g[d+82580>>2]=ae;g[d+82576>>2]=ae;t=d+82540|0;ae=+g[t>>2];g[d+82572>>2]=ae;g[d+82568>>2]=ae;Q=d+82536|0;ae=+g[Q>>2];g[d+82564>>2]=ae;g[d+82560>>2]=ae;P=d+82532|0;g[R>>2]=+g[P>>2];R=d+82528|0;g[O>>2]=+g[R>>2];O=d+82524|0;g[N>>2]=+g[O>>2];N=d+82520|0;g[S>>2]=+g[N>>2];S=d+82516|0;ae=+g[S>>2];g[t>>2]=ae;g[Q>>2]=ae;Q=d+82512|0;ae=+g[Q>>2];g[P>>2]=ae;g[R>>2]=ae;R=d+82508|0;g[O>>2]=+g[R>>2];O=d+82504|0;g[N>>2]=+g[O>>2];N=d+82500|0;ae=+g[N>>2];g[S>>2]=ae;g[Q>>2]=ae;Q=d+82496|0;ae=+g[Q>>2];g[R>>2]=ae;g[O>>2]=ae;O=d+82492|0;ae=+g[O>>2];g[N>>2]=ae;N=d+82488|0;ad=+g[N>>2];g[Q>>2]=(ae+ad)*.5;g[O>>2]=ad;O=d+82484|0;ad=+g[O>>2];g[N>>2]=ad;g[O>>2]=(ad+ +g[X>>2])*.5;O=d+83372|0;ad=+g[O>>2];g[d+83428>>2]=ad;g[d+83424>>2]=ad;N=d+83368|0;ad=+g[N>>2];g[d+83420>>2]=ad;g[d+83416>>2]=ad;g[d+83412>>2]=ad;g[d+83408>>2]=ad;Q=d+83364|0;ad=+g[Q>>2];g[d+83404>>2]=ad;g[d+83400>>2]=ad;R=d+83360|0;ad=+g[R>>2];g[d+83396>>2]=ad;g[d+83392>>2]=ad;S=d+83356|0;ad=+g[S>>2];g[d+83388>>2]=ad;g[d+83384>>2]=ad;P=d+83352|0;ad=+g[P>>2];g[d+83380>>2]=ad;g[d+83376>>2]=ad;t=d+83348|0;g[O>>2]=+g[t>>2];O=d+83344|0;g[N>>2]=+g[O>>2];N=d+83340|0;g[Q>>2]=+g[N>>2];Q=d+83336|0;g[R>>2]=+g[Q>>2];R=d+83332|0;ad=+g[R>>2];g[S>>2]=ad;g[P>>2]=ad;P=d+83328|0;ad=+g[P>>2];g[t>>2]=ad;g[O>>2]=ad;O=d+83324|0;g[N>>2]=+g[O>>2];N=d+83320|0;g[Q>>2]=+g[N>>2];Q=d+83316|0;ad=+g[Q>>2];g[R>>2]=ad;g[P>>2]=ad;P=d+83312|0;ad=+g[P>>2];g[O>>2]=ad;g[N>>2]=ad;N=d+83308|0;ad=+g[N>>2];g[Q>>2]=ad;Q=d+83304|0;ae=+g[Q>>2];g[P>>2]=(ad+ae)*.5;g[N>>2]=ae;N=d+83300|0;ae=+g[N>>2];g[Q>>2]=ae;g[N>>2]=(ae+ +g[d+83296>>2])*.5;N=d+84188|0;ae=+g[N>>2];g[d+84244>>2]=ae;g[d+84240>>2]=ae;Q=d+84184|0;ae=+g[Q>>2];g[d+84236>>2]=ae;g[d+84232>>2]=ae;g[d+84228>>2]=ae;g[d+84224>>2]=ae;P=d+84180|0;ae=+g[P>>2];g[d+84220>>2]=ae;g[d+84216>>2]=ae;O=d+84176|0;ae=+g[O>>2];g[d+84212>>2]=ae;g[d+84208>>2]=ae;R=d+84172|0;ae=+g[R>>2];g[d+84204>>2]=ae;g[d+84200>>2]=ae;t=d+84168|0;ae=+g[t>>2];g[d+84196>>2]=ae;g[d+84192>>2]=ae;S=d+84164|0;g[N>>2]=+g[S>>2];N=d+84160|0;g[Q>>2]=+g[N>>2];Q=d+84156|0;g[P>>2]=+g[Q>>2];P=d+84152|0;g[O>>2]=+g[P>>2];O=d+84148|0;ae=+g[O>>2];g[R>>2]=ae;g[t>>2]=ae;t=d+84144|0;ae=+g[t>>2];g[S>>2]=ae;g[N>>2]=ae;N=d+84140|0;g[Q>>2]=+g[N>>2];Q=d+84136|0;g[P>>2]=+g[Q>>2];P=d+84132|0;ae=+g[P>>2];g[O>>2]=ae;g[t>>2]=ae;t=d+84128|0;ae=+g[t>>2];g[N>>2]=ae;g[Q>>2]=ae;Q=d+84124|0;ae=+g[Q>>2];g[P>>2]=ae;P=d+84120|0;ad=+g[P>>2];g[t>>2]=(ae+ad)*.5;g[Q>>2]=ad;Q=d+84116|0;ad=+g[Q>>2];g[P>>2]=ad;g[Q>>2]=(ad+ +g[u>>2])*.5;Q=d+85004|0;ad=+g[Q>>2];g[d+85060>>2]=ad;g[d+85056>>2]=ad;P=d+85e3|0;ad=+g[P>>2];g[d+85052>>2]=ad;g[d+85048>>2]=ad;g[d+85044>>2]=ad;g[d+85040>>2]=ad;t=d+84996|0;ad=+g[t>>2];g[d+85036>>2]=ad;g[d+85032>>2]=ad;N=d+84992|0;ad=+g[N>>2];g[d+85028>>2]=ad;g[d+85024>>2]=ad;O=d+84988|0;ad=+g[O>>2];g[d+85020>>2]=ad;g[d+85016>>2]=ad;S=d+84984|0;ad=+g[S>>2];g[d+85012>>2]=ad;g[d+85008>>2]=ad;R=d+84980|0;g[Q>>2]=+g[R>>2];Q=d+84976|0;g[P>>2]=+g[Q>>2];P=d+84972|0;g[t>>2]=+g[P>>2];t=d+84968|0;g[N>>2]=+g[t>>2];N=d+84964|0;ad=+g[N>>2];g[O>>2]=ad;g[S>>2]=ad;S=d+84960|0;ad=+g[S>>2];g[R>>2]=ad;g[Q>>2]=ad;Q=d+84956|0;g[P>>2]=+g[Q>>2];P=d+84952|0;g[t>>2]=+g[P>>2];t=d+84948|0;ad=+g[t>>2];g[N>>2]=ad;g[S>>2]=ad;S=d+84944|0;ad=+g[S>>2];g[Q>>2]=ad;g[P>>2]=ad;P=d+84940|0;ad=+g[P>>2];g[t>>2]=ad;t=d+84936|0;ae=+g[t>>2];g[S>>2]=(ad+ae)*.5;g[P>>2]=ae;P=d+84932|0;ae=+g[P>>2];g[t>>2]=ae;g[P>>2]=(ae+ +g[d+84928>>2])*.5;P=d+85820|0;ae=+g[P>>2];g[d+85876>>2]=ae;g[d+85872>>2]=ae;t=d+85816|0;ae=+g[t>>2];g[d+85868>>2]=ae;g[d+85864>>2]=ae;g[d+85860>>2]=ae;g[d+85856>>2]=ae;S=d+85812|0;ae=+g[S>>2];g[d+85852>>2]=ae;g[d+85848>>2]=ae;Q=d+85808|0;ae=+g[Q>>2];g[d+85844>>2]=ae;g[d+85840>>2]=ae;N=d+85804|0;ae=+g[N>>2];g[d+85836>>2]=ae;g[d+85832>>2]=ae;R=d+85800|0;ae=+g[R>>2];g[d+85828>>2]=ae;g[d+85824>>2]=ae;O=d+85796|0;g[P>>2]=+g[O>>2];P=d+85792|0;g[t>>2]=+g[P>>2];t=d+85788|0;g[S>>2]=+g[t>>2];S=d+85784|0;g[Q>>2]=+g[S>>2];Q=d+85780|0;ae=+g[Q>>2];g[N>>2]=ae;g[R>>2]=ae;R=d+85776|0;ae=+g[R>>2];g[O>>2]=ae;g[P>>2]=ae;P=d+85772|0;g[t>>2]=+g[P>>2];t=d+85768|0;g[S>>2]=+g[t>>2];S=d+85764|0;ae=+g[S>>2];g[Q>>2]=ae;g[R>>2]=ae;R=d+85760|0;ae=+g[R>>2];g[P>>2]=ae;g[t>>2]=ae;t=d+85756|0;ae=+g[t>>2];g[S>>2]=ae;S=d+85752|0;ad=+g[S>>2];g[R>>2]=(ae+ad)*.5;g[t>>2]=ad;t=d+85748|0;ad=+g[t>>2];g[S>>2]=ad;g[t>>2]=(ad+ +g[af>>2])*.5;t=d+86636|0;ad=+g[t>>2];g[d+86692>>2]=ad;g[d+86688>>2]=ad;S=d+86632|0;ad=+g[S>>2];g[d+86684>>2]=ad;g[d+86680>>2]=ad;g[d+86676>>2]=ad;g[d+86672>>2]=ad;R=d+86628|0;ad=+g[R>>2];g[d+86668>>2]=ad;g[d+86664>>2]=ad;P=d+86624|0;ad=+g[P>>2];g[d+86660>>2]=ad;g[d+86656>>2]=ad;Q=d+86620|0;ad=+g[Q>>2];g[d+86652>>2]=ad;g[d+86648>>2]=ad;O=d+86616|0;ad=+g[O>>2];g[d+86644>>2]=ad;g[d+86640>>2]=ad;N=d+86612|0;g[t>>2]=+g[N>>2];t=d+86608|0;g[S>>2]=+g[t>>2];S=d+86604|0;g[R>>2]=+g[S>>2];R=d+86600|0;g[P>>2]=+g[R>>2];P=d+86596|0;ad=+g[P>>2];g[Q>>2]=ad;g[O>>2]=ad;O=d+86592|0;ad=+g[O>>2];g[N>>2]=ad;g[t>>2]=ad;t=d+86588|0;g[S>>2]=+g[t>>2];S=d+86584|0;g[R>>2]=+g[S>>2];R=d+86580|0;ad=+g[R>>2];g[P>>2]=ad;g[O>>2]=ad;O=d+86576|0;ad=+g[O>>2];g[t>>2]=ad;g[S>>2]=ad;S=d+86572|0;ad=+g[S>>2];g[R>>2]=ad;R=d+86568|0;ae=+g[R>>2];g[O>>2]=(ad+ae)*.5;g[S>>2]=ae;S=d+86564|0;ae=+g[S>>2];g[R>>2]=ae;g[S>>2]=(ae+ +g[d+86560>>2])*.5;NM(ag|0,0,17);NM(h|0,0,17)}else{rz(n,k,j,W,1);rz(o,d+246|0,c[d+28>>2]|0,c[l>>2]|0,1);if((c[d+48>>2]|0)!=0){S=d+16|0;rz(p,d+416|0,c[S>>2]|0,c[l>>2]|0,0);rz(q,d+586|0,c[S>>2]|0,c[l>>2]|0,0)}if((c[v>>2]|0)==0){break}rA(C|0);rA(d+81664|0);rA(X|0);rA(d+83296|0);rA(u|0);rA(d+84928|0);rA(af|0);rA(d+86560|0);NM(ag|0,0,17);NM(h|0,0,17)}}while(0);if((c[l>>2]|0)<=0){rv(D,e,y,A);rv(D,f,z,A);i=b;return 0}h=c[1969400+(A<<2)>>2]|0;ag=r|0;af=r|0;u=r+4|0;X=r+8|0;C=r+12|0;v=d+48|0;W=s|0;j=s|0;k=s+4|0;S=s+8|0;R=s+12|0;O=r+16|0;t=s+16|0;P=r+20|0;N=s+20|0;Q=r+24|0;M=s+24|0;L=r+28|0;r=s+28|0;s=c[o>>2]|0;o=c[n>>2]|0;n=d+8|0;K=c[q>>2]|0;q=c[p>>2]|0;p=0;while(1){G=p+1|0;if(Y){F=0;do{I=a[s+(p*34|0)+F|0]|0;H=(a[o+(p*34|0)+F|0]|0)+7+((c[n>>2]|0)*23|0)|0;ae=+g[m+(H<<7)+(I<<4)>>2];ad=+g[m+(H<<7)+(I<<4)+4>>2];ac=+g[m+(H<<7)+(I<<4)+8>>2];_=+g[m+(H<<7)+(I<<4)+12>>2];do{if((c[v>>2]|0)==0){ah=ae;ai=ad;aj=ac;ak=_}else{if((F<<1|0)>(h|0)){ah=ae;ai=ad;aj=ac;ak=_;break}I=d+87376+F|0;H=(a[K+(p*34|0)+F|0]|0)+(a[I]<<3)|0;Z=d+87410+F|0;al=(a[q+(p*34|0)+F|0]|0)+(a[Z]<<3)|0;am=+g[2199400+(H<<2)>>2];an=+g[2201448+(H<<2)>>2];ao=+g[2199400+(al<<2)>>2];ap=+g[2201448+(al<<2)>>2];a[I]=H&63;a[Z]=al&63;aq=am*ao+an*ap;ar=an*ao-am*ap;g[d+81664+(G*136|0)+(F<<2)>>2]=ae*an;g[d+83296+(G*136|0)+(F<<2)>>2]=ad*ar;g[d+84928+(G*136|0)+(F<<2)>>2]=ac*an;g[d+86560+(G*136|0)+(F<<2)>>2]=_*ar;ah=ae*am;ai=ad*aq;aj=ac*am;ak=_*aq}}while(0);g[d+80848+(G*136|0)+(F<<2)>>2]=ah;g[d+82480+(G*136|0)+(F<<2)>>2]=ai;g[d+84112+(G*136|0)+(F<<2)>>2]=aj;g[d+85744+(G*136|0)+(F<<2)>>2]=ak;F=F+1|0;}while((F|0)<(h|0));F=d+52+(p<<2)|0;T=d+52+(G<<2)|0;U=c[B>>2]|0;V=0;do{al=c[F>>2]|0;Z=(c[T>>2]|0)-al|0;$=1.0/+(Z|0);H=a[J+V|0]|0;aa=+g[d+80848+(p*136|0)+(H<<2)>>2];g[af>>2]=aa;ab=+g[d+82480+(p*136|0)+(H<<2)>>2];g[u>>2]=ab;_=+g[d+84112+(p*136|0)+(H<<2)>>2];g[X>>2]=_;ac=+g[d+85744+(p*136|0)+(H<<2)>>2];g[C>>2]=ac;I=c[v>>2]|0;as=(I|0)==0;do{if(!as){if(E){if((V-9|0)>>>0<5){at=3205}else{at=3206}}else{if((V|0)<2){at=3205}else{at=3206}}if((at|0)==3205){at=0;g[O>>2]=-0.0- +g[d+81664+(p*136|0)+(H<<2)>>2];g[P>>2]=-0.0- +g[d+83296+(p*136|0)+(H<<2)>>2];g[Q>>2]=-0.0- +g[d+84928+(p*136|0)+(H<<2)>>2];g[L>>2]=-0.0- +g[d+86560+(p*136|0)+(H<<2)>>2];break}else if((at|0)==3206){at=0;g[O>>2]=+g[d+81664+(p*136|0)+(H<<2)>>2];g[P>>2]=+g[d+83296+(p*136|0)+(H<<2)>>2];g[Q>>2]=+g[d+84928+(p*136|0)+(H<<2)>>2];g[L>>2]=+g[d+86560+(p*136|0)+(H<<2)>>2];break}}}while(0);g[j>>2]=$*(+g[d+80848+(G*136|0)+(H<<2)>>2]-aa);g[k>>2]=$*(+g[d+82480+(G*136|0)+(H<<2)>>2]-ab);g[S>>2]=$*(+g[d+84112+(G*136|0)+(H<<2)>>2]-_);g[R>>2]=$*(+g[d+85744+(G*136|0)+(H<<2)>>2]-ac);if(!as){g[t>>2]=$*(+g[d+81664+(G*136|0)+(H<<2)>>2]- +g[O>>2]);g[N>>2]=$*(+g[d+83296+(G*136|0)+(H<<2)>>2]- +g[P>>2]);g[M>>2]=$*(+g[d+84928+(G*136|0)+(H<<2)>>2]- +g[Q>>2]);g[r>>2]=$*(+g[d+86560+(G*136|0)+(H<<2)>>2]- +g[L>>2])}au=al+1|0;cX[c[d+87468+(((I|0)!=0)<<2)>>2]&255](w+(V<<8)+(au<<3)|0,x+(V<<8)+(au<<3)|0,ag,W,Z);V=V+1|0;}while((V|0)<(U|0))}if((G|0)<(c[l>>2]|0)){p=G}else{break}}rv(D,e,y,A);rv(D,f,z,A);i=b;return 0}function rv(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0.0,k=0.0,l=0.0,m=0,n=0,o=0,p=0.0,q=0.0,r=0.0,s=0.0;if((e|0)==0){e=0;do{g[b+(e<<8)>>2]=+g[d+(e<<3)>>2]+ +g[d+256+(e<<3)>>2]+ +g[d+512+(e<<3)>>2]+ +g[d+768+(e<<3)>>2]+ +g[d+1024+(e<<3)>>2]+ +g[d+1280+(e<<3)>>2];g[b+9728+(e<<8)>>2]=+g[d+(e<<3)+4>>2]+ +g[d+256+(e<<3)+4>>2]+ +g[d+512+(e<<3)+4>>2]+ +g[d+768+(e<<3)+4>>2]+ +g[d+1024+(e<<3)+4>>2]+ +g[d+1280+(e<<3)+4>>2];g[b+(e<<8)+4>>2]=+g[d+1536+(e<<3)>>2]+ +g[d+1792+(e<<3)>>2];g[b+9728+(e<<8)+4>>2]=+g[d+1536+(e<<3)+4>>2]+ +g[d+1792+(e<<3)+4>>2];g[b+(e<<8)+8>>2]=+g[d+2048+(e<<3)>>2]+ +g[d+2304+(e<<3)>>2];g[b+9728+(e<<8)+8>>2]=+g[d+2048+(e<<3)+4>>2]+ +g[d+2304+(e<<3)+4>>2];e=e+1|0;}while((e|0)<32);dq[c[a+16>>2]&127](b,d+1792|0,3,32);return}else{f=0}do{e=b+(f<<8)|0;NM(e|0,0,20);h=b+9728+(f<<8)|0;NM(h|0,0,20);i=e|0;e=h|0;h=0;j=0.0;k=0.0;do{j=+g[d+(h<<8)+(f<<3)>>2]+j;g[i>>2]=j;k=+g[d+(h<<8)+(f<<3)+4>>2]+k;g[e>>2]=k;h=h+1|0;}while((h|0)<12);h=b+(f<<8)+4|0;e=b+9728+(f<<8)+4|0;k=+g[d+3072+(f<<3)>>2]+ +g[h>>2];g[h>>2]=k;j=+g[d+3072+(f<<3)+4>>2]+ +g[e>>2];g[e>>2]=j;l=+g[d+3328+(f<<3)>>2]+k;g[h>>2]=l;k=+g[d+3328+(f<<3)+4>>2]+j;g[e>>2]=k;j=+g[d+3584+(f<<3)>>2]+l;g[h>>2]=j;l=+g[d+3584+(f<<3)+4>>2]+k;g[e>>2]=l;k=+g[d+3840+(f<<3)>>2]+j;g[h>>2]=k;j=+g[d+3840+(f<<3)+4>>2]+l;g[e>>2]=j;l=+g[d+4096+(f<<3)>>2]+k;g[h>>2]=l;k=+g[d+4096+(f<<3)+4>>2]+j;g[e>>2]=k;j=+g[d+4352+(f<<3)>>2]+l;g[h>>2]=j;l=+g[d+4352+(f<<3)+4>>2]+k;g[e>>2]=l;k=+g[d+4608+(f<<3)>>2]+j;g[h>>2]=k;j=+g[d+4608+(f<<3)+4>>2]+l;g[e>>2]=j;g[h>>2]=+g[d+4864+(f<<3)>>2]+k;g[e>>2]=+g[d+4864+(f<<3)+4>>2]+j;e=b+(f<<8)+8|0;h=b+9728+(f<<8)+8|0;i=b+(f<<8)+12|0;m=b+9728+(f<<8)+12|0;n=b+(f<<8)+16|0;o=b+9728+(f<<8)+16|0;j=+g[d+5120+(f<<3)>>2]+ +g[e>>2];g[e>>2]=j;k=+g[d+5120+(f<<3)+4>>2]+ +g[h>>2];g[h>>2]=k;l=+g[d+6144+(f<<3)>>2]+ +g[i>>2];g[i>>2]=l;p=+g[d+6144+(f<<3)+4>>2]+ +g[m>>2];g[m>>2]=p;q=+g[d+7168+(f<<3)>>2]+ +g[n>>2];g[n>>2]=q;r=+g[d+7168+(f<<3)+4>>2]+ +g[o>>2];g[o>>2]=r;s=+g[d+5376+(f<<3)>>2]+j;g[e>>2]=s;j=+g[d+5376+(f<<3)+4>>2]+k;g[h>>2]=j;k=+g[d+6400+(f<<3)>>2]+l;g[i>>2]=k;l=+g[d+6400+(f<<3)+4>>2]+p;g[m>>2]=l;p=+g[d+7424+(f<<3)>>2]+q;g[n>>2]=p;q=+g[d+7424+(f<<3)+4>>2]+r;g[o>>2]=q;r=+g[d+5632+(f<<3)>>2]+s;g[e>>2]=r;s=+g[d+5632+(f<<3)+4>>2]+j;g[h>>2]=s;j=+g[d+6656+(f<<3)>>2]+k;g[i>>2]=j;k=+g[d+6656+(f<<3)+4>>2]+l;g[m>>2]=k;l=+g[d+7680+(f<<3)>>2]+p;g[n>>2]=l;p=+g[d+7680+(f<<3)+4>>2]+q;g[o>>2]=p;g[e>>2]=+g[d+5888+(f<<3)>>2]+r;g[h>>2]=+g[d+5888+(f<<3)+4>>2]+s;g[i>>2]=+g[d+6912+(f<<3)>>2]+j;g[m>>2]=+g[d+6912+(f<<3)+4>>2]+k;g[n>>2]=+g[d+7936+(f<<3)>>2]+l;g[o>>2]=+g[d+7936+(f<<3)+4>>2]+p;f=f+1|0;}while((f|0)<32);dq[c[a+16>>2]&127](b,d+6912|0,5,32);return}function rw(){var b=0,d=0.0,e=0.0,f=0,h=0,i=0.0,j=0.0,k=0,l=0,m=0.0,n=0.0,o=0.0,p=0,q=0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0,y=0.0,z=0,A=0.0,B=0,C=0,D=0;c[499475]=2593320;c[499477]=1544;qk(1997896,9,61,174176,1,1,173928,4,4,0,0,0,4)|0;c[499479]=2579624;c[499481]=832;qk(1997912,9,61,173712,1,1,173584,2,2,0,0,0,4)|0;c[499483]=2575528;c[499485]=1024;qk(1997928,9,29,174360,1,1,174240,4,4,0,0,0,4)|0;c[499487]=2571384;c[499489]=1036;qk(1997944,9,29,173896,1,1,173776,4,4,0,0,0,4)|0;c[499491]=2569208;c[499493]=544;qk(1997960,9,15,174488,1,1,174456,2,2,0,0,0,4)|0;c[499495]=2591144;c[499497]=544;qk(1997976,9,15,174440,1,1,174408,2,2,0,0,0,4)|0;c[499499]=2589096;c[499501]=512;qk(1997992,9,8,173576,1,1,173568,1,1,0,0,0,4)|0;c[499503]=2587048;c[499505]=512;qk(1998008,9,8,173560,1,1,173552,1,1,0,0,0,4)|0;c[499507]=2585e3;c[499509]=512;qk(1998024,9,8,173528,1,1,173520,1,1,0,0,0,4)|0;c[499511]=2582952;c[499513]=512;qk(1998040,9,8,173512,1,1,173504,1,1,0,0,0,4)|0;b=0;while(1){d=+g[90304+(b<<2)>>2]*.25;e=+g[90272+(b<<2)>>2]*.25;f=b<<6;h=0;do{i=d+ +g[90304+(h<<2)>>2]*.5;j=e+ +g[90272+(h<<2)>>2]*.5;k=(h<<3)+f|0;l=0;do{m=i+ +g[90304+(l<<2)>>2];n=j+ +g[90272+(l<<2)>>2];o=1.0/+S(+(m*m+n*n));p=k+l|0;g[2199400+(p<<2)>>2]=m*o;g[2201448+(p<<2)>>2]=n*o;l=l+1|0;}while((l|0)<8);h=h+1|0;}while((h|0)<8);h=b+1|0;if((h|0)<8){b=h}else{q=0;break}}while(1){e=+g[90336+(q<<2)>>2];d=e*e;j=1.4142135381698608/+S(+(d+1.0));i=e*j;o=j-i;n=e*2.0;m=d+-1.0;d=e+1.0/e;e=d*d;b=0;do{d=+g[90600+(b<<2)>>2]*.5;r=o*d*.7071067690849304;s=d+r;g[3127952+(q<<7)+(b<<4)>>2]=i*+U(+s);t=r-d;g[3127952+(q<<7)+(b<<4)+4>>2]=j*+U(+t);g[3127952+(q<<7)+(b<<4)+8>>2]=i*+V(+s);g[3127952+(q<<7)+(b<<4)+12>>2]=j*+V(+t);if(b>>>0<5){u=+g[90520+(b<<2)>>2]}else{u=.05000000074505806}t=+_(+(n*u),+m)*.5;s=+S(+((u*u*4.0+-4.0)/e+1.0));d=+Z(+(+S(+((1.0-s)/(s+1.0)))));if(t<0.0){v=t+1.5707963267948966}else{v=t}t=+U(+v);s=+V(+v);r=+U(+d);w=+V(+d);d=t*1.4142135623730951;t=r;g[3122064+(q<<7)+(b<<4)>>2]=d*t;r=s;g[3122064+(q<<7)+(b<<4)+4>>2]=r*1.4142135623730951*t;t=w;g[3122064+(q<<7)+(b<<4)+8>>2]=r*-1.4142135623730951*t;g[3122064+(q<<7)+(b<<4)+12>>2]=d*t;b=b+1|0;}while((b|0)<8);b=q+1|0;if((b|0)<46){q=b}else{x=0;break}}while(1){if(x>>>0<10){y=+(a[90584+x|0]|0)*.125}else{y=+(x|0)+ -6.5}v=y*-1.3508848635140327;g[3119664+(x*24|0)>>2]=+U(+v);g[3119668+(x*24|0)>>2]=+V(+v);v=y*-2.356194490192345;g[3119672+(x*24|0)>>2]=+U(+v);g[3119676+(x*24|0)>>2]=+V(+v);v=y*-1.0901326597838268;g[3119680+(x*24|0)>>2]=+U(+v);g[3119684+(x*24|0)>>2]=+V(+v);v=y*-1.225221089959176;g[2198600+(x<<3)>>2]=+U(+v);g[2198604+(x<<3)>>2]=+V(+v);q=x+1|0;if((q|0)<30){x=q}else{z=0;break}}while(1){if(z>>>0<32){A=+(a[90552+z|0]|0)/24.0}else{A=+(z|0)+ -26.5}y=A*-1.3508848635140327;g[3120864+(z*24|0)>>2]=+U(+y);g[3120868+(z*24|0)>>2]=+V(+y);y=A*-2.356194490192345;g[3120872+(z*24|0)>>2]=+U(+y);g[3120876+(z*24|0)>>2]=+V(+y);y=A*-1.0901326597838268;g[3120880+(z*24|0)>>2]=+U(+y);g[3120884+(z*24|0)>>2]=+V(+y);y=A*-1.225221089959176;g[2199e3+(z<<3)>>2]=+U(+y);g[2199004+(z<<3)>>2]=+V(+y);x=z+1|0;if((x|0)<50){z=x}else{B=0;break}}while(1){A=(+(B|0)+.5)*6.283185307179586;y=A*-6.0*.125;g[2908264+(B<<6)>>2]=+U(+y)*.007460829336196184;g[2908268+(B<<6)>>2]=+V(+y)*-.007460829336196184;y=A*-5.0*.125;g[2908272+(B<<6)>>2]=+U(+y)*.02270421013236046;g[2908276+(B<<6)>>2]=+V(+y)*-.02270421013236046;y=A*-4.0*.125;g[2908280+(B<<6)>>2]=+U(+y)*.045468658208847046;g[2908284+(B<<6)>>2]=+V(+y)*-.045468658208847046;y=A*-3.0*.125;g[2908288+(B<<6)>>2]=+U(+y)*.0726611390709877;g[2908292+(B<<6)>>2]=+V(+y)*-.0726611390709877;y=A*-2.0*.125;g[2908296+(B<<6)>>2]=+U(+y)*.09885108470916748;g[2908300+(B<<6)>>2]=+V(+y)*-.09885108470916748;y=A*-1.0*.125;g[2908304+(B<<6)>>2]=+U(+y)*.1179371029138565;g[2908308+(B<<6)>>2]=+V(+y)*-.1179371029138565;y=A*0.0*.125;g[2908312+(B<<6)>>2]=+U(+y)*.125;g[2908316+(B<<6)>>2]=+V(+y)*-.125;z=B+1|0;if((z|0)<8){B=z}else{C=0;break}}while(1){y=(+(C|0)+.5)*6.283185307179586;A=y*-6.0/12.0;g[2907496+(C<<6)>>2]=+U(+A)*.040811799466609955;g[2907500+(C<<6)>>2]=+V(+A)*-.040811799466609955;A=y*-5.0/12.0;g[2907504+(C<<6)>>2]=+U(+A)*.03812811151146889;g[2907508+(C<<6)>>2]=+V(+A)*-.03812811151146889;A=y*-4.0/12.0;g[2907512+(C<<6)>>2]=+U(+A)*.05144908279180527;g[2907516+(C<<6)>>2]=+V(+A)*-.05144908279180527;A=y*-3.0/12.0;g[2907520+(C<<6)>>2]=+U(+A)*.06399831175804138;g[2907524+(C<<6)>>2]=+V(+A)*-.06399831175804138;A=y*-2.0/12.0;g[2907528+(C<<6)>>2]=+U(+A)*.07428313791751862;g[2907532+(C<<6)>>2]=+V(+A)*-.07428313791751862;A=y*-1.0/12.0;g[2907536+(C<<6)>>2]=+U(+A)*.08100347965955734;g[2907540+(C<<6)>>2]=+V(+A)*-.08100347965955734;A=y*0.0/12.0;g[2907544+(C<<6)>>2]=+U(+A)*.0833333358168602;g[2907548+(C<<6)>>2]=+V(+A)*-.0833333358168602;B=C+1|0;if((B|0)<12){C=B}else{D=0;break}}do{A=(+(D|0)+.5)*6.283185307179586;y=A*-6.0*.125;g[2906984+(D<<6)>>2]=+U(+y)*.01565675623714924;g[2906988+(D<<6)>>2]=+V(+y)*-.01565675623714924;y=A*-5.0*.125;g[2906992+(D<<6)>>2]=+U(+y)*.037527162581682205;g[2906996+(D<<6)>>2]=+V(+y)*-.037527162581682205;y=A*-4.0*.125;g[2907e3+(D<<6)>>2]=+U(+y)*.054178912192583084;g[2907004+(D<<6)>>2]=+V(+y)*-.054178912192583084;y=A*-3.0*.125;g[2907008+(D<<6)>>2]=+U(+y)*.08417043834924698;g[2907012+(D<<6)>>2]=+V(+y)*-.08417043834924698;y=A*-2.0*.125;g[2907016+(D<<6)>>2]=+U(+y)*.10307344049215317;g[2907020+(D<<6)>>2]=+V(+y)*-.10307344049215317;y=A*-1.0*.125;g[2907024+(D<<6)>>2]=+U(+y)*.12222452461719513;g[2907028+(D<<6)>>2]=+V(+y)*-.12222452461719513;y=A*0.0*.125;g[2907032+(D<<6)>>2]=+U(+y)*.125;g[2907036+(D<<6)>>2]=+V(+y)*-.125;D=D+1|0;}while((D|0)<8);g[726682]=1.0853208515459211e-17;g[726683]=.059082113206386566;g[726684]=.03444669395685196;g[726685]=.03444669395685196;g[726686]=-0.0;g[726687]=0.0;g[726688]=-.05500388517975807;g[726689]=.05500388517975807;g[726690]=1.0094949418724715e-17;g[726691]=.16486303508281708;g[726692]=.16461344063282013;g[726693]=.16461344063282013;g[726694]=.25;g[726695]=-0.0;g[726698]=-3.2559625546377634e-17;g[726699]=-.059082113206386566;g[726700]=-.03444669395685196;g[726701]=.03444669395685196;g[726702]=-0.0;g[726703]=0.0;g[726704]=.05500388517975807;g[726705]=.05500388517975807;g[726706]=-3.0284849083354756e-17;g[726707]=-.16486303508281708;g[726708]=-.16461344063282013;g[726709]=.16461344063282013;g[726710]=.25;g[726711]=-0.0;g[726714]=1.5921695514895825e-16;g[726715]=.059082113206386566;g[726716]=-.03444669395685196;g[726717]=-.03444669395685196;g[726718]=-0.0;g[726719]=0.0;g[726720]=.05500388517975807;g[726721]=-.05500388517975807;g[726722]=5.047474626644296e-17;g[726723]=.16486303508281708;g[726724]=-.16461344063282013;g[726725]=-.16461344063282013;g[726726]=.25;g[726727]=-0.0;g[726730]=2.897845958089261e-17;g[726731]=-.059082113206386566;g[726732]=.03444669395685196;g[726733]=-.03444669395685196;g[726734]=-0.0;g[726735]=0.0;g[726736]=-.05500388517975807;g[726737]=-.05500388517975807;g[726738]=-7.066464344953116e-17;g[726739]=-.16486303508281708;g[726740]=.16461344063282013;g[726741]=-.16461344063282013;g[726742]=.25;g[726743]=-0.0;return}function rx(a){a=a|0;rC(a+87444|0);return}function ry(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0;h=c[b>>2]|0;if((e|0)==10|(e|0)==5){if((f|0)<=0){return}if((g|0)==0){i=0;do{j=d+(i*34|0)|0;a[h+(i*34|0)+16|0]=0;k=d+(i*34|0)+4|0;a[h+(i*34|0)+15|0]=a[k]|0;a[h+(i*34|0)+14|0]=a[k]|0;a[h+(i*34|0)+13|0]=a[k]|0;a[h+(i*34|0)+12|0]=a[k]|0;k=d+(i*34|0)+3|0;a[h+(i*34|0)+11|0]=a[k]|0;a[h+(i*34|0)+10|0]=a[k]|0;k=d+(i*34|0)+2|0;a[h+(i*34|0)+9|0]=a[k]|0;a[h+(i*34|0)+8|0]=a[k]|0;a[h+(i*34|0)+7|0]=a[k]|0;a[h+(i*34|0)+6|0]=a[k]|0;k=d+(i*34|0)+1|0;a[h+(i*34|0)+5|0]=a[k]|0;a[h+(i*34|0)+4|0]=a[k]|0;a[h+(i*34|0)+3|0]=a[k]|0;a[h+(i*34|0)+2|0]=a[j]|0;a[h+(i*34|0)+1|0]=a[j]|0;a[h+(i*34|0)|0]=a[j]|0;i=i+1|0;}while((i|0)<(f|0));return}else{i=0;do{j=d+(i*34|0)|0;k=d+(i*34|0)+9|0;a[h+(i*34|0)+33|0]=a[k]|0;a[h+(i*34|0)+32|0]=a[k]|0;a[h+(i*34|0)+31|0]=a[k]|0;a[h+(i*34|0)+30|0]=a[k]|0;a[h+(i*34|0)+29|0]=a[k]|0;a[h+(i*34|0)+28|0]=a[k]|0;k=d+(i*34|0)+8|0;a[h+(i*34|0)+27|0]=a[k]|0;a[h+(i*34|0)+26|0]=a[k]|0;a[h+(i*34|0)+25|0]=a[k]|0;a[h+(i*34|0)+24|0]=a[k]|0;k=d+(i*34|0)+7|0;a[h+(i*34|0)+23|0]=a[k]|0;a[h+(i*34|0)+22|0]=a[k]|0;a[h+(i*34|0)+21|0]=a[k]|0;a[h+(i*34|0)+20|0]=a[k]|0;k=d+(i*34|0)+6|0;a[h+(i*34|0)+19|0]=a[k]|0;a[h+(i*34|0)+18|0]=a[k]|0;k=d+(i*34|0)+5|0;a[h+(i*34|0)+17|0]=a[k]|0;a[h+(i*34|0)+16|0]=a[k]|0;k=d+(i*34|0)+4|0;a[h+(i*34|0)+15|0]=a[k]|0;a[h+(i*34|0)+14|0]=a[k]|0;a[h+(i*34|0)+13|0]=a[k]|0;a[h+(i*34|0)+12|0]=a[k]|0;k=d+(i*34|0)+3|0;a[h+(i*34|0)+11|0]=a[k]|0;a[h+(i*34|0)+10|0]=a[k]|0;k=d+(i*34|0)+2|0;a[h+(i*34|0)+9|0]=a[k]|0;a[h+(i*34|0)+8|0]=a[k]|0;a[h+(i*34|0)+7|0]=a[k]|0;a[h+(i*34|0)+6|0]=a[k]|0;k=d+(i*34|0)+1|0;a[h+(i*34|0)+5|0]=a[k]|0;a[h+(i*34|0)+4|0]=a[k]|0;a[h+(i*34|0)+3|0]=a[k]|0;a[h+(i*34|0)+2|0]=a[j]|0;a[h+(i*34|0)+1|0]=a[j]|0;a[h+(i*34|0)|0]=a[j]|0;i=i+1|0;}while((i|0)<(f|0));return}}else if((e|0)==20|(e|0)==11){if((f|0)<=0){return}if((g|0)==0){g=0;do{e=d+(g*34|0)|0;a[h+(g*34|0)+16|0]=a[d+(g*34|0)+10|0]|0;i=d+(g*34|0)+9|0;a[h+(g*34|0)+15|0]=a[i]|0;a[h+(g*34|0)+14|0]=a[i]|0;i=d+(g*34|0)+8|0;a[h+(g*34|0)+13|0]=a[i]|0;a[h+(g*34|0)+12|0]=a[i]|0;a[h+(g*34|0)+11|0]=a[d+(g*34|0)+7|0]|0;a[h+(g*34|0)+10|0]=a[d+(g*34|0)+6|0]|0;i=d+(g*34|0)+5|0;a[h+(g*34|0)+9|0]=a[i]|0;a[h+(g*34|0)+8|0]=a[i]|0;i=d+(g*34|0)+4|0;a[h+(g*34|0)+7|0]=a[i]|0;a[h+(g*34|0)+6|0]=a[i]|0;i=d+(g*34|0)+3|0;a[h+(g*34|0)+5|0]=a[i]|0;j=d+(g*34|0)+2|0;a[h+(g*34|0)+4|0]=(((a[i]|0)+(a[j]|0)|0)/2|0)&255;a[h+(g*34|0)+3|0]=a[j]|0;j=d+(g*34|0)+1|0;a[h+(g*34|0)+2|0]=a[j]|0;a[h+(g*34|0)+1|0]=(((a[j]|0)+(a[e]|0)|0)/2|0)&255;a[h+(g*34|0)|0]=a[e]|0;g=g+1|0;}while((g|0)<(f|0));return}else{g=0;do{e=d+(g*34|0)|0;j=d+(g*34|0)+19|0;a[h+(g*34|0)+33|0]=a[j]|0;a[h+(g*34|0)+32|0]=a[j]|0;j=d+(g*34|0)+18|0;a[h+(g*34|0)+31|0]=a[j]|0;a[h+(g*34|0)+30|0]=a[j]|0;a[h+(g*34|0)+29|0]=a[j]|0;a[h+(g*34|0)+28|0]=a[j]|0;j=d+(g*34|0)+17|0;a[h+(g*34|0)+27|0]=a[j]|0;a[h+(g*34|0)+26|0]=a[j]|0;j=d+(g*34|0)+16|0;a[h+(g*34|0)+25|0]=a[j]|0;a[h+(g*34|0)+24|0]=a[j]|0;j=d+(g*34|0)+15|0;a[h+(g*34|0)+23|0]=a[j]|0;a[h+(g*34|0)+22|0]=a[j]|0;j=d+(g*34|0)+14|0;a[h+(g*34|0)+21|0]=a[j]|0;a[h+(g*34|0)+20|0]=a[j]|0;a[h+(g*34|0)+19|0]=a[d+(g*34|0)+13|0]|0;a[h+(g*34|0)+18|0]=a[d+(g*34|0)+12|0]|0;a[h+(g*34|0)+17|0]=a[d+(g*34|0)+11|0]|0;a[h+(g*34|0)+16|0]=a[d+(g*34|0)+10|0]|0;j=d+(g*34|0)+9|0;a[h+(g*34|0)+15|0]=a[j]|0;a[h+(g*34|0)+14|0]=a[j]|0;j=d+(g*34|0)+8|0;a[h+(g*34|0)+13|0]=a[j]|0;a[h+(g*34|0)+12|0]=a[j]|0;a[h+(g*34|0)+11|0]=a[d+(g*34|0)+7|0]|0;a[h+(g*34|0)+10|0]=a[d+(g*34|0)+6|0]|0;j=d+(g*34|0)+5|0;a[h+(g*34|0)+9|0]=a[j]|0;a[h+(g*34|0)+8|0]=a[j]|0;j=d+(g*34|0)+4|0;a[h+(g*34|0)+7|0]=a[j]|0;a[h+(g*34|0)+6|0]=a[j]|0;j=d+(g*34|0)+3|0;a[h+(g*34|0)+5|0]=a[j]|0;i=d+(g*34|0)+2|0;a[h+(g*34|0)+4|0]=(((a[j]|0)+(a[i]|0)|0)/2|0)&255;a[h+(g*34|0)+3|0]=a[i]|0;i=d+(g*34|0)+1|0;a[h+(g*34|0)+2|0]=a[i]|0;a[h+(g*34|0)+1|0]=(((a[i]|0)+(a[e]|0)|0)/2|0)&255;a[h+(g*34|0)|0]=a[e]|0;g=g+1|0;}while((g|0)<(f|0));return}}else{c[b>>2]=d;return}}function rz(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0;h=c[b>>2]|0;if((e|0)==34|(e|0)==17){if((f|0)<=0){return}if((g|0)==0){i=0;do{j=d+(i*34|0)+1|0;a[h+(i*34|0)|0]=(((a[d+(i*34|0)|0]<<1)+(a[j]|0)|0)/3|0)&255;a[h+(i*34|0)+1|0]=(((a[d+(i*34|0)+2|0]<<1)+(a[j]|0)|0)/3|0)&255;j=d+(i*34|0)+4|0;a[h+(i*34|0)+2|0]=(((a[d+(i*34|0)+3|0]<<1)+(a[j]|0)|0)/3|0)&255;a[h+(i*34|0)+3|0]=(((a[d+(i*34|0)+5|0]<<1)+(a[j]|0)|0)/3|0)&255;a[h+(i*34|0)+4|0]=(((a[d+(i*34|0)+7|0]|0)+(a[d+(i*34|0)+6|0]|0)|0)/2|0)&255;a[h+(i*34|0)+5|0]=(((a[d+(i*34|0)+9|0]|0)+(a[d+(i*34|0)+8|0]|0)|0)/2|0)&255;a[h+(i*34|0)+6|0]=a[d+(i*34|0)+10|0]|0;a[h+(i*34|0)+7|0]=a[d+(i*34|0)+11|0]|0;a[h+(i*34|0)+8|0]=(((a[d+(i*34|0)+13|0]|0)+(a[d+(i*34|0)+12|0]|0)|0)/2|0)&255;a[h+(i*34|0)+9|0]=(((a[d+(i*34|0)+15|0]|0)+(a[d+(i*34|0)+14|0]|0)|0)/2|0)&255;a[h+(i*34|0)+10|0]=a[d+(i*34|0)+16|0]|0;i=i+1|0;}while((i|0)<(f|0));return}else{i=0;do{j=d+(i*34|0)+1|0;a[h+(i*34|0)|0]=(((a[d+(i*34|0)|0]<<1)+(a[j]|0)|0)/3|0)&255;a[h+(i*34|0)+1|0]=(((a[d+(i*34|0)+2|0]<<1)+(a[j]|0)|0)/3|0)&255;j=d+(i*34|0)+4|0;a[h+(i*34|0)+2|0]=(((a[d+(i*34|0)+3|0]<<1)+(a[j]|0)|0)/3|0)&255;a[h+(i*34|0)+3|0]=(((a[d+(i*34|0)+5|0]<<1)+(a[j]|0)|0)/3|0)&255;a[h+(i*34|0)+4|0]=(((a[d+(i*34|0)+7|0]|0)+(a[d+(i*34|0)+6|0]|0)|0)/2|0)&255;a[h+(i*34|0)+5|0]=(((a[d+(i*34|0)+9|0]|0)+(a[d+(i*34|0)+8|0]|0)|0)/2|0)&255;a[h+(i*34|0)+6|0]=a[d+(i*34|0)+10|0]|0;a[h+(i*34|0)+7|0]=a[d+(i*34|0)+11|0]|0;a[h+(i*34|0)+8|0]=(((a[d+(i*34|0)+13|0]|0)+(a[d+(i*34|0)+12|0]|0)|0)/2|0)&255;a[h+(i*34|0)+9|0]=(((a[d+(i*34|0)+15|0]|0)+(a[d+(i*34|0)+14|0]|0)|0)/2|0)&255;a[h+(i*34|0)+10|0]=a[d+(i*34|0)+16|0]|0;a[h+(i*34|0)+11|0]=a[d+(i*34|0)+17|0]|0;a[h+(i*34|0)+12|0]=a[d+(i*34|0)+18|0]|0;a[h+(i*34|0)+13|0]=a[d+(i*34|0)+19|0]|0;a[h+(i*34|0)+14|0]=(((a[d+(i*34|0)+21|0]|0)+(a[d+(i*34|0)+20|0]|0)|0)/2|0)&255;a[h+(i*34|0)+15|0]=(((a[d+(i*34|0)+23|0]|0)+(a[d+(i*34|0)+22|0]|0)|0)/2|0)&255;a[h+(i*34|0)+16|0]=(((a[d+(i*34|0)+25|0]|0)+(a[d+(i*34|0)+24|0]|0)|0)/2|0)&255;a[h+(i*34|0)+17|0]=(((a[d+(i*34|0)+27|0]|0)+(a[d+(i*34|0)+26|0]|0)|0)/2|0)&255;a[h+(i*34|0)+18|0]=(((a[d+(i*34|0)+29|0]|0)+(a[d+(i*34|0)+28|0]|0)+(a[d+(i*34|0)+30|0]|0)+(a[d+(i*34|0)+31|0]|0)|0)/4|0)&255;a[h+(i*34|0)+19|0]=(((a[d+(i*34|0)+33|0]|0)+(a[d+(i*34|0)+32|0]|0)|0)/2|0)&255;i=i+1|0;}while((i|0)<(f|0));return}}else if((e|0)==10|(e|0)==5){if((f|0)<=0){return}if((g|0)==0){g=0;do{a[h+(g*34|0)+10|0]=0;e=4;while(1){i=a[d+(g*34|0)+e|0]|0;j=e<<1;a[h+(g*34|0)+j|0]=i;a[(j|1)+(h+(g*34|0))|0]=i;if((e|0)>0){e=e-1|0}else{break}}g=g+1|0;}while((g|0)<(f|0));return}else{g=0;do{e=9;while(1){i=a[d+(g*34|0)+e|0]|0;j=e<<1;a[h+(g*34|0)+j|0]=i;a[(j|1)+(h+(g*34|0))|0]=i;if((e|0)>0){e=e-1|0}else{break}}g=g+1|0;}while((g|0)<(f|0));return}}else{c[b>>2]=d;return}}function rA(a){a=a|0;var b=0,c=0.0,d=0,e=0,f=0,h=0,i=0,j=0;b=a+4|0;c=+g[b>>2];g[a>>2]=(+g[a>>2]*2.0+c)*.3333333432674408;d=a+8|0;g[b>>2]=(c+ +g[d>>2]*2.0)*.3333333432674408;b=a+12|0;e=a+16|0;c=+g[e>>2];g[d>>2]=(+g[b>>2]*2.0+c)*.3333333432674408;d=a+20|0;g[b>>2]=(c+ +g[d>>2]*2.0)*.3333333432674408;b=a+24|0;f=a+28|0;g[e>>2]=(+g[b>>2]+ +g[f>>2])*.5;e=a+32|0;h=a+36|0;g[d>>2]=(+g[e>>2]+ +g[h>>2])*.5;d=a+40|0;g[b>>2]=+g[d>>2];b=a+44|0;g[f>>2]=+g[b>>2];f=a+48|0;i=a+52|0;g[e>>2]=(+g[f>>2]+ +g[i>>2])*.5;e=a+56|0;j=a+60|0;g[h>>2]=(+g[e>>2]+ +g[j>>2])*.5;h=a+64|0;g[d>>2]=+g[h>>2];d=a+68|0;g[b>>2]=+g[d>>2];b=a+72|0;g[f>>2]=+g[b>>2];f=a+76|0;g[i>>2]=+g[f>>2];g[e>>2]=(+g[a+80>>2]+ +g[a+84>>2])*.5;g[j>>2]=(+g[a+88>>2]+ +g[a+92>>2])*.5;g[h>>2]=(+g[a+96>>2]+ +g[a+100>>2])*.5;g[d>>2]=(+g[a+104>>2]+ +g[a+108>>2])*.5;g[b>>2]=(+g[a+112>>2]+ +g[a+116>>2]+ +g[a+120>>2]+ +g[a+124>>2])*.25;g[f>>2]=(+g[a+128>>2]+ +g[a+132>>2])*.5;return}function rB(e,f,g,h,i,j){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;k=c[f+16>>2]|0;l=c[1997900+(h<<4)>>2]|0;if((j|0)==0){if((k|0)<=0){return}j=e+8|0;h=e+16|0;m=e|0;n=0;o=0;do{p=c[j>>2]|0;q=c[h>>2]|0;r=c[m>>2]|0;s=r+(p>>>3)|0;t=(cV(d[s]|d[s+1|0]<<8|d[s+2|0]<<16|d[s+3|0]<<24|0)|0)<<(p&7)>>>23;s=b[l+(t<<2)>>1]|0;u=b[l+(t<<2)+2>>1]|0;t=u<<16>>16;do{if(u<<16>>16<0){v=p+9|0;w=q>>>0>v>>>0?v:q;v=r+(w>>>3)|0;x=((cV(d[v]|d[v+1|0]<<8|d[v+2|0]<<16|d[v+3|0]<<24|0)|0)<<(w&7)>>>((t+32|0)>>>0))+s|0;v=b[l+(x<<2)>>1]|0;y=b[l+(x<<2)+2>>1]|0;x=y<<16>>16;if(y<<16>>16>=0){z=v;A=w;B=x;break}y=w-t|0;w=q>>>0>y>>>0?y:q;y=r+(w>>>3)|0;C=((cV(d[y]|d[y+1|0]<<8|d[y+2|0]<<16|d[y+3|0]<<24|0)|0)<<(w&7)>>>((x+32|0)>>>0))+v|0;z=b[l+(C<<2)>>1]|0;A=w;B=b[l+(C<<2)+2>>1]|0}else{z=s;A=p;B=t}}while(0);t=A+B|0;c[j>>2]=q>>>0>t>>>0?t:q;o=z+o&7;a[g+(i*34|0)+n|0]=o&255;n=n+1|0;}while((n|0)<(k|0));return}if((i|0)==0){D=c[f+40>>2]|0}else{D=i}f=D-1|0;D=(f|0)>0?f:0;if((k|0)<=0){return}f=e+8|0;n=e+16|0;o=e|0;e=0;do{z=d[g+(D*34|0)+e|0]|0;j=c[f>>2]|0;B=c[n>>2]|0;A=c[o>>2]|0;m=A+(j>>>3)|0;h=(cV(d[m]|d[m+1|0]<<8|d[m+2|0]<<16|d[m+3|0]<<24|0)|0)<<(j&7)>>>23;m=b[l+(h<<2)>>1]|0;t=b[l+(h<<2)+2>>1]|0;h=t<<16>>16;do{if(t<<16>>16<0){p=j+9|0;s=B>>>0>p>>>0?p:B;p=A+(s>>>3)|0;r=((cV(d[p]|d[p+1|0]<<8|d[p+2|0]<<16|d[p+3|0]<<24|0)|0)<<(s&7)>>>((h+32|0)>>>0))+m|0;p=b[l+(r<<2)>>1]|0;u=b[l+(r<<2)+2>>1]|0;r=u<<16>>16;if(u<<16>>16>=0){E=p;F=s;G=r;break}u=s-h|0;s=B>>>0>u>>>0?u:B;u=A+(s>>>3)|0;C=((cV(d[u]|d[u+1|0]<<8|d[u+2|0]<<16|d[u+3|0]<<24|0)|0)<<(s&7)>>>((r+32|0)>>>0))+p|0;E=b[l+(C<<2)>>1]|0;F=s;G=b[l+(C<<2)+2>>1]|0}else{E=m;F=j;G=h}}while(0);h=F+G|0;c[f>>2]=B>>>0>h>>>0?h:B;a[g+(i*34|0)+e|0]=E+z&7;e=e+1|0;}while((e|0)<(k|0));return}function rC(a){a=a|0;c[a>>2]=78;c[a+4>>2]=28;c[a+8>>2]=4;c[a+12>>2]=12;c[a+16>>2]=60;c[a+20>>2]=2;c[a+24>>2]=10;c[a+28>>2]=32;return}function rD(a,b,c){a=a|0;b=b|0;c=c|0;var d=0,e=0.0,f=0.0,h=0;if((c|0)>0){d=0}else{return}do{e=+g[b+(d<<3)>>2];f=+g[b+(d<<3)+4>>2];h=a+(d<<2)|0;g[h>>2]=+g[h>>2]+(e*e+f*f);d=d+1|0;}while((d|0)<(c|0));return}function rE(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0;if((d|0)>0){e=0}else{return}do{f=c+(e<<2)|0;g[a+(e<<3)>>2]=+g[b+(e<<3)>>2]*+g[f>>2];g[a+(e<<3)+4>>2]=+g[b+(e<<3)+4>>2]*+g[f>>2];e=e+1|0;}while((e|0)<(d|0));return}function rF(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0.0,k=0.0,l=0.0,m=0,n=0.0,o=0,p=0.0,q=0.0,r=0.0,s=0.0;if((e|0)<=0){return}f=b+48|0;h=b+52|0;i=0;do{j=+g[c+(i<<6)+48>>2];k=j*+g[h>>2];l=j*+g[f>>2];m=0;do{j=+g[b+(m<<3)>>2];n=+g[b+(m<<3)+4>>2];o=12-m|0;p=+g[b+(o<<3)>>2];q=+g[b+(o<<3)+4>>2];r=+g[c+(i<<6)+(m<<3)>>2];s=+g[c+(i<<6)+(m<<3)+4>>2];l=l+((j+p)*r-(n-q)*s);k=k+(r*(n+q)+(j-p)*s);m=m+1|0;}while((m|0)<6);m=ac(i,d)|0;g[a+(m<<3)>>2]=l;g[a+(m<<3)+4>>2]=k;i=i+1|0;}while((i|0)<(e|0));return}function rG(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;if((c|0)<64&(d|0)>0){e=c}else{return}do{c=0;do{g[a+(e<<8)+(c<<3)>>2]=+g[b+(c<<8)+(e<<2)>>2];g[a+(e<<8)+(c<<3)+4>>2]=+g[b+9728+(c<<8)+(e<<2)>>2];c=c+1|0;}while((c|0)<(d|0));e=e+1|0;}while((e|0)<64);return}function rH(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;if((c|0)<64&(d|0)>0){e=c}else{return}do{c=0;do{g[a+(c<<8)+(e<<2)>>2]=+g[b+(e<<8)+(c<<3)>>2];g[a+9728+(c<<8)+(e<<2)>>2]=+g[b+(e<<8)+(c<<3)+4>>2];c=c+1|0;}while((c|0)<(d|0));e=e+1|0;}while((e|0)<64);return}function rI(a,b,c,d,e,f,h,i){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;h=+h;i=i|0;var j=0.0,k=0.0,l=0.0,m=0,n=0,o=0.0,p=0.0,q=0.0,r=0.0,s=0.0,t=0,u=0,v=0.0,w=0.0;j=h*.6514390707015991;k=h*.5647181272506714;l=h*.4895416498184204;if((i|0)<=0){return}m=d+4|0;n=0;do{h=+g[b+(n<<3)>>2];o=+g[d>>2];p=+g[b+(n<<3)+4>>2];q=+g[m>>2];r=h*o-p*q;s=o*p+h*q;t=n+2|0;u=n+5|0;q=+g[c+(t<<3)>>2];h=+g[c+(t<<3)+4>>2];p=+g[e>>2];o=+g[e+4>>2];v=q*p-h*o-r*j;w=h*p+q*o-s*j;g[c+(u<<3)>>2]=r+j*v;g[c+(u<<3)+4>>2]=s+j*w;t=n+1|0;s=+g[c+296+(t<<3)>>2];r=+g[c+296+(t<<3)+4>>2];o=+g[e+8>>2];q=+g[e+12>>2];p=s*o-r*q-v*k;h=r*o+s*q-w*k;g[c+296+(u<<3)>>2]=v+k*p;g[c+296+(u<<3)+4>>2]=w+k*h;w=+g[c+592+(n<<3)>>2];v=+g[c+592+(n<<3)+4>>2];q=+g[e+16>>2];s=+g[e+20>>2];o=w*q-v*s-p*l;r=v*q+w*s-h*l;g[c+592+(u<<3)>>2]=p+l*o;g[c+592+(u<<3)+4>>2]=h+l*r;u=f+(n<<2)|0;g[a+(n<<3)>>2]=o*+g[u>>2];g[a+(n<<3)+4>>2]=r*+g[u>>2];n=n+1|0;}while((n|0)<(i|0));return}function rJ(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0.0,h=0.0,i=0.0,j=0.0,k=0.0,l=0.0,m=0.0,n=0.0,o=0.0,p=0,q=0.0,r=0,s=0.0,t=0,u=0.0;f=+g[d>>2];h=+g[d+4>>2];i=+g[d+8>>2];j=+g[d+12>>2];if((e|0)<=0){return}d=0;k=+g[c+12>>2];l=+g[c+8>>2];m=+g[c+4>>2];n=+g[c>>2];do{c=a+(d<<3)|0;o=+g[c>>2];p=a+(d<<3)+4|0;q=+g[p>>2];r=b+(d<<3)|0;s=+g[r>>2];t=b+(d<<3)+4|0;u=+g[t>>2];n=f+n;m=h+m;l=i+l;k=j+k;g[c>>2]=n*o+l*s;g[p>>2]=n*q+l*u;g[r>>2]=m*o+k*s;g[t>>2]=m*q+k*u;d=d+1|0;}while((d|0)<(e|0));return}function rK(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0.0,h=0.0,i=0.0,j=0.0,k=0.0,l=0.0,m=0.0,n=0.0,o=0.0,p=0.0,q=0.0,r=0.0,s=0.0,t=0.0,u=0.0,v=0.0,w=0.0,x=0,y=0.0,z=0,A=0.0,B=0,C=0.0;f=+g[d>>2];h=+g[d+16>>2];i=+g[d+4>>2];j=+g[d+20>>2];k=+g[d+8>>2];l=+g[d+24>>2];m=+g[d+12>>2];n=+g[d+28>>2];if((e|0)<=0){return}o=+g[c+28>>2];p=+g[c+12>>2];q=+g[c+24>>2];r=+g[c+8>>2];d=0;s=+g[c+20>>2];t=+g[c+4>>2];u=+g[c+16>>2];v=+g[c>>2];do{c=a+(d<<3)|0;w=+g[c>>2];x=a+(d<<3)+4|0;y=+g[x>>2];z=b+(d<<3)|0;A=+g[z>>2];B=b+(d<<3)+4|0;C=+g[B>>2];v=f+v;t=i+t;r=k+r;p=m+p;u=h+u;s=j+s;q=l+q;o=n+o;g[c>>2]=v*w+r*A-u*y-q*C;g[x>>2]=q*A+(u*w+(v*y+r*C));g[z>>2]=t*w+p*A-s*y-o*C;g[B>>2]=o*A+(s*w+(t*y+p*C));d=d+1|0;}while((d|0)<(e|0));return}function rL(){return 4454}function rM(){return 1646728}function rN(){return 346760}function rO(){var a=0,b=0;a=A7(16272)|0;b=a;if((a|0)==0){return b|0}c[a>>2]=346760;Br(a);return b|0}function rP(a,b,d,e,f,g,h,i,j,k,l){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0;do{if((a|0)==0){m=A7(16272)|0;if((m|0)==0){n=0;return n|0}else{c[m>>2]=346760;Br(m);o=m;break}}else{o=a}}while(0);c[o+4>>2]=k;c[o+8>>2]=l;l=o;Bh(l,1692912,b,d,0)|0;Bh(l,1672024,e,(e|0)<0|0?-1:0,0)|0;Bh(l,1657384,f,(f|0)<0|0?-1:0,0)|0;Bh(l,1638384,g,h,0)|0;Bh(l,1629912,i,(i|0)<0|0?-1:0,0)|0;Bh(l,1623680,j,(j|0)<0|0?-1:0,0)|0;Bh(l,1617904,-1,-1,0)|0;j=o+24|0;i=zs(c[j>>2]|0,c[j+4>>2]|0)|0;Bh(l,1613128,i,(i|0)<0|0?-1:0,0)|0;i=o+32|0;j=zs(c[i>>2]|0,c[i+4>>2]|0)|0;Bh(l,1740176,j,(j|0)<0|0?-1:0,0)|0;Bh(l,1735656,0,0,0)|0;n=o;return n|0}function rQ(a){a=a|0;var b=0,d=0,e=0;b=c[a>>2]|0;if((b|0)==0){d=a;A6(d);return}A4(c[b+5988>>2]|0);NM(b+5860|0,0,152);A4(c[b+6140>>2]|0);NM(b+6012|0,0,152);A4(c[b+6292>>2]|0);NM(b+6164|0,0,152);A4(c[b+6596>>2]|0);NM(b+6468|0,0,152);A4(c[b+6748>>2]|0);NM(b+6620|0,0,152);A4(c[b+6900>>2]|0);NM(b+6772|0,0,152);A4(c[b+5444>>2]|0);NM(b+5316|0,0,152);A4(c[b+5596>>2]|0);NM(b+5468|0,0,152);r3(b+6964|0);r3(b+6968|0);r3(b+6972|0);e=c[b+6980>>2]|0;if((e|0)!=0){c_[c[e+4>>2]&127](b+6976|0)}sJ(b);d=a;A6(d);return}function rR(a){a=a|0;var b=0,d=0,e=0,f=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0.0,R=0.0,S=0.0,T=0,U=0,V=0,W=0;b=i;i=i+2048|0;d=b|0;e=b+1024|0;c[a+6924>>2]=0;c[a+6928>>2]=0;c[a+6932>>2]=0;A4(c[a+5988>>2]|0);f=a+5860|0;NM(f|0,0,152);A4(c[a+6140>>2]|0);j=a+6012|0;NM(j|0,0,152);A4(c[a+6292>>2]|0);k=a+6164|0;NM(k|0,0,152);A4(c[a+6596>>2]|0);l=a+6468|0;NM(l|0,0,152);A4(c[a+6748>>2]|0);m=a+6620|0;NM(m|0,0,152);A4(c[a+6900>>2]|0);n=a+6772|0;NM(n|0,0,152);o=a+84|0;A4(c[a+5444>>2]|0);NM(a+5316|0,0,152);A4(c[a+5596>>2]|0);NM(a+5468|0,0,152);p=a+5708|0;NM(p|0,0,128);q=a+6316|0;NM(q|0,0,128);r=a+6964|0;r3(r);s=a+6968|0;r3(s);t=a+6972|0;r3(t);sJ(a);c[a+6936>>2]=0;u=a+12|0;v=c[u>>2]|0;if((v|0)>9){AN(a,16,1730744,(w=i,i=i+8|0,c[w>>2]=v,w)|0);i=w;x=-22;i=b;return x|0}v=a+20|0;y=c[v>>2]|0;if((y|0)>9){AN(a,16,1726384,(w=i,i=i+8|0,c[w>>2]=y,w)|0);i=w;x=-22;i=b;return x|0}y=a+24|0;if((zs(c[y>>2]|0,c[y+4>>2]|0)|0)>32){z=c[y+4>>2]|0;AN(a,24,1722072,(w=i,i=i+16|0,c[w>>2]=c[y>>2],c[w+8>>2]=z,w)|0);i=w;c[y>>2]=0;c[y+4>>2]=0}z=a+32|0;if((zs(c[z>>2]|0,c[z+4>>2]|0)|0)>32){A=c[z+4>>2]|0;AN(a,24,1718056,(w=i,i=i+16|0,c[w>>2]=c[z>>2],c[w+8>>2]=A,w)|0);i=w;c[z>>2]=0;c[z+4>>2]=0}A=a+80|0;if((c[A>>2]|0)!=0){AN(a,16,1714056,(w=i,i=i+1|0,i=i+7&-8,c[w>>2]=0,w)|0);i=w;x=-22;i=b;return x|0}B=a+6980|0;c[B>>2]=70816;C=a+76|0;D=c[C>>2]|0;if((D|0)==0){E=c[a+5840>>2]|0;c[C>>2]=E;if((E|0)==0){F=3399}else{H=E;F=3396}}else{H=D;F=3396}do{if((F|0)==3396){D=c[y>>2]|0;E=c[y+4>>2]|0;if((D|0)==0&(E|0)==0){F=3400;break}if((H|0)==(zs(D,E)|0)){F=3399;break}AN(a,24,1711432,(w=i,i=i+1|0,i=i+7&-8,c[w>>2]=0,w)|0);i=w;c[y>>2]=0;c[y+4>>2]=0;F=3400}}while(0);if((F|0)==3399){H=c[y>>2]|0;E=c[y+4>>2]|0;if((H|0)==0&(E|0)==0){F=3400}else{I=E;J=H}}if((F|0)==3400){H=zu(c[C>>2]|0)|0;E=G;c[y>>2]=H;c[y+4>>2]=E;I=E;J=H}H=c[z>>2]|0;E=c[z+4>>2]|0;if((H|0)==0&(E|0)==0){D=zu(c[a+6448>>2]|0)|0;K=G;c[z>>2]=D;c[z+4>>2]=K;L=K;M=D;N=c[y+4>>2]|0;O=c[y>>2]|0}else{L=E;M=H;N=I;O=J}do{if((M|0)==(O|0)&(L|0)==(N|0)){if(+g[a+64>>2]!=1.0){P=1;break}P=(c[a+5704>>2]|0)!=0|0}else{P=1}}while(0);N=a+5700|0;c[N>>2]=P;P=a+16|0;L=c[P>>2]|0;L4724:do{if((L|0)==-1){if((B5(c[u>>2]|0)|0)<7){c[P>>2]=6;break}do{if((B5(c[u>>2]|0)|0)==7){if((B5(c[v>>2]|0)|0)!=7){break}if((c[N>>2]|0)!=0){break}if((c[A>>2]|0)==1){break}c[P>>2]=7;break L4724}}while(0);if((B5(c[u>>2]|0)|0)<9){c[P>>2]=8;break}else{AN(a,48,1708056,(w=i,i=i+1|0,i=i+7&-8,c[w>>2]=0,w)|0);i=w;c[P>>2]=9;break}}else{if((L-6|0)>>>0<4){break}O=B2(L)|0;AN(a,16,1705896,(w=i,i=i+8|0,c[w>>2]=O,w)|0);i=w;x=-22;i=b;return x|0}}while(0);L=c[u>>2]|0;c[a+5856>>2]=L;c[a+5844>>2]=B7(L)|0;c[a+5852>>2]=B8(L)|0;L=c[v>>2]|0;c[a+6464>>2]=L;c[a+6452>>2]=B7(L)|0;c[a+6460>>2]=B8(L)|0;L=a+5688|0;A=c[L>>2]|0;O=c[L+4>>2]|0;if((A|0)==0&(O|0)==(-2147483648|0)){L=a+6952|0;c[L>>2]=0;c[L+4>>2]=-2147483648;Q=+g[a+5684>>2]}else{L=a+5684|0;R=+g[L>>2];do{if(R!=0.0){S=R}else{if(+g[a+5668>>2]<1.7014117331926443e+38){S=R;break}g[L>>2]=1.0;S=1.0}}while(0);L=c[a+44>>2]|0;M=N2(L,(L|0)<0|0?-1:0,A,O)|0;O=G;A=a+6944|0;c[A>>2]=M;c[A+4>>2]=O;A=a+6952|0;c[A>>2]=M;c[A+4>>2]=O;Q=S}O=a+5684|0;do{if(Q!=0.0){A=a+5668|0;if(+g[A>>2]>=1.7014117331926443e+38){g[A>>2]=.0010000000474974513}S=+g[O>>2];if(S<=1.0001){break}g[a+5680>>2]=S/+(c[a+40>>2]|0)}}while(0);O=a+44|0;A=c[O>>2]|0;M=a+40|0;L=c[M>>2]|0;do{if((A|0)==(L|0)){if((c[a+48>>2]&1|0)!=0){F=3432;break}c_[c[(c[B>>2]|0)+4>>2]&127](a+6976|0)}else{F=3432}}while(0);if((F|0)==3432){J=a+6976|0;c[J>>2]=di[c[c[B>>2]>>2]&3](c[J>>2]|0,A,L,c[a+5624>>2]|0,c[a+5628>>2]|0,c[a+5632>>2]|0,+h[a+5640>>3],c[P>>2]|0,c[a+5648>>2]|0,c[a+5652>>2]|0,+h[a+5656>>3],c[a+5664>>2]|0)|0}do{if(((c[P>>2]|0)-6|0)>>>0>=4){if((c[a+6976>>2]|0)==0){break}AN(a,16,1704080,(w=i,i=i+1|0,i=i+7&-8,c[w>>2]=0,w)|0);i=w;x=-1;i=b;return x|0}}while(0);L=a+5840|0;A=c[L>>2]|0;if((A|0)==0){J=zs(c[y>>2]|0,c[y+4>>2]|0)|0;c[L>>2]=J;T=J}else{T=A}if((c[C>>2]|0)==0){c[C>>2]=T}A=a+6448|0;J=c[A>>2]|0;if((J|0)==0){B=zs(c[z>>2]|0,c[z+4>>2]|0)|0;c[A>>2]=B;U=c[L>>2]|0;V=B}else{U=T;V=J}if((U|0)==0){if(!((c[y>>2]|0)==0&(c[y+4>>2]|0)==0)){AN(0,0,1701712,(w=i,i=i+24|0,c[w>>2]=1698608,c[w+8>>2]=1696544,c[w+16>>2]=375,w)|0);i=w;cw();return 0}AN(a,16,1694200,(w=i,i=i+1|0,i=i+7&-8,c[w>>2]=0,w)|0);i=w;x=-1;i=b;return x|0}do{if((c[z>>2]|0)==0&(c[z+4>>2]|0)==0){F=3450}else{if((c[y>>2]|0)==0&(c[y+4>>2]|0)==0){F=3450;break}W=c[C>>2]|0}}while(0);do{if((F|0)==3450){J=c[C>>2]|0;if((J|0)==(V|0)){W=V;break}if((c[a+5704>>2]|0)!=0){W=J;break}J=d|0;zt(J,1024,U,c[y>>2]|0,c[y+4>>2]|0);T=e|0;zt(T,1024,c[A>>2]|0,c[z>>2]|0,c[z+4>>2]|0);AN(a,16,1690600,(w=i,i=i+16|0,c[w>>2]=J,c[w+8>>2]=T,w)|0);i=w;x=-1;i=b;return x|0}}while(0);if((W|0)==0){AN(0,0,1701712,(w=i,i=i+24|0,c[w>>2]=1688648,c[w+8>>2]=1696544,c[w+16>>2]=389,w)|0);i=w;cw();return 0}if((V|0)==0){AN(0,0,1701712,(w=i,i=i+24|0,c[w>>2]=1685944,c[w+8>>2]=1696544,c[w+16>>2]=390,w)|0);i=w;cw();return 0}w=a+5696|0;c[w>>2]=+(((V|0)/(U|0)|0)-1|0)<+(c[O>>2]|0)/+(c[M>>2]|0)+-1.0;NN(l|0,p|0,152)|0;NN(m|0,p|0,152)|0;NN(n|0,q|0,152)|0;n=a+6976|0;do{if((c[n>>2]|0)==0){if((c[N>>2]|0)!=0){break}if((c[a+72>>2]|0)!=0){break}if((c[o>>2]|0)!=0){break}c[t>>2]=r_(c[v>>2]|0,c[u>>2]|0,U,0,0)|0;x=0;i=b;return x|0}}while(0);U=a+72|0;c[r>>2]=r_(c[P>>2]|0,c[u>>2]|0,W,c[U>>2]|0,0)|0;W=r_(c[v>>2]|0,c[P>>2]|0,c[A>>2]|0,0,0)|0;c[s>>2]=W;if((c[r>>2]|0)==0|(W|0)==0){x=-12;i=b;return x|0}NN(f|0,p|0,152)|0;NN(k|0,q|0,152)|0;NN(j|0,p|0,152)|0;do{if((c[U>>2]|0)!=0){p=c[C>>2]|0;c[a+6144>>2]=p;c[a+5992>>2]=p;if((c[n>>2]|0)==0){break}c[a+6600>>2]=p}}while(0);do{if((c[w>>2]|0)==0){C=c[A>>2]|0;c[a+6144>>2]=C;if((c[n>>2]|0)==0){break}c[a+6600>>2]=C}}while(0);A=c[P>>2]|0;c[a+6008>>2]=A;c[a+5996>>2]=B7(A)|0;c[a+6004>>2]=B8(A)|0;A=c[P>>2]|0;c[a+6160>>2]=A;c[a+6148>>2]=B7(A)|0;c[a+6156>>2]=B8(A)|0;A=c[P>>2]|0;c[a+6312>>2]=A;c[a+6300>>2]=B7(A)|0;c[a+6308>>2]=B8(A)|0;if((c[n>>2]|0)!=0){n=c[P>>2]|0;c[a+6616>>2]=n;c[a+6604>>2]=B7(n)|0;c[a+6612>>2]=B8(n)|0}n=sv(a,c[v>>2]|0,c[P>>2]|0)|0;if((n|0)<0){x=n;i=b;return x|0}do{if((c[N>>2]|0)==0){if((c[o>>2]|0)==0){x=0}else{break}i=b;return x|0}}while(0);x=sA(a)|0;i=b;return x|0}function rS(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;d=i;i=i+152|0;e=d|0;if((b|0)<0){f=-22;i=d;return f|0}g=a+136|0;h=c[g>>2]|0;j=a+132|0;k=c[j>>2]|0;if(((1073741823/(h|0)|0|0)/(k|0)|0|0)<(b|0)){f=-22;i=d;return f|0}l=a+140|0;if((c[l>>2]|0)>=(b|0)){f=0;i=d;return f|0}m=b<<1;b=(ac(h,m)|0)+31&-32;n=e;o=a;NN(n|0,o|0,152)|0;if((h|0)==0){AN(0,0,1701712,(p=i,i=i+24|0,c[p>>2]=1684e3,c[p+8>>2]=1696544,c[p+16>>2]=459,p)|0);i=p;cw();return 0}if((k|0)==0){AN(0,0,1701712,(p=i,i=i+24|0,c[p>>2]=1682256,c[p+8>>2]=1696544,c[p+16>>2]=460,p)|0);i=p;cw();return 0}p=A7(ac(k,b)|0)|0;k=a+128|0;c[k>>2]=p;if((p|0)==0){f=-12;i=d;return f|0}h=c[j>>2]|0;o=a+144|0;L4847:do{if((h|0)>0){n=0;q=p;while(1){if((c[o>>2]|0)==0){r=c[g>>2]|0}else{r=b}s=q+(ac(r,n)|0)|0;c[a+(n<<2)>>2]=s;if((c[o>>2]|0)!=0){t=c[e+(n<<2)>>2]|0;u=ac(c[g>>2]|0,c[l>>2]|0)|0;NN(s|0,t|0,u)|0}u=n+1|0;t=c[j>>2]|0;if((u|0)>=(t|0)){v=t;break L4847}n=u;q=c[k>>2]|0}}else{v=h}}while(0);if((c[o>>2]|0)==0){o=c[a>>2]|0;a=c[e>>2]|0;h=ac(c[l>>2]|0,v)|0;v=ac(h,c[g>>2]|0)|0;NN(o|0,a|0,v)|0}A4(c[e+128>>2]|0);c[l>>2]=m;f=1;i=d;return f|0}function rT(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0;g=i;i=i+128|0;h=g|0;j=h;k=i;i=i+152|0;l=a+5708|0;m=a+6316|0;n=a+6960|0;o=c[n>>2]|0;L4863:do{if((o|0)>0){p=a+6772|0;q=h|0;r=a+6916|0;s=p|0;t=a+6904|0;u=a+6772|0;if((b|0)==0){v=f;w=o;while(1){x=rS(p,(w|0)>16384?16384:w)|0;if((x|0)<0){y=x;z=3608;break}do{if((c[r>>2]|0)==0){c[q>>2]=c[s>>2]}else{x=c[t>>2]|0;if((x|0)<=0){break}A=(x|0)>1?x<<2:4;NN(j|0,u|0,A)|0}}while(0);A=c[n>>2]|0;c[n>>2]=-A;x=rT(a,q,(A|0)>16384?16384:A,e,v)|0;A=-(c[n>>2]|0)|0;c[n>>2]=A;if((x|0)<=0){y=0;z=3614;break}B=A-x|0;c[n>>2]=B;if((B|0)>0){v=0;w=B}else{C=0;break L4863}}if((z|0)==3608){i=g;return y|0}else if((z|0)==3614){i=g;return y|0}}else{D=f;E=o}while(1){w=rS(p,(E|0)>16384?16384:E)|0;if((w|0)<0){y=w;z=3609;break}do{if((c[r>>2]|0)==0){c[q>>2]=c[s>>2]}else{w=c[t>>2]|0;if((w|0)<=0){break}v=(w|0)>1?w<<2:4;NN(j|0,u|0,v)|0}}while(0);v=c[n>>2]|0;c[n>>2]=-v;w=rT(a,q,(v|0)>16384?16384:v,e,D)|0;v=c[n>>2]|0;B=-v|0;c[n>>2]=B;if((w|0)>0){x=B-w|0;c[n>>2]=x;F=x}else{if((v|0)==0){F=B}else{y=0;z=3607;break}}if((F|0)>0){D=0;E=F}else{C=0;break L4863}}if((z|0)==3609){i=g;return y|0}else if((z|0)==3607){i=g;return y|0}}else{C=f}}while(0);L4894:do{if((e|0)==0){if((c[a+6976>>2]|0)==0){if((c[a+6928>>2]|0)==0){y=0}else{break}i=g;return y|0}f=a+6936|0;if((c[f>>2]|0)==0){z=c[(c[a+6980>>2]|0)+12>>2]|0;c1[z&255](a)|0}c[a+6932>>2]=0;c[f>>2]=1}else{f=a+5840|0;z=(c[f>>2]|0)>0;if((c[a+5852>>2]|0)!=0){if(z){H=0}else{break}while(1){c[a+5708+(H<<2)>>2]=c[e+(H<<2)>>2];H=H+1|0;if((H|0)>=(c[f>>2]|0)){break L4894}}}if(!z){break}F=a+5844|0;E=0;do{D=c[e>>2]|0;c[a+5708+(E<<2)>>2]=D+(ac(c[F>>2]|0,E)|0);E=E+1|0;}while((E|0)<(c[f>>2]|0))}}while(0);L4913:do{if((b|0)==0){NM(m|0,0,128)}else{e=a+6448|0;H=(c[e>>2]|0)>0;if((c[a+6460>>2]|0)!=0){if(H){I=0}else{break}while(1){c[a+6316+(I<<2)>>2]=c[b+(I<<2)>>2];I=I+1|0;if((I|0)>=(c[e>>2]|0)){break L4913}}}if(!H){break}f=a+6452|0;E=0;do{F=c[b>>2]|0;c[a+6316+(E<<2)>>2]=F+(ac(c[f>>2]|0,E)|0);E=E+1|0;}while((E|0)<(c[e>>2]|0))}}while(0);if((c[a+6976>>2]|0)!=0){b=rU(a,m,d,l,C)|0;if((b|0)<=0){y=b;i=g;return y|0}if((c[n>>2]|0)!=0){y=b;i=g;return y|0}I=c[a+40>>2]|0;e=N2(I,(I|0)<0|0?-1:0,b,(b|0)<0|0?-1:0)|0;I=a+6944|0;E=NU(e,G,c[I>>2]|0,c[I+4>>2]|0)|0;c[I>>2]=E;c[I+4>>2]=G;y=b;i=g;return y|0}b=k;I=l;NN(b|0,I|0,152)|0;I=a+6928|0;b=c[I>>2]|0;E=(b|0)<(d|0)?b:d;do{if((E|0)==0){J=0;K=d;L=b}else{e=a+6924|0;f=c[e>>2]|0;H=k+132|0;F=c[H>>2]|0;z=(F|0)>0;do{if((c[a+6612>>2]|0)==0){if(!z){break}D=k+136|0;j=F-1|0;o=c[a+6468>>2]|0;h=j+(ac(F,f)|0)|0;c[k+(j<<2)>>2]=o+(ac(h,c[D>>2]|0)|0);if((j|0)>0){M=j}else{break}do{M=M-1|0;j=M+(ac(c[H>>2]|0,f)|0)|0;c[k+(M<<2)>>2]=o+(ac(j,c[D>>2]|0)|0);}while((M|0)>0)}else{if(!z){break}D=k+136|0;o=0;do{j=c[a+6468+(o<<2)>>2]|0;c[k+(o<<2)>>2]=j+(ac(c[D>>2]|0,f)|0);o=o+1|0;}while((o|0)<(c[H>>2]|0))}}while(0);H=rU(a,m,E,k,E)|0;if((H|0)<0){y=H;i=g;return y|0}c[I>>2]=(c[I>>2]|0)-H;c[e>>2]=(c[e>>2]|0)+H;f=a+6448|0;z=c[f>>2]|0;F=(z|0)>0;do{if((c[a+6460>>2]|0)==0){if(!F){break}o=m|0;D=a+6452|0;j=z-1|0;h=c[o>>2]|0;q=j+(ac(z,H)|0)|0;c[a+6316+(j<<2)>>2]=h+(ac(q,c[D>>2]|0)|0);if((j|0)>0){N=j}else{break}do{N=N-1|0;j=c[o>>2]|0;q=N+(ac(c[f>>2]|0,H)|0)|0;c[a+6316+(N<<2)>>2]=j+(ac(q,c[D>>2]|0)|0);}while((N|0)>0)}else{if(!F){break}D=a+6452|0;o=0;do{q=a+6316+(o<<2)|0;j=c[q>>2]|0;c[q>>2]=j+(ac(c[D>>2]|0,H)|0);o=o+1|0;}while((o|0)<(c[f>>2]|0))}}while(0);f=d-H|0;F=c[I>>2]|0;if((F|0)!=0){J=H;K=f;L=F;break}c[e>>2]=0;J=H;K=f;L=0}}while(0);do{if((C|0)==0){O=J}else{d=a+6924|0;N=c[d>>2]|0;E=C-K|0;M=E+N+L|0;b=(C|0)>(K|0);do{if(b){f=a+6468|0;if((M|0)<=(c[a+6608>>2]|0)|(E+L|0)>(N|0)){F=rS(f,M)|0;if((F|0)<0){y=F}else{break}i=g;return y|0}F=k+132|0;z=c[F>>2]|0;o=(z|0)>0;do{if((c[a+6612>>2]|0)==0){if(!o){break}D=k+136|0;j=z-1|0;q=c[f>>2]|0;h=j+(ac(z,N)|0)|0;c[k+(j<<2)>>2]=q+(ac(h,c[D>>2]|0)|0);if((j|0)>0){P=j}else{break}do{P=P-1|0;j=P+(ac(c[F>>2]|0,N)|0)|0;c[k+(P<<2)>>2]=q+(ac(j,c[D>>2]|0)|0);}while((P|0)>0)}else{if(!o){break}D=k+136|0;q=0;do{j=c[a+6468+(q<<2)>>2]|0;c[k+(q<<2)>>2]=j+(ac(c[D>>2]|0,N)|0);q=q+1|0;}while((q|0)<(c[F>>2]|0))}}while(0);rV(f,k,L);c[d>>2]=0}}while(0);if((K|0)==0){Q=J;R=C}else{N=b?K:C;M=rU(a,m,N,l,N)|0;if((M|0)<0){y=M;i=g;return y|0}N=a+5840|0;E=c[N>>2]|0;H=(E|0)>0;do{if((c[a+5852>>2]|0)==0){if(!H){break}e=l|0;F=a+5844|0;o=E-1|0;z=c[e>>2]|0;q=o+(ac(E,M)|0)|0;c[a+5708+(o<<2)>>2]=z+(ac(q,c[F>>2]|0)|0);if((o|0)>0){S=o}else{break}do{S=S-1|0;o=c[e>>2]|0;q=S+(ac(c[N>>2]|0,M)|0)|0;c[a+5708+(S<<2)>>2]=o+(ac(q,c[F>>2]|0)|0);}while((S|0)>0)}else{if(!H){break}F=a+5844|0;e=0;do{f=a+5708+(e<<2)|0;q=c[f>>2]|0;c[f>>2]=q+(ac(c[F>>2]|0,M)|0);e=e+1|0;}while((e|0)<(c[N>>2]|0))}}while(0);Q=M+J|0;R=C-M|0}if((R|0)==0){O=Q;break}N=(c[I>>2]|0)+(c[d>>2]|0)|0;H=k+132|0;E=c[H>>2]|0;b=(E|0)>0;do{if((c[a+6612>>2]|0)==0){if(!b){break}e=k+136|0;F=E-1|0;q=c[a+6468>>2]|0;f=F+(ac(E,N)|0)|0;c[k+(F<<2)>>2]=q+(ac(f,c[e>>2]|0)|0);if((F|0)>0){T=F}else{break}do{T=T-1|0;F=T+(ac(c[H>>2]|0,N)|0)|0;c[k+(T<<2)>>2]=q+(ac(F,c[e>>2]|0)|0);}while((T|0)>0)}else{if(!b){break}e=k+136|0;q=0;do{F=c[a+6468+(q<<2)>>2]|0;c[k+(q<<2)>>2]=F+(ac(c[e>>2]|0,N)|0);q=q+1|0;}while((q|0)<(c[H>>2]|0))}}while(0);rV(k,l,R);c[I>>2]=(c[I>>2]|0)+R;O=Q}}while(0);if((O|0)<=0){y=O;i=g;return y|0}if((c[n>>2]|0)!=0){y=O;i=g;return y|0}n=c[a+40>>2]|0;Q=N2(n,(n|0)<0|0?-1:0,O,(O|0)<0|0?-1:0)|0;n=a+6944|0;a=NU(Q,G,c[n>>2]|0,c[n+4>>2]|0)|0;c[n>>2]=a;c[n+4>>2]=G;y=O;i=g;return y|0}function rU(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;g=i;i=i+304|0;h=g|0;j=g+152|0;k=c[a+6972>>2]|0;if((k|0)!=0){if((c[a+6976>>2]|0)!=0){AN(0,0,1701712,(l=i,i=i+24|0,c[l>>2]=1667640,c[l+8>>2]=1696544,c[l+16>>2]=616,l)|0);i=l;cw();return 0}r4(k,b,e,f)|0;m=d;i=g;return m|0}k=a+5860|0;n=rS(k,f)|0;if((n|0)<0){m=n;i=g;return m|0}n=a+5696|0;o=a+6012|0;p=c[a+6144>>2]|0;do{if((c[n>>2]|0)==0){if((p|0)!=(c[a+6448>>2]|0)){AN(0,0,1701712,(l=i,i=i+24|0,c[l>>2]=1664920,c[l+8>>2]=1696544,c[l+16>>2]=631,l)|0);i=l;cw();return 0}q=rS(o,f)|0;if((q|0)<0){m=q}else{break}i=g;return m|0}else{if((p|0)!=(c[a+76>>2]|0)){AN(0,0,1701712,(l=i,i=i+24|0,c[l>>2]=1666368,c[l+8>>2]=1696544,c[l+16>>2]=627,l)|0);i=l;cw();return 0}q=rS(o,d)|0;if((q|0)<0){m=q}else{break}i=g;return m|0}}while(0);p=a+6164|0;q=rS(p,d)|0;if((q|0)<0){m=q;i=g;return m|0}q=j;r=o;NN(q|0,r|0,152)|0;r=h;q=p;NN(r|0,q|0,152)|0;q=a+16|0;r=c[q>>2]|0;do{if((r|0)==(c[a+12>>2]|0)){if((c[a+5852>>2]|0)==0){s=k;break}s=(c[a+72>>2]|0)==0?e:k}else{s=k}}while(0);k=c[n>>2]|0;if((k|0)==0){p=(c[a+5700>>2]|0)==0?s:j;if((c[a+6976>>2]|0)==0){t=p;u=3640}else{v=h;w=p}}else{p=(c[a+6976>>2]|0)==0?s:j;if((c[a+5700>>2]|0)==0){t=p;u=3640}else{v=h;w=p}}if((u|0)==3640){v=t;w=t}do{if((r|0)==(c[a+20>>2]|0)){if((c[a+6460>>2]|0)==0){x=s;y=v;z=w;break}if((r|0)==7){if((c[a+5620>>2]&31|0)!=0){x=s;y=v;z=w;break}}if((v|0)!=(e|0)){if((v|0)==(s|0)){x=b;y=b;z=b;break}x=s;y=b;z=(v|0)==(w|0)?b:w;break}t=(d|0)>(f|0)?f:d;if((c[a+5852>>2]|0)==0){AN(0,0,1701712,(l=i,i=i+24|0,c[l>>2]=1663568,c[l+8>>2]=1696544,c[l+16>>2]=658,l)|0);i=l;cw();return 0}rV(b,e,t);m=t;i=g;return m|0}else{x=s;y=v;z=w}}while(0);if((x|0)==(e|0)){A=k}else{k=c[a+6964>>2]|0;r4(k,x,e,f)|0;A=c[n>>2]|0}n=(x|0)!=(z|0);do{if((A|0)==0){if(n){k=(z|0)==(b|0)|0;sK(a,z,x,f,k)|0}if((z|0)==(y|0)){B=d;break}B=rY(a,y,d,z,f)|0}else{if(n){C=rY(a,z,d,x,f)|0}else{C=d}if((z|0)==(y|0)){B=C;break}k=(y|0)==(b|0)|0;sK(a,y,z,C,k)|0;B=C}}while(0);if((y|0)==(b|0)|(B|0)==0){m=B;i=g;return m|0}C=a+84|0;if((c[C>>2]|0)==0){D=y}else{z=(B|0)>65536?B:65536;do{if((y|0)==(e|0)){d=a+5468|0;f=rS(d,z)|0;if((f|0)<0){m=f}else{E=d;break}i=g;return m|0}else{E=y}}while(0);e=a+5316|0;d=rS(e,z)|0;if((d|0)<0){m=d;i=g;return m|0}z=a+5448|0;f=c[z>>2]|0;if((d|0)!=0&(f|0)>0){d=a+5456|0;x=a+5464|0;n=0;while(1){A=c[a+5316+(n<<2)>>2]|0;k=c[d>>2]|0;w=NW(1942904875,2874,n|0)|0;su(a,A,k,w,c[x>>2]|0);w=n+1|0;k=c[z>>2]|0;if((w|0)<(k|0)){n=w}else{F=k;break}}}else{F=f}f=y+132|0;if((F|0)!=(c[f>>2]|0)){AN(0,0,1701712,(l=i,i=i+24|0,c[l>>2]=1662048,c[l+8>>2]=1696544,c[l+16>>2]=700,l)|0);i=l;cw();return 0}l=a+88|0;F=c[l>>2]|0;if((F+B|0)>(c[a+5456>>2]|0)){c[l>>2]=0;G=0}else{G=F}L5105:do{if((c[C>>2]|0)>>>0<64){F=a+16260|0;n=c[F>>2]|0;if((n|0)==0){if((c[f>>2]|0)<=0){break}z=a+16256|0;x=a+5452|0;d=a+11084|0;k=0;w=G;while(1){A=c[z>>2]|0;v=c[E+(k<<2)>>2]|0;s=c[y+(k<<2)>>2]|0;r=c[a+5316+(k<<2)>>2]|0;t=r+(ac(w,c[x>>2]|0)|0)|0;dc[A&255](v,s,t,c[d>>2]|0,0,0,B);t=k+1|0;if((t|0)>=(c[f>>2]|0)){break L5105}k=t;w=c[l>>2]|0}}w=B&-16;k=ac(c[y+136>>2]|0,w)|0;L5114:do{if((w|0)!=0){if((c[f>>2]|0)<=0){break}d=a+5452|0;x=a+11088|0;z=0;t=n;s=G;while(1){v=c[E+(z<<2)>>2]|0;A=c[y+(z<<2)>>2]|0;r=c[a+5316+(z<<2)>>2]|0;u=r+(ac(s,c[d>>2]|0)|0)|0;dc[t&255](v,A,u,c[x>>2]|0,0,0,w);u=z+1|0;if((u|0)>=(c[f>>2]|0)){break L5114}z=u;t=c[F>>2]|0;s=c[l>>2]|0}}}while(0);if((B|0)==(w|0)){break}if((c[f>>2]|0)<=0){break}F=a+16256|0;n=a+5452|0;s=k+w|0;t=a+11084|0;z=B-w|0;x=0;do{d=c[F>>2]|0;u=(c[E+(x<<2)>>2]|0)+k|0;A=(c[y+(x<<2)>>2]|0)+k|0;v=c[a+5316+(x<<2)>>2]|0;r=v+(s+(ac(c[l>>2]|0,c[n>>2]|0)|0))|0;dc[d&255](u,A,r,c[t>>2]|0,0,0,z);x=x+1|0;}while((x|0)<(c[f>>2]|0))}else{x=c[q>>2]|0;if((x|0)==8){sy(a,E,y,e,B);break}else if((x|0)==9){sz(a,E,y,e,B);break}else if((x|0)==6){sw(a,E,y,e,B);break}else if((x|0)==7){sx(a,E,y,e,B);break}else{break}}}while(0);c[l>>2]=(c[l>>2]|0)+B;D=E}r4(c[a+6968>>2]|0,b,D,B)|0;m=B;i=g;return m|0}function rV(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;e=i;f=c[a+144>>2]|0;if((f|0)!=(c[b+144>>2]|0)){AN(0,0,1701712,(g=i,i=i+24|0,c[g>>2]=1672912,c[g+8>>2]=1696544,c[g+16>>2]=478,g)|0);i=g;cw()}h=a+136|0;j=c[h>>2]|0;if((j|0)!=(c[b+136>>2]|0)){AN(0,0,1701712,(g=i,i=i+24|0,c[g>>2]=1670848,c[g+8>>2]=1696544,c[g+16>>2]=479,g)|0);i=g;cw()}k=a+132|0;l=c[k>>2]|0;if((l|0)!=(c[b+132>>2]|0)){AN(0,0,1701712,(g=i,i=i+24|0,c[g>>2]=1669072,c[g+8>>2]=1696544,c[g+16>>2]=480,g)|0);i=g;cw()}if((f|0)==0){f=c[a>>2]|0;g=c[b>>2]|0;m=ac(ac(j,d)|0,l)|0;NN(f|0,g|0,m)|0;i=e;return}if((l|0)>0){n=0;o=j}else{i=e;return}while(1){j=c[a+(n<<2)>>2]|0;l=c[b+(n<<2)>>2]|0;m=ac(o,d)|0;NN(j|0,l|0,m)|0;m=n+1|0;if((m|0)>=(c[k>>2]|0)){break}n=m;o=c[h>>2]|0}i=e;return}function rW(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0;d=i;i=i+128|0;e=d|0;f=e;if((b|0)<1){g=0;i=d;return g|0}else{h=b}while(1){if((h|0)<=16384){break}b=rW(a,16384)|0;if((b|0)<0){g=b;j=3739;break}else{h=h-16384|0}}if((j|0)==3739){i=d;return g|0}j=a+6620|0;b=rS(j,h)|0;if((b|0)<0){g=b;i=d;return g|0}b=a+6764|0;do{if((c[b>>2]|0)==0){k=c[j>>2]|0;l=c[a+6756>>2]|0;m=ac(l,h)|0;NM(k|0,((l|0)==1?-128:0)|0,ac(m,c[a+6752>>2]|0)|0)}else{m=a+6752|0;if((c[m>>2]|0)<=0){break}l=a+6756|0;k=0;do{n=c[a+6620+(k<<2)>>2]|0;o=c[l>>2]|0;NM(n|0,((o|0)==1?-128:0)|0,ac(o,h)|0);k=k+1|0;}while((k|0)<(c[m>>2]|0))}}while(0);m=e|0;do{if((c[b>>2]|0)==0){c[m>>2]=c[j>>2]}else{e=c[a+6752>>2]|0;if((e|0)<=0){break}k=a+6620|0;l=(e|0)>1?e<<2:4;NN(f|0,k|0,l)|0}}while(0);AN(a,40,1678448,(f=i,i=i+8|0,c[f>>2]=h,f)|0);i=f;g=rT(a,0,0,m,h)|0;i=d;return g|0}function rX(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0.0,B=0.0,C=0.0,D=0.0,E=0.0,F=0.0,H=0.0,I=0,J=0,K=0,L=0,M=0;e=i;if((b|0)==0&(d|0)==(-2147483648|0)){f=a+6944|0;j=c[f+4>>2]|0;k=c[f>>2]|0;i=e;return(G=j,k)|0}f=a+6952|0;if((c[f>>2]|0)==0&(c[f+4>>2]|0)==(-2147483648|0)){c[f>>2]=b;c[f+4>>2]=d;l=a+6944|0;c[l>>2]=b;c[l+4>>2]=d}l=a+5668|0;m=+g[l>>2]<3.4028234663852886e+38;n=a+40|0;o=c[n>>2]|0;p=o;q=(o|0)<0|0?-1:0;r=a+44|0;s=c[r>>2]|0;t=N2(s,(s|0)<0|0?-1:0,p,q)|0;s=G;u=a+6980|0;v=c[u>>2]|0;do{if((v|0)==0){w=3748}else{if((c[a+6976>>2]|0)==0){w=3748;break}x=c6[c[v+20>>2]&255](a,t,s)|0;y=G;z=x}}while(0);if((w|0)==3748){w=c[a+6928>>2]|0;v=N2(w,(w|0)<0|0?-1:0,t,s)|0;s=o>>1;o=NU(s,(s|0)<0|0?-1:0,v,G)|0;v=N0(o,G,p,q)|0;y=G;z=v}v=NV(b,d,z,y)|0;y=G;z=a+6944|0;if(!m){c[z>>2]=v;c[z+4>>2]=y;j=y;k=v;i=e;return(G=j,k)|0}m=c[z>>2]|0;d=c[z+4>>2]|0;b=NV(v,y,m,d)|0;y=G;v=a+6960|0;q=c[v>>2]|0;p=c[n>>2]|0;n=p;o=(p|0)<0|0?-1:0;s=N2(n,o,q,(q|0)<0|0?-1:0)|0;t=NU(s,G,b,y)|0;y=G;b=c[r>>2]|0;r=b;s=(b|0)<0|0?-1:0;w=N2(r,s,n,o)|0;A=(+(t>>>0)+ +(y|0)*4294967296.0)/(+(w>>>0)+ +(G|0)*4294967296.0);B=+R(+A);L5197:do{if(B>+g[l>>2]){do{if(!((m|0)==(c[f>>2]|0)&(d|0)==(c[f+4>>2]|0))){if(B>+g[a+5672>>2]){break}C=+g[a+5676>>2];if(C==0.0){break L5197}D=+g[a+5680>>2];if(D==0.0){break L5197}w=~~(+(b|0)*C);if(D<0.0){E=+(-p|0)}else{E=1.0}C=D/E;D=A;F=-0.0-C;if(D<F){H=F}else{H=D>C?C:D}x=~~(+(w|0)*H);AN(a,40,1674656,(I=i,i=i+24|0,h[I>>3]=A,c[I+8>>2]=x,c[I+16>>2]=w,I)|0);i=I;if((a|0)==0|(w|0)<0){break L5197}if(!((w|0)!=0|(x|0)==0)){break L5197}J=a+6976|0;if((c[J>>2]|0)==0){K=a+48|0;c[K>>2]=c[K>>2]|1;if((rR(a)|0)<0){break L5197}}K=c[(c[u>>2]|0)+16>>2]|0;if((K|0)==0){break L5197}L=c[J>>2]|0;c6[K&255](L,x,w)|0;break L5197}}while(0);w=0;if((y|0)>(w|0)|(y|0)==(w|0)&t>>>0>0>>>0){w=N0(t,y,r,s)|0;M=rW(a,w)|0}else{w=NV(0,0,t,y)|0;x=N0(w,G,n,o)|0;w=x;x=q+w|0;c[v>>2]=x;if((x|0)<1){break}AN(a,40,1680336,(I=i,i=i+8|0,c[I>>2]=w,I)|0);i=I;M=rT(a,0,c[v>>2]|0,0,0)|0}if((M|0)>=0){break}AN(a,16,1676392,(I=i,i=i+8|0,h[I>>3]=A,I)|0);i=I}}while(0);j=c[z+4>>2]|0;k=c[z>>2]|0;i=e;return(G=j,k)|0}function rY(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0;g=i;i=i+464|0;h=g|0;j=g+152|0;k=g+304|0;l=g+456|0;m=j;n=b;NN(m|0,n|0,152)|0;m=k;NN(m|0,n|0,152)|0;n=h;m=e;NN(n|0,m|0,152)|0;m=a+6932|0;n=a+6928|0;e=a+6924|0;b=a+6612|0;o=k+132|0;p=a+6468|0;q=k+136|0;r=a+6980|0;s=a+6976|0;t=j+144|0;u=j+132|0;v=j|0;w=j+136|0;x=h+144|0;y=h+132|0;z=h|0;A=h+136|0;B=a+6936|0;C=a+6468|0;D=a+6608|0;E=C|0;F=0;G=0;H=f;f=d;d=(c[m>>2]|0)==0;L5226:while(1){do{if(d){I=c[n>>2]|0;if((I|0)==0){J=F;K=G;L=H;M=f;break}N=c[e>>2]|0;O=c[o>>2]|0;P=(O|0)>0;do{if((c[b>>2]|0)==0){if(!P){break}Q=O-1|0;R=c[p>>2]|0;S=Q+(ac(O,N)|0)|0;c[k+(Q<<2)>>2]=R+(ac(S,c[q>>2]|0)|0);if((Q|0)>0){T=Q}else{break}do{T=T-1|0;Q=T+(ac(c[o>>2]|0,N)|0)|0;c[k+(T<<2)>>2]=R+(ac(Q,c[q>>2]|0)|0);}while((T|0)>0)}else{if(P){U=0}else{break}do{R=c[a+6468+(U<<2)>>2]|0;c[k+(U<<2)>>2]=R+(ac(c[q>>2]|0,N)|0);U=U+1|0;}while((U|0)<(c[o>>2]|0))}}while(0);N=c0[c[(c[r>>2]|0)+8>>2]&15](c[s>>2]|0,j,f,k,I,l)|0;P=f-N|0;O=N+G|0;R=c[u>>2]|0;Q=(R|0)>0;do{if((c[t>>2]|0)==0){if(!Q){break}S=R-1|0;V=c[v>>2]|0;W=S+(ac(R,N)|0)|0;c[j+(S<<2)>>2]=V+(ac(W,c[w>>2]|0)|0);if((S|0)>0){X=S}else{break}do{X=X-1|0;S=c[v>>2]|0;W=X+(ac(c[u>>2]|0,N)|0)|0;c[j+(X<<2)>>2]=S+(ac(W,c[w>>2]|0)|0);}while((X|0)>0)}else{if(Q){Y=0}else{break}do{W=j+(Y<<2)|0;S=c[W>>2]|0;c[W>>2]=S+(ac(c[w>>2]|0,N)|0);Y=Y+1|0;}while((Y|0)<(c[u>>2]|0))}}while(0);N=c[l>>2]|0;Q=(c[n>>2]|0)-N|0;c[n>>2]=Q;c[e>>2]=(c[e>>2]|0)+N;if((H|0)==0){Z=O;_=P;break L5226}if((Q|0)>(F|0)){J=F;K=O;L=H;M=P;break}N=-Q|0;R=c[y>>2]|0;I=(R|0)>0;do{if((c[x>>2]|0)==0){if(!I){break}S=R-1|0;W=c[z>>2]|0;V=S+(ac(R,N)|0)|0;c[h+(S<<2)>>2]=W+(ac(V,c[A>>2]|0)|0);if((S|0)>0){$=S}else{break}do{$=$-1|0;S=c[z>>2]|0;V=$+(ac(c[y>>2]|0,N)|0)|0;c[h+($<<2)>>2]=S+(ac(V,c[A>>2]|0)|0);}while(($|0)>0)}else{if(I){aa=0}else{break}do{V=h+(aa<<2)|0;S=c[V>>2]|0;c[V>>2]=S+(ac(c[A>>2]|0,N)|0);aa=aa+1|0;}while((aa|0)<(c[y>>2]|0))}}while(0);c[n>>2]=0;c[e>>2]=0;J=0;K=O;L=Q+H|0;M=P}else{J=F;K=G;L=H;M=f}}while(0);L5261:do{if((c[B>>2]|L|0)==0){ab=K;ad=L;ae=M}else{if((c[n>>2]|0)!=0){ab=K;ad=L;ae=M;break}c[e>>2]=0;N=c0[c[(c[r>>2]|0)+8>>2]&15](c[s>>2]|0,j,M,h,L,l)|0;I=M-N|0;R=N+K|0;S=c[u>>2]|0;V=(S|0)>0;do{if((c[t>>2]|0)==0){if(!V){break}W=S-1|0;af=c[v>>2]|0;ag=W+(ac(S,N)|0)|0;c[j+(W<<2)>>2]=af+(ac(ag,c[w>>2]|0)|0);if((W|0)>0){ah=W}else{break}do{ah=ah-1|0;W=c[v>>2]|0;ag=ah+(ac(c[u>>2]|0,N)|0)|0;c[j+(ah<<2)>>2]=W+(ac(ag,c[w>>2]|0)|0);}while((ah|0)>0)}else{if(V){ai=0}else{break}do{ag=j+(ai<<2)|0;W=c[ag>>2]|0;c[ag>>2]=W+(ac(c[w>>2]|0,N)|0);ai=ai+1|0;}while((ai|0)<(c[u>>2]|0))}}while(0);N=c[l>>2]|0;V=L-N|0;S=c[y>>2]|0;P=(S|0)>0;if((c[x>>2]|0)!=0){if(P){aj=0}else{ab=R;ad=V;ae=I;break}while(1){Q=h+(aj<<2)|0;O=c[Q>>2]|0;c[Q>>2]=O+(ac(c[A>>2]|0,N)|0);O=aj+1|0;if((O|0)<(c[y>>2]|0)){aj=O}else{ab=R;ad=V;ae=I;break L5261}}}if(!P){ab=R;ad=V;ae=I;break}O=S-1|0;Q=c[z>>2]|0;W=O+(ac(S,N)|0)|0;c[h+(O<<2)>>2]=Q+(ac(W,c[A>>2]|0)|0);if((O|0)>0){ak=O}else{ab=R;ad=V;ae=I;break}while(1){O=ak-1|0;W=c[z>>2]|0;Q=O+(ac(c[y>>2]|0,N)|0)|0;c[h+(O<<2)>>2]=W+(ac(Q,c[A>>2]|0)|0);if((O|0)>0){ak=O}else{ab=R;ad=V;ae=I;break}}}}while(0);I=c[e>>2]|0;V=c[n>>2]|0;R=I+ad+V|0;if((R|0)<=(c[D>>2]|0)|(V+ad|0)>(I|0)){N=rS(C,R)|0;if((N|0)<0){al=N;am=3842;break}}else{N=c[o>>2]|0;R=(N|0)>0;do{if((c[b>>2]|0)==0){if(!R){break}S=N-1|0;P=c[E>>2]|0;O=S+(ac(N,I)|0)|0;c[k+(S<<2)>>2]=P+(ac(O,c[q>>2]|0)|0);if((S|0)>0){an=S}else{break}do{an=an-1|0;S=an+(ac(c[o>>2]|0,I)|0)|0;c[k+(an<<2)>>2]=P+(ac(S,c[q>>2]|0)|0);}while((an|0)>0)}else{if(R){ao=0}else{break}do{P=c[a+6468+(ao<<2)>>2]|0;c[k+(ao<<2)>>2]=P+(ac(c[q>>2]|0,I)|0);ao=ao+1|0;}while((ao|0)<(c[o>>2]|0))}}while(0);rV(C,k,V);c[e>>2]=0}if((ad|0)==0){Z=ab;_=ae;break}I=c[n>>2]|0;if((I|0)==0){ap=ad}else{R=I+2|0;ap=(R|0)>=(ad|0)|(ae|0)==0?ad:R}R=I+(c[e>>2]|0)|0;I=c[o>>2]|0;N=(I|0)>0;do{if((c[b>>2]|0)==0){if(!N){break}P=I-1|0;S=c[E>>2]|0;O=P+(ac(I,R)|0)|0;c[k+(P<<2)>>2]=S+(ac(O,c[q>>2]|0)|0);if((P|0)>0){aq=P}else{break}do{aq=aq-1|0;P=aq+(ac(c[o>>2]|0,R)|0)|0;c[k+(aq<<2)>>2]=S+(ac(P,c[q>>2]|0)|0);}while((aq|0)>0)}else{if(N){ar=0}else{break}do{S=c[a+6468+(ar<<2)>>2]|0;c[k+(ar<<2)>>2]=S+(ac(c[q>>2]|0,R)|0);ar=ar+1|0;}while((ar|0)<(c[o>>2]|0))}}while(0);rV(k,h,ap);R=c[n>>2]|0;c[n>>2]=R+ap;N=ad-ap|0;I=ap+J|0;V=c[y>>2]|0;S=(V|0)>0;do{if((c[x>>2]|0)==0){if(!S){break}P=V-1|0;O=c[z>>2]|0;Q=P+(ac(V,ap)|0)|0;c[h+(P<<2)>>2]=O+(ac(Q,c[A>>2]|0)|0);if((P|0)>0){as=P}else{break}do{as=as-1|0;P=c[z>>2]|0;Q=as+(ac(c[y>>2]|0,ap)|0)|0;c[h+(as<<2)>>2]=P+(ac(Q,c[A>>2]|0)|0);}while((as|0)>0)}else{if(S){at=0}else{break}do{Q=h+(at<<2)|0;P=c[Q>>2]|0;c[Q>>2]=P+(ac(c[A>>2]|0,ap)|0);at=at+1|0;}while((at|0)<(c[y>>2]|0))}}while(0);c[m>>2]=0;if((R|0)==0&(ad|0)==(ap|0)){Z=ab;_=ae;break}else{F=I;G=ab;H=N;f=ae;d=1}}if((am|0)==3842){i=g;return al|0}c[m>>2]=(_|0)!=0;al=Z;i=g;return al|0}function rZ(a){a=a|0;return 1717496}function r_(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0;g=B4(b)|0;h=c[234312+(((B4(d)|0)*10|0)+g<<2)>>2]|0;if((h|0)==0){i=0;return i|0}g=A7(32)|0;j=g;if((g|0)==0){i=0;return i|0}if((e|0)==1){k=B5(d)|0;l=B5(b)|0;m=k}else{l=b;m=d}c[g>>2]=e;c[g+12>>2]=h;c[g+20>>2]=f;if((m|0)==5|(m|0)==0){h=g+24|0;e=h|0;y=-2139062144;a[e]=y&255;y=y>>8;a[e+1|0]=y&255;y=y>>8;a[e+2|0]=y&255;y=y>>8;a[e+3|0]=y&255;e=h+4|0;y=-2139062144;a[e]=y&255;y=y>>8;a[e+1|0]=y&255;y=y>>8;a[e+2|0]=y&255;y=y>>8;a[e+3|0]=y&255}if(!((l|0)==(m|0)&(f|0)==0)){i=j;return i|0}f=B7(m)|0;if((f|0)==1){c[g+16>>2]=112;i=j;return i|0}else if((f|0)==2){c[g+16>>2]=114;i=j;return i|0}else if((f|0)==4){c[g+16>>2]=116;i=j;return i|0}else if((f|0)==8){c[g+16>>2]=42;i=j;return i|0}else{i=j;return i|0}return 0}function r$(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=c[a>>2]|0;a=c[b>>2]|0;NN(e|0,a|0,d)|0;return}function r0(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=c[a>>2]|0;a=c[b>>2]|0;b=d<<1;NN(e|0,a|0,b)|0;return}function r1(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=c[a>>2]|0;a=c[b>>2]|0;b=d<<2;NN(e|0,a|0,b)|0;return}function r2(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;e=c[a>>2]|0;a=c[b>>2]|0;b=d<<3;NN(e|0,a|0,b)|0;return}function r3(a){a=a|0;A6(a);return}function r4(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0;f=i;g=b+144|0;h=c[g>>2]|0;j=(h|0)==0;k=c[b+132>>2]|0;l=ac(c[b+136>>2]|0,j?k:1)|0;m=a|0;n=b+132|0;if((c[m>>2]|0)!=(k|0)){AN(0,0,1644e3,(o=i,i=i+24|0,c[o>>2]=1719720,c[o+8>>2]=1691440,c[o+16>>2]=175,o)|0);i=o;cw();return 0}o=c[a+4>>2]|0;if((o|0)==0){p=0}else{if((c[d+144>>2]|0)==0){q=1;r=3876}else{s=c[d+132>>2]|0;if((s|0)>0){q=s;r=3876}else{t=0}}if((r|0)==3876){s=0;u=0;while(1){v=c[d+(u<<2)>>2]|s;w=u+1|0;if((w|0)<(q|0)){s=v;u=w}else{t=v;break}}}p=o&t}t=c[a+8>>2]|0;if((t|0)==0){x=p}else{if(j){y=1;r=3882}else{if((k|0)>0){y=k;r=3882}else{z=0}}if((r|0)==3882){r=0;o=0;while(1){u=c[b+(o<<2)>>2]|r;s=o+1|0;if((s|0)<(y|0)){r=u;o=s}else{z=u;break}}}x=t&z|p}p=a+16|0;z=c[p>>2]|0;do{if((z|0)==0){A=0;B=k}else{if(!((c[a+20>>2]|0)==0&(x|0)==0)){A=0;B=k;break}t=e&-16;L5380:do{if((t|0)>0){if((h|0)!=(c[d+144>>2]|0)){dm[z&127](b|0,d|0,t);break}if(j){C=1}else{if((k|0)>0){C=k}else{break}}o=0;r=z;y=h;while(1){if((y|0)==0){D=c[n>>2]|0}else{D=1}u=ac(D,t)|0;dm[r&127](b+(o<<2)|0,d+(o<<2)|0,u);u=o+1|0;if((u|0)>=(C|0)){break L5380}o=u;r=c[p>>2]|0;y=c[g>>2]|0}}}while(0);if((t|0)==(e|0)){i=f;return 0}else{A=t;B=c[m>>2]|0;break}}}while(0);if((B|0)<=0){i=f;return 0}g=a+20|0;p=a+24|0;C=ac(l,e)|0;e=a+12|0;a=ac(A,l)|0;D=d+144|0;n=d+132|0;h=d+136|0;z=0;k=B;while(1){B=c[g>>2]|0;if((B|0)==0){E=z}else{E=c[B+(z<<2)>>2]|0}if((E|0)<0){F=p;G=0}else{if((c[D>>2]|0)==0){H=ac(c[h>>2]|0,c[n>>2]|0)|0}else{H=c[h>>2]|0}F=c[d+(E<<2)>>2]|0;G=H}B=c[b+(z<<2)>>2]|0;if((B|0)==0){I=k}else{j=c[e>>2]|0;x=F+(ac(G,A)|0)|0;cX[j&255](B+a|0,x,G,l,B+C|0);I=c[m>>2]|0}B=z+1|0;if((B|0)<(I|0)){z=B;k=I}else{break}}i=f;return 0}function r5(b,c,d,e,f){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;g=f+(e*-3|0)|0;if(g>>>0>b>>>0){h=d<<1;i=e<<1;j=h+d|0;k=i+e|0;l=j+d|0;m=k+e|0;n=c;o=b;while(1){a[o]=a[n]|0;a[o+e|0]=a[n+d|0]|0;a[o+i|0]=a[n+h|0]|0;a[o+k|0]=a[n+j|0]|0;p=n+l|0;q=o+m|0;if(q>>>0<g>>>0){n=p;o=q}else{r=p;s=q;break}}}else{r=c;s=b}if(s>>>0<f>>>0){t=r;u=s}else{return}while(1){a[u]=a[t]|0;s=u+e|0;if(s>>>0<f>>>0){t=t+d|0;u=s}else{break}}return}function r6(a,c,e,f,g){a=a|0;c=c|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=g+(f*-3|0)|0;if(h>>>0>a>>>0){i=e<<1;j=f<<1;k=i+e|0;l=j+f|0;m=k+e|0;n=l+f|0;o=c;p=a;while(1){b[p>>1]=(d[o]|0)<<8^-32768;b[p+f>>1]=(d[o+e|0]|0)<<8^-32768;b[p+j>>1]=(d[o+i|0]|0)<<8^-32768;b[p+l>>1]=(d[o+k|0]|0)<<8^-32768;q=o+m|0;r=p+n|0;if(r>>>0<h>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=c;t=a}if(t>>>0<g>>>0){u=s;v=t}else{return}while(1){b[v>>1]=(d[u]|0)<<8^-32768;t=v+f|0;if(t>>>0<g>>>0){u=u+e|0;v=t}else{break}}return}function r7(a,b,e,f,g){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=g+(f*-3|0)|0;if(h>>>0>a>>>0){i=e<<1;j=f<<1;k=i+e|0;l=j+f|0;m=k+e|0;n=l+f|0;o=b;p=a;while(1){c[p>>2]=(d[o]|0)<<24^-2147483648;c[p+f>>2]=(d[o+e|0]|0)<<24^-2147483648;c[p+j>>2]=(d[o+i|0]|0)<<24^-2147483648;c[p+l>>2]=(d[o+k|0]|0)<<24^-2147483648;q=o+m|0;r=p+n|0;if(r>>>0<h>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=b;t=a}if(t>>>0<g>>>0){u=s;v=t}else{return}while(1){c[v>>2]=(d[u]|0)<<24^-2147483648;t=v+f|0;if(t>>>0<g>>>0){u=u+e|0;v=t}else{break}}return}function r8(a,b,c,e,f){a=a|0;b=b|0;c=c|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=f+(e*-3|0)|0;if(h>>>0>a>>>0){i=c<<1;j=e<<1;k=i+c|0;l=j+e|0;m=k+c|0;n=l+e|0;o=b;p=a;while(1){g[p>>2]=+((d[o]|0)-128|0)*.0078125;g[p+e>>2]=+((d[o+c|0]|0)-128|0)*.0078125;g[p+j>>2]=+((d[o+i|0]|0)-128|0)*.0078125;g[p+l>>2]=+((d[o+k|0]|0)-128|0)*.0078125;q=o+m|0;r=p+n|0;if(r>>>0<h>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=b;t=a}if(t>>>0<f>>>0){u=s;v=t}else{return}while(1){g[v>>2]=+((d[u]|0)-128|0)*.0078125;t=v+e|0;if(t>>>0<f>>>0){u=u+c|0;v=t}else{break}}return}function r9(a,b,c,e,f){a=a|0;b=b|0;c=c|0;e=e|0;f=f|0;var g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;g=f+(e*-3|0)|0;if(g>>>0>a>>>0){i=c<<1;j=e<<1;k=i+c|0;l=j+e|0;m=k+c|0;n=l+e|0;o=b;p=a;while(1){h[p>>3]=+((d[o]|0)-128|0)*.0078125;h[p+e>>3]=+((d[o+c|0]|0)-128|0)*.0078125;h[p+j>>3]=+((d[o+i|0]|0)-128|0)*.0078125;h[p+l>>3]=+((d[o+k|0]|0)-128|0)*.0078125;q=o+m|0;r=p+n|0;if(r>>>0<g>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=b;t=a}if(t>>>0<f>>>0){u=s;v=t}else{return}while(1){h[v>>3]=+((d[u]|0)-128|0)*.0078125;t=v+e|0;if(t>>>0<f>>>0){u=u+c|0;v=t}else{break}}return}function sa(b,c,d,f,g){b=b|0;c=c|0;d=d|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=g+(f*-3|0)|0;if(h>>>0>b>>>0){i=d<<1;j=f<<1;k=i+d|0;l=j+f|0;m=k+d|0;n=l+f|0;o=c;p=b;while(1){a[p]=((e[o>>1]|0)>>>8)+128&255;a[p+f|0]=((e[o+d>>1]|0)>>>8)+128&255;a[p+j|0]=((e[o+i>>1]|0)>>>8)+128&255;a[p+l|0]=((e[o+k>>1]|0)>>>8)+128&255;q=o+m|0;r=p+n|0;if(r>>>0<h>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=c;t=b}if(t>>>0<g>>>0){u=s;v=t}else{return}while(1){a[v]=((e[u>>1]|0)>>>8)+128&255;t=v+f|0;if(t>>>0<g>>>0){u=u+d|0;v=t}else{break}}return}function sb(a,c,d,e,f){a=a|0;c=c|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;g=f+(e*-3|0)|0;if(g>>>0>a>>>0){h=d<<1;i=e<<1;j=h+d|0;k=i+e|0;l=j+d|0;m=k+e|0;n=c;o=a;while(1){b[o>>1]=b[n>>1]|0;b[o+e>>1]=b[n+d>>1]|0;b[o+i>>1]=b[n+h>>1]|0;b[o+k>>1]=b[n+j>>1]|0;p=n+l|0;q=o+m|0;if(q>>>0<g>>>0){n=p;o=q}else{r=p;s=q;break}}}else{r=c;s=a}if(s>>>0<f>>>0){t=r;u=s}else{return}while(1){b[u>>1]=b[t>>1]|0;s=u+e|0;if(s>>>0<f>>>0){t=t+d|0;u=s}else{break}}return}function sc(a,b,d,f,g){a=a|0;b=b|0;d=d|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=g+(f*-3|0)|0;if(h>>>0>a>>>0){i=d<<1;j=f<<1;k=i+d|0;l=j+f|0;m=k+d|0;n=l+f|0;o=b;p=a;while(1){c[p>>2]=(e[o>>1]|0)<<16;c[p+f>>2]=(e[o+d>>1]|0)<<16;c[p+j>>2]=(e[o+i>>1]|0)<<16;c[p+l>>2]=(e[o+k>>1]|0)<<16;q=o+m|0;r=p+n|0;if(r>>>0<h>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=b;t=a}if(t>>>0<g>>>0){u=s;v=t}else{return}while(1){c[v>>2]=(e[u>>1]|0)<<16;t=v+f|0;if(t>>>0<g>>>0){u=u+d|0;v=t}else{break}}return}function sd(a,c,d,e,f){a=a|0;c=c|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=f+(e*-3|0)|0;if(h>>>0>a>>>0){i=d<<1;j=e<<1;k=i+d|0;l=j+e|0;m=k+d|0;n=l+e|0;o=c;p=a;while(1){g[p>>2]=+(b[o>>1]|0)*30517578125.0e-15;g[p+e>>2]=+(b[o+d>>1]|0)*30517578125.0e-15;g[p+j>>2]=+(b[o+i>>1]|0)*30517578125.0e-15;g[p+l>>2]=+(b[o+k>>1]|0)*30517578125.0e-15;q=o+m|0;r=p+n|0;if(r>>>0<h>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=c;t=a}if(t>>>0<f>>>0){u=s;v=t}else{return}while(1){g[v>>2]=+(b[u>>1]|0)*30517578125.0e-15;t=v+e|0;if(t>>>0<f>>>0){u=u+d|0;v=t}else{break}}return}function se(a,c,d,e,f){a=a|0;c=c|0;d=d|0;e=e|0;f=f|0;var g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;g=f+(e*-3|0)|0;if(g>>>0>a>>>0){i=d<<1;j=e<<1;k=i+d|0;l=j+e|0;m=k+d|0;n=l+e|0;o=c;p=a;while(1){h[p>>3]=+(b[o>>1]|0)*30517578125.0e-15;h[p+e>>3]=+(b[o+d>>1]|0)*30517578125.0e-15;h[p+j>>3]=+(b[o+i>>1]|0)*30517578125.0e-15;h[p+l>>3]=+(b[o+k>>1]|0)*30517578125.0e-15;q=o+m|0;r=p+n|0;if(r>>>0<g>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=c;t=a}if(t>>>0<f>>>0){u=s;v=t}else{return}while(1){h[v>>3]=+(b[u>>1]|0)*30517578125.0e-15;t=v+e|0;if(t>>>0<f>>>0){u=u+d|0;v=t}else{break}}return}function sf(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=g+(f*-3|0)|0;if(h>>>0>b>>>0){i=e<<1;j=f<<1;k=i+e|0;l=j+f|0;m=k+e|0;n=l+f|0;o=d;p=b;while(1){a[p]=((c[o>>2]|0)>>>24)+128&255;a[p+f|0]=((c[o+e>>2]|0)>>>24)+128&255;a[p+j|0]=((c[o+i>>2]|0)>>>24)+128&255;a[p+l|0]=((c[o+k>>2]|0)>>>24)+128&255;q=o+m|0;r=p+n|0;if(r>>>0<h>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=d;t=b}if(t>>>0<g>>>0){u=s;v=t}else{return}while(1){a[v]=((c[u>>2]|0)>>>24)+128&255;t=v+f|0;if(t>>>0<g>>>0){u=u+e|0;v=t}else{break}}return}function sg(a,d,e,f,g){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=g+(f*-3|0)|0;if(h>>>0>a>>>0){i=e<<1;j=f<<1;k=i+e|0;l=j+f|0;m=k+e|0;n=l+f|0;o=d;p=a;while(1){b[p>>1]=(c[o>>2]|0)>>>16&65535;b[p+f>>1]=(c[o+e>>2]|0)>>>16&65535;b[p+j>>1]=(c[o+i>>2]|0)>>>16&65535;b[p+l>>1]=(c[o+k>>2]|0)>>>16&65535;q=o+m|0;r=p+n|0;if(r>>>0<h>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=d;t=a}if(t>>>0<g>>>0){u=s;v=t}else{return}while(1){b[v>>1]=(c[u>>2]|0)>>>16&65535;t=v+f|0;if(t>>>0<g>>>0){u=u+e|0;v=t}else{break}}return}function sh(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;g=f+(e*-3|0)|0;if(g>>>0>a>>>0){h=d<<1;i=e<<1;j=h+d|0;k=i+e|0;l=j+d|0;m=k+e|0;n=b;o=a;while(1){c[o>>2]=c[n>>2];c[o+e>>2]=c[n+d>>2];c[o+i>>2]=c[n+h>>2];c[o+k>>2]=c[n+j>>2];p=n+l|0;q=o+m|0;if(q>>>0<g>>>0){n=p;o=q}else{r=p;s=q;break}}}else{r=b;s=a}if(s>>>0<f>>>0){t=r;u=s}else{return}while(1){c[u>>2]=c[t>>2];s=u+e|0;if(s>>>0<f>>>0){t=t+d|0;u=s}else{break}}return}function si(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;h=f+(e*-3|0)|0;if(h>>>0>a>>>0){i=d<<1;j=e<<1;k=i+d|0;l=j+e|0;m=k+d|0;n=l+e|0;o=b;p=a;while(1){g[p>>2]=+(c[o>>2]|0)*4.656612873077393e-10;g[p+e>>2]=+(c[o+d>>2]|0)*4.656612873077393e-10;g[p+j>>2]=+(c[o+i>>2]|0)*4.656612873077393e-10;g[p+l>>2]=+(c[o+k>>2]|0)*4.656612873077393e-10;q=o+m|0;r=p+n|0;if(r>>>0<h>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=b;t=a}if(t>>>0<f>>>0){u=s;v=t}else{return}while(1){g[v>>2]=+(c[u>>2]|0)*4.656612873077393e-10;t=v+e|0;if(t>>>0<f>>>0){u=u+d|0;v=t}else{break}}return}function sj(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;g=f+(e*-3|0)|0;if(g>>>0>a>>>0){i=d<<1;j=e<<1;k=i+d|0;l=j+e|0;m=k+d|0;n=l+e|0;o=b;p=a;while(1){h[p>>3]=+(c[o>>2]|0)*4.656612873077393e-10;h[p+e>>3]=+(c[o+d>>2]|0)*4.656612873077393e-10;h[p+j>>3]=+(c[o+i>>2]|0)*4.656612873077393e-10;h[p+l>>3]=+(c[o+k>>2]|0)*4.656612873077393e-10;q=o+m|0;r=p+n|0;if(r>>>0<g>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=b;t=a}if(t>>>0<f>>>0){u=s;v=t}else{return}while(1){h[v>>3]=+(c[u>>2]|0)*4.656612873077393e-10;t=v+e|0;if(t>>>0<f>>>0){u=u+d|0;v=t}else{break}}return}function sk(b,c,d,e,f){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;h=f+(e*-3|0)|0;if(h>>>0>b>>>0){i=d<<1;j=e<<1;k=i+d|0;l=j+e|0;m=k+d|0;n=l+e|0;o=b;p=c;while(1){q=bR(+(+g[p>>2]*128.0))|0;r=q+128|0;if(r>>>0>255){s=-128-q>>31&255}else{s=r&255}a[o]=s;r=bR(+(+g[p+d>>2]*128.0))|0;q=r+128|0;if(q>>>0>255){t=-128-r>>31&255}else{t=q&255}a[o+e|0]=t;q=bR(+(+g[p+i>>2]*128.0))|0;r=q+128|0;if(r>>>0>255){u=-128-q>>31&255}else{u=r&255}a[o+j|0]=u;r=bR(+(+g[p+k>>2]*128.0))|0;q=r+128|0;if(q>>>0>255){v=-128-r>>31&255}else{v=q&255}a[o+l|0]=v;q=p+m|0;r=o+n|0;if(r>>>0<h>>>0){o=r;p=q}else{w=r;x=q;break}}}else{w=b;x=c}if(w>>>0<f>>>0){y=w;z=x}else{return}while(1){x=bR(+(+g[z>>2]*128.0))|0;w=x+128|0;if(w>>>0>255){A=-128-x>>31&255}else{A=w&255}a[y]=A;w=y+e|0;if(w>>>0<f>>>0){y=w;z=z+d|0}else{break}}return}function sl(a,c,d,e,f){a=a|0;c=c|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;h=f+(e*-3|0)|0;if(h>>>0>a>>>0){i=d<<1;j=e<<1;k=i+d|0;l=j+e|0;m=k+d|0;n=l+e|0;o=a;p=c;while(1){q=bR(+(+g[p>>2]*32768.0))|0;if((q+32768|0)>>>0>65535){r=(q>>31^32767)&65535}else{r=q&65535}b[o>>1]=r;q=bR(+(+g[p+d>>2]*32768.0))|0;if((q+32768|0)>>>0>65535){s=(q>>31^32767)&65535}else{s=q&65535}b[o+e>>1]=s;q=bR(+(+g[p+i>>2]*32768.0))|0;if((q+32768|0)>>>0>65535){t=(q>>31^32767)&65535}else{t=q&65535}b[o+j>>1]=t;q=bR(+(+g[p+k>>2]*32768.0))|0;if((q+32768|0)>>>0>65535){u=(q>>31^32767)&65535}else{u=q&65535}b[o+l>>1]=u;q=p+m|0;v=o+n|0;if(v>>>0<h>>>0){o=v;p=q}else{w=v;x=q;break}}}else{w=a;x=c}if(w>>>0<f>>>0){y=w;z=x}else{return}while(1){x=bR(+(+g[z>>2]*32768.0))|0;if((x+32768|0)>>>0>65535){A=(x>>31^32767)&65535}else{A=x&65535}b[y>>1]=A;x=y+e|0;if(x>>>0<f>>>0){y=x;z=z+d|0}else{break}}return}function sm(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;h=f+(e*-3|0)|0;if(h>>>0>a>>>0){i=d<<1;j=e<<1;k=i+d|0;l=j+e|0;m=k+d|0;n=l+e|0;o=a;p=b;while(1){q=cS(+(+g[p>>2]*2147483648.0))|0;r=G;s=NU(q,r,-2147483648,0)|0;t=G;u=0;if(t>>>0>u>>>0|t>>>0==u>>>0&s>>>0>-1>>>0){s=r>>31|((r|0)<0|0?-1:0)<<1;((r|0)<0|0?-1:0)>>31|((r|0)<0|0?-1:0)<<1;v=s^2147483647}else{v=q}c[o>>2]=v;q=cS(+(+g[p+d>>2]*2147483648.0))|0;s=G;r=NU(q,s,-2147483648,0)|0;u=G;t=0;if(u>>>0>t>>>0|u>>>0==t>>>0&r>>>0>-1>>>0){r=s>>31|((s|0)<0|0?-1:0)<<1;((s|0)<0|0?-1:0)>>31|((s|0)<0|0?-1:0)<<1;w=r^2147483647}else{w=q}c[o+e>>2]=w;q=cS(+(+g[p+i>>2]*2147483648.0))|0;r=G;s=NU(q,r,-2147483648,0)|0;t=G;u=0;if(t>>>0>u>>>0|t>>>0==u>>>0&s>>>0>-1>>>0){s=r>>31|((r|0)<0|0?-1:0)<<1;((r|0)<0|0?-1:0)>>31|((r|0)<0|0?-1:0)<<1;x=s^2147483647}else{x=q}c[o+j>>2]=x;q=cS(+(+g[p+k>>2]*2147483648.0))|0;s=G;r=NU(q,s,-2147483648,0)|0;u=G;t=0;if(u>>>0>t>>>0|u>>>0==t>>>0&r>>>0>-1>>>0){r=s>>31|((s|0)<0|0?-1:0)<<1;((s|0)<0|0?-1:0)>>31|((s|0)<0|0?-1:0)<<1;y=r^2147483647}else{y=q}c[o+l>>2]=y;q=p+m|0;r=o+n|0;if(r>>>0<h>>>0){o=r;p=q}else{z=r;A=q;break}}}else{z=a;A=b}if(z>>>0<f>>>0){B=z;C=A}else{return}while(1){A=cS(+(+g[C>>2]*2147483648.0))|0;z=G;b=NU(A,z,-2147483648,0)|0;a=G;p=0;if(a>>>0>p>>>0|a>>>0==p>>>0&b>>>0>-1>>>0){b=z>>31|((z|0)<0|0?-1:0)<<1;((z|0)<0|0?-1:0)>>31|((z|0)<0|0?-1:0)<<1;D=b^2147483647}else{D=A}c[B>>2]=D;A=B+e|0;if(A>>>0<f>>>0){B=A;C=C+d|0}else{break}}return}function sn(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;f=e+(d*-3|0)|0;if(f>>>0>a>>>0){h=c<<1;i=d<<1;j=h+c|0;k=i+d|0;l=j+c|0;m=k+d|0;n=b;o=a;while(1){g[o>>2]=+g[n>>2];g[o+d>>2]=+g[n+c>>2];g[o+i>>2]=+g[n+h>>2];g[o+k>>2]=+g[n+j>>2];p=n+l|0;q=o+m|0;if(q>>>0<f>>>0){n=p;o=q}else{r=p;s=q;break}}}else{r=b;s=a}if(s>>>0<e>>>0){t=r;u=s}else{return}while(1){g[u>>2]=+g[t>>2];s=u+d|0;if(s>>>0<e>>>0){t=t+c|0;u=s}else{break}}return}function so(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;f=e+(d*-3|0)|0;if(f>>>0>a>>>0){i=c<<1;j=d<<1;k=i+c|0;l=j+d|0;m=k+c|0;n=l+d|0;o=b;p=a;while(1){h[p>>3]=+g[o>>2];h[p+d>>3]=+g[o+c>>2];h[p+j>>3]=+g[o+i>>2];h[p+l>>3]=+g[o+k>>2];q=o+m|0;r=p+n|0;if(r>>>0<f>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=b;t=a}if(t>>>0<e>>>0){u=s;v=t}else{return}while(1){h[v>>3]=+g[u>>2];t=v+d|0;if(t>>>0<e>>>0){u=u+c|0;v=t}else{break}}return}function sp(b,c,d,e,f){b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;var g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;g=f+(e*-3|0)|0;if(g>>>0>b>>>0){i=d<<1;j=e<<1;k=i+d|0;l=j+e|0;m=k+d|0;n=l+e|0;o=b;p=c;while(1){q=bR(+(+h[p>>3]*128.0))|0;r=q+128|0;if(r>>>0>255){s=-128-q>>31&255}else{s=r&255}a[o]=s;r=bR(+(+h[p+d>>3]*128.0))|0;q=r+128|0;if(q>>>0>255){t=-128-r>>31&255}else{t=q&255}a[o+e|0]=t;q=bR(+(+h[p+i>>3]*128.0))|0;r=q+128|0;if(r>>>0>255){u=-128-q>>31&255}else{u=r&255}a[o+j|0]=u;r=bR(+(+h[p+k>>3]*128.0))|0;q=r+128|0;if(q>>>0>255){v=-128-r>>31&255}else{v=q&255}a[o+l|0]=v;q=p+m|0;r=o+n|0;if(r>>>0<g>>>0){o=r;p=q}else{w=r;x=q;break}}}else{w=b;x=c}if(w>>>0<f>>>0){y=w;z=x}else{return}while(1){x=bR(+(+h[z>>3]*128.0))|0;w=x+128|0;if(w>>>0>255){A=-128-x>>31&255}else{A=w&255}a[y]=A;w=y+e|0;if(w>>>0<f>>>0){y=w;z=z+d|0}else{break}}return}function sq(a,c,d,e,f){a=a|0;c=c|0;d=d|0;e=e|0;f=f|0;var g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;g=f+(e*-3|0)|0;if(g>>>0>a>>>0){i=d<<1;j=e<<1;k=i+d|0;l=j+e|0;m=k+d|0;n=l+e|0;o=a;p=c;while(1){q=bR(+(+h[p>>3]*32768.0))|0;if((q+32768|0)>>>0>65535){r=(q>>31^32767)&65535}else{r=q&65535}b[o>>1]=r;q=bR(+(+h[p+d>>3]*32768.0))|0;if((q+32768|0)>>>0>65535){s=(q>>31^32767)&65535}else{s=q&65535}b[o+e>>1]=s;q=bR(+(+h[p+i>>3]*32768.0))|0;if((q+32768|0)>>>0>65535){t=(q>>31^32767)&65535}else{t=q&65535}b[o+j>>1]=t;q=bR(+(+h[p+k>>3]*32768.0))|0;if((q+32768|0)>>>0>65535){u=(q>>31^32767)&65535}else{u=q&65535}b[o+l>>1]=u;q=p+m|0;v=o+n|0;if(v>>>0<g>>>0){o=v;p=q}else{w=v;x=q;break}}}else{w=a;x=c}if(w>>>0<f>>>0){y=w;z=x}else{return}while(1){x=bR(+(+h[z>>3]*32768.0))|0;if((x+32768|0)>>>0>65535){A=(x>>31^32767)&65535}else{A=x&65535}b[y>>1]=A;x=y+e|0;if(x>>>0<f>>>0){y=x;z=z+d|0}else{break}}return}function sr(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;g=f+(e*-3|0)|0;if(g>>>0>a>>>0){i=d<<1;j=e<<1;k=i+d|0;l=j+e|0;m=k+d|0;n=l+e|0;o=a;p=b;while(1){q=cS(+(+h[p>>3]*2147483648.0))|0;r=G;s=NU(q,r,-2147483648,0)|0;t=G;u=0;if(t>>>0>u>>>0|t>>>0==u>>>0&s>>>0>-1>>>0){s=r>>31|((r|0)<0|0?-1:0)<<1;((r|0)<0|0?-1:0)>>31|((r|0)<0|0?-1:0)<<1;v=s^2147483647}else{v=q}c[o>>2]=v;q=cS(+(+h[p+d>>3]*2147483648.0))|0;s=G;r=NU(q,s,-2147483648,0)|0;u=G;t=0;if(u>>>0>t>>>0|u>>>0==t>>>0&r>>>0>-1>>>0){r=s>>31|((s|0)<0|0?-1:0)<<1;((s|0)<0|0?-1:0)>>31|((s|0)<0|0?-1:0)<<1;w=r^2147483647}else{w=q}c[o+e>>2]=w;q=cS(+(+h[p+i>>3]*2147483648.0))|0;r=G;s=NU(q,r,-2147483648,0)|0;t=G;u=0;if(t>>>0>u>>>0|t>>>0==u>>>0&s>>>0>-1>>>0){s=r>>31|((r|0)<0|0?-1:0)<<1;((r|0)<0|0?-1:0)>>31|((r|0)<0|0?-1:0)<<1;x=s^2147483647}else{x=q}c[o+j>>2]=x;q=cS(+(+h[p+k>>3]*2147483648.0))|0;s=G;r=NU(q,s,-2147483648,0)|0;u=G;t=0;if(u>>>0>t>>>0|u>>>0==t>>>0&r>>>0>-1>>>0){r=s>>31|((s|0)<0|0?-1:0)<<1;((s|0)<0|0?-1:0)>>31|((s|0)<0|0?-1:0)<<1;y=r^2147483647}else{y=q}c[o+l>>2]=y;q=p+m|0;r=o+n|0;if(r>>>0<g>>>0){o=r;p=q}else{z=r;A=q;break}}}else{z=a;A=b}if(z>>>0<f>>>0){B=z;C=A}else{return}while(1){A=cS(+(+h[C>>3]*2147483648.0))|0;z=G;b=NU(A,z,-2147483648,0)|0;a=G;p=0;if(a>>>0>p>>>0|a>>>0==p>>>0&b>>>0>-1>>>0){b=z>>31|((z|0)<0|0?-1:0)<<1;((z|0)<0|0?-1:0)>>31|((z|0)<0|0?-1:0)<<1;D=b^2147483647}else{D=A}c[B>>2]=D;A=B+e|0;if(A>>>0<f>>>0){B=A;C=C+d|0}else{break}}return}function ss(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;f=e+(d*-3|0)|0;if(f>>>0>a>>>0){i=c<<1;j=d<<1;k=i+c|0;l=j+d|0;m=k+c|0;n=l+d|0;o=b;p=a;while(1){g[p>>2]=+h[o>>3];g[p+d>>2]=+h[o+c>>3];g[p+j>>2]=+h[o+i>>3];g[p+l>>2]=+h[o+k>>3];q=o+m|0;r=p+n|0;if(r>>>0<f>>>0){o=q;p=r}else{s=q;t=r;break}}}else{s=b;t=a}if(t>>>0<e>>>0){u=s;v=t}else{return}while(1){g[v>>2]=+h[u>>3];t=v+d|0;if(t>>>0<e>>>0){u=u+c|0;v=t}else{break}}return}function st(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0,g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;f=e+(d*-3|0)|0;if(f>>>0>a>>>0){g=c<<1;i=d<<1;j=g+c|0;k=i+d|0;l=j+c|0;m=k+d|0;n=b;o=a;while(1){h[o>>3]=+h[n>>3];h[o+d>>3]=+h[n+c>>3];h[o+i>>3]=+h[n+g>>3];h[o+k>>3]=+h[n+j>>3];p=n+l|0;q=o+m|0;if(q>>>0<f>>>0){n=p;o=q}else{r=p;s=q;break}}}else{r=b;s=a}if(s>>>0<e>>>0){t=r;u=s}else{return}while(1){h[u>>3]=+h[t>>3];s=u+d|0;if(s>>>0<e>>>0){t=t+c|0;u=s}else{break}}return}function su(a,d,e,f,j){a=a|0;d=d|0;e=e|0;f=f|0;j=j|0;var k=0,l=0,m=0.0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0.0,v=0,w=0,x=0,y=0.0,z=0,A=0.0,B=0.0;k=i;l=a+84|0;m=+g[a+96>>2];a=e+2|0;n=A1(a<<3)|0;o=n;L5784:do{if((a|0)>0){p=c[l>>2]|0;q=(p|0)==1;r=p>>>0<72;p=f;s=0;while(1){t=(ac(p,1664525)|0)+1013904223|0;if(q){u=+(t>>>0>>>0)/4294967295.0+-.5;v=t}else{if(!r){break}w=(ac(t,1664525)|0)+1013904223|0;u=+(t>>>0>>>0)/4294967295.0- +(w>>>0>>>0)/4294967295.0;v=w}h[o+(s<<3)>>3]=u;w=s+1|0;if((w|0)<(a|0)){p=v;s=w}else{break L5784}}AN(0,0,1643752,(x=i,i=i+24|0,c[x>>2]=1719656,c[x+8>>2]=1691376,c[x+16>>2]=39,x)|0);i=x;cw()}}while(0);if((e|0)<=0){A4(n);i=k;return}v=l|0;l=d;a=d;f=d;s=d;if((j|0)==7){d=0;while(1){p=c[v>>2]|0;if((p|0)==3){y=(+h[o+(d+1<<3)>>3]*2.0- +h[o+(d<<3)>>3]- +h[o+(d+2<<3)>>3])/2.449489742783178}else{if(p>>>0>=72){z=4207;break}y=+h[o+(d<<3)>>3]}c[a+(d<<2)>>2]=~~(m*y);p=d+1|0;if((p|0)<(e|0)){d=p}else{z=4209;break}}if((z|0)==4207){AN(0,0,1643752,(x=i,i=i+24|0,c[x>>2]=1719656,c[x+8>>2]=1691376,c[x+16>>2]=53,x)|0);i=x;cw()}else if((z|0)==4209){A4(n);i=k;return}}else if((j|0)==6){d=0;while(1){a=c[v>>2]|0;if((a|0)==3){A=(+h[o+(d+1<<3)>>3]*2.0- +h[o+(d<<3)>>3]- +h[o+(d+2<<3)>>3])/2.449489742783178}else{if(a>>>0>=72){z=4206;break}A=+h[o+(d<<3)>>3]}b[l+(d<<1)>>1]=~~(m*A);a=d+1|0;if((a|0)<(e|0)){d=a}else{z=4211;break}}if((z|0)==4211){A4(n);i=k;return}else if((z|0)==4206){AN(0,0,1643752,(x=i,i=i+24|0,c[x>>2]=1719656,c[x+8>>2]=1691376,c[x+16>>2]=53,x)|0);i=x;cw()}}else{d=0;while(1){l=c[v>>2]|0;if((l|0)==3){B=(+h[o+(d+1<<3)>>3]*2.0- +h[o+(d<<3)>>3]- +h[o+(d+2<<3)>>3])/2.449489742783178}else{if(l>>>0>=72){z=4208;break}B=+h[o+(d<<3)>>3]}A=m*B;if((j|0)==9){h[s+(d<<3)>>3]=A}else if((j|0)==8){g[f+(d<<2)>>2]=A}else{z=4203;break}l=d+1|0;if((l|0)<(e|0)){d=l}else{z=4212;break}}if((z|0)==4203){AN(0,0,1643752,(x=i,i=i+24|0,c[x>>2]=1671688,c[x+8>>2]=1691376,c[x+16>>2]=68,x)|0);i=x;cw()}else if((z|0)==4208){AN(0,0,1643752,(x=i,i=i+24|0,c[x>>2]=1719656,c[x+8>>2]=1691376,c[x+16>>2]=53,x)|0);i=x;cw()}else if((z|0)==4212){A4(n);i=k;return}}}function sv(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,j=0,k=0,l=0.0,m=0,n=0,o=0,p=0.0,q=0.0,r=0.0,s=0,t=0,u=0,v=0;e=i;f=a+84|0;if(((c[f>>2]|0)-4|0)>>>0<61){j=-22;i=e;return j|0}k=B4(b)|0;b=B4(d)|0;do{if((b-3|0)>>>0<2){d=(k|0)==2;l=(k|0)==0?.0078125:(k|0)==1?30517578125.0e-15:d?-4.656612873077393e-10:0.0;m=0;n=d}else{d=(b|0)==2;o=(k|0)==2;if(!(d&o)){l=0.0;m=d;n=o;break}l=(c[a+5620>>2]&31|0)==0?0.0:1.0;m=1;n=1}}while(0);o=(k|0)==0;p=+g[a+92>>2]*((b|0)==1&o?256.0:m&o?16777216.0:m&(k|0)==1?65536.0:l);do{if(n){m=c[a+5620>>2]|0;if((m|0)==0){q=p;break}q=p*+(1<<32-m|0)}else{q=p}}while(0);c[a+112>>2]=0;p=q;n=a+96|0;g[n>>2]=p;g[a+104>>2]=p;if(q!=0.0){r=1.0/q}else{r=0.0}m=a+108|0;g[m>>2]=r;NM(a+196|0,0,5120);o=c[a+44>>2]|0;b=0;d=234856;while(1){s=c[234840+(b*24|0)>>2]|0;if(+R(+(+(o-s|0)))/+(s|0)<=.05){if((c[234860+(b*24|0)>>2]|0)==(c[f>>2]|0)){t=4226;break}}s=b+1|0;if((s|0)==15){t=4231;break}else{b=s;d=234856+(s*24|0)|0}}do{if((t|0)==4226){o=c[234848+(b*24|0)>>2]|0;c[a+100>>2]=o;if((b|0)!=15){s=c[d>>2]|0;u=0;do{g[a+116+(u<<2)>>2]=+h[s+(u<<3)>>3];u=u+1|0;}while(u>>>0<o>>>0)}r=+$(+(+(c[234852+(b*24|0)>>2]|0)*2.302585092994046*.005))*2.0;q=1.0-r/+(1<<((B7(k)|0)<<3)|0);g[m>>2]=q*+g[m>>2]}else if((t|0)==4231){if((c[f>>2]|0)>>>0<=64){break}AN(a,24,1657024,(v=i,i=i+1|0,i=i+7&-8,c[v>>2]=0,v)|0);i=v;c[f>>2]=3}}while(0);if((c[a+6304>>2]|0)!=0){AN(0,0,1643752,(v=i,i=i+24|0,c[v>>2]=1638024,c[v+8>>2]=1691376,c[v+16>>2]=122,v)|0);i=v;cw();return 0}v=a+5316|0;t=a+6164|0;NN(v|0,t|0,152)|0;v=a+5468|0;NN(v|0,t|0,152)|0;if((c[f>>2]|0)>>>0<=64){j=0;i=e;return j|0}c[a+5452>>2]=4;c[a+5464>>2]=8;g[n>>2]=1.0;j=0;i=e;return j|0}function sw(a,d,e,f,h){a=a|0;d=d|0;e=e|0;f=f|0;h=h|0;var i=0,j=0,k=0,l=0.0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0.0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0.0,C=0,D=0.0,E=0,F=0.0,G=0,H=0.0,I=0.0,J=0,K=0,L=0;i=a+112|0;j=c[i>>2]|0;k=c[a+100>>2]|0;l=+g[a+108>>2];m=e+132|0;n=c[m>>2]|0;if((n|0)<=0){o=j;c[i>>2]=o;return}p=a+88|0;q=(h|0)>0;r=k-2|0;s=(r|0)>0;t=+g[a+104>>2];u=0;v=n;n=j;while(1){j=c[f+(u<<2)>>2]|0;w=c[p>>2]|0;x=c[e+(u<<2)>>2]|0;y=c[d+(u<<2)>>2]|0;if(q){z=0;A=n;do{B=l*+(b[x+(z<<1)>>1]|0);if(s){C=0;D=B;while(1){E=C+A|0;F=D-(+g[a+116+(C<<2)>>2]*+g[a+196+(u*160|0)+(E<<2)>>2]+ +g[a+116+((C|1)<<2)>>2]*+g[a+196+(u*160|0)+(E+1<<2)>>2]+ +g[a+116+((C|2)<<2)>>2]*+g[a+196+(u*160|0)+(E+2<<2)>>2]+ +g[a+116+((C|3)<<2)>>2]*+g[a+196+(u*160|0)+(E+3<<2)>>2]);E=C+4|0;if((E|0)<(r|0)){C=E;D=F}else{G=E;H=F;break}}}else{G=0;H=B}if((G|0)<(k|0)){I=H- +g[a+116+(G<<2)>>2]*+g[a+196+(u*160|0)+(G+A<<2)>>2]}else{I=H}A=((A|0)!=0?A:k)-1|0;D=+bR(+(I+ +g[j+(z+w<<2)>>2]));F=D-I;g[a+196+(u*160|0)+(A<<2)>>2]=F;g[a+196+(u*160|0)+(A+k<<2)>>2]=F;F=t*D;D=F>32767.0?32767.0:F;if(D>-32768.0){J=~~D}else{J=-32768}b[y+(z<<1)>>1]=J;z=z+1|0;}while((z|0)<(h|0));K=A;L=c[m>>2]|0}else{K=n;L=v}z=u+1|0;if((z|0)>=(L|0)){o=K;break}u=z;v=L;n=c[i>>2]|0}c[i>>2]=o;return}function sx(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0.0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0.0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0.0,B=0,C=0.0,D=0,E=0.0,F=0,G=0.0,H=0.0,I=0,J=0,K=0;h=a+112|0;i=c[h>>2]|0;j=c[a+100>>2]|0;k=+g[a+108>>2];l=d+132|0;m=c[l>>2]|0;if((m|0)<=0){n=i;c[h>>2]=n;return}o=a+88|0;p=(f|0)>0;q=j-2|0;r=(q|0)>0;s=+g[a+104>>2];t=0;u=m;m=i;while(1){i=c[e+(t<<2)>>2]|0;v=c[o>>2]|0;w=c[d+(t<<2)>>2]|0;x=c[b+(t<<2)>>2]|0;if(p){y=0;z=m;do{A=k*+(c[w+(y<<2)>>2]|0);if(r){B=0;C=A;while(1){D=B+z|0;E=C-(+g[a+116+(B<<2)>>2]*+g[a+196+(t*160|0)+(D<<2)>>2]+ +g[a+116+((B|1)<<2)>>2]*+g[a+196+(t*160|0)+(D+1<<2)>>2]+ +g[a+116+((B|2)<<2)>>2]*+g[a+196+(t*160|0)+(D+2<<2)>>2]+ +g[a+116+((B|3)<<2)>>2]*+g[a+196+(t*160|0)+(D+3<<2)>>2]);D=B+4|0;if((D|0)<(q|0)){B=D;C=E}else{F=D;G=E;break}}}else{F=0;G=A}if((F|0)<(j|0)){H=G- +g[a+116+(F<<2)>>2]*+g[a+196+(t*160|0)+(F+z<<2)>>2]}else{H=G}z=((z|0)!=0?z:j)-1|0;C=+bR(+(H+ +g[i+(y+v<<2)>>2]));E=C-H;g[a+196+(t*160|0)+(z<<2)>>2]=E;g[a+196+(t*160|0)+(z+j<<2)>>2]=E;E=s*C;C=E>2147483647.0?2147483647.0:E;if(C>-2147483648.0){I=~~C}else{I=-2147483648}c[x+(y<<2)>>2]=I;y=y+1|0;}while((y|0)<(f|0));J=z;K=c[l>>2]|0}else{J=m;K=u}y=t+1|0;if((y|0)>=(K|0)){n=J;break}t=y;u=K;m=c[h>>2]|0}c[h>>2]=n;return}function sy(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var h=0,i=0,j=0,k=0.0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0.0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0.0,B=0,C=0.0,D=0,E=0.0,F=0,G=0.0,H=0.0,I=0,J=0;h=a+112|0;i=c[h>>2]|0;j=c[a+100>>2]|0;k=+g[a+108>>2];l=d+132|0;m=c[l>>2]|0;if((m|0)<=0){n=i;c[h>>2]=n;return}o=a+88|0;p=(f|0)>0;q=j-2|0;r=(q|0)>0;s=+g[a+104>>2];t=0;u=m;m=i;while(1){i=c[e+(t<<2)>>2]|0;v=c[o>>2]|0;w=c[d+(t<<2)>>2]|0;x=c[b+(t<<2)>>2]|0;if(p){y=0;z=m;do{A=k*+g[w+(y<<2)>>2];if(r){B=0;C=A;while(1){D=B+z|0;E=C-(+g[a+116+(B<<2)>>2]*+g[a+196+(t*160|0)+(D<<2)>>2]+ +g[a+116+((B|1)<<2)>>2]*+g[a+196+(t*160|0)+(D+1<<2)>>2]+ +g[a+116+((B|2)<<2)>>2]*+g[a+196+(t*160|0)+(D+2<<2)>>2]+ +g[a+116+((B|3)<<2)>>2]*+g[a+196+(t*160|0)+(D+3<<2)>>2]);D=B+4|0;if((D|0)<(q|0)){B=D;C=E}else{F=D;G=E;break}}}else{F=0;G=A}if((F|0)<(j|0)){H=G- +g[a+116+(F<<2)>>2]*+g[a+196+(t*160|0)+(F+z<<2)>>2]}else{H=G}z=((z|0)!=0?z:j)-1|0;C=+bR(+(H+ +g[i+(y+v<<2)>>2]));E=C-H;g[a+196+(t*160|0)+(z<<2)>>2]=E;g[a+196+(t*160|0)+(z+j<<2)>>2]=E;g[x+(y<<2)>>2]=s*C;y=y+1|0;}while((y|0)<(f|0));I=z;J=c[l>>2]|0}else{I=m;J=u}y=t+1|0;if((y|0)>=(J|0)){n=I;break}t=y;u=J;m=c[h>>2]|0}c[h>>2]=n;return}function sz(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0,s=0,t=0.0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0.0,C=0,D=0.0,E=0,F=0.0,G=0,H=0.0,I=0.0,J=0,K=0;i=a+112|0;j=c[i>>2]|0;k=c[a+100>>2]|0;l=d+132|0;m=c[l>>2]|0;if((m|0)<=0){n=j;c[i>>2]=n;return}o=a+88|0;p=(f|0)>0;q=+g[a+108>>2];r=k-2|0;s=(r|0)>0;t=+g[a+104>>2];u=0;v=m;m=j;while(1){j=c[e+(u<<2)>>2]|0;w=c[o>>2]|0;x=c[d+(u<<2)>>2]|0;y=c[b+(u<<2)>>2]|0;if(p){z=0;A=m;do{B=q*+h[x+(z<<3)>>3];if(s){C=0;D=B;while(1){E=C+A|0;F=D-(+g[a+116+(C<<2)>>2]*+g[a+196+(u*160|0)+(E<<2)>>2]+ +g[a+116+((C|1)<<2)>>2]*+g[a+196+(u*160|0)+(E+1<<2)>>2]+ +g[a+116+((C|2)<<2)>>2]*+g[a+196+(u*160|0)+(E+2<<2)>>2]+ +g[a+116+((C|3)<<2)>>2]*+g[a+196+(u*160|0)+(E+3<<2)>>2]);E=C+4|0;if((E|0)<(r|0)){C=E;D=F}else{G=E;H=F;break}}}else{G=0;H=B}if((G|0)<(k|0)){I=H- +g[a+116+(G<<2)>>2]*+g[a+196+(u*160|0)+(G+A<<2)>>2]}else{I=H}A=((A|0)!=0?A:k)-1|0;D=+bR(+(I+ +g[j+(z+w<<2)>>2]));F=D-I;g[a+196+(u*160|0)+(A<<2)>>2]=F;g[a+196+(u*160|0)+(A+k<<2)>>2]=F;h[y+(z<<3)>>3]=t*D;z=z+1|0;}while((z|0)<(f|0));J=A;K=c[l>>2]|0}else{J=m;K=v}z=u+1|0;if((z|0)>=(K|0)){n=J;break}u=z;v=K;m=c[i>>2]|0}c[i>>2]=n;return}function sA(b){b=b|0;var d=0,e=0,f=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0.0,F=0.0,H=0.0,I=0,J=0,K=0.0,L=0.0,M=0.0,N=0.0,O=0,P=0,Q=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0;d=i;i=i+33024|0;e=d+128|0;f=b+24|0;j=zs(c[f>>2]|0,c[f+4>>2]|0)|0;k=b+32|0;l=zs(c[k>>2]|0,c[k+4>>2]|0)|0;m=b+16264|0;c[m>>2]=0;L5959:do{if((c[b+5704>>2]|0)==0){n=d+32896|0;NM(e|0,0,32768);o=c[b+68>>2]|0;p=c[f>>2]|0;q=c[f+4>>2]|0;r=d|0;L5961:do{if((p&1610612736|0)==1610612736&(q&0|0)==0){s=3}else{do{if(!((p|0)==4&(q|0)==0|(p|0)==0&(q|0)==0)){t=NU(p,q,-1,-1)|0;if(!((t&p|0)==0&(G&q|0)==0)){break}zt(r,128,-1,p,q);AN(b,40,1613064,(u=i,i=i+8|0,c[u>>2]=r,u)|0);i=u;s=4;break L5961}}while(0);s=p}}while(0);p=s;q=(s|0)<0|0?-1:0;do{if(!((p&7|0)==0&(q&0|0)==0)){t=p&3;v=q&0;w=(t|0)==0&(v|0)==0;if(!w){x=NU(p,q,3,0)|0;if((x&t|0)==0&(G&v|0)==0){break}}v=p&1536;t=q&0;if(!((v|0)==0&(t|0)==0)){x=NU(v,t,2047,0)|0;if((x&v|0)==0&(G&t|0)==0){break}}t=p&48;v=q&0;if(!((t|0)==0&(v|0)==0)){x=NU(t,v,63,0)|0;if((x&t|0)==0&(G&v|0)==0){break}}v=p&192;t=q&0;if(!((v|0)==0&(t|0)==0)){x=NU(v,t,255,0)|0;if((x&v|0)==0&(G&t|0)==0){break}}if((zs(p,q)|0)>=32){break}t=c[k>>2]|0;v=c[k+4>>2]|0;L5983:do{if((t&1610612736|0)==1610612736&(v&0|0)==0){y=3}else{do{if(!((t|0)==4&(v|0)==0|(t|0)==0&(v|0)==0)){x=NU(t,v,-1,-1)|0;if(!((x&t|0)==0&(G&v|0)==0)){break}zt(r,128,-1,t,v);AN(b,40,1613064,(u=i,i=i+8|0,c[u>>2]=r,u)|0);i=u;y=4;break L5983}}while(0);y=t}}while(0);t=y;v=(y|0)<0|0?-1:0;do{if(!((t&7|0)==0&(v&0|0)==0)){x=t&3;z=v&0;if(!((x|0)==0&(z|0)==0)){A=NU(t,v,3,0)|0;if((A&x|0)==0&(G&z|0)==0){break}}A=t&1536;B=v&0;if(!((A|0)==0&(B|0)==0)){C=NU(A,B,2047,0)|0;if((C&A|0)==0&(G&B|0)==0){break}}B=t&48;A=v&0;if(!((B|0)==0&(A|0)==0)){C=NU(B,A,63,0)|0;if((C&B|0)==0&(G&A|0)==0){break}}A=t&192;B=v&0;if(!((A|0)==0&(B|0)==0)){C=NU(A,B,255,0)|0;if((C&A|0)==0&(G&B|0)==0){break}}if((zs(t,v)|0)>=32){break}NM(b+6984|0,0,4096);B=y&s;A=B;C=(B|0)<0|0?-1:0;B=0;do{D=NW(1,0,B|0)|0;if(!((D&A|0)==0&(G&C|0)==0)){h[e+(B<<9)+(B<<3)>>3]=1.0}B=B+1|0;}while((B|0)<64);B=p&~t;C=q&~v;do{if(!((B&4|0)==0&(C&0|0)==0)){if(!((x|0)==3&(z|0)==0)){AN(0,0,1643368,(u=i,i=i+24|0,c[u>>2]=1719592,c[u+8>>2]=1691344,c[u+16>>2]=160,u)|0);i=u;cw();return 0}if(w){A=e+16|0;h[A>>3]=+h[A>>3]+.7071067811865476;A=e+528|0;h[A>>3]=+h[A>>3]+.7071067811865476;break}else{E=+g[b+56>>2];A=e+16|0;h[A>>3]=+h[A>>3]+E;A=e+528|0;h[A>>3]=E+ +h[A>>3];break}}}while(0);do{if(!((B&3|0)==0&(C&0|0)==0)){if((t&4|0)==0&(v&0|0)==0){AN(0,0,1643368,(u=i,i=i+24|0,c[u>>2]=1719592,c[u+8>>2]=1691344,c[u+16>>2]=169,u)|0);i=u;cw();return 0}z=e+1024|0;h[z>>3]=+h[z>>3]+.7071067811865476;z=e+1032|0;h[z>>3]=+h[z>>3]+.7071067811865476;if((p&4|0)==0&(q&0|0)==0){break}h[e+1040>>3]=+g[b+56>>2]*1.4142135623730951}}while(0);do{if(!((B&256|0)==0&(C&0|0)==0)){if(!((t&16|0)==0&(v&0|0)==0)){z=e+2112|0;h[z>>3]=+h[z>>3]+.7071067811865476;z=e+2624|0;h[z>>3]=+h[z>>3]+.7071067811865476;break}if(!((t&512|0)==0&(v&0|0)==0)){z=e+4672|0;h[z>>3]=+h[z>>3]+.7071067811865476;z=e+5184|0;h[z>>3]=+h[z>>3]+.7071067811865476;break}if((t&1|0)==0&(v&0|0)==0){if((t&4|0)==0&(v&0|0)==0){AN(0,0,1643368,(u=i,i=i+24|0,c[u>>2]=1719592,c[u+8>>2]=1691344,c[u+16>>2]=196,u)|0);i=u;cw();return 0}else{z=e+1088|0;h[z>>3]=+h[z>>3]+ +g[b+52>>2]*.7071067811865476;break}}if((o-1|0)>>>0>=2){E=+g[b+52>>2]*.7071067811865476;z=e+64|0;h[z>>3]=+h[z>>3]+E;z=e+576|0;h[z>>3]=+h[z>>3]+E;break}E=+g[b+52>>2];if((B&528|0)==0&(C&0|0)==0){z=e+64|0;h[z>>3]=+h[z>>3]-E;z=e+576|0;h[z>>3]=E+ +h[z>>3];break}else{F=E*.7071067811865476;z=e+64|0;h[z>>3]=+h[z>>3]-F;z=e+576|0;h[z>>3]=F+ +h[z>>3];break}}}while(0);do{if(!((B&16|0)==0&(C&0|0)==0)){if(!((t&256|0)==0&(v&0|0)==0)){z=e+4128|0;h[z>>3]=+h[z>>3]+.7071067811865476;z=e+4136|0;h[z>>3]=+h[z>>3]+.7071067811865476;break}if(!((t&512|0)==0&(v&0|0)==0)){z=e+4640|0;F=+h[z>>3];if((p&512|0)==0&(q&0|0)==0){h[z>>3]=F+1.0;x=e+5160|0;h[x>>3]=+h[x>>3]+1.0;break}else{h[z>>3]=F+.7071067811865476;z=e+5160|0;h[z>>3]=+h[z>>3]+.7071067811865476;break}}if((t&1|0)==0&(v&0|0)==0){if((t&4|0)==0&(v&0|0)==0){AN(0,0,1643368,(u=i,i=i+24|0,c[u>>2]=1719592,c[u+8>>2]=1691344,c[u+16>>2]=229,u)|0);i=u;cw();return 0}else{F=+g[b+52>>2]*.7071067811865476;z=e+1056|0;h[z>>3]=+h[z>>3]+F;z=e+1064|0;h[z>>3]=+h[z>>3]+F;break}}if((o|0)==1){F=+g[b+52>>2]*.7071067811865476;z=e+32|0;h[z>>3]=+h[z>>3]-F;z=e+40|0;h[z>>3]=+h[z>>3]-F;z=e+544|0;h[z>>3]=F+ +h[z>>3];z=e+552|0;h[z>>3]=F+ +h[z>>3];break}F=+g[b+52>>2];if((o|0)==2){E=F*1.224744871391589;z=e+32|0;h[z>>3]=+h[z>>3]-E;H=F*.7071067811865476;z=e+40|0;h[z>>3]=+h[z>>3]-H;z=e+544|0;h[z>>3]=H+ +h[z>>3];z=e+552|0;h[z>>3]=E+ +h[z>>3];break}else{z=e+32|0;h[z>>3]=F+ +h[z>>3];z=e+552|0;h[z>>3]=F+ +h[z>>3];break}}}while(0);do{if(!((B&512|0)==0&(C&0|0)==0)){if(!((t&16|0)==0&(v&0|0)==0)){z=e+2120|0;F=+h[z>>3];if((p&16|0)==0&(q&0|0)==0){h[z>>3]=F+1.0;x=e+2640|0;h[x>>3]=+h[x>>3]+1.0;break}else{h[z>>3]=F+.7071067811865476;z=e+2640|0;h[z>>3]=+h[z>>3]+.7071067811865476;break}}if(!((t&256|0)==0&(v&0|0)==0)){z=e+4168|0;h[z>>3]=+h[z>>3]+.7071067811865476;z=e+4176|0;h[z>>3]=+h[z>>3]+.7071067811865476;break}if((t&1|0)==0&(v&0|0)==0){if((t&4|0)==0&(v&0|0)==0){AN(0,0,1643368,(u=i,i=i+24|0,c[u>>2]=1719592,c[u+8>>2]=1691344,c[u+16>>2]=265,u)|0);i=u;cw();return 0}else{F=+g[b+52>>2]*.7071067811865476;z=e+1096|0;h[z>>3]=+h[z>>3]+F;z=e+1104|0;h[z>>3]=+h[z>>3]+F;break}}if((o|0)==1){F=+g[b+52>>2]*.7071067811865476;z=e+72|0;h[z>>3]=+h[z>>3]-F;z=e+80|0;h[z>>3]=+h[z>>3]-F;z=e+584|0;h[z>>3]=F+ +h[z>>3];z=e+592|0;h[z>>3]=F+ +h[z>>3];break}F=+g[b+52>>2];if((o|0)==2){E=F*1.224744871391589;z=e+72|0;h[z>>3]=+h[z>>3]-E;H=F*.7071067811865476;z=e+80|0;h[z>>3]=+h[z>>3]-H;z=e+584|0;h[z>>3]=H+ +h[z>>3];z=e+592|0;h[z>>3]=E+ +h[z>>3];break}else{z=e+72|0;h[z>>3]=F+ +h[z>>3];z=e+592|0;h[z>>3]=F+ +h[z>>3];break}}}while(0);do{if(!((B&64|0)==0&(C&0|0)==0)){if(!((t&1|0)==0&(v&0|0)==0)){z=e+48|0;h[z>>3]=+h[z>>3]+1.0;z=e+568|0;h[z>>3]=+h[z>>3]+1.0;break}if((t&4|0)==0&(v&0|0)==0){AN(0,0,1643368,(u=i,i=i+24|0,c[u>>2]=1719592,c[u+8>>2]=1691344,c[u+16>>2]=276,u)|0);i=u;cw();return 0}else{z=e+1072|0;h[z>>3]=+h[z>>3]+.7071067811865476;z=e+1080|0;h[z>>3]=+h[z>>3]+.7071067811865476;break}}}while(0);do{if((B&8|0)==0&(C&0|0)==0){I=0;J=0;K=0.0}else{if(!((t&4|0)==0&(v&0|0)==0)){z=e+1048|0;h[z>>3]=+h[z>>3]+ +g[b+60>>2];I=0;J=0;K=0.0;break}if((t&1|0)==0&(v&0|0)==0){AN(0,0,1643368,(u=i,i=i+24|0,c[u>>2]=1719592,c[u+8>>2]=1691344,c[u+16>>2]=286,u)|0);i=u;cw();return 0}else{F=+g[b+60>>2]*.7071067811865476;z=e+24|0;h[z>>3]=+h[z>>3]+F;z=e+536|0;h[z>>3]=+h[z>>3]+F;I=0;J=0;K=0.0;break}}}while(0);while(1){C=0;F=0.0;B=0;while(1){E=+h[e+(J<<9)+(C<<3)>>3];g[b+6984+(I<<7)+(B<<2)>>2]=E;if(E!=0.0){L=F+ +R(+E)}else{L=F}z=NW(1,0,C|0)|0;x=C+1|0;if((x|0)<64){C=x;F=L;B=(((z&p|0)!=0|(G&q|0)!=0)&1)+B|0}else{break}}M=K>L?K:L;B=NW(1,0,J|0)|0;C=J+1|0;if((C|0)<64){I=(((B&t|0)!=0|(G&v|0)!=0)&1)+I|0;J=C;K=M}else{break}}C=b+64|0;F=+g[C>>2];if(F<0.0){N=-0.0-F}else{N=M}if((B4(c[b+20>>2]|0)|0)<3){if(N>1.0){O=0;P=4417}}else{if((B4(c[b+16>>2]|0)|0)<3&N>1.0){O=0;P=4417}}if((P|0)==4417){while(1){P=0;B=0;do{z=b+6984+(O<<7)+(B<<2)|0;g[z>>2]=+g[z>>2]/N;B=B+1|0;}while((B|0)<32);B=O+1|0;if((B|0)<32){O=B;P=4417}else{break}}}if(+g[C>>2]>0.0){B=0;do{z=0;do{x=b+6984+(B<<7)+(z<<2)|0;g[x>>2]=+g[C>>2]*+g[x>>2];z=z+1|0;}while((z|0)<32);B=B+1|0;}while((B|0)<32)}if((zs(t,v)|0)>0){Q=0}else{break L5959}while(1){if((zs(p,q)|0)>0){B=0;do{AN(0,48,1623528,(u=i,i=i+8|0,h[u>>3]=+g[b+6984+(Q<<7)+(B<<2)>>2],u)|0);i=u;B=B+1|0;}while((B|0)<(zs(p,q)|0))}AN(0,48,1617864,(u=i,i=i+1|0,i=i+7&-8,c[u>>2]=0,u)|0);i=u;Q=Q+1|0;if((Q|0)>=(zs(t,v)|0)){break L5959}}}}while(0);zt(n,128,-1,c[k>>2]|0,c[k+4>>2]|0);AN(b,16,1629688,(u=i,i=i+8|0,c[u>>2]=n,u)|0);i=u;S=-22;i=d;return S|0}}while(0);zt(n,128,-1,c[f>>2]|0,c[f+4>>2]|0);AN(b,16,1637968,(u=i,i=i+8|0,c[u>>2]=n,u)|0);i=u;S=-22;i=d;return S|0}}while(0);Q=c[b+6160>>2]|0;if((Q|0)==6){O=b+11080|0;c[O>>2]=A8(ac(l,j)|0,4)|0;J=A7(4)|0;I=b+11084|0;c[I>>2]=J;if((l|0)>0&(j|0)>0){e=0;do{s=ac(e,j)|0;y=0;do{q=bR(+(+g[b+6984+(e<<7)+(y<<2)>>2]*32768.0))|0;c[(c[O>>2]|0)+(y+s<<2)>>2]=q;y=y+1|0;}while((y|0)<(j|0));e=e+1|0;}while((e|0)<(l|0));T=c[I>>2]|0}else{T=J}c[T>>2]=32768;c[b+16248>>2]=14;c[b+16256>>2]=6;do{if((c[k>>2]|0)==3&(c[k+4>>2]|0)==0){T=c[f>>2]|0;J=c[f+4>>2]|0;I=1599;e=0;if((T|0)==1551&(J|0)==0|(T|0)==63&(J|0)==0){if(+g[b+6992>>2]!=+g[b+7120>>2]){P=4450;break}if(+g[b+6996>>2]!=+g[b+7124>>2]){P=4450;break}if(+g[b+6988>>2]!=0.0){P=4450;break}if(+g[b+7004>>2]!=0.0){P=4450;break}if(+g[b+7112>>2]!=0.0){P=4450;break}if(+g[b+7128>>2]!=0.0){P=4450;break}else{U=86;break}}else if(!((T|0)==(I|0)&(J|0)==(e|0))){P=4450;break}if(+g[b+6992>>2]!=+g[b+7120>>2]){P=4450;break}if(+g[b+6996>>2]!=+g[b+7124>>2]){P=4450;break}if(+g[b+6988>>2]!=0.0){P=4450;break}if(+g[b+7004>>2]!=0.0){P=4450;break}if(+g[b+7112>>2]!=0.0){P=4450;break}if(+g[b+7128>>2]!=0.0){P=4450;break}if(+g[b+7012>>2]!=0.0){P=4450;break}if(+g[b+7136>>2]!=0.0){P=4450}else{U=66}}else{P=4450}}while(0);if((P|0)==4450){U=0}c[m>>2]=U;V=0}else if((Q|0)==8){U=b+11080|0;c[U>>2]=A8(ac(l,j)|0,4)|0;e=A7(4)|0;J=b+11084|0;c[J>>2]=e;if((l|0)>0&(j|0)>0){I=0;do{T=ac(I,j)|0;O=0;do{g[(c[U>>2]|0)+(O+T<<2)>>2]=+g[b+6984+(I<<7)+(O<<2)>>2];O=O+1|0;}while((O|0)<(j|0));I=I+1|0;}while((I|0)<(l|0));W=c[J>>2]|0}else{W=e}g[W>>2]=1.0;c[b+16248>>2]=16;c[b+16256>>2]=136;do{if((c[k>>2]|0)==3&(c[k+4>>2]|0)==0){W=c[f>>2]|0;e=c[f+4>>2]|0;J=1599;I=0;if((W|0)==1551&(e|0)==0|(W|0)==63&(e|0)==0){if(+g[b+6992>>2]!=+g[b+7120>>2]){P=4473;break}if(+g[b+6996>>2]!=+g[b+7124>>2]){P=4473;break}if(+g[b+6988>>2]!=0.0){P=4473;break}if(+g[b+7004>>2]!=0.0){P=4473;break}if(+g[b+7112>>2]!=0.0){P=4473;break}if(+g[b+7128>>2]!=0.0){P=4473;break}else{X=58;break}}else if(!((W|0)==(J|0)&(e|0)==(I|0))){P=4473;break}if(+g[b+6992>>2]!=+g[b+7120>>2]){P=4473;break}if(+g[b+6996>>2]!=+g[b+7124>>2]){P=4473;break}if(+g[b+6988>>2]!=0.0){P=4473;break}if(+g[b+7004>>2]!=0.0){P=4473;break}if(+g[b+7112>>2]!=0.0){P=4473;break}if(+g[b+7128>>2]!=0.0){P=4473;break}if(+g[b+7012>>2]!=0.0){P=4473;break}if(+g[b+7136>>2]!=0.0){P=4473}else{X=74}}else{P=4473}}while(0);if((P|0)==4473){X=0}c[m>>2]=X;V=0}else if((Q|0)==9){X=b+11080|0;c[X>>2]=A8(ac(l,j)|0,8)|0;I=A7(8)|0;e=b+11084|0;c[e>>2]=I;if((l|0)>0&(j|0)>0){J=0;do{W=ac(J,j)|0;U=0;do{h[(c[X>>2]|0)+(U+W<<3)>>3]=+g[b+6984+(J<<7)+(U<<2)>>2];U=U+1|0;}while((U|0)<(j|0));J=J+1|0;}while((J|0)<(l|0));Y=c[e>>2]|0}else{Y=I}h[Y>>3]=1.0;c[b+16248>>2]=88;c[b+16256>>2]=72;do{if((c[k>>2]|0)==3&(c[k+4>>2]|0)==0){Y=c[f>>2]|0;I=c[f+4>>2]|0;e=1599;l=0;if((Y|0)==1551&(I|0)==0|(Y|0)==63&(I|0)==0){if(+g[b+6992>>2]!=+g[b+7120>>2]){P=4496;break}if(+g[b+6996>>2]!=+g[b+7124>>2]){P=4496;break}if(+g[b+6988>>2]!=0.0){P=4496;break}if(+g[b+7004>>2]!=0.0){P=4496;break}if(+g[b+7112>>2]!=0.0){P=4496;break}if(+g[b+7128>>2]!=0.0){P=4496;break}else{Z=10;break}}else if(!((Y|0)==(e|0)&(I|0)==(l|0))){P=4496;break}if(+g[b+6992>>2]!=+g[b+7120>>2]){P=4496;break}if(+g[b+6996>>2]!=+g[b+7124>>2]){P=4496;break}if(+g[b+6988>>2]!=0.0){P=4496;break}if(+g[b+7004>>2]!=0.0){P=4496;break}if(+g[b+7112>>2]!=0.0){P=4496;break}if(+g[b+7128>>2]!=0.0){P=4496;break}if(+g[b+7012>>2]!=0.0){P=4496;break}if(+g[b+7136>>2]!=0.0){P=4496}else{Z=30}}else{P=4496}}while(0);if((P|0)==4496){Z=0}c[m>>2]=Z;V=0}else if((Q|0)==7){Q=A7(4)|0;c[b+11084>>2]=Q;c[Q>>2]=32768;c[b+16248>>2]=66;c[b+16256>>2]=94;do{if((c[k>>2]|0)==3&(c[k+4>>2]|0)==0){Q=c[f>>2]|0;Z=c[f+4>>2]|0;l=1599;I=0;if((Q|0)==1551&(Z|0)==0|(Q|0)==63&(Z|0)==0){if(+g[b+6992>>2]!=+g[b+7120>>2]){P=4514;break}if(+g[b+6996>>2]!=+g[b+7124>>2]){P=4514;break}if(+g[b+6988>>2]!=0.0){P=4514;break}if(+g[b+7004>>2]!=0.0){P=4514;break}if(+g[b+7112>>2]!=0.0){P=4514;break}if(+g[b+7128>>2]!=0.0){P=4514;break}else{_=2;break}}else if(!((Q|0)==(l|0)&(Z|0)==(I|0))){P=4514;break}if(+g[b+6992>>2]!=+g[b+7120>>2]){P=4514;break}if(+g[b+6996>>2]!=+g[b+7124>>2]){P=4514;break}if(+g[b+6988>>2]!=0.0){P=4514;break}if(+g[b+7004>>2]!=0.0){P=4514;break}if(+g[b+7112>>2]!=0.0){P=4514;break}if(+g[b+7128>>2]!=0.0){P=4514;break}if(+g[b+7012>>2]!=0.0){P=4514;break}if(+g[b+7136>>2]!=0.0){P=4514}else{_=14}}else{P=4514}}while(0);if((P|0)==4514){_=0}c[m>>2]=_;V=0}else{AN(0,0,1643368,(u=i,i=i+24|0,c[u>>2]=1719592,c[u+8>>2]=1691344,c[u+16>>2]=385,u)|0);i=u;cw();return 0}while(1){u=0;_=0;while(1){m=b+6984+(V<<7)+(u<<2)|0;c[b+11096+(V<<7)+(u<<2)>>2]=bR(+(+g[m>>2]*32768.0))|0;if(+g[m>>2]!=0.0){m=_+1|0;a[b+15192+(V*33|0)+m|0]=u&255;$=m}else{$=_}m=u+1|0;if((m|0)<32){u=m;_=$}else{break}}a[b+15192+(V*33|0)|0]=$&255;_=V+1|0;if((_|0)<32){V=_}else{S=0;break}}i=d;return S|0}function sB(a,d,e,f,g){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;h=c[e+(f<<2)>>2]|0;if((g|0)>0){i=0}else{return}do{b[a+(i<<1)>>1]=((ac(b[d+(i<<1)>>1]|0,h)|0)+16384|0)>>>15&65535;i=i+1|0;}while((i|0)<(g|0));return}function sC(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0;j=c[f+(g<<2)>>2]|0;g=c[f+(h<<2)>>2]|0;if((i|0)>0){k=0}else{return}do{h=ac(b[d+(k<<1)>>1]|0,j)|0;b[a+(k<<1)>>1]=(h+16384+(ac(b[e+(k<<1)>>1]|0,g)|0)|0)>>>15&65535;k=k+1|0;}while((k|0)<(i|0));return}function sD(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0.0,h=0;f=+g[c+(d<<2)>>2];if((e|0)>0){h=0}else{return}do{g[a+(h<<2)>>2]=f*+g[b+(h<<2)>>2];h=h+1|0;}while((h|0)<(e|0));return}function sE(a,b,c,d,e,f,h){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;h=h|0;var i=0.0,j=0.0,k=0;i=+g[d+(e<<2)>>2];j=+g[d+(f<<2)>>2];if((h|0)>0){k=0}else{return}do{g[a+(k<<2)>>2]=i*+g[b+(k<<2)>>2]+j*+g[c+(k<<2)>>2];k=k+1|0;}while((k|0)<(h|0));return}function sF(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;var f=0.0,g=0;f=+h[c+(d<<3)>>3];if((e|0)>0){g=0}else{return}do{h[a+(g<<3)>>3]=f*+h[b+(g<<3)>>3];g=g+1|0;}while((g|0)<(e|0));return}function sG(a,b,c,d,e,f,g){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var i=0.0,j=0.0,k=0;i=+h[d+(e<<3)>>3];j=+h[d+(f<<3)>>3];if((g|0)>0){k=0}else{return}do{h[a+(k<<3)>>3]=i*+h[b+(k<<3)>>3]+j*+h[c+(k<<3)>>3];k=k+1|0;}while((k|0)<(g|0));return}function sH(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0;g=c[d+(e<<2)>>2]|0;e=g;d=(g|0)<0|0?-1:0;if((f|0)>0){h=0}else{return}do{g=c[b+(h<<2)>>2]|0;i=N2(g,(g|0)<0|0?-1:0,e,d)|0;g=NU(i,G,16384,0)|0;c[a+(h<<2)>>2]=g>>>15|G<<17;h=h+1|0;}while((h|0)<(f|0));return}function sI(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0;i=c[e+(f<<2)>>2]|0;f=i;j=(i|0)<0|0?-1:0;i=c[e+(g<<2)>>2]|0;g=i;e=(i|0)<0|0?-1:0;if((h|0)>0){k=0}else{return}do{i=c[b+(k<<2)>>2]|0;l=N2(i,(i|0)<0|0?-1:0,f,j)|0;i=G;m=c[d+(k<<2)>>2]|0;n=N2(m,(m|0)<0|0?-1:0,g,e)|0;m=G;o=NU(l,i,16384,0)|0;i=NU(o,G,n,m)|0;c[a+(k<<2)>>2]=i>>>15|G<<17;k=k+1|0;}while((k|0)<(h|0));return}function sJ(a){a=a|0;A6(a+11080|0);A6(a+11084|0);A6(a+11092|0);A6(a+11088|0);return}function sK(e,f,j,k,l){e=e|0;f=f|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0.0,R=0.0,S=0.0,T=0.0,U=0;m=i;n=c[e+16264>>2]|0;if((n|0)!=0){dq[n&127](f|0,j|0,c[e+11080>>2]|0,k);i=m;return 0}n=e+16260|0;if((c[n>>2]|0)==0){if((c[e+16252>>2]|0)==0){o=0;p=0}else{q=4571}}else{q=4571}if((q|0)==4571){q=k&-16;o=ac(c[f+136>>2]|0,q)|0;p=q}q=f+132|0;r=c[q>>2]|0;s=e+32|0;if((r|0)!=(zs(c[s>>2]|0,c[s+4>>2]|0)|0)){AN(0,0,1643368,(t=i,i=i+24|0,c[t>>2]=1671440,c[t+8>>2]=1691344,c[t+16>>2]=424,t)|0);i=t;cw();return 0}s=j+132|0;r=c[s>>2]|0;u=e+24|0;if((r|0)!=(zs(c[u>>2]|0,c[u+4>>2]|0)|0)){AN(0,0,1643368,(t=i,i=i+24|0,c[t>>2]=1656928,c[t+8>>2]=1691344,c[t+16>>2]=425,t)|0);i=t;cw();return 0}if((c[q>>2]|0)<=0){i=m;return 0}t=e+16|0;u=(k|0)>0;r=(l|0)==0;l=e+16252|0;v=(p|0)==0;w=(p|0)==(k|0);x=e+16248|0;y=e+11080|0;z=k-p|0;A=e+11092|0;B=f+136|0;C=e+16256|0;D=0;do{E=e+15192+(D*33|0)|0;F=a[E]|0;G=F&255;L6342:do{if((G|0)==1){H=d[e+15192+(D*33|0)+1|0]|0;if(+g[e+6984+(D<<7)+(H<<2)>>2]==1.0){if(r){c[f+(D<<2)>>2]=c[j+(H<<2)>>2];break}else{I=c[f+(D<<2)>>2]|0;J=c[j+(H<<2)>>2]|0;K=ac(c[B>>2]|0,k)|0;NN(I|0,J|0,K)|0;break}}K=c[l>>2]|0;if(!((K|0)==0|v)){J=c[f+(D<<2)>>2]|0;I=c[j+(H<<2)>>2]|0;L=c[A>>2]|0;M=(ac(c[s>>2]|0,D)|0)+H|0;cX[K&255](J,I,L,M,p)}if(w){break}M=c[x>>2]|0;L=(c[f+(D<<2)>>2]|0)+o|0;I=(c[j+(H<<2)>>2]|0)+o|0;J=c[y>>2]|0;K=(ac(c[s>>2]|0,D)|0)+H|0;cX[M&255](L,I,J,K,z)}else if((G|0)==0){if(r){break}K=c[f+(D<<2)>>2]|0;NM(K|0,0,ac(B7(c[t>>2]|0)|0,k)|0)}else if((G|0)==2){K=d[e+15192+(D*33|0)+1|0]|0;J=d[e+15192+(D*33|0)+2|0]|0;I=c[n>>2]|0;if((I|0)==0|v){L=c[C>>2]|0;M=c[f+(D<<2)>>2]|0;H=c[j+(K<<2)>>2]|0;N=c[j+(J<<2)>>2]|0;O=c[y>>2]|0;P=ac(c[s>>2]|0,D)|0;dc[L&255](M,H,N,O,P+K|0,P+J|0,p)}else{P=c[f+(D<<2)>>2]|0;O=c[j+(K<<2)>>2]|0;N=c[j+(J<<2)>>2]|0;H=c[A>>2]|0;M=ac(c[s>>2]|0,D)|0;dc[I&255](P,O,N,H,M+K|0,M+J|0,p)}if(w){break}M=c[C>>2]|0;H=(c[f+(D<<2)>>2]|0)+o|0;N=(c[j+(K<<2)>>2]|0)+o|0;O=(c[j+(J<<2)>>2]|0)+o|0;P=c[y>>2]|0;I=ac(c[s>>2]|0,D)|0;dc[M&255](H,N,O,P,I+K|0,I+J|0,z)}else{J=c[t>>2]|0;if((J|0)==9){if(!u){break}I=f+(D<<2)|0;K=0;P=F;while(1){if(P<<24>>24==0){Q=0.0}else{O=P&255;N=0;R=0.0;while(1){H=N+1|0;M=d[e+15192+(D*33|0)+H|0]|0;S=R+ +h[(c[j+(M<<2)>>2]|0)+(K<<3)>>3]*+g[e+6984+(D<<7)+(M<<2)>>2];if((H|0)<(O|0)){N=H;R=S}else{Q=S;break}}}h[(c[I>>2]|0)+(K<<3)>>3]=Q;N=K+1|0;if((N|0)>=(k|0)){break L6342}K=N;P=a[E]|0}}else if((J|0)==8){if(!u){break}P=f+(D<<2)|0;K=0;I=F;while(1){if(I<<24>>24==0){T=0.0}else{N=I&255;O=0;R=0.0;while(1){H=O+1|0;M=d[e+15192+(D*33|0)+H|0]|0;S=R+ +g[(c[j+(M<<2)>>2]|0)+(K<<2)>>2]*+g[e+6984+(D<<7)+(M<<2)>>2];if((H|0)<(N|0)){O=H;R=S}else{T=S;break}}}g[(c[P>>2]|0)+(K<<2)>>2]=T;O=K+1|0;if((O|0)>=(k|0)){break L6342}K=O;I=a[E]|0}}else{if(!u){break}I=f+(D<<2)|0;K=0;P=F;while(1){if(P<<24>>24==0){U=0}else{J=P&255;O=0;N=0;do{O=O+1|0;H=d[e+15192+(D*33|0)+O|0]|0;N=(ac(b[(c[j+(H<<2)>>2]|0)+(K<<1)>>1]|0,c[e+11096+(D<<7)+(H<<2)>>2]|0)|0)+N|0;}while((O|0)<(J|0));U=(N+16384|0)>>>15&65535}b[(c[I>>2]|0)+(K<<1)>>1]=U;J=K+1|0;if((J|0)>=(k|0)){break L6342}K=J;P=a[E]|0}}}}while(0);D=D+1|0;}while((D|0)<(c[q>>2]|0));i=m;return 0}function sL(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;if((e|0)<=0){return}f=b+8|0;g=d+8|0;h=b+12|0;i=d+12|0;j=b+16|0;k=d+16|0;l=b+4|0;m=d+28|0;n=b+20|0;o=d+44|0;p=a+4|0;q=0;do{r=c[(c[f>>2]|0)+(q<<2)>>2]|0;s=c[g>>2]|0;t=N2(s,(s|0)<0|0?-1:0,r,(r|0)<0|0?-1:0)|0;r=G;s=c[(c[h>>2]|0)+(q<<2)>>2]|0;u=c[i>>2]|0;v=N2(u,(u|0)<0|0?-1:0,s,(s|0)<0|0?-1:0)|0;s=NU(v,G,t,r)|0;r=G;t=c[(c[b>>2]|0)+(q<<2)>>2]|0;v=c[d>>2]|0;u=N2(v,(v|0)<0|0?-1:0,t,(t|0)<0|0?-1:0)|0;t=G;v=c[(c[j>>2]|0)+(q<<2)>>2]|0;w=c[k>>2]|0;x=N2(w,(w|0)<0|0?-1:0,v,(v|0)<0|0?-1:0)|0;v=G;w=NU(s,r,16384,0)|0;r=G;s=NU(w,r,u,t)|0;t=NU(s,G,x,v)|0;c[(c[a>>2]|0)+(q<<2)>>2]=t>>>15|G<<17;t=c[(c[l>>2]|0)+(q<<2)>>2]|0;v=c[m>>2]|0;x=N2(v,(v|0)<0|0?-1:0,t,(t|0)<0|0?-1:0)|0;t=G;v=c[(c[n>>2]|0)+(q<<2)>>2]|0;s=c[o>>2]|0;u=N2(s,(s|0)<0|0?-1:0,v,(v|0)<0|0?-1:0)|0;v=G;s=NU(w,r,x,t)|0;t=NU(s,G,u,v)|0;c[(c[p>>2]|0)+(q<<2)>>2]=t>>>15|G<<17;q=q+1|0;}while((q|0)<(e|0));return}function sM(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;if((e|0)<=0){return}f=b+8|0;g=d+8|0;h=b+12|0;i=d+12|0;j=b+16|0;k=d+16|0;l=b+24|0;m=d+24|0;n=b+4|0;o=d+36|0;p=b+20|0;q=d+52|0;r=b+28|0;s=d+60|0;t=a+4|0;u=0;do{v=c[(c[f>>2]|0)+(u<<2)>>2]|0;w=c[g>>2]|0;x=N2(w,(w|0)<0|0?-1:0,v,(v|0)<0|0?-1:0)|0;v=G;w=c[(c[h>>2]|0)+(u<<2)>>2]|0;y=c[i>>2]|0;z=N2(y,(y|0)<0|0?-1:0,w,(w|0)<0|0?-1:0)|0;w=NU(z,G,x,v)|0;v=G;x=c[(c[b>>2]|0)+(u<<2)>>2]|0;z=c[d>>2]|0;y=N2(z,(z|0)<0|0?-1:0,x,(x|0)<0|0?-1:0)|0;x=G;z=c[(c[j>>2]|0)+(u<<2)>>2]|0;A=c[k>>2]|0;B=N2(A,(A|0)<0|0?-1:0,z,(z|0)<0|0?-1:0)|0;z=G;A=c[(c[l>>2]|0)+(u<<2)>>2]|0;C=c[m>>2]|0;D=N2(C,(C|0)<0|0?-1:0,A,(A|0)<0|0?-1:0)|0;A=G;C=NU(w,v,16384,0)|0;v=G;w=NU(C,v,y,x)|0;x=NU(w,G,B,z)|0;z=NU(x,G,D,A)|0;c[(c[a>>2]|0)+(u<<2)>>2]=z>>>15|G<<17;z=c[(c[n>>2]|0)+(u<<2)>>2]|0;A=c[o>>2]|0;D=N2(A,(A|0)<0|0?-1:0,z,(z|0)<0|0?-1:0)|0;z=G;A=c[(c[p>>2]|0)+(u<<2)>>2]|0;x=c[q>>2]|0;B=N2(x,(x|0)<0|0?-1:0,A,(A|0)<0|0?-1:0)|0;A=G;x=c[(c[r>>2]|0)+(u<<2)>>2]|0;w=c[s>>2]|0;y=N2(w,(w|0)<0|0?-1:0,x,(x|0)<0|0?-1:0)|0;x=G;w=NU(C,v,D,z)|0;z=NU(w,G,B,A)|0;A=NU(z,G,y,x)|0;c[(c[t>>2]|0)+(u<<2)>>2]=A>>>15|G<<17;u=u+1|0;}while((u|0)<(e|0));return}function sN(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0.0;if((e|0)<=0){return}f=b+8|0;g=d+16|0;i=b+12|0;j=d+24|0;k=b+16|0;l=d+32|0;m=b+4|0;n=d+56|0;o=b+20|0;p=d+88|0;q=a+4|0;r=0;do{s=+h[(c[f>>2]|0)+(r<<3)>>3]*+h[g>>3]+ +h[(c[i>>2]|0)+(r<<3)>>3]*+h[j>>3];h[(c[a>>2]|0)+(r<<3)>>3]=s+ +h[(c[b>>2]|0)+(r<<3)>>3]*+h[d>>3]+ +h[(c[k>>2]|0)+(r<<3)>>3]*+h[l>>3];h[(c[q>>2]|0)+(r<<3)>>3]=s+ +h[(c[m>>2]|0)+(r<<3)>>3]*+h[n>>3]+ +h[(c[o>>2]|0)+(r<<3)>>3]*+h[p>>3];r=r+1|0;}while((r|0)<(e|0));return}function sO(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0.0;if((e|0)<=0){return}f=b+8|0;g=d+16|0;i=b+12|0;j=d+24|0;k=b+16|0;l=d+32|0;m=b+24|0;n=d+48|0;o=b+4|0;p=d+72|0;q=b+20|0;r=d+104|0;s=b+28|0;t=d+120|0;u=a+4|0;v=0;do{w=+h[(c[f>>2]|0)+(v<<3)>>3]*+h[g>>3]+ +h[(c[i>>2]|0)+(v<<3)>>3]*+h[j>>3];h[(c[a>>2]|0)+(v<<3)>>3]=w+ +h[(c[b>>2]|0)+(v<<3)>>3]*+h[d>>3]+ +h[(c[k>>2]|0)+(v<<3)>>3]*+h[l>>3]+ +h[(c[m>>2]|0)+(v<<3)>>3]*+h[n>>3];h[(c[u>>2]|0)+(v<<3)>>3]=w+ +h[(c[o>>2]|0)+(v<<3)>>3]*+h[p>>3]+ +h[(c[q>>2]|0)+(v<<3)>>3]*+h[r>>3]+ +h[(c[s>>2]|0)+(v<<3)>>3]*+h[t>>3];v=v+1|0;}while((v|0)<(e|0));return}function sP(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0.0;if((e|0)<=0){return}f=b+8|0;h=d+8|0;i=b+12|0;j=d+12|0;k=b+16|0;l=d+16|0;m=b+4|0;n=d+28|0;o=b+20|0;p=d+44|0;q=a+4|0;r=0;do{s=+g[(c[f>>2]|0)+(r<<2)>>2]*+g[h>>2]+ +g[(c[i>>2]|0)+(r<<2)>>2]*+g[j>>2];g[(c[a>>2]|0)+(r<<2)>>2]=s+ +g[(c[b>>2]|0)+(r<<2)>>2]*+g[d>>2]+ +g[(c[k>>2]|0)+(r<<2)>>2]*+g[l>>2];g[(c[q>>2]|0)+(r<<2)>>2]=s+ +g[(c[m>>2]|0)+(r<<2)>>2]*+g[n>>2]+ +g[(c[o>>2]|0)+(r<<2)>>2]*+g[p>>2];r=r+1|0;}while((r|0)<(e|0));return}function sQ(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0.0;if((e|0)<=0){return}f=b+8|0;h=d+8|0;i=b+12|0;j=d+12|0;k=b+16|0;l=d+16|0;m=b+24|0;n=d+24|0;o=b+4|0;p=d+36|0;q=b+20|0;r=d+52|0;s=b+28|0;t=d+60|0;u=a+4|0;v=0;do{w=+g[(c[f>>2]|0)+(v<<2)>>2]*+g[h>>2]+ +g[(c[i>>2]|0)+(v<<2)>>2]*+g[j>>2];g[(c[a>>2]|0)+(v<<2)>>2]=w+ +g[(c[b>>2]|0)+(v<<2)>>2]*+g[d>>2]+ +g[(c[k>>2]|0)+(v<<2)>>2]*+g[l>>2]+ +g[(c[m>>2]|0)+(v<<2)>>2]*+g[n>>2];g[(c[u>>2]|0)+(v<<2)>>2]=w+ +g[(c[o>>2]|0)+(v<<2)>>2]*+g[p>>2]+ +g[(c[q>>2]|0)+(v<<2)>>2]*+g[r>>2]+ +g[(c[s>>2]|0)+(v<<2)>>2]*+g[t>>2];v=v+1|0;}while((v|0)<(e|0));return}function sR(a,d,e,f){a=a|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;if((f|0)<=0){return}g=d+8|0;h=e+8|0;i=d+12|0;j=e+12|0;k=d+16|0;l=e+16|0;m=d+4|0;n=e+28|0;o=d+20|0;p=e+44|0;q=a+4|0;r=0;do{s=ac(b[(c[g>>2]|0)+(r<<1)>>1]|0,c[h>>2]|0)|0;t=(ac(b[(c[i>>2]|0)+(r<<1)>>1]|0,c[j>>2]|0)|0)+s|0;s=ac(b[(c[d>>2]|0)+(r<<1)>>1]|0,c[e>>2]|0)|0;u=t+16384|0;t=(u+s+(ac(b[(c[k>>2]|0)+(r<<1)>>1]|0,c[l>>2]|0)|0)|0)>>>15&65535;b[(c[a>>2]|0)+(r<<1)>>1]=t;t=ac(b[(c[m>>2]|0)+(r<<1)>>1]|0,c[n>>2]|0)|0;s=(u+t+(ac(b[(c[o>>2]|0)+(r<<1)>>1]|0,c[p>>2]|0)|0)|0)>>>15&65535;b[(c[q>>2]|0)+(r<<1)>>1]=s;r=r+1|0;}while((r|0)<(f|0));return}function sS(a,d,e,f){a=a|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;if((f|0)<=0){return}g=d+8|0;h=e+8|0;i=d+12|0;j=e+12|0;k=d+16|0;l=e+16|0;m=d+24|0;n=e+24|0;o=d+4|0;p=e+36|0;q=d+20|0;r=e+52|0;s=d+28|0;t=e+60|0;u=a+4|0;v=0;do{w=ac(b[(c[g>>2]|0)+(v<<1)>>1]|0,c[h>>2]|0)|0;x=(ac(b[(c[i>>2]|0)+(v<<1)>>1]|0,c[j>>2]|0)|0)+w|0;w=ac(b[(c[d>>2]|0)+(v<<1)>>1]|0,c[e>>2]|0)|0;y=ac(b[(c[k>>2]|0)+(v<<1)>>1]|0,c[l>>2]|0)|0;z=x+16384|0;x=(z+w+y+(ac(b[(c[m>>2]|0)+(v<<1)>>1]|0,c[n>>2]|0)|0)|0)>>>15&65535;b[(c[a>>2]|0)+(v<<1)>>1]=x;x=ac(b[(c[o>>2]|0)+(v<<1)>>1]|0,c[p>>2]|0)|0;y=ac(b[(c[q>>2]|0)+(v<<1)>>1]|0,c[r>>2]|0)|0;w=(z+x+y+(ac(b[(c[s>>2]|0)+(v<<1)>>1]|0,c[t>>2]|0)|0)|0)>>>15&65535;b[(c[u>>2]|0)+(v<<1)>>1]=w;v=v+1|0;}while((v|0)<(f|0));return}function sT(a,d,e,f,g,h,i){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0;j=a+24|0;k=c[j>>2]|0;l=a+28|0;m=c[l>>2]|0;n=a+20|0;o=c[n>>2]|0;p=a+32|0;q=c[p>>2]|0;r=(o|0)%(q|0)|0;s=(o|0)/(q|0)|0;t=a+36|0;u=c[t>>2]|0;v=a+8|0;L6454:do{if((u|0)==0){w=c[v>>2]|0;do{if((w|0)==1){x=a+40|0;if((c[x>>2]|0)!=0){break}y=o;z=q;A=(q|0)<0|0?-1:0;B=N0(0,y,z,A)|0;C=G;D=(h|0)<0|0?-1:0;E=g-1-k|0;F=N2(z,A,E,(E|0)<0|0?-1:0)|0;E=N0(F,G,y,(o|0)<0|0?-1:0)|0;y=G;F=(D|0)>(y|0)|(D|0)==(y|0)&h>>>0>E>>>0?E:h;if((F|0)>0){E=k;y=0;D=0;while(1){b[d+(D<<1)>>1]=b[e+(E<<1)>>1]|0;A=NU(y,E,B,C)|0;z=D+1|0;if((z|0)<(F|0)){E=G;y=A;D=z}else{break}}H=F;I=c[p>>2]|0;J=c[x>>2]|0}else{H=0;I=q;J=0}D=(ac(H,s)|0)+k|0;y=N2(H,(H|0)<0|0?-1:0,r,(r|0)<0|0?-1:0)|0;E=NU(y,G,m,(m|0)<0|0?-1:0)|0;y=G;C=I;B=(I|0)<0|0?-1:0;z=N0(E,y,C,B)|0;A=NU(z,G,D,0)|0;D=A;A=N1(E,y,C,B)|0;c[f>>2]=D>>J;K=H;L=0;M=s;N=r;O=A;P=c[a+44>>2]&D;break L6454}}while(0);if(!((c[a+48>>2]|0)==0&(k|0)>-1)){Q=0;R=4678;break}D=a+40|0;L6467:do{if((h|0)>0){A=a+44|0;B=a+4|0;C=a+12|0;y=k;E=m;z=0;S=0;T=w;while(1){U=(y>>c[D>>2])+z|0;V=c[A>>2]&y;W=c[B>>2]|0;X=ac(c[C>>2]|0,V)|0;if((T+U|0)>(g|0)){Y=U;Z=V;_=E;$=S;break L6467}do{if((T|0)>0){aa=0;ab=0;do{ab=(ac(b[W+(aa+X<<1)>>1]|0,b[e+(aa+U<<1)>>1]|0)|0)+ab|0;aa=aa+1|0;}while((aa|0)<(T|0));aa=ab+16384|0;ad=aa>>15;if((ad+32768|0)>>>0<=65535){ae=ad&65535;break}ae=(aa>>31^32767)&65535}else{ae=0}}while(0);b[d+(S<<1)>>1]=ae;X=E+r|0;W=V+s|0;aa=c[p>>2]|0;if((X|0)<(aa|0)){af=X;ag=W}else{af=X-aa|0;ag=W+1|0}W=S+1|0;if((W|0)>=(h|0)){Y=U;Z=ag;_=af;$=W;break L6467}y=ag;E=af;z=U;S=W;T=c[v>>2]|0}}else{Y=0;Z=k;_=m;$=0}}while(0);c[f>>2]=Y;K=$;L=0;M=s;N=r;O=_;P=Z}else{Q=u;R=4678}}while(0);if((R|0)==4678){u=a+40|0;Z=a+44|0;_=a+4|0;$=a+12|0;Y=a+48|0;af=a+16|0;a=0;ag=0;ae=Q;Q=s;s=r;r=m;m=k;k=q;L6485:while(1){q=a;ah=ag;ai=r;H=m;J=k;while(1){if((ah|0)>=(h|0)){aj=q;ak=H;break L6485}al=(H>>c[u>>2])+q|0;I=c[Z>>2]&H;o=c[_>>2]|0;D=c[$>>2]|0;w=ac(D,I)|0;T=c[v>>2]|0;if(!((T+al|0)<=(g|0)&(-al|0)<(g|0))){aj=al;ak=I;break L6485}L6491:do{if((al|0)<0){if((T|0)>0){am=0;an=0}else{ao=0;break}while(1){S=am+al|0;z=(ac(b[o+(am+w<<1)>>1]|0,b[e+(((S|0)>-1?S:-S|0)<<1)>>1]|0)|0)+an|0;S=am+1|0;if((S|0)<(T|0)){am=S;an=z}else{ap=z;R=4703;break}}}else{U=(T|0)>0;if((c[Y>>2]|0)==0){if(U){aq=0;ar=0}else{ao=0;break}while(1){V=(ac(b[o+(aq+w<<1)>>1]|0,b[e+(aq+al<<1)>>1]|0)|0)+ar|0;z=aq+1|0;if((z|0)<(T|0)){aq=z;ar=V}else{ap=V;R=4703;break L6491}}}if(U){V=0;z=0;S=0;while(1){E=b[e+(V+al<<1)>>1]|0;y=V+w|0;C=(ac(b[o+(y<<1)>>1]|0,E)|0)+z|0;B=(ac(b[o+(y+D<<1)>>1]|0,E)|0)+S|0;E=V+1|0;if((E|0)<(T|0)){V=E;z=C;S=B}else{as=C;at=B;break}}}else{as=0;at=0}S=at-as|0;z=N2(S,(S|0)<0|0?-1:0,ai,(ai|0)<0|0?-1:0)|0;S=N0(z,G,J,(J|0)<0|0?-1:0)|0;z=NU(S,G,as,0)|0;ap=z;R=4703}}while(0);do{if((R|0)==4703){R=0;T=ap+16384|0;D=T>>15;if((D+32768|0)>>>0<=65535){ao=D&65535;break}ao=(T>>31^32767)&65535}}while(0);b[d+(ah<<1)>>1]=ao;T=s+ai|0;D=I+Q|0;au=c[p>>2]|0;if((T|0)<(au|0)){av=T;aw=D}else{av=T-au|0;aw=D+1|0}D=ah+1|0;if((D|0)==(ae|0)){break}else{q=al;ah=D;ai=av;H=aw;J=au}}J=c[af>>2]|0;a=al;ag=ae;ae=0;Q=(J|0)/(au|0)|0;s=(J|0)%(au|0)|0;r=av;m=aw;k=au}au=(aj|0)>0;c[f>>2]=au?aj:0;K=ah;L=(ae|0)==0?0:ae-ah|0;M=Q;N=s;O=ai;P=((au?0:aj)<<c[u>>2])+ak|0}if((i|0)==0){return K|0}c[l>>2]=O;c[j>>2]=P;c[n>>2]=(ac(c[p>>2]|0,M)|0)+N;c[t>>2]=L;return K|0}function sU(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0;i=a+24|0;j=c[i>>2]|0;k=a+28|0;l=c[k>>2]|0;m=a+20|0;n=c[m>>2]|0;o=a+32|0;p=c[o>>2]|0;q=(n|0)%(p|0)|0;r=(n|0)/(p|0)|0;s=a+36|0;t=c[s>>2]|0;u=a+8|0;L6520:do{if((t|0)==0){v=c[u>>2]|0;do{if((v|0)==1){w=a+40|0;if((c[w>>2]|0)!=0){break}x=n;y=p;z=(p|0)<0|0?-1:0;A=N0(0,x,y,z)|0;B=G;C=(g|0)<0|0?-1:0;D=f-1-j|0;E=N2(y,z,D,(D|0)<0|0?-1:0)|0;D=N0(E,G,x,(n|0)<0|0?-1:0)|0;x=G;E=(C|0)>(x|0)|(C|0)==(x|0)&g>>>0>D>>>0?D:g;if((E|0)>0){D=j;x=0;C=0;while(1){c[b+(C<<2)>>2]=c[d+(D<<2)>>2];z=NU(x,D,A,B)|0;y=C+1|0;if((y|0)<(E|0)){D=G;x=z;C=y}else{break}}F=E;H=c[o>>2]|0;I=c[w>>2]|0}else{F=0;H=p;I=0}C=(ac(F,r)|0)+j|0;x=N2(F,(F|0)<0|0?-1:0,q,(q|0)<0|0?-1:0)|0;D=NU(x,G,l,(l|0)<0|0?-1:0)|0;x=G;B=H;A=(H|0)<0|0?-1:0;y=N0(D,x,B,A)|0;z=NU(y,G,C,0)|0;C=z;z=N1(D,x,B,A)|0;c[e>>2]=C>>I;J=F;K=0;L=r;M=q;N=z;O=c[a+44>>2]&C;break L6520}}while(0);if(!((c[a+48>>2]|0)==0&(j|0)>-1)){P=0;Q=4724;break}C=a+40|0;L6533:do{if((g|0)>0){z=a+44|0;A=a+4|0;B=a+12|0;x=j;D=l;y=0;R=0;S=v;while(1){T=(x>>c[C>>2])+y|0;U=c[z>>2]&x;V=c[A>>2]|0;W=ac(c[B>>2]|0,U)|0;if((S+T|0)>(f|0)){X=T;Y=U;Z=D;_=R;break L6533}do{if((S|0)>0){$=0;aa=0;ab=0;do{ad=c[d+($+T<<2)>>2]|0;ae=c[V+($+W<<2)>>2]|0;af=N2(ae,(ae|0)<0|0?-1:0,ad,(ad|0)<0|0?-1:0)|0;ab=NU(af,G,ab,aa)|0;aa=G;$=$+1|0;}while(($|0)<(S|0));$=NU(ab,aa,536870912,0)|0;af=G;ad=$>>>30|af<<2;$=NU(ad,af>>30|((af|0)<0|0?-1:0)<<2,-2147483648,0)|0;ae=G;ag=0;if(!(ae>>>0>ag>>>0|ae>>>0==ag>>>0&$>>>0>-1>>>0)){ah=ad;break}((af|0)<0|0?-1:0)>>31|((af|0)<0|0?-1:0)<<1;ah=(af>>31|((af|0)<0|0?-1:0)<<1)^2147483647}else{ah=0}}while(0);c[b+(R<<2)>>2]=ah;W=D+q|0;V=U+r|0;af=c[o>>2]|0;if((W|0)<(af|0)){ai=W;aj=V}else{ai=W-af|0;aj=V+1|0}V=R+1|0;if((V|0)>=(g|0)){X=T;Y=aj;Z=ai;_=V;break L6533}x=aj;D=ai;y=T;R=V;S=c[u>>2]|0}}else{X=0;Y=j;Z=l;_=0}}while(0);c[e>>2]=X;J=_;K=0;L=r;M=q;N=Z;O=Y}else{P=t;Q=4724}}while(0);if((Q|0)==4724){t=a+40|0;Y=a+44|0;Z=a+4|0;_=a+12|0;X=a+48|0;ai=a+16|0;a=0;aj=0;ah=P;P=r;r=q;q=l;l=j;j=p;L6551:while(1){p=a;ak=aj;al=q;F=l;I=j;while(1){if((ak|0)>=(g|0)){am=p;an=F;break L6551}ao=(F>>c[t>>2])+p|0;H=c[Y>>2]&F;n=c[Z>>2]|0;C=c[_>>2]|0;v=ac(C,H)|0;S=c[u>>2]|0;if(!((S+ao|0)<=(f|0)&(-ao|0)<(f|0))){am=ao;an=H;break L6551}L6557:do{if((ao|0)<0){if((S|0)>0){ap=0;aq=0;ar=0}else{as=0;break}while(1){R=ap+ao|0;y=c[d+(((R|0)>-1?R:-R|0)<<2)>>2]|0;R=c[n+(ap+v<<2)>>2]|0;D=N2(R,(R|0)<0|0?-1:0,y,(y|0)<0|0?-1:0)|0;y=NU(D,G,ar,aq)|0;D=G;R=ap+1|0;if((R|0)<(S|0)){ap=R;aq=D;ar=y}else{at=D;au=y;Q=4749;break}}}else{T=(S|0)>0;if((c[X>>2]|0)==0){if(T){av=0;aw=0;ax=0}else{as=0;break}while(1){U=c[d+(av+ao<<2)>>2]|0;y=c[n+(av+v<<2)>>2]|0;D=N2(y,(y|0)<0|0?-1:0,U,(U|0)<0|0?-1:0)|0;U=NU(D,G,ax,aw)|0;D=G;y=av+1|0;if((y|0)<(S|0)){av=y;aw=D;ax=U}else{at=D;au=U;Q=4749;break L6557}}}if(T){U=0;D=0;y=0;R=0;x=0;while(1){B=c[d+(U+ao<<2)>>2]|0;A=B;z=(B|0)<0|0?-1:0;B=U+v|0;w=c[n+(B<<2)>>2]|0;E=N2(w,(w|0)<0|0?-1:0,A,z)|0;w=NU(E,G,y,D)|0;E=G;V=c[n+(B+C<<2)>>2]|0;B=N2(V,(V|0)<0|0?-1:0,A,z)|0;z=NU(B,G,x,R)|0;B=G;A=U+1|0;if((A|0)<(S|0)){U=A;D=E;y=w;R=B;x=z}else{ay=E;az=w;aA=B;aB=z;break}}}else{ay=0;az=0;aA=0;aB=0}x=NV(aB,aA,az,ay)|0;R=N2(x,G,al,(al|0)<0|0?-1:0)|0;x=N0(R,G,I,(I|0)<0|0?-1:0)|0;R=NU(x,G,az,ay)|0;at=G;au=R;Q=4749}}while(0);do{if((Q|0)==4749){Q=0;S=NU(au,at,536870912,0)|0;C=G;n=S>>>30|C<<2;S=NU(n,C>>30|((C|0)<0|0?-1:0)<<2,-2147483648,0)|0;v=G;R=0;if(!(v>>>0>R>>>0|v>>>0==R>>>0&S>>>0>-1>>>0)){as=n;break}n=C>>31|((C|0)<0|0?-1:0)<<1;((C|0)<0|0?-1:0)>>31|((C|0)<0|0?-1:0)<<1;as=n^2147483647}}while(0);c[b+(ak<<2)>>2]=as;n=r+al|0;C=H+P|0;aC=c[o>>2]|0;if((n|0)<(aC|0)){aD=n;aE=C}else{aD=n-aC|0;aE=C+1|0}C=ak+1|0;if((C|0)==(ah|0)){break}else{p=ao;ak=C;al=aD;F=aE;I=aC}}I=c[ai>>2]|0;a=ao;aj=ah;ah=0;P=(I|0)/(aC|0)|0;r=(I|0)%(aC|0)|0;q=aD;l=aE;j=aC}aC=(am|0)>0;c[e>>2]=aC?am:0;J=ak;K=(ah|0)==0?0:ah-ak|0;L=P;M=r;N=al;O=((aC?0:am)<<c[t>>2])+an|0}if((h|0)==0){return J|0}c[k>>2]=N;c[i>>2]=O;c[m>>2]=(ac(c[o>>2]|0,L)|0)+M;c[s>>2]=K;return J|0}function sV(a,b,d,e,f,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0.0,ad=0.0,ae=0,af=0.0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0.0,ap=0.0,aq=0,ar=0.0,as=0.0,at=0.0,au=0.0,av=0.0,aw=0,ax=0,ay=0;j=a+24|0;k=c[j>>2]|0;l=a+28|0;m=c[l>>2]|0;n=a+20|0;o=c[n>>2]|0;p=a+32|0;q=c[p>>2]|0;r=(o|0)%(q|0)|0;s=(o|0)/(q|0)|0;t=a+36|0;u=c[t>>2]|0;v=a+8|0;L6586:do{if((u|0)==0){w=c[v>>2]|0;do{if((w|0)==1){x=a+40|0;if((c[x>>2]|0)!=0){break}y=o;z=q;A=(q|0)<0|0?-1:0;B=N0(0,y,z,A)|0;C=G;D=(h|0)<0|0?-1:0;E=f-1-k|0;F=N2(z,A,E,(E|0)<0|0?-1:0)|0;E=N0(F,G,y,(o|0)<0|0?-1:0)|0;y=G;F=(D|0)>(y|0)|(D|0)==(y|0)&h>>>0>E>>>0?E:h;if((F|0)>0){E=k;y=0;D=0;while(1){g[b+(D<<2)>>2]=+g[d+(E<<2)>>2];A=NU(y,E,B,C)|0;z=D+1|0;if((z|0)<(F|0)){E=G;y=A;D=z}else{break}}H=F;I=c[p>>2]|0;J=c[x>>2]|0}else{H=0;I=q;J=0}D=(ac(H,s)|0)+k|0;y=N2(H,(H|0)<0|0?-1:0,r,(r|0)<0|0?-1:0)|0;E=NU(y,G,m,(m|0)<0|0?-1:0)|0;y=G;C=I;B=(I|0)<0|0?-1:0;z=N0(E,y,C,B)|0;A=NU(z,G,D,0)|0;D=A;A=N1(E,y,C,B)|0;c[e>>2]=D>>J;K=H;L=0;M=s;N=r;O=A;P=c[a+44>>2]&D;break L6586}}while(0);if(!((c[a+48>>2]|0)==0&(k|0)>-1)){Q=0;R=4770;break}D=a+40|0;L6599:do{if((h|0)>0){A=a+44|0;B=a+4|0;C=a+12|0;y=k;E=m;z=0;S=0;T=w;while(1){U=(y>>c[D>>2])+z|0;V=c[A>>2]&y;W=c[B>>2]|0;X=ac(c[C>>2]|0,V)|0;if((T+U|0)>(f|0)){Y=U;Z=V;_=E;$=S;break L6599}if((T|0)>0){aa=0;ab=0.0;while(1){ad=ab+ +g[d+(aa+U<<2)>>2]*+g[W+(aa+X<<2)>>2];ae=aa+1|0;if((ae|0)<(T|0)){aa=ae;ab=ad}else{af=ad;break}}}else{af=0.0}g[b+(S<<2)>>2]=af;aa=E+r|0;X=V+s|0;W=c[p>>2]|0;if((aa|0)<(W|0)){ag=aa;ah=X}else{ag=aa-W|0;ah=X+1|0}X=S+1|0;if((X|0)>=(h|0)){Y=U;Z=ah;_=ag;$=X;break L6599}y=ah;E=ag;z=U;S=X;T=c[v>>2]|0}}else{Y=0;Z=k;_=m;$=0}}while(0);c[e>>2]=Y;K=$;L=0;M=s;N=r;O=_;P=Z}else{Q=u;R=4770}}while(0);if((R|0)==4770){R=a+40|0;u=a+44|0;Z=a+4|0;_=a+12|0;$=a+48|0;Y=a+16|0;a=0;ag=0;ah=Q;Q=s;s=r;r=m;m=k;k=q;L6615:while(1){q=a;ai=ag;aj=r;H=m;J=k;while(1){if((ai|0)>=(h|0)){ak=q;al=H;break L6615}am=(H>>c[R>>2])+q|0;I=c[u>>2]&H;o=c[Z>>2]|0;D=c[_>>2]|0;w=ac(D,I)|0;T=c[v>>2]|0;if(!((T+am|0)<=(f|0)&(-am|0)<(f|0))){ak=am;al=I;break L6615}L6621:do{if((am|0)<0){if((T|0)>0){an=0;ao=0.0}else{ap=0.0;break}while(1){S=an+am|0;af=ao+ +g[d+(((S|0)>-1?S:-S|0)<<2)>>2]*+g[o+(an+w<<2)>>2];S=an+1|0;if((S|0)<(T|0)){an=S;ao=af}else{ap=af;break}}}else{U=(T|0)>0;if((c[$>>2]|0)==0){if(U){aq=0;ar=0.0}else{ap=0.0;break}while(1){af=ar+ +g[d+(aq+am<<2)>>2]*+g[o+(aq+w<<2)>>2];V=aq+1|0;if((V|0)<(T|0)){aq=V;ar=af}else{ap=af;break L6621}}}if(U){V=0;af=0.0;ab=0.0;while(1){ad=+g[d+(V+am<<2)>>2];S=V+w|0;as=af+ad*+g[o+(S<<2)>>2];at=ab+ad*+g[o+(S+D<<2)>>2];S=V+1|0;if((S|0)<(T|0)){V=S;af=as;ab=at}else{au=as;av=at;break}}}else{au=0.0;av=0.0}ap=au+ +(aj|0)*(av-au)/+(J|0)}}while(0);g[b+(ai<<2)>>2]=ap;T=s+aj|0;D=I+Q|0;aw=c[p>>2]|0;if((T|0)<(aw|0)){ax=T;ay=D}else{ax=T-aw|0;ay=D+1|0}D=ai+1|0;if((D|0)==(ah|0)){break}else{q=am;ai=D;aj=ax;H=ay;J=aw}}J=c[Y>>2]|0;a=am;ag=ah;ah=0;Q=(J|0)/(aw|0)|0;s=(J|0)%(aw|0)|0;r=ax;m=ay;k=aw}aw=(ak|0)>0;c[e>>2]=aw?ak:0;K=ai;L=(ah|0)==0?0:ah-ai|0;M=Q;N=s;O=aj;P=((aw?0:ak)<<c[R>>2])+al|0}if((i|0)==0){return K|0}c[l>>2]=O;c[j>>2]=P;c[n>>2]=(ac(c[p>>2]|0,M)|0)+N;c[t>>2]=L;return K|0}function sW(a,b,d,e,f,g,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0.0,ad=0.0,ae=0,af=0.0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0.0,ap=0.0,aq=0,ar=0.0,as=0.0,at=0.0,au=0.0,av=0.0,aw=0,ax=0,ay=0;j=a+24|0;k=c[j>>2]|0;l=a+28|0;m=c[l>>2]|0;n=a+20|0;o=c[n>>2]|0;p=a+32|0;q=c[p>>2]|0;r=(o|0)%(q|0)|0;s=(o|0)/(q|0)|0;t=a+36|0;u=c[t>>2]|0;v=a+8|0;L6647:do{if((u|0)==0){w=c[v>>2]|0;do{if((w|0)==1){x=a+40|0;if((c[x>>2]|0)!=0){break}y=o;z=q;A=(q|0)<0|0?-1:0;B=N0(0,y,z,A)|0;C=G;D=(g|0)<0|0?-1:0;E=f-1-k|0;F=N2(z,A,E,(E|0)<0|0?-1:0)|0;E=N0(F,G,y,(o|0)<0|0?-1:0)|0;y=G;F=(D|0)>(y|0)|(D|0)==(y|0)&g>>>0>E>>>0?E:g;if((F|0)>0){E=k;y=0;D=0;while(1){h[b+(D<<3)>>3]=+h[d+(E<<3)>>3];A=NU(y,E,B,C)|0;z=D+1|0;if((z|0)<(F|0)){E=G;y=A;D=z}else{break}}H=F;I=c[p>>2]|0;J=c[x>>2]|0}else{H=0;I=q;J=0}D=(ac(H,s)|0)+k|0;y=N2(H,(H|0)<0|0?-1:0,r,(r|0)<0|0?-1:0)|0;E=NU(y,G,m,(m|0)<0|0?-1:0)|0;y=G;C=I;B=(I|0)<0|0?-1:0;z=N0(E,y,C,B)|0;A=NU(z,G,D,0)|0;D=A;A=N1(E,y,C,B)|0;c[e>>2]=D>>J;K=H;L=0;M=s;N=r;O=A;P=c[a+44>>2]&D;break L6647}}while(0);if(!((c[a+48>>2]|0)==0&(k|0)>-1)){Q=0;R=4812;break}D=a+40|0;L6660:do{if((g|0)>0){A=a+44|0;B=a+4|0;C=a+12|0;y=k;E=m;z=0;S=0;T=w;while(1){U=(y>>c[D>>2])+z|0;V=c[A>>2]&y;W=c[B>>2]|0;X=ac(c[C>>2]|0,V)|0;if((T+U|0)>(f|0)){Y=U;Z=V;_=E;$=S;break L6660}if((T|0)>0){aa=0;ab=0.0;while(1){ad=ab+ +h[d+(aa+U<<3)>>3]*+h[W+(aa+X<<3)>>3];ae=aa+1|0;if((ae|0)<(T|0)){aa=ae;ab=ad}else{af=ad;break}}}else{af=0.0}h[b+(S<<3)>>3]=af;aa=E+r|0;X=V+s|0;W=c[p>>2]|0;if((aa|0)<(W|0)){ag=aa;ah=X}else{ag=aa-W|0;ah=X+1|0}X=S+1|0;if((X|0)>=(g|0)){Y=U;Z=ah;_=ag;$=X;break L6660}y=ah;E=ag;z=U;S=X;T=c[v>>2]|0}}else{Y=0;Z=k;_=m;$=0}}while(0);c[e>>2]=Y;K=$;L=0;M=s;N=r;O=_;P=Z}else{Q=u;R=4812}}while(0);if((R|0)==4812){R=a+40|0;u=a+44|0;Z=a+4|0;_=a+12|0;$=a+48|0;Y=a+16|0;a=0;ag=0;ah=Q;Q=s;s=r;r=m;m=k;k=q;L6676:while(1){q=a;ai=ag;aj=r;H=m;J=k;while(1){if((ai|0)>=(g|0)){ak=q;al=H;break L6676}am=(H>>c[R>>2])+q|0;I=c[u>>2]&H;o=c[Z>>2]|0;D=c[_>>2]|0;w=ac(D,I)|0;T=c[v>>2]|0;if(!((T+am|0)<=(f|0)&(-am|0)<(f|0))){ak=am;al=I;break L6676}L6682:do{if((am|0)<0){if((T|0)>0){an=0;ao=0.0}else{ap=0.0;break}while(1){S=an+am|0;af=ao+ +h[d+(((S|0)>-1?S:-S|0)<<3)>>3]*+h[o+(an+w<<3)>>3];S=an+1|0;if((S|0)<(T|0)){an=S;ao=af}else{ap=af;break}}}else{U=(T|0)>0;if((c[$>>2]|0)==0){if(U){aq=0;ar=0.0}else{ap=0.0;break}while(1){af=ar+ +h[d+(aq+am<<3)>>3]*+h[o+(aq+w<<3)>>3];V=aq+1|0;if((V|0)<(T|0)){aq=V;ar=af}else{ap=af;break L6682}}}if(U){V=0;af=0.0;ab=0.0;while(1){ad=+h[d+(V+am<<3)>>3];S=V+w|0;as=af+ad*+h[o+(S<<3)>>3];at=ab+ad*+h[o+(S+D<<3)>>3];S=V+1|0;if((S|0)<(T|0)){V=S;af=as;ab=at}else{au=as;av=at;break}}}else{au=0.0;av=0.0}ap=au+ +(aj|0)*(av-au)/+(J|0)}}while(0);h[b+(ai<<3)>>3]=ap;T=s+aj|0;D=I+Q|0;aw=c[p>>2]|0;if((T|0)<(aw|0)){ax=T;ay=D}else{ax=T-aw|0;ay=D+1|0}D=ai+1|0;if((D|0)==(ah|0)){break}else{q=am;ai=D;aj=ax;H=ay;J=aw}}J=c[Y>>2]|0;a=am;ag=ah;ah=0;Q=(J|0)/(aw|0)|0;s=(J|0)%(aw|0)|0;r=ax;m=ay;k=aw}aw=(ak|0)>0;c[e>>2]=aw?ak:0;K=ai;L=(ah|0)==0?0:ah-ai|0;M=Q;N=s;O=aj;P=((aw?0:ak)<<c[R>>2])+al|0}if((i|0)==0){return K|0}c[l>>2]=O;c[j>>2]=P;c[n>>2]=(ac(c[p>>2]|0,M)|0)+N;c[t>>2]=L;return K|0}function sX(a,d,e,f,j,k,l,m,n,o,p,q){a=a|0;d=d|0;e=e|0;f=f|0;j=j|0;k=k|0;l=+l;m=m|0;n=n|0;o=o|0;p=+p;q=q|0;var r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0.0,L=0.0,M=0.0,N=0.0,O=0,P=0,Q=0,T=0,W=0,X=0.0,Y=0,Z=0.0,_=0.0,$=0.0,aa=0.0,ad=0.0,ae=0,af=0.0,ag=0.0,ah=0.0,ai=0.0,aj=0.0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0;q=i;p=+(d|0)*(l!=0.0?l:.97)/+(e|0);l=p>1.0?1.0:p;r=1<<j;do{if((a|0)==0){s=4854}else{if((c[a+40>>2]|0)!=(j|0)){s=4854;break}if((c[a+48>>2]|0)!=(k|0)){s=4854;break}if(+h[a+64>>3]!=l){s=4854;break}t=c[a+8>>2]|0;u=~~+ab(+(+(f|0)/l));if((t|0)!=(((u|0)>1?u:1)|0)){s=4854;break}if((c[a+72>>2]|0)!=(m|0)){s=4854;break}if((c[a+52>>2]|0)!=(n|0)){s=4854;break}if((c[a+56>>2]|0)==(o|0)){v=a;s=4899}else{s=4854}}}while(0);do{if((s|0)==4854){a=A7(88)|0;u=a;if((a|0)==0){w=0;i=q;return w|0}t=a+72|0;c[t>>2]=m;x=B7(m)|0;y=a+76|0;c[y>>2]=x;z=c[t>>2]|0;if((z|0)==8|(z|0)==9){c[a+80>>2]=0}else if((z|0)==6){c[a+80>>2]=15}else if((z|0)==7){c[a+80>>2]=30}else{AN(0,16,1671392,(A=i,i=i+1|0,i=i+7&-8,c[A>>2]=0,A)|0);i=A;AN(0,0,1643120,(A=i,i=i+24|0,c[A>>2]=1656920,c[A+8>>2]=1691296,c[A+16>>2]=229,A)|0);i=A;cw();return 0}c[a+40>>2]=j;c[a+44>>2]=r-1;c[a+48>>2]=k;h[a+64>>3]=l;z=~~+ab(+(+(f|0)/l));B=(z|0)>1?z:1;z=a+8|0;c[z>>2]=B;C=B+7&-8;B=a+12|0;c[B>>2]=C;D=A8(C,ac(x,r+1|0)|0)|0;x=a+4|0;c[x>>2]=D;c[a+52>>2]=n;c[a+56>>2]=o;if((D|0)==0){E=u;break}C=c[z>>2]|0;z=c[B>>2]|0;F=1<<c[a+80>>2];a=A1(C<<3)|0;H=a;I=(C-1|0)/2|0;if((a|0)==0){E=u;break}L6729:do{if((r|0)>0){J=(C|0)>0;p=+(r|0);K=l*+(C|0);L=K*3.141592653589793;M=+(o|0);N=+(F|0);O=D;P=D;Q=D;T=D;W=0;L6731:while(1){if(J){X=+(W|0)/p;if((n|0)==2){Y=0;Z=0.0;while(1){_=l*(+(Y-I|0)-X)*3.141592653589793;if(_==0.0){$=1.0}else{$=+V(+_)/_}aa=_*2.0/L;_=1.0-aa*aa;aa=M*+S(+(_>0.0?_:0.0));_=aa*aa*.25;aa=1.0;ad=1.0;ae=0;while(1){af=ad*_*+h[336312+(ae<<3)>>3];ag=aa+af;if(ag!=aa){aa=ag;ad=af;ae=ae+1|0}else{break}}ad=$*ag;h[H+(Y<<3)>>3]=ad;aa=Z+ad;ae=Y+1|0;if((ae|0)<(C|0)){Y=ae;Z=aa}else{ah=aa;break}}}else if((n|0)==0){Y=0;Z=0.0;while(1){aa=+R(+(l*(+(Y-I|0)-X)));if(aa<1.0){ai=1.0-aa*aa*3.0+aa*aa*aa*2.0+(aa*(-0.0-aa)+aa*aa*aa)*-.5}else{ai=(aa*aa*aa+(aa*8.0+-4.0-aa*aa*5.0))*-.5}h[H+(Y<<3)>>3]=ai;aa=Z+ai;ae=Y+1|0;if((ae|0)<(C|0)){Y=ae;Z=aa}else{ah=aa;break}}}else{Y=0;Z=0.0;while(1){aa=l*(+(Y-I|0)-X)*3.141592653589793;if(aa==0.0){aj=1.0}else{aj=+V(+aa)/aa}if((n|0)!=1){break L6731}ad=aa*2.0/K+3.141592653589793;aa=.3635819- +U(+ad)*.4891775;_=aa+ +U(+(ad*2.0))*.1365995;aa=aj*(_- +U(+(ad*3.0))*.0106411);h[H+(Y<<3)>>3]=aa;ad=Z+aa;ae=Y+1|0;if((ae|0)<(C|0)){Y=ae;Z=ad}else{ah=ad;break}}}}else{ah=0.0}Y=c[t>>2]|0;do{if((Y|0)==7){if(!J){break}ae=ac(W,z)|0;ak=0;do{al=cS(+(N*+h[H+(ak<<3)>>3]/ah))|0;am=G;an=NU(al,am,-2147483648,0)|0;ao=G;ap=0;if(ao>>>0>ap>>>0|ao>>>0==ap>>>0&an>>>0>-1>>>0){an=am>>31|((am|0)<0|0?-1:0)<<1;((am|0)<0|0?-1:0)>>31|((am|0)<0|0?-1:0)<<1;aq=an^2147483647}else{aq=al}c[Q+(ak+ae<<2)>>2]=aq;ak=ak+1|0;}while((ak|0)<(C|0))}else if((Y|0)==8){if(!J){break}ak=ac(W,z)|0;ae=0;do{g[P+(ae+ak<<2)>>2]=N*+h[H+(ae<<3)>>3]/ah;ae=ae+1|0;}while((ae|0)<(C|0))}else if((Y|0)==6){if(!J){break}ae=ac(W,z)|0;ak=0;do{al=bR(+(N*+h[H+(ak<<3)>>3]/ah))|0;if((al|0)<-32768){ar=-32768}else{ar=(al|0)>32767?32767:al&65535}b[T+(ak+ae<<1)>>1]=ar;ak=ak+1|0;}while((ak|0)<(C|0))}else if((Y|0)==9){if(!J){break}ak=ac(W,z)|0;ae=0;do{h[O+(ae+ak<<3)>>3]=N*+h[H+(ae<<3)>>3]/ah;ae=ae+1|0;}while((ae|0)<(C|0))}}while(0);W=W+1|0;if((W|0)>=(r|0)){break L6729}}AN(0,0,1643120,(A=i,i=i+24|0,c[A>>2]=1656920,c[A+8>>2]=1691296,c[A+16>>2]=130,A)|0);i=A;cw();return 0}}while(0);A4(a);C=c[x>>2]|0;H=c[B>>2]|0;z=c[y>>2]|0;t=C+(ac((H<<j)+1|0,z)|0)|0;I=ac(H-1|0,z)|0;NN(t|0,C|0,I)|0;I=c[x>>2]|0;C=c[B>>2]|0;t=c[y>>2]|0;z=I+(ac(C<<j,t)|0)|0;H=I+(ac(C-1|0,t)|0)|0;NN(z|0,H|0,t)|0;v=u;s=4899}}while(0);do{if((s|0)==4899){c[v+36>>2]=0;j=v+20|0;A=N2(r,(r|0)<0|0?-1:0,e,(e|0)<0|0?-1:0)|0;if((BX(v+32|0,j,d,(d|0)<0|0?-1:0,A,G,1073741823,0)|0)==0){E=v;break}c[v+16>>2]=c[j>>2];c[v+24>>2]=ac(((c[v+8>>2]|0)-1|0)/2|0,-r|0)|0;c[v+28>>2]=0;w=v;i=q;return w|0}}while(0);A4(c[E+4>>2]|0);A4(E);w=0;i=q;return w|0}function sY(a){a=a|0;var b=0;b=c[a>>2]|0;if((b|0)==0){return}A6(b+4|0);A6(a);return}function sZ(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;zz()|0;h=b+132|0;i=c[h>>2]|0;if((i|0)<=0){j=-1;return j|0}k=a+72|0;l=-1;m=0;n=i;while(1){i=c[k>>2]|0;if((i|0)==8){o=m+1|0;p=sV(a,c[b+(m<<2)>>2]|0,c[e+(m<<2)>>2]|0,g,f,d,(o|0)==(n|0)|0)|0;q=o}else if((i|0)==6){o=m+1|0;p=sT(a,c[b+(m<<2)>>2]|0,c[e+(m<<2)>>2]|0,g,f,d,(o|0)==(n|0)|0)|0;q=o}else if((i|0)==9){o=m+1|0;p=sW(a,c[b+(m<<2)>>2]|0,c[e+(m<<2)>>2]|0,g,f,d,(o|0)==(n|0)|0)|0;q=o}else if((i|0)==7){i=m+1|0;p=sU(a,c[b+(m<<2)>>2]|0,c[e+(m<<2)>>2]|0,g,f,d,(i|0)==(n|0)|0)|0;q=i}else{p=l;q=m+1|0}i=c[h>>2]|0;if((q|0)<(i|0)){l=p;m=q;n=i}else{j=p;break}}return j|0}function s_(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;b=i;d=a+6924|0;e=a+6928|0;f=rS(a+6468|0,(c[e>>2]<<1)+(c[d>>2]|0)|0)|0;if((f|0)<0){g=f;i=b;return g|0}if((c[a+6612>>2]|0)==0){AN(0,0,1643120,(f=i,i=i+24|0,c[f>>2]=1719384,c[f+8>>2]=1691296,c[f+16>>2]=354,f)|0);i=f;cw();return 0}f=a+6600|0;h=c[f>>2]|0;if((h|0)>0){j=a+6604|0;k=0;l=c[e>>2]|0;m=h;while(1){if((l|0)>0){h=a+6468+(k<<2)|0;n=0;o=l;do{p=c[h>>2]|0;q=(c[d>>2]|0)+o|0;r=c[j>>2]|0;s=p+(ac(q+n|0,r)|0)|0;t=p+(ac(q+~n|0,r)|0)|0;NN(s|0,t|0,r)|0;n=n+1|0;o=c[e>>2]|0;}while((n|0)<(o|0));u=o;v=c[f>>2]|0}else{u=l;v=m}n=k+1|0;if((n|0)<(v|0)){k=n;l=u;m=v}else{w=u;break}}}else{w=c[e>>2]|0}c[e>>2]=((w+1|0)/2|0)+w;g=0;i=b;return g|0}function s$(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;c[a+36>>2]=d;e=c[a+16>>2]|0;if((d|0)==0){c[a+20>>2]=e;return 0}else{f=e;g=(e|0)<0|0?-1:0;e=N2(f,g,b,(b|0)<0|0?-1:0)|0;b=N0(e,G,d,(d|0)<0|0?-1:0)|0;d=NV(f,g,b,G)|0;c[a+20>>2]=d;return 0}return 0}function s0(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0;e=c[a+6976>>2]|0;f=(c[a+6928>>2]|0)-(((c[e+8>>2]|0)-1|0)/2|0)|0;g=c[e+40>>2]|0;h=NW(f|0,((f|0)<0|0?-1:0)|0,g|0)|0;f=c[e+24>>2]|0;i=NV(h,G,f,(f|0)<0|0?-1:0)|0;f=c[e+32>>2]|0;h=f;j=(f|0)<0|0?-1:0;f=N2(i,G,h,j)|0;i=c[e+28>>2]|0;e=NV(f,G,i,(i|0)<0|0?-1:0)|0;i=G;f=c[a+40>>2]|0;a=N2(f,(f|0)<0|0?-1:0,h,j)|0;j=NW(a|0,G|0,g|0)|0;g=AW(e,i,b,d,j,G)|0;return(G=G,g)|0}function s1(a){a=a|0;return 1643504}function s2(){return 70768}function s3(){return 131940}function s4(){return 1641192}function s5(b){b=b|0;var c=0;if(b>>>0>=321){c=0;return c|0}c=a[233912+b|0]&1;return c|0}function s6(a){a=a|0;var b=0;if(a>>>0>=321){b=0;return b|0}b=(d[233912+a|0]|0)>>>1&1;return b|0}function s7(e,f,g,h,j,k,l,m){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,z=0.0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0;n=i;o=e+52|0;p=BP(c[o>>2]|0)|0;q=e+56|0;r=BP(c[q>>2]|0)|0;s=e+11228|0;t=f;c[s>>2]=c[t>>2];c[s+4>>2]=c[t+4>>2];c[s+8>>2]=c[t+8>>2];c[s+12>>2]=c[t+12>>2];t=e+11244|0;s=h;c[t>>2]=c[s>>2];c[t+4>>2]=c[s+4>>2];c[t+8>>2]=c[s+8>>2];c[t+12>>2]=c[s+12>>2];s=BP(c[o>>2]|0)|0;if((s|0)==0){AN(0,0,1717904,(u=i,i=i+24|0,c[u>>2]=1661968,c[u+8>>2]=1660560,c[u+16>>2]=642,u)|0);i=u;cw();return 0}if((a[s+7|0]&32)==0){if((d[s+4|0]|0)>1){v=j}else{w=4963}}else{w=4963}do{if((w|0)==4963){s=c[o>>2]|0;if((s|0)==8|(s|0)==66|(s|0)==31|(s|0)==32){v=j;break}v=0}}while(0);j=BP(c[q>>2]|0)|0;if((j|0)==0){AN(0,0,1717904,(u=i,i=i+24|0,c[u>>2]=1661968,c[u+8>>2]=1660560,c[u+16>>2]=642,u)|0);i=u;cw();return 0}if((a[j+7|0]&32)==0){if((d[j+4|0]|0)>1){x=g}else{w=4969}}else{w=4969}do{if((w|0)==4969){j=c[q>>2]|0;if((j|0)==8|(j|0)==66|(j|0)==31|(j|0)==32){x=g;break}x=0}}while(0);c[e+11220>>2]=k;c[e+11216>>2]=l;c[e+11224>>2]=m;c[e+11260>>2]=x;c[e+11264>>2]=v;v=e+23820|0;NN(v|0,235560,24)|0;c[e+23812>>2]=2225240;c[e+23816>>2]=2233432;if((b[1120811]|0)==0){v=0;do{z=+(v|0)/4095.0;b[2225240+(v<<1)>>1]=(bR(+(+T(+z,2.5999999046325684)*4095.0))|0)&65535;b[2233432+(v<<1)>>1]=(bR(+(+T(+z,+.4545454446934474)*4095.0))|0)&65535;v=v+1|0;}while((v|0)<4096)}v=BP(c[o>>2]|0)|0;if((v|0)==0){AN(0,0,1717904,(u=i,i=i+24|0,c[u>>2]=1661968,c[u+8>>2]=1660560,c[u+16>>2]=642,u)|0);i=u;cw();return 0}if((a[v+7|0]&32)==0){if((d[v+4|0]|0)>1){w=4978}else{w=4977}}else{w=4977}if((w|0)==4977){v=c[o>>2]|0;if((v|0)==8|(v|0)==66|(v|0)==31|(v|0)==32){w=4978}}do{if((w|0)==4978){v=BP(c[q>>2]|0)|0;if((v|0)==0){AN(0,0,1717904,(u=i,i=i+24|0,c[u>>2]=1661968,c[u+8>>2]=1660560,c[u+16>>2]=642,u)|0);i=u;cw();return 0}do{if((a[v+7|0]&32)==0){if((d[v+4|0]|0)>1){A=-1}else{break}i=n;return A|0}}while(0);v=c[q>>2]|0;if((v|0)==8|(v|0)==66|(v|0)==31|(v|0)==32){A=-1}else{break}i=n;return A|0}}while(0);c[e+60>>2]=BN(p)|0;c[e+64>>2]=BN(r)|0;r=BP(c[o>>2]|0)|0;if((r|0)==0){AN(0,0,1717904,(u=i,i=i+24|0,c[u>>2]=1661968,c[u+8>>2]=1660560,c[u+16>>2]=642,u)|0);i=u;cw();return 0}if((a[r+7|0]&32)==0){if((d[r+4|0]|0)<=1){w=4987}}else{w=4987}do{if((w|0)==4987){r=c[o>>2]|0;if((r|0)==8|(r|0)==66|(r|0)==31|(r|0)==32){break}tr(e,f,x,k,l,m)|0}}while(0);m=c[h>>2]|0;l=m;k=(m|0)<0|0?-1:0;m=c[h+4>>2]|0;x=m;f=(m|0)<0|0?-1:0;m=-(c[h+8>>2]|0)|0;(m|0)<0|0?-1:0;o=-(c[h+12>>2]|0)|0;(o|0)<0|0?-1:0;w=e+10496|0;r=0;u=m;m=0;p=(u|0)>(m|0)|(u|0)==(m|0)&r>>>0>0>>>0;m=x>>>1|f<<31;q=f>>1|((f|0)<0|0?-1:0)<<31;v=NV(0,0,m,q)|0;g=NU(p?m:v,p?q:G,r,u)|0;u=N0(g,G,x,f)|0;g=G;r=0;q=o;o=0;p=(q|0)>(o|0)|(q|0)==(o|0)&r>>>0>0>>>0;o=l>>>1|k<<31;v=k>>1|((k|0)<0|0?-1:0)<<31;m=NV(0,0,o,v)|0;j=NU(p?o:m,p?v:G,r,q)|0;q=N0(j,G,l,k)|0;j=G;r=NV(0,1,u,g)|0;v=G;p=NV(r,v,q,j)|0;m=G;o=N2(p,m,76309,0)|0;s=G;t=0;B=(s|0)>(t|0)|(s|0)==(t|0)&o>>>0>0>>>0;t=NU(B?32768:-32768,B?0:-1,o,s)|0;s=N0(t,G,65536,0)|0;t=G;o=N2(p,m,x,f)|0;f=G;x=0;B=(f|0)>(x|0)|(f|0)==(x|0)&o>>>0>0>>>0;x=NU(B?32768:-32768,B?0:-1,o,f)|0;f=N0(x,G,65536,0)|0;x=G;o=N2(p,m,l,k)|0;k=G;l=0;B=(k|0)>(l|0)|(k|0)==(l|0)&o>>>0>0>>>0;l=NU(B?32768:-32768,B?0:-1,o,k)|0;k=N0(l,G,65536,0)|0;l=G;o=q<<15|0>>>17;B=j<<15|q>>>17;q=0;j=(B|0)>(q|0)|(B|0)==(q|0)&o>>>0>0>>>0;q=s>>>1|t<<31;C=t>>1|((t|0)<0|0?-1:0)<<31;D=NV(0,0,q,C)|0;E=G;F=NU(j?q:D,j?C:E,o,B)|0;H=N0(F,G,s,t)|0;F=NV(0,0,H,G)|0;c[w>>2]=F;F=NU(q,C,0,32768)|0;H=N0(F,G,s,t)|0;F=e+10500|0;c[F>>2]=H;H=u<<15|0>>>17;I=g<<15|u>>>17;J=0;K=(I|0)>(J|0)|(I|0)==(J|0)&H>>>0>0>>>0;J=NU(K?q:D,K?C:E,H,I)|0;E=N0(J,G,s,t)|0;t=NV(0,0,E,G)|0;E=e+10504|0;c[E>>2]=t;t=f>>>1|x<<31;s=x>>1|((x|0)<0|0?-1:0)<<31;J=NV(0,0,t,s)|0;C=G;D=NU(j?t:J,j?s:C,o,B)|0;B=N0(D,G,f,x)|0;D=e+10508|0;c[D>>2]=B;B=NU(t,s,0,32768)|0;o=N0(B,G,f,x)|0;B=NV(0,0,o,G)|0;o=e+10512|0;c[o>>2]=B;B=NU(p,m,u,g)|0;g=B<<15|0>>>17;u=G<<15|B>>>17;B=0;m=(u|0)>(B|0)|(u|0)==(B|0)&g>>>0>0>>>0;B=NU(m?t:J,m?s:C,g,u)|0;u=N0(B,G,f,x)|0;x=e+10516|0;c[x>>2]=u;u=r<<15|0>>>17;f=v<<15|r>>>17;r=0;v=(f|0)>(r|0)|(f|0)==(r|0)&u>>>0>0>>>0;r=k>>>1|l<<31;B=l>>1|((l|0)<0|0?-1:0)<<31;g=NV(0,0,r,B)|0;C=G;s=NU(v?r:g,v?B:C,u,f)|0;f=N0(s,G,k,l)|0;s=e+10520|0;c[s>>2]=f;f=NU(r,B,0,32768)|0;u=N0(f,G,k,l)|0;f=NV(0,0,u,G)|0;u=e+10524|0;c[u>>2]=f;f=NU(K?r:g,K?B:C,H,I)|0;I=N0(f,G,k,l)|0;l=e+10528|0;c[l>>2]=I;if((h|0)==236600){c[E>>2]=3208;c[l>>2]=-2332;c[x>>2]=14392;c[F>>2]=16519;c[u>>2]=-12061;c[o>>2]=-9528;c[w>>2]=8414;c[s>>2]=14392;c[D>>2]=-4865}D=e+10560|0;s=0;while(1){w=a[235584+s|0]|0;if(w<<24>>24>-1){L=c[e+10496+(w<<24>>24<<2)>>2]&65535}else{L=0}w=D+(s<<1)|0;y=L;a[w]=y&255;y=y>>8;a[w+1|0]=y&255;w=s+1|0;if(w>>>0<280){s=w}else{A=0;break}}i=n;return A|0}function s8(f,g,h){f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0;j=i;i=i+16|0;k=j|0;NM(k|0,0,16);l=f+8|0;m=c[l>>2]|0;n=f+12|0;o=c[n>>2]|0;p=f+19592|0;q=c[p>>2]|0;r=f+16|0;s=c[r>>2]|0;t=(q<<1)+81&-16;u=f+56|0;v=c[u>>2]|0;w=f+52|0;x=c[w>>2]|0;y=BP(v)|0;z=BP(x)|0;zz()|0;A=f+2284|0;B=c[A>>2]|0;if((c[540998]|0)==0){vE()}C=(m|0)==(q|0);D=(o|0)==(s|0);E=C&D;switch(v|0){case 34:{F=33;break};case 12:{F=0;break};case 13:{F=4;break};case 320:{F=7;break};case 14:{F=5;break};default:{F=v}}switch(x|0){case 13:{H=4;break};case 14:{H=5;break};case 34:{H=33;break};case 320:{H=7;break};case 12:{H=0;break};default:{H=x}}if((F|0)==(c[u>>2]|0)){if((H|0)!=(c[w>>2]|0)){I=5015}}else{I=5015}if((I|0)==5015){AN(f,24,1671216,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J}if((F|0)==295){K=27}else if((F|0)==296){K=28}else if((F|0)==298){K=30}else if((F|0)==297){K=29}else{K=F}if((H|0)==298){L=30}else if((H|0)==295){L=27}else if((H|0)==296){L=28}else if((H|0)==297){L=29}else{L=H}if((K|0)==110){M=42}else if((K|0)==111){M=41}else{M=K}if((L|0)==110){N=42}else if((L|0)==111){N=41}else{N=L}if((M|0)==(c[u>>2]|0)){if((N|0)!=(c[w>>2]|0)){I=5034}}else{I=5034}if((I|0)==5034){c[u>>2]=M;c[w>>2]=N}L=M>>>0<321;do{if(E){if(!L){I=5041;break}if((M-110|0)>>>0>1){I=5040;break}if((BV(M)|0)!=(N|0)){I=5040}}else{if(L){I=5040}else{I=5041}}}while(0);do{if((I|0)==5040){if((a[233912+M|0]&1)==0){I=5041;break}if(N>>>0<321){if((a[233912+N|0]&2)!=0){break}}L=BL(N)|0;AN(f,16,1637800,(J=i,i=i+8|0,c[J>>2]=L,J)|0);i=J;O=-22;i=j;return O|0}}while(0);if((I|0)==5041){L=BL(M)|0;AN(f,16,1656680,(J=i,i=i+8|0,c[J>>2]=L,J)|0);i=J;O=-22;i=j;return O|0}L=B&2047;do{if((L|0)!=0){if((B+2047&L|0)!=0){break}if((m|0)<1|(o|0)<1|(q|0)<1|(s|0)<1){AN(f,16,1623416,(J=i,i=i+32|0,c[J>>2]=m,c[J+8>>2]=o,c[J+16>>2]=q,c[J+24>>2]=s,J)|0);i=J;O=-22;i=j;return O|0}E=(h|0)==0?k:h;K=(g|0)==0?k:g;H=m;F=q>>1;x=NU(F,(F|0)<0|0?-1:0,H<<16|0>>>16,((m|0)<0|0?-1:0)<<16|H>>>16)|0;H=N0(x,G,q,(q|0)<0|0?-1:0)|0;x=f+36|0;c[x>>2]=H;H=o;F=s>>1;v=NU(F,(F|0)<0|0?-1:0,H<<16|0>>>16,((o|0)<0|0?-1:0)<<16|H>>>16)|0;H=s;F=(s|0)<0|0?-1:0;P=N0(v,G,H,F)|0;v=f+44|0;c[v>>2]=P;c[f+60>>2]=BN(z)|0;c[f+64>>2]=BN(y)|0;P=f+19608|0;c[P>>2]=262148;c[P+4>>2]=262148;P=K+4|0;Q=c[P>>2]|0;if((Q|0)==0){I=5052}else{if((c[Q+4>>2]|0)>1){R=1}else{I=5052}}do{if((I|0)==5052){Q=c[K+12>>2]|0;if((Q|0)!=0){if((c[Q+4>>2]|0)>1){R=1;break}}Q=c[E+4>>2]|0;if((Q|0)!=0){if((c[Q+4>>2]|0)>1){R=1;break}}Q=c[E+12>>2]|0;if((Q|0)==0){R=0;break}R=(c[Q+4>>2]|0)>1}}while(0);Q=K|0;S=c[Q>>2]|0;if((S|0)==0){I=5060}else{if((c[S+4>>2]|0)>1){T=1}else{I=5060}}do{if((I|0)==5060){S=c[K+8>>2]|0;if((S|0)!=0){if((c[S+4>>2]|0)>1){T=1;break}}S=c[E>>2]|0;if((S|0)!=0){if((c[S+4>>2]|0)>1){T=1;break}}S=c[E+8>>2]|0;if((S|0)==0){T=0;break}T=(c[S+4>>2]|0)>1}}while(0);S=f+76|0;U=f+80|0;V=BP(M)|0;c[S>>2]=d[V+5|0]|0;c[U>>2]=d[V+6|0]|0;V=f+84|0;W=f+88|0;X=BP(N)|0;c[V>>2]=d[X+5|0]|0;c[W>>2]=d[X+6|0]|0;X=(N|0)==10;Y=(N|0)==9;Z=(N|0)==64;_=(N-41|0)>>>0<2|(N|0)==291|(N|0)==292|(N|0)==30|(N|0)==29|(N|0)==2|(N|0)==43|(N|0)==44|(N|0)==45|(N|0)==46|(N|0)==63|(N|0)==62|(N|0)==22|(N|0)==23|(N|0)==24|X|Y|(N|0)==67|(N|0)==68|(N|0)==293|(N|0)==294|(N|0)==28|(N|0)==27|(N|0)==3|(N|0)==47|(N|0)==48|(N|0)==49|(N|0)==50|(N|0)==65|Z|(N|0)==19|(N|0)==20|(N|0)==21|X|Y;do{if(_){I=5070}else{Y=BP(N)|0;if((Y|0)==0){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1661968,c[J+8>>2]=1660560,c[J+16>>2]=656,J)|0);i=J;cw();return 0}else{if((a[Y+7|0]&32)==0){$=B;break}else{I=5070;break}}}}while(0);do{if((I|0)==5070){if((B&8192|0)!=0){$=B;break}if((q&1|0)==0){$=B;break}AN(f,48,1617760,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;Y=B|8192;c[A>>2]=Y;$=Y}}while(0);do{if((N|0)==24|(N|0)==22|(N|0)==21|(N|0)==19){if(($&8396800|0)==8388608){Y=BL(N)|0;AN(f,48,1612896,(J=i,i=i+8|0,c[J>>2]=Y,J)|0);i=J;Y=$|8192;c[A>>2]=Y;aa=Y}else{aa=$}if((aa&8396800|0)!=8192){ab=aa;break}Y=BL(N)|0;AN(f,48,1739952,(J=i,i=i+8|0,c[J>>2]=Y,J)|0);i=J;Y=aa|8388608;c[A>>2]=Y;ab=Y}else{ab=$}}while(0);Y=BP(N)|0;if((Y|0)==0){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1661968,c[J+8>>2]=1660560,c[J+16>>2]=786,J)|0);i=J;cw();return 0}do{if((a[Y+7|0]&48)==48){if((ab&8192|0)!=0){ac=ab;break}X=BL(N)|0;AN(f,48,1735376,(J=i,i=i+8|0,c[J>>2]=X,J)|0);i=J;X=ab|8192;c[A>>2]=X;ac=X}else{ac=ab}}while(0);L7027:do{if((ac&8192|0)==0){ad=ac}else{do{if(!_){Y=BP(N)|0;if((Y|0)==0){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1661968,c[J+8>>2]=1660560,c[J+16>>2]=656,J)|0);i=J;cw();return 0}else{if((a[Y+7|0]&32)==0){ad=ac;break L7027}else{break}}}}while(0);Y=BP(N)|0;if((Y|0)==0){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1661968,c[J+8>>2]=1660560,c[J+16>>2]=786,J)|0);i=J;cw();return 0}if((a[Y+7|0]&48)==48){ad=ac;break}if(!((N-27|0)>>>0>1&(N|0)!=30&(N|0)!=29&(N|0)!=2&(N|0)!=3&(N|0)!=21&(N|0)!=24&(N|0)!=19&(N|0)!=22)){ad=ac;break}Y=BL(N)|0;AN(f,24,1730528,(J=i,i=i+8|0,c[J>>2]=Y,J)|0);i=J;Y=ac&-8193;c[A>>2]=Y;ad=Y}}while(0);do{if(_){I=5097}else{Y=BP(N)|0;if((Y|0)==0){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1661968,c[J+8>>2]=1660560,c[J+16>>2]=656,J)|0);i=J;cw();return 0}else{if((a[Y+7|0]&32)==0){break}else{I=5097;break}}}}while(0);do{if((I|0)==5097){if((ad&8192|0)!=0){break}c[V>>2]=1}}while(0);Y=ad>>>16&3;c[f+92>>2]=Y;c[U>>2]=(c[U>>2]|0)+Y;Y=(M|0)==10;X=(M|0)==9;ae=(M-41|0)>>>0<2|(M|0)==291|(M|0)==292|(M|0)==30|(M|0)==29|(M|0)==2|(M|0)==43|(M|0)==44|(M|0)==45|(M|0)==46|(M|0)==63|(M|0)==62|(M|0)==22|(M|0)==23|(M|0)==24|Y|X|(M|0)==67|(M|0)==68|(M|0)==293|(M|0)==294|(M|0)==28|(M|0)==27|(M|0)==3|(M|0)==47|(M|0)==48|(M|0)==49|(M|0)==50|(M|0)==65|(M|0)==64|(M|0)==19|(M|0)==20|(M|0)==21|Y|X;do{if(ae){I=5103}else{X=BP(M)|0;if((X|0)==0){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1661968,c[J+8>>2]=1660560,c[J+16>>2]=656,J)|0);i=J;cw();return 0}else{if((a[X+7|0]&32)==0){break}else{I=5103;break}}}}while(0);L7055:do{if((I|0)==5103){if((ad&16384|0)!=0){break}switch(M|0){case 316:case 315:case 314:case 313:case 88:case 87:case 86:case 85:case 84:case 83:case 24:case 23:case 22:case 21:case 20:case 19:{break L7055;break};default:{}}if((q>>c[V>>2]|0)>(m>>1|0)){if((ad&1|0)==0){break}}c[S>>2]=1}}while(0);X=f+20|0;c[X>>2]=-(-m>>c[S>>2]);Y=f+24|0;c[Y>>2]=-(-o>>c[U>>2]);af=f+28|0;c[af>>2]=-(-q>>c[V>>2]);ag=f+32|0;c[ag>>2]=-(-s>>c[W>>2]);ah=(m<<2)+186&-32;ai=A1(ah)|0;c[f+2208>>2]=ai;if(!((ai|0)!=0|(ah|0)==0)){AN(f,16,1726200,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;O=-1;i=j;return O|0}L7067:do{if(!(T|C&D^1|R)){do{if(!((c[f+11260>>2]|0)==(c[f+11264>>2]|0)|_)){ah=BP(N)|0;if((ah|0)==0){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1661968,c[J+8>>2]=1660560,c[J+16>>2]=656,J)|0);i=J;cw();return 0}else{if((a[ah+7|0]&32)==0){break L7067}else{break}}}}while(0);wK(f);if((c[f+4>>2]|0)==0){break}if((ad&4096|0)==0){O=0;i=j;return O|0}ah=BL(M)|0;ai=BL(N)|0;AN(f,32,1721896,(J=i,i=i+16|0,c[J>>2]=ah,c[J+8>>2]=ai,J)|0);i=J;O=0;i=j;return O|0}}while(0);_=((e[y+8>>1]|0)>>>11&15)+1|0;W=f+72|0;c[W>>2]=_>>>0<8?8:_;_=z+8|0;V=((e[_>>1]|0)>>>11&15)+1|0;S=f+68|0;c[S>>2]=V>>>0<8?8:V;do{if(ae){I=5122}else{V=BP(M)|0;if((V|0)==0){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1661968,c[J+8>>2]=1660560,c[J+16>>2]=656,J)|0);i=J;cw();return 0}else{if((a[V+7|0]&32)!=0|(M|0)==11){I=5122;break}else{break}}}}while(0);if((I|0)==5122){c[W>>2]=16}ae=t<<((c[S>>2]|0)==16);c[f+2276>>2]=0;V=c[X>>2]|0;ai=V;ah=c[af>>2]|0;aj=ah>>1;ak=NU(aj,(aj|0)<0|0?-1:0,ai<<16|0>>>16,((V|0)<0|0?-1:0)<<16|ai>>>16)|0;ai=N0(ak,G,ah,(ah|0)<0|0?-1:0)|0;ah=f+40|0;c[ah>>2]=ai;ai=c[Y>>2]|0;ak=ai;V=c[ag>>2]|0;aj=V>>1;al=NU(aj,(aj|0)<0|0?-1:0,ak<<16|0>>>16,((ai|0)<0|0?-1:0)<<16|ak>>>16)|0;ak=N0(al,G,V,(V|0)<0|0?-1:0)|0;V=f+48|0;c[V>>2]=ak;ak=(ad&1|0)==0;al=(ad&64|0)!=0;ai=al?ad|4:ad;aj=f+104|0;if((s9(f+2212|0,f+2228|0,f+2244|0,c[x>>2]|0,m,q,16384,ai,c[Q>>2]|0,c[E>>2]|0,aj)|0)<0){O=-1;i=j;return O|0}am=al?ad|2:ad;if((s9(f+2216|0,f+2232|0,f+2248|0,c[ah>>2]|0,c[X>>2]|0,c[af>>2]|0,16384,am,c[K+8>>2]|0,c[E+8>>2]|0,aj)|0)<0){O=-1;i=j;return O|0}an=f+2236|0;ao=f+2252|0;if((s9(f+2220|0,an,ao,c[v>>2]|0,o,s,4096,ai,c[P>>2]|0,c[E+4>>2]|0,aj)|0)<0){O=-1;i=j;return O|0}ai=f+2240|0;ap=f+2256|0;if((s9(f+2224|0,ai,ap,c[V>>2]|0,c[Y>>2]|0,c[ag>>2]|0,4096,am,c[K+12>>2]|0,c[E+12>>2]|0,aj)|0)<0){O=-1;i=j;return O|0}aj=c[ao>>2]|0;ao=f+2184|0;c[ao>>2]=aj;am=c[ap>>2]|0;ap=f+2188|0;c[ap>>2]=am;if((s|0)>0){aq=c[ag>>2]|0;ar=c[an>>2]|0;an=c[ai>>2]|0;ai=c[U>>2]|0;as=0;at=aj;au=am;while(1){av=N2(aq,(aq|0)<0|0?-1:0,as,(as|0)<0|0?-1:0)|0;aw=N0(av,G,H,F)|0;av=c[ar+(as<<2)>>2]|0;ax=av-1+aj|0;ay=an+(aw<<2)|0;aw=c[ay>>2]|0;az=aw-1+am<<ai;aA=((ax|0)>(az|0)?ax:az)>>ai<<ai;if((at+av|0)<(aA|0)){az=aA-av|0;c[ao>>2]=az;aB=az;aC=c[ay>>2]|0}else{aB=at;aC=aw}aw=aA>>ai;if((au+aC|0)<(aw|0)){aA=aw-aC|0;c[ap>>2]=aA;aD=aA}else{aD=au}aA=as+1|0;if((aA|0)<(s|0)){as=aA;at=aB;au=aD}else{break}}}au=A7((c[p>>2]<<2)+8|0)|0;c[f+11200>>2]=au;at=c[p>>2]|0;if((au|0)==0){if((at&1073741823|0)==1073741822){I=5137}}else{I=5137}do{if((I|0)==5137){au=A7((at<<2)+8|0)|0;c[f+11204>>2]=au;as=c[p>>2]|0;if((au|0)==0){if((as&1073741823|0)!=1073741822){break}}au=A7((as<<2)+8|0)|0;c[f+11208>>2]=au;as=c[p>>2]|0;if((au|0)==0){if((as&1073741823|0)!=1073741822){break}}au=A7((as<<2)+8|0)|0;c[f+11212>>2]=au;if((au|0)==0){if((c[p>>2]&1073741823|0)!=1073741822){break}}au=A1((c[ao>>2]|0)*12|0)|0;as=f+2168|0;c[as>>2]=au;do{if((au|0)==0){if(((c[ao>>2]|0)*12|0|0)==0){break}AN(f,16,1726200,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;O=-1;i=j;return O|0}}while(0);au=A1((c[ap>>2]|0)*12|0)|0;ai=f+2172|0;c[ai>>2]=au;am=c[ap>>2]|0;do{if((au|0)==0){if((am*12|0|0)==0){break}AN(f,16,1726200,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;O=-1;i=j;return O|0}}while(0);au=A1(am*12|0)|0;an=f+2176|0;c[an>>2]=au;do{if((au|0)==0){if(((c[ap>>2]|0)*12|0|0)==0){break}AN(f,16,1726200,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;O=-1;i=j;return O|0}}while(0);au=c[u>>2]|0;am=BP(au)|0;if((am|0)==0){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1661968,c[J+8>>2]=1660560,c[J+16>>2]=744,J)|0);i=J;cw();return 0}if((au|0)==11){I=5153}else{if((a[am+7|0]|0)<0){I=5153}else{I=5150}}do{if((I|0)==5153){am=c[w>>2]|0;au=BP(am)|0;if((au|0)==0){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1661968,c[J+8>>2]=1660560,c[J+16>>2]=744,J)|0);i=J;cw();return 0}if((am|0)!=11){if((a[au+7|0]|0)>=0){I=5150;break}}au=A7((c[ao>>2]|0)*12|0)|0;c[f+2180>>2]=au;if((au|0)!=0){I=5150;break}au=c[ao>>2]|0;if((au*12|0|0)==0){aE=au;break}AN(f,16,1726200,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;O=-1;i=j;return O|0}}while(0);if((I|0)==5150){aE=c[ao>>2]|0}au=ae+16|0;L7159:do{if((aE|0)>0){am=(au|0)==0;aj=0;while(1){ar=A7(au)|0;c[(c[as>>2]|0)+((c[ao>>2]|0)+aj<<2)>>2]=ar;ar=c[as>>2]|0;F=c[ar+((c[ao>>2]|0)+aj<<2)>>2]|0;if(!((F|0)!=0|am)){break}c[ar+(aj<<2)>>2]=F;aj=aj+1|0;if((aj|0)>=(c[ao>>2]|0)){break L7159}}AN(f,16,1726200,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;O=-1;i=j;return O|0}}while(0);as=ae>>1;c[f+23736>>2]=(64/(c[S>>2]&-8|0)|0)+as;c[f+23744>>2]=au;aj=c[ap>>2]|0;L7167:do{if((aj|0)>0){am=(ae<<1)+32|0;F=(am|0)==0;ar=as+8|0;H=0;while(1){aq=A1(am)|0;c[(c[ai>>2]|0)+((c[ap>>2]|0)+H<<2)>>2]=aq;aq=c[ai>>2]|0;U=c[aq+((c[ap>>2]|0)+H<<2)>>2]|0;if(!((U|0)!=0|F)){break}c[aq+(H<<2)>>2]=U;U=(c[(c[ai>>2]|0)+(H<<2)>>2]|0)+(ar<<1)|0;c[(c[an>>2]|0)+((c[ap>>2]|0)+H<<2)>>2]=U;c[(c[an>>2]|0)+(H<<2)>>2]=U;U=H+1|0;aq=c[ap>>2]|0;if((U|0)<(aq|0)){H=U}else{aF=aq;break L7167}}AN(f,16,1726200,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;O=-1;i=j;return O|0}else{aF=aj}}while(0);aj=f+2180|0;do{if((c[aj>>2]|0)==0){aG=aF}else{if((c[ao>>2]|0)<=0){aG=aF;break}L7178:do{if((au|0)==0){an=0;do{as=A7(0)|0;c[(c[aj>>2]|0)+((c[ao>>2]|0)+an<<2)>>2]=as;as=c[aj>>2]|0;c[as+(an<<2)>>2]=c[as+((c[ao>>2]|0)+an<<2)>>2];an=an+1|0;}while((an|0)<(c[ao>>2]|0))}else{an=0;while(1){as=A7(au)|0;c[(c[aj>>2]|0)+((c[ao>>2]|0)+an<<2)>>2]=as;as=c[aj>>2]|0;H=c[as+((c[ao>>2]|0)+an<<2)>>2]|0;if((H|0)==0){break}c[as+(an<<2)>>2]=H;an=an+1|0;if((an|0)>=(c[ao>>2]|0)){break L7178}}AN(f,16,1726200,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;O=-1;i=j;return O|0}}while(0);aG=c[ap>>2]|0}}while(0);L7188:do{if((aG|0)>0){aj=((ae|0)/2|0)+1|0;au=(aj|0)>0;an=ae|1;H=(an|0)>0;as=0;L7190:while(1){do{if((b[_>>1]&30720)==30720){if((c[S>>2]|0)<=14){break L7190}if(au){aH=0}else{break}do{c[(c[(c[ai>>2]|0)+(as<<2)>>2]|0)+(aH<<2)>>2]=262144;aH=aH+1|0;}while((aH|0)<(aj|0))}else{if(H){aI=0}else{break}do{b[(c[(c[ai>>2]|0)+(as<<2)>>2]|0)+(aI<<1)>>1]=16384;aI=aI+1|0;}while((aI|0)<(an|0))}}while(0);as=as+1|0;if((as|0)>=(c[ap>>2]|0)){break L7188}}AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1713952,c[J+8>>2]=1711376,c[J+16>>2]=1489,J)|0);i=J;cw();return 0}}while(0);if((c[ag>>2]|0)>(s|0)){AN(0,0,1717904,(J=i,i=i+24|0,c[J>>2]=1708016,c[J+8>>2]=1711376,c[J+16>>2]=1496,J)|0);i=J;cw();return 0}if((ad&4096|0)!=0){do{if(ak){if((ad&2|0)!=0){AN(f,32,1704024,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}if((ad&4|0)!=0){AN(f,32,1701584,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}if((ad&8|0)!=0){AN(f,32,1698504,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}if((ad&16|0)!=0){AN(f,32,1696448,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}if((ad&32|0)!=0){AN(f,32,1694104,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}if(al){AN(f,32,1690512,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}if((ad&128|0)!=0){AN(f,32,1687480,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}if((ad&256|0)!=0){AN(f,32,1685872,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}if((ad&512|0)!=0){AN(f,32,1683904,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}ai=f;if((ad&1024|0)==0){AN(ai,32,1680272,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}else{AN(ai,32,1682216,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;break}}else{AN(f,32,1705760,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J}}while(0);ai=f;as=BL(M)|0;switch(N|0){case 65:case 63:case 62:case 50:case 48:{aJ=1676352;break};default:{aJ=Z?1676352:3119064}}an=BL(N)|0;AN(ai,32,1678344,(J=i,i=i+24|0,c[J>>2]=as,c[J+8>>2]=aJ,c[J+16>>2]=an,J)|0);i=J;AN(ai,32,1672856,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;AN(ai,40,1670808,(J=i,i=i+32|0,c[J>>2]=m,c[J+8>>2]=o,c[J+16>>2]=q,c[J+24>>2]=s,J)|0);i=J;an=c[n>>2]|0;as=c[p>>2]|0;H=c[r>>2]|0;aj=c[x>>2]|0;au=c[v>>2]|0;AN(ai,48,1668976,(J=i,i=i+48|0,c[J>>2]=c[l>>2],c[J+8>>2]=an,c[J+16>>2]=as,c[J+24>>2]=H,c[J+32>>2]=aj,c[J+40>>2]=au,J)|0);i=J;au=c[Y>>2]|0;aj=c[af>>2]|0;H=c[ag>>2]|0;as=c[ah>>2]|0;an=c[V>>2]|0;AN(ai,48,1667544,(J=i,i=i+48|0,c[J>>2]=c[X>>2],c[J+8>>2]=au,c[J+16>>2]=aj,c[J+24>>2]=H,c[J+32>>2]=as,c[J+40>>2]=an,J)|0);i=J}c[f+4>>2]=wu(f)|0;O=0;i=j;return O|0}}while(0);AN(f,16,1726200,(J=i,i=i+1|0,i=i+7&-8,c[J>>2]=0,J)|0);i=J;O=-1;i=j;return O|0}}while(0);AN(f,16,1629600,(J=i,i=i+8|0,c[J>>2]=L,J)|0);i=J;O=-22;i=j;return O|0}function s9(a,e,f,g,j,k,l,m,n,o,p){a=a|0;e=e|0;f=f|0;g=g|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,S=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0.0,ad=0,ae=0,af=0,ag=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0.0,aG=0.0,aH=0.0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0.0,aX=0.0,aY=0,aZ=0,a_=0.0,a$=0.0,a0=0.0,a1=0.0,a2=0.0,a3=0.0,a4=0.0,a5=0.0,a6=0.0,a7=0.0,a8=0.0,a9=0.0,ba=0.0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0;q=i;i=i+8|0;r=q|0;c[r>>2]=0;s=(j|0)/(k|0)|0;t=s>>>0>65535;u=t?16:0;v=t?s>>>16:s;s=(v&65280|0)==0;if(s){w=v;x=u}else{w=v>>>8;x=u|8}if(((d[247648+w|0]|0)+x|0)>8){y=8}else{if(s){z=v;A=u}else{z=v>>>8;A=u|8}y=(d[247648+z|0]|0)+A|0}A=54-y|0;y=NW(1,0,A|0)|0;z=G;u=k+3|0;v=u<<2;s=A1(v)|0;c[e>>2]=s;L7261:do{if((s|0)!=0|(v|0)==0){x=g-65536|0;L7264:do{if((((x|0)>-1?x:65536-g|0)|0)<10){w=k<<3;t=A7(w)|0;B=t;c[r>>2]=B;if(!((t|0)!=0|(w|0)==0)){AN(0,16,1726200,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;D=0;break L7261}if((k|0)>0){E=0;F=B}else{H=1;break}while(1){B=F+(E<<3)|0;c[B>>2]=y;c[B+4>>2]=z;c[(c[e>>2]|0)+(E<<2)>>2]=E;B=E+1|0;if((B|0)>=(k|0)){H=1;break L7264}E=B;F=c[r>>2]|0}}else{if((m&16|0)!=0){B=k<<3;w=A1(B)|0;c[r>>2]=w;if(!((w|0)!=0|(B|0)==0)){AN(0,16,1726200,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;D=0;break L7261}if((k|0)<=0){H=1;break}B=((g|0)/2|0)-32768|0;w=g;t=(g|0)<0|0?-1:0;I=(B|0)<0|0?-1:0;J=B;B=0;while(1){K=NU(J,I,32768,0)|0;c[(c[e>>2]|0)+(B<<2)>>2]=K>>>16|G<<16;K=(c[r>>2]|0)+(B<<3)|0;c[K>>2]=y;c[K+4>>2]=z;K=NU(J,I,w,t)|0;L=B+1|0;if((L|0)<(k|0)){I=G;J=K;B=L}else{H=1;break L7264}}}B=(g|0)<65537;if(B){if((m&33|0)!=0){M=5266}}else{if((m&1|0)!=0){M=5266}}if((M|0)==5266){J=k<<4;I=A1(J)|0;c[r>>2]=I;if(!((I|0)!=0|(J|0)==0)){AN(0,16,1726200,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;D=0;break L7261}if((k|0)<=0){H=2;break}J=((g|0)/2|0)-32768|0;I=y>>>16|z<<16;t=z>>16|((z|0)<0|0?-1:0)<<16;w=g;L=(g|0)<0|0?-1:0;K=(J|0)<0|0?-1:0;N=J;J=0;while(1){O=N>>>16|K<<16;c[(c[e>>2]|0)+(J<<2)>>2]=O;P=J<<1;S=c[r>>2]|0;W=O;X=NV(W<<16|0>>>16,((O|0)<0|0?-1:0)<<16|W>>>16,N,K)|0;W=G;Y=-1;Z=(W|0)>(Y|0)|(W|0)==(Y|0)&X>>>0>-1>>>0;Y=NV(0,0,X,W)|0;_=N2(Z?X:Y,Z?W:G,I,t)|0;W=NV(y,z,_,G)|0;_=G;Z=0;Y=(_|0)<(Z|0)|(_|0)==(Z|0)&W>>>0<0>>>0;Z=S+(P<<3)|0;c[Z>>2]=Y?0:W;c[Z+4>>2]=Y?0:_;_=O+1|0;O=_;Y=NV(O<<16|0>>>16,((_|0)<0|0?-1:0)<<16|O>>>16,N,K)|0;O=G;_=-1;Z=(O|0)>(_|0)|(O|0)==(_|0)&Y>>>0>-1>>>0;_=NV(0,0,Y,O)|0;W=N2(Z?Y:_,Z?O:G,I,t)|0;O=NV(y,z,W,G)|0;W=G;Z=0;_=(W|0)<(Z|0)|(W|0)==(Z|0)&O>>>0<0>>>0;Z=S+((P|1)<<3)|0;c[Z>>2]=_?0:O;c[Z+4>>2]=_?0:W;W=NU(N,K,w,L)|0;_=J+1|0;if((_|0)<(k|0)){K=G;N=W;J=_}else{H=2;break L7264}}}J=(m&4|0)!=0;do{if(J){$=4}else{if((m&8|0)!=0){$=8;break}if((m&32|0)!=0){$=1;break}if((m&128|0)!=0){$=8;break}if((m&512|0)!=0){aa=+h[p>>3];if(aa==123456.0){$=6;break}$=~~+ab(+(aa*2.0));break}if((m&1280|0)!=0){$=20;break}if((m&2|0)!=0){$=2;break}AN(0,0,1717904,(C=i,i=i+24|0,c[C>>2]=1659336,c[C+8>>2]=1711376,c[C+16>>2]=344,C)|0);i=C;cw();return 0}}while(0);if(B){ad=$}else{ad=(k-1+(ac($,j)|0)|0)/(k|0)|0}N=ad+1|0;K=j-2|0;L=(N|0)>(K|0)?K:N;N=(L|0)>1?L:1;L=ac(N,k<<3)|0;K=A1(L)|0;c[r>>2]=K;if(!((K|0)!=0|(L|0)==0)){AN(0,16,1726200,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;D=0;break L7261}L7312:do{if((k|0)>0){L=(N<<16)-131072|0;K=L;w=(L|0)<0|0?-1:0;L=(N|0)>0;t=g<<1;I=t;_=(t|0)<0|0?-1:0;t=(g|0)>65536;W=k;Z=(k|0)<0|0?-1:0;O=j;P=(j|0)<0|0?-1:0;S=p+8|0;Y=NX(0,4194304,A|0)|0;X=G;ae=(m&8|0)==0;af=(m&32|0)==0;ag=(m&128|0)==0;ai=(m&256|0)==0;aj=(m&512|0)==0;ak=(m&2|0)==0;al=(m&1024|0)==0;aa=+(y>>>0)+ +(z|0)*4294967296.0;am=y>>>30|z<<2;an=z>>30|((z|0)<0|0?-1:0)<<2;ao=g;ap=(g|0)<0|0?-1:0;aq=z>>14|((z|0)<0|0?-1:0)<<18;ar=((z|0)<0|0?-1:0)>>14|((z|0)<0|0?-1:0)<<18;as=(x|0)<0|0?-1:0;at=x;au=0;L7314:while(1){av=NV(at,as,K,w)|0;aw=N0(av,G,131072,0)|0;av=aw;c[(c[e>>2]|0)+(au<<2)>>2]=av;if(L){aw=ac(au,N)|0;ax=av;av=0;while(1){ay=ax;az=NV(ay<<17|0>>>15,((ax|0)<0|0?-1:0)<<17|ay>>>15,at,as)|0;ay=G;aA=-1;aB=(ay|0)>(aA|0)|(ay|0)==(aA|0)&az>>>0>-1>>>0;aA=NV(0,0,az,ay)|0;aC=aB?az:aA;aA=aC<<13|0>>>19;az=(aB?ay:G)<<13|aC>>>19;if(t){aC=N2(aA,az,W,Z)|0;ay=N0(aC,G,O,P)|0;aD=G;aE=ay}else{aD=az;aE=aA}aF=(+(aE>>>0)+ +(aD|0)*4294967296.0)*9.313225746154785e-10;do{if(J){aG=+h[p>>3];if(aG!=123456.0){aH=aG*16777216.0;aI=+R(aH)>=1.0?aH>0.0?(ah(+Q(aH/4294967296.0),4294967295.0)|0)>>>0:~~+ab((aH- +(~~aH>>>0))/4294967296.0)>>>0:0;aJ=~~aH>>>0}else{aI=0;aJ=0}aH=+h[S>>3];if(aH!=123456.0){aG=aH*16777216.0;aK=+R(aG)>=1.0?aG>0.0?(ah(+Q(aG/4294967296.0),4294967295.0)|0)>>>0:~~+ab((aG- +(~~aG>>>0))/4294967296.0)>>>0:0;aL=~~aG>>>0}else{aK=0;aL=10066329}aA=0;do{if((aD|0)>(aA|0)|(aD|0)==(aA|0)&aE>>>0>2147483647>>>0){aM=0;aN=0}else{az=N2(aE,aD,aE,aD)|0;ay=G;aC=az>>>30|ay<<2;az=ay>>>30|0<<2;ay=N2(aC,az,aE,aD)|0;aB=G;aO=ay>>>30|aB<<2;ay=aB>>30|((aB|0)<0|0?-1:0)<<2;aB=0;if((aD|0)<(aB|0)|(aD|0)==(aB|0)&aE>>>0<1073741824>>>0){aB=N2(aJ,aI,-9,-1)|0;aP=NU(aB,G,201326592,0)|0;aB=G;aQ=N2(aL,aK,6,0)|0;aR=G;aS=NV(aP,aB,aQ,aR)|0;aB=N2(aS,G,aO,ay)|0;aS=G;aP=N2(aJ,aI,12,0)|0;aT=NU(aP,G,-301989888,-1)|0;aP=NU(aT,G,aQ,aR)|0;aR=N2(aP,G,aC,az)|0;aP=G;aQ=NV(100663296,0,aJ<<1|0>>>31,aI<<1|aJ>>>31)|0;aT=NU(aR,aP,aQ<<30|0>>>2,G<<30|aQ>>>2)|0;aQ=NU(aT,G,aB,aS)|0;aM=G;aN=aQ;break}else{aQ=N2(aL,aK,-6,-1)|0;aS=NV(aQ,G,aJ,aI)|0;aQ=N2(aS,G,aO,ay)|0;ay=G;aO=N2(aJ,aI,6,0)|0;aS=G;aB=N2(aL,aK,30,0)|0;aT=NU(aB,G,aO,aS)|0;aS=N2(aT,G,aC,az)|0;az=NU(aQ,ay,aS,G)|0;aS=G;ay=N2(aJ,aI,-12,-1)|0;aQ=G;aC=N2(aL,aK,-48,-1)|0;aT=NU(aC,G,ay,aQ)|0;aQ=N2(aT,G,aE,aD)|0;aT=NU(az,aS,aQ,G)|0;aQ=G;aS=N2(aL,aK,24,0)|0;az=NU(aS,G,aJ<<3|0>>>29,aI<<3|aJ>>>29)|0;aS=NU(aT,aQ,az<<30|0>>>2,G<<30|az>>>2)|0;aM=G;aN=aS;break}}}while(0);aA=N0(aN,aM,Y,X)|0;aU=G;aV=aA}else{if(!ae){aG=+h[p>>3];aH=aG!=123456.0?aG:1.0;do{if(aF<1.0){aG=+U(+(aF*3.141592653589793));if(aG<0.0){aW=aG;M=5304;break}aX=+T(+aG,+aH)}else{aW=-1.0;M=5304}}while(0);if((M|0)==5304){M=0;aX=-0.0- +T(+(-0.0-aW),+aH)}aG=aa*(aX*.5+.5);aU=+R(aG)>=1.0?aG>0.0?(ah(+Q(aG/4294967296.0),4294967295.0)|0)>>>0:~~+ab((aG- +(~~aG>>>0))/4294967296.0)>>>0:0;aV=~~aG>>>0;break}if(!af){aA=NU(aE,aD,-536870912,-1)|0;aS=N2(aA,G,ao,ap)|0;aA=G;az=-8192;do{if((aA|0)<(az|0)|(aA|0)==(az|0)&aS>>>0<0>>>0){aY=16384;aZ=0}else{aQ=8192;if(!((aA|0)<(aQ|0)|(aA|0)==(aQ|0)&aS>>>0<0>>>0)){aY=0;aZ=0;break}aQ=NV(536870912,0,aE,aD)|0;aT=N2(aQ,G,ao,ap)|0;aQ=NU(aT,G,0,8192)|0;aY=G;aZ=aQ}}while(0);aS=N2(aZ,aY,aq,ar)|0;aU=G;aV=aS;break}if(!ag){aH=+h[p>>3];aG=aa*+b4(+(aF*aF*(-0.0-(aH!=123456.0?aH:3.0))));aU=+R(aG)>=1.0?aG>0.0?(ah(+Q(aG/4294967296.0),4294967295.0)|0)>>>0:~~+ab((aG- +(~~aG>>>0))/4294967296.0)>>>0:0;aV=~~aG>>>0;break}if(!ai){if((aE|0)==0&(aD|0)==0){a_=1.0}else{aG=aF*3.141592653589793;a_=+V(+aG)/aG}aG=aa*a_;aU=+R(aG)>=1.0?aG>0.0?(ah(+Q(aG/4294967296.0),4294967295.0)|0)>>>0:~~+ab((aG- +(~~aG>>>0))/4294967296.0)>>>0:0;aV=~~aG>>>0;break}if(!aj){aG=+h[p>>3];aH=aG!=123456.0?aG:3.0;if((aE|0)==0&(aD|0)==0){a$=1.0}else{aG=aF*3.141592653589793;a0=+V(+aG);a$=a0*+V(+(aG/aH))/(aF*aF*3.141592653589793*3.141592653589793/aH)}if(aF>aH){aU=0;aV=0;break}aH=aa*a$;aU=+R(aH)>=1.0?aH>0.0?(ah(+Q(aH/4294967296.0),4294967295.0)|0)>>>0:~~+ab((aH- +(~~aH>>>0))/4294967296.0)>>>0:0;aV=~~aH>>>0;break}if(!ak){aS=NV(1073741824,0,aE,aD)|0;aA=G;az=0;aQ=(aA|0)<(az|0)|(aA|0)==(az|0)&aS>>>0<0>>>0;az=N2(aQ?0:aS,aQ?0:aA,am,an)|0;aU=G;aV=az;break}if(al){break L7314}if(aF>1.0){aH=0.0;aG=-2.196152422706632;a0=1.196152422706632;a1=aF;while(1){a2=a0*3.0;a3=a2+(aH+aG*2.0);a4=aG+a2;a2=-0.0-aH-aG*3.0-a0*6.0;a5=a1+-1.0;if(a5>1.0){aH=a3;aG=a4;a0=a2;a1=a5}else{a6=0.0;a7=a3;a8=a4;a9=a2;ba=a5;break}}}else{a6=1.0;a7=0.0;a8=-2.196152422706632;a9=1.196152422706632;ba=aF}a1=aa*(a6+ba*(a7+ba*(a8+ba*a9)));aU=+R(a1)>=1.0?a1>0.0?(ah(+Q(a1/4294967296.0),4294967295.0)|0)>>>0:~~+ab((a1- +(~~a1>>>0))/4294967296.0)>>>0:0;aV=~~a1>>>0}}while(0);az=(c[r>>2]|0)+(av+aw<<3)|0;c[az>>2]=aV;c[az+4>>2]=aU;az=av+1|0;if((az|0)<(N|0)){ax=ax+1|0;av=az}else{break}}}av=NU(at,as,I,_)|0;ax=au+1|0;if((ax|0)<(k|0)){as=G;at=av;au=ax}else{break L7312}}AN(0,0,1717904,(C=i,i=i+24|0,c[C>>2]=1659336,c[C+8>>2]=1711376,c[C+16>>2]=443,C)|0);i=C;cw();return 0}}while(0);if((N|0)>0){H=N;break}AN(0,0,1717904,(C=i,i=i+24|0,c[C>>2]=1658160,c[C+8>>2]=1711376,c[C+16>>2]=456,C)|0);i=C;cw();return 0}}while(0);x=(n|0)!=0;if(x){bb=H-1+(c[n+4>>2]|0)|0}else{bb=H}if((o|0)==0){bc=bb}else{bc=bb-1+(c[o+4>>2]|0)|0}if((bc|0)<=0){AN(0,0,1717904,(C=i,i=i+24|0,c[C>>2]=1656056,c[C+8>>2]=1711376,c[C+16>>2]=462,C)|0);i=C;cw();return 0}J=k<<3;B=ac(J,bc)|0;au=A7(B)|0;at=au;if(!((au|0)!=0|(B|0)==0)){AN(0,16,1726200,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;D=at;break}B=(k|0)>0;if(!B){A6(r);AN(0,0,1717904,(C=i,i=i+24|0,c[C>>2]=1654264,c[C+8>>2]=1711376,c[C+16>>2]=544,C)|0);i=C;cw();return 0}au=n+4|0;as=(H|0)>0;_=n|0;I=((bc-1|0)/-2|0)+((H-1|0)/2|0)|0;al=0;do{L7402:do{if(x){an=c[au>>2]|0;if((an|0)<=0){break}am=ac(al,H)|0;ak=c[r>>2]|0;aj=ac(al,bc)|0;if(as){bd=0}else{ai=0;while(1){ai=ai+1|0;if((ai|0)>=(an|0)){break L7402}}}do{ai=bd+aj|0;ag=c[_>>2]|0;ar=0;do{aq=ak+(ar+am<<3)|0;ap=at+(ai+ar<<3)|0;aa=+((c[ap>>2]|0)>>>0)+ +(c[ap+4>>2]|0)*4294967296.0+ +h[ag+(bd<<3)>>3]*(+((c[aq>>2]|0)>>>0)+ +(c[aq+4>>2]|0)*4294967296.0);aq=+R(aa)>=1.0?aa>0.0?(ah(+Q(aa/4294967296.0),4294967295.0)|0)>>>0:~~+ab((aa- +(~~aa>>>0))/4294967296.0)>>>0:0;c[ap>>2]=~~aa>>>0;c[ap+4>>2]=aq;ar=ar+1|0;}while((ar|0)<(H|0));bd=bd+1|0;}while((bd|0)<(an|0))}else{if(!as){break}an=ac(al,H)|0;am=c[r>>2]|0;ak=ac(al,bc)|0;aj=0;do{ar=am+(aj+an<<3)|0;ag=c[ar+4>>2]|0;ai=at+(aj+ak<<3)|0;c[ai>>2]=c[ar>>2];c[ai+4>>2]=ag;aj=aj+1|0;}while((aj|0)<(H|0))}}while(0);N=(c[e>>2]|0)+(al<<2)|0;c[N>>2]=I+(c[N>>2]|0);al=al+1|0;}while((al|0)<(k|0));A6(r);al=k-1|0;if(!B){AN(0,0,1717904,(C=i,i=i+24|0,c[C>>2]=1654264,c[C+8>>2]=1711376,c[C+16>>2]=544,C)|0);i=C;cw();return 0}aa=(+(y>>>0)+ +(z|0)*4294967296.0)*.002;I=(bc|0)>1;as=0;_=al;au=ac(bc,k)|0;while(1){x=ac(_,bc)|0;N=at+(x<<3)|0;aj=(_|0)<(al|0);ak=_+1|0;an=x+1|0;am=0;ag=0;ai=0;while(1){ar=c[N>>2]|0;aq=c[N+4>>2]|0;ap=-1;ao=(aq|0)>(ap|0)|(aq|0)==(ap|0)&ar>>>0>-1>>>0;ap=NV(0,0,ar,aq)|0;af=NU(ao?ar:ap,ao?aq:G,ai,ag)|0;aq=G;if(+(af>>>0)+ +(aq|0)*4294967296.0>aa){be=0;bf=0;bg=bc;break}if(aj){ao=c[e>>2]|0;if((c[ao+(_<<2)>>2]|0)>=(c[ao+(ak<<2)>>2]|0)){be=0;bf=0;bg=bc;break}}if(I){ao=1;ap=an;while(1){ar=at+(ap<<3)|0;ae=c[ar+4>>2]|0;X=at+(ap-1<<3)|0;c[X>>2]=c[ar>>2];c[X+4>>2]=ae;ae=ao+1|0;if((ae|0)<(bc|0)){ao=ae;ap=ae+x|0}else{bh=au;break}}}else{bh=an}ap=at+(bh-1<<3)|0;c[ap>>2]=0;c[ap+4>>2]=0;ap=(c[e>>2]|0)+(_<<2)|0;c[ap>>2]=(c[ap>>2]|0)+1;ap=am+1|0;if((ap|0)<(bc|0)){am=ap;ag=aq;ai=af}else{be=0;bf=0;bg=bc;break}}while(1){ai=bg-1|0;if((ai|0)<=0){break}ag=at+(ai+x<<3)|0;am=c[ag>>2]|0;an=c[ag+4>>2]|0;ag=-1;ak=(an|0)>(ag|0)|(an|0)==(ag|0)&am>>>0>-1>>>0;ag=NV(0,0,am,an)|0;aj=NU(ak?am:ag,ak?an:G,bf,be)|0;an=G;if(+(aj>>>0)+ +(an|0)*4294967296.0>aa){break}else{be=an;bf=aj;bg=ai}}bi=(bg|0)>(as|0)?bg:as;if((_|0)>0){as=bi;_=_-1|0;au=au-bc|0}else{break}}if((bi|0)<=0){AN(0,0,1717904,(C=i,i=i+24|0,c[C>>2]=1654264,c[C+8>>2]=1711376,c[C+16>>2]=544,C)|0);i=C;cw();return 0}au=A1(ac(J,bi)|0)|0;c[r>>2]=au;if((bi|0)>255|(au|0)==0){AN(0,16,1652760,(C=i,i=i+8|0,c[C>>2]=bi,C)|0);i=C;D=at;break}c[f>>2]=bi;if((m&4096|0)!=0){AN(0,40,1650944,(C=i,i=i+16|0,c[C>>2]=bc,c[C+8>>2]=bi,C)|0);i=C}do{if(B){au=c[r>>2]|0;_=(m&524288|0)==0;as=0;do{I=ac(as,bc)|0;x=ac(as,bi)|0;if(_){ai=0;do{if((ai|0)<(bc|0)){aj=at+(ai+I<<3)|0;an=c[aj+4>>2]|0;ak=au+(ai+x<<3)|0;c[ak>>2]=c[aj>>2];c[ak+4>>2]=an}else{an=au+(ai+x<<3)|0;c[an>>2]=0;c[an+4>>2]=0}ai=ai+1|0;}while((ai|0)<(bi|0))}else{ai=0;do{if((ai|0)<(bc|0)){af=at+(ai+I<<3)|0;aq=c[af+4>>2]|0;an=au+(ai+x<<3)|0;c[an>>2]=c[af>>2];c[an+4>>2]=aq}else{aq=au+(ai+x<<3)|0;c[aq>>2]=0;c[aq+4>>2]=0}ai=ai+1|0;}while((ai|0)<(bi|0))}as=as+1|0;}while((as|0)<(k|0));if(!B){break}as=bi-2|0;au=(as|0)>-1;_=j-bi|0;ai=bi-1|0;x=(bi|0)>1;I=0;do{aq=c[e>>2]|0;an=aq+(I<<2)|0;af=c[an>>2]|0;if((af|0)<0){L7472:do{if(x){ak=ac(I,bi)|0;aj=c[r>>2]|0;ag=1;am=af;while(1){N=am+ag|0;ap=aj+(ag+ak<<3)|0;ao=aj+(((N|0)>0?N:0)+ak<<3)|0;N=NU(c[ao>>2]|0,c[ao+4>>2]|0,c[ap>>2]|0,c[ap+4>>2]|0)|0;c[ao>>2]=N;c[ao+4>>2]=G;c[ap>>2]=0;c[ap+4>>2]=0;ap=ag+1|0;ao=(c[e>>2]|0)+(I<<2)|0;if((ap|0)>=(bi|0)){bj=ao;break L7472}ag=ap;am=c[ao>>2]|0}}else{bj=an}}while(0);c[bj>>2]=0;an=c[e>>2]|0;bk=an;bl=c[an+(I<<2)>>2]|0}else{bk=aq;bl=af}an=bl+bi|0;if((an|0)>(j|0)){am=an-j|0;if(au){an=ac(I,bi)|0;ag=c[r>>2]|0;ak=as;while(1){aj=ak+am|0;ao=ag+(ak+an<<3)|0;ap=ag+(((aj|0)>(ai|0)?ai:aj)+an<<3)|0;aj=NU(c[ap>>2]|0,c[ap+4>>2]|0,c[ao>>2]|0,c[ao+4>>2]|0)|0;c[ap>>2]=aj;c[ap+4>>2]=G;c[ao>>2]=0;c[ao+4>>2]=0;if((ak|0)>0){ak=ak-1|0}else{break}}bm=c[e>>2]|0}else{bm=bk}c[bm+(I<<2)>>2]=_}I=I+1|0;}while((I|0)<(k|0))}}while(0);J=A7(ac(u<<1,c[f>>2]|0)|0)|0;c[a>>2]=J;do{if((J|0)==0){if(((ac(c[f>>2]|0,u)|0)&2147483647|0)==0){break}AN(0,16,1726200,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;D=at;break L7261}}while(0);if(B){J=(l|0)/2|0;I=J;_=(J|0)<0|0?-1:0;J=l;ai=(l|0)<0|0?-1:0;as=0;do{au=ac(as,bi)|0;x=c[r>>2]|0;ak=0;an=0;ag=0;do{am=x+(ak+au<<3)|0;ag=NU(c[am>>2]|0,c[am+4>>2]|0,ag,an)|0;an=G;ak=ak+1|0;}while((ak|0)<(bi|0));ak=NU(I,_,ag,an)|0;au=N0(ak,G,J,ai)|0;ak=G;x=c[f>>2]|0;L7500:do{if((x|0)>0){am=ac(as,bi)|0;af=au>>>1|ak<<31;aq=ak>>1|((ak|0)<0|0?-1:0)<<31;ao=NV(0,0,af,aq)|0;ap=G;aj=0;N=0;ae=0;X=x;while(1){ar=(c[r>>2]|0)+(aj+am<<3)|0;Y=NU(c[ar>>2]|0,c[ar+4>>2]|0,ae,N)|0;ar=G;S=0;P=(ar|0)>(S|0)|(ar|0)==(S|0)&Y>>>0>0>>>0;S=NU(P?af:ao,P?aq:ap,Y,ar)|0;P=N0(S,G,au,ak)|0;S=(ac(X,as)|0)+aj|0;b[(c[a>>2]|0)+(S<<1)>>1]=P&65535;S=aj+1|0;O=c[f>>2]|0;if((S|0)>=(O|0)){break L7500}Z=P;P=N2(Z,(Z|0)<0|0?-1:0,au,ak)|0;Z=NV(Y,ar,P,G)|0;aj=S;N=G;ae=Z;X=O}}}while(0);as=as+1|0;}while((as|0)<(k|0))}as=c[e>>2]|0;ai=c[as+(al<<2)>>2]|0;c[as+(k+2<<2)>>2]=ai;c[(c[e>>2]|0)+(k+1<<2)>>2]=ai;c[(c[e>>2]|0)+(k<<2)>>2]=ai;ai=c[f>>2]|0;if((ai|0)>0){bn=0;bo=ai}else{bp=0;bq=at;br=c[r>>2]|0;bs=br;A4(bs);bt=bq;A4(bt);i=q;return bp|0}while(1){ai=(ac(bo,al)|0)+bn|0;as=c[a>>2]|0;J=b[as+(ai<<1)>>1]|0;b[as+(ai+(bo*3|0)<<1)>>1]=J;b[(c[a>>2]|0)+((c[f>>2]<<1)+ai<<1)>>1]=J;b[(c[a>>2]|0)+((c[f>>2]|0)+ai<<1)>>1]=J;J=bn+1|0;ai=c[f>>2]|0;if((J|0)<(ai|0)){bn=J;bo=ai}else{bp=0;bq=at;break}}br=c[r>>2]|0;bs=br;A4(bs);bt=bq;A4(bt);i=q;return bp|0}else{AN(0,16,1726200,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;D=0}}while(0);AN(0,16,1649400,(C=i,i=i+1|0,i=i+7&-8,c[C>>2]=0,C)|0);i=C;bp=-1;bq=D;br=c[r>>2]|0;bs=br;A4(bs);bt=bq;A4(bt);i=q;return bp|0}function ta(a,b,d,e,f,g,i,j,k,l){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;m=A7(23936)|0;n=m;if((m|0)==0){o=0;return o|0}c[m>>2]=70768;Br(m);c[m+2284>>2]=i;c[m+8>>2]=a;c[m+12>>2]=b;c[m+19592>>2]=e;c[m+16>>2]=f;switch(d|0){case 8:{p=1;q=8;break};case 13:{p=1;q=4;break};case 34:{p=1;q=33;break};case 320:{p=1;q=7;break};case 12:{p=1;q=0;break};case 14:{p=1;q=5;break};default:{p=0;q=d}}c[m+11260>>2]=p;switch(g|0){case 320:{r=1;s=7;break};case 12:{r=1;s=0;break};case 13:{r=1;s=4;break};case 14:{r=1;s=5;break};case 34:{r=1;s=33;break};case 8:{r=1;s=8;break};default:{r=0;s=g}}c[m+11264>>2]=r;if((q|0)==297){t=1;u=29}else if((q|0)==298){t=4;u=30}else if((q|0)==295){t=1;u=27}else if((q|0)==296){t=4;u=28}else{t=0;u=q}c[m+11268>>2]=t;if((s|0)==297){v=1;w=29}else if((s|0)==298){v=4;w=30}else if((s|0)==295){v=1;w=27}else if((s|0)==296){v=4;w=28}else{v=0;w=s}c[m+11272>>2]=v;if((u|0)==111){x=1;y=41}else if((u|0)==110){x=1;y=42}else{x=0;y=u}c[m+11276>>2]=x;if((w|0)==111){z=1;A=41}else if((w|0)==110){z=1;A=42}else{z=0;A=w}c[m+11280>>2]=z;c[m+56>>2]=y;c[m+52>>2]=A;if((l|0)!=0){h[m+104>>3]=+h[l>>3];h[m+112>>3]=+h[l+8>>3]}s7(n,236600,p,236600,r,0,65536,65536)|0;if((s8(n,j,k)|0)>=0){o=n;return o|0}tb(n);o=0;return o|0}function tb(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0;if((a|0)==0){return}b=a+2168|0;d=c[b>>2]|0;if((d|0)!=0){e=a+2184|0;L7567:do{if((c[e>>2]|0)>0){f=0;g=d;while(1){A6(g+(f<<2)|0);h=f+1|0;if((h|0)>=(c[e>>2]|0)){break L7567}f=h;g=c[b>>2]|0}}}while(0);A6(b)}b=a+2172|0;e=c[b>>2]|0;if((e|0)!=0){d=a+2188|0;L7575:do{if((c[d>>2]|0)>0){g=0;f=e;while(1){A6(f+(g<<2)|0);h=g+1|0;if((h|0)>=(c[d>>2]|0)){break L7575}g=h;f=c[b>>2]|0}}}while(0);A6(b);A6(a+2176|0)}b=a+2180|0;d=c[b>>2]|0;if((d|0)!=0){e=a+2184|0;L7583:do{if((c[e>>2]|0)>0){f=0;g=d;while(1){A6(g+(f<<2)|0);h=f+1|0;if((h|0)>=(c[e>>2]|0)){break L7583}f=h;g=c[b>>2]|0}}}while(0);A6(b)}A6(a+11200|0);A6(a+11204|0);A6(a+11208|0);A6(a+11212|0);A6(a+2220|0);A6(a+2224|0);A6(a+2212|0);A6(a+2216|0);A6(a+2236|0);A6(a+2240|0);A6(a+2228|0);A6(a+2232|0);A6(a+2288|0);A6(a+2208|0);A4(a);return}function tc(b){b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0;d=i;e=b+56|0;f=BL(c[e>>2]|0)|0;g=b+52|0;h=BL(c[g>>2]|0)|0;AN(b,24,1640160,(b=i,i=i+16|0,c[b>>2]=f,c[b+8>>2]=h,b)|0);i=b;L7591:do{switch(c[g>>2]|0){case 22:case 19:{j=10;break};case 3:{j=12;break};case 28:case 30:{k=5488;break};case 62:case 64:{j=34;break};case 24:case 21:{j=48;break};case 23:case 20:{j=30;break};case 67:case 68:{j=38;break};case 10:{j=40;break};case 46:case 50:{j=14;break};case 2:{j=26;break};case 27:case 29:{h=c[e>>2]|0;f=BP(h)|0;if((f|0)==0){AN(0,0,1691032,(b=i,i=i+24|0,c[b>>2]=1671176,c[b+8>>2]=1656416,c[b+16>>2]=744,b)|0);i=b;cw();return 0}if((h|0)==11){j=58;break L7591}if((a[f+7|0]|0)<0){j=58}else{k=5488}break};case 44:case 48:{j=42;break};case 41:case 42:{j=8;break};default:{j=0}}}while(0);do{if((k|0)==5488){g=c[e>>2]|0;f=BP(g)|0;if((f|0)==0){AN(0,0,1691032,(b=i,i=i+24|0,c[b>>2]=1671176,c[b+8>>2]=1656416,c[b+16>>2]=744,b)|0);i=b;cw();return 0}if((g|0)==11){j=54;break}j=(a[f+7|0]|0)<0?54:64}}while(0);i=d;return j|0}function td(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0;if((c[b+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=e+4|0;l=f+4|0;m=e+8|0;n=f+8|0;o=b+19592|0;p=g+1|0;q=g;r=1;s=0;while(1){t=c[i>>2]|0;u=s+g|0;v=c[j>>2]|0;w=t+(ac(v,u)|0)|0;x=t+(ac(v,u+1|0)|0)|0;u=c[e>>2]|0;y=c[f>>2]|0;z=ac(y,s)|0;A=u+z|0;B=u+(z+y)|0;C=c[k>>2]|0;D=s>>1;E=ac(c[l>>2]|0,D)|0;F=C+E|0;G=c[m>>2]|0;H=ac(c[n>>2]|0,D)|0;D=G+H|0;I=c[o>>2]|0;J=I>>3;if((J|0)==0){K=D;L=F;M=B;N=A;O=x;P=w;Q=I}else{I=J*48|0;R=(ac(v,p)|0)+I|0;S=(ac(v,q)|0)+I|0;I=J<<3;v=(ac(y,r)|0)+I|0;y=z+I|0;I=J<<2;z=H+I|0;H=E+I|0;I=D;D=J;J=F;F=B;B=A;A=x;x=w;while(1){w=D-1|0;E=(d[I]|0)+128|0;T=c[b+2292+(E<<2)>>2]|0;U=(d[J]|0)+128|0;V=c[b+4340+(U<<2)>>2]|0;W=c[b+6388+(E<<2)>>2]|0;E=c[b+8436+(U<<2)>>2]|0;U=d[B]|0;X=a[E+U|0]|0;a[x+1|0]=X;a[x]=X;X=a[V+(U+W)|0]|0;a[x+3|0]=X;a[x+2|0]=X;X=a[T+U|0]|0;a[x+5|0]=X;a[x+4|0]=X;X=d[B+1|0]|0;U=a[E+X|0]|0;a[x+7|0]=U;a[x+6|0]=U;U=a[V+(X+W)|0]|0;a[x+9|0]=U;a[x+8|0]=U;U=a[T+X|0]|0;a[x+11|0]=U;a[x+10|0]=U;U=d[F]|0;X=a[E+U|0]|0;a[A+1|0]=X;a[A]=X;X=a[V+(U+W)|0]|0;a[A+3|0]=X;a[A+2|0]=X;X=a[T+U|0]|0;a[A+5|0]=X;a[A+4|0]=X;X=d[F+1|0]|0;U=a[E+X|0]|0;a[A+7|0]=U;a[A+6|0]=U;U=a[V+(X+W)|0]|0;a[A+9|0]=U;a[A+8|0]=U;U=a[T+X|0]|0;a[A+11|0]=U;a[A+10|0]=U;U=(d[I+1|0]|0)+128|0;X=c[b+2292+(U<<2)>>2]|0;T=(d[J+1|0]|0)+128|0;W=c[b+4340+(T<<2)>>2]|0;V=c[b+6388+(U<<2)>>2]|0;U=c[b+8436+(T<<2)>>2]|0;T=d[F+2|0]|0;E=a[U+T|0]|0;a[A+13|0]=E;a[A+12|0]=E;E=a[W+(T+V)|0]|0;a[A+15|0]=E;a[A+14|0]=E;E=a[X+T|0]|0;a[A+17|0]=E;a[A+16|0]=E;E=d[F+3|0]|0;T=a[U+E|0]|0;a[A+19|0]=T;a[A+18|0]=T;T=a[W+(E+V)|0]|0;a[A+21|0]=T;a[A+20|0]=T;T=a[X+E|0]|0;a[A+23|0]=T;a[A+22|0]=T;T=d[B+2|0]|0;E=a[U+T|0]|0;a[x+13|0]=E;a[x+12|0]=E;E=a[W+(T+V)|0]|0;a[x+15|0]=E;a[x+14|0]=E;E=a[X+T|0]|0;a[x+17|0]=E;a[x+16|0]=E;E=d[B+3|0]|0;T=a[U+E|0]|0;a[x+19|0]=T;a[x+18|0]=T;T=a[W+(E+V)|0]|0;a[x+21|0]=T;a[x+20|0]=T;T=a[X+E|0]|0;a[x+23|0]=T;a[x+22|0]=T;T=(d[I+2|0]|0)+128|0;E=c[b+2292+(T<<2)>>2]|0;X=(d[J+2|0]|0)+128|0;V=c[b+4340+(X<<2)>>2]|0;W=c[b+6388+(T<<2)>>2]|0;T=c[b+8436+(X<<2)>>2]|0;X=d[B+4|0]|0;U=a[T+X|0]|0;a[x+25|0]=U;a[x+24|0]=U;U=a[V+(X+W)|0]|0;a[x+27|0]=U;a[x+26|0]=U;U=a[E+X|0]|0;a[x+29|0]=U;a[x+28|0]=U;U=d[B+5|0]|0;X=a[T+U|0]|0;a[x+31|0]=X;a[x+30|0]=X;X=a[V+(U+W)|0]|0;a[x+33|0]=X;a[x+32|0]=X;X=a[E+U|0]|0;a[x+35|0]=X;a[x+34|0]=X;X=d[F+4|0]|0;U=a[T+X|0]|0;a[A+25|0]=U;a[A+24|0]=U;U=a[V+(X+W)|0]|0;a[A+27|0]=U;a[A+26|0]=U;U=a[E+X|0]|0;a[A+29|0]=U;a[A+28|0]=U;U=d[F+5|0]|0;X=a[T+U|0]|0;a[A+31|0]=X;a[A+30|0]=X;X=a[V+(U+W)|0]|0;a[A+33|0]=X;a[A+32|0]=X;X=a[E+U|0]|0;a[A+35|0]=X;a[A+34|0]=X;X=(d[I+3|0]|0)+128|0;U=c[b+2292+(X<<2)>>2]|0;E=(d[J+3|0]|0)+128|0;W=c[b+4340+(E<<2)>>2]|0;V=c[b+6388+(X<<2)>>2]|0;X=c[b+8436+(E<<2)>>2]|0;E=d[F+6|0]|0;T=a[X+E|0]|0;a[A+37|0]=T;a[A+36|0]=T;T=a[W+(E+V)|0]|0;a[A+39|0]=T;a[A+38|0]=T;T=a[U+E|0]|0;a[A+41|0]=T;a[A+40|0]=T;T=d[F+7|0]|0;E=a[X+T|0]|0;a[A+43|0]=E;a[A+42|0]=E;E=a[W+(T+V)|0]|0;a[A+45|0]=E;a[A+44|0]=E;E=a[U+T|0]|0;a[A+47|0]=E;a[A+46|0]=E;E=d[B+6|0]|0;T=a[X+E|0]|0;a[x+37|0]=T;a[x+36|0]=T;T=a[W+(E+V)|0]|0;a[x+39|0]=T;a[x+38|0]=T;T=a[U+E|0]|0;a[x+41|0]=T;a[x+40|0]=T;T=d[B+7|0]|0;E=a[X+T|0]|0;a[x+43|0]=E;a[x+42|0]=E;E=a[W+(T+V)|0]|0;a[x+45|0]=E;a[x+44|0]=E;E=a[U+T|0]|0;a[x+47|0]=E;a[x+46|0]=E;if((w|0)==0){break}else{I=I+4|0;D=w;J=J+4|0;F=F+8|0;B=B+8|0;A=A+48|0;x=x+48|0}}K=G+z|0;L=C+H|0;M=u+v|0;N=u+y|0;O=t+R|0;P=t+S|0;Q=c[o>>2]|0}if((Q&4|0)==0){Y=P;Z=O;_=N;$=M;aa=L;ab=K;ad=Q}else{x=(d[K]|0)+128|0;A=c[b+2292+(x<<2)>>2]|0;B=(d[L]|0)+128|0;F=c[b+4340+(B<<2)>>2]|0;J=c[b+6388+(x<<2)>>2]|0;x=c[b+8436+(B<<2)>>2]|0;B=d[N]|0;D=a[x+B|0]|0;a[P+1|0]=D;a[P]=D;D=a[F+(B+J)|0]|0;a[P+3|0]=D;a[P+2|0]=D;D=a[A+B|0]|0;a[P+5|0]=D;a[P+4|0]=D;D=d[N+1|0]|0;B=a[x+D|0]|0;a[P+7|0]=B;a[P+6|0]=B;B=a[F+(D+J)|0]|0;a[P+9|0]=B;a[P+8|0]=B;B=a[A+D|0]|0;a[P+11|0]=B;a[P+10|0]=B;B=d[M]|0;D=a[x+B|0]|0;a[O+1|0]=D;a[O]=D;D=a[F+(B+J)|0]|0;a[O+3|0]=D;a[O+2|0]=D;D=a[A+B|0]|0;a[O+5|0]=D;a[O+4|0]=D;D=d[M+1|0]|0;B=a[x+D|0]|0;a[O+7|0]=B;a[O+6|0]=B;B=a[F+(D+J)|0]|0;a[O+9|0]=B;a[O+8|0]=B;B=a[A+D|0]|0;a[O+11|0]=B;a[O+10|0]=B;B=(d[K+1|0]|0)+128|0;D=c[b+2292+(B<<2)>>2]|0;A=(d[L+1|0]|0)+128|0;J=c[b+4340+(A<<2)>>2]|0;F=c[b+6388+(B<<2)>>2]|0;B=c[b+8436+(A<<2)>>2]|0;A=d[M+2|0]|0;x=a[B+A|0]|0;a[O+13|0]=x;a[O+12|0]=x;x=a[J+(A+F)|0]|0;a[O+15|0]=x;a[O+14|0]=x;x=a[D+A|0]|0;a[O+17|0]=x;a[O+16|0]=x;x=d[M+3|0]|0;A=a[B+x|0]|0;a[O+19|0]=A;a[O+18|0]=A;A=a[J+(x+F)|0]|0;a[O+21|0]=A;a[O+20|0]=A;A=a[D+x|0]|0;a[O+23|0]=A;a[O+22|0]=A;A=d[N+2|0]|0;x=a[B+A|0]|0;a[P+13|0]=x;a[P+12|0]=x;x=a[J+(A+F)|0]|0;a[P+15|0]=x;a[P+14|0]=x;x=a[D+A|0]|0;a[P+17|0]=x;a[P+16|0]=x;x=d[N+3|0]|0;A=a[B+x|0]|0;a[P+19|0]=A;a[P+18|0]=A;A=a[J+(x+F)|0]|0;a[P+21|0]=A;a[P+20|0]=A;A=a[D+x|0]|0;a[P+23|0]=A;a[P+22|0]=A;Y=P+24|0;Z=O+24|0;_=N+4|0;$=M+4|0;aa=L+2|0;ab=K+2|0;ad=c[o>>2]|0}if((ad&2|0)!=0){A=(d[ab]|0)+128|0;x=c[b+2292+(A<<2)>>2]|0;D=(d[aa]|0)+128|0;F=c[b+4340+(D<<2)>>2]|0;J=c[b+6388+(A<<2)>>2]|0;A=c[b+8436+(D<<2)>>2]|0;D=d[_]|0;B=a[A+D|0]|0;a[Y+1|0]=B;a[Y]=B;B=a[F+(D+J)|0]|0;a[Y+3|0]=B;a[Y+2|0]=B;B=a[x+D|0]|0;a[Y+5|0]=B;a[Y+4|0]=B;B=d[_+1|0]|0;D=a[A+B|0]|0;a[Y+7|0]=D;a[Y+6|0]=D;D=a[F+(B+J)|0]|0;a[Y+9|0]=D;a[Y+8|0]=D;D=a[x+B|0]|0;a[Y+11|0]=D;a[Y+10|0]=D;D=d[$]|0;B=a[A+D|0]|0;a[Z+1|0]=B;a[Z]=B;B=a[F+(D+J)|0]|0;a[Z+3|0]=B;a[Z+2|0]=B;B=a[x+D|0]|0;a[Z+5|0]=B;a[Z+4|0]=B;B=d[$+1|0]|0;D=a[A+B|0]|0;a[Z+7|0]=D;a[Z+6|0]=D;D=a[F+(B+J)|0]|0;a[Z+9|0]=D;a[Z+8|0]=D;D=a[x+B|0]|0;a[Z+11|0]=D;a[Z+10|0]=D}D=s+2|0;if((D|0)>=(h|0)){break}p=p+2|0;q=q+2|0;r=r+2|0;s=D}return h|0}function te(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0;if((c[b+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=e+4|0;l=f+4|0;m=e+8|0;n=f+8|0;o=b+19592|0;p=g+1|0;q=g;r=1;s=0;while(1){t=c[i>>2]|0;u=s+g|0;v=c[j>>2]|0;w=t+(ac(v,u)|0)|0;x=t+(ac(v,u+1|0)|0)|0;u=c[e>>2]|0;y=c[f>>2]|0;z=ac(y,s)|0;A=u+z|0;B=u+(z+y)|0;C=c[k>>2]|0;D=s>>1;E=ac(c[l>>2]|0,D)|0;F=C+E|0;G=c[m>>2]|0;H=ac(c[n>>2]|0,D)|0;D=G+H|0;I=c[o>>2]|0;J=I>>3;if((J|0)==0){K=D;L=F;M=B;N=A;O=x;P=w;Q=I}else{I=J*48|0;R=(ac(v,p)|0)+I|0;S=(ac(v,q)|0)+I|0;I=J<<3;v=(ac(y,r)|0)+I|0;y=z+I|0;I=J<<2;z=H+I|0;H=E+I|0;I=D;D=J;J=F;F=B;B=A;A=x;x=w;while(1){w=D-1|0;E=(d[I]|0)+128|0;T=c[b+2292+(E<<2)>>2]|0;U=(d[J]|0)+128|0;V=c[b+4340+(U<<2)>>2]|0;W=c[b+6388+(E<<2)>>2]|0;E=c[b+8436+(U<<2)>>2]|0;U=d[B]|0;X=a[T+U|0]|0;a[x+1|0]=X;a[x]=X;X=a[V+(U+W)|0]|0;a[x+3|0]=X;a[x+2|0]=X;X=a[E+U|0]|0;a[x+5|0]=X;a[x+4|0]=X;X=d[B+1|0]|0;U=a[T+X|0]|0;a[x+7|0]=U;a[x+6|0]=U;U=a[V+(X+W)|0]|0;a[x+9|0]=U;a[x+8|0]=U;U=a[E+X|0]|0;a[x+11|0]=U;a[x+10|0]=U;U=d[F]|0;X=a[T+U|0]|0;a[A+1|0]=X;a[A]=X;X=a[V+(U+W)|0]|0;a[A+3|0]=X;a[A+2|0]=X;X=a[E+U|0]|0;a[A+5|0]=X;a[A+4|0]=X;X=d[F+1|0]|0;U=a[T+X|0]|0;a[A+7|0]=U;a[A+6|0]=U;U=a[V+(X+W)|0]|0;a[A+9|0]=U;a[A+8|0]=U;U=a[E+X|0]|0;a[A+11|0]=U;a[A+10|0]=U;U=(d[I+1|0]|0)+128|0;X=c[b+2292+(U<<2)>>2]|0;E=(d[J+1|0]|0)+128|0;W=c[b+4340+(E<<2)>>2]|0;V=c[b+6388+(U<<2)>>2]|0;U=c[b+8436+(E<<2)>>2]|0;E=d[F+2|0]|0;T=a[X+E|0]|0;a[A+13|0]=T;a[A+12|0]=T;T=a[W+(E+V)|0]|0;a[A+15|0]=T;a[A+14|0]=T;T=a[U+E|0]|0;a[A+17|0]=T;a[A+16|0]=T;T=d[F+3|0]|0;E=a[X+T|0]|0;a[A+19|0]=E;a[A+18|0]=E;E=a[W+(T+V)|0]|0;a[A+21|0]=E;a[A+20|0]=E;E=a[U+T|0]|0;a[A+23|0]=E;a[A+22|0]=E;E=d[B+2|0]|0;T=a[X+E|0]|0;a[x+13|0]=T;a[x+12|0]=T;T=a[W+(E+V)|0]|0;a[x+15|0]=T;a[x+14|0]=T;T=a[U+E|0]|0;a[x+17|0]=T;a[x+16|0]=T;T=d[B+3|0]|0;E=a[X+T|0]|0;a[x+19|0]=E;a[x+18|0]=E;E=a[W+(T+V)|0]|0;a[x+21|0]=E;a[x+20|0]=E;E=a[U+T|0]|0;a[x+23|0]=E;a[x+22|0]=E;E=(d[I+2|0]|0)+128|0;T=c[b+2292+(E<<2)>>2]|0;U=(d[J+2|0]|0)+128|0;V=c[b+4340+(U<<2)>>2]|0;W=c[b+6388+(E<<2)>>2]|0;E=c[b+8436+(U<<2)>>2]|0;U=d[B+4|0]|0;X=a[T+U|0]|0;a[x+25|0]=X;a[x+24|0]=X;X=a[V+(U+W)|0]|0;a[x+27|0]=X;a[x+26|0]=X;X=a[E+U|0]|0;a[x+29|0]=X;a[x+28|0]=X;X=d[B+5|0]|0;U=a[T+X|0]|0;a[x+31|0]=U;a[x+30|0]=U;U=a[V+(X+W)|0]|0;a[x+33|0]=U;a[x+32|0]=U;U=a[E+X|0]|0;a[x+35|0]=U;a[x+34|0]=U;U=d[F+4|0]|0;X=a[T+U|0]|0;a[A+25|0]=X;a[A+24|0]=X;X=a[V+(U+W)|0]|0;a[A+27|0]=X;a[A+26|0]=X;X=a[E+U|0]|0;a[A+29|0]=X;a[A+28|0]=X;X=d[F+5|0]|0;U=a[T+X|0]|0;a[A+31|0]=U;a[A+30|0]=U;U=a[V+(X+W)|0]|0;a[A+33|0]=U;a[A+32|0]=U;U=a[E+X|0]|0;a[A+35|0]=U;a[A+34|0]=U;U=(d[I+3|0]|0)+128|0;X=c[b+2292+(U<<2)>>2]|0;E=(d[J+3|0]|0)+128|0;W=c[b+4340+(E<<2)>>2]|0;V=c[b+6388+(U<<2)>>2]|0;U=c[b+8436+(E<<2)>>2]|0;E=d[F+6|0]|0;T=a[X+E|0]|0;a[A+37|0]=T;a[A+36|0]=T;T=a[W+(E+V)|0]|0;a[A+39|0]=T;a[A+38|0]=T;T=a[U+E|0]|0;a[A+41|0]=T;a[A+40|0]=T;T=d[F+7|0]|0;E=a[X+T|0]|0;a[A+43|0]=E;a[A+42|0]=E;E=a[W+(T+V)|0]|0;a[A+45|0]=E;a[A+44|0]=E;E=a[U+T|0]|0;a[A+47|0]=E;a[A+46|0]=E;E=d[B+6|0]|0;T=a[X+E|0]|0;a[x+37|0]=T;a[x+36|0]=T;T=a[W+(E+V)|0]|0;a[x+39|0]=T;a[x+38|0]=T;T=a[U+E|0]|0;a[x+41|0]=T;a[x+40|0]=T;T=d[B+7|0]|0;E=a[X+T|0]|0;a[x+43|0]=E;a[x+42|0]=E;E=a[W+(T+V)|0]|0;a[x+45|0]=E;a[x+44|0]=E;E=a[U+T|0]|0;a[x+47|0]=E;a[x+46|0]=E;if((w|0)==0){break}else{I=I+4|0;D=w;J=J+4|0;F=F+8|0;B=B+8|0;A=A+48|0;x=x+48|0}}K=G+z|0;L=C+H|0;M=u+v|0;N=u+y|0;O=t+R|0;P=t+S|0;Q=c[o>>2]|0}if((Q&4|0)==0){Y=P;Z=O;_=N;$=M;aa=L;ab=K;ad=Q}else{x=(d[K]|0)+128|0;A=c[b+2292+(x<<2)>>2]|0;B=(d[L]|0)+128|0;F=c[b+4340+(B<<2)>>2]|0;J=c[b+6388+(x<<2)>>2]|0;x=c[b+8436+(B<<2)>>2]|0;B=d[N]|0;D=a[A+B|0]|0;a[P+1|0]=D;a[P]=D;D=a[F+(B+J)|0]|0;a[P+3|0]=D;a[P+2|0]=D;D=a[x+B|0]|0;a[P+5|0]=D;a[P+4|0]=D;D=d[N+1|0]|0;B=a[A+D|0]|0;a[P+7|0]=B;a[P+6|0]=B;B=a[F+(D+J)|0]|0;a[P+9|0]=B;a[P+8|0]=B;B=a[x+D|0]|0;a[P+11|0]=B;a[P+10|0]=B;B=d[M]|0;D=a[A+B|0]|0;a[O+1|0]=D;a[O]=D;D=a[F+(B+J)|0]|0;a[O+3|0]=D;a[O+2|0]=D;D=a[x+B|0]|0;a[O+5|0]=D;a[O+4|0]=D;D=d[M+1|0]|0;B=a[A+D|0]|0;a[O+7|0]=B;a[O+6|0]=B;B=a[F+(D+J)|0]|0;a[O+9|0]=B;a[O+8|0]=B;B=a[x+D|0]|0;a[O+11|0]=B;a[O+10|0]=B;B=(d[K+1|0]|0)+128|0;D=c[b+2292+(B<<2)>>2]|0;x=(d[L+1|0]|0)+128|0;J=c[b+4340+(x<<2)>>2]|0;F=c[b+6388+(B<<2)>>2]|0;B=c[b+8436+(x<<2)>>2]|0;x=d[M+2|0]|0;A=a[D+x|0]|0;a[O+13|0]=A;a[O+12|0]=A;A=a[J+(x+F)|0]|0;a[O+15|0]=A;a[O+14|0]=A;A=a[B+x|0]|0;a[O+17|0]=A;a[O+16|0]=A;A=d[M+3|0]|0;x=a[D+A|0]|0;a[O+19|0]=x;a[O+18|0]=x;x=a[J+(A+F)|0]|0;a[O+21|0]=x;a[O+20|0]=x;x=a[B+A|0]|0;a[O+23|0]=x;a[O+22|0]=x;x=d[N+2|0]|0;A=a[D+x|0]|0;a[P+13|0]=A;a[P+12|0]=A;A=a[J+(x+F)|0]|0;a[P+15|0]=A;a[P+14|0]=A;A=a[B+x|0]|0;a[P+17|0]=A;a[P+16|0]=A;A=d[N+3|0]|0;x=a[D+A|0]|0;a[P+19|0]=x;a[P+18|0]=x;x=a[J+(A+F)|0]|0;a[P+21|0]=x;a[P+20|0]=x;x=a[B+A|0]|0;a[P+23|0]=x;a[P+22|0]=x;Y=P+24|0;Z=O+24|0;_=N+4|0;$=M+4|0;aa=L+2|0;ab=K+2|0;ad=c[o>>2]|0}if((ad&2|0)!=0){x=(d[ab]|0)+128|0;A=c[b+2292+(x<<2)>>2]|0;B=(d[aa]|0)+128|0;F=c[b+4340+(B<<2)>>2]|0;J=c[b+6388+(x<<2)>>2]|0;x=c[b+8436+(B<<2)>>2]|0;B=d[_]|0;D=a[A+B|0]|0;a[Y+1|0]=D;a[Y]=D;D=a[F+(B+J)|0]|0;a[Y+3|0]=D;a[Y+2|0]=D;D=a[x+B|0]|0;a[Y+5|0]=D;a[Y+4|0]=D;D=d[_+1|0]|0;B=a[A+D|0]|0;a[Y+7|0]=B;a[Y+6|0]=B;B=a[F+(D+J)|0]|0;a[Y+9|0]=B;a[Y+8|0]=B;B=a[x+D|0]|0;a[Y+11|0]=B;a[Y+10|0]=B;B=d[$]|0;D=a[A+B|0]|0;a[Z+1|0]=D;a[Z]=D;D=a[F+(B+J)|0]|0;a[Z+3|0]=D;a[Z+2|0]=D;D=a[x+B|0]|0;a[Z+5|0]=D;a[Z+4|0]=D;D=d[$+1|0]|0;B=a[A+D|0]|0;a[Z+7|0]=B;a[Z+6|0]=B;B=a[F+(D+J)|0]|0;a[Z+9|0]=B;a[Z+8|0]=B;B=a[x+D|0]|0;a[Z+11|0]=B;a[Z+10|0]=B}B=s+2|0;if((B|0)>=(h|0)){break}p=p+2|0;q=q+2|0;r=r+2|0;s=B}return h|0}function tf(a,b,e,f,g,h,i){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0;if((g|0)<=0){return g|0}j=b+4|0;k=e+4|0;l=b+8|0;m=e+8|0;n=a+19592|0;o=b+12|0;p=e+12|0;q=f+1|0;r=f;s=1;t=0;while(1){u=c[h>>2]|0;v=t+f|0;w=c[i>>2]|0;x=u+(ac(w,v)|0)|0;y=u+(ac(w,v+1|0)|0)|0;v=c[b>>2]|0;z=c[e>>2]|0;A=ac(z,t)|0;B=v+A|0;C=v+(A+z)|0;D=c[j>>2]|0;E=t>>1;F=ac(c[k>>2]|0,E)|0;G=D+F|0;H=c[l>>2]|0;I=ac(c[m>>2]|0,E)|0;E=H+I|0;J=c[n>>2]|0;K=J>>3;L=c[o>>2]|0;M=c[p>>2]|0;N=ac(M,t)|0;O=L+N|0;P=L+(N+M)|0;if((K|0)==0){Q=E;R=O;S=P;T=G;U=C;V=B;W=y;X=x;Y=J}else{J=K<<5;Z=u+((ac(w,q)|0)+J)|0;_=u+((ac(w,r)|0)+J)|0;J=K<<3;w=(ac(z,s)|0)+J|0;z=A+J|0;A=K<<2;u=I+A|0;I=F+A|0;A=(ac(M,s)|0)+J|0;M=N+J|0;J=E;E=O;O=P;P=K;K=G;G=C;C=B;B=y;y=x;while(1){x=P-1|0;N=(d[J]|0)+128|0;F=c[a+2292+(N<<2)>>2]|0;$=(d[K]|0)+128|0;aa=(c[a+4340+($<<2)>>2]|0)+(c[a+6388+(N<<2)>>2]|0)|0;N=c[a+8436+($<<2)>>2]|0;$=d[C]|0;c[y>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+(d[E]|0);$=d[C+1|0]|0;c[y+4>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+(d[E+1|0]|0);$=d[G]|0;c[B>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+(d[O]|0);$=d[G+1|0]|0;c[B+4>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+(d[O+1|0]|0);$=(d[J+1|0]|0)+128|0;N=c[a+2292+($<<2)>>2]|0;F=(d[K+1|0]|0)+128|0;aa=(c[a+4340+(F<<2)>>2]|0)+(c[a+6388+($<<2)>>2]|0)|0;$=c[a+8436+(F<<2)>>2]|0;F=d[G+2|0]|0;c[B+8>>2]=(c[aa+(F<<2)>>2]|0)+(c[N+(F<<2)>>2]|0)+(c[$+(F<<2)>>2]|0)+(d[O+2|0]|0);F=d[G+3|0]|0;c[B+12>>2]=(c[aa+(F<<2)>>2]|0)+(c[N+(F<<2)>>2]|0)+(c[$+(F<<2)>>2]|0)+(d[O+3|0]|0);F=d[C+2|0]|0;c[y+8>>2]=(c[aa+(F<<2)>>2]|0)+(c[N+(F<<2)>>2]|0)+(c[$+(F<<2)>>2]|0)+(d[E+2|0]|0);F=d[C+3|0]|0;c[y+12>>2]=(c[aa+(F<<2)>>2]|0)+(c[N+(F<<2)>>2]|0)+(c[$+(F<<2)>>2]|0)+(d[E+3|0]|0);F=(d[J+2|0]|0)+128|0;$=c[a+2292+(F<<2)>>2]|0;N=(d[K+2|0]|0)+128|0;aa=(c[a+4340+(N<<2)>>2]|0)+(c[a+6388+(F<<2)>>2]|0)|0;F=c[a+8436+(N<<2)>>2]|0;N=d[C+4|0]|0;c[y+16>>2]=(c[aa+(N<<2)>>2]|0)+(c[$+(N<<2)>>2]|0)+(c[F+(N<<2)>>2]|0)+(d[E+4|0]|0);N=d[C+5|0]|0;c[y+20>>2]=(c[aa+(N<<2)>>2]|0)+(c[$+(N<<2)>>2]|0)+(c[F+(N<<2)>>2]|0)+(d[E+5|0]|0);N=d[G+4|0]|0;c[B+16>>2]=(c[aa+(N<<2)>>2]|0)+(c[$+(N<<2)>>2]|0)+(c[F+(N<<2)>>2]|0)+(d[O+4|0]|0);N=d[G+5|0]|0;c[B+20>>2]=(c[aa+(N<<2)>>2]|0)+(c[$+(N<<2)>>2]|0)+(c[F+(N<<2)>>2]|0)+(d[O+5|0]|0);N=(d[J+3|0]|0)+128|0;F=c[a+2292+(N<<2)>>2]|0;$=(d[K+3|0]|0)+128|0;aa=(c[a+4340+($<<2)>>2]|0)+(c[a+6388+(N<<2)>>2]|0)|0;N=c[a+8436+($<<2)>>2]|0;$=d[G+6|0]|0;c[B+24>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+(d[O+6|0]|0);$=d[G+7|0]|0;c[B+28>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+(d[O+7|0]|0);$=d[C+6|0]|0;c[y+24>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+(d[E+6|0]|0);$=d[C+7|0]|0;c[y+28>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+(d[E+7|0]|0);if((x|0)==0){break}else{J=J+4|0;E=E+8|0;O=O+8|0;P=x;K=K+4|0;G=G+8|0;C=C+8|0;B=B+32|0;y=y+32|0}}Q=H+u|0;R=L+M|0;S=L+A|0;T=D+I|0;U=v+w|0;V=v+z|0;W=Z;X=_;Y=c[n>>2]|0}if((Y&4|0)==0){ab=X;ad=W;ae=V;af=U;ag=T;ah=S;ai=R;aj=Q;ak=Y}else{y=(d[Q]|0)+128|0;B=c[a+2292+(y<<2)>>2]|0;C=(d[T]|0)+128|0;G=(c[a+4340+(C<<2)>>2]|0)+(c[a+6388+(y<<2)>>2]|0)|0;y=c[a+8436+(C<<2)>>2]|0;C=d[V]|0;c[X>>2]=(c[G+(C<<2)>>2]|0)+(c[B+(C<<2)>>2]|0)+(c[y+(C<<2)>>2]|0)+(d[R]|0);C=d[V+1|0]|0;c[X+4>>2]=(c[G+(C<<2)>>2]|0)+(c[B+(C<<2)>>2]|0)+(c[y+(C<<2)>>2]|0)+(d[R+1|0]|0);C=d[U]|0;c[W>>2]=(c[G+(C<<2)>>2]|0)+(c[B+(C<<2)>>2]|0)+(c[y+(C<<2)>>2]|0)+(d[S]|0);C=d[U+1|0]|0;c[W+4>>2]=(c[G+(C<<2)>>2]|0)+(c[B+(C<<2)>>2]|0)+(c[y+(C<<2)>>2]|0)+(d[S+1|0]|0);C=(d[Q+1|0]|0)+128|0;y=c[a+2292+(C<<2)>>2]|0;B=(d[T+1|0]|0)+128|0;G=(c[a+4340+(B<<2)>>2]|0)+(c[a+6388+(C<<2)>>2]|0)|0;C=c[a+8436+(B<<2)>>2]|0;B=d[U+2|0]|0;c[W+8>>2]=(c[G+(B<<2)>>2]|0)+(c[y+(B<<2)>>2]|0)+(c[C+(B<<2)>>2]|0)+(d[S+2|0]|0);B=d[U+3|0]|0;c[W+12>>2]=(c[G+(B<<2)>>2]|0)+(c[y+(B<<2)>>2]|0)+(c[C+(B<<2)>>2]|0)+(d[S+3|0]|0);B=d[V+2|0]|0;c[X+8>>2]=(c[G+(B<<2)>>2]|0)+(c[y+(B<<2)>>2]|0)+(c[C+(B<<2)>>2]|0)+(d[R+2|0]|0);B=d[V+3|0]|0;c[X+12>>2]=(c[G+(B<<2)>>2]|0)+(c[y+(B<<2)>>2]|0)+(c[C+(B<<2)>>2]|0)+(d[R+3|0]|0);ab=X+16|0;ad=W+16|0;ae=V+4|0;af=U+4|0;ag=T+2|0;ah=S+4|0;ai=R+4|0;aj=Q+2|0;ak=c[n>>2]|0}if((ak&2|0)!=0){B=(d[aj]|0)+128|0;C=c[a+2292+(B<<2)>>2]|0;y=(d[ag]|0)+128|0;G=(c[a+4340+(y<<2)>>2]|0)+(c[a+6388+(B<<2)>>2]|0)|0;B=c[a+8436+(y<<2)>>2]|0;y=d[ae]|0;c[ab>>2]=(c[G+(y<<2)>>2]|0)+(c[C+(y<<2)>>2]|0)+(c[B+(y<<2)>>2]|0)+(d[ai]|0);y=d[ae+1|0]|0;c[ab+4>>2]=(c[G+(y<<2)>>2]|0)+(c[C+(y<<2)>>2]|0)+(c[B+(y<<2)>>2]|0)+(d[ai+1|0]|0);y=d[af]|0;c[ad>>2]=(c[G+(y<<2)>>2]|0)+(c[C+(y<<2)>>2]|0)+(c[B+(y<<2)>>2]|0)+(d[ah]|0);y=d[af+1|0]|0;c[ad+4>>2]=(c[G+(y<<2)>>2]|0)+(c[C+(y<<2)>>2]|0)+(c[B+(y<<2)>>2]|0)+(d[ah+1|0]|0)}y=t+2|0;if((y|0)>=(g|0)){break}q=q+2|0;r=r+2|0;s=s+2|0;t=y}return g|0}function tg(a,b,e,f,g,h,i){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0;if((g|0)<=0){return g|0}j=b+4|0;k=e+4|0;l=b+8|0;m=e+8|0;n=a+19592|0;o=b+12|0;p=e+12|0;q=f+1|0;r=f;s=1;t=0;while(1){u=c[h>>2]|0;v=t+f|0;w=c[i>>2]|0;x=u+(ac(w,v)|0)|0;y=u+(ac(w,v+1|0)|0)|0;v=c[b>>2]|0;z=c[e>>2]|0;A=ac(z,t)|0;B=v+A|0;C=v+(A+z)|0;D=c[j>>2]|0;E=t>>1;F=ac(c[k>>2]|0,E)|0;G=D+F|0;H=c[l>>2]|0;I=ac(c[m>>2]|0,E)|0;E=H+I|0;J=c[n>>2]|0;K=J>>3;L=c[o>>2]|0;M=c[p>>2]|0;N=ac(M,t)|0;O=L+N|0;P=L+(N+M)|0;if((K|0)==0){Q=E;R=O;S=P;T=G;U=C;V=B;W=y;X=x;Y=J}else{J=K<<5;Z=u+((ac(w,q)|0)+J)|0;_=u+((ac(w,r)|0)+J)|0;J=K<<3;w=(ac(z,s)|0)+J|0;z=A+J|0;A=K<<2;u=I+A|0;I=F+A|0;A=(ac(M,s)|0)+J|0;M=N+J|0;J=E;E=O;O=P;P=K;K=G;G=C;C=B;B=y;y=x;while(1){x=P-1|0;N=(d[J]|0)+128|0;F=c[a+2292+(N<<2)>>2]|0;$=(d[K]|0)+128|0;aa=(c[a+4340+($<<2)>>2]|0)+(c[a+6388+(N<<2)>>2]|0)|0;N=c[a+8436+($<<2)>>2]|0;$=d[C]|0;c[y>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+((d[E]|0)<<24);$=d[C+1|0]|0;c[y+4>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+((d[E+1|0]|0)<<24);$=d[G]|0;c[B>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+((d[O]|0)<<24);$=d[G+1|0]|0;c[B+4>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+((d[O+1|0]|0)<<24);$=(d[J+1|0]|0)+128|0;N=c[a+2292+($<<2)>>2]|0;F=(d[K+1|0]|0)+128|0;aa=(c[a+4340+(F<<2)>>2]|0)+(c[a+6388+($<<2)>>2]|0)|0;$=c[a+8436+(F<<2)>>2]|0;F=d[G+2|0]|0;c[B+8>>2]=(c[aa+(F<<2)>>2]|0)+(c[N+(F<<2)>>2]|0)+(c[$+(F<<2)>>2]|0)+((d[O+2|0]|0)<<24);F=d[G+3|0]|0;c[B+12>>2]=(c[aa+(F<<2)>>2]|0)+(c[N+(F<<2)>>2]|0)+(c[$+(F<<2)>>2]|0)+((d[O+3|0]|0)<<24);F=d[C+2|0]|0;c[y+8>>2]=(c[aa+(F<<2)>>2]|0)+(c[N+(F<<2)>>2]|0)+(c[$+(F<<2)>>2]|0)+((d[E+2|0]|0)<<24);F=d[C+3|0]|0;c[y+12>>2]=(c[aa+(F<<2)>>2]|0)+(c[N+(F<<2)>>2]|0)+(c[$+(F<<2)>>2]|0)+((d[E+3|0]|0)<<24);F=(d[J+2|0]|0)+128|0;$=c[a+2292+(F<<2)>>2]|0;N=(d[K+2|0]|0)+128|0;aa=(c[a+4340+(N<<2)>>2]|0)+(c[a+6388+(F<<2)>>2]|0)|0;F=c[a+8436+(N<<2)>>2]|0;N=d[C+4|0]|0;c[y+16>>2]=(c[aa+(N<<2)>>2]|0)+(c[$+(N<<2)>>2]|0)+(c[F+(N<<2)>>2]|0)+((d[E+4|0]|0)<<24);N=d[C+5|0]|0;c[y+20>>2]=(c[aa+(N<<2)>>2]|0)+(c[$+(N<<2)>>2]|0)+(c[F+(N<<2)>>2]|0)+((d[E+5|0]|0)<<24);N=d[G+4|0]|0;c[B+16>>2]=(c[aa+(N<<2)>>2]|0)+(c[$+(N<<2)>>2]|0)+(c[F+(N<<2)>>2]|0)+((d[O+4|0]|0)<<24);N=d[G+5|0]|0;c[B+20>>2]=(c[aa+(N<<2)>>2]|0)+(c[$+(N<<2)>>2]|0)+(c[F+(N<<2)>>2]|0)+((d[O+5|0]|0)<<24);N=(d[J+3|0]|0)+128|0;F=c[a+2292+(N<<2)>>2]|0;$=(d[K+3|0]|0)+128|0;aa=(c[a+4340+($<<2)>>2]|0)+(c[a+6388+(N<<2)>>2]|0)|0;N=c[a+8436+($<<2)>>2]|0;$=d[G+6|0]|0;c[B+24>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+((d[O+6|0]|0)<<24);$=d[G+7|0]|0;c[B+28>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+((d[O+7|0]|0)<<24);$=d[C+6|0]|0;c[y+24>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+((d[E+6|0]|0)<<24);$=d[C+7|0]|0;c[y+28>>2]=(c[aa+($<<2)>>2]|0)+(c[F+($<<2)>>2]|0)+(c[N+($<<2)>>2]|0)+((d[E+7|0]|0)<<24);if((x|0)==0){break}else{J=J+4|0;E=E+8|0;O=O+8|0;P=x;K=K+4|0;G=G+8|0;C=C+8|0;B=B+32|0;y=y+32|0}}Q=H+u|0;R=L+M|0;S=L+A|0;T=D+I|0;U=v+w|0;V=v+z|0;W=Z;X=_;Y=c[n>>2]|0}if((Y&4|0)==0){ab=X;ad=W;ae=V;af=U;ag=T;ah=S;ai=R;aj=Q;ak=Y}else{y=(d[Q]|0)+128|0;B=c[a+2292+(y<<2)>>2]|0;C=(d[T]|0)+128|0;G=(c[a+4340+(C<<2)>>2]|0)+(c[a+6388+(y<<2)>>2]|0)|0;y=c[a+8436+(C<<2)>>2]|0;C=d[V]|0;c[X>>2]=(c[G+(C<<2)>>2]|0)+(c[B+(C<<2)>>2]|0)+(c[y+(C<<2)>>2]|0)+((d[R]|0)<<24);C=d[V+1|0]|0;c[X+4>>2]=(c[G+(C<<2)>>2]|0)+(c[B+(C<<2)>>2]|0)+(c[y+(C<<2)>>2]|0)+((d[R+1|0]|0)<<24);C=d[U]|0;c[W>>2]=(c[G+(C<<2)>>2]|0)+(c[B+(C<<2)>>2]|0)+(c[y+(C<<2)>>2]|0)+((d[S]|0)<<24);C=d[U+1|0]|0;c[W+4>>2]=(c[G+(C<<2)>>2]|0)+(c[B+(C<<2)>>2]|0)+(c[y+(C<<2)>>2]|0)+((d[S+1|0]|0)<<24);C=(d[Q+1|0]|0)+128|0;y=c[a+2292+(C<<2)>>2]|0;B=(d[T+1|0]|0)+128|0;G=(c[a+4340+(B<<2)>>2]|0)+(c[a+6388+(C<<2)>>2]|0)|0;C=c[a+8436+(B<<2)>>2]|0;B=d[U+2|0]|0;c[W+8>>2]=(c[G+(B<<2)>>2]|0)+(c[y+(B<<2)>>2]|0)+(c[C+(B<<2)>>2]|0)+((d[S+2|0]|0)<<24);B=d[U+3|0]|0;c[W+12>>2]=(c[G+(B<<2)>>2]|0)+(c[y+(B<<2)>>2]|0)+(c[C+(B<<2)>>2]|0)+((d[S+3|0]|0)<<24);B=d[V+2|0]|0;c[X+8>>2]=(c[G+(B<<2)>>2]|0)+(c[y+(B<<2)>>2]|0)+(c[C+(B<<2)>>2]|0)+((d[R+2|0]|0)<<24);B=d[V+3|0]|0;c[X+12>>2]=(c[G+(B<<2)>>2]|0)+(c[y+(B<<2)>>2]|0)+(c[C+(B<<2)>>2]|0)+((d[R+3|0]|0)<<24);ab=X+16|0;ad=W+16|0;ae=V+4|0;af=U+4|0;ag=T+2|0;ah=S+4|0;ai=R+4|0;aj=Q+2|0;ak=c[n>>2]|0}if((ak&2|0)!=0){B=(d[aj]|0)+128|0;C=c[a+2292+(B<<2)>>2]|0;y=(d[ag]|0)+128|0;G=(c[a+4340+(y<<2)>>2]|0)+(c[a+6388+(B<<2)>>2]|0)|0;B=c[a+8436+(y<<2)>>2]|0;y=d[ae]|0;c[ab>>2]=(c[G+(y<<2)>>2]|0)+(c[C+(y<<2)>>2]|0)+(c[B+(y<<2)>>2]|0)+((d[ai]|0)<<24);y=d[ae+1|0]|0;c[ab+4>>2]=(c[G+(y<<2)>>2]|0)+(c[C+(y<<2)>>2]|0)+(c[B+(y<<2)>>2]|0)+((d[ai+1|0]|0)<<24);y=d[af]|0;c[ad>>2]=(c[G+(y<<2)>>2]|0)+(c[C+(y<<2)>>2]|0)+(c[B+(y<<2)>>2]|0)+((d[ah]|0)<<24);y=d[af+1|0]|0;c[ad+4>>2]=(c[G+(y<<2)>>2]|0)+(c[C+(y<<2)>>2]|0)+(c[B+(y<<2)>>2]|0)+((d[ah+1|0]|0)<<24)}y=t+2|0;if((y|0)>=(g|0)){break}q=q+2|0;r=r+2|0;s=s+2|0;t=y}return g|0}function th(a,b,e,f,g,h,i){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0;if((c[a+56>>2]|0)==4){j=e+4|0;c[j>>2]=c[j>>2]<<1;j=e+8|0;c[j>>2]=c[j>>2]<<1}if((g|0)<=0){return g|0}j=b+4|0;k=e+4|0;l=b+8|0;m=e+8|0;n=a+19592|0;o=f+1|0;p=f;q=1;r=0;while(1){s=c[h>>2]|0;t=r+f|0;u=c[i>>2]|0;v=s+(ac(u,t)|0)|0;w=s+(ac(u,t+1|0)|0)|0;t=c[b>>2]|0;x=c[e>>2]|0;y=ac(x,r)|0;z=t+y|0;A=t+(y+x)|0;B=c[j>>2]|0;C=r>>1;D=ac(c[k>>2]|0,C)|0;E=B+D|0;F=c[l>>2]|0;G=ac(c[m>>2]|0,C)|0;C=F+G|0;H=c[n>>2]|0;I=H>>3;if((I|0)==0){J=C;K=E;L=A;M=z;N=w;O=v;P=H}else{H=I<<5;Q=s+((ac(u,o)|0)+H)|0;R=s+((ac(u,p)|0)+H)|0;H=I<<3;u=(ac(x,q)|0)+H|0;x=y+H|0;H=I<<2;y=G+H|0;G=D+H|0;H=C;C=I;I=E;E=A;A=z;z=w;w=v;while(1){v=C-1|0;D=(d[H]|0)+128|0;s=c[a+2292+(D<<2)>>2]|0;S=(d[I]|0)+128|0;T=(c[a+4340+(S<<2)>>2]|0)+(c[a+6388+(D<<2)>>2]|0)|0;D=c[a+8436+(S<<2)>>2]|0;S=d[A]|0;c[w>>2]=(c[T+(S<<2)>>2]|0)+(c[s+(S<<2)>>2]|0)+(c[D+(S<<2)>>2]|0);S=d[A+1|0]|0;c[w+4>>2]=(c[T+(S<<2)>>2]|0)+(c[s+(S<<2)>>2]|0)+(c[D+(S<<2)>>2]|0);S=d[E]|0;c[z>>2]=(c[T+(S<<2)>>2]|0)+(c[s+(S<<2)>>2]|0)+(c[D+(S<<2)>>2]|0);S=d[E+1|0]|0;c[z+4>>2]=(c[T+(S<<2)>>2]|0)+(c[s+(S<<2)>>2]|0)+(c[D+(S<<2)>>2]|0);S=(d[H+1|0]|0)+128|0;D=c[a+2292+(S<<2)>>2]|0;s=(d[I+1|0]|0)+128|0;T=(c[a+4340+(s<<2)>>2]|0)+(c[a+6388+(S<<2)>>2]|0)|0;S=c[a+8436+(s<<2)>>2]|0;s=d[E+2|0]|0;c[z+8>>2]=(c[T+(s<<2)>>2]|0)+(c[D+(s<<2)>>2]|0)+(c[S+(s<<2)>>2]|0);s=d[E+3|0]|0;c[z+12>>2]=(c[T+(s<<2)>>2]|0)+(c[D+(s<<2)>>2]|0)+(c[S+(s<<2)>>2]|0);s=d[A+2|0]|0;c[w+8>>2]=(c[T+(s<<2)>>2]|0)+(c[D+(s<<2)>>2]|0)+(c[S+(s<<2)>>2]|0);s=d[A+3|0]|0;c[w+12>>2]=(c[T+(s<<2)>>2]|0)+(c[D+(s<<2)>>2]|0)+(c[S+(s<<2)>>2]|0);s=(d[H+2|0]|0)+128|0;S=c[a+2292+(s<<2)>>2]|0;D=(d[I+2|0]|0)+128|0;T=(c[a+4340+(D<<2)>>2]|0)+(c[a+6388+(s<<2)>>2]|0)|0;s=c[a+8436+(D<<2)>>2]|0;D=d[A+4|0]|0;c[w+16>>2]=(c[T+(D<<2)>>2]|0)+(c[S+(D<<2)>>2]|0)+(c[s+(D<<2)>>2]|0);D=d[A+5|0]|0;c[w+20>>2]=(c[T+(D<<2)>>2]|0)+(c[S+(D<<2)>>2]|0)+(c[s+(D<<2)>>2]|0);D=d[E+4|0]|0;c[z+16>>2]=(c[T+(D<<2)>>2]|0)+(c[S+(D<<2)>>2]|0)+(c[s+(D<<2)>>2]|0);D=d[E+5|0]|0;c[z+20>>2]=(c[T+(D<<2)>>2]|0)+(c[S+(D<<2)>>2]|0)+(c[s+(D<<2)>>2]|0);D=(d[H+3|0]|0)+128|0;s=c[a+2292+(D<<2)>>2]|0;S=(d[I+3|0]|0)+128|0;T=(c[a+4340+(S<<2)>>2]|0)+(c[a+6388+(D<<2)>>2]|0)|0;D=c[a+8436+(S<<2)>>2]|0;S=d[E+6|0]|0;c[z+24>>2]=(c[T+(S<<2)>>2]|0)+(c[s+(S<<2)>>2]|0)+(c[D+(S<<2)>>2]|0);S=d[E+7|0]|0;c[z+28>>2]=(c[T+(S<<2)>>2]|0)+(c[s+(S<<2)>>2]|0)+(c[D+(S<<2)>>2]|0);S=d[A+6|0]|0;c[w+24>>2]=(c[T+(S<<2)>>2]|0)+(c[s+(S<<2)>>2]|0)+(c[D+(S<<2)>>2]|0);S=d[A+7|0]|0;c[w+28>>2]=(c[T+(S<<2)>>2]|0)+(c[s+(S<<2)>>2]|0)+(c[D+(S<<2)>>2]|0);if((v|0)==0){break}else{H=H+4|0;C=v;I=I+4|0;E=E+8|0;A=A+8|0;z=z+32|0;w=w+32|0}}J=F+y|0;K=B+G|0;L=t+u|0;M=t+x|0;N=Q;O=R;P=c[n>>2]|0}if((P&4|0)==0){U=O;V=N;W=M;X=L;Y=K;Z=J;_=P}else{w=(d[J]|0)+128|0;z=c[a+2292+(w<<2)>>2]|0;A=(d[K]|0)+128|0;E=(c[a+4340+(A<<2)>>2]|0)+(c[a+6388+(w<<2)>>2]|0)|0;w=c[a+8436+(A<<2)>>2]|0;A=d[M]|0;c[O>>2]=(c[E+(A<<2)>>2]|0)+(c[z+(A<<2)>>2]|0)+(c[w+(A<<2)>>2]|0);A=d[M+1|0]|0;c[O+4>>2]=(c[E+(A<<2)>>2]|0)+(c[z+(A<<2)>>2]|0)+(c[w+(A<<2)>>2]|0);A=d[L]|0;c[N>>2]=(c[E+(A<<2)>>2]|0)+(c[z+(A<<2)>>2]|0)+(c[w+(A<<2)>>2]|0);A=d[L+1|0]|0;c[N+4>>2]=(c[E+(A<<2)>>2]|0)+(c[z+(A<<2)>>2]|0)+(c[w+(A<<2)>>2]|0);A=(d[J+1|0]|0)+128|0;w=c[a+2292+(A<<2)>>2]|0;z=(d[K+1|0]|0)+128|0;E=(c[a+4340+(z<<2)>>2]|0)+(c[a+6388+(A<<2)>>2]|0)|0;A=c[a+8436+(z<<2)>>2]|0;z=d[L+2|0]|0;c[N+8>>2]=(c[E+(z<<2)>>2]|0)+(c[w+(z<<2)>>2]|0)+(c[A+(z<<2)>>2]|0);z=d[L+3|0]|0;c[N+12>>2]=(c[E+(z<<2)>>2]|0)+(c[w+(z<<2)>>2]|0)+(c[A+(z<<2)>>2]|0);z=d[M+2|0]|0;c[O+8>>2]=(c[E+(z<<2)>>2]|0)+(c[w+(z<<2)>>2]|0)+(c[A+(z<<2)>>2]|0);z=d[M+3|0]|0;c[O+12>>2]=(c[E+(z<<2)>>2]|0)+(c[w+(z<<2)>>2]|0)+(c[A+(z<<2)>>2]|0);U=O+16|0;V=N+16|0;W=M+4|0;X=L+4|0;Y=K+2|0;Z=J+2|0;_=c[n>>2]|0}if((_&2|0)!=0){z=(d[Z]|0)+128|0;A=c[a+2292+(z<<2)>>2]|0;w=(d[Y]|0)+128|0;E=(c[a+4340+(w<<2)>>2]|0)+(c[a+6388+(z<<2)>>2]|0)|0;z=c[a+8436+(w<<2)>>2]|0;w=d[W]|0;c[U>>2]=(c[E+(w<<2)>>2]|0)+(c[A+(w<<2)>>2]|0)+(c[z+(w<<2)>>2]|0);w=d[W+1|0]|0;c[U+4>>2]=(c[E+(w<<2)>>2]|0)+(c[A+(w<<2)>>2]|0)+(c[z+(w<<2)>>2]|0);w=d[X]|0;c[V>>2]=(c[E+(w<<2)>>2]|0)+(c[A+(w<<2)>>2]|0)+(c[z+(w<<2)>>2]|0);w=d[X+1|0]|0;c[V+4>>2]=(c[E+(w<<2)>>2]|0)+(c[A+(w<<2)>>2]|0)+(c[z+(w<<2)>>2]|0)}w=r+2|0;if((w|0)>=(g|0)){break}o=o+2|0;p=p+2|0;q=q+2|0;r=w}return g|0}function ti(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0;if((c[b+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=e+4|0;l=f+4|0;m=e+8|0;n=f+8|0;o=b+19592|0;p=g+1|0;q=g;r=1;s=0;while(1){t=c[i>>2]|0;u=s+g|0;v=c[j>>2]|0;w=t+(ac(v,u)|0)|0;x=t+(ac(v,u+1|0)|0)|0;u=c[e>>2]|0;y=c[f>>2]|0;z=ac(y,s)|0;A=u+z|0;B=u+(z+y)|0;C=c[k>>2]|0;D=s>>1;E=ac(c[l>>2]|0,D)|0;F=C+E|0;G=c[m>>2]|0;H=ac(c[n>>2]|0,D)|0;D=G+H|0;I=c[o>>2]|0;J=I>>3;if((J|0)==0){K=D;L=F;M=B;N=A;O=x;P=w;Q=I}else{I=J*24|0;R=(ac(v,p)|0)+I|0;S=(ac(v,q)|0)+I|0;I=J<<3;v=(ac(y,r)|0)+I|0;y=z+I|0;I=J<<2;z=H+I|0;H=E+I|0;I=D;D=J;J=F;F=B;B=A;A=x;x=w;while(1){w=D-1|0;E=(d[I]|0)+128|0;T=c[b+2292+(E<<2)>>2]|0;U=(d[J]|0)+128|0;V=c[b+4340+(U<<2)>>2]|0;W=c[b+6388+(E<<2)>>2]|0;E=c[b+8436+(U<<2)>>2]|0;U=d[B]|0;a[x]=a[T+U|0]|0;a[x+1|0]=a[V+(U+W)|0]|0;a[x+2|0]=a[E+U|0]|0;U=d[B+1|0]|0;a[x+3|0]=a[T+U|0]|0;a[x+4|0]=a[V+(U+W)|0]|0;a[x+5|0]=a[E+U|0]|0;U=d[F]|0;a[A]=a[T+U|0]|0;a[A+1|0]=a[V+(U+W)|0]|0;a[A+2|0]=a[E+U|0]|0;U=d[F+1|0]|0;a[A+3|0]=a[T+U|0]|0;a[A+4|0]=a[V+(U+W)|0]|0;a[A+5|0]=a[E+U|0]|0;U=(d[I+1|0]|0)+128|0;E=c[b+2292+(U<<2)>>2]|0;W=(d[J+1|0]|0)+128|0;V=c[b+4340+(W<<2)>>2]|0;T=c[b+6388+(U<<2)>>2]|0;U=c[b+8436+(W<<2)>>2]|0;W=d[F+2|0]|0;a[A+6|0]=a[E+W|0]|0;a[A+7|0]=a[V+(W+T)|0]|0;a[A+8|0]=a[U+W|0]|0;W=d[F+3|0]|0;a[A+9|0]=a[E+W|0]|0;a[A+10|0]=a[V+(W+T)|0]|0;a[A+11|0]=a[U+W|0]|0;W=d[B+2|0]|0;a[x+6|0]=a[E+W|0]|0;a[x+7|0]=a[V+(W+T)|0]|0;a[x+8|0]=a[U+W|0]|0;W=d[B+3|0]|0;a[x+9|0]=a[E+W|0]|0;a[x+10|0]=a[V+(W+T)|0]|0;a[x+11|0]=a[U+W|0]|0;W=(d[I+2|0]|0)+128|0;U=c[b+2292+(W<<2)>>2]|0;T=(d[J+2|0]|0)+128|0;V=c[b+4340+(T<<2)>>2]|0;E=c[b+6388+(W<<2)>>2]|0;W=c[b+8436+(T<<2)>>2]|0;T=d[B+4|0]|0;a[x+12|0]=a[U+T|0]|0;a[x+13|0]=a[V+(T+E)|0]|0;a[x+14|0]=a[W+T|0]|0;T=d[B+5|0]|0;a[x+15|0]=a[U+T|0]|0;a[x+16|0]=a[V+(T+E)|0]|0;a[x+17|0]=a[W+T|0]|0;T=d[F+4|0]|0;a[A+12|0]=a[U+T|0]|0;a[A+13|0]=a[V+(T+E)|0]|0;a[A+14|0]=a[W+T|0]|0;T=d[F+5|0]|0;a[A+15|0]=a[U+T|0]|0;a[A+16|0]=a[V+(T+E)|0]|0;a[A+17|0]=a[W+T|0]|0;T=(d[I+3|0]|0)+128|0;W=c[b+2292+(T<<2)>>2]|0;E=(d[J+3|0]|0)+128|0;V=c[b+4340+(E<<2)>>2]|0;U=c[b+6388+(T<<2)>>2]|0;T=c[b+8436+(E<<2)>>2]|0;E=d[F+6|0]|0;a[A+18|0]=a[W+E|0]|0;a[A+19|0]=a[V+(E+U)|0]|0;a[A+20|0]=a[T+E|0]|0;E=d[F+7|0]|0;a[A+21|0]=a[W+E|0]|0;a[A+22|0]=a[V+(E+U)|0]|0;a[A+23|0]=a[T+E|0]|0;E=d[B+6|0]|0;a[x+18|0]=a[W+E|0]|0;a[x+19|0]=a[V+(E+U)|0]|0;a[x+20|0]=a[T+E|0]|0;E=d[B+7|0]|0;a[x+21|0]=a[W+E|0]|0;a[x+22|0]=a[V+(E+U)|0]|0;a[x+23|0]=a[T+E|0]|0;if((w|0)==0){break}else{I=I+4|0;D=w;J=J+4|0;F=F+8|0;B=B+8|0;A=A+24|0;x=x+24|0}}K=G+z|0;L=C+H|0;M=u+v|0;N=u+y|0;O=t+R|0;P=t+S|0;Q=c[o>>2]|0}if((Q&4|0)==0){X=P;Y=O;Z=N;_=M;$=L;aa=K;ab=Q}else{x=(d[K]|0)+128|0;A=c[b+2292+(x<<2)>>2]|0;B=(d[L]|0)+128|0;F=c[b+4340+(B<<2)>>2]|0;J=c[b+6388+(x<<2)>>2]|0;x=c[b+8436+(B<<2)>>2]|0;B=d[N]|0;a[P]=a[A+B|0]|0;a[P+1|0]=a[F+(B+J)|0]|0;a[P+2|0]=a[x+B|0]|0;B=d[N+1|0]|0;a[P+3|0]=a[A+B|0]|0;a[P+4|0]=a[F+(B+J)|0]|0;a[P+5|0]=a[x+B|0]|0;B=d[M]|0;a[O]=a[A+B|0]|0;a[O+1|0]=a[F+(B+J)|0]|0;a[O+2|0]=a[x+B|0]|0;B=d[M+1|0]|0;a[O+3|0]=a[A+B|0]|0;a[O+4|0]=a[F+(B+J)|0]|0;a[O+5|0]=a[x+B|0]|0;B=(d[K+1|0]|0)+128|0;x=c[b+2292+(B<<2)>>2]|0;J=(d[L+1|0]|0)+128|0;F=c[b+4340+(J<<2)>>2]|0;A=c[b+6388+(B<<2)>>2]|0;B=c[b+8436+(J<<2)>>2]|0;J=d[M+2|0]|0;a[O+6|0]=a[x+J|0]|0;a[O+7|0]=a[F+(J+A)|0]|0;a[O+8|0]=a[B+J|0]|0;J=d[M+3|0]|0;a[O+9|0]=a[x+J|0]|0;a[O+10|0]=a[F+(J+A)|0]|0;a[O+11|0]=a[B+J|0]|0;J=d[N+2|0]|0;a[P+6|0]=a[x+J|0]|0;a[P+7|0]=a[F+(J+A)|0]|0;a[P+8|0]=a[B+J|0]|0;J=d[N+3|0]|0;a[P+9|0]=a[x+J|0]|0;a[P+10|0]=a[F+(J+A)|0]|0;a[P+11|0]=a[B+J|0]|0;X=P+12|0;Y=O+12|0;Z=N+4|0;_=M+4|0;$=L+2|0;aa=K+2|0;ab=c[o>>2]|0}if((ab&2|0)!=0){J=(d[aa]|0)+128|0;B=c[b+2292+(J<<2)>>2]|0;A=(d[$]|0)+128|0;F=c[b+4340+(A<<2)>>2]|0;x=c[b+6388+(J<<2)>>2]|0;J=c[b+8436+(A<<2)>>2]|0;A=d[Z]|0;a[X]=a[B+A|0]|0;a[X+1|0]=a[F+(A+x)|0]|0;a[X+2|0]=a[J+A|0]|0;A=d[Z+1|0]|0;a[X+3|0]=a[B+A|0]|0;a[X+4|0]=a[F+(A+x)|0]|0;a[X+5|0]=a[J+A|0]|0;A=d[_]|0;a[Y]=a[B+A|0]|0;a[Y+1|0]=a[F+(A+x)|0]|0;a[Y+2|0]=a[J+A|0]|0;A=d[_+1|0]|0;a[Y+3|0]=a[B+A|0]|0;a[Y+4|0]=a[F+(A+x)|0]|0;a[Y+5|0]=a[J+A|0]|0}A=s+2|0;if((A|0)>=(h|0)){break}p=p+2|0;q=q+2|0;r=r+2|0;s=A}return h|0}function tj(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0;if((c[b+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=e+4|0;l=f+4|0;m=e+8|0;n=f+8|0;o=b+19592|0;p=g+1|0;q=g;r=1;s=0;while(1){t=c[i>>2]|0;u=s+g|0;v=c[j>>2]|0;w=t+(ac(v,u)|0)|0;x=t+(ac(v,u+1|0)|0)|0;u=c[e>>2]|0;y=c[f>>2]|0;z=ac(y,s)|0;A=u+z|0;B=u+(z+y)|0;C=c[k>>2]|0;D=s>>1;E=ac(c[l>>2]|0,D)|0;F=C+E|0;G=c[m>>2]|0;H=ac(c[n>>2]|0,D)|0;D=G+H|0;I=c[o>>2]|0;J=I>>3;if((J|0)==0){K=D;L=F;M=B;N=A;O=x;P=w;Q=I}else{I=J*24|0;R=(ac(v,p)|0)+I|0;S=(ac(v,q)|0)+I|0;I=J<<3;v=(ac(y,r)|0)+I|0;y=z+I|0;I=J<<2;z=H+I|0;H=E+I|0;I=D;D=J;J=F;F=B;B=A;A=x;x=w;while(1){w=D-1|0;E=(d[I]|0)+128|0;T=c[b+2292+(E<<2)>>2]|0;U=(d[J]|0)+128|0;V=c[b+4340+(U<<2)>>2]|0;W=c[b+6388+(E<<2)>>2]|0;E=c[b+8436+(U<<2)>>2]|0;U=d[B]|0;a[x]=a[E+U|0]|0;a[x+1|0]=a[V+(U+W)|0]|0;a[x+2|0]=a[T+U|0]|0;U=d[B+1|0]|0;a[x+3|0]=a[E+U|0]|0;a[x+4|0]=a[V+(U+W)|0]|0;a[x+5|0]=a[T+U|0]|0;U=d[F]|0;a[A]=a[E+U|0]|0;a[A+1|0]=a[V+(U+W)|0]|0;a[A+2|0]=a[T+U|0]|0;U=d[F+1|0]|0;a[A+3|0]=a[E+U|0]|0;a[A+4|0]=a[V+(U+W)|0]|0;a[A+5|0]=a[T+U|0]|0;U=(d[I+1|0]|0)+128|0;T=c[b+2292+(U<<2)>>2]|0;W=(d[J+1|0]|0)+128|0;V=c[b+4340+(W<<2)>>2]|0;E=c[b+6388+(U<<2)>>2]|0;U=c[b+8436+(W<<2)>>2]|0;W=d[F+2|0]|0;a[A+6|0]=a[U+W|0]|0;a[A+7|0]=a[V+(W+E)|0]|0;a[A+8|0]=a[T+W|0]|0;W=d[F+3|0]|0;a[A+9|0]=a[U+W|0]|0;a[A+10|0]=a[V+(W+E)|0]|0;a[A+11|0]=a[T+W|0]|0;W=d[B+2|0]|0;a[x+6|0]=a[U+W|0]|0;a[x+7|0]=a[V+(W+E)|0]|0;a[x+8|0]=a[T+W|0]|0;W=d[B+3|0]|0;a[x+9|0]=a[U+W|0]|0;a[x+10|0]=a[V+(W+E)|0]|0;a[x+11|0]=a[T+W|0]|0;W=(d[I+2|0]|0)+128|0;T=c[b+2292+(W<<2)>>2]|0;E=(d[J+2|0]|0)+128|0;V=c[b+4340+(E<<2)>>2]|0;U=c[b+6388+(W<<2)>>2]|0;W=c[b+8436+(E<<2)>>2]|0;E=d[B+4|0]|0;a[x+12|0]=a[W+E|0]|0;a[x+13|0]=a[V+(E+U)|0]|0;a[x+14|0]=a[T+E|0]|0;E=d[B+5|0]|0;a[x+15|0]=a[W+E|0]|0;a[x+16|0]=a[V+(E+U)|0]|0;a[x+17|0]=a[T+E|0]|0;E=d[F+4|0]|0;a[A+12|0]=a[W+E|0]|0;a[A+13|0]=a[V+(E+U)|0]|0;a[A+14|0]=a[T+E|0]|0;E=d[F+5|0]|0;a[A+15|0]=a[W+E|0]|0;a[A+16|0]=a[V+(E+U)|0]|0;a[A+17|0]=a[T+E|0]|0;E=(d[I+3|0]|0)+128|0;T=c[b+2292+(E<<2)>>2]|0;U=(d[J+3|0]|0)+128|0;V=c[b+4340+(U<<2)>>2]|0;W=c[b+6388+(E<<2)>>2]|0;E=c[b+8436+(U<<2)>>2]|0;U=d[F+6|0]|0;a[A+18|0]=a[E+U|0]|0;a[A+19|0]=a[V+(U+W)|0]|0;a[A+20|0]=a[T+U|0]|0;U=d[F+7|0]|0;a[A+21|0]=a[E+U|0]|0;a[A+22|0]=a[V+(U+W)|0]|0;a[A+23|0]=a[T+U|0]|0;U=d[B+6|0]|0;a[x+18|0]=a[E+U|0]|0;a[x+19|0]=a[V+(U+W)|0]|0;a[x+20|0]=a[T+U|0]|0;U=d[B+7|0]|0;a[x+21|0]=a[E+U|0]|0;a[x+22|0]=a[V+(U+W)|0]|0;a[x+23|0]=a[T+U|0]|0;if((w|0)==0){break}else{I=I+4|0;D=w;J=J+4|0;F=F+8|0;B=B+8|0;A=A+24|0;x=x+24|0}}K=G+z|0;L=C+H|0;M=u+v|0;N=u+y|0;O=t+R|0;P=t+S|0;Q=c[o>>2]|0}if((Q&4|0)==0){X=P;Y=O;Z=N;_=M;$=L;aa=K;ab=Q}else{x=(d[K]|0)+128|0;A=c[b+2292+(x<<2)>>2]|0;B=(d[L]|0)+128|0;F=c[b+4340+(B<<2)>>2]|0;J=c[b+6388+(x<<2)>>2]|0;x=c[b+8436+(B<<2)>>2]|0;B=d[N]|0;a[P]=a[x+B|0]|0;a[P+1|0]=a[F+(B+J)|0]|0;a[P+2|0]=a[A+B|0]|0;B=d[N+1|0]|0;a[P+3|0]=a[x+B|0]|0;a[P+4|0]=a[F+(B+J)|0]|0;a[P+5|0]=a[A+B|0]|0;B=d[M]|0;a[O]=a[x+B|0]|0;a[O+1|0]=a[F+(B+J)|0]|0;a[O+2|0]=a[A+B|0]|0;B=d[M+1|0]|0;a[O+3|0]=a[x+B|0]|0;a[O+4|0]=a[F+(B+J)|0]|0;a[O+5|0]=a[A+B|0]|0;B=(d[K+1|0]|0)+128|0;A=c[b+2292+(B<<2)>>2]|0;J=(d[L+1|0]|0)+128|0;F=c[b+4340+(J<<2)>>2]|0;x=c[b+6388+(B<<2)>>2]|0;B=c[b+8436+(J<<2)>>2]|0;J=d[M+2|0]|0;a[O+6|0]=a[B+J|0]|0;a[O+7|0]=a[F+(J+x)|0]|0;a[O+8|0]=a[A+J|0]|0;J=d[M+3|0]|0;a[O+9|0]=a[B+J|0]|0;a[O+10|0]=a[F+(J+x)|0]|0;a[O+11|0]=a[A+J|0]|0;J=d[N+2|0]|0;a[P+6|0]=a[B+J|0]|0;a[P+7|0]=a[F+(J+x)|0]|0;a[P+8|0]=a[A+J|0]|0;J=d[N+3|0]|0;a[P+9|0]=a[B+J|0]|0;a[P+10|0]=a[F+(J+x)|0]|0;a[P+11|0]=a[A+J|0]|0;X=P+12|0;Y=O+12|0;Z=N+4|0;_=M+4|0;$=L+2|0;aa=K+2|0;ab=c[o>>2]|0}if((ab&2|0)!=0){J=(d[aa]|0)+128|0;A=c[b+2292+(J<<2)>>2]|0;x=(d[$]|0)+128|0;F=c[b+4340+(x<<2)>>2]|0;B=c[b+6388+(J<<2)>>2]|0;J=c[b+8436+(x<<2)>>2]|0;x=d[Z]|0;a[X]=a[J+x|0]|0;a[X+1|0]=a[F+(x+B)|0]|0;a[X+2|0]=a[A+x|0]|0;x=d[Z+1|0]|0;a[X+3|0]=a[J+x|0]|0;a[X+4|0]=a[F+(x+B)|0]|0;a[X+5|0]=a[A+x|0]|0;x=d[_]|0;a[Y]=a[J+x|0]|0;a[Y+1|0]=a[F+(x+B)|0]|0;a[Y+2|0]=a[A+x|0]|0;x=d[_+1|0]|0;a[Y+3|0]=a[J+x|0]|0;a[Y+4|0]=a[F+(x+B)|0]|0;a[Y+5|0]=a[A+x|0]|0}x=s+2|0;if((x|0)>=(h|0)){break}p=p+2|0;q=q+2|0;r=r+2|0;s=x}return h|0}function tk(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0;if((c[a+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=e+4|0;l=f+4|0;m=e+8|0;n=f+8|0;o=a+19592|0;p=d[262016]|0;q=d[262040]|0;r=d[262024]|0;s=d[262017]|0;t=d[262041]|0;u=d[262025]|0;v=d[262048]|0;w=d[262032]|0;x=d[262049]|0;y=d[262033]|0;z=d[262026]|0;A=d[262050]|0;B=d[262034]|0;C=d[262027]|0;D=d[262051]|0;E=d[262035]|0;F=d[262018]|0;G=d[262042]|0;H=d[262019]|0;I=d[262043]|0;J=d[262020]|0;K=d[262044]|0;L=d[262028]|0;M=d[262021]|0;N=d[262045]|0;O=d[262029]|0;P=d[262052]|0;Q=d[262036]|0;R=d[262053]|0;S=d[262037]|0;T=d[262030]|0;U=d[262054]|0;V=d[262038]|0;W=d[262031]|0;X=d[262055]|0;Y=d[262039]|0;Z=d[262022]|0;_=d[262046]|0;$=d[262023]|0;aa=d[262047]|0;ab=0;do{ad=c[i>>2]|0;ae=ab+g|0;af=c[j>>2]|0;ag=c[e>>2]|0;ah=c[f>>2]|0;ai=ac(ah,ab)|0;aj=ab>>1;ak=c[o>>2]>>3;if((ak|0)!=0){al=c[n>>2]|0;am=c[l>>2]|0;an=ac(af,ae+1|0)|0;ao=ac(af,ae)|0;ae=ac(al,aj)|0;al=c[m>>2]|0;af=ac(am,aj)|0;aj=al+ae|0;ae=ak;ak=(c[k>>2]|0)+af|0;af=ag+(ai+ah)|0;ah=ag+ai|0;ai=ad+an|0;an=ad+ao|0;while(1){ao=ae-1|0;ad=(d[aj]|0)+128|0;ag=c[a+2292+(ad<<2)>>2]|0;al=(d[ak]|0)+128|0;am=(c[a+4340+(al<<2)>>2]|0)+(c[a+6388+(ad<<2)>>2]|0)|0;ad=c[a+8436+(al<<2)>>2]|0;al=d[ah]|0;b[an>>1]=((b[am+(q+al<<1)>>1]|0)+(b[ag+(p+al<<1)>>1]|0)&65535)+(b[ad+(r+al<<1)>>1]|0)&65535;al=d[ah+1|0]|0;b[an+2>>1]=((b[am+(t+al<<1)>>1]|0)+(b[ag+(s+al<<1)>>1]|0)&65535)+(b[ad+(u+al<<1)>>1]|0)&65535;al=d[af]|0;b[ai>>1]=((b[am+(v+al<<1)>>1]|0)+(b[ag+(r+al<<1)>>1]|0)&65535)+(b[ad+(w+al<<1)>>1]|0)&65535;al=d[af+1|0]|0;b[ai+2>>1]=((b[am+(x+al<<1)>>1]|0)+(b[ag+(u+al<<1)>>1]|0)&65535)+(b[ad+(y+al<<1)>>1]|0)&65535;al=(d[aj+1|0]|0)+128|0;ad=c[a+2292+(al<<2)>>2]|0;ag=(d[ak+1|0]|0)+128|0;am=(c[a+4340+(ag<<2)>>2]|0)+(c[a+6388+(al<<2)>>2]|0)|0;al=c[a+8436+(ag<<2)>>2]|0;ag=d[af+2|0]|0;b[ai+4>>1]=((b[am+(A+ag<<1)>>1]|0)+(b[ad+(z+ag<<1)>>1]|0)&65535)+(b[al+(B+ag<<1)>>1]|0)&65535;ag=d[af+3|0]|0;b[ai+6>>1]=((b[am+(D+ag<<1)>>1]|0)+(b[ad+(C+ag<<1)>>1]|0)&65535)+(b[al+(E+ag<<1)>>1]|0)&65535;ag=d[ah+2|0]|0;b[an+4>>1]=((b[am+(G+ag<<1)>>1]|0)+(b[ad+(F+ag<<1)>>1]|0)&65535)+(b[al+(z+ag<<1)>>1]|0)&65535;ag=d[ah+3|0]|0;b[an+6>>1]=((b[am+(I+ag<<1)>>1]|0)+(b[ad+(H+ag<<1)>>1]|0)&65535)+(b[al+(C+ag<<1)>>1]|0)&65535;ag=(d[aj+2|0]|0)+128|0;al=c[a+2292+(ag<<2)>>2]|0;ad=(d[ak+2|0]|0)+128|0;am=(c[a+4340+(ad<<2)>>2]|0)+(c[a+6388+(ag<<2)>>2]|0)|0;ag=c[a+8436+(ad<<2)>>2]|0;ad=d[ah+4|0]|0;b[an+8>>1]=((b[am+(K+ad<<1)>>1]|0)+(b[al+(J+ad<<1)>>1]|0)&65535)+(b[ag+(L+ad<<1)>>1]|0)&65535;ad=d[ah+5|0]|0;b[an+10>>1]=((b[am+(N+ad<<1)>>1]|0)+(b[al+(M+ad<<1)>>1]|0)&65535)+(b[ag+(O+ad<<1)>>1]|0)&65535;ad=d[af+4|0]|0;b[ai+8>>1]=((b[am+(P+ad<<1)>>1]|0)+(b[al+(L+ad<<1)>>1]|0)&65535)+(b[ag+(Q+ad<<1)>>1]|0)&65535;ad=d[af+5|0]|0;b[ai+10>>1]=((b[am+(R+ad<<1)>>1]|0)+(b[al+(O+ad<<1)>>1]|0)&65535)+(b[ag+(S+ad<<1)>>1]|0)&65535;ad=(d[aj+3|0]|0)+128|0;ag=c[a+2292+(ad<<2)>>2]|0;al=(d[ak+3|0]|0)+128|0;am=(c[a+4340+(al<<2)>>2]|0)+(c[a+6388+(ad<<2)>>2]|0)|0;ad=c[a+8436+(al<<2)>>2]|0;al=d[af+6|0]|0;b[ai+12>>1]=((b[am+(U+al<<1)>>1]|0)+(b[ag+(T+al<<1)>>1]|0)&65535)+(b[ad+(V+al<<1)>>1]|0)&65535;al=d[af+7|0]|0;b[ai+14>>1]=((b[am+(X+al<<1)>>1]|0)+(b[ag+(W+al<<1)>>1]|0)&65535)+(b[ad+(Y+al<<1)>>1]|0)&65535;al=d[ah+6|0]|0;b[an+12>>1]=((b[am+(_+al<<1)>>1]|0)+(b[ag+(Z+al<<1)>>1]|0)&65535)+(b[ad+(T+al<<1)>>1]|0)&65535;al=d[ah+7|0]|0;b[an+14>>1]=((b[am+(aa+al<<1)>>1]|0)+(b[ag+($+al<<1)>>1]|0)&65535)+(b[ad+(W+al<<1)>>1]|0)&65535;if((ao|0)==0){break}else{aj=aj+4|0;ae=ao;ak=ak+4|0;af=af+8|0;ah=ah+8|0;ai=ai+16|0;an=an+16|0}}}ab=ab+2|0;}while((ab|0)<(h|0));return h|0}function tl(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0;if((c[a+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=e+4|0;l=f+4|0;m=e+8|0;n=f+8|0;o=a+19592|0;p=d[262016]|0;q=d[262017]|0;r=d[262024]|0;s=d[262025]|0;t=d[262032]|0;u=d[262033]|0;v=d[262026]|0;w=d[262027]|0;x=d[262034]|0;y=d[262035]|0;z=d[262018]|0;A=d[262019]|0;B=d[262020]|0;C=d[262021]|0;D=d[262028]|0;E=d[262029]|0;F=d[262036]|0;G=d[262037]|0;H=d[262030]|0;I=d[262031]|0;J=d[262038]|0;K=d[262039]|0;L=d[262022]|0;M=d[262023]|0;N=0;do{O=c[i>>2]|0;P=N+g|0;Q=c[j>>2]|0;R=c[e>>2]|0;S=c[f>>2]|0;T=ac(S,N)|0;U=N>>1;V=c[o>>2]>>3;if((V|0)!=0){W=c[n>>2]|0;X=c[l>>2]|0;Y=ac(Q,P+1|0)|0;Z=ac(Q,P)|0;P=ac(W,U)|0;W=c[m>>2]|0;Q=ac(X,U)|0;U=(c[k>>2]|0)+Q|0;Q=W+P|0;P=V;V=R+(T+S)|0;S=R+T|0;T=O+Y|0;Y=O+Z|0;while(1){Z=P-1|0;O=(d[Q]|0)+128|0;R=c[a+2292+(O<<2)>>2]|0;W=(d[U]|0)+128|0;X=(c[a+4340+(W<<2)>>2]|0)+(c[a+6388+(O<<2)>>2]|0)|0;O=c[a+8436+(W<<2)>>2]|0;W=d[S]|0;b[Y>>1]=((b[X+(q+W<<1)>>1]|0)+(b[R+(p+W<<1)>>1]|0)&65535)+(b[O+(r+W<<1)>>1]|0)&65535;W=d[S+1|0]|0;b[Y+2>>1]=((b[X+(p+W<<1)>>1]|0)+(b[R+(q+W<<1)>>1]|0)&65535)+(b[O+(s+W<<1)>>1]|0)&65535;W=d[V]|0;b[T>>1]=((b[X+(s+W<<1)>>1]|0)+(b[R+(r+W<<1)>>1]|0)&65535)+(b[O+(t+W<<1)>>1]|0)&65535;W=d[V+1|0]|0;b[T+2>>1]=((b[X+(r+W<<1)>>1]|0)+(b[R+(s+W<<1)>>1]|0)&65535)+(b[O+(u+W<<1)>>1]|0)&65535;W=(d[Q+1|0]|0)+128|0;O=c[a+2292+(W<<2)>>2]|0;R=(d[U+1|0]|0)+128|0;X=(c[a+4340+(R<<2)>>2]|0)+(c[a+6388+(W<<2)>>2]|0)|0;W=c[a+8436+(R<<2)>>2]|0;R=d[V+2|0]|0;b[T+4>>1]=((b[X+(w+R<<1)>>1]|0)+(b[O+(v+R<<1)>>1]|0)&65535)+(b[W+(x+R<<1)>>1]|0)&65535;R=d[V+3|0]|0;b[T+6>>1]=((b[X+(v+R<<1)>>1]|0)+(b[O+(w+R<<1)>>1]|0)&65535)+(b[W+(y+R<<1)>>1]|0)&65535;R=d[S+2|0]|0;b[Y+4>>1]=((b[X+(A+R<<1)>>1]|0)+(b[O+(z+R<<1)>>1]|0)&65535)+(b[W+(v+R<<1)>>1]|0)&65535;R=d[S+3|0]|0;b[Y+6>>1]=((b[X+(z+R<<1)>>1]|0)+(b[O+(A+R<<1)>>1]|0)&65535)+(b[W+(w+R<<1)>>1]|0)&65535;R=(d[Q+2|0]|0)+128|0;W=c[a+2292+(R<<2)>>2]|0;O=(d[U+2|0]|0)+128|0;X=(c[a+4340+(O<<2)>>2]|0)+(c[a+6388+(R<<2)>>2]|0)|0;R=c[a+8436+(O<<2)>>2]|0;O=d[S+4|0]|0;b[Y+8>>1]=((b[X+(C+O<<1)>>1]|0)+(b[W+(B+O<<1)>>1]|0)&65535)+(b[R+(D+O<<1)>>1]|0)&65535;O=d[S+5|0]|0;b[Y+10>>1]=((b[X+(B+O<<1)>>1]|0)+(b[W+(C+O<<1)>>1]|0)&65535)+(b[R+(E+O<<1)>>1]|0)&65535;O=d[V+4|0]|0;b[T+8>>1]=((b[X+(E+O<<1)>>1]|0)+(b[W+(D+O<<1)>>1]|0)&65535)+(b[R+(F+O<<1)>>1]|0)&65535;O=d[V+5|0]|0;b[T+10>>1]=((b[X+(D+O<<1)>>1]|0)+(b[W+(E+O<<1)>>1]|0)&65535)+(b[R+(G+O<<1)>>1]|0)&65535;O=(d[Q+3|0]|0)+128|0;R=c[a+2292+(O<<2)>>2]|0;W=(d[U+3|0]|0)+128|0;X=(c[a+4340+(W<<2)>>2]|0)+(c[a+6388+(O<<2)>>2]|0)|0;O=c[a+8436+(W<<2)>>2]|0;W=d[V+6|0]|0;b[T+12>>1]=((b[X+(I+W<<1)>>1]|0)+(b[R+(H+W<<1)>>1]|0)&65535)+(b[O+(J+W<<1)>>1]|0)&65535;W=d[V+7|0]|0;b[T+14>>1]=((b[X+(H+W<<1)>>1]|0)+(b[R+(I+W<<1)>>1]|0)&65535)+(b[O+(K+W<<1)>>1]|0)&65535;W=d[S+6|0]|0;b[Y+12>>1]=((b[X+(M+W<<1)>>1]|0)+(b[R+(L+W<<1)>>1]|0)&65535)+(b[O+(H+W<<1)>>1]|0)&65535;W=d[S+7|0]|0;b[Y+14>>1]=((b[X+(L+W<<1)>>1]|0)+(b[R+(M+W<<1)>>1]|0)&65535)+(b[O+(I+W<<1)>>1]|0)&65535;if((Z|0)==0){break}else{U=U+4|0;Q=Q+4|0;P=Z;V=V+8|0;S=S+8|0;T=T+16|0;Y=Y+16|0}}}N=N+2|0;}while((N|0)<(h|0));return h|0}function tm(a,e,f,g,h,i,j){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;if((c[a+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=e+4|0;l=f+4|0;m=e+8|0;n=f+8|0;o=a+19592|0;p=0;do{q=c[i>>2]|0;r=p+g|0;s=c[j>>2]|0;t=c[e>>2]|0;u=c[f>>2]|0;v=ac(u,p)|0;w=p>>1;x=c[o>>2]>>3;if((x|0)!=0){y=c[n>>2]|0;z=c[l>>2]|0;A=ac(s,r+1|0)|0;B=ac(s,r)|0;r=ac(y,w)|0;y=c[m>>2]|0;s=ac(z,w)|0;w=p&2;z=d[261976+(w<<3)|0]|0;C=d[261977+(w<<3)|0]|0;D=d[261984+(w<<3)|0]|0;E=d[261985+(w<<3)|0]|0;F=d[261986+(w<<3)|0]|0;G=d[261987+(w<<3)|0]|0;H=d[261978+(w<<3)|0]|0;I=d[261979+(w<<3)|0]|0;J=d[261980+(w<<3)|0]|0;K=d[261981+(w<<3)|0]|0;L=d[261988+(w<<3)|0]|0;M=d[261989+(w<<3)|0]|0;N=d[261990+(w<<3)|0]|0;O=d[261991+(w<<3)|0]|0;P=d[261982+(w<<3)|0]|0;Q=d[261983+(w<<3)|0]|0;w=t+(v+u)|0;u=(c[k>>2]|0)+s|0;s=y+r|0;r=x;x=t+v|0;v=q+A|0;A=q+B|0;while(1){B=r-1|0;q=(d[s]|0)+128|0;t=c[a+2292+(q<<2)>>2]|0;y=(d[u]|0)+128|0;R=(c[a+4340+(y<<2)>>2]|0)+(c[a+6388+(q<<2)>>2]|0)|0;q=c[a+8436+(y<<2)>>2]|0;y=z+(d[x]|0)|0;b[A>>1]=((b[R+(y<<1)>>1]|0)+(b[t+(y<<1)>>1]|0)&65535)+(b[q+(y<<1)>>1]|0)&65535;y=C+(d[x+1|0]|0)|0;b[A+2>>1]=((b[R+(y<<1)>>1]|0)+(b[t+(y<<1)>>1]|0)&65535)+(b[q+(y<<1)>>1]|0)&65535;y=D+(d[w]|0)|0;b[v>>1]=((b[R+(y<<1)>>1]|0)+(b[t+(y<<1)>>1]|0)&65535)+(b[q+(y<<1)>>1]|0)&65535;y=E+(d[w+1|0]|0)|0;b[v+2>>1]=((b[R+(y<<1)>>1]|0)+(b[t+(y<<1)>>1]|0)&65535)+(b[q+(y<<1)>>1]|0)&65535;y=(d[s+1|0]|0)+128|0;q=c[a+2292+(y<<2)>>2]|0;t=(d[u+1|0]|0)+128|0;R=(c[a+4340+(t<<2)>>2]|0)+(c[a+6388+(y<<2)>>2]|0)|0;y=c[a+8436+(t<<2)>>2]|0;t=F+(d[w+2|0]|0)|0;b[v+4>>1]=((b[R+(t<<1)>>1]|0)+(b[q+(t<<1)>>1]|0)&65535)+(b[y+(t<<1)>>1]|0)&65535;t=G+(d[w+3|0]|0)|0;b[v+6>>1]=((b[R+(t<<1)>>1]|0)+(b[q+(t<<1)>>1]|0)&65535)+(b[y+(t<<1)>>1]|0)&65535;t=H+(d[x+2|0]|0)|0;b[A+4>>1]=((b[R+(t<<1)>>1]|0)+(b[q+(t<<1)>>1]|0)&65535)+(b[y+(t<<1)>>1]|0)&65535;t=I+(d[x+3|0]|0)|0;b[A+6>>1]=((b[R+(t<<1)>>1]|0)+(b[q+(t<<1)>>1]|0)&65535)+(b[y+(t<<1)>>1]|0)&65535;t=(d[s+2|0]|0)+128|0;y=c[a+2292+(t<<2)>>2]|0;q=(d[u+2|0]|0)+128|0;R=(c[a+4340+(q<<2)>>2]|0)+(c[a+6388+(t<<2)>>2]|0)|0;t=c[a+8436+(q<<2)>>2]|0;q=J+(d[x+4|0]|0)|0;b[A+8>>1]=((b[R+(q<<1)>>1]|0)+(b[y+(q<<1)>>1]|0)&65535)+(b[t+(q<<1)>>1]|0)&65535;q=K+(d[x+5|0]|0)|0;b[A+10>>1]=((b[R+(q<<1)>>1]|0)+(b[y+(q<<1)>>1]|0)&65535)+(b[t+(q<<1)>>1]|0)&65535;q=L+(d[w+4|0]|0)|0;b[v+8>>1]=((b[R+(q<<1)>>1]|0)+(b[y+(q<<1)>>1]|0)&65535)+(b[t+(q<<1)>>1]|0)&65535;q=M+(d[w+5|0]|0)|0;b[v+10>>1]=((b[R+(q<<1)>>1]|0)+(b[y+(q<<1)>>1]|0)&65535)+(b[t+(q<<1)>>1]|0)&65535;q=(d[s+3|0]|0)+128|0;t=c[a+2292+(q<<2)>>2]|0;y=(d[u+3|0]|0)+128|0;R=(c[a+4340+(y<<2)>>2]|0)+(c[a+6388+(q<<2)>>2]|0)|0;q=c[a+8436+(y<<2)>>2]|0;y=N+(d[w+6|0]|0)|0;b[v+12>>1]=((b[R+(y<<1)>>1]|0)+(b[t+(y<<1)>>1]|0)&65535)+(b[q+(y<<1)>>1]|0)&65535;y=O+(d[w+7|0]|0)|0;b[v+14>>1]=((b[R+(y<<1)>>1]|0)+(b[t+(y<<1)>>1]|0)&65535)+(b[q+(y<<1)>>1]|0)&65535;y=P+(d[x+6|0]|0)|0;b[A+12>>1]=((b[R+(y<<1)>>1]|0)+(b[t+(y<<1)>>1]|0)&65535)+(b[q+(y<<1)>>1]|0)&65535;y=Q+(d[x+7|0]|0)|0;b[A+14>>1]=((b[R+(y<<1)>>1]|0)+(b[t+(y<<1)>>1]|0)&65535)+(b[q+(y<<1)>>1]|0)&65535;if((B|0)==0){break}else{w=w+8|0;u=u+4|0;s=s+4|0;r=B;x=x+8|0;v=v+16|0;A=A+16|0}}}p=p+2|0;}while((p|0)<(h|0));return h|0}function tn(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0;if((c[b+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=e+4|0;l=f+4|0;m=e+8|0;n=f+8|0;o=b+19592|0;p=0;do{q=c[i>>2]|0;r=p+g|0;s=c[j>>2]|0;t=c[e>>2]|0;u=c[f>>2]|0;v=ac(u,p)|0;w=p>>1;x=c[o>>2]>>3;if((x|0)!=0){y=c[l>>2]|0;z=ac(c[n>>2]|0,w)|0;A=c[m>>2]|0;B=ac(y,w)|0;w=c[k>>2]|0;y=ac(s,r+1|0)|0;C=q+(ac(s,r)|0)|0;r=p&6;s=d[261768+(r<<3)|0]|0;D=d[261696+(r<<3)|0]|0;E=d[261769+(r<<3)|0]|0;F=d[261697+(r<<3)|0]|0;G=d[261776+(r<<3)|0]|0;H=d[261704+(r<<3)|0]|0;I=d[261777+(r<<3)|0]|0;J=d[261705+(r<<3)|0]|0;K=d[261778+(r<<3)|0]|0;L=d[261706+(r<<3)|0]|0;M=d[261779+(r<<3)|0]|0;N=d[261707+(r<<3)|0]|0;O=d[261770+(r<<3)|0]|0;P=d[261698+(r<<3)|0]|0;Q=d[261771+(r<<3)|0]|0;R=d[261699+(r<<3)|0]|0;S=d[261772+(r<<3)|0]|0;T=d[261700+(r<<3)|0]|0;U=d[261773+(r<<3)|0]|0;V=d[261701+(r<<3)|0]|0;W=d[261780+(r<<3)|0]|0;X=d[261708+(r<<3)|0]|0;Y=d[261781+(r<<3)|0]|0;Z=d[261709+(r<<3)|0]|0;_=d[261782+(r<<3)|0]|0;$=d[261710+(r<<3)|0]|0;aa=d[261783+(r<<3)|0]|0;ab=d[261711+(r<<3)|0]|0;ad=d[261774+(r<<3)|0]|0;ae=d[261702+(r<<3)|0]|0;af=d[261775+(r<<3)|0]|0;ag=d[261703+(r<<3)|0]|0;r=w+B|0;B=A+z|0;z=x;x=t+(v+u)|0;u=t+v|0;v=q+y|0;y=C;while(1){C=z-1|0;q=(d[B]|0)+128|0;t=c[b+2292+(q<<2)>>2]|0;A=(d[r]|0)+128|0;w=c[b+4340+(A<<2)>>2]|0;ah=c[b+6388+(q<<2)>>2]|0;q=c[b+8436+(A<<2)>>2]|0;A=d[u]|0;ai=s+A|0;a[y]=((a[w+(ai+ah)|0]|0)+(a[t+ai|0]|0)&255)+(a[q+(D+A)|0]|0)&255;A=d[u+1|0]|0;ai=E+A|0;a[y+1|0]=((a[w+(ai+ah)|0]|0)+(a[t+ai|0]|0)&255)+(a[q+(F+A)|0]|0)&255;A=d[x]|0;ai=G+A|0;a[v]=((a[w+(ai+ah)|0]|0)+(a[t+ai|0]|0)&255)+(a[q+(H+A)|0]|0)&255;A=d[x+1|0]|0;ai=I+A|0;a[v+1|0]=((a[w+(ai+ah)|0]|0)+(a[t+ai|0]|0)&255)+(a[q+(J+A)|0]|0)&255;A=(d[B+1|0]|0)+128|0;q=c[b+2292+(A<<2)>>2]|0;ai=(d[r+1|0]|0)+128|0;t=c[b+4340+(ai<<2)>>2]|0;ah=c[b+6388+(A<<2)>>2]|0;A=c[b+8436+(ai<<2)>>2]|0;ai=d[x+2|0]|0;w=K+ai|0;a[v+2|0]=((a[t+(w+ah)|0]|0)+(a[q+w|0]|0)&255)+(a[A+(L+ai)|0]|0)&255;ai=d[x+3|0]|0;w=M+ai|0;a[v+3|0]=((a[t+(w+ah)|0]|0)+(a[q+w|0]|0)&255)+(a[A+(N+ai)|0]|0)&255;ai=d[u+2|0]|0;w=O+ai|0;a[y+2|0]=((a[t+(w+ah)|0]|0)+(a[q+w|0]|0)&255)+(a[A+(P+ai)|0]|0)&255;ai=d[u+3|0]|0;w=Q+ai|0;a[y+3|0]=((a[t+(w+ah)|0]|0)+(a[q+w|0]|0)&255)+(a[A+(R+ai)|0]|0)&255;ai=(d[B+2|0]|0)+128|0;A=c[b+2292+(ai<<2)>>2]|0;w=(d[r+2|0]|0)+128|0;q=c[b+4340+(w<<2)>>2]|0;ah=c[b+6388+(ai<<2)>>2]|0;ai=c[b+8436+(w<<2)>>2]|0;w=d[u+4|0]|0;t=S+w|0;a[y+4|0]=((a[q+(t+ah)|0]|0)+(a[A+t|0]|0)&255)+(a[ai+(T+w)|0]|0)&255;w=d[u+5|0]|0;t=U+w|0;a[y+5|0]=((a[q+(t+ah)|0]|0)+(a[A+t|0]|0)&255)+(a[ai+(V+w)|0]|0)&255;w=d[x+4|0]|0;t=W+w|0;a[v+4|0]=((a[q+(t+ah)|0]|0)+(a[A+t|0]|0)&255)+(a[ai+(X+w)|0]|0)&255;w=d[x+5|0]|0;t=Y+w|0;a[v+5|0]=((a[q+(t+ah)|0]|0)+(a[A+t|0]|0)&255)+(a[ai+(Z+w)|0]|0)&255;w=(d[B+3|0]|0)+128|0;ai=c[b+2292+(w<<2)>>2]|0;t=(d[r+3|0]|0)+128|0;A=c[b+4340+(t<<2)>>2]|0;ah=c[b+6388+(w<<2)>>2]|0;w=c[b+8436+(t<<2)>>2]|0;t=d[x+6|0]|0;q=_+t|0;a[v+6|0]=((a[A+(q+ah)|0]|0)+(a[ai+q|0]|0)&255)+(a[w+($+t)|0]|0)&255;t=d[x+7|0]|0;q=aa+t|0;a[v+7|0]=((a[A+(q+ah)|0]|0)+(a[ai+q|0]|0)&255)+(a[w+(ab+t)|0]|0)&255;t=d[u+6|0]|0;q=ad+t|0;a[y+6|0]=((a[A+(q+ah)|0]|0)+(a[ai+q|0]|0)&255)+(a[w+(ae+t)|0]|0)&255;t=d[u+7|0]|0;q=af+t|0;a[y+7|0]=((a[A+(q+ah)|0]|0)+(a[ai+q|0]|0)&255)+(a[w+(ag+t)|0]|0)&255;if((C|0)==0){break}else{r=r+4|0;B=B+4|0;z=C;x=x+8|0;u=u+8|0;v=v+8|0;y=y+8|0}}}p=p+2|0;}while((p|0)<(h|0));return h|0}function to(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0;if((c[b+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=e+4|0;l=f+4|0;m=e+8|0;n=f+8|0;o=b+19592|0;p=0;do{q=c[i>>2]|0;r=p+g|0;s=c[j>>2]|0;t=c[e>>2]|0;u=c[f>>2]|0;v=ac(u,p)|0;w=p>>1;x=c[o>>2]>>3;if((x|0)!=0){y=c[l>>2]|0;z=ac(c[n>>2]|0,w)|0;A=c[m>>2]|0;B=ac(y,w)|0;w=c[k>>2]|0;y=ac(s,r+1|0)|0;C=q+(ac(s,r)|0)|0;r=p&6;s=d[261840+(r<<3)|0]|0;D=d[261696+(r<<3)|0]|0;E=d[261841+(r<<3)|0]|0;F=d[261697+(r<<3)|0]|0;G=d[261848+(r<<3)|0]|0;H=d[261704+(r<<3)|0]|0;I=d[261849+(r<<3)|0]|0;J=d[261705+(r<<3)|0]|0;K=d[261850+(r<<3)|0]|0;L=d[261706+(r<<3)|0]|0;M=d[261851+(r<<3)|0]|0;N=d[261707+(r<<3)|0]|0;O=d[261842+(r<<3)|0]|0;P=d[261698+(r<<3)|0]|0;Q=d[261843+(r<<3)|0]|0;R=d[261699+(r<<3)|0]|0;S=d[261844+(r<<3)|0]|0;T=d[261700+(r<<3)|0]|0;U=d[261845+(r<<3)|0]|0;V=d[261701+(r<<3)|0]|0;W=d[261852+(r<<3)|0]|0;X=d[261708+(r<<3)|0]|0;Y=d[261853+(r<<3)|0]|0;Z=d[261709+(r<<3)|0]|0;_=d[261854+(r<<3)|0]|0;$=d[261710+(r<<3)|0]|0;aa=d[261855+(r<<3)|0]|0;ab=d[261711+(r<<3)|0]|0;ad=d[261846+(r<<3)|0]|0;ae=d[261702+(r<<3)|0]|0;af=d[261847+(r<<3)|0]|0;ag=d[261703+(r<<3)|0]|0;r=A+z|0;z=x;x=w+B|0;B=t+(v+u)|0;u=t+v|0;v=q+y|0;y=C;while(1){C=z-1|0;q=(d[r]|0)+128|0;t=c[b+2292+(q<<2)>>2]|0;w=(d[x]|0)+128|0;A=c[b+4340+(w<<2)>>2]|0;ah=c[b+6388+(q<<2)>>2]|0;q=c[b+8436+(w<<2)>>2]|0;w=d[u]|0;ai=s+w|0;aj=d[u+1|0]|0;ak=E+aj|0;a[y]=(((a[A+(aj+ah+F)|0]|0)+(a[t+ak|0]|0)&255)+(a[q+ak|0]|0)&255)<<4|((a[A+(w+ah+D)|0]|0)+(a[t+ai|0]|0)&255)+(a[q+ai|0]|0)&255;ai=d[B]|0;w=G+ai|0;ak=d[B+1|0]|0;aj=I+ak|0;a[v]=(((a[A+(ak+ah+J)|0]|0)+(a[t+aj|0]|0)&255)+(a[q+aj|0]|0)&255)<<4|((a[A+(ai+ah+H)|0]|0)+(a[t+w|0]|0)&255)+(a[q+w|0]|0)&255;w=(d[r+1|0]|0)+128|0;q=c[b+2292+(w<<2)>>2]|0;t=(d[x+1|0]|0)+128|0;ah=c[b+4340+(t<<2)>>2]|0;ai=c[b+6388+(w<<2)>>2]|0;w=c[b+8436+(t<<2)>>2]|0;t=d[B+2|0]|0;A=K+t|0;aj=d[B+3|0]|0;ak=M+aj|0;a[v+1|0]=(((a[ah+(aj+ai+N)|0]|0)+(a[q+ak|0]|0)&255)+(a[w+ak|0]|0)&255)<<4|((a[ah+(t+ai+L)|0]|0)+(a[q+A|0]|0)&255)+(a[w+A|0]|0)&255;A=d[u+2|0]|0;t=O+A|0;ak=d[u+3|0]|0;aj=Q+ak|0;a[y+1|0]=(((a[ah+(ak+ai+R)|0]|0)+(a[q+aj|0]|0)&255)+(a[w+aj|0]|0)&255)<<4|((a[ah+(A+ai+P)|0]|0)+(a[q+t|0]|0)&255)+(a[w+t|0]|0)&255;t=(d[r+2|0]|0)+128|0;w=c[b+2292+(t<<2)>>2]|0;q=(d[x+2|0]|0)+128|0;ai=c[b+4340+(q<<2)>>2]|0;A=c[b+6388+(t<<2)>>2]|0;t=c[b+8436+(q<<2)>>2]|0;q=d[u+4|0]|0;ah=S+q|0;aj=d[u+5|0]|0;ak=U+aj|0;a[y+2|0]=(((a[ai+(aj+A+V)|0]|0)+(a[w+ak|0]|0)&255)+(a[t+ak|0]|0)&255)<<4|((a[ai+(q+A+T)|0]|0)+(a[w+ah|0]|0)&255)+(a[t+ah|0]|0)&255;ah=d[B+4|0]|0;q=W+ah|0;ak=d[B+5|0]|0;aj=Y+ak|0;a[v+2|0]=(((a[ai+(ak+A+Z)|0]|0)+(a[w+aj|0]|0)&255)+(a[t+aj|0]|0)&255)<<4|((a[ai+(ah+A+X)|0]|0)+(a[w+q|0]|0)&255)+(a[t+q|0]|0)&255;q=(d[r+3|0]|0)+128|0;t=c[b+2292+(q<<2)>>2]|0;w=(d[x+3|0]|0)+128|0;A=c[b+4340+(w<<2)>>2]|0;ah=c[b+6388+(q<<2)>>2]|0;q=c[b+8436+(w<<2)>>2]|0;w=d[B+6|0]|0;ai=_+w|0;aj=d[B+7|0]|0;ak=aa+aj|0;a[v+3|0]=(((a[A+(aj+ah+ab)|0]|0)+(a[t+ak|0]|0)&255)+(a[q+ak|0]|0)&255)<<4|((a[A+(w+ah+$)|0]|0)+(a[t+ai|0]|0)&255)+(a[q+ai|0]|0)&255;ai=d[u+6|0]|0;w=ad+ai|0;ak=d[u+7|0]|0;aj=af+ak|0;a[y+3|0]=(((a[A+(ak+ah+ag)|0]|0)+(a[t+aj|0]|0)&255)+(a[q+aj|0]|0)&255)<<4|((a[A+(ai+ah+ae)|0]|0)+(a[t+w|0]|0)&255)+(a[q+w|0]|0)&255;if((C|0)==0){break}else{r=r+4|0;z=C;x=x+4|0;B=B+8|0;u=u+8|0;v=v+4|0;y=y+4|0}}}p=p+2|0;}while((p|0)<(h|0));return h|0}function tp(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0;if((c[b+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=e+4|0;l=f+4|0;m=e+8|0;n=f+8|0;o=b+19592|0;p=0;do{q=c[i>>2]|0;r=p+g|0;s=c[j>>2]|0;t=c[e>>2]|0;u=c[f>>2]|0;v=ac(u,p)|0;w=p>>1;x=c[o>>2]>>3;if((x|0)!=0){y=c[l>>2]|0;z=ac(c[n>>2]|0,w)|0;A=c[m>>2]|0;B=ac(y,w)|0;w=c[k>>2]|0;y=ac(s,r+1|0)|0;C=q+(ac(s,r)|0)|0;r=p&6;s=d[261840+(r<<3)|0]|0;D=d[261696+(r<<3)|0]|0;E=d[261841+(r<<3)|0]|0;F=d[261697+(r<<3)|0]|0;G=d[261848+(r<<3)|0]|0;H=d[261704+(r<<3)|0]|0;I=d[261849+(r<<3)|0]|0;J=d[261705+(r<<3)|0]|0;K=d[261850+(r<<3)|0]|0;L=d[261706+(r<<3)|0]|0;M=d[261851+(r<<3)|0]|0;N=d[261707+(r<<3)|0]|0;O=d[261842+(r<<3)|0]|0;P=d[261698+(r<<3)|0]|0;Q=d[261843+(r<<3)|0]|0;R=d[261699+(r<<3)|0]|0;S=d[261844+(r<<3)|0]|0;T=d[261700+(r<<3)|0]|0;U=d[261845+(r<<3)|0]|0;V=d[261701+(r<<3)|0]|0;W=d[261852+(r<<3)|0]|0;X=d[261708+(r<<3)|0]|0;Y=d[261853+(r<<3)|0]|0;Z=d[261709+(r<<3)|0]|0;_=d[261854+(r<<3)|0]|0;$=d[261710+(r<<3)|0]|0;aa=d[261855+(r<<3)|0]|0;ab=d[261711+(r<<3)|0]|0;ad=d[261846+(r<<3)|0]|0;ae=d[261702+(r<<3)|0]|0;af=d[261847+(r<<3)|0]|0;ag=d[261703+(r<<3)|0]|0;r=w+B|0;B=A+z|0;z=x;x=t+(v+u)|0;u=t+v|0;v=q+y|0;y=C;while(1){C=z-1|0;q=(d[B]|0)+128|0;t=c[b+2292+(q<<2)>>2]|0;A=(d[r]|0)+128|0;w=c[b+4340+(A<<2)>>2]|0;ah=c[b+6388+(q<<2)>>2]|0;q=c[b+8436+(A<<2)>>2]|0;A=d[u]|0;ai=s+A|0;a[y]=((a[w+(A+ah+D)|0]|0)+(a[t+ai|0]|0)&255)+(a[q+ai|0]|0)&255;ai=d[u+1|0]|0;A=E+ai|0;a[y+1|0]=((a[w+(ai+ah+F)|0]|0)+(a[t+A|0]|0)&255)+(a[q+A|0]|0)&255;A=d[x]|0;ai=G+A|0;a[v]=((a[w+(A+ah+H)|0]|0)+(a[t+ai|0]|0)&255)+(a[q+ai|0]|0)&255;ai=d[x+1|0]|0;A=I+ai|0;a[v+1|0]=((a[w+(ai+ah+J)|0]|0)+(a[t+A|0]|0)&255)+(a[q+A|0]|0)&255;A=(d[B+1|0]|0)+128|0;q=c[b+2292+(A<<2)>>2]|0;t=(d[r+1|0]|0)+128|0;ah=c[b+4340+(t<<2)>>2]|0;ai=c[b+6388+(A<<2)>>2]|0;A=c[b+8436+(t<<2)>>2]|0;t=d[x+2|0]|0;w=K+t|0;a[v+2|0]=((a[ah+(t+ai+L)|0]|0)+(a[q+w|0]|0)&255)+(a[A+w|0]|0)&255;w=d[x+3|0]|0;t=M+w|0;a[v+3|0]=((a[ah+(w+ai+N)|0]|0)+(a[q+t|0]|0)&255)+(a[A+t|0]|0)&255;t=d[u+2|0]|0;w=O+t|0;a[y+2|0]=((a[ah+(t+ai+P)|0]|0)+(a[q+w|0]|0)&255)+(a[A+w|0]|0)&255;w=d[u+3|0]|0;t=Q+w|0;a[y+3|0]=((a[ah+(w+ai+R)|0]|0)+(a[q+t|0]|0)&255)+(a[A+t|0]|0)&255;t=(d[B+2|0]|0)+128|0;A=c[b+2292+(t<<2)>>2]|0;q=(d[r+2|0]|0)+128|0;ai=c[b+4340+(q<<2)>>2]|0;w=c[b+6388+(t<<2)>>2]|0;t=c[b+8436+(q<<2)>>2]|0;q=d[u+4|0]|0;ah=S+q|0;a[y+4|0]=((a[ai+(q+w+T)|0]|0)+(a[A+ah|0]|0)&255)+(a[t+ah|0]|0)&255;ah=d[u+5|0]|0;q=U+ah|0;a[y+5|0]=((a[ai+(ah+w+V)|0]|0)+(a[A+q|0]|0)&255)+(a[t+q|0]|0)&255;q=d[x+4|0]|0;ah=W+q|0;a[v+4|0]=((a[ai+(q+w+X)|0]|0)+(a[A+ah|0]|0)&255)+(a[t+ah|0]|0)&255;ah=d[x+5|0]|0;q=Y+ah|0;a[v+5|0]=((a[ai+(ah+w+Z)|0]|0)+(a[A+q|0]|0)&255)+(a[t+q|0]|0)&255;q=(d[B+3|0]|0)+128|0;t=c[b+2292+(q<<2)>>2]|0;A=(d[r+3|0]|0)+128|0;w=c[b+4340+(A<<2)>>2]|0;ah=c[b+6388+(q<<2)>>2]|0;q=c[b+8436+(A<<2)>>2]|0;A=d[x+6|0]|0;ai=_+A|0;a[v+6|0]=((a[w+(A+ah+$)|0]|0)+(a[t+ai|0]|0)&255)+(a[q+ai|0]|0)&255;ai=d[x+7|0]|0;A=aa+ai|0;a[v+7|0]=((a[w+(ai+ah+ab)|0]|0)+(a[t+A|0]|0)&255)+(a[q+A|0]|0)&255;A=d[u+6|0]|0;ai=ad+A|0;a[y+6|0]=((a[w+(A+ah+ae)|0]|0)+(a[t+ai|0]|0)&255)+(a[q+ai|0]|0)&255;ai=d[u+7|0]|0;A=af+ai|0;a[y+7|0]=((a[w+(ai+ah+ag)|0]|0)+(a[t+A|0]|0)&255)+(a[q+A|0]|0)&255;if((C|0)==0){break}else{r=r+4|0;B=B+4|0;z=C;x=x+8|0;u=u+8|0;v=v+8|0;y=y+8|0}}}p=p+2|0;}while((p|0)<(h|0));return h|0}function tq(b,e,f,g,h,i,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0;if((c[b+56>>2]|0)==4){k=f+4|0;c[k>>2]=c[k>>2]<<1;k=f+8|0;c[k>>2]=c[k>>2]<<1}if((h|0)<=0){return h|0}k=b+19592|0;l=b+5364|0;m=b+7412|0;b=0;do{n=c[i>>2]|0;o=b+g|0;p=c[j>>2]|0;q=c[e>>2]|0;r=c[f>>2]|0;s=ac(r,b)|0;t=c[k>>2]>>3;if((t|0)!=0){u=ac(p,o+1|0)|0;v=n+(ac(p,o)|0)|0;o=b&6;p=d[261840+(o<<3)|0]|0;w=d[261841+(o<<3)|0]|0;x=d[261848+(o<<3)|0]|0;y=d[261849+(o<<3)|0]|0;z=d[261850+(o<<3)|0]|0;A=d[261851+(o<<3)|0]|0;B=d[261842+(o<<3)|0]|0;C=d[261843+(o<<3)|0]|0;D=d[261844+(o<<3)|0]|0;E=d[261845+(o<<3)|0]|0;F=d[261852+(o<<3)|0]|0;G=d[261853+(o<<3)|0]|0;H=d[261854+(o<<3)|0]|0;I=d[261855+(o<<3)|0]|0;J=d[261846+(o<<3)|0]|0;K=d[261847+(o<<3)|0]|0;o=q+(s+r)|0;r=t;t=q+s|0;s=n+u|0;u=v;while(1){v=r-1|0;n=c[l>>2]|0;q=c[m>>2]|0;L=(((a[n+((d[o+6|0]|0)+q+H)|0]|0)+(((a[n+((d[o+5|0]|0)+q+G)|0]|0)+(((a[n+((d[o+4|0]|0)+q+F)|0]|0)+(((a[n+((d[o+3|0]|0)+q+A)|0]|0)+(((a[n+((d[o+2|0]|0)+q+z)|0]|0)+(((a[n+((d[o+1|0]|0)+q+y)|0]|0)+(a[n+((d[o]|0)+q+x)|0]<<1)&255)<<1)&255)<<1)&255)<<1)&255)<<1)&255)<<1)&255)<<1)+(a[n+((d[o+7|0]|0)+q+I)|0]|0)&255;a[u]=(((a[n+((d[t+6|0]|0)+q+J)|0]|0)+(((a[n+((d[t+5|0]|0)+q+E)|0]|0)+(((a[n+((d[t+4|0]|0)+q+D)|0]|0)+(((a[n+((d[t+3|0]|0)+q+C)|0]|0)+(((a[n+((d[t+2|0]|0)+q+B)|0]|0)+(((a[n+((d[t+1|0]|0)+q+w)|0]|0)+(a[n+((d[t]|0)+q+p)|0]<<1)&255)<<1)&255)<<1)&255)<<1)&255)<<1)&255)<<1)&255)<<1)+(a[n+((d[t+7|0]|0)+q+K)|0]|0)&255;a[s]=L;if((v|0)==0){break}else{o=o+8|0;r=v;t=t+8|0;s=s+1|0;u=u+1|0}}}b=b+2|0;}while((b|0)<(h|0));return h|0}function tr(e,f,g,h,j,k){e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0;l=i;m=e+52|0;n=c[m>>2]|0;switch(n|0){case 30:case 29:case 3:case 43:case 44:case 45:case 46:case 63:case 62:case 22:case 23:case 24:case 10:{o=1;break};default:{o=0}}switch(n|0){case 43:case 45:case 63:case 47:case 49:case 65:{p=1;break};default:{p=0}}n=c[e+60>>2]|0;q=(g|0)!=0;g=q?384:326;r=c[f>>2]|0;s=r;t=(r|0)<0|0?-1:0;r=c[f+4>>2]|0;u=r;v=(r|0)<0|0?-1:0;r=-(c[f+8>>2]|0)|0;w=r;x=(r|0)<0|0?-1:0;r=-(c[f+12>>2]|0)|0;f=r;y=(r|0)<0|0?-1:0;if(q){q=N2(s,t,224,0)|0;r=N0(q,G,255,0)|0;q=G;z=N2(u,v,224,0)|0;A=N0(z,G,255,0)|0;z=G;B=N2(w,x,224,0)|0;C=N0(B,G,255,0)|0;B=G;D=N2(f,y,224,0)|0;E=N0(D,G,255,0)|0;F=0;H=0;I=0;J=65536;K=G;L=E;M=B;N=C;O=z;P=A;Q=q;R=r}else{F=0;H=1048576;I=0;J=76309;K=y;L=f;M=x;N=w;O=v;P=u;Q=t;R=s}s=j;t=(j|0)<0|0?-1:0;j=N2(J,I,s,t)|0;I=G;J=j>>>16|I<<16;j=I>>16|((I|0)<0|0?-1:0)<<16;I=N2(k,(k|0)<0|0?-1:0,s,t)|0;t=G;N2(I,t,R,Q)|0;Q=G;R=Q;s=(Q|0)<0|0?-1:0;N2(I,t,P,O)|0;O=G;P=O;Q=(O|0)<0|0?-1:0;N2(I,t,N,M)|0;M=G;N=M;O=(M|0)<0|0?-1:0;N2(I,t,L,K)|0;K=G;L=K;t=(K|0)<0|0?-1:0;K=h<<8;h=NV(H,F,K,(K|0)<0|0?-1:0)|0;K=G;F=e+11384|0;c[F>>2]=67109888;c[F+4>>2]=67109888;F=e+11392|0;c[F>>2]=67109888;c[F+4>>2]=67109888;F=NU(J<<13|0>>>19,j<<13|J>>>19,32768,0)|0;H=G;I=F>>>16|H<<16;F=I;M=(F|0)<-32767;if(M){S=-2147450880;T=-2147450880}else{k=(F|0)>32767;u=N2(I&65535,(H>>>16|0<<16)&0,65537,65537)|0;S=k?2147450879:G;T=k?2147450879:u}u=e+11336|0;c[u>>2]=T;c[u+4>>2]=S;S=NU(R<<13|0>>>19,s<<13|R>>>19,32768,0)|0;u=G;T=S>>>16|u<<16;S=T;k=(S|0)<-32767;if(k){U=-2147450880;V=-2147450880}else{H=(S|0)>32767;v=N2(T&65535,(u>>>16|0<<16)&0,65537,65537)|0;U=H?2147450879:G;V=H?2147450879:v}v=e+11344|0;c[v>>2]=V;c[v+4>>2]=U;U=NU(P<<13|0>>>19,Q<<13|P>>>19,32768,0)|0;v=G;V=U>>>16|v<<16;U=V;H=(U|0)<-32767;if(H){W=-2147450880;X=-2147450880}else{u=(U|0)>32767;w=N2(V&65535,(v>>>16|0<<16)&0,65537,65537)|0;W=u?2147450879:G;X=u?2147450879:w}w=e+11352|0;c[w>>2]=X;c[w+4>>2]=W;W=NU(L<<13|0>>>19,t<<13|L>>>19,32768,0)|0;w=G;X=W>>>16|w<<16;W=X;u=(W|0)<-32767;if(u){Y=-2147450880;Z=-2147450880}else{v=(W|0)>32767;x=N2(X&65535,(w>>>16|0<<16)&0,65537,65537)|0;Y=v?2147450879:G;Z=v?2147450879:x}x=e+11360|0;c[x>>2]=Z;c[x+4>>2]=Y;Y=NU(N<<13|0>>>19,O<<13|N>>>19,32768,0)|0;x=G;Z=Y>>>16|x<<16;Y=Z;v=(Y|0)<-32767;if(v){_=-2147450880;$=-2147450880}else{w=(Y|0)>32767;f=N2(Z&65535,(x>>>16|0<<16)&0,65537,65537)|0;_=w?2147450879:G;$=w?2147450879:f}f=e+11368|0;c[f>>2]=$;c[f+4>>2]=_;_=NU(h<<3|0>>>29,K<<3|h>>>29,32768,0)|0;f=G;$=_>>>16|f<<16;_=$;if((_|0)<-32767){aa=-2147450880;ab=-2147450880}else{w=(_|0)>32767;_=N2($&65535,(f>>>16|0<<16)&0,65537,65537)|0;aa=w?2147450879:G;ab=w?2147450879:_}_=e+11376|0;c[_>>2]=ab;c[_+4>>2]=aa;if(M){ac=-32768}else{ac=(F|0)>32767?32767:(I&65535)<<16>>16}c[e+11288>>2]=ac;ac=h>>>7|K<<25;I=ac;if((I|0)<-32767){ad=-32768}else{ad=(I|0)>32767?32767:(ac&65535)<<16>>16}c[e+11284>>2]=ad;if(k){ae=-32768}else{ae=(S|0)>32767?32767:(T&65535)<<16>>16}c[e+11292>>2]=ae;if(u){af=-32768}else{af=(W|0)>32767?32767:(X&65535)<<16>>16}c[e+11296>>2]=af;if(v){ag=-32768}else{ag=(Y|0)>32767?32767:(Z&65535)<<16>>16}c[e+11300>>2]=ag;if(H){ah=-32768}else{ah=(U|0)>32767?32767:(V&65535)<<16>>16}c[e+11304>>2]=ah;ah=N0(R<<16|0>>>16|32768,s<<16|R>>>16|0,J,j)|0;R=G;s=N0(P<<16|0>>>16|32768,Q<<16|P>>>16|0,J,j)|0;P=G;Q=N0(N<<16|0>>>16|32768,O<<16|N>>>16|0,J,j)|0;N=G;O=N0(L<<16|0>>>16|32768,t<<16|L>>>16|0,J,j)|0;L=G;t=e+2288|0;A4(c[t>>2]|0);switch(n|0){case 8:{V=o?5:0;U=o?2:3;H=o?0:6;ag=A1(3072)|0;c[t>>2]=ag;Z=NV(-25165824,-1,h,K)|0;Y=0;v=G;af=Z;while(1){Z=NU(af,v,32768,0)|0;X=Z>>>16|G<<16;Z=X;if(Z>>>0>255){ai=-Z>>31&255}else{ai=X&255}X=ai&255;Z=((X+18|0)>>>0)/36|0;a[ag+(Y+16)|0]=Z<<V&255;a[ag+(Y+1040)|0]=Z<<U&255;a[ag+(Y+2085)|0]=(((X+43|0)>>>0)/85|0)<<H&255;X=NU(af,v,J,j)|0;Z=Y+1|0;if((Z|0)<986){Y=Z;v=G;af=X}else{break}}af=g-(ah>>>9|R<<23)|0;v=0;do{Y=v-128|0;if((Y|0)<0){aj=0;ak=0}else{H=(Y|0)>255;aj=H?0:(Y|0)<0|0?-1:0;ak=H?255:Y}Y=N2(ak,aj,ah,R)|0;c[e+2292+(v<<2)>>2]=ag+(af+(Y>>>16|G<<16));v=v+1|0;}while((v|0)<512);v=(g|1024)-(Q>>>9|N<<23)|0;af=0;do{aj=af-128|0;if((aj|0)<0){al=0;am=0}else{ak=(aj|0)>255;al=ak?0:(aj|0)<0|0?-1:0;am=ak?255:aj}aj=N2(am,al,Q,N)|0;c[e+4340+(af<<2)>>2]=ag+(v+(aj>>>16|G<<16));af=af+1|0;}while((af|0)<512);af=(g|2048)-(s>>>9|P<<23)|0;v=0;do{al=v-128|0;if((al|0)<0){an=0;ao=0}else{am=(al|0)>255;an=am?0:(al|0)<0|0?-1:0;ao=am?255:al}al=N2(ao,an,s,P)|0;c[e+8436+(v<<2)>>2]=ag+(af+(al>>>16|G<<16));v=v+1|0;}while((v|0)<512);v=O>>>9|L<<23;af=L>>>9|0<<23;ag=0;while(1){an=ag-128|0;if((an|0)<0){ap=0;aq=0}else{ao=(an|0)>255;ap=ao?0:(an|0)<0|0?-1:0;aq=ao?255:an}an=N2(aq,ap,O,L)|0;ao=G;al=NV(an>>>16|ao<<16,ao>>>16|0<<16,v,af)|0;c[e+6388+(ag<<2)>>2]=al;al=ag+1|0;if((al|0)<512){ag=al}else{ar=0;break}}i=l;return ar|0};case 32:case 64:{ag=c[m>>2]|0;if((ag|0)==29){as=8}else{as=(ag|0)==27?8:0}ag=as|(o?16:0);af=as+8|0;v=as|(o?0:16);ap=c[e+56>>2]|0;aq=BP(ap)|0;if((aq|0)==0){AN(0,0,1691032,(at=i,i=i+24|0,c[at>>2]=1671176,c[at+8>>2]=1656416,c[at+16>>2]=744,at)|0);i=at;cw();return 0}do{if((ap|0)==11){au=0;av=1}else{if((a[aq+7|0]|0)<0){au=0;av=1;break}au=as+24&24;av=0}}while(0);as=A1(12288)|0;c[t>>2]=as;aq=as;as=NV(-25165824,-1,h,K)|0;ap=av?0:255<<au;au=0;av=G;al=as;while(1){as=NU(al,av,32768,0)|0;ao=as>>>16|G<<16;as=ao;if(as>>>0>255){aw=-as>>31&255}else{aw=ao&255}ao=aw&255;c[aq+(au<<2)>>2]=(ao<<ag)+ap;c[aq+(au+1024<<2)>>2]=ao<<af;c[aq+(au+2048<<2)>>2]=ao<<v;ao=NU(al,av,J,j)|0;as=au+1|0;if((as|0)<1024){au=as;av=G;al=ao}else{break}}al=aq+(g<<2)|0;R>>9|((R|0)<0|0?-1:0)<<23;av=(ah>>>9|R<<23)<<2|0>>>30;au=0;do{v=au-128|0;if((v|0)<0){ax=0;ay=0}else{af=(v|0)>255;ax=af?0:(v|0)<0|0?-1:0;ay=af?255:v}v=N2(ay,ax,ah,R)|0;af=G;af>>16|((af|0)<0|0?-1:0)<<16;c[e+2292+(au<<2)>>2]=al+(((v>>>16|af<<16)<<2|0>>>30)-av);au=au+1|0;}while((au|0)<512);au=aq+((g|1024)<<2)|0;N>>9|((N|0)<0|0?-1:0)<<23;av=(Q>>>9|N<<23)<<2|0>>>30;al=0;do{ax=al-128|0;if((ax|0)<0){az=0;aA=0}else{ay=(ax|0)>255;az=ay?0:(ax|0)<0|0?-1:0;aA=ay?255:ax}ax=N2(aA,az,Q,N)|0;ay=G;ay>>16|((ay|0)<0|0?-1:0)<<16;c[e+4340+(al<<2)>>2]=au+(((ax>>>16|ay<<16)<<2|0>>>30)-av);al=al+1|0;}while((al|0)<512);al=aq+((g|2048)<<2)|0;P>>9|((P|0)<0|0?-1:0)<<23;aq=(s>>>9|P<<23)<<2|0>>>30;av=0;do{au=av-128|0;if((au|0)<0){aB=0;aC=0}else{az=(au|0)>255;aB=az?0:(au|0)<0|0?-1:0;aC=az?255:au}au=N2(aC,aB,s,P)|0;az=G;az>>16|((az|0)<0|0?-1:0)<<16;c[e+8436+(av<<2)>>2]=al+(((au>>>16|az<<16)<<2|0>>>30)-aq);av=av+1|0;}while((av|0)<512);av=NV(0,0,O>>>9|L<<23,L>>>9|0<<23)|0;aq=av;av=aq;al=(aq|0)<0|0?-1:0;aq=0;while(1){aB=aq-128|0;if((aB|0)<0){aD=0;aE=0}else{aC=(aB|0)>255;aD=aC?0:(aB|0)<0|0?-1:0;aE=aC?255:aB}aB=N2(aE,aD,O,L)|0;aC=G;az=NU(aB>>>16|aC<<16,aC>>16|((aC|0)<0|0?-1:0)<<16,av,al)|0;c[e+6388+(aq<<2)>>2]=az<<2|0>>>30;az=aq+1|0;if((az|0)<512){aq=az}else{ar=0;break}}i=l;return ar|0};case 1:{aq=A1(1024)|0;c[t>>2]=aq;al=NV(-25165824,-1,h,K)|0;av=0;aD=G;aE=al;while(1){al=NU(aE,aD,32768,0)|0;az=al>>>16|G<<16;al=az;if(al>>>0>255){aF=-al>>31&255}else{aF=az&255}a[aq+(av+110)|0]=(aF&255)>>>7;az=NU(aE,aD,J,j)|0;al=av+1|0;if((al|0)<914){av=al;aD=G;aE=az}else{break}}aE=g-(Q>>>9|N<<23)|0;aD=0;do{av=aD-128|0;if((av|0)<0){aG=0;aH=0}else{aF=(av|0)>255;aG=aF?0:(av|0)<0|0?-1:0;aH=aF?255:av}av=N2(aH,aG,Q,N)|0;c[e+4340+(aD<<2)>>2]=aq+(aE+(av>>>16|G<<16));aD=aD+1|0;}while((aD|0)<512);aD=O>>>9|L<<23;aE=L>>>9|0<<23;aq=0;while(1){aG=aq-128|0;if((aG|0)<0){aI=0;aJ=0}else{aH=(aG|0)>255;aI=aH?0:(aG|0)<0|0?-1:0;aJ=aH?255:aG}aG=N2(aJ,aI,O,L)|0;aH=G;av=NV(aG>>>16|aH<<16,aH>>>16|0<<16,aD,aE)|0;c[e+6388+(aq<<2)>>2]=av;av=aq+1|0;if((av|0)<512){aq=av}else{ar=0;break}}i=l;return ar|0};case 4:case 132:{aq=o?3:0;aE=o?0:3;aD=A1(3072)|0;c[t>>2]=aD;aI=NV(-25165824,-1,h,K)|0;aJ=0;av=G;aH=aI;while(1){aI=NU(aH,av,32768,0)|0;aG=aI>>>16|G<<16;aI=aG;if(aI>>>0>255){aK=-aI>>31&255}else{aK=aG&255}aG=aK&255;aI=aG>>>7;a[aD+(aJ+110)|0]=aI<<aq&255;a[aD+(aJ+1061)|0]=(((aG+43|0)>>>0)/85|0)<<1&255;a[aD+(aJ+2158)|0]=aI<<aE&255;aI=NU(aH,av,J,j)|0;aG=aJ+1|0;if((aG|0)<914){aJ=aG;av=G;aH=aI}else{break}}aH=g-(ah>>>9|R<<23)|0;av=0;do{aJ=av-128|0;if((aJ|0)<0){aL=0;aM=0}else{aE=(aJ|0)>255;aL=aE?0:(aJ|0)<0|0?-1:0;aM=aE?255:aJ}aJ=N2(aM,aL,ah,R)|0;c[e+2292+(av<<2)>>2]=aD+(aH+(aJ>>>16|G<<16));av=av+1|0;}while((av|0)<512);av=(g|1024)-(Q>>>9|N<<23)|0;aH=0;do{aL=aH-128|0;if((aL|0)<0){aN=0;aO=0}else{aM=(aL|0)>255;aN=aM?0:(aL|0)<0|0?-1:0;aO=aM?255:aL}aL=N2(aO,aN,Q,N)|0;c[e+4340+(aH<<2)>>2]=aD+(av+(aL>>>16|G<<16));aH=aH+1|0;}while((aH|0)<512);aH=(g|2048)-(s>>>9|P<<23)|0;av=0;do{aN=av-128|0;if((aN|0)<0){aP=0;aQ=0}else{aO=(aN|0)>255;aP=aO?0:(aN|0)<0|0?-1:0;aQ=aO?255:aN}aN=N2(aQ,aP,s,P)|0;c[e+8436+(av<<2)>>2]=aD+(aH+(aN>>>16|G<<16));av=av+1|0;}while((av|0)<512);av=O>>>9|L<<23;aH=L>>>9|0<<23;aD=0;while(1){aP=aD-128|0;if((aP|0)<0){aR=0;aS=0}else{aQ=(aP|0)>255;aR=aQ?0:(aP|0)<0|0?-1:0;aS=aQ?255:aP}aP=N2(aS,aR,O,L)|0;aQ=G;aN=NV(aP>>>16|aQ<<16,aQ>>>16|0<<16,av,aH)|0;c[e+6388+(aD<<2)>>2]=aN;aN=aD+1|0;if((aN|0)<512){aD=aN}else{ar=0;break}}i=l;return ar|0};case 24:case 48:{aD=A1(1024)|0;c[t>>2]=aD;aH=NV(-25165824,-1,h,K)|0;av=0;aR=G;aS=aH;while(1){aH=NU(aS,aR,32768,0)|0;aN=aH>>>16|G<<16;aH=aN;if(aH>>>0>255){aT=-aH>>31&255}else{aT=aN&255}a[aD+av|0]=aT;aN=NU(aS,aR,J,j)|0;aH=av+1|0;if((aH|0)<1024){av=aH;aR=G;aS=aN}else{break}}aS=g-(ah>>>9|R<<23)|0;aR=0;do{av=aR-128|0;if((av|0)<0){aU=0;aV=0}else{aT=(av|0)>255;aU=aT?0:(av|0)<0|0?-1:0;aV=aT?255:av}av=N2(aV,aU,ah,R)|0;c[e+2292+(aR<<2)>>2]=aD+(aS+(av>>>16|G<<16));aR=aR+1|0;}while((aR|0)<512);aR=g-(Q>>>9|N<<23)|0;aS=0;do{aU=aS-128|0;if((aU|0)<0){aW=0;aX=0}else{aV=(aU|0)>255;aW=aV?0:(aU|0)<0|0?-1:0;aX=aV?255:aU}aU=N2(aX,aW,Q,N)|0;c[e+4340+(aS<<2)>>2]=aD+(aR+(aU>>>16|G<<16));aS=aS+1|0;}while((aS|0)<512);aS=g-(s>>>9|P<<23)|0;aR=0;do{aW=aR-128|0;if((aW|0)<0){aY=0;aZ=0}else{aX=(aW|0)>255;aY=aX?0:(aW|0)<0|0?-1:0;aZ=aX?255:aW}aW=N2(aZ,aY,s,P)|0;c[e+8436+(aR<<2)>>2]=aD+(aS+(aW>>>16|G<<16));aR=aR+1|0;}while((aR|0)<512);aR=O>>>9|L<<23;aS=L>>>9|0<<23;aD=0;while(1){aY=aD-128|0;if((aY|0)<0){a_=0;a$=0}else{aZ=(aY|0)>255;a_=aZ?0:(aY|0)<0|0?-1:0;a$=aZ?255:aY}aY=N2(a$,a_,O,L)|0;aZ=G;aW=NV(aY>>>16|aZ<<16,aZ>>>16|0<<16,aR,aS)|0;c[e+6388+(aD<<2)>>2]=aW;aW=aD+1|0;if((aW|0)<512){aD=aW}else{ar=0;break}}i=l;return ar|0};case 12:{aD=o?8:0;aS=o?0:8;aR=A1(6144)|0;c[t>>2]=aR;a_=aR;aR=NV(-25165824,-1,h,K)|0;a$=0;aW=G;aZ=aR;while(1){aR=NU(aZ,aW,32768,0)|0;aY=aR>>>16|G<<16;aR=aY;if(aR>>>0>255){a0=-aR>>31&255}else{a0=aY&255}aY=(a0&255)>>>4;b[a_+(a$<<1)>>1]=aY<<aD&65535;b[a_+(a$+1024<<1)>>1]=aY<<4&65535;b[a_+(a$+2048<<1)>>1]=aY<<aS&65535;aY=NU(aZ,aW,J,j)|0;aR=a$+1|0;if((aR|0)<1024){a$=aR;aW=G;aZ=aY}else{break}}if(p){aZ=0;do{aW=a_+(aZ<<1)|0;b[aW>>1]=cT(b[aW>>1]|0)|0;aZ=aZ+1|0;}while((aZ|0)<3072)}aZ=a_+(g<<1)|0;R>>9|((R|0)<0|0?-1:0)<<23;aW=(ah>>>9|R<<23)<<1|0>>>31;a$=0;do{aS=a$-128|0;if((aS|0)<0){a1=0;a2=0}else{aD=(aS|0)>255;a1=aD?0:(aS|0)<0|0?-1:0;a2=aD?255:aS}aS=N2(a2,a1,ah,R)|0;aD=G;aD>>16|((aD|0)<0|0?-1:0)<<16;c[e+2292+(a$<<2)>>2]=aZ+(((aS>>>16|aD<<16)<<1|0>>>31)-aW);a$=a$+1|0;}while((a$|0)<512);a$=a_+((g|1024)<<1)|0;N>>9|((N|0)<0|0?-1:0)<<23;aW=(Q>>>9|N<<23)<<1|0>>>31;aZ=0;do{a1=aZ-128|0;if((a1|0)<0){a3=0;a4=0}else{a2=(a1|0)>255;a3=a2?0:(a1|0)<0|0?-1:0;a4=a2?255:a1}a1=N2(a4,a3,Q,N)|0;a2=G;a2>>16|((a2|0)<0|0?-1:0)<<16;c[e+4340+(aZ<<2)>>2]=a$+(((a1>>>16|a2<<16)<<1|0>>>31)-aW);aZ=aZ+1|0;}while((aZ|0)<512);aZ=a_+((g|2048)<<1)|0;P>>9|((P|0)<0|0?-1:0)<<23;a_=(s>>>9|P<<23)<<1|0>>>31;aW=0;do{a$=aW-128|0;if((a$|0)<0){a5=0;a6=0}else{a3=(a$|0)>255;a5=a3?0:(a$|0)<0|0?-1:0;a6=a3?255:a$}a$=N2(a6,a5,s,P)|0;a3=G;a3>>16|((a3|0)<0|0?-1:0)<<16;c[e+8436+(aW<<2)>>2]=aZ+(((a$>>>16|a3<<16)<<1|0>>>31)-a_);aW=aW+1|0;}while((aW|0)<512);aW=NV(0,0,O>>>9|L<<23,L>>>9|0<<23)|0;a_=aW;aW=a_;aZ=(a_|0)<0|0?-1:0;a_=0;while(1){a5=a_-128|0;if((a5|0)<0){a7=0;a8=0}else{a6=(a5|0)>255;a7=a6?0:(a5|0)<0|0?-1:0;a8=a6?255:a5}a5=N2(a8,a7,O,L)|0;a6=G;a3=NU(a5>>>16|a6<<16,a6>>16|((a6|0)<0|0?-1:0)<<16,aW,aZ)|0;c[e+6388+(a_<<2)>>2]=a3<<1|0>>>31;a3=a_+1|0;if((a3|0)<512){a_=a3}else{ar=0;break}}i=l;return ar|0};case 15:case 16:{a_=n-5|0;aZ=o?a_:0;aW=o?0:a_;a_=A1(6144)|0;c[t>>2]=a_;o=a_;a_=NV(-25165824,-1,h,K)|0;K=18-n|0;h=0;a7=G;a8=a_;while(1){a_=NU(a8,a7,32768,0)|0;a3=a_>>>16|G<<16;a_=a3;if(a_>>>0>255){a9=-a_>>31&255}else{a9=a3&255}a3=a9&255;a_=a3>>>3;b[o+(h<<1)>>1]=a_<<aZ&65535;b[o+(h+1024<<1)>>1]=a3>>>(K>>>0)<<5&65535;b[o+(h+2048<<1)>>1]=a_<<aW&65535;a_=NU(a8,a7,J,j)|0;a3=h+1|0;if((a3|0)<1024){h=a3;a7=G;a8=a_}else{break}}if(p){p=0;do{a8=o+(p<<1)|0;b[a8>>1]=cT(b[a8>>1]|0)|0;p=p+1|0;}while((p|0)<3072)}p=o+(g<<1)|0;R>>9|((R|0)<0|0?-1:0)<<23;a8=(ah>>>9|R<<23)<<1|0>>>31;a7=0;do{h=a7-128|0;if((h|0)<0){ba=0;bb=0}else{j=(h|0)>255;ba=j?0:(h|0)<0|0?-1:0;bb=j?255:h}h=N2(bb,ba,ah,R)|0;j=G;j>>16|((j|0)<0|0?-1:0)<<16;c[e+2292+(a7<<2)>>2]=p+(((h>>>16|j<<16)<<1|0>>>31)-a8);a7=a7+1|0;}while((a7|0)<512);a7=o+((g|1024)<<1)|0;N>>9|((N|0)<0|0?-1:0)<<23;a8=(Q>>>9|N<<23)<<1|0>>>31;p=0;do{R=p-128|0;if((R|0)<0){bc=0;bd=0}else{ah=(R|0)>255;bc=ah?0:(R|0)<0|0?-1:0;bd=ah?255:R}R=N2(bd,bc,Q,N)|0;ah=G;ah>>16|((ah|0)<0|0?-1:0)<<16;c[e+4340+(p<<2)>>2]=a7+(((R>>>16|ah<<16)<<1|0>>>31)-a8);p=p+1|0;}while((p|0)<512);p=o+((g|2048)<<1)|0;P>>9|((P|0)<0|0?-1:0)<<23;g=(s>>>9|P<<23)<<1|0>>>31;o=0;do{a8=o-128|0;if((a8|0)<0){be=0;bf=0}else{a7=(a8|0)>255;be=a7?0:(a8|0)<0|0?-1:0;bf=a7?255:a8}a8=N2(bf,be,s,P)|0;a7=G;a7>>16|((a7|0)<0|0?-1:0)<<16;c[e+8436+(o<<2)>>2]=p+(((a8>>>16|a7<<16)<<1|0>>>31)-g);o=o+1|0;}while((o|0)<512);o=NV(0,0,O>>>9|L<<23,L>>>9|0<<23)|0;g=o;o=g;p=(g|0)<0|0?-1:0;g=0;while(1){P=g-128|0;if((P|0)<0){bg=0;bh=0}else{s=(P|0)>255;bg=s?0:(P|0)<0|0?-1:0;bh=s?255:P}P=N2(bh,bg,O,L)|0;s=G;be=NU(P>>>16|s<<16,s>>16|((s|0)<0|0?-1:0)<<16,o,p)|0;c[e+6388+(g<<2)>>2]=be<<1|0>>>31;be=g+1|0;if((be|0)<512){g=be}else{ar=0;break}}i=l;return ar|0};default:{c[t>>2]=0;t=BP(c[m>>2]|0)|0;if((t|0)==0){AN(0,0,1691032,(at=i,i=i+24|0,c[at>>2]=1671176,c[at+8>>2]=1656416,c[at+16>>2]=772,at)|0);i=at;cw();return 0}do{if((d[t+4|0]|0)>1){if((a[t+7|0]&16)==0|(n|0)<25){break}else{ar=-1}i=l;return ar|0}}while(0);AN(e,16,1718800,(at=i,i=i+8|0,c[at>>2]=n,at)|0);i=at;ar=-1;i=l;return ar|0}}return 0}function ts(d,f,g,h,j,k,l,m){d=d|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;n=i;o=c[d+52>>2]|0;p=BP(o)|0;q=BP(o)|0;if((q|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=619,r)|0);i=r;cw()}s=(b[q+8>>1]&30720)==30720;q=BP(o)|0;t=(q|0)==0;do{if(s){if(t){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}c[g>>2]=(a[q+7|0]&1)!=0?78:66;u=BP(o)|0;if((u|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}else{c[f>>2]=(a[u+7|0]&1)!=0?114:94;break}}else{if(t){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=626,r)|0);i=r;cw()}if((((e[q+8>>1]|0)>>>11&15)-8&65535)>=6){c[f>>2]=106;c[g>>2]=56;if((o-25|0)>>>0>=2){break}c[h>>2]=60;break}u=(e[p+8>>1]|0)>>>11&15;if((u<<16>>16|0)==8){v=BP(o)|0;if((v|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}c[g>>2]=(a[v+7|0]&1)!=0?74:120;v=BP(o)|0;if((v|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}else{c[f>>2]=(a[v+7|0]&1)!=0?82:64;break}}else if((u<<16>>16|0)==9){v=BP(o)|0;if((v|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}c[g>>2]=(a[v+7|0]&1)!=0?32:90;v=BP(o)|0;if((v|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}else{c[f>>2]=(a[v+7|0]&1)!=0?86:108;break}}else if((u<<16>>16|0)==11){v=BP(o)|0;if((v|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}c[g>>2]=(a[v+7|0]&1)!=0?14:42;v=BP(o)|0;if((v|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}else{c[f>>2]=(a[v+7|0]&1)!=0?48:120;break}}else if((u<<16>>16|0)==13){u=BP(o)|0;if((u|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}c[g>>2]=(a[u+7|0]&1)!=0?26:40;u=BP(o)|0;if((u|0)==0){AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1671168,c[r+8>>2]=1656384,c[r+16>>2]=635,r)|0);i=r;cw()}else{c[f>>2]=(a[u+7|0]&1)!=0?128:24;break}}else{AN(0,0,1639912,(r=i,i=i+24|0,c[r>>2]=1718768,c[r+8>>2]=1690944,c[r+16>>2]=1829,r)|0);i=r;cw()}}}while(0);do{if((c[d+2284>>2]&8192|0)==0){w=5949}else{L8214:do{switch(o|0){case 28:{if((c[d+2180>>2]|0)==0){c[l>>2]=30;c[k>>2]=12;c[j>>2]=24;break L8214}else{c[l>>2]=68;c[k>>2]=18;c[j>>2]=10;break L8214}break};case 27:{if((c[d+2180>>2]|0)==0){c[l>>2]=20;c[k>>2]=14;c[j>>2]=32;break L8214}else{c[l>>2]=16;c[k>>2]=34;c[j>>2]=58;break L8214}break};case 30:{if((c[d+2180>>2]|0)==0){c[l>>2]=44;c[k>>2]=64;c[j>>2]=64;break L8214}else{c[l>>2]=76;c[k>>2]=70;c[j>>2]=84;break L8214}break};case 29:{if((c[d+2180>>2]|0)==0){c[l>>2]=12;c[k>>2]=24;c[j>>2]=56;break L8214}else{c[l>>2]=8;c[k>>2]=60;c[j>>2]=74;break L8214}break};case 2:{c[l>>2]=52;c[k>>2]=22;c[j>>2]=6;break};case 3:{c[l>>2]=28;c[k>>2]=10;c[j>>2]=92;break};case 21:{c[l>>2]=56;c[k>>2]=40;c[j>>2]=14;break};case 24:{c[l>>2]=40;c[k>>2]=72;c[j>>2]=4;break};case 19:{c[l>>2]=10;c[k>>2]=56;c[j>>2]=54;break};case 22:{c[l>>2]=78;c[k>>2]=48;c[j>>2]=46;break};case 82:case 83:case 84:case 85:case 86:case 313:case 314:case 315:case 316:case 87:case 88:case 317:{c[m>>2]=6;break};default:{}}}while(0);if((c[l>>2]|0)!=0){w=5974;break}if((c[m>>2]|0)==0){w=5949}else{w=5974}}}while(0);L8240:do{if((w|0)==5949){switch(o|0){case 23:case 20:{c[j>>2]=28;c[k>>2]=20;c[l>>2]=48;w=5974;break L8240;break};case 22:case 19:{c[j>>2]=36;c[k>>2]=46;c[l>>2]=72;w=5974;break L8240;break};case 24:case 21:{c[j>>2]=2;c[k>>2]=16;c[l>>2]=34;w=5974;break L8240;break};case 1:{w=5977;break L8240;break};case 17:{w=5978;break L8240;break};case 9:{w=5975;break L8240;break};case 46:case 45:case 50:case 49:{c[j>>2]=22;c[k>>2]=28;c[l>>2]=50;w=5974;break L8240;break};case 292:{if((c[d+2180>>2]|0)==0){c[j>>2]=18;c[k>>2]=36;c[l>>2]=26;w=5974;break L8240}c[j>>2]=52;c[k>>2]=32;c[l>>2]=62;i=n;return};case 291:{if((c[d+2180>>2]|0)==0){c[j>>2]=62;c[k>>2]=30;c[l>>2]=22;w=5974;break L8240}c[j>>2]=94;c[k>>2]=68;c[l>>2]=70;i=n;return};case 42:{c[j>>2]=68;c[k>>2]=66;c[l>>2]=14;i=n;return};case 41:{c[j>>2]=30;c[k>>2]=52;c[l>>2]=38;i=n;return};case 68:{c[j>>2]=66;c[k>>2]=2;c[l>>2]=32;i=n;return};case 67:{c[j>>2]=50;c[k>>2]=42;c[l>>2]=64;i=n;return};case 30:case 28:{if((c[d+2180>>2]|0)==0){c[j>>2]=86;c[k>>2]=8;c[l>>2]=74;w=5974;break L8240}else{c[j>>2]=34;c[k>>2]=58;c[l>>2]=54;w=5974;break L8240}break};case 29:case 27:{if((c[d+2180>>2]|0)==0){c[j>>2]=44;c[k>>2]=4;c[l>>2]=66;w=5974;break L8240}else{c[j>>2]=8;c[k>>2]=76;c[l>>2]=2;w=5974;break L8240}break};case 2:{c[j>>2]=78;c[k>>2]=62;c[l>>2]=42;i=n;return};case 3:{c[j>>2]=82;c[k>>2]=38;c[l>>2]=24;i=n;return};case 44:case 43:case 48:case 47:{c[j>>2]=70;c[k>>2]=80;c[l>>2]=4;w=5974;break L8240;break};case 62:case 63:case 64:case 65:{c[j>>2]=76;c[k>>2]=74;c[l>>2]=18;w=5974;break L8240;break};case 10:{w=5976;break L8240;break};default:{i=n;return}}}}while(0);do{if((w|0)==5974){if((o|0)==1){w=5977;break}else if((o|0)==17){w=5978;break}else if((o|0)==9){w=5975;break}else if((o|0)==10){w=5976;break}i=n;return}}while(0);if((w|0)==5977){c[j>>2]=90;c[k>>2]=6;c[l>>2]=58;i=n;return}else if((w|0)==5978){c[j>>2]=16;c[k>>2]=26;c[l>>2]=60;i=n;return}else if((w|0)==5975){c[j>>2]=88;c[k>>2]=82;c[l>>2]=46;i=n;return}else if((w|0)==5976){c[j>>2]=72;c[k>>2]=78;c[l>>2]=36;i=n;return}}function tt(d,e,f,g,h,i,j){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0;j=g;if((h|0)<=0){return}if((e|0)>0){k=0}else{NM(g|0,0,h<<1|0);return}do{g=0;i=-1073725440;do{i=(ac(b[d+(g<<1)>>1]|0,c[(c[f+(g<<2)>>2]|0)+(k<<2)>>2]|0)|0)+i|0;g=g+1|0;}while((g|0)<(e|0));g=i>>15;if((g+32768|0)>>>0>65535){l=(i>>31^32767)&65535}else{l=g&65535}g=j+(k<<1)|0;y=cT(l^-32768|0)|0;a[g]=y&255;y=y>>8;a[g+1|0]=y&255;k=k+1|0;}while((k|0)<(h|0));return}function tu(d,e,f,g,h,i,j){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0;j=g;if((h|0)<=0){return}if((e|0)>0){k=0}else{NM(g|0,0,h<<1|0);return}do{g=0;i=-1073725440;do{i=(ac(b[d+(g<<1)>>1]|0,c[(c[f+(g<<2)>>2]|0)+(k<<2)>>2]|0)|0)+i|0;g=g+1|0;}while((g|0)<(e|0));g=i>>15;if((g+32768|0)>>>0>65535){l=(i>>31^32767)&65535}else{l=g&65535}g=j+(k<<1)|0;y=l^-32768;a[g]=y&255;y=y>>8;a[g+1|0]=y&255;k=k+1|0;}while((k|0)<(h|0));return}function tv(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;g=b;b=d;if((e|0)>0){h=0}else{return}do{d=(c[g+(h<<2)>>2]|0)+4>>3;if(d>>>0>65535){i=-d>>31&65535}else{i=d&65535}d=b+(h<<1)|0;y=cT(i|0)|0;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;h=h+1|0;}while((h|0)<(e|0));return}function tw(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;g=b;b=d;if((e|0)>0){h=0}else{return}do{d=(c[g+(h<<2)>>2]|0)+4>>3;if(d>>>0>65535){i=-d>>31&65535}else{i=d&65535}d=b+(h<<1)|0;y=i;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;h=h+1|0;}while((h|0)<(e|0));return}function tx(d,e,f,g,h,i,j){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0;j=g;if((h|0)<=0){return}if((e|0)>0){k=0}else{NM(g|0,0,h<<1|0);return}do{g=0;i=131072;do{i=(ac(b[d+(g<<1)>>1]|0,b[(c[f+(g<<2)>>2]|0)+(k<<1)>>1]|0)|0)+i|0;g=g+1|0;}while((g|0)<(e|0));g=i>>18;if(g>>>0>511){l=-g>>31&511}else{l=g&65535}g=j+(k<<1)|0;y=cT(l|0)|0;a[g]=y&255;y=y>>8;a[g+1|0]=y&255;k=k+1|0;}while((k|0)<(h|0));return}function ty(d,e,f,g,h,i,j){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0;j=g;if((h|0)<=0){return}g=(e|0)>0;i=0;do{do{if(g){k=0;l=131072;do{l=(ac(b[d+(k<<1)>>1]|0,b[(c[f+(k<<2)>>2]|0)+(i<<1)>>1]|0)|0)+l|0;k=k+1|0;}while((k|0)<(e|0));k=l>>18;if(k>>>0<=511){m=k&65535;n=6051;break}o=-k>>31&511}else{m=0;n=6051}}while(0);if((n|0)==6051){n=0;o=m}k=j+(i<<1)|0;y=o;a[k]=y&255;y=y>>8;a[k+1|0]=y&255;i=i+1|0;}while((i|0)<(h|0));return}function tz(c,d,e,f,g){c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;g=d;if((e|0)>0){h=0}else{return}do{d=(b[c+(h<<1)>>1]|0)+32>>6;if(d>>>0>511){i=-d>>31&511}else{i=d&65535}d=g+(h<<1)|0;y=cT(i|0)|0;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;h=h+1|0;}while((h|0)<(e|0));return}function tA(c,d,e,f,g){c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;g=d;if((e|0)>0){h=0}else{return}do{d=(b[c+(h<<1)>>1]|0)+32>>6;if(d>>>0>511){i=-d>>31&511}else{i=d&65535}d=g+(h<<1)|0;y=i;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;h=h+1|0;}while((h|0)<(e|0));return}function tB(d,e,f,g,h,i,j){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0;j=g;if((h|0)<=0){return}g=(e|0)>0;i=0;do{do{if(g){k=0;l=65536;do{l=(ac(b[d+(k<<1)>>1]|0,b[(c[f+(k<<2)>>2]|0)+(i<<1)>>1]|0)|0)+l|0;k=k+1|0;}while((k|0)<(e|0));k=l>>17;if(k>>>0<=1023){m=k&65535;n=6078;break}o=-k>>31&1023}else{m=0;n=6078}}while(0);if((n|0)==6078){n=0;o=m}k=j+(i<<1)|0;y=cT(o|0)|0;a[k]=y&255;y=y>>8;a[k+1|0]=y&255;i=i+1|0;}while((i|0)<(h|0));return}function tC(d,e,f,g,h,i,j){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0;j=g;if((h|0)<=0){return}g=(e|0)>0;i=0;do{do{if(g){k=0;l=65536;do{l=(ac(b[d+(k<<1)>>1]|0,b[(c[f+(k<<2)>>2]|0)+(i<<1)>>1]|0)|0)+l|0;k=k+1|0;}while((k|0)<(e|0));k=l>>17;if(k>>>0<=1023){m=k&65535;n=6089;break}o=-k>>31&1023}else{m=0;n=6089}}while(0);if((n|0)==6089){n=0;o=m}k=j+(i<<1)|0;y=o;a[k]=y&255;y=y>>8;a[k+1|0]=y&255;i=i+1|0;}while((i|0)<(h|0));return}function tD(c,d,e,f,g){c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;g=d;if((e|0)>0){h=0}else{return}do{d=(b[c+(h<<1)>>1]|0)+16>>5;if(d>>>0>1023){i=-d>>31&1023}else{i=d&65535}d=g+(h<<1)|0;y=cT(i|0)|0;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;h=h+1|0;}while((h|0)<(e|0));return}function tE(c,d,e,f,g){c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;g=d;if((e|0)>0){h=0}else{return}do{d=(b[c+(h<<1)>>1]|0)+16>>5;if(d>>>0>1023){i=-d>>31&1023}else{i=d&65535}d=g+(h<<1)|0;y=i;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;h=h+1|0;}while((h|0)<(e|0));return}function tF(d,e,f,g,h,i,j){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0;j=g;if((h|0)<=0){return}g=(e|0)>0;i=0;do{do{if(g){k=0;l=16384;do{l=(ac(b[d+(k<<1)>>1]|0,b[(c[f+(k<<2)>>2]|0)+(i<<1)>>1]|0)|0)+l|0;k=k+1|0;}while((k|0)<(e|0));k=l>>15;if(k>>>0<=4095){m=k&65535;n=6116;break}o=-k>>31&4095}else{m=0;n=6116}}while(0);if((n|0)==6116){n=0;o=m}k=j+(i<<1)|0;y=cT(o|0)|0;a[k]=y&255;y=y>>8;a[k+1|0]=y&255;i=i+1|0;}while((i|0)<(h|0));return}function tG(d,e,f,g,h,i,j){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0;j=g;if((h|0)<=0){return}g=(e|0)>0;i=0;do{do{if(g){k=0;l=16384;do{l=(ac(b[d+(k<<1)>>1]|0,b[(c[f+(k<<2)>>2]|0)+(i<<1)>>1]|0)|0)+l|0;k=k+1|0;}while((k|0)<(e|0));k=l>>15;if(k>>>0<=4095){m=k&65535;n=6127;break}o=-k>>31&4095}else{m=0;n=6127}}while(0);if((n|0)==6127){n=0;o=m}k=j+(i<<1)|0;y=o;a[k]=y&255;y=y>>8;a[k+1|0]=y&255;i=i+1|0;}while((i|0)<(h|0));return}function tH(c,d,e,f,g){c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;g=d;if((e|0)>0){h=0}else{return}do{d=(b[c+(h<<1)>>1]|0)+4>>3;if(d>>>0>4095){i=-d>>31&4095}else{i=d&65535}d=g+(h<<1)|0;y=cT(i|0)|0;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;h=h+1|0;}while((h|0)<(e|0));return}function tI(c,d,e,f,g){c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;g=d;if((e|0)>0){h=0}else{return}do{d=(b[c+(h<<1)>>1]|0)+4>>3;if(d>>>0>4095){i=-d>>31&4095}else{i=d&65535}d=g+(h<<1)|0;y=i;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;h=h+1|0;}while((h|0)<(e|0));return}function tJ(d,e,f,g,h,i,j){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0;j=g;if((h|0)<=0){return}g=(e|0)>0;i=0;do{do{if(g){k=0;l=4096;do{l=(ac(b[d+(k<<1)>>1]|0,b[(c[f+(k<<2)>>2]|0)+(i<<1)>>1]|0)|0)+l|0;k=k+1|0;}while((k|0)<(e|0));k=l>>13;if(k>>>0<=16383){m=k&65535;n=6154;break}o=-k>>31&16383}else{m=0;n=6154}}while(0);if((n|0)==6154){n=0;o=m}k=j+(i<<1)|0;y=cT(o|0)|0;a[k]=y&255;y=y>>8;a[k+1|0]=y&255;i=i+1|0;}while((i|0)<(h|0));return}function tK(d,e,f,g,h,i,j){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0;j=g;if((h|0)<=0){return}g=(e|0)>0;i=0;do{do{if(g){k=0;l=4096;do{l=(ac(b[d+(k<<1)>>1]|0,b[(c[f+(k<<2)>>2]|0)+(i<<1)>>1]|0)|0)+l|0;k=k+1|0;}while((k|0)<(e|0));k=l>>13;if(k>>>0<=16383){m=k&65535;n=6165;break}o=-k>>31&16383}else{m=0;n=6165}}while(0);if((n|0)==6165){n=0;o=m}k=j+(i<<1)|0;y=o;a[k]=y&255;y=y>>8;a[k+1|0]=y&255;i=i+1|0;}while((i|0)<(h|0));return}function tL(c,d,e,f,g){c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;g=d;if((e|0)>0){h=0}else{return}do{d=(b[c+(h<<1)>>1]|0)+1>>1;if(d>>>0>16383){i=-d>>31&16383}else{i=d&65535}d=g+(h<<1)|0;y=cT(i|0)|0;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;h=h+1|0;}while((h|0)<(e|0));return}function tM(c,d,e,f,g){c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0;g=d;if((e|0)>0){h=0}else{return}do{d=(b[c+(h<<1)>>1]|0)+1>>1;if(d>>>0>16383){i=-d>>31&16383}else{i=d&65535}d=g+(h<<1)|0;y=i;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;h=h+1|0;}while((h|0)<(e|0));return}function tN(c,e,f,g,h){c=c|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0;if((f|0)>0){i=0}else{return}do{j=(d[g+(i+h&7)|0]|0)+(b[c+(i<<1)>>1]|0)>>7;if(j>>>0>255){k=-j>>31&255}else{k=j&255}a[e+i|0]=k;i=i+1|0;}while((i|0)<(f|0));return}function tO(e,f,g,h,i,j,k){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;if((i|0)<=0){return}l=(f|0)>0;m=0;do{n=d[j+(m+k&7)|0]<<12;if(l){o=n;p=0;while(1){q=(ac(b[e+(p<<1)>>1]|0,b[(c[g+(p<<2)>>2]|0)+(m<<1)>>1]|0)|0)+o|0;r=p+1|0;if((r|0)<(f|0)){o=q;p=r}else{s=q;break}}}else{s=n}p=s>>19;if(p>>>0>255){t=-p>>31&255}else{t=p&255}a[h+m|0]=t;m=m+1|0;}while((m|0)<(i|0));return}function tP(e,f,g,h,i,j,k){e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;l=c[e+23800>>2]|0;m=(k|0)>0;if((c[e+52>>2]|0)==25){if(!m){return}e=(g|0)>0;n=0;do{o=d[l+(n&7)|0]<<12;p=d[l+(n+3&7)|0]<<12;if(e){q=o;r=p;s=0;while(1){t=b[f+(s<<1)>>1]|0;u=(ac(t,b[(c[h+(s<<2)>>2]|0)+(n<<1)>>1]|0)|0)+q|0;v=(ac(b[(c[i+(s<<2)>>2]|0)+(n<<1)>>1]|0,t)|0)+r|0;t=s+1|0;if((t|0)<(g|0)){q=u;r=v;s=t}else{w=u;x=v;break}}}else{w=o;x=p}s=w>>19;if(s>>>0>255){y=-s>>31&255}else{y=s&255}s=n<<1;a[j+s|0]=y;r=x>>19;if(r>>>0>255){z=-r>>31&255}else{z=r&255}a[j+(s|1)|0]=z;n=n+1|0;}while((n|0)<(k|0));return}else{if(!m){return}m=(g|0)>0;n=0;do{z=d[l+(n&7)|0]<<12;x=d[l+(n+3&7)|0]<<12;if(m){y=z;w=x;e=0;while(1){s=b[f+(e<<1)>>1]|0;r=(ac(s,b[(c[h+(e<<2)>>2]|0)+(n<<1)>>1]|0)|0)+y|0;q=(ac(b[(c[i+(e<<2)>>2]|0)+(n<<1)>>1]|0,s)|0)+w|0;s=e+1|0;if((s|0)<(g|0)){y=r;w=q;e=s}else{A=r;B=q;break}}}else{A=z;B=x}e=B>>19;if(e>>>0>255){C=-e>>31&255}else{C=e&255}e=n<<1;a[j+e|0]=C;w=A>>19;if(w>>>0>255){D=-w>>31&255}else{D=w&255}a[j+(e|1)|0]=D;n=n+1|0;}while((n|0)<(k|0));return}}function tQ(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0;if((n|0)<=0){p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}o=(g|0)>0;z=(k|0)>0;A=d+11284|0;B=d+11288|0;C=d+11292|0;D=d+11296|0;E=d+11300|0;F=d+11304|0;G=m;m=0;while(1){if(o){H=0;I=512;do{I=(ac(b[e+(H<<1)>>1]|0,b[(c[f+(H<<2)>>2]|0)+(m<<1)>>1]|0)|0)+I|0;H=H+1|0;}while((H|0)<(g|0));J=I>>10}else{J=0}if(z){H=0;K=-67108352;L=-67108352;do{M=b[h+(H<<1)>>1]|0;K=(ac(M,b[(c[i+(H<<2)>>2]|0)+(m<<1)>>1]|0)|0)+K|0;L=(ac(b[(c[j+(H<<2)>>2]|0)+(m<<1)>>1]|0,M)|0)+L|0;H=H+1|0;}while((H|0)<(k|0));N=K>>10;O=L>>10}else{N=-65536;O=-65536}do{if(o){H=262144;I=0;do{H=(ac(b[e+(I<<1)>>1]|0,b[(c[l+(I<<2)>>2]|0)+(m<<1)>>1]|0)|0)+H|0;I=I+1|0;}while((I|0)<(g|0));I=H>>19;if(!((I&256|0)!=0&I>>>0>255)){P=I&255;break}P=-I>>31&255}else{P=0}}while(0);L=(ac(J-(c[A>>2]|0)|0,c[B>>2]|0)|0)+2097152|0;K=L+(ac(c[C>>2]|0,O)|0)|0;I=(ac(c[D>>2]|0,O)|0)+L|0;M=I+(ac(c[E>>2]|0,N)|0)|0;I=(ac(c[F>>2]|0,N)|0)+L|0;L=I>>>22&255;Q=M>>>22&255;R=K>>>22&255;do{if((M|K|I)>>>0>1073741823){if(K>>>0>1073741823){S=-K>>31>>>22&255}else{S=R}if(M>>>0>1073741823){T=-M>>31>>>22&255}else{T=Q}if(I>>>0<=1073741823){U=S;V=T;W=L;break}U=S;V=T;W=-I>>31>>>22&255}else{U=R;V=Q;W=L}}while(0);a[G]=U;a[G+1|0]=V;a[G+2|0]=W;a[G+3|0]=P;L=m+1|0;if((L|0)<(n|0)){G=G+4|0;m=L}else{p=n;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}function tR(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;m=c[e>>2]|0;n=c[e+4>>2]|0;e=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=c[h>>2]|0;q=c[h+4>>2]|0;h=4096-k|0;r=4096-l|0;if((j|0)<=0){s=0;t=d+11200|0;u=c[t>>2]|0;v=u+(s<<2)|0;c[v>>2]=0;w=d+11204|0;x=c[w>>2]|0;y=x+(s<<2)|0;c[y>>2]=0;z=d+11208|0;A=c[z>>2]|0;B=A+(s<<2)|0;c[B>>2]=0;return}C=d+11284|0;D=d+11288|0;E=d+11292|0;F=d+11296|0;G=d+11300|0;H=d+11304|0;I=i;i=0;while(1){J=ac(b[m+(i<<1)>>1]|0,h)|0;K=(ac(b[n+(i<<1)>>1]|0,k)|0)+J>>10;J=ac(b[e+(i<<1)>>1]|0,r)|0;L=J-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;J=ac(b[f+(i<<1)>>1]|0,r)|0;M=J-67108864+(ac(b[p+(i<<1)>>1]|0,l)|0)>>10;J=ac(b[g+(i<<1)>>1]|0,h)|0;N=J+262144+(ac(b[q+(i<<1)>>1]|0,k)|0)>>19;if((N&256|0)!=0&N>>>0>255){O=-N>>31&255}else{O=N&255}N=(ac(K-(c[C>>2]|0)|0,c[D>>2]|0)|0)+2097152|0;K=N+(ac(c[E>>2]|0,M)|0)|0;J=(ac(c[F>>2]|0,M)|0)+N|0;M=J+(ac(c[G>>2]|0,L)|0)|0;J=(ac(c[H>>2]|0,L)|0)+N|0;N=J>>>22&255;L=M>>>22&255;P=K>>>22&255;do{if((M|K|J)>>>0>1073741823){if(K>>>0>1073741823){Q=-K>>31>>>22&255}else{Q=P}if(M>>>0>1073741823){R=-M>>31>>>22&255}else{R=L}if(J>>>0<=1073741823){S=Q;T=R;U=N;break}S=Q;T=R;U=-J>>31>>>22&255}else{S=P;T=L;U=N}}while(0);a[I]=S;a[I+1|0]=T;a[I+2|0]=U;a[I+3|0]=O;N=i+1|0;if((N|0)<(j|0)){I=I+4|0;i=N}else{s=j;break}}t=d+11200|0;u=c[t>>2]|0;v=u+(s<<2)|0;c[v>>2]=0;w=d+11204|0;x=c[w>>2]|0;y=x+(s<<2)|0;c[y>>2]=0;z=d+11208|0;A=c[z>>2]|0;B=A+(s<<2)|0;c[B>>2]=0;return}function tS(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;l=c[f>>2]|0;m=c[g>>2]|0;if((k|0)<2048){if((j|0)<=0){n=0;o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}k=d+11284|0;x=d+11288|0;y=d+11292|0;z=d+11296|0;A=d+11300|0;B=d+11304|0;C=i;D=0;while(1){E=(b[l+(D<<1)>>1]<<2)-65536|0;F=(b[m+(D<<1)>>1]<<2)-65536|0;G=(b[h+(D<<1)>>1]|0)+64>>7;if((G&256|0)!=0&G>>>0>255){H=-G>>31&255}else{H=G&255}G=(ac((b[e+(D<<1)>>1]<<2)-(c[k>>2]|0)|0,c[x>>2]|0)|0)+2097152|0;I=G+(ac(c[y>>2]|0,F)|0)|0;J=(ac(c[z>>2]|0,F)|0)+G|0;F=J+(ac(c[A>>2]|0,E)|0)|0;J=(ac(c[B>>2]|0,E)|0)+G|0;G=J>>>22&255;E=F>>>22&255;K=I>>>22&255;do{if((F|I|J)>>>0>1073741823){if(I>>>0>1073741823){L=-I>>31>>>22&255}else{L=K}if(F>>>0>1073741823){M=-F>>31>>>22&255}else{M=E}if(J>>>0<=1073741823){N=L;O=M;P=G;break}N=L;O=M;P=-J>>31>>>22&255}else{N=K;O=E;P=G}}while(0);a[C]=N;a[C+1|0]=O;a[C+2|0]=P;a[C+3|0]=H;G=D+1|0;if((G|0)<(j|0)){C=C+4|0;D=G}else{n=j;break}}o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}else{D=c[f+4>>2]|0;f=c[g+4>>2]|0;if((j|0)<=0){n=0;o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}g=d+11284|0;C=d+11288|0;H=d+11292|0;P=d+11296|0;O=d+11300|0;N=d+11304|0;M=i;i=0;while(1){L=((b[D+(i<<1)>>1]|0)+(b[l+(i<<1)>>1]|0)<<1)-65536|0;B=((b[f+(i<<1)>>1]|0)+(b[m+(i<<1)>>1]|0)<<1)-65536|0;A=(b[h+(i<<1)>>1]|0)+64>>7;if((A&256|0)!=0&A>>>0>255){Q=-A>>31&255}else{Q=A&255}A=(ac((b[e+(i<<1)>>1]<<2)-(c[g>>2]|0)|0,c[C>>2]|0)|0)+2097152|0;z=A+(ac(c[H>>2]|0,B)|0)|0;y=(ac(c[P>>2]|0,B)|0)+A|0;B=y+(ac(c[O>>2]|0,L)|0)|0;y=(ac(c[N>>2]|0,L)|0)+A|0;A=y>>>22&255;L=B>>>22&255;x=z>>>22&255;do{if((B|z|y)>>>0>1073741823){if(z>>>0>1073741823){R=-z>>31>>>22&255}else{R=x}if(B>>>0>1073741823){S=-B>>31>>>22&255}else{S=L}if(y>>>0<=1073741823){T=R;U=S;V=A;break}T=R;U=S;V=-y>>31>>>22&255}else{T=x;U=L;V=A}}while(0);a[M]=T;a[M+1|0]=U;a[M+2|0]=V;a[M+3|0]=Q;A=i+1|0;if((A|0)<(j|0)){M=M+4|0;i=A}else{n=j;break}}o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}}function tT(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;if((n|0)<=0){p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}o=(g|0)>0;l=(k|0)>0;z=d+11284|0;A=d+11288|0;B=d+11292|0;C=d+11296|0;D=d+11300|0;E=d+11304|0;F=m;m=0;while(1){if(o){G=0;H=512;do{H=(ac(b[e+(G<<1)>>1]|0,b[(c[f+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+H|0;G=G+1|0;}while((G|0)<(g|0));I=H>>10}else{I=0}if(l){G=0;J=-67108352;K=-67108352;do{L=b[h+(G<<1)>>1]|0;J=(ac(L,b[(c[i+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+J|0;K=(ac(b[(c[j+(G<<2)>>2]|0)+(m<<1)>>1]|0,L)|0)+K|0;G=G+1|0;}while((G|0)<(k|0));M=J>>10;N=K>>10}else{M=-65536;N=-65536}G=(ac(I-(c[z>>2]|0)|0,c[A>>2]|0)|0)+2097152|0;H=G+(ac(c[B>>2]|0,N)|0)|0;L=(ac(c[C>>2]|0,N)|0)+G|0;O=L+(ac(c[D>>2]|0,M)|0)|0;L=(ac(c[E>>2]|0,M)|0)+G|0;G=L>>>22&255;P=O>>>22&255;Q=H>>>22&255;do{if((O|H|L)>>>0>1073741823){if(H>>>0>1073741823){R=-H>>31>>>22&255}else{R=Q}if(O>>>0>1073741823){S=-O>>31>>>22&255}else{S=P}if(L>>>0<=1073741823){T=R;U=S;V=G;break}T=R;U=S;V=-L>>31>>>22&255}else{T=Q;U=P;V=G}}while(0);a[F]=T;a[F+1|0]=U;a[F+2|0]=V;a[F+3|0]=-1;G=m+1|0;if((G|0)<(n|0)){F=F+4|0;m=G}else{p=n;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}function tU(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;if((j|0)<=0){q=0;r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}A=d+11284|0;B=d+11288|0;C=d+11292|0;D=d+11296|0;E=d+11300|0;F=d+11304|0;G=i;i=0;while(1){H=ac(b[m+(i<<1)>>1]|0,g)|0;I=(ac(b[h+(i<<1)>>1]|0,k)|0)+H>>10;H=ac(b[e+(i<<1)>>1]|0,p)|0;J=H-67108864+(ac(b[n+(i<<1)>>1]|0,l)|0)>>10;H=ac(b[f+(i<<1)>>1]|0,p)|0;K=H-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;H=(ac(I-(c[A>>2]|0)|0,c[B>>2]|0)|0)+2097152|0;I=(ac(K,c[C>>2]|0)|0)+H|0;L=(ac(K,c[D>>2]|0)|0)+H|0;K=L+(ac(c[E>>2]|0,J)|0)|0;L=(ac(c[F>>2]|0,J)|0)+H|0;H=L>>>22&255;J=K>>>22&255;M=I>>>22&255;do{if((K|I|L)>>>0>1073741823){if(I>>>0>1073741823){N=-I>>31>>>22&255}else{N=M}if(K>>>0>1073741823){O=-K>>31>>>22&255}else{O=J}if(L>>>0<=1073741823){P=N;Q=O;R=H;break}P=N;Q=O;R=-L>>31>>>22&255}else{P=M;Q=J;R=H}}while(0);a[G]=P;a[G+1|0]=Q;a[G+2|0]=R;a[G+3|0]=-1;H=i+1|0;if((H|0)<(j|0)){G=G+4|0;i=H}else{q=j;break}}r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}function tV(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;l=c[f>>2]|0;h=c[g>>2]|0;do{if((k|0)<2048){if((j|0)<=0){m=0;break}n=d+11284|0;o=d+11288|0;p=d+11292|0;q=d+11296|0;r=d+11300|0;s=d+11304|0;t=i;u=0;while(1){v=(b[l+(u<<1)>>1]<<2)-65536|0;w=(b[h+(u<<1)>>1]<<2)-65536|0;x=(ac((b[e+(u<<1)>>1]<<2)-(c[n>>2]|0)|0,c[o>>2]|0)|0)+2097152|0;y=x+(ac(w,c[p>>2]|0)|0)|0;z=(ac(c[q>>2]|0,w)|0)+x|0;w=z+(ac(c[r>>2]|0,v)|0)|0;z=(ac(c[s>>2]|0,v)|0)+x|0;x=z>>>22&255;v=w>>>22&255;A=y>>>22&255;do{if((w|y|z)>>>0>1073741823){if(y>>>0>1073741823){B=-y>>31>>>22&255}else{B=A}if(w>>>0>1073741823){C=-w>>31>>>22&255}else{C=v}if(z>>>0<=1073741823){D=B;E=C;F=x;break}D=B;E=C;F=-z>>31>>>22&255}else{D=A;E=v;F=x}}while(0);a[t]=D;a[t+1|0]=E;a[t+2|0]=F;a[t+3|0]=-1;x=u+1|0;if((x|0)<(j|0)){t=t+4|0;u=x}else{m=j;break}}}else{u=c[f+4>>2]|0;t=c[g+4>>2]|0;if((j|0)<=0){m=0;break}s=d+11284|0;r=d+11288|0;q=d+11292|0;p=d+11296|0;o=d+11300|0;n=d+11304|0;x=i;v=0;while(1){A=((b[u+(v<<1)>>1]|0)+(b[l+(v<<1)>>1]|0)<<1)-65536|0;z=((b[t+(v<<1)>>1]|0)+(b[h+(v<<1)>>1]|0)<<1)-65536|0;w=(ac((b[e+(v<<1)>>1]<<2)-(c[s>>2]|0)|0,c[r>>2]|0)|0)+2097152|0;y=(ac(z,c[q>>2]|0)|0)+w|0;G=(ac(z,c[p>>2]|0)|0)+w|0;z=G+(ac(c[o>>2]|0,A)|0)|0;G=(ac(c[n>>2]|0,A)|0)+w|0;w=G>>>22&255;A=z>>>22&255;H=y>>>22&255;do{if((z|y|G)>>>0>1073741823){if(y>>>0>1073741823){I=-y>>31>>>22&255}else{I=H}if(z>>>0>1073741823){J=-z>>31>>>22&255}else{J=A}if(G>>>0<=1073741823){K=I;L=J;M=w;break}K=I;L=J;M=-G>>31>>>22&255}else{K=H;L=A;M=w}}while(0);a[x]=K;a[x+1|0]=L;a[x+2|0]=M;a[x+3|0]=-1;w=v+1|0;if((w|0)<(j|0)){x=x+4|0;v=w}else{m=j;break}}}}while(0);c[(c[d+11200>>2]|0)+(m<<2)>>2]=0;c[(c[d+11204>>2]|0)+(m<<2)>>2]=0;c[(c[d+11208>>2]|0)+(m<<2)>>2]=0;return}function tW(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0;if((n|0)<=0){p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}o=(g|0)>0;z=(k|0)>0;A=d+11284|0;B=d+11288|0;C=d+11292|0;D=d+11296|0;E=d+11300|0;F=d+11304|0;G=m;m=0;while(1){if(o){H=0;I=512;do{I=(ac(b[e+(H<<1)>>1]|0,b[(c[f+(H<<2)>>2]|0)+(m<<1)>>1]|0)|0)+I|0;H=H+1|0;}while((H|0)<(g|0));J=I>>10}else{J=0}if(z){H=0;K=-67108352;L=-67108352;do{M=b[h+(H<<1)>>1]|0;K=(ac(M,b[(c[i+(H<<2)>>2]|0)+(m<<1)>>1]|0)|0)+K|0;L=(ac(b[(c[j+(H<<2)>>2]|0)+(m<<1)>>1]|0,M)|0)+L|0;H=H+1|0;}while((H|0)<(k|0));N=K>>10;O=L>>10}else{N=-65536;O=-65536}do{if(o){H=262144;I=0;do{H=(ac(b[e+(I<<1)>>1]|0,b[(c[l+(I<<2)>>2]|0)+(m<<1)>>1]|0)|0)+H|0;I=I+1|0;}while((I|0)<(g|0));I=H>>19;if(!((I&256|0)!=0&I>>>0>255)){P=I&255;break}P=-I>>31&255}else{P=0}}while(0);L=(ac(J-(c[A>>2]|0)|0,c[B>>2]|0)|0)+2097152|0;K=L+(ac(c[C>>2]|0,O)|0)|0;I=(ac(c[D>>2]|0,O)|0)+L|0;M=I+(ac(c[E>>2]|0,N)|0)|0;I=(ac(c[F>>2]|0,N)|0)+L|0;L=I>>>22&255;Q=M>>>22&255;R=K>>>22&255;do{if((M|K|I)>>>0>1073741823){if(K>>>0>1073741823){S=-K>>31>>>22&255}else{S=R}if(M>>>0>1073741823){T=-M>>31>>>22&255}else{T=Q}if(I>>>0<=1073741823){U=S;V=T;W=L;break}U=S;V=T;W=-I>>31>>>22&255}else{U=R;V=Q;W=L}}while(0);a[G]=P;a[G+1|0]=U;a[G+2|0]=V;a[G+3|0]=W;L=m+1|0;if((L|0)<(n|0)){G=G+4|0;m=L}else{p=n;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}function tX(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;m=c[e>>2]|0;n=c[e+4>>2]|0;e=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=c[h>>2]|0;q=c[h+4>>2]|0;h=4096-k|0;r=4096-l|0;if((j|0)<=0){s=0;t=d+11200|0;u=c[t>>2]|0;v=u+(s<<2)|0;c[v>>2]=0;w=d+11204|0;x=c[w>>2]|0;y=x+(s<<2)|0;c[y>>2]=0;z=d+11208|0;A=c[z>>2]|0;B=A+(s<<2)|0;c[B>>2]=0;return}C=d+11284|0;D=d+11288|0;E=d+11292|0;F=d+11296|0;G=d+11300|0;H=d+11304|0;I=i;i=0;while(1){J=ac(b[m+(i<<1)>>1]|0,h)|0;K=(ac(b[n+(i<<1)>>1]|0,k)|0)+J>>10;J=ac(b[e+(i<<1)>>1]|0,r)|0;L=J-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;J=ac(b[f+(i<<1)>>1]|0,r)|0;M=J-67108864+(ac(b[p+(i<<1)>>1]|0,l)|0)>>10;J=ac(b[g+(i<<1)>>1]|0,h)|0;N=J+262144+(ac(b[q+(i<<1)>>1]|0,k)|0)>>19;if((N&256|0)!=0&N>>>0>255){O=-N>>31&255}else{O=N&255}N=(ac(K-(c[C>>2]|0)|0,c[D>>2]|0)|0)+2097152|0;K=N+(ac(c[E>>2]|0,M)|0)|0;J=(ac(c[F>>2]|0,M)|0)+N|0;M=J+(ac(c[G>>2]|0,L)|0)|0;J=(ac(c[H>>2]|0,L)|0)+N|0;N=J>>>22&255;L=M>>>22&255;P=K>>>22&255;do{if((M|K|J)>>>0>1073741823){if(K>>>0>1073741823){Q=-K>>31>>>22&255}else{Q=P}if(M>>>0>1073741823){R=-M>>31>>>22&255}else{R=L}if(J>>>0<=1073741823){S=Q;T=R;U=N;break}S=Q;T=R;U=-J>>31>>>22&255}else{S=P;T=L;U=N}}while(0);a[I]=O;a[I+1|0]=S;a[I+2|0]=T;a[I+3|0]=U;N=i+1|0;if((N|0)<(j|0)){I=I+4|0;i=N}else{s=j;break}}t=d+11200|0;u=c[t>>2]|0;v=u+(s<<2)|0;c[v>>2]=0;w=d+11204|0;x=c[w>>2]|0;y=x+(s<<2)|0;c[y>>2]=0;z=d+11208|0;A=c[z>>2]|0;B=A+(s<<2)|0;c[B>>2]=0;return}function tY(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;l=c[f>>2]|0;m=c[g>>2]|0;if((k|0)<2048){if((j|0)<=0){n=0;o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}k=d+11284|0;x=d+11288|0;y=d+11292|0;z=d+11296|0;A=d+11300|0;B=d+11304|0;C=i;D=0;while(1){E=(b[l+(D<<1)>>1]<<2)-65536|0;F=(b[m+(D<<1)>>1]<<2)-65536|0;G=(b[h+(D<<1)>>1]|0)+64>>7;if((G&256|0)!=0&G>>>0>255){H=-G>>31&255}else{H=G&255}G=(ac((b[e+(D<<1)>>1]<<2)-(c[k>>2]|0)|0,c[x>>2]|0)|0)+2097152|0;I=G+(ac(c[y>>2]|0,F)|0)|0;J=(ac(c[z>>2]|0,F)|0)+G|0;F=J+(ac(c[A>>2]|0,E)|0)|0;J=(ac(c[B>>2]|0,E)|0)+G|0;G=J>>>22&255;E=F>>>22&255;K=I>>>22&255;do{if((F|I|J)>>>0>1073741823){if(I>>>0>1073741823){L=-I>>31>>>22&255}else{L=K}if(F>>>0>1073741823){M=-F>>31>>>22&255}else{M=E}if(J>>>0<=1073741823){N=L;O=M;P=G;break}N=L;O=M;P=-J>>31>>>22&255}else{N=K;O=E;P=G}}while(0);a[C]=H;a[C+1|0]=N;a[C+2|0]=O;a[C+3|0]=P;G=D+1|0;if((G|0)<(j|0)){C=C+4|0;D=G}else{n=j;break}}o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}else{D=c[f+4>>2]|0;f=c[g+4>>2]|0;if((j|0)<=0){n=0;o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}g=d+11284|0;C=d+11288|0;P=d+11292|0;O=d+11296|0;N=d+11300|0;H=d+11304|0;M=i;i=0;while(1){L=((b[D+(i<<1)>>1]|0)+(b[l+(i<<1)>>1]|0)<<1)-65536|0;B=((b[f+(i<<1)>>1]|0)+(b[m+(i<<1)>>1]|0)<<1)-65536|0;A=(b[h+(i<<1)>>1]|0)+64>>7;if((A&256|0)!=0&A>>>0>255){Q=-A>>31&255}else{Q=A&255}A=(ac((b[e+(i<<1)>>1]<<2)-(c[g>>2]|0)|0,c[C>>2]|0)|0)+2097152|0;z=A+(ac(c[P>>2]|0,B)|0)|0;y=(ac(c[O>>2]|0,B)|0)+A|0;B=y+(ac(c[N>>2]|0,L)|0)|0;y=(ac(c[H>>2]|0,L)|0)+A|0;A=y>>>22&255;L=B>>>22&255;x=z>>>22&255;do{if((B|z|y)>>>0>1073741823){if(z>>>0>1073741823){R=-z>>31>>>22&255}else{R=x}if(B>>>0>1073741823){S=-B>>31>>>22&255}else{S=L}if(y>>>0<=1073741823){T=R;U=S;V=A;break}T=R;U=S;V=-y>>31>>>22&255}else{T=x;U=L;V=A}}while(0);a[M]=Q;a[M+1|0]=T;a[M+2|0]=U;a[M+3|0]=V;A=i+1|0;if((A|0)<(j|0)){M=M+4|0;i=A}else{n=j;break}}o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}}function tZ(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;if((n|0)<=0){p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}o=(g|0)>0;l=(k|0)>0;z=d+11284|0;A=d+11288|0;B=d+11292|0;C=d+11296|0;D=d+11300|0;E=d+11304|0;F=m;m=0;while(1){if(o){G=0;H=512;do{H=(ac(b[e+(G<<1)>>1]|0,b[(c[f+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+H|0;G=G+1|0;}while((G|0)<(g|0));I=H>>10}else{I=0}if(l){G=0;J=-67108352;K=-67108352;do{L=b[h+(G<<1)>>1]|0;J=(ac(L,b[(c[i+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+J|0;K=(ac(b[(c[j+(G<<2)>>2]|0)+(m<<1)>>1]|0,L)|0)+K|0;G=G+1|0;}while((G|0)<(k|0));M=J>>10;N=K>>10}else{M=-65536;N=-65536}G=(ac(I-(c[z>>2]|0)|0,c[A>>2]|0)|0)+2097152|0;H=G+(ac(c[B>>2]|0,N)|0)|0;L=(ac(c[C>>2]|0,N)|0)+G|0;O=L+(ac(c[D>>2]|0,M)|0)|0;L=(ac(c[E>>2]|0,M)|0)+G|0;G=L>>>22&255;P=O>>>22&255;Q=H>>>22&255;do{if((O|H|L)>>>0>1073741823){if(H>>>0>1073741823){R=-H>>31>>>22&255}else{R=Q}if(O>>>0>1073741823){S=-O>>31>>>22&255}else{S=P}if(L>>>0<=1073741823){T=R;U=S;V=G;break}T=R;U=S;V=-L>>31>>>22&255}else{T=Q;U=P;V=G}}while(0);a[F]=-1;a[F+1|0]=T;a[F+2|0]=U;a[F+3|0]=V;G=m+1|0;if((G|0)<(n|0)){F=F+4|0;m=G}else{p=n;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}function t_(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;if((j|0)<=0){q=0;r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}A=d+11284|0;B=d+11288|0;C=d+11292|0;D=d+11296|0;E=d+11300|0;F=d+11304|0;G=i;i=0;while(1){H=ac(b[m+(i<<1)>>1]|0,g)|0;I=(ac(b[h+(i<<1)>>1]|0,k)|0)+H>>10;H=ac(b[e+(i<<1)>>1]|0,p)|0;J=H-67108864+(ac(b[n+(i<<1)>>1]|0,l)|0)>>10;H=ac(b[f+(i<<1)>>1]|0,p)|0;K=H-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;H=(ac(I-(c[A>>2]|0)|0,c[B>>2]|0)|0)+2097152|0;I=(ac(K,c[C>>2]|0)|0)+H|0;L=(ac(K,c[D>>2]|0)|0)+H|0;K=L+(ac(c[E>>2]|0,J)|0)|0;L=(ac(c[F>>2]|0,J)|0)+H|0;H=L>>>22&255;J=K>>>22&255;M=I>>>22&255;do{if((K|I|L)>>>0>1073741823){if(I>>>0>1073741823){N=-I>>31>>>22&255}else{N=M}if(K>>>0>1073741823){O=-K>>31>>>22&255}else{O=J}if(L>>>0<=1073741823){P=N;Q=O;R=H;break}P=N;Q=O;R=-L>>31>>>22&255}else{P=M;Q=J;R=H}}while(0);a[G]=-1;a[G+1|0]=P;a[G+2|0]=Q;a[G+3|0]=R;H=i+1|0;if((H|0)<(j|0)){G=G+4|0;i=H}else{q=j;break}}r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}function t$(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;l=c[f>>2]|0;h=c[g>>2]|0;do{if((k|0)<2048){if((j|0)<=0){m=0;break}n=d+11284|0;o=d+11288|0;p=d+11292|0;q=d+11296|0;r=d+11300|0;s=d+11304|0;t=i;u=0;while(1){v=(b[l+(u<<1)>>1]<<2)-65536|0;w=(b[h+(u<<1)>>1]<<2)-65536|0;x=(ac((b[e+(u<<1)>>1]<<2)-(c[n>>2]|0)|0,c[o>>2]|0)|0)+2097152|0;y=x+(ac(w,c[p>>2]|0)|0)|0;z=(ac(c[q>>2]|0,w)|0)+x|0;w=z+(ac(c[r>>2]|0,v)|0)|0;z=(ac(c[s>>2]|0,v)|0)+x|0;x=z>>>22&255;v=w>>>22&255;A=y>>>22&255;do{if((w|y|z)>>>0>1073741823){if(y>>>0>1073741823){B=-y>>31>>>22&255}else{B=A}if(w>>>0>1073741823){C=-w>>31>>>22&255}else{C=v}if(z>>>0<=1073741823){D=B;E=C;F=x;break}D=B;E=C;F=-z>>31>>>22&255}else{D=A;E=v;F=x}}while(0);a[t]=-1;a[t+1|0]=D;a[t+2|0]=E;a[t+3|0]=F;x=u+1|0;if((x|0)<(j|0)){t=t+4|0;u=x}else{m=j;break}}}else{u=c[f+4>>2]|0;t=c[g+4>>2]|0;if((j|0)<=0){m=0;break}s=d+11284|0;r=d+11288|0;q=d+11292|0;p=d+11296|0;o=d+11300|0;n=d+11304|0;x=i;v=0;while(1){A=((b[u+(v<<1)>>1]|0)+(b[l+(v<<1)>>1]|0)<<1)-65536|0;z=((b[t+(v<<1)>>1]|0)+(b[h+(v<<1)>>1]|0)<<1)-65536|0;w=(ac((b[e+(v<<1)>>1]<<2)-(c[s>>2]|0)|0,c[r>>2]|0)|0)+2097152|0;y=(ac(z,c[q>>2]|0)|0)+w|0;G=(ac(z,c[p>>2]|0)|0)+w|0;z=G+(ac(c[o>>2]|0,A)|0)|0;G=(ac(c[n>>2]|0,A)|0)+w|0;w=G>>>22&255;A=z>>>22&255;H=y>>>22&255;do{if((z|y|G)>>>0>1073741823){if(y>>>0>1073741823){I=-y>>31>>>22&255}else{I=H}if(z>>>0>1073741823){J=-z>>31>>>22&255}else{J=A}if(G>>>0<=1073741823){K=I;L=J;M=w;break}K=I;L=J;M=-G>>31>>>22&255}else{K=H;L=A;M=w}}while(0);a[x]=-1;a[x+1|0]=K;a[x+2|0]=L;a[x+3|0]=M;w=v+1|0;if((w|0)<(j|0)){x=x+4|0;v=w}else{m=j;break}}}}while(0);c[(c[d+11200>>2]|0)+(m<<2)>>2]=0;c[(c[d+11204>>2]|0)+(m<<2)>>2]=0;c[(c[d+11208>>2]|0)+(m<<2)>>2]=0;return}function t0(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0;if((n|0)<=0){p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}o=(g|0)>0;z=(k|0)>0;A=d+11284|0;B=d+11288|0;C=d+11292|0;D=d+11296|0;E=d+11300|0;F=d+11304|0;G=m;m=0;while(1){if(o){H=0;I=512;do{I=(ac(b[e+(H<<1)>>1]|0,b[(c[f+(H<<2)>>2]|0)+(m<<1)>>1]|0)|0)+I|0;H=H+1|0;}while((H|0)<(g|0));J=I>>10}else{J=0}if(z){H=0;K=-67108352;L=-67108352;do{M=b[h+(H<<1)>>1]|0;K=(ac(M,b[(c[i+(H<<2)>>2]|0)+(m<<1)>>1]|0)|0)+K|0;L=(ac(b[(c[j+(H<<2)>>2]|0)+(m<<1)>>1]|0,M)|0)+L|0;H=H+1|0;}while((H|0)<(k|0));N=K>>10;O=L>>10}else{N=-65536;O=-65536}do{if(o){H=262144;I=0;do{H=(ac(b[e+(I<<1)>>1]|0,b[(c[l+(I<<2)>>2]|0)+(m<<1)>>1]|0)|0)+H|0;I=I+1|0;}while((I|0)<(g|0));I=H>>19;if(!((I&256|0)!=0&I>>>0>255)){P=I&255;break}P=-I>>31&255}else{P=0}}while(0);L=(ac(J-(c[A>>2]|0)|0,c[B>>2]|0)|0)+2097152|0;K=L+(ac(c[C>>2]|0,O)|0)|0;I=(ac(c[D>>2]|0,O)|0)+L|0;M=I+(ac(c[E>>2]|0,N)|0)|0;I=(ac(c[F>>2]|0,N)|0)+L|0;L=I>>>22&255;Q=M>>>22&255;R=K>>>22&255;do{if((M|K|I)>>>0>1073741823){if(K>>>0>1073741823){S=-K>>31>>>22&255}else{S=R}if(M>>>0>1073741823){T=-M>>31>>>22&255}else{T=Q}if(I>>>0<=1073741823){U=S;V=T;W=L;break}U=S;V=T;W=-I>>31>>>22&255}else{U=R;V=Q;W=L}}while(0);a[G]=W;a[G+1|0]=V;a[G+2|0]=U;a[G+3|0]=P;L=m+1|0;if((L|0)<(n|0)){G=G+4|0;m=L}else{p=n;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}function t1(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;m=c[e>>2]|0;n=c[e+4>>2]|0;e=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=c[h>>2]|0;q=c[h+4>>2]|0;h=4096-k|0;r=4096-l|0;if((j|0)<=0){s=0;t=d+11200|0;u=c[t>>2]|0;v=u+(s<<2)|0;c[v>>2]=0;w=d+11204|0;x=c[w>>2]|0;y=x+(s<<2)|0;c[y>>2]=0;z=d+11208|0;A=c[z>>2]|0;B=A+(s<<2)|0;c[B>>2]=0;return}C=d+11284|0;D=d+11288|0;E=d+11292|0;F=d+11296|0;G=d+11300|0;H=d+11304|0;I=i;i=0;while(1){J=ac(b[m+(i<<1)>>1]|0,h)|0;K=(ac(b[n+(i<<1)>>1]|0,k)|0)+J>>10;J=ac(b[e+(i<<1)>>1]|0,r)|0;L=J-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;J=ac(b[f+(i<<1)>>1]|0,r)|0;M=J-67108864+(ac(b[p+(i<<1)>>1]|0,l)|0)>>10;J=ac(b[g+(i<<1)>>1]|0,h)|0;N=J+262144+(ac(b[q+(i<<1)>>1]|0,k)|0)>>19;if((N&256|0)!=0&N>>>0>255){O=-N>>31&255}else{O=N&255}N=(ac(K-(c[C>>2]|0)|0,c[D>>2]|0)|0)+2097152|0;K=N+(ac(c[E>>2]|0,M)|0)|0;J=(ac(c[F>>2]|0,M)|0)+N|0;M=J+(ac(c[G>>2]|0,L)|0)|0;J=(ac(c[H>>2]|0,L)|0)+N|0;N=J>>>22&255;L=M>>>22&255;P=K>>>22&255;do{if((M|K|J)>>>0>1073741823){if(K>>>0>1073741823){Q=-K>>31>>>22&255}else{Q=P}if(M>>>0>1073741823){R=-M>>31>>>22&255}else{R=L}if(J>>>0<=1073741823){S=Q;T=R;U=N;break}S=Q;T=R;U=-J>>31>>>22&255}else{S=P;T=L;U=N}}while(0);a[I]=U;a[I+1|0]=T;a[I+2|0]=S;a[I+3|0]=O;N=i+1|0;if((N|0)<(j|0)){I=I+4|0;i=N}else{s=j;break}}t=d+11200|0;u=c[t>>2]|0;v=u+(s<<2)|0;c[v>>2]=0;w=d+11204|0;x=c[w>>2]|0;y=x+(s<<2)|0;c[y>>2]=0;z=d+11208|0;A=c[z>>2]|0;B=A+(s<<2)|0;c[B>>2]=0;return}function t2(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;l=c[f>>2]|0;m=c[g>>2]|0;if((k|0)<2048){if((j|0)<=0){n=0;o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}k=d+11284|0;x=d+11288|0;y=d+11292|0;z=d+11296|0;A=d+11300|0;B=d+11304|0;C=i;D=0;while(1){E=(b[l+(D<<1)>>1]<<2)-65536|0;F=(b[m+(D<<1)>>1]<<2)-65536|0;G=(b[h+(D<<1)>>1]|0)+64>>7;if((G&256|0)!=0&G>>>0>255){H=-G>>31&255}else{H=G&255}G=(ac((b[e+(D<<1)>>1]<<2)-(c[k>>2]|0)|0,c[x>>2]|0)|0)+2097152|0;I=G+(ac(c[y>>2]|0,F)|0)|0;J=(ac(c[z>>2]|0,F)|0)+G|0;F=J+(ac(c[A>>2]|0,E)|0)|0;J=(ac(c[B>>2]|0,E)|0)+G|0;G=J>>>22&255;E=F>>>22&255;K=I>>>22&255;do{if((F|I|J)>>>0>1073741823){if(I>>>0>1073741823){L=-I>>31>>>22&255}else{L=K}if(F>>>0>1073741823){M=-F>>31>>>22&255}else{M=E}if(J>>>0<=1073741823){N=L;O=M;P=G;break}N=L;O=M;P=-J>>31>>>22&255}else{N=K;O=E;P=G}}while(0);a[C]=P;a[C+1|0]=O;a[C+2|0]=N;a[C+3|0]=H;G=D+1|0;if((G|0)<(j|0)){C=C+4|0;D=G}else{n=j;break}}o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}else{D=c[f+4>>2]|0;f=c[g+4>>2]|0;if((j|0)<=0){n=0;o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}g=d+11284|0;C=d+11288|0;H=d+11292|0;N=d+11296|0;O=d+11300|0;P=d+11304|0;M=i;i=0;while(1){L=((b[D+(i<<1)>>1]|0)+(b[l+(i<<1)>>1]|0)<<1)-65536|0;B=((b[f+(i<<1)>>1]|0)+(b[m+(i<<1)>>1]|0)<<1)-65536|0;A=(b[h+(i<<1)>>1]|0)+64>>7;if((A&256|0)!=0&A>>>0>255){Q=-A>>31&255}else{Q=A&255}A=(ac((b[e+(i<<1)>>1]<<2)-(c[g>>2]|0)|0,c[C>>2]|0)|0)+2097152|0;z=A+(ac(c[H>>2]|0,B)|0)|0;y=(ac(c[N>>2]|0,B)|0)+A|0;B=y+(ac(c[O>>2]|0,L)|0)|0;y=(ac(c[P>>2]|0,L)|0)+A|0;A=y>>>22&255;L=B>>>22&255;x=z>>>22&255;do{if((B|z|y)>>>0>1073741823){if(z>>>0>1073741823){R=-z>>31>>>22&255}else{R=x}if(B>>>0>1073741823){S=-B>>31>>>22&255}else{S=L}if(y>>>0<=1073741823){T=R;U=S;V=A;break}T=R;U=S;V=-y>>31>>>22&255}else{T=x;U=L;V=A}}while(0);a[M]=V;a[M+1|0]=U;a[M+2|0]=T;a[M+3|0]=Q;A=i+1|0;if((A|0)<(j|0)){M=M+4|0;i=A}else{n=j;break}}o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}}function t3(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;if((n|0)<=0){p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}o=(g|0)>0;l=(k|0)>0;z=d+11284|0;A=d+11288|0;B=d+11292|0;C=d+11296|0;D=d+11300|0;E=d+11304|0;F=m;m=0;while(1){if(o){G=0;H=512;do{H=(ac(b[e+(G<<1)>>1]|0,b[(c[f+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+H|0;G=G+1|0;}while((G|0)<(g|0));I=H>>10}else{I=0}if(l){G=0;J=-67108352;K=-67108352;do{L=b[h+(G<<1)>>1]|0;J=(ac(L,b[(c[i+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+J|0;K=(ac(b[(c[j+(G<<2)>>2]|0)+(m<<1)>>1]|0,L)|0)+K|0;G=G+1|0;}while((G|0)<(k|0));M=J>>10;N=K>>10}else{M=-65536;N=-65536}G=(ac(I-(c[z>>2]|0)|0,c[A>>2]|0)|0)+2097152|0;H=G+(ac(c[B>>2]|0,N)|0)|0;L=(ac(c[C>>2]|0,N)|0)+G|0;O=L+(ac(c[D>>2]|0,M)|0)|0;L=(ac(c[E>>2]|0,M)|0)+G|0;G=L>>>22&255;P=O>>>22&255;Q=H>>>22&255;do{if((O|H|L)>>>0>1073741823){if(H>>>0>1073741823){R=-H>>31>>>22&255}else{R=Q}if(O>>>0>1073741823){S=-O>>31>>>22&255}else{S=P}if(L>>>0<=1073741823){T=R;U=S;V=G;break}T=R;U=S;V=-L>>31>>>22&255}else{T=Q;U=P;V=G}}while(0);a[F]=V;a[F+1|0]=U;a[F+2|0]=T;a[F+3|0]=-1;G=m+1|0;if((G|0)<(n|0)){F=F+4|0;m=G}else{p=n;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}function t4(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;if((j|0)<=0){q=0;r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}A=d+11284|0;B=d+11288|0;C=d+11292|0;D=d+11296|0;E=d+11300|0;F=d+11304|0;G=i;i=0;while(1){H=ac(b[m+(i<<1)>>1]|0,g)|0;I=(ac(b[h+(i<<1)>>1]|0,k)|0)+H>>10;H=ac(b[e+(i<<1)>>1]|0,p)|0;J=H-67108864+(ac(b[n+(i<<1)>>1]|0,l)|0)>>10;H=ac(b[f+(i<<1)>>1]|0,p)|0;K=H-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;H=(ac(I-(c[A>>2]|0)|0,c[B>>2]|0)|0)+2097152|0;I=(ac(K,c[C>>2]|0)|0)+H|0;L=(ac(K,c[D>>2]|0)|0)+H|0;K=L+(ac(c[E>>2]|0,J)|0)|0;L=(ac(c[F>>2]|0,J)|0)+H|0;H=L>>>22&255;J=K>>>22&255;M=I>>>22&255;do{if((K|I|L)>>>0>1073741823){if(I>>>0>1073741823){N=-I>>31>>>22&255}else{N=M}if(K>>>0>1073741823){O=-K>>31>>>22&255}else{O=J}if(L>>>0<=1073741823){P=N;Q=O;R=H;break}P=N;Q=O;R=-L>>31>>>22&255}else{P=M;Q=J;R=H}}while(0);a[G]=R;a[G+1|0]=Q;a[G+2|0]=P;a[G+3|0]=-1;H=i+1|0;if((H|0)<(j|0)){G=G+4|0;i=H}else{q=j;break}}r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}function t5(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;l=c[f>>2]|0;h=c[g>>2]|0;do{if((k|0)<2048){if((j|0)<=0){m=0;break}n=d+11284|0;o=d+11288|0;p=d+11292|0;q=d+11296|0;r=d+11300|0;s=d+11304|0;t=i;u=0;while(1){v=(b[l+(u<<1)>>1]<<2)-65536|0;w=(b[h+(u<<1)>>1]<<2)-65536|0;x=(ac((b[e+(u<<1)>>1]<<2)-(c[n>>2]|0)|0,c[o>>2]|0)|0)+2097152|0;y=x+(ac(w,c[p>>2]|0)|0)|0;z=(ac(c[q>>2]|0,w)|0)+x|0;w=z+(ac(c[r>>2]|0,v)|0)|0;z=(ac(c[s>>2]|0,v)|0)+x|0;x=z>>>22&255;v=w>>>22&255;A=y>>>22&255;do{if((w|y|z)>>>0>1073741823){if(y>>>0>1073741823){B=-y>>31>>>22&255}else{B=A}if(w>>>0>1073741823){C=-w>>31>>>22&255}else{C=v}if(z>>>0<=1073741823){D=B;E=C;F=x;break}D=B;E=C;F=-z>>31>>>22&255}else{D=A;E=v;F=x}}while(0);a[t]=F;a[t+1|0]=E;a[t+2|0]=D;a[t+3|0]=-1;x=u+1|0;if((x|0)<(j|0)){t=t+4|0;u=x}else{m=j;break}}}else{u=c[f+4>>2]|0;t=c[g+4>>2]|0;if((j|0)<=0){m=0;break}s=d+11284|0;r=d+11288|0;q=d+11292|0;p=d+11296|0;o=d+11300|0;n=d+11304|0;x=i;v=0;while(1){A=((b[u+(v<<1)>>1]|0)+(b[l+(v<<1)>>1]|0)<<1)-65536|0;z=((b[t+(v<<1)>>1]|0)+(b[h+(v<<1)>>1]|0)<<1)-65536|0;w=(ac((b[e+(v<<1)>>1]<<2)-(c[s>>2]|0)|0,c[r>>2]|0)|0)+2097152|0;y=(ac(z,c[q>>2]|0)|0)+w|0;G=(ac(z,c[p>>2]|0)|0)+w|0;z=G+(ac(c[o>>2]|0,A)|0)|0;G=(ac(c[n>>2]|0,A)|0)+w|0;w=G>>>22&255;A=z>>>22&255;H=y>>>22&255;do{if((z|y|G)>>>0>1073741823){if(y>>>0>1073741823){I=-y>>31>>>22&255}else{I=H}if(z>>>0>1073741823){J=-z>>31>>>22&255}else{J=A}if(G>>>0<=1073741823){K=I;L=J;M=w;break}K=I;L=J;M=-G>>31>>>22&255}else{K=H;L=A;M=w}}while(0);a[x]=M;a[x+1|0]=L;a[x+2|0]=K;a[x+3|0]=-1;w=v+1|0;if((w|0)<(j|0)){x=x+4|0;v=w}else{m=j;break}}}}while(0);c[(c[d+11200>>2]|0)+(m<<2)>>2]=0;c[(c[d+11204>>2]|0)+(m<<2)>>2]=0;c[(c[d+11208>>2]|0)+(m<<2)>>2]=0;return}function t6(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0;if((n|0)<=0){p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}o=(g|0)>0;z=(k|0)>0;A=d+11284|0;B=d+11288|0;C=d+11292|0;D=d+11296|0;E=d+11300|0;F=d+11304|0;G=m;m=0;while(1){if(o){H=0;I=512;do{I=(ac(b[e+(H<<1)>>1]|0,b[(c[f+(H<<2)>>2]|0)+(m<<1)>>1]|0)|0)+I|0;H=H+1|0;}while((H|0)<(g|0));J=I>>10}else{J=0}if(z){H=0;K=-67108352;L=-67108352;do{M=b[h+(H<<1)>>1]|0;K=(ac(M,b[(c[i+(H<<2)>>2]|0)+(m<<1)>>1]|0)|0)+K|0;L=(ac(b[(c[j+(H<<2)>>2]|0)+(m<<1)>>1]|0,M)|0)+L|0;H=H+1|0;}while((H|0)<(k|0));N=K>>10;O=L>>10}else{N=-65536;O=-65536}do{if(o){H=262144;I=0;do{H=(ac(b[e+(I<<1)>>1]|0,b[(c[l+(I<<2)>>2]|0)+(m<<1)>>1]|0)|0)+H|0;I=I+1|0;}while((I|0)<(g|0));I=H>>19;if(!((I&256|0)!=0&I>>>0>255)){P=I&255;break}P=-I>>31&255}else{P=0}}while(0);L=(ac(J-(c[A>>2]|0)|0,c[B>>2]|0)|0)+2097152|0;K=L+(ac(c[C>>2]|0,O)|0)|0;I=(ac(c[D>>2]|0,O)|0)+L|0;M=I+(ac(c[E>>2]|0,N)|0)|0;I=(ac(c[F>>2]|0,N)|0)+L|0;L=I>>>22&255;Q=M>>>22&255;R=K>>>22&255;do{if((M|K|I)>>>0>1073741823){if(K>>>0>1073741823){S=-K>>31>>>22&255}else{S=R}if(M>>>0>1073741823){T=-M>>31>>>22&255}else{T=Q}if(I>>>0<=1073741823){U=S;V=T;W=L;break}U=S;V=T;W=-I>>31>>>22&255}else{U=R;V=Q;W=L}}while(0);a[G]=P;a[G+1|0]=W;a[G+2|0]=V;a[G+3|0]=U;L=m+1|0;if((L|0)<(n|0)){G=G+4|0;m=L}else{p=n;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}function t7(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;m=c[e>>2]|0;n=c[e+4>>2]|0;e=c[f>>2]|0;o=c[f+4>>2]|0;f=c[g>>2]|0;p=c[g+4>>2]|0;g=c[h>>2]|0;q=c[h+4>>2]|0;h=4096-k|0;r=4096-l|0;if((j|0)<=0){s=0;t=d+11200|0;u=c[t>>2]|0;v=u+(s<<2)|0;c[v>>2]=0;w=d+11204|0;x=c[w>>2]|0;y=x+(s<<2)|0;c[y>>2]=0;z=d+11208|0;A=c[z>>2]|0;B=A+(s<<2)|0;c[B>>2]=0;return}C=d+11284|0;D=d+11288|0;E=d+11292|0;F=d+11296|0;G=d+11300|0;H=d+11304|0;I=i;i=0;while(1){J=ac(b[m+(i<<1)>>1]|0,h)|0;K=(ac(b[n+(i<<1)>>1]|0,k)|0)+J>>10;J=ac(b[e+(i<<1)>>1]|0,r)|0;L=J-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;J=ac(b[f+(i<<1)>>1]|0,r)|0;M=J-67108864+(ac(b[p+(i<<1)>>1]|0,l)|0)>>10;J=ac(b[g+(i<<1)>>1]|0,h)|0;N=J+262144+(ac(b[q+(i<<1)>>1]|0,k)|0)>>19;if((N&256|0)!=0&N>>>0>255){O=-N>>31&255}else{O=N&255}N=(ac(K-(c[C>>2]|0)|0,c[D>>2]|0)|0)+2097152|0;K=N+(ac(c[E>>2]|0,M)|0)|0;J=(ac(c[F>>2]|0,M)|0)+N|0;M=J+(ac(c[G>>2]|0,L)|0)|0;J=(ac(c[H>>2]|0,L)|0)+N|0;N=J>>>22&255;L=M>>>22&255;P=K>>>22&255;do{if((M|K|J)>>>0>1073741823){if(K>>>0>1073741823){Q=-K>>31>>>22&255}else{Q=P}if(M>>>0>1073741823){R=-M>>31>>>22&255}else{R=L}if(J>>>0<=1073741823){S=Q;T=R;U=N;break}S=Q;T=R;U=-J>>31>>>22&255}else{S=P;T=L;U=N}}while(0);a[I]=O;a[I+1|0]=U;a[I+2|0]=T;a[I+3|0]=S;N=i+1|0;if((N|0)<(j|0)){I=I+4|0;i=N}else{s=j;break}}t=d+11200|0;u=c[t>>2]|0;v=u+(s<<2)|0;c[v>>2]=0;w=d+11204|0;x=c[w>>2]|0;y=x+(s<<2)|0;c[y>>2]=0;z=d+11208|0;A=c[z>>2]|0;B=A+(s<<2)|0;c[B>>2]=0;return}function t8(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;l=c[f>>2]|0;m=c[g>>2]|0;if((k|0)<2048){if((j|0)<=0){n=0;o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}k=d+11284|0;x=d+11288|0;y=d+11292|0;z=d+11296|0;A=d+11300|0;B=d+11304|0;C=i;D=0;while(1){E=(b[l+(D<<1)>>1]<<2)-65536|0;F=(b[m+(D<<1)>>1]<<2)-65536|0;G=(b[h+(D<<1)>>1]|0)+64>>7;if((G&256|0)!=0&G>>>0>255){H=-G>>31&255}else{H=G&255}G=(ac((b[e+(D<<1)>>1]<<2)-(c[k>>2]|0)|0,c[x>>2]|0)|0)+2097152|0;I=G+(ac(c[y>>2]|0,F)|0)|0;J=(ac(c[z>>2]|0,F)|0)+G|0;F=J+(ac(c[A>>2]|0,E)|0)|0;J=(ac(c[B>>2]|0,E)|0)+G|0;G=J>>>22&255;E=F>>>22&255;K=I>>>22&255;do{if((F|I|J)>>>0>1073741823){if(I>>>0>1073741823){L=-I>>31>>>22&255}else{L=K}if(F>>>0>1073741823){M=-F>>31>>>22&255}else{M=E}if(J>>>0<=1073741823){N=L;O=M;P=G;break}N=L;O=M;P=-J>>31>>>22&255}else{N=K;O=E;P=G}}while(0);a[C]=H;a[C+1|0]=P;a[C+2|0]=O;a[C+3|0]=N;G=D+1|0;if((G|0)<(j|0)){C=C+4|0;D=G}else{n=j;break}}o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}else{D=c[f+4>>2]|0;f=c[g+4>>2]|0;if((j|0)<=0){n=0;o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}g=d+11284|0;C=d+11288|0;N=d+11292|0;O=d+11296|0;P=d+11300|0;H=d+11304|0;M=i;i=0;while(1){L=((b[D+(i<<1)>>1]|0)+(b[l+(i<<1)>>1]|0)<<1)-65536|0;B=((b[f+(i<<1)>>1]|0)+(b[m+(i<<1)>>1]|0)<<1)-65536|0;A=(b[h+(i<<1)>>1]|0)+64>>7;if((A&256|0)!=0&A>>>0>255){Q=-A>>31&255}else{Q=A&255}A=(ac((b[e+(i<<1)>>1]<<2)-(c[g>>2]|0)|0,c[C>>2]|0)|0)+2097152|0;z=A+(ac(c[N>>2]|0,B)|0)|0;y=(ac(c[O>>2]|0,B)|0)+A|0;B=y+(ac(c[P>>2]|0,L)|0)|0;y=(ac(c[H>>2]|0,L)|0)+A|0;A=y>>>22&255;L=B>>>22&255;x=z>>>22&255;do{if((B|z|y)>>>0>1073741823){if(z>>>0>1073741823){R=-z>>31>>>22&255}else{R=x}if(B>>>0>1073741823){S=-B>>31>>>22&255}else{S=L}if(y>>>0<=1073741823){T=R;U=S;V=A;break}T=R;U=S;V=-y>>31>>>22&255}else{T=x;U=L;V=A}}while(0);a[M]=Q;a[M+1|0]=V;a[M+2|0]=U;a[M+3|0]=T;A=i+1|0;if((A|0)<(j|0)){M=M+4|0;i=A}else{n=j;break}}o=d+11200|0;p=c[o>>2]|0;q=p+(n<<2)|0;c[q>>2]=0;r=d+11204|0;s=c[r>>2]|0;t=s+(n<<2)|0;c[t>>2]=0;u=d+11208|0;v=c[u>>2]|0;w=v+(n<<2)|0;c[w>>2]=0;return}}function t9(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;if((n|0)<=0){p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}o=(g|0)>0;l=(k|0)>0;z=d+11284|0;A=d+11288|0;B=d+11292|0;C=d+11296|0;D=d+11300|0;E=d+11304|0;F=m;m=0;while(1){if(o){G=0;H=512;do{H=(ac(b[e+(G<<1)>>1]|0,b[(c[f+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+H|0;G=G+1|0;}while((G|0)<(g|0));I=H>>10}else{I=0}if(l){G=0;J=-67108352;K=-67108352;do{L=b[h+(G<<1)>>1]|0;J=(ac(L,b[(c[i+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+J|0;K=(ac(b[(c[j+(G<<2)>>2]|0)+(m<<1)>>1]|0,L)|0)+K|0;G=G+1|0;}while((G|0)<(k|0));M=J>>10;N=K>>10}else{M=-65536;N=-65536}G=(ac(I-(c[z>>2]|0)|0,c[A>>2]|0)|0)+2097152|0;H=G+(ac(c[B>>2]|0,N)|0)|0;L=(ac(c[C>>2]|0,N)|0)+G|0;O=L+(ac(c[D>>2]|0,M)|0)|0;L=(ac(c[E>>2]|0,M)|0)+G|0;G=L>>>22&255;P=O>>>22&255;Q=H>>>22&255;do{if((O|H|L)>>>0>1073741823){if(H>>>0>1073741823){R=-H>>31>>>22&255}else{R=Q}if(O>>>0>1073741823){S=-O>>31>>>22&255}else{S=P}if(L>>>0<=1073741823){T=R;U=S;V=G;break}T=R;U=S;V=-L>>31>>>22&255}else{T=Q;U=P;V=G}}while(0);a[F]=-1;a[F+1|0]=V;a[F+2|0]=U;a[F+3|0]=T;G=m+1|0;if((G|0)<(n|0)){F=F+4|0;m=G}else{p=n;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}function ua(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;if((j|0)<=0){q=0;r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}A=d+11284|0;B=d+11288|0;C=d+11292|0;D=d+11296|0;E=d+11300|0;F=d+11304|0;G=i;i=0;while(1){H=ac(b[m+(i<<1)>>1]|0,g)|0;I=(ac(b[h+(i<<1)>>1]|0,k)|0)+H>>10;H=ac(b[e+(i<<1)>>1]|0,p)|0;J=H-67108864+(ac(b[n+(i<<1)>>1]|0,l)|0)>>10;H=ac(b[f+(i<<1)>>1]|0,p)|0;K=H-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;H=(ac(I-(c[A>>2]|0)|0,c[B>>2]|0)|0)+2097152|0;I=(ac(K,c[C>>2]|0)|0)+H|0;L=(ac(K,c[D>>2]|0)|0)+H|0;K=L+(ac(c[E>>2]|0,J)|0)|0;L=(ac(c[F>>2]|0,J)|0)+H|0;H=L>>>22&255;J=K>>>22&255;M=I>>>22&255;do{if((K|I|L)>>>0>1073741823){if(I>>>0>1073741823){N=-I>>31>>>22&255}else{N=M}if(K>>>0>1073741823){O=-K>>31>>>22&255}else{O=J}if(L>>>0<=1073741823){P=N;Q=O;R=H;break}P=N;Q=O;R=-L>>31>>>22&255}else{P=M;Q=J;R=H}}while(0);a[G]=-1;a[G+1|0]=R;a[G+2|0]=Q;a[G+3|0]=P;H=i+1|0;if((H|0)<(j|0)){G=G+4|0;i=H}else{q=j;break}}r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}function ub(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;l=c[f>>2]|0;h=c[g>>2]|0;do{if((k|0)<2048){if((j|0)<=0){m=0;break}n=d+11284|0;o=d+11288|0;p=d+11292|0;q=d+11296|0;r=d+11300|0;s=d+11304|0;t=i;u=0;while(1){v=(b[l+(u<<1)>>1]<<2)-65536|0;w=(b[h+(u<<1)>>1]<<2)-65536|0;x=(ac((b[e+(u<<1)>>1]<<2)-(c[n>>2]|0)|0,c[o>>2]|0)|0)+2097152|0;y=x+(ac(w,c[p>>2]|0)|0)|0;z=(ac(c[q>>2]|0,w)|0)+x|0;w=z+(ac(c[r>>2]|0,v)|0)|0;z=(ac(c[s>>2]|0,v)|0)+x|0;x=z>>>22&255;v=w>>>22&255;A=y>>>22&255;do{if((w|y|z)>>>0>1073741823){if(y>>>0>1073741823){B=-y>>31>>>22&255}else{B=A}if(w>>>0>1073741823){C=-w>>31>>>22&255}else{C=v}if(z>>>0<=1073741823){D=B;E=C;F=x;break}D=B;E=C;F=-z>>31>>>22&255}else{D=A;E=v;F=x}}while(0);a[t]=-1;a[t+1|0]=F;a[t+2|0]=E;a[t+3|0]=D;x=u+1|0;if((x|0)<(j|0)){t=t+4|0;u=x}else{m=j;break}}}else{u=c[f+4>>2]|0;t=c[g+4>>2]|0;if((j|0)<=0){m=0;break}s=d+11284|0;r=d+11288|0;q=d+11292|0;p=d+11296|0;o=d+11300|0;n=d+11304|0;x=i;v=0;while(1){A=((b[u+(v<<1)>>1]|0)+(b[l+(v<<1)>>1]|0)<<1)-65536|0;z=((b[t+(v<<1)>>1]|0)+(b[h+(v<<1)>>1]|0)<<1)-65536|0;w=(ac((b[e+(v<<1)>>1]<<2)-(c[s>>2]|0)|0,c[r>>2]|0)|0)+2097152|0;y=(ac(z,c[q>>2]|0)|0)+w|0;G=(ac(z,c[p>>2]|0)|0)+w|0;z=G+(ac(c[o>>2]|0,A)|0)|0;G=(ac(c[n>>2]|0,A)|0)+w|0;w=G>>>22&255;A=z>>>22&255;H=y>>>22&255;do{if((z|y|G)>>>0>1073741823){if(y>>>0>1073741823){I=-y>>31>>>22&255}else{I=H}if(z>>>0>1073741823){J=-z>>31>>>22&255}else{J=A}if(G>>>0<=1073741823){K=I;L=J;M=w;break}K=I;L=J;M=-G>>31>>>22&255}else{K=H;L=A;M=w}}while(0);a[x]=-1;a[x+1|0]=M;a[x+2|0]=L;a[x+3|0]=K;w=v+1|0;if((w|0)<(j|0)){x=x+4|0;v=w}else{m=j;break}}}}while(0);c[(c[d+11200>>2]|0)+(m<<2)>>2]=0;c[(c[d+11204>>2]|0)+(m<<2)>>2]=0;c[(c[d+11208>>2]|0)+(m<<2)>>2]=0;return}function uc(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;if((n|0)<=0){p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}o=(g|0)>0;l=(k|0)>0;z=d+11284|0;A=d+11288|0;B=d+11292|0;C=d+11296|0;D=d+11300|0;E=d+11304|0;F=m;m=0;while(1){if(o){G=0;H=512;do{H=(ac(b[e+(G<<1)>>1]|0,b[(c[f+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+H|0;G=G+1|0;}while((G|0)<(g|0));I=H>>10}else{I=0}if(l){G=0;J=-67108352;K=-67108352;do{L=b[h+(G<<1)>>1]|0;J=(ac(L,b[(c[i+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+J|0;K=(ac(b[(c[j+(G<<2)>>2]|0)+(m<<1)>>1]|0,L)|0)+K|0;G=G+1|0;}while((G|0)<(k|0));M=J>>10;N=K>>10}else{M=-65536;N=-65536}G=(ac(I-(c[z>>2]|0)|0,c[A>>2]|0)|0)+2097152|0;H=G+(ac(c[B>>2]|0,N)|0)|0;L=(ac(c[C>>2]|0,N)|0)+G|0;O=L+(ac(c[D>>2]|0,M)|0)|0;L=(ac(c[E>>2]|0,M)|0)+G|0;G=L>>>22&255;P=O>>>22&255;Q=H>>>22&255;do{if((O|H|L)>>>0>1073741823){if(H>>>0>1073741823){R=-H>>31>>>22&255}else{R=Q}if(O>>>0>1073741823){S=-O>>31>>>22&255}else{S=P}if(L>>>0<=1073741823){T=R;U=S;V=G;break}T=R;U=S;V=-L>>31>>>22&255}else{T=Q;U=P;V=G}}while(0);a[F]=T;a[F+1|0]=U;a[F+2|0]=V;G=m+1|0;if((G|0)<(n|0)){F=F+3|0;m=G}else{p=n;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}function ud(d,e,f,g,h,i,j,k,l,m){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0;m=c[e>>2]|0;h=c[e+4>>2]|0;e=c[f>>2]|0;n=c[f+4>>2]|0;f=c[g>>2]|0;o=c[g+4>>2]|0;g=4096-k|0;p=4096-l|0;if((j|0)<=0){q=0;r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}A=d+11284|0;B=d+11288|0;C=d+11292|0;D=d+11296|0;E=d+11300|0;F=d+11304|0;G=i;i=0;while(1){H=ac(b[m+(i<<1)>>1]|0,g)|0;I=(ac(b[h+(i<<1)>>1]|0,k)|0)+H>>10;H=ac(b[e+(i<<1)>>1]|0,p)|0;J=H-67108864+(ac(b[n+(i<<1)>>1]|0,l)|0)>>10;H=ac(b[f+(i<<1)>>1]|0,p)|0;K=H-67108864+(ac(b[o+(i<<1)>>1]|0,l)|0)>>10;H=(ac(I-(c[A>>2]|0)|0,c[B>>2]|0)|0)+2097152|0;I=(ac(K,c[C>>2]|0)|0)+H|0;L=(ac(K,c[D>>2]|0)|0)+H|0;K=L+(ac(c[E>>2]|0,J)|0)|0;L=(ac(c[F>>2]|0,J)|0)+H|0;H=L>>>22&255;J=K>>>22&255;M=I>>>22&255;do{if((K|I|L)>>>0>1073741823){if(I>>>0>1073741823){N=-I>>31>>>22&255}else{N=M}if(K>>>0>1073741823){O=-K>>31>>>22&255}else{O=J}if(L>>>0<=1073741823){P=N;Q=O;R=H;break}P=N;Q=O;R=-L>>31>>>22&255}else{P=M;Q=J;R=H}}while(0);a[G]=P;a[G+1|0]=Q;a[G+2|0]=R;H=i+1|0;if((H|0)<(j|0)){G=G+3|0;i=H}else{q=j;break}}r=d+11200|0;s=c[r>>2]|0;t=s+(q<<2)|0;c[t>>2]=0;u=d+11204|0;v=c[u>>2]|0;w=v+(q<<2)|0;c[w>>2]=0;x=d+11208|0;y=c[x>>2]|0;z=y+(q<<2)|0;c[z>>2]=0;return}function ue(d,e,f,g,h,i,j,k,l){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;l=c[f>>2]|0;h=c[g>>2]|0;do{if((k|0)<2048){if((j|0)<=0){m=0;break}n=d+11284|0;o=d+11288|0;p=d+11292|0;q=d+11296|0;r=d+11300|0;s=d+11304|0;t=i;u=0;while(1){v=(b[l+(u<<1)>>1]<<2)-65536|0;w=(b[h+(u<<1)>>1]<<2)-65536|0;x=(ac((b[e+(u<<1)>>1]<<2)-(c[n>>2]|0)|0,c[o>>2]|0)|0)+2097152|0;y=x+(ac(w,c[p>>2]|0)|0)|0;z=(ac(c[q>>2]|0,w)|0)+x|0;w=z+(ac(c[r>>2]|0,v)|0)|0;z=(ac(c[s>>2]|0,v)|0)+x|0;x=z>>>22&255;v=w>>>22&255;A=y>>>22&255;do{if((w|y|z)>>>0>1073741823){if(y>>>0>1073741823){B=-y>>31>>>22&255}else{B=A}if(w>>>0>1073741823){C=-w>>31>>>22&255}else{C=v}if(z>>>0<=1073741823){D=B;E=C;F=x;break}D=B;E=C;F=-z>>31>>>22&255}else{D=A;E=v;F=x}}while(0);a[t]=D;a[t+1|0]=E;a[t+2|0]=F;x=u+1|0;if((x|0)<(j|0)){t=t+3|0;u=x}else{m=j;break}}}else{u=c[f+4>>2]|0;t=c[g+4>>2]|0;if((j|0)<=0){m=0;break}s=d+11284|0;r=d+11288|0;q=d+11292|0;p=d+11296|0;o=d+11300|0;n=d+11304|0;x=i;v=0;while(1){A=((b[u+(v<<1)>>1]|0)+(b[l+(v<<1)>>1]|0)<<1)-65536|0;z=((b[t+(v<<1)>>1]|0)+(b[h+(v<<1)>>1]|0)<<1)-65536|0;w=(ac((b[e+(v<<1)>>1]<<2)-(c[s>>2]|0)|0,c[r>>2]|0)|0)+2097152|0;y=(ac(z,c[q>>2]|0)|0)+w|0;G=(ac(z,c[p>>2]|0)|0)+w|0;z=G+(ac(c[o>>2]|0,A)|0)|0;G=(ac(c[n>>2]|0,A)|0)+w|0;w=G>>>22&255;A=z>>>22&255;H=y>>>22&255;do{if((z|y|G)>>>0>1073741823){if(y>>>0>1073741823){I=-y>>31>>>22&255}else{I=H}if(z>>>0>1073741823){J=-z>>31>>>22&255}else{J=A}if(G>>>0<=1073741823){K=I;L=J;M=w;break}K=I;L=J;M=-G>>31>>>22&255}else{K=H;L=A;M=w}}while(0);a[x]=K;a[x+1|0]=L;a[x+2|0]=M;w=v+1|0;if((w|0)<(j|0)){x=x+3|0;v=w}else{m=j;break}}}}while(0);c[(c[d+11200>>2]|0)+(m<<2)>>2]=0;c[(c[d+11204>>2]|0)+(m<<2)>>2]=0;c[(c[d+11208>>2]|0)+(m<<2)>>2]=0;return}function uf(d,e,f,g,h,i,j,k,l,m,n,o){d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0;if((n|0)<=0){p=0;q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}o=(g|0)>0;l=(k|0)>0;z=d+11284|0;A=d+11288|0;B=d+11292|0;C=d+11296|0;D=d+11300|0;E=d+11304|0;F=m;m=0;while(1){if(o){G=0;H=512;do{H=(ac(b[e+(G<<1)>>1]|0,b[(c[f+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+H|0;G=G+1|0;}while((G|0)<(g|0));I=H>>10}else{I=0}if(l){G=0;J=-67108352;K=-67108352;do{L=b[h+(G<<1)>>1]|0;J=(ac(L,b[(c[i+(G<<2)>>2]|0)+(m<<1)>>1]|0)|0)+J|0;K=(ac(b[(c[j+(G<<2)>>2]|0)+(m<<1)>>1]|0,L)|0)+K|0;G=G+1|0;}while((G|0)<(k|0));M=J>>10;N=K>>10}else{M=-65536;N=-65536}G=(ac(I-(c[z>>2]|0)|0,c[A>>2]|0)|0)+2097152|0;H=G+(ac(c[B>>2]|0,N)|0)|0;L=(ac(c[C>>2]|0,N)|0)+G|0;O=L+(ac(c[D>>2]|0,M)|0)|0;L=(ac(c[E>>2]|0,M)|0)+G|0;G=L>>>22&255;P=O>>>22&255;Q=H>>>22&255;do{if((O|H|L)>>>0>1073741823){if(H>>>0>1073741823){R=-H>>31>>>22&255}else{R=Q}if(O>>>0>1073741823){S=-O>>31>>>22&255}else{S=P}if(L>>>0<=1073741823){T=R;U=S;V=G;break}T=R;U=S;V=-L>>31>>>22&255}else{T=Q;U=P;V=G}}while(0);a[F]=V;a[F+1|0]=U;a[F+2|0]=T;G=m+1|0;if((G|0)<(n|0)){F=F+3|0;m=G}else{p=n;break}}q=d+11200|0;r=c[q>>2]|0;s=r+(p<<2)|0;c[s>>2]=0;t=d+11204|0;u=c[t>>2]|0;v=u+(p<<2)|0;c[v>>2]=0;w=d+11208|0;x=c[w>>2]|0;y=x+(p<<2)|0;c[y>>2]=0;return}
function Km(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0;h=i;i=i+48|0;j=h|0;k=h+16|0;l=h+40|0;m=b+8|0;n=b+176|0;o=c[n>>2]|0;p=b+172|0;q=c[p>>2]|0;I_(j|0,10);r=q+112|0;c[r>>2]=0;s=q+120|0;c[s>>2]=0;t=q+124|0;c[t>>2]=0;u=b+20|0;v=c[u>>2]|0;L11739:do{if((v|0)>0){w=k;x=k|0;y=k+8|0;z=b+180|0;A=b+12|0;B=b+168|0;C=b+216|0;D=b|0;E=0;F=v;while(1){G=b+24+(E*24|0)|0;c[w>>2]=c[G>>2];c[w+4>>2]=c[G+4>>2];c[w+8>>2]=c[G+8>>2];c[w+12>>2]=c[G+12>>2];c[w+16>>2]=c[G+16>>2];c[w+20>>2]=c[G+20>>2];G=c[x>>2]|0;if((G|0)==0|(G|0)==1|(G|0)==3){G=c[y>>2]|0;H=o+(E<<2)|0;I=q+(E<<2)|0;if((G|0)>0){J=0;while(1){K=c[(c[H>>2]|0)+(J<<2)>>2]|0;L=c[(c[I>>2]|0)+2e3+(J<<2)>>2]|0;c[K+2096>>2]=L;c[K+2104>>2]=L+9312;c[K+2100>>2]=L+8592;c[K+2116>>2]=L+8352;c[K+2112>>2]=L+8112;c[K+2108>>2]=L+7872;L=J+1|0;K=c[y>>2]|0;if((L|0)<(K|0)){J=L}else{M=K;break}}}else{M=G}J=c[z>>2]|0;K=c[J+3688+(E<<2)>>2]|0;L=c[J+3736>>2]|0;N=J|0;J=c[H>>2]|0;O=b+24+(E*24|0)+12|0;P=c[A>>2]|0;KG(M,K,L,N,J,e,O,P)|0;P=c[D>>2]|0;O=KM(k,c[(c[(c[B>>2]|0)+80>>2]|0)+64+(E<<2)>>2]|0,c[H>>2]|0,c[I>>2]|0,c[C>>2]|0,c[P+48>>2]|0,a[P+52|0]|0)|0;if((O|0)!=0){Q=O;break}c[(c[I>>2]|0)+8>>2]=0;c[(c[I>>2]|0)+12>>2]=0;I_((c[I>>2]|0)+16|0,12);O=0;do{P=j+O|0;do{if((a[P]|0)==0){if((c[g+(O<<4)+12>>2]|0)!=(E|0)){break}J=g+(O<<4)+4|0;if((c[J>>2]|0)==0){break}N=g+(O<<4)|0;if((c[N>>2]|0)==0){break}L=(c[I>>2]|0)+12|0;K=c[L>>2]|0;c[L>>2]=K+1;c[(c[I>>2]|0)+16+(K*12|0)>>2]=c[g+(O<<4)+8>>2];c[(c[I>>2]|0)+16+(K*12|0)+4>>2]=c[J>>2];c[(c[I>>2]|0)+16+(K*12|0)+8>>2]=c[N>>2];N=c[D>>2]|0;J=Kp(0,(c[I>>2]|0)+16+(K*12|0)|0,0,0,c[N+48>>2]|0,c[C>>2]|0,a[N+52|0]|0)|0;N=(c[I>>2]|0)+8|0;c[N>>2]=(c[N>>2]|0)+J;a[P]=1}}while(0);O=O+1|0;}while((O|0)<10);c[r>>2]=(c[r>>2]|0)+(c[(c[I>>2]|0)+8>>2]|0);c[s>>2]=(c[s>>2]|0)+(c[c[I>>2]>>2]|0);c[t>>2]=(c[t>>2]|0)+(c[(c[I>>2]|0)+1984>>2]|0);R=c[u>>2]|0}else{R=F}O=E+1|0;if((O|0)<(R|0)){E=O;F=R}else{S=D;T=C;break L11739}}i=h;return Q|0}else{S=b|0;T=b+216|0}}while(0);R=q+96|0;c[R>>2]=0;u=q+116|0;c[u>>2]=0;I_(q+48|0,48);t=b+4|0;s=0;do{r=j+s|0;do{if((a[r]|0)==0){if((c[g+(s<<4)+12>>2]|0)!=-1){break}k=g+(s<<4)|0;if((c[k>>2]|0)==0){break}e=g+(s<<4)+8|0;if((c[e>>2]|0)==2){M=c[t>>2]|0;v=c[g+(s<<4)+4>>2]|0;if((M|0)==0){U=v>>>3>>>0>(c[(c[S>>2]|0)+60>>2]|0)>>>0?0:v}else{U=M}V=v>>>0<U>>>0?v:U}else{V=c[g+(s<<4)+4>>2]|0}if((V|0)==0){break}v=c[R>>2]|0;c[R>>2]=v+1;M=q+48+(v*12|0)|0;c[M>>2]=c[e>>2];c[q+48+(v*12|0)+4>>2]=V;c[q+48+(v*12|0)+8>>2]=c[k>>2];k=c[S>>2]|0;v=Kp(0,M,0,0,c[k+48>>2]|0,c[T>>2]|0,a[k+52|0]|0)|0;c[u>>2]=(c[u>>2]|0)+v;if((c[e>>2]|0)==2){e=g+(s<<4)+4|0;c[e>>2]=(c[e>>2]|0)-V}a[r]=1}}while(0);s=s+1|0;}while((s|0)<10);s=c[S>>2]|0;if((c[s+48>>2]&72|0)==0){c[u>>2]=(c[u>>2]|0)+3;W=c[S>>2]|0}else{W=s}c[l>>2]=0;s=b+168|0;KN(c[s>>2]|0,m,l,c[W+4>>2]|0,c[W>>2]|0,c[W+44>>2]|0)|0;W=ac(c[l>>2]|0,c[(c[S>>2]|0)+12>>2]|0)|0;c[l>>2]=W;u=H$(d,(c[(c[s>>2]|0)+24>>2]|0)+W|0)|0;c[c[s>>2]>>2]=u;u=c[S>>2]|0;W=KO(c[s>>2]|0,n,p,c[l>>2]|0,m,c[T>>2]|0,c[u+48>>2]|0,a[u+52|0]|0)|0;if((W|0)!=0){Q=W;i=h;return Q|0}W=c[s>>2]|0;u=KP(m,W,W+52|0,p)|0;if((u|0)!=0){Q=u;i=h;return Q|0}u=c[S>>2]|0;W=KR(m,c[s>>2]|0,q,q|0,d,c[T>>2]|0,c[u+48>>2]|0,a[u+52|0]|0)|0;if((W|0)!=0){Q=W;i=h;return Q|0}W=c[q+140>>2]|0;KQ(m,c[s>>2]|0,p);switch(c[b+192>>2]|0){case 1:case 2:case 3:case 4:case 5:{X=2147483647;break};case 6:case 7:{X=0;break};case 0:{X=c[(c[s>>2]|0)+24>>2]|0;break};default:{X=0}}HY(d,W,X,c[b+16>>2]|0)|0;b=c[S>>2]|0;S=Ks(d,m,q,o,c[s>>2]|0,c[T>>2]|0,c[b+48>>2]|0,a[b+52|0]|0)|0;if((S|0)!=0){Q=S;i=h;return Q|0}H_(d,f)|0;Q=0;i=h;return Q|0}function Kn(a){a=a|0;var b=0,d=0;b=c[a>>2]|0;if((b|0)==0){return}d=b+200|0;if((c[d>>2]|0)!=0){JI(d)}KC(b+180|0,b+176|0);KK(b+168|0,b+172|0);JK(a);return}function Ko(a,b,d,e,f,g,h,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0;k=i;i=i+8|0;l=k|0;m=(d|0)/(c[h+4>>2]|0)|0;L11812:do{switch(e|0){case 1:case 2:case 3:case 4:case 5:{if((b|0)!=0){c[a>>2]=b;n=0;o=b;break L11812}switch(j|0){case 2:case 3:case 4:case 5:case 6:case 7:{h=c[1921968+(e*12|0)>>2]|0;c[a>>2]=h;n=0;o=h;break L11812;break};case 1:{h=c[1921964+(e*12|0)>>2]|0;c[a>>2]=h;n=0;o=h;break L11812;break};default:{p=12512;i=k;return p|0}}break};case 0:case 7:case 6:{if((b|0)!=0){h=f>>1;d=(h|0)>2e4?2e4:h;h=(d|0)>(b|0)?b:d;c[a>>2]=h;n=0;o=h;break L11812}switch(j|0){case 2:case 3:case 4:case 5:case 6:case 7:{q=1;break};case 1:{q=0;break};default:{p=12512;i=k;return p|0}}do{if((g|0)==480|(g|0)==512){if((f|0)==8e3|(f|0)==11025|(f|0)==12e3|(f|0)==16e3|(f|0)==22050){r=1917112;s=7;t=8582;break}else if((f|0)==24e3){r=1917e3;s=8;t=8582;break}else if((f|0)==44100){r=1916752;s=10;t=8582;break}else if((f|0)==48e3|(f|0)==64e3|(f|0)==88200|(f|0)==96e3){r=1916616;s=10;t=8582;break}else if((f|0)==32e3){r=1916888;s=8;t=8582;break}else{u=-1;break}}else if((g|0)==960|(g|0)==1024){r=1926232;s=8;t=8582}else{u=-1}}while(0);L11836:do{if((t|0)==8582){h=0;while(1){v=c[r+(h*12|0)>>2]|0;w=h+1|0;if((v|0)<=(m|0)){x=c[r+(w*12|0)>>2]|0;if((x|0)>(m|0)){break}}if((w|0)<(s|0)){h=w}else{u=-1;break L11836}}if((g|0)==960|(g|0)==1024){if((q|0)==0){y=r+(h*12|0)+4|0}else{y=r+(h*12|0)+8|0}u=c[y>>2]|0;break}else if(!((g|0)==480|(g|0)==512)){u=-1;break}c[l>>2]=0;if((q|0)==0){z=r+(w*12|0)+4|0;A=r+(h*12|0)+4|0}else{z=r+(w*12|0)+8|0;A=r+(h*12|0)+8|0}d=c[A>>2]|0;B=c[z>>2]|0;C=HQ(m-v|0,x-v|0,l)|0;D=B-d|0;N2(C,(C|0)<0|0?-1:0,D,(D|0)<0|0?-1:0)|0;D=G<<1|0>>>31;C=c[l>>2]|0;if((C|0)>0){E=D<<C}else{E=D>>-C}u=E+d|0}}while(0);c[a>>2]=u;n=(u|0)==-1?16640:0;o=u;break};default:{c[a>>2]=0;p=12328;i=k;return p|0}}}while(0);u=(f|0)/2|0;c[a>>2]=(o|0)<(u|0)?o:u;p=n;i=k;return p|0}function Kp(a,b,e,f,g,h,i){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0;i=b+4|0;h=c[i>>2]|0;f=(a|0)==0;if(f){j=0}else{j=HW(a)|0}if((g&72|0)==0){k=b|0;l=c[k>>2]|0;if((l|0)!=2){if((h|0)<=6){m=0;return m|0}n=b+8|0;o=j+4|0;p=j|0;q=c[497857]&6;r=c[497858]|0;s=r&15;t=c[497862]|0;u=j+8|0;if((j|0)==0){v=0;w=h;x=l;while(1){if(x>>>0<2){y=w-7|0;z=(y|0)>119;A=0;B=(z^1)<<31>>31;C=z?w-15|0:y}else{A=7;B=-1;C=w}y=A+C>>3;z=(y|0)>269?269:y;y=z<<3;D=(y|0)<(C|0)?y:C;y=((((z|0)>14?z-14|0:B)>>31)+8|7)+v+(Kq(0,x,c[n>>2]|0,D)|0)|0;z=C-D|0;if((z|0)<=6){m=y;break}v=y;w=z;x=c[k>>2]|0}return m|0}else{E=0;F=h;G=l}while(1){if(G>>>0<2){l=F-7|0;x=(l|0)>119;H=0;I=(x^1)<<31>>31;J=x?F-15|0:l}else{H=7;I=-1;J=F}l=H+J>>3;x=(l|0)>269?269:l;l=(x|0)>14?x-14|0:I;w=c[o>>2]|0;v=w+3|0;if(v>>>0<32){c[o>>2]=v;K=v;L=c[p>>2]<<3|q}else{Hw(u,c[p>>2]|0,w);c[o>>2]=3;K=3;L=q}c[p>>2]=L;w=K+4|0;v=w>>>0<32;L11892:do{if((l|0)>-1){do{if(v){c[o>>2]=w;C=L<<4|s;c[p>>2]=C;B=K+12|0;if(B>>>0<32){M=C;N=B;break}Hw(u,C,w);c[o>>2]=8;c[p>>2]=t&l;break L11892}else{Hw(u,L,K);c[o>>2]=4;c[p>>2]=s;M=s;N=12}}while(0);c[o>>2]=N;c[p>>2]=M<<8|t&l}else{if(v){c[o>>2]=w;c[p>>2]=L<<4|r&x;break}else{Hw(u,L,K);c[o>>2]=4;c[p>>2]=r&x;break}}}while(0);w=x<<3;v=(w|0)<(J|0)?w:J;w=((l>>31)+8|7)+E+(Kq(j,c[k>>2]|0,c[n>>2]|0,v)|0)|0;C=J-v|0;if((C|0)<=6){m=w;break}E=w;F=C;G=c[k>>2]|0}return m|0}k=c[i>>2]>>3;i=c[b+8>>2]|0;if((k|0)<=0){m=0;return m|0}G=c[497857]&4;F=c[497858]&e;e=c[497862]|0;E=e&255;J=0;n=k;while(1){k=(n|0)>510?510:n;if((k|0)>254){O=k-255|0;P=J+24|0}else{O=-1;P=J+16|0}r=n-k|0;p=P+(k<<3)|0;if(!f){o=HW(a)|0;K=o+4|0;L=c[K>>2]|0;u=L+3|0;if(u>>>0<32){c[K>>2]=u;u=o|0;c[u>>2]=c[u>>2]<<3|G}else{u=o|0;Hw(o+8|0,c[u>>2]|0,L);c[K>>2]=3;c[u>>2]=G}u=H1(a,0)|0;L=c[K>>2]|0;t=L+4|0;do{if(t>>>0<32){c[K>>2]=t;M=o|0;N=c[M>>2]<<4|F;c[M>>2]=N;s=L+5|0;if(s>>>0<32){Q=N;R=s;S=8682;break}Hw(o+8|0,N,t);c[K>>2]=1;c[M>>2]=0;T=1;U=0}else{M=o|0;Hw(o+8|0,c[M>>2]|0,L);c[K>>2]=4;c[M>>2]=F;Q=F;R=5;S=8682}}while(0);if((S|0)==8682){S=0;c[K>>2]=R;L=Q<<1;c[o>>2]=L;T=R;U=L}L=T+8|0;t=L>>>0<32;L11928:do{if((O|0)>-1){do{if(t){c[K>>2]=L;l=o|0;x=U<<8|E;c[l>>2]=x;M=T+16|0;if(M>>>0<32){V=x;W=M;break}Hw(o+8|0,x,L);c[K>>2]=8;x=O&e;c[l>>2]=x;X=8;Y=x;break L11928}else{Hw(o+8|0,U,T);c[K>>2]=8;c[o>>2]=E;V=E;W=16}}while(0);c[K>>2]=W;x=V<<8|O&e;c[o>>2]=x;X=W;Y=x}else{if(t){c[K>>2]=L;x=U<<8|k&e;c[o>>2]=x;X=L;Y=x;break}else{Hw(o+8|0,U,T);c[K>>2]=8;x=k&e;c[o>>2]=x;X=8;Y=x;break}}}while(0);if((k|0)>0){L=o|0;t=o+8|0;x=0;l=X;M=Y;while(1){N=d[i+x|0]|0;s=l+8|0;if(s>>>0<32){c[K>>2]=s;Z=s;_=N&e|M<<8}else{Hw(t,M,l);c[K>>2]=8;Z=8;_=N&e}c[L>>2]=_;N=x+1|0;if((N|0)<(k|0)){x=N;l=Z;M=_}else{break}}}H2(a,u)}if((r|0)>0){J=p;n=r}else{m=p;break}}return m|0}if((g&131072|0)==0){n=c[b>>2]|0;do{if((g&16|0)!=0){if((n-13|0)>>>0>=2){break}if((j|0)==0){m=h;return m|0}J=c[b+8>>2]|0;if((h|0)>7){a=c[497862]|0;_=j+4|0;Z=j|0;e=j+8|0;i=h-8|0;Y=i>>>3;X=Y<<3;T=0;U=h;W=c[_>>2]|0;while(1){O=d[J+T|0]|0;V=W+8|0;if(V>>>0<32){c[_>>2]=V;$=V;aa=c[Z>>2]<<8|a&O}else{Hw(e,c[Z>>2]|0,W);c[_>>2]=8;$=8;aa=a&O}c[Z>>2]=aa;O=U-8|0;if((O|0)>7){T=T+1|0;U=O;W=$}else{break}}ab=Y+1|0;ac=i-X|0}else{ab=0;ac=h}if((ac|0)<=0){m=h;return m|0}W=(d[J+ab|0]|0)>>>((8-ac|0)>>>0);U=c[1991416+(ac<<2)>>2]|0;T=j+4|0;Z=c[T>>2]|0;a=Z+ac|0;if(a>>>0<32){c[T>>2]=a;a=j|0;c[a>>2]=c[a>>2]<<ac|W&U;m=h;return m|0}else{a=j|0;Hw(j+8|0,c[a>>2]|0,Z);c[T>>2]=ac;c[a>>2]=W&U;m=h;return m|0}}}while(0);m=Kq(j,n,c[b+8>>2]|0,h)|0;return m|0}n=c[b+8>>2]|0;ac=(j|0)!=0;if(((c[b>>2]|0)-13|0)>>>0>=2){if(!ac){m=h;return m|0}b=j+4|0;ab=c[b>>2]|0;if((h|0)>7){$=j|0;aa=j+8|0;g=h;U=ab;while(1){W=U+8|0;if(W>>>0<32){c[b>>2]=W;ad=W;ae=c[$>>2]<<8}else{Hw(aa,c[$>>2]|0,U);c[b>>2]=8;ad=8;ae=0}c[$>>2]=ae;W=g-8|0;if((W|0)>7){g=W;U=ad}else{break}}af=h&7;ag=ad}else{af=h;ag=ab}ab=ag+af|0;if(ab>>>0<32){c[b>>2]=ab;ab=j|0;c[ab>>2]=c[ab>>2]<<af;m=h;return m|0}else{ab=j|0;Hw(j+8|0,c[ab>>2]|0,ag);c[b>>2]=af;c[ab>>2]=0;m=h;return m|0}}if(!ac){m=h;return m|0}ac=h-1|0;ab=j+4|0;af=c[ab>>2]|0;b=j+40|0;ag=(c[b>>2]|0)==0;do{if(af>>>0>ac>>>0){if(ag){c[ab>>2]=af-ac;break}else{ah=j+8|0;S=8618;break}}else{ad=j+8|0;if(!ag){ah=ad;S=8618;break}Hy(ad,af,0);ai=j|0;aj=ad;S=8619}}while(0);if((S|0)==8618){ag=j|0;Hw(ah,c[ag>>2]|0,af);ai=ag;aj=ah;S=8619}if((S|0)==8619){c[ab>>2]=0;c[ai>>2]=0;Hz(aj,ac,c[b>>2]&255)}if((h|0)>7){ac=c[497862]|0;aj=j|0;ai=j+8|0;S=h-8|0;ah=S>>>3;ag=ah<<3;af=0;ad=h;U=c[ab>>2]|0;while(1){g=d[n+af|0]|0;ae=U+8|0;if(ae>>>0<33){c[ab>>2]=ae;ak=ae;al=c[aj>>2]<<8|ac&g}else{Hx(ai,c[aj>>2]|0,U);c[ab>>2]=8;ak=8;al=ac&g}c[aj>>2]=al;g=ad-8|0;if((g|0)>7){af=af+1|0;ad=g;U=ak}else{break}}am=ah+1|0;an=S-ag|0}else{am=0;an=h}do{if((an|0)>0){ag=(d[n+am|0]|0)>>>((8-an|0)>>>0);S=c[1991416+(an<<2)>>2]|0;ah=c[ab>>2]|0;ak=ah+an|0;if(ak>>>0<33){c[ab>>2]=ak;ak=j|0;c[ak>>2]=c[ak>>2]<<an|ag&S;break}else{ak=j|0;Hx(j+8|0,c[ak>>2]|0,ah);c[ab>>2]=an;c[ak>>2]=ag&S;break}}}while(0);an=j+8|0;if((c[b>>2]|0)==0){Hz(an,c[ab>>2]|0,0);ao=j|0}else{am=j|0;Hx(an,c[am>>2]|0,c[ab>>2]|0);ao=am}c[ab>>2]=0;c[ao>>2]=0;if((c[b>>2]|0)==0){Hy(an,0,0)}else{Hw(an,0,0)}c[ab>>2]=0;c[j>>2]=0;Hz(an,h+1|0,c[b>>2]&255);m=h;return m|0}function Kq(a,b,e,f){a=a|0;b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0;if((f|0)<=3){g=0;return g|0}h=(a|0)!=0;do{if(h){i=c[497858]|0;j=a+4|0;k=c[j>>2]|0;l=k+4|0;if(l>>>0<32){c[j>>2]=l;l=a|0;c[l>>2]=c[l>>2]<<4|i&b;break}else{l=a|0;Hw(a+8|0,c[l>>2]|0,k);c[j>>2]=4;c[l>>2]=i&b;break}}}while(0);switch(b|0){case 2:{b=f+7>>3;if((b|0)>254){i=b-255|0;l=(i>>>0)/255|0;m=i+(l*-255|0)|0;n=l+2|0}else{m=b;n=1}do{if(h){l=a+4|0;i=c[l>>2]|0;j=i+4|0;if(j>>>0<32){c[l>>2]=j;k=a|0;o=c[k>>2]<<4;c[k>>2]=o;p=j;q=o}else{o=a|0;Hw(a+8|0,c[o>>2]|0,i);c[l>>2]=4;c[o>>2]=0;p=4;q=0}o=c[497862]|0;i=p+8|0;j=i>>>0<32;if((n|0)>1){k=a|0;r=a+8|0;s=1;t=p;u=i;v=j;w=q;while(1){if(v){c[l>>2]=u;x=u;y=w<<8|o&255}else{Hw(r,w,t);c[l>>2]=8;x=8;y=o&255}c[k>>2]=y;z=s+1|0;A=x+8|0;B=A>>>0<32;if((z|0)<(n|0)){s=z;t=x;u=A;v=B;w=y}else{C=x;D=A;E=B;F=y;break}}}else{C=p;D=i;E=j;F=q}if(E){c[l>>2]=D;w=F<<8|o&m;c[a>>2]=w;G=D;H=w}else{Hw(a+8|0,F,C);c[l>>2]=8;w=o&m;c[a>>2]=w;G=8;H=w}if((b|0)<=0){break}w=a|0;v=a+8|0;u=0;t=G;s=H;while(1){k=d[e+u|0]|0;r=t+8|0;if(r>>>0<32){c[l>>2]=r;I=r;J=s<<8|o&k}else{Hw(v,s,t);c[l>>2]=8;I=8;J=o&k}c[w>>2]=J;k=u+1|0;if((k|0)<(b|0)){u=k;t=I;s=J}else{break}}}}while(0);g=(b<<3|4)+(n<<3|4)|0;return g|0};case 11:case 13:case 14:{do{if(h){if((f|0)>7){n=c[497862]|0;b=a+4|0;J=a|0;I=a+8|0;H=f-8|0;G=H>>>3;m=G<<3;C=0;F=f;D=c[b>>2]|0;while(1){E=d[e+C|0]|0;q=D+8|0;if(q>>>0<32){c[b>>2]=q;K=q;L=c[J>>2]<<8|n&E}else{Hw(I,c[J>>2]|0,D);c[b>>2]=8;K=8;L=n&E}c[J>>2]=L;E=F-8|0;if((E|0)>7){C=C+1|0;F=E;D=K}else{break}}M=G+1|0;N=H-m|0}else{M=0;N=f}if((N|0)<=0){break}D=(d[e+M|0]|0)>>>((8-N|0)>>>0);F=c[1991416+(N<<2)>>2]|0;C=a+4|0;J=c[C>>2]|0;n=J+N|0;if(n>>>0<32){c[C>>2]=n;n=a|0;c[n>>2]=c[n>>2]<<N|D&F;break}else{n=a|0;Hw(a+8|0,c[n>>2]|0,J);c[C>>2]=N;c[n>>2]=D&F;break}}}while(0);g=f+4|0;return g|0};case 1:{O=165;break};default:{O=0}}do{if(h){N=a+4|0;M=c[N>>2]|0;e=M+4|0;if(e>>>0<32){c[N>>2]=e;K=a|0;L=c[K>>2]<<4;c[K>>2]=L;P=e;Q=L}else{L=a|0;Hw(a+8|0,c[L>>2]|0,M);c[N>>2]=4;c[L>>2]=0;P=4;Q=0}L=f-8|0;if((L|0)<=7){break}M=a|0;e=c[497862]&O;K=a+8|0;F=L;L=P;D=Q;while(1){n=L+8|0;if(n>>>0<32){c[N>>2]=n;R=n;S=D<<8|e}else{Hw(K,D,L);c[N>>2]=8;R=8;S=e}c[M>>2]=S;n=F-8|0;if((n|0)>7){F=n;L=R;D=S}else{break}}}}while(0);g=(f&-8|4)-4|0;return g|0}function Kr(a,b,d,e,f,g,h,i,j,k){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0,cb=0,cc=0,cd=0,ce=0;l=(a|0)!=0;if(l){m=HW(a)|0}else{m=0}n=b|0;o=c[n>>2]|0;if((o|0)==0|(o|0)==3){p=1}else{p=2}o=HD(h,i,p,0)|0;L12128:do{if((o|0)==0){q=12288;r=0}else{do{if((g&72|0)==0){if((m|0)==0){s=3;break}i=c[n>>2]|0;h=c[497857]|0;t=m+4|0;u=c[t>>2]|0;v=u+3|0;if(v>>>0<32){c[t>>2]=v;v=m|0;c[v>>2]=c[v>>2]<<3|h&i;s=3;break}else{v=m|0;Hw(m+8|0,c[v>>2]|0,u);c[t>>2]=3;c[v>>2]=h&i;s=3;break}}else{s=0}}while(0);i=k<<24>>24==0;h=(d|0)==0;v=(m|0)==0;t=b+4|0;u=c[497858]|0;w=m+4|0;x=m|0;y=m+8|0;z=e+8|0;A=c[497855]|0;B=(g&16|0)==0;C=c[497856]|0;D=c[497860]|0;E=(g&24|0)==0;F=c[497861]|0;G=g>>>3&1^11;H=e+12|0;I=C&2;J=C&1;K=A&1;L=c[497862]|0;M=m+40|0;N=c[497863]|0;O=(g&2|0)==0?0:12288;P=g>>>1&2;Q=p&255;R=-1;S=-1;T=0;U=0;V=0;W=o;X=s;L12137:while(1){if(i){if(h){Y=c[f+(U<<2)>>2]|0;Z=0;_=Y|0;$=Y+4|0;aa=Y+8|0;ab=0;ad=0;ae=0;af=0;ag=0;ah=Y}else{Y=c[d+(U<<2)>>2]|0;ai=Y+6628|0;Z=c[Y+6384+(c[Y+7628>>2]<<2)>>2]|0;_=Y+6636|0;$=Y+6644|0;aa=Y+6640|0;ab=c[ai>>2]|0;ad=c[Y+6624>>2]|0;ae=Y+6144|0;af=Y+6384|0;ag=ai;ah=c[f+(U<<2)>>2]|0}aj=Z;ak=c[_>>2]|0;al=c[$>>2]|0;am=c[aa>>2]|0;an=ab;ao=ad;ap=ah+288|0;aq=ae;ar=af;as=ag}else{aj=0;ak=0;al=0;am=0;an=0;ao=0;ap=0;aq=0;ar=0;as=0}if(h){at=c[(c[f+(U<<2)>>2]|0)+12>>2]|0}else{at=an}L12149:do{switch(c[(c[W>>2]|0)+(V<<2)>>2]|0){case 9:{ai=c[f+(U<<2)>>2]|0;if(v){au=0;av=0}else{if((c[M>>2]|0)==0){Hy(y,c[w>>2]|0,0)}else{Hw(y,c[x>>2]|0,c[w>>2]|0)}c[w>>2]=0;c[x>>2]=0;Y=HA(y)|0;aw=c[ar+(c[as+1e3>>2]<<2)>>2]|0;ax=as+20|0;L12157:do{if((c[ax>>2]|0)>0){ay=0;az=1;aA=ao+70-aw|0;aB=aw;aC=0;while(1){aD=c[as+24+(aC<<4)>>2]|0;L12161:do{if((aD|0)==14|(aD|0)==15){aE=c[as+24+(aC<<4)+4>>2]|0;aF=(c[as+24+(aC<<4)+8>>2]|0)+aE|0;aG=aE;aE=ay;while(1){if((aG|0)>=(aF|0)){aH=aB;aI=aA;aJ=az;aK=aE;break L12161}aL=c[ai+1856+(aG<<2)>>2]|0;if((MZ(aL-aE|0,m)|0)==0){aG=aG+1|0;aE=aL}else{aM=1;break L12157}}}else if((aD|0)==0){aH=aB;aI=aA;aJ=az;aK=ay}else{aE=c[as+24+(aC<<4)+4>>2]|0;aG=c[as+24+(aC<<4)+8>>2]|0;aF=aG+aE|0;if((aD|0)!=13){aL=aE;aN=aB;while(1){if((aL|0)>=(aF|0)){aH=aN;aI=aA;aJ=az;aK=ay;break L12161}if((c[aq+(aL<<2)>>2]|0)==0){aO=aN;aP=0}else{aQ=c[ar+(aL<<2)>>2]|0;aO=aQ;aP=aN-aQ|0}if((MZ(aP,m)|0)==0){aL=aL+1|0;aN=aO}else{aM=1;break L12157}}}if((aG|0)>0){aR=az;aS=aA;aT=aE}else{aH=aB;aI=aA;aJ=az;aK=ay;break}while(1){aN=c[ai+1376+(aT<<2)>>2]|0;aL=aN-aS|0;do{if((aR|0)==0){if((MZ(aL,m)|0)!=0){aM=1;break L12157}}else{aQ=aL+256|0;aU=c[w>>2]|0;aV=aU+9|0;if(aV>>>0<32){c[w>>2]=aV;c[x>>2]=c[x>>2]<<9|aQ&N;break}else{Hw(y,c[x>>2]|0,aU);c[w>>2]=9;c[x>>2]=aQ&N;break}}}while(0);aL=aT+1|0;if((aL|0)<(aF|0)){aR=0;aS=aN;aT=aL}else{aH=aB;aI=aN;aJ=0;aK=ay;break}}}}while(0);aD=aC+1|0;if((aD|0)<(c[ax>>2]|0)){ay=aK;az=aJ;aA=aI;aB=aH;aC=aD}else{aW=8935;break}}}else{aW=8935}}while(0);if((aW|0)==8935){aW=0;if((c[M>>2]|0)==0){Hy(y,c[w>>2]|0,0)}else{Hw(y,c[x>>2]|0,c[w>>2]|0)}c[w>>2]=0;c[x>>2]=0;aM=(HA(y)|0)-Y|0}ax=c[d+(U<<2)>>2]|0;au=(aM|0)==((c[ax+7624>>2]|0)+(c[ax+7620>>2]|0)|0)?0:16864;av=aM}aX=au;aY=av+X|0;aW=9048;break};case 3:{ax=c[f+(U<<2)>>2]|0;ai=c[ax+16>>2]|0;aw=c[ax+20>>2]|0;if((at|0)==2){aZ=15}else{aZ=B?G:6}L12197:do{if(!v){L12199:do{if(B){ax=c[w>>2]|0;aC=ax+1|0;do{if(aC>>>0<32){c[w>>2]=aC;aB=c[x>>2]<<1;c[x>>2]=aB;aA=ax+3|0;if(aA>>>0<32){a_=aB;a$=aA;aW=8828;break}Hw(y,aB,aC);c[w>>2]=2;aB=C&at;c[x>>2]=aB;a0=aB;a1=(ai|0)==2?1:ai;a2=3}else{Hw(y,c[x>>2]|0,ax);c[w>>2]=1;c[x>>2]=0;a_=0;a$=3;aW=8828}}while(0);do{if((aW|0)==8828){aW=0;c[w>>2]=a$;ax=a_<<2|C&at;c[x>>2]=ax;aC=(ai|0)==2?1:ai;aB=a$+1|0;if(aB>>>0<32){a0=ax;a1=aC;a2=aB;break}Hw(y,ax,a$);c[w>>2]=1;c[x>>2]=A&aC;break L12199}}while(0);c[w>>2]=a2;c[x>>2]=a0<<1|a1&A}}while(0);if((at|0)==2){aC=c[w>>2]|0;ax=aC+4|0;do{if(ax>>>0<32){c[w>>2]=ax;aB=c[x>>2]<<4|u&am;c[x>>2]=aB;aA=aC+11|0;if(aA>>>0<32){a3=aB;a4=aA;break}Hw(y,aB,ax);c[w>>2]=7;c[x>>2]=F&aw;break L12197}else{Hw(y,c[x>>2]|0,aC);c[w>>2]=4;aB=u&am;c[x>>2]=aB;a3=aB;a4=11}}while(0);c[w>>2]=a4;c[x>>2]=a3<<7|F&aw;break}else if(!((at|0)==0|(at|0)==1|(at|0)==3)){break}aC=c[w>>2]|0;ax=aC+6|0;if(ax>>>0<32){c[w>>2]=ax;a5=ax;a6=c[x>>2]<<6|D&am}else{Hw(y,c[x>>2]|0,aC);c[w>>2]=6;a5=6;a6=D&am}c[x>>2]=a6;if(!E){break}aC=a5+1|0;if(aC>>>0<32){c[w>>2]=aC;c[x>>2]=a6<<1;break}else{Hw(y,a6,a5);c[w>>2]=1;c[x>>2]=0;break}}}while(0);a7=aZ+X|0;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 12:{L12228:do{if((ap|0)==0){be=0}else{aw=(at|0)==2;ai=aw?8:1;Y=0;aC=0;do{aC=(c[ap+(Y<<2)>>2]|0)==0?aC:1;Y=Y+1|0;}while((Y|0)<(ai|0));if(v){if((aC|0)==0){be=0;break}Y=aw?1:2;ax=(aw?3:5)+(aw?4:6)|0;aB=ax+2|0;aA=0;az=0;while(1){ay=az+Y|0;aD=c[ap+(aA<<2)>>2]|0;do{if((aD|0)==0){bf=ay}else{aF=ay+1|0;if((aD|0)<=0){bf=aF;break}aE=ap+32+(aA<<2)|0;aG=0;aL=aF;while(1){aF=aL+ax|0;aQ=c[ap+128+(aA<<3)+(aG<<2)>>2]|0;do{if((aQ|0)==0){bg=aF}else{aU=aF+2|0;L12245:do{if((c[aE>>2]|0)==4){aV=0;while(1){if((aV|0)>=(aQ|0)){bh=3;break L12245}if(((c[ap+320+(aA*96|0)+(aG*48|0)+(aV<<2)>>2]|0)+4|0)>>>0>7){bh=4;break}else{aV=aV+1|0}}}else{aV=0;while(1){if((aV|0)>=(aQ|0)){bh=2;break L12245}if(((c[ap+320+(aA*96|0)+(aG*48|0)+(aV<<2)>>2]|0)+2|0)>>>0>3){bh=3;break}else{aV=aV+1|0}}}}while(0);if((aQ|0)<=0){bg=aU;break}bg=aB+aL+(ac(bh,(aQ|0)>1?aQ:1)|0)|0}}while(0);aQ=aG+1|0;if((aQ|0)<(aD|0)){aG=aQ;aL=bg}else{bf=bg;break}}}}while(0);aD=aA+1|0;if((aD|0)<(ai|0)){aA=aD;az=bf}else{be=bf;break L12228}}}if((aC|0)!=1){be=0;break}az=aw?1:2;aA=c[1991416+(az<<2)>>2]|0;aB=aw?4:6;ax=1991416+(aB<<2)|0;Y=aw?3:5;aD=1991416+(Y<<2)|0;ay=Y+aB|0;aL=0;aG=0;aE=c[w>>2]|0;while(1){aQ=ap+(aL<<2)|0;aF=c[aQ>>2]|0;aN=aE+az|0;if(aN>>>0<32){c[w>>2]=aN;bi=aN;bj=c[x>>2]<<az|aF&aA}else{Hw(y,c[x>>2]|0,aE);c[w>>2]=az;bi=az;bj=aF&aA}c[x>>2]=bj;aF=aG+az|0;do{if((c[aQ>>2]|0)==0){bk=aF;bl=bi}else{aN=ap+32+(aL<<2)|0;aV=(c[aN>>2]|0)==4|0;bm=bi+1|0;if(bm>>>0<32){c[w>>2]=bm;bn=bm;bo=aV&A|bj<<1}else{Hw(y,bj,bi);c[w>>2]=1;bn=1;bo=aV&A}c[x>>2]=bo;aV=aF+1|0;if((c[aQ>>2]|0)<=0){bk=aV;bl=bn;break}bm=c[ax>>2]|0;bp=c[aD>>2]|0;bq=0;br=aV;aV=bn;bs=bo;while(1){bt=c[ap+64+(aL<<3)+(bq<<2)>>2]|0;bu=aV+aB|0;if(bu>>>0<32){c[w>>2]=bu;bv=bu;bw=bt&bm|bs<<aB}else{Hw(y,bs,aV);c[w>>2]=aB;bv=aB;bw=bt&bm}c[x>>2]=bw;bt=ap+128+(aL<<3)+(bq<<2)|0;bu=c[bt>>2]|0;bx=bv+Y|0;if(bx>>>0<32){c[w>>2]=bx;by=bx;bz=bu&bp|bw<<Y}else{Hw(y,bw,bv);c[w>>2]=Y;by=Y;bz=bu&bp}c[x>>2]=bz;bu=ay+br|0;do{if((c[bt>>2]|0)==0){bA=bu;bB=by;bC=bz}else{bx=c[ap+192+(aL<<3)+(bq<<2)>>2]|0;bD=by+1|0;if(bD>>>0<32){c[w>>2]=bD;bE=bD;bF=bx&A|bz<<1}else{Hw(y,bz,by);c[w>>2]=1;bE=1;bF=bx&A}c[x>>2]=bF;bx=c[aN>>2]|0;bD=c[bt>>2]|0;L12287:do{if((bx|0)==4){bG=0;while(1){if((bG|0)>=(bD|0)){bH=3;break L12287}if(((c[ap+320+(aL*96|0)+(bq*48|0)+(bG<<2)>>2]|0)+4|0)>>>0>7){bH=4;break}else{bG=bG+1|0}}}else{bG=0;while(1){if((bG|0)>=(bD|0)){bH=2;break L12287}if(((c[ap+320+(aL*96|0)+(bq*48|0)+(bG<<2)>>2]|0)+2|0)>>>0>3){bH=3;break}else{bG=bG+1|0}}}}while(0);bD=bx-bH|0;aU=bE+1|0;if(aU>>>0<32){c[w>>2]=aU;bI=aU;bJ=bD&A|bF<<1}else{Hw(y,bF,bE);c[w>>2]=1;bI=1;bJ=bD&A}c[x>>2]=bJ;bD=bu+2|0;if((c[bt>>2]|0)<=0){bA=bD;bB=bI;bC=bJ;break}aU=c[1913536+(bH<<2)>>2]|0;bG=c[1991416+(bH<<2)>>2]|0;bK=0;bL=bD;bD=bI;bM=bJ;while(1){bN=c[ap+320+(aL*96|0)+(bq*48|0)+(bK<<2)>>2]&aU;bO=bD+bH|0;if(bO>>>0<32){c[w>>2]=bO;bP=bO;bQ=bN&bG|bM<<bH}else{Hw(y,bM,bD);c[w>>2]=bH;bP=bH;bQ=bN&bG}c[x>>2]=bQ;bN=bL+bH|0;bO=bK+1|0;if((bO|0)<(c[bt>>2]|0)){bK=bO;bL=bN;bD=bP;bM=bQ}else{bA=bN;bB=bP;bC=bQ;break}}}}while(0);bt=bq+1|0;if((bt|0)<(c[aQ>>2]|0)){bq=bt;br=bA;aV=bB;bs=bC}else{bk=bA;bl=bB;break}}}}while(0);aQ=aL+1|0;if((aQ|0)<(ai|0)){aL=aQ;aG=bk;aE=bl}else{be=bk;break}}}}while(0);a7=be+X|0;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 8:{if(v){bR=0;bS=0}else{if((c[M>>2]|0)==0){Hy(y,c[w>>2]|0,0)}else{Hw(y,c[x>>2]|0,c[w>>2]|0)}c[w>>2]=0;c[x>>2]=0;aE=HA(y)|0;aG=c[as>>2]|0;if((aG|0)==0|(aG|0)==1|(aG|0)==3){bT=31;bU=5}else if((aG|0)==2){bT=7;bU=3}else{bT=0;bU=0}aG=as+20|0;if((c[aG>>2]|0)>0){aL=c[1991416+(bU<<2)>>2]|0;ai=aL&bT;ay=0;Y=c[w>>2]|0;while(1){aB=c[as+24+(ay<<4)>>2]|0;aD=Y+4|0;if(aD>>>0<32){c[w>>2]=aD;bV=aD;bW=c[x>>2]<<4|aB&u}else{Hw(y,c[x>>2]|0,Y);c[w>>2]=4;bV=4;bW=aB&u}c[x>>2]=bW;aB=c[as+24+(ay<<4)+8>>2]|0;aD=bV+bU|0;ax=aD>>>0<32;if((aB|0)<(bT|0)){bX=aB;bY=bV;bZ=aD;b_=ax;b$=bW}else{az=aB;aB=bV;aA=aD;aD=ax;ax=bW;while(1){if(aD){c[w>>2]=aA;b0=aA;b1=ax<<bU|ai}else{Hw(y,ax,aB);c[w>>2]=bU;b0=bU;b1=ai}c[x>>2]=b1;aw=az-bT|0;aC=b0+bU|0;aQ=aC>>>0<32;if((aw|0)<(bT|0)){bX=aw;bY=b0;bZ=aC;b_=aQ;b$=b1;break}else{az=aw;aB=b0;aA=aC;aD=aQ;ax=b1}}}if(b_){c[w>>2]=bZ;b2=bZ;b3=bX&aL|b$<<bU}else{Hw(y,b$,bY);c[w>>2]=bU;b2=bU;b3=bX&aL}c[x>>2]=b3;ax=ay+1|0;if((ax|0)<(c[aG>>2]|0)){ay=ax;Y=b2}else{break}}}if((c[M>>2]|0)==0){Hy(y,c[w>>2]|0,0)}else{Hw(y,c[x>>2]|0,c[w>>2]|0)}c[w>>2]=0;c[x>>2]=0;Y=(HA(y)|0)-aE|0;bR=(Y|0)==(c[(c[d+(U<<2)>>2]|0)+7612>>2]|0)?0:16896;bS=Y}aX=bR;aY=bS+X|0;aW=9048;break};case 2:{do{if(!v){Y=ao+100-aj-(-15-(c[(c[f+(U<<2)>>2]|0)+268>>2]|0)<<2)|0;ay=c[w>>2]|0;aG=ay+8|0;if(aG>>>0<32){c[w>>2]=aG;c[x>>2]=c[x>>2]<<8|Y&L;break}else{Hw(y,c[x>>2]|0,ay);c[w>>2]=8;c[x>>2]=Y&L;break}}}while(0);a7=X+8|0;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 13:{do{if(!v){aE=c[w>>2]|0;Y=aE+1|0;if(Y>>>0<32){c[w>>2]=Y;c[x>>2]=c[x>>2]<<1;break}else{Hw(y,c[x>>2]|0,aE);c[w>>2]=1;c[x>>2]=0;break}}}while(0);a7=X+1|0;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 15:{aX=P;aY=X;aW=9048;break};case 17:{if(v){a7=X;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break L12149}aE=c[f+(U<<2)>>2]|0;Y=d+(U<<2)|0;ay=c[Y>>2]|0;if((c[M>>2]|0)==0){Hy(y,c[w>>2]|0,0)}else{Hw(y,c[x>>2]|0,c[w>>2]|0)}c[w>>2]=0;c[x>>2]=0;aG=HA(y)|0;aL=as+20|0;ai=c[aL>>2]|0;if((ai|0)>0){ax=0;aD=ai;while(1){ai=as+24+(ax<<4)|0;aA=c[ai>>2]|0;do{if((aA|0)==13){b4=aD}else{aB=c[as+24+(ax<<4)+4>>2]|0;az=c[as+24+(ax<<4)+8>>2]|0;aQ=az+aB|0;if((az|0)<=0){b4=aD;break}az=c[aE+24+(aB<<2)>>2]|0;aC=ay+4096+(az<<1)|0;aw=aB+1|0;aB=(c[aE+24+(aw<<2)>>2]|0)-az|0;MY(aC,aB,aA,m)|0;if((aw|0)<(aQ|0)){aB=aw;do{aw=c[aE+24+(aB<<2)>>2]|0;aB=aB+1|0;MY(ay+4096+(aw<<1)|0,(c[aE+24+(aB<<2)>>2]|0)-aw|0,c[ai>>2]|0,m)|0;}while((aB|0)<(aQ|0))}b4=c[aL>>2]|0}}while(0);ai=ax+1|0;if((ai|0)<(b4|0)){ax=ai;aD=b4}else{break}}}if((c[M>>2]|0)==0){Hy(y,c[w>>2]|0,0)}else{Hw(y,c[x>>2]|0,c[w>>2]|0)}c[w>>2]=0;c[x>>2]=0;aD=(HA(y)|0)-aG|0;if((aD|0)!=(c[(c[Y>>2]|0)+7616>>2]|0)){b5=16928;aW=9054;break L12137}a7=aD+X|0;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 5:{if(i){b6=c[H>>2]|0}else{b6=0}do{if(v){if(!((b6|0)==1&(ak|0)>0)){b7=2;break}if((am|0)>0){aD=0;ax=2;while(1){aL=ax+am|0;aE=aD+al|0;if((aE|0)<(ak|0)){aD=aE;ax=aL}else{b7=aL;break}}}else{ax=0;while(1){aD=ax+al|0;if((aD|0)<(ak|0)){ax=aD}else{b7=2;break}}}}else{if((b6|0)==0){ax=c[w>>2]|0;aD=ax+2|0;if(aD>>>0<32){c[w>>2]=aD;c[x>>2]=c[x>>2]<<2;b7=2;break}else{Hw(y,c[x>>2]|0,ax);c[w>>2]=2;c[x>>2]=0;b7=2;break}}else if((b6|0)==1){ax=c[w>>2]|0;aD=ax+2|0;if(aD>>>0<32){c[w>>2]=aD;b8=aD;b9=c[x>>2]<<2|J}else{Hw(y,c[x>>2]|0,ax);c[w>>2]=2;b8=2;b9=J}c[x>>2]=b9;if((ak|0)<=0){b7=2;break}ax=(am|0)>0;aD=0;aL=2;aE=b8;ay=b9;while(1){if(ax){ai=0;aA=aE;aQ=ay;while(1){aB=aA+1|0;aw=aB>>>0<32;do{if((c[e+16+(ai+aD<<2)>>2]&1|0)==0){if(aw){c[w>>2]=aB;aC=aQ<<1;c[x>>2]=aC;ca=aB;cb=aC;break}else{Hw(y,aQ,aA);c[w>>2]=1;c[x>>2]=0;ca=1;cb=0;break}}else{if(aw){c[w>>2]=aB;aC=aQ<<1|K;c[x>>2]=aC;ca=aB;cb=aC;break}else{Hw(y,aQ,aA);c[w>>2]=1;c[x>>2]=K;ca=1;cb=K;break}}}while(0);aB=ai+1|0;if((aB|0)<(am|0)){ai=aB;aA=ca;aQ=cb}else{break}}cc=aL+am|0;cd=ca;ce=cb}else{cc=aL;cd=aE;ce=ay}aQ=aD+al|0;if((aQ|0)<(ak|0)){aD=aQ;aL=cc;aE=cd;ay=ce}else{b7=cc;break}}}else if((b6|0)==2){ay=c[w>>2]|0;aE=ay+2|0;if(aE>>>0<32){c[w>>2]=aE;c[x>>2]=c[x>>2]<<2|I;b7=2;break}else{Hw(y,c[x>>2]|0,ay);c[w>>2]=2;c[x>>2]=I;b7=2;break}}else{b7=0;break}}}while(0);a7=b7+X|0;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 1:{Y=c[z>>2]|0;do{if(!v){aG=c[w>>2]|0;ay=aG+1|0;if(ay>>>0<32){c[w>>2]=ay;c[x>>2]=c[x>>2]<<1|A&Y;break}else{Hw(y,c[x>>2]|0,aG);c[w>>2]=1;c[x>>2]=A&Y;break}}}while(0);a7=X+1|0;a8=W;a9=V;ba=U;bb=Y;bc=S;bd=R;break};case 10:{do{if(!v){aG=c[w>>2]|0;ay=aG+1|0;if(ay>>>0<32){c[w>>2]=ay;c[x>>2]=c[x>>2]<<1;break}else{Hw(y,c[x>>2]|0,aG);c[w>>2]=1;c[x>>2]=0;break}}}while(0);a7=X+1|0;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 11:{do{if(!(v|(ap|0)==0)){Y=(at|0)==2?8:1;aG=0;while(1){if((aG|0)>=(Y|0)){aW=8950;break}if((c[ap+(aG<<2)>>2]|0)==0){aG=aG+1|0}else{aW=8953;break}}if((aW|0)==8950){aW=0;aG=c[w>>2]|0;Y=aG+1|0;if(Y>>>0<32){c[w>>2]=Y;c[x>>2]=c[x>>2]<<1;break}else{Hw(y,c[x>>2]|0,aG);c[w>>2]=1;c[x>>2]=0;break}}else if((aW|0)==8953){aW=0;aG=c[w>>2]|0;Y=aG+1|0;if(Y>>>0<32){c[w>>2]=Y;c[x>>2]=c[x>>2]<<1|K;break}else{Hw(y,c[x>>2]|0,aG);c[w>>2]=1;c[x>>2]=K;break}}}}while(0);a7=X+1|0;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 29:{if(!l){a7=X;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break L12149}a7=X;a8=W;a9=V;ba=U;bb=T;bc=H1(a,192)|0;bd=R;break};case 30:{if(!l){a7=X;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break L12149}a7=X;a8=W;a9=V;ba=U;bb=T;bc=S;bd=H1(a,128)|0;break};case 31:case 34:{if(!l){a7=X;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break L12149}H2(a,S);a7=X;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 32:{if(!l){a7=X;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break L12149}H2(a,R);a7=X;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 33:{if(!l){a7=X;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break L12149}a7=X;a8=W;a9=V;ba=U;bb=T;bc=H1(a,0)|0;bd=R;break};case 35:{a7=X;a8=W;a9=V;ba=(U+1|0)%(Q|0)|0;bb=T;bc=S;bd=R;break};case 37:{a7=X;a8=c[W+4+(T<<2)>>2]|0;a9=-1;ba=U;bb=T;bc=S;bd=R;break};case 16:{aX=O;aY=X;aW=9048;break};case 7:case 14:{a7=X;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 0:{do{if(!v){aG=c[t>>2]|0;Y=c[w>>2]|0;ay=Y+4|0;if(ay>>>0<32){c[w>>2]=ay;c[x>>2]=c[x>>2]<<4|u&aG;break}else{Hw(y,c[x>>2]|0,Y);c[w>>2]=4;c[x>>2]=u&aG;break}}}while(0);a7=X+4|0;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};case 6:{do{if(!v){aG=c[w>>2]|0;Y=aG+1|0;if(Y>>>0<32){c[w>>2]=Y;c[x>>2]=c[x>>2]<<1;break}else{Hw(y,c[x>>2]|0,aG);c[w>>2]=1;c[x>>2]=0;break}}}while(0);a7=X+1|0;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R;break};default:{b5=2;aW=9053;break L12137}}}while(0);if((aW|0)==9048){aW=0;if((aX|0)==0){a7=aY;a8=W;a9=V;ba=U;bb=T;bc=S;bd=R}else{b5=aX;aW=9055;break}}aG=a9+1|0;if((c[(c[a8>>2]|0)+(aG<<2)>>2]|0)==38){q=0;r=a7;break L12128}else{R=bd;S=bc;T=bb;U=ba;V=aG;W=a8;X=a7}}if((aW|0)==9053){return b5|0}else if((aW|0)==9054){return b5|0}else if((aW|0)==9055){return b5|0}}}while(0);if((j|0)==0){b5=q;return b5|0}c[j>>2]=r;b5=q;return b5|0}function Ks(b,d,e,f,g,h,j,k){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0;l=i;i=i+40|0;m=l|0;n=l+8|0;o=l+32|0;p=HW(b)|0;q=p+40|0;r=p+8|0;if((c[q>>2]|0)==0){Hy(r,c[p+4>>2]|0,0);s=p|0}else{t=p|0;Hw(r,c[t>>2]|0,c[p+4>>2]|0);s=t}t=p+4|0;c[t>>2]=0;c[s>>2]=0;s=HA(r)|0;c[m>>2]=s;u=d+12|0;L12503:do{if((c[u>>2]|0)>0){v=n;w=n|0;x=(j&64|0)==0;y=p|0;z=0;A=s;while(1){B=d+16+(z*24|0)|0;c[v>>2]=c[B>>2];c[v+4>>2]=c[B+4>>2];c[v+8>>2]=c[B+8>>2];c[v+12>>2]=c[B+12>>2];c[v+16>>2]=c[B+16>>2];c[v+20>>2]=c[B+20>>2];B=c[w>>2]|0;if(!((B|0)==0|(B|0)==1|(B|0)==3)){C=16672;D=9123;break}B=e+(z<<2)|0;E=c[f+(z<<2)>>2]|0;F=Kr(b,n,(c[B>>2]|0)+2e3|0,E,E|0,j,h,k,0,0)|0;if((F|0)!=0){C=F;D=9125;break}do{if(x){F=c[B>>2]|0;if((c[F+12>>2]|0)>0){G=0;H=F}else{break}do{Kp(b,H+16+(G*12|0)|0,0,0,j,0,0)|0;G=G+1|0;H=c[B>>2]|0;}while((G|0)<(c[H+12>>2]|0))}}while(0);if((c[w>>2]|0)==4){I=A}else{if((c[q>>2]|0)==0){Hy(r,c[t>>2]|0,0)}else{Hw(r,c[y>>2]|0,c[t>>2]|0)}c[t>>2]=0;c[y>>2]=0;B=HA(r)|0;c[m>>2]=B-A+(c[m>>2]|0);I=B}B=z+1|0;if((B|0)<(c[u>>2]|0)){z=B;A=I}else{J=I;break L12503}}if((D|0)==9123){i=l;return C|0}else if((D|0)==9125){i=l;return C|0}}else{J=s}}while(0);I=(j&131072|0)==0;do{if((j&131136|0)==64){I_(o|0,6);H=c[u>>2]|0;if((j&16|0)==0){K=H}else{if((H|0)>0){L=0;M=H}else{break}while(1){H=e+(L<<2)|0;G=c[H>>2]|0;if((c[G+12>>2]|0)>0){k=0;h=G;while(1){if(((c[h+16+(k*12|0)>>2]|0)-13|0)>>>0<2){G=h+16+(k*12|0)|0;Kp(b,G,0,0,j,0,0)|0;a[o+L+k|0]=1;N=c[H>>2]|0}else{N=h}G=k+1|0;if((G|0)<(c[N+12>>2]|0)){k=G;h=N}else{break}}O=c[u>>2]|0}else{O=M}h=L+1|0;if((h|0)<(O|0)){L=h;M=O}else{K=O;break}}}if((K|0)>0){P=0;Q=K}else{break}while(1){h=e+(P<<2)|0;k=c[h>>2]|0;if((c[k+12>>2]|0)>0){H=0;G=k;while(1){if((a[o+P+H|0]|0)==0){k=G+16+(H*12|0)|0;Kp(b,k,0,0,j,0,0)|0;R=c[h>>2]|0}else{R=G}k=H+1|0;if((k|0)<(c[R+12>>2]|0)){H=k;G=R}else{break}}S=c[u>>2]|0}else{S=Q}G=P+1|0;if((G|0)<(S|0)){P=G;Q=S}else{break}}}}while(0);S=e+96|0;do{if(I){T=1;U=e+108|0}else{Q=c[S>>2]|0;P=0;while(1){if((P|0)>=(Q|0)){break}V=e+48+(P*12|0)|0;if(((c[V>>2]|0)-13|0)>>>0<2){D=9095;break}else{P=P+1|0}}if((D|0)==9095){IY(e+48+(Q*12|0)|0,V,12)}if((c[q>>2]|0)==0){Hy(r,c[t>>2]|0,0);W=p|0}else{P=p|0;Hw(r,c[P>>2]|0,c[t>>2]|0);W=P}c[t>>2]=0;c[W>>2]=0;P=(HA(r)|0)-s|0;u=e+108|0;R=P+(c[u>>2]|0)&7;if((R|0)!=(c[e+136>>2]|0)){C=16448;i=l;return C|0}P=c[t>>2]|0;o=P+R|0;if(o>>>0<32){c[t>>2]=o;o=p|0;c[o>>2]=c[o>>2]<<R;T=0;U=u;break}else{o=p|0;Hw(r,c[o>>2]|0,P);c[t>>2]=R;c[o>>2]=0;T=0;U=u;break}}}while(0);W=c[S>>2]|0;c[e+48+(W*12|0)>>2]=1;c[e+48+(W*12|0)+4>>2]=c[U>>2];U=(c[S>>2]|0)+1|0;c[S>>2]=U;if((U|0)>0){U=0;do{Kp(b,e+48+(U*12|0)|0,0,0,j,0,0)|0;U=U+1|0;}while((U|0)<(c[S>>2]|0)&(U|0)<4)}do{if((j&72|0)==0){U=c[497857]|0;S=c[t>>2]|0;W=S+3|0;if(W>>>0<32){c[t>>2]=W;W=p|0;c[W>>2]=c[W>>2]<<3|U&7;break}else{W=p|0;Hw(r,c[W>>2]|0,S);c[t>>2]=3;c[W>>2]=U&7;break}}}while(0);do{if((T|0)!=0){if((c[q>>2]|0)==0){Hy(r,c[t>>2]|0,0);X=p|0}else{j=p|0;Hw(r,c[j>>2]|0,c[t>>2]|0);X=j}c[t>>2]=0;c[X>>2]=0;j=s-(HA(r)|0)&7;if((j|0)!=(c[e+136>>2]|0)){C=16448;i=l;return C|0}U=c[t>>2]|0;W=U+j|0;if(W>>>0<32){c[t>>2]=W;W=p|0;c[W>>2]=c[W>>2]<<j;break}else{W=p|0;Hw(r,c[W>>2]|0,U);c[t>>2]=j;c[W>>2]=0;break}}}while(0);c[m>>2]=(c[m>>2]|0)-J;if((c[q>>2]|0)==0){Hy(r,c[t>>2]|0,0);Y=p|0}else{q=p|0;Hw(r,c[q>>2]|0,c[t>>2]|0);Y=q}c[t>>2]=0;c[Y>>2]=0;Y=HA(r)|0;c[m>>2]=(c[m>>2]|0)+Y;HZ(b,m)|0;C=(c[m>>2]|0)==((c[g>>2]|0)+(c[e+140>>2]|0)|0)?0:16448;i=l;return C|0}function Kt(a){a=a|0;var b=0,d=0;do{if((a|0)==0){b=-1}else{d=IQ(1,384)|0;if((d|0)==0){b=-1;break}I_(d,384);c[a>>2]=d;b=0}}while(0);return b|0}function Ku(a){a=a|0;var b=0,d=0;do{if((a|0)==0){b=-1}else{d=c[a>>2]|0;if((d|0)==0){b=0;break}IR(d);c[a>>2]=0;b=0}}while(0);return b|0}function Kv(b,d,e,f,g,h,j,k){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0.0,o=0.0,p=0.0,q=0,r=0;l=i;i=i+160|0;m=l|0;n=+(f|0)*23437500203726813.0e-21*.0009765625;o=n*2147483648.0;do{if(n<0.0){p=o+-.5;if(p<=-2147483648.0){q=-2147483648;break}q=~~p}else{p=o+.5;if(p>=2147483647.0){q=2147483647;break}q=~~p}}while(0);c[b+244>>2]=q;c[b+152>>2]=f;c[b+156>>2]=g;c[b+160>>2]=h;a[b+164|0]=k;if((Kw(b,d,e)|0)!=0){r=-1;i=l;return r|0}if((Hl(h,j,m)|0)!=0){r=-2;i=l;return r|0}j=b+176|0;NM(j|0,-1|0,36);switch(h|0){case 5:{c[j>>2]=c[m+52>>2];c[b+180>>2]=c[m+56>>2];c[b+184>>2]=c[m+28>>2];c[b+192>>2]=c[m+76>>2];c[b+196>>2]=c[m+80>>2];break};case 3:{c[j>>2]=c[m+52>>2];c[b+180>>2]=c[m+56>>2];c[b+184>>2]=c[m+28>>2];break};case 1:{c[b+184>>2]=c[m+28>>2];break};case 4:{c[j>>2]=c[m+52>>2];c[b+180>>2]=c[m+56>>2];c[b+184>>2]=c[m+28>>2];c[b+200>>2]=c[m+76>>2];break};case 2:{c[j>>2]=c[m+28>>2];c[b+180>>2]=c[m+32>>2];break};case 7:{c[j>>2]=c[m+52>>2];c[b+180>>2]=c[m+56>>2];c[b+184>>2]=c[m+28>>2];c[b+188>>2]=c[m+124>>2];c[b+192>>2]=c[m+76>>2];c[b+196>>2]=c[m+80>>2];c[b+204>>2]=c[m+100>>2];c[b+208>>2]=c[m+104>>2];break};case 6:{c[j>>2]=c[m+52>>2];c[b+180>>2]=c[m+56>>2];c[b+184>>2]=c[m+28>>2];c[b+188>>2]=c[m+100>>2];c[b+192>>2]=c[m+76>>2];c[b+196>>2]=c[m+80>>2];break};default:{r=-1;i=l;return r|0}}c[b+172>>2]=c[m+8>>2];c[b+168>>2]=c[m+4>>2];c[b+216>>2]=-283115520;c[b+212>>2]=-283115520;I_(b+220|0,8);I_(b+228|0,8);I_(b+236|0,8);I_(b+248|0,8);I_(b+256|0,128);r=0;i=l;return r|0}function Kw(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;c[a+144>>2]=b;c[a+148>>2]=d;d=a+156|0;e=a+152|0;f=0;g=b;L12634:while(1){switch(g|0){case 4:{h=3;break};case 6:{h=5;break};case 5:{h=4;break};case 0:case 1:{h=0;break};case 2:{h=1;break};case 3:{h=2;break};default:{i=-1;j=9167;break L12634}}c[a+(f<<2)>>2]=c[1925992+(h<<2)>>2];c[a+8+(f<<2)>>2]=c[1927544+(h<<2)>>2];b=c[1926016+(h<<2)>>2]|0;c[a+16+(f<<2)>>2]=b;k=c[1914528+(h<<2)>>2]|0;c[a+24+(f<<2)>>2]=k;c[a+32+(f<<2)>>2]=c[1926680+(h<<2)>>2];c[a+40+(f<<2)>>2]=c[1926424+(h<<2)>>2];l=c[1922184+(h<<2)>>2]|0;c[a+48+(f<<2)>>2]=l;c[a+56+(f<<2)>>2]=c[1927520+(h<<2)>>2];c[a+64+(f<<2)>>2]=c[1927472+(h<<2)>>2];c[a+72+(f<<2)>>2]=c[1914504+(h<<2)>>2];m=k-b|0;N2(l,(l|0)<0|0?-1:0,m,(m|0)<0|0?-1:0)|0;c[a+80+(f<<2)>>2]=-(G<<1|0>>>31);c[a+88+(f<<2)>>2]=Kx(c[1926400+(h<<2)>>2]|0,c[d>>2]|0,c[e>>2]|0)|0;c[a+96+(f<<2)>>2]=Kx(c[1926704+(h<<2)>>2]|0,c[d>>2]|0,c[e>>2]|0)|0;c[a+104+(f<<2)>>2]=Kx(c[1926376+(h<<2)>>2]|0,c[d>>2]|0,c[e>>2]|0)|0;c[a+112+(f<<2)>>2]=Kx(c[1926656+(h<<2)>>2]|0,c[d>>2]|0,c[e>>2]|0)|0;c[a+120+(f<<2)>>2]=(c[1928408+(h<<2)>>2]<<8|0)/(c[e>>2]|0)|0;c[a+128+(f<<2)>>2]=c[1926728+(h<<2)>>2];c[a+136+(f<<2)>>2]=c[1927496+(h<<2)>>2];c[a+220+(f<<2)>>2]=0;m=f+1|0;if((m|0)>=2){i=0;j=9166;break}f=m;g=c[a+144+(m<<2)>>2]|0}if((j|0)==9166){return i|0}else if((j|0)==9167){return i|0}return 0}function Kx(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0;e=i;i=i+8|0;f=e|0;g=HQ(b<<11,d<<11,f)|0;d=c[f>>2]|0;b=d-10|0;if((b|0)>0){h=g<<b}else{h=g>>10-d}d=HP(a,h,f)|0;h=c[f>>2]|0;a=h+10|0;if((a|0)>0){j=d<<a}else{j=d>>-10-h}h=HQ(2097152,j,f)|0;j=c[f>>2]|0;d=j-10|0;if((d|0)>0){k=h<<d}else{k=h>>10-j}N2(k,(k|0)<0|0?-1:0,1549081984,0)|0;k=HI(-(G<<2|0>>>30)|0,10,f)|0;j=c[f>>2]|0;if((j|0)>0){l=k<<j;m=2147483647-l|0;i=e;return m|0}else{l=k>>-j;m=2147483647-l|0;i=e;return m|0}return 0}function Ky(d,f,g,h,i,j,k,l,m){d=d|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0,cb=0,cc=0,cd=0,ce=0,cf=0,cg=0,ch=0,ci=0,cj=0,ck=0,cl=0,cm=0,cn=0,co=0,cp=0,cq=0,cr=0,cs=0,ct=0,cu=0,cv=0,cw=0,cx=0,cy=0,cz=0,cA=0,cB=0,cC=0,cD=0,cE=0,cF=0,cG=0,cH=0,cI=0,cJ=0,cK=0,cL=0,cM=0,cN=0,cO=0,cP=0,cQ=0,cR=0,cS=0,cT=0,cU=0,cV=0,cW=0,cX=0,cY=0,cZ=0;do{if((c[d+144>>2]|0)==0){if((c[d+148>>2]|0)!=0){n=9185;break}c[d+220>>2]=0;c[d+224>>2]=0;o=d+152|0}else{n=9185}}while(0);if((n|0)==9185){p=d+152|0;q=c[p>>2]|0;r=(q|0)>32?32:q;s=a[d+164|0]|0;t=1;while(1){u=t+1|0;if(1<<t>>>0<r>>>0){t=u}else{break}}v=d+168|0;w=c[v>>2]|0;x=(w|0)>0;do{if(s<<24>>24==0){if(!x){y=31;z=0;break}A=c[d+188>>2]|0;B=t-1|0;C=~q;D=(C|0)>-33?C:-33;E=0;F=31;H=0;while(1){if((H|0)==(A|0)){I=F;J=E}else{K=0;L=F;M=E;while(1){N=K+r|0;O=(N|0)<(q|0)?N:q;if((K|0)<(O|0)){N=D-K|0;P=~((N|0)>(C|0)?N:C);N=K+1|0;Q=(N|0)<(P|0);R=0;S=K;do{T=b[f+((ac(w,S)|0)+H<<1)>>1]|0;R=((ac(T,T)|0)>>>(B>>>0))+R|0;S=S+1|0;}while((S|0)<(O|0));U=R;V=Q?P:N}else{U=0;V=K}O=(U|0)>0?U:-U|0;if((O|0)>-1){S=~O;O=0;while(1){T=O+1|0;W=S<<1;if((W|0)<0){S=W;O=T}else{X=T;break}}}else{X=0}O=(M|0)>0?M:-M|0;if((O|0)>-1){S=~O;O=0;while(1){N=O+1|0;P=S<<1;if((P|0)<0){S=P;O=N}else{Y=N;break}}}else{Y=0}O=u-X|0;S=1-Y+L|0;N=(O|0)>(S|0)?O:S;if((U|0)==0){Z=N}else{Z=N+((M|0)!=0)|0}N=t-Z|0;if((N|0)>0){_=U<<N}else{_=U>>-N}N=L-Z|0;if((N|0)>0){$=M<<N}else{$=M>>-N}N=$+_|0;S=(N|0)!=0?Z:31;if((V|0)<(q|0)){K=V;L=S;M=N}else{I=S;J=N;break}}}M=H+1|0;if((M|0)<(w|0)){E=J;F=I;H=M}else{y=I;z=J;break}}}else{if(!x){y=31;z=0;break}H=d+188|0;F=t+3|0;E=t+4|0;B=t-2|0;C=~q;D=(C|0)>-33?C:-33;C=0;A=31;M=0;L=w;while(1){if((M|0)==(c[H>>2]|0)){aa=A;ab=C;ad=L}else{K=d+256+(M<<4)|0;N=d+256+(M<<4)+4|0;S=d+256+(M<<4)+8|0;O=d+256+(M<<4)+12|0;P=c[p>>2]|0;Q=0;R=c[K>>2]|0;T=c[N>>2]|0;W=c[S>>2]|0;ae=c[O>>2]|0;af=A;ag=C;while(1){ah=Q+r|0;ai=(ah|0)<(P|0)?ah:P;if((Q|0)<(ai|0)){ah=D-Q|0;aj=~P;ak=~((ah|0)>(aj|0)?ah:aj);aj=Q+1|0;ah=(aj|0)<(ak|0);al=0;am=ae;an=W;ao=T;ap=R;aq=Q;while(1){ar=e[f+((ac(L,aq)|0)+M<<1)>>1]<<16>>2;as=ar-ao|0;N2(as,(as|0)<0|0?-1:0,1139254272,0)|0;as=G<<1|0>>>31;N2(an,(an|0)<0|0?-1:0,-2045220096,-1)|0;at=as-(G<<1|0>>>31)|0;N2(am,(am|0)<0|0?-1:0,-48293384,-1)|0;au=at-(G<<1|0>>>31)|0;at=au;as=(au|0)<0|0?-1:0;N2(at,as,at,as)|0;av=(G>>>(B>>>0))+al|0;as=aq+1|0;if((as|0)<(ai|0)){al=av;am=an;an=au;ao=ap;ap=ar;aq=as}else{break}}aw=av;ax=an;ay=au;az=ap;aA=ar;aB=ah?ak:aj}else{aw=0;ax=ae;ay=W;az=T;aA=R;aB=Q}aq=(aw|0)>0?aw:-aw|0;if((aq|0)>-1){ao=~aq;aq=0;while(1){am=aq+1|0;al=ao<<1;if((al|0)<0){ao=al;aq=am}else{aC=am;break}}}else{aC=0}aq=(ag|0)>0?ag:-ag|0;if((aq|0)>-1){ao=~aq;aq=0;while(1){aj=aq+1|0;ak=ao<<1;if((ak|0)<0){ao=ak;aq=aj}else{aD=aj;break}}}else{aD=0}aq=E-aC|0;ao=1-aD+af|0;aj=(aq|0)>(ao|0)?aq:ao;if((aw|0)==0){aE=aj}else{aE=aj+((ag|0)!=0)|0}aj=F-aE|0;if((aj|0)>0){aF=aw<<aj}else{aF=aw>>-aj}aj=af-aE|0;if((aj|0)>0){aG=ag<<aj}else{aG=ag>>-aj}aH=aG+aF|0;aI=(aH|0)!=0?aE:31;if((aB|0)<(P|0)){Q=aB;R=aA;T=az;W=ay;ae=ax;af=aI;ag=aH}else{break}}c[K>>2]=aA;c[N>>2]=az;c[S>>2]=ay;c[O>>2]=ax;aa=aI;ab=aH;ad=c[v>>2]|0}ag=M+1|0;if((ag|0)<(ad|0)){C=ab;A=aa;M=ag;L=ad}else{y=aa;z=ab;break}}}}while(0);ab=(y<<25)-402653184+(HE(z)|0)|0;z=ab-(HE(c[p>>2]<<19)|0)|0;N2(z,(z|0)<0|0?-1:0,646457024,0)|0;ab=(G<<1|0>>>31)+10066330|0;N2(ab,(ab|0)<0|0?-1:0,1342177280,0)|0;ab=-65011712-(g<<5)+((z|0)>-1114654080?G<<1|0>>>31:-203423750)|0;z=ab;y=(ab|0)<0|0?-1:0;aa=0;while(1){do{if((c[d+144+(aa<<2)>>2]|0)==0){c[d+220+(aa<<2)>>2]=0}else{do{if((ab|0)>(c[d+(aa<<2)>>2]|0)){ad=c[d+8+(aa<<2)>>2]|0;if((ab|0)<(ad|0)){v=ab-ad|0;ad=c[d+40+(aa<<2)>>2]|0;aH=v;aI=(v|0)<0|0?-1:0;v=ad;ax=(ad|0)<0|0?-1:0;N2(v,ax,aH,aI)|0;aJ=G<<1|0>>>31;break}aI=c[d+16+(aa<<2)>>2]|0;if((ab|0)<=(aI|0)){aJ=0;break}aH=c[d+24+(aa<<2)>>2]|0;if((ab|0)<=(aH|0)){ax=ab-aI|0;aI=c[d+48+(aa<<2)>>2]|0;v=ax;ad=(ax|0)<0|0?-1:0;ax=aI;ay=(aI|0)<0|0?-1:0;N2(ax,ay,v,ad)|0;aJ=G<<1|0>>>31;break}if((ab|0)<(c[d+32+(aa<<2)>>2]|0)){ad=ab-aH|0;aH=c[d+56+(aa<<2)>>2]|0;v=ad;ay=(ad|0)<0|0?-1:0;ad=aH;ax=(aH|0)<0|0?-1:0;N2(ad,ax,v,ay)|0;aJ=(G<<1|0>>>31)-(c[d+80+(aa<<2)>>2]|0)|0;break}else{aJ=-(c[d+72+(aa<<2)>>2]|0)|0;break}}else{aJ=c[d+64+(aa<<2)>>2]|0}}while(0);O=d+212+(aa<<2)|0;S=c[O>>2]|0;N=ab-S|0;K=d+220+(aa<<2)|0;ay=c[K>>2]|0;v=(aJ|0)<(ay|0);do{if(v){if((N|0)>(c[d+128+(aa<<2)>>2]|0)){aK=d+88+(aa<<2)|0;n=9256;break}else{aL=c[d+104+(aa<<2)>>2]|0;n=9257;break}}else{if((N|0)<(-(c[d+136+(aa<<2)>>2]|0)|0)){aK=d+96+(aa<<2)|0;n=9256;break}else{aM=c[d+112+(aa<<2)>>2]|0;n=9258;break}}}while(0);if((n|0)==9256){n=0;N=c[aK>>2]|0;if(v){aL=N;n=9257}else{aM=N;n=9258}}do{if((n|0)==9258){n=0;N=c[d+228+(aa<<2)>>2]|0;if((N|0)==0){aN=0;aO=aM;n=9260;break}aP=ay;aQ=N;aR=d+228+(aa<<2)|0;n=9261}else if((n|0)==9257){n=0;aN=c[d+228+(aa<<2)>>2]|0;aO=aL;n=9260}}while(0);if((n|0)==9260){n=0;v=2147483647-aO|0;N=v;ax=(v|0)<0|0?-1:0;v=S;ad=(S|0)<0|0?-1:0;N2(v,ad,N,ax)|0;ad=G<<1|0>>>31;v=aO;aH=(aO|0)<0|0?-1:0;N2(v,aH,z,y)|0;c[O>>2]=ad+(G<<1|0>>>31);ad=ay;aI=(ay|0)<0|0?-1:0;N2(ad,aI,N,ax)|0;ax=G<<1|0>>>31;N=aJ;aI=(aJ|0)<0|0?-1:0;N2(v,aH,N,aI)|0;aI=ax+(G<<1|0>>>31)|0;c[K>>2]=aI;ax=d+228+(aa<<2)|0;if((aN|0)==0){aS=aI;aT=ax}else{aP=aI;aQ=aN;aR=ax;n=9261}}if((n|0)==9261){n=0;c[aR>>2]=aQ-1;aS=aP;aT=aR}if((aJ|0)>=(aS|0)){break}c[aT>>2]=c[d+120+(aa<<2)>>2]}}while(0);ax=aa+1|0;if((ax|0)<2){aa=ax}else{o=p;break}}}p=c[o>>2]|0;if((p|0)<=0){aU=0;aV=0;aW=d+248|0;aX=c[aW>>2]|0;c[aW>>2]=aU;aY=(aU|0)>(aX|0);aZ=aY?aU:aX;a_=HE(aZ)|0;a$=a_+100663296|0;a0=(a$|0)>-668792448;a1=a$;a2=(a$|0)<0|0?-1:0;a3=1292914048;a4=0;a5=N2(a1,a2,a3,a4)|0;a6=G;a7=a6;a8=0;a9=0<<1|0>>>31;ba=a7<<1|0>>>31;bb=ba;bc=(ba|0)<0|0?-1:0;bd=1342177280;be=0;bf=N2(bb,bc,bd,be)|0;bg=G;bh=bg;bi=0;bj=bh<<1|0>>>31;bk=bi<<1|bh>>>31;bl=bj;bm=bl;bn=bm+419430|0;bo=a0?bn:-251238818;bp=d+220|0;bq=c[bp>>2]|0;br=bo+bq|0;bs=d+252|0;bt=c[bs>>2]|0;c[bs>>2]=aV;bu=(aV|0)>(bt|0);bv=bu?aV:bt;bw=HE(bv)|0;bx=bw+100663296|0;by=(bx|0)>-668792448;bz=bx;bA=(bx|0)<0|0?-1:0;bB=1292914048;bC=0;bD=N2(bz,bA,bB,bC)|0;bE=G;bF=bE;bG=0;bH=0<<1|0>>>31;bI=bF<<1|0>>>31;bJ=bI;bK=(bI|0)<0|0?-1:0;bL=1342177280;bM=0;bN=N2(bJ,bK,bL,bM)|0;bO=G;bP=bO;bQ=0;bR=bP<<1|0>>>31;bS=bQ<<1|bP>>>31;bT=bR;bU=bT;bV=bU+419430|0;bW=by?bV:-251238818;bX=d+224|0;bY=c[bX>>2]|0;bZ=bW+bY|0;b_=g-h|0;b$=b_<<5;b0=g-i|0;b1=b0<<5;b2=d+244|0;b3=c[b2>>2]|0;b4=d+236|0;b5=c[b4>>2]|0;b6=b5+b3|0;b7=b$-br|0;b8=(b6|0)<(b7|0);b9=b8?b6:b7;c[b4>>2]=b9;ca=b3<<1;cb=d+240|0;cc=c[cb>>2]|0;cd=cc+ca|0;ce=b1-bZ|0;cf=(cd|0)<(ce|0);cg=cf?cd:ce;c[cb>>2]=cg;ch=c[bp>>2]|0;ci=(b9|0)<0;cj=ci?b9:0;ck=cj+ch|0;cl=ck>>5;c[l>>2]=cl;cm=c[bX>>2]|0;cn=c[cb>>2]|0;co=(cn|0)<0;cp=co?cn:0;cq=cp+cm|0;cr=cq>>5;c[m>>2]=cr;return 0}o=c[d+168>>2]|0;aa=(o|0)>0;aT=c[d+172>>2]|0;aS=aT>>>0>2;aJ=d+192|0;aR=d+204|0;aP=d+196|0;aQ=d+208|0;n=d+200|0;aN=d+184|0;y=d+176|0;z=d+180|0;aO=k;aL=(k|0)<0|0?-1:0;k=j;aM=(j|0)<0|0?-1:0;j=aT>>>0>1;aT=0;aK=0;ab=0;while(1){ax=ac(o,aT)|0;if(aa){aI=0;N=0;do{aH=b[f+(N+ax<<1)>>1]|0;v=aH<<16>>16>0?aH:-aH&65535;aI=aI<<16>>16>v<<16>>16?aI:v;N=N+1|0;}while((N|0)<(o|0));cs=(aI&65535)<<16>>3}else{cs=0}N=(aK|0)>(cs|0)?aK:cs;if(aS){v=c[aJ>>2]|0;aH=(v|0)>-1;if(aH){ad=e[f+(v+ax<<1)>>1]<<16;az=ad;aA=(ad|0)<0|0?-1:0;ad=1518270976;aB=0;N2(az,aA,ad,aB)|0;ct=-(G>>2)|0}else{ct=0}aB=c[aR>>2]|0;ad=(aB|0)>-1;if(ad){aA=e[f+(aB+ax<<1)>>1]<<16;az=aA;aE=(aA|0)<0|0?-1:0;aA=1518270976;aF=0;N2(az,aE,aA,aF)|0;cu=ct-(G>>2)|0}else{cu=ct}aF=c[aP>>2]|0;aA=(aF|0)>-1;if(aA){aE=e[f+(aF+ax<<1)>>1]<<16;az=aE;aG=(aE|0)<0|0?-1:0;aE=1518270976;aw=0;N2(az,aG,aE,aw)|0;cv=cu-(G>>2)|0}else{cv=cu}aw=c[aQ>>2]|0;aE=(aw|0)>-1;if(aE){aG=e[f+(aw+ax<<1)>>1]<<16;az=aG;aD=(aG|0)<0|0?-1:0;aG=1518270976;aC=0;N2(az,aD,aG,aC)|0;cw=cv-(G>>2)|0}else{cw=cv}if((v|aB|0)<0){cx=cw}else{aC=cw;aG=(cw|0)<0|0?-1:0;aD=1518270976;az=0;N2(aC,aG,aD,az)|0;cx=G<<1|0>>>31}az=c[n>>2]|0;aD=(az|0)>-1;if(aD){aG=e[f+(az+ax<<1)>>1]<<16;aC=aG;ar=(aG|0)<0|0?-1:0;aG=1518270976;au=0;N2(aC,ar,aG,au)|0;cy=cx-(G>>2)|0}else{cy=cx}au=c[aN>>2]|0;aG=(au|0)>-1;if(aG){ar=e[f+(au+ax<<1)>>1]<<16;aC=ar;av=(ar|0)<0|0?-1:0;ar=1518270976;r=0;N2(aC,av,ar,r)|0;cz=(G>>2)+cy|0}else{cz=cy}r=e[f+((c[y>>2]|0)+ax<<1)>>1]<<16>>3;ar=r+cz|0;av=(ar|0)>0?ar:-ar|0;ar=(N|0)>(av|0)?N:av;if(aH){av=e[f+(v+ax<<1)>>1]<<16;aC=av;w=(av|0)<0|0?-1:0;av=1518270976;q=0;N2(aC,w,av,q)|0;cA=G>>2}else{cA=0}if(ad){q=e[f+(aB+ax<<1)>>1]<<16;av=q;w=(q|0)<0|0?-1:0;q=1518270976;aC=0;N2(av,w,q,aC)|0;cB=(G>>2)+cA|0}else{cB=cA}if(aA){aC=e[f+(aF+ax<<1)>>1]<<16;q=aC;w=(aC|0)<0|0?-1:0;aC=1518270976;av=0;N2(q,w,aC,av)|0;cC=(G>>2)+cB|0}else{cC=cB}if(aE){av=e[f+(aw+ax<<1)>>1]<<16;aC=av;w=(av|0)<0|0?-1:0;av=1518270976;q=0;N2(aC,w,av,q)|0;cD=(G>>2)+cC|0}else{cD=cC}if((aF|aw|0)<0){cE=cD}else{q=cD;av=(cD|0)<0|0?-1:0;w=1518270976;aC=0;N2(q,av,w,aC)|0;cE=G<<1|0>>>31}if(aD){aC=e[f+(az+ax<<1)>>1]<<16;w=aC;av=(aC|0)<0|0?-1:0;aC=1518270976;q=0;N2(w,av,aC,q)|0;cF=(G>>2)+cE|0}else{cF=cE}if(aG){q=e[f+(au+ax<<1)>>1]<<16;aC=q;av=(q|0)<0|0?-1:0;q=1518270976;w=0;N2(aC,av,q,w)|0;cG=(G>>2)+cF|0}else{cG=cF}w=e[f+((c[z>>2]|0)+ax<<1)>>1]<<16>>3;q=w+cG|0;av=(q|0)>0?q:-q|0;q=(ar|0)>(av|0)?ar:av;if(aH){aH=e[f+(v+ax<<1)>>1]<<16;av=aH;ar=(aH|0)<0|0?-1:0;N2(av,ar,aO,aL)|0;cH=G>>2}else{cH=0}if(ad){ad=e[f+(aB+ax<<1)>>1]<<16;ar=ad;av=(ad|0)<0|0?-1:0;N2(ar,av,aO,aL)|0;cI=(G>>2)+cH|0}else{cI=cH}if((v|aB|0)<0){cJ=cI}else{aB=cI;v=(cI|0)<0|0?-1:0;av=1518270976;ar=0;N2(aB,v,av,ar)|0;cJ=G<<1|0>>>31}if(aD){ar=e[f+(az+ax<<1)>>1]<<16;av=ar;v=(ar|0)<0|0?-1:0;ar=1503238528;aB=0;N2(av,v,ar,aB)|0;aB=G<<1|0>>>31;ar=aB;v=(aB|0)<0|0?-1:0;N2(ar,v,aO,aL)|0;cK=(G>>2)+cJ|0}else{cK=cJ}if(aG){v=e[f+(au+ax<<1)>>1]<<16;ar=v;aB=(v|0)<0|0?-1:0;N2(ar,aB,k,aM)|0;cL=(G>>2)+cK|0}else{cL=cK}aB=r+cL|0;r=(aB|0)>0?aB:-aB|0;aB=(q|0)>(r|0)?q:r;if(aA){aA=e[f+(aF+ax<<1)>>1]<<16;r=aA;q=(aA|0)<0|0?-1:0;N2(r,q,aO,aL)|0;cM=G>>2}else{cM=0}if(aE){aE=e[f+(aw+ax<<1)>>1]<<16;q=aE;r=(aE|0)<0|0?-1:0;N2(q,r,aO,aL)|0;cN=(G>>2)+cM|0}else{cN=cM}if((aF|aw|0)<0){cO=cN}else{aw=cN;aF=(cN|0)<0|0?-1:0;r=1518270976;q=0;N2(aw,aF,r,q)|0;cO=G<<1|0>>>31}if(aD){aD=e[f+(az+ax<<1)>>1]<<16;az=aD;q=(aD|0)<0|0?-1:0;aD=1503238528;r=0;N2(az,q,aD,r)|0;r=G<<1|0>>>31;aD=r;q=(r|0)<0|0?-1:0;N2(aD,q,aO,aL)|0;cP=(G>>2)+cO|0}else{cP=cO}if(aG){aG=e[f+(au+ax<<1)>>1]<<16;au=aG;q=(aG|0)<0|0?-1:0;N2(au,q,k,aM)|0;cQ=(G>>2)+cP|0}else{cQ=cP}q=w+cQ|0;w=(q|0)>0?q:-q|0;cR=(aB|0)>(w|0)?aB:w}else{cR=N}N=(cR|0)>(ab|0)?cR:ab;if(j){w=c[aJ>>2]|0;if((w|0)>-1){aB=e[f+(w+ax<<1)>>1]<<16;q=aB;au=(aB|0)<0|0?-1:0;N2(q,au,aO,aL)|0;cS=G>>2}else{cS=0}au=c[aR>>2]|0;if((au|0)>-1){q=e[f+(au+ax<<1)>>1]<<16;aB=q;aG=(q|0)<0|0?-1:0;N2(aB,aG,aO,aL)|0;cT=(G>>2)+cS|0}else{cT=cS}aG=c[aP>>2]|0;if((aG|0)>-1){aB=e[f+(aG+ax<<1)>>1]<<16;aG=aB;q=(aB|0)<0|0?-1:0;N2(aG,q,aO,aL)|0;cU=(G>>2)+cT|0}else{cU=cT}q=c[aQ>>2]|0;if((q|0)>-1){aG=e[f+(q+ax<<1)>>1]<<16;q=aG;aB=(aG|0)<0|0?-1:0;N2(q,aB,aO,aL)|0;cV=(G>>2)+cU|0}else{cV=cU}if((w|au|0)<0){cW=cV}else{au=cV;w=(cV|0)<0|0?-1:0;aB=1518270976;q=0;N2(au,w,aB,q)|0;cW=G<<1|0>>>31}q=c[n>>2]|0;if((q|0)>-1){aB=e[f+(q+ax<<1)>>1]<<16;q=aB;w=(aB|0)<0|0?-1:0;aB=1503238528;au=0;N2(q,w,aB,au)|0;au=G<<1|0>>>31;aB=au;w=(au|0)<0|0?-1:0;N2(aB,w,aO,aL)|0;cX=(G>>2)+cW|0}else{cX=cW}w=c[aN>>2]|0;if((w|0)>-1){aB=e[f+(w+ax<<1)>>1]<<16;w=aB;au=(aB|0)<0|0?-1:0;N2(w,au,k,aM)|0;cY=(G>>2)+cX|0}else{cY=cX}au=(e[f+((c[y>>2]|0)+ax<<1)>>1]<<16>>3)+cY+(e[f+((c[z>>2]|0)+ax<<1)>>1]<<16>>3)|0;w=(au|0)>0?au:-au|0;cZ=(N|0)>(w|0)?N:w}else{cZ=N}N=aT+1|0;if((N|0)<(p|0)){aT=N;aK=cR;ab=cZ}else{aU=cR;aV=cZ;break}}aW=d+248|0;aX=c[aW>>2]|0;c[aW>>2]=aU;aY=(aU|0)>(aX|0);aZ=aY?aU:aX;a_=HE(aZ)|0;a$=a_+100663296|0;a0=(a$|0)>-668792448;a1=a$;a2=(a$|0)<0|0?-1:0;a3=1292914048;a4=0;a5=N2(a1,a2,a3,a4)|0;a6=G;a7=a6;a8=0;a9=0<<1|0>>>31;ba=a7<<1|0>>>31;bb=ba;bc=(ba|0)<0|0?-1:0;bd=1342177280;be=0;bf=N2(bb,bc,bd,be)|0;bg=G;bh=bg;bi=0;bj=bh<<1|0>>>31;bk=bi<<1|bh>>>31;bl=bj;bm=bl;bn=bm+419430|0;bo=a0?bn:-251238818;bp=d+220|0;bq=c[bp>>2]|0;br=bo+bq|0;bs=d+252|0;bt=c[bs>>2]|0;c[bs>>2]=aV;bu=(aV|0)>(bt|0);bv=bu?aV:bt;bw=HE(bv)|0;bx=bw+100663296|0;by=(bx|0)>-668792448;bz=bx;bA=(bx|0)<0|0?-1:0;bB=1292914048;bC=0;bD=N2(bz,bA,bB,bC)|0;bE=G;bF=bE;bG=0;bH=0<<1|0>>>31;bI=bF<<1|0>>>31;bJ=bI;bK=(bI|0)<0|0?-1:0;bL=1342177280;bM=0;bN=N2(bJ,bK,bL,bM)|0;bO=G;bP=bO;bQ=0;bR=bP<<1|0>>>31;bS=bQ<<1|bP>>>31;bT=bR;bU=bT;bV=bU+419430|0;bW=by?bV:-251238818;bX=d+224|0;bY=c[bX>>2]|0;bZ=bW+bY|0;b_=g-h|0;b$=b_<<5;b0=g-i|0;b1=b0<<5;b2=d+244|0;b3=c[b2>>2]|0;b4=d+236|0;b5=c[b4>>2]|0;b6=b5+b3|0;b7=b$-br|0;b8=(b6|0)<(b7|0);b9=b8?b6:b7;c[b4>>2]=b9;ca=b3<<1;cb=d+240|0;cc=c[cb>>2]|0;cd=cc+ca|0;ce=b1-bZ|0;cf=(cd|0)<(ce|0);cg=cf?cd:ce;c[cb>>2]=cg;ch=c[bp>>2]|0;ci=(b9|0)<0;cj=ci?b9:0;ck=cj+ch|0;cl=ck>>5;c[l>>2]=cl;cm=c[bX>>2]|0;cn=c[cb>>2]|0;co=(cn|0)<0;cp=co?cn:0;cq=cp+cm|0;cr=cq>>5;c[m>>2]=cr;return 0}function Kz(a){a=a|0;return c[a+144>>2]|0}function KA(a){a=a|0;return c[a+148>>2]|0}function KB(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0;f=JN(0)|0;c[a>>2]=f;L12903:do{if((f|0)!=0){g=0;while(1){if((g|0)>=(b|0)){h=0;break}i=JL(g)|0;c[f+3688+(g<<2)>>2]=i;if((i|0)==0){break L12903}else{g=g+1|0}}while(1){if((h|0)>=(d|0)){break}g=JP(h)|0;i=f+3712+(h<<2)|0;c[i>>2]=g;if((g|0)==0){break L12903}g=JR(h)|0;c[c[i>>2]>>2]=g;if((c[c[i>>2]>>2]|0)==0){break L12903}else{h=h+1|0}}c[f+3736>>2]=JT(0,e)|0;j=0;return j|0}}while(0);KC(a,0);j=12576;return j|0}function KC(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;do{if((a|0)!=0){d=c[a>>2]|0;if((d|0)==0){break}e=d+3712|0;f=c[e>>2]|0;if((f|0)!=0){g=f|0;if((c[g>>2]|0)!=0){JS(g)}JQ(e)}e=d+3716|0;g=c[e>>2]|0;if((g|0)!=0){f=g|0;if((c[f>>2]|0)!=0){JS(f)}JQ(e)}e=d+3720|0;f=c[e>>2]|0;if((f|0)!=0){g=f|0;if((c[g>>2]|0)!=0){JS(g)}JQ(e)}e=d+3724|0;g=c[e>>2]|0;if((g|0)!=0){f=g|0;if((c[f>>2]|0)!=0){JS(f)}JQ(e)}e=d+3728|0;f=c[e>>2]|0;if((f|0)!=0){g=f|0;if((c[g>>2]|0)!=0){JS(g)}JQ(e)}e=d+3732|0;g=c[e>>2]|0;if((g|0)!=0){f=g|0;if((c[f>>2]|0)!=0){JS(f)}JQ(e)}e=d+3688|0;if((c[e>>2]|0)!=0){JM(e)}e=d+3692|0;if((c[e>>2]|0)!=0){JM(e)}e=d+3696|0;if((c[e>>2]|0)!=0){JM(e)}e=d+3700|0;if((c[e>>2]|0)!=0){JM(e)}e=d+3704|0;if((c[e>>2]|0)!=0){JM(e)}e=d+3708|0;if((c[e>>2]|0)!=0){JM(e)}JO(a)}}while(0);if((b|0)==0){return}a=c[b>>2]|0;if((a|0)==0){return}e=a+24|0;if((c[e>>2]|0)==0){h=a}else{J$(e);h=c[b>>2]|0}e=h+28|0;if((c[e>>2]|0)==0){i=h}else{J$(e);i=c[b>>2]|0}e=i+32|0;if((c[e>>2]|0)==0){j=i}else{J$(e);j=c[b>>2]|0}e=j+36|0;if((c[e>>2]|0)==0){k=j}else{J$(e);k=c[b>>2]|0}e=k+40|0;if((c[e>>2]|0)==0){l=k}else{J$(e);l=c[b>>2]|0}e=l+44|0;if((c[e>>2]|0)==0){m=l}else{J$(e);m=c[b>>2]|0}e=m|0;if((c[e>>2]|0)==0){n=m}else{JZ(e);n=c[b>>2]|0}e=n+4|0;if((c[e>>2]|0)==0){o=n}else{JZ(e);o=c[b>>2]|0}e=o+8|0;if((c[e>>2]|0)==0){p=o}else{JZ(e);p=c[b>>2]|0}e=p+12|0;if((c[e>>2]|0)==0){q=p}else{JZ(e);q=c[b>>2]|0}e=q+16|0;if((c[e>>2]|0)==0){r=q}else{JZ(e);r=c[b>>2]|0}e=r+20|0;if((c[e>>2]|0)!=0){JZ(e)}JX(b);return}function KD(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;if((e|0)<=0){g=0;return g|0}f=(b|0)>0;L13021:do{if((d|0)>0){h=0;i=0;j=0;while(1){k=JW(h)|0;l=a+(h<<2)|0;c[l>>2]=k;if((k|0)==0){break L13021}else{m=0;n=j}while(1){k=J_(n)|0;c[(c[l>>2]|0)+24+(m<<2)>>2]=k;k=m+1|0;if((k|0)<(d|0)){m=k;n=n+1|0}else{break}}k=j+d|0;if(f){o=0;p=i;while(1){q=JY(p)|0;c[(c[l>>2]|0)+(o<<2)>>2]=q;if((c[(c[l>>2]|0)+(o<<2)>>2]|0)==0){break L13021}q=p+1|0;r=o+1|0;if((r|0)<(b|0)){o=r;p=q}else{s=q;break}}}else{s=i}p=h+1|0;if((p|0)<(e|0)){h=p;i=s;j=k}else{g=0;break}}return g|0}else{if(f){t=0;u=0}else{j=0;while(1){i=JW(j)|0;c[a+(j<<2)>>2]=i;if((i|0)==0){break L13021}i=j+1|0;if((i|0)<(e|0)){j=i}else{g=0;break}}return g|0}while(1){j=JW(t)|0;i=a+(t<<2)|0;c[i>>2]=j;if((j|0)==0){break L13021}else{v=0;w=u}do{j=JY(w)|0;c[(c[i>>2]|0)+(v<<2)>>2]=j;if((c[(c[i>>2]|0)+(v<<2)>>2]|0)==0){break L13021}w=w+1|0;v=v+1|0;}while((v|0)<(b|0));i=t+1|0;if((i|0)<(e|0)){t=i;u=w}else{g=0;break}}return g|0}}while(0);KC(0,a);g=12576;return g|0}function KE(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;do{if((e|0)>2){if((c[g+4>>2]|0)!=2){h=0;break}i=c[a+3712>>2]|0;I_(c[i>>2]|0,4096);if((f|0)==23){j=1}else{j=(f|0)==39|0}M5(i+4100|0,j);h=1}else{h=0}}while(0);j=(e|0)==2?0:3;i=g+12|0;k=c[i>>2]|0;if((k|0)>0){l=a+3712+(e-1<<2)|0;e=(f|0)==23;m=(f|0)==39|0;f=0;n=h;h=k;while(1){o=g+16+(f*24|0)+8|0;if((c[o>>2]|0)>0){p=g+16+(f*24|0)|0;q=a+3688+(f<<2)|0;r=0;s=n;do{if((c[p>>2]|0)==3){c[(c[q>>2]|0)+(r<<2)>>2]=c[l>>2];c[(c[(c[q>>2]|0)+(r<<2)>>2]|0)+4520>>2]=1}else{c[(c[q>>2]|0)+(r<<2)>>2]=c[a+3712+(s<<2)>>2];if((s|0)>=(j|0)){t=c[(c[q>>2]|0)+(r<<2)>>2]|0;I_(c[t>>2]|0,4096);M5(t+4100|0,e?1:m)}c[(c[(c[q>>2]|0)+(r<<2)>>2]|0)+4520>>2]=0}s=s+1|0;r=r+1|0;}while((r|0)<(c[o>>2]|0));u=s;v=c[i>>2]|0}else{u=n;v=h}o=f+1|0;if((o|0)<(v|0)){f=o;n=u;h=v}else{w=v;break}}}else{w=k}if((d|0)>0){x=0;y=w}else{return 0}while(1){if((y|0)>0){w=b+(x<<2)|0;k=0;v=0;h=y;while(1){u=g+16+(k*24|0)+8|0;if((c[u>>2]|0)>0){n=0;f=v;while(1){z=f+1|0;m=c[w>>2]|0;c[(c[m+(k<<2)>>2]|0)+(n<<2)>>2]=c[m+24+(f<<2)>>2];m=n+1|0;if((m|0)<(c[u>>2]|0)){n=m;f=z}else{break}}A=z;B=c[i>>2]|0}else{A=v;B=h}f=k+1|0;if((f|0)<(B|0)){k=f;v=A;h=B}else{C=B;break}}}else{C=y}h=x+1|0;if((h|0)<(d|0)){x=h;y=C}else{break}}return 0}function KF(a,b,d,e,f,g,h,i,j,k,l,m){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;n=c[d+8>>2]|0;o=Hn(c[d>>2]|0)|0;p=(o-1|0)>>>0<2?o:0;if((b|0)==23){q=1}else if((b|0)==39){q=2}else{q=0}o=a+3740|0;c[o>>2]=f;r=(g|0)/(n|0)|0;s=a|0;t=Nn(r,e,i,0,f,k,s,q)|0;if((t|0)!=0){u=t;return u|0}t=(ac(p,g)|0)/(n|0)|0;n=l>>>13&1;l=Mz(t,e,p,0,c[o>>2]|0,n,a+1468|0,s,h&2,h&8)|0;if((l|0)!=0){u=l;return u|0}do{if((f|0)>512){l=a+1844|0;s=Nn(r,e,i,2,c[o>>2]|0,k,l,q)|0;if((s|0)!=0){u=s;return u|0}s=Mz(t,e,p,2,c[o>>2]|0,n,a+3312|0,l,h&1,h&4)|0;if((s|0)==0){break}else{u=s}return u|0}}while(0);h=d+12|0;n=c[h>>2]|0;if((n|0)>0){o=(m|0)==0;m=a|0;p=a+224|0;t=(b|0)==23;q=(b|0)==39|0;b=0;k=n;while(1){n=d+16+(b*24|0)+8|0;if((c[n>>2]|0)>0){i=a+3688+(b<<2)|0;f=0;do{if(!o){s=c[(c[i>>2]|0)+(f<<2)>>2]|0;I_(c[s>>2]|0,4096);M5(s+4100|0,t?1:q)}s=c[(c[i>>2]|0)+(f<<2)>>2]|0;Nl(s+4308|0,s+4516|0,c[m>>2]|0,p,s+4512|0);f=f+1|0;}while((f|0)<(c[n>>2]|0));v=c[h>>2]|0}else{v=k}n=b+1|0;if((n|0)<(v|0)){b=n;k=v}else{w=m;break}}}else{w=a|0}m=Mt(a+1676|0,r,e,j,c[w>>2]|0,a+12|0,c[d+24>>2]|0,(c[a+220>>2]|0)==0|0)|0;if((m|0)!=0){u=m;return u|0}u=Mt(a+3520|0,r,e,j,c[a+1844>>2]|0,a+1856|0,c[d+48>>2]|0,(c[a+2064>>2]|0)==0|0)|0;return u|0}function KG(a,d,e,f,g,h,j,k){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0;l=i;i=i+2424|0;m=l|0;n=l+8|0;o=l+16|0;p=l+224|0;q=l+232|0;r=l+240|0;s=l+248|0;t=l+256|0;u=l+264|0;v=l+272|0;w=l+280|0;x=l+288|0;y=l+296|0;z=l+304|0;A=l+312|0;B=l+320|0;C=l+328|0;D=l+336|0;E=l+2384|0;F=l+2416|0;H=F;I=f+1844|0;J=g|0;K=d|0;L=f+220|0;M=c[L>>2]|0;N=f+1836|0;O=c[N>>2]|0;if((M|0)==0){P=((O*9|0|0)/16|0)+O|0}else{P=O}O=(a|0)>0;if(O){Q=0;do{R=e+(Q*3380|0)|0;c[p+(Q<<2)>>2]=R;c[q+(Q<<2)>>2]=e+6760+(Q*556|0);c[r+(Q<<2)>>2]=e+7872+(Q*600|0);c[R>>2]=c[(c[g+(Q<<2)>>2]|0)+2096>>2];Q=Q+1|0;}while((Q|0)<(a|0));S=c[L>>2]|0}else{S=M}do{if((S|0)==2){if(O){T=0}else{break}while(1){M=c[c[d+(T<<2)>>2]>>2]|0;Q=c[j+(T<<2)>>2]|0;e=c[N>>2]|0;if((e|0)>0){R=0;do{b[M+(R+P<<1)>>1]=b[h+((ac(R,k)|0)+Q<<1)>>1]|0;R=R+1|0;}while((R|0)<(e|0))}e=T+1|0;if((e|0)<(a|0)){T=e}else{U=9515;break}}}else{if(O){e=D|0;R=0;do{Q=d+(R<<2)|0;c[(c[Q>>2]|0)+4100>>2]=e;M=c[Q>>2]|0;V=c[M+4100>>2]|0;W=c[j+(R<<2)>>2]|0;X=c[N>>2]|0;if((X|0)>0){Y=0;do{b[V+(Y<<1)>>1]=b[h+((ac(Y,k)|0)+W<<1)>>1]|0;Y=Y+1|0;}while((Y|0)<(X|0));Z=c[Q>>2]|0;_=c[N>>2]|0}else{Z=M;_=X}M6(Z+4100|0,_,c[Z+4520>>2]|0)|0;Y=c[Q>>2]|0;IY((c[Y>>2]|0)+(P<<1)|0,c[Y+4100>>2]|0,(c[N>>2]<<1)-P<<1);R=R+1|0;}while((R|0)<(a|0))}if((M7((c[K>>2]|0)+4100|0,(c[d+4>>2]|0)+4100|0,a,1)|0)==0){U=9515;break}else{$=12288}i=l;return $|0}}while(0);L13155:do{if((U|0)==9515){if(O){aa=0}else{break}do{c[C+(aa<<2)>>2]=(c[(c[d+(aa<<2)>>2]|0)+4104>>2]|0)==2;aa=aa+1|0;}while((aa|0)<(a|0));if(O){ab=0}else{break}do{Z=s+(ab<<2)|0;if((c[C+(ab<<2)>>2]|0)==0){c[Z>>2]=f;c[t+(ab<<2)>>2]=c[N>>2];c[u+(ab<<2)>>2]=1;c[v+(ab<<2)>>2]=60;_=c[p+(ab<<2)>>2]|0;c[w+(ab<<2)>>2]=_+1444;c[x+(ab<<2)>>2]=_+484;c[y+(ab<<2)>>2]=_+2644;c[z+(ab<<2)>>2]=_+964;c[A+(ab<<2)>>2]=_+1924;c[B+(ab<<2)>>2]=_+4}else{c[Z>>2]=I;c[t+(ab<<2)>>2]=(c[N>>2]|0)/8|0;c[u+(ab<<2)>>2]=8;c[v+(ab<<2)>>2]=15;Z=c[p+(ab<<2)>>2]|0;c[w+(ab<<2)>>2]=Z+1444;c[x+(ab<<2)>>2]=Z+484;c[y+(ab<<2)>>2]=Z+2644;c[z+(ab<<2)>>2]=Z+964;c[A+(ab<<2)>>2]=Z+1924;c[B+(ab<<2)>>2]=Z+4}ab=ab+1|0;}while((ab|0)<(a|0));if(O){ad=1;ae=0}else{break}while(1){Z=d+(ae<<2)|0;_=c[s+(ae<<2)>>2]|0;if((c[(c[Z>>2]|0)+4520>>2]|0)==0){D=p+(ae<<2)|0;c[(c[D>>2]|0)+3372>>2]=c[_+4>>2];c[(c[D>>2]|0)+3376>>2]=c[_+436>>2]}else{D=p+(ae<<2)|0;c[(c[D>>2]|0)+3372>>2]=c[_+8>>2];c[(c[D>>2]|0)+3376>>2]=c[_+440>>2]}D=c[u+(ae<<2)>>2]|0;if((D|0)>0){T=c[t+(ae<<2)>>2]|0;S=p+(ae<<2)|0;R=_+220|0;_=ad;e=0;Y=c[S>>2]|0;while(1){W=ac(T,e)|0;V=c[Z>>2]|0;K1((c[V>>2]|0)+(W<<1)|0,(c[Y>>2]|0)+(W<<2)|0,c[V+4104>>2]|0,c[V+4108>>2]|0,V+4112|0,c[N>>2]|0,n,c[R>>2]|0,V+4|0)|0;V=c[S>>2]|0;af=c[V+3376>>2]|0;I_((c[V>>2]|0)+(af+W<<2)|0,T-af<<2);if((c[L>>2]|0)!=0){af=c[S>>2]|0;V=(c[af>>2]|0)+(W-6+(c[af+3376>>2]|0)<<2)|0;af=c[V>>2]|0;ag=af;ah=(af|0)<0|0?-1:0;af=1840644096;ai=0;N2(ag,ah,af,ai)|0;c[V>>2]=G<<1|0>>>31;V=c[S>>2]|0;ai=(c[V>>2]|0)+(W-5+(c[V+3376>>2]|0)<<2)|0;V=c[ai>>2]|0;af=V;ah=(V|0)<0|0?-1:0;V=1533870080;ag=0;N2(af,ah,V,ag)|0;c[ai>>2]=G<<1|0>>>31;ai=c[S>>2]|0;ag=(c[ai>>2]|0)+(W-4+(c[ai+3376>>2]|0)<<2)|0;ai=c[ag>>2]|0;V=ai;ah=(ai|0)<0|0?-1:0;ai=1227096064;af=0;N2(V,ah,ai,af)|0;c[ag>>2]=G<<1|0>>>31;ag=c[S>>2]|0;af=(c[ag>>2]|0)+(W-3+(c[ag+3376>>2]|0)<<2)|0;ag=c[af>>2]|0;ai=ag;ah=(ag|0)<0|0?-1:0;ag=920322048;V=0;N2(ai,ah,ag,V)|0;c[af>>2]=G<<1|0>>>31;af=c[S>>2]|0;V=(c[af>>2]|0)+(W-2+(c[af+3376>>2]|0)<<2)|0;af=c[V>>2]|0;ag=af;ah=(af|0)<0|0?-1:0;af=613548032;ai=0;N2(ag,ah,af,ai)|0;c[V>>2]=G<<1|0>>>31;V=c[S>>2]|0;ai=(c[V>>2]|0)+(W-1+(c[V+3376>>2]|0)<<2)|0;V=c[ai>>2]|0;af=V;ah=(V|0)<0|0?-1:0;V=306774016;ag=0;N2(af,ah,V,ag)|0;c[ai>>2]=G<<1|0>>>31}ai=c[S>>2]|0;ag=c[ai+3376>>2]|0;V=ai|0;L13180:do{if((_|0)==1){ah=0;while(1){if((ah|0)>=(ag|0)){aj=1;break L13180}if((c[(c[V>>2]|0)+(ah+W<<2)>>2]|0)==0){ah=ah+1|0}else{aj=0;break}}}else{aj=_}}while(0);W=e+1|0;if((W|0)<(D|0)){_=aj;e=W;Y=ai}else{ak=aj;al=ai;break}}}else{ak=ad;al=c[p+(ae<<2)>>2]|0}c[al+3124>>2]=c[n>>2];Y=c[c[Z>>2]>>2]|0;e=c[N>>2]|0;IZ(Y,Y+(e<<1)|0,e<<1);e=c[c[Z>>2]>>2]|0;Y=c[N>>2]|0;_=(Y<<1)-P|0;D=c[j+(ae<<2)>>2]|0;S=P-Y|0;if((S|0)>0){T=0;do{b[e+(T+Y<<1)>>1]=b[h+((ac(T+_|0,k)|0)+D<<1)>>1]|0;T=T+1|0;}while((T|0)<(S|0))}S=ae+1|0;if((S|0)<(a|0)){ad=ak;ae=S}else{break}}if((ak|0)!=0){if(O){am=0}else{break}while(1){S=p+(am<<2)|0;c[(c[S>>2]|0)+3124>>2]=0;T=c[u+(am<<2)>>2]|0;if((T|0)>0){D=w+(am<<2)|0;_=v+(am<<2)|0;Y=x+(am<<2)|0;e=z+(am<<2)|0;Z=B+(am<<2)|0;R=0;Q=c[S>>2]|0;while(1){if((c[Q+3372>>2]|0)>0){X=c[D>>2]|0;M=ac(c[_>>2]|0,R)|0;W=c[Y>>2]|0;V=c[e>>2]|0;ag=c[Z>>2]|0;ah=0;while(1){af=M+ah|0;c[X+(af<<2)>>2]=0;c[W+(af<<2)>>2]=0;c[V+(af<<2)>>2]=-2147483648;c[ag+(af<<2)>>2]=0;af=ah+1|0;an=c[S>>2]|0;if((af|0)<(c[an+3372>>2]|0)){ah=af}else{ao=an;break}}}else{ao=Q}ah=R+1|0;if((ah|0)<(T|0)){R=ah;Q=ao}else{break}}}am=am+1|0;if((am|0)>=(a|0)){break L13155}}}do{if(O){Q=31;R=0;while(1){T=c[u+(R<<2)>>2]|0;if((T|0)>0){S=c[t+(R<<2)>>2]|0;Z=p+(R<<2)|0;e=(c[s+(R<<2)>>2]|0)+12|0;Y=c[w+(R<<2)>>2]|0;_=c[v+(R<<2)>>2]|0;D=c[Z>>2]|0;ah=Q;ag=0;V=D;W=c[D+3372>>2]|0;while(1){D=ac(S,ag)|0;X=(c[V>>2]|0)+(D<<2)|0;D=ac(_,ag)|0;MR(X,e,Y+(D<<2)|0,W);X=c[Z>>2]|0;M=c[X+3372>>2]|0;if((M|0)>0){ai=ah;an=0;while(1){af=c[Y+(D+an<<2)>>2]|0;ap=(ai|0)<(af|0)?ai:af;af=an+1|0;if((af|0)<(M|0)){ai=ap;an=af}else{aq=ap;break}}}else{aq=ah}an=ag+1|0;if((an|0)<(T|0)){ah=aq;ag=an;V=X;W=M}else{ar=aq;break}}}else{ar=Q}W=R+1|0;if((W|0)<(a|0)){Q=ar;R=W}else{break}}if(!O){as=31;at=ar;break}R=ar-4|0;Q=0;W=0;while(1){V=c[u+(W<<2)>>2]|0;if((V|0)>0){ag=c[t+(W<<2)>>2]|0;ah=p+(W<<2)|0;T=c[w+(W<<2)>>2]|0;Y=c[v+(W<<2)>>2]|0;Z=(c[s+(W<<2)>>2]|0)+12|0;e=c[x+(W<<2)>>2]|0;_=c[z+(W<<2)>>2]|0;S=Q;an=0;while(1){ai=ac(ag,an)|0;D=c[ah>>2]|0;ap=(c[D>>2]|0)+(ai<<2)|0;ai=ac(Y,an)|0;af=MS(ap,T+(ai<<2)|0,Z,c[D+3372>>2]|0,e+(ai<<2)|0,_+(ai<<2)|0,R)|0;ai=(S|0)>(af|0)?S:af;af=an+1|0;if((af|0)<(V|0)){S=ai;an=af}else{au=ai;break}}}else{au=Q}an=W+1|0;if((an|0)<(a|0)){Q=au;W=an}else{break}}if((au|0)==0){as=31;at=ar;break}W=au>>31^au;if((W|0)>-1){Q=~W;W=0;while(1){R=Q<<1;if((R|0)<0){Q=R;W=W+1|0}else{av=W;break}}}else{av=-1}as=ar-4+(av>>1)|0;at=ar}else{as=31;at=31}}while(0);W=(((c[C>>2]|0)!=0)<<31>>31)+as|0;Q=(at|0)<(W|0)?at:W;W=(c[(c[p>>2]|0)+3124>>2]|0)+3|0;R=(Q|0)>(W|0)?W:Q;Q=R<<26;if(O){W=0;do{an=c[u+(W<<2)>>2]|0;if((an|0)>0){S=p+(W<<2)|0;V=w+(W<<2)|0;_=v+(W<<2)|0;e=x+(W<<2)|0;Z=B+(W<<2)|0;T=z+(W<<2)|0;Y=0;ah=c[S>>2]|0;while(1){if((c[ah+3372>>2]|0)>0){ag=c[V>>2]|0;ai=ac(c[_>>2]|0,Y)|0;af=c[e>>2]|0;D=c[Z>>2]|0;ap=c[T>>2]|0;aw=0;while(1){ax=ai+aw|0;ay=(c[ag+(ax<<2)>>2]|0)-4|0;az=((ay|0)<0?0:ay)-R<<1;ay=(az|0)<31?az:31;if((ay|0)>-1){az=af+(ax<<2)|0;c[az>>2]=c[az>>2]>>ay}else{az=af+(ax<<2)|0;c[az>>2]=c[az>>2]<<-ay}ay=c[af+(ax<<2)>>2]|0;N2(ay,(ay|0)<0|0?-1:0,43256336,0)|0;c[D+(ax<<2)>>2]=G<<1|0>>>31;ay=ap+(ax<<2)|0;c[ay>>2]=(c[ay>>2]|0)+Q;ay=aw+1|0;ax=c[S>>2]|0;if((ay|0)<(c[ax+3372>>2]|0)){aw=ay}else{aA=ax;break}}}else{aA=ah}aw=Y+1|0;if((aw|0)<(an|0)){Y=aw;ah=aA}else{break}}}W=W+1|0;}while((W|0)<(a|0))}if((R|0)==0|O^1){break}else{aB=0}do{W=c[u+(aB<<2)>>2]|0;if((W|0)>0){Q=p+(aB<<2)|0;ah=w+(aB<<2)|0;Y=v+(aB<<2)|0;an=c[t+(aB<<2)>>2]|0;S=0;T=c[Q>>2]|0;while(1){Z=ac(an,S)|0;if((c[T+3376>>2]|0)>0){e=0;_=T;while(1){V=(c[_>>2]|0)+(e+Z<<2)|0;c[V>>2]=c[V>>2]<<R;V=e+1|0;aw=c[Q>>2]|0;if((V|0)<(c[aw+3376>>2]|0)){e=V;_=aw}else{aC=aw;break}}}else{aC=T}if((c[aC+3372>>2]|0)>0){_=c[ah>>2]|0;e=c[Y>>2]|0;Z=0;while(1){aw=_+((ac(e,S)|0)+Z<<2)|0;c[aw>>2]=(c[aw>>2]|0)-R;aw=Z+1|0;V=c[Q>>2]|0;if((aw|0)<(c[V+3372>>2]|0)){Z=aw}else{aD=V;break}}}else{aD=aC}Z=S+1|0;if((Z|0)<(W|0)){S=Z;T=aD}else{aE=aD;break}}}else{aE=c[p+(aB<<2)>>2]|0}T=aE+3124|0;c[T>>2]=(c[T>>2]|0)-R;aB=aB+1|0;}while((aB|0)<(a|0))}}while(0);do{if((c[(c[K>>2]|0)+4520>>2]|0)==0){if(O){aB=0;do{if((c[C+(aB<<2)>>2]|0)==0){aE=c[p+(aB<<2)>>2]|0;aD=c[s+(aB<<2)>>2]|0;K0(c[aE>>2]|0,c[w+(aB<<2)>>2]|0,c[z+(aB<<2)>>2]|0,o+(aB*102|0)|0,c[aE+3372>>2]|0,aD+12|0,c[aD+1832>>2]|0)}aB=aB+1|0;}while((aB|0)<(a|0))}if((c[f+1512>>2]|0)==0){if((c[f+3356>>2]|0)==0){U=9593;break}}c[F>>2]=0;c[F+4>>2]=0;do{if(O){aB=0;do{aD=c[u+(aB<<2)>>2]|0;if((aD|0)>0){aE=c[t+(aB<<2)>>2]|0;aC=c[q+(aB<<2)>>2]|0;aA=c[s+(aB<<2)>>2]|0;at=aA+1468|0;as=g+(aB<<2)|0;ar=aA|0;aA=p+(aB<<2)|0;av=d+(aB<<2)|0;au=0;do{aq=ac(aE,au)|0;MB(aC,at,(c[as>>2]|0)+288|0,c[ar>>2]|0,(c[c[aA>>2]>>2]|0)+(aq<<2)|0,au,c[(c[av>>2]|0)+4104>>2]|0)|0;au=au+1|0;}while((au|0)<(aD|0))}aB=aB+1|0;}while((aB|0)<(a|0));if((a|0)!=2){aF=1;break}ME(c[q+4>>2]|0,c[q>>2]|0,(c[g+4>>2]|0)+288|0,(c[J>>2]|0)+288|0,c[(c[d+4>>2]|0)+4104>>2]|0,c[(c[K>>2]|0)+4104>>2]|0,(c[s+4>>2]|0)+1468|0);aF=0}else{aF=1}}while(0);aB=c[u>>2]|0;if((aB|0)>0){aD=(c[C>>2]|0)==0;au=c[q>>2]|0;av=au+4|0;aA=c[q+4>>2]|0;ar=aA+4|0;as=0;do{if(aD){at=(c[av>>2]|0)!=0;if(at|aF){aG=at}else{aG=(c[ar>>2]|0)!=0}c[E+(as<<2)>>2]=aG&1}else{at=(c[au+4+(as<<3)>>2]|0)!=0;if(at|aF){aH=at}else{aH=(c[aA+4+(as<<3)>>2]|0)!=0}c[E+(as<<2)>>2]=aH&1}as=as+1|0;}while((as|0)<(aB|0))}if(!O){aI=0;break}as=E|0;aA=0;do{do{if((c[as>>2]|0)!=0){if((c[C+(aA<<2)>>2]|0)!=0){break}au=(c[s+(aA<<2)>>2]|0)+436|0;ar=p+(aA<<2)|0;if((c[au>>2]|0)>0){av=0;do{aD=(c[c[ar>>2]>>2]|0)+(av<<2)|0;c[aD>>2]=c[aD>>2]>>1;av=av+1|0;}while((av|0)<(c[au>>2]|0))}au=c[ar>>2]|0;if((c[au+3372>>2]|0)>0){av=c[B+(aA<<2)>>2]|0;aD=0;while(1){at=av+(aD<<2)|0;c[at>>2]=c[at>>2]>>2;at=aD+1|0;aC=c[ar>>2]|0;if((at|0)<(c[aC+3372>>2]|0)){aD=at}else{aJ=aC;break}}}else{aJ=au}aD=aJ+3124|0;c[aD>>2]=(c[aD>>2]|0)+1}}while(0);aA=aA+1|0;}while((aA|0)<(a|0));if(O){aK=0;aL=aB}else{aI=0;break}while(1){if((aL|0)>0){aA=c[t+(aK<<2)>>2]|0;as=g+(aK<<2)|0;aD=c[q+(aK<<2)>>2]|0;ar=c[s+(aK<<2)>>2]|0;av=ar|0;aC=ar+1468|0;at=p+(aK<<2)|0;aE=d+(aK<<2)|0;aq=w+(aK<<2)|0;am=v+(aK<<2)|0;ao=0;do{ak=ac(aA,ao)|0;ae=c[at>>2]|0;MF((c[as>>2]|0)+288|0,aD,c[av>>2]|0,aC,c[ar+12+(c[ae+3372>>2]<<2)>>2]|0,(c[ae>>2]|0)+(ak<<2)|0,ao,c[(c[aE>>2]|0)+4104>>2]|0)|0;if((c[E+(ao<<2)>>2]|0)!=0){ae=c[at>>2]|0;ad=(c[ae>>2]|0)+(ak<<2)|0;ak=c[aq>>2]|0;k=ak+((ac(c[am>>2]|0,ao)|0)<<2)|0;MR(ad,ar+12|0,k,c[ae+3372>>2]|0)}ao=ao+1|0;}while((ao|0)<(aL|0))}ao=aK+1|0;if((ao|0)>=(a|0)){break}aK=ao;aL=c[u+(ao<<2)>>2]|0}if(O){aM=0;aN=0;aO=aB}else{aI=0;break}while(1){if((aO|0)>0){ao=C+(aN<<2)|0;ar=p+(aN<<2)|0;am=w+(aN<<2)|0;aq=s+(aN<<2)|0;at=x+(aN<<2)|0;aE=z+(aN<<2)|0;aC=H+(aN<<2)|0;av=t+(aN<<2)|0;aD=v+(aN<<2)|0;as=aM;aA=0;while(1){do{if((c[E+(aA<<2)>>2]|0)==0){aP=as}else{ae=c[ar>>2]|0;k=c[ae>>2]|0;if((c[ao>>2]|0)==0){ad=MT(k,c[am>>2]|0,(c[aq>>2]|0)+12|0,c[ae+3372>>2]|0,c[at>>2]|0,c[aE>>2]|0)|0;c[aC>>2]=ad;aP=(as|0)>(ad|0)?as:ad;break}else{ad=k+((ac(c[av>>2]|0,aA)|0)<<2)|0;k=c[am>>2]|0;ak=ac(c[aD>>2]|0,aA)|0;MU(ad,k+(ak<<2)|0,(c[aq>>2]|0)+12|0,c[ae+3372>>2]|0,(c[at>>2]|0)+(ak<<2)|0);aP=as;break}}}while(0);au=aA+1|0;if((au|0)<(aO|0)){as=aP;aA=au}else{aQ=aP;break}}}else{aQ=aM}aA=aN+1|0;if((aA|0)>=(a|0)){break}aM=aQ;aN=aA;aO=c[u+(aA<<2)>>2]|0}if(!O){aI=0;break}aB=(aQ|0)==0;aA=aQ<<1;as=0;while(1){do{if(!aB){if((c[C+(as<<2)>>2]|0)!=0){break}at=(c[s+(as<<2)>>2]|0)+436|0;aq=p+(as<<2)|0;if((c[at>>2]|0)>0){aD=0;do{am=(c[c[aq>>2]>>2]|0)+(aD<<2)|0;c[am>>2]=c[am>>2]>>aQ;aD=aD+1|0;}while((aD|0)<(c[at>>2]|0))}at=aQ-(c[H+(as<<2)>>2]|0)|0;aD=at<<1;am=c[aq>>2]|0;if((c[am+3372>>2]|0)>0){av=at<<26;at=c[z+(as<<2)>>2]|0;aC=c[x+(as<<2)>>2]|0;aE=c[B+(as<<2)>>2]|0;ao=0;while(1){ar=at+(ao<<2)|0;c[ar>>2]=(c[ar>>2]|0)-av;ar=aC+(ao<<2)|0;c[ar>>2]=c[ar>>2]>>aD;ar=aE+(ao<<2)|0;c[ar>>2]=c[ar>>2]>>aA;ar=ao+1|0;au=c[aq>>2]|0;if((ar|0)<(c[au+3372>>2]|0)){ao=ar}else{aR=au;break}}}else{aR=am}ao=aR+3124|0;c[ao>>2]=(c[ao>>2]|0)+aQ}}while(0);ao=as+1|0;if((ao|0)<(a|0)){as=ao}else{U=9593;break}}}else{c[(c[q>>2]|0)+4>>2]=0;U=9593}}while(0);do{if((U|0)==9593){if(O){aS=0}else{aI=0;break}do{aQ=p+(aS<<2)|0;aR=c[aQ>>2]|0;z=c[aR+3124>>2]<<1;H=z-4|0;do{if((H|0)>-1){aT=c[(c[s+(aS<<2)>>2]|0)+444>>2]>>H}else{if((H|0)<(((c[C+(aS<<2)>>2]|0)==0?0:-6)|0)){aT=2147483647;break}aT=c[(c[s+(aS<<2)>>2]|0)+444>>2]<<4-z}}while(0);H=c[u+(aS<<2)>>2]|0;L13380:do{if((H|0)>0){aO=s+(aS<<2)|0;aN=B+(aS<<2)|0;aM=v+(aS<<2)|0;aP=d+(aS<<2)|0;E=y+(aS<<2)|0;aL=x+(aS<<2)|0;aK=z;aJ=0;aH=aR;while(1){aF=c[aH+3372>>2]|0;aG=c[aN>>2]|0;F=c[aM>>2]|0;if((aF|0)>0){as=ac(F,aJ)|0;aA=0;while(1){aB=aG+(as+aA<<2)|0;ao=c[aB>>2]|0;c[aB>>2]=(ao|0)<(aT|0)?ao:aT;ao=aA+1|0;aB=c[(c[aQ>>2]|0)+3372>>2]|0;if((ao|0)<(aB|0)){aA=ao}else{aU=aB;break}}}else{aU=aF}aA=c[aO>>2]|0;as=ac(F,aJ)|0;M=aG+(as<<2)|0;K$(aU,aA+448|0,aA+652|0,M);X=aK+16|0;do{if((X|0)>-1){aB=(X|0)>31?31:X;ao=c[aQ>>2]|0;if((c[ao+3372>>2]|0)>0){aV=0}else{aW=aB;aX=ao;break}while(1){ao=aG+(as+aV<<2)|0;aq=c[ao>>2]>>4;aE=c[aA+224+(aV<<2)>>2]>>aB;c[ao>>2]=(aq|0)>(aE|0)?aq:aE;aE=aV+1|0;aq=c[aQ>>2]|0;if((aE|0)<(c[aq+3372>>2]|0)){aV=aE}else{aW=aB;aX=aq;break}}}else{aB=-16-aK|0;aq=(aB|0)>31?31:aB;aB=c[aQ>>2]|0;if((c[aB+3372>>2]|0)>0){aY=0}else{aW=aq;aX=aB;break}while(1){aB=aG+(as+aY<<2)|0;aE=c[aB>>2]>>4;ao=c[aA+224+(aY<<2)>>2]<<aq;c[aB>>2]=(aE|0)>(ao|0)?aE:ao;ao=aY+1|0;aE=c[aQ>>2]|0;if((ao|0)<(c[aE+3372>>2]|0)){aY=ao}else{aW=aq;aX=aE;break}}}}while(0);aG=c[aP>>2]|0;do{if((c[aG+4520>>2]|0)==0){if((c[aG+4104>>2]|0)==3){if((c[aX+3372>>2]|0)>0){X=0;F=aG;while(1){c[F+4308+(X<<2)>>2]=2147483647;aF=X+1|0;aq=c[aP>>2]|0;if((aF|0)<(c[(c[aQ>>2]|0)+3372>>2]|0)){X=aF;F=aq}else{aZ=aq;break}}}else{aZ=aG}c[aZ+4512>>2]=0;c[(c[aP>>2]|0)+4516>>2]=0;a_=c[aP>>2]|0;a$=c[aQ>>2]|0}else{a_=aG;a$=aX}Nm(a_+4308|0,c[a_+4516>>2]|0,c[a$+3372>>2]|0,c[aA+428>>2]|0,b[aA+432>>1]|0,M,c[a$+3124>>2]|0,a_+4512|0);c[(c[aP>>2]|0)+4516>>2]=1;F=c[aP>>2]|0;if((c[F+4104>>2]|0)!=1){break}if((c[(c[aQ>>2]|0)+3372>>2]|0)>0){X=0;aq=F;while(1){c[aq+4308+(X<<2)>>2]=2147483647;aF=X+1|0;aE=c[aP>>2]|0;if((aF|0)<(c[(c[aQ>>2]|0)+3372>>2]|0)){X=aF;aq=aE}else{a0=aE;break}}}else{a0=F}c[a0+4512>>2]=0;c[(c[aP>>2]|0)+4516>>2]=0}}while(0);M=(c[E>>2]|0)+(as<<2)|0;IY(M,(c[aL>>2]|0)+(as<<2)|0,c[(c[aQ>>2]|0)+3372>>2]<<2);K$(c[(c[aQ>>2]|0)+3372>>2]|0,aA+856|0,aA+1060|0,M);M=aJ+1|0;if((M|0)>=(H|0)){break L13380}aK=aW;aJ=M;aH=c[aQ>>2]|0}}}while(0);aS=aS+1|0;}while((aS|0)<(a|0));aQ=(a|0)==2;if(!aQ){aI=0;break}H=c[u+4>>2]|0;if((H|0)<=0){aI=1;break}aR=c[t+4>>2]|0;z=p|0;aH=p+4|0;aJ=c[w>>2]|0;aK=c[v>>2]|0;aL=c[w+4>>2]|0;E=c[v+4>>2]|0;aP=(c[s+4>>2]|0)+12|0;aO=c[A>>2]|0;aM=c[A+4>>2]|0;aN=d+4|0;am=0;while(1){M=ac(aR,am)|0;aG=c[z>>2]|0;aq=(c[aG>>2]|0)+(M<<2)|0;X=c[aH>>2]|0;aE=(c[X>>2]|0)+(M<<2)|0;M=ac(aK,am)|0;aF=ac(E,am)|0;MV(aq,aE,aJ+(M<<2)|0,aL+(aF<<2)|0,aP,c[aG+3372>>2]|0,aO+(M<<2)|0,aM+(aF<<2)|0,(c[(c[aN>>2]|0)+4104>>2]|0)!=2|0,aG+2404|0,X+2404|0);X=am+1|0;if((X|0)<(H|0)){am=X}else{aI=aQ;break}}}}while(0);do{if(O){A=f+12|0;s=f+1264|0;v=I|0;w=f+1856|0;t=f+3108|0;u=f+3680|0;aS=0;do{do{if((c[C+(aS<<2)>>2]|0)==0){aW=p+(aS<<2)|0;a0=c[aW>>2]|0;a_=a0+3372|0;a$=c[a_>>2]|0;while(1){aX=a$-1|0;if((a$|0)<=0){break}aZ=c[f+12+(aX<<2)>>2]|0;aY=c[f+12+(a$<<2)>>2]|0;do{aY=aY-1|0;if((aY|0)<(aZ|0)){break}}while((c[(c[a0>>2]|0)+(aY<<2)>>2]|0)==0);if((aY|0)>(aZ|0)){break}else{a$=aX}}aA=m+(aS<<2)|0;c[aA>>2]=a$;as=c[a_>>2]|0;aV=(as|0)>5?5:as;c[aA>>2]=(aV|0)>(a$|0)?aV:a$;aV=g+(aS<<2)|0;IY(c[(c[aV>>2]|0)+2116>>2]|0,a0+964|0,c[a_>>2]<<2);IY((c[aV>>2]|0)+24|0,A,244);IY(c[(c[aV>>2]|0)+2112>>2]|0,s,c[(c[aW>>2]|0)+3372>>2]<<2);aA=c[aW>>2]|0;HG(aA+4|0,c[(c[aV>>2]|0)+2108>>2]|0,c[aA+3372>>2]|0);if((c[(c[aW>>2]|0)+3372>>2]|0)>0){a1=0}else{break}do{aA=(c[(c[aV>>2]|0)+2108>>2]|0)+(a1<<2)|0;as=c[aA>>2]|0;c[aA>>2]=(as|0)>-1107296256?as:-1107296256;a1=a1+1|0;}while((a1|0)<(c[(c[aW>>2]|0)+3372>>2]|0))}else{aW=c[d+(aS<<2)>>2]|0;aV=c[aW+4144>>2]|0;a_=c[v>>2]|0;a0=ac(a_,aV)|0;a$=p+(aS<<2)|0;as=c[a$>>2]|0;aA=g+(aS<<2)|0;Ne(c[as>>2]|0,as+4|0,as+484|0,as+1924|0,as+2644|0,a_,c[as+3372>>2]|0,w,t,as+3128|0,m+(aS<<2)|0,c[(c[aA>>2]|0)+2112>>2]|0,aV,aW+4148|0,c[u>>2]|0);aW=(a0|0)>0;do{if(aW){aV=0;do{as=c[a$>>2]|0;HG(as+484+(aV<<2)|0,(c[(c[aA>>2]|0)+2116>>2]|0)+(aV<<2)|0,c[as+3372>>2]|0);aV=(c[v>>2]|0)+aV|0;}while((aV|0)<(a0|0));if(!aW){break}aV=0;aX=c[a$>>2]|0;while(1){HG(aX+4+(aV<<2)|0,(c[(c[aA>>2]|0)+2108>>2]|0)+(aV<<2)|0,c[aX+3372>>2]|0);aZ=c[a$>>2]|0;if((c[aZ+3372>>2]|0)>0){aY=0;while(1){as=(c[(c[aA>>2]|0)+2108>>2]|0)+(aY+aV<<2)|0;a_=c[as>>2]|0;c[as>>2]=(a_|0)>-1107296256?a_:-1107296256;a_=aY+1|0;as=c[a$>>2]|0;if((a_|0)<(c[as+3372>>2]|0)){aY=a_}else{a2=as;break}}}else{a2=aZ}aY=(c[v>>2]|0)+aV|0;if((aY|0)<(a0|0)){aV=aY;aX=a2}else{break}}}}while(0);if(aI&aW){aX=0;do{aV=c[a$>>2]|0;HG(aV+1924+(aX<<2)|0,aV+2404+(aX<<2)|0,c[aV+3372>>2]|0);aX=(c[v>>2]|0)+aX|0;}while((aX|0)<(a0|0))}IY((c[aA>>2]|0)+24|0,(c[a$>>2]|0)+3128|0,244)}}while(0);aS=aS+1|0;}while((aS|0)<(a|0));if(O){a3=0}else{U=9709;break}do{aS=g+(a3<<2)|0;I_((c[aS>>2]|0)+1616|0,240);I_((c[aS>>2]|0)+1856|0,240);a3=a3+1|0;}while((a3|0)<(a|0));if(O){aS=f+1676|0;v=0;do{u=c[d+(v<<2)>>2]|0;if((c[u+4520>>2]|0)==0){t=c[p+(v<<2)>>2]|0;w=c[g+(v<<2)>>2]|0;s=c[q+(v<<2)>>2]|0;Mu(aS,c[r+(v<<2)>>2]|0,c[u+4104>>2]|0,c[t+3372>>2]|0,c[m+(v<<2)>>2]|0,c[w+2108>>2]|0,f+(((c[C+(v<<2)>>2]|0)!=0|0)*1844|0)+12|0,c[t>>2]|0,t+1444|0,o+(v*102|0)|0,c[w+416>>2]|0,c[s+8>>2]|0,c[s+4>>2]|0,c[w+2116>>2]|0,w+1376|0)}v=v+1|0;}while((v|0)<(a|0))}if(aI){U=9717}}else{U=9709}}while(0);if((U|0)==9709){if(aI){U=9717}}do{if((U|0)==9717){aI=g+12|0;c[aI>>2]=0;c[g+8>>2]=1;o=m|0;q=c[o>>2]|0;a3=m+4|0;a2=c[a3>>2]|0;a1=(q|0)>(a2|0)?q:a2;c[a3>>2]=a1;c[o>>2]=a1;a3=c[K>>2]|0;a2=p|0;q=c[a2>>2]|0;if((c[a3+4104>>2]|0)==2){v=c[p+4>>2]|0;aS=c[q>>2]|0;w=c[v>>2]|0;s=g+4|0;t=c[s>>2]|0;u=c[t+2108>>2]|0;A=c[(c[J>>2]|0)+2116>>2]|0;a0=c[t+2116>>2]|0;aX=g+16|0;aW=I|0;aV=c[aW>>2]|0;aY=ac(aV,c[a3+4144>>2]|0)|0;Nf(q+484|0,v+484|0,aS,w,q+4|0,v+4|0,u,q+2644|0,v+2644|0,A,a0,aI,aX,aY,aV,a1,q+3128|0,(c[f+1840>>2]|0)!=0|0,t+1616|0,t+1856|0,r|0);t=(c[s>>2]|0)+1616|0;s=c[aW>>2]|0;aW=ac(s,c[(c[K>>2]|0)+4144>>2]|0)|0;Ni(a2,J,t,aI,aX,aW,s,c[o>>2]|0,(c[J>>2]|0)+24|0);break}else{s=p+4|0;aW=g+4|0;aX=f+1676|0;t=r|0;a1=r+4|0;Mw(c[q+3372>>2]|0,q+484|0,(c[s>>2]|0)+484|0,c[(c[J>>2]|0)+2116>>2]|0,c[(c[aW>>2]|0)+2116>>2]|0,q+1924|0,aX,c[t>>2]|0,c[a1>>2]|0);q=c[a2>>2]|0;aV=c[s>>2]|0;s=c[aW>>2]|0;aY=g+16|0;a0=c[f>>2]|0;Nf(q+484|0,aV+484|0,c[q>>2]|0,c[aV>>2]|0,q+4|0,aV+4|0,c[s+2108>>2]|0,q+2644|0,aV+2644|0,c[(c[J>>2]|0)+2116>>2]|0,c[s+2116>>2]|0,aI,aY,a0,a0,c[o>>2]|0,f+12|0,(c[f+1840>>2]|0)!=0|0,s+1616|0,s+1856|0,t);s=c[(c[a2>>2]|0)+3372>>2]|0;Ni(a2,J,(c[aW>>2]|0)+1616|0,aI,aY,s,s,c[o>>2]|0,(c[J>>2]|0)+24|0);Mx(c[(c[a2>>2]|0)+3372>>2]|0,aX,c[t>>2]|0,c[a1>>2]|0,aY,aI);break}}}while(0);if(O){a4=0}else{$=0;i=l;return $|0}do{J=p+(a4<<2)|0;K=c[J>>2]|0;U=c[K+3372>>2]|0;do{if((c[(c[d+(a4<<2)>>2]|0)+4520>>2]|0)==0){aI=c[g+(a4<<2)>>2]|0;Mv(U,f+(a4*1844|0)+1676|0,(c[r+(a4<<2)>>2]|0)+360|0,K+964|0,aI+1376|0,c[aI+2108>>2]|0)}else{if((U|0)<=0){break}aI=g+(a4<<2)|0;aY=0;do{c[(c[aI>>2]|0)+1376+(aY<<2)>>2]=-2147483648;aY=aY+1|0;}while((aY|0)<(c[(c[J>>2]|0)+3372>>2]|0))}}while(0);a4=a4+1|0;}while((a4|0)<(a|0));if(!O){$=0;i=l;return $|0}O=f+4|0;f=I|0;I=0;while(1){a4=g+(I<<2)|0;c[(c[a4>>2]|0)+8>>2]=c[m+(I<<2)>>2];r=p+(I<<2)|0;c[(c[a4>>2]|0)+268>>2]=c[(c[r>>2]|0)+3124>>2];if((c[C+(I<<2)>>2]|0)==0){c[c[a4>>2]>>2]=c[O>>2];c[(c[a4>>2]|0)+4>>2]=c[O>>2];J=d+(I<<2)|0;c[(c[a4>>2]|0)+12>>2]=c[(c[J>>2]|0)+4104>>2];c[(c[a4>>2]|0)+16>>2]=c[(c[J>>2]|0)+4108>>2]}else{J=ac(c[f>>2]|0,c[(c[d+(I<<2)>>2]|0)+4144>>2]|0)|0;c[c[a4>>2]>>2]=J;c[(c[a4>>2]|0)+4>>2]=c[f>>2];c[(c[a4>>2]|0)+12>>2]=2;c[(c[a4>>2]|0)+16>>2]=0}J=d+(I<<2)|0;U=c[J>>2]|0;K=c[U+4144>>2]|0;if((K|0)>0){aY=0;aI=0;while(1){a1=aI<<1;t=c[U+4148+(aY<<2)>>2]|0;if((t|0)>1){aX=1;a2=a1;while(1){o=a2<<1|1;s=aX+1|0;if((s|0)<(t|0)){aX=s;a2=o}else{a5=o;break}}}else{a5=a1}a2=aY+1|0;if((a2|0)<(K|0)){aY=a2;aI=a5}else{a6=a5;break}}}else{a6=0}c[(c[a4>>2]|0)+20>>2]=a6;IY((c[a4>>2]|0)+272|0,(c[J>>2]|0)+4148|0,16);IY(c[(c[a4>>2]|0)+2100>>2]|0,(c[r>>2]|0)+484|0,240);IY(c[(c[a4>>2]|0)+2104>>2]|0,(c[r>>2]|0)+2644|0,240);aI=I+1|0;if((aI|0)<(a|0)){I=aI}else{$=0;break}}i=l;return $|0}function KH(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;if((e|0)<=0){g=0;return g|0}h=(b|0)>0;if((d|0)>0){i=0;j=0;k=0;L13509:while(1){l=Kc(i)|0;m=a+(i<<2)|0;c[m>>2]=l;if((l|0)==0){g=12576;n=9759;break}else{o=0;p=k}do{l=Kg(p,f)|0;c[(c[m>>2]|0)+24+(o<<2)>>2]=l;if((c[(c[m>>2]|0)+24+(o<<2)>>2]|0)==0){g=12576;n=9767;break L13509}p=p+1|0;o=o+1|0;}while((o|0)<(d|0));if(h){l=0;q=j;while(1){r=Ke(q)|0;c[(c[m>>2]|0)+(l<<2)>>2]=r;if((c[(c[m>>2]|0)+(l<<2)>>2]|0)==0){g=12576;n=9764;break L13509}r=q+1|0;s=l+1|0;if((s|0)<(b|0)){l=s;q=r}else{t=r;break}}}else{t=j}q=i+1|0;if((q|0)<(e|0)){i=q;j=t;k=p}else{g=0;n=9760;break}}if((n|0)==9759){return g|0}else if((n|0)==9767){return g|0}else if((n|0)==9764){return g|0}else if((n|0)==9760){return g|0}}if(h){u=0;v=0}else{h=0;while(1){p=Kc(h)|0;c[a+(h<<2)>>2]=p;if((p|0)==0){g=12576;n=9761;break}p=h+1|0;if((p|0)<(e|0)){h=p}else{g=0;n=9765;break}}if((n|0)==9761){return g|0}else if((n|0)==9765){return g|0}}L13533:while(1){h=Kc(u)|0;p=a+(u<<2)|0;c[p>>2]=h;if((h|0)==0){g=12576;n=9768;break}else{w=0;x=v}do{h=Ke(x)|0;c[(c[p>>2]|0)+(w<<2)>>2]=h;if((c[(c[p>>2]|0)+(w<<2)>>2]|0)==0){g=12576;n=9762;break L13533}x=x+1|0;w=w+1|0;}while((w|0)<(b|0));p=u+1|0;if((p|0)<(e|0)){u=p;v=x}else{g=0;n=9766;break}}if((n|0)==9768){return g|0}else if((n|0)==9762){return g|0}else if((n|0)==9766){return g|0}return 0}function KI(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;if((b|0)<=0){return 0}e=d+12|0;f=0;g=c[e>>2]|0;while(1){if((g|0)>0){h=a+(f<<2)|0;i=0;j=0;k=g;while(1){l=d+16+(i*24|0)+8|0;if((c[l>>2]|0)>0){m=0;n=j;do{o=c[h>>2]|0;c[(c[o+(i<<2)>>2]|0)+2e3+(m<<2)>>2]=c[o+24+(n<<2)>>2];n=n+1|0;m=m+1|0;}while((m|0)<(c[l>>2]|0));p=n;q=c[e>>2]|0}else{p=j;q=k}l=i+1|0;if((l|0)<(q|0)){i=l;j=p;k=q}else{r=q;break}}}else{r=g}k=f+1|0;if((k|0)<(b|0)){f=k;g=r}else{break}}return 0}function KJ(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0;e=J0(0)|0;c[a>>2]=e;L13561:do{if((e|0)!=0){if((ML(e+80|0,b)|0)!=0){break}if((Na(e+76|0,d)|0)==0){f=0}else{break}while(1){if((f|0)>=(b|0)){g=0;break}h=J6(f)|0;c[e+52+(f<<2)>>2]=h;if((h|0)==0){break L13561}else{f=f+1|0}}return g|0}}while(0);KK(a,0);g=12576;return g|0}function KK(a,b){a=a|0;b=b|0;var d=0,e=0;do{if((b|0)!=0){d=c[b>>2]|0;if((d|0)==0){break}e=d|0;if((c[e>>2]|0)!=0){Kf(e)}e=d+4|0;if((c[e>>2]|0)!=0){Kf(e)}e=d+8|0;if((c[e>>2]|0)!=0){Kf(e)}e=d+12|0;if((c[e>>2]|0)!=0){Kf(e)}e=d+16|0;if((c[e>>2]|0)!=0){Kf(e)}e=d+20|0;if((c[e>>2]|0)!=0){Kf(e)}Kd(b)}}while(0);if((a|0)==0){return}b=c[a>>2]|0;if((b|0)==0){return}e=b+80|0;if((c[e>>2]|0)!=0){MP(e)}e=b+76|0;if((c[e>>2]|0)!=0){Nb(e)}e=b+52|0;if((c[e>>2]|0)!=0){J7(e)}e=b+56|0;if((c[e>>2]|0)!=0){J7(e)}e=b+60|0;if((c[e>>2]|0)!=0){J7(e)}e=b+64|0;if((c[e>>2]|0)!=0){J7(e)}e=b+68|0;if((c[e>>2]|0)!=0){J7(e)}e=b+72|0;if((c[e>>2]|0)!=0){J7(e)}J1(a);return}function KL(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;d=c[b+8>>2]|0;c[a+4>>2]=d;c[a+8>>2]=c[b+56>>2];e=b|0;c[a+12>>2]=c[(c[e>>2]|0)+12>>2];f=b+16|0;g=c[f>>2]|0;c[a+28>>2]=g;c[a+24>>2]=c[f>>2];c[a+44>>2]=c[b+44>>2];f=c[b+24>>2]|0;h=a+16|0;c[h>>2]=f;c[a+36>>2]=c[b+36>>2];c[a+32>>2]=c[b+40>>2];if((f|0)==7|(f|0)==0|(f|0)==6){f=(g|0)/(c[(c[e>>2]|0)+8>>2]|0)|0;if((f|0)>50){i=0}else{i=(f|0)>0?1:2}c[a+20>>2]=i}else{c[a+20>>2]=0}c[a+48>>2]=c[b+60>>2];i=c[b+20>>2]|0;c[a>>2]=i;f=c[e>>2]|0;Hm(a,f,c[b+48>>2]|0,((c[b+12>>2]|0)/(c[b+52>>2]|0)|0)-i|0,(d|0)/(c[f+8>>2]|0)|0)|0;f=c[h>>2]|0;if(f>>>0<8){h=c[1920868+(f<<3)>>2]|0;c[a+40>>2]=h;j=h;k=a+80|0;l=c[k>>2]|0;m=b+28|0;n=c[m>>2]|0;o=a+52|0;p=c[e>>2]|0;q=p+12|0;r=c[q>>2]|0;MM(l,n,o,r,j);return 0}else{c[a+40>>2]=0;j=0;k=a+80|0;l=c[k>>2]|0;m=b+28|0;n=c[m>>2]|0;o=a+52|0;p=c[e>>2]|0;q=p+12|0;r=c[q>>2]|0;MM(l,n,o,r,j);return 0}return 0}function KM(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0;i=c[a+8>>2]|0;j=d|0;k=e+2e3|0;KX(k,j,i);MG(e+40|0,j,k,d+12|0,b,i);return Kr(0,a,0,d,j,g,f,h,e|0,0)|0}function KN(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;b=a+48|0;a=ac(g>>3,e)|0;e=(c[b>>2]|0)-((a|0)%(f|0)|0)|0;g=(e|0)<1;c[b>>2]=(g?f:0)+e;c[d>>2]=(g&1)+((a|0)/(f|0)|0)<<3;return 0}function KO(a,d,e,f,g,h,j,k){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0;l=i;i=i+1192|0;m=l|0;n=l+24|0;o=l+32|0;p=l+512|0;q=l+992|0;r=l+1e3|0;s=l+1024|0;t=l+1048|0;u=l+1072|0;v=l+1120|0;w=l+1168|0;x=a+24|0;y=c[x>>2]|0;if((y|0)<0){z=16544;i=l;return z|0}A=c[a+28>>2]|0;if((y|0)>(A|0)){z=16545;i=l;return z|0}B=g+12|0;C=c[B>>2]|0;D=C-1|0;E=a+4|0;F=(c[E>>2]|0)-f|0;H=(y|0)<(F|0)?y:F;y=(A|0)<(F|0)?A:F;do{if((H|0)==0){I=0}else{F=H>>31^H;if((F|0)<=-1){I=-1;break}A=~F;F=0;while(1){J=A<<1;if((J|0)<0){A=J;F=F+1|0}else{I=F;break}}}}while(0);do{if((y|0)==0){K=0}else{F=y>>31^y;if((F|0)<=-1){K=-1;break}A=~F;F=0;while(1){J=A<<1;if((J|0)<0){A=J;F=F+1|0}else{K=F;break}}}}while(0);F=H<<I;A=F;J=(F|0)<0|0?-1:0;F=y<<K;L=F;M=(F|0)<0|0?-1:0;F=C;C=D;D=0;N=0;L13658:while(1){O=F;while(1){P=O-1|0;if((O|0)<=0){break L13658}Q=c[g+16+(P*24|0)>>2]|0;if((Q|0)==0|(Q|0)==1|(Q|0)==3){break}else{O=P}}O=a+52+(P<<2)|0;Q=c[O>>2]|0;R=c[Q+16>>2]|0;N2(R,(R|0)<0|0?-1:0,A,J)|0;c[Q+8>>2]=(G<<1|0>>>31)>>I;Q=c[O>>2]|0;R=(c[Q+8>>2]|0)+D|0;S=c[Q+16>>2]|0;N2(S,(S|0)<0|0?-1:0,L,M)|0;c[Q+12>>2]=(G<<1|0>>>31)>>K;F=P;C=P;D=R;N=(c[(c[O>>2]|0)+12>>2]|0)+N|0}P=a+52+(C<<2)|0;C=(c[P>>2]|0)+8|0;c[C>>2]=H-D+(c[C>>2]|0);C=(c[P>>2]|0)+12|0;c[C>>2]=y-N+(c[C>>2]|0);C=c[B>>2]|0;if((C|0)>0){N=0;do{y=c[g+16+(N*24|0)>>2]|0;if((y|0)==0|(y|0)==1|(y|0)==3){c[s+(N<<2)>>2]=c[(c[e>>2]|0)+(N<<2)>>2]}N=N+1|0;}while((N|0)<(C|0))}C=a+16|0;N=c[C>>2]|0;if((N|0)==7|(N|0)==0|(N|0)==6){N=c[E>>2]|0;c[(c[e>>2]|0)+104>>2]=((N|0)<(f|0)?N:f)-(c[a>>2]|0)&-8;N=c[e>>2]|0;y=N+104|0;c[y>>2]=(c[y>>2]|0)-((c[N+120>>2]|0)+(c[N+116>>2]|0)+(c[N+112>>2]|0));N=c[e>>2]|0;c[N+100>>2]=(c[E>>2]&-8)-(c[N+116>>2]|0)-(c[N+120>>2]|0)-(c[N+112>>2]|0);N=c[e>>2]|0;y=c[N+104>>2]|0;P=(c[x>>2]|0)+y|0;do{if((P|0)<0){x=m;D=c[B>>2]|0;if((D|0)>0){H=m|0;F=0;K=0;M=D;while(1){L=g+16+(F*24|0)|0;c[x>>2]=c[L>>2];c[x+4>>2]=c[L+4>>2];c[x+8>>2]=c[L+8>>2];c[x+12>>2]=c[L+12>>2];c[x+16>>2]=c[L+16>>2];c[x+20>>2]=c[L+20>>2];L=c[H>>2]|0;if((L|0)==0|(L|0)==1|(L|0)==3){c[n>>2]=0;L=c[(c[d>>2]|0)+(F<<2)>>2]|0;I=L|0;Kr(0,m,0,L,I,0,2,-1,n,1)|0;T=(c[n>>2]|0)+K|0;U=c[B>>2]|0}else{T=K;U=M}I=F+1|0;if((I|0)<(U|0)){F=I;K=T;M=U}else{break}}V=T;W=c[e>>2]|0;X=U}else{V=0;W=N;X=D}if((P|0)<(V-(c[W+120>>2]|0)|0)){z=16544;i=l;return z|0}else{Y=c[W+104>>2]|0;Z=X;break}}else{Y=y;Z=c[B>>2]|0}}while(0);y=Z-1|0;if((Z|0)>0){Z=Y;X=(Y|0)<0|0?-1:0;W=y;V=y;P=0;while(1){N=c[g+16+(W*24|0)>>2]|0;if((N|0)==0|(N|0)==1|(N|0)==3){N=c[(c[a+52+(W<<2)>>2]|0)+16>>2]|0;U=N;T=(N|0)<0|0?-1:0;N2(U,T,Z,X)|0;T=s+(W<<2)|0;c[(c[T>>2]|0)+28>>2]=G<<1|0>>>31;_=(c[(c[T>>2]|0)+28>>2]|0)+P|0;$=W}else{_=P;$=V}if((W|0)>0){W=W-1|0;V=$;P=_}else{aa=$;ab=_;break}}}else{aa=y;ab=0}y=(c[s+(aa<<2)>>2]|0)+28|0;c[y>>2]=Y-ab+(c[y>>2]|0);c[(c[e>>2]|0)+128>>2]=0;y=c[B>>2]|0;if((y|0)>0){ab=a+80|0;Y=a+44|0;aa=a+20|0;_=0;$=y;y=f;while(1){P=c[g+16+(_*24|0)>>2]|0;if((P|0)==0|(P|0)==1|(P|0)==3){P=c[ab>>2]|0;V=c[(c[d>>2]|0)+(_<<2)>>2]|0;W=s+(_<<2)|0;X=c[W>>2]|0;Z=a+52+(_<<2)|0;T=c[Z>>2]|0;MN(P,c[P+64+(_<<2)>>2]|0,V|0,X+40|0,X+32|0,X+36|0,c[g+16+(_*24|0)+8>>2]|0,c[V+8>>2]|0,c[X+28>>2]|0,c[T+8>>2]|0,c[T+12>>2]|0,c[Y>>2]|0,c[aa>>2]|0);T=(c[(c[Z>>2]|0)+8>>2]|0)+y|0;Z=(c[e>>2]|0)+128|0;c[Z>>2]=(c[Z>>2]|0)+(c[(c[W>>2]|0)+36>>2]|0);ac=c[B>>2]|0;ad=T}else{ac=$;ad=y}T=_+1|0;if((T|0)<(ac|0)){_=T;$=ac;y=ad}else{ae=ad;break}}}else{ae=f}f=c[E>>2]|0;af=(f|0)<(ae|0)?f:ae}else{ae=c[e>>2]|0;c[ae+104>>2]=((c[E>>2]|0)-(c[a>>2]|0)&-8)-(c[ae+116>>2]|0)-(c[ae+120>>2]|0)-(c[ae+112>>2]|0);ae=c[e>>2]|0;c[ae+100>>2]=c[ae+104>>2];af=c[E>>2]|0}E=a+80|0;ae=c[C>>2]|0;if((ae|0)==7|(ae|0)==0){ag=1}else{ag=(ae|0)==6|0}MO((c[E>>2]|0)+64|0,s|0,c[e>>2]|0,c[d>>2]|0,ag,g);ag=a+36|0;ae=c[B>>2]|0;if((ae|0)>0){C=0;f=ae;while(1){ae=c[g+16+(C*24|0)>>2]|0;ad=c[g+16+(C*24|0)+8>>2]|0;if((ae|0)==0|(ae|0)==1|(ae|0)==3){KY(c[(c[d>>2]|0)+(C<<2)>>2]|0,(c[s+(C<<2)>>2]|0)+2e3|0,c[ag>>2]|0,ad);c[w+(C<<2)>>2]=1;c[t+(C<<2)>>2]=0;if((ad|0)>0){ae=0;do{c[u+(C<<3)+(ae<<2)>>2]=1;c[v+(C<<3)+(ae<<2)>>2]=1;ae=ae+1|0;}while((ae|0)<(ad|0))}ah=c[B>>2]|0}else{ah=f}ad=C+1|0;if((ad|0)<(ah|0)){C=ad;f=ah}else{break}}}c[(c[e>>2]|0)+132>>2]=-1;ah=a+32|0;f=r;C=b[988380]|0;ag=b[988364]|0;ad=r+8|0;ae=r|0;D=a+76|0;y=a|0;ac=w;$=u;_=0;aa=-1;while(1){if((_|0)>=(c[B>>2]|0)){Y=(c[e>>2]|0)+132|0;c[Y>>2]=0;ab=c[B>>2]|0;if((ab|0)>0){T=0;W=ab;Z=0;while(1){X=c[g+16+(T*24|0)>>2]|0;if((X|0)==0|(X|0)==1|(X|0)==3){X=(c[(c[s+(T<<2)>>2]|0)+4>>2]|0)+Z|0;c[Y>>2]=X;ai=c[B>>2]|0;aj=X}else{ai=W;aj=Z}X=T+1|0;if((X|0)<(ai|0)){T=X;W=ai;Z=aj}else{ak=ai;break}}}else{ak=ab}Z=c[e>>2]|0;W=c[Z+132>>2]|0;if((W|0)==-1){al=0}else{T=c[y>>2]|0;if((ak|0)>0){Y=0;X=0;while(1){V=c[g+16+(Y*24|0)>>2]|0;if((V|0)==0|(V|0)==1|(V|0)==3){V=c[s+(Y<<2)>>2]|0;am=(c[V+4>>2]|0)+X+(c[V>>2]|0)+(c[V+8>>2]|0)|0}else{am=X}V=Y+1|0;if((V|0)<(ak|0)){Y=V;X=am}else{an=am;break}}}else{an=0}X=(c[Z+116>>2]|0)+an|0;Y=X+T+((8-((X|0)%8|0)|0)%8|0)|0;if((Y|0)<(af|0)){if((aa|0)==1){ao=9966}else{ap=0}}else{if((W|0)==0&(aa|0)==1){ao=9966}else{ap=0}}if((ao|0)==9966){ao=0;ap=1}if((Y|0)>(af|0)&(aa|0)==0){z=0;ao=9985;break}else{al=ap}}Y=(ak|0)>0;if(Y){X=1;ab=0;while(1){V=c[g+16+(ab*24|0)>>2]|0;if((V|0)==0|(V|0)==1|(V|0)==3){aq=(c[t+(ab<<2)>>2]|0)>=(c[ah>>2]|0)&X}else{aq=X}V=ab+1|0;if((V|0)<(ak|0)){X=aq;ab=V}else{ar=aq;break}}}else{ar=1}ab=(W|0)>(c[Z+100>>2]|0);if((al|0)!=0&(ab^1)){as=al;at=aa}else{X=c[y>>2]|0;if(Y){T=0;V=0;while(1){P=c[g+16+(T*24|0)>>2]|0;if((P|0)==0|(P|0)==1|(P|0)==3){P=c[s+(T<<2)>>2]|0;au=(c[P+4>>2]|0)+V+(c[P>>2]|0)+(c[P+8>>2]|0)|0}else{au=V}P=T+1|0;if((P|0)<(ak|0)){T=P;V=au}else{av=au;break}}}else{av=0}V=(c[Z+116>>2]|0)+av|0;T=V+X+((8-((V|0)%8|0)|0)%8|0)|0;V=(W|0)>-1?1:al;if((ar|0)==0){aw=V}else{aw=(T|0)<(af|0)?1:V}I_(ac,24);I_($,48);as=ab?0:(T|0)>(af|0)?0:aw;at=ab?1:(T|0)>=(af|0)|0}if((as|0)==0){_=0;aa=at;continue}else{z=0;ao=9984;break}}T=c[g+16+(_*24|0)>>2]|0;V=c[g+16+(_*24|0)+8>>2]|0;do{if((T|0)==0|(T|0)==1|(T|0)==3){Y=w+(_<<2)|0;P=t+(_<<2)|0;U=s+(_<<2)|0;N=a+52+(_<<2)|0;n=(V|0)>0;m=(V|0)==2|0;M=aa;K=(c[Y>>2]|0)==0;L13772:while(1){L13774:do{if(K){F=c[ah>>2]|0;H=(M|0)!=0?1:-1;x=c[(c[d>>2]|0)+(_<<2)>>2]|0;I=c[e>>2]|0;L=c[U>>2]|0;J=c[N>>2]|0;A=c[P>>2]|0;c[P>>2]=A+1;if((A|0)<(F|0)){if(n){ax=0}else{ao=9934;break L13772}while(1){if((c[u+(_<<3)+(ax<<2)>>2]|0)==0){F=(c[L+2e3+(ax<<2)>>2]|0)+6624|0;c[F>>2]=(c[F>>2]|0)+H;c[v+(_<<3)+(ax<<2)>>2]=1}ax=ax+1|0;if((ax|0)>=(V|0)){break L13774}}}H=c[L+4>>2]|0;if((H|0)==0){break}F=L+28|0;A=H+8|0;H=A-(c[J+8>>2]|0)-(c[F>>2]|0)|0;O=L|0;R=(c[O>>2]|0)+A-(c[J+4>>2]|0)|0;A=(H|0)>(R|0)?H:R;if((A|0)>0){R=L+2e3|0;H=x|0;if(n){Q=0;do{S=c[x+(Q<<2)>>2]|0;ay=L+2e3+(Q<<2)|0;az=c[ay>>2]|0;if((c[az+6648>>2]|0)>0){aA=0;aB=az;while(1){az=c[aB+6652+(aA<<4)>>2]|0;if((c[aB+6652+(aA<<4)+8>>2]|0)>0){aC=(az|0)==13;aD=c[aB+6652+(aA<<4)+4>>2]|0;aE=aB;while(1){aF=o+(Q*240|0)+(aD<<2)|0;c[aF>>2]=0;if(aC){aG=aE;aH=aD+1|0}else{aI=c[S+24+(aD<<2)>>2]|0;aJ=aD+1|0;c[aF>>2]=MX(aE+4096+(aI<<1)|0,(c[S+24+(aJ<<2)>>2]|0)-aI|0,az)|0;aG=c[ay>>2]|0;aH=aJ}c[p+(Q*240|0)+(aD<<2)>>2]=aA;if((aH|0)<((c[aG+6652+(aA<<4)+8>>2]|0)+(c[aG+6652+(aA<<4)+4>>2]|0)|0)){aD=aH;aE=aG}else{aK=aG;break}}}else{aK=aB}aE=aA+1|0;if((aE|0)<(c[aK+6648>>2]|0)){aA=aE;aB=aK}else{break}}}Q=Q+1|0;}while((Q|0)<(V|0))}Q=0;J=c[(c[R>>2]|0)+6640>>2]|0;while(1){aL=J-1|0;if((J|0)<=0){break}aB=c[H>>2]|0;if((c[aB>>2]|0)>0){aA=0;ay=Q;S=aB;while(1){if(n){aB=aA+aL|0;aE=ay;aD=0;while(1){az=c[p+(aD*240|0)+(aB<<2)>>2]|0;aC=L+2e3+(aD<<2)|0;aJ=(c[aC>>2]|0)+6652+(az<<4)+8|0;c[aJ>>2]=(c[aJ>>2]|0)-1;aJ=(c[o+(aD*240|0)+(aB<<2)>>2]|0)+aE|0;if((c[(c[aC>>2]|0)+6652+(az<<4)+8>>2]|0)==0){aM=(((c[(c[x+(aD<<2)>>2]|0)+12>>2]|0)!=2?C:ag)<<16>>16)+aJ|0}else{aM=aJ}aJ=aD+1|0;if((aJ|0)<(V|0)){aE=aM;aD=aJ}else{break}}aN=aM;aO=c[H>>2]|0}else{aN=ay;aO=S}aD=(c[aO+4>>2]|0)+aA|0;if((aD|0)<(c[aO>>2]|0)){aA=aD;ay=aN;S=aO}else{aP=aN;break}}}else{aP=Q}if((aP|0)<(A|0)){Q=aP;J=aL}else{break}}Q=(J|0)==0?0:aL;if(n){A=(Q|0)==0;R=x+12|0;S=0;do{c[(c[L+2e3+(S<<2)>>2]|0)+6640>>2]=Q;ay=x+(S<<2)|0;c[(c[ay>>2]|0)+8>>2]=Q;if(A){I_((c[ay>>2]|0)+288|0,1088);I_(R,244)}S=S+1|0;}while((S|0)<(V|0))}I_(f,24);c[ad>>2]=V;c[ae>>2]=m;Kr(0,r,0,x,H,j,h,k,q,0)|0;S=c[q>>2]|0;R=(c[O>>2]|0)-S|0;c[O>>2]=S;c[F>>2]=(c[F>>2]|0)+R;S=I+120|0;c[S>>2]=(c[S>>2]|0)-R;S=I+104|0;c[S>>2]=(c[S>>2]|0)+R;S=I+100|0;c[S>>2]=(c[S>>2]|0)+R}else{if(n){aQ=0}else{ao=9934;break L13772}do{R=(c[L+2e3+(aQ<<2)>>2]|0)+6624|0;c[R>>2]=(c[R>>2]|0)+1;aQ=aQ+1|0;}while((aQ|0)<(V|0))}if(n){aR=0}else{ao=9934;break L13772}do{c[v+(_<<3)+(aR<<2)>>2]=1;aR=aR+1|0;}while((aR|0)<(V|0))}}while(0);c[Y>>2]=1;if(n){aS=0;aT=M}else{ao=9936;break}while(1){L=u+(_<<3)+(aS<<2)|0;c[L>>2]=1;I=v+(_<<3)+(aS<<2)|0;do{if((c[I>>2]|0)==0){aU=aT}else{F=c[(c[U>>2]|0)+2e3+(aS<<2)>>2]|0;O=c[(c[(c[d>>2]|0)+(_<<2)>>2]|0)+(aS<<2)>>2]|0;c[I>>2]=0;H=O|0;x=O+8|0;R=O+4|0;KS(c[H>>2]|0,c[x>>2]|0,c[R>>2]|0,O+24|0,F|0,c[F+6624>>2]|0,F+6384|0,F+4096|0);S=c[H>>2]|0;H=c[x>>2]|0;x=c[R>>2]|0;if((S|0)<=0){aU=aT;break}R=(H|0)>0;A=0;Q=0;while(1){if(R){J=0;ay=A;while(1){aA=J+Q|0;aD=c[O+24+(aA<<2)>>2]|0;aE=c[O+24+(aA+1<<2)>>2]|0;if((aD|0)<(aE|0)){aB=aD;aD=0;while(1){aJ=b[F+4096+(aB<<1)>>1]|0;az=(aJ<<16>>16>0?aJ:-aJ&65535)<<16>>16;aJ=(az|0)>(aD|0)?az:aD;az=aB+1|0;if((az|0)<(aE|0)){aB=az;aD=aJ}else{aV=aJ;break}}}else{aV=0}c[F+6144+(aA<<2)>>2]=aV;aD=(aV|0)>(ay|0)?aV:ay;aB=J+1|0;if((aB|0)<(H|0)){J=aB;ay=aD}else{aW=aD;break}}}else{aW=A}ay=Q+x|0;if((ay|0)<(S|0)){A=aW;Q=ay}else{break}}if((aW|0)<=8191){aU=aT;break}c[L>>2]=0;c[Y>>2]=0;aU=1}}while(0);L=aS+1|0;if((L|0)<(V|0)){aS=L;aT=aU}else{break}}if((c[Y>>2]|0)==0){M=aU;K=1}else{aX=aU;ao=9949;break}}if((ao|0)==9934){ao=0;c[Y>>2]=1;aX=M;ao=9949}else if((ao|0)==9936){ao=0;c[(c[U>>2]|0)+4>>2]=0;aY=M}do{if((ao|0)==9949){ao=0;c[(c[U>>2]|0)+4>>2]=0;if(n){aZ=0}else{aY=aX;break}while(1){K=c[(c[U>>2]|0)+2e3+(aZ<<2)>>2]|0;m=c[(c[(c[d>>2]|0)+(_<<2)>>2]|0)+(aZ<<2)>>2]|0;P=M9(c[D>>2]|0,K+4096|0,K+6144|0,K+6384|0,c[m+12>>2]|0,c[m>>2]|0,c[m+8>>2]|0,c[m+4>>2]|0,m+24|0,K+6628|0,m+1376|0,m+1616|0,m+1856|0,j)|0;m=(c[U>>2]|0)+4|0;c[m>>2]=(c[m>>2]|0)+P;P=aZ+1|0;if((P|0)<(V|0)){aZ=P}else{aY=aX;break}}}}while(0);n=(c[(c[E>>2]|0)+64+(_<<2)>>2]|0)+56|0;if((c[n>>2]|0)!=-1){a_=aY;break}c[n>>2]=c[(c[U>>2]|0)+4>>2];a_=aY}else{a_=aa}}while(0);_=_+1|0;aa=a_}if((ao|0)==9984){i=l;return z|0}else if((ao|0)==9985){i=l;return z|0}return 0}function KP(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0;switch(c[b+16>>2]|0){case 1:case 2:case 3:case 4:case 5:{d=c[e>>2]|0;c[d+108>>2]=(c[d+104>>2]|0)-(c[d+132>>2]|0)&7;return 0};case 7:case 6:{return 0};default:{d=c[e>>2]|0;e=(c[d+104>>2]|0)-(c[d+132>>2]|0)|0;a=e&7;f=(c[b+28>>2]|0)-(c[b+24>>2]|0)-7|0;b=e-((f|0)<0?0:f&-8)|0;c[d+108>>2]=(a|0)>(b|0)?a:b;return 0}}return 0}function KQ(a,b,d){a=a|0;b=b|0;d=d|0;var e=0;if(((c[b+16>>2]|0)-1|0)>>>0<6){a=c[b+4>>2]|0;e=c[b+28>>2]|0;c[b+24>>2]=(a|0)<(e|0)?a:e;return}else{e=c[d>>2]|0;d=b+24|0;c[d>>2]=(c[e+104>>2]|0)-(c[e+132>>2]|0)-(c[e+108>>2]|0)-(c[e+136>>2]|0)+(c[d>>2]|0);return}}function KR(a,b,d,e,f,g,h,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;e=i;i=i+16|0;a=e|0;k=d+120|0;l=d+132|0;m=d+108|0;n=d+112|0;o=d+116|0;p=(c[l>>2]|0)+(c[k>>2]|0)+(c[m>>2]|0)+(c[n>>2]|0)+(c[o>>2]|0)|0;q=d+140|0;c[q>>2]=p;do{if((c[b+16>>2]|0)==0){r=H$(f,p)|0;s=b|0;t=c[s>>2]|0;if((r|0)==(t|0)){u=s;break}v=b+24|0;w=c[v>>2]|0;x=d+104|0;y=t-r|0;t=y-(c[b+28>>2]|0)+w+(c[x>>2]|0)-(c[l>>2]|0)-(c[m>>2]|0)|0;if((t|0)<0){z=0}else{z=t+7&-8}c[v>>2]=y-z+w;c[m>>2]=(c[m>>2]|0)+z;w=(c[q>>2]|0)+z|0;c[q>>2]=w;c[x>>2]=(c[x>>2]|0)+z;x=H$(f,w)|0;c[s>>2]=x;if((x|0)==(r|0)){u=s;break}c[v>>2]=r-x+(c[v>>2]|0);u=s}else{u=b|0}}while(0);c[u>>2]=H$(f,c[q>>2]|0)|0;f=c[m>>2]|0;I_(a,12);c[a>>2]=1;c[a+4>>2]=f;u=Kp(0,a,0,0,h,g,j)|0;c[m>>2]=u;j=c[k>>2]|0;k=c[l>>2]|0;l=c[n>>2]|0;n=c[o>>2]|0;o=7-((u-1+j+k+l+n|0)%8|0)|0;if((u-f+o|0)==8&(u|0)>8){f=u-8|0;c[m>>2]=f;A=f}else{A=u}u=j+o+k+A+l+n|0;c[q>>2]=u;if((u|0)>(c[b+4>>2]|0)){B=16416;i=e;return B|0}if((u|0)<(c[b+8>>2]|0)){B=16416;i=e;return B|0}c[d+136>>2]=o;B=0;i=e;return B|0}function KS(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0;if((a|0)<=0){return}if((b|0)>0){j=0}else{k=0;do{k=k+d|0;}while((k|0)<(a|0));return}do{k=0;do{l=k+j|0;m=c[e+(l<<2)>>2]|0;KT(g-(c[h+(l<<2)>>2]|0)|0,(c[e+(l+1<<2)>>2]|0)-m|0,f+(m<<2)|0,i+(m<<1)|0);k=k+1|0;}while((k|0)<(b|0));j=j+d|0;}while((j|0)<(a|0));return}function KT(a,d,f,g){a=a|0;d=d|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=-a|0;a=(h>>2)+1|0;if((d|0)<=0){return}i=e[1976864+((h&3)<<1)>>1]<<16;h=i;j=(i|0)<0|0?-1:0;i=0;do{k=c[f+(i<<2)>>2]|0;N2(k,(k|0)<0|0?-1:0,h,j)|0;k=G;do{if((k|0)<0){l=-k|0;if((l|0)>-1){m=k-1|0;n=0;while(1){o=m<<1;if((o|0)<0){m=o;n=n+1|0}else{p=n;break}}}else{p=-1}n=a-p+1|0;m=((n>>2)*-3|0)+12|0;if((m|0)<32){q=((ac(b[1978928+((l<<p>>21&-513)<<1)>>1]|0,b[1976872+((n&3)<<1)>>1]|0)|0)>>m)+13284>>15}else{q=0}b[g+(i<<1)>>1]=-q&65535}else{if((k|0)<=0){b[g+(i<<1)>>1]=0;break}m=~k;n=0;while(1){o=m<<1;if((o|0)<0){m=o;n=n+1|0}else{break}}m=a-n+1|0;l=((m>>2)*-3|0)+12|0;if((l|0)<32){r=(((ac(b[1978928+((k<<n>>21&-513)<<1)>>1]|0,b[1976872+((m&3)<<1)>>1]|0)|0)>>l)+13284|0)>>>15&65535}else{r=0}b[g+(i<<1)>>1]=r}}while(0);i=i+1|0;}while((i|0)<(d|0));return}function KU(a,d,e,f){a=a|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;g=i;i=i+8|0;h=g|0;L13944:do{if((e|0)>0){j=0;k=0;while(1){l=a+(j<<2)|0;m=d+(j<<1)|0;KT(f,1,l,m);n=b[m>>1]|0;if((n<<16>>16>0?n:-n&65535)<<16>>16>8191){o=0;break}KV(f,m,h);m=c[h>>2]|0;n=c[l>>2]>>1;l=((m|0)>0?m:-m|0)-((n|0)>0?n:-n|0)|0;n=(l|0)>0?l:-l|0;do{if((n|0)==0){p=0;q=10054}else{l=n>>31^n;if((l|0)<=-1){p=-1;q=10054;break}m=~l;l=0;while(1){r=m<<1;if((r|0)<0){m=r;l=l+1|0}else{break}}if((l|0)<=0){p=l;q=10054;break}s=n<<l;t=l}}while(0);if((q|0)==10054){q=0;s=n>>-p;t=p}m=s;r=(s|0)<0|0?-1:0;N2(m,r,m,r)|0;r=G<<1|0>>>31;m=(t<<1)-2|0;u=(m|0)<31?m:31;m=((u|0)<0?r<<-u:r>>u)+k|0;u=j+1|0;if((u|0)<(e|0)){j=u;k=m}else{v=m;break L13944}}i=g;return o|0}else{v=0}}while(0);o=HE(v)|0;i=g;return o|0}function KV(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;g=a&3;h=1-(a>>2)|0;a=b[e>>1]|0;e=a<<16>>16;if(a<<16>>16<0){i=-e|0;j=i>>31^i;if((j|0)>-1){k=~j;j=0;while(1){l=k<<1;if((l|0)<0){k=l;j=j+1|0}else{m=j;break}}}else{m=-1}j=31-m|0;k=c[1976880+((i<<m>>21&-513)<<2)>>2]|0;m=c[1976504+(g*56|0)+(j<<2)>>2]|0;i=k;l=(k|0)<0|0?-1:0;k=m;n=(m|0)<0|0?-1:0;N2(k,n,i,l)|0;l=G<<1|0>>>31;i=h-(d[1976448+(g*14|0)+j|0]|0)|0;if((i|0)<0){o=l<<-i}else{o=l>>i}c[f>>2]=-o;return}if(a<<16>>16<=0){c[f>>2]=0;return}a=e>>31^e;if((a|0)>-1){o=~a;a=0;while(1){i=o<<1;if((i|0)<0){o=i;a=a+1|0}else{p=a;break}}}else{p=-1}a=31-p|0;o=c[1976880+((e<<p>>21&-513)<<2)>>2]|0;p=c[1976504+(g*56|0)+(a<<2)>>2]|0;N2(p,(p|0)<0|0?-1:0,o,(o|0)<0|0?-1:0)|0;o=G<<1|0>>>31;p=h-(d[1976448+(g*14|0)+a|0]|0)|0;if((p|0)<0){q=o<<-p}else{q=o>>p}c[f>>2]=q;return}function KW(a,d,e,f,g,h){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;j=i;i=i+8|0;k=j|0;L13990:do{if((e|0)>0){l=0;m=0;n=0;while(1){o=d+(l<<1)|0;p=b[o>>1]|0;if((p<<16>>16>0?p:-p&65535)<<16>>16>8191){break}KV(f,o,k);o=c[k>>2]|0;p=o;q=(o|0)<0|0?-1:0;N2(p,q,p,q)|0;q=(G<<1|0>>>31)+m|0;p=c[a+(l<<2)>>2]>>1;r=((o|0)>0?o:-o|0)-((p|0)>0?p:-p|0)|0;p=(r|0)>0?r:-r|0;do{if((p|0)==0){s=0;t=10090}else{r=p>>31^p;if((r|0)<=-1){s=-1;t=10090;break}o=~r;r=0;while(1){u=o<<1;if((u|0)<0){o=u;r=r+1|0}else{break}}if((r|0)<=0){s=r;t=10090;break}v=p<<r;w=r}}while(0);if((t|0)==10090){t=0;v=p>>-s;w=s}o=v;u=(v|0)<0|0?-1:0;N2(o,u,o,u)|0;u=G<<1|0>>>31;o=(w<<1)-2|0;x=(o|0)<31?o:31;o=((x|0)<0?u<<-x:u>>x)+n|0;x=l+1|0;if((x|0)<(e|0)){l=x;m=q;n=o}else{y=q;z=o;break L13990}}c[g>>2]=0;A=0;c[h>>2]=A;i=j;return}else{y=0;z=0}}while(0);c[g>>2]=(HE(y)|0)+67108864;A=HE(z)|0;c[h>>2]=A;i=j;return}function KX(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;if((d|0)>0){e=0}else{return}do{f=c[a+(e<<2)>>2]|0;g=c[b+(e<<2)>>2]|0;h=c[g>>2]|0;i=c[g+8>>2]|0;j=g+4|0;k=c[j>>2]|0;L14013:do{if((h|0)>0){l=g+2096|0;if((i|0)>0){m=0}else{n=0;o=k;while(1){if((o|0)>0){p=0;while(1){c[f+7632+(p+n<<2)>>2]=-2147483648;q=p+1|0;r=c[j>>2]|0;if((q|0)<(r|0)){p=q}else{s=r;break}}}else{s=o}p=n+k|0;if((p|0)<(h|0)){n=p;o=s}else{break L14013}}}do{o=0;do{n=o+m|0;p=c[g+24+(n<<2)>>2]|0;r=g+24+(n+1<<2)|0;if((p|0)<(c[r>>2]|0)){q=0;t=p;while(1){p=c[(c[l>>2]|0)+(t<<2)>>2]|0;u=((HN((p|0)>0?p:-p|0)|0)>>6)+q|0;p=t+1|0;if((p|0)<(c[r>>2]|0)){q=u;t=p}else{v=u;break}}}else{v=0}c[f+7632+(n<<2)>>2]=HE(v)|0;o=o+1|0;}while((o|0)<(i|0));if((i|0)<(c[j>>2]|0)){o=i;do{c[f+7632+(o+m<<2)>>2]=-2147483648;o=o+1|0;}while((o|0)<(c[j>>2]|0))}m=m+k|0;}while((m|0)<(h|0))}}while(0);e=e+1|0;}while((e|0)<(d|0));return}function KY(a,e,f,g){a=a|0;e=e|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0,cb=0,cc=0,cd=0,ce=0,cf=0,cg=0,ch=0,ci=0,cj=0,ck=0,cl=0,cm=0,cn=0,co=0,cp=0,cq=0,cr=0,cs=0,ct=0,cu=0,cv=0,cw=0,cx=0,cy=0,cz=0,cA=0,cB=0,cC=0,cD=0,cE=0,cF=0,cG=0,cH=0,cI=0,cJ=0,cK=0,cL=0,cM=0,cN=0;h=i;i=i+3984|0;j=h|0;k=h+8|0;l=h+248|0;m=h+488|0;n=h+728|0;o=h+736|0;p=h+976|0;q=h+1216|0;r=h+3264|0;s=h+3504|0;t=h+3744|0;if((g|0)<=0){i=h;return}u=t;v=(f|0)>0;w=(f|0)>1;f=t|0;x=s|0;y=k;z=k|0;A=0;while(1){B=c[e+(A<<2)>>2]|0;C=c[a+(A<<2)>>2]|0;D=B+6384|0;E=B+6624|0;F=B+7632|0;if(v){I_(B+4096|0,2048)}H=C|0;I=c[H>>2]|0;if((I|0)>0){J=0;while(1){c[B+6384+(J<<2)>>2]=-2147483648;K=J+1|0;L=c[H>>2]|0;if((K|0)<(L|0)){J=K}else{M=L;break}}}else{M=I}J=0;do{c[r+(J<<2)>>2]=-2147483648;J=J+1|0;}while((J|0)<60);if((M|0)>0){J=C+8|0;I=C+4|0;L=0;K=c[J>>2]|0;N=M;while(1){if((K|0)>0){O=0;P=K;while(1){Q=O+L|0;R=c[B+7872+(Q<<2)>>2]|0;S=c[B+8352+(Q<<2)>>2]|0;T=p+(Q<<2)|0;c[T>>2]=S;if((S|0)>(R|0)){S=R-108887904-(c[B+7632+(Q<<2)>>2]|0)|0;U=S;V=(S|0)<0|0?-1:0;S=646457024;W=0;N2(U,V,S,W)|0;W=G<<1|0>>>31;S=W>>3;V=S;U=(S|0)<0|0?-1:0;S=1843614720;X=0;N2(V,U,S,X)|0;X=(G<<1|0>>>31)+W>>22;W=C+24+(Q<<2)|0;S=c[W>>2]|0;U=C+24+(Q+1<<2)|0;V=c[U>>2]|0;if((S|0)<(V|0)){Y=0;Z=S;while(1){S=c[B+(Z<<2)>>2]|0;_=(S|0)>0?S:-S|0;S=(_|0)>(Y|0)?_:Y;_=Z+1|0;if((_|0)<(V|0)){Y=S;Z=_}else{$=S;break}}}else{$=0}Z=HE($)|0;if((-2147483648-Z|0)<-581606912){Y=(Z-581606912>>23)+1|0;c[r+(Q<<2)>>2]=Y;aa=Y}else{c[r+(Q<<2)>>2]=-255;aa=-255}Y=(X|0)>(aa|0)?X:aa;if(v){Z=c[W>>2]|0;V=B+(Z<<2)|0;S=B+4096+(Z<<1)|0;_=q+(Z<<1)|0;ab=(c[U>>2]|0)-Z|0;Z=o+(Q<<2)|0;ac=KU(V,S,ab,Y)|0;c[Z>>2]=Y;ad=R+10802114|0;L14072:do{if((ac|0)>(ad|0)){ae=(ab|0)>0;af=S;ag=_;ah=ab<<1;ai=0;aj=ac;ak=Y;L14092:while(1){al=ak;am=ai;an=aj;while(1){if((an|0)<=(ad|0)){break L14092}ao=am+1|0;if((am|0)>=3){break L14092}ap=al+1|0;aq=KU(V,_,ab,ap)|0;if((aq|0)<(aj|0)){break}else{al=ap;am=ao;an=aq}}if(!ae){ai=ao;aj=aq;ak=ap;continue}NN(af|0,ag|0,ah)|0;ai=ao;aj=aq;ak=ap}if((aj|0)<=(ad|0)){ar=ak;as=aj;break}if(ae){at=ak;au=aj;av=0;aw=Y}else{ai=ak;an=aj;am=0;al=Y;while(1){if(!((am|0)<1&(al|0)>(aa|0))){ar=ai;as=an;break L14072}ax=al-1|0;ay=KU(V,_,ab,ax)|0;az=(ay|0)<(an|0);aA=az?ax:ai;aB=az?ay:an;c[Z>>2]=ax;if((ay|0)>(ad|0)){ai=aA;an=aB;am=am+1|0;al=ax}else{ar=aA;as=aB;break L14072}}}while(1){if(!((av|0)<1&(aw|0)>(aa|0))){ar=at;as=au;break L14072}al=aw-1|0;am=KU(V,_,ab,al)|0;if((am|0)<(au|0)){NN(af|0,ag|0,ah)|0;aC=am;aD=al}else{aC=au;aD=at}c[Z>>2]=al;if((am|0)>(ad|0)){at=aD;au=aC;av=av+1|0;aw=al}else{ar=aD;as=aC;break}}}else{ah=ac+10802114|0;ag=(ah|0)<(R|0)?ah:R;ah=Y+1|0;af=KU(V,_,ab,ah)|0;al=(af|0)<(ag|0);if((ab|0)>0){if(al){c[Z>>2]=ah;am=S;an=_;ai=ab<<1;NN(am|0,an|0,ai)|0;aE=af;aF=ah}else{aE=ac;aF=Y}ai=Y+2|0;an=KU(V,_,ab,ai)|0;if((an|0)<(ag|0)){c[Z>>2]=aF+1;am=S;aj=_;ak=(ab|0)>1?ab<<1:2;NN(am|0,aj|0,ak)|0;aG=an;aH=ai}else{aG=aE;aH=aF}ai=Y+3|0;an=KU(V,_,ab,ai)|0;if((an|0)>=(ag|0)){ar=aH;as=aG;break}c[Z>>2]=aH+1;ak=S;aj=_;am=(ab|0)>1?ab<<1:2;NN(ak|0,aj|0,am)|0;ar=ai;as=an;break}else{if(al){c[Z>>2]=ah;aI=af;aJ=ah}else{aI=ac;aJ=Y}ah=Y+2|0;af=KU(V,_,ab,ah)|0;if((af|0)<(ag|0)){c[Z>>2]=aJ+1;aK=af;aL=ah}else{aK=aI;aL=aJ}ah=Y+3|0;af=KU(V,_,ab,ah)|0;if((af|0)>=(ag|0)){ar=aL;as=aK;break}c[Z>>2]=aL+1;ar=ah;as=af;break}}}while(0);c[T>>2]=as;aM=ar}else{aM=Y}c[B+6384+(Q<<2)>>2]=aM;aN=c[J>>2]|0}else{aN=P}Z=O+1|0;if((Z|0)<(aN|0)){O=Z;P=aN}else{break}}aO=aN;aP=c[H>>2]|0}else{aO=K;aP=N}P=(c[I>>2]|0)+L|0;if((P|0)<(aP|0)){L=P;K=aO;N=aP}else{aQ=aP;break}}}else{aQ=M}if(w){N=(aQ|0)>0;if(N){K=0;do{c[s+(K<<2)>>2]=-2147483648;K=K+1|0;}while((K|0)<(aQ|0))}K=c[C+4>>2]|0;L=c[C+8>>2]|0;I_(u,aQ<<2);if(N){I=(L|0)>0;J=0;do{if(I){P=0;do{O=P+J|0;Z=B+8352+(O<<2)|0;if((c[Z>>2]|0)>(c[B+7872+(O<<2)>>2]|0)){ab=HE((c[C+24+(O+1<<2)>>2]|0)-(c[C+24+(O<<2)>>2]|0)<<24)|0;c[t+(O<<2)>>2]=(HH((c[B+7632+(O<<2)>>2]|0)-(-234881024-ab+(c[Z>>2]|0)>>2)|0)|0)>>1}P=P+1|0;}while((P|0)<(L|0))}J=J+K|0;}while((J|0)<(aQ|0))}J=c[H>>2]|0;if((J|0)>0){K=0;while(1){c[k+(K<<2)>>2]=2147483647;c[l+(K<<2)>>2]=2147483647;c[m+(K<<2)>>2]=2147483647;L=K+1|0;if((L|0)<(J|0)){K=L}else{aR=-1;aS=0;aT=-1;aU=0;aV=0;aW=2147483647;aX=2147483647;aY=J;aZ=0;break}}}else{aR=-1;aS=0;aT=-1;aU=0;aV=0;aW=2147483647;aX=2147483647;aY=J;aZ=0}while(1){K=aT;while(1){a_=K+1|0;if((a_|0)>=(aY|0)){a$=10186;break}a0=B+6384+(a_<<2)|0;a1=c[a0>>2]|0;if((a1|0)==-2147483648){K=a_}else{a$=10180;break}}L14144:do{if((a$|0)==10180){a$=0;K=(aT|0)>-1;do{if((aR|aT|0)>-1){L=c[B+6384+(aT<<2)>>2]|0;c[j>>2]=L;I=B+6384+(aR<<2)|0;N=c[I>>2]|0;P=(N|0)<(a1|0)?N:a1;Z=(N|0)>(a1|0)?N:a1;if(K){a2=L;a3=Z;a4=P;a5=I}else{a6=aS;a7=0;a8=P;a9=L;ba=aY;bb=Z;bc=a0;bd=I;be=a_;break L14144}}else{if((aR|0)==-1&K){I=c[B+6384+(aT<<2)>>2]|0;c[j>>2]=I;a2=I;a3=a1;a4=a1;a5=j;break}if(!K){a6=aS;a7=0;a8=aW;a9=aZ;ba=aY;bb=aX;bc=aV;bd=aU;be=a_;break L14144}I=c[r+(aT<<2)>>2]|0;bf=aU;bg=aV;bh=aX;bi=aZ;bj=(aW|0)>(I|0)?aW:I;a$=10191;break L14144}}while(0);K=c[r+(aT<<2)>>2]|0;bf=a5;bg=a0;bh=a3;bi=a2;bj=(a4|0)>(K|0)?a4:K;a$=10191}else if((a$|0)==10186){a$=0;K=(aT|0)>-1;Q=(aR|0)>-1;if((aR|aT|0)>-1&(a_|0)==(aY|0)){Y=c[B+6384+(aT<<2)>>2]|0;c[j>>2]=Y;T=B+6384+(aR<<2)|0;I=c[T>>2]|0;if(K){bk=Y;bl=I;bm=I;bn=j;bo=T}else{a6=aS;a7=0;a8=I;a9=Y;ba=aY;bb=I;bc=j;bd=T;be=aY;break}}else{if(K){bk=aZ;bl=aX;bm=aW;bn=aV;bo=aU}else{a6=aS;a7=0;a8=aW;a9=aZ;ba=aY;bb=aX;bc=aV;bd=aU;be=a_;break}}K=c[r+(aT<<2)>>2]|0;T=(bm|0)>(K|0)?bm:K;if(Q){bf=bo;bg=bn;bh=bl;bi=bk;bj=T;a$=10191}else{a6=aS;a7=0;a8=T;a9=bk;ba=aY;bb=bl;bc=bn;bd=bo;be=a_}}}while(0);L14159:do{if((a$|0)==10191){a$=0;T=-bi|0;if((bi|0)<=(bj|0)|(bi|0)>(bj+60|0)){a6=aS;a7=0;a8=bj;a9=bi;ba=aY;bb=bh;bc=bg;bd=bf;be=a_;break}Q=bh-60|0;if((bi|0)<(Q|0)){a6=aS;a7=0;a8=bj;a9=bi;ba=aY;bb=bh;bc=bg;bd=bf;be=a_;break}K=k+(aT<<2)|0;do{if((c[bf>>2]|0)==(c[K>>2]|0)){if((c[bg>>2]|0)!=(c[l+(aT<<2)>>2]|0)){break}if((aS|0)>=(c[m+(aT<<2)>>2]|0)){a6=aS;a7=0;a8=bj;a9=bi;ba=aY;bb=bh;bc=bg;bd=bf;be=a_;break L14159}}}while(0);I=c[C+24+(aT<<2)>>2]|0;Y=(c[C+24+(aT+1<<2)>>2]|0)-I|0;Z=s+(aT<<2)|0;L=c[Z>>2]|0;if((L|0)==-2147483648){P=((c[B+8352+(aT<<2)>>2]|0)-201326592-(c[B+7632+(aT<<2)>>2]|0)>>1)+46219344|0;c[Z>>2]=P;bp=P}else{bp=L}L=t+(aT<<2)|0;P=c[L>>2]|0;Z=bi<<24;N=N2(Z,(Z|0)<0|0?-1:0,805306368,0)|0;Z=bp-(N>>>31|G<<1)|0;if((Z|0)>50331647){N=N2(Z,(Z|0)<0|0?-1:0,P,(P|0)<0|0?-1:0)|0;bq=G;br=N}else{N=Z;ab=(Z|0)<0|0?-1:0;Z=1201210624;O=0;N2(N,ab,Z,O)|0;O=(G<<1|0>>>31)+22178274|0;Z=N2(O,(O|0)<0|0?-1:0,P,(P|0)<0|0?-1:0)|0;bq=G;br=Z}Z=bq<<1|0>>>31;P=Z;O=(Z|0)<0|0?-1:0;Z=1503238528;ab=0;N2(P,O,Z,ab)|0;ab=o+(aT<<2)|0;Z=aS-(G<<1|0>>>31)-((d[1979952+(bi+60-(c[bg>>2]|0))|0]|0)+(d[1979952+(60-bi+(c[bf>>2]|0))|0]|0)<<17)|0;O=B+(I<<2)|0;P=q+(I<<1)|0;N=p+(aT<<2)|0;_=B+6384+(aT<<2)|0;V=(Y|0)>0;ac=B+4096+(I<<1)|0;I=P;S=Y<<1;R=~bj;ad=(T|0)>(R|0);U=1;W=aS;X=0;af=bi;ah=c[ab>>2]|0;while(1){ag=af-1|0;c[j>>2]=ag;do{if((ag|0)>=(ah|0)|(ag|0)<(Q|0)){bs=U;bt=W;bu=X;bv=ah}else{al=c[L>>2]|0;an=ag<<24;ai=N2(an,(an|0)<0|0?-1:0,805306368,0)|0;an=bp-(ai>>>31|G<<1)|0;if((an|0)>50331647){ai=N2(al,(al|0)<0|0?-1:0,an,(an|0)<0|0?-1:0)|0;bw=G;bx=ai}else{ai=an;am=(an|0)<0|0?-1:0;an=1201210624;aj=0;N2(ai,am,an,aj)|0;aj=(G<<1|0>>>31)+22178274|0;an=N2(al,(al|0)<0|0?-1:0,aj,(aj|0)<0|0?-1:0)|0;bw=G;bx=an}an=bw<<1|0>>>31;aj=an;al=(an|0)<0|0?-1:0;an=1503238528;am=0;N2(aj,al,an,am)|0;am=Z+(G<<1|0>>>31)+((d[1979952+(af+59-(c[bg>>2]|0))|0]|0)+(d[1979952+(61-af+(c[bf>>2]|0))|0]|0)<<17)|0;if((am|0)>=1310720){bs=0;bt=W;bu=X;bv=ah;break}an=KU(O,P,Y,ag)|0;do{if((an|0)<(c[N>>2]|0)){c[_>>2]=ag;c[N>>2]=an;if(!V){by=am;bz=1;break}NN(ac|0,I|0,S)|0;by=am;bz=1}else{by=W;bz=X}}while(0);if((U|0)==0){bs=0;bt=by;bu=bz;bv=ah;break}c[ab>>2]=ag;bs=U;bt=by;bu=bz;bv=ag}}while(0);if((ag|0)>(bj|0)){U=bs;W=bt;X=bu;af=ag;ah=bv}else{break}}c[K>>2]=c[bf>>2];c[l+(aT<<2)>>2]=c[bg>>2];c[m+(aT<<2)>>2]=bt;a6=bt;a7=bu;a8=bj;a9=~(ad?T:R);ba=c[H>>2]|0;bb=bh;bc=bg;bd=bf;be=a_}}while(0);ah=(a7|0)==0;af=ah?be:-1;if((af|0)<(ba|0)){aR=ah?aT:-1;aS=a6;aT=af;aU=ah?bd:0;aV=ah?bc:0;aW=ah?a8:2147483647;aX=ah?bb:2147483647;aY=ba;aZ=a9}else{break}}J=D;do{if((ba|0)>0){ah=-2147483648;af=2147483647;X=0;while(1){W=c[B+6384+(X<<2)>>2]|0;if((W|0)==-2147483648){bA=af;bB=ah}else{bA=(af|0)<(W|0)?af:W;bB=(ah|0)>(W|0)?ah:W}W=X+1|0;if((W|0)<(ba|0)){ah=bB;af=bA;X=W}else{break}}if((bB|0)==-2147483648){bC=ba;break}if((bB|0)>(bA+60|0)){bC=ba;break}else{bD=0;bE=bB}while(1){NN(y|0,J|0,240)|0;X=bE-1|0;af=0;ah=bD;while(1){W=af;while(1){if((W|0)>=(ba|0)){bF=0;break}U=c[B+6384+(W<<2)>>2]|0;if((U|0)==-2147483648|(U|0)<=(X|0)){W=W+1|0}else{bF=1;break}}ag=W;do{ag=ag+1|0;if((ag|0)>=(ba|0)){break}U=c[B+6384+(ag<<2)>>2]|0;}while((U|0)==-2147483648|(U|0)>(X|0));L14212:do{if(bF){U=(W|0)<(ag|0);if(U){ab=W;while(1){if((c[B+6384+(ab<<2)>>2]|0)!=-2147483648){if((X|0)<(c[r+(ab<<2)>>2]|0)){bG=ah;break L14212}}S=ab+1|0;if((S|0)<(ag|0)){ab=S}else{bH=W;break}}do{ab=k+(bH<<2)|0;if((c[ab>>2]|0)!=-2147483648){c[ab>>2]=X}bH=bH+1|0;}while((bH|0)<(ag|0))}ab=KZ(D,z,ba,W,ag)|0;S=ab+ah+(K_(C,D,z,x,F,f,W,ag)|0)|0;if((S|0)>1310719|U^1){bG=ah;break}else{bI=0;bJ=0;bK=W}while(1){if((c[k+(bK<<2)>>2]|0)==-2147483648){bL=bJ;bM=bI;bN=bK+1|0}else{bO=((HH(c[p+(bK<<2)>>2]|0)|0)>>3)+bI|0;ab=bK+1|0;I=c[C+24+(bK<<2)>>2]|0;ac=KU(B+(I<<2)|0,q+(I<<1)|0,(c[C+24+(ab<<2)>>2]|0)-I|0,X)|0;c[l+(bK<<2)>>2]=ac;if((ac|0)>(c[B+7872+(bK<<2)>>2]|0)){a$=10240;break}bL=((HH(ac)|0)>>3)+bJ|0;bM=bO;bN=ab}if((bN|0)<(ag|0)){bI=bM;bJ=bL;bK=bN}else{bP=bL;bQ=bM;break}}if((a$|0)==10240){a$=0;bP=bO<<1;bQ=bO}U=(bP|0)>=(bQ|0);if(U){bG=U?ah:S;break}else{bR=W}while(1){U=B+6384+(bR<<2)|0;ab=bR+1|0;do{if((c[U>>2]|0)!=-2147483648){ac=c[C+24+(bR<<2)>>2]|0;I=(c[C+24+(ab<<2)>>2]|0)-ac|0;c[U>>2]=X;c[p+(bR<<2)>>2]=c[l+(bR<<2)>>2];if((I|0)<=0){break}V=B+4096+(ac<<1)|0;N=q+(ac<<1)|0;ac=I<<1;NN(V|0,N|0,ac)|0}}while(0);if((ab|0)<(ag|0)){bR=ab}else{bG=S;break}}}else{bG=ah}}while(0);if((ag|0)>(ba|0)){break}else{af=ag;ah=bG}}if((X|0)>(bA|0)){bD=bG;bE=X}else{break}}bC=c[H>>2]|0}else{bC=ba}}while(0);ah=(bC|0)>0;if(ah){af=2147483647;R=-2147483648;T=0;while(1){ad=c[B+6384+(T<<2)>>2]|0;if((ad|0)==-2147483648){bS=R;bT=af}else{bS=(R|0)>(ad|0)?R:ad;bT=(af|0)<(ad|0)?af:ad}ad=T+1|0;if((ad|0)<(bC|0)){af=bT;R=bS;T=ad}else{bU=bT;bV=bS;break}}}else{bU=2147483647;bV=-2147483648}T=bV-bU|0;R=(T|0)<58;af=bC<<2;ad=(T|0)>60;T=0;K=0;W=-2147483648;while(1){S=K;while(1){bW=B+6384+(S<<2)|0;bX=c[bW>>2]|0;if((S|0)>=(bC|0)){a$=10258;break}U=S+1|0;if((bX|0)==-2147483648){S=U}else{bY=1;bZ=U;break}}if((a$|0)==10258){a$=0;bY=0;bZ=S+1|0}L14261:do{if((bZ|0)<(bC|0)){U=bZ;while(1){ac=c[B+6384+(U<<2)>>2]|0;if(!((ac|0)==-2147483648|(ac|0)==(bX|0))){b_=ac;b$=U;break L14261}ac=U+1|0;if((ac|0)<(bC|0)){U=ac}else{b_=bX;b$=ac;break}}}else{b_=bX;b$=bZ}}while(0);U=(W|0)==-2147483648?bX:W;X=(U|0)>(b_|0)?U:b_;ac=(U|0)<(b_|0)?U:b_;U=(X|0)>(bX|0);N=U?X:bX;if((X|0)<(bX|0)){b0=X}else{b0=(bX|0)<(ac|0)?bX:ac}L14269:do{if(bY){if((N-b0|0)>=61){b1=T;break}if(U){ac=(S|0)<(b$|0);L14274:do{if(ac){X=S;V=bX;while(1){if((V|0)!=-2147483648){I=c[B+7872+(X<<2)>>2]|0;_=I;Y=(I|0)<0|0?-1:0;P=715827904;O=0;N2(_,Y,P,O)|0;O=G<<1|0>>>31;P=c[p+(X<<2)>>2]|0;Y=P;_=(P|0)<0|0?-1:0;P=715827904;Z=0;N2(Y,_,P,Z)|0;Z=(G<<2|0>>>30)+O|0;O=(c[B+8352+(X<<2)>>2]|0)-334396231|0;P=(Z|0)>(O|0)?Z:O;c[m+(X<<2)>>2]=(P|0)<(I|0)?P:I}I=X+1|0;if((I|0)>=(b$|0)){b2=1;b3=T;b4=bX;break L14274}X=I;V=c[B+6384+(I<<2)>>2]|0}}else{b2=1;b3=T;b4=bX}}while(0);L14281:while(1){V=b2;X=b4;while(1){b5=X;do{b5=b5+1|0;if((b5|0)>(N|0)){break L14281}NN(y|0,J|0,240)|0;if(ac){ag=S;do{I=k+(ag<<2)|0;if((c[I>>2]|0)!=-2147483648){c[I>>2]=b5}ag=ag+1|0;}while((ag|0)<(b$|0))}ag=KZ(D,z,bC,S,b$)|0;b6=ag+b3+(K_(C,D,z,x,F,f,S,b$)|0)|0;}while((b6|0)>=0);L14296:do{if(ac){ag=S;while(1){if((c[k+(ag<<2)>>2]|0)==-2147483648){b7=ag+1|0}else{ab=c[C+24+(ag<<2)>>2]|0;I=ag+1|0;b8=KU(B+(ab<<2)|0,q+(ab<<1)|0,(c[C+24+(I<<2)>>2]|0)-ab|0,b5)|0;c[l+(ag<<2)>>2]=b8;if((b8|0)>(c[m+(ag<<2)>>2]|0)){break}else{b7=I}}if((b7|0)<(b$|0)){ag=b7}else{b9=V;ca=1;break L14296}}b9=(b8|0)==(c[B+8352+(ag<<2)>>2]|0)?0:V;ca=0}else{b9=V;ca=1}}while(0);if((b9|0)==0){break L14281}if((ca|0)==0){V=b9;X=b5}else{break}}if(ac){cb=S}else{b2=b9;b3=b6;b4=b5;continue}while(1){X=B+6384+(cb<<2)|0;do{if((c[X>>2]|0)==-2147483648){cc=cb+1|0}else{c[X>>2]=b5;c[p+(cb<<2)>>2]=c[l+(cb<<2)>>2];V=cb+1|0;I=C+24+(V<<2)|0;ab=C+24+(cb<<2)|0;P=c[ab>>2]|0;if(((c[I>>2]|0)-P|0)>0){cd=0;ce=P}else{cc=V;break}while(1){P=cd+ce|0;b[B+4096+(P<<1)>>1]=b[q+(P<<1)>>1]|0;P=cd+1|0;O=c[ab>>2]|0;if((P|0)<((c[I>>2]|0)-O|0)){cd=P;ce=O}else{cc=V;break}}}}while(0);if((cc|0)<(b$|0)){cb=cc}else{b2=b9;b3=b6;b4=b5;continue L14281}}}cf=b3;cg=c[bW>>2]|0}else{cf=T;cg=bX}do{if((b0|0)>=(bX|0)|(bX|0)!=(cg|0)|ad){ch=cf}else{NN(y|0,J|0,240)|0;ac=(S|0)<(b$|0);if(ac){X=S;V=0;while(1){I=k+(X<<2)|0;if((c[I>>2]|0)==-2147483648){ci=V}else{c[I>>2]=b0;ci=(b0|0)<(c[r+(X<<2)>>2]|0)?1:V}I=X+1|0;if((I|0)<(b$|0)){X=I;V=ci}else{break}}if((ci|0)!=0){ch=cf;break}}V=KZ(D,z,bC,S,b$)|0;X=V+cf+(K_(C,D,z,x,F,f,S,b$)|0)|0;if((X|0)>=0){ch=cf;break}L14328:do{if(ac){V=0;I=0;ab=S;while(1){if((c[k+(ab<<2)>>2]|0)==-2147483648){cj=I;ck=V;cl=ab+1|0}else{cm=((HH(c[p+(ab<<2)>>2]|0)|0)>>3)+V|0;ag=c[C+24+(ab<<2)>>2]|0;O=ab+1|0;P=KU(B+(ag<<2)|0,q+(ag<<1)|0,(c[C+24+(O<<2)>>2]|0)-ag|0,b0)|0;c[l+(ab<<2)>>2]=P;if((P|0)>(c[B+7872+(ab<<2)>>2]|0)){break}cj=((HH(P)|0)>>3)+I|0;ck=cm;cl=O}if((cl|0)<(b$|0)){V=ck;I=cj;ab=cl}else{cn=cj;co=ck;break L14328}}cn=cm<<1;co=cm}else{cn=0;co=0}}while(0);ab=co;I=(co|0)<0|0?-1:0;V=1717986944;O=0;N2(ab,I,V,O)|0;O=(cn|0)>=(G<<1|0>>>31|0);if(O|ac^1){ch=O?cf:X;break}else{cp=S}while(1){O=B+6384+(cp<<2)|0;do{if((c[O>>2]|0)==-2147483648){cq=cp+1|0}else{c[O>>2]=b0;c[p+(cp<<2)>>2]=c[l+(cp<<2)>>2];V=cp+1|0;I=C+24+(V<<2)|0;ab=C+24+(cp<<2)|0;P=c[ab>>2]|0;if(((c[I>>2]|0)-P|0)>0){cr=0;cs=P}else{cq=V;break}while(1){P=cr+cs|0;b[B+4096+(P<<1)>>1]=b[q+(P<<1)>>1]|0;P=cr+1|0;ag=c[ab>>2]|0;if((P|0)<((c[I>>2]|0)-ag|0)){cr=P;cs=ag}else{cq=V;break}}}}while(0);if((cq|0)<(b$|0)){cp=cq}else{ch=X;break}}}}while(0);if(!R){b1=ch;break}if(ah){NN(y|0,J|0,af)|0}X=k+(S<<2)|0;ac=(S|0)<(b$|0);O=ac^1;V=ch;I=1;while(1){ab=c[X>>2]|0;ag=ab-1|0;L14353:do{if(ac){P=S;Z=ab;while(1){if((Z|0)!=-2147483648){c[k+(P<<2)>>2]=ag}_=P+1|0;if((_|0)>=(b$|0)){break L14353}P=_;Z=c[k+(_<<2)>>2]|0}}}while(0);ab=(KZ(D,z,bC,S,b$)|0)+V|0;L14361:do{if((ab|0)>0|O){ct=V}else{Z=0;P=0;_=S;while(1){if((c[k+(_<<2)>>2]|0)==-2147483648){cu=P;cv=Z;cw=_+1|0}else{Y=c[C+24+(_<<2)>>2]|0;L=_+1|0;Q=l+(_<<2)|0;KW(B+(Y<<2)|0,B+4096+(Y<<1)|0,(c[C+24+(L<<2)>>2]|0)-Y|0,ag,n,Q);Y=c[p+(_<<2)>>2]|0;am=HH(Y)|0;an=HH(c[Q>>2]|0)|0;if((c[Q>>2]|0)>(Y+5572475|0)){ct=V;break L14361}if((c[n>>2]|0)<((c[B+8352+(_<<2)>>2]|0)-16722942|0)){ct=V;break L14361}else{cu=(an>>3)+P|0;cv=(am>>3)+Z|0;cw=L}}if((cw|0)<(b$|0)){Z=cv;P=cu;_=cw}else{break}}_=(cu|0)>=(cv|0);if(_){ct=_?V:ab;break}else{cx=S}while(1){_=B+6384+(cx<<2)|0;if((c[_>>2]|0)!=-2147483648){c[_>>2]=ag;c[p+(cx<<2)>>2]=c[l+(cx<<2)>>2]}_=cx+1|0;if((_|0)<(b$|0)){cx=_}else{ct=ab;break}}}}while(0);if((I|0)>=3){b1=ct;break L14269}V=ct;I=I+1|0}}else{b1=T}}while(0);if((b$|0)>(bC|0)){break}else{T=b1;K=b$;W=bX}}cy=c[H>>2]|0}else{cy=aQ}do{if((cy|0)>0){W=C+8|0;K=c[W>>2]|0;T=(K|0)>0;D=C+4|0;af=c[D>>2]|0;J=2147483647;ah=0;while(1){if(T){R=J;F=0;while(1){ad=c[B+6384+(F+ah<<2)>>2]|0;if((ad|0)==-2147483648){cz=R}else{cz=(R|0)<(ad|0)?R:ad}ad=F+1|0;if((ad|0)<(K|0)){R=cz;F=ad}else{cA=cz;break}}}else{cA=J}F=ah+af|0;if((F|0)<(cy|0)){J=cA;ah=F}else{break}}ah=cA+60|0;J=0;T=K;F=cy;R=af;ad=K;while(1){if((T|0)>0){S=0;do{N=S+J|0;U=B+6384+(N<<2)|0;I=c[U>>2]|0;do{if((I|0)!=-2147483648&(ah|0)<(I|0)){c[U>>2]=ah;if(!v){break}V=c[C+24+(N<<2)>>2]|0;c[p+(N<<2)>>2]=KU(B+(V<<2)|0,B+4096+(V<<1)|0,(c[C+24+(N+1<<2)>>2]|0)-V|0,ah)|0}}while(0);S=S+1|0;cB=c[W>>2]|0;}while((S|0)<(cB|0));cC=cB;cD=c[H>>2]|0;cE=c[D>>2]|0;cF=cB}else{cC=T;cD=F;cE=R;cF=ad}S=cE+J|0;if((S|0)<(cD|0)){J=S;T=cC;F=cD;R=cE;ad=cF}else{break}}if((cD|0)<=0){a$=10370;break}ad=(cF|0)>0;R=-2147483648;F=0;while(1){if(ad){T=R;J=0;while(1){ah=c[B+6384+(J+F<<2)>>2]|0;K=(T|0)>(ah|0)?T:ah;ah=J+1|0;if((ah|0)<(cF|0)){T=K;J=ah}else{cG=K;break}}}else{cG=R}J=F+cE|0;if((J|0)<(cD|0)){R=cG;F=J}else{break}}if((cG|0)==-2147483648){a$=10370;break}c[E>>2]=cG;F=c[H>>2]|0;if((F|0)<=0){break}R=0;ad=c[W>>2]|0;J=F;while(1){if((ad|0)>0){F=0;do{T=F+R|0;K=B+6384+(T<<2)|0;ah=c[K>>2]|0;do{if((ah|0)==-2147483648){c[K>>2]=0;af=c[C+24+(T<<2)>>2]|0;S=C+24+(T+1<<2)|0;if((af|0)<(c[S>>2]|0)){cH=af}else{break}do{c[B+(cH<<2)>>2]=0;cH=cH+1|0;}while((cH|0)<(c[S>>2]|0))}else{c[K>>2]=cG-ah}}while(0);F=F+1|0;cI=c[W>>2]|0;}while((F|0)<(cI|0));cJ=cI;cK=c[H>>2]|0}else{cJ=ad;cK=J}F=(c[D>>2]|0)+R|0;if((F|0)<(cK|0)){R=F;ad=cJ;J=cK}else{break}}}else{a$=10370}}while(0);do{if((a$|0)==10370){a$=0;c[E>>2]=0;J=c[H>>2]|0;if((J|0)<=0){break}ad=C+8|0;R=C+4|0;D=0;W=c[ad>>2]|0;F=J;while(1){if((W|0)>0){J=0;do{ah=J+D|0;c[B+6384+(ah<<2)>>2]=0;K=c[C+24+(ah<<2)>>2]|0;T=C+24+(ah+1<<2)|0;if((K|0)<(c[T>>2]|0)){ah=K;do{c[B+(ah<<2)>>2]=0;ah=ah+1|0;}while((ah|0)<(c[T>>2]|0))}J=J+1|0;cL=c[ad>>2]|0;}while((J|0)<(cL|0));cM=cL;cN=c[H>>2]|0}else{cM=W;cN=F}J=(c[R>>2]|0)+D|0;if((J|0)<(cN|0)){D=J;W=cM;F=cN}else{break}}}}while(0);A=A+1|0;if((A|0)>=(g|0)){break}}i=h;return}function KZ(a,b,e,f,g){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;h=f;while(1){if((h|0)>=(g|0)){i=f;break}if((c[a+(h<<2)>>2]|0)==-2147483648){h=h+1|0}else{i=f;break}}while(1){j=i-1|0;if((i|0)<=0){k=h;l=0;break}m=c[a+(j<<2)>>2]|0;if((m|0)==-2147483648){i=j}else{n=10387;break}}if((n|0)==10387){k=h;l=(d[1979952+((c[b+(j<<2)>>2]|0)+60-(c[b+(h<<2)>>2]|0))|0]|0)-(d[1979952+(m+60-(c[a+(h<<2)>>2]|0))|0]|0)|0}L14452:while(1){h=k;do{h=h+1|0;if((h|0)>=(g|0)){o=g;break L14452}p=c[a+(h<<2)>>2]|0;}while((p|0)==-2147483648);m=(d[1979952+((c[b+(k<<2)>>2]|0)+60-(c[b+(h<<2)>>2]|0))|0]|0)+l-(d[1979952+(60-p+(c[a+(k<<2)>>2]|0))|0]|0)|0;k=h;l=m}while(1){if((o|0)>=(e|0)){q=l;n=10397;break}r=c[a+(o<<2)>>2]|0;if((r|0)==-2147483648){o=o+1|0}else{break}}if((n|0)==10397){s=q<<17;return s|0}q=(d[1979952+((c[b+(k<<2)>>2]|0)+60-(c[b+(o<<2)>>2]|0))|0]|0)+l-(d[1979952+(60-r+(c[a+(k<<2)>>2]|0))|0]|0)|0;s=q<<17;return s|0}function K_(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;if((h|0)>=(i|0)){j=0;return j|0}k=a+2116|0;a=h;h=0;while(1){l=b+(a<<2)|0;m=c[l>>2]|0;if((m|0)==-2147483648){n=h}else{o=e+(a<<2)|0;p=c[o>>2]|0;if((p|0)==-2147483648){q=((c[(c[k>>2]|0)+(a<<2)>>2]|0)-201326592-(c[f+(a<<2)>>2]|0)>>1)+46219344|0;c[o>>2]=q;r=c[l>>2]|0;s=q}else{r=m;s=p}p=r<<24;m=N2(p,(p|0)<0|0?-1:0,805306368,0)|0;p=s-(m>>>31|G<<1)|0;m=c[d+(a<<2)>>2]<<24;q=N2(m,(m|0)<0|0?-1:0,805306368,0)|0;m=s-(q>>>31|G<<1)|0;if((p|0)>50331647){t=p}else{q=p;l=(p|0)<0|0?-1:0;p=1201210624;o=0;N2(q,l,p,o)|0;t=(G<<1|0>>>31)+22178274|0}if((m|0)>50331647){u=m}else{o=m;p=(m|0)<0|0?-1:0;m=1201210624;l=0;N2(o,p,m,l)|0;u=(G<<1|0>>>31)+22178274|0}l=c[g+(a<<2)>>2]|0;m=u-t|0;p=l;o=(l|0)<0|0?-1:0;l=m;q=(m|0)<0|0?-1:0;N2(l,q,p,o)|0;o=G<<1|0>>>31;p=o;q=(o|0)<0|0?-1:0;o=1503238528;l=0;N2(p,q,o,l)|0;n=(G<<1|0>>>31)+h|0}l=a+1|0;if((l|0)<(i|0)){a=l;h=n}else{j=n;break}}return j|0}function K$(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0;if((a|0)>1){f=1;g=c[e>>2]|0;do{h=e+(f<<2)|0;i=c[h>>2]|0;j=c[d+(f<<2)>>2]|0;N2(j,(j|0)<0|0?-1:0,g,(g|0)<0|0?-1:0)|0;j=G<<1|0>>>31;g=(i|0)>(j|0)?i:j;c[h>>2]=g;f=f+1|0;}while((f|0)<(a|0))}f=a-2|0;if((f|0)<=-1){return}g=f;f=c[e+(a-1<<2)>>2]|0;while(1){a=e+(g<<2)|0;d=c[a>>2]|0;h=c[b+(g<<2)>>2]|0;N2(h,(h|0)<0|0?-1:0,f,(f|0)<0|0?-1:0)|0;h=G<<1|0>>>31;j=(d|0)>(h|0)?d:h;c[a>>2]=j;if((g|0)>0){g=g-1|0;f=j}else{break}}return}function K0(a,d,e,f,g,h,j){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;k=i;i=i+4096|0;l=k|0;m=c[h+(g<<2)>>2]|0;if((j|0)==0){i=k;return}j=l|0;M8(a,m,j);if((m|0)>1){n=1;o=c[j>>2]|0;do{p=l+(n<<2)|0;q=c[p>>2]|0;N2(q,(q|0)<0|0?-1:0,1610612736,0)|0;o=(o>>>3|((o|0)<0|0?-1:0)<<29)+G<<1;c[p>>2]=o;n=n+1|0;}while((n|0)<(m|0))}if((g|0)>0){r=0;s=j;t=a}else{i=k;return}while(1){a=(c[d+(r<<2)>>2]|0)-4|0;j=(a|0)<0?0:a;a=r+1|0;m=(c[h+(a<<2)>>2]|0)-(c[h+(r<<2)>>2]|0)|0;do{if((m|0)>0){n=m;o=0;l=s;p=t;while(1){q=n-1|0;u=c[p>>2]<<j;v=u;w=(u|0)<0|0?-1:0;N2(v,w,v,w)|0;w=c[l>>2]|0;N2(G,0,w,(w|0)<0|0?-1:0)|0;x=G+o|0;if((q|0)>0){n=q;o=x;l=l+4|0;p=p+4|0}else{break}}p=s+(m<<2)|0;l=t+(m<<2)|0;if((x|0)==0){y=l;z=p;A=10434;break}o=HE(x)|0;n=o-(c[e+(r<<2)>>2]|0)+(100663296-(j<<26))|0;if((n|0)<=-111465353){b[f+(r<<1)>>1]=32767;B=l;C=p;break}if((n|0)<1){o=n;q=(n|0)<0|0?-1:0;n=-646457015;w=-1;N2(o,q,n,w)|0;b[f+(r<<1)>>1]=(G>>>9|0<<23)&65535;B=l;C=p;break}else{b[f+(r<<1)>>1]=0;B=l;C=p;break}}else{y=t;z=s;A=10434}}while(0);if((A|0)==10434){A=0;b[f+(r<<1)>>1]=32767;B=y;C=z}if((a|0)<(g|0)){r=a;s=C;t=B}else{break}}i=k;return}function K1(a,d,f,g,h,i,j,k,l){a=a|0;d=d|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0;c[j>>2]=2;if((f|0)==0){if((g|0)==2){m=(i*3|0)>>2}else{m=0}n=i-m|0;o=a;p=i;q=n;r=n}else if((f|0)==1){o=a;p=i;q=i;r=i>>3}else if((f|0)==2){n=i>>3;o=a+(((n|0)/2|0)+(n*3|0)<<1)|0;p=n;q=n;r=n}else if((f|0)==3){o=a;p=i;q=i>>3;r=i}else{s=-1;return s|0}a=p-r>>1;f=p-q>>1;n=HC(q,c[h>>2]|0)|0;m=HC(r,g)|0;do{if((k|0)==2){t=(i|0)==512?1985456:1988528;u=(i|0)/4|0;if((i|0)>3){v=((i*3|0|0)/4|0)+i|0;w=(i|0)/2|0;x=w+i|0;y=i<<1;z=i-1|0;A=0;B=0;while(1){C=B-1|0;D=b[o+(C+v<<1)>>1]|0;E=ac(b[t+(C+w<<1)>>1]|0,D)|0;F=w+A|0;H=b[o+(v+A<<1)>>1]|0;I=(ac(b[t+(F<<1)>>1]|0,H)|0)+E<<1;E=C+x|0;C=ac(b[t+(E<<1)>>1]|0,D)|0;D=(ac(b[t+(x+A<<1)>>1]|0,H)|0)+C|0;C=l+(F<<2)|0;F=c[C>>2]|0;H=A+y|0;J=e[t+(H<<1)>>1]<<16;N2(J,(J|0)<0|0?-1:0,F,(F|0)<0|0?-1:0)|0;F=D+(G>>1)|0;D=l+(A<<2)|0;c[C>>2]=c[D>>2];c[D>>2]=I;I=c[C>>2]|0;C=l+(E<<2)|0;E=c[C>>2]|0;D=e[t+(H+w<<1)>>1]<<16;N2(D,(D|0)<0|0?-1:0,E,(E|0)<0|0?-1:0)|0;c[d+(A<<2)>>2]=(G>>1)+I;c[d+(z+B<<2)>>2]=F;c[C>>2]=F;F=A+1|0;C=~A;if((F|0)<(u|0)){A=F;B=C}else{K=w;break}}}else{K=(i|0)/2|0}if((u|0)>=(K|0)){break}w=(i*3|0|0)/4|0;B=i-1|0;A=i<<1;z=i-u|0;y=u;do{x=B-y|0;v=b[o+(x+w<<1)>>1]|0;C=ac(v<<1,b[t+(K+~y<<1)>>1]|0)|0;F=x+K|0;I=ac(b[t+(F<<1)>>1]|0,v)|0;v=K+y|0;E=l+(v<<2)|0;D=c[E>>2]|0;H=y+A|0;J=e[t+(H<<1)>>1]<<16;N2(J,(J|0)<0|0?-1:0,D,(D|0)<0|0?-1:0)|0;D=(G>>1)+I|0;I=l+(y<<2)|0;J=c[I>>2]|0;c[E>>2]=(ac(b[o+(z+y<<1)>>1]<<1,b[t+(v<<1)>>1]|0)|0)+J;c[I>>2]=C;C=c[E>>2]|0;E=l+(F<<2)|0;F=c[E>>2]|0;I=e[t+(H+K<<1)>>1]<<16;N2(I,(I|0)<0|0?-1:0,F,(F|0)<0|0?-1:0)|0;c[d+(y<<2)>>2]=(G>>1)+C;c[d+(x<<2)>>2]=D;c[E>>2]=D;y=y+1|0;}while((y|0)<(K|0))}else{if((f|0)>0){y=p-1|0;t=(p|0)/2|0;z=0;do{c[d+(t+z<<2)>>2]=-(b[o+(y-z<<1)>>1]|0)<<15;z=z+1|0;}while((z|0)<(f|0))}z=(q|0)/2|0;if((q|0)>1){y=p-1-f|0;t=(p|0)/2|0;A=0;do{w=A+f|0;B=n+(A<<2)|0;u=ac(b[B+2>>1]|0,b[o+(w<<1)>>1]|0)|0;c[d+(w+t<<2)>>2]=u-(ac(b[B>>1]|0,b[o+(y-A<<1)>>1]|0)|0);A=A+1|0;}while((A|0)<(z|0))}if((a|0)>0){z=(p|0)/2|0;A=0;do{c[d+(z+~A<<2)>>2]=-(b[o+(A+p<<1)>>1]|0)<<15;A=A+1|0;}while((A|0)<(a|0))}A=(r|0)/2|0;if((r|0)<=1){break}z=a+p|0;y=(p<<1)-1-a|0;t=(p|0)/2|0;B=~a;u=0;w=0;while(1){D=m+(u<<2)|0;E=ac(b[D>>1]|0,b[o+(z+u<<1)>>1]|0)|0;c[d+(w+B+t<<2)>>2]=-((ac(b[D+2>>1]|0,b[o+(y+w<<1)>>1]|0)|0)+E|0);E=u+1|0;D=~u;if((E|0)<(A|0)){u=E;w=D}else{break}}}}while(0);La(d,p,j);c[h>>2]=g;s=0;return s|0}function K2(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;c[a+148>>2]=b;c[a+156>>2]=d;c[a+152>>2]=e;c[a+160>>2]=f;return 0}function K3(b,e,f,g,h){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0;if((e|0)==0){i=1928608}else if((e|0)==1){i=1928592}else if((e|0)==2){i=1928576}else{j=-1;return j|0}c[b+164>>2]=i;e=d[i+7|0]|0;c[b+128>>2]=e-1;c[b+132>>2]=0;c[b+136>>2]=f;c[b+140>>2]=g;a[b+144|0]=0;k=a[i|0]|0;l=k&255;m=ac(l<<3,e)|0;if(m>>>0>(c[b+156>>2]|0)>>>0){j=-2;return j|0}m=a[i+8|0]|0;i=m&255;n=ac(g+f+(l*-2|0)<<2,i)|0;if(n>>>0>(c[b+160>>2]|0)>>>0){j=-2;return j|0}if(k<<24>>24==0){o=0}else{n=e<<1;p=0;q=c[b+148>>2]|0;while(1){c[b+(p<<2)>>2]=q;c[b+12+(p<<2)>>2]=q+(e<<2);r=p+1|0;if((r|0)<(l|0)){p=r;q=q+(n<<2)|0}else{o=k;break}}}if(m<<24>>24==0){s=0}else{k=o&255;n=f-k|0;q=g-k+n|0;k=0;p=c[b+152>>2]|0;while(1){c[b+24+(k<<2)>>2]=p;c[b+76+(k<<2)>>2]=p+(n<<2);l=k+1|0;if((l|0)<(i|0)){k=l;p=p+(q<<2)|0}else{s=m;break}}}if((h|0)==0){j=0;return j|0}h=o&255;if(o<<24>>24==0){t=0}else{m=e<<2;e=0;while(1){I_(c[b+(e<<2)>>2]|0,m);I_(c[b+12+(e<<2)>>2]|0,m);q=e+1|0;if((q|0)<(h|0)){e=q}else{t=o;break}}}if((h|0)>=(f|0)|s<<24>>24==0){j=0;return j|0}h=t&255;t=f-h<<2;f=g-h<<2;h=s&255;s=0;while(1){I_(c[b+24+(s<<2)>>2]|0,t);I_(c[b+76+(s<<2)>>2]|0,f);g=s+1|0;if((g|0)<(h|0)){s=g}else{j=0;break}}return j|0}function K4(b,e,f,g,h){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0;j=i;i=i+96|0;k=j|0;l=b+164|0;m=c[l>>2]|0;n=a[m|0]|0;o=n&255;p=b+128|0;q=c[p>>2]|0;r=q+1|0;s=(r|0)<(d[m+7|0]|0)?r:0;r=c[m+12>>2]|0;m=r+(s<<2)|0;L14605:do{if(n<<24>>24==0){t=0}else{u=r+(s+1<<2)|0;v=r+(s+11<<2)|0;w=r+(s+3<<2)|0;x=r+(s+9<<2)|0;y=r+(s+5<<2)|0;z=r+(s+7<<2)|0;A=r+(s+6<<2)|0;B=r+(s+4<<2)|0;C=r+(s+8<<2)|0;D=r+(s+12<<2)|0;E=k;F=-k&4;H=E+F|0;I=E+(F+4)|0;J=E+(F|8)|0;K=E+(F+12)|0;L=E+(F|16)|0;M=E+(F+20)|0;N=E+(F|24)|0;O=E+(F+28)|0;P=r+(s+10<<2)|0;Q=r+(s+2<<2)|0;R=E+(F|32)|0;S=E+(F+36)|0;T=E+(F|40)|0;U=E+(F+44)|0;V=E+(F|48)|0;W=E+(F+52)|0;X=E+(F|56)|0;Y=E+(F+60)|0;F=0;E=0;while(1){Z=b+(E<<2)|0;c[(c[Z>>2]|0)+(q<<2)>>2]=c[e+(E<<2)>>2];_=b+12+(E<<2)|0;c[(c[_>>2]|0)+(q<<2)>>2]=c[f+(E<<2)>>2];$=c[Z>>2]|0;Z=c[_>>2]|0;_=g+(F<<2)|0;aa=h+(F<<2)|0;ab=a[(c[l>>2]|0)+4+E|0]|0;L14609:do{switch(ab<<24>>24|0){case 8:case-8:{ac=c[A>>2]|0;ad=c[$+(ac<<2)>>2]|0;c[H>>2]=ad>>>4|((ad|0)<0|0?-1:0)<<28;ad=c[Z+(ac<<2)>>2]|0;c[I>>2]=ad>>>4|((ad|0)<0|0?-1:0)<<28;ad=c[z>>2]|0;ac=c[$+(ad<<2)>>2]|0;ae=c[Z+(ad<<2)>>2]|0;ad=ac;af=(ac|0)<0|0?-1:0;N2(ad,af,233963520,0)|0;ac=G;ag=ae;ah=(ae|0)<0|0?-1:0;N2(ag,ah,-96927744,-1)|0;ae=ac-G|0;N2(ad,af,-96927744,-1)|0;af=G;N2(ag,ah,233963520,0)|0;c[J>>2]=ae;c[K>>2]=G+af;af=c[m>>2]|0;ae=c[$+(af<<2)>>2]|0;ah=c[Z+(af<<2)>>2]|0;af=ae;ag=(ae|0)<0|0?-1:0;N2(af,ag,-11337728,-1)|0;ae=G;ad=ah;ac=(ah|0)<0|0?-1:0;N2(ad,ac,11337728,0)|0;ah=G;N2(af,ag,11337728,0)|0;ag=G;N2(ad,ac,-11337728,-1)|0;ac=G;ad=c[C>>2]|0;af=c[$+(ad<<2)>>2]|0;ai=c[Z+(ad<<2)>>2]|0;ad=af;aj=(af|0)<0|0?-1:0;N2(ad,aj,150077440,0)|0;af=G;ak=ai;al=(ai|0)<0|0?-1:0;N2(ak,al,-150077440,-1)|0;ai=G;N2(ad,aj,-150077440,-1)|0;aj=G;N2(ak,al,150077440,0)|0;c[L>>2]=ae-ah+af-ai;c[M>>2]=ac+ag+aj+G;aj=c[u>>2]|0;ag=c[$+(aj<<2)>>2]|0;ac=c[Z+(aj<<2)>>2]|0;aj=ag;ai=(ag|0)<0|0?-1:0;N2(aj,ai,-18677760,-1)|0;ag=G;af=ac;ah=(ac|0)<0|0?-1:0;N2(af,ah,45023232,0)|0;ac=G;N2(aj,ai,45023232,0)|0;ai=G;N2(af,ah,-18677760,-1)|0;ah=G;af=c[x>>2]|0;aj=c[$+(af<<2)>>2]|0;ae=c[Z+(af<<2)>>2]|0;af=aj;al=(aj|0)<0|0?-1:0;N2(af,al,59703296,0)|0;aj=G;ak=ae;ad=(ae|0)<0|0?-1:0;N2(ak,ad,-144179200,-1)|0;ae=G;N2(af,al,-144179200,-1)|0;al=G;N2(ak,ad,59703296,0)|0;ad=ag-ac+aj-ae|0;c[N>>2]=ad;ae=ah+ai+al+G|0;c[O>>2]=ae;al=c[P>>2]|0;ai=c[Z+(al<<2)>>2]|0;N2(ai,(ai|0)<0|0?-1:0,97648640,0)|0;ai=G;ah=c[Q>>2]|0;aj=c[Z+(ah<<2)>>2]|0;N2(aj,(aj|0)<0|0?-1:0,97648640,0)|0;aj=ai-G|0;ai=c[$+(ah<<2)>>2]|0;N2(ai,(ai|0)<0|0?-1:0,97648640,0)|0;ai=G;ah=c[$+(al<<2)>>2]|0;N2(ah,(ah|0)<0|0?-1:0,97648640,0)|0;ah=ai-G|0;ai=c[w>>2]|0;al=c[$+(ai<<2)>>2]|0;ac=c[Z+(ai<<2)>>2]|0;ai=al;ag=(al|0)<0|0?-1:0;N2(ai,ag,59703296,0)|0;al=G;ak=ac;af=(ac|0)<0|0?-1:0;N2(ak,af,144179200,0)|0;ac=G;N2(ai,ag,144179200,0)|0;ag=G;N2(ak,af,59703296,0)|0;af=G;ak=c[v>>2]|0;ai=c[$+(ak<<2)>>2]|0;am=c[Z+(ak<<2)>>2]|0;ak=ai;an=(ai|0)<0|0?-1:0;N2(ak,an,-18677760,-1)|0;ai=G;ao=am;ap=(am|0)<0|0?-1:0;N2(ao,ap,-45023232,-1)|0;am=G;N2(ak,an,-45023232,-1)|0;an=G;N2(ao,ap,-18677760,-1)|0;ap=al-ac+ai-am|0;am=af+ag+an+G|0;an=c[B>>2]|0;ag=c[$+(an<<2)>>2]|0;af=c[Z+(an<<2)>>2]|0;N2(ag,(ag|0)<0|0?-1:0,150077440,0)|0;ag=G;N2(af,(af|0)<0|0?-1:0,150077440,0)|0;af=G;an=c[D>>2]|0;ai=c[$+(an<<2)>>2]|0;ac=c[Z+(an<<2)>>2]|0;N2(ai,(ai|0)<0|0?-1:0,-11337728,-1)|0;ai=G;N2(ac,(ac|0)<0|0?-1:0,-11337728,-1)|0;ac=G;an=ag-af+ai-ac|0;al=af+ag+ai+ac|0;ac=c[y>>2]|0;ai=c[$+(ac<<2)>>2]|0;ag=c[Z+(ac<<2)>>2]|0;ac=ai;af=(ai|0)<0|0?-1:0;N2(ac,af,233963520,0)|0;ai=G;ao=ag;ak=(ag|0)<0|0?-1:0;N2(ao,ak,96927744,0)|0;ag=ai-G|0;N2(ac,af,96927744,0)|0;af=G;N2(ao,ak,233963520,0)|0;ak=G+af|0;af=(c[H>>2]|0)+aj>>1;ao=(c[I>>2]|0)+ah>>1;ac=(c[L>>2]|0)+an>>1;ai=(c[M>>2]|0)+al>>1;aq=af-aj|0;aj=ac-an|0;an=ao-ah|0;ah=ai-al|0;al=(c[J>>2]|0)+ap>>1;ar=(c[K>>2]|0)+am>>1;as=ag+ad>>1;ad=ak+ae>>1;ae=al-ap|0;ap=as-ag|0;ag=ar-am|0;am=ad-ak|0;ak=am+ae|0;at=ae-am|0;am=ag-ap|0;ae=ag+ap|0;ap=ac+af>>1;ag=ai+ao>>1;au=as+al>>1;av=au+ap|0;c[H>>2]=av;aw=ar+ad>>1;ax=aw+ag|0;c[I>>2]=ax;ay=ap-au|0;c[R>>2]=ay;au=ag-aw|0;c[S>>2]=au;aw=af-ac>>1;ac=ao-ai>>1;ai=ar-ad>>1;ad=ai+aw|0;c[L>>2]=ad;ar=al-as>>1;as=ac-ar|0;c[M>>2]=as;al=aw-ai|0;c[V>>2]=al;ai=ar+ac|0;c[W>>2]=ai;ac=am+ak|0;N2(ac,(ac|0)<0|0?-1:0,1518469120,0)|0;ac=G;ar=am-ak|0;N2(ar,(ar|0)<0|0?-1:0,1518469120,0)|0;ar=G;ak=ah+aq>>1;am=ac+ak|0;c[J>>2]=am;aw=an-aj>>1;ao=ar+aw|0;c[K>>2]=ao;af=ak-ac|0;c[T>>2]=af;ac=aw-ar|0;c[U>>2]=ac;ar=ae-at|0;N2(ar,(ar|0)<0|0?-1:0,1518469120,0)|0;ar=G;aw=ae+at|0;N2(aw,(aw|0)<0|0?-1:0,1518469120,0)|0;aw=G;at=aq-ah>>1;ah=ar+at|0;c[N>>2]=ah;aq=an+aj>>1;aj=aq-aw|0;c[O>>2]=aj;an=at-ar|0;c[X>>2]=an;ar=aw+aq|0;c[Y>>2]=ar;if(ab<<24>>24>-1){c[_>>2]=av<<3;c[aa>>2]=ax<<3;aq=F+1|0;c[g+(aq<<2)>>2]=am<<3;c[h+(aq<<2)>>2]=ao<<3;aq=F+2|0;c[g+(aq<<2)>>2]=ad<<3;c[h+(aq<<2)>>2]=as<<3;aq=F+3|0;c[g+(aq<<2)>>2]=ah<<3;c[h+(aq<<2)>>2]=aj<<3;aq=F+4|0;c[g+(aq<<2)>>2]=ay<<3;c[h+(aq<<2)>>2]=au<<3;aq=F+5|0;c[g+(aq<<2)>>2]=af<<3;c[h+(aq<<2)>>2]=ac<<3;aq=F+6|0;c[g+(aq<<2)>>2]=al<<3;c[h+(aq<<2)>>2]=ai<<3;aq=F+7|0;c[g+(aq<<2)>>2]=an<<3;c[h+(aq<<2)>>2]=ar<<3;break L14609}else{c[_>>2]=an<<3;c[aa>>2]=ar<<3;ar=F+1|0;c[g+(ar<<2)>>2]=av<<3;c[h+(ar<<2)>>2]=ax<<3;ax=F+2|0;c[g+(ax<<2)>>2]=al<<3;c[h+(ax<<2)>>2]=ai<<3;ai=F+3|0;c[g+(ai<<2)>>2]=am<<3;c[h+(ai<<2)>>2]=ao<<3;ao=F+4|0;c[g+(ao<<2)>>2]=af+ad<<3;c[h+(ao<<2)>>2]=ac+as<<3;as=F+5|0;c[g+(as<<2)>>2]=ah+ay<<3;c[h+(as<<2)>>2]=aj+au<<3;break L14609}break};case 2:case-2:{au=c[u>>2]|0;aj=c[$+(au<<2)>>2]|0;N2(aj,(aj|0)<0|0?-1:0,40763392,0)|0;aj=G;as=c[v>>2]|0;ay=c[$+(as<<2)>>2]|0;N2(ay,(ay|0)<0|0?-1:0,40763392,0)|0;ay=G;ah=c[Z+(au<<2)>>2]|0;N2(ah,(ah|0)<0|0?-1:0,40763392,0)|0;ah=G;au=c[Z+(as<<2)>>2]|0;N2(au,(au|0)<0|0?-1:0,40763392,0)|0;au=G;as=c[w>>2]|0;ac=c[$+(as<<2)>>2]|0;N2(ac,(ac|0)<0|0?-1:0,-156631040,-1)|0;ac=G;ao=c[x>>2]|0;ad=c[$+(ao<<2)>>2]|0;N2(ad,(ad|0)<0|0?-1:0,-156631040,-1)|0;ad=G;af=c[Z+(as<<2)>>2]|0;N2(af,(af|0)<0|0?-1:0,-156631040,-1)|0;af=G;as=c[Z+(ao<<2)>>2]|0;N2(as,(as|0)<0|0?-1:0,-156631040,-1)|0;as=G;ao=c[y>>2]|0;ai=c[$+(ao<<2)>>2]|0;N2(ai,(ai|0)<0|0?-1:0,657063936,0)|0;ai=G;am=c[z>>2]|0;ax=c[$+(am<<2)>>2]|0;N2(ax,(ax|0)<0|0?-1:0,657063936,0)|0;ax=ay+aj+ac+ad+ai+G|0;ai=c[Z+(ao<<2)>>2]|0;N2(ai,(ai|0)<0|0?-1:0,657063936,0)|0;ai=G;ao=c[Z+(am<<2)>>2]|0;N2(ao,(ao|0)<0|0?-1:0,657063936,0)|0;ao=au+ah+af+as+ai+G|0;ai=c[A>>2]|0;as=c[$+(ai<<2)>>2]|0;af=as>>>2|((as|0)<0|0?-1:0)<<30;as=c[Z+(ai<<2)>>2]|0;ai=as>>>2|((as|0)<0|0?-1:0)<<30;as=af+ax<<1;if(ab<<24>>24>-1){c[_>>2]=as;c[aa>>2]=ai+ao<<1;ah=F+1|0;c[g+(ah<<2)>>2]=af-ax<<1;c[h+(ah<<2)>>2]=ai-ao<<1;break L14609}else{ah=F+1|0;c[g+(ah<<2)>>2]=as;c[h+(ah<<2)>>2]=ai+ao<<1;c[_>>2]=af-ax<<1;c[aa>>2]=ai-ao<<1;break L14609}break};case 4:case-4:{ao=c[A>>2]|0;ai=c[$+(ao<<2)>>2]|0;N2(ai,(ai|0)<0|0?-1:0,2147483647,0)|0;ai=c[Z+(ao<<2)>>2]|0;ao=G<<1|0>>>31;ax=(ao>>3|((ao|0)<0|0?-1:0)<<29)<<1|0>>>31;N2(ai,(ai|0)<0|0?-1:0,2147483647,0)|0;ai=G<<1|0>>>31;ao=(ai>>3|((ai|0)<0|0?-1:0)<<29)<<1|0>>>31;ai=c[w>>2]|0;af=c[$+(ai<<2)>>2]|0;ah=af;as=(af|0)<0|0?-1:0;N2(ah,as,-1518500224,-1)|0;af=G;au=c[Z+(ai<<2)>>2]|0;ai=au;am=(au|0)<0|0?-1:0;N2(ai,am,1518500224,0)|0;au=af-G<<1;N2(au,(au|0)<0|0?-1:0,92733440,0)|0;au=G<<1|0>>>31;af=c[z>>2]|0;ad=c[$+(af<<2)>>2]|0;ac=ad;aj=(ad|0)<0|0?-1:0;N2(ac,aj,1518500224,0)|0;ad=G;ay=c[Z+(af<<2)>>2]|0;af=ay;al=(ay|0)<0|0?-1:0;N2(af,al,-1518500224,-1)|0;ay=ad-G<<1;N2(ay,(ay|0)<0|0?-1:0,455868416,0)|0;ay=(G<<1|0>>>31)+au|0;au=c[v>>2]|0;ad=c[$+(au<<2)>>2]|0;ar=ad;av=(ad|0)<0|0?-1:0;N2(ar,av,-1518500224,-1)|0;ad=G;an=c[Z+(au<<2)>>2]|0;au=an;aq=(an|0)<0|0?-1:0;N2(au,aq,1518500224,0)|0;an=ad-G<<1;N2(an,(an|0)<0|0?-1:0,-17039360,-1)|0;an=ay+(G<<1|0>>>31)|0;N2(ah,as,1518500224,0)|0;as=G;N2(ai,am,-1518500224,-1)|0;am=G+as<<1;N2(am,(am|0)<0|0?-1:0,92733440,0)|0;am=G<<1|0>>>31;N2(ac,aj,-1518500224,-1)|0;aj=G;N2(af,al,1518500224,0)|0;al=G+aj<<1;N2(al,(al|0)<0|0?-1:0,455868416,0)|0;al=(G<<1|0>>>31)+am|0;N2(ar,av,1518500224,0)|0;av=G;N2(au,aq,-1518500224,-1)|0;aq=G+av<<1;N2(aq,(aq|0)<0|0?-1:0,-17039360,-1)|0;aq=al+(G<<1|0>>>31)|0;al=c[m>>2]|0;av=c[$+(al<<2)>>2]|0;au=c[Z+(al<<2)>>2]|0;N2(au,(au|0)<0|0?-1:0,-2147483648,-1)|0;au=-G<<1;N2(au,(au|0)<0|0?-1:0,-6553600,-1)|0;au=G<<1|0>>>31;al=c[B>>2]|0;ar=c[$+(al<<2)>>2]|0;am=c[Z+(al<<2)>>2]|0;N2(am,(am|0)<0|0?-1:0,2147483647,0)|0;am=-G<<1;N2(am,(am|0)<0|0?-1:0,269352960,0)|0;am=(G<<1|0>>>31)+au|0;au=c[C>>2]|0;al=c[$+(au<<2)>>2]|0;aj=c[Z+(au<<2)>>2]|0;N2(aj,(aj|0)<0|0?-1:0,-2147483648,-1)|0;aj=-G<<1;N2(aj,(aj|0)<0|0?-1:0,269352960,0)|0;aj=am+(G<<1|0>>>31)|0;am=c[D>>2]|0;au=c[$+(am<<2)>>2]|0;af=c[Z+(am<<2)>>2]|0;N2(af,(af|0)<0|0?-1:0,2147483647,0)|0;af=-G<<1;N2(af,(af|0)<0|0?-1:0,-6553600,-1)|0;af=aj+(G<<1|0>>>31)|0;N2(av,(av|0)<0|0?-1:0,-2147483648,-1)|0;av=G<<1|0>>>31;N2(av,(av|0)<0|0?-1:0,-6553600,-1)|0;av=G<<1|0>>>31;N2(ar,(ar|0)<0|0?-1:0,2147483647,0)|0;ar=G<<1|0>>>31;N2(ar,(ar|0)<0|0?-1:0,269352960,0)|0;ar=(G<<1|0>>>31)+av|0;N2(al,(al|0)<0|0?-1:0,-2147483648,-1)|0;al=G<<1|0>>>31;N2(al,(al|0)<0|0?-1:0,269352960,0)|0;al=ar+(G<<1|0>>>31)|0;N2(au,(au|0)<0|0?-1:0,2147483647,0)|0;au=G<<1|0>>>31;N2(au,(au|0)<0|0?-1:0,-6553600,-1)|0;au=al+(G<<1|0>>>31)|0;al=c[u>>2]|0;ar=c[$+(al<<2)>>2]|0;N2(ar,(ar|0)<0|0?-1:0,-1518500224,-1)|0;ar=G;av=c[Z+(al<<2)>>2]|0;N2(av,(av|0)<0|0?-1:0,-1518500224,-1)|0;av=G;al=ar-av<<1;N2(al,(al|0)<0|0?-1:0,-17039360,-1)|0;al=G<<1|0>>>31;aj=c[y>>2]|0;am=c[$+(aj<<2)>>2]|0;N2(am,(am|0)<0|0?-1:0,1518500224,0)|0;am=G;ac=c[Z+(aj<<2)>>2]|0;N2(ac,(ac|0)<0|0?-1:0,1518500224,0)|0;ac=G;aj=am-ac<<1;N2(aj,(aj|0)<0|0?-1:0,455868416,0)|0;aj=(G<<1|0>>>31)+al|0;al=c[x>>2]|0;as=c[$+(al<<2)>>2]|0;N2(as,(as|0)<0|0?-1:0,-1518500224,-1)|0;as=G;ai=c[Z+(al<<2)>>2]|0;N2(ai,(ai|0)<0|0?-1:0,-1518500224,-1)|0;ai=G;al=as-ai<<1;N2(al,(al|0)<0|0?-1:0,92733440,0)|0;al=aj+(G<<1|0>>>31)|0;aj=av+ar<<1;N2(aj,(aj|0)<0|0?-1:0,-17039360,-1)|0;aj=G<<1|0>>>31;ar=ac+am<<1;N2(ar,(ar|0)<0|0?-1:0,455868416,0)|0;ar=(G<<1|0>>>31)+aj|0;aj=ai+as<<1;N2(aj,(aj|0)<0|0?-1:0,92733440,0)|0;aj=ar+(G<<1|0>>>31)|0;c[_>>2]=an+ax+af+al;c[aa>>2]=aq+ao+au+aj;ar=F+1|0;c[g+(ar<<2)>>2]=aq+ax-af-aj;c[h+(ar<<2)>>2]=ao-an-au+al;ar=F+2|0;c[g+(ar<<2)>>2]=ax-an+af-al;c[h+(ar<<2)>>2]=ao-aq+au-aj;ar=F+3|0;c[g+(ar<<2)>>2]=ax-aq-af+aj;c[h+(ar<<2)>>2]=an+ao-au-al;break};default:{}}}while(0);aa=(d[(c[l>>2]|0)+1+E|0]|0)+F|0;_=E+1|0;if((_|0)<(o|0)){F=aa;E=_}else{t=aa;break L14605}}}}while(0);c[p>>2]=s;s=b+136|0;p=c[s>>2]|0;if((p|0)<=(o|0)){i=j;return 0}m=g+(t<<2)|0;if((a[b+144|0]|0)!=0){IY(m,e+(o<<2)|0,p-o<<2);IY(h+(t<<2)|0,f+(o<<2)|0,(c[b+140>>2]|0)-o<<2);i=j;return 0}g=b+132|0;IY(m,c[b+24+(c[g>>2]<<2)>>2]|0,p-o<<2);p=b+140|0;IY(h+(t<<2)|0,c[b+76+(c[g>>2]<<2)>>2]|0,(c[p>>2]|0)-o<<2);IY(c[b+24+(c[g>>2]<<2)>>2]|0,e+(o<<2)|0,(c[s>>2]|0)-o<<2);IY(c[b+76+(c[g>>2]<<2)>>2]|0,f+(o<<2)|0,(c[p>>2]|0)-o<<2);o=(c[g>>2]|0)+1|0;c[g>>2]=o;if((o|0)<(d[(c[l>>2]|0)+8|0]|0)){i=j;return 0}c[g>>2]=0;i=j;return 0}function K5(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0;if((b|0)==2){f=1928576}else if((b|0)==0){f=1928608}else if((b|0)==1){f=1928592}else{g=-1;return g|0}c[a+8>>2]=f;c[a>>2]=d;c[a+4>>2]=e;g=0;return g|0}function K6(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;h=b+8|0;i=c[h>>2]|0;j=a[i|0]|0;k=j&255;L14640:do{if(j<<24>>24==0){l=0}else{m=0;n=0;o=i;while(1){p=a[o+1+n|0]|0;q=p&255;if(p<<24>>24==0){r=0;s=0}else{p=0;t=0;u=0;while(1){v=p+m|0;w=(c[d+(v<<2)>>2]|0)+t|0;x=(c[e+(v<<2)>>2]|0)+u|0;v=p+1|0;if((v|0)<(q|0)){p=v;t=w;u=x}else{r=w;s=x;break}}}c[f+(n<<2)>>2]=r;c[g+(n<<2)>>2]=s;u=q+m|0;t=n+1|0;if((t|0)>=(k|0)){l=u;break L14640}m=u;n=t;o=c[h>>2]|0}}}while(0);h=c[b>>2]|0;if((h|0)<=(k|0)){return 0}IY(f+(k<<2)|0,d+(l<<2)|0,h-k<<2);IY(g+(k<<2)|0,e+(l<<2)|0,(c[b+4>>2]|0)-k<<2);return 0}function K7(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0;f=(e|0)>64?6:5;g=c[b>>2]|0;h=c[b-8>>2]|0;i=g;j=(g|0)<0|0?-1:0;g=h;k=(h|0)<0|0?-1:0;N2(g,k,i,j)|0;h=G;l=c[d>>2]|0;m=c[d-8>>2]|0;n=l;o=(l|0)<0|0?-1:0;l=m;p=(m|0)<0|0?-1:0;N2(l,p,n,o)|0;m=G+h>>f;N2(n,o,g,k)|0;o=G;N2(l,p,i,j)|0;j=o-G>>f;o=b-4|0;i=d-4|0;n=e-1|0;h=c[o>>2]|0;q=c[i>>2]|0;if((n|0)==0){r=0;s=0;t=0;u=m;v=j}else{w=0;x=0;y=0;z=m;m=j;j=o;o=i;i=n;A=h;B=q;while(1){C=A;D=(A|0)<0|0?-1:0;N2(C,D,C,D)|0;E=G;F=B;H=(B|0)<0|0?-1:0;N2(F,H,F,H)|0;I=((G+E|0)>>>(f>>>0))+y|0;E=j+4|0;J=c[E>>2]|0;K=J;L=(J|0)<0|0?-1:0;N2(K,L,C,D)|0;M=G;N=o+4|0;O=c[N>>2]|0;P=O;Q=(O|0)<0|0?-1:0;N2(P,Q,F,H)|0;R=(G+M>>f)+w|0;N2(P,Q,C,D)|0;Q=G;N2(K,L,F,H)|0;L=(Q-G>>f)+x|0;Q=c[j+8>>2]|0;K=Q;P=(Q|0)<0|0?-1:0;N2(K,P,C,D)|0;Q=G;M=c[o+8>>2]|0;S=M;T=(M|0)<0|0?-1:0;N2(S,T,F,H)|0;M=(G+Q>>f)+z|0;N2(S,T,C,D)|0;D=G;N2(K,P,F,H)|0;H=(D-G>>f)+m|0;D=i-1|0;if((D|0)==0){r=R;s=L;t=I;u=M;v=H;break}else{w=R;x=L;y=I;z=M;m=H;j=E;o=N;i=D;A=J;B=O}}}N2(g,k,g,k)|0;B=G;N2(l,p,l,p)|0;A=((G+B|0)>>>(f>>>0))+t|0;B=e-2|0;e=c[b+(B<<2)>>2]|0;i=e;o=(e|0)<0|0?-1:0;N2(i,o,i,o)|0;e=G;j=c[d+(B<<2)>>2]|0;B=j;m=(j|0)<0|0?-1:0;N2(B,m,B,m)|0;j=((G+e|0)>>>(f>>>0))+t|0;t=c[b+(n<<2)>>2]|0;b=t;e=(t|0)<0|0?-1:0;N2(b,e,b,e)|0;t=G;z=c[d+(n<<2)>>2]|0;n=z;d=(z|0)<0|0?-1:0;N2(n,d,n,d)|0;z=(G+t|0)>>>(f>>>0);t=h;y=(h|0)<0|0?-1:0;N2(t,y,t,y)|0;h=G;x=q;w=(q|0)<0|0?-1:0;N2(x,w,x,w)|0;q=z-((G+h|0)>>>(f>>>0))+j|0;N2(t,y,g,k)|0;h=G;N2(x,w,l,p)|0;z=(G+h>>f)+r|0;N2(b,e,i,o)|0;h=G;N2(n,d,B,m)|0;O=(G+h>>f)+r|0;N2(x,w,g,k)|0;k=G;N2(t,y,l,p)|0;p=(k-G>>f)+s|0;N2(n,d,i,o)|0;o=G;N2(b,e,B,m)|0;m=(o-G>>f)+s|0;s=((v|0)>0?v:-v|0)|((u|0)>0?u:-u|0)|A|j|((O|0)>0?O:-O|0)|((m|0)>0?m:-m|0)|q|((z|0)>0?z:-z|0)|((p|0)>0?p:-p|0);if((s|0)>-1){o=~s;s=0;while(1){B=s+1|0;e=o<<1;if((e|0)<0){o=e;s=B}else{U=B;break}}}else{U=0}s=U-1|0;o=-2-f+U|0;c[a>>2]=q<<s;q=j<<s;c[a+4>>2]=q;j=A<<s;c[a+8>>2]=j;c[a+12>>2]=O<<s;O=z<<s;c[a+20>>2]=O;c[a+24>>2]=m<<s;m=p<<s;c[a+32>>2]=m;c[a+16>>2]=u<<s;c[a+28>>2]=v<<s;N2(j,(j|0)<0|0?-1:0,q,(q|0)<0|0?-1:0)|0;q=G>>1;j=O;s=(O|0)<0|0?-1:0;N2(j,s,j,s)|0;s=G;j=m;O=(m|0)<0|0?-1:0;N2(j,O,j,O)|0;O=q-((G+s|0)>>>1)|0;s=a+36|0;c[s>>2]=O;q=(O|0)>0?O:-O|0;if((q|0)==0){V=0;W=O<<V;c[s>>2]=W;X=V-2|0;Y=a+40|0;c[Y>>2]=X;return o|0}j=q>>31^q;if((j|0)<=-1){V=-1;W=O<<V;c[s>>2]=W;X=V-2|0;Y=a+40|0;c[Y>>2]=X;return o|0}q=~j;j=0;while(1){m=q<<1;if((m|0)<0){q=m;j=j+1|0}else{V=j;break}}W=O<<V;c[s>>2]=W;X=V-2|0;Y=a+40|0;c[Y>>2]=X;return o|0}function K8(a,b,d,f){a=a|0;b=b|0;d=d|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;g=d>>1;h=(d|0)==64?1:2;i=d>>2;if((i|0)>1){j=h<<2;k=b+((g<<1)-2<<2)|0;l=1;m=b;while(1){n=m+8|0;o=c[a+(d-l<<2)>>2]|0;p=c[a+(l<<2)>>2]|0;q=c[73472+((ac(l,h)|0)<<2)>>2]|0;r=q&-65536;s=q<<16;q=s;t=(s|0)<0|0?-1:0;s=o;u=(o|0)<0|0?-1:0;N2(q,t,s,u)|0;o=G;v=r;w=(r|0)<0|0?-1:0;r=p;x=(p|0)<0|0?-1:0;N2(v,w,r,x)|0;p=o-G|0;N2(v,w,s,u)|0;u=G;N2(q,t,r,x)|0;x=u+G|0;u=c[a+(l+g<<2)>>2]|0;r=g-l|0;t=c[a+(r<<2)>>2]|0;q=c[73472+((ac(r,h)|0)<<2)>>2]|0;r=q&-65536;s=q<<16;q=s;w=(s|0)<0|0?-1:0;s=u;v=(u|0)<0|0?-1:0;N2(q,w,s,v)|0;u=G;o=r;y=(r|0)<0|0?-1:0;r=t;z=(t|0)<0|0?-1:0;N2(o,y,r,z)|0;t=u-G|0;N2(o,y,s,v)|0;v=G;N2(q,w,r,z)|0;z=v+G>>1;v=t>>1;t=x>>1;x=z-t|0;r=p>>1;p=v+r|0;w=c[73472+((ac(j,l)|0)<<2)>>2]|0;q=w&-65536;s=w<<16;w=s;y=(s|0)<0|0?-1:0;s=x;o=(x|0)<0|0?-1:0;N2(s,o,w,y)|0;x=G;u=q;A=(q|0)<0|0?-1:0;q=p;B=(p|0)<0|0?-1:0;N2(q,B,u,A)|0;p=x-G|0;N2(s,o,u,A)|0;A=G;N2(q,B,w,y)|0;y=A+G|0;A=z+t>>1;c[n>>2]=A-y;c[k>>2]=y+A;A=r-v>>1;c[m+12>>2]=A-p;c[k+4>>2]=-(A+p|0);p=l+1|0;if((p|0)<(i|0)){k=k-8|0;l=p;m=n}else{break}}}m=c[a+(g<<2)>>2]|0;l=(e[36800]|0)<<16;N2(l,(l|0)<0|0?-1:0,m,(m|0)<0|0?-1:0)|0;m=G;c[b>>2]=m+(c[a>>2]>>1)>>1;c[b+4>>2]=(c[a>>2]>>1)-m>>1;m=(g|0)/2|0;l=c[a+(d-m<<2)>>2]|0;k=c[a+(m<<2)>>2]|0;m=c[18384]|0;j=m&-65536;h=m<<16;m=h;n=(h|0)<0|0?-1:0;h=l;p=(l|0)<0|0?-1:0;N2(m,n,h,p)|0;l=G;A=j;v=(j|0)<0|0?-1:0;j=k;r=(k|0)<0|0?-1:0;N2(A,v,j,r)|0;k=l-G|0;N2(A,v,h,p)|0;p=G;N2(m,n,j,r)|0;c[b+(g<<2)>>2]=p+G>>1;c[b+(g+1<<2)>>2]=k>>1;Lc(g,b,f);if((i|0)==0){C=c[f>>2]|0;D=C+2|0;c[f>>2]=D;return}g=b+(d<<2)|0;d=i;i=b;b=a;while(1){a=d-1|0;k=c[i+4>>2]|0;p=c[g-4>>2]|0;r=g-8|0;j=c[r>>2]|0;c[b>>2]=c[i>>2];c[b+4>>2]=p;c[b+8>>2]=k;c[b+12>>2]=j;if((a|0)==0){break}else{g=r;d=a;i=i+8|0;b=b+16|0}}C=c[f>>2]|0;D=C+2|0;c[f>>2]=D;return}function K9(a,b,d,f){a=a|0;b=b|0;d=d|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;g=d>>1;h=32>>>(((d>>6)+4|0)>>>0);i=d>>2;do{if((i|0)==0){Lc(g,b,f)}else{j=a;k=i;l=b+(d-1<<2)|0;m=b;while(1){n=k-1|0;o=c[j+4>>2]>>1;p=c[j+8>>2]>>1;q=c[j+12>>2]>>1;c[m>>2]=c[j>>2]>>1;c[m+4>>2]=p;c[l>>2]=o;c[l-4>>2]=q;if((n|0)==0){break}else{j=j+16|0;k=n;l=l-8|0;m=m+8|0}}Lc(g,b,f);if((i|0)<=1){break}m=h<<2;l=1;k=b+((g<<1)-2<<2)|0;j=b;while(1){n=j+8|0;q=c[j+12>>2]>>1;o=c[k+4>>2]>>1;p=o+q|0;r=c[k>>2]>>1;s=c[n>>2]>>1;t=r-s|0;u=c[73472+((ac(m,l)|0)<<2)>>2]|0;v=u&-65536;w=u<<16;u=w;x=(w|0)<0|0?-1:0;w=t;y=(t|0)<0|0?-1:0;N2(u,x,w,y)|0;t=G;z=v;A=(v|0)<0|0?-1:0;v=p;B=(p|0)<0|0?-1:0;N2(z,A,v,B)|0;p=t-G|0;N2(z,A,w,y)|0;y=G;N2(u,x,v,B)|0;B=p<<1;p=y+G<<1;y=s+r|0;r=q-o|0;o=p+y|0;q=-(B+r|0)|0;s=c[73472+((ac(l,h)|0)<<2)>>2]|0;v=s&-65536;x=s<<16;s=x;u=(x|0)<0|0?-1:0;x=o;w=(o|0)<0|0?-1:0;N2(x,w,s,u)|0;o=G;A=v;z=(v|0)<0|0?-1:0;v=q;t=(q|0)<0|0?-1:0;N2(v,t,A,z)|0;q=o-G|0;N2(x,w,A,z)|0;z=G;N2(v,t,s,u)|0;c[a+(d-l<<2)>>2]=z+G;c[a+(l<<2)>>2]=q;q=y-p|0;p=r-B|0;B=g-l|0;r=c[73472+((ac(B,h)|0)<<2)>>2]|0;y=r&-65536;z=r<<16;r=z;u=(z|0)<0|0?-1:0;z=q;s=(q|0)<0|0?-1:0;N2(z,s,r,u)|0;q=G;t=y;v=(y|0)<0|0?-1:0;y=p;A=(p|0)<0|0?-1:0;N2(y,A,t,v)|0;p=q-G|0;N2(z,s,t,v)|0;v=G;N2(y,A,r,u)|0;c[a+(l+g<<2)>>2]=v+G;c[a+(B<<2)>>2]=p;p=l+1|0;if((p|0)<(i|0)){l=p;k=k-8|0;j=n}else{break}}}}while(0);i=c[b+(g<<2)>>2]|0;j=c[b+(g+1<<2)>>2]|0;k=(g|0)/2|0;l=c[73472+((ac(k,h)|0)<<2)>>2]|0;h=l&-65536;m=l<<16;l=m;n=(m|0)<0|0?-1:0;m=i;p=(i|0)<0|0?-1:0;N2(l,n,m,p)|0;i=G;B=h;v=(h|0)<0|0?-1:0;h=j;u=(j|0)<0|0?-1:0;N2(B,v,h,u)|0;j=i-G|0;N2(B,v,m,p)|0;p=G;N2(l,n,h,u)|0;c[a+(d-k<<2)>>2]=p+G;c[a+(k<<2)>>2]=j;j=b+4|0;c[a>>2]=(c[j>>2]>>1)+(c[b>>2]>>1);k=(c[b>>2]>>1)-(c[j>>2]>>1)|0;j=(e[36800]|0)<<16;N2(j,(j|0)<0|0?-1:0,k,(k|0)<0|0?-1:0)|0;c[a+(g<<2)>>2]=G<<1|0>>>31;c[f>>2]=(c[f>>2]|0)+2;return}function La(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0;e=b>>1;if((b|0)>-1){f=~b;g=0;while(1){h=g+1|0;i=f<<1;if((i|0)<0){f=i;g=h}else{j=h;break}}}else{j=0}g=31-j|0;j=g-2|0;f=b>>j;if((f|0)==4){k=1962120;l=c[24+(j<<2)>>2]|0;m=1<<10-g}else if((f|0)==7){k=1963152;l=c[60+(g-1<<2)>>2]|0;m=1<<9-g}else{k=0;l=0;m=0}g=a+(b-2<<2)|0;f=e-1|0;if((f|0)>0){j=e-2&-2;h=b-4-j|0;i=j+2|0;j=g;n=0;o=a;while(1){p=j+4|0;q=c[p>>2]|0;r=c[o>>2]|0;s=o+4|0;t=c[s>>2]|0;u=c[j>>2]|0;v=c[l+(n<<2)>>2]|0;w=v&-65536;x=v<<16;v=x;y=(x|0)<0|0?-1:0;x=q;z=(q|0)<0|0?-1:0;N2(v,y,x,z)|0;q=G;A=w;B=(w|0)<0|0?-1:0;w=r;C=(r|0)<0|0?-1:0;N2(A,B,w,C)|0;r=q-G|0;N2(A,B,x,z)|0;z=G;N2(v,y,w,C)|0;C=z+G|0;z=c[l+((n|1)<<2)>>2]|0;w=z&-65536;y=z<<16;z=y;v=(y|0)<0|0?-1:0;y=u;x=(u|0)<0|0?-1:0;N2(z,v,y,x)|0;u=G;B=w;A=(w|0)<0|0?-1:0;w=t;q=(t|0)<0|0?-1:0;N2(B,A,w,q)|0;t=G;N2(B,A,y,x)|0;x=G;N2(z,v,w,q)|0;c[o>>2]=C;c[s>>2]=r;c[j>>2]=x+G;c[p>>2]=t-u;u=n+2|0;if((u|0)<(f|0)){j=j-8|0;n=u;o=o+8|0}else{break}}D=a+(h<<2)|0;E=i;F=a+(i<<2)|0}else{D=g;E=0;F=a}i=(e&1|0)==0;if(!i){h=c[D+4>>2]|0;D=c[F>>2]|0;o=c[l+(E<<2)>>2]|0;E=o&-65536;l=o<<16;o=l;n=(l|0)<0|0?-1:0;l=h;j=(h|0)<0|0?-1:0;N2(o,n,l,j)|0;h=G;f=E;u=(E|0)<0|0?-1:0;E=D;t=(D|0)<0|0?-1:0;N2(f,u,E,t)|0;D=h-G|0;N2(f,u,l,j)|0;j=G;N2(o,n,E,t)|0;c[F>>2]=j+G;c[F+4>>2]=D}Lc(e,a,d);D=c[g>>2]|0;F=a+(b-1<<2)|0;j=c[F>>2]|0;c[F>>2]=-(c[a+4>>2]>>1);c[a>>2]=c[a>>2]>>1;F=e+1>>1;if((F|0)>1){e=F<<1;t=b-e|0;b=e-2|0;e=a;E=g;n=D;o=j;l=m;u=1;while(1){f=c[k+(l<<2)>>2]|0;h=f&-65536;p=f<<16;f=p;x=(p|0)<0|0?-1:0;p=n;r=(n|0)<0|0?-1:0;N2(f,x,p,r)|0;s=G;C=h;q=(h|0)<0|0?-1:0;h=o;w=(o|0)<0|0?-1:0;N2(C,q,h,w)|0;v=s-G|0;N2(C,q,p,r)|0;r=G;N2(f,x,h,w)|0;c[e+4>>2]=v;c[E>>2]=r+G;r=e+8|0;v=E-8|0;w=c[e+12>>2]|0;h=c[r>>2]|0;p=w;s=(w|0)<0|0?-1:0;N2(f,x,p,s)|0;w=G;z=h;y=(h|0)<0|0?-1:0;N2(z,y,C,q)|0;h=G;N2(C,q,p,s)|0;s=G;N2(z,y,f,x)|0;H=c[v>>2]|0;x=E-4|0;I=c[x>>2]|0;c[x>>2]=h-w;c[r>>2]=G+s;s=u+1|0;if((s|0)<(F|0)){e=r;E=v;n=H;o=I;l=l+m|0;u=s}else{break}}J=a+(b<<2)|0;K=a+(t<<2)|0;L=H;M=I}else{J=a;K=g;L=D;M=j}if(!i){N=c[d>>2]|0;O=N+2|0;c[d>>2]=O;return}N2(L,(L|0)<0|0?-1:0,1518469120,0)|0;L=G;N2(M,(M|0)<0|0?-1:0,1518469120,0)|0;M=G;c[K>>2]=M+L;c[J+4>>2]=L-M;N=c[d>>2]|0;O=N+2|0;c[d>>2]=O;return}function Lb(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0;e=b>>1;if((b|0)>-1){f=~b;g=0;while(1){h=g+1|0;i=f<<1;if((i|0)<0){f=i;g=h}else{j=h;break}}}else{j=0}g=31-j|0;j=g-2|0;f=b>>j;if((f|0)==7){k=1963152;l=c[60+(g-1<<2)>>2]|0;m=1<<9-g}else if((f|0)==4){k=1962120;l=c[24+(j<<2)>>2]|0;m=1<<10-g}else{k=0;l=0;m=0}g=a+(b-2<<2)|0;j=e-1|0;if((j|0)>0){f=e-2&-2;h=b-4-f|0;i=f+2|0;f=g;n=0;o=a;while(1){p=f+4|0;q=c[p>>2]|0;r=-(c[o>>2]|0)|0;s=o+4|0;t=c[s>>2]|0;u=-(c[f>>2]|0)|0;v=c[l+(n<<2)>>2]|0;w=v&-65536;x=v<<16;v=x;y=(x|0)<0|0?-1:0;x=q;z=(q|0)<0|0?-1:0;N2(v,y,x,z)|0;q=G;A=w;B=(w|0)<0|0?-1:0;w=r;C=(r|0)<0|0?-1:0;N2(A,B,w,C)|0;r=q-G|0;N2(A,B,x,z)|0;z=G;N2(v,y,w,C)|0;C=z+G|0;z=c[l+((n|1)<<2)>>2]|0;w=z&-65536;y=z<<16;z=y;v=(y|0)<0|0?-1:0;y=u;x=(u|0)<0|0?-1:0;N2(z,v,y,x)|0;u=G;B=w;A=(w|0)<0|0?-1:0;w=t;q=(t|0)<0|0?-1:0;N2(B,A,w,q)|0;t=G;N2(B,A,y,x)|0;x=G;N2(z,v,w,q)|0;c[o>>2]=C;c[s>>2]=r;c[f>>2]=x+G;c[p>>2]=t-u;u=n+2|0;if((u|0)<(j|0)){f=f-8|0;n=u;o=o+8|0}else{break}}D=a+(h<<2)|0;E=i;F=a+(i<<2)|0}else{D=g;E=0;F=a}i=(e&1|0)==0;if(!i){h=c[D+4>>2]|0;D=-(c[F>>2]|0)|0;o=c[l+(E<<2)>>2]|0;E=o&-65536;l=o<<16;o=l;n=(l|0)<0|0?-1:0;l=h;f=(h|0)<0|0?-1:0;N2(o,n,l,f)|0;h=G;j=E;u=(E|0)<0|0?-1:0;E=D;t=(D|0)<0|0?-1:0;N2(j,u,E,t)|0;D=h-G|0;N2(j,u,l,f)|0;f=G;N2(o,n,E,t)|0;c[F>>2]=f+G;c[F+4>>2]=D}Lc(e,a,d);D=c[g>>2]|0;F=a+(b-1<<2)|0;f=c[F>>2]|0;c[F>>2]=-(c[a>>2]>>1);c[a>>2]=c[a+4>>2]>>1;F=e+1>>1;if((F|0)>1){e=F<<1;t=b-e|0;b=e-2|0;e=a;E=g;n=D;o=f;l=m;u=1;while(1){j=c[k+(l<<2)>>2]|0;h=j&-65536;p=j<<16;j=p;x=(p|0)<0|0?-1:0;p=n;r=(n|0)<0|0?-1:0;N2(j,x,p,r)|0;s=G;C=h;q=(h|0)<0|0?-1:0;h=o;w=(o|0)<0|0?-1:0;N2(C,q,h,w)|0;v=G;N2(C,q,p,r)|0;r=G;N2(j,x,h,w)|0;c[E>>2]=v-s;c[e+4>>2]=-(r+G|0);r=e+8|0;s=E-8|0;v=c[e+12>>2]|0;w=c[r>>2]|0;h=v;p=(v|0)<0|0?-1:0;N2(j,x,h,p)|0;v=G;z=w;y=(w|0)<0|0?-1:0;N2(z,y,C,q)|0;w=v-G|0;N2(C,q,h,p)|0;p=G;N2(z,y,j,x)|0;H=c[s>>2]|0;x=E-4|0;I=c[x>>2]|0;c[r>>2]=w;c[x>>2]=-(p+G|0);p=u+1|0;if((p|0)<(F|0)){e=r;E=s;n=H;o=I;l=l+m|0;u=p}else{break}}J=a+(b<<2)|0;K=a+(t<<2)|0;L=H;M=I}else{J=a;K=g;L=D;M=f}if(!i){N=c[d>>2]|0;O=N+2|0;c[d>>2]=O;return}N2(L,(L|0)<0|0?-1:0,1518469120,0)|0;L=G;N2(M,(M|0)<0|0?-1:0,1518469120,0)|0;M=G;c[J+4>>2]=-(M+L|0);c[K>>2]=M-L;N=c[d>>2]|0;O=N+2|0;c[d>>2]=O;return}function Lc(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0;e=i;i=i+6360|0;f=e|0;g=e+480|0;h=e+2400|0;j=e+6240|0;switch(a|0){case 60:{a=f|0;k=b;l=a;m=0;while(1){n=c[k+240>>2]|0;o=c[k+244>>2]|0;p=c[k+360>>2]|0;q=c[k+364>>2]|0;r=n+(c[k>>2]|0)>>1;s=p+(c[k+120>>2]|0)>>1;t=o+(c[k+4>>2]|0)>>1;u=q+(c[k+124>>2]|0)>>1;c[l>>2]=s+r;c[l+4>>2]=u+t;v=r-n|0;n=t-o|0;c[l+16>>2]=r-s;c[l+20>>2]=t-u;t=s-p|0;p=u-q|0;c[l+8>>2]=p+v;c[l+24>>2]=v-p;c[l+12>>2]=n-t;c[l+28>>2]=t+n;n=m+1|0;if((n|0)<15){k=k+8|0;l=l+32|0;m=n}else{break}}Lg(a,4,60,1966176,1967736);a=0;do{m=a<<1;l=a<<3;c[j+(m<<2)>>2]=c[f+(l<<2)>>2];c[j+((m|1)<<2)>>2]=c[f+((l|1)<<2)>>2];a=a+1|0;}while((a|0)<15);a=j|0;Lf(a);l=0;while(1){m=l<<1;k=l<<3;c[b+(k<<2)>>2]=c[j+(m<<2)>>2];c[b+((k|1)<<2)>>2]=c[j+((m|1)<<2)>>2];m=l+1|0;if((m|0)<15){l=m}else{w=0;break}}do{l=w<<1;m=w<<3;c[j+(l<<2)>>2]=c[f+((m|2)<<2)>>2];c[j+((l|1)<<2)>>2]=c[f+((m|3)<<2)>>2];w=w+1|0;}while((w|0)<15);Lf(a);w=0;while(1){m=w<<1;l=w<<3;c[b+((l|2)<<2)>>2]=c[j+(m<<2)>>2];c[b+((l|3)<<2)>>2]=c[j+((m|1)<<2)>>2];m=w+1|0;if((m|0)<15){w=m}else{x=0;break}}do{w=x<<1;m=x<<3;c[j+(w<<2)>>2]=c[f+((m|4)<<2)>>2];c[j+((w|1)<<2)>>2]=c[f+((m|5)<<2)>>2];x=x+1|0;}while((x|0)<15);Lf(a);x=0;while(1){m=x<<1;w=x<<3;c[b+((w|4)<<2)>>2]=c[j+(m<<2)>>2];c[b+((w|5)<<2)>>2]=c[j+((m|1)<<2)>>2];m=x+1|0;if((m|0)<15){x=m}else{y=0;break}}do{x=y<<1;m=y<<3;c[j+(x<<2)>>2]=c[f+((m|6)<<2)>>2];c[j+((x|1)<<2)>>2]=c[f+((m|7)<<2)>>2];y=y+1|0;}while((y|0)<15);Lf(a);a=0;do{y=a<<1;f=a<<3;c[b+((f|6)<<2)>>2]=c[j+(y<<2)>>2];c[b+((f|7)<<2)>>2]=c[j+((y|1)<<2)>>2];a=a+1|0;}while((a|0)<15);c[d>>2]=(c[d>>2]|0)+5;i=e;return};case 5:{a=b+8|0;y=c[a>>2]|0;f=b+32|0;m=c[f>>2]|0;x=m+y|0;w=y-m|0;m=b+16|0;y=c[m>>2]|0;l=b+24|0;k=c[l>>2]|0;n=k+y|0;t=y-k|0;k=x-n|0;N2(k,(k|0)<0|0?-1:0,1200488448,0)|0;k=G<<1|0>>>31;y=n+x|0;x=y+(c[b>>2]|0)|0;c[b>>2]=x;N2(y,(y|0)<0|0?-1:0,-1342177280,-1)|0;y=(G<<2|0>>>30)+x|0;x=y-k|0;n=y+k|0;k=t+w|0;N2(k,(k|0)<0|0?-1:0,2042363904,0)|0;k=G<<1|0>>>31;N2(w,(w|0)<0|0?-1:0,-1652293632,-1)|0;w=k+(G<<2|0>>>30)|0;N2(t,(t|0)<0|0?-1:0,-780140544,-1)|0;t=k+(G<<1|0>>>31)|0;k=b+12|0;y=c[k>>2]|0;p=b+36|0;v=c[p>>2]|0;q=v+y|0;u=y-v|0;v=b+20|0;y=c[v>>2]|0;s=b+28|0;r=c[s>>2]|0;o=r+y|0;z=y-r|0;r=q-o|0;N2(r,(r|0)<0|0?-1:0,1200488448,0)|0;r=G<<1|0>>>31;y=o+q|0;q=b+4|0;o=y+(c[q>>2]|0)|0;c[q>>2]=o;N2(y,(y|0)<0|0?-1:0,-1342177280,-1)|0;y=(G<<2|0>>>30)+o|0;o=y-r|0;q=y+r|0;r=z+u|0;N2(r,(r|0)<0|0?-1:0,2042363904,0)|0;r=G<<1|0>>>31;N2(u,(u|0)<0|0?-1:0,-1652293632,-1)|0;u=r+(G<<2|0>>>30)|0;N2(z,(z|0)<0|0?-1:0,-780140544,-1)|0;z=r+(G<<1|0>>>31)|0;c[a>>2]=z+n;c[f>>2]=n-z;c[m>>2]=x-u;c[l>>2]=u+x;c[k>>2]=q-t;c[p>>2]=q+t;c[v>>2]=o+w;c[s>>2]=o-w;i=e;return};case 4:{w=b+16|0;o=c[w>>2]|0;s=o+(c[b>>2]|0)>>1;v=b+8|0;t=b+24|0;q=c[t>>2]|0;p=q+(c[v>>2]|0)>>1;k=b+4|0;x=b+20|0;u=c[x>>2]|0;l=u+(c[k>>2]|0)>>1;m=b+12|0;z=b+28|0;n=c[z>>2]|0;f=n+(c[m>>2]|0)>>1;c[b>>2]=p+s;c[k>>2]=f+l;k=s-o|0;o=l-u|0;c[w>>2]=s-p;c[x>>2]=l-f;l=p-q|0;q=f-n|0;c[v>>2]=q+k;c[t>>2]=k-q;c[m>>2]=o-l;c[z>>2]=l+o;c[d>>2]=(c[d>>2]|0)+1;i=e;return};case 3:{o=b+8|0;l=c[o>>2]|0;z=b+16|0;m=c[z>>2]|0;q=m+l|0;k=l-m|0;N2(k,(k|0)<0|0?-1:0,-1859780608,-1)|0;k=G<<1|0>>>31;m=c[b>>2]|0;c[b>>2]=m+q;l=m-(q>>1)|0;q=b+12|0;m=c[q>>2]|0;t=b+20|0;v=c[t>>2]|0;n=v+m|0;f=m-v|0;N2(f,(f|0)<0|0?-1:0,-1859780608,-1)|0;f=G<<1|0>>>31;v=b+4|0;m=c[v>>2]|0;c[v>>2]=m+n;v=m-(n>>1)|0;c[o>>2]=l-f;c[z>>2]=f+l;c[q>>2]=v+k;c[t>>2]=v-k;i=e;return};case 32:{Ld(b);c[d>>2]=(c[d>>2]|0)+4;i=e;return};case 256:{Lh(b,8,1962120,512);c[d>>2]=(c[d>>2]|0)+7;i=e;return};case 480:{k=h|0;h=b;v=k;t=0;while(1){q=0;do{l=q<<1;f=q*30|0;c[v+(l<<2)>>2]=c[h+(f<<2)>>2];c[v+((l|1)<<2)>>2]=c[h+((f|1)<<2)>>2];q=q+1|0;}while((q|0)<32);Ld(v);q=t+1|0;if((q|0)<15){h=h+8|0;v=v+256|0;t=q}else{break}}Lg(k,32,480,1966296,1967856);t=j|0;v=k;k=0;h=b;while(1){q=0;do{f=q<<1;l=q<<6;c[j+(f<<2)>>2]=c[v+(l<<2)>>2];c[j+((f|1)<<2)>>2]=c[v+((l|1)<<2)>>2];q=q+1|0;}while((q|0)<15);Lf(t);q=0;do{l=q<<1;f=q<<6;c[h+(f<<2)>>2]=c[j+(l<<2)>>2];c[h+((f|1)<<2)>>2]=c[j+((l|1)<<2)>>2];q=q+1|0;}while((q|0)<15);q=k+1|0;if((q|0)<32){v=v+8|0;k=q;h=h+8|0}else{break}}c[d>>2]=(c[d>>2]|0)+8;i=e;return};case 16:{Le(b);c[d>>2]=(c[d>>2]|0)+3;i=e;return};case 8:{h=b+32|0;k=c[h>>2]|0;v=k+(c[b>>2]|0)>>1;t=b+16|0;q=b+48|0;l=c[q>>2]|0;f=b+4|0;z=b+36|0;o=c[z>>2]|0;n=o+(c[f>>2]|0)>>1;m=b+20|0;p=b+52|0;x=c[p>>2]|0;s=l+(c[t>>2]|0)>>1;w=x+(c[m>>2]|0)>>1;u=v-k|0;k=s-l|0;l=n-o|0;o=w-x|0;x=b+8|0;a=b+40|0;r=c[a>>2]|0;y=r+(c[x>>2]|0)>>1;A=b+24|0;B=b+56|0;C=c[B>>2]|0;D=b+12|0;E=b+44|0;F=c[E>>2]|0;H=F+(c[D>>2]|0)>>1;I=b+28|0;J=b+60|0;K=c[J>>2]|0;L=C+(c[A>>2]|0)>>1;M=K+(c[I>>2]|0)>>1;N=y-r|0;r=L-C|0;C=H-F|0;F=M-K|0;K=F+N|0;O=N-F|0;F=C-r|0;N=C+r|0;r=s+v>>1;C=w+n>>1;P=L+y>>1;c[b>>2]=P+r;Q=M+H>>1;c[f>>2]=Q+C;c[h>>2]=r-P;c[z>>2]=C-Q;Q=v-s>>1;s=n-w>>1;w=H-M>>1;c[t>>2]=w+Q;t=y-L>>1;c[m>>2]=s-t;c[q>>2]=Q-w;c[p>>2]=t+s;s=K+F|0;N2(s,(s|0)<0|0?-1:0,1518469120,0)|0;s=G;t=F-K|0;N2(t,(t|0)<0|0?-1:0,1518469120,0)|0;t=G;K=o+u>>1;c[x>>2]=s+K;x=l-k>>1;c[D>>2]=t+x;c[a>>2]=K-s;c[E>>2]=x-t;t=N-O|0;N2(t,(t|0)<0|0?-1:0,1518469120,0)|0;t=G;x=O+N|0;N2(x,(x|0)<0|0?-1:0,1518469120,0)|0;x=G;N=u-o>>1;c[A>>2]=t+N;A=l+k>>1;c[I>>2]=A-x;c[B>>2]=N-t;c[J>>2]=x+A;c[d>>2]=(c[d>>2]|0)+2;i=e;return};case 15:{Lf(b);c[d>>2]=(c[d>>2]|0)+2;i=e;return};case 64:{Lh(b,6,1962120,512);c[d>>2]=(c[d>>2]|0)+5;i=e;return};case 240:{A=g|0;g=b;x=A;J=0;while(1){t=0;do{N=t<<1;B=t*30|0;c[x+(N<<2)>>2]=c[g+(B<<2)>>2];c[x+((N|1)<<2)>>2]=c[g+((B|1)<<2)>>2];t=t+1|0;}while((t|0)<16);Le(x);t=J+1|0;if((t|0)<15){g=g+8|0;x=x+128|0;J=t}else{break}}Lg(A,16,240,1967256,1968816);J=j|0;x=A;A=0;g=b;while(1){t=0;do{B=t<<1;N=t<<5;c[j+(B<<2)>>2]=c[x+(N<<2)>>2];c[j+((B|1)<<2)>>2]=c[x+((N|1)<<2)>>2];t=t+1|0;}while((t|0)<15);Lf(J);t=0;do{N=t<<1;B=t<<5;c[g+(B<<2)>>2]=c[j+(N<<2)>>2];c[g+((B|1)<<2)>>2]=c[j+((N|1)<<2)>>2];t=t+1|0;}while((t|0)<15);t=A+1|0;if((t|0)<16){x=x+8|0;A=t;g=g+8|0}else{break}}c[d>>2]=(c[d>>2]|0)+7;i=e;return};case 512:{Lh(b,9,1962120,512);c[d>>2]=(c[d>>2]|0)+8;i=e;return};default:{i=e;return}}}function Ld(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0;b=i;i=i+256|0;d=b|0;e=a+128|0;f=c[e>>2]|0;g=f+(c[a>>2]|0)>>1;h=a+64|0;j=a+192|0;k=c[j>>2]|0;l=a+4|0;m=a+132|0;n=c[m>>2]|0;o=n+(c[l>>2]|0)>>1;p=a+68|0;q=a+196|0;r=c[q>>2]|0;s=r+(c[p>>2]|0)|0;t=k+(c[h>>2]|0)>>1;u=d|0;c[u>>2]=t+g;c[d+16>>2]=g-t;v=s>>1;c[d+4>>2]=v+o;c[d+20>>2]=o-v;s=g-f|0;f=t-k|0;k=o-n|0;n=v-r|0;c[d+8>>2]=n+s;c[d+24>>2]=s-n;c[d+12>>2]=k-f;c[d+28>>2]=k+f;f=a+32|0;k=a+160|0;n=c[k>>2]|0;s=n+(c[f>>2]|0)>>1;r=a+96|0;v=a+224|0;o=c[v>>2]|0;t=a+36|0;g=a+164|0;w=c[g>>2]|0;x=w+(c[t>>2]|0)>>1;y=a+100|0;z=a+228|0;A=c[z>>2]|0;B=A+(c[y>>2]|0)|0;C=o+(c[r>>2]|0)>>1;c[d+32>>2]=C+s;c[d+48>>2]=s-C;D=B>>1;c[d+36>>2]=D+x;c[d+52>>2]=x-D;B=s-n|0;n=C-o|0;o=x-w|0;w=D-A|0;c[d+40>>2]=w+B;c[d+56>>2]=B-w;c[d+44>>2]=o-n;c[d+60>>2]=o+n;n=a+16|0;o=a+144|0;w=c[o>>2]|0;B=w+(c[n>>2]|0)>>1;A=a+80|0;D=a+208|0;x=c[D>>2]|0;C=a+20|0;s=a+148|0;E=c[s>>2]|0;F=E+(c[C>>2]|0)>>1;H=a+84|0;I=a+212|0;J=c[I>>2]|0;K=J+(c[H>>2]|0)|0;L=x+(c[A>>2]|0)>>1;c[d+64>>2]=L+B;c[d+80>>2]=B-L;M=K>>1;c[d+68>>2]=M+F;c[d+84>>2]=F-M;K=B-w|0;w=L-x|0;x=F-E|0;E=M-J|0;c[d+72>>2]=E+K;c[d+88>>2]=K-E;c[d+76>>2]=x-w;c[d+92>>2]=x+w;w=a+48|0;x=a+176|0;E=c[x>>2]|0;K=E+(c[w>>2]|0)>>1;J=a+112|0;M=a+240|0;F=c[M>>2]|0;L=a+52|0;B=a+180|0;N=c[B>>2]|0;O=N+(c[L>>2]|0)>>1;P=a+116|0;Q=a+244|0;R=c[Q>>2]|0;S=R+(c[P>>2]|0)|0;T=F+(c[J>>2]|0)>>1;c[d+96>>2]=T+K;c[d+112>>2]=K-T;U=S>>1;c[d+100>>2]=U+O;c[d+116>>2]=O-U;S=K-E|0;E=T-F|0;F=O-N|0;N=U-R|0;c[d+104>>2]=N+S;c[d+120>>2]=S-N;c[d+108>>2]=F-E;c[d+124>>2]=F+E;E=a+8|0;F=a+136|0;N=c[F>>2]|0;S=N+(c[E>>2]|0)>>1;R=a+72|0;U=a+200|0;O=c[U>>2]|0;T=a+12|0;K=a+140|0;V=c[K>>2]|0;W=V+(c[T>>2]|0)>>1;X=a+76|0;Y=a+204|0;Z=c[Y>>2]|0;_=Z+(c[X>>2]|0)|0;$=O+(c[R>>2]|0)>>1;c[d+128>>2]=$+S;c[d+144>>2]=S-$;aa=_>>1;c[d+132>>2]=aa+W;c[d+148>>2]=W-aa;_=S-N|0;N=$-O|0;O=W-V|0;V=aa-Z|0;c[d+136>>2]=V+_;c[d+152>>2]=_-V;c[d+140>>2]=O-N;c[d+156>>2]=O+N;N=a+40|0;O=a+168|0;V=c[O>>2]|0;_=V+(c[N>>2]|0)>>1;Z=a+104|0;aa=a+232|0;W=c[aa>>2]|0;$=a+44|0;S=a+172|0;ab=c[S>>2]|0;ac=ab+(c[$>>2]|0)>>1;ad=a+108|0;ae=a+236|0;af=c[ae>>2]|0;ag=af+(c[ad>>2]|0)|0;ah=W+(c[Z>>2]|0)>>1;c[d+160>>2]=ah+_;c[d+176>>2]=_-ah;ai=ag>>1;c[d+164>>2]=ai+ac;c[d+180>>2]=ac-ai;ag=_-V|0;V=ah-W|0;W=ac-ab|0;ab=ai-af|0;c[d+168>>2]=ab+ag;c[d+184>>2]=ag-ab;c[d+172>>2]=W-V;c[d+188>>2]=W+V;V=a+24|0;W=a+152|0;ab=c[W>>2]|0;ag=ab+(c[V>>2]|0)>>1;af=a+88|0;ai=a+216|0;ac=c[ai>>2]|0;ah=a+28|0;_=a+156|0;aj=c[_>>2]|0;ak=aj+(c[ah>>2]|0)>>1;al=a+92|0;am=a+220|0;an=c[am>>2]|0;ao=an+(c[al>>2]|0)|0;ap=ac+(c[af>>2]|0)>>1;c[d+192>>2]=ap+ag;c[d+208>>2]=ag-ap;aq=ao>>1;c[d+196>>2]=aq+ak;c[d+212>>2]=ak-aq;ao=ag-ab|0;ab=ap-ac|0;ac=ak-aj|0;aj=aq-an|0;c[d+200>>2]=aj+ao;c[d+216>>2]=ao-aj;c[d+204>>2]=ac-ab;c[d+220>>2]=ac+ab;ab=a+56|0;ac=a+184|0;aj=c[ac>>2]|0;ao=aj+(c[ab>>2]|0)>>1;an=a+120|0;aq=a+248|0;ak=c[aq>>2]|0;ap=a+60|0;ag=a+188|0;ar=c[ag>>2]|0;as=ar+(c[ap>>2]|0)>>1;at=a+124|0;au=a+252|0;av=c[au>>2]|0;aw=av+(c[at>>2]|0)|0;ax=ak+(c[an>>2]|0)>>1;c[d+224>>2]=ax+ao;c[d+240>>2]=ao-ax;ay=aw>>1;c[d+228>>2]=ay+as;c[d+244>>2]=as-ay;aw=ao-aj|0;aj=ax-ak|0;ak=as-ar|0;ar=ay-av|0;c[d+232>>2]=ar+aw;c[d+248>>2]=aw-ar;c[d+236>>2]=ak-aj;c[d+252>>2]=ak+aj;aj=a;ak=u;u=4;while(1){d=c[ak+36>>2]|0;ar=c[ak>>2]>>1;aw=c[ak+4>>2]>>1;av=c[ak+32>>2]>>1;c[aj>>2]=ar+av;ay=d>>1;c[aj+4>>2]=aw+ay;c[aj+32>>2]=ar-av;c[aj+36>>2]=aw-ay;ay=c[ak+48>>2]|0;aw=c[ak+16>>2]>>1;av=c[ak+20>>2]>>1;ar=c[ak+52>>2]>>1;c[aj+16>>2]=aw+ar;d=ay>>1;c[aj+20>>2]=av-d;c[aj+48>>2]=aw-ar;c[aj+52>>2]=av+d;d=c[ak+40>>2]|0;N2(d,(d|0)<0|0?-1:0,1518469120,0)|0;d=G;av=c[ak+44>>2]|0;N2(av,(av|0)<0|0?-1:0,1518469120,0)|0;av=G;ar=av-d|0;aw=av+d|0;d=c[ak+12>>2]|0;av=c[ak+8>>2]>>1;c[aj+8>>2]=aw+av;ay=d>>1;c[aj+12>>2]=ar+ay;c[aj+40>>2]=av-aw;c[aj+44>>2]=ay-ar;ar=c[ak+56>>2]|0;N2(ar,(ar|0)<0|0?-1:0,1518469120,0)|0;ar=G;ay=c[ak+60>>2]|0;N2(ay,(ay|0)<0|0?-1:0,1518469120,0)|0;ay=G;aw=ay-ar|0;av=ay+ar|0;ar=c[ak+28>>2]|0;ay=c[ak+24>>2]>>1;c[aj+24>>2]=aw+ay;d=ar>>1;c[aj+28>>2]=d-av;c[aj+56>>2]=ay-aw;c[aj+60>>2]=av+d;d=u-1|0;if((d|0)==0){break}else{aj=aj+64|0;ak=ak+64|0;u=d}}u=c[p>>2]|0;ak=c[a>>2]>>1;aj=c[l>>2]>>1;d=c[h>>2]>>1;c[a>>2]=ak+d;av=u>>1;c[l>>2]=aj+av;c[h>>2]=ak-d;c[p>>2]=aj-av;av=c[r>>2]|0;aj=c[f>>2]>>1;d=c[t>>2]>>1;ak=c[y>>2]>>1;c[f>>2]=aj+ak;u=av>>1;c[t>>2]=d-u;c[r>>2]=aj-ak;c[y>>2]=d+u;u=c[q>>2]|0;d=c[e>>2]>>1;ak=c[m>>2]>>1;aj=c[j>>2]>>1;c[e>>2]=d+aj;av=u>>1;c[m>>2]=ak+av;c[j>>2]=d-aj;c[q>>2]=ak-av;av=c[v>>2]|0;ak=c[k>>2]>>1;aj=c[g>>2]>>1;d=c[z>>2]>>1;c[k>>2]=ak+d;u=av>>1;c[g>>2]=aj-u;c[v>>2]=ak-d;c[z>>2]=aj+u;u=c[X>>2]|0;aj=c[R>>2]|0;d=u;ak=(u|0)<0|0?-1:0;N2(d,ak,1984036864,0)|0;u=G;av=aj;aw=(aj|0)<0|0?-1:0;N2(av,aw,821821440,0)|0;aj=u-G|0;N2(d,ak,821821440,0)|0;ak=G;N2(av,aw,1984036864,0)|0;aw=G+ak|0;ak=c[T>>2]|0;av=c[E>>2]>>1;c[E>>2]=aw+av;d=ak>>1;c[T>>2]=aj+d;c[R>>2]=av-aw;c[X>>2]=d-aj;aj=c[ad>>2]|0;d=c[Z>>2]|0;aw=aj;av=(aj|0)<0|0?-1:0;N2(aw,av,1984036864,0)|0;aj=G;ak=d;u=(d|0)<0|0?-1:0;N2(ak,u,821821440,0)|0;d=aj-G|0;N2(aw,av,821821440,0)|0;av=G;N2(ak,u,1984036864,0)|0;u=G+av|0;av=c[$>>2]|0;ak=c[N>>2]>>1;c[N>>2]=d+ak;aw=av>>1;c[$>>2]=aw-u;c[Z>>2]=ak-d;c[ad>>2]=u+aw;aw=c[Y>>2]|0;u=c[U>>2]|0;d=aw;ak=(aw|0)<0|0?-1:0;N2(d,ak,1984036864,0)|0;aw=G;av=u;aj=(u|0)<0|0?-1:0;N2(av,aj,821821440,0)|0;u=aw-G|0;N2(d,ak,821821440,0)|0;ak=G;N2(av,aj,1984036864,0)|0;aj=G+ak|0;ak=c[K>>2]|0;av=c[F>>2]>>1;c[F>>2]=aj+av;d=ak>>1;c[K>>2]=u+d;c[U>>2]=av-aj;c[Y>>2]=d-u;u=c[ae>>2]|0;d=c[aa>>2]|0;aj=u;av=(u|0)<0|0?-1:0;N2(aj,av,1984036864,0)|0;u=G;ak=d;aw=(d|0)<0|0?-1:0;N2(ak,aw,821821440,0)|0;d=u-G|0;N2(aj,av,821821440,0)|0;av=G;N2(ak,aw,1984036864,0)|0;aw=G+av|0;av=c[S>>2]|0;ak=c[O>>2]>>1;c[O>>2]=d+ak;aj=av>>1;c[S>>2]=aj-aw;c[aa>>2]=ak-d;c[ae>>2]=aw+aj;aj=c[A>>2]|0;N2(aj,(aj|0)<0|0?-1:0,1518469120,0)|0;aj=G;aw=c[H>>2]|0;N2(aw,(aw|0)<0|0?-1:0,1518469120,0)|0;aw=G;d=aw-aj|0;ak=aw+aj|0;aj=c[C>>2]|0;aw=c[n>>2]>>1;c[n>>2]=ak+aw;av=aj>>1;c[C>>2]=d+av;c[A>>2]=aw-ak;c[H>>2]=av-d;d=c[J>>2]|0;N2(d,(d|0)<0|0?-1:0,1518469120,0)|0;d=G;av=c[P>>2]|0;N2(av,(av|0)<0|0?-1:0,1518469120,0)|0;av=G;ak=av-d|0;aw=av+d|0;d=c[L>>2]|0;av=c[w>>2]>>1;c[w>>2]=ak+av;aj=d>>1;c[L>>2]=aj-aw;c[J>>2]=av-ak;c[P>>2]=aw+aj;aj=c[D>>2]|0;N2(aj,(aj|0)<0|0?-1:0,1518469120,0)|0;aj=G;aw=c[I>>2]|0;N2(aw,(aw|0)<0|0?-1:0,1518469120,0)|0;aw=G;ak=aw-aj|0;av=aw+aj|0;aj=c[s>>2]|0;aw=c[o>>2]>>1;c[o>>2]=av+aw;d=aj>>1;c[s>>2]=ak+d;c[D>>2]=aw-av;c[I>>2]=d-ak;ak=c[M>>2]|0;N2(ak,(ak|0)<0|0?-1:0,1518469120,0)|0;ak=G;d=c[Q>>2]|0;N2(d,(d|0)<0|0?-1:0,1518469120,0)|0;d=G;av=d-ak|0;aw=d+ak|0;ak=c[B>>2]|0;d=c[x>>2]>>1;c[x>>2]=av+d;aj=ak>>1;c[B>>2]=aj-aw;c[M>>2]=d-av;c[Q>>2]=aw+aj;aj=c[al>>2]|0;aw=c[af>>2]|0;av=aj;d=(aj|0)<0|0?-1:0;N2(av,d,821821440,0)|0;aj=G;ak=aw;u=(aw|0)<0|0?-1:0;N2(ak,u,1984036864,0)|0;aw=aj-G|0;N2(av,d,1984036864,0)|0;d=G;N2(ak,u,821821440,0)|0;u=G+d|0;d=c[ah>>2]|0;ak=c[V>>2]>>1;c[V>>2]=u+ak;av=d>>1;c[ah>>2]=aw+av;c[af>>2]=ak-u;c[al>>2]=av-aw;aw=c[at>>2]|0;av=c[an>>2]|0;u=aw;ak=(aw|0)<0|0?-1:0;N2(u,ak,821821440,0)|0;aw=G;d=av;aj=(av|0)<0|0?-1:0;N2(d,aj,1984036864,0)|0;av=aw-G|0;N2(u,ak,1984036864,0)|0;ak=G;N2(d,aj,821821440,0)|0;aj=G+ak|0;ak=c[ap>>2]|0;d=c[ab>>2]>>1;c[ab>>2]=av+d;u=ak>>1;c[ap>>2]=u-aj;c[an>>2]=d-av;c[at>>2]=aj+u;u=c[am>>2]|0;aj=c[ai>>2]|0;av=u;d=(u|0)<0|0?-1:0;N2(av,d,821821440,0)|0;u=G;ak=aj;aw=(aj|0)<0|0?-1:0;N2(ak,aw,1984036864,0)|0;aj=u-G|0;N2(av,d,1984036864,0)|0;d=G;N2(ak,aw,821821440,0)|0;aw=G+d|0;d=c[_>>2]|0;ak=c[W>>2]>>1;c[W>>2]=aw+ak;av=d>>1;c[_>>2]=aj+av;c[ai>>2]=ak-aw;c[am>>2]=av-aj;aj=c[au>>2]|0;av=c[aq>>2]|0;aw=aj;ak=(aj|0)<0|0?-1:0;N2(aw,ak,821821440,0)|0;aj=G;d=av;u=(av|0)<0|0?-1:0;N2(d,u,1984036864,0)|0;av=aj-G|0;N2(aw,ak,1984036864,0)|0;ak=G;N2(d,u,821821440,0)|0;u=G+ak|0;ak=c[ag>>2]|0;d=c[ac>>2]>>1;c[ac>>2]=av+d;aw=ak>>1;c[ag>>2]=aw-u;c[aq>>2]=d-av;c[au>>2]=u+aw;aw=c[m>>2]|0;u=c[a>>2]>>1;av=c[l>>2]>>1;d=c[e>>2]>>1;c[a>>2]=u+d;a=aw>>1;c[l>>2]=av+a;c[e>>2]=u-d;c[m>>2]=av-a;a=c[j>>2]|0;av=c[h>>2]>>1;m=c[p>>2]>>1;d=c[q>>2]>>1;c[h>>2]=av+d;h=a>>1;c[p>>2]=m-h;c[j>>2]=av-d;c[q>>2]=m+h;h=c[K>>2]|0;m=c[F>>2]|0;q=h;d=(h|0)<0|0?-1:0;N2(q,d,2106195968,0)|0;h=G;av=m;j=(m|0)<0|0?-1:0;N2(av,j,418971648,0)|0;m=h-G|0;N2(q,d,418971648,0)|0;d=G;N2(av,j,2106195968,0)|0;j=G+d|0;d=c[T>>2]|0;av=c[E>>2]>>1;c[E>>2]=j+av;E=d>>1;c[T>>2]=m+E;c[F>>2]=av-j;c[K>>2]=E-m;m=c[Y>>2]|0;E=c[U>>2]|0;K=m;j=(m|0)<0|0?-1:0;N2(K,j,2106195968,0)|0;m=G;av=E;F=(E|0)<0|0?-1:0;N2(av,F,418971648,0)|0;E=m-G|0;N2(K,j,418971648,0)|0;j=G;N2(av,F,2106195968,0)|0;F=G+j|0;j=c[X>>2]|0;av=c[R>>2]>>1;c[R>>2]=E+av;R=j>>1;c[X>>2]=R-F;c[U>>2]=av-E;c[Y>>2]=F+R;R=c[s>>2]|0;F=c[o>>2]|0;Y=R;E=(R|0)<0|0?-1:0;N2(Y,E,1984036864,0)|0;R=G;av=F;U=(F|0)<0|0?-1:0;N2(av,U,821821440,0)|0;F=R-G|0;N2(Y,E,821821440,0)|0;E=G;N2(av,U,1984036864,0)|0;U=G+E|0;E=c[C>>2]|0;av=c[n>>2]>>1;c[n>>2]=U+av;n=E>>1;c[C>>2]=F+n;c[o>>2]=av-U;c[s>>2]=n-F;F=c[I>>2]|0;n=c[D>>2]|0;s=F;U=(F|0)<0|0?-1:0;N2(s,U,1984036864,0)|0;F=G;av=n;o=(n|0)<0|0?-1:0;N2(av,o,821821440,0)|0;n=F-G|0;N2(s,U,821821440,0)|0;U=G;N2(av,o,1984036864,0)|0;o=G+U|0;U=c[H>>2]|0;av=c[A>>2]>>1;c[A>>2]=n+av;A=U>>1;c[H>>2]=A-o;c[D>>2]=av-n;c[I>>2]=o+A;A=c[_>>2]|0;o=c[W>>2]|0;I=A;n=(A|0)<0|0?-1:0;N2(I,n,1785593856,0)|0;A=G;av=o;D=(o|0)<0|0?-1:0;N2(av,D,1193082880,0)|0;o=A-G|0;N2(I,n,1193082880,0)|0;n=G;N2(av,D,1785593856,0)|0;D=G+n|0;n=c[ah>>2]|0;av=c[V>>2]>>1;c[V>>2]=D+av;V=n>>1;c[ah>>2]=o+V;c[W>>2]=av-D;c[_>>2]=V-o;o=c[am>>2]|0;V=c[ai>>2]|0;_=o;D=(o|0)<0|0?-1:0;N2(_,D,1785593856,0)|0;o=G;av=V;W=(V|0)<0|0?-1:0;N2(av,W,1193082880,0)|0;V=o-G|0;N2(_,D,1193082880,0)|0;D=G;N2(av,W,1785593856,0)|0;W=G+D|0;D=c[al>>2]|0;av=c[af>>2]>>1;c[af>>2]=V+av;af=D>>1;c[al>>2]=af-W;c[ai>>2]=av-V;c[am>>2]=W+af;af=c[k>>2]|0;N2(af,(af|0)<0|0?-1:0,1518469120,0)|0;af=G;W=c[g>>2]|0;N2(W,(W|0)<0|0?-1:0,1518469120,0)|0;W=G;am=W-af|0;V=W+af|0;af=c[t>>2]|0;W=c[f>>2]>>1;c[f>>2]=V+W;f=af>>1;c[t>>2]=am+f;c[k>>2]=W-V;c[g>>2]=f-am;am=c[v>>2]|0;N2(am,(am|0)<0|0?-1:0,1518469120,0)|0;am=G;f=c[z>>2]|0;N2(f,(f|0)<0|0?-1:0,1518469120,0)|0;f=G;g=f-am|0;V=f+am|0;am=c[y>>2]|0;f=c[r>>2]>>1;c[r>>2]=g+f;r=am>>1;c[y>>2]=r-V;c[v>>2]=f-g;c[z>>2]=V+r;r=c[S>>2]|0;V=c[O>>2]|0;z=r;g=(r|0)<0|0?-1:0;N2(z,g,1193082880,0)|0;r=G;f=V;v=(V|0)<0|0?-1:0;N2(f,v,1785593856,0)|0;V=r-G|0;N2(z,g,1785593856,0)|0;g=G;N2(f,v,1193082880,0)|0;v=G+g|0;g=c[$>>2]|0;f=c[N>>2]>>1;c[N>>2]=v+f;N=g>>1;c[$>>2]=V+N;c[O>>2]=f-v;c[S>>2]=N-V;V=c[ae>>2]|0;N=c[aa>>2]|0;S=V;v=(V|0)<0|0?-1:0;N2(S,v,1193082880,0)|0;V=G;f=N;O=(N|0)<0|0?-1:0;N2(f,O,1785593856,0)|0;N=V-G|0;N2(S,v,1785593856,0)|0;v=G;N2(f,O,1193082880,0)|0;O=G+v|0;v=c[ad>>2]|0;f=c[Z>>2]>>1;c[Z>>2]=N+f;Z=v>>1;c[ad>>2]=Z-O;c[aa>>2]=f-N;c[ae>>2]=O+Z;Z=c[B>>2]|0;O=c[x>>2]|0;ae=Z;N=(Z|0)<0|0?-1:0;N2(ae,N,821821440,0)|0;Z=G;f=O;aa=(O|0)<0|0?-1:0;N2(f,aa,1984036864,0)|0;O=Z-G|0;N2(ae,N,1984036864,0)|0;N=G;N2(f,aa,821821440,0)|0;aa=G+N|0;N=c[L>>2]|0;f=c[w>>2]>>1;c[w>>2]=aa+f;w=N>>1;c[L>>2]=O+w;c[x>>2]=f-aa;c[B>>2]=w-O;O=c[Q>>2]|0;w=c[M>>2]|0;B=O;aa=(O|0)<0|0?-1:0;N2(B,aa,821821440,0)|0;O=G;f=w;x=(w|0)<0|0?-1:0;N2(f,x,1984036864,0)|0;w=O-G|0;N2(B,aa,1984036864,0)|0;aa=G;N2(f,x,821821440,0)|0;x=G+aa|0;aa=c[P>>2]|0;f=c[J>>2]>>1;c[J>>2]=w+f;J=aa>>1;c[P>>2]=J-x;c[M>>2]=f-w;c[Q>>2]=x+J;J=c[ag>>2]|0;x=c[ac>>2]|0;Q=J;w=(J|0)<0|0?-1:0;N2(Q,w,418971648,0)|0;J=G;f=x;M=(x|0)<0|0?-1:0;N2(f,M,2106195968,0)|0;x=J-G|0;N2(Q,w,2106195968,0)|0;w=G;N2(f,M,418971648,0)|0;M=G+w|0;w=c[ap>>2]|0;f=c[ab>>2]>>1;c[ab>>2]=M+f;ab=w>>1;c[ap>>2]=x+ab;c[ac>>2]=f-M;c[ag>>2]=ab-x;x=c[au>>2]|0;ab=c[aq>>2]|0;ag=x;M=(x|0)<0|0?-1:0;N2(ag,M,418971648,0)|0;x=G;f=ab;ac=(ab|0)<0|0?-1:0;N2(f,ac,2106195968,0)|0;ab=x-G|0;N2(ag,M,2106195968,0)|0;M=G;N2(f,ac,418971648,0)|0;ac=G+M|0;M=c[at>>2]|0;f=c[an>>2]>>1;c[an>>2]=ab+f;an=M>>1;c[at>>2]=an-ac;c[aq>>2]=f-ab;c[au>>2]=ac+an;i=b;return}function Le(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0;b=a+64|0;d=c[b>>2]|0;e=(d>>1)+(c[a>>2]>>1)|0;f=a+32|0;g=a+96|0;h=c[g>>2]|0;i=(h>>1)+(c[f>>2]>>1)|0;j=a+4|0;k=a+68|0;l=c[k>>2]|0;m=(l>>1)+(c[j>>2]>>1)|0;n=a+36|0;o=a+100|0;p=c[o>>2]|0;q=(p>>1)+(c[n>>2]>>1)|0;r=e-d|0;d=i-h|0;h=m-l|0;l=q-p|0;p=a+16|0;s=a+80|0;t=c[s>>2]|0;u=(t>>1)+(c[p>>2]>>1)|0;v=a+48|0;w=a+112|0;x=c[w>>2]|0;y=(x>>1)+(c[v>>2]>>1)|0;z=a+20|0;A=a+84|0;B=c[A>>2]|0;C=(B>>1)+(c[z>>2]>>1)|0;D=a+52|0;E=a+116|0;F=c[E>>2]|0;H=(F>>1)+(c[D>>2]>>1)|0;I=u-t|0;t=y-x|0;x=C-B|0;B=H-F|0;F=B+I|0;J=I-B|0;B=x-t|0;I=x+t|0;t=a+8|0;x=a+72|0;K=c[x>>2]|0;L=(K>>1)+(c[t>>2]>>1)|0;M=a+40|0;N=a+104|0;O=c[N>>2]|0;P=(O>>1)+(c[M>>2]>>1)|0;Q=a+12|0;R=a+76|0;S=c[R>>2]|0;T=(S>>1)+(c[Q>>2]>>1)|0;U=a+44|0;V=a+108|0;W=c[V>>2]|0;X=(W>>1)+(c[U>>2]>>1)|0;Y=L-K|0;K=P-O|0;O=T-S|0;S=X-W|0;W=a+24|0;Z=a+88|0;_=c[Z>>2]|0;$=(_>>1)+(c[W>>2]>>1)|0;aa=a+56|0;ab=a+120|0;ac=c[ab>>2]|0;ad=(ac>>1)+(c[aa>>2]>>1)|0;ae=a+28|0;af=a+92|0;ag=c[af>>2]|0;ah=(ag>>1)+(c[ae>>2]>>1)|0;ai=a+60|0;aj=a+124|0;ak=c[aj>>2]|0;al=(ak>>1)+(c[ai>>2]>>1)|0;am=$-_|0;_=ad-ac|0;ac=ah-ag|0;ag=al-ak|0;ak=ag+am|0;an=am-ag|0;ag=ac-_|0;am=ac+_|0;_=i+e>>1;ac=q+m>>1;ao=y+u>>1;c[a>>2]=ao+_;ap=H+C>>1;c[j>>2]=ap+ac;c[f>>2]=_-ao;c[n>>2]=ac-ap;ap=e-i>>1;i=m-q>>1;q=C-H>>1;c[p>>2]=q+ap;H=u-y>>1;c[z>>2]=i-H;c[v>>2]=ap-q;c[D>>2]=H+i;i=P+L>>1;H=X+T>>1;q=ad+$>>1;c[b>>2]=q+i;ap=al+ah>>1;c[k>>2]=ap+H;c[g>>2]=i-q;c[o>>2]=H-ap;ap=L-P>>1;P=T-X>>1;X=ah-al>>1;c[s>>2]=X+ap;al=$-ad>>1;c[A>>2]=P-al;c[w>>2]=ap-X;c[E>>2]=al+P;N2(F,(F|0)<0|0?-1:0,1518469120,0)|0;F=G;N2(B,(B|0)<0|0?-1:0,1518469120,0)|0;B=G;P=B-F|0;al=F+B|0;B=l+r>>1;c[t>>2]=al+B;F=h-d>>1;c[Q>>2]=P+F;c[M>>2]=B-al;c[U>>2]=F-P;N2(J,(J|0)<0|0?-1:0,1518469120,0)|0;J=G;N2(I,(I|0)<0|0?-1:0,1518469120,0)|0;I=G;P=I-J|0;F=J+I|0;I=r-l>>1;c[W>>2]=P+I;l=h+d>>1;c[ae>>2]=l-F;c[aa>>2]=I-P;c[ai>>2]=F+l;N2(ak,(ak|0)<0|0?-1:0,1518469120,0)|0;ak=G;N2(ag,(ag|0)<0|0?-1:0,1518469120,0)|0;ag=G;l=ag-ak|0;F=ak+ag|0;ag=S+Y>>1;ak=F+ag|0;c[x>>2]=ak;P=O-K>>1;I=l+P|0;c[R>>2]=I;c[N>>2]=ag-F;c[V>>2]=P-l;N2(an,(an|0)<0|0?-1:0,1518469120,0)|0;an=G;N2(am,(am|0)<0|0?-1:0,1518469120,0)|0;am=G;l=am-an|0;P=an+am|0;am=Y-S>>1;c[Z>>2]=l+am;S=O+K>>1;c[af>>2]=S-P;c[ab>>2]=am-l;c[aj>>2]=P+S;S=c[k>>2]|0;P=c[a>>2]>>1;l=c[j>>2]>>1;am=c[b>>2]>>1;c[a>>2]=P+am;a=S>>1;c[j>>2]=l+a;c[b>>2]=P-am;c[k>>2]=l-a;a=c[g>>2]|0;l=c[f>>2]>>1;k=c[n>>2]>>1;am=c[o>>2]>>1;c[f>>2]=l+am;f=a>>1;c[n>>2]=k-f;c[g>>2]=l-am;c[o>>2]=k+f;f=I;k=(I|0)<0|0?-1:0;N2(f,k,1984036864,0)|0;I=G;o=ak;am=(ak|0)<0|0?-1:0;N2(o,am,821821440,0)|0;ak=I-G|0;N2(f,k,821821440,0)|0;k=G;N2(o,am,1984036864,0)|0;am=G+k|0;k=c[Q>>2]|0;o=c[t>>2]>>1;c[t>>2]=am+o;t=k>>1;c[Q>>2]=ak+t;c[x>>2]=o-am;c[R>>2]=t-ak;ak=c[V>>2]|0;t=c[N>>2]|0;R=ak;am=(ak|0)<0|0?-1:0;N2(R,am,1984036864,0)|0;ak=G;o=t;x=(t|0)<0|0?-1:0;N2(o,x,821821440,0)|0;t=ak-G|0;N2(R,am,821821440,0)|0;am=G;N2(o,x,1984036864,0)|0;x=G+am|0;am=c[U>>2]|0;o=c[M>>2]>>1;c[M>>2]=t+o;M=am>>1;c[U>>2]=M-x;c[N>>2]=o-t;c[V>>2]=x+M;M=c[s>>2]|0;N2(M,(M|0)<0|0?-1:0,1518469120,0)|0;M=G;x=c[A>>2]|0;N2(x,(x|0)<0|0?-1:0,1518469120,0)|0;x=G;V=x-M|0;t=x+M|0;M=c[z>>2]|0;x=c[p>>2]>>1;c[p>>2]=t+x;p=M>>1;c[z>>2]=V+p;c[s>>2]=x-t;c[A>>2]=p-V;V=c[w>>2]|0;N2(V,(V|0)<0|0?-1:0,1518469120,0)|0;V=G;p=c[E>>2]|0;N2(p,(p|0)<0|0?-1:0,1518469120,0)|0;p=G;A=p-V|0;t=p+V|0;V=c[D>>2]|0;p=c[v>>2]>>1;c[v>>2]=A+p;v=V>>1;c[D>>2]=v-t;c[w>>2]=p-A;c[E>>2]=t+v;v=c[af>>2]|0;t=c[Z>>2]|0;E=v;A=(v|0)<0|0?-1:0;N2(E,A,821821440,0)|0;v=G;p=t;w=(t|0)<0|0?-1:0;N2(p,w,1984036864,0)|0;t=v-G|0;N2(E,A,1984036864,0)|0;A=G;N2(p,w,821821440,0)|0;w=G+A|0;A=c[ae>>2]|0;p=c[W>>2]>>1;c[W>>2]=w+p;W=A>>1;c[ae>>2]=t+W;c[Z>>2]=p-w;c[af>>2]=W-t;t=c[aj>>2]|0;W=c[ab>>2]|0;af=t;w=(t|0)<0|0?-1:0;N2(af,w,821821440,0)|0;t=G;p=W;Z=(W|0)<0|0?-1:0;N2(p,Z,1984036864,0)|0;W=t-G|0;N2(af,w,1984036864,0)|0;w=G;N2(p,Z,821821440,0)|0;Z=G+w|0;w=c[ai>>2]|0;p=c[aa>>2]>>1;c[aa>>2]=W+p;aa=w>>1;c[ai>>2]=aa-Z;c[ab>>2]=p-W;c[aj>>2]=Z+aa;return}function Lf(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;b=i;i=i+240|0;d=b|0;e=b+120|0;f=0;g=0;h=0;while(1){j=c[a+(f<<2)>>2]|0;k=c[a+(f+1<<2)>>2]|0;l=f+10|0;m=(l|0)>29?f-20|0:l;l=c[a+(m<<2)>>2]|0;n=c[a+(m+1<<2)>>2]|0;o=m+10|0;p=(o|0)>29?m-20|0:o;o=c[a+(p<<2)>>2]|0;m=c[a+(p+1<<2)>>2]|0;q=p+16|0;r=o+l|0;s=l-o|0;N2(s,(s|0)<0|0?-1:0,-1859780608,-1)|0;s=G<<1|0>>>31;c[d+(g<<2)>>2]=j+r>>2;o=j-(r>>1)|0;r=m+n|0;j=n-m|0;N2(j,(j|0)<0|0?-1:0,-1859780608,-1)|0;j=G<<1|0>>>31;c[d+((g|1)<<2)>>2]=k+r>>2;m=k-(r>>1)|0;c[d+(g+2<<2)>>2]=o-j>>2;c[d+(g+4<<2)>>2]=j+o>>2;c[d+(g+3<<2)>>2]=m+s>>2;c[d+(g+5<<2)>>2]=m-s>>2;s=h+1|0;if((s|0)<5){f=(q|0)>29?p-14|0:q;g=g+6|0;h=s}else{t=0;u=0;break}}while(1){h=u<<1;g=c[d+(h<<2)>>2]|0;f=h|1;s=c[d+(f<<2)>>2]|0;q=c[d+(h+6<<2)>>2]|0;p=c[d+(f+6<<2)>>2]|0;m=c[d+(h+12<<2)>>2]|0;o=c[d+(f+12<<2)>>2]|0;j=c[d+(h+18<<2)>>2]|0;r=c[d+(f+18<<2)>>2]|0;k=c[d+(h+24<<2)>>2]|0;h=c[d+(f+24<<2)>>2]|0;f=k+q|0;n=q-k|0;k=j+m|0;q=m-j|0;j=f-k|0;N2(j,(j|0)<0|0?-1:0,1200488448,0)|0;j=G<<1|0>>>31;m=k+f|0;f=m+g|0;c[e+(t<<2)>>2]=f;N2(m,(m|0)<0|0?-1:0,-1342177280,-1)|0;m=(G<<2|0>>>30)+f|0;f=m-j|0;g=m+j|0;j=q+n|0;N2(j,(j|0)<0|0?-1:0,2042363904,0)|0;j=G<<1|0>>>31;N2(n,(n|0)<0|0?-1:0,-1652293632,-1)|0;n=j+(G<<2|0>>>30)|0;N2(q,(q|0)<0|0?-1:0,-780140544,-1)|0;q=j+(G<<1|0>>>31)|0;j=h+p|0;m=p-h|0;h=r+o|0;p=o-r|0;r=j-h|0;N2(r,(r|0)<0|0?-1:0,1200488448,0)|0;r=G<<1|0>>>31;o=h+j|0;j=o+s|0;c[e+((t|1)<<2)>>2]=j;N2(o,(o|0)<0|0?-1:0,-1342177280,-1)|0;o=(G<<2|0>>>30)+j|0;j=o-r|0;s=o+r|0;r=p+m|0;N2(r,(r|0)<0|0?-1:0,2042363904,0)|0;r=G<<1|0>>>31;N2(m,(m|0)<0|0?-1:0,-1652293632,-1)|0;m=r+(G<<2|0>>>30)|0;N2(p,(p|0)<0|0?-1:0,-780140544,-1)|0;p=r+(G<<1|0>>>31)|0;c[e+(t+2<<2)>>2]=p+g;c[e+(t+8<<2)>>2]=g-p;c[e+(t+4<<2)>>2]=f-m;c[e+(t+6<<2)>>2]=m+f;c[e+(t+3<<2)>>2]=s-q;c[e+(t+9<<2)>>2]=s+q;c[e+(t+5<<2)>>2]=j+n;c[e+(t+7<<2)>>2]=j-n;n=u+1|0;if((n|0)<3){t=t+10|0;u=n}else{v=0;w=0;x=0;break}}while(1){c[a+(w<<2)>>2]=c[e+(v<<2)>>2];c[a+((w|1)<<2)>>2]=c[e+(v+1<<2)>>2];u=v+12|0;t=(u|0)>29?v-18|0:u;c[a+(w+2<<2)>>2]=c[e+(t<<2)>>2];c[a+(w+3<<2)>>2]=c[e+(t+1<<2)>>2];u=t+12|0;d=(u|0)>29?t-18|0:u;c[a+(w+4<<2)>>2]=c[e+(d<<2)>>2];c[a+(w+5<<2)>>2]=c[e+(d+1<<2)>>2];u=d+12|0;t=(u|0)>29?d-18|0:u;c[a+(w+6<<2)>>2]=c[e+(t<<2)>>2];c[a+(w+7<<2)>>2]=c[e+(t+1<<2)>>2];u=t+12|0;d=(u|0)>29?t-18|0:u;c[a+(w+8<<2)>>2]=c[e+(d<<2)>>2];c[a+(w+9<<2)>>2]=c[e+(d+1<<2)>>2];u=x+1|0;if((u|0)<3){v=d+2|0;w=w+10|0;x=u}else{break}}i=b;return}function Lg(a,d,f,g,h){a=a|0;d=d|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;if((d|0)>0){i=0;while(1){j=i<<1;k=a+(j<<2)|0;l=a+((j|1)<<2)|0;j=c[l>>2]|0;c[k>>2]=c[k>>2]>>2;c[l>>2]=j>>2;j=i+1|0;if((j|0)<(d|0)){i=j}else{m=d;break}}}else{m=0}if((m|0)>=(f|0)){return}i=d-1|0;j=h;h=g;g=m;while(1){m=g<<1;l=a+(m<<2)|0;k=a+((m|1)<<2)|0;m=c[k>>2]|0;c[l>>2]=c[l>>2]>>2;c[k>>2]=m>>2;m=g+1|0;k=g+d|0;if((m|0)<(k|0)){l=j+(i<<1)|0;n=j;o=h;p=m;while(1){m=p<<1;q=a+(m<<2)|0;r=c[q>>2]>>1;s=a+((m|1)<<2)|0;m=c[s>>2]>>1;t=b[n>>1]|0;u=(e[o>>1]|0)<<16;v=u;w=(u|0)<0|0?-1:0;u=m;x=(m|0)<0|0?-1:0;N2(v,w,u,x)|0;m=G;y=(t&65535)<<16;t=y;z=(y|0)<0|0?-1:0;y=r;A=(r|0)<0|0?-1:0;N2(t,z,y,A)|0;c[s>>2]=m-G;N2(t,z,u,x)|0;x=G;N2(v,w,y,A)|0;c[q>>2]=x+G;x=p+1|0;if((x|0)<(k|0)){n=n+2|0;o=o+2|0;p=x}else{break}}B=l;C=h+(i<<1)|0}else{B=j;C=h}if((k|0)<(f|0)){j=B;h=C;g=k}else{break}}return}function Lh(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0;f=1<<b;g=f-1|0;if((g|0)>1){h=0;i=1;while(1){j=h;k=f;do{k=k>>1;j=j^k;}while((j&k|0)==0);if((j|0)>(i|0)){k=i<<1;l=a+(k<<2)|0;m=c[l>>2]|0;n=j<<1;o=a+(n<<2)|0;c[l>>2]=c[o>>2];c[o>>2]=m;m=a+((k|1)<<2)|0;k=c[m>>2]|0;o=a+((n|1)<<2)|0;c[m>>2]=c[o>>2];c[o>>2]=k}k=i+1|0;if((k|0)<(g|0)){h=j;i=k}else{break}}}i=f<<1;if((i|0)>0){h=0;do{g=a+(h<<2)|0;k=a+((h|2)<<2)|0;o=c[k>>2]|0;m=o+(c[g>>2]|0)>>1;n=a+((h|4)<<2)|0;l=a+((h|6)<<2)|0;p=c[l>>2]|0;q=p+(c[n>>2]|0)>>1;r=a+((h|1)<<2)|0;s=a+((h|3)<<2)|0;t=c[s>>2]|0;u=t+(c[r>>2]|0)>>1;v=a+((h|5)<<2)|0;w=a+((h|7)<<2)|0;x=c[w>>2]|0;y=x+(c[v>>2]|0)>>1;c[g>>2]=q+m;c[n>>2]=m-q;c[r>>2]=y+u;c[v>>2]=u-y;v=m-o|0;o=q-p|0;p=u-t|0;t=y-x|0;c[k>>2]=t+v;c[l>>2]=v-t;c[s>>2]=p-o;c[w>>2]=p+o;h=h+8|0;}while((h|0)<(i|0))}if((b|0)<3){return}i=e<<2;e=(f|0)>0;h=3;do{o=1<<h;p=o>>1;w=i>>h;if(e){s=p<<1;t=0;do{v=t<<1;l=v+s|0;k=a+((l|1)<<2)|0;x=c[k>>2]>>1;y=a+(l<<2)|0;l=c[y>>2]>>1;u=a+(v<<2)|0;q=c[u>>2]>>1;m=a+((v|1)<<2)|0;r=c[m>>2]>>1;c[u>>2]=q+l;c[m>>2]=r+x;c[y>>2]=q-l;c[k>>2]=r-x;x=v+p|0;v=x+s|0;r=a+(v+1<<2)|0;k=c[r>>2]>>1;l=a+(v<<2)|0;v=c[l>>2]>>1;q=a+(x<<2)|0;y=c[q>>2]>>1;m=a+(x+1<<2)|0;x=c[m>>2]>>1;c[q>>2]=y+k;c[m>>2]=x-v;c[l>>2]=y-k;c[r>>2]=x+v;t=t+o|0;}while((t|0)<(f|0))}t=(p|0)/4|0;do{if((p|0)>7){s=p<<1;j=(p|0)/2|0;if(e){z=1}else{break}while(1){v=c[d+((ac(z,w)|0)<<2)>>2]|0;x=v&-65536;r=v<<16;v=r;k=(r|0)<0|0?-1:0;r=x;y=(x|0)<0|0?-1:0;x=0;do{l=x+z<<1;m=l+s|0;q=a+((m|1)<<2)|0;u=c[q>>2]|0;n=a+(m<<2)|0;m=c[n>>2]|0;g=u;A=(u|0)<0|0?-1:0;N2(g,A,v,k)|0;u=G;B=m;C=(m|0)<0|0?-1:0;N2(B,C,r,y)|0;m=u-G|0;N2(g,A,r,y)|0;A=G;N2(B,C,v,k)|0;C=G+A|0;A=a+(l<<2)|0;B=c[A>>2]>>1;g=a+((l|1)<<2)|0;u=c[g>>2]>>1;c[A>>2]=C+B;c[g>>2]=m+u;c[n>>2]=B-C;c[q>>2]=u-m;m=l+p|0;l=m+s|0;u=a+(l+1<<2)|0;q=c[u>>2]|0;C=a+(l<<2)|0;l=c[C>>2]|0;B=q;n=(q|0)<0|0?-1:0;N2(B,n,v,k)|0;q=G;g=l;A=(l|0)<0|0?-1:0;N2(g,A,r,y)|0;l=q-G|0;N2(B,n,r,y)|0;n=G;N2(g,A,v,k)|0;A=G+n|0;n=a+(m<<2)|0;g=c[n>>2]>>1;B=a+(m+1<<2)|0;m=c[B>>2]>>1;c[n>>2]=l+g;c[B>>2]=m-A;c[C>>2]=g-l;c[u>>2]=A+m;m=x-z+j<<1;A=m+s|0;u=a+(A<<2)|0;l=c[u>>2]|0;g=a+((A|1)<<2)|0;A=c[g>>2]|0;C=l;B=(l|0)<0|0?-1:0;N2(C,B,v,k)|0;l=G;n=A;q=(A|0)<0|0?-1:0;N2(n,q,r,y)|0;A=l-G|0;N2(C,B,r,y)|0;B=G;N2(n,q,v,k)|0;q=G+B|0;B=a+(m<<2)|0;n=c[B>>2]>>1;C=a+((m|1)<<2)|0;l=c[C>>2]>>1;c[B>>2]=q+n;c[C>>2]=l-A;c[u>>2]=n-q;c[g>>2]=A+l;l=m+p|0;m=l+s|0;A=a+(m<<2)|0;g=c[A>>2]|0;q=a+(m+1<<2)|0;m=c[q>>2]|0;n=g;u=(g|0)<0|0?-1:0;N2(n,u,v,k)|0;g=G;C=m;B=(m|0)<0|0?-1:0;N2(C,B,r,y)|0;m=g-G|0;N2(n,u,r,y)|0;u=G;N2(C,B,v,k)|0;B=G+u|0;u=a+(l<<2)|0;C=c[u>>2]>>1;n=a+(l+1<<2)|0;l=c[n>>2]>>1;c[u>>2]=C-m;c[n>>2]=l-B;c[A>>2]=m+C;c[q>>2]=B+l;x=x+o|0;}while((x|0)<(f|0));x=z+1|0;if((x|0)<(t|0)){z=x}else{D=10710;break}}}else{D=10710}}while(0);do{if((D|0)==10710){D=0;if(!e){break}w=p<<1;s=0;do{j=s+t<<1;x=j+w|0;k=a+((x|1)<<2)|0;v=c[k>>2]|0;y=a+(x<<2)|0;x=c[y>>2]|0;N2(v,(v|0)<0|0?-1:0,1518469120,0)|0;v=G;N2(x,(x|0)<0|0?-1:0,1518469120,0)|0;x=G;r=v-x|0;l=x+v|0;v=a+(j<<2)|0;x=c[v>>2]>>1;B=a+((j|1)<<2)|0;q=c[B>>2]>>1;c[v>>2]=l+x;c[B>>2]=r+q;c[y>>2]=x-l;c[k>>2]=q-r;r=j+p|0;j=r+w|0;q=a+(j+1<<2)|0;k=c[q>>2]|0;l=a+(j<<2)|0;j=c[l>>2]|0;N2(k,(k|0)<0|0?-1:0,1518469120,0)|0;k=G;N2(j,(j|0)<0|0?-1:0,1518469120,0)|0;j=G;x=k-j|0;y=j+k|0;k=a+(r<<2)|0;j=c[k>>2]>>1;B=a+(r+1<<2)|0;r=c[B>>2]>>1;c[k>>2]=x+j;c[B>>2]=r-y;c[l>>2]=j-x;c[q>>2]=y+r;s=s+o|0;}while((s|0)<(f|0))}}while(0);h=h+1|0;}while((h|0)<=(b|0));return}function Li(a,f,g,h,j,k){a=a|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0;l=i;i=i+24|0;m=l|0;n=l+8|0;o=l+16|0;p=a+24|0;q=c[p>>2]|0;r=a+4|0;s=q>>1;if((s|0)==0){t=q}else{u=j<<1;v=s;s=h;h=(c[r>>2]|0)+((q*9|0)<<1)|0;while(1){b[h>>1]=b[s>>1]|0;b[h+2>>1]=b[s+(j<<1)>>1]|0;w=v-1|0;if((w|0)==0){break}else{v=w;s=s+(u<<1)|0;h=h+4|0}}t=c[p>>2]|0}h=a+48|0;u=c[a>>2]|0;s=d[a+52|0]|0;v=c[r>>2]|0;j=t<<1;L14912:do{if((c[h>>2]&2|0)==0){w=(t*10|0)-1|0;x=s*5|0;y=t<<3;z=y-1|0;A=ac(b[v+(w<<1)>>1]|0,b[u>>1]|0)|0;B=-j|0;C=w-j|0;w=(ac(b[v+(C<<1)>>1]|0,b[u+2>>1]|0)|0)+A|0;A=C-j|0;C=w+(ac(b[v+(A<<1)>>1]|0,b[u+4>>1]|0)|0)|0;w=A-j|0;A=C+(ac(b[v+(w<<1)>>1]|0,b[u+6>>1]|0)|0)|0;C=w-j|0;c[k>>2]=A+(ac(b[v+(C<<1)>>1]|0,b[u+8>>1]|0)|0)<<1;A=t-1|0;w=x+1|0;D=t<<2;E=x+2|0;F=D+j|0;H=x+3|0;I=F+j|0;J=x+4|0;K=I+(-y|1)|0;y=B<<1;L=y-j|0;M=L-j|0;N=M+z|0;O=0;P=u;Q=v+(C+z<<1)|0;z=v;C=k;R=k+(j-1<<2)|0;while(1){S=P+(x<<1)|0;T=C+4|0;U=ac(b[z>>1]|0,b[S>>1]|0)|0;V=P+(w<<1)|0;W=(ac(b[z+(j<<1)>>1]|0,b[V>>1]|0)|0)+U|0;U=P+(E<<1)|0;X=W+(ac(b[z+(D<<1)>>1]|0,b[U>>1]|0)|0)|0;W=P+(H<<1)|0;Y=X+(ac(b[z+(F<<1)>>1]|0,b[W>>1]|0)|0)|0;X=P+(J<<1)|0;c[R>>2]=Y+(ac(b[z+(I<<1)>>1]|0,b[X>>1]|0)|0)<<1;if((O|0)>=(A|0)){break L14912}Y=ac(b[Q>>1]|0,b[S>>1]|0)|0;Z=(ac(b[Q+(B<<1)>>1]|0,b[V>>1]|0)|0)+Y|0;Y=Z+(ac(b[Q+(y<<1)>>1]|0,b[U>>1]|0)|0)|0;U=Y+(ac(b[Q+(L<<1)>>1]|0,b[W>>1]|0)|0)|0;c[T>>2]=U+(ac(b[Q+(M<<1)>>1]|0,b[X>>1]|0)|0)<<1;O=O+1|0;P=S;Q=Q+(N<<1)|0;z=z+(K<<1)|0;C=T;R=R-4|0}}else{if((j|0)<=0){break}R=s*5|0;C=R-5|0;K=j-1|0;z=R-4|0;N=R-3|0;Q=R-2|0;P=t<<2;O=R-1|0;M=t*6|0;L=t<<3;y=v;B=u;A=0;while(1){I=ac(b[y>>1]|0,b[B+(C<<1)>>1]|0)|0;J=(ac(b[y+(j<<1)>>1]|0,b[B+(z<<1)>>1]|0)|0)+I|0;I=J+(ac(b[y+(P<<1)>>1]|0,b[B+(N<<1)>>1]|0)|0)|0;J=I+(ac(b[y+(M<<1)>>1]|0,b[B+(Q<<1)>>1]|0)|0)|0;c[k+(K-A<<2)>>2]=J+(ac(b[y+(L<<1)>>1]|0,b[B+(O<<1)>>1]|0)|0)<<1;J=A+1|0;if((J|0)<(j|0)){y=y+2|0;B=B+(R<<1)|0;A=J}else{break}}}}while(0);j=c[h>>2]|0;if((j&1|0)==0){h=c[p>>2]|0;u=h<<1;c[o>>2]=0;if((h|0)>0){v=u-1|0;t=u-2|0;u=0;s=0;while(1){A=c[k+(u<<2)>>2]>>1;R=u|1;B=c[k+(R<<2)>>2]>>1;y=c[k+(v+s<<2)>>2]>>1;O=c[k+(t+s<<2)>>2]>>1;c[f+(u<<2)>>2]=A-y;c[f+(R<<2)>>2]=B-O;c[g+(u<<2)>>2]=y+A;c[g+(R<<2)>>2]=O+B;B=u+2|0;O=-2-u|0;if((B|0)<(h|0)){u=B;s=O}else{break}}}La(f,h,o);Lb(g,h,o);o=c[a+12>>2]|0;h=c[a+16>>2]|0;s=a+32|0;if((c[s>>2]|0)>0){_=0}else{$=c[r>>2]|0;aa=$;ab=c[p>>2]|0;ad=aa+(ab<<1)|0;ae=ad;af=q*18|0;IZ($,ae,af);i=l;return}do{a=g+(_<<2)|0;u=f+(_<<2)|0;t=c[a>>2]|0;v=c[u>>2]|0;O=b[h+(_<<1)>>1]|0;B=e[o+(_<<1)>>1]<<16;R=B;A=(B|0)<0|0?-1:0;B=t;y=(t|0)<0|0?-1:0;N2(R,A,B,y)|0;t=G<<1|0>>>31;L=(O&65535)<<16;O=L;K=(L|0)<0|0?-1:0;L=v;Q=(v|0)<0|0?-1:0;N2(O,K,L,Q)|0;c[a>>2]=t-(G<<1|0>>>31);N2(O,K,B,y)|0;y=G<<1|0>>>31;N2(R,A,L,Q)|0;c[u>>2]=y+(G<<1|0>>>31);_=_+1|0;}while((_|0)<(c[s>>2]|0));$=c[r>>2]|0;aa=$;ab=c[p>>2]|0;ad=aa+(ab<<1)|0;ae=ad;af=q*18|0;IZ($,ae,af);i=l;return}s=c[p>>2]|0;if((j&4|0)!=0){j=s>>1;_=(s>>6)+1|0;c[n>>2]=_;if((j|0)>0){o=s-1|0;h=(s<<1)-1|0;g=j-1|0;y=0;u=0;while(1){c[f+(y+j<<2)>>2]=(c[k+(o+u<<2)>>2]>>1)-(c[k+(y<<2)>>2]>>_);c[f+(g+u<<2)>>2]=(c[k+(h+u<<2)>>2]>>_)+(c[k+(y+s<<2)>>2]>>1);Q=y+1|0;L=~y;if((Q|0)<(j|0)){y=Q;u=L}else{break}}}La(f,s,n);$=c[r>>2]|0;aa=$;ab=c[p>>2]|0;ad=aa+(ab<<1)|0;ae=ad;af=q*18|0;IZ($,ae,af);i=l;return}n=s>>1;u=k+((n*3|0)<<2)|0;c[f>>2]=c[u>>2]>>1;y=n-1|0;if((y|0)!=0){j=u;_=u;u=f;h=y;do{j=j-4|0;_=_+4|0;u=u+4|0;c[u>>2]=(c[_>>2]>>1)+(c[j>>2]>>1);h=h-1|0;}while((h|0)!=0)}if((n|0)!=(s|0)){h=s-n|0;j=k+(n<<1<<2)|0;_=k;u=f+(n<<2)|0;while(1){c[u>>2]=(c[j>>2]>>1)-(c[_>>2]>>1);n=h-1|0;if((n|0)==0){break}else{h=n;j=j-4|0;_=_+4|0;u=u+4|0}}}K8(f,k,s,m);$=c[r>>2]|0;aa=$;ab=c[p>>2]|0;ad=aa+(ab<<1)|0;ae=ad;af=q*18|0;IZ($,ae,af);i=l;return}function Lj(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0;i=c[a+24>>2]|0;j=e|0;c[j>>2]=-7;c[j>>2]=-7-(c[a+20>>2]|0);j=a+28|0;if((c[j>>2]|0)<=0){return}e=a+48|0;k=ac(i,g)|0;i=0;l=f;while(1){if((c[e>>2]&1|0)==0){m=c[d+(i<<2)>>2]|0}else{m=0}Li(a,c[b+(i<<2)>>2]|0,m,l,g,h);f=i+1|0;if((f|0)<(c[j>>2]|0)){i=f;l=l+(k<<1)|0}else{break}}return}function Lk(a,f,g,h,j,k,l,m){a=a|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0;n=i;i=i+24|0;o=n|0;p=n+8|0;q=n+16|0;r=a+48|0;s=c[r>>2]|0;do{if((s&1|0)==0){t=a+24|0;u=c[t>>2]|0;v=u>>1;c[q>>2]=0;w=m+(u<<2)|0;if((s&4|0)==0){x=s}else{y=a+32|0;if((c[y>>2]|0)>0){z=a+12|0;A=a+16|0;B=-h|0;if((h|0)>0){C=0;while(1){D=c[g+(C<<2)>>2]<<h;E=c[f+(C<<2)>>2]<<h;F=D;H=(D|0)<0|0?-1:0;D=E;I=(E|0)<0|0?-1:0;E=b[(c[A>>2]|0)+(C<<1)>>1]|0;J=(e[(c[z>>2]|0)+(C<<1)>>1]|0)<<16;K=J;L=(J|0)<0|0?-1:0;N2(K,L,F,H)|0;J=G<<1|0>>>31;M=(E&65535)<<16;E=M;N=(M|0)<0|0?-1:0;N2(E,N,D,I)|0;c[m+(C+u<<2)>>2]=J-(G<<1|0>>>31);N2(E,N,F,H)|0;H=G<<1|0>>>31;N2(K,L,D,I)|0;c[m+(C<<2)>>2]=H+(G<<1|0>>>31);H=C+1|0;if((H|0)<(c[y>>2]|0)){C=H}else{O=H;break}}}else{C=0;while(1){H=c[g+(C<<2)>>2]>>B;I=c[f+(C<<2)>>2]>>B;D=H;L=(H|0)<0|0?-1:0;H=I;K=(I|0)<0|0?-1:0;I=b[(c[A>>2]|0)+(C<<1)>>1]|0;F=(e[(c[z>>2]|0)+(C<<1)>>1]|0)<<16;N=F;E=(F|0)<0|0?-1:0;N2(N,E,D,L)|0;F=G<<1|0>>>31;J=(I&65535)<<16;I=J;M=(J|0)<0|0?-1:0;N2(I,M,H,K)|0;c[m+(C+u<<2)>>2]=F-(G<<1|0>>>31);N2(I,M,D,L)|0;L=G<<1|0>>>31;N2(N,E,H,K)|0;c[m+(C<<2)>>2]=L+(G<<1|0>>>31);L=C+1|0;if((L|0)<(c[y>>2]|0)){C=L}else{O=L;break}}}}else{O=0}C=a+36|0;if((O|0)<(c[C>>2]|0)){y=a+12|0;z=a+16|0;A=-j|0;if((j|0)>0){B=O;do{L=c[g+(B<<2)>>2]<<j;K=c[f+(B<<2)>>2]<<j;H=L;E=(L|0)<0|0?-1:0;L=K;N=(K|0)<0|0?-1:0;K=b[(c[z>>2]|0)+(B<<1)>>1]|0;D=(e[(c[y>>2]|0)+(B<<1)>>1]|0)<<16;M=D;I=(D|0)<0|0?-1:0;N2(M,I,H,E)|0;D=G<<1|0>>>31;F=(K&65535)<<16;K=F;J=(F|0)<0|0?-1:0;N2(K,J,L,N)|0;c[m+(B+u<<2)>>2]=D-(G<<1|0>>>31);N2(K,J,H,E)|0;E=G<<1|0>>>31;N2(M,I,L,N)|0;c[m+(B<<2)>>2]=E+(G<<1|0>>>31);B=B+1|0;}while((B|0)<(c[C>>2]|0))}else{B=O;do{E=c[g+(B<<2)>>2]>>A;N=c[f+(B<<2)>>2]>>A;L=E;I=(E|0)<0|0?-1:0;E=N;M=(N|0)<0|0?-1:0;N=b[(c[z>>2]|0)+(B<<1)>>1]|0;H=(e[(c[y>>2]|0)+(B<<1)>>1]|0)<<16;J=H;K=(H|0)<0|0?-1:0;N2(J,K,L,I)|0;H=G<<1|0>>>31;D=(N&65535)<<16;N=D;F=(D|0)<0|0?-1:0;N2(N,F,E,M)|0;c[m+(B+u<<2)>>2]=H-(G<<1|0>>>31);N2(N,F,L,I)|0;I=G<<1|0>>>31;N2(J,K,E,M)|0;c[m+(B<<2)>>2]=I+(G<<1|0>>>31);B=B+1|0;}while((B|0)<(c[C>>2]|0))}}x=c[r>>2]|0}if((x&4|0)==0){C=a+32|0;Lo(m,f,c[C>>2]|0,h);B=c[C>>2]|0;y=a+36|0;Lo(m+(B<<2)|0,f+(B<<2)|0,(c[y>>2]|0)-B|0,j);Lo(w,g,c[C>>2]|0,h);B=c[C>>2]|0;Lo(m+(B+u<<2)|0,g+(B<<2)|0,(c[y>>2]|0)-B|0,j);P=y}else{P=a+36|0}y=c[P>>2]|0;I_(m+(y<<2)|0,(c[t>>2]|0)-y<<2);y=c[P>>2]|0;I_(m+(y+u<<2)|0,(c[t>>2]|0)-y<<2);La(m,u,q);Lb(w,u,q);y=(v|0)>0;if((c[r>>2]&4|0)==0){if(!y){break}B=u-1|0;C=(u<<1)-1|0;z=0;A=0;while(1){I=m+(z<<2)|0;M=c[I>>2]|0;E=m+(C+A<<2)|0;K=c[E>>2]|0;J=m+(B+A<<2)|0;L=c[J>>2]|0;F=m+(z+u<<2)|0;N=c[F>>2]|0;c[I>>2]=N-M>>1;c[E>>2]=N+M>>1;c[J>>2]=K-L>>1;c[F>>2]=L+K>>1;K=z+1|0;L=~z;if((K|0)<(v|0)){z=K;A=L}else{break}}}else{if(!y){break}A=u-1|0;z=(u<<1)-1|0;B=0;C=0;while(1){w=m+(B<<2)|0;t=c[w>>2]|0;L=m+(z+C<<2)|0;K=c[L>>2]|0;F=m+(A+C<<2)|0;J=c[F>>2]|0;M=m+(B+u<<2)|0;N=c[M>>2]|0;c[w>>2]=t-N>>1;c[L>>2]=-(t+N|0)>>1;c[F>>2]=J-K>>1;c[M>>2]=-(J+K|0)>>1;K=B+1|0;J=~B;if((K|0)<(v|0)){B=K;C=J}else{break}}}}else{if((s&4|0)==0){C=c[a+24>>2]|0;B=C>>1;v=m+(C<<2)|0;u=a+32|0;Lo(m,f,c[u>>2]|0,h);A=c[u>>2]|0;u=a+36|0;Lo(m+(A<<2)|0,f+(A<<2)|0,(c[u>>2]|0)-A|0,j);A=c[u>>2]|0;I_(m+(A<<2)|0,C-A<<2);K9(m,v,C,o);A=m+(B<<2)|0;c[v>>2]=c[A>>2];c[m+(B+C<<2)>>2]=0;v=c[m>>2]|0;c[m>>2]=c[A>>2];c[A>>2]=v;v=(B|0)/2|0;if((B|0)>3){A=1;do{u=C-A|0;z=m+(u<<2)|0;y=c[z>>2]|0;J=B-A|0;c[m+(J+C<<2)>>2]=y;K=A+B|0;c[m+(K+C<<2)>>2]=-y;y=m+(K<<2)|0;K=c[y>>2]|0;c[m+(A+C<<2)>>2]=K;c[m+(u+C<<2)>>2]=-K;K=m+(A<<2)|0;c[y>>2]=c[K>>2];y=m+(J<<2)|0;c[z>>2]=c[y>>2];z=c[K>>2]|0;c[K>>2]=c[y>>2];c[y>>2]=z;A=A+1|0;}while((A|0)<(v|0))}A=v+B|0;z=m+(A<<2)|0;y=c[z>>2]|0;c[m+(v+C<<2)>>2]=y;c[m+(A+C<<2)>>2]=-y;c[z>>2]=c[m+(v<<2)>>2];break}else{z=c[a+24>>2]|0;y=z>>1;c[p>>2]=0;A=m+(y<<2)|0;K=a+32|0;Lo(A,f,c[K>>2]|0,h);J=c[K>>2]|0;K=a+36|0;Lo(m+(J+y<<2)|0,f+(J<<2)|0,(c[K>>2]|0)-J|0,j);J=c[K>>2]|0;I_(m+(J+y<<2)|0,z-J<<2);La(A,z,p);if((y|0)<=0){break}A=z-1|0;J=(z<<1)-1|0;K=0;u=0;while(1){c[m+(K<<2)>>2]=c[m+(A+u<<2)>>2];c[m+(J+u<<2)>>2]=-(c[m+(K+z<<2)>>2]|0);M=K+1|0;F=~K;if((M|0)<(y|0)){K=M;u=F}else{break}}}}}while(0);p=c[a+24>>2]|0;j=c[a>>2]|0;if((c[r>>2]&2|0)!=0){r=15-(c[a+40>>2]|0)|0;if((p|0)<=0){i=n;return}f=(d[a+52|0]|0)*5|0;h=j+(((c[a+8>>2]|0)/2|0)<<1)|0;o=j;s=c[a+4>>2]|0;q=p;while(1){P=q-1|0;g=c[m+(P+p<<2)>>2]|0;x=c[m+(P<<2)>>2]|0;O=c[s>>2]|0;u=(e[h+8>>1]|0)<<16;K=x;y=(x|0)<0|0?-1:0;N2(u,(u|0)<0|0?-1:0,K,y)|0;u=G+O|0;O=((u|0)>0?u:-u|0)>>r;if((O|0)>32767){Q=32767}else{Q=(O|0)<-32768?-32768:O&65535}b[k+((ac(P,l)|0)<<1)>>1]=(u|0)<0?-Q&65535:Q;u=s+4|0;O=c[u>>2]|0;x=(e[o+8>>1]|0)<<16;z=g;J=(g|0)<0|0?-1:0;N2(x,(x|0)<0|0?-1:0,z,J)|0;c[s>>2]=G+O;O=s+8|0;x=c[O>>2]|0;g=(e[h+6>>1]|0)<<16;N2(g,(g|0)<0|0?-1:0,K,y)|0;c[u>>2]=G+x;x=s+12|0;u=c[x>>2]|0;g=(e[o+6>>1]|0)<<16;N2(g,(g|0)<0|0?-1:0,z,J)|0;c[O>>2]=G+u;u=s+16|0;O=c[u>>2]|0;g=(e[h+4>>1]|0)<<16;N2(g,(g|0)<0|0?-1:0,K,y)|0;c[x>>2]=G+O;O=s+20|0;x=c[O>>2]|0;g=(e[o+4>>1]|0)<<16;N2(g,(g|0)<0|0?-1:0,z,J)|0;c[u>>2]=G+x;x=s+24|0;u=c[x>>2]|0;g=(e[h+2>>1]|0)<<16;N2(g,(g|0)<0|0?-1:0,K,y)|0;c[O>>2]=G+u;u=s+28|0;O=c[u>>2]|0;g=(e[o+2>>1]|0)<<16;N2(g,(g|0)<0|0?-1:0,z,J)|0;c[x>>2]=G+O;O=s+32|0;x=c[O>>2]|0;g=(e[h>>1]|0)<<16;N2(g,(g|0)<0|0?-1:0,K,y)|0;c[u>>2]=G+x;x=(e[o>>1]|0)<<16;N2(x,(x|0)<0|0?-1:0,z,J)|0;c[O>>2]=G;if((P|0)>0){h=h+(f<<1)|0;o=o+(f<<1)|0;s=s+36|0;q=P}else{break}}i=n;return}q=15-(c[a+40>>2]|0)|0;s=(d[a+52|0]|0)*5|0;f=-s|0;if((p|0)<=0){i=n;return}o=a+44|0;h=s+4|0;Q=s+3|0;r=s+2|0;P=s+1|0;O=j+(((c[a+8>>2]|0)/2|0)-s<<1)|0;J=c[a+4>>2]|0;a=p;z=j;while(1){j=a-1|0;x=z+(s<<1)|0;u=c[m+(j+p<<2)>>2]|0;y=c[m+(j<<2)>>2]|0;K=c[J>>2]|0;g=(e[O>>1]|0)<<16;A=y;v=(y|0)<0|0?-1:0;N2(g,(g|0)<0|0?-1:0,A,v)|0;g=G+K|0;K=c[o>>2]|0;if((K|0)==-2147483648){R=g}else{y=g;C=(g|0)<0|0?-1:0;g=K;B=(K|0)<0|0?-1:0;N2(y,C,g,B)|0;R=G<<1|0>>>31}B=((R|0)>0?R:-R|0)>>q;if((B|0)>32767){S=32767}else{S=(B|0)<-32768?-32768:B&65535}b[k+((ac(j,l)|0)<<1)>>1]=(R|0)<0?-S&65535:S;B=J+4|0;g=c[B>>2]|0;C=(e[z+(h<<1)>>1]|0)<<16;y=u;K=(u|0)<0|0?-1:0;N2(C,(C|0)<0|0?-1:0,y,K)|0;c[J>>2]=G+g;g=J+8|0;C=c[g>>2]|0;u=(e[O+2>>1]|0)<<16;N2(u,(u|0)<0|0?-1:0,A,v)|0;c[B>>2]=G+C;C=J+12|0;B=c[C>>2]|0;u=(e[z+(Q<<1)>>1]|0)<<16;N2(u,(u|0)<0|0?-1:0,y,K)|0;c[g>>2]=G+B;B=J+16|0;g=c[B>>2]|0;u=(e[O+4>>1]|0)<<16;N2(u,(u|0)<0|0?-1:0,A,v)|0;c[C>>2]=G+g;g=J+20|0;C=c[g>>2]|0;u=(e[z+(r<<1)>>1]|0)<<16;N2(u,(u|0)<0|0?-1:0,y,K)|0;c[B>>2]=G+C;C=J+24|0;B=c[C>>2]|0;u=(e[O+6>>1]|0)<<16;N2(u,(u|0)<0|0?-1:0,A,v)|0;c[g>>2]=G+B;B=J+28|0;g=c[B>>2]|0;u=(e[z+(P<<1)>>1]|0)<<16;N2(u,(u|0)<0|0?-1:0,y,K)|0;c[C>>2]=G+g;g=J+32|0;C=c[g>>2]|0;u=(e[O+8>>1]|0)<<16;N2(u,(u|0)<0|0?-1:0,A,v)|0;c[B>>2]=G+C;C=(e[x>>1]|0)<<16;N2(C,(C|0)<0|0?-1:0,y,K)|0;c[g>>2]=G;if((j|0)>0){O=O+(f<<1)|0;J=J+36|0;a=j;z=x}else{break}}i=n;return}function Ll(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;i=Lm(a,b,d,e,f,g,h)|0;if((h&8|0)!=0){return i|0}h=c[a+4>>2]|0;if((h|0)==0){return i|0}I_(h,(c[a+24>>2]|0)*18|0);return i|0}function Lm(b,d,e,f,g,h,i){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0;I_(b,56);if((i&16|0)!=0){j=-1;return j|0}do{if((i&4|0)==0){k=i}else{l=i|2;c[b+20>>2]=1;a[b+52|0]=1;if((h|0)==64){c[b+12>>2]=87256;c[b+16>>2]=86872;c[b>>2]=87640;c[b+8>>2]=640;k=l;break}else if((h|0)==32){c[b+12>>2]=87512;c[b+16>>2]=87128;c[b>>2]=88920;c[b+8>>2]=320;k=l;break}else{j=-1;return j|0}}}while(0);i=k&4;do{if((k&20|0)==0){if((h|0)==32){c[b>>2]=89560;c[b+12>>2]=87576;c[b+16>>2]=87192;a[b+52|0]=2;c[b+8>>2]=640;c[b+20>>2]=0;m=8;n=1;break}else if((h|0)==64){c[b>>2]=89560;c[b+12>>2]=87384;c[b+16>>2]=87e3;a[b+52|0]=1;c[b+8>>2]=640;c[b+20>>2]=0;m=8;n=0;break}else{j=-1;return j|0}}else{m=(c[b+20>>2]|0)+8|0;n=(a[b+52|0]|0)==2}}while(0);c[b+48>>2]=k;c[b+24>>2]=h;c[b+28>>2]=e;c[b+32>>2]=f;c[b+36>>2]=(h|0)>(g|0)?g:h;c[b+4>>2]=d;d=b+40|0;c[d>>2]=m;if(n){o=10823}else{if((i|0)!=0&(h|0)==32){o=10823}}if((o|0)==10823){c[d>>2]=m-1}c[b+44>>2]=-2147483648;j=0;return j|0}function Ln(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0;i=a+40|0;j=c[i>>2]|0;k=Lm(a,b,d,e,f,g,h)|0;g=c[a+4>>2]|0;if((g|0)==0){return k|0}if((h&8|0)==0){I_(g,(c[a+24>>2]|0)*36|0);return k|0}if((a|0)==0){return k|0}IF(g,(c[a+24>>2]|0)*9|0,j-(c[i>>2]|0)|0);return k|0}function Lo(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;if((e|0)==0){if((a|0)==(b|0)){return}IZ(a,b,d<<2);return}if((e|0)>0){f=(e|0)<31?e:31;g=d&3;if((g|0)==0){h=b;i=a}else{j=d&3;k=a+(j<<2)|0;l=b;m=g;g=a;while(1){n=m-1|0;c[g>>2]=c[l>>2]<<f;if((n|0)==0){break}else{l=l+4|0;m=n;g=g+4|0}}h=b+(j<<2)|0;i=k}k=d>>2;if((k|0)==0){return}else{o=h;p=k;q=i}while(1){i=p-1|0;c[q>>2]=c[o>>2]<<f;c[q+4>>2]=c[o+4>>2]<<f;c[q+8>>2]=c[o+8>>2]<<f;c[q+12>>2]=c[o+12>>2]<<f;if((i|0)==0){break}else{o=o+16|0;p=i;q=q+16|0}}return}else{q=-e|0;e=(q|0)<31?q:31;q=d&3;if((q|0)==0){r=b;s=a}else{p=d&3;o=a+(p<<2)|0;f=b;i=q;q=a;while(1){a=i-1|0;c[q>>2]=c[f>>2]>>e;if((a|0)==0){break}else{f=f+4|0;i=a;q=q+4|0}}r=b+(p<<2)|0;s=o}o=d>>2;if((o|0)==0){return}else{t=r;u=o;v=s}while(1){s=u-1|0;c[v>>2]=c[t>>2]>>e;c[v+4>>2]=c[t+4>>2]>>e;c[v+8>>2]=c[t+8>>2]>>e;c[v+12>>2]=c[t+12>>2]>>e;if((s|0)==0){break}else{t=t+16|0;u=s;v=v+16|0}}return}}function Lp(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0;e=c[a+4>>2]|0;if((c[a+24>>2]|0)!=0){return 0}f=c[a+8>>2]|0;g=b+8|0;if((c[b+40>>2]|0)==0){Hy(g,c[b+4>>2]|0,0);h=b|0}else{i=b|0;Hw(g,c[i>>2]|0,c[b+4>>2]|0);h=i}i=b+4|0;c[i>>2]=0;c[h>>2]=0;h=HA(g)|0;j=a+16|0;c[j>>2]=(d|0)>1048575;k=c[497862]|0;l=c[i>>2]|0;m=l+8|0;do{if(m>>>0<32){c[i>>2]=m;n=b|0;o=c[n>>2]<<8|k&65;c[n>>2]=o;n=l+16|0;if(n>>>0<32){p=o;q=n;r=10871;break}Hw(g,o,m);c[i>>2]=8;o=k&68;c[b>>2]=o;s=o;t=16;r=10873}else{o=b|0;Hw(g,c[o>>2]|0,l);c[i>>2]=8;n=k&65;c[o>>2]=n;p=n;q=16;r=10871}}while(0);do{if((r|0)==10871){c[i>>2]=q;l=p<<8|k&68;c[b>>2]=l;m=q+8|0;if(m>>>0<32){s=l;t=m;r=10873;break}Hw(g,l,q);c[i>>2]=8;l=k&73;c[b>>2]=l;u=l;v=16;r=10875}}while(0);do{if((r|0)==10873){c[i>>2]=t;q=s<<8|k&73;c[b>>2]=q;p=t+8|0;if(p>>>0<32){u=q;v=p;r=10875;break}Hw(g,q,t);q=k&70;c[i>>2]=9;c[b>>2]=q<<1;w=q<<2;x=10;r=10879}}while(0);do{if((r|0)==10875){c[i>>2]=v;t=u<<8|k&70;c[b>>2]=t;s=v+1|0;if(s>>>0>=32){Hw(g,t,v);c[i>>2]=1;c[b>>2]=0;w=0;x=2;r=10879;break}c[i>>2]=s;q=t<<1;c[b>>2]=q;p=v+2|0;if(p>>>0>=32){Hw(g,q,s);c[i>>2]=1;c[b>>2]=0;y=0;z=2;r=10882;break}c[i>>2]=p;s=t<<2;c[b>>2]=s;t=v+3|0;if(t>>>0<32){y=s;z=t;r=10882;break}Hw(g,s,p);c[i>>2]=1;c[b>>2]=0;A=0;B=(c[j>>2]|0)!=0|0;C=c[497855]|0;D=2;r=10884}}while(0);if((r|0)==10879){c[i>>2]=x;c[b>>2]=w;y=w;z=x|1;r=10882}do{if((r|0)==10882){c[i>>2]=z;x=y<<1;c[b>>2]=x;w=(c[j>>2]|0)!=0|0;v=c[497855]|0;k=z+1|0;if(k>>>0<32){A=x;B=w;C=v;D=k;r=10884;break}Hw(g,x,z);c[i>>2]=1;x=w&v;c[b>>2]=x;E=x;F=c[497877]|0;G=24;r=10886}}while(0);do{if((r|0)==10884){c[i>>2]=D;z=A<<1|B&C;c[b>>2]=z;y=c[497877]|0;x=D+23|0;if(x>>>0<32){E=z;F=y;G=x;r=10886;break}Hw(g,z,D);c[i>>2]=23;z=y&f;c[b>>2]=z;H=z;I=27;r=10887}}while(0);do{if((r|0)==10886){c[i>>2]=G;D=E<<23|F&f;c[b>>2]=D;C=G+4|0;if(C>>>0<32){H=D;I=C;r=10887;break}Hw(g,D,G);c[i>>2]=4;c[b>>2]=0;J=4;K=0}}while(0);if((r|0)==10887){c[i>>2]=I;r=H<<4;c[b>>2]=r;J=I;K=r}do{if((c[j>>2]|0)==0){r=c[497874]|0;I=J+20|0;if(I>>>0<32){c[i>>2]=I;c[b>>2]=K<<20|r&d;break}else{Hw(g,K,J);c[i>>2]=20;c[b>>2]=r&d;break}}}while(0);Ly(b,c[a>>2]|0,e,c[a+20>>2]|0,c[a+12>>2]|0,0,0,h)|0;return 0}function Lq(a){a=a|0;var b=0;if((c[a+24>>2]|0)!=0){b=0;return b|0}b=Lz(c[a>>2]|0,0,(c[a+16>>2]|0)==0?83:63)|0;return b|0}function Lr(b,c,d){b=b|0;c=c|0;d=d|0;var e=0;if((a[b+11|0]|0)!=0){e=0;return e|0}e=Nq(b+36|0,c,d)|0;return e|0}function Ls(b,c,d){b=b|0;c=c|0;d=d|0;if((a[b+11|0]|0)!=0){return}Nr(b+36|0,c,d)|0;return}function Lt(b){b=b|0;var d=0,e=0,f=0,g=0,h=0;d=(a[b+11|0]|0)==0;do{if((c[b+28>>2]|0)==0){if(!d){e=56;break}f=a[b+22|0]|0;if(f<<24>>24==0){g=72;h=10917;break}g=((f&255)<<4)+72|0;h=10917}else{if(d){g=0;h=10917}else{e=0}}}while(0);if((h|0)==10917){e=(a[b+22|0]|0)==0?g:g+16|0}c[b+24>>2]=e;return e|0}function Lu(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;e=d+36|0;if(((c[e>>2]|0)-1|0)>>>0>3){f=-1;return f|0}g=d|0;if(((c[g>>2]|0)-1|0)>>>0>3){f=-1;return f|0}h=d+48|0;a[b+9|0]=((c[h>>2]|0)>>>20&1^1)&255;a[b+10|0]=0;a[b+11|0]=((c[h>>2]|0)>>>22&1^1)&255;a[b+12|0]=(c[g>>2]|0)+255&255;g=d+12|0;h=c[g>>2]|0;i=0;while(1){if(i>>>0>=16){break}if((c[1922120+(i<<2)>>2]|0)==(h|0)){break}else{i=i+1|0}}a[b+13|0]=i>>>0>15?15:i&255;c[b>>2]=c[g>>2];a[b+14|0]=0;c[b+4>>2]=c[d+8>>2];d=b+15|0;y=0;a[d]=y&255;y=y>>8;a[d+1|0]=y&255;y=y>>8;a[d+2|0]=y&255;y=y>>8;a[d+3|0]=y&255;a[b+22|0]=(c[e>>2]|0)+255&255;No(b+36|0,32773,65535,16);c[b+28>>2]=0;f=0;return f|0}function Lv(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0;h=b+28|0;i=c[h>>2]|0;j=b+11|0;k=(a[j]|0)==0;do{if((i|0)==0){if(!k){l=56;m=10935;break}n=a[b+22|0]|0;if(n<<24>>24==0){o=72;p=0;m=10936;break}o=((n&255)<<4)+72|0;p=n;m=10936}else{if(!k){l=0;m=10935;break}o=0;p=a[b+22|0]|0;m=10936}}while(0);if((m|0)==10936){k=b+24|0;c[k>>2]=p<<24>>24==0?o:o+16|0;Np(b+36|0);q=c[h>>2]|0;r=k}else if((m|0)==10935){k=b+24|0;c[k>>2]=l;q=i;r=k}do{if((q|0)==0){k=e+8|0;Hu(k);c[e+4>>2]=0;c[e>>2]=0;c[e+40>>2]=1;s=k;t=e+40|0;m=10941}else{k=e+8|0;i=e+40|0;if((c[e+40>>2]|0)!=0){s=k;t=i;m=10941;break}Hy(k,c[e+4>>2]|0,0);u=e|0;v=k;w=i}}while(0);if((m|0)==10941){q=e|0;Hw(s,c[q>>2]|0,c[e+4>>2]|0);u=q;v=s;w=t}t=e+4|0;c[t>>2]=0;c[u>>2]=0;c[b+32>>2]=HA(v)|0;if((c[h>>2]|0)!=0){return 0}Hu(v);c[t>>2]=0;h=e|0;c[h>>2]=0;c[w>>2]=1;w=b+22|0;do{if((a[w]|0)==0){if((a[j]|0)!=0){m=10945;break}u=Nq(b+36|0,e,0)|0;s=c[t>>2]|0;q=c[497866]|0;i=s+12|0;if(i>>>0<32){x=u;y=q;z=i;m=10947;break}Hw(v,c[h>>2]|0,s);c[t>>2]=12;A=q&4095;B=12;C=u}else{m=10945}}while(0);if((m|0)==10945){x=0;y=c[497866]|0;z=12;m=10947}if((m|0)==10947){c[t>>2]=z;A=c[h>>2]<<12|y&4095;B=z;C=x}c[h>>2]=A;x=d[b+9|0]|0;z=c[497855]|0;y=B+1|0;if(y>>>0<32){c[t>>2]=y;D=A<<1|z&x;E=y}else{Hw(v,A,B);c[t>>2]=1;D=z&x;E=1}c[h>>2]=D;x=d[b+10|0]|0;B=c[497856]|0;A=E+2|0;do{if(A>>>0<32){c[t>>2]=A;y=D<<2|B&x;c[h>>2]=y;u=d[j]|0;q=E+3|0;if(q>>>0<32){F=y;G=u;H=q;m=10956;break}Hw(v,y,A);c[t>>2]=1;y=z&u;c[h>>2]=y;I=y;J=d[b+12|0]|0;K=3;m=10957}else{Hw(v,D,E);c[t>>2]=2;y=B&x;c[h>>2]=y;F=y;G=d[j]|0;H=3;m=10956}}while(0);do{if((m|0)==10956){c[t>>2]=H;x=F<<1|z&G;c[h>>2]=x;E=d[b+12|0]|0;D=H+2|0;if(D>>>0<32){I=x;J=E;K=D;m=10957;break}Hw(v,x,H);c[t>>2]=2;L=B&E;M=2}}while(0);if((m|0)==10957){c[t>>2]=K;L=I<<2|B&J;M=K}c[h>>2]=L;K=d[b+13|0]|0;J=c[497858]|0;I=M+4|0;do{if(I>>>0<32){c[t>>2]=I;H=L<<4|J&K;c[h>>2]=H;G=d[b+14|0]|0;F=M+5|0;if(F>>>0<32){N=H;O=G;P=F;m=10962;break}Hw(v,H,I);c[t>>2]=1;Q=z&G}else{Hw(v,L,M);c[t>>2]=4;G=J&K;c[h>>2]=G;N=G;O=d[b+14|0]|0;P=5;m=10962}}while(0);if((m|0)==10962){c[t>>2]=P;Q=N<<1|z&O}c[h>>2]=Q;Q=Lx(c[b+4>>2]|0)|0;O=c[497857]|0;N=c[t>>2]|0;P=N+3|0;do{if(P>>>0<32){c[t>>2]=P;K=c[h>>2]<<3|O&Q;c[h>>2]=K;J=d[b+15|0]|0;M=N+4|0;if(M>>>0<32){R=K;S=J;T=M;m=10968;break}Hw(v,K,P);c[t>>2]=1;K=z&J;c[h>>2]=K;U=K;V=d[b+16|0]|0;W=2;m=10970}else{Hw(v,c[h>>2]|0,N);c[t>>2]=3;K=O&Q;c[h>>2]=K;R=K;S=d[b+15|0]|0;T=4;m=10968}}while(0);do{if((m|0)==10968){c[t>>2]=T;Q=R<<1|z&S;c[h>>2]=Q;O=d[b+16|0]|0;N=T+1|0;if(N>>>0<32){U=Q;V=O;W=N;m=10970;break}Hw(v,Q,T);c[t>>2]=1;Q=z&O;c[h>>2]=Q;X=Q;Y=d[b+17|0]|0;Z=2;m=10972}}while(0);do{if((m|0)==10970){c[t>>2]=W;T=U<<1|z&V;c[h>>2]=T;S=d[b+17|0]|0;R=W+1|0;if(R>>>0<32){X=T;Y=S;Z=R;m=10972;break}Hw(v,T,W);c[t>>2]=1;T=z&S;c[h>>2]=T;_=T;$=d[b+18|0]|0;aa=2;m=10973}}while(0);do{if((m|0)==10972){c[t>>2]=Z;W=X<<1|z&Y;c[h>>2]=W;V=d[b+18|0]|0;U=Z+1|0;if(U>>>0<32){_=W;$=V;aa=U;m=10973;break}Hw(v,W,Z);c[t>>2]=1;ab=z&V;ac=1}}while(0);if((m|0)==10973){c[t>>2]=aa;ab=_<<1|z&$;ac=aa}c[h>>2]=ab;aa=(c[r>>2]|0)+g>>3;g=c[497867]|0;r=ac+13|0;do{if(r>>>0<32){c[t>>2]=r;$=ab<<13|aa&g;c[h>>2]=$;z=c[497865]|0;_=ac+24|0;if(_>>>0<32){ad=$;ae=z;af=_;m=10979;break}Hw(v,$,r);c[t>>2]=11;$=z&f;c[h>>2]=$;ag=$;ah=d[w]|0;ai=13;m=10980}else{Hw(v,ab,ac);c[t>>2]=13;$=aa&g;c[h>>2]=$;ad=$;ae=c[497865]|0;af=24;m=10979}}while(0);do{if((m|0)==10979){c[t>>2]=af;g=ad<<11|ae&f;c[h>>2]=g;aa=d[w]|0;ac=af+2|0;if(ac>>>0<32){ag=g;ah=aa;ai=ac;m=10980;break}Hw(v,g,af);c[t>>2]=2;aj=B&aa;ak=2}}while(0);if((m|0)==10980){c[t>>2]=ai;aj=ag<<2|B&ah;ak=ai}c[h>>2]=aj;if((a[j]|0)!=0){return 0}if((a[w]|0)==0){j=b+36|0;Nr(j,e,C)|0;al=c[t>>2]|0}else{C=0;e=ak;ak=aj;while(1){aj=e+16|0;if(aj>>>0<32){c[t>>2]=aj;am=ak<<16;an=aj}else{Hw(v,ak,e);c[t>>2]=16;am=0;an=16}c[h>>2]=am;aj=C+1|0;if((aj|0)<(d[w]|0)){C=aj;e=an;ak=am}else{al=an;break}}}an=al+16|0;if(an>>>0<32){c[t>>2]=an;c[h>>2]=c[h>>2]<<16;return 0}else{Hw(v,c[h>>2]|0,al);c[t>>2]=16;c[h>>2]=0;return 0}return 0}function Lw(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0;g=i;i=i+96|0;h=g|0;j=g+48|0;k=b+11|0;if((a[k]|0)==0){l=h+8|0;Ht(l,c[e+28>>2]|0,c[e+32>>2]|0,0);m=h+4|0;c[m>>2]=0;n=h|0;c[n>>2]=0;o=h+40|0;c[o>>2]=1;Hw(l,0,0);c[m>>2]=0;c[n>>2]=0;Hz(l,56,c[o>>2]&255);h=b+22|0;p=(a[h]|0)==0;q=(Ns(b+36|0)|0)&65535;r=c[497870]|0;do{if(p){s=c[m>>2]|0;t=s+16|0;if(t>>>0<32){c[m>>2]=t;c[n>>2]=c[n>>2]<<16|r&q;break}else{Hw(l,c[n>>2]|0,s);c[m>>2]=16;c[n>>2]=r&q;break}}else{s=e+4|0;t=c[s>>2]|0;u=t+16|0;if(u>>>0<32){c[s>>2]=u;u=e|0;c[u>>2]=c[u>>2]<<16|r&q}else{u=e|0;Hw(e+8|0,c[u>>2]|0,t);c[s>>2]=16;c[u>>2]=r&q}u=c[b+28>>2]|0;if((u|0)>=(d[h]|0)){break}t=u<<4;u=c[m>>2]|0;v=(c[o>>2]|0)==0;do{if(u>>>0>t>>>0){if(!v){w=11012;break}c[m>>2]=u-t}else{if(!v){w=11012;break}Hy(l,u,0);w=11013}}while(0);if((w|0)==11012){Hw(l,c[n>>2]|0,u);w=11013}if((w|0)==11013){c[m>>2]=0;c[n>>2]=0;Hz(l,t,c[o>>2]&255)}v=e+8|0;if((c[e+40>>2]|0)==0){Hy(v,c[s>>2]|0,0);x=e|0}else{y=e|0;Hw(v,c[y>>2]|0,c[s>>2]|0);x=y}c[s>>2]=0;c[x>>2]=0;y=HA(v)|0;v=y-72-(d[h]<<4)>>3;y=c[m>>2]|0;z=y+16|0;if(z>>>0<32){c[m>>2]=z;c[n>>2]=c[n>>2]<<16|v&r;break}else{Hw(l,c[n>>2]|0,y);c[m>>2]=16;c[n>>2]=v&r;break}}}while(0);if((c[o>>2]|0)==0){Hy(l,c[m>>2]|0,0)}else{Hw(l,c[n>>2]|0,c[m>>2]|0)}c[m>>2]=0;c[n>>2]=0;A=h}else{A=b+22|0}h=a[A]|0;do{if(h<<24>>24!=0){if((c[b+28>>2]|0)!=(h&255|0)){break}n=j+8|0;Ht(n,c[e+28>>2]|0,c[e+32>>2]|0,0);m=j+4|0;c[m>>2]=0;l=j|0;c[l>>2]=0;o=j+40|0;c[o>>2]=1;do{if((a[k]|0)==0){r=b+36|0;Np(r);x=Nq(r,j,0)|0;r=(c[o>>2]|0)==0;q=c[m>>2]|0;if(q>>>0>30){if(!r){B=q;C=x;w=11032;break}c[m>>2]=q-30;D=x;break}else{if(!r){B=q;C=x;w=11032;break}Hy(n,q,0);E=x;w=11033;break}}else{B=0;C=0;w=11032}}while(0);if((w|0)==11032){Hw(n,c[l>>2]|0,B);E=C;w=11033}if((w|0)==11033){c[m>>2]=0;c[l>>2]=0;Hz(n,30,c[o>>2]&255);D=E}s=e+8|0;if((c[e+40>>2]|0)==0){Hy(s,c[e+4>>2]|0,0);F=e|0}else{t=e|0;Hw(s,c[t>>2]|0,c[e+4>>2]|0);F=t}c[e+4>>2]=0;c[F>>2]=0;t=(HA(s)|0)>>>3;s=c[497867]|0;u=c[m>>2]|0;x=u+13|0;if(x>>>0<32){c[m>>2]=x;G=c[l>>2]<<13|s&t;H=x}else{Hw(n,c[l>>2]|0,u);c[m>>2]=13;G=s&t;H=13}c[l>>2]=G;do{if((a[k]|0)==0){t=d[A]<<4|13;s=(c[o>>2]|0)==0;do{if(H>>>0>t>>>0){if(!s){w=11046;break}c[m>>2]=H-t}else{if(!s){w=11046;break}Hy(n,H,0);w=11047}}while(0);if((w|0)==11046){Hw(n,G,H);w=11047}if((w|0)==11047){c[m>>2]=0;c[l>>2]=0;Hz(n,t,c[o>>2]&255)}s=b+36|0;Nr(s,j,D)|0;u=(Ns(s)|0)&65535;s=c[497870]|0;x=c[m>>2]|0;q=x+16|0;if(q>>>0<32){c[m>>2]=q;r=c[l>>2]<<16|s&u;c[l>>2]=r;I=q;J=r;break}else{Hw(n,c[l>>2]|0,x);c[m>>2]=16;x=s&u;c[l>>2]=x;I=16;J=x;break}}else{I=H;J=G}}while(0);if((c[o>>2]|0)==0){Hy(n,I,0)}else{Hw(n,J,I)}c[m>>2]=0;c[l>>2]=0}}while(0);I=(c[f>>2]|0)-(c[b+32>>2]|0)|0;c[f>>2]=I;if((a[k]|0)!=0){K=b+28|0;L=c[K>>2]|0;M=L+1|0;c[K>>2]=M;i=g;return}if((a[A]|0)==0){K=b+28|0;L=c[K>>2]|0;M=L+1|0;c[K>>2]=M;i=g;return}c[f>>2]=I+16;K=b+28|0;L=c[K>>2]|0;M=L+1|0;c[K>>2]=M;i=g;return}function Lx(a){a=a|0;return((a-1|0)>>>0<7?a:0)|0}function Ly(b,e,f,g,h,i,j,k){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0;l=0;while(1){if(l>>>0>=16){m=-1;n=11159;break}if((c[1922056+(l<<2)>>2]|0)==(f|0)){break}else{l=l+1|0}}if((n|0)==11159){return m|0}if((l|0)==15){m=-1;return m|0}do{if((e|0)==1){o=1926772}else{if((e|0)==2){o=1926812;break}if((e|0)==3){o=1926852;break}if((e|0)==4){o=1926892;break}if((e|0)==5){o=1926932;break}if((e|0)==6){o=1926972;break}if((e|0)==7){o=1927012;break}if((e|0)==16){o=1927052;break}if((e|0)==17){o=1927092;break}if((e|0)==18){o=1927132;break}if((e|0)==19){o=1927172;break}if((e|0)==21){o=1927212;break}if((e|0)==22){o=1927252;break}if((e|0)==23){o=1927292;break}f=(e|0)==30;if(f){o=f?1927332:0;break}else{m=-1}return m|0}}while(0);f=o+4|0;p=c[497858]|0;q=b+4|0;r=c[q>>2]|0;s=r+4|0;do{if(s>>>0<32){c[q>>2]=s;t=b|0;u=c[t>>2]<<4|p&g;c[t>>2]=u;t=c[497856]|0;v=r+6|0;if(v>>>0<32){w=u;x=t;y=v;n=11086;break}Hw(b+8|0,u,s);c[q>>2]=2;u=t&h;c[b>>2]=u;z=t;A=u;B=6;n=11088}else{u=b|0;Hw(b+8|0,c[u>>2]|0,r);c[q>>2]=4;t=p&g;c[u>>2]=t;w=t;x=c[497856]|0;y=6;n=11086}}while(0);do{if((n|0)==11086){c[q>>2]=y;g=w<<2|x&h;c[b>>2]=g;r=y+4|0;if(r>>>0<32){z=x;A=g;B=r;n=11088;break}Hw(b+8|0,g,y);c[q>>2]=4;g=p&l;c[b>>2]=g;r=o|0;C=x;D=g;E=r;F=d[r]|0;G=8;n=11090}}while(0);do{if((n|0)==11088){c[q>>2]=B;x=A<<4|p&l;c[b>>2]=x;y=o|0;h=d[y]|0;w=B+4|0;if(w>>>0<32){C=z;D=x;E=y;F=h;G=w;n=11090;break}Hw(b+8|0,x,B);c[q>>2]=4;x=p&h;c[b>>2]=x;h=o+1|0;H=y;I=z;J=x;K=h;L=d[h]|0;M=8;n=11092}}while(0);do{if((n|0)==11090){c[q>>2]=G;z=D<<4|p&F;c[b>>2]=z;B=o+1|0;l=d[B]|0;A=G+4|0;if(A>>>0<32){H=E;I=C;J=z;K=B;L=l;M=A;n=11092;break}Hw(b+8|0,z,G);c[q>>2]=4;z=p&l;c[b>>2]=z;l=o+2|0;N=B;O=C;P=E;Q=z;R=l;S=d[l]|0;T=8;n=11094}}while(0);do{if((n|0)==11092){c[q>>2]=M;E=J<<4|p&L;c[b>>2]=E;C=o+2|0;G=d[C]|0;F=M+4|0;if(F>>>0<32){N=K;O=I;P=H;Q=E;R=C;S=G;T=F;n=11094;break}Hw(b+8|0,E,M);c[q>>2]=4;E=p&G;c[b>>2]=E;G=o+3|0;U=C;V=H;W=I;X=K;Y=E;Z=G;_=d[G]|0;$=6;n=11096}}while(0);do{if((n|0)==11094){c[q>>2]=T;K=Q<<4|p&S;c[b>>2]=K;I=o+3|0;H=d[I]|0;M=T+2|0;if(M>>>0<32){U=R;V=P;W=O;X=N;Y=K;Z=I;_=H;$=M;n=11096;break}Hw(b+8|0,K,T);K=O&H;c[q>>2]=5;c[b>>2]=K<<3;aa=R;ab=P;ac=O;ad=N;ae=I;af=K<<7;ag=9;n=11100}}while(0);do{if((n|0)==11096){c[q>>2]=$;N=Y<<2|W&_;c[b>>2]=N;O=$+3|0;if(O>>>0>=32){Hw(b+8|0,N,$);c[q>>2]=3;c[b>>2]=0;aa=U;ab=V;ac=W;ad=X;ae=Z;af=0;ag=7;n=11100;break}c[q>>2]=O;P=N<<3;c[b>>2]=P;R=$+7|0;if(R>>>0>=32){Hw(b+8|0,P,O);c[q>>2]=5;c[b>>2]=0;ah=U;ai=V;aj=W;ak=X;al=Z;am=0;an=6;n=11104;break}c[q>>2]=R;O=N<<7;c[b>>2]=O;N=$+8|0;if(N>>>0<32){ao=Z;ap=X;aq=W;ar=V;as=U;at=O;au=N;n=11103;break}Hw(b+8|0,O,R);c[q>>2]=1;c[b>>2]=0;ah=U;ai=V;aj=W;ak=X;al=Z;am=0;an=2;n=11104}}while(0);if((n|0)==11100){c[q>>2]=ag;c[b>>2]=af;ao=ae;ap=ad;aq=ac;ar=ab;as=aa;at=af;au=ag+1|0;n=11103}do{if((n|0)==11103){c[q>>2]=au;ag=at<<1;c[b>>2]=ag;af=au+1|0;if(af>>>0<32){ah=as;ai=ar;aj=aq;ak=ap;al=ao;am=ag;an=af;n=11104;break}Hw(b+8|0,ag,au);c[q>>2]=1;c[b>>2]=0;av=1;aw=0;ax=as;ay=ar;az=aq;aA=ap;aB=ao}}while(0);if((n|0)==11104){c[q>>2]=an;ao=am<<1;c[b>>2]=ao;av=an;aw=ao;ax=ah;ay=ai;az=aj;aA=ak;aB=al}L15444:do{if((i|0)==0){n=11115}else{if((e-5|0)>>>0>=2){n=11115;break}al=c[497855]|0;ak=av+1|0;do{if(ak>>>0<32){c[q>>2]=ak;aj=aw<<1|al&1;c[b>>2]=aj;ai=i+3&3;ah=av+3|0;if(ah>>>0<32){aC=aj;aD=ai;aE=ah;n=11112;break}Hw(b+8|0,aj,ak);c[q>>2]=2;aj=az&ai;c[b>>2]=aj;aF=aj;aG=j&1;aH=3}else{Hw(b+8|0,aw,av);c[q>>2]=1;aj=al&1;c[b>>2]=aj;aC=aj;aD=i+3&3;aE=3;n=11112}}while(0);do{if((n|0)==11112){c[q>>2]=aE;ak=aC<<2|az&aD;c[b>>2]=ak;aj=j&1;ai=aE+1|0;if(ai>>>0<32){aF=ak;aG=aj;aH=ai;break}Hw(b+8|0,ak,aE);c[q>>2]=1;ak=al&aj;c[b>>2]=ak;aI=1;aJ=ak;break L15444}}while(0);c[q>>2]=aH;ak=aF<<1|al&aG;c[b>>2]=ak;aI=aH;aJ=ak}}while(0);do{if((n|0)==11115){aH=av+1|0;if(aH>>>0<32){c[q>>2]=aH;aG=aw<<1;c[b>>2]=aG;aI=aH;aJ=aG;break}else{Hw(b+8|0,aw,av);c[q>>2]=1;c[b>>2]=0;aI=1;aJ=0;break}}}while(0);if((a[ay]|0)==0){aK=0;aL=0;aM=f;aN=aI;aO=aJ}else{av=c[497855]|0;aw=b|0;aG=b+8|0;aH=0;aF=0;aE=f;f=0;j=aI;aI=aJ;while(1){aJ=aE+4|0;aD=(c[aE>>2]|0)==1;az=aD&1;aC=(az^1)+aH&255;i=az+aF&255;az=aD?aF:aH;e=aD&1;aD=j+1|0;do{if(aD>>>0<32){c[q>>2]=aD;ak=aI<<1|e&av;c[aw>>2]=ak;aj=az&255;ai=j+5|0;if(ai>>>0<32){aP=ak;aQ=aj;aR=ai;n=11125;break}Hw(aG,ak,aD);c[q>>2]=4;aS=4;aT=p&aj}else{Hw(aG,aI,j);c[q>>2]=1;aj=e&av;c[aw>>2]=aj;aP=aj;aQ=az&255;aR=5;n=11125}}while(0);if((n|0)==11125){n=0;c[q>>2]=aR;aS=aR;aT=aP<<4|p&aQ}c[aw>>2]=aT;az=f+1|0;if((az|0)<(d[ay]|0)){aH=aC;aF=i;aE=aJ;f=az;j=aS;aI=aT}else{aK=aC;aL=i;aM=aJ;aN=aS;aO=aT;break}}}if((a[aA]|0)==0){aU=aK;aV=aL;aW=aM;aX=aN;aY=aO}else{aT=c[497855]|0;aS=b|0;aI=b+8|0;j=aK;aK=aL;aL=aM;aM=0;f=aN;aN=aO;while(1){aO=aL+4|0;aE=(c[aL>>2]|0)==1;aF=aE&1;aH=(aF^1)+j&255;ay=aF+aK&255;aF=aE?aK:j;aw=aE&1;aE=f+1|0;do{if(aE>>>0<32){c[q>>2]=aE;aQ=aN<<1|aw&aT;c[aS>>2]=aQ;aP=aF&255;aR=f+5|0;if(aR>>>0<32){aZ=aQ;a_=aP;a$=aR;n=11133;break}Hw(aI,aQ,aE);c[q>>2]=4;a0=4;a1=p&aP}else{Hw(aI,aN,f);c[q>>2]=1;aP=aw&aT;c[aS>>2]=aP;aZ=aP;a_=aF&255;a$=5;n=11133}}while(0);if((n|0)==11133){n=0;c[q>>2]=a$;a0=a$;a1=aZ<<4|p&a_}c[aS>>2]=a1;aF=aM+1|0;if((aF|0)<(d[aA]|0)){j=aH;aK=ay;aL=aO;aM=aF;f=a0;aN=a1}else{aU=aH;aV=ay;aW=aO;aX=a0;aY=a1;break}}}if((a[ax]|0)==0){a2=aX;a3=aY}else{a1=c[497855]|0;a0=b|0;aN=b+8|0;f=aU;aU=aV;aV=aW;aW=0;aM=aX;aX=aY;while(1){aY=aV+4|0;aL=(c[aV>>2]|0)==1;aK=aL&1;j=(aK^1)+f&255;aA=aK+aU&255;aK=aL?aU:f;aS=aL&1;aL=aM+1|0;do{if(aL>>>0<32){c[q>>2]=aL;a_=aX<<1|aS&a1;c[a0>>2]=a_;aZ=aK&255;a$=aM+5|0;if(a$>>>0<32){a4=a_;a5=aZ;a6=a$;n=11142;break}Hw(aN,a_,aL);c[q>>2]=4;a7=4;a8=p&aZ}else{Hw(aN,aX,aM);c[q>>2]=1;aZ=aS&a1;c[a0>>2]=aZ;a4=aZ;a5=aK&255;a6=5;n=11142}}while(0);if((n|0)==11142){n=0;c[q>>2]=a6;a7=a6;a8=a4<<4|p&a5}c[a0>>2]=a8;aK=aW+1|0;if((aK|0)<(d[ax]|0)){f=j;aU=aA;aV=aY;aW=aK;aM=a7;aX=a8}else{a2=a7;a3=a8;break}}}if((a[aB]|0)==0){a9=a2;ba=a3;bb=b+8|0}else{a8=b|0;a7=b+8|0;aX=0;aM=0;aW=a2;a2=a3;while(1){a3=aX&255;aV=aW+4|0;if(aV>>>0<32){c[q>>2]=aV;bc=aV;bd=a2<<4|p&a3}else{Hw(a7,a2,aW);c[q>>2]=4;bc=4;bd=p&a3}c[a8>>2]=bd;a3=aM+1|0;if((a3|0)<(d[aB]|0)){aX=aX+1&255;aM=a3;aW=bc;a2=bd}else{a9=bc;ba=bd;bb=a7;break}}}a7=b+40|0;if((c[a7>>2]|0)==0){Hy(bb,a9,0)}else{Hw(bb,ba,a9)}c[q>>2]=0;c[b>>2]=0;a9=(c[a7>>2]|0)==0;ba=HA(bb)|0;if(a9){Hz(bb,ba-k&7,c[a7>>2]&255)}else{Hw(bb,0,k-ba&7)}ba=c[q>>2]|0;k=ba+8|0;if(k>>>0<32){c[q>>2]=k;k=b|0;c[k>>2]=c[k>>2]<<8;m=0;return m|0}else{k=b|0;Hw(bb,c[k>>2]|0,ba);c[q>>2]=8;c[k>>2]=0;m=0;return m|0}return 0}function Lz(a,b,c){a=a|0;b=b|0;c=c|0;var e=0,f=0,g=0,h=0,i=0;do{if((a|0)==1){e=1926772}else{if((a|0)==2){e=1926812;break}if((a|0)==3){e=1926852;break}if((a|0)==4){e=1926892;break}if((a|0)==5){e=1926932;break}if((a|0)==6){e=1926972;break}if((a|0)==7){e=1927012;break}if((a|0)==16){e=1927052;break}if((a|0)==17){e=1927092;break}if((a|0)==18){e=1927132;break}if((a|0)==19){e=1927172;break}if((a|0)==21){e=1927212;break}if((a|0)==22){e=1927252;break}if((a|0)==23){e=1927292;break}f=(a|0)==30;if(f){e=f?1927332:0;break}else{g=-1}return g|0}}while(0);f=c+34|0;if((b|0)==0){h=f}else{h=(a-5|0)>>>0<2?c+37|0:f}f=((d[e+3|0]|0)<<2)+h+(((d[e+1|0]|0)+(d[e|0]|0)+(d[e+2|0]|0)|0)*5|0)|0;e=(f|0)%8|0;if((e|0)==0){i=f}else{i=8-e+f|0}g=i+8|0;return g|0}function LA(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0;e=a+40|0;f=a+8|0;if((c[e>>2]|0)==0){Hy(f,c[a+4>>2]|0,0);g=a|0}else{h=a|0;Hw(f,c[h>>2]|0,c[a+4>>2]|0);g=h}h=a+4|0;c[h>>2]=0;c[g>>2]=0;g=HA(f)|0;i=b|0;j=c[i>>2]|0;switch(j|0){case 17:case 19:case 20:case 21:case 22:case 23:case 39:case 42:{k=1;break};default:{k=0}}l=b+4|0;m=c[l>>2]|0;L15553:do{if((m|0)==5|(m|0)==29){n=c[497859]|0;o=c[h>>2]|0;p=o+5|0;if(p>>>0<32){c[h>>2]=p;q=a|0;r=c[q>>2]<<5|n&m;c[q>>2]=r;s=p;t=r;u=n;break}else{r=a|0;Hw(f,c[r>>2]|0,o);c[h>>2]=5;o=n&m;c[r>>2]=o;s=5;t=o;u=n;break}}else{n=c[497859]|0;o=c[h>>2]|0;r=o+5|0;p=r>>>0<32;if((j|0)<=31){if(p){c[h>>2]=r;q=a|0;v=c[q>>2]<<5|n&j;c[q>>2]=v;s=r;t=v;u=n;break}else{v=a|0;Hw(f,c[v>>2]|0,o);c[h>>2]=5;q=n&j;c[v>>2]=q;s=5;t=q;u=n;break}}do{if(p){c[h>>2]=r;q=a|0;v=c[q>>2]<<5|n&31;c[q>>2]=v;q=j-32|0;w=c[497860]|0;x=o+11|0;if(x>>>0<32){y=v;z=q;A=w;B=x;break}Hw(f,v,r);c[h>>2]=6;v=w&q;c[a>>2]=v;s=6;t=v;u=n;break L15553}else{v=a|0;Hw(f,c[v>>2]|0,o);c[h>>2]=5;q=n&31;c[v>>2]=q;y=q;z=j-32|0;A=c[497860]|0;B=11}}while(0);c[h>>2]=B;o=y<<6|A&z;c[a>>2]=o;s=B;t=o;u=n}}while(0);B=b+12|0;z=c[B>>2]|0;A=0;while(1){if(A>>>0>=16){break}if((c[1922056+(A<<2)>>2]|0)==(z|0)){break}else{A=A+1|0}}y=A>>>0>15?15:A;A=c[497858]|0;j=s+4|0;if(j>>>0<32){c[h>>2]=j;m=t<<4|A&y;c[a>>2]=m;C=j;D=m}else{Hw(f,t,s);c[h>>2]=4;s=A&y;c[a>>2]=s;C=4;D=s}do{if((y|0)==15){s=c[497878]|0;t=C+24|0;if(t>>>0<32){c[h>>2]=t;c[a>>2]=D<<24|s&z;break}else{Hw(f,D,C);c[h>>2]=24;c[a>>2]=s&z;break}}}while(0);z=b+8|0;C=c[z>>2]|0;do{if((C|0)==-1){D=c[b+28>>2]|0;if((D-1|0)>>>0>=8){c[z>>2]=0;E=0;break}y=(D|0)==8?7:D;c[z>>2]=y;if((y|0)==-1){F=-1}else{E=y;break}return F|0}else{E=C}}while(0);C=(E-1|0)>>>0<7?E:0;E=c[h>>2]|0;y=E+4|0;if(y>>>0<32){c[h>>2]=y;D=a|0;s=c[D>>2]<<4|C&A;c[D>>2]=s;G=y;H=s}else{s=a|0;Hw(f,c[s>>2]|0,E);c[h>>2]=4;E=C&A;c[s>>2]=E;G=4;H=E}E=c[l>>2]|0;L15596:do{if((E|0)==5|(E|0)==29){l=c[b+16>>2]|0;s=0;while(1){if(s>>>0>=16){break}if((c[1922056+(s<<2)>>2]|0)==(l|0)){break}else{s=s+1|0}}n=s>>>0>15?15:s;C=G+4|0;if(C>>>0<32){c[h>>2]=C;y=H<<4|A&n;c[a>>2]=y;I=C;J=y}else{Hw(f,H,G);c[h>>2]=4;y=A&n;c[a>>2]=y;I=4;J=y}do{if((n|0)==15){y=c[497878]|0;C=I+24|0;if(C>>>0<32){c[h>>2]=C;D=J<<24|y&l;c[a>>2]=D;K=C;L=D;break}else{Hw(f,J,I);c[h>>2]=24;D=y&l;c[a>>2]=D;K=24;L=D;break}}else{K=I;L=J}}while(0);l=c[i>>2]|0;n=K+5|0;s=n>>>0<32;if((l|0)<=31){if(s){c[h>>2]=n;D=L<<5|u&l;c[a>>2]=D;M=n;N=D;break}else{Hw(f,L,K);c[h>>2]=5;D=u&l;c[a>>2]=D;M=5;N=D;break}}do{if(s){c[h>>2]=n;D=L<<5|u&31;c[a>>2]=D;y=l-32|0;C=c[497860]|0;t=K+11|0;if(t>>>0<32){O=D;P=y;Q=C;R=t;break}Hw(f,D,n);c[h>>2]=6;D=C&y;c[a>>2]=D;M=6;N=D;break L15596}else{Hw(f,L,K);c[h>>2]=5;D=u&31;c[a>>2]=D;O=D;P=l-32|0;Q=c[497860]|0;R=11}}while(0);c[h>>2]=R;l=O<<6|Q&P;c[a>>2]=l;M=R;N=l}else{M=G;N=H}}while(0);H=c[i>>2]|0;L15624:do{switch(H|0){case 1:case 2:case 3:case 4:case 6:case 7:case 17:case 19:case 20:case 21:case 22:case 23:{G=c[b+24>>2]|0;if((G|0)==960){S=1}else{S=(G|0)==480|0}G=c[497855]|0;R=M+1|0;do{if(R>>>0<32){c[h>>2]=R;P=a|0;Q=N<<1|G&S;c[P>>2]=Q;O=M+2|0;if(O>>>0>=32){Hw(f,Q,R);c[h>>2]=1;c[P>>2]=0;T=0;U=2;V=11250;break}c[h>>2]=O;K=Q<<1;c[P>>2]=K;Q=M+3|0;if(Q>>>0<32){T=K;U=Q;V=11250;break}Hw(f,K,O);c[h>>2]=1;c[P>>2]=G&k}else{Hw(f,N,M);c[h>>2]=2;P=(G&S)<<1;c[a>>2]=P;T=P;U=3;V=11250}}while(0);if((V|0)==11250){c[h>>2]=U;c[a>>2]=T<<1|G&k}R=c[z>>2]|0;if((R|0)==0|(R-1|0)>>>0>6){P=c[B>>2]|0;Ly(a,R,P,0,1,0,0,g)|0}if((k|0)==0){break L15624}L15643:do{switch(H|0){case 22:{P=c[b+40>>2]|0;R=c[h>>2]|0;O=R+5|0;do{if(O>>>0<32){c[h>>2]=O;K=a|0;Q=c[K>>2]<<5|u&P;c[K>>2]=Q;L=c[b+44>>2]|0;J=c[497865]|0;I=R+16|0;if(I>>>0<32){W=Q;X=L;Y=J;Z=I;break}Hw(f,Q,O);c[h>>2]=11;Q=J&L;c[K>>2]=Q;_=12;$=Q;break L15643}else{Q=a|0;Hw(f,c[Q>>2]|0,R);c[h>>2]=5;K=u&P;c[Q>>2]=K;W=K;X=c[b+44>>2]|0;Y=c[497865]|0;Z=16}}while(0);c[h>>2]=Z;c[a>>2]=W<<11|X&Y;aa=Z;V=11269;break};case 23:case 20:case 19:case 17:{P=b+48|0;R=(c[P>>2]|0)>>>25&1;O=c[h>>2]|0;K=O+1|0;do{if(K>>>0<32){c[h>>2]=K;Q=a|0;L=c[Q>>2]<<1|R&G;c[Q>>2]=L;J=(c[P>>2]|0)>>>24&1;I=O+2|0;if(I>>>0<32){ab=L;ac=J;ad=I;V=11266;break}Hw(f,L,K);c[h>>2]=1;L=J&G;c[Q>>2]=L;ae=L;af=(c[P>>2]|0)>>>26&1;ag=2}else{L=a|0;Hw(f,c[L>>2]|0,O);c[h>>2]=1;Q=R&G;c[L>>2]=Q;ab=Q;ac=(c[P>>2]|0)>>>24&1;ad=2;V=11266}}while(0);do{if((V|0)==11266){c[h>>2]=ad;R=a|0;O=ab<<1|ac&G;c[R>>2]=O;K=(c[P>>2]|0)>>>26&1;Q=ad+1|0;if(Q>>>0<32){ae=O;af=K;ag=Q;break}Hw(f,O,ad);c[h>>2]=1;O=K&G;c[R>>2]=O;_=2;$=O;break L15643}}while(0);c[h>>2]=ag;c[a>>2]=ae<<1|af&G;aa=ag;V=11269;break};default:{aa=c[h>>2]|0;V=11269}}}while(0);do{if((V|0)==11269){G=aa+1|0;P=a|0;O=c[P>>2]|0;if(G>>>0<32){_=G;$=O;break}Hw(f,O,aa);c[h>>2]=1;c[P>>2]=0;break L15624}}while(0);c[h>>2]=_;c[a>>2]=$<<1;break};case 39:{if((c[z>>2]|0)==16){F=-1;return F|0}P=(c[b+24>>2]|0)==480|0;O=c[497855]|0;G=M+1|0;do{if(G>>>0<32){c[h>>2]=G;R=a|0;K=N<<1|P&O;c[R>>2]=K;Q=b+48|0;L=(c[Q>>2]|0)>>>25&1;J=M+2|0;if(J>>>0<32){ah=K;ai=Q;aj=L;ak=J;V=11277;break}Hw(f,K,G);c[h>>2]=1;K=L&O;c[R>>2]=K;al=Q;am=K;an=(c[Q>>2]|0)>>>24&1;ao=2;V=11279}else{Hw(f,N,M);c[h>>2]=1;Q=P&O;c[a>>2]=Q;K=b+48|0;ah=Q;ai=K;aj=(c[K>>2]|0)>>>25&1;ak=2;V=11277}}while(0);do{if((V|0)==11277){c[h>>2]=ak;P=a|0;G=ah<<1|aj&O;c[P>>2]=G;K=(c[ai>>2]|0)>>>24&1;Q=ak+1|0;if(Q>>>0<32){al=ai;am=G;an=K;ao=Q;V=11279;break}Hw(f,G,ak);c[h>>2]=1;G=K&O;c[P>>2]=G;ap=ai;aq=G;ar=(c[ai>>2]|0)>>>26&1;as=2;V=11281}}while(0);do{if((V|0)==11279){c[h>>2]=ao;G=a|0;P=am<<1|an&O;c[G>>2]=P;K=(c[al>>2]|0)>>>26&1;Q=ao+1|0;if(Q>>>0<32){ap=al;aq=P;ar=K;as=Q;V=11281;break}Hw(f,P,ao);c[h>>2]=1;P=K&O;c[G>>2]=P;at=al;au=P;av=(c[al>>2]|0)>>>23&1;aw=2;V=11282}}while(0);do{if((V|0)==11281){c[h>>2]=as;P=a|0;G=aq<<1|ar&O;c[P>>2]=G;K=(c[ap>>2]|0)>>>23&1;Q=as+1|0;if(Q>>>0<32){at=ap;au=G;av=K;aw=Q;V=11282;break}Hw(f,G,as);c[h>>2]=1;G=K&O;c[P>>2]=G;ax=1;ay=G;az=ap}}while(0);if((V|0)==11282){c[h>>2]=aw;G=au<<1|av&O;c[a>>2]=G;ax=aw;ay=G;az=at}do{if((c[az>>2]&8388608|0)==0){aA=ax}else{G=(c[B>>2]|0)!=(c[b+16>>2]|0)|0;P=ax+1|0;do{if(P>>>0<32){c[h>>2]=P;K=a|0;Q=G&O|ay<<1;c[K>>2]=Q;R=(c[az>>2]|0)>>>16&1;L=ax+2|0;if(L>>>0<32){aB=Q;aC=R;aD=L;V=11288;break}Hw(f,Q,P);c[h>>2]=1;c[K>>2]=R&O;aE=1}else{Hw(f,ay,ax);c[h>>2]=1;R=G&O;c[a>>2]=R;aB=R;aC=(c[az>>2]|0)>>>16&1;aD=2;V=11288}}while(0);if((V|0)==11288){c[h>>2]=aD;c[a>>2]=aB<<1|aC&O;aE=aD}G=d+16|0;if((c[G>>2]|0)==0){aA=aE;break}P=c[z>>2]|0;do{if((P|0)==1){aF=1926772}else{if((P|0)==2){aF=1926812;break}if((P|0)==3){aF=1926852;break}if((P|0)==4){aF=1926892;break}if((P|0)==5){aF=1926932;break}if((P|0)==6){aF=1926972;break}if((P|0)==7){aF=1927012;break}if((P|0)==16){aF=1927052;break}if((P|0)==17){aF=1927092;break}if((P|0)==18){aF=1927132;break}if((P|0)==19){aF=1927172;break}if((P|0)==21){aF=1927212;break}if((P|0)==22){aF=1927252;break}if((P|0)==23){aF=1927292;break}aF=(P|0)==30?1927332:0}}while(0);P=d+20|0;R=0;do{K=c[aF+4+(R<<2)>>2]|0;if((K|0)==0|(K|0)==1){Q=c[G>>2]|0;L=c[P>>2]|0;J=c[i>>2]|0;dp[Q&7](L,a,0,0,0,J,K,R)|0}else if((K|0)==(-1|0)){break}R=R+1|0;}while((R|0)<8);aA=c[h>>2]|0}}while(0);O=aA+4|0;if(O>>>0<32){c[h>>2]=O;O=a|0;c[O>>2]=c[O>>2]<<4;break L15624}else{O=a|0;Hw(f,c[O>>2]|0,aA);c[h>>2]=4;c[O>>2]=0;break L15624}break};default:{F=-1;return F|0}}}while(0);L15722:do{switch(c[i>>2]|0){case 17:case 19:case 20:case 21:case 22:case 23:case 24:case 25:case 26:case 27:case 39:{aA=c[h>>2]|0;aF=aA+2|0;if(aF>>>0<32){c[h>>2]=aF;aF=a|0;c[aF>>2]=c[aF>>2]<<2;break L15722}else{aF=a|0;Hw(f,c[aF>>2]|0,aA);c[h>>2]=2;c[aF>>2]=0;break L15722}break};default:{}}}while(0);if((c[e>>2]|0)==0){Hy(f,c[h>>2]|0,0);aG=a|0}else{e=a|0;Hw(f,c[e>>2]|0,c[h>>2]|0);aG=e}c[h>>2]=0;c[aG>>2]=0;F=0;return F|0}function LB(e,f,g,h){e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0;i=e+63|0;a[i]=0;j=e+78|0;a[j]=0;k=e+62|0;l=d[k]|0;m=c[497855]|0;n=f+4|0;o=c[n>>2]|0;p=o+1|0;if(p>>>0<32){c[n>>2]=p;p=f|0;c[p>>2]=c[p>>2]<<1|m&l}else{p=f|0;Hw(f+8|0,c[p>>2]|0,o);c[n>>2]=1;c[p>>2]=m&l}a[j]=(a[j]|0)+1&255;if((a[k]|0)==1){l=d[i]|0;p=c[n>>2]|0;o=p+1|0;if(o>>>0<32){c[n>>2]=o;o=f|0;c[o>>2]=c[o>>2]<<1|m&l}else{o=f|0;Hw(f+8|0,c[o>>2]|0,p);c[n>>2]=1;c[o>>2]=m&l}a[j]=(a[j]|0)+1&255}if((a[i]|0)!=0){q=0;return q|0}if((a[k]|0)==1){i=c[e+48>>2]|0;do{if((i|0)<256){r=1}else{if((i|0)<65536){r=2;break}r=(i|0)<16777216?3:4}}while(0);l=r-1|0;o=c[497856]|0;p=c[n>>2]|0;s=p+2|0;if(s>>>0<32){c[n>>2]=s;t=f|0;u=c[t>>2]<<2|o&l;c[t>>2]=u;v=s;w=u;x=f+8|0}else{u=f+8|0;s=f|0;Hw(u,c[s>>2]|0,p);c[n>>2]=2;p=o&l;c[s>>2]=p;v=2;w=p;x=u}u=c[497862]|0;p=f|0;s=0;o=v;v=w;while(1){w=i>>(l-s<<3)&255;t=o+8|0;if(t>>>0<32){c[n>>2]=t;y=t;z=v<<8|w&u}else{Hw(x,v,o);c[n>>2]=8;y=8;z=w&u}c[p>>2]=z;w=s+1|0;if((w|0)<(r|0)){s=w;o=y;v=z}else{break}}a[j]=(d[j]|0)+(r<<3|2)&255}r=e+67|0;z=(a[r]|0)!=0|0;v=c[n>>2]|0;y=v+1|0;if(y>>>0<32){c[n>>2]=y;o=f|0;s=c[o>>2]<<1|z&m;c[o>>2]=s;A=y;B=s}else{s=f|0;Hw(f+8|0,c[s>>2]|0,v);c[n>>2]=1;v=z&m;c[s>>2]=v;A=1;B=v}v=(d[e+69|0]|0)-1|0;s=c[497860]|0;z=A+6|0;if(z>>>0<32){c[n>>2]=z;y=B<<6|v&s;c[f>>2]=y;C=z;D=y}else{Hw(f+8|0,B,A);c[n>>2]=6;A=v&s;c[f>>2]=A;C=6;D=A}A=e+64|0;v=(d[A]|0)-1|0;B=c[497858]|0;y=C+4|0;if(y>>>0<32){c[n>>2]=y;c[f>>2]=D<<4|v&B}else{Hw(f+8|0,D,C);c[n>>2]=4;c[f>>2]=v&B}a[j]=(a[j]|0)+11&255;L15778:do{if((a[A]|0)!=0){B=c[497857]|0;v=f|0;C=f+8|0;D=f+40|0;y=c[497856]|0;z=c[497862]|0;o=z&g;p=B&1;u=c[497863]|0;x=B&4;l=B&6;i=0;w=0;L15780:while(1){t=(d[e+65+i|0]|0)-1|0;E=c[n>>2]|0;F=E+3|0;if(F>>>0<32){c[n>>2]=F;G=c[v>>2]<<3|t&B}else{Hw(C,c[v>>2]|0,E);c[n>>2]=3;G=t&B}c[v>>2]=G;a[j]=(a[j]|0)+3&255;t=e+(i*28|0)|0;E=e+28+(i<<2)|0;F=c[E>>2]|0;H=e+(i*28|0)+24|0;c[H>>2]=-1;L15786:do{if((F|0)==0){I=w}else{L15788:do{if((a[k]|0)==1){J=c[n>>2]|0;K=(c[D>>2]|0)==0;do{if(J>>>0>2){if(!K){L=11370;break}c[n>>2]=J-2;break L15788}else{if(!K){L=11370;break}Hy(C,J,0)}}while(0);if((L|0)==11370){L=0;Hw(C,c[v>>2]|0,J)}c[n>>2]=0;c[v>>2]=0;Hz(C,2,c[D>>2]&255)}}while(0);if((c[D>>2]|0)==0){Hy(C,c[n>>2]|0,0)}else{Hw(C,c[v>>2]|0,c[n>>2]|0)}c[n>>2]=0;c[v>>2]=0;K=HA(C)|0;M=c[E>>2]|0;LA(f,M,h)|0;if((c[D>>2]|0)==0){Hy(C,c[n>>2]|0,0)}else{Hw(C,c[v>>2]|0,c[n>>2]|0)}c[n>>2]=0;c[v>>2]=0;M=(HA(C)|0)-K|0;if((a[k]|0)==1){K=M+2|0;N=c[n>>2]|0;O=N+K|0;P=(c[D>>2]|0)==0;do{if(O>>>0<32){if(!P){L=11384;break}c[n>>2]=O;Hy(C,O,0);c[n>>2]=0;c[v>>2]=0}else{if(!P){L=11384;break}Hy(C,N,0);L=11385}}while(0);if((L|0)==11384){L=0;Hw(C,c[v>>2]|0,N);L=11385}if((L|0)==11385){L=0;c[n>>2]=0;c[v>>2]=0;Hy(C,K,c[D>>2]&255)}do{if((M|0)<256){Q=1}else{if((M|0)<65536){Q=2;break}Q=(M|0)<16777216?3:4}}while(0);K=Q-1|0;N=c[n>>2]|0;P=N+2|0;if(P>>>0<32){c[n>>2]=P;R=P;S=c[v>>2]<<2|y&K}else{Hw(C,c[v>>2]|0,N);c[n>>2]=2;R=2;S=y&K}c[v>>2]=S;N=0;P=R;O=S;while(1){T=M>>(K-N<<3)&255;U=P+8|0;if(U>>>0<32){c[n>>2]=U;V=U;W=O<<8|T&z}else{Hw(C,O,P);c[n>>2]=8;V=8;W=T&z}c[v>>2]=W;T=N+1|0;if((T|0)<(Q|0)){N=T;P=V;O=W}else{break}}a[j]=(d[j]|0)+(Q<<3|2)&255;LA(f,c[E>>2]|0,h)|0}a[j]=(d[j]|0)+M&255;if((a[r]|0)==0&(w|0)>15){q=6;L=11461;break L15780}O=w+1|0;c[H>>2]=w;P=F|0;switch(c[P>>2]|0){case 8:{c[t>>2]=4;N=c[n>>2]|0;K=N+3|0;if(K>>>0<32){c[n>>2]=K;X=c[v>>2]<<3|x}else{Hw(C,c[v>>2]|0,N);c[n>>2]=3;X=x}c[v>>2]=X;a[j]=(a[j]|0)+3&255;N=F+32|0;K=0;while(1){if((K|0)>=62){q=11;L=11463;break L15780}if((b[1918440+(K<<1)>>1]|0)==(c[N>>2]|0)){break}else{K=K+1|0}}if((K|0)>61){q=11;L=11459;break L15780}M=c[n>>2]|0;T=M+6|0;if(T>>>0<32){c[n>>2]=T;Y=c[v>>2]<<6|s&K}else{Hw(C,c[v>>2]|0,M);c[n>>2]=6;Y=s&K}c[v>>2]=Y;a[j]=(a[j]|0)+6&255;c[e+(i*28|0)+4>>2]=c[N>>2];I=O;break L15786;break};case 9:{c[t>>2]=6;M=c[n>>2]|0;T=M+3|0;if(T>>>0<32){c[n>>2]=T;Z=c[v>>2]<<3|l}else{Hw(C,c[v>>2]|0,M);c[n>>2]=3;Z=l}c[v>>2]=Z;a[j]=(a[j]|0)+3&255;M=F+32|0;T=c[M>>2]|0;if((T|0)==80){_=1}else if((T|0)==40){_=0}else{q=12;L=11462;break L15780}T=c[n>>2]|0;U=T+1|0;if(U>>>0<32){c[n>>2]=U;$=c[v>>2]<<1|m&_}else{Hw(C,c[v>>2]|0,T);c[n>>2]=1;$=m&_}c[v>>2]=$;a[j]=(a[j]|0)+1&255;c[e+(i*28|0)+4>>2]=c[M>>2];I=O;break L15786;break};case 1:case 2:case 3:case 4:case 6:case 23:case 39:case 42:case 50:{c[t>>2]=0;M=c[n>>2]|0;T=M+3|0;do{if(T>>>0<32){c[n>>2]=T;U=c[v>>2]<<3;c[v>>2]=U;aa=M+11|0;if(aa>>>0<32){ab=U;ac=aa;L=11403;break}Hw(C,U,T);c[n>>2]=8;ad=o}else{Hw(C,c[v>>2]|0,M);c[n>>2]=3;c[v>>2]=0;ab=0;ac=11;L=11403}}while(0);if((L|0)==11403){L=0;c[n>>2]=ac;ad=ab<<8|o}c[v>>2]=ad;a[j]=(a[j]|0)+11&255;if((a[r]|0)!=0){I=O;break L15786}M=c[P>>2]|0;if(!((M|0)==6|(M|0)==20)){I=O;break L15786}M=c[c[e+28+(i<<2)-4>>2]>>2]|0;if(!((M|0)==8|(M|0)==24)){I=O;break L15786}M=c[n>>2]|0;T=M+6|0;if(T>>>0<32){c[n>>2]=T;ae=c[v>>2]<<6}else{Hw(C,c[v>>2]|0,M);c[n>>2]=6;ae=0}c[v>>2]=ae;a[j]=(a[j]|0)+6&255;I=O;break L15786;break};case 7:{c[t>>2]=1;M=(c[F+32>>2]|0)+7>>3;T=M-20|0;if((T|0)<0){q=10;L=11464;break L15780}N=c[n>>2]|0;K=N+3|0;do{if(K>>>0<32){c[n>>2]=K;U=c[v>>2]<<3|p;c[v>>2]=U;aa=N+12|0;if(aa>>>0<32){af=U;ag=aa;L=11416;break}Hw(C,U,K);c[n>>2]=9;ah=u&T}else{Hw(C,c[v>>2]|0,N);c[n>>2]=3;c[v>>2]=p;af=p;ag=12;L=11416}}while(0);if((L|0)==11416){L=0;c[n>>2]=ag;ah=af<<9|u&T}c[v>>2]=ah;a[j]=(a[j]|0)+12&255;c[e+(i*28|0)+4>>2]=M<<3;I=O;break L15786;break};default:{q=13;L=11465;break L15780}}}}while(0);F=i+1|0;if((F|0)<(d[A]|0)){i=F;w=I}else{break L15778}}if((L|0)==11461){return q|0}else if((L|0)==11462){return q|0}else if((L|0)==11463){return q|0}else if((L|0)==11464){return q|0}else if((L|0)==11465){return q|0}else if((L|0)==11459){return q|0}}}while(0);I=e+56|0;e=(c[I>>2]|0)!=0|0;A=c[n>>2]|0;ah=A+1|0;if(ah>>>0<32){c[n>>2]=ah;ah=f|0;c[ah>>2]=c[ah>>2]<<1|e&m}else{ah=f|0;Hw(f+8|0,c[ah>>2]|0,A);c[n>>2]=1;c[ah>>2]=e&m}a[j]=(a[j]|0)+1&255;e=c[I>>2]|0;L15900:do{if((e|0)!=0){ah=e;A=0;do{ah=ah>>8;A=A+1|0;}while((ah|0)!=0);ah=f|0;af=c[497862]|0;ag=f+8|0;ae=A;r=e;while(1){ad=r>>>(ae<<3>>>0)&255;ab=ae-1|0;ac=(ab|0)>0;$=ac&1;_=c[n>>2]|0;Z=_+1|0;do{if(Z>>>0<32){c[n>>2]=Z;Y=c[ah>>2]<<1|$&m;c[ah>>2]=Y;s=_+9|0;if(s>>>0<32){ai=Y;aj=s;L=11449;break}Hw(ag,Y,Z);c[n>>2]=8;ak=af&ad}else{Hw(ag,c[ah>>2]|0,_);c[n>>2]=1;Y=$&m;c[ah>>2]=Y;ai=Y;aj=9;L=11449}}while(0);if((L|0)==11449){L=0;c[n>>2]=aj;ak=ai<<8|af&ad}c[ah>>2]=ak;a[j]=(a[j]|0)+9&255;if(!ac){break L15900}ae=ab;r=c[I>>2]|0}}}while(0);I=c[n>>2]|0;ak=I+1|0;if(ak>>>0<32){c[n>>2]=ak;ak=f|0;c[ak>>2]=c[ak>>2]<<1}else{ak=f|0;Hw(f+8|0,c[ak>>2]|0,I);c[n>>2]=1;c[ak>>2]=0}a[j]=(a[j]|0)+1&255;q=0;return q|0}function LC(b,e){b=b|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;f=c[b+36>>2]|0;if(!((f|0)==10|(f|0)==7|(f|0)==6)){g=0;return g|0}do{if((a[b+68|0]|0)==0){h=(f|0)==10?24:0;if((a[b+61|0]|0)==0){i=0}else{i=(a[b+60|0]|0)==0|0}do{if((f|0)==7){j=h}else{k=h|1;if((i|0)==0){j=k;break}j=(d[b+78|0]|0)+k|0}}while(0);h=(c[b+56>>2]<<3)+j|0;k=(h|0)%8|0;if((k|0)==0){a[b+77|0]=0;l=h;break}else{m=8-k|0;a[b+77|0]=m&255;l=(m&255)+h|0;break}}else{l=0}}while(0);L15937:do{if((a[b+67|0]|0)==0){if((c[b+32>>2]|0)!=0){n=0;break}j=b+52|0;c[j>>2]=0;i=a[b+64|0]|0;if(i<<24>>24==0){o=4}else{f=i&255;i=4;h=0;m=e;k=0;while(1){if((c[b+(h*28|0)+24>>2]|0)>-1){p=i+4|0;q=c[b+(h*28|0)>>2]|0;if((q|0)==0){r=m-p|0;s=p;while(1){t=s+8|0;if(r>>>0>2039){r=r-2040|0;s=t}else{u=r;v=t;break}}}else if((q|0)==1|(q|0)==4|(q|0)==6){u=m;v=p}else{n=0;break L15937}r=k+1|0;c[j>>2]=r;w=u;x=v;y=r}else{w=m;x=i;y=k}r=h+1|0;if((r|0)<(f|0)){i=x;h=r;m=w;k=y}else{o=x;break}}}n=o+4|0}else{k=a[b+64|0]|0;if(k<<24>>24==0){n=0;break}m=k&255;k=0;h=0;i=e;while(1){do{if((c[b+(h*28|0)+24>>2]|0)>-1){f=c[b+(h*28|0)>>2]|0;if((f|0)==1|(f|0)==4|(f|0)==6){z=i;A=k+2|0;break}else if((f|0)!=0){n=0;break L15937}if((i|0)==0){z=0;A=k;break}f=i-k|0;j=k;while(1){r=j+8|0;if(f>>>0>2039){f=f-2040|0;j=r}else{z=f;A=r;break}}}else{z=i;A=k}}while(0);p=h+1|0;if((p|0)<(m|0)){k=A;h=p;i=z}else{n=A;break}}}}while(0);g=n+l|0;return g|0}function LD(b,d,e,f,g){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;h=b+68|0;do{if((a[h]|0)==0){i=d+8|0;Hu(i);c[d+4>>2]=0;c[d>>2]=0;c[d+40>>2]=1;j=i;k=d+40|0;l=11504}else{i=d+8|0;m=d+40|0;if((c[d+40>>2]|0)!=0){j=i;k=m;l=11504;break}Hy(i,c[d+4>>2]|0,0);n=d|0;o=i;p=m}}while(0);if((l|0)==11504){l=d|0;Hw(j,c[l>>2]|0,c[d+4>>2]|0);n=l;o=j;p=k}k=d+4|0;c[k>>2]=0;c[n>>2]=0;c[b+72>>2]=HA(o)|0;if((c[b+36>>2]|0)!=10){q=LE(b,d,e,f,g)|0;return q|0}if((a[h]|0)!=0){q=LE(b,d,e,f,g)|0;return q|0}h=c[497865]|0;n=c[k>>2]|0;j=n+11|0;if(j>>>0<32){c[k>>2]=j;j=d|0;c[j>>2]=c[j>>2]<<11|h&695}else{j=d|0;Hw(o,c[j>>2]|0,n);c[k>>2]=11;c[j>>2]=h&695}h=b+40|0;c[h>>2]=0;if((c[p>>2]|0)==0){Hy(o,c[k>>2]|0,0);r=d|0}else{p=d|0;Hw(o,c[p>>2]|0,c[k>>2]|0);r=p}c[k>>2]=0;c[r>>2]=0;c[b+44>>2]=HA(o)|0;r=c[h>>2]|0;h=c[497867]|0;p=c[k>>2]|0;j=p+13|0;if(j>>>0<32){c[k>>2]=j;j=d|0;c[j>>2]=c[j>>2]<<13|h&r;q=LE(b,d,e,f,g)|0;return q|0}else{j=d|0;Hw(o,c[j>>2]|0,p);c[k>>2]=13;c[j>>2]=h&r;q=LE(b,d,e,f,g)|0;return q|0}return 0}function LE(b,e,f,g,h){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;do{if((a[b+68|0]|0)==0){if((a[b+61|0]|0)==0){i=0}else{i=(a[b+60|0]|0)==0|0}if((c[b+36>>2]|0)==7){break}if((i|0)==0){j=c[497855]|0;k=e+4|0;l=c[k>>2]|0;m=l+1|0;if(m>>>0<32){c[k>>2]=m;m=e|0;c[m>>2]=c[m>>2]<<1|j&1;break}else{m=e|0;Hw(e+8|0,c[m>>2]|0,l);c[k>>2]=1;c[m>>2]=j&1;break}}j=e+4|0;m=c[j>>2]|0;k=m+1|0;if(k>>>0<32){c[j>>2]=k;k=e|0;c[k>>2]=c[k>>2]<<1}else{k=e|0;Hw(e+8|0,c[k>>2]|0,m);c[j>>2]=1;c[k>>2]=0}LB(b,e,g,h)|0}}while(0);h=b+64|0;g=a[h]|0;if(g<<24>>24==0){n=0;return n|0}i=(f&7|0)==0;k=(f|0)>2039;j=c[497862]|0;m=e+4|0;l=e|0;o=j&255;p=e+8|0;e=((f-2040|0)>>>0)%2040|0;q=0;r=g;while(1){g=b+65+q|0;if((a[g]|0)==0){s=r}else{if(!i){n=14;t=11556;break}if(k){u=0;do{v=f;w=c[m>>2]|0;while(1){x=w+8|0;if(x>>>0<32){c[m>>2]=x;y=x;z=c[l>>2]<<8|o}else{Hw(p,c[l>>2]|0,w);c[m>>2]=8;y=8;z=o}c[l>>2]=z;x=v-2040|0;if((x|0)>2039){v=x;w=y}else{break}}w=e>>>3;v=y+8|0;if(v>>>0<32){c[m>>2]=v;A=c[l>>2]<<8|w&j}else{Hw(p,c[l>>2]|0,y);c[m>>2]=8;A=w&j}c[l>>2]=A;u=u+1|0;}while((u|0)<(d[g]|0))}else{u=0;do{w=c[m>>2]|0;v=f>>3;x=w+8|0;if(x>>>0<32){c[m>>2]=x;B=c[l>>2]<<8|v&j}else{Hw(p,c[l>>2]|0,w);c[m>>2]=8;B=v&j}c[l>>2]=B;u=u+1|0;}while((u|0)<(d[g]|0))}s=a[h]|0}g=q+1|0;if((g|0)<(s&255|0)){q=g;r=s}else{n=0;t=11555;break}}if((t|0)==11556){return n|0}else if((t|0)==11555){return n|0}return 0}function LF(b,e){b=b|0;e=e|0;var f=0;f=(c[e>>2]|0)-(c[b+72>>2]|0)|0;c[e>>2]=f;if((a[b+68|0]|0)!=0){return}c[e>>2]=(d[b+77|0]|0)+f;return}function LG(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;g=i;i=i+48|0;h=g|0;j=b+68|0;k=(a[j]|0)+1&255;a[j]=k;l=b+69|0;if((k&255)<(d[l]|0)){c[f>>2]=0;i=g;return}if((c[b+36>>2]|0)==10){k=e+8|0;if((c[e+40>>2]|0)==0){Hy(k,c[e+4>>2]|0,0);m=e|0}else{n=e|0;Hw(k,c[n>>2]|0,c[e+4>>2]|0);m=n}n=e+4|0;c[n>>2]=0;c[m>>2]=0;m=b+40|0;c[m>>2]=(((HA(k)|0)+7|0)>>>3)-3;k=h+8|0;Ht(k,c[e+28>>2]|0,c[e+32>>2]|0,0);o=h+4|0;c[o>>2]=0;p=h|0;c[p>>2]=0;q=h+40|0;c[q>>2]=1;h=c[b+44>>2]|0;Hw(k,0,0);c[o>>2]=0;c[p>>2]=0;Hz(k,h,c[q>>2]&255);h=c[m>>2]|0;m=c[497867]|0;r=c[o>>2]|0;s=r+13|0;if(s>>>0<32){c[o>>2]=s;t=c[p>>2]<<13|m&h;u=s}else{Hw(k,c[p>>2]|0,r);c[o>>2]=13;t=m&h;u=13}c[p>>2]=t;if((c[q>>2]|0)==0){Hy(k,u,0)}else{Hw(k,t,u)}c[o>>2]=0;c[p>>2]=0;v=n}else{v=e+4|0}n=d[b+77|0]|0;p=c[v>>2]|0;o=p+n|0;if(o>>>0<32){c[v>>2]=o;o=e|0;c[o>>2]=c[o>>2]<<n;w=e+8|0}else{o=e+8|0;u=e|0;Hw(o,c[u>>2]|0,p);c[v>>2]=n;c[u>>2]=0;w=o}a[j]=0;j=e+40|0;if((c[j>>2]|0)==0){Hy(w,c[v>>2]|0,0);x=e|0}else{o=e|0;Hw(w,c[o>>2]|0,c[v>>2]|0);x=o}c[v>>2]=0;c[x>>2]=0;if((c[j>>2]|0)==0){Hy(w,0,0)}else{Hw(w,0,0)}c[v>>2]=0;c[e>>2]=0;c[f>>2]=((HA(w)|0)+7|0)>>>3;w=a[b+61|0]|0;if(w<<24>>24==0){i=g;return}f=b+60|0;e=(a[f]|0)+1&255;a[f]=e;if((e&255)<(w&255)){i=g;return}a[f]=0;a[l]=a[b+76|0]|0;i=g;return}function LH(b,d,e,f,g,h){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0;i=a[e+53|0]|0;c[b+28>>2]=e;c[b+24>>2]=0;if((b|0)==0){j=2;return j|0}c[b+36>>2]=g;a[b+64|0]=1;a[b+62|0]=(f|0)!=0|0;a[b+65|0]=1;a[b+67|0]=1;a[b+66|0]=0;c[b+56>>2]=0;c[b+32>>2]=0;a[b+68|0]=0;f=b+69|0;a[f]=1;g=b+76|0;a[g]=1;c[b+40>>2]=0;a[b+60|0]=0;a[b+61|0]=i;i=c[e+36>>2]|0;if((i-1|0)>>>0>1){j=7;return j|0}e=i&255;a[g]=e;a[f]=e;LE(b,d,0,0,h)|0;j=0;return j|0}function LI(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;if((b|0)==0|(d|0)==0|(a|0)==0){f=1;return f|0}c[a+1316>>2]=e;if((e|0)==1){c[a+1040>>2]=39008;c[a+1056>>2]=38624;c[a+1044>>2]=306112;c[a+1060>>2]=305936;c[a+1048>>2]=39264;c[a+1064>>2]=38688;c[a+1052>>2]=306216;c[a+1068>>2]=305968;c[a+1024>>2]=39008;c[a+1032>>2]=38624;c[a+1028>>2]=39264;c[a+1036>>2]=38688;c[a+1016>>2]=12;c[a+1020>>2]=31;c[a+1184>>2]=6;c[a+1180>>2]=5;g=12}else if((e|0)==0){c[a+1040>>2]=39520;c[a+1056>>2]=38752;c[a+1044>>2]=306320;c[a+1060>>2]=306e3;c[a+1048>>2]=40008;c[a+1064>>2]=38880;c[a+1052>>2]=306520;c[a+1068>>2]=306056;c[a+1024>>2]=39520;c[a+1032>>2]=38752;c[a+1028>>2]=40008;c[a+1036>>2]=38880;c[a+1016>>2]=24;c[a+1020>>2]=60;c[a+1184>>2]=7;c[a+1180>>2]=6;g=24}else{f=1;return f|0}c[a+1092>>2]=40560;e=a+1088|0;c[e>>2]=40496;c[a+1100>>2]=305832;h=a+1096|0;c[h>>2]=305800;c[a+1108>>2]=39264;i=a+1104|0;c[i>>2]=38688;c[a+1116>>2]=306216;j=a+1112|0;c[j>>2]=305968;c[a+1076>>2]=40560;k=a+1072|0;c[k>>2]=40496;c[a+1084>>2]=39264;l=a+1080|0;c[l>>2]=38688;m=a+1192|0;c[m>>2]=5;n=a+1188|0;c[n>>2]=5;c[b+96>>2]=g;c[b+100>>2]=c[a+1016>>2];g=a+1020|0;c[b+88>>2]=c[g>>2];c[b+92>>2]=c[g>>2];o=b+80|0;c[o>>2]=c[g>>2];p=b+84|0;c[p>>2]=c[g>>2];c[b+120>>2]=c[a+1056>>2];c[b+124>>2]=c[a+1060>>2];c[b+112>>2]=c[a+1032>>2];c[b+128>>2]=c[a+1064>>2];c[b+132>>2]=c[a+1068>>2];c[b+116>>2]=c[a+1036>>2];c[p>>2]=c[g>>2];c[o>>2]=c[g>>2];c[b+104>>2]=c[a+1184>>2];c[b+108>>2]=c[a+1180>>2];c[d+96>>2]=12;c[d+100>>2]=12;c[d+88>>2]=31;c[d+92>>2]=31;c[d+80>>2]=31;c[d+84>>2]=31;c[d+120>>2]=c[e>>2];c[d+124>>2]=c[h>>2];c[d+112>>2]=c[k>>2];c[d+128>>2]=c[i>>2];c[d+132>>2]=c[j>>2];c[d+116>>2]=c[l>>2];c[d+104>>2]=c[m>>2];c[d+108>>2]=c[n>>2];c[b+4>>2]=0;c[d+4>>2]=0;f=0;return f|0}function LJ(b,e,f,g,h,j,k,l){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0;m=i;i=i+96|0;n=m|0;o=m+48|0;p=c[f>>2]|0;q=c[f+68>>2]>>16;r=q+32768+((ac(c[f+76>>2]|0,c[f+72>>2]|0)|0)>>16)|0;q=(h|0)!=0;if(q){s=c[f+132>>2]|0;t=c[f+128>>2]|0;u=c[f+124>>2]|0;v=c[f+120>>2]|0;w=c[f+100>>2]|0;x=c[f+96>>2]|0;y=c[f+92>>2]|0;z=c[f+88>>2]|0}else{A=c[f+80>>2]|0;B=c[f+84>>2]|0;C=c[f+112>>2]|0;D=c[f+116>>2]|0;s=D;t=D;u=C;v=C;w=B;x=A;y=B;z=A}A=(k|0)==1;k=(h|0)==1&A&1;if((c[f+64>>2]|0)==0){c[f+4>>2]=0}if((l|0)!=0){c[f+4>>2]=0}if((j|0)<=0){i=m;return}l=f+12|0;B=f+8|0;C=n|0;D=q&A;q=D?f+108|0:f+104|0;E=f+4|0;F=f+16|0;G=(p|0)>-1;H=(p|0)>0;I=-p|0;J=f+16+I|0;K=1-p+(f+16)|0;L=f+17|0;M=f+18|0;N=o|0;O=(h|0)==0;h=-y|0;P=-z|0;Q=-x|0;R=-w|0;S=0;T=0;U=b;b=(c[E>>2]|0)==0;while(1){V=e+(T<<2)|0;W=c[V>>2]|0;X=c[((W|0)==1?l:B)>>2]|0;Y=a[U]|0;Z=Y<<24>>24;a[C]=Z>>k&255;_=c[q>>2]|0;L16125:do{if(b){$=S}else{aa=Z-(a[F]|0)>>k&255;a[N]=aa;ab=aa<<24>>24;ad=aa<<24>>24<0;do{if(O){if(ad){ae=(ab|0)>(P|0)?ab:P}else{ae=(ab|0)<(z|0)?ab:z}if((ae|0)==(ab|0)){af=v+(ab+z)|0;break}else{a[N]=ae&255;$=1e4;break L16125}}else{if(A){if(ad){ag=(ab|0)>(Q|0)?ab:Q}else{ag=(ab|0)<(x|0)?ab:x}if((ag|0)==(ab|0)){af=u+(ab+x)|0;break}else{a[N]=ag&255;$=1e4;break L16125}}else{if(ad){ah=(ab|0)>(P|0)?ab:P}else{ah=(ab|0)<(z|0)?ab:z}if((ah|0)==(ab|0)){af=v+(ab+z)|0;break}else{a[N]=ah&255;$=1e4;break L16125}}}}while(0);$=d[af]|0}}while(0);do{if((W|0)==0){if(!G){a[F]=Y;a[L]=Y;a[M]=Y;break}if(H){a[F]=Y;break}else{a[J]=Y;a[K]=Y;break}}else{a[F]=Y}}while(0);Y=X-1|0;W=(Y|0)>0;do{if(D){if(W){Z=X;ab=Y;while(1){ad=a[U+ab|0]|0;aa=U+(Z-2)|0;if((ad-(a[aa]|0)|0)>(w|0)){a[aa]=ad-w&255}ad=ab-1|0;if((ad|0)>0){Z=ab;ab=ad}else{break}}}if((X|0)<=1){ai=_;aj=$;break}ab=1;Z=a[U]|0;while(1){ad=Z<<24>>24;aa=U+ab|0;ak=a[aa]|0;if((ad-(ak<<24>>24)|0)>(w|0)){al=ad-w&255;a[aa]=al;am=al}else{am=ak}ak=ab+1|0;if((ak|0)<(X|0)){ab=ak;Z=am}else{an=11666;break}}}else{if(W){Z=X;ab=Y;while(1){ak=a[U+ab|0]|0;al=U+(Z-2)|0;if((ak-(a[al]|0)|0)>(y|0)){a[al]=ak-y&255}ak=ab-1|0;if((ak|0)>0){Z=ab;ab=ak}else{break}}}if((X|0)<=1){ai=_;aj=$;break}ab=1;Z=a[U]|0;while(1){ak=Z<<24>>24;al=U+ab|0;aa=a[al]|0;if((ak-(aa<<24>>24)|0)>(y|0)){ad=ak-y&255;a[al]=ad;ao=ad}else{ao=aa}aa=ab+1|0;if((aa|0)<(X|0)){ab=aa;Z=ao}else{an=11666;break}}}}while(0);do{if((an|0)==11666){an=0;if((X|0)>1){ap=1;aq=U;ar=_;as=$}else{ai=_;aj=$;break}while(1){Y=aq+1|0;W=a[Y]|0;Z=(W<<24>>24)-(a[aq]|0)>>k&255;ab=n+ap|0;a[ab]=Z;aa=Z<<24>>24;ad=Z<<24>>24<0;do{if(O){if(ad){at=(aa|0)>(h|0)?aa:h}else{at=(aa|0)<(y|0)?aa:y}if((at|0)==(aa|0)){au=t+(aa+y)|0;an=11687;break}else{a[ab]=at&255;av=1e4;break}}else{if(A){if(ad){aw=(aa|0)>(R|0)?aa:R}else{aw=(aa|0)<(w|0)?aa:w}if((aw|0)==(aa|0)){au=s+(aa+w)|0;an=11687;break}else{a[ab]=aw&255;av=1e4;break}}else{if(ad){ax=(aa|0)>(h|0)?aa:h}else{ax=(aa|0)<(y|0)?aa:y}if((ax|0)==(aa|0)){au=t+(aa+y)|0;an=11687;break}else{a[ab]=ax&255;av=1e4;break}}}}while(0);if((an|0)==11687){an=0;av=d[au]|0}ab=av+ar|0;aa=c[V>>2]|0;if((c[E>>2]|0)!=0){do{if((aa|0)==0){if(G){if((ap|0)<(p|0)){ay=ap;break}ay=(ap<<1)-p|0;break}ad=ap<<1;if((ap|0)<(I|0)){ay=ad+ap|0;break}else{ay=ad-p|0;break}}else{ay=ap}}while(0);a[o+ap|0]=(W-(a[f+16+ay|0]|0)&255)<<24>>24>>k&255}do{if((aa|0)==0){if(G){if((ap|0)<(p|0)){a[f+16+ap|0]=W;break}else{ad=ap<<1;a[ad-p+(f+16)|0]=W;a[(ad|1)-p+(f+16)|0]=W;break}}else{if((ap|0)<(I|0)){ad=ap*3|0;a[f+16+ad|0]=W;a[ad+1+(f+16)|0]=W;a[ad+2+(f+16)|0]=W;break}else{ad=ap<<1;a[ad-p+(f+16)|0]=W;a[(ad|1)-p+(f+16)|0]=W;break}}}else{a[f+16+ap|0]=W}}while(0);if((c[E>>2]|0)==0){az=as}else{W=o+ap|0;aa=a[W]|0;ad=aa<<24>>24;Z=aa<<24>>24<0;do{if(O){if(Z){aA=(ad|0)>(P|0)?ad:P}else{aA=(ad|0)<(z|0)?ad:z}if((aA|0)==(ad|0)){aB=v+(ad+z)|0;an=11727;break}else{a[W]=aA&255;aC=1e4;break}}else{if(A){if(Z){aD=(ad|0)>(Q|0)?ad:Q}else{aD=(ad|0)<(x|0)?ad:x}if((aD|0)==(ad|0)){aB=u+(ad+x)|0;an=11727;break}else{a[W]=aD&255;aC=1e4;break}}else{if(Z){aE=(ad|0)>(P|0)?ad:P}else{aE=(ad|0)<(z|0)?ad:z}if((aE|0)==(ad|0)){aB=v+(ad+z)|0;an=11727;break}else{a[W]=aE&255;aC=1e4;break}}}}while(0);if((an|0)==11727){an=0;aC=d[aB]|0}az=aC+as|0}W=ap+1|0;if((W|0)<(X|0)){ap=W;aq=Y;ar=ab;as=az}else{ai=ab;aj=az;break}}}}while(0);do{if((T|0)==0){if((c[E>>2]|0)==0){an=11736;break}if((ai|0)>(((ac(aj,r)|0)>>14)+1>>1|0)){an=11735}else{an=11736}}else{if((aj|0)>=(ai|0)){an=11736;break}if((c[E>>2]|0)==0){an=11736}else{an=11735}}}while(0);if((an|0)==11736){an=0;c[E>>2]=0;c[g+(T<<2)>>2]=0;IY(U,C,X)}else if((an|0)==11735){an=0;c[g+(T<<2)>>2]=1;IY(U,N,X)}c[E>>2]=1;V=T+1|0;if((V|0)<(j|0)){S=aj;T=V;U=U+X|0;b=0}else{break}}i=m;return}function LK(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;I_(a,136);c[a+64>>2]=d;c[a+68>>2]=e;c[a+72>>2]=f;c[a+76>>2]=0;c[a+4>>2]=0;f=c[b>>2]|0;c[a+8>>2]=f;e=c[b+4>>2]|0;c[a+12>>2]=e;c[a>>2]=(f<<1)-e;return 0}function LL(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0;g=a+12|0;h=a+20|0;Hu(h);i=a+16|0;c[i>>2]=0;j=g|0;c[j>>2]=0;c[a+52>>2]=1;Ht(a+64|0,b,d,0);c[a+60>>2]=0;c[a+56>>2]=0;c[a+96>>2]=1;if((f&4|0)==0){k=0;return k|0}a=c[i>>2]|0;if((f&8|0)==0){f=a+10|0;if(f>>>0<32){c[i>>2]=f;c[j>>2]=c[j>>2]<<10;k=0;return k|0}else{Hw(h,c[j>>2]|0,a);c[i>>2]=10;c[j>>2]=0;k=0;return k|0}}else{f=a+8|0;if(f>>>0<32){c[i>>2]=f;l=c[j>>2]<<8}else{Hw(h,c[j>>2]|0,a);c[i>>2]=8;l=0}c[j>>2]=l;No(e,29,65535,8);k=Nq(e,g,0)|0;return k|0}return 0}function LM(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;f=i;i=i+48|0;g=f|0;if((a|0)==0){i=f;return}h=a+8|0;c[h>>2]=0;do{if((e&8|0)==0){do{if((e&1|0)==0){j=(c[a+4>>2]|0)+(c[a>>2]|0)|0;k=e&4;l=(8-((((k|0)==0?j:j+10|0)+4|0)%8|0)|0)%8|0;c[h>>2]=l;j=a+12|0;m=a+16|0;n=c[m>>2]|0;o=n+l|0;if(o>>>0<32){c[m>>2]=o;o=j|0;c[o>>2]=c[o>>2]<<l;p=k;break}else{o=j|0;Hw(a+20|0,c[o>>2]|0,n);c[m>>2]=l;c[o>>2]=0;p=k;break}}else{p=e&4}}while(0);if((p|0)==0){q=a+56|0;break}k=g;o=a+12|0;NN(k|0,o|0,44)|0;o=g+8|0;Hu(o);k=g+4|0;c[k>>2]=0;l=g|0;c[l>>2]=0;c[g+40>>2]=0;m=(c[a+4>>2]|0)+(c[a>>2]|0)+(c[h>>2]|0)|0;if((m|0)>0){n=c[497855]&1;j=0;r=0;s=0;t=0;while(1){if(s>>>0>1){u=s;v=t}else{w=31-s|0;x=t<<w|(Hv(o,w)|0);c[l>>2]=x;y=(c[k>>2]|0)+w|0;c[k>>2]=y;u=y;v=x}x=u-1|0;c[k>>2]=x;y=j<<1;z=(n&v>>>(x>>>0)|0)!=0^(j&512)!=0?y^563:y;y=r+1|0;if((y|0)<(m|0)){j=z;r=y;s=x;t=v}else{break}}A=z&1023}else{A=0}t=a+56|0;s=c[497864]|0;r=a+60|0;j=c[r>>2]|0;m=j+10|0;if(m>>>0<32){c[r>>2]=m;m=t|0;c[m>>2]=c[m>>2]<<10|s&A;q=t;break}else{m=t|0;Hw(a+64|0,c[m>>2]|0,j);c[r>>2]=10;c[m>>2]=s&A;q=t;break}}else{t=a+12|0;Nr(b,t,d)|0;t=a+56|0;s=(Ns(b)|0)&65535^255;m=c[497862]|0;r=a+60|0;j=c[r>>2]|0;n=j+8|0;if(n>>>0<32){c[r>>2]=n;n=t|0;c[n>>2]=c[n>>2]<<8|s&m;q=t;break}else{n=t|0;Hw(a+64|0,c[n>>2]|0,j);c[r>>2]=8;c[n>>2]=s&m;q=t;break}}}while(0);b=a+64|0;if((c[a+96>>2]|0)==0){Hy(b,c[a+60>>2]|0,0);B=q|0}else{d=q|0;Hw(b,c[d>>2]|0,c[a+60>>2]|0);B=d}c[a+60>>2]=0;c[B>>2]=0;i=f;return}function LN(b,d,e,f,g,h,j){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0;j=i;i=i+4096|0;g=j|0;e=f+1212|0;d=f+1208|0;k=c[d>>2]|0;l=f+760|0;m=c[f+1216>>2]|0;n=f+888|0;o=c[b+36>>2]|0;p=f+1220|0;q=c[p>>2]|0;r=f+6344|0;s=f+1152|0;t=(q|0)>0;if((c[e>>2]|0)==0){if(t){u=(o|0)==0;v=32;w=0;do{if(u){x=0;y=11791}else{z=o;A=c[f+760+(w+m<<2)>>2]|0;B=0;while(1){C=c[A>>2]|0;D=C>>31^C|B;C=z-1|0;if((C|0)==0){break}else{z=C;A=A+4|0;B=D}}if((D|0)>-1){x=D;y=11791}else{E=-1}}if((y|0)==11791){y=0;B=~x;A=0;while(1){z=B<<1;if((z|0)<0){B=z;A=A+1|0}else{E=A;break}}}A=(E|0)<0?0:E;if(u){F=0;y=11797}else{B=o;z=c[f+888+(w+m<<2)>>2]|0;C=0;while(1){H=c[z>>2]|0;I=H>>31^H|C;H=B-1|0;if((H|0)==0){break}else{B=H;z=z+4|0;C=I}}if((I|0)>-1){F=I;y=11797}else{J=-1}}if((y|0)==11797){y=0;C=~F;z=0;while(1){B=C<<1;if((B|0)<0){C=B;z=z+1|0}else{J=z;break}}}z=(J|0)<0?0:J;C=(A|0)<(z|0)?A:z;v=(v|0)<(C|0)?v:C;w=w+1|0;}while((w|0)<(q|0));if((v|0)>30){y=11801}else{K=v}}else{y=11801}if((y|0)==11801){K=15-(c[r>>2]|0)|0}v=K-1|0;K=(v|0)<0?0:v;v=K+(c[r>>2]|0)|0;c[r>>2]=v;w=g|0;do{if(t&(o|0)>0){J=w;F=0;I=0;while(1){u=I+m|0;E=c[f+760+(u<<2)>>2]|0;x=c[f+888+(u<<2)>>2]|0;u=J;D=F;C=0;while(1){z=E+(C<<2)|0;B=x+(C<<2)|0;H=c[z>>2]<<K;L=c[B>>2]<<K;M=H;N=(H|0)<0|0?-1:0;N2(M,N,M,N)|0;N=G;M=L;O=(L|0)<0|0?-1:0;N2(M,O,M,O)|0;O=G+N|0;c[u>>2]=O;P=(D|0)>(O|0)?D:O;c[z>>2]=H;c[B>>2]=L;L=C+1|0;if((L|0)<(o|0)){u=u+4|0;D=P;C=L}else{break}}C=I+1|0;if((C|0)<(q|0)){J=J+(o<<2)|0;F=P;I=C}else{break}}I=(c[r>>2]<<1)-1|0;c[s>>2]=I;if((P|0)==0){Q=0;R=I;break}F=P>>31^P;if((F|0)<=-1){Q=-1;R=I;break}J=~F;F=0;while(1){A=J<<1;if((A|0)<0){J=A;F=F+1|0}else{Q=F;R=I;break}}}else{I=(v<<1)-1|0;c[s>>2]=I;Q=0;R=I}}while(0);if(t){v=0;P=w;while(1){Lo(c[f+1020+(v+k<<2)>>2]|0,P,o,Q);w=v+1|0;if((w|0)<(q|0)){v=w;P=P+(o<<2)|0}else{break}}S=c[s>>2]|0}else{S=R}c[s>>2]=S+Q;T=f+1620|0;U=b+12|0;V=c[U>>2]|0;W=b+48|0;X=c[W>>2]|0;Y=X+V|0;Z=a[Y]|0;_=Z&255;$=c[r>>2]|0;IE(T,l,n,_,$);aa=f|0;ab=f+1020|0;ac=f+1148|0;ad=h+544|0;ae=c[d>>2]|0;af=c[e>>2]|0;ag=f+1236|0;ah=c[ag>>2]|0;ai=f+1240|0;aj=c[ai>>2]|0;IN(aa,ab,ac,ad,ae,af,ah,aj);ak=c[W>>2]|0;al=c[d>>2]|0;am=c[e>>2]|0;an=c[U>>2]|0;ao=c[ag>>2]|0;ap=c[p>>2]|0;IM(ab,ac,aa,ak,ad,al,am,an,ao,ap);i=j;return}if(t){Q=(o|0)==0;S=32;R=0;do{if(Q){aq=0;y=11821}else{P=o;v=c[f+760+(R+m<<2)>>2]|0;w=0;while(1){K=c[v>>2]|0;ar=K>>31^K|w;K=P-1|0;if((K|0)==0){break}else{P=K;v=v+4|0;w=ar}}if((ar|0)>-1){aq=ar;y=11821}else{as=-1}}if((y|0)==11821){y=0;w=~aq;v=0;while(1){P=w<<1;if((P|0)<0){w=P;v=v+1|0}else{as=v;break}}}v=(as|0)<0?0:as;if(Q){at=0;y=11827}else{w=o;P=c[f+888+(R+m<<2)>>2]|0;K=0;while(1){I=c[P>>2]|0;au=I>>31^I|K;I=w-1|0;if((I|0)==0){break}else{w=I;P=P+4|0;K=au}}if((au|0)>-1){at=au;y=11827}else{av=-1}}if((y|0)==11827){y=0;K=~at;P=0;while(1){w=K<<1;if((w|0)<0){K=w;P=P+1|0}else{av=P;break}}}P=(av|0)<0?0:av;K=(v|0)<(P|0)?v:P;S=(S|0)<(K|0)?S:K;R=R+1|0;}while((R|0)<(q|0));if((S|0)>30){y=11831}else{aw=S}}else{y=11831}if((y|0)==11831){aw=15-(c[r>>2]|0)|0}S=aw-1|0;aw=(S|0)<0?0:S;S=aw+(c[r>>2]|0)|0;c[r>>2]=S;R=g|0;L16434:do{if(t){if((o|0)>0){ax=R;ay=0;az=0}else{g=0;while(1){av=g+2|0;if((av|0)<(q|0)){g=av}else{y=11838;break L16434}}}while(1){g=az+m|0;v=c[f+760+(g<<2)>>2]|0;av=c[f+888+(g<<2)>>2]|0;g=(az|1)+m|0;at=c[f+760+(g<<2)>>2]|0;au=c[f+888+(g<<2)>>2]|0;g=ax;Q=ay;as=0;while(1){aq=v+(as<<2)|0;ar=at+(as<<2)|0;K=c[ar>>2]|0;P=av+(as<<2)|0;w=au+(as<<2)|0;I=c[w>>2]|0;F=c[aq>>2]<<aw;J=c[P>>2]<<aw;A=F;C=(F|0)<0|0?-1:0;N2(A,C,A,C)|0;C=G;A=J;D=(J|0)<0|0?-1:0;N2(A,D,A,D)|0;D=(G+C|0)>>>1;C=K<<aw;K=I<<aw;I=C;A=(C|0)<0|0?-1:0;N2(I,A,I,A)|0;A=G;I=K;u=(K|0)<0|0?-1:0;N2(I,u,I,u)|0;u=((G+A|0)>>>1)+D|0;c[g>>2]=u;aA=(Q|0)>(u|0)?Q:u;c[aq>>2]=F;c[ar>>2]=C;c[P>>2]=J;c[w>>2]=K;K=as+1|0;if((K|0)<(o|0)){g=g+4|0;Q=aA;as=K}else{break}}as=az+2|0;if((as|0)<(q|0)){ax=ax+(o<<2)|0;ay=aA;az=as}else{break}}as=(c[r>>2]<<1)-1|0;c[s>>2]=as;if((aA|0)==0){aB=0;aC=as;break}Q=aA>>31^aA;if((Q|0)<=-1){aB=-1;aC=as;break}g=~Q;Q=0;while(1){au=g<<1;if((au|0)<0){g=au;Q=Q+1|0}else{aB=Q;aC=as;break}}}else{y=11838}}while(0);if((y|0)==11838){y=(S<<1)-1|0;c[s>>2]=y;aB=0;aC=y}y=q>>1;if((y|0)>0){q=0;S=R;while(1){Lo(c[f+1020+(q+k<<2)>>2]|0,S,o,aB);R=q+1|0;if((R|0)<(y|0)){q=R;S=S+(o<<2)|0}else{break}}aD=c[s>>2]|0}else{aD=aC}c[s>>2]=aD+aB;T=f+1620|0;U=b+12|0;V=c[U>>2]|0;W=b+48|0;X=c[W>>2]|0;Y=X+V|0;Z=a[Y]|0;_=Z&255;$=c[r>>2]|0;IE(T,l,n,_,$);aa=f|0;ab=f+1020|0;ac=f+1148|0;ad=h+544|0;ae=c[d>>2]|0;af=c[e>>2]|0;ag=f+1236|0;ah=c[ag>>2]|0;ai=f+1240|0;aj=c[ai>>2]|0;IN(aa,ab,ac,ad,ae,af,ah,aj);ak=c[W>>2]|0;al=c[d>>2]|0;am=c[e>>2]|0;an=c[U>>2]|0;ao=c[ag>>2]|0;ap=c[p>>2]|0;IM(ab,ac,aa,ak,ad,al,am,an,ao,ap);i=j;return}function LO(b,e,f,g,h,j,k,l,m,n){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0;o=i;i=i+216|0;p=o|0;q=o+8|0;r=o+104|0;s=o+120|0;t=o+128|0;u=o+136|0;v=o+176|0;w=p|0;c[w>>2]=h;x=p+4|0;c[x>>2]=j;y=c[h+1212>>2]|0;z=c[b+56>>2]|0;A=c[b+4>>2]|0;B=b|0;C=(c[B>>2]&1|0)==0?1914456:1914480;do{if((z|0)==2){D=l+544|0;E=(a[l+1093|0]|0)==0;if((a[l+545|0]|0)==0){F=l+1092|0;G=a[F]|0;H=D|0;if(E){I=a[H]|0;J=(G&255)>(I&255)?G:I;a[H]=J;a[F]=J;break}else{a[H]=G;break}}else{if(E){a[l+1092|0]=a[D|0]|0;break}else{E=l+1092|0;G=a[E]|0;H=D|0;D=a[H]|0;J=(G&255)<(D&255)?G:D;a[H]=J;a[E]=J;break}}}}while(0);J=h+6342|0;E=l|0;c[E>>2]=LX(h+1240|0,l+544|0,h+1204|0,d[J]|0,C)|0;H=h+1436|0;c[h+5856>>2]=H;do{if((a[J]|0)!=0){if((a[l+546|0]|0)==0){break}c[h+1600>>2]=c[h+1552>>2]}}while(0);do{if((z|0)==2){c[l+548>>2]=c[E>>2];c[j+5856>>2]=H;K=2}else if((z|0)==1|(z|0)==3){J=j+6342|0;D=l+548|0;c[D>>2]=LX(j+1240|0,l+1092|0,j+1204|0,d[J]|0,C)|0;c[j+5856>>2]=j+1436;do{if((a[J]|0)!=0){if((a[l+1094|0]|0)==0){break}c[j+1600>>2]=c[j+1552>>2]}}while(0);J=c[E>>2]|0;G=c[J>>2]|0;F=c[D>>2]|0;if((G|0)!=(c[F>>2]|0)){K=1;break}I=G+1|0;L=0;while(1){if((L|0)>=(I|0)){M=z;break}if((c[J+4+(L<<2)>>2]|0)==(c[F+4+(L<<2)>>2]|0)){L=L+1|0}else{M=1;break}}L=0;while(1){if((L|0)>=(G|0)){N=M;break}if((c[J+28+(L<<2)>>2]|0)==(c[F+28+(L<<2)>>2]|0)){L=L+1|0}else{N=1;break}}K=(c[J+48>>2]|0)==(c[F+48>>2]|0)?N:1}else{K=z}}while(0);z=(A|0)>0;do{if(z){N=b+80|0;M=(n|0)==0;C=b+48|0;H=b+12|0;L=e|0;G=0;I=h;while(1){D=l+(G*548|0)+544|0;a[I+1204|0]=a[D]|0;a[I+1205|0]=a[l+(G*548|0)+545|0]|0;O=l+(G*548|0)|0;P=c[c[O>>2]>>2]|0;Q=l+(G*548|0)+547|0;a[Q]=P&255;R=I+4736|0;c[I+5932>>2]=P&255;do{if((c[(c[I+5856>>2]|0)+8>>2]|0)==0){if((a[Q]|0)!=1){S=11885;break}if((a[I+6342|0]|0)==0){c[I+6056>>2]=0;T=0}else{P=d[N]|0;c[I+6056>>2]=P;T=P}if((T|0)==(c[I+6052>>2]|0)){break}P=I+488|0;U=I+624|0;LI(R,P,U,T)|0}else{S=11885}}while(0);do{if((S|0)==11885){S=0;Q=c[L>>2]|0;if((Q|0)==(c[I+6052>>2]|0)){break}U=I+488|0;P=I+624|0;LI(R,U,P,Q)|0}}while(0);if(M){IG(I+1620|0,I+4748|0,l+(G*548|0)+4|0,I+5864|0,I+5868|0,I+1156|0,c[O>>2]|0,D,c[C>>2]|0,c[H>>2]|0,c[I+4788>>2]|0,c[B>>2]|0)}R=c[I+460>>2]|0;Q=I+6348|0;if((c[I+456>>2]|0)<(R|0)&(R|0)>64424509){a[Q]=1;c[I+4748>>2]=3;c[I+4752>>2]=3;c[I+4756>>2]=3;c[I+4760>>2]=3;c[I+4764>>2]=3;c[I+4768>>2]=3;c[I+4772>>2]=3;c[I+4776>>2]=3;c[I+4780>>2]=3;c[I+4784>>2]=3;V=3}else{a[Q]=0;V=c[I+4748>>2]|0}c[I+4744>>2]=V;c[I+6044>>2]=c[I+3740>>2];Q=G+1|0;if((Q|0)>=(A|0)){break}G=Q;I=c[p+(Q<<2)>>2]|0}if(!z){break}I=b+12|0;G=b+8|0;H=0;do{C=l+(H*548|0)+547|0;if((a[C]|0)!=0){M=l+(H*548|0)|0;L=c[p+(H<<2)>>2]|0;N=0;do{c[L+5936+(N<<2)>>2]=c[((c[(c[M>>2]|0)+28+(N<<2)>>2]|0)==1?I:G)>>2];N=N+1|0;}while((N|0)<(d[C]|0))}H=H+1|0;}while((H|0)<(A|0))}}while(0);if((K|0)==1){LP(h+1020|0,0,c[h+1148>>2]|0,c[h+1152>>2]|0,0,c[E>>2]|0,l+284|0,0,b,h,0,0,y);V=c[x>>2]|0;T=l+548|0;LP(V+1020|0,0,c[V+1148>>2]|0,c[V+1152>>2]|0,0,c[T>>2]|0,l+832|0,0,b,V,0,0,y);n=c[l+4>>2]|0;if((n|0)>1006632960){W=30}else{H=n>>25;W=(H|0)==0?0:H+1&255}H=l+534|0;a[H]=W;W=c[l+8>>2]|0;if((W|0)>1006632960){X=30}else{n=W>>25;X=(n|0)==0?0:n+1&255}a[l+535|0]=X;X=c[l+12>>2]|0;if((X|0)>1006632960){Y=30}else{n=X>>25;Y=(n|0)==0?0:n+1&255}a[l+536|0]=Y;Y=c[l+16>>2]|0;if((Y|0)>1006632960){Z=30}else{n=Y>>25;Z=(n|0)==0?0:n+1&255}a[l+537|0]=Z;Z=c[l+20>>2]|0;if((Z|0)>1006632960){_=30}else{n=Z>>25;_=(n|0)==0?0:n+1&255}a[l+538|0]=_;_=c[l+24>>2]|0;if((_|0)>1006632960){$=30}else{n=_>>25;$=(n|0)==0?0:n+1&255}a[l+539|0]=$;$=c[l+28>>2]|0;if(($|0)>1006632960){aa=30}else{n=$>>25;aa=(n|0)==0?0:n+1&255}a[l+540|0]=aa;aa=c[l+32>>2]|0;if((aa|0)>1006632960){ab=30}else{n=aa>>25;ab=(n|0)==0?0:n+1&255}a[l+541|0]=ab;ab=c[l+36>>2]|0;if((ab|0)>1006632960){ac=30}else{n=ab>>25;ac=(n|0)==0?0:n+1&255}a[l+542|0]=ac;ac=c[l+40>>2]|0;if((ac|0)>1006632960){ad=30}else{n=ac>>25;ad=(n|0)==0?0:n+1&255}a[l+543|0]=ad;ad=m|0;n=c[w>>2]|0;ac=g+12|0;LJ(H,ad,n+624|0,n+5976|0,0,(c[c[E>>2]>>2]|0)>1?2:1,0,c[ac>>2]|0);H=c[l+552>>2]|0;if((H|0)>1006632960){ae=30}else{ab=H>>25;ae=(ab|0)==0?0:ab+1&255}ab=l+1082|0;a[ab]=ae;ae=c[l+556>>2]|0;if((ae|0)>1006632960){af=30}else{H=ae>>25;af=(H|0)==0?0:H+1&255}a[l+1083|0]=af;af=c[l+560>>2]|0;if((af|0)>1006632960){ag=30}else{H=af>>25;ag=(H|0)==0?0:H+1&255}a[l+1084|0]=ag;ag=c[l+564>>2]|0;if((ag|0)>1006632960){ah=30}else{H=ag>>25;ah=(H|0)==0?0:H+1&255}a[l+1085|0]=ah;ah=c[l+568>>2]|0;if((ah|0)>1006632960){ai=30}else{H=ah>>25;ai=(H|0)==0?0:H+1&255}a[l+1086|0]=ai;ai=c[l+572>>2]|0;if((ai|0)>1006632960){aj=30}else{H=ai>>25;aj=(H|0)==0?0:H+1&255}a[l+1087|0]=aj;aj=c[l+576>>2]|0;if((aj|0)>1006632960){ak=30}else{H=aj>>25;ak=(H|0)==0?0:H+1&255}a[l+1088|0]=ak;ak=c[l+580>>2]|0;if((ak|0)>1006632960){al=30}else{H=ak>>25;al=(H|0)==0?0:H+1&255}a[l+1089|0]=al;al=c[l+584>>2]|0;if((al|0)>1006632960){am=30}else{H=al>>25;am=(H|0)==0?0:H+1&255}a[l+1090|0]=am;am=c[l+588>>2]|0;if((am|0)>1006632960){an=30}else{H=am>>25;an=(H|0)==0?0:H+1&255}a[l+1091|0]=an;LJ(ab,ad,V+624|0,V+5976|0,0,(c[c[T>>2]>>2]|0)>1?2:1,0,c[ac>>2]|0);ao=n;ap=V;S=11917}else if((K|0)==2){LP(h+1020|0,j+1020|0,c[h+1148>>2]|0,c[h+1152>>2]|0,j+1148|0,c[E>>2]|0,l+284|0,l+832|0,b,h,2,m+40|0,y);V=l+4|0;n=l+552|0;LR(V,n);ac=c[V>>2]|0;if((ac|0)>1006632960){aq=30}else{V=ac>>25;aq=(V|0)==0?0:V+1&255}V=l+534|0;a[V]=aq;aq=c[l+8>>2]|0;if((aq|0)>1006632960){ar=30}else{ac=aq>>25;ar=(ac|0)==0?0:ac+1&255}a[l+535|0]=ar;ar=c[l+12>>2]|0;if((ar|0)>1006632960){as=30}else{ac=ar>>25;as=(ac|0)==0?0:ac+1&255}a[l+536|0]=as;as=c[l+16>>2]|0;if((as|0)>1006632960){at=30}else{ac=as>>25;at=(ac|0)==0?0:ac+1&255}a[l+537|0]=at;at=c[l+20>>2]|0;if((at|0)>1006632960){au=30}else{ac=at>>25;au=(ac|0)==0?0:ac+1&255}a[l+538|0]=au;au=c[l+24>>2]|0;if((au|0)>1006632960){av=30}else{ac=au>>25;av=(ac|0)==0?0:ac+1&255}a[l+539|0]=av;av=c[l+28>>2]|0;if((av|0)>1006632960){aw=30}else{ac=av>>25;aw=(ac|0)==0?0:ac+1&255}a[l+540|0]=aw;aw=c[l+32>>2]|0;if((aw|0)>1006632960){ax=30}else{ac=aw>>25;ax=(ac|0)==0?0:ac+1&255}a[l+541|0]=ax;ax=c[l+36>>2]|0;if((ax|0)>1006632960){ay=30}else{ac=ax>>25;ay=(ac|0)==0?0:ac+1&255}a[l+542|0]=ay;ay=c[l+40>>2]|0;if((ay|0)>1006632960){az=30}else{ac=ay>>25;az=(ac|0)==0?0:ac+1&255}a[l+543|0]=az;az=m|0;ac=c[w>>2]|0;ay=g+12|0;LJ(V,az,ac+624|0,ac+5976|0,1,(c[c[E>>2]>>2]|0)>1?2:1,0,c[ay>>2]|0);V=l+1082|0;LQ(V,n,1);n=c[x>>2]|0;LJ(V,az,n+624|0,n+5976|0,1,(c[c[l+548>>2]>>2]|0)>1?2:1,1,c[ay>>2]|0);ao=ac;ap=n;S=11917}else if((K|0)==3){LP(h+1020|0,0,c[h+1148>>2]|0,c[h+1152>>2]|0,0,c[E>>2]|0,l+284|0,0,b,h,0,0,y);n=c[x>>2]|0;ac=n+1020|0;ay=n+1148|0;LP(ac,0,c[ay>>2]|0,c[n+1152>>2]|0,0,c[l+548>>2]|0,l+832|0,0,b,n,0,0,y);az=c[w>>2]|0;LP(az+1020|0,ac,c[az+1148>>2]|0,c[az+1152>>2]|0,ay,c[E>>2]|0,l+44|0,l+592|0,b,az,2,m+40|0,y);ay=l+4|0;ac=c[ay>>2]|0;if((ac|0)>1006632960){aA=30}else{V=ac>>25;aA=(V|0)==0?0:V+1&255}a[l+534|0]=aA;aA=l+8|0;V=c[aA>>2]|0;if((V|0)>1006632960){aB=30}else{ac=V>>25;aB=(ac|0)==0?0:ac+1&255}a[l+535|0]=aB;aB=l+12|0;ac=c[aB>>2]|0;if((ac|0)>1006632960){aC=30}else{V=ac>>25;aC=(V|0)==0?0:V+1&255}a[l+536|0]=aC;aC=l+16|0;V=c[aC>>2]|0;if((V|0)>1006632960){aD=30}else{ac=V>>25;aD=(ac|0)==0?0:ac+1&255}a[l+537|0]=aD;aD=l+20|0;ac=c[aD>>2]|0;if((ac|0)>1006632960){aE=30}else{V=ac>>25;aE=(V|0)==0?0:V+1&255}a[l+538|0]=aE;aE=l+24|0;V=c[aE>>2]|0;if((V|0)>1006632960){aF=30}else{ac=V>>25;aF=(ac|0)==0?0:ac+1&255}a[l+539|0]=aF;aF=l+28|0;ac=c[aF>>2]|0;if((ac|0)>1006632960){aG=30}else{V=ac>>25;aG=(V|0)==0?0:V+1&255}a[l+540|0]=aG;aG=l+32|0;V=c[aG>>2]|0;if((V|0)>1006632960){aH=30}else{ac=V>>25;aH=(ac|0)==0?0:ac+1&255}a[l+541|0]=aH;aH=l+36|0;ac=c[aH>>2]|0;if((ac|0)>1006632960){aI=30}else{V=ac>>25;aI=(V|0)==0?0:V+1&255}a[l+542|0]=aI;aI=l+40|0;V=c[aI>>2]|0;if((V|0)>1006632960){aJ=30}else{ac=V>>25;aJ=(ac|0)==0?0:ac+1&255}a[l+543|0]=aJ;aJ=l+552|0;ac=c[aJ>>2]|0;if((ac|0)>1006632960){aK=30}else{V=ac>>25;aK=(V|0)==0?0:V+1&255}a[l+1082|0]=aK;aK=c[l+556>>2]|0;if((aK|0)>1006632960){aL=30}else{V=aK>>25;aL=(V|0)==0?0:V+1&255}a[l+1083|0]=aL;aL=c[l+560>>2]|0;if((aL|0)>1006632960){aM=30}else{V=aL>>25;aM=(V|0)==0?0:V+1&255}a[l+1084|0]=aM;aM=c[l+564>>2]|0;if((aM|0)>1006632960){aN=30}else{V=aM>>25;aN=(V|0)==0?0:V+1&255}a[l+1085|0]=aN;aN=c[l+568>>2]|0;if((aN|0)>1006632960){aO=30}else{V=aN>>25;aO=(V|0)==0?0:V+1&255}a[l+1086|0]=aO;aO=c[l+572>>2]|0;if((aO|0)>1006632960){aP=30}else{V=aO>>25;aP=(V|0)==0?0:V+1&255}a[l+1087|0]=aP;aP=c[l+576>>2]|0;if((aP|0)>1006632960){aQ=30}else{V=aP>>25;aQ=(V|0)==0?0:V+1&255}a[l+1088|0]=aQ;aQ=c[l+580>>2]|0;if((aQ|0)>1006632960){aR=30}else{V=aQ>>25;aR=(V|0)==0?0:V+1&255}a[l+1089|0]=aR;aR=c[l+584>>2]|0;if((aR|0)>1006632960){aS=30}else{V=aR>>25;aS=(V|0)==0?0:V+1&255}a[l+1090|0]=aS;aS=c[l+588>>2]|0;if((aS|0)>1006632960){aT=30}else{V=aS>>25;aT=(V|0)==0?0:V+1&255}a[l+1091|0]=aT;LR(ay,aJ);aT=c[ay>>2]|0;if((aT|0)>1006632960){aU=30}else{ay=aT>>25;aU=(ay|0)==0?0:ay+1&255}a[l+524|0]=aU;aU=c[aA>>2]|0;if((aU|0)>1006632960){aV=30}else{aA=aU>>25;aV=(aA|0)==0?0:aA+1&255}a[l+525|0]=aV;aV=c[aB>>2]|0;if((aV|0)>1006632960){aW=30}else{aB=aV>>25;aW=(aB|0)==0?0:aB+1&255}a[l+526|0]=aW;aW=c[aC>>2]|0;if((aW|0)>1006632960){aX=30}else{aC=aW>>25;aX=(aC|0)==0?0:aC+1&255}a[l+527|0]=aX;aX=c[aD>>2]|0;if((aX|0)>1006632960){aY=30}else{aD=aX>>25;aY=(aD|0)==0?0:aD+1&255}a[l+528|0]=aY;aY=c[aE>>2]|0;if((aY|0)>1006632960){aZ=30}else{aE=aY>>25;aZ=(aE|0)==0?0:aE+1&255}a[l+529|0]=aZ;aZ=c[aF>>2]|0;if((aZ|0)>1006632960){a_=30}else{aF=aZ>>25;a_=(aF|0)==0?0:aF+1&255}a[l+530|0]=a_;a_=c[aG>>2]|0;if((a_|0)>1006632960){a$=30}else{aG=a_>>25;a$=(aG|0)==0?0:aG+1&255}a[l+531|0]=a$;a$=c[aH>>2]|0;if((a$|0)>1006632960){a0=30}else{aH=a$>>25;a0=(aH|0)==0?0:aH+1&255}a[l+532|0]=a0;a0=c[aI>>2]|0;if((a0|0)>1006632960){a1=30}else{aI=a0>>25;a1=(aI|0)==0?0:aI+1&255}a[l+533|0]=a1;LQ(l+1072|0,aJ,1);ao=az;ap=n;S=11917}else if((K|0)==0){LP(h+1020|0,0,c[h+1148>>2]|0,c[h+1152>>2]|0,0,c[E>>2]|0,l+284|0,0,b,h,0,0,y);y=c[l+4>>2]|0;if((y|0)>1006632960){a2=30}else{b=y>>25;a2=(b|0)==0?0:b+1&255}b=l+534|0;a[b]=a2;a2=c[l+8>>2]|0;if((a2|0)>1006632960){a3=30}else{y=a2>>25;a3=(y|0)==0?0:y+1&255}a[l+535|0]=a3;a3=c[l+12>>2]|0;if((a3|0)>1006632960){a4=30}else{y=a3>>25;a4=(y|0)==0?0:y+1&255}a[l+536|0]=a4;a4=c[l+16>>2]|0;if((a4|0)>1006632960){a5=30}else{y=a4>>25;a5=(y|0)==0?0:y+1&255}a[l+537|0]=a5;a5=c[l+20>>2]|0;if((a5|0)>1006632960){a6=30}else{y=a5>>25;a6=(y|0)==0?0:y+1&255}a[l+538|0]=a6;a6=c[l+24>>2]|0;if((a6|0)>1006632960){a7=30}else{y=a6>>25;a7=(y|0)==0?0:y+1&255}a[l+539|0]=a7;a7=c[l+28>>2]|0;if((a7|0)>1006632960){a8=30}else{y=a7>>25;a8=(y|0)==0?0:y+1&255}a[l+540|0]=a8;a8=c[l+32>>2]|0;if((a8|0)>1006632960){a9=30}else{y=a8>>25;a9=(y|0)==0?0:y+1&255}a[l+541|0]=a9;a9=c[l+36>>2]|0;if((a9|0)>1006632960){ba=30}else{y=a9>>25;ba=(y|0)==0?0:y+1&255}a[l+542|0]=ba;ba=c[l+40>>2]|0;if((ba|0)>1006632960){bb=30}else{y=ba>>25;bb=(y|0)==0?0:y+1&255}a[l+543|0]=bb;bb=c[w>>2]|0;LJ(b,m|0,bb+624|0,bb+5976|0,0,(c[c[E>>2]>>2]|0)>1?2:1,0,c[g+12>>2]|0);ao=bb;ap=j;S=11917}else{bc=h;bd=j;S=11951}do{if((S|0)==11917){if((K|0)==2){j=e+64|0;c[j>>2]=1;c[ao+6048>>2]=0;c[ap+6048>>2]=1;h=c[E>>2]|0;bb=c[w>>2]|0;b=g+12|0;LJ(l+284|0,h+28|0,bb+488|0,bb+5956|0,c[j>>2]|0,c[h>>2]|0,0,c[b>>2]|0);h=c[l+548>>2]|0;y=c[x>>2]|0;LJ(l+832|0,h+28|0,y+488|0,y+5956|0,c[j>>2]|0,c[h>>2]|0,1,c[b>>2]|0);bc=bb;bd=y;S=11951;break}else if((K|0)==3){L16746:do{if(z){y=e+68|0;bb=0;b=ao;while(1){IY(q+(bb*48|0)|0,b+504|0,48);IY(r+(bb*5|0)|0,b+640|0,5);h=b+492|0;c[s+(bb<<2)>>2]=c[h>>2];j=b+628|0;c[t+(bb<<2)>>2]=c[j>>2];if((c[y>>2]|0)!=0){c[h>>2]=0;c[j>>2]=0}j=bb+1|0;if((j|0)>=(A|0)){break L16746}bb=j;b=c[p+(j<<2)>>2]|0}}}while(0);b=c[E>>2]|0;bb=g+12|0;LJ(l+284|0,b+28|0,ao+488|0,ao+5956|0,0,c[b>>2]|0,0,c[bb>>2]|0);b=l+548|0;y=c[b>>2]|0;LJ(l+832|0,y+28|0,ap+488|0,ap+5956|0,0,c[y>>2]|0,0,c[bb>>2]|0);y=l+547|0;j=a[y]|0;if(j<<24>>24!=0){h=0;ba=0;a9=j;while(1){j=ao+5936+(ba<<2)|0;if((c[j>>2]|0)>0){a8=h;a7=0;do{c[ao+4792+(ba*192|0)+(a7<<2)>>2]=a[l+284+a8|0]|0;c[ap+4792+(ba*192|0)+(a7<<2)>>2]=a[l+832+a8|0]|0;a8=a8+1|0;a7=a7+1|0;}while((a7|0)<(c[j>>2]|0));be=a8;bf=a[y]|0}else{be=h;bf=a9}j=ba+1|0;if((j|0)<(bf&255|0)){h=be;ba=j;a9=bf}else{break}}}a9=l+534|0;ba=m|0;LJ(a9,ba,ao+624|0,ao+5976|0,0,(c[c[E>>2]>>2]|0)>1?2:1,0,c[bb>>2]|0);a[ao+5996|0]=a[a9]|0;a[ao+5997|0]=a[l+535|0]|0;a[ao+5998|0]=a[l+536|0]|0;a[ao+5999|0]=a[l+537|0]|0;a[ao+6e3|0]=a[l+538|0]|0;a[ao+6001|0]=a[l+539|0]|0;a[ao+6002|0]=a[l+540|0]|0;a[ao+6003|0]=a[l+541|0]|0;a[ao+6004|0]=a[l+542|0]|0;a[ao+6005|0]=a[l+543|0]|0;a9=l+1082|0;h=c[x>>2]|0;j=h+624|0;a7=h+5976|0;LJ(a9,ba,j,a7,0,(c[c[b>>2]>>2]|0)>1?2:1,0,c[bb>>2]|0);a6=h+5996|0;a[a6]=a[a9]|0;a9=h+5997|0;a[a9]=a[l+1083|0]|0;a5=h+5998|0;a[a5]=a[l+1084|0]|0;a4=h+5999|0;a[a4]=a[l+1085|0]|0;a3=h+6e3|0;a[a3]=a[l+1086|0]|0;a2=h+6001|0;a[a2]=a[l+1087|0]|0;n=h+6002|0;a[n]=a[l+1088|0]|0;az=h+6003|0;a[az]=a[l+1089|0]|0;aJ=h+6004|0;a[aJ]=a[l+1090|0]|0;a1=h+6005|0;a[a1]=a[l+1091|0]|0;aI=e+64|0;c[aI>>2]=0;a0=c[w>>2]|0;aH=a0+6048|0;c[aH>>2]=0;a$=h+6048|0;c[a$>>2]=0;aG=a0+4736|0;a_=h+4736|0;aF=H8(e,f,g,aG,a_,k,c[B>>2]|0)|0;L16764:do{if(z){aZ=e+68|0;aE=0;aY=a0;while(1){aD=0;do{aX=aY+504+aD|0;aC=a[aX]|0;aW=q+(aE*48|0)+aD|0;a[aX]=a[aW]|0;a[aW]=aC;aD=aD+1|0;}while((aD|0)<48);aD=aY+640|0;aC=a[aD]|0;aW=r+(aE*5|0)|0;a[aD]=a[aW]|0;a[aW]=aC;aC=aY+641|0;aW=a[aC]|0;aD=r+(aE*5|0)+1|0;a[aC]=a[aD]|0;a[aD]=aW;aW=aY+642|0;aD=a[aW]|0;aC=r+(aE*5|0)+2|0;a[aW]=a[aC]|0;a[aC]=aD;aD=aY+643|0;aC=a[aD]|0;aW=r+(aE*5|0)+3|0;a[aD]=a[aW]|0;a[aW]=aC;aC=aY+644|0;aW=a[aC]|0;aD=r+(aE*5|0)+4|0;a[aC]=a[aD]|0;a[aD]=aW;aW=aY+492|0;aD=c[aW>>2]|0;aC=s+(aE<<2)|0;c[aW>>2]=c[aC>>2];c[aC>>2]=aD;aD=aY+628|0;aC=c[aD>>2]|0;aX=t+(aE<<2)|0;c[aD>>2]=c[aX>>2];c[aX>>2]=aC;IY(u+(aE*20|0)|0,aY+5956|0,20);IY(v+(aE*20|0)|0,aY+5976|0,20);if((c[aZ>>2]|0)==0){c[aW>>2]=0;c[aD>>2]=0}aD=aE+1|0;if((aD|0)>=(A|0)){break L16764}aE=aD;aY=c[p+(aD<<2)>>2]|0}}}while(0);aY=c[E>>2]|0;aE=a0+5956|0;LJ(l+44|0,aY+28|0,a0+488|0,aE,1,c[aY>>2]|0,0,c[bb>>2]|0);aY=c[b>>2]|0;LJ(l+592|0,aY+28|0,h+488|0,h+5956|0,1,c[aY>>2]|0,1,c[bb>>2]|0);aY=a[y]|0;if(aY<<24>>24!=0){aZ=0;a8=0;aD=aY;while(1){aY=a0+5936+(a8<<2)|0;if((c[aY>>2]|0)>0){aW=aZ;aC=0;do{c[a0+4792+(a8*192|0)+(aC<<2)>>2]=a[l+44+aW|0]|0;c[h+4792+(a8*192|0)+(aC<<2)>>2]=a[l+592+aW|0]|0;aW=aW+1|0;aC=aC+1|0;}while((aC|0)<(c[aY>>2]|0));bg=aW;bh=a[y]|0}else{bg=aZ;bh=aD}aY=a8+1|0;if((aY|0)<(bh&255|0)){aZ=bg;a8=aY;aD=bh}else{break}}}aD=l+524|0;LJ(aD,ba,a0+624|0,a0+5976|0,1,(c[c[E>>2]>>2]|0)>1?2:1,0,c[bb>>2]|0);a[a0+5996|0]=a[aD]|0;a[a0+5997|0]=a[l+525|0]|0;a[a0+5998|0]=a[l+526|0]|0;a[a0+5999|0]=a[l+527|0]|0;a[a0+6e3|0]=a[l+528|0]|0;a[a0+6001|0]=a[l+529|0]|0;a[a0+6002|0]=a[l+530|0]|0;a[a0+6003|0]=a[l+531|0]|0;a[a0+6004|0]=a[l+532|0]|0;a[a0+6005|0]=a[l+533|0]|0;aD=l+1072|0;LJ(aD,ba,j,a7,1,(c[c[b>>2]>>2]|0)>1?2:1,1,c[bb>>2]|0);a[a6]=a[aD]|0;a[a9]=a[l+1073|0]|0;a[a5]=a[l+1074|0]|0;a[a4]=a[l+1075|0]|0;a[a3]=a[l+1076|0]|0;a[a2]=a[l+1077|0]|0;a[n]=a[l+1078|0]|0;a[az]=a[l+1079|0]|0;a[aJ]=a[l+1080|0]|0;a[a1]=a[l+1081|0]|0;c[aI>>2]=1;c[aH>>2]=0;c[a$>>2]=1;aD=a0+5864|0;a8=c[aD>>2]|0;aZ=h+5864|0;y=c[aZ>>2]|0;aY=H8(e,f,g,aG,a_,k,c[B>>2]|0)|0;c[aD>>2]=a8;c[aZ>>2]=y;L16785:do{if((aY|0)<(aF|0)){if(z){bi=0}else{bj=1;break}while(1){IY(l+(bi*548|0)+284|0,l+(bi*548|0)+44|0,240);IY(l+(bi*548|0)+534|0,l+(bi*548|0)+524|0,10);y=bi+1|0;if((y|0)<(A|0)){bi=y}else{bj=1;break}}}else{if(z){bk=0;bl=a0}else{bj=0;break}while(1){IY(bl+504|0,q+(bk*48|0)|0,48);c[bl+492>>2]=c[s+(bk<<2)>>2];IY(bl+640|0,r+(bk*5|0)|0,5);IY(bl+5956|0,u+(bk*20|0)|0,20);IY(bl+5976|0,v+(bk*20|0)|0,20);c[bl+628>>2]=c[t+(bk<<2)>>2];aW=bk+1|0;if((aW|0)>=(A|0)){bj=0;break L16785}bk=aW;bl=c[p+(aW<<2)>>2]|0}}}while(0);c[aI>>2]=bj;c[aH>>2]=0;c[a$>>2]=bj;if((c[aE>>2]|0)==1){bm=a0;S=11956;break}else{bn=a0;bo=h;S=11955;break}}else if((K|0)==0){aF=e+64|0;c[aF>>2]=0;c[ao+6048>>2]=0;aY=c[E>>2]|0;a_=ao+5956|0;LJ(l+284|0,aY+28|0,ao+488|0,a_,c[aF>>2]|0,c[aY>>2]|0,0,c[g+12>>2]|0);aY=ao+564|0;if((c[a_>>2]|0)==1){c[aY>>2]=(c[aY>>2]|0)+1;break}else{c[aY>>2]=0;break}}else if((K|0)==1){aY=e+64|0;c[aY>>2]=0;c[ao+6048>>2]=0;c[ap+6048>>2]=0;a_=c[E>>2]|0;aF=c[w>>2]|0;aG=g+12|0;LJ(l+284|0,a_+28|0,aF+488|0,aF+5956|0,c[aY>>2]|0,c[a_>>2]|0,0,c[aG>>2]|0);a_=c[l+548>>2]|0;a1=c[x>>2]|0;LJ(l+832|0,a_+28|0,a1+488|0,a1+5956|0,c[aY>>2]|0,c[a_>>2]|0,0,c[aG>>2]|0);bc=aF;bd=a1;S=11951;break}else{bc=ao;bd=ap;S=11951;break}}}while(0);if((S|0)==11951){if((c[bc+5956>>2]|0)==1){bm=bc;S=11956}else{bn=bc;bo=bd;S=11955}}do{if((S|0)==11955){if((c[bo+5956>>2]|0)==1){bm=bn;S=11956;break}c[bn+564>>2]=0;c[(c[x>>2]|0)+564>>2]=0}}while(0);if((S|0)==11956){bn=bm+564|0;c[bn>>2]=(c[bn>>2]|0)+1;bn=(c[x>>2]|0)+564|0;c[bn>>2]=(c[bn>>2]|0)+1}do{if(z){bn=0;do{bm=l+(bn*548|0)+547|0;bo=a[bm]|0;bd=c[p+(bn<<2)>>2]|0;if(bo<<24>>24!=0){bc=0;ap=0;ao=bo;while(1){bo=bd+5936+(ap<<2)|0;if((c[bo>>2]|0)>0){E=bc;K=0;do{c[bd+4792+(ap*192|0)+(K<<2)>>2]=a[l+(bn*548|0)+284+E|0]|0;E=E+1|0;K=K+1|0;}while((K|0)<(c[bo>>2]|0));bp=E;bq=a[bm]|0}else{bp=bc;bq=ao}bo=ap+1|0;if((bo|0)<(bq&255|0)){bc=bp;ap=bo;ao=bq}else{break}}}a[bd+5996|0]=a[l+(bn*548|0)+534|0]|0;a[bd+5997|0]=a[l+(bn*548|0)+535|0]|0;a[bd+5998|0]=a[l+(bn*548|0)+536|0]|0;a[bd+5999|0]=a[l+(bn*548|0)+537|0]|0;a[bd+6e3|0]=a[l+(bn*548|0)+538|0]|0;a[bd+6001|0]=a[l+(bn*548|0)+539|0]|0;a[bd+6002|0]=a[l+(bn*548|0)+540|0]|0;a[bd+6003|0]=a[l+(bn*548|0)+541|0]|0;a[bd+6004|0]=a[l+(bn*548|0)+542|0]|0;a[bd+6005|0]=a[l+(bn*548|0)+543|0]|0;bn=bn+1|0;}while((bn|0)<(A|0));bn=c[w>>2]|0;h=bn+4736|0;if((A|0)!=2){br=bn;bs=h;S=11968;break}H7(e,f,g,h,(c[x>>2]|0)+4736|0,k,c[B>>2]|0)|0;bt=bn}else{bn=c[w>>2]|0;br=bn;bs=bn+4736|0;S=11968}}while(0);if((S|0)==11968){S=c[B>>2]|0;H5(e,f,g,bs,k,S)|0;bt=br}if(z){bu=1;bv=bt}else{bw=e+64|0;bx=c[bw>>2]|0;by=e+68|0;c[by>>2]=bx;i=o;return}while(1){bt=c[bv+1220>>2]>>c[bv+1212>>2];z=bv+1208|0;if((c[z>>2]|0)>0){br=0;do{IY(c[bv+1020+(br<<2)>>2]|0,c[bv+1020+(br+bt<<2)>>2]|0,256);br=br+1|0;}while((br|0)<(c[z>>2]|0))}c[bv+1148>>2]=c[bv+1152>>2];if((bu|0)>=(A|0)){break}z=c[p+(bu<<2)>>2]|0;bu=bu+1|0;bv=z}bw=e+64|0;bx=c[bw>>2]|0;by=e+68|0;c[by>>2]=bx;i=o;return}function LP(b,e,f,g,h,j,k,l,m,n,o,p,q){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;var r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0;r=i;i=i+528|0;s=r|0;t=r+8|0;u=r+16|0;v=r+144|0;w=r+272|0;x=r+400|0;y=n+6052|0;z=c[y>>2]|0;A=2-z|0;B=c[j>>2]|0;C=(c[j+48>>2]|0)-1|0;D=c[n+1236>>2]|0;E=(f|0)<(g|0)?f:g;F=(o|0)==2;if(F){o=c[h>>2]|0;H=(E|0)<(o|0)?E:o;I=c[h+4>>2]|0;h=((H|0)<(I|0)?H:I)-7|0;c[p>>2]=0;J=o-h|0;K=I-h|0;L=h}else{J=0;K=0;L=E-7|0}E=f-L|0;f=g-L|0;if((B|0)<=0){i=r;return}g=(D|0)<2?2:D;h=m+68|0;I=n+6348|0;o=m+36|0;H=24-((z|0)==0)|0;z=H-1|0;M=n+5864|0;N=n+1208|0;O=m+48|0;P=m+44|0;Q=0;R=0;while(1){S=ac(c[j+4+(R<<2)>>2]|0,D)|0;T=R+1|0;U=ac(c[j+4+(T<<2)>>2]|0,D)|0;V=c[j+28+(R<<2)>>2]|0;W=c[m+8+(V<<2)>>2]|0;if((W|0)>-1){X=~W;Y=0;while(1){Z=Y+1|0;_=X<<1;if((_|0)<0){X=_;Y=Z}else{$=Z;break}}}else{$=0}Y=32-$|0;X=U-((R|0)==(C|0)?g:0)|0;Z=(W|0)>0;do{if(Z){_=m+44+(V<<2)|0;aa=(V|0)==1;ab=X-S|0;ad=0;ae=0;af=0;while(1){ag=c[_>>2]|0;ah=d[ag+af|0]|0;ai=af+1|0;aj=d[ag+ai|0]|0;ag=(af|0)==0;do{if(aa){if(!ag){ak=ah;break}ak=((aj-ah|0)>1)+ah|0}else{if(!ag){ak=ah;break}ak=((aj-ah|0)>2)+ah|0}}while(0);ah=w+(af<<2)|0;c[ah>>2]=0;L16860:do{if((c[M>>2]|0)==0){ag=c[N>>2]<<q;al=(X|0)<(ag|0)?X:ag;am=12152}else{do{if(aa){if((a[n+5868+af|0]|0)==0){am=12131;break}c[ah>>2]=1;ag=c[N>>2]<<q;an=(X|0)<(ag|0)?X:ag}else{ag=c[O>>2]|0;ao=c[P>>2]|0;ap=a[ao+af|0]|0;aq=0;while(1){if((d[ag+aq|0]|0)<(ap&255)){aq=aq+1|0}else{break}}ap=a[ao+ai|0]|0;ar=0;while(1){if((d[ag+ar|0]|0)<(ap&255)){ar=ar+1|0}else{break}}if((aq|0)<(ar|0)){as=aq;at=0}else{am=12131;break}while(1){if((a[n+5868+as|0]|0)==0){au=at}else{c[ah>>2]=1;au=1}ap=as+1|0;if((ap|0)<(ar|0)){as=ap;at=au}else{break}}ar=c[N>>2]<<q;aq=(X|0)<(ar|0)?X:ar;if((au|0)==0){al=aq;am=12152;break L16860}else{an=aq}}}while(0);if((am|0)==12131){am=0;aq=c[N>>2]<<q;al=(X|0)<(aq|0)?X:aq;am=12152;break}c[x+(af<<2)>>2]=ab;aq=(ak|0)<(aj|0);if(aq){ar=0;ap=ak;while(1){ag=ap+1|0;ao=LW(ap,ag,S,X,an,b,q,E,f)|0;av=(ar|0)>(ao|0)?ar:ao;if((ag|0)<(aj|0)){ar=av;ap=ag}else{aw=av;break}}}else{aw=0}ap=aj-ak|0;ar=(ap|0)>2;if(ar){do{if((aw|0)==0){ax=0}else{av=aw>>31^aw;if((av|0)<=-1){ax=-1;break}ag=~av;av=0;while(1){ao=ag<<1;if((ao|0)<0){ag=ao;av=av+1|0}else{ax=av;break}}}}while(0);av=aw<<ax;ag=av;ao=(av|0)<0|0?-1:0;av=854928832;ay=0;N2(ag,ao,av,ay)|0;az=(G<<1|0>>>31)>>ax}else{az=aw>>((ap|0)>1)}if(!F){aA=0;aB=az;break}if(aq){ay=0;av=ak;while(1){ao=av+1|0;ag=LW(av,ao,S,X,an,e,q,J,K)|0;aC=(ay|0)>(ag|0)?ay:ag;if((ao|0)<(aj|0)){ay=aC;av=ao}else{aD=aC;break}}}else{aD=0}if(!ar){aA=aD>>((ap|0)>1);aB=az;break}do{if((aD|0)==0){aE=0}else{av=aD>>31^aD;if((av|0)<=-1){aE=-1;break}ay=~av;av=0;while(1){aq=ay<<1;if((aq|0)<0){ay=aq;av=av+1|0}else{aE=av;break}}}}while(0);ap=aD<<aE;ar=ap;av=(ap|0)<0|0?-1:0;ap=854928832;ay=0;N2(ar,av,ap,ay)|0;aA=(G<<1|0>>>31)>>aE;aB=az}}while(0);do{if((am|0)==12152){am=0;c[x+(af<<2)>>2]=ac(aj-ak|0,ab)|0;ah=LW(ak,aj,S,X,al,b,q,E,f)|0;if(!F){aA=0;aB=ah;break}aA=LW(ak,aj,S,X,al,e,q,J,K)|0;aB=ah}}while(0);c[u+(af<<2)>>2]=aB;c[v+(af<<2)>>2]=aA;aF=(aB>>Y)+ae|0;aG=(aA>>Y)+ad|0;if((ai|0)<(W|0)){ad=aG;ae=aF;af=ai}else{break}}if(!Z){aH=Q;break}af=172720+(W+1<<2)|0;ae=Q;ad=0;while(1){ab=c[u+(ad<<2)>>2]|0;aa=c[v+(ad<<2)>>2]|0;do{if((c[w+(ad<<2)>>2]|0)==0){if((a[I]|0)==0){aI=aa;aJ=ab;am=12172;break}do{if((ab|0)>0){c[s>>2]=0;_=HQ(aF,ab,s)|0;aj=c[af>>2]|0;ah=_;ay=(_|0)<0|0?-1:0;_=aj;ap=(aj|0)<0|0?-1:0;N2(_,ap,ah,ay)|0;ay=G<<1|0>>>31;ah=(c[s>>2]|0)+Y|0;c[s>>2]=ah;if((ah|0)>-1&(ay|0)>(2147483647>>>(ah>>>0)|0)){aK=ab;break}if((ah|0)>0){aL=ay<<ah}else{aL=ay>>-ah}ah=aL;ay=(aL|0)<0|0?-1:0;ap=ab;_=(ab|0)<0|0?-1:0;N2(ah,ay,ap,_)|0;aK=G<<1|0>>>31}else{aK=ab}}while(0);if(!F){aM=aK;aN=0;aO=aa;break}if((aa|0)<=0){aP=aK;aQ=aa;am=12173;break}c[t>>2]=0;_=HQ(aG,aa,t)|0;ap=c[af>>2]|0;ay=_;ah=(_|0)<0|0?-1:0;_=ap;aj=(ap|0)<0|0?-1:0;N2(_,aj,ay,ah)|0;ah=G<<1|0>>>31;ay=(c[t>>2]|0)+Y|0;c[t>>2]=ay;if((ay|0)>-1&(ah|0)>(2147483647>>>(ay>>>0)|0)){aI=aa;aJ=aK;am=12172;break}if((ay|0)>0){aR=ah<<ay}else{aR=ah>>-ay}ay=aR;ah=(aR|0)<0|0?-1:0;aj=aa;_=(aa|0)<0|0?-1:0;N2(ay,ah,aj,_)|0;aI=G<<1|0>>>31;aJ=aK;am=12172}else{aI=aa;aJ=ab;am=12172}}while(0);if((am|0)==12172){am=0;if(F){aP=aJ;aQ=aI;am=12173}else{aM=aJ;aN=0;aO=aI}}if((am|0)==12173){am=0;aM=aQ+aP>>1;aN=aP;aO=aQ}if((aM|0)>0){ab=aM>>31^aM;if((ab|0)>-1){aa=~ab;ab=0;while(1){ai=aa<<1;if((ai|0)<0){aa=ai;ab=ab+1|0}else{aS=ab;break}}}else{aS=-1}ab=HE(aM<<aS)|0;aT=486539264-(aS+L<<24)+(ab-(HE(ac(c[x+(ad<<2)>>2]<<16,c[o>>2]|0)|0)|0)>>1)|0}else{aT=-2147483648}ab=(aT|0)>0?aT:0;a[k+ae|0]=((((ab|0)<1073741824?ab:1073741824)>>z)+1|0)>>>1&255;if(F){if((aO|0)==0){aU=1073741824}else{do{if((aN|0)==0){aV=0}else{ab=aN>>31^aN;if((ab|0)<=-1){aV=-1;break}aa=~ab;ab=0;while(1){ai=aa<<1;if((ai|0)<0){aa=ai;ab=ab+1|0}else{aV=ab;break}}}}while(0);ab=aO>>31^aO;if((ab|0)>-1){aa=~ab;ab=0;while(1){ai=aa<<1;if((ai|0)<0){aa=ai;ab=ab+1|0}else{aW=ab;break}}}else{aW=-1}ab=HE(aN<<aV)|0;aU=ab-(HE(aO<<aW)|0)-(aV-aW<<25)|0}ab=(aU>>H)+1>>1;aa=c[y>>2]|0;ai=(ab|0)>0?1:-1;_=ac(ai,ab)|0;ab=d[1914632+aa|0]|0;if(aa>>>0<2){aj=2147483647;ah=0;ay=0;do{ap=_-(d[1914608+(aa*10|0)+ah|0]|0)|0;av=(ap|0)>0?ap:-ap|0;ap=(av|0)<(aj|0);aj=ap?av:aj;ay=ap?ah&255:ay;ah=ah+1|0;}while((ah|0)<(ab|0));aX=aj;aY=ay&255}else{aX=2147483647;aY=0}ah=d[ab-1+(1914608+(aa*10|0))|0]|0;a[l+ae|0]=(ac(d[1914608+(aa*10|0)+aY|0]|0,ai)|0)+ah&255;ah=c[p>>2]|0;c[p>>2]=(aX|0)>(ah|0)?aX:ah}ah=ad+1|0;if((ah|0)<(W|0)){ae=ae+1|0;ad=ah}else{break}}aH=Q+W|0}else{aH=Q}}while(0);do{if((c[h>>2]|0)==0){aZ=aH}else{Y=aH-W|0;if(!Z){aZ=Y;break}X=(V|0)==1;S=Y;Y=0;while(1){do{if(X){U=a[n+1156+Y|0]|0;if(U<<24>>24==0){break}ad=ac(U&255,A)|0;U=k+S|0;a[U]=(d[U]|0)-ad&255}}while(0);ai=k+S|0;aa=a[ai]|0;a[ai]=aa<<24>>24<0?0:aa;aa=Y+1|0;if((aa|0)<(W|0)){S=S+1|0;Y=aa}else{aZ=aH;break}}}}while(0);if((T|0)<(B|0)){Q=aZ;R=T}else{break}}i=r;return}function LQ(b,e,f){b=b|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;g=(f|0)==0;f=0;do{h=c[e+(f<<2)>>2]|0;if((h|0)>1006632960){i=30}else{j=h>>25;i=(j|0)==0?0:j+1|0}if(g){k=i&255}else{j=(i|0)<-30?-30:i;h=(j|0)>0?1:-1;l=ac(h,j)|0;j=(l|0)>0?l:-l|0;m=l-2|0;n=(m|0)>0?m:2-l|0;m=(n|0)<(j|0);o=m?n:j;j=l-4|0;n=(j|0)>0?j:4-l|0;j=(n|0)<(o|0);p=j?n:o;o=l-8|0;n=(o|0)>0?o:8-l|0;o=(n|0)<(p|0);q=l-12|0;k=(ac(d[1914618+((((q|0)>0?q:12-l|0)|0)<((o?n:p)|0)?4:o?3:j?2:m&1)|0]|0,h)|0)+12&255}a[b+f|0]=k;f=f+1|0;}while((f|0)<10);return}function LR(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0;d=0;do{e=a+(d<<2)|0;f=201326592-(c[e>>2]|0)|0;g=b+(d<<2)|0;h=201326592-(c[g>>2]|0)|0;i=(h|0)<0;j=HH(h)|0;k=i?j:j<<24;j=(f|0)<0;l=HH(201326592-(c[e>>2]|0)|0)|0;m=j?l:l<<24;l=i^1;if(!(j^1|l)){c[e>>2]=201326592-(HE((m>>1)+(k>>1)|0)|0);i=HE(m)|0;c[g>>2]=i-(HE(k)|0)}if((h|f|0)>-1){c[e>>2]=-33554432-(HE((m>>1)+(k>>1)|0)|0);f=HE(m)|0;c[g>>2]=f-(HE(k)|0)}if(!(j|l)){c[e>>2]=-33554432-(HE((m>>1)+(k>>8)|0)|0);l=(HE(m)|0)+234881024|0;c[g>>2]=l-(HE(k)|0)}if(j&(h|0)>-1){c[e>>2]=-33554432-(HE((m>>8)+(k>>1)|0)|0);e=HE(m)|0;c[g>>2]=e-234881024-(HE(k)|0)}d=d+1|0;}while((d|0)<10);return}function LS(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0;f=JC(b)|0;I_(a,480);c[a+256>>2]=f;c[a+260>>2]=f;c[a+264>>2]=f+256;c[a+268>>2]=f+512;c[a+272>>2]=f+768;c[a+276>>2]=f+1024;c[a+280>>2]=f+1280;c[a+284>>2]=f+1536;c[a+288>>2]=f+1792;c[a+292>>2]=f+2048;c[a+296>>2]=f+2304;c[a+300>>2]=f+2560;c[a+304>>2]=f+2816;c[a+308>>2]=f+3072;c[a+312>>2]=f+3328;c[a+316>>2]=f+3584;c[a+320>>2]=f+3840;f=JE(d,e)|0;c[a+324>>2]=f;c[a+328>>2]=f+256;c[a+332>>2]=f+512;c[a+336>>2]=f+768;c[a+340>>2]=f+1024;c[a+344>>2]=f+1280;c[a+348>>2]=f+1536;c[a+352>>2]=f+1792;c[a+356>>2]=f+2048;c[a+360>>2]=f+2304;c[a+364>>2]=f+2560;c[a+368>>2]=f+2816;c[a+372>>2]=f+3072;c[a+376>>2]=f+3328;c[a+380>>2]=f+3584;c[a+384>>2]=f+3840;f=JF(0,e)|0;d=JG(0,e)|0;e=0;do{b=e<<6;c[a+(e<<2)>>2]=f+(b<<2);c[a+128+(e<<2)>>2]=d+(b<<2);e=e+1|0;}while((e|0)<32);return 0}function LT(b,d,e,f,g,h,i,j,k,l,m){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0;if((m&1|0)==0){m=ac(i,h)|0;c[b+448>>2]=m;n=m}else{m=d>>1;c[b+448>>2]=m;n=m}c[b+456>>2]=0;a[b+444|0]=0;a[b+445|0]=0;c[b+460>>2]=d;c[b+464>>2]=e;c[b+468>>2]=f;c[b+472>>2]=g;c[b+476>>2]=h;g=(h|0)>1|0;c[b+452>>2]=g;c[b+448>>2]=n>>g;h=JE(k,l)|0;c[b+324>>2]=h;c[b+328>>2]=h+256;c[b+332>>2]=h+512;c[b+336>>2]=h+768;c[b+340>>2]=h+1024;c[b+344>>2]=h+1280;c[b+348>>2]=h+1536;c[b+352>>2]=h+1792;c[b+356>>2]=h+2048;c[b+360>>2]=h+2304;c[b+364>>2]=h+2560;c[b+368>>2]=h+2816;c[b+372>>2]=h+3072;c[b+376>>2]=h+3328;c[b+380>>2]=h+3584;c[b+384>>2]=h+3840;h=n+d>>g;g=(j|0)!=0;if(g&(h|0)>0){j=0;do{I_(c[b+260+(j<<2)>>2]|0,256);j=j+1|0;}while((j|0)<(h|0))}if((d|0)>0){h=0;do{I_(c[b+(h<<2)>>2]|0,256);I_(c[b+128+(h<<2)>>2]|0,256);h=h+1|0;}while((h|0)<(d|0))}I_(b+396|0,48);if(!g){return 0}c[b+392>>2]=15;c[b+388>>2]=15;return 0}function LU(a){a=a|0;if((a|0)==0){return}JD(a+256|0);return}function LV(a){a=a|0;return ac((c[a+448>>2]<<1)-(c[a+456>>2]|0)|0,c[a+464>>2]|0)|0}function LW(a,b,d,e,f,g,h,i,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;if((b|0)==(a|0)){k=31}else{k=(HK(b-a|0)|0)>>25}l=(i|0)<5?i:5;m=(j|0)<5?j:5;n=i-l|0;i=(n|0)<(k|0)?n:k;o=j-m|0;j=(o|0)<(k|0)?o:k;L17041:do{if((a|0)<(b|0)){if((d|0)<(f|0)){p=0;q=0;r=a}else{k=0;s=a;while(1){if((d|0)<(e|0)){t=0;u=d;while(1){v=(c[(c[g+(u>>h<<2)>>2]|0)+(s<<2)>>2]>>m)+t|0;w=u+1|0;if((w|0)<(e|0)){t=v;u=w}else{x=v;break}}}else{x=0}u=(x>>j)+k|0;t=s+1|0;if((t|0)<(b|0)){k=u;s=t}else{y=0;z=u;break L17041}}}while(1){s=0;k=d;do{s=(c[(c[g+(k>>h<<2)>>2]|0)+(r<<2)>>2]>>l)+s|0;k=k+1|0;}while((k|0)<(f|0));if((f|0)<(e|0)){k=0;u=f;while(1){t=(c[(c[g+(u>>h<<2)>>2]|0)+(r<<2)>>2]>>m)+k|0;v=u+1|0;if((v|0)<(e|0)){k=t;u=v}else{A=t;break}}}else{A=0}u=(s>>i)+p|0;k=(A>>j)+q|0;t=r+1|0;if((t|0)<(b|0)){p=u;q=k;r=t}else{y=u;z=k;break}}}else{y=0;z=0}}while(0);r=n-i|0;i=o-j|0;return(z>>((i|0)<31?i:31))+(y>>((r|0)<31?r:31))|0}function LX(b,e,f,g,h){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0;i=b+44|0;j=b+92|0;k=b+140|0;l=b+172|0;m=b+160|0;n=b+192|0;o=b+88|0;p=b+136|0;q=b+40|0;r=b+164|0;s=b+168|0;t=b+36|0;u=c[b+32>>2]|0;v=c[b+8>>2]|0;w=c[b+4>>2]|0;x=c[b+24>>2]|0;y=c[b+28>>2]|0;z=b+196|0;A=z|0;B=c[A>>2]|0;C=b+200|0;D=c[C>>2]|0;E=c[b>>2]|0;F=d[e]|0;G=d[e+1|0]|0;e=h+12|0;c[b+16>>2]=h;H=c[b+12>>2]|0;I=(g|0)!=0;do{if(I){if((a[f+1|0]|0)==0){J=G;K=F;break}g=(D-(d[f]|0)|0)<4;J=g?1:G;K=g?0:F}else{J=G;K=F}}while(0);L17065:do{if((w|0)==0){L17068:do{switch(c[t>>2]|0){case 1:{if((J|0)==0){L=(c[q>>2]|0)==0?2:3;M=12272;break L17068}else{c[q>>2]=0;N=3;M=12273;break L17068}break};case 2:{if((J|0)==0){L=0;M=12272}else{N=1;M=12273}break};case 3:{if((J|0)==0){L=(c[q>>2]|0)==0?2:3;M=12272;break L17068}else{c[q>>2]=0;N=3;M=12273;break L17068}break};case 4:case 0:{if((J|0)==0){L=0;M=12272}else{N=1;M=12273}break};default:{N=0;M=12273}}}while(0);do{if((M|0)==12272){c[t>>2]=L;O=0;P=0;Q=0;R=0;S=L}else if((M|0)==12273){c[t>>2]=N;F=(J|0)!=0;if(F&I){c[t>>2]=0;T=4}else{if(F){T=N}else{O=0;P=0;Q=0;R=0;S=N;break}}F=E+B+K|0;c[o>>2]=0;c[p>>2]=0;G=c[h>>2]|0;if((G|0)!=0){Mr(i,o,F-G|0);Mr(j,p,c[e>>2]|0)}Mr(i,o,F);G=c[h+4>>2]|0;if((G|0)==0){U=F}else{f=G+F|0;Mr(i,o,f);Mr(j,p,c[h+16>>2]|0);U=f}f=c[h+8>>2]|0;if((f|0)!=0){Mr(i,o,f+U|0);Mr(j,p,c[h+20>>2]|0)}Mr(j,p,1);f=c[i>>2]|0;G=c[o>>2]|0;L17095:do{if((G|0)>0){g=1;V=f;W=f;while(1){X=(W|0)<(V|0)?W:V;if((g|0)>=(G|0)){Y=1;Z=f;_=f;break}$=c[b+44+(g<<2)>>2]|0;g=g+1|0;V=X;W=$}while(1){W=(_|0)>(Z|0)?_:Z;if((Y|0)>=(G|0)){aa=W;ab=X;break L17095}V=c[b+44+(Y<<2)>>2]|0;Y=Y+1|0;Z=W;_=V}}else{aa=f;ab=f}}while(0);if((D|0)==15){if((K|0)<4){O=F;P=5;Q=aa;R=ab;S=T;break}O=F;P=(K-4|0)>>>0<2?3:7;Q=aa;R=ab;S=T;break}else if((D|0)==16){if((K|0)<4){O=F;P=6;Q=aa;R=ab;S=T;break}O=F;P=(K-4|0)>>>0<2?4:8;Q=aa;R=ab;S=T;break}else{O=F;P=8;Q=aa;R=ab;S=T;break}}}while(0);switch(S|0){case 1:{f=R-B|0;do{if((f|0)>(y|0)){G=1;while(1){V=G+1|0;W=(((f|0)/(V|0)|0)-2>>1<<1)+2|0;ad=(W|0)>8?8:W;if((f-(ac(ad,G)|0)|0)>(y|0)){G=V}else{break}}F=G-1|0;if((F|0)<0){break}else{ae=0;af=R}do{af=af-ad|0;Mq(i,o,af);Mq(j,p,1);ae=ae+1|0;}while((ae|0)<=(F|0))}}while(0);f=B-Q+(D<<1)|0;do{if((f|0)>0){if((f|0)>(y|0)){ag=1}else{ah=f;M=12314;break}while(1){ai=ag+1|0;F=(((f|0)/(ai|0)|0)-2>>1<<1)+2|0;aj=(F|0)>(P|0)?P:F;ak=f-(ac(aj,ag)|0)|0;if((ak|0)>(y|0)){ag=ai}else{break}}F=ag-1|0;if((F|0)>=0){G=0;V=Q;do{V=V+aj|0;Mr(i,o,V);Mr(j,p,1);G=G+1|0;}while((G|0)<=(F|0))}if((ag|0)==0){ah=ak;M=12314}else{al=ai}}else{c[o>>2]=(c[o>>2]|0)-1;c[p>>2]=(c[p>>2]|0)-1;ah=f;M=12314}}while(0);do{if((M|0)==12314){if((ah|0)>=(x|0)){al=1;break}f=c[o>>2]|0;if((u|0)!=0){c[q>>2]=1;Mr(i,o,(c[b+44+(f-1<<2)>>2]|0)+8|0);Mr(j,p,1);al=2;break}F=f-1|0;if((ah|0)==1){c[o>>2]=F;c[p>>2]=F;al=1;break}G=b+44+(F<<2)|0;F=c[G>>2]|0;if((F-(c[b+44+(f-2<<2)>>2]|0)|0)<=2){al=1;break}c[G>>2]=F-2;c[b+92+((c[p>>2]|0)-1<<2)>>2]=0;al=1}}while(0);F=c[o>>2]|0;G=D+B|0;f=0;while(1){if((f|0)>=(F|0)){am=0;break}if((c[b+44+(f<<2)>>2]|0)<(G|0)){f=f+1|0}else{am=f;break}}L17143:do{if((F|0)>0){f=0;while(1){if((c[b+44+(f<<2)>>2]|0)>=(O|0)){an=f;break L17143}G=f+1|0;if((G|0)<(F|0)){f=G}else{an=-99;break}}}else{an=0}}while(0);c[m>>2]=0;c[n>>2]=0;if((am|0)<(F|0)){f=am;G=0;while(1){c[b+140+(G<<2)>>2]=(c[b+44+(f<<2)>>2]|0)-D;c[b+172+(G<<2)>>2]=c[b+92+(f<<2)>>2];c[m>>2]=(c[m>>2]|0)+1;c[n>>2]=(c[n>>2]|0)+1;V=f+1|0;if((V|0)<(F|0)){f=V;G=G+1|0}else{break}}ao=c[o>>2]|0}else{ao=F}c[r>>2]=(an|0)==-99?-99:an-am|0;c[s>>2]=1-al-am+F;LY(z,1,i,ao,j,c[p>>2]|0,am,an,c[q>>2]|0,4711);break L17065;break};case 2:{LY(z,2,k,c[m>>2]|0,l,c[n>>2]|0,4711,c[r>>2]|0,c[q>>2]|0,4711);break L17065;break};case 4:{G=b+312|0;if((D|0)==16){ap=1922544;aq=1922824+(K*24|0)|0}else if((D|0)==8){ap=1924576;aq=1925504+(K*20|0)|0}else if((D|0)==15){ap=1922544;aq=1923208+(K*24|0)|0}else{ap=0;aq=0}f=c[aq>>2]|0;if((f|0)>1){V=1;do{c[b+316+(V<<2)>>2]=c[aq+(V+2<<2)>>2];V=V+1|0;}while((V|0)<(f|0))}V=b+316|0;c[V>>2]=0;c[b+316+(f<<2)>>2]=D;if((f|0)>0){F=0;W=c[V>>2]|0;while(1){V=F+1|0;g=c[b+316+(V<<2)>>2]|0;$=ap+(g-W<<2)|0;c[b+340+(F<<2)>>2]=c[$>>2];c[b+236+(F<<2)>>2]=c[$>>2];if((V|0)<(f|0)){F=V;W=g}else{break}}}c[G>>2]=f;c[b+360>>2]=c[aq+8>>2];W=c[aq+4>>2]|0;c[b+368>>2]=0;c[b+372>>2]=c[b+316+(((W|0)!=0?W:1)<<2)>>2];c[b+376>>2]=D;c[b+364>>2]=2;c[b+204>>2]=4;c[b+212>>2]=K;c[b+208>>2]=f;ar=G;return ar|0};case 3:{W=c[q>>2]|0;if((W|0)!=0){LY(z,3,k,c[m>>2]|0,l,c[n>>2]|0,4711,c[r>>2]|0,W,4711);c[q>>2]=0;c[k>>2]=(c[b+256>>2]|0)-D;c[l>>2]=1;c[m>>2]=1;c[n>>2]=1;c[r>>2]=-4711;c[s>>2]=-4711;break L17065}W=c[s>>2]|0;F=(W|0)>0;do{if((D|0)==9){if(F){c[m>>2]=W;c[n>>2]=W;as=W}else{as=c[m>>2]|0}g=c[b+140+(as-1<<2)>>2]|0;V=(R-g|0)<0;if((as|0)==1|V^1){at=as;au=g;av=V}else{V=as;while(1){g=V-1|0;$=c[b+140+(V-2<<2)>>2]|0;aw=(R-$|0)<0;if((g|0)==1|aw^1){at=g;au=$;av=aw;break}else{V=g}}}if(av){V=c[o>>2]|0;if((V|0)>0){g=0;aw=0;$=V;while(1){V=c[b+44+(g<<2)>>2]|0;if((V|0)>(au|0)){c[b+44+(aw<<2)>>2]=V;c[b+92+(aw<<2)>>2]=c[b+92+(g<<2)>>2];ax=aw+1|0;ay=c[o>>2]|0}else{ax=aw;ay=$}V=g+1|0;if((V|0)<(ay|0)){g=V;aw=ax;$=ay}else{az=V;break}}}else{az=0}c[o>>2]=az;c[p>>2]=az;aA=c[i>>2]|0}else{aA=R}c[m>>2]=at;c[n>>2]=at;$=at-1|0;aw=aA-au|0;if((aw|0)>(x|0)){g=(aw|0)>(y|0);if(!((aw|0)<(x|0)|g)){Mp(i,o,k,at);Mp(j,p,l,c[n>>2]|0);aB=$;break}do{if(g){V=1;while(1){aC=V+1|0;aD=(((aw|0)/(aC|0)|0)-2>>1<<1)+2|0;aE=(aD|0)>8?8:aD;if((aw-(ac(aE,V)|0)|0)>(y|0)){V=aC}else{break}}aC=V-1|0;if((aC|0)<0){aF=at;break}else{aG=0;aH=aA}do{aH=aH-aE|0;Mq(i,o,aH);Mq(j,p,1);aG=aG+1|0;}while((aG|0)<=(aC|0));aF=c[m>>2]|0}else{aF=at}}while(0);Mp(i,o,k,aF);Mp(j,p,l,c[n>>2]|0);aB=$;break}aw=c[o>>2]|0;if((c[h>>2]|0)!=0){g=aw-1|0;c[o>>2]=g;Mp(i,o,k,at);c[p>>2]=g;Mp(b+96|0,p,l,c[n>>2]|0);aB=$;break}if((at|0)>1){Mp(i,o,k,$);Mp(j,p,l,(c[m>>2]|0)-1|0);aB=at-2|0;break}g=aw-1|0;if((g|0)>0){aw=0;while(1){aC=aw+1|0;c[b+44+(aw<<2)>>2]=c[b+44+(aC<<2)>>2];if((aC|0)<((c[o>>2]|0)-1|0)){aw=aC}else{break}}}if(((c[p>>2]|0)-1|0)>0){aw=0;while(1){aC=aw+1|0;c[b+92+(aw<<2)>>2]=c[b+92+(aC<<2)>>2];if((aC|0)<((c[p>>2]|0)-1|0)){aw=aC}else{break}}}c[o>>2]=g;c[p>>2]=g;Mp(i,o,k,c[m>>2]|0);Mp(j,p,l,c[n>>2]|0);aB=$}else{if(F){c[m>>2]=W;c[n>>2]=W;aI=W}else{aI=c[m>>2]|0}aw=R-(c[b+140+(aI-1<<2)>>2]|0)|0;if((aw|0)<0){aC=aI;while(1){V=aC-1|0;aD=R-(c[b+140+(aC-2<<2)>>2]|0)|0;if((aD|0)<0){aC=V}else{aJ=V;aK=aD;break}}}else{aJ=aI;aK=aw}c[m>>2]=aJ;c[n>>2]=aJ;aC=aJ-1|0;$=c[o>>2]|0;if((aK|0)>(y|0)){g=1;while(1){aD=g+1|0;V=(((aK|0)/(aD|0)|0)-2>>1<<1)+2|0;aL=(V|0)>8?8:V;if((aK-(ac(aL,g)|0)|0)>(y|0)){g=aD}else{break}}aw=g-1|0;if((aw|0)<0){aM=aJ}else{aD=0;V=R;do{V=V-aL|0;Mq(i,o,V);Mq(j,p,1);aD=aD+1|0;}while((aD|0)<=(aw|0));aM=c[m>>2]|0}Mp(i,o,k,aM);Mp(j,p,l,c[n>>2]|0);aB=aC;break}if((aK|0)>=(x|0)){Mp(i,o,k,aJ);Mp(j,p,l,c[n>>2]|0);aB=aC;break}if((c[h>>2]|0)!=0){aw=$-1|0;c[o>>2]=aw;Mp(i,o,k,aJ);c[p>>2]=aw;Mp(b+96|0,p,l,c[n>>2]|0);aB=aC;break}if((aJ|0)>1){Mp(i,o,k,aC);Mp(j,p,l,(c[m>>2]|0)-1|0);aB=aJ-2|0;break}aw=$-1|0;if((aw|0)>0){aD=0;while(1){V=aD+1|0;c[b+44+(aD<<2)>>2]=c[b+44+(V<<2)>>2];if((V|0)<((c[o>>2]|0)-1|0)){aD=V}else{break}}}if(((c[p>>2]|0)-1|0)>0){aD=0;while(1){$=aD+1|0;c[b+92+(aD<<2)>>2]=c[b+92+($<<2)>>2];if(($|0)<((c[p>>2]|0)-1|0)){aD=$}else{break}}}c[o>>2]=aw;c[p>>2]=aw;Mp(i,o,k,c[m>>2]|0);Mp(j,p,l,c[n>>2]|0);aB=aC}}while(0);W=B-Q+(D<<1)|0;do{if((W|0)>0){if((W|0)>(y|0)){aN=1}else{aO=W;M=12393;break}while(1){aP=aN+1|0;F=(((W|0)/(aP|0)|0)-2>>1<<1)+2|0;aQ=(F|0)>(P|0)?P:F;aR=W-(ac(aQ,aN)|0)|0;if((aR|0)>(y|0)){aN=aP}else{break}}aC=aN-1|0;if((aC|0)>=0){aw=0;F=Q;do{F=F+aQ|0;Mr(i,o,F);Mr(j,p,1);aw=aw+1|0;}while((aw|0)<=(aC|0))}if((aN|0)==0){aO=aR;M=12393}else{aS=aP}}else{c[o>>2]=(c[o>>2]|0)-1;c[p>>2]=(c[p>>2]|0)-1;aO=W;M=12393}}while(0);do{if((M|0)==12393){if((aO|0)>=(x|0)){aS=1;break}W=c[o>>2]|0;if((u|0)!=0){c[q>>2]=1;Mr(i,o,(c[b+44+(W-1<<2)>>2]|0)+8|0);Mr(j,p,1);aS=2;break}aC=W-1|0;if((aO|0)==1){c[o>>2]=aC;c[p>>2]=aC;aS=1;break}aw=b+44+(aC<<2)|0;aC=c[aw>>2]|0;if((aC-(c[b+44+(W-2<<2)>>2]|0)|0)<=2){aS=1;break}c[aw>>2]=aC-2;c[b+92+((c[p>>2]|0)-1<<2)>>2]=0;aS=1}}while(0);aC=c[o>>2]|0;aw=D+B|0;W=0;while(1){if((W|0)>=(aC|0)){aT=0;break}if((c[b+44+(W<<2)>>2]|0)<(aw|0)){W=W+1|0}else{aT=W;break}}L17284:do{if((aC|0)>0){W=0;while(1){if((c[b+44+(W<<2)>>2]|0)>=(O|0)){aU=W;break L17284}aw=W+1|0;if((aw|0)<(aC|0)){W=aw}else{aU=-99;break}}}else{aU=0}}while(0);c[m>>2]=0;c[n>>2]=0;if((aT|0)<(aC|0)){W=aT;aw=0;while(1){c[b+140+(aw<<2)>>2]=(c[b+44+(W<<2)>>2]|0)-D;c[b+172+(aw<<2)>>2]=c[b+92+(W<<2)>>2];c[m>>2]=(c[m>>2]|0)+1;c[n>>2]=(c[n>>2]|0)+1;F=W+1|0;if((F|0)<(aC|0)){W=F;aw=aw+1|0}else{break}}aV=c[o>>2]|0}else{aV=aC}c[r>>2]=(aU|0)==-99?-99:aU-aT|0;c[s>>2]=1-aS-aT+aC;LY(z,3,i,aV,j,c[p>>2]|0,aT,aU,0,aB);break L17065;break};case 0:{c[b+208>>2]=(K|0)==0?1:2;c[b+204>>2]=0;break L17065;break};default:{break L17065}}}else{c[t>>2]=0;c[b+208>>2]=v;c[b+204>>2]=0}}while(0);v=b+312|0;t=b+236|0;K=c[b+204>>2]|0;aB=c[A>>2]|0;A=c[C>>2]|0;do{if((K|0)==0){C=c[b+208>>2]|0;L17298:do{if((C|0)==2){switch(A|0){case 15:{IY(v,1925048,68);break L17298;break};case 16:{IY(v,1924976,68);break L17298;break};case 9:{IY(v,1925120,68);break L17298;break};case 18:{IY(v,1924904,68);break L17298;break};case 8:{IY(v,1922680,68);break L17298;break};default:{break L17298}}}else if((C|0)==1){switch(A|0){case 8:{IY(v,1922752,68);break L17298;break};case 15:{IY(v,1925336,68);break L17298;break};case 16:{IY(v,1925264,68);break L17298;break};case 9:{IY(v,1925408,68);break L17298;break};case 18:{IY(v,1925192,68);break L17298;break};default:{break L17298}}}else if((C|0)==4){switch(A|0){case 15:{IY(v,1924760,68);break L17298;break};case 16:{IY(v,1924688,68);break L17298;break};case 9:{IY(v,1924832,68);break L17298;break};case 18:{IY(v,1924616,68);break L17298;break};case 8:{IY(v,1922608,68);break L17298;break};default:{break L17298}}}}while(0);do{if((H|0)==0){C=v|0;if((c[C>>2]|0)>0){aW=0}else{break}do{c[b+340+(aW<<2)>>2]=0;aW=aW+1|0;}while((aW|0)<(c[C>>2]|0))}}while(0);c[t>>2]=c[b+340>>2];aX=0;aY=0}else if((K|0)==1|(K|0)==2){C=c[b+216>>2]|0;aC=C+1|0;c[v>>2]=aC;aU=c[b+212>>2]|0;aT=b+364|0;if((C|0)==0){c[aT>>2]=1;aX=aC;aY=aU;break}else{c[aT>>2]=2;aX=aC;aY=aU;break}}else{aX=0;aY=0}}while(0);do{if((K|0)==2){c[b+316>>2]=aY;t=aX-1|0;if((t|0)>0){aW=0;H=aY;do{H=(c[b+224+(aW<<2)>>2]|0)+H|0;aW=aW+1|0;c[b+316+(aW<<2)>>2]=H;}while((aW|0)<(t|0))}c[b+316+(aX<<2)>>2]=A+aB;aW=c[b+220>>2]|0;if(aW>>>0<2){c[b+360>>2]=0}else{c[b+360>>2]=aW-1}if((aX|0)>0){H=0;do{c[b+340+(H<<2)>>2]=c[b+236+(H<<2)>>2];H=H+1|0;}while((H|0)<(aX|0))}if((aW|0)==0){c[b+372>>2]=c[b+320>>2];aZ=aX;break}else if((aW|0)==1){c[b+372>>2]=c[b+316+(t<<2)>>2];aZ=aX;break}else{c[b+372>>2]=c[b+316+(c[b+360>>2]<<2)>>2];aZ=aX;break}}else if((K|0)==1){c[b+316>>2]=aB;c[b+316+(aX<<2)>>2]=aY;H=aX-1|0;if((H|0)>0){aU=0;aC=H;aT=aY;while(1){C=aT-(c[b+224+(aU<<2)>>2]|0)|0;c[b+316+(aC<<2)>>2]=C;p=aU+1|0;if((p|0)<(H|0)){aU=p;aC=aC-1|0;aT=C}else{break}}}aT=c[b+220>>2]|0;if((aT|0)==0){c[b+360>>2]=0}else{c[b+360>>2]=aX+1-aT}if((aX|0)>0){aC=0;aU=H;while(1){c[b+340+(aU<<2)>>2]=c[b+236+(aC<<2)>>2];t=aC+1|0;if((t|0)<(aX|0)){aC=t;aU=aU-1|0}else{break}}}if(aT>>>0<2){c[b+372>>2]=c[b+316+(H<<2)>>2];aZ=aX;break}else{c[b+372>>2]=c[b+316+(c[b+360>>2]<<2)>>2];aZ=aX;break}}else if((K|0)==3){aU=b+260|0;aC=c[aU>>2]|0;t=b+264|0;aW=(c[t>>2]|0)+aC|0;C=aW+1|0;c[v>>2]=C;p=b+252|0;j=c[p>>2]|0;c[b+316>>2]=j;if((aC|0)>0){aC=0;aV=1;i=j;while(1){j=(c[b+268+(aC<<2)>>2]|0)+i|0;c[b+316+(aV<<2)>>2]=j;z=aC+1|0;if((z|0)<(c[aU>>2]|0)){aC=z;aV=aV+1|0;i=j}else{break}}}i=b+256|0;aV=c[i>>2]|0;c[b+316+(C<<2)>>2]=aV;if((c[t>>2]|0)>0){aC=0;aU=aW;H=aV;while(1){aV=H-(c[b+280+(aC<<2)>>2]|0)|0;c[b+316+(aU<<2)>>2]=aV;aT=aC+1|0;if((aT|0)<(c[t>>2]|0)){aC=aT;aU=aU-1|0;H=aV}else{break}}}H=c[b+220>>2]|0;if((H|0)==0){c[b+360>>2]=0}else{c[b+360>>2]=aW+2-H}if((C|0)>0){aU=0;do{c[b+340+(aU<<2)>>2]=c[b+292+(aU<<2)>>2];aU=aU+1|0;}while((aU|0)<(C|0))}aU=b+364|0;if((aW|0)==0){c[aU>>2]=1;c[b+368>>2]=c[p>>2];c[b+372>>2]=c[i>>2];aZ=C;break}c[aU>>2]=2;c[b+368>>2]=c[p>>2];if(H>>>0<2){c[b+372>>2]=c[b+316+(aW<<2)>>2]}else{c[b+372>>2]=c[b+316+(c[b+360>>2]<<2)>>2]}c[b+376>>2]=c[i>>2];aZ=C}else{aZ=aX}}while(0);if((K-1|0)>>>0>=2){ar=v;return ar|0}c[b+368>>2]=c[b+316>>2];K=c[b+316+(aZ<<2)>>2]|0;if((aZ|0)==1){c[b+372>>2]=K;ar=v;return ar|0}else{c[b+376>>2]=K;ar=v;return ar|0}return 0}function LY(a,b,d,e,f,g,h,j,k,l){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0;m=i;i=i+24|0;n=m|0;o=m+8|0;p=m+16|0;q=a+40|0;r=a+28|0;s=a+72|0;t=a+84|0;c[n>>2]=0;c[o>>2]=0;c[p>>2]=0;if((b|0)==2){u=c[d>>2]|0;c[n>>2]=0;if((e|0)>1){v=1;w=u;while(1){x=d+(v<<2)|0;Mr(r,n,(c[x>>2]|0)-w|0);y=v+1|0;if((y|0)>=(e|0)){break}v=y;w=c[x>>2]|0}z=c[n>>2]|0}else{z=0}IY(q,f,g<<2);c[a+8>>2]=2;c[a+16>>2]=u;c[a+20>>2]=z;c[a+24>>2]=(j|0)>-1?j+1|0:0;i=m;return}else if((b|0)==1){z=c[d+(h<<2)>>2]|0;c[n>>2]=0;u=(h|0)>0;do{if(u){g=h;q=z;while(1){w=g-1|0;v=d+(w<<2)|0;Mr(r,n,q-(c[v>>2]|0)|0);if((w|0)<=0){break}g=w;q=c[v>>2]|0}q=c[n>>2]|0;if(!u){A=q;break}g=h-1|0;v=0;while(1){c[a+40+(v<<2)>>2]=c[f+(g-v<<2)>>2];w=v+1|0;if((w|0)<(h|0)){v=w}else{A=q;break}}}else{A=0}}while(0);c[a+40+(h<<2)>>2]=1;if((h|0)>=(j|0)&(j|0)!=-99){B=h+1-j|0}else{B=0}c[a+8>>2]=1;c[a+16>>2]=z;c[a+20>>2]=A;c[a+24>>2]=B;i=m;return}else if((b|0)==3){do{if((k|0)==0){b=h+1|0;B=c[d>>2]|0;A=c[d+(h<<2)>>2]|0;z=h-1-l|0;c[p>>2]=0;L17437:do{if((l|0)>=1){u=1;n=B;while(1){r=d+(u<<2)|0;Mr(s,p,(c[r>>2]|0)-n|0);q=u+1|0;if((q|0)>(l|0)){break L17437}u=q;n=c[r>>2]|0}}}while(0);c[o>>2]=0;n=b-z|0;if((n|0)<=(h|0)){u=h;while(1){r=u-1|0;Mr(t,o,(c[d+(u<<2)>>2]|0)-(c[d+(r<<2)>>2]|0)|0);if((r|0)<(n|0)){break}else{u=r}}}u=(h|0)>=(j|0)&(j|0)!=-99?b-j|0:0;if((h|0)>0){C=0}else{D=l;E=z;F=A;G=B;H=u;break}while(1){c[a+96+(C<<2)>>2]=c[f+(C<<2)>>2];n=C+1|0;if((n|0)<(h|0)){C=n}else{D=l;E=z;F=A;G=B;H=u;break}}}else{u=c[d>>2]|0;B=e-1|0;A=c[d+(B<<2)>>2]|0;z=e-2|0;b=(z|0)>2;n=b?e-4|0:0;r=b?2:z;c[p>>2]=0;L17425:do{if((r|0)>=1){z=1;b=u;while(1){q=d+(z<<2)|0;Mr(s,p,(c[q>>2]|0)-b|0);v=z+1|0;if((v|0)>(r|0)){break L17425}z=v;b=c[q>>2]|0}}}while(0);c[o>>2]=0;b=e-n|0;if((B|0)>=(b|0)){z=B;while(1){q=z-1|0;Mr(t,o,(c[d+(z<<2)>>2]|0)-(c[d+(q<<2)>>2]|0)|0);if((q|0)<(b|0)){break}else{z=q}}}z=(j|0)>0?e-j|0:0;if((B|0)>0){I=0}else{D=r;E=n;F=A;G=u;H=z;break}while(1){c[a+96+(I<<2)>>2]=c[f+(I<<2)>>2];b=I+1|0;if((b|0)<(B|0)){I=b}else{D=r;E=n;F=A;G=u;H=z;break}}}}while(0);c[a+8>>2]=3;c[a+56>>2]=G;c[a+60>>2]=F;c[a+64>>2]=D;c[a+68>>2]=E;c[a+24>>2]=H;i=m;return}else{i=m;return}}function LZ(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;I_(a,380);c[a+36>>2]=0;c[a+40>>2]=0;c[a+32>>2]=b;c[a+8>>2]=d;c[a+4>>2]=e;c[a+12>>2]=g;c[a+88>>2]=0;c[a+160>>2]=0;c[a+136>>2]=0;c[a+192>>2]=0;c[a+164>>2]=0;c[a+168>>2]=0;g=a+196|0;c[a+200>>2]=f;if((h|0)!=0){c[a+24>>2]=2;c[a+28>>2]=16;c[a>>2]=0;return}if((f|0)==15){c[a+24>>2]=4;c[a+28>>2]=12;c[g>>2]=0;c[a>>2]=4;return}else if((f|0)==16){c[a+24>>2]=4;c[a+28>>2]=12;c[g>>2]=0;c[a>>2]=4;return}else if((f|0)==9){c[a+24>>2]=2;c[a+28>>2]=8;c[g>>2]=0;c[a>>2]=4;return}else if((f|0)==18){c[a+24>>2]=4;c[a+28>>2]=15;c[g>>2]=0;c[a>>2]=8;return}else{return}}function L_(b,d,e,f,g,h,j,k){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0;l=i;i=i+592|0;m=l|0;n=l+40|0;o=l+80|0;p=l+336|0;q=b+124|0;if((c[q>>2]|0)<=0){i=l;return}r=b|0;s=o;t=p;u=c[172720+(h-g<<2)>>2]|0;v=(g|0)<(h|0);w=u;x=(u|0)<0|0?-1:0;u=b+132|0;y=m;z=n;A=m|0;B=n|0;C=(j|0)==1;j=0;while(1){D=c[b+84+(j<<2)>>2]|0;E=j+1|0;F=c[b+84+(E<<2)>>2]|0;H=b+176+(j*76|0)|0;I=c[r>>2]|0;I_(s,256);I_(t,256);J=F-D|0;K=c[172720+(J<<2)>>2]|0;L=b+176+(j*76|0)+72|0;c[L>>2]=0;L17472:do{if(v){if((D|0)<(F|0)){M=g;N=0}else{O=g;P=0;while(1){Q=c[e+(O<<2)>>2]|0;N2(Q,(Q|0)<0|0?-1:0,w,x)|0;P=(G<<1|0>>>31)+P|0;c[L>>2]=P;O=O+1|0;if((O|0)>=(h|0)){break L17472}}}do{O=c[d+(M<<2)>>2]|0;P=D;do{Q=c[O+(P<<2)>>2]|0;N2(Q,(Q|0)<0|0?-1:0,w,x)|0;Q=o+(P<<2)|0;c[Q>>2]=(G<<1|0>>>31)+(c[Q>>2]|0);Q=a[f+P|0]|0;if(Q<<24>>24!=-1){R=c[O+(Q<<24>>24<<2)>>2]|0;Q=R;S=(R|0)<0|0?-1:0;N2(Q,S,w,x)|0;S=p+(P<<2)|0;c[S>>2]=(G<<1|0>>>31)+(c[S>>2]|0)}P=P+1|0;}while((P|0)<(F|0));P=c[e+(M<<2)>>2]|0;N2(P,(P|0)<0|0?-1:0,w,x)|0;N=(G<<1|0>>>31)+N|0;c[L>>2]=N;M=M+1|0;}while((M|0)<(h|0))}}while(0);if((D|0)<(F|0)){P=K;O=(K|0)<0|0?-1:0;S=0;Q=0;R=D;do{T=c[o+(R<<2)>>2]|0;N2(T,(T|0)<0|0?-1:0,P,O)|0;S=G+S|0;T=c[p+(R<<2)>>2]|0;N2(T,(T|0)<0|0?-1:0,P,O)|0;Q=G+Q|0;R=R+1|0;}while((R|0)<(F|0));U=S<<1;V=Q<<1}else{U=0;V=0}Mn(o+(D<<2)|0,J);Mn(p+(D<<2)|0,J);R=(J|0)<(I|0)?J:I;if((R|0)>0){O=c[172720+(R<<2)>>2]|0;P=F-R|0;K=O;T=(O|0)<0|0?-1:0;O=0;W=0;X=0;do{Y=X+P|0;Z=c[o+(Y<<2)>>2]|0;N2(Z,(Z|0)<0|0?-1:0,K,T)|0;W=G+W|0;Z=c[p+(Y<<2)>>2]|0;N2(Z,(Z|0)<0|0?-1:0,K,T)|0;O=G+O|0;X=X+1|0;}while((X|0)<(R|0));_=O<<1;$=W<<1}else{_=0;$=0}R=F-1|0;c[b+176+(j*76|0)+64>>2]=c[o+(R<<2)>>2];c[b+176+(j*76|0)+68>>2]=c[p+(R<<2)>>2];R=b+176+(j*76|0)+4|0;IZ(H,R,8);X=b+176+(j*76|0)+16|0;IZ(b+176+(j*76|0)+12|0,X,8);T=b+176+(j*76|0)+28|0;IZ(b+176+(j*76|0)+24|0,T,8);K=b+176+(j*76|0)+40|0;IZ(b+176+(j*76|0)+36|0,K,8);c[b+176+(j*76|0)+8>>2]=U;c[b+176+(j*76|0)+20>>2]=V;c[b+176+(j*76|0)+32>>2]=$;c[b+176+(j*76|0)+44>>2]=_;P=b+176+(j*76|0)+48|0;I=b+176+(j*76|0)+52|0;NM(P|0,0,16);J=c[b+176+(j*76|0)>>2]|0;D=c[b+176+(j*76|0)+12>>2]|0;Q=c[b+176+(j*76|0)+24>>2]|0;S=c[b+176+(j*76|0)+36>>2]|0;Z=c[R>>2]|0;N2(Z,(Z|0)<0|0?-1:0,805306368,0)|0;Z=(G<<1|0>>>31)+((J>>>4|((J|0)<0|0?-1:0)<<28)<<1|0>>>31)|0;J=c[X>>2]|0;N2(J,(J|0)<0|0?-1:0,805306368,0)|0;J=(G<<1|0>>>31)+((D>>>4|((D|0)<0|0?-1:0)<<28)<<1|0>>>31)|0;D=c[T>>2]|0;N2(D,(D|0)<0|0?-1:0,805306368,0)|0;D=(G<<1|0>>>31)+((Q>>>4|((Q|0)<0|0?-1:0)<<28)<<1|0>>>31)|0;Q=c[K>>2]|0;N2(Q,(Q|0)<0|0?-1:0,805306368,0)|0;Q=((U>>>2|((U|0)<0|0?-1:0)<<30)<<1|0>>>31)+Z|0;c[P>>2]=Q;c[I>>2]=((V>>>2|((V|0)<0|0?-1:0)<<30)<<1|0>>>31)+J;c[b+176+(j*76|0)+56>>2]=(($>>>2|(($|0)<0|0?-1:0)<<30)<<1|0>>>31)+D;c[b+176+(j*76|0)+60>>2]=((_>>>2|((_|0)<0|0?-1:0)<<30)<<1|0>>>31)+((G<<1|0>>>31)+((S>>>4|((S|0)<0|0?-1:0)<<28)<<1|0>>>31));S=c[u>>2]|0;D=b+4+(j<<2)|0;J=b+44+(j<<2)|0;P=c[S>>2]|0;Z=c[S+4>>2]|0;K=c[S+8>>2]|0;T=c[S+12>>2]|0;X=c[S+16>>2]|0;R=c[S+20>>2]|0;Y=(HE((Q|0)>1?Q:1)|0)+668792448|0;N2(Y,(Y|0)<0|0?-1:0,1610612736,0)|0;Y=G;Q=c[I>>2]|0;I=(HE((Q|0)>1?Q:1)|0)+668792448|0;N2(I,(I|0)<0|0?-1:0,1610612736,0)|0;I=G;Q=(HE((c[L>>2]|0)+1|0)|0)+1610612736|0;N2(Q,(Q|0)<0|0?-1:0,1610612736,0)|0;Q=G;IY(y,P,T<<2);IY(z,Z,X<<2);aa=c[D>>2]|0;if((aa|0)<(T|0)){c[m+(aa<<2)>>2]=(c[P+(aa<<2)>>2]|0)+4194304}if((aa|0)>0){ab=aa-1|0;c[m+(ab<<2)>>2]=(c[P+(ab<<2)>>2]|0)-4194304}ab=c[J>>2]|0;if((ab|0)<(X|0)){c[n+(ab<<2)>>2]=(c[Z+(ab<<2)>>2]|0)+4194304}if((ab|0)>0){P=ab-1|0;c[n+(P<<2)>>2]=(c[Z+(P<<2)>>2]|0)-4194304}P=c[A>>2]|0;L17510:do{if((P|0)>(I|0)){ac=0}else{L17512:do{if((T|0)>1){Z=1;ab=P;while(1){if((ab|0)<=(I|0)){if((c[m+(Z<<2)>>2]|0)>(I|0)){ac=Z;break L17510}}aa=Z+1|0;if((aa|0)>=(T|0)){break L17512}ad=c[m+(Z<<2)>>2]|0;Z=aa;ab=ad}}}while(0);ac=(c[m+(T-1<<2)>>2]|0)>(I|0)?0:T}}while(0);T=c[B>>2]|0;L17521:do{if((T|0)>(Y|0)){ae=0}else{L17523:do{if((X|0)>1){I=1;P=T;while(1){if((P|0)<=(Y|0)){if((c[n+(I<<2)>>2]|0)>(Y|0)){ae=I;break L17521}}L=I+1|0;if((L|0)>=(X|0)){break L17523}H=c[n+(I<<2)>>2]|0;I=L;P=H}}}while(0);ae=(c[n+(X-1<<2)>>2]|0)>(Y|0)?0:X}}while(0);X=c[K>>2]|0;L17532:do{if((X|0)>(Q|0)){af=0}else{L17534:do{if((R|0)>1){Y=1;T=X;while(1){if((T|0)<=(Q|0)){if((c[K+(Y<<2)>>2]|0)>(Q|0)){af=Y;break L17532}}P=Y+1|0;if((P|0)>=(R|0)){break L17534}I=c[K+(Y<<2)>>2]|0;Y=P;T=I}}}while(0);af=(c[K+(R-1<<2)>>2]|0)>(Q|0)?0:R}}while(0);c[D>>2]=ac;c[J>>2]=ae;if(C){ag=S+124+(ac*20|0)+(ae<<2)|0}else{ag=S+24+(ac*20|0)+(ae<<2)|0}R=(c[S+224+(af<<2)>>2]|0)+(c[ag>>2]|0)|0;c[k+(j<<2)>>2]=(R|0)>0?R:0;if((E|0)<(c[q>>2]|0)){j=E}else{break}}i=l;return}function L$(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0;I_(a,980);c[a+132>>2]=(e|0)!=0?1916368:1921600;e=a+128|0;c[e>>2]=d;if((d|0)>0){f=0;while(1){I_(a+176+(f*76|0)|0,76);c[a+136+(f<<2)>>2]=0;c[a+44+(f<<2)>>2]=0;c[a+4+(f<<2)>>2]=0;g=f+1|0;h=c[e>>2]|0;if((g|0)<(h|0)){f=g}else{i=h;break}}}else{i=d}c[a>>2]=1;IY(a+84|0,b,(i<<2)+4|0);c[a+124>>2]=i;return 0}function L0(a,b,d){a=a|0;b=b|0;d=d|0;c[a>>2]=1;IY(a+84|0,b,(d<<2)+4|0);c[a+124>>2]=d;return 0}function L1(b,e,f,g,h,j,k,l,m,n,o,p){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;var q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0;q=i;i=i+1304|0;r=q|0;s=q+48|0;t=q+96|0;u=q+104|0;v=q+120|0;w=q+136|0;x=q+152|0;y=a[j+1|0]|0;z=a[j]|0;j=c[b+32>>2]|0;A=c[b+40>>2]|0;B=c[b+36>>2]|0;C=b+12|0;D=c[C>>2]|0;E=b+16|0;F=c[E>>2]|0;H=c[b+28>>2]|0;I=b+20|0;J=c[I>>2]|0;K=b+1224|0;L=c[K>>2]|0;M=c[L>>2]|0;N=c[L+52>>2]|0;O=b+840|0;c[u>>2]=O;P=b+456|0;c[v>>2]=P;Q=b+72|0;c[w>>2]=Q;c[u+4>>2]=b+1032;c[v+4>>2]=b+648;c[w+4>>2]=b+264;R=y&255;S=z&255;z=b+1228|0;c[u+8>>2]=x;c[v+8>>2]=x+192;c[w+8>>2]=x+384;c[u+12>>2]=x+576;c[v+12>>2]=x+768;c[w+12>>2]=x+960;if(y<<24>>24==0){if((D|0)!=0&(F|0)==0){T=1;U=1;V=0}else{W=12597}}else{D=H+S|0;y=(D|0)<(c[h+4+(c[h>>2]<<2)>>2]|0);if((A|0)>1){X=A>>((D|0)<=(c[b+44>>2]>>1|0))}else{X=A}if(y){T=1;U=y&1;V=X}else{W=12597}}do{if((W|0)==12597){if((F|0)==0){T=0;U=0;V=0;break}X=(c[h+4>>2]|0)-(J+H)+(c[b+44>>2]|0)|0;T=(((X|0)>0?X:-X|0)|0)<(M|0)|0;U=0;V=0}}while(0);c[C>>2]=R;c[E>>2]=U;c[I>>2]=S;if((A|0)>0){S=(n|0)>0;I=0;do{U=I+j|0;E=e+(U<<2)|0;R=c[E>>2]|0;C=c[w+(U<<2)>>2]|0;if(S){M=0;H=0;do{J=a[m+H|0]|0;h=a[m+(H+1)|0]|0;do{if((J&255)<(h&255)){F=J;X=0;y=0;do{D=F&255;x=c[R+(D<<2)>>2]|0;X=(X|0)>(x|0)?X:x;x=c[R+(a[g+D|0]<<2)>>2]|0;y=(y|0)>(x|0)?y:x;F=F+1&255;}while((F&255)<(h&255));if((y|0)<=2146){Y=X;W=12609;break}F=HQ(X,y,t)|0;N2(F,(F|0)<0|0?-1:0,1125899904,0)|0;F=G<<1|0>>>31;x=(c[t>>2]|0)-19|0;D=(x|0)<-31?-31:x;if((D|0)>0){Z=F<<D}else{Z=F>>-D}c[C+(H<<2)>>2]=Z}else{Y=0;W=12609}}while(0);if((W|0)==12609){W=0;c[C+(H<<2)>>2]=Y}M=M+1&255;H=M&255;}while((H|0)<(n|0));H=c[E>>2]|0;M=c[v+(U<<2)>>2]|0;C=c[u+(U<<2)>>2]|0;R=0;while(1){h=a[m+R|0]|0;J=h&255;D=R+1|0;F=a[m+D|0]|0;x=F&255;_=M+(R<<2)|0;c[_>>2]=536870911;$=C+(R<<2)|0;c[$>>2]=536870911;aa=x-J|0;do{if((aa|0)>1){ab=c[172720+(aa<<2)>>2]|0;ac=ab;ad=(ab|0)<0|0?-1:0;if((h&255)<(F&255)){ab=0;ae=0;af=0;ag=0;ah=2147483647;ai=2147483647;aj=J;do{ak=c[H+(aj<<2)>>2]|0;al=c[H+(a[g+aj|0]<<2)>>2]|0;N2(ak,(ak|0)<0|0?-1:0,ac,ad)|0;af=(G<<1|0>>>31)+af|0;N2(al,(al|0)<0|0?-1:0,ac,ad)|0;ag=(G<<1|0>>>31)+ag|0;do{if((ak|0)==0){am=0}else{an=ak>>31^ak;if((an|0)<=-1){am=-1;break}ao=~an;an=0;while(1){ap=ao<<1;if((ap|0)<0){ao=ap;an=an+1|0}else{am=an;break}}}}while(0);do{if((al|0)==0){aq=0}else{an=al>>31^al;if((an|0)<=-1){aq=-1;break}ao=~an;an=0;while(1){ap=ao<<1;if((ap|0)<0){ao=ap;an=an+1|0}else{aq=an;break}}}}while(0);an=ak<<am;N2(an,(an|0)<0|0?-1:0,ah,(ah|0)<0|0?-1:0)|0;ah=G<<1|0>>>31;an=al<<aq;N2(an,(an|0)<0|0?-1:0,ai,(ai|0)<0|0?-1:0)|0;ai=G<<1|0>>>31;ab=am+ab|0;ae=aq+ae|0;aj=aj+1|0;}while((aj|0)<(x|0));if((ah|0)>0){ar=ai;as=ah;at=ag;au=af;av=ae;aw=ab;W=12624}else{ax=0;ay=ai;az=ag;aA=af;aB=ae}}else{ar=2147483647;as=2147483647;at=0;au=0;av=0;aw=0;W=12624}if((W|0)==12624){W=0;aj=HE(as)|0;y=aj;X=(aj|0)<0|0?-1:0;N2(y,X,ac,ad)|0;X=G<<1|0>>>31;y=-aw<<23;aj=y;an=(y|0)<0|0?-1:0;N2(aj,an,ac,ad)|0;ax=HH(X+(G<<3|0>>>29)|0)|0;ay=ar;az=at;aA=au;aB=av}if((ay|0)>0){X=HE(ay)|0;an=X;aj=(X|0)<0|0?-1:0;N2(an,aj,ac,ad)|0;aj=G<<1|0>>>31;an=-aB<<23;X=an;y=(an|0)<0|0?-1:0;N2(X,y,ac,ad)|0;aC=HH(aj+(G<<3|0>>>29)|0)|0}else{aC=0}if((aA|0)!=0){c[_>>2]=Ms(ax,aA,536870911)|0}if((az|0)==0){break}c[$>>2]=Ms(aC,az,536870911)|0}}while(0);if((D|0)<(n|0)){R=D}else{break}}}I=I+1|0;}while((I|0)<(A|0));aD=c[K>>2]|0}else{aD=L}L=c[b+48>>2]|0;K=r;I=s;I_(l,n);if((B|0)<2|(T|0)==0|(V|0)==0){aE=0}else{az=n<<2;IY(c[b+1228+(V*12|0)>>2]|0,c[z>>2]|0,az);IY(c[b+1228+(V*12|0)+4>>2]|0,c[b+1232>>2]|0,az);I_(c[b+1228+((V-1|0)*12|0)+8>>2]|0,n);aE=V}V=(aE|0)<(B|0);if(V){az=B-1|0;aC=n<<2;aA=aD+4|0;ax=aE;while(1){if((ax|0)>0){IY(c[b+1228+(ax*12|0)+8>>2]|0,c[b+56+(ax-1<<2)>>2]|0,n)}aB=b+56+(ax<<2)|0;I_(c[aB>>2]|0,n);if((ax|0)<(az|0)){ay=ax+1|0;av=b+1228+(ay*12|0)|0;I_(c[av>>2]|0,aC);au=b+1228+(ay*12|0)+4|0;I_(c[au>>2]|0,aC);at=b+1228+(ay*12|0)+8|0;I_(c[at>>2]|0,n);ar=c[e+(ax<<2)>>2]|0;aw=c[w+(ax<<2)>>2]|0;as=c[aB>>2]|0;aq=c[v+(ax<<2)>>2]|0;am=c[u+(ax<<2)>>2]|0;g=c[b+1228+(ax*12|0)>>2]|0;Y=c[b+1228+(ax*12|0)+4>>2]|0;Z=c[b+1228+(ax*12|0)+8>>2]|0;t=c[av>>2]|0;av=c[au>>2]|0;au=c[at>>2]|0;NN(K|0,aA|0,48)|0;L6(ar,aw,n,as,m,aq,am,g,Y,Z,t,av,au,r);aF=ay}else{ay=b+1228+(ax*12|0)|0;I_(c[ay>>2]|0,aC);au=b+1228+(ax*12|0)+4|0;I_(c[au>>2]|0,aC);av=b+1228+(ax*12|0)+8|0;I_(c[av>>2]|0,n);t=c[e+(ax<<2)>>2]|0;Z=c[w+(ax<<2)>>2]|0;Y=c[aB>>2]|0;aB=c[v+(ax<<2)>>2]|0;g=c[u+(ax<<2)>>2]|0;am=c[ay>>2]|0;ay=c[au>>2]|0;au=c[av>>2]|0;NN(I|0,aA|0,48)|0;L6(t,Z,n,Y,m,aB,g,am,ay,au,am,ay,au,s);aF=ax+1|0}if((aF|0)<(B|0)){ax=aF}else{break}}}aF=c[aD+40>>2]|0;ax=c[aD+44>>2]|0;s=c[aD+48>>2]|0;aD=(n|0)>0;if(V&aD){aA=aE;do{I=b+56+(aA<<2)|0;aC=0;do{r=l+aC|0;if((a[r]|0)==0){aG=(a[(c[I>>2]|0)+aC|0]|0)!=0|0}else{aG=1}a[r]=aG;aC=aC+1|0;}while((aC|0)<(n|0));aA=aA+1|0;}while((aA|0)<(B|0))}L17644:do{if((T|0)==1){aA=n-1|0;if((aA|0)>0){aG=e+(aE<<2)|0;aC=0;while(1){I=a[m+aC|0]|0;r=aC+1|0;K=a[m+r|0]|0;az=l+aC|0;L17656:do{if((a[az]|0)!=0){au=l+r|0;if((a[au]|0)==0){break}ay=I&255;am=K&255;g=c[aG>>2]|0;aB=c[g+(ay<<2)>>2]|0;L17659:do{if(V){L17661:do{if((I&255)<(K&255)){Y=aE;Z=aB;t=ay;av=aE;aq=g;while(1){as=ay;aw=Z;ar=t;at=av;do{S=c[aq+(as<<2)>>2]|0;R=(S|0)>(aw|0);aw=R?S:aw;ar=R?as:ar;at=R?Y:at;as=as+1|0;}while((as|0)<(am|0));as=Y+1|0;if((as|0)>=(B|0)){aH=at;aI=ar;aJ=aw;break L17661}Y=as;Z=aw;t=ar;av=at;aq=c[e+(as<<2)>>2]|0}}else{aH=aE;aI=ay;aJ=aB}}while(0);al=c[g+(am<<2)>>2]|0;ak=a[m+(aC+2)|0]|0;aq=ak&255;if((K&255)<(ak&255)){aK=aE;aL=al;aM=am;aN=aE;aO=g}else{aP=al;aQ=am;aR=aE;aS=aH;aT=aI;aU=aJ;break}while(1){al=am;ak=aL;av=aM;t=aN;do{Z=c[aO+(al<<2)>>2]|0;Y=(Z|0)>(ak|0);ak=Y?Z:ak;av=Y?al:av;t=Y?aK:t;al=al+1|0;}while((al|0)<(aq|0));al=aK+1|0;if((al|0)>=(B|0)){aP=ak;aQ=av;aR=t;aS=aH;aT=aI;aU=aJ;break L17659}aK=al;aL=ak;aM=av;aN=t;aO=c[e+(al<<2)>>2]|0}}else{aP=c[g+(am<<2)>>2]|0;aQ=am;aR=aE;aS=aE;aT=ay;aU=aB}}while(0);if((aQ-aT|0)>=2){break}aB=a[L+aC|0]|0;do{if((aB<<24>>24|0)==0){if((a[L+r|0]|0)!=1){break}a[az]=0;if(V){aV=aE}else{break L17656}while(1){a[(c[b+56+(aV<<2)>>2]|0)+aC|0]=0;aV=aV+1|0;if((aV|0)>=(B|0)){break L17656}}}else if((aB<<24>>24|0)==1){if((a[L+r|0]|0)!=0){break}a[au]=0;if(V){aW=aE}else{break L17656}while(1){a[(c[b+56+(aW<<2)>>2]|0)+r|0]=0;aW=aW+1|0;if((aW|0)>=(B|0)){break L17656}}}}while(0);if((aU|0)>(aP|0)){aB=c[f+(aS<<2)>>2]|0;if((c[aB+(aQ<<2)>>2]|0)>=0){break}if((c[aB+(aT<<2)>>2]|0)<=0){break}a[au]=0;if(V){aX=aE}else{break}do{a[(c[b+56+(aX<<2)>>2]|0)+r|0]=0;aX=aX+1|0;}while((aX|0)<(B|0))}else{au=c[f+(aR<<2)>>2]|0;if((c[au+(aQ<<2)>>2]|0)>=0){break}if((c[au+(aT<<2)>>2]|0)<=0){break}a[az]=0;if(V){aY=aE}else{break}do{a[(c[b+56+(aY<<2)>>2]|0)+aC|0]=0;aY=aY+1|0;}while((aY|0)<(B|0))}}}while(0);if((r|0)<(aA|0)){aC=r}else{break}}}aC=d[m+n|0]|0;aA=aC-3|0;aG=a[m]|0;L17701:do{if((aA|0)>(aG&255|0)){az=aC;K=aA;I=-2147483648;D=aG;while(1){au=c[p+(K<<2)>>2]|0;aB=c[p+(K+2<<2)>>2]|0;if((au|0)!=0&(au|0)>(aB|0)){ay=au>>1;au=HE(ay)|0;am=(HE(ay-(aB>>1)|0)|0)-au|0;au=(am|0)>(I|0);aB=au?K:az;ay=au?am:I;if((ay|0)>(aF|0)){aZ=ay;a_=aB;break L17701}a$=ay;a0=aB;a1=a[m]|0}else{a$=I;a0=az;a1=D}aB=K-1|0;if((aB|0)>(a1&255|0)){az=a0;K=aB;I=a$;D=a1}else{aZ=a$;a_=a0;break}}}else{aZ=-2147483648;a_=aC}}while(0);aC=a_+2|0;if((aA|0)>(aC|0)){aG=aA;D=-2147483648;while(1){I=c[p+(aG<<2)>>2]|0;K=c[p+(aG+2<<2)>>2]|0;do{if((I|0)!=0&(I|0)>(K|0)){az=I>>1;r=HE(az)|0;aB=(HE(az-(K>>1)|0)|0)-r|0;a2=(aB|0)>(D|0)?aB:D}else{if(!((K|0)!=0&(K|0)>(I|0))){a2=D;break}aB=K>>1;r=HE(aB)|0;az=(HE(aB-(I>>1)|0)|0)-r|0;a2=(az|0)>(D|0)?az:D}}while(0);I=aG-1|0;if((I|0)>(aC|0)){aG=I;D=a2}else{a3=a2;break}}}else{a3=-2147483648}if(!((aZ|0)>(aF|0)&(a3|0)<(s|0))){break}D=a_-5|0;aG=p+(aC<<2)|0;aA=a_;while(1){I=aA-1|0;if(!((I|0)>(D|0)&(aA|0)>0)){break}K=p+(I<<2)|0;az=c[K>>2]|0;if((az|0)==0){break L17644}r=c[aG>>2]|0;if((az|0)<=(r|0)){break L17644}aB=HE((az>>1)-(r>>1)|0)|0;if((aB-(HE(c[K>>2]>>1)|0)|0)<(ax|0)){break L17644}else{aA=I}}L17725:do{if(aD){aA=0;while(1){aG=aA+1|0;if((a_|0)>=(d[m+aA|0]|0)){if((a_|0)<(d[m+aG|0]|0)){a4=aA;break L17725}}if((aG|0)<(n|0)){aA=aG}else{a4=aG;break}}}else{a4=0}}while(0);aA=l+a4|0;if((a[aA]|0)==0){break}a[aA]=0;if(V){a5=aE}else{break}do{a[(c[b+56+(a5<<2)>>2]|0)+a4|0]=0;a5=a5+1|0;}while((a5|0)<(B|0))}else{if(aD){a6=0}else{break}do{aA=l+a6|0;if(((d[aA]|0)-(d[L+a6|0]|0)|0)>0){a[aA]=0}a6=a6+1|0;}while((a6|0)<(n|0))}}while(0);c[k>>2]=0;a6=0;while(1){if((a6|0)>=(n|0)){break}if((a[l+a6|0]|0)==0){a6=a6+1|0}else{W=12716;break}}if((W|0)==12716){c[k>>2]=1}IY(L,l,n);IY(c[b+1236>>2]|0,l,n<<2);L17742:do{if(aD){L=z|0;k=b+1232|0;if(V){a7=0}else{W=0;while(1){c[(c[L>>2]|0)+(W<<2)>>2]=0;c[(c[k>>2]|0)+(W<<2)>>2]=0;W=W+1|0;if((W|0)>=(n|0)){break L17742}}}do{c[(c[L>>2]|0)+(a7<<2)>>2]=0;c[(c[k>>2]|0)+(a7<<2)>>2]=0;if((a[l+a7|0]|0)==1){W=aE;do{a6=c[(c[b+1228+(W*12|0)>>2]|0)+(a7<<2)>>2]|0;if((a6|0)!=0){c[(c[L>>2]|0)+(a7<<2)>>2]=a6}a6=c[(c[b+1228+(W*12|0)+4>>2]|0)+(a7<<2)>>2]|0;if((a6|0)!=0){c[(c[k>>2]|0)+(a7<<2)>>2]=a6}W=W+1|0;}while((W|0)<(B|0))}a7=a7+1|0;}while((a7|0)<(n|0))}}while(0);a7=c[b+52>>2]|0;I_(o,n);do{if(aD){b=(B|0)>0;aE=0;while(1){do{if((a[l+aE|0]|0)==0){a8=aE+1|0}else{V=a[m+aE|0]|0;z=V&255;k=aE+1|0;L=a[m+k|0]|0;W=L&255;if(b){a6=(V&255)<(L&255);L=0;V=0;a5=0;a4=0;while(1){if(a6){a_=c[e+(L<<2)>>2]|0;ax=z;p=V;s=a5;a3=a4;while(1){aF=c[a_+(ax<<2)>>2]|0;aZ=(aF|0)>(a3|0);a2=aZ?ax:p;a0=aZ?L:s;a$=aZ?aF:a3;aF=ax+1|0;if((aF|0)<(W|0)){ax=aF;p=a2;s=a0;a3=a$}else{a9=a2;ba=a0;bb=a$;break}}}else{a9=V;ba=a5;bb=a4}a3=L+1|0;if((a3|0)<(B|0)){L=a3;V=a9;a5=ba;a4=bb}else{bc=a9;bd=ba;break}}}else{bc=0;bd=0}do{if(!((bc|0)!=(z|0)|(aE|0)==0)){a4=aE-1|0;if((a[l+a4|0]|0)!=0){break}a5=z-1|0;V=c[f+(bd<<2)>>2]|0;if((c[V+(a5<<2)>>2]|0)<=0){break}if((c[V+(z<<2)>>2]|0)>=0){break}V=HE(c[(c[e+(bd<<2)>>2]|0)+(a5<<2)>>2]|0)|0;a5=V+668792448|0;L=(((a5|0)>0?a5:-668792448-V|0)>>24)+1>>1;a[o+a4|0]=((L|0)>(N|0)?N:L)&255}}while(0);if(!((bc|0)==(W-1|0)&(k|0)<(n|0))){a8=k;break}if((a[l+k|0]|0)!=0){a8=k;break}z=c[f+(bd<<2)>>2]|0;if((c[z+(bc<<2)>>2]|0)<=0){a8=k;break}L=bc+1|0;if((c[z+(L<<2)>>2]|0)>=0){a8=k;break}z=HE(c[(c[e+(bd<<2)>>2]|0)+(L<<2)>>2]|0)|0;L=z+668792448|0;a4=(((L|0)>0?L:-668792448-z|0)>>24)+1>>1;a[o+k|0]=((a4|0)>(N|0)?N:a4)&255;a8=k}}while(0);if((a8|0)<(n|0)){aE=a8}else{break}}if((T|0)==0){be=0}else{break}do{aE=o+be|0;do{if((a[aE]|0)!=0){if((a[a7+be|0]|0)!=0){break}a[aE]=0}}while(0);be=be+1|0;}while((be|0)<(n|0))}}while(0);IY(a7,o,n);if((j|0)>0){bf=0;bg=Q;bh=P;bi=O}else{i=q;return}while(1){O=bf+A|0;IY(bg,c[w+(O<<2)>>2]|0,192);IY(bh,c[v+(O<<2)>>2]|0,192);IY(bi,c[u+(O<<2)>>2]|0,192);O=bf+1|0;if((O|0)>=(j|0)){break}bf=O;bg=c[w+(O<<2)>>2]|0;bh=c[v+(O<<2)>>2]|0;bi=c[u+(O<<2)>>2]|0}i=q;return}function L2(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=Jk(b)|0;e=Jq(b)|0;f=Js(b)|0;g=Ju(b)|0;I_(a,1276);c[a+52>>2]=Jm(b)|0;c[a+48>>2]=Jo(b)|0;c[a+1228>>2]=f;c[a+1232>>2]=g;c[a+56>>2]=d;c[a+1236>>2]=e;c[a+1240>>2]=f+192;c[a+1244>>2]=g+192;c[a+60>>2]=d+48;c[a+1248>>2]=e+48;c[a+1252>>2]=f+384;c[a+1256>>2]=g+384;c[a+64>>2]=d+96;c[a+1260>>2]=e+96;c[a+1264>>2]=f+576;c[a+1268>>2]=g+576;c[a+68>>2]=d+144;c[a+1272>>2]=e+144;return 0}function L3(a,b,d,e,f,g,h,i,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0;if((d|0)==960){c[a+28>>2]=0;c[a+44>>2]=15}else if((d|0)==1024){c[a+28>>2]=0;c[a+44>>2]=16}else if((d|0)==1920){c[a+28>>2]=4;c[a+44>>2]=15}else if((d|0)==2048){c[a+28>>2]=4;c[a+44>>2]=16}else{k=-1;return k|0}c[a+1224>>2]=(j&1|0)==0?1914552:1927568;c[a>>2]=f;c[a+8>>2]=b;c[a+4>>2]=e;c[a+36>>2]=g;c[a+32>>2]=h;c[a+40>>2]=i;if((g|0)>0){i=0;do{I_(c[a+1228+(i*12|0)>>2]|0,192);I_(c[a+1228+(i*12|0)+4>>2]|0,192);I_(c[a+56+(i<<2)>>2]|0,48);I_(c[a+1228+(i*12|0)+8>>2]|0,48);i=i+1|0;}while((i|0)<(g|0))}I_(a+72|0,192);I_(a+456|0,192);I_(a+840|0,192);I_(a+264|0,192);I_(a+648|0,192);I_(a+1032|0,192);I_(c[a+52>>2]|0,48);I_(c[a+48>>2]|0,48);c[a+12>>2]=0;c[a+16>>2]=0;c[a+20>>2]=0;k=0;return k|0}function L4(a){a=a|0;if((a|0)==0){return}Jl(a+56|0);Jr(a+1236|0);Jt(a+1228|0);Jv(a+1232|0);Jn(a+52|0);Jp(a+48|0);return}function L5(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;e=i;i=i+240|0;f=e|0;g=e+192|0;h=b+4|0;j=c[h>>2]|0;c[h>>2]=d;h=g|0;k=b+48|0;IY(h,c[k>>2]|0,j);l=(j|0)<(d|0);do{if(l){m=d-j|0;if((m|0)>0){n=0;do{a[(c[k>>2]|0)+n|0]=0;n=n+1|0;}while((n|0)<(m|0))}if((j|0)>0){o=0}else{break}do{a[(c[k>>2]|0)+(o+m)|0]=a[g+o|0]|0;o=o+1|0;}while((o|0)<(j|0))}else{if((d|0)<=0){break}m=j-d|0;n=0;do{a[(c[k>>2]|0)+n|0]=a[g+(m+n)|0]|0;n=n+1|0;}while((n|0)<(d|0))}}while(0);k=f;o=b+1228|0;n=j<<2;IY(k,c[o>>2]|0,n);do{if(l){m=d-j|0;if((m|0)>0){p=0;do{c[(c[o>>2]|0)+(p<<2)>>2]=0;p=p+1|0;}while((p|0)<(m|0))}if((j|0)>0){q=0}else{r=12793;break}while(1){c[(c[o>>2]|0)+(q+m<<2)>>2]=c[f+(q<<2)>>2];p=q+1|0;if((p|0)<(j|0)){q=p}else{r=12793;break}}}else{if((d|0)<=0){m=b+1232|0;IY(k,c[m>>2]|0,n);s=m;r=12794;break}m=j-d|0;p=0;while(1){c[(c[o>>2]|0)+(p<<2)>>2]=c[f+(m+p<<2)>>2];t=p+1|0;if((t|0)<(d|0)){p=t}else{r=12793;break}}}}while(0);do{if((r|0)==12793){o=b+1232|0;IY(k,c[o>>2]|0,n);if(!l){s=o;r=12794;break}q=d-j|0;if((q|0)>0){p=0;do{c[(c[o>>2]|0)+(p<<2)>>2]=0;p=p+1|0;}while((p|0)<(q|0))}if((j|0)>0){u=0}else{break}do{c[(c[o>>2]|0)+(u+q<<2)>>2]=c[f+(u<<2)>>2];u=u+1|0;}while((u|0)<(j|0))}}while(0);do{if((r|0)==12794){if((d|0)<=0){break}u=j-d|0;n=0;do{c[(c[s>>2]|0)+(n<<2)>>2]=c[f+(u+n<<2)>>2];n=n+1|0;}while((n|0)<(d|0))}}while(0);f=b+1236|0;IY(h,c[f>>2]|0,j);do{if(l){s=d-j|0;if((s|0)>0){n=0;do{a[(c[f>>2]|0)+n|0]=0;n=n+1|0;}while((n|0)<(s|0))}if((j|0)>0){v=0}else{r=12810;break}while(1){a[(c[f>>2]|0)+(v+s)|0]=a[g+v|0]|0;n=v+1|0;if((n|0)<(j|0)){v=n}else{r=12810;break}}}else{if((d|0)<=0){s=b+52|0;IY(h,c[s>>2]|0,j);w=s;break}s=j-d|0;n=0;while(1){a[(c[f>>2]|0)+n|0]=a[g+(s+n)|0]|0;u=n+1|0;if((u|0)<(d|0)){n=u}else{r=12810;break}}}}while(0);do{if((r|0)==12810){f=b+52|0;IY(h,c[f>>2]|0,j);if(!l){w=f;break}v=d-j|0;if((v|0)>0){n=0;do{a[(c[f>>2]|0)+n|0]=0;n=n+1|0;}while((n|0)<(v|0))}if((j|0)>0){x=0}else{i=e;return 0}do{a[(c[f>>2]|0)+(x+v)|0]=a[g+x|0]|0;x=x+1|0;}while((x|0)<(j|0));i=e;return 0}}while(0);if((d|0)<=0){i=e;return 0}x=j-d|0;j=0;do{a[(c[w>>2]|0)+j|0]=a[g+(x+j)|0]|0;j=j+1|0;}while((j|0)<(d|0));i=e;return 0}function L6(b,d,e,f,g,h,j,k,l,m,n,o,p,q){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;var r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0;p=i;m=q;q=i;i=i+48|0;NN(q,m,48)|0;m=(e|0)>0;do{if(m){r=c[q>>2]|0;s=c[q+16>>2]|0;t=c[q+32>>2]|0;u=t;v=(t|0)<0|0?-1:0;t=c[q+4>>2]|0;w=0;do{x=c[k+(w<<2)>>2]|0;y=(x|0)==0;if(y){z=r}else{A=x;B=(x|0)<0|0?-1:0;N2(u,v,A,B)|0;B=G<<1|0>>>31;z=(B|0)>(t|0)?B:t}B=d+(w<<2)|0;do{if((c[B>>2]|0)>(((z|0)<(r|0)?z:r)|0)){a[f+w|0]=1;c[n+(w<<2)>>2]=c[B>>2]}else{if(y){break}c[l+(w<<2)>>2]=s}}while(0);w=w+1|0;}while((w|0)<(e|0));if(!m){C=12826;break}w=c[q+28>>2]|0;s=w;r=(w|0)<0|0?-1:0;w=c[q+16>>2]|0;t=c[q+8>>2]|0;v=0;while(1){u=v+1|0;y=a[g+u|0]|0;B=y&255;A=c[l+(v<<2)>>2]|0;N2(s,r,A,(A|0)<0|0?-1:0)|0;x=G<<1|0>>>31;D=(x|0)>(w|0)?x:w;x=(D|0)<(t|0)?D:t;do{if((A|0)!=0){D=a[g+v|0]|0;if((D&255)>=(y&255)){break}E=f+v|0;F=o+(v<<2)|0;H=D&255;do{D=b+(H<<2)|0;if((c[D>>2]|0)>(x|0)){a[E]=1;c[F>>2]=c[D>>2]}H=H+1|0;}while((H|0)<(B|0))}}while(0);if((u|0)<(e|0)){v=u}else{I=t;break}}}else{C=12826}}while(0);if((C|0)==12826){I=c[q+8>>2]|0}if(!m){i=p;return}m=e-1|0;C=c[q+12>>2]|0;l=c[q+20>>2]|0;n=c[q+24>>2]|0;q=0;while(1){z=a[g+q|0]|0;k=z&255;t=q+1|0;v=a[g+t|0]|0;w=v&255;r=f+q|0;L17931:do{if((a[r]|0)==0){if((w-k|0)>1){if((z&255)>=(v&255)){break}s=j+(q<<2)|0;B=h+(q<<2)|0;x=o+(q<<2)|0;y=k;while(1){A=b+(y<<2)|0;do{if((c[A>>2]|0)>(I|0)){if((c[s>>2]|0)<=(l|0)){break}if((c[B>>2]|0)>=(n|0)){break}a[r]=1;c[x>>2]=c[A>>2]}}while(0);y=y+1|0;if((y|0)>=(w|0)){break L17931}}}if((q|0)>=(m|0)){break}y=b+(k<<2)|0;x=(c[y>>2]|0)>(I|0);if((q|0)<=0){if(!x){break}if((c[d+(t<<2)>>2]|0)>=(C|0)){break}a[r]=1;c[o+(q<<2)>>2]=c[y>>2];break}if(!x){break}if((c[d+(t<<2)>>2]|0)>=(C|0)){if((c[d+(q-1<<2)>>2]|0)>=(C|0)){break}}a[r]=1;c[o+(q<<2)>>2]=c[y>>2]}}while(0);if((t|0)<(e|0)){q=t}else{break}}i=p;return}function L7(b,d,e,f,g,h,j,k,l,m,n){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;var o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0;o=i;i=i+24|0;p=o|0;q=o+8|0;r=o+16|0;s=b+256|0;t=c[s>>2]|0;u=c[d+52>>2]|0;do{if((n&1|0)==0){c[q>>2]=j;if((u|0)==1){c[r>>2]=j+2;v=1;break}d=j+1|0;c[r>>2]=d;c[q+4>>2]=d;c[r+4>>2]=j+2;if((u|0)>0){v=u;break}i=o;return}else{c[q>>2]=j;c[r>>2]=((k|0)<2?k:2)+j;v=1}}while(0);j=(t|0)>0;k=b+204|0;u=b+208|0;n=b+268|0;d=(h|0)==1;w=(h|0)==0;h=0;do{if(j){x=ac(h,t)|0;y=c[q+(h<<2)>>2]|0;z=c[r+(h<<2)>>2]|0;A=c[172720+(z-y<<2)>>2]|0;B=(y|0)<(z|0);C=A;D=(A|0)<0|0?-1:0;A=0;while(1){E=e+(A+x<<2)|0;F=c[b+212+(A<<2)>>2]|0;H=A+1|0;I=c[b+212+(H<<2)>>2]|0;J=c[k>>2]|0;K=c[b+160+(A<<2)>>2]|0;L=c[u>>2]|0;M=c[n>>2]|0;N=c[m+(A<<2)>>2]|0;O=c[172720+(I-F<<2)>>2]|0;P=(F|0)<(I|0);do{if(d){if(P&B){Q=0;R=0;S=F}else{T=0;U=0;break}while(1){V=y;W=0;do{X=c[(c[f+(V<<2)>>2]|0)+(S<<2)>>2]|0;N2(X,(X|0)<0|0?-1:0,C,D)|0;W=G+W|0;V=V+1|0;}while((V|0)<(z|0));V=W<<1;X=(Q|0)>(V|0);Y=a[g+S|0]|0;Z=y;_=0;do{$=c[(c[f+(Z<<2)>>2]|0)+(Y<<2)>>2]|0;N2($,($|0)<0|0?-1:0,C,D)|0;_=G+_|0;Z=Z+1|0;}while((Z|0)<(z|0));Z=X?Q:V;Y=_<<1;W=(R|0)>(Y|0)?R:Y;Y=S+1|0;if((Y|0)<(I|0)){Q=Z;R=W;S=Y}else{T=W;U=Z;break}}}else{if(!P){T=0;U=0;break}Z=O;W=(O|0)<0|0?-1:0;Y=0;$=0;aa=F;while(1){if(B){ab=y;ad=0;do{ae=c[(c[f+(ab<<2)>>2]|0)+(aa<<2)>>2]|0;N2(ae,(ae|0)<0|0?-1:0,C,D)|0;ad=G+ad|0;ab=ab+1|0;}while((ab|0)<(z|0));ab=ad<<1;N2(ab,(ab|0)<0|0?-1:0,Z,W)|0;ab=(G<<1|0>>>31)+Y|0;_=a[g+aa|0]|0;V=y;X=0;do{ae=c[(c[f+(V<<2)>>2]|0)+(_<<2)>>2]|0;N2(ae,(ae|0)<0|0?-1:0,C,D)|0;X=G+X|0;V=V+1|0;}while((V|0)<(z|0));V=X<<1;af=(V|0)<0|0?-1:0;ag=V;ah=ab}else{af=0;ag=0;ah=Y}N2(ag,af,Z,W)|0;V=(G<<1|0>>>31)+$|0;_=aa+1|0;if((_|0)<(I|0)){Y=ah;$=V;aa=_}else{T=V;U=ah;break}}}}while(0);I=(U|0)<3&(T|0)<3;F=I?218171:U;O=I?218171:T;I=(F|0)>2147?F:2147;F=(O|0)>2147?O:2147;do{if(d){ai=2147}else{if(!((N|0)!=0&(N-1|0)>>>0>1&(N|0)>(M|0))){ai=2147;break}O=HQ(F,I,p)|0;P=L;aa=(L|0)<0|0?-1:0;$=O;Y=(O|0)<0|0?-1:0;N2($,Y,P,aa)|0;aa=G<<1|0>>>31;P=aa;Y=(aa|0)<0|0?-1:0;aa=1125899904;$=0;N2(P,Y,aa,$)|0;$=(G<<1|0>>>31)>>19-(c[p>>2]|0);ai=($|0)<2147?2147:$}}while(0);L=HQ(ai,I,p)|0;F=c[p>>2]|0;M=F-2|0;c[p>>2]=M;N=(M|0)>0;do{if(N&(L|0)>(2147483647>>>(M>>>0)|0)){aj=2147483647}else{if(N){aj=L<<M;break}else{aj=L>>2-F;break}}}while(0);if(w){F=aj;L=(aj|0)<0|0?-1:0;M=K;N=(K|0)<0|0?-1:0;N2(F,L,M,N)|0;ak=G<<4|0>>>28}else{ak=aj}c[E>>2]=(ak|0)<(J|0)?ak:J;if((H|0)<(t|0)){A=H}else{break}}}h=h+1|0;}while((h|0)<(v|0));h=c[s>>2]|0;s=c[b+200>>2]|0;ak=(l|0)==0;l=h<<2;aj=b+120|0;w=(h|0)>0;p=b;ai=b+40|0;d=b+80|0;T=s+4|0;U=s+8|0;ah=s+12|0;af=0;do{if(ak){IY(p,ai,l);IY(ai,d,l);IY(d,aj,l);IY(aj,e+((ac(af,h)|0)<<2)|0,l)}else{ag=e+((ac(af,h)|0)<<2)|0;IY(p,ag,l);IY(ai,ag,l);IY(d,ag,l);IY(aj,ag,l)}if(w){ag=ac(af,h)|0;f=0;do{g=c[s>>2]|0;S=c[b+(f<<2)>>2]|0;N2(S,(S|0)<0|0?-1:0,g,(g|0)<0|0?-1:0)|0;g=G;S=c[T>>2]|0;R=c[b+40+(f<<2)>>2]|0;N2(R,(R|0)<0|0?-1:0,S,(S|0)<0|0?-1:0)|0;S=G+g|0;g=c[U>>2]|0;R=c[b+80+(f<<2)>>2]|0;N2(R,(R|0)<0|0?-1:0,g,(g|0)<0|0?-1:0)|0;g=S+G|0;S=c[ah>>2]|0;R=c[b+120+(f<<2)>>2]|0;N2(R,(R|0)<0|0?-1:0,S,(S|0)<0|0?-1:0)|0;c[e+(f+ag<<2)>>2]=g+G<<1;f=f+1|0;}while((f|0)<(h|0))}af=af+1|0;}while((af|0)<(v|0));af=(t|0)>0;h=0;do{if(af){b=ac(h,t)|0;ah=0;do{U=e+(ah+b<<2)|0;c[U>>2]=134217728-(HE((c[U>>2]|0)+1|0)|0);ah=ah+1|0;}while((ah|0)<(t|0))}h=h+1|0;}while((h|0)<(v|0));i=o;return}function L8(a,b,d,e,f,g,h,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0;k=i;i=i+16|0;l=k|0;m=k+8|0;I_(a,272);c[a+200>>2]=1926752;n=a+208|0;if((j|0)==0){c[n>>2]=536870912;c[a+268>>2]=2}else{c[n>>2]=2147483647;c[a+268>>2]=1}c[a+264>>2]=h;c[a+260>>2]=f;if((b|0)==6){c[a+204>>2]=2147483647}else if((b|0)==3){c[a+204>>2]=1073741824}else if((b|0)==(-3|0)){c[a+204>>2]=268435456}else{c[a+204>>2]=2147483647}if((L9(a,d,e)|0)!=0){o=1;i=k;return o|0}do{if((g|0)==0){p=268435455}else{e=HQ(g,3,l)|0;d=HT(2,31,e,c[l>>2]|0,m)|0;e=c[m>>2]|0;b=e-3|0;if((b|0)>0){p=d<<b;break}else{p=d>>3-e;break}}}while(0);m=a+256|0;if((c[m>>2]|0)>0){q=0}else{o=0;i=k;return o|0}while(1){c[a+160+(q<<2)>>2]=p;l=q+1|0;if((l|0)<(c[m>>2]|0)){q=l}else{o=0;break}}i=k;return o|0}function L9(a,b,e){a=a|0;b=b|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;f=i;i=i+112|0;g=f|0;h=f+96|0;j=f+104|0;k=a+260|0;do{if((c[k>>2]|0)==0){c[a+256>>2]=1;l=1}else{m=HQ(d[b+e|0]|0,d[b]|0,h)|0;n=HS(m,c[h>>2]|0,j)|0;m=c[k>>2]<<24;o=m;p=(m|0)<0|0?-1:0;m=n;q=(n|0)<0|0?-1:0;N2(o,p,m,q)|0;q=G<<1|0>>>31;m=c[j>>2]|0;p=m-23|0;if((p|0)>0){r=q<<p}else{r=q>>23-m}m=r+1>>1;q=a+256|0;p=(m|0)>5?5:m;c[q>>2]=p;if((p|0)!=0){l=p;break}c[q>>2]=1;l=1}}while(0);c[g>>2]=0;if((e|0)>0){r=0;j=e;e=l;k=0;while(1){h=r+1|0;q=(j|0)/(e|0)|0;p=j-q|0;m=q+k|0;c[g+(h<<2)>>2]=m;if((p|0)>0){r=h;j=p;e=e-1|0;k=m}else{s=h;break}}}else{s=0}if((s|0)!=(l|0)){t=1;i=f;return t|0}if((l|0)<0){t=0;i=f;return t|0}else{u=0}while(1){c[a+212+(u<<2)>>2]=d[b+(c[g+(u<<2)>>2]|0)|0]|0;s=u+1|0;if((s|0)>(l|0)){t=0;break}else{u=s}}i=f;return t|0}function Ma(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0;do{if((g|0)==0){if((f|0)==0){i=Mb(a,b,e,1916248,1915368,14,28,h)|0;break}else if((f|0)==1){i=Mb(a,b,e,1917584,1916e3,30,60,h)|0;break}else{c[h>>2]=1;i=0;break}}else if((g|0)==1){if((f|0)==1){i=Mc(a,b,d,e,1917336,1915632,30,60,h)|0;break}else if((f|0)==0){i=Mc(a,b,d,e,1915880,1915248,14,28,h)|0;break}else{c[h>>2]=1;i=0;break}}else{c[h>>2]=1;i=0}}while(0);return i|0}function Mb(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;if((d|0)<=0){j=0;return j|0}k=a+4|0;l=a|0;m=a+8|0;if((a|0)==0){a=0;n=0;o=0;while(1){p=c[b+(o<<2)>>2]|0;q=p-n+g|0;if((q|0)>(h|0)|(q|0)<0){c[i>>2]=1;r=(q|0)>0?h:0}else{r=q}q=(c[f+(r<<2)>>2]&255)+a|0;s=o+1|0;if((s|0)<(d|0)){a=q;n=p;o=s}else{j=q;break}}return j|0}else{t=0;u=0;v=0}while(1){o=c[b+(v<<2)>>2]|0;n=o-u+g|0;if((n|0)>(h|0)|(n|0)<0){c[i>>2]=1;w=(n|0)>0?h:0}else{w=n}n=c[e+(w<<2)>>2]|0;a=c[f+(w<<2)>>2]|0;r=c[1991416+(a<<2)>>2]|0;q=c[k>>2]|0;s=q+a|0;if(s>>>0<32){c[k>>2]=s;x=c[l>>2]<<a|r&n}else{Hw(m,c[l>>2]|0,q);c[k>>2]=a;x=r&n}c[l>>2]=x;n=(a&255)+t|0;a=v+1|0;if((a|0)<(d|0)){t=n;u=o;v=a}else{j=n;break}}return j|0}function Mc(a,b,d,e,f,g,h,i,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;if((e|0)<=0){k=0;return k|0}l=a+4|0;m=a|0;n=a+8|0;if((a|0)==0){a=0;o=0;while(1){p=(c[b+(o<<2)>>2]|0)-(c[d+(o<<2)>>2]|0)+h|0;if((p|0)>(i|0)|(p|0)<0){c[j>>2]=1;q=(p|0)>0?i:0}else{q=p}p=(c[g+(q<<2)>>2]&255)+a|0;r=o+1|0;if((r|0)<(e|0)){a=p;o=r}else{k=p;break}}return k|0}else{s=0;t=0}while(1){o=(c[b+(t<<2)>>2]|0)-(c[d+(t<<2)>>2]|0)+h|0;if((o|0)>(i|0)|(o|0)<0){c[j>>2]=1;u=(o|0)>0?i:0}else{u=o}o=c[f+(u<<2)>>2]|0;a=c[g+(u<<2)>>2]|0;q=c[1991416+(a<<2)>>2]|0;p=c[l>>2]|0;r=p+a|0;if(r>>>0<32){c[l>>2]=r;v=c[m>>2]<<a|q&o}else{Hw(n,c[m>>2]|0,p);c[l>>2]=a;v=q&o}c[m>>2]=v;o=(a&255)+s|0;a=t+1|0;if((a|0)<(e|0)){s=o;t=a}else{k=o;break}}return k|0}function Md(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0;if((f|0)==0){h=Mb(a,b,e,1921536,1919680,7,14,g)|0;return h|0}else if((f|0)==1){h=Mc(a,b,d,e,1921472,1919616,7,14,g)|0;return h|0}else{c[g>>2]=1;h=0;return h|0}return 0}function Me(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0;d=i;i=i+8|0;e=d|0;c[e>>2]=0;if((a|0)==0){f=0;i=d;return f|0}g=a|0;h=c[g>>2]|0;j=(b|0)==0;if(j){k=h}else{l=c[497855]|0;m=b+4|0;n=c[m>>2]|0;o=n+1|0;if(o>>>0<32){c[m>>2]=o;o=b|0;c[o>>2]=c[o>>2]<<1|l&h}else{o=b|0;Hw(b+8|0,c[o>>2]|0,n);c[m>>2]=1;c[o>>2]=l&h}k=c[g>>2]|0}if((k|0)==0){p=4;q=0}else{k=a+4|0;g=c[k>>2]|0;if(j){r=g}else{h=c[497855]|0;l=b+4|0;o=c[l>>2]|0;m=o+1|0;if(m>>>0<32){c[l>>2]=m;m=b|0;c[m>>2]=c[m>>2]<<1|h&g}else{m=b|0;Hw(b+8|0,c[m>>2]|0,o);c[l>>2]=1;c[m>>2]=h&g}r=c[k>>2]|0}do{if((r|0)==0){s=2;t=13018}else{k=c[a+8>>2]|0;if(j){u=5;v=a+12|0;break}g=c[497857]|0;h=b+4|0;m=c[h>>2]|0;l=m+3|0;if(l>>>0<32){c[h>>2]=l;o=b|0;c[o>>2]=c[o>>2]<<3|g&k;w=5;x=a+12|0;y=l;t=13020;break}else{l=b|0;Hw(b+8|0,c[l>>2]|0,m);c[h>>2]=3;c[l>>2]=g&k;s=5;t=13018;break}}}while(0);do{if((t|0)==13018){r=a+12|0;if(j){u=s;v=r;break}w=s;x=r;y=c[b+4>>2]|0;t=13020}}while(0);do{if((t|0)==13020){s=c[x>>2]|0;r=c[497855]|0;k=b+4|0;g=y+1|0;if(g>>>0<32){c[k>>2]=g;g=b|0;c[g>>2]=c[g>>2]<<1|r&s;u=w;v=x;break}else{g=b|0;Hw(b+8|0,c[g>>2]|0,y);c[k>>2]=1;c[g>>2]=r&s;u=w;v=x;break}}}while(0);if((c[v>>2]|0)==0){z=u+1|0}else{v=c[a+16>>2]|0;do{if(!j){x=c[497857]|0;w=b+4|0;y=c[w>>2]|0;s=y+3|0;if(s>>>0<32){c[w>>2]=s;s=b|0;c[s>>2]=c[s>>2]<<3|x&v;break}else{s=b|0;Hw(b+8|0,c[s>>2]|0,y);c[w>>2]=3;c[s>>2]=x&v;break}}}while(0);z=u+4|0}u=(c[a+20>>2]|0)!=0|0;do{if(!j){v=c[497855]|0;x=b+4|0;s=c[x>>2]|0;w=s+1|0;if(w>>>0<32){c[x>>2]=w;w=b|0;c[w>>2]=c[w>>2]<<1|v&u;break}else{w=b|0;Hw(b+8|0,c[w>>2]|0,s);c[x>>2]=1;c[w>>2]=v&u;break}}}while(0);p=z+4|0;q=u}u=a+24|0;z=c[u>>2]|0;do{if(!j){v=c[497855]|0;w=b+4|0;x=c[w>>2]|0;s=x+1|0;if(s>>>0<32){c[w>>2]=s;s=b|0;c[s>>2]=c[s>>2]<<1|v&z;break}else{s=b|0;Hw(b+8|0,c[s>>2]|0,x);c[w>>2]=1;c[s>>2]=v&z;break}}}while(0);z=a+28|0;v=c[z>>2]|0;s=c[u>>2]|0;if((v|0)==1){A=(s|0)==0|0}else if((v|0)==2){A=(s|0)==0?2:1}else if((v|0)==3){A=2}else if((v|0)==4){A=3}else{A=0}do{if(!j){v=c[497856]|0;s=b+4|0;w=c[s>>2]|0;x=w+2|0;if(x>>>0<32){c[s>>2]=x;x=b|0;c[x>>2]=c[x>>2]<<2|v&A;break}else{x=b|0;Hw(b+8|0,c[x>>2]|0,w);c[s>>2]=2;c[x>>2]=v&A;break}}}while(0);L18211:do{if((c[u>>2]|0)==1){A=c[z>>2]|0;if((A|0)<=0){B=p;break}v=c[497859]|0;x=b+4|0;s=b|0;w=b+8|0;if(j){y=p;r=0;while(1){g=y+5|0;k=r+1|0;if((k|0)<(A|0)){y=g;r=k}else{B=g;break L18211}}}else{C=p;D=0}while(1){r=c[a+32+(D<<2)>>2]|0;y=c[x>>2]|0;A=y+5|0;if(A>>>0<32){c[x>>2]=A;E=c[s>>2]<<5|v&r}else{Hw(w,c[s>>2]|0,y);c[x>>2]=5;E=v&r}c[s>>2]=E;r=C+5|0;y=D+1|0;if((y|0)<(c[z>>2]|0)){C=r;D=y}else{B=r;break}}}else{B=p}}while(0);do{if((c[a+4>>2]|0)==1){if((c[z>>2]|0)<=0){F=B;break}p=a+8|0;D=c[497855]|0;C=b+4|0;E=b|0;u=b+8|0;s=B;v=0;x=a+384|0;while(1){w=a+48+(v<<2)|0;r=c[w>>2]|0;do{if(!j){y=c[C>>2]|0;A=y+1|0;if(A>>>0<32){c[C>>2]=A;c[E>>2]=c[E>>2]<<1|D&r;break}else{Hw(u,c[E>>2]|0,y);c[C>>2]=1;c[E>>2]=D&r;break}}}while(0);r=a+64+(v*80|0)|0;y=c[p>>2]|0;if((y|0)>5){G=0}else{G=c[1914784+(((y|0)>2?y-3|0:y)<<2)>>2]|0}A=s+1+(Ma(b,r,x,G,(y|0)>2|0,c[w>>2]|0,e)|0)|0;y=v+1|0;if((y|0)<(c[z>>2]|0)){s=A;v=y;x=r}else{F=A;break}}}else{F=B}}while(0);do{if((c[a+12>>2]|0)==1){if((c[z>>2]|0)<=0){H=F;break}B=a+16|0;G=c[497855]|0;x=b+4|0;v=b|0;s=b+8|0;p=F;D=0;E=a+800|0;while(1){C=a+464+(D<<2)|0;u=c[C>>2]|0;do{if(!j){A=c[x>>2]|0;r=A+1|0;if(r>>>0<32){c[x>>2]=r;c[v>>2]=c[v>>2]<<1|G&u;break}else{Hw(s,c[v>>2]|0,A);c[x>>2]=1;c[v>>2]=G&u;break}}}while(0);u=p+1|0;w=a+480+(D*80|0)|0;A=c[B>>2]|0;if((A|0)>5){I=0}else{I=c[1914784+(((A|0)>2?A-3|0:A)<<2)>>2]|0}A=c[C>>2]|0;if((A|0)==0){J=Mb(b,w,I,1921536,1919680,7,14,e)|0}else if((A|0)==1){J=Mc(b,w,E,I,1921472,1919616,7,14,e)|0}else{c[e>>2]=1;J=0}A=u+J|0;u=D+1|0;if((u|0)<(c[z>>2]|0)){p=A;D=u;E=w}else{H=A;break}}}else{H=F}}while(0);if((q|0)==0){f=H;i=d;return f|0}if((c[a+20>>2]|0)==1){q=(Mf(a,0)|0)+9>>3;L18264:do{if((q|0)<15){if(j){K=4;L=1;break}F=c[497858]|0;z=b+4|0;J=c[z>>2]|0;e=J+4|0;if(e>>>0<32){c[z>>2]=e;I=b|0;E=c[I>>2]<<4|F&q;c[I>>2]=E;M=4;N=e;O=E;P=z;t=13094;break}else{E=b|0;Hw(b+8|0,c[E>>2]|0,J);c[z>>2]=4;z=F&q;c[E>>2]=z;Q=4;R=4;S=z;t=13092;break}}else{if(j){K=12;L=1;break}z=c[497858]|0;E=b+4|0;F=c[E>>2]|0;J=F+4|0;do{if(J>>>0<32){c[E>>2]=J;e=b|0;I=c[e>>2]<<4|z&15;c[e>>2]=I;D=q-15|0;p=c[497862]|0;B=F+12|0;if(B>>>0<32){T=I;U=D;V=p;W=B;break}Hw(b+8|0,I,J);c[E>>2]=8;I=p&D;c[e>>2]=I;Q=12;R=8;S=I;t=13092;break L18264}else{I=b|0;Hw(b+8|0,c[I>>2]|0,F);c[E>>2]=4;e=z&15;c[I>>2]=e;T=e;U=q-15|0;V=c[497862]|0;W=12}}while(0);c[E>>2]=W;z=T<<8|U&V;c[b>>2]=z;M=12;N=W;O=z;P=E;t=13094}}while(0);do{if((t|0)==13092){if(j){K=Q;L=1;break}M=Q;N=R;O=S;P=b+4|0;t=13094}}while(0);do{if((t|0)==13094){S=N+2|0;if(S>>>0<32){c[P>>2]=S;c[b>>2]=O<<2;K=M;L=0;break}else{Hw(b+8|0,O,N);c[P>>2]=2;c[b>>2]=0;K=M;L=0;break}}}while(0);M=(Mf(a,b)|0)+2|0;a=(M|0)%8|0;if((a|0)==0){X=M}else{P=8-a|0;do{if(!L){a=b+4|0;N=c[a>>2]|0;O=N+P|0;if(O>>>0<32){c[a>>2]=O;O=b|0;c[O>>2]=c[O>>2]<<P;break}else{O=b|0;Hw(b+8|0,c[O>>2]|0,N);c[a>>2]=P;c[O>>2]=0;break}}}while(0);X=(P&255)+M|0}Y=X+K|0}else{Y=0}f=Y+H|0;i=d;return f|0}function Mf(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;d=i;i=i+8|0;e=d|0;c[e>>2]=0;f=a+20|0;g=c[f>>2]|0;h=(b|0)==0;if(h){j=g}else{k=c[497855]|0;l=b+4|0;m=c[l>>2]|0;n=m+1|0;if(n>>>0<32){c[l>>2]=n;n=b|0;c[n>>2]=c[n>>2]<<1|k&g}else{n=b|0;Hw(b+8|0,c[n>>2]|0,m);c[l>>2]=1;c[n>>2]=k&g}j=c[f>>2]|0}if((j|0)!=1){o=0;i=d;return o|0}j=a+1216|0;f=a+1632|0;g=a+28|0;if((c[g>>2]|0)>0){k=a+8|0;n=c[497855]|0;l=b+4|0;m=b|0;p=b+8|0;q=0;r=1;while(1){s=a+880+(q<<2)|0;t=c[s>>2]|0;do{if(!h){u=c[l>>2]|0;v=u+1|0;if(v>>>0<32){c[l>>2]=v;c[m>>2]=c[m>>2]<<1|n&t;break}else{Hw(p,c[m>>2]|0,u);c[l>>2]=1;c[m>>2]=n&t;break}}}while(0);t=a+896+(q*80|0)|0;u=c[k>>2]|0;if((u|0)>5){w=0}else{w=c[1914784+(((u|0)>2?u-3|0:u)<<2)>>2]|0}u=c[s>>2]|0;if((u|0)==0){x=Mb(b,t,w,1921440,1919584,0,7,e)|0}else if((u|0)==1){x=Mc(b,t,j,w,1921408,1919552,0,7,e)|0}else{c[e>>2]=1;x=0}t=a+1296+(q<<2)|0;u=c[t>>2]|0;do{if(!h){v=c[l>>2]|0;y=v+1|0;if(y>>>0<32){c[l>>2]=y;c[m>>2]=c[m>>2]<<1|n&u;break}else{Hw(p,c[m>>2]|0,v);c[l>>2]=1;c[m>>2]=n&u;break}}}while(0);u=a+1312+(q*80|0)|0;s=c[k>>2]|0;if((s|0)>5){z=0}else{z=c[1914784+(((s|0)>2?s-3|0:s)<<2)>>2]|0}s=c[t>>2]|0;if((s|0)==0){A=Mb(b,u,z,1921104,1919520,0,7,e)|0}else if((s|0)==1){A=Mc(b,u,f,z,1921072,1919488,0,7,e)|0}else{c[e>>2]=1;A=0}u=q+1|0;s=r+2+x+A|0;if((u|0)<(c[g>>2]|0)){q=u;r=s}else{B=s;break}}}else{B=1}if(h){o=B;i=d;return o|0}h=b+4|0;r=c[h>>2]|0;q=r+1|0;if(q>>>0<32){c[h>>2]=q;q=b|0;c[q>>2]=c[q>>2]<<1;o=B;i=d;return o|0}else{q=b|0;Hw(b+8|0,c[q>>2]|0,r);c[h>>2]=1;c[q>>2]=0;o=B;i=d;return o|0}return 0}function Mg(a){a=a|0;var b=0,d=0;do{if((a|0)==0){b=32}else{d=Jy(0)|0;if((d|0)==0){b=33;break}I_(d,1356);c[a>>2]=d;b=0}}while(0);return b|0}function Mh(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0;if((b|0)==0){f=32;return f|0}g=b;I_(g,884);NM(g|0,0,16);NM(b+352|0,0,96);NM(b+784|0,0,80);g=0;do{c[b+448+(g<<2)>>2]=0;c[b+32+(g*80|0)>>2]=0;c[b+464+(g*80|0)>>2]=0;c[b+32+(g*80|0)+4>>2]=0;c[b+464+(g*80|0)+4>>2]=0;c[b+32+(g*80|0)+8>>2]=0;c[b+464+(g*80|0)+8>>2]=0;c[b+32+(g*80|0)+12>>2]=0;c[b+464+(g*80|0)+12>>2]=0;c[b+32+(g*80|0)+16>>2]=0;c[b+464+(g*80|0)+16>>2]=0;c[b+32+(g*80|0)+20>>2]=0;c[b+464+(g*80|0)+20>>2]=0;c[b+32+(g*80|0)+24>>2]=0;c[b+464+(g*80|0)+24>>2]=0;c[b+32+(g*80|0)+28>>2]=0;c[b+464+(g*80|0)+28>>2]=0;c[b+32+(g*80|0)+32>>2]=0;c[b+464+(g*80|0)+32>>2]=0;c[b+32+(g*80|0)+36>>2]=0;c[b+464+(g*80|0)+36>>2]=0;c[b+32+(g*80|0)+40>>2]=0;c[b+464+(g*80|0)+40>>2]=0;c[b+32+(g*80|0)+44>>2]=0;c[b+464+(g*80|0)+44>>2]=0;c[b+32+(g*80|0)+48>>2]=0;c[b+464+(g*80|0)+48>>2]=0;c[b+32+(g*80|0)+52>>2]=0;c[b+464+(g*80|0)+52>>2]=0;c[b+32+(g*80|0)+56>>2]=0;c[b+464+(g*80|0)+56>>2]=0;c[b+32+(g*80|0)+60>>2]=0;c[b+464+(g*80|0)+60>>2]=0;c[b+32+(g*80|0)+64>>2]=0;c[b+464+(g*80|0)+64>>2]=0;c[b+32+(g*80|0)+68>>2]=0;c[b+464+(g*80|0)+68>>2]=0;c[b+32+(g*80|0)+72>>2]=0;c[b+464+(g*80|0)+72>>2]=0;c[b+32+(g*80|0)+76>>2]=0;c[b+464+(g*80|0)+76>>2]=0;g=g+1|0;}while((g|0)<4);c[b+864>>2]=0;c[b+868>>2]=10;c[b+872>>2]=20;c[b+876>>2]=20;c[b+880>>2]=10;if(!((d|0)==10|(d|0)==20)){f=64;return f|0}g=b+888|0;c[g>>2]=12;h=b+892|0;c[h>>2]=10;IY(b+896|0,1918312,92);IY(b+1092|0,1919400,(c[h>>2]|0)+(c[g>>2]|0)<<2);IY(b+1284|0,1918288,(c[h>>2]|0)+(c[g>>2]|0)|0);i=b+884|0;c[i>>2]=d;c[b+1332>>2]=e;e=(c[h>>2]|0)+(c[g>>2]|0)|0;I_(b+1336|0,20);if((e|0)>0){j=0}else{f=0;return f|0}while(1){g=(c[b+1092+(j<<2)>>2]>>((c[i>>2]|0)==10))+(b+1336)|0;h=a[g]|0;d=a[b+1284+j|0]|0;if(h<<24>>24==0){k=d+5&255}else{k=((d&255)>(h&255)?d:h)+1&255}a[g]=k;g=j+1|0;if((g|0)<(e|0)){j=g}else{f=0;break}}return f|0}function Mi(a){a=a|0;if((a|0)==0){return 0}Jz(a);return 0}function Mj(a,b,e,f,g,h,j){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0;k=i;i=i+3272|0;l=k|0;m=k+320|0;n=k+640|0;o=k+648|0;p=k+664|0;q=k+680|0;r=k+688|0;s=k+1008|0;t=k+1328|0;u=k+1352|0;v=a+884|0;w=c[v>>2]|0;x=(c[a+892>>2]|0)+(c[a+888>>2]|0)|0;y=f>>>0<4?f:4;f=y+1|0;if((f|0)>0){z=c[172720+(y<<2)>>2]|0;A=h<<16;h=z;B=(z|0)<0|0?-1:0;z=0;do{C=ac(A,z)|0;N2(h,B,C,(C|0)<0|0?-1:0)|0;c[t+(z<<2)>>2]=G+16384>>15;z=z+1|0;}while((z|0)<(f|0))}f=(y|0)>0;do{if(f){z=(w|0)>0;B=(x|0)>0;h=0;while(1){if(z){A=0;do{c[u+1600+(h*80|0)+(A<<2)>>2]=1;c[u+1280+(h*80|0)+(A<<2)>>2]=1;c[u+320+(h*80|0)+(A<<2)>>2]=1;c[u+(h*80|0)+(A<<2)>>2]=1;A=A+1|0;}while((A|0)<(w|0))}if(B){A=c[t+(h<<2)>>2]|0;C=h+1|0;D=c[t+(C<<2)>>2]|0;E=(A|0)<(D|0);F=c[v>>2]|0;H=0;while(1){I=c[a+1092+(H<<2)>>2]>>((F|0)==10);J=d[a+1336+I|0]|0;K=u+(h*80|0)+(I<<2)|0;L=c[K>>2]|0;M=u+320+(h*80|0)+(I<<2)|0;N=c[M>>2]|0;O=u+1280+(h*80|0)+(I<<2)|0;P=c[O>>2]|0;Q=u+1600+(h*80|0)+(I<<2)|0;R=c[Q>>2]|0;S=d[e+I|0]|0;if(E){I=c[a+896+(H<<2)>>2]|0;T=H+1|0;U=c[a+896+(T<<2)>>2]|0;V=(I|0)<(U|0);W=L;X=N;Y=P;Z=R;_=A;while(1){if(V){$=c[g+(_<<4)>>2]|0;aa=c[g+(_<<4)+4>>2]|0;ab=c[g+(_<<4)+8>>2]|0;ad=c[g+(_<<4)+12>>2]|0;ae=W;af=X;ag=Y;ah=Z;ai=I;while(1){aj=c[$+(ai<<2)>>2]<<S;ak=c[aa+(ai<<2)>>2]<<S;al=c[ab+(ai<<2)>>2]<<S;am=c[ad+(ai<<2)>>2]<<S;an=aj;ao=(aj|0)<0|0?-1:0;N2(an,ao,an,ao)|0;aj=G;ap=ak;aq=(ak|0)<0|0?-1:0;N2(ap,aq,ap,aq)|0;ak=((G+aj|0)>>>(J>>>0))+ae|0;aj=al;ar=(al|0)<0|0?-1:0;N2(aj,ar,aj,ar)|0;al=G;as=am;at=(am|0)<0|0?-1:0;N2(as,at,as,at)|0;am=((G+al|0)>>>(J>>>0))+af|0;N2(aj,ar,an,ao)|0;al=G;N2(as,at,ap,aq)|0;au=(G+al>>J)+ag|0;N2(aj,ar,ap,aq)|0;aq=G;N2(as,at,an,ao)|0;ao=(aq-G>>J)+ah|0;aq=ai+1|0;if((aq|0)<(U|0)){ae=ak;af=am;ag=au;ah=ao;ai=aq}else{av=ak;aw=am;ax=au;ay=ao;break}}}else{av=W;aw=X;ax=Y;ay=Z}ai=_+1|0;if((ai|0)<(D|0)){W=av;X=aw;Y=ax;Z=ay;_=ai}else{az=av;aA=aw;aB=ax;aC=ay;aD=T;break}}}else{az=L;aA=N;aB=P;aC=R;aD=H+1|0}c[K>>2]=(az|0)<0?0:az;c[M>>2]=(aA|0)<0?0:aA;c[O>>2]=aB;c[Q>>2]=aC;if((aD|0)<(x|0)){H=aD}else{aE=C;break}}}else{aE=h+1|0}HG(u+(h*80|0)|0,u+640+(h*80|0)|0,w);HG(u+320+(h*80|0)|0,u+960+(h*80|0)|0,w);if((aE|0)<(y|0)){h=aE}else{break}}h=u+640|0;B=u+960|0;z=(w|0)>0;if(f&z){aF=0}else{aG=h;aH=B;aI=z;break}while(1){C=0;do{H=(c[u+640+(aF*80|0)+(C<<2)>>2]|0)-(c[u+960+(aF*80|0)+(C<<2)>>2]|0)|0;N2(H,(H|0)<0|0?-1:0,101008904,0)|0;H=G;D=(H|0)<16777215?H:16777215;c[r+(aF*80|0)+(C<<2)>>2]=(D|0)>-16777216?D<<7:-2147483648;C=C+1|0;}while((C|0)<(w|0));C=aF+1|0;if((C|0)<(y|0)){aF=C}else{aG=h;aH=B;aI=z;break}}}else{aG=u+640|0;aH=u+960|0;aI=(w|0)>0}}while(0);aF=u+1280|0;f=u+1600|0;aE=s|0;Mk(aG,aH,aF,f,aE,y,w);aD=w<<25;x=aD;aC=(aD|0)<0|0?-1:0;N2(x,aC,22151168,0)|0;aD=G;aB=aD;N2(x,aC,1207959552,0)|0;aC=G;x=aC;L18417:do{if((y|0)<2){aJ=y}else{aA=(aD|aC|0)<0;az=y;while(1){ay=(az|0)/2|0;if(aI){ax=0;aw=1;while(1){av=ax<<1;g=av|1;e=0;v=0;z=0;do{B=(c[r+(av*80|0)+(e<<2)>>2]>>1)-(c[r+(g*80|0)+(e<<2)>>2]>>1)|0;h=(c[s+(av*80|0)+(e<<2)>>2]>>1)-(c[s+(g*80|0)+(e<<2)>>2]>>1)|0;C=B;D=(B|0)<0|0?-1:0;N2(C,D,C,D)|0;v=(G>>>4|0<<28)+v|0;D=h;C=(h|0)<0|0?-1:0;N2(D,C,D,C)|0;z=(G>>>4|0<<28)+z|0;e=e+1|0;}while((e|0)<(w|0));e=(v|0)>(aB|0)|(z|0)>(x|0)?0:aw;g=ax+1|0;if((g|0)>=(ay|0)|(e|0)==0){aK=e;break}else{ax=g;aw=e}}}else{aw=0;ax=1;while(1){e=aA?0:ax;g=aw+1|0;if((g|0)>=(ay|0)|(e|0)==0){aK=e;break}else{aw=g;ax=e}}}if((aK|0)==0){aJ=az;break L18417}aL=az>>1;ax=(aL|0)>0;if(ax){aM=0}else{break}do{aw=aM<<1;ay=aw|1;e=u+(aM*80|0)|0;if(aI){g=0;do{c[u+(aM*80|0)+(g<<2)>>2]=(c[u+(ay*80|0)+(g<<2)>>2]>>1)+(c[u+(aw*80|0)+(g<<2)>>2]>>1);g=g+1|0;}while((g|0)<(w|0));g=0;while(1){c[u+320+(aM*80|0)+(g<<2)>>2]=(c[u+320+(ay*80|0)+(g<<2)>>2]>>1)+(c[u+320+(aw*80|0)+(g<<2)>>2]>>1);z=g+1|0;if((z|0)<(w|0)){g=z}else{aN=0;break}}while(1){c[u+1280+(aM*80|0)+(aN<<2)>>2]=(c[u+1280+(ay*80|0)+(aN<<2)>>2]>>1)+(c[u+1280+(aw*80|0)+(aN<<2)>>2]>>1);g=aN+1|0;if((g|0)<(w|0)){aN=g}else{aO=0;break}}do{c[u+1600+(aM*80|0)+(aO<<2)>>2]=(c[u+1600+(ay*80|0)+(aO<<2)>>2]>>1)+(c[u+1600+(aw*80|0)+(aO<<2)>>2]>>1);aO=aO+1|0;}while((aO|0)<(w|0))}HG(e,u+640+(aM*80|0)|0,w);HG(u+320+(aM*80|0)|0,u+960+(aM*80|0)|0,w);c[t+(aM<<2)>>2]=c[t+(aw<<2)>>2];aM=aM+1|0;}while((aM|0)<(aL|0));c[t+(aL<<2)>>2]=c[t+(aL<<1<<2)>>2];if(ax&aI){ay=0;do{g=0;do{z=(c[u+640+(ay*80|0)+(g<<2)>>2]|0)-(c[u+960+(ay*80|0)+(g<<2)>>2]|0)|0;N2(z,(z|0)<0|0?-1:0,101008904,0)|0;z=G;v=(z|0)<16777215?z:16777215;c[r+(ay*80|0)+(g<<2)>>2]=(v|0)>-16777216?v<<7:-2147483648;g=g+1|0;}while((g|0)<(w|0));ay=ay+1|0;}while((ay|0)<(aL|0))}Mk(aG,aH,aF,f,aE,aL,w);if((aL|0)<2){aJ=aL;break L18417}else{az=aL}}c[t+(aL<<2)>>2]=c[t+(aL<<1<<2)>>2];Mk(aG,aH,aF,f,aE,aL,w);aJ=aL}}while(0);if((j|0)!=0){c[a+868>>2]=10;c[a+872>>2]=20;c[a+876>>2]=20;c[a+880>>2]=10}j=c[a+1332>>2]|0;c[n>>2]=0;aL=(aJ|0)>0;L18458:do{if(aL){aE=0;f=0;aF=0;while(1){if(aI){aH=0;aG=0;while(1){u=c[r+(f*80|0)+(aH<<2)>>2]|0;aM=u>>1;aO=0;while(1){if((aO|0)>=14){aP=13224;break}aN=aO+1|0;aK=aM-(c[1927744+(aN<<2)>>2]>>1)|0;x=c[1927744+(aO<<2)>>2]|0;aB=aM-(x>>1)|0;if((((aK|0)>0?aK:-aK|0)|0)>(((aB|0)>0?aB:-aB|0)|0)){aQ=x;break}else{aO=aN}}if((aP|0)==13224){aP=0;aQ=c[1927744+(aO<<2)>>2]|0}aM=u-aQ|0;aR=(((aM|0)>0?aM:-aM|0)>>6)+aG|0;c[m+(f*80|0)+(aH<<2)>>2]=aO-7;aM=aH+1|0;if((aM|0)<(w|0)){aH=aM;aG=aR}else{aS=0;aT=0;break}}do{aG=c[r+(f*80|0)+(aS<<2)>>2]|0;aH=aG>>1;aM=0;while(1){if((aM|0)>=30){aP=13229;break}aN=aM+1|0;x=aH-(c[1924448+(aN<<2)>>2]>>1)|0;aB=c[1924448+(aM<<2)>>2]|0;aK=aH-(aB>>1)|0;if((((x|0)>0?x:-x|0)|0)>(((aK|0)>0?aK:-aK|0)|0)){aU=aB;break}else{aM=aN}}if((aP|0)==13229){aP=0;aU=c[1924448+(aM<<2)>>2]|0}aH=aG-aU|0;aT=(((aH|0)>0?aH:-aH|0)>>6)+aT|0;c[l+(f*80|0)+(aS<<2)>>2]=aM-15;aS=aS+1|0;}while((aS|0)<(w|0));aV=aT;aW=aR+aF|0}else{aV=0;aW=aF}aX=aV+aE|0;aH=f+1|0;if((aH|0)<(aJ|0)){aE=aX;f=aH;aF=aW}else{break}}aF=a|0;c[aF>>2]=0;do{if(aI){f=0;aE=0;while(1){aH=0;aY=aE;do{aO=c[m+(f*80|0)+(aH<<2)>>2]|0;aY=((aO|0)>0?aO:-aO|0)+aY|0;aH=aH+1|0;}while((aH|0)<(w|0));aH=f+1|0;if((aH|0)<(aJ|0)){f=aH;aE=aY}else{break}}aE=(ac(aJ,w)|0)<<16;N2(aE,(aE|0)<0|0?-1:0,1503238528,0)|0;if((aY|0)<=(G+16384>>15|0)){aZ=aI;break}c[aF>>2]=1;aE=m|0;f=Ma(0,aE,0,w,0,0,n)|0;aH=a+872|0;do{if((c[aH>>2]|0)>19){a_=8388607}else{if((c[a+12>>2]|0)==1){a_=8388607;break}a_=Ma(0,aE,a+352|0,w,0,1,n)|0}}while(0);aE=(a_|0)>(f|0);aM=aE?f:a_;c[o>>2]=aE&1^1;aE=(aJ|0)>1;if(aE){aG=aM;aO=1;while(1){u=m+(aO*80|0)|0;aN=Ma(0,u,0,w,0,0,n)|0;aB=Ma(0,u,m+((aO-1|0)*80|0)|0,w,0,1,n)|0;u=(aB|0)>(aN|0);c[o+(aO<<2)>>2]=u&1^1;aK=(u?aN:aB)+aG|0;aB=aO+1|0;if((aB|0)<(aJ|0)){aG=aK;aO=aB}else{a$=aK;break}}}else{a$=aM}aO=l|0;aG=Ma(0,aO,0,w,1,0,n)|0;do{if((c[aH>>2]|0)>19){a0=8388607}else{if((c[a+12>>2]|0)==0){a0=8388607;break}a0=Ma(0,aO,a+352|0,w,1,1,n)|0}}while(0);aO=(a0|0)>(aG|0);aM=aO?aG:a0;c[p>>2]=aO&1^1;if(aE){aO=aM;f=1;while(1){aK=l+(f*80|0)|0;aB=Ma(0,aK,0,w,1,0,n)|0;aN=Ma(0,aK,l+((f-1|0)*80|0)|0,w,1,1,n)|0;aK=(aN|0)>(aB|0);c[p+(f<<2)>>2]=aK&1^1;u=(aK?aB:aN)+aO|0;aN=f+1|0;if((aN|0)<(aJ|0)){aO=u;f=aN}else{a1=u;break}}}else{a1=aM}do{if((a1|0)==(a$|0)){f=(aX|0)<(aW|0);a2=f?8388607:a$;a3=f?a$:8388607}else{f=(ac(w*104857|0,aJ)|0)>>1;aO=(aX>>1)+f>>1;aE=j;aG=(j|0)<0|0?-1:0;u=aX;aN=(aX|0)<0|0?-1:0;N2(u,aN,aE,aG)|0;aN=G<<1|0>>>31;if((((aO|0)>(aN|0)?aO:aN)|0)<(aW>>2|0)){a2=8388607;a3=a1;break}aN=(aW>>1)+f>>1;f=aW;aO=(aW|0)<0|0?-1:0;N2(f,aO,aE,aG)|0;aG=G<<1|0>>>31;a2=a$;a3=(((aN|0)>(aG|0)?aN:aG)|0)<(aX>>2|0)?8388607:a1}}while(0);aM=a+8|0;if((a3|0)<(a2|0)){c[aM>>2]=1;if(!aL){a4=aI;break L18458}aG=w<<2;aN=0;do{c[a+16+(aN<<2)>>2]=c[p+(aN<<2)>>2];IY(a+32+(aN*80|0)|0,l+(aN*80|0)|0,aG);aN=aN+1|0;}while((aN|0)<(aJ|0))}else{c[aM>>2]=0;if(!aL){a4=aI;break L18458}aN=w<<2;aG=0;do{c[a+16+(aG<<2)>>2]=c[o+(aG<<2)>>2];IY(a+32+(aG*80|0)|0,m+(aG*80|0)|0,aN);aG=aG+1|0;}while((aG|0)<(aJ|0))}if(aL){a5=0}else{a4=aI;break L18458}while(1){if((c[a+16+(a5<<2)>>2]|0)==1){a6=(c[aH>>2]|0)+1|0}else{a6=0}c[aH>>2]=a6;aG=a5+1|0;if((aG|0)<(aJ|0)){a5=aG}else{a4=aI;break L18458}}}else{aZ=0}}while(0);c[a+872>>2]=20;if(!aL){a4=aZ;break}aF=w<<2;aH=0;while(1){c[a+16+(aH<<2)>>2]=0;I_(a+32+(aH*80|0)|0,aF);aG=aH+1|0;if((aG|0)<(aJ|0)){aH=aG}else{a4=aZ;break}}}else{c[a>>2]=0;c[a+872>>2]=20;a4=aI}}while(0);c[q>>2]=0;aI=a+784|0;L18531:do{if(aL){aZ=0;do{if(a4){a5=0;do{a6=s+(aZ*80|0)+(a5<<2)|0;m=0;while(1){if((m|0)>=7){break}o=c[a6>>2]>>1;l=m+1|0;p=o-(c[1914672+(l<<2)>>2]>>1)|0;a2=o-(c[1914672+(m<<2)>>2]>>1)|0;if((((p|0)>0?p:-p|0)|0)>(((a2|0)>0?a2:-a2|0)|0)){break}else{m=l}}c[a+464+(aZ*80|0)+(a5<<2)>>2]=m;a5=a5+1|0;}while((a5|0)<(w|0))}aZ=aZ+1|0;}while((aZ|0)<(aJ|0));aZ=a+432|0;c[aZ>>2]=0;do{if(a4){a5=0;a6=0;while(1){l=0;a7=a6;do{a7=(c[a+464+(a5*80|0)+(l<<2)>>2]|0)+a7|0;l=l+1|0;}while((l|0)<(w|0));l=a5+1|0;if((l|0)<(aJ|0)){a5=l;a6=a7}else{break}}a6=(ac(aJ,w)|0)<<16;if((a7|0)<=((a6>>>2|((a6|0)<0|0?-1:0)<<30)+16384>>15|0)){a8=a4;break}c[aZ>>2]=1;if(!aL){a9=a4;break L18531}a6=a+876|0;a5=0;l=aI;while(1){m=a+464+(a5*80|0)|0;a2=Md(0,m,0,w,0,q)|0;if((c[a6>>2]|0)<20){ba=Md(0,m,l,w,1,q)|0}else{ba=8388607}p=a+448+(a5<<2)|0;if((a2|0)>(ba|0)){c[p>>2]=1;bb=(c[a6>>2]|0)+1|0}else{c[p>>2]=0;bb=0}c[a6>>2]=bb;p=a5+1|0;if((p|0)<(aJ|0)){a5=p;l=m}else{a9=a4;break L18531}}}else{a8=0}}while(0);c[a+876>>2]=20;if(!aL){a9=a8;break}aZ=w<<2;l=0;while(1){c[a+448+(l<<2)>>2]=0;I_(a+464+(l*80|0)|0,aZ);a5=l+1|0;if((a5|0)<(aJ|0)){l=a5}else{a9=a8;break}}}else{c[a+432>>2]=0;c[a+876>>2]=20;a9=a4}}while(0);a4=a+868|0;do{if((c[a4>>2]|0)<10){if((c[a+8>>2]|0)!=(c[a+12>>2]|0)){aP=13297;break}if((c[a+440>>2]|0)!=(c[a+444>>2]|0)){aP=13297;break}if((c[a>>2]|0)!=(c[a+4>>2]|0)){aP=13297;break}if((c[a+432>>2]|0)!=(c[a+436>>2]|0)){aP=13297;break}c[b>>2]=0}else{aP=13297}}while(0);if((aP|0)==13297){c[b>>2]=1;c[a4>>2]=0}a4=a+880|0;do{if((c[a4>>2]|0)<10){aP=(c[a+8>>2]|0)==0?2:3;a8=(ac(aP,w)|0)/4|0;if((c[a+864>>2]|0)==(aJ|0)&(aJ|0)==1){bc=0;bd=0;be=1}else{break}do{bb=(c[a+32+(bc<<2)>>2]|0)-(c[a+352+(bc<<2)>>2]|0)|0;ba=(bb|0)>0?bb:-bb|0;bd=ba+bd|0;be=(ba|0)>(aP|0)|(bd|0)>(a8|0)?0:be;bc=bc+1|0;}while((bc|0)<(w|0)&(be|0)>0);if((be|0)==0){break}a8=(w<<1|0)/4|0;aP=0;ba=0;bb=1;do{q=(c[a+464+(aP<<2)>>2]|0)-(c[a+784+(aP<<2)>>2]|0)|0;aI=(q|0)>0?q:-q|0;ba=aI+ba|0;bb=(aI|0)>2|(ba|0)>(a8|0)?0:bb;aP=aP+1|0;}while((aP|0)<(w|0)&(bb|0)>0);if((bb|0)==0){break}c[b+28>>2]=0;c[a4>>2]=(c[a4>>2]|0)+1;i=k;return 0}}while(0);c[a4>>2]=0;if(!aL){i=k;return 0}aL=a|0;c[b+4>>2]=c[aL>>2];a4=a+8|0;be=(w|0)==20;bc=be&1;bd=be?4:3;c[b+8>>2]=(c[a4>>2]|0)==1?bd:bc;be=a+432|0;c[b+12>>2]=c[be>>2];aP=a+440|0;c[b+16>>2]=(c[aP>>2]|0)==1?bd:bc;c[b+20>>2]=0;c[b+24>>2]=0;bc=b+28|0;c[bc>>2]=aJ;bd=b+32|0;a8=t+4|0;t=aJ<<2;NN(bd|0,a8|0,t)|0;t=0;do{c[b+48+(t<<2)>>2]=c[a+16+(t<<2)>>2];if(a9){a8=0;do{c[b+64+(t*80|0)+(a8<<2)>>2]=c[a+32+(t*80|0)+(a8<<2)>>2];a8=a8+1|0;}while((a8|0)<(w|0))}t=t+1|0;bf=c[bc>>2]|0;}while((t|0)<(bf|0));L18598:do{if((bf|0)>0){if(a9){bg=0}else{t=0;while(1){c[b+464+(t<<2)>>2]=c[a+448+(t<<2)>>2];t=t+1|0;if((t|0)>=(c[bc>>2]|0)){break L18598}}}do{c[b+464+(bg<<2)>>2]=c[a+448+(bg<<2)>>2];t=0;do{c[b+480+(bg*80|0)+(t<<2)>>2]=c[a+464+(bg*80|0)+(t<<2)>>2];t=t+1|0;}while((t|0)<(w|0));bg=bg+1|0;}while((bg|0)<(c[bc>>2]|0))}}while(0);I_(b+896|0,320);NM(b+880|0,0,16);NM(b+1296|0,0,16);I_(b+1216|0,80);I_(b+1632|0,80);bc=0;do{c[b+384+(bc<<2)>>2]=c[a+352+(bc<<2)>>2];c[b+800+(bc<<2)>>2]=c[a+784+(bc<<2)>>2];bc=bc+1|0;}while((bc|0)<20);c[a+864>>2]=aJ;c[a+4>>2]=c[aL>>2];c[a+436>>2]=c[be>>2];c[a+12>>2]=c[a4>>2];c[a+444>>2]=c[aP>>2];if(!a9){i=k;return 0}a9=aJ-1|0;aJ=0;do{c[a+352+(aJ<<2)>>2]=c[a+32+(a9*80|0)+(aJ<<2)>>2];c[a+784+(aJ<<2)>>2]=c[a+464+(a9*80|0)+(aJ<<2)>>2];aJ=aJ+1|0;}while((aJ|0)<(w|0));i=k;return 0}function Mk(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;if((h|0)==10){i=5}else if((h|0)==20){i=11}else{i=h}if((g|0)<=0){return}j=(i|0)>0;k=0;do{if(j){l=0;while(1){m=HH(~((c[b+(k*80|0)+(l<<2)>>2]>>1)+(c[a+(k*80|0)+(l<<2)>>2]>>1)))|0;do{if((m|0)==0){n=0}else{o=m>>31^m;if((o|0)<=-1){n=-1;break}p=~o;o=0;while(1){q=p<<1;if((q|0)<0){p=q;o=o+1|0}else{n=o;break}}}}while(0);o=31-n|0;p=c[d+(k*80|0)+(l<<2)>>2]|0;q=m<<n;N2(q,(q|0)<0|0?-1:0,p,(p|0)<0|0?-1:0)|0;p=G<<1|0>>>31;q=2147483647>>>(o>>>0);if((p|0)>(q|0)){r=2147483647}else{r=(p|0)<(~q|0)?-2147483648:p<<o}c[f+(k*80|0)+(l<<2)>>2]=r;o=l+1|0;if((o|0)<(i|0)){l=o}else{s=i;break}}}else{s=0}if((s|0)<(h|0)){l=s;do{o=c[d+(k*80|0)+(l<<2)>>2]|0;p=(o|0)>0?o:-o|0;q=c[e+(k*80|0)+(l<<2)>>2]|0;t=(q|0)>0?q:-q|0;u=(p|0)>(t|0)?p:t;do{if((u|0)==0){v=0}else{t=u>>31^u;if((t|0)<=-1){v=-1;break}p=~t;t=0;while(1){w=p<<1;if((w|0)<0){p=w;t=t+1|0}else{v=t;break}}}}while(0);u=o<<v;m=u;t=(u|0)<0|0?-1:0;N2(m,t,m,t)|0;t=G;m=q<<v;u=m;p=(m|0)<0|0?-1:0;N2(u,p,u,p)|0;p=HH(33554432-(v<<25)+((HE((G+t|0)>>>1)|0)>>1)|0)|0;t=HH(~((c[b+(k*80|0)+(l<<2)>>2]>>1)+(c[a+(k*80|0)+(l<<2)>>2]>>1)))|0;do{if((t|0)==0){x=0}else{u=t>>31^t;if((u|0)<=-1){x=-1;break}m=~u;u=0;while(1){w=m<<1;if((w|0)<0){m=w;u=u+1|0}else{x=u;break}}}}while(0);q=t<<x;do{if((p|0)==0){y=0}else{o=p>>31^p;if((o|0)<=-1){y=-1;break}u=~o;o=0;while(1){m=u<<1;if((m|0)<0){u=m;o=o+1|0}else{y=o;break}}}}while(0);t=p<<y;N2(t,(t|0)<0|0?-1:0,q,(q|0)<0|0?-1:0)|0;t=G<<1|0>>>31;o=31-x-y|0;if((o|0)<0){z=t>>-o}else{z=(t|0)<(2147483647>>>(o>>>0)|0)?t<<o:2147483647}c[f+(k*80|0)+(l<<2)>>2]=z;l=l+1|0;}while((l|0)<(h|0))}k=k+1|0;}while((k|0)<(g|0));return}function Ml(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;I_(a,128);c[a+144>>2]=0;e=1927624;f=1;while(1){if(f>>>0>=5){break}g=c[1925480+(f<<2)>>2]|0;if((c[g+8>>2]|0)>(b|0)){e=g;f=f+1|0}else{break}}c[a+128>>2]=c[e>>2];c[a+132>>2]=c[e+4>>2];c[a+140>>2]=c[e+12>>2];c[a+152>>2]=c[e+16>>2];c[a+136>>2]=c[e+8>>2];c[a+148>>2]=d;c[a+156>>2]=d-1;return 1}function Mm(a,d,f,g,h,i,j){a=a|0;d=d|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0;c[i>>2]=0;k=a+148|0;l=c[k>>2]|0;if((f|0)<=0){m=l;n=(f|0)/(m|0)|0;c[i>>2]=n;return 0}o=a+128|0;p=a+144|0;q=a+140|0;r=a+132|0;s=h;h=0;t=l;while(1){if((t|0)>0){l=0;u=0;v=c[p>>2]|0;w=c[q>>2]|0;while(1){x=v^1;if((w|0)>0){y=b[d+((ac(u+h|0,g)|0)<<1)>>1]|0;z=c[o>>2]|0;A=y<<4;y=c[a+(v<<2)>>2]|0;B=c[a+(x<<2)>>2]|0;C=0;while(1){D=C+1|0;E=c[a+(D<<3)+(v<<2)>>2]|0;F=a+(D<<3)+(x<<2)|0;H=c[F>>2]|0;I=e[z>>1]<<16;N2(I,(I|0)<0|0?-1:0,y,(y|0)<0|0?-1:0)|0;I=(G<<1|0>>>31)+A|0;J=e[z+2>>1]<<16;N2(J,(J|0)<0|0?-1:0,B,(B|0)<0|0?-1:0)|0;J=I+(G<<1|0>>>31)|0;I=e[z+4>>1]<<16;N2(I,(I|0)<0|0?-1:0,E,(E|0)<0|0?-1:0)|0;I=J-(G<<1|0>>>31)|0;J=e[z+6>>1]<<16;N2(J,(J|0)<0|0?-1:0,H,(H|0)<0|0?-1:0)|0;K=I-(G<<1|0>>>31)|0;c[F>>2]=K<<1;c[a+(C<<3)+(x<<2)>>2]=A<<1;L=c[q>>2]|0;if((D|0)<(L|0)){z=z+8|0;A=K;y=E;B=H;C=D}else{break}}M=K;N=L;O=c[p>>2]|0}else{M=l;N=w;O=v}C=O^1;c[p>>2]=C;B=u+1|0;if((B|0)<(t|0)){l=M;u=B;v=C;w=N}else{P=M;break}}}else{P=0}w=c[r>>2]|0;N2(w,(w|0)<0|0?-1:0,P,(P|0)<0|0?-1:0)|0;w=(G<<1|0>>>31)+8>>4;if((w|0)>32767){Q=32767}else{Q=(w|0)<-32768?-32768:w&65535}b[s>>1]=Q;w=c[k>>2]|0;v=w+h|0;if((v|0)<(f|0)){s=s+(j<<1)|0;h=v;t=w}else{m=w;break}}n=(f|0)/(m|0)|0;c[i>>2]=n;return 0}function Mn(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;d=1;while(1){e=(d*3|0)+1|0;if((e|0)>(b|0)){f=e;break}else{d=e}}while(1){d=(f|0)/3|0;e=d+1|0;if((e|0)<=(b|0)){g=d;h=e;while(1){e=c[a+(g<<2)>>2]|0;i=h;while(1){j=i-d|0;k=c[a+(j-1<<2)>>2]|0;if((k|0)<=(e|0)){l=i;break}c[a+(i-1<<2)>>2]=k;if((j|0)>(d|0)){i=j}else{l=j;break}}c[a+(l-1<<2)>>2]=e;i=h+1|0;if((i|0)>(b|0)){break}else{g=h;h=i}}}if((f|0)>5){f=d}else{break}}return}function Mo(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0;d=1;while(1){e=(d*3|0)+1|0;if((e|0)>(b|0)){f=e;break}else{d=e}}while(1){d=(f|0)/3|0;e=d+1|0;if((e|0)<=(b|0)){g=d;h=e;while(1){e=c[a+(g<<2)>>2]|0;i=h;while(1){j=i-d|0;k=c[a+(j-1<<2)>>2]|0;if((k|0)<=(e|0)){l=i;break}c[a+(i-1<<2)>>2]=k;if((j|0)>(d|0)){i=j}else{l=j;break}}c[a+(l-1<<2)>>2]=e;i=h+1|0;if((i|0)>(b|0)){break}else{g=h;h=i}}}if((f|0)>5){f=d}else{break}}return}function Mp(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0;if((e|0)<=0){return}f=e;e=c[b>>2]|0;do{f=f-1|0;g=c[d+(f<<2)>>2]|0;if((e|0)>0){h=e;while(1){i=h-1|0;c[a+(h<<2)>>2]=c[a+(i<<2)>>2];if((i|0)>0){h=i}else{break}}}c[a>>2]=g;e=(c[b>>2]|0)+1|0;c[b>>2]=e;}while((f|0)>0);return}function Mq(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0;e=c[b>>2]|0;if((e|0)>0){f=e;while(1){e=f-1|0;c[a+(f<<2)>>2]=c[a+(e<<2)>>2];if((e|0)>0){f=e}else{break}}}c[a>>2]=d;c[b>>2]=(c[b>>2]|0)+1;return}function Mr(a,b,d){a=a|0;b=b|0;d=d|0;c[a+(c[b>>2]<<2)>>2]=d;c[b>>2]=(c[b>>2]|0)+1;return}function Ms(a,b,c){a=a|0;b=b|0;c=c|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0;if((a|0)==0){d=0;return d|0}e=a>>31^a;if((e|0)>-1){f=~e;e=0;while(1){g=f<<1;if((g|0)<0){f=g;e=e+1|0}else{h=e;break}}}else{h=-1}do{if((b|0)==0){i=0}else{e=b>>31^b;if((e|0)<=-1){i=-1;break}f=~e;e=0;while(1){g=f<<1;if((g|0)<0){f=g;e=e+1|0}else{i=e;break}}}}while(0);do{if((c|0)==0){j=0}else{e=c>>31^c;if((e|0)<=-1){j=-1;break}f=~e;e=0;while(1){g=f<<1;if((g|0)<0){f=g;e=e+1|0}else{j=e;break}}}}while(0);e=a<<h;a=c<<j;N2(a,(a|0)<0|0?-1:0,e,(e|0)<0|0?-1:0)|0;e=G;a=h-1|0;h=a+j|0;if((e>>((h|0)<31?h:31)|0)>=(b|0)){d=2147483647;return d|0}h=HO(e,b<<i,15)|0;b=a-i+j|0;j=(b|0)<31?b:31;if((j|0)<0){d=h<<-j;return d|0}else{d=h>>j;return d|0}return 0}function Mt(a,b,d,e,f,g,h,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0;k=i;i=i+8|0;l=k|0;c[l>>2]=e;e=Nk(a|0,b,d,f,g,l,h,j)|0;if((e|0)!=0){m=e;i=k;return m|0}c[a+148>>2]=0;c[a+152>>2]=773094113;c[a+156>>2]=c[l>>2];m=0;i=k;return m|0}function Mu(a,d,e,f,g,h,i,j,k,l,m,n,o,p,q){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;var r=0,s=0,t=0,u=0,v=0,w=0,x=0;r=b[a+2>>1]|0;s=r&65535;t=(c[a+156>>2]|0)!=0;do{if((s&32|0)==0){if(!t){return}if((s&16|0)==0|(e|0)==0){break}if((f|0)<=0){return}NM(d+360|0,0,f<<2|0);return}else{if(!((e|0)==2|t^1)){break}I_(d+360|0,240);u=0;do{c[q+(u<<2)>>2]=-2147483648;u=u+1|0;}while((u|0)<60);return}}while(0);t=a|0;if((r&32)==0){v=(m|0)>3|0}else{v=1}do{if((s&4|0)==0){w=13451}else{if((c[a+12>>2]|0)>(n|0)|(v|0)==0){w=13451;break}if((s&8|0)!=0){if(!((c[a+16>>2]|0)>(n|0)|(o|0)==0)){w=13451;break}}I_(d,f<<1)}}while(0);if((w|0)==13451){Nj(j,k,f,i,d|0,t,l)}l=b[a>>1]|0;t=(f|0)>0;if(t){i=0;do{L18805:do{if((i|0)<(l|0)){c[d+360+(i<<2)>>2]=0}else{do{if((b[d+(i<<1)>>1]|0)>16384){if(((c[h+(i<<2)>>2]|0)+19628084|0)>=(c[p+(i<<2)>>2]|0)){break}c[d+360+(i<<2)>>2]=1;break L18805}}while(0);c[d+360+(i<<2)>>2]=0}}while(0);i=i+1|0;}while((i|0)<(f|0))}do{if((b[d>>1]|0)>16384){if((c[d+364>>2]|0)==0){break}c[d+360>>2]=1}}while(0);i=g-1|0;h=(i|0)>1;if(h){l=a+144|0;k=1;do{do{if((b[d+(k<<1)>>1]|0)>(b[l>>1]|0)){if((c[d+360+(k-1<<2)>>2]|0)==0){break}if((c[d+360+(k+1<<2)>>2]|0)==0){break}c[d+360+(k<<2)>>2]=1}}while(0);k=k+1|0;}while((k|0)<(i|0))}do{if((g|0)>0){k=d+360+(g-2<<2)|0;do{if((b[d+(i<<1)>>1]|0)>(b[a+144>>1]|0)){if((c[k>>2]|0)==0){break}c[d+360+(i<<2)>>2]=1}}while(0);if((c[k>>2]|0)!=0){break}c[d+360+(i<<2)>>2]=0}}while(0);if((c[d+364>>2]|0)==0){c[d+360>>2]=0}if(h){h=1;while(1){a=h+1|0;do{if((c[d+360+(h-1<<2)>>2]|0)==0){if((c[d+360+(a<<2)>>2]|0)!=0){break}c[d+360+(h<<2)>>2]=0}}while(0);if((a|0)<(i|0)){h=a}else{break}}}if(t){x=0}else{return}do{if((c[d+360+(x<<2)>>2]|0)!=0){c[q+(x<<2)>>2]=60-(16777216-(c[p+(x<<2)>>2]|0)>>24)}x=x+1|0;}while((x|0)<(f|0));return}function Mv(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;h=(a|0)>0;if((c[b+156>>2]|0)==0){if(h){i=0}else{return}do{c[f+(i<<2)>>2]=-2147483648;i=i+1|0;}while((i|0)<(a|0));return}if(h){j=0;k=1;l=0}else{return}while(1){h=f+(l<<2)|0;do{if((c[d+(l<<2)>>2]|0)==0){c[h>>2]=-2147483648;m=k;n=j}else{if((c[h>>2]|0)!=-2147483648){c[g+(l<<2)>>2]=(c[e+(l<<2)>>2]|0)+33554432}i=c[h>>2]|0;if((k|0)!=0){m=0;n=i;break}b=i-j|0;if((b|0)>60){o=i+60-b|0;c[h>>2]=o;m=0;n=o;break}if((b|0)>=-60){m=0;n=i;break}o=i-60-b|0;c[h>>2]=o;m=0;n=o}}while(0);h=l+1|0;if((h|0)<(a|0)){j=n;k=m;l=h}else{break}}return}function Mw(a,b,d,e,f,g,h,i,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0;if((c[h+156>>2]|0)!=0&(a|0)>0){k=0}else{return}do{h=(c[f+(k<<2)>>2]>>1)+(c[e+(k<<2)>>2]>>1)|0;do{if((h|0)<-1073741824){l=0}else{m=(c[g+(k<<2)>>2]|0)-((c[d+(k<<2)>>2]>>1)+(c[b+(k<<2)>>2]>>1)>>1)|0;n=33554432-h+(HE((m|0)>0?m:-m|0)|0)|0;if((n|0)>-1){l=2147483647;break}o=HH(n)|0;l=(m|0)<0?-o|0:o}}while(0);c[i+120+(k<<2)>>2]=l;c[j+120+(k<<2)>>2]=l;k=k+1|0;}while((k|0)<(a|0));return}function Mx(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0;if(!((c[b+156>>2]|0)!=0&(a|0)>0)){return}h=b+152|0;b=0;do{i=f+(b<<2)|0;j=d+360+(b<<2)|0;L18897:do{if((c[i>>2]|0)!=0){k=e+360+(b<<2)|0;do{if((c[j>>2]|0)!=0){if((c[k>>2]|0)==0){break}if((c[d+120+(b<<2)>>2]|0)>(c[h>>2]|0)){break L18897}c[i>>2]=0;c[g>>2]=1;break L18897}}while(0);c[j>>2]=0;c[k>>2]=0}}while(0);do{if((c[j>>2]|0)!=0){if((c[e+360+(b<<2)>>2]|0)==0){break}if((c[d+120+(b<<2)>>2]|0)<=(c[h>>2]|0)){break}c[i>>2]=1;c[g>>2]=1}}while(0);b=b+1|0;}while((b|0)<(a|0));return}function My(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0;f=c[e+(d<<2)>>2]|0;g=(((ac(a<<2,f)|0)/(b|0)|0)+1|0)/2|0;if((g|0)<(f|0)){h=0}else{i=d;return i|0}while(1){j=h+1|0;k=e+(j<<2)|0;if((h|0)>=(d|0)){break}if((c[k>>2]|0)>(g|0)){break}else{h=j}}i=(g-(c[e+(h<<2)>>2]|0)|0)>((c[k>>2]|0)-g|0)?j:h;return i|0}function Mz(b,d,e,f,g,h,i,j,k,l){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0;if((e|0)<1){m=1;return m|0}l=i+44|0;c[l>>2]=(k|0)!=0;k=(f|0)==2;f=k?5:12;n=i+48|0;o=(b|0)<16e3?f-2|0:f;c[n>>2]=o;c[i+52>>2]=k?3:4;if((g|0)==512){p=5;q=1920992}else if((g|0)==480){p=5;q=1921032}else if((g|0)==960|(g|0)==1024){p=12;q=1920768}else{c[i+200>>2]=-1;m=1;return m|0}f=k&1;r=-1;s=0;while(1){if((s|0)>=(p|0)){t=r;break}u=a[q+(s<<3)+4+f|0]|0;if((c[q+(s<<3)>>2]|0)>(d|0)){r=u;s=s+1|0}else{t=u;break}}s=i+200|0;c[s>>2]=t;if((t|0)<0){m=1;return m|0}r=c[j+4>>2]|0;q=(t|0)<(r|0)?t:r;c[s>>2]=q;s=c[j+12+(q<<2)>>2]|0;c[i+204>>2]=s;if((g|0)==960|(g|0)==1024){if(k){v=0}else{v=(d|0)<18783?4:8}c[i+188>>2]=v;r=c[j+12+(v<<2)>>2]|0;c[i+196>>2]=r;v=((s-r|0)/4|0)+r|0;r=q;while(1){w=j+12+(r<<2)|0;if((c[w>>2]|0)>(v|0)){r=r-1|0}else{break}}c[i+184>>2]=r;c[i+192>>2]=c[w>>2];c[i+8>>2]=1437;c[i+12>>2]=1500;c[i+16>>2]=o;c[i+20>>2]=o-7;c[i+24>>2]=0;c[i+28>>2]=0;c[i+32>>2]=-1;c[i+36>>2]=-1;c[i>>2]=1;c[i+4>>2]=1;c[i+40>>2]=1;w=i+56|0;if(k){IY(w,1926344,32);IY(i+120|0,1926344,32);m=0;return m|0}else{IY(w,1926592,64);IY(i+120|0,1926592,64);m=0;return m|0}}else if((g|0)==480|(g|0)==512){w=(h|0)!=0|0;h=(e|0)!=1|0;do{if((c[1928e3+(w<<2)>>2]|0)>(b|0)){x=0}else{if((c[1928008+(w<<2)>>2]|0)<(b|0)){x=0;break}x=1928016+(h*60|0)|0}}while(0);do{if((c[1928136+(w<<2)>>2]|0)>(b|0)){y=x}else{if((c[1928144+(w<<2)>>2]|0)<(b|0)){y=x;break}y=1928152+(h*60|0)|0}}while(0);do{if((c[1928272+(w<<2)>>2]|0)>(b|0)){z=13570}else{if((c[1928280+(w<<2)>>2]|0)<(b|0)){z=13570;break}A=1928288+(h*60|0)|0}}while(0);do{if((z|0)==13570){if((y|0)!=0){A=y;break}c[l>>2]=0;m=0;return m|0}}while(0);y=j|0;z=c[y>>2]|0;h=c[j+12+(z<<2)>>2]|0;b=(((ac(c[A+16>>2]<<2,h)|0)/(d|0)|0)+1|0)/2|0;if((b|0)<(h|0)){h=0;while(1){B=h+1|0;C=c[j+12+(B<<2)>>2]|0;if((h|0)>=(z|0)|(C|0)>(b|0)){break}else{h=B}}D=(b-(c[j+12+(h<<2)>>2]|0)|0)>(C-b|0)?B:h}else{D=z}c[i+184>>2]=D;c[i+192>>2]=c[j+12+(D<<2)>>2];D=c[y>>2]|0;y=c[j+12+(D<<2)>>2]|0;z=(((ac(c[A+20>>2]<<2,y)|0)/(d|0)|0)+1|0)/2|0;if((z|0)<(y|0)){y=0;while(1){E=y+1|0;F=c[j+12+(E<<2)>>2]|0;if((y|0)>=(D|0)|(F|0)>(z|0)){break}else{y=E}}G=(z-(c[j+12+(y<<2)>>2]|0)|0)>(F-z|0)?E:y}else{G=D}c[i+188>>2]=G;c[i+196>>2]=c[j+12+(G<<2)>>2];c[i+8>>2]=c[A+8>>2];c[i+12>>2]=c[A+12>>2];c[i+16>>2]=c[A+24>>2];c[i+20>>2]=c[A+28>>2];c[i+24>>2]=c[A+32>>2];c[i+28>>2]=c[A+36>>2];c[i+32>>2]=c[A+40>>2];c[i+36>>2]=c[A+44>>2];c[i>>2]=c[A>>2];c[i+4>>2]=c[A+4>>2];c[i+40>>2]=c[A+56>>2];MA(i+56|0,o+1|0,d,g,c[A+48>>2]|0);MA(i+120|0,(c[n>>2]|0)+1|0,d,g,c[A+52>>2]|0);m=0;return m|0}else{c[l>>2]=0;m=0;return m|0}return 0}function MA(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;g=i;i=i+16|0;h=g|0;j=g+8|0;k=HQ(d,~~(+(e|0)*1.0e3),h)|0;N2(k,(k|0)<0|0?-1:0,1686633600,0)|0;k=HP(f,G<<1|0>>>31,j)|0;f=k;e=(k|0)<0|0?-1:0;if((b|0)<=0){i=g;return}N2(f,e,f,e)|0;e=-G|0;f=e;k=(e|0)<0|0?-1:0;e=((c[j>>2]|0)+(c[h>>2]|0)<<1)+14|0;j=0;do{d=j<<27|67108864;l=d;m=(d|0)<0|0?-1:0;N2(l<<1|0>>>31,m<<1|l>>>31,l,m)|0;m=G;N2(m,(m|0)<0|0?-1:0,f,k)|0;m=HT(1459376200,2,G<<1|0>>>31,e,h)|0;l=a+(j<<2)|0;c[l>>2]=m;d=c[h>>2]|0;L18985:do{if((d|0)>0){do{if((m|0)!=0){n=m>>31^m;if((n|0)>-1){o=~n;n=0;while(1){p=o<<1;if((p|0)<0){o=p;n=n+1|0}else{q=n;break}}}else{q=-1}if((q|0)>=(d|0)){break}r=(m|0)>0?2147483647:-2147483648;break L18985}}while(0);r=m<<d}else{if((d|0)<-31){r=0;break}r=m>>-d}}while(0);c[l>>2]=r;j=j+1|0;}while((j|0)<(b|0));i=g;return}function MB(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0;j=i;i=i+4288|0;k=j|0;l=j+8|0;m=j+16|0;n=j+24|0;o=j+32|0;p=j+4128|0;q=j+4184|0;r=j+4240|0;s=(h|0)==2;if(s){t=a+4+(g<<3)|0}else{t=a+4|0}h=a+552|0;c[h>>2]=0;a=t|0;c[a>>2]=0;u=t+4|0;c[u>>2]=1e3;t=d+(g<<2)|0;c[t>>2]=0;v=b+52|0;c[d+32+(g<<2)>>2]=c[v>>2];w=b+48|0;if((c[w>>2]|0)>0){x=0;do{c[d+320+(g*96|0)+48+(x<<2)>>2]=0;c[d+320+(g*96|0)+(x<<2)>>2]=0;x=x+1|0;}while((x|0)<(c[w>>2]|0))}x=d+64+(g<<3)+4|0;c[x>>2]=0;y=d+64+(g<<3)|0;c[y>>2]=0;z=d+128+(g<<3)+4|0;c[z>>2]=0;A=d+128+(g<<3)|0;c[A>>2]=0;if((c[b+44>>2]|0)==0){i=j;return 0}B=c[w>>2]|0;if((B|0)<=0){i=j;return 0}w=b+192|0;C=c[b+204>>2]|0;D=p|0;E=q|0;F=(B<<2)+4|0;I_(p,F);I_(q,F);do{if((c[b+36>>2]|0)==-1){H=13614}else{if((c[b+32>>2]|0)==-1){H=13614;break}F=c[w>>2]|0;I=(C-F|0)/3|0;J=I+F|0;K=c[b+196>>2]|0;L=F;M=J;N=J+I|0}}while(0);if((H|0)==13614){w=c[b+196>>2]|0;I=C-w|0;K=w;L=((I|0)/4|0)+w|0;M=((I|0)/2|0)+w|0;N=((I*3|0|0)/4|0)+w|0}w=(K|0)<(L|0);if(w){I=K;J=0;do{F=c[f+(I<<2)>>2]|0;O=(F|0)>0?F:-F|0;J=(J|0)>(O|0)?J:O;I=I+1|0;}while((I|0)<(L|0));do{if((J|0)==0){P=0}else{I=J>>31^J;if((I|0)<=-1){P=-1;break}O=~I;I=0;while(1){F=O<<1;if((F|0)<0){O=F;I=I+1|0}else{P=I;break}}}}while(0);J=K;while(1){c[o+(J<<2)>>2]=c[f+(J<<2)>>2]<<P;I=J+1|0;if((I|0)<(L|0)){J=I}else{Q=P;break}}}else{Q=0}P=(L|0)<(M|0);if(P){J=L;I=0;do{O=c[f+(J<<2)>>2]|0;F=(O|0)>0?O:-O|0;I=(I|0)>(F|0)?I:F;J=J+1|0;}while((J|0)<(M|0));do{if((I|0)==0){R=0}else{J=I>>31^I;if((J|0)<=-1){R=-1;break}F=~J;J=0;while(1){O=F<<1;if((O|0)<0){F=O;J=J+1|0}else{R=J;break}}}}while(0);I=L;while(1){c[o+(I<<2)>>2]=c[f+(I<<2)>>2]<<R;J=I+1|0;if((J|0)<(M|0)){I=J}else{S=R;break}}}else{S=0}R=(M|0)<(N|0);if(R){I=M;J=0;do{F=c[f+(I<<2)>>2]|0;O=(F|0)>0?F:-F|0;J=(J|0)>(O|0)?J:O;I=I+1|0;}while((I|0)<(N|0));do{if((J|0)==0){T=0}else{I=J>>31^J;if((I|0)<=-1){T=-1;break}O=~I;I=0;while(1){F=O<<1;if((F|0)<0){O=F;I=I+1|0}else{T=I;break}}}}while(0);J=M;while(1){c[o+(J<<2)>>2]=c[f+(J<<2)>>2]<<T;I=J+1|0;if((I|0)<(N|0)){J=I}else{U=T;break}}}else{U=0}T=(N|0)<(C|0);if(T){J=N;I=0;do{O=c[f+(J<<2)>>2]|0;F=(O|0)>0?O:-O|0;I=(I|0)>(F|0)?I:F;J=J+1|0;}while((J|0)<(C|0));do{if((I|0)==0){V=0}else{J=I>>31^I;if((J|0)<=-1){V=-1;break}F=~J;J=0;while(1){O=F<<1;if((O|0)<0){F=O;J=J+1|0}else{V=J;break}}}}while(0);I=N;while(1){c[o+(I<<2)>>2]=c[f+(I<<2)>>2]<<V;J=I+1|0;if((J|0)<(C|0)){I=J}else{W=V;break}}}else{W=0}V=L-K|0;I=1;while(1){if((1<<I|0)<(V|0)){I=I+1|0}else{break}}V=M-L|0;f=1;while(1){if((1<<f|0)<(V|0)){f=f+1|0}else{break}}V=N-M|0;J=1;while(1){if((1<<J|0)<(V|0)){J=J+1|0}else{break}}V=C-N|0;F=1;while(1){if((1<<F|0)<(V|0)){F=F+1|0}else{break}}if(w){V=K;O=0;while(1){X=c[o+(V<<2)>>2]|0;Y=X;Z=(X|0)<0|0?-1:0;N2(Y,Z,Y,Z)|0;Z=((G<<1|0>>>31)>>I)+O|0;Y=V+1|0;if((Y|0)<(L|0)){V=Y;O=Z}else{_=Z;break}}}else{_=0}if(P){P=L;O=0;while(1){V=c[o+(P<<2)>>2]|0;Z=V;Y=(V|0)<0|0?-1:0;N2(Z,Y,Z,Y)|0;Y=((G<<1|0>>>31)>>f)+O|0;Z=P+1|0;if((Z|0)<(M|0)){P=Z;O=Y}else{$=Y;break}}}else{$=0}if(R){R=M;O=0;while(1){P=c[o+(R<<2)>>2]|0;Y=P;Z=(P|0)<0|0?-1:0;N2(Y,Z,Y,Z)|0;Z=((G<<1|0>>>31)>>J)+O|0;Y=R+1|0;if((Y|0)<(N|0)){R=Y;O=Z}else{aa=Z;break}}}else{aa=0}if(T){T=N;O=0;while(1){R=c[o+(T<<2)>>2]|0;Z=R;Y=(R|0)<0|0?-1:0;N2(Z,Y,Z,Y)|0;Y=((G<<1|0>>>31)>>F)+O|0;Z=T+1|0;if((Z|0)<(C|0)){T=Z;O=Y}else{ab=Y;break}}}else{ab=0}do{if((_|0)!=0){O=I+(Q*-2|0)|0;if((O|0)>-1){ad=(O|0)>31?0:8>>>(O>>>0);ae=_}else{T=-O|0;ad=8;ae=_>>((T|0)>31?31:T)}if((ae|0)>(ad|0)){c[k>>2]=0;T=HM(_,k)|0;Y=T;Z=(T|0)<0|0?-1:0;N2(Y,Z,Y,Z)|0;Z=G<<1|0>>>31;af=(Z|0)<0|0?-1:0;ag=Z;ah=(c[k>>2]<<1)-1|0}else{af=0;ag=2147483647;ah=O+27|0}O=_;Z=(_|0)<0|0?-1:0;N2(ag,af,O,Z)|0;Z=G<<1|0>>>31;O=(ah|0)>0;if(O){ai=Z<<ah}else{ai=Z>>-ah}c[D>>2]=ai;if((B|0)<1){break}Z=B+3|0;Y=-ah|0;if(w){T=1;do{R=T*3|0;if((R|0)<=(Z|0)){P=L-T|0;if((P|0)>(K|0)){V=K;X=0;do{aj=c[o+(V<<2)>>2]|0;ak=c[o+(V+T<<2)>>2]|0;N2(ak,(ak|0)<0|0?-1:0,aj,(aj|0)<0|0?-1:0)|0;X=((G<<1|0>>>31)>>I)+X|0;V=V+1|0;}while((V|0)<(P|0));al=(X|0)<0|0?-1:0;am=X}else{al=0;am=0}N2(am,al,ag,af)|0;P=G<<1|0>>>31;V=O?P<<ah:P>>Y;P=c[b+120+(R<<2)>>2]|0;aj=V;ak=(V|0)<0|0?-1:0;V=P;an=(P|0)<0|0?-1:0;N2(aj,ak,V,an)|0;c[p+(T<<2)>>2]=G<<1|0>>>31}T=T+1|0;}while((T|0)<=(B|0))}else{T=1;do{an=T*3|0;if((an|0)<=(Z|0)){V=L-T|0;if((V|0)>(K|0)){ak=K;aj=0;do{P=c[o+(ak<<2)>>2]|0;ao=c[o+(ak+T<<2)>>2]|0;N2(ao,(ao|0)<0|0?-1:0,P,(P|0)<0|0?-1:0)|0;aj=((G<<1|0>>>31)>>I)+aj|0;ak=ak+1|0;}while((ak|0)<(V|0));ap=(aj|0)<0|0?-1:0;aq=aj}else{ap=0;aq=0}N2(aq,ap,ag,af)|0;V=G<<1|0>>>31;ak=O?V<<ah:V>>Y;V=c[b+120+(an<<2)>>2]|0;R=ak;X=(ak|0)<0|0?-1:0;ak=V;P=(V|0)<0|0?-1:0;N2(R,X,ak,P)|0;c[p+(T<<2)>>2]=G<<1|0>>>31}T=T+1|0;}while((T|0)<=(B|0))}}}while(0);p=(aa|0)==0;ah=(ab|0)==0;do{if((aa|$|ab|0)!=0){if(($|0)==0){ar=0;as=0}else{af=f+(S*-2|0)|0;if((af|0)>-1){at=(af|0)>31?0:8>>>(af>>>0);au=$}else{ag=-af|0;at=8;au=$>>((ag|0)>31?31:ag)}if((au|0)>(at|0)){c[l>>2]=0;ag=HM($,l)|0;ap=ag;aq=(ag|0)<0|0?-1:0;N2(ap,aq,ap,aq)|0;av=G<<1|0>>>31;aw=c[l>>2]<<1}else{av=2147483647;aw=af+28|0}ar=av;as=aw-2|0}if(p){ax=0;ay=0}else{af=J+(U*-2|0)|0;if((af|0)>-1){az=(af|0)>31?0:8>>>(af>>>0);aA=aa}else{aq=-af|0;az=8;aA=aa>>((aq|0)>31?31:aq)}if((aA|0)>(az|0)){c[m>>2]=0;aq=HM(aa,m)|0;ap=aq;ag=(aq|0)<0|0?-1:0;N2(ap,ag,ap,ag)|0;aB=G<<1|0>>>31;aC=c[m>>2]<<1}else{aB=2147483647;aC=af+28|0}ax=aB;ay=aC-2|0}if(ah){aD=0;aE=0}else{af=F+(W*-2|0)|0;if((af|0)>-1){aF=(af|0)>31?0:8>>>(af>>>0);aG=ab}else{ag=-af|0;aF=8;aG=ab>>((ag|0)>31?31:ag)}if((aG|0)>(aF|0)){c[n>>2]=0;ag=HM(ab,n)|0;ap=ag;aq=(ag|0)<0|0?-1:0;N2(ap,aq,ap,aq)|0;aH=G<<1|0>>>31;aI=c[n>>2]<<1}else{aH=2147483647;aI=af+28|0}aD=aH;aE=aI-2|0}af=$;aq=($|0)<0|0?-1:0;ap=ar;ag=(ar|0)<0|0?-1:0;N2(ap,ag,af,aq)|0;aq=G<<1|0>>>31;af=(as|0)>0;if(af){aJ=aq<<as}else{aJ=aq>>-as}aq=aa;I=(aa|0)<0|0?-1:0;K=ax;al=(ax|0)<0|0?-1:0;N2(K,al,aq,I)|0;I=G<<1|0>>>31;aq=(ay|0)>0;if(aq){aK=I<<ay}else{aK=I>>-ay}I=aK+aJ|0;am=ab;w=(ab|0)<0|0?-1:0;ai=aD;_=(aD|0)<0|0?-1:0;N2(ai,_,am,w)|0;w=G<<1|0>>>31;am=(aE|0)>0;if(am){aL=w<<aE}else{aL=w>>-aE}c[E>>2]=I+aL;if((B|0)<1){break}I=-aE|0;w=-ay|0;k=-as|0;ad=1;do{ae=M-ad|0;if((ae|0)>(L|0)){Q=L;T=0;do{Y=c[o+(Q<<2)>>2]|0;O=c[o+(Q+ad<<2)>>2]|0;N2(O,(O|0)<0|0?-1:0,Y,(Y|0)<0|0?-1:0)|0;T=((G<<1|0>>>31)>>f)+T|0;Q=Q+1|0;}while((Q|0)<(ae|0));aM=(T|0)<0|0?-1:0;aN=T}else{aM=0;aN=0}N2(aN,aM,ap,ag)|0;ae=G<<1|0>>>31;Q=af?ae<<as:ae>>k;ae=N-ad|0;if((ae|0)>(M|0)){an=M;aj=0;do{Y=c[o+(an<<2)>>2]|0;O=c[o+(an+ad<<2)>>2]|0;N2(O,(O|0)<0|0?-1:0,Y,(Y|0)<0|0?-1:0)|0;aj=((G<<1|0>>>31)>>J)+aj|0;an=an+1|0;}while((an|0)<(ae|0));aO=(aj|0)<0|0?-1:0;aP=aj}else{aO=0;aP=0}N2(aP,aO,K,al)|0;ae=G<<1|0>>>31;an=(aq?ae<<ay:ae>>w)+Q|0;ae=C-ad|0;if((ae|0)>(N|0)){T=N;Y=0;do{O=c[o+(T<<2)>>2]|0;Z=c[o+(T+ad<<2)>>2]|0;N2(Z,(Z|0)<0|0?-1:0,O,(O|0)<0|0?-1:0)|0;Y=((G<<1|0>>>31)>>F)+Y|0;T=T+1|0;}while((T|0)<(ae|0));aQ=(Y|0)<0|0?-1:0;aR=Y}else{aQ=0;aR=0}N2(aR,aQ,ai,_)|0;ae=G<<1|0>>>31;T=an+(am?ae<<aE:ae>>I)|0;ae=c[b+56+(ad<<2)>>2]|0;N2(T,(T|0)<0|0?-1:0,ae,(ae|0)<0|0?-1:0)|0;c[q+(ad<<2)>>2]=G<<1|0>>>31;ad=ad+1|0;}while((ad|0)<=(B|0))}}while(0);B=r|0;r=b+16|0;c[u>>2]=MC(E,B,c[r>>2]|0)|0;MD(B,d+320+(g*96|0)|0,c[r>>2]|0,c[v>>2]|0);E=c[r>>2]|0;while(1){aS=E-1|0;if((E|0)<=0){H=13737;break}if((c[d+320+(g*96|0)+(aS<<2)>>2]|0)==0){E=aS}else{H=13738;break}}if((H|0)==13737){c[A>>2]=E;aT=0}else if((H|0)==13738){c[A>>2]=E;E=0;q=aS;while(1){aS=c[d+320+(g*96|0)+(q<<2)>>2]|0;aE=(ac(aS,aS)|0)+E|0;if((q|0)>0){E=aE;q=q-1|0}else{aT=aE;break}}}q=d+192+(g<<3)|0;c[q>>2]=c[b+24>>2];E=b+184|0;c[y>>2]=e-(c[E>>2]|0);do{if((c[u>>2]|0)<=(c[b+8>>2]|0)){if((aT|0)>(((c[r>>2]|0)/2|0)+2|0)){break}i=j;return 0}}while(0);c[a>>2]=1;c[t>>2]=(c[t>>2]|0)+1;if(s){i=j;return 0}if((c[b+4>>2]|0)==0){i=j;return 0}if((c[b+40>>2]|0)==0){i=j;return 0}s=b+20|0;a=MC(D,B,c[s>>2]|0)|0;MD(B,d+320+(g*96|0)+48|0,c[s>>2]|0,c[v>>2]|0);v=c[s>>2]|0;while(1){aU=v-1|0;if((v|0)<=0){H=13748;break}if((c[d+320+(g*96|0)+48+(aU<<2)>>2]|0)==0){v=aU}else{H=13749;break}}if((H|0)==13748){c[z>>2]=v;aV=0}else if((H|0)==13749){c[z>>2]=v;v=0;z=aU;while(1){aU=c[d+320+(g*96|0)+48+(z<<2)>>2]|0;B=(ac(aU,aU)|0)+v|0;if((z|0)>0){v=B;z=z-1|0}else{aV=B;break}}}z=d+192+(g<<3)+4|0;c[z>>2]=c[b+28>>2];v=b+188|0;c[x>>2]=(c[E>>2]|0)-(c[v>>2]|0);if((a|0)>(c[b+12>>2]|0)){b=c[s>>2]|0;if((a|0)<(b*16e3|0|0)){aW=b}else{H=13753}}else{H=13753}do{if((H|0)==13753){if((aV|0)<=9){i=j;return 0}b=c[s>>2]|0;if((aV|0)<(b*22|0|0)){aW=b;break}i=j;return 0}}while(0);if((aW|0)>0){aW=0;aV=0;do{aW=(I$((c[d+320+(g*96|0)+(aV<<2)>>2]|0)-(c[d+320+(g*96|0)+48+(aV<<2)>>2]|0)|0)|0)+aW|0;aV=aV+1|0;}while((aV|0)<(c[s>>2]|0));if((aW|0)<2){aX=aV;H=13758}}else{aX=0;H=13758}do{if((H|0)==13758){if((c[z>>2]|0)!=(c[q>>2]|0)){break}c[h>>2]=1;c[y>>2]=e-(c[v>>2]|0);aV=aX;while(1){if((aV|0)>=(c[A>>2]|0)){aY=aV;break}if((I$(c[d+320+(g*96|0)+(aV<<2)>>2]|0)|0)>1){aY=aV;break}else{aV=aV+1|0}}while(1){aZ=aY-1|0;if((aY|0)<=0){break}if((c[d+320+(g*96|0)+(aZ<<2)>>2]|0)==0){aY=aZ}else{break}}if((aZ|0)>=(c[A>>2]|0)){i=j;return 0}c[A>>2]=aY;i=j;return 0}}while(0);c[t>>2]=(c[t>>2]|0)+1;i=j;return 0}function MC(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;e=i;i=i+56|0;f=e|0;g=e+8|0;c[f>>2]=0;h=c[a>>2]|0;if((h|0)==0){I_(b,d<<2);j=1e3;i=e;return j|0}IY(g,a+4|0,d<<2);L19269:do{if((d|0)>0){k=g|0;l=0;while(1){m=c[k>>2]|0;n=m>>31;o=n^m;m=c[a>>2]|0;if((m|0)<(o|0)){break L19269}p=(HO(o,m,16)|0)^~n;c[b+(l<<2)>>2]=p;n=d-l|0;if((n|0)>0){m=p;o=(p|0)<0|0?-1:0;p=n;do{p=p-1|0;n=a+(p<<2)|0;q=c[n>>2]|0;N2(q,(q|0)<0|0?-1:0,m,o)|0;q=G<<1|0>>>31;r=k+(p<<2)|0;s=c[r>>2]|0;N2(s,(s|0)<0|0?-1:0,m,o)|0;c[r>>2]=q+s;c[n>>2]=(G<<1|0>>>31)+(c[n>>2]|0);}while((p|0)>0)}p=l+1|0;if((p|0)<(d|0)){k=k+4|0;l=p}else{break}}}}while(0);d=c[a>>2]|0;b=HQ((h|0)>0?h:-h|0,(d|0)>0?d:-d|0,f)|0;N2(b,(b|0)<0|0?-1:0,2097152e3,0)|0;b=G<<1|0>>>31;d=c[a>>2]|0;N2(d,(d|0)<0|0?-1:0,h,(h|0)<0|0?-1:0)|0;h=(G|0)<0?-b|0:b;b=c[f>>2]|0;f=b-21|0;if((f|0)>0){j=h<<f;i=e;return j|0}else{j=h>>21-b;i=e;return j|0}return 0}function MD(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0;if((d|0)<=0){return}f=c[494105]|0;g=c[494106]|0;h=c[494107]|0;i=c[494108]|0;j=c[494109]|0;k=c[494110]|0;l=c[494111]|0;m=c[494089]|0;n=c[494090]|0;o=c[494091]|0;p=c[494092]|0;q=c[494093]|0;r=c[494094]|0;s=c[494095]|0;t=c[494096]|0;u=c[494097]|0;v=c[494098]|0;w=c[494099]|0;x=c[494100]|0;y=c[494101]|0;z=c[494102]|0;A=c[494103]|0;if((e|0)==3){e=0;do{B=c[a+(e<<2)>>2]|0;c[b+(e<<2)>>2]=(l|0)<(B|0)?3:(k|0)<(B|0)?2:(j|0)<(B|0)?1:(i|0)<(B|0)?0:(h|0)<(B|0)?-1:(g|0)<(B|0)?-2:(f|0)<(B|0)|-4;e=e+1|0;}while((e|0)<(d|0));return}else{e=0;do{f=c[a+(e<<2)>>2]|0;c[b+(e<<2)>>2]=(A|0)<(f|0)?7:(z|0)<(f|0)?6:(y|0)<(f|0)?5:(x|0)<(f|0)?4:(w|0)<(f|0)?3:(v|0)<(f|0)?2:(u|0)<(f|0)?1:(t|0)<(f|0)?0:(s|0)<(f|0)?-1:(r|0)<(f|0)?-2:(q|0)<(f|0)?-3:(p|0)<(f|0)?-4:(o|0)<(f|0)?-5:(n|0)<(f|0)?-6:(m|0)<(f|0)|-8;e=e+1|0;}while((e|0)<(d|0));return}}function ME(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0;i=(f|0)==2;if(!((g|0)!=2|i)){return}if(!((f|0)!=2|(g|0)==2)){return}g=i?8:1;i=h+48|0;h=b+552|0;f=a+552|0;j=0;do{k=b+4+(j<<3)|0;l=a+4+(j<<3)|0;if((c[l>>2]|0)==0){if((c[k>>2]|0)!=0){m=0;n=0;o=13808}}else{m=0;n=0;o=13808}L19307:do{if((o|0)==13808){while(1){o=0;if((n|0)>=(c[i>>2]|0)){break}p=I$((c[d+320+(j*96|0)+(n<<2)>>2]|0)-(c[e+320+(j*96|0)+(n<<2)>>2]|0)|0)|0;q=p+m|0;if((p|0)>1|(q|0)>2){break L19307}m=q;n=n+1|0;o=13808}if((c[k>>2]|0)==0){c[d+(j<<2)>>2]=0;c[l>>2]=0;break}q=d+(j<<2)|0;if((c[l>>2]|0)==0){o=13814}else{if((c[q>>2]|0)>(c[e+(j<<2)>>2]|0)){o=13814}}if((o|0)==13814){o=0;c[q>>2]=1;c[l>>2]=1}c[f>>2]=c[h>>2];c[d+128+(j<<3)>>2]=c[e+128+(j<<3)>>2];c[d+64+(j<<3)>>2]=c[e+64+(j<<3)>>2];c[d+192+(j<<3)>>2]=c[e+192+(j<<3)>>2];c[d+256+(j<<3)>>2]=c[e+256+(j<<3)>>2];if((c[i>>2]|0)>0){r=0}else{break}do{c[d+320+(j*96|0)+(r<<2)>>2]=c[e+320+(j*96|0)+(r<<2)>>2];r=r+1|0;}while((r|0)<(c[i>>2]|0))}}while(0);j=j+1|0;}while((j|0)<(g|0));return}function MF(a,d,f,g,h,j,k,l){a=a|0;d=d|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0;h=i;i=i+240|0;f=h|0;m=h+48|0;n=h+96|0;o=h+144|0;p=h+192|0;do{if((l|0)==2){if((c[d+4+(k<<3)>>2]|0)==0){q=1}else{break}i=h;return q|0}else{if((c[d+4>>2]|0)==0){q=1}else{break}i=h;return q|0}}while(0);l=g+196|0;r=g+192|0;s=a+(k<<2)|0;if((c[s>>2]|0)<=0){q=0;i=h;return q|0}t=p|0;u=g+52|0;v=n|0;w=f;x=m;y=0;z=g+204|0;g=(c[d+552>>2]|0)==0?r:l;d=0;while(1){A=c[g>>2]|0;B=c[z>>2]|0;C=c[a+128+(k<<3)+(y<<2)>>2]|0;D=(C|0)>0;do{if(D){if((c[u>>2]|0)==4){E=0;do{c[p+(E<<2)>>2]=c[1976256+((c[a+320+(k*96|0)+(y*48|0)+(E<<2)>>2]|0)+8<<2)>>2];E=E+1|0;}while((E|0)<(C|0))}else{E=0;do{c[p+(E<<2)>>2]=c[1976320+((c[a+320+(k*96|0)+(y*48|0)+(E<<2)>>2]|0)+4<<2)>>2];E=E+1|0;}while((E|0)<(C|0))}E=c[t>>2]|0;c[v>>2]=E>>6;if((C|0)>1){F=1;while(1){H=F-1|0;I=0;do{c[o+(I<<2)>>2]=c[n+(H-I<<2)>>2];I=I+1|0;}while((I|0)<(F|0));I=c[p+(F<<2)>>2]|0;H=0;do{J=c[o+(H<<2)>>2]|0;N2(J,(J|0)<0|0?-1:0,I,(I|0)<0|0?-1:0)|0;J=n+(H<<2)|0;c[J>>2]=(G<<1|0>>>31)+(c[J>>2]|0);H=H+1|0;}while((H|0)<(F|0));c[n+(F<<2)>>2]=I>>6;H=F+1|0;if((H|0)<(C|0)){F=H}else{K=0;L=0;break}}}else{K=0;L=0}do{F=c[n+(K<<2)>>2]|0;H=(F|0)>0?F:-F|0;L=(L|0)>(H|0)?L:H;K=K+1|0;}while((K|0)<(C|0));do{if((L|0)==0){M=0}else{H=L>>31^L;if((H|0)<=-1){M=-1;break}F=~H;H=0;while(1){J=F<<1;if((J|0)<0){F=J;H=H+1|0}else{M=H;break}}}}while(0);H=(M|0)<6?M:6;F=0;do{I=n+(F<<2)|0;c[I>>2]=c[I>>2]<<H;F=F+1|0;}while((F|0)<(C|0));F=B-A|0;I=7-H|0;if(D){N=0}else{O=E;break}do{b[m+(N<<1)>>1]=(c[n+(N<<2)>>2]|0)>>>16&65535;N=N+1|0;}while((N|0)<(C|0));IY(m+(C<<1)|0,x,C<<1);I_(w,C<<2);if((F|0)<=0){O=E;break}H=C-1|0;J=0;P=0;while(1){Q=C-P|0;R=0;S=0;do{T=c[f+(S<<2)>>2]|0;U=(e[m+(Q+S<<1)>>1]|0)<<16;N2(U,(U|0)<0|0?-1:0,T,(T|0)<0|0?-1:0)|0;R=G+R|0;S=S+1|0;}while((S|0)<(C|0));S=P-1|0;Q=(S|0)<0?H:S;S=j+(J+A<<2)|0;T=c[S>>2]|0;c[f+(Q<<2)>>2]=T;c[S>>2]=T+(R<<I);T=J+1|0;if((T|0)<(F|0)){J=T;P=Q}else{O=E;break}}}else{c[v>>2]=d>>6;O=d}}while(0);A=y+1|0;if((A|0)<(c[s>>2]|0)){y=A;z=r;g=l;d=O}else{q=0;break}}i=h;return q|0}function MG(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0;h=c[f+8>>2]|0;i=(g|0)>0;do{if(i){j=0;do{k=c[b+(j<<2)>>2]|0;Ng(a+(j*972|0)|0,c[k+2116>>2]|0,c[k+2108>>2]|0,(c[d+(j<<2)>>2]|0)+7632|0,k+24|0,c[k>>2]|0,c[k+4>>2]|0,c[k+8>>2]|0);j=j+1|0;}while((j|0)<(g|0));j=a+1956|0;c[j>>2]=h;k=1;l=0;do{k=(c[(c[b+(l<<2)>>2]|0)+12>>2]|0)==2?0:k;I_((c[d+(l<<2)>>2]|0)+9072|0,240);l=l+1|0;}while((l|0)<(g|0));l=(k|0)==0;m=0;n=0;while(1){o=c[b+(n<<2)>>2]|0;if(l){c[f+76+(n<<2)>>2]=1610612736;c[f+84+(n<<2)>>2]=1;p=m}else{q=o|0;if((c[q>>2]|0)>0){r=o+2116|0;s=o+2100|0;t=0;u=0;v=0;w=0;x=0;y=0;while(1){z=HH(c[(c[r>>2]|0)+(x<<2)>>2]>>1)|0;A=HH(c[(c[r>>2]|0)+(x<<2)>>2]>>2)|0;B=(c[a+(n*972|0)+(x<<2)>>2]|0)+y|0;C=(c[(c[s>>2]|0)+(x<<2)>>2]>>6)+t|0;D=(z>>6)+v|0;E=(A>>6)+w|0;N2(A,(A|0)<0|0?-1:0,z,(z|0)<0|0?-1:0)|0;z=((G<<1|0>>>31)>>6)+u|0;A=x+1|0;if((A|0)<(c[q>>2]|0)){t=C;u=z;v=D;w=E;x=A;y=B}else{F=C;H=z;I=D;J=E;K=B;break}}}else{F=0;H=0;I=0;J=0;K=0}y=HE(F)|0;x=HE(J)|0;w=(HE(I)|0)-y|0;v=(HE(H)|0)-y|0;do{if((HR(K,c[o+24+(c[q>>2]<<2)>>2]|0)|0)<402653184){u=f+76+(n<<2)|0;c[u>>2]=402653184;L=0;M=u;N=0}else{u=HR(K,c[o+24+(c[q>>2]<<2)>>2]|0)|0;t=f+76+(n<<2)|0;c[t>>2]=u;s=(u|0)>1677721600;u=s&1;if(!s){L=0;M=t;N=u;break}L=(c[f+84+(n<<2)>>2]|0)!=0|0;M=t;N=u}}while(0);u=c[q>>2]|0;L19400:do{if((u|0)>0){t=o+2100|0;s=o+2116|0;r=x-y|0;B=d+(n<<2)|0;if((n|0)==1){E=0;D=u;while(1){do{if((((c[e+4+(E<<2)>>2]|0)==0?L:m)|0)==0){O=D}else{if((c[(c[t>>2]|0)+(E<<2)>>2]|0)<=0){O=D;break}z=c[M>>2]|0;do{if((z|0)>1744830464){C=c[(c[s>>2]|0)+(E<<2)>>2]|0;c[(c[B>>2]|0)+9072+(E<<2)>>2]=C+r+(C>>1)>>1}else{C=c[(c[s>>2]|0)+(E<<2)>>2]|0;if((z|0)>1711276032){c[(c[B>>2]|0)+9072+(E<<2)>>2]=C+w>>1;break}else{c[(c[B>>2]|0)+9072+(E<<2)>>2]=(C>>1)+v>>1;break}}}while(0);z=(c[B>>2]|0)+9072+(E<<2)|0;C=c[z>>2]|0;c[z>>2]=(C|0)<0?C:0;O=c[q>>2]|0}}while(0);C=E+1|0;if((C|0)<(O|0)){E=C;D=O}else{break L19400}}}D=(L|0)==0;E=0;C=u;while(1){do{if(D){P=C}else{if((c[(c[t>>2]|0)+(E<<2)>>2]|0)<=0){P=C;break}z=c[M>>2]|0;do{if((z|0)>1744830464){A=c[(c[s>>2]|0)+(E<<2)>>2]|0;c[(c[B>>2]|0)+9072+(E<<2)>>2]=A+r+(A>>1)>>1}else{A=c[(c[s>>2]|0)+(E<<2)>>2]|0;if((z|0)>1711276032){c[(c[B>>2]|0)+9072+(E<<2)>>2]=A+w>>1;break}else{c[(c[B>>2]|0)+9072+(E<<2)>>2]=(A>>1)+v>>1;break}}}while(0);z=(c[B>>2]|0)+9072+(E<<2)|0;A=c[z>>2]|0;c[z>>2]=(A|0)<0?A:0;P=c[q>>2]|0}}while(0);A=E+1|0;if((A|0)<(P|0)){E=A;C=P}else{break}}}}while(0);c[f+84+(n<<2)>>2]=N;p=L}q=n+1|0;if((q|0)<(g|0)){m=p;n=q}else{break}}if(i){Q=0}else{R=j;break}while(1){n=c[d+(Q<<2)>>2]|0;m=b+(Q<<2)|0;l=c[m>>2]|0;if((c[l>>2]|0)>0){k=0;q=l;while(1){if((c[q+8>>2]|0)>0){l=0;while(1){v=l+k|0;w=c[n+9072+(v<<2)>>2]|0;c[n+8832+(v<<2)>>2]=(c[n+8352+(v<<2)>>2]|0)-w;u=n+7872+(v<<2)|0;c[u>>2]=(c[u>>2]|0)-w;w=l+1|0;u=c[m>>2]|0;if((w|0)<(c[u+8>>2]|0)){l=w}else{S=u;break}}}else{S=q}l=(c[S+4>>2]|0)+k|0;if((l|0)<(c[S>>2]|0)){k=l;q=S}else{break}}}q=Q+1|0;if((q|0)<(g|0)){Q=q}else{R=j;break}}}else{j=a+1956|0;c[j>>2]=h;R=j}}while(0);h=a+1944|0;c[h>>2]=c[R>>2];R=a+1948|0;c[R>>2]=0;Q=a+1952|0;c[Q>>2]=0;if(i){T=0}else{return}do{i=c[d+(T<<2)>>2]|0;S=c[b+(T<<2)>>2]|0;Nh(a+(T*972|0)|0,i+8832|0,i+7872|0,c[S>>2]|0,c[S+4>>2]|0,c[S+8>>2]|0,S+1616|0,S+1856|0);c[h>>2]=(c[h>>2]|0)+(c[a+(T*972|0)+960>>2]|0);c[R>>2]=(c[R>>2]|0)+(c[a+(T*972|0)+964>>2]|0);c[Q>>2]=(c[Q>>2]|0)+(c[a+(T*972|0)+968>>2]|0);T=T+1|0;}while((T|0)<(g|0));return}function MH(b,d,e,f,g,h,j,k){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0;l=i;i=i+24|0;m=l|0;n=l+8|0;o=l+16|0;p=k+j|0;q=c[h>>2]|0;if((j|0)>0){r=q;s=k}else{t=q;c[h>>2]=t;i=l;return}L19454:while(1){L19456:do{if((c[b+16+(s*24|0)>>2]|0)==4){u=r}else{q=b+16+(s*24|0)+8|0;k=c[q>>2]|0;if((k|0)>0){j=c[e+(s<<2)>>2]|0;v=0;do{w=c[j+(v<<2)>>2]|0;c[m+(v<<2)>>2]=(c[w+8>>2]|0)-1;c[n+(v<<2)>>2]=c[w>>2];c[o+(v<<2)>>2]=c[w+4>>2];v=v+1|0;}while((v|0)<(k|0))}v=d+(s<<2)|0;j=c[v>>2]|0;w=j+1984|0;x=r;y=0;z=k;while(1){A=y;while(1){if((A|0)<(z|0)){break}if((c[w>>2]|0)>(g|0)){A=0}else{u=x;break L19456}}B=c[(c[v>>2]|0)+2e3+(A<<2)>>2]|0;C=m+(A<<2)|0;D=c[C>>2]|0;if((D|0)>-1){c[C>>2]=D-1;C=c[n+(A<<2)>>2]|0;if((C|0)>0){E=c[o+(A<<2)>>2]|0;F=0;G=0;while(1){H=F+D|0;do{if((a[f+(s*120|0)+(A*60|0)+H|0]|0)==0){I=G}else{J=B+8112+(H<<2)|0;if((c[J>>2]|0)>=-10802114){I=G;break}c[J>>2]=-10802114;J=c[B+8832+(H<<2)>>2]|0;K=B+7872+(H<<2)|0;if((J|0)<((c[K>>2]|0)+10802114|0)){I=G;break}c[K>>2]=J-10802114;J=j+40+(A*972|0)+240+(H<<2)|0;K=G-(c[J>>2]>>16)|0;L=(c[j+40+(A*972|0)+(H<<2)>>2]|0)*98304|0;c[J>>2]=L;I=K+(L>>16)|0}}while(0);H=E+F|0;if((H|0)<(C|0)){F=H;G=I}else{M=I;break}}}else{M=0}c[w>>2]=(c[w>>2]|0)+M;G=j+40+(A*972|0)+960|0;c[G>>2]=(c[G>>2]|0)+M;G=M+x|0;if((c[w>>2]|0)<=(g|0)){t=G;N=13932;break L19454}O=G;P=c[q>>2]|0}else{if((A|0)==(z-1|0)){t=x;N=13934;break L19454}else{O=x;P=z}}x=O;y=A+1|0;z=P}}}while(0);z=s+1|0;if((z|0)<(p|0)){r=u;s=z}else{t=u;N=13931;break}}if((N|0)==13932){c[h>>2]=t;i=l;return}else if((N|0)==13931){c[h>>2]=t;i=l;return}else if((N|0)==13934){c[h>>2]=t;i=l;return}}function MI(b,d,e,f,g,h){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0;g=i;i=i+696|0;j=g|0;k=g+64|0;l=g+96|0;m=g+216|0;n=(h|0)>0;if(n){o=0;do{p=d+(o<<2)|0;q=c[p>>2]|0;if((c[q>>2]|0)>0){r=0;s=q;while(1){if((c[s+8>>2]|0)>0){q=0;t=s;while(1){u=q+r|0;c[m+(o*240|0)+(u<<2)>>2]=HH(c[(c[t+2108>>2]|0)+(u<<2)>>2]>>2)|0;u=q+1|0;v=c[p>>2]|0;if((u|0)<(c[v+8>>2]|0)){q=u;t=v}else{w=v;break}}}else{w=s}t=(c[w+4>>2]|0)+r|0;if((t|0)<(c[w>>2]|0)){r=t;s=w}else{break}}}o=o+1|0;}while((o|0)<(h|0))}MJ(b,d,e+32|0,h);MK(b,d,l|0,f,h,e+20|0);f=c[e+68>>2]|0;o=e+72|0;do{if(n){e=0;w=0;s=0;do{r=b+(s<<2)|0;x=c[d+(s<<2)>>2]|0;p=x|0;t=c[p>>2]|0;q=(t|0)>0;if(q){v=c[x+8>>2]|0;u=(v|0)>0;y=c[x+4>>2]|0;z=x+2100|0;A=0;B=0;C=0;while(1){D=j+(A<<3)+(s<<2)|0;c[D>>2]=0;if(u){E=c[z>>2]|0;F=0;H=0;do{H=(c[E+(F+C<<2)>>2]>>8)+H|0;F=F+1|0;}while((F|0)<(v|0));c[D>>2]=H;I=H}else{I=0}F=I+B|0;E=C+y|0;if((E|0)<(t|0)){A=A+1|0;B=F;C=E}else{J=F;break}}}else{J=0}w=J+w|0;do{if((c[(c[d>>2]|0)+12>>2]|0)==2){K=0;L=1073741824}else{C=c[r>>2]|0;if(!q){K=0;L=2147483647;break}B=x+8|0;A=x+4|0;y=x+2116|0;v=x+2108|0;z=x+2100|0;u=0;F=0;E=0;M=0;N=c[B>>2]|0;O=t;while(1){if((N|0)>0){P=0;Q=F;R=E;S=M;T=N;while(1){U=P+u|0;if((c[(c[y>>2]|0)+(U<<2)>>2]|0)>(c[(c[v>>2]|0)+(U<<2)>>2]|0)){V=((HH(c[C+7632+(U<<2)>>2]|0)|0)>>4)+R|0;W=(c[(c[z>>2]|0)+(U<<2)>>2]>>8)+S|0;X=V;Y=(c[x+24+(U+1<<2)>>2]|0)+Q-(c[x+24+(U<<2)>>2]|0)|0;Z=c[B>>2]|0}else{W=S;X=R;Y=Q;Z=T}U=P+1|0;if((U|0)<(Z|0)){P=U;Q=Y;R=X;S=W;T=Z}else{break}}_=Y;$=X;aa=W;ab=Z;ac=c[p>>2]|0}else{_=F;$=E;aa=M;ab=N;ac=O}T=(c[A>>2]|0)+u|0;if((T|0)<(ac|0)){u=T;F=_;E=$;M=aa;N=ab;O=ac}else{break}}if((_|0)<=0){K=0;L=2147483647;break}O=(HE($)|0)>>>1;N=(HE(aa)|0)>>3;M=HE(_<<15)|0;N2(M,(M|0)<0|0?-1:0,1610612736,0)|0;M=HH(O+2080374784-N-G<<1)|0;K=(M|0)<0|0?-1:0;L=M}}while(0);N2(L,K,J,(J|0)<0|0?-1:0)|0;e=(G<<1|0>>>31)+e|0;s=s+1|0;}while((s|0)<(h|0));if((w|0)<=(e|0)){ad=2147483647;ae=w;af=x;break}if((w|0)>-1){s=~w;p=0;while(1){t=s<<1;if((t|0)<0){s=t;p=p+1|0}else{ag=p;break}}}else{ag=-1}ad=HO(e<<ag,w<<ag,16)|0;ae=w;af=x}else{ad=2147483647;ae=0;af=0}}while(0);x=c[o>>2]|0;N2(x,(x|0)<0|0?-1:0,1610612736,0)|0;x=(G<<1|0>>>31)+((ad>>>3|((ad|0)<0|0?-1:0)<<29)<<1|0>>>31)|0;ag=(ad|0)<(x|0)?ad:x;c[o>>2]=ag;o=ag-429496736|0;N2(o,(o|0)<0|0?-1:0,1252698752,0)|0;o=(G<<1|0>>>31)+107374184|0;ag=(o|0)<53687092?53687092:o;o=(ag|0)>536870911?2147483644:ag<<2;L19540:do{if((c[(c[d>>2]|0)+12>>2]|0)==2){ag=af|0;if((c[ag>>2]|0)<=0){break}x=ae>>3;ad=f;J=(f|0)<0|0?-1:0;K=o;L=(o|0)<0|0?-1:0;N2(K,L,ad,J)|0;J=G<<1|0>>>31;ad=J;L=(J|0)<0|0?-1:0;J=af+4|0;if(n){ah=0;ai=0}else{K=(x|0)>0?0:ae>>5;_=0;aa=0;while(1){$=HH((HE(K)|0)>>2)|0;N2($,($|0)<0|0?-1:0,ad,L)|0;c[k+(aa<<2)>>2]=G<<5|0>>>27;$=(c[J>>2]|0)+_|0;if(($|0)<(c[ag>>2]|0)){_=$;aa=aa+1|0}else{break L19540}}}while(1){aa=0;_=0;do{aa=(c[j+(ai<<3)+(_<<2)>>2]|0)+aa|0;_=_+1|0;}while((_|0)<(h|0));_=c[1914816+(c[(c[d>>2]|0)+272+(ai<<2)>>2]<<2)>>2]|0;N2(_,(_|0)<0|0?-1:0,aa,(aa|0)<0|0?-1:0)|0;_=G<<1|0>>>31;K=HH((HE(((_|0)<(x|0)?_:x)>>2)|0)>>2)|0;N2(K,(K|0)<0|0?-1:0,ad,L)|0;c[k+(ai<<2)>>2]=G<<5|0>>>27;K=(c[J>>2]|0)+ah|0;if((K|0)<(c[ag>>2]|0)){ah=K;ai=ai+1|0}else{break}}}else{N2(o,(o|0)<0|0?-1:0,f,(f|0)<0|0?-1:0)|0;ag=G<<1|0>>>31;J=HH((HE(ae)|0)>>2)|0;N2(J,(J|0)<0|0?-1:0,ag,(ag|0)<0|0?-1:0)|0;c[k>>2]=G<<3|0>>>29}}while(0);if(!n){i=g;return}n=c[k>>2]|0;ae=0;do{f=c[b+(ae<<2)>>2]|0;o=d+(ae<<2)|0;ai=c[o>>2]|0;ah=ai|0;j=c[ah>>2]|0;if((j|0)>0){af=ai+8|0;ag=ai+4|0;J=0;L=c[af>>2]|0;ad=j;while(1){if((L|0)>0){j=0;x=L;while(1){K=j+J|0;_=c[f+8832+(K<<2)>>2]|0;w=f+7872+(K<<2)|0;e=c[w>>2]|0;$=c[m+(ae*240|0)+(K<<2)>>2]|0;do{if((e|0)>-1107296257&(_|0)>(e|0)){ac=l+(ae*60|0)+K|0;ab=a[ac]|0;if(ab<<24>>24==2){aj=x;break}do{if((c[(c[o>>2]|0)+12>>2]|0)==2){Z=(j|0)/(c[ag>>2]|0)|0;W=ai+272+(Z<<2)|0;X=c[W>>2]|0;Y=c[1914640+(X<<2)>>2]|0;I=Y;p=(Y|0)<0|0?-1:0;Y=1513975936;s=0;N2(I,p,Y,s)|0;s=G<<1|0>>>31;Y=$;p=($|0)<0|0?-1:0;I=s;t=(s|0)<0|0?-1:0;N2(I,t,Y,p)|0;p=G<<3|0>>>29;Y=c[k+(Z<<2)>>2]|0;do{if((p|0)>(32768-Y|0)){if((Y|0)>=(2147483647-p|0)){ak=-201326592;al=X;break}Z=((HE(p+Y|0)|0)<<2)-201326592|0;ak=Z;al=c[W>>2]|0}else{ak=1946157056;al=X}}while(0);am=(HK(al)|0)+ak|0}else{if((n|0)>=(2147483647-$|0)){am=0;break}am=(HE($+n|0)|0)<<2}}while(0);X=c[f+8112+(K<<2)>>2]|0;if((am-_|0)<=(X|0)|ab<<24>>24==0){an=am}else{if((X|0)>(-2147483648-_|0)){W=X+_|0;ao=(W|0)>(e|0)?W:e}else{ao=e}a[ac]=2;an=ao}W=(an|0)<-1073741824?-2147483648:an;if((_+2147483647|0)>323250667){X=_-323250667|0;ap=(W|0)>(X|0)?W:X}else{ap=W}c[w>>2]=(ap|0)<-1107296256?-1107296256:ap;aj=c[af>>2]|0}else{aj=x}}while(0);w=j+1|0;if((w|0)<(aj|0)){j=w;x=aj}else{break}}aq=aj;ar=c[ah>>2]|0}else{aq=L;ar=ad}x=(c[ag>>2]|0)+J|0;if((x|0)<(ar|0)){J=x;L=aq;ad=ar}else{break}}}ae=ae+1|0;}while((ae|0)<(h|0));i=g;return}function MJ(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0;if((e|0)<=0){return}f=d+4|0;g=d+16|0;h=d+12|0;i=d|0;d=0;do{j=b+(d<<2)|0;k=c[j>>2]|0;l=c[k>>2]|0;do{if((l|0)>0){m=c[k+8>>2]|0;do{if((m|0)>0){n=0;o=0;p=0;while(1){q=(m|0)>1?m:1;r=c[k+2100>>2]|0;s=0;t=p;do{t=(c[r+(s+o<<2)>>2]>>6)+t|0;s=s+1|0;}while((s|0)<(m|0));u=n+q|0;s=(c[k+4>>2]|0)+o|0;if((s|0)<(l|0)){n=u;o=s;p=t}else{break}}if((t|0)==0|(u|0)==0){v=-2147483648;w=k;x=l;break}p=HK(u)|0;o=201326592-p+(HE(t)|0)|0;p=c[j>>2]|0;v=o;w=p;x=c[p>>2]|0}else{p=0;while(1){o=(c[k+4>>2]|0)+p|0;if((o|0)<(l|0)){p=o}else{v=-2147483648;w=k;x=l;break}}}}while(0);if((x|0)<=0){break}m=a+(d<<2)|0;p=0;o=w;while(1){if((c[o+8>>2]|0)>0){n=0;s=o;while(1){r=n+p|0;y=c[m>>2]|0;z=c[y+8352+(r<<2)>>2]|0;if((z+(c[f>>2]|0)|0)<(v|0)){A=v-z|0;z=A;B=(A|0)<0|0?-1:0;A=646457024;C=0;N2(z,B,A,C)|0;C=G<<1|0>>>31;A=c[g>>2]|0;B=c[h>>2]|0;z=B;D=(B|0)<0|0?-1:0;B=C;E=(C|0)<0|0?-1:0;N2(z,D,B,E)|0;E=(G<<1|0>>>31)+A|0;A=c[i>>2]|0;B=(E|0)>(A|0)?E:A;A=y+8112+(r<<2)|0;y=c[A>>2]|0;E=y;D=(y|0)<0|0?-1:0;y=B;z=(B|0)<0|0?-1:0;N2(y,z,E,D)|0;c[A>>2]=G<<7|0>>>25;A=(c[m>>2]|0)+8112+(r<<2)|0;r=c[A>>2]|0;c[A>>2]=(r|0)>-10802114?-10802114:r;F=c[j>>2]|0}else{F=s}r=n+1|0;if((r|0)<(c[F+8>>2]|0)){n=r;s=F}else{H=F;break}}}else{H=o}s=(c[H+4>>2]|0)+p|0;if((s|0)<(c[H>>2]|0)){p=s;o=H}else{break}}}}while(0);d=d+1|0;}while((d|0)<(e|0));return}function MK(b,d,e,f,g,h){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0;i=(g|0)>0;if(i){j=0;do{k=c[b+(j<<2)>>2]|0;l=d+(j<<2)|0;m=c[l>>2]|0;n=(c[m>>2]|0)>0;do{if((c[m+12>>2]|0)==2){if(n){o=0;p=m}else{break}while(1){if((c[p+8>>2]|0)>0){q=0;while(1){r=k+9312+(q+o<<2)|0;s=c[r>>2]|0;N2(s,(s|0)<0|0?-1:0,1352914688,0)|0;c[r>>2]=G<<1|0>>>31;r=q+1|0;s=c[l>>2]|0;if((r|0)<(c[s+8>>2]|0)){q=r}else{t=s;break}}}else{t=p}q=(c[t+4>>2]|0)+o|0;if((q|0)<(c[t>>2]|0)){o=q;p=t}else{break}}}else{if(n){u=0;v=m}else{break}while(1){if((c[v+8>>2]|0)>0){q=0;while(1){s=k+9312+(q+u<<2)|0;c[s>>2]=c[s>>2]>>1;s=q+1|0;r=c[l>>2]|0;if((s|0)<(c[r+8>>2]|0)){q=s}else{w=r;break}}}else{w=v}q=(c[w+4>>2]|0)+u|0;if((q|0)<(c[w>>2]|0)){u=q;v=w}else{break}}}}while(0);j=j+1|0;}while((j|0)<(g|0))}if((c[h>>2]|0)!=0){if(i){x=0}else{return}do{h=c[b+(x<<2)>>2]|0;j=d+(x<<2)|0;w=c[j>>2]|0;if((c[w>>2]|0)>0){v=0;u=w;while(1){w=c[u+8>>2]|0;if((w|0)>0){t=0;p=w;while(1){w=t+v|0;o=(c[h+8592+(((t|0)<(p-1|0))+w<<2)>>2]>>1)+(c[h+8592+((((t|0)>0)<<31>>31)+w<<2)>>2]>>1)|0;l=HE(o)|0;k=c[h+8592+(w<<2)>>2]|0;m=c[h+8352+(w<<2)>>2]|0;if((k|0)>(o|0)){o=l-m-10802114|0;if((c[(c[j>>2]|0)+12>>2]|0)==0){y=(o|0)>-55767585?o:-55767585}else{y=(o|0)>-33554432?o:-33554432}o=h+8112+(w<<2)|0;n=c[o>>2]|0;c[o>>2]=(n|0)<(y|0)?n:y}if((m+33554432|0)<(l|0)&(k|0)>0){k=h+8112+(w<<2)|0;w=c[k>>2]|0;n=l-33554432-m+w|0;m=(n|0)>-10802114?-10802114:n;n=w+55697826|0;c[k>>2]=(m|0)<(n|0)?m:n}n=t+1|0;m=c[j>>2]|0;k=c[m+8>>2]|0;if((n|0)<(k|0)){t=n;p=k}else{z=m;break}}}else{z=u}p=(c[z+4>>2]|0)+v|0;if((p|0)<(c[z>>2]|0)){v=p;u=z}else{break}}}x=x+1|0;}while((x|0)<(g|0))}do{if((g|0)==2){x=c[b>>2]|0;z=c[b+4>>2]|0;y=c[d>>2]|0;u=y|0;v=c[u>>2]|0;if((v|0)<=0){break}j=y+8|0;h=y+4|0;y=0;p=c[j>>2]|0;t=v;while(1){if((p|0)>0){v=0;do{m=v+y|0;do{if((c[f+4+(m<<2)>>2]|0)!=0){k=c[x+8352+(m<<2)>>2]|0;n=z+8352+(m<<2)|0;w=c[n>>2]|0;l=(k|0)>(w|0)?k:w;w=x+8112+(m<<2)|0;o=c[w>>2]|0;if(((o>>1)-33554432+(l>>1)|0)<-1073741823){A=-2147483648}else{A=o-67108864+l|0}l=x+8592+(m<<2)|0;q=(c[l>>2]|0)>0?A-k|0:0;k=(o|0)>(q|0)?o:q;if((k|0)<1){B=(k|0)<-10802114?k:-10802114}else{B=k}c[w>>2]=B;w=c[z+8592+(m<<2)>>2]|0;if((w|0)>0){C=A-(c[n>>2]|0)|0}else{C=0}n=z+8112+(m<<2)|0;k=c[n>>2]|0;q=(k|0)>(C|0)?k:C;if((q|0)<1){D=(q|0)<-10802114?q:-10802114}else{D=q}c[n>>2]=D;n=x+9312+(m<<2)|0;if((c[l>>2]|0)>(c[n>>2]|0)){q=w;k=(w|0)<0|0?-1:0;o=1932735232;r=0;N2(q,k,o,r)|0;r=G<<1|0>>>31;c[z+9312+(m<<2)>>2]=r;E=r}else{E=c[z+9312+(m<<2)>>2]|0}if((w|0)<=(E|0)){break}w=c[l>>2]|0;l=w;r=(w|0)<0|0?-1:0;w=1932735232;o=0;N2(l,r,w,o)|0;c[n>>2]=G<<1|0>>>31}}while(0);v=v+1|0;F=c[j>>2]|0;}while((v|0)<(F|0));H=F;I=c[u>>2]|0}else{H=p;I=t}v=(c[h>>2]|0)+y|0;if((v|0)<(I|0)){y=v;p=H;t=I}else{break}}}}while(0);if(i){J=0}else{return}do{i=c[b+(J<<2)>>2]|0;I=c[d+(J<<2)>>2]|0;H=I|0;F=c[H>>2]|0;if((F|0)>0){E=I+8|0;D=I+4|0;I=0;C=c[E>>2]|0;A=F;while(1){if((C|0)>0){F=0;do{B=F+I|0;do{if((c[i+9312+(B<<2)>>2]|0)>(c[i+8592+(B<<2)>>2]|0)){K=14076}else{if((c[i+8112+(B<<2)>>2]|0)>0){K=14076;break}a[e+(J*60|0)+B|0]=1}}while(0);if((K|0)==14076){K=0;a[e+(J*60|0)+B|0]=0}F=F+1|0;L=c[E>>2]|0;}while((F|0)<(L|0));M=L;N=c[H>>2]|0}else{M=C;N=A}F=(c[D>>2]|0)+I|0;if((F|0)<(N|0)){I=F;C=M;A=N}else{break}}}J=J+1|0;}while((J|0)<(g|0));return}function ML(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;d=J2(0)|0;L19722:do{if((d|0)==0){e=1}else{f=0;while(1){if((f|0)>=(b|0)){e=0;break L19722}g=J4(f)|0;c[d+64+(f<<2)>>2]=g;if((g|0)==0){e=1;break}else{f=f+1|0}}}}while(0);c[a>>2]=d;return e|0}function MM(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0;c[a>>2]=429496730;c[a+4>>2]=2040109466;c[a+8>>2]=-107374182;c[a+12>>2]=644245094;c[a+16>>2]=429496730;c[a+20>>2]=2040109466;c[a+24>>2]=-214748365;c[a+28>>2]=858993459;c[a+32>>2]=429496736;c[a+36>>2]=1610612735;c[a+40>>2]=0;c[a+44>>2]=429496736;c[a+48>>2]=429496736;c[a+52>>2]=1610612735;c[a+56>>2]=-107374184;c[a+60>>2]=1073741824;if((e|0)<=0){return}g=b<<16;b=g;h=(g|0)<0|0?-1:0;N2(b,h,1717986944,0)|0;g=G+16384>>16;N2(b,h,1288490240,0)|0;h=G+16384>>15;b=0;do{i=c[a+64+(b<<2)>>2]|0;j=c[c[d+(b<<2)>>2]>>2]|0;c[i>>2]=g;c[i+4>>2]=h;c[i+72>>2]=644245120;k=i+8|0;c[k>>2]=0;c[i+68>>2]=f;do{if((j|0)<32e3){l=j<<16;m=l;n=(l|0)<0|0?-1:0;l=6710887;o=0;N2(m,n,l,o)|0;o=100-(G+16384>>15)|0;c[k>>2]=(o|0)<50?50:o;o=i+20|0;if((j|0)>2e4){p=o;q=14095;break}c[o>>2]=0;c[i+24>>2]=0;c[i+28>>2]=0}else{p=i+20|0;q=14095}}while(0);if((q|0)==14095){q=0;c[p>>2]=1;c[i+24>>2]=15;c[i+28>>2]=3}c[i+32>>2]=8388608;c[i+36>>2]=111465408;c[i+44>>2]=-805306368;c[i+48>>2]=46137344;c[i+60>>2]=1073741824;c[i+64>>2]=1;c[i+56>>2]=-1;c[i+52>>2]=0;c[i+12>>2]=1267015296;c[i+16>>2]=1;b=b+1|0;}while((b|0)<(e|0));return}function MN(a,b,d,e,f,g,h,j,k,l,m,n,o){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;var p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0.0,aa=0.0,ab=0,ac=0,ad=0,ae=0,af=0;j=i;i=i+16|0;p=j|0;q=j+8|0;r=c[e+1944>>2]|0;e=c[(c[d>>2]|0)+12>>2]|0;do{if((h|0)==2){if((e|0)!=2){if((c[(c[d+4>>2]|0)+12>>2]|0)!=2){s=0;break}}s=2}else{s=e}}while(0);do{if((k|0)>0){if((o|0)!=0){e=b+12|0;d=c[e>>2]|0;h=b+16|0;t=c[h>>2]|0;u=k<<17;v=d;w=(d|0)<0|0?-1:0;d=u;x=(u|0)<0|0?-1:0;N2(v,w,d,x)|0;x=(G<<1|0>>>31)>>17-t;c[f>>2]=x;y=x;z=e;A=h;break}if((m|0)>(l|0)){B=HR(l,m)|0}else{B=2147483647}h=(s|0)==2;e=h?a+32|0:a|0;x=b|0;t=c[x>>2]|0;d=(t|0)<(r|0)?r:t;w=b+4|0;v=c[w>>2]|0;u=(d|0)<(v|0)?d:v;d=c[e>>2]|0;C=c[e+4>>2]|0;D=c[e+12>>2]|0;E=(B|0)>(d|0)?B:d;F=((E|0)<(C|0)?E:C)-d|0;d=F;C=(F|0)<0|0?-1:0;F=h?780903145:1002159036;E=h?0:0;N2(d,C,F,E)|0;E=D-(G<<1|0>>>31)|0;D=c[e+16>>2]|0;F=c[e+20>>2]|0;C=c[e+24>>2]|0;e=(B|0)>(D|0)?B:D;d=((e|0)<(F|0)?e:F)-D|0;D=d;F=(d|0)<0|0?-1:0;d=h?2147483647:1431655765;e=h?0:0;N2(D,F,d,e)|0;e=G<<1|0>>>31;d=HR(u-t|0,v-t|0)|0;t=E+C+e|0;e=t;C=(t|0)<0|0?-1:0;t=d;v=(d|0)<0|0?-1:0;N2(e,C,t,v)|0;v=1073741823-(E>>1)+((G<<1|0>>>31)>>1)|0;E=HQ(l,k,p)|0;t=31-(c[p>>2]|0)|0;c[p>>2]=t;C=(t|0)<31?t:31;e=31-C|0;d=(E>>t-C>>1)+(1503238554>>>(e>>>0)>>1)|0;t=C-1|0;C=v>>e;e=(t|0)>24?24:t;v=((C|0)<(d|0)?C:d)>>t-e;t=24-e|0;do{if((n>>t|0)<(v|0)){H=n}else{if((e|0)<24){H=v<<t;break}else{H=v>>e-24;break}}}while(0);e=r<<16;v=e;t=(e|0)<0|0?-1:0;e=357913952;d=0;N2(v,t,e,d)|0;d=G+16384>>15;e=c[w>>2]|0;do{if((e|0)<(r|0)){t=r-e<<16;v=t;C=(t|0)<0|0?-1:0;t=644245120;E=0;N2(v,C,t,E)|0;E=(c[x>>2]|0)+(G+16384>>15)|0;c[x>>2]=E;t=2147483647;u=0;N2(v,C,t,u)|0;u=(G+16384>>15)+e|0;c[w>>2]=u;I=u;J=E}else{E=c[x>>2]|0;if((E|0)>(r|0)){u=E-r<<16;t=u;C=(u|0)<0|0?-1:0;u=300647712;v=0;N2(t,C,u,v)|0;v=E-(G+16384>>15)|0;c[x>>2]=v;u=150323856;F=0;N2(t,C,u,F)|0;F=e-(G+16384>>15)|0;c[w>>2]=F;I=F;J=v;break}else{v=r-E<<16;F=v;u=(v|0)<0|0?-1:0;v=644245120;C=0;N2(F,u,v,C)|0;C=(G+16384>>15)+E|0;c[x>>2]=C;E=e-r<<16;v=E;u=(E|0)<0|0?-1:0;E=150323856;F=0;N2(v,u,E,F)|0;F=e-(G+16384>>15)|0;c[w>>2]=F;I=F;J=C;break}}}while(0);if((I-J|0)<(d|0)){e=r-J|0;C=(e|0)<0?0:e;e=I-r|0;F=(e|0)<0?0:e;e=F+C|0;E=HR(F,e)|0;F=d<<16;u=E;v=(E|0)<0|0?-1:0;E=F;t=(F|0)<0|0?-1:0;N2(u,v,E,t)|0;v=(G+16384>>15)+r|0;u=HR(C,e)|0;e=u;C=(u|0)<0|0?-1:0;N2(e,C,E,t)|0;t=r-(G+16384>>15)|0;c[w>>2]=v;c[x>>2]=(t|0)<0?0:t}t=b+12|0;v=c[t>>2]|0;E=H;C=(H|0)<0|0?-1:0;e=v;u=(v|0)<0|0?-1:0;N2(e,u,E,C)|0;C=G<<1|0>>>31;E=b+16|0;u=c[E>>2]|0;e=k<<17;v=C;F=(C|0)<0|0?-1:0;C=e;D=(e|0)<0|0?-1:0;N2(v,F,C,D)|0;D=(G<<1|0>>>31)>>10-u;c[f>>2]=D;y=D;z=t;A=E}else{c[f>>2]=0;y=0;z=b+12|0;A=b+16|0}}while(0);k=b+60|0;H=b+64|0;I=b+52|0;J=c[I>>2]|0;n=b+56|0;b=c[n>>2]|0;p=c[z>>2]|0;z=c[A>>2]|0;if((b|0)<=0){K=1073741824;c[k>>2]=K;c[H>>2]=1;L=c[f>>2]|0;M=L<<17;N=M;O=(M|0)<0|0?-1:0;P=K;Q=(K|0)<0|0?-1:0;R=N2(P,Q,N,O)|0;S=G;T=S;U=0;V=T<<1|0>>>31;W=U<<1|T>>>31;X=V;Y=X;Z=Y>>16;c[g>>2]=Z;_=c[f>>2]|0;c[I>>2]=_;c[n>>2]=-1;i=j;return}$=+(((y|0)<(r|0)?y:r)|0);aa=+(J|0);if(!($<aa*1.5&$>aa*.699999988079071)){K=1073741824;c[k>>2]=K;c[H>>2]=1;L=c[f>>2]|0;M=L<<17;N=M;O=(M|0)<0|0?-1:0;P=K;Q=(K|0)<0|0?-1:0;R=N2(P,Q,N,O)|0;S=G;T=S;U=0;V=T<<1|0>>>31;W=U<<1|T>>>31;X=V;Y=X;Z=Y>>16;c[g>>2]=Z;_=c[f>>2]|0;c[I>>2]=_;c[n>>2]=-1;i=j;return}r=p;y=(p|0)<0|0?-1:0;N2(r,y,1288490240,0)|0;p=G<<1|0>>>31;A=b<<17;b=A;l=(A|0)<0|0?-1:0;N2(p,(p|0)<0|0?-1:0,b,l)|0;if(((G<<1|0>>>31)>>16-z|0)<=(J|0)){K=1073741824;c[k>>2]=K;c[H>>2]=1;L=c[f>>2]|0;M=L<<17;N=M;O=(M|0)<0|0?-1:0;P=K;Q=(K|0)<0|0?-1:0;R=N2(P,Q,N,O)|0;S=G;T=S;U=0;V=T<<1|0>>>31;W=U<<1|T>>>31;X=V;Y=X;Z=Y>>16;c[g>>2]=Z;_=c[f>>2]|0;c[I>>2]=_;c[n>>2]=-1;i=j;return}N2(r,y,1395864320,0)|0;p=G<<1|0>>>31;N2(p,(p|0)<0|0?-1:0,b,l)|0;p=17-z|0;if(((G<<1|0>>>31)>>p|0)>=(J|0)){K=1073741824;c[k>>2]=K;c[H>>2]=1;L=c[f>>2]|0;M=L<<17;N=M;O=(M|0)<0|0?-1:0;P=K;Q=(K|0)<0|0?-1:0;R=N2(P,Q,N,O)|0;S=G;T=S;U=0;V=T<<1|0>>>31;W=U<<1|T>>>31;X=V;Y=X;Z=Y>>16;c[g>>2]=Z;_=c[f>>2]|0;c[I>>2]=_;c[n>>2]=-1;i=j;return}z=c[k>>2]|0;c[q>>2]=0;N2(b,l,r,y)|0;y=(G<<1|0>>>31)>>p;p=HQ(J,y,q)|0;r=p;l=(p|0)<0|0?-1:0;if((y|0)<(J|0)){J=966367616;y=0;N2(r,l,J,y)|0;y=G<<1|0>>>31;J=c[q>>2]|0;if((J|0)>0){ab=y<<J}else{ab=y>>-J}J=(ab|0)<1234803072?ab:1234803072;ac=(J|0)>1073741824?J:1073741824}else{J=1181116032;ab=0;N2(r,l,J,ab)|0;ab=G<<1|0>>>31;J=c[q>>2]|0;q=-J|0;l=(J|0)<0?1073741824<<q:1073741824>>>(J>>>0);r=(ab|0)<(l|0)?ab:l;l=(J|0)>0?r<<J:r>>q;ac=(l|0)>912680576?l:912680576}l=(z|0)<1073741824;do{if((ac|0)>1073741824&l){ad=1073741824;ae=14137}else{q=(z|0)>1073741824;if((ac|0)<1073741824&q){ad=1073741824;ae=14137;break}if(!(l&(ac|0)<(z|0)|q&(ac|0)>(z|0))){ad=z;ae=14137;break}q=z;r=(z|0)<0|0?-1:0;J=1825361152;ab=0;N2(q,r,J,ab)|0;ab=G<<1|0>>>31;J=ac;r=(ac|0)<0|0?-1:0;q=322122560;y=0;N2(J,r,q,y)|0;af=(G<<1|0>>>31)+ab|0}}while(0);if((ae|0)==14137){ae=ad;z=(ad|0)<0|0?-1:0;ad=1503238528;l=0;N2(ae,z,ad,l)|0;l=G<<1|0>>>31;ad=ac;z=(ac|0)<0|0?-1:0;ac=644245120;ae=0;N2(ad,z,ac,ae)|0;af=l+(G<<1|0>>>31)|0}l=(af|0)<1234803072?af:1234803072;K=(l|0)>912680550?l:912680550;c[k>>2]=K;c[H>>2]=1;L=c[f>>2]|0;M=L<<17;N=M;O=(M|0)<0|0?-1:0;P=K;Q=(K|0)<0|0?-1:0;R=N2(P,Q,N,O)|0;S=G;T=S;U=0;V=T<<1|0>>>31;W=U<<1|T>>>31;X=V;Y=X;Z=Y>>16;c[g>>2]=Z;_=c[f>>2]|0;c[I>>2]=_;c[n>>2]=-1;i=j;return}function MO(b,e,f,g,h,j){b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0,bs=0,bt=0,bu=0,bv=0,bw=0,bx=0,by=0,bz=0,bA=0,bB=0,bC=0,bD=0,bE=0,bF=0,bG=0,bH=0,bI=0,bJ=0,bK=0,bL=0,bM=0,bN=0,bO=0,bP=0,bQ=0,bR=0,bS=0,bT=0,bU=0,bV=0,bW=0,bX=0,bY=0,bZ=0,b_=0,b$=0,b0=0,b1=0,b2=0,b3=0,b4=0,b5=0,b6=0,b7=0,b8=0,b9=0,ca=0,cb=0,cc=0,cd=0,ce=0,cf=0,cg=0,ch=0;f=i;i=i+4272|0;k=f|0;l=f+48|0;m=f+528|0;n=f+536|0;o=f+544|0;p=f+552|0;q=f+568|0;r=f+576|0;s=f+584|0;t=f+592|0;u=f+616|0;v=f+624|0;w=f+1344|0;x=f+4224|0;y=f+4232|0;z=f+4240|0;A=f+4248|0;B=f+4256|0;C=f+4264|0;D=j+12|0;E=c[D>>2]|0;F=(E|0)>0;if((h|0)==0){if(F){H=0;I=E}else{i=f;return}while(1){h=c[j+16+(H*24|0)>>2]|0;if((h|0)==0|(h|0)==1|(h|0)==3){h=c[g+(H<<2)>>2]|0;MI((c[e+(H<<2)>>2]|0)+2e3|0,h|0,c[b+(H<<2)>>2]|0,h+12|0,0,c[j+16+(H*24|0)+8>>2]|0);J=c[D>>2]|0}else{J=I}h=H+1|0;if((h|0)<(J|0)){H=h;I=J}else{K=J;break}}}else{if(!F){i=f;return}F=v|0;J=m;I=n;H=n;h=I+4|0;L=m;M=J+4|0;N=p|0;O=p+4|0;P=p+8|0;Q=p+12|0;R=0;S=E;while(1){E=j+16+(R*24|0)|0;T=c[E>>2]|0;L19818:do{if((T|0)==0|(T|0)==1|(T|0)==3){U=e+(R<<2)|0;V=c[U>>2]|0;if((c[V+32>>2]|0)>=(c[V+1984>>2]|0)){break}W=c[V+36>>2]|0;c[x>>2]=0;V=R+1|0;X=(V|0)>(S|0)?S:V;V=(X|0)>(R|0);if(V){Y=R;Z=0;_=0;$=0;aa=T;while(1){if((aa|0)==4){ab=$;ac=_;ad=Z}else{ae=c[j+16+(Y*24|0)+8>>2]|0;af=e+(Y<<2)|0;ag=c[af>>2]|0;ah=g+(Y<<2)|0;ai=c[ah>>2]|0;if((ae|0)>0){aj=0;do{ak=ai+(aj<<2)|0;al=c[ak>>2]|0;if((c[al>>2]|0)>0){am=0;an=al;while(1){if((c[an+8>>2]|0)>0){al=0;ao=an;while(1){ap=al+am|0;c[w+(Y*480|0)+(aj*240|0)+(ap<<2)>>2]=HH(c[(c[ao+2108>>2]|0)+(ap<<2)>>2]>>2)|0;ap=al+1|0;aq=c[ak>>2]|0;if((ap|0)<(c[aq+8>>2]|0)){al=ap;ao=aq}else{ar=aq;break}}}else{ar=an}ao=(c[ar+4>>2]|0)+am|0;if((ao|0)<(c[ar>>2]|0)){am=ao;an=ar}else{break}}}aj=aj+1|0;}while((aj|0)<(ae|0));as=c[af>>2]|0;at=c[ah>>2]|0}else{as=ag;at=ai}aj=b+(Y<<2)|0;MJ(as+2e3|0,at|0,(c[aj>>2]|0)+32|0,ae);an=c[ah>>2]|0;MK((c[af>>2]|0)+2e3|0,an|0,v+(Y*120|0)|0,an+12|0,ae,(c[aj>>2]|0)+20|0);aj=c[ag+1992>>2]|0;ab=(c[ag+1988>>2]|0)+$|0;ac=(c[ag+1984>>2]|0)+_|0;ad=((aj|0)>1?aj:1)+Z|0}aj=Y+1|0;if((aj|0)>=(X|0)){break}Y=aj;Z=ad;_=ac;$=ab;aa=c[j+16+(aj*24|0)>>2]|0}aa=ab-W|0;$=ad<<2;_=(aa|0)>-1;Z=ab-ac|0;Y=(Z|0)>-1;aj=-Z|0;an=-aa|0;am=R;ak=0;while(1){if((c[j+16+(am*24|0)>>2]|0)==4){au=ak}else{ao=c[j+16+(am*24|0)+8>>2]|0;al=e+(am<<2)|0;aq=c[al>>2]|0;c[y>>2]=0;c[z>>2]=0;if(_){av=HQ(aa,$,y)|0}else{av=-(HQ(an,$,y)|0)|0}ap=HI(av,c[y>>2]|0,y)|0;c[y>>2]=31-(c[y>>2]|0);if(Y){aw=HQ(Z,$,z)|0}else{aw=-(HQ(aj,$,z)|0)|0}ax=HI(aw,c[z>>2]|0,z)|0;ay=31-(c[z>>2]|0)|0;c[z>>2]=ay;az=c[y>>2]|0;aA=((az|0)<(ay|0)?az:ay)-1|0;aB=aA-az|0;if((aB|0)>0){aC=ap<<aB}else{aC=ap>>-aB}aB=aA-ay|0;if((aB|0)>0){aD=ax<<aB}else{aD=ax>>-aB}aB=aC-aD|0;c[t+(am<<2)>>2]=aB;ax=aA&255;a[u+am|0]=ax;aA=g+(am<<2)|0;MQ((c[al>>2]|0)+2e3|0,c[aA>>2]|0,v+(am*120|0)|0,w+(am*480|0)|0,ao,aB,ax);ax=c[aA>>2]|0;aA=c[al>>2]|0;al=c[aq+1996>>2]|0;aB=aq+1984|0;c[aB>>2]=al;ay=aq+1988|0;c[ay>>2]=0;ap=aq+1992|0;c[ap>>2]=0;if((ao|0)>0){az=0;while(1){aE=c[aA+2e3+(az<<2)>>2]|0;aF=c[ax+(az<<2)>>2]|0;Nh(aq+40+(az*972|0)|0,aE+8832|0,aE+7872|0,c[aF>>2]|0,c[aF+4>>2]|0,c[aF+8>>2]|0,aF+1616|0,aF+1856|0);aF=(c[aB>>2]|0)+(c[aq+40+(az*972|0)+960>>2]|0)|0;c[aB>>2]=aF;c[ay>>2]=(c[ay>>2]|0)+(c[aq+40+(az*972|0)+964>>2]|0);c[ap>>2]=(c[ap>>2]|0)+(c[aq+40+(az*972|0)+968>>2]|0);aE=az+1|0;if((aE|0)<(ao|0)){az=aE}else{aG=aF;break}}}else{aG=al}az=aG+ak|0;c[x>>2]=az;au=az}az=am+1|0;if((az|0)<(X|0)){am=az;ak=au}else{aH=au;break}}}else{aH=0}ak=aH-W|0;am=(ak|0)>0?ak:-ak|0;ak=W<<16;$=ak;aj=(ak|0)<0|0?-1:0;ak=107374184;Z=0;N2($,aj,ak,Z)|0;do{if((am|0)>(G+16384>>15|0)){if(V){Z=0;ak=0;Y=0;an=R;while(1){if((c[j+16+(an*24|0)>>2]|0)==4){aI=Y;aJ=ak;aK=Z}else{aa=c[j+16+(an*24|0)+8>>2]|0;_=c[e+(an<<2)>>2]|0;az=c[g+(an<<2)>>2]|0;ao=c[_+1996>>2]|0;if((aa|0)>0){aq=ao;ap=0;ay=0;aB=0;while(1){ax=c[az+(aB<<2)>>2]|0;aA=c[ax>>2]|0;if((aA|0)>0){ag=c[ax+8>>2]|0;ae=(ag|0)>0;af=c[ax+4>>2]|0;ax=0;ah=aq;ai=ap;aF=ay;while(1){if(ae){aE=0;aL=ah;aM=ai;aN=aF;while(1){aO=aE+ax|0;if((d[v+(an*120|0)+(aB*60|0)+aO|0]|0)<2){aP=(c[_+40+(aB*972|0)+720+(aO<<2)>>2]|0)+aN|0;aQ=(c[_+40+(aB*972|0)+480+(aO<<2)>>2]|0)+aM|0;aR=(c[_+40+(aB*972|0)+240+(aO<<2)>>2]|0)+aL|0}else{aP=aN;aQ=aM;aR=aL}aO=aE+1|0;if((aO|0)<(ag|0)){aE=aO;aL=aR;aM=aQ;aN=aP}else{aS=aR;aT=aQ;aU=aP;break}}}else{aS=ah;aT=ai;aU=aF}aN=ax+af|0;if((aN|0)<(aA|0)){ax=aN;ah=aS;ai=aT;aF=aU}else{aV=aS;aW=aT;aX=aU;break}}}else{aV=aq;aW=ap;aX=ay}aF=aB+1|0;if((aF|0)<(aa|0)){aq=aV;ap=aW;ay=aX;aB=aF}else{aY=aV;aZ=aW;a_=aX;break}}}else{aY=ao;aZ=0;a_=0}aI=(aY>>16)+Y|0;aJ=(aZ>>16)+ak|0;aK=a_+Z|0}aB=an+1|0;if((aB|0)<(X|0)){Z=aK;ak=aJ;Y=aI;an=aB}else{a$=aK;a0=aJ;a1=aI;break}}}else{a$=0;a0=0;a1=0}do{if((aH|0)>(W|0)){an=a1-aH+W|0;if((a$|0)<=0){break}if(!V){a2=14233;break}Y=a0-((an|0)<0?0:an)|0;an=a$<<2;ak=(Y|0)>-1;Z=a0-a1|0;al=(Z|0)>-1;aB=-Z|0;ay=-Y|0;ap=R;do{if((c[j+16+(ap*24|0)>>2]|0)!=4){c[A>>2]=0;c[B>>2]=0;if(ak){a3=HQ(Y,an,A)|0}else{a3=-(HQ(ay,an,A)|0)|0}aq=HI(a3,c[A>>2]|0,A)|0;c[A>>2]=31-(c[A>>2]|0);if(al){a4=HQ(Z,an,B)|0}else{a4=-(HQ(aB,an,B)|0)|0}aa=HI(a4,c[B>>2]|0,B)|0;_=31-(c[B>>2]|0)|0;c[B>>2]=_;az=c[A>>2]|0;aF=((az|0)<(_|0)?az:_)-1|0;ai=aF-az|0;if((ai|0)>0){a5=aq<<ai}else{a5=aq>>-ai}ai=aF-_|0;if((ai|0)>0){a6=aa<<ai}else{a6=aa>>-ai}ai=a5-a6|0;c[A>>2]=aF;aa=t+(ap<<2)|0;_=c[aa>>2]|0;aq=u+ap|0;az=a[aq]|0;c[B>>2]=az;ah=((aF|0)<(az|0)?aF:az)-1|0;ax=ah-aF|0;if((ax|0)>0){a7=ai<<ax}else{a7=ai>>-ax}ax=ah-az|0;if((ax|0)>0){a8=_<<ax}else{a8=_>>-ax}c[aa>>2]=a8+a7;a[aq]=ah&255}ap=ap+1|0;}while((ap|0)<(X|0))}else{if(V){a9=R}else{a2=14233;break}do{do{if((c[j+16+(a9*24|0)>>2]|0)!=4){c[C>>2]=0;ap=HQ(aH,W,C)|0;an=t+(a9<<2)|0;aB=c[an>>2]|0;Z=aB;al=(aB|0)<0|0?-1:0;aB=ap;ay=(ap|0)<0|0?-1:0;N2(Z,al,aB,ay)|0;c[an>>2]=G<<1|0>>>31;an=u+a9|0;a[an]=(d[an]|0)-(c[C>>2]|0)&255;an=c[j+16+(a9*24|0)+8>>2]|0;ay=c[g+(a9<<2)>>2]|0;if((an|0)>0){ba=0}else{break}do{aB=c[ay+(ba<<2)>>2]|0;al=c[aB>>2]|0;if((al|0)>0){Z=c[aB+8>>2]|0;ap=(Z|0)>0;Y=c[aB+4>>2]|0;aB=0;do{if(ap){ak=0;do{ao=ak+aB+(v+(a9*120|0)+(ba*60|0))|0;if((a[ao]|0)==2){a[ao]=1}ak=ak+1|0;}while((ak|0)<(Z|0))}aB=Y+aB|0;}while((aB|0)<(al|0))}ba=ba+1|0;}while((ba|0)<(an|0))}}while(0);a9=a9+1|0;}while((a9|0)<(X|0))}}while(0);if((a2|0)==14233){a2=0;c[x>>2]=0;bb=0;break}c[x>>2]=0;if(V){bc=R;bd=0}else{bb=0;break}while(1){if((c[j+16+(bc*24|0)>>2]|0)==4){be=bd}else{an=c[j+16+(bc*24|0)+8>>2]|0;ay=e+(bc<<2)|0;al=c[ay>>2]|0;aB=g+(bc<<2)|0;MQ(al+2e3|0,c[aB>>2]|0,v+(bc*120|0)|0,w+(bc*480|0)|0,an,c[t+(bc<<2)>>2]|0,a[u+bc|0]|0);Y=c[aB>>2]|0;aB=c[ay>>2]|0;ay=c[al+1996>>2]|0;Z=al+1984|0;c[Z>>2]=ay;ap=al+1988|0;c[ap>>2]=0;ak=al+1992|0;c[ak>>2]=0;if((an|0)>0){ao=0;while(1){ah=c[aB+2e3+(ao<<2)>>2]|0;aq=c[Y+(ao<<2)>>2]|0;Nh(al+40+(ao*972|0)|0,ah+8832|0,ah+7872|0,c[aq>>2]|0,c[aq+4>>2]|0,c[aq+8>>2]|0,aq+1616|0,aq+1856|0);aq=(c[Z>>2]|0)+(c[al+40+(ao*972|0)+960>>2]|0)|0;c[Z>>2]=aq;c[ap>>2]=(c[ap>>2]|0)+(c[al+40+(ao*972|0)+964>>2]|0);c[ak>>2]=(c[ak>>2]|0)+(c[al+40+(ao*972|0)+968>>2]|0);ah=ao+1|0;if((ah|0)<(an|0)){ao=ah}else{bf=aq;break}}}else{bf=ay}ao=bf+bd|0;c[x>>2]=ao;be=ao}ao=bc+1|0;if((ao|0)<(X|0)){bc=ao;bd=be}else{bb=be;break}}}else{bb=aH}}while(0);do{if((bb|0)>(W|0)){am=W-bb|0;ao=j+16+(R*24|0)+8|0;an=c[ao>>2]|0;L19961:do{if((an|0)>0){al=c[U>>2]|0;ak=0;do{c[k+(R<<3)+(ak<<2)>>2]=(c[al+2e3+(ak<<2)>>2]|0)+4096;ak=ak+1|0;}while((ak|0)<(an|0));ak=(am|0)>0;if((c[E>>2]|0)==4){bg=0;break}ay=g+(R<<2)|0;ap=t+(R<<2)|0;Z=u+R|0;Y=0;aB=0;aq=an;ah=al;while(1){aa=c[(c[ay>>2]|0)+(aB<<2)>>2]|0;ax=aa|0;_=c[ax>>2]|0;if((_|0)>0){az=aa+8|0;ai=aa+4|0;aa=k+(R<<3)+(aB<<2)|0;aF=Y;aA=0;af=c[az>>2]|0;ag=_;while(1){if((af|0)>0){_=aF;ae=0;while(1){aN=ae+aA|0;aM=ah+40+(aB*972|0)+720+(aN<<2)|0;aL=c[aM>>2]|0;if((aL|0)==0){c[l+(aB*240|0)+(aN<<2)>>2]=-2147483648;bh=-2147483648}else{aE=HK(aL)|0;c[l+(aB*240|0)+(aN<<2)>>2]=aE;bh=aE}do{if((d[v+(R*120|0)+(aB*60|0)+aN|0]|0)<2|ak){if((c[aM>>2]|0)==0){a2=14276;break}aE=c[w+(R*480|0)+(aB*240|0)+(aN<<2)>>2]|0;aL=c[ap>>2]|0;if((aE|0)<=(-aL|0)){c[(c[aa>>2]|0)+(aN<<2)>>2]=2147483647;bi=_;break}do{if((aE|0)==0){bj=0}else{aO=aE>>31^aE;if((aO|0)<=-1){bj=-1;break}bk=~aO;aO=0;while(1){bl=bk<<1;if((bl|0)<0){bk=bl;aO=aO+1|0}else{bj=aO;break}}}}while(0);do{if((aL|0)==0){bm=0}else{aO=aL>>31^aL;if((aO|0)<=-1){bm=-1;break}bk=~aO;aO=0;while(1){bl=bk<<1;if((bl|0)<0){bk=bl;aO=aO+1|0}else{bm=aO;break}}}}while(0);aO=a[Z]|0;bk=bm-31+aO|0;bl=(bj|0)<(bk|0)?bj:bk;bk=bl-1|0;if((bk|0)>0){bn=aE<<bk}else{bn=aE>>1-bl}bl=bk+(31-aO)|0;if((bl|0)>0){bo=aL<<bl}else{bo=aL>>-bl}bl=(HE(bo+bn|0)|0)-(bk<<25)|0;do{if((bl|0)<0){bk=bh-bl|0;c[(c[aa>>2]|0)+(aN<<2)>>2]=bk;bp=bk}else{if((bh|0)>(bl^-2147483648|0)){bk=bh-bl|0;c[(c[aa>>2]|0)+(aN<<2)>>2]=bk;bp=bk;break}else{c[(c[aa>>2]|0)+(aN<<2)>>2]=bh;bp=bh;break}}}while(0);bi=(HH(bp)|0)+_|0}else{a2=14276}}while(0);if((a2|0)==14276){a2=0;c[(c[aa>>2]|0)+(aN<<2)>>2]=-2147483648;bi=_}aM=ae+1|0;bq=c[az>>2]|0;if((aM|0)<(bq|0)){_=bi;ae=aM}else{break}}br=bi;bs=bq;bt=c[ax>>2]|0}else{br=aF;bs=af;bt=ag}ae=(c[ai>>2]|0)+aA|0;if((ae|0)<(bt|0)){aF=br;aA=ae;af=bs;ag=bt}else{break}}bu=br;bv=c[ao>>2]|0}else{bu=Y;bv=aq}ag=aB+1|0;if((ag|0)>=(bv|0)){bg=bu;break L19961}Y=bu;aB=ag;aq=bv;ah=c[U>>2]|0}}else{bg=0}}while(0);an=(am|0)<0;ah=HE(an?-am|0:am)|0;aq=(bb|0)==(W|0);aB=ah-201326592-(HE(bg)|0)|0;do{if((c[E>>2]|0)!=4){ah=c[ao>>2]|0;if((ah|0)<=0){break}Y=g+(R<<2)|0;Z=0;ap=ah;while(1){ah=c[U>>2]|0;ak=c[ah+2e3+(Z<<2)>>2]|0;ay=c[(c[Y>>2]|0)+(Z<<2)>>2]|0;al=ay|0;ag=c[al>>2]|0;if((ag|0)>0){af=ay+8|0;aA=ay+4|0;ay=k+(R<<3)+(Z<<2)|0;aF=0;ai=c[af>>2]|0;ax=ag;while(1){if((ai|0)>0){ag=0;az=ai;while(1){aa=ag+aF|0;if((c[ah+40+(Z*972|0)+720+(aa<<2)>>2]|0)>0){ae=c[(c[ay>>2]|0)+(aa<<2)>>2]|0;do{if((ae|0)==-2147483648|aq){_=ak+7872+(aa<<2)|0;bw=0;bx=_;by=c[_>>2]|0;bz=c[ak+8832+(aa<<2)>>2]|0;a2=14294}else{_=HH(aB+ae-(c[l+(Z*240|0)+(aa<<2)>>2]|0)|0)|0;aM=an?_:-_|0;_=(aM|0)<671088640?aM:671088640;aM=ak+7872+(aa<<2)|0;bl=c[aM>>2]|0;aL=c[ak+8832+(aa<<2)>>2]|0;if((_|0)>=0){bw=_;bx=aM;by=bl;bz=aL;a2=14294;break}bA=(bl|0)>(-2147483648-_|0)?_+bl|0:-2147483648;bB=aM;bC=bl;bD=aL}}while(0);if((a2|0)==14294){a2=0;bA=bw+by|0;bB=bx;bC=by;bD=bz}ae=c[ak+8112+(aa<<2)>>2]|0;do{if((bA-bD|0)>(ae|0)){aN=v+(R*120|0)+(Z*60|0)+aa|0;if((a[aN]|0)!=1){bE=bA;break}a[aN]=2;bE=(bD|0)>(bC-ae|0)?ae+bD|0:bC}else{bE=bA}}while(0);c[bB>>2]=bE;bF=c[af>>2]|0}else{bF=az}ae=ag+1|0;if((ae|0)<(bF|0)){ag=ae;az=bF}else{break}}bG=bF;bH=c[al>>2]|0}else{bG=ai;bH=ax}az=(c[aA>>2]|0)+aF|0;if((az|0)<(bH|0)){aF=az;ai=bG;ax=bH}else{break}}bI=c[ao>>2]|0}else{bI=ap}ax=Z+1|0;if((ax|0)<(bI|0)){Z=ax;ap=bI}else{break}}}}while(0);c[x>>2]=0;if(V){bJ=R;bK=0}else{bL=0;break}while(1){if((c[j+16+(bJ*24|0)>>2]|0)==4){bM=bK}else{ao=c[j+16+(bJ*24|0)+8>>2]|0;an=c[e+(bJ<<2)>>2]|0;aB=c[g+(bJ<<2)>>2]|0;aq=c[an+1996>>2]|0;am=an+1984|0;c[am>>2]=aq;ap=an+1988|0;c[ap>>2]=0;Z=an+1992|0;c[Z>>2]=0;if((ao|0)>0){Y=0;while(1){ax=c[an+2e3+(Y<<2)>>2]|0;ai=c[aB+(Y<<2)>>2]|0;Nh(an+40+(Y*972|0)|0,ax+8832|0,ax+7872|0,c[ai>>2]|0,c[ai+4>>2]|0,c[ai+8>>2]|0,ai+1616|0,ai+1856|0);ai=(c[am>>2]|0)+(c[an+40+(Y*972|0)+960>>2]|0)|0;c[am>>2]=ai;c[ap>>2]=(c[ap>>2]|0)+(c[an+40+(Y*972|0)+964>>2]|0);c[Z>>2]=(c[Z>>2]|0)+(c[an+40+(Y*972|0)+968>>2]|0);ax=Y+1|0;if((ax|0)<(ao|0)){Y=ax}else{bN=ai;break}}}else{bN=aq}Y=bN+bK|0;c[x>>2]=Y;bM=Y}Y=bJ+1|0;if((Y|0)<(X|0)){bJ=Y;bK=bM}else{bL=bM;break}}}else{bL=bb}}while(0);if((bL|0)<=(W|0)){break}X=322122560;V=0;N2($,aj,X,V)|0;MH(j,e,g,F,(G+16384>>15)+W|0,x,1,R);V=c[x>>2]|0;if((V|0)<=(W|0)){break}if((c[E>>2]|0)==4){break}X=c[U>>2]|0;Y=j+16+(R*24|0)+8|0;ao=c[Y>>2]|0;c[m>>2]=0;c[m+4>>2]=0;c[n>>2]=0;c[n+4>>2]=0;an=g+(R<<2)|0;do{if((ao|0)>0){Z=0;ap=X;while(1){c[J+(Z<<2)>>2]=c[ap+2e3+(Z<<2)>>2];am=c[(c[an>>2]|0)+(Z<<2)>>2]|0;c[I+(Z<<2)>>2]=am;aB=am|0;ai=c[aB>>2]|0;if((ai|0)>0){ax=am+4|0;aF=am+8|0;am=0;aA=ai;ai=c[ax>>2]|0;while(1){al=c[aF>>2]|0;if((al|0)<(ai|0)){af=al;do{c[X+40+(Z*972|0)+240+(af+am<<2)>>2]=0;af=af+1|0;bO=c[ax>>2]|0;}while((af|0)<(bO|0));bP=bO;bQ=c[aB>>2]|0}else{bP=ai;bQ=aA}af=bP+am|0;if((af|0)<(bQ|0)){am=af;aA=bQ;ai=bP}else{break}}}ai=Z+1|0;if((ai|0)>=(ao|0)){break}Z=ai;ap=c[U>>2]|0}L20081:do{if((ao|0)==2){ap=c[H>>2]|0;if((c[ap+12>>2]|0)!=(c[(c[h>>2]|0)+12>>2]|0)){bR=V;break}Z=ap+8|0;ai=ap|0;aA=ap+4|0;ap=V;am=0;while(1){if((am|0)>=(c[Z>>2]|0)){bR=ap;break L20081}if((c[ai>>2]|0)>0){aB=0;ax=ap;while(1){aF=aB+am|0;L20090:do{if((c[(c[an>>2]|0)+16+(aF<<2)>>2]|0)==0){bS=ax}else{aq=c[L>>2]|0;af=c[aq+8832+(aF<<2)>>2]|0;al=c[M>>2]|0;ak=c[al+8832+(aF<<2)>>2]|0;ay=v+(R*120|0)+60+aF|0;do{if((a[ay]|0)!=0){if(((c[aq+8112+(aF<<2)>>2]>>1)-22178274|0)<=((ak>>1)-(af>>1)|0)){break}a[ay]=0;c[al+7872+(aF<<2)>>2]=ak+33554432;bS=ax-(c[X+1252+(aF<<2)>>2]>>16)|0;break L20090}}while(0);ay=v+(R*120|0)+aF|0;if((a[ay]|0)==0){bS=ax;break}if(((c[al+8112+(aF<<2)>>2]>>1)-22178274|0)<=((af>>1)-(ak>>1)|0)){bS=ax;break}a[ay]=0;c[aq+7872+(aF<<2)>>2]=af+33554432;bS=ax-(c[X+280+(aF<<2)>>2]>>16)|0}}while(0);aF=(c[aA>>2]|0)+aB|0;if((aF|0)<(c[ai>>2]|0)){aB=aF;ax=bS}else{bT=bS;break}}}else{bT=ap}if((bT|0)<=(W|0)){break L19818}ap=bT;am=am+1|0}}else{bR=V}}while(0);am=c[b+(R<<2)>>2]|0;ap=am+28|0;ai=am+24|0;am=0;while(1){if((c[(c[I+(am<<2)>>2]|0)+12>>2]|0)==2){c[o+(am<<2)>>2]=c[ap>>2]}else{c[o+(am<<2)>>2]=c[ai>>2]}aA=am+1|0;if((aA|0)<(ao|0)){am=aA}else{bU=0;bV=0;bW=0;bX=0;break}}while(1){am=c[I+(bU<<2)>>2]|0;ai=c[am+8>>2]|0;ap=J+(bU<<2)|0;aA=c[am+4>>2]|0;Z=c[am>>2]|0;am=bX;ax=bW;aB=bV;aF=c[o+(bU<<2)>>2]|0;ay=0;while(1){if((aF|0)<(ai|0)){ah=aF;az=aB;ag=ax;ae=am;while(1){aa=ah+ay|0;do{if((a[v+(R*120|0)+(bU*60|0)+aa|0]|0)==0){bY=ae;bZ=ag;b_=az}else{aN=c[ap>>2]|0;if((c[aN+8832+(aa<<2)>>2]|0)<=(c[aN+7872+(aa<<2)>>2]|0)){bY=ae;bZ=ag;b_=az;break}aL=c[aN+8352+(aa<<2)>>2]|0;bY=(c[aN+8592+(aa<<2)>>2]>>6)+ae|0;bZ=ag+1|0;b_=(az|0)<(aL|0)?az:aL}}while(0);aa=ah+1|0;if((aa|0)<(ai|0)){ah=aa;az=b_;ag=bZ;ae=bY}else{b$=b_;b0=bZ;b1=bY;break}}}else{b$=aB;b0=ax;b1=am}ae=ay+aA|0;if((ae|0)<(Z|0)){am=b1;ax=b0;aB=b$;aF=0;ay=ae}else{break}}ay=bU+1|0;if((ay|0)<(ao|0)){bU=ay;bV=b$;bW=b0;bX=b1}else{break}}if((b1|0)==0|(b0|0)==0){b2=0;b3=bR;b4=b$;break}ay=HE(b1)|0;b2=ay+201326592-(HK(b0)|0)|0;b3=bR;b4=b$}else{b2=0;b3=V;b4=0}}while(0);V=b2-b4|0;ao=V;U=(V|0)<0|0?-1:0;V=306783392;aj=0;N2(ao,U,V,aj)|0;c[N>>2]=(G<<1|0>>>31)+b4;aj=920350144;V=0;N2(ao,U,aj,V)|0;c[O>>2]=(G<<1|0>>>31)+b4;V=1533916928;aj=0;N2(ao,U,V,aj)|0;c[P>>2]=(G<<1|0>>>31)+b4;c[Q>>2]=b2;aj=0;V=b3;while(1){if((aj|0)>=4){break L19818}U=c[Y>>2]|0;if((U|0)>0){ao=c[an>>2]|0;$=0;do{ay=c[ao+($<<2)>>2]|0;c[q+($<<2)>>2]=(c[ay+8>>2]|0)-1;c[r+($<<2)>>2]=c[ay>>2];c[s+($<<2)>>2]=c[ay+4>>2];$=$+1|0;}while(($|0)<(U|0))}$=p+(aj<<2)|0;ao=1;ay=V;aF=0;aB=U;L20132:while(1){b5=(ay|0)>(W|0);ax=ao;am=aF;while(1){if((am|0)<(aB|0)){break}if((ax|0)==0&b5){ax=1;am=0}else{break L20132}}Z=q+(am<<2)|0;aA=c[Z>>2]|0;if((aA|0)<(c[o+(am<<2)>>2]|0)){b6=ax;b7=ay;b8=aB}else{c[Z>>2]=aA-1;Z=c[r+(am<<2)>>2]|0;if((Z|0)>0){ai=c[s+(am<<2)>>2]|0;ap=J+(am<<2)|0;ae=ay;ag=0;while(1){az=ag+aA|0;ah=v+(R*120|0)+(am*60|0)+az|0;do{if((a[ah]|0)==0){b9=ae}else{aa=c[ap>>2]|0;if((c[aa+8352+(az<<2)>>2]|0)>=(c[$>>2]|0)){b9=ae;break}a[ah]=0;c[aa+7872+(az<<2)>>2]=(c[aa+8832+(az<<2)>>2]|0)+33554432;b9=ae-(c[X+40+(am*972|0)+240+(az<<2)>>2]>>16)|0}}while(0);az=ag+ai|0;if((az|0)<(Z|0)){ae=b9;ag=az}else{ca=b9;break}}}else{ca=ay}if((ca|0)<=(W|0)){break L19818}b6=0;b7=ca;b8=c[Y>>2]|0}ao=b6;ay=b7;aF=am+1|0;aB=b8}if(b5){aj=aj+1|0;V=ay}else{break}}}}while(0);E=R+1|0;T=c[D>>2]|0;if((E|0)<(T|0)){R=E;S=T}else{K=T;break}}}S=j+12|0;if((K|0)>0){cb=0;cc=K}else{i=f;return}while(1){K=j+16+(cb*24|0)+8|0;R=c[K>>2]|0;if((R|0)>0){D=e+(cb<<2)|0;b5=g+(cb<<2)|0;b8=0;b7=R;R=c[b5>>2]|0;while(1){b6=c[(c[D>>2]|0)+2e3+(b8<<2)>>2]|0;ca=c[R+(b8<<2)>>2]|0;if((c[ca>>2]|0)>0){b9=0;v=R;J=ca;while(1){if((c[J+8>>2]|0)>0){ca=0;while(1){s=ca+b9|0;r=b6+7872+(s<<2)|0;c[r>>2]=(c[r>>2]|0)+(c[b6+9072+(s<<2)>>2]|0);s=ca+1|0;r=c[b5>>2]|0;o=c[r+(b8<<2)>>2]|0;if((s|0)<(c[o+8>>2]|0)){ca=s}else{cd=r;ce=o;break}}}else{cd=v;ce=J}ca=(c[ce+4>>2]|0)+b9|0;if((ca|0)<(c[ce>>2]|0)){b9=ca;v=cd;J=ce}else{break}}cf=c[K>>2]|0;cg=cd}else{cf=b7;cg=R}J=b8+1|0;if((J|0)<(cf|0)){b8=J;b7=cf;R=cg}else{break}}ch=c[S>>2]|0}else{ch=cc}R=cb+1|0;if((R|0)<(ch|0)){cb=R;cc=ch}else{break}}i=f;return}function MP(a){a=a|0;var b=0,d=0;b=c[a>>2]|0;if((b|0)==0){return}d=b+64|0;if((c[d>>2]|0)!=0){J5(d)}d=b+68|0;if((c[d>>2]|0)!=0){J5(d)}d=b+72|0;if((c[d>>2]|0)!=0){J5(d)}d=b+76|0;if((c[d>>2]|0)!=0){J5(d)}d=b+80|0;if((c[d>>2]|0)!=0){J5(d)}d=b+84|0;if((c[d>>2]|0)!=0){J5(d)}J3(a);return}function MQ(b,d,e,f,g,h,i){b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0;if((g|0)<=0){return}j=(h|0)==0;k=31-(i<<24>>24)|0;i=h>>31^h;l=~i;m=0;do{n=c[b+(m<<2)>>2]|0;o=d+(m<<2)|0;p=c[o>>2]|0;if((c[p>>2]|0)>0){q=0;r=p;while(1){if((c[r+8>>2]|0)>0){p=0;s=r;while(1){t=p+q|0;u=c[n+8832+(t<<2)>>2]|0;v=n+7872+(t<<2)|0;w=c[v>>2]|0;x=c[f+(m*240|0)+(t<<2)>>2]|0;do{if((u|0)>(w|0)){y=e+(m*60|0)+t|0;if((a[y]|0)==2){z=s;break}do{if((x|0)==0){A=0}else{B=x>>31^x;if((B|0)<=-1){A=-1;break}C=~B;B=0;while(1){D=C<<1;if((D|0)<0){C=D;B=B+1|0}else{A=B;break}}}}while(0);if(j|(i|0)<0){E=(j^1)<<31>>31}else{B=l;C=0;while(1){D=B<<1;if((D|0)<0){B=D;C=C+1|0}else{E=C;break}}}C=E-k|0;B=(A|0)<(C|0)?A:C;C=B-1|0;if((C|0)>0){F=x<<C}else{F=x>>1-B}B=C+k|0;if((B|0)>0){G=h<<B}else{G=h>>-B}B=G+F|0;D=(HE((B|0)>0?B:-B|0)|0)-(C<<25)<<2;C=c[n+8112+(t<<2)>>2]|0;do{if((D-u|0)>(C|0)){if((a[y]|0)==0){H=D;break}if((C|0)>(-2147483648-u|0)){B=C+u|0;I=(B|0)>(w|0)?B:w}else{I=w}a[y]=2;H=I}else{H=D}}while(0);if((u+2147483647|0)>323250667){D=u-323250667|0;J=(H|0)>(D|0)?H:D}else{J=H}c[v>>2]=J;z=c[o>>2]|0}else{z=s}}while(0);v=p+1|0;if((v|0)<(c[z+8>>2]|0)){p=v;s=z}else{K=z;break}}}else{K=r}s=(c[K+4>>2]|0)+q|0;if((s|0)<(c[K>>2]|0)){q=s;r=K}else{break}}}m=m+1|0;}while((m|0)<(g|0));return}function MR(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0;if((e|0)>0){f=0}else{return}while(1){g=c[b+(f<<2)>>2]|0;h=f+1|0;i=c[b+(h<<2)>>2]|0;do{if((g|0)<(i|0)){j=g;k=0;do{l=c[a+(j<<2)>>2]|0;m=(l|0)>0?l:-l|0;k=(k|0)>(m|0)?k:m;j=j+1|0;}while((j|0)<(i|0));if((k|0)==0){n=30;break}if((k|0)<=-1){n=-1;break}j=~k;m=0;while(1){l=j<<1;if((l|0)<0){j=l;m=m+1|0}else{n=m;break}}}else{n=30}}while(0);c[d+(f<<2)>>2]=n;if((h|0)<(e|0)){f=h}else{break}}return}function MS(a,b,d,e,f,g,h){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0;if((e|0)>0){i=0;j=-2147483648;k=0;while(1){l=(c[b+(k<<2)>>2]|0)-4|0;m=(l|0)<0?0:l;l=c[d+(k<<2)>>2]|0;n=k+1|0;o=c[d+(n<<2)>>2]|0;if((l|0)<(o|0)){p=l;l=0;do{q=c[a+(p<<2)>>2]<<m;r=q;s=(q|0)<0|0?-1:0;N2(r,s,r,s)|0;l=G+l|0;p=p+1|0;}while((p|0)<(o|0));t=l<<1}else{t=0}c[f+(k<<2)>>2]=t;o=HE(t)|0;if((o|0)==-2147483648){u=-2147483648}else{u=o-(m<<26)|0}c[g+(k<<2)>>2]=u;o=(u|0)>(j|0);p=o?k:i;if((n|0)<(e|0)){i=p;j=o?u:j;k=n}else{v=p;break}}}else{v=0}k=(c[b+(v<<2)>>2]|0)-4|0;b=h-((k|0)<0?0:k)<<1;k=(b|0)>-31?b:-31;b=c[f+(v<<2)>>2]|0;if((k|0)>0){w=b<<k;return w|0}else{w=b>>-k;return w|0}return 0}function MT(a,b,d,e,f,g){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;if((e|0)>0){h=0;do{i=c[b+(h<<2)>>2]|0;j=i-4|0;do{if((j|0)>-1){k=c[d+(h<<2)>>2]|0;l=c[d+(h+1<<2)>>2]|0;if((k|0)<(l|0)){m=0;n=k}else{o=0;break}while(1){k=c[a+(n<<2)>>2]<<j;p=k;q=(k|0)<0|0?-1:0;N2(p,q,p,q)|0;q=G+m|0;p=n+1|0;if((p|0)<(l|0)){m=q;n=p}else{o=q;break}}}else{l=4-i|0;q=c[d+(h<<2)>>2]|0;p=c[d+(h+1<<2)>>2]|0;if((q|0)<(p|0)){r=0;s=q}else{o=0;break}while(1){q=c[a+(s<<2)>>2]>>l;k=q;t=(q|0)<0|0?-1:0;N2(k,t,k,t)|0;t=G+r|0;k=s+1|0;if((k|0)<(p|0)){r=t;s=k}else{o=t;break}}}}while(0);c[f+(h<<2)>>2]=o<<1;h=h+1|0;}while((h|0)<(e|0))}HG(f,g,e);h=(e|0)==0;if(h){u=0;return u|0}else{v=0;w=e}do{w=w-1|0;o=(c[b+(w<<2)>>2]<<26)-268435456|0;s=g+(w<<2)|0;r=c[s>>2]|0;a=(r|0)<((o>>1)-1073741824|0)?-2147483648:r-o|0;c[s>>2]=a;v=(v|0)>(a|0)?v:a;}while((w|0)!=0);if((v|0)<1){if(h){u=0;return u|0}else{x=e}while(1){w=x-1|0;a=(c[b+(w<<2)>>2]<<1)-8|0;s=(a|0)<31?a:31;a=f+(w<<2)|0;o=c[a>>2]|0;c[a>>2]=(s|0)<0?o<<-s:o>>s;if((w|0)==0){u=0;break}else{x=w}}return u|0}x=(v-1|0)>>>26;v=x+1|0;if(h){u=v;return u|0}h=x+2147483645|0;x=v<<26;w=e;while(1){e=w-1|0;s=h+(c[b+(e<<2)>>2]|0)<<1;o=(s|0)<31?s:31;s=g+(e<<2)|0;c[s>>2]=(c[s>>2]|0)-x;s=f+(e<<2)|0;a=c[s>>2]|0;c[s>>2]=(o|0)<0?a<<-o:a>>o;if((e|0)==0){u=v;break}else{w=e}}return u|0}function MU(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;g=(e|0)>0;if(g){h=0}else{return}while(1){i=c[b+(h<<2)>>2]|0;j=i-3|0;k=c[d+(h<<2)>>2]|0;l=h+1|0;m=c[d+(l<<2)>>2]|0;if((k|0)<(m|0)){n=3-i|0;if((j|0)>0){i=0;o=k;while(1){p=c[a+(o<<2)>>2]<<j;q=p;r=(p|0)<0|0?-1:0;N2(q,r,q,r)|0;r=G+i|0;q=o+1|0;if((q|0)<(m|0)){i=r;o=q}else{s=r;break}}}else{o=0;i=k;while(1){j=c[a+(i<<2)>>2]>>n;r=j;q=(j|0)<0|0?-1:0;N2(r,q,r,q)|0;q=G+o|0;r=i+1|0;if((r|0)<(m|0)){o=q;i=r}else{s=q;break}}}}else{s=0}c[f+(h<<2)>>2]=s;if((l|0)<(e|0)){h=l}else{break}}if(g){t=0}else{return}do{g=(c[b+(t<<2)>>2]<<1)-7|0;h=(g|0)<31?g:31;g=(h|0)>-31?h:-31;h=f+(t<<2)|0;s=c[h>>2]|0;a=-g|0;L20327:do{if((g|0)<0){do{if((s|0)!=0){d=s>>31^s;if((d|0)>-1){i=~d;d=0;while(1){o=i<<1;if((o|0)<0){i=o;d=d+1|0}else{u=d;break}}}else{u=-1}if((u|0)>=(a|0)){break}v=(s|0)>0?2147483647:-2147483648;break L20327}}while(0);v=s<<a}else{v=(a|0)<-31?0:s>>g}}while(0);c[h>>2]=v;t=t+1|0;}while((t|0)<(e|0));return}function MV(a,b,d,e,f,g,h,i,j,k,l){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;m=(g|0)>0;if(m){n=0;do{o=c[d+(n<<2)>>2]|0;p=c[e+(n<<2)>>2]|0;q=((o|0)<(p|0)?o:p)-4|0;p=(q|0)<0?0:q;q=c[f+(n<<2)>>2]|0;o=c[f+(n+1<<2)>>2]|0;r=(q|0)<(o|0);do{if((p|0)>0){if(!r){s=0;t=0;break}u=p-1|0;v=0;w=0;x=q;while(1){y=c[a+(x<<2)>>2]<<u;z=c[b+(x<<2)>>2]<<u;A=z+y|0;B=y-z|0;z=A;y=(A|0)<0|0?-1:0;N2(z,y,z,y)|0;y=G+w|0;z=B;A=(B|0)<0|0?-1:0;N2(z,A,z,A)|0;A=G+v|0;z=x+1|0;if((z|0)<(o|0)){v=A;w=y;x=z}else{s=y;t=A;break}}}else{if(r){C=0;D=0;E=q}else{s=0;t=0;break}while(1){x=c[a+(E<<2)>>2]>>1;w=c[b+(E<<2)>>2]>>1;v=w+x|0;u=x-w|0;w=v;x=(v|0)<0|0?-1:0;N2(w,x,w,x)|0;x=G+D|0;w=u;v=(u|0)<0|0?-1:0;N2(w,v,w,v)|0;v=G+C|0;w=E+1|0;if((w|0)<(o|0)){C=v;D=x;E=w}else{s=x;t=v;break}}}}while(0);c[h+(n<<2)>>2]=s<<1;c[i+(n<<2)>>2]=t<<1;n=n+1|0;}while((n|0)<(g|0))}n=(j|0)!=0;if(n){HG(h,k,g);HG(i,l,g)}if(!m){return}if(n){F=0}else{n=0;do{m=c[d+(n<<2)>>2]|0;j=c[e+(n<<2)>>2]|0;t=(((m|0)<(j|0)?m:j)<<1)-8|0;j=(t|0)<0?0:t;t=(j|0)<31?j:31;j=h+(n<<2)|0;c[j>>2]=c[j>>2]>>t;j=i+(n<<2)|0;c[j>>2]=c[j>>2]>>t;n=n+1|0;}while((n|0)<(g|0));return}do{n=c[d+(F<<2)>>2]|0;t=c[e+(F<<2)>>2]|0;j=(((n|0)<(t|0)?n:t)<<1)-8|0;t=(j|0)<0?0:j;j=t<<25;n=k+(F<<2)|0;m=c[n>>2]|0;if((m|0)!=-2147483648){c[n>>2]=m-j}m=l+(F<<2)|0;n=c[m>>2]|0;if((n|0)!=-2147483648){c[m>>2]=n-j}j=(t|0)<31?t:31;t=h+(F<<2)|0;c[t>>2]=c[t>>2]>>j;t=i+(F<<2)|0;c[t>>2]=c[t>>2]>>j;F=F+1|0;}while((F|0)<(g|0));return}function MW(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;c[e>>2]=(d|0)==0?0:536870911;dm[c[1926160+(((d|0)<16?d:16)<<2)>>2]&127](a,b,e);return 0}function MX(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0;switch(f|0){case 6:{if((e|0)>0){g=0;h=0}else{i=0;return i|0}while(1){f=(c[1981016+(((b[a+(g<<1)>>1]|0)+4|0)*36|0)+((b[a+((g|1)<<1)>>1]|0)+4<<2)>>2]&65535)+h|0;j=g+2|0;if((j|0)<(e|0)){g=j;h=f}else{i=f;break}}return i|0};case 8:{if((e|0)>0){k=0;l=0}else{i=0;return i|0}while(1){h=b[a+(k<<1)>>1]|0;g=h<<16>>16;if(h<<16>>16==0){m=0;n=g}else{m=1;n=h<<16>>16>0?g:-g|0}g=b[a+((k|1)<<1)>>1]|0;h=g<<16>>16;if(g<<16>>16==0){o=m;p=h}else{o=m+1|0;p=g<<16>>16>0?h:-h|0}h=o+l+(c[1980760+(n<<5)+(p<<2)>>2]&65535)|0;g=k+2|0;if((g|0)<(e|0)){k=g;l=h}else{i=h;break}}return i|0};case 9:{if((e|0)>0){q=0;r=0}else{i=0;return i|0}while(1){l=b[a+(q<<1)>>1]|0;k=l<<16>>16;if(l<<16>>16==0){s=0;t=k}else{s=1;t=l<<16>>16>0?k:-k|0}k=b[a+((q|1)<<1)>>1]|0;l=k<<16>>16;if(k<<16>>16==0){u=s;v=l}else{u=s+1|0;v=k<<16>>16>0?l:-l|0}l=u+r+(c[1980080+(t*52|0)+(v<<2)>>2]>>16)|0;k=q+2|0;if((k|0)<(e|0)){q=k;r=l}else{i=l;break}}return i|0};case 4:{if((e|0)>0){w=0;x=0}else{i=0;return i|0}while(1){r=b[a+(w<<1)>>1]|0;q=r<<16>>16;if(r<<16>>16==0){y=0;z=q}else{y=1;z=r<<16>>16>0?q:-q|0}q=b[a+((w|1)<<1)>>1]|0;r=q<<16>>16;if(q<<16>>16==0){A=y;B=r}else{A=y+1|0;B=q<<16>>16>0?r:-r|0}r=b[a+((w|2)<<1)>>1]|0;q=r<<16>>16;if(r<<16>>16==0){C=A;D=q}else{C=A+1|0;D=r<<16>>16>0?q:-q|0}q=b[a+((w|3)<<1)>>1]|0;r=q<<16>>16;if(q<<16>>16==0){E=C;F=r}else{E=C+1|0;F=q<<16>>16>0?r:-r|0}r=E+x+(c[1981344+(z*108|0)+(B*36|0)+(D*12|0)+(F<<2)>>2]&65535)|0;q=w+4|0;if((q|0)<(e|0)){w=q;x=r}else{i=r;break}}return i|0};case 5:{if((e|0)>0){G=0;H=0}else{i=0;return i|0}while(1){x=(c[1981016+(((b[a+(G<<1)>>1]|0)+4|0)*36|0)+((b[a+((G|1)<<1)>>1]|0)+4<<2)>>2]>>16)+H|0;w=G+2|0;if((w|0)<(e|0)){G=w;H=x}else{i=x;break}}return i|0};case 1:{if((e|0)>0){I=0;J=0}else{i=0;return i|0}while(1){H=(c[1981672+(((b[a+(I<<1)>>1]|0)+1|0)*108|0)+(((b[a+((I|1)<<1)>>1]|0)+1|0)*36|0)+(((b[a+((I|2)<<1)>>1]|0)+1|0)*12|0)+((b[a+((I|3)<<1)>>1]|0)+1<<2)>>2]>>16)+J|0;G=I+4|0;if((G|0)<(e|0)){I=G;J=H}else{i=H;break}}return i|0};case 7:{if((e|0)>0){K=0;L=0}else{i=0;return i|0}while(1){J=b[a+(K<<1)>>1]|0;I=J<<16>>16;if(J<<16>>16==0){M=0;N=I}else{M=1;N=J<<16>>16>0?I:-I|0}I=b[a+((K|1)<<1)>>1]|0;J=I<<16>>16;if(I<<16>>16==0){O=M;P=J}else{O=M+1|0;P=I<<16>>16>0?J:-J|0}J=O+L+(c[1980760+(N<<5)+(P<<2)>>2]>>16)|0;I=K+2|0;if((I|0)<(e|0)){K=I;L=J}else{i=J;break}}return i|0};case 11:{if((e|0)>0){Q=0;R=0}else{i=0;return i|0}while(1){L=b[a+(Q<<1)>>1]|0;K=L<<16>>16;if(L<<16>>16==0){S=0;T=K}else{S=1;T=L<<16>>16>0?K:-K|0}K=b[a+((Q|1)<<1)>>1]|0;L=K<<16>>16;if(K<<16>>16==0){U=S;V=L}else{U=S+1|0;V=K<<16>>16>0?L:-L|0}L=U+R+(d[((V|0)<16?V:16)+(1982e3+(((T|0)<16?T:16)*17|0))|0]|0)|0;if((T|0)>15){K=T>>1;if((K|0)>15){P=L;N=0;O=K;while(1){K=P+1|0;M=N+1|0;J=O>>1;if((J|0)>15){P=K;N=M;O=J}else{W=K;X=M;break}}}else{W=L;X=0}Y=W+5+X|0}else{Y=L}if((V|0)>15){O=V>>1;if((O|0)>15){N=Y;P=0;M=O;while(1){O=N+1|0;K=P+1|0;J=M>>1;if((J|0)>15){N=O;P=K;M=J}else{Z=O;_=K;break}}}else{Z=Y;_=0}$=Z+5+_|0}else{$=Y}M=Q+2|0;if((M|0)<(e|0)){Q=M;R=$}else{i=$;break}}return i|0};case 2:{if((e|0)>0){aa=0;ab=0}else{i=0;return i|0}while(1){$=(c[1981672+(((b[a+(aa<<1)>>1]|0)+1|0)*108|0)+(((b[a+((aa|1)<<1)>>1]|0)+1|0)*36|0)+(((b[a+((aa|2)<<1)>>1]|0)+1|0)*12|0)+((b[a+((aa|3)<<1)>>1]|0)+1<<2)>>2]&65535)+ab|0;R=aa+4|0;if((R|0)<(e|0)){aa=R;ab=$}else{i=$;break}}return i|0};case 10:{if((e|0)>0){ac=0;ad=0}else{i=0;return i|0}while(1){ab=b[a+(ac<<1)>>1]|0;aa=ab<<16>>16;if(ab<<16>>16==0){ae=0;af=aa}else{ae=1;af=ab<<16>>16>0?aa:-aa|0}aa=b[a+((ac|1)<<1)>>1]|0;ab=aa<<16>>16;if(aa<<16>>16==0){ag=ae;ah=ab}else{ag=ae+1|0;ah=aa<<16>>16>0?ab:-ab|0}ab=ag+ad+(c[1980080+(af*52|0)+(ah<<2)>>2]&65535)|0;aa=ac+2|0;if((aa|0)<(e|0)){ac=aa;ad=ab}else{i=ab;break}}return i|0};case 3:{if((e|0)>0){ai=0;aj=0}else{i=0;return i|0}while(1){ad=b[a+(ai<<1)>>1]|0;ac=ad<<16>>16;if(ad<<16>>16==0){ak=0;al=ac}else{ak=1;al=ad<<16>>16>0?ac:-ac|0}ac=b[a+((ai|1)<<1)>>1]|0;ad=ac<<16>>16;if(ac<<16>>16==0){am=ak;an=ad}else{am=ak+1|0;an=ac<<16>>16>0?ad:-ad|0}ad=b[a+((ai|2)<<1)>>1]|0;ac=ad<<16>>16;if(ad<<16>>16==0){ao=am;ap=ac}else{ao=am+1|0;ap=ad<<16>>16>0?ac:-ac|0}ac=b[a+((ai|3)<<1)>>1]|0;ad=ac<<16>>16;if(ac<<16>>16==0){aq=ao;ar=ad}else{aq=ao+1|0;ar=ac<<16>>16>0?ad:-ad|0}ad=aq+aj+(c[1981344+(al*108|0)+(an*36|0)+(ap*12|0)+(ar<<2)>>2]>>16)|0;ac=ai+4|0;if((ac|0)<(e|0)){ai=ac;aj=ad}else{i=ad;break}}return i|0};default:{i=0;return i|0}}return 0}function MY(a,f,g,h){a=a|0;f=f|0;g=g|0;h=h|0;var i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0;switch(g|0){case 11:{if((f|0)<=0){return 0}g=h+4|0;i=h|0;j=h+8|0;k=c[497855]&1;l=0;do{m=b[a+(l<<1)>>1]|0;n=m<<16>>16;do{if(m<<16>>16==0){o=0;p=0;q=n}else{if(m<<16>>16>=0){o=1;p=0;q=n;break}o=1;p=1;q=-n|0}}while(0);n=b[a+((l|1)<<1)>>1]|0;m=n<<16>>16;do{if(n<<16>>16==0){r=o;s=p;t=m}else{u=o+1|0;v=p<<1;if(n<<16>>16>=0){r=u;s=v;t=m;break}r=u;s=v|1;t=-m|0}}while(0);m=(q|0)<16?q:16;n=(t|0)<16?t:16;v=e[1984224+(m*34|0)+(n<<1)>>1]|0;u=d[1982e3+(m*17|0)+n|0]|0;n=c[1991416+(u<<2)>>2]|0;m=c[g>>2]|0;w=m+u|0;if(w>>>0<32){c[g>>2]=w;x=w;y=c[i>>2]<<u|n&v}else{Hw(j,c[i>>2]|0,m);c[g>>2]=u;x=u;y=n&v}c[i>>2]=y;v=c[1991416+(r<<2)>>2]|0;n=x+r|0;if(n>>>0<32){c[g>>2]=n;z=n;A=y<<r|v&s}else{Hw(j,y,x);c[g>>2]=r;z=r;A=v&s}c[i>>2]=A;do{if((q|0)>15){v=q>>1;if((v|0)>15){n=0;u=v;v=z;m=A;while(1){w=v+1|0;if(w>>>0<32){c[g>>2]=w;B=w;C=m<<1|k}else{Hw(j,m,v);c[g>>2]=1;B=1;C=k}c[i>>2]=C;w=u>>1;if((w|0)>15){n=n+1|0;u=w;v=B;m=C}else{break}}D=n+5|0;E=B;F=C}else{D=4;E=z;F=A}m=E+1|0;if(m>>>0<32){c[g>>2]=m;G=m;H=F<<1}else{Hw(j,F,E);c[g>>2]=1;G=1;H=0}c[i>>2]=H;m=q-(1<<D)|0;v=c[1991416+(D<<2)>>2]|0;u=G+D|0;if(u>>>0<32){c[g>>2]=u;w=H<<D|v&m;c[i>>2]=w;I=u;J=w;break}else{Hw(j,H,G);c[g>>2]=D;w=v&m;c[i>>2]=w;I=D;J=w;break}}else{I=z;J=A}}while(0);do{if((t|0)>15){w=t>>1;if((w|0)>15){m=0;v=w;w=I;u=J;while(1){K=w+1|0;if(K>>>0<32){c[g>>2]=K;L=K;M=u<<1|k}else{Hw(j,u,w);c[g>>2]=1;L=1;M=k}c[i>>2]=M;K=v>>1;if((K|0)>15){m=m+1|0;v=K;w=L;u=M}else{break}}N=m+5|0;O=L;P=M}else{N=4;O=I;P=J}u=O+1|0;if(u>>>0<32){c[g>>2]=u;Q=u;R=P<<1}else{Hw(j,P,O);c[g>>2]=1;Q=1;R=0}c[i>>2]=R;u=t-(1<<N)|0;w=c[1991416+(N<<2)>>2]|0;v=Q+N|0;if(v>>>0<32){c[g>>2]=v;c[i>>2]=R<<N|w&u;break}else{Hw(j,R,Q);c[g>>2]=N;c[i>>2]=w&u;break}}}while(0);l=l+2|0;}while((l|0)<(f|0));return 0};case 2:{if((f|0)<=0){return 0}l=h+4|0;i=h|0;N=h+8|0;g=0;Q=c[l>>2]|0;while(1){R=(b[a+(g<<1)>>1]|0)+1|0;j=(b[a+((g|1)<<1)>>1]|0)+1|0;t=(b[a+((g|2)<<1)>>1]|0)+1|0;O=(b[a+((g|3)<<1)>>1]|0)+1|0;P=e[1984056+(R*54|0)+(j*18|0)+(t*6|0)+(O<<1)>>1]|0;J=c[1981672+(R*108|0)+(j*36|0)+(t*12|0)+(O<<2)>>2]&65535;O=c[1991416+(J<<2)>>2]|0;t=Q+J|0;if(t>>>0<32){c[l>>2]=t;S=t;T=c[i>>2]<<J|O&P}else{Hw(N,c[i>>2]|0,Q);c[l>>2]=J;S=J;T=O&P}c[i>>2]=T;P=g+4|0;if((P|0)<(f|0)){g=P;Q=S}else{break}}return 0};case 4:{if((f|0)<=0){return 0}S=h+4|0;Q=h|0;g=h+8|0;T=0;do{i=b[a+(T<<1)>>1]|0;l=i<<16>>16;do{if(i<<16>>16==0){U=0;V=0;W=l}else{if(i<<16>>16>=0){U=1;V=0;W=l;break}U=1;V=1;W=-l|0}}while(0);l=b[a+((T|1)<<1)>>1]|0;i=l<<16>>16;do{if(l<<16>>16==0){X=U;Y=V;Z=i}else{N=U+1|0;P=V<<1;if(l<<16>>16>=0){X=N;Y=P;Z=i;break}X=N;Y=P|1;Z=-i|0}}while(0);i=b[a+((T|2)<<1)>>1]|0;l=i<<16>>16;do{if(i<<16>>16==0){_=X;$=Y;aa=l}else{P=X+1|0;N=Y<<1;if(i<<16>>16>=0){_=P;$=N;aa=l;break}_=P;$=N|1;aa=-l|0}}while(0);l=b[a+((T|3)<<1)>>1]|0;i=l<<16>>16;do{if(l<<16>>16==0){ab=_;ac=$;ad=i}else{N=_+1|0;P=$<<1;if(l<<16>>16>=0){ab=N;ac=P;ad=i;break}ab=N;ac=P|1;ad=-i|0}}while(0);i=e[1983720+(W*54|0)+(Z*18|0)+(aa*6|0)+(ad<<1)>>1]|0;l=c[1981344+(W*108|0)+(Z*36|0)+(aa*12|0)+(ad<<2)>>2]&65535;P=c[1991416+(l<<2)>>2]|0;N=c[S>>2]|0;O=N+l|0;if(O>>>0<32){c[S>>2]=O;ae=O;af=c[Q>>2]<<l|P&i}else{Hw(g,c[Q>>2]|0,N);c[S>>2]=l;ae=l;af=P&i}c[Q>>2]=af;i=c[1991416+(ab<<2)>>2]|0;P=ae+ab|0;if(P>>>0<32){c[S>>2]=P;ag=af<<ab|i&ac}else{Hw(g,af,ae);c[S>>2]=ab;ag=i&ac}c[Q>>2]=ag;T=T+4|0;}while((T|0)<(f|0));return 0};case 7:{if((f|0)<=0){return 0}T=h+4|0;ag=h|0;Q=h+8|0;ac=0;do{ab=b[a+(ac<<1)>>1]|0;S=ab<<16>>16;do{if(ab<<16>>16==0){ah=0;ai=0;aj=S}else{if(ab<<16>>16>=0){ah=1;ai=0;aj=S;break}ah=1;ai=1;aj=-S|0}}while(0);S=b[a+((ac|1)<<1)>>1]|0;ab=S<<16>>16;do{if(S<<16>>16==0){ak=ah;al=ai;am=ab}else{ae=ah+1|0;af=ai<<1;if(S<<16>>16>=0){ak=ae;al=af;am=ab;break}ak=ae;al=af|1;am=-ab|0}}while(0);ab=e[1983256+(aj<<4)+(am<<1)>>1]|0;S=c[1980760+(aj<<5)+(am<<2)>>2]>>16;af=c[1991416+(S<<2)>>2]|0;ae=c[T>>2]|0;g=ae+S|0;if(g>>>0<32){c[T>>2]=g;an=g;ao=c[ag>>2]<<S|af&ab}else{Hw(Q,c[ag>>2]|0,ae);c[T>>2]=S;an=S;ao=af&ab}c[ag>>2]=ao;ab=c[1991416+(ak<<2)>>2]|0;af=an+ak|0;if(af>>>0<32){c[T>>2]=af;ap=ao<<ak|ab&al}else{Hw(Q,ao,an);c[T>>2]=ak;ap=ab&al}c[ag>>2]=ap;ac=ac+2|0;}while((ac|0)<(f|0));return 0};case 8:{if((f|0)<=0){return 0}ac=h+4|0;ap=h|0;ag=h+8|0;al=0;do{ak=b[a+(al<<1)>>1]|0;T=ak<<16>>16;do{if(ak<<16>>16==0){aq=0;ar=0;as=T}else{if(ak<<16>>16>=0){aq=1;ar=0;as=T;break}aq=1;ar=1;as=-T|0}}while(0);T=b[a+((al|1)<<1)>>1]|0;ak=T<<16>>16;do{if(T<<16>>16==0){at=aq;au=ar;av=ak}else{an=aq+1|0;ao=ar<<1;if(T<<16>>16>=0){at=an;au=ao;av=ak;break}at=an;au=ao|1;av=-ak|0}}while(0);ak=e[1983128+(as<<4)+(av<<1)>>1]|0;T=c[1980760+(as<<5)+(av<<2)>>2]&65535;ao=c[1991416+(T<<2)>>2]|0;an=c[ac>>2]|0;Q=an+T|0;if(Q>>>0<32){c[ac>>2]=Q;aw=Q;ax=c[ap>>2]<<T|ao&ak}else{Hw(ag,c[ap>>2]|0,an);c[ac>>2]=T;aw=T;ax=ao&ak}c[ap>>2]=ax;ak=c[1991416+(at<<2)>>2]|0;ao=aw+at|0;if(ao>>>0<32){c[ac>>2]=ao;ay=ax<<at|ak&au}else{Hw(ag,ax,aw);c[ac>>2]=at;ay=ak&au}c[ap>>2]=ay;al=al+2|0;}while((al|0)<(f|0));return 0};case 5:{if((f|0)<=0){return 0}al=h+4|0;ay=h|0;ap=h+8|0;au=0;at=c[al>>2]|0;while(1){ac=(b[a+(au<<1)>>1]|0)+4|0;aw=(b[a+((au|1)<<1)>>1]|0)+4|0;ax=e[1983552+(ac*18|0)+(aw<<1)>>1]|0;ag=c[1981016+(ac*36|0)+(aw<<2)>>2]>>16;aw=c[1991416+(ag<<2)>>2]|0;ac=at+ag|0;if(ac>>>0<32){c[al>>2]=ac;az=ac;aA=c[ay>>2]<<ag|aw&ax}else{Hw(ap,c[ay>>2]|0,at);c[al>>2]=ag;az=ag;aA=aw&ax}c[ay>>2]=aA;ax=au+2|0;if((ax|0)<(f|0)){au=ax;at=az}else{break}}return 0};case 1:{if((f|0)<=0){return 0}az=h+4|0;at=h|0;au=h+8|0;aA=0;ay=c[az>>2]|0;while(1){al=(b[a+(aA<<1)>>1]|0)+1|0;ap=(b[a+((aA|1)<<1)>>1]|0)+1|0;ax=(b[a+((aA|2)<<1)>>1]|0)+1|0;aw=(b[a+((aA|3)<<1)>>1]|0)+1|0;ag=e[1985288+(al*54|0)+(ap*18|0)+(ax*6|0)+(aw<<1)>>1]|0;ac=c[1981672+(al*108|0)+(ap*36|0)+(ax*12|0)+(aw<<2)>>2]>>16;aw=c[1991416+(ac<<2)>>2]|0;ax=ay+ac|0;if(ax>>>0<32){c[az>>2]=ax;aB=ax;aC=c[at>>2]<<ac|aw&ag}else{Hw(au,c[at>>2]|0,ay);c[az>>2]=ac;aB=ac;aC=aw&ag}c[at>>2]=aC;ag=aA+4|0;if((ag|0)<(f|0)){aA=ag;ay=aB}else{break}}return 0};case 6:{if((f|0)<=0){return 0}aB=h+4|0;ay=h|0;aA=h+8|0;aC=0;at=c[aB>>2]|0;while(1){az=(b[a+(aC<<1)>>1]|0)+4|0;au=(b[a+((aC|1)<<1)>>1]|0)+4|0;ag=e[1983384+(az*18|0)+(au<<1)>>1]|0;aw=c[1981016+(az*36|0)+(au<<2)>>2]&65535;au=c[1991416+(aw<<2)>>2]|0;az=at+aw|0;if(az>>>0<32){c[aB>>2]=az;aD=az;aE=c[ay>>2]<<aw|au&ag}else{Hw(aA,c[ay>>2]|0,at);c[aB>>2]=aw;aD=aw;aE=au&ag}c[ay>>2]=aE;ag=aC+2|0;if((ag|0)<(f|0)){aC=ag;at=aD}else{break}}return 0};case 10:{if((f|0)<=0){return 0}aD=h+4|0;at=h|0;aC=h+8|0;aE=0;do{ay=b[a+(aE<<1)>>1]|0;aB=ay<<16>>16;do{if(ay<<16>>16==0){aF=0;aG=0;aH=aB}else{if(ay<<16>>16>=0){aF=1;aG=0;aH=aB;break}aF=1;aG=1;aH=-aB|0}}while(0);aB=b[a+((aE|1)<<1)>>1]|0;ay=aB<<16>>16;do{if(aB<<16>>16==0){aI=aF;aJ=aG;aK=ay}else{aA=aF+1|0;ag=aG<<1;if(aB<<16>>16>=0){aI=aA;aJ=ag;aK=ay;break}aI=aA;aJ=ag|1;aK=-ay|0}}while(0);ay=e[1984944+(aH*26|0)+(aK<<1)>>1]|0;aB=c[1980080+(aH*52|0)+(aK<<2)>>2]&65535;ag=c[1991416+(aB<<2)>>2]|0;aA=c[aD>>2]|0;au=aA+aB|0;if(au>>>0<32){c[aD>>2]=au;aL=au;aM=c[at>>2]<<aB|ag&ay}else{Hw(aC,c[at>>2]|0,aA);c[aD>>2]=aB;aL=aB;aM=ag&ay}c[at>>2]=aM;ay=c[1991416+(aI<<2)>>2]|0;ag=aL+aI|0;if(ag>>>0<32){c[aD>>2]=ag;aN=aM<<aI|ay&aJ}else{Hw(aC,aM,aL);c[aD>>2]=aI;aN=ay&aJ}c[at>>2]=aN;aE=aE+2|0;}while((aE|0)<(f|0));return 0};case 9:{if((f|0)<=0){return 0}aE=h+4|0;aN=h|0;at=h+8|0;aJ=0;do{aI=b[a+(aJ<<1)>>1]|0;aD=aI<<16>>16;do{if(aI<<16>>16==0){aO=0;aP=0;aQ=aD}else{if(aI<<16>>16>=0){aO=1;aP=0;aQ=aD;break}aO=1;aP=1;aQ=-aD|0}}while(0);aD=b[a+((aJ|1)<<1)>>1]|0;aI=aD<<16>>16;do{if(aD<<16>>16==0){aR=aO;aS=aP;aT=aI}else{aL=aO+1|0;aM=aP<<1;if(aD<<16>>16>=0){aR=aL;aS=aM;aT=aI;break}aR=aL;aS=aM|1;aT=-aI|0}}while(0);aI=e[1982784+(aQ*26|0)+(aT<<1)>>1]|0;aD=c[1980080+(aQ*52|0)+(aT<<2)>>2]>>16;aM=c[1991416+(aD<<2)>>2]|0;aL=c[aE>>2]|0;aC=aL+aD|0;if(aC>>>0<32){c[aE>>2]=aC;aU=aC;aV=c[aN>>2]<<aD|aM&aI}else{Hw(at,c[aN>>2]|0,aL);c[aE>>2]=aD;aU=aD;aV=aM&aI}c[aN>>2]=aV;aI=c[1991416+(aR<<2)>>2]|0;aM=aU+aR|0;if(aM>>>0<32){c[aE>>2]=aM;aW=aV<<aR|aI&aS}else{Hw(at,aV,aU);c[aE>>2]=aR;aW=aI&aS}c[aN>>2]=aW;aJ=aJ+2|0;}while((aJ|0)<(f|0));return 0};case 3:{if((f|0)<=0){return 0}aJ=h+4|0;aW=h|0;aN=h+8|0;h=0;do{aS=b[a+(h<<1)>>1]|0;aR=aS<<16>>16;do{if(aS<<16>>16==0){aX=0;aY=0;aZ=aR}else{if(aS<<16>>16>=0){aX=1;aY=0;aZ=aR;break}aX=1;aY=1;aZ=-aR|0}}while(0);aR=b[a+((h|1)<<1)>>1]|0;aS=aR<<16>>16;do{if(aR<<16>>16==0){a_=aX;a$=aY;a0=aS}else{aE=aX+1|0;aU=aY<<1;if(aR<<16>>16>=0){a_=aE;a$=aU;a0=aS;break}a_=aE;a$=aU|1;a0=-aS|0}}while(0);aS=b[a+((h|2)<<1)>>1]|0;aR=aS<<16>>16;do{if(aS<<16>>16==0){a1=a_;a2=a$;a3=aR}else{aU=a_+1|0;aE=a$<<1;if(aS<<16>>16>=0){a1=aU;a2=aE;a3=aR;break}a1=aU;a2=aE|1;a3=-aR|0}}while(0);aR=b[a+((h|3)<<1)>>1]|0;aS=aR<<16>>16;do{if(aR<<16>>16==0){a4=a1;a5=a2;a6=aS}else{aE=a1+1|0;aU=a2<<1;if(aR<<16>>16>=0){a4=aE;a5=aU;a6=aS;break}a4=aE;a5=aU|1;a6=-aS|0}}while(0);aS=e[1983888+(aZ*54|0)+(a0*18|0)+(a3*6|0)+(a6<<1)>>1]|0;aR=c[1981344+(aZ*108|0)+(a0*36|0)+(a3*12|0)+(a6<<2)>>2]>>16;aU=c[1991416+(aR<<2)>>2]|0;aE=c[aJ>>2]|0;aV=aE+aR|0;if(aV>>>0<32){c[aJ>>2]=aV;a7=aV;a8=c[aW>>2]<<aR|aU&aS}else{Hw(aN,c[aW>>2]|0,aE);c[aJ>>2]=aR;a7=aR;a8=aU&aS}c[aW>>2]=a8;aS=c[1991416+(a4<<2)>>2]|0;aU=a7+a4|0;if(aU>>>0<32){c[aJ>>2]=aU;a9=a8<<a4|aS&a5}else{Hw(aN,a8,a7);c[aJ>>2]=a4;a9=aS&a5}c[aW>>2]=a9;h=h+4|0;}while((h|0)<(f|0));return 0};default:{return 0}}return 0}function MZ(a,b){a=a|0;b=b|0;var e=0,f=0,g=0,h=0,i=0,j=0;if((((a|0)>0?a:-a|0)|0)>60){e=1;return e|0}f=a+60|0;a=c[1982296+(f<<2)>>2]|0;g=d[1979952+f|0]|0;f=c[1991416+(g<<2)>>2]|0;h=b+4|0;i=c[h>>2]|0;j=i+g|0;if(j>>>0<32){c[h>>2]=j;j=b|0;c[j>>2]=c[j>>2]<<g|f&a;e=0;return e|0}else{j=b|0;Hw(b+8|0,c[j>>2]|0,i);c[h>>2]=g;c[j>>2]=f&a;e=0;return e|0}return 0}function M_(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0;if((e|0)>0){g=0;h=0;i=0;j=0;k=0;l=0;m=0;n=0;while(1){o=b[a+(n<<1)>>1]|0;p=o<<16>>16;q=b[a+((n|1)<<1)>>1]|0;r=q<<16>>16;s=b[a+((n|2)<<1)>>1]|0;t=s<<16>>16;u=b[a+((n|3)<<1)>>1]|0;v=u<<16>>16;w=(c[1981672+((p+1|0)*108|0)+((r+1|0)*36|0)+((t+1|0)*12|0)+(v+1<<2)>>2]|0)+m|0;x=(c[1981016+((p+4|0)*36|0)+(r+4<<2)>>2]|0)+k+(c[1981016+((t+4|0)*36|0)+(v+4<<2)>>2]|0)|0;y=o<<16>>16>0?p:-p|0;p=q<<16>>16>0?r:-r|0;r=s<<16>>16>0?t:-t|0;t=u<<16>>16>0?v:-v|0;v=(c[1981344+(y*108|0)+(p*36|0)+(r*12|0)+(t<<2)>>2]|0)+l|0;u=(c[1980760+(y<<5)+(p<<2)>>2]|0)+g+(c[1980760+(r<<5)+(t<<2)>>2]|0)|0;s=(c[1980080+(y*52|0)+(p<<2)>>2]|0)+h+(c[1980080+(r*52|0)+(t<<2)>>2]|0)|0;q=(d[1982e3+(y*17|0)+p|0]|0)+i+(d[1982e3+(r*17|0)+t|0]|0)|0;o=((y|0)>0)+j+((p|0)>0)+((r|0)>0)+((t|0)>0)|0;t=n+4|0;if((t|0)<(e|0)){g=u;h=s;i=q;j=o;k=x;l=v;m=w;n=t}else{z=u;A=s;B=q;C=o;D=x;E=v;F=w;break}}}else{z=0;A=0;B=0;C=0;D=0;E=0;F=0}c[f+4>>2]=F>>16;c[f+8>>2]=F&65535;c[f+12>>2]=(E>>16)+C;c[f+16>>2]=(E&65535)+C;c[f+20>>2]=D>>16;c[f+24>>2]=D&65535;c[f+28>>2]=C+(z>>16);c[f+32>>2]=C+(z&65535);c[f+36>>2]=C+(A>>16);c[f+40>>2]=C+(A&65535);c[f+44>>2]=C+B;return}function M$(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0;if((e|0)>0){g=0;h=0;i=0;j=0;k=0;l=0;m=0;while(1){n=b[a+(m<<1)>>1]|0;o=n<<16>>16;p=b[a+((m|1)<<1)>>1]|0;q=p<<16>>16;r=b[a+((m|2)<<1)>>1]|0;s=r<<16>>16;t=b[a+((m|3)<<1)>>1]|0;u=t<<16>>16;v=(c[1981016+((o+4|0)*36|0)+(q+4<<2)>>2]|0)+k+(c[1981016+((s+4|0)*36|0)+(u+4<<2)>>2]|0)|0;w=n<<16>>16>0?o:-o|0;o=p<<16>>16>0?q:-q|0;q=r<<16>>16>0?s:-s|0;s=t<<16>>16>0?u:-u|0;u=(c[1981344+(w*108|0)+(o*36|0)+(q*12|0)+(s<<2)>>2]|0)+l|0;t=(c[1980760+(w<<5)+(o<<2)>>2]|0)+j+(c[1980760+(q<<5)+(s<<2)>>2]|0)|0;r=(c[1980080+(w*52|0)+(o<<2)>>2]|0)+g+(c[1980080+(q*52|0)+(s<<2)>>2]|0)|0;p=(d[1982e3+(w*17|0)+o|0]|0)+h+(d[1982e3+(q*17|0)+s|0]|0)|0;n=((w|0)>0)+i+((o|0)>0)+((q|0)>0)+((s|0)>0)|0;s=m+4|0;if((s|0)<(e|0)){g=r;h=p;i=n;j=t;k=v;l=u;m=s}else{x=r;y=p;z=n;A=t;B=v;C=u;break}}}else{x=0;y=0;z=0;A=0;B=0;C=0}c[f+4>>2]=536870911;c[f+8>>2]=536870911;c[f+12>>2]=(C>>16)+z;c[f+16>>2]=(C&65535)+z;c[f+20>>2]=B>>16;c[f+24>>2]=B&65535;c[f+28>>2]=(A>>16)+z;c[f+32>>2]=(A&65535)+z;c[f+36>>2]=z+(x>>16);c[f+40>>2]=z+(x&65535);c[f+44>>2]=z+y;return}function M0(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0;if((e|0)>0){g=0;h=0;i=0;j=0;k=0;l=0;while(1){m=b[a+(l<<1)>>1]|0;n=m<<16>>16;o=b[a+((l|1)<<1)>>1]|0;p=o<<16>>16;q=(c[1981016+((n+4|0)*36|0)+(p+4<<2)>>2]|0)+k|0;r=m<<16>>16>0?n:-n|0;n=o<<16>>16>0?p:-p|0;p=(c[1980760+(r<<5)+(n<<2)>>2]|0)+g|0;o=(c[1980080+(r*52|0)+(n<<2)>>2]|0)+h|0;m=(d[1982e3+(r*17|0)+n|0]|0)+i|0;s=((r|0)>0)+j+((n|0)>0)|0;n=l+2|0;if((n|0)<(e|0)){g=p;h=o;i=m;j=s;k=q;l=n}else{t=p;u=o;v=m;w=s;x=q;break}}}else{t=0;u=0;v=0;w=0;x=0}c[f+4>>2]=536870911;c[f+8>>2]=536870911;c[f+12>>2]=536870911;c[f+16>>2]=536870911;c[f+20>>2]=x>>16;c[f+24>>2]=x&65535;c[f+28>>2]=w+(t>>16);c[f+32>>2]=w+(t&65535);c[f+36>>2]=w+(u>>16);c[f+40>>2]=w+(u&65535);c[f+44>>2]=w+v;return}function M1(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;if((e|0)>0){g=0;h=0;i=0;j=0;k=0;while(1){l=b[a+(k<<1)>>1]|0;m=l<<16>>16>0?l:-l&65535;l=m<<16>>16;n=b[a+((k|1)<<1)>>1]|0;o=n<<16>>16>0?n:-n&65535;n=o<<16>>16;p=(c[1980760+(l<<5)+(n<<2)>>2]|0)+j|0;q=(c[1980080+(l*52|0)+(n<<2)>>2]|0)+g|0;r=(d[1982e3+(l*17|0)+n|0]|0)+h|0;n=(m<<16>>16>0)+i+(o<<16>>16>0)|0;o=k+2|0;if((o|0)<(e|0)){g=q;h=r;i=n;j=p;k=o}else{s=q;t=r;u=n;v=p;break}}}else{s=0;t=0;u=0;v=0}c[f+4>>2]=536870911;c[f+8>>2]=536870911;c[f+12>>2]=536870911;c[f+16>>2]=536870911;c[f+20>>2]=536870911;c[f+24>>2]=536870911;c[f+28>>2]=(v>>16)+u;c[f+32>>2]=(v&65535)+u;c[f+36>>2]=u+(s>>16);c[f+40>>2]=u+(s&65535);c[f+44>>2]=u+t;return}function M2(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;if((e|0)>0){g=0;h=0;i=0;j=0;while(1){k=b[a+(j<<1)>>1]|0;l=k<<16>>16>0?k:-k&65535;k=l<<16>>16;m=b[a+((j|1)<<1)>>1]|0;n=m<<16>>16>0?m:-m&65535;m=n<<16>>16;o=(c[1980080+(k*52|0)+(m<<2)>>2]|0)+i|0;p=(d[1982e3+(k*17|0)+m|0]|0)+g|0;m=(l<<16>>16>0)+h+(n<<16>>16>0)|0;n=j+2|0;if((n|0)<(e|0)){g=p;h=m;i=o;j=n}else{q=p;r=m;s=o;break}}}else{q=0;r=0;s=0}c[f+4>>2]=536870911;c[f+8>>2]=536870911;c[f+12>>2]=536870911;c[f+16>>2]=536870911;c[f+20>>2]=536870911;c[f+24>>2]=536870911;c[f+28>>2]=536870911;c[f+32>>2]=536870911;c[f+36>>2]=(s>>16)+r;c[f+40>>2]=(s&65535)+r;c[f+44>>2]=r+q;return}function M3(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;if((e|0)>0){g=0;h=0;i=0;while(1){j=b[a+(i<<1)>>1]|0;k=j<<16>>16>0?j:-j&65535;j=b[a+((i|1)<<1)>>1]|0;l=j<<16>>16>0?j:-j&65535;j=(d[(l<<16>>16)+(1982e3+((k<<16>>16)*17|0))|0]|0)+h|0;m=(k<<16>>16>0)+g+(l<<16>>16>0)|0;l=i+2|0;if((l|0)<(e|0)){g=m;h=j;i=l}else{n=m;o=j;break}}}else{n=0;o=0}c[f+4>>2]=536870911;c[f+8>>2]=536870911;c[f+12>>2]=536870911;c[f+16>>2]=536870911;c[f+20>>2]=536870911;c[f+24>>2]=536870911;c[f+28>>2]=536870911;c[f+32>>2]=536870911;c[f+36>>2]=536870911;c[f+40>>2]=536870911;c[f+44>>2]=o+n;return}function M4(a,e,f){a=a|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0;if((e|0)>0){g=0;h=0;i=0;j=0;while(1){k=b[a+(j<<1)>>1]|0;l=k<<16>>16>0?k:-k&65535;k=l<<16>>16;m=b[a+((j|1)<<1)>>1]|0;n=m<<16>>16>0?m:-m&65535;m=n<<16>>16;o=(l<<16>>16>0)+h+(n<<16>>16>0)|0;p=(d[((m|0)<16?m:16)+(1982e3+(((k|0)<16?k:16)*17|0))|0]|0)+i|0;if(l<<16>>16>15){l=k;k=g+5|0;while(1){q=l>>1;if((q|0)>15){l=q;k=k+2|0}else{r=k;break}}}else{r=g}if(n<<16>>16>15){k=m;l=r+5|0;while(1){q=k>>1;if((q|0)>15){k=q;l=l+2|0}else{s=l;break}}}else{s=r}l=j+2|0;if((l|0)<(e|0)){g=s;h=o;i=p;j=l}else{t=s;u=o;v=p;break}}}else{t=0;u=0;v=0}c[f>>2]=536870911;c[f+4>>2]=536870911;c[f+8>>2]=536870911;c[f+12>>2]=536870911;c[f+16>>2]=536870911;c[f+20>>2]=536870911;c[f+24>>2]=536870911;c[f+28>>2]=536870911;c[f+32>>2]=536870911;c[f+36>>2]=536870911;c[f+40>>2]=536870911;c[f+44>>2]=u+t+v;return}function M5(a,b){a=a|0;b=b|0;var d=0,e=0;I_(a,208);d=a+16|0;if((b|0)==0){c[d>>2]=8;c[a+36>>2]=1;c[a+40>>2]=1;e=1}else{c[d>>2]=4;c[a+36>>2]=0;c[a+40>>2]=0;e=0}c[a+44>>2]=4;c[a+4>>2]=0;c[a+8>>2]=c[1917832+(e*20|0)>>2];return}function M6(a,d,e){a=a|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0;f=a+16|0;g=c[f>>2]|0;if((e|0)!=0){c[a+4>>2]=0;c[a+8>>2]=0;c[a+44>>2]=1;c[a+48>>2]=1;return 0}e=a+20|0;h=a+24|0;c[h>>2]=c[e>>2];i=a+28|0;j=a+32|0;c[j>>2]=c[i>>2];IY(a+68|0,a+100|0,32);IY(a+132|0,a+164|0,32);k=a+36|0;do{if((c[k>>2]|0)!=0){l=a+48|0;I_(l,16);c[a+44>>2]=4;IY(l,1917208+(c[j>>2]<<4)|0,16);if((c[e>>2]|0)==1){c[a+64>>2]=c[a+68+(c[j>>2]<<2)>>2];break}else{c[a+64>>2]=0;break}}}while(0);l=d>>((g|0)==4?2:3);L20844:do{if((c[f>>2]|0)!=0){d=a+200|0;m=a+204|0;n=c[d>>2]|0;o=c[m>>2]|0;if((l|0)<=0){p=0;while(1){c[a+100+(p<<2)>>2]=0;c[a+164+(p<<2)>>2]=0;c[d>>2]=n;c[m>>2]=o;p=p+1|0;if(p>>>0>=(c[f>>2]|0)>>>0){break L20844}}}p=c[a>>2]|0;q=0;r=n;s=o;while(1){t=0;u=r;v=s;w=0;x=p;y=0;while(1){z=b[x>>1]<<15;A=z-u|0;N2(A,(A|0)<0|0?-1:0,1620901888,0)|0;A=G;N2(v,(v|0)<0|0?-1:0,-1094123520,-1)|0;B=A-G<<1;A=z;C=(z|0)<0|0?-1:0;N2(A,C,A,C)|0;D=(G>>>4|0<<28)+w|0;C=B;A=(B|0)<0|0?-1:0;N2(C,A,C,A)|0;E=(G>>>4|0<<28)+t|0;A=y+1|0;if((A|0)<(l|0)){t=E;u=z;v=B;w=D;x=x+2|0;y=A}else{break}}c[a+100+(q<<2)>>2]=D;c[a+164+(q<<2)>>2]=E;c[d>>2]=z;c[m>>2]=B;y=q+1|0;if(y>>>0<(c[f>>2]|0)>>>0){p=p+(l<<1)|0;q=y;r=z;s=B}else{break}}}}while(0);c[e>>2]=0;B=g-1|0;z=a+132+(B<<2)|0;do{if((g|0)==0){F=14870}else{l=a+196|0;f=c[z>>2]|0;E=0;D=0;s=0;r=c[l>>2]|0;while(1){N2(r,(r|0)<0|0?-1:0,1503264768,0)|0;q=G;N2(f,(f|0)<0|0?-1:0,644245120,0)|0;p=q+G<<1;c[l>>2]=p;q=a+164+(D<<2)|0;m=c[q>>2]|0;N2(m,(m|0)<0|0?-1:0,214761472,0)|0;if((G<<1|0>>>31|0)>(p|0)){c[e>>2]=1;c[i>>2]=D;H=1;I=c[q>>2]|0}else{H=s;I=m}J=(E|0)>(I|0)?E:I;m=D+1|0;if(m>>>0<g>>>0){f=I;E=J;D=m;s=H;r=p}else{break}}if((J|0)<15625){F=14870;break}if((H|0)==0){F=14872}else{K=H}}}while(0);if((F|0)==14870){c[e>>2]=0;F=14872}do{if((F|0)==14872){if((c[h>>2]|0)!=1){K=0;break}H=c[z>>2]>>4;J=c[a+168>>2]|0;I=J;g=(J|0)<0|0?-1:0;J=1342177280;r=0;N2(I,g,J,r)|0;if((H|0)<=(G<<1|0>>>31|0)){K=0;break}if((c[j>>2]|0)!=(B|0)){K=0;break}c[e>>2]=1;c[i>>2]=0;K=1}}while(0);i=a+4|0;e=c[i>>2]|0;if((c[a+40>>2]|0)==0){L=1914736+(K*24|0)+(e<<2)|0}else{L=1926496+((c[h>>2]|0)*48|0)+(K*24|0)+(e<<2)|0}e=c[L>>2]|0;c[i>>2]=e;c[a+8>>2]=c[1917832+((c[k>>2]|0)*20|0)+(e<<2)>>2];return 0}function M7(b,e,f,g){b=b|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;h=(f|0)==2;f=(g|0)==1;do{if(h&f){g=b+4|0;i=e+4|0;j=a[(c[i>>2]|0)+(1915208+((d[1915208+(c[g>>2]|0)|0]|0)*5|0))|0]|0;k=j&255;if(j<<24>>24==5){l=-1;return l|0}else{c[g>>2]=k;c[i>>2]=k;j=b+36|0;m=c[j>>2]|0;c[b+8>>2]=c[1917832+(m*20|0)+(c[g>>2]<<2)>>2];c[e+8>>2]=c[1917832+(m*20|0)+(c[i>>2]<<2)>>2];n=k;o=j;break}}else{n=0;o=b+36|0}}while(0);j=b+4|0;if((c[o>>2]|0)==0){p=14908}else{k=c[j>>2]|0;do{if(h){if(!f){if((k|0)!=2){c[b+44>>2]=1;c[b+48>>2]=1;NM(b+52|0,0,12)}if((c[e+4>>2]|0)==2){break}c[e+44>>2]=1;c[e+48>>2]=1;NM(e+52|0,0,12);break}i=c[e+4>>2]|0;if((n|0)!=2){c[b+44>>2]=1;c[e+44>>2]=1;c[b+48>>2]=1;c[e+48>>2]=1;c[b+52>>2]=0;c[e+52>>2]=0;c[b+56>>2]=0;c[e+56>>2]=0;c[b+60>>2]=0;c[e+60>>2]=0;break}m=(k|0)==2;g=(i|0)==2;if(m&g){if((c[b+64>>2]|0)>(c[e+64>>2]|0)){c[e+44>>2]=c[b+44>>2];c[e+48>>2]=c[b+48>>2];c[e+52>>2]=c[b+52>>2];c[e+56>>2]=c[b+56>>2];c[e+60>>2]=c[b+60>>2];break}else{c[b+44>>2]=c[e+44>>2];c[b+48>>2]=c[e+48>>2];c[b+52>>2]=c[e+52>>2];c[b+56>>2]=c[e+56>>2];c[b+60>>2]=c[e+60>>2];break}}if(!(g|m^1)){c[e+44>>2]=c[b+44>>2];c[e+48>>2]=c[b+48>>2];c[e+52>>2]=c[b+52>>2];c[e+56>>2]=c[b+56>>2];c[e+60>>2]=c[b+60>>2];break}g=e+44|0;if((i|0)!=2|m){c[g>>2]=2;c[b+44>>2]=2;c[e+48>>2]=4;c[b+48>>2]=4;c[e+52>>2]=4;c[b+52>>2]=4;break}else{c[b+44>>2]=c[g>>2];c[b+48>>2]=c[e+48>>2];c[b+52>>2]=c[e+52>>2];c[b+56>>2]=c[e+56>>2];c[b+60>>2]=c[e+60>>2];break}}else{if((k|0)==2){break}c[b+44>>2]=1;c[b+48>>2]=1;NM(b+52|0,0,12)}}while(0);if((c[o>>2]|0)==0){p=14908}}do{if((p|0)==14908){o=c[j>>2]|0;if((o|0)==0|(o|0)==3){break}c[j>>2]=0;c[b+8>>2]=2}}while(0);if(!h){l=0;return l|0}if((c[e+36>>2]|0)!=0){l=0;return l|0}h=e+4|0;b=c[h>>2]|0;if((b|0)==0|(b|0)==3){l=0;return l|0}c[h>>2]=0;c[e+8>>2]=2;l=0;return l|0}function M8(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0;e=b-2|0;f=c[a>>2]|0;g=c[a+8>>2]|0;do{if((e|0)>2){h=f>>31^f;i=g>>31^g;j=2;while(1){k=j+2|0;l=c[a+(k<<2)>>2]|0;m=l>>31^l;l=(m>>1)+(h>>1)|0;if((l|0)<(i|0)){if((i|0)>-1){n=~i;o=0;while(1){p=n<<1;if((p|0)<0){n=p;o=o+1|0}else{q=o;break}}}else{q=-1}o=HO(l<<q,i<<q,8)|0;n=o;p=(o|0)<0|0?-1:0;N2(n,p,n,p)|0;c[d+(j<<2)>>2]=G<<1|0>>>31}else{c[d+(j<<2)>>2]=2147483647}if((k|0)<(e|0)){h=i;i=m;j=k}else{break}}j=c[a+4>>2]|0;i=c[a+12>>2]|0;if((e|0)<=3){break}h=j>>31^j;j=i>>31^i;i=3;while(1){p=i+2|0;n=c[a+(p<<2)>>2]|0;o=n>>31^n;n=(o>>1)+(h>>1)|0;if((n|0)<(j|0)){if((j|0)>-1){r=~j;s=0;while(1){t=r<<1;if((t|0)<0){r=t;s=s+1|0}else{u=s;break}}}else{u=-1}s=HO(n<<u,j<<u,8)|0;r=s;k=(s|0)<0|0?-1:0;N2(r,k,r,k)|0;c[d+(i<<2)>>2]=G<<1|0>>>31}else{c[d+(i<<2)>>2]=2147483647}if((p|0)<(e|0)){h=j;j=o;i=p}else{break}}}}while(0);e=c[d+8>>2]|0;c[d>>2]=e;c[d+4>>2]=e;e=b-3|0;do{c[d+(e<<2)>>2]=1073741824;e=e+1|0;}while((e|0)<(b|0));return}function M9(a,e,f,g,h,i,j,k,l,m,n,o,p,q){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;var r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0,a3=0,a4=0,a5=0,a6=0,a7=0,a8=0,a9=0,ba=0,bb=0,bc=0,bd=0,be=0,bf=0,bg=0,bh=0,bi=0,bj=0,bk=0,bl=0,bm=0,bn=0,bo=0,bp=0,bq=0,br=0;c[m>>2]=h;r=m+8|0;c[r>>2]=i;s=m+16|0;c[s>>2]=k;c[m+4>>2]=(i|0)/(k|0)|0;k=m+12|0;c[k>>2]=j;t=c[a+4>>2]|0;u=c[a>>2]|0;a=q&1;if((h|0)==0|(h|0)==1|(h|0)==3){v=1976760}else if((h|0)==2){v=1976728}else{v=0}h=m+20|0;c[h>>2]=0;q=m+988|0;c[q>>2]=0;w=m+984|0;c[w>>2]=0;L20965:do{if((j|0)!=0&(i|0)>0){x=(a|0)==0;y=0;z=j;A=0;while(1){B=m+24+(A<<4)|0;do{if((z|0)>0){C=0;while(1){D=C+A|0;c[m+24+(D<<4)+8>>2]=1;c[m+24+(D<<4)+4>>2]=C;c[m+24+(D<<4)+12>>2]=536870911;c[m+24+(D<<4)>>2]=-1;D=C+1|0;E=C+y|0;F=c[l+(E<<2)>>2]|0;MW(e+(F<<1)|0,(c[l+(D+y<<2)>>2]|0)-F|0,c[f+(E<<2)>>2]|0,u+(C*48|0)|0)|0;if((D|0)<(z|0)){C=D}else{break}}C=c[k>>2]|0;if((C|0)>0){G=0}else{H=C;break}do{D=G+A|0;E=m+24+(D<<4)|0;F=m+24+(D<<4)+12|0;do{if((c[F>>2]|0)==536870911){D=G+y|0;if((c[n+(D<<2)>>2]|0)!=-2147483648){c[E>>2]=13;c[F>>2]=0;break}I=c[o+(D<<2)>>2]|0;if((I|0)==0){c[F>>2]=Nc(u+(G*48|0)|0,E|0)|0;break}else{c[E>>2]=I;c[F>>2]=0;break}}}while(0);G=G+1|0;}while((G|0)<(C|0));H=c[k>>2]|0}else{H=z}}while(0);C=(H|0)>1;L20986:do{if(x){if(C){J=0;K=1}else{L=0;M=0;N=14971;break}while(1){F=J+A|0;E=m+24+(F<<4)|0;I=m+24+(F<<4)+8|0;D=m+24+(F<<4)+12|0;O=u+(J*48|0)|0;P=u+(J*48|0)+4|0;Q=u+(J*48|0)+8|0;R=u+(J*48|0)+12|0;S=u+(J*48|0)+16|0;T=u+(J*48|0)+20|0;U=u+(J*48|0)+24|0;V=u+(J*48|0)+28|0;W=u+(J*48|0)+32|0;X=u+(J*48|0)+36|0;Y=u+(J*48|0)+40|0;Z=u+(J*48|0)+44|0;_=J;$=K;while(1){aa=$+A|0;ab=c[I>>2]|0;if((c[E>>2]|0)!=(c[m+24+(aa<<4)>>2]|0)){break}c[I>>2]=ab+1;c[D>>2]=(c[D>>2]|0)+(c[m+24+(aa<<4)+12>>2]|0);aa=(c[u+($*48|0)>>2]|0)+(c[O>>2]|0)|0;c[O>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+4>>2]|0)+(c[P>>2]|0)|0;c[P>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+8>>2]|0)+(c[Q>>2]|0)|0;c[Q>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+12>>2]|0)+(c[R>>2]|0)|0;c[R>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+16>>2]|0)+(c[S>>2]|0)|0;c[S>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+20>>2]|0)+(c[T>>2]|0)|0;c[T>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+24>>2]|0)+(c[U>>2]|0)|0;c[U>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+28>>2]|0)+(c[V>>2]|0)|0;c[V>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+32>>2]|0)+(c[W>>2]|0)|0;c[W>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+36>>2]|0)+(c[X>>2]|0)|0;c[X>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+40>>2]|0)+(c[Y>>2]|0)|0;c[Y>>2]=(aa|0)<536870911?aa:536870911;aa=(c[u+($*48|0)+44>>2]|0)+(c[Z>>2]|0)|0;c[Z>>2]=(aa|0)<536870911?aa:536870911;aa=$+1|0;if((aa|0)<(H|0)){_=$;$=aa}else{ac=$;ad=J;N=14970;break L20986}}c[D>>2]=(c[D>>2]|0)+(b[v+(ab<<1)>>1]|0);c[m+24+(_+A<<4)+4>>2]=c[m+24+(F<<4)+4>>2];Z=$+1|0;if((Z|0)<(H|0)){J=$;K=Z}else{ac=$;ad=$;N=14970;break}}}else{if(C){ae=0;af=1}else{ag=0;ah=0;N=14972;break}while(1){Z=ae+A|0;Y=m+24+(Z<<4)|0;X=m+24+(Z<<4)+8|0;W=m+24+(Z<<4)+12|0;V=u+(ae*48|0)|0;U=u+(ae*48|0)+4|0;T=u+(ae*48|0)+8|0;S=u+(ae*48|0)+12|0;R=u+(ae*48|0)+16|0;Q=u+(ae*48|0)+20|0;P=u+(ae*48|0)+24|0;O=u+(ae*48|0)+28|0;I=u+(ae*48|0)+32|0;E=u+(ae*48|0)+36|0;aa=u+(ae*48|0)+40|0;ai=u+(ae*48|0)+44|0;aj=ae;ak=af;while(1){al=c[Y>>2]|0;am=ak+A|0;if((al|0)!=(c[m+24+(am<<4)>>2]|0)){break}c[X>>2]=(c[X>>2]|0)+1;c[W>>2]=(c[W>>2]|0)+(c[m+24+(am<<4)+12>>2]|0);am=(c[u+(ak*48|0)>>2]|0)+(c[V>>2]|0)|0;c[V>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+4>>2]|0)+(c[U>>2]|0)|0;c[U>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+8>>2]|0)+(c[T>>2]|0)|0;c[T>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+12>>2]|0)+(c[S>>2]|0)|0;c[S>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+16>>2]|0)+(c[R>>2]|0)|0;c[R>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+20>>2]|0)+(c[Q>>2]|0)|0;c[Q>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+24>>2]|0)+(c[P>>2]|0)|0;c[P>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+28>>2]|0)+(c[O>>2]|0)|0;c[O>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+32>>2]|0)+(c[I>>2]|0)|0;c[I>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+36>>2]|0)+(c[E>>2]|0)|0;c[E>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+40>>2]|0)+(c[aa>>2]|0)|0;c[aa>>2]=(am|0)<536870911?am:536870911;am=(c[u+(ak*48|0)+44>>2]|0)+(c[ai>>2]|0)|0;c[ai>>2]=(am|0)<536870911?am:536870911;am=ak+1|0;if((am|0)<(H|0)){aj=ak;ak=am}else{ac=ak;ad=ae;N=14970;break L20986}}if((al|0)==11|(al|0)>15){an=5}else{an=b[v+(c[X>>2]<<1)>>1]|0}c[W>>2]=(c[W>>2]|0)+an;c[m+24+(aj+A<<4)+4>>2]=c[m+24+(Z<<4)+4>>2];ai=ak+1|0;if((ai|0)<(H|0)){ae=ak;af=ai}else{ac=ak;ad=ak;N=14970;break}}}}while(0);if((N|0)==14970){N=0;if(x){L=ad;M=ac;N=14971}else{ag=ad;ah=ac;N=14972}}if((N|0)==14971){N=0;ao=L+A|0;ap=L;aq=M;N=14973}else if((N|0)==14972){N=0;C=ag+A|0;ai=c[m+24+(C<<4)>>2]|0;if((ai|0)==11|(ai|0)>15){ar=5;as=ag;at=ah}else{ao=C;ap=ag;aq=ah;N=14973}}if((N|0)==14973){N=0;ar=b[v+(c[m+24+(ao<<4)+8>>2]<<1)>>1]|0;as=ap;at=aq}C=as+A|0;ai=m+24+(C<<4)+12|0;c[ai>>2]=(c[ai>>2]|0)+ar;c[m+24+(at+A<<4)+4>>2]=c[m+24+(C<<4)+4>>2];C=c[k>>2]|0;ai=m+24+(A<<4)+8|0;aa=c[ai>>2]|0;if((aa|0)<(C|0)){E=0;I=ai;O=aa;do{c[t+(E<<2)>>2]=Nd(B,u,v,E,O)|0;E=(c[I>>2]|0)+E|0;I=m+24+(E+A<<4)+8|0;O=(c[I>>2]|0)+E|0;}while((O|0)<(C|0));au=c[ai>>2]|0}else{au=aa}L21017:do{if((au|0)<(C|0)){O=A-1|0;E=0;I=0;P=au;Q=0;while(1){R=c[t+(E<<2)>>2]|0;S=(R|0)>(I|0);T=S?E:Q;U=S?R:I;R=E+P|0;S=c[m+24+(R+A<<4)+8>>2]|0;if((S+R|0)<(C|0)){E=R;I=U;P=S;Q=T;continue}if((U|0)<1){break L21017}S=T+A|0;R=m+24+(S<<4)+8|0;V=c[R>>2]|0;Y=V+T|0;$=Y+A|0;c[R>>2]=(c[m+24+($<<4)+8>>2]|0)+V;V=m+24+(S<<4)+12|0;c[V>>2]=(c[m+24+($<<4)+12>>2]|0)-U+(c[V>>2]|0);V=u+(T*48|0)|0;U=(c[u+(Y*48|0)>>2]|0)+(c[V>>2]|0)|0;c[V>>2]=(U|0)<536870911?U:536870911;U=u+(T*48|0)+4|0;V=(c[u+(Y*48|0)+4>>2]|0)+(c[U>>2]|0)|0;c[U>>2]=(V|0)<536870911?V:536870911;V=u+(T*48|0)+8|0;U=(c[u+(Y*48|0)+8>>2]|0)+(c[V>>2]|0)|0;c[V>>2]=(U|0)<536870911?U:536870911;U=u+(T*48|0)+12|0;V=(c[u+(Y*48|0)+12>>2]|0)+(c[U>>2]|0)|0;c[U>>2]=(V|0)<536870911?V:536870911;V=u+(T*48|0)+16|0;U=(c[u+(Y*48|0)+16>>2]|0)+(c[V>>2]|0)|0;c[V>>2]=(U|0)<536870911?U:536870911;U=u+(T*48|0)+20|0;V=(c[u+(Y*48|0)+20>>2]|0)+(c[U>>2]|0)|0;c[U>>2]=(V|0)<536870911?V:536870911;V=u+(T*48|0)+24|0;U=(c[u+(Y*48|0)+24>>2]|0)+(c[V>>2]|0)|0;c[V>>2]=(U|0)<536870911?U:536870911;U=u+(T*48|0)+28|0;V=(c[u+(Y*48|0)+28>>2]|0)+(c[U>>2]|0)|0;c[U>>2]=(V|0)<536870911?V:536870911;V=u+(T*48|0)+32|0;U=(c[u+(Y*48|0)+32>>2]|0)+(c[V>>2]|0)|0;c[V>>2]=(U|0)<536870911?U:536870911;U=u+(T*48|0)+36|0;V=(c[u+(Y*48|0)+36>>2]|0)+(c[U>>2]|0)|0;c[U>>2]=(V|0)<536870911?V:536870911;V=u+(T*48|0)+40|0;U=(c[u+(Y*48|0)+40>>2]|0)+(c[V>>2]|0)|0;c[V>>2]=(U|0)<536870911?U:536870911;U=u+(T*48|0)+44|0;V=(c[u+(Y*48|0)+44>>2]|0)+(c[U>>2]|0)|0;c[U>>2]=(V|0)<536870911?V:536870911;if((T|0)!=0){V=c[m+24+(T+O<<4)+4>>2]|0;c[t+(V<<2)>>2]=Nd(B,u,v,V,T)|0}V=(c[R>>2]|0)+T|0;c[m+24+(V+O<<4)+4>>2]=c[m+24+(S<<4)+4>>2];if((V|0)<(C|0)){c[t+(T<<2)>>2]=Nd(B,u,v,T,V)|0}V=c[ai>>2]|0;if((V|0)<(C|0)){E=0;I=0;P=V;Q=0}else{break}}}}while(0);C=c[k>>2]|0;L21030:do{if((C|0)>0){if(x){ai=0;while(1){B=ai+A|0;aa=m+24+(B<<4)|0;Q=aa|0;if(((c[Q>>2]|0)-13|0)>>>0<3){c[m+24+(B<<4)+12>>2]=0;av=c[m+24+(B<<4)+8>>2]|0}else{P=u+(ai*48|0)|0;Nc(P,Q)|0;Q=c[m+24+(B<<4)+8>>2]|0;c[q>>2]=(c[m+24+(B<<4)+12>>2]|0)-(b[v+(Q<<1)>>1]|0)+(c[q>>2]|0);av=Q}Q=m+24+(B<<4)+4|0;c[Q>>2]=(c[Q>>2]|0)+y;c[w>>2]=(c[w>>2]|0)+(b[v+(av<<1)>>1]|0);Q=c[h>>2]|0;c[h>>2]=Q+1;P=m+24+(Q<<4)|0;Q=aa;c[P>>2]=c[Q>>2];c[P+4>>2]=c[Q+4>>2];c[P+8>>2]=c[Q+8>>2];c[P+12>>2]=c[Q+12>>2];Q=(c[m+24+(B<<4)+8>>2]|0)+ai|0;B=c[k>>2]|0;if((Q|0)<(B|0)){ai=Q}else{aw=B;break L21030}}}else{ax=0}while(1){ai=ax+A|0;B=m+24+(ai<<4)|0;Q=B|0;P=c[Q>>2]|0;if((P-13|0)>>>0<3){c[m+24+(ai<<4)+12>>2]=0;ay=P}else{P=u+(ax*48|0)|0;Nc(P,Q)|0;P=c[Q>>2]|0;if((P|0)==11|(P|0)>15){az=5}else{az=b[v+(c[m+24+(ai<<4)+8>>2]<<1)>>1]|0}c[q>>2]=(c[m+24+(ai<<4)+12>>2]|0)-az+(c[q>>2]|0);ay=P}P=m+24+(ai<<4)+4|0;c[P>>2]=(c[P>>2]|0)+y;P=m+24+(ai<<4)+8|0;if((ay|0)==11|(ay|0)>15){aA=5}else{aA=b[v+(c[P>>2]<<1)>>1]|0}c[w>>2]=(c[w>>2]|0)+aA;ai=c[h>>2]|0;c[h>>2]=ai+1;Q=m+24+(ai<<4)|0;ai=B;c[Q>>2]=c[ai>>2];c[Q+4>>2]=c[ai+4>>2];c[Q+8>>2]=c[ai+8>>2];c[Q+12>>2]=c[ai+12>>2];ai=(c[P>>2]|0)+ax|0;P=c[k>>2]|0;if((ai|0)<(P|0)){ax=ai}else{aw=P;break}}}else{aw=C}}while(0);C=(c[s>>2]|0)+y|0;if((C|0)>=(c[r>>2]|0)){break L20965}y=C;z=aw;A=c[h>>2]|0}}}while(0);aw=m+992|0;c[aw>>2]=0;if((g|0)==0){aB=0;aC=c[h>>2]|0}else{r=m+1e3|0;c[r>>2]=0;s=c[h>>2]|0;h=0;while(1){if((h|0)>=(s|0)){aD=0;break}if((c[m+24+(h<<4)>>2]|0)==0){h=h+1|0}else{N=15009;break}}if((N|0)==15009){N=c[m+24+(h<<4)+4>>2]|0;c[r>>2]=N;aD=c[g+(N<<2)>>2]|0}if((s|0)>0){aE=aD;aF=0;aG=0;aH=0;aI=0;aJ=0;aK=0}else{c[m+996>>2]=0;aL=0;aM=0;aN=c[q>>2]|0;aO=c[w>>2]|0;aP=aO+aN|0;aQ=aP+aM|0;aR=aQ+aL|0;return aR|0}while(1){aD=c[m+24+(aI<<4)>>2]|0;do{if((aD|0)==14|(aD|0)==15){N=m+24+(aI<<4)+8|0;if((c[N>>2]|0)<=0){aS=aH;aT=aG;aU=aF;aV=aE;aW=aJ;aX=aK;break}r=c[m+24+(aI<<4)+4>>2]|0;h=aH;ax=r;k=aJ;while(1){aA=c[p+(ax<<2)>>2]|0;v=(d[1979952+(60-h+aA)|0]|0)+k|0;c[aw>>2]=v;ay=ax+1|0;if((ay|0)<((c[N>>2]|0)+r|0)){h=aA;ax=ay;k=v}else{aS=aA;aT=aG;aU=aF;aV=aE;aW=v;aX=v;break}}}else if((aD|0)==0|(aD|0)==13){aS=aH;aT=aG;aU=aF;aV=aE;aW=aJ;aX=aK}else{k=c[m+24+(aI<<4)+4>>2]|0;ax=c[m+24+(aI<<4)+8>>2]|0;h=ax+k|0;if((ax|0)<=0){aS=aH;aT=aG;aU=aF;aV=aE;aW=aJ;aX=aK;break}ax=h-1|0;r=aE;N=aF;v=aG;aA=k;k=aJ;while(1){do{if((c[f+(aA<<2)>>2]|0)==0){if((v|0)!=0){aY=v-1|0;aZ=0;a_=r;break}L21078:do{if((aA|0)==(ax|0)){a$=0;a0=0;a1=N;a2=r}else{ay=aA;az=0;while(1){a3=ay+1|0;if((a3|0)>=(h|0)){a$=az;a0=0;a1=N;a2=r;break L21078}if((c[f+(a3<<2)>>2]|0)==0){ay=a3;az=az+1|0}else{break}}ay=(c[g+(a3<<2)>>2]|0)-r|0;if((((ay|0)>0?ay:-ay|0)|0)<61){a$=az;a0=1;a1=0;a2=r;break}ay=c[g+(aA<<2)>>2]|0;a$=0;a0=1;a1=r-ay|0;a2=ay}}while(0);ak=g+(aA<<2)|0;Z=a$;aj=a0;W=a1;X=a2;ay=aI;L21085:while(1){u=Z;av=aj;a4=W;t=ay;while(1){au=t+1|0;at=(av|0)==0;if((au|0)<(s|0)&at){a5=u;a6=au}else{a7=u;a8=at;break L21085}L21089:while(1){a9=a6;while(1){at=c[m+24+(a9<<4)>>2]|0;if(!((at|0)==0|(at|0)==13)){break}at=a9+1|0;if((at|0)<(s|0)){a9=at}else{a7=a5;a8=1;break L21085}}at=c[m+24+(a9<<4)+4>>2]|0;au=c[m+24+(a9<<4)+8>>2]|0;ar=au+at|0;if((au|0)>0){ba=at;bb=a5;while(1){if((c[f+(ba<<2)>>2]|0)!=0){break L21089}at=bb+1|0;au=ba+1|0;if((au|0)<(ar|0)){ba=au;bb=at}else{bc=at;break}}}else{bc=a5}ar=a9+1|0;if((ar|0)<(s|0)){a5=bc;a6=ar}else{a7=bc;a8=1;break L21085}}ar=(c[g+(ba<<2)>>2]|0)-X|0;if((((ar|0)>0?ar:-ar|0)|0)<61){u=bb;av=1;a4=0;t=a9}else{break}}t=c[ak>>2]|0;Z=0;aj=1;W=X-t|0;X=t;ay=a9}aY=a8?0:a7;aZ=a8?0:a4;a_=X}else{ay=c[g+(aA<<2)>>2]|0;aY=v;aZ=r-ay|0;a_=ay}}while(0);ay=(d[aZ+1980012|0]|0)+k|0;c[aw>>2]=ay;W=aA+1|0;if((W|0)<(h|0)){r=a_;N=aZ;v=aY;aA=W;k=ay}else{aS=aH;aT=aY;aU=aZ;aV=a_;aW=ay;aX=ay;break}}}}while(0);aD=aI+1|0;if((aD|0)<(s|0)){aE=aV;aF=aU;aG=aT;aH=aS;aI=aD;aJ=aW;aK=aX}else{aB=aX;aC=s;break}}}s=m+996|0;c[s>>2]=0;if((aC|0)>0){bd=0;be=0;bf=1;bg=0;bh=0}else{aL=0;aM=aB;aN=c[q>>2]|0;aO=c[w>>2]|0;aP=aO+aN|0;aQ=aP+aM|0;aR=aQ+aL|0;return aR|0}while(1){do{if((c[m+24+(bd<<4)>>2]|0)==13){aX=c[m+24+(bd<<4)+4>>2]|0;aK=c[m+24+(bd<<4)+8>>2]|0;aW=aK+aX|0;if((aK|0)>0){bi=aX;bj=be;bk=bf;bl=bg}else{bm=bf;bn=be;bo=bg;bp=bh;break}while(1){if((bk|0)==0){aX=c[n+(bi<<2)>>2]|0;aK=(d[1979952+(60-bj+aX)|0]|0)+bl|0;c[s>>2]=aK;bq=aX;br=aK}else{aK=bl+9|0;c[s>>2]=aK;bq=c[n+(bi<<2)>>2]|0;br=aK}aK=bi+1|0;if((aK|0)<(aW|0)){bi=aK;bj=bq;bk=0;bl=br}else{bm=0;bn=bq;bo=br;bp=br;break}}}else{bm=bf;bn=be;bo=bg;bp=bh}}while(0);aW=bd+1|0;if((aW|0)<(aC|0)){bd=aW;be=bn;bf=bm;bg=bo;bh=bp}else{aL=bp;aM=aB;break}}aN=c[q>>2]|0;aO=c[w>>2]|0;aP=aO+aN|0;aQ=aP+aM|0;aR=aQ+aL|0;return aR|0}function Na(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0;d=J8(0)|0;e=(d|0)==0;do{if(!e){c[a>>2]=d;f=d|0;c[f>>2]=Ka(0,b)|0;g=Kb(0,b)|0;c[d+4>>2]=g;if((c[f>>2]|0)==0|(g|0)==0){h=1}else{break}return h|0}}while(0);h=e&1;return h|0}function Nb(a){a=a|0;if((c[a>>2]|0)==0){return}J9(a);return}function Nc(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0;d=c[a>>2]|0;if((d|0)<536870911){c[b>>2]=0;e=d}else{e=536870911}d=c[a+4>>2]|0;if((d|0)<(e|0)){c[b>>2]=1;f=d}else{f=e}e=c[a+8>>2]|0;if((e|0)<(f|0)){c[b>>2]=2;g=e}else{g=f}f=c[a+12>>2]|0;if((f|0)<(g|0)){c[b>>2]=3;h=f}else{h=g}g=c[a+16>>2]|0;if((g|0)<(h|0)){c[b>>2]=4;i=g}else{i=h}h=c[a+20>>2]|0;if((h|0)<(i|0)){c[b>>2]=5;j=h}else{j=i}i=c[a+24>>2]|0;if((i|0)<(j|0)){c[b>>2]=6;k=i}else{k=j}j=c[a+28>>2]|0;if((j|0)<(k|0)){c[b>>2]=7;l=j}else{l=k}k=c[a+32>>2]|0;if((k|0)<(l|0)){c[b>>2]=8;m=k}else{m=l}l=c[a+36>>2]|0;if((l|0)<(m|0)){c[b>>2]=9;n=l}else{n=m}m=c[a+40>>2]|0;if((m|0)<(n|0)){c[b>>2]=10;o=m}else{o=n}n=c[a+44>>2]|0;if((n|0)>=(o|0)){p=o;return p|0}c[b>>2]=11;p=n;return p|0}function Nd(a,d,e,f,g){a=a|0;d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0,k=0;h=(c[d+(g*48|0)>>2]|0)+(c[d+(f*48|0)>>2]|0)|0;i=(h|0)<536870911?h:536870911;h=(c[d+(g*48|0)+4>>2]|0)+(c[d+(f*48|0)+4>>2]|0)|0;j=(h|0)<(i|0)?h:i;i=(c[d+(g*48|0)+8>>2]|0)+(c[d+(f*48|0)+8>>2]|0)|0;h=(i|0)<(j|0)?i:j;j=(c[d+(g*48|0)+12>>2]|0)+(c[d+(f*48|0)+12>>2]|0)|0;i=(j|0)<(h|0)?j:h;h=(c[d+(g*48|0)+16>>2]|0)+(c[d+(f*48|0)+16>>2]|0)|0;j=(h|0)<(i|0)?h:i;i=(c[d+(g*48|0)+20>>2]|0)+(c[d+(f*48|0)+20>>2]|0)|0;h=(i|0)<(j|0)?i:j;j=(c[d+(g*48|0)+24>>2]|0)+(c[d+(f*48|0)+24>>2]|0)|0;i=(j|0)<(h|0)?j:h;h=(c[d+(g*48|0)+28>>2]|0)+(c[d+(f*48|0)+28>>2]|0)|0;j=(h|0)<(i|0)?h:i;i=(c[d+(g*48|0)+32>>2]|0)+(c[d+(f*48|0)+32>>2]|0)|0;h=(i|0)<(j|0)?i:j;j=(c[d+(g*48|0)+36>>2]|0)+(c[d+(f*48|0)+36>>2]|0)|0;i=(j|0)<(h|0)?j:h;h=(c[d+(g*48|0)+40>>2]|0)+(c[d+(f*48|0)+40>>2]|0)|0;j=(h|0)<(i|0)?h:i;i=(c[d+(g*48|0)+44>>2]|0)+(c[d+(f*48|0)+44>>2]|0)|0;d=c[a+(f<<4)>>2]|0;do{if((d|0)!=13){h=c[a+(g<<4)>>2]|0;if((h|0)==13|(d|0)==14|(h|0)==14|(d|0)==15|(h|0)==15){break}else{k=(c[a+(g<<4)+12>>2]|0)-((i|0)<(j|0)?i:j)-(b[e+((c[a+(f<<4)+8>>2]|0)+(c[a+(g<<4)+8>>2]|0)<<1)>>1]|0)+(c[a+(f<<4)+12>>2]|0)|0}return k|0}}while(0);k=-1;return k|0}function Ne(a,b,d,e,f,g,h,j,k,l,m,n,o,p,q){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;var r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0;r=i;i=i+4096|0;s=r|0;t=(q|0)/8|0;u=0;v=0;while(1){w=ac(u,t)|0;x=h;L21183:while(1){y=x-1|0;if((y|0)<(v|0)){z=v;break}A=c[j+(y<<2)>>2]|0;B=c[j+(x<<2)>>2]|0;while(1){C=B-1|0;if((C|0)<(A|0)){x=y;continue L21183}if((c[a+(C+w<<2)>>2]|0)==0){B=C}else{z=y;break L21183}}}w=u+1|0;if((w|0)<8){u=w;v=z}else{break}}c[m>>2]=(z|0)>0?z+1|0:1;z=(o|0)>0;if(!z){c[l>>2]=q;D=a;E=s;F=q<<2;IY(D,E,F);i=r;return}m=h+1|0;v=(m|0)>0;u=0;w=0;x=0;do{y=p+(u<<2)|0;if(v){B=0;A=x;while(1){c[l+(A<<2)>>2]=(ac(c[y>>2]|0,c[j+(B<<2)>>2]|0)|0)+w;C=B+1|0;if((C|0)<(m|0)){B=C;A=A+1|0}else{break}}G=m;H=m+x|0}else{G=0;H=x}x=g-G+H|0;w=(ac(c[y>>2]|0,t)|0)+w|0;u=u+1|0;}while((u|0)<(o|0));c[l+(x<<2)>>2]=q;if(!z){D=a;E=s;F=q<<2;IY(D,E,F);i=r;return}do{if((h|0)>0){x=0;l=0;while(1){u=0;w=l;while(1){c[n+(w<<2)>>2]=c[k+(u<<2)>>2];H=u+1|0;if((H|0)<(h|0)){u=H;w=w+1|0}else{break}}w=x+1|0;if((w|0)<(o|0)){x=w;l=l+g|0}else{break}}if(z){break}D=a;E=s;F=q<<2;IY(D,E,F);i=r;return}}while(0);k=(h|0)>0;n=b;l=0;x=0;y=0;while(1){w=p+(l<<2)|0;if(k){u=0;H=y;while(1){G=c[b+(x*60|0)+(u<<2)>>2]|0;m=c[w>>2]|0;if((m|0)>1){v=1;A=G;while(1){B=c[b+((v+x|0)*60|0)+(u<<2)>>2]|0;C=(2147483647-B|0)>(A|0)?B+A|0:2147483647;B=v+1|0;if((B|0)<(m|0)){v=B;A=C}else{I=C;break}}}else{I=G}c[n+(H<<2)>>2]=I;A=u+1|0;if((A|0)<(h|0)){u=A;H=H+1|0}else{break}}J=h;K=y+h|0}else{J=0;K=y}H=l+1|0;if((H|0)<(o|0)){l=H;x=(c[w>>2]|0)+x|0;y=g-J+K|0}else{break}}if(!z){D=a;E=s;F=q<<2;IY(D,E,F);i=r;return}K=(h|0)>0;J=d;y=0;x=0;l=0;while(1){I=p+(y<<2)|0;if(K){n=0;b=l;while(1){k=c[d+(x*60|0)+(n<<2)>>2]|0;H=c[I>>2]|0;if((H|0)>1){u=1;A=k;while(1){v=c[d+((u+x|0)*60|0)+(n<<2)>>2]|0;m=(2147483647-v|0)>(A|0)?v+A|0:2147483647;v=u+1|0;if((v|0)<(H|0)){u=v;A=m}else{L=m;break}}}else{L=k}c[J+(b<<2)>>2]=L;A=n+1|0;if((A|0)<(h|0)){n=A;b=b+1|0}else{break}}M=h;N=l+h|0}else{M=0;N=l}b=y+1|0;if((b|0)<(o|0)){y=b;x=(c[I>>2]|0)+x|0;l=g-M+N|0}else{break}}if(!z){D=a;E=s;F=q<<2;IY(D,E,F);i=r;return}N=(h|0)>0;M=e;l=0;x=0;y=0;while(1){L=p+(l<<2)|0;if(N){J=0;d=y;while(1){K=c[e+(x*60|0)+(J<<2)>>2]|0;b=c[L>>2]|0;if((b|0)>1){n=1;w=K;while(1){A=c[e+((n+x|0)*60|0)+(J<<2)>>2]|0;u=(2147483647-A|0)>(w|0)?A+w|0:2147483647;A=n+1|0;if((A|0)<(b|0)){n=A;w=u}else{O=u;break}}}else{O=K}c[M+(d<<2)>>2]=O;w=J+1|0;if((w|0)<(h|0)){J=w;d=d+1|0}else{break}}P=h;Q=y+h|0}else{P=0;Q=y}d=l+1|0;if((d|0)<(o|0)){l=d;x=(c[L>>2]|0)+x|0;y=g-P+Q|0}else{break}}if(!z){D=a;E=s;F=q<<2;IY(D,E,F);i=r;return}Q=(h|0)>0;P=f;y=0;x=0;l=0;while(1){O=p+(y<<2)|0;if(Q){M=0;e=l;while(1){N=c[f+(x*60|0)+(M<<2)>>2]|0;d=c[O>>2]|0;if((d|0)>1){J=1;I=N;while(1){w=c[f+((J+x|0)*60|0)+(M<<2)>>2]|0;n=(2147483647-w|0)>(I|0)?w+I|0:2147483647;w=J+1|0;if((w|0)<(d|0)){J=w;I=n}else{R=n;break}}}else{R=N}c[P+(e<<2)>>2]=R;I=M+1|0;if((I|0)<(h|0)){M=I;e=e+1|0}else{break}}S=h;T=l+h|0}else{S=0;T=l}e=y+1|0;if((e|0)<(o|0)){y=e;x=(c[O>>2]|0)+x|0;l=g-S+T|0}else{break}}if(!z){D=a;E=s;F=q<<2;IY(D,E,F);i=r;return}z=j+(g<<2)|0;if((h|0)>0){U=0;V=0;W=0}else{D=a;E=s;F=q<<2;IY(D,E,F);i=r;return}while(1){g=ac(V,t)|0;T=c[p+(U<<2)>>2]|0;S=0;l=W;x=c[j>>2]|0;while(1){y=S+1|0;R=c[j+(y<<2)>>2]|0;P=R-x|0;L21281:do{if((T|0)>0){if((P|0)>0){X=0;Y=l;Z=a+(x+g<<2)|0}else{f=0;while(1){Q=f+1|0;if((Q|0)<(T|0)){f=Q}else{_=l;break L21281}}}while(1){f=P;Q=Y;e=Z;while(1){c[s+(Q<<2)>>2]=c[e>>2];M=f-1|0;if((M|0)>0){f=M;Q=Q+1|0;e=e+4|0}else{break}}e=P+Y|0;Q=X+1|0;if((Q|0)<(T|0)){X=Q;Y=e;Z=Z+(t<<2)|0}else{_=e;break}}}else{_=l}}while(0);if((y|0)<(h|0)){S=y;l=_;x=R}else{break}}x=(ac((c[z>>2]|0)-(c[j+(h<<2)>>2]|0)|0,T)|0)+_|0;l=U+1|0;if((l|0)<(o|0)){U=l;V=T+V|0;W=x}else{break}}D=a;E=s;F=q<<2;IY(D,E,F);i=r;return}function Nf(a,b,d,e,f,g,h,j,k,l,m,n,o,p,q,r,s,t,u,v,w){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;n=n|0;o=o|0;p=p|0;q=q|0;r=r|0;s=s|0;t=t|0;u=u|0;v=v|0;w=w|0;var x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0,aJ=0,aK=0,aL=0,aM=0,aN=0,aO=0,aP=0,aQ=0,aR=0,aS=0,aT=0,aU=0,aV=0,aW=0,aX=0,aY=0,aZ=0,a_=0,a$=0,a0=0,a1=0,a2=0;j=i;i=i+1248|0;x=j|0;y=j+16|0;z=j+256|0;A=j+264|0;B=j+272|0;C=j+512|0;D=j+752|0;E=j+992|0;F=j+1232|0;H=j+1240|0;I=p<<2;I_(u,I);I_(E,I);J=D;I_(J,I);I_(v,I);I_(B,I);if((t|0)==0){i=j;return}I_(y,240);I_(C,240);I_(x,16);I_(J,240);J=(p|0)>0;if(!J){i=j;return}t=(r|0)>0;if(t){I=0;K=0;while(1){L=x+(I<<2)|0;c[L>>2]=0;M=0;do{N=M+K|0;O=(c[l+(N<<2)>>2]|0)-(c[m+(N<<2)>>2]|0)|0;P=(O|0)<-1006632960?-1006632960:O;c[D+(N<<2)>>2]=(P|0)>1006632960?1006632960:P;P=a+(N<<2)|0;O=c[P>>2]|0;if((O|0)>-1){Q=~O;R=0;while(1){S=Q<<1;if((S|0)<0){Q=S;R=R+1|0}else{T=R;break}}}else{T=-1}R=(T|0)<0?0:T;Q=b+(N<<2)|0;S=c[Q>>2]|0;if((S|0)>-1){U=~S;V=0;while(1){W=U<<1;if((W|0)<0){U=W;V=V+1|0}else{X=V;break}}}else{X=-1}V=(X|0)<0?0:X;U=(R|0)<(V|0)?R:V;V=U&-4;W=(HN(HN((O<<V>>1)+(S<<V>>1)|0)|0)|0)>>(U>>2);c[C+(N<<2)>>2]=W;c[L>>2]=(W>>6)+(c[L>>2]|0);W=c[P>>2]|0;U=c[Q>>2]|0;N2(U,(U|0)<0|0?-1:0,1503238528,0)|0;do{if((W|0)>=(G<<1|0>>>31|0)){V=W;Y=(W|0)<0|0?-1:0;Z=1503238528;_=0;N2(V,Y,Z,_)|0;if((G<<1|0>>>31|0)>(U|0)){break}c[B+(N<<2)>>2]=268435456}}while(0);M=M+1|0;}while((M|0)<(r|0));M=K+q|0;if((M|0)<(p|0)){I=I+1|0;K=M}else{$=0;aa=0;break}}}else{K=0;I=0;while(1){c[x+(K<<2)>>2]=0;X=I+q|0;if((X|0)<(p|0)){K=K+1|0;I=X}else{$=0;aa=0;break}}}while(1){I=c[x+($<<2)>>2]|0;if((I|0)==0){ab=0;ac=0;ad=0}else{K=HQ(2147483647,I,z)|0;ab=(K|0)<0|0?-1:0;ac=K;ad=(c[z>>2]|0)-5|0}K=(ad|0)>-31?ad:-31;I=(K|0)<31?K:31;c[z>>2]=I;L21328:do{if(t){K=0;X=I;while(1){T=K+aa|0;l=C+(T<<2)|0;M=c[l>>2]&-64;N2(M,(M|0)<0|0?-1:0,ac,ab)|0;M=G;if((X|0)>0){ae=M<<X}else{ae=M>>-X}c[l>>2]=ae;M=y+(T<<2)|0;c[M>>2]=0;L=c[s+(T+1<<2)>>2]|0;N=c[s+(T<<2)>>2]|0;U=c[172720+(L-N>>1<<2)>>2]|0;do{if((U|0)>0){W=(N|0)<(L|0);if(W){af=0;ag=N}else{ah=0;break}do{Q=c[d+(ag<<2)>>2]|0;P=(Q|0)>0?Q:-Q|0;af=(af|0)>(P|0)?af:P;ag=ag+1|0;}while((ag|0)<(L|0));do{if((af|0)==0){ai=30}else{if((af|0)<=-1){ai=-1;break}P=~af;Q=0;while(1){S=P<<1;if((S|0)<0){P=S;Q=Q+1|0}else{ai=Q;break}}}}while(0);Q=0;P=N;do{S=c[e+(P<<2)>>2]|0;O=(S|0)>0?S:-S|0;Q=(Q|0)>(O|0)?Q:O;P=P+1|0;}while((P|0)<(L|0));do{if((Q|0)==0){aj=30}else{if((Q|0)<=-1){aj=-1;break}P=~Q;O=0;while(1){S=P<<1;if((S|0)<0){P=S;O=O+1|0}else{aj=O;break}}}}while(0);Q=(ai|0)<(aj|0)?ai:aj;O=U;P=(U|0)<0|0?-1:0;S=0;R=0;_=N;do{Z=c[d+(_<<2)>>2]<<Q;N2(Z,(Z|0)<0|0?-1:0,O,P)|0;R=G+R|0;Z=c[e+(_<<2)>>2]<<Q;N2(Z,(Z|0)<0|0?-1:0,O,P)|0;S=G+S|0;_=_+1|0;}while((_|0)<(L|0));N2(R,(R|0)<0|0?-1:0,O,P)|0;_=G;N2(S,(S|0)<0|0?-1:0,O,P)|0;Z=G;if(W){ak=0;al=0;am=0;an=N}else{ah=0;break}do{Y=c[d+(an<<2)>>2]<<Q;N2(Y,(Y|0)<0|0?-1:0,O,P)|0;Y=G-_|0;V=c[e+(an<<2)>>2]<<Q;N2(V,(V|0)<0|0?-1:0,O,P)|0;V=G-Z|0;ao=Y;ap=(Y|0)<0|0?-1:0;Y=V;aq=(V|0)<0|0?-1:0;N2(Y,aq,ao,ap)|0;am=G+am|0;N2(ao,ap,ao,ap)|0;ak=G+ak|0;N2(Y,aq,Y,aq)|0;al=G+al|0;an=an+1|0;}while((an|0)<(L|0));Z=am<<1;P=ak<<1;O=al<<1;if(!((P|0)>0&(O|0)>0)){ah=0;break}c[A>>2]=0;if((P|0)>-1){Q=~P;_=0;while(1){W=Q<<1;if((W|0)<0){Q=W;_=_+1|0}else{ar=_;break}}}else{ar=-1}_=(ar|0)<0?0:ar;if((O|0)>-1){Q=~O;W=0;while(1){S=Q<<1;if((S|0)<0){Q=S;W=W+1|0}else{as=W;break}}}else{as=-1}W=((as|0)<0?0:as)+_|0;Q=W&-2;S=(_|0)<(Q|0)?_:Q;R=P<<S;aq=O<<Q-S;N2(aq,(aq|0)<0|0?-1:0,R,(R|0)<0|0?-1:0)|0;R=HN(G<<1|0>>>31)|0;if((Z|0)<0){at=-(HQ(-Z|0,R,A)|0)|0}else{at=HQ(Z,R,A)|0}c[M>>2]=at;R=(c[A>>2]|0)+(W>>1)|0;W=(R|0)>-31?R:-31;R=(W|0)<31?W:31;c[A>>2]=R;if((R|0)<0){W=at>>-R;c[M>>2]=W;ah=W;break}if((((at|0)>0?at:-at|0)|0)<=(2147483647>>>(R>>>0)|0)){W=at<<R;c[M>>2]=W;ah=W;break}if((at|0)<0){c[M>>2]=-2147483647;ah=-2147483647;break}else{c[M>>2]=2147483647;ah=2147483647;break}}else{ah=0}}while(0);M=B+(T<<2)|0;do{if((c[M>>2]|0)!=268435456){L=536870912-(ah>>2)|0;N=c[l>>2]|0;U=L;W=(L|0)<0|0?-1:0;L=N;R=(N|0)<0|0?-1:0;N2(L,R,U,W)|0;c[M>>2]=G;if((((ah|0)>0?ah:-ah|0)|0)<2040109440){break}c[E+(T<<2)>>2]=1}}while(0);T=K+1|0;if((T|0)>=(r|0)){break L21328}K=T;X=c[z>>2]|0}}}while(0);I=aa+q|0;if((I|0)<(p|0)){$=$+1|0;aa=I}else{break}}aa=r-1|0;$=0;z=0;ah=0;while(1){if(t){at=0;A=z;as=0;ar=0;al=0;ak=ah;am=0;while(1){an=at+$|0;aj=E+(an<<2)|0;do{if((c[aj>>2]|0)==1){if((ar|0)==0){au=c[D+(an<<2)>>2]|0;av=an}else{au=ak;av=A}aw=av;ax=1;ay=(c[B+(an<<2)>>2]>>3)+al|0;az=au;aA=(c[C+(an<<2)>>2]>>6)+am|0;aB=15239}else{if((as|0)==0){aC=ak;aD=ar;aE=0;aF=A;aG=al;aH=am;break}ai=c[B+(an<<2)>>2]|0;af=(ai>>3)+al|0;ag=(c[C+(an<<2)>>2]>>6)+am|0;if(!((ai|0)<2684354&(af|0)<1342177)){aC=ak;aD=ar;aE=0;aF=A;aG=af;aH=ag;break}c[aj>>2]=1;aw=A;ax=as;ay=af;az=ak;aA=ag;aB=15239}}while(0);do{if((aB|0)==15239){aB=0;ag=c[D+(an<<2)>>2]|0;af=az-ag|0;if((((af|0)>0?af:-af|0)|0)<33554432){aC=ag;aD=ar+1|0;aE=ax;aF=aw;aG=ay;aH=aA;break}c[aj>>2]=0;aC=az;aD=ar;aE=0;aF=aw;aG=ay;aH=aA}}while(0);do{if((aD|0)>0){if(!((aE|0)==0|(at|0)==(aa|0))){aI=aH;aJ=aG;aK=aD;break}if((aD|0)<6){if((aF|0)>(an|0)){aI=0;aJ=0;aK=0;break}else{aL=aF}}else{if((aH|0)>3355442|(aF|0)>(an|0)){aI=0;aJ=0;aK=0;break}else{aL=aF}}while(1){c[E+(aL<<2)>>2]=0;aj=aL+1|0;if((aj|0)>(an|0)){aI=0;aJ=0;aK=0;break}else{aL=aj}}}else{aI=aH;aJ=aG;aK=aD}}while(0);an=at+1|0;if((an|0)<(r|0)){at=an;A=aF;as=aE;ar=aK;al=aJ;ak=aC;am=aI}else{aM=aF;aN=aC;break}}}else{aM=z;aN=ah}am=$+q|0;if((am|0)<(p|0)){$=am;z=aM;ah=aN}else{break}}if(!J){i=j;return}J=(r|0)>0;aN=w+4|0;ah=0;aM=0;while(1){if(J){z=ah;$=0;while(1){aC=$+aM|0;aF=o+(aC<<2)|0;c[aF>>2]=0;do{if((c[E+(aC<<2)>>2]|0)==0){aO=z}else{aI=a+(aC<<2)|0;if((c[aI>>2]|0)<(c[f+(aC<<2)>>2]|0)){aJ=c[b+(aC<<2)>>2]|0;aK=aJ;aE=(aJ|0)<0|0?-1:0;aJ=1431655808;aD=0;N2(aK,aE,aJ,aD)|0;if((G<<1|0>>>31|0)>(c[g+(aC<<2)>>2]|0)){aO=z;break}}aD=c[w>>2]|0;do{if((aD|0)!=0){aJ=aD+360+(aC<<2)|0;if((c[aJ>>2]|0)!=0){c[aJ>>2]=0}aJ=(c[aN>>2]|0)+360+(aC<<2)|0;if((c[aJ>>2]|0)==0){break}c[aJ>>2]=0}}while(0);aD=s+(aC+1<<2)|0;aJ=c[aD>>2]|0;aE=s+(aC<<2)|0;aK=c[aE>>2]|0;aG=c[172720+(aJ-aK>>1<<2)>>2]|0;aH=(aK|0)<(aJ|0);do{if(aH){aL=0;aa=aK;do{aA=c[d+(aa<<2)>>2]|0;ay=(aA|0)>0?aA:-aA|0;aL=(aL|0)>(ay|0)?aL:ay;aa=aa+1|0;}while((aa|0)<(aJ|0));do{if((aL|0)==0){aP=30}else{if((aL|0)<=-1){aP=-1;break}aa=~aL;Z=0;while(1){O=aa<<1;if((O|0)<0){aa=O;Z=Z+1|0}else{aP=Z;break}}}}while(0);if(aH){aQ=0;aR=aK}else{aS=aP;aT=30;aB=15275;break}do{aL=c[e+(aR<<2)>>2]|0;Z=(aL|0)>0?aL:-aL|0;aQ=(aQ|0)>(Z|0)?aQ:Z;aR=aR+1|0;}while((aR|0)<(aJ|0));do{if((aQ|0)==0){aU=30}else{if((aQ|0)<=-1){aU=-1;break}Z=~aQ;aL=0;while(1){aa=Z<<1;if((aa|0)<0){Z=aa;aL=aL+1|0}else{aU=aL;break}}}}while(0);if(!aH){aS=aP;aT=aU;aB=15275;break}aL=aG;Z=(aG|0)<0|0?-1:0;aa=0;O=aK;do{P=c[d+(O<<2)>>2]<<aP;_=c[e+(O<<2)>>2]<<aU;N2(_,(_|0)<0|0?-1:0,P,(P|0)<0|0?-1:0)|0;P=G;N2(P,(P|0)<0|0?-1:0,aL,Z)|0;aa=G+aa|0;O=O+1|0;}while((O|0)<(aJ|0));O=(aP|0)<(aU|0)?aP:aU;if((aa&1073741824|0)==0){aV=O;aB=15277;break}if(aH){Z=aK;aL=0;while(1){P=(c[d+(Z<<2)>>2]<<O>>1)-(c[e+(Z<<2)>>2]<<O>>1)|0;_=P;ay=(P|0)<0|0?-1:0;N2(_,ay,_,ay)|0;ay=(G>>>5|0<<27)+aL|0;_=Z+1|0;if((_|0)<(aJ|0)){Z=_;aL=ay}else{aW=ay;break}}}else{aW=0}c[aF>>2]=1;aL=HQ(c[aI>>2]|0,aW,F)|0;Z=(c[F>>2]|0)+(O<<1)|0;aa=Z-8|0;if((aa&1|0)==0){aX=aL;aY=aa}else{aX=aL>>1;aY=Z-7|0}Z=(aY>>1)+1|0;aL=(Z|0)>-31?Z:-31;Z=(aL|0)<31?aL:31;aL=HN(aX)|0;if((Z|0)<0){aa=-Z|0;ay=c[aE>>2]|0;if((ay|0)>=(c[aD>>2]|0)){break}_=aL;P=(aL|0)<0|0?-1:0;aA=ay;do{ay=d+(aA<<2)|0;aw=c[ay>>2]|0;N2(aw,(aw|0)<0|0?-1:0,_,P)|0;aw=G;az=e+(aA<<2)|0;ax=c[az>>2]|0;N2(ax,(ax|0)<0|0?-1:0,_,P)|0;c[ay>>2]=aw-G>>aa;c[az>>2]=0;aA=aA+1|0;}while((aA|0)<(c[aD>>2]|0))}else{aA=c[aE>>2]|0;if((aA|0)>=(c[aD>>2]|0)){break}aa=aL;P=(aL|0)<0|0?-1:0;_=aA;do{aA=d+(_<<2)|0;O=c[aA>>2]|0;N2(O,(O|0)<0|0?-1:0,aa,P)|0;O=G;az=e+(_<<2)|0;aw=c[az>>2]|0;N2(aw,(aw|0)<0|0?-1:0,aa,P)|0;c[aA>>2]=O-G<<Z;c[az>>2]=0;_=_+1|0;}while((_|0)<(c[aD>>2]|0))}}else{aS=30;aT=30;aB=15275}}while(0);if((aB|0)==15275){aB=0;aV=(aS|0)<(aT|0)?aS:aT;aB=15277}do{if((aB|0)==15277){aB=0;if(aH){aG=aK;_=0;while(1){Z=(c[e+(aG<<2)>>2]<<aV>>1)+(c[d+(aG<<2)>>2]<<aV>>1)|0;P=Z;aa=(Z|0)<0|0?-1:0;N2(P,aa,P,aa)|0;aa=(G>>>5|0<<27)+_|0;P=aG+1|0;if((P|0)<(aJ|0)){aG=P;_=aa}else{aZ=aa;break}}}else{aZ=0}c[aF>>2]=0;_=HQ(c[aI>>2]|0,aZ,H)|0;aG=(c[H>>2]|0)+(aV<<1)|0;aa=aG-8|0;if((aa&1|0)==0){a_=_;a$=aa}else{a_=_>>1;a$=aG-7|0}aG=(a$>>1)+1|0;_=(aG|0)>-31?aG:-31;aG=(_|0)<31?_:31;_=HN(a_)|0;if((aG|0)<0){aa=-aG|0;P=c[aE>>2]|0;if((P|0)>=(c[aD>>2]|0)){break}Z=_;aL=(_|0)<0|0?-1:0;az=P;do{P=d+(az<<2)|0;O=c[P>>2]|0;N2(O,(O|0)<0|0?-1:0,Z,aL)|0;O=G;aA=e+(az<<2)|0;aw=c[aA>>2]|0;N2(aw,(aw|0)<0|0?-1:0,Z,aL)|0;c[P>>2]=G+O>>aa;c[aA>>2]=0;az=az+1|0;}while((az|0)<(c[aD>>2]|0))}else{az=c[aE>>2]|0;if((az|0)>=(c[aD>>2]|0)){break}aa=_;aL=(_|0)<0|0?-1:0;Z=az;do{az=d+(Z<<2)|0;aA=c[az>>2]|0;N2(aA,(aA|0)<0|0?-1:0,aa,aL)|0;aA=G;O=e+(Z<<2)|0;P=c[O>>2]|0;N2(P,(P|0)<0|0?-1:0,aa,aL)|0;c[az>>2]=G+aA<<aG;c[O>>2]=0;Z=Z+1|0;}while((Z|0)<(c[aD>>2]|0))}}}while(0);c[u+(aC<<2)>>2]=15;aD=c[D+(aC<<2)>>2]|0;aE=aD>>1;if((aD|0)<0){aD=(aE-4194304>>23)+1|0;c[v+(aC<<2)>>2]=aD;a0=aD}else{aD=aE+4194304>>23;c[v+(aC<<2)>>2]=aD;a0=aD}aD=v+(aC<<2)|0;aE=a0-z|0;do{if((aE|0)<-60){aI=z-60|0;c[aD>>2]=aI;a1=aI}else{if((aE+60|0)<=120){a1=a0;break}aI=z+60|0;c[aD>>2]=aI;a1=aI}}while(0);c[b+(aC<<2)>>2]=0;c[m+(aC<<2)>>2]=-2147483648;c[g+(aC<<2)>>2]=0;c[h+(aC<<2)>>2]=-1107296256;c[k+(aC<<2)>>2]=0;c[n>>2]=1;aO=a1}}while(0);aC=$+1|0;if((aC|0)<(r|0)){z=aO;$=aC}else{a2=aO;break}}}else{a2=ah}$=aM+q|0;if(($|0)<(p|0)){ah=a2;aM=$}else{break}}i=j;return}function Ng(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0;if((g|0)<=0){return}if((i|0)>0){j=0}else{k=0;do{k=k+h|0;}while((k|0)<(g|0));return}do{k=0;do{l=k+j|0;m=c[b+(l<<2)>>2]|0;if((m|0)>(c[d+(l<<2)>>2]|0)){n=(c[f+(l+1<<2)>>2]|0)-(c[f+(l<<2)>>2]|0)|0;o=((HK(n)|0)>>1)+(-m>>1)>>1;m=HH((c[e+(l<<2)>>2]|0)+201326592+o|0)|0;c[a+(l<<2)>>2]=(n|0)<(m|0)?n:m}else{c[a+(l<<2)>>2]=0}k=k+1|0;}while((k|0)<(i|0));j=j+h|0;}while((j|0)<(g|0));return}function Nh(a,b,e,f,g,h,i,j){a=a|0;b=b|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0;k=a+960|0;c[k>>2]=0;l=a+964|0;c[l>>2]=0;m=a+968|0;c[m>>2]=0;if((f|0)<=0){n=0;o=0;p=n>>16;c[k>>2]=p;q=o>>16;c[l>>2]=q;return}r=(h|0)>0;s=0;t=0;u=0;v=0;while(1){if(r){w=0;x=s;while(1){y=w+t|0;z=b+(y<<2)|0;A=c[z>>2]|0;B=c[e+(y<<2)>>2]|0;do{if((A|0)>(B|0)){C=A-B|0;D=c[a+(y<<2)>>2]|0;if((C|0)>100663295){E=D<<23;F=C;H=(C|0)<0|0?-1:0;I=E;J=(E|0)<0|0?-1:0;N2(I,J,F,H)|0;H=G;c[a+240+(y<<2)>>2]=H;F=c[z>>2]|0;E=F;K=(F|0)<0|0?-1:0;N2(E,K,I,J)|0;c[a+480+(y<<2)>>2]=G;L=D;M=H}else{H=C;J=(C|0)<0|0?-1:0;C=1201210655;I=0;N2(H,J,C,I)|0;I=(G<<1|0>>>31)+44356547|0;C=D<<23;J=I;H=(I|0)<0|0?-1:0;I=C;K=(C|0)<0|0?-1:0;N2(I,K,J,H)|0;H=G;c[a+240+(y<<2)>>2]=H;J=c[z>>2]|0;C=J;E=(J|0)<0|0?-1:0;J=1201210655;F=0;N2(C,E,J,F)|0;F=(G<<1|0>>>31)+44356547|0;J=F;E=(F|0)<0|0?-1:0;N2(J,E,I,K)|0;c[a+480+(y<<2)>>2]=G;K=D<<16;D=K;I=(K|0)<0|0?-1:0;K=1201210655;E=0;N2(D,I,K,E)|0;L=G+16384>>15;M=H}c[a+720+(y<<2)>>2]=L;N=x;O=M;P=a+480+(y<<2)|0}else{if((c[i+(y<<2)>>2]|0)==0){c[a+240+(y<<2)>>2]=0;H=a+480+(y<<2)|0;c[H>>2]=0;c[a+720+(y<<2)>>2]=0;N=x;O=0;P=H;break}else{H=c[j+(y<<2)>>2]|0;E=(d[1979952+(60-x+H)|0]|0)<<16;c[a+240+(y<<2)>>2]=E;K=a+480+(y<<2)|0;c[K>>2]=0;c[a+720+(y<<2)>>2]=0;N=H;O=E;P=K;break}}}while(0);z=(c[k>>2]|0)+O|0;c[k>>2]=z;B=(c[l>>2]|0)+(c[P>>2]|0)|0;c[l>>2]=B;c[m>>2]=(c[m>>2]|0)+(c[a+720+(y<<2)>>2]|0);A=w+1|0;if((A|0)<(h|0)){w=A;x=N}else{Q=N;R=z;S=B;break}}}else{Q=s;R=u;S=v}x=t+g|0;if((x|0)<(f|0)){s=Q;t=x;u=R;v=S}else{n=R;o=S;break}}p=n>>16;c[k>>2]=p;q=o>>16;c[l>>2]=q;return}function Ni(a,b,d,e,f,g,h,i,j){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0;k=c[a>>2]|0;l=c[a+4>>2]|0;a=c[b>>2]|0;m=c[a+2116>>2]|0;n=c[b+4>>2]|0;b=c[n+2116>>2]|0;o=c[a+2108>>2]|0;a=c[n+2108>>2]|0;n=c[k>>2]|0;p=c[l>>2]|0;q=(g|0)>0;do{if(q){r=(i|0)>0;s=(d|0)==0;t=0;u=0;v=0;while(1){L21559:do{if(r){if(s){w=t;x=u;y=0;while(1){z=y+v|0;A=o+(z<<2)|0;B=c[A>>2]|0;C=a+(z<<2)|0;D=c[C>>2]|0;E=(B|0)<(D|0)?B:D;F=m+(z<<2)|0;G=c[F>>2]|0;H=b+(z<<2)|0;I=c[H>>2]|0;J=k+2404+(z<<2)|0;K=c[J>>2]|0;L=l+2404+(z<<2)|0;M=c[L>>2]|0;N=f+(z<<2)|0;if((E-(((K|0)>(E|0)?K:E)>>1)-(((M|0)>(E|0)?M:E)>>1)|0)>((D>>1)+(B>>1)-(((G|0)>(B|0)?G:B)>>1)-(((I|0)>(D|0)?I:D)>>1)|0)){c[N>>2]=1;D=c[j+(z<<2)>>2]|0;I=j+(z+1<<2)|0;if((D|0)<(c[I>>2]|0)){B=D;do{D=n+(B<<2)|0;G=c[D>>2]>>1;M=p+(B<<2)|0;K=c[M>>2]>>1;c[D>>2]=K+G;c[M>>2]=G-K;B=B+1|0;}while((B|0)<(c[I>>2]|0))}I=k+4+(z<<2)|0;B=c[I>>2]|0;K=l+4+(z<<2)|0;G=c[K>>2]|0;M=(B|0)<(G|0)?B:G;c[K>>2]=M;c[I>>2]=M;c[C>>2]=E;c[A>>2]=E;c[k+484+(z<<2)>>2]=c[k+1924+(z<<2)>>2];c[l+484+(z<<2)>>2]=c[l+1924+(z<<2)>>2];c[F>>2]=c[J>>2];c[H>>2]=c[L>>2];M=k+2644+(z<<2)|0;I=c[M>>2]|0;K=l+2644+(z<<2)|0;G=c[K>>2]|0;B=((I|0)<(G|0)?I:G)>>1;c[K>>2]=B;c[M>>2]=B;O=x;P=1}else{c[N>>2]=0;O=x+1|0;P=w}B=y+1|0;if((B|0)<(i|0)){w=P;x=O;y=B}else{Q=P;R=O;break L21559}}}else{S=t;T=u;U=0}while(1){y=U+v|0;do{if((c[d+(y<<2)>>2]|0)==0){x=o+(y<<2)|0;w=c[x>>2]|0;B=a+(y<<2)|0;M=c[B>>2]|0;K=(w|0)<(M|0)?w:M;G=m+(y<<2)|0;I=c[G>>2]|0;D=b+(y<<2)|0;V=c[D>>2]|0;W=k+2404+(y<<2)|0;X=c[W>>2]|0;Y=l+2404+(y<<2)|0;Z=c[Y>>2]|0;_=f+(y<<2)|0;if((K-(((X|0)>(K|0)?X:K)>>1)-(((Z|0)>(K|0)?Z:K)>>1)|0)<=((M>>1)+(w>>1)-(((I|0)>(w|0)?I:w)>>1)-(((V|0)>(M|0)?V:M)>>1)|0)){c[_>>2]=0;$=T+1|0;aa=S;break}c[_>>2]=1;_=c[j+(y<<2)>>2]|0;M=j+(y+1<<2)|0;if((_|0)<(c[M>>2]|0)){V=_;do{_=n+(V<<2)|0;w=c[_>>2]>>1;I=p+(V<<2)|0;Z=c[I>>2]>>1;c[_>>2]=Z+w;c[I>>2]=w-Z;V=V+1|0;}while((V|0)<(c[M>>2]|0))}M=k+4+(y<<2)|0;V=c[M>>2]|0;Z=l+4+(y<<2)|0;w=c[Z>>2]|0;I=(V|0)<(w|0)?V:w;c[Z>>2]=I;c[M>>2]=I;c[B>>2]=K;c[x>>2]=K;c[k+484+(y<<2)>>2]=c[k+1924+(y<<2)>>2];c[l+484+(y<<2)>>2]=c[l+1924+(y<<2)>>2];c[G>>2]=c[W>>2];c[D>>2]=c[Y>>2];I=k+2644+(y<<2)|0;M=c[I>>2]|0;Z=l+2644+(y<<2)|0;w=c[Z>>2]|0;V=((M|0)<(w|0)?M:w)>>1;c[Z>>2]=V;c[I>>2]=V;$=T;aa=1}else{$=9;aa=(c[f+(y<<2)>>2]|0)==0?S:1}}while(0);y=U+1|0;if((y|0)<(i|0)){S=aa;T=$;U=y}else{Q=aa;R=$;break}}}else{Q=t;R=u}}while(0);y=v+h|0;if((y|0)<(g|0)){t=Q;u=R;v=y}else{break}}if((Q|0)!=1){break}do{if((R|0)!=0){if((R|0)<(i|0)&(R|0)<9){break}c[e>>2]=1;return}}while(0);c[e>>2]=2;if(!q){return}v=(i|0)>0;u=(d|0)==0;t=0;do{L21598:do{if(v){if(u){s=0;while(1){r=s+t|0;y=f+(r<<2)|0;if((c[y>>2]|0)==0){c[y>>2]=1;y=c[j+(r<<2)>>2]|0;N=j+(r+1<<2)|0;if((y|0)<(c[N>>2]|0)){z=y;do{y=n+(z<<2)|0;L=c[y>>2]>>1;H=p+(z<<2)|0;J=c[H>>2]>>1;c[y>>2]=J+L;c[H>>2]=L-J;z=z+1|0;}while((z|0)<(c[N>>2]|0))}N=k+4+(r<<2)|0;z=c[N>>2]|0;J=l+4+(r<<2)|0;L=c[J>>2]|0;H=(z|0)<(L|0)?z:L;c[J>>2]=H;c[N>>2]=H;H=o+(r<<2)|0;N=c[H>>2]|0;J=a+(r<<2)|0;L=c[J>>2]|0;z=(N|0)<(L|0)?N:L;c[J>>2]=z;c[H>>2]=z;c[k+484+(r<<2)>>2]=c[k+1924+(r<<2)>>2];c[l+484+(r<<2)>>2]=c[l+1924+(r<<2)>>2];c[m+(r<<2)>>2]=c[k+2404+(r<<2)>>2];c[b+(r<<2)>>2]=c[l+2404+(r<<2)>>2];z=k+2644+(r<<2)|0;H=c[z>>2]|0;J=l+2644+(r<<2)|0;L=c[J>>2]|0;N=((H|0)<(L|0)?H:L)>>1;c[J>>2]=N;c[z>>2]=N}s=s+1|0;if((s|0)>=(i|0)){break L21598}}}else{ab=0}do{s=ab+t|0;do{if((c[d+(s<<2)>>2]|0)==0){N=f+(s<<2)|0;if((c[N>>2]|0)!=0){break}c[N>>2]=1;N=c[j+(s<<2)>>2]|0;z=j+(s+1<<2)|0;if((N|0)<(c[z>>2]|0)){J=N;do{N=n+(J<<2)|0;L=c[N>>2]>>1;H=p+(J<<2)|0;y=c[H>>2]>>1;c[N>>2]=y+L;c[H>>2]=L-y;J=J+1|0;}while((J|0)<(c[z>>2]|0))}z=k+4+(s<<2)|0;J=c[z>>2]|0;Y=l+4+(s<<2)|0;D=c[Y>>2]|0;W=(J|0)<(D|0)?J:D;c[Y>>2]=W;c[z>>2]=W;W=o+(s<<2)|0;z=c[W>>2]|0;Y=a+(s<<2)|0;D=c[Y>>2]|0;J=(z|0)<(D|0)?z:D;c[Y>>2]=J;c[W>>2]=J;c[k+484+(s<<2)>>2]=c[k+1924+(s<<2)>>2];c[l+484+(s<<2)>>2]=c[l+1924+(s<<2)>>2];c[m+(s<<2)>>2]=c[k+2404+(s<<2)>>2];c[b+(s<<2)>>2]=c[l+2404+(s<<2)>>2];J=k+2644+(s<<2)|0;W=c[J>>2]|0;Y=l+2644+(s<<2)|0;D=c[Y>>2]|0;z=((W|0)<(D|0)?W:D)>>1;c[Y>>2]=z;c[J>>2]=z}}while(0);ab=ab+1|0;}while((ab|0)<(i|0))}}while(0);t=t+h|0;}while((t|0)<(g|0));return}}while(0);c[e>>2]=0;return}function Nj(a,d,f,g,h,i,j){a=a|0;d=d|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;var k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0;if((f|0)<=0){return}k=i|0;l=i+20|0;m=i+2|0;n=i+8|0;o=0;while(1){p=o+1|0;q=c[g+(o<<2)>>2]|0;r=(c[g+(p<<2)>>2]|0)-q|0;do{if((o|0)<(b[k>>1]|0)){s=15392}else{if((r|0)<(c[l>>2]|0)){s=15392;break}t=b[m>>1]|0;do{if((t&1)==0){u=32767;s=15403}else{v=(c[d+(o<<2)>>2]|0)-3|0;w=(v|0)<0?0:v;v=r>>2;x=q+v|0;if((v|0)>0){y=v<<1;z=v*3|0;A=0;B=0;C=0;D=0;E=q;while(1){F=c[a+(E<<2)>>2]<<w;H=F;I=(F|0)<0|0?-1:0;N2(H,I,H,I)|0;I=G+A|0;H=c[a+(E+v<<2)>>2]<<w;F=H;J=(H|0)<0|0?-1:0;N2(F,J,F,J)|0;J=G+B|0;F=c[a+(E+y<<2)>>2]<<w;H=F;K=(F|0)<0|0?-1:0;N2(H,K,H,K)|0;K=G+C|0;H=c[a+(E+z<<2)>>2]<<w;F=H;L=(H|0)<0|0?-1:0;N2(F,L,F,L)|0;L=G+D|0;F=E+1|0;if((F|0)<(x|0)){A=I;B=J;C=K;D=L;E=F}else{M=I;N=J;O=K;P=L;break}}}else{M=0;N=0;O=0;P=0}E=(M|0)>(N|0)?M:N;D=(E|0)>(O|0)?E:O;E=(D|0)>(P|0)?D:P;D=(M|0)<(N|0)?M:N;C=(D|0)<(O|0)?D:O;D=(C|0)<(P|0)?C:P;do{if((E|0)==0){Q=0}else{C=E>>31^E;if((C|0)<=-1){Q=-1;break}B=~C;C=0;while(1){A=B<<1;if((A|0)<0){B=A;C=C+1|0}else{Q=C;break}}}}while(0);C=D<<Q;if((C|0)<1){R=0;break}B=e[i+24+(o<<1)>>1]<<16;A=E<<Q;x=A;w=(A|0)<0|0?-1:0;A=B;z=(B|0)<0|0?-1:0;N2(A,z,x,w)|0;u=((C>>>2|((C|0)<0|0?-1:0)<<30)<<1|0>>>31|0)>(G|0)?32767:0;s=15403}}while(0);do{if((s|0)==15403){s=0;if(!((t&2)!=0&(u&65535)>16384)){R=u;break}C=c[n>>2]|0;if((C|0)<1){S=0}else{S=((C>>>2|((C|0)<0|0?-1:0)<<30)<<1|0>>>31|0)>(e[j+(o<<1)>>1]<<16>>1|0)?32767:0}R=(u&65535)<(S&65535)?u:S}}while(0);b[h+(o<<1)>>1]=R}}while(0);if((s|0)==15392){s=0;b[h+(o<<1)>>1]=0}if((p|0)<(f|0)){o=p}else{break}}return}function Nk(d,f,g,h,j,k,l,m){d=d|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;m=m|0;var n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0;n=i;i=i+8|0;o=n|0;p=(m|0)==0;q=d+2|0;r=p?1928624:1915488;b[q>>1]=p?0:32;if((c[k>>2]|0)<1){s=0;i=n;return s|0}p=(m|0)!=0;if(p){t=1915552;u=5}else{t=(l|0)>1?1921280:1924320;u=8}l=0;while(1){if((c[t+(l<<4)>>2]|0)>>>0<=f>>>0){if((c[t+(l<<4)+4>>2]|0)>>>0>=f>>>0){v=l;break}}m=l+1|0;if((m|0)<(u|0)){l=m}else{v=m;break}}if((v|0)>9){s=16480;i=n;return s|0}do{if((g|0)==22050){w=t+(v<<4)+8|0;x=15428}else if((g|0)==48e3){w=t+(v<<4)+12|0;x=15428}else if((g|0)==44100){w=t+(v<<4)+11|0;x=15428}else if((g|0)==32e3){w=t+(v<<4)+10|0;x=15428}else if((g|0)==24e3){w=t+(v<<4)+9|0;x=15428}else{if(!p){break}w=t+(v<<4)+12|0;x=15428}}while(0);do{if((x|0)==15428){v=a[w]|0;if(v<<24>>24==0){break}t=(v&255)-1|0;b[d>>1]=(My(b[r+(t<<4)>>1]|0,g,h,j)|0)&65535;b[q>>1]=b[q>>1]|b[r+(t<<4)+14>>1];v=e[r+(t<<4)+2>>1]<<16;p=d+4|0;c[p>>2]=v;c[d+8>>2]=e[r+(t<<4)+4>>1]<<16;c[d+12>>2]=b[r+(t<<4)+6>>1]|0;c[d+16>>2]=b[r+(t<<4)+8>>1]|0;c[d+20>>2]=b[r+(t<<4)+12>>1]|0;b[d+144>>1]=b[r+(t<<4)+10>>1]|0;t=h-1|0;L21685:do{if((t|0)>0){l=0;u=v;while(1){f=l+1|0;m=HT(u,0,(c[j+(f<<2)>>2]|0)-(c[j+(l<<2)>>2]|0)|0,26,o)|0;y=c[o>>2]|0;if((y|0)>0){z=m<<y>>>16&65535}else{z=m>>-y>>>16&65535}b[d+24+(l<<1)>>1]=z;if((f|0)>=(t|0)){break L21685}l=f;u=c[p>>2]|0}}}while(0);b[d+24+(h<<1)>>1]=b[d+24+(t<<1)>>1]|0;s=0;i=n;return s|0}}while(0);c[k>>2]=0;s=0;i=n;return s|0}function Nl(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;c[f>>2]=8;IY(a,e,d<<2);c[b>>2]=1;return}function Nm(a,b,d,e,f,g,h,i){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;var j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0;if((b|0)==0){IY(a,g,d<<2);c[i>>2]=h;return}b=c[i>>2]|0;if((b|0)<(h|0)){j=h-b<<1;if((d|0)<=0){c[i>>2]=h;return}k=(f&65535)<<16;l=k;m=(k|0)<0|0?-1:0;k=0;do{n=a+(k<<2)|0;o=ac(c[n>>2]>>j,e)|0;p=g+(k<<2)|0;q=c[p>>2]|0;N2(q,(q|0)<0|0?-1:0,l,m)|0;r=G<<1|0>>>31;c[n>>2]=q;n=(q|0)<(o|0)?q:o;c[p>>2]=(n|0)>(r|0)?n:r;k=k+1|0;}while((k|0)<(d|0));c[i>>2]=h;return}if((d|0)<=0){c[i>>2]=h;return}k=e>>1;e=(f&65535)<<16;f=e;m=(e|0)<0|0?-1:0;e=b-h<<1|1;b=0;do{l=a+(b<<2)|0;j=ac(c[l>>2]|0,k)|0;r=g+(b<<2)|0;n=c[r>>2]|0;N2(n,(n|0)<0|0?-1:0,f,m)|0;p=G<<1|0>>>31;c[l>>2]=n;n=c[r>>2]|0;if((n>>e|0)>(j|0)){l=j<<e;c[r>>2]=l;s=l}else{s=n}c[r>>2]=(s|0)>(p|0)?s:p;b=b+1|0;}while((b|0)<(d|0));c[i>>2]=h;return}function Nn(a,e,f,g,h,j,k,l){a=a|0;e=e|0;f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;l=l|0;var m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ad=0,ae=0,af=0,ag=0;m=i;i=i+280|0;n=m|0;o=m+8|0;p=m+16|0;q=m+24|0;r=m+32|0;s=m+40|0;t=m+48|0;u=m+56|0;v=m+64|0;w=m+72|0;x=(h|0)/8|0;I_(k,1844);c[k+1836>>2]=h;c[k+220>>2]=l;if((j|0)==0){y=0}else{y=((a|0)/(f|0)|0|0)<5|0}c[k+1840>>2]=y;y=k|0;if((h|0)==480){z=144;A=1924176}else if((h|0)==512){z=144;A=1924032}else if((h|0)==1024|(h|0)==960){z=12;A=1928432}else{B=8320;i=m;return B|0}j=0;while(1){if((j|0)>=(z|0)){C=0;D=h;break}if((c[A+(j*12|0)>>2]|0)==(e|0)){E=15468;break}else{j=j+1|0}}do{if((E|0)==15468){if((g|0)==0|(g|0)==1|(g|0)==3){l=c[A+(j*12|0)+4>>2]|0;c[y>>2]=d[l|0]|0;C=l+1|0;D=h;break}else if((g|0)==2){l=c[A+(j*12|0)+8>>2]|0;c[y>>2]=d[l|0]|0;C=l+1|0;D=x;break}else{C=0;D=h;break}}}while(0);if((j|0)==(z|0)){B=12544;i=m;return B|0}else{F=0;H=0}while(1){z=c[y>>2]|0;if((H|0)>=(z|0)){I=F;J=H;K=z;break}c[k+12+(H<<2)>>2]=F;L=(d[C+H|0]|0)+F|0;M=H+1|0;if((L|0)<(D|0)){F=L;H=M}else{E=15474;break}}if((E|0)==15474){I=L;J=M;K=c[y>>2]|0}M=(J|0)<(K|0)?J:K;c[y>>2]=M;c[k+12+(M<<2)>>2]=(I|0)<(D|0)?I:D;D=c[y>>2]|0;I=c[k+12+(D<<2)>>2]|0;do{if((D|0)>0){if((I|0)==480){M=0;while(1){K=ac(c[k+12+(M<<2)>>2]|0,e)|0;N2(K,(K|0)<0|0?-1:0,1145324612,0)|0;K=G<<5|0>>>27;J=K;L=(K|0)<0|0?-1:0;N2(J,L,1172812403,0)|0;K=G<<1|0>>>31;N2(J,L,1671257674,0)|0;L=G<<3|0>>>29;J=Nu(K)|0;K=Nu(L)|0;N2(K,(K|0)<0|0?-1:0,892547891,0)|0;K=G<<1|0>>>31;L=J;E=(J|0)<0|0?-1:0;N2(L,E,L,E)|0;E=G<<1|0>>>31;N2(E,(E|0)<0|0?-1:0,469762048,0)|0;E=(G<<1|0>>>31)+K|0;K=M+1|0;L=ac(c[k+12+(K<<2)>>2]|0,e)|0;N2(L,(L|0)<0|0?-1:0,1145324612,0)|0;L=G<<5|0>>>27;J=L;H=(L|0)<0|0?-1:0;N2(J,H,1172812403,0)|0;L=G<<1|0>>>31;N2(J,H,1671257674,0)|0;H=G<<3|0>>>29;J=Nu(L)|0;L=Nu(H)|0;N2(L,(L|0)<0|0?-1:0,892547891,0)|0;L=G<<1|0>>>31;H=J;F=(J|0)<0|0?-1:0;N2(H,F,H,F)|0;F=G<<1|0>>>31;N2(F,(F|0)<0|0?-1:0,469762048,0)|0;F=((G<<1|0>>>31)+L>>1)+(E>>1)|0;c[w+(M<<2)>>2]=(F|0)<805306368?F:805306368;if((K|0)<(D|0)){M=K}else{break}}}else if((I|0)==1024){M=e<<2;K=0;while(1){F=ac(M,c[k+12+(K<<2)>>2]|0)|0;E=F;L=(F|0)<0|0?-1:0;N2(E,L,1172812403,0)|0;F=G<<1|0>>>31;N2(E,L,1671257674,0)|0;L=G<<3|0>>>29;E=Nu(F)|0;F=Nu(L)|0;N2(F,(F|0)<0|0?-1:0,892547891,0)|0;F=G<<1|0>>>31;L=E;H=(E|0)<0|0?-1:0;N2(L,H,L,H)|0;H=G<<1|0>>>31;N2(H,(H|0)<0|0?-1:0,469762048,0)|0;H=(G<<1|0>>>31)+F|0;F=K+1|0;L=ac(M,c[k+12+(F<<2)>>2]|0)|0;E=L;J=(L|0)<0|0?-1:0;N2(E,J,1172812403,0)|0;L=G<<1|0>>>31;N2(E,J,1671257674,0)|0;J=G<<3|0>>>29;E=Nu(L)|0;L=Nu(J)|0;N2(L,(L|0)<0|0?-1:0,892547891,0)|0;L=G<<1|0>>>31;J=E;C=(E|0)<0|0?-1:0;N2(J,C,J,C)|0;C=G<<1|0>>>31;N2(C,(C|0)<0|0?-1:0,469762048,0)|0;C=((G<<1|0>>>31)+L>>1)+(H>>1)|0;c[w+(K<<2)>>2]=(C|0)<805306368?C:805306368;if((F|0)<(D|0)){K=F}else{break}}}else{K=0;while(1){M=ac(c[k+12+(K<<2)>>2]|0,e)|0;if((I|0)==128){N=M<<5}else if((I|0)==512){N=M<<3}else{N=0}M=N;F=(N|0)<0|0?-1:0;N2(M,F,1172812403,0)|0;C=G<<1|0>>>31;N2(M,F,1671257674,0)|0;F=G<<3|0>>>29;M=Nu(C)|0;C=Nu(F)|0;N2(C,(C|0)<0|0?-1:0,892547891,0)|0;C=G<<1|0>>>31;F=M;H=(M|0)<0|0?-1:0;N2(F,H,F,H)|0;H=G<<1|0>>>31;N2(H,(H|0)<0|0?-1:0,469762048,0)|0;H=(G<<1|0>>>31)+C|0;C=K+1|0;F=ac(c[k+12+(C<<2)>>2]|0,e)|0;if((I|0)==128){O=F<<5}else if((I|0)==512){O=F<<3}else{O=0}F=O;M=(O|0)<0|0?-1:0;N2(F,M,1172812403,0)|0;L=G<<1|0>>>31;N2(F,M,1671257674,0)|0;M=G<<3|0>>>29;F=Nu(L)|0;L=Nu(M)|0;N2(L,(L|0)<0|0?-1:0,892547891,0)|0;L=G<<1|0>>>31;M=F;J=(F|0)<0|0?-1:0;N2(M,J,M,J)|0;J=G<<1|0>>>31;N2(J,(J|0)<0|0?-1:0,469762048,0)|0;J=((G<<1|0>>>31)+L>>1)+(H>>1)|0;c[w+(K<<2)>>2]=(J|0)<805306368?J:805306368;if((C|0)<(D|0)){K=C}else{break}}}K=c[y>>2]|0;if((K|0)<=0){P=K;break}C=0;J=c[k+12>>2]|0;while(1){H=C+1|0;L=c[k+12+(H<<2)>>2]|0;c[k+224+(C<<2)>>2]=ac(L-J|0,5533794)|0;if((H|0)<(K|0)){C=H;J=L}else{break}}P=c[y>>2]|0}else{P=D}}while(0);D=(g|0)==2;if(D){Q=0;R=1073741824;S=0;T=805306368}else{g=(a|0)>2e4;Q=0;R=1610612736;S=g?0:0;T=g?1073741824:805306368}if((P|0)>0){g=P-1|0;O=k+448+(g<<2)|0;I=k+856+(g<<2)|0;g=0;do{if((g|0)>0){N=g-1|0;J=(c[w+(g<<2)>>2]|0)-(c[w+(N<<2)>>2]|0)|0;C=J;K=(J|0)<0|0?-1:0;J=805306368;L=0;N2(C,K,J,L)|0;L=HT(1342177280,4,-(G<<1|0>>>31)|0,8,p)|0;J=c[p>>2]|0;H=(J|0)<-31?-31:J;J=(H|0)>31?31:H;c[p>>2]=J;H=(J|0)>0;do{if(H&(L|0)>(2147483647>>>(J>>>0)|0)){U=2147483647}else{if(H){U=L<<J;break}else{U=L>>-J;break}}}while(0);c[k+652+(g<<2)>>2]=U;J=1610612736;L=0;N2(C,K,J,L)|0;L=HT(1342177280,4,-(G<<1|0>>>31)|0,8,n)|0;J=c[n>>2]|0;H=(J|0)<-31?-31:J;J=(H|0)>31?31:H;c[n>>2]=J;H=(J|0)>0;do{if(H&(L|0)>(2147483647>>>(J>>>0)|0)){V=2147483647}else{if(H){V=L<<J;break}else{V=L>>-J;break}}}while(0);c[k+448+(N<<2)>>2]=V;N2(C,K,T,S)|0;J=HT(1342177280,4,-(G<<1|0>>>31)|0,8,o)|0;L=c[o>>2]|0;H=(L|0)<-31?-31:L;L=(H|0)>31?31:H;c[o>>2]=L;H=(L|0)>0;do{if(H&(J|0)>(2147483647>>>(L>>>0)|0)){W=2147483647}else{if(H){W=J<<L;break}else{W=J>>-L;break}}}while(0);c[k+1060+(g<<2)>>2]=W;N2(C,K,R,Q)|0;L=HT(1342177280,4,-(G<<1|0>>>31)|0,8,q)|0;J=c[q>>2]|0;H=(J|0)<-31?-31:J;J=(H|0)>31?31:H;c[q>>2]=J;H=(J|0)>0;do{if(H&(L|0)>(2147483647>>>(J>>>0)|0)){X=2147483647}else{if(H){X=L<<J;break}else{X=L>>-J;break}}}while(0);c[k+856+(N<<2)>>2]=X}else{c[k+652+(g<<2)>>2]=0;c[O>>2]=0;c[k+1060+(g<<2)>>2]=0;c[I>>2]=0}g=g+1|0;}while((g|0)<(P|0))}c[k+428>>2]=2;b[k+432>>1]=328;P=k+444|0;c[P>>2]=1999999999;g=f<<1;if(D){f=(ac(x,g)|0)/(e|0)|0;c[k+436>>2]=f;c[k+440>>2]=0;c[P>>2]=31249999;Y=f;Z=0}else{f=(ac(g,h)|0)/(e|0)|0;c[k+436>>2]=f;c[k+440>>2]=12;Y=f;Z=12}f=c[y>>2]|0;y=0;while(1){if((y|0)>=(f|0)){break}if((c[k+12+(y<<2)>>2]|0)<(Y|0)){y=y+1|0}else{break}}c[k+4>>2]=y;Y=0;while(1){if((Y|0)>=(f|0)){break}if((c[k+12+(Y<<2)>>2]|0)<(Z|0)){Y=Y+1|0}else{break}}c[k+8>>2]=Y;Y=c[k+12+(f<<2)>>2]|0;f=ac(c[k+12+(y<<2)>>2]|0,e)|0;if((Y|0)==512){_=f<<3}else if((Y|0)==480){N2(f,(f|0)<0|0?-1:0,1145324612,0)|0;_=G<<5|0>>>27}else if((Y|0)==1024){_=f<<2}else if((Y|0)==128){_=f<<5}else{_=0}f=_;Z=(_|0)<0|0?-1:0;N2(f,Z,1172812403,0)|0;_=G<<1|0>>>31;N2(f,Z,1671257674,0)|0;Z=G<<3|0>>>29;f=Nu(_)|0;_=Nu(Z)|0;N2(_,(_|0)<0|0?-1:0,892547891,0)|0;_=G<<1|0>>>31;Z=f;h=(f|0)<0|0?-1:0;N2(Z,h,Z,h)|0;h=G<<1|0>>>31;N2(h,(h|0)<0|0?-1:0,469762048,0)|0;h=(G<<1|0>>>31)+_|0;_=HQ((h|0)<805306368?h:805306368,838860800,r)|0;c[r>>2]=31-(c[r>>2]|0);h=HQ(a,e,s)|0;a=31-(c[s>>2]|0)|0;N2(h,(h|0)<0|0?-1:0,1267015352,0)|0;h=G<<1|0>>>31;N2(h,(h|0)<0|0?-1:0,1649267442,0)|0;h=G<<1|0>>>31;Z=a+4|0;c[s>>2]=Z;if((Y|0)==512){f=a-5|0;c[s>>2]=f;$=h;aa=f}else if((Y|0)==480){f=a-5|0;c[s>>2]=f;N2(h,(h|0)<0|0?-1:0,2013265920,0)|0;$=G<<1|0>>>31;aa=f}else if((Y|0)==1024){f=a-6|0;c[s>>2]=f;$=h;aa=f}else if((Y|0)==128){f=a-3|0;c[s>>2]=f;$=h;aa=f}else{$=h;aa=Z}if(D){D=$;Z=($|0)<0|0?-1:0;h=1610612736;f=0;N2(D,Z,h,f)|0;c[s>>2]=aa-1;ab=G<<1|0>>>31}else{ab=$}$=HQ(ab,_,t)|0;if((y|0)<=0){B=0;i=m;return B|0}_=$;ab=($|0)<0|0?-1:0;$=(c[t>>2]|0)-25+(c[r>>2]|0)-(c[s>>2]|0)|0;s=0;while(1){r=s+1|0;aa=k+12+(r<<2)|0;f=ac(c[aa>>2]|0,e)|0;if((Y|0)==1024){ad=f<<2}else if((Y|0)==128){ad=f<<5}else if((Y|0)==512){ad=f<<3}else if((Y|0)==480){N2(f,(f|0)<0|0?-1:0,1145324612,0)|0;ad=G<<5|0>>>27}else{ad=0}f=ad;h=(ad|0)<0|0?-1:0;N2(f,h,1172812403,0)|0;Z=G<<1|0>>>31;N2(f,h,1671257674,0)|0;h=G<<3|0>>>29;f=Nu(Z)|0;Z=Nu(h)|0;N2(Z,(Z|0)<0|0?-1:0,892547891,0)|0;Z=G<<1|0>>>31;h=f;D=(f|0)<0|0?-1:0;N2(h,D,h,D)|0;D=G<<1|0>>>31;N2(D,(D|0)<0|0?-1:0,469762048,0)|0;D=G<<1|0>>>31;h=k+12+(s<<2)|0;f=ac(c[h>>2]|0,e)|0;if((Y|0)==1024){ae=f<<2}else if((Y|0)==128){ae=f<<5}else if((Y|0)==512){ae=f<<3}else if((Y|0)==480){N2(f,(f|0)<0|0?-1:0,1145324612,0)|0;ae=G<<5|0>>>27}else{ae=0}f=ae;a=(ae|0)<0|0?-1:0;N2(f,a,1172812403,0)|0;g=G<<1|0>>>31;N2(f,a,1671257674,0)|0;a=G<<3|0>>>29;f=Nu(g)|0;g=Nu(a)|0;N2(g,(g|0)<0|0?-1:0,892547891,0)|0;g=G<<1|0>>>31;a=f;P=(f|0)<0|0?-1:0;N2(a,P,a,P)|0;P=G<<1|0>>>31;N2(P,(P|0)<0|0?-1:0,469762048,0)|0;P=D+Z-g-(G<<1|0>>>31)|0;N2(P,(P|0)<0|0?-1:0,_,ab)|0;P=HQ(G<<1|0>>>31,(c[aa>>2]|0)-(c[h>>2]|0)|0,t)|0;h=HI(P,$+(c[t>>2]|0)|0,u)|0;P=31-(c[u>>2]|0)|0;c[u>>2]=P;aa=(P|0)<30?P:30;c[v>>2]=aa;if((aa|0)<0){af=0}else{af=1610612736>>>((31-aa|0)>>>0)}g=(h>>P-aa>>1)-af|0;P=aa-1|0;c[v>>2]=P;aa=(P|0)>0?1<<P:0;P=HQ(aa,(aa|0)>(g|0)?aa:g,v)|0;g=31-(c[v>>2]|0)|0;c[v>>2]=g;if((g|0)>30){ag=P>>g-30}else{ag=P}P=(ag|0)>858993459?858993459:ag;c[k+1264+(s<<2)>>2]=HE((P|0)<3221225?6442450:P<<1)|0;if((r|0)<(y|0)){s=r}else{B=0;break}}i=m;return B|0}function No(d,e,f,g){d=d|0;e=e|0;f=f|0;g=g|0;var h=0,i=0,j=0;h=g&255;a[d+58|0]=h;b[d+52>>1]=e&65535;i=f&65535;b[d+56>>1]=i;if((g|0)==0){j=0}else{j=1<<g-1&65535}b[d+54>>1]=j;b[d+68>>1]=i;a[d|0]=0;a[d+16|0]=0;a[d+32|0]=0;c[d+60>>2]=0;c[d+64>>2]=0;i=d+48|0;c[i>>2]=0;if(h<<24>>24!=16){return}if((e|0)==4129){c[i>>2]=1920256;return}else if((e|0)==32773){c[i>>2]=1919744;return}else{c[i>>2]=0;return}}function Np(d){d=d|0;b[d+68>>1]=b[d+56>>1]|0;a[d|0]=0;a[d+16|0]=0;a[d+32|0]=0;c[d+60>>2]=0;c[d+64>>2]=0;return}function Nq(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,i=0;f=b+60|0;g=c[f>>2]|0;a[b+(g<<4)|0]=1;c[b+(g<<4)+4>>2]=e;e=d+8|0;if((c[d+40>>2]|0)==0){Hy(e,c[d+4>>2]|0,0);h=d|0}else{i=d|0;Hw(e,c[i>>2]|0,c[d+4>>2]|0);h=i}c[d+4>>2]=0;c[h>>2]=0;c[b+(g<<4)+12>>2]=HA(e)|0;c[b+(g<<4)+8>>2]=0;c[f>>2]=(((c[f>>2]|0)+1|0)>>>0)%3|0;return g|0}function Nr(d,f,g){d=d|0;f=f|0;g=g|0;var h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0;h=i;i=i+48|0;j=h|0;k=f+40|0;l=c[k>>2]|0;if((l|0)==1){m=f+8|0;n=f|0;o=f+4|0;Hw(m,c[n>>2]|0,c[o>>2]|0);c[o>>2]=0;c[n>>2]=0;n=HA(m)|0;m=n-(c[d+(g<<4)+12>>2]|0)|0;c[d+(g<<4)+8>>2]=m;p=m}else{m=c[d+(g<<4)+12>>2]|0;n=f+8|0;if((l|0)==0){Hy(n,c[f+4>>2]|0,0);q=f|0}else{l=f|0;Hw(n,c[l>>2]|0,c[f+4>>2]|0);q=l}c[f+4>>2]=0;c[q>>2]=0;q=m-(HA(n)|0)|0;c[d+(g<<4)+8>>2]=q;p=q}q=d+(g<<4)+4|0;if((c[q>>2]|0)==0){c[q>>2]=p}p=d+68|0;n=b[p>>1]|0;if((c[k>>2]|0)==0){k=f;NN(j|0,k|0,44)|0;k=c[d+(g<<4)+12>>2]|0;m=j+8|0;if((c[j+40>>2]|0)==0){Hy(m,c[j+4>>2]|0,0);r=j|0}else{l=j|0;Hw(m,c[l>>2]|0,c[j+4>>2]|0);r=l}c[j+4>>2]=0;c[r>>2]=0;Nt(j,(HA(m)|0)-k|0)}else{Ht(j+8|0,c[f+28>>2]|0,c[f+32>>2]|0,c[f+8>>2]|0);c[j+4>>2]=0;c[j>>2]=0;c[j+40>>2]=0;Nt(j,c[d+(g<<4)+12>>2]|0)}f=c[q>>2]|0;q=(f|0)>-1?f:-f|0;if((f|0)>0){f=c[d+(g<<4)+8>>2]|0;if((f&-8|0)<(q|0)){s=f}else{t=15591}}else{t=15591}if((t|0)==15591){s=q}t=s>>3;f=s&7;s=d+48|0;k=c[s>>2]|0;do{if((k|0)==0){m=b[d+54>>1]|0;r=t<<3;if((r|0)<=0){u=r;v=n;break}l=j+4|0;o=j|0;w=e[d+52>>1]|0;x=c[497855]&65535;y=j+8|0;z=n;A=0;B=c[l>>2]|0;while(1){if(B>>>0>1){C=B;D=c[o>>2]|0}else{E=31-B|0;F=c[o>>2]<<E;G=F|(Hv(y,E)|0);c[o>>2]=G;F=(c[l>>2]|0)+E|0;c[l>>2]=F;C=F;D=G}G=C-1|0;c[l>>2]=G;F=((ac(x&D>>>(G>>>0)^(z&m)<<16>>16!=0,w)|0)^z<<1&65535)&65535;E=A+1|0;if((E|0)<(r|0)){z=F;A=E;B=G}else{u=r;v=F;break}}}else{if((t|0)<=0){u=0;v=n;break}r=j+4|0;B=j|0;A=c[497862]&255;z=j+8|0;w=0;m=n;x=c[r>>2]|0;do{if(x>>>0>8){H=x;I=c[B>>2]|0}else{l=31-x|0;o=c[B>>2]<<l;y=o|(Hv(z,l)|0);c[B>>2]=y;o=(c[r>>2]|0)+l|0;c[r>>2]=o;H=o;I=y}x=H-8|0;c[r>>2]=x;m=b[k+((A&I>>>(x>>>0)^(m&65535)>>>8)<<1)>>1]^m<<8;w=w+1|0;}while((w|0)<(t|0));u=t<<3;v=m}}while(0);t=q-u|0;if((f|0)==0){J=t;K=v}else{u=b[d+54>>1]|0;q=j+4|0;I=j|0;k=e[d+52>>1]|0;H=c[497855]&65535;n=j+8|0;j=v;v=0;D=c[q>>2]|0;do{if(D>>>0>1){L=D;M=c[I>>2]|0}else{C=31-D|0;w=c[I>>2]<<C;x=w|(Hv(n,C)|0);c[I>>2]=x;w=(c[q>>2]|0)+C|0;c[q>>2]=w;L=w;M=x}D=L-1|0;c[q>>2]=D;j=((ac(H&M>>>(D>>>0)^(j&u)<<16>>16!=0,k)|0)^j<<1&65535)&65535;v=v+1|0;}while((v|0)<(f|0));J=t-f|0;K=j}if((J|0)==0){N=K;b[p>>1]=N;O=d+(g<<4)|0;a[O]=0;P=d+64|0;Q=c[P>>2]|0;R=Q+1|0;S=(R>>>0)%3|0;c[P>>2]=S;i=h;return 0}j=c[s>>2]|0;do{if((j|0)!=0&(J|0)>8){s=J>>3;if((s|0)>0){f=0;t=K;do{t=b[j+((t&65535)>>>8<<1)>>1]^t<<8;f=f+1|0;}while((f|0)<(s|0));T=t;U=s<<3}else{T=K;U=0}if((J|0)==(U|0)){N=T;b[p>>1]=N;O=d+(g<<4)|0;a[O]=0;P=d+64|0;Q=c[P>>2]|0;R=Q+1|0;S=(R>>>0)%3|0;c[P>>2]=S;i=h;return 0}else{V=T;W=J-U|0;break}}else{V=K;W=J}}while(0);J=b[d+54>>1]|0;K=b[d+52>>1]|0;if((W|0)>0){X=V;Y=0}else{N=V;b[p>>1]=N;O=d+(g<<4)|0;a[O]=0;P=d+64|0;Q=c[P>>2]|0;R=Q+1|0;S=(R>>>0)%3|0;c[P>>2]=S;i=h;return 0}while(1){V=K&(-((X&J)<<16>>16!=0|0)&65535)^X<<1;U=Y+1|0;if((U|0)<(W|0)){X=V;Y=U}else{N=V;break}}b[p>>1]=N;O=d+(g<<4)|0;a[O]=0;P=d+64|0;Q=c[P>>2]|0;R=Q+1|0;S=(R>>>0)%3|0;c[P>>2]=S;i=h;return 0}function Ns(a){a=a|0;return(b[a+54>>1]<<1)-1&65535&b[a+68>>1]|0}function Nt(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;if((b|0)>-1){d=a+4|0;e=c[d>>2]|0;f=a+40|0;g=(c[f>>2]|0)==0;do{if(e>>>0>b>>>0){if(!g){h=a+8|0;i=15635;break}c[d>>2]=e-b;return}else{j=a+8|0;if(!g){h=j;i=15635;break}Hy(j,e,0);k=a|0;l=j}}while(0);if((i|0)==15635){g=a|0;Hw(h,c[g>>2]|0,e);k=g;l=h}c[d>>2]=0;c[k>>2]=0;Hz(l,b,c[f>>2]&255);return}else{f=-b|0;l=a+4|0;k=c[l>>2]|0;d=k-b|0;b=a+40|0;h=(c[b>>2]|0)==0;do{if(d>>>0<32){if(!h){m=a+8|0;i=15643;break}c[l>>2]=d;Hy(a+8|0,d,0);c[l>>2]=0;c[a>>2]=0;return}else{g=a+8|0;if(!h){m=g;i=15643;break}Hy(g,k,0);n=a|0;o=g}}while(0);if((i|0)==15643){i=a|0;Hw(m,c[i>>2]|0,k);n=i;o=m}c[l>>2]=0;c[n>>2]=0;Hy(o,f,c[b>>2]&255);return}}function Nu(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,j=0,k=0,l=0,m=0;b=i;i=i+16|0;d=b|0;e=b+8|0;f=(a|0)<0?-a|0:a;do{if((f|0)<25071872){g=f;h=(f|0)<0|0?-1:0;N2(g,h,g,h)|0;h=G<<1|0>>>31;g=h;j=(h|0)<0|0?-1:0;h=645922816;k=0;N2(g,j,h,k)|0;k=HQ(f,(G<<1|0>>>31)+524288|0,d)|0;h=c[d>>2]|0;j=h-7|0;if((j|0)>0){l=k<<j;break}else{l=k>>7-h;break}}else{if((f|0)<42949673){h=(f<<5)-1073741824|0;k=(h>>1)+843314842|0;j=h;g=(h|0)<0|0?-1:0;N2(j,g,j,g)|0;l=k-G|0;break}k=f;g=(f|0)<0|0?-1:0;N2(k,g,k,g)|0;g=HQ(f,G+77824|0,e)|0;k=c[e>>2]|0;j=k-8|0;if((j|0)>0){m=g<<j}else{m=g>>8-k}l=1686630144-m|0}}while(0);i=b;return((a|0)>-1?l:-l|0)|0}function Nv(a,b){a=a|0;b=b|0;var e=0,f=0;e=Nw(a)|0;do{if((e|0)!=-1){a=Nw(b)|0;if((a|0)==-1){break}if((d[299824+e|0]|0)>207){break}f=a<<16|e;return f|0}}while(0);c[(ch()|0)>>2]=22;f=-1;return f|0}function Nw(b){b=b|0;var c=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;c=a[b]|0;L22004:do{if(c<<24>>24==0){d=299824;while(1){if((a[d]|0)==0){e=d;break L22004}f=NL(d|0)|0;g=d+(f+1)|0;do{if((a[g]|0)==0){h=a[d+(f+2)|0]|0;if((h&255)>128){i=d+(f+3)|0;break}else{i=d+(f+3+(((128-(h&255)|0)>>>2)*5|0))|0;break}}else{i=g}}while(0);if((a[i]|0)==0){j=-1;break}else{d=i}}return j|0}else{d=299824;while(1){g=b;f=d;h=c;while(1){k=a[f]|0;if(k<<24>>24==0){l=f;m=h;n=15683;break}o=h&255;p=o|32;L22010:do{if(h<<24>>24==0){q=g;r=p}else{s=g;t=o;u=p;while(1){v=s+1|0;if(!((u-97|0)>>>0>26&(t-48|0)>>>0>10)){q=s;r=u;break L22010}w=a[v]|0;x=w&255;y=x|32;if(w<<24>>24==0){q=v;r=y;break}else{s=v;t=x;u=y}}}}while(0);if((r|0)!=(k&255|0)){break}p=q+1|0;o=f+1|0;u=a[p]|0;if(u<<24>>24==0){l=o;m=0;n=15683;break}else{g=p;f=o;h=u}}if((n|0)==15683){n=0;if(m<<24>>24==(a[l]|0)){break}}h=NL(d|0)|0;f=d+(h+1)|0;do{if((a[f]|0)==0){g=a[d+(h+2)|0]|0;if((g&255)>128){z=d+(h+3)|0;break}else{z=d+(h+3+(((128-(g&255)|0)>>>2)*5|0))|0;break}}else{z=f}}while(0);if((a[z]|0)==0){j=-1;n=15695;break}else{d=z}}if((n|0)==15695){return j|0}if((a[d]|0)==0){e=d;break}else{A=d}while(1){f=A+((NL(A|0)|0)+1)|0;if((a[f]|0)==0){e=f;break}else{A=f}}}}while(0);j=e+1-299824|0;return j|0}function Nx(a){a=a|0;return 0}function Ny(f,g,h,j,k){f=f|0;g=g|0;h=h|0;j=j|0;k=k|0;var l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0,aH=0,aI=0;l=i;i=i+8|0;m=l|0;n=m;o=i;i=i+4|0;i=i+7&-8;p=i;i=i+4|0;i=i+7&-8;q=i;i=i+4|0;i=i+7&-8;r=i;i=i+4|0;i=i+7&-8;s=f;f=s&65535;t=s>>>16;s=t+1|0;u=f+1|0;c[m>>2]=0;c[m+4>>2]=0;m=a[299824+t|0]|0;if((g|0)==0){v=0;i=l;return v|0}w=c[g>>2]|0;if((w|0)==0){v=0;i=l;return v|0}x=c[h>>2]|0;if((x|0)==0){v=0;i=l;return v|0}y=m&255;z=y+128|0;A=t+2|0;t=y&3;B=t^1;C=t^2;D=t^3;E=y&1;F=E^1;G=(y-196|0)>>>0<2;H=E|2;I=F|2;J=m<<24>>24==-38;K=m<<24>>24==-40;m=d[299824+f|0]|0;L=m-192|0;M=L>>>0>8;N=f;O=1686640+L|0;L=m+128|0;P=128-m|0;Q=f+2|0;f=r|0;r=m&1;R=r^1;S=r|2;T=R|2;U=m&3;V=U^1;W=U^2;X=U^3;Y=(y-192|0)>>>0<7;Z=0;_=w;w=x;L22053:while(1){x=a[_]|0;$=x&255;L22055:do{if(x<<24>>24<0|Y){switch(y|0){case 232:{if(w>>>0<2){aa=22;break L22053}ab=a[_+1|0]|0;ac=ab&255;ad=$-161|0;ae=ac-161|0;if(!(ad>>>0>92|ae>>>0>93)){af=b[139272+(ad*188|0)+(ae<<1)>>1]|0;if(af<<16>>16==0){ag=84;ah=15810;break L22053}else{ai=2;aj=af&65535;ah=15786;break L22055}}af=$-129|0;if(af>>>0>92){ag=84;ah=15810;break L22053}if(af>>>0>68&(ab&255)>82){ag=84;ah=15810;break L22053}ab=ac-65|0;do{if(ab>>>0<26){ak=ab}else{if((ac-97|0)>>>0<26){ak=ac-71|0;break}if((ac-129|0)>>>0>=126){ag=84;ah=15810;break L22053}ak=ac-77|0}}while(0);if(af>>>0<32){al=af*178|0}else{al=(ad*84|0)+5696|0}ac=ak+44032+al|0;if(ac>>>0<44032){ai=2;aj=ac;ah=15786;break L22055}else{am=ac;an=44032}while(1){ac=am-an|0;ab=0;ae=0;while(1){ao=ab;ap=0;do{ao=(((e[139272+(ae*188|0)+(ap<<1)>>1]|0)-an|0)>>>0<=ac>>>0)+ao|0;ap=ap+1|0;}while((ap|0)<94);ap=ae+1|0;if((ap|0)<93){ab=ao;ae=ap}else{break}}ae=am+1|0;ab=ao+am|0;if(ae>>>0>ab>>>0){ai=2;aj=ab;ah=15786;break L22055}else{am=ab;an=ae}}break};case 209:{if(($-161|0)>>>0<63){ai=1;aj=$+65216|0;ah=15786;break L22055}if(w>>>0<2){aa=22;break L22053}ad=a[_+1|0]|0;af=ad&255;ae=$-129|0;if(ae>>>0<31){aq=ae}else{if(($-224|0)>>>0>=16){ag=84;ah=15810;break L22053}aq=$-193|0}ae=aq<<1;ab=af-64|0;if(ab>>>0<95){if(ad<<24>>24==127){ag=84;ah=15810;break L22053}ar=ab+(ad<<24>>24>>7<<24>>24)|0;as=ae}else{ad=af-159|0;ab=ad>>>0<94;ar=ab?ad:af;as=ae|ab&1}ab=b[156928+(as*188|0)+(ar<<1)>>1]|0;if(ab<<16>>16==0){ag=84;ah=15810;break L22053}else{ai=2;aj=ab&65535;ah=15786;break L22055}break};case 200:{ab=Nz(o,_,w,n)|0;if((ab|0)==0){at=1}else if((ab|0)==(-1|0)){ag=84;ah=15810;break L22053}else if((ab|0)==(-2|0)){aa=22;break L22053}else{at=ab}ai=at;aj=c[o>>2]|0;ah=15786;break L22055;break};case 196:case 197:case 194:case 193:{if(w>>>0<2){aa=22;break L22053}ab=d[_+E|0]<<8|d[_+F|0];if((ab-56320|0)>>>0<1024){ag=84;ah=15810;break L22053}if((ab-55296|0)>>>0>=1024){ai=2;aj=ab;ah=15786;break L22055}if(G){ag=84;ah=15810;break L22053}if(w>>>0<4){aa=22;break L22053}ae=(d[_+H|0]<<8|d[_+I|0])-56320|0;if(ae>>>0>1023){ag=84;ah=15810;break L22053}ai=4;aj=(ab<<10)-56557568+ae|0;ah=15786;break L22055;break};case 217:case 216:{break};case 198:{if(w>>>0<4){aa=22;break L22053}au=c[_>>2]|0;ah=15710;break};case 192:case 195:{if(w>>>0<4){aa=22;break L22053}au=d[_+B|0]<<16|d[_+t|0]<<24|d[_+C|0]<<8|d[_+D|0];ah=15710;break};case 218:{if((x&255)<161){ag=84;ah=15810;break L22053}break};case 208:{if(w>>>0<2){aa=22;break L22053}ae=d[_+1|0]|0;if(x<<24>>24==-114){if((ae-161|0)>>>0>62){ag=84;ah=15810;break L22053}ai=2;aj=ae+65216|0;ah=15786;break L22055}else{ab=$-161|0;af=ae-161|0;if(ab>>>0>83|af>>>0>93){ag=84;ah=15810;break L22053}ae=b[156928+(ab*188|0)+(af<<1)>>1]|0;if(ae<<16>>16==0){ag=84;ah=15810;break L22053}else{ai=2;aj=ae&65535;ah=15786;break L22055}}break};case 224:{if(w>>>0<2){aa=22;break L22053}ae=d[_+1|0]|0;af=ae-64|0;if(af>>>0>190|(ae-127|0)>>>0<34){ag=84;ah=15810;break L22053}ab=af>>>0>62?ae-98|0:af;af=$-161|0;if(af>>>0<=88){ae=e[308360+(af*314|0)+(ab<<1)>>1]|0;do{if((af|0)==39){if((ab|0)==60|(ab|0)==58){av=131072;break}av=((ab|0)==66)<<17}else{av=0}}while(0);af=av|ae;if((af|0)==0){ag=84;ah=15810;break L22053}else{ai=2;aj=af;ah=15786;break L22055}}af=$-135|0;if(af>>>0>119){ag=84;ah=15810;break L22053}ad=ab+(((x&255)<161?af:$-224|0)*157|0)|0;af=e[175112+(ad<<1)>>1]|0;ac=af|(((e[175112+((ad>>>4)+4867<<1)>>1]|0)>>>((ad&15)>>>0)|0)%2|0)<<17;if((ac&-65792|0)!=56320){if((ac|0)==0){ag=84;ah=15810;break L22053}else{ai=2;aj=ac;ah=15786;break L22055}}if(M){aw=2}else{aw=a[O]|0}if(aw>>>0>(c[k>>2]|0)>>>0){ag=7;ah=15810;break L22053}c[p>>2]=1729528+(af&255);c[q>>2]=4;ax=(Ny(N,p,q,j,k)|0)+Z|0;ay=2;break L22055;break};case 199:{ag=84;ah=15810;break L22053;break};default:{if($>>>0<z>>>0){ai=1;aj=$;ah=15786;break L22055}af=$-z|0;ac=(af*5|0)>>>2;ad=af<<1&6;af=d[299824+(A+ac)|0]<<8-ad&1023|(d[299824+(ac+s)|0]|0)>>>(ad>>>0);ad=(af|0)==0?$:e[132448+(af<<1)>>1]|0;if((ad|0)==1){ag=84;ah=15810;break L22053}else{ai=1;aj=ad;ah=15786;break L22055}}}if((ah|0)==15710){ah=0;if((au-55296|0)>>>0<2048|au>>>0>1114111){ag=84;ah=15810;break L22053}else{ai=4;aj=au;ah=15786;break}}ad=$-129|0;if(ad>>>0>125){ag=84;ah=15810;break L22053}if(w>>>0<2){aa=22;break L22053}af=a[_+1|0]|0;ac=af&255;if((af&255)<161&J){ag=84;ah=15810;break L22053}ap=ac-64|0;if(!(ap>>>0>190|af<<24>>24==127)){ai=2;aj=e[185480+(ad*380|0)+((ap>>>0>63?ac-65|0:ap)<<1)>>1]|0;ah=15786;break}if(!((ac-48|0)>>>0<10&K)){ag=84;ah=15810;break L22053}if(w>>>0<4){aa=22;break L22053}ap=(d[_+2|0]|0)-129|0;if(ap>>>0>126){ag=84;ah=15810;break L22053}af=(d[_+3|0]|0)-48|0;if(af>>>0>9){ag=84;ah=15810;break L22053}az=((ac+(ad*10|0)|0)*1260|0)-60352+(ap*10|0)+af|0;af=0;while(1){ap=az-af|0;ad=0;ac=0;while(1){aA=ad;aB=0;do{aA=(((e[185480+(ac*380|0)+(aB<<1)>>1]|0)-af|0)>>>0<=ap>>>0)+aA|0;aB=aB+1|0;}while((aB|0)<190);aB=ac+1|0;if((aB|0)<126){ad=aA;ac=aB}else{break}}ac=az+1|0;ad=aA+az|0;if(ac>>>0>ad>>>0){ai=4;aj=ad;ah=15786;break}else{az=ad;af=ac}}}else{ai=1;aj=$;ah=15786}}while(0);L22162:do{if((ah|0)==15786){ah=0;switch(m|0){case 192:case 195:{if((c[k>>2]|0)>>>0<4){ag=7;ah=15810;break L22053}$=c[j>>2]|0;a[$+U|0]=aj>>>24&255;a[$+V|0]=aj>>>16&255;a[$+W|0]=aj>>>8&255;a[$+X|0]=aj&255;c[j>>2]=$+4;c[k>>2]=(c[k>>2]|0)-4;ax=Z;ay=ai;break L22162;break};case 198:{if((c[k>>2]|0)>>>0<4){ag=7;ah=15810;break L22053}$=c[j>>2]|0;c[$>>2]=aj;c[j>>2]=$+4;c[k>>2]=(c[k>>2]|0)-4;ax=Z;ay=ai;break L22162;break};case 199:{if(aj>>>0>127){aC=Z;ah=15795}else{aD=Z;aE=aj}break};case 200:{if((c[k>>2]|0)>>>0<4){$=NB(f,aj)|0;if((c[k>>2]|0)>>>0<$>>>0){ag=7;ah=15810;break L22053}x=c[j>>2]|0;NN(x|0,f|0,$)|0;aF=$;aG=x}else{x=c[j>>2]|0;aF=NB(x,aj)|0;aG=x}c[j>>2]=aG+aF;c[k>>2]=(c[k>>2]|0)-aF;ax=Z;ay=ai;break L22162;break};case 196:case 197:case 194:case 193:{if(aj>>>0<65536|G){x=aj>>>0>65535?65533:aj;if((c[k>>2]|0)>>>0<2){ag=7;ah=15810;break L22053}$=c[j>>2]|0;a[$+r|0]=x>>>8&255;a[$+R|0]=x&255;c[j>>2]=$+2;c[k>>2]=(c[k>>2]|0)-2;ax=Z;ay=ai;break L22162}else{if((c[k>>2]|0)>>>0<4){ag=7;ah=15810;break L22053}$=aj-65536|0;x=c[j>>2]|0;a[x+r|0]=($>>>18|216)&255;a[x+R|0]=$>>>10&255;a[x+S|0]=($>>>8&3|220)&255;a[x+T|0]=$&255;c[j>>2]=x+4;c[k>>2]=(c[k>>2]|0)-4;ax=Z;ay=ai;break L22162}break};default:{aD=Z;aE=aj}}L22182:while(1){if((ah|0)==15795){ah=0;aD=aC+1|0;aE=42}if((c[k>>2]|0)==0){ag=7;ah=15810;break L22053}if(aE>>>0<L>>>0){aH=aE&255;break}else{aI=0}while(1){if(aI>>>0>=P>>>0){aC=aD;ah=15795;continue L22182}x=(aI*5|0)>>>2;$=aI<<1&6;if((aE|0)==(e[132448+((d[299824+(Q+x)|0]<<8-$&1023|(d[299824+(x+u)|0]|0)>>>($>>>0))<<1)>>1]|0)){ah=15801;break L22182}else{aI=aI+1|0}}}if((ah|0)==15801){ah=0;aH=aI+128&255}$=c[j>>2]|0;c[j>>2]=$+1;a[$]=aH;c[k>>2]=(c[k>>2]|0)-1;ax=aD;ay=ai}}while(0);$=(c[g>>2]|0)+ay|0;c[g>>2]=$;x=w-ay|0;c[h>>2]=x;if((w|0)==(ay|0)){v=ax;ah=15813;break}else{Z=ax;_=$;w=x}}if((ah|0)==15813){i=l;return v|0}else if((ah|0)==15810){aa=ag}c[(ch()|0)>>2]=aa;v=-1;i=l;return v|0}function Nz(b,d,e,f){b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,r=0,s=0,t=0,u=0,v=0,w=0;g=i;i=i+8|0;h=g|0;c[h>>2]=b;j=((f|0)==0?2203720:f)|0;f=c[j>>2]|0;L22200:do{if((d|0)==0){if((f|0)==0){k=0}else{break}i=g;return k|0}else{if((b|0)==0){l=h;c[h>>2]=l;m=l}else{m=b}if((e|0)==0){k=-2;i=g;return k|0}do{if((f|0)==0){l=a[d]|0;n=l&255;if(l<<24>>24>-1){c[m>>2]=n;k=l<<24>>24!=0|0;i=g;return k|0}else{l=n-194|0;if(l>>>0>50){break L22200}o=d+1|0;p=c[q+(l<<2)>>2]|0;r=e-1|0;break}}else{o=d;p=f;r=e}}while(0);L22218:do{if((r|0)==0){s=p}else{l=a[o]|0;n=(l&255)>>>3;if((n-16|n+(p>>26))>>>0>7){break L22200}else{t=o;u=p;v=r;w=l}while(1){t=t+1|0;u=(w&255)-128|u<<6;v=v-1|0;if((u|0)>=0){break}if((v|0)==0){s=u;break L22218}w=a[t]|0;if(((w&255)-128|0)>>>0>63){break L22200}}c[j>>2]=0;c[m>>2]=u;k=e-v|0;i=g;return k|0}}while(0);c[j>>2]=s;k=-2;i=g;return k|0}}while(0);c[j>>2]=0;c[(ch()|0)>>2]=84;k=-1;i=g;return k|0}function NA(b,d,e){b=b|0;d=d|0;e=e|0;var f=0;if((b|0)==0){f=1;return f|0}if(d>>>0<128){a[b]=d&255;f=1;return f|0}if(d>>>0<2048){a[b]=(d>>>6|192)&255;a[b+1|0]=(d&63|128)&255;f=2;return f|0}if(d>>>0<55296|(d-57344|0)>>>0<8192){a[b]=(d>>>12|224)&255;a[b+1|0]=(d>>>6&63|128)&255;a[b+2|0]=(d&63|128)&255;f=3;return f|0}if((d-65536|0)>>>0<1048576){a[b]=(d>>>18|240)&255;a[b+1|0]=(d>>>12&63|128)&255;a[b+2|0]=(d>>>6&63|128)&255;a[b+3|0]=(d&63|128)&255;f=4;return f|0}else{c[(ch()|0)>>2]=84;f=-1;return f|0}return 0}function NB(a,b){a=a|0;b=b|0;var c=0;if((a|0)==0){c=0}else{c=NA(a,b,0)|0}return c|0}function NC(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0,P=0,Q=0,R=0,S=0,T=0,U=0,V=0,W=0,X=0,Y=0,Z=0,_=0,$=0,aa=0,ab=0,ac=0,ad=0,ae=0,af=0,ag=0,ah=0,ai=0,aj=0,ak=0,al=0,am=0,an=0,ao=0,ap=0,aq=0,ar=0,as=0,at=0,au=0,av=0,aw=0,ax=0,ay=0,az=0,aA=0,aB=0,aC=0,aD=0,aE=0,aF=0,aG=0;do{if(a>>>0<245){if(a>>>0<11){b=16}else{b=a+11&-8}d=b>>>3;e=c[779796]|0;f=e>>>(d>>>0);if((f&3|0)!=0){g=(f&1^1)+d|0;h=g<<1;i=3119224+(h<<2)|0;j=3119224+(h+2<<2)|0;h=c[j>>2]|0;k=h+8|0;l=c[k>>2]|0;do{if((i|0)==(l|0)){c[779796]=e&~(1<<g)}else{if(l>>>0<(c[779800]|0)>>>0){cw();return 0}m=l+12|0;if((c[m>>2]|0)==(h|0)){c[m>>2]=i;c[j>>2]=l;break}else{cw();return 0}}}while(0);l=g<<3;c[h+4>>2]=l|3;j=h+(l|4)|0;c[j>>2]=c[j>>2]|1;n=k;return n|0}if(b>>>0<=(c[779798]|0)>>>0){o=b;break}if((f|0)!=0){j=2<<d;l=f<<d&(j|-j);j=(l&-l)-1|0;l=j>>>12&16;i=j>>>(l>>>0);j=i>>>5&8;m=i>>>(j>>>0);i=m>>>2&4;p=m>>>(i>>>0);m=p>>>1&2;q=p>>>(m>>>0);p=q>>>1&1;r=(j|l|i|m|p)+(q>>>(p>>>0))|0;p=r<<1;q=3119224+(p<<2)|0;m=3119224+(p+2<<2)|0;p=c[m>>2]|0;i=p+8|0;l=c[i>>2]|0;do{if((q|0)==(l|0)){c[779796]=e&~(1<<r)}else{if(l>>>0<(c[779800]|0)>>>0){cw();return 0}j=l+12|0;if((c[j>>2]|0)==(p|0)){c[j>>2]=q;c[m>>2]=l;break}else{cw();return 0}}}while(0);l=r<<3;m=l-b|0;c[p+4>>2]=b|3;q=p;e=q+b|0;c[q+(b|4)>>2]=m|1;c[q+l>>2]=m;l=c[779798]|0;if((l|0)!=0){q=c[779801]|0;d=l>>>3;l=d<<1;f=3119224+(l<<2)|0;k=c[779796]|0;h=1<<d;do{if((k&h|0)==0){c[779796]=k|h;s=f;t=3119224+(l+2<<2)|0}else{d=3119224+(l+2<<2)|0;g=c[d>>2]|0;if(g>>>0>=(c[779800]|0)>>>0){s=g;t=d;break}cw();return 0}}while(0);c[t>>2]=q;c[s+12>>2]=q;c[q+8>>2]=s;c[q+12>>2]=f}c[779798]=m;c[779801]=e;n=i;return n|0}l=c[779797]|0;if((l|0)==0){o=b;break}h=(l&-l)-1|0;l=h>>>12&16;k=h>>>(l>>>0);h=k>>>5&8;p=k>>>(h>>>0);k=p>>>2&4;r=p>>>(k>>>0);p=r>>>1&2;d=r>>>(p>>>0);r=d>>>1&1;g=c[3119488+((h|l|k|p|r)+(d>>>(r>>>0))<<2)>>2]|0;r=g;d=g;p=(c[g+4>>2]&-8)-b|0;while(1){g=c[r+16>>2]|0;if((g|0)==0){k=c[r+20>>2]|0;if((k|0)==0){break}else{u=k}}else{u=g}g=(c[u+4>>2]&-8)-b|0;k=g>>>0<p>>>0;r=u;d=k?u:d;p=k?g:p}r=d;i=c[779800]|0;if(r>>>0<i>>>0){cw();return 0}e=r+b|0;m=e;if(r>>>0>=e>>>0){cw();return 0}e=c[d+24>>2]|0;f=c[d+12>>2]|0;do{if((f|0)==(d|0)){q=d+20|0;g=c[q>>2]|0;if((g|0)==0){k=d+16|0;l=c[k>>2]|0;if((l|0)==0){v=0;break}else{w=l;x=k}}else{w=g;x=q}while(1){q=w+20|0;g=c[q>>2]|0;if((g|0)!=0){w=g;x=q;continue}q=w+16|0;g=c[q>>2]|0;if((g|0)==0){break}else{w=g;x=q}}if(x>>>0<i>>>0){cw();return 0}else{c[x>>2]=0;v=w;break}}else{q=c[d+8>>2]|0;if(q>>>0<i>>>0){cw();return 0}g=q+12|0;if((c[g>>2]|0)!=(d|0)){cw();return 0}k=f+8|0;if((c[k>>2]|0)==(d|0)){c[g>>2]=f;c[k>>2]=q;v=f;break}else{cw();return 0}}}while(0);L78:do{if((e|0)!=0){f=d+28|0;i=3119488+(c[f>>2]<<2)|0;do{if((d|0)==(c[i>>2]|0)){c[i>>2]=v;if((v|0)!=0){break}c[779797]=c[779797]&~(1<<c[f>>2]);break L78}else{if(e>>>0<(c[779800]|0)>>>0){cw();return 0}q=e+16|0;if((c[q>>2]|0)==(d|0)){c[q>>2]=v}else{c[e+20>>2]=v}if((v|0)==0){break L78}}}while(0);if(v>>>0<(c[779800]|0)>>>0){cw();return 0}c[v+24>>2]=e;f=c[d+16>>2]|0;do{if((f|0)!=0){if(f>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[v+16>>2]=f;c[f+24>>2]=v;break}}}while(0);f=c[d+20>>2]|0;if((f|0)==0){break}if(f>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[v+20>>2]=f;c[f+24>>2]=v;break}}}while(0);if(p>>>0<16){e=p+b|0;c[d+4>>2]=e|3;f=r+(e+4)|0;c[f>>2]=c[f>>2]|1}else{c[d+4>>2]=b|3;c[r+(b|4)>>2]=p|1;c[r+(p+b)>>2]=p;f=c[779798]|0;if((f|0)!=0){e=c[779801]|0;i=f>>>3;f=i<<1;q=3119224+(f<<2)|0;k=c[779796]|0;g=1<<i;do{if((k&g|0)==0){c[779796]=k|g;y=q;z=3119224+(f+2<<2)|0}else{i=3119224+(f+2<<2)|0;l=c[i>>2]|0;if(l>>>0>=(c[779800]|0)>>>0){y=l;z=i;break}cw();return 0}}while(0);c[z>>2]=e;c[y+12>>2]=e;c[e+8>>2]=y;c[e+12>>2]=q}c[779798]=p;c[779801]=m}f=d+8|0;if((f|0)==0){o=b;break}else{n=f}return n|0}else{if(a>>>0>4294967231){o=-1;break}f=a+11|0;g=f&-8;k=c[779797]|0;if((k|0)==0){o=g;break}r=-g|0;i=f>>>8;do{if((i|0)==0){A=0}else{if(g>>>0>16777215){A=31;break}f=(i+1048320|0)>>>16&8;l=i<<f;h=(l+520192|0)>>>16&4;j=l<<h;l=(j+245760|0)>>>16&2;B=14-(h|f|l)+(j<<l>>>15)|0;A=g>>>((B+7|0)>>>0)&1|B<<1}}while(0);i=c[3119488+(A<<2)>>2]|0;L126:do{if((i|0)==0){C=0;D=r;E=0}else{if((A|0)==31){F=0}else{F=25-(A>>>1)|0}d=0;m=r;p=i;q=g<<F;e=0;while(1){B=c[p+4>>2]&-8;l=B-g|0;if(l>>>0<m>>>0){if((B|0)==(g|0)){C=p;D=l;E=p;break L126}else{G=p;H=l}}else{G=d;H=m}l=c[p+20>>2]|0;B=c[p+16+(q>>>31<<2)>>2]|0;j=(l|0)==0|(l|0)==(B|0)?e:l;if((B|0)==0){C=G;D=H;E=j;break}else{d=G;m=H;p=B;q=q<<1;e=j}}}}while(0);if((E|0)==0&(C|0)==0){i=2<<A;r=k&(i|-i);if((r|0)==0){o=g;break}i=(r&-r)-1|0;r=i>>>12&16;e=i>>>(r>>>0);i=e>>>5&8;q=e>>>(i>>>0);e=q>>>2&4;p=q>>>(e>>>0);q=p>>>1&2;m=p>>>(q>>>0);p=m>>>1&1;I=c[3119488+((i|r|e|q|p)+(m>>>(p>>>0))<<2)>>2]|0}else{I=E}if((I|0)==0){J=D;K=C}else{p=I;m=D;q=C;while(1){e=(c[p+4>>2]&-8)-g|0;r=e>>>0<m>>>0;i=r?e:m;e=r?p:q;r=c[p+16>>2]|0;if((r|0)!=0){p=r;m=i;q=e;continue}r=c[p+20>>2]|0;if((r|0)==0){J=i;K=e;break}else{p=r;m=i;q=e}}}if((K|0)==0){o=g;break}if(J>>>0>=((c[779798]|0)-g|0)>>>0){o=g;break}q=K;m=c[779800]|0;if(q>>>0<m>>>0){cw();return 0}p=q+g|0;k=p;if(q>>>0>=p>>>0){cw();return 0}e=c[K+24>>2]|0;i=c[K+12>>2]|0;do{if((i|0)==(K|0)){r=K+20|0;d=c[r>>2]|0;if((d|0)==0){j=K+16|0;B=c[j>>2]|0;if((B|0)==0){L=0;break}else{M=B;N=j}}else{M=d;N=r}while(1){r=M+20|0;d=c[r>>2]|0;if((d|0)!=0){M=d;N=r;continue}r=M+16|0;d=c[r>>2]|0;if((d|0)==0){break}else{M=d;N=r}}if(N>>>0<m>>>0){cw();return 0}else{c[N>>2]=0;L=M;break}}else{r=c[K+8>>2]|0;if(r>>>0<m>>>0){cw();return 0}d=r+12|0;if((c[d>>2]|0)!=(K|0)){cw();return 0}j=i+8|0;if((c[j>>2]|0)==(K|0)){c[d>>2]=i;c[j>>2]=r;L=i;break}else{cw();return 0}}}while(0);L176:do{if((e|0)!=0){i=K+28|0;m=3119488+(c[i>>2]<<2)|0;do{if((K|0)==(c[m>>2]|0)){c[m>>2]=L;if((L|0)!=0){break}c[779797]=c[779797]&~(1<<c[i>>2]);break L176}else{if(e>>>0<(c[779800]|0)>>>0){cw();return 0}r=e+16|0;if((c[r>>2]|0)==(K|0)){c[r>>2]=L}else{c[e+20>>2]=L}if((L|0)==0){break L176}}}while(0);if(L>>>0<(c[779800]|0)>>>0){cw();return 0}c[L+24>>2]=e;i=c[K+16>>2]|0;do{if((i|0)!=0){if(i>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[L+16>>2]=i;c[i+24>>2]=L;break}}}while(0);i=c[K+20>>2]|0;if((i|0)==0){break}if(i>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[L+20>>2]=i;c[i+24>>2]=L;break}}}while(0);do{if(J>>>0<16){e=J+g|0;c[K+4>>2]=e|3;i=q+(e+4)|0;c[i>>2]=c[i>>2]|1}else{c[K+4>>2]=g|3;c[q+(g|4)>>2]=J|1;c[q+(J+g)>>2]=J;i=J>>>3;if(J>>>0<256){e=i<<1;m=3119224+(e<<2)|0;r=c[779796]|0;j=1<<i;do{if((r&j|0)==0){c[779796]=r|j;O=m;P=3119224+(e+2<<2)|0}else{i=3119224+(e+2<<2)|0;d=c[i>>2]|0;if(d>>>0>=(c[779800]|0)>>>0){O=d;P=i;break}cw();return 0}}while(0);c[P>>2]=k;c[O+12>>2]=k;c[q+(g+8)>>2]=O;c[q+(g+12)>>2]=m;break}e=p;j=J>>>8;do{if((j|0)==0){Q=0}else{if(J>>>0>16777215){Q=31;break}r=(j+1048320|0)>>>16&8;i=j<<r;d=(i+520192|0)>>>16&4;B=i<<d;i=(B+245760|0)>>>16&2;l=14-(d|r|i)+(B<<i>>>15)|0;Q=J>>>((l+7|0)>>>0)&1|l<<1}}while(0);j=3119488+(Q<<2)|0;c[q+(g+28)>>2]=Q;c[q+(g+20)>>2]=0;c[q+(g+16)>>2]=0;m=c[779797]|0;l=1<<Q;if((m&l|0)==0){c[779797]=m|l;c[j>>2]=e;c[q+(g+24)>>2]=j;c[q+(g+12)>>2]=e;c[q+(g+8)>>2]=e;break}if((Q|0)==31){R=0}else{R=25-(Q>>>1)|0}l=J<<R;m=c[j>>2]|0;while(1){if((c[m+4>>2]&-8|0)==(J|0)){break}S=m+16+(l>>>31<<2)|0;j=c[S>>2]|0;if((j|0)==0){T=151;break}else{l=l<<1;m=j}}if((T|0)==151){if(S>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[S>>2]=e;c[q+(g+24)>>2]=m;c[q+(g+12)>>2]=e;c[q+(g+8)>>2]=e;break}}l=m+8|0;j=c[l>>2]|0;i=c[779800]|0;if(m>>>0<i>>>0){cw();return 0}if(j>>>0<i>>>0){cw();return 0}else{c[j+12>>2]=e;c[l>>2]=e;c[q+(g+8)>>2]=j;c[q+(g+12)>>2]=m;c[q+(g+24)>>2]=0;break}}}while(0);q=K+8|0;if((q|0)==0){o=g;break}else{n=q}return n|0}}while(0);K=c[779798]|0;if(o>>>0<=K>>>0){S=K-o|0;J=c[779801]|0;if(S>>>0>15){R=J;c[779801]=R+o;c[779798]=S;c[R+(o+4)>>2]=S|1;c[R+K>>2]=S;c[J+4>>2]=o|3}else{c[779798]=0;c[779801]=0;c[J+4>>2]=K|3;S=J+(K+4)|0;c[S>>2]=c[S>>2]|1}n=J+8|0;return n|0}J=c[779799]|0;if(o>>>0<J>>>0){S=J-o|0;c[779799]=S;J=c[779802]|0;K=J;c[779802]=K+o;c[K+(o+4)>>2]=S|1;c[J+4>>2]=o|3;n=J+8|0;return n|0}do{if((c[550924]|0)==0){J=aP(30)|0;if((J-1&J|0)==0){c[550926]=J;c[550925]=J;c[550927]=-1;c[550928]=-1;c[550929]=0;c[779907]=0;c[550924]=(cU(0)|0)&-16^1431655768;break}else{cw();return 0}}}while(0);J=o+48|0;S=c[550926]|0;K=o+47|0;R=S+K|0;Q=-S|0;S=R&Q;if(S>>>0<=o>>>0){n=0;return n|0}O=c[779906]|0;do{if((O|0)!=0){P=c[779904]|0;L=P+S|0;if(L>>>0<=P>>>0|L>>>0>O>>>0){n=0}else{break}return n|0}}while(0);L268:do{if((c[779907]&4|0)==0){O=c[779802]|0;L270:do{if((O|0)==0){T=181}else{L=O;P=3119632;while(1){U=P|0;M=c[U>>2]|0;if(M>>>0<=L>>>0){V=P+4|0;if((M+(c[V>>2]|0)|0)>>>0>L>>>0){break}}M=c[P+8>>2]|0;if((M|0)==0){T=181;break L270}else{P=M}}if((P|0)==0){T=181;break}L=R-(c[779799]|0)&Q;if(L>>>0>=2147483647){W=0;break}m=cf(L|0)|0;e=(m|0)==((c[U>>2]|0)+(c[V>>2]|0)|0);X=e?m:-1;Y=e?L:0;Z=m;_=L;T=190}}while(0);do{if((T|0)==181){O=cf(0)|0;if((O|0)==-1){W=0;break}g=O;L=c[550925]|0;m=L-1|0;if((m&g|0)==0){$=S}else{$=S-g+(m+g&-L)|0}L=c[779904]|0;g=L+$|0;if(!($>>>0>o>>>0&$>>>0<2147483647)){W=0;break}m=c[779906]|0;if((m|0)!=0){if(g>>>0<=L>>>0|g>>>0>m>>>0){W=0;break}}m=cf($|0)|0;g=(m|0)==(O|0);X=g?O:-1;Y=g?$:0;Z=m;_=$;T=190}}while(0);L290:do{if((T|0)==190){m=-_|0;if((X|0)!=-1){aa=Y;ab=X;T=201;break L268}do{if((Z|0)!=-1&_>>>0<2147483647&_>>>0<J>>>0){g=c[550926]|0;O=K-_+g&-g;if(O>>>0>=2147483647){ac=_;break}if((cf(O|0)|0)==-1){cf(m|0)|0;W=Y;break L290}else{ac=O+_|0;break}}else{ac=_}}while(0);if((Z|0)==-1){W=Y}else{aa=ac;ab=Z;T=201;break L268}}}while(0);c[779907]=c[779907]|4;ad=W;T=198}else{ad=0;T=198}}while(0);do{if((T|0)==198){if(S>>>0>=2147483647){break}W=cf(S|0)|0;Z=cf(0)|0;if(!((Z|0)!=-1&(W|0)!=-1&W>>>0<Z>>>0)){break}ac=Z-W|0;Z=ac>>>0>(o+40|0)>>>0;Y=Z?W:-1;if((Y|0)!=-1){aa=Z?ac:ad;ab=Y;T=201}}}while(0);do{if((T|0)==201){ad=(c[779904]|0)+aa|0;c[779904]=ad;if(ad>>>0>(c[779905]|0)>>>0){c[779905]=ad}ad=c[779802]|0;L310:do{if((ad|0)==0){S=c[779800]|0;if((S|0)==0|ab>>>0<S>>>0){c[779800]=ab}c[779908]=ab;c[779909]=aa;c[779911]=0;c[779805]=c[550924];c[779804]=-1;S=0;do{Y=S<<1;ac=3119224+(Y<<2)|0;c[3119224+(Y+3<<2)>>2]=ac;c[3119224+(Y+2<<2)>>2]=ac;S=S+1|0;}while(S>>>0<32);S=ab+8|0;if((S&7|0)==0){ae=0}else{ae=-S&7}S=aa-40-ae|0;c[779802]=ab+ae;c[779799]=S;c[ab+(ae+4)>>2]=S|1;c[ab+(aa-36)>>2]=40;c[779803]=c[550928]}else{S=3119632;while(1){af=c[S>>2]|0;ag=S+4|0;ah=c[ag>>2]|0;if((ab|0)==(af+ah|0)){T=213;break}ac=c[S+8>>2]|0;if((ac|0)==0){break}else{S=ac}}do{if((T|0)==213){if((c[S+12>>2]&8|0)!=0){break}ac=ad;if(!(ac>>>0>=af>>>0&ac>>>0<ab>>>0)){break}c[ag>>2]=ah+aa;ac=c[779802]|0;Y=(c[779799]|0)+aa|0;Z=ac;W=ac+8|0;if((W&7|0)==0){ai=0}else{ai=-W&7}W=Y-ai|0;c[779802]=Z+ai;c[779799]=W;c[Z+(ai+4)>>2]=W|1;c[Z+(Y+4)>>2]=40;c[779803]=c[550928];break L310}}while(0);if(ab>>>0<(c[779800]|0)>>>0){c[779800]=ab}S=ab+aa|0;Y=3119632;while(1){aj=Y|0;if((c[aj>>2]|0)==(S|0)){T=223;break}Z=c[Y+8>>2]|0;if((Z|0)==0){break}else{Y=Z}}do{if((T|0)==223){if((c[Y+12>>2]&8|0)!=0){break}c[aj>>2]=ab;S=Y+4|0;c[S>>2]=(c[S>>2]|0)+aa;S=ab+8|0;if((S&7|0)==0){ak=0}else{ak=-S&7}S=ab+(aa+8)|0;if((S&7|0)==0){al=0}else{al=-S&7}S=ab+(al+aa)|0;Z=S;W=ak+o|0;ac=ab+W|0;_=ac;K=S-(ab+ak)-o|0;c[ab+(ak+4)>>2]=o|3;do{if((Z|0)==(c[779802]|0)){J=(c[779799]|0)+K|0;c[779799]=J;c[779802]=_;c[ab+(W+4)>>2]=J|1}else{if((Z|0)==(c[779801]|0)){J=(c[779798]|0)+K|0;c[779798]=J;c[779801]=_;c[ab+(W+4)>>2]=J|1;c[ab+(J+W)>>2]=J;break}J=aa+4|0;X=c[ab+(J+al)>>2]|0;if((X&3|0)==1){$=X&-8;V=X>>>3;L355:do{if(X>>>0<256){U=c[ab+((al|8)+aa)>>2]|0;Q=c[ab+(aa+12+al)>>2]|0;R=3119224+(V<<1<<2)|0;do{if((U|0)!=(R|0)){if(U>>>0<(c[779800]|0)>>>0){cw();return 0}if((c[U+12>>2]|0)==(Z|0)){break}cw();return 0}}while(0);if((Q|0)==(U|0)){c[779796]=c[779796]&~(1<<V);break}do{if((Q|0)==(R|0)){am=Q+8|0}else{if(Q>>>0<(c[779800]|0)>>>0){cw();return 0}m=Q+8|0;if((c[m>>2]|0)==(Z|0)){am=m;break}cw();return 0}}while(0);c[U+12>>2]=Q;c[am>>2]=U}else{R=S;m=c[ab+((al|24)+aa)>>2]|0;P=c[ab+(aa+12+al)>>2]|0;do{if((P|0)==(R|0)){O=al|16;g=ab+(J+O)|0;L=c[g>>2]|0;if((L|0)==0){e=ab+(O+aa)|0;O=c[e>>2]|0;if((O|0)==0){an=0;break}else{ao=O;ap=e}}else{ao=L;ap=g}while(1){g=ao+20|0;L=c[g>>2]|0;if((L|0)!=0){ao=L;ap=g;continue}g=ao+16|0;L=c[g>>2]|0;if((L|0)==0){break}else{ao=L;ap=g}}if(ap>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[ap>>2]=0;an=ao;break}}else{g=c[ab+((al|8)+aa)>>2]|0;if(g>>>0<(c[779800]|0)>>>0){cw();return 0}L=g+12|0;if((c[L>>2]|0)!=(R|0)){cw();return 0}e=P+8|0;if((c[e>>2]|0)==(R|0)){c[L>>2]=P;c[e>>2]=g;an=P;break}else{cw();return 0}}}while(0);if((m|0)==0){break}P=ab+(aa+28+al)|0;U=3119488+(c[P>>2]<<2)|0;do{if((R|0)==(c[U>>2]|0)){c[U>>2]=an;if((an|0)!=0){break}c[779797]=c[779797]&~(1<<c[P>>2]);break L355}else{if(m>>>0<(c[779800]|0)>>>0){cw();return 0}Q=m+16|0;if((c[Q>>2]|0)==(R|0)){c[Q>>2]=an}else{c[m+20>>2]=an}if((an|0)==0){break L355}}}while(0);if(an>>>0<(c[779800]|0)>>>0){cw();return 0}c[an+24>>2]=m;R=al|16;P=c[ab+(R+aa)>>2]|0;do{if((P|0)!=0){if(P>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[an+16>>2]=P;c[P+24>>2]=an;break}}}while(0);P=c[ab+(J+R)>>2]|0;if((P|0)==0){break}if(P>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[an+20>>2]=P;c[P+24>>2]=an;break}}}while(0);aq=ab+(($|al)+aa)|0;ar=$+K|0}else{aq=Z;ar=K}J=aq+4|0;c[J>>2]=c[J>>2]&-2;c[ab+(W+4)>>2]=ar|1;c[ab+(ar+W)>>2]=ar;J=ar>>>3;if(ar>>>0<256){V=J<<1;X=3119224+(V<<2)|0;P=c[779796]|0;m=1<<J;do{if((P&m|0)==0){c[779796]=P|m;as=X;at=3119224+(V+2<<2)|0}else{J=3119224+(V+2<<2)|0;U=c[J>>2]|0;if(U>>>0>=(c[779800]|0)>>>0){as=U;at=J;break}cw();return 0}}while(0);c[at>>2]=_;c[as+12>>2]=_;c[ab+(W+8)>>2]=as;c[ab+(W+12)>>2]=X;break}V=ac;m=ar>>>8;do{if((m|0)==0){au=0}else{if(ar>>>0>16777215){au=31;break}P=(m+1048320|0)>>>16&8;$=m<<P;J=($+520192|0)>>>16&4;U=$<<J;$=(U+245760|0)>>>16&2;Q=14-(J|P|$)+(U<<$>>>15)|0;au=ar>>>((Q+7|0)>>>0)&1|Q<<1}}while(0);m=3119488+(au<<2)|0;c[ab+(W+28)>>2]=au;c[ab+(W+20)>>2]=0;c[ab+(W+16)>>2]=0;X=c[779797]|0;Q=1<<au;if((X&Q|0)==0){c[779797]=X|Q;c[m>>2]=V;c[ab+(W+24)>>2]=m;c[ab+(W+12)>>2]=V;c[ab+(W+8)>>2]=V;break}if((au|0)==31){av=0}else{av=25-(au>>>1)|0}Q=ar<<av;X=c[m>>2]|0;while(1){if((c[X+4>>2]&-8|0)==(ar|0)){break}aw=X+16+(Q>>>31<<2)|0;m=c[aw>>2]|0;if((m|0)==0){T=296;break}else{Q=Q<<1;X=m}}if((T|0)==296){if(aw>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[aw>>2]=V;c[ab+(W+24)>>2]=X;c[ab+(W+12)>>2]=V;c[ab+(W+8)>>2]=V;break}}Q=X+8|0;m=c[Q>>2]|0;$=c[779800]|0;if(X>>>0<$>>>0){cw();return 0}if(m>>>0<$>>>0){cw();return 0}else{c[m+12>>2]=V;c[Q>>2]=V;c[ab+(W+8)>>2]=m;c[ab+(W+12)>>2]=X;c[ab+(W+24)>>2]=0;break}}}while(0);n=ab+(ak|8)|0;return n|0}}while(0);Y=ad;W=3119632;while(1){ax=c[W>>2]|0;if(ax>>>0<=Y>>>0){ay=c[W+4>>2]|0;az=ax+ay|0;if(az>>>0>Y>>>0){break}}W=c[W+8>>2]|0}W=ax+(ay-39)|0;if((W&7|0)==0){aA=0}else{aA=-W&7}W=ax+(ay-47+aA)|0;ac=W>>>0<(ad+16|0)>>>0?Y:W;W=ac+8|0;_=ab+8|0;if((_&7|0)==0){aB=0}else{aB=-_&7}_=aa-40-aB|0;c[779802]=ab+aB;c[779799]=_;c[ab+(aB+4)>>2]=_|1;c[ab+(aa-36)>>2]=40;c[779803]=c[550928];c[ac+4>>2]=27;c[W>>2]=c[779908];c[W+4>>2]=c[779909];c[W+8>>2]=c[779910];c[W+12>>2]=c[779911];c[779908]=ab;c[779909]=aa;c[779911]=0;c[779910]=W;W=ac+28|0;c[W>>2]=7;if((ac+32|0)>>>0<az>>>0){_=W;while(1){W=_+4|0;c[W>>2]=7;if((_+8|0)>>>0<az>>>0){_=W}else{break}}}if((ac|0)==(Y|0)){break}_=ac-ad|0;W=Y+(_+4)|0;c[W>>2]=c[W>>2]&-2;c[ad+4>>2]=_|1;c[Y+_>>2]=_;W=_>>>3;if(_>>>0<256){K=W<<1;Z=3119224+(K<<2)|0;S=c[779796]|0;m=1<<W;do{if((S&m|0)==0){c[779796]=S|m;aC=Z;aD=3119224+(K+2<<2)|0}else{W=3119224+(K+2<<2)|0;Q=c[W>>2]|0;if(Q>>>0>=(c[779800]|0)>>>0){aC=Q;aD=W;break}cw();return 0}}while(0);c[aD>>2]=ad;c[aC+12>>2]=ad;c[ad+8>>2]=aC;c[ad+12>>2]=Z;break}K=ad;m=_>>>8;do{if((m|0)==0){aE=0}else{if(_>>>0>16777215){aE=31;break}S=(m+1048320|0)>>>16&8;Y=m<<S;ac=(Y+520192|0)>>>16&4;W=Y<<ac;Y=(W+245760|0)>>>16&2;Q=14-(ac|S|Y)+(W<<Y>>>15)|0;aE=_>>>((Q+7|0)>>>0)&1|Q<<1}}while(0);m=3119488+(aE<<2)|0;c[ad+28>>2]=aE;c[ad+20>>2]=0;c[ad+16>>2]=0;Z=c[779797]|0;Q=1<<aE;if((Z&Q|0)==0){c[779797]=Z|Q;c[m>>2]=K;c[ad+24>>2]=m;c[ad+12>>2]=ad;c[ad+8>>2]=ad;break}if((aE|0)==31){aF=0}else{aF=25-(aE>>>1)|0}Q=_<<aF;Z=c[m>>2]|0;while(1){if((c[Z+4>>2]&-8|0)==(_|0)){break}aG=Z+16+(Q>>>31<<2)|0;m=c[aG>>2]|0;if((m|0)==0){T=331;break}else{Q=Q<<1;Z=m}}if((T|0)==331){if(aG>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[aG>>2]=K;c[ad+24>>2]=Z;c[ad+12>>2]=ad;c[ad+8>>2]=ad;break}}Q=Z+8|0;_=c[Q>>2]|0;m=c[779800]|0;if(Z>>>0<m>>>0){cw();return 0}if(_>>>0<m>>>0){cw();return 0}else{c[_+12>>2]=K;c[Q>>2]=K;c[ad+8>>2]=_;c[ad+12>>2]=Z;c[ad+24>>2]=0;break}}}while(0);ad=c[779799]|0;if(ad>>>0<=o>>>0){break}_=ad-o|0;c[779799]=_;ad=c[779802]|0;Q=ad;c[779802]=Q+o;c[Q+(o+4)>>2]=_|1;c[ad+4>>2]=o|3;n=ad+8|0;return n|0}}while(0);c[(ch()|0)>>2]=12;n=0;return n|0}function ND(a){a=a|0;var b=0,d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0,O=0;if((a|0)==0){return}b=a-8|0;d=b;e=c[779800]|0;if(b>>>0<e>>>0){cw()}f=c[a-4>>2]|0;g=f&3;if((g|0)==1){cw()}h=f&-8;i=a+(h-8)|0;j=i;L527:do{if((f&1|0)==0){k=c[b>>2]|0;if((g|0)==0){return}l=-8-k|0;m=a+l|0;n=m;o=k+h|0;if(m>>>0<e>>>0){cw()}if((n|0)==(c[779801]|0)){p=a+(h-4)|0;if((c[p>>2]&3|0)!=3){q=n;r=o;break}c[779798]=o;c[p>>2]=c[p>>2]&-2;c[a+(l+4)>>2]=o|1;c[i>>2]=o;return}p=k>>>3;if(k>>>0<256){k=c[a+(l+8)>>2]|0;s=c[a+(l+12)>>2]|0;t=3119224+(p<<1<<2)|0;do{if((k|0)!=(t|0)){if(k>>>0<e>>>0){cw()}if((c[k+12>>2]|0)==(n|0)){break}cw()}}while(0);if((s|0)==(k|0)){c[779796]=c[779796]&~(1<<p);q=n;r=o;break}do{if((s|0)==(t|0)){u=s+8|0}else{if(s>>>0<e>>>0){cw()}v=s+8|0;if((c[v>>2]|0)==(n|0)){u=v;break}cw()}}while(0);c[k+12>>2]=s;c[u>>2]=k;q=n;r=o;break}t=m;p=c[a+(l+24)>>2]|0;v=c[a+(l+12)>>2]|0;do{if((v|0)==(t|0)){w=a+(l+20)|0;x=c[w>>2]|0;if((x|0)==0){y=a+(l+16)|0;z=c[y>>2]|0;if((z|0)==0){A=0;break}else{B=z;C=y}}else{B=x;C=w}while(1){w=B+20|0;x=c[w>>2]|0;if((x|0)!=0){B=x;C=w;continue}w=B+16|0;x=c[w>>2]|0;if((x|0)==0){break}else{B=x;C=w}}if(C>>>0<e>>>0){cw()}else{c[C>>2]=0;A=B;break}}else{w=c[a+(l+8)>>2]|0;if(w>>>0<e>>>0){cw()}x=w+12|0;if((c[x>>2]|0)!=(t|0)){cw()}y=v+8|0;if((c[y>>2]|0)==(t|0)){c[x>>2]=v;c[y>>2]=w;A=v;break}else{cw()}}}while(0);if((p|0)==0){q=n;r=o;break}v=a+(l+28)|0;m=3119488+(c[v>>2]<<2)|0;do{if((t|0)==(c[m>>2]|0)){c[m>>2]=A;if((A|0)!=0){break}c[779797]=c[779797]&~(1<<c[v>>2]);q=n;r=o;break L527}else{if(p>>>0<(c[779800]|0)>>>0){cw()}k=p+16|0;if((c[k>>2]|0)==(t|0)){c[k>>2]=A}else{c[p+20>>2]=A}if((A|0)==0){q=n;r=o;break L527}}}while(0);if(A>>>0<(c[779800]|0)>>>0){cw()}c[A+24>>2]=p;t=c[a+(l+16)>>2]|0;do{if((t|0)!=0){if(t>>>0<(c[779800]|0)>>>0){cw()}else{c[A+16>>2]=t;c[t+24>>2]=A;break}}}while(0);t=c[a+(l+20)>>2]|0;if((t|0)==0){q=n;r=o;break}if(t>>>0<(c[779800]|0)>>>0){cw()}else{c[A+20>>2]=t;c[t+24>>2]=A;q=n;r=o;break}}else{q=d;r=h}}while(0);d=q;if(d>>>0>=i>>>0){cw()}A=a+(h-4)|0;e=c[A>>2]|0;if((e&1|0)==0){cw()}do{if((e&2|0)==0){if((j|0)==(c[779802]|0)){B=(c[779799]|0)+r|0;c[779799]=B;c[779802]=q;c[q+4>>2]=B|1;if((q|0)!=(c[779801]|0)){return}c[779801]=0;c[779798]=0;return}if((j|0)==(c[779801]|0)){B=(c[779798]|0)+r|0;c[779798]=B;c[779801]=q;c[q+4>>2]=B|1;c[d+B>>2]=B;return}B=(e&-8)+r|0;C=e>>>3;L630:do{if(e>>>0<256){u=c[a+h>>2]|0;g=c[a+(h|4)>>2]|0;b=3119224+(C<<1<<2)|0;do{if((u|0)!=(b|0)){if(u>>>0<(c[779800]|0)>>>0){cw()}if((c[u+12>>2]|0)==(j|0)){break}cw()}}while(0);if((g|0)==(u|0)){c[779796]=c[779796]&~(1<<C);break}do{if((g|0)==(b|0)){D=g+8|0}else{if(g>>>0<(c[779800]|0)>>>0){cw()}f=g+8|0;if((c[f>>2]|0)==(j|0)){D=f;break}cw()}}while(0);c[u+12>>2]=g;c[D>>2]=u}else{b=i;f=c[a+(h+16)>>2]|0;t=c[a+(h|4)>>2]|0;do{if((t|0)==(b|0)){p=a+(h+12)|0;v=c[p>>2]|0;if((v|0)==0){m=a+(h+8)|0;k=c[m>>2]|0;if((k|0)==0){E=0;break}else{F=k;G=m}}else{F=v;G=p}while(1){p=F+20|0;v=c[p>>2]|0;if((v|0)!=0){F=v;G=p;continue}p=F+16|0;v=c[p>>2]|0;if((v|0)==0){break}else{F=v;G=p}}if(G>>>0<(c[779800]|0)>>>0){cw()}else{c[G>>2]=0;E=F;break}}else{p=c[a+h>>2]|0;if(p>>>0<(c[779800]|0)>>>0){cw()}v=p+12|0;if((c[v>>2]|0)!=(b|0)){cw()}m=t+8|0;if((c[m>>2]|0)==(b|0)){c[v>>2]=t;c[m>>2]=p;E=t;break}else{cw()}}}while(0);if((f|0)==0){break}t=a+(h+20)|0;u=3119488+(c[t>>2]<<2)|0;do{if((b|0)==(c[u>>2]|0)){c[u>>2]=E;if((E|0)!=0){break}c[779797]=c[779797]&~(1<<c[t>>2]);break L630}else{if(f>>>0<(c[779800]|0)>>>0){cw()}g=f+16|0;if((c[g>>2]|0)==(b|0)){c[g>>2]=E}else{c[f+20>>2]=E}if((E|0)==0){break L630}}}while(0);if(E>>>0<(c[779800]|0)>>>0){cw()}c[E+24>>2]=f;b=c[a+(h+8)>>2]|0;do{if((b|0)!=0){if(b>>>0<(c[779800]|0)>>>0){cw()}else{c[E+16>>2]=b;c[b+24>>2]=E;break}}}while(0);b=c[a+(h+12)>>2]|0;if((b|0)==0){break}if(b>>>0<(c[779800]|0)>>>0){cw()}else{c[E+20>>2]=b;c[b+24>>2]=E;break}}}while(0);c[q+4>>2]=B|1;c[d+B>>2]=B;if((q|0)!=(c[779801]|0)){H=B;break}c[779798]=B;return}else{c[A>>2]=e&-2;c[q+4>>2]=r|1;c[d+r>>2]=r;H=r}}while(0);r=H>>>3;if(H>>>0<256){d=r<<1;e=3119224+(d<<2)|0;A=c[779796]|0;E=1<<r;do{if((A&E|0)==0){c[779796]=A|E;I=e;J=3119224+(d+2<<2)|0}else{r=3119224+(d+2<<2)|0;h=c[r>>2]|0;if(h>>>0>=(c[779800]|0)>>>0){I=h;J=r;break}cw()}}while(0);c[J>>2]=q;c[I+12>>2]=q;c[q+8>>2]=I;c[q+12>>2]=e;return}e=q;I=H>>>8;do{if((I|0)==0){K=0}else{if(H>>>0>16777215){K=31;break}J=(I+1048320|0)>>>16&8;d=I<<J;E=(d+520192|0)>>>16&4;A=d<<E;d=(A+245760|0)>>>16&2;r=14-(E|J|d)+(A<<d>>>15)|0;K=H>>>((r+7|0)>>>0)&1|r<<1}}while(0);I=3119488+(K<<2)|0;c[q+28>>2]=K;c[q+20>>2]=0;c[q+16>>2]=0;r=c[779797]|0;d=1<<K;do{if((r&d|0)==0){c[779797]=r|d;c[I>>2]=e;c[q+24>>2]=I;c[q+12>>2]=q;c[q+8>>2]=q}else{if((K|0)==31){L=0}else{L=25-(K>>>1)|0}A=H<<L;J=c[I>>2]|0;while(1){if((c[J+4>>2]&-8|0)==(H|0)){break}M=J+16+(A>>>31<<2)|0;E=c[M>>2]|0;if((E|0)==0){N=508;break}else{A=A<<1;J=E}}if((N|0)==508){if(M>>>0<(c[779800]|0)>>>0){cw()}else{c[M>>2]=e;c[q+24>>2]=J;c[q+12>>2]=q;c[q+8>>2]=q;break}}A=J+8|0;B=c[A>>2]|0;E=c[779800]|0;if(J>>>0<E>>>0){cw()}if(B>>>0<E>>>0){cw()}else{c[B+12>>2]=e;c[A>>2]=e;c[q+8>>2]=B;c[q+12>>2]=J;c[q+24>>2]=0;break}}}while(0);q=(c[779804]|0)-1|0;c[779804]=q;if((q|0)==0){O=3119640}else{return}while(1){q=c[O>>2]|0;if((q|0)==0){break}else{O=q+8|0}}c[779804]=-1;return}function NE(a,b){a=a|0;b=b|0;var d=0,e=0;do{if((a|0)==0){d=0}else{e=ac(b,a)|0;if((b|a)>>>0<=65535){d=e;break}d=((e>>>0)/(a>>>0)|0|0)==(b|0)?e:-1}}while(0);b=NC(d)|0;if((b|0)==0){return b|0}if((c[b-4>>2]&3|0)==0){return b|0}NM(b|0,0,d|0);return b|0}function NF(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0;if((a|0)==0){d=NC(b)|0;return d|0}if(b>>>0>4294967231){c[(ch()|0)>>2]=12;d=0;return d|0}if(b>>>0<11){e=16}else{e=b+11&-8}f=NG(a-8|0,e)|0;if((f|0)!=0){d=f+8|0;return d|0}f=NC(b)|0;if((f|0)==0){d=0;return d|0}e=c[a-4>>2]|0;g=(e&-8)-((e&3|0)==0?8:4)|0;e=g>>>0<b>>>0?g:b;NN(f|0,a|0,e)|0;ND(a);d=f;return d|0}
function NG(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0;d=a+4|0;e=c[d>>2]|0;f=e&-8;g=a;h=g+f|0;i=h;j=c[779800]|0;if(g>>>0<j>>>0){cw();return 0}k=e&3;if(!((k|0)!=1&g>>>0<h>>>0)){cw();return 0}l=g+(f|4)|0;m=c[l>>2]|0;if((m&1|0)==0){cw();return 0}if((k|0)==0){if(b>>>0<256){n=0;return n|0}do{if(f>>>0>=(b+4|0)>>>0){if((f-b|0)>>>0>c[550926]<<1>>>0){break}else{n=a}return n|0}}while(0);n=0;return n|0}if(f>>>0>=b>>>0){k=f-b|0;if(k>>>0<=15){n=a;return n|0}c[d>>2]=e&1|b|2;c[g+(b+4)>>2]=k|3;c[l>>2]=c[l>>2]|1;NJ(g+b|0,k);n=a;return n|0}if((i|0)==(c[779802]|0)){k=(c[779799]|0)+f|0;if(k>>>0<=b>>>0){n=0;return n|0}l=k-b|0;c[d>>2]=e&1|b|2;c[g+(b+4)>>2]=l|1;c[779802]=g+b;c[779799]=l;n=a;return n|0}if((i|0)==(c[779801]|0)){l=(c[779798]|0)+f|0;if(l>>>0<b>>>0){n=0;return n|0}k=l-b|0;if(k>>>0>15){c[d>>2]=e&1|b|2;c[g+(b+4)>>2]=k|1;c[g+l>>2]=k;o=g+(l+4)|0;c[o>>2]=c[o>>2]&-2;p=g+b|0;q=k}else{c[d>>2]=e&1|l|2;e=g+(l+4)|0;c[e>>2]=c[e>>2]|1;p=0;q=0}c[779798]=q;c[779801]=p;n=a;return n|0}if((m&2|0)!=0){n=0;return n|0}p=(m&-8)+f|0;if(p>>>0<b>>>0){n=0;return n|0}q=p-b|0;e=m>>>3;L828:do{if(m>>>0<256){l=c[g+(f+8)>>2]|0;k=c[g+(f+12)>>2]|0;o=3119224+(e<<1<<2)|0;do{if((l|0)!=(o|0)){if(l>>>0<j>>>0){cw();return 0}if((c[l+12>>2]|0)==(i|0)){break}cw();return 0}}while(0);if((k|0)==(l|0)){c[779796]=c[779796]&~(1<<e);break}do{if((k|0)==(o|0)){r=k+8|0}else{if(k>>>0<j>>>0){cw();return 0}s=k+8|0;if((c[s>>2]|0)==(i|0)){r=s;break}cw();return 0}}while(0);c[l+12>>2]=k;c[r>>2]=l}else{o=h;s=c[g+(f+24)>>2]|0;t=c[g+(f+12)>>2]|0;do{if((t|0)==(o|0)){u=g+(f+20)|0;v=c[u>>2]|0;if((v|0)==0){w=g+(f+16)|0;x=c[w>>2]|0;if((x|0)==0){y=0;break}else{z=x;A=w}}else{z=v;A=u}while(1){u=z+20|0;v=c[u>>2]|0;if((v|0)!=0){z=v;A=u;continue}u=z+16|0;v=c[u>>2]|0;if((v|0)==0){break}else{z=v;A=u}}if(A>>>0<j>>>0){cw();return 0}else{c[A>>2]=0;y=z;break}}else{u=c[g+(f+8)>>2]|0;if(u>>>0<j>>>0){cw();return 0}v=u+12|0;if((c[v>>2]|0)!=(o|0)){cw();return 0}w=t+8|0;if((c[w>>2]|0)==(o|0)){c[v>>2]=t;c[w>>2]=u;y=t;break}else{cw();return 0}}}while(0);if((s|0)==0){break}t=g+(f+28)|0;l=3119488+(c[t>>2]<<2)|0;do{if((o|0)==(c[l>>2]|0)){c[l>>2]=y;if((y|0)!=0){break}c[779797]=c[779797]&~(1<<c[t>>2]);break L828}else{if(s>>>0<(c[779800]|0)>>>0){cw();return 0}k=s+16|0;if((c[k>>2]|0)==(o|0)){c[k>>2]=y}else{c[s+20>>2]=y}if((y|0)==0){break L828}}}while(0);if(y>>>0<(c[779800]|0)>>>0){cw();return 0}c[y+24>>2]=s;o=c[g+(f+16)>>2]|0;do{if((o|0)!=0){if(o>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[y+16>>2]=o;c[o+24>>2]=y;break}}}while(0);o=c[g+(f+20)>>2]|0;if((o|0)==0){break}if(o>>>0<(c[779800]|0)>>>0){cw();return 0}else{c[y+20>>2]=o;c[o+24>>2]=y;break}}}while(0);if(q>>>0<16){c[d>>2]=p|c[d>>2]&1|2;y=g+(p|4)|0;c[y>>2]=c[y>>2]|1;n=a;return n|0}else{c[d>>2]=c[d>>2]&1|b|2;c[g+(b+4)>>2]=q|3;d=g+(p|4)|0;c[d>>2]=c[d>>2]|1;NJ(g+b|0,q);n=a;return n|0}return 0}function NH(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0;d=a>>>0<16?16:a;if((d-1&d|0)==0){e=d}else{a=16;while(1){if(a>>>0<d>>>0){a=a<<1}else{e=a;break}}}if((-64-e|0)>>>0<=b>>>0){c[(ch()|0)>>2]=12;f=0;return f|0}if(b>>>0<11){g=16}else{g=b+11&-8}b=NC(e+12+g|0)|0;if((b|0)==0){f=0;return f|0}a=b-8|0;d=a;h=e-1|0;do{if((b&h|0)==0){i=d}else{j=b+h&-e;k=j-8|0;l=a;if((k-l|0)>>>0>15){m=k}else{m=j+(e-8)|0}j=m;k=m-l|0;l=b-4|0;n=c[l>>2]|0;o=(n&-8)-k|0;if((n&3|0)==0){c[m>>2]=(c[a>>2]|0)+k;c[m+4>>2]=o;i=j;break}else{n=m+4|0;c[n>>2]=o|c[n>>2]&1|2;n=m+(o+4)|0;c[n>>2]=c[n>>2]|1;c[l>>2]=k|c[l>>2]&1|2;l=b+(k-4)|0;c[l>>2]=c[l>>2]|1;NJ(d,k);i=j;break}}}while(0);d=i+4|0;b=c[d>>2]|0;do{if((b&3|0)!=0){m=b&-8;if(m>>>0<=(g+16|0)>>>0){break}a=m-g|0;e=i;c[d>>2]=g|b&1|2;c[e+(g|4)>>2]=a|3;h=e+(m|4)|0;c[h>>2]=c[h>>2]|1;NJ(e+g|0,a)}}while(0);f=i+8|0;return f|0}function NI(a,b,d){a=a|0;b=b|0;d=d|0;var e=0,f=0,g=0;do{if((b|0)==8){e=NC(d)|0}else{f=b>>>2;if((b&3|0)!=0|(f|0)==0){g=22;return g|0}if((f+1073741823&f|0)!=0){g=22;return g|0}if((-64-b|0)>>>0<d>>>0){g=12;return g|0}else{e=NH(b>>>0<16?16:b,d)|0;break}}}while(0);if((e|0)==0){g=12;return g|0}c[a>>2]=e;g=0;return g|0}function NJ(a,b){a=a|0;b=b|0;var d=0,e=0,f=0,g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,G=0,H=0,I=0,J=0,K=0,L=0;d=a;e=d+b|0;f=e;g=c[a+4>>2]|0;L951:do{if((g&1|0)==0){h=c[a>>2]|0;if((g&3|0)==0){return}i=d+(-h|0)|0;j=i;k=h+b|0;l=c[779800]|0;if(i>>>0<l>>>0){cw()}if((j|0)==(c[779801]|0)){m=d+(b+4)|0;if((c[m>>2]&3|0)!=3){n=j;o=k;break}c[779798]=k;c[m>>2]=c[m>>2]&-2;c[d+(4-h)>>2]=k|1;c[e>>2]=k;return}m=h>>>3;if(h>>>0<256){p=c[d+(8-h)>>2]|0;q=c[d+(12-h)>>2]|0;r=3119224+(m<<1<<2)|0;do{if((p|0)!=(r|0)){if(p>>>0<l>>>0){cw()}if((c[p+12>>2]|0)==(j|0)){break}cw()}}while(0);if((q|0)==(p|0)){c[779796]=c[779796]&~(1<<m);n=j;o=k;break}do{if((q|0)==(r|0)){s=q+8|0}else{if(q>>>0<l>>>0){cw()}t=q+8|0;if((c[t>>2]|0)==(j|0)){s=t;break}cw()}}while(0);c[p+12>>2]=q;c[s>>2]=p;n=j;o=k;break}r=i;m=c[d+(24-h)>>2]|0;t=c[d+(12-h)>>2]|0;do{if((t|0)==(r|0)){u=16-h|0;v=d+(u+4)|0;w=c[v>>2]|0;if((w|0)==0){x=d+u|0;u=c[x>>2]|0;if((u|0)==0){y=0;break}else{z=u;A=x}}else{z=w;A=v}while(1){v=z+20|0;w=c[v>>2]|0;if((w|0)!=0){z=w;A=v;continue}v=z+16|0;w=c[v>>2]|0;if((w|0)==0){break}else{z=w;A=v}}if(A>>>0<l>>>0){cw()}else{c[A>>2]=0;y=z;break}}else{v=c[d+(8-h)>>2]|0;if(v>>>0<l>>>0){cw()}w=v+12|0;if((c[w>>2]|0)!=(r|0)){cw()}x=t+8|0;if((c[x>>2]|0)==(r|0)){c[w>>2]=t;c[x>>2]=v;y=t;break}else{cw()}}}while(0);if((m|0)==0){n=j;o=k;break}t=d+(28-h)|0;l=3119488+(c[t>>2]<<2)|0;do{if((r|0)==(c[l>>2]|0)){c[l>>2]=y;if((y|0)!=0){break}c[779797]=c[779797]&~(1<<c[t>>2]);n=j;o=k;break L951}else{if(m>>>0<(c[779800]|0)>>>0){cw()}i=m+16|0;if((c[i>>2]|0)==(r|0)){c[i>>2]=y}else{c[m+20>>2]=y}if((y|0)==0){n=j;o=k;break L951}}}while(0);if(y>>>0<(c[779800]|0)>>>0){cw()}c[y+24>>2]=m;r=16-h|0;t=c[d+r>>2]|0;do{if((t|0)!=0){if(t>>>0<(c[779800]|0)>>>0){cw()}else{c[y+16>>2]=t;c[t+24>>2]=y;break}}}while(0);t=c[d+(r+4)>>2]|0;if((t|0)==0){n=j;o=k;break}if(t>>>0<(c[779800]|0)>>>0){cw()}else{c[y+20>>2]=t;c[t+24>>2]=y;n=j;o=k;break}}else{n=a;o=b}}while(0);a=c[779800]|0;if(e>>>0<a>>>0){cw()}y=d+(b+4)|0;z=c[y>>2]|0;do{if((z&2|0)==0){if((f|0)==(c[779802]|0)){A=(c[779799]|0)+o|0;c[779799]=A;c[779802]=n;c[n+4>>2]=A|1;if((n|0)!=(c[779801]|0)){return}c[779801]=0;c[779798]=0;return}if((f|0)==(c[779801]|0)){A=(c[779798]|0)+o|0;c[779798]=A;c[779801]=n;c[n+4>>2]=A|1;c[n+A>>2]=A;return}A=(z&-8)+o|0;s=z>>>3;L1050:do{if(z>>>0<256){g=c[d+(b+8)>>2]|0;t=c[d+(b+12)>>2]|0;h=3119224+(s<<1<<2)|0;do{if((g|0)!=(h|0)){if(g>>>0<a>>>0){cw()}if((c[g+12>>2]|0)==(f|0)){break}cw()}}while(0);if((t|0)==(g|0)){c[779796]=c[779796]&~(1<<s);break}do{if((t|0)==(h|0)){B=t+8|0}else{if(t>>>0<a>>>0){cw()}m=t+8|0;if((c[m>>2]|0)==(f|0)){B=m;break}cw()}}while(0);c[g+12>>2]=t;c[B>>2]=g}else{h=e;m=c[d+(b+24)>>2]|0;l=c[d+(b+12)>>2]|0;do{if((l|0)==(h|0)){i=d+(b+20)|0;p=c[i>>2]|0;if((p|0)==0){q=d+(b+16)|0;v=c[q>>2]|0;if((v|0)==0){C=0;break}else{D=v;E=q}}else{D=p;E=i}while(1){i=D+20|0;p=c[i>>2]|0;if((p|0)!=0){D=p;E=i;continue}i=D+16|0;p=c[i>>2]|0;if((p|0)==0){break}else{D=p;E=i}}if(E>>>0<a>>>0){cw()}else{c[E>>2]=0;C=D;break}}else{i=c[d+(b+8)>>2]|0;if(i>>>0<a>>>0){cw()}p=i+12|0;if((c[p>>2]|0)!=(h|0)){cw()}q=l+8|0;if((c[q>>2]|0)==(h|0)){c[p>>2]=l;c[q>>2]=i;C=l;break}else{cw()}}}while(0);if((m|0)==0){break}l=d+(b+28)|0;g=3119488+(c[l>>2]<<2)|0;do{if((h|0)==(c[g>>2]|0)){c[g>>2]=C;if((C|0)!=0){break}c[779797]=c[779797]&~(1<<c[l>>2]);break L1050}else{if(m>>>0<(c[779800]|0)>>>0){cw()}t=m+16|0;if((c[t>>2]|0)==(h|0)){c[t>>2]=C}else{c[m+20>>2]=C}if((C|0)==0){break L1050}}}while(0);if(C>>>0<(c[779800]|0)>>>0){cw()}c[C+24>>2]=m;h=c[d+(b+16)>>2]|0;do{if((h|0)!=0){if(h>>>0<(c[779800]|0)>>>0){cw()}else{c[C+16>>2]=h;c[h+24>>2]=C;break}}}while(0);h=c[d+(b+20)>>2]|0;if((h|0)==0){break}if(h>>>0<(c[779800]|0)>>>0){cw()}else{c[C+20>>2]=h;c[h+24>>2]=C;break}}}while(0);c[n+4>>2]=A|1;c[n+A>>2]=A;if((n|0)!=(c[779801]|0)){F=A;break}c[779798]=A;return}else{c[y>>2]=z&-2;c[n+4>>2]=o|1;c[n+o>>2]=o;F=o}}while(0);o=F>>>3;if(F>>>0<256){z=o<<1;y=3119224+(z<<2)|0;C=c[779796]|0;b=1<<o;do{if((C&b|0)==0){c[779796]=C|b;G=y;H=3119224+(z+2<<2)|0}else{o=3119224+(z+2<<2)|0;d=c[o>>2]|0;if(d>>>0>=(c[779800]|0)>>>0){G=d;H=o;break}cw()}}while(0);c[H>>2]=n;c[G+12>>2]=n;c[n+8>>2]=G;c[n+12>>2]=y;return}y=n;G=F>>>8;do{if((G|0)==0){I=0}else{if(F>>>0>16777215){I=31;break}H=(G+1048320|0)>>>16&8;z=G<<H;b=(z+520192|0)>>>16&4;C=z<<b;z=(C+245760|0)>>>16&2;o=14-(b|H|z)+(C<<z>>>15)|0;I=F>>>((o+7|0)>>>0)&1|o<<1}}while(0);G=3119488+(I<<2)|0;c[n+28>>2]=I;c[n+20>>2]=0;c[n+16>>2]=0;o=c[779797]|0;z=1<<I;if((o&z|0)==0){c[779797]=o|z;c[G>>2]=y;c[n+24>>2]=G;c[n+12>>2]=n;c[n+8>>2]=n;return}if((I|0)==31){J=0}else{J=25-(I>>>1)|0}I=F<<J;J=c[G>>2]|0;while(1){if((c[J+4>>2]&-8|0)==(F|0)){break}K=J+16+(I>>>31<<2)|0;G=c[K>>2]|0;if((G|0)==0){L=833;break}else{I=I<<1;J=G}}if((L|0)==833){if(K>>>0<(c[779800]|0)>>>0){cw()}c[K>>2]=y;c[n+24>>2]=J;c[n+12>>2]=n;c[n+8>>2]=n;return}K=J+8|0;L=c[K>>2]|0;I=c[779800]|0;if(J>>>0<I>>>0){cw()}if(L>>>0<I>>>0){cw()}c[L+12>>2]=y;c[K>>2]=y;c[n+8>>2]=L;c[n+12>>2]=J;c[n+24>>2]=0;return}function NK(b,d){b=b|0;d=d|0;var e=0,f=0,g=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0.0,r=0,s=0,t=0,u=0,v=0.0,w=0,x=0,y=0,z=0.0,A=0.0,B=0,C=0,D=0,E=0.0,F=0,G=0,H=0,I=0,J=0,K=0,L=0,M=0,N=0.0,O=0,P=0,Q=0.0,R=0.0,S=0.0;e=b;while(1){f=e+1|0;if((a$(a[e]|0)|0)==0){break}else{e=f}}g=a[e]|0;if((g<<24>>24|0)==45){i=f;j=1}else if((g<<24>>24|0)==43){i=f;j=0}else{i=e;j=0}e=-1;f=0;g=i;while(1){k=a[g]|0;if(((k<<24>>24)-48|0)>>>0<10){l=e}else{if(k<<24>>24!=46|(e|0)>-1){break}else{l=f}}e=l;f=f+1|0;g=g+1|0}l=g+(-f|0)|0;i=(e|0)<0;m=((i^1)<<31>>31)+f|0;n=(m|0)>18;o=(n?-18:-m|0)+(i?f:e)|0;e=n?18:m;do{if((e|0)==0){p=b;q=0.0}else{if((e|0)>9){m=l;n=e;f=0;while(1){i=a[m]|0;r=m+1|0;if(i<<24>>24==46){s=a[r]|0;t=m+2|0}else{s=i;t=r}u=(f*10|0)-48+(s<<24>>24)|0;r=n-1|0;if((r|0)>9){m=t;n=r;f=u}else{break}}v=+(u|0)*1.0e9;w=9;x=t;y=881}else{if((e|0)>0){v=0.0;w=e;x=l;y=881}else{z=0.0;A=0.0}}if((y|0)==881){f=x;n=w;m=0;while(1){r=a[f]|0;i=f+1|0;if(r<<24>>24==46){B=a[i]|0;C=f+2|0}else{B=r;C=i}D=(m*10|0)-48+(B<<24>>24)|0;i=n-1|0;if((i|0)>0){f=C;n=i;m=D}else{break}}z=+(D|0);A=v}E=A+z;do{if((k<<24>>24|0)==69|(k<<24>>24|0)==101){m=g+1|0;n=a[m]|0;if((n<<24>>24|0)==45){F=g+2|0;G=1}else if((n<<24>>24|0)==43){F=g+2|0;G=0}else{F=m;G=0}m=a[F]|0;if(((m<<24>>24)-48|0)>>>0<10){H=F;I=0;J=m}else{K=0;L=F;M=G;break}while(1){m=(I*10|0)-48+(J<<24>>24)|0;n=H+1|0;f=a[n]|0;if(((f<<24>>24)-48|0)>>>0<10){H=n;I=m;J=f}else{K=m;L=n;M=G;break}}}else{K=0;L=g;M=0}}while(0);n=o+((M|0)==0?K:-K|0)|0;m=(n|0)<0?-n|0:n;if((m|0)>511){c[(ch()|0)>>2]=34;N=1.0;O=90888;P=511;y=898}else{if((m|0)==0){Q=1.0}else{N=1.0;O=90888;P=m;y=898}}if((y|0)==898){while(1){y=0;if((P&1|0)==0){R=N}else{R=N*+h[O>>3]}m=P>>1;if((m|0)==0){Q=R;break}else{N=R;O=O+8|0;P=m;y=898}}}if((n|0)>-1){p=L;q=E*Q;break}else{p=L;q=E/Q;break}}}while(0);if((d|0)!=0){c[d>>2]=p}if((j|0)==0){S=q;return+S}S=-0.0-q;return+S}function NL(b){b=b|0;var c=0;c=b;while(a[c]|0){c=c+1|0}return c-b|0}function NM(b,d,e){b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0;f=b+e|0;if((e|0)>=20){d=d&255;e=b&3;g=d|d<<8|d<<16|d<<24;h=f&~3;if(e){e=b+4-e|0;while((b|0)<(e|0)){a[b]=d;b=b+1|0}}while((b|0)<(h|0)){c[b>>2]=g;b=b+4|0}}while((b|0)<(f|0)){a[b]=d;b=b+1|0}}function NN(b,d,e){b=b|0;d=d|0;e=e|0;var f=0;f=b|0;if((b&3)==(d&3)){while(b&3){if((e|0)==0)return f|0;a[b]=a[d]|0;b=b+1|0;d=d+1|0;e=e-1|0}while((e|0)>=4){c[b>>2]=c[d>>2];b=b+4|0;d=d+4|0;e=e-4|0}}while((e|0)>0){a[b]=a[d]|0;b=b+1|0;d=d+1|0;e=e-1|0}return f|0}function NO(a,b,c){a=a|0;b=b|0;c=c|0;var e=0,f=0,g=0;while((e|0)<(c|0)){f=d[a+e|0]|0;g=d[b+e|0]|0;if((f|0)!=(g|0))return((f|0)>(g|0)?1:-1)|0;e=e+1|0}return 0}function NP(b,c,d){b=b|0;c=c|0;d=d|0;if((c|0)<(b|0)&(b|0)<(c+d|0)){c=c+d|0;b=b+d|0;while((d|0)>0){b=b-1|0;c=c-1|0;d=d-1|0;a[b]=a[c]|0}}else{NN(b,c,d)|0}}function NQ(b,c){b=b|0;c=c|0;var d=0;do{a[b+d|0]=a[c+d|0];d=d+1|0}while(a[c+(d-1)|0]|0);return b|0}function NR(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;e=a+c>>>0;c=b+d>>>0;d=c>>>0<b>>>0|0;if(e>>>0<a>>>0){c=c+1>>>0;d=d|!c}return(G=c,H=d,e|0)|0}function NS(a){a=a|0;if((a|0)<65)return a|0;if((a|0)>90)return a|0;return a-65+97|0}function NT(b,c){b=b|0;c=c|0;var d=0,e=0;d=b+(NL(b)|0)|0;do{a[d+e|0]=a[c+e|0];e=e+1|0}while(a[c+(e-1)|0]|0);return b|0}function NU(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;e=a+c>>>0;return(G=b+d+(e>>>0<a>>>0|0)>>>0,e|0)|0}function NV(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;e=b-d>>>0;e=b-d-(c>>>0>a>>>0|0)>>>0;return(G=e,a-c>>>0|0)|0}function NW(a,b,c){a=a|0;b=b|0;c=c|0;if((c|0)<32){G=b<<c|(a&(1<<c)-1<<32-c)>>>32-c;return a<<c}G=a<<c-32;return 0}function NX(a,b,c){a=a|0;b=b|0;c=c|0;if((c|0)<32){G=b>>>c;return a>>>c|(b&(1<<c)-1)<<32-c}G=0;return b>>>c-32|0}function NY(a,b,c){a=a|0;b=b|0;c=c|0;if((c|0)<32){G=b>>c;return a>>>c|(b&(1<<c)-1)<<32-c}G=(b|0)<0?-1:0;return b>>c-32|0}function NZ(b){b=b|0;var c=0;c=a[n+(b>>>24)|0]|0;if((c|0)<8)return c|0;c=a[n+(b>>16&255)|0]|0;if((c|0)<8)return c+8|0;c=a[n+(b>>8&255)|0]|0;if((c|0)<8)return c+16|0;return(a[n+(b&255)|0]|0)+24|0}function N_(b){b=b|0;var c=0;c=a[m+(b&255)|0]|0;if((c|0)<8)return c|0;c=a[m+(b>>8&255)|0]|0;if((c|0)<8)return c+8|0;c=a[m+(b>>16&255)|0]|0;if((c|0)<8)return c+16|0;return(a[m+(b>>>24)|0]|0)+24|0}function N$(a,b){a=a|0;b=b|0;var c=0,d=0,e=0,f=0;c=a&65535;d=b&65535;e=ac(d,c)|0;f=a>>>16;a=(e>>>16)+(ac(d,f)|0)|0;d=b>>>16;b=ac(d,c)|0;return(G=(a>>>16)+(ac(d,f)|0)+(((a&65535)+b|0)>>>16)|0,a+b<<16|e&65535|0)|0}function N0(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0,g=0,h=0,i=0;e=b>>31|((b|0)<0?-1:0)<<1;f=((b|0)<0?-1:0)>>31|((b|0)<0?-1:0)<<1;g=d>>31|((d|0)<0?-1:0)<<1;h=((d|0)<0?-1:0)>>31|((d|0)<0?-1:0)<<1;i=NV(e^a,f^b,e,f)|0;b=G;a=g^e;e=h^f;f=NV((N5(i,b,NV(g^c,h^d,g,h)|0,G,0)|0)^a,G^e,a,e)|0;return(G=G,f)|0}function N1(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0,h=0,j=0,k=0,l=0,m=0;f=i;i=i+8|0;g=f|0;h=b>>31|((b|0)<0?-1:0)<<1;j=((b|0)<0?-1:0)>>31|((b|0)<0?-1:0)<<1;k=e>>31|((e|0)<0?-1:0)<<1;l=((e|0)<0?-1:0)>>31|((e|0)<0?-1:0)<<1;m=NV(h^a,j^b,h,j)|0;b=G;a=NV(k^d,l^e,k,l)|0;N5(m,b,a,G,g)|0;a=NV(c[g>>2]^h,c[g+4>>2]^j,h,j)|0;j=G;i=f;return(G=j,a)|0}function N2(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0,f=0;e=a;a=c;c=N$(e,a)|0;f=G;return(G=(ac(b,a)|0)+(ac(d,e)|0)+f|f&0,c|0|0)|0}function N3(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;var e=0;e=N5(a,b,c,d,0)|0;return(G=G,e)|0}function N4(a,b,d,e){a=a|0;b=b|0;d=d|0;e=e|0;var f=0,g=0;f=i;i=i+8|0;g=f|0;N5(a,b,d,e,g)|0;i=f;return(G=c[g+4>>2]|0,c[g>>2]|0)|0}function N5(a,b,d,e,f){a=a|0;b=b|0;d=d|0;e=e|0;f=f|0;var g=0,h=0,i=0,j=0,k=0,l=0,m=0,n=0,o=0,p=0,q=0,r=0,s=0,t=0,u=0,v=0,w=0,x=0,y=0,z=0,A=0,B=0,C=0,D=0,E=0,F=0,H=0,I=0,J=0,K=0,L=0,M=0;g=a;h=b;i=h;j=d;k=e;l=k;if((i|0)==0){m=(f|0)!=0;if((l|0)==0){if(m){c[f>>2]=(g>>>0)%(j>>>0);c[f+4>>2]=0}n=0;o=(g>>>0)/(j>>>0)>>>0;return(G=n,o)|0}else{if(!m){n=0;o=0;return(G=n,o)|0}c[f>>2]=a|0;c[f+4>>2]=b&0;n=0;o=0;return(G=n,o)|0}}m=(l|0)==0;do{if((j|0)==0){if(m){if((f|0)!=0){c[f>>2]=(i>>>0)%(j>>>0);c[f+4>>2]=0}n=0;o=(i>>>0)/(j>>>0)>>>0;return(G=n,o)|0}if((g|0)==0){if((f|0)!=0){c[f>>2]=0;c[f+4>>2]=(i>>>0)%(l>>>0)}n=0;o=(i>>>0)/(l>>>0)>>>0;return(G=n,o)|0}p=l-1|0;if((p&l|0)==0){if((f|0)!=0){c[f>>2]=a|0;c[f+4>>2]=p&i|b&0}n=0;o=i>>>((N_(l|0)|0)>>>0);return(G=n,o)|0}p=(NZ(l|0)|0)-(NZ(i|0)|0)|0;if(p>>>0<=30){q=p+1|0;r=31-p|0;s=q;t=i<<r|g>>>(q>>>0);u=i>>>(q>>>0);v=0;w=g<<r;break}if((f|0)==0){n=0;o=0;return(G=n,o)|0}c[f>>2]=a|0;c[f+4>>2]=h|b&0;n=0;o=0;return(G=n,o)|0}else{if(!m){r=(NZ(l|0)|0)-(NZ(i|0)|0)|0;if(r>>>0<=31){q=r+1|0;p=31-r|0;x=r-31>>31;s=q;t=g>>>(q>>>0)&x|i<<p;u=i>>>(q>>>0)&x;v=0;w=g<<p;break}if((f|0)==0){n=0;o=0;return(G=n,o)|0}c[f>>2]=a|0;c[f+4>>2]=h|b&0;n=0;o=0;return(G=n,o)|0}p=j-1|0;if((p&j|0)!=0){x=(NZ(j|0)|0)+33-(NZ(i|0)|0)|0;q=64-x|0;r=32-x|0;y=r>>31;z=x-32|0;A=z>>31;s=x;t=r-1>>31&i>>>(z>>>0)|(i<<r|g>>>(x>>>0))&A;u=A&i>>>(x>>>0);v=g<<q&y;w=(i<<q|g>>>(z>>>0))&y|g<<r&x-33>>31;break}if((f|0)!=0){c[f>>2]=p&g;c[f+4>>2]=0}if((j|0)==1){n=h|b&0;o=a|0|0;return(G=n,o)|0}else{p=N_(j|0)|0;n=i>>>(p>>>0)|0;o=i<<32-p|g>>>(p>>>0)|0;return(G=n,o)|0}}}while(0);if((s|0)==0){B=w;C=v;D=u;E=t;F=0;H=0}else{g=d|0|0;d=k|e&0;e=NU(g,d,-1,-1)|0;k=G;i=w;w=v;v=u;u=t;t=s;s=0;while(1){I=w>>>31|i<<1;J=s|w<<1;j=u<<1|i>>>31|0;a=u>>>31|v<<1|0;NV(e,k,j,a)|0;b=G;h=b>>31|((b|0)<0?-1:0)<<1;K=h&1;L=NV(j,a,h&g,(((b|0)<0?-1:0)>>31|((b|0)<0?-1:0)<<1)&d)|0;M=G;b=t-1|0;if((b|0)==0){break}else{i=I;w=J;v=M;u=L;t=b;s=K}}B=I;C=J;D=M;E=L;F=0;H=K}K=C;C=0;if((f|0)!=0){c[f>>2]=E;c[f+4>>2]=D}n=(K|0)>>>31|(B|C)<<1|(C<<1|K>>>31)&0|F;o=(K<<1|0>>>31)&-2|H;return(G=n,o)|0}function N6(a){a=+a;return+R(+a)}function N7(a){a=+a;return+X(+a)}function N8(a){a=+a;return+a7(+a)}function N9(a){a=+a;return+V(+a)}function Oa(a){a=+a;return+Y(+a)}function Ob(a){a=+a;return+W(+a)}function Oc(a){a=+a;return+bB(+a)}function Od(a){a=+a;return+aa(+a)}function Oe(a){a=+a;return+be(+a)}function Of(a){a=+a;return+Z(+a)}function Og(a){a=+a;return+U(+a)}function Oh(a){a=+a;return+$(+a)}function Oi(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;cX[a&255](b|0,c|0,d|0,e|0,f|0)}function Oj(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=+d;e=e|0;cY[a&15](b|0,c|0,+d,e|0)}function Ok(a,b,c,d){a=a|0;b=b|0;c=+c;d=+d;return+cZ[a&1](b|0,+c,+d)}function Ol(a,b){a=a|0;b=b|0;c_[a&127](b|0)}function Om(a,b,c){a=a|0;b=b|0;c=c|0;c$[a&127](b|0,c|0)}function On(a,b,c,d,e,f,g){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;return c0[a&15](b|0,c|0,d|0,e|0,f|0,g|0)|0}function Oo(a,b){a=a|0;b=b|0;return c1[a&255](b|0)|0}function Op(a,b,c,d,e,f,g,h,i){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=+h;i=i|0;c2[a&3](b|0,c|0,d|0,e|0,f|0,g|0,+h,i|0)}function Oq(a,b,c,d,e,f,g,h){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;return c3[a&127](b|0,c|0,d|0,e|0,f|0,g|0,h|0)|0}function Or(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;return c4[a&63](b|0,c|0,d|0,e|0,f|0)|0}function Os(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=+d;e=e|0;f=f|0;return c5[a&7](b|0,c|0,+d,e|0,f|0)|0}function Ot(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;return c6[a&255](b|0,c|0,d|0)|0}function Ou(a,b,c){a=a|0;b=b|0;c=+c;return+c7[a&1](b|0,+c)}function Ov(a,b,c,d,e,f,g,h,i){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;c8[a&7](b|0,c|0,d|0,e|0,f|0,g|0,h|0,i|0)}function Ow(a,b,c,d,e,f,g){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;c9[a&127](b|0,c|0,d|0,e|0,f|0,g|0)}function Ox(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;return+da[a&3](b|0,c|0,d|0)}function Oy(a,b,c,d,e,f,g,h,i,j){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;return db[a&1](b|0,c|0,d|0,e|0,f|0,g|0,h|0,i|0,j|0)|0}function Oz(a,b,c,d,e,f,g,h){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;dc[a&255](b|0,c|0,d|0,e|0,f|0,g|0,h|0)}function OA(a,b){a=a|0;b=+b;return+dd[a&31](+b)}function OB(a,b,c,d,e,f,g,h,i,j){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;de[a&127](b|0,c|0,d|0,e|0,f|0,g|0,h|0,i|0,j|0)}function OC(a,b,c,d,e,f,g,h,i,j,k){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;df[a&127](b|0,c|0,d|0,e|0,f|0,g|0,h|0,i|0,j|0,k|0)}function OD(a,b,c){a=a|0;b=b|0;c=c|0;return dg[a&255](b|0,c|0)|0}function OE(a,b,c){a=a|0;b=b|0;c=c|0;return+dh[a&3](b|0,c|0)}function OF(a,b,c,d,e,f,g,h,i,j,k,l,m){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=+h;i=i|0;j=j|0;k=k|0;l=+l;m=m|0;return di[a&3](b|0,c|0,d|0,e|0,f|0,g|0,+h,i|0,j|0,k|0,+l,m|0)|0}function OG(a,b,c,d,e,f,g,h,i,j,k,l,m){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;m=m|0;dj[a&127](b|0,c|0,d|0,e|0,f|0,g|0,h|0,i|0,j|0,k|0,l|0,m|0)}function OH(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;return dk[a&63](b|0,c|0,d|0,e|0)|0}function OI(a,b,c,d,e,f,g,h){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=+f;g=g|0;h=h|0;dl[a&3](b|0,c|0,d|0,e|0,+f,g|0,h|0)}function OJ(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;dm[a&127](b|0,c|0,d|0)}function OK(a){a=a|0;dn[a&1]()}function OL(a,b,c,d,e,f,g,h,i){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;return dp[a&7](b|0,c|0,d|0,e|0,f|0,g|0,h|0,i|0)|0}function OM(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;dq[a&127](b|0,c|0,d|0,e|0)}function ON(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;ad(0)}function OO(a,b,c,d){a=a|0;b=b|0;c=+c;d=d|0;ad(1)}function OP(a,b,c){a=a|0;b=+b;c=+c;ad(2);return 0.0}function OQ(a){a=a|0;ad(3)}function OR(a,b){a=a|0;b=b|0;ad(4)}function OS(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;ad(5);return 0}function OT(a){a=a|0;ad(6);return 0}function OU(a,b,c,d,e,f,g,h){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=+g;h=h|0;ad(7)}function OV(a,b,c,d,e,f,g){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;ad(8);return 0}function OW(a,b,c,d,e){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;ad(9);return 0}function OX(a,b,c,d,e){a=a|0;b=b|0;c=+c;d=d|0;e=e|0;ad(10);return 0}function OY(a,b,c){a=a|0;b=b|0;c=c|0;ad(11);return 0}function OZ(a,b){a=a|0;b=+b;ad(12);return 0.0}function O_(a,b,c,d,e,f,g,h){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;ad(13)}function O$(a,b,c,d,e,f){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;ad(14)}function O0(a,b,c){a=a|0;b=b|0;c=c|0;ad(15);return 0.0}function O1(a,b,c,d,e,f,g,h,i){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;ad(16);return 0}function O2(a,b,c,d,e,f,g){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;ad(17)}function O3(a){a=+a;ad(18);return 0.0}function O4(a,b,c,d,e,f,g,h,i){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;ad(19)}function O5(a,b,c,d,e,f,g,h,i,j){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;ad(20)}function O6(a,b){a=a|0;b=b|0;ad(21);return 0}function O7(a,b){a=a|0;b=b|0;ad(22);return 0.0}function O8(a,b,c,d,e,f,g,h,i,j,k,l){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=+g;h=h|0;i=i|0;j=j|0;k=+k;l=l|0;ad(23);return 0}function O9(a,b,c,d,e,f,g,h,i,j,k,l){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;i=i|0;j=j|0;k=k|0;l=l|0;ad(24)}function Pa(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;ad(25);return 0}function Pb(a,b,c,d,e,f,g){a=a|0;b=b|0;c=c|0;d=d|0;e=+e;f=f|0;g=g|0;ad(26)}function Pc(a,b,c){a=a|0;b=b|0;c=c|0;ad(27)}function Pd(){ad(28)}function Pe(a,b,c,d,e,f,g,h){a=a|0;b=b|0;c=c|0;d=d|0;e=e|0;f=f|0;g=g|0;h=h|0;ad(29);return 0}function Pf(a,b,c,d){a=a|0;b=b|0;c=c|0;d=d|0;ad(30)}
// EMSCRIPTEN_END_FUNCS
var cX=[ON,ON,xq,ON,rF,ON,ss,ON,sg,ON,rJ,ON,or,ON,sB,ON,sD,ON,p8,ON,sq,ON,p6,ON,tM,ON,xp,ON,js,ON,xm,ON,rK,ON,r6,ON,si,ON,r5,ON,sf,ON,sp,ON,r9,ON,xh,ON,tH,ON,r8,ON,sb,ON,xi,ON,rf,ON,sn,ON,sl,ON,Ac,ON,tA,ON,sH,ON,xl,ON,se,ON,sa,ON,xn,ON,sk,ON,sr,ON,sh,ON,tz,ON,Ad,ON,tD,ON,sF,ON,jq,ON,so,ON,tw,ON,xj,ON,r7,ON,sd,ON,om,ON,xg,ON,tN,ON,tE,ON,ju,ON,sc,ON,tv,ON,xk,ON,xo,ON,tI,ON,sm,ON,sj,ON,st,ON,tL,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON,ON];var cY=[OO,OO,qV,OO,Ab,OO,z9,OO,Aa,OO,OO,OO,OO,OO,OO,OO];var cZ=[OP,OP];var c_=[OQ,OQ,qD,OQ,FU,OQ,qE,OQ,qP,OQ,gt,OQ,Gz,OQ,q6,OQ,gG,OQ,GK,OQ,eI,OQ,hl,OQ,sY,OQ,pM,OQ,qQ,OQ,q9,OQ,jr,OQ,qJ,OQ,FV,OQ,hA,OQ,qM,OQ,qL,OQ,GS,OQ,qF,OQ,qI,OQ,GL,OQ,qK,OQ,nD,OQ,jt,OQ,oJ,OQ,qN,OQ,pD,OQ,gN,OQ,GA,OQ,qH,OQ,qO,OQ,qC,OQ,eD,OQ,q8,OQ,qG,OQ,og,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ,OQ];var c$=[OR,OR,gD,OR,qs,OR,Cd,OR,EO,OR,p_,OR,qr,OR,qu,OR,qt,OR,qp,OR,wG,OR,q4,OR,rb,OR,Eb,OR,zf,OR,pw,OR,qy,OR,ra,OR,Ce,OR,qv,OR,lU,OR,rd,OR,qz,OR,FW,OR,px,OR,pX,OR,zp,OR,pW,OR,qq,OR,GH,OR,gC,OR,wI,OR,wC,OR,wE,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR,OR];var c0=[OS,OS,sZ,OS,od,OS,p7,OS,oY,OS,OS,OS,OS,OS,OS,OS];var c1=[OT,OT,l_,OT,gO,OT,pJ,OT,fa,OT,n9,OT,oB,OT,gc,OT,ga,OT,ho,OT,oC,OT,lh,OT,gu,OT,ob,OT,na,OT,jE,OT,oK,OT,iG,OT,hx,OT,hy,OT,hD,OT,hN,OT,k1,OT,n3,OT,lZ,OT,lv,OT,s1,OT,oz,OT,hQ,OT,gH,OT,hE,OT,h0,OT,n5,OT,li,OT,GT,OT,n2,OT,zh,OT,nb,OT,ht,OT,gL,OT,gJ,OT,ja,OT,hn,OT,m6,OT,lt,OT,hu,OT,gM,OT,ro,OT,AI,OT,iK,OT,k$,OT,n6,OT,hp,OT,kB,OT,i9,OT,n0,OT,hB,OT,hX,OT,kY,OT,hk,OT,hr,OT,jH,OT,rZ,OT,zg,OT,hV,OT,gK,OT,s_,OT,gI,OT,kD,OT,pL,OT,kX,OT,jF,OT,gF,OT,lX,OT,pB,OT,gw,OT,h_,OT,lk,OT,m7,OT,n8,OT,hO,OT,ha,OT,eE,OT,oe,OT,id,OT,hF,OT,pF,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT,OT];var c2=[OU,OU,rI,OU];var c3=[OV,OV,w0,OV,w$,OV,w1,OV,te,OV,tn,OV,tj,OV,tl,OV,wX,OV,wM,OV,wT,OV,wW,OV,wL,OV,ti,OV,wY,OV,to,OV,wQ,OV,tm,OV,w3,OV,td,OV,tq,OV,tk,OV,wS,OV,wv,OV,tp,OV,wR,OV,wP,OV,tg,OV,wU,OV,tf,OV,w_,OV,wV,OV,th,OV,wN,OV,w2,OV,wZ,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV,OV];var c4=[OW,OW,f$,OW,lm,OW,m9,OW,lD,OW,F3,OW,F2,OW,ll,OW,lB,OW,hS,OW,jI,OW,lQ,OW,p5,OW,hR,OW,F1,OW,F5,OW,oZ,OW,FZ,OW,k_,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW,OW];var c5=[OX,OX,En,OX,Em,OX,OX,OX];var c6=[OY,OY,jS,OY,jO,OY,ef,OY,fs,OY,d1,OY,ka,OY,d_,OY,eb,OY,kx,OY,fh,OY,ki,OY,fj,OY,j5,OY,d3,OY,fv,OY,ee,OY,kh,OY,i7,OY,s0,OY,e4,OY,lc,OY,ks,OY,dU,OY,ec,OY,eh,OY,kj,OY,jC,OY,ku,OY,kw,OY,kk,OY,jV,OY,kn,OY,kl,OY,DW,OY,kc,OY,kb,OY,ke,OY,kd,OY,oV,OY,fk,OY,kg,OY,jX,OY,d0,OY,jD,OY,jQ,OY,i0,OY,h8,OY,kq,OY,d2,OY,dZ,OY,dV,OY,j$,OY,i6,OY,i1,OY,fm,OY,eg,OY,ft,OY,jR,OY,j9,OY,d9,OY,d4,OY,jZ,OY,fi,OY,fq,OY,fb,OY,fc,OY,iA,OY,dP,OY,jT,OY,jY,OY,ko,OY,j_,OY,kt,OY,d7,OY,kf,OY,j6,OY,dR,OY,j8,OY,kp,OY,s$,OY,dQ,OY,fo,OY,dY,OY,j2,OY,dO,OY,jP,OY,h9,OY,kr,OY,dS,OY,km,OY,d8,OY,fd,OY,j1,OY,jU,OY,jW,OY,ed,OY,j4,OY,j3,OY,dT,OY,hv,OY,i5,OY,d6,OY,fu,OY,dX,OY,j0,OY,d$,OY,j7,OY,jJ,OY,dW,OY,fp,OY,d5,OY,kv,OY,fr,OY,ea,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY,OY];var c7=[OZ,OZ];var c8=[O_,O_,wn,O_,wy,O_,O_,O_];var c9=[O$,O$,yS,O$,yy,O$,yC,O$,wm,O$,yF,O$,yA,O$,yx,O$,yw,O$,yW,O$,rh,O$,rg,O$,rj,O$,ri,O$,ot,O$,ys,O$,yK,O$,yU,O$,yu,O$,yO,O$,yE,O$,yJ,O$,oq,O$,yT,O$,yp,O$,yL,O$,yB,O$,wx,O$,yv,O$,yI,O$,yQ,O$,yP,O$,yG,O$,yM,O$,yt,O$,yH,O$,yN,O$,yz,O$,yr,O$,yR,O$,yV,O$,yq,O$,yD,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$,O$];var da=[O0,O0,z6,O0];var db=[O1,O1];var dc=[O2,O2,xe,O2,xw,O2,sC,O2,x3,O2,xU,O2,xT,O2,tF,O2,xs,O2,xQ,O2,wB,O2,xV,O2,xr,O2,tJ,O2,xv,O2,yb,O2,tB,O2,x9,O2,xO,O2,xL,O2,tK,O2,tG,O2,xR,O2,xC,O2,xb,O2,yc,O2,ya,O2,xA,O2,tO,O2,xd,O2,tP,O2,x1,O2,xc,O2,tu,O2,xK,O2,xS,O2,sG,O2,tx,O2,xu,O2,tt,O2,xZ,O2,xH,O2,xF,O2,xM,O2,x6,O2,tC,O2,xz,O2,sI,O2,xI,O2,xW,O2,xx,O2,x4,O2,xE,O2,xy,O2,x8,O2,x7,O2,xY,O2,xP,O2,x5,O2,xX,O2,ty,O2,xG,O2,xN,O2,x$,O2,wz,O2,xB,O2,xt,O2,xf,O2,sE,O2,xD,O2,wA,O2,ww,O2,xJ,O2,x2,O2,x_,O2,x0,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2,O2];var dd=[O3,O3,N6,O3,N7,O3,N8,O3,N9,O3,Oa,O3,Ob,O3,Oc,O3,z_,O3,Od,O3,Oe,O3,Of,O3,Og,O3,Oh,O3,O3,O3,O3,O3];var de=[O4,O4,vo,O4,un,O4,ue,O4,uZ,O4,tS,O4,wk,O4,uk,O4,vA,O4,uy,O4,wr,O4,vc,O4,tV,O4,wj,O4,vl,O4,uK,O4,t$,O4,uT,O4,vi,O4,wl,O4,ws,O4,wh,O4,u0,O4,ut,O4,wq,O4,uQ,O4,uv,O4,uq,O4,ub,O4,tY,O4,wt,O4,uE,O4,t5,O4,uN,O4,uH,O4,u9,O4,vu,O4,t8,O4,vf,O4,u3,O4,wi,O4,u6,O4,t2,O4,uW,O4,vr,O4,vx,O4,uh,O4,uB,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4,O4];var df=[O5,O5,uO,O5,u1,O5,vy,O5,uX,O5,ug,O5,tU,O5,t_,O5,vp,O5,tR,O5,vm,O5,ud,O5,ua,O5,vB,O5,vd,O5,uF,O5,uw,O5,tX,O5,uz,O5,u7,O5,uj,O5,uR,O5,wo,O5,vj,O5,us,O5,vD,O5,uL,O5,wp,O5,up,O5,uU,O5,t7,O5,u4,O5,t4,O5,uI,O5,uC,O5,t1,O5,um,O5,vg,O5,u_,O5,vv,O5,va,O5,vs,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5,O5];var dg=[O6,O6,gx,O6,kZ,O6,gQ,O6,kC,O6,GO,O6,f9,O6,lj,O6,i4,O6,Gx,O6,Gy,O6,gb,O6,CX,O6,dN,O6,oA,O6,lH,O6,lO,O6,eS,O6,GM,O6,lG,O6,lY,O6,k0,O6,lM,O6,BF,O6,hP,O6,hW,O6,dM,O6,lI,O6,qm,O6,GB,O6,hC,O6,GU,O6,GR,O6,lz,O6,lK,O6,gy,O6,hz,O6,o_,O6,gv,O6,lN,O6,lJ,O6,lP,O6,Ge,O6,FY,O6,ic,O6,FX,O6,h$,O6,lu,O6,fF,O6,lA,O6,qd,O6,iu,O6,GI,O6,lT,O6,hq,O6,hH,O6,nx,O6,jG,O6,hG,O6,lL,O6,jb,O6,GJ,O6,m8,O6,lE,O6,hs,O6,lS,O6,hm,O6,lC,O6,FS,O6,k2,O6,lF,O6,it,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6,O6];var dh=[O7,O7,q7,O7];var di=[O8,O8,sX,O8];var dj=[O9,O9,u$,O9,vb,O9,uu,O9,t6,O9,uo,O9,t9,O9,uJ,O9,tW,O9,vh,O9,tZ,O9,uG,O9,u8,O9,uA,O9,uf,O9,tT,O9,uP,O9,vq,O9,vw,O9,uM,O9,ul,O9,u5,O9,t3,O9,vt,O9,vn,O9,ve,O9,uc,O9,uV,O9,ui,O9,vz,O9,vC,O9,ux,O9,uS,O9,u2,O9,tQ,O9,uD,O9,vk,O9,uY,O9,t0,O9,ur,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9,O9];var dk=[Pa,Pa,jN,Pa,Gt,Pa,DX,Pa,Gu,Pa,jM,Pa,pb,Pa,oL,Pa,GC,Pa,By,Pa,i8,Pa,pK,Pa,n1,Pa,GN,Pa,pd,Pa,D_,Pa,pC,Pa,jK,Pa,of,Pa,n7,Pa,i2,Pa,n4,Pa,DZ,Pa,DY,Pa,oa,Pa,jL,Pa,h3,Pa,pG,Pa,Pa,Pa,Pa,Pa,Pa,Pa,Pa,Pa];var dl=[Pb,Pb,re,Pb];var dm=[Pc,Pc,vU,Pc,vP,Pc,vO,Pc,q$,Pc,M4,Pc,M$,Pc,GQ,Pc,wg,Pc,vL,Pc,vK,Pc,vW,Pc,Af,Pc,qW,Pc,M_,Pc,q1,Pc,wF,Pc,vY,Pc,vH,Pc,vR,Pc,vI,Pc,r2,Pc,v6,Pc,M1,Pc,vX,Pc,v7,Pc,v1,Pc,wD,Pc,wb,Pc,wH,Pc,v$,Pc,q0,Pc,vF,Pc,vV,Pc,v5,Pc,v9,Pc,v2,Pc,v4,Pc,v3,Pc,rD,Pc,M2,Pc,wc,Pc,wf,Pc,v_,Pc,vT,Pc,vG,Pc,v8,Pc,rc,Pc,M0,Pc,wa,Pc,v0,Pc,vQ,Pc,we,Pc,wJ,Pc,wd,Pc,vS,Pc,r$,Pc,r0,Pc,r1,Pc,M3,Pc,vN,Pc,vJ,Pc,vZ,Pc,vM,Pc];var dn=[Pd,Pd];var dp=[Pe,Pe,F4,Pe,He,Pe,F$,Pe];var dq=[Pf,Pf,sL,Pf,CQ,Pf,CT,Pf,CW,Pf,sN,Pf,rG,Pf,sM,Pf,w5,Pf,w6,Pf,ye,Pf,pY,Pf,qT,Pf,pZ,Pf,rE,Pf,sO,Pf,AL,Pf,yn,Pf,yk,Pf,CS,Pf,qX,Pf,Ep,Pf,eJ,Pf,z8,Pf,ym,Pf,yj,Pf,fG,Pf,eU,Pf,yf,Pf,sP,Pf,rH,Pf,yl,Pf,CP,Pf,sS,Pf,yg,Pf,w7,Pf,yh,Pf,sQ,Pf,yi,Pf,w8,Pf,yo,Pf,Eo,Pf,Ae,Pf,sR,Pf,yd,Pf,w4,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf,Pf];return{_memcmp:NO,_strlen:NL,_strcat:NT,_free:ND,_main:eH,_realloc:NF,_memmove:NP,_tolower:NS,_memset:NM,_malloc:NC,_memcpy:NN,_llvm_uadd_with_overflow_i64:NR,_strcpy:NQ,_calloc:NE,runPostSets:dH,stackAlloc:dr,stackSave:ds,stackRestore:dt,setThrew:du,setTempRet0:dx,setTempRet1:dy,setTempRet2:dz,setTempRet3:dA,setTempRet4:dB,setTempRet5:dC,setTempRet6:dD,setTempRet7:dE,setTempRet8:dF,setTempRet9:dG,dynCall_viiiii:Oi,dynCall_viifi:Oj,dynCall_fiff:Ok,dynCall_vi:Ol,dynCall_vii:Om,dynCall_iiiiiii:On,dynCall_ii:Oo,dynCall_viiiiiifi:Op,dynCall_iiiiiiii:Oq,dynCall_iiiiii:Or,dynCall_iiifii:Os,dynCall_iiii:Ot,dynCall_fif:Ou,dynCall_viiiiiiii:Ov,dynCall_viiiiii:Ow,dynCall_fiii:Ox,dynCall_iiiiiiiiii:Oy,dynCall_viiiiiii:Oz,dynCall_ff:OA,dynCall_viiiiiiiii:OB,dynCall_viiiiiiiiii:OC,dynCall_iii:OD,dynCall_fii:OE,dynCall_iiiiiiifiiifi:OF,dynCall_viiiiiiiiiiii:OG,dynCall_iiiii:OH,dynCall_viiiifii:OI,dynCall_viii:OJ,dynCall_v:OK,dynCall_iiiiiiiii:OL,dynCall_viiii:OM}})
// EMSCRIPTEN_END_ASM
({ "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array }, { "abort": abort, "assert": assert, "asmPrintInt": asmPrintInt, "asmPrintFloat": asmPrintFloat, "min": Math_min, "invoke_viiiii": invoke_viiiii, "invoke_viifi": invoke_viifi, "invoke_fiff": invoke_fiff, "invoke_vi": invoke_vi, "invoke_vii": invoke_vii, "invoke_iiiiiii": invoke_iiiiiii, "invoke_ii": invoke_ii, "invoke_viiiiiifi": invoke_viiiiiifi, "invoke_iiiiiiii": invoke_iiiiiiii, "invoke_iiiiii": invoke_iiiiii, "invoke_iiifii": invoke_iiifii, "invoke_iiii": invoke_iiii, "invoke_fif": invoke_fif, "invoke_viiiiiiii": invoke_viiiiiiii, "invoke_viiiiii": invoke_viiiiii, "invoke_fiii": invoke_fiii, "invoke_iiiiiiiiii": invoke_iiiiiiiiii, "invoke_viiiiiii": invoke_viiiiiii, "invoke_ff": invoke_ff, "invoke_viiiiiiiii": invoke_viiiiiiiii, "invoke_viiiiiiiiii": invoke_viiiiiiiiii, "invoke_iii": invoke_iii, "invoke_fii": invoke_fii, "invoke_iiiiiiifiiifi": invoke_iiiiiiifiiifi, "invoke_viiiiiiiiiiii": invoke_viiiiiiiiiiii, "invoke_iiiii": invoke_iiiii, "invoke_viiiifii": invoke_viiiifii, "invoke_viii": invoke_viii, "invoke_v": invoke_v, "invoke_iiiiiiiii": invoke_iiiiiiiii, "invoke_viiii": invoke_viiii, "_llvm_lifetime_end": _llvm_lifetime_end, "_lseek": _lseek, "_sysconf": _sysconf, "__scanString": __scanString, "_fclose": _fclose, "_strtoul": _strtoul, "_fflush": _fflush, "_strtol": _strtol, "__isLeapYear": __isLeapYear, "_fwrite": _fwrite, "_send": _send, "_frexp": _frexp, "_fputs": _fputs, "_cbrt": _cbrt, "_isspace": _isspace, "_localtime": _localtime, "_read": _read, "_ceil": _ceil, "_perror": _perror, "_strstr": _strstr, "_fsync": _fsync, "_signal": _signal, "_tanh": _tanh, "_strcmp": _strcmp, "_memchr": _memchr, "_strncmp": _strncmp, "_cosf": _cosf, "_snprintf": _snprintf, "_fgetc": _fgetc, "_cosh": _cosh, "__getFloat": __getFloat, "_fgets": _fgets, "_close": _close, "_strchr": _strchr, "_tcgetattr": _tcgetattr, "_clock": _clock, "___setErrNo": ___setErrNo, "_uncompress": _uncompress, "_access": _access, "_ftell": _ftell, "_exit": _exit, "_sprintf": _sprintf, "_strrchr": _strrchr, "_atan2f": _atan2f, "_setrlimit": _setrlimit, "_log2": _log2, "_strcspn": _strcspn, "_fputc": _fputc, "_ferror": _ferror, "_llvm_uadd_with_overflow_i32": _llvm_uadd_with_overflow_i32, "_gmtime": _gmtime, "_localtime_r": _localtime_r, "_sinh": _sinh, "_sinf": _sinf, "_recv": _recv, "__parseInt64": __parseInt64, "_trunc": _trunc, "_cos": _cos, "_putchar": _putchar, "_bsearch": _bsearch, "__exit": __exit, "_nanosleep": _nanosleep, "_strftime": _strftime, "_llvm_va_end": _llvm_va_end, "_tzset": _tzset, "_fabsf": _fabsf, "_floorf": _floorf, "_llvm_va_copy": _llvm_va_copy, "_rint": _rint, "_ldexp": _ldexp, "_printf": _printf, "_pread": _pread, "_fopen": _fopen, "_open": _open, "_sqrtf": _sqrtf, "_select": _select, "_getchar": _getchar, "_usleep": _usleep, "_llvm_pow_f32": _llvm_pow_f32, "__arraySum": __arraySum, "_log": _log, "_puts": _puts, "_mktime": _mktime, "_exp2": _exp2, "_scanf": _scanf, "_qsort": _qsort, "_log10": _log10, "_isatty": _isatty, "__formatString": __formatString, "_getenv": _getenv, "_gettimeofday": _gettimeofday, "_atoi": _atoi, "_vfprintf": _vfprintf, "_llvm_pow_f64": _llvm_pow_f64, "_sbrk": _sbrk, "_fscanf": _fscanf, "___errno_location": ___errno_location, "_strerror": _strerror, "_getrusage": _getrusage, "_fstat": _fstat, "_llvm_lifetime_start": _llvm_lifetime_start, "_strspn": _strspn, "__parseInt": __parseInt, "_ungetc": _ungetc, "_exp": _exp, "_vsprintf": _vsprintf, "_gmtime_r": _gmtime_r, "_vsnprintf": _vsnprintf, "_sscanf": _sscanf, "___assert_fail": ___assert_fail, "_fread": _fread, "_abort": _abort, "_fprintf": _fprintf, "_strtoll": _strtoll, "_tan": _tan, "___buildEnvironment": ___buildEnvironment, "__reallyNegative": __reallyNegative, "__addDays": __addDays, "_atanf": _atanf, "_fabs": _fabs, "_floor": _floor, "_fseek": _fseek, "_sqrt": _sqrt, "_write": _write, "_sin": _sin, "_stat": _stat, "_ff_guess_image2_codec": _ff_guess_image2_codec, "_tcsetattr": _tcsetattr, "_atan": _atan, "_asin": _asin, "_acos": _acos, "_pwrite": _pwrite, "_strerror_r": _strerror_r, "_llrint": _llrint, "_llvm_bswap_i16": _llvm_bswap_i16, "_time": _time, "_llvm_bswap_i32": _llvm_bswap_i32, "_setvbuf": _setvbuf, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8, "ctlz_i8": ctlz_i8, "NaN": NaN, "Infinity": Infinity, "_stderr": _stderr, "_stdout": _stdout, "___fsmu8": ___fsmu8 }, buffer);
var _memcmp = Module["_memcmp"] = asm["_memcmp"];
var _strlen = Module["_strlen"] = asm["_strlen"];
var _strcat = Module["_strcat"] = asm["_strcat"];
var _free = Module["_free"] = asm["_free"];
var _main = Module["_main"] = asm["_main"];
var _realloc = Module["_realloc"] = asm["_realloc"];
var _memmove = Module["_memmove"] = asm["_memmove"];
var _tolower = Module["_tolower"] = asm["_tolower"];
var _memset = Module["_memset"] = asm["_memset"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _llvm_uadd_with_overflow_i64 = Module["_llvm_uadd_with_overflow_i64"] = asm["_llvm_uadd_with_overflow_i64"];
var _strcpy = Module["_strcpy"] = asm["_strcpy"];
var _calloc = Module["_calloc"] = asm["_calloc"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var dynCall_viiiii = Module["dynCall_viiiii"] = asm["dynCall_viiiii"];
var dynCall_viifi = Module["dynCall_viifi"] = asm["dynCall_viifi"];
var dynCall_fiff = Module["dynCall_fiff"] = asm["dynCall_fiff"];
var dynCall_vi = Module["dynCall_vi"] = asm["dynCall_vi"];
var dynCall_vii = Module["dynCall_vii"] = asm["dynCall_vii"];
var dynCall_iiiiiii = Module["dynCall_iiiiiii"] = asm["dynCall_iiiiiii"];
var dynCall_ii = Module["dynCall_ii"] = asm["dynCall_ii"];
var dynCall_viiiiiifi = Module["dynCall_viiiiiifi"] = asm["dynCall_viiiiiifi"];
var dynCall_iiiiiiii = Module["dynCall_iiiiiiii"] = asm["dynCall_iiiiiiii"];
var dynCall_iiiiii = Module["dynCall_iiiiii"] = asm["dynCall_iiiiii"];
var dynCall_iiifii = Module["dynCall_iiifii"] = asm["dynCall_iiifii"];
var dynCall_iiii = Module["dynCall_iiii"] = asm["dynCall_iiii"];
var dynCall_fif = Module["dynCall_fif"] = asm["dynCall_fif"];
var dynCall_viiiiiiii = Module["dynCall_viiiiiiii"] = asm["dynCall_viiiiiiii"];
var dynCall_viiiiii = Module["dynCall_viiiiii"] = asm["dynCall_viiiiii"];
var dynCall_fiii = Module["dynCall_fiii"] = asm["dynCall_fiii"];
var dynCall_iiiiiiiiii = Module["dynCall_iiiiiiiiii"] = asm["dynCall_iiiiiiiiii"];
var dynCall_viiiiiii = Module["dynCall_viiiiiii"] = asm["dynCall_viiiiiii"];
var dynCall_ff = Module["dynCall_ff"] = asm["dynCall_ff"];
var dynCall_viiiiiiiii = Module["dynCall_viiiiiiiii"] = asm["dynCall_viiiiiiiii"];
var dynCall_viiiiiiiiii = Module["dynCall_viiiiiiiiii"] = asm["dynCall_viiiiiiiiii"];
var dynCall_iii = Module["dynCall_iii"] = asm["dynCall_iii"];
var dynCall_fii = Module["dynCall_fii"] = asm["dynCall_fii"];
var dynCall_iiiiiiifiiifi = Module["dynCall_iiiiiiifiiifi"] = asm["dynCall_iiiiiiifiiifi"];
var dynCall_viiiiiiiiiiii = Module["dynCall_viiiiiiiiiiii"] = asm["dynCall_viiiiiiiiiiii"];
var dynCall_iiiii = Module["dynCall_iiiii"] = asm["dynCall_iiiii"];
var dynCall_viiiifii = Module["dynCall_viiiifii"] = asm["dynCall_viiiifii"];
var dynCall_viii = Module["dynCall_viii"] = asm["dynCall_viii"];
var dynCall_v = Module["dynCall_v"] = asm["dynCall_v"];
var dynCall_iiiiiiiii = Module["dynCall_iiiiiiiii"] = asm["dynCall_iiiiiiiii"];
var dynCall_viiii = Module["dynCall_viiii"] = asm["dynCall_viiii"];
Runtime.stackAlloc = function(size) { return asm['stackAlloc'](size) };
Runtime.stackSave = function() { return asm['stackSave']() };
Runtime.stackRestore = function(top) { asm['stackRestore'](top) };
// TODO: strip out parts of this we do not need
//======= begin closure i64 code =======
// Copyright 2009 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
/**
 * @fileoverview Defines a Long class for representing a 64-bit two's-complement
 * integer value, which faithfully simulates the behavior of a Java "long". This
 * implementation is derived from LongLib in GWT.
 *
 */
var i64Math = (function() { // Emscripten wrapper
  var goog = { math: {} };
  /**
   * Constructs a 64-bit two's-complement integer, given its low and high 32-bit
   * values as *signed* integers.  See the from* functions below for more
   * convenient ways of constructing Longs.
   *
   * The internal representation of a long is the two given signed, 32-bit values.
   * We use 32-bit pieces because these are the size of integers on which
   * Javascript performs bit-operations.  For operations like addition and
   * multiplication, we split each number into 16-bit pieces, which can easily be
   * multiplied within Javascript's floating-point representation without overflow
   * or change in sign.
   *
   * In the algorithms below, we frequently reduce the negative case to the
   * positive case by negating the input(s) and then post-processing the result.
   * Note that we must ALWAYS check specially whether those values are MIN_VALUE
   * (-2^63) because -MIN_VALUE == MIN_VALUE (since 2^63 cannot be represented as
   * a positive number, it overflows back into a negative).  Not handling this
   * case would often result in infinite recursion.
   *
   * @param {number} low  The low (signed) 32 bits of the long.
   * @param {number} high  The high (signed) 32 bits of the long.
   * @constructor
   */
  goog.math.Long = function(low, high) {
    /**
     * @type {number}
     * @private
     */
    this.low_ = low | 0;  // force into 32 signed bits.
    /**
     * @type {number}
     * @private
     */
    this.high_ = high | 0;  // force into 32 signed bits.
  };
  // NOTE: Common constant values ZERO, ONE, NEG_ONE, etc. are defined below the
  // from* methods on which they depend.
  /**
   * A cache of the Long representations of small integer values.
   * @type {!Object}
   * @private
   */
  goog.math.Long.IntCache_ = {};
  /**
   * Returns a Long representing the given (32-bit) integer value.
   * @param {number} value The 32-bit integer in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromInt = function(value) {
    if (-128 <= value && value < 128) {
      var cachedObj = goog.math.Long.IntCache_[value];
      if (cachedObj) {
        return cachedObj;
      }
    }
    var obj = new goog.math.Long(value | 0, value < 0 ? -1 : 0);
    if (-128 <= value && value < 128) {
      goog.math.Long.IntCache_[value] = obj;
    }
    return obj;
  };
  /**
   * Returns a Long representing the given value, provided that it is a finite
   * number.  Otherwise, zero is returned.
   * @param {number} value The number in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromNumber = function(value) {
    if (isNaN(value) || !isFinite(value)) {
      return goog.math.Long.ZERO;
    } else if (value <= -goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MIN_VALUE;
    } else if (value + 1 >= goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MAX_VALUE;
    } else if (value < 0) {
      return goog.math.Long.fromNumber(-value).negate();
    } else {
      return new goog.math.Long(
          (value % goog.math.Long.TWO_PWR_32_DBL_) | 0,
          (value / goog.math.Long.TWO_PWR_32_DBL_) | 0);
    }
  };
  /**
   * Returns a Long representing the 64-bit integer that comes by concatenating
   * the given high and low bits.  Each is assumed to use 32 bits.
   * @param {number} lowBits The low 32-bits.
   * @param {number} highBits The high 32-bits.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromBits = function(lowBits, highBits) {
    return new goog.math.Long(lowBits, highBits);
  };
  /**
   * Returns a Long representation of the given string, written using the given
   * radix.
   * @param {string} str The textual representation of the Long.
   * @param {number=} opt_radix The radix in which the text is written.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromString = function(str, opt_radix) {
    if (str.length == 0) {
      throw Error('number format error: empty string');
    }
    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }
    if (str.charAt(0) == '-') {
      return goog.math.Long.fromString(str.substring(1), radix).negate();
    } else if (str.indexOf('-') >= 0) {
      throw Error('number format error: interior "-" character: ' + str);
    }
    // Do several (8) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 8));
    var result = goog.math.Long.ZERO;
    for (var i = 0; i < str.length; i += 8) {
      var size = Math.min(8, str.length - i);
      var value = parseInt(str.substring(i, i + size), radix);
      if (size < 8) {
        var power = goog.math.Long.fromNumber(Math.pow(radix, size));
        result = result.multiply(power).add(goog.math.Long.fromNumber(value));
      } else {
        result = result.multiply(radixToPower);
        result = result.add(goog.math.Long.fromNumber(value));
      }
    }
    return result;
  };
  // NOTE: the compiler should inline these constant values below and then remove
  // these variables, so there should be no runtime penalty for these.
  /**
   * Number used repeated below in calculations.  This must appear before the
   * first call to any from* function below.
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_16_DBL_ = 1 << 16;
  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_24_DBL_ = 1 << 24;
  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_32_DBL_ =
      goog.math.Long.TWO_PWR_16_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;
  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_31_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ / 2;
  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_48_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;
  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_64_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_32_DBL_;
  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_63_DBL_ =
      goog.math.Long.TWO_PWR_64_DBL_ / 2;
  /** @type {!goog.math.Long} */
  goog.math.Long.ZERO = goog.math.Long.fromInt(0);
  /** @type {!goog.math.Long} */
  goog.math.Long.ONE = goog.math.Long.fromInt(1);
  /** @type {!goog.math.Long} */
  goog.math.Long.NEG_ONE = goog.math.Long.fromInt(-1);
  /** @type {!goog.math.Long} */
  goog.math.Long.MAX_VALUE =
      goog.math.Long.fromBits(0xFFFFFFFF | 0, 0x7FFFFFFF | 0);
  /** @type {!goog.math.Long} */
  goog.math.Long.MIN_VALUE = goog.math.Long.fromBits(0, 0x80000000 | 0);
  /**
   * @type {!goog.math.Long}
   * @private
   */
  goog.math.Long.TWO_PWR_24_ = goog.math.Long.fromInt(1 << 24);
  /** @return {number} The value, assuming it is a 32-bit integer. */
  goog.math.Long.prototype.toInt = function() {
    return this.low_;
  };
  /** @return {number} The closest floating-point representation to this value. */
  goog.math.Long.prototype.toNumber = function() {
    return this.high_ * goog.math.Long.TWO_PWR_32_DBL_ +
           this.getLowBitsUnsigned();
  };
  /**
   * @param {number=} opt_radix The radix in which the text should be written.
   * @return {string} The textual representation of this value.
   */
  goog.math.Long.prototype.toString = function(opt_radix) {
    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }
    if (this.isZero()) {
      return '0';
    }
    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        // We need to change the Long value before it can be negated, so we remove
        // the bottom-most digit in this base and then recurse to do the rest.
        var radixLong = goog.math.Long.fromNumber(radix);
        var div = this.div(radixLong);
        var rem = div.multiply(radixLong).subtract(this);
        return div.toString(radix) + rem.toInt().toString(radix);
      } else {
        return '-' + this.negate().toString(radix);
      }
    }
    // Do several (6) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 6));
    var rem = this;
    var result = '';
    while (true) {
      var remDiv = rem.div(radixToPower);
      var intval = rem.subtract(remDiv.multiply(radixToPower)).toInt();
      var digits = intval.toString(radix);
      rem = remDiv;
      if (rem.isZero()) {
        return digits + result;
      } else {
        while (digits.length < 6) {
          digits = '0' + digits;
        }
        result = '' + digits + result;
      }
    }
  };
  /** @return {number} The high 32-bits as a signed value. */
  goog.math.Long.prototype.getHighBits = function() {
    return this.high_;
  };
  /** @return {number} The low 32-bits as a signed value. */
  goog.math.Long.prototype.getLowBits = function() {
    return this.low_;
  };
  /** @return {number} The low 32-bits as an unsigned value. */
  goog.math.Long.prototype.getLowBitsUnsigned = function() {
    return (this.low_ >= 0) ?
        this.low_ : goog.math.Long.TWO_PWR_32_DBL_ + this.low_;
  };
  /**
   * @return {number} Returns the number of bits needed to represent the absolute
   *     value of this Long.
   */
  goog.math.Long.prototype.getNumBitsAbs = function() {
    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        return 64;
      } else {
        return this.negate().getNumBitsAbs();
      }
    } else {
      var val = this.high_ != 0 ? this.high_ : this.low_;
      for (var bit = 31; bit > 0; bit--) {
        if ((val & (1 << bit)) != 0) {
          break;
        }
      }
      return this.high_ != 0 ? bit + 33 : bit + 1;
    }
  };
  /** @return {boolean} Whether this value is zero. */
  goog.math.Long.prototype.isZero = function() {
    return this.high_ == 0 && this.low_ == 0;
  };
  /** @return {boolean} Whether this value is negative. */
  goog.math.Long.prototype.isNegative = function() {
    return this.high_ < 0;
  };
  /** @return {boolean} Whether this value is odd. */
  goog.math.Long.prototype.isOdd = function() {
    return (this.low_ & 1) == 1;
  };
  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long equals the other.
   */
  goog.math.Long.prototype.equals = function(other) {
    return (this.high_ == other.high_) && (this.low_ == other.low_);
  };
  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long does not equal the other.
   */
  goog.math.Long.prototype.notEquals = function(other) {
    return (this.high_ != other.high_) || (this.low_ != other.low_);
  };
  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than the other.
   */
  goog.math.Long.prototype.lessThan = function(other) {
    return this.compare(other) < 0;
  };
  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than or equal to the other.
   */
  goog.math.Long.prototype.lessThanOrEqual = function(other) {
    return this.compare(other) <= 0;
  };
  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than the other.
   */
  goog.math.Long.prototype.greaterThan = function(other) {
    return this.compare(other) > 0;
  };
  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than or equal to the other.
   */
  goog.math.Long.prototype.greaterThanOrEqual = function(other) {
    return this.compare(other) >= 0;
  };
  /**
   * Compares this Long with the given one.
   * @param {goog.math.Long} other Long to compare against.
   * @return {number} 0 if they are the same, 1 if the this is greater, and -1
   *     if the given one is greater.
   */
  goog.math.Long.prototype.compare = function(other) {
    if (this.equals(other)) {
      return 0;
    }
    var thisNeg = this.isNegative();
    var otherNeg = other.isNegative();
    if (thisNeg && !otherNeg) {
      return -1;
    }
    if (!thisNeg && otherNeg) {
      return 1;
    }
    // at this point, the signs are the same, so subtraction will not overflow
    if (this.subtract(other).isNegative()) {
      return -1;
    } else {
      return 1;
    }
  };
  /** @return {!goog.math.Long} The negation of this value. */
  goog.math.Long.prototype.negate = function() {
    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.MIN_VALUE;
    } else {
      return this.not().add(goog.math.Long.ONE);
    }
  };
  /**
   * Returns the sum of this and the given Long.
   * @param {goog.math.Long} other Long to add to this one.
   * @return {!goog.math.Long} The sum of this and the given Long.
   */
  goog.math.Long.prototype.add = function(other) {
    // Divide each number into 4 chunks of 16 bits, and then sum the chunks.
    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;
    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;
    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 + b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 + b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 + b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 + b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };
  /**
   * Returns the difference of this and the given Long.
   * @param {goog.math.Long} other Long to subtract from this.
   * @return {!goog.math.Long} The difference of this and the given Long.
   */
  goog.math.Long.prototype.subtract = function(other) {
    return this.add(other.negate());
  };
  /**
   * Returns the product of this and the given long.
   * @param {goog.math.Long} other Long to multiply with this.
   * @return {!goog.math.Long} The product of this and the other.
   */
  goog.math.Long.prototype.multiply = function(other) {
    if (this.isZero()) {
      return goog.math.Long.ZERO;
    } else if (other.isZero()) {
      return goog.math.Long.ZERO;
    }
    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return other.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return this.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    }
    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().multiply(other.negate());
      } else {
        return this.negate().multiply(other).negate();
      }
    } else if (other.isNegative()) {
      return this.multiply(other.negate()).negate();
    }
    // If both longs are small, use float multiplication
    if (this.lessThan(goog.math.Long.TWO_PWR_24_) &&
        other.lessThan(goog.math.Long.TWO_PWR_24_)) {
      return goog.math.Long.fromNumber(this.toNumber() * other.toNumber());
    }
    // Divide each long into 4 chunks of 16 bits, and then add up 4x4 products.
    // We can skip products that would overflow.
    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;
    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;
    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 * b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 * b00;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c16 += a00 * b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 * b00;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a16 * b16;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a00 * b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 * b00 + a32 * b16 + a16 * b32 + a00 * b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };
  /**
   * Returns this Long divided by the given one.
   * @param {goog.math.Long} other Long by which to divide.
   * @return {!goog.math.Long} This Long divided by the given one.
   */
  goog.math.Long.prototype.div = function(other) {
    if (other.isZero()) {
      throw Error('division by zero');
    } else if (this.isZero()) {
      return goog.math.Long.ZERO;
    }
    if (this.equals(goog.math.Long.MIN_VALUE)) {
      if (other.equals(goog.math.Long.ONE) ||
          other.equals(goog.math.Long.NEG_ONE)) {
        return goog.math.Long.MIN_VALUE;  // recall that -MIN_VALUE == MIN_VALUE
      } else if (other.equals(goog.math.Long.MIN_VALUE)) {
        return goog.math.Long.ONE;
      } else {
        // At this point, we have |other| >= 2, so |this/other| < |MIN_VALUE|.
        var halfThis = this.shiftRight(1);
        var approx = halfThis.div(other).shiftLeft(1);
        if (approx.equals(goog.math.Long.ZERO)) {
          return other.isNegative() ? goog.math.Long.ONE : goog.math.Long.NEG_ONE;
        } else {
          var rem = this.subtract(other.multiply(approx));
          var result = approx.add(rem.div(other));
          return result;
        }
      }
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.ZERO;
    }
    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().div(other.negate());
      } else {
        return this.negate().div(other).negate();
      }
    } else if (other.isNegative()) {
      return this.div(other.negate()).negate();
    }
    // Repeat the following until the remainder is less than other:  find a
    // floating-point that approximates remainder / other *from below*, add this
    // into the result, and subtract it from the remainder.  It is critical that
    // the approximate value is less than or equal to the real value so that the
    // remainder never becomes negative.
    var res = goog.math.Long.ZERO;
    var rem = this;
    while (rem.greaterThanOrEqual(other)) {
      // Approximate the result of division. This may be a little greater or
      // smaller than the actual value.
      var approx = Math.max(1, Math.floor(rem.toNumber() / other.toNumber()));
      // We will tweak the approximate result by changing it in the 48-th digit or
      // the smallest non-fractional digit, whichever is larger.
      var log2 = Math.ceil(Math.log(approx) / Math.LN2);
      var delta = (log2 <= 48) ? 1 : Math.pow(2, log2 - 48);
      // Decrease the approximation until it is smaller than the remainder.  Note
      // that if it is too large, the product overflows and is negative.
      var approxRes = goog.math.Long.fromNumber(approx);
      var approxRem = approxRes.multiply(other);
      while (approxRem.isNegative() || approxRem.greaterThan(rem)) {
        approx -= delta;
        approxRes = goog.math.Long.fromNumber(approx);
        approxRem = approxRes.multiply(other);
      }
      // We know the answer can't be zero... and actually, zero would cause
      // infinite recursion since we would make no progress.
      if (approxRes.isZero()) {
        approxRes = goog.math.Long.ONE;
      }
      res = res.add(approxRes);
      rem = rem.subtract(approxRem);
    }
    return res;
  };
  /**
   * Returns this Long modulo the given one.
   * @param {goog.math.Long} other Long by which to mod.
   * @return {!goog.math.Long} This Long modulo the given one.
   */
  goog.math.Long.prototype.modulo = function(other) {
    return this.subtract(this.div(other).multiply(other));
  };
  /** @return {!goog.math.Long} The bitwise-NOT of this value. */
  goog.math.Long.prototype.not = function() {
    return goog.math.Long.fromBits(~this.low_, ~this.high_);
  };
  /**
   * Returns the bitwise-AND of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to AND.
   * @return {!goog.math.Long} The bitwise-AND of this and the other.
   */
  goog.math.Long.prototype.and = function(other) {
    return goog.math.Long.fromBits(this.low_ & other.low_,
                                   this.high_ & other.high_);
  };
  /**
   * Returns the bitwise-OR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to OR.
   * @return {!goog.math.Long} The bitwise-OR of this and the other.
   */
  goog.math.Long.prototype.or = function(other) {
    return goog.math.Long.fromBits(this.low_ | other.low_,
                                   this.high_ | other.high_);
  };
  /**
   * Returns the bitwise-XOR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to XOR.
   * @return {!goog.math.Long} The bitwise-XOR of this and the other.
   */
  goog.math.Long.prototype.xor = function(other) {
    return goog.math.Long.fromBits(this.low_ ^ other.low_,
                                   this.high_ ^ other.high_);
  };
  /**
   * Returns this Long with bits shifted to the left by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the left by the given amount.
   */
  goog.math.Long.prototype.shiftLeft = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var low = this.low_;
      if (numBits < 32) {
        var high = this.high_;
        return goog.math.Long.fromBits(
            low << numBits,
            (high << numBits) | (low >>> (32 - numBits)));
      } else {
        return goog.math.Long.fromBits(0, low << (numBits - 32));
      }
    }
  };
  /**
   * Returns this Long with bits shifted to the right by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount.
   */
  goog.math.Long.prototype.shiftRight = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >> numBits);
      } else {
        return goog.math.Long.fromBits(
            high >> (numBits - 32),
            high >= 0 ? 0 : -1);
      }
    }
  };
  /**
   * Returns this Long with bits shifted to the right by the given amount, with
   * the new top bits matching the current sign bit.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount, with
   *     zeros placed into the new leading bits.
   */
  goog.math.Long.prototype.shiftRightUnsigned = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >>> numBits);
      } else if (numBits == 32) {
        return goog.math.Long.fromBits(high, 0);
      } else {
        return goog.math.Long.fromBits(high >>> (numBits - 32), 0);
      }
    }
  };
  //======= begin jsbn =======
  var navigator = { appName: 'Modern Browser' }; // polyfill a little
  // Copyright (c) 2005  Tom Wu
  // All Rights Reserved.
  // http://www-cs-students.stanford.edu/~tjw/jsbn/
  /*
   * Copyright (c) 2003-2005  Tom Wu
   * All Rights Reserved.
   *
   * Permission is hereby granted, free of charge, to any person obtaining
   * a copy of this software and associated documentation files (the
   * "Software"), to deal in the Software without restriction, including
   * without limitation the rights to use, copy, modify, merge, publish,
   * distribute, sublicense, and/or sell copies of the Software, and to
   * permit persons to whom the Software is furnished to do so, subject to
   * the following conditions:
   *
   * The above copyright notice and this permission notice shall be
   * included in all copies or substantial portions of the Software.
   *
   * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
   * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
   * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
   *
   * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
   * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
   * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
   * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
   * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   *
   * In addition, the following condition applies:
   *
   * All redistributions must retain an intact copy of this copyright notice
   * and disclaimer.
   */
  // Basic JavaScript BN library - subset useful for RSA encryption.
  // Bits per digit
  var dbits;
  // JavaScript engine analysis
  var canary = 0xdeadbeefcafe;
  var j_lm = ((canary&0xffffff)==0xefcafe);
  // (public) Constructor
  function BigInteger(a,b,c) {
    if(a != null)
      if("number" == typeof a) this.fromNumber(a,b,c);
      else if(b == null && "string" != typeof a) this.fromString(a,256);
      else this.fromString(a,b);
  }
  // return new, unset BigInteger
  function nbi() { return new BigInteger(null); }
  // am: Compute w_j += (x*this_i), propagate carries,
  // c is initial carry, returns final carry.
  // c < 3*dvalue, x < 2*dvalue, this_i < dvalue
  // We need to select the fastest one that works in this environment.
  // am1: use a single mult and divide to get the high bits,
  // max digit bits should be 26 because
  // max internal value = 2*dvalue^2-2*dvalue (< 2^53)
  function am1(i,x,w,j,c,n) {
    while(--n >= 0) {
      var v = x*this[i++]+w[j]+c;
      c = Math.floor(v/0x4000000);
      w[j++] = v&0x3ffffff;
    }
    return c;
  }
  // am2 avoids a big mult-and-extract completely.
  // Max digit bits should be <= 30 because we do bitwise ops
  // on values up to 2*hdvalue^2-hdvalue-1 (< 2^31)
  function am2(i,x,w,j,c,n) {
    var xl = x&0x7fff, xh = x>>15;
    while(--n >= 0) {
      var l = this[i]&0x7fff;
      var h = this[i++]>>15;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x7fff)<<15)+w[j]+(c&0x3fffffff);
      c = (l>>>30)+(m>>>15)+xh*h+(c>>>30);
      w[j++] = l&0x3fffffff;
    }
    return c;
  }
  // Alternately, set max digit bits to 28 since some
  // browsers slow down when dealing with 32-bit numbers.
  function am3(i,x,w,j,c,n) {
    var xl = x&0x3fff, xh = x>>14;
    while(--n >= 0) {
      var l = this[i]&0x3fff;
      var h = this[i++]>>14;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x3fff)<<14)+w[j]+c;
      c = (l>>28)+(m>>14)+xh*h;
      w[j++] = l&0xfffffff;
    }
    return c;
  }
  if(j_lm && (navigator.appName == "Microsoft Internet Explorer")) {
    BigInteger.prototype.am = am2;
    dbits = 30;
  }
  else if(j_lm && (navigator.appName != "Netscape")) {
    BigInteger.prototype.am = am1;
    dbits = 26;
  }
  else { // Mozilla/Netscape seems to prefer am3
    BigInteger.prototype.am = am3;
    dbits = 28;
  }
  BigInteger.prototype.DB = dbits;
  BigInteger.prototype.DM = ((1<<dbits)-1);
  BigInteger.prototype.DV = (1<<dbits);
  var BI_FP = 52;
  BigInteger.prototype.FV = Math.pow(2,BI_FP);
  BigInteger.prototype.F1 = BI_FP-dbits;
  BigInteger.prototype.F2 = 2*dbits-BI_FP;
  // Digit conversions
  var BI_RM = "0123456789abcdefghijklmnopqrstuvwxyz";
  var BI_RC = new Array();
  var rr,vv;
  rr = "0".charCodeAt(0);
  for(vv = 0; vv <= 9; ++vv) BI_RC[rr++] = vv;
  rr = "a".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;
  rr = "A".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;
  function int2char(n) { return BI_RM.charAt(n); }
  function intAt(s,i) {
    var c = BI_RC[s.charCodeAt(i)];
    return (c==null)?-1:c;
  }
  // (protected) copy this to r
  function bnpCopyTo(r) {
    for(var i = this.t-1; i >= 0; --i) r[i] = this[i];
    r.t = this.t;
    r.s = this.s;
  }
  // (protected) set from integer value x, -DV <= x < DV
  function bnpFromInt(x) {
    this.t = 1;
    this.s = (x<0)?-1:0;
    if(x > 0) this[0] = x;
    else if(x < -1) this[0] = x+DV;
    else this.t = 0;
  }
  // return bigint initialized to value
  function nbv(i) { var r = nbi(); r.fromInt(i); return r; }
  // (protected) set from string and radix
  function bnpFromString(s,b) {
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 256) k = 8; // byte array
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else { this.fromRadix(s,b); return; }
    this.t = 0;
    this.s = 0;
    var i = s.length, mi = false, sh = 0;
    while(--i >= 0) {
      var x = (k==8)?s[i]&0xff:intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-") mi = true;
        continue;
      }
      mi = false;
      if(sh == 0)
        this[this.t++] = x;
      else if(sh+k > this.DB) {
        this[this.t-1] |= (x&((1<<(this.DB-sh))-1))<<sh;
        this[this.t++] = (x>>(this.DB-sh));
      }
      else
        this[this.t-1] |= x<<sh;
      sh += k;
      if(sh >= this.DB) sh -= this.DB;
    }
    if(k == 8 && (s[0]&0x80) != 0) {
      this.s = -1;
      if(sh > 0) this[this.t-1] |= ((1<<(this.DB-sh))-1)<<sh;
    }
    this.clamp();
    if(mi) BigInteger.ZERO.subTo(this,this);
  }
  // (protected) clamp off excess high words
  function bnpClamp() {
    var c = this.s&this.DM;
    while(this.t > 0 && this[this.t-1] == c) --this.t;
  }
  // (public) return string representation in given radix
  function bnToString(b) {
    if(this.s < 0) return "-"+this.negate().toString(b);
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else return this.toRadix(b);
    var km = (1<<k)-1, d, m = false, r = "", i = this.t;
    var p = this.DB-(i*this.DB)%k;
    if(i-- > 0) {
      if(p < this.DB && (d = this[i]>>p) > 0) { m = true; r = int2char(d); }
      while(i >= 0) {
        if(p < k) {
          d = (this[i]&((1<<p)-1))<<(k-p);
          d |= this[--i]>>(p+=this.DB-k);
        }
        else {
          d = (this[i]>>(p-=k))&km;
          if(p <= 0) { p += this.DB; --i; }
        }
        if(d > 0) m = true;
        if(m) r += int2char(d);
      }
    }
    return m?r:"0";
  }
  // (public) -this
  function bnNegate() { var r = nbi(); BigInteger.ZERO.subTo(this,r); return r; }
  // (public) |this|
  function bnAbs() { return (this.s<0)?this.negate():this; }
  // (public) return + if this > a, - if this < a, 0 if equal
  function bnCompareTo(a) {
    var r = this.s-a.s;
    if(r != 0) return r;
    var i = this.t;
    r = i-a.t;
    if(r != 0) return (this.s<0)?-r:r;
    while(--i >= 0) if((r=this[i]-a[i]) != 0) return r;
    return 0;
  }
  // returns bit length of the integer x
  function nbits(x) {
    var r = 1, t;
    if((t=x>>>16) != 0) { x = t; r += 16; }
    if((t=x>>8) != 0) { x = t; r += 8; }
    if((t=x>>4) != 0) { x = t; r += 4; }
    if((t=x>>2) != 0) { x = t; r += 2; }
    if((t=x>>1) != 0) { x = t; r += 1; }
    return r;
  }
  // (public) return the number of bits in "this"
  function bnBitLength() {
    if(this.t <= 0) return 0;
    return this.DB*(this.t-1)+nbits(this[this.t-1]^(this.s&this.DM));
  }
  // (protected) r = this << n*DB
  function bnpDLShiftTo(n,r) {
    var i;
    for(i = this.t-1; i >= 0; --i) r[i+n] = this[i];
    for(i = n-1; i >= 0; --i) r[i] = 0;
    r.t = this.t+n;
    r.s = this.s;
  }
  // (protected) r = this >> n*DB
  function bnpDRShiftTo(n,r) {
    for(var i = n; i < this.t; ++i) r[i-n] = this[i];
    r.t = Math.max(this.t-n,0);
    r.s = this.s;
  }
  // (protected) r = this << n
  function bnpLShiftTo(n,r) {
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<cbs)-1;
    var ds = Math.floor(n/this.DB), c = (this.s<<bs)&this.DM, i;
    for(i = this.t-1; i >= 0; --i) {
      r[i+ds+1] = (this[i]>>cbs)|c;
      c = (this[i]&bm)<<bs;
    }
    for(i = ds-1; i >= 0; --i) r[i] = 0;
    r[ds] = c;
    r.t = this.t+ds+1;
    r.s = this.s;
    r.clamp();
  }
  // (protected) r = this >> n
  function bnpRShiftTo(n,r) {
    r.s = this.s;
    var ds = Math.floor(n/this.DB);
    if(ds >= this.t) { r.t = 0; return; }
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<bs)-1;
    r[0] = this[ds]>>bs;
    for(var i = ds+1; i < this.t; ++i) {
      r[i-ds-1] |= (this[i]&bm)<<cbs;
      r[i-ds] = this[i]>>bs;
    }
    if(bs > 0) r[this.t-ds-1] |= (this.s&bm)<<cbs;
    r.t = this.t-ds;
    r.clamp();
  }
  // (protected) r = this - a
  function bnpSubTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]-a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c -= a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c -= a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c -= a.s;
    }
    r.s = (c<0)?-1:0;
    if(c < -1) r[i++] = this.DV+c;
    else if(c > 0) r[i++] = c;
    r.t = i;
    r.clamp();
  }
  // (protected) r = this * a, r != this,a (HAC 14.12)
  // "this" should be the larger one if appropriate.
  function bnpMultiplyTo(a,r) {
    var x = this.abs(), y = a.abs();
    var i = x.t;
    r.t = i+y.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < y.t; ++i) r[i+x.t] = x.am(0,y[i],r,i,0,x.t);
    r.s = 0;
    r.clamp();
    if(this.s != a.s) BigInteger.ZERO.subTo(r,r);
  }
  // (protected) r = this^2, r != this (HAC 14.16)
  function bnpSquareTo(r) {
    var x = this.abs();
    var i = r.t = 2*x.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < x.t-1; ++i) {
      var c = x.am(i,x[i],r,2*i,0,1);
      if((r[i+x.t]+=x.am(i+1,2*x[i],r,2*i+1,c,x.t-i-1)) >= x.DV) {
        r[i+x.t] -= x.DV;
        r[i+x.t+1] = 1;
      }
    }
    if(r.t > 0) r[r.t-1] += x.am(i,x[i],r,2*i,0,1);
    r.s = 0;
    r.clamp();
  }
  // (protected) divide this by m, quotient and remainder to q, r (HAC 14.20)
  // r != q, this != m.  q or r may be null.
  function bnpDivRemTo(m,q,r) {
    var pm = m.abs();
    if(pm.t <= 0) return;
    var pt = this.abs();
    if(pt.t < pm.t) {
      if(q != null) q.fromInt(0);
      if(r != null) this.copyTo(r);
      return;
    }
    if(r == null) r = nbi();
    var y = nbi(), ts = this.s, ms = m.s;
    var nsh = this.DB-nbits(pm[pm.t-1]);	// normalize modulus
    if(nsh > 0) { pm.lShiftTo(nsh,y); pt.lShiftTo(nsh,r); }
    else { pm.copyTo(y); pt.copyTo(r); }
    var ys = y.t;
    var y0 = y[ys-1];
    if(y0 == 0) return;
    var yt = y0*(1<<this.F1)+((ys>1)?y[ys-2]>>this.F2:0);
    var d1 = this.FV/yt, d2 = (1<<this.F1)/yt, e = 1<<this.F2;
    var i = r.t, j = i-ys, t = (q==null)?nbi():q;
    y.dlShiftTo(j,t);
    if(r.compareTo(t) >= 0) {
      r[r.t++] = 1;
      r.subTo(t,r);
    }
    BigInteger.ONE.dlShiftTo(ys,t);
    t.subTo(y,y);	// "negative" y so we can replace sub with am later
    while(y.t < ys) y[y.t++] = 0;
    while(--j >= 0) {
      // Estimate quotient digit
      var qd = (r[--i]==y0)?this.DM:Math.floor(r[i]*d1+(r[i-1]+e)*d2);
      if((r[i]+=y.am(0,qd,r,j,0,ys)) < qd) {	// Try it out
        y.dlShiftTo(j,t);
        r.subTo(t,r);
        while(r[i] < --qd) r.subTo(t,r);
      }
    }
    if(q != null) {
      r.drShiftTo(ys,q);
      if(ts != ms) BigInteger.ZERO.subTo(q,q);
    }
    r.t = ys;
    r.clamp();
    if(nsh > 0) r.rShiftTo(nsh,r);	// Denormalize remainder
    if(ts < 0) BigInteger.ZERO.subTo(r,r);
  }
  // (public) this mod a
  function bnMod(a) {
    var r = nbi();
    this.abs().divRemTo(a,null,r);
    if(this.s < 0 && r.compareTo(BigInteger.ZERO) > 0) a.subTo(r,r);
    return r;
  }
  // Modular reduction using "classic" algorithm
  function Classic(m) { this.m = m; }
  function cConvert(x) {
    if(x.s < 0 || x.compareTo(this.m) >= 0) return x.mod(this.m);
    else return x;
  }
  function cRevert(x) { return x; }
  function cReduce(x) { x.divRemTo(this.m,null,x); }
  function cMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }
  function cSqrTo(x,r) { x.squareTo(r); this.reduce(r); }
  Classic.prototype.convert = cConvert;
  Classic.prototype.revert = cRevert;
  Classic.prototype.reduce = cReduce;
  Classic.prototype.mulTo = cMulTo;
  Classic.prototype.sqrTo = cSqrTo;
  // (protected) return "-1/this % 2^DB"; useful for Mont. reduction
  // justification:
  //         xy == 1 (mod m)
  //         xy =  1+km
  //   xy(2-xy) = (1+km)(1-km)
  // x[y(2-xy)] = 1-k^2m^2
  // x[y(2-xy)] == 1 (mod m^2)
  // if y is 1/x mod m, then y(2-xy) is 1/x mod m^2
  // should reduce x and y(2-xy) by m^2 at each step to keep size bounded.
  // JS multiply "overflows" differently from C/C++, so care is needed here.
  function bnpInvDigit() {
    if(this.t < 1) return 0;
    var x = this[0];
    if((x&1) == 0) return 0;
    var y = x&3;		// y == 1/x mod 2^2
    y = (y*(2-(x&0xf)*y))&0xf;	// y == 1/x mod 2^4
    y = (y*(2-(x&0xff)*y))&0xff;	// y == 1/x mod 2^8
    y = (y*(2-(((x&0xffff)*y)&0xffff)))&0xffff;	// y == 1/x mod 2^16
    // last step - calculate inverse mod DV directly;
    // assumes 16 < DB <= 32 and assumes ability to handle 48-bit ints
    y = (y*(2-x*y%this.DV))%this.DV;		// y == 1/x mod 2^dbits
    // we really want the negative inverse, and -DV < y < DV
    return (y>0)?this.DV-y:-y;
  }
  // Montgomery reduction
  function Montgomery(m) {
    this.m = m;
    this.mp = m.invDigit();
    this.mpl = this.mp&0x7fff;
    this.mph = this.mp>>15;
    this.um = (1<<(m.DB-15))-1;
    this.mt2 = 2*m.t;
  }
  // xR mod m
  function montConvert(x) {
    var r = nbi();
    x.abs().dlShiftTo(this.m.t,r);
    r.divRemTo(this.m,null,r);
    if(x.s < 0 && r.compareTo(BigInteger.ZERO) > 0) this.m.subTo(r,r);
    return r;
  }
  // x/R mod m
  function montRevert(x) {
    var r = nbi();
    x.copyTo(r);
    this.reduce(r);
    return r;
  }
  // x = x/R mod m (HAC 14.32)
  function montReduce(x) {
    while(x.t <= this.mt2)	// pad x so am has enough room later
      x[x.t++] = 0;
    for(var i = 0; i < this.m.t; ++i) {
      // faster way of calculating u0 = x[i]*mp mod DV
      var j = x[i]&0x7fff;
      var u0 = (j*this.mpl+(((j*this.mph+(x[i]>>15)*this.mpl)&this.um)<<15))&x.DM;
      // use am to combine the multiply-shift-add into one call
      j = i+this.m.t;
      x[j] += this.m.am(0,u0,x,i,0,this.m.t);
      // propagate carry
      while(x[j] >= x.DV) { x[j] -= x.DV; x[++j]++; }
    }
    x.clamp();
    x.drShiftTo(this.m.t,x);
    if(x.compareTo(this.m) >= 0) x.subTo(this.m,x);
  }
  // r = "x^2/R mod m"; x != r
  function montSqrTo(x,r) { x.squareTo(r); this.reduce(r); }
  // r = "xy/R mod m"; x,y != r
  function montMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }
  Montgomery.prototype.convert = montConvert;
  Montgomery.prototype.revert = montRevert;
  Montgomery.prototype.reduce = montReduce;
  Montgomery.prototype.mulTo = montMulTo;
  Montgomery.prototype.sqrTo = montSqrTo;
  // (protected) true iff this is even
  function bnpIsEven() { return ((this.t>0)?(this[0]&1):this.s) == 0; }
  // (protected) this^e, e < 2^32, doing sqr and mul with "r" (HAC 14.79)
  function bnpExp(e,z) {
    if(e > 0xffffffff || e < 1) return BigInteger.ONE;
    var r = nbi(), r2 = nbi(), g = z.convert(this), i = nbits(e)-1;
    g.copyTo(r);
    while(--i >= 0) {
      z.sqrTo(r,r2);
      if((e&(1<<i)) > 0) z.mulTo(r2,g,r);
      else { var t = r; r = r2; r2 = t; }
    }
    return z.revert(r);
  }
  // (public) this^e % m, 0 <= e < 2^32
  function bnModPowInt(e,m) {
    var z;
    if(e < 256 || m.isEven()) z = new Classic(m); else z = new Montgomery(m);
    return this.exp(e,z);
  }
  // protected
  BigInteger.prototype.copyTo = bnpCopyTo;
  BigInteger.prototype.fromInt = bnpFromInt;
  BigInteger.prototype.fromString = bnpFromString;
  BigInteger.prototype.clamp = bnpClamp;
  BigInteger.prototype.dlShiftTo = bnpDLShiftTo;
  BigInteger.prototype.drShiftTo = bnpDRShiftTo;
  BigInteger.prototype.lShiftTo = bnpLShiftTo;
  BigInteger.prototype.rShiftTo = bnpRShiftTo;
  BigInteger.prototype.subTo = bnpSubTo;
  BigInteger.prototype.multiplyTo = bnpMultiplyTo;
  BigInteger.prototype.squareTo = bnpSquareTo;
  BigInteger.prototype.divRemTo = bnpDivRemTo;
  BigInteger.prototype.invDigit = bnpInvDigit;
  BigInteger.prototype.isEven = bnpIsEven;
  BigInteger.prototype.exp = bnpExp;
  // public
  BigInteger.prototype.toString = bnToString;
  BigInteger.prototype.negate = bnNegate;
  BigInteger.prototype.abs = bnAbs;
  BigInteger.prototype.compareTo = bnCompareTo;
  BigInteger.prototype.bitLength = bnBitLength;
  BigInteger.prototype.mod = bnMod;
  BigInteger.prototype.modPowInt = bnModPowInt;
  // "constants"
  BigInteger.ZERO = nbv(0);
  BigInteger.ONE = nbv(1);
  // jsbn2 stuff
  // (protected) convert from radix string
  function bnpFromRadix(s,b) {
    this.fromInt(0);
    if(b == null) b = 10;
    var cs = this.chunkSize(b);
    var d = Math.pow(b,cs), mi = false, j = 0, w = 0;
    for(var i = 0; i < s.length; ++i) {
      var x = intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-" && this.signum() == 0) mi = true;
        continue;
      }
      w = b*w+x;
      if(++j >= cs) {
        this.dMultiply(d);
        this.dAddOffset(w,0);
        j = 0;
        w = 0;
      }
    }
    if(j > 0) {
      this.dMultiply(Math.pow(b,j));
      this.dAddOffset(w,0);
    }
    if(mi) BigInteger.ZERO.subTo(this,this);
  }
  // (protected) return x s.t. r^x < DV
  function bnpChunkSize(r) { return Math.floor(Math.LN2*this.DB/Math.log(r)); }
  // (public) 0 if this == 0, 1 if this > 0
  function bnSigNum() {
    if(this.s < 0) return -1;
    else if(this.t <= 0 || (this.t == 1 && this[0] <= 0)) return 0;
    else return 1;
  }
  // (protected) this *= n, this >= 0, 1 < n < DV
  function bnpDMultiply(n) {
    this[this.t] = this.am(0,n-1,this,0,0,this.t);
    ++this.t;
    this.clamp();
  }
  // (protected) this += n << w words, this >= 0
  function bnpDAddOffset(n,w) {
    if(n == 0) return;
    while(this.t <= w) this[this.t++] = 0;
    this[w] += n;
    while(this[w] >= this.DV) {
      this[w] -= this.DV;
      if(++w >= this.t) this[this.t++] = 0;
      ++this[w];
    }
  }
  // (protected) convert to radix string
  function bnpToRadix(b) {
    if(b == null) b = 10;
    if(this.signum() == 0 || b < 2 || b > 36) return "0";
    var cs = this.chunkSize(b);
    var a = Math.pow(b,cs);
    var d = nbv(a), y = nbi(), z = nbi(), r = "";
    this.divRemTo(d,y,z);
    while(y.signum() > 0) {
      r = (a+z.intValue()).toString(b).substr(1) + r;
      y.divRemTo(d,y,z);
    }
    return z.intValue().toString(b) + r;
  }
  // (public) return value as integer
  function bnIntValue() {
    if(this.s < 0) {
      if(this.t == 1) return this[0]-this.DV;
      else if(this.t == 0) return -1;
    }
    else if(this.t == 1) return this[0];
    else if(this.t == 0) return 0;
    // assumes 16 < DB < 32
    return ((this[1]&((1<<(32-this.DB))-1))<<this.DB)|this[0];
  }
  // (protected) r = this + a
  function bnpAddTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]+a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c += a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c += a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += a.s;
    }
    r.s = (c<0)?-1:0;
    if(c > 0) r[i++] = c;
    else if(c < -1) r[i++] = this.DV+c;
    r.t = i;
    r.clamp();
  }
  BigInteger.prototype.fromRadix = bnpFromRadix;
  BigInteger.prototype.chunkSize = bnpChunkSize;
  BigInteger.prototype.signum = bnSigNum;
  BigInteger.prototype.dMultiply = bnpDMultiply;
  BigInteger.prototype.dAddOffset = bnpDAddOffset;
  BigInteger.prototype.toRadix = bnpToRadix;
  BigInteger.prototype.intValue = bnIntValue;
  BigInteger.prototype.addTo = bnpAddTo;
  //======= end jsbn =======
  // Emscripten wrapper
  var Wrapper = {
    abs: function(l, h) {
      var x = new goog.math.Long(l, h);
      var ret;
      if (x.isNegative()) {
        ret = x.negate();
      } else {
        ret = x;
      }
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
    },
    ensureTemps: function() {
      if (Wrapper.ensuredTemps) return;
      Wrapper.ensuredTemps = true;
      Wrapper.two32 = new BigInteger();
      Wrapper.two32.fromString('4294967296', 10);
      Wrapper.two64 = new BigInteger();
      Wrapper.two64.fromString('18446744073709551616', 10);
      Wrapper.temp1 = new BigInteger();
      Wrapper.temp2 = new BigInteger();
    },
    lh2bignum: function(l, h) {
      var a = new BigInteger();
      a.fromString(h.toString(), 10);
      var b = new BigInteger();
      a.multiplyTo(Wrapper.two32, b);
      var c = new BigInteger();
      c.fromString(l.toString(), 10);
      var d = new BigInteger();
      c.addTo(b, d);
      return d;
    },
    stringify: function(l, h, unsigned) {
      var ret = new goog.math.Long(l, h).toString();
      if (unsigned && ret[0] == '-') {
        // unsign slowly using jsbn bignums
        Wrapper.ensureTemps();
        var bignum = new BigInteger();
        bignum.fromString(ret, 10);
        ret = new BigInteger();
        Wrapper.two64.addTo(bignum, ret);
        ret = ret.toString(10);
      }
      return ret;
    },
    fromString: function(str, base, min, max, unsigned) {
      Wrapper.ensureTemps();
      var bignum = new BigInteger();
      bignum.fromString(str, base);
      var bigmin = new BigInteger();
      bigmin.fromString(min, 10);
      var bigmax = new BigInteger();
      bigmax.fromString(max, 10);
      if (unsigned && bignum.compareTo(BigInteger.ZERO) < 0) {
        var temp = new BigInteger();
        bignum.addTo(Wrapper.two64, temp);
        bignum = temp;
      }
      var error = false;
      if (bignum.compareTo(bigmin) < 0) {
        bignum = bigmin;
        error = true;
      } else if (bignum.compareTo(bigmax) > 0) {
        bignum = bigmax;
        error = true;
      }
      var ret = goog.math.Long.fromString(bignum.toString()); // min-max checks should have clamped this to a range goog.math.Long can handle well
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
      if (error) throw 'range error';
    }
  };
  return Wrapper;
})();
//======= end closure i64 code =======
// === Auto-generated postamble setup entry stuff ===
if (memoryInitializer) {
  function applyData(data) {
    HEAPU8.set(data, STATIC_BASE);
  }
  if (ENVIRONMENT_IS_NODE || ENVIRONMENT_IS_SHELL) {
    applyData(Module['readBinary'](memoryInitializer));
  } else {
    addRunDependency('memory initializer');
    Browser.asyncLoad(memoryInitializer, function(data) {
      applyData(data);
      removeRunDependency('memory initializer');
    }, function(data) {
      throw 'could not load memory initializer ' + memoryInitializer;
    });
  }
}
function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;
var initialStackTop;
var preloadStartTime = null;
var calledMain = false;
dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun'] && shouldRunNow) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}
Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');
  args = args || [];
  if (ENVIRONMENT_IS_WEB && preloadStartTime !== null) {
    Module.printErr('preload time: ' + (Date.now() - preloadStartTime) + ' ms');
  }
  ensureInitRuntime();
  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString("/bin/this.program"), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);
  initialStackTop = STACKTOP;
  try {
    var ret = Module['_main'](argc, argv, 0);
    // if we're not running an evented main loop, it's time to exit
    if (!Module['noExitRuntime']) {
      exit(ret);
    }
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}
function run(args) {
  args = args || Module['arguments'];
  if (preloadStartTime === null) preloadStartTime = Date.now();
  if (runDependencies > 0) {
    Module.printErr('run() called, but dependencies remain, so not running');
    return;
  }
  preRun();
  if (runDependencies > 0) {
    // a preRun added a dependency, run will be called later
    return;
  }
  function doRun() {
    ensureInitRuntime();
    preMain();
    Module['calledRun'] = true;
    if (Module['_main'] && shouldRunNow) {
      Module['callMain'](args);
    }
    postRun();
  }
  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      if (!ABORT) doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;
function exit(status) {
  ABORT = true;
  EXITSTATUS = status;
  STACKTOP = initialStackTop;
  // exit the runtime
  exitRuntime();
  // TODO We should handle this differently based on environment.
  // In the browser, the best we can do is throw an exception
  // to halt execution, but in node we could process.exit and
  // I'd imagine SM shell would have something equivalent.
  // This would let us set a proper exit status (which
  // would be great for checking test exit statuses).
  // https://github.com/kripken/emscripten/issues/1371
  // throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;
function abort(text) {
  if (text) {
    Module.print(text);
    Module.printErr(text);
  }
  ABORT = true;
  EXITSTATUS = 1;
  throw 'abort() at ' + stackTrace();
}
Module['abort'] = Module.abort = abort;
// {{PRE_RUN_ADDITIONS}}
if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}
// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}
run();
// {{POST_RUN_ADDITIONS}}
// {{MODULE_ADDITIONS}}
    // return output files buffers
    return Module['return'];
}
